      SUBROUTINE ZFFTI (N,WSAVE)
      IMPLICIT NONE
      INTEGER :: N, IW1, IW2
      DOUBLE PRECISION :: WSAVE(1)
      IF (N .EQ. 1) RETURN
      IW1 = N+N+1
      IW2 = IW1+N+N
      CALL ZFFTI1 (N,WSAVE(IW1),WSAVE(IW2))
      RETURN
      END
