/* Generated by Cython 0.29.6 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_6"
#define CYTHON_HEX_VERSION 0x001D06F0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
  #define PyObject_Unicode             PyObject_Str
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : (Py_INCREF(func), func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__scipy__linalg___decomp_update
#define __PYX_HAVE_API__scipy__linalg___decomp_update
/* Early includes */
#include <string.h>
#include <stdlib.h>
#include <limits.h>
#include <float.h>
#include <math.h>
#include <stdio.h>
#include "numpy/arrayobject.h"
#include "numpy/ufuncobject.h"
#include "numpy/npy_math.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

/* Header.proto */
#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "_decomp_update.pyx",
  "__init__.pxd",
  "type.pxd",
};
/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* NoFastGil.proto */
#define __Pyx_PyGILState_Ensure PyGILState_Ensure
#define __Pyx_PyGILState_Release PyGILState_Release
#define __Pyx_FastGIL_Remember()
#define __Pyx_FastGIL_Forget()
#define __Pyx_FastGilFuncInit()


/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":776
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":777
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":778
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":779
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":783
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":784
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":785
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":786
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":790
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":791
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":800
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":801
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":802
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":804
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":805
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":806
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":808
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":809
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":811
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":812
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":813
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;

/* "cython_blas.pxd":15
 * # The original libraries should be linked directly.
 * 
 * ctypedef float s             # <<<<<<<<<<<<<<
 * ctypedef double d
 * ctypedef float complex c
 */
typedef float __pyx_t_5scipy_6linalg_11cython_blas_s;

/* "cython_blas.pxd":16
 * 
 * ctypedef float s
 * ctypedef double d             # <<<<<<<<<<<<<<
 * ctypedef float complex c
 * ctypedef double complex z
 */
typedef double __pyx_t_5scipy_6linalg_11cython_blas_d;

/* "cython_lapack.pxd":15
 * # The original libraries should be linked directly.
 * 
 * ctypedef float s             # <<<<<<<<<<<<<<
 * ctypedef double d
 * ctypedef float complex c
 */
typedef float __pyx_t_5scipy_6linalg_13cython_lapack_s;

/* "cython_lapack.pxd":16
 * 
 * ctypedef float s
 * ctypedef double d             # <<<<<<<<<<<<<<
 * ctypedef float complex c
 * ctypedef double complex z
 */
typedef double __pyx_t_5scipy_6linalg_13cython_lapack_d;
/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif
static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif
static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);


/*--- Type declarations ---*/

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":815
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":816
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":817
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":819
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;

/* "cython_lapack.pxd":22
 * # Function pointer type declarations for
 * # gees and gges families of functions.
 * ctypedef bint cselect1(c*)             # <<<<<<<<<<<<<<
 * ctypedef bint cselect2(c*, c*)
 * ctypedef bint dselect2(d*, d*)
 */
typedef int __pyx_t_5scipy_6linalg_13cython_lapack_cselect1(__pyx_t_float_complex *);

/* "cython_lapack.pxd":23
 * # gees and gges families of functions.
 * ctypedef bint cselect1(c*)
 * ctypedef bint cselect2(c*, c*)             # <<<<<<<<<<<<<<
 * ctypedef bint dselect2(d*, d*)
 * ctypedef bint dselect3(d*, d*, d*)
 */
typedef int __pyx_t_5scipy_6linalg_13cython_lapack_cselect2(__pyx_t_float_complex *, __pyx_t_float_complex *);

/* "cython_lapack.pxd":24
 * ctypedef bint cselect1(c*)
 * ctypedef bint cselect2(c*, c*)
 * ctypedef bint dselect2(d*, d*)             # <<<<<<<<<<<<<<
 * ctypedef bint dselect3(d*, d*, d*)
 * ctypedef bint sselect2(s*, s*)
 */
typedef int __pyx_t_5scipy_6linalg_13cython_lapack_dselect2(__pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *);

/* "cython_lapack.pxd":25
 * ctypedef bint cselect2(c*, c*)
 * ctypedef bint dselect2(d*, d*)
 * ctypedef bint dselect3(d*, d*, d*)             # <<<<<<<<<<<<<<
 * ctypedef bint sselect2(s*, s*)
 * ctypedef bint sselect3(s*, s*, s*)
 */
typedef int __pyx_t_5scipy_6linalg_13cython_lapack_dselect3(__pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *);

/* "cython_lapack.pxd":26
 * ctypedef bint dselect2(d*, d*)
 * ctypedef bint dselect3(d*, d*, d*)
 * ctypedef bint sselect2(s*, s*)             # <<<<<<<<<<<<<<
 * ctypedef bint sselect3(s*, s*, s*)
 * ctypedef bint zselect1(z*)
 */
typedef int __pyx_t_5scipy_6linalg_13cython_lapack_sselect2(__pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *);

/* "cython_lapack.pxd":27
 * ctypedef bint dselect3(d*, d*, d*)
 * ctypedef bint sselect2(s*, s*)
 * ctypedef bint sselect3(s*, s*, s*)             # <<<<<<<<<<<<<<
 * ctypedef bint zselect1(z*)
 * ctypedef bint zselect2(z*, z*)
 */
typedef int __pyx_t_5scipy_6linalg_13cython_lapack_sselect3(__pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *);

/* "cython_lapack.pxd":28
 * ctypedef bint sselect2(s*, s*)
 * ctypedef bint sselect3(s*, s*, s*)
 * ctypedef bint zselect1(z*)             # <<<<<<<<<<<<<<
 * ctypedef bint zselect2(z*, z*)
 * 
 */
typedef int __pyx_t_5scipy_6linalg_13cython_lapack_zselect1(__pyx_t_double_complex *);

/* "cython_lapack.pxd":29
 * ctypedef bint sselect3(s*, s*, s*)
 * ctypedef bint zselect1(z*)
 * ctypedef bint zselect2(z*, z*)             # <<<<<<<<<<<<<<
 * 
 * cdef void cbbcsd(char *jobu1, char *jobu2, char *jobv1t, char *jobv2t, char *trans, int *m, int *p, int *q, s *theta, s *phi, c *u1, int *ldu1, c *u2, int *ldu2, c *v1t, int *ldv1t, c *v2t, int *ldv2t, s *b11d, s *b11e, s *b12d, s *b12e, s *b21d, s *b21e, s *b22d, s *b22e, s *rwork, int *lrwork, int *info) nogil
 */
typedef int __pyx_t_5scipy_6linalg_13cython_lapack_zselect2(__pyx_t_double_complex *, __pyx_t_double_complex *);

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* IncludeStringH.proto */
#include <string.h>

/* JoinPyUnicode.proto */
static PyObject* __Pyx_PyUnicode_Join(PyObject* value_tuple, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char);

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* RealImag.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if defined(__cplusplus) && CYTHON_CCOMPLEX\
        && (defined(_WIN32) || defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5 || __GNUC__ == 4 && __GNUC_MINOR__ >= 4 )) || __cplusplus >= 201103)
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_float(a, b)   ((a)==(b))
    #define __Pyx_c_sum_float(a, b)  ((a)+(b))
    #define __Pyx_c_diff_float(a, b) ((a)-(b))
    #define __Pyx_c_prod_float(a, b) ((a)*(b))
    #define __Pyx_c_quot_float(a, b) ((a)/(b))
    #define __Pyx_c_neg_float(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_float(z) ((z)==(float)0)
    #define __Pyx_c_conj_float(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (::std::abs(z))
        #define __Pyx_c_pow_float(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_float(z) ((z)==0)
    #define __Pyx_c_conj_float(z)    (conjf(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (cabsf(z))
        #define __Pyx_c_pow_float(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_double(a, b) ((a)-(b))
    #define __Pyx_c_prod_double(a, b) ((a)*(b))
    #define __Pyx_c_quot_double(a, b) ((a)/(b))
    #define __Pyx_c_neg_double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_double(z) ((z)==(double)0)
    #define __Pyx_c_conj_double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (::std::abs(z))
        #define __Pyx_c_pow_double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_double(z) ((z)==0)
    #define __Pyx_c_conj_double(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (cabs(z))
        #define __Pyx_c_pow_double(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Py_intptr_t(Py_intptr_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__NPY_TYPES(enum NPY_TYPES value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* FunctionImport.proto */
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'cython' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'libc.limits' */

/* Module declarations from 'libc.float' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'libc' */

/* Module declarations from 'libc.math' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;
static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *, char *, char *, int *); /*proto*/
static CYTHON_INLINE int __pyx_f_5numpy_import_array(void); /*proto*/

/* Module declarations from 'scipy.linalg.cython_blas' */
static void (*__pyx_f_5scipy_6linalg_11cython_blas_caxpy)(int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_ccopy)(int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_cgemm)(char *, char *, int *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_cgemv)(char *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_cgeru)(int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_cscal)(int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_cswap)(int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_ctrmm)(char *, char *, char *, char *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_daxpy)(int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_dcopy)(int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_dgemm)(char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_dgemv)(char *, int *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_dger)(int *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *); /*proto*/
static __pyx_t_5scipy_6linalg_11cython_blas_d (*__pyx_f_5scipy_6linalg_11cython_blas_dnrm2)(int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_drot)(int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, __pyx_t_5scipy_6linalg_11cython_blas_d *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_dscal)(int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_dswap)(int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_dtrmm)(char *, char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *); /*proto*/
static __pyx_t_5scipy_6linalg_11cython_blas_d (*__pyx_f_5scipy_6linalg_11cython_blas_dznrm2)(int *, __pyx_t_double_complex *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_saxpy)(int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *); /*proto*/
static __pyx_t_5scipy_6linalg_11cython_blas_s (*__pyx_f_5scipy_6linalg_11cython_blas_scnrm2)(int *, __pyx_t_float_complex *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_scopy)(int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_sgemm)(char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_sgemv)(char *, int *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_sger)(int *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *); /*proto*/
static __pyx_t_5scipy_6linalg_11cython_blas_s (*__pyx_f_5scipy_6linalg_11cython_blas_snrm2)(int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_srot)(int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, __pyx_t_5scipy_6linalg_11cython_blas_s *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_sscal)(int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_sswap)(int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_strmm)(char *, char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_zaxpy)(int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_zcopy)(int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_zgemm)(char *, char *, int *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_zgemv)(char *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_zgeru)(int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_zscal)(int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_zswap)(int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_11cython_blas_ztrmm)(char *, char *, char *, char *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *); /*proto*/

/* Module declarations from 'scipy.linalg.cython_lapack' */
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_cgeqrf)(int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_clarf)(char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_clarfg)(int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_clartg)(__pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_float_complex *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_crot)(int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_cunmqr)(char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_dgeqrf)(int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_dlarf)(char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_dlarfg)(int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_dlartg)(__pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_dormqr)(char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_sgeqrf)(int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_slarf)(char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_slarfg)(int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_slartg)(__pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_sormqr)(char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_zgeqrf)(int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_zlarf)(char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_zlarfg)(int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_zlartg)(__pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_double_complex *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_zrot)(int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *); /*proto*/
static void (*__pyx_f_5scipy_6linalg_13cython_lapack_zunmqr)(char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *); /*proto*/

/* Module declarations from 'scipy.linalg' */

/* Module declarations from 'scipy.linalg._decomp_update' */
static int __pyx_v_5scipy_6linalg_14_decomp_update_ARRAY_ANYORDER;
static int __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
static PyObject *__pyx_f_5scipy_6linalg_14_decomp_update_form_qTu(PyArrayObject *, PyArrayObject *, void *, int *, int); /*proto*/
static PyObject *__pyx_f_5scipy_6linalg_14_decomp_update_validate_array(PyArrayObject *, int); /*proto*/
static PyObject *__pyx_f_5scipy_6linalg_14_decomp_update_validate_qr(PyObject *, PyObject *, int, int, int, int, int); /*proto*/
static void *__pyx_f_5scipy_6linalg_14_decomp_update_extract(PyArrayObject *, int *); /*proto*/
static PyObject *__pyx_f_5scipy_6linalg_14_decomp_update_qr_insert_row(PyObject *, PyObject *, PyObject *, int, int, int); /*proto*/
static PyObject *__pyx_f_5scipy_6linalg_14_decomp_update_qr_insert_col(PyObject *, PyObject *, PyObject *, int, PyObject *, int, int); /*proto*/
static CYTHON_INLINE float *__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(float *, int *, int, int); /*proto*/
static CYTHON_INLINE double *__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(double *, int *, int, int); /*proto*/
static CYTHON_INLINE __pyx_t_float_complex *__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_t_float_complex *, int *, int, int); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex *__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_t_double_complex *, int *, int, int); /*proto*/
static CYTHON_INLINE float *__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index1(float *, int *, int); /*proto*/
static CYTHON_INLINE double *__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index1(double *, int *, int); /*proto*/
static CYTHON_INLINE __pyx_t_float_complex *__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_t_float_complex *, int *, int); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex *__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_t_double_complex *, int *, int); /*proto*/
static CYTHON_INLINE float *__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(float *, int *, int); /*proto*/
static CYTHON_INLINE double *__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(double *, int *, int); /*proto*/
static CYTHON_INLINE __pyx_t_float_complex *__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_t_float_complex *, int *, int); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex *__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_t_double_complex *, int *, int); /*proto*/
static CYTHON_INLINE float *__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(float *, int *, int); /*proto*/
static CYTHON_INLINE double *__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(double *, int *, int); /*proto*/
static CYTHON_INLINE __pyx_t_float_complex *__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_t_float_complex *, int *, int); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex *__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_t_double_complex *, int *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_copy(int, float *, int, float *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_copy(int, double *, int, double *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_copy(int, __pyx_t_float_complex *, int, __pyx_t_float_complex *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_copy(int, __pyx_t_double_complex *, int, __pyx_t_double_complex *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_swap(int, float *, int, float *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_swap(int, double *, int, double *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_swap(int, __pyx_t_float_complex *, int, __pyx_t_float_complex *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_swap(int, __pyx_t_double_complex *, int, __pyx_t_double_complex *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_scal(int, float, float *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_scal(int, double, double *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_scal(int, __pyx_t_float_complex, __pyx_t_float_complex *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_scal(int, __pyx_t_double_complex, __pyx_t_double_complex *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_axpy(int, float, float *, int, float *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_axpy(int, double, double *, int, double *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_axpy(int, __pyx_t_float_complex, __pyx_t_float_complex *, int, __pyx_t_float_complex *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_axpy(int, __pyx_t_double_complex, __pyx_t_double_complex *, int, __pyx_t_double_complex *, int); /*proto*/
static CYTHON_INLINE float __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(int, float *, int); /*proto*/
static CYTHON_INLINE double __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(int, double *, int); /*proto*/
static CYTHON_INLINE __pyx_t_float_complex __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(int, __pyx_t_float_complex *, int); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(int, __pyx_t_double_complex *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_lartg(float *, float *, float *, float *); /*proto*/
static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_lartg(double *, double *, double *, double *); /*proto*/
static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_float_complex *); /*proto*/
static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *); /*proto*/
static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(int, float *, int, float *, int, float, float); /*proto*/
static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(int, double *, int, double *, int, double, double); /*proto*/
static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(int, __pyx_t_float_complex *, int, __pyx_t_float_complex *, int, __pyx_t_float_complex, __pyx_t_float_complex); /*proto*/
static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(int, __pyx_t_double_complex *, int, __pyx_t_double_complex *, int, __pyx_t_double_complex, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_larfg(int, float *, float *, int, float *); /*proto*/
static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_larfg(int, double *, double *, int, double *); /*proto*/
static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_larfg(int, __pyx_t_float_complex *, __pyx_t_float_complex *, int, __pyx_t_float_complex *); /*proto*/
static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_larfg(int, __pyx_t_double_complex *, __pyx_t_double_complex *, int, __pyx_t_double_complex *); /*proto*/
static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_larf(char *, int, int, float *, int, float, float *, int, float *); /*proto*/
static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_larf(char *, int, int, double *, int, double, double *, int, double *); /*proto*/
static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_larf(char *, int, int, __pyx_t_float_complex *, int, __pyx_t_float_complex, __pyx_t_float_complex *, int, __pyx_t_float_complex *); /*proto*/
static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_larf(char *, int, int, __pyx_t_double_complex *, int, __pyx_t_double_complex, __pyx_t_double_complex *, int, __pyx_t_double_complex *); /*proto*/
static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_ger(int, int, float, float *, int, float *, int, float *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_ger(int, int, double, double *, int, double *, int, double *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_ger(int, int, __pyx_t_float_complex, __pyx_t_float_complex *, int, __pyx_t_float_complex *, int, __pyx_t_float_complex *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_ger(int, int, __pyx_t_double_complex, __pyx_t_double_complex *, int, __pyx_t_double_complex *, int, __pyx_t_double_complex *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemv(char *, int, int, float, float *, int, float *, int, float, float *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemv(char *, int, int, double, double *, int, double *, int, double, double *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemv(char *, int, int, __pyx_t_float_complex, __pyx_t_float_complex *, int, __pyx_t_float_complex *, int, __pyx_t_float_complex, __pyx_t_float_complex *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemv(char *, int, int, __pyx_t_double_complex, __pyx_t_double_complex *, int, __pyx_t_double_complex *, int, __pyx_t_double_complex, __pyx_t_double_complex *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemm(char *, char *, int, int, int, float, float *, int, float *, int, float, float *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemm(char *, char *, int, int, int, double, double *, int, double *, int, double, double *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemm(char *, char *, int, int, int, __pyx_t_float_complex, __pyx_t_float_complex *, int, __pyx_t_float_complex *, int, __pyx_t_float_complex, __pyx_t_float_complex *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemm(char *, char *, int, int, int, __pyx_t_double_complex, __pyx_t_double_complex *, int, __pyx_t_double_complex *, int, __pyx_t_double_complex, __pyx_t_double_complex *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_trmm(char *, char *, char *, char *, int, int, float, float *, int, float *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_trmm(char *, char *, char *, char *, int, int, double, double *, int, double *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_trmm(char *, char *, char *, char *, int, int, __pyx_t_float_complex, __pyx_t_float_complex *, int, __pyx_t_float_complex *, int); /*proto*/
static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_trmm(char *, char *, char *, char *, int, int, __pyx_t_double_complex, __pyx_t_double_complex *, int, __pyx_t_double_complex *, int); /*proto*/
static CYTHON_INLINE int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_geqrf(int, int, float *, int, float *, float *, int); /*proto*/
static CYTHON_INLINE int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_geqrf(int, int, double *, int, double *, double *, int); /*proto*/
static CYTHON_INLINE int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_geqrf(int, int, __pyx_t_float_complex *, int, __pyx_t_float_complex *, __pyx_t_float_complex *, int); /*proto*/
static CYTHON_INLINE int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_geqrf(int, int, __pyx_t_double_complex *, int, __pyx_t_double_complex *, __pyx_t_double_complex *, int); /*proto*/
static CYTHON_INLINE int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(char *, char *, int, int, int, float *, int, float *, float *, int, float *, int); /*proto*/
static CYTHON_INLINE int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(char *, char *, int, int, int, double *, int, double *, double *, int, double *, int); /*proto*/
static CYTHON_INLINE int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(char *, char *, int, int, int, __pyx_t_float_complex *, int, __pyx_t_float_complex *, __pyx_t_float_complex *, int, __pyx_t_float_complex *, int); /*proto*/
static CYTHON_INLINE int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(char *, char *, int, int, int, __pyx_t_double_complex *, int, __pyx_t_double_complex *, __pyx_t_double_complex *, int, __pyx_t_double_complex *, int); /*proto*/
static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(int, float *, int *); /*proto*/
static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(int, double *, int *); /*proto*/
static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(int, __pyx_t_float_complex *, int *); /*proto*/
static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(int, __pyx_t_double_complex *, int *); /*proto*/
static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_2d_conj(int, int, float *, int *); /*proto*/
static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_2d_conj(int, int, double *, int *); /*proto*/
static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_2d_conj(int, int, __pyx_t_float_complex *, int *); /*proto*/
static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_2d_conj(int, int, __pyx_t_double_complex *, int *); /*proto*/
static float __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_sqrt(float); /*proto*/
static double __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_sqrt(double); /*proto*/
static __pyx_t_float_complex __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_sqrt(__pyx_t_float_complex); /*proto*/
static __pyx_t_double_complex __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_sqrt(__pyx_t_double_complex); /*proto*/
static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(float, float); /*proto*/
static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(double, double); /*proto*/
static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_t_float_complex, __pyx_t_float_complex); /*proto*/
static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_t_double_complex, __pyx_t_double_complex); /*proto*/
static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_equal(float, float); /*proto*/
static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_equal(double, double); /*proto*/
static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_equal(__pyx_t_float_complex, __pyx_t_float_complex); /*proto*/
static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_equal(__pyx_t_double_complex, __pyx_t_double_complex); /*proto*/
static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_to_lwork(float, float); /*proto*/
static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_to_lwork(double, double); /*proto*/
static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_to_lwork(__pyx_t_float_complex, __pyx_t_float_complex); /*proto*/
static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_to_lwork(__pyx_t_double_complex, __pyx_t_double_complex); /*proto*/
static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_reorthx(int, int, float *, int *, int, int, float *, float *); /*proto*/
static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_reorthx(int, int, double *, int *, int, int, double *, double *); /*proto*/
static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_reorthx(int, int, __pyx_t_float_complex *, int *, int, int, __pyx_t_float_complex *, __pyx_t_float_complex *); /*proto*/
static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_reorthx(int, int, __pyx_t_double_complex *, int *, int, int, __pyx_t_double_complex *, __pyx_t_double_complex *); /*proto*/
static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_delete(int, int, float *, int *, int, float *, int *, int, int, int); /*proto*/
static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_delete(int, int, double *, int *, int, double *, int *, int, int, int); /*proto*/
static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_delete(int, int, __pyx_t_float_complex *, int *, int, __pyx_t_float_complex *, int *, int, int, int); /*proto*/
static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_delete(int, int, __pyx_t_double_complex *, int *, int, __pyx_t_double_complex *, int *, int, int, int); /*proto*/
static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_delete(int, int, float *, int *, float *, int *, int, int); /*proto*/
static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_delete(int, int, double *, int *, double *, int *, int, int); /*proto*/
static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_delete(int, int, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int, int); /*proto*/
static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_delete(int, int, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int, int); /*proto*/
static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_delete(int, int, int, float *, int *, float *, int *, int); /*proto*/
static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_delete(int, int, int, double *, int *, double *, int *, int); /*proto*/
static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_delete(int, int, int, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int); /*proto*/
static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_delete(int, int, int, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int); /*proto*/
static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_delete(int, int, int, float *, int *, float *, int *, int, int); /*proto*/
static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_delete(int, int, int, double *, int *, double *, int *, int, int); /*proto*/
static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_delete(int, int, int, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int, int); /*proto*/
static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_delete(int, int, int, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int, int); /*proto*/
static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_insert(int, int, float *, int *, float *, int *, float *, int *, int); /*proto*/
static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_insert(int, int, double *, int *, double *, int *, double *, int *, int); /*proto*/
static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_insert(int, int, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int); /*proto*/
static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_insert(int, int, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int); /*proto*/
static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_row_insert(int, int, float *, int *, float *, int *, int); /*proto*/
static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_row_insert(int, int, double *, int *, double *, int *, int); /*proto*/
static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_row_insert(int, int, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int); /*proto*/
static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_row_insert(int, int, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int); /*proto*/
static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_block_row_insert(int, int, float *, int *, float *, int *, float *, int *, int, int); /*proto*/
static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_block_row_insert(int, int, double *, int *, double *, int *, double *, int *, int, int); /*proto*/
static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_block_row_insert(int, int, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int, int); /*proto*/
static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_block_row_insert(int, int, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int, int); /*proto*/
static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_insert(int, int, float *, int *, float *, int *, int, int); /*proto*/
static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_insert(int, int, double *, int *, double *, int *, int, int); /*proto*/
static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_insert(int, int, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int, int); /*proto*/
static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_insert(int, int, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int, int); /*proto*/
static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_col_insert(int, int, float *, int *, float *, int *, float *, int *, int, int, int, float *); /*proto*/
static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_col_insert(int, int, double *, int *, double *, int *, double *, int *, int, int, int, double *); /*proto*/
static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_col_insert(int, int, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int, int, int, __pyx_t_float_complex *); /*proto*/
static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_col_insert(int, int, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int, int, int, __pyx_t_double_complex *); /*proto*/
static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_insert(int, int, float *, int *, float *, int *, int); /*proto*/
static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_insert(int, int, double *, int *, double *, int *, int); /*proto*/
static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_insert(int, int, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int); /*proto*/
static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_insert(int, int, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int); /*proto*/
static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_insert(int, int, float *, int *, float *, int *, int, int); /*proto*/
static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_insert(int, int, double *, int *, double *, int *, int, int); /*proto*/
static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_insert(int, int, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int, int); /*proto*/
static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_insert(int, int, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int, int); /*proto*/
static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_1_update(int, int, float *, int *, int, float *, int *, float *, int *, float *, int *, float *, int *); /*proto*/
static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_1_update(int, int, double *, int *, int, double *, int *, double *, int *, double *, int *, double *, int *); /*proto*/
static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_1_update(int, int, __pyx_t_float_complex *, int *, int, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *); /*proto*/
static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_1_update(int, int, __pyx_t_double_complex *, int *, int, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *); /*proto*/
static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_p_update(int, int, int, float *, int *, int, float *, int *, float *, int *, float *, int *, float *, int *); /*proto*/
static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_p_update(int, int, int, double *, int *, int, double *, int *, double *, int *, double *, int *, double *, int *); /*proto*/
static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_p_update(int, int, int, __pyx_t_float_complex *, int *, int, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *); /*proto*/
static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_p_update(int, int, int, __pyx_t_double_complex *, int *, int, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *); /*proto*/
static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_1_update(int, int, float *, int *, float *, int *, float *, int *, float *, int *); /*proto*/
static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_1_update(int, int, double *, int *, double *, int *, double *, int *, double *, int *); /*proto*/
static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_1_update(int, int, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *); /*proto*/
static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_1_update(int, int, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *); /*proto*/
static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_p_update(int, int, int, float *, int *, float *, int *, float *, int *, float *, int *); /*proto*/
static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_p_update(int, int, int, double *, int *, double *, int *, double *, int *, double *, int *); /*proto*/
static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_p_update(int, int, int, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *); /*proto*/
static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_p_update(int, int, int, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *); /*proto*/
static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_hessenberg_qr(int, int, float *, int *, float *, int *, int); /*proto*/
static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_hessenberg_qr(int, int, double *, int *, double *, int *, int); /*proto*/
static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_hessenberg_qr(int, int, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int); /*proto*/
static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_hessenberg_qr(int, int, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int); /*proto*/
static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_p_subdiag_qr(int, int, int, float *, int *, float *, int *, int, int, float *); /*proto*/
static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_p_subdiag_qr(int, int, int, double *, int *, double *, int *, int, int, double *); /*proto*/
static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_p_subdiag_qr(int, int, int, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int, int, __pyx_t_float_complex *); /*proto*/
static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_p_subdiag_qr(int, int, int, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int, int, __pyx_t_double_complex *); /*proto*/
static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_reorth(int, int, float *, int *, int, float *, int *, float *, float *); /*proto*/
static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_reorth(int, int, double *, int *, int, double *, int *, double *, double *); /*proto*/
static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_reorth(int, int, __pyx_t_float_complex *, int *, int, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *); /*proto*/
static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_reorth(int, int, __pyx_t_double_complex *, int *, int, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *); /*proto*/
#define __Pyx_MODULE_NAME "scipy.linalg._decomp_update"
extern int __pyx_module_is_main_scipy__linalg___decomp_update;
int __pyx_module_is_main_scipy__linalg___decomp_update = 0;

/* Implementation of 'scipy.linalg._decomp_update' */
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_MemoryError;
static PyObject *__pyx_builtin_RuntimeError;
static PyObject *__pyx_builtin_ImportError;
static const char __pyx_k_F[] = "F";
static const char __pyx_k_Q[] = "Q";
static const char __pyx_k_R[] = "R";
static const char __pyx_k_T[] = "T";
static const char __pyx_k_a[] = "a";
static const char __pyx_k_b[] = "b";
static const char __pyx_k_k[] = "k";
static const char __pyx_k_m[] = "m";
static const char __pyx_k_n[] = "n";
static const char __pyx_k_o[] = "o";
static const char __pyx_k_p[] = "p";
static const char __pyx_k_q[] = "q";
static const char __pyx_k_s[] = "s";
static const char __pyx_k_u[] = "u";
static const char __pyx_k_v[] = "v";
static const char __pyx_k_k1[] = "k1";
static const char __pyx_k_np[] = "np";
static const char __pyx_k_p1[] = "p1";
static const char __pyx_k_q1[] = "q1";
static const char __pyx_k_qs[] = "qs";
static const char __pyx_k_r1[] = "r1";
static const char __pyx_k_rs[] = "rs";
static const char __pyx_k_ss[] = "ss";
static const char __pyx_k_u1[] = "u1";
static const char __pyx_k_us[] = "us";
static const char __pyx_k_v1[] = "v1";
static const char __pyx_k_vs[] = "vs";
static const char __pyx_k__11[] = " ";
static const char __pyx_k_all[] = "all";
static const char __pyx_k_col[] = "col";
static const char __pyx_k_dot[] = "dot";
static const char __pyx_k_qTu[] = "qTu";
static const char __pyx_k_row[] = "row";
static const char __pyx_k_conj[] = "conj";
static const char __pyx_k_copy[] = "copy";
static const char __pyx_k_full[] = "full";
static const char __pyx_k_info[] = "info";
static const char __pyx_k_intp[] = "intp";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_ndim[] = "ndim";
static const char __pyx_k_norm[] = "norm";
static const char __pyx_k_qTus[] = "qTus";
static const char __pyx_k_qisF[] = "qisF";
static const char __pyx_k_qnew[] = "qnew";
static const char __pyx_k_qptr[] = "qptr";
static const char __pyx_k_rnew[] = "rnew";
static const char __pyx_k_rptr[] = "rptr";
static const char __pyx_k_sptr[] = "sptr";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_uptr[] = "uptr";
static const char __pyx_k_vptr[] = "vptr";
static const char __pyx_k_all_2[] = "__all__";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_p_eco[] = "p_eco";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_ravel[] = "ravel";
static const char __pyx_k_rcond[] = "rcond";
static const char __pyx_k_shape[] = "shape";
static const char __pyx_k_which[] = "which";
static const char __pyx_k_delete[] = "delete";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_insert[] = "insert";
static const char __pyx_k_length[] = "length";
static const char __pyx_k_linalg[] = "linalg";
static const char __pyx_k_p_full[] = "p_full";
static const char __pyx_k_qTuptr[] = "qTuptr";
static const char __pyx_k_qTuvoid[] = "qTuvoid";
static const char __pyx_k_economic[] = "economic";
static const char __pyx_k_form_qTu[] = "_form_qTu";
static const char __pyx_k_isfinite[] = "isfinite";
static const char __pyx_k_typecode[] = "typecode";
static const char __pyx_k_uv_flags[] = "uv_flags";
static const char __pyx_k_chkfinite[] = "chkfinite";
static const char __pyx_k_conjugate[] = "conjugate";
static const char __pyx_k_overwrite[] = "overwrite";
static const char __pyx_k_qr_delete[] = "qr_delete";
static const char __pyx_k_qr_insert[] = "qr_insert";
static const char __pyx_k_qr_update[] = "qr_update";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_ImportError[] = "ImportError";
static const char __pyx_k_LinAlgError[] = "LinAlgError";
static const char __pyx_k_MemoryError[] = "MemoryError";
static const char __pyx_k_RuntimeError[] = "RuntimeError";
static const char __pyx_k_check_finite[] = "check_finite";
static const char __pyx_k_numpy_linalg[] = "numpy.linalg";
static const char __pyx_k_overwrite_qr[] = "overwrite_qr";
static const char __pyx_k_overwrite_qru[] = "overwrite_qru";
static const char __pyx_k_overwrite_qruv[] = "overwrite_qruv";
static const char __pyx_k_decomp_update_pyx[] = "_decomp_update.pyx";
static const char __pyx_k_p_is_out_of_range[] = "'p' is out of range";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_k_is_out_of_bounds[] = "'k' is out of bounds";
static const char __pyx_k_Q_and_R_must_be_2_D[] = "Q and R must be 2-D";
static const char __pyx_k_qr_delete_line_1451[] = "qr_delete (line 1451)";
static const char __pyx_k_qr_insert_line_1695[] = "qr_insert (line 1695)";
static const char __pyx_k_qr_update_line_2159[] = "qr_update (line 2159)";
static const char __pyx_k_Inputs_must_be_arrays[] = "Inputs must be arrays";
static const char __pyx_k_for_Q_and_R_respectively[] = " for Q and R respectively";
static const char __pyx_k_u_must_be_either_1_or_2_D[] = "'u' must be either 1- or 2-D";
static const char __pyx_k_v_must_be_either_1_or_2_D[] = "v must be either 1- or 2-D.";
static const char __pyx_k_ndarray_is_not_C_contiguous[] = "ndarray is not C contiguous";
static const char __pyx_k_scipy_linalg__decomp_update[] = "scipy.linalg._decomp_update";
static const char __pyx_k_u_must_be_either_1_or_2_D_2[] = "u must be either 1- or 2-D.";
static const char __pyx_k_rcond_is_unused_when_inserting[] = "'rcond' is unused when inserting rows and must be None";
static const char __pyx_k_u_must_have_the_same_type_as_Q[] = "'u' must have the same type as 'Q' and 'R'";
static const char __pyx_k_u_shape_0_must_equal_Q_shape_0[] = "u.shape[0] must equal Q.shape[0]";
static const char __pyx_k_v_shape_0_must_equal_R_shape_1[] = "v.shape[0] must equal R.shape[1]";
static const char __pyx_k_Expected_M_M_M_N_or_M_N_N_N_but[] = "Expected (M,M) (M,N) or (M,N) (N,N) but found ";
static const char __pyx_k_One_of_the_columns_of_u_lies_in[] = "One of the columns of u lies in the span of Q. Found reciprocal condition number of %s for Q augmented with u/||u||.";
static const char __pyx_k_Only_arrays_with_dtypes_float32[] = "Only arrays with dtypes float32, float64, complex64, and complex128 are supported.";
static const char __pyx_k_QR_downdate_on_row_or_column_de[] = "\n    QR downdate on row or column deletions\n\n    If ``A = Q R`` is the QR factorization of ``A``, return the QR\n    factorization of ``A`` where ``p`` rows or columns have been removed\n    starting at row or column ``k``.\n\n    Parameters\n    ----------\n    Q : (M, M) or (M, N) array_like\n        Unitary/orthogonal matrix from QR decomposition.\n    R : (M, N) or (N, N) array_like\n        Upper triangular matrix from QR decomposition.\n    k : int\n        Index of the first row or column to delete.\n    p : int, optional\n        Number of rows or columns to delete, defaults to 1.\n    which: {'row', 'col'}, optional\n        Determines if rows or columns will be deleted, defaults to 'row'\n    overwrite_qr : bool, optional\n        If True, consume Q and R, overwriting their contents with their\n        downdated versions, and returning approriately sized views.\n        Defaults to False.\n    check_finite : bool, optional\n        Whether to check that the input matrix contains only finite numbers.\n        Disabling may give a performance gain, but may result in problems\n        (crashes, non-termination) if the inputs do contain infinities or NaNs.\n        Default is True.\n\n    Returns\n    -------\n    Q1 : ndarray\n        Updated unitary/orthogonal factor\n    R1 : ndarray\n        Updated upper triangular factor\n\n    See Also\n    --------\n    qr, qr_multiply, qr_insert, qr_update\n\n    Notes\n    -----\n    This routine does not guarantee that the diagonal entries of ``R1`` are\n    positive.\n\n    .. versionadded:: 0.16.0\n\n    References\n    ----------\n    .. [1] Golub, G. H. & Van Loan, C. F. Matrix Computations, 3rd Ed.\n           (Johns Hopkins University Press, 1996).\n\n    .. [2] Daniel, J. W., Gragg, W. B., Kaufman, L. & Stewart, G. W.\n           Reorthogonalization and stable algorithms for updating the\n           Gram-Schmidt QR factorization. Math. Comput. 30, 772-795 (1976).\n\n    .. [3] Reichel, L. & Gragg, W. B. Al""gorithm 686: FORTRAN Subroutines for\n           Updating the QR Decomposition. ACM Trans. Math. Softw. 16, 369-377\n           (1990).\n\n    Examples\n    --------\n    >>> from scipy import linalg\n    >>> a = np.array([[  3.,  -2.,  -2.],\n    ...               [  6.,  -9.,  -3.],\n    ...               [ -3.,  10.,   1.],\n    ...               [  6.,  -7.,   4.],\n    ...               [  7.,   8.,  -6.]])\n    >>> q, r = linalg.qr(a)\n\n    Given this QR decomposition, update q and r when 2 rows are removed.\n\n    >>> q1, r1 = linalg.qr_delete(q, r, 2, 2, 'row', False)\n    >>> q1\n    array([[ 0.30942637,  0.15347579,  0.93845645],  # may vary (signs)\n           [ 0.61885275,  0.71680171, -0.32127338],\n           [ 0.72199487, -0.68017681, -0.12681844]])\n    >>> r1\n    array([[  9.69535971,  -0.4125685 ,  -6.80738023],  # may vary (signs)\n           [  0.        , -12.19958144,   1.62370412],\n           [  0.        ,   0.        ,  -0.15218213]])\n\n    The update is equivalent, but faster than the following.\n\n    >>> a1 = np.delete(a, slice(2,4), 0)\n    >>> a1\n    array([[ 3., -2., -2.],\n           [ 6., -9., -3.],\n           [ 7.,  8., -6.]])\n    >>> q_direct, r_direct = linalg.qr(a1)\n\n    Check that we have equivalent results:\n\n    >>> np.dot(q1, r1)\n    array([[ 3., -2., -2.],\n           [ 6., -9., -3.],\n           [ 7.,  8., -6.]])\n    >>> np.allclose(np.dot(q1, r1), a1)\n    True\n\n    And the updated Q is still unitary:\n\n    >>> np.allclose(np.dot(q1.T, q1), np.eye(3))\n    True\n\n    ";
static const char __pyx_k_QR_update_on_row_or_column_inse[] = "\n    QR update on row or column insertions\n\n    If ``A = Q R`` is the QR factorization of ``A``, return the QR\n    factorization of ``A`` where rows or columns have been inserted starting\n    at row or column ``k``.\n\n    Parameters\n    ----------\n    Q : (M, M) array_like\n        Unitary/orthogonal matrix from the QR decomposition of A.\n    R : (M, N) array_like\n        Upper triangular matrix from the QR decomposition of A.\n    u : (N,), (p, N), (M,), or (M, p) array_like\n        Rows or columns to insert\n    k : int\n        Index before which `u` is to be inserted.\n    which: {'row', 'col'}, optional\n        Determines if rows or columns will be inserted, defaults to 'row'\n    rcond : float\n        Lower bound on the reciprocal condition number of ``Q`` augmented with\n        ``u/||u||`` Only used when updating economic mode (thin, (M,N) (N,N))\n        decompositions.  If None, machine precision is used.  Defaults to\n        None.\n    overwrite_qru : bool, optional\n        If True, consume Q, R, and u, if possible, while performing the update,\n        otherwise make copies as necessary. Defaults to False.\n    check_finite : bool, optional\n        Whether to check that the input matrices contain only finite numbers.\n        Disabling may give a performance gain, but may result in problems\n        (crashes, non-termination) if the inputs do contain infinities or NaNs.\n        Default is True.\n\n    Returns\n    -------\n    Q1 : ndarray\n        Updated unitary/orthogonal factor\n    R1 : ndarray\n        Updated upper triangular factor\n\n    Raises\n    ------\n    LinAlgError :\n        If updating a (M,N) (N,N) factorization and the reciprocal condition\n        number of Q augmented with u/||u|| is smaller than rcond.\n\n    See Also\n    --------\n    qr, qr_multiply, qr_delete, qr_update\n\n    Notes\n    -----\n    This routine does not guarantee that the diagonal entries of ``R1`` are\n    positive.\n\n    .. versionadded:: 0"".16.0\n\n    References\n    ----------\n\n    .. [1] Golub, G. H. & Van Loan, C. F. Matrix Computations, 3rd Ed.\n           (Johns Hopkins University Press, 1996).\n\n    .. [2] Daniel, J. W., Gragg, W. B., Kaufman, L. & Stewart, G. W.\n           Reorthogonalization and stable algorithms for updating the\n           Gram-Schmidt QR factorization. Math. Comput. 30, 772-795 (1976).\n\n    .. [3] Reichel, L. & Gragg, W. B. Algorithm 686: FORTRAN Subroutines for\n           Updating the QR Decomposition. ACM Trans. Math. Softw. 16, 369-377\n           (1990).\n\n    Examples\n    --------\n    >>> from scipy import linalg\n    >>> a = np.array([[  3.,  -2.,  -2.],\n    ...               [  6.,  -7.,   4.],\n    ...               [  7.,   8.,  -6.]])\n    >>> q, r = linalg.qr(a)\n\n    Given this QR decomposition, update q and r when 2 rows are inserted.\n\n    >>> u = np.array([[  6.,  -9.,  -3.],\n    ...               [ -3.,  10.,   1.]])\n    >>> q1, r1 = linalg.qr_insert(q, r, u, 2, 'row')\n    >>> q1\n    array([[-0.25445668,  0.02246245,  0.18146236, -0.72798806,  0.60979671],  # may vary (signs)\n           [-0.50891336,  0.23226178, -0.82836478, -0.02837033, -0.00828114],\n           [-0.50891336,  0.35715302,  0.38937158,  0.58110733,  0.35235345],\n           [ 0.25445668, -0.52202743, -0.32165498,  0.36263239,  0.65404509],\n           [-0.59373225, -0.73856549,  0.16065817, -0.0063658 , -0.27595554]])\n    >>> r1\n    array([[-11.78982612,   6.44623587,   3.81685018],  # may vary (signs)\n           [  0.        , -16.01393278,   3.72202865],\n           [  0.        ,   0.        ,  -6.13010256],\n           [  0.        ,   0.        ,   0.        ],\n           [  0.        ,   0.        ,   0.        ]])\n\n    The update is equivalent, but faster than the following.\n\n    >>> a1 = np.insert(a, 2, u, 0)\n    >>> a1\n    array([[  3.,  -2.,  -2.],\n           [  6.,  -7.,   4.],\n           [  6.,  -9.,  -3.],\n           [ -3.,  10.,   1.],\n        ""   [  7.,   8.,  -6.]])\n    >>> q_direct, r_direct = linalg.qr(a1)\n\n    Check that we have equivalent results:\n\n    >>> np.dot(q1, r1)\n    array([[  3.,  -2.,  -2.],\n           [  6.,  -7.,   4.],\n           [  6.,  -9.,  -3.],\n           [ -3.,  10.,   1.],\n           [  7.,   8.,  -6.]])\n\n    >>> np.allclose(np.dot(q1, r1), a1)\n    True\n\n    And the updated Q is still unitary:\n\n    >>> np.allclose(np.dot(q1.T, q1), np.eye(5))\n    True\n\n    ";
static const char __pyx_k_Rank_k_QR_update_If_A_Q_R_is_th[] = "\n    Rank-k QR update\n\n    If ``A = Q R`` is the QR factorization of ``A``, return the QR\n    factorization of ``A + u v**T`` for real ``A`` or ``A + u v**H``\n    for complex ``A``.\n\n    Parameters\n    ----------\n    Q : (M, M) or (M, N) array_like\n        Unitary/orthogonal matrix from the qr decomposition of A.\n    R : (M, N) or (N, N) array_like\n        Upper triangular matrix from the qr decomposition of A.\n    u : (M,) or (M, k) array_like\n        Left update vector\n    v : (N,) or (N, k) array_like\n        Right update vector\n    overwrite_qruv : bool, optional\n        If True, consume Q, R, u, and v, if possible, while performing the\n        update, otherwise make copies as necessary. Defaults to False.\n    check_finite : bool, optional\n        Whether to check that the input matrix contains only finite numbers.\n        Disabling may give a performance gain, but may result in problems\n        (crashes, non-termination) if the inputs do contain infinities or NaNs.\n        Default is True.\n\n    Returns\n    -------\n    Q1 : ndarray\n        Updated unitary/orthogonal factor\n    R1 : ndarray\n        Updated upper triangular factor\n\n    See Also\n    --------\n    qr, qr_multiply, qr_delete, qr_insert\n\n    Notes\n    -----\n    This routine does not guarantee that the diagonal entries of `R1` are\n    real or positive.\n\n    .. versionadded:: 0.16.0\n\n    References\n    ----------\n    .. [1] Golub, G. H. & Van Loan, C. F. Matrix Computations, 3rd Ed.\n           (Johns Hopkins University Press, 1996).\n\n    .. [2] Daniel, J. W., Gragg, W. B., Kaufman, L. & Stewart, G. W.\n           Reorthogonalization and stable algorithms for updating the\n           Gram-Schmidt QR factorization. Math. Comput. 30, 772-795 (1976).\n\n    .. [3] Reichel, L. & Gragg, W. B. Algorithm 686: FORTRAN Subroutines for\n           Updating the QR Decomposition. ACM Trans. Math. Softw. 16, 369-377\n           (1990).\n\n    Examples\n    --------\n   "" >>> from scipy import linalg\n    >>> a = np.array([[  3.,  -2.,  -2.],\n    ...               [  6.,  -9.,  -3.],\n    ...               [ -3.,  10.,   1.],\n    ...               [  6.,  -7.,   4.],\n    ...               [  7.,   8.,  -6.]])\n    >>> q, r = linalg.qr(a)\n\n    Given this q, r decomposition, perform a rank 1 update.\n\n    >>> u = np.array([7., -2., 4., 3., 5.])\n    >>> v = np.array([1., 3., -5.])\n    >>> q_up, r_up = linalg.qr_update(q, r, u, v, False)\n    >>> q_up\n    array([[ 0.54073807,  0.18645997,  0.81707661, -0.02136616,  0.06902409],  # may vary (signs)\n           [ 0.21629523, -0.63257324,  0.06567893,  0.34125904, -0.65749222],\n           [ 0.05407381,  0.64757787, -0.12781284, -0.20031219, -0.72198188],\n           [ 0.48666426, -0.30466718, -0.27487277, -0.77079214,  0.0256951 ],\n           [ 0.64888568,  0.23001   , -0.4859845 ,  0.49883891,  0.20253783]])\n    >>> r_up\n    array([[ 18.49324201,  24.11691794, -44.98940746],  # may vary (signs)\n           [  0.        ,  31.95894662, -27.40998201],\n           [  0.        ,   0.        ,  -9.25451794],\n           [  0.        ,   0.        ,   0.        ],\n           [  0.        ,   0.        ,   0.        ]])\n\n    The update is equivalent, but faster than the following.\n\n    >>> a_up = a + np.outer(u, v)\n    >>> q_direct, r_direct = linalg.qr(a_up)\n\n    Check that we have equivalent results:\n\n    >>> np.allclose(np.dot(q_up, r_up), a_up)\n    True\n\n    And the updated Q is still unitary:\n\n    >>> np.allclose(np.dot(q_up.T, q_up), np.eye(5))\n    True\n\n    Updating economic (reduced, thin) decompositions is also possible:\n\n    >>> qe, re = linalg.qr(a, mode='economic')\n    >>> qe_up, re_up = linalg.qr_update(qe, re, u, v, False)\n    >>> qe_up\n    array([[ 0.54073807,  0.18645997,  0.81707661],  # may vary (signs)\n           [ 0.21629523, -0.63257324,  0.06567893],\n           [ 0.05407381,  0.64757787, -0.12781284],\n           [ 0.48666426, -0.30466""718, -0.27487277],\n           [ 0.64888568,  0.23001   , -0.4859845 ]])\n    >>> re_up\n    array([[ 18.49324201,  24.11691794, -44.98940746],  # may vary (signs)\n           [  0.        ,  31.95894662, -27.40998201],\n           [  0.        ,   0.        ,  -9.25451794]])\n    >>> np.allclose(np.dot(qe_up, re_up), a_up)\n    True\n    >>> np.allclose(np.dot(qe_up.T, qe_up), np.eye(3))\n    True\n\n    Similarly to the above, perform a rank 2 update.\n\n    >>> u2 = np.array([[ 7., -1,],\n    ...                [-2.,  4.],\n    ...                [ 4.,  2.],\n    ...                [ 3., -6.],\n    ...                [ 5.,  3.]])\n    >>> v2 = np.array([[ 1., 2.],\n    ...                [ 3., 4.],\n    ...                [-5., 2]])\n    >>> q_up2, r_up2 = linalg.qr_update(q, r, u2, v2, False)\n    >>> q_up2\n    array([[-0.33626508, -0.03477253,  0.61956287, -0.64352987, -0.29618884],  # may vary (signs)\n           [-0.50439762,  0.58319694, -0.43010077, -0.33395279,  0.33008064],\n           [-0.21016568, -0.63123106,  0.0582249 , -0.13675572,  0.73163206],\n           [ 0.12609941,  0.49694436,  0.64590024,  0.31191919,  0.47187344],\n           [-0.75659643, -0.11517748,  0.10284903,  0.5986227 , -0.21299983]])\n    >>> r_up2\n    array([[-23.79075451, -41.1084062 ,  24.71548348],  # may vary (signs)\n           [  0.        , -33.83931057,  11.02226551],\n           [  0.        ,   0.        ,  48.91476811],\n           [  0.        ,   0.        ,   0.        ],\n           [  0.        ,   0.        ,   0.        ]])\n\n    This update is also a valid qr decomposition of ``A + U V**T``.\n\n    >>> a_up2 = a + np.dot(u2, v2.T)\n    >>> np.allclose(a_up2, np.dot(q_up2, r_up2))\n    True\n    >>> np.allclose(np.dot(q_up2.T, q_up2), np.eye(5))\n    True\n\n    ";
static const char __pyx_k_Routines_for_updating_QR_decomp[] = "\nRoutines for updating QR decompositions\n\n.. versionadded: 0.16.0\n\n";
static const char __pyx_k_numpy_core_multiarray_failed_to[] = "numpy.core.multiarray failed to import";
static const char __pyx_k_q_and_u_must_have_the_same_type[] = "q and u must have the same type.";
static const char __pyx_k_rcond_is_not_used_when_updating[] = "'rcond' is not used when updating full, (M,M) (M,N) decompositions and must be None.";
static const char __pyx_k_u_and_v_must_have_the_same_type[] = "u and v must have the same type as Q and R";
static const char __pyx_k_u_should_be_either_M_or_M_p_whe[] = "'u' should be either (M,) or (M,p) when inserting columns. Found %s.";
static const char __pyx_k_u_should_be_either_N_or_p_N_whe[] = "'u' should be either (N,) or (p,N) when inserting rows. Found %s.";
static const char __pyx_k_unknown_dtype_code_in_numpy_pxd[] = "unknown dtype code in numpy.pxd (%d)";
static const char __pyx_k_which_must_be_either_row_or_col[] = "'which' must be either 'row' or 'col'";
static const char __pyx_k_Format_string_allocated_too_shor[] = "Format string allocated too short, see comment in numpy.pxd";
static const char __pyx_k_Input_array_too_large_for_use_wi[] = "Input array too large for use with BLAS";
static const char __pyx_k_Non_native_byte_order_not_suppor[] = "Non-native byte order not supported";
static const char __pyx_k_Q_and_R_do_not_have_compatible_s[] = "Q and R do not have compatible shapes. Expected (M,M) (M,N) or (M,N) (N,N) but found ";
static const char __pyx_k_Q_and_R_must_have_the_same_dtype[] = "Q and R must have the same dtype";
static const char __pyx_k_Reorthogonalization_Failed_unabl[] = "Reorthogonalization Failed, unable to perform row deletion.";
static const char __pyx_k_Second_dimension_of_u_and_v_must[] = "Second dimension of u and v must be the same";
static const char __pyx_k_Shape_of_u_is_incorrect_should_b[] = "Shape of u is incorrect, should be 1 <= u.ndim <= 2";
static const char __pyx_k_The_0_th_argument_to_geqrf_was_i[] = "The {0}th argument to ?geqrf was invalid";
static const char __pyx_k_The_0_th_argument_to_ormqr_unmqr[] = "The {0}th argument to ?ormqr/?unmqr was invalid";
static const char __pyx_k_Unable_to_allocate_memory_for_ar[] = "Unable to allocate memory for array.";
static const char __pyx_k_Update_rank_larger_than_np_dot_Q[] = "Update rank larger than np.dot(Q, R).";
static const char __pyx_k_array_must_not_contain_infs_or_N[] = "array must not contain infs or NaNs";
static const char __pyx_k_ndarray_is_not_Fortran_contiguou[] = "ndarray is not Fortran contiguous";
static const char __pyx_k_numpy_core_umath_failed_to_impor[] = "numpy.core.umath failed to import";
static const char __pyx_k_q_and_u_must_be_a_blas_compatibl[] = "q and u must be a blas compatible type: f d F or D";
static const char __pyx_k_q_must_be_either_F_or_C_contiguo[] = "q must be either F or C contiguous";
static const char __pyx_k_u_and_v_must_have_the_same_numbe[] = "u and v must have the same number of dimensions";
static const char __pyx_k_u_must_have_the_same_type_as_Q_2[] = "'u' must have the same type as Q and R";
static const char __pyx_k_Format_string_allocated_too_shor_2[] = "Format string allocated too short.";
static const char __pyx_k_Unable_to_allocate_memory_for_ar_2[] = "Unable to allocate memory for array";
static PyObject *__pyx_kp_u_Expected_M_M_M_N_or_M_N_N_N_but;
static PyObject *__pyx_n_u_F;
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor;
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor_2;
static PyObject *__pyx_n_s_ImportError;
static PyObject *__pyx_kp_u_Input_array_too_large_for_use_wi;
static PyObject *__pyx_kp_u_Inputs_must_be_arrays;
static PyObject *__pyx_n_s_LinAlgError;
static PyObject *__pyx_n_s_MemoryError;
static PyObject *__pyx_kp_u_Non_native_byte_order_not_suppor;
static PyObject *__pyx_kp_u_One_of_the_columns_of_u_lies_in;
static PyObject *__pyx_kp_u_Only_arrays_with_dtypes_float32;
static PyObject *__pyx_n_s_Q;
static PyObject *__pyx_kp_u_QR_downdate_on_row_or_column_de;
static PyObject *__pyx_kp_u_QR_update_on_row_or_column_inse;
static PyObject *__pyx_kp_u_Q_and_R_do_not_have_compatible_s;
static PyObject *__pyx_kp_u_Q_and_R_must_be_2_D;
static PyObject *__pyx_kp_u_Q_and_R_must_have_the_same_dtype;
static PyObject *__pyx_n_s_R;
static PyObject *__pyx_kp_u_Rank_k_QR_update_If_A_Q_R_is_th;
static PyObject *__pyx_kp_u_Reorthogonalization_Failed_unabl;
static PyObject *__pyx_n_s_RuntimeError;
static PyObject *__pyx_kp_u_Second_dimension_of_u_and_v_must;
static PyObject *__pyx_kp_u_Shape_of_u_is_incorrect_should_b;
static PyObject *__pyx_n_s_T;
static PyObject *__pyx_kp_u_The_0_th_argument_to_geqrf_was_i;
static PyObject *__pyx_kp_u_The_0_th_argument_to_ormqr_unmqr;
static PyObject *__pyx_kp_u_Unable_to_allocate_memory_for_ar;
static PyObject *__pyx_kp_u_Unable_to_allocate_memory_for_ar_2;
static PyObject *__pyx_kp_u_Update_rank_larger_than_np_dot_Q;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_kp_u__11;
static PyObject *__pyx_n_s_a;
static PyObject *__pyx_n_s_all;
static PyObject *__pyx_n_s_all_2;
static PyObject *__pyx_kp_u_array_must_not_contain_infs_or_N;
static PyObject *__pyx_n_s_b;
static PyObject *__pyx_n_s_check_finite;
static PyObject *__pyx_n_s_chkfinite;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_u_col;
static PyObject *__pyx_n_s_conj;
static PyObject *__pyx_n_s_conjugate;
static PyObject *__pyx_n_s_copy;
static PyObject *__pyx_kp_s_decomp_update_pyx;
static PyObject *__pyx_n_s_delete;
static PyObject *__pyx_n_s_dot;
static PyObject *__pyx_n_s_economic;
static PyObject *__pyx_kp_u_for_Q_and_R_respectively;
static PyObject *__pyx_n_s_form_qTu;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_full;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_info;
static PyObject *__pyx_n_s_insert;
static PyObject *__pyx_n_s_intp;
static PyObject *__pyx_n_s_isfinite;
static PyObject *__pyx_n_s_k;
static PyObject *__pyx_n_s_k1;
static PyObject *__pyx_kp_u_k_is_out_of_bounds;
static PyObject *__pyx_n_s_length;
static PyObject *__pyx_n_s_linalg;
static PyObject *__pyx_n_s_m;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_n;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_kp_u_ndarray_is_not_C_contiguous;
static PyObject *__pyx_kp_u_ndarray_is_not_Fortran_contiguou;
static PyObject *__pyx_n_s_ndim;
static PyObject *__pyx_n_s_norm;
static PyObject *__pyx_n_s_np;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_kp_u_numpy_core_multiarray_failed_to;
static PyObject *__pyx_kp_u_numpy_core_umath_failed_to_impor;
static PyObject *__pyx_n_s_numpy_linalg;
static PyObject *__pyx_n_s_o;
static PyObject *__pyx_n_s_overwrite;
static PyObject *__pyx_n_s_overwrite_qr;
static PyObject *__pyx_n_s_overwrite_qru;
static PyObject *__pyx_n_s_overwrite_qruv;
static PyObject *__pyx_n_s_p;
static PyObject *__pyx_n_s_p1;
static PyObject *__pyx_n_s_p_eco;
static PyObject *__pyx_n_s_p_full;
static PyObject *__pyx_kp_u_p_is_out_of_range;
static PyObject *__pyx_n_s_q;
static PyObject *__pyx_n_s_q1;
static PyObject *__pyx_n_s_qTu;
static PyObject *__pyx_n_s_qTuptr;
static PyObject *__pyx_n_s_qTus;
static PyObject *__pyx_n_s_qTuvoid;
static PyObject *__pyx_kp_u_q_and_u_must_be_a_blas_compatibl;
static PyObject *__pyx_kp_u_q_and_u_must_have_the_same_type;
static PyObject *__pyx_kp_u_q_must_be_either_F_or_C_contiguo;
static PyObject *__pyx_n_s_qisF;
static PyObject *__pyx_n_s_qnew;
static PyObject *__pyx_n_s_qptr;
static PyObject *__pyx_n_s_qr_delete;
static PyObject *__pyx_n_u_qr_delete;
static PyObject *__pyx_kp_u_qr_delete_line_1451;
static PyObject *__pyx_n_s_qr_insert;
static PyObject *__pyx_n_u_qr_insert;
static PyObject *__pyx_kp_u_qr_insert_line_1695;
static PyObject *__pyx_n_s_qr_update;
static PyObject *__pyx_n_u_qr_update;
static PyObject *__pyx_kp_u_qr_update_line_2159;
static PyObject *__pyx_n_s_qs;
static PyObject *__pyx_n_s_r1;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_ravel;
static PyObject *__pyx_n_s_rcond;
static PyObject *__pyx_kp_u_rcond_is_not_used_when_updating;
static PyObject *__pyx_kp_u_rcond_is_unused_when_inserting;
static PyObject *__pyx_n_s_rnew;
static PyObject *__pyx_n_u_row;
static PyObject *__pyx_n_s_rptr;
static PyObject *__pyx_n_s_rs;
static PyObject *__pyx_n_s_s;
static PyObject *__pyx_n_s_scipy_linalg__decomp_update;
static PyObject *__pyx_n_u_shape;
static PyObject *__pyx_n_s_sptr;
static PyObject *__pyx_n_s_ss;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_typecode;
static PyObject *__pyx_n_s_u;
static PyObject *__pyx_n_s_u1;
static PyObject *__pyx_kp_u_u_and_v_must_have_the_same_numbe;
static PyObject *__pyx_kp_u_u_and_v_must_have_the_same_type;
static PyObject *__pyx_kp_u_u_must_be_either_1_or_2_D;
static PyObject *__pyx_kp_u_u_must_be_either_1_or_2_D_2;
static PyObject *__pyx_kp_u_u_must_have_the_same_type_as_Q;
static PyObject *__pyx_kp_u_u_must_have_the_same_type_as_Q_2;
static PyObject *__pyx_kp_u_u_shape_0_must_equal_Q_shape_0;
static PyObject *__pyx_kp_u_u_should_be_either_M_or_M_p_whe;
static PyObject *__pyx_kp_u_u_should_be_either_N_or_p_N_whe;
static PyObject *__pyx_kp_u_unknown_dtype_code_in_numpy_pxd;
static PyObject *__pyx_n_s_uptr;
static PyObject *__pyx_n_s_us;
static PyObject *__pyx_n_s_uv_flags;
static PyObject *__pyx_n_s_v;
static PyObject *__pyx_n_s_v1;
static PyObject *__pyx_kp_u_v_must_be_either_1_or_2_D;
static PyObject *__pyx_kp_u_v_shape_0_must_equal_R_shape_1;
static PyObject *__pyx_n_s_vptr;
static PyObject *__pyx_n_s_vs;
static PyObject *__pyx_n_s_which;
static PyObject *__pyx_kp_u_which_must_be_either_row_or_col;
static PyObject *__pyx_pf_5scipy_6linalg_14_decomp_update__form_qTu(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_5scipy_6linalg_14_decomp_update_2qr_delete(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_Q, PyObject *__pyx_v_R, PyObject *__pyx_v_k, int __pyx_v_p, PyObject *__pyx_v_which, PyObject *__pyx_v_overwrite_qr, PyObject *__pyx_v_check_finite); /* proto */
static PyObject *__pyx_pf_5scipy_6linalg_14_decomp_update_4qr_insert(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_Q, PyObject *__pyx_v_R, PyObject *__pyx_v_u, PyObject *__pyx_v_k, PyObject *__pyx_v_which, PyObject *__pyx_v_rcond, PyObject *__pyx_v_overwrite_qru, PyObject *__pyx_v_check_finite); /* proto */
static PyObject *__pyx_pf_5scipy_6linalg_14_decomp_update_6qr_update(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_Q, PyObject *__pyx_v_R, PyObject *__pyx_v_u, PyObject *__pyx_v_v, PyObject *__pyx_v_overwrite_qruv, PyObject *__pyx_v_check_finite); /* proto */
static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info); /* proto */
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_slice__14;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__30;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__32;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_tuple__34;
static PyObject *__pyx_tuple__35;
static PyObject *__pyx_tuple__36;
static PyObject *__pyx_tuple__37;
static PyObject *__pyx_tuple__38;
static PyObject *__pyx_tuple__39;
static PyObject *__pyx_tuple__40;
static PyObject *__pyx_tuple__42;
static PyObject *__pyx_tuple__44;
static PyObject *__pyx_tuple__46;
static PyObject *__pyx_codeobj__41;
static PyObject *__pyx_codeobj__43;
static PyObject *__pyx_codeobj__45;
static PyObject *__pyx_codeobj__47;
/* Late includes */

/* "scipy/linalg/_decomp_update.pyx":83
 *     double_complex
 * 
 * cdef inline blas_t* index2(blas_t* a, int* as, int i, int j) nogil:             # <<<<<<<<<<<<<<
 *     return a + i*as[0] + j*as[1]
 * 
 */

static CYTHON_INLINE float *__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(float *__pyx_v_a, int *__pyx_v_as, int __pyx_v_i, int __pyx_v_j) {
  float *__pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":84
 * 
 * cdef inline blas_t* index2(blas_t* a, int* as, int i, int j) nogil:
 *     return a + i*as[0] + j*as[1]             # <<<<<<<<<<<<<<
 * 
 * cdef inline blas_t* index1(blas_t* a, int* as, int i) nogil:
 */
  __pyx_r = ((__pyx_v_a + (__pyx_v_i * (__pyx_v_as[0]))) + (__pyx_v_j * (__pyx_v_as[1])));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":83
 *     double_complex
 * 
 * cdef inline blas_t* index2(blas_t* a, int* as, int i, int j) nogil:             # <<<<<<<<<<<<<<
 *     return a + i*as[0] + j*as[1]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE double *__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(double *__pyx_v_a, int *__pyx_v_as, int __pyx_v_i, int __pyx_v_j) {
  double *__pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":84
 * 
 * cdef inline blas_t* index2(blas_t* a, int* as, int i, int j) nogil:
 *     return a + i*as[0] + j*as[1]             # <<<<<<<<<<<<<<
 * 
 * cdef inline blas_t* index1(blas_t* a, int* as, int i) nogil:
 */
  __pyx_r = ((__pyx_v_a + (__pyx_v_i * (__pyx_v_as[0]))) + (__pyx_v_j * (__pyx_v_as[1])));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":83
 *     double_complex
 * 
 * cdef inline blas_t* index2(blas_t* a, int* as, int i, int j) nogil:             # <<<<<<<<<<<<<<
 *     return a + i*as[0] + j*as[1]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_float_complex *__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_t_float_complex *__pyx_v_a, int *__pyx_v_as, int __pyx_v_i, int __pyx_v_j) {
  __pyx_t_float_complex *__pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":84
 * 
 * cdef inline blas_t* index2(blas_t* a, int* as, int i, int j) nogil:
 *     return a + i*as[0] + j*as[1]             # <<<<<<<<<<<<<<
 * 
 * cdef inline blas_t* index1(blas_t* a, int* as, int i) nogil:
 */
  __pyx_r = ((__pyx_v_a + (__pyx_v_i * (__pyx_v_as[0]))) + (__pyx_v_j * (__pyx_v_as[1])));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":83
 *     double_complex
 * 
 * cdef inline blas_t* index2(blas_t* a, int* as, int i, int j) nogil:             # <<<<<<<<<<<<<<
 *     return a + i*as[0] + j*as[1]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex *__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_t_double_complex *__pyx_v_a, int *__pyx_v_as, int __pyx_v_i, int __pyx_v_j) {
  __pyx_t_double_complex *__pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":84
 * 
 * cdef inline blas_t* index2(blas_t* a, int* as, int i, int j) nogil:
 *     return a + i*as[0] + j*as[1]             # <<<<<<<<<<<<<<
 * 
 * cdef inline blas_t* index1(blas_t* a, int* as, int i) nogil:
 */
  __pyx_r = ((__pyx_v_a + (__pyx_v_i * (__pyx_v_as[0]))) + (__pyx_v_j * (__pyx_v_as[1])));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":83
 *     double_complex
 * 
 * cdef inline blas_t* index2(blas_t* a, int* as, int i, int j) nogil:             # <<<<<<<<<<<<<<
 *     return a + i*as[0] + j*as[1]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":86
 *     return a + i*as[0] + j*as[1]
 * 
 * cdef inline blas_t* index1(blas_t* a, int* as, int i) nogil:             # <<<<<<<<<<<<<<
 *     return a + i*as[0]
 * 
 */

static CYTHON_INLINE float *__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index1(float *__pyx_v_a, int *__pyx_v_as, int __pyx_v_i) {
  float *__pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":87
 * 
 * cdef inline blas_t* index1(blas_t* a, int* as, int i) nogil:
 *     return a + i*as[0]             # <<<<<<<<<<<<<<
 * 
 * cdef inline blas_t* row(blas_t* a, int* as, int i) nogil:
 */
  __pyx_r = (__pyx_v_a + (__pyx_v_i * (__pyx_v_as[0])));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":86
 *     return a + i*as[0] + j*as[1]
 * 
 * cdef inline blas_t* index1(blas_t* a, int* as, int i) nogil:             # <<<<<<<<<<<<<<
 *     return a + i*as[0]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE double *__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index1(double *__pyx_v_a, int *__pyx_v_as, int __pyx_v_i) {
  double *__pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":87
 * 
 * cdef inline blas_t* index1(blas_t* a, int* as, int i) nogil:
 *     return a + i*as[0]             # <<<<<<<<<<<<<<
 * 
 * cdef inline blas_t* row(blas_t* a, int* as, int i) nogil:
 */
  __pyx_r = (__pyx_v_a + (__pyx_v_i * (__pyx_v_as[0])));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":86
 *     return a + i*as[0] + j*as[1]
 * 
 * cdef inline blas_t* index1(blas_t* a, int* as, int i) nogil:             # <<<<<<<<<<<<<<
 *     return a + i*as[0]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_float_complex *__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_t_float_complex *__pyx_v_a, int *__pyx_v_as, int __pyx_v_i) {
  __pyx_t_float_complex *__pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":87
 * 
 * cdef inline blas_t* index1(blas_t* a, int* as, int i) nogil:
 *     return a + i*as[0]             # <<<<<<<<<<<<<<
 * 
 * cdef inline blas_t* row(blas_t* a, int* as, int i) nogil:
 */
  __pyx_r = (__pyx_v_a + (__pyx_v_i * (__pyx_v_as[0])));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":86
 *     return a + i*as[0] + j*as[1]
 * 
 * cdef inline blas_t* index1(blas_t* a, int* as, int i) nogil:             # <<<<<<<<<<<<<<
 *     return a + i*as[0]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex *__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_t_double_complex *__pyx_v_a, int *__pyx_v_as, int __pyx_v_i) {
  __pyx_t_double_complex *__pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":87
 * 
 * cdef inline blas_t* index1(blas_t* a, int* as, int i) nogil:
 *     return a + i*as[0]             # <<<<<<<<<<<<<<
 * 
 * cdef inline blas_t* row(blas_t* a, int* as, int i) nogil:
 */
  __pyx_r = (__pyx_v_a + (__pyx_v_i * (__pyx_v_as[0])));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":86
 *     return a + i*as[0] + j*as[1]
 * 
 * cdef inline blas_t* index1(blas_t* a, int* as, int i) nogil:             # <<<<<<<<<<<<<<
 *     return a + i*as[0]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":89
 *     return a + i*as[0]
 * 
 * cdef inline blas_t* row(blas_t* a, int* as, int i) nogil:             # <<<<<<<<<<<<<<
 *     return a + i*as[0]
 * 
 */

static CYTHON_INLINE float *__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(float *__pyx_v_a, int *__pyx_v_as, int __pyx_v_i) {
  float *__pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":90
 * 
 * cdef inline blas_t* row(blas_t* a, int* as, int i) nogil:
 *     return a + i*as[0]             # <<<<<<<<<<<<<<
 * 
 * cdef inline blas_t* col(blas_t* a, int* as, int j) nogil:
 */
  __pyx_r = (__pyx_v_a + (__pyx_v_i * (__pyx_v_as[0])));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":89
 *     return a + i*as[0]
 * 
 * cdef inline blas_t* row(blas_t* a, int* as, int i) nogil:             # <<<<<<<<<<<<<<
 *     return a + i*as[0]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE double *__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(double *__pyx_v_a, int *__pyx_v_as, int __pyx_v_i) {
  double *__pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":90
 * 
 * cdef inline blas_t* row(blas_t* a, int* as, int i) nogil:
 *     return a + i*as[0]             # <<<<<<<<<<<<<<
 * 
 * cdef inline blas_t* col(blas_t* a, int* as, int j) nogil:
 */
  __pyx_r = (__pyx_v_a + (__pyx_v_i * (__pyx_v_as[0])));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":89
 *     return a + i*as[0]
 * 
 * cdef inline blas_t* row(blas_t* a, int* as, int i) nogil:             # <<<<<<<<<<<<<<
 *     return a + i*as[0]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_float_complex *__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_t_float_complex *__pyx_v_a, int *__pyx_v_as, int __pyx_v_i) {
  __pyx_t_float_complex *__pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":90
 * 
 * cdef inline blas_t* row(blas_t* a, int* as, int i) nogil:
 *     return a + i*as[0]             # <<<<<<<<<<<<<<
 * 
 * cdef inline blas_t* col(blas_t* a, int* as, int j) nogil:
 */
  __pyx_r = (__pyx_v_a + (__pyx_v_i * (__pyx_v_as[0])));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":89
 *     return a + i*as[0]
 * 
 * cdef inline blas_t* row(blas_t* a, int* as, int i) nogil:             # <<<<<<<<<<<<<<
 *     return a + i*as[0]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex *__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_t_double_complex *__pyx_v_a, int *__pyx_v_as, int __pyx_v_i) {
  __pyx_t_double_complex *__pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":90
 * 
 * cdef inline blas_t* row(blas_t* a, int* as, int i) nogil:
 *     return a + i*as[0]             # <<<<<<<<<<<<<<
 * 
 * cdef inline blas_t* col(blas_t* a, int* as, int j) nogil:
 */
  __pyx_r = (__pyx_v_a + (__pyx_v_i * (__pyx_v_as[0])));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":89
 *     return a + i*as[0]
 * 
 * cdef inline blas_t* row(blas_t* a, int* as, int i) nogil:             # <<<<<<<<<<<<<<
 *     return a + i*as[0]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":92
 *     return a + i*as[0]
 * 
 * cdef inline blas_t* col(blas_t* a, int* as, int j) nogil:             # <<<<<<<<<<<<<<
 *     return a + j*as[1]
 * 
 */

static CYTHON_INLINE float *__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(float *__pyx_v_a, int *__pyx_v_as, int __pyx_v_j) {
  float *__pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":93
 * 
 * cdef inline blas_t* col(blas_t* a, int* as, int j) nogil:
 *     return a + j*as[1]             # <<<<<<<<<<<<<<
 * 
 * cdef inline void copy(int n, blas_t* x, int incx, blas_t* y, int incy) nogil:
 */
  __pyx_r = (__pyx_v_a + (__pyx_v_j * (__pyx_v_as[1])));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":92
 *     return a + i*as[0]
 * 
 * cdef inline blas_t* col(blas_t* a, int* as, int j) nogil:             # <<<<<<<<<<<<<<
 *     return a + j*as[1]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE double *__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(double *__pyx_v_a, int *__pyx_v_as, int __pyx_v_j) {
  double *__pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":93
 * 
 * cdef inline blas_t* col(blas_t* a, int* as, int j) nogil:
 *     return a + j*as[1]             # <<<<<<<<<<<<<<
 * 
 * cdef inline void copy(int n, blas_t* x, int incx, blas_t* y, int incy) nogil:
 */
  __pyx_r = (__pyx_v_a + (__pyx_v_j * (__pyx_v_as[1])));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":92
 *     return a + i*as[0]
 * 
 * cdef inline blas_t* col(blas_t* a, int* as, int j) nogil:             # <<<<<<<<<<<<<<
 *     return a + j*as[1]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_float_complex *__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_t_float_complex *__pyx_v_a, int *__pyx_v_as, int __pyx_v_j) {
  __pyx_t_float_complex *__pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":93
 * 
 * cdef inline blas_t* col(blas_t* a, int* as, int j) nogil:
 *     return a + j*as[1]             # <<<<<<<<<<<<<<
 * 
 * cdef inline void copy(int n, blas_t* x, int incx, blas_t* y, int incy) nogil:
 */
  __pyx_r = (__pyx_v_a + (__pyx_v_j * (__pyx_v_as[1])));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":92
 *     return a + i*as[0]
 * 
 * cdef inline blas_t* col(blas_t* a, int* as, int j) nogil:             # <<<<<<<<<<<<<<
 *     return a + j*as[1]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex *__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_t_double_complex *__pyx_v_a, int *__pyx_v_as, int __pyx_v_j) {
  __pyx_t_double_complex *__pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":93
 * 
 * cdef inline blas_t* col(blas_t* a, int* as, int j) nogil:
 *     return a + j*as[1]             # <<<<<<<<<<<<<<
 * 
 * cdef inline void copy(int n, blas_t* x, int incx, blas_t* y, int incy) nogil:
 */
  __pyx_r = (__pyx_v_a + (__pyx_v_j * (__pyx_v_as[1])));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":92
 *     return a + i*as[0]
 * 
 * cdef inline blas_t* col(blas_t* a, int* as, int j) nogil:             # <<<<<<<<<<<<<<
 *     return a + j*as[1]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":95
 *     return a + j*as[1]
 * 
 * cdef inline void copy(int n, blas_t* x, int incx, blas_t* y, int incy) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         blas_pointers.scopy(&n, x, &incx, y, &incy)
 */

static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_copy(int __pyx_v_n, float *__pyx_v_x, int __pyx_v_incx, float *__pyx_v_y, int __pyx_v_incy) {

  /* "scipy/linalg/_decomp_update.pyx":97
 * cdef inline void copy(int n, blas_t* x, int incx, blas_t* y, int incy) nogil:
 *     if blas_t is float:
 *         blas_pointers.scopy(&n, x, &incx, y, &incy)             # <<<<<<<<<<<<<<
 *     elif blas_t is double:
 *         blas_pointers.dcopy(&n, x, &incx, y, &incy)
 */
  __pyx_f_5scipy_6linalg_11cython_blas_scopy((&__pyx_v_n), __pyx_v_x, (&__pyx_v_incx), __pyx_v_y, (&__pyx_v_incy));

  /* "scipy/linalg/_decomp_update.pyx":95
 *     return a + j*as[1]
 * 
 * cdef inline void copy(int n, blas_t* x, int incx, blas_t* y, int incy) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         blas_pointers.scopy(&n, x, &incx, y, &incy)
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_copy(int __pyx_v_n, double *__pyx_v_x, int __pyx_v_incx, double *__pyx_v_y, int __pyx_v_incy) {

  /* "scipy/linalg/_decomp_update.pyx":99
 *         blas_pointers.scopy(&n, x, &incx, y, &incy)
 *     elif blas_t is double:
 *         blas_pointers.dcopy(&n, x, &incx, y, &incy)             # <<<<<<<<<<<<<<
 *     elif blas_t is float_complex:
 *         blas_pointers.ccopy(&n, x, &incx, y, &incy)
 */
  __pyx_f_5scipy_6linalg_11cython_blas_dcopy((&__pyx_v_n), __pyx_v_x, (&__pyx_v_incx), __pyx_v_y, (&__pyx_v_incy));

  /* "scipy/linalg/_decomp_update.pyx":95
 *     return a + j*as[1]
 * 
 * cdef inline void copy(int n, blas_t* x, int incx, blas_t* y, int incy) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         blas_pointers.scopy(&n, x, &incx, y, &incy)
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_copy(int __pyx_v_n, __pyx_t_float_complex *__pyx_v_x, int __pyx_v_incx, __pyx_t_float_complex *__pyx_v_y, int __pyx_v_incy) {

  /* "scipy/linalg/_decomp_update.pyx":101
 *         blas_pointers.dcopy(&n, x, &incx, y, &incy)
 *     elif blas_t is float_complex:
 *         blas_pointers.ccopy(&n, x, &incx, y, &incy)             # <<<<<<<<<<<<<<
 *     else:  # blas_t is double_complex:
 *         blas_pointers.zcopy(&n, x, &incx, y, &incy)
 */
  __pyx_f_5scipy_6linalg_11cython_blas_ccopy((&__pyx_v_n), __pyx_v_x, (&__pyx_v_incx), __pyx_v_y, (&__pyx_v_incy));

  /* "scipy/linalg/_decomp_update.pyx":95
 *     return a + j*as[1]
 * 
 * cdef inline void copy(int n, blas_t* x, int incx, blas_t* y, int incy) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         blas_pointers.scopy(&n, x, &incx, y, &incy)
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_copy(int __pyx_v_n, __pyx_t_double_complex *__pyx_v_x, int __pyx_v_incx, __pyx_t_double_complex *__pyx_v_y, int __pyx_v_incy) {

  /* "scipy/linalg/_decomp_update.pyx":103
 *         blas_pointers.ccopy(&n, x, &incx, y, &incy)
 *     else:  # blas_t is double_complex:
 *         blas_pointers.zcopy(&n, x, &incx, y, &incy)             # <<<<<<<<<<<<<<
 * 
 * cdef inline void swap(int n, blas_t* x, int incx, blas_t* y, int incy) nogil:
 */
  __pyx_f_5scipy_6linalg_11cython_blas_zcopy((&__pyx_v_n), __pyx_v_x, (&__pyx_v_incx), __pyx_v_y, (&__pyx_v_incy));

  /* "scipy/linalg/_decomp_update.pyx":95
 *     return a + j*as[1]
 * 
 * cdef inline void copy(int n, blas_t* x, int incx, blas_t* y, int incy) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         blas_pointers.scopy(&n, x, &incx, y, &incy)
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":105
 *         blas_pointers.zcopy(&n, x, &incx, y, &incy)
 * 
 * cdef inline void swap(int n, blas_t* x, int incx, blas_t* y, int incy) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         blas_pointers.sswap(&n, x, &incx, y, &incy)
 */

static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_swap(int __pyx_v_n, float *__pyx_v_x, int __pyx_v_incx, float *__pyx_v_y, int __pyx_v_incy) {

  /* "scipy/linalg/_decomp_update.pyx":107
 * cdef inline void swap(int n, blas_t* x, int incx, blas_t* y, int incy) nogil:
 *     if blas_t is float:
 *         blas_pointers.sswap(&n, x, &incx, y, &incy)             # <<<<<<<<<<<<<<
 *     elif blas_t is double:
 *         blas_pointers.dswap(&n, x, &incx, y, &incy)
 */
  __pyx_f_5scipy_6linalg_11cython_blas_sswap((&__pyx_v_n), __pyx_v_x, (&__pyx_v_incx), __pyx_v_y, (&__pyx_v_incy));

  /* "scipy/linalg/_decomp_update.pyx":105
 *         blas_pointers.zcopy(&n, x, &incx, y, &incy)
 * 
 * cdef inline void swap(int n, blas_t* x, int incx, blas_t* y, int incy) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         blas_pointers.sswap(&n, x, &incx, y, &incy)
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_swap(int __pyx_v_n, double *__pyx_v_x, int __pyx_v_incx, double *__pyx_v_y, int __pyx_v_incy) {

  /* "scipy/linalg/_decomp_update.pyx":109
 *         blas_pointers.sswap(&n, x, &incx, y, &incy)
 *     elif blas_t is double:
 *         blas_pointers.dswap(&n, x, &incx, y, &incy)             # <<<<<<<<<<<<<<
 *     elif blas_t is float_complex:
 *         blas_pointers.cswap(&n, x, &incx, y, &incy)
 */
  __pyx_f_5scipy_6linalg_11cython_blas_dswap((&__pyx_v_n), __pyx_v_x, (&__pyx_v_incx), __pyx_v_y, (&__pyx_v_incy));

  /* "scipy/linalg/_decomp_update.pyx":105
 *         blas_pointers.zcopy(&n, x, &incx, y, &incy)
 * 
 * cdef inline void swap(int n, blas_t* x, int incx, blas_t* y, int incy) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         blas_pointers.sswap(&n, x, &incx, y, &incy)
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_swap(int __pyx_v_n, __pyx_t_float_complex *__pyx_v_x, int __pyx_v_incx, __pyx_t_float_complex *__pyx_v_y, int __pyx_v_incy) {

  /* "scipy/linalg/_decomp_update.pyx":111
 *         blas_pointers.dswap(&n, x, &incx, y, &incy)
 *     elif blas_t is float_complex:
 *         blas_pointers.cswap(&n, x, &incx, y, &incy)             # <<<<<<<<<<<<<<
 *     else:  # blas_t is double_complex:
 *         blas_pointers.zswap(&n, x, &incx, y, &incy)
 */
  __pyx_f_5scipy_6linalg_11cython_blas_cswap((&__pyx_v_n), __pyx_v_x, (&__pyx_v_incx), __pyx_v_y, (&__pyx_v_incy));

  /* "scipy/linalg/_decomp_update.pyx":105
 *         blas_pointers.zcopy(&n, x, &incx, y, &incy)
 * 
 * cdef inline void swap(int n, blas_t* x, int incx, blas_t* y, int incy) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         blas_pointers.sswap(&n, x, &incx, y, &incy)
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_swap(int __pyx_v_n, __pyx_t_double_complex *__pyx_v_x, int __pyx_v_incx, __pyx_t_double_complex *__pyx_v_y, int __pyx_v_incy) {

  /* "scipy/linalg/_decomp_update.pyx":113
 *         blas_pointers.cswap(&n, x, &incx, y, &incy)
 *     else:  # blas_t is double_complex:
 *         blas_pointers.zswap(&n, x, &incx, y, &incy)             # <<<<<<<<<<<<<<
 * 
 * cdef inline void scal(int n, blas_t a, blas_t* x, int incx) nogil:
 */
  __pyx_f_5scipy_6linalg_11cython_blas_zswap((&__pyx_v_n), __pyx_v_x, (&__pyx_v_incx), __pyx_v_y, (&__pyx_v_incy));

  /* "scipy/linalg/_decomp_update.pyx":105
 *         blas_pointers.zcopy(&n, x, &incx, y, &incy)
 * 
 * cdef inline void swap(int n, blas_t* x, int incx, blas_t* y, int incy) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         blas_pointers.sswap(&n, x, &incx, y, &incy)
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":115
 *         blas_pointers.zswap(&n, x, &incx, y, &incy)
 * 
 * cdef inline void scal(int n, blas_t a, blas_t* x, int incx) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         blas_pointers.sscal(&n, &a, x, &incx)
 */

static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_scal(int __pyx_v_n, float __pyx_v_a, float *__pyx_v_x, int __pyx_v_incx) {

  /* "scipy/linalg/_decomp_update.pyx":117
 * cdef inline void scal(int n, blas_t a, blas_t* x, int incx) nogil:
 *     if blas_t is float:
 *         blas_pointers.sscal(&n, &a, x, &incx)             # <<<<<<<<<<<<<<
 *     elif blas_t is double:
 *         blas_pointers.dscal(&n, &a, x, &incx)
 */
  __pyx_f_5scipy_6linalg_11cython_blas_sscal((&__pyx_v_n), (&__pyx_v_a), __pyx_v_x, (&__pyx_v_incx));

  /* "scipy/linalg/_decomp_update.pyx":115
 *         blas_pointers.zswap(&n, x, &incx, y, &incy)
 * 
 * cdef inline void scal(int n, blas_t a, blas_t* x, int incx) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         blas_pointers.sscal(&n, &a, x, &incx)
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_scal(int __pyx_v_n, double __pyx_v_a, double *__pyx_v_x, int __pyx_v_incx) {

  /* "scipy/linalg/_decomp_update.pyx":119
 *         blas_pointers.sscal(&n, &a, x, &incx)
 *     elif blas_t is double:
 *         blas_pointers.dscal(&n, &a, x, &incx)             # <<<<<<<<<<<<<<
 *     elif blas_t is float_complex:
 *         blas_pointers.cscal(&n, &a, x, &incx)
 */
  __pyx_f_5scipy_6linalg_11cython_blas_dscal((&__pyx_v_n), (&__pyx_v_a), __pyx_v_x, (&__pyx_v_incx));

  /* "scipy/linalg/_decomp_update.pyx":115
 *         blas_pointers.zswap(&n, x, &incx, y, &incy)
 * 
 * cdef inline void scal(int n, blas_t a, blas_t* x, int incx) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         blas_pointers.sscal(&n, &a, x, &incx)
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_scal(int __pyx_v_n, __pyx_t_float_complex __pyx_v_a, __pyx_t_float_complex *__pyx_v_x, int __pyx_v_incx) {

  /* "scipy/linalg/_decomp_update.pyx":121
 *         blas_pointers.dscal(&n, &a, x, &incx)
 *     elif blas_t is float_complex:
 *         blas_pointers.cscal(&n, &a, x, &incx)             # <<<<<<<<<<<<<<
 *     else:  # blas_t is double_complex:
 *         blas_pointers.zscal(&n, &a, x, &incx)
 */
  __pyx_f_5scipy_6linalg_11cython_blas_cscal((&__pyx_v_n), (&__pyx_v_a), __pyx_v_x, (&__pyx_v_incx));

  /* "scipy/linalg/_decomp_update.pyx":115
 *         blas_pointers.zswap(&n, x, &incx, y, &incy)
 * 
 * cdef inline void scal(int n, blas_t a, blas_t* x, int incx) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         blas_pointers.sscal(&n, &a, x, &incx)
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_scal(int __pyx_v_n, __pyx_t_double_complex __pyx_v_a, __pyx_t_double_complex *__pyx_v_x, int __pyx_v_incx) {

  /* "scipy/linalg/_decomp_update.pyx":123
 *         blas_pointers.cscal(&n, &a, x, &incx)
 *     else:  # blas_t is double_complex:
 *         blas_pointers.zscal(&n, &a, x, &incx)             # <<<<<<<<<<<<<<
 * 
 * cdef inline void axpy(int n, blas_t a, blas_t* x, int incx,
 */
  __pyx_f_5scipy_6linalg_11cython_blas_zscal((&__pyx_v_n), (&__pyx_v_a), __pyx_v_x, (&__pyx_v_incx));

  /* "scipy/linalg/_decomp_update.pyx":115
 *         blas_pointers.zswap(&n, x, &incx, y, &incy)
 * 
 * cdef inline void scal(int n, blas_t a, blas_t* x, int incx) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         blas_pointers.sscal(&n, &a, x, &incx)
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":125
 *         blas_pointers.zscal(&n, &a, x, &incx)
 * 
 * cdef inline void axpy(int n, blas_t a, blas_t* x, int incx,             # <<<<<<<<<<<<<<
 *                       blas_t* y, int incy) nogil:
 *     if blas_t is float:
 */

static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_axpy(int __pyx_v_n, float __pyx_v_a, float *__pyx_v_x, int __pyx_v_incx, float *__pyx_v_y, int __pyx_v_incy) {

  /* "scipy/linalg/_decomp_update.pyx":128
 *                       blas_t* y, int incy) nogil:
 *     if blas_t is float:
 *         blas_pointers.saxpy(&n, &a, x, &incx, y, &incy)             # <<<<<<<<<<<<<<
 *     elif blas_t is double:
 *         blas_pointers.daxpy(&n, &a, x, &incx, y, &incy)
 */
  __pyx_f_5scipy_6linalg_11cython_blas_saxpy((&__pyx_v_n), (&__pyx_v_a), __pyx_v_x, (&__pyx_v_incx), __pyx_v_y, (&__pyx_v_incy));

  /* "scipy/linalg/_decomp_update.pyx":125
 *         blas_pointers.zscal(&n, &a, x, &incx)
 * 
 * cdef inline void axpy(int n, blas_t a, blas_t* x, int incx,             # <<<<<<<<<<<<<<
 *                       blas_t* y, int incy) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_axpy(int __pyx_v_n, double __pyx_v_a, double *__pyx_v_x, int __pyx_v_incx, double *__pyx_v_y, int __pyx_v_incy) {

  /* "scipy/linalg/_decomp_update.pyx":130
 *         blas_pointers.saxpy(&n, &a, x, &incx, y, &incy)
 *     elif blas_t is double:
 *         blas_pointers.daxpy(&n, &a, x, &incx, y, &incy)             # <<<<<<<<<<<<<<
 *     elif blas_t is float_complex:
 *         blas_pointers.caxpy(&n, &a, x, &incx, y, &incy)
 */
  __pyx_f_5scipy_6linalg_11cython_blas_daxpy((&__pyx_v_n), (&__pyx_v_a), __pyx_v_x, (&__pyx_v_incx), __pyx_v_y, (&__pyx_v_incy));

  /* "scipy/linalg/_decomp_update.pyx":125
 *         blas_pointers.zscal(&n, &a, x, &incx)
 * 
 * cdef inline void axpy(int n, blas_t a, blas_t* x, int incx,             # <<<<<<<<<<<<<<
 *                       blas_t* y, int incy) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_axpy(int __pyx_v_n, __pyx_t_float_complex __pyx_v_a, __pyx_t_float_complex *__pyx_v_x, int __pyx_v_incx, __pyx_t_float_complex *__pyx_v_y, int __pyx_v_incy) {

  /* "scipy/linalg/_decomp_update.pyx":132
 *         blas_pointers.daxpy(&n, &a, x, &incx, y, &incy)
 *     elif blas_t is float_complex:
 *         blas_pointers.caxpy(&n, &a, x, &incx, y, &incy)             # <<<<<<<<<<<<<<
 *     else:  # blas_t is double_complex:
 *         blas_pointers.zaxpy(&n, &a, x, &incx, y, &incy)
 */
  __pyx_f_5scipy_6linalg_11cython_blas_caxpy((&__pyx_v_n), (&__pyx_v_a), __pyx_v_x, (&__pyx_v_incx), __pyx_v_y, (&__pyx_v_incy));

  /* "scipy/linalg/_decomp_update.pyx":125
 *         blas_pointers.zscal(&n, &a, x, &incx)
 * 
 * cdef inline void axpy(int n, blas_t a, blas_t* x, int incx,             # <<<<<<<<<<<<<<
 *                       blas_t* y, int incy) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_axpy(int __pyx_v_n, __pyx_t_double_complex __pyx_v_a, __pyx_t_double_complex *__pyx_v_x, int __pyx_v_incx, __pyx_t_double_complex *__pyx_v_y, int __pyx_v_incy) {

  /* "scipy/linalg/_decomp_update.pyx":134
 *         blas_pointers.caxpy(&n, &a, x, &incx, y, &incy)
 *     else:  # blas_t is double_complex:
 *         blas_pointers.zaxpy(&n, &a, x, &incx, y, &incy)             # <<<<<<<<<<<<<<
 * 
 * cdef inline blas_t nrm2(int n, blas_t* x, int incx) nogil:
 */
  __pyx_f_5scipy_6linalg_11cython_blas_zaxpy((&__pyx_v_n), (&__pyx_v_a), __pyx_v_x, (&__pyx_v_incx), __pyx_v_y, (&__pyx_v_incy));

  /* "scipy/linalg/_decomp_update.pyx":125
 *         blas_pointers.zscal(&n, &a, x, &incx)
 * 
 * cdef inline void axpy(int n, blas_t a, blas_t* x, int incx,             # <<<<<<<<<<<<<<
 *                       blas_t* y, int incy) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":136
 *         blas_pointers.zaxpy(&n, &a, x, &incx, y, &incy)
 * 
 * cdef inline blas_t nrm2(int n, blas_t* x, int incx) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         return blas_pointers.snrm2(&n, x, &incx)
 */

static CYTHON_INLINE float __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(int __pyx_v_n, float *__pyx_v_x, int __pyx_v_incx) {
  float __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":138
 * cdef inline blas_t nrm2(int n, blas_t* x, int incx) nogil:
 *     if blas_t is float:
 *         return blas_pointers.snrm2(&n, x, &incx)             # <<<<<<<<<<<<<<
 *     elif blas_t is double:
 *         return blas_pointers.dnrm2(&n, x, &incx)
 */
  __pyx_r = __pyx_f_5scipy_6linalg_11cython_blas_snrm2((&__pyx_v_n), __pyx_v_x, (&__pyx_v_incx));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":136
 *         blas_pointers.zaxpy(&n, &a, x, &incx, y, &incy)
 * 
 * cdef inline blas_t nrm2(int n, blas_t* x, int incx) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         return blas_pointers.snrm2(&n, x, &incx)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE double __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(int __pyx_v_n, double *__pyx_v_x, int __pyx_v_incx) {
  double __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":140
 *         return blas_pointers.snrm2(&n, x, &incx)
 *     elif blas_t is double:
 *         return blas_pointers.dnrm2(&n, x, &incx)             # <<<<<<<<<<<<<<
 *     elif blas_t is float_complex:
 *         return blas_pointers.scnrm2(&n, x, &incx)
 */
  __pyx_r = __pyx_f_5scipy_6linalg_11cython_blas_dnrm2((&__pyx_v_n), __pyx_v_x, (&__pyx_v_incx));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":136
 *         blas_pointers.zaxpy(&n, &a, x, &incx, y, &incy)
 * 
 * cdef inline blas_t nrm2(int n, blas_t* x, int incx) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         return blas_pointers.snrm2(&n, x, &incx)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_float_complex __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(int __pyx_v_n, __pyx_t_float_complex *__pyx_v_x, int __pyx_v_incx) {
  __pyx_t_float_complex __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":142
 *         return blas_pointers.dnrm2(&n, x, &incx)
 *     elif blas_t is float_complex:
 *         return blas_pointers.scnrm2(&n, x, &incx)             # <<<<<<<<<<<<<<
 *     else:  # blas_t is double_complex:
 *         return blas_pointers.dznrm2(&n, x, &incx)
 */
  __pyx_r = __pyx_t_float_complex_from_parts(__pyx_f_5scipy_6linalg_11cython_blas_scnrm2((&__pyx_v_n), __pyx_v_x, (&__pyx_v_incx)), 0);
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":136
 *         blas_pointers.zaxpy(&n, &a, x, &incx, y, &incy)
 * 
 * cdef inline blas_t nrm2(int n, blas_t* x, int incx) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         return blas_pointers.snrm2(&n, x, &incx)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(int __pyx_v_n, __pyx_t_double_complex *__pyx_v_x, int __pyx_v_incx) {
  __pyx_t_double_complex __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":144
 *         return blas_pointers.scnrm2(&n, x, &incx)
 *     else:  # blas_t is double_complex:
 *         return blas_pointers.dznrm2(&n, x, &incx)             # <<<<<<<<<<<<<<
 * 
 * cdef inline void lartg(blas_t* a, blas_t* b, blas_t* c, blas_t* s) nogil:
 */
  __pyx_r = __pyx_t_double_complex_from_parts(__pyx_f_5scipy_6linalg_11cython_blas_dznrm2((&__pyx_v_n), __pyx_v_x, (&__pyx_v_incx)), 0);
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":136
 *         blas_pointers.zaxpy(&n, &a, x, &incx, y, &incy)
 * 
 * cdef inline blas_t nrm2(int n, blas_t* x, int incx) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         return blas_pointers.snrm2(&n, x, &incx)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":146
 *         return blas_pointers.dznrm2(&n, x, &incx)
 * 
 * cdef inline void lartg(blas_t* a, blas_t* b, blas_t* c, blas_t* s) nogil:             # <<<<<<<<<<<<<<
 *     cdef blas_t g
 *     if blas_t is float:
 */

static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_lartg(float *__pyx_v_a, float *__pyx_v_b, float *__pyx_v_c, float *__pyx_v_s) {
  float __pyx_v_g;

  /* "scipy/linalg/_decomp_update.pyx":149
 *     cdef blas_t g
 *     if blas_t is float:
 *         lapack_pointers.slartg(a, b, c, s, &g)             # <<<<<<<<<<<<<<
 *     elif blas_t is double:
 *         lapack_pointers.dlartg(a, b, c, s, &g)
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_slartg(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_s, (&__pyx_v_g));

  /* "scipy/linalg/_decomp_update.pyx":159
 *         lapack_pointers.zlartg(a, b, <double*>c, s, &g)
 *     # make this function more like the BLAS drotg
 *     a[0] = g             # <<<<<<<<<<<<<<
 *     b[0] = 0
 * 
 */
  (__pyx_v_a[0]) = __pyx_v_g;

  /* "scipy/linalg/_decomp_update.pyx":160
 *     # make this function more like the BLAS drotg
 *     a[0] = g
 *     b[0] = 0             # <<<<<<<<<<<<<<
 * 
 * cdef inline void rot(int n, blas_t* x, int incx, blas_t* y, int incy,
 */
  (__pyx_v_b[0]) = 0.0;

  /* "scipy/linalg/_decomp_update.pyx":146
 *         return blas_pointers.dznrm2(&n, x, &incx)
 * 
 * cdef inline void lartg(blas_t* a, blas_t* b, blas_t* c, blas_t* s) nogil:             # <<<<<<<<<<<<<<
 *     cdef blas_t g
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_lartg(double *__pyx_v_a, double *__pyx_v_b, double *__pyx_v_c, double *__pyx_v_s) {
  double __pyx_v_g;

  /* "scipy/linalg/_decomp_update.pyx":151
 *         lapack_pointers.slartg(a, b, c, s, &g)
 *     elif blas_t is double:
 *         lapack_pointers.dlartg(a, b, c, s, &g)             # <<<<<<<<<<<<<<
 *     elif blas_t is float_complex:
 *         c[0] = 0. # init imag
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_dlartg(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_s, (&__pyx_v_g));

  /* "scipy/linalg/_decomp_update.pyx":159
 *         lapack_pointers.zlartg(a, b, <double*>c, s, &g)
 *     # make this function more like the BLAS drotg
 *     a[0] = g             # <<<<<<<<<<<<<<
 *     b[0] = 0
 * 
 */
  (__pyx_v_a[0]) = __pyx_v_g;

  /* "scipy/linalg/_decomp_update.pyx":160
 *     # make this function more like the BLAS drotg
 *     a[0] = g
 *     b[0] = 0             # <<<<<<<<<<<<<<
 * 
 * cdef inline void rot(int n, blas_t* x, int incx, blas_t* y, int incy,
 */
  (__pyx_v_b[0]) = 0.0;

  /* "scipy/linalg/_decomp_update.pyx":146
 *         return blas_pointers.dznrm2(&n, x, &incx)
 * 
 * cdef inline void lartg(blas_t* a, blas_t* b, blas_t* c, blas_t* s) nogil:             # <<<<<<<<<<<<<<
 *     cdef blas_t g
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_t_float_complex *__pyx_v_a, __pyx_t_float_complex *__pyx_v_b, __pyx_t_float_complex *__pyx_v_c, __pyx_t_float_complex *__pyx_v_s) {
  __pyx_t_float_complex __pyx_v_g;

  /* "scipy/linalg/_decomp_update.pyx":153
 *         lapack_pointers.dlartg(a, b, c, s, &g)
 *     elif blas_t is float_complex:
 *         c[0] = 0. # init imag             # <<<<<<<<<<<<<<
 *         lapack_pointers.clartg(a, b, <float*>c, s, &g)
 *     else:
 */
  (__pyx_v_c[0]) = __pyx_t_float_complex_from_parts(0., 0);

  /* "scipy/linalg/_decomp_update.pyx":154
 *     elif blas_t is float_complex:
 *         c[0] = 0. # init imag
 *         lapack_pointers.clartg(a, b, <float*>c, s, &g)             # <<<<<<<<<<<<<<
 *     else:
 *         c[0] = 0. # init imag
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_clartg(__pyx_v_a, __pyx_v_b, ((float *)__pyx_v_c), __pyx_v_s, (&__pyx_v_g));

  /* "scipy/linalg/_decomp_update.pyx":159
 *         lapack_pointers.zlartg(a, b, <double*>c, s, &g)
 *     # make this function more like the BLAS drotg
 *     a[0] = g             # <<<<<<<<<<<<<<
 *     b[0] = 0
 * 
 */
  (__pyx_v_a[0]) = __pyx_v_g;

  /* "scipy/linalg/_decomp_update.pyx":160
 *     # make this function more like the BLAS drotg
 *     a[0] = g
 *     b[0] = 0             # <<<<<<<<<<<<<<
 * 
 * cdef inline void rot(int n, blas_t* x, int incx, blas_t* y, int incy,
 */
  (__pyx_v_b[0]) = __pyx_t_float_complex_from_parts(0, 0);

  /* "scipy/linalg/_decomp_update.pyx":146
 *         return blas_pointers.dznrm2(&n, x, &incx)
 * 
 * cdef inline void lartg(blas_t* a, blas_t* b, blas_t* c, blas_t* s) nogil:             # <<<<<<<<<<<<<<
 *     cdef blas_t g
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_t_double_complex *__pyx_v_a, __pyx_t_double_complex *__pyx_v_b, __pyx_t_double_complex *__pyx_v_c, __pyx_t_double_complex *__pyx_v_s) {
  __pyx_t_double_complex __pyx_v_g;

  /* "scipy/linalg/_decomp_update.pyx":156
 *         lapack_pointers.clartg(a, b, <float*>c, s, &g)
 *     else:
 *         c[0] = 0. # init imag             # <<<<<<<<<<<<<<
 *         lapack_pointers.zlartg(a, b, <double*>c, s, &g)
 *     # make this function more like the BLAS drotg
 */
  (__pyx_v_c[0]) = __pyx_t_double_complex_from_parts(0., 0);

  /* "scipy/linalg/_decomp_update.pyx":157
 *     else:
 *         c[0] = 0. # init imag
 *         lapack_pointers.zlartg(a, b, <double*>c, s, &g)             # <<<<<<<<<<<<<<
 *     # make this function more like the BLAS drotg
 *     a[0] = g
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_zlartg(__pyx_v_a, __pyx_v_b, ((double *)__pyx_v_c), __pyx_v_s, (&__pyx_v_g));

  /* "scipy/linalg/_decomp_update.pyx":159
 *         lapack_pointers.zlartg(a, b, <double*>c, s, &g)
 *     # make this function more like the BLAS drotg
 *     a[0] = g             # <<<<<<<<<<<<<<
 *     b[0] = 0
 * 
 */
  (__pyx_v_a[0]) = __pyx_v_g;

  /* "scipy/linalg/_decomp_update.pyx":160
 *     # make this function more like the BLAS drotg
 *     a[0] = g
 *     b[0] = 0             # <<<<<<<<<<<<<<
 * 
 * cdef inline void rot(int n, blas_t* x, int incx, blas_t* y, int incy,
 */
  (__pyx_v_b[0]) = __pyx_t_double_complex_from_parts(0, 0);

  /* "scipy/linalg/_decomp_update.pyx":146
 *         return blas_pointers.dznrm2(&n, x, &incx)
 * 
 * cdef inline void lartg(blas_t* a, blas_t* b, blas_t* c, blas_t* s) nogil:             # <<<<<<<<<<<<<<
 *     cdef blas_t g
 *     if blas_t is float:
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":162
 *     b[0] = 0
 * 
 * cdef inline void rot(int n, blas_t* x, int incx, blas_t* y, int incy,             # <<<<<<<<<<<<<<
 *                      blas_t c, blas_t s) nogil:
 *     if blas_t is float:
 */

static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(int __pyx_v_n, float *__pyx_v_x, int __pyx_v_incx, float *__pyx_v_y, int __pyx_v_incy, float __pyx_v_c, float __pyx_v_s) {

  /* "scipy/linalg/_decomp_update.pyx":165
 *                      blas_t c, blas_t s) nogil:
 *     if blas_t is float:
 *         blas_pointers.srot(&n, x, &incx, y, &incy, &c, &s)             # <<<<<<<<<<<<<<
 *     elif blas_t is double:
 *         blas_pointers.drot(&n, x, &incx, y, &incy, &c, &s)
 */
  __pyx_f_5scipy_6linalg_11cython_blas_srot((&__pyx_v_n), __pyx_v_x, (&__pyx_v_incx), __pyx_v_y, (&__pyx_v_incy), (&__pyx_v_c), (&__pyx_v_s));

  /* "scipy/linalg/_decomp_update.pyx":162
 *     b[0] = 0
 * 
 * cdef inline void rot(int n, blas_t* x, int incx, blas_t* y, int incy,             # <<<<<<<<<<<<<<
 *                      blas_t c, blas_t s) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(int __pyx_v_n, double *__pyx_v_x, int __pyx_v_incx, double *__pyx_v_y, int __pyx_v_incy, double __pyx_v_c, double __pyx_v_s) {

  /* "scipy/linalg/_decomp_update.pyx":167
 *         blas_pointers.srot(&n, x, &incx, y, &incy, &c, &s)
 *     elif blas_t is double:
 *         blas_pointers.drot(&n, x, &incx, y, &incy, &c, &s)             # <<<<<<<<<<<<<<
 *     elif blas_t is float_complex:
 *         lapack_pointers.crot(&n, x, &incx, y, &incy, <float*>&c, &s)
 */
  __pyx_f_5scipy_6linalg_11cython_blas_drot((&__pyx_v_n), __pyx_v_x, (&__pyx_v_incx), __pyx_v_y, (&__pyx_v_incy), (&__pyx_v_c), (&__pyx_v_s));

  /* "scipy/linalg/_decomp_update.pyx":162
 *     b[0] = 0
 * 
 * cdef inline void rot(int n, blas_t* x, int incx, blas_t* y, int incy,             # <<<<<<<<<<<<<<
 *                      blas_t c, blas_t s) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(int __pyx_v_n, __pyx_t_float_complex *__pyx_v_x, int __pyx_v_incx, __pyx_t_float_complex *__pyx_v_y, int __pyx_v_incy, __pyx_t_float_complex __pyx_v_c, __pyx_t_float_complex __pyx_v_s) {

  /* "scipy/linalg/_decomp_update.pyx":169
 *         blas_pointers.drot(&n, x, &incx, y, &incy, &c, &s)
 *     elif blas_t is float_complex:
 *         lapack_pointers.crot(&n, x, &incx, y, &incy, <float*>&c, &s)             # <<<<<<<<<<<<<<
 *     else:
 *         lapack_pointers.zrot(&n, x, &incx, y, &incy, <double*>&c, &s)
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_crot((&__pyx_v_n), __pyx_v_x, (&__pyx_v_incx), __pyx_v_y, (&__pyx_v_incy), ((float *)(&__pyx_v_c)), (&__pyx_v_s));

  /* "scipy/linalg/_decomp_update.pyx":162
 *     b[0] = 0
 * 
 * cdef inline void rot(int n, blas_t* x, int incx, blas_t* y, int incy,             # <<<<<<<<<<<<<<
 *                      blas_t c, blas_t s) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(int __pyx_v_n, __pyx_t_double_complex *__pyx_v_x, int __pyx_v_incx, __pyx_t_double_complex *__pyx_v_y, int __pyx_v_incy, __pyx_t_double_complex __pyx_v_c, __pyx_t_double_complex __pyx_v_s) {

  /* "scipy/linalg/_decomp_update.pyx":171
 *         lapack_pointers.crot(&n, x, &incx, y, &incy, <float*>&c, &s)
 *     else:
 *         lapack_pointers.zrot(&n, x, &incx, y, &incy, <double*>&c, &s)             # <<<<<<<<<<<<<<
 * 
 * cdef inline void larfg(int n, blas_t* alpha, blas_t* x, int incx,
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_zrot((&__pyx_v_n), __pyx_v_x, (&__pyx_v_incx), __pyx_v_y, (&__pyx_v_incy), ((double *)(&__pyx_v_c)), (&__pyx_v_s));

  /* "scipy/linalg/_decomp_update.pyx":162
 *     b[0] = 0
 * 
 * cdef inline void rot(int n, blas_t* x, int incx, blas_t* y, int incy,             # <<<<<<<<<<<<<<
 *                      blas_t c, blas_t s) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":173
 *         lapack_pointers.zrot(&n, x, &incx, y, &incy, <double*>&c, &s)
 * 
 * cdef inline void larfg(int n, blas_t* alpha, blas_t* x, int incx,             # <<<<<<<<<<<<<<
 *                        blas_t* tau) nogil:
 *     if blas_t is float:
 */

static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_larfg(int __pyx_v_n, float *__pyx_v_alpha, float *__pyx_v_x, int __pyx_v_incx, float *__pyx_v_tau) {

  /* "scipy/linalg/_decomp_update.pyx":176
 *                        blas_t* tau) nogil:
 *     if blas_t is float:
 *         lapack_pointers.slarfg(&n, alpha, x, &incx, tau)             # <<<<<<<<<<<<<<
 *     elif blas_t is double:
 *         lapack_pointers.dlarfg(&n, alpha, x, &incx, tau)
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_slarfg((&__pyx_v_n), __pyx_v_alpha, __pyx_v_x, (&__pyx_v_incx), __pyx_v_tau);

  /* "scipy/linalg/_decomp_update.pyx":173
 *         lapack_pointers.zrot(&n, x, &incx, y, &incy, <double*>&c, &s)
 * 
 * cdef inline void larfg(int n, blas_t* alpha, blas_t* x, int incx,             # <<<<<<<<<<<<<<
 *                        blas_t* tau) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_larfg(int __pyx_v_n, double *__pyx_v_alpha, double *__pyx_v_x, int __pyx_v_incx, double *__pyx_v_tau) {

  /* "scipy/linalg/_decomp_update.pyx":178
 *         lapack_pointers.slarfg(&n, alpha, x, &incx, tau)
 *     elif blas_t is double:
 *         lapack_pointers.dlarfg(&n, alpha, x, &incx, tau)             # <<<<<<<<<<<<<<
 *     elif blas_t is float_complex:
 *         lapack_pointers.clarfg(&n, alpha, x, &incx, tau)
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_dlarfg((&__pyx_v_n), __pyx_v_alpha, __pyx_v_x, (&__pyx_v_incx), __pyx_v_tau);

  /* "scipy/linalg/_decomp_update.pyx":173
 *         lapack_pointers.zrot(&n, x, &incx, y, &incy, <double*>&c, &s)
 * 
 * cdef inline void larfg(int n, blas_t* alpha, blas_t* x, int incx,             # <<<<<<<<<<<<<<
 *                        blas_t* tau) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_larfg(int __pyx_v_n, __pyx_t_float_complex *__pyx_v_alpha, __pyx_t_float_complex *__pyx_v_x, int __pyx_v_incx, __pyx_t_float_complex *__pyx_v_tau) {

  /* "scipy/linalg/_decomp_update.pyx":180
 *         lapack_pointers.dlarfg(&n, alpha, x, &incx, tau)
 *     elif blas_t is float_complex:
 *         lapack_pointers.clarfg(&n, alpha, x, &incx, tau)             # <<<<<<<<<<<<<<
 *     else:  # blas_t is double_complex:
 *         lapack_pointers.zlarfg(&n, alpha, x, &incx, tau)
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_clarfg((&__pyx_v_n), __pyx_v_alpha, __pyx_v_x, (&__pyx_v_incx), __pyx_v_tau);

  /* "scipy/linalg/_decomp_update.pyx":173
 *         lapack_pointers.zrot(&n, x, &incx, y, &incy, <double*>&c, &s)
 * 
 * cdef inline void larfg(int n, blas_t* alpha, blas_t* x, int incx,             # <<<<<<<<<<<<<<
 *                        blas_t* tau) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_larfg(int __pyx_v_n, __pyx_t_double_complex *__pyx_v_alpha, __pyx_t_double_complex *__pyx_v_x, int __pyx_v_incx, __pyx_t_double_complex *__pyx_v_tau) {

  /* "scipy/linalg/_decomp_update.pyx":182
 *         lapack_pointers.clarfg(&n, alpha, x, &incx, tau)
 *     else:  # blas_t is double_complex:
 *         lapack_pointers.zlarfg(&n, alpha, x, &incx, tau)             # <<<<<<<<<<<<<<
 * 
 * cdef inline void larf(char* side, int m, int n, blas_t* v, int incv, blas_t tau,
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_zlarfg((&__pyx_v_n), __pyx_v_alpha, __pyx_v_x, (&__pyx_v_incx), __pyx_v_tau);

  /* "scipy/linalg/_decomp_update.pyx":173
 *         lapack_pointers.zrot(&n, x, &incx, y, &incy, <double*>&c, &s)
 * 
 * cdef inline void larfg(int n, blas_t* alpha, blas_t* x, int incx,             # <<<<<<<<<<<<<<
 *                        blas_t* tau) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":184
 *         lapack_pointers.zlarfg(&n, alpha, x, &incx, tau)
 * 
 * cdef inline void larf(char* side, int m, int n, blas_t* v, int incv, blas_t tau,             # <<<<<<<<<<<<<<
 *     blas_t* c, int ldc, blas_t* work) nogil:
 *     if blas_t is float:
 */

static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_larf(char *__pyx_v_side, int __pyx_v_m, int __pyx_v_n, float *__pyx_v_v, int __pyx_v_incv, float __pyx_v_tau, float *__pyx_v_c, int __pyx_v_ldc, float *__pyx_v_work) {

  /* "scipy/linalg/_decomp_update.pyx":187
 *     blas_t* c, int ldc, blas_t* work) nogil:
 *     if blas_t is float:
 *         lapack_pointers.slarf(side, &m, &n, v, &incv, &tau, c, &ldc, work)             # <<<<<<<<<<<<<<
 *     elif blas_t is double:
 *         lapack_pointers.dlarf(side, &m, &n, v, &incv, &tau, c, &ldc, work)
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_slarf(__pyx_v_side, (&__pyx_v_m), (&__pyx_v_n), __pyx_v_v, (&__pyx_v_incv), (&__pyx_v_tau), __pyx_v_c, (&__pyx_v_ldc), __pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":184
 *         lapack_pointers.zlarfg(&n, alpha, x, &incx, tau)
 * 
 * cdef inline void larf(char* side, int m, int n, blas_t* v, int incv, blas_t tau,             # <<<<<<<<<<<<<<
 *     blas_t* c, int ldc, blas_t* work) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_larf(char *__pyx_v_side, int __pyx_v_m, int __pyx_v_n, double *__pyx_v_v, int __pyx_v_incv, double __pyx_v_tau, double *__pyx_v_c, int __pyx_v_ldc, double *__pyx_v_work) {

  /* "scipy/linalg/_decomp_update.pyx":189
 *         lapack_pointers.slarf(side, &m, &n, v, &incv, &tau, c, &ldc, work)
 *     elif blas_t is double:
 *         lapack_pointers.dlarf(side, &m, &n, v, &incv, &tau, c, &ldc, work)             # <<<<<<<<<<<<<<
 *     elif blas_t is float_complex:
 *         lapack_pointers.clarf(side, &m, &n, v, &incv, &tau, c, &ldc, work)
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_dlarf(__pyx_v_side, (&__pyx_v_m), (&__pyx_v_n), __pyx_v_v, (&__pyx_v_incv), (&__pyx_v_tau), __pyx_v_c, (&__pyx_v_ldc), __pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":184
 *         lapack_pointers.zlarfg(&n, alpha, x, &incx, tau)
 * 
 * cdef inline void larf(char* side, int m, int n, blas_t* v, int incv, blas_t tau,             # <<<<<<<<<<<<<<
 *     blas_t* c, int ldc, blas_t* work) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_larf(char *__pyx_v_side, int __pyx_v_m, int __pyx_v_n, __pyx_t_float_complex *__pyx_v_v, int __pyx_v_incv, __pyx_t_float_complex __pyx_v_tau, __pyx_t_float_complex *__pyx_v_c, int __pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work) {

  /* "scipy/linalg/_decomp_update.pyx":191
 *         lapack_pointers.dlarf(side, &m, &n, v, &incv, &tau, c, &ldc, work)
 *     elif blas_t is float_complex:
 *         lapack_pointers.clarf(side, &m, &n, v, &incv, &tau, c, &ldc, work)             # <<<<<<<<<<<<<<
 *     else:  # blas_t is double_complex:
 *         lapack_pointers.zlarf(side, &m, &n, v, &incv, &tau, c, &ldc, work)
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_clarf(__pyx_v_side, (&__pyx_v_m), (&__pyx_v_n), __pyx_v_v, (&__pyx_v_incv), (&__pyx_v_tau), __pyx_v_c, (&__pyx_v_ldc), __pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":184
 *         lapack_pointers.zlarfg(&n, alpha, x, &incx, tau)
 * 
 * cdef inline void larf(char* side, int m, int n, blas_t* v, int incv, blas_t tau,             # <<<<<<<<<<<<<<
 *     blas_t* c, int ldc, blas_t* work) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_larf(char *__pyx_v_side, int __pyx_v_m, int __pyx_v_n, __pyx_t_double_complex *__pyx_v_v, int __pyx_v_incv, __pyx_t_double_complex __pyx_v_tau, __pyx_t_double_complex *__pyx_v_c, int __pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work) {

  /* "scipy/linalg/_decomp_update.pyx":193
 *         lapack_pointers.clarf(side, &m, &n, v, &incv, &tau, c, &ldc, work)
 *     else:  # blas_t is double_complex:
 *         lapack_pointers.zlarf(side, &m, &n, v, &incv, &tau, c, &ldc, work)             # <<<<<<<<<<<<<<
 * 
 * cdef inline void ger(int m, int n, blas_t alpha, blas_t* x, int incx, blas_t* y,
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_zlarf(__pyx_v_side, (&__pyx_v_m), (&__pyx_v_n), __pyx_v_v, (&__pyx_v_incv), (&__pyx_v_tau), __pyx_v_c, (&__pyx_v_ldc), __pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":184
 *         lapack_pointers.zlarfg(&n, alpha, x, &incx, tau)
 * 
 * cdef inline void larf(char* side, int m, int n, blas_t* v, int incv, blas_t tau,             # <<<<<<<<<<<<<<
 *     blas_t* c, int ldc, blas_t* work) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":195
 *         lapack_pointers.zlarf(side, &m, &n, v, &incv, &tau, c, &ldc, work)
 * 
 * cdef inline void ger(int m, int n, blas_t alpha, blas_t* x, int incx, blas_t* y,             # <<<<<<<<<<<<<<
 *         int incy, blas_t* a, int lda) nogil:
 *     if blas_t is float:
 */

static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_ger(int __pyx_v_m, int __pyx_v_n, float __pyx_v_alpha, float *__pyx_v_x, int __pyx_v_incx, float *__pyx_v_y, int __pyx_v_incy, float *__pyx_v_a, int __pyx_v_lda) {

  /* "scipy/linalg/_decomp_update.pyx":198
 *         int incy, blas_t* a, int lda) nogil:
 *     if blas_t is float:
 *         blas_pointers.sger(&m, &n, &alpha, x, &incx, y, &incy, a, &lda)             # <<<<<<<<<<<<<<
 *     elif blas_t is double:
 *         blas_pointers.dger(&m, &n, &alpha, x, &incx, y, &incy, a, &lda)
 */
  __pyx_f_5scipy_6linalg_11cython_blas_sger((&__pyx_v_m), (&__pyx_v_n), (&__pyx_v_alpha), __pyx_v_x, (&__pyx_v_incx), __pyx_v_y, (&__pyx_v_incy), __pyx_v_a, (&__pyx_v_lda));

  /* "scipy/linalg/_decomp_update.pyx":195
 *         lapack_pointers.zlarf(side, &m, &n, v, &incv, &tau, c, &ldc, work)
 * 
 * cdef inline void ger(int m, int n, blas_t alpha, blas_t* x, int incx, blas_t* y,             # <<<<<<<<<<<<<<
 *         int incy, blas_t* a, int lda) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_ger(int __pyx_v_m, int __pyx_v_n, double __pyx_v_alpha, double *__pyx_v_x, int __pyx_v_incx, double *__pyx_v_y, int __pyx_v_incy, double *__pyx_v_a, int __pyx_v_lda) {

  /* "scipy/linalg/_decomp_update.pyx":200
 *         blas_pointers.sger(&m, &n, &alpha, x, &incx, y, &incy, a, &lda)
 *     elif blas_t is double:
 *         blas_pointers.dger(&m, &n, &alpha, x, &incx, y, &incy, a, &lda)             # <<<<<<<<<<<<<<
 *     elif blas_t is float_complex:
 *         blas_pointers.cgeru(&m, &n, &alpha, x, &incx, y, &incy, a, &lda)
 */
  __pyx_f_5scipy_6linalg_11cython_blas_dger((&__pyx_v_m), (&__pyx_v_n), (&__pyx_v_alpha), __pyx_v_x, (&__pyx_v_incx), __pyx_v_y, (&__pyx_v_incy), __pyx_v_a, (&__pyx_v_lda));

  /* "scipy/linalg/_decomp_update.pyx":195
 *         lapack_pointers.zlarf(side, &m, &n, v, &incv, &tau, c, &ldc, work)
 * 
 * cdef inline void ger(int m, int n, blas_t alpha, blas_t* x, int incx, blas_t* y,             # <<<<<<<<<<<<<<
 *         int incy, blas_t* a, int lda) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_ger(int __pyx_v_m, int __pyx_v_n, __pyx_t_float_complex __pyx_v_alpha, __pyx_t_float_complex *__pyx_v_x, int __pyx_v_incx, __pyx_t_float_complex *__pyx_v_y, int __pyx_v_incy, __pyx_t_float_complex *__pyx_v_a, int __pyx_v_lda) {

  /* "scipy/linalg/_decomp_update.pyx":202
 *         blas_pointers.dger(&m, &n, &alpha, x, &incx, y, &incy, a, &lda)
 *     elif blas_t is float_complex:
 *         blas_pointers.cgeru(&m, &n, &alpha, x, &incx, y, &incy, a, &lda)             # <<<<<<<<<<<<<<
 *     else:
 *         blas_pointers.zgeru(&m, &n, &alpha, x, &incx, y, &incy, a, &lda)
 */
  __pyx_f_5scipy_6linalg_11cython_blas_cgeru((&__pyx_v_m), (&__pyx_v_n), (&__pyx_v_alpha), __pyx_v_x, (&__pyx_v_incx), __pyx_v_y, (&__pyx_v_incy), __pyx_v_a, (&__pyx_v_lda));

  /* "scipy/linalg/_decomp_update.pyx":195
 *         lapack_pointers.zlarf(side, &m, &n, v, &incv, &tau, c, &ldc, work)
 * 
 * cdef inline void ger(int m, int n, blas_t alpha, blas_t* x, int incx, blas_t* y,             # <<<<<<<<<<<<<<
 *         int incy, blas_t* a, int lda) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_ger(int __pyx_v_m, int __pyx_v_n, __pyx_t_double_complex __pyx_v_alpha, __pyx_t_double_complex *__pyx_v_x, int __pyx_v_incx, __pyx_t_double_complex *__pyx_v_y, int __pyx_v_incy, __pyx_t_double_complex *__pyx_v_a, int __pyx_v_lda) {

  /* "scipy/linalg/_decomp_update.pyx":204
 *         blas_pointers.cgeru(&m, &n, &alpha, x, &incx, y, &incy, a, &lda)
 *     else:
 *         blas_pointers.zgeru(&m, &n, &alpha, x, &incx, y, &incy, a, &lda)             # <<<<<<<<<<<<<<
 * 
 * cdef inline void gemv(char* trans, int m, int n, blas_t alpha, blas_t* a,
 */
  __pyx_f_5scipy_6linalg_11cython_blas_zgeru((&__pyx_v_m), (&__pyx_v_n), (&__pyx_v_alpha), __pyx_v_x, (&__pyx_v_incx), __pyx_v_y, (&__pyx_v_incy), __pyx_v_a, (&__pyx_v_lda));

  /* "scipy/linalg/_decomp_update.pyx":195
 *         lapack_pointers.zlarf(side, &m, &n, v, &incv, &tau, c, &ldc, work)
 * 
 * cdef inline void ger(int m, int n, blas_t alpha, blas_t* x, int incx, blas_t* y,             # <<<<<<<<<<<<<<
 *         int incy, blas_t* a, int lda) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":206
 *         blas_pointers.zgeru(&m, &n, &alpha, x, &incx, y, &incy, a, &lda)
 * 
 * cdef inline void gemv(char* trans, int m, int n, blas_t alpha, blas_t* a,             # <<<<<<<<<<<<<<
 *         int lda, blas_t* x, int incx, blas_t beta, blas_t* y, int incy) nogil:
 *     if blas_t is float:
 */

static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemv(char *__pyx_v_trans, int __pyx_v_m, int __pyx_v_n, float __pyx_v_alpha, float *__pyx_v_a, int __pyx_v_lda, float *__pyx_v_x, int __pyx_v_incx, float __pyx_v_beta, float *__pyx_v_y, int __pyx_v_incy) {

  /* "scipy/linalg/_decomp_update.pyx":209
 *         int lda, blas_t* x, int incx, blas_t beta, blas_t* y, int incy) nogil:
 *     if blas_t is float:
 *         blas_pointers.sgemv(trans, &m, &n, &alpha, a, &lda, x, &incx,             # <<<<<<<<<<<<<<
 *                 &beta, y, &incy)
 *     elif blas_t is double:
 */
  __pyx_f_5scipy_6linalg_11cython_blas_sgemv(__pyx_v_trans, (&__pyx_v_m), (&__pyx_v_n), (&__pyx_v_alpha), __pyx_v_a, (&__pyx_v_lda), __pyx_v_x, (&__pyx_v_incx), (&__pyx_v_beta), __pyx_v_y, (&__pyx_v_incy));

  /* "scipy/linalg/_decomp_update.pyx":206
 *         blas_pointers.zgeru(&m, &n, &alpha, x, &incx, y, &incy, a, &lda)
 * 
 * cdef inline void gemv(char* trans, int m, int n, blas_t alpha, blas_t* a,             # <<<<<<<<<<<<<<
 *         int lda, blas_t* x, int incx, blas_t beta, blas_t* y, int incy) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemv(char *__pyx_v_trans, int __pyx_v_m, int __pyx_v_n, double __pyx_v_alpha, double *__pyx_v_a, int __pyx_v_lda, double *__pyx_v_x, int __pyx_v_incx, double __pyx_v_beta, double *__pyx_v_y, int __pyx_v_incy) {

  /* "scipy/linalg/_decomp_update.pyx":212
 *                 &beta, y, &incy)
 *     elif blas_t is double:
 *         blas_pointers.dgemv(trans, &m, &n, &alpha, a, &lda, x, &incx,             # <<<<<<<<<<<<<<
 *                 &beta, y, &incy)
 *     elif blas_t is float_complex:
 */
  __pyx_f_5scipy_6linalg_11cython_blas_dgemv(__pyx_v_trans, (&__pyx_v_m), (&__pyx_v_n), (&__pyx_v_alpha), __pyx_v_a, (&__pyx_v_lda), __pyx_v_x, (&__pyx_v_incx), (&__pyx_v_beta), __pyx_v_y, (&__pyx_v_incy));

  /* "scipy/linalg/_decomp_update.pyx":206
 *         blas_pointers.zgeru(&m, &n, &alpha, x, &incx, y, &incy, a, &lda)
 * 
 * cdef inline void gemv(char* trans, int m, int n, blas_t alpha, blas_t* a,             # <<<<<<<<<<<<<<
 *         int lda, blas_t* x, int incx, blas_t beta, blas_t* y, int incy) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemv(char *__pyx_v_trans, int __pyx_v_m, int __pyx_v_n, __pyx_t_float_complex __pyx_v_alpha, __pyx_t_float_complex *__pyx_v_a, int __pyx_v_lda, __pyx_t_float_complex *__pyx_v_x, int __pyx_v_incx, __pyx_t_float_complex __pyx_v_beta, __pyx_t_float_complex *__pyx_v_y, int __pyx_v_incy) {

  /* "scipy/linalg/_decomp_update.pyx":215
 *                 &beta, y, &incy)
 *     elif blas_t is float_complex:
 *         blas_pointers.cgemv(trans, &m, &n, &alpha, a, &lda, x, &incx,             # <<<<<<<<<<<<<<
 *                 &beta, y, &incy)
 *     else:  # blas_t is double_complex:
 */
  __pyx_f_5scipy_6linalg_11cython_blas_cgemv(__pyx_v_trans, (&__pyx_v_m), (&__pyx_v_n), (&__pyx_v_alpha), __pyx_v_a, (&__pyx_v_lda), __pyx_v_x, (&__pyx_v_incx), (&__pyx_v_beta), __pyx_v_y, (&__pyx_v_incy));

  /* "scipy/linalg/_decomp_update.pyx":206
 *         blas_pointers.zgeru(&m, &n, &alpha, x, &incx, y, &incy, a, &lda)
 * 
 * cdef inline void gemv(char* trans, int m, int n, blas_t alpha, blas_t* a,             # <<<<<<<<<<<<<<
 *         int lda, blas_t* x, int incx, blas_t beta, blas_t* y, int incy) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemv(char *__pyx_v_trans, int __pyx_v_m, int __pyx_v_n, __pyx_t_double_complex __pyx_v_alpha, __pyx_t_double_complex *__pyx_v_a, int __pyx_v_lda, __pyx_t_double_complex *__pyx_v_x, int __pyx_v_incx, __pyx_t_double_complex __pyx_v_beta, __pyx_t_double_complex *__pyx_v_y, int __pyx_v_incy) {

  /* "scipy/linalg/_decomp_update.pyx":218
 *                 &beta, y, &incy)
 *     else:  # blas_t is double_complex:
 *         blas_pointers.zgemv(trans, &m, &n, &alpha, a, &lda, x, &incx,             # <<<<<<<<<<<<<<
 *                 &beta, y, &incy)
 * 
 */
  __pyx_f_5scipy_6linalg_11cython_blas_zgemv(__pyx_v_trans, (&__pyx_v_m), (&__pyx_v_n), (&__pyx_v_alpha), __pyx_v_a, (&__pyx_v_lda), __pyx_v_x, (&__pyx_v_incx), (&__pyx_v_beta), __pyx_v_y, (&__pyx_v_incy));

  /* "scipy/linalg/_decomp_update.pyx":206
 *         blas_pointers.zgeru(&m, &n, &alpha, x, &incx, y, &incy, a, &lda)
 * 
 * cdef inline void gemv(char* trans, int m, int n, blas_t alpha, blas_t* a,             # <<<<<<<<<<<<<<
 *         int lda, blas_t* x, int incx, blas_t beta, blas_t* y, int incy) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":221
 *                 &beta, y, &incy)
 * 
 * cdef inline void gemm(char* transa, char* transb, int m, int n, int k,             # <<<<<<<<<<<<<<
 *         blas_t alpha, blas_t* a, int lda, blas_t* b, int ldb, blas_t beta,
 *         blas_t* c, int ldc) nogil:
 */

static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemm(char *__pyx_v_transa, char *__pyx_v_transb, int __pyx_v_m, int __pyx_v_n, int __pyx_v_k, float __pyx_v_alpha, float *__pyx_v_a, int __pyx_v_lda, float *__pyx_v_b, int __pyx_v_ldb, float __pyx_v_beta, float *__pyx_v_c, int __pyx_v_ldc) {

  /* "scipy/linalg/_decomp_update.pyx":225
 *         blas_t* c, int ldc) nogil:
 *     if blas_t is float:
 *         blas_pointers.sgemm(transa, transb, &m, &n, &k, &alpha, a, &lda,             # <<<<<<<<<<<<<<
 *                 b, &ldb, &beta, c, &ldc)
 *     elif blas_t is double:
 */
  __pyx_f_5scipy_6linalg_11cython_blas_sgemm(__pyx_v_transa, __pyx_v_transb, (&__pyx_v_m), (&__pyx_v_n), (&__pyx_v_k), (&__pyx_v_alpha), __pyx_v_a, (&__pyx_v_lda), __pyx_v_b, (&__pyx_v_ldb), (&__pyx_v_beta), __pyx_v_c, (&__pyx_v_ldc));

  /* "scipy/linalg/_decomp_update.pyx":221
 *                 &beta, y, &incy)
 * 
 * cdef inline void gemm(char* transa, char* transb, int m, int n, int k,             # <<<<<<<<<<<<<<
 *         blas_t alpha, blas_t* a, int lda, blas_t* b, int ldb, blas_t beta,
 *         blas_t* c, int ldc) nogil:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemm(char *__pyx_v_transa, char *__pyx_v_transb, int __pyx_v_m, int __pyx_v_n, int __pyx_v_k, double __pyx_v_alpha, double *__pyx_v_a, int __pyx_v_lda, double *__pyx_v_b, int __pyx_v_ldb, double __pyx_v_beta, double *__pyx_v_c, int __pyx_v_ldc) {

  /* "scipy/linalg/_decomp_update.pyx":228
 *                 b, &ldb, &beta, c, &ldc)
 *     elif blas_t is double:
 *         blas_pointers.dgemm(transa, transb, &m, &n, &k, &alpha, a, &lda,             # <<<<<<<<<<<<<<
 *                 b, &ldb, &beta, c, &ldc)
 *     elif blas_t is float_complex:
 */
  __pyx_f_5scipy_6linalg_11cython_blas_dgemm(__pyx_v_transa, __pyx_v_transb, (&__pyx_v_m), (&__pyx_v_n), (&__pyx_v_k), (&__pyx_v_alpha), __pyx_v_a, (&__pyx_v_lda), __pyx_v_b, (&__pyx_v_ldb), (&__pyx_v_beta), __pyx_v_c, (&__pyx_v_ldc));

  /* "scipy/linalg/_decomp_update.pyx":221
 *                 &beta, y, &incy)
 * 
 * cdef inline void gemm(char* transa, char* transb, int m, int n, int k,             # <<<<<<<<<<<<<<
 *         blas_t alpha, blas_t* a, int lda, blas_t* b, int ldb, blas_t beta,
 *         blas_t* c, int ldc) nogil:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemm(char *__pyx_v_transa, char *__pyx_v_transb, int __pyx_v_m, int __pyx_v_n, int __pyx_v_k, __pyx_t_float_complex __pyx_v_alpha, __pyx_t_float_complex *__pyx_v_a, int __pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int __pyx_v_ldb, __pyx_t_float_complex __pyx_v_beta, __pyx_t_float_complex *__pyx_v_c, int __pyx_v_ldc) {

  /* "scipy/linalg/_decomp_update.pyx":231
 *                 b, &ldb, &beta, c, &ldc)
 *     elif blas_t is float_complex:
 *         blas_pointers.cgemm(transa, transb, &m, &n, &k, &alpha, a, &lda,             # <<<<<<<<<<<<<<
 *                 b, &ldb, &beta, c, &ldc)
 *     else:  # blas_t is double_complex:
 */
  __pyx_f_5scipy_6linalg_11cython_blas_cgemm(__pyx_v_transa, __pyx_v_transb, (&__pyx_v_m), (&__pyx_v_n), (&__pyx_v_k), (&__pyx_v_alpha), __pyx_v_a, (&__pyx_v_lda), __pyx_v_b, (&__pyx_v_ldb), (&__pyx_v_beta), __pyx_v_c, (&__pyx_v_ldc));

  /* "scipy/linalg/_decomp_update.pyx":221
 *                 &beta, y, &incy)
 * 
 * cdef inline void gemm(char* transa, char* transb, int m, int n, int k,             # <<<<<<<<<<<<<<
 *         blas_t alpha, blas_t* a, int lda, blas_t* b, int ldb, blas_t beta,
 *         blas_t* c, int ldc) nogil:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemm(char *__pyx_v_transa, char *__pyx_v_transb, int __pyx_v_m, int __pyx_v_n, int __pyx_v_k, __pyx_t_double_complex __pyx_v_alpha, __pyx_t_double_complex *__pyx_v_a, int __pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int __pyx_v_ldb, __pyx_t_double_complex __pyx_v_beta, __pyx_t_double_complex *__pyx_v_c, int __pyx_v_ldc) {

  /* "scipy/linalg/_decomp_update.pyx":234
 *                 b, &ldb, &beta, c, &ldc)
 *     else:  # blas_t is double_complex:
 *         blas_pointers.zgemm(transa, transb, &m, &n, &k, &alpha, a, &lda,             # <<<<<<<<<<<<<<
 *                 b, &ldb, &beta, c, &ldc)
 * 
 */
  __pyx_f_5scipy_6linalg_11cython_blas_zgemm(__pyx_v_transa, __pyx_v_transb, (&__pyx_v_m), (&__pyx_v_n), (&__pyx_v_k), (&__pyx_v_alpha), __pyx_v_a, (&__pyx_v_lda), __pyx_v_b, (&__pyx_v_ldb), (&__pyx_v_beta), __pyx_v_c, (&__pyx_v_ldc));

  /* "scipy/linalg/_decomp_update.pyx":221
 *                 &beta, y, &incy)
 * 
 * cdef inline void gemm(char* transa, char* transb, int m, int n, int k,             # <<<<<<<<<<<<<<
 *         blas_t alpha, blas_t* a, int lda, blas_t* b, int ldb, blas_t beta,
 *         blas_t* c, int ldc) nogil:
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":237
 *                 b, &ldb, &beta, c, &ldc)
 * 
 * cdef inline void trmm(char* side, char* uplo, char* transa, char* diag, int m,             # <<<<<<<<<<<<<<
 *         int n, blas_t alpha, blas_t* a, int lda, blas_t* b, int ldb) nogil:
 *     if blas_t is float:
 */

static CYTHON_INLINE void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_trmm(char *__pyx_v_side, char *__pyx_v_uplo, char *__pyx_v_transa, char *__pyx_v_diag, int __pyx_v_m, int __pyx_v_n, float __pyx_v_alpha, float *__pyx_v_a, int __pyx_v_lda, float *__pyx_v_b, int __pyx_v_ldb) {

  /* "scipy/linalg/_decomp_update.pyx":240
 *         int n, blas_t alpha, blas_t* a, int lda, blas_t* b, int ldb) nogil:
 *     if blas_t is float:
 *         blas_pointers.strmm(side, uplo, transa, diag, &m, &n, &alpha, a, &lda,             # <<<<<<<<<<<<<<
 *                 b, &ldb)
 *     elif blas_t is double:
 */
  __pyx_f_5scipy_6linalg_11cython_blas_strmm(__pyx_v_side, __pyx_v_uplo, __pyx_v_transa, __pyx_v_diag, (&__pyx_v_m), (&__pyx_v_n), (&__pyx_v_alpha), __pyx_v_a, (&__pyx_v_lda), __pyx_v_b, (&__pyx_v_ldb));

  /* "scipy/linalg/_decomp_update.pyx":237
 *                 b, &ldb, &beta, c, &ldc)
 * 
 * cdef inline void trmm(char* side, char* uplo, char* transa, char* diag, int m,             # <<<<<<<<<<<<<<
 *         int n, blas_t alpha, blas_t* a, int lda, blas_t* b, int ldb) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_trmm(char *__pyx_v_side, char *__pyx_v_uplo, char *__pyx_v_transa, char *__pyx_v_diag, int __pyx_v_m, int __pyx_v_n, double __pyx_v_alpha, double *__pyx_v_a, int __pyx_v_lda, double *__pyx_v_b, int __pyx_v_ldb) {

  /* "scipy/linalg/_decomp_update.pyx":243
 *                 b, &ldb)
 *     elif blas_t is double:
 *         blas_pointers.dtrmm(side, uplo, transa, diag, &m, &n, &alpha, a, &lda,             # <<<<<<<<<<<<<<
 *                 b, &ldb)
 *     elif blas_t is float_complex:
 */
  __pyx_f_5scipy_6linalg_11cython_blas_dtrmm(__pyx_v_side, __pyx_v_uplo, __pyx_v_transa, __pyx_v_diag, (&__pyx_v_m), (&__pyx_v_n), (&__pyx_v_alpha), __pyx_v_a, (&__pyx_v_lda), __pyx_v_b, (&__pyx_v_ldb));

  /* "scipy/linalg/_decomp_update.pyx":237
 *                 b, &ldb, &beta, c, &ldc)
 * 
 * cdef inline void trmm(char* side, char* uplo, char* transa, char* diag, int m,             # <<<<<<<<<<<<<<
 *         int n, blas_t alpha, blas_t* a, int lda, blas_t* b, int ldb) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_trmm(char *__pyx_v_side, char *__pyx_v_uplo, char *__pyx_v_transa, char *__pyx_v_diag, int __pyx_v_m, int __pyx_v_n, __pyx_t_float_complex __pyx_v_alpha, __pyx_t_float_complex *__pyx_v_a, int __pyx_v_lda, __pyx_t_float_complex *__pyx_v_b, int __pyx_v_ldb) {

  /* "scipy/linalg/_decomp_update.pyx":246
 *                 b, &ldb)
 *     elif blas_t is float_complex:
 *         blas_pointers.ctrmm(side, uplo, transa, diag, &m, &n, &alpha, a, &lda,             # <<<<<<<<<<<<<<
 *                 b, &ldb)
 *     else:  # blas_t is double_complex:
 */
  __pyx_f_5scipy_6linalg_11cython_blas_ctrmm(__pyx_v_side, __pyx_v_uplo, __pyx_v_transa, __pyx_v_diag, (&__pyx_v_m), (&__pyx_v_n), (&__pyx_v_alpha), __pyx_v_a, (&__pyx_v_lda), __pyx_v_b, (&__pyx_v_ldb));

  /* "scipy/linalg/_decomp_update.pyx":237
 *                 b, &ldb, &beta, c, &ldc)
 * 
 * cdef inline void trmm(char* side, char* uplo, char* transa, char* diag, int m,             # <<<<<<<<<<<<<<
 *         int n, blas_t alpha, blas_t* a, int lda, blas_t* b, int ldb) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

static CYTHON_INLINE void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_trmm(char *__pyx_v_side, char *__pyx_v_uplo, char *__pyx_v_transa, char *__pyx_v_diag, int __pyx_v_m, int __pyx_v_n, __pyx_t_double_complex __pyx_v_alpha, __pyx_t_double_complex *__pyx_v_a, int __pyx_v_lda, __pyx_t_double_complex *__pyx_v_b, int __pyx_v_ldb) {

  /* "scipy/linalg/_decomp_update.pyx":249
 *                 b, &ldb)
 *     else:  # blas_t is double_complex:
 *         blas_pointers.ztrmm(side, uplo, transa, diag, &m, &n, &alpha, a, &lda,             # <<<<<<<<<<<<<<
 *                 b, &ldb)
 * 
 */
  __pyx_f_5scipy_6linalg_11cython_blas_ztrmm(__pyx_v_side, __pyx_v_uplo, __pyx_v_transa, __pyx_v_diag, (&__pyx_v_m), (&__pyx_v_n), (&__pyx_v_alpha), __pyx_v_a, (&__pyx_v_lda), __pyx_v_b, (&__pyx_v_ldb));

  /* "scipy/linalg/_decomp_update.pyx":237
 *                 b, &ldb, &beta, c, &ldc)
 * 
 * cdef inline void trmm(char* side, char* uplo, char* transa, char* diag, int m,             # <<<<<<<<<<<<<<
 *         int n, blas_t alpha, blas_t* a, int lda, blas_t* b, int ldb) nogil:
 *     if blas_t is float:
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":252
 *                 b, &ldb)
 * 
 * cdef inline int geqrf(int m, int n, blas_t* a, int lda, blas_t* tau,             # <<<<<<<<<<<<<<
 *                       blas_t* work, int lwork) nogil:
 *     cdef int info
 */

static CYTHON_INLINE int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_geqrf(int __pyx_v_m, int __pyx_v_n, float *__pyx_v_a, int __pyx_v_lda, float *__pyx_v_tau, float *__pyx_v_work, int __pyx_v_lwork) {
  int __pyx_v_info;
  int __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":256
 *     cdef int info
 *     if blas_t is float:
 *         lapack_pointers.sgeqrf(&m, &n, a, &lda, tau, work, &lwork, &info)             # <<<<<<<<<<<<<<
 *     elif blas_t is double:
 *         lapack_pointers.dgeqrf(&m, &n, a, &lda, tau, work, &lwork, &info)
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_sgeqrf((&__pyx_v_m), (&__pyx_v_n), __pyx_v_a, (&__pyx_v_lda), __pyx_v_tau, __pyx_v_work, (&__pyx_v_lwork), (&__pyx_v_info));

  /* "scipy/linalg/_decomp_update.pyx":263
 *     else:  # blas_t is double_complex:
 *         lapack_pointers.zgeqrf(&m, &n, a, &lda, tau, work, &lwork, &info)
 *     return info             # <<<<<<<<<<<<<<
 * 
 * cdef inline int ormqr(char* side, char* trans, int m, int n, int k, blas_t* a,
 */
  __pyx_r = __pyx_v_info;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":252
 *                 b, &ldb)
 * 
 * cdef inline int geqrf(int m, int n, blas_t* a, int lda, blas_t* tau,             # <<<<<<<<<<<<<<
 *                       blas_t* work, int lwork) nogil:
 *     cdef int info
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_geqrf(int __pyx_v_m, int __pyx_v_n, double *__pyx_v_a, int __pyx_v_lda, double *__pyx_v_tau, double *__pyx_v_work, int __pyx_v_lwork) {
  int __pyx_v_info;
  int __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":258
 *         lapack_pointers.sgeqrf(&m, &n, a, &lda, tau, work, &lwork, &info)
 *     elif blas_t is double:
 *         lapack_pointers.dgeqrf(&m, &n, a, &lda, tau, work, &lwork, &info)             # <<<<<<<<<<<<<<
 *     elif blas_t is float_complex:
 *         lapack_pointers.cgeqrf(&m, &n, a, &lda, tau, work, &lwork, &info)
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_dgeqrf((&__pyx_v_m), (&__pyx_v_n), __pyx_v_a, (&__pyx_v_lda), __pyx_v_tau, __pyx_v_work, (&__pyx_v_lwork), (&__pyx_v_info));

  /* "scipy/linalg/_decomp_update.pyx":263
 *     else:  # blas_t is double_complex:
 *         lapack_pointers.zgeqrf(&m, &n, a, &lda, tau, work, &lwork, &info)
 *     return info             # <<<<<<<<<<<<<<
 * 
 * cdef inline int ormqr(char* side, char* trans, int m, int n, int k, blas_t* a,
 */
  __pyx_r = __pyx_v_info;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":252
 *                 b, &ldb)
 * 
 * cdef inline int geqrf(int m, int n, blas_t* a, int lda, blas_t* tau,             # <<<<<<<<<<<<<<
 *                       blas_t* work, int lwork) nogil:
 *     cdef int info
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_geqrf(int __pyx_v_m, int __pyx_v_n, __pyx_t_float_complex *__pyx_v_a, int __pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_work, int __pyx_v_lwork) {
  int __pyx_v_info;
  int __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":260
 *         lapack_pointers.dgeqrf(&m, &n, a, &lda, tau, work, &lwork, &info)
 *     elif blas_t is float_complex:
 *         lapack_pointers.cgeqrf(&m, &n, a, &lda, tau, work, &lwork, &info)             # <<<<<<<<<<<<<<
 *     else:  # blas_t is double_complex:
 *         lapack_pointers.zgeqrf(&m, &n, a, &lda, tau, work, &lwork, &info)
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_cgeqrf((&__pyx_v_m), (&__pyx_v_n), __pyx_v_a, (&__pyx_v_lda), __pyx_v_tau, __pyx_v_work, (&__pyx_v_lwork), (&__pyx_v_info));

  /* "scipy/linalg/_decomp_update.pyx":263
 *     else:  # blas_t is double_complex:
 *         lapack_pointers.zgeqrf(&m, &n, a, &lda, tau, work, &lwork, &info)
 *     return info             # <<<<<<<<<<<<<<
 * 
 * cdef inline int ormqr(char* side, char* trans, int m, int n, int k, blas_t* a,
 */
  __pyx_r = __pyx_v_info;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":252
 *                 b, &ldb)
 * 
 * cdef inline int geqrf(int m, int n, blas_t* a, int lda, blas_t* tau,             # <<<<<<<<<<<<<<
 *                       blas_t* work, int lwork) nogil:
 *     cdef int info
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_geqrf(int __pyx_v_m, int __pyx_v_n, __pyx_t_double_complex *__pyx_v_a, int __pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_work, int __pyx_v_lwork) {
  int __pyx_v_info;
  int __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":262
 *         lapack_pointers.cgeqrf(&m, &n, a, &lda, tau, work, &lwork, &info)
 *     else:  # blas_t is double_complex:
 *         lapack_pointers.zgeqrf(&m, &n, a, &lda, tau, work, &lwork, &info)             # <<<<<<<<<<<<<<
 *     return info
 * 
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_zgeqrf((&__pyx_v_m), (&__pyx_v_n), __pyx_v_a, (&__pyx_v_lda), __pyx_v_tau, __pyx_v_work, (&__pyx_v_lwork), (&__pyx_v_info));

  /* "scipy/linalg/_decomp_update.pyx":263
 *     else:  # blas_t is double_complex:
 *         lapack_pointers.zgeqrf(&m, &n, a, &lda, tau, work, &lwork, &info)
 *     return info             # <<<<<<<<<<<<<<
 * 
 * cdef inline int ormqr(char* side, char* trans, int m, int n, int k, blas_t* a,
 */
  __pyx_r = __pyx_v_info;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":252
 *                 b, &ldb)
 * 
 * cdef inline int geqrf(int m, int n, blas_t* a, int lda, blas_t* tau,             # <<<<<<<<<<<<<<
 *                       blas_t* work, int lwork) nogil:
 *     cdef int info
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":265
 *     return info
 * 
 * cdef inline int ormqr(char* side, char* trans, int m, int n, int k, blas_t* a,             # <<<<<<<<<<<<<<
 *     int lda, blas_t* tau, blas_t* c, int ldc, blas_t* work, int lwork) nogil:
 *     cdef int info = 0
 */

static CYTHON_INLINE int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(char *__pyx_v_side, char *__pyx_v_trans, int __pyx_v_m, int __pyx_v_n, int __pyx_v_k, float *__pyx_v_a, int __pyx_v_lda, float *__pyx_v_tau, float *__pyx_v_c, int __pyx_v_ldc, float *__pyx_v_work, int __pyx_v_lwork) {
  int __pyx_v_info;
  int __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":267
 * cdef inline int ormqr(char* side, char* trans, int m, int n, int k, blas_t* a,
 *     int lda, blas_t* tau, blas_t* c, int ldc, blas_t* work, int lwork) nogil:
 *     cdef int info = 0             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         lapack_pointers.sormqr(side, trans, &m, &n, &k, a, &lda, tau, c, &ldc,
 */
  __pyx_v_info = 0;

  /* "scipy/linalg/_decomp_update.pyx":269
 *     cdef int info = 0
 *     if blas_t is float:
 *         lapack_pointers.sormqr(side, trans, &m, &n, &k, a, &lda, tau, c, &ldc,             # <<<<<<<<<<<<<<
 *                 work, &lwork, &info)
 *     elif blas_t is double:
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_sormqr(__pyx_v_side, __pyx_v_trans, (&__pyx_v_m), (&__pyx_v_n), (&__pyx_v_k), __pyx_v_a, (&__pyx_v_lda), __pyx_v_tau, __pyx_v_c, (&__pyx_v_ldc), __pyx_v_work, (&__pyx_v_lwork), (&__pyx_v_info));

  /* "scipy/linalg/_decomp_update.pyx":280
 *         lapack_pointers.zunmqr(side, trans, &m, &n, &k, a, &lda, tau, c, &ldc,
 *                 work, &lwork, &info)
 *     return info             # <<<<<<<<<<<<<<
 * 
 * #------------------------------------------------------------------------------
 */
  __pyx_r = __pyx_v_info;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":265
 *     return info
 * 
 * cdef inline int ormqr(char* side, char* trans, int m, int n, int k, blas_t* a,             # <<<<<<<<<<<<<<
 *     int lda, blas_t* tau, blas_t* c, int ldc, blas_t* work, int lwork) nogil:
 *     cdef int info = 0
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(char *__pyx_v_side, char *__pyx_v_trans, int __pyx_v_m, int __pyx_v_n, int __pyx_v_k, double *__pyx_v_a, int __pyx_v_lda, double *__pyx_v_tau, double *__pyx_v_c, int __pyx_v_ldc, double *__pyx_v_work, int __pyx_v_lwork) {
  int __pyx_v_info;
  int __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":267
 * cdef inline int ormqr(char* side, char* trans, int m, int n, int k, blas_t* a,
 *     int lda, blas_t* tau, blas_t* c, int ldc, blas_t* work, int lwork) nogil:
 *     cdef int info = 0             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         lapack_pointers.sormqr(side, trans, &m, &n, &k, a, &lda, tau, c, &ldc,
 */
  __pyx_v_info = 0;

  /* "scipy/linalg/_decomp_update.pyx":272
 *                 work, &lwork, &info)
 *     elif blas_t is double:
 *         lapack_pointers.dormqr(side, trans, &m, &n, &k, a, &lda, tau, c, &ldc,             # <<<<<<<<<<<<<<
 *                 work, &lwork, &info)
 *     elif blas_t is float_complex:
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_dormqr(__pyx_v_side, __pyx_v_trans, (&__pyx_v_m), (&__pyx_v_n), (&__pyx_v_k), __pyx_v_a, (&__pyx_v_lda), __pyx_v_tau, __pyx_v_c, (&__pyx_v_ldc), __pyx_v_work, (&__pyx_v_lwork), (&__pyx_v_info));

  /* "scipy/linalg/_decomp_update.pyx":280
 *         lapack_pointers.zunmqr(side, trans, &m, &n, &k, a, &lda, tau, c, &ldc,
 *                 work, &lwork, &info)
 *     return info             # <<<<<<<<<<<<<<
 * 
 * #------------------------------------------------------------------------------
 */
  __pyx_r = __pyx_v_info;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":265
 *     return info
 * 
 * cdef inline int ormqr(char* side, char* trans, int m, int n, int k, blas_t* a,             # <<<<<<<<<<<<<<
 *     int lda, blas_t* tau, blas_t* c, int ldc, blas_t* work, int lwork) nogil:
 *     cdef int info = 0
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(char *__pyx_v_side, char *__pyx_v_trans, int __pyx_v_m, int __pyx_v_n, int __pyx_v_k, __pyx_t_float_complex *__pyx_v_a, int __pyx_v_lda, __pyx_t_float_complex *__pyx_v_tau, __pyx_t_float_complex *__pyx_v_c, int __pyx_v_ldc, __pyx_t_float_complex *__pyx_v_work, int __pyx_v_lwork) {
  int __pyx_v_info;
  int __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":267
 * cdef inline int ormqr(char* side, char* trans, int m, int n, int k, blas_t* a,
 *     int lda, blas_t* tau, blas_t* c, int ldc, blas_t* work, int lwork) nogil:
 *     cdef int info = 0             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         lapack_pointers.sormqr(side, trans, &m, &n, &k, a, &lda, tau, c, &ldc,
 */
  __pyx_v_info = 0;

  /* "scipy/linalg/_decomp_update.pyx":275
 *                 work, &lwork, &info)
 *     elif blas_t is float_complex:
 *         lapack_pointers.cunmqr(side, trans, &m, &n, &k, a, &lda, tau, c, &ldc,             # <<<<<<<<<<<<<<
 *                 work, &lwork, &info)
 *     else:
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_cunmqr(__pyx_v_side, __pyx_v_trans, (&__pyx_v_m), (&__pyx_v_n), (&__pyx_v_k), __pyx_v_a, (&__pyx_v_lda), __pyx_v_tau, __pyx_v_c, (&__pyx_v_ldc), __pyx_v_work, (&__pyx_v_lwork), (&__pyx_v_info));

  /* "scipy/linalg/_decomp_update.pyx":280
 *         lapack_pointers.zunmqr(side, trans, &m, &n, &k, a, &lda, tau, c, &ldc,
 *                 work, &lwork, &info)
 *     return info             # <<<<<<<<<<<<<<
 * 
 * #------------------------------------------------------------------------------
 */
  __pyx_r = __pyx_v_info;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":265
 *     return info
 * 
 * cdef inline int ormqr(char* side, char* trans, int m, int n, int k, blas_t* a,             # <<<<<<<<<<<<<<
 *     int lda, blas_t* tau, blas_t* c, int ldc, blas_t* work, int lwork) nogil:
 *     cdef int info = 0
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(char *__pyx_v_side, char *__pyx_v_trans, int __pyx_v_m, int __pyx_v_n, int __pyx_v_k, __pyx_t_double_complex *__pyx_v_a, int __pyx_v_lda, __pyx_t_double_complex *__pyx_v_tau, __pyx_t_double_complex *__pyx_v_c, int __pyx_v_ldc, __pyx_t_double_complex *__pyx_v_work, int __pyx_v_lwork) {
  int __pyx_v_info;
  int __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":267
 * cdef inline int ormqr(char* side, char* trans, int m, int n, int k, blas_t* a,
 *     int lda, blas_t* tau, blas_t* c, int ldc, blas_t* work, int lwork) nogil:
 *     cdef int info = 0             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         lapack_pointers.sormqr(side, trans, &m, &n, &k, a, &lda, tau, c, &ldc,
 */
  __pyx_v_info = 0;

  /* "scipy/linalg/_decomp_update.pyx":278
 *                 work, &lwork, &info)
 *     else:
 *         lapack_pointers.zunmqr(side, trans, &m, &n, &k, a, &lda, tau, c, &ldc,             # <<<<<<<<<<<<<<
 *                 work, &lwork, &info)
 *     return info
 */
  __pyx_f_5scipy_6linalg_13cython_lapack_zunmqr(__pyx_v_side, __pyx_v_trans, (&__pyx_v_m), (&__pyx_v_n), (&__pyx_v_k), __pyx_v_a, (&__pyx_v_lda), __pyx_v_tau, __pyx_v_c, (&__pyx_v_ldc), __pyx_v_work, (&__pyx_v_lwork), (&__pyx_v_info));

  /* "scipy/linalg/_decomp_update.pyx":280
 *         lapack_pointers.zunmqr(side, trans, &m, &n, &k, a, &lda, tau, c, &ldc,
 *                 work, &lwork, &info)
 *     return info             # <<<<<<<<<<<<<<
 * 
 * #------------------------------------------------------------------------------
 */
  __pyx_r = __pyx_v_info;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":265
 *     return info
 * 
 * cdef inline int ormqr(char* side, char* trans, int m, int n, int k, blas_t* a,             # <<<<<<<<<<<<<<
 *     int lda, blas_t* tau, blas_t* c, int ldc, blas_t* work, int lwork) nogil:
 *     cdef int info = 0
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":286
 * #------------------------------------------------------------------------------
 * 
 * cdef void blas_t_conj(int n, blas_t* x, int* xs) nogil:             # <<<<<<<<<<<<<<
 *     cdef int j
 *     if blas_t is float_complex or blas_t is double_complex:
 */

static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(CYTHON_UNUSED int __pyx_v_n, CYTHON_UNUSED float *__pyx_v_x, CYTHON_UNUSED int *__pyx_v_xs) {

  /* "scipy/linalg/_decomp_update.pyx":288
 * cdef void blas_t_conj(int n, blas_t* x, int* xs) nogil:
 *     cdef int j
 *     if blas_t is float_complex or blas_t is double_complex:             # <<<<<<<<<<<<<<
 *         for j in range(n):
 *             index1(x, xs, j)[0] = index1(x, xs, j)[0].conjugate()
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":286
 * #------------------------------------------------------------------------------
 * 
 * cdef void blas_t_conj(int n, blas_t* x, int* xs) nogil:             # <<<<<<<<<<<<<<
 *     cdef int j
 *     if blas_t is float_complex or blas_t is double_complex:
 */

static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(CYTHON_UNUSED int __pyx_v_n, CYTHON_UNUSED double *__pyx_v_x, CYTHON_UNUSED int *__pyx_v_xs) {

  /* "scipy/linalg/_decomp_update.pyx":288
 * cdef void blas_t_conj(int n, blas_t* x, int* xs) nogil:
 *     cdef int j
 *     if blas_t is float_complex or blas_t is double_complex:             # <<<<<<<<<<<<<<
 *         for j in range(n):
 *             index1(x, xs, j)[0] = index1(x, xs, j)[0].conjugate()
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":286
 * #------------------------------------------------------------------------------
 * 
 * cdef void blas_t_conj(int n, blas_t* x, int* xs) nogil:             # <<<<<<<<<<<<<<
 *     cdef int j
 *     if blas_t is float_complex or blas_t is double_complex:
 */

static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(int __pyx_v_n, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_xs) {
  int __pyx_v_j;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":289
 *     cdef int j
 *     if blas_t is float_complex or blas_t is double_complex:
 *         for j in range(n):             # <<<<<<<<<<<<<<
 *             index1(x, xs, j)[0] = index1(x, xs, j)[0].conjugate()
 * 
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":290
 *     if blas_t is float_complex or blas_t is double_complex:
 *         for j in range(n):
 *             index1(x, xs, j)[0] = index1(x, xs, j)[0].conjugate()             # <<<<<<<<<<<<<<
 * 
 * cdef void blas_t_2d_conj(int m, int n, blas_t* x, int* xs) nogil:
 */
    (__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_x, __pyx_v_xs, __pyx_v_j)[0]) = __Pyx_c_conj_float((__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_x, __pyx_v_xs, __pyx_v_j)[0]));
  }

  /* "scipy/linalg/_decomp_update.pyx":286
 * #------------------------------------------------------------------------------
 * 
 * cdef void blas_t_conj(int n, blas_t* x, int* xs) nogil:             # <<<<<<<<<<<<<<
 *     cdef int j
 *     if blas_t is float_complex or blas_t is double_complex:
 */

  /* function exit code */
}

static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(int __pyx_v_n, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_xs) {
  int __pyx_v_j;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":289
 *     cdef int j
 *     if blas_t is float_complex or blas_t is double_complex:
 *         for j in range(n):             # <<<<<<<<<<<<<<
 *             index1(x, xs, j)[0] = index1(x, xs, j)[0].conjugate()
 * 
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":290
 *     if blas_t is float_complex or blas_t is double_complex:
 *         for j in range(n):
 *             index1(x, xs, j)[0] = index1(x, xs, j)[0].conjugate()             # <<<<<<<<<<<<<<
 * 
 * cdef void blas_t_2d_conj(int m, int n, blas_t* x, int* xs) nogil:
 */
    (__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_x, __pyx_v_xs, __pyx_v_j)[0]) = __Pyx_c_conj_double((__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_x, __pyx_v_xs, __pyx_v_j)[0]));
  }

  /* "scipy/linalg/_decomp_update.pyx":286
 * #------------------------------------------------------------------------------
 * 
 * cdef void blas_t_conj(int n, blas_t* x, int* xs) nogil:             # <<<<<<<<<<<<<<
 *     cdef int j
 *     if blas_t is float_complex or blas_t is double_complex:
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":292
 *             index1(x, xs, j)[0] = index1(x, xs, j)[0].conjugate()
 * 
 * cdef void blas_t_2d_conj(int m, int n, blas_t* x, int* xs) nogil:             # <<<<<<<<<<<<<<
 *     cdef int i, j
 *     if blas_t is float_complex or blas_t is double_complex:
 */

static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_2d_conj(CYTHON_UNUSED int __pyx_v_m, CYTHON_UNUSED int __pyx_v_n, CYTHON_UNUSED float *__pyx_v_x, CYTHON_UNUSED int *__pyx_v_xs) {

  /* "scipy/linalg/_decomp_update.pyx":294
 * cdef void blas_t_2d_conj(int m, int n, blas_t* x, int* xs) nogil:
 *     cdef int i, j
 *     if blas_t is float_complex or blas_t is double_complex:             # <<<<<<<<<<<<<<
 *         for i in range(m):
 *             for j in range(n):
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":292
 *             index1(x, xs, j)[0] = index1(x, xs, j)[0].conjugate()
 * 
 * cdef void blas_t_2d_conj(int m, int n, blas_t* x, int* xs) nogil:             # <<<<<<<<<<<<<<
 *     cdef int i, j
 *     if blas_t is float_complex or blas_t is double_complex:
 */

static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_2d_conj(CYTHON_UNUSED int __pyx_v_m, CYTHON_UNUSED int __pyx_v_n, CYTHON_UNUSED double *__pyx_v_x, CYTHON_UNUSED int *__pyx_v_xs) {

  /* "scipy/linalg/_decomp_update.pyx":294
 * cdef void blas_t_2d_conj(int m, int n, blas_t* x, int* xs) nogil:
 *     cdef int i, j
 *     if blas_t is float_complex or blas_t is double_complex:             # <<<<<<<<<<<<<<
 *         for i in range(m):
 *             for j in range(n):
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":292
 *             index1(x, xs, j)[0] = index1(x, xs, j)[0].conjugate()
 * 
 * cdef void blas_t_2d_conj(int m, int n, blas_t* x, int* xs) nogil:             # <<<<<<<<<<<<<<
 *     cdef int i, j
 *     if blas_t is float_complex or blas_t is double_complex:
 */

static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_2d_conj(int __pyx_v_m, int __pyx_v_n, __pyx_t_float_complex *__pyx_v_x, int *__pyx_v_xs) {
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;

  /* "scipy/linalg/_decomp_update.pyx":295
 *     cdef int i, j
 *     if blas_t is float_complex or blas_t is double_complex:
 *         for i in range(m):             # <<<<<<<<<<<<<<
 *             for j in range(n):
 *                 index2(x, xs, i, j)[0] = index2(x, xs, i, j)[0].conjugate()
 */
  __pyx_t_1 = __pyx_v_m;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":296
 *     if blas_t is float_complex or blas_t is double_complex:
 *         for i in range(m):
 *             for j in range(n):             # <<<<<<<<<<<<<<
 *                 index2(x, xs, i, j)[0] = index2(x, xs, i, j)[0].conjugate()
 * 
 */
    __pyx_t_4 = __pyx_v_n;
    __pyx_t_5 = __pyx_t_4;
    for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {
      __pyx_v_j = __pyx_t_6;

      /* "scipy/linalg/_decomp_update.pyx":297
 *         for i in range(m):
 *             for j in range(n):
 *                 index2(x, xs, i, j)[0] = index2(x, xs, i, j)[0].conjugate()             # <<<<<<<<<<<<<<
 * 
 * cdef blas_t blas_t_sqrt(blas_t x) nogil:
 */
      (__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_x, __pyx_v_xs, __pyx_v_i, __pyx_v_j)[0]) = __Pyx_c_conj_float((__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_x, __pyx_v_xs, __pyx_v_i, __pyx_v_j)[0]));
    }
  }

  /* "scipy/linalg/_decomp_update.pyx":292
 *             index1(x, xs, j)[0] = index1(x, xs, j)[0].conjugate()
 * 
 * cdef void blas_t_2d_conj(int m, int n, blas_t* x, int* xs) nogil:             # <<<<<<<<<<<<<<
 *     cdef int i, j
 *     if blas_t is float_complex or blas_t is double_complex:
 */

  /* function exit code */
}

static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_2d_conj(int __pyx_v_m, int __pyx_v_n, __pyx_t_double_complex *__pyx_v_x, int *__pyx_v_xs) {
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;

  /* "scipy/linalg/_decomp_update.pyx":295
 *     cdef int i, j
 *     if blas_t is float_complex or blas_t is double_complex:
 *         for i in range(m):             # <<<<<<<<<<<<<<
 *             for j in range(n):
 *                 index2(x, xs, i, j)[0] = index2(x, xs, i, j)[0].conjugate()
 */
  __pyx_t_1 = __pyx_v_m;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":296
 *     if blas_t is float_complex or blas_t is double_complex:
 *         for i in range(m):
 *             for j in range(n):             # <<<<<<<<<<<<<<
 *                 index2(x, xs, i, j)[0] = index2(x, xs, i, j)[0].conjugate()
 * 
 */
    __pyx_t_4 = __pyx_v_n;
    __pyx_t_5 = __pyx_t_4;
    for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {
      __pyx_v_j = __pyx_t_6;

      /* "scipy/linalg/_decomp_update.pyx":297
 *         for i in range(m):
 *             for j in range(n):
 *                 index2(x, xs, i, j)[0] = index2(x, xs, i, j)[0].conjugate()             # <<<<<<<<<<<<<<
 * 
 * cdef blas_t blas_t_sqrt(blas_t x) nogil:
 */
      (__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_x, __pyx_v_xs, __pyx_v_i, __pyx_v_j)[0]) = __Pyx_c_conj_double((__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_x, __pyx_v_xs, __pyx_v_i, __pyx_v_j)[0]));
    }
  }

  /* "scipy/linalg/_decomp_update.pyx":292
 *             index1(x, xs, j)[0] = index1(x, xs, j)[0].conjugate()
 * 
 * cdef void blas_t_2d_conj(int m, int n, blas_t* x, int* xs) nogil:             # <<<<<<<<<<<<<<
 *     cdef int i, j
 *     if blas_t is float_complex or blas_t is double_complex:
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":299
 *                 index2(x, xs, i, j)[0] = index2(x, xs, i, j)[0].conjugate()
 * 
 * cdef blas_t blas_t_sqrt(blas_t x) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         return sqrt(x)
 */

static float __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_sqrt(float __pyx_v_x) {
  float __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":301
 * cdef blas_t blas_t_sqrt(blas_t x) nogil:
 *     if blas_t is float:
 *         return sqrt(x)             # <<<<<<<<<<<<<<
 *     elif blas_t is double:
 *         return sqrt(x)
 */
  __pyx_r = sqrt(__pyx_v_x);
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":299
 *                 index2(x, xs, i, j)[0] = index2(x, xs, i, j)[0].conjugate()
 * 
 * cdef blas_t blas_t_sqrt(blas_t x) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         return sqrt(x)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static double __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_sqrt(double __pyx_v_x) {
  double __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":303
 *         return sqrt(x)
 *     elif blas_t is double:
 *         return sqrt(x)             # <<<<<<<<<<<<<<
 *     elif blas_t is float_complex:
 *         return <float_complex>sqrt(<double>((<float*>&x)[0]))
 */
  __pyx_r = sqrt(__pyx_v_x);
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":299
 *                 index2(x, xs, i, j)[0] = index2(x, xs, i, j)[0].conjugate()
 * 
 * cdef blas_t blas_t_sqrt(blas_t x) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         return sqrt(x)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static __pyx_t_float_complex __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_sqrt(__pyx_t_float_complex __pyx_v_x) {
  __pyx_t_float_complex __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":305
 *         return sqrt(x)
 *     elif blas_t is float_complex:
 *         return <float_complex>sqrt(<double>((<float*>&x)[0]))             # <<<<<<<<<<<<<<
 *     else:
 *         return sqrt((<double*>&x)[0])
 */
  __pyx_r = __pyx_t_float_complex_from_parts(((float)sqrt(((double)(((float *)(&__pyx_v_x))[0])))), 0);
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":299
 *                 index2(x, xs, i, j)[0] = index2(x, xs, i, j)[0].conjugate()
 * 
 * cdef blas_t blas_t_sqrt(blas_t x) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         return sqrt(x)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static __pyx_t_double_complex __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_sqrt(__pyx_t_double_complex __pyx_v_x) {
  __pyx_t_double_complex __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":307
 *         return <float_complex>sqrt(<double>((<float*>&x)[0]))
 *     else:
 *         return sqrt((<double*>&x)[0])             # <<<<<<<<<<<<<<
 * 
 * cdef bint blas_t_less_than(blas_t x, blas_t y) nogil:
 */
  __pyx_r = __pyx_t_double_complex_from_parts(sqrt((((double *)(&__pyx_v_x))[0])), 0);
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":299
 *                 index2(x, xs, i, j)[0] = index2(x, xs, i, j)[0].conjugate()
 * 
 * cdef blas_t blas_t_sqrt(blas_t x) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float:
 *         return sqrt(x)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":309
 *         return sqrt((<double*>&x)[0])
 * 
 * cdef bint blas_t_less_than(blas_t x, blas_t y) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float or blas_t is double:
 *         return x < y
 */

static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(float __pyx_v_x, float __pyx_v_y) {
  int __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":311
 * cdef bint blas_t_less_than(blas_t x, blas_t y) nogil:
 *     if blas_t is float or blas_t is double:
 *         return x < y             # <<<<<<<<<<<<<<
 *     else:
 *         return x.real < y.real
 */
  __pyx_r = (__pyx_v_x < __pyx_v_y);
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":309
 *         return sqrt((<double*>&x)[0])
 * 
 * cdef bint blas_t_less_than(blas_t x, blas_t y) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float or blas_t is double:
 *         return x < y
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(double __pyx_v_x, double __pyx_v_y) {
  int __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":311
 * cdef bint blas_t_less_than(blas_t x, blas_t y) nogil:
 *     if blas_t is float or blas_t is double:
 *         return x < y             # <<<<<<<<<<<<<<
 *     else:
 *         return x.real < y.real
 */
  __pyx_r = (__pyx_v_x < __pyx_v_y);
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":309
 *         return sqrt((<double*>&x)[0])
 * 
 * cdef bint blas_t_less_than(blas_t x, blas_t y) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float or blas_t is double:
 *         return x < y
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_t_float_complex __pyx_v_x, __pyx_t_float_complex __pyx_v_y) {
  int __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":313
 *         return x < y
 *     else:
 *         return x.real < y.real             # <<<<<<<<<<<<<<
 * 
 * cdef bint blas_t_less_equal(blas_t x, blas_t y) nogil:
 */
  __pyx_r = (__Pyx_CREAL(__pyx_v_x) < __Pyx_CREAL(__pyx_v_y));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":309
 *         return sqrt((<double*>&x)[0])
 * 
 * cdef bint blas_t_less_than(blas_t x, blas_t y) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float or blas_t is double:
 *         return x < y
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_t_double_complex __pyx_v_x, __pyx_t_double_complex __pyx_v_y) {
  int __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":313
 *         return x < y
 *     else:
 *         return x.real < y.real             # <<<<<<<<<<<<<<
 * 
 * cdef bint blas_t_less_equal(blas_t x, blas_t y) nogil:
 */
  __pyx_r = (__Pyx_CREAL(__pyx_v_x) < __Pyx_CREAL(__pyx_v_y));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":309
 *         return sqrt((<double*>&x)[0])
 * 
 * cdef bint blas_t_less_than(blas_t x, blas_t y) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float or blas_t is double:
 *         return x < y
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":315
 *         return x.real < y.real
 * 
 * cdef bint blas_t_less_equal(blas_t x, blas_t y) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float or blas_t is double:
 *         return x <= y
 */

static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_equal(float __pyx_v_x, float __pyx_v_y) {
  int __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":317
 * cdef bint blas_t_less_equal(blas_t x, blas_t y) nogil:
 *     if blas_t is float or blas_t is double:
 *         return x <= y             # <<<<<<<<<<<<<<
 *     else:
 *         return x.real <= y.real
 */
  __pyx_r = (__pyx_v_x <= __pyx_v_y);
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":315
 *         return x.real < y.real
 * 
 * cdef bint blas_t_less_equal(blas_t x, blas_t y) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float or blas_t is double:
 *         return x <= y
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_equal(double __pyx_v_x, double __pyx_v_y) {
  int __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":317
 * cdef bint blas_t_less_equal(blas_t x, blas_t y) nogil:
 *     if blas_t is float or blas_t is double:
 *         return x <= y             # <<<<<<<<<<<<<<
 *     else:
 *         return x.real <= y.real
 */
  __pyx_r = (__pyx_v_x <= __pyx_v_y);
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":315
 *         return x.real < y.real
 * 
 * cdef bint blas_t_less_equal(blas_t x, blas_t y) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float or blas_t is double:
 *         return x <= y
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_equal(__pyx_t_float_complex __pyx_v_x, __pyx_t_float_complex __pyx_v_y) {
  int __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":319
 *         return x <= y
 *     else:
 *         return x.real <= y.real             # <<<<<<<<<<<<<<
 * 
 * cdef int to_lwork(blas_t a, blas_t b) nogil:
 */
  __pyx_r = (__Pyx_CREAL(__pyx_v_x) <= __Pyx_CREAL(__pyx_v_y));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":315
 *         return x.real < y.real
 * 
 * cdef bint blas_t_less_equal(blas_t x, blas_t y) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float or blas_t is double:
 *         return x <= y
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_equal(__pyx_t_double_complex __pyx_v_x, __pyx_t_double_complex __pyx_v_y) {
  int __pyx_r;

  /* "scipy/linalg/_decomp_update.pyx":319
 *         return x <= y
 *     else:
 *         return x.real <= y.real             # <<<<<<<<<<<<<<
 * 
 * cdef int to_lwork(blas_t a, blas_t b) nogil:
 */
  __pyx_r = (__Pyx_CREAL(__pyx_v_x) <= __Pyx_CREAL(__pyx_v_y));
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":315
 *         return x.real < y.real
 * 
 * cdef bint blas_t_less_equal(blas_t x, blas_t y) nogil:             # <<<<<<<<<<<<<<
 *     if blas_t is float or blas_t is double:
 *         return x <= y
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":321
 *         return x.real <= y.real
 * 
 * cdef int to_lwork(blas_t a, blas_t b) nogil:             # <<<<<<<<<<<<<<
 *     cdef int ai, bi
 *     if blas_t is float or blas_t is double:
 */

static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_to_lwork(float __pyx_v_a, float __pyx_v_b) {
  int __pyx_v_ai;
  int __pyx_v_bi;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":324
 *     cdef int ai, bi
 *     if blas_t is float or blas_t is double:
 *         ai = <int>a             # <<<<<<<<<<<<<<
 *         bi = <int>b
 *     elif blas_t is float_complex:
 */
  __pyx_v_ai = ((int)__pyx_v_a);

  /* "scipy/linalg/_decomp_update.pyx":325
 *     if blas_t is float or blas_t is double:
 *         ai = <int>a
 *         bi = <int>b             # <<<<<<<<<<<<<<
 *     elif blas_t is float_complex:
 *         ai = <int>((<float*>&a)[0])
 */
  __pyx_v_bi = ((int)__pyx_v_b);

  /* "scipy/linalg/_decomp_update.pyx":332
 *         ai = <int>((<double*>&a)[0])
 *         bi = <int>((<double*>&b)[0])
 *     return max(ai, bi)             # <<<<<<<<<<<<<<
 * 
 * #------------------------------------------------------------------------------
 */
  __pyx_t_1 = __pyx_v_bi;
  __pyx_t_2 = __pyx_v_ai;
  if (((__pyx_t_1 > __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_r = __pyx_t_3;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":321
 *         return x.real <= y.real
 * 
 * cdef int to_lwork(blas_t a, blas_t b) nogil:             # <<<<<<<<<<<<<<
 *     cdef int ai, bi
 *     if blas_t is float or blas_t is double:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_to_lwork(double __pyx_v_a, double __pyx_v_b) {
  int __pyx_v_ai;
  int __pyx_v_bi;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":324
 *     cdef int ai, bi
 *     if blas_t is float or blas_t is double:
 *         ai = <int>a             # <<<<<<<<<<<<<<
 *         bi = <int>b
 *     elif blas_t is float_complex:
 */
  __pyx_v_ai = ((int)__pyx_v_a);

  /* "scipy/linalg/_decomp_update.pyx":325
 *     if blas_t is float or blas_t is double:
 *         ai = <int>a
 *         bi = <int>b             # <<<<<<<<<<<<<<
 *     elif blas_t is float_complex:
 *         ai = <int>((<float*>&a)[0])
 */
  __pyx_v_bi = ((int)__pyx_v_b);

  /* "scipy/linalg/_decomp_update.pyx":332
 *         ai = <int>((<double*>&a)[0])
 *         bi = <int>((<double*>&b)[0])
 *     return max(ai, bi)             # <<<<<<<<<<<<<<
 * 
 * #------------------------------------------------------------------------------
 */
  __pyx_t_1 = __pyx_v_bi;
  __pyx_t_2 = __pyx_v_ai;
  if (((__pyx_t_1 > __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_r = __pyx_t_3;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":321
 *         return x.real <= y.real
 * 
 * cdef int to_lwork(blas_t a, blas_t b) nogil:             # <<<<<<<<<<<<<<
 *     cdef int ai, bi
 *     if blas_t is float or blas_t is double:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_to_lwork(__pyx_t_float_complex __pyx_v_a, __pyx_t_float_complex __pyx_v_b) {
  int __pyx_v_ai;
  int __pyx_v_bi;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":327
 *         bi = <int>b
 *     elif blas_t is float_complex:
 *         ai = <int>((<float*>&a)[0])             # <<<<<<<<<<<<<<
 *         bi = <int>((<float*>&b)[0])
 *     elif blas_t is double_complex:
 */
  __pyx_v_ai = ((int)(((float *)(&__pyx_v_a))[0]));

  /* "scipy/linalg/_decomp_update.pyx":328
 *     elif blas_t is float_complex:
 *         ai = <int>((<float*>&a)[0])
 *         bi = <int>((<float*>&b)[0])             # <<<<<<<<<<<<<<
 *     elif blas_t is double_complex:
 *         ai = <int>((<double*>&a)[0])
 */
  __pyx_v_bi = ((int)(((float *)(&__pyx_v_b))[0]));

  /* "scipy/linalg/_decomp_update.pyx":332
 *         ai = <int>((<double*>&a)[0])
 *         bi = <int>((<double*>&b)[0])
 *     return max(ai, bi)             # <<<<<<<<<<<<<<
 * 
 * #------------------------------------------------------------------------------
 */
  __pyx_t_1 = __pyx_v_bi;
  __pyx_t_2 = __pyx_v_ai;
  if (((__pyx_t_1 > __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_r = __pyx_t_3;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":321
 *         return x.real <= y.real
 * 
 * cdef int to_lwork(blas_t a, blas_t b) nogil:             # <<<<<<<<<<<<<<
 *     cdef int ai, bi
 *     if blas_t is float or blas_t is double:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_to_lwork(__pyx_t_double_complex __pyx_v_a, __pyx_t_double_complex __pyx_v_b) {
  int __pyx_v_ai;
  int __pyx_v_bi;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":330
 *         bi = <int>((<float*>&b)[0])
 *     elif blas_t is double_complex:
 *         ai = <int>((<double*>&a)[0])             # <<<<<<<<<<<<<<
 *         bi = <int>((<double*>&b)[0])
 *     return max(ai, bi)
 */
  __pyx_v_ai = ((int)(((double *)(&__pyx_v_a))[0]));

  /* "scipy/linalg/_decomp_update.pyx":331
 *     elif blas_t is double_complex:
 *         ai = <int>((<double*>&a)[0])
 *         bi = <int>((<double*>&b)[0])             # <<<<<<<<<<<<<<
 *     return max(ai, bi)
 * 
 */
  __pyx_v_bi = ((int)(((double *)(&__pyx_v_b))[0]));

  /* "scipy/linalg/_decomp_update.pyx":332
 *         ai = <int>((<double*>&a)[0])
 *         bi = <int>((<double*>&b)[0])
 *     return max(ai, bi)             # <<<<<<<<<<<<<<
 * 
 * #------------------------------------------------------------------------------
 */
  __pyx_t_1 = __pyx_v_bi;
  __pyx_t_2 = __pyx_v_ai;
  if (((__pyx_t_1 > __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_r = __pyx_t_3;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":321
 *         return x.real <= y.real
 * 
 * cdef int to_lwork(blas_t a, blas_t b) nogil:             # <<<<<<<<<<<<<<
 *     cdef int ai, bi
 *     if blas_t is float or blas_t is double:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":338
 * #------------------------------------------------------------------------------
 * 
 * cdef bint reorthx(int m, int n, blas_t* q, int* qs, bint qisF, int j, blas_t* u, blas_t* s) nogil:             # <<<<<<<<<<<<<<
 *     # U should be all zeros on entry., and m > 1
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 */

static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_reorthx(int __pyx_v_m, int __pyx_v_n, float *__pyx_v_q, int *__pyx_v_qs, int __pyx_v_qisF, int __pyx_v_j, float *__pyx_v_u, float *__pyx_v_s) {
  float __pyx_v_wnorm;
  float __pyx_v_wpnorm;
  char *__pyx_v_T;
  char *__pyx_v_N;
  CYTHON_UNUSED char *__pyx_v_C;
  int __pyx_v_ss;
  float __pyx_v_inv_root2;
  int __pyx_r;
  int __pyx_t_1;

  /* "scipy/linalg/_decomp_update.pyx":341
 *     # U should be all zeros on entry., and m > 1
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 *     cdef char* T = 'T'             # <<<<<<<<<<<<<<
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'
 */
  __pyx_v_T = ((char *)"T");

  /* "scipy/linalg/_decomp_update.pyx":342
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'             # <<<<<<<<<<<<<<
 *     cdef char* C = 'C'
 *     cdef int ss = 1
 */
  __pyx_v_N = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":343
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'             # <<<<<<<<<<<<<<
 *     cdef int ss = 1
 *     cdef blas_t inv_root2 = NPY_SQRT1_2
 */
  __pyx_v_C = ((char *)"C");

  /* "scipy/linalg/_decomp_update.pyx":344
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'
 *     cdef int ss = 1             # <<<<<<<<<<<<<<
 *     cdef blas_t inv_root2 = NPY_SQRT1_2
 * 
 */
  __pyx_v_ss = 1;

  /* "scipy/linalg/_decomp_update.pyx":345
 *     cdef char* C = 'C'
 *     cdef int ss = 1
 *     cdef blas_t inv_root2 = NPY_SQRT1_2             # <<<<<<<<<<<<<<
 * 
 *     # u starts out as the jth basis vector.
 */
  __pyx_v_inv_root2 = NPY_SQRT1_2;

  /* "scipy/linalg/_decomp_update.pyx":348
 * 
 *     # u starts out as the jth basis vector.
 *     u[j] = 1             # <<<<<<<<<<<<<<
 * 
 *     # s = Q.T.dot(u) = jth row of Q.
 */
  (__pyx_v_u[__pyx_v_j]) = 1.0;

  /* "scipy/linalg/_decomp_update.pyx":351
 * 
 *     # s = Q.T.dot(u) = jth row of Q.
 *     copy(n, row(q, qs, j), qs[1], s, 1)             # <<<<<<<<<<<<<<
 *     blas_t_conj(n, s, &ss)
 * 
 */
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_n, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[1]), __pyx_v_s, 1);

  /* "scipy/linalg/_decomp_update.pyx":352
 *     # s = Q.T.dot(u) = jth row of Q.
 *     copy(n, row(q, qs, j), qs[1], s, 1)
 *     blas_t_conj(n, s, &ss)             # <<<<<<<<<<<<<<
 * 
 *     # make u be the part of u that is not in span(q)
 */
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_n, __pyx_v_s, (&__pyx_v_ss));

  /* "scipy/linalg/_decomp_update.pyx":356
 *     # make u be the part of u that is not in span(q)
 *     # i.e. u -= q.dot(s)
 *     if qisF:             # <<<<<<<<<<<<<<
 *         gemv(N, m, n, -1, q, qs[1], s, 1, 1, u, 1)
 *     else:
 */
  __pyx_t_1 = (__pyx_v_qisF != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":357
 *     # i.e. u -= q.dot(s)
 *     if qisF:
 *         gemv(N, m, n, -1, q, qs[1], s, 1, 1, u, 1)             # <<<<<<<<<<<<<<
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, 1)
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_n, -1.0, __pyx_v_q, (__pyx_v_qs[1]), __pyx_v_s, 1, 1.0, __pyx_v_u, 1);

    /* "scipy/linalg/_decomp_update.pyx":356
 *     # make u be the part of u that is not in span(q)
 *     # i.e. u -= q.dot(s)
 *     if qisF:             # <<<<<<<<<<<<<<
 *         gemv(N, m, n, -1, q, qs[1], s, 1, 1, u, 1)
 *     else:
 */
    goto __pyx_L3;
  }

  /* "scipy/linalg/_decomp_update.pyx":359
 *         gemv(N, m, n, -1, q, qs[1], s, 1, 1, u, 1)
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, 1)             # <<<<<<<<<<<<<<
 *     wnorm = nrm2(m, u, 1)
 * 
 */
  /*else*/ {
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_n, __pyx_v_m, -1.0, __pyx_v_q, __pyx_v_n, __pyx_v_s, 1, 1.0, __pyx_v_u, 1);
  }
  __pyx_L3:;

  /* "scipy/linalg/_decomp_update.pyx":360
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, 1)
 *     wnorm = nrm2(m, u, 1)             # <<<<<<<<<<<<<<
 * 
 *     if blas_t_less_than(inv_root2, wnorm):
 */
  __pyx_v_wnorm = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_m, __pyx_v_u, 1);

  /* "scipy/linalg/_decomp_update.pyx":362
 *     wnorm = nrm2(m, u, 1)
 * 
 *     if blas_t_less_than(inv_root2, wnorm):             # <<<<<<<<<<<<<<
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, 1)
 */
  __pyx_t_1 = (__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_v_inv_root2, __pyx_v_wnorm) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":364
 *     if blas_t_less_than(inv_root2, wnorm):
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, 1)             # <<<<<<<<<<<<<<
 *         s[n] = wnorm
 *         return True
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, (1.0 / __pyx_v_wnorm), __pyx_v_u, 1);

    /* "scipy/linalg/_decomp_update.pyx":365
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, 1)
 *         s[n] = wnorm             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
    (__pyx_v_s[__pyx_v_n]) = __pyx_v_wnorm;

    /* "scipy/linalg/_decomp_update.pyx":366
 *             scal(m, 1/wnorm, u, 1)
 *         s[n] = wnorm
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     # if the above check failed, try one reorthogonalization
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":362
 *     wnorm = nrm2(m, u, 1)
 * 
 *     if blas_t_less_than(inv_root2, wnorm):             # <<<<<<<<<<<<<<
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, 1)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":369
 * 
 *     # if the above check failed, try one reorthogonalization
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, qs[1], u, 1, 0, s+n, 1)
 */
  __pyx_t_1 = (__pyx_v_qisF != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":371
 *     if qisF:
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, qs[1], u, 1, 0, s+n, 1)             # <<<<<<<<<<<<<<
 *         else:
 *             gemv(C, m, n, 1, q, qs[1], u, 1, 0, s+n, 1)
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_m, __pyx_v_n, 1.0, __pyx_v_q, (__pyx_v_qs[1]), __pyx_v_u, 1, 0.0, (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":374
 *         else:
 *             gemv(C, m, n, 1, q, qs[1], u, 1, 0, s+n, 1)
 *         gemv(N, m, n, -1, q, qs[1], s+n, 1, 1, u, 1)             # <<<<<<<<<<<<<<
 *     else:
 *         if blas_t is float or blas_t is double:
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_n, -1.0, __pyx_v_q, (__pyx_v_qs[1]), (__pyx_v_s + __pyx_v_n), 1, 1.0, __pyx_v_u, 1);

    /* "scipy/linalg/_decomp_update.pyx":369
 * 
 *     # if the above check failed, try one reorthogonalization
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, qs[1], u, 1, 0, s+n, 1)
 */
    goto __pyx_L5;
  }

  /* "scipy/linalg/_decomp_update.pyx":376
 *         gemv(N, m, n, -1, q, qs[1], s+n, 1, 1, u, 1)
 *     else:
 *         if blas_t is float or blas_t is double:             # <<<<<<<<<<<<<<
 *             gemv(N, n, m, 1, q, n, u, 1, 0, s+n, 1)
 *         else:
 */
  /*else*/ {

    /* "scipy/linalg/_decomp_update.pyx":377
 *     else:
 *         if blas_t is float or blas_t is double:
 *             gemv(N, n, m, 1, q, n, u, 1, 0, s+n, 1)             # <<<<<<<<<<<<<<
 *         else:
 *             blas_t_conj(m, u, &ss)
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_n, __pyx_v_m, 1.0, __pyx_v_q, __pyx_v_n, __pyx_v_u, 1, 0.0, (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":383
 *             blas_t_conj(m, u, &ss)
 *             blas_t_conj(n, s+n, &ss)
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, 1)             # <<<<<<<<<<<<<<
 *     wpnorm = nrm2(m, u, 1)
 * 
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_n, __pyx_v_m, -1.0, __pyx_v_q, __pyx_v_n, (__pyx_v_s + __pyx_v_n), 1, 1.0, __pyx_v_u, 1);
  }
  __pyx_L5:;

  /* "scipy/linalg/_decomp_update.pyx":384
 *             blas_t_conj(n, s+n, &ss)
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, 1)
 *     wpnorm = nrm2(m, u, 1)             # <<<<<<<<<<<<<<
 * 
 *     if blas_t_less_than(wpnorm, wnorm*inv_root2): # u lies in span(q)
 */
  __pyx_v_wpnorm = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_m, __pyx_v_u, 1);

  /* "scipy/linalg/_decomp_update.pyx":386
 *     wpnorm = nrm2(m, u, 1)
 * 
 *     if blas_t_less_than(wpnorm, wnorm*inv_root2): # u lies in span(q)             # <<<<<<<<<<<<<<
 *         scal(m, 0, u, 1)
 *         axpy(n, 1, s, 1, s+n, 1)
 */
  __pyx_t_1 = (__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_v_wpnorm, (__pyx_v_wnorm * __pyx_v_inv_root2)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":387
 * 
 *     if blas_t_less_than(wpnorm, wnorm*inv_root2): # u lies in span(q)
 *         scal(m, 0, u, 1)             # <<<<<<<<<<<<<<
 *         axpy(n, 1, s, 1, s+n, 1)
 *         s[n] = 0
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, 0.0, __pyx_v_u, 1);

    /* "scipy/linalg/_decomp_update.pyx":388
 *     if blas_t_less_than(wpnorm, wnorm*inv_root2): # u lies in span(q)
 *         scal(m, 0, u, 1)
 *         axpy(n, 1, s, 1, s+n, 1)             # <<<<<<<<<<<<<<
 *         s[n] = 0
 *         return False
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, 1.0, __pyx_v_s, 1, (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":389
 *         scal(m, 0, u, 1)
 *         axpy(n, 1, s, 1, s+n, 1)
 *         s[n] = 0             # <<<<<<<<<<<<<<
 *         return False
 * 
 */
    (__pyx_v_s[__pyx_v_n]) = 0.0;

    /* "scipy/linalg/_decomp_update.pyx":390
 *         axpy(n, 1, s, 1, s+n, 1)
 *         s[n] = 0
 *         return False             # <<<<<<<<<<<<<<
 * 
 *     scal(m, 1/wpnorm, u, 1)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":386
 *     wpnorm = nrm2(m, u, 1)
 * 
 *     if blas_t_less_than(wpnorm, wnorm*inv_root2): # u lies in span(q)             # <<<<<<<<<<<<<<
 *         scal(m, 0, u, 1)
 *         axpy(n, 1, s, 1, s+n, 1)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":392
 *         return False
 * 
 *     scal(m, 1/wpnorm, u, 1)             # <<<<<<<<<<<<<<
 *     axpy(n, 1, s, 1, s+n, 1)
 *     s[n] = wpnorm
 */
  if (unlikely(__pyx_v_wpnorm == 0)) {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
    #endif
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    #ifdef WITH_THREAD
    __Pyx_PyGILState_Release(__pyx_gilstate_save);
    #endif
    __PYX_ERR(0, 392, __pyx_L1_error)
  }
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, (1.0 / __pyx_v_wpnorm), __pyx_v_u, 1);

  /* "scipy/linalg/_decomp_update.pyx":393
 * 
 *     scal(m, 1/wpnorm, u, 1)
 *     axpy(n, 1, s, 1, s+n, 1)             # <<<<<<<<<<<<<<
 *     s[n] = wpnorm
 *     return True
 */
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, 1.0, __pyx_v_s, 1, (__pyx_v_s + __pyx_v_n), 1);

  /* "scipy/linalg/_decomp_update.pyx":394
 *     scal(m, 1/wpnorm, u, 1)
 *     axpy(n, 1, s, 1, s+n, 1)
 *     s[n] = wpnorm             # <<<<<<<<<<<<<<
 *     return True
 * 
 */
  (__pyx_v_s[__pyx_v_n]) = __pyx_v_wpnorm;

  /* "scipy/linalg/_decomp_update.pyx":395
 *     axpy(n, 1, s, 1, s+n, 1)
 *     s[n] = wpnorm
 *     return True             # <<<<<<<<<<<<<<
 * 
 * cdef int thin_qr_row_delete(int m, int n, blas_t* q, int* qs, bint qisF,
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":338
 * #------------------------------------------------------------------------------
 * 
 * cdef bint reorthx(int m, int n, blas_t* q, int* qs, bint qisF, int j, blas_t* u, blas_t* s) nogil:             # <<<<<<<<<<<<<<
 *     # U should be all zeros on entry., and m > 1
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.linalg._decomp_update.reorthx", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_reorthx(int __pyx_v_m, int __pyx_v_n, double *__pyx_v_q, int *__pyx_v_qs, int __pyx_v_qisF, int __pyx_v_j, double *__pyx_v_u, double *__pyx_v_s) {
  double __pyx_v_wnorm;
  double __pyx_v_wpnorm;
  char *__pyx_v_T;
  char *__pyx_v_N;
  CYTHON_UNUSED char *__pyx_v_C;
  int __pyx_v_ss;
  double __pyx_v_inv_root2;
  int __pyx_r;
  int __pyx_t_1;

  /* "scipy/linalg/_decomp_update.pyx":341
 *     # U should be all zeros on entry., and m > 1
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 *     cdef char* T = 'T'             # <<<<<<<<<<<<<<
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'
 */
  __pyx_v_T = ((char *)"T");

  /* "scipy/linalg/_decomp_update.pyx":342
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'             # <<<<<<<<<<<<<<
 *     cdef char* C = 'C'
 *     cdef int ss = 1
 */
  __pyx_v_N = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":343
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'             # <<<<<<<<<<<<<<
 *     cdef int ss = 1
 *     cdef blas_t inv_root2 = NPY_SQRT1_2
 */
  __pyx_v_C = ((char *)"C");

  /* "scipy/linalg/_decomp_update.pyx":344
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'
 *     cdef int ss = 1             # <<<<<<<<<<<<<<
 *     cdef blas_t inv_root2 = NPY_SQRT1_2
 * 
 */
  __pyx_v_ss = 1;

  /* "scipy/linalg/_decomp_update.pyx":345
 *     cdef char* C = 'C'
 *     cdef int ss = 1
 *     cdef blas_t inv_root2 = NPY_SQRT1_2             # <<<<<<<<<<<<<<
 * 
 *     # u starts out as the jth basis vector.
 */
  __pyx_v_inv_root2 = NPY_SQRT1_2;

  /* "scipy/linalg/_decomp_update.pyx":348
 * 
 *     # u starts out as the jth basis vector.
 *     u[j] = 1             # <<<<<<<<<<<<<<
 * 
 *     # s = Q.T.dot(u) = jth row of Q.
 */
  (__pyx_v_u[__pyx_v_j]) = 1.0;

  /* "scipy/linalg/_decomp_update.pyx":351
 * 
 *     # s = Q.T.dot(u) = jth row of Q.
 *     copy(n, row(q, qs, j), qs[1], s, 1)             # <<<<<<<<<<<<<<
 *     blas_t_conj(n, s, &ss)
 * 
 */
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_n, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[1]), __pyx_v_s, 1);

  /* "scipy/linalg/_decomp_update.pyx":352
 *     # s = Q.T.dot(u) = jth row of Q.
 *     copy(n, row(q, qs, j), qs[1], s, 1)
 *     blas_t_conj(n, s, &ss)             # <<<<<<<<<<<<<<
 * 
 *     # make u be the part of u that is not in span(q)
 */
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_n, __pyx_v_s, (&__pyx_v_ss));

  /* "scipy/linalg/_decomp_update.pyx":356
 *     # make u be the part of u that is not in span(q)
 *     # i.e. u -= q.dot(s)
 *     if qisF:             # <<<<<<<<<<<<<<
 *         gemv(N, m, n, -1, q, qs[1], s, 1, 1, u, 1)
 *     else:
 */
  __pyx_t_1 = (__pyx_v_qisF != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":357
 *     # i.e. u -= q.dot(s)
 *     if qisF:
 *         gemv(N, m, n, -1, q, qs[1], s, 1, 1, u, 1)             # <<<<<<<<<<<<<<
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, 1)
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_n, -1.0, __pyx_v_q, (__pyx_v_qs[1]), __pyx_v_s, 1, 1.0, __pyx_v_u, 1);

    /* "scipy/linalg/_decomp_update.pyx":356
 *     # make u be the part of u that is not in span(q)
 *     # i.e. u -= q.dot(s)
 *     if qisF:             # <<<<<<<<<<<<<<
 *         gemv(N, m, n, -1, q, qs[1], s, 1, 1, u, 1)
 *     else:
 */
    goto __pyx_L3;
  }

  /* "scipy/linalg/_decomp_update.pyx":359
 *         gemv(N, m, n, -1, q, qs[1], s, 1, 1, u, 1)
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, 1)             # <<<<<<<<<<<<<<
 *     wnorm = nrm2(m, u, 1)
 * 
 */
  /*else*/ {
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_n, __pyx_v_m, -1.0, __pyx_v_q, __pyx_v_n, __pyx_v_s, 1, 1.0, __pyx_v_u, 1);
  }
  __pyx_L3:;

  /* "scipy/linalg/_decomp_update.pyx":360
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, 1)
 *     wnorm = nrm2(m, u, 1)             # <<<<<<<<<<<<<<
 * 
 *     if blas_t_less_than(inv_root2, wnorm):
 */
  __pyx_v_wnorm = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_m, __pyx_v_u, 1);

  /* "scipy/linalg/_decomp_update.pyx":362
 *     wnorm = nrm2(m, u, 1)
 * 
 *     if blas_t_less_than(inv_root2, wnorm):             # <<<<<<<<<<<<<<
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, 1)
 */
  __pyx_t_1 = (__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_v_inv_root2, __pyx_v_wnorm) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":364
 *     if blas_t_less_than(inv_root2, wnorm):
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, 1)             # <<<<<<<<<<<<<<
 *         s[n] = wnorm
 *         return True
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, (1.0 / __pyx_v_wnorm), __pyx_v_u, 1);

    /* "scipy/linalg/_decomp_update.pyx":365
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, 1)
 *         s[n] = wnorm             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
    (__pyx_v_s[__pyx_v_n]) = __pyx_v_wnorm;

    /* "scipy/linalg/_decomp_update.pyx":366
 *             scal(m, 1/wnorm, u, 1)
 *         s[n] = wnorm
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     # if the above check failed, try one reorthogonalization
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":362
 *     wnorm = nrm2(m, u, 1)
 * 
 *     if blas_t_less_than(inv_root2, wnorm):             # <<<<<<<<<<<<<<
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, 1)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":369
 * 
 *     # if the above check failed, try one reorthogonalization
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, qs[1], u, 1, 0, s+n, 1)
 */
  __pyx_t_1 = (__pyx_v_qisF != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":371
 *     if qisF:
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, qs[1], u, 1, 0, s+n, 1)             # <<<<<<<<<<<<<<
 *         else:
 *             gemv(C, m, n, 1, q, qs[1], u, 1, 0, s+n, 1)
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_m, __pyx_v_n, 1.0, __pyx_v_q, (__pyx_v_qs[1]), __pyx_v_u, 1, 0.0, (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":374
 *         else:
 *             gemv(C, m, n, 1, q, qs[1], u, 1, 0, s+n, 1)
 *         gemv(N, m, n, -1, q, qs[1], s+n, 1, 1, u, 1)             # <<<<<<<<<<<<<<
 *     else:
 *         if blas_t is float or blas_t is double:
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_n, -1.0, __pyx_v_q, (__pyx_v_qs[1]), (__pyx_v_s + __pyx_v_n), 1, 1.0, __pyx_v_u, 1);

    /* "scipy/linalg/_decomp_update.pyx":369
 * 
 *     # if the above check failed, try one reorthogonalization
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, qs[1], u, 1, 0, s+n, 1)
 */
    goto __pyx_L5;
  }

  /* "scipy/linalg/_decomp_update.pyx":376
 *         gemv(N, m, n, -1, q, qs[1], s+n, 1, 1, u, 1)
 *     else:
 *         if blas_t is float or blas_t is double:             # <<<<<<<<<<<<<<
 *             gemv(N, n, m, 1, q, n, u, 1, 0, s+n, 1)
 *         else:
 */
  /*else*/ {

    /* "scipy/linalg/_decomp_update.pyx":377
 *     else:
 *         if blas_t is float or blas_t is double:
 *             gemv(N, n, m, 1, q, n, u, 1, 0, s+n, 1)             # <<<<<<<<<<<<<<
 *         else:
 *             blas_t_conj(m, u, &ss)
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_n, __pyx_v_m, 1.0, __pyx_v_q, __pyx_v_n, __pyx_v_u, 1, 0.0, (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":383
 *             blas_t_conj(m, u, &ss)
 *             blas_t_conj(n, s+n, &ss)
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, 1)             # <<<<<<<<<<<<<<
 *     wpnorm = nrm2(m, u, 1)
 * 
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_n, __pyx_v_m, -1.0, __pyx_v_q, __pyx_v_n, (__pyx_v_s + __pyx_v_n), 1, 1.0, __pyx_v_u, 1);
  }
  __pyx_L5:;

  /* "scipy/linalg/_decomp_update.pyx":384
 *             blas_t_conj(n, s+n, &ss)
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, 1)
 *     wpnorm = nrm2(m, u, 1)             # <<<<<<<<<<<<<<
 * 
 *     if blas_t_less_than(wpnorm, wnorm*inv_root2): # u lies in span(q)
 */
  __pyx_v_wpnorm = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_m, __pyx_v_u, 1);

  /* "scipy/linalg/_decomp_update.pyx":386
 *     wpnorm = nrm2(m, u, 1)
 * 
 *     if blas_t_less_than(wpnorm, wnorm*inv_root2): # u lies in span(q)             # <<<<<<<<<<<<<<
 *         scal(m, 0, u, 1)
 *         axpy(n, 1, s, 1, s+n, 1)
 */
  __pyx_t_1 = (__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_v_wpnorm, (__pyx_v_wnorm * __pyx_v_inv_root2)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":387
 * 
 *     if blas_t_less_than(wpnorm, wnorm*inv_root2): # u lies in span(q)
 *         scal(m, 0, u, 1)             # <<<<<<<<<<<<<<
 *         axpy(n, 1, s, 1, s+n, 1)
 *         s[n] = 0
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, 0.0, __pyx_v_u, 1);

    /* "scipy/linalg/_decomp_update.pyx":388
 *     if blas_t_less_than(wpnorm, wnorm*inv_root2): # u lies in span(q)
 *         scal(m, 0, u, 1)
 *         axpy(n, 1, s, 1, s+n, 1)             # <<<<<<<<<<<<<<
 *         s[n] = 0
 *         return False
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, 1.0, __pyx_v_s, 1, (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":389
 *         scal(m, 0, u, 1)
 *         axpy(n, 1, s, 1, s+n, 1)
 *         s[n] = 0             # <<<<<<<<<<<<<<
 *         return False
 * 
 */
    (__pyx_v_s[__pyx_v_n]) = 0.0;

    /* "scipy/linalg/_decomp_update.pyx":390
 *         axpy(n, 1, s, 1, s+n, 1)
 *         s[n] = 0
 *         return False             # <<<<<<<<<<<<<<
 * 
 *     scal(m, 1/wpnorm, u, 1)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":386
 *     wpnorm = nrm2(m, u, 1)
 * 
 *     if blas_t_less_than(wpnorm, wnorm*inv_root2): # u lies in span(q)             # <<<<<<<<<<<<<<
 *         scal(m, 0, u, 1)
 *         axpy(n, 1, s, 1, s+n, 1)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":392
 *         return False
 * 
 *     scal(m, 1/wpnorm, u, 1)             # <<<<<<<<<<<<<<
 *     axpy(n, 1, s, 1, s+n, 1)
 *     s[n] = wpnorm
 */
  if (unlikely(__pyx_v_wpnorm == 0)) {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
    #endif
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    #ifdef WITH_THREAD
    __Pyx_PyGILState_Release(__pyx_gilstate_save);
    #endif
    __PYX_ERR(0, 392, __pyx_L1_error)
  }
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, (1.0 / __pyx_v_wpnorm), __pyx_v_u, 1);

  /* "scipy/linalg/_decomp_update.pyx":393
 * 
 *     scal(m, 1/wpnorm, u, 1)
 *     axpy(n, 1, s, 1, s+n, 1)             # <<<<<<<<<<<<<<
 *     s[n] = wpnorm
 *     return True
 */
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, 1.0, __pyx_v_s, 1, (__pyx_v_s + __pyx_v_n), 1);

  /* "scipy/linalg/_decomp_update.pyx":394
 *     scal(m, 1/wpnorm, u, 1)
 *     axpy(n, 1, s, 1, s+n, 1)
 *     s[n] = wpnorm             # <<<<<<<<<<<<<<
 *     return True
 * 
 */
  (__pyx_v_s[__pyx_v_n]) = __pyx_v_wpnorm;

  /* "scipy/linalg/_decomp_update.pyx":395
 *     axpy(n, 1, s, 1, s+n, 1)
 *     s[n] = wpnorm
 *     return True             # <<<<<<<<<<<<<<
 * 
 * cdef int thin_qr_row_delete(int m, int n, blas_t* q, int* qs, bint qisF,
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":338
 * #------------------------------------------------------------------------------
 * 
 * cdef bint reorthx(int m, int n, blas_t* q, int* qs, bint qisF, int j, blas_t* u, blas_t* s) nogil:             # <<<<<<<<<<<<<<
 *     # U should be all zeros on entry., and m > 1
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.linalg._decomp_update.reorthx", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_reorthx(int __pyx_v_m, int __pyx_v_n, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_qs, int __pyx_v_qisF, int __pyx_v_j, __pyx_t_float_complex *__pyx_v_u, __pyx_t_float_complex *__pyx_v_s) {
  __pyx_t_float_complex __pyx_v_wnorm;
  __pyx_t_float_complex __pyx_v_wpnorm;
  char *__pyx_v_T;
  char *__pyx_v_N;
  char *__pyx_v_C;
  int __pyx_v_ss;
  __pyx_t_float_complex __pyx_v_inv_root2;
  int __pyx_r;
  int __pyx_t_1;
  __pyx_t_double_complex __pyx_t_2;
  __pyx_t_double_complex __pyx_t_3;
  __pyx_t_double_complex __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":341
 *     # U should be all zeros on entry., and m > 1
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 *     cdef char* T = 'T'             # <<<<<<<<<<<<<<
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'
 */
  __pyx_v_T = ((char *)"T");

  /* "scipy/linalg/_decomp_update.pyx":342
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'             # <<<<<<<<<<<<<<
 *     cdef char* C = 'C'
 *     cdef int ss = 1
 */
  __pyx_v_N = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":343
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'             # <<<<<<<<<<<<<<
 *     cdef int ss = 1
 *     cdef blas_t inv_root2 = NPY_SQRT1_2
 */
  __pyx_v_C = ((char *)"C");

  /* "scipy/linalg/_decomp_update.pyx":344
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'
 *     cdef int ss = 1             # <<<<<<<<<<<<<<
 *     cdef blas_t inv_root2 = NPY_SQRT1_2
 * 
 */
  __pyx_v_ss = 1;

  /* "scipy/linalg/_decomp_update.pyx":345
 *     cdef char* C = 'C'
 *     cdef int ss = 1
 *     cdef blas_t inv_root2 = NPY_SQRT1_2             # <<<<<<<<<<<<<<
 * 
 *     # u starts out as the jth basis vector.
 */
  __pyx_v_inv_root2 = __pyx_t_float_complex_from_parts(NPY_SQRT1_2, 0);

  /* "scipy/linalg/_decomp_update.pyx":348
 * 
 *     # u starts out as the jth basis vector.
 *     u[j] = 1             # <<<<<<<<<<<<<<
 * 
 *     # s = Q.T.dot(u) = jth row of Q.
 */
  (__pyx_v_u[__pyx_v_j]) = __pyx_t_float_complex_from_parts(1, 0);

  /* "scipy/linalg/_decomp_update.pyx":351
 * 
 *     # s = Q.T.dot(u) = jth row of Q.
 *     copy(n, row(q, qs, j), qs[1], s, 1)             # <<<<<<<<<<<<<<
 *     blas_t_conj(n, s, &ss)
 * 
 */
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_n, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[1]), __pyx_v_s, 1);

  /* "scipy/linalg/_decomp_update.pyx":352
 *     # s = Q.T.dot(u) = jth row of Q.
 *     copy(n, row(q, qs, j), qs[1], s, 1)
 *     blas_t_conj(n, s, &ss)             # <<<<<<<<<<<<<<
 * 
 *     # make u be the part of u that is not in span(q)
 */
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_n, __pyx_v_s, (&__pyx_v_ss));

  /* "scipy/linalg/_decomp_update.pyx":356
 *     # make u be the part of u that is not in span(q)
 *     # i.e. u -= q.dot(s)
 *     if qisF:             # <<<<<<<<<<<<<<
 *         gemv(N, m, n, -1, q, qs[1], s, 1, 1, u, 1)
 *     else:
 */
  __pyx_t_1 = (__pyx_v_qisF != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":357
 *     # i.e. u -= q.dot(s)
 *     if qisF:
 *         gemv(N, m, n, -1, q, qs[1], s, 1, 1, u, 1)             # <<<<<<<<<<<<<<
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, 1)
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_n, __pyx_t_float_complex_from_parts(-1L, 0), __pyx_v_q, (__pyx_v_qs[1]), __pyx_v_s, 1, __pyx_t_float_complex_from_parts(1, 0), __pyx_v_u, 1);

    /* "scipy/linalg/_decomp_update.pyx":356
 *     # make u be the part of u that is not in span(q)
 *     # i.e. u -= q.dot(s)
 *     if qisF:             # <<<<<<<<<<<<<<
 *         gemv(N, m, n, -1, q, qs[1], s, 1, 1, u, 1)
 *     else:
 */
    goto __pyx_L3;
  }

  /* "scipy/linalg/_decomp_update.pyx":359
 *         gemv(N, m, n, -1, q, qs[1], s, 1, 1, u, 1)
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, 1)             # <<<<<<<<<<<<<<
 *     wnorm = nrm2(m, u, 1)
 * 
 */
  /*else*/ {
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_n, __pyx_v_m, __pyx_t_float_complex_from_parts(-1L, 0), __pyx_v_q, __pyx_v_n, __pyx_v_s, 1, __pyx_t_float_complex_from_parts(1, 0), __pyx_v_u, 1);
  }
  __pyx_L3:;

  /* "scipy/linalg/_decomp_update.pyx":360
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, 1)
 *     wnorm = nrm2(m, u, 1)             # <<<<<<<<<<<<<<
 * 
 *     if blas_t_less_than(inv_root2, wnorm):
 */
  __pyx_v_wnorm = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_m, __pyx_v_u, 1);

  /* "scipy/linalg/_decomp_update.pyx":362
 *     wnorm = nrm2(m, u, 1)
 * 
 *     if blas_t_less_than(inv_root2, wnorm):             # <<<<<<<<<<<<<<
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, 1)
 */
  __pyx_t_1 = (__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_v_inv_root2, __pyx_v_wnorm) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":364
 *     if blas_t_less_than(inv_root2, wnorm):
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, 1)             # <<<<<<<<<<<<<<
 *         s[n] = wnorm
 *         return True
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, __Pyx_c_quot_float(__pyx_t_float_complex_from_parts(1, 0), __pyx_v_wnorm), __pyx_v_u, 1);

    /* "scipy/linalg/_decomp_update.pyx":365
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, 1)
 *         s[n] = wnorm             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
    (__pyx_v_s[__pyx_v_n]) = __pyx_v_wnorm;

    /* "scipy/linalg/_decomp_update.pyx":366
 *             scal(m, 1/wnorm, u, 1)
 *         s[n] = wnorm
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     # if the above check failed, try one reorthogonalization
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":362
 *     wnorm = nrm2(m, u, 1)
 * 
 *     if blas_t_less_than(inv_root2, wnorm):             # <<<<<<<<<<<<<<
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, 1)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":369
 * 
 *     # if the above check failed, try one reorthogonalization
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, qs[1], u, 1, 0, s+n, 1)
 */
  __pyx_t_1 = (__pyx_v_qisF != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":373
 *             gemv(T, m, n, 1, q, qs[1], u, 1, 0, s+n, 1)
 *         else:
 *             gemv(C, m, n, 1, q, qs[1], u, 1, 0, s+n, 1)             # <<<<<<<<<<<<<<
 *         gemv(N, m, n, -1, q, qs[1], s+n, 1, 1, u, 1)
 *     else:
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_C, __pyx_v_m, __pyx_v_n, __pyx_t_float_complex_from_parts(1, 0), __pyx_v_q, (__pyx_v_qs[1]), __pyx_v_u, 1, __pyx_t_float_complex_from_parts(0, 0), (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":374
 *         else:
 *             gemv(C, m, n, 1, q, qs[1], u, 1, 0, s+n, 1)
 *         gemv(N, m, n, -1, q, qs[1], s+n, 1, 1, u, 1)             # <<<<<<<<<<<<<<
 *     else:
 *         if blas_t is float or blas_t is double:
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_n, __pyx_t_float_complex_from_parts(-1L, 0), __pyx_v_q, (__pyx_v_qs[1]), (__pyx_v_s + __pyx_v_n), 1, __pyx_t_float_complex_from_parts(1, 0), __pyx_v_u, 1);

    /* "scipy/linalg/_decomp_update.pyx":369
 * 
 *     # if the above check failed, try one reorthogonalization
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, qs[1], u, 1, 0, s+n, 1)
 */
    goto __pyx_L5;
  }

  /* "scipy/linalg/_decomp_update.pyx":376
 *         gemv(N, m, n, -1, q, qs[1], s+n, 1, 1, u, 1)
 *     else:
 *         if blas_t is float or blas_t is double:             # <<<<<<<<<<<<<<
 *             gemv(N, n, m, 1, q, n, u, 1, 0, s+n, 1)
 *         else:
 */
  /*else*/ {

    /* "scipy/linalg/_decomp_update.pyx":379
 *             gemv(N, n, m, 1, q, n, u, 1, 0, s+n, 1)
 *         else:
 *             blas_t_conj(m, u, &ss)             # <<<<<<<<<<<<<<
 *             gemv(N, n, m, 1, q, n, u, 1, 0, s+n, 1)
 *             blas_t_conj(m, u, &ss)
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_m, __pyx_v_u, (&__pyx_v_ss));

    /* "scipy/linalg/_decomp_update.pyx":380
 *         else:
 *             blas_t_conj(m, u, &ss)
 *             gemv(N, n, m, 1, q, n, u, 1, 0, s+n, 1)             # <<<<<<<<<<<<<<
 *             blas_t_conj(m, u, &ss)
 *             blas_t_conj(n, s+n, &ss)
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_n, __pyx_v_m, __pyx_t_float_complex_from_parts(1, 0), __pyx_v_q, __pyx_v_n, __pyx_v_u, 1, __pyx_t_float_complex_from_parts(0, 0), (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":381
 *             blas_t_conj(m, u, &ss)
 *             gemv(N, n, m, 1, q, n, u, 1, 0, s+n, 1)
 *             blas_t_conj(m, u, &ss)             # <<<<<<<<<<<<<<
 *             blas_t_conj(n, s+n, &ss)
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, 1)
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_m, __pyx_v_u, (&__pyx_v_ss));

    /* "scipy/linalg/_decomp_update.pyx":382
 *             gemv(N, n, m, 1, q, n, u, 1, 0, s+n, 1)
 *             blas_t_conj(m, u, &ss)
 *             blas_t_conj(n, s+n, &ss)             # <<<<<<<<<<<<<<
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, 1)
 *     wpnorm = nrm2(m, u, 1)
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_n, (__pyx_v_s + __pyx_v_n), (&__pyx_v_ss));

    /* "scipy/linalg/_decomp_update.pyx":383
 *             blas_t_conj(m, u, &ss)
 *             blas_t_conj(n, s+n, &ss)
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, 1)             # <<<<<<<<<<<<<<
 *     wpnorm = nrm2(m, u, 1)
 * 
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_n, __pyx_v_m, __pyx_t_float_complex_from_parts(-1L, 0), __pyx_v_q, __pyx_v_n, (__pyx_v_s + __pyx_v_n), 1, __pyx_t_float_complex_from_parts(1, 0), __pyx_v_u, 1);
  }
  __pyx_L5:;

  /* "scipy/linalg/_decomp_update.pyx":384
 *             blas_t_conj(n, s+n, &ss)
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, 1)
 *     wpnorm = nrm2(m, u, 1)             # <<<<<<<<<<<<<<
 * 
 *     if blas_t_less_than(wpnorm, wnorm*inv_root2): # u lies in span(q)
 */
  __pyx_v_wpnorm = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_m, __pyx_v_u, 1);

  /* "scipy/linalg/_decomp_update.pyx":386
 *     wpnorm = nrm2(m, u, 1)
 * 
 *     if blas_t_less_than(wpnorm, wnorm*inv_root2): # u lies in span(q)             # <<<<<<<<<<<<<<
 *         scal(m, 0, u, 1)
 *         axpy(n, 1, s, 1, s+n, 1)
 */
  __pyx_t_1 = (__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_v_wpnorm, __Pyx_c_prod_float(__pyx_v_wnorm, __pyx_v_inv_root2)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":387
 * 
 *     if blas_t_less_than(wpnorm, wnorm*inv_root2): # u lies in span(q)
 *         scal(m, 0, u, 1)             # <<<<<<<<<<<<<<
 *         axpy(n, 1, s, 1, s+n, 1)
 *         s[n] = 0
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, __pyx_t_float_complex_from_parts(0, 0), __pyx_v_u, 1);

    /* "scipy/linalg/_decomp_update.pyx":388
 *     if blas_t_less_than(wpnorm, wnorm*inv_root2): # u lies in span(q)
 *         scal(m, 0, u, 1)
 *         axpy(n, 1, s, 1, s+n, 1)             # <<<<<<<<<<<<<<
 *         s[n] = 0
 *         return False
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, __pyx_t_float_complex_from_parts(1, 0), __pyx_v_s, 1, (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":389
 *         scal(m, 0, u, 1)
 *         axpy(n, 1, s, 1, s+n, 1)
 *         s[n] = 0             # <<<<<<<<<<<<<<
 *         return False
 * 
 */
    (__pyx_v_s[__pyx_v_n]) = __pyx_t_float_complex_from_parts(0, 0);

    /* "scipy/linalg/_decomp_update.pyx":390
 *         axpy(n, 1, s, 1, s+n, 1)
 *         s[n] = 0
 *         return False             # <<<<<<<<<<<<<<
 * 
 *     scal(m, 1/wpnorm, u, 1)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":386
 *     wpnorm = nrm2(m, u, 1)
 * 
 *     if blas_t_less_than(wpnorm, wnorm*inv_root2): # u lies in span(q)             # <<<<<<<<<<<<<<
 *         scal(m, 0, u, 1)
 *         axpy(n, 1, s, 1, s+n, 1)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":392
 *         return False
 * 
 *     scal(m, 1/wpnorm, u, 1)             # <<<<<<<<<<<<<<
 *     axpy(n, 1, s, 1, s+n, 1)
 *     s[n] = wpnorm
 */
  __pyx_t_2 = __pyx_t_double_complex_from_parts(1, 0);
  __pyx_t_3 = __pyx_t_double_complex_from_parts(__Pyx_CREAL(__pyx_v_wpnorm), __Pyx_CIMAG(__pyx_v_wpnorm));
  if (unlikely(__Pyx_c_is_zero_double(__pyx_t_3))) {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
    #endif
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    #ifdef WITH_THREAD
    __Pyx_PyGILState_Release(__pyx_gilstate_save);
    #endif
    __PYX_ERR(0, 392, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_c_quot_double(__pyx_t_2, __pyx_t_3);
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, __pyx_t_float_complex_from_parts(__Pyx_CREAL(__pyx_t_4), __Pyx_CIMAG(__pyx_t_4)), __pyx_v_u, 1);

  /* "scipy/linalg/_decomp_update.pyx":393
 * 
 *     scal(m, 1/wpnorm, u, 1)
 *     axpy(n, 1, s, 1, s+n, 1)             # <<<<<<<<<<<<<<
 *     s[n] = wpnorm
 *     return True
 */
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, __pyx_t_float_complex_from_parts(1, 0), __pyx_v_s, 1, (__pyx_v_s + __pyx_v_n), 1);

  /* "scipy/linalg/_decomp_update.pyx":394
 *     scal(m, 1/wpnorm, u, 1)
 *     axpy(n, 1, s, 1, s+n, 1)
 *     s[n] = wpnorm             # <<<<<<<<<<<<<<
 *     return True
 * 
 */
  (__pyx_v_s[__pyx_v_n]) = __pyx_v_wpnorm;

  /* "scipy/linalg/_decomp_update.pyx":395
 *     axpy(n, 1, s, 1, s+n, 1)
 *     s[n] = wpnorm
 *     return True             # <<<<<<<<<<<<<<
 * 
 * cdef int thin_qr_row_delete(int m, int n, blas_t* q, int* qs, bint qisF,
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":338
 * #------------------------------------------------------------------------------
 * 
 * cdef bint reorthx(int m, int n, blas_t* q, int* qs, bint qisF, int j, blas_t* u, blas_t* s) nogil:             # <<<<<<<<<<<<<<
 *     # U should be all zeros on entry., and m > 1
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.linalg._decomp_update.reorthx", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_reorthx(int __pyx_v_m, int __pyx_v_n, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_qs, int __pyx_v_qisF, int __pyx_v_j, __pyx_t_double_complex *__pyx_v_u, __pyx_t_double_complex *__pyx_v_s) {
  __pyx_t_double_complex __pyx_v_wnorm;
  __pyx_t_double_complex __pyx_v_wpnorm;
  char *__pyx_v_T;
  char *__pyx_v_N;
  char *__pyx_v_C;
  int __pyx_v_ss;
  __pyx_t_double_complex __pyx_v_inv_root2;
  int __pyx_r;
  int __pyx_t_1;
  __pyx_t_double_complex __pyx_t_2;

  /* "scipy/linalg/_decomp_update.pyx":341
 *     # U should be all zeros on entry., and m > 1
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 *     cdef char* T = 'T'             # <<<<<<<<<<<<<<
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'
 */
  __pyx_v_T = ((char *)"T");

  /* "scipy/linalg/_decomp_update.pyx":342
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'             # <<<<<<<<<<<<<<
 *     cdef char* C = 'C'
 *     cdef int ss = 1
 */
  __pyx_v_N = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":343
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'             # <<<<<<<<<<<<<<
 *     cdef int ss = 1
 *     cdef blas_t inv_root2 = NPY_SQRT1_2
 */
  __pyx_v_C = ((char *)"C");

  /* "scipy/linalg/_decomp_update.pyx":344
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'
 *     cdef int ss = 1             # <<<<<<<<<<<<<<
 *     cdef blas_t inv_root2 = NPY_SQRT1_2
 * 
 */
  __pyx_v_ss = 1;

  /* "scipy/linalg/_decomp_update.pyx":345
 *     cdef char* C = 'C'
 *     cdef int ss = 1
 *     cdef blas_t inv_root2 = NPY_SQRT1_2             # <<<<<<<<<<<<<<
 * 
 *     # u starts out as the jth basis vector.
 */
  __pyx_v_inv_root2 = __pyx_t_double_complex_from_parts(NPY_SQRT1_2, 0);

  /* "scipy/linalg/_decomp_update.pyx":348
 * 
 *     # u starts out as the jth basis vector.
 *     u[j] = 1             # <<<<<<<<<<<<<<
 * 
 *     # s = Q.T.dot(u) = jth row of Q.
 */
  (__pyx_v_u[__pyx_v_j]) = __pyx_t_double_complex_from_parts(1, 0);

  /* "scipy/linalg/_decomp_update.pyx":351
 * 
 *     # s = Q.T.dot(u) = jth row of Q.
 *     copy(n, row(q, qs, j), qs[1], s, 1)             # <<<<<<<<<<<<<<
 *     blas_t_conj(n, s, &ss)
 * 
 */
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_n, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[1]), __pyx_v_s, 1);

  /* "scipy/linalg/_decomp_update.pyx":352
 *     # s = Q.T.dot(u) = jth row of Q.
 *     copy(n, row(q, qs, j), qs[1], s, 1)
 *     blas_t_conj(n, s, &ss)             # <<<<<<<<<<<<<<
 * 
 *     # make u be the part of u that is not in span(q)
 */
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_n, __pyx_v_s, (&__pyx_v_ss));

  /* "scipy/linalg/_decomp_update.pyx":356
 *     # make u be the part of u that is not in span(q)
 *     # i.e. u -= q.dot(s)
 *     if qisF:             # <<<<<<<<<<<<<<
 *         gemv(N, m, n, -1, q, qs[1], s, 1, 1, u, 1)
 *     else:
 */
  __pyx_t_1 = (__pyx_v_qisF != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":357
 *     # i.e. u -= q.dot(s)
 *     if qisF:
 *         gemv(N, m, n, -1, q, qs[1], s, 1, 1, u, 1)             # <<<<<<<<<<<<<<
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, 1)
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_n, __pyx_t_double_complex_from_parts(-1L, 0), __pyx_v_q, (__pyx_v_qs[1]), __pyx_v_s, 1, __pyx_t_double_complex_from_parts(1, 0), __pyx_v_u, 1);

    /* "scipy/linalg/_decomp_update.pyx":356
 *     # make u be the part of u that is not in span(q)
 *     # i.e. u -= q.dot(s)
 *     if qisF:             # <<<<<<<<<<<<<<
 *         gemv(N, m, n, -1, q, qs[1], s, 1, 1, u, 1)
 *     else:
 */
    goto __pyx_L3;
  }

  /* "scipy/linalg/_decomp_update.pyx":359
 *         gemv(N, m, n, -1, q, qs[1], s, 1, 1, u, 1)
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, 1)             # <<<<<<<<<<<<<<
 *     wnorm = nrm2(m, u, 1)
 * 
 */
  /*else*/ {
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_n, __pyx_v_m, __pyx_t_double_complex_from_parts(-1L, 0), __pyx_v_q, __pyx_v_n, __pyx_v_s, 1, __pyx_t_double_complex_from_parts(1, 0), __pyx_v_u, 1);
  }
  __pyx_L3:;

  /* "scipy/linalg/_decomp_update.pyx":360
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, 1)
 *     wnorm = nrm2(m, u, 1)             # <<<<<<<<<<<<<<
 * 
 *     if blas_t_less_than(inv_root2, wnorm):
 */
  __pyx_v_wnorm = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_m, __pyx_v_u, 1);

  /* "scipy/linalg/_decomp_update.pyx":362
 *     wnorm = nrm2(m, u, 1)
 * 
 *     if blas_t_less_than(inv_root2, wnorm):             # <<<<<<<<<<<<<<
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, 1)
 */
  __pyx_t_1 = (__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_v_inv_root2, __pyx_v_wnorm) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":364
 *     if blas_t_less_than(inv_root2, wnorm):
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, 1)             # <<<<<<<<<<<<<<
 *         s[n] = wnorm
 *         return True
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, __Pyx_c_quot_double(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_wnorm), __pyx_v_u, 1);

    /* "scipy/linalg/_decomp_update.pyx":365
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, 1)
 *         s[n] = wnorm             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
    (__pyx_v_s[__pyx_v_n]) = __pyx_v_wnorm;

    /* "scipy/linalg/_decomp_update.pyx":366
 *             scal(m, 1/wnorm, u, 1)
 *         s[n] = wnorm
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     # if the above check failed, try one reorthogonalization
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":362
 *     wnorm = nrm2(m, u, 1)
 * 
 *     if blas_t_less_than(inv_root2, wnorm):             # <<<<<<<<<<<<<<
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, 1)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":369
 * 
 *     # if the above check failed, try one reorthogonalization
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, qs[1], u, 1, 0, s+n, 1)
 */
  __pyx_t_1 = (__pyx_v_qisF != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":373
 *             gemv(T, m, n, 1, q, qs[1], u, 1, 0, s+n, 1)
 *         else:
 *             gemv(C, m, n, 1, q, qs[1], u, 1, 0, s+n, 1)             # <<<<<<<<<<<<<<
 *         gemv(N, m, n, -1, q, qs[1], s+n, 1, 1, u, 1)
 *     else:
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_C, __pyx_v_m, __pyx_v_n, __pyx_t_double_complex_from_parts(1, 0), __pyx_v_q, (__pyx_v_qs[1]), __pyx_v_u, 1, __pyx_t_double_complex_from_parts(0, 0), (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":374
 *         else:
 *             gemv(C, m, n, 1, q, qs[1], u, 1, 0, s+n, 1)
 *         gemv(N, m, n, -1, q, qs[1], s+n, 1, 1, u, 1)             # <<<<<<<<<<<<<<
 *     else:
 *         if blas_t is float or blas_t is double:
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_n, __pyx_t_double_complex_from_parts(-1L, 0), __pyx_v_q, (__pyx_v_qs[1]), (__pyx_v_s + __pyx_v_n), 1, __pyx_t_double_complex_from_parts(1, 0), __pyx_v_u, 1);

    /* "scipy/linalg/_decomp_update.pyx":369
 * 
 *     # if the above check failed, try one reorthogonalization
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, qs[1], u, 1, 0, s+n, 1)
 */
    goto __pyx_L5;
  }

  /* "scipy/linalg/_decomp_update.pyx":376
 *         gemv(N, m, n, -1, q, qs[1], s+n, 1, 1, u, 1)
 *     else:
 *         if blas_t is float or blas_t is double:             # <<<<<<<<<<<<<<
 *             gemv(N, n, m, 1, q, n, u, 1, 0, s+n, 1)
 *         else:
 */
  /*else*/ {

    /* "scipy/linalg/_decomp_update.pyx":379
 *             gemv(N, n, m, 1, q, n, u, 1, 0, s+n, 1)
 *         else:
 *             blas_t_conj(m, u, &ss)             # <<<<<<<<<<<<<<
 *             gemv(N, n, m, 1, q, n, u, 1, 0, s+n, 1)
 *             blas_t_conj(m, u, &ss)
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_m, __pyx_v_u, (&__pyx_v_ss));

    /* "scipy/linalg/_decomp_update.pyx":380
 *         else:
 *             blas_t_conj(m, u, &ss)
 *             gemv(N, n, m, 1, q, n, u, 1, 0, s+n, 1)             # <<<<<<<<<<<<<<
 *             blas_t_conj(m, u, &ss)
 *             blas_t_conj(n, s+n, &ss)
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_n, __pyx_v_m, __pyx_t_double_complex_from_parts(1, 0), __pyx_v_q, __pyx_v_n, __pyx_v_u, 1, __pyx_t_double_complex_from_parts(0, 0), (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":381
 *             blas_t_conj(m, u, &ss)
 *             gemv(N, n, m, 1, q, n, u, 1, 0, s+n, 1)
 *             blas_t_conj(m, u, &ss)             # <<<<<<<<<<<<<<
 *             blas_t_conj(n, s+n, &ss)
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, 1)
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_m, __pyx_v_u, (&__pyx_v_ss));

    /* "scipy/linalg/_decomp_update.pyx":382
 *             gemv(N, n, m, 1, q, n, u, 1, 0, s+n, 1)
 *             blas_t_conj(m, u, &ss)
 *             blas_t_conj(n, s+n, &ss)             # <<<<<<<<<<<<<<
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, 1)
 *     wpnorm = nrm2(m, u, 1)
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_n, (__pyx_v_s + __pyx_v_n), (&__pyx_v_ss));

    /* "scipy/linalg/_decomp_update.pyx":383
 *             blas_t_conj(m, u, &ss)
 *             blas_t_conj(n, s+n, &ss)
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, 1)             # <<<<<<<<<<<<<<
 *     wpnorm = nrm2(m, u, 1)
 * 
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_n, __pyx_v_m, __pyx_t_double_complex_from_parts(-1L, 0), __pyx_v_q, __pyx_v_n, (__pyx_v_s + __pyx_v_n), 1, __pyx_t_double_complex_from_parts(1, 0), __pyx_v_u, 1);
  }
  __pyx_L5:;

  /* "scipy/linalg/_decomp_update.pyx":384
 *             blas_t_conj(n, s+n, &ss)
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, 1)
 *     wpnorm = nrm2(m, u, 1)             # <<<<<<<<<<<<<<
 * 
 *     if blas_t_less_than(wpnorm, wnorm*inv_root2): # u lies in span(q)
 */
  __pyx_v_wpnorm = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_m, __pyx_v_u, 1);

  /* "scipy/linalg/_decomp_update.pyx":386
 *     wpnorm = nrm2(m, u, 1)
 * 
 *     if blas_t_less_than(wpnorm, wnorm*inv_root2): # u lies in span(q)             # <<<<<<<<<<<<<<
 *         scal(m, 0, u, 1)
 *         axpy(n, 1, s, 1, s+n, 1)
 */
  __pyx_t_1 = (__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_v_wpnorm, __Pyx_c_prod_double(__pyx_v_wnorm, __pyx_v_inv_root2)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":387
 * 
 *     if blas_t_less_than(wpnorm, wnorm*inv_root2): # u lies in span(q)
 *         scal(m, 0, u, 1)             # <<<<<<<<<<<<<<
 *         axpy(n, 1, s, 1, s+n, 1)
 *         s[n] = 0
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, __pyx_t_double_complex_from_parts(0, 0), __pyx_v_u, 1);

    /* "scipy/linalg/_decomp_update.pyx":388
 *     if blas_t_less_than(wpnorm, wnorm*inv_root2): # u lies in span(q)
 *         scal(m, 0, u, 1)
 *         axpy(n, 1, s, 1, s+n, 1)             # <<<<<<<<<<<<<<
 *         s[n] = 0
 *         return False
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, __pyx_t_double_complex_from_parts(1, 0), __pyx_v_s, 1, (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":389
 *         scal(m, 0, u, 1)
 *         axpy(n, 1, s, 1, s+n, 1)
 *         s[n] = 0             # <<<<<<<<<<<<<<
 *         return False
 * 
 */
    (__pyx_v_s[__pyx_v_n]) = __pyx_t_double_complex_from_parts(0, 0);

    /* "scipy/linalg/_decomp_update.pyx":390
 *         axpy(n, 1, s, 1, s+n, 1)
 *         s[n] = 0
 *         return False             # <<<<<<<<<<<<<<
 * 
 *     scal(m, 1/wpnorm, u, 1)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":386
 *     wpnorm = nrm2(m, u, 1)
 * 
 *     if blas_t_less_than(wpnorm, wnorm*inv_root2): # u lies in span(q)             # <<<<<<<<<<<<<<
 *         scal(m, 0, u, 1)
 *         axpy(n, 1, s, 1, s+n, 1)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":392
 *         return False
 * 
 *     scal(m, 1/wpnorm, u, 1)             # <<<<<<<<<<<<<<
 *     axpy(n, 1, s, 1, s+n, 1)
 *     s[n] = wpnorm
 */
  __pyx_t_2 = __pyx_t_double_complex_from_parts(1, 0);
  if (unlikely(__Pyx_c_is_zero_double(__pyx_v_wpnorm))) {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
    #endif
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    #ifdef WITH_THREAD
    __Pyx_PyGILState_Release(__pyx_gilstate_save);
    #endif
    __PYX_ERR(0, 392, __pyx_L1_error)
  }
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, __Pyx_c_quot_double(__pyx_t_2, __pyx_v_wpnorm), __pyx_v_u, 1);

  /* "scipy/linalg/_decomp_update.pyx":393
 * 
 *     scal(m, 1/wpnorm, u, 1)
 *     axpy(n, 1, s, 1, s+n, 1)             # <<<<<<<<<<<<<<
 *     s[n] = wpnorm
 *     return True
 */
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, __pyx_t_double_complex_from_parts(1, 0), __pyx_v_s, 1, (__pyx_v_s + __pyx_v_n), 1);

  /* "scipy/linalg/_decomp_update.pyx":394
 *     scal(m, 1/wpnorm, u, 1)
 *     axpy(n, 1, s, 1, s+n, 1)
 *     s[n] = wpnorm             # <<<<<<<<<<<<<<
 *     return True
 * 
 */
  (__pyx_v_s[__pyx_v_n]) = __pyx_v_wpnorm;

  /* "scipy/linalg/_decomp_update.pyx":395
 *     axpy(n, 1, s, 1, s+n, 1)
 *     s[n] = wpnorm
 *     return True             # <<<<<<<<<<<<<<
 * 
 * cdef int thin_qr_row_delete(int m, int n, blas_t* q, int* qs, bint qisF,
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":338
 * #------------------------------------------------------------------------------
 * 
 * cdef bint reorthx(int m, int n, blas_t* q, int* qs, bint qisF, int j, blas_t* u, blas_t* s) nogil:             # <<<<<<<<<<<<<<
 *     # U should be all zeros on entry., and m > 1
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.linalg._decomp_update.reorthx", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":397
 *     return True
 * 
 * cdef int thin_qr_row_delete(int m, int n, blas_t* q, int* qs, bint qisF,             # <<<<<<<<<<<<<<
 *                             blas_t* r, int* rs, int k, int p_eco,
 *                             int p_full) nogil:
 */

static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_delete(int __pyx_v_m, int __pyx_v_n, float *__pyx_v_q, int *__pyx_v_qs, int __pyx_v_qisF, float *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p_eco, int __pyx_v_p_full) {
  CYTHON_UNUSED int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_argmin_row_norm;
  size_t __pyx_v_usize;
  float *__pyx_v_s;
  float *__pyx_v_u;
  int __pyx_v_us[2];
  int __pyx_v_ss[2];
  float __pyx_v_c;
  float __pyx_v_sn;
  float __pyx_v_min_row_norm;
  float __pyx_v_row_norm;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;

  /* "scipy/linalg/_decomp_update.pyx":401
 *                             int p_full) nogil:
 *     cdef int i, j, argmin_row_norm
 *     cdef size_t usize = (m + 3*n + 1) * sizeof(blas_t)             # <<<<<<<<<<<<<<
 *     cdef blas_t* s
 *     cdef blas_t* u
 */
  __pyx_v_usize = (((__pyx_v_m + (3 * __pyx_v_n)) + 1) * (sizeof(float)));

  /* "scipy/linalg/_decomp_update.pyx":409
 *     cdef blas_t c, sn, min_row_norm, row_norm
 * 
 *     u = <blas_t*>libc.stdlib.malloc(usize)             # <<<<<<<<<<<<<<
 *     if not u:
 *         return MEMORY_ERROR
 */
  __pyx_v_u = ((float *)malloc(__pyx_v_usize));

  /* "scipy/linalg/_decomp_update.pyx":410
 * 
 *     u = <blas_t*>libc.stdlib.malloc(usize)
 *     if not u:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 *     s = u + m
 */
  __pyx_t_1 = ((!(__pyx_v_u != 0)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":411
 *     u = <blas_t*>libc.stdlib.malloc(usize)
 *     if not u:
 *         return MEMORY_ERROR             # <<<<<<<<<<<<<<
 *     s = u + m
 *     ss[0] = 1
 */
    __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":410
 * 
 *     u = <blas_t*>libc.stdlib.malloc(usize)
 *     if not u:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 *     s = u + m
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":412
 *     if not u:
 *         return MEMORY_ERROR
 *     s = u + m             # <<<<<<<<<<<<<<
 *     ss[0] = 1
 *     ss[1] = 0
 */
  __pyx_v_s = (__pyx_v_u + __pyx_v_m);

  /* "scipy/linalg/_decomp_update.pyx":413
 *         return MEMORY_ERROR
 *     s = u + m
 *     ss[0] = 1             # <<<<<<<<<<<<<<
 *     ss[1] = 0
 *     us[0] = 1
 */
  (__pyx_v_ss[0]) = 1;

  /* "scipy/linalg/_decomp_update.pyx":414
 *     s = u + m
 *     ss[0] = 1
 *     ss[1] = 0             # <<<<<<<<<<<<<<
 *     us[0] = 1
 *     us[1] = 0
 */
  (__pyx_v_ss[1]) = 0;

  /* "scipy/linalg/_decomp_update.pyx":415
 *     ss[0] = 1
 *     ss[1] = 0
 *     us[0] = 1             # <<<<<<<<<<<<<<
 *     us[1] = 0
 * 
 */
  (__pyx_v_us[0]) = 1;

  /* "scipy/linalg/_decomp_update.pyx":416
 *     ss[1] = 0
 *     us[0] = 1
 *     us[1] = 0             # <<<<<<<<<<<<<<
 * 
 *     for i in range(p_eco):
 */
  (__pyx_v_us[1]) = 0;

  /* "scipy/linalg/_decomp_update.pyx":418
 *     us[1] = 0
 * 
 *     for i in range(p_eco):             # <<<<<<<<<<<<<<
 *         memset(u, 0, usize)
 *         # permute q such that row k is the last row.
 */
  __pyx_t_2 = __pyx_v_p_eco;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "scipy/linalg/_decomp_update.pyx":419
 * 
 *     for i in range(p_eco):
 *         memset(u, 0, usize)             # <<<<<<<<<<<<<<
 *         # permute q such that row k is the last row.
 *         if k != m-1:
 */
    (void)(memset(__pyx_v_u, 0, __pyx_v_usize));

    /* "scipy/linalg/_decomp_update.pyx":421
 *         memset(u, 0, usize)
 *         # permute q such that row k is the last row.
 *         if k != m-1:             # <<<<<<<<<<<<<<
 *             for j in range(k, m-1):
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])
 */
    __pyx_t_1 = ((__pyx_v_k != (__pyx_v_m - 1)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":422
 *         # permute q such that row k is the last row.
 *         if k != m-1:
 *             for j in range(k, m-1):             # <<<<<<<<<<<<<<
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])
 * 
 */
      __pyx_t_5 = (__pyx_v_m - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = __pyx_v_k; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
        __pyx_v_j = __pyx_t_7;

        /* "scipy/linalg/_decomp_update.pyx":423
 *         if k != m-1:
 *             for j in range(k, m-1):
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])             # <<<<<<<<<<<<<<
 * 
 *         if not reorthx(m, n, q, qs, qisF, m-1, u, s):
 */
        __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_swap(__pyx_v_n, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[1]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[1]));
      }

      /* "scipy/linalg/_decomp_update.pyx":421
 *         memset(u, 0, usize)
 *         # permute q such that row k is the last row.
 *         if k != m-1:             # <<<<<<<<<<<<<<
 *             for j in range(k, m-1):
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":425
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])
 * 
 *         if not reorthx(m, n, q, qs, qisF, m-1, u, s):             # <<<<<<<<<<<<<<
 *             # if we get here it means that this basis vector lies in span(q).
 *             # we want to use s[:n+1] but we need a vector into null(q)
 */
    __pyx_t_1 = ((!(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_reorthx(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_qisF, (__pyx_v_m - 1), __pyx_v_u, __pyx_v_s) != 0)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":429
 *             # we want to use s[:n+1] but we need a vector into null(q)
 *             # find the row of q with the smallest norm and try that. (Daniel, p785)
 *             min_row_norm = nrm2(n, row(q, qs, 0), qs[1])             # <<<<<<<<<<<<<<
 *             argmin_row_norm = 0
 *             for j in range(1, m):
 */
      __pyx_v_min_row_norm = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_n, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, 0), (__pyx_v_qs[1]));

      /* "scipy/linalg/_decomp_update.pyx":430
 *             # find the row of q with the smallest norm and try that. (Daniel, p785)
 *             min_row_norm = nrm2(n, row(q, qs, 0), qs[1])
 *             argmin_row_norm = 0             # <<<<<<<<<<<<<<
 *             for j in range(1, m):
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])
 */
      __pyx_v_argmin_row_norm = 0;

      /* "scipy/linalg/_decomp_update.pyx":431
 *             min_row_norm = nrm2(n, row(q, qs, 0), qs[1])
 *             argmin_row_norm = 0
 *             for j in range(1, m):             # <<<<<<<<<<<<<<
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])
 *                 if blas_t_less_than(row_norm, min_row_norm):
 */
      __pyx_t_7 = __pyx_v_m;
      __pyx_t_8 = __pyx_t_7;
      for (__pyx_t_9 = 1; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
        __pyx_v_j = __pyx_t_9;

        /* "scipy/linalg/_decomp_update.pyx":432
 *             argmin_row_norm = 0
 *             for j in range(1, m):
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])             # <<<<<<<<<<<<<<
 *                 if blas_t_less_than(row_norm, min_row_norm):
 *                     min_row_norm = row_norm
 */
        __pyx_v_row_norm = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_n, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[1]));

        /* "scipy/linalg/_decomp_update.pyx":433
 *             for j in range(1, m):
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])
 *                 if blas_t_less_than(row_norm, min_row_norm):             # <<<<<<<<<<<<<<
 *                     min_row_norm = row_norm
 *                     argmin_row_norm = j
 */
        __pyx_t_1 = (__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_v_row_norm, __pyx_v_min_row_norm) != 0);
        if (__pyx_t_1) {

          /* "scipy/linalg/_decomp_update.pyx":434
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])
 *                 if blas_t_less_than(row_norm, min_row_norm):
 *                     min_row_norm = row_norm             # <<<<<<<<<<<<<<
 *                     argmin_row_norm = j
 *             memset(u, 0, m*sizeof(blas_t))
 */
          __pyx_v_min_row_norm = __pyx_v_row_norm;

          /* "scipy/linalg/_decomp_update.pyx":435
 *                 if blas_t_less_than(row_norm, min_row_norm):
 *                     min_row_norm = row_norm
 *                     argmin_row_norm = j             # <<<<<<<<<<<<<<
 *             memset(u, 0, m*sizeof(blas_t))
 *             if not reorthx(m, n, q, qs, qisF, argmin_row_norm, u, s):
 */
          __pyx_v_argmin_row_norm = __pyx_v_j;

          /* "scipy/linalg/_decomp_update.pyx":433
 *             for j in range(1, m):
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])
 *                 if blas_t_less_than(row_norm, min_row_norm):             # <<<<<<<<<<<<<<
 *                     min_row_norm = row_norm
 *                     argmin_row_norm = j
 */
        }
      }

      /* "scipy/linalg/_decomp_update.pyx":436
 *                     min_row_norm = row_norm
 *                     argmin_row_norm = j
 *             memset(u, 0, m*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *             if not reorthx(m, n, q, qs, qisF, argmin_row_norm, u, s):
 *                 # failed, quit.
 */
      (void)(memset(__pyx_v_u, 0, (__pyx_v_m * (sizeof(float)))));

      /* "scipy/linalg/_decomp_update.pyx":437
 *                     argmin_row_norm = j
 *             memset(u, 0, m*sizeof(blas_t))
 *             if not reorthx(m, n, q, qs, qisF, argmin_row_norm, u, s):             # <<<<<<<<<<<<<<
 *                 # failed, quit.
 *                 libc.stdlib.free(u)
 */
      __pyx_t_1 = ((!(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_reorthx(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_qisF, __pyx_v_argmin_row_norm, __pyx_v_u, __pyx_v_s) != 0)) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":439
 *             if not reorthx(m, n, q, qs, qisF, argmin_row_norm, u, s):
 *                 # failed, quit.
 *                 libc.stdlib.free(u)             # <<<<<<<<<<<<<<
 *                 return 0
 *             s[n] = 0
 */
        free(__pyx_v_u);

        /* "scipy/linalg/_decomp_update.pyx":440
 *                 # failed, quit.
 *                 libc.stdlib.free(u)
 *                 return 0             # <<<<<<<<<<<<<<
 *             s[n] = 0
 * 
 */
        __pyx_r = 0;
        goto __pyx_L0;

        /* "scipy/linalg/_decomp_update.pyx":437
 *                     argmin_row_norm = j
 *             memset(u, 0, m*sizeof(blas_t))
 *             if not reorthx(m, n, q, qs, qisF, argmin_row_norm, u, s):             # <<<<<<<<<<<<<<
 *                 # failed, quit.
 *                 libc.stdlib.free(u)
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":441
 *                 libc.stdlib.free(u)
 *                 return 0
 *             s[n] = 0             # <<<<<<<<<<<<<<
 * 
 *         memset(s+2*n, 0, n*sizeof(blas_t))
 */
      (__pyx_v_s[__pyx_v_n]) = 0.0;

      /* "scipy/linalg/_decomp_update.pyx":425
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])
 * 
 *         if not reorthx(m, n, q, qs, qisF, m-1, u, s):             # <<<<<<<<<<<<<<
 *             # if we get here it means that this basis vector lies in span(q).
 *             # we want to use s[:n+1] but we need a vector into null(q)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":443
 *             s[n] = 0
 * 
 *         memset(s+2*n, 0, n*sizeof(blas_t))             # <<<<<<<<<<<<<<
 * 
 *         # what happens here...
 */
    (void)(memset((__pyx_v_s + (2 * __pyx_v_n)), 0, (__pyx_v_n * (sizeof(float)))));

    /* "scipy/linalg/_decomp_update.pyx":446
 * 
 *         # what happens here...
 *         for j in range(n-1, -1, -1):             # <<<<<<<<<<<<<<
 *             lartg(index1(s, ss, n), index1(s, ss, j), &c, &sn)
 *             rot(n-j, index1(s+2*n, ss, j), ss[0], index2(r, rs,j, j), rs[1], c, sn)
 */
    for (__pyx_t_7 = (__pyx_v_n - 1); __pyx_t_7 > -1; __pyx_t_7-=1) {
      __pyx_v_j = __pyx_t_7;

      /* "scipy/linalg/_decomp_update.pyx":447
 *         # what happens here...
 *         for j in range(n-1, -1, -1):
 *             lartg(index1(s, ss, n), index1(s, ss, j), &c, &sn)             # <<<<<<<<<<<<<<
 *             rot(n-j, index1(s+2*n, ss, j), ss[0], index2(r, rs,j, j), rs[1], c, sn)
 *             rot(m-1, u, us[0], col(q, qs, j), qs[0], c, sn.conjugate())
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, __pyx_v_n), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, __pyx_v_j), (&__pyx_v_c), (&__pyx_v_sn));

      /* "scipy/linalg/_decomp_update.pyx":448
 *         for j in range(n-1, -1, -1):
 *             lartg(index1(s, ss, n), index1(s, ss, j), &c, &sn)
 *             rot(n-j, index1(s+2*n, ss, j), ss[0], index2(r, rs,j, j), rs[1], c, sn)             # <<<<<<<<<<<<<<
 *             rot(m-1, u, us[0], col(q, qs, j), qs[0], c, sn.conjugate())
 *         m -= 1
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_n - __pyx_v_j), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index1((__pyx_v_s + (2 * __pyx_v_n)), __pyx_v_ss, __pyx_v_j), (__pyx_v_ss[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_sn);

      /* "scipy/linalg/_decomp_update.pyx":449
 *             lartg(index1(s, ss, n), index1(s, ss, j), &c, &sn)
 *             rot(n-j, index1(s+2*n, ss, j), ss[0], index2(r, rs,j, j), rs[1], c, sn)
 *             rot(m-1, u, us[0], col(q, qs, j), qs[0], c, sn.conjugate())             # <<<<<<<<<<<<<<
 *         m -= 1
 * 
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_m - 1), __pyx_v_u, (__pyx_v_us[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_sn));
    }

    /* "scipy/linalg/_decomp_update.pyx":450
 *             rot(n-j, index1(s+2*n, ss, j), ss[0], index2(r, rs,j, j), rs[1], c, sn)
 *             rot(m-1, u, us[0], col(q, qs, j), qs[0], c, sn.conjugate())
 *         m -= 1             # <<<<<<<<<<<<<<
 * 
 *     libc.stdlib.free(u)
 */
    __pyx_v_m = (__pyx_v_m - 1);
  }

  /* "scipy/linalg/_decomp_update.pyx":452
 *         m -= 1
 * 
 *     libc.stdlib.free(u)             # <<<<<<<<<<<<<<
 * 
 *     if p_full:
 */
  free(__pyx_v_u);

  /* "scipy/linalg/_decomp_update.pyx":454
 *     libc.stdlib.free(u)
 * 
 *     if p_full:             # <<<<<<<<<<<<<<
 *         qr_block_row_delete(m, n, q, qs, r, rs, k, p_full)
 *     return 1
 */
  __pyx_t_1 = (__pyx_v_p_full != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":455
 * 
 *     if p_full:
 *         qr_block_row_delete(m, n, q, qs, r, rs, k, p_full)             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_delete(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, __pyx_v_k, __pyx_v_p_full);

    /* "scipy/linalg/_decomp_update.pyx":454
 *     libc.stdlib.free(u)
 * 
 *     if p_full:             # <<<<<<<<<<<<<<
 *         qr_block_row_delete(m, n, q, qs, r, rs, k, p_full)
 *     return 1
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":456
 *     if p_full:
 *         qr_block_row_delete(m, n, q, qs, r, rs, k, p_full)
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * cdef void qr_block_row_delete(int m, int n, blas_t* q, int* qs,
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":397
 *     return True
 * 
 * cdef int thin_qr_row_delete(int m, int n, blas_t* q, int* qs, bint qisF,             # <<<<<<<<<<<<<<
 *                             blas_t* r, int* rs, int k, int p_eco,
 *                             int p_full) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_delete(int __pyx_v_m, int __pyx_v_n, double *__pyx_v_q, int *__pyx_v_qs, int __pyx_v_qisF, double *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p_eco, int __pyx_v_p_full) {
  CYTHON_UNUSED int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_argmin_row_norm;
  size_t __pyx_v_usize;
  double *__pyx_v_s;
  double *__pyx_v_u;
  int __pyx_v_us[2];
  int __pyx_v_ss[2];
  double __pyx_v_c;
  double __pyx_v_sn;
  double __pyx_v_min_row_norm;
  double __pyx_v_row_norm;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;

  /* "scipy/linalg/_decomp_update.pyx":401
 *                             int p_full) nogil:
 *     cdef int i, j, argmin_row_norm
 *     cdef size_t usize = (m + 3*n + 1) * sizeof(blas_t)             # <<<<<<<<<<<<<<
 *     cdef blas_t* s
 *     cdef blas_t* u
 */
  __pyx_v_usize = (((__pyx_v_m + (3 * __pyx_v_n)) + 1) * (sizeof(double)));

  /* "scipy/linalg/_decomp_update.pyx":409
 *     cdef blas_t c, sn, min_row_norm, row_norm
 * 
 *     u = <blas_t*>libc.stdlib.malloc(usize)             # <<<<<<<<<<<<<<
 *     if not u:
 *         return MEMORY_ERROR
 */
  __pyx_v_u = ((double *)malloc(__pyx_v_usize));

  /* "scipy/linalg/_decomp_update.pyx":410
 * 
 *     u = <blas_t*>libc.stdlib.malloc(usize)
 *     if not u:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 *     s = u + m
 */
  __pyx_t_1 = ((!(__pyx_v_u != 0)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":411
 *     u = <blas_t*>libc.stdlib.malloc(usize)
 *     if not u:
 *         return MEMORY_ERROR             # <<<<<<<<<<<<<<
 *     s = u + m
 *     ss[0] = 1
 */
    __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":410
 * 
 *     u = <blas_t*>libc.stdlib.malloc(usize)
 *     if not u:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 *     s = u + m
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":412
 *     if not u:
 *         return MEMORY_ERROR
 *     s = u + m             # <<<<<<<<<<<<<<
 *     ss[0] = 1
 *     ss[1] = 0
 */
  __pyx_v_s = (__pyx_v_u + __pyx_v_m);

  /* "scipy/linalg/_decomp_update.pyx":413
 *         return MEMORY_ERROR
 *     s = u + m
 *     ss[0] = 1             # <<<<<<<<<<<<<<
 *     ss[1] = 0
 *     us[0] = 1
 */
  (__pyx_v_ss[0]) = 1;

  /* "scipy/linalg/_decomp_update.pyx":414
 *     s = u + m
 *     ss[0] = 1
 *     ss[1] = 0             # <<<<<<<<<<<<<<
 *     us[0] = 1
 *     us[1] = 0
 */
  (__pyx_v_ss[1]) = 0;

  /* "scipy/linalg/_decomp_update.pyx":415
 *     ss[0] = 1
 *     ss[1] = 0
 *     us[0] = 1             # <<<<<<<<<<<<<<
 *     us[1] = 0
 * 
 */
  (__pyx_v_us[0]) = 1;

  /* "scipy/linalg/_decomp_update.pyx":416
 *     ss[1] = 0
 *     us[0] = 1
 *     us[1] = 0             # <<<<<<<<<<<<<<
 * 
 *     for i in range(p_eco):
 */
  (__pyx_v_us[1]) = 0;

  /* "scipy/linalg/_decomp_update.pyx":418
 *     us[1] = 0
 * 
 *     for i in range(p_eco):             # <<<<<<<<<<<<<<
 *         memset(u, 0, usize)
 *         # permute q such that row k is the last row.
 */
  __pyx_t_2 = __pyx_v_p_eco;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "scipy/linalg/_decomp_update.pyx":419
 * 
 *     for i in range(p_eco):
 *         memset(u, 0, usize)             # <<<<<<<<<<<<<<
 *         # permute q such that row k is the last row.
 *         if k != m-1:
 */
    (void)(memset(__pyx_v_u, 0, __pyx_v_usize));

    /* "scipy/linalg/_decomp_update.pyx":421
 *         memset(u, 0, usize)
 *         # permute q such that row k is the last row.
 *         if k != m-1:             # <<<<<<<<<<<<<<
 *             for j in range(k, m-1):
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])
 */
    __pyx_t_1 = ((__pyx_v_k != (__pyx_v_m - 1)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":422
 *         # permute q such that row k is the last row.
 *         if k != m-1:
 *             for j in range(k, m-1):             # <<<<<<<<<<<<<<
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])
 * 
 */
      __pyx_t_5 = (__pyx_v_m - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = __pyx_v_k; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
        __pyx_v_j = __pyx_t_7;

        /* "scipy/linalg/_decomp_update.pyx":423
 *         if k != m-1:
 *             for j in range(k, m-1):
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])             # <<<<<<<<<<<<<<
 * 
 *         if not reorthx(m, n, q, qs, qisF, m-1, u, s):
 */
        __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_swap(__pyx_v_n, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[1]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[1]));
      }

      /* "scipy/linalg/_decomp_update.pyx":421
 *         memset(u, 0, usize)
 *         # permute q such that row k is the last row.
 *         if k != m-1:             # <<<<<<<<<<<<<<
 *             for j in range(k, m-1):
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":425
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])
 * 
 *         if not reorthx(m, n, q, qs, qisF, m-1, u, s):             # <<<<<<<<<<<<<<
 *             # if we get here it means that this basis vector lies in span(q).
 *             # we want to use s[:n+1] but we need a vector into null(q)
 */
    __pyx_t_1 = ((!(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_reorthx(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_qisF, (__pyx_v_m - 1), __pyx_v_u, __pyx_v_s) != 0)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":429
 *             # we want to use s[:n+1] but we need a vector into null(q)
 *             # find the row of q with the smallest norm and try that. (Daniel, p785)
 *             min_row_norm = nrm2(n, row(q, qs, 0), qs[1])             # <<<<<<<<<<<<<<
 *             argmin_row_norm = 0
 *             for j in range(1, m):
 */
      __pyx_v_min_row_norm = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_n, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, 0), (__pyx_v_qs[1]));

      /* "scipy/linalg/_decomp_update.pyx":430
 *             # find the row of q with the smallest norm and try that. (Daniel, p785)
 *             min_row_norm = nrm2(n, row(q, qs, 0), qs[1])
 *             argmin_row_norm = 0             # <<<<<<<<<<<<<<
 *             for j in range(1, m):
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])
 */
      __pyx_v_argmin_row_norm = 0;

      /* "scipy/linalg/_decomp_update.pyx":431
 *             min_row_norm = nrm2(n, row(q, qs, 0), qs[1])
 *             argmin_row_norm = 0
 *             for j in range(1, m):             # <<<<<<<<<<<<<<
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])
 *                 if blas_t_less_than(row_norm, min_row_norm):
 */
      __pyx_t_7 = __pyx_v_m;
      __pyx_t_8 = __pyx_t_7;
      for (__pyx_t_9 = 1; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
        __pyx_v_j = __pyx_t_9;

        /* "scipy/linalg/_decomp_update.pyx":432
 *             argmin_row_norm = 0
 *             for j in range(1, m):
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])             # <<<<<<<<<<<<<<
 *                 if blas_t_less_than(row_norm, min_row_norm):
 *                     min_row_norm = row_norm
 */
        __pyx_v_row_norm = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_n, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[1]));

        /* "scipy/linalg/_decomp_update.pyx":433
 *             for j in range(1, m):
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])
 *                 if blas_t_less_than(row_norm, min_row_norm):             # <<<<<<<<<<<<<<
 *                     min_row_norm = row_norm
 *                     argmin_row_norm = j
 */
        __pyx_t_1 = (__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_v_row_norm, __pyx_v_min_row_norm) != 0);
        if (__pyx_t_1) {

          /* "scipy/linalg/_decomp_update.pyx":434
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])
 *                 if blas_t_less_than(row_norm, min_row_norm):
 *                     min_row_norm = row_norm             # <<<<<<<<<<<<<<
 *                     argmin_row_norm = j
 *             memset(u, 0, m*sizeof(blas_t))
 */
          __pyx_v_min_row_norm = __pyx_v_row_norm;

          /* "scipy/linalg/_decomp_update.pyx":435
 *                 if blas_t_less_than(row_norm, min_row_norm):
 *                     min_row_norm = row_norm
 *                     argmin_row_norm = j             # <<<<<<<<<<<<<<
 *             memset(u, 0, m*sizeof(blas_t))
 *             if not reorthx(m, n, q, qs, qisF, argmin_row_norm, u, s):
 */
          __pyx_v_argmin_row_norm = __pyx_v_j;

          /* "scipy/linalg/_decomp_update.pyx":433
 *             for j in range(1, m):
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])
 *                 if blas_t_less_than(row_norm, min_row_norm):             # <<<<<<<<<<<<<<
 *                     min_row_norm = row_norm
 *                     argmin_row_norm = j
 */
        }
      }

      /* "scipy/linalg/_decomp_update.pyx":436
 *                     min_row_norm = row_norm
 *                     argmin_row_norm = j
 *             memset(u, 0, m*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *             if not reorthx(m, n, q, qs, qisF, argmin_row_norm, u, s):
 *                 # failed, quit.
 */
      (void)(memset(__pyx_v_u, 0, (__pyx_v_m * (sizeof(double)))));

      /* "scipy/linalg/_decomp_update.pyx":437
 *                     argmin_row_norm = j
 *             memset(u, 0, m*sizeof(blas_t))
 *             if not reorthx(m, n, q, qs, qisF, argmin_row_norm, u, s):             # <<<<<<<<<<<<<<
 *                 # failed, quit.
 *                 libc.stdlib.free(u)
 */
      __pyx_t_1 = ((!(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_reorthx(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_qisF, __pyx_v_argmin_row_norm, __pyx_v_u, __pyx_v_s) != 0)) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":439
 *             if not reorthx(m, n, q, qs, qisF, argmin_row_norm, u, s):
 *                 # failed, quit.
 *                 libc.stdlib.free(u)             # <<<<<<<<<<<<<<
 *                 return 0
 *             s[n] = 0
 */
        free(__pyx_v_u);

        /* "scipy/linalg/_decomp_update.pyx":440
 *                 # failed, quit.
 *                 libc.stdlib.free(u)
 *                 return 0             # <<<<<<<<<<<<<<
 *             s[n] = 0
 * 
 */
        __pyx_r = 0;
        goto __pyx_L0;

        /* "scipy/linalg/_decomp_update.pyx":437
 *                     argmin_row_norm = j
 *             memset(u, 0, m*sizeof(blas_t))
 *             if not reorthx(m, n, q, qs, qisF, argmin_row_norm, u, s):             # <<<<<<<<<<<<<<
 *                 # failed, quit.
 *                 libc.stdlib.free(u)
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":441
 *                 libc.stdlib.free(u)
 *                 return 0
 *             s[n] = 0             # <<<<<<<<<<<<<<
 * 
 *         memset(s+2*n, 0, n*sizeof(blas_t))
 */
      (__pyx_v_s[__pyx_v_n]) = 0.0;

      /* "scipy/linalg/_decomp_update.pyx":425
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])
 * 
 *         if not reorthx(m, n, q, qs, qisF, m-1, u, s):             # <<<<<<<<<<<<<<
 *             # if we get here it means that this basis vector lies in span(q).
 *             # we want to use s[:n+1] but we need a vector into null(q)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":443
 *             s[n] = 0
 * 
 *         memset(s+2*n, 0, n*sizeof(blas_t))             # <<<<<<<<<<<<<<
 * 
 *         # what happens here...
 */
    (void)(memset((__pyx_v_s + (2 * __pyx_v_n)), 0, (__pyx_v_n * (sizeof(double)))));

    /* "scipy/linalg/_decomp_update.pyx":446
 * 
 *         # what happens here...
 *         for j in range(n-1, -1, -1):             # <<<<<<<<<<<<<<
 *             lartg(index1(s, ss, n), index1(s, ss, j), &c, &sn)
 *             rot(n-j, index1(s+2*n, ss, j), ss[0], index2(r, rs,j, j), rs[1], c, sn)
 */
    for (__pyx_t_7 = (__pyx_v_n - 1); __pyx_t_7 > -1; __pyx_t_7-=1) {
      __pyx_v_j = __pyx_t_7;

      /* "scipy/linalg/_decomp_update.pyx":447
 *         # what happens here...
 *         for j in range(n-1, -1, -1):
 *             lartg(index1(s, ss, n), index1(s, ss, j), &c, &sn)             # <<<<<<<<<<<<<<
 *             rot(n-j, index1(s+2*n, ss, j), ss[0], index2(r, rs,j, j), rs[1], c, sn)
 *             rot(m-1, u, us[0], col(q, qs, j), qs[0], c, sn.conjugate())
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, __pyx_v_n), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, __pyx_v_j), (&__pyx_v_c), (&__pyx_v_sn));

      /* "scipy/linalg/_decomp_update.pyx":448
 *         for j in range(n-1, -1, -1):
 *             lartg(index1(s, ss, n), index1(s, ss, j), &c, &sn)
 *             rot(n-j, index1(s+2*n, ss, j), ss[0], index2(r, rs,j, j), rs[1], c, sn)             # <<<<<<<<<<<<<<
 *             rot(m-1, u, us[0], col(q, qs, j), qs[0], c, sn.conjugate())
 *         m -= 1
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_n - __pyx_v_j), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index1((__pyx_v_s + (2 * __pyx_v_n)), __pyx_v_ss, __pyx_v_j), (__pyx_v_ss[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_sn);

      /* "scipy/linalg/_decomp_update.pyx":449
 *             lartg(index1(s, ss, n), index1(s, ss, j), &c, &sn)
 *             rot(n-j, index1(s+2*n, ss, j), ss[0], index2(r, rs,j, j), rs[1], c, sn)
 *             rot(m-1, u, us[0], col(q, qs, j), qs[0], c, sn.conjugate())             # <<<<<<<<<<<<<<
 *         m -= 1
 * 
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_m - 1), __pyx_v_u, (__pyx_v_us[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_sn));
    }

    /* "scipy/linalg/_decomp_update.pyx":450
 *             rot(n-j, index1(s+2*n, ss, j), ss[0], index2(r, rs,j, j), rs[1], c, sn)
 *             rot(m-1, u, us[0], col(q, qs, j), qs[0], c, sn.conjugate())
 *         m -= 1             # <<<<<<<<<<<<<<
 * 
 *     libc.stdlib.free(u)
 */
    __pyx_v_m = (__pyx_v_m - 1);
  }

  /* "scipy/linalg/_decomp_update.pyx":452
 *         m -= 1
 * 
 *     libc.stdlib.free(u)             # <<<<<<<<<<<<<<
 * 
 *     if p_full:
 */
  free(__pyx_v_u);

  /* "scipy/linalg/_decomp_update.pyx":454
 *     libc.stdlib.free(u)
 * 
 *     if p_full:             # <<<<<<<<<<<<<<
 *         qr_block_row_delete(m, n, q, qs, r, rs, k, p_full)
 *     return 1
 */
  __pyx_t_1 = (__pyx_v_p_full != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":455
 * 
 *     if p_full:
 *         qr_block_row_delete(m, n, q, qs, r, rs, k, p_full)             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_delete(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, __pyx_v_k, __pyx_v_p_full);

    /* "scipy/linalg/_decomp_update.pyx":454
 *     libc.stdlib.free(u)
 * 
 *     if p_full:             # <<<<<<<<<<<<<<
 *         qr_block_row_delete(m, n, q, qs, r, rs, k, p_full)
 *     return 1
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":456
 *     if p_full:
 *         qr_block_row_delete(m, n, q, qs, r, rs, k, p_full)
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * cdef void qr_block_row_delete(int m, int n, blas_t* q, int* qs,
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":397
 *     return True
 * 
 * cdef int thin_qr_row_delete(int m, int n, blas_t* q, int* qs, bint qisF,             # <<<<<<<<<<<<<<
 *                             blas_t* r, int* rs, int k, int p_eco,
 *                             int p_full) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_delete(int __pyx_v_m, int __pyx_v_n, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_qs, int __pyx_v_qisF, __pyx_t_float_complex *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p_eco, int __pyx_v_p_full) {
  CYTHON_UNUSED int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_argmin_row_norm;
  size_t __pyx_v_usize;
  __pyx_t_float_complex *__pyx_v_s;
  __pyx_t_float_complex *__pyx_v_u;
  int __pyx_v_us[2];
  int __pyx_v_ss[2];
  __pyx_t_float_complex __pyx_v_c;
  __pyx_t_float_complex __pyx_v_sn;
  __pyx_t_float_complex __pyx_v_min_row_norm;
  __pyx_t_float_complex __pyx_v_row_norm;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;

  /* "scipy/linalg/_decomp_update.pyx":401
 *                             int p_full) nogil:
 *     cdef int i, j, argmin_row_norm
 *     cdef size_t usize = (m + 3*n + 1) * sizeof(blas_t)             # <<<<<<<<<<<<<<
 *     cdef blas_t* s
 *     cdef blas_t* u
 */
  __pyx_v_usize = (((__pyx_v_m + (3 * __pyx_v_n)) + 1) * (sizeof(__pyx_t_float_complex)));

  /* "scipy/linalg/_decomp_update.pyx":409
 *     cdef blas_t c, sn, min_row_norm, row_norm
 * 
 *     u = <blas_t*>libc.stdlib.malloc(usize)             # <<<<<<<<<<<<<<
 *     if not u:
 *         return MEMORY_ERROR
 */
  __pyx_v_u = ((__pyx_t_float_complex *)malloc(__pyx_v_usize));

  /* "scipy/linalg/_decomp_update.pyx":410
 * 
 *     u = <blas_t*>libc.stdlib.malloc(usize)
 *     if not u:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 *     s = u + m
 */
  __pyx_t_1 = ((!(__pyx_v_u != 0)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":411
 *     u = <blas_t*>libc.stdlib.malloc(usize)
 *     if not u:
 *         return MEMORY_ERROR             # <<<<<<<<<<<<<<
 *     s = u + m
 *     ss[0] = 1
 */
    __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":410
 * 
 *     u = <blas_t*>libc.stdlib.malloc(usize)
 *     if not u:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 *     s = u + m
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":412
 *     if not u:
 *         return MEMORY_ERROR
 *     s = u + m             # <<<<<<<<<<<<<<
 *     ss[0] = 1
 *     ss[1] = 0
 */
  __pyx_v_s = (__pyx_v_u + __pyx_v_m);

  /* "scipy/linalg/_decomp_update.pyx":413
 *         return MEMORY_ERROR
 *     s = u + m
 *     ss[0] = 1             # <<<<<<<<<<<<<<
 *     ss[1] = 0
 *     us[0] = 1
 */
  (__pyx_v_ss[0]) = 1;

  /* "scipy/linalg/_decomp_update.pyx":414
 *     s = u + m
 *     ss[0] = 1
 *     ss[1] = 0             # <<<<<<<<<<<<<<
 *     us[0] = 1
 *     us[1] = 0
 */
  (__pyx_v_ss[1]) = 0;

  /* "scipy/linalg/_decomp_update.pyx":415
 *     ss[0] = 1
 *     ss[1] = 0
 *     us[0] = 1             # <<<<<<<<<<<<<<
 *     us[1] = 0
 * 
 */
  (__pyx_v_us[0]) = 1;

  /* "scipy/linalg/_decomp_update.pyx":416
 *     ss[1] = 0
 *     us[0] = 1
 *     us[1] = 0             # <<<<<<<<<<<<<<
 * 
 *     for i in range(p_eco):
 */
  (__pyx_v_us[1]) = 0;

  /* "scipy/linalg/_decomp_update.pyx":418
 *     us[1] = 0
 * 
 *     for i in range(p_eco):             # <<<<<<<<<<<<<<
 *         memset(u, 0, usize)
 *         # permute q such that row k is the last row.
 */
  __pyx_t_2 = __pyx_v_p_eco;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "scipy/linalg/_decomp_update.pyx":419
 * 
 *     for i in range(p_eco):
 *         memset(u, 0, usize)             # <<<<<<<<<<<<<<
 *         # permute q such that row k is the last row.
 *         if k != m-1:
 */
    (void)(memset(__pyx_v_u, 0, __pyx_v_usize));

    /* "scipy/linalg/_decomp_update.pyx":421
 *         memset(u, 0, usize)
 *         # permute q such that row k is the last row.
 *         if k != m-1:             # <<<<<<<<<<<<<<
 *             for j in range(k, m-1):
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])
 */
    __pyx_t_1 = ((__pyx_v_k != (__pyx_v_m - 1)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":422
 *         # permute q such that row k is the last row.
 *         if k != m-1:
 *             for j in range(k, m-1):             # <<<<<<<<<<<<<<
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])
 * 
 */
      __pyx_t_5 = (__pyx_v_m - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = __pyx_v_k; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
        __pyx_v_j = __pyx_t_7;

        /* "scipy/linalg/_decomp_update.pyx":423
 *         if k != m-1:
 *             for j in range(k, m-1):
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])             # <<<<<<<<<<<<<<
 * 
 *         if not reorthx(m, n, q, qs, qisF, m-1, u, s):
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_swap(__pyx_v_n, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[1]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[1]));
      }

      /* "scipy/linalg/_decomp_update.pyx":421
 *         memset(u, 0, usize)
 *         # permute q such that row k is the last row.
 *         if k != m-1:             # <<<<<<<<<<<<<<
 *             for j in range(k, m-1):
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":425
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])
 * 
 *         if not reorthx(m, n, q, qs, qisF, m-1, u, s):             # <<<<<<<<<<<<<<
 *             # if we get here it means that this basis vector lies in span(q).
 *             # we want to use s[:n+1] but we need a vector into null(q)
 */
    __pyx_t_1 = ((!(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_reorthx(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_qisF, (__pyx_v_m - 1), __pyx_v_u, __pyx_v_s) != 0)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":429
 *             # we want to use s[:n+1] but we need a vector into null(q)
 *             # find the row of q with the smallest norm and try that. (Daniel, p785)
 *             min_row_norm = nrm2(n, row(q, qs, 0), qs[1])             # <<<<<<<<<<<<<<
 *             argmin_row_norm = 0
 *             for j in range(1, m):
 */
      __pyx_v_min_row_norm = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_n, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, 0), (__pyx_v_qs[1]));

      /* "scipy/linalg/_decomp_update.pyx":430
 *             # find the row of q with the smallest norm and try that. (Daniel, p785)
 *             min_row_norm = nrm2(n, row(q, qs, 0), qs[1])
 *             argmin_row_norm = 0             # <<<<<<<<<<<<<<
 *             for j in range(1, m):
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])
 */
      __pyx_v_argmin_row_norm = 0;

      /* "scipy/linalg/_decomp_update.pyx":431
 *             min_row_norm = nrm2(n, row(q, qs, 0), qs[1])
 *             argmin_row_norm = 0
 *             for j in range(1, m):             # <<<<<<<<<<<<<<
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])
 *                 if blas_t_less_than(row_norm, min_row_norm):
 */
      __pyx_t_7 = __pyx_v_m;
      __pyx_t_8 = __pyx_t_7;
      for (__pyx_t_9 = 1; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
        __pyx_v_j = __pyx_t_9;

        /* "scipy/linalg/_decomp_update.pyx":432
 *             argmin_row_norm = 0
 *             for j in range(1, m):
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])             # <<<<<<<<<<<<<<
 *                 if blas_t_less_than(row_norm, min_row_norm):
 *                     min_row_norm = row_norm
 */
        __pyx_v_row_norm = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_n, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[1]));

        /* "scipy/linalg/_decomp_update.pyx":433
 *             for j in range(1, m):
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])
 *                 if blas_t_less_than(row_norm, min_row_norm):             # <<<<<<<<<<<<<<
 *                     min_row_norm = row_norm
 *                     argmin_row_norm = j
 */
        __pyx_t_1 = (__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_v_row_norm, __pyx_v_min_row_norm) != 0);
        if (__pyx_t_1) {

          /* "scipy/linalg/_decomp_update.pyx":434
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])
 *                 if blas_t_less_than(row_norm, min_row_norm):
 *                     min_row_norm = row_norm             # <<<<<<<<<<<<<<
 *                     argmin_row_norm = j
 *             memset(u, 0, m*sizeof(blas_t))
 */
          __pyx_v_min_row_norm = __pyx_v_row_norm;

          /* "scipy/linalg/_decomp_update.pyx":435
 *                 if blas_t_less_than(row_norm, min_row_norm):
 *                     min_row_norm = row_norm
 *                     argmin_row_norm = j             # <<<<<<<<<<<<<<
 *             memset(u, 0, m*sizeof(blas_t))
 *             if not reorthx(m, n, q, qs, qisF, argmin_row_norm, u, s):
 */
          __pyx_v_argmin_row_norm = __pyx_v_j;

          /* "scipy/linalg/_decomp_update.pyx":433
 *             for j in range(1, m):
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])
 *                 if blas_t_less_than(row_norm, min_row_norm):             # <<<<<<<<<<<<<<
 *                     min_row_norm = row_norm
 *                     argmin_row_norm = j
 */
        }
      }

      /* "scipy/linalg/_decomp_update.pyx":436
 *                     min_row_norm = row_norm
 *                     argmin_row_norm = j
 *             memset(u, 0, m*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *             if not reorthx(m, n, q, qs, qisF, argmin_row_norm, u, s):
 *                 # failed, quit.
 */
      (void)(memset(__pyx_v_u, 0, (__pyx_v_m * (sizeof(__pyx_t_float_complex)))));

      /* "scipy/linalg/_decomp_update.pyx":437
 *                     argmin_row_norm = j
 *             memset(u, 0, m*sizeof(blas_t))
 *             if not reorthx(m, n, q, qs, qisF, argmin_row_norm, u, s):             # <<<<<<<<<<<<<<
 *                 # failed, quit.
 *                 libc.stdlib.free(u)
 */
      __pyx_t_1 = ((!(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_reorthx(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_qisF, __pyx_v_argmin_row_norm, __pyx_v_u, __pyx_v_s) != 0)) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":439
 *             if not reorthx(m, n, q, qs, qisF, argmin_row_norm, u, s):
 *                 # failed, quit.
 *                 libc.stdlib.free(u)             # <<<<<<<<<<<<<<
 *                 return 0
 *             s[n] = 0
 */
        free(__pyx_v_u);

        /* "scipy/linalg/_decomp_update.pyx":440
 *                 # failed, quit.
 *                 libc.stdlib.free(u)
 *                 return 0             # <<<<<<<<<<<<<<
 *             s[n] = 0
 * 
 */
        __pyx_r = 0;
        goto __pyx_L0;

        /* "scipy/linalg/_decomp_update.pyx":437
 *                     argmin_row_norm = j
 *             memset(u, 0, m*sizeof(blas_t))
 *             if not reorthx(m, n, q, qs, qisF, argmin_row_norm, u, s):             # <<<<<<<<<<<<<<
 *                 # failed, quit.
 *                 libc.stdlib.free(u)
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":441
 *                 libc.stdlib.free(u)
 *                 return 0
 *             s[n] = 0             # <<<<<<<<<<<<<<
 * 
 *         memset(s+2*n, 0, n*sizeof(blas_t))
 */
      (__pyx_v_s[__pyx_v_n]) = __pyx_t_float_complex_from_parts(0, 0);

      /* "scipy/linalg/_decomp_update.pyx":425
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])
 * 
 *         if not reorthx(m, n, q, qs, qisF, m-1, u, s):             # <<<<<<<<<<<<<<
 *             # if we get here it means that this basis vector lies in span(q).
 *             # we want to use s[:n+1] but we need a vector into null(q)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":443
 *             s[n] = 0
 * 
 *         memset(s+2*n, 0, n*sizeof(blas_t))             # <<<<<<<<<<<<<<
 * 
 *         # what happens here...
 */
    (void)(memset((__pyx_v_s + (2 * __pyx_v_n)), 0, (__pyx_v_n * (sizeof(__pyx_t_float_complex)))));

    /* "scipy/linalg/_decomp_update.pyx":446
 * 
 *         # what happens here...
 *         for j in range(n-1, -1, -1):             # <<<<<<<<<<<<<<
 *             lartg(index1(s, ss, n), index1(s, ss, j), &c, &sn)
 *             rot(n-j, index1(s+2*n, ss, j), ss[0], index2(r, rs,j, j), rs[1], c, sn)
 */
    for (__pyx_t_7 = (__pyx_v_n - 1); __pyx_t_7 > -1; __pyx_t_7-=1) {
      __pyx_v_j = __pyx_t_7;

      /* "scipy/linalg/_decomp_update.pyx":447
 *         # what happens here...
 *         for j in range(n-1, -1, -1):
 *             lartg(index1(s, ss, n), index1(s, ss, j), &c, &sn)             # <<<<<<<<<<<<<<
 *             rot(n-j, index1(s+2*n, ss, j), ss[0], index2(r, rs,j, j), rs[1], c, sn)
 *             rot(m-1, u, us[0], col(q, qs, j), qs[0], c, sn.conjugate())
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, __pyx_v_n), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, __pyx_v_j), (&__pyx_v_c), (&__pyx_v_sn));

      /* "scipy/linalg/_decomp_update.pyx":448
 *         for j in range(n-1, -1, -1):
 *             lartg(index1(s, ss, n), index1(s, ss, j), &c, &sn)
 *             rot(n-j, index1(s+2*n, ss, j), ss[0], index2(r, rs,j, j), rs[1], c, sn)             # <<<<<<<<<<<<<<
 *             rot(m-1, u, us[0], col(q, qs, j), qs[0], c, sn.conjugate())
 *         m -= 1
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_n - __pyx_v_j), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index1((__pyx_v_s + (2 * __pyx_v_n)), __pyx_v_ss, __pyx_v_j), (__pyx_v_ss[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_sn);

      /* "scipy/linalg/_decomp_update.pyx":449
 *             lartg(index1(s, ss, n), index1(s, ss, j), &c, &sn)
 *             rot(n-j, index1(s+2*n, ss, j), ss[0], index2(r, rs,j, j), rs[1], c, sn)
 *             rot(m-1, u, us[0], col(q, qs, j), qs[0], c, sn.conjugate())             # <<<<<<<<<<<<<<
 *         m -= 1
 * 
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_m - 1), __pyx_v_u, (__pyx_v_us[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_float(__pyx_v_sn));
    }

    /* "scipy/linalg/_decomp_update.pyx":450
 *             rot(n-j, index1(s+2*n, ss, j), ss[0], index2(r, rs,j, j), rs[1], c, sn)
 *             rot(m-1, u, us[0], col(q, qs, j), qs[0], c, sn.conjugate())
 *         m -= 1             # <<<<<<<<<<<<<<
 * 
 *     libc.stdlib.free(u)
 */
    __pyx_v_m = (__pyx_v_m - 1);
  }

  /* "scipy/linalg/_decomp_update.pyx":452
 *         m -= 1
 * 
 *     libc.stdlib.free(u)             # <<<<<<<<<<<<<<
 * 
 *     if p_full:
 */
  free(__pyx_v_u);

  /* "scipy/linalg/_decomp_update.pyx":454
 *     libc.stdlib.free(u)
 * 
 *     if p_full:             # <<<<<<<<<<<<<<
 *         qr_block_row_delete(m, n, q, qs, r, rs, k, p_full)
 *     return 1
 */
  __pyx_t_1 = (__pyx_v_p_full != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":455
 * 
 *     if p_full:
 *         qr_block_row_delete(m, n, q, qs, r, rs, k, p_full)             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_delete(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, __pyx_v_k, __pyx_v_p_full);

    /* "scipy/linalg/_decomp_update.pyx":454
 *     libc.stdlib.free(u)
 * 
 *     if p_full:             # <<<<<<<<<<<<<<
 *         qr_block_row_delete(m, n, q, qs, r, rs, k, p_full)
 *     return 1
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":456
 *     if p_full:
 *         qr_block_row_delete(m, n, q, qs, r, rs, k, p_full)
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * cdef void qr_block_row_delete(int m, int n, blas_t* q, int* qs,
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":397
 *     return True
 * 
 * cdef int thin_qr_row_delete(int m, int n, blas_t* q, int* qs, bint qisF,             # <<<<<<<<<<<<<<
 *                             blas_t* r, int* rs, int k, int p_eco,
 *                             int p_full) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_delete(int __pyx_v_m, int __pyx_v_n, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_qs, int __pyx_v_qisF, __pyx_t_double_complex *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p_eco, int __pyx_v_p_full) {
  CYTHON_UNUSED int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_argmin_row_norm;
  size_t __pyx_v_usize;
  __pyx_t_double_complex *__pyx_v_s;
  __pyx_t_double_complex *__pyx_v_u;
  int __pyx_v_us[2];
  int __pyx_v_ss[2];
  __pyx_t_double_complex __pyx_v_c;
  __pyx_t_double_complex __pyx_v_sn;
  __pyx_t_double_complex __pyx_v_min_row_norm;
  __pyx_t_double_complex __pyx_v_row_norm;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;

  /* "scipy/linalg/_decomp_update.pyx":401
 *                             int p_full) nogil:
 *     cdef int i, j, argmin_row_norm
 *     cdef size_t usize = (m + 3*n + 1) * sizeof(blas_t)             # <<<<<<<<<<<<<<
 *     cdef blas_t* s
 *     cdef blas_t* u
 */
  __pyx_v_usize = (((__pyx_v_m + (3 * __pyx_v_n)) + 1) * (sizeof(__pyx_t_double_complex)));

  /* "scipy/linalg/_decomp_update.pyx":409
 *     cdef blas_t c, sn, min_row_norm, row_norm
 * 
 *     u = <blas_t*>libc.stdlib.malloc(usize)             # <<<<<<<<<<<<<<
 *     if not u:
 *         return MEMORY_ERROR
 */
  __pyx_v_u = ((__pyx_t_double_complex *)malloc(__pyx_v_usize));

  /* "scipy/linalg/_decomp_update.pyx":410
 * 
 *     u = <blas_t*>libc.stdlib.malloc(usize)
 *     if not u:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 *     s = u + m
 */
  __pyx_t_1 = ((!(__pyx_v_u != 0)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":411
 *     u = <blas_t*>libc.stdlib.malloc(usize)
 *     if not u:
 *         return MEMORY_ERROR             # <<<<<<<<<<<<<<
 *     s = u + m
 *     ss[0] = 1
 */
    __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":410
 * 
 *     u = <blas_t*>libc.stdlib.malloc(usize)
 *     if not u:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 *     s = u + m
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":412
 *     if not u:
 *         return MEMORY_ERROR
 *     s = u + m             # <<<<<<<<<<<<<<
 *     ss[0] = 1
 *     ss[1] = 0
 */
  __pyx_v_s = (__pyx_v_u + __pyx_v_m);

  /* "scipy/linalg/_decomp_update.pyx":413
 *         return MEMORY_ERROR
 *     s = u + m
 *     ss[0] = 1             # <<<<<<<<<<<<<<
 *     ss[1] = 0
 *     us[0] = 1
 */
  (__pyx_v_ss[0]) = 1;

  /* "scipy/linalg/_decomp_update.pyx":414
 *     s = u + m
 *     ss[0] = 1
 *     ss[1] = 0             # <<<<<<<<<<<<<<
 *     us[0] = 1
 *     us[1] = 0
 */
  (__pyx_v_ss[1]) = 0;

  /* "scipy/linalg/_decomp_update.pyx":415
 *     ss[0] = 1
 *     ss[1] = 0
 *     us[0] = 1             # <<<<<<<<<<<<<<
 *     us[1] = 0
 * 
 */
  (__pyx_v_us[0]) = 1;

  /* "scipy/linalg/_decomp_update.pyx":416
 *     ss[1] = 0
 *     us[0] = 1
 *     us[1] = 0             # <<<<<<<<<<<<<<
 * 
 *     for i in range(p_eco):
 */
  (__pyx_v_us[1]) = 0;

  /* "scipy/linalg/_decomp_update.pyx":418
 *     us[1] = 0
 * 
 *     for i in range(p_eco):             # <<<<<<<<<<<<<<
 *         memset(u, 0, usize)
 *         # permute q such that row k is the last row.
 */
  __pyx_t_2 = __pyx_v_p_eco;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "scipy/linalg/_decomp_update.pyx":419
 * 
 *     for i in range(p_eco):
 *         memset(u, 0, usize)             # <<<<<<<<<<<<<<
 *         # permute q such that row k is the last row.
 *         if k != m-1:
 */
    (void)(memset(__pyx_v_u, 0, __pyx_v_usize));

    /* "scipy/linalg/_decomp_update.pyx":421
 *         memset(u, 0, usize)
 *         # permute q such that row k is the last row.
 *         if k != m-1:             # <<<<<<<<<<<<<<
 *             for j in range(k, m-1):
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])
 */
    __pyx_t_1 = ((__pyx_v_k != (__pyx_v_m - 1)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":422
 *         # permute q such that row k is the last row.
 *         if k != m-1:
 *             for j in range(k, m-1):             # <<<<<<<<<<<<<<
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])
 * 
 */
      __pyx_t_5 = (__pyx_v_m - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = __pyx_v_k; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
        __pyx_v_j = __pyx_t_7;

        /* "scipy/linalg/_decomp_update.pyx":423
 *         if k != m-1:
 *             for j in range(k, m-1):
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])             # <<<<<<<<<<<<<<
 * 
 *         if not reorthx(m, n, q, qs, qisF, m-1, u, s):
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_swap(__pyx_v_n, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[1]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[1]));
      }

      /* "scipy/linalg/_decomp_update.pyx":421
 *         memset(u, 0, usize)
 *         # permute q such that row k is the last row.
 *         if k != m-1:             # <<<<<<<<<<<<<<
 *             for j in range(k, m-1):
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":425
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])
 * 
 *         if not reorthx(m, n, q, qs, qisF, m-1, u, s):             # <<<<<<<<<<<<<<
 *             # if we get here it means that this basis vector lies in span(q).
 *             # we want to use s[:n+1] but we need a vector into null(q)
 */
    __pyx_t_1 = ((!(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_reorthx(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_qisF, (__pyx_v_m - 1), __pyx_v_u, __pyx_v_s) != 0)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":429
 *             # we want to use s[:n+1] but we need a vector into null(q)
 *             # find the row of q with the smallest norm and try that. (Daniel, p785)
 *             min_row_norm = nrm2(n, row(q, qs, 0), qs[1])             # <<<<<<<<<<<<<<
 *             argmin_row_norm = 0
 *             for j in range(1, m):
 */
      __pyx_v_min_row_norm = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_n, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, 0), (__pyx_v_qs[1]));

      /* "scipy/linalg/_decomp_update.pyx":430
 *             # find the row of q with the smallest norm and try that. (Daniel, p785)
 *             min_row_norm = nrm2(n, row(q, qs, 0), qs[1])
 *             argmin_row_norm = 0             # <<<<<<<<<<<<<<
 *             for j in range(1, m):
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])
 */
      __pyx_v_argmin_row_norm = 0;

      /* "scipy/linalg/_decomp_update.pyx":431
 *             min_row_norm = nrm2(n, row(q, qs, 0), qs[1])
 *             argmin_row_norm = 0
 *             for j in range(1, m):             # <<<<<<<<<<<<<<
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])
 *                 if blas_t_less_than(row_norm, min_row_norm):
 */
      __pyx_t_7 = __pyx_v_m;
      __pyx_t_8 = __pyx_t_7;
      for (__pyx_t_9 = 1; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
        __pyx_v_j = __pyx_t_9;

        /* "scipy/linalg/_decomp_update.pyx":432
 *             argmin_row_norm = 0
 *             for j in range(1, m):
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])             # <<<<<<<<<<<<<<
 *                 if blas_t_less_than(row_norm, min_row_norm):
 *                     min_row_norm = row_norm
 */
        __pyx_v_row_norm = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_n, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[1]));

        /* "scipy/linalg/_decomp_update.pyx":433
 *             for j in range(1, m):
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])
 *                 if blas_t_less_than(row_norm, min_row_norm):             # <<<<<<<<<<<<<<
 *                     min_row_norm = row_norm
 *                     argmin_row_norm = j
 */
        __pyx_t_1 = (__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_v_row_norm, __pyx_v_min_row_norm) != 0);
        if (__pyx_t_1) {

          /* "scipy/linalg/_decomp_update.pyx":434
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])
 *                 if blas_t_less_than(row_norm, min_row_norm):
 *                     min_row_norm = row_norm             # <<<<<<<<<<<<<<
 *                     argmin_row_norm = j
 *             memset(u, 0, m*sizeof(blas_t))
 */
          __pyx_v_min_row_norm = __pyx_v_row_norm;

          /* "scipy/linalg/_decomp_update.pyx":435
 *                 if blas_t_less_than(row_norm, min_row_norm):
 *                     min_row_norm = row_norm
 *                     argmin_row_norm = j             # <<<<<<<<<<<<<<
 *             memset(u, 0, m*sizeof(blas_t))
 *             if not reorthx(m, n, q, qs, qisF, argmin_row_norm, u, s):
 */
          __pyx_v_argmin_row_norm = __pyx_v_j;

          /* "scipy/linalg/_decomp_update.pyx":433
 *             for j in range(1, m):
 *                 row_norm = nrm2(n, row(q, qs, j), qs[1])
 *                 if blas_t_less_than(row_norm, min_row_norm):             # <<<<<<<<<<<<<<
 *                     min_row_norm = row_norm
 *                     argmin_row_norm = j
 */
        }
      }

      /* "scipy/linalg/_decomp_update.pyx":436
 *                     min_row_norm = row_norm
 *                     argmin_row_norm = j
 *             memset(u, 0, m*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *             if not reorthx(m, n, q, qs, qisF, argmin_row_norm, u, s):
 *                 # failed, quit.
 */
      (void)(memset(__pyx_v_u, 0, (__pyx_v_m * (sizeof(__pyx_t_double_complex)))));

      /* "scipy/linalg/_decomp_update.pyx":437
 *                     argmin_row_norm = j
 *             memset(u, 0, m*sizeof(blas_t))
 *             if not reorthx(m, n, q, qs, qisF, argmin_row_norm, u, s):             # <<<<<<<<<<<<<<
 *                 # failed, quit.
 *                 libc.stdlib.free(u)
 */
      __pyx_t_1 = ((!(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_reorthx(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_qisF, __pyx_v_argmin_row_norm, __pyx_v_u, __pyx_v_s) != 0)) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":439
 *             if not reorthx(m, n, q, qs, qisF, argmin_row_norm, u, s):
 *                 # failed, quit.
 *                 libc.stdlib.free(u)             # <<<<<<<<<<<<<<
 *                 return 0
 *             s[n] = 0
 */
        free(__pyx_v_u);

        /* "scipy/linalg/_decomp_update.pyx":440
 *                 # failed, quit.
 *                 libc.stdlib.free(u)
 *                 return 0             # <<<<<<<<<<<<<<
 *             s[n] = 0
 * 
 */
        __pyx_r = 0;
        goto __pyx_L0;

        /* "scipy/linalg/_decomp_update.pyx":437
 *                     argmin_row_norm = j
 *             memset(u, 0, m*sizeof(blas_t))
 *             if not reorthx(m, n, q, qs, qisF, argmin_row_norm, u, s):             # <<<<<<<<<<<<<<
 *                 # failed, quit.
 *                 libc.stdlib.free(u)
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":441
 *                 libc.stdlib.free(u)
 *                 return 0
 *             s[n] = 0             # <<<<<<<<<<<<<<
 * 
 *         memset(s+2*n, 0, n*sizeof(blas_t))
 */
      (__pyx_v_s[__pyx_v_n]) = __pyx_t_double_complex_from_parts(0, 0);

      /* "scipy/linalg/_decomp_update.pyx":425
 *                 swap(n, row(q, qs, j), qs[1], row(q, qs, j+1), qs[1])
 * 
 *         if not reorthx(m, n, q, qs, qisF, m-1, u, s):             # <<<<<<<<<<<<<<
 *             # if we get here it means that this basis vector lies in span(q).
 *             # we want to use s[:n+1] but we need a vector into null(q)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":443
 *             s[n] = 0
 * 
 *         memset(s+2*n, 0, n*sizeof(blas_t))             # <<<<<<<<<<<<<<
 * 
 *         # what happens here...
 */
    (void)(memset((__pyx_v_s + (2 * __pyx_v_n)), 0, (__pyx_v_n * (sizeof(__pyx_t_double_complex)))));

    /* "scipy/linalg/_decomp_update.pyx":446
 * 
 *         # what happens here...
 *         for j in range(n-1, -1, -1):             # <<<<<<<<<<<<<<
 *             lartg(index1(s, ss, n), index1(s, ss, j), &c, &sn)
 *             rot(n-j, index1(s+2*n, ss, j), ss[0], index2(r, rs,j, j), rs[1], c, sn)
 */
    for (__pyx_t_7 = (__pyx_v_n - 1); __pyx_t_7 > -1; __pyx_t_7-=1) {
      __pyx_v_j = __pyx_t_7;

      /* "scipy/linalg/_decomp_update.pyx":447
 *         # what happens here...
 *         for j in range(n-1, -1, -1):
 *             lartg(index1(s, ss, n), index1(s, ss, j), &c, &sn)             # <<<<<<<<<<<<<<
 *             rot(n-j, index1(s+2*n, ss, j), ss[0], index2(r, rs,j, j), rs[1], c, sn)
 *             rot(m-1, u, us[0], col(q, qs, j), qs[0], c, sn.conjugate())
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, __pyx_v_n), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, __pyx_v_j), (&__pyx_v_c), (&__pyx_v_sn));

      /* "scipy/linalg/_decomp_update.pyx":448
 *         for j in range(n-1, -1, -1):
 *             lartg(index1(s, ss, n), index1(s, ss, j), &c, &sn)
 *             rot(n-j, index1(s+2*n, ss, j), ss[0], index2(r, rs,j, j), rs[1], c, sn)             # <<<<<<<<<<<<<<
 *             rot(m-1, u, us[0], col(q, qs, j), qs[0], c, sn.conjugate())
 *         m -= 1
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_n - __pyx_v_j), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index1((__pyx_v_s + (2 * __pyx_v_n)), __pyx_v_ss, __pyx_v_j), (__pyx_v_ss[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_sn);

      /* "scipy/linalg/_decomp_update.pyx":449
 *             lartg(index1(s, ss, n), index1(s, ss, j), &c, &sn)
 *             rot(n-j, index1(s+2*n, ss, j), ss[0], index2(r, rs,j, j), rs[1], c, sn)
 *             rot(m-1, u, us[0], col(q, qs, j), qs[0], c, sn.conjugate())             # <<<<<<<<<<<<<<
 *         m -= 1
 * 
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_m - 1), __pyx_v_u, (__pyx_v_us[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_double(__pyx_v_sn));
    }

    /* "scipy/linalg/_decomp_update.pyx":450
 *             rot(n-j, index1(s+2*n, ss, j), ss[0], index2(r, rs,j, j), rs[1], c, sn)
 *             rot(m-1, u, us[0], col(q, qs, j), qs[0], c, sn.conjugate())
 *         m -= 1             # <<<<<<<<<<<<<<
 * 
 *     libc.stdlib.free(u)
 */
    __pyx_v_m = (__pyx_v_m - 1);
  }

  /* "scipy/linalg/_decomp_update.pyx":452
 *         m -= 1
 * 
 *     libc.stdlib.free(u)             # <<<<<<<<<<<<<<
 * 
 *     if p_full:
 */
  free(__pyx_v_u);

  /* "scipy/linalg/_decomp_update.pyx":454
 *     libc.stdlib.free(u)
 * 
 *     if p_full:             # <<<<<<<<<<<<<<
 *         qr_block_row_delete(m, n, q, qs, r, rs, k, p_full)
 *     return 1
 */
  __pyx_t_1 = (__pyx_v_p_full != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":455
 * 
 *     if p_full:
 *         qr_block_row_delete(m, n, q, qs, r, rs, k, p_full)             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_delete(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, __pyx_v_k, __pyx_v_p_full);

    /* "scipy/linalg/_decomp_update.pyx":454
 *     libc.stdlib.free(u)
 * 
 *     if p_full:             # <<<<<<<<<<<<<<
 *         qr_block_row_delete(m, n, q, qs, r, rs, k, p_full)
 *     return 1
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":456
 *     if p_full:
 *         qr_block_row_delete(m, n, q, qs, r, rs, k, p_full)
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * cdef void qr_block_row_delete(int m, int n, blas_t* q, int* qs,
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":397
 *     return True
 * 
 * cdef int thin_qr_row_delete(int m, int n, blas_t* q, int* qs, bint qisF,             # <<<<<<<<<<<<<<
 *                             blas_t* r, int* rs, int k, int p_eco,
 *                             int p_full) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":458
 *     return 1
 * 
 * cdef void qr_block_row_delete(int m, int n, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *                               blas_t* r, int* rs, int k, int p) nogil:
 *     cdef int i, j
 */

static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_delete(int __pyx_v_m, int __pyx_v_n, float *__pyx_v_q, int *__pyx_v_qs, float *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p) {
  int __pyx_v_i;
  int __pyx_v_j;
  float __pyx_v_c;
  float __pyx_v_s;
  float *__pyx_v_W;
  int *__pyx_v_ws;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;

  /* "scipy/linalg/_decomp_update.pyx":465
 *     cdef int* ws
 * 
 *     if k != 0:             # <<<<<<<<<<<<<<
 *         for j in range(k, 0, -1):
 *             swap(m, row(q, qs, j+p-1), qs[1], row(q, qs, j-1), qs[1])
 */
  __pyx_t_1 = ((__pyx_v_k != 0) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":466
 * 
 *     if k != 0:
 *         for j in range(k, 0, -1):             # <<<<<<<<<<<<<<
 *             swap(m, row(q, qs, j+p-1), qs[1], row(q, qs, j-1), qs[1])
 * 
 */
    for (__pyx_t_2 = __pyx_v_k; __pyx_t_2 > 0; __pyx_t_2-=1) {
      __pyx_v_j = __pyx_t_2;

      /* "scipy/linalg/_decomp_update.pyx":467
 *     if k != 0:
 *         for j in range(k, 0, -1):
 *             swap(m, row(q, qs, j+p-1), qs[1], row(q, qs, j-1), qs[1])             # <<<<<<<<<<<<<<
 * 
 *     # W is the block of rows to be removed from q, has shape, (p,m)
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_swap(__pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, ((__pyx_v_j + __pyx_v_p) - 1)), (__pyx_v_qs[1]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, (__pyx_v_j - 1)), (__pyx_v_qs[1]));
    }

    /* "scipy/linalg/_decomp_update.pyx":465
 *     cdef int* ws
 * 
 *     if k != 0:             # <<<<<<<<<<<<<<
 *         for j in range(k, 0, -1):
 *             swap(m, row(q, qs, j+p-1), qs[1], row(q, qs, j-1), qs[1])
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":470
 * 
 *     # W is the block of rows to be removed from q, has shape, (p,m)
 *     W = q             # <<<<<<<<<<<<<<
 *     ws = qs
 * 
 */
  __pyx_v_W = __pyx_v_q;

  /* "scipy/linalg/_decomp_update.pyx":471
 *     # W is the block of rows to be removed from q, has shape, (p,m)
 *     W = q
 *     ws = qs             # <<<<<<<<<<<<<<
 * 
 *     for j in range(p):
 */
  __pyx_v_ws = __pyx_v_qs;

  /* "scipy/linalg/_decomp_update.pyx":473
 *     ws = qs
 * 
 *     for j in range(p):             # <<<<<<<<<<<<<<
 *         blas_t_conj(m, row(W, ws, j), &ws[1])
 * 
 */
  __pyx_t_2 = __pyx_v_p;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_j = __pyx_t_4;

    /* "scipy/linalg/_decomp_update.pyx":474
 * 
 *     for j in range(p):
 *         blas_t_conj(m, row(W, ws, j), &ws[1])             # <<<<<<<<<<<<<<
 * 
 *     for i in range(p):
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_W, __pyx_v_ws, __pyx_v_j), (&(__pyx_v_ws[1])));
  }

  /* "scipy/linalg/_decomp_update.pyx":476
 *         blas_t_conj(m, row(W, ws, j), &ws[1])
 * 
 *     for i in range(p):             # <<<<<<<<<<<<<<
 *         for j in range(m-2, i-1, -1):
 *             lartg(index2(W, ws, i, j), index2(W, ws, i, j+1), &c, &s)
 */
  __pyx_t_2 = __pyx_v_p;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "scipy/linalg/_decomp_update.pyx":477
 * 
 *     for i in range(p):
 *         for j in range(m-2, i-1, -1):             # <<<<<<<<<<<<<<
 *             lartg(index2(W, ws, i, j), index2(W, ws, i, j+1), &c, &s)
 * 
 */
    __pyx_t_5 = (__pyx_v_i - 1);
    __pyx_t_6 = __pyx_t_5;
    for (__pyx_t_7 = (__pyx_v_m - 2); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
      __pyx_v_j = __pyx_t_7;

      /* "scipy/linalg/_decomp_update.pyx":478
 *     for i in range(p):
 *         for j in range(m-2, i-1, -1):
 *             lartg(index2(W, ws, i, j), index2(W, ws, i, j+1), &c, &s)             # <<<<<<<<<<<<<<
 * 
 *             # update W
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_W, __pyx_v_ws, __pyx_v_i, __pyx_v_j), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_W, __pyx_v_ws, __pyx_v_i, (__pyx_v_j + 1)), (&__pyx_v_c), (&__pyx_v_s));

      /* "scipy/linalg/_decomp_update.pyx":481
 * 
 *             # update W
 *             if i+1 < p:             # <<<<<<<<<<<<<<
 *                 rot(p-i-1, index2(W, ws, i+1, j), ws[0],
 *                     index2(W, ws, i+1, j+1), ws[0], c, s)
 */
      __pyx_t_1 = (((__pyx_v_i + 1) < __pyx_v_p) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":482
 *             # update W
 *             if i+1 < p:
 *                 rot(p-i-1, index2(W, ws, i+1, j), ws[0],             # <<<<<<<<<<<<<<
 *                     index2(W, ws, i+1, j+1), ws[0], c, s)
 * 
 */
        __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_p - __pyx_v_i) - 1), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_W, __pyx_v_ws, (__pyx_v_i + 1), __pyx_v_j), (__pyx_v_ws[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_W, __pyx_v_ws, (__pyx_v_i + 1), (__pyx_v_j + 1)), (__pyx_v_ws[0]), __pyx_v_c, __pyx_v_s);

        /* "scipy/linalg/_decomp_update.pyx":481
 * 
 *             # update W
 *             if i+1 < p:             # <<<<<<<<<<<<<<
 *                 rot(p-i-1, index2(W, ws, i+1, j), ws[0],
 *                     index2(W, ws, i+1, j+1), ws[0], c, s)
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":486
 * 
 *             # update r if there is a nonzero row.
 *             if j-i < n:             # <<<<<<<<<<<<<<
 *                 rot(n-j+i, index2(r, rs, j, j-i), rs[1],
 *                     index2(r, rs, j+1, j-i), rs[1], c, s)
 */
      __pyx_t_1 = (((__pyx_v_j - __pyx_v_i) < __pyx_v_n) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":487
 *             # update r if there is a nonzero row.
 *             if j-i < n:
 *                 rot(n-j+i, index2(r, rs, j, j-i), rs[1],             # <<<<<<<<<<<<<<
 *                     index2(r, rs, j+1, j-i), rs[1], c, s)
 * 
 */
        __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) + __pyx_v_i), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j - __pyx_v_i)), (__pyx_v_rs[1]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_j - __pyx_v_i)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

        /* "scipy/linalg/_decomp_update.pyx":486
 * 
 *             # update r if there is a nonzero row.
 *             if j-i < n:             # <<<<<<<<<<<<<<
 *                 rot(n-j+i, index2(r, rs, j, j-i), rs[1],
 *                     index2(r, rs, j+1, j-i), rs[1], c, s)
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":491
 * 
 *             # update q
 *             rot(m-p, index2(q, qs, p, j), qs[0], index2(q, qs, p, j+1), qs[0],             # <<<<<<<<<<<<<<
 *                 c, s.conjugate())
 * 
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_m - __pyx_v_p), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_p, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_p, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_s));
    }
  }

  /* "scipy/linalg/_decomp_update.pyx":458
 *     return 1
 * 
 * cdef void qr_block_row_delete(int m, int n, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *                               blas_t* r, int* rs, int k, int p) nogil:
 *     cdef int i, j
 */

  /* function exit code */
}

static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_delete(int __pyx_v_m, int __pyx_v_n, double *__pyx_v_q, int *__pyx_v_qs, double *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p) {
  int __pyx_v_i;
  int __pyx_v_j;
  double __pyx_v_c;
  double __pyx_v_s;
  double *__pyx_v_W;
  int *__pyx_v_ws;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;

  /* "scipy/linalg/_decomp_update.pyx":465
 *     cdef int* ws
 * 
 *     if k != 0:             # <<<<<<<<<<<<<<
 *         for j in range(k, 0, -1):
 *             swap(m, row(q, qs, j+p-1), qs[1], row(q, qs, j-1), qs[1])
 */
  __pyx_t_1 = ((__pyx_v_k != 0) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":466
 * 
 *     if k != 0:
 *         for j in range(k, 0, -1):             # <<<<<<<<<<<<<<
 *             swap(m, row(q, qs, j+p-1), qs[1], row(q, qs, j-1), qs[1])
 * 
 */
    for (__pyx_t_2 = __pyx_v_k; __pyx_t_2 > 0; __pyx_t_2-=1) {
      __pyx_v_j = __pyx_t_2;

      /* "scipy/linalg/_decomp_update.pyx":467
 *     if k != 0:
 *         for j in range(k, 0, -1):
 *             swap(m, row(q, qs, j+p-1), qs[1], row(q, qs, j-1), qs[1])             # <<<<<<<<<<<<<<
 * 
 *     # W is the block of rows to be removed from q, has shape, (p,m)
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_swap(__pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, ((__pyx_v_j + __pyx_v_p) - 1)), (__pyx_v_qs[1]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, (__pyx_v_j - 1)), (__pyx_v_qs[1]));
    }

    /* "scipy/linalg/_decomp_update.pyx":465
 *     cdef int* ws
 * 
 *     if k != 0:             # <<<<<<<<<<<<<<
 *         for j in range(k, 0, -1):
 *             swap(m, row(q, qs, j+p-1), qs[1], row(q, qs, j-1), qs[1])
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":470
 * 
 *     # W is the block of rows to be removed from q, has shape, (p,m)
 *     W = q             # <<<<<<<<<<<<<<
 *     ws = qs
 * 
 */
  __pyx_v_W = __pyx_v_q;

  /* "scipy/linalg/_decomp_update.pyx":471
 *     # W is the block of rows to be removed from q, has shape, (p,m)
 *     W = q
 *     ws = qs             # <<<<<<<<<<<<<<
 * 
 *     for j in range(p):
 */
  __pyx_v_ws = __pyx_v_qs;

  /* "scipy/linalg/_decomp_update.pyx":473
 *     ws = qs
 * 
 *     for j in range(p):             # <<<<<<<<<<<<<<
 *         blas_t_conj(m, row(W, ws, j), &ws[1])
 * 
 */
  __pyx_t_2 = __pyx_v_p;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_j = __pyx_t_4;

    /* "scipy/linalg/_decomp_update.pyx":474
 * 
 *     for j in range(p):
 *         blas_t_conj(m, row(W, ws, j), &ws[1])             # <<<<<<<<<<<<<<
 * 
 *     for i in range(p):
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_W, __pyx_v_ws, __pyx_v_j), (&(__pyx_v_ws[1])));
  }

  /* "scipy/linalg/_decomp_update.pyx":476
 *         blas_t_conj(m, row(W, ws, j), &ws[1])
 * 
 *     for i in range(p):             # <<<<<<<<<<<<<<
 *         for j in range(m-2, i-1, -1):
 *             lartg(index2(W, ws, i, j), index2(W, ws, i, j+1), &c, &s)
 */
  __pyx_t_2 = __pyx_v_p;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "scipy/linalg/_decomp_update.pyx":477
 * 
 *     for i in range(p):
 *         for j in range(m-2, i-1, -1):             # <<<<<<<<<<<<<<
 *             lartg(index2(W, ws, i, j), index2(W, ws, i, j+1), &c, &s)
 * 
 */
    __pyx_t_5 = (__pyx_v_i - 1);
    __pyx_t_6 = __pyx_t_5;
    for (__pyx_t_7 = (__pyx_v_m - 2); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
      __pyx_v_j = __pyx_t_7;

      /* "scipy/linalg/_decomp_update.pyx":478
 *     for i in range(p):
 *         for j in range(m-2, i-1, -1):
 *             lartg(index2(W, ws, i, j), index2(W, ws, i, j+1), &c, &s)             # <<<<<<<<<<<<<<
 * 
 *             # update W
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_W, __pyx_v_ws, __pyx_v_i, __pyx_v_j), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_W, __pyx_v_ws, __pyx_v_i, (__pyx_v_j + 1)), (&__pyx_v_c), (&__pyx_v_s));

      /* "scipy/linalg/_decomp_update.pyx":481
 * 
 *             # update W
 *             if i+1 < p:             # <<<<<<<<<<<<<<
 *                 rot(p-i-1, index2(W, ws, i+1, j), ws[0],
 *                     index2(W, ws, i+1, j+1), ws[0], c, s)
 */
      __pyx_t_1 = (((__pyx_v_i + 1) < __pyx_v_p) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":482
 *             # update W
 *             if i+1 < p:
 *                 rot(p-i-1, index2(W, ws, i+1, j), ws[0],             # <<<<<<<<<<<<<<
 *                     index2(W, ws, i+1, j+1), ws[0], c, s)
 * 
 */
        __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_p - __pyx_v_i) - 1), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_W, __pyx_v_ws, (__pyx_v_i + 1), __pyx_v_j), (__pyx_v_ws[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_W, __pyx_v_ws, (__pyx_v_i + 1), (__pyx_v_j + 1)), (__pyx_v_ws[0]), __pyx_v_c, __pyx_v_s);

        /* "scipy/linalg/_decomp_update.pyx":481
 * 
 *             # update W
 *             if i+1 < p:             # <<<<<<<<<<<<<<
 *                 rot(p-i-1, index2(W, ws, i+1, j), ws[0],
 *                     index2(W, ws, i+1, j+1), ws[0], c, s)
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":486
 * 
 *             # update r if there is a nonzero row.
 *             if j-i < n:             # <<<<<<<<<<<<<<
 *                 rot(n-j+i, index2(r, rs, j, j-i), rs[1],
 *                     index2(r, rs, j+1, j-i), rs[1], c, s)
 */
      __pyx_t_1 = (((__pyx_v_j - __pyx_v_i) < __pyx_v_n) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":487
 *             # update r if there is a nonzero row.
 *             if j-i < n:
 *                 rot(n-j+i, index2(r, rs, j, j-i), rs[1],             # <<<<<<<<<<<<<<
 *                     index2(r, rs, j+1, j-i), rs[1], c, s)
 * 
 */
        __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) + __pyx_v_i), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j - __pyx_v_i)), (__pyx_v_rs[1]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_j - __pyx_v_i)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

        /* "scipy/linalg/_decomp_update.pyx":486
 * 
 *             # update r if there is a nonzero row.
 *             if j-i < n:             # <<<<<<<<<<<<<<
 *                 rot(n-j+i, index2(r, rs, j, j-i), rs[1],
 *                     index2(r, rs, j+1, j-i), rs[1], c, s)
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":491
 * 
 *             # update q
 *             rot(m-p, index2(q, qs, p, j), qs[0], index2(q, qs, p, j+1), qs[0],             # <<<<<<<<<<<<<<
 *                 c, s.conjugate())
 * 
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_m - __pyx_v_p), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_p, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_p, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_s));
    }
  }

  /* "scipy/linalg/_decomp_update.pyx":458
 *     return 1
 * 
 * cdef void qr_block_row_delete(int m, int n, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *                               blas_t* r, int* rs, int k, int p) nogil:
 *     cdef int i, j
 */

  /* function exit code */
}

static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_delete(int __pyx_v_m, int __pyx_v_n, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_float_complex *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p) {
  int __pyx_v_i;
  int __pyx_v_j;
  __pyx_t_float_complex __pyx_v_c;
  __pyx_t_float_complex __pyx_v_s;
  __pyx_t_float_complex *__pyx_v_W;
  int *__pyx_v_ws;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;

  /* "scipy/linalg/_decomp_update.pyx":465
 *     cdef int* ws
 * 
 *     if k != 0:             # <<<<<<<<<<<<<<
 *         for j in range(k, 0, -1):
 *             swap(m, row(q, qs, j+p-1), qs[1], row(q, qs, j-1), qs[1])
 */
  __pyx_t_1 = ((__pyx_v_k != 0) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":466
 * 
 *     if k != 0:
 *         for j in range(k, 0, -1):             # <<<<<<<<<<<<<<
 *             swap(m, row(q, qs, j+p-1), qs[1], row(q, qs, j-1), qs[1])
 * 
 */
    for (__pyx_t_2 = __pyx_v_k; __pyx_t_2 > 0; __pyx_t_2-=1) {
      __pyx_v_j = __pyx_t_2;

      /* "scipy/linalg/_decomp_update.pyx":467
 *     if k != 0:
 *         for j in range(k, 0, -1):
 *             swap(m, row(q, qs, j+p-1), qs[1], row(q, qs, j-1), qs[1])             # <<<<<<<<<<<<<<
 * 
 *     # W is the block of rows to be removed from q, has shape, (p,m)
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_swap(__pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, ((__pyx_v_j + __pyx_v_p) - 1)), (__pyx_v_qs[1]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, (__pyx_v_j - 1)), (__pyx_v_qs[1]));
    }

    /* "scipy/linalg/_decomp_update.pyx":465
 *     cdef int* ws
 * 
 *     if k != 0:             # <<<<<<<<<<<<<<
 *         for j in range(k, 0, -1):
 *             swap(m, row(q, qs, j+p-1), qs[1], row(q, qs, j-1), qs[1])
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":470
 * 
 *     # W is the block of rows to be removed from q, has shape, (p,m)
 *     W = q             # <<<<<<<<<<<<<<
 *     ws = qs
 * 
 */
  __pyx_v_W = __pyx_v_q;

  /* "scipy/linalg/_decomp_update.pyx":471
 *     # W is the block of rows to be removed from q, has shape, (p,m)
 *     W = q
 *     ws = qs             # <<<<<<<<<<<<<<
 * 
 *     for j in range(p):
 */
  __pyx_v_ws = __pyx_v_qs;

  /* "scipy/linalg/_decomp_update.pyx":473
 *     ws = qs
 * 
 *     for j in range(p):             # <<<<<<<<<<<<<<
 *         blas_t_conj(m, row(W, ws, j), &ws[1])
 * 
 */
  __pyx_t_2 = __pyx_v_p;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_j = __pyx_t_4;

    /* "scipy/linalg/_decomp_update.pyx":474
 * 
 *     for j in range(p):
 *         blas_t_conj(m, row(W, ws, j), &ws[1])             # <<<<<<<<<<<<<<
 * 
 *     for i in range(p):
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_W, __pyx_v_ws, __pyx_v_j), (&(__pyx_v_ws[1])));
  }

  /* "scipy/linalg/_decomp_update.pyx":476
 *         blas_t_conj(m, row(W, ws, j), &ws[1])
 * 
 *     for i in range(p):             # <<<<<<<<<<<<<<
 *         for j in range(m-2, i-1, -1):
 *             lartg(index2(W, ws, i, j), index2(W, ws, i, j+1), &c, &s)
 */
  __pyx_t_2 = __pyx_v_p;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "scipy/linalg/_decomp_update.pyx":477
 * 
 *     for i in range(p):
 *         for j in range(m-2, i-1, -1):             # <<<<<<<<<<<<<<
 *             lartg(index2(W, ws, i, j), index2(W, ws, i, j+1), &c, &s)
 * 
 */
    __pyx_t_5 = (__pyx_v_i - 1);
    __pyx_t_6 = __pyx_t_5;
    for (__pyx_t_7 = (__pyx_v_m - 2); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
      __pyx_v_j = __pyx_t_7;

      /* "scipy/linalg/_decomp_update.pyx":478
 *     for i in range(p):
 *         for j in range(m-2, i-1, -1):
 *             lartg(index2(W, ws, i, j), index2(W, ws, i, j+1), &c, &s)             # <<<<<<<<<<<<<<
 * 
 *             # update W
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_W, __pyx_v_ws, __pyx_v_i, __pyx_v_j), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_W, __pyx_v_ws, __pyx_v_i, (__pyx_v_j + 1)), (&__pyx_v_c), (&__pyx_v_s));

      /* "scipy/linalg/_decomp_update.pyx":481
 * 
 *             # update W
 *             if i+1 < p:             # <<<<<<<<<<<<<<
 *                 rot(p-i-1, index2(W, ws, i+1, j), ws[0],
 *                     index2(W, ws, i+1, j+1), ws[0], c, s)
 */
      __pyx_t_1 = (((__pyx_v_i + 1) < __pyx_v_p) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":482
 *             # update W
 *             if i+1 < p:
 *                 rot(p-i-1, index2(W, ws, i+1, j), ws[0],             # <<<<<<<<<<<<<<
 *                     index2(W, ws, i+1, j+1), ws[0], c, s)
 * 
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_p - __pyx_v_i) - 1), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_W, __pyx_v_ws, (__pyx_v_i + 1), __pyx_v_j), (__pyx_v_ws[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_W, __pyx_v_ws, (__pyx_v_i + 1), (__pyx_v_j + 1)), (__pyx_v_ws[0]), __pyx_v_c, __pyx_v_s);

        /* "scipy/linalg/_decomp_update.pyx":481
 * 
 *             # update W
 *             if i+1 < p:             # <<<<<<<<<<<<<<
 *                 rot(p-i-1, index2(W, ws, i+1, j), ws[0],
 *                     index2(W, ws, i+1, j+1), ws[0], c, s)
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":486
 * 
 *             # update r if there is a nonzero row.
 *             if j-i < n:             # <<<<<<<<<<<<<<
 *                 rot(n-j+i, index2(r, rs, j, j-i), rs[1],
 *                     index2(r, rs, j+1, j-i), rs[1], c, s)
 */
      __pyx_t_1 = (((__pyx_v_j - __pyx_v_i) < __pyx_v_n) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":487
 *             # update r if there is a nonzero row.
 *             if j-i < n:
 *                 rot(n-j+i, index2(r, rs, j, j-i), rs[1],             # <<<<<<<<<<<<<<
 *                     index2(r, rs, j+1, j-i), rs[1], c, s)
 * 
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) + __pyx_v_i), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j - __pyx_v_i)), (__pyx_v_rs[1]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_j - __pyx_v_i)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

        /* "scipy/linalg/_decomp_update.pyx":486
 * 
 *             # update r if there is a nonzero row.
 *             if j-i < n:             # <<<<<<<<<<<<<<
 *                 rot(n-j+i, index2(r, rs, j, j-i), rs[1],
 *                     index2(r, rs, j+1, j-i), rs[1], c, s)
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":491
 * 
 *             # update q
 *             rot(m-p, index2(q, qs, p, j), qs[0], index2(q, qs, p, j+1), qs[0],             # <<<<<<<<<<<<<<
 *                 c, s.conjugate())
 * 
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_m - __pyx_v_p), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_p, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_p, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_float(__pyx_v_s));
    }
  }

  /* "scipy/linalg/_decomp_update.pyx":458
 *     return 1
 * 
 * cdef void qr_block_row_delete(int m, int n, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *                               blas_t* r, int* rs, int k, int p) nogil:
 *     cdef int i, j
 */

  /* function exit code */
}

static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_delete(int __pyx_v_m, int __pyx_v_n, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_double_complex *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p) {
  int __pyx_v_i;
  int __pyx_v_j;
  __pyx_t_double_complex __pyx_v_c;
  __pyx_t_double_complex __pyx_v_s;
  __pyx_t_double_complex *__pyx_v_W;
  int *__pyx_v_ws;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;

  /* "scipy/linalg/_decomp_update.pyx":465
 *     cdef int* ws
 * 
 *     if k != 0:             # <<<<<<<<<<<<<<
 *         for j in range(k, 0, -1):
 *             swap(m, row(q, qs, j+p-1), qs[1], row(q, qs, j-1), qs[1])
 */
  __pyx_t_1 = ((__pyx_v_k != 0) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":466
 * 
 *     if k != 0:
 *         for j in range(k, 0, -1):             # <<<<<<<<<<<<<<
 *             swap(m, row(q, qs, j+p-1), qs[1], row(q, qs, j-1), qs[1])
 * 
 */
    for (__pyx_t_2 = __pyx_v_k; __pyx_t_2 > 0; __pyx_t_2-=1) {
      __pyx_v_j = __pyx_t_2;

      /* "scipy/linalg/_decomp_update.pyx":467
 *     if k != 0:
 *         for j in range(k, 0, -1):
 *             swap(m, row(q, qs, j+p-1), qs[1], row(q, qs, j-1), qs[1])             # <<<<<<<<<<<<<<
 * 
 *     # W is the block of rows to be removed from q, has shape, (p,m)
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_swap(__pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, ((__pyx_v_j + __pyx_v_p) - 1)), (__pyx_v_qs[1]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, (__pyx_v_j - 1)), (__pyx_v_qs[1]));
    }

    /* "scipy/linalg/_decomp_update.pyx":465
 *     cdef int* ws
 * 
 *     if k != 0:             # <<<<<<<<<<<<<<
 *         for j in range(k, 0, -1):
 *             swap(m, row(q, qs, j+p-1), qs[1], row(q, qs, j-1), qs[1])
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":470
 * 
 *     # W is the block of rows to be removed from q, has shape, (p,m)
 *     W = q             # <<<<<<<<<<<<<<
 *     ws = qs
 * 
 */
  __pyx_v_W = __pyx_v_q;

  /* "scipy/linalg/_decomp_update.pyx":471
 *     # W is the block of rows to be removed from q, has shape, (p,m)
 *     W = q
 *     ws = qs             # <<<<<<<<<<<<<<
 * 
 *     for j in range(p):
 */
  __pyx_v_ws = __pyx_v_qs;

  /* "scipy/linalg/_decomp_update.pyx":473
 *     ws = qs
 * 
 *     for j in range(p):             # <<<<<<<<<<<<<<
 *         blas_t_conj(m, row(W, ws, j), &ws[1])
 * 
 */
  __pyx_t_2 = __pyx_v_p;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_j = __pyx_t_4;

    /* "scipy/linalg/_decomp_update.pyx":474
 * 
 *     for j in range(p):
 *         blas_t_conj(m, row(W, ws, j), &ws[1])             # <<<<<<<<<<<<<<
 * 
 *     for i in range(p):
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_W, __pyx_v_ws, __pyx_v_j), (&(__pyx_v_ws[1])));
  }

  /* "scipy/linalg/_decomp_update.pyx":476
 *         blas_t_conj(m, row(W, ws, j), &ws[1])
 * 
 *     for i in range(p):             # <<<<<<<<<<<<<<
 *         for j in range(m-2, i-1, -1):
 *             lartg(index2(W, ws, i, j), index2(W, ws, i, j+1), &c, &s)
 */
  __pyx_t_2 = __pyx_v_p;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "scipy/linalg/_decomp_update.pyx":477
 * 
 *     for i in range(p):
 *         for j in range(m-2, i-1, -1):             # <<<<<<<<<<<<<<
 *             lartg(index2(W, ws, i, j), index2(W, ws, i, j+1), &c, &s)
 * 
 */
    __pyx_t_5 = (__pyx_v_i - 1);
    __pyx_t_6 = __pyx_t_5;
    for (__pyx_t_7 = (__pyx_v_m - 2); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
      __pyx_v_j = __pyx_t_7;

      /* "scipy/linalg/_decomp_update.pyx":478
 *     for i in range(p):
 *         for j in range(m-2, i-1, -1):
 *             lartg(index2(W, ws, i, j), index2(W, ws, i, j+1), &c, &s)             # <<<<<<<<<<<<<<
 * 
 *             # update W
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_W, __pyx_v_ws, __pyx_v_i, __pyx_v_j), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_W, __pyx_v_ws, __pyx_v_i, (__pyx_v_j + 1)), (&__pyx_v_c), (&__pyx_v_s));

      /* "scipy/linalg/_decomp_update.pyx":481
 * 
 *             # update W
 *             if i+1 < p:             # <<<<<<<<<<<<<<
 *                 rot(p-i-1, index2(W, ws, i+1, j), ws[0],
 *                     index2(W, ws, i+1, j+1), ws[0], c, s)
 */
      __pyx_t_1 = (((__pyx_v_i + 1) < __pyx_v_p) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":482
 *             # update W
 *             if i+1 < p:
 *                 rot(p-i-1, index2(W, ws, i+1, j), ws[0],             # <<<<<<<<<<<<<<
 *                     index2(W, ws, i+1, j+1), ws[0], c, s)
 * 
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_p - __pyx_v_i) - 1), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_W, __pyx_v_ws, (__pyx_v_i + 1), __pyx_v_j), (__pyx_v_ws[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_W, __pyx_v_ws, (__pyx_v_i + 1), (__pyx_v_j + 1)), (__pyx_v_ws[0]), __pyx_v_c, __pyx_v_s);

        /* "scipy/linalg/_decomp_update.pyx":481
 * 
 *             # update W
 *             if i+1 < p:             # <<<<<<<<<<<<<<
 *                 rot(p-i-1, index2(W, ws, i+1, j), ws[0],
 *                     index2(W, ws, i+1, j+1), ws[0], c, s)
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":486
 * 
 *             # update r if there is a nonzero row.
 *             if j-i < n:             # <<<<<<<<<<<<<<
 *                 rot(n-j+i, index2(r, rs, j, j-i), rs[1],
 *                     index2(r, rs, j+1, j-i), rs[1], c, s)
 */
      __pyx_t_1 = (((__pyx_v_j - __pyx_v_i) < __pyx_v_n) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":487
 *             # update r if there is a nonzero row.
 *             if j-i < n:
 *                 rot(n-j+i, index2(r, rs, j, j-i), rs[1],             # <<<<<<<<<<<<<<
 *                     index2(r, rs, j+1, j-i), rs[1], c, s)
 * 
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) + __pyx_v_i), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j - __pyx_v_i)), (__pyx_v_rs[1]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_j - __pyx_v_i)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

        /* "scipy/linalg/_decomp_update.pyx":486
 * 
 *             # update r if there is a nonzero row.
 *             if j-i < n:             # <<<<<<<<<<<<<<
 *                 rot(n-j+i, index2(r, rs, j, j-i), rs[1],
 *                     index2(r, rs, j+1, j-i), rs[1], c, s)
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":491
 * 
 *             # update q
 *             rot(m-p, index2(q, qs, p, j), qs[0], index2(q, qs, p, j+1), qs[0],             # <<<<<<<<<<<<<<
 *                 c, s.conjugate())
 * 
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_m - __pyx_v_p), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_p, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_p, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_double(__pyx_v_s));
    }
  }

  /* "scipy/linalg/_decomp_update.pyx":458
 *     return 1
 * 
 * cdef void qr_block_row_delete(int m, int n, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *                               blas_t* r, int* rs, int k, int p) nogil:
 *     cdef int i, j
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":494
 *                 c, s.conjugate())
 * 
 * cdef void qr_col_delete(int m, int o, int n, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                         int* rs, int k) nogil:
 *     """
 */

static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_delete(int __pyx_v_m, int __pyx_v_o, int __pyx_v_n, float *__pyx_v_q, int *__pyx_v_qs, float *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k) {
  int __pyx_v_j;
  int __pyx_v_limit;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  long __pyx_t_4;
  long __pyx_t_5;

  /* "scipy/linalg/_decomp_update.pyx":501
 *     """
 *     cdef int j
 *     cdef int limit = min(o, n)             # <<<<<<<<<<<<<<
 * 
 *     for j in range(k, n-1):
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_v_o;
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":503
 *     cdef int limit = min(o, n)
 * 
 *     for j in range(k, n-1):             # <<<<<<<<<<<<<<
 *         copy(limit, col(r, rs, j+1), rs[0], col(r, rs, j), rs[0])
 * 
 */
  __pyx_t_4 = (__pyx_v_n - 1);
  __pyx_t_5 = __pyx_t_4;
  for (__pyx_t_3 = __pyx_v_k; __pyx_t_3 < __pyx_t_5; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":504
 * 
 *     for j in range(k, n-1):
 *         copy(limit, col(r, rs, j+1), rs[0], col(r, rs, j), rs[0])             # <<<<<<<<<<<<<<
 * 
 *     hessenberg_qr(m, n-1, q, qs, r, rs, k)
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_limit, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1)), (__pyx_v_rs[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, __pyx_v_j), (__pyx_v_rs[0]));
  }

  /* "scipy/linalg/_decomp_update.pyx":506
 *         copy(limit, col(r, rs, j+1), rs[0], col(r, rs, j), rs[0])
 * 
 *     hessenberg_qr(m, n-1, q, qs, r, rs, k)             # <<<<<<<<<<<<<<
 * 
 * cdef int qr_block_col_delete(int m, int o, int n, blas_t* q, int* qs,
 */
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_hessenberg_qr(__pyx_v_m, (__pyx_v_n - 1), __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, __pyx_v_k);

  /* "scipy/linalg/_decomp_update.pyx":494
 *                 c, s.conjugate())
 * 
 * cdef void qr_col_delete(int m, int o, int n, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                         int* rs, int k) nogil:
 *     """
 */

  /* function exit code */
}

static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_delete(int __pyx_v_m, int __pyx_v_o, int __pyx_v_n, double *__pyx_v_q, int *__pyx_v_qs, double *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k) {
  int __pyx_v_j;
  int __pyx_v_limit;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  long __pyx_t_4;
  long __pyx_t_5;

  /* "scipy/linalg/_decomp_update.pyx":501
 *     """
 *     cdef int j
 *     cdef int limit = min(o, n)             # <<<<<<<<<<<<<<
 * 
 *     for j in range(k, n-1):
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_v_o;
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":503
 *     cdef int limit = min(o, n)
 * 
 *     for j in range(k, n-1):             # <<<<<<<<<<<<<<
 *         copy(limit, col(r, rs, j+1), rs[0], col(r, rs, j), rs[0])
 * 
 */
  __pyx_t_4 = (__pyx_v_n - 1);
  __pyx_t_5 = __pyx_t_4;
  for (__pyx_t_3 = __pyx_v_k; __pyx_t_3 < __pyx_t_5; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":504
 * 
 *     for j in range(k, n-1):
 *         copy(limit, col(r, rs, j+1), rs[0], col(r, rs, j), rs[0])             # <<<<<<<<<<<<<<
 * 
 *     hessenberg_qr(m, n-1, q, qs, r, rs, k)
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_limit, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1)), (__pyx_v_rs[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, __pyx_v_j), (__pyx_v_rs[0]));
  }

  /* "scipy/linalg/_decomp_update.pyx":506
 *         copy(limit, col(r, rs, j+1), rs[0], col(r, rs, j), rs[0])
 * 
 *     hessenberg_qr(m, n-1, q, qs, r, rs, k)             # <<<<<<<<<<<<<<
 * 
 * cdef int qr_block_col_delete(int m, int o, int n, blas_t* q, int* qs,
 */
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_hessenberg_qr(__pyx_v_m, (__pyx_v_n - 1), __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, __pyx_v_k);

  /* "scipy/linalg/_decomp_update.pyx":494
 *                 c, s.conjugate())
 * 
 * cdef void qr_col_delete(int m, int o, int n, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                         int* rs, int k) nogil:
 *     """
 */

  /* function exit code */
}

static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_delete(int __pyx_v_m, int __pyx_v_o, int __pyx_v_n, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_float_complex *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k) {
  int __pyx_v_j;
  int __pyx_v_limit;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  long __pyx_t_4;
  long __pyx_t_5;

  /* "scipy/linalg/_decomp_update.pyx":501
 *     """
 *     cdef int j
 *     cdef int limit = min(o, n)             # <<<<<<<<<<<<<<
 * 
 *     for j in range(k, n-1):
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_v_o;
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":503
 *     cdef int limit = min(o, n)
 * 
 *     for j in range(k, n-1):             # <<<<<<<<<<<<<<
 *         copy(limit, col(r, rs, j+1), rs[0], col(r, rs, j), rs[0])
 * 
 */
  __pyx_t_4 = (__pyx_v_n - 1);
  __pyx_t_5 = __pyx_t_4;
  for (__pyx_t_3 = __pyx_v_k; __pyx_t_3 < __pyx_t_5; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":504
 * 
 *     for j in range(k, n-1):
 *         copy(limit, col(r, rs, j+1), rs[0], col(r, rs, j), rs[0])             # <<<<<<<<<<<<<<
 * 
 *     hessenberg_qr(m, n-1, q, qs, r, rs, k)
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_limit, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1)), (__pyx_v_rs[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, __pyx_v_j), (__pyx_v_rs[0]));
  }

  /* "scipy/linalg/_decomp_update.pyx":506
 *         copy(limit, col(r, rs, j+1), rs[0], col(r, rs, j), rs[0])
 * 
 *     hessenberg_qr(m, n-1, q, qs, r, rs, k)             # <<<<<<<<<<<<<<
 * 
 * cdef int qr_block_col_delete(int m, int o, int n, blas_t* q, int* qs,
 */
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_hessenberg_qr(__pyx_v_m, (__pyx_v_n - 1), __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, __pyx_v_k);

  /* "scipy/linalg/_decomp_update.pyx":494
 *                 c, s.conjugate())
 * 
 * cdef void qr_col_delete(int m, int o, int n, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                         int* rs, int k) nogil:
 *     """
 */

  /* function exit code */
}

static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_delete(int __pyx_v_m, int __pyx_v_o, int __pyx_v_n, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_double_complex *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k) {
  int __pyx_v_j;
  int __pyx_v_limit;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  long __pyx_t_4;
  long __pyx_t_5;

  /* "scipy/linalg/_decomp_update.pyx":501
 *     """
 *     cdef int j
 *     cdef int limit = min(o, n)             # <<<<<<<<<<<<<<
 * 
 *     for j in range(k, n-1):
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_v_o;
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":503
 *     cdef int limit = min(o, n)
 * 
 *     for j in range(k, n-1):             # <<<<<<<<<<<<<<
 *         copy(limit, col(r, rs, j+1), rs[0], col(r, rs, j), rs[0])
 * 
 */
  __pyx_t_4 = (__pyx_v_n - 1);
  __pyx_t_5 = __pyx_t_4;
  for (__pyx_t_3 = __pyx_v_k; __pyx_t_3 < __pyx_t_5; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":504
 * 
 *     for j in range(k, n-1):
 *         copy(limit, col(r, rs, j+1), rs[0], col(r, rs, j), rs[0])             # <<<<<<<<<<<<<<
 * 
 *     hessenberg_qr(m, n-1, q, qs, r, rs, k)
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_limit, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1)), (__pyx_v_rs[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, __pyx_v_j), (__pyx_v_rs[0]));
  }

  /* "scipy/linalg/_decomp_update.pyx":506
 *         copy(limit, col(r, rs, j+1), rs[0], col(r, rs, j), rs[0])
 * 
 *     hessenberg_qr(m, n-1, q, qs, r, rs, k)             # <<<<<<<<<<<<<<
 * 
 * cdef int qr_block_col_delete(int m, int o, int n, blas_t* q, int* qs,
 */
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_hessenberg_qr(__pyx_v_m, (__pyx_v_n - 1), __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, __pyx_v_k);

  /* "scipy/linalg/_decomp_update.pyx":494
 *                 c, s.conjugate())
 * 
 * cdef void qr_col_delete(int m, int o, int n, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                         int* rs, int k) nogil:
 *     """
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":508
 *     hessenberg_qr(m, n-1, q, qs, r, rs, k)
 * 
 * cdef int qr_block_col_delete(int m, int o, int n, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *                               blas_t* r, int* rs, int k, int p) nogil:
 *     """
 */

static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_delete(int __pyx_v_m, int __pyx_v_o, int __pyx_v_n, float *__pyx_v_q, int *__pyx_v_qs, float *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p) {
  int __pyx_v_j;
  int __pyx_v_limit;
  float *__pyx_v_work;
  int __pyx_v_worksize;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":515
 *     """
 *     cdef int j
 *     cdef int limit = min(o, n)             # <<<<<<<<<<<<<<
 *     cdef blas_t* work
 *     cdef int worksize = max(m, n)
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_v_o;
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":517
 *     cdef int limit = min(o, n)
 *     cdef blas_t* work
 *     cdef int worksize = max(m, n)             # <<<<<<<<<<<<<<
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize*sizeof(blas_t))
 */
  __pyx_t_3 = __pyx_v_n;
  __pyx_t_1 = __pyx_v_m;
  if (((__pyx_t_3 > __pyx_t_1) != 0)) {
    __pyx_t_2 = __pyx_t_3;
  } else {
    __pyx_t_2 = __pyx_t_1;
  }
  __pyx_v_worksize = __pyx_t_2;

  /* "scipy/linalg/_decomp_update.pyx":519
 *     cdef int worksize = max(m, n)
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *     if not work:
 *         return MEMORY_ERROR
 */
  __pyx_v_work = ((float *)malloc((__pyx_v_worksize * (sizeof(float)))));

  /* "scipy/linalg/_decomp_update.pyx":520
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize*sizeof(blas_t))
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  __pyx_t_4 = ((!(__pyx_v_work != 0)) != 0);
  if (__pyx_t_4) {

    /* "scipy/linalg/_decomp_update.pyx":521
 *     work = <blas_t*>libc.stdlib.malloc(worksize*sizeof(blas_t))
 *     if not work:
 *         return MEMORY_ERROR             # <<<<<<<<<<<<<<
 * 
 *     # move the columns to removed to the end
 */
    __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":520
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize*sizeof(blas_t))
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":524
 * 
 *     # move the columns to removed to the end
 *     for j in range(k, n-p):             # <<<<<<<<<<<<<<
 *         copy(limit, col(r, rs, j+p), rs[0], col(r, rs, j), rs[0])
 * 
 */
  __pyx_t_2 = (__pyx_v_n - __pyx_v_p);
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_1 = __pyx_v_k; __pyx_t_1 < __pyx_t_3; __pyx_t_1+=1) {
    __pyx_v_j = __pyx_t_1;

    /* "scipy/linalg/_decomp_update.pyx":525
 *     # move the columns to removed to the end
 *     for j in range(k, n-p):
 *         copy(limit, col(r, rs, j+p), rs[0], col(r, rs, j), rs[0])             # <<<<<<<<<<<<<<
 * 
 *     p_subdiag_qr(m, o, n-p, q, qs, r, rs, k, p, work)
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_limit, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + __pyx_v_p)), (__pyx_v_rs[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, __pyx_v_j), (__pyx_v_rs[0]));
  }

  /* "scipy/linalg/_decomp_update.pyx":527
 *         copy(limit, col(r, rs, j+p), rs[0], col(r, rs, j), rs[0])
 * 
 *     p_subdiag_qr(m, o, n-p, q, qs, r, rs, k, p, work)             # <<<<<<<<<<<<<<
 * 
 *     libc.stdlib.free(work)
 */
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_p_subdiag_qr(__pyx_v_m, __pyx_v_o, (__pyx_v_n - __pyx_v_p), __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, __pyx_v_k, __pyx_v_p, __pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":529
 *     p_subdiag_qr(m, o, n-p, q, qs, r, rs, k, p, work)
 * 
 *     libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  free(__pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":530
 * 
 *     libc.stdlib.free(work)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef void thin_qr_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":508
 *     hessenberg_qr(m, n-1, q, qs, r, rs, k)
 * 
 * cdef int qr_block_col_delete(int m, int o, int n, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *                               blas_t* r, int* rs, int k, int p) nogil:
 *     """
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_delete(int __pyx_v_m, int __pyx_v_o, int __pyx_v_n, double *__pyx_v_q, int *__pyx_v_qs, double *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p) {
  int __pyx_v_j;
  int __pyx_v_limit;
  double *__pyx_v_work;
  int __pyx_v_worksize;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":515
 *     """
 *     cdef int j
 *     cdef int limit = min(o, n)             # <<<<<<<<<<<<<<
 *     cdef blas_t* work
 *     cdef int worksize = max(m, n)
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_v_o;
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":517
 *     cdef int limit = min(o, n)
 *     cdef blas_t* work
 *     cdef int worksize = max(m, n)             # <<<<<<<<<<<<<<
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize*sizeof(blas_t))
 */
  __pyx_t_3 = __pyx_v_n;
  __pyx_t_1 = __pyx_v_m;
  if (((__pyx_t_3 > __pyx_t_1) != 0)) {
    __pyx_t_2 = __pyx_t_3;
  } else {
    __pyx_t_2 = __pyx_t_1;
  }
  __pyx_v_worksize = __pyx_t_2;

  /* "scipy/linalg/_decomp_update.pyx":519
 *     cdef int worksize = max(m, n)
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *     if not work:
 *         return MEMORY_ERROR
 */
  __pyx_v_work = ((double *)malloc((__pyx_v_worksize * (sizeof(double)))));

  /* "scipy/linalg/_decomp_update.pyx":520
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize*sizeof(blas_t))
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  __pyx_t_4 = ((!(__pyx_v_work != 0)) != 0);
  if (__pyx_t_4) {

    /* "scipy/linalg/_decomp_update.pyx":521
 *     work = <blas_t*>libc.stdlib.malloc(worksize*sizeof(blas_t))
 *     if not work:
 *         return MEMORY_ERROR             # <<<<<<<<<<<<<<
 * 
 *     # move the columns to removed to the end
 */
    __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":520
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize*sizeof(blas_t))
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":524
 * 
 *     # move the columns to removed to the end
 *     for j in range(k, n-p):             # <<<<<<<<<<<<<<
 *         copy(limit, col(r, rs, j+p), rs[0], col(r, rs, j), rs[0])
 * 
 */
  __pyx_t_2 = (__pyx_v_n - __pyx_v_p);
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_1 = __pyx_v_k; __pyx_t_1 < __pyx_t_3; __pyx_t_1+=1) {
    __pyx_v_j = __pyx_t_1;

    /* "scipy/linalg/_decomp_update.pyx":525
 *     # move the columns to removed to the end
 *     for j in range(k, n-p):
 *         copy(limit, col(r, rs, j+p), rs[0], col(r, rs, j), rs[0])             # <<<<<<<<<<<<<<
 * 
 *     p_subdiag_qr(m, o, n-p, q, qs, r, rs, k, p, work)
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_limit, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + __pyx_v_p)), (__pyx_v_rs[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, __pyx_v_j), (__pyx_v_rs[0]));
  }

  /* "scipy/linalg/_decomp_update.pyx":527
 *         copy(limit, col(r, rs, j+p), rs[0], col(r, rs, j), rs[0])
 * 
 *     p_subdiag_qr(m, o, n-p, q, qs, r, rs, k, p, work)             # <<<<<<<<<<<<<<
 * 
 *     libc.stdlib.free(work)
 */
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_p_subdiag_qr(__pyx_v_m, __pyx_v_o, (__pyx_v_n - __pyx_v_p), __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, __pyx_v_k, __pyx_v_p, __pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":529
 *     p_subdiag_qr(m, o, n-p, q, qs, r, rs, k, p, work)
 * 
 *     libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  free(__pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":530
 * 
 *     libc.stdlib.free(work)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef void thin_qr_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":508
 *     hessenberg_qr(m, n-1, q, qs, r, rs, k)
 * 
 * cdef int qr_block_col_delete(int m, int o, int n, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *                               blas_t* r, int* rs, int k, int p) nogil:
 *     """
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_delete(int __pyx_v_m, int __pyx_v_o, int __pyx_v_n, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_float_complex *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p) {
  int __pyx_v_j;
  int __pyx_v_limit;
  __pyx_t_float_complex *__pyx_v_work;
  int __pyx_v_worksize;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":515
 *     """
 *     cdef int j
 *     cdef int limit = min(o, n)             # <<<<<<<<<<<<<<
 *     cdef blas_t* work
 *     cdef int worksize = max(m, n)
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_v_o;
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":517
 *     cdef int limit = min(o, n)
 *     cdef blas_t* work
 *     cdef int worksize = max(m, n)             # <<<<<<<<<<<<<<
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize*sizeof(blas_t))
 */
  __pyx_t_3 = __pyx_v_n;
  __pyx_t_1 = __pyx_v_m;
  if (((__pyx_t_3 > __pyx_t_1) != 0)) {
    __pyx_t_2 = __pyx_t_3;
  } else {
    __pyx_t_2 = __pyx_t_1;
  }
  __pyx_v_worksize = __pyx_t_2;

  /* "scipy/linalg/_decomp_update.pyx":519
 *     cdef int worksize = max(m, n)
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *     if not work:
 *         return MEMORY_ERROR
 */
  __pyx_v_work = ((__pyx_t_float_complex *)malloc((__pyx_v_worksize * (sizeof(__pyx_t_float_complex)))));

  /* "scipy/linalg/_decomp_update.pyx":520
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize*sizeof(blas_t))
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  __pyx_t_4 = ((!(__pyx_v_work != 0)) != 0);
  if (__pyx_t_4) {

    /* "scipy/linalg/_decomp_update.pyx":521
 *     work = <blas_t*>libc.stdlib.malloc(worksize*sizeof(blas_t))
 *     if not work:
 *         return MEMORY_ERROR             # <<<<<<<<<<<<<<
 * 
 *     # move the columns to removed to the end
 */
    __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":520
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize*sizeof(blas_t))
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":524
 * 
 *     # move the columns to removed to the end
 *     for j in range(k, n-p):             # <<<<<<<<<<<<<<
 *         copy(limit, col(r, rs, j+p), rs[0], col(r, rs, j), rs[0])
 * 
 */
  __pyx_t_2 = (__pyx_v_n - __pyx_v_p);
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_1 = __pyx_v_k; __pyx_t_1 < __pyx_t_3; __pyx_t_1+=1) {
    __pyx_v_j = __pyx_t_1;

    /* "scipy/linalg/_decomp_update.pyx":525
 *     # move the columns to removed to the end
 *     for j in range(k, n-p):
 *         copy(limit, col(r, rs, j+p), rs[0], col(r, rs, j), rs[0])             # <<<<<<<<<<<<<<
 * 
 *     p_subdiag_qr(m, o, n-p, q, qs, r, rs, k, p, work)
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_limit, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + __pyx_v_p)), (__pyx_v_rs[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, __pyx_v_j), (__pyx_v_rs[0]));
  }

  /* "scipy/linalg/_decomp_update.pyx":527
 *         copy(limit, col(r, rs, j+p), rs[0], col(r, rs, j), rs[0])
 * 
 *     p_subdiag_qr(m, o, n-p, q, qs, r, rs, k, p, work)             # <<<<<<<<<<<<<<
 * 
 *     libc.stdlib.free(work)
 */
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_p_subdiag_qr(__pyx_v_m, __pyx_v_o, (__pyx_v_n - __pyx_v_p), __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, __pyx_v_k, __pyx_v_p, __pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":529
 *     p_subdiag_qr(m, o, n-p, q, qs, r, rs, k, p, work)
 * 
 *     libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  free(__pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":530
 * 
 *     libc.stdlib.free(work)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef void thin_qr_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":508
 *     hessenberg_qr(m, n-1, q, qs, r, rs, k)
 * 
 * cdef int qr_block_col_delete(int m, int o, int n, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *                               blas_t* r, int* rs, int k, int p) nogil:
 *     """
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_delete(int __pyx_v_m, int __pyx_v_o, int __pyx_v_n, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_double_complex *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p) {
  int __pyx_v_j;
  int __pyx_v_limit;
  __pyx_t_double_complex *__pyx_v_work;
  int __pyx_v_worksize;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":515
 *     """
 *     cdef int j
 *     cdef int limit = min(o, n)             # <<<<<<<<<<<<<<
 *     cdef blas_t* work
 *     cdef int worksize = max(m, n)
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_v_o;
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":517
 *     cdef int limit = min(o, n)
 *     cdef blas_t* work
 *     cdef int worksize = max(m, n)             # <<<<<<<<<<<<<<
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize*sizeof(blas_t))
 */
  __pyx_t_3 = __pyx_v_n;
  __pyx_t_1 = __pyx_v_m;
  if (((__pyx_t_3 > __pyx_t_1) != 0)) {
    __pyx_t_2 = __pyx_t_3;
  } else {
    __pyx_t_2 = __pyx_t_1;
  }
  __pyx_v_worksize = __pyx_t_2;

  /* "scipy/linalg/_decomp_update.pyx":519
 *     cdef int worksize = max(m, n)
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *     if not work:
 *         return MEMORY_ERROR
 */
  __pyx_v_work = ((__pyx_t_double_complex *)malloc((__pyx_v_worksize * (sizeof(__pyx_t_double_complex)))));

  /* "scipy/linalg/_decomp_update.pyx":520
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize*sizeof(blas_t))
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  __pyx_t_4 = ((!(__pyx_v_work != 0)) != 0);
  if (__pyx_t_4) {

    /* "scipy/linalg/_decomp_update.pyx":521
 *     work = <blas_t*>libc.stdlib.malloc(worksize*sizeof(blas_t))
 *     if not work:
 *         return MEMORY_ERROR             # <<<<<<<<<<<<<<
 * 
 *     # move the columns to removed to the end
 */
    __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":520
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize*sizeof(blas_t))
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":524
 * 
 *     # move the columns to removed to the end
 *     for j in range(k, n-p):             # <<<<<<<<<<<<<<
 *         copy(limit, col(r, rs, j+p), rs[0], col(r, rs, j), rs[0])
 * 
 */
  __pyx_t_2 = (__pyx_v_n - __pyx_v_p);
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_1 = __pyx_v_k; __pyx_t_1 < __pyx_t_3; __pyx_t_1+=1) {
    __pyx_v_j = __pyx_t_1;

    /* "scipy/linalg/_decomp_update.pyx":525
 *     # move the columns to removed to the end
 *     for j in range(k, n-p):
 *         copy(limit, col(r, rs, j+p), rs[0], col(r, rs, j), rs[0])             # <<<<<<<<<<<<<<
 * 
 *     p_subdiag_qr(m, o, n-p, q, qs, r, rs, k, p, work)
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_limit, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + __pyx_v_p)), (__pyx_v_rs[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, __pyx_v_j), (__pyx_v_rs[0]));
  }

  /* "scipy/linalg/_decomp_update.pyx":527
 *         copy(limit, col(r, rs, j+p), rs[0], col(r, rs, j), rs[0])
 * 
 *     p_subdiag_qr(m, o, n-p, q, qs, r, rs, k, p, work)             # <<<<<<<<<<<<<<
 * 
 *     libc.stdlib.free(work)
 */
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_p_subdiag_qr(__pyx_v_m, __pyx_v_o, (__pyx_v_n - __pyx_v_p), __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, __pyx_v_k, __pyx_v_p, __pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":529
 *     p_subdiag_qr(m, o, n-p, q, qs, r, rs, k, p, work)
 * 
 *     libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  free(__pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":530
 * 
 *     libc.stdlib.free(work)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef void thin_qr_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":508
 *     hessenberg_qr(m, n-1, q, qs, r, rs, k)
 * 
 * cdef int qr_block_col_delete(int m, int o, int n, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *                               blas_t* r, int* rs, int k, int p) nogil:
 *     """
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":532
 *     return 0
 * 
 * cdef void thin_qr_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                              int* rs, blas_t* u, int* us, int k) nogil:
 *     cdef int j
 */

static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_insert(int __pyx_v_m, int __pyx_v_n, float *__pyx_v_q, int *__pyx_v_qs, float *__pyx_v_r, int *__pyx_v_rs, float *__pyx_v_u, int *__pyx_v_us, int __pyx_v_k) {
  int __pyx_v_j;
  float __pyx_v_c;
  float __pyx_v_s;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":537
 *     cdef blas_t c, s
 * 
 *     for j in range(n):             # <<<<<<<<<<<<<<
 *         lartg(index2(r, rs, j, j), index1(u, us, j), &c, &s)
 *         if j+1 < n:
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":538
 * 
 *     for j in range(n):
 *         lartg(index2(r, rs, j, j), index1(u, us, j), &c, &s)             # <<<<<<<<<<<<<<
 *         if j+1 < n:
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1], index1(u, us, j+1), us[0],
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_u, __pyx_v_us, __pyx_v_j), (&__pyx_v_c), (&__pyx_v_s));

    /* "scipy/linalg/_decomp_update.pyx":539
 *     for j in range(n):
 *         lartg(index2(r, rs, j, j), index1(u, us, j), &c, &s)
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1], index1(u, us, j+1), us[0],
 *                 c, s)
 */
    __pyx_t_4 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
    if (__pyx_t_4) {

      /* "scipy/linalg/_decomp_update.pyx":540
 *         lartg(index2(r, rs, j, j), index1(u, us, j), &c, &s)
 *         if j+1 < n:
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1], index1(u, us, j+1), us[0],             # <<<<<<<<<<<<<<
 *                 c, s)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, n), qs[0], c, s.conjugate())
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1)), (__pyx_v_us[0]), __pyx_v_c, __pyx_v_s);

      /* "scipy/linalg/_decomp_update.pyx":539
 *     for j in range(n):
 *         lartg(index2(r, rs, j, j), index1(u, us, j), &c, &s)
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1], index1(u, us, j+1), us[0],
 *                 c, s)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":542
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1], index1(u, us, j+1), us[0],
 *                 c, s)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, n), qs[0], c, s.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     # permute q
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_n), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_s));
  }

  /* "scipy/linalg/_decomp_update.pyx":545
 * 
 *     # permute q
 *     for j in range(m-1, k, -1):             # <<<<<<<<<<<<<<
 *         swap(n, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])
 * 
 */
  __pyx_t_1 = __pyx_v_k;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = (__pyx_v_m - 1); __pyx_t_3 > __pyx_t_2; __pyx_t_3-=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":546
 *     # permute q
 *     for j in range(m-1, k, -1):
 *         swap(n, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])             # <<<<<<<<<<<<<<
 * 
 * cdef void qr_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_swap(__pyx_v_n, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[1]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, (__pyx_v_j - 1)), (__pyx_v_qs[1]));
  }

  /* "scipy/linalg/_decomp_update.pyx":532
 *     return 0
 * 
 * cdef void thin_qr_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                              int* rs, blas_t* u, int* us, int k) nogil:
 *     cdef int j
 */

  /* function exit code */
}

static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_insert(int __pyx_v_m, int __pyx_v_n, double *__pyx_v_q, int *__pyx_v_qs, double *__pyx_v_r, int *__pyx_v_rs, double *__pyx_v_u, int *__pyx_v_us, int __pyx_v_k) {
  int __pyx_v_j;
  double __pyx_v_c;
  double __pyx_v_s;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":537
 *     cdef blas_t c, s
 * 
 *     for j in range(n):             # <<<<<<<<<<<<<<
 *         lartg(index2(r, rs, j, j), index1(u, us, j), &c, &s)
 *         if j+1 < n:
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":538
 * 
 *     for j in range(n):
 *         lartg(index2(r, rs, j, j), index1(u, us, j), &c, &s)             # <<<<<<<<<<<<<<
 *         if j+1 < n:
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1], index1(u, us, j+1), us[0],
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_u, __pyx_v_us, __pyx_v_j), (&__pyx_v_c), (&__pyx_v_s));

    /* "scipy/linalg/_decomp_update.pyx":539
 *     for j in range(n):
 *         lartg(index2(r, rs, j, j), index1(u, us, j), &c, &s)
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1], index1(u, us, j+1), us[0],
 *                 c, s)
 */
    __pyx_t_4 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
    if (__pyx_t_4) {

      /* "scipy/linalg/_decomp_update.pyx":540
 *         lartg(index2(r, rs, j, j), index1(u, us, j), &c, &s)
 *         if j+1 < n:
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1], index1(u, us, j+1), us[0],             # <<<<<<<<<<<<<<
 *                 c, s)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, n), qs[0], c, s.conjugate())
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1)), (__pyx_v_us[0]), __pyx_v_c, __pyx_v_s);

      /* "scipy/linalg/_decomp_update.pyx":539
 *     for j in range(n):
 *         lartg(index2(r, rs, j, j), index1(u, us, j), &c, &s)
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1], index1(u, us, j+1), us[0],
 *                 c, s)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":542
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1], index1(u, us, j+1), us[0],
 *                 c, s)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, n), qs[0], c, s.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     # permute q
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_n), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_s));
  }

  /* "scipy/linalg/_decomp_update.pyx":545
 * 
 *     # permute q
 *     for j in range(m-1, k, -1):             # <<<<<<<<<<<<<<
 *         swap(n, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])
 * 
 */
  __pyx_t_1 = __pyx_v_k;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = (__pyx_v_m - 1); __pyx_t_3 > __pyx_t_2; __pyx_t_3-=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":546
 *     # permute q
 *     for j in range(m-1, k, -1):
 *         swap(n, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])             # <<<<<<<<<<<<<<
 * 
 * cdef void qr_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_swap(__pyx_v_n, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[1]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, (__pyx_v_j - 1)), (__pyx_v_qs[1]));
  }

  /* "scipy/linalg/_decomp_update.pyx":532
 *     return 0
 * 
 * cdef void thin_qr_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                              int* rs, blas_t* u, int* us, int k) nogil:
 *     cdef int j
 */

  /* function exit code */
}

static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_insert(int __pyx_v_m, int __pyx_v_n, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_float_complex *__pyx_v_r, int *__pyx_v_rs, __pyx_t_float_complex *__pyx_v_u, int *__pyx_v_us, int __pyx_v_k) {
  int __pyx_v_j;
  __pyx_t_float_complex __pyx_v_c;
  __pyx_t_float_complex __pyx_v_s;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":537
 *     cdef blas_t c, s
 * 
 *     for j in range(n):             # <<<<<<<<<<<<<<
 *         lartg(index2(r, rs, j, j), index1(u, us, j), &c, &s)
 *         if j+1 < n:
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":538
 * 
 *     for j in range(n):
 *         lartg(index2(r, rs, j, j), index1(u, us, j), &c, &s)             # <<<<<<<<<<<<<<
 *         if j+1 < n:
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1], index1(u, us, j+1), us[0],
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_u, __pyx_v_us, __pyx_v_j), (&__pyx_v_c), (&__pyx_v_s));

    /* "scipy/linalg/_decomp_update.pyx":539
 *     for j in range(n):
 *         lartg(index2(r, rs, j, j), index1(u, us, j), &c, &s)
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1], index1(u, us, j+1), us[0],
 *                 c, s)
 */
    __pyx_t_4 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
    if (__pyx_t_4) {

      /* "scipy/linalg/_decomp_update.pyx":540
 *         lartg(index2(r, rs, j, j), index1(u, us, j), &c, &s)
 *         if j+1 < n:
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1], index1(u, us, j+1), us[0],             # <<<<<<<<<<<<<<
 *                 c, s)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, n), qs[0], c, s.conjugate())
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1)), (__pyx_v_us[0]), __pyx_v_c, __pyx_v_s);

      /* "scipy/linalg/_decomp_update.pyx":539
 *     for j in range(n):
 *         lartg(index2(r, rs, j, j), index1(u, us, j), &c, &s)
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1], index1(u, us, j+1), us[0],
 *                 c, s)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":542
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1], index1(u, us, j+1), us[0],
 *                 c, s)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, n), qs[0], c, s.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     # permute q
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_n), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_float(__pyx_v_s));
  }

  /* "scipy/linalg/_decomp_update.pyx":545
 * 
 *     # permute q
 *     for j in range(m-1, k, -1):             # <<<<<<<<<<<<<<
 *         swap(n, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])
 * 
 */
  __pyx_t_1 = __pyx_v_k;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = (__pyx_v_m - 1); __pyx_t_3 > __pyx_t_2; __pyx_t_3-=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":546
 *     # permute q
 *     for j in range(m-1, k, -1):
 *         swap(n, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])             # <<<<<<<<<<<<<<
 * 
 * cdef void qr_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_swap(__pyx_v_n, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[1]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, (__pyx_v_j - 1)), (__pyx_v_qs[1]));
  }

  /* "scipy/linalg/_decomp_update.pyx":532
 *     return 0
 * 
 * cdef void thin_qr_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                              int* rs, blas_t* u, int* us, int k) nogil:
 *     cdef int j
 */

  /* function exit code */
}

static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_insert(int __pyx_v_m, int __pyx_v_n, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_double_complex *__pyx_v_r, int *__pyx_v_rs, __pyx_t_double_complex *__pyx_v_u, int *__pyx_v_us, int __pyx_v_k) {
  int __pyx_v_j;
  __pyx_t_double_complex __pyx_v_c;
  __pyx_t_double_complex __pyx_v_s;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":537
 *     cdef blas_t c, s
 * 
 *     for j in range(n):             # <<<<<<<<<<<<<<
 *         lartg(index2(r, rs, j, j), index1(u, us, j), &c, &s)
 *         if j+1 < n:
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":538
 * 
 *     for j in range(n):
 *         lartg(index2(r, rs, j, j), index1(u, us, j), &c, &s)             # <<<<<<<<<<<<<<
 *         if j+1 < n:
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1], index1(u, us, j+1), us[0],
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_u, __pyx_v_us, __pyx_v_j), (&__pyx_v_c), (&__pyx_v_s));

    /* "scipy/linalg/_decomp_update.pyx":539
 *     for j in range(n):
 *         lartg(index2(r, rs, j, j), index1(u, us, j), &c, &s)
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1], index1(u, us, j+1), us[0],
 *                 c, s)
 */
    __pyx_t_4 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
    if (__pyx_t_4) {

      /* "scipy/linalg/_decomp_update.pyx":540
 *         lartg(index2(r, rs, j, j), index1(u, us, j), &c, &s)
 *         if j+1 < n:
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1], index1(u, us, j+1), us[0],             # <<<<<<<<<<<<<<
 *                 c, s)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, n), qs[0], c, s.conjugate())
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1)), (__pyx_v_us[0]), __pyx_v_c, __pyx_v_s);

      /* "scipy/linalg/_decomp_update.pyx":539
 *     for j in range(n):
 *         lartg(index2(r, rs, j, j), index1(u, us, j), &c, &s)
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1], index1(u, us, j+1), us[0],
 *                 c, s)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":542
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1], index1(u, us, j+1), us[0],
 *                 c, s)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, n), qs[0], c, s.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     # permute q
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_n), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_double(__pyx_v_s));
  }

  /* "scipy/linalg/_decomp_update.pyx":545
 * 
 *     # permute q
 *     for j in range(m-1, k, -1):             # <<<<<<<<<<<<<<
 *         swap(n, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])
 * 
 */
  __pyx_t_1 = __pyx_v_k;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = (__pyx_v_m - 1); __pyx_t_3 > __pyx_t_2; __pyx_t_3-=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":546
 *     # permute q
 *     for j in range(m-1, k, -1):
 *         swap(n, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])             # <<<<<<<<<<<<<<
 * 
 * cdef void qr_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_swap(__pyx_v_n, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[1]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, (__pyx_v_j - 1)), (__pyx_v_qs[1]));
  }

  /* "scipy/linalg/_decomp_update.pyx":532
 *     return 0
 * 
 * cdef void thin_qr_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                              int* rs, blas_t* u, int* us, int k) nogil:
 *     cdef int j
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":548
 *         swap(n, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])
 * 
 * cdef void qr_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                         int k) nogil:
 *     cdef int j
 */

static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_row_insert(int __pyx_v_m, int __pyx_v_n, float *__pyx_v_q, int *__pyx_v_qs, float *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k) {
  int __pyx_v_j;
  float __pyx_v_c;
  float __pyx_v_s;
  int __pyx_v_limit;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;

  /* "scipy/linalg/_decomp_update.pyx":552
 *     cdef int j
 *     cdef blas_t c, s
 *     cdef int limit = min(m-1, n)             # <<<<<<<<<<<<<<
 * 
 *     for j in range(limit):
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = (__pyx_v_m - 1);
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":554
 *     cdef int limit = min(m-1, n)
 * 
 *     for j in range(limit):             # <<<<<<<<<<<<<<
 *         lartg(index2(r, rs, j, j), index2(r, rs, m-1, j), &c, &s)
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1], index2(r, rs, m-1, j+1), rs[1],
 */
  __pyx_t_1 = __pyx_v_limit;
  __pyx_t_4 = __pyx_t_1;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_j = __pyx_t_5;

    /* "scipy/linalg/_decomp_update.pyx":555
 * 
 *     for j in range(limit):
 *         lartg(index2(r, rs, j, j), index2(r, rs, m-1, j), &c, &s)             # <<<<<<<<<<<<<<
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1], index2(r, rs, m-1, j+1), rs[1],
 *                 c, s)
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_m - 1), __pyx_v_j), (&__pyx_v_c), (&__pyx_v_s));

    /* "scipy/linalg/_decomp_update.pyx":556
 *     for j in range(limit):
 *         lartg(index2(r, rs, j, j), index2(r, rs, m-1, j), &c, &s)
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1], index2(r, rs, m-1, j+1), rs[1],             # <<<<<<<<<<<<<<
 *                 c, s)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, m-1), qs[0], c, s.conjugate())
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_m - 1), (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

    /* "scipy/linalg/_decomp_update.pyx":558
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1], index2(r, rs, m-1, j+1), rs[1],
 *                 c, s)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, m-1), qs[0], c, s.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     # permute q
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_m - 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_s));
  }

  /* "scipy/linalg/_decomp_update.pyx":561
 * 
 *     # permute q
 *     for j in range(m-1, k, -1):             # <<<<<<<<<<<<<<
 *         swap(m, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])
 * 
 */
  __pyx_t_1 = __pyx_v_k;
  __pyx_t_4 = __pyx_t_1;
  for (__pyx_t_5 = (__pyx_v_m - 1); __pyx_t_5 > __pyx_t_4; __pyx_t_5-=1) {
    __pyx_v_j = __pyx_t_5;

    /* "scipy/linalg/_decomp_update.pyx":562
 *     # permute q
 *     for j in range(m-1, k, -1):
 *         swap(m, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])             # <<<<<<<<<<<<<<
 * 
 * cdef int thin_qr_block_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r,
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_swap(__pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[1]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, (__pyx_v_j - 1)), (__pyx_v_qs[1]));
  }

  /* "scipy/linalg/_decomp_update.pyx":548
 *         swap(n, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])
 * 
 * cdef void qr_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                         int k) nogil:
 *     cdef int j
 */

  /* function exit code */
}

static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_row_insert(int __pyx_v_m, int __pyx_v_n, double *__pyx_v_q, int *__pyx_v_qs, double *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k) {
  int __pyx_v_j;
  double __pyx_v_c;
  double __pyx_v_s;
  int __pyx_v_limit;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;

  /* "scipy/linalg/_decomp_update.pyx":552
 *     cdef int j
 *     cdef blas_t c, s
 *     cdef int limit = min(m-1, n)             # <<<<<<<<<<<<<<
 * 
 *     for j in range(limit):
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = (__pyx_v_m - 1);
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":554
 *     cdef int limit = min(m-1, n)
 * 
 *     for j in range(limit):             # <<<<<<<<<<<<<<
 *         lartg(index2(r, rs, j, j), index2(r, rs, m-1, j), &c, &s)
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1], index2(r, rs, m-1, j+1), rs[1],
 */
  __pyx_t_1 = __pyx_v_limit;
  __pyx_t_4 = __pyx_t_1;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_j = __pyx_t_5;

    /* "scipy/linalg/_decomp_update.pyx":555
 * 
 *     for j in range(limit):
 *         lartg(index2(r, rs, j, j), index2(r, rs, m-1, j), &c, &s)             # <<<<<<<<<<<<<<
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1], index2(r, rs, m-1, j+1), rs[1],
 *                 c, s)
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_m - 1), __pyx_v_j), (&__pyx_v_c), (&__pyx_v_s));

    /* "scipy/linalg/_decomp_update.pyx":556
 *     for j in range(limit):
 *         lartg(index2(r, rs, j, j), index2(r, rs, m-1, j), &c, &s)
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1], index2(r, rs, m-1, j+1), rs[1],             # <<<<<<<<<<<<<<
 *                 c, s)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, m-1), qs[0], c, s.conjugate())
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_m - 1), (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

    /* "scipy/linalg/_decomp_update.pyx":558
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1], index2(r, rs, m-1, j+1), rs[1],
 *                 c, s)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, m-1), qs[0], c, s.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     # permute q
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_m - 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_s));
  }

  /* "scipy/linalg/_decomp_update.pyx":561
 * 
 *     # permute q
 *     for j in range(m-1, k, -1):             # <<<<<<<<<<<<<<
 *         swap(m, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])
 * 
 */
  __pyx_t_1 = __pyx_v_k;
  __pyx_t_4 = __pyx_t_1;
  for (__pyx_t_5 = (__pyx_v_m - 1); __pyx_t_5 > __pyx_t_4; __pyx_t_5-=1) {
    __pyx_v_j = __pyx_t_5;

    /* "scipy/linalg/_decomp_update.pyx":562
 *     # permute q
 *     for j in range(m-1, k, -1):
 *         swap(m, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])             # <<<<<<<<<<<<<<
 * 
 * cdef int thin_qr_block_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r,
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_swap(__pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[1]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, (__pyx_v_j - 1)), (__pyx_v_qs[1]));
  }

  /* "scipy/linalg/_decomp_update.pyx":548
 *         swap(n, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])
 * 
 * cdef void qr_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                         int k) nogil:
 *     cdef int j
 */

  /* function exit code */
}

static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_row_insert(int __pyx_v_m, int __pyx_v_n, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_float_complex *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k) {
  int __pyx_v_j;
  __pyx_t_float_complex __pyx_v_c;
  __pyx_t_float_complex __pyx_v_s;
  int __pyx_v_limit;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;

  /* "scipy/linalg/_decomp_update.pyx":552
 *     cdef int j
 *     cdef blas_t c, s
 *     cdef int limit = min(m-1, n)             # <<<<<<<<<<<<<<
 * 
 *     for j in range(limit):
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = (__pyx_v_m - 1);
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":554
 *     cdef int limit = min(m-1, n)
 * 
 *     for j in range(limit):             # <<<<<<<<<<<<<<
 *         lartg(index2(r, rs, j, j), index2(r, rs, m-1, j), &c, &s)
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1], index2(r, rs, m-1, j+1), rs[1],
 */
  __pyx_t_1 = __pyx_v_limit;
  __pyx_t_4 = __pyx_t_1;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_j = __pyx_t_5;

    /* "scipy/linalg/_decomp_update.pyx":555
 * 
 *     for j in range(limit):
 *         lartg(index2(r, rs, j, j), index2(r, rs, m-1, j), &c, &s)             # <<<<<<<<<<<<<<
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1], index2(r, rs, m-1, j+1), rs[1],
 *                 c, s)
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_m - 1), __pyx_v_j), (&__pyx_v_c), (&__pyx_v_s));

    /* "scipy/linalg/_decomp_update.pyx":556
 *     for j in range(limit):
 *         lartg(index2(r, rs, j, j), index2(r, rs, m-1, j), &c, &s)
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1], index2(r, rs, m-1, j+1), rs[1],             # <<<<<<<<<<<<<<
 *                 c, s)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, m-1), qs[0], c, s.conjugate())
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_m - 1), (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

    /* "scipy/linalg/_decomp_update.pyx":558
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1], index2(r, rs, m-1, j+1), rs[1],
 *                 c, s)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, m-1), qs[0], c, s.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     # permute q
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_m - 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_float(__pyx_v_s));
  }

  /* "scipy/linalg/_decomp_update.pyx":561
 * 
 *     # permute q
 *     for j in range(m-1, k, -1):             # <<<<<<<<<<<<<<
 *         swap(m, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])
 * 
 */
  __pyx_t_1 = __pyx_v_k;
  __pyx_t_4 = __pyx_t_1;
  for (__pyx_t_5 = (__pyx_v_m - 1); __pyx_t_5 > __pyx_t_4; __pyx_t_5-=1) {
    __pyx_v_j = __pyx_t_5;

    /* "scipy/linalg/_decomp_update.pyx":562
 *     # permute q
 *     for j in range(m-1, k, -1):
 *         swap(m, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])             # <<<<<<<<<<<<<<
 * 
 * cdef int thin_qr_block_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r,
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_swap(__pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[1]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, (__pyx_v_j - 1)), (__pyx_v_qs[1]));
  }

  /* "scipy/linalg/_decomp_update.pyx":548
 *         swap(n, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])
 * 
 * cdef void qr_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                         int k) nogil:
 *     cdef int j
 */

  /* function exit code */
}

static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_row_insert(int __pyx_v_m, int __pyx_v_n, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_double_complex *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k) {
  int __pyx_v_j;
  __pyx_t_double_complex __pyx_v_c;
  __pyx_t_double_complex __pyx_v_s;
  int __pyx_v_limit;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;

  /* "scipy/linalg/_decomp_update.pyx":552
 *     cdef int j
 *     cdef blas_t c, s
 *     cdef int limit = min(m-1, n)             # <<<<<<<<<<<<<<
 * 
 *     for j in range(limit):
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = (__pyx_v_m - 1);
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":554
 *     cdef int limit = min(m-1, n)
 * 
 *     for j in range(limit):             # <<<<<<<<<<<<<<
 *         lartg(index2(r, rs, j, j), index2(r, rs, m-1, j), &c, &s)
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1], index2(r, rs, m-1, j+1), rs[1],
 */
  __pyx_t_1 = __pyx_v_limit;
  __pyx_t_4 = __pyx_t_1;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_j = __pyx_t_5;

    /* "scipy/linalg/_decomp_update.pyx":555
 * 
 *     for j in range(limit):
 *         lartg(index2(r, rs, j, j), index2(r, rs, m-1, j), &c, &s)             # <<<<<<<<<<<<<<
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1], index2(r, rs, m-1, j+1), rs[1],
 *                 c, s)
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_m - 1), __pyx_v_j), (&__pyx_v_c), (&__pyx_v_s));

    /* "scipy/linalg/_decomp_update.pyx":556
 *     for j in range(limit):
 *         lartg(index2(r, rs, j, j), index2(r, rs, m-1, j), &c, &s)
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1], index2(r, rs, m-1, j+1), rs[1],             # <<<<<<<<<<<<<<
 *                 c, s)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, m-1), qs[0], c, s.conjugate())
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_m - 1), (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

    /* "scipy/linalg/_decomp_update.pyx":558
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1], index2(r, rs, m-1, j+1), rs[1],
 *                 c, s)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, m-1), qs[0], c, s.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     # permute q
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_m - 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_double(__pyx_v_s));
  }

  /* "scipy/linalg/_decomp_update.pyx":561
 * 
 *     # permute q
 *     for j in range(m-1, k, -1):             # <<<<<<<<<<<<<<
 *         swap(m, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])
 * 
 */
  __pyx_t_1 = __pyx_v_k;
  __pyx_t_4 = __pyx_t_1;
  for (__pyx_t_5 = (__pyx_v_m - 1); __pyx_t_5 > __pyx_t_4; __pyx_t_5-=1) {
    __pyx_v_j = __pyx_t_5;

    /* "scipy/linalg/_decomp_update.pyx":562
 *     # permute q
 *     for j in range(m-1, k, -1):
 *         swap(m, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])             # <<<<<<<<<<<<<<
 * 
 * cdef int thin_qr_block_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r,
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_swap(__pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[1]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_q, __pyx_v_qs, (__pyx_v_j - 1)), (__pyx_v_qs[1]));
  }

  /* "scipy/linalg/_decomp_update.pyx":548
 *         swap(n, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])
 * 
 * cdef void qr_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                         int k) nogil:
 *     cdef int j
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":564
 *         swap(m, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])
 * 
 * cdef int thin_qr_block_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                                   int* rs, blas_t* u, int* us, int k,
 *                                   int p) nogil:
 */

static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_block_row_insert(int __pyx_v_m, int __pyx_v_n, float *__pyx_v_q, int *__pyx_v_qs, float *__pyx_v_r, int *__pyx_v_rs, float *__pyx_v_u, int *__pyx_v_us, int __pyx_v_k, int __pyx_v_p) {
  int __pyx_v_j;
  float __pyx_v_rjj;
  float __pyx_v_tau;
  float *__pyx_v_work;
  char *__pyx_v_T;
  char *__pyx_v_N;
  size_t __pyx_v_worksize;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":571
 *     cdef blas_t rjj, tau
 *     cdef blas_t* work
 *     cdef char* T = 'T'             # <<<<<<<<<<<<<<
 *     cdef char* N = 'N'
 *     cdef size_t worksize = m * sizeof(blas_t)
 */
  __pyx_v_T = ((char *)"T");

  /* "scipy/linalg/_decomp_update.pyx":572
 *     cdef blas_t* work
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'             # <<<<<<<<<<<<<<
 *     cdef size_t worksize = m * sizeof(blas_t)
 * 
 */
  __pyx_v_N = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":573
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'
 *     cdef size_t worksize = m * sizeof(blas_t)             # <<<<<<<<<<<<<<
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize)
 */
  __pyx_v_worksize = (__pyx_v_m * (sizeof(float)));

  /* "scipy/linalg/_decomp_update.pyx":575
 *     cdef size_t worksize = m * sizeof(blas_t)
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize)             # <<<<<<<<<<<<<<
 *     if not work:
 *         return MEMORY_ERROR
 */
  __pyx_v_work = ((float *)malloc(__pyx_v_worksize));

  /* "scipy/linalg/_decomp_update.pyx":576
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize)
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  __pyx_t_1 = ((!(__pyx_v_work != 0)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":577
 *     work = <blas_t*>libc.stdlib.malloc(worksize)
 *     if not work:
 *         return MEMORY_ERROR             # <<<<<<<<<<<<<<
 * 
 *     # possible FIX
 */
    __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":576
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize)
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":582
 *     # as this is written it requires F order q, r, and u.  But thats not
 *     # strictly necessary. C order should also work too with a little fiddling.
 *     for j in range(n):             # <<<<<<<<<<<<<<
 *         rjj = index2(r, rs, j, j)[0]
 *         larfg(p+1, &rjj, col(u, us, j), us[0], &tau)
 */
  __pyx_t_2 = __pyx_v_n;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_j = __pyx_t_4;

    /* "scipy/linalg/_decomp_update.pyx":583
 *     # strictly necessary. C order should also work too with a little fiddling.
 *     for j in range(n):
 *         rjj = index2(r, rs, j, j)[0]             # <<<<<<<<<<<<<<
 *         larfg(p+1, &rjj, col(u, us, j), us[0], &tau)
 * 
 */
    __pyx_v_rjj = (__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]);

    /* "scipy/linalg/_decomp_update.pyx":584
 *     for j in range(n):
 *         rjj = index2(r, rs, j, j)[0]
 *         larfg(p+1, &rjj, col(u, us, j), us[0], &tau)             # <<<<<<<<<<<<<<
 * 
 *         # here we apply the reflector by hand instead of calling larf
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_larfg((__pyx_v_p + 1), (&__pyx_v_rjj), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), (&__pyx_v_tau));

    /* "scipy/linalg/_decomp_update.pyx":590
 *         # are separate.  This also permits the reflector to always be
 *         # p+1 long, rather than having a max of n+p.
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             copy(n-j-1, index2(r, rs, j, j+1), rs[1], work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])
 */
    __pyx_t_1 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":591
 *         # p+1 long, rather than having a max of n+p.
 *         if j+1 < n:
 *             copy(n-j-1, index2(r, rs, j, j+1), rs[1], work, 1)             # <<<<<<<<<<<<<<
 *             blas_t_conj(p, col(u, us, j), &us[0])
 *             gemv(T, p, n-j-1, 1, index2(u, us, 0, j+1), p, col(u, us, j), us[0],
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_copy(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_work, 1);

      /* "scipy/linalg/_decomp_update.pyx":592
 *         if j+1 < n:
 *             copy(n-j-1, index2(r, rs, j, j+1), rs[1], work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])             # <<<<<<<<<<<<<<
 *             gemv(T, p, n-j-1, 1, index2(u, us, 0, j+1), p, col(u, us, j), us[0],
 *                 1, work, 1)
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_p, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (&(__pyx_v_us[0])));

      /* "scipy/linalg/_decomp_update.pyx":593
 *             copy(n-j-1, index2(r, rs, j, j+1), rs[1], work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])
 *             gemv(T, p, n-j-1, 1, index2(u, us, 0, j+1), p, col(u, us, j), us[0],             # <<<<<<<<<<<<<<
 *                 1, work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_p, ((__pyx_v_n - __pyx_v_j) - 1), 1.0, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, 0, (__pyx_v_j + 1)), __pyx_v_p, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), 1.0, __pyx_v_work, 1);

      /* "scipy/linalg/_decomp_update.pyx":595
 *             gemv(T, p, n-j-1, 1, index2(u, us, 0, j+1), p, col(u, us, j), us[0],
 *                 1, work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])             # <<<<<<<<<<<<<<
 *             ger(p, n-j-1, -tau.conjugate(), col(u, us, j), us[0], work, 1,
 *                 index2(u, us, 0, j+1), p)
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_p, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (&(__pyx_v_us[0])));

      /* "scipy/linalg/_decomp_update.pyx":596
 *                 1, work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])
 *             ger(p, n-j-1, -tau.conjugate(), col(u, us, j), us[0], work, 1,             # <<<<<<<<<<<<<<
 *                 index2(u, us, 0, j+1), p)
 *             axpy(n-j-1, -tau.conjugate(), work, 1, index2(r, rs, j, j+1), rs[1])
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_ger(__pyx_v_p, ((__pyx_v_n - __pyx_v_j) - 1), (- (__pyx_v_tau)), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), __pyx_v_work, 1, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, 0, (__pyx_v_j + 1)), __pyx_v_p);

      /* "scipy/linalg/_decomp_update.pyx":598
 *             ger(p, n-j-1, -tau.conjugate(), col(u, us, j), us[0], work, 1,
 *                 index2(u, us, 0, j+1), p)
 *             axpy(n-j-1, -tau.conjugate(), work, 1, index2(r, rs, j, j+1), rs[1])             # <<<<<<<<<<<<<<
 *         index2(r, rs, j, j)[0] = rjj
 * 
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_axpy(((__pyx_v_n - __pyx_v_j) - 1), (- (__pyx_v_tau)), __pyx_v_work, 1, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]));

      /* "scipy/linalg/_decomp_update.pyx":590
 *         # are separate.  This also permits the reflector to always be
 *         # p+1 long, rather than having a max of n+p.
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             copy(n-j-1, index2(r, rs, j, j+1), rs[1], work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":599
 *                 index2(u, us, 0, j+1), p)
 *             axpy(n-j-1, -tau.conjugate(), work, 1, index2(r, rs, j, j+1), rs[1])
 *         index2(r, rs, j, j)[0] = rjj             # <<<<<<<<<<<<<<
 * 
 *         # now apply this reflector to q
 */
    (__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]) = __pyx_v_rjj;

    /* "scipy/linalg/_decomp_update.pyx":602
 * 
 *         # now apply this reflector to q
 *         copy(m, col(q, qs, j), qs[0], work, 1)             # <<<<<<<<<<<<<<
 *         gemv(N, m, p, 1, index2(q, qs, 0, n), m, col(u, us, j), us[0],
 *              1, work, 1)
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_v_work, 1);

    /* "scipy/linalg/_decomp_update.pyx":603
 *         # now apply this reflector to q
 *         copy(m, col(q, qs, j), qs[0], work, 1)
 *         gemv(N, m, p, 1, index2(q, qs, 0, n), m, col(u, us, j), us[0],             # <<<<<<<<<<<<<<
 *              1, work, 1)
 *         blas_t_conj(p, col(u, us, j), &us[0])
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_p, 1.0, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_n), __pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), 1.0, __pyx_v_work, 1);

    /* "scipy/linalg/_decomp_update.pyx":605
 *         gemv(N, m, p, 1, index2(q, qs, 0, n), m, col(u, us, j), us[0],
 *              1, work, 1)
 *         blas_t_conj(p, col(u, us, j), &us[0])             # <<<<<<<<<<<<<<
 *         ger(m, p, -tau, work, 1, col(u, us, j), us[0],
 *             index2(q, qs, 0, n), m)
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_p, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (&(__pyx_v_us[0])));

    /* "scipy/linalg/_decomp_update.pyx":606
 *              1, work, 1)
 *         blas_t_conj(p, col(u, us, j), &us[0])
 *         ger(m, p, -tau, work, 1, col(u, us, j), us[0],             # <<<<<<<<<<<<<<
 *             index2(q, qs, 0, n), m)
 *         axpy(m, -tau, work, 1, col(q, qs, j), qs[0])
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_ger(__pyx_v_m, __pyx_v_p, (-__pyx_v_tau), __pyx_v_work, 1, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_n), __pyx_v_m);

    /* "scipy/linalg/_decomp_update.pyx":608
 *         ger(m, p, -tau, work, 1, col(u, us, j), us[0],
 *             index2(q, qs, 0, n), m)
 *         axpy(m, -tau, work, 1, col(q, qs, j), qs[0])             # <<<<<<<<<<<<<<
 * 
 *     # permute the rows of q, work columnwise, since q is fortran order
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_m, (-__pyx_v_tau), __pyx_v_work, 1, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]));
  }

  /* "scipy/linalg/_decomp_update.pyx":611
 * 
 *     # permute the rows of q, work columnwise, since q is fortran order
 *     if k != m-p:             # <<<<<<<<<<<<<<
 *         for j in range(n):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 */
  __pyx_t_1 = ((__pyx_v_k != (__pyx_v_m - __pyx_v_p)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":612
 *     # permute the rows of q, work columnwise, since q is fortran order
 *     if k != m-p:
 *         for j in range(n):             # <<<<<<<<<<<<<<
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 */
    __pyx_t_2 = __pyx_v_n;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_j = __pyx_t_4;

      /* "scipy/linalg/_decomp_update.pyx":613
 *     if k != m-p:
 *         for j in range(n):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)             # <<<<<<<<<<<<<<
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_copy((__pyx_v_m - __pyx_v_k), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_k, __pyx_v_j), (__pyx_v_qs[0]), __pyx_v_work, 1);

      /* "scipy/linalg/_decomp_update.pyx":614
 *         for j in range(n):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])             # <<<<<<<<<<<<<<
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 * 
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_p, (__pyx_v_work + ((__pyx_v_m - __pyx_v_k) - __pyx_v_p)), 1, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_k, __pyx_v_j), (__pyx_v_qs[0]));

      /* "scipy/linalg/_decomp_update.pyx":615
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])             # <<<<<<<<<<<<<<
 * 
 *     libc.stdlib.free(work)
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_copy(((__pyx_v_m - __pyx_v_k) - __pyx_v_p), __pyx_v_work, 1, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, (__pyx_v_k + __pyx_v_p), __pyx_v_j), (__pyx_v_qs[0]));
    }

    /* "scipy/linalg/_decomp_update.pyx":611
 * 
 *     # permute the rows of q, work columnwise, since q is fortran order
 *     if k != m-p:             # <<<<<<<<<<<<<<
 *         for j in range(n):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":617
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 * 
 *     libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 * 
 * cdef int qr_block_row_insert(int m, int n, blas_t* q, int* qs,
 */
  free(__pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":564
 *         swap(m, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])
 * 
 * cdef int thin_qr_block_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                                   int* rs, blas_t* u, int* us, int k,
 *                                   int p) nogil:
 */

  /* function exit code */
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_block_row_insert(int __pyx_v_m, int __pyx_v_n, double *__pyx_v_q, int *__pyx_v_qs, double *__pyx_v_r, int *__pyx_v_rs, double *__pyx_v_u, int *__pyx_v_us, int __pyx_v_k, int __pyx_v_p) {
  int __pyx_v_j;
  double __pyx_v_rjj;
  double __pyx_v_tau;
  double *__pyx_v_work;
  char *__pyx_v_T;
  char *__pyx_v_N;
  size_t __pyx_v_worksize;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":571
 *     cdef blas_t rjj, tau
 *     cdef blas_t* work
 *     cdef char* T = 'T'             # <<<<<<<<<<<<<<
 *     cdef char* N = 'N'
 *     cdef size_t worksize = m * sizeof(blas_t)
 */
  __pyx_v_T = ((char *)"T");

  /* "scipy/linalg/_decomp_update.pyx":572
 *     cdef blas_t* work
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'             # <<<<<<<<<<<<<<
 *     cdef size_t worksize = m * sizeof(blas_t)
 * 
 */
  __pyx_v_N = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":573
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'
 *     cdef size_t worksize = m * sizeof(blas_t)             # <<<<<<<<<<<<<<
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize)
 */
  __pyx_v_worksize = (__pyx_v_m * (sizeof(double)));

  /* "scipy/linalg/_decomp_update.pyx":575
 *     cdef size_t worksize = m * sizeof(blas_t)
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize)             # <<<<<<<<<<<<<<
 *     if not work:
 *         return MEMORY_ERROR
 */
  __pyx_v_work = ((double *)malloc(__pyx_v_worksize));

  /* "scipy/linalg/_decomp_update.pyx":576
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize)
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  __pyx_t_1 = ((!(__pyx_v_work != 0)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":577
 *     work = <blas_t*>libc.stdlib.malloc(worksize)
 *     if not work:
 *         return MEMORY_ERROR             # <<<<<<<<<<<<<<
 * 
 *     # possible FIX
 */
    __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":576
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize)
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":582
 *     # as this is written it requires F order q, r, and u.  But thats not
 *     # strictly necessary. C order should also work too with a little fiddling.
 *     for j in range(n):             # <<<<<<<<<<<<<<
 *         rjj = index2(r, rs, j, j)[0]
 *         larfg(p+1, &rjj, col(u, us, j), us[0], &tau)
 */
  __pyx_t_2 = __pyx_v_n;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_j = __pyx_t_4;

    /* "scipy/linalg/_decomp_update.pyx":583
 *     # strictly necessary. C order should also work too with a little fiddling.
 *     for j in range(n):
 *         rjj = index2(r, rs, j, j)[0]             # <<<<<<<<<<<<<<
 *         larfg(p+1, &rjj, col(u, us, j), us[0], &tau)
 * 
 */
    __pyx_v_rjj = (__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]);

    /* "scipy/linalg/_decomp_update.pyx":584
 *     for j in range(n):
 *         rjj = index2(r, rs, j, j)[0]
 *         larfg(p+1, &rjj, col(u, us, j), us[0], &tau)             # <<<<<<<<<<<<<<
 * 
 *         # here we apply the reflector by hand instead of calling larf
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_larfg((__pyx_v_p + 1), (&__pyx_v_rjj), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), (&__pyx_v_tau));

    /* "scipy/linalg/_decomp_update.pyx":590
 *         # are separate.  This also permits the reflector to always be
 *         # p+1 long, rather than having a max of n+p.
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             copy(n-j-1, index2(r, rs, j, j+1), rs[1], work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])
 */
    __pyx_t_1 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":591
 *         # p+1 long, rather than having a max of n+p.
 *         if j+1 < n:
 *             copy(n-j-1, index2(r, rs, j, j+1), rs[1], work, 1)             # <<<<<<<<<<<<<<
 *             blas_t_conj(p, col(u, us, j), &us[0])
 *             gemv(T, p, n-j-1, 1, index2(u, us, 0, j+1), p, col(u, us, j), us[0],
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_copy(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_work, 1);

      /* "scipy/linalg/_decomp_update.pyx":592
 *         if j+1 < n:
 *             copy(n-j-1, index2(r, rs, j, j+1), rs[1], work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])             # <<<<<<<<<<<<<<
 *             gemv(T, p, n-j-1, 1, index2(u, us, 0, j+1), p, col(u, us, j), us[0],
 *                 1, work, 1)
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_p, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (&(__pyx_v_us[0])));

      /* "scipy/linalg/_decomp_update.pyx":593
 *             copy(n-j-1, index2(r, rs, j, j+1), rs[1], work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])
 *             gemv(T, p, n-j-1, 1, index2(u, us, 0, j+1), p, col(u, us, j), us[0],             # <<<<<<<<<<<<<<
 *                 1, work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_p, ((__pyx_v_n - __pyx_v_j) - 1), 1.0, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, 0, (__pyx_v_j + 1)), __pyx_v_p, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), 1.0, __pyx_v_work, 1);

      /* "scipy/linalg/_decomp_update.pyx":595
 *             gemv(T, p, n-j-1, 1, index2(u, us, 0, j+1), p, col(u, us, j), us[0],
 *                 1, work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])             # <<<<<<<<<<<<<<
 *             ger(p, n-j-1, -tau.conjugate(), col(u, us, j), us[0], work, 1,
 *                 index2(u, us, 0, j+1), p)
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_p, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (&(__pyx_v_us[0])));

      /* "scipy/linalg/_decomp_update.pyx":596
 *                 1, work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])
 *             ger(p, n-j-1, -tau.conjugate(), col(u, us, j), us[0], work, 1,             # <<<<<<<<<<<<<<
 *                 index2(u, us, 0, j+1), p)
 *             axpy(n-j-1, -tau.conjugate(), work, 1, index2(r, rs, j, j+1), rs[1])
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_ger(__pyx_v_p, ((__pyx_v_n - __pyx_v_j) - 1), (- (__pyx_v_tau)), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), __pyx_v_work, 1, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, 0, (__pyx_v_j + 1)), __pyx_v_p);

      /* "scipy/linalg/_decomp_update.pyx":598
 *             ger(p, n-j-1, -tau.conjugate(), col(u, us, j), us[0], work, 1,
 *                 index2(u, us, 0, j+1), p)
 *             axpy(n-j-1, -tau.conjugate(), work, 1, index2(r, rs, j, j+1), rs[1])             # <<<<<<<<<<<<<<
 *         index2(r, rs, j, j)[0] = rjj
 * 
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_axpy(((__pyx_v_n - __pyx_v_j) - 1), (- (__pyx_v_tau)), __pyx_v_work, 1, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]));

      /* "scipy/linalg/_decomp_update.pyx":590
 *         # are separate.  This also permits the reflector to always be
 *         # p+1 long, rather than having a max of n+p.
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             copy(n-j-1, index2(r, rs, j, j+1), rs[1], work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":599
 *                 index2(u, us, 0, j+1), p)
 *             axpy(n-j-1, -tau.conjugate(), work, 1, index2(r, rs, j, j+1), rs[1])
 *         index2(r, rs, j, j)[0] = rjj             # <<<<<<<<<<<<<<
 * 
 *         # now apply this reflector to q
 */
    (__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]) = __pyx_v_rjj;

    /* "scipy/linalg/_decomp_update.pyx":602
 * 
 *         # now apply this reflector to q
 *         copy(m, col(q, qs, j), qs[0], work, 1)             # <<<<<<<<<<<<<<
 *         gemv(N, m, p, 1, index2(q, qs, 0, n), m, col(u, us, j), us[0],
 *              1, work, 1)
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_v_work, 1);

    /* "scipy/linalg/_decomp_update.pyx":603
 *         # now apply this reflector to q
 *         copy(m, col(q, qs, j), qs[0], work, 1)
 *         gemv(N, m, p, 1, index2(q, qs, 0, n), m, col(u, us, j), us[0],             # <<<<<<<<<<<<<<
 *              1, work, 1)
 *         blas_t_conj(p, col(u, us, j), &us[0])
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_p, 1.0, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_n), __pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), 1.0, __pyx_v_work, 1);

    /* "scipy/linalg/_decomp_update.pyx":605
 *         gemv(N, m, p, 1, index2(q, qs, 0, n), m, col(u, us, j), us[0],
 *              1, work, 1)
 *         blas_t_conj(p, col(u, us, j), &us[0])             # <<<<<<<<<<<<<<
 *         ger(m, p, -tau, work, 1, col(u, us, j), us[0],
 *             index2(q, qs, 0, n), m)
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_p, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (&(__pyx_v_us[0])));

    /* "scipy/linalg/_decomp_update.pyx":606
 *              1, work, 1)
 *         blas_t_conj(p, col(u, us, j), &us[0])
 *         ger(m, p, -tau, work, 1, col(u, us, j), us[0],             # <<<<<<<<<<<<<<
 *             index2(q, qs, 0, n), m)
 *         axpy(m, -tau, work, 1, col(q, qs, j), qs[0])
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_ger(__pyx_v_m, __pyx_v_p, (-__pyx_v_tau), __pyx_v_work, 1, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_n), __pyx_v_m);

    /* "scipy/linalg/_decomp_update.pyx":608
 *         ger(m, p, -tau, work, 1, col(u, us, j), us[0],
 *             index2(q, qs, 0, n), m)
 *         axpy(m, -tau, work, 1, col(q, qs, j), qs[0])             # <<<<<<<<<<<<<<
 * 
 *     # permute the rows of q, work columnwise, since q is fortran order
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_m, (-__pyx_v_tau), __pyx_v_work, 1, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]));
  }

  /* "scipy/linalg/_decomp_update.pyx":611
 * 
 *     # permute the rows of q, work columnwise, since q is fortran order
 *     if k != m-p:             # <<<<<<<<<<<<<<
 *         for j in range(n):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 */
  __pyx_t_1 = ((__pyx_v_k != (__pyx_v_m - __pyx_v_p)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":612
 *     # permute the rows of q, work columnwise, since q is fortran order
 *     if k != m-p:
 *         for j in range(n):             # <<<<<<<<<<<<<<
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 */
    __pyx_t_2 = __pyx_v_n;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_j = __pyx_t_4;

      /* "scipy/linalg/_decomp_update.pyx":613
 *     if k != m-p:
 *         for j in range(n):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)             # <<<<<<<<<<<<<<
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_copy((__pyx_v_m - __pyx_v_k), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_k, __pyx_v_j), (__pyx_v_qs[0]), __pyx_v_work, 1);

      /* "scipy/linalg/_decomp_update.pyx":614
 *         for j in range(n):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])             # <<<<<<<<<<<<<<
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 * 
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_p, (__pyx_v_work + ((__pyx_v_m - __pyx_v_k) - __pyx_v_p)), 1, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_k, __pyx_v_j), (__pyx_v_qs[0]));

      /* "scipy/linalg/_decomp_update.pyx":615
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])             # <<<<<<<<<<<<<<
 * 
 *     libc.stdlib.free(work)
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_copy(((__pyx_v_m - __pyx_v_k) - __pyx_v_p), __pyx_v_work, 1, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, (__pyx_v_k + __pyx_v_p), __pyx_v_j), (__pyx_v_qs[0]));
    }

    /* "scipy/linalg/_decomp_update.pyx":611
 * 
 *     # permute the rows of q, work columnwise, since q is fortran order
 *     if k != m-p:             # <<<<<<<<<<<<<<
 *         for j in range(n):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":617
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 * 
 *     libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 * 
 * cdef int qr_block_row_insert(int m, int n, blas_t* q, int* qs,
 */
  free(__pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":564
 *         swap(m, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])
 * 
 * cdef int thin_qr_block_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                                   int* rs, blas_t* u, int* us, int k,
 *                                   int p) nogil:
 */

  /* function exit code */
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_block_row_insert(int __pyx_v_m, int __pyx_v_n, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_float_complex *__pyx_v_r, int *__pyx_v_rs, __pyx_t_float_complex *__pyx_v_u, int *__pyx_v_us, int __pyx_v_k, int __pyx_v_p) {
  int __pyx_v_j;
  __pyx_t_float_complex __pyx_v_rjj;
  __pyx_t_float_complex __pyx_v_tau;
  __pyx_t_float_complex *__pyx_v_work;
  char *__pyx_v_T;
  char *__pyx_v_N;
  size_t __pyx_v_worksize;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":571
 *     cdef blas_t rjj, tau
 *     cdef blas_t* work
 *     cdef char* T = 'T'             # <<<<<<<<<<<<<<
 *     cdef char* N = 'N'
 *     cdef size_t worksize = m * sizeof(blas_t)
 */
  __pyx_v_T = ((char *)"T");

  /* "scipy/linalg/_decomp_update.pyx":572
 *     cdef blas_t* work
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'             # <<<<<<<<<<<<<<
 *     cdef size_t worksize = m * sizeof(blas_t)
 * 
 */
  __pyx_v_N = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":573
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'
 *     cdef size_t worksize = m * sizeof(blas_t)             # <<<<<<<<<<<<<<
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize)
 */
  __pyx_v_worksize = (__pyx_v_m * (sizeof(__pyx_t_float_complex)));

  /* "scipy/linalg/_decomp_update.pyx":575
 *     cdef size_t worksize = m * sizeof(blas_t)
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize)             # <<<<<<<<<<<<<<
 *     if not work:
 *         return MEMORY_ERROR
 */
  __pyx_v_work = ((__pyx_t_float_complex *)malloc(__pyx_v_worksize));

  /* "scipy/linalg/_decomp_update.pyx":576
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize)
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  __pyx_t_1 = ((!(__pyx_v_work != 0)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":577
 *     work = <blas_t*>libc.stdlib.malloc(worksize)
 *     if not work:
 *         return MEMORY_ERROR             # <<<<<<<<<<<<<<
 * 
 *     # possible FIX
 */
    __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":576
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize)
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":582
 *     # as this is written it requires F order q, r, and u.  But thats not
 *     # strictly necessary. C order should also work too with a little fiddling.
 *     for j in range(n):             # <<<<<<<<<<<<<<
 *         rjj = index2(r, rs, j, j)[0]
 *         larfg(p+1, &rjj, col(u, us, j), us[0], &tau)
 */
  __pyx_t_2 = __pyx_v_n;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_j = __pyx_t_4;

    /* "scipy/linalg/_decomp_update.pyx":583
 *     # strictly necessary. C order should also work too with a little fiddling.
 *     for j in range(n):
 *         rjj = index2(r, rs, j, j)[0]             # <<<<<<<<<<<<<<
 *         larfg(p+1, &rjj, col(u, us, j), us[0], &tau)
 * 
 */
    __pyx_v_rjj = (__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]);

    /* "scipy/linalg/_decomp_update.pyx":584
 *     for j in range(n):
 *         rjj = index2(r, rs, j, j)[0]
 *         larfg(p+1, &rjj, col(u, us, j), us[0], &tau)             # <<<<<<<<<<<<<<
 * 
 *         # here we apply the reflector by hand instead of calling larf
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_larfg((__pyx_v_p + 1), (&__pyx_v_rjj), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), (&__pyx_v_tau));

    /* "scipy/linalg/_decomp_update.pyx":590
 *         # are separate.  This also permits the reflector to always be
 *         # p+1 long, rather than having a max of n+p.
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             copy(n-j-1, index2(r, rs, j, j+1), rs[1], work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])
 */
    __pyx_t_1 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":591
 *         # p+1 long, rather than having a max of n+p.
 *         if j+1 < n:
 *             copy(n-j-1, index2(r, rs, j, j+1), rs[1], work, 1)             # <<<<<<<<<<<<<<
 *             blas_t_conj(p, col(u, us, j), &us[0])
 *             gemv(T, p, n-j-1, 1, index2(u, us, 0, j+1), p, col(u, us, j), us[0],
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_copy(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_work, 1);

      /* "scipy/linalg/_decomp_update.pyx":592
 *         if j+1 < n:
 *             copy(n-j-1, index2(r, rs, j, j+1), rs[1], work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])             # <<<<<<<<<<<<<<
 *             gemv(T, p, n-j-1, 1, index2(u, us, 0, j+1), p, col(u, us, j), us[0],
 *                 1, work, 1)
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_p, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (&(__pyx_v_us[0])));

      /* "scipy/linalg/_decomp_update.pyx":593
 *             copy(n-j-1, index2(r, rs, j, j+1), rs[1], work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])
 *             gemv(T, p, n-j-1, 1, index2(u, us, 0, j+1), p, col(u, us, j), us[0],             # <<<<<<<<<<<<<<
 *                 1, work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_p, ((__pyx_v_n - __pyx_v_j) - 1), __pyx_t_float_complex_from_parts(1, 0), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, 0, (__pyx_v_j + 1)), __pyx_v_p, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), __pyx_t_float_complex_from_parts(1, 0), __pyx_v_work, 1);

      /* "scipy/linalg/_decomp_update.pyx":595
 *             gemv(T, p, n-j-1, 1, index2(u, us, 0, j+1), p, col(u, us, j), us[0],
 *                 1, work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])             # <<<<<<<<<<<<<<
 *             ger(p, n-j-1, -tau.conjugate(), col(u, us, j), us[0], work, 1,
 *                 index2(u, us, 0, j+1), p)
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_p, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (&(__pyx_v_us[0])));

      /* "scipy/linalg/_decomp_update.pyx":596
 *                 1, work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])
 *             ger(p, n-j-1, -tau.conjugate(), col(u, us, j), us[0], work, 1,             # <<<<<<<<<<<<<<
 *                 index2(u, us, 0, j+1), p)
 *             axpy(n-j-1, -tau.conjugate(), work, 1, index2(r, rs, j, j+1), rs[1])
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_ger(__pyx_v_p, ((__pyx_v_n - __pyx_v_j) - 1), __Pyx_c_neg_float(__Pyx_c_conj_float(__pyx_v_tau)), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), __pyx_v_work, 1, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, 0, (__pyx_v_j + 1)), __pyx_v_p);

      /* "scipy/linalg/_decomp_update.pyx":598
 *             ger(p, n-j-1, -tau.conjugate(), col(u, us, j), us[0], work, 1,
 *                 index2(u, us, 0, j+1), p)
 *             axpy(n-j-1, -tau.conjugate(), work, 1, index2(r, rs, j, j+1), rs[1])             # <<<<<<<<<<<<<<
 *         index2(r, rs, j, j)[0] = rjj
 * 
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_axpy(((__pyx_v_n - __pyx_v_j) - 1), __Pyx_c_neg_float(__Pyx_c_conj_float(__pyx_v_tau)), __pyx_v_work, 1, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]));

      /* "scipy/linalg/_decomp_update.pyx":590
 *         # are separate.  This also permits the reflector to always be
 *         # p+1 long, rather than having a max of n+p.
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             copy(n-j-1, index2(r, rs, j, j+1), rs[1], work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":599
 *                 index2(u, us, 0, j+1), p)
 *             axpy(n-j-1, -tau.conjugate(), work, 1, index2(r, rs, j, j+1), rs[1])
 *         index2(r, rs, j, j)[0] = rjj             # <<<<<<<<<<<<<<
 * 
 *         # now apply this reflector to q
 */
    (__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]) = __pyx_v_rjj;

    /* "scipy/linalg/_decomp_update.pyx":602
 * 
 *         # now apply this reflector to q
 *         copy(m, col(q, qs, j), qs[0], work, 1)             # <<<<<<<<<<<<<<
 *         gemv(N, m, p, 1, index2(q, qs, 0, n), m, col(u, us, j), us[0],
 *              1, work, 1)
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_v_work, 1);

    /* "scipy/linalg/_decomp_update.pyx":603
 *         # now apply this reflector to q
 *         copy(m, col(q, qs, j), qs[0], work, 1)
 *         gemv(N, m, p, 1, index2(q, qs, 0, n), m, col(u, us, j), us[0],             # <<<<<<<<<<<<<<
 *              1, work, 1)
 *         blas_t_conj(p, col(u, us, j), &us[0])
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_p, __pyx_t_float_complex_from_parts(1, 0), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_n), __pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), __pyx_t_float_complex_from_parts(1, 0), __pyx_v_work, 1);

    /* "scipy/linalg/_decomp_update.pyx":605
 *         gemv(N, m, p, 1, index2(q, qs, 0, n), m, col(u, us, j), us[0],
 *              1, work, 1)
 *         blas_t_conj(p, col(u, us, j), &us[0])             # <<<<<<<<<<<<<<
 *         ger(m, p, -tau, work, 1, col(u, us, j), us[0],
 *             index2(q, qs, 0, n), m)
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_p, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (&(__pyx_v_us[0])));

    /* "scipy/linalg/_decomp_update.pyx":606
 *              1, work, 1)
 *         blas_t_conj(p, col(u, us, j), &us[0])
 *         ger(m, p, -tau, work, 1, col(u, us, j), us[0],             # <<<<<<<<<<<<<<
 *             index2(q, qs, 0, n), m)
 *         axpy(m, -tau, work, 1, col(q, qs, j), qs[0])
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_ger(__pyx_v_m, __pyx_v_p, __Pyx_c_neg_float(__pyx_v_tau), __pyx_v_work, 1, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_n), __pyx_v_m);

    /* "scipy/linalg/_decomp_update.pyx":608
 *         ger(m, p, -tau, work, 1, col(u, us, j), us[0],
 *             index2(q, qs, 0, n), m)
 *         axpy(m, -tau, work, 1, col(q, qs, j), qs[0])             # <<<<<<<<<<<<<<
 * 
 *     # permute the rows of q, work columnwise, since q is fortran order
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_m, __Pyx_c_neg_float(__pyx_v_tau), __pyx_v_work, 1, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]));
  }

  /* "scipy/linalg/_decomp_update.pyx":611
 * 
 *     # permute the rows of q, work columnwise, since q is fortran order
 *     if k != m-p:             # <<<<<<<<<<<<<<
 *         for j in range(n):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 */
  __pyx_t_1 = ((__pyx_v_k != (__pyx_v_m - __pyx_v_p)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":612
 *     # permute the rows of q, work columnwise, since q is fortran order
 *     if k != m-p:
 *         for j in range(n):             # <<<<<<<<<<<<<<
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 */
    __pyx_t_2 = __pyx_v_n;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_j = __pyx_t_4;

      /* "scipy/linalg/_decomp_update.pyx":613
 *     if k != m-p:
 *         for j in range(n):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)             # <<<<<<<<<<<<<<
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_copy((__pyx_v_m - __pyx_v_k), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_k, __pyx_v_j), (__pyx_v_qs[0]), __pyx_v_work, 1);

      /* "scipy/linalg/_decomp_update.pyx":614
 *         for j in range(n):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])             # <<<<<<<<<<<<<<
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 * 
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_p, (__pyx_v_work + ((__pyx_v_m - __pyx_v_k) - __pyx_v_p)), 1, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_k, __pyx_v_j), (__pyx_v_qs[0]));

      /* "scipy/linalg/_decomp_update.pyx":615
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])             # <<<<<<<<<<<<<<
 * 
 *     libc.stdlib.free(work)
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_copy(((__pyx_v_m - __pyx_v_k) - __pyx_v_p), __pyx_v_work, 1, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, (__pyx_v_k + __pyx_v_p), __pyx_v_j), (__pyx_v_qs[0]));
    }

    /* "scipy/linalg/_decomp_update.pyx":611
 * 
 *     # permute the rows of q, work columnwise, since q is fortran order
 *     if k != m-p:             # <<<<<<<<<<<<<<
 *         for j in range(n):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":617
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 * 
 *     libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 * 
 * cdef int qr_block_row_insert(int m, int n, blas_t* q, int* qs,
 */
  free(__pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":564
 *         swap(m, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])
 * 
 * cdef int thin_qr_block_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                                   int* rs, blas_t* u, int* us, int k,
 *                                   int p) nogil:
 */

  /* function exit code */
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_block_row_insert(int __pyx_v_m, int __pyx_v_n, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_double_complex *__pyx_v_r, int *__pyx_v_rs, __pyx_t_double_complex *__pyx_v_u, int *__pyx_v_us, int __pyx_v_k, int __pyx_v_p) {
  int __pyx_v_j;
  __pyx_t_double_complex __pyx_v_rjj;
  __pyx_t_double_complex __pyx_v_tau;
  __pyx_t_double_complex *__pyx_v_work;
  char *__pyx_v_T;
  char *__pyx_v_N;
  size_t __pyx_v_worksize;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":571
 *     cdef blas_t rjj, tau
 *     cdef blas_t* work
 *     cdef char* T = 'T'             # <<<<<<<<<<<<<<
 *     cdef char* N = 'N'
 *     cdef size_t worksize = m * sizeof(blas_t)
 */
  __pyx_v_T = ((char *)"T");

  /* "scipy/linalg/_decomp_update.pyx":572
 *     cdef blas_t* work
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'             # <<<<<<<<<<<<<<
 *     cdef size_t worksize = m * sizeof(blas_t)
 * 
 */
  __pyx_v_N = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":573
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'
 *     cdef size_t worksize = m * sizeof(blas_t)             # <<<<<<<<<<<<<<
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize)
 */
  __pyx_v_worksize = (__pyx_v_m * (sizeof(__pyx_t_double_complex)));

  /* "scipy/linalg/_decomp_update.pyx":575
 *     cdef size_t worksize = m * sizeof(blas_t)
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize)             # <<<<<<<<<<<<<<
 *     if not work:
 *         return MEMORY_ERROR
 */
  __pyx_v_work = ((__pyx_t_double_complex *)malloc(__pyx_v_worksize));

  /* "scipy/linalg/_decomp_update.pyx":576
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize)
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  __pyx_t_1 = ((!(__pyx_v_work != 0)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":577
 *     work = <blas_t*>libc.stdlib.malloc(worksize)
 *     if not work:
 *         return MEMORY_ERROR             # <<<<<<<<<<<<<<
 * 
 *     # possible FIX
 */
    __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":576
 * 
 *     work = <blas_t*>libc.stdlib.malloc(worksize)
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":582
 *     # as this is written it requires F order q, r, and u.  But thats not
 *     # strictly necessary. C order should also work too with a little fiddling.
 *     for j in range(n):             # <<<<<<<<<<<<<<
 *         rjj = index2(r, rs, j, j)[0]
 *         larfg(p+1, &rjj, col(u, us, j), us[0], &tau)
 */
  __pyx_t_2 = __pyx_v_n;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_j = __pyx_t_4;

    /* "scipy/linalg/_decomp_update.pyx":583
 *     # strictly necessary. C order should also work too with a little fiddling.
 *     for j in range(n):
 *         rjj = index2(r, rs, j, j)[0]             # <<<<<<<<<<<<<<
 *         larfg(p+1, &rjj, col(u, us, j), us[0], &tau)
 * 
 */
    __pyx_v_rjj = (__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]);

    /* "scipy/linalg/_decomp_update.pyx":584
 *     for j in range(n):
 *         rjj = index2(r, rs, j, j)[0]
 *         larfg(p+1, &rjj, col(u, us, j), us[0], &tau)             # <<<<<<<<<<<<<<
 * 
 *         # here we apply the reflector by hand instead of calling larf
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_larfg((__pyx_v_p + 1), (&__pyx_v_rjj), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), (&__pyx_v_tau));

    /* "scipy/linalg/_decomp_update.pyx":590
 *         # are separate.  This also permits the reflector to always be
 *         # p+1 long, rather than having a max of n+p.
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             copy(n-j-1, index2(r, rs, j, j+1), rs[1], work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])
 */
    __pyx_t_1 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":591
 *         # p+1 long, rather than having a max of n+p.
 *         if j+1 < n:
 *             copy(n-j-1, index2(r, rs, j, j+1), rs[1], work, 1)             # <<<<<<<<<<<<<<
 *             blas_t_conj(p, col(u, us, j), &us[0])
 *             gemv(T, p, n-j-1, 1, index2(u, us, 0, j+1), p, col(u, us, j), us[0],
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_copy(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_work, 1);

      /* "scipy/linalg/_decomp_update.pyx":592
 *         if j+1 < n:
 *             copy(n-j-1, index2(r, rs, j, j+1), rs[1], work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])             # <<<<<<<<<<<<<<
 *             gemv(T, p, n-j-1, 1, index2(u, us, 0, j+1), p, col(u, us, j), us[0],
 *                 1, work, 1)
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_p, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (&(__pyx_v_us[0])));

      /* "scipy/linalg/_decomp_update.pyx":593
 *             copy(n-j-1, index2(r, rs, j, j+1), rs[1], work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])
 *             gemv(T, p, n-j-1, 1, index2(u, us, 0, j+1), p, col(u, us, j), us[0],             # <<<<<<<<<<<<<<
 *                 1, work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_p, ((__pyx_v_n - __pyx_v_j) - 1), __pyx_t_double_complex_from_parts(1, 0), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, 0, (__pyx_v_j + 1)), __pyx_v_p, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), __pyx_t_double_complex_from_parts(1, 0), __pyx_v_work, 1);

      /* "scipy/linalg/_decomp_update.pyx":595
 *             gemv(T, p, n-j-1, 1, index2(u, us, 0, j+1), p, col(u, us, j), us[0],
 *                 1, work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])             # <<<<<<<<<<<<<<
 *             ger(p, n-j-1, -tau.conjugate(), col(u, us, j), us[0], work, 1,
 *                 index2(u, us, 0, j+1), p)
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_p, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (&(__pyx_v_us[0])));

      /* "scipy/linalg/_decomp_update.pyx":596
 *                 1, work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])
 *             ger(p, n-j-1, -tau.conjugate(), col(u, us, j), us[0], work, 1,             # <<<<<<<<<<<<<<
 *                 index2(u, us, 0, j+1), p)
 *             axpy(n-j-1, -tau.conjugate(), work, 1, index2(r, rs, j, j+1), rs[1])
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_ger(__pyx_v_p, ((__pyx_v_n - __pyx_v_j) - 1), __Pyx_c_neg_double(__Pyx_c_conj_double(__pyx_v_tau)), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), __pyx_v_work, 1, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, 0, (__pyx_v_j + 1)), __pyx_v_p);

      /* "scipy/linalg/_decomp_update.pyx":598
 *             ger(p, n-j-1, -tau.conjugate(), col(u, us, j), us[0], work, 1,
 *                 index2(u, us, 0, j+1), p)
 *             axpy(n-j-1, -tau.conjugate(), work, 1, index2(r, rs, j, j+1), rs[1])             # <<<<<<<<<<<<<<
 *         index2(r, rs, j, j)[0] = rjj
 * 
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_axpy(((__pyx_v_n - __pyx_v_j) - 1), __Pyx_c_neg_double(__Pyx_c_conj_double(__pyx_v_tau)), __pyx_v_work, 1, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]));

      /* "scipy/linalg/_decomp_update.pyx":590
 *         # are separate.  This also permits the reflector to always be
 *         # p+1 long, rather than having a max of n+p.
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             copy(n-j-1, index2(r, rs, j, j+1), rs[1], work, 1)
 *             blas_t_conj(p, col(u, us, j), &us[0])
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":599
 *                 index2(u, us, 0, j+1), p)
 *             axpy(n-j-1, -tau.conjugate(), work, 1, index2(r, rs, j, j+1), rs[1])
 *         index2(r, rs, j, j)[0] = rjj             # <<<<<<<<<<<<<<
 * 
 *         # now apply this reflector to q
 */
    (__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]) = __pyx_v_rjj;

    /* "scipy/linalg/_decomp_update.pyx":602
 * 
 *         # now apply this reflector to q
 *         copy(m, col(q, qs, j), qs[0], work, 1)             # <<<<<<<<<<<<<<
 *         gemv(N, m, p, 1, index2(q, qs, 0, n), m, col(u, us, j), us[0],
 *              1, work, 1)
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_v_work, 1);

    /* "scipy/linalg/_decomp_update.pyx":603
 *         # now apply this reflector to q
 *         copy(m, col(q, qs, j), qs[0], work, 1)
 *         gemv(N, m, p, 1, index2(q, qs, 0, n), m, col(u, us, j), us[0],             # <<<<<<<<<<<<<<
 *              1, work, 1)
 *         blas_t_conj(p, col(u, us, j), &us[0])
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_p, __pyx_t_double_complex_from_parts(1, 0), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_n), __pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), __pyx_t_double_complex_from_parts(1, 0), __pyx_v_work, 1);

    /* "scipy/linalg/_decomp_update.pyx":605
 *         gemv(N, m, p, 1, index2(q, qs, 0, n), m, col(u, us, j), us[0],
 *              1, work, 1)
 *         blas_t_conj(p, col(u, us, j), &us[0])             # <<<<<<<<<<<<<<
 *         ger(m, p, -tau, work, 1, col(u, us, j), us[0],
 *             index2(q, qs, 0, n), m)
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_p, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (&(__pyx_v_us[0])));

    /* "scipy/linalg/_decomp_update.pyx":606
 *              1, work, 1)
 *         blas_t_conj(p, col(u, us, j), &us[0])
 *         ger(m, p, -tau, work, 1, col(u, us, j), us[0],             # <<<<<<<<<<<<<<
 *             index2(q, qs, 0, n), m)
 *         axpy(m, -tau, work, 1, col(q, qs, j), qs[0])
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_ger(__pyx_v_m, __pyx_v_p, __Pyx_c_neg_double(__pyx_v_tau), __pyx_v_work, 1, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_n), __pyx_v_m);

    /* "scipy/linalg/_decomp_update.pyx":608
 *         ger(m, p, -tau, work, 1, col(u, us, j), us[0],
 *             index2(q, qs, 0, n), m)
 *         axpy(m, -tau, work, 1, col(q, qs, j), qs[0])             # <<<<<<<<<<<<<<
 * 
 *     # permute the rows of q, work columnwise, since q is fortran order
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_m, __Pyx_c_neg_double(__pyx_v_tau), __pyx_v_work, 1, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]));
  }

  /* "scipy/linalg/_decomp_update.pyx":611
 * 
 *     # permute the rows of q, work columnwise, since q is fortran order
 *     if k != m-p:             # <<<<<<<<<<<<<<
 *         for j in range(n):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 */
  __pyx_t_1 = ((__pyx_v_k != (__pyx_v_m - __pyx_v_p)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":612
 *     # permute the rows of q, work columnwise, since q is fortran order
 *     if k != m-p:
 *         for j in range(n):             # <<<<<<<<<<<<<<
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 */
    __pyx_t_2 = __pyx_v_n;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_j = __pyx_t_4;

      /* "scipy/linalg/_decomp_update.pyx":613
 *     if k != m-p:
 *         for j in range(n):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)             # <<<<<<<<<<<<<<
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_copy((__pyx_v_m - __pyx_v_k), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_k, __pyx_v_j), (__pyx_v_qs[0]), __pyx_v_work, 1);

      /* "scipy/linalg/_decomp_update.pyx":614
 *         for j in range(n):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])             # <<<<<<<<<<<<<<
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 * 
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_p, (__pyx_v_work + ((__pyx_v_m - __pyx_v_k) - __pyx_v_p)), 1, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_k, __pyx_v_j), (__pyx_v_qs[0]));

      /* "scipy/linalg/_decomp_update.pyx":615
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])             # <<<<<<<<<<<<<<
 * 
 *     libc.stdlib.free(work)
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_copy(((__pyx_v_m - __pyx_v_k) - __pyx_v_p), __pyx_v_work, 1, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, (__pyx_v_k + __pyx_v_p), __pyx_v_j), (__pyx_v_qs[0]));
    }

    /* "scipy/linalg/_decomp_update.pyx":611
 * 
 *     # permute the rows of q, work columnwise, since q is fortran order
 *     if k != m-p:             # <<<<<<<<<<<<<<
 *         for j in range(n):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":617
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 * 
 *     libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 * 
 * cdef int qr_block_row_insert(int m, int n, blas_t* q, int* qs,
 */
  free(__pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":564
 *         swap(m, row(q, qs, j), qs[1], row(q, qs, j-1), qs[1])
 * 
 * cdef int thin_qr_block_row_insert(int m, int n, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                                   int* rs, blas_t* u, int* us, int k,
 *                                   int p) nogil:
 */

  /* function exit code */
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":619
 *     libc.stdlib.free(work)
 * 
 * cdef int qr_block_row_insert(int m, int n, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *                              blas_t* r, int* rs, int k, int p) nogil:
 *     # this should someday call lapack's xtpqrt (requires lapack >= 3.4
 */

static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_insert(int __pyx_v_m, int __pyx_v_n, float *__pyx_v_q, int *__pyx_v_qs, float *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p) {
  int __pyx_v_j;
  int __pyx_v_hlen;
  float __pyx_v_rjj;
  float __pyx_v_tau;
  float *__pyx_v_work;
  char *__pyx_v_sideL;
  char *__pyx_v_sideR;
  int __pyx_v_limit;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":638
 *     cdef blas_t rjj, tau
 *     cdef blas_t* work
 *     cdef char* sideL = 'L'             # <<<<<<<<<<<<<<
 *     cdef char* sideR = 'R'
 *     # for tall or sqr + rows should be n. for fat + rows should be new m
 */
  __pyx_v_sideL = ((char *)"L");

  /* "scipy/linalg/_decomp_update.pyx":639
 *     cdef blas_t* work
 *     cdef char* sideL = 'L'
 *     cdef char* sideR = 'R'             # <<<<<<<<<<<<<<
 *     # for tall or sqr + rows should be n. for fat + rows should be new m
 *     cdef int limit = min(m, n)
 */
  __pyx_v_sideR = ((char *)"R");

  /* "scipy/linalg/_decomp_update.pyx":641
 *     cdef char* sideR = 'R'
 *     # for tall or sqr + rows should be n. for fat + rows should be new m
 *     cdef int limit = min(m, n)             # <<<<<<<<<<<<<<
 * 
 *     work = <blas_t*>libc.stdlib.malloc(max(m,n)*sizeof(blas_t))
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_v_m;
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":643
 *     cdef int limit = min(m, n)
 * 
 *     work = <blas_t*>libc.stdlib.malloc(max(m,n)*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *     if not work:
 *         return MEMORY_ERROR
 */
  __pyx_t_3 = __pyx_v_n;
  __pyx_t_1 = __pyx_v_m;
  if (((__pyx_t_3 > __pyx_t_1) != 0)) {
    __pyx_t_2 = __pyx_t_3;
  } else {
    __pyx_t_2 = __pyx_t_1;
  }
  __pyx_v_work = ((float *)malloc((__pyx_t_2 * (sizeof(float)))));

  /* "scipy/linalg/_decomp_update.pyx":644
 * 
 *     work = <blas_t*>libc.stdlib.malloc(max(m,n)*sizeof(blas_t))
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  __pyx_t_4 = ((!(__pyx_v_work != 0)) != 0);
  if (__pyx_t_4) {

    /* "scipy/linalg/_decomp_update.pyx":645
 *     work = <blas_t*>libc.stdlib.malloc(max(m,n)*sizeof(blas_t))
 *     if not work:
 *         return MEMORY_ERROR             # <<<<<<<<<<<<<<
 * 
 *     for j in range(limit):
 */
    __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":644
 * 
 *     work = <blas_t*>libc.stdlib.malloc(max(m,n)*sizeof(blas_t))
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":647
 *         return MEMORY_ERROR
 * 
 *     for j in range(limit):             # <<<<<<<<<<<<<<
 *         rjj = index2(r, rs, j, j)[0]
 *         hlen = m-j
 */
  __pyx_t_2 = __pyx_v_limit;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_1 = 0; __pyx_t_1 < __pyx_t_3; __pyx_t_1+=1) {
    __pyx_v_j = __pyx_t_1;

    /* "scipy/linalg/_decomp_update.pyx":648
 * 
 *     for j in range(limit):
 *         rjj = index2(r, rs, j, j)[0]             # <<<<<<<<<<<<<<
 *         hlen = m-j
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 */
    __pyx_v_rjj = (__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]);

    /* "scipy/linalg/_decomp_update.pyx":649
 *     for j in range(limit):
 *         rjj = index2(r, rs, j, j)[0]
 *         hlen = m-j             # <<<<<<<<<<<<<<
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1
 */
    __pyx_v_hlen = (__pyx_v_m - __pyx_v_j);

    /* "scipy/linalg/_decomp_update.pyx":650
 *         rjj = index2(r, rs, j, j)[0]
 *         hlen = m-j
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)             # <<<<<<<<<<<<<<
 *         index2(r, rs, j, j)[0] = 1
 *         if j+1 < n:
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_larfg(__pyx_v_hlen, (&__pyx_v_rjj), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (__pyx_v_rs[0]), (&__pyx_v_tau));

    /* "scipy/linalg/_decomp_update.pyx":651
 *         hlen = m-j
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1             # <<<<<<<<<<<<<<
 *         if j+1 < n:
 *             larf(sideL, hlen, n-j-1, index2(r, rs, j, j), rs[0],
 */
    (__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]) = 1.0;

    /* "scipy/linalg/_decomp_update.pyx":652
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             larf(sideL, hlen, n-j-1, index2(r, rs, j, j), rs[0],
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 */
    __pyx_t_4 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
    if (__pyx_t_4) {

      /* "scipy/linalg/_decomp_update.pyx":653
 *         index2(r, rs, j, j)[0] = 1
 *         if j+1 < n:
 *             larf(sideL, hlen, n-j-1, index2(r, rs, j, j), rs[0],             # <<<<<<<<<<<<<<
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 *         larf(sideR, m, hlen, index2(r, rs, j, j), rs[0], tau,
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_larf(__pyx_v_sideL, __pyx_v_hlen, ((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[0]),  (__pyx_v_tau), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_work);

      /* "scipy/linalg/_decomp_update.pyx":652
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             larf(sideL, hlen, n-j-1, index2(r, rs, j, j), rs[0],
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":655
 *             larf(sideL, hlen, n-j-1, index2(r, rs, j, j), rs[0],
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 *         larf(sideR, m, hlen, index2(r, rs, j, j), rs[0], tau,             # <<<<<<<<<<<<<<
 *                 index2(q, qs, 0, j), qs[1], work)
 *         memset(index2(r, rs, j, j), 0, hlen*sizeof(blas_t))
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_larf(__pyx_v_sideR, __pyx_v_m, __pyx_v_hlen, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[0]), __pyx_v_tau, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_j), (__pyx_v_qs[1]), __pyx_v_work);

    /* "scipy/linalg/_decomp_update.pyx":657
 *         larf(sideR, m, hlen, index2(r, rs, j, j), rs[0], tau,
 *                 index2(q, qs, 0, j), qs[1], work)
 *         memset(index2(r, rs, j, j), 0, hlen*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *         index2(r, rs, j, j)[0] = rjj
 * 
 */
    (void)(memset(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), 0, (__pyx_v_hlen * (sizeof(float)))));

    /* "scipy/linalg/_decomp_update.pyx":658
 *                 index2(q, qs, 0, j), qs[1], work)
 *         memset(index2(r, rs, j, j), 0, hlen*sizeof(blas_t))
 *         index2(r, rs, j, j)[0] = rjj             # <<<<<<<<<<<<<<
 * 
 *     # permute the rows., work columnwise, since q is fortran order
 */
    (__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]) = __pyx_v_rjj;
  }

  /* "scipy/linalg/_decomp_update.pyx":661
 * 
 *     # permute the rows., work columnwise, since q is fortran order
 *     if k != m-p:             # <<<<<<<<<<<<<<
 *         for j in range(m):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 */
  __pyx_t_4 = ((__pyx_v_k != (__pyx_v_m - __pyx_v_p)) != 0);
  if (__pyx_t_4) {

    /* "scipy/linalg/_decomp_update.pyx":662
 *     # permute the rows., work columnwise, since q is fortran order
 *     if k != m-p:
 *         for j in range(m):             # <<<<<<<<<<<<<<
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 */
    __pyx_t_2 = __pyx_v_m;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_1 = 0; __pyx_t_1 < __pyx_t_3; __pyx_t_1+=1) {
      __pyx_v_j = __pyx_t_1;

      /* "scipy/linalg/_decomp_update.pyx":663
 *     if k != m-p:
 *         for j in range(m):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)             # <<<<<<<<<<<<<<
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_copy((__pyx_v_m - __pyx_v_k), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_k, __pyx_v_j), (__pyx_v_qs[0]), __pyx_v_work, 1);

      /* "scipy/linalg/_decomp_update.pyx":664
 *         for j in range(m):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])             # <<<<<<<<<<<<<<
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 * 
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_p, (__pyx_v_work + ((__pyx_v_m - __pyx_v_k) - __pyx_v_p)), 1, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_k, __pyx_v_j), (__pyx_v_qs[0]));

      /* "scipy/linalg/_decomp_update.pyx":665
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])             # <<<<<<<<<<<<<<
 * 
 *     libc.stdlib.free(work)
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_copy(((__pyx_v_m - __pyx_v_k) - __pyx_v_p), __pyx_v_work, 1, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, (__pyx_v_k + __pyx_v_p), __pyx_v_j), (__pyx_v_qs[0]));
    }

    /* "scipy/linalg/_decomp_update.pyx":661
 * 
 *     # permute the rows., work columnwise, since q is fortran order
 *     if k != m-p:             # <<<<<<<<<<<<<<
 *         for j in range(m):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":667
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 * 
 *     libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  free(__pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":668
 * 
 *     libc.stdlib.free(work)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef int thin_qr_col_insert(int m, int n, blas_t* q, int* qs, blas_t* r,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":619
 *     libc.stdlib.free(work)
 * 
 * cdef int qr_block_row_insert(int m, int n, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *                              blas_t* r, int* rs, int k, int p) nogil:
 *     # this should someday call lapack's xtpqrt (requires lapack >= 3.4
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_insert(int __pyx_v_m, int __pyx_v_n, double *__pyx_v_q, int *__pyx_v_qs, double *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p) {
  int __pyx_v_j;
  int __pyx_v_hlen;
  double __pyx_v_rjj;
  double __pyx_v_tau;
  double *__pyx_v_work;
  char *__pyx_v_sideL;
  char *__pyx_v_sideR;
  int __pyx_v_limit;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":638
 *     cdef blas_t rjj, tau
 *     cdef blas_t* work
 *     cdef char* sideL = 'L'             # <<<<<<<<<<<<<<
 *     cdef char* sideR = 'R'
 *     # for tall or sqr + rows should be n. for fat + rows should be new m
 */
  __pyx_v_sideL = ((char *)"L");

  /* "scipy/linalg/_decomp_update.pyx":639
 *     cdef blas_t* work
 *     cdef char* sideL = 'L'
 *     cdef char* sideR = 'R'             # <<<<<<<<<<<<<<
 *     # for tall or sqr + rows should be n. for fat + rows should be new m
 *     cdef int limit = min(m, n)
 */
  __pyx_v_sideR = ((char *)"R");

  /* "scipy/linalg/_decomp_update.pyx":641
 *     cdef char* sideR = 'R'
 *     # for tall or sqr + rows should be n. for fat + rows should be new m
 *     cdef int limit = min(m, n)             # <<<<<<<<<<<<<<
 * 
 *     work = <blas_t*>libc.stdlib.malloc(max(m,n)*sizeof(blas_t))
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_v_m;
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":643
 *     cdef int limit = min(m, n)
 * 
 *     work = <blas_t*>libc.stdlib.malloc(max(m,n)*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *     if not work:
 *         return MEMORY_ERROR
 */
  __pyx_t_3 = __pyx_v_n;
  __pyx_t_1 = __pyx_v_m;
  if (((__pyx_t_3 > __pyx_t_1) != 0)) {
    __pyx_t_2 = __pyx_t_3;
  } else {
    __pyx_t_2 = __pyx_t_1;
  }
  __pyx_v_work = ((double *)malloc((__pyx_t_2 * (sizeof(double)))));

  /* "scipy/linalg/_decomp_update.pyx":644
 * 
 *     work = <blas_t*>libc.stdlib.malloc(max(m,n)*sizeof(blas_t))
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  __pyx_t_4 = ((!(__pyx_v_work != 0)) != 0);
  if (__pyx_t_4) {

    /* "scipy/linalg/_decomp_update.pyx":645
 *     work = <blas_t*>libc.stdlib.malloc(max(m,n)*sizeof(blas_t))
 *     if not work:
 *         return MEMORY_ERROR             # <<<<<<<<<<<<<<
 * 
 *     for j in range(limit):
 */
    __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":644
 * 
 *     work = <blas_t*>libc.stdlib.malloc(max(m,n)*sizeof(blas_t))
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":647
 *         return MEMORY_ERROR
 * 
 *     for j in range(limit):             # <<<<<<<<<<<<<<
 *         rjj = index2(r, rs, j, j)[0]
 *         hlen = m-j
 */
  __pyx_t_2 = __pyx_v_limit;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_1 = 0; __pyx_t_1 < __pyx_t_3; __pyx_t_1+=1) {
    __pyx_v_j = __pyx_t_1;

    /* "scipy/linalg/_decomp_update.pyx":648
 * 
 *     for j in range(limit):
 *         rjj = index2(r, rs, j, j)[0]             # <<<<<<<<<<<<<<
 *         hlen = m-j
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 */
    __pyx_v_rjj = (__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]);

    /* "scipy/linalg/_decomp_update.pyx":649
 *     for j in range(limit):
 *         rjj = index2(r, rs, j, j)[0]
 *         hlen = m-j             # <<<<<<<<<<<<<<
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1
 */
    __pyx_v_hlen = (__pyx_v_m - __pyx_v_j);

    /* "scipy/linalg/_decomp_update.pyx":650
 *         rjj = index2(r, rs, j, j)[0]
 *         hlen = m-j
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)             # <<<<<<<<<<<<<<
 *         index2(r, rs, j, j)[0] = 1
 *         if j+1 < n:
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_larfg(__pyx_v_hlen, (&__pyx_v_rjj), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (__pyx_v_rs[0]), (&__pyx_v_tau));

    /* "scipy/linalg/_decomp_update.pyx":651
 *         hlen = m-j
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1             # <<<<<<<<<<<<<<
 *         if j+1 < n:
 *             larf(sideL, hlen, n-j-1, index2(r, rs, j, j), rs[0],
 */
    (__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]) = 1.0;

    /* "scipy/linalg/_decomp_update.pyx":652
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             larf(sideL, hlen, n-j-1, index2(r, rs, j, j), rs[0],
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 */
    __pyx_t_4 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
    if (__pyx_t_4) {

      /* "scipy/linalg/_decomp_update.pyx":653
 *         index2(r, rs, j, j)[0] = 1
 *         if j+1 < n:
 *             larf(sideL, hlen, n-j-1, index2(r, rs, j, j), rs[0],             # <<<<<<<<<<<<<<
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 *         larf(sideR, m, hlen, index2(r, rs, j, j), rs[0], tau,
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_larf(__pyx_v_sideL, __pyx_v_hlen, ((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[0]),  (__pyx_v_tau), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_work);

      /* "scipy/linalg/_decomp_update.pyx":652
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             larf(sideL, hlen, n-j-1, index2(r, rs, j, j), rs[0],
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":655
 *             larf(sideL, hlen, n-j-1, index2(r, rs, j, j), rs[0],
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 *         larf(sideR, m, hlen, index2(r, rs, j, j), rs[0], tau,             # <<<<<<<<<<<<<<
 *                 index2(q, qs, 0, j), qs[1], work)
 *         memset(index2(r, rs, j, j), 0, hlen*sizeof(blas_t))
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_larf(__pyx_v_sideR, __pyx_v_m, __pyx_v_hlen, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[0]), __pyx_v_tau, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_j), (__pyx_v_qs[1]), __pyx_v_work);

    /* "scipy/linalg/_decomp_update.pyx":657
 *         larf(sideR, m, hlen, index2(r, rs, j, j), rs[0], tau,
 *                 index2(q, qs, 0, j), qs[1], work)
 *         memset(index2(r, rs, j, j), 0, hlen*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *         index2(r, rs, j, j)[0] = rjj
 * 
 */
    (void)(memset(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), 0, (__pyx_v_hlen * (sizeof(double)))));

    /* "scipy/linalg/_decomp_update.pyx":658
 *                 index2(q, qs, 0, j), qs[1], work)
 *         memset(index2(r, rs, j, j), 0, hlen*sizeof(blas_t))
 *         index2(r, rs, j, j)[0] = rjj             # <<<<<<<<<<<<<<
 * 
 *     # permute the rows., work columnwise, since q is fortran order
 */
    (__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]) = __pyx_v_rjj;
  }

  /* "scipy/linalg/_decomp_update.pyx":661
 * 
 *     # permute the rows., work columnwise, since q is fortran order
 *     if k != m-p:             # <<<<<<<<<<<<<<
 *         for j in range(m):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 */
  __pyx_t_4 = ((__pyx_v_k != (__pyx_v_m - __pyx_v_p)) != 0);
  if (__pyx_t_4) {

    /* "scipy/linalg/_decomp_update.pyx":662
 *     # permute the rows., work columnwise, since q is fortran order
 *     if k != m-p:
 *         for j in range(m):             # <<<<<<<<<<<<<<
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 */
    __pyx_t_2 = __pyx_v_m;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_1 = 0; __pyx_t_1 < __pyx_t_3; __pyx_t_1+=1) {
      __pyx_v_j = __pyx_t_1;

      /* "scipy/linalg/_decomp_update.pyx":663
 *     if k != m-p:
 *         for j in range(m):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)             # <<<<<<<<<<<<<<
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_copy((__pyx_v_m - __pyx_v_k), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_k, __pyx_v_j), (__pyx_v_qs[0]), __pyx_v_work, 1);

      /* "scipy/linalg/_decomp_update.pyx":664
 *         for j in range(m):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])             # <<<<<<<<<<<<<<
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 * 
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_p, (__pyx_v_work + ((__pyx_v_m - __pyx_v_k) - __pyx_v_p)), 1, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_k, __pyx_v_j), (__pyx_v_qs[0]));

      /* "scipy/linalg/_decomp_update.pyx":665
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])             # <<<<<<<<<<<<<<
 * 
 *     libc.stdlib.free(work)
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_copy(((__pyx_v_m - __pyx_v_k) - __pyx_v_p), __pyx_v_work, 1, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, (__pyx_v_k + __pyx_v_p), __pyx_v_j), (__pyx_v_qs[0]));
    }

    /* "scipy/linalg/_decomp_update.pyx":661
 * 
 *     # permute the rows., work columnwise, since q is fortran order
 *     if k != m-p:             # <<<<<<<<<<<<<<
 *         for j in range(m):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":667
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 * 
 *     libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  free(__pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":668
 * 
 *     libc.stdlib.free(work)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef int thin_qr_col_insert(int m, int n, blas_t* q, int* qs, blas_t* r,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":619
 *     libc.stdlib.free(work)
 * 
 * cdef int qr_block_row_insert(int m, int n, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *                              blas_t* r, int* rs, int k, int p) nogil:
 *     # this should someday call lapack's xtpqrt (requires lapack >= 3.4
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_insert(int __pyx_v_m, int __pyx_v_n, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_float_complex *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p) {
  int __pyx_v_j;
  int __pyx_v_hlen;
  __pyx_t_float_complex __pyx_v_rjj;
  __pyx_t_float_complex __pyx_v_tau;
  __pyx_t_float_complex *__pyx_v_work;
  char *__pyx_v_sideL;
  char *__pyx_v_sideR;
  int __pyx_v_limit;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":638
 *     cdef blas_t rjj, tau
 *     cdef blas_t* work
 *     cdef char* sideL = 'L'             # <<<<<<<<<<<<<<
 *     cdef char* sideR = 'R'
 *     # for tall or sqr + rows should be n. for fat + rows should be new m
 */
  __pyx_v_sideL = ((char *)"L");

  /* "scipy/linalg/_decomp_update.pyx":639
 *     cdef blas_t* work
 *     cdef char* sideL = 'L'
 *     cdef char* sideR = 'R'             # <<<<<<<<<<<<<<
 *     # for tall or sqr + rows should be n. for fat + rows should be new m
 *     cdef int limit = min(m, n)
 */
  __pyx_v_sideR = ((char *)"R");

  /* "scipy/linalg/_decomp_update.pyx":641
 *     cdef char* sideR = 'R'
 *     # for tall or sqr + rows should be n. for fat + rows should be new m
 *     cdef int limit = min(m, n)             # <<<<<<<<<<<<<<
 * 
 *     work = <blas_t*>libc.stdlib.malloc(max(m,n)*sizeof(blas_t))
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_v_m;
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":643
 *     cdef int limit = min(m, n)
 * 
 *     work = <blas_t*>libc.stdlib.malloc(max(m,n)*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *     if not work:
 *         return MEMORY_ERROR
 */
  __pyx_t_3 = __pyx_v_n;
  __pyx_t_1 = __pyx_v_m;
  if (((__pyx_t_3 > __pyx_t_1) != 0)) {
    __pyx_t_2 = __pyx_t_3;
  } else {
    __pyx_t_2 = __pyx_t_1;
  }
  __pyx_v_work = ((__pyx_t_float_complex *)malloc((__pyx_t_2 * (sizeof(__pyx_t_float_complex)))));

  /* "scipy/linalg/_decomp_update.pyx":644
 * 
 *     work = <blas_t*>libc.stdlib.malloc(max(m,n)*sizeof(blas_t))
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  __pyx_t_4 = ((!(__pyx_v_work != 0)) != 0);
  if (__pyx_t_4) {

    /* "scipy/linalg/_decomp_update.pyx":645
 *     work = <blas_t*>libc.stdlib.malloc(max(m,n)*sizeof(blas_t))
 *     if not work:
 *         return MEMORY_ERROR             # <<<<<<<<<<<<<<
 * 
 *     for j in range(limit):
 */
    __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":644
 * 
 *     work = <blas_t*>libc.stdlib.malloc(max(m,n)*sizeof(blas_t))
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":647
 *         return MEMORY_ERROR
 * 
 *     for j in range(limit):             # <<<<<<<<<<<<<<
 *         rjj = index2(r, rs, j, j)[0]
 *         hlen = m-j
 */
  __pyx_t_2 = __pyx_v_limit;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_1 = 0; __pyx_t_1 < __pyx_t_3; __pyx_t_1+=1) {
    __pyx_v_j = __pyx_t_1;

    /* "scipy/linalg/_decomp_update.pyx":648
 * 
 *     for j in range(limit):
 *         rjj = index2(r, rs, j, j)[0]             # <<<<<<<<<<<<<<
 *         hlen = m-j
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 */
    __pyx_v_rjj = (__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]);

    /* "scipy/linalg/_decomp_update.pyx":649
 *     for j in range(limit):
 *         rjj = index2(r, rs, j, j)[0]
 *         hlen = m-j             # <<<<<<<<<<<<<<
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1
 */
    __pyx_v_hlen = (__pyx_v_m - __pyx_v_j);

    /* "scipy/linalg/_decomp_update.pyx":650
 *         rjj = index2(r, rs, j, j)[0]
 *         hlen = m-j
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)             # <<<<<<<<<<<<<<
 *         index2(r, rs, j, j)[0] = 1
 *         if j+1 < n:
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_larfg(__pyx_v_hlen, (&__pyx_v_rjj), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (__pyx_v_rs[0]), (&__pyx_v_tau));

    /* "scipy/linalg/_decomp_update.pyx":651
 *         hlen = m-j
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1             # <<<<<<<<<<<<<<
 *         if j+1 < n:
 *             larf(sideL, hlen, n-j-1, index2(r, rs, j, j), rs[0],
 */
    (__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]) = __pyx_t_float_complex_from_parts(1, 0);

    /* "scipy/linalg/_decomp_update.pyx":652
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             larf(sideL, hlen, n-j-1, index2(r, rs, j, j), rs[0],
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 */
    __pyx_t_4 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
    if (__pyx_t_4) {

      /* "scipy/linalg/_decomp_update.pyx":653
 *         index2(r, rs, j, j)[0] = 1
 *         if j+1 < n:
 *             larf(sideL, hlen, n-j-1, index2(r, rs, j, j), rs[0],             # <<<<<<<<<<<<<<
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 *         larf(sideR, m, hlen, index2(r, rs, j, j), rs[0], tau,
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_larf(__pyx_v_sideL, __pyx_v_hlen, ((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[0]), __Pyx_c_conj_float(__pyx_v_tau), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_work);

      /* "scipy/linalg/_decomp_update.pyx":652
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             larf(sideL, hlen, n-j-1, index2(r, rs, j, j), rs[0],
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":655
 *             larf(sideL, hlen, n-j-1, index2(r, rs, j, j), rs[0],
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 *         larf(sideR, m, hlen, index2(r, rs, j, j), rs[0], tau,             # <<<<<<<<<<<<<<
 *                 index2(q, qs, 0, j), qs[1], work)
 *         memset(index2(r, rs, j, j), 0, hlen*sizeof(blas_t))
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_larf(__pyx_v_sideR, __pyx_v_m, __pyx_v_hlen, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[0]), __pyx_v_tau, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_j), (__pyx_v_qs[1]), __pyx_v_work);

    /* "scipy/linalg/_decomp_update.pyx":657
 *         larf(sideR, m, hlen, index2(r, rs, j, j), rs[0], tau,
 *                 index2(q, qs, 0, j), qs[1], work)
 *         memset(index2(r, rs, j, j), 0, hlen*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *         index2(r, rs, j, j)[0] = rjj
 * 
 */
    (void)(memset(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), 0, (__pyx_v_hlen * (sizeof(__pyx_t_float_complex)))));

    /* "scipy/linalg/_decomp_update.pyx":658
 *                 index2(q, qs, 0, j), qs[1], work)
 *         memset(index2(r, rs, j, j), 0, hlen*sizeof(blas_t))
 *         index2(r, rs, j, j)[0] = rjj             # <<<<<<<<<<<<<<
 * 
 *     # permute the rows., work columnwise, since q is fortran order
 */
    (__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]) = __pyx_v_rjj;
  }

  /* "scipy/linalg/_decomp_update.pyx":661
 * 
 *     # permute the rows., work columnwise, since q is fortran order
 *     if k != m-p:             # <<<<<<<<<<<<<<
 *         for j in range(m):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 */
  __pyx_t_4 = ((__pyx_v_k != (__pyx_v_m - __pyx_v_p)) != 0);
  if (__pyx_t_4) {

    /* "scipy/linalg/_decomp_update.pyx":662
 *     # permute the rows., work columnwise, since q is fortran order
 *     if k != m-p:
 *         for j in range(m):             # <<<<<<<<<<<<<<
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 */
    __pyx_t_2 = __pyx_v_m;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_1 = 0; __pyx_t_1 < __pyx_t_3; __pyx_t_1+=1) {
      __pyx_v_j = __pyx_t_1;

      /* "scipy/linalg/_decomp_update.pyx":663
 *     if k != m-p:
 *         for j in range(m):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)             # <<<<<<<<<<<<<<
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_copy((__pyx_v_m - __pyx_v_k), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_k, __pyx_v_j), (__pyx_v_qs[0]), __pyx_v_work, 1);

      /* "scipy/linalg/_decomp_update.pyx":664
 *         for j in range(m):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])             # <<<<<<<<<<<<<<
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 * 
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_p, (__pyx_v_work + ((__pyx_v_m - __pyx_v_k) - __pyx_v_p)), 1, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_k, __pyx_v_j), (__pyx_v_qs[0]));

      /* "scipy/linalg/_decomp_update.pyx":665
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])             # <<<<<<<<<<<<<<
 * 
 *     libc.stdlib.free(work)
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_copy(((__pyx_v_m - __pyx_v_k) - __pyx_v_p), __pyx_v_work, 1, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, (__pyx_v_k + __pyx_v_p), __pyx_v_j), (__pyx_v_qs[0]));
    }

    /* "scipy/linalg/_decomp_update.pyx":661
 * 
 *     # permute the rows., work columnwise, since q is fortran order
 *     if k != m-p:             # <<<<<<<<<<<<<<
 *         for j in range(m):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":667
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 * 
 *     libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  free(__pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":668
 * 
 *     libc.stdlib.free(work)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef int thin_qr_col_insert(int m, int n, blas_t* q, int* qs, blas_t* r,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":619
 *     libc.stdlib.free(work)
 * 
 * cdef int qr_block_row_insert(int m, int n, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *                              blas_t* r, int* rs, int k, int p) nogil:
 *     # this should someday call lapack's xtpqrt (requires lapack >= 3.4
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_insert(int __pyx_v_m, int __pyx_v_n, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_double_complex *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p) {
  int __pyx_v_j;
  int __pyx_v_hlen;
  __pyx_t_double_complex __pyx_v_rjj;
  __pyx_t_double_complex __pyx_v_tau;
  __pyx_t_double_complex *__pyx_v_work;
  char *__pyx_v_sideL;
  char *__pyx_v_sideR;
  int __pyx_v_limit;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":638
 *     cdef blas_t rjj, tau
 *     cdef blas_t* work
 *     cdef char* sideL = 'L'             # <<<<<<<<<<<<<<
 *     cdef char* sideR = 'R'
 *     # for tall or sqr + rows should be n. for fat + rows should be new m
 */
  __pyx_v_sideL = ((char *)"L");

  /* "scipy/linalg/_decomp_update.pyx":639
 *     cdef blas_t* work
 *     cdef char* sideL = 'L'
 *     cdef char* sideR = 'R'             # <<<<<<<<<<<<<<
 *     # for tall or sqr + rows should be n. for fat + rows should be new m
 *     cdef int limit = min(m, n)
 */
  __pyx_v_sideR = ((char *)"R");

  /* "scipy/linalg/_decomp_update.pyx":641
 *     cdef char* sideR = 'R'
 *     # for tall or sqr + rows should be n. for fat + rows should be new m
 *     cdef int limit = min(m, n)             # <<<<<<<<<<<<<<
 * 
 *     work = <blas_t*>libc.stdlib.malloc(max(m,n)*sizeof(blas_t))
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_v_m;
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":643
 *     cdef int limit = min(m, n)
 * 
 *     work = <blas_t*>libc.stdlib.malloc(max(m,n)*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *     if not work:
 *         return MEMORY_ERROR
 */
  __pyx_t_3 = __pyx_v_n;
  __pyx_t_1 = __pyx_v_m;
  if (((__pyx_t_3 > __pyx_t_1) != 0)) {
    __pyx_t_2 = __pyx_t_3;
  } else {
    __pyx_t_2 = __pyx_t_1;
  }
  __pyx_v_work = ((__pyx_t_double_complex *)malloc((__pyx_t_2 * (sizeof(__pyx_t_double_complex)))));

  /* "scipy/linalg/_decomp_update.pyx":644
 * 
 *     work = <blas_t*>libc.stdlib.malloc(max(m,n)*sizeof(blas_t))
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  __pyx_t_4 = ((!(__pyx_v_work != 0)) != 0);
  if (__pyx_t_4) {

    /* "scipy/linalg/_decomp_update.pyx":645
 *     work = <blas_t*>libc.stdlib.malloc(max(m,n)*sizeof(blas_t))
 *     if not work:
 *         return MEMORY_ERROR             # <<<<<<<<<<<<<<
 * 
 *     for j in range(limit):
 */
    __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":644
 * 
 *     work = <blas_t*>libc.stdlib.malloc(max(m,n)*sizeof(blas_t))
 *     if not work:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":647
 *         return MEMORY_ERROR
 * 
 *     for j in range(limit):             # <<<<<<<<<<<<<<
 *         rjj = index2(r, rs, j, j)[0]
 *         hlen = m-j
 */
  __pyx_t_2 = __pyx_v_limit;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_1 = 0; __pyx_t_1 < __pyx_t_3; __pyx_t_1+=1) {
    __pyx_v_j = __pyx_t_1;

    /* "scipy/linalg/_decomp_update.pyx":648
 * 
 *     for j in range(limit):
 *         rjj = index2(r, rs, j, j)[0]             # <<<<<<<<<<<<<<
 *         hlen = m-j
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 */
    __pyx_v_rjj = (__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]);

    /* "scipy/linalg/_decomp_update.pyx":649
 *     for j in range(limit):
 *         rjj = index2(r, rs, j, j)[0]
 *         hlen = m-j             # <<<<<<<<<<<<<<
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1
 */
    __pyx_v_hlen = (__pyx_v_m - __pyx_v_j);

    /* "scipy/linalg/_decomp_update.pyx":650
 *         rjj = index2(r, rs, j, j)[0]
 *         hlen = m-j
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)             # <<<<<<<<<<<<<<
 *         index2(r, rs, j, j)[0] = 1
 *         if j+1 < n:
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_larfg(__pyx_v_hlen, (&__pyx_v_rjj), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (__pyx_v_rs[0]), (&__pyx_v_tau));

    /* "scipy/linalg/_decomp_update.pyx":651
 *         hlen = m-j
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1             # <<<<<<<<<<<<<<
 *         if j+1 < n:
 *             larf(sideL, hlen, n-j-1, index2(r, rs, j, j), rs[0],
 */
    (__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]) = __pyx_t_double_complex_from_parts(1, 0);

    /* "scipy/linalg/_decomp_update.pyx":652
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             larf(sideL, hlen, n-j-1, index2(r, rs, j, j), rs[0],
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 */
    __pyx_t_4 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
    if (__pyx_t_4) {

      /* "scipy/linalg/_decomp_update.pyx":653
 *         index2(r, rs, j, j)[0] = 1
 *         if j+1 < n:
 *             larf(sideL, hlen, n-j-1, index2(r, rs, j, j), rs[0],             # <<<<<<<<<<<<<<
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 *         larf(sideR, m, hlen, index2(r, rs, j, j), rs[0], tau,
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_larf(__pyx_v_sideL, __pyx_v_hlen, ((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[0]), __Pyx_c_conj_double(__pyx_v_tau), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_work);

      /* "scipy/linalg/_decomp_update.pyx":652
 *         larfg(hlen, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             larf(sideL, hlen, n-j-1, index2(r, rs, j, j), rs[0],
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":655
 *             larf(sideL, hlen, n-j-1, index2(r, rs, j, j), rs[0],
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 *         larf(sideR, m, hlen, index2(r, rs, j, j), rs[0], tau,             # <<<<<<<<<<<<<<
 *                 index2(q, qs, 0, j), qs[1], work)
 *         memset(index2(r, rs, j, j), 0, hlen*sizeof(blas_t))
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_larf(__pyx_v_sideR, __pyx_v_m, __pyx_v_hlen, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[0]), __pyx_v_tau, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_j), (__pyx_v_qs[1]), __pyx_v_work);

    /* "scipy/linalg/_decomp_update.pyx":657
 *         larf(sideR, m, hlen, index2(r, rs, j, j), rs[0], tau,
 *                 index2(q, qs, 0, j), qs[1], work)
 *         memset(index2(r, rs, j, j), 0, hlen*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *         index2(r, rs, j, j)[0] = rjj
 * 
 */
    (void)(memset(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), 0, (__pyx_v_hlen * (sizeof(__pyx_t_double_complex)))));

    /* "scipy/linalg/_decomp_update.pyx":658
 *                 index2(q, qs, 0, j), qs[1], work)
 *         memset(index2(r, rs, j, j), 0, hlen*sizeof(blas_t))
 *         index2(r, rs, j, j)[0] = rjj             # <<<<<<<<<<<<<<
 * 
 *     # permute the rows., work columnwise, since q is fortran order
 */
    (__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]) = __pyx_v_rjj;
  }

  /* "scipy/linalg/_decomp_update.pyx":661
 * 
 *     # permute the rows., work columnwise, since q is fortran order
 *     if k != m-p:             # <<<<<<<<<<<<<<
 *         for j in range(m):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 */
  __pyx_t_4 = ((__pyx_v_k != (__pyx_v_m - __pyx_v_p)) != 0);
  if (__pyx_t_4) {

    /* "scipy/linalg/_decomp_update.pyx":662
 *     # permute the rows., work columnwise, since q is fortran order
 *     if k != m-p:
 *         for j in range(m):             # <<<<<<<<<<<<<<
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 */
    __pyx_t_2 = __pyx_v_m;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_1 = 0; __pyx_t_1 < __pyx_t_3; __pyx_t_1+=1) {
      __pyx_v_j = __pyx_t_1;

      /* "scipy/linalg/_decomp_update.pyx":663
 *     if k != m-p:
 *         for j in range(m):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)             # <<<<<<<<<<<<<<
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_copy((__pyx_v_m - __pyx_v_k), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_k, __pyx_v_j), (__pyx_v_qs[0]), __pyx_v_work, 1);

      /* "scipy/linalg/_decomp_update.pyx":664
 *         for j in range(m):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])             # <<<<<<<<<<<<<<
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 * 
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_p, (__pyx_v_work + ((__pyx_v_m - __pyx_v_k) - __pyx_v_p)), 1, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, __pyx_v_k, __pyx_v_j), (__pyx_v_qs[0]));

      /* "scipy/linalg/_decomp_update.pyx":665
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 *             copy(p, work+(m-k-p), 1, index2(q, qs, k, j), qs[0])
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])             # <<<<<<<<<<<<<<
 * 
 *     libc.stdlib.free(work)
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_copy(((__pyx_v_m - __pyx_v_k) - __pyx_v_p), __pyx_v_work, 1, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, (__pyx_v_k + __pyx_v_p), __pyx_v_j), (__pyx_v_qs[0]));
    }

    /* "scipy/linalg/_decomp_update.pyx":661
 * 
 *     # permute the rows., work columnwise, since q is fortran order
 *     if k != m-p:             # <<<<<<<<<<<<<<
 *         for j in range(m):
 *             copy(m-k, index2(q, qs, k, j), qs[0], work, 1)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":667
 *             copy(m-k-p, work, 1, index2(q, qs, k+p, j), qs[0])
 * 
 *     libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  free(__pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":668
 * 
 *     libc.stdlib.free(work)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef int thin_qr_col_insert(int m, int n, blas_t* q, int* qs, blas_t* r,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":619
 *     libc.stdlib.free(work)
 * 
 * cdef int qr_block_row_insert(int m, int n, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *                              blas_t* r, int* rs, int k, int p) nogil:
 *     # this should someday call lapack's xtpqrt (requires lapack >= 3.4
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":670
 *     return 0
 * 
 * cdef int thin_qr_col_insert(int m, int n, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                              int* rs, blas_t* u, int* us, int k, int p_eco,
 *                              int p_full, blas_t* rcond) nogil:
 */

static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_col_insert(int __pyx_v_m, int __pyx_v_n, float *__pyx_v_q, int *__pyx_v_qs, float *__pyx_v_r, int *__pyx_v_rs, float *__pyx_v_u, int *__pyx_v_us, int __pyx_v_k, int __pyx_v_p_eco, int __pyx_v_p_full, float *__pyx_v_rcond) {
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_info;
  float __pyx_v_c;
  float __pyx_v_sn;
  CYTHON_UNUSED float __pyx_v_rc0;
  float __pyx_v_rc;
  float *__pyx_v_s;
  char *__pyx_v_N;
  char *__pyx_v_T;
  CYTHON_UNUSED char *__pyx_v_C;
  char *__pyx_v_TC;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;

  /* "scipy/linalg/_decomp_update.pyx":678
 *     cdef blas_t rc0, rc;
 *     cdef blas_t* s
 *     cdef char* N = 'N'             # <<<<<<<<<<<<<<
 *     cdef char* T = 'T'
 *     cdef char* C = 'C'
 */
  __pyx_v_N = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":679
 *     cdef blas_t* s
 *     cdef char* N = 'N'
 *     cdef char* T = 'T'             # <<<<<<<<<<<<<<
 *     cdef char* C = 'C'
 *     cdef char* TC
 */
  __pyx_v_T = ((char *)"T");

  /* "scipy/linalg/_decomp_update.pyx":680
 *     cdef char* N = 'N'
 *     cdef char* T = 'T'
 *     cdef char* C = 'C'             # <<<<<<<<<<<<<<
 *     cdef char* TC
 * 
 */
  __pyx_v_C = ((char *)"C");

  /* "scipy/linalg/_decomp_update.pyx":684
 * 
 *     if blas_t is float or blas_t is double:
 *         TC = T             # <<<<<<<<<<<<<<
 *         rc0 = rcond[0]
 *     elif blas_t is float_complex:
 */
  __pyx_v_TC = __pyx_v_T;

  /* "scipy/linalg/_decomp_update.pyx":685
 *     if blas_t is float or blas_t is double:
 *         TC = T
 *         rc0 = rcond[0]             # <<<<<<<<<<<<<<
 *     elif blas_t is float_complex:
 *         TC = C
 */
  __pyx_v_rc0 = (__pyx_v_rcond[0]);

  /* "scipy/linalg/_decomp_update.pyx":699
 *     # original system, not the new system.
 * 
 *     s = <blas_t*>libc.stdlib.malloc(2*(n+p_eco)*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *     if not s:
 *         return MEMORY_ERROR
 */
  __pyx_v_s = ((float *)malloc(((2 * (__pyx_v_n + __pyx_v_p_eco)) * (sizeof(float)))));

  /* "scipy/linalg/_decomp_update.pyx":700
 * 
 *     s = <blas_t*>libc.stdlib.malloc(2*(n+p_eco)*sizeof(blas_t))
 *     if not s:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  __pyx_t_1 = ((!(__pyx_v_s != 0)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":701
 *     s = <blas_t*>libc.stdlib.malloc(2*(n+p_eco)*sizeof(blas_t))
 *     if not s:
 *         return MEMORY_ERROR             # <<<<<<<<<<<<<<
 * 
 *     for j in range(p_eco):
 */
    __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":700
 * 
 *     s = <blas_t*>libc.stdlib.malloc(2*(n+p_eco)*sizeof(blas_t))
 *     if not s:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":703
 *         return MEMORY_ERROR
 * 
 *     for j in range(p_eco):             # <<<<<<<<<<<<<<
 *         rc = rcond[0]
 *         info = reorth(m, n+j, q, qs, True, col(u, us, j), us, s,  &rc)
 */
  __pyx_t_2 = __pyx_v_p_eco;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_j = __pyx_t_4;

    /* "scipy/linalg/_decomp_update.pyx":704
 * 
 *     for j in range(p_eco):
 *         rc = rcond[0]             # <<<<<<<<<<<<<<
 *         info = reorth(m, n+j, q, qs, True, col(u, us, j), us, s,  &rc)
 *         if info == 2:
 */
    __pyx_v_rc = (__pyx_v_rcond[0]);

    /* "scipy/linalg/_decomp_update.pyx":705
 *     for j in range(p_eco):
 *         rc = rcond[0]
 *         info = reorth(m, n+j, q, qs, True, col(u, us, j), us, s,  &rc)             # <<<<<<<<<<<<<<
 *         if info == 2:
 *             if blas_t is float or blas_t is double:
 */
    __pyx_v_info = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_reorth(__pyx_v_m, (__pyx_v_n + __pyx_v_j), __pyx_v_q, __pyx_v_qs, 1, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), __pyx_v_us, __pyx_v_s, (&__pyx_v_rc));

    /* "scipy/linalg/_decomp_update.pyx":706
 *         rc = rcond[0]
 *         info = reorth(m, n+j, q, qs, True, col(u, us, j), us, s,  &rc)
 *         if info == 2:             # <<<<<<<<<<<<<<
 *             if blas_t is float or blas_t is double:
 *                 rcond[0] = rc;
 */
    __pyx_t_1 = ((__pyx_v_info == 2) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":708
 *         if info == 2:
 *             if blas_t is float or blas_t is double:
 *                 rcond[0] = rc;             # <<<<<<<<<<<<<<
 *             elif blas_t is float_complex:
 *                 rcond[0] = (<float*>&rc)[0]
 */
      (__pyx_v_rcond[0]) = __pyx_v_rc;

      /* "scipy/linalg/_decomp_update.pyx":713
 *             else:
 *                 rcond[0] = (<double*>&rc)[0]
 *             libc.stdlib.free(s)             # <<<<<<<<<<<<<<
 *             return info
 *         copy(m, col(u, us, j), us[0], col(q, qs, n+j), qs[0])
 */
      free(__pyx_v_s);

      /* "scipy/linalg/_decomp_update.pyx":714
 *                 rcond[0] = (<double*>&rc)[0]
 *             libc.stdlib.free(s)
 *             return info             # <<<<<<<<<<<<<<
 *         copy(m, col(u, us, j), us[0], col(q, qs, n+j), qs[0])
 *         copy(n+j+1, s, 1, col(r, rs, k+j), rs[0])
 */
      __pyx_r = __pyx_v_info;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":706
 *         rc = rcond[0]
 *         info = reorth(m, n+j, q, qs, True, col(u, us, j), us, s,  &rc)
 *         if info == 2:             # <<<<<<<<<<<<<<
 *             if blas_t is float or blas_t is double:
 *                 rcond[0] = rc;
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":715
 *             libc.stdlib.free(s)
 *             return info
 *         copy(m, col(u, us, j), us[0], col(q, qs, n+j), qs[0])             # <<<<<<<<<<<<<<
 *         copy(n+j+1, s, 1, col(r, rs, k+j), rs[0])
 * 
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_n + __pyx_v_j)), (__pyx_v_qs[0]));

    /* "scipy/linalg/_decomp_update.pyx":716
 *             return info
 *         copy(m, col(u, us, j), us[0], col(q, qs, n+j), qs[0])
 *         copy(n+j+1, s, 1, col(r, rs, k+j), rs[0])             # <<<<<<<<<<<<<<
 * 
 *         for i in range(n-2+1, k-1, -1):
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_copy(((__pyx_v_n + __pyx_v_j) + 1), __pyx_v_s, 1, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, (__pyx_v_k + __pyx_v_j)), (__pyx_v_rs[0]));

    /* "scipy/linalg/_decomp_update.pyx":718
 *         copy(n+j+1, s, 1, col(r, rs, k+j), rs[0])
 * 
 *         for i in range(n-2+1, k-1, -1):             # <<<<<<<<<<<<<<
 *             lartg(index2(r, rs, i+j, k+j), index2(r, rs, i+j+1, k+j), &c, &sn)
 *             rot(n-i, index2(r, rs, i+j, i+p_eco+p_full), rs[1],
 */
    __pyx_t_5 = (__pyx_v_k - 1);
    __pyx_t_6 = __pyx_t_5;
    for (__pyx_t_7 = ((__pyx_v_n - 2) + 1); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
      __pyx_v_i = __pyx_t_7;

      /* "scipy/linalg/_decomp_update.pyx":719
 * 
 *         for i in range(n-2+1, k-1, -1):
 *             lartg(index2(r, rs, i+j, k+j), index2(r, rs, i+j+1, k+j), &c, &sn)             # <<<<<<<<<<<<<<
 *             rot(n-i, index2(r, rs, i+j, i+p_eco+p_full), rs[1],
 *                 index2(r, rs, i+j+1, i+p_eco+p_full), rs[1], c, sn)
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_i + __pyx_v_j), (__pyx_v_k + __pyx_v_j)), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, ((__pyx_v_i + __pyx_v_j) + 1), (__pyx_v_k + __pyx_v_j)), (&__pyx_v_c), (&__pyx_v_sn));

      /* "scipy/linalg/_decomp_update.pyx":720
 *         for i in range(n-2+1, k-1, -1):
 *             lartg(index2(r, rs, i+j, k+j), index2(r, rs, i+j+1, k+j), &c, &sn)
 *             rot(n-i, index2(r, rs, i+j, i+p_eco+p_full), rs[1],             # <<<<<<<<<<<<<<
 *                 index2(r, rs, i+j+1, i+p_eco+p_full), rs[1], c, sn)
 *             rot(m, col(q, qs, i+j), qs[0], col(q, qs, i+j+1), qs[0],
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_n - __pyx_v_i), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_i + __pyx_v_j), ((__pyx_v_i + __pyx_v_p_eco) + __pyx_v_p_full)), (__pyx_v_rs[1]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, ((__pyx_v_i + __pyx_v_j) + 1), ((__pyx_v_i + __pyx_v_p_eco) + __pyx_v_p_full)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_sn);

      /* "scipy/linalg/_decomp_update.pyx":722
 *             rot(n-i, index2(r, rs, i+j, i+p_eco+p_full), rs[1],
 *                 index2(r, rs, i+j+1, i+p_eco+p_full), rs[1], c, sn)
 *             rot(m, col(q, qs, i+j), qs[0], col(q, qs, i+j+1), qs[0],             # <<<<<<<<<<<<<<
 *                 c, sn.conjugate())
 *     libc.stdlib.free(s)
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_i + __pyx_v_j)), (__pyx_v_qs[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, ((__pyx_v_i + __pyx_v_j) + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_sn));
    }
  }

  /* "scipy/linalg/_decomp_update.pyx":724
 *             rot(m, col(q, qs, i+j), qs[0], col(q, qs, i+j+1), qs[0],
 *                 c, sn.conjugate())
 *     libc.stdlib.free(s)             # <<<<<<<<<<<<<<
 * 
 *     if p_full > 0:
 */
  free(__pyx_v_s);

  /* "scipy/linalg/_decomp_update.pyx":726
 *     libc.stdlib.free(s)
 * 
 *     if p_full > 0:             # <<<<<<<<<<<<<<
 *         # if this is true, we have ensured the u is also F contiguous.
 *         gemm(TC, N, m, p_full, m, 1, q, m, col(u, us, p_eco), m, 0,
 */
  __pyx_t_1 = ((__pyx_v_p_full > 0) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":728
 *     if p_full > 0:
 *         # if this is true, we have ensured the u is also F contiguous.
 *         gemm(TC, N, m, p_full, m, 1, q, m, col(u, us, p_eco), m, 0,             # <<<<<<<<<<<<<<
 *              col(r, rs, k+p_eco), m)
 *         qr_block_col_insert(m, n+p_eco+p_full, q, qs, r, rs, k+p_eco, p_full)
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemm(__pyx_v_TC, __pyx_v_N, __pyx_v_m, __pyx_v_p_full, __pyx_v_m, 1.0, __pyx_v_q, __pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_p_eco), __pyx_v_m, 0.0, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, (__pyx_v_k + __pyx_v_p_eco)), __pyx_v_m);

    /* "scipy/linalg/_decomp_update.pyx":730
 *         gemm(TC, N, m, p_full, m, 1, q, m, col(u, us, p_eco), m, 0,
 *              col(r, rs, k+p_eco), m)
 *         qr_block_col_insert(m, n+p_eco+p_full, q, qs, r, rs, k+p_eco, p_full)             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
    (void)(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_insert(__pyx_v_m, ((__pyx_v_n + __pyx_v_p_eco) + __pyx_v_p_full), __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, (__pyx_v_k + __pyx_v_p_eco), __pyx_v_p_full));

    /* "scipy/linalg/_decomp_update.pyx":726
 *     libc.stdlib.free(s)
 * 
 *     if p_full > 0:             # <<<<<<<<<<<<<<
 *         # if this is true, we have ensured the u is also F contiguous.
 *         gemm(TC, N, m, p_full, m, 1, q, m, col(u, us, p_eco), m, 0,
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":732
 *         qr_block_col_insert(m, n+p_eco+p_full, q, qs, r, rs, k+p_eco, p_full)
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef void qr_col_insert(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":670
 *     return 0
 * 
 * cdef int thin_qr_col_insert(int m, int n, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                              int* rs, blas_t* u, int* us, int k, int p_eco,
 *                              int p_full, blas_t* rcond) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_col_insert(int __pyx_v_m, int __pyx_v_n, double *__pyx_v_q, int *__pyx_v_qs, double *__pyx_v_r, int *__pyx_v_rs, double *__pyx_v_u, int *__pyx_v_us, int __pyx_v_k, int __pyx_v_p_eco, int __pyx_v_p_full, double *__pyx_v_rcond) {
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_info;
  double __pyx_v_c;
  double __pyx_v_sn;
  CYTHON_UNUSED double __pyx_v_rc0;
  double __pyx_v_rc;
  double *__pyx_v_s;
  char *__pyx_v_N;
  char *__pyx_v_T;
  CYTHON_UNUSED char *__pyx_v_C;
  char *__pyx_v_TC;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;

  /* "scipy/linalg/_decomp_update.pyx":678
 *     cdef blas_t rc0, rc;
 *     cdef blas_t* s
 *     cdef char* N = 'N'             # <<<<<<<<<<<<<<
 *     cdef char* T = 'T'
 *     cdef char* C = 'C'
 */
  __pyx_v_N = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":679
 *     cdef blas_t* s
 *     cdef char* N = 'N'
 *     cdef char* T = 'T'             # <<<<<<<<<<<<<<
 *     cdef char* C = 'C'
 *     cdef char* TC
 */
  __pyx_v_T = ((char *)"T");

  /* "scipy/linalg/_decomp_update.pyx":680
 *     cdef char* N = 'N'
 *     cdef char* T = 'T'
 *     cdef char* C = 'C'             # <<<<<<<<<<<<<<
 *     cdef char* TC
 * 
 */
  __pyx_v_C = ((char *)"C");

  /* "scipy/linalg/_decomp_update.pyx":684
 * 
 *     if blas_t is float or blas_t is double:
 *         TC = T             # <<<<<<<<<<<<<<
 *         rc0 = rcond[0]
 *     elif blas_t is float_complex:
 */
  __pyx_v_TC = __pyx_v_T;

  /* "scipy/linalg/_decomp_update.pyx":685
 *     if blas_t is float or blas_t is double:
 *         TC = T
 *         rc0 = rcond[0]             # <<<<<<<<<<<<<<
 *     elif blas_t is float_complex:
 *         TC = C
 */
  __pyx_v_rc0 = (__pyx_v_rcond[0]);

  /* "scipy/linalg/_decomp_update.pyx":699
 *     # original system, not the new system.
 * 
 *     s = <blas_t*>libc.stdlib.malloc(2*(n+p_eco)*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *     if not s:
 *         return MEMORY_ERROR
 */
  __pyx_v_s = ((double *)malloc(((2 * (__pyx_v_n + __pyx_v_p_eco)) * (sizeof(double)))));

  /* "scipy/linalg/_decomp_update.pyx":700
 * 
 *     s = <blas_t*>libc.stdlib.malloc(2*(n+p_eco)*sizeof(blas_t))
 *     if not s:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  __pyx_t_1 = ((!(__pyx_v_s != 0)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":701
 *     s = <blas_t*>libc.stdlib.malloc(2*(n+p_eco)*sizeof(blas_t))
 *     if not s:
 *         return MEMORY_ERROR             # <<<<<<<<<<<<<<
 * 
 *     for j in range(p_eco):
 */
    __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":700
 * 
 *     s = <blas_t*>libc.stdlib.malloc(2*(n+p_eco)*sizeof(blas_t))
 *     if not s:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":703
 *         return MEMORY_ERROR
 * 
 *     for j in range(p_eco):             # <<<<<<<<<<<<<<
 *         rc = rcond[0]
 *         info = reorth(m, n+j, q, qs, True, col(u, us, j), us, s,  &rc)
 */
  __pyx_t_2 = __pyx_v_p_eco;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_j = __pyx_t_4;

    /* "scipy/linalg/_decomp_update.pyx":704
 * 
 *     for j in range(p_eco):
 *         rc = rcond[0]             # <<<<<<<<<<<<<<
 *         info = reorth(m, n+j, q, qs, True, col(u, us, j), us, s,  &rc)
 *         if info == 2:
 */
    __pyx_v_rc = (__pyx_v_rcond[0]);

    /* "scipy/linalg/_decomp_update.pyx":705
 *     for j in range(p_eco):
 *         rc = rcond[0]
 *         info = reorth(m, n+j, q, qs, True, col(u, us, j), us, s,  &rc)             # <<<<<<<<<<<<<<
 *         if info == 2:
 *             if blas_t is float or blas_t is double:
 */
    __pyx_v_info = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_reorth(__pyx_v_m, (__pyx_v_n + __pyx_v_j), __pyx_v_q, __pyx_v_qs, 1, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), __pyx_v_us, __pyx_v_s, (&__pyx_v_rc));

    /* "scipy/linalg/_decomp_update.pyx":706
 *         rc = rcond[0]
 *         info = reorth(m, n+j, q, qs, True, col(u, us, j), us, s,  &rc)
 *         if info == 2:             # <<<<<<<<<<<<<<
 *             if blas_t is float or blas_t is double:
 *                 rcond[0] = rc;
 */
    __pyx_t_1 = ((__pyx_v_info == 2) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":708
 *         if info == 2:
 *             if blas_t is float or blas_t is double:
 *                 rcond[0] = rc;             # <<<<<<<<<<<<<<
 *             elif blas_t is float_complex:
 *                 rcond[0] = (<float*>&rc)[0]
 */
      (__pyx_v_rcond[0]) = __pyx_v_rc;

      /* "scipy/linalg/_decomp_update.pyx":713
 *             else:
 *                 rcond[0] = (<double*>&rc)[0]
 *             libc.stdlib.free(s)             # <<<<<<<<<<<<<<
 *             return info
 *         copy(m, col(u, us, j), us[0], col(q, qs, n+j), qs[0])
 */
      free(__pyx_v_s);

      /* "scipy/linalg/_decomp_update.pyx":714
 *                 rcond[0] = (<double*>&rc)[0]
 *             libc.stdlib.free(s)
 *             return info             # <<<<<<<<<<<<<<
 *         copy(m, col(u, us, j), us[0], col(q, qs, n+j), qs[0])
 *         copy(n+j+1, s, 1, col(r, rs, k+j), rs[0])
 */
      __pyx_r = __pyx_v_info;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":706
 *         rc = rcond[0]
 *         info = reorth(m, n+j, q, qs, True, col(u, us, j), us, s,  &rc)
 *         if info == 2:             # <<<<<<<<<<<<<<
 *             if blas_t is float or blas_t is double:
 *                 rcond[0] = rc;
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":715
 *             libc.stdlib.free(s)
 *             return info
 *         copy(m, col(u, us, j), us[0], col(q, qs, n+j), qs[0])             # <<<<<<<<<<<<<<
 *         copy(n+j+1, s, 1, col(r, rs, k+j), rs[0])
 * 
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_n + __pyx_v_j)), (__pyx_v_qs[0]));

    /* "scipy/linalg/_decomp_update.pyx":716
 *             return info
 *         copy(m, col(u, us, j), us[0], col(q, qs, n+j), qs[0])
 *         copy(n+j+1, s, 1, col(r, rs, k+j), rs[0])             # <<<<<<<<<<<<<<
 * 
 *         for i in range(n-2+1, k-1, -1):
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_copy(((__pyx_v_n + __pyx_v_j) + 1), __pyx_v_s, 1, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, (__pyx_v_k + __pyx_v_j)), (__pyx_v_rs[0]));

    /* "scipy/linalg/_decomp_update.pyx":718
 *         copy(n+j+1, s, 1, col(r, rs, k+j), rs[0])
 * 
 *         for i in range(n-2+1, k-1, -1):             # <<<<<<<<<<<<<<
 *             lartg(index2(r, rs, i+j, k+j), index2(r, rs, i+j+1, k+j), &c, &sn)
 *             rot(n-i, index2(r, rs, i+j, i+p_eco+p_full), rs[1],
 */
    __pyx_t_5 = (__pyx_v_k - 1);
    __pyx_t_6 = __pyx_t_5;
    for (__pyx_t_7 = ((__pyx_v_n - 2) + 1); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
      __pyx_v_i = __pyx_t_7;

      /* "scipy/linalg/_decomp_update.pyx":719
 * 
 *         for i in range(n-2+1, k-1, -1):
 *             lartg(index2(r, rs, i+j, k+j), index2(r, rs, i+j+1, k+j), &c, &sn)             # <<<<<<<<<<<<<<
 *             rot(n-i, index2(r, rs, i+j, i+p_eco+p_full), rs[1],
 *                 index2(r, rs, i+j+1, i+p_eco+p_full), rs[1], c, sn)
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_i + __pyx_v_j), (__pyx_v_k + __pyx_v_j)), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, ((__pyx_v_i + __pyx_v_j) + 1), (__pyx_v_k + __pyx_v_j)), (&__pyx_v_c), (&__pyx_v_sn));

      /* "scipy/linalg/_decomp_update.pyx":720
 *         for i in range(n-2+1, k-1, -1):
 *             lartg(index2(r, rs, i+j, k+j), index2(r, rs, i+j+1, k+j), &c, &sn)
 *             rot(n-i, index2(r, rs, i+j, i+p_eco+p_full), rs[1],             # <<<<<<<<<<<<<<
 *                 index2(r, rs, i+j+1, i+p_eco+p_full), rs[1], c, sn)
 *             rot(m, col(q, qs, i+j), qs[0], col(q, qs, i+j+1), qs[0],
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_n - __pyx_v_i), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_i + __pyx_v_j), ((__pyx_v_i + __pyx_v_p_eco) + __pyx_v_p_full)), (__pyx_v_rs[1]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, ((__pyx_v_i + __pyx_v_j) + 1), ((__pyx_v_i + __pyx_v_p_eco) + __pyx_v_p_full)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_sn);

      /* "scipy/linalg/_decomp_update.pyx":722
 *             rot(n-i, index2(r, rs, i+j, i+p_eco+p_full), rs[1],
 *                 index2(r, rs, i+j+1, i+p_eco+p_full), rs[1], c, sn)
 *             rot(m, col(q, qs, i+j), qs[0], col(q, qs, i+j+1), qs[0],             # <<<<<<<<<<<<<<
 *                 c, sn.conjugate())
 *     libc.stdlib.free(s)
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_i + __pyx_v_j)), (__pyx_v_qs[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, ((__pyx_v_i + __pyx_v_j) + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_sn));
    }
  }

  /* "scipy/linalg/_decomp_update.pyx":724
 *             rot(m, col(q, qs, i+j), qs[0], col(q, qs, i+j+1), qs[0],
 *                 c, sn.conjugate())
 *     libc.stdlib.free(s)             # <<<<<<<<<<<<<<
 * 
 *     if p_full > 0:
 */
  free(__pyx_v_s);

  /* "scipy/linalg/_decomp_update.pyx":726
 *     libc.stdlib.free(s)
 * 
 *     if p_full > 0:             # <<<<<<<<<<<<<<
 *         # if this is true, we have ensured the u is also F contiguous.
 *         gemm(TC, N, m, p_full, m, 1, q, m, col(u, us, p_eco), m, 0,
 */
  __pyx_t_1 = ((__pyx_v_p_full > 0) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":728
 *     if p_full > 0:
 *         # if this is true, we have ensured the u is also F contiguous.
 *         gemm(TC, N, m, p_full, m, 1, q, m, col(u, us, p_eco), m, 0,             # <<<<<<<<<<<<<<
 *              col(r, rs, k+p_eco), m)
 *         qr_block_col_insert(m, n+p_eco+p_full, q, qs, r, rs, k+p_eco, p_full)
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemm(__pyx_v_TC, __pyx_v_N, __pyx_v_m, __pyx_v_p_full, __pyx_v_m, 1.0, __pyx_v_q, __pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_p_eco), __pyx_v_m, 0.0, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, (__pyx_v_k + __pyx_v_p_eco)), __pyx_v_m);

    /* "scipy/linalg/_decomp_update.pyx":730
 *         gemm(TC, N, m, p_full, m, 1, q, m, col(u, us, p_eco), m, 0,
 *              col(r, rs, k+p_eco), m)
 *         qr_block_col_insert(m, n+p_eco+p_full, q, qs, r, rs, k+p_eco, p_full)             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
    (void)(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_insert(__pyx_v_m, ((__pyx_v_n + __pyx_v_p_eco) + __pyx_v_p_full), __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, (__pyx_v_k + __pyx_v_p_eco), __pyx_v_p_full));

    /* "scipy/linalg/_decomp_update.pyx":726
 *     libc.stdlib.free(s)
 * 
 *     if p_full > 0:             # <<<<<<<<<<<<<<
 *         # if this is true, we have ensured the u is also F contiguous.
 *         gemm(TC, N, m, p_full, m, 1, q, m, col(u, us, p_eco), m, 0,
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":732
 *         qr_block_col_insert(m, n+p_eco+p_full, q, qs, r, rs, k+p_eco, p_full)
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef void qr_col_insert(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":670
 *     return 0
 * 
 * cdef int thin_qr_col_insert(int m, int n, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                              int* rs, blas_t* u, int* us, int k, int p_eco,
 *                              int p_full, blas_t* rcond) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_col_insert(int __pyx_v_m, int __pyx_v_n, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_float_complex *__pyx_v_r, int *__pyx_v_rs, __pyx_t_float_complex *__pyx_v_u, int *__pyx_v_us, int __pyx_v_k, int __pyx_v_p_eco, int __pyx_v_p_full, __pyx_t_float_complex *__pyx_v_rcond) {
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_info;
  __pyx_t_float_complex __pyx_v_c;
  __pyx_t_float_complex __pyx_v_sn;
  CYTHON_UNUSED __pyx_t_float_complex __pyx_v_rc0;
  __pyx_t_float_complex __pyx_v_rc;
  __pyx_t_float_complex *__pyx_v_s;
  char *__pyx_v_N;
  CYTHON_UNUSED char *__pyx_v_T;
  char *__pyx_v_C;
  char *__pyx_v_TC;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;

  /* "scipy/linalg/_decomp_update.pyx":678
 *     cdef blas_t rc0, rc;
 *     cdef blas_t* s
 *     cdef char* N = 'N'             # <<<<<<<<<<<<<<
 *     cdef char* T = 'T'
 *     cdef char* C = 'C'
 */
  __pyx_v_N = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":679
 *     cdef blas_t* s
 *     cdef char* N = 'N'
 *     cdef char* T = 'T'             # <<<<<<<<<<<<<<
 *     cdef char* C = 'C'
 *     cdef char* TC
 */
  __pyx_v_T = ((char *)"T");

  /* "scipy/linalg/_decomp_update.pyx":680
 *     cdef char* N = 'N'
 *     cdef char* T = 'T'
 *     cdef char* C = 'C'             # <<<<<<<<<<<<<<
 *     cdef char* TC
 * 
 */
  __pyx_v_C = ((char *)"C");

  /* "scipy/linalg/_decomp_update.pyx":687
 *         rc0 = rcond[0]
 *     elif blas_t is float_complex:
 *         TC = C             # <<<<<<<<<<<<<<
 *         rc0 = (<float*>rcond)[0]
 *     else:
 */
  __pyx_v_TC = __pyx_v_C;

  /* "scipy/linalg/_decomp_update.pyx":688
 *     elif blas_t is float_complex:
 *         TC = C
 *         rc0 = (<float*>rcond)[0]             # <<<<<<<<<<<<<<
 *     else:
 *         TC = C
 */
  __pyx_v_rc0 = __pyx_t_float_complex_from_parts((((float *)__pyx_v_rcond)[0]), 0);

  /* "scipy/linalg/_decomp_update.pyx":699
 *     # original system, not the new system.
 * 
 *     s = <blas_t*>libc.stdlib.malloc(2*(n+p_eco)*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *     if not s:
 *         return MEMORY_ERROR
 */
  __pyx_v_s = ((__pyx_t_float_complex *)malloc(((2 * (__pyx_v_n + __pyx_v_p_eco)) * (sizeof(__pyx_t_float_complex)))));

  /* "scipy/linalg/_decomp_update.pyx":700
 * 
 *     s = <blas_t*>libc.stdlib.malloc(2*(n+p_eco)*sizeof(blas_t))
 *     if not s:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  __pyx_t_1 = ((!(__pyx_v_s != 0)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":701
 *     s = <blas_t*>libc.stdlib.malloc(2*(n+p_eco)*sizeof(blas_t))
 *     if not s:
 *         return MEMORY_ERROR             # <<<<<<<<<<<<<<
 * 
 *     for j in range(p_eco):
 */
    __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":700
 * 
 *     s = <blas_t*>libc.stdlib.malloc(2*(n+p_eco)*sizeof(blas_t))
 *     if not s:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":703
 *         return MEMORY_ERROR
 * 
 *     for j in range(p_eco):             # <<<<<<<<<<<<<<
 *         rc = rcond[0]
 *         info = reorth(m, n+j, q, qs, True, col(u, us, j), us, s,  &rc)
 */
  __pyx_t_2 = __pyx_v_p_eco;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_j = __pyx_t_4;

    /* "scipy/linalg/_decomp_update.pyx":704
 * 
 *     for j in range(p_eco):
 *         rc = rcond[0]             # <<<<<<<<<<<<<<
 *         info = reorth(m, n+j, q, qs, True, col(u, us, j), us, s,  &rc)
 *         if info == 2:
 */
    __pyx_v_rc = (__pyx_v_rcond[0]);

    /* "scipy/linalg/_decomp_update.pyx":705
 *     for j in range(p_eco):
 *         rc = rcond[0]
 *         info = reorth(m, n+j, q, qs, True, col(u, us, j), us, s,  &rc)             # <<<<<<<<<<<<<<
 *         if info == 2:
 *             if blas_t is float or blas_t is double:
 */
    __pyx_v_info = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_reorth(__pyx_v_m, (__pyx_v_n + __pyx_v_j), __pyx_v_q, __pyx_v_qs, 1, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), __pyx_v_us, __pyx_v_s, (&__pyx_v_rc));

    /* "scipy/linalg/_decomp_update.pyx":706
 *         rc = rcond[0]
 *         info = reorth(m, n+j, q, qs, True, col(u, us, j), us, s,  &rc)
 *         if info == 2:             # <<<<<<<<<<<<<<
 *             if blas_t is float or blas_t is double:
 *                 rcond[0] = rc;
 */
    __pyx_t_1 = ((__pyx_v_info == 2) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":710
 *                 rcond[0] = rc;
 *             elif blas_t is float_complex:
 *                 rcond[0] = (<float*>&rc)[0]             # <<<<<<<<<<<<<<
 *             else:
 *                 rcond[0] = (<double*>&rc)[0]
 */
      (__pyx_v_rcond[0]) = __pyx_t_float_complex_from_parts((((float *)(&__pyx_v_rc))[0]), 0);

      /* "scipy/linalg/_decomp_update.pyx":713
 *             else:
 *                 rcond[0] = (<double*>&rc)[0]
 *             libc.stdlib.free(s)             # <<<<<<<<<<<<<<
 *             return info
 *         copy(m, col(u, us, j), us[0], col(q, qs, n+j), qs[0])
 */
      free(__pyx_v_s);

      /* "scipy/linalg/_decomp_update.pyx":714
 *                 rcond[0] = (<double*>&rc)[0]
 *             libc.stdlib.free(s)
 *             return info             # <<<<<<<<<<<<<<
 *         copy(m, col(u, us, j), us[0], col(q, qs, n+j), qs[0])
 *         copy(n+j+1, s, 1, col(r, rs, k+j), rs[0])
 */
      __pyx_r = __pyx_v_info;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":706
 *         rc = rcond[0]
 *         info = reorth(m, n+j, q, qs, True, col(u, us, j), us, s,  &rc)
 *         if info == 2:             # <<<<<<<<<<<<<<
 *             if blas_t is float or blas_t is double:
 *                 rcond[0] = rc;
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":715
 *             libc.stdlib.free(s)
 *             return info
 *         copy(m, col(u, us, j), us[0], col(q, qs, n+j), qs[0])             # <<<<<<<<<<<<<<
 *         copy(n+j+1, s, 1, col(r, rs, k+j), rs[0])
 * 
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_n + __pyx_v_j)), (__pyx_v_qs[0]));

    /* "scipy/linalg/_decomp_update.pyx":716
 *             return info
 *         copy(m, col(u, us, j), us[0], col(q, qs, n+j), qs[0])
 *         copy(n+j+1, s, 1, col(r, rs, k+j), rs[0])             # <<<<<<<<<<<<<<
 * 
 *         for i in range(n-2+1, k-1, -1):
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_copy(((__pyx_v_n + __pyx_v_j) + 1), __pyx_v_s, 1, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, (__pyx_v_k + __pyx_v_j)), (__pyx_v_rs[0]));

    /* "scipy/linalg/_decomp_update.pyx":718
 *         copy(n+j+1, s, 1, col(r, rs, k+j), rs[0])
 * 
 *         for i in range(n-2+1, k-1, -1):             # <<<<<<<<<<<<<<
 *             lartg(index2(r, rs, i+j, k+j), index2(r, rs, i+j+1, k+j), &c, &sn)
 *             rot(n-i, index2(r, rs, i+j, i+p_eco+p_full), rs[1],
 */
    __pyx_t_5 = (__pyx_v_k - 1);
    __pyx_t_6 = __pyx_t_5;
    for (__pyx_t_7 = ((__pyx_v_n - 2) + 1); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
      __pyx_v_i = __pyx_t_7;

      /* "scipy/linalg/_decomp_update.pyx":719
 * 
 *         for i in range(n-2+1, k-1, -1):
 *             lartg(index2(r, rs, i+j, k+j), index2(r, rs, i+j+1, k+j), &c, &sn)             # <<<<<<<<<<<<<<
 *             rot(n-i, index2(r, rs, i+j, i+p_eco+p_full), rs[1],
 *                 index2(r, rs, i+j+1, i+p_eco+p_full), rs[1], c, sn)
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_i + __pyx_v_j), (__pyx_v_k + __pyx_v_j)), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, ((__pyx_v_i + __pyx_v_j) + 1), (__pyx_v_k + __pyx_v_j)), (&__pyx_v_c), (&__pyx_v_sn));

      /* "scipy/linalg/_decomp_update.pyx":720
 *         for i in range(n-2+1, k-1, -1):
 *             lartg(index2(r, rs, i+j, k+j), index2(r, rs, i+j+1, k+j), &c, &sn)
 *             rot(n-i, index2(r, rs, i+j, i+p_eco+p_full), rs[1],             # <<<<<<<<<<<<<<
 *                 index2(r, rs, i+j+1, i+p_eco+p_full), rs[1], c, sn)
 *             rot(m, col(q, qs, i+j), qs[0], col(q, qs, i+j+1), qs[0],
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_n - __pyx_v_i), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_i + __pyx_v_j), ((__pyx_v_i + __pyx_v_p_eco) + __pyx_v_p_full)), (__pyx_v_rs[1]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, ((__pyx_v_i + __pyx_v_j) + 1), ((__pyx_v_i + __pyx_v_p_eco) + __pyx_v_p_full)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_sn);

      /* "scipy/linalg/_decomp_update.pyx":722
 *             rot(n-i, index2(r, rs, i+j, i+p_eco+p_full), rs[1],
 *                 index2(r, rs, i+j+1, i+p_eco+p_full), rs[1], c, sn)
 *             rot(m, col(q, qs, i+j), qs[0], col(q, qs, i+j+1), qs[0],             # <<<<<<<<<<<<<<
 *                 c, sn.conjugate())
 *     libc.stdlib.free(s)
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_i + __pyx_v_j)), (__pyx_v_qs[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, ((__pyx_v_i + __pyx_v_j) + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_float(__pyx_v_sn));
    }
  }

  /* "scipy/linalg/_decomp_update.pyx":724
 *             rot(m, col(q, qs, i+j), qs[0], col(q, qs, i+j+1), qs[0],
 *                 c, sn.conjugate())
 *     libc.stdlib.free(s)             # <<<<<<<<<<<<<<
 * 
 *     if p_full > 0:
 */
  free(__pyx_v_s);

  /* "scipy/linalg/_decomp_update.pyx":726
 *     libc.stdlib.free(s)
 * 
 *     if p_full > 0:             # <<<<<<<<<<<<<<
 *         # if this is true, we have ensured the u is also F contiguous.
 *         gemm(TC, N, m, p_full, m, 1, q, m, col(u, us, p_eco), m, 0,
 */
  __pyx_t_1 = ((__pyx_v_p_full > 0) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":728
 *     if p_full > 0:
 *         # if this is true, we have ensured the u is also F contiguous.
 *         gemm(TC, N, m, p_full, m, 1, q, m, col(u, us, p_eco), m, 0,             # <<<<<<<<<<<<<<
 *              col(r, rs, k+p_eco), m)
 *         qr_block_col_insert(m, n+p_eco+p_full, q, qs, r, rs, k+p_eco, p_full)
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemm(__pyx_v_TC, __pyx_v_N, __pyx_v_m, __pyx_v_p_full, __pyx_v_m, __pyx_t_float_complex_from_parts(1, 0), __pyx_v_q, __pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_p_eco), __pyx_v_m, __pyx_t_float_complex_from_parts(0, 0), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, (__pyx_v_k + __pyx_v_p_eco)), __pyx_v_m);

    /* "scipy/linalg/_decomp_update.pyx":730
 *         gemm(TC, N, m, p_full, m, 1, q, m, col(u, us, p_eco), m, 0,
 *              col(r, rs, k+p_eco), m)
 *         qr_block_col_insert(m, n+p_eco+p_full, q, qs, r, rs, k+p_eco, p_full)             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
    (void)(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_insert(__pyx_v_m, ((__pyx_v_n + __pyx_v_p_eco) + __pyx_v_p_full), __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, (__pyx_v_k + __pyx_v_p_eco), __pyx_v_p_full));

    /* "scipy/linalg/_decomp_update.pyx":726
 *     libc.stdlib.free(s)
 * 
 *     if p_full > 0:             # <<<<<<<<<<<<<<
 *         # if this is true, we have ensured the u is also F contiguous.
 *         gemm(TC, N, m, p_full, m, 1, q, m, col(u, us, p_eco), m, 0,
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":732
 *         qr_block_col_insert(m, n+p_eco+p_full, q, qs, r, rs, k+p_eco, p_full)
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef void qr_col_insert(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":670
 *     return 0
 * 
 * cdef int thin_qr_col_insert(int m, int n, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                              int* rs, blas_t* u, int* us, int k, int p_eco,
 *                              int p_full, blas_t* rcond) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_col_insert(int __pyx_v_m, int __pyx_v_n, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_double_complex *__pyx_v_r, int *__pyx_v_rs, __pyx_t_double_complex *__pyx_v_u, int *__pyx_v_us, int __pyx_v_k, int __pyx_v_p_eco, int __pyx_v_p_full, __pyx_t_double_complex *__pyx_v_rcond) {
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_info;
  __pyx_t_double_complex __pyx_v_c;
  __pyx_t_double_complex __pyx_v_sn;
  CYTHON_UNUSED __pyx_t_double_complex __pyx_v_rc0;
  __pyx_t_double_complex __pyx_v_rc;
  __pyx_t_double_complex *__pyx_v_s;
  char *__pyx_v_N;
  CYTHON_UNUSED char *__pyx_v_T;
  char *__pyx_v_C;
  char *__pyx_v_TC;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;

  /* "scipy/linalg/_decomp_update.pyx":678
 *     cdef blas_t rc0, rc;
 *     cdef blas_t* s
 *     cdef char* N = 'N'             # <<<<<<<<<<<<<<
 *     cdef char* T = 'T'
 *     cdef char* C = 'C'
 */
  __pyx_v_N = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":679
 *     cdef blas_t* s
 *     cdef char* N = 'N'
 *     cdef char* T = 'T'             # <<<<<<<<<<<<<<
 *     cdef char* C = 'C'
 *     cdef char* TC
 */
  __pyx_v_T = ((char *)"T");

  /* "scipy/linalg/_decomp_update.pyx":680
 *     cdef char* N = 'N'
 *     cdef char* T = 'T'
 *     cdef char* C = 'C'             # <<<<<<<<<<<<<<
 *     cdef char* TC
 * 
 */
  __pyx_v_C = ((char *)"C");

  /* "scipy/linalg/_decomp_update.pyx":690
 *         rc0 = (<float*>rcond)[0]
 *     else:
 *         TC = C             # <<<<<<<<<<<<<<
 *         rc0 = (<double*>rcond)[0]
 * 
 */
  __pyx_v_TC = __pyx_v_C;

  /* "scipy/linalg/_decomp_update.pyx":691
 *     else:
 *         TC = C
 *         rc0 = (<double*>rcond)[0]             # <<<<<<<<<<<<<<
 * 
 *     # on entry, Q and R have both been increased in size, Q via the appending
 */
  __pyx_v_rc0 = __pyx_t_double_complex_from_parts((((double *)__pyx_v_rcond)[0]), 0);

  /* "scipy/linalg/_decomp_update.pyx":699
 *     # original system, not the new system.
 * 
 *     s = <blas_t*>libc.stdlib.malloc(2*(n+p_eco)*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *     if not s:
 *         return MEMORY_ERROR
 */
  __pyx_v_s = ((__pyx_t_double_complex *)malloc(((2 * (__pyx_v_n + __pyx_v_p_eco)) * (sizeof(__pyx_t_double_complex)))));

  /* "scipy/linalg/_decomp_update.pyx":700
 * 
 *     s = <blas_t*>libc.stdlib.malloc(2*(n+p_eco)*sizeof(blas_t))
 *     if not s:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  __pyx_t_1 = ((!(__pyx_v_s != 0)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":701
 *     s = <blas_t*>libc.stdlib.malloc(2*(n+p_eco)*sizeof(blas_t))
 *     if not s:
 *         return MEMORY_ERROR             # <<<<<<<<<<<<<<
 * 
 *     for j in range(p_eco):
 */
    __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":700
 * 
 *     s = <blas_t*>libc.stdlib.malloc(2*(n+p_eco)*sizeof(blas_t))
 *     if not s:             # <<<<<<<<<<<<<<
 *         return MEMORY_ERROR
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":703
 *         return MEMORY_ERROR
 * 
 *     for j in range(p_eco):             # <<<<<<<<<<<<<<
 *         rc = rcond[0]
 *         info = reorth(m, n+j, q, qs, True, col(u, us, j), us, s,  &rc)
 */
  __pyx_t_2 = __pyx_v_p_eco;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_j = __pyx_t_4;

    /* "scipy/linalg/_decomp_update.pyx":704
 * 
 *     for j in range(p_eco):
 *         rc = rcond[0]             # <<<<<<<<<<<<<<
 *         info = reorth(m, n+j, q, qs, True, col(u, us, j), us, s,  &rc)
 *         if info == 2:
 */
    __pyx_v_rc = (__pyx_v_rcond[0]);

    /* "scipy/linalg/_decomp_update.pyx":705
 *     for j in range(p_eco):
 *         rc = rcond[0]
 *         info = reorth(m, n+j, q, qs, True, col(u, us, j), us, s,  &rc)             # <<<<<<<<<<<<<<
 *         if info == 2:
 *             if blas_t is float or blas_t is double:
 */
    __pyx_v_info = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_reorth(__pyx_v_m, (__pyx_v_n + __pyx_v_j), __pyx_v_q, __pyx_v_qs, 1, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), __pyx_v_us, __pyx_v_s, (&__pyx_v_rc));

    /* "scipy/linalg/_decomp_update.pyx":706
 *         rc = rcond[0]
 *         info = reorth(m, n+j, q, qs, True, col(u, us, j), us, s,  &rc)
 *         if info == 2:             # <<<<<<<<<<<<<<
 *             if blas_t is float or blas_t is double:
 *                 rcond[0] = rc;
 */
    __pyx_t_1 = ((__pyx_v_info == 2) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":712
 *                 rcond[0] = (<float*>&rc)[0]
 *             else:
 *                 rcond[0] = (<double*>&rc)[0]             # <<<<<<<<<<<<<<
 *             libc.stdlib.free(s)
 *             return info
 */
      (__pyx_v_rcond[0]) = __pyx_t_double_complex_from_parts((((double *)(&__pyx_v_rc))[0]), 0);

      /* "scipy/linalg/_decomp_update.pyx":713
 *             else:
 *                 rcond[0] = (<double*>&rc)[0]
 *             libc.stdlib.free(s)             # <<<<<<<<<<<<<<
 *             return info
 *         copy(m, col(u, us, j), us[0], col(q, qs, n+j), qs[0])
 */
      free(__pyx_v_s);

      /* "scipy/linalg/_decomp_update.pyx":714
 *                 rcond[0] = (<double*>&rc)[0]
 *             libc.stdlib.free(s)
 *             return info             # <<<<<<<<<<<<<<
 *         copy(m, col(u, us, j), us[0], col(q, qs, n+j), qs[0])
 *         copy(n+j+1, s, 1, col(r, rs, k+j), rs[0])
 */
      __pyx_r = __pyx_v_info;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":706
 *         rc = rcond[0]
 *         info = reorth(m, n+j, q, qs, True, col(u, us, j), us, s,  &rc)
 *         if info == 2:             # <<<<<<<<<<<<<<
 *             if blas_t is float or blas_t is double:
 *                 rcond[0] = rc;
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":715
 *             libc.stdlib.free(s)
 *             return info
 *         copy(m, col(u, us, j), us[0], col(q, qs, n+j), qs[0])             # <<<<<<<<<<<<<<
 *         copy(n+j+1, s, 1, col(r, rs, k+j), rs[0])
 * 
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_copy(__pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), (__pyx_v_us[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_n + __pyx_v_j)), (__pyx_v_qs[0]));

    /* "scipy/linalg/_decomp_update.pyx":716
 *             return info
 *         copy(m, col(u, us, j), us[0], col(q, qs, n+j), qs[0])
 *         copy(n+j+1, s, 1, col(r, rs, k+j), rs[0])             # <<<<<<<<<<<<<<
 * 
 *         for i in range(n-2+1, k-1, -1):
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_copy(((__pyx_v_n + __pyx_v_j) + 1), __pyx_v_s, 1, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, (__pyx_v_k + __pyx_v_j)), (__pyx_v_rs[0]));

    /* "scipy/linalg/_decomp_update.pyx":718
 *         copy(n+j+1, s, 1, col(r, rs, k+j), rs[0])
 * 
 *         for i in range(n-2+1, k-1, -1):             # <<<<<<<<<<<<<<
 *             lartg(index2(r, rs, i+j, k+j), index2(r, rs, i+j+1, k+j), &c, &sn)
 *             rot(n-i, index2(r, rs, i+j, i+p_eco+p_full), rs[1],
 */
    __pyx_t_5 = (__pyx_v_k - 1);
    __pyx_t_6 = __pyx_t_5;
    for (__pyx_t_7 = ((__pyx_v_n - 2) + 1); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
      __pyx_v_i = __pyx_t_7;

      /* "scipy/linalg/_decomp_update.pyx":719
 * 
 *         for i in range(n-2+1, k-1, -1):
 *             lartg(index2(r, rs, i+j, k+j), index2(r, rs, i+j+1, k+j), &c, &sn)             # <<<<<<<<<<<<<<
 *             rot(n-i, index2(r, rs, i+j, i+p_eco+p_full), rs[1],
 *                 index2(r, rs, i+j+1, i+p_eco+p_full), rs[1], c, sn)
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_i + __pyx_v_j), (__pyx_v_k + __pyx_v_j)), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, ((__pyx_v_i + __pyx_v_j) + 1), (__pyx_v_k + __pyx_v_j)), (&__pyx_v_c), (&__pyx_v_sn));

      /* "scipy/linalg/_decomp_update.pyx":720
 *         for i in range(n-2+1, k-1, -1):
 *             lartg(index2(r, rs, i+j, k+j), index2(r, rs, i+j+1, k+j), &c, &sn)
 *             rot(n-i, index2(r, rs, i+j, i+p_eco+p_full), rs[1],             # <<<<<<<<<<<<<<
 *                 index2(r, rs, i+j+1, i+p_eco+p_full), rs[1], c, sn)
 *             rot(m, col(q, qs, i+j), qs[0], col(q, qs, i+j+1), qs[0],
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_n - __pyx_v_i), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_i + __pyx_v_j), ((__pyx_v_i + __pyx_v_p_eco) + __pyx_v_p_full)), (__pyx_v_rs[1]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, ((__pyx_v_i + __pyx_v_j) + 1), ((__pyx_v_i + __pyx_v_p_eco) + __pyx_v_p_full)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_sn);

      /* "scipy/linalg/_decomp_update.pyx":722
 *             rot(n-i, index2(r, rs, i+j, i+p_eco+p_full), rs[1],
 *                 index2(r, rs, i+j+1, i+p_eco+p_full), rs[1], c, sn)
 *             rot(m, col(q, qs, i+j), qs[0], col(q, qs, i+j+1), qs[0],             # <<<<<<<<<<<<<<
 *                 c, sn.conjugate())
 *     libc.stdlib.free(s)
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_i + __pyx_v_j)), (__pyx_v_qs[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, ((__pyx_v_i + __pyx_v_j) + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_double(__pyx_v_sn));
    }
  }

  /* "scipy/linalg/_decomp_update.pyx":724
 *             rot(m, col(q, qs, i+j), qs[0], col(q, qs, i+j+1), qs[0],
 *                 c, sn.conjugate())
 *     libc.stdlib.free(s)             # <<<<<<<<<<<<<<
 * 
 *     if p_full > 0:
 */
  free(__pyx_v_s);

  /* "scipy/linalg/_decomp_update.pyx":726
 *     libc.stdlib.free(s)
 * 
 *     if p_full > 0:             # <<<<<<<<<<<<<<
 *         # if this is true, we have ensured the u is also F contiguous.
 *         gemm(TC, N, m, p_full, m, 1, q, m, col(u, us, p_eco), m, 0,
 */
  __pyx_t_1 = ((__pyx_v_p_full > 0) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":728
 *     if p_full > 0:
 *         # if this is true, we have ensured the u is also F contiguous.
 *         gemm(TC, N, m, p_full, m, 1, q, m, col(u, us, p_eco), m, 0,             # <<<<<<<<<<<<<<
 *              col(r, rs, k+p_eco), m)
 *         qr_block_col_insert(m, n+p_eco+p_full, q, qs, r, rs, k+p_eco, p_full)
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemm(__pyx_v_TC, __pyx_v_N, __pyx_v_m, __pyx_v_p_full, __pyx_v_m, __pyx_t_double_complex_from_parts(1, 0), __pyx_v_q, __pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_p_eco), __pyx_v_m, __pyx_t_double_complex_from_parts(0, 0), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_r, __pyx_v_rs, (__pyx_v_k + __pyx_v_p_eco)), __pyx_v_m);

    /* "scipy/linalg/_decomp_update.pyx":730
 *         gemm(TC, N, m, p_full, m, 1, q, m, col(u, us, p_eco), m, 0,
 *              col(r, rs, k+p_eco), m)
 *         qr_block_col_insert(m, n+p_eco+p_full, q, qs, r, rs, k+p_eco, p_full)             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
    (void)(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_insert(__pyx_v_m, ((__pyx_v_n + __pyx_v_p_eco) + __pyx_v_p_full), __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, (__pyx_v_k + __pyx_v_p_eco), __pyx_v_p_full));

    /* "scipy/linalg/_decomp_update.pyx":726
 *     libc.stdlib.free(s)
 * 
 *     if p_full > 0:             # <<<<<<<<<<<<<<
 *         # if this is true, we have ensured the u is also F contiguous.
 *         gemm(TC, N, m, p_full, m, 1, q, m, col(u, us, p_eco), m, 0,
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":732
 *         qr_block_col_insert(m, n+p_eco+p_full, q, qs, r, rs, k+p_eco, p_full)
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef void qr_col_insert(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":670
 *     return 0
 * 
 * cdef int thin_qr_col_insert(int m, int n, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                              int* rs, blas_t* u, int* us, int k, int p_eco,
 *                              int p_full, blas_t* rcond) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":734
 *     return 0
 * 
 * cdef void qr_col_insert(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                         int k) nogil:
 *     cdef int j
 */

static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_insert(int __pyx_v_m, int __pyx_v_n, float *__pyx_v_q, int *__pyx_v_qs, float *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k) {
  int __pyx_v_j;
  float __pyx_v_c;
  float __pyx_v_s;
  long __pyx_t_1;
  long __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":740
 *     cdef blas_t* work
 * 
 *     for j in range(m-2, k-1, -1):             # <<<<<<<<<<<<<<
 *         lartg(index2(r, rs, j, k), index2(r, rs, j+1, k), &c, &s)
 * 
 */
  __pyx_t_1 = (__pyx_v_k - 1);
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = (__pyx_v_m - 2); __pyx_t_3 > __pyx_t_2; __pyx_t_3-=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":741
 * 
 *     for j in range(m-2, k-1, -1):
 *         lartg(index2(r, rs, j, k), index2(r, rs, j+1, k), &c, &s)             # <<<<<<<<<<<<<<
 * 
 *         # update r if j is a nonzero row
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_k), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_k), (&__pyx_v_c), (&__pyx_v_s));

    /* "scipy/linalg/_decomp_update.pyx":744
 * 
 *         # update r if j is a nonzero row
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 */
    __pyx_t_4 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
    if (__pyx_t_4) {

      /* "scipy/linalg/_decomp_update.pyx":745
 *         # update r if j is a nonzero row
 *         if j+1 < n:
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],             # <<<<<<<<<<<<<<
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 * 
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

      /* "scipy/linalg/_decomp_update.pyx":744
 * 
 *         # update r if j is a nonzero row
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":749
 * 
 *         # update the columns of q
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())             # <<<<<<<<<<<<<<
 * 
 * cdef int qr_block_col_insert(int m, int n, blas_t* q, int* qs,
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_s));
  }

  /* "scipy/linalg/_decomp_update.pyx":734
 *     return 0
 * 
 * cdef void qr_col_insert(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                         int k) nogil:
 *     cdef int j
 */

  /* function exit code */
}

static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_insert(int __pyx_v_m, int __pyx_v_n, double *__pyx_v_q, int *__pyx_v_qs, double *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k) {
  int __pyx_v_j;
  double __pyx_v_c;
  double __pyx_v_s;
  long __pyx_t_1;
  long __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":740
 *     cdef blas_t* work
 * 
 *     for j in range(m-2, k-1, -1):             # <<<<<<<<<<<<<<
 *         lartg(index2(r, rs, j, k), index2(r, rs, j+1, k), &c, &s)
 * 
 */
  __pyx_t_1 = (__pyx_v_k - 1);
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = (__pyx_v_m - 2); __pyx_t_3 > __pyx_t_2; __pyx_t_3-=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":741
 * 
 *     for j in range(m-2, k-1, -1):
 *         lartg(index2(r, rs, j, k), index2(r, rs, j+1, k), &c, &s)             # <<<<<<<<<<<<<<
 * 
 *         # update r if j is a nonzero row
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_k), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_k), (&__pyx_v_c), (&__pyx_v_s));

    /* "scipy/linalg/_decomp_update.pyx":744
 * 
 *         # update r if j is a nonzero row
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 */
    __pyx_t_4 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
    if (__pyx_t_4) {

      /* "scipy/linalg/_decomp_update.pyx":745
 *         # update r if j is a nonzero row
 *         if j+1 < n:
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],             # <<<<<<<<<<<<<<
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 * 
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

      /* "scipy/linalg/_decomp_update.pyx":744
 * 
 *         # update r if j is a nonzero row
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":749
 * 
 *         # update the columns of q
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())             # <<<<<<<<<<<<<<
 * 
 * cdef int qr_block_col_insert(int m, int n, blas_t* q, int* qs,
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_s));
  }

  /* "scipy/linalg/_decomp_update.pyx":734
 *     return 0
 * 
 * cdef void qr_col_insert(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                         int k) nogil:
 *     cdef int j
 */

  /* function exit code */
}

static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_insert(int __pyx_v_m, int __pyx_v_n, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_float_complex *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k) {
  int __pyx_v_j;
  __pyx_t_float_complex __pyx_v_c;
  __pyx_t_float_complex __pyx_v_s;
  long __pyx_t_1;
  long __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":740
 *     cdef blas_t* work
 * 
 *     for j in range(m-2, k-1, -1):             # <<<<<<<<<<<<<<
 *         lartg(index2(r, rs, j, k), index2(r, rs, j+1, k), &c, &s)
 * 
 */
  __pyx_t_1 = (__pyx_v_k - 1);
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = (__pyx_v_m - 2); __pyx_t_3 > __pyx_t_2; __pyx_t_3-=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":741
 * 
 *     for j in range(m-2, k-1, -1):
 *         lartg(index2(r, rs, j, k), index2(r, rs, j+1, k), &c, &s)             # <<<<<<<<<<<<<<
 * 
 *         # update r if j is a nonzero row
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_k), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_k), (&__pyx_v_c), (&__pyx_v_s));

    /* "scipy/linalg/_decomp_update.pyx":744
 * 
 *         # update r if j is a nonzero row
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 */
    __pyx_t_4 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
    if (__pyx_t_4) {

      /* "scipy/linalg/_decomp_update.pyx":745
 *         # update r if j is a nonzero row
 *         if j+1 < n:
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],             # <<<<<<<<<<<<<<
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 * 
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

      /* "scipy/linalg/_decomp_update.pyx":744
 * 
 *         # update r if j is a nonzero row
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":749
 * 
 *         # update the columns of q
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())             # <<<<<<<<<<<<<<
 * 
 * cdef int qr_block_col_insert(int m, int n, blas_t* q, int* qs,
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_float(__pyx_v_s));
  }

  /* "scipy/linalg/_decomp_update.pyx":734
 *     return 0
 * 
 * cdef void qr_col_insert(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                         int k) nogil:
 *     cdef int j
 */

  /* function exit code */
}

static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_insert(int __pyx_v_m, int __pyx_v_n, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_double_complex *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k) {
  int __pyx_v_j;
  __pyx_t_double_complex __pyx_v_c;
  __pyx_t_double_complex __pyx_v_s;
  long __pyx_t_1;
  long __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":740
 *     cdef blas_t* work
 * 
 *     for j in range(m-2, k-1, -1):             # <<<<<<<<<<<<<<
 *         lartg(index2(r, rs, j, k), index2(r, rs, j+1, k), &c, &s)
 * 
 */
  __pyx_t_1 = (__pyx_v_k - 1);
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = (__pyx_v_m - 2); __pyx_t_3 > __pyx_t_2; __pyx_t_3-=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":741
 * 
 *     for j in range(m-2, k-1, -1):
 *         lartg(index2(r, rs, j, k), index2(r, rs, j+1, k), &c, &s)             # <<<<<<<<<<<<<<
 * 
 *         # update r if j is a nonzero row
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_k), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_k), (&__pyx_v_c), (&__pyx_v_s));

    /* "scipy/linalg/_decomp_update.pyx":744
 * 
 *         # update r if j is a nonzero row
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 */
    __pyx_t_4 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
    if (__pyx_t_4) {

      /* "scipy/linalg/_decomp_update.pyx":745
 *         # update r if j is a nonzero row
 *         if j+1 < n:
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],             # <<<<<<<<<<<<<<
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 * 
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

      /* "scipy/linalg/_decomp_update.pyx":744
 * 
 *         # update r if j is a nonzero row
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":749
 * 
 *         # update the columns of q
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())             # <<<<<<<<<<<<<<
 * 
 * cdef int qr_block_col_insert(int m, int n, blas_t* q, int* qs,
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_double(__pyx_v_s));
  }

  /* "scipy/linalg/_decomp_update.pyx":734
 *     return 0
 * 
 * cdef void qr_col_insert(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                         int k) nogil:
 *     cdef int j
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":751
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())
 * 
 * cdef int qr_block_col_insert(int m, int n, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *                               blas_t* r, int* rs, int k, int p) nogil:
 *     cdef int i, j
 */

static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_insert(int __pyx_v_m, int __pyx_v_n, float *__pyx_v_q, int *__pyx_v_qs, float *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p) {
  int __pyx_v_i;
  int __pyx_v_j;
  float __pyx_v_c;
  float __pyx_v_s;
  float *__pyx_v_tau;
  float *__pyx_v_work;
  int __pyx_v_info;
  int __pyx_v_lwork;
  char *__pyx_v_side;
  char *__pyx_v_trans;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;

  /* "scipy/linalg/_decomp_update.pyx":755
 *     cdef int i, j
 *     cdef blas_t c, s
 *     cdef blas_t* tau = NULL             # <<<<<<<<<<<<<<
 *     cdef blas_t* work = NULL
 *     cdef int info, lwork
 */
  __pyx_v_tau = NULL;

  /* "scipy/linalg/_decomp_update.pyx":756
 *     cdef blas_t c, s
 *     cdef blas_t* tau = NULL
 *     cdef blas_t* work = NULL             # <<<<<<<<<<<<<<
 *     cdef int info, lwork
 *     cdef char* side = 'R'
 */
  __pyx_v_work = NULL;

  /* "scipy/linalg/_decomp_update.pyx":758
 *     cdef blas_t* work = NULL
 *     cdef int info, lwork
 *     cdef char* side = 'R'             # <<<<<<<<<<<<<<
 *     cdef char* trans = 'N'
 * 
 */
  __pyx_v_side = ((char *)"R");

  /* "scipy/linalg/_decomp_update.pyx":759
 *     cdef int info, lwork
 *     cdef char* side = 'R'
 *     cdef char* trans = 'N'             # <<<<<<<<<<<<<<
 * 
 *     if m >= n:
 */
  __pyx_v_trans = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":761
 *     cdef char* trans = 'N'
 * 
 *     if m >= n:             # <<<<<<<<<<<<<<
 *         # if m > n, r looks like this.
 *         # x x x x x x x x x x
 */
  __pyx_t_1 = ((__pyx_v_m >= __pyx_v_n) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":780
 *         # query the workspace,
 *         # set tau to point at something to keep new MKL working.
 *         tau = &c             # <<<<<<<<<<<<<<
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, &c, -1)
 *         if info < 0:
 */
    __pyx_v_tau = (&__pyx_v_c);

    /* "scipy/linalg/_decomp_update.pyx":781
 *         # set tau to point at something to keep new MKL working.
 *         tau = &c
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, &c, -1)             # <<<<<<<<<<<<<<
 *         if info < 0:
 *             return libc.stdlib.abs(info)
 */
    __pyx_v_info = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_geqrf(((__pyx_v_m - __pyx_v_n) + __pyx_v_p), __pyx_v_p, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - __pyx_v_p), __pyx_v_k), __pyx_v_m, __pyx_v_tau, (&__pyx_v_c), -1);

    /* "scipy/linalg/_decomp_update.pyx":782
 *         tau = &c
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, &c, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":783
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, &c, -1)
 *         if info < 0:
 *             return libc.stdlib.abs(info)             # <<<<<<<<<<<<<<
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 *                      tau, index2(q, qs, 0, n-p), m, &s, -1)
 */
      __pyx_r = abs(__pyx_v_info);
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":782
 *         tau = &c
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, &c, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":784
 *         if info < 0:
 *             return libc.stdlib.abs(info)
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,             # <<<<<<<<<<<<<<
 *                      tau, index2(q, qs, 0, n-p), m, &s, -1)
 *         if info < 0:
 */
    __pyx_v_info = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(__pyx_v_side, __pyx_v_trans, __pyx_v_m, (__pyx_v_m - (__pyx_v_n - __pyx_v_p)), __pyx_v_p, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - __pyx_v_p), __pyx_v_k), __pyx_v_m, __pyx_v_tau, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, (__pyx_v_n - __pyx_v_p)), __pyx_v_m, (&__pyx_v_s), -1);

    /* "scipy/linalg/_decomp_update.pyx":786
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 *                      tau, index2(q, qs, 0, n-p), m, &s, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":787
 *                      tau, index2(q, qs, 0, n-p), m, &s, -1)
 *         if info < 0:
 *             return info             # <<<<<<<<<<<<<<
 * 
 *         # we're only doing one allocation, so use the larger
 */
      __pyx_r = __pyx_v_info;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":786
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 *                      tau, index2(q, qs, 0, n-p), m, &s, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":790
 * 
 *         # we're only doing one allocation, so use the larger
 *         lwork = to_lwork(c, s)             # <<<<<<<<<<<<<<
 * 
 *         # allocate the workspace + tau
 */
    __pyx_v_lwork = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_to_lwork(__pyx_v_c, __pyx_v_s);

    /* "scipy/linalg/_decomp_update.pyx":793
 * 
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n+p, p))*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *         if not work:
 *             return MEMORY_ERROR
 */
    __pyx_t_2 = __pyx_v_p;
    __pyx_t_3 = ((__pyx_v_m - __pyx_v_n) + __pyx_v_p);
    if (((__pyx_t_2 < __pyx_t_3) != 0)) {
      __pyx_t_4 = __pyx_t_2;
    } else {
      __pyx_t_4 = __pyx_t_3;
    }
    __pyx_v_work = ((float *)malloc(((__pyx_v_lwork + __pyx_t_4) * (sizeof(float)))));

    /* "scipy/linalg/_decomp_update.pyx":794
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n+p, p))*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 *         tau = work + lwork
 */
    __pyx_t_1 = ((!(__pyx_v_work != 0)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":795
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n+p, p))*sizeof(blas_t))
 *         if not work:
 *             return MEMORY_ERROR             # <<<<<<<<<<<<<<
 *         tau = work + lwork
 * 
 */
      __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":794
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n+p, p))*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 *         tau = work + lwork
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":796
 *         if not work:
 *             return MEMORY_ERROR
 *         tau = work + lwork             # <<<<<<<<<<<<<<
 * 
 *         # qr
 */
    __pyx_v_tau = (__pyx_v_work + __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":799
 * 
 *         # qr
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, work, lwork)             # <<<<<<<<<<<<<<
 *         if info < 0:
 *             return libc.stdlib.abs(info)
 */
    __pyx_v_info = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_geqrf(((__pyx_v_m - __pyx_v_n) + __pyx_v_p), __pyx_v_p, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - __pyx_v_p), __pyx_v_k), __pyx_v_m, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":800
 *         # qr
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 * 
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":801
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, work, lwork)
 *         if info < 0:
 *             return libc.stdlib.abs(info)             # <<<<<<<<<<<<<<
 * 
 *         # apply the Q from this small qr to the last (m-(n-p)) columns of q.
 */
      __pyx_r = abs(__pyx_v_info);
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":800
 *         # qr
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":804
 * 
 *         # apply the Q from this small qr to the last (m-(n-p)) columns of q.
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,             # <<<<<<<<<<<<<<
 *                 tau, index2(q, qs, 0, n-p), m, work, lwork)
 *         if info < 0:
 */
    __pyx_v_info = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(__pyx_v_side, __pyx_v_trans, __pyx_v_m, (__pyx_v_m - (__pyx_v_n - __pyx_v_p)), __pyx_v_p, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - __pyx_v_p), __pyx_v_k), __pyx_v_m, __pyx_v_tau, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, (__pyx_v_n - __pyx_v_p)), __pyx_v_m, __pyx_v_work, __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":806
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 *                 tau, index2(q, qs, 0, n-p), m, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":807
 *                 tau, index2(q, qs, 0, n-p), m, work, lwork)
 *         if info < 0:
 *             return info             # <<<<<<<<<<<<<<
 * 
 *         libc.stdlib.free(work)
 */
      __pyx_r = __pyx_v_info;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":806
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 *                 tau, index2(q, qs, 0, n-p), m, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":809
 *             return info
 * 
 *         libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 * 
 *         # zero the reflectors since we're done with them
 */
    free(__pyx_v_work);

    /* "scipy/linalg/_decomp_update.pyx":813
 *         # zero the reflectors since we're done with them
 *         # memset can be used here, since r is always fortran order
 *         for j in range(p):             # <<<<<<<<<<<<<<
 *             memset(index2(r, rs, n-p+1+j, k+j), 0, (m-(n-p+1+j))*sizeof(blas_t))
 * 
 */
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_2 = __pyx_t_4;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_j = __pyx_t_3;

      /* "scipy/linalg/_decomp_update.pyx":814
 *         # memset can be used here, since r is always fortran order
 *         for j in range(p):
 *             memset(index2(r, rs, n-p+1+j, k+j), 0, (m-(n-p+1+j))*sizeof(blas_t))             # <<<<<<<<<<<<<<
 * 
 *         # now we have something that looks like
 */
      (void)(memset(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (((__pyx_v_n - __pyx_v_p) + 1) + __pyx_v_j), (__pyx_v_k + __pyx_v_j)), 0, ((__pyx_v_m - (((__pyx_v_n - __pyx_v_p) + 1) + __pyx_v_j)) * (sizeof(float)))));
    }

    /* "scipy/linalg/_decomp_update.pyx":832
 *         # and the rest of the columns need to be eliminated using rotations.
 * 
 *         for i in range(p):             # <<<<<<<<<<<<<<
 *             for j in range(n-p+i-1, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 */
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_2 = __pyx_t_4;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "scipy/linalg/_decomp_update.pyx":833
 * 
 *         for i in range(p):
 *             for j in range(n-p+i-1, k+i-1, -1):             # <<<<<<<<<<<<<<
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:
 */
      __pyx_t_5 = ((__pyx_v_k + __pyx_v_i) - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = (((__pyx_v_n - __pyx_v_p) + __pyx_v_i) - 1); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
        __pyx_v_j = __pyx_t_7;

        /* "scipy/linalg/_decomp_update.pyx":834
 *         for i in range(p):
 *             for j in range(n-p+i-1, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)             # <<<<<<<<<<<<<<
 *                 if j+1 < n:
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 */
        __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_k + __pyx_v_i)), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_k + __pyx_v_i)), (&__pyx_v_c), (&__pyx_v_s));

        /* "scipy/linalg/_decomp_update.pyx":835
 *             for j in range(n-p+i-1, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:             # <<<<<<<<<<<<<<
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 */
        __pyx_t_1 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
        if (__pyx_t_1) {

          /* "scipy/linalg/_decomp_update.pyx":836
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],             # <<<<<<<<<<<<<<
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],
 */
          __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot((((__pyx_v_n - __pyx_v_k) - __pyx_v_i) - 1), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, ((__pyx_v_k + __pyx_v_i) + 1)), (__pyx_v_rs[1]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), ((__pyx_v_k + __pyx_v_i) + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

          /* "scipy/linalg/_decomp_update.pyx":835
 *             for j in range(n-p+i-1, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:             # <<<<<<<<<<<<<<
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":838
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],             # <<<<<<<<<<<<<<
 *                         c, s.conjugate())
 *     else:
 */
        __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_s));
      }
    }

    /* "scipy/linalg/_decomp_update.pyx":761
 *     cdef char* trans = 'N'
 * 
 *     if m >= n:             # <<<<<<<<<<<<<<
 *         # if m > n, r looks like this.
 *         # x x x x x x x x x x
 */
    goto __pyx_L3;
  }

  /* "scipy/linalg/_decomp_update.pyx":842
 *     else:
 *         # this case we can only uses givens rotations.
 *         for i in range(p):             # <<<<<<<<<<<<<<
 *             for j in range(m-2, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 */
  /*else*/ {
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_2 = __pyx_t_4;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "scipy/linalg/_decomp_update.pyx":843
 *         # this case we can only uses givens rotations.
 *         for i in range(p):
 *             for j in range(m-2, k+i-1, -1):             # <<<<<<<<<<<<<<
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:
 */
      __pyx_t_5 = ((__pyx_v_k + __pyx_v_i) - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = (__pyx_v_m - 2); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
        __pyx_v_j = __pyx_t_7;

        /* "scipy/linalg/_decomp_update.pyx":844
 *         for i in range(p):
 *             for j in range(m-2, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)             # <<<<<<<<<<<<<<
 *                 if j+1 < n:
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 */
        __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_k + __pyx_v_i)), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_k + __pyx_v_i)), (&__pyx_v_c), (&__pyx_v_s));

        /* "scipy/linalg/_decomp_update.pyx":845
 *             for j in range(m-2, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:             # <<<<<<<<<<<<<<
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 */
        __pyx_t_1 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
        if (__pyx_t_1) {

          /* "scipy/linalg/_decomp_update.pyx":846
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],             # <<<<<<<<<<<<<<
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],
 */
          __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot((((__pyx_v_n - __pyx_v_k) - __pyx_v_i) - 1), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, ((__pyx_v_k + __pyx_v_i) + 1)), (__pyx_v_rs[1]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), ((__pyx_v_k + __pyx_v_i) + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

          /* "scipy/linalg/_decomp_update.pyx":845
 *             for j in range(m-2, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:             # <<<<<<<<<<<<<<
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":848
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],             # <<<<<<<<<<<<<<
 *                         c, s.conjugate())
 *     return 0
 */
        __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_s));
      }
    }
  }
  __pyx_L3:;

  /* "scipy/linalg/_decomp_update.pyx":850
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],
 *                         c, s.conjugate())
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef void thin_qr_rank_1_update(int m, int n, blas_t* q, int* qs, bint qisF,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":751
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())
 * 
 * cdef int qr_block_col_insert(int m, int n, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *                               blas_t* r, int* rs, int k, int p) nogil:
 *     cdef int i, j
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_insert(int __pyx_v_m, int __pyx_v_n, double *__pyx_v_q, int *__pyx_v_qs, double *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p) {
  int __pyx_v_i;
  int __pyx_v_j;
  double __pyx_v_c;
  double __pyx_v_s;
  double *__pyx_v_tau;
  double *__pyx_v_work;
  int __pyx_v_info;
  int __pyx_v_lwork;
  char *__pyx_v_side;
  char *__pyx_v_trans;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;

  /* "scipy/linalg/_decomp_update.pyx":755
 *     cdef int i, j
 *     cdef blas_t c, s
 *     cdef blas_t* tau = NULL             # <<<<<<<<<<<<<<
 *     cdef blas_t* work = NULL
 *     cdef int info, lwork
 */
  __pyx_v_tau = NULL;

  /* "scipy/linalg/_decomp_update.pyx":756
 *     cdef blas_t c, s
 *     cdef blas_t* tau = NULL
 *     cdef blas_t* work = NULL             # <<<<<<<<<<<<<<
 *     cdef int info, lwork
 *     cdef char* side = 'R'
 */
  __pyx_v_work = NULL;

  /* "scipy/linalg/_decomp_update.pyx":758
 *     cdef blas_t* work = NULL
 *     cdef int info, lwork
 *     cdef char* side = 'R'             # <<<<<<<<<<<<<<
 *     cdef char* trans = 'N'
 * 
 */
  __pyx_v_side = ((char *)"R");

  /* "scipy/linalg/_decomp_update.pyx":759
 *     cdef int info, lwork
 *     cdef char* side = 'R'
 *     cdef char* trans = 'N'             # <<<<<<<<<<<<<<
 * 
 *     if m >= n:
 */
  __pyx_v_trans = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":761
 *     cdef char* trans = 'N'
 * 
 *     if m >= n:             # <<<<<<<<<<<<<<
 *         # if m > n, r looks like this.
 *         # x x x x x x x x x x
 */
  __pyx_t_1 = ((__pyx_v_m >= __pyx_v_n) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":780
 *         # query the workspace,
 *         # set tau to point at something to keep new MKL working.
 *         tau = &c             # <<<<<<<<<<<<<<
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, &c, -1)
 *         if info < 0:
 */
    __pyx_v_tau = (&__pyx_v_c);

    /* "scipy/linalg/_decomp_update.pyx":781
 *         # set tau to point at something to keep new MKL working.
 *         tau = &c
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, &c, -1)             # <<<<<<<<<<<<<<
 *         if info < 0:
 *             return libc.stdlib.abs(info)
 */
    __pyx_v_info = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_geqrf(((__pyx_v_m - __pyx_v_n) + __pyx_v_p), __pyx_v_p, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - __pyx_v_p), __pyx_v_k), __pyx_v_m, __pyx_v_tau, (&__pyx_v_c), -1);

    /* "scipy/linalg/_decomp_update.pyx":782
 *         tau = &c
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, &c, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":783
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, &c, -1)
 *         if info < 0:
 *             return libc.stdlib.abs(info)             # <<<<<<<<<<<<<<
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 *                      tau, index2(q, qs, 0, n-p), m, &s, -1)
 */
      __pyx_r = abs(__pyx_v_info);
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":782
 *         tau = &c
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, &c, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":784
 *         if info < 0:
 *             return libc.stdlib.abs(info)
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,             # <<<<<<<<<<<<<<
 *                      tau, index2(q, qs, 0, n-p), m, &s, -1)
 *         if info < 0:
 */
    __pyx_v_info = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(__pyx_v_side, __pyx_v_trans, __pyx_v_m, (__pyx_v_m - (__pyx_v_n - __pyx_v_p)), __pyx_v_p, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - __pyx_v_p), __pyx_v_k), __pyx_v_m, __pyx_v_tau, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, (__pyx_v_n - __pyx_v_p)), __pyx_v_m, (&__pyx_v_s), -1);

    /* "scipy/linalg/_decomp_update.pyx":786
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 *                      tau, index2(q, qs, 0, n-p), m, &s, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":787
 *                      tau, index2(q, qs, 0, n-p), m, &s, -1)
 *         if info < 0:
 *             return info             # <<<<<<<<<<<<<<
 * 
 *         # we're only doing one allocation, so use the larger
 */
      __pyx_r = __pyx_v_info;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":786
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 *                      tau, index2(q, qs, 0, n-p), m, &s, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":790
 * 
 *         # we're only doing one allocation, so use the larger
 *         lwork = to_lwork(c, s)             # <<<<<<<<<<<<<<
 * 
 *         # allocate the workspace + tau
 */
    __pyx_v_lwork = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_to_lwork(__pyx_v_c, __pyx_v_s);

    /* "scipy/linalg/_decomp_update.pyx":793
 * 
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n+p, p))*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *         if not work:
 *             return MEMORY_ERROR
 */
    __pyx_t_2 = __pyx_v_p;
    __pyx_t_3 = ((__pyx_v_m - __pyx_v_n) + __pyx_v_p);
    if (((__pyx_t_2 < __pyx_t_3) != 0)) {
      __pyx_t_4 = __pyx_t_2;
    } else {
      __pyx_t_4 = __pyx_t_3;
    }
    __pyx_v_work = ((double *)malloc(((__pyx_v_lwork + __pyx_t_4) * (sizeof(double)))));

    /* "scipy/linalg/_decomp_update.pyx":794
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n+p, p))*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 *         tau = work + lwork
 */
    __pyx_t_1 = ((!(__pyx_v_work != 0)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":795
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n+p, p))*sizeof(blas_t))
 *         if not work:
 *             return MEMORY_ERROR             # <<<<<<<<<<<<<<
 *         tau = work + lwork
 * 
 */
      __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":794
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n+p, p))*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 *         tau = work + lwork
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":796
 *         if not work:
 *             return MEMORY_ERROR
 *         tau = work + lwork             # <<<<<<<<<<<<<<
 * 
 *         # qr
 */
    __pyx_v_tau = (__pyx_v_work + __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":799
 * 
 *         # qr
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, work, lwork)             # <<<<<<<<<<<<<<
 *         if info < 0:
 *             return libc.stdlib.abs(info)
 */
    __pyx_v_info = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_geqrf(((__pyx_v_m - __pyx_v_n) + __pyx_v_p), __pyx_v_p, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - __pyx_v_p), __pyx_v_k), __pyx_v_m, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":800
 *         # qr
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 * 
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":801
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, work, lwork)
 *         if info < 0:
 *             return libc.stdlib.abs(info)             # <<<<<<<<<<<<<<
 * 
 *         # apply the Q from this small qr to the last (m-(n-p)) columns of q.
 */
      __pyx_r = abs(__pyx_v_info);
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":800
 *         # qr
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":804
 * 
 *         # apply the Q from this small qr to the last (m-(n-p)) columns of q.
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,             # <<<<<<<<<<<<<<
 *                 tau, index2(q, qs, 0, n-p), m, work, lwork)
 *         if info < 0:
 */
    __pyx_v_info = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(__pyx_v_side, __pyx_v_trans, __pyx_v_m, (__pyx_v_m - (__pyx_v_n - __pyx_v_p)), __pyx_v_p, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - __pyx_v_p), __pyx_v_k), __pyx_v_m, __pyx_v_tau, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, (__pyx_v_n - __pyx_v_p)), __pyx_v_m, __pyx_v_work, __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":806
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 *                 tau, index2(q, qs, 0, n-p), m, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":807
 *                 tau, index2(q, qs, 0, n-p), m, work, lwork)
 *         if info < 0:
 *             return info             # <<<<<<<<<<<<<<
 * 
 *         libc.stdlib.free(work)
 */
      __pyx_r = __pyx_v_info;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":806
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 *                 tau, index2(q, qs, 0, n-p), m, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":809
 *             return info
 * 
 *         libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 * 
 *         # zero the reflectors since we're done with them
 */
    free(__pyx_v_work);

    /* "scipy/linalg/_decomp_update.pyx":813
 *         # zero the reflectors since we're done with them
 *         # memset can be used here, since r is always fortran order
 *         for j in range(p):             # <<<<<<<<<<<<<<
 *             memset(index2(r, rs, n-p+1+j, k+j), 0, (m-(n-p+1+j))*sizeof(blas_t))
 * 
 */
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_2 = __pyx_t_4;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_j = __pyx_t_3;

      /* "scipy/linalg/_decomp_update.pyx":814
 *         # memset can be used here, since r is always fortran order
 *         for j in range(p):
 *             memset(index2(r, rs, n-p+1+j, k+j), 0, (m-(n-p+1+j))*sizeof(blas_t))             # <<<<<<<<<<<<<<
 * 
 *         # now we have something that looks like
 */
      (void)(memset(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (((__pyx_v_n - __pyx_v_p) + 1) + __pyx_v_j), (__pyx_v_k + __pyx_v_j)), 0, ((__pyx_v_m - (((__pyx_v_n - __pyx_v_p) + 1) + __pyx_v_j)) * (sizeof(double)))));
    }

    /* "scipy/linalg/_decomp_update.pyx":832
 *         # and the rest of the columns need to be eliminated using rotations.
 * 
 *         for i in range(p):             # <<<<<<<<<<<<<<
 *             for j in range(n-p+i-1, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 */
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_2 = __pyx_t_4;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "scipy/linalg/_decomp_update.pyx":833
 * 
 *         for i in range(p):
 *             for j in range(n-p+i-1, k+i-1, -1):             # <<<<<<<<<<<<<<
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:
 */
      __pyx_t_5 = ((__pyx_v_k + __pyx_v_i) - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = (((__pyx_v_n - __pyx_v_p) + __pyx_v_i) - 1); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
        __pyx_v_j = __pyx_t_7;

        /* "scipy/linalg/_decomp_update.pyx":834
 *         for i in range(p):
 *             for j in range(n-p+i-1, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)             # <<<<<<<<<<<<<<
 *                 if j+1 < n:
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 */
        __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_k + __pyx_v_i)), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_k + __pyx_v_i)), (&__pyx_v_c), (&__pyx_v_s));

        /* "scipy/linalg/_decomp_update.pyx":835
 *             for j in range(n-p+i-1, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:             # <<<<<<<<<<<<<<
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 */
        __pyx_t_1 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
        if (__pyx_t_1) {

          /* "scipy/linalg/_decomp_update.pyx":836
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],             # <<<<<<<<<<<<<<
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],
 */
          __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot((((__pyx_v_n - __pyx_v_k) - __pyx_v_i) - 1), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, ((__pyx_v_k + __pyx_v_i) + 1)), (__pyx_v_rs[1]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), ((__pyx_v_k + __pyx_v_i) + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

          /* "scipy/linalg/_decomp_update.pyx":835
 *             for j in range(n-p+i-1, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:             # <<<<<<<<<<<<<<
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":838
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],             # <<<<<<<<<<<<<<
 *                         c, s.conjugate())
 *     else:
 */
        __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_s));
      }
    }

    /* "scipy/linalg/_decomp_update.pyx":761
 *     cdef char* trans = 'N'
 * 
 *     if m >= n:             # <<<<<<<<<<<<<<
 *         # if m > n, r looks like this.
 *         # x x x x x x x x x x
 */
    goto __pyx_L3;
  }

  /* "scipy/linalg/_decomp_update.pyx":842
 *     else:
 *         # this case we can only uses givens rotations.
 *         for i in range(p):             # <<<<<<<<<<<<<<
 *             for j in range(m-2, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 */
  /*else*/ {
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_2 = __pyx_t_4;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "scipy/linalg/_decomp_update.pyx":843
 *         # this case we can only uses givens rotations.
 *         for i in range(p):
 *             for j in range(m-2, k+i-1, -1):             # <<<<<<<<<<<<<<
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:
 */
      __pyx_t_5 = ((__pyx_v_k + __pyx_v_i) - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = (__pyx_v_m - 2); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
        __pyx_v_j = __pyx_t_7;

        /* "scipy/linalg/_decomp_update.pyx":844
 *         for i in range(p):
 *             for j in range(m-2, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)             # <<<<<<<<<<<<<<
 *                 if j+1 < n:
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 */
        __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_k + __pyx_v_i)), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_k + __pyx_v_i)), (&__pyx_v_c), (&__pyx_v_s));

        /* "scipy/linalg/_decomp_update.pyx":845
 *             for j in range(m-2, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:             # <<<<<<<<<<<<<<
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 */
        __pyx_t_1 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
        if (__pyx_t_1) {

          /* "scipy/linalg/_decomp_update.pyx":846
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],             # <<<<<<<<<<<<<<
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],
 */
          __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot((((__pyx_v_n - __pyx_v_k) - __pyx_v_i) - 1), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, ((__pyx_v_k + __pyx_v_i) + 1)), (__pyx_v_rs[1]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), ((__pyx_v_k + __pyx_v_i) + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

          /* "scipy/linalg/_decomp_update.pyx":845
 *             for j in range(m-2, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:             # <<<<<<<<<<<<<<
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":848
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],             # <<<<<<<<<<<<<<
 *                         c, s.conjugate())
 *     return 0
 */
        __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_s));
      }
    }
  }
  __pyx_L3:;

  /* "scipy/linalg/_decomp_update.pyx":850
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],
 *                         c, s.conjugate())
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef void thin_qr_rank_1_update(int m, int n, blas_t* q, int* qs, bint qisF,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":751
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())
 * 
 * cdef int qr_block_col_insert(int m, int n, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *                               blas_t* r, int* rs, int k, int p) nogil:
 *     cdef int i, j
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_insert(int __pyx_v_m, int __pyx_v_n, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_float_complex *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p) {
  int __pyx_v_i;
  int __pyx_v_j;
  __pyx_t_float_complex __pyx_v_c;
  __pyx_t_float_complex __pyx_v_s;
  __pyx_t_float_complex *__pyx_v_tau;
  __pyx_t_float_complex *__pyx_v_work;
  int __pyx_v_info;
  int __pyx_v_lwork;
  char *__pyx_v_side;
  char *__pyx_v_trans;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;

  /* "scipy/linalg/_decomp_update.pyx":755
 *     cdef int i, j
 *     cdef blas_t c, s
 *     cdef blas_t* tau = NULL             # <<<<<<<<<<<<<<
 *     cdef blas_t* work = NULL
 *     cdef int info, lwork
 */
  __pyx_v_tau = NULL;

  /* "scipy/linalg/_decomp_update.pyx":756
 *     cdef blas_t c, s
 *     cdef blas_t* tau = NULL
 *     cdef blas_t* work = NULL             # <<<<<<<<<<<<<<
 *     cdef int info, lwork
 *     cdef char* side = 'R'
 */
  __pyx_v_work = NULL;

  /* "scipy/linalg/_decomp_update.pyx":758
 *     cdef blas_t* work = NULL
 *     cdef int info, lwork
 *     cdef char* side = 'R'             # <<<<<<<<<<<<<<
 *     cdef char* trans = 'N'
 * 
 */
  __pyx_v_side = ((char *)"R");

  /* "scipy/linalg/_decomp_update.pyx":759
 *     cdef int info, lwork
 *     cdef char* side = 'R'
 *     cdef char* trans = 'N'             # <<<<<<<<<<<<<<
 * 
 *     if m >= n:
 */
  __pyx_v_trans = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":761
 *     cdef char* trans = 'N'
 * 
 *     if m >= n:             # <<<<<<<<<<<<<<
 *         # if m > n, r looks like this.
 *         # x x x x x x x x x x
 */
  __pyx_t_1 = ((__pyx_v_m >= __pyx_v_n) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":780
 *         # query the workspace,
 *         # set tau to point at something to keep new MKL working.
 *         tau = &c             # <<<<<<<<<<<<<<
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, &c, -1)
 *         if info < 0:
 */
    __pyx_v_tau = (&__pyx_v_c);

    /* "scipy/linalg/_decomp_update.pyx":781
 *         # set tau to point at something to keep new MKL working.
 *         tau = &c
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, &c, -1)             # <<<<<<<<<<<<<<
 *         if info < 0:
 *             return libc.stdlib.abs(info)
 */
    __pyx_v_info = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_geqrf(((__pyx_v_m - __pyx_v_n) + __pyx_v_p), __pyx_v_p, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - __pyx_v_p), __pyx_v_k), __pyx_v_m, __pyx_v_tau, (&__pyx_v_c), -1);

    /* "scipy/linalg/_decomp_update.pyx":782
 *         tau = &c
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, &c, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":783
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, &c, -1)
 *         if info < 0:
 *             return libc.stdlib.abs(info)             # <<<<<<<<<<<<<<
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 *                      tau, index2(q, qs, 0, n-p), m, &s, -1)
 */
      __pyx_r = abs(__pyx_v_info);
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":782
 *         tau = &c
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, &c, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":784
 *         if info < 0:
 *             return libc.stdlib.abs(info)
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,             # <<<<<<<<<<<<<<
 *                      tau, index2(q, qs, 0, n-p), m, &s, -1)
 *         if info < 0:
 */
    __pyx_v_info = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(__pyx_v_side, __pyx_v_trans, __pyx_v_m, (__pyx_v_m - (__pyx_v_n - __pyx_v_p)), __pyx_v_p, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - __pyx_v_p), __pyx_v_k), __pyx_v_m, __pyx_v_tau, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, (__pyx_v_n - __pyx_v_p)), __pyx_v_m, (&__pyx_v_s), -1);

    /* "scipy/linalg/_decomp_update.pyx":786
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 *                      tau, index2(q, qs, 0, n-p), m, &s, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":787
 *                      tau, index2(q, qs, 0, n-p), m, &s, -1)
 *         if info < 0:
 *             return info             # <<<<<<<<<<<<<<
 * 
 *         # we're only doing one allocation, so use the larger
 */
      __pyx_r = __pyx_v_info;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":786
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 *                      tau, index2(q, qs, 0, n-p), m, &s, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":790
 * 
 *         # we're only doing one allocation, so use the larger
 *         lwork = to_lwork(c, s)             # <<<<<<<<<<<<<<
 * 
 *         # allocate the workspace + tau
 */
    __pyx_v_lwork = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_to_lwork(__pyx_v_c, __pyx_v_s);

    /* "scipy/linalg/_decomp_update.pyx":793
 * 
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n+p, p))*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *         if not work:
 *             return MEMORY_ERROR
 */
    __pyx_t_2 = __pyx_v_p;
    __pyx_t_3 = ((__pyx_v_m - __pyx_v_n) + __pyx_v_p);
    if (((__pyx_t_2 < __pyx_t_3) != 0)) {
      __pyx_t_4 = __pyx_t_2;
    } else {
      __pyx_t_4 = __pyx_t_3;
    }
    __pyx_v_work = ((__pyx_t_float_complex *)malloc(((__pyx_v_lwork + __pyx_t_4) * (sizeof(__pyx_t_float_complex)))));

    /* "scipy/linalg/_decomp_update.pyx":794
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n+p, p))*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 *         tau = work + lwork
 */
    __pyx_t_1 = ((!(__pyx_v_work != 0)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":795
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n+p, p))*sizeof(blas_t))
 *         if not work:
 *             return MEMORY_ERROR             # <<<<<<<<<<<<<<
 *         tau = work + lwork
 * 
 */
      __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":794
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n+p, p))*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 *         tau = work + lwork
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":796
 *         if not work:
 *             return MEMORY_ERROR
 *         tau = work + lwork             # <<<<<<<<<<<<<<
 * 
 *         # qr
 */
    __pyx_v_tau = (__pyx_v_work + __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":799
 * 
 *         # qr
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, work, lwork)             # <<<<<<<<<<<<<<
 *         if info < 0:
 *             return libc.stdlib.abs(info)
 */
    __pyx_v_info = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_geqrf(((__pyx_v_m - __pyx_v_n) + __pyx_v_p), __pyx_v_p, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - __pyx_v_p), __pyx_v_k), __pyx_v_m, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":800
 *         # qr
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 * 
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":801
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, work, lwork)
 *         if info < 0:
 *             return libc.stdlib.abs(info)             # <<<<<<<<<<<<<<
 * 
 *         # apply the Q from this small qr to the last (m-(n-p)) columns of q.
 */
      __pyx_r = abs(__pyx_v_info);
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":800
 *         # qr
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":804
 * 
 *         # apply the Q from this small qr to the last (m-(n-p)) columns of q.
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,             # <<<<<<<<<<<<<<
 *                 tau, index2(q, qs, 0, n-p), m, work, lwork)
 *         if info < 0:
 */
    __pyx_v_info = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(__pyx_v_side, __pyx_v_trans, __pyx_v_m, (__pyx_v_m - (__pyx_v_n - __pyx_v_p)), __pyx_v_p, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - __pyx_v_p), __pyx_v_k), __pyx_v_m, __pyx_v_tau, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, (__pyx_v_n - __pyx_v_p)), __pyx_v_m, __pyx_v_work, __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":806
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 *                 tau, index2(q, qs, 0, n-p), m, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":807
 *                 tau, index2(q, qs, 0, n-p), m, work, lwork)
 *         if info < 0:
 *             return info             # <<<<<<<<<<<<<<
 * 
 *         libc.stdlib.free(work)
 */
      __pyx_r = __pyx_v_info;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":806
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 *                 tau, index2(q, qs, 0, n-p), m, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":809
 *             return info
 * 
 *         libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 * 
 *         # zero the reflectors since we're done with them
 */
    free(__pyx_v_work);

    /* "scipy/linalg/_decomp_update.pyx":813
 *         # zero the reflectors since we're done with them
 *         # memset can be used here, since r is always fortran order
 *         for j in range(p):             # <<<<<<<<<<<<<<
 *             memset(index2(r, rs, n-p+1+j, k+j), 0, (m-(n-p+1+j))*sizeof(blas_t))
 * 
 */
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_2 = __pyx_t_4;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_j = __pyx_t_3;

      /* "scipy/linalg/_decomp_update.pyx":814
 *         # memset can be used here, since r is always fortran order
 *         for j in range(p):
 *             memset(index2(r, rs, n-p+1+j, k+j), 0, (m-(n-p+1+j))*sizeof(blas_t))             # <<<<<<<<<<<<<<
 * 
 *         # now we have something that looks like
 */
      (void)(memset(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (((__pyx_v_n - __pyx_v_p) + 1) + __pyx_v_j), (__pyx_v_k + __pyx_v_j)), 0, ((__pyx_v_m - (((__pyx_v_n - __pyx_v_p) + 1) + __pyx_v_j)) * (sizeof(__pyx_t_float_complex)))));
    }

    /* "scipy/linalg/_decomp_update.pyx":832
 *         # and the rest of the columns need to be eliminated using rotations.
 * 
 *         for i in range(p):             # <<<<<<<<<<<<<<
 *             for j in range(n-p+i-1, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 */
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_2 = __pyx_t_4;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "scipy/linalg/_decomp_update.pyx":833
 * 
 *         for i in range(p):
 *             for j in range(n-p+i-1, k+i-1, -1):             # <<<<<<<<<<<<<<
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:
 */
      __pyx_t_5 = ((__pyx_v_k + __pyx_v_i) - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = (((__pyx_v_n - __pyx_v_p) + __pyx_v_i) - 1); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
        __pyx_v_j = __pyx_t_7;

        /* "scipy/linalg/_decomp_update.pyx":834
 *         for i in range(p):
 *             for j in range(n-p+i-1, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)             # <<<<<<<<<<<<<<
 *                 if j+1 < n:
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_k + __pyx_v_i)), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_k + __pyx_v_i)), (&__pyx_v_c), (&__pyx_v_s));

        /* "scipy/linalg/_decomp_update.pyx":835
 *             for j in range(n-p+i-1, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:             # <<<<<<<<<<<<<<
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 */
        __pyx_t_1 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
        if (__pyx_t_1) {

          /* "scipy/linalg/_decomp_update.pyx":836
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],             # <<<<<<<<<<<<<<
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],
 */
          __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot((((__pyx_v_n - __pyx_v_k) - __pyx_v_i) - 1), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, ((__pyx_v_k + __pyx_v_i) + 1)), (__pyx_v_rs[1]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), ((__pyx_v_k + __pyx_v_i) + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

          /* "scipy/linalg/_decomp_update.pyx":835
 *             for j in range(n-p+i-1, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:             # <<<<<<<<<<<<<<
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":838
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],             # <<<<<<<<<<<<<<
 *                         c, s.conjugate())
 *     else:
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_float(__pyx_v_s));
      }
    }

    /* "scipy/linalg/_decomp_update.pyx":761
 *     cdef char* trans = 'N'
 * 
 *     if m >= n:             # <<<<<<<<<<<<<<
 *         # if m > n, r looks like this.
 *         # x x x x x x x x x x
 */
    goto __pyx_L3;
  }

  /* "scipy/linalg/_decomp_update.pyx":842
 *     else:
 *         # this case we can only uses givens rotations.
 *         for i in range(p):             # <<<<<<<<<<<<<<
 *             for j in range(m-2, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 */
  /*else*/ {
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_2 = __pyx_t_4;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "scipy/linalg/_decomp_update.pyx":843
 *         # this case we can only uses givens rotations.
 *         for i in range(p):
 *             for j in range(m-2, k+i-1, -1):             # <<<<<<<<<<<<<<
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:
 */
      __pyx_t_5 = ((__pyx_v_k + __pyx_v_i) - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = (__pyx_v_m - 2); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
        __pyx_v_j = __pyx_t_7;

        /* "scipy/linalg/_decomp_update.pyx":844
 *         for i in range(p):
 *             for j in range(m-2, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)             # <<<<<<<<<<<<<<
 *                 if j+1 < n:
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_k + __pyx_v_i)), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_k + __pyx_v_i)), (&__pyx_v_c), (&__pyx_v_s));

        /* "scipy/linalg/_decomp_update.pyx":845
 *             for j in range(m-2, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:             # <<<<<<<<<<<<<<
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 */
        __pyx_t_1 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
        if (__pyx_t_1) {

          /* "scipy/linalg/_decomp_update.pyx":846
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],             # <<<<<<<<<<<<<<
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],
 */
          __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot((((__pyx_v_n - __pyx_v_k) - __pyx_v_i) - 1), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, ((__pyx_v_k + __pyx_v_i) + 1)), (__pyx_v_rs[1]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), ((__pyx_v_k + __pyx_v_i) + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

          /* "scipy/linalg/_decomp_update.pyx":845
 *             for j in range(m-2, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:             # <<<<<<<<<<<<<<
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":848
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],             # <<<<<<<<<<<<<<
 *                         c, s.conjugate())
 *     return 0
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_float(__pyx_v_s));
      }
    }
  }
  __pyx_L3:;

  /* "scipy/linalg/_decomp_update.pyx":850
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],
 *                         c, s.conjugate())
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef void thin_qr_rank_1_update(int m, int n, blas_t* q, int* qs, bint qisF,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":751
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())
 * 
 * cdef int qr_block_col_insert(int m, int n, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *                               blas_t* r, int* rs, int k, int p) nogil:
 *     cdef int i, j
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_insert(int __pyx_v_m, int __pyx_v_n, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_double_complex *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p) {
  int __pyx_v_i;
  int __pyx_v_j;
  __pyx_t_double_complex __pyx_v_c;
  __pyx_t_double_complex __pyx_v_s;
  __pyx_t_double_complex *__pyx_v_tau;
  __pyx_t_double_complex *__pyx_v_work;
  int __pyx_v_info;
  int __pyx_v_lwork;
  char *__pyx_v_side;
  char *__pyx_v_trans;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;

  /* "scipy/linalg/_decomp_update.pyx":755
 *     cdef int i, j
 *     cdef blas_t c, s
 *     cdef blas_t* tau = NULL             # <<<<<<<<<<<<<<
 *     cdef blas_t* work = NULL
 *     cdef int info, lwork
 */
  __pyx_v_tau = NULL;

  /* "scipy/linalg/_decomp_update.pyx":756
 *     cdef blas_t c, s
 *     cdef blas_t* tau = NULL
 *     cdef blas_t* work = NULL             # <<<<<<<<<<<<<<
 *     cdef int info, lwork
 *     cdef char* side = 'R'
 */
  __pyx_v_work = NULL;

  /* "scipy/linalg/_decomp_update.pyx":758
 *     cdef blas_t* work = NULL
 *     cdef int info, lwork
 *     cdef char* side = 'R'             # <<<<<<<<<<<<<<
 *     cdef char* trans = 'N'
 * 
 */
  __pyx_v_side = ((char *)"R");

  /* "scipy/linalg/_decomp_update.pyx":759
 *     cdef int info, lwork
 *     cdef char* side = 'R'
 *     cdef char* trans = 'N'             # <<<<<<<<<<<<<<
 * 
 *     if m >= n:
 */
  __pyx_v_trans = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":761
 *     cdef char* trans = 'N'
 * 
 *     if m >= n:             # <<<<<<<<<<<<<<
 *         # if m > n, r looks like this.
 *         # x x x x x x x x x x
 */
  __pyx_t_1 = ((__pyx_v_m >= __pyx_v_n) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":780
 *         # query the workspace,
 *         # set tau to point at something to keep new MKL working.
 *         tau = &c             # <<<<<<<<<<<<<<
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, &c, -1)
 *         if info < 0:
 */
    __pyx_v_tau = (&__pyx_v_c);

    /* "scipy/linalg/_decomp_update.pyx":781
 *         # set tau to point at something to keep new MKL working.
 *         tau = &c
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, &c, -1)             # <<<<<<<<<<<<<<
 *         if info < 0:
 *             return libc.stdlib.abs(info)
 */
    __pyx_v_info = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_geqrf(((__pyx_v_m - __pyx_v_n) + __pyx_v_p), __pyx_v_p, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - __pyx_v_p), __pyx_v_k), __pyx_v_m, __pyx_v_tau, (&__pyx_v_c), -1);

    /* "scipy/linalg/_decomp_update.pyx":782
 *         tau = &c
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, &c, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":783
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, &c, -1)
 *         if info < 0:
 *             return libc.stdlib.abs(info)             # <<<<<<<<<<<<<<
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 *                      tau, index2(q, qs, 0, n-p), m, &s, -1)
 */
      __pyx_r = abs(__pyx_v_info);
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":782
 *         tau = &c
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, &c, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":784
 *         if info < 0:
 *             return libc.stdlib.abs(info)
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,             # <<<<<<<<<<<<<<
 *                      tau, index2(q, qs, 0, n-p), m, &s, -1)
 *         if info < 0:
 */
    __pyx_v_info = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(__pyx_v_side, __pyx_v_trans, __pyx_v_m, (__pyx_v_m - (__pyx_v_n - __pyx_v_p)), __pyx_v_p, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - __pyx_v_p), __pyx_v_k), __pyx_v_m, __pyx_v_tau, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, (__pyx_v_n - __pyx_v_p)), __pyx_v_m, (&__pyx_v_s), -1);

    /* "scipy/linalg/_decomp_update.pyx":786
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 *                      tau, index2(q, qs, 0, n-p), m, &s, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":787
 *                      tau, index2(q, qs, 0, n-p), m, &s, -1)
 *         if info < 0:
 *             return info             # <<<<<<<<<<<<<<
 * 
 *         # we're only doing one allocation, so use the larger
 */
      __pyx_r = __pyx_v_info;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":786
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 *                      tau, index2(q, qs, 0, n-p), m, &s, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":790
 * 
 *         # we're only doing one allocation, so use the larger
 *         lwork = to_lwork(c, s)             # <<<<<<<<<<<<<<
 * 
 *         # allocate the workspace + tau
 */
    __pyx_v_lwork = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_to_lwork(__pyx_v_c, __pyx_v_s);

    /* "scipy/linalg/_decomp_update.pyx":793
 * 
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n+p, p))*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *         if not work:
 *             return MEMORY_ERROR
 */
    __pyx_t_2 = __pyx_v_p;
    __pyx_t_3 = ((__pyx_v_m - __pyx_v_n) + __pyx_v_p);
    if (((__pyx_t_2 < __pyx_t_3) != 0)) {
      __pyx_t_4 = __pyx_t_2;
    } else {
      __pyx_t_4 = __pyx_t_3;
    }
    __pyx_v_work = ((__pyx_t_double_complex *)malloc(((__pyx_v_lwork + __pyx_t_4) * (sizeof(__pyx_t_double_complex)))));

    /* "scipy/linalg/_decomp_update.pyx":794
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n+p, p))*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 *         tau = work + lwork
 */
    __pyx_t_1 = ((!(__pyx_v_work != 0)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":795
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n+p, p))*sizeof(blas_t))
 *         if not work:
 *             return MEMORY_ERROR             # <<<<<<<<<<<<<<
 *         tau = work + lwork
 * 
 */
      __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":794
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n+p, p))*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 *         tau = work + lwork
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":796
 *         if not work:
 *             return MEMORY_ERROR
 *         tau = work + lwork             # <<<<<<<<<<<<<<
 * 
 *         # qr
 */
    __pyx_v_tau = (__pyx_v_work + __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":799
 * 
 *         # qr
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, work, lwork)             # <<<<<<<<<<<<<<
 *         if info < 0:
 *             return libc.stdlib.abs(info)
 */
    __pyx_v_info = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_geqrf(((__pyx_v_m - __pyx_v_n) + __pyx_v_p), __pyx_v_p, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - __pyx_v_p), __pyx_v_k), __pyx_v_m, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":800
 *         # qr
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 * 
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":801
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, work, lwork)
 *         if info < 0:
 *             return libc.stdlib.abs(info)             # <<<<<<<<<<<<<<
 * 
 *         # apply the Q from this small qr to the last (m-(n-p)) columns of q.
 */
      __pyx_r = abs(__pyx_v_info);
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":800
 *         # qr
 *         info = geqrf(m-n+p, p, index2(r, rs, n-p, k), m, tau, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":804
 * 
 *         # apply the Q from this small qr to the last (m-(n-p)) columns of q.
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,             # <<<<<<<<<<<<<<
 *                 tau, index2(q, qs, 0, n-p), m, work, lwork)
 *         if info < 0:
 */
    __pyx_v_info = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(__pyx_v_side, __pyx_v_trans, __pyx_v_m, (__pyx_v_m - (__pyx_v_n - __pyx_v_p)), __pyx_v_p, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - __pyx_v_p), __pyx_v_k), __pyx_v_m, __pyx_v_tau, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, (__pyx_v_n - __pyx_v_p)), __pyx_v_m, __pyx_v_work, __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":806
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 *                 tau, index2(q, qs, 0, n-p), m, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":807
 *                 tau, index2(q, qs, 0, n-p), m, work, lwork)
 *         if info < 0:
 *             return info             # <<<<<<<<<<<<<<
 * 
 *         libc.stdlib.free(work)
 */
      __pyx_r = __pyx_v_info;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":806
 *         info = ormqr(side, trans, m, m-(n-p), p, index2(r, rs, n-p, k), m,
 *                 tau, index2(q, qs, 0, n-p), m, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":809
 *             return info
 * 
 *         libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 * 
 *         # zero the reflectors since we're done with them
 */
    free(__pyx_v_work);

    /* "scipy/linalg/_decomp_update.pyx":813
 *         # zero the reflectors since we're done with them
 *         # memset can be used here, since r is always fortran order
 *         for j in range(p):             # <<<<<<<<<<<<<<
 *             memset(index2(r, rs, n-p+1+j, k+j), 0, (m-(n-p+1+j))*sizeof(blas_t))
 * 
 */
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_2 = __pyx_t_4;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_j = __pyx_t_3;

      /* "scipy/linalg/_decomp_update.pyx":814
 *         # memset can be used here, since r is always fortran order
 *         for j in range(p):
 *             memset(index2(r, rs, n-p+1+j, k+j), 0, (m-(n-p+1+j))*sizeof(blas_t))             # <<<<<<<<<<<<<<
 * 
 *         # now we have something that looks like
 */
      (void)(memset(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (((__pyx_v_n - __pyx_v_p) + 1) + __pyx_v_j), (__pyx_v_k + __pyx_v_j)), 0, ((__pyx_v_m - (((__pyx_v_n - __pyx_v_p) + 1) + __pyx_v_j)) * (sizeof(__pyx_t_double_complex)))));
    }

    /* "scipy/linalg/_decomp_update.pyx":832
 *         # and the rest of the columns need to be eliminated using rotations.
 * 
 *         for i in range(p):             # <<<<<<<<<<<<<<
 *             for j in range(n-p+i-1, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 */
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_2 = __pyx_t_4;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "scipy/linalg/_decomp_update.pyx":833
 * 
 *         for i in range(p):
 *             for j in range(n-p+i-1, k+i-1, -1):             # <<<<<<<<<<<<<<
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:
 */
      __pyx_t_5 = ((__pyx_v_k + __pyx_v_i) - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = (((__pyx_v_n - __pyx_v_p) + __pyx_v_i) - 1); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
        __pyx_v_j = __pyx_t_7;

        /* "scipy/linalg/_decomp_update.pyx":834
 *         for i in range(p):
 *             for j in range(n-p+i-1, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)             # <<<<<<<<<<<<<<
 *                 if j+1 < n:
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_k + __pyx_v_i)), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_k + __pyx_v_i)), (&__pyx_v_c), (&__pyx_v_s));

        /* "scipy/linalg/_decomp_update.pyx":835
 *             for j in range(n-p+i-1, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:             # <<<<<<<<<<<<<<
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 */
        __pyx_t_1 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
        if (__pyx_t_1) {

          /* "scipy/linalg/_decomp_update.pyx":836
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],             # <<<<<<<<<<<<<<
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],
 */
          __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot((((__pyx_v_n - __pyx_v_k) - __pyx_v_i) - 1), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, ((__pyx_v_k + __pyx_v_i) + 1)), (__pyx_v_rs[1]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), ((__pyx_v_k + __pyx_v_i) + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

          /* "scipy/linalg/_decomp_update.pyx":835
 *             for j in range(n-p+i-1, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:             # <<<<<<<<<<<<<<
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":838
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],             # <<<<<<<<<<<<<<
 *                         c, s.conjugate())
 *     else:
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_double(__pyx_v_s));
      }
    }

    /* "scipy/linalg/_decomp_update.pyx":761
 *     cdef char* trans = 'N'
 * 
 *     if m >= n:             # <<<<<<<<<<<<<<
 *         # if m > n, r looks like this.
 *         # x x x x x x x x x x
 */
    goto __pyx_L3;
  }

  /* "scipy/linalg/_decomp_update.pyx":842
 *     else:
 *         # this case we can only uses givens rotations.
 *         for i in range(p):             # <<<<<<<<<<<<<<
 *             for j in range(m-2, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 */
  /*else*/ {
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_2 = __pyx_t_4;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "scipy/linalg/_decomp_update.pyx":843
 *         # this case we can only uses givens rotations.
 *         for i in range(p):
 *             for j in range(m-2, k+i-1, -1):             # <<<<<<<<<<<<<<
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:
 */
      __pyx_t_5 = ((__pyx_v_k + __pyx_v_i) - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = (__pyx_v_m - 2); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
        __pyx_v_j = __pyx_t_7;

        /* "scipy/linalg/_decomp_update.pyx":844
 *         for i in range(p):
 *             for j in range(m-2, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)             # <<<<<<<<<<<<<<
 *                 if j+1 < n:
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_k + __pyx_v_i)), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_k + __pyx_v_i)), (&__pyx_v_c), (&__pyx_v_s));

        /* "scipy/linalg/_decomp_update.pyx":845
 *             for j in range(m-2, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:             # <<<<<<<<<<<<<<
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 */
        __pyx_t_1 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
        if (__pyx_t_1) {

          /* "scipy/linalg/_decomp_update.pyx":846
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],             # <<<<<<<<<<<<<<
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],
 */
          __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot((((__pyx_v_n - __pyx_v_k) - __pyx_v_i) - 1), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, ((__pyx_v_k + __pyx_v_i) + 1)), (__pyx_v_rs[1]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), ((__pyx_v_k + __pyx_v_i) + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

          /* "scipy/linalg/_decomp_update.pyx":845
 *             for j in range(m-2, k+i-1, -1):
 *                 lartg(index2(r, rs, j, k+i), index2(r, rs, j+1, k+i), &c, &s)
 *                 if j+1 < n:             # <<<<<<<<<<<<<<
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":848
 *                     rot(n-k-i-1, index2(r, rs, j, k+i+1), rs[1],
 *                             index2(r, rs, j+1, k+i+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],             # <<<<<<<<<<<<<<
 *                         c, s.conjugate())
 *     return 0
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_double(__pyx_v_s));
      }
    }
  }
  __pyx_L3:;

  /* "scipy/linalg/_decomp_update.pyx":850
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],
 *                         c, s.conjugate())
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef void thin_qr_rank_1_update(int m, int n, blas_t* q, int* qs, bint qisF,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":751
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())
 * 
 * cdef int qr_block_col_insert(int m, int n, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *                               blas_t* r, int* rs, int k, int p) nogil:
 *     cdef int i, j
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":852
 *     return 0
 * 
 * cdef void thin_qr_rank_1_update(int m, int n, blas_t* q, int* qs, bint qisF,             # <<<<<<<<<<<<<<
 *     blas_t* r, int* rs, blas_t* u, int* us, blas_t* v, int* vs, blas_t* s,
 *     int* ss) nogil:
 */

static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_1_update(int __pyx_v_m, int __pyx_v_n, float *__pyx_v_q, int *__pyx_v_qs, int __pyx_v_qisF, float *__pyx_v_r, int *__pyx_v_rs, float *__pyx_v_u, int *__pyx_v_us, float *__pyx_v_v, int *__pyx_v_vs, float *__pyx_v_s, int *__pyx_v_ss) {
  int __pyx_v_j;
  float __pyx_v_c;
  float __pyx_v_sn;
  float __pyx_v_rlast;
  float __pyx_v_t;
  float __pyx_v_rcond;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":859
 *     """
 *     cdef int j
 *     cdef blas_t c, sn, rlast, t, rcond = 0.0             # <<<<<<<<<<<<<<
 * 
 *     reorth(m, n, q, qs, qisF, u, us, s, &rcond)
 */
  __pyx_v_rcond = 0.0;

  /* "scipy/linalg/_decomp_update.pyx":861
 *     cdef blas_t c, sn, rlast, t, rcond = 0.0
 * 
 *     reorth(m, n, q, qs, qisF, u, us, s, &rcond)             # <<<<<<<<<<<<<<
 * 
 *     # reduce s with givens, using u as the n+1 column of q
 */
  (void)(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_reorth(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_qisF, __pyx_v_u, __pyx_v_us, __pyx_v_s, (&__pyx_v_rcond)));

  /* "scipy/linalg/_decomp_update.pyx":865
 *     # reduce s with givens, using u as the n+1 column of q
 *     # do the first one since the rots will be different.
 *     lartg(index1(s, ss, n-1), index1(s, ss, n), &c, &sn)             # <<<<<<<<<<<<<<
 *     t = index2(r, rs, n-1, n-1)[0]
 *     rlast = -t * sn.conjugate()
 */
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, (__pyx_v_n - 1)), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, __pyx_v_n), (&__pyx_v_c), (&__pyx_v_sn));

  /* "scipy/linalg/_decomp_update.pyx":866
 *     # do the first one since the rots will be different.
 *     lartg(index1(s, ss, n-1), index1(s, ss, n), &c, &sn)
 *     t = index2(r, rs, n-1, n-1)[0]             # <<<<<<<<<<<<<<
 *     rlast = -t * sn.conjugate()
 *     index2(r, rs, n-1, n-1)[0] = t * c
 */
  __pyx_v_t = (__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - 1), (__pyx_v_n - 1))[0]);

  /* "scipy/linalg/_decomp_update.pyx":867
 *     lartg(index1(s, ss, n-1), index1(s, ss, n), &c, &sn)
 *     t = index2(r, rs, n-1, n-1)[0]
 *     rlast = -t * sn.conjugate()             # <<<<<<<<<<<<<<
 *     index2(r, rs, n-1, n-1)[0] = t * c
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 */
  __pyx_v_rlast = ((-__pyx_v_t) *  (__pyx_v_sn));

  /* "scipy/linalg/_decomp_update.pyx":868
 *     t = index2(r, rs, n-1, n-1)[0]
 *     rlast = -t * sn.conjugate()
 *     index2(r, rs, n-1, n-1)[0] = t * c             # <<<<<<<<<<<<<<
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 * 
 */
  (__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - 1), (__pyx_v_n - 1))[0]) = (__pyx_v_t * __pyx_v_c);

  /* "scipy/linalg/_decomp_update.pyx":869
 *     rlast = -t * sn.conjugate()
 *     index2(r, rs, n-1, n-1)[0] = t * c
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     for j in range(n-2, -1, -1):
 */
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_n - 1)), (__pyx_v_qs[0]), __pyx_v_u, (__pyx_v_us[0]), __pyx_v_c,  (__pyx_v_sn));

  /* "scipy/linalg/_decomp_update.pyx":871
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 * 
 *     for j in range(n-2, -1, -1):             # <<<<<<<<<<<<<<
 *         lartg(index1(s, ss, j), index1(s, ss, j+1), &c, &sn)
 *         rot(n-j, index2(r, rs, j, j), rs[1],
 */
  for (__pyx_t_1 = (__pyx_v_n - 2); __pyx_t_1 > -1; __pyx_t_1-=1) {
    __pyx_v_j = __pyx_t_1;

    /* "scipy/linalg/_decomp_update.pyx":872
 * 
 *     for j in range(n-2, -1, -1):
 *         lartg(index1(s, ss, j), index1(s, ss, j+1), &c, &sn)             # <<<<<<<<<<<<<<
 *         rot(n-j, index2(r, rs, j, j), rs[1],
 *                 index2(r, rs, j+1, j), rs[1], c, sn)
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, __pyx_v_j), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, (__pyx_v_j + 1)), (&__pyx_v_c), (&__pyx_v_sn));

    /* "scipy/linalg/_decomp_update.pyx":873
 *     for j in range(n-2, -1, -1):
 *         lartg(index1(s, ss, j), index1(s, ss, j+1), &c, &sn)
 *         rot(n-j, index2(r, rs, j, j), rs[1],             # <<<<<<<<<<<<<<
 *                 index2(r, rs, j+1, j), rs[1], c, sn)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, sn.conjugate())
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_n - __pyx_v_j), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[1]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_sn);

    /* "scipy/linalg/_decomp_update.pyx":875
 *         rot(n-j, index2(r, rs, j, j), rs[1],
 *                 index2(r, rs, j+1, j), rs[1], c, sn)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, sn.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     # add v to the first row of r
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_sn));
  }

  /* "scipy/linalg/_decomp_update.pyx":878
 * 
 *     # add v to the first row of r
 *     blas_t_conj(n, v, vs)             # <<<<<<<<<<<<<<
 *     axpy(n, s[0],  v, vs[0], row(r, rs, 0), rs[1])
 * 
 */
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_n, __pyx_v_v, __pyx_v_vs);

  /* "scipy/linalg/_decomp_update.pyx":879
 *     # add v to the first row of r
 *     blas_t_conj(n, v, vs)
 *     axpy(n, s[0],  v, vs[0], row(r, rs, 0), rs[1])             # <<<<<<<<<<<<<<
 * 
 *     # now r is upper hessenberg with the only value in the last row stored in
 */
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, (__pyx_v_s[0]), __pyx_v_v, (__pyx_v_vs[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, 0), (__pyx_v_rs[1]));

  /* "scipy/linalg/_decomp_update.pyx":884
 *     # rlast (This is very similar to hessenberg_qr below, but this loop ends
 *     # at n-1 instead of n)
 *     for j in range(n-1):             # <<<<<<<<<<<<<<
 *         lartg(index2(r, rs, j, j), index2(r, rs, j+1, j), &c, &sn)
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 */
  __pyx_t_2 = (__pyx_v_n - 1);
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_1 = 0; __pyx_t_1 < __pyx_t_3; __pyx_t_1+=1) {
    __pyx_v_j = __pyx_t_1;

    /* "scipy/linalg/_decomp_update.pyx":885
 *     # at n-1 instead of n)
 *     for j in range(n-1):
 *         lartg(index2(r, rs, j, j), index2(r, rs, j+1, j), &c, &sn)             # <<<<<<<<<<<<<<
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                 index2(r, rs, j+1, j+1), rs[1], c, sn)
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (&__pyx_v_c), (&__pyx_v_sn));

    /* "scipy/linalg/_decomp_update.pyx":886
 *     for j in range(n-1):
 *         lartg(index2(r, rs, j, j), index2(r, rs, j+1, j), &c, &sn)
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1],             # <<<<<<<<<<<<<<
 *                 index2(r, rs, j+1, j+1), rs[1], c, sn)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, sn.conjugate())
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_sn);

    /* "scipy/linalg/_decomp_update.pyx":888
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                 index2(r, rs, j+1, j+1), rs[1], c, sn)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, sn.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     # handle the extra value in rlast
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_sn));
  }

  /* "scipy/linalg/_decomp_update.pyx":891
 * 
 *     # handle the extra value in rlast
 *     lartg(index2(r, rs, n-1, n-1), &rlast, &c, &sn)             # <<<<<<<<<<<<<<
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 * 
 */
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - 1), (__pyx_v_n - 1)), (&__pyx_v_rlast), (&__pyx_v_c), (&__pyx_v_sn));

  /* "scipy/linalg/_decomp_update.pyx":892
 *     # handle the extra value in rlast
 *     lartg(index2(r, rs, n-1, n-1), &rlast, &c, &sn)
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())             # <<<<<<<<<<<<<<
 * 
 * cdef void thin_qr_rank_p_update(int m, int n, int p, blas_t* q, int* qs,
 */
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_n - 1)), (__pyx_v_qs[0]), __pyx_v_u, (__pyx_v_us[0]), __pyx_v_c,  (__pyx_v_sn));

  /* "scipy/linalg/_decomp_update.pyx":852
 *     return 0
 * 
 * cdef void thin_qr_rank_1_update(int m, int n, blas_t* q, int* qs, bint qisF,             # <<<<<<<<<<<<<<
 *     blas_t* r, int* rs, blas_t* u, int* us, blas_t* v, int* vs, blas_t* s,
 *     int* ss) nogil:
 */

  /* function exit code */
}

static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_1_update(int __pyx_v_m, int __pyx_v_n, double *__pyx_v_q, int *__pyx_v_qs, int __pyx_v_qisF, double *__pyx_v_r, int *__pyx_v_rs, double *__pyx_v_u, int *__pyx_v_us, double *__pyx_v_v, int *__pyx_v_vs, double *__pyx_v_s, int *__pyx_v_ss) {
  int __pyx_v_j;
  double __pyx_v_c;
  double __pyx_v_sn;
  double __pyx_v_rlast;
  double __pyx_v_t;
  double __pyx_v_rcond;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":859
 *     """
 *     cdef int j
 *     cdef blas_t c, sn, rlast, t, rcond = 0.0             # <<<<<<<<<<<<<<
 * 
 *     reorth(m, n, q, qs, qisF, u, us, s, &rcond)
 */
  __pyx_v_rcond = 0.0;

  /* "scipy/linalg/_decomp_update.pyx":861
 *     cdef blas_t c, sn, rlast, t, rcond = 0.0
 * 
 *     reorth(m, n, q, qs, qisF, u, us, s, &rcond)             # <<<<<<<<<<<<<<
 * 
 *     # reduce s with givens, using u as the n+1 column of q
 */
  (void)(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_reorth(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_qisF, __pyx_v_u, __pyx_v_us, __pyx_v_s, (&__pyx_v_rcond)));

  /* "scipy/linalg/_decomp_update.pyx":865
 *     # reduce s with givens, using u as the n+1 column of q
 *     # do the first one since the rots will be different.
 *     lartg(index1(s, ss, n-1), index1(s, ss, n), &c, &sn)             # <<<<<<<<<<<<<<
 *     t = index2(r, rs, n-1, n-1)[0]
 *     rlast = -t * sn.conjugate()
 */
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, (__pyx_v_n - 1)), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, __pyx_v_n), (&__pyx_v_c), (&__pyx_v_sn));

  /* "scipy/linalg/_decomp_update.pyx":866
 *     # do the first one since the rots will be different.
 *     lartg(index1(s, ss, n-1), index1(s, ss, n), &c, &sn)
 *     t = index2(r, rs, n-1, n-1)[0]             # <<<<<<<<<<<<<<
 *     rlast = -t * sn.conjugate()
 *     index2(r, rs, n-1, n-1)[0] = t * c
 */
  __pyx_v_t = (__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - 1), (__pyx_v_n - 1))[0]);

  /* "scipy/linalg/_decomp_update.pyx":867
 *     lartg(index1(s, ss, n-1), index1(s, ss, n), &c, &sn)
 *     t = index2(r, rs, n-1, n-1)[0]
 *     rlast = -t * sn.conjugate()             # <<<<<<<<<<<<<<
 *     index2(r, rs, n-1, n-1)[0] = t * c
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 */
  __pyx_v_rlast = ((-__pyx_v_t) *  (__pyx_v_sn));

  /* "scipy/linalg/_decomp_update.pyx":868
 *     t = index2(r, rs, n-1, n-1)[0]
 *     rlast = -t * sn.conjugate()
 *     index2(r, rs, n-1, n-1)[0] = t * c             # <<<<<<<<<<<<<<
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 * 
 */
  (__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - 1), (__pyx_v_n - 1))[0]) = (__pyx_v_t * __pyx_v_c);

  /* "scipy/linalg/_decomp_update.pyx":869
 *     rlast = -t * sn.conjugate()
 *     index2(r, rs, n-1, n-1)[0] = t * c
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     for j in range(n-2, -1, -1):
 */
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_n - 1)), (__pyx_v_qs[0]), __pyx_v_u, (__pyx_v_us[0]), __pyx_v_c,  (__pyx_v_sn));

  /* "scipy/linalg/_decomp_update.pyx":871
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 * 
 *     for j in range(n-2, -1, -1):             # <<<<<<<<<<<<<<
 *         lartg(index1(s, ss, j), index1(s, ss, j+1), &c, &sn)
 *         rot(n-j, index2(r, rs, j, j), rs[1],
 */
  for (__pyx_t_1 = (__pyx_v_n - 2); __pyx_t_1 > -1; __pyx_t_1-=1) {
    __pyx_v_j = __pyx_t_1;

    /* "scipy/linalg/_decomp_update.pyx":872
 * 
 *     for j in range(n-2, -1, -1):
 *         lartg(index1(s, ss, j), index1(s, ss, j+1), &c, &sn)             # <<<<<<<<<<<<<<
 *         rot(n-j, index2(r, rs, j, j), rs[1],
 *                 index2(r, rs, j+1, j), rs[1], c, sn)
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, __pyx_v_j), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, (__pyx_v_j + 1)), (&__pyx_v_c), (&__pyx_v_sn));

    /* "scipy/linalg/_decomp_update.pyx":873
 *     for j in range(n-2, -1, -1):
 *         lartg(index1(s, ss, j), index1(s, ss, j+1), &c, &sn)
 *         rot(n-j, index2(r, rs, j, j), rs[1],             # <<<<<<<<<<<<<<
 *                 index2(r, rs, j+1, j), rs[1], c, sn)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, sn.conjugate())
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_n - __pyx_v_j), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[1]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_sn);

    /* "scipy/linalg/_decomp_update.pyx":875
 *         rot(n-j, index2(r, rs, j, j), rs[1],
 *                 index2(r, rs, j+1, j), rs[1], c, sn)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, sn.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     # add v to the first row of r
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_sn));
  }

  /* "scipy/linalg/_decomp_update.pyx":878
 * 
 *     # add v to the first row of r
 *     blas_t_conj(n, v, vs)             # <<<<<<<<<<<<<<
 *     axpy(n, s[0],  v, vs[0], row(r, rs, 0), rs[1])
 * 
 */
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_n, __pyx_v_v, __pyx_v_vs);

  /* "scipy/linalg/_decomp_update.pyx":879
 *     # add v to the first row of r
 *     blas_t_conj(n, v, vs)
 *     axpy(n, s[0],  v, vs[0], row(r, rs, 0), rs[1])             # <<<<<<<<<<<<<<
 * 
 *     # now r is upper hessenberg with the only value in the last row stored in
 */
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, (__pyx_v_s[0]), __pyx_v_v, (__pyx_v_vs[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, 0), (__pyx_v_rs[1]));

  /* "scipy/linalg/_decomp_update.pyx":884
 *     # rlast (This is very similar to hessenberg_qr below, but this loop ends
 *     # at n-1 instead of n)
 *     for j in range(n-1):             # <<<<<<<<<<<<<<
 *         lartg(index2(r, rs, j, j), index2(r, rs, j+1, j), &c, &sn)
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 */
  __pyx_t_2 = (__pyx_v_n - 1);
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_1 = 0; __pyx_t_1 < __pyx_t_3; __pyx_t_1+=1) {
    __pyx_v_j = __pyx_t_1;

    /* "scipy/linalg/_decomp_update.pyx":885
 *     # at n-1 instead of n)
 *     for j in range(n-1):
 *         lartg(index2(r, rs, j, j), index2(r, rs, j+1, j), &c, &sn)             # <<<<<<<<<<<<<<
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                 index2(r, rs, j+1, j+1), rs[1], c, sn)
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (&__pyx_v_c), (&__pyx_v_sn));

    /* "scipy/linalg/_decomp_update.pyx":886
 *     for j in range(n-1):
 *         lartg(index2(r, rs, j, j), index2(r, rs, j+1, j), &c, &sn)
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1],             # <<<<<<<<<<<<<<
 *                 index2(r, rs, j+1, j+1), rs[1], c, sn)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, sn.conjugate())
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_sn);

    /* "scipy/linalg/_decomp_update.pyx":888
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                 index2(r, rs, j+1, j+1), rs[1], c, sn)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, sn.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     # handle the extra value in rlast
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_sn));
  }

  /* "scipy/linalg/_decomp_update.pyx":891
 * 
 *     # handle the extra value in rlast
 *     lartg(index2(r, rs, n-1, n-1), &rlast, &c, &sn)             # <<<<<<<<<<<<<<
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 * 
 */
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - 1), (__pyx_v_n - 1)), (&__pyx_v_rlast), (&__pyx_v_c), (&__pyx_v_sn));

  /* "scipy/linalg/_decomp_update.pyx":892
 *     # handle the extra value in rlast
 *     lartg(index2(r, rs, n-1, n-1), &rlast, &c, &sn)
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())             # <<<<<<<<<<<<<<
 * 
 * cdef void thin_qr_rank_p_update(int m, int n, int p, blas_t* q, int* qs,
 */
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_n - 1)), (__pyx_v_qs[0]), __pyx_v_u, (__pyx_v_us[0]), __pyx_v_c,  (__pyx_v_sn));

  /* "scipy/linalg/_decomp_update.pyx":852
 *     return 0
 * 
 * cdef void thin_qr_rank_1_update(int m, int n, blas_t* q, int* qs, bint qisF,             # <<<<<<<<<<<<<<
 *     blas_t* r, int* rs, blas_t* u, int* us, blas_t* v, int* vs, blas_t* s,
 *     int* ss) nogil:
 */

  /* function exit code */
}

static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_1_update(int __pyx_v_m, int __pyx_v_n, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_qs, int __pyx_v_qisF, __pyx_t_float_complex *__pyx_v_r, int *__pyx_v_rs, __pyx_t_float_complex *__pyx_v_u, int *__pyx_v_us, __pyx_t_float_complex *__pyx_v_v, int *__pyx_v_vs, __pyx_t_float_complex *__pyx_v_s, int *__pyx_v_ss) {
  int __pyx_v_j;
  __pyx_t_float_complex __pyx_v_c;
  __pyx_t_float_complex __pyx_v_sn;
  __pyx_t_float_complex __pyx_v_rlast;
  __pyx_t_float_complex __pyx_v_t;
  __pyx_t_float_complex __pyx_v_rcond;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":859
 *     """
 *     cdef int j
 *     cdef blas_t c, sn, rlast, t, rcond = 0.0             # <<<<<<<<<<<<<<
 * 
 *     reorth(m, n, q, qs, qisF, u, us, s, &rcond)
 */
  __pyx_v_rcond = __pyx_t_float_complex_from_parts(0.0, 0);

  /* "scipy/linalg/_decomp_update.pyx":861
 *     cdef blas_t c, sn, rlast, t, rcond = 0.0
 * 
 *     reorth(m, n, q, qs, qisF, u, us, s, &rcond)             # <<<<<<<<<<<<<<
 * 
 *     # reduce s with givens, using u as the n+1 column of q
 */
  (void)(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_reorth(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_qisF, __pyx_v_u, __pyx_v_us, __pyx_v_s, (&__pyx_v_rcond)));

  /* "scipy/linalg/_decomp_update.pyx":865
 *     # reduce s with givens, using u as the n+1 column of q
 *     # do the first one since the rots will be different.
 *     lartg(index1(s, ss, n-1), index1(s, ss, n), &c, &sn)             # <<<<<<<<<<<<<<
 *     t = index2(r, rs, n-1, n-1)[0]
 *     rlast = -t * sn.conjugate()
 */
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, (__pyx_v_n - 1)), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, __pyx_v_n), (&__pyx_v_c), (&__pyx_v_sn));

  /* "scipy/linalg/_decomp_update.pyx":866
 *     # do the first one since the rots will be different.
 *     lartg(index1(s, ss, n-1), index1(s, ss, n), &c, &sn)
 *     t = index2(r, rs, n-1, n-1)[0]             # <<<<<<<<<<<<<<
 *     rlast = -t * sn.conjugate()
 *     index2(r, rs, n-1, n-1)[0] = t * c
 */
  __pyx_v_t = (__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - 1), (__pyx_v_n - 1))[0]);

  /* "scipy/linalg/_decomp_update.pyx":867
 *     lartg(index1(s, ss, n-1), index1(s, ss, n), &c, &sn)
 *     t = index2(r, rs, n-1, n-1)[0]
 *     rlast = -t * sn.conjugate()             # <<<<<<<<<<<<<<
 *     index2(r, rs, n-1, n-1)[0] = t * c
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 */
  __pyx_v_rlast = __Pyx_c_prod_float(__Pyx_c_neg_float(__pyx_v_t), __Pyx_c_conj_float(__pyx_v_sn));

  /* "scipy/linalg/_decomp_update.pyx":868
 *     t = index2(r, rs, n-1, n-1)[0]
 *     rlast = -t * sn.conjugate()
 *     index2(r, rs, n-1, n-1)[0] = t * c             # <<<<<<<<<<<<<<
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 * 
 */
  (__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - 1), (__pyx_v_n - 1))[0]) = __Pyx_c_prod_float(__pyx_v_t, __pyx_v_c);

  /* "scipy/linalg/_decomp_update.pyx":869
 *     rlast = -t * sn.conjugate()
 *     index2(r, rs, n-1, n-1)[0] = t * c
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     for j in range(n-2, -1, -1):
 */
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_n - 1)), (__pyx_v_qs[0]), __pyx_v_u, (__pyx_v_us[0]), __pyx_v_c, __Pyx_c_conj_float(__pyx_v_sn));

  /* "scipy/linalg/_decomp_update.pyx":871
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 * 
 *     for j in range(n-2, -1, -1):             # <<<<<<<<<<<<<<
 *         lartg(index1(s, ss, j), index1(s, ss, j+1), &c, &sn)
 *         rot(n-j, index2(r, rs, j, j), rs[1],
 */
  for (__pyx_t_1 = (__pyx_v_n - 2); __pyx_t_1 > -1; __pyx_t_1-=1) {
    __pyx_v_j = __pyx_t_1;

    /* "scipy/linalg/_decomp_update.pyx":872
 * 
 *     for j in range(n-2, -1, -1):
 *         lartg(index1(s, ss, j), index1(s, ss, j+1), &c, &sn)             # <<<<<<<<<<<<<<
 *         rot(n-j, index2(r, rs, j, j), rs[1],
 *                 index2(r, rs, j+1, j), rs[1], c, sn)
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, __pyx_v_j), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, (__pyx_v_j + 1)), (&__pyx_v_c), (&__pyx_v_sn));

    /* "scipy/linalg/_decomp_update.pyx":873
 *     for j in range(n-2, -1, -1):
 *         lartg(index1(s, ss, j), index1(s, ss, j+1), &c, &sn)
 *         rot(n-j, index2(r, rs, j, j), rs[1],             # <<<<<<<<<<<<<<
 *                 index2(r, rs, j+1, j), rs[1], c, sn)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, sn.conjugate())
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_n - __pyx_v_j), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[1]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_sn);

    /* "scipy/linalg/_decomp_update.pyx":875
 *         rot(n-j, index2(r, rs, j, j), rs[1],
 *                 index2(r, rs, j+1, j), rs[1], c, sn)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, sn.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     # add v to the first row of r
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_float(__pyx_v_sn));
  }

  /* "scipy/linalg/_decomp_update.pyx":878
 * 
 *     # add v to the first row of r
 *     blas_t_conj(n, v, vs)             # <<<<<<<<<<<<<<
 *     axpy(n, s[0],  v, vs[0], row(r, rs, 0), rs[1])
 * 
 */
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_n, __pyx_v_v, __pyx_v_vs);

  /* "scipy/linalg/_decomp_update.pyx":879
 *     # add v to the first row of r
 *     blas_t_conj(n, v, vs)
 *     axpy(n, s[0],  v, vs[0], row(r, rs, 0), rs[1])             # <<<<<<<<<<<<<<
 * 
 *     # now r is upper hessenberg with the only value in the last row stored in
 */
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, (__pyx_v_s[0]), __pyx_v_v, (__pyx_v_vs[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, 0), (__pyx_v_rs[1]));

  /* "scipy/linalg/_decomp_update.pyx":884
 *     # rlast (This is very similar to hessenberg_qr below, but this loop ends
 *     # at n-1 instead of n)
 *     for j in range(n-1):             # <<<<<<<<<<<<<<
 *         lartg(index2(r, rs, j, j), index2(r, rs, j+1, j), &c, &sn)
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 */
  __pyx_t_2 = (__pyx_v_n - 1);
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_1 = 0; __pyx_t_1 < __pyx_t_3; __pyx_t_1+=1) {
    __pyx_v_j = __pyx_t_1;

    /* "scipy/linalg/_decomp_update.pyx":885
 *     # at n-1 instead of n)
 *     for j in range(n-1):
 *         lartg(index2(r, rs, j, j), index2(r, rs, j+1, j), &c, &sn)             # <<<<<<<<<<<<<<
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                 index2(r, rs, j+1, j+1), rs[1], c, sn)
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (&__pyx_v_c), (&__pyx_v_sn));

    /* "scipy/linalg/_decomp_update.pyx":886
 *     for j in range(n-1):
 *         lartg(index2(r, rs, j, j), index2(r, rs, j+1, j), &c, &sn)
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1],             # <<<<<<<<<<<<<<
 *                 index2(r, rs, j+1, j+1), rs[1], c, sn)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, sn.conjugate())
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_sn);

    /* "scipy/linalg/_decomp_update.pyx":888
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                 index2(r, rs, j+1, j+1), rs[1], c, sn)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, sn.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     # handle the extra value in rlast
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_float(__pyx_v_sn));
  }

  /* "scipy/linalg/_decomp_update.pyx":891
 * 
 *     # handle the extra value in rlast
 *     lartg(index2(r, rs, n-1, n-1), &rlast, &c, &sn)             # <<<<<<<<<<<<<<
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 * 
 */
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - 1), (__pyx_v_n - 1)), (&__pyx_v_rlast), (&__pyx_v_c), (&__pyx_v_sn));

  /* "scipy/linalg/_decomp_update.pyx":892
 *     # handle the extra value in rlast
 *     lartg(index2(r, rs, n-1, n-1), &rlast, &c, &sn)
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())             # <<<<<<<<<<<<<<
 * 
 * cdef void thin_qr_rank_p_update(int m, int n, int p, blas_t* q, int* qs,
 */
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_n - 1)), (__pyx_v_qs[0]), __pyx_v_u, (__pyx_v_us[0]), __pyx_v_c, __Pyx_c_conj_float(__pyx_v_sn));

  /* "scipy/linalg/_decomp_update.pyx":852
 *     return 0
 * 
 * cdef void thin_qr_rank_1_update(int m, int n, blas_t* q, int* qs, bint qisF,             # <<<<<<<<<<<<<<
 *     blas_t* r, int* rs, blas_t* u, int* us, blas_t* v, int* vs, blas_t* s,
 *     int* ss) nogil:
 */

  /* function exit code */
}

static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_1_update(int __pyx_v_m, int __pyx_v_n, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_qs, int __pyx_v_qisF, __pyx_t_double_complex *__pyx_v_r, int *__pyx_v_rs, __pyx_t_double_complex *__pyx_v_u, int *__pyx_v_us, __pyx_t_double_complex *__pyx_v_v, int *__pyx_v_vs, __pyx_t_double_complex *__pyx_v_s, int *__pyx_v_ss) {
  int __pyx_v_j;
  __pyx_t_double_complex __pyx_v_c;
  __pyx_t_double_complex __pyx_v_sn;
  __pyx_t_double_complex __pyx_v_rlast;
  __pyx_t_double_complex __pyx_v_t;
  __pyx_t_double_complex __pyx_v_rcond;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":859
 *     """
 *     cdef int j
 *     cdef blas_t c, sn, rlast, t, rcond = 0.0             # <<<<<<<<<<<<<<
 * 
 *     reorth(m, n, q, qs, qisF, u, us, s, &rcond)
 */
  __pyx_v_rcond = __pyx_t_double_complex_from_parts(0.0, 0);

  /* "scipy/linalg/_decomp_update.pyx":861
 *     cdef blas_t c, sn, rlast, t, rcond = 0.0
 * 
 *     reorth(m, n, q, qs, qisF, u, us, s, &rcond)             # <<<<<<<<<<<<<<
 * 
 *     # reduce s with givens, using u as the n+1 column of q
 */
  (void)(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_reorth(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_qisF, __pyx_v_u, __pyx_v_us, __pyx_v_s, (&__pyx_v_rcond)));

  /* "scipy/linalg/_decomp_update.pyx":865
 *     # reduce s with givens, using u as the n+1 column of q
 *     # do the first one since the rots will be different.
 *     lartg(index1(s, ss, n-1), index1(s, ss, n), &c, &sn)             # <<<<<<<<<<<<<<
 *     t = index2(r, rs, n-1, n-1)[0]
 *     rlast = -t * sn.conjugate()
 */
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, (__pyx_v_n - 1)), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, __pyx_v_n), (&__pyx_v_c), (&__pyx_v_sn));

  /* "scipy/linalg/_decomp_update.pyx":866
 *     # do the first one since the rots will be different.
 *     lartg(index1(s, ss, n-1), index1(s, ss, n), &c, &sn)
 *     t = index2(r, rs, n-1, n-1)[0]             # <<<<<<<<<<<<<<
 *     rlast = -t * sn.conjugate()
 *     index2(r, rs, n-1, n-1)[0] = t * c
 */
  __pyx_v_t = (__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - 1), (__pyx_v_n - 1))[0]);

  /* "scipy/linalg/_decomp_update.pyx":867
 *     lartg(index1(s, ss, n-1), index1(s, ss, n), &c, &sn)
 *     t = index2(r, rs, n-1, n-1)[0]
 *     rlast = -t * sn.conjugate()             # <<<<<<<<<<<<<<
 *     index2(r, rs, n-1, n-1)[0] = t * c
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 */
  __pyx_v_rlast = __Pyx_c_prod_double(__Pyx_c_neg_double(__pyx_v_t), __Pyx_c_conj_double(__pyx_v_sn));

  /* "scipy/linalg/_decomp_update.pyx":868
 *     t = index2(r, rs, n-1, n-1)[0]
 *     rlast = -t * sn.conjugate()
 *     index2(r, rs, n-1, n-1)[0] = t * c             # <<<<<<<<<<<<<<
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 * 
 */
  (__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - 1), (__pyx_v_n - 1))[0]) = __Pyx_c_prod_double(__pyx_v_t, __pyx_v_c);

  /* "scipy/linalg/_decomp_update.pyx":869
 *     rlast = -t * sn.conjugate()
 *     index2(r, rs, n-1, n-1)[0] = t * c
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     for j in range(n-2, -1, -1):
 */
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_n - 1)), (__pyx_v_qs[0]), __pyx_v_u, (__pyx_v_us[0]), __pyx_v_c, __Pyx_c_conj_double(__pyx_v_sn));

  /* "scipy/linalg/_decomp_update.pyx":871
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 * 
 *     for j in range(n-2, -1, -1):             # <<<<<<<<<<<<<<
 *         lartg(index1(s, ss, j), index1(s, ss, j+1), &c, &sn)
 *         rot(n-j, index2(r, rs, j, j), rs[1],
 */
  for (__pyx_t_1 = (__pyx_v_n - 2); __pyx_t_1 > -1; __pyx_t_1-=1) {
    __pyx_v_j = __pyx_t_1;

    /* "scipy/linalg/_decomp_update.pyx":872
 * 
 *     for j in range(n-2, -1, -1):
 *         lartg(index1(s, ss, j), index1(s, ss, j+1), &c, &sn)             # <<<<<<<<<<<<<<
 *         rot(n-j, index2(r, rs, j, j), rs[1],
 *                 index2(r, rs, j+1, j), rs[1], c, sn)
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, __pyx_v_j), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_s, __pyx_v_ss, (__pyx_v_j + 1)), (&__pyx_v_c), (&__pyx_v_sn));

    /* "scipy/linalg/_decomp_update.pyx":873
 *     for j in range(n-2, -1, -1):
 *         lartg(index1(s, ss, j), index1(s, ss, j+1), &c, &sn)
 *         rot(n-j, index2(r, rs, j, j), rs[1],             # <<<<<<<<<<<<<<
 *                 index2(r, rs, j+1, j), rs[1], c, sn)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, sn.conjugate())
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_n - __pyx_v_j), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[1]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_sn);

    /* "scipy/linalg/_decomp_update.pyx":875
 *         rot(n-j, index2(r, rs, j, j), rs[1],
 *                 index2(r, rs, j+1, j), rs[1], c, sn)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, sn.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     # add v to the first row of r
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_double(__pyx_v_sn));
  }

  /* "scipy/linalg/_decomp_update.pyx":878
 * 
 *     # add v to the first row of r
 *     blas_t_conj(n, v, vs)             # <<<<<<<<<<<<<<
 *     axpy(n, s[0],  v, vs[0], row(r, rs, 0), rs[1])
 * 
 */
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_n, __pyx_v_v, __pyx_v_vs);

  /* "scipy/linalg/_decomp_update.pyx":879
 *     # add v to the first row of r
 *     blas_t_conj(n, v, vs)
 *     axpy(n, s[0],  v, vs[0], row(r, rs, 0), rs[1])             # <<<<<<<<<<<<<<
 * 
 *     # now r is upper hessenberg with the only value in the last row stored in
 */
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, (__pyx_v_s[0]), __pyx_v_v, (__pyx_v_vs[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, 0), (__pyx_v_rs[1]));

  /* "scipy/linalg/_decomp_update.pyx":884
 *     # rlast (This is very similar to hessenberg_qr below, but this loop ends
 *     # at n-1 instead of n)
 *     for j in range(n-1):             # <<<<<<<<<<<<<<
 *         lartg(index2(r, rs, j, j), index2(r, rs, j+1, j), &c, &sn)
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 */
  __pyx_t_2 = (__pyx_v_n - 1);
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_1 = 0; __pyx_t_1 < __pyx_t_3; __pyx_t_1+=1) {
    __pyx_v_j = __pyx_t_1;

    /* "scipy/linalg/_decomp_update.pyx":885
 *     # at n-1 instead of n)
 *     for j in range(n-1):
 *         lartg(index2(r, rs, j, j), index2(r, rs, j+1, j), &c, &sn)             # <<<<<<<<<<<<<<
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                 index2(r, rs, j+1, j+1), rs[1], c, sn)
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (&__pyx_v_c), (&__pyx_v_sn));

    /* "scipy/linalg/_decomp_update.pyx":886
 *     for j in range(n-1):
 *         lartg(index2(r, rs, j, j), index2(r, rs, j+1, j), &c, &sn)
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1],             # <<<<<<<<<<<<<<
 *                 index2(r, rs, j+1, j+1), rs[1], c, sn)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, sn.conjugate())
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_sn);

    /* "scipy/linalg/_decomp_update.pyx":888
 *         rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                 index2(r, rs, j+1, j+1), rs[1], c, sn)
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, sn.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     # handle the extra value in rlast
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_double(__pyx_v_sn));
  }

  /* "scipy/linalg/_decomp_update.pyx":891
 * 
 *     # handle the extra value in rlast
 *     lartg(index2(r, rs, n-1, n-1), &rlast, &c, &sn)             # <<<<<<<<<<<<<<
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 * 
 */
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_n - 1), (__pyx_v_n - 1)), (&__pyx_v_rlast), (&__pyx_v_c), (&__pyx_v_sn));

  /* "scipy/linalg/_decomp_update.pyx":892
 *     # handle the extra value in rlast
 *     lartg(index2(r, rs, n-1, n-1), &rlast, &c, &sn)
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())             # <<<<<<<<<<<<<<
 * 
 * cdef void thin_qr_rank_p_update(int m, int n, int p, blas_t* q, int* qs,
 */
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_n - 1)), (__pyx_v_qs[0]), __pyx_v_u, (__pyx_v_us[0]), __pyx_v_c, __Pyx_c_conj_double(__pyx_v_sn));

  /* "scipy/linalg/_decomp_update.pyx":852
 *     return 0
 * 
 * cdef void thin_qr_rank_1_update(int m, int n, blas_t* q, int* qs, bint qisF,             # <<<<<<<<<<<<<<
 *     blas_t* r, int* rs, blas_t* u, int* us, blas_t* v, int* vs, blas_t* s,
 *     int* ss) nogil:
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":894
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 * 
 * cdef void thin_qr_rank_p_update(int m, int n, int p, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *     bint qisF, blas_t* r, int* rs, blas_t* u, int* us, blas_t* v, int* vs,
 *     blas_t* s, int* ss) nogil:
 */

static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_p_update(int __pyx_v_m, int __pyx_v_n, int __pyx_v_p, float *__pyx_v_q, int *__pyx_v_qs, int __pyx_v_qisF, float *__pyx_v_r, int *__pyx_v_rs, float *__pyx_v_u, int *__pyx_v_us, float *__pyx_v_v, int *__pyx_v_vs, float *__pyx_v_s, int *__pyx_v_ss) {
  int __pyx_v_j;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":902
 *     cdef int j
 * 
 *     for j in range(p):             # <<<<<<<<<<<<<<
 *         thin_qr_rank_1_update(m, n, q, qs, qisF, r, rs, col(u, us, j), us,
 *                               col(v, vs, j), vs, s, ss)
 */
  __pyx_t_1 = __pyx_v_p;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":903
 * 
 *     for j in range(p):
 *         thin_qr_rank_1_update(m, n, q, qs, qisF, r, rs, col(u, us, j), us,             # <<<<<<<<<<<<<<
 *                               col(v, vs, j), vs, s, ss)
 * 
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_1_update(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_qisF, __pyx_v_r, __pyx_v_rs, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), __pyx_v_us, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_v, __pyx_v_vs, __pyx_v_j), __pyx_v_vs, __pyx_v_s, __pyx_v_ss);
  }

  /* "scipy/linalg/_decomp_update.pyx":894
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 * 
 * cdef void thin_qr_rank_p_update(int m, int n, int p, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *     bint qisF, blas_t* r, int* rs, blas_t* u, int* us, blas_t* v, int* vs,
 *     blas_t* s, int* ss) nogil:
 */

  /* function exit code */
}

static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_p_update(int __pyx_v_m, int __pyx_v_n, int __pyx_v_p, double *__pyx_v_q, int *__pyx_v_qs, int __pyx_v_qisF, double *__pyx_v_r, int *__pyx_v_rs, double *__pyx_v_u, int *__pyx_v_us, double *__pyx_v_v, int *__pyx_v_vs, double *__pyx_v_s, int *__pyx_v_ss) {
  int __pyx_v_j;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":902
 *     cdef int j
 * 
 *     for j in range(p):             # <<<<<<<<<<<<<<
 *         thin_qr_rank_1_update(m, n, q, qs, qisF, r, rs, col(u, us, j), us,
 *                               col(v, vs, j), vs, s, ss)
 */
  __pyx_t_1 = __pyx_v_p;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":903
 * 
 *     for j in range(p):
 *         thin_qr_rank_1_update(m, n, q, qs, qisF, r, rs, col(u, us, j), us,             # <<<<<<<<<<<<<<
 *                               col(v, vs, j), vs, s, ss)
 * 
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_1_update(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_qisF, __pyx_v_r, __pyx_v_rs, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), __pyx_v_us, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_v, __pyx_v_vs, __pyx_v_j), __pyx_v_vs, __pyx_v_s, __pyx_v_ss);
  }

  /* "scipy/linalg/_decomp_update.pyx":894
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 * 
 * cdef void thin_qr_rank_p_update(int m, int n, int p, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *     bint qisF, blas_t* r, int* rs, blas_t* u, int* us, blas_t* v, int* vs,
 *     blas_t* s, int* ss) nogil:
 */

  /* function exit code */
}

static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_p_update(int __pyx_v_m, int __pyx_v_n, int __pyx_v_p, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_qs, int __pyx_v_qisF, __pyx_t_float_complex *__pyx_v_r, int *__pyx_v_rs, __pyx_t_float_complex *__pyx_v_u, int *__pyx_v_us, __pyx_t_float_complex *__pyx_v_v, int *__pyx_v_vs, __pyx_t_float_complex *__pyx_v_s, int *__pyx_v_ss) {
  int __pyx_v_j;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":902
 *     cdef int j
 * 
 *     for j in range(p):             # <<<<<<<<<<<<<<
 *         thin_qr_rank_1_update(m, n, q, qs, qisF, r, rs, col(u, us, j), us,
 *                               col(v, vs, j), vs, s, ss)
 */
  __pyx_t_1 = __pyx_v_p;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":903
 * 
 *     for j in range(p):
 *         thin_qr_rank_1_update(m, n, q, qs, qisF, r, rs, col(u, us, j), us,             # <<<<<<<<<<<<<<
 *                               col(v, vs, j), vs, s, ss)
 * 
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_1_update(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_qisF, __pyx_v_r, __pyx_v_rs, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), __pyx_v_us, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_v, __pyx_v_vs, __pyx_v_j), __pyx_v_vs, __pyx_v_s, __pyx_v_ss);
  }

  /* "scipy/linalg/_decomp_update.pyx":894
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 * 
 * cdef void thin_qr_rank_p_update(int m, int n, int p, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *     bint qisF, blas_t* r, int* rs, blas_t* u, int* us, blas_t* v, int* vs,
 *     blas_t* s, int* ss) nogil:
 */

  /* function exit code */
}

static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_p_update(int __pyx_v_m, int __pyx_v_n, int __pyx_v_p, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_qs, int __pyx_v_qisF, __pyx_t_double_complex *__pyx_v_r, int *__pyx_v_rs, __pyx_t_double_complex *__pyx_v_u, int *__pyx_v_us, __pyx_t_double_complex *__pyx_v_v, int *__pyx_v_vs, __pyx_t_double_complex *__pyx_v_s, int *__pyx_v_ss) {
  int __pyx_v_j;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":902
 *     cdef int j
 * 
 *     for j in range(p):             # <<<<<<<<<<<<<<
 *         thin_qr_rank_1_update(m, n, q, qs, qisF, r, rs, col(u, us, j), us,
 *                               col(v, vs, j), vs, s, ss)
 */
  __pyx_t_1 = __pyx_v_p;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":903
 * 
 *     for j in range(p):
 *         thin_qr_rank_1_update(m, n, q, qs, qisF, r, rs, col(u, us, j), us,             # <<<<<<<<<<<<<<
 *                               col(v, vs, j), vs, s, ss)
 * 
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_1_update(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_qisF, __pyx_v_r, __pyx_v_rs, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_u, __pyx_v_us, __pyx_v_j), __pyx_v_us, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_v, __pyx_v_vs, __pyx_v_j), __pyx_v_vs, __pyx_v_s, __pyx_v_ss);
  }

  /* "scipy/linalg/_decomp_update.pyx":894
 *     rot(m, col(q, qs, n-1), qs[0], u, us[0], c, sn.conjugate())
 * 
 * cdef void thin_qr_rank_p_update(int m, int n, int p, blas_t* q, int* qs,             # <<<<<<<<<<<<<<
 *     bint qisF, blas_t* r, int* rs, blas_t* u, int* us, blas_t* v, int* vs,
 *     blas_t* s, int* ss) nogil:
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":906
 *                               col(v, vs, j), vs, s, ss)
 * 
 * cdef void qr_rank_1_update(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                            blas_t* u, int* us, blas_t* v, int* vs) nogil:
 *     """ here we will assume that the u = Q.T.dot(u) and not the bare u.
 */

static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_1_update(int __pyx_v_m, int __pyx_v_n, float *__pyx_v_q, int *__pyx_v_qs, float *__pyx_v_r, int *__pyx_v_rs, float *__pyx_v_u, int *__pyx_v_us, float *__pyx_v_v, int *__pyx_v_vs) {
  int __pyx_v_j;
  float __pyx_v_c;
  float __pyx_v_s;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "scipy/linalg/_decomp_update.pyx":922
 *     # triangular.
 * 
 *     for j in range(m-2, -1, -1):             # <<<<<<<<<<<<<<
 *         lartg(index1(u, us, j), index1(u, us, j+1), &c, &s)
 * 
 */
  for (__pyx_t_1 = (__pyx_v_m - 2); __pyx_t_1 > -1; __pyx_t_1-=1) {
    __pyx_v_j = __pyx_t_1;

    /* "scipy/linalg/_decomp_update.pyx":923
 * 
 *     for j in range(m-2, -1, -1):
 *         lartg(index1(u, us, j), index1(u, us, j+1), &c, &s)             # <<<<<<<<<<<<<<
 * 
 *         # update jth and (j+1)th rows of r.
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_u, __pyx_v_us, __pyx_v_j), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1)), (&__pyx_v_c), (&__pyx_v_s));

    /* "scipy/linalg/_decomp_update.pyx":926
 * 
 *         # update jth and (j+1)th rows of r.
 *         if n-j > 0:             # <<<<<<<<<<<<<<
 *             rot(n-j, index2(r, rs, j, j), rs[1], index2(r, rs, j+1, j), rs[1], c, s)
 * 
 */
    __pyx_t_2 = (((__pyx_v_n - __pyx_v_j) > 0) != 0);
    if (__pyx_t_2) {

      /* "scipy/linalg/_decomp_update.pyx":927
 *         # update jth and (j+1)th rows of r.
 *         if n-j > 0:
 *             rot(n-j, index2(r, rs, j, j), rs[1], index2(r, rs, j+1, j), rs[1], c, s)             # <<<<<<<<<<<<<<
 * 
 *         # update jth and (j+1)th cols of q.
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_n - __pyx_v_j), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[1]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

      /* "scipy/linalg/_decomp_update.pyx":926
 * 
 *         # update jth and (j+1)th rows of r.
 *         if n-j > 0:             # <<<<<<<<<<<<<<
 *             rot(n-j, index2(r, rs, j, j), rs[1], index2(r, rs, j+1, j), rs[1], c, s)
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":930
 * 
 *         # update jth and (j+1)th cols of q.
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     # add v to the first row
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_s));
  }

  /* "scipy/linalg/_decomp_update.pyx":933
 * 
 *     # add v to the first row
 *     blas_t_conj(n, v, vs)             # <<<<<<<<<<<<<<
 *     axpy(n, u[0],  v, vs[0], row(r, rs, 0), rs[1])
 * 
 */
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_n, __pyx_v_v, __pyx_v_vs);

  /* "scipy/linalg/_decomp_update.pyx":934
 *     # add v to the first row
 *     blas_t_conj(n, v, vs)
 *     axpy(n, u[0],  v, vs[0], row(r, rs, 0), rs[1])             # <<<<<<<<<<<<<<
 * 
 *     # return to q, r form
 */
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, (__pyx_v_u[0]), __pyx_v_v, (__pyx_v_vs[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, 0), (__pyx_v_rs[1]));

  /* "scipy/linalg/_decomp_update.pyx":937
 * 
 *     # return to q, r form
 *     hessenberg_qr(m, n, q, qs, r, rs, 0)             # <<<<<<<<<<<<<<
 *     # no return, return q, r from python driver.
 * 
 */
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_hessenberg_qr(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, 0);

  /* "scipy/linalg/_decomp_update.pyx":906
 *                               col(v, vs, j), vs, s, ss)
 * 
 * cdef void qr_rank_1_update(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                            blas_t* u, int* us, blas_t* v, int* vs) nogil:
 *     """ here we will assume that the u = Q.T.dot(u) and not the bare u.
 */

  /* function exit code */
}

static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_1_update(int __pyx_v_m, int __pyx_v_n, double *__pyx_v_q, int *__pyx_v_qs, double *__pyx_v_r, int *__pyx_v_rs, double *__pyx_v_u, int *__pyx_v_us, double *__pyx_v_v, int *__pyx_v_vs) {
  int __pyx_v_j;
  double __pyx_v_c;
  double __pyx_v_s;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "scipy/linalg/_decomp_update.pyx":922
 *     # triangular.
 * 
 *     for j in range(m-2, -1, -1):             # <<<<<<<<<<<<<<
 *         lartg(index1(u, us, j), index1(u, us, j+1), &c, &s)
 * 
 */
  for (__pyx_t_1 = (__pyx_v_m - 2); __pyx_t_1 > -1; __pyx_t_1-=1) {
    __pyx_v_j = __pyx_t_1;

    /* "scipy/linalg/_decomp_update.pyx":923
 * 
 *     for j in range(m-2, -1, -1):
 *         lartg(index1(u, us, j), index1(u, us, j+1), &c, &s)             # <<<<<<<<<<<<<<
 * 
 *         # update jth and (j+1)th rows of r.
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_u, __pyx_v_us, __pyx_v_j), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1)), (&__pyx_v_c), (&__pyx_v_s));

    /* "scipy/linalg/_decomp_update.pyx":926
 * 
 *         # update jth and (j+1)th rows of r.
 *         if n-j > 0:             # <<<<<<<<<<<<<<
 *             rot(n-j, index2(r, rs, j, j), rs[1], index2(r, rs, j+1, j), rs[1], c, s)
 * 
 */
    __pyx_t_2 = (((__pyx_v_n - __pyx_v_j) > 0) != 0);
    if (__pyx_t_2) {

      /* "scipy/linalg/_decomp_update.pyx":927
 *         # update jth and (j+1)th rows of r.
 *         if n-j > 0:
 *             rot(n-j, index2(r, rs, j, j), rs[1], index2(r, rs, j+1, j), rs[1], c, s)             # <<<<<<<<<<<<<<
 * 
 *         # update jth and (j+1)th cols of q.
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_n - __pyx_v_j), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[1]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

      /* "scipy/linalg/_decomp_update.pyx":926
 * 
 *         # update jth and (j+1)th rows of r.
 *         if n-j > 0:             # <<<<<<<<<<<<<<
 *             rot(n-j, index2(r, rs, j, j), rs[1], index2(r, rs, j+1, j), rs[1], c, s)
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":930
 * 
 *         # update jth and (j+1)th cols of q.
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     # add v to the first row
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_s));
  }

  /* "scipy/linalg/_decomp_update.pyx":933
 * 
 *     # add v to the first row
 *     blas_t_conj(n, v, vs)             # <<<<<<<<<<<<<<
 *     axpy(n, u[0],  v, vs[0], row(r, rs, 0), rs[1])
 * 
 */
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_n, __pyx_v_v, __pyx_v_vs);

  /* "scipy/linalg/_decomp_update.pyx":934
 *     # add v to the first row
 *     blas_t_conj(n, v, vs)
 *     axpy(n, u[0],  v, vs[0], row(r, rs, 0), rs[1])             # <<<<<<<<<<<<<<
 * 
 *     # return to q, r form
 */
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, (__pyx_v_u[0]), __pyx_v_v, (__pyx_v_vs[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, 0), (__pyx_v_rs[1]));

  /* "scipy/linalg/_decomp_update.pyx":937
 * 
 *     # return to q, r form
 *     hessenberg_qr(m, n, q, qs, r, rs, 0)             # <<<<<<<<<<<<<<
 *     # no return, return q, r from python driver.
 * 
 */
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_hessenberg_qr(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, 0);

  /* "scipy/linalg/_decomp_update.pyx":906
 *                               col(v, vs, j), vs, s, ss)
 * 
 * cdef void qr_rank_1_update(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                            blas_t* u, int* us, blas_t* v, int* vs) nogil:
 *     """ here we will assume that the u = Q.T.dot(u) and not the bare u.
 */

  /* function exit code */
}

static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_1_update(int __pyx_v_m, int __pyx_v_n, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_float_complex *__pyx_v_r, int *__pyx_v_rs, __pyx_t_float_complex *__pyx_v_u, int *__pyx_v_us, __pyx_t_float_complex *__pyx_v_v, int *__pyx_v_vs) {
  int __pyx_v_j;
  __pyx_t_float_complex __pyx_v_c;
  __pyx_t_float_complex __pyx_v_s;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "scipy/linalg/_decomp_update.pyx":922
 *     # triangular.
 * 
 *     for j in range(m-2, -1, -1):             # <<<<<<<<<<<<<<
 *         lartg(index1(u, us, j), index1(u, us, j+1), &c, &s)
 * 
 */
  for (__pyx_t_1 = (__pyx_v_m - 2); __pyx_t_1 > -1; __pyx_t_1-=1) {
    __pyx_v_j = __pyx_t_1;

    /* "scipy/linalg/_decomp_update.pyx":923
 * 
 *     for j in range(m-2, -1, -1):
 *         lartg(index1(u, us, j), index1(u, us, j+1), &c, &s)             # <<<<<<<<<<<<<<
 * 
 *         # update jth and (j+1)th rows of r.
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_u, __pyx_v_us, __pyx_v_j), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1)), (&__pyx_v_c), (&__pyx_v_s));

    /* "scipy/linalg/_decomp_update.pyx":926
 * 
 *         # update jth and (j+1)th rows of r.
 *         if n-j > 0:             # <<<<<<<<<<<<<<
 *             rot(n-j, index2(r, rs, j, j), rs[1], index2(r, rs, j+1, j), rs[1], c, s)
 * 
 */
    __pyx_t_2 = (((__pyx_v_n - __pyx_v_j) > 0) != 0);
    if (__pyx_t_2) {

      /* "scipy/linalg/_decomp_update.pyx":927
 *         # update jth and (j+1)th rows of r.
 *         if n-j > 0:
 *             rot(n-j, index2(r, rs, j, j), rs[1], index2(r, rs, j+1, j), rs[1], c, s)             # <<<<<<<<<<<<<<
 * 
 *         # update jth and (j+1)th cols of q.
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_n - __pyx_v_j), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[1]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

      /* "scipy/linalg/_decomp_update.pyx":926
 * 
 *         # update jth and (j+1)th rows of r.
 *         if n-j > 0:             # <<<<<<<<<<<<<<
 *             rot(n-j, index2(r, rs, j, j), rs[1], index2(r, rs, j+1, j), rs[1], c, s)
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":930
 * 
 *         # update jth and (j+1)th cols of q.
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     # add v to the first row
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_float(__pyx_v_s));
  }

  /* "scipy/linalg/_decomp_update.pyx":933
 * 
 *     # add v to the first row
 *     blas_t_conj(n, v, vs)             # <<<<<<<<<<<<<<
 *     axpy(n, u[0],  v, vs[0], row(r, rs, 0), rs[1])
 * 
 */
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_n, __pyx_v_v, __pyx_v_vs);

  /* "scipy/linalg/_decomp_update.pyx":934
 *     # add v to the first row
 *     blas_t_conj(n, v, vs)
 *     axpy(n, u[0],  v, vs[0], row(r, rs, 0), rs[1])             # <<<<<<<<<<<<<<
 * 
 *     # return to q, r form
 */
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, (__pyx_v_u[0]), __pyx_v_v, (__pyx_v_vs[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, 0), (__pyx_v_rs[1]));

  /* "scipy/linalg/_decomp_update.pyx":937
 * 
 *     # return to q, r form
 *     hessenberg_qr(m, n, q, qs, r, rs, 0)             # <<<<<<<<<<<<<<
 *     # no return, return q, r from python driver.
 * 
 */
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_hessenberg_qr(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, 0);

  /* "scipy/linalg/_decomp_update.pyx":906
 *                               col(v, vs, j), vs, s, ss)
 * 
 * cdef void qr_rank_1_update(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                            blas_t* u, int* us, blas_t* v, int* vs) nogil:
 *     """ here we will assume that the u = Q.T.dot(u) and not the bare u.
 */

  /* function exit code */
}

static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_1_update(int __pyx_v_m, int __pyx_v_n, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_double_complex *__pyx_v_r, int *__pyx_v_rs, __pyx_t_double_complex *__pyx_v_u, int *__pyx_v_us, __pyx_t_double_complex *__pyx_v_v, int *__pyx_v_vs) {
  int __pyx_v_j;
  __pyx_t_double_complex __pyx_v_c;
  __pyx_t_double_complex __pyx_v_s;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "scipy/linalg/_decomp_update.pyx":922
 *     # triangular.
 * 
 *     for j in range(m-2, -1, -1):             # <<<<<<<<<<<<<<
 *         lartg(index1(u, us, j), index1(u, us, j+1), &c, &s)
 * 
 */
  for (__pyx_t_1 = (__pyx_v_m - 2); __pyx_t_1 > -1; __pyx_t_1-=1) {
    __pyx_v_j = __pyx_t_1;

    /* "scipy/linalg/_decomp_update.pyx":923
 * 
 *     for j in range(m-2, -1, -1):
 *         lartg(index1(u, us, j), index1(u, us, j+1), &c, &s)             # <<<<<<<<<<<<<<
 * 
 *         # update jth and (j+1)th rows of r.
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_u, __pyx_v_us, __pyx_v_j), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index1(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1)), (&__pyx_v_c), (&__pyx_v_s));

    /* "scipy/linalg/_decomp_update.pyx":926
 * 
 *         # update jth and (j+1)th rows of r.
 *         if n-j > 0:             # <<<<<<<<<<<<<<
 *             rot(n-j, index2(r, rs, j, j), rs[1], index2(r, rs, j+1, j), rs[1], c, s)
 * 
 */
    __pyx_t_2 = (((__pyx_v_n - __pyx_v_j) > 0) != 0);
    if (__pyx_t_2) {

      /* "scipy/linalg/_decomp_update.pyx":927
 *         # update jth and (j+1)th rows of r.
 *         if n-j > 0:
 *             rot(n-j, index2(r, rs, j, j), rs[1], index2(r, rs, j+1, j), rs[1], c, s)             # <<<<<<<<<<<<<<
 * 
 *         # update jth and (j+1)th cols of q.
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot((__pyx_v_n - __pyx_v_j), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[1]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

      /* "scipy/linalg/_decomp_update.pyx":926
 * 
 *         # update jth and (j+1)th rows of r.
 *         if n-j > 0:             # <<<<<<<<<<<<<<
 *             rot(n-j, index2(r, rs, j, j), rs[1], index2(r, rs, j+1, j), rs[1], c, s)
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":930
 * 
 *         # update jth and (j+1)th cols of q.
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())             # <<<<<<<<<<<<<<
 * 
 *     # add v to the first row
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_double(__pyx_v_s));
  }

  /* "scipy/linalg/_decomp_update.pyx":933
 * 
 *     # add v to the first row
 *     blas_t_conj(n, v, vs)             # <<<<<<<<<<<<<<
 *     axpy(n, u[0],  v, vs[0], row(r, rs, 0), rs[1])
 * 
 */
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_n, __pyx_v_v, __pyx_v_vs);

  /* "scipy/linalg/_decomp_update.pyx":934
 *     # add v to the first row
 *     blas_t_conj(n, v, vs)
 *     axpy(n, u[0],  v, vs[0], row(r, rs, 0), rs[1])             # <<<<<<<<<<<<<<
 * 
 *     # return to q, r form
 */
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, (__pyx_v_u[0]), __pyx_v_v, (__pyx_v_vs[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, 0), (__pyx_v_rs[1]));

  /* "scipy/linalg/_decomp_update.pyx":937
 * 
 *     # return to q, r form
 *     hessenberg_qr(m, n, q, qs, r, rs, 0)             # <<<<<<<<<<<<<<
 *     # no return, return q, r from python driver.
 * 
 */
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_hessenberg_qr(__pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, 0);

  /* "scipy/linalg/_decomp_update.pyx":906
 *                               col(v, vs, j), vs, s, ss)
 * 
 * cdef void qr_rank_1_update(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                            blas_t* u, int* us, blas_t* v, int* vs) nogil:
 *     """ here we will assume that the u = Q.T.dot(u) and not the bare u.
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":940
 *     # no return, return q, r from python driver.
 * 
 * cdef int qr_rank_p_update(int m, int n, int p, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                         int* rs, blas_t* u, int* us, blas_t* v, int* vs) nogil:
 *     cdef int i, j
 */

static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_p_update(int __pyx_v_m, int __pyx_v_n, int __pyx_v_p, float *__pyx_v_q, int *__pyx_v_qs, float *__pyx_v_r, int *__pyx_v_rs, float *__pyx_v_u, int *__pyx_v_us, float *__pyx_v_v, int *__pyx_v_vs) {
  int __pyx_v_i;
  int __pyx_v_j;
  float __pyx_v_c;
  float __pyx_v_s;
  float *__pyx_v_tau;
  float *__pyx_v_work;
  int __pyx_v_info;
  int __pyx_v_lwork;
  char *__pyx_v_sideR;
  char *__pyx_v_sideL;
  char *__pyx_v_uplo;
  char *__pyx_v_trans;
  char *__pyx_v_diag;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;

  /* "scipy/linalg/_decomp_update.pyx":944
 *     cdef int i, j
 *     cdef blas_t c, s
 *     cdef blas_t* tau = NULL             # <<<<<<<<<<<<<<
 *     cdef blas_t* work = NULL
 *     cdef int info, lwork
 */
  __pyx_v_tau = NULL;

  /* "scipy/linalg/_decomp_update.pyx":945
 *     cdef blas_t c, s
 *     cdef blas_t* tau = NULL
 *     cdef blas_t* work = NULL             # <<<<<<<<<<<<<<
 *     cdef int info, lwork
 *     cdef char* sideR = 'R'
 */
  __pyx_v_work = NULL;

  /* "scipy/linalg/_decomp_update.pyx":947
 *     cdef blas_t* work = NULL
 *     cdef int info, lwork
 *     cdef char* sideR = 'R'             # <<<<<<<<<<<<<<
 *     cdef char* sideL = 'L'
 *     cdef char* uplo = 'U'
 */
  __pyx_v_sideR = ((char *)"R");

  /* "scipy/linalg/_decomp_update.pyx":948
 *     cdef int info, lwork
 *     cdef char* sideR = 'R'
 *     cdef char* sideL = 'L'             # <<<<<<<<<<<<<<
 *     cdef char* uplo = 'U'
 *     cdef char* trans = 'N'
 */
  __pyx_v_sideL = ((char *)"L");

  /* "scipy/linalg/_decomp_update.pyx":949
 *     cdef char* sideR = 'R'
 *     cdef char* sideL = 'L'
 *     cdef char* uplo = 'U'             # <<<<<<<<<<<<<<
 *     cdef char* trans = 'N'
 *     cdef char* diag = 'N'
 */
  __pyx_v_uplo = ((char *)"U");

  /* "scipy/linalg/_decomp_update.pyx":950
 *     cdef char* sideL = 'L'
 *     cdef char* uplo = 'U'
 *     cdef char* trans = 'N'             # <<<<<<<<<<<<<<
 *     cdef char* diag = 'N'
 * 
 */
  __pyx_v_trans = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":951
 *     cdef char* uplo = 'U'
 *     cdef char* trans = 'N'
 *     cdef char* diag = 'N'             # <<<<<<<<<<<<<<
 * 
 *     if m > n:
 */
  __pyx_v_diag = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":953
 *     cdef char* diag = 'N'
 * 
 *     if m > n:             # <<<<<<<<<<<<<<
 *         # query the workspace
 *         # below p_subdiag_qr will need workspace of size m, which is the
 */
  __pyx_t_1 = ((__pyx_v_m > __pyx_v_n) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":958
 *         # minimum, ormqr will also require.
 *         # set tau to point at something, to keep new MKL working.
 *         tau = &c             # <<<<<<<<<<<<<<
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, &c, -1)
 *         if info < 0:
 */
    __pyx_v_tau = (&__pyx_v_c);

    /* "scipy/linalg/_decomp_update.pyx":959
 *         # set tau to point at something, to keep new MKL working.
 *         tau = &c
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, &c, -1)             # <<<<<<<<<<<<<<
 *         if info < 0:
 *             return libc.stdlib.abs(info)
 */
    __pyx_v_info = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_geqrf((__pyx_v_m - __pyx_v_n), __pyx_v_p, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_n, 0), __pyx_v_m, __pyx_v_tau, (&__pyx_v_c), -1);

    /* "scipy/linalg/_decomp_update.pyx":960
 *         tau = &c
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, &c, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":961
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, &c, -1)
 *         if info < 0:
 *             return libc.stdlib.abs(info)             # <<<<<<<<<<<<<<
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 *                 index2(q, qs, 0, n), m, &s, -1)
 */
      __pyx_r = abs(__pyx_v_info);
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":960
 *         tau = &c
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, &c, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":962
 *         if info < 0:
 *             return libc.stdlib.abs(info)
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,             # <<<<<<<<<<<<<<
 *                 index2(q, qs, 0, n), m, &s, -1)
 *         if info < 0:
 */
    __pyx_v_info = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(__pyx_v_sideR, __pyx_v_trans, __pyx_v_m, (__pyx_v_m - __pyx_v_n), __pyx_v_p, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_n, 0), __pyx_v_m, __pyx_v_tau, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_n), __pyx_v_m, (&__pyx_v_s), -1);

    /* "scipy/linalg/_decomp_update.pyx":964
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 *                 index2(q, qs, 0, n), m, &s, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":965
 *                 index2(q, qs, 0, n), m, &s, -1)
 *         if info < 0:
 *             return info             # <<<<<<<<<<<<<<
 * 
 *         # we're only doing one allocation, so use the larger
 */
      __pyx_r = __pyx_v_info;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":964
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 *                 index2(q, qs, 0, n), m, &s, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":968
 * 
 *         # we're only doing one allocation, so use the larger
 *         lwork = to_lwork(c, s)             # <<<<<<<<<<<<<<
 * 
 *         # allocate the workspace + tau
 */
    __pyx_v_lwork = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_to_lwork(__pyx_v_c, __pyx_v_s);

    /* "scipy/linalg/_decomp_update.pyx":971
 * 
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n, p))*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *         if not work:
 *             return MEMORY_ERROR
 */
    __pyx_t_2 = __pyx_v_p;
    __pyx_t_3 = (__pyx_v_m - __pyx_v_n);
    if (((__pyx_t_2 < __pyx_t_3) != 0)) {
      __pyx_t_4 = __pyx_t_2;
    } else {
      __pyx_t_4 = __pyx_t_3;
    }
    __pyx_v_work = ((float *)malloc(((__pyx_v_lwork + __pyx_t_4) * (sizeof(float)))));

    /* "scipy/linalg/_decomp_update.pyx":972
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n, p))*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 *         tau = work + lwork
 */
    __pyx_t_1 = ((!(__pyx_v_work != 0)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":973
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n, p))*sizeof(blas_t))
 *         if not work:
 *             return MEMORY_ERROR             # <<<<<<<<<<<<<<
 *         tau = work + lwork
 * 
 */
      __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":972
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n, p))*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 *         tau = work + lwork
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":974
 *         if not work:
 *             return MEMORY_ERROR
 *         tau = work + lwork             # <<<<<<<<<<<<<<
 * 
 *         # qr
 */
    __pyx_v_tau = (__pyx_v_work + __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":977
 * 
 *         # qr
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, work, lwork)             # <<<<<<<<<<<<<<
 *         if info < 0:
 *             libc.stdlib.free(work)
 */
    __pyx_v_info = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_geqrf((__pyx_v_m - __pyx_v_n), __pyx_v_p, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_n, 0), __pyx_v_m, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":978
 *         # qr
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             libc.stdlib.free(work)
 *             return libc.stdlib.abs(info)
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":979
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, work, lwork)
 *         if info < 0:
 *             libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 * 
 */
      free(__pyx_v_work);

      /* "scipy/linalg/_decomp_update.pyx":980
 *         if info < 0:
 *             libc.stdlib.free(work)
 *             return libc.stdlib.abs(info)             # <<<<<<<<<<<<<<
 * 
 *         # apply the Q from this small qr to the last (m-n) columns of q.
 */
      __pyx_r = abs(__pyx_v_info);
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":978
 *         # qr
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             libc.stdlib.free(work)
 *             return libc.stdlib.abs(info)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":983
 * 
 *         # apply the Q from this small qr to the last (m-n) columns of q.
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,             # <<<<<<<<<<<<<<
 *                 index2(q, qs, 0, n), m, work, lwork)
 *         if info < 0:
 */
    __pyx_v_info = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(__pyx_v_sideR, __pyx_v_trans, __pyx_v_m, (__pyx_v_m - __pyx_v_n), __pyx_v_p, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_n, 0), __pyx_v_m, __pyx_v_tau, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_n), __pyx_v_m, __pyx_v_work, __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":985
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 *                 index2(q, qs, 0, n), m, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             libc.stdlib.free(work)
 *             return info
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":986
 *                 index2(q, qs, 0, n), m, work, lwork)
 *         if info < 0:
 *             libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
      free(__pyx_v_work);

      /* "scipy/linalg/_decomp_update.pyx":987
 *         if info < 0:
 *             libc.stdlib.free(work)
 *             return info             # <<<<<<<<<<<<<<
 * 
 *         # reduce u the rest of the way to upper triangular using givens.
 */
      __pyx_r = __pyx_v_info;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":985
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 *                 index2(q, qs, 0, n), m, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             libc.stdlib.free(work)
 *             return info
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":990
 * 
 *         # reduce u the rest of the way to upper triangular using givens.
 *         for i in range(p):             # <<<<<<<<<<<<<<
 *             for j in range(n+i-1, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 */
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_2 = __pyx_t_4;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "scipy/linalg/_decomp_update.pyx":991
 *         # reduce u the rest of the way to upper triangular using givens.
 *         for i in range(p):
 *             for j in range(n+i-1, i-1, -1):             # <<<<<<<<<<<<<<
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:
 */
      __pyx_t_5 = (__pyx_v_i - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = ((__pyx_v_n + __pyx_v_i) - 1); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
        __pyx_v_j = __pyx_t_7;

        /* "scipy/linalg/_decomp_update.pyx":992
 *         for i in range(p):
 *             for j in range(n+i-1, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)             # <<<<<<<<<<<<<<
 *                 if p-i-1:
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 */
        __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_j, __pyx_v_i), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1), __pyx_v_i), (&__pyx_v_c), (&__pyx_v_s));

        /* "scipy/linalg/_decomp_update.pyx":993
 *             for j in range(n+i-1, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:             # <<<<<<<<<<<<<<
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 */
        __pyx_t_1 = (((__pyx_v_p - __pyx_v_i) - 1) != 0);
        if (__pyx_t_1) {

          /* "scipy/linalg/_decomp_update.pyx":994
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],             # <<<<<<<<<<<<<<
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)
 */
          __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_p - __pyx_v_i) - 1), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_j, (__pyx_v_i + 1)), (__pyx_v_us[1]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1), (__pyx_v_i + 1)), (__pyx_v_us[1]), __pyx_v_c, __pyx_v_s);

          /* "scipy/linalg/_decomp_update.pyx":993
 *             for j in range(n+i-1, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:             # <<<<<<<<<<<<<<
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":996
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)             # <<<<<<<<<<<<<<
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],
 *                         c, s.conjugate())
 */
        __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_n, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, __pyx_v_j), (__pyx_v_rs[1]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

        /* "scipy/linalg/_decomp_update.pyx":997
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],             # <<<<<<<<<<<<<<
 *                         c, s.conjugate())
 * 
 */
        __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_s));
      }
    }

    /* "scipy/linalg/_decomp_update.pyx":953
 *     cdef char* diag = 'N'
 * 
 *     if m > n:             # <<<<<<<<<<<<<<
 *         # query the workspace
 *         # below p_subdiag_qr will need workspace of size m, which is the
 */
    goto __pyx_L3;
  }

  /* "scipy/linalg/_decomp_update.pyx":1002
 *     else: # m == n or m < n
 *         # reduce u to upper triangular using givens.
 *         for i in range(p):             # <<<<<<<<<<<<<<
 *             for j in range(m-2, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 */
  /*else*/ {
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_2 = __pyx_t_4;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "scipy/linalg/_decomp_update.pyx":1003
 *         # reduce u to upper triangular using givens.
 *         for i in range(p):
 *             for j in range(m-2, i-1, -1):             # <<<<<<<<<<<<<<
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:
 */
      __pyx_t_5 = (__pyx_v_i - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = (__pyx_v_m - 2); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
        __pyx_v_j = __pyx_t_7;

        /* "scipy/linalg/_decomp_update.pyx":1004
 *         for i in range(p):
 *             for j in range(m-2, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)             # <<<<<<<<<<<<<<
 *                 if p-i-1:
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 */
        __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_j, __pyx_v_i), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1), __pyx_v_i), (&__pyx_v_c), (&__pyx_v_s));

        /* "scipy/linalg/_decomp_update.pyx":1005
 *             for j in range(m-2, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:             # <<<<<<<<<<<<<<
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 */
        __pyx_t_1 = (((__pyx_v_p - __pyx_v_i) - 1) != 0);
        if (__pyx_t_1) {

          /* "scipy/linalg/_decomp_update.pyx":1006
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],             # <<<<<<<<<<<<<<
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)
 */
          __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_p - __pyx_v_i) - 1), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_j, (__pyx_v_i + 1)), (__pyx_v_us[1]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1), (__pyx_v_i + 1)), (__pyx_v_us[1]), __pyx_v_c, __pyx_v_s);

          /* "scipy/linalg/_decomp_update.pyx":1005
 *             for j in range(m-2, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:             # <<<<<<<<<<<<<<
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":1008
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)             # <<<<<<<<<<<<<<
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],
 *                         c, s.conjugate())
 */
        __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_n, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, __pyx_v_j), (__pyx_v_rs[1]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

        /* "scipy/linalg/_decomp_update.pyx":1009
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],             # <<<<<<<<<<<<<<
 *                         c, s.conjugate())
 * 
 */
        __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_s));
      }
    }

    /* "scipy/linalg/_decomp_update.pyx":1013
 * 
 *         # allocate workspace
 *         work = <blas_t*>libc.stdlib.malloc(n*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *         if not work:
 *             return MEMORY_ERROR
 */
    __pyx_v_work = ((float *)malloc((__pyx_v_n * (sizeof(float)))));

    /* "scipy/linalg/_decomp_update.pyx":1014
 *         # allocate workspace
 *         work = <blas_t*>libc.stdlib.malloc(n*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 * 
 */
    __pyx_t_1 = ((!(__pyx_v_work != 0)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":1015
 *         work = <blas_t*>libc.stdlib.malloc(n*sizeof(blas_t))
 *         if not work:
 *             return MEMORY_ERROR             # <<<<<<<<<<<<<<
 * 
 *     # now form UV**H and add it to R.
 */
      __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":1014
 *         # allocate workspace
 *         work = <blas_t*>libc.stdlib.malloc(n*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 * 
 */
    }
  }
  __pyx_L3:;

  /* "scipy/linalg/_decomp_update.pyx":1019
 *     # now form UV**H and add it to R.
 *     # This won't fill in any more of R than we have already.
 *     blas_t_2d_conj(p, n, v, vs)             # <<<<<<<<<<<<<<
 *     trmm(sideL, uplo, trans, diag, p, n, 1, u, m, v, p)
 * 
 */
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_2d_conj(__pyx_v_p, __pyx_v_n, __pyx_v_v, __pyx_v_vs);

  /* "scipy/linalg/_decomp_update.pyx":1020
 *     # This won't fill in any more of R than we have already.
 *     blas_t_2d_conj(p, n, v, vs)
 *     trmm(sideL, uplo, trans, diag, p, n, 1, u, m, v, p)             # <<<<<<<<<<<<<<
 * 
 *     # (should this be n, p length adds instead since these are fortran contig?)
 */
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_trmm(__pyx_v_sideL, __pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_p, __pyx_v_n, 1.0, __pyx_v_u, __pyx_v_m, __pyx_v_v, __pyx_v_p);

  /* "scipy/linalg/_decomp_update.pyx":1023
 * 
 *     # (should this be n, p length adds instead since these are fortran contig?)
 *     for j in range(p):             # <<<<<<<<<<<<<<
 *         axpy(n, 1, row(v, vs, j), vs[1], row(r, rs, j), rs[1])
 * 
 */
  __pyx_t_4 = __pyx_v_p;
  __pyx_t_2 = __pyx_t_4;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":1024
 *     # (should this be n, p length adds instead since these are fortran contig?)
 *     for j in range(p):
 *         axpy(n, 1, row(v, vs, j), vs[1], row(r, rs, j), rs[1])             # <<<<<<<<<<<<<<
 * 
 *     # now r has p subdiagonals, eliminate them with reflectors.
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, 1.0, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_v, __pyx_v_vs, __pyx_v_j), (__pyx_v_vs[1]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, __pyx_v_j), (__pyx_v_rs[1]));
  }

  /* "scipy/linalg/_decomp_update.pyx":1027
 * 
 *     # now r has p subdiagonals, eliminate them with reflectors.
 *     p_subdiag_qr(m, m, n, q, qs, r, rs, 0, p, work)             # <<<<<<<<<<<<<<
 * 
 *     libc.stdlib.free(work)
 */
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_p_subdiag_qr(__pyx_v_m, __pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, 0, __pyx_v_p, __pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":1029
 *     p_subdiag_qr(m, m, n, q, qs, r, rs, 0, p, work)
 * 
 *     libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  free(__pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":1030
 * 
 *     libc.stdlib.free(work)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef void hessenberg_qr(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":940
 *     # no return, return q, r from python driver.
 * 
 * cdef int qr_rank_p_update(int m, int n, int p, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                         int* rs, blas_t* u, int* us, blas_t* v, int* vs) nogil:
 *     cdef int i, j
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_p_update(int __pyx_v_m, int __pyx_v_n, int __pyx_v_p, double *__pyx_v_q, int *__pyx_v_qs, double *__pyx_v_r, int *__pyx_v_rs, double *__pyx_v_u, int *__pyx_v_us, double *__pyx_v_v, int *__pyx_v_vs) {
  int __pyx_v_i;
  int __pyx_v_j;
  double __pyx_v_c;
  double __pyx_v_s;
  double *__pyx_v_tau;
  double *__pyx_v_work;
  int __pyx_v_info;
  int __pyx_v_lwork;
  char *__pyx_v_sideR;
  char *__pyx_v_sideL;
  char *__pyx_v_uplo;
  char *__pyx_v_trans;
  char *__pyx_v_diag;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;

  /* "scipy/linalg/_decomp_update.pyx":944
 *     cdef int i, j
 *     cdef blas_t c, s
 *     cdef blas_t* tau = NULL             # <<<<<<<<<<<<<<
 *     cdef blas_t* work = NULL
 *     cdef int info, lwork
 */
  __pyx_v_tau = NULL;

  /* "scipy/linalg/_decomp_update.pyx":945
 *     cdef blas_t c, s
 *     cdef blas_t* tau = NULL
 *     cdef blas_t* work = NULL             # <<<<<<<<<<<<<<
 *     cdef int info, lwork
 *     cdef char* sideR = 'R'
 */
  __pyx_v_work = NULL;

  /* "scipy/linalg/_decomp_update.pyx":947
 *     cdef blas_t* work = NULL
 *     cdef int info, lwork
 *     cdef char* sideR = 'R'             # <<<<<<<<<<<<<<
 *     cdef char* sideL = 'L'
 *     cdef char* uplo = 'U'
 */
  __pyx_v_sideR = ((char *)"R");

  /* "scipy/linalg/_decomp_update.pyx":948
 *     cdef int info, lwork
 *     cdef char* sideR = 'R'
 *     cdef char* sideL = 'L'             # <<<<<<<<<<<<<<
 *     cdef char* uplo = 'U'
 *     cdef char* trans = 'N'
 */
  __pyx_v_sideL = ((char *)"L");

  /* "scipy/linalg/_decomp_update.pyx":949
 *     cdef char* sideR = 'R'
 *     cdef char* sideL = 'L'
 *     cdef char* uplo = 'U'             # <<<<<<<<<<<<<<
 *     cdef char* trans = 'N'
 *     cdef char* diag = 'N'
 */
  __pyx_v_uplo = ((char *)"U");

  /* "scipy/linalg/_decomp_update.pyx":950
 *     cdef char* sideL = 'L'
 *     cdef char* uplo = 'U'
 *     cdef char* trans = 'N'             # <<<<<<<<<<<<<<
 *     cdef char* diag = 'N'
 * 
 */
  __pyx_v_trans = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":951
 *     cdef char* uplo = 'U'
 *     cdef char* trans = 'N'
 *     cdef char* diag = 'N'             # <<<<<<<<<<<<<<
 * 
 *     if m > n:
 */
  __pyx_v_diag = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":953
 *     cdef char* diag = 'N'
 * 
 *     if m > n:             # <<<<<<<<<<<<<<
 *         # query the workspace
 *         # below p_subdiag_qr will need workspace of size m, which is the
 */
  __pyx_t_1 = ((__pyx_v_m > __pyx_v_n) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":958
 *         # minimum, ormqr will also require.
 *         # set tau to point at something, to keep new MKL working.
 *         tau = &c             # <<<<<<<<<<<<<<
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, &c, -1)
 *         if info < 0:
 */
    __pyx_v_tau = (&__pyx_v_c);

    /* "scipy/linalg/_decomp_update.pyx":959
 *         # set tau to point at something, to keep new MKL working.
 *         tau = &c
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, &c, -1)             # <<<<<<<<<<<<<<
 *         if info < 0:
 *             return libc.stdlib.abs(info)
 */
    __pyx_v_info = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_geqrf((__pyx_v_m - __pyx_v_n), __pyx_v_p, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_n, 0), __pyx_v_m, __pyx_v_tau, (&__pyx_v_c), -1);

    /* "scipy/linalg/_decomp_update.pyx":960
 *         tau = &c
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, &c, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":961
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, &c, -1)
 *         if info < 0:
 *             return libc.stdlib.abs(info)             # <<<<<<<<<<<<<<
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 *                 index2(q, qs, 0, n), m, &s, -1)
 */
      __pyx_r = abs(__pyx_v_info);
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":960
 *         tau = &c
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, &c, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":962
 *         if info < 0:
 *             return libc.stdlib.abs(info)
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,             # <<<<<<<<<<<<<<
 *                 index2(q, qs, 0, n), m, &s, -1)
 *         if info < 0:
 */
    __pyx_v_info = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(__pyx_v_sideR, __pyx_v_trans, __pyx_v_m, (__pyx_v_m - __pyx_v_n), __pyx_v_p, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_n, 0), __pyx_v_m, __pyx_v_tau, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_n), __pyx_v_m, (&__pyx_v_s), -1);

    /* "scipy/linalg/_decomp_update.pyx":964
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 *                 index2(q, qs, 0, n), m, &s, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":965
 *                 index2(q, qs, 0, n), m, &s, -1)
 *         if info < 0:
 *             return info             # <<<<<<<<<<<<<<
 * 
 *         # we're only doing one allocation, so use the larger
 */
      __pyx_r = __pyx_v_info;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":964
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 *                 index2(q, qs, 0, n), m, &s, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":968
 * 
 *         # we're only doing one allocation, so use the larger
 *         lwork = to_lwork(c, s)             # <<<<<<<<<<<<<<
 * 
 *         # allocate the workspace + tau
 */
    __pyx_v_lwork = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_to_lwork(__pyx_v_c, __pyx_v_s);

    /* "scipy/linalg/_decomp_update.pyx":971
 * 
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n, p))*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *         if not work:
 *             return MEMORY_ERROR
 */
    __pyx_t_2 = __pyx_v_p;
    __pyx_t_3 = (__pyx_v_m - __pyx_v_n);
    if (((__pyx_t_2 < __pyx_t_3) != 0)) {
      __pyx_t_4 = __pyx_t_2;
    } else {
      __pyx_t_4 = __pyx_t_3;
    }
    __pyx_v_work = ((double *)malloc(((__pyx_v_lwork + __pyx_t_4) * (sizeof(double)))));

    /* "scipy/linalg/_decomp_update.pyx":972
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n, p))*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 *         tau = work + lwork
 */
    __pyx_t_1 = ((!(__pyx_v_work != 0)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":973
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n, p))*sizeof(blas_t))
 *         if not work:
 *             return MEMORY_ERROR             # <<<<<<<<<<<<<<
 *         tau = work + lwork
 * 
 */
      __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":972
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n, p))*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 *         tau = work + lwork
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":974
 *         if not work:
 *             return MEMORY_ERROR
 *         tau = work + lwork             # <<<<<<<<<<<<<<
 * 
 *         # qr
 */
    __pyx_v_tau = (__pyx_v_work + __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":977
 * 
 *         # qr
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, work, lwork)             # <<<<<<<<<<<<<<
 *         if info < 0:
 *             libc.stdlib.free(work)
 */
    __pyx_v_info = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_geqrf((__pyx_v_m - __pyx_v_n), __pyx_v_p, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_n, 0), __pyx_v_m, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":978
 *         # qr
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             libc.stdlib.free(work)
 *             return libc.stdlib.abs(info)
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":979
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, work, lwork)
 *         if info < 0:
 *             libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 * 
 */
      free(__pyx_v_work);

      /* "scipy/linalg/_decomp_update.pyx":980
 *         if info < 0:
 *             libc.stdlib.free(work)
 *             return libc.stdlib.abs(info)             # <<<<<<<<<<<<<<
 * 
 *         # apply the Q from this small qr to the last (m-n) columns of q.
 */
      __pyx_r = abs(__pyx_v_info);
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":978
 *         # qr
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             libc.stdlib.free(work)
 *             return libc.stdlib.abs(info)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":983
 * 
 *         # apply the Q from this small qr to the last (m-n) columns of q.
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,             # <<<<<<<<<<<<<<
 *                 index2(q, qs, 0, n), m, work, lwork)
 *         if info < 0:
 */
    __pyx_v_info = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(__pyx_v_sideR, __pyx_v_trans, __pyx_v_m, (__pyx_v_m - __pyx_v_n), __pyx_v_p, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_n, 0), __pyx_v_m, __pyx_v_tau, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_n), __pyx_v_m, __pyx_v_work, __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":985
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 *                 index2(q, qs, 0, n), m, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             libc.stdlib.free(work)
 *             return info
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":986
 *                 index2(q, qs, 0, n), m, work, lwork)
 *         if info < 0:
 *             libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
      free(__pyx_v_work);

      /* "scipy/linalg/_decomp_update.pyx":987
 *         if info < 0:
 *             libc.stdlib.free(work)
 *             return info             # <<<<<<<<<<<<<<
 * 
 *         # reduce u the rest of the way to upper triangular using givens.
 */
      __pyx_r = __pyx_v_info;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":985
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 *                 index2(q, qs, 0, n), m, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             libc.stdlib.free(work)
 *             return info
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":990
 * 
 *         # reduce u the rest of the way to upper triangular using givens.
 *         for i in range(p):             # <<<<<<<<<<<<<<
 *             for j in range(n+i-1, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 */
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_2 = __pyx_t_4;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "scipy/linalg/_decomp_update.pyx":991
 *         # reduce u the rest of the way to upper triangular using givens.
 *         for i in range(p):
 *             for j in range(n+i-1, i-1, -1):             # <<<<<<<<<<<<<<
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:
 */
      __pyx_t_5 = (__pyx_v_i - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = ((__pyx_v_n + __pyx_v_i) - 1); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
        __pyx_v_j = __pyx_t_7;

        /* "scipy/linalg/_decomp_update.pyx":992
 *         for i in range(p):
 *             for j in range(n+i-1, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)             # <<<<<<<<<<<<<<
 *                 if p-i-1:
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 */
        __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_j, __pyx_v_i), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1), __pyx_v_i), (&__pyx_v_c), (&__pyx_v_s));

        /* "scipy/linalg/_decomp_update.pyx":993
 *             for j in range(n+i-1, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:             # <<<<<<<<<<<<<<
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 */
        __pyx_t_1 = (((__pyx_v_p - __pyx_v_i) - 1) != 0);
        if (__pyx_t_1) {

          /* "scipy/linalg/_decomp_update.pyx":994
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],             # <<<<<<<<<<<<<<
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)
 */
          __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_p - __pyx_v_i) - 1), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_j, (__pyx_v_i + 1)), (__pyx_v_us[1]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1), (__pyx_v_i + 1)), (__pyx_v_us[1]), __pyx_v_c, __pyx_v_s);

          /* "scipy/linalg/_decomp_update.pyx":993
 *             for j in range(n+i-1, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:             # <<<<<<<<<<<<<<
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":996
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)             # <<<<<<<<<<<<<<
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],
 *                         c, s.conjugate())
 */
        __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_n, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, __pyx_v_j), (__pyx_v_rs[1]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

        /* "scipy/linalg/_decomp_update.pyx":997
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],             # <<<<<<<<<<<<<<
 *                         c, s.conjugate())
 * 
 */
        __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_s));
      }
    }

    /* "scipy/linalg/_decomp_update.pyx":953
 *     cdef char* diag = 'N'
 * 
 *     if m > n:             # <<<<<<<<<<<<<<
 *         # query the workspace
 *         # below p_subdiag_qr will need workspace of size m, which is the
 */
    goto __pyx_L3;
  }

  /* "scipy/linalg/_decomp_update.pyx":1002
 *     else: # m == n or m < n
 *         # reduce u to upper triangular using givens.
 *         for i in range(p):             # <<<<<<<<<<<<<<
 *             for j in range(m-2, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 */
  /*else*/ {
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_2 = __pyx_t_4;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "scipy/linalg/_decomp_update.pyx":1003
 *         # reduce u to upper triangular using givens.
 *         for i in range(p):
 *             for j in range(m-2, i-1, -1):             # <<<<<<<<<<<<<<
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:
 */
      __pyx_t_5 = (__pyx_v_i - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = (__pyx_v_m - 2); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
        __pyx_v_j = __pyx_t_7;

        /* "scipy/linalg/_decomp_update.pyx":1004
 *         for i in range(p):
 *             for j in range(m-2, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)             # <<<<<<<<<<<<<<
 *                 if p-i-1:
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 */
        __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_j, __pyx_v_i), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1), __pyx_v_i), (&__pyx_v_c), (&__pyx_v_s));

        /* "scipy/linalg/_decomp_update.pyx":1005
 *             for j in range(m-2, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:             # <<<<<<<<<<<<<<
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 */
        __pyx_t_1 = (((__pyx_v_p - __pyx_v_i) - 1) != 0);
        if (__pyx_t_1) {

          /* "scipy/linalg/_decomp_update.pyx":1006
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],             # <<<<<<<<<<<<<<
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)
 */
          __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_p - __pyx_v_i) - 1), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_j, (__pyx_v_i + 1)), (__pyx_v_us[1]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1), (__pyx_v_i + 1)), (__pyx_v_us[1]), __pyx_v_c, __pyx_v_s);

          /* "scipy/linalg/_decomp_update.pyx":1005
 *             for j in range(m-2, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:             # <<<<<<<<<<<<<<
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":1008
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)             # <<<<<<<<<<<<<<
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],
 *                         c, s.conjugate())
 */
        __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_n, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, __pyx_v_j), (__pyx_v_rs[1]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

        /* "scipy/linalg/_decomp_update.pyx":1009
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],             # <<<<<<<<<<<<<<
 *                         c, s.conjugate())
 * 
 */
        __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_s));
      }
    }

    /* "scipy/linalg/_decomp_update.pyx":1013
 * 
 *         # allocate workspace
 *         work = <blas_t*>libc.stdlib.malloc(n*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *         if not work:
 *             return MEMORY_ERROR
 */
    __pyx_v_work = ((double *)malloc((__pyx_v_n * (sizeof(double)))));

    /* "scipy/linalg/_decomp_update.pyx":1014
 *         # allocate workspace
 *         work = <blas_t*>libc.stdlib.malloc(n*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 * 
 */
    __pyx_t_1 = ((!(__pyx_v_work != 0)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":1015
 *         work = <blas_t*>libc.stdlib.malloc(n*sizeof(blas_t))
 *         if not work:
 *             return MEMORY_ERROR             # <<<<<<<<<<<<<<
 * 
 *     # now form UV**H and add it to R.
 */
      __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":1014
 *         # allocate workspace
 *         work = <blas_t*>libc.stdlib.malloc(n*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 * 
 */
    }
  }
  __pyx_L3:;

  /* "scipy/linalg/_decomp_update.pyx":1019
 *     # now form UV**H and add it to R.
 *     # This won't fill in any more of R than we have already.
 *     blas_t_2d_conj(p, n, v, vs)             # <<<<<<<<<<<<<<
 *     trmm(sideL, uplo, trans, diag, p, n, 1, u, m, v, p)
 * 
 */
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_2d_conj(__pyx_v_p, __pyx_v_n, __pyx_v_v, __pyx_v_vs);

  /* "scipy/linalg/_decomp_update.pyx":1020
 *     # This won't fill in any more of R than we have already.
 *     blas_t_2d_conj(p, n, v, vs)
 *     trmm(sideL, uplo, trans, diag, p, n, 1, u, m, v, p)             # <<<<<<<<<<<<<<
 * 
 *     # (should this be n, p length adds instead since these are fortran contig?)
 */
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_trmm(__pyx_v_sideL, __pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_p, __pyx_v_n, 1.0, __pyx_v_u, __pyx_v_m, __pyx_v_v, __pyx_v_p);

  /* "scipy/linalg/_decomp_update.pyx":1023
 * 
 *     # (should this be n, p length adds instead since these are fortran contig?)
 *     for j in range(p):             # <<<<<<<<<<<<<<
 *         axpy(n, 1, row(v, vs, j), vs[1], row(r, rs, j), rs[1])
 * 
 */
  __pyx_t_4 = __pyx_v_p;
  __pyx_t_2 = __pyx_t_4;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":1024
 *     # (should this be n, p length adds instead since these are fortran contig?)
 *     for j in range(p):
 *         axpy(n, 1, row(v, vs, j), vs[1], row(r, rs, j), rs[1])             # <<<<<<<<<<<<<<
 * 
 *     # now r has p subdiagonals, eliminate them with reflectors.
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, 1.0, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_v, __pyx_v_vs, __pyx_v_j), (__pyx_v_vs[1]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, __pyx_v_j), (__pyx_v_rs[1]));
  }

  /* "scipy/linalg/_decomp_update.pyx":1027
 * 
 *     # now r has p subdiagonals, eliminate them with reflectors.
 *     p_subdiag_qr(m, m, n, q, qs, r, rs, 0, p, work)             # <<<<<<<<<<<<<<
 * 
 *     libc.stdlib.free(work)
 */
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_p_subdiag_qr(__pyx_v_m, __pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, 0, __pyx_v_p, __pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":1029
 *     p_subdiag_qr(m, m, n, q, qs, r, rs, 0, p, work)
 * 
 *     libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  free(__pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":1030
 * 
 *     libc.stdlib.free(work)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef void hessenberg_qr(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":940
 *     # no return, return q, r from python driver.
 * 
 * cdef int qr_rank_p_update(int m, int n, int p, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                         int* rs, blas_t* u, int* us, blas_t* v, int* vs) nogil:
 *     cdef int i, j
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_p_update(int __pyx_v_m, int __pyx_v_n, int __pyx_v_p, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_float_complex *__pyx_v_r, int *__pyx_v_rs, __pyx_t_float_complex *__pyx_v_u, int *__pyx_v_us, __pyx_t_float_complex *__pyx_v_v, int *__pyx_v_vs) {
  int __pyx_v_i;
  int __pyx_v_j;
  __pyx_t_float_complex __pyx_v_c;
  __pyx_t_float_complex __pyx_v_s;
  __pyx_t_float_complex *__pyx_v_tau;
  __pyx_t_float_complex *__pyx_v_work;
  int __pyx_v_info;
  int __pyx_v_lwork;
  char *__pyx_v_sideR;
  char *__pyx_v_sideL;
  char *__pyx_v_uplo;
  char *__pyx_v_trans;
  char *__pyx_v_diag;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;

  /* "scipy/linalg/_decomp_update.pyx":944
 *     cdef int i, j
 *     cdef blas_t c, s
 *     cdef blas_t* tau = NULL             # <<<<<<<<<<<<<<
 *     cdef blas_t* work = NULL
 *     cdef int info, lwork
 */
  __pyx_v_tau = NULL;

  /* "scipy/linalg/_decomp_update.pyx":945
 *     cdef blas_t c, s
 *     cdef blas_t* tau = NULL
 *     cdef blas_t* work = NULL             # <<<<<<<<<<<<<<
 *     cdef int info, lwork
 *     cdef char* sideR = 'R'
 */
  __pyx_v_work = NULL;

  /* "scipy/linalg/_decomp_update.pyx":947
 *     cdef blas_t* work = NULL
 *     cdef int info, lwork
 *     cdef char* sideR = 'R'             # <<<<<<<<<<<<<<
 *     cdef char* sideL = 'L'
 *     cdef char* uplo = 'U'
 */
  __pyx_v_sideR = ((char *)"R");

  /* "scipy/linalg/_decomp_update.pyx":948
 *     cdef int info, lwork
 *     cdef char* sideR = 'R'
 *     cdef char* sideL = 'L'             # <<<<<<<<<<<<<<
 *     cdef char* uplo = 'U'
 *     cdef char* trans = 'N'
 */
  __pyx_v_sideL = ((char *)"L");

  /* "scipy/linalg/_decomp_update.pyx":949
 *     cdef char* sideR = 'R'
 *     cdef char* sideL = 'L'
 *     cdef char* uplo = 'U'             # <<<<<<<<<<<<<<
 *     cdef char* trans = 'N'
 *     cdef char* diag = 'N'
 */
  __pyx_v_uplo = ((char *)"U");

  /* "scipy/linalg/_decomp_update.pyx":950
 *     cdef char* sideL = 'L'
 *     cdef char* uplo = 'U'
 *     cdef char* trans = 'N'             # <<<<<<<<<<<<<<
 *     cdef char* diag = 'N'
 * 
 */
  __pyx_v_trans = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":951
 *     cdef char* uplo = 'U'
 *     cdef char* trans = 'N'
 *     cdef char* diag = 'N'             # <<<<<<<<<<<<<<
 * 
 *     if m > n:
 */
  __pyx_v_diag = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":953
 *     cdef char* diag = 'N'
 * 
 *     if m > n:             # <<<<<<<<<<<<<<
 *         # query the workspace
 *         # below p_subdiag_qr will need workspace of size m, which is the
 */
  __pyx_t_1 = ((__pyx_v_m > __pyx_v_n) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":958
 *         # minimum, ormqr will also require.
 *         # set tau to point at something, to keep new MKL working.
 *         tau = &c             # <<<<<<<<<<<<<<
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, &c, -1)
 *         if info < 0:
 */
    __pyx_v_tau = (&__pyx_v_c);

    /* "scipy/linalg/_decomp_update.pyx":959
 *         # set tau to point at something, to keep new MKL working.
 *         tau = &c
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, &c, -1)             # <<<<<<<<<<<<<<
 *         if info < 0:
 *             return libc.stdlib.abs(info)
 */
    __pyx_v_info = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_geqrf((__pyx_v_m - __pyx_v_n), __pyx_v_p, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_n, 0), __pyx_v_m, __pyx_v_tau, (&__pyx_v_c), -1);

    /* "scipy/linalg/_decomp_update.pyx":960
 *         tau = &c
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, &c, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":961
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, &c, -1)
 *         if info < 0:
 *             return libc.stdlib.abs(info)             # <<<<<<<<<<<<<<
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 *                 index2(q, qs, 0, n), m, &s, -1)
 */
      __pyx_r = abs(__pyx_v_info);
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":960
 *         tau = &c
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, &c, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":962
 *         if info < 0:
 *             return libc.stdlib.abs(info)
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,             # <<<<<<<<<<<<<<
 *                 index2(q, qs, 0, n), m, &s, -1)
 *         if info < 0:
 */
    __pyx_v_info = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(__pyx_v_sideR, __pyx_v_trans, __pyx_v_m, (__pyx_v_m - __pyx_v_n), __pyx_v_p, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_n, 0), __pyx_v_m, __pyx_v_tau, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_n), __pyx_v_m, (&__pyx_v_s), -1);

    /* "scipy/linalg/_decomp_update.pyx":964
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 *                 index2(q, qs, 0, n), m, &s, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":965
 *                 index2(q, qs, 0, n), m, &s, -1)
 *         if info < 0:
 *             return info             # <<<<<<<<<<<<<<
 * 
 *         # we're only doing one allocation, so use the larger
 */
      __pyx_r = __pyx_v_info;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":964
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 *                 index2(q, qs, 0, n), m, &s, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":968
 * 
 *         # we're only doing one allocation, so use the larger
 *         lwork = to_lwork(c, s)             # <<<<<<<<<<<<<<
 * 
 *         # allocate the workspace + tau
 */
    __pyx_v_lwork = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_to_lwork(__pyx_v_c, __pyx_v_s);

    /* "scipy/linalg/_decomp_update.pyx":971
 * 
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n, p))*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *         if not work:
 *             return MEMORY_ERROR
 */
    __pyx_t_2 = __pyx_v_p;
    __pyx_t_3 = (__pyx_v_m - __pyx_v_n);
    if (((__pyx_t_2 < __pyx_t_3) != 0)) {
      __pyx_t_4 = __pyx_t_2;
    } else {
      __pyx_t_4 = __pyx_t_3;
    }
    __pyx_v_work = ((__pyx_t_float_complex *)malloc(((__pyx_v_lwork + __pyx_t_4) * (sizeof(__pyx_t_float_complex)))));

    /* "scipy/linalg/_decomp_update.pyx":972
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n, p))*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 *         tau = work + lwork
 */
    __pyx_t_1 = ((!(__pyx_v_work != 0)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":973
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n, p))*sizeof(blas_t))
 *         if not work:
 *             return MEMORY_ERROR             # <<<<<<<<<<<<<<
 *         tau = work + lwork
 * 
 */
      __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":972
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n, p))*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 *         tau = work + lwork
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":974
 *         if not work:
 *             return MEMORY_ERROR
 *         tau = work + lwork             # <<<<<<<<<<<<<<
 * 
 *         # qr
 */
    __pyx_v_tau = (__pyx_v_work + __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":977
 * 
 *         # qr
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, work, lwork)             # <<<<<<<<<<<<<<
 *         if info < 0:
 *             libc.stdlib.free(work)
 */
    __pyx_v_info = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_geqrf((__pyx_v_m - __pyx_v_n), __pyx_v_p, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_n, 0), __pyx_v_m, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":978
 *         # qr
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             libc.stdlib.free(work)
 *             return libc.stdlib.abs(info)
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":979
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, work, lwork)
 *         if info < 0:
 *             libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 * 
 */
      free(__pyx_v_work);

      /* "scipy/linalg/_decomp_update.pyx":980
 *         if info < 0:
 *             libc.stdlib.free(work)
 *             return libc.stdlib.abs(info)             # <<<<<<<<<<<<<<
 * 
 *         # apply the Q from this small qr to the last (m-n) columns of q.
 */
      __pyx_r = abs(__pyx_v_info);
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":978
 *         # qr
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             libc.stdlib.free(work)
 *             return libc.stdlib.abs(info)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":983
 * 
 *         # apply the Q from this small qr to the last (m-n) columns of q.
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,             # <<<<<<<<<<<<<<
 *                 index2(q, qs, 0, n), m, work, lwork)
 *         if info < 0:
 */
    __pyx_v_info = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(__pyx_v_sideR, __pyx_v_trans, __pyx_v_m, (__pyx_v_m - __pyx_v_n), __pyx_v_p, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_n, 0), __pyx_v_m, __pyx_v_tau, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_n), __pyx_v_m, __pyx_v_work, __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":985
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 *                 index2(q, qs, 0, n), m, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             libc.stdlib.free(work)
 *             return info
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":986
 *                 index2(q, qs, 0, n), m, work, lwork)
 *         if info < 0:
 *             libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
      free(__pyx_v_work);

      /* "scipy/linalg/_decomp_update.pyx":987
 *         if info < 0:
 *             libc.stdlib.free(work)
 *             return info             # <<<<<<<<<<<<<<
 * 
 *         # reduce u the rest of the way to upper triangular using givens.
 */
      __pyx_r = __pyx_v_info;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":985
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 *                 index2(q, qs, 0, n), m, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             libc.stdlib.free(work)
 *             return info
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":990
 * 
 *         # reduce u the rest of the way to upper triangular using givens.
 *         for i in range(p):             # <<<<<<<<<<<<<<
 *             for j in range(n+i-1, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 */
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_2 = __pyx_t_4;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "scipy/linalg/_decomp_update.pyx":991
 *         # reduce u the rest of the way to upper triangular using givens.
 *         for i in range(p):
 *             for j in range(n+i-1, i-1, -1):             # <<<<<<<<<<<<<<
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:
 */
      __pyx_t_5 = (__pyx_v_i - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = ((__pyx_v_n + __pyx_v_i) - 1); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
        __pyx_v_j = __pyx_t_7;

        /* "scipy/linalg/_decomp_update.pyx":992
 *         for i in range(p):
 *             for j in range(n+i-1, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)             # <<<<<<<<<<<<<<
 *                 if p-i-1:
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_j, __pyx_v_i), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1), __pyx_v_i), (&__pyx_v_c), (&__pyx_v_s));

        /* "scipy/linalg/_decomp_update.pyx":993
 *             for j in range(n+i-1, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:             # <<<<<<<<<<<<<<
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 */
        __pyx_t_1 = (((__pyx_v_p - __pyx_v_i) - 1) != 0);
        if (__pyx_t_1) {

          /* "scipy/linalg/_decomp_update.pyx":994
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],             # <<<<<<<<<<<<<<
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)
 */
          __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_p - __pyx_v_i) - 1), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_j, (__pyx_v_i + 1)), (__pyx_v_us[1]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1), (__pyx_v_i + 1)), (__pyx_v_us[1]), __pyx_v_c, __pyx_v_s);

          /* "scipy/linalg/_decomp_update.pyx":993
 *             for j in range(n+i-1, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:             # <<<<<<<<<<<<<<
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":996
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)             # <<<<<<<<<<<<<<
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],
 *                         c, s.conjugate())
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_n, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, __pyx_v_j), (__pyx_v_rs[1]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

        /* "scipy/linalg/_decomp_update.pyx":997
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],             # <<<<<<<<<<<<<<
 *                         c, s.conjugate())
 * 
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_float(__pyx_v_s));
      }
    }

    /* "scipy/linalg/_decomp_update.pyx":953
 *     cdef char* diag = 'N'
 * 
 *     if m > n:             # <<<<<<<<<<<<<<
 *         # query the workspace
 *         # below p_subdiag_qr will need workspace of size m, which is the
 */
    goto __pyx_L3;
  }

  /* "scipy/linalg/_decomp_update.pyx":1002
 *     else: # m == n or m < n
 *         # reduce u to upper triangular using givens.
 *         for i in range(p):             # <<<<<<<<<<<<<<
 *             for j in range(m-2, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 */
  /*else*/ {
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_2 = __pyx_t_4;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "scipy/linalg/_decomp_update.pyx":1003
 *         # reduce u to upper triangular using givens.
 *         for i in range(p):
 *             for j in range(m-2, i-1, -1):             # <<<<<<<<<<<<<<
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:
 */
      __pyx_t_5 = (__pyx_v_i - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = (__pyx_v_m - 2); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
        __pyx_v_j = __pyx_t_7;

        /* "scipy/linalg/_decomp_update.pyx":1004
 *         for i in range(p):
 *             for j in range(m-2, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)             # <<<<<<<<<<<<<<
 *                 if p-i-1:
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_j, __pyx_v_i), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1), __pyx_v_i), (&__pyx_v_c), (&__pyx_v_s));

        /* "scipy/linalg/_decomp_update.pyx":1005
 *             for j in range(m-2, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:             # <<<<<<<<<<<<<<
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 */
        __pyx_t_1 = (((__pyx_v_p - __pyx_v_i) - 1) != 0);
        if (__pyx_t_1) {

          /* "scipy/linalg/_decomp_update.pyx":1006
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],             # <<<<<<<<<<<<<<
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)
 */
          __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_p - __pyx_v_i) - 1), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_j, (__pyx_v_i + 1)), (__pyx_v_us[1]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1), (__pyx_v_i + 1)), (__pyx_v_us[1]), __pyx_v_c, __pyx_v_s);

          /* "scipy/linalg/_decomp_update.pyx":1005
 *             for j in range(m-2, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:             # <<<<<<<<<<<<<<
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":1008
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)             # <<<<<<<<<<<<<<
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],
 *                         c, s.conjugate())
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_n, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, __pyx_v_j), (__pyx_v_rs[1]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

        /* "scipy/linalg/_decomp_update.pyx":1009
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],             # <<<<<<<<<<<<<<
 *                         c, s.conjugate())
 * 
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_float(__pyx_v_s));
      }
    }

    /* "scipy/linalg/_decomp_update.pyx":1013
 * 
 *         # allocate workspace
 *         work = <blas_t*>libc.stdlib.malloc(n*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *         if not work:
 *             return MEMORY_ERROR
 */
    __pyx_v_work = ((__pyx_t_float_complex *)malloc((__pyx_v_n * (sizeof(__pyx_t_float_complex)))));

    /* "scipy/linalg/_decomp_update.pyx":1014
 *         # allocate workspace
 *         work = <blas_t*>libc.stdlib.malloc(n*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 * 
 */
    __pyx_t_1 = ((!(__pyx_v_work != 0)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":1015
 *         work = <blas_t*>libc.stdlib.malloc(n*sizeof(blas_t))
 *         if not work:
 *             return MEMORY_ERROR             # <<<<<<<<<<<<<<
 * 
 *     # now form UV**H and add it to R.
 */
      __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":1014
 *         # allocate workspace
 *         work = <blas_t*>libc.stdlib.malloc(n*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 * 
 */
    }
  }
  __pyx_L3:;

  /* "scipy/linalg/_decomp_update.pyx":1019
 *     # now form UV**H and add it to R.
 *     # This won't fill in any more of R than we have already.
 *     blas_t_2d_conj(p, n, v, vs)             # <<<<<<<<<<<<<<
 *     trmm(sideL, uplo, trans, diag, p, n, 1, u, m, v, p)
 * 
 */
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_2d_conj(__pyx_v_p, __pyx_v_n, __pyx_v_v, __pyx_v_vs);

  /* "scipy/linalg/_decomp_update.pyx":1020
 *     # This won't fill in any more of R than we have already.
 *     blas_t_2d_conj(p, n, v, vs)
 *     trmm(sideL, uplo, trans, diag, p, n, 1, u, m, v, p)             # <<<<<<<<<<<<<<
 * 
 *     # (should this be n, p length adds instead since these are fortran contig?)
 */
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_trmm(__pyx_v_sideL, __pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_p, __pyx_v_n, __pyx_t_float_complex_from_parts(1, 0), __pyx_v_u, __pyx_v_m, __pyx_v_v, __pyx_v_p);

  /* "scipy/linalg/_decomp_update.pyx":1023
 * 
 *     # (should this be n, p length adds instead since these are fortran contig?)
 *     for j in range(p):             # <<<<<<<<<<<<<<
 *         axpy(n, 1, row(v, vs, j), vs[1], row(r, rs, j), rs[1])
 * 
 */
  __pyx_t_4 = __pyx_v_p;
  __pyx_t_2 = __pyx_t_4;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":1024
 *     # (should this be n, p length adds instead since these are fortran contig?)
 *     for j in range(p):
 *         axpy(n, 1, row(v, vs, j), vs[1], row(r, rs, j), rs[1])             # <<<<<<<<<<<<<<
 * 
 *     # now r has p subdiagonals, eliminate them with reflectors.
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, __pyx_t_float_complex_from_parts(1, 0), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_v, __pyx_v_vs, __pyx_v_j), (__pyx_v_vs[1]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, __pyx_v_j), (__pyx_v_rs[1]));
  }

  /* "scipy/linalg/_decomp_update.pyx":1027
 * 
 *     # now r has p subdiagonals, eliminate them with reflectors.
 *     p_subdiag_qr(m, m, n, q, qs, r, rs, 0, p, work)             # <<<<<<<<<<<<<<
 * 
 *     libc.stdlib.free(work)
 */
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_p_subdiag_qr(__pyx_v_m, __pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, 0, __pyx_v_p, __pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":1029
 *     p_subdiag_qr(m, m, n, q, qs, r, rs, 0, p, work)
 * 
 *     libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  free(__pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":1030
 * 
 *     libc.stdlib.free(work)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef void hessenberg_qr(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":940
 *     # no return, return q, r from python driver.
 * 
 * cdef int qr_rank_p_update(int m, int n, int p, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                         int* rs, blas_t* u, int* us, blas_t* v, int* vs) nogil:
 *     cdef int i, j
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_p_update(int __pyx_v_m, int __pyx_v_n, int __pyx_v_p, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_double_complex *__pyx_v_r, int *__pyx_v_rs, __pyx_t_double_complex *__pyx_v_u, int *__pyx_v_us, __pyx_t_double_complex *__pyx_v_v, int *__pyx_v_vs) {
  int __pyx_v_i;
  int __pyx_v_j;
  __pyx_t_double_complex __pyx_v_c;
  __pyx_t_double_complex __pyx_v_s;
  __pyx_t_double_complex *__pyx_v_tau;
  __pyx_t_double_complex *__pyx_v_work;
  int __pyx_v_info;
  int __pyx_v_lwork;
  char *__pyx_v_sideR;
  char *__pyx_v_sideL;
  char *__pyx_v_uplo;
  char *__pyx_v_trans;
  char *__pyx_v_diag;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;

  /* "scipy/linalg/_decomp_update.pyx":944
 *     cdef int i, j
 *     cdef blas_t c, s
 *     cdef blas_t* tau = NULL             # <<<<<<<<<<<<<<
 *     cdef blas_t* work = NULL
 *     cdef int info, lwork
 */
  __pyx_v_tau = NULL;

  /* "scipy/linalg/_decomp_update.pyx":945
 *     cdef blas_t c, s
 *     cdef blas_t* tau = NULL
 *     cdef blas_t* work = NULL             # <<<<<<<<<<<<<<
 *     cdef int info, lwork
 *     cdef char* sideR = 'R'
 */
  __pyx_v_work = NULL;

  /* "scipy/linalg/_decomp_update.pyx":947
 *     cdef blas_t* work = NULL
 *     cdef int info, lwork
 *     cdef char* sideR = 'R'             # <<<<<<<<<<<<<<
 *     cdef char* sideL = 'L'
 *     cdef char* uplo = 'U'
 */
  __pyx_v_sideR = ((char *)"R");

  /* "scipy/linalg/_decomp_update.pyx":948
 *     cdef int info, lwork
 *     cdef char* sideR = 'R'
 *     cdef char* sideL = 'L'             # <<<<<<<<<<<<<<
 *     cdef char* uplo = 'U'
 *     cdef char* trans = 'N'
 */
  __pyx_v_sideL = ((char *)"L");

  /* "scipy/linalg/_decomp_update.pyx":949
 *     cdef char* sideR = 'R'
 *     cdef char* sideL = 'L'
 *     cdef char* uplo = 'U'             # <<<<<<<<<<<<<<
 *     cdef char* trans = 'N'
 *     cdef char* diag = 'N'
 */
  __pyx_v_uplo = ((char *)"U");

  /* "scipy/linalg/_decomp_update.pyx":950
 *     cdef char* sideL = 'L'
 *     cdef char* uplo = 'U'
 *     cdef char* trans = 'N'             # <<<<<<<<<<<<<<
 *     cdef char* diag = 'N'
 * 
 */
  __pyx_v_trans = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":951
 *     cdef char* uplo = 'U'
 *     cdef char* trans = 'N'
 *     cdef char* diag = 'N'             # <<<<<<<<<<<<<<
 * 
 *     if m > n:
 */
  __pyx_v_diag = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":953
 *     cdef char* diag = 'N'
 * 
 *     if m > n:             # <<<<<<<<<<<<<<
 *         # query the workspace
 *         # below p_subdiag_qr will need workspace of size m, which is the
 */
  __pyx_t_1 = ((__pyx_v_m > __pyx_v_n) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":958
 *         # minimum, ormqr will also require.
 *         # set tau to point at something, to keep new MKL working.
 *         tau = &c             # <<<<<<<<<<<<<<
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, &c, -1)
 *         if info < 0:
 */
    __pyx_v_tau = (&__pyx_v_c);

    /* "scipy/linalg/_decomp_update.pyx":959
 *         # set tau to point at something, to keep new MKL working.
 *         tau = &c
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, &c, -1)             # <<<<<<<<<<<<<<
 *         if info < 0:
 *             return libc.stdlib.abs(info)
 */
    __pyx_v_info = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_geqrf((__pyx_v_m - __pyx_v_n), __pyx_v_p, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_n, 0), __pyx_v_m, __pyx_v_tau, (&__pyx_v_c), -1);

    /* "scipy/linalg/_decomp_update.pyx":960
 *         tau = &c
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, &c, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":961
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, &c, -1)
 *         if info < 0:
 *             return libc.stdlib.abs(info)             # <<<<<<<<<<<<<<
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 *                 index2(q, qs, 0, n), m, &s, -1)
 */
      __pyx_r = abs(__pyx_v_info);
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":960
 *         tau = &c
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, &c, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":962
 *         if info < 0:
 *             return libc.stdlib.abs(info)
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,             # <<<<<<<<<<<<<<
 *                 index2(q, qs, 0, n), m, &s, -1)
 *         if info < 0:
 */
    __pyx_v_info = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(__pyx_v_sideR, __pyx_v_trans, __pyx_v_m, (__pyx_v_m - __pyx_v_n), __pyx_v_p, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_n, 0), __pyx_v_m, __pyx_v_tau, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_n), __pyx_v_m, (&__pyx_v_s), -1);

    /* "scipy/linalg/_decomp_update.pyx":964
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 *                 index2(q, qs, 0, n), m, &s, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":965
 *                 index2(q, qs, 0, n), m, &s, -1)
 *         if info < 0:
 *             return info             # <<<<<<<<<<<<<<
 * 
 *         # we're only doing one allocation, so use the larger
 */
      __pyx_r = __pyx_v_info;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":964
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 *                 index2(q, qs, 0, n), m, &s, -1)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":968
 * 
 *         # we're only doing one allocation, so use the larger
 *         lwork = to_lwork(c, s)             # <<<<<<<<<<<<<<
 * 
 *         # allocate the workspace + tau
 */
    __pyx_v_lwork = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_to_lwork(__pyx_v_c, __pyx_v_s);

    /* "scipy/linalg/_decomp_update.pyx":971
 * 
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n, p))*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *         if not work:
 *             return MEMORY_ERROR
 */
    __pyx_t_2 = __pyx_v_p;
    __pyx_t_3 = (__pyx_v_m - __pyx_v_n);
    if (((__pyx_t_2 < __pyx_t_3) != 0)) {
      __pyx_t_4 = __pyx_t_2;
    } else {
      __pyx_t_4 = __pyx_t_3;
    }
    __pyx_v_work = ((__pyx_t_double_complex *)malloc(((__pyx_v_lwork + __pyx_t_4) * (sizeof(__pyx_t_double_complex)))));

    /* "scipy/linalg/_decomp_update.pyx":972
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n, p))*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 *         tau = work + lwork
 */
    __pyx_t_1 = ((!(__pyx_v_work != 0)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":973
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n, p))*sizeof(blas_t))
 *         if not work:
 *             return MEMORY_ERROR             # <<<<<<<<<<<<<<
 *         tau = work + lwork
 * 
 */
      __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":972
 *         # allocate the workspace + tau
 *         work = <blas_t*>libc.stdlib.malloc((lwork+min(m-n, p))*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 *         tau = work + lwork
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":974
 *         if not work:
 *             return MEMORY_ERROR
 *         tau = work + lwork             # <<<<<<<<<<<<<<
 * 
 *         # qr
 */
    __pyx_v_tau = (__pyx_v_work + __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":977
 * 
 *         # qr
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, work, lwork)             # <<<<<<<<<<<<<<
 *         if info < 0:
 *             libc.stdlib.free(work)
 */
    __pyx_v_info = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_geqrf((__pyx_v_m - __pyx_v_n), __pyx_v_p, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_n, 0), __pyx_v_m, __pyx_v_tau, __pyx_v_work, __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":978
 *         # qr
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             libc.stdlib.free(work)
 *             return libc.stdlib.abs(info)
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":979
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, work, lwork)
 *         if info < 0:
 *             libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 *             return libc.stdlib.abs(info)
 * 
 */
      free(__pyx_v_work);

      /* "scipy/linalg/_decomp_update.pyx":980
 *         if info < 0:
 *             libc.stdlib.free(work)
 *             return libc.stdlib.abs(info)             # <<<<<<<<<<<<<<
 * 
 *         # apply the Q from this small qr to the last (m-n) columns of q.
 */
      __pyx_r = abs(__pyx_v_info);
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":978
 *         # qr
 *         info = geqrf(m-n, p, index2(u, us, n, 0), m, tau, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             libc.stdlib.free(work)
 *             return libc.stdlib.abs(info)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":983
 * 
 *         # apply the Q from this small qr to the last (m-n) columns of q.
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,             # <<<<<<<<<<<<<<
 *                 index2(q, qs, 0, n), m, work, lwork)
 *         if info < 0:
 */
    __pyx_v_info = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_ormqr(__pyx_v_sideR, __pyx_v_trans, __pyx_v_m, (__pyx_v_m - __pyx_v_n), __pyx_v_p, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_n, 0), __pyx_v_m, __pyx_v_tau, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_n), __pyx_v_m, __pyx_v_work, __pyx_v_lwork);

    /* "scipy/linalg/_decomp_update.pyx":985
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 *                 index2(q, qs, 0, n), m, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             libc.stdlib.free(work)
 *             return info
 */
    __pyx_t_1 = ((__pyx_v_info < 0) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":986
 *                 index2(q, qs, 0, n), m, work, lwork)
 *         if info < 0:
 *             libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 *             return info
 * 
 */
      free(__pyx_v_work);

      /* "scipy/linalg/_decomp_update.pyx":987
 *         if info < 0:
 *             libc.stdlib.free(work)
 *             return info             # <<<<<<<<<<<<<<
 * 
 *         # reduce u the rest of the way to upper triangular using givens.
 */
      __pyx_r = __pyx_v_info;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":985
 *         info = ormqr(sideR, trans, m, m-n, p, index2(u, us, n, 0), m, tau,
 *                 index2(q, qs, 0, n), m, work, lwork)
 *         if info < 0:             # <<<<<<<<<<<<<<
 *             libc.stdlib.free(work)
 *             return info
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":990
 * 
 *         # reduce u the rest of the way to upper triangular using givens.
 *         for i in range(p):             # <<<<<<<<<<<<<<
 *             for j in range(n+i-1, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 */
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_2 = __pyx_t_4;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "scipy/linalg/_decomp_update.pyx":991
 *         # reduce u the rest of the way to upper triangular using givens.
 *         for i in range(p):
 *             for j in range(n+i-1, i-1, -1):             # <<<<<<<<<<<<<<
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:
 */
      __pyx_t_5 = (__pyx_v_i - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = ((__pyx_v_n + __pyx_v_i) - 1); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
        __pyx_v_j = __pyx_t_7;

        /* "scipy/linalg/_decomp_update.pyx":992
 *         for i in range(p):
 *             for j in range(n+i-1, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)             # <<<<<<<<<<<<<<
 *                 if p-i-1:
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_j, __pyx_v_i), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1), __pyx_v_i), (&__pyx_v_c), (&__pyx_v_s));

        /* "scipy/linalg/_decomp_update.pyx":993
 *             for j in range(n+i-1, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:             # <<<<<<<<<<<<<<
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 */
        __pyx_t_1 = (((__pyx_v_p - __pyx_v_i) - 1) != 0);
        if (__pyx_t_1) {

          /* "scipy/linalg/_decomp_update.pyx":994
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],             # <<<<<<<<<<<<<<
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)
 */
          __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_p - __pyx_v_i) - 1), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_j, (__pyx_v_i + 1)), (__pyx_v_us[1]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1), (__pyx_v_i + 1)), (__pyx_v_us[1]), __pyx_v_c, __pyx_v_s);

          /* "scipy/linalg/_decomp_update.pyx":993
 *             for j in range(n+i-1, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:             # <<<<<<<<<<<<<<
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":996
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)             # <<<<<<<<<<<<<<
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],
 *                         c, s.conjugate())
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_n, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, __pyx_v_j), (__pyx_v_rs[1]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

        /* "scipy/linalg/_decomp_update.pyx":997
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],             # <<<<<<<<<<<<<<
 *                         c, s.conjugate())
 * 
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_double(__pyx_v_s));
      }
    }

    /* "scipy/linalg/_decomp_update.pyx":953
 *     cdef char* diag = 'N'
 * 
 *     if m > n:             # <<<<<<<<<<<<<<
 *         # query the workspace
 *         # below p_subdiag_qr will need workspace of size m, which is the
 */
    goto __pyx_L3;
  }

  /* "scipy/linalg/_decomp_update.pyx":1002
 *     else: # m == n or m < n
 *         # reduce u to upper triangular using givens.
 *         for i in range(p):             # <<<<<<<<<<<<<<
 *             for j in range(m-2, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 */
  /*else*/ {
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_2 = __pyx_t_4;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "scipy/linalg/_decomp_update.pyx":1003
 *         # reduce u to upper triangular using givens.
 *         for i in range(p):
 *             for j in range(m-2, i-1, -1):             # <<<<<<<<<<<<<<
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:
 */
      __pyx_t_5 = (__pyx_v_i - 1);
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = (__pyx_v_m - 2); __pyx_t_7 > __pyx_t_6; __pyx_t_7-=1) {
        __pyx_v_j = __pyx_t_7;

        /* "scipy/linalg/_decomp_update.pyx":1004
 *         for i in range(p):
 *             for j in range(m-2, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)             # <<<<<<<<<<<<<<
 *                 if p-i-1:
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_j, __pyx_v_i), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1), __pyx_v_i), (&__pyx_v_c), (&__pyx_v_s));

        /* "scipy/linalg/_decomp_update.pyx":1005
 *             for j in range(m-2, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:             # <<<<<<<<<<<<<<
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 */
        __pyx_t_1 = (((__pyx_v_p - __pyx_v_i) - 1) != 0);
        if (__pyx_t_1) {

          /* "scipy/linalg/_decomp_update.pyx":1006
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],             # <<<<<<<<<<<<<<
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)
 */
          __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_p - __pyx_v_i) - 1), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, __pyx_v_j, (__pyx_v_i + 1)), (__pyx_v_us[1]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_u, __pyx_v_us, (__pyx_v_j + 1), (__pyx_v_i + 1)), (__pyx_v_us[1]), __pyx_v_c, __pyx_v_s);

          /* "scipy/linalg/_decomp_update.pyx":1005
 *             for j in range(m-2, i-1, -1):
 *                 lartg(index2(u, us, j, i), index2(u, us, j+1, i), &c, &s)
 *                 if p-i-1:             # <<<<<<<<<<<<<<
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":1008
 *                     rot(p-i-1, index2(u, us, j, i+1), us[1],
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)             # <<<<<<<<<<<<<<
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],
 *                         c, s.conjugate())
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_n, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, __pyx_v_j), (__pyx_v_rs[1]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

        /* "scipy/linalg/_decomp_update.pyx":1009
 *                             index2(u, us, j+1, i+1), us[1], c, s)
 *                 rot(n, row(r, rs, j), rs[1], row(r, rs, j+1), rs[1], c, s)
 *                 rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0],             # <<<<<<<<<<<<<<
 *                         c, s.conjugate())
 * 
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_double(__pyx_v_s));
      }
    }

    /* "scipy/linalg/_decomp_update.pyx":1013
 * 
 *         # allocate workspace
 *         work = <blas_t*>libc.stdlib.malloc(n*sizeof(blas_t))             # <<<<<<<<<<<<<<
 *         if not work:
 *             return MEMORY_ERROR
 */
    __pyx_v_work = ((__pyx_t_double_complex *)malloc((__pyx_v_n * (sizeof(__pyx_t_double_complex)))));

    /* "scipy/linalg/_decomp_update.pyx":1014
 *         # allocate workspace
 *         work = <blas_t*>libc.stdlib.malloc(n*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 * 
 */
    __pyx_t_1 = ((!(__pyx_v_work != 0)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":1015
 *         work = <blas_t*>libc.stdlib.malloc(n*sizeof(blas_t))
 *         if not work:
 *             return MEMORY_ERROR             # <<<<<<<<<<<<<<
 * 
 *     # now form UV**H and add it to R.
 */
      __pyx_r = __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":1014
 *         # allocate workspace
 *         work = <blas_t*>libc.stdlib.malloc(n*sizeof(blas_t))
 *         if not work:             # <<<<<<<<<<<<<<
 *             return MEMORY_ERROR
 * 
 */
    }
  }
  __pyx_L3:;

  /* "scipy/linalg/_decomp_update.pyx":1019
 *     # now form UV**H and add it to R.
 *     # This won't fill in any more of R than we have already.
 *     blas_t_2d_conj(p, n, v, vs)             # <<<<<<<<<<<<<<
 *     trmm(sideL, uplo, trans, diag, p, n, 1, u, m, v, p)
 * 
 */
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_2d_conj(__pyx_v_p, __pyx_v_n, __pyx_v_v, __pyx_v_vs);

  /* "scipy/linalg/_decomp_update.pyx":1020
 *     # This won't fill in any more of R than we have already.
 *     blas_t_2d_conj(p, n, v, vs)
 *     trmm(sideL, uplo, trans, diag, p, n, 1, u, m, v, p)             # <<<<<<<<<<<<<<
 * 
 *     # (should this be n, p length adds instead since these are fortran contig?)
 */
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_trmm(__pyx_v_sideL, __pyx_v_uplo, __pyx_v_trans, __pyx_v_diag, __pyx_v_p, __pyx_v_n, __pyx_t_double_complex_from_parts(1, 0), __pyx_v_u, __pyx_v_m, __pyx_v_v, __pyx_v_p);

  /* "scipy/linalg/_decomp_update.pyx":1023
 * 
 *     # (should this be n, p length adds instead since these are fortran contig?)
 *     for j in range(p):             # <<<<<<<<<<<<<<
 *         axpy(n, 1, row(v, vs, j), vs[1], row(r, rs, j), rs[1])
 * 
 */
  __pyx_t_4 = __pyx_v_p;
  __pyx_t_2 = __pyx_t_4;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":1024
 *     # (should this be n, p length adds instead since these are fortran contig?)
 *     for j in range(p):
 *         axpy(n, 1, row(v, vs, j), vs[1], row(r, rs, j), rs[1])             # <<<<<<<<<<<<<<
 * 
 *     # now r has p subdiagonals, eliminate them with reflectors.
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, __pyx_t_double_complex_from_parts(1, 0), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_v, __pyx_v_vs, __pyx_v_j), (__pyx_v_vs[1]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_row(__pyx_v_r, __pyx_v_rs, __pyx_v_j), (__pyx_v_rs[1]));
  }

  /* "scipy/linalg/_decomp_update.pyx":1027
 * 
 *     # now r has p subdiagonals, eliminate them with reflectors.
 *     p_subdiag_qr(m, m, n, q, qs, r, rs, 0, p, work)             # <<<<<<<<<<<<<<
 * 
 *     libc.stdlib.free(work)
 */
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_p_subdiag_qr(__pyx_v_m, __pyx_v_m, __pyx_v_n, __pyx_v_q, __pyx_v_qs, __pyx_v_r, __pyx_v_rs, 0, __pyx_v_p, __pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":1029
 *     p_subdiag_qr(m, m, n, q, qs, r, rs, 0, p, work)
 * 
 *     libc.stdlib.free(work)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  free(__pyx_v_work);

  /* "scipy/linalg/_decomp_update.pyx":1030
 * 
 *     libc.stdlib.free(work)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef void hessenberg_qr(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":940
 *     # no return, return q, r from python driver.
 * 
 * cdef int qr_rank_p_update(int m, int n, int p, blas_t* q, int* qs, blas_t* r,             # <<<<<<<<<<<<<<
 *                         int* rs, blas_t* u, int* us, blas_t* v, int* vs) nogil:
 *     cdef int i, j
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":1032
 *     return 0
 * 
 * cdef void hessenberg_qr(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                         int k) nogil:
 *     """Reduce an upper hessenberg matrix r, to upper triangular, starting in
 */

static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_hessenberg_qr(int __pyx_v_m, int __pyx_v_n, float *__pyx_v_q, int *__pyx_v_qs, float *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k) {
  int __pyx_v_j;
  float __pyx_v_c;
  float __pyx_v_s;
  int __pyx_v_limit;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;

  /* "scipy/linalg/_decomp_update.pyx":1040
 *     cdef int j
 *     cdef blas_t c, s
 *     cdef int limit = min(m-1, n)             # <<<<<<<<<<<<<<
 * 
 *     for j in range(k, limit):
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = (__pyx_v_m - 1);
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":1042
 *     cdef int limit = min(m-1, n)
 * 
 *     for j in range(k, limit):             # <<<<<<<<<<<<<<
 *         lartg(index2(r, rs, j, j), index2(r, rs, j+1, j), &c, &s)
 * 
 */
  __pyx_t_1 = __pyx_v_limit;
  __pyx_t_4 = __pyx_t_1;
  for (__pyx_t_5 = __pyx_v_k; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_j = __pyx_t_5;

    /* "scipy/linalg/_decomp_update.pyx":1043
 * 
 *     for j in range(k, limit):
 *         lartg(index2(r, rs, j, j), index2(r, rs, j+1, j), &c, &s)             # <<<<<<<<<<<<<<
 * 
 *         # update the rest of r
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (&__pyx_v_c), (&__pyx_v_s));

    /* "scipy/linalg/_decomp_update.pyx":1046
 * 
 *         # update the rest of r
 *         if j+1 < m:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 */
    __pyx_t_6 = (((__pyx_v_j + 1) < __pyx_v_m) != 0);
    if (__pyx_t_6) {

      /* "scipy/linalg/_decomp_update.pyx":1047
 *         # update the rest of r
 *         if j+1 < m:
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],             # <<<<<<<<<<<<<<
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 * 
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

      /* "scipy/linalg/_decomp_update.pyx":1046
 * 
 *         # update the rest of r
 *         if j+1 < m:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1051
 * 
 *         # update q
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())             # <<<<<<<<<<<<<<
 * 
 * cdef void p_subdiag_qr(int m, int o, int n, blas_t* q, int* qs, blas_t* r, int* rs,
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_s));
  }

  /* "scipy/linalg/_decomp_update.pyx":1032
 *     return 0
 * 
 * cdef void hessenberg_qr(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                         int k) nogil:
 *     """Reduce an upper hessenberg matrix r, to upper triangular, starting in
 */

  /* function exit code */
}

static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_hessenberg_qr(int __pyx_v_m, int __pyx_v_n, double *__pyx_v_q, int *__pyx_v_qs, double *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k) {
  int __pyx_v_j;
  double __pyx_v_c;
  double __pyx_v_s;
  int __pyx_v_limit;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;

  /* "scipy/linalg/_decomp_update.pyx":1040
 *     cdef int j
 *     cdef blas_t c, s
 *     cdef int limit = min(m-1, n)             # <<<<<<<<<<<<<<
 * 
 *     for j in range(k, limit):
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = (__pyx_v_m - 1);
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":1042
 *     cdef int limit = min(m-1, n)
 * 
 *     for j in range(k, limit):             # <<<<<<<<<<<<<<
 *         lartg(index2(r, rs, j, j), index2(r, rs, j+1, j), &c, &s)
 * 
 */
  __pyx_t_1 = __pyx_v_limit;
  __pyx_t_4 = __pyx_t_1;
  for (__pyx_t_5 = __pyx_v_k; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_j = __pyx_t_5;

    /* "scipy/linalg/_decomp_update.pyx":1043
 * 
 *     for j in range(k, limit):
 *         lartg(index2(r, rs, j, j), index2(r, rs, j+1, j), &c, &s)             # <<<<<<<<<<<<<<
 * 
 *         # update the rest of r
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (&__pyx_v_c), (&__pyx_v_s));

    /* "scipy/linalg/_decomp_update.pyx":1046
 * 
 *         # update the rest of r
 *         if j+1 < m:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 */
    __pyx_t_6 = (((__pyx_v_j + 1) < __pyx_v_m) != 0);
    if (__pyx_t_6) {

      /* "scipy/linalg/_decomp_update.pyx":1047
 *         # update the rest of r
 *         if j+1 < m:
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],             # <<<<<<<<<<<<<<
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 * 
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

      /* "scipy/linalg/_decomp_update.pyx":1046
 * 
 *         # update the rest of r
 *         if j+1 < m:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1051
 * 
 *         # update q
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())             # <<<<<<<<<<<<<<
 * 
 * cdef void p_subdiag_qr(int m, int o, int n, blas_t* q, int* qs, blas_t* r, int* rs,
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c,  (__pyx_v_s));
  }

  /* "scipy/linalg/_decomp_update.pyx":1032
 *     return 0
 * 
 * cdef void hessenberg_qr(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                         int k) nogil:
 *     """Reduce an upper hessenberg matrix r, to upper triangular, starting in
 */

  /* function exit code */
}

static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_hessenberg_qr(int __pyx_v_m, int __pyx_v_n, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_float_complex *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k) {
  int __pyx_v_j;
  __pyx_t_float_complex __pyx_v_c;
  __pyx_t_float_complex __pyx_v_s;
  int __pyx_v_limit;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;

  /* "scipy/linalg/_decomp_update.pyx":1040
 *     cdef int j
 *     cdef blas_t c, s
 *     cdef int limit = min(m-1, n)             # <<<<<<<<<<<<<<
 * 
 *     for j in range(k, limit):
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = (__pyx_v_m - 1);
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":1042
 *     cdef int limit = min(m-1, n)
 * 
 *     for j in range(k, limit):             # <<<<<<<<<<<<<<
 *         lartg(index2(r, rs, j, j), index2(r, rs, j+1, j), &c, &s)
 * 
 */
  __pyx_t_1 = __pyx_v_limit;
  __pyx_t_4 = __pyx_t_1;
  for (__pyx_t_5 = __pyx_v_k; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_j = __pyx_t_5;

    /* "scipy/linalg/_decomp_update.pyx":1043
 * 
 *     for j in range(k, limit):
 *         lartg(index2(r, rs, j, j), index2(r, rs, j+1, j), &c, &s)             # <<<<<<<<<<<<<<
 * 
 *         # update the rest of r
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (&__pyx_v_c), (&__pyx_v_s));

    /* "scipy/linalg/_decomp_update.pyx":1046
 * 
 *         # update the rest of r
 *         if j+1 < m:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 */
    __pyx_t_6 = (((__pyx_v_j + 1) < __pyx_v_m) != 0);
    if (__pyx_t_6) {

      /* "scipy/linalg/_decomp_update.pyx":1047
 *         # update the rest of r
 *         if j+1 < m:
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],             # <<<<<<<<<<<<<<
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 * 
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

      /* "scipy/linalg/_decomp_update.pyx":1046
 * 
 *         # update the rest of r
 *         if j+1 < m:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1051
 * 
 *         # update q
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())             # <<<<<<<<<<<<<<
 * 
 * cdef void p_subdiag_qr(int m, int o, int n, blas_t* q, int* qs, blas_t* r, int* rs,
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_float(__pyx_v_s));
  }

  /* "scipy/linalg/_decomp_update.pyx":1032
 *     return 0
 * 
 * cdef void hessenberg_qr(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                         int k) nogil:
 *     """Reduce an upper hessenberg matrix r, to upper triangular, starting in
 */

  /* function exit code */
}

static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_hessenberg_qr(int __pyx_v_m, int __pyx_v_n, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_double_complex *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k) {
  int __pyx_v_j;
  __pyx_t_double_complex __pyx_v_c;
  __pyx_t_double_complex __pyx_v_s;
  int __pyx_v_limit;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;

  /* "scipy/linalg/_decomp_update.pyx":1040
 *     cdef int j
 *     cdef blas_t c, s
 *     cdef int limit = min(m-1, n)             # <<<<<<<<<<<<<<
 * 
 *     for j in range(k, limit):
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = (__pyx_v_m - 1);
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":1042
 *     cdef int limit = min(m-1, n)
 * 
 *     for j in range(k, limit):             # <<<<<<<<<<<<<<
 *         lartg(index2(r, rs, j, j), index2(r, rs, j+1, j), &c, &s)
 * 
 */
  __pyx_t_1 = __pyx_v_limit;
  __pyx_t_4 = __pyx_t_1;
  for (__pyx_t_5 = __pyx_v_k; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_j = __pyx_t_5;

    /* "scipy/linalg/_decomp_update.pyx":1043
 * 
 *     for j in range(k, limit):
 *         lartg(index2(r, rs, j, j), index2(r, rs, j+1, j), &c, &s)             # <<<<<<<<<<<<<<
 * 
 *         # update the rest of r
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_lartg(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (&__pyx_v_c), (&__pyx_v_s));

    /* "scipy/linalg/_decomp_update.pyx":1046
 * 
 *         # update the rest of r
 *         if j+1 < m:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 */
    __pyx_t_6 = (((__pyx_v_j + 1) < __pyx_v_m) != 0);
    if (__pyx_t_6) {

      /* "scipy/linalg/_decomp_update.pyx":1047
 *         # update the rest of r
 *         if j+1 < m:
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],             # <<<<<<<<<<<<<<
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 * 
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_c, __pyx_v_s);

      /* "scipy/linalg/_decomp_update.pyx":1046
 * 
 *         # update the rest of r
 *         if j+1 < m:             # <<<<<<<<<<<<<<
 *             rot(n-j-1, index2(r, rs, j, j+1), rs[1],
 *                     index2(r, rs, j+1, j+1), rs[1], c, s)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1051
 * 
 *         # update q
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())             # <<<<<<<<<<<<<<
 * 
 * cdef void p_subdiag_qr(int m, int o, int n, blas_t* q, int* qs, blas_t* r, int* rs,
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_rot(__pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, __pyx_v_j), (__pyx_v_qs[0]), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(__pyx_v_q, __pyx_v_qs, (__pyx_v_j + 1)), (__pyx_v_qs[0]), __pyx_v_c, __Pyx_c_conj_double(__pyx_v_s));
  }

  /* "scipy/linalg/_decomp_update.pyx":1032
 *     return 0
 * 
 * cdef void hessenberg_qr(int m, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                         int k) nogil:
 *     """Reduce an upper hessenberg matrix r, to upper triangular, starting in
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":1053
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())
 * 
 * cdef void p_subdiag_qr(int m, int o, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                        int k, int p, blas_t* work) nogil:
 *     """ Reduce a matrix r to upper triangular form by eliminating the lower p
 */

static void __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_p_subdiag_qr(int __pyx_v_m, int __pyx_v_o, int __pyx_v_n, float *__pyx_v_q, int *__pyx_v_qs, float *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p, float *__pyx_v_work) {
  int __pyx_v_j;
  int __pyx_v_last;
  float __pyx_v_tau;
  float __pyx_v_rjj;
  int __pyx_v_limit;
  char *__pyx_v_sideR;
  char *__pyx_v_sideL;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;

  /* "scipy/linalg/_decomp_update.pyx":1065
 *     cdef blas_t tau
 *     cdef blas_t rjj
 *     cdef int limit = min(m-1, n)             # <<<<<<<<<<<<<<
 *     cdef char* sideR = 'R'
 *     cdef char* sideL = 'L'
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = (__pyx_v_m - 1);
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":1066
 *     cdef blas_t rjj
 *     cdef int limit = min(m-1, n)
 *     cdef char* sideR = 'R'             # <<<<<<<<<<<<<<
 *     cdef char* sideL = 'L'
 * 
 */
  __pyx_v_sideR = ((char *)"R");

  /* "scipy/linalg/_decomp_update.pyx":1067
 *     cdef int limit = min(m-1, n)
 *     cdef char* sideR = 'R'
 *     cdef char* sideL = 'L'             # <<<<<<<<<<<<<<
 * 
 *     # R now has p subdiagonal values to be removed starting from col k.
 */
  __pyx_v_sideL = ((char *)"L");

  /* "scipy/linalg/_decomp_update.pyx":1070
 * 
 *     # R now has p subdiagonal values to be removed starting from col k.
 *     for j in range(k, limit):             # <<<<<<<<<<<<<<
 *         # length of the reflector
 *         last = min(p+1, o-j)
 */
  __pyx_t_1 = __pyx_v_limit;
  __pyx_t_4 = __pyx_t_1;
  for (__pyx_t_5 = __pyx_v_k; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_j = __pyx_t_5;

    /* "scipy/linalg/_decomp_update.pyx":1072
 *     for j in range(k, limit):
 *         # length of the reflector
 *         last = min(p+1, o-j)             # <<<<<<<<<<<<<<
 *         rjj = index2(r, rs, j, j)[0]
 *         larfg(last, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 */
    __pyx_t_6 = (__pyx_v_o - __pyx_v_j);
    __pyx_t_3 = (__pyx_v_p + 1);
    if (((__pyx_t_6 < __pyx_t_3) != 0)) {
      __pyx_t_2 = __pyx_t_6;
    } else {
      __pyx_t_2 = __pyx_t_3;
    }
    __pyx_v_last = __pyx_t_2;

    /* "scipy/linalg/_decomp_update.pyx":1073
 *         # length of the reflector
 *         last = min(p+1, o-j)
 *         rjj = index2(r, rs, j, j)[0]             # <<<<<<<<<<<<<<
 *         larfg(last, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1
 */
    __pyx_v_rjj = (__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]);

    /* "scipy/linalg/_decomp_update.pyx":1074
 *         last = min(p+1, o-j)
 *         rjj = index2(r, rs, j, j)[0]
 *         larfg(last, &rjj, index2(r, rs, j+1, j), rs[0], &tau)             # <<<<<<<<<<<<<<
 *         index2(r, rs, j, j)[0] = 1
 * 
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_larfg(__pyx_v_last, (&__pyx_v_rjj), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (__pyx_v_rs[0]), (&__pyx_v_tau));

    /* "scipy/linalg/_decomp_update.pyx":1075
 *         rjj = index2(r, rs, j, j)[0]
 *         larfg(last, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1             # <<<<<<<<<<<<<<
 * 
 *         # apply the reflector to r if necessary
 */
    (__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]) = 1.0;

    /* "scipy/linalg/_decomp_update.pyx":1078
 * 
 *         # apply the reflector to r if necessary
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             larf(sideL, last, n-j-1, index2(r, rs, j, j), rs[0],
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 */
    __pyx_t_7 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
    if (__pyx_t_7) {

      /* "scipy/linalg/_decomp_update.pyx":1079
 *         # apply the reflector to r if necessary
 *         if j+1 < n:
 *             larf(sideL, last, n-j-1, index2(r, rs, j, j), rs[0],             # <<<<<<<<<<<<<<
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 * 
 */
      __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_larf(__pyx_v_sideL, __pyx_v_last, ((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[0]),  (__pyx_v_tau), __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_work);

      /* "scipy/linalg/_decomp_update.pyx":1078
 * 
 *         # apply the reflector to r if necessary
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             larf(sideL, last, n-j-1, index2(r, rs, j, j), rs[0],
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1083
 * 
 *         # apply the reflector to q
 *         larf(sideR, m, last, index2(r, rs, j, j), rs[0], tau,             # <<<<<<<<<<<<<<
 *                 index2(q, qs, 0, j), qs[1], work)
 * 
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_larf(__pyx_v_sideR, __pyx_v_m, __pyx_v_last, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[0]), __pyx_v_tau, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_j), (__pyx_v_qs[1]), __pyx_v_work);

    /* "scipy/linalg/_decomp_update.pyx":1087
 * 
 *         # rezero the householder vector we no longer need.
 *         memset(index2(r, rs, j+1, j), 0, (last-1)*sizeof(blas_t))             # <<<<<<<<<<<<<<
 * 
 *         # restore the rjj element
 */
    (void)(memset(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), 0, ((__pyx_v_last - 1) * (sizeof(float)))));

    /* "scipy/linalg/_decomp_update.pyx":1090
 * 
 *         # restore the rjj element
 *         index2(r, rs, j, j)[0] = rjj             # <<<<<<<<<<<<<<
 * 
 * cdef int reorth(int m, int n, blas_t* q, int* qs, bint qisF, blas_t* u,
 */
    (__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]) = __pyx_v_rjj;
  }

  /* "scipy/linalg/_decomp_update.pyx":1053
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())
 * 
 * cdef void p_subdiag_qr(int m, int o, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                        int k, int p, blas_t* work) nogil:
 *     """ Reduce a matrix r to upper triangular form by eliminating the lower p
 */

  /* function exit code */
}

static void __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_p_subdiag_qr(int __pyx_v_m, int __pyx_v_o, int __pyx_v_n, double *__pyx_v_q, int *__pyx_v_qs, double *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p, double *__pyx_v_work) {
  int __pyx_v_j;
  int __pyx_v_last;
  double __pyx_v_tau;
  double __pyx_v_rjj;
  int __pyx_v_limit;
  char *__pyx_v_sideR;
  char *__pyx_v_sideL;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;

  /* "scipy/linalg/_decomp_update.pyx":1065
 *     cdef blas_t tau
 *     cdef blas_t rjj
 *     cdef int limit = min(m-1, n)             # <<<<<<<<<<<<<<
 *     cdef char* sideR = 'R'
 *     cdef char* sideL = 'L'
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = (__pyx_v_m - 1);
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":1066
 *     cdef blas_t rjj
 *     cdef int limit = min(m-1, n)
 *     cdef char* sideR = 'R'             # <<<<<<<<<<<<<<
 *     cdef char* sideL = 'L'
 * 
 */
  __pyx_v_sideR = ((char *)"R");

  /* "scipy/linalg/_decomp_update.pyx":1067
 *     cdef int limit = min(m-1, n)
 *     cdef char* sideR = 'R'
 *     cdef char* sideL = 'L'             # <<<<<<<<<<<<<<
 * 
 *     # R now has p subdiagonal values to be removed starting from col k.
 */
  __pyx_v_sideL = ((char *)"L");

  /* "scipy/linalg/_decomp_update.pyx":1070
 * 
 *     # R now has p subdiagonal values to be removed starting from col k.
 *     for j in range(k, limit):             # <<<<<<<<<<<<<<
 *         # length of the reflector
 *         last = min(p+1, o-j)
 */
  __pyx_t_1 = __pyx_v_limit;
  __pyx_t_4 = __pyx_t_1;
  for (__pyx_t_5 = __pyx_v_k; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_j = __pyx_t_5;

    /* "scipy/linalg/_decomp_update.pyx":1072
 *     for j in range(k, limit):
 *         # length of the reflector
 *         last = min(p+1, o-j)             # <<<<<<<<<<<<<<
 *         rjj = index2(r, rs, j, j)[0]
 *         larfg(last, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 */
    __pyx_t_6 = (__pyx_v_o - __pyx_v_j);
    __pyx_t_3 = (__pyx_v_p + 1);
    if (((__pyx_t_6 < __pyx_t_3) != 0)) {
      __pyx_t_2 = __pyx_t_6;
    } else {
      __pyx_t_2 = __pyx_t_3;
    }
    __pyx_v_last = __pyx_t_2;

    /* "scipy/linalg/_decomp_update.pyx":1073
 *         # length of the reflector
 *         last = min(p+1, o-j)
 *         rjj = index2(r, rs, j, j)[0]             # <<<<<<<<<<<<<<
 *         larfg(last, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1
 */
    __pyx_v_rjj = (__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]);

    /* "scipy/linalg/_decomp_update.pyx":1074
 *         last = min(p+1, o-j)
 *         rjj = index2(r, rs, j, j)[0]
 *         larfg(last, &rjj, index2(r, rs, j+1, j), rs[0], &tau)             # <<<<<<<<<<<<<<
 *         index2(r, rs, j, j)[0] = 1
 * 
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_larfg(__pyx_v_last, (&__pyx_v_rjj), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (__pyx_v_rs[0]), (&__pyx_v_tau));

    /* "scipy/linalg/_decomp_update.pyx":1075
 *         rjj = index2(r, rs, j, j)[0]
 *         larfg(last, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1             # <<<<<<<<<<<<<<
 * 
 *         # apply the reflector to r if necessary
 */
    (__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]) = 1.0;

    /* "scipy/linalg/_decomp_update.pyx":1078
 * 
 *         # apply the reflector to r if necessary
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             larf(sideL, last, n-j-1, index2(r, rs, j, j), rs[0],
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 */
    __pyx_t_7 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
    if (__pyx_t_7) {

      /* "scipy/linalg/_decomp_update.pyx":1079
 *         # apply the reflector to r if necessary
 *         if j+1 < n:
 *             larf(sideL, last, n-j-1, index2(r, rs, j, j), rs[0],             # <<<<<<<<<<<<<<
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 * 
 */
      __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_larf(__pyx_v_sideL, __pyx_v_last, ((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[0]),  (__pyx_v_tau), __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_work);

      /* "scipy/linalg/_decomp_update.pyx":1078
 * 
 *         # apply the reflector to r if necessary
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             larf(sideL, last, n-j-1, index2(r, rs, j, j), rs[0],
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1083
 * 
 *         # apply the reflector to q
 *         larf(sideR, m, last, index2(r, rs, j, j), rs[0], tau,             # <<<<<<<<<<<<<<
 *                 index2(q, qs, 0, j), qs[1], work)
 * 
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_larf(__pyx_v_sideR, __pyx_v_m, __pyx_v_last, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[0]), __pyx_v_tau, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_j), (__pyx_v_qs[1]), __pyx_v_work);

    /* "scipy/linalg/_decomp_update.pyx":1087
 * 
 *         # rezero the householder vector we no longer need.
 *         memset(index2(r, rs, j+1, j), 0, (last-1)*sizeof(blas_t))             # <<<<<<<<<<<<<<
 * 
 *         # restore the rjj element
 */
    (void)(memset(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), 0, ((__pyx_v_last - 1) * (sizeof(double)))));

    /* "scipy/linalg/_decomp_update.pyx":1090
 * 
 *         # restore the rjj element
 *         index2(r, rs, j, j)[0] = rjj             # <<<<<<<<<<<<<<
 * 
 * cdef int reorth(int m, int n, blas_t* q, int* qs, bint qisF, blas_t* u,
 */
    (__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]) = __pyx_v_rjj;
  }

  /* "scipy/linalg/_decomp_update.pyx":1053
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())
 * 
 * cdef void p_subdiag_qr(int m, int o, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                        int k, int p, blas_t* work) nogil:
 *     """ Reduce a matrix r to upper triangular form by eliminating the lower p
 */

  /* function exit code */
}

static void __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_p_subdiag_qr(int __pyx_v_m, int __pyx_v_o, int __pyx_v_n, __pyx_t_float_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_float_complex *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p, __pyx_t_float_complex *__pyx_v_work) {
  int __pyx_v_j;
  int __pyx_v_last;
  __pyx_t_float_complex __pyx_v_tau;
  __pyx_t_float_complex __pyx_v_rjj;
  int __pyx_v_limit;
  char *__pyx_v_sideR;
  char *__pyx_v_sideL;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;

  /* "scipy/linalg/_decomp_update.pyx":1065
 *     cdef blas_t tau
 *     cdef blas_t rjj
 *     cdef int limit = min(m-1, n)             # <<<<<<<<<<<<<<
 *     cdef char* sideR = 'R'
 *     cdef char* sideL = 'L'
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = (__pyx_v_m - 1);
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":1066
 *     cdef blas_t rjj
 *     cdef int limit = min(m-1, n)
 *     cdef char* sideR = 'R'             # <<<<<<<<<<<<<<
 *     cdef char* sideL = 'L'
 * 
 */
  __pyx_v_sideR = ((char *)"R");

  /* "scipy/linalg/_decomp_update.pyx":1067
 *     cdef int limit = min(m-1, n)
 *     cdef char* sideR = 'R'
 *     cdef char* sideL = 'L'             # <<<<<<<<<<<<<<
 * 
 *     # R now has p subdiagonal values to be removed starting from col k.
 */
  __pyx_v_sideL = ((char *)"L");

  /* "scipy/linalg/_decomp_update.pyx":1070
 * 
 *     # R now has p subdiagonal values to be removed starting from col k.
 *     for j in range(k, limit):             # <<<<<<<<<<<<<<
 *         # length of the reflector
 *         last = min(p+1, o-j)
 */
  __pyx_t_1 = __pyx_v_limit;
  __pyx_t_4 = __pyx_t_1;
  for (__pyx_t_5 = __pyx_v_k; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_j = __pyx_t_5;

    /* "scipy/linalg/_decomp_update.pyx":1072
 *     for j in range(k, limit):
 *         # length of the reflector
 *         last = min(p+1, o-j)             # <<<<<<<<<<<<<<
 *         rjj = index2(r, rs, j, j)[0]
 *         larfg(last, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 */
    __pyx_t_6 = (__pyx_v_o - __pyx_v_j);
    __pyx_t_3 = (__pyx_v_p + 1);
    if (((__pyx_t_6 < __pyx_t_3) != 0)) {
      __pyx_t_2 = __pyx_t_6;
    } else {
      __pyx_t_2 = __pyx_t_3;
    }
    __pyx_v_last = __pyx_t_2;

    /* "scipy/linalg/_decomp_update.pyx":1073
 *         # length of the reflector
 *         last = min(p+1, o-j)
 *         rjj = index2(r, rs, j, j)[0]             # <<<<<<<<<<<<<<
 *         larfg(last, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1
 */
    __pyx_v_rjj = (__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]);

    /* "scipy/linalg/_decomp_update.pyx":1074
 *         last = min(p+1, o-j)
 *         rjj = index2(r, rs, j, j)[0]
 *         larfg(last, &rjj, index2(r, rs, j+1, j), rs[0], &tau)             # <<<<<<<<<<<<<<
 *         index2(r, rs, j, j)[0] = 1
 * 
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_larfg(__pyx_v_last, (&__pyx_v_rjj), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (__pyx_v_rs[0]), (&__pyx_v_tau));

    /* "scipy/linalg/_decomp_update.pyx":1075
 *         rjj = index2(r, rs, j, j)[0]
 *         larfg(last, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1             # <<<<<<<<<<<<<<
 * 
 *         # apply the reflector to r if necessary
 */
    (__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]) = __pyx_t_float_complex_from_parts(1, 0);

    /* "scipy/linalg/_decomp_update.pyx":1078
 * 
 *         # apply the reflector to r if necessary
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             larf(sideL, last, n-j-1, index2(r, rs, j, j), rs[0],
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 */
    __pyx_t_7 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
    if (__pyx_t_7) {

      /* "scipy/linalg/_decomp_update.pyx":1079
 *         # apply the reflector to r if necessary
 *         if j+1 < n:
 *             larf(sideL, last, n-j-1, index2(r, rs, j, j), rs[0],             # <<<<<<<<<<<<<<
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 * 
 */
      __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_larf(__pyx_v_sideL, __pyx_v_last, ((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[0]), __Pyx_c_conj_float(__pyx_v_tau), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_work);

      /* "scipy/linalg/_decomp_update.pyx":1078
 * 
 *         # apply the reflector to r if necessary
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             larf(sideL, last, n-j-1, index2(r, rs, j, j), rs[0],
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1083
 * 
 *         # apply the reflector to q
 *         larf(sideR, m, last, index2(r, rs, j, j), rs[0], tau,             # <<<<<<<<<<<<<<
 *                 index2(q, qs, 0, j), qs[1], work)
 * 
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_larf(__pyx_v_sideR, __pyx_v_m, __pyx_v_last, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[0]), __pyx_v_tau, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_j), (__pyx_v_qs[1]), __pyx_v_work);

    /* "scipy/linalg/_decomp_update.pyx":1087
 * 
 *         # rezero the householder vector we no longer need.
 *         memset(index2(r, rs, j+1, j), 0, (last-1)*sizeof(blas_t))             # <<<<<<<<<<<<<<
 * 
 *         # restore the rjj element
 */
    (void)(memset(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), 0, ((__pyx_v_last - 1) * (sizeof(__pyx_t_float_complex)))));

    /* "scipy/linalg/_decomp_update.pyx":1090
 * 
 *         # restore the rjj element
 *         index2(r, rs, j, j)[0] = rjj             # <<<<<<<<<<<<<<
 * 
 * cdef int reorth(int m, int n, blas_t* q, int* qs, bint qisF, blas_t* u,
 */
    (__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]) = __pyx_v_rjj;
  }

  /* "scipy/linalg/_decomp_update.pyx":1053
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())
 * 
 * cdef void p_subdiag_qr(int m, int o, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                        int k, int p, blas_t* work) nogil:
 *     """ Reduce a matrix r to upper triangular form by eliminating the lower p
 */

  /* function exit code */
}

static void __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_p_subdiag_qr(int __pyx_v_m, int __pyx_v_o, int __pyx_v_n, __pyx_t_double_complex *__pyx_v_q, int *__pyx_v_qs, __pyx_t_double_complex *__pyx_v_r, int *__pyx_v_rs, int __pyx_v_k, int __pyx_v_p, __pyx_t_double_complex *__pyx_v_work) {
  int __pyx_v_j;
  int __pyx_v_last;
  __pyx_t_double_complex __pyx_v_tau;
  __pyx_t_double_complex __pyx_v_rjj;
  int __pyx_v_limit;
  char *__pyx_v_sideR;
  char *__pyx_v_sideL;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;

  /* "scipy/linalg/_decomp_update.pyx":1065
 *     cdef blas_t tau
 *     cdef blas_t rjj
 *     cdef int limit = min(m-1, n)             # <<<<<<<<<<<<<<
 *     cdef char* sideR = 'R'
 *     cdef char* sideL = 'L'
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = (__pyx_v_m - 1);
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_limit = __pyx_t_3;

  /* "scipy/linalg/_decomp_update.pyx":1066
 *     cdef blas_t rjj
 *     cdef int limit = min(m-1, n)
 *     cdef char* sideR = 'R'             # <<<<<<<<<<<<<<
 *     cdef char* sideL = 'L'
 * 
 */
  __pyx_v_sideR = ((char *)"R");

  /* "scipy/linalg/_decomp_update.pyx":1067
 *     cdef int limit = min(m-1, n)
 *     cdef char* sideR = 'R'
 *     cdef char* sideL = 'L'             # <<<<<<<<<<<<<<
 * 
 *     # R now has p subdiagonal values to be removed starting from col k.
 */
  __pyx_v_sideL = ((char *)"L");

  /* "scipy/linalg/_decomp_update.pyx":1070
 * 
 *     # R now has p subdiagonal values to be removed starting from col k.
 *     for j in range(k, limit):             # <<<<<<<<<<<<<<
 *         # length of the reflector
 *         last = min(p+1, o-j)
 */
  __pyx_t_1 = __pyx_v_limit;
  __pyx_t_4 = __pyx_t_1;
  for (__pyx_t_5 = __pyx_v_k; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_j = __pyx_t_5;

    /* "scipy/linalg/_decomp_update.pyx":1072
 *     for j in range(k, limit):
 *         # length of the reflector
 *         last = min(p+1, o-j)             # <<<<<<<<<<<<<<
 *         rjj = index2(r, rs, j, j)[0]
 *         larfg(last, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 */
    __pyx_t_6 = (__pyx_v_o - __pyx_v_j);
    __pyx_t_3 = (__pyx_v_p + 1);
    if (((__pyx_t_6 < __pyx_t_3) != 0)) {
      __pyx_t_2 = __pyx_t_6;
    } else {
      __pyx_t_2 = __pyx_t_3;
    }
    __pyx_v_last = __pyx_t_2;

    /* "scipy/linalg/_decomp_update.pyx":1073
 *         # length of the reflector
 *         last = min(p+1, o-j)
 *         rjj = index2(r, rs, j, j)[0]             # <<<<<<<<<<<<<<
 *         larfg(last, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1
 */
    __pyx_v_rjj = (__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]);

    /* "scipy/linalg/_decomp_update.pyx":1074
 *         last = min(p+1, o-j)
 *         rjj = index2(r, rs, j, j)[0]
 *         larfg(last, &rjj, index2(r, rs, j+1, j), rs[0], &tau)             # <<<<<<<<<<<<<<
 *         index2(r, rs, j, j)[0] = 1
 * 
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_larfg(__pyx_v_last, (&__pyx_v_rjj), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), (__pyx_v_rs[0]), (&__pyx_v_tau));

    /* "scipy/linalg/_decomp_update.pyx":1075
 *         rjj = index2(r, rs, j, j)[0]
 *         larfg(last, &rjj, index2(r, rs, j+1, j), rs[0], &tau)
 *         index2(r, rs, j, j)[0] = 1             # <<<<<<<<<<<<<<
 * 
 *         # apply the reflector to r if necessary
 */
    (__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]) = __pyx_t_double_complex_from_parts(1, 0);

    /* "scipy/linalg/_decomp_update.pyx":1078
 * 
 *         # apply the reflector to r if necessary
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             larf(sideL, last, n-j-1, index2(r, rs, j, j), rs[0],
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 */
    __pyx_t_7 = (((__pyx_v_j + 1) < __pyx_v_n) != 0);
    if (__pyx_t_7) {

      /* "scipy/linalg/_decomp_update.pyx":1079
 *         # apply the reflector to r if necessary
 *         if j+1 < n:
 *             larf(sideL, last, n-j-1, index2(r, rs, j, j), rs[0],             # <<<<<<<<<<<<<<
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 * 
 */
      __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_larf(__pyx_v_sideL, __pyx_v_last, ((__pyx_v_n - __pyx_v_j) - 1), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[0]), __Pyx_c_conj_double(__pyx_v_tau), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, (__pyx_v_j + 1)), (__pyx_v_rs[1]), __pyx_v_work);

      /* "scipy/linalg/_decomp_update.pyx":1078
 * 
 *         # apply the reflector to r if necessary
 *         if j+1 < n:             # <<<<<<<<<<<<<<
 *             larf(sideL, last, n-j-1, index2(r, rs, j, j), rs[0],
 *                     tau.conjugate(), index2(r, rs, j, j+1), rs[1], work)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1083
 * 
 *         # apply the reflector to q
 *         larf(sideR, m, last, index2(r, rs, j, j), rs[0], tau,             # <<<<<<<<<<<<<<
 *                 index2(q, qs, 0, j), qs[1], work)
 * 
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_larf(__pyx_v_sideR, __pyx_v_m, __pyx_v_last, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j), (__pyx_v_rs[0]), __pyx_v_tau, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_q, __pyx_v_qs, 0, __pyx_v_j), (__pyx_v_qs[1]), __pyx_v_work);

    /* "scipy/linalg/_decomp_update.pyx":1087
 * 
 *         # rezero the householder vector we no longer need.
 *         memset(index2(r, rs, j+1, j), 0, (last-1)*sizeof(blas_t))             # <<<<<<<<<<<<<<
 * 
 *         # restore the rjj element
 */
    (void)(memset(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, (__pyx_v_j + 1), __pyx_v_j), 0, ((__pyx_v_last - 1) * (sizeof(__pyx_t_double_complex)))));

    /* "scipy/linalg/_decomp_update.pyx":1090
 * 
 *         # restore the rjj element
 *         index2(r, rs, j, j)[0] = rjj             # <<<<<<<<<<<<<<
 * 
 * cdef int reorth(int m, int n, blas_t* q, int* qs, bint qisF, blas_t* u,
 */
    (__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_index2(__pyx_v_r, __pyx_v_rs, __pyx_v_j, __pyx_v_j)[0]) = __pyx_v_rjj;
  }

  /* "scipy/linalg/_decomp_update.pyx":1053
 *         rot(m, col(q, qs, j), qs[0], col(q, qs, j+1), qs[0], c, s.conjugate())
 * 
 * cdef void p_subdiag_qr(int m, int o, int n, blas_t* q, int* qs, blas_t* r, int* rs,             # <<<<<<<<<<<<<<
 *                        int k, int p, blas_t* work) nogil:
 *     """ Reduce a matrix r to upper triangular form by eliminating the lower p
 */

  /* function exit code */
}

/* "scipy/linalg/_decomp_update.pyx":1092
 *         index2(r, rs, j, j)[0] = rjj
 * 
 * cdef int reorth(int m, int n, blas_t* q, int* qs, bint qisF, blas_t* u,             # <<<<<<<<<<<<<<
 *                 int* us, blas_t* s, blas_t* RCOND) nogil:
 *     """Given a (m,n) matrix q with orthonormal columns and a (m,) vector u,
 */

static int __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_reorth(int __pyx_v_m, int __pyx_v_n, float *__pyx_v_q, CYTHON_UNUSED int *__pyx_v_qs, int __pyx_v_qisF, float *__pyx_v_u, int *__pyx_v_us, float *__pyx_v_s, float *__pyx_v_RCOND) {
  float __pyx_v_unorm;
  float __pyx_v_snorm;
  float __pyx_v_wnorm;
  float __pyx_v_wpnorm;
  float __pyx_v_sigma_max;
  float __pyx_v_sigma_min;
  float __pyx_v_rc;
  char *__pyx_v_T;
  char *__pyx_v_N;
  CYTHON_UNUSED char *__pyx_v_C;
  CYTHON_UNUSED int __pyx_v_ss;
  float __pyx_v_inv_root2;
  int __pyx_r;
  int __pyx_t_1;

  /* "scipy/linalg/_decomp_update.pyx":1109
 *     """
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 *     cdef char* T = 'T'             # <<<<<<<<<<<<<<
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'
 */
  __pyx_v_T = ((char *)"T");

  /* "scipy/linalg/_decomp_update.pyx":1110
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'             # <<<<<<<<<<<<<<
 *     cdef char* C = 'C'
 *     cdef int ss = 1
 */
  __pyx_v_N = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":1111
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'             # <<<<<<<<<<<<<<
 *     cdef int ss = 1
 *     cdef blas_t inv_root2 = NPY_SQRT1_2
 */
  __pyx_v_C = ((char *)"C");

  /* "scipy/linalg/_decomp_update.pyx":1112
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'
 *     cdef int ss = 1             # <<<<<<<<<<<<<<
 *     cdef blas_t inv_root2 = NPY_SQRT1_2
 * 
 */
  __pyx_v_ss = 1;

  /* "scipy/linalg/_decomp_update.pyx":1113
 *     cdef char* C = 'C'
 *     cdef int ss = 1
 *     cdef blas_t inv_root2 = NPY_SQRT1_2             # <<<<<<<<<<<<<<
 * 
 *     # normalize u
 */
  __pyx_v_inv_root2 = NPY_SQRT1_2;

  /* "scipy/linalg/_decomp_update.pyx":1116
 * 
 *     # normalize u
 *     unorm = nrm2(m, u, us[0])             # <<<<<<<<<<<<<<
 *     scal(m, 1/unorm, u, us[0])
 * 
 */
  __pyx_v_unorm = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_m, __pyx_v_u, (__pyx_v_us[0]));

  /* "scipy/linalg/_decomp_update.pyx":1117
 *     # normalize u
 *     unorm = nrm2(m, u, us[0])
 *     scal(m, 1/unorm, u, us[0])             # <<<<<<<<<<<<<<
 * 
 *     # decompose u into q's columns.
 */
  if (unlikely(__pyx_v_unorm == 0)) {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
    #endif
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    #ifdef WITH_THREAD
    __Pyx_PyGILState_Release(__pyx_gilstate_save);
    #endif
    __PYX_ERR(0, 1117, __pyx_L1_error)
  }
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, (1.0 / __pyx_v_unorm), __pyx_v_u, (__pyx_v_us[0]));

  /* "scipy/linalg/_decomp_update.pyx":1120
 * 
 *     # decompose u into q's columns.
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s, 1)
 */
  __pyx_t_1 = (__pyx_v_qisF != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":1122
 *     if qisF:
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s, 1)             # <<<<<<<<<<<<<<
 *         else:
 *             gemv(C, m, n, 1, q, m, u, us[0], 0, s, 1)
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_m, __pyx_v_n, 1.0, __pyx_v_q, __pyx_v_m, __pyx_v_u, (__pyx_v_us[0]), 0.0, __pyx_v_s, 1);

    /* "scipy/linalg/_decomp_update.pyx":1120
 * 
 *     # decompose u into q's columns.
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s, 1)
 */
    goto __pyx_L3;
  }

  /* "scipy/linalg/_decomp_update.pyx":1126
 *             gemv(C, m, n, 1, q, m, u, us[0], 0, s, 1)
 *     else:
 *         if blas_t is float or blas_t is double:             # <<<<<<<<<<<<<<
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s, 1)
 *         else:
 */
  /*else*/ {

    /* "scipy/linalg/_decomp_update.pyx":1127
 *     else:
 *         if blas_t is float or blas_t is double:
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s, 1)             # <<<<<<<<<<<<<<
 *         else:
 *             blas_t_conj(m, u, us)
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_n, __pyx_v_m, 1.0, __pyx_v_q, __pyx_v_n, __pyx_v_u, (__pyx_v_us[0]), 0.0, __pyx_v_s, 1);
  }
  __pyx_L3:;

  /* "scipy/linalg/_decomp_update.pyx":1135
 * 
 *     # sigma_max is the largest singular value of q augmented with u/unorm
 *     snorm = nrm2(n, s, 1)             # <<<<<<<<<<<<<<
 *     sigma_max = blas_t_sqrt(1 + snorm)
 * 
 */
  __pyx_v_snorm = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_n, __pyx_v_s, 1);

  /* "scipy/linalg/_decomp_update.pyx":1136
 *     # sigma_max is the largest singular value of q augmented with u/unorm
 *     snorm = nrm2(n, s, 1)
 *     sigma_max = blas_t_sqrt(1 + snorm)             # <<<<<<<<<<<<<<
 * 
 *     # make u be the part of u that is not in span(q)
 */
  __pyx_v_sigma_max = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_sqrt((1.0 + __pyx_v_snorm));

  /* "scipy/linalg/_decomp_update.pyx":1140
 *     # make u be the part of u that is not in span(q)
 *     # i.e. u -= q.dot(s)
 *     if qisF:             # <<<<<<<<<<<<<<
 *         gemv(N, m, n, -1, q, m, s, 1, 1, u, us[0])
 *     else:
 */
  __pyx_t_1 = (__pyx_v_qisF != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":1141
 *     # i.e. u -= q.dot(s)
 *     if qisF:
 *         gemv(N, m, n, -1, q, m, s, 1, 1, u, us[0])             # <<<<<<<<<<<<<<
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, us[0])
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_n, -1.0, __pyx_v_q, __pyx_v_m, __pyx_v_s, 1, 1.0, __pyx_v_u, (__pyx_v_us[0]));

    /* "scipy/linalg/_decomp_update.pyx":1140
 *     # make u be the part of u that is not in span(q)
 *     # i.e. u -= q.dot(s)
 *     if qisF:             # <<<<<<<<<<<<<<
 *         gemv(N, m, n, -1, q, m, s, 1, 1, u, us[0])
 *     else:
 */
    goto __pyx_L4;
  }

  /* "scipy/linalg/_decomp_update.pyx":1143
 *         gemv(N, m, n, -1, q, m, s, 1, 1, u, us[0])
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, us[0])             # <<<<<<<<<<<<<<
 *     wnorm = nrm2(m, u, us[0])
 * 
 */
  /*else*/ {
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_n, __pyx_v_m, -1.0, __pyx_v_q, __pyx_v_n, __pyx_v_s, 1, 1.0, __pyx_v_u, (__pyx_v_us[0]));
  }
  __pyx_L4:;

  /* "scipy/linalg/_decomp_update.pyx":1144
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, us[0])
 *     wnorm = nrm2(m, u, us[0])             # <<<<<<<<<<<<<<
 * 
 *     # sigma_min is the smallest singular value of q augmented with u/unorm
 */
  __pyx_v_wnorm = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_m, __pyx_v_u, (__pyx_v_us[0]));

  /* "scipy/linalg/_decomp_update.pyx":1149
 *     # the others are == 1, since q is orthonormal.
 *     with cython.cdivision(True):
 *         sigma_min = wnorm / sigma_max             # <<<<<<<<<<<<<<
 *         rc = sigma_min / sigma_max
 * 
 */
  __pyx_v_sigma_min = (__pyx_v_wnorm / __pyx_v_sigma_max);

  /* "scipy/linalg/_decomp_update.pyx":1150
 *     with cython.cdivision(True):
 *         sigma_min = wnorm / sigma_max
 *         rc = sigma_min / sigma_max             # <<<<<<<<<<<<<<
 * 
 *     # check the conditioning of the problem.
 */
  __pyx_v_rc = (__pyx_v_sigma_min / __pyx_v_sigma_max);

  /* "scipy/linalg/_decomp_update.pyx":1153
 * 
 *     # check the conditioning of the problem.
 *     if blas_t_less_than(rc, RCOND[0]):             # <<<<<<<<<<<<<<
 *         RCOND[0] = rc
 *         return 2
 */
  __pyx_t_1 = (__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_v_rc, (__pyx_v_RCOND[0])) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":1154
 *     # check the conditioning of the problem.
 *     if blas_t_less_than(rc, RCOND[0]):
 *         RCOND[0] = rc             # <<<<<<<<<<<<<<
 *         return 2
 *     RCOND[0] = rc
 */
    (__pyx_v_RCOND[0]) = __pyx_v_rc;

    /* "scipy/linalg/_decomp_update.pyx":1155
 *     if blas_t_less_than(rc, RCOND[0]):
 *         RCOND[0] = rc
 *         return 2             # <<<<<<<<<<<<<<
 *     RCOND[0] = rc
 * 
 */
    __pyx_r = 2;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":1153
 * 
 *     # check the conditioning of the problem.
 *     if blas_t_less_than(rc, RCOND[0]):             # <<<<<<<<<<<<<<
 *         RCOND[0] = rc
 *         return 2
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1156
 *         RCOND[0] = rc
 *         return 2
 *     RCOND[0] = rc             # <<<<<<<<<<<<<<
 * 
 *     if blas_t_less_than(inv_root2, wnorm):
 */
  (__pyx_v_RCOND[0]) = __pyx_v_rc;

  /* "scipy/linalg/_decomp_update.pyx":1158
 *     RCOND[0] = rc
 * 
 *     if blas_t_less_than(inv_root2, wnorm):             # <<<<<<<<<<<<<<
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, us[0])
 */
  __pyx_t_1 = (__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_v_inv_root2, __pyx_v_wnorm) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":1160
 *     if blas_t_less_than(inv_root2, wnorm):
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, us[0])             # <<<<<<<<<<<<<<
 *         scal(n, unorm, s, 1)
 *         s[n] = unorm*wnorm
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, (1.0 / __pyx_v_wnorm), __pyx_v_u, (__pyx_v_us[0]));

    /* "scipy/linalg/_decomp_update.pyx":1161
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, us[0])
 *         scal(n, unorm, s, 1)             # <<<<<<<<<<<<<<
 *         s[n] = unorm*wnorm
 *         return 0
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_n, __pyx_v_unorm, __pyx_v_s, 1);

    /* "scipy/linalg/_decomp_update.pyx":1162
 *             scal(m, 1/wnorm, u, us[0])
 *         scal(n, unorm, s, 1)
 *         s[n] = unorm*wnorm             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
    (__pyx_v_s[__pyx_v_n]) = (__pyx_v_unorm * __pyx_v_wnorm);

    /* "scipy/linalg/_decomp_update.pyx":1163
 *         scal(n, unorm, s, 1)
 *         s[n] = unorm*wnorm
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     # if we get here, u does not yet contain the orthogonal vector we are
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":1158
 *     RCOND[0] = rc
 * 
 *     if blas_t_less_than(inv_root2, wnorm):             # <<<<<<<<<<<<<<
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, us[0])
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1168
 *     # looking for. According to Reichel, one reorthogonalization will always
 *     # be enough.
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s+n, 1)
 */
  __pyx_t_1 = (__pyx_v_qisF != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":1170
 *     if qisF:
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s+n, 1)             # <<<<<<<<<<<<<<
 *         else:
 *             gemv(C, m, n, 1, q, m, u, us[0], 0, s+n, 1)
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_m, __pyx_v_n, 1.0, __pyx_v_q, __pyx_v_m, __pyx_v_u, (__pyx_v_us[0]), 0.0, (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":1173
 *         else:
 *             gemv(C, m, n, 1, q, m, u, us[0], 0, s+n, 1)
 *         gemv(N, m, n, -1, q, m, s+n, 1, 1, u, us[0])             # <<<<<<<<<<<<<<
 *     else:
 *         if blas_t is float or blas_t is double:
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_n, -1.0, __pyx_v_q, __pyx_v_m, (__pyx_v_s + __pyx_v_n), 1, 1.0, __pyx_v_u, (__pyx_v_us[0]));

    /* "scipy/linalg/_decomp_update.pyx":1168
 *     # looking for. According to Reichel, one reorthogonalization will always
 *     # be enough.
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s+n, 1)
 */
    goto __pyx_L7;
  }

  /* "scipy/linalg/_decomp_update.pyx":1175
 *         gemv(N, m, n, -1, q, m, s+n, 1, 1, u, us[0])
 *     else:
 *         if blas_t is float or blas_t is double:             # <<<<<<<<<<<<<<
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s+n, 1)
 *         else:
 */
  /*else*/ {

    /* "scipy/linalg/_decomp_update.pyx":1176
 *     else:
 *         if blas_t is float or blas_t is double:
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s+n, 1)             # <<<<<<<<<<<<<<
 *         else:
 *             blas_t_conj(m, u, us)
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_n, __pyx_v_m, 1.0, __pyx_v_q, __pyx_v_n, __pyx_v_u, (__pyx_v_us[0]), 0.0, (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":1182
 *             blas_t_conj(m, u, us)
 *             blas_t_conj(n, s+n, &ss)
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, us[0])             # <<<<<<<<<<<<<<
 * 
 *     wpnorm = nrm2(m, u, us[0])
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_n, __pyx_v_m, -1.0, __pyx_v_q, __pyx_v_n, (__pyx_v_s + __pyx_v_n), 1, 1.0, __pyx_v_u, (__pyx_v_us[0]));
  }
  __pyx_L7:;

  /* "scipy/linalg/_decomp_update.pyx":1184
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, us[0])
 * 
 *     wpnorm = nrm2(m, u, us[0])             # <<<<<<<<<<<<<<
 * 
 *     if blas_t_less_equal(wpnorm, wnorm*inv_root2): # u lies in span(q)
 */
  __pyx_v_wpnorm = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_m, __pyx_v_u, (__pyx_v_us[0]));

  /* "scipy/linalg/_decomp_update.pyx":1186
 *     wpnorm = nrm2(m, u, us[0])
 * 
 *     if blas_t_less_equal(wpnorm, wnorm*inv_root2): # u lies in span(q)             # <<<<<<<<<<<<<<
 *         scal(m, 0, u, us[0])
 *         axpy(n, 1, s, 1, s+n, 1)
 */
  __pyx_t_1 = (__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_equal(__pyx_v_wpnorm, (__pyx_v_wnorm * __pyx_v_inv_root2)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":1187
 * 
 *     if blas_t_less_equal(wpnorm, wnorm*inv_root2): # u lies in span(q)
 *         scal(m, 0, u, us[0])             # <<<<<<<<<<<<<<
 *         axpy(n, 1, s, 1, s+n, 1)
 *         scal(n, unorm, s, 1)
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, 0.0, __pyx_v_u, (__pyx_v_us[0]));

    /* "scipy/linalg/_decomp_update.pyx":1188
 *     if blas_t_less_equal(wpnorm, wnorm*inv_root2): # u lies in span(q)
 *         scal(m, 0, u, us[0])
 *         axpy(n, 1, s, 1, s+n, 1)             # <<<<<<<<<<<<<<
 *         scal(n, unorm, s, 1)
 *         s[n] = 0
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, 1.0, __pyx_v_s, 1, (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":1189
 *         scal(m, 0, u, us[0])
 *         axpy(n, 1, s, 1, s+n, 1)
 *         scal(n, unorm, s, 1)             # <<<<<<<<<<<<<<
 *         s[n] = 0
 *         return 1
 */
    __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_n, __pyx_v_unorm, __pyx_v_s, 1);

    /* "scipy/linalg/_decomp_update.pyx":1190
 *         axpy(n, 1, s, 1, s+n, 1)
 *         scal(n, unorm, s, 1)
 *         s[n] = 0             # <<<<<<<<<<<<<<
 *         return 1
 * 
 */
    (__pyx_v_s[__pyx_v_n]) = 0.0;

    /* "scipy/linalg/_decomp_update.pyx":1191
 *         scal(n, unorm, s, 1)
 *         s[n] = 0
 *         return 1             # <<<<<<<<<<<<<<
 * 
 *     with cython.cdivision(True):
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":1186
 *     wpnorm = nrm2(m, u, us[0])
 * 
 *     if blas_t_less_equal(wpnorm, wnorm*inv_root2): # u lies in span(q)             # <<<<<<<<<<<<<<
 *         scal(m, 0, u, us[0])
 *         axpy(n, 1, s, 1, s+n, 1)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1194
 * 
 *     with cython.cdivision(True):
 *         scal(m, 1/wpnorm, u, us[0])             # <<<<<<<<<<<<<<
 *     axpy(n, 1, s, 1, s+n, 1)
 *     scal(n, unorm, s, 1)
 */
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, (1.0 / __pyx_v_wpnorm), __pyx_v_u, (__pyx_v_us[0]));

  /* "scipy/linalg/_decomp_update.pyx":1195
 *     with cython.cdivision(True):
 *         scal(m, 1/wpnorm, u, us[0])
 *     axpy(n, 1, s, 1, s+n, 1)             # <<<<<<<<<<<<<<
 *     scal(n, unorm, s, 1)
 *     s[n] = wpnorm*unorm
 */
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, 1.0, __pyx_v_s, 1, (__pyx_v_s + __pyx_v_n), 1);

  /* "scipy/linalg/_decomp_update.pyx":1196
 *         scal(m, 1/wpnorm, u, us[0])
 *     axpy(n, 1, s, 1, s+n, 1)
 *     scal(n, unorm, s, 1)             # <<<<<<<<<<<<<<
 *     s[n] = wpnorm*unorm
 *     return 0
 */
  __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_n, __pyx_v_unorm, __pyx_v_s, 1);

  /* "scipy/linalg/_decomp_update.pyx":1197
 *     axpy(n, 1, s, 1, s+n, 1)
 *     scal(n, unorm, s, 1)
 *     s[n] = wpnorm*unorm             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  (__pyx_v_s[__pyx_v_n]) = (__pyx_v_wpnorm * __pyx_v_unorm);

  /* "scipy/linalg/_decomp_update.pyx":1198
 *     scal(n, unorm, s, 1)
 *     s[n] = wpnorm*unorm
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * def _form_qTu(object a, object b):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":1092
 *         index2(r, rs, j, j)[0] = rjj
 * 
 * cdef int reorth(int m, int n, blas_t* q, int* qs, bint qisF, blas_t* u,             # <<<<<<<<<<<<<<
 *                 int* us, blas_t* s, blas_t* RCOND) nogil:
 *     """Given a (m,n) matrix q with orthonormal columns and a (m,) vector u,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.linalg._decomp_update.reorth", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_reorth(int __pyx_v_m, int __pyx_v_n, double *__pyx_v_q, CYTHON_UNUSED int *__pyx_v_qs, int __pyx_v_qisF, double *__pyx_v_u, int *__pyx_v_us, double *__pyx_v_s, double *__pyx_v_RCOND) {
  double __pyx_v_unorm;
  double __pyx_v_snorm;
  double __pyx_v_wnorm;
  double __pyx_v_wpnorm;
  double __pyx_v_sigma_max;
  double __pyx_v_sigma_min;
  double __pyx_v_rc;
  char *__pyx_v_T;
  char *__pyx_v_N;
  CYTHON_UNUSED char *__pyx_v_C;
  CYTHON_UNUSED int __pyx_v_ss;
  double __pyx_v_inv_root2;
  int __pyx_r;
  int __pyx_t_1;

  /* "scipy/linalg/_decomp_update.pyx":1109
 *     """
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 *     cdef char* T = 'T'             # <<<<<<<<<<<<<<
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'
 */
  __pyx_v_T = ((char *)"T");

  /* "scipy/linalg/_decomp_update.pyx":1110
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'             # <<<<<<<<<<<<<<
 *     cdef char* C = 'C'
 *     cdef int ss = 1
 */
  __pyx_v_N = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":1111
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'             # <<<<<<<<<<<<<<
 *     cdef int ss = 1
 *     cdef blas_t inv_root2 = NPY_SQRT1_2
 */
  __pyx_v_C = ((char *)"C");

  /* "scipy/linalg/_decomp_update.pyx":1112
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'
 *     cdef int ss = 1             # <<<<<<<<<<<<<<
 *     cdef blas_t inv_root2 = NPY_SQRT1_2
 * 
 */
  __pyx_v_ss = 1;

  /* "scipy/linalg/_decomp_update.pyx":1113
 *     cdef char* C = 'C'
 *     cdef int ss = 1
 *     cdef blas_t inv_root2 = NPY_SQRT1_2             # <<<<<<<<<<<<<<
 * 
 *     # normalize u
 */
  __pyx_v_inv_root2 = NPY_SQRT1_2;

  /* "scipy/linalg/_decomp_update.pyx":1116
 * 
 *     # normalize u
 *     unorm = nrm2(m, u, us[0])             # <<<<<<<<<<<<<<
 *     scal(m, 1/unorm, u, us[0])
 * 
 */
  __pyx_v_unorm = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_m, __pyx_v_u, (__pyx_v_us[0]));

  /* "scipy/linalg/_decomp_update.pyx":1117
 *     # normalize u
 *     unorm = nrm2(m, u, us[0])
 *     scal(m, 1/unorm, u, us[0])             # <<<<<<<<<<<<<<
 * 
 *     # decompose u into q's columns.
 */
  if (unlikely(__pyx_v_unorm == 0)) {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
    #endif
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    #ifdef WITH_THREAD
    __Pyx_PyGILState_Release(__pyx_gilstate_save);
    #endif
    __PYX_ERR(0, 1117, __pyx_L1_error)
  }
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, (1.0 / __pyx_v_unorm), __pyx_v_u, (__pyx_v_us[0]));

  /* "scipy/linalg/_decomp_update.pyx":1120
 * 
 *     # decompose u into q's columns.
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s, 1)
 */
  __pyx_t_1 = (__pyx_v_qisF != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":1122
 *     if qisF:
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s, 1)             # <<<<<<<<<<<<<<
 *         else:
 *             gemv(C, m, n, 1, q, m, u, us[0], 0, s, 1)
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_m, __pyx_v_n, 1.0, __pyx_v_q, __pyx_v_m, __pyx_v_u, (__pyx_v_us[0]), 0.0, __pyx_v_s, 1);

    /* "scipy/linalg/_decomp_update.pyx":1120
 * 
 *     # decompose u into q's columns.
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s, 1)
 */
    goto __pyx_L3;
  }

  /* "scipy/linalg/_decomp_update.pyx":1126
 *             gemv(C, m, n, 1, q, m, u, us[0], 0, s, 1)
 *     else:
 *         if blas_t is float or blas_t is double:             # <<<<<<<<<<<<<<
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s, 1)
 *         else:
 */
  /*else*/ {

    /* "scipy/linalg/_decomp_update.pyx":1127
 *     else:
 *         if blas_t is float or blas_t is double:
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s, 1)             # <<<<<<<<<<<<<<
 *         else:
 *             blas_t_conj(m, u, us)
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_n, __pyx_v_m, 1.0, __pyx_v_q, __pyx_v_n, __pyx_v_u, (__pyx_v_us[0]), 0.0, __pyx_v_s, 1);
  }
  __pyx_L3:;

  /* "scipy/linalg/_decomp_update.pyx":1135
 * 
 *     # sigma_max is the largest singular value of q augmented with u/unorm
 *     snorm = nrm2(n, s, 1)             # <<<<<<<<<<<<<<
 *     sigma_max = blas_t_sqrt(1 + snorm)
 * 
 */
  __pyx_v_snorm = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_n, __pyx_v_s, 1);

  /* "scipy/linalg/_decomp_update.pyx":1136
 *     # sigma_max is the largest singular value of q augmented with u/unorm
 *     snorm = nrm2(n, s, 1)
 *     sigma_max = blas_t_sqrt(1 + snorm)             # <<<<<<<<<<<<<<
 * 
 *     # make u be the part of u that is not in span(q)
 */
  __pyx_v_sigma_max = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_sqrt((1.0 + __pyx_v_snorm));

  /* "scipy/linalg/_decomp_update.pyx":1140
 *     # make u be the part of u that is not in span(q)
 *     # i.e. u -= q.dot(s)
 *     if qisF:             # <<<<<<<<<<<<<<
 *         gemv(N, m, n, -1, q, m, s, 1, 1, u, us[0])
 *     else:
 */
  __pyx_t_1 = (__pyx_v_qisF != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":1141
 *     # i.e. u -= q.dot(s)
 *     if qisF:
 *         gemv(N, m, n, -1, q, m, s, 1, 1, u, us[0])             # <<<<<<<<<<<<<<
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, us[0])
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_n, -1.0, __pyx_v_q, __pyx_v_m, __pyx_v_s, 1, 1.0, __pyx_v_u, (__pyx_v_us[0]));

    /* "scipy/linalg/_decomp_update.pyx":1140
 *     # make u be the part of u that is not in span(q)
 *     # i.e. u -= q.dot(s)
 *     if qisF:             # <<<<<<<<<<<<<<
 *         gemv(N, m, n, -1, q, m, s, 1, 1, u, us[0])
 *     else:
 */
    goto __pyx_L4;
  }

  /* "scipy/linalg/_decomp_update.pyx":1143
 *         gemv(N, m, n, -1, q, m, s, 1, 1, u, us[0])
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, us[0])             # <<<<<<<<<<<<<<
 *     wnorm = nrm2(m, u, us[0])
 * 
 */
  /*else*/ {
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_n, __pyx_v_m, -1.0, __pyx_v_q, __pyx_v_n, __pyx_v_s, 1, 1.0, __pyx_v_u, (__pyx_v_us[0]));
  }
  __pyx_L4:;

  /* "scipy/linalg/_decomp_update.pyx":1144
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, us[0])
 *     wnorm = nrm2(m, u, us[0])             # <<<<<<<<<<<<<<
 * 
 *     # sigma_min is the smallest singular value of q augmented with u/unorm
 */
  __pyx_v_wnorm = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_m, __pyx_v_u, (__pyx_v_us[0]));

  /* "scipy/linalg/_decomp_update.pyx":1149
 *     # the others are == 1, since q is orthonormal.
 *     with cython.cdivision(True):
 *         sigma_min = wnorm / sigma_max             # <<<<<<<<<<<<<<
 *         rc = sigma_min / sigma_max
 * 
 */
  __pyx_v_sigma_min = (__pyx_v_wnorm / __pyx_v_sigma_max);

  /* "scipy/linalg/_decomp_update.pyx":1150
 *     with cython.cdivision(True):
 *         sigma_min = wnorm / sigma_max
 *         rc = sigma_min / sigma_max             # <<<<<<<<<<<<<<
 * 
 *     # check the conditioning of the problem.
 */
  __pyx_v_rc = (__pyx_v_sigma_min / __pyx_v_sigma_max);

  /* "scipy/linalg/_decomp_update.pyx":1153
 * 
 *     # check the conditioning of the problem.
 *     if blas_t_less_than(rc, RCOND[0]):             # <<<<<<<<<<<<<<
 *         RCOND[0] = rc
 *         return 2
 */
  __pyx_t_1 = (__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_v_rc, (__pyx_v_RCOND[0])) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":1154
 *     # check the conditioning of the problem.
 *     if blas_t_less_than(rc, RCOND[0]):
 *         RCOND[0] = rc             # <<<<<<<<<<<<<<
 *         return 2
 *     RCOND[0] = rc
 */
    (__pyx_v_RCOND[0]) = __pyx_v_rc;

    /* "scipy/linalg/_decomp_update.pyx":1155
 *     if blas_t_less_than(rc, RCOND[0]):
 *         RCOND[0] = rc
 *         return 2             # <<<<<<<<<<<<<<
 *     RCOND[0] = rc
 * 
 */
    __pyx_r = 2;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":1153
 * 
 *     # check the conditioning of the problem.
 *     if blas_t_less_than(rc, RCOND[0]):             # <<<<<<<<<<<<<<
 *         RCOND[0] = rc
 *         return 2
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1156
 *         RCOND[0] = rc
 *         return 2
 *     RCOND[0] = rc             # <<<<<<<<<<<<<<
 * 
 *     if blas_t_less_than(inv_root2, wnorm):
 */
  (__pyx_v_RCOND[0]) = __pyx_v_rc;

  /* "scipy/linalg/_decomp_update.pyx":1158
 *     RCOND[0] = rc
 * 
 *     if blas_t_less_than(inv_root2, wnorm):             # <<<<<<<<<<<<<<
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, us[0])
 */
  __pyx_t_1 = (__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_v_inv_root2, __pyx_v_wnorm) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":1160
 *     if blas_t_less_than(inv_root2, wnorm):
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, us[0])             # <<<<<<<<<<<<<<
 *         scal(n, unorm, s, 1)
 *         s[n] = unorm*wnorm
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, (1.0 / __pyx_v_wnorm), __pyx_v_u, (__pyx_v_us[0]));

    /* "scipy/linalg/_decomp_update.pyx":1161
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, us[0])
 *         scal(n, unorm, s, 1)             # <<<<<<<<<<<<<<
 *         s[n] = unorm*wnorm
 *         return 0
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_n, __pyx_v_unorm, __pyx_v_s, 1);

    /* "scipy/linalg/_decomp_update.pyx":1162
 *             scal(m, 1/wnorm, u, us[0])
 *         scal(n, unorm, s, 1)
 *         s[n] = unorm*wnorm             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
    (__pyx_v_s[__pyx_v_n]) = (__pyx_v_unorm * __pyx_v_wnorm);

    /* "scipy/linalg/_decomp_update.pyx":1163
 *         scal(n, unorm, s, 1)
 *         s[n] = unorm*wnorm
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     # if we get here, u does not yet contain the orthogonal vector we are
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":1158
 *     RCOND[0] = rc
 * 
 *     if blas_t_less_than(inv_root2, wnorm):             # <<<<<<<<<<<<<<
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, us[0])
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1168
 *     # looking for. According to Reichel, one reorthogonalization will always
 *     # be enough.
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s+n, 1)
 */
  __pyx_t_1 = (__pyx_v_qisF != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":1170
 *     if qisF:
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s+n, 1)             # <<<<<<<<<<<<<<
 *         else:
 *             gemv(C, m, n, 1, q, m, u, us[0], 0, s+n, 1)
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_m, __pyx_v_n, 1.0, __pyx_v_q, __pyx_v_m, __pyx_v_u, (__pyx_v_us[0]), 0.0, (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":1173
 *         else:
 *             gemv(C, m, n, 1, q, m, u, us[0], 0, s+n, 1)
 *         gemv(N, m, n, -1, q, m, s+n, 1, 1, u, us[0])             # <<<<<<<<<<<<<<
 *     else:
 *         if blas_t is float or blas_t is double:
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_n, -1.0, __pyx_v_q, __pyx_v_m, (__pyx_v_s + __pyx_v_n), 1, 1.0, __pyx_v_u, (__pyx_v_us[0]));

    /* "scipy/linalg/_decomp_update.pyx":1168
 *     # looking for. According to Reichel, one reorthogonalization will always
 *     # be enough.
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s+n, 1)
 */
    goto __pyx_L7;
  }

  /* "scipy/linalg/_decomp_update.pyx":1175
 *         gemv(N, m, n, -1, q, m, s+n, 1, 1, u, us[0])
 *     else:
 *         if blas_t is float or blas_t is double:             # <<<<<<<<<<<<<<
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s+n, 1)
 *         else:
 */
  /*else*/ {

    /* "scipy/linalg/_decomp_update.pyx":1176
 *     else:
 *         if blas_t is float or blas_t is double:
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s+n, 1)             # <<<<<<<<<<<<<<
 *         else:
 *             blas_t_conj(m, u, us)
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_n, __pyx_v_m, 1.0, __pyx_v_q, __pyx_v_n, __pyx_v_u, (__pyx_v_us[0]), 0.0, (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":1182
 *             blas_t_conj(m, u, us)
 *             blas_t_conj(n, s+n, &ss)
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, us[0])             # <<<<<<<<<<<<<<
 * 
 *     wpnorm = nrm2(m, u, us[0])
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_n, __pyx_v_m, -1.0, __pyx_v_q, __pyx_v_n, (__pyx_v_s + __pyx_v_n), 1, 1.0, __pyx_v_u, (__pyx_v_us[0]));
  }
  __pyx_L7:;

  /* "scipy/linalg/_decomp_update.pyx":1184
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, us[0])
 * 
 *     wpnorm = nrm2(m, u, us[0])             # <<<<<<<<<<<<<<
 * 
 *     if blas_t_less_equal(wpnorm, wnorm*inv_root2): # u lies in span(q)
 */
  __pyx_v_wpnorm = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_m, __pyx_v_u, (__pyx_v_us[0]));

  /* "scipy/linalg/_decomp_update.pyx":1186
 *     wpnorm = nrm2(m, u, us[0])
 * 
 *     if blas_t_less_equal(wpnorm, wnorm*inv_root2): # u lies in span(q)             # <<<<<<<<<<<<<<
 *         scal(m, 0, u, us[0])
 *         axpy(n, 1, s, 1, s+n, 1)
 */
  __pyx_t_1 = (__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_equal(__pyx_v_wpnorm, (__pyx_v_wnorm * __pyx_v_inv_root2)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":1187
 * 
 *     if blas_t_less_equal(wpnorm, wnorm*inv_root2): # u lies in span(q)
 *         scal(m, 0, u, us[0])             # <<<<<<<<<<<<<<
 *         axpy(n, 1, s, 1, s+n, 1)
 *         scal(n, unorm, s, 1)
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, 0.0, __pyx_v_u, (__pyx_v_us[0]));

    /* "scipy/linalg/_decomp_update.pyx":1188
 *     if blas_t_less_equal(wpnorm, wnorm*inv_root2): # u lies in span(q)
 *         scal(m, 0, u, us[0])
 *         axpy(n, 1, s, 1, s+n, 1)             # <<<<<<<<<<<<<<
 *         scal(n, unorm, s, 1)
 *         s[n] = 0
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, 1.0, __pyx_v_s, 1, (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":1189
 *         scal(m, 0, u, us[0])
 *         axpy(n, 1, s, 1, s+n, 1)
 *         scal(n, unorm, s, 1)             # <<<<<<<<<<<<<<
 *         s[n] = 0
 *         return 1
 */
    __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_n, __pyx_v_unorm, __pyx_v_s, 1);

    /* "scipy/linalg/_decomp_update.pyx":1190
 *         axpy(n, 1, s, 1, s+n, 1)
 *         scal(n, unorm, s, 1)
 *         s[n] = 0             # <<<<<<<<<<<<<<
 *         return 1
 * 
 */
    (__pyx_v_s[__pyx_v_n]) = 0.0;

    /* "scipy/linalg/_decomp_update.pyx":1191
 *         scal(n, unorm, s, 1)
 *         s[n] = 0
 *         return 1             # <<<<<<<<<<<<<<
 * 
 *     with cython.cdivision(True):
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":1186
 *     wpnorm = nrm2(m, u, us[0])
 * 
 *     if blas_t_less_equal(wpnorm, wnorm*inv_root2): # u lies in span(q)             # <<<<<<<<<<<<<<
 *         scal(m, 0, u, us[0])
 *         axpy(n, 1, s, 1, s+n, 1)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1194
 * 
 *     with cython.cdivision(True):
 *         scal(m, 1/wpnorm, u, us[0])             # <<<<<<<<<<<<<<
 *     axpy(n, 1, s, 1, s+n, 1)
 *     scal(n, unorm, s, 1)
 */
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, (1.0 / __pyx_v_wpnorm), __pyx_v_u, (__pyx_v_us[0]));

  /* "scipy/linalg/_decomp_update.pyx":1195
 *     with cython.cdivision(True):
 *         scal(m, 1/wpnorm, u, us[0])
 *     axpy(n, 1, s, 1, s+n, 1)             # <<<<<<<<<<<<<<
 *     scal(n, unorm, s, 1)
 *     s[n] = wpnorm*unorm
 */
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, 1.0, __pyx_v_s, 1, (__pyx_v_s + __pyx_v_n), 1);

  /* "scipy/linalg/_decomp_update.pyx":1196
 *         scal(m, 1/wpnorm, u, us[0])
 *     axpy(n, 1, s, 1, s+n, 1)
 *     scal(n, unorm, s, 1)             # <<<<<<<<<<<<<<
 *     s[n] = wpnorm*unorm
 *     return 0
 */
  __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_n, __pyx_v_unorm, __pyx_v_s, 1);

  /* "scipy/linalg/_decomp_update.pyx":1197
 *     axpy(n, 1, s, 1, s+n, 1)
 *     scal(n, unorm, s, 1)
 *     s[n] = wpnorm*unorm             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  (__pyx_v_s[__pyx_v_n]) = (__pyx_v_wpnorm * __pyx_v_unorm);

  /* "scipy/linalg/_decomp_update.pyx":1198
 *     scal(n, unorm, s, 1)
 *     s[n] = wpnorm*unorm
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * def _form_qTu(object a, object b):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":1092
 *         index2(r, rs, j, j)[0] = rjj
 * 
 * cdef int reorth(int m, int n, blas_t* q, int* qs, bint qisF, blas_t* u,             # <<<<<<<<<<<<<<
 *                 int* us, blas_t* s, blas_t* RCOND) nogil:
 *     """Given a (m,n) matrix q with orthonormal columns and a (m,) vector u,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.linalg._decomp_update.reorth", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_reorth(int __pyx_v_m, int __pyx_v_n, __pyx_t_float_complex *__pyx_v_q, CYTHON_UNUSED int *__pyx_v_qs, int __pyx_v_qisF, __pyx_t_float_complex *__pyx_v_u, int *__pyx_v_us, __pyx_t_float_complex *__pyx_v_s, __pyx_t_float_complex *__pyx_v_RCOND) {
  __pyx_t_float_complex __pyx_v_unorm;
  __pyx_t_float_complex __pyx_v_snorm;
  __pyx_t_float_complex __pyx_v_wnorm;
  __pyx_t_float_complex __pyx_v_wpnorm;
  __pyx_t_float_complex __pyx_v_sigma_max;
  __pyx_t_float_complex __pyx_v_sigma_min;
  __pyx_t_float_complex __pyx_v_rc;
  char *__pyx_v_T;
  char *__pyx_v_N;
  char *__pyx_v_C;
  int __pyx_v_ss;
  __pyx_t_float_complex __pyx_v_inv_root2;
  int __pyx_r;
  __pyx_t_double_complex __pyx_t_1;
  __pyx_t_double_complex __pyx_t_2;
  __pyx_t_double_complex __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/linalg/_decomp_update.pyx":1109
 *     """
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 *     cdef char* T = 'T'             # <<<<<<<<<<<<<<
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'
 */
  __pyx_v_T = ((char *)"T");

  /* "scipy/linalg/_decomp_update.pyx":1110
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'             # <<<<<<<<<<<<<<
 *     cdef char* C = 'C'
 *     cdef int ss = 1
 */
  __pyx_v_N = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":1111
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'             # <<<<<<<<<<<<<<
 *     cdef int ss = 1
 *     cdef blas_t inv_root2 = NPY_SQRT1_2
 */
  __pyx_v_C = ((char *)"C");

  /* "scipy/linalg/_decomp_update.pyx":1112
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'
 *     cdef int ss = 1             # <<<<<<<<<<<<<<
 *     cdef blas_t inv_root2 = NPY_SQRT1_2
 * 
 */
  __pyx_v_ss = 1;

  /* "scipy/linalg/_decomp_update.pyx":1113
 *     cdef char* C = 'C'
 *     cdef int ss = 1
 *     cdef blas_t inv_root2 = NPY_SQRT1_2             # <<<<<<<<<<<<<<
 * 
 *     # normalize u
 */
  __pyx_v_inv_root2 = __pyx_t_float_complex_from_parts(NPY_SQRT1_2, 0);

  /* "scipy/linalg/_decomp_update.pyx":1116
 * 
 *     # normalize u
 *     unorm = nrm2(m, u, us[0])             # <<<<<<<<<<<<<<
 *     scal(m, 1/unorm, u, us[0])
 * 
 */
  __pyx_v_unorm = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_m, __pyx_v_u, (__pyx_v_us[0]));

  /* "scipy/linalg/_decomp_update.pyx":1117
 *     # normalize u
 *     unorm = nrm2(m, u, us[0])
 *     scal(m, 1/unorm, u, us[0])             # <<<<<<<<<<<<<<
 * 
 *     # decompose u into q's columns.
 */
  __pyx_t_1 = __pyx_t_double_complex_from_parts(1, 0);
  __pyx_t_2 = __pyx_t_double_complex_from_parts(__Pyx_CREAL(__pyx_v_unorm), __Pyx_CIMAG(__pyx_v_unorm));
  if (unlikely(__Pyx_c_is_zero_double(__pyx_t_2))) {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
    #endif
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    #ifdef WITH_THREAD
    __Pyx_PyGILState_Release(__pyx_gilstate_save);
    #endif
    __PYX_ERR(0, 1117, __pyx_L1_error)
  }
  __pyx_t_3 = __Pyx_c_quot_double(__pyx_t_1, __pyx_t_2);
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, __pyx_t_float_complex_from_parts(__Pyx_CREAL(__pyx_t_3), __Pyx_CIMAG(__pyx_t_3)), __pyx_v_u, (__pyx_v_us[0]));

  /* "scipy/linalg/_decomp_update.pyx":1120
 * 
 *     # decompose u into q's columns.
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s, 1)
 */
  __pyx_t_4 = (__pyx_v_qisF != 0);
  if (__pyx_t_4) {

    /* "scipy/linalg/_decomp_update.pyx":1124
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s, 1)
 *         else:
 *             gemv(C, m, n, 1, q, m, u, us[0], 0, s, 1)             # <<<<<<<<<<<<<<
 *     else:
 *         if blas_t is float or blas_t is double:
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_C, __pyx_v_m, __pyx_v_n, __pyx_t_float_complex_from_parts(1, 0), __pyx_v_q, __pyx_v_m, __pyx_v_u, (__pyx_v_us[0]), __pyx_t_float_complex_from_parts(0, 0), __pyx_v_s, 1);

    /* "scipy/linalg/_decomp_update.pyx":1120
 * 
 *     # decompose u into q's columns.
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s, 1)
 */
    goto __pyx_L3;
  }

  /* "scipy/linalg/_decomp_update.pyx":1126
 *             gemv(C, m, n, 1, q, m, u, us[0], 0, s, 1)
 *     else:
 *         if blas_t is float or blas_t is double:             # <<<<<<<<<<<<<<
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s, 1)
 *         else:
 */
  /*else*/ {

    /* "scipy/linalg/_decomp_update.pyx":1129
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s, 1)
 *         else:
 *             blas_t_conj(m, u, us)             # <<<<<<<<<<<<<<
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s, 1)
 *             blas_t_conj(m, u, us)
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_m, __pyx_v_u, __pyx_v_us);

    /* "scipy/linalg/_decomp_update.pyx":1130
 *         else:
 *             blas_t_conj(m, u, us)
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s, 1)             # <<<<<<<<<<<<<<
 *             blas_t_conj(m, u, us)
 *             blas_t_conj(n, s, &ss)
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_n, __pyx_v_m, __pyx_t_float_complex_from_parts(1, 0), __pyx_v_q, __pyx_v_n, __pyx_v_u, (__pyx_v_us[0]), __pyx_t_float_complex_from_parts(0, 0), __pyx_v_s, 1);

    /* "scipy/linalg/_decomp_update.pyx":1131
 *             blas_t_conj(m, u, us)
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s, 1)
 *             blas_t_conj(m, u, us)             # <<<<<<<<<<<<<<
 *             blas_t_conj(n, s, &ss)
 * 
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_m, __pyx_v_u, __pyx_v_us);

    /* "scipy/linalg/_decomp_update.pyx":1132
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s, 1)
 *             blas_t_conj(m, u, us)
 *             blas_t_conj(n, s, &ss)             # <<<<<<<<<<<<<<
 * 
 *     # sigma_max is the largest singular value of q augmented with u/unorm
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_n, __pyx_v_s, (&__pyx_v_ss));
  }
  __pyx_L3:;

  /* "scipy/linalg/_decomp_update.pyx":1135
 * 
 *     # sigma_max is the largest singular value of q augmented with u/unorm
 *     snorm = nrm2(n, s, 1)             # <<<<<<<<<<<<<<
 *     sigma_max = blas_t_sqrt(1 + snorm)
 * 
 */
  __pyx_v_snorm = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_n, __pyx_v_s, 1);

  /* "scipy/linalg/_decomp_update.pyx":1136
 *     # sigma_max is the largest singular value of q augmented with u/unorm
 *     snorm = nrm2(n, s, 1)
 *     sigma_max = blas_t_sqrt(1 + snorm)             # <<<<<<<<<<<<<<
 * 
 *     # make u be the part of u that is not in span(q)
 */
  __pyx_v_sigma_max = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_sqrt(__Pyx_c_sum_float(__pyx_t_float_complex_from_parts(1, 0), __pyx_v_snorm));

  /* "scipy/linalg/_decomp_update.pyx":1140
 *     # make u be the part of u that is not in span(q)
 *     # i.e. u -= q.dot(s)
 *     if qisF:             # <<<<<<<<<<<<<<
 *         gemv(N, m, n, -1, q, m, s, 1, 1, u, us[0])
 *     else:
 */
  __pyx_t_4 = (__pyx_v_qisF != 0);
  if (__pyx_t_4) {

    /* "scipy/linalg/_decomp_update.pyx":1141
 *     # i.e. u -= q.dot(s)
 *     if qisF:
 *         gemv(N, m, n, -1, q, m, s, 1, 1, u, us[0])             # <<<<<<<<<<<<<<
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, us[0])
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_n, __pyx_t_float_complex_from_parts(-1L, 0), __pyx_v_q, __pyx_v_m, __pyx_v_s, 1, __pyx_t_float_complex_from_parts(1, 0), __pyx_v_u, (__pyx_v_us[0]));

    /* "scipy/linalg/_decomp_update.pyx":1140
 *     # make u be the part of u that is not in span(q)
 *     # i.e. u -= q.dot(s)
 *     if qisF:             # <<<<<<<<<<<<<<
 *         gemv(N, m, n, -1, q, m, s, 1, 1, u, us[0])
 *     else:
 */
    goto __pyx_L4;
  }

  /* "scipy/linalg/_decomp_update.pyx":1143
 *         gemv(N, m, n, -1, q, m, s, 1, 1, u, us[0])
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, us[0])             # <<<<<<<<<<<<<<
 *     wnorm = nrm2(m, u, us[0])
 * 
 */
  /*else*/ {
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_n, __pyx_v_m, __pyx_t_float_complex_from_parts(-1L, 0), __pyx_v_q, __pyx_v_n, __pyx_v_s, 1, __pyx_t_float_complex_from_parts(1, 0), __pyx_v_u, (__pyx_v_us[0]));
  }
  __pyx_L4:;

  /* "scipy/linalg/_decomp_update.pyx":1144
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, us[0])
 *     wnorm = nrm2(m, u, us[0])             # <<<<<<<<<<<<<<
 * 
 *     # sigma_min is the smallest singular value of q augmented with u/unorm
 */
  __pyx_v_wnorm = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_m, __pyx_v_u, (__pyx_v_us[0]));

  /* "scipy/linalg/_decomp_update.pyx":1149
 *     # the others are == 1, since q is orthonormal.
 *     with cython.cdivision(True):
 *         sigma_min = wnorm / sigma_max             # <<<<<<<<<<<<<<
 *         rc = sigma_min / sigma_max
 * 
 */
  __pyx_v_sigma_min = __Pyx_c_quot_float(__pyx_v_wnorm, __pyx_v_sigma_max);

  /* "scipy/linalg/_decomp_update.pyx":1150
 *     with cython.cdivision(True):
 *         sigma_min = wnorm / sigma_max
 *         rc = sigma_min / sigma_max             # <<<<<<<<<<<<<<
 * 
 *     # check the conditioning of the problem.
 */
  __pyx_v_rc = __Pyx_c_quot_float(__pyx_v_sigma_min, __pyx_v_sigma_max);

  /* "scipy/linalg/_decomp_update.pyx":1153
 * 
 *     # check the conditioning of the problem.
 *     if blas_t_less_than(rc, RCOND[0]):             # <<<<<<<<<<<<<<
 *         RCOND[0] = rc
 *         return 2
 */
  __pyx_t_4 = (__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_v_rc, (__pyx_v_RCOND[0])) != 0);
  if (__pyx_t_4) {

    /* "scipy/linalg/_decomp_update.pyx":1154
 *     # check the conditioning of the problem.
 *     if blas_t_less_than(rc, RCOND[0]):
 *         RCOND[0] = rc             # <<<<<<<<<<<<<<
 *         return 2
 *     RCOND[0] = rc
 */
    (__pyx_v_RCOND[0]) = __pyx_v_rc;

    /* "scipy/linalg/_decomp_update.pyx":1155
 *     if blas_t_less_than(rc, RCOND[0]):
 *         RCOND[0] = rc
 *         return 2             # <<<<<<<<<<<<<<
 *     RCOND[0] = rc
 * 
 */
    __pyx_r = 2;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":1153
 * 
 *     # check the conditioning of the problem.
 *     if blas_t_less_than(rc, RCOND[0]):             # <<<<<<<<<<<<<<
 *         RCOND[0] = rc
 *         return 2
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1156
 *         RCOND[0] = rc
 *         return 2
 *     RCOND[0] = rc             # <<<<<<<<<<<<<<
 * 
 *     if blas_t_less_than(inv_root2, wnorm):
 */
  (__pyx_v_RCOND[0]) = __pyx_v_rc;

  /* "scipy/linalg/_decomp_update.pyx":1158
 *     RCOND[0] = rc
 * 
 *     if blas_t_less_than(inv_root2, wnorm):             # <<<<<<<<<<<<<<
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, us[0])
 */
  __pyx_t_4 = (__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_v_inv_root2, __pyx_v_wnorm) != 0);
  if (__pyx_t_4) {

    /* "scipy/linalg/_decomp_update.pyx":1160
 *     if blas_t_less_than(inv_root2, wnorm):
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, us[0])             # <<<<<<<<<<<<<<
 *         scal(n, unorm, s, 1)
 *         s[n] = unorm*wnorm
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, __Pyx_c_quot_float(__pyx_t_float_complex_from_parts(1, 0), __pyx_v_wnorm), __pyx_v_u, (__pyx_v_us[0]));

    /* "scipy/linalg/_decomp_update.pyx":1161
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, us[0])
 *         scal(n, unorm, s, 1)             # <<<<<<<<<<<<<<
 *         s[n] = unorm*wnorm
 *         return 0
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_n, __pyx_v_unorm, __pyx_v_s, 1);

    /* "scipy/linalg/_decomp_update.pyx":1162
 *             scal(m, 1/wnorm, u, us[0])
 *         scal(n, unorm, s, 1)
 *         s[n] = unorm*wnorm             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
    (__pyx_v_s[__pyx_v_n]) = __Pyx_c_prod_float(__pyx_v_unorm, __pyx_v_wnorm);

    /* "scipy/linalg/_decomp_update.pyx":1163
 *         scal(n, unorm, s, 1)
 *         s[n] = unorm*wnorm
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     # if we get here, u does not yet contain the orthogonal vector we are
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":1158
 *     RCOND[0] = rc
 * 
 *     if blas_t_less_than(inv_root2, wnorm):             # <<<<<<<<<<<<<<
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, us[0])
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1168
 *     # looking for. According to Reichel, one reorthogonalization will always
 *     # be enough.
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s+n, 1)
 */
  __pyx_t_4 = (__pyx_v_qisF != 0);
  if (__pyx_t_4) {

    /* "scipy/linalg/_decomp_update.pyx":1172
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s+n, 1)
 *         else:
 *             gemv(C, m, n, 1, q, m, u, us[0], 0, s+n, 1)             # <<<<<<<<<<<<<<
 *         gemv(N, m, n, -1, q, m, s+n, 1, 1, u, us[0])
 *     else:
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_C, __pyx_v_m, __pyx_v_n, __pyx_t_float_complex_from_parts(1, 0), __pyx_v_q, __pyx_v_m, __pyx_v_u, (__pyx_v_us[0]), __pyx_t_float_complex_from_parts(0, 0), (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":1173
 *         else:
 *             gemv(C, m, n, 1, q, m, u, us[0], 0, s+n, 1)
 *         gemv(N, m, n, -1, q, m, s+n, 1, 1, u, us[0])             # <<<<<<<<<<<<<<
 *     else:
 *         if blas_t is float or blas_t is double:
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_n, __pyx_t_float_complex_from_parts(-1L, 0), __pyx_v_q, __pyx_v_m, (__pyx_v_s + __pyx_v_n), 1, __pyx_t_float_complex_from_parts(1, 0), __pyx_v_u, (__pyx_v_us[0]));

    /* "scipy/linalg/_decomp_update.pyx":1168
 *     # looking for. According to Reichel, one reorthogonalization will always
 *     # be enough.
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s+n, 1)
 */
    goto __pyx_L7;
  }

  /* "scipy/linalg/_decomp_update.pyx":1175
 *         gemv(N, m, n, -1, q, m, s+n, 1, 1, u, us[0])
 *     else:
 *         if blas_t is float or blas_t is double:             # <<<<<<<<<<<<<<
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s+n, 1)
 *         else:
 */
  /*else*/ {

    /* "scipy/linalg/_decomp_update.pyx":1178
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s+n, 1)
 *         else:
 *             blas_t_conj(m, u, us)             # <<<<<<<<<<<<<<
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s+n, 1)
 *             blas_t_conj(m, u, us)
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_m, __pyx_v_u, __pyx_v_us);

    /* "scipy/linalg/_decomp_update.pyx":1179
 *         else:
 *             blas_t_conj(m, u, us)
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s+n, 1)             # <<<<<<<<<<<<<<
 *             blas_t_conj(m, u, us)
 *             blas_t_conj(n, s+n, &ss)
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_n, __pyx_v_m, __pyx_t_float_complex_from_parts(1, 0), __pyx_v_q, __pyx_v_n, __pyx_v_u, (__pyx_v_us[0]), __pyx_t_float_complex_from_parts(0, 0), (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":1180
 *             blas_t_conj(m, u, us)
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s+n, 1)
 *             blas_t_conj(m, u, us)             # <<<<<<<<<<<<<<
 *             blas_t_conj(n, s+n, &ss)
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, us[0])
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_m, __pyx_v_u, __pyx_v_us);

    /* "scipy/linalg/_decomp_update.pyx":1181
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s+n, 1)
 *             blas_t_conj(m, u, us)
 *             blas_t_conj(n, s+n, &ss)             # <<<<<<<<<<<<<<
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, us[0])
 * 
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_n, (__pyx_v_s + __pyx_v_n), (&__pyx_v_ss));

    /* "scipy/linalg/_decomp_update.pyx":1182
 *             blas_t_conj(m, u, us)
 *             blas_t_conj(n, s+n, &ss)
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, us[0])             # <<<<<<<<<<<<<<
 * 
 *     wpnorm = nrm2(m, u, us[0])
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_n, __pyx_v_m, __pyx_t_float_complex_from_parts(-1L, 0), __pyx_v_q, __pyx_v_n, (__pyx_v_s + __pyx_v_n), 1, __pyx_t_float_complex_from_parts(1, 0), __pyx_v_u, (__pyx_v_us[0]));
  }
  __pyx_L7:;

  /* "scipy/linalg/_decomp_update.pyx":1184
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, us[0])
 * 
 *     wpnorm = nrm2(m, u, us[0])             # <<<<<<<<<<<<<<
 * 
 *     if blas_t_less_equal(wpnorm, wnorm*inv_root2): # u lies in span(q)
 */
  __pyx_v_wpnorm = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_m, __pyx_v_u, (__pyx_v_us[0]));

  /* "scipy/linalg/_decomp_update.pyx":1186
 *     wpnorm = nrm2(m, u, us[0])
 * 
 *     if blas_t_less_equal(wpnorm, wnorm*inv_root2): # u lies in span(q)             # <<<<<<<<<<<<<<
 *         scal(m, 0, u, us[0])
 *         axpy(n, 1, s, 1, s+n, 1)
 */
  __pyx_t_4 = (__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_equal(__pyx_v_wpnorm, __Pyx_c_prod_float(__pyx_v_wnorm, __pyx_v_inv_root2)) != 0);
  if (__pyx_t_4) {

    /* "scipy/linalg/_decomp_update.pyx":1187
 * 
 *     if blas_t_less_equal(wpnorm, wnorm*inv_root2): # u lies in span(q)
 *         scal(m, 0, u, us[0])             # <<<<<<<<<<<<<<
 *         axpy(n, 1, s, 1, s+n, 1)
 *         scal(n, unorm, s, 1)
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, __pyx_t_float_complex_from_parts(0, 0), __pyx_v_u, (__pyx_v_us[0]));

    /* "scipy/linalg/_decomp_update.pyx":1188
 *     if blas_t_less_equal(wpnorm, wnorm*inv_root2): # u lies in span(q)
 *         scal(m, 0, u, us[0])
 *         axpy(n, 1, s, 1, s+n, 1)             # <<<<<<<<<<<<<<
 *         scal(n, unorm, s, 1)
 *         s[n] = 0
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, __pyx_t_float_complex_from_parts(1, 0), __pyx_v_s, 1, (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":1189
 *         scal(m, 0, u, us[0])
 *         axpy(n, 1, s, 1, s+n, 1)
 *         scal(n, unorm, s, 1)             # <<<<<<<<<<<<<<
 *         s[n] = 0
 *         return 1
 */
    __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_n, __pyx_v_unorm, __pyx_v_s, 1);

    /* "scipy/linalg/_decomp_update.pyx":1190
 *         axpy(n, 1, s, 1, s+n, 1)
 *         scal(n, unorm, s, 1)
 *         s[n] = 0             # <<<<<<<<<<<<<<
 *         return 1
 * 
 */
    (__pyx_v_s[__pyx_v_n]) = __pyx_t_float_complex_from_parts(0, 0);

    /* "scipy/linalg/_decomp_update.pyx":1191
 *         scal(n, unorm, s, 1)
 *         s[n] = 0
 *         return 1             # <<<<<<<<<<<<<<
 * 
 *     with cython.cdivision(True):
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":1186
 *     wpnorm = nrm2(m, u, us[0])
 * 
 *     if blas_t_less_equal(wpnorm, wnorm*inv_root2): # u lies in span(q)             # <<<<<<<<<<<<<<
 *         scal(m, 0, u, us[0])
 *         axpy(n, 1, s, 1, s+n, 1)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1194
 * 
 *     with cython.cdivision(True):
 *         scal(m, 1/wpnorm, u, us[0])             # <<<<<<<<<<<<<<
 *     axpy(n, 1, s, 1, s+n, 1)
 *     scal(n, unorm, s, 1)
 */
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, __Pyx_c_quot_float(__pyx_t_float_complex_from_parts(1, 0), __pyx_v_wpnorm), __pyx_v_u, (__pyx_v_us[0]));

  /* "scipy/linalg/_decomp_update.pyx":1195
 *     with cython.cdivision(True):
 *         scal(m, 1/wpnorm, u, us[0])
 *     axpy(n, 1, s, 1, s+n, 1)             # <<<<<<<<<<<<<<
 *     scal(n, unorm, s, 1)
 *     s[n] = wpnorm*unorm
 */
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, __pyx_t_float_complex_from_parts(1, 0), __pyx_v_s, 1, (__pyx_v_s + __pyx_v_n), 1);

  /* "scipy/linalg/_decomp_update.pyx":1196
 *         scal(m, 1/wpnorm, u, us[0])
 *     axpy(n, 1, s, 1, s+n, 1)
 *     scal(n, unorm, s, 1)             # <<<<<<<<<<<<<<
 *     s[n] = wpnorm*unorm
 *     return 0
 */
  __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_n, __pyx_v_unorm, __pyx_v_s, 1);

  /* "scipy/linalg/_decomp_update.pyx":1197
 *     axpy(n, 1, s, 1, s+n, 1)
 *     scal(n, unorm, s, 1)
 *     s[n] = wpnorm*unorm             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  (__pyx_v_s[__pyx_v_n]) = __Pyx_c_prod_float(__pyx_v_wpnorm, __pyx_v_unorm);

  /* "scipy/linalg/_decomp_update.pyx":1198
 *     scal(n, unorm, s, 1)
 *     s[n] = wpnorm*unorm
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * def _form_qTu(object a, object b):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":1092
 *         index2(r, rs, j, j)[0] = rjj
 * 
 * cdef int reorth(int m, int n, blas_t* q, int* qs, bint qisF, blas_t* u,             # <<<<<<<<<<<<<<
 *                 int* us, blas_t* s, blas_t* RCOND) nogil:
 *     """Given a (m,n) matrix q with orthonormal columns and a (m,) vector u,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.linalg._decomp_update.reorth", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

static int __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_reorth(int __pyx_v_m, int __pyx_v_n, __pyx_t_double_complex *__pyx_v_q, CYTHON_UNUSED int *__pyx_v_qs, int __pyx_v_qisF, __pyx_t_double_complex *__pyx_v_u, int *__pyx_v_us, __pyx_t_double_complex *__pyx_v_s, __pyx_t_double_complex *__pyx_v_RCOND) {
  __pyx_t_double_complex __pyx_v_unorm;
  __pyx_t_double_complex __pyx_v_snorm;
  __pyx_t_double_complex __pyx_v_wnorm;
  __pyx_t_double_complex __pyx_v_wpnorm;
  __pyx_t_double_complex __pyx_v_sigma_max;
  __pyx_t_double_complex __pyx_v_sigma_min;
  __pyx_t_double_complex __pyx_v_rc;
  char *__pyx_v_T;
  char *__pyx_v_N;
  char *__pyx_v_C;
  int __pyx_v_ss;
  __pyx_t_double_complex __pyx_v_inv_root2;
  int __pyx_r;
  __pyx_t_double_complex __pyx_t_1;
  int __pyx_t_2;

  /* "scipy/linalg/_decomp_update.pyx":1109
 *     """
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 *     cdef char* T = 'T'             # <<<<<<<<<<<<<<
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'
 */
  __pyx_v_T = ((char *)"T");

  /* "scipy/linalg/_decomp_update.pyx":1110
 *     cdef blas_t unorm, snorm, wnorm, wpnorm, sigma_max, sigma_min, rc
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'             # <<<<<<<<<<<<<<
 *     cdef char* C = 'C'
 *     cdef int ss = 1
 */
  __pyx_v_N = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":1111
 *     cdef char* T = 'T'
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'             # <<<<<<<<<<<<<<
 *     cdef int ss = 1
 *     cdef blas_t inv_root2 = NPY_SQRT1_2
 */
  __pyx_v_C = ((char *)"C");

  /* "scipy/linalg/_decomp_update.pyx":1112
 *     cdef char* N = 'N'
 *     cdef char* C = 'C'
 *     cdef int ss = 1             # <<<<<<<<<<<<<<
 *     cdef blas_t inv_root2 = NPY_SQRT1_2
 * 
 */
  __pyx_v_ss = 1;

  /* "scipy/linalg/_decomp_update.pyx":1113
 *     cdef char* C = 'C'
 *     cdef int ss = 1
 *     cdef blas_t inv_root2 = NPY_SQRT1_2             # <<<<<<<<<<<<<<
 * 
 *     # normalize u
 */
  __pyx_v_inv_root2 = __pyx_t_double_complex_from_parts(NPY_SQRT1_2, 0);

  /* "scipy/linalg/_decomp_update.pyx":1116
 * 
 *     # normalize u
 *     unorm = nrm2(m, u, us[0])             # <<<<<<<<<<<<<<
 *     scal(m, 1/unorm, u, us[0])
 * 
 */
  __pyx_v_unorm = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_m, __pyx_v_u, (__pyx_v_us[0]));

  /* "scipy/linalg/_decomp_update.pyx":1117
 *     # normalize u
 *     unorm = nrm2(m, u, us[0])
 *     scal(m, 1/unorm, u, us[0])             # <<<<<<<<<<<<<<
 * 
 *     # decompose u into q's columns.
 */
  __pyx_t_1 = __pyx_t_double_complex_from_parts(1, 0);
  if (unlikely(__Pyx_c_is_zero_double(__pyx_v_unorm))) {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
    #endif
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    #ifdef WITH_THREAD
    __Pyx_PyGILState_Release(__pyx_gilstate_save);
    #endif
    __PYX_ERR(0, 1117, __pyx_L1_error)
  }
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, __Pyx_c_quot_double(__pyx_t_1, __pyx_v_unorm), __pyx_v_u, (__pyx_v_us[0]));

  /* "scipy/linalg/_decomp_update.pyx":1120
 * 
 *     # decompose u into q's columns.
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s, 1)
 */
  __pyx_t_2 = (__pyx_v_qisF != 0);
  if (__pyx_t_2) {

    /* "scipy/linalg/_decomp_update.pyx":1124
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s, 1)
 *         else:
 *             gemv(C, m, n, 1, q, m, u, us[0], 0, s, 1)             # <<<<<<<<<<<<<<
 *     else:
 *         if blas_t is float or blas_t is double:
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_C, __pyx_v_m, __pyx_v_n, __pyx_t_double_complex_from_parts(1, 0), __pyx_v_q, __pyx_v_m, __pyx_v_u, (__pyx_v_us[0]), __pyx_t_double_complex_from_parts(0, 0), __pyx_v_s, 1);

    /* "scipy/linalg/_decomp_update.pyx":1120
 * 
 *     # decompose u into q's columns.
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s, 1)
 */
    goto __pyx_L3;
  }

  /* "scipy/linalg/_decomp_update.pyx":1126
 *             gemv(C, m, n, 1, q, m, u, us[0], 0, s, 1)
 *     else:
 *         if blas_t is float or blas_t is double:             # <<<<<<<<<<<<<<
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s, 1)
 *         else:
 */
  /*else*/ {

    /* "scipy/linalg/_decomp_update.pyx":1129
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s, 1)
 *         else:
 *             blas_t_conj(m, u, us)             # <<<<<<<<<<<<<<
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s, 1)
 *             blas_t_conj(m, u, us)
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_m, __pyx_v_u, __pyx_v_us);

    /* "scipy/linalg/_decomp_update.pyx":1130
 *         else:
 *             blas_t_conj(m, u, us)
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s, 1)             # <<<<<<<<<<<<<<
 *             blas_t_conj(m, u, us)
 *             blas_t_conj(n, s, &ss)
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_n, __pyx_v_m, __pyx_t_double_complex_from_parts(1, 0), __pyx_v_q, __pyx_v_n, __pyx_v_u, (__pyx_v_us[0]), __pyx_t_double_complex_from_parts(0, 0), __pyx_v_s, 1);

    /* "scipy/linalg/_decomp_update.pyx":1131
 *             blas_t_conj(m, u, us)
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s, 1)
 *             blas_t_conj(m, u, us)             # <<<<<<<<<<<<<<
 *             blas_t_conj(n, s, &ss)
 * 
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_m, __pyx_v_u, __pyx_v_us);

    /* "scipy/linalg/_decomp_update.pyx":1132
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s, 1)
 *             blas_t_conj(m, u, us)
 *             blas_t_conj(n, s, &ss)             # <<<<<<<<<<<<<<
 * 
 *     # sigma_max is the largest singular value of q augmented with u/unorm
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_n, __pyx_v_s, (&__pyx_v_ss));
  }
  __pyx_L3:;

  /* "scipy/linalg/_decomp_update.pyx":1135
 * 
 *     # sigma_max is the largest singular value of q augmented with u/unorm
 *     snorm = nrm2(n, s, 1)             # <<<<<<<<<<<<<<
 *     sigma_max = blas_t_sqrt(1 + snorm)
 * 
 */
  __pyx_v_snorm = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_n, __pyx_v_s, 1);

  /* "scipy/linalg/_decomp_update.pyx":1136
 *     # sigma_max is the largest singular value of q augmented with u/unorm
 *     snorm = nrm2(n, s, 1)
 *     sigma_max = blas_t_sqrt(1 + snorm)             # <<<<<<<<<<<<<<
 * 
 *     # make u be the part of u that is not in span(q)
 */
  __pyx_v_sigma_max = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_sqrt(__Pyx_c_sum_double(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_snorm));

  /* "scipy/linalg/_decomp_update.pyx":1140
 *     # make u be the part of u that is not in span(q)
 *     # i.e. u -= q.dot(s)
 *     if qisF:             # <<<<<<<<<<<<<<
 *         gemv(N, m, n, -1, q, m, s, 1, 1, u, us[0])
 *     else:
 */
  __pyx_t_2 = (__pyx_v_qisF != 0);
  if (__pyx_t_2) {

    /* "scipy/linalg/_decomp_update.pyx":1141
 *     # i.e. u -= q.dot(s)
 *     if qisF:
 *         gemv(N, m, n, -1, q, m, s, 1, 1, u, us[0])             # <<<<<<<<<<<<<<
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, us[0])
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_n, __pyx_t_double_complex_from_parts(-1L, 0), __pyx_v_q, __pyx_v_m, __pyx_v_s, 1, __pyx_t_double_complex_from_parts(1, 0), __pyx_v_u, (__pyx_v_us[0]));

    /* "scipy/linalg/_decomp_update.pyx":1140
 *     # make u be the part of u that is not in span(q)
 *     # i.e. u -= q.dot(s)
 *     if qisF:             # <<<<<<<<<<<<<<
 *         gemv(N, m, n, -1, q, m, s, 1, 1, u, us[0])
 *     else:
 */
    goto __pyx_L4;
  }

  /* "scipy/linalg/_decomp_update.pyx":1143
 *         gemv(N, m, n, -1, q, m, s, 1, 1, u, us[0])
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, us[0])             # <<<<<<<<<<<<<<
 *     wnorm = nrm2(m, u, us[0])
 * 
 */
  /*else*/ {
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_n, __pyx_v_m, __pyx_t_double_complex_from_parts(-1L, 0), __pyx_v_q, __pyx_v_n, __pyx_v_s, 1, __pyx_t_double_complex_from_parts(1, 0), __pyx_v_u, (__pyx_v_us[0]));
  }
  __pyx_L4:;

  /* "scipy/linalg/_decomp_update.pyx":1144
 *     else:
 *         gemv(T, n, m, -1, q, n, s, 1, 1, u, us[0])
 *     wnorm = nrm2(m, u, us[0])             # <<<<<<<<<<<<<<
 * 
 *     # sigma_min is the smallest singular value of q augmented with u/unorm
 */
  __pyx_v_wnorm = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_m, __pyx_v_u, (__pyx_v_us[0]));

  /* "scipy/linalg/_decomp_update.pyx":1149
 *     # the others are == 1, since q is orthonormal.
 *     with cython.cdivision(True):
 *         sigma_min = wnorm / sigma_max             # <<<<<<<<<<<<<<
 *         rc = sigma_min / sigma_max
 * 
 */
  __pyx_v_sigma_min = __Pyx_c_quot_double(__pyx_v_wnorm, __pyx_v_sigma_max);

  /* "scipy/linalg/_decomp_update.pyx":1150
 *     with cython.cdivision(True):
 *         sigma_min = wnorm / sigma_max
 *         rc = sigma_min / sigma_max             # <<<<<<<<<<<<<<
 * 
 *     # check the conditioning of the problem.
 */
  __pyx_v_rc = __Pyx_c_quot_double(__pyx_v_sigma_min, __pyx_v_sigma_max);

  /* "scipy/linalg/_decomp_update.pyx":1153
 * 
 *     # check the conditioning of the problem.
 *     if blas_t_less_than(rc, RCOND[0]):             # <<<<<<<<<<<<<<
 *         RCOND[0] = rc
 *         return 2
 */
  __pyx_t_2 = (__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_v_rc, (__pyx_v_RCOND[0])) != 0);
  if (__pyx_t_2) {

    /* "scipy/linalg/_decomp_update.pyx":1154
 *     # check the conditioning of the problem.
 *     if blas_t_less_than(rc, RCOND[0]):
 *         RCOND[0] = rc             # <<<<<<<<<<<<<<
 *         return 2
 *     RCOND[0] = rc
 */
    (__pyx_v_RCOND[0]) = __pyx_v_rc;

    /* "scipy/linalg/_decomp_update.pyx":1155
 *     if blas_t_less_than(rc, RCOND[0]):
 *         RCOND[0] = rc
 *         return 2             # <<<<<<<<<<<<<<
 *     RCOND[0] = rc
 * 
 */
    __pyx_r = 2;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":1153
 * 
 *     # check the conditioning of the problem.
 *     if blas_t_less_than(rc, RCOND[0]):             # <<<<<<<<<<<<<<
 *         RCOND[0] = rc
 *         return 2
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1156
 *         RCOND[0] = rc
 *         return 2
 *     RCOND[0] = rc             # <<<<<<<<<<<<<<
 * 
 *     if blas_t_less_than(inv_root2, wnorm):
 */
  (__pyx_v_RCOND[0]) = __pyx_v_rc;

  /* "scipy/linalg/_decomp_update.pyx":1158
 *     RCOND[0] = rc
 * 
 *     if blas_t_less_than(inv_root2, wnorm):             # <<<<<<<<<<<<<<
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, us[0])
 */
  __pyx_t_2 = (__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_than(__pyx_v_inv_root2, __pyx_v_wnorm) != 0);
  if (__pyx_t_2) {

    /* "scipy/linalg/_decomp_update.pyx":1160
 *     if blas_t_less_than(inv_root2, wnorm):
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, us[0])             # <<<<<<<<<<<<<<
 *         scal(n, unorm, s, 1)
 *         s[n] = unorm*wnorm
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, __Pyx_c_quot_double(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_wnorm), __pyx_v_u, (__pyx_v_us[0]));

    /* "scipy/linalg/_decomp_update.pyx":1161
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, us[0])
 *         scal(n, unorm, s, 1)             # <<<<<<<<<<<<<<
 *         s[n] = unorm*wnorm
 *         return 0
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_n, __pyx_v_unorm, __pyx_v_s, 1);

    /* "scipy/linalg/_decomp_update.pyx":1162
 *             scal(m, 1/wnorm, u, us[0])
 *         scal(n, unorm, s, 1)
 *         s[n] = unorm*wnorm             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
    (__pyx_v_s[__pyx_v_n]) = __Pyx_c_prod_double(__pyx_v_unorm, __pyx_v_wnorm);

    /* "scipy/linalg/_decomp_update.pyx":1163
 *         scal(n, unorm, s, 1)
 *         s[n] = unorm*wnorm
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     # if we get here, u does not yet contain the orthogonal vector we are
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":1158
 *     RCOND[0] = rc
 * 
 *     if blas_t_less_than(inv_root2, wnorm):             # <<<<<<<<<<<<<<
 *         with cython.cdivision(True):
 *             scal(m, 1/wnorm, u, us[0])
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1168
 *     # looking for. According to Reichel, one reorthogonalization will always
 *     # be enough.
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s+n, 1)
 */
  __pyx_t_2 = (__pyx_v_qisF != 0);
  if (__pyx_t_2) {

    /* "scipy/linalg/_decomp_update.pyx":1172
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s+n, 1)
 *         else:
 *             gemv(C, m, n, 1, q, m, u, us[0], 0, s+n, 1)             # <<<<<<<<<<<<<<
 *         gemv(N, m, n, -1, q, m, s+n, 1, 1, u, us[0])
 *     else:
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_C, __pyx_v_m, __pyx_v_n, __pyx_t_double_complex_from_parts(1, 0), __pyx_v_q, __pyx_v_m, __pyx_v_u, (__pyx_v_us[0]), __pyx_t_double_complex_from_parts(0, 0), (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":1173
 *         else:
 *             gemv(C, m, n, 1, q, m, u, us[0], 0, s+n, 1)
 *         gemv(N, m, n, -1, q, m, s+n, 1, 1, u, us[0])             # <<<<<<<<<<<<<<
 *     else:
 *         if blas_t is float or blas_t is double:
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_n, __pyx_t_double_complex_from_parts(-1L, 0), __pyx_v_q, __pyx_v_m, (__pyx_v_s + __pyx_v_n), 1, __pyx_t_double_complex_from_parts(1, 0), __pyx_v_u, (__pyx_v_us[0]));

    /* "scipy/linalg/_decomp_update.pyx":1168
 *     # looking for. According to Reichel, one reorthogonalization will always
 *     # be enough.
 *     if qisF:             # <<<<<<<<<<<<<<
 *         if blas_t is float or blas_t is double:
 *             gemv(T, m, n, 1, q, m, u, us[0], 0, s+n, 1)
 */
    goto __pyx_L7;
  }

  /* "scipy/linalg/_decomp_update.pyx":1175
 *         gemv(N, m, n, -1, q, m, s+n, 1, 1, u, us[0])
 *     else:
 *         if blas_t is float or blas_t is double:             # <<<<<<<<<<<<<<
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s+n, 1)
 *         else:
 */
  /*else*/ {

    /* "scipy/linalg/_decomp_update.pyx":1178
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s+n, 1)
 *         else:
 *             blas_t_conj(m, u, us)             # <<<<<<<<<<<<<<
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s+n, 1)
 *             blas_t_conj(m, u, us)
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_m, __pyx_v_u, __pyx_v_us);

    /* "scipy/linalg/_decomp_update.pyx":1179
 *         else:
 *             blas_t_conj(m, u, us)
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s+n, 1)             # <<<<<<<<<<<<<<
 *             blas_t_conj(m, u, us)
 *             blas_t_conj(n, s+n, &ss)
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_n, __pyx_v_m, __pyx_t_double_complex_from_parts(1, 0), __pyx_v_q, __pyx_v_n, __pyx_v_u, (__pyx_v_us[0]), __pyx_t_double_complex_from_parts(0, 0), (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":1180
 *             blas_t_conj(m, u, us)
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s+n, 1)
 *             blas_t_conj(m, u, us)             # <<<<<<<<<<<<<<
 *             blas_t_conj(n, s+n, &ss)
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, us[0])
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_m, __pyx_v_u, __pyx_v_us);

    /* "scipy/linalg/_decomp_update.pyx":1181
 *             gemv(N, n, m, 1, q, n, u, us[0], 0, s+n, 1)
 *             blas_t_conj(m, u, us)
 *             blas_t_conj(n, s+n, &ss)             # <<<<<<<<<<<<<<
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, us[0])
 * 
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_n, (__pyx_v_s + __pyx_v_n), (&__pyx_v_ss));

    /* "scipy/linalg/_decomp_update.pyx":1182
 *             blas_t_conj(m, u, us)
 *             blas_t_conj(n, s+n, &ss)
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, us[0])             # <<<<<<<<<<<<<<
 * 
 *     wpnorm = nrm2(m, u, us[0])
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_n, __pyx_v_m, __pyx_t_double_complex_from_parts(-1L, 0), __pyx_v_q, __pyx_v_n, (__pyx_v_s + __pyx_v_n), 1, __pyx_t_double_complex_from_parts(1, 0), __pyx_v_u, (__pyx_v_us[0]));
  }
  __pyx_L7:;

  /* "scipy/linalg/_decomp_update.pyx":1184
 *         gemv(T, n, m, -1, q, n, s+n, 1, 1, u, us[0])
 * 
 *     wpnorm = nrm2(m, u, us[0])             # <<<<<<<<<<<<<<
 * 
 *     if blas_t_less_equal(wpnorm, wnorm*inv_root2): # u lies in span(q)
 */
  __pyx_v_wpnorm = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_nrm2(__pyx_v_m, __pyx_v_u, (__pyx_v_us[0]));

  /* "scipy/linalg/_decomp_update.pyx":1186
 *     wpnorm = nrm2(m, u, us[0])
 * 
 *     if blas_t_less_equal(wpnorm, wnorm*inv_root2): # u lies in span(q)             # <<<<<<<<<<<<<<
 *         scal(m, 0, u, us[0])
 *         axpy(n, 1, s, 1, s+n, 1)
 */
  __pyx_t_2 = (__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_less_equal(__pyx_v_wpnorm, __Pyx_c_prod_double(__pyx_v_wnorm, __pyx_v_inv_root2)) != 0);
  if (__pyx_t_2) {

    /* "scipy/linalg/_decomp_update.pyx":1187
 * 
 *     if blas_t_less_equal(wpnorm, wnorm*inv_root2): # u lies in span(q)
 *         scal(m, 0, u, us[0])             # <<<<<<<<<<<<<<
 *         axpy(n, 1, s, 1, s+n, 1)
 *         scal(n, unorm, s, 1)
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, __pyx_t_double_complex_from_parts(0, 0), __pyx_v_u, (__pyx_v_us[0]));

    /* "scipy/linalg/_decomp_update.pyx":1188
 *     if blas_t_less_equal(wpnorm, wnorm*inv_root2): # u lies in span(q)
 *         scal(m, 0, u, us[0])
 *         axpy(n, 1, s, 1, s+n, 1)             # <<<<<<<<<<<<<<
 *         scal(n, unorm, s, 1)
 *         s[n] = 0
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, __pyx_t_double_complex_from_parts(1, 0), __pyx_v_s, 1, (__pyx_v_s + __pyx_v_n), 1);

    /* "scipy/linalg/_decomp_update.pyx":1189
 *         scal(m, 0, u, us[0])
 *         axpy(n, 1, s, 1, s+n, 1)
 *         scal(n, unorm, s, 1)             # <<<<<<<<<<<<<<
 *         s[n] = 0
 *         return 1
 */
    __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_n, __pyx_v_unorm, __pyx_v_s, 1);

    /* "scipy/linalg/_decomp_update.pyx":1190
 *         axpy(n, 1, s, 1, s+n, 1)
 *         scal(n, unorm, s, 1)
 *         s[n] = 0             # <<<<<<<<<<<<<<
 *         return 1
 * 
 */
    (__pyx_v_s[__pyx_v_n]) = __pyx_t_double_complex_from_parts(0, 0);

    /* "scipy/linalg/_decomp_update.pyx":1191
 *         scal(n, unorm, s, 1)
 *         s[n] = 0
 *         return 1             # <<<<<<<<<<<<<<
 * 
 *     with cython.cdivision(True):
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":1186
 *     wpnorm = nrm2(m, u, us[0])
 * 
 *     if blas_t_less_equal(wpnorm, wnorm*inv_root2): # u lies in span(q)             # <<<<<<<<<<<<<<
 *         scal(m, 0, u, us[0])
 *         axpy(n, 1, s, 1, s+n, 1)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1194
 * 
 *     with cython.cdivision(True):
 *         scal(m, 1/wpnorm, u, us[0])             # <<<<<<<<<<<<<<
 *     axpy(n, 1, s, 1, s+n, 1)
 *     scal(n, unorm, s, 1)
 */
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_m, __Pyx_c_quot_double(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_wpnorm), __pyx_v_u, (__pyx_v_us[0]));

  /* "scipy/linalg/_decomp_update.pyx":1195
 *     with cython.cdivision(True):
 *         scal(m, 1/wpnorm, u, us[0])
 *     axpy(n, 1, s, 1, s+n, 1)             # <<<<<<<<<<<<<<
 *     scal(n, unorm, s, 1)
 *     s[n] = wpnorm*unorm
 */
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_axpy(__pyx_v_n, __pyx_t_double_complex_from_parts(1, 0), __pyx_v_s, 1, (__pyx_v_s + __pyx_v_n), 1);

  /* "scipy/linalg/_decomp_update.pyx":1196
 *         scal(m, 1/wpnorm, u, us[0])
 *     axpy(n, 1, s, 1, s+n, 1)
 *     scal(n, unorm, s, 1)             # <<<<<<<<<<<<<<
 *     s[n] = wpnorm*unorm
 *     return 0
 */
  __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_scal(__pyx_v_n, __pyx_v_unorm, __pyx_v_s, 1);

  /* "scipy/linalg/_decomp_update.pyx":1197
 *     axpy(n, 1, s, 1, s+n, 1)
 *     scal(n, unorm, s, 1)
 *     s[n] = wpnorm*unorm             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  (__pyx_v_s[__pyx_v_n]) = __Pyx_c_prod_double(__pyx_v_wpnorm, __pyx_v_unorm);

  /* "scipy/linalg/_decomp_update.pyx":1198
 *     scal(n, unorm, s, 1)
 *     s[n] = wpnorm*unorm
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * def _form_qTu(object a, object b):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":1092
 *         index2(r, rs, j, j)[0] = rjj
 * 
 * cdef int reorth(int m, int n, blas_t* q, int* qs, bint qisF, blas_t* u,             # <<<<<<<<<<<<<<
 *                 int* us, blas_t* s, blas_t* RCOND) nogil:
 *     """Given a (m,n) matrix q with orthonormal columns and a (m,) vector u,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.linalg._decomp_update.reorth", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":1200
 *     return 0
 * 
 * def _form_qTu(object a, object b):             # <<<<<<<<<<<<<<
 *     """ this function only exists to expose the cdef version below for testing
 *         purposes. Here we perform minimal input validation to ensure that the
 */

/* Python wrapper */
static PyObject *__pyx_pw_5scipy_6linalg_14_decomp_update_1_form_qTu(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5scipy_6linalg_14_decomp_update__form_qTu[] = " this function only exists to expose the cdef version below for testing\n        purposes. Here we perform minimal input validation to ensure that the\n        inputs meet the requirements below.\n    ";
static PyMethodDef __pyx_mdef_5scipy_6linalg_14_decomp_update_1_form_qTu = {"_form_qTu", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5scipy_6linalg_14_decomp_update_1_form_qTu, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5scipy_6linalg_14_decomp_update__form_qTu};
static PyObject *__pyx_pw_5scipy_6linalg_14_decomp_update_1_form_qTu(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_a = 0;
  PyObject *__pyx_v_b = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_form_qTu (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_a,&__pyx_n_s_b,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_a)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_form_qTu", 1, 2, 2, 1); __PYX_ERR(0, 1200, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_form_qTu") < 0)) __PYX_ERR(0, 1200, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_a = values[0];
    __pyx_v_b = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_form_qTu", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1200, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("scipy.linalg._decomp_update._form_qTu", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5scipy_6linalg_14_decomp_update__form_qTu(__pyx_self, __pyx_v_a, __pyx_v_b);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5scipy_6linalg_14_decomp_update__form_qTu(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyArrayObject *__pyx_v_q = 0;
  PyArrayObject *__pyx_v_u = 0;
  PyArrayObject *__pyx_v_qTu = 0;
  int __pyx_v_typecode;
  void *__pyx_v_qTuvoid;
  int __pyx_v_qTus[2];
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("_form_qTu", 0);

  /* "scipy/linalg/_decomp_update.pyx":1210
 *     cdef int qTus[2]
 * 
 *     if not cnp.PyArray_Check(a) or not cnp.PyArray_Check(b):             # <<<<<<<<<<<<<<
 *         raise ValueError('Inputs must be arrays')
 * 
 */
  __pyx_t_2 = ((!(PyArray_Check(__pyx_v_a) != 0)) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((!(PyArray_Check(__pyx_v_b) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "scipy/linalg/_decomp_update.pyx":1211
 * 
 *     if not cnp.PyArray_Check(a) or not cnp.PyArray_Check(b):
 *         raise ValueError('Inputs must be arrays')             # <<<<<<<<<<<<<<
 * 
 *     q = a
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1211, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1211, __pyx_L1_error)

    /* "scipy/linalg/_decomp_update.pyx":1210
 *     cdef int qTus[2]
 * 
 *     if not cnp.PyArray_Check(a) or not cnp.PyArray_Check(b):             # <<<<<<<<<<<<<<
 *         raise ValueError('Inputs must be arrays')
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1213
 *         raise ValueError('Inputs must be arrays')
 * 
 *     q = a             # <<<<<<<<<<<<<<
 *     u = b
 * 
 */
  if (!(likely(((__pyx_v_a) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_a, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1213, __pyx_L1_error)
  __pyx_t_3 = __pyx_v_a;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_v_q = ((PyArrayObject *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "scipy/linalg/_decomp_update.pyx":1214
 * 
 *     q = a
 *     u = b             # <<<<<<<<<<<<<<
 * 
 *     typecode = cnp.PyArray_TYPE(q)
 */
  if (!(likely(((__pyx_v_b) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_b, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1214, __pyx_L1_error)
  __pyx_t_3 = __pyx_v_b;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_v_u = ((PyArrayObject *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "scipy/linalg/_decomp_update.pyx":1216
 *     u = b
 * 
 *     typecode = cnp.PyArray_TYPE(q)             # <<<<<<<<<<<<<<
 *     if cnp.PyArray_TYPE(u) != typecode:
 *         raise ValueError('q and u must have the same type.')
 */
  __pyx_v_typecode = PyArray_TYPE(__pyx_v_q);

  /* "scipy/linalg/_decomp_update.pyx":1217
 * 
 *     typecode = cnp.PyArray_TYPE(q)
 *     if cnp.PyArray_TYPE(u) != typecode:             # <<<<<<<<<<<<<<
 *         raise ValueError('q and u must have the same type.')
 * 
 */
  __pyx_t_1 = ((PyArray_TYPE(__pyx_v_u) != __pyx_v_typecode) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "scipy/linalg/_decomp_update.pyx":1218
 *     typecode = cnp.PyArray_TYPE(q)
 *     if cnp.PyArray_TYPE(u) != typecode:
 *         raise ValueError('q and u must have the same type.')             # <<<<<<<<<<<<<<
 * 
 *     if not (typecode == cnp.NPY_FLOAT or typecode == cnp.NPY_DOUBLE \
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1218, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1218, __pyx_L1_error)

    /* "scipy/linalg/_decomp_update.pyx":1217
 * 
 *     typecode = cnp.PyArray_TYPE(q)
 *     if cnp.PyArray_TYPE(u) != typecode:             # <<<<<<<<<<<<<<
 *         raise ValueError('q and u must have the same type.')
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1220
 *         raise ValueError('q and u must have the same type.')
 * 
 *     if not (typecode == cnp.NPY_FLOAT or typecode == cnp.NPY_DOUBLE \             # <<<<<<<<<<<<<<
 *             or typecode == cnp.NPY_CFLOAT or typecode == cnp.NPY_CDOUBLE):
 *         raise ValueError('q and u must be a blas compatible type: f d F or D')
 */
  switch (__pyx_v_typecode) {
    case NPY_FLOAT:
    case NPY_DOUBLE:
    case NPY_CFLOAT:

    /* "scipy/linalg/_decomp_update.pyx":1221
 * 
 *     if not (typecode == cnp.NPY_FLOAT or typecode == cnp.NPY_DOUBLE \
 *             or typecode == cnp.NPY_CFLOAT or typecode == cnp.NPY_CDOUBLE):             # <<<<<<<<<<<<<<
 *         raise ValueError('q and u must be a blas compatible type: f d F or D')
 * 
 */
    case NPY_CDOUBLE:

    /* "scipy/linalg/_decomp_update.pyx":1220
 *         raise ValueError('q and u must have the same type.')
 * 
 *     if not (typecode == cnp.NPY_FLOAT or typecode == cnp.NPY_DOUBLE \             # <<<<<<<<<<<<<<
 *             or typecode == cnp.NPY_CFLOAT or typecode == cnp.NPY_CDOUBLE):
 *         raise ValueError('q and u must be a blas compatible type: f d F or D')
 */
    __pyx_t_1 = 1;
    break;
    default:
    __pyx_t_1 = 0;
    break;
  }
  __pyx_t_2 = ((!__pyx_t_1) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "scipy/linalg/_decomp_update.pyx":1222
 *     if not (typecode == cnp.NPY_FLOAT or typecode == cnp.NPY_DOUBLE \
 *             or typecode == cnp.NPY_CFLOAT or typecode == cnp.NPY_CDOUBLE):
 *         raise ValueError('q and u must be a blas compatible type: f d F or D')             # <<<<<<<<<<<<<<
 * 
 *     q = validate_array(q, True)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1222, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1222, __pyx_L1_error)

    /* "scipy/linalg/_decomp_update.pyx":1220
 *         raise ValueError('q and u must have the same type.')
 * 
 *     if not (typecode == cnp.NPY_FLOAT or typecode == cnp.NPY_DOUBLE \             # <<<<<<<<<<<<<<
 *             or typecode == cnp.NPY_CFLOAT or typecode == cnp.NPY_CDOUBLE):
 *         raise ValueError('q and u must be a blas compatible type: f d F or D')
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1224
 *         raise ValueError('q and u must be a blas compatible type: f d F or D')
 * 
 *     q = validate_array(q, True)             # <<<<<<<<<<<<<<
 *     u = validate_array(u, True)
 * 
 */
  __pyx_t_3 = __pyx_f_5scipy_6linalg_14_decomp_update_validate_array(__pyx_v_q, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1224, __pyx_L1_error)
  __Pyx_DECREF_SET(__pyx_v_q, ((PyArrayObject *)__pyx_t_3));
  __pyx_t_3 = 0;

  /* "scipy/linalg/_decomp_update.pyx":1225
 * 
 *     q = validate_array(q, True)
 *     u = validate_array(u, True)             # <<<<<<<<<<<<<<
 * 
 *     qTu = cnp.PyArray_ZEROS(u.ndim, u.shape, typecode, 1)
 */
  __pyx_t_3 = __pyx_f_5scipy_6linalg_14_decomp_update_validate_array(__pyx_v_u, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1225, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1225, __pyx_L1_error)
  __Pyx_DECREF_SET(__pyx_v_u, ((PyArrayObject *)__pyx_t_3));
  __pyx_t_3 = 0;

  /* "scipy/linalg/_decomp_update.pyx":1227
 *     u = validate_array(u, True)
 * 
 *     qTu = cnp.PyArray_ZEROS(u.ndim, u.shape, typecode, 1)             # <<<<<<<<<<<<<<
 *     qTuvoid = extract(qTu, qTus)
 *     form_qTu(q, u, qTuvoid, qTus, 0)
 */
  __pyx_t_3 = PyArray_ZEROS(__pyx_v_u->nd, __pyx_v_u->dimensions, __pyx_v_typecode, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1227, __pyx_L1_error)
  __pyx_v_qTu = ((PyArrayObject *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "scipy/linalg/_decomp_update.pyx":1228
 * 
 *     qTu = cnp.PyArray_ZEROS(u.ndim, u.shape, typecode, 1)
 *     qTuvoid = extract(qTu, qTus)             # <<<<<<<<<<<<<<
 *     form_qTu(q, u, qTuvoid, qTus, 0)
 *     return qTu
 */
  __pyx_v_qTuvoid = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_qTu, __pyx_v_qTus);

  /* "scipy/linalg/_decomp_update.pyx":1229
 *     qTu = cnp.PyArray_ZEROS(u.ndim, u.shape, typecode, 1)
 *     qTuvoid = extract(qTu, qTus)
 *     form_qTu(q, u, qTuvoid, qTus, 0)             # <<<<<<<<<<<<<<
 *     return qTu
 * 
 */
  __pyx_t_3 = __pyx_f_5scipy_6linalg_14_decomp_update_form_qTu(__pyx_v_q, __pyx_v_u, __pyx_v_qTuvoid, __pyx_v_qTus, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1229, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "scipy/linalg/_decomp_update.pyx":1230
 *     qTuvoid = extract(qTu, qTus)
 *     form_qTu(q, u, qTuvoid, qTus, 0)
 *     return qTu             # <<<<<<<<<<<<<<
 * 
 * cdef form_qTu(cnp.ndarray q, cnp.ndarray u, void* qTuvoid, int* qTus,
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_qTu));
  __pyx_r = ((PyObject *)__pyx_v_qTu);
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":1200
 *     return 0
 * 
 * def _form_qTu(object a, object b):             # <<<<<<<<<<<<<<
 *     """ this function only exists to expose the cdef version below for testing
 *         purposes. Here we perform minimal input validation to ensure that the
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("scipy.linalg._decomp_update._form_qTu", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_q);
  __Pyx_XDECREF((PyObject *)__pyx_v_u);
  __Pyx_XDECREF((PyObject *)__pyx_v_qTu);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":1232
 *     return qTu
 * 
 * cdef form_qTu(cnp.ndarray q, cnp.ndarray u, void* qTuvoid, int* qTus,             # <<<<<<<<<<<<<<
 *               int k):
 *     """ assuming here that q and u have compatible shapes, and are the same
 */

static PyObject *__pyx_f_5scipy_6linalg_14_decomp_update_form_qTu(PyArrayObject *__pyx_v_q, PyArrayObject *__pyx_v_u, void *__pyx_v_qTuvoid, int *__pyx_v_qTus, int __pyx_v_k) {
  int __pyx_v_m;
  int __pyx_v_n;
  int __pyx_v_typecode;
  char *__pyx_v_T;
  char *__pyx_v_C;
  char *__pyx_v_N;
  void *__pyx_v_qvoid;
  void *__pyx_v_uvoid;
  int __pyx_v_qs[2];
  int __pyx_v_us[2];
  int __pyx_v_ldu;
  npy_intp __pyx_v_p;
  char *__pyx_v_utrans;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("form_qTu", 0);
  __Pyx_INCREF((PyObject *)__pyx_v_u);

  /* "scipy/linalg/_decomp_update.pyx":1243
 *         hold the matrix product of q.T.conj() and u.
 *     """
 *     cdef int m = q.shape[0]             # <<<<<<<<<<<<<<
 *     cdef int n = q.shape[1]
 *     cdef int typecode = cnp.PyArray_TYPE(q)
 */
  __pyx_v_m = (__pyx_v_q->dimensions[0]);

  /* "scipy/linalg/_decomp_update.pyx":1244
 *     """
 *     cdef int m = q.shape[0]
 *     cdef int n = q.shape[1]             # <<<<<<<<<<<<<<
 *     cdef int typecode = cnp.PyArray_TYPE(q)
 *     cdef cnp.ndarray qTu
 */
  __pyx_v_n = (__pyx_v_q->dimensions[1]);

  /* "scipy/linalg/_decomp_update.pyx":1245
 *     cdef int m = q.shape[0]
 *     cdef int n = q.shape[1]
 *     cdef int typecode = cnp.PyArray_TYPE(q)             # <<<<<<<<<<<<<<
 *     cdef cnp.ndarray qTu
 *     cdef char* T = 'T'
 */
  __pyx_v_typecode = PyArray_TYPE(__pyx_v_q);

  /* "scipy/linalg/_decomp_update.pyx":1247
 *     cdef int typecode = cnp.PyArray_TYPE(q)
 *     cdef cnp.ndarray qTu
 *     cdef char* T = 'T'             # <<<<<<<<<<<<<<
 *     cdef char* C = 'C'
 *     cdef char* N = 'N'
 */
  __pyx_v_T = ((char *)"T");

  /* "scipy/linalg/_decomp_update.pyx":1248
 *     cdef cnp.ndarray qTu
 *     cdef char* T = 'T'
 *     cdef char* C = 'C'             # <<<<<<<<<<<<<<
 *     cdef char* N = 'N'
 *     cdef void* qvoid
 */
  __pyx_v_C = ((char *)"C");

  /* "scipy/linalg/_decomp_update.pyx":1249
 *     cdef char* T = 'T'
 *     cdef char* C = 'C'
 *     cdef char* N = 'N'             # <<<<<<<<<<<<<<
 *     cdef void* qvoid
 *     cdef void* uvoid
 */
  __pyx_v_N = ((char *)"N");

  /* "scipy/linalg/_decomp_update.pyx":1256
 *     cdef int ldu
 * 
 *     if cnp.PyArray_CHKFLAGS(q, cnp.NPY_F_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *         qvoid = extract(q, qs)
 *         if u.ndim == 1:
 */
  __pyx_t_1 = (PyArray_CHKFLAGS(__pyx_v_q, NPY_F_CONTIGUOUS) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":1257
 * 
 *     if cnp.PyArray_CHKFLAGS(q, cnp.NPY_F_CONTIGUOUS):
 *         qvoid = extract(q, qs)             # <<<<<<<<<<<<<<
 *         if u.ndim == 1:
 *             uvoid = extract(u, us)
 */
    __pyx_v_qvoid = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_q, __pyx_v_qs);

    /* "scipy/linalg/_decomp_update.pyx":1258
 *     if cnp.PyArray_CHKFLAGS(q, cnp.NPY_F_CONTIGUOUS):
 *         qvoid = extract(q, qs)
 *         if u.ndim == 1:             # <<<<<<<<<<<<<<
 *             uvoid = extract(u, us)
 *             if typecode == cnp.NPY_FLOAT:
 */
    switch (__pyx_v_u->nd) {
      case 1:

      /* "scipy/linalg/_decomp_update.pyx":1259
 *         qvoid = extract(q, qs)
 *         if u.ndim == 1:
 *             uvoid = extract(u, us)             # <<<<<<<<<<<<<<
 *             if typecode == cnp.NPY_FLOAT:
 *                 gemv(T, m, n, 1, <float*>qvoid, m,
 */
      __pyx_v_uvoid = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_u, __pyx_v_us);

      /* "scipy/linalg/_decomp_update.pyx":1260
 *         if u.ndim == 1:
 *             uvoid = extract(u, us)
 *             if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                 gemv(T, m, n, 1, <float*>qvoid, m,
 *                         <float*>uvoid, us[0], 0, col(<float*>qTuvoid, qTus, k), qTus[0])
 */
      __pyx_t_1 = ((__pyx_v_typecode == NPY_FLOAT) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":1261
 *             uvoid = extract(u, us)
 *             if typecode == cnp.NPY_FLOAT:
 *                 gemv(T, m, n, 1, <float*>qvoid, m,             # <<<<<<<<<<<<<<
 *                         <float*>uvoid, us[0], 0, col(<float*>qTuvoid, qTus, k), qTus[0])
 *             if typecode == cnp.NPY_DOUBLE:
 */
        __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_m, __pyx_v_n, 1.0, ((float *)__pyx_v_qvoid), __pyx_v_m, ((float *)__pyx_v_uvoid), (__pyx_v_us[0]), 0.0, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(((float *)__pyx_v_qTuvoid), __pyx_v_qTus, __pyx_v_k), (__pyx_v_qTus[0]));

        /* "scipy/linalg/_decomp_update.pyx":1260
 *         if u.ndim == 1:
 *             uvoid = extract(u, us)
 *             if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                 gemv(T, m, n, 1, <float*>qvoid, m,
 *                         <float*>uvoid, us[0], 0, col(<float*>qTuvoid, qTus, k), qTus[0])
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":1263
 *                 gemv(T, m, n, 1, <float*>qvoid, m,
 *                         <float*>uvoid, us[0], 0, col(<float*>qTuvoid, qTus, k), qTus[0])
 *             if typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                 gemv(T, m, n, 1, <double*>qvoid, m,
 *                         <double*>uvoid, us[0], 0, col(<double*>qTuvoid, qTus, k), qTus[0])
 */
      __pyx_t_1 = ((__pyx_v_typecode == NPY_DOUBLE) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":1264
 *                         <float*>uvoid, us[0], 0, col(<float*>qTuvoid, qTus, k), qTus[0])
 *             if typecode == cnp.NPY_DOUBLE:
 *                 gemv(T, m, n, 1, <double*>qvoid, m,             # <<<<<<<<<<<<<<
 *                         <double*>uvoid, us[0], 0, col(<double*>qTuvoid, qTus, k), qTus[0])
 *             if typecode == cnp.NPY_CFLOAT:
 */
        __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_T, __pyx_v_m, __pyx_v_n, 1.0, ((double *)__pyx_v_qvoid), __pyx_v_m, ((double *)__pyx_v_uvoid), (__pyx_v_us[0]), 0.0, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(((double *)__pyx_v_qTuvoid), __pyx_v_qTus, __pyx_v_k), (__pyx_v_qTus[0]));

        /* "scipy/linalg/_decomp_update.pyx":1263
 *                 gemv(T, m, n, 1, <float*>qvoid, m,
 *                         <float*>uvoid, us[0], 0, col(<float*>qTuvoid, qTus, k), qTus[0])
 *             if typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                 gemv(T, m, n, 1, <double*>qvoid, m,
 *                         <double*>uvoid, us[0], 0, col(<double*>qTuvoid, qTus, k), qTus[0])
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":1266
 *                 gemv(T, m, n, 1, <double*>qvoid, m,
 *                         <double*>uvoid, us[0], 0, col(<double*>qTuvoid, qTus, k), qTus[0])
 *             if typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                 gemv(C, m, n, 1, <float_complex*>qvoid, m,
 *                         <float_complex*>uvoid, us[0], 0,
 */
      __pyx_t_1 = ((__pyx_v_typecode == NPY_CFLOAT) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":1267
 *                         <double*>uvoid, us[0], 0, col(<double*>qTuvoid, qTus, k), qTus[0])
 *             if typecode == cnp.NPY_CFLOAT:
 *                 gemv(C, m, n, 1, <float_complex*>qvoid, m,             # <<<<<<<<<<<<<<
 *                         <float_complex*>uvoid, us[0], 0,
 *                         col(<float_complex*>qTuvoid, qTus, k), qTus[0])
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_C, __pyx_v_m, __pyx_v_n, __pyx_t_float_complex_from_parts(1, 0), ((__pyx_t_float_complex *)__pyx_v_qvoid), __pyx_v_m, ((__pyx_t_float_complex *)__pyx_v_uvoid), (__pyx_v_us[0]), __pyx_t_float_complex_from_parts(0, 0), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(((__pyx_t_float_complex *)__pyx_v_qTuvoid), __pyx_v_qTus, __pyx_v_k), (__pyx_v_qTus[0]));

        /* "scipy/linalg/_decomp_update.pyx":1266
 *                 gemv(T, m, n, 1, <double*>qvoid, m,
 *                         <double*>uvoid, us[0], 0, col(<double*>qTuvoid, qTus, k), qTus[0])
 *             if typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                 gemv(C, m, n, 1, <float_complex*>qvoid, m,
 *                         <float_complex*>uvoid, us[0], 0,
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":1270
 *                         <float_complex*>uvoid, us[0], 0,
 *                         col(<float_complex*>qTuvoid, qTus, k), qTus[0])
 *             if typecode == cnp.NPY_CDOUBLE:             # <<<<<<<<<<<<<<
 *                 gemv(C, m, n, 1, <double_complex*>qvoid, m,
 *                         <double_complex*>uvoid, us[0], 0,
 */
      __pyx_t_1 = ((__pyx_v_typecode == NPY_CDOUBLE) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":1271
 *                         col(<float_complex*>qTuvoid, qTus, k), qTus[0])
 *             if typecode == cnp.NPY_CDOUBLE:
 *                 gemv(C, m, n, 1, <double_complex*>qvoid, m,             # <<<<<<<<<<<<<<
 *                         <double_complex*>uvoid, us[0], 0,
 *                         col(<double_complex*>qTuvoid, qTus, k), qTus[0])
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_C, __pyx_v_m, __pyx_v_n, __pyx_t_double_complex_from_parts(1, 0), ((__pyx_t_double_complex *)__pyx_v_qvoid), __pyx_v_m, ((__pyx_t_double_complex *)__pyx_v_uvoid), (__pyx_v_us[0]), __pyx_t_double_complex_from_parts(0, 0), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(((__pyx_t_double_complex *)__pyx_v_qTuvoid), __pyx_v_qTus, __pyx_v_k), (__pyx_v_qTus[0]));

        /* "scipy/linalg/_decomp_update.pyx":1270
 *                         <float_complex*>uvoid, us[0], 0,
 *                         col(<float_complex*>qTuvoid, qTus, k), qTus[0])
 *             if typecode == cnp.NPY_CDOUBLE:             # <<<<<<<<<<<<<<
 *                 gemv(C, m, n, 1, <double_complex*>qvoid, m,
 *                         <double_complex*>uvoid, us[0], 0,
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":1258
 *     if cnp.PyArray_CHKFLAGS(q, cnp.NPY_F_CONTIGUOUS):
 *         qvoid = extract(q, qs)
 *         if u.ndim == 1:             # <<<<<<<<<<<<<<
 *             uvoid = extract(u, us)
 *             if typecode == cnp.NPY_FLOAT:
 */
      break;
      case 2:

      /* "scipy/linalg/_decomp_update.pyx":1275
 *                         col(<double_complex*>qTuvoid, qTus, k), qTus[0])
 *         elif u.ndim == 2:
 *             p = u.shape[1]             # <<<<<<<<<<<<<<
 *             if cnp.PyArray_CHKFLAGS(u, cnp.NPY_F_CONTIGUOUS):
 *                 utrans = N
 */
      __pyx_v_p = (__pyx_v_u->dimensions[1]);

      /* "scipy/linalg/_decomp_update.pyx":1276
 *         elif u.ndim == 2:
 *             p = u.shape[1]
 *             if cnp.PyArray_CHKFLAGS(u, cnp.NPY_F_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *                 utrans = N
 *                 uvoid = extract(u, us)
 */
      __pyx_t_1 = (PyArray_CHKFLAGS(__pyx_v_u, NPY_F_CONTIGUOUS) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":1277
 *             p = u.shape[1]
 *             if cnp.PyArray_CHKFLAGS(u, cnp.NPY_F_CONTIGUOUS):
 *                 utrans = N             # <<<<<<<<<<<<<<
 *                 uvoid = extract(u, us)
 *                 ldu = u.shape[0]
 */
        __pyx_v_utrans = __pyx_v_N;

        /* "scipy/linalg/_decomp_update.pyx":1278
 *             if cnp.PyArray_CHKFLAGS(u, cnp.NPY_F_CONTIGUOUS):
 *                 utrans = N
 *                 uvoid = extract(u, us)             # <<<<<<<<<<<<<<
 *                 ldu = u.shape[0]
 *             elif cnp.PyArray_CHKFLAGS(u, cnp.NPY_C_CONTIGUOUS):
 */
        __pyx_v_uvoid = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_u, __pyx_v_us);

        /* "scipy/linalg/_decomp_update.pyx":1279
 *                 utrans = N
 *                 uvoid = extract(u, us)
 *                 ldu = u.shape[0]             # <<<<<<<<<<<<<<
 *             elif cnp.PyArray_CHKFLAGS(u, cnp.NPY_C_CONTIGUOUS):
 *                 utrans = T
 */
        __pyx_v_ldu = (__pyx_v_u->dimensions[0]);

        /* "scipy/linalg/_decomp_update.pyx":1276
 *         elif u.ndim == 2:
 *             p = u.shape[1]
 *             if cnp.PyArray_CHKFLAGS(u, cnp.NPY_F_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *                 utrans = N
 *                 uvoid = extract(u, us)
 */
        goto __pyx_L8;
      }

      /* "scipy/linalg/_decomp_update.pyx":1280
 *                 uvoid = extract(u, us)
 *                 ldu = u.shape[0]
 *             elif cnp.PyArray_CHKFLAGS(u, cnp.NPY_C_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *                 utrans = T
 *                 uvoid = extract(u, us)
 */
      __pyx_t_1 = (PyArray_CHKFLAGS(__pyx_v_u, NPY_C_CONTIGUOUS) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":1281
 *                 ldu = u.shape[0]
 *             elif cnp.PyArray_CHKFLAGS(u, cnp.NPY_C_CONTIGUOUS):
 *                 utrans = T             # <<<<<<<<<<<<<<
 *                 uvoid = extract(u, us)
 *                 ldu = u.shape[1]
 */
        __pyx_v_utrans = __pyx_v_T;

        /* "scipy/linalg/_decomp_update.pyx":1282
 *             elif cnp.PyArray_CHKFLAGS(u, cnp.NPY_C_CONTIGUOUS):
 *                 utrans = T
 *                 uvoid = extract(u, us)             # <<<<<<<<<<<<<<
 *                 ldu = u.shape[1]
 *             else:
 */
        __pyx_v_uvoid = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_u, __pyx_v_us);

        /* "scipy/linalg/_decomp_update.pyx":1283
 *                 utrans = T
 *                 uvoid = extract(u, us)
 *                 ldu = u.shape[1]             # <<<<<<<<<<<<<<
 *             else:
 *                 u = PyArray_FromArray(u, NULL, cnp.NPY_F_CONTIGUOUS)
 */
        __pyx_v_ldu = (__pyx_v_u->dimensions[1]);

        /* "scipy/linalg/_decomp_update.pyx":1280
 *                 uvoid = extract(u, us)
 *                 ldu = u.shape[0]
 *             elif cnp.PyArray_CHKFLAGS(u, cnp.NPY_C_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *                 utrans = T
 *                 uvoid = extract(u, us)
 */
        goto __pyx_L8;
      }

      /* "scipy/linalg/_decomp_update.pyx":1285
 *                 ldu = u.shape[1]
 *             else:
 *                 u = PyArray_FromArray(u, NULL, cnp.NPY_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 utrans = N
 *                 uvoid = extract(u, us)
 */
      /*else*/ {
        __pyx_t_2 = ((PyObject *)PyArray_FromArray(__pyx_v_u, NULL, NPY_F_CONTIGUOUS)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1285, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF_SET(__pyx_v_u, ((PyArrayObject *)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "scipy/linalg/_decomp_update.pyx":1286
 *             else:
 *                 u = PyArray_FromArray(u, NULL, cnp.NPY_F_CONTIGUOUS)
 *                 utrans = N             # <<<<<<<<<<<<<<
 *                 uvoid = extract(u, us)
 *                 ldu = u.shape[0]
 */
        __pyx_v_utrans = __pyx_v_N;

        /* "scipy/linalg/_decomp_update.pyx":1287
 *                 u = PyArray_FromArray(u, NULL, cnp.NPY_F_CONTIGUOUS)
 *                 utrans = N
 *                 uvoid = extract(u, us)             # <<<<<<<<<<<<<<
 *                 ldu = u.shape[0]
 *             if typecode == cnp.NPY_FLOAT:
 */
        __pyx_v_uvoid = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_u, __pyx_v_us);

        /* "scipy/linalg/_decomp_update.pyx":1288
 *                 utrans = N
 *                 uvoid = extract(u, us)
 *                 ldu = u.shape[0]             # <<<<<<<<<<<<<<
 *             if typecode == cnp.NPY_FLOAT:
 *                 gemm(T, utrans, m, p, m, 1, <float*>qvoid, m,
 */
        __pyx_v_ldu = (__pyx_v_u->dimensions[0]);
      }
      __pyx_L8:;

      /* "scipy/linalg/_decomp_update.pyx":1289
 *                 uvoid = extract(u, us)
 *                 ldu = u.shape[0]
 *             if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                 gemm(T, utrans, m, p, m, 1, <float*>qvoid, m,
 *                         <float*>uvoid, ldu, 0, col(<float*>qTuvoid, qTus, k), m)
 */
      __pyx_t_1 = ((__pyx_v_typecode == NPY_FLOAT) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":1290
 *                 ldu = u.shape[0]
 *             if typecode == cnp.NPY_FLOAT:
 *                 gemm(T, utrans, m, p, m, 1, <float*>qvoid, m,             # <<<<<<<<<<<<<<
 *                         <float*>uvoid, ldu, 0, col(<float*>qTuvoid, qTus, k), m)
 *             if typecode == cnp.NPY_DOUBLE:
 */
        __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemm(__pyx_v_T, __pyx_v_utrans, __pyx_v_m, __pyx_v_p, __pyx_v_m, 1.0, ((float *)__pyx_v_qvoid), __pyx_v_m, ((float *)__pyx_v_uvoid), __pyx_v_ldu, 0.0, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(((float *)__pyx_v_qTuvoid), __pyx_v_qTus, __pyx_v_k), __pyx_v_m);

        /* "scipy/linalg/_decomp_update.pyx":1289
 *                 uvoid = extract(u, us)
 *                 ldu = u.shape[0]
 *             if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                 gemm(T, utrans, m, p, m, 1, <float*>qvoid, m,
 *                         <float*>uvoid, ldu, 0, col(<float*>qTuvoid, qTus, k), m)
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":1292
 *                 gemm(T, utrans, m, p, m, 1, <float*>qvoid, m,
 *                         <float*>uvoid, ldu, 0, col(<float*>qTuvoid, qTus, k), m)
 *             if typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                 gemm(T, utrans, m, p, m, 1, <double*>qvoid, m,
 *                         <double*>uvoid, ldu, 0, col(<double*>qTuvoid, qTus, k), m)
 */
      __pyx_t_1 = ((__pyx_v_typecode == NPY_DOUBLE) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":1293
 *                         <float*>uvoid, ldu, 0, col(<float*>qTuvoid, qTus, k), m)
 *             if typecode == cnp.NPY_DOUBLE:
 *                 gemm(T, utrans, m, p, m, 1, <double*>qvoid, m,             # <<<<<<<<<<<<<<
 *                         <double*>uvoid, ldu, 0, col(<double*>qTuvoid, qTus, k), m)
 *             if typecode == cnp.NPY_CFLOAT:
 */
        __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemm(__pyx_v_T, __pyx_v_utrans, __pyx_v_m, __pyx_v_p, __pyx_v_m, 1.0, ((double *)__pyx_v_qvoid), __pyx_v_m, ((double *)__pyx_v_uvoid), __pyx_v_ldu, 0.0, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(((double *)__pyx_v_qTuvoid), __pyx_v_qTus, __pyx_v_k), __pyx_v_m);

        /* "scipy/linalg/_decomp_update.pyx":1292
 *                 gemm(T, utrans, m, p, m, 1, <float*>qvoid, m,
 *                         <float*>uvoid, ldu, 0, col(<float*>qTuvoid, qTus, k), m)
 *             if typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                 gemm(T, utrans, m, p, m, 1, <double*>qvoid, m,
 *                         <double*>uvoid, ldu, 0, col(<double*>qTuvoid, qTus, k), m)
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":1295
 *                 gemm(T, utrans, m, p, m, 1, <double*>qvoid, m,
 *                         <double*>uvoid, ldu, 0, col(<double*>qTuvoid, qTus, k), m)
 *             if typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                 gemm(C, utrans, m, p, m, 1, <float_complex*>qvoid, m,
 *                         <float_complex*>uvoid, ldu, 0,
 */
      __pyx_t_1 = ((__pyx_v_typecode == NPY_CFLOAT) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":1296
 *                         <double*>uvoid, ldu, 0, col(<double*>qTuvoid, qTus, k), m)
 *             if typecode == cnp.NPY_CFLOAT:
 *                 gemm(C, utrans, m, p, m, 1, <float_complex*>qvoid, m,             # <<<<<<<<<<<<<<
 *                         <float_complex*>uvoid, ldu, 0,
 *                         col(<float_complex*>qTuvoid, qTus, k), m)
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemm(__pyx_v_C, __pyx_v_utrans, __pyx_v_m, __pyx_v_p, __pyx_v_m, __pyx_t_float_complex_from_parts(1, 0), ((__pyx_t_float_complex *)__pyx_v_qvoid), __pyx_v_m, ((__pyx_t_float_complex *)__pyx_v_uvoid), __pyx_v_ldu, __pyx_t_float_complex_from_parts(0, 0), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(((__pyx_t_float_complex *)__pyx_v_qTuvoid), __pyx_v_qTus, __pyx_v_k), __pyx_v_m);

        /* "scipy/linalg/_decomp_update.pyx":1295
 *                 gemm(T, utrans, m, p, m, 1, <double*>qvoid, m,
 *                         <double*>uvoid, ldu, 0, col(<double*>qTuvoid, qTus, k), m)
 *             if typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                 gemm(C, utrans, m, p, m, 1, <float_complex*>qvoid, m,
 *                         <float_complex*>uvoid, ldu, 0,
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":1299
 *                         <float_complex*>uvoid, ldu, 0,
 *                         col(<float_complex*>qTuvoid, qTus, k), m)
 *             if typecode == cnp.NPY_CDOUBLE:             # <<<<<<<<<<<<<<
 *                 gemm(C, utrans, m, p, m, 1, <double_complex*>qvoid, m,
 *                         <double_complex*>uvoid, ldu, 0,
 */
      __pyx_t_1 = ((__pyx_v_typecode == NPY_CDOUBLE) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":1300
 *                         col(<float_complex*>qTuvoid, qTus, k), m)
 *             if typecode == cnp.NPY_CDOUBLE:
 *                 gemm(C, utrans, m, p, m, 1, <double_complex*>qvoid, m,             # <<<<<<<<<<<<<<
 *                         <double_complex*>uvoid, ldu, 0,
 *                         col(<double_complex*>qTuvoid, qTus, k), m)
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemm(__pyx_v_C, __pyx_v_utrans, __pyx_v_m, __pyx_v_p, __pyx_v_m, __pyx_t_double_complex_from_parts(1, 0), ((__pyx_t_double_complex *)__pyx_v_qvoid), __pyx_v_m, ((__pyx_t_double_complex *)__pyx_v_uvoid), __pyx_v_ldu, __pyx_t_double_complex_from_parts(0, 0), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(((__pyx_t_double_complex *)__pyx_v_qTuvoid), __pyx_v_qTus, __pyx_v_k), __pyx_v_m);

        /* "scipy/linalg/_decomp_update.pyx":1299
 *                         <float_complex*>uvoid, ldu, 0,
 *                         col(<float_complex*>qTuvoid, qTus, k), m)
 *             if typecode == cnp.NPY_CDOUBLE:             # <<<<<<<<<<<<<<
 *                 gemm(C, utrans, m, p, m, 1, <double_complex*>qvoid, m,
 *                         <double_complex*>uvoid, ldu, 0,
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":1274
 *                         <double_complex*>uvoid, us[0], 0,
 *                         col(<double_complex*>qTuvoid, qTus, k), qTus[0])
 *         elif u.ndim == 2:             # <<<<<<<<<<<<<<
 *             p = u.shape[1]
 *             if cnp.PyArray_CHKFLAGS(u, cnp.NPY_F_CONTIGUOUS):
 */
      break;
      default: break;
    }

    /* "scipy/linalg/_decomp_update.pyx":1256
 *     cdef int ldu
 * 
 *     if cnp.PyArray_CHKFLAGS(q, cnp.NPY_F_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *         qvoid = extract(q, qs)
 *         if u.ndim == 1:
 */
    goto __pyx_L3;
  }

  /* "scipy/linalg/_decomp_update.pyx":1304
 *                         col(<double_complex*>qTuvoid, qTus, k), m)
 * 
 *     elif cnp.PyArray_CHKFLAGS(q, cnp.NPY_C_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *         qvoid = extract(q, qs)
 *         if u.ndim == 1:
 */
  __pyx_t_1 = (PyArray_CHKFLAGS(__pyx_v_q, NPY_C_CONTIGUOUS) != 0);
  if (likely(__pyx_t_1)) {

    /* "scipy/linalg/_decomp_update.pyx":1305
 * 
 *     elif cnp.PyArray_CHKFLAGS(q, cnp.NPY_C_CONTIGUOUS):
 *         qvoid = extract(q, qs)             # <<<<<<<<<<<<<<
 *         if u.ndim == 1:
 *             uvoid = extract(u, us)
 */
    __pyx_v_qvoid = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_q, __pyx_v_qs);

    /* "scipy/linalg/_decomp_update.pyx":1306
 *     elif cnp.PyArray_CHKFLAGS(q, cnp.NPY_C_CONTIGUOUS):
 *         qvoid = extract(q, qs)
 *         if u.ndim == 1:             # <<<<<<<<<<<<<<
 *             uvoid = extract(u, us)
 *             if typecode == cnp.NPY_FLOAT:
 */
    switch (__pyx_v_u->nd) {
      case 1:

      /* "scipy/linalg/_decomp_update.pyx":1307
 *         qvoid = extract(q, qs)
 *         if u.ndim == 1:
 *             uvoid = extract(u, us)             # <<<<<<<<<<<<<<
 *             if typecode == cnp.NPY_FLOAT:
 *                 gemv(N, m, n, 1, <float*>qvoid, n,
 */
      __pyx_v_uvoid = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_u, __pyx_v_us);

      /* "scipy/linalg/_decomp_update.pyx":1308
 *         if u.ndim == 1:
 *             uvoid = extract(u, us)
 *             if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                 gemv(N, m, n, 1, <float*>qvoid, n,
 *                         <float*>uvoid, us[0], 0, col(<float*>qTuvoid, qTus, k), qTus[0])
 */
      __pyx_t_1 = ((__pyx_v_typecode == NPY_FLOAT) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":1309
 *             uvoid = extract(u, us)
 *             if typecode == cnp.NPY_FLOAT:
 *                 gemv(N, m, n, 1, <float*>qvoid, n,             # <<<<<<<<<<<<<<
 *                         <float*>uvoid, us[0], 0, col(<float*>qTuvoid, qTus, k), qTus[0])
 *             if typecode == cnp.NPY_DOUBLE:
 */
        __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_n, 1.0, ((float *)__pyx_v_qvoid), __pyx_v_n, ((float *)__pyx_v_uvoid), (__pyx_v_us[0]), 0.0, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(((float *)__pyx_v_qTuvoid), __pyx_v_qTus, __pyx_v_k), (__pyx_v_qTus[0]));

        /* "scipy/linalg/_decomp_update.pyx":1308
 *         if u.ndim == 1:
 *             uvoid = extract(u, us)
 *             if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                 gemv(N, m, n, 1, <float*>qvoid, n,
 *                         <float*>uvoid, us[0], 0, col(<float*>qTuvoid, qTus, k), qTus[0])
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":1311
 *                 gemv(N, m, n, 1, <float*>qvoid, n,
 *                         <float*>uvoid, us[0], 0, col(<float*>qTuvoid, qTus, k), qTus[0])
 *             if typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                 gemv(N, m, n, 1, <double*>qvoid, n,
 *                         <double*>uvoid, us[0], 0, col(<double*>qTuvoid, qTus, k), qTus[0])
 */
      __pyx_t_1 = ((__pyx_v_typecode == NPY_DOUBLE) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":1312
 *                         <float*>uvoid, us[0], 0, col(<float*>qTuvoid, qTus, k), qTus[0])
 *             if typecode == cnp.NPY_DOUBLE:
 *                 gemv(N, m, n, 1, <double*>qvoid, n,             # <<<<<<<<<<<<<<
 *                         <double*>uvoid, us[0], 0, col(<double*>qTuvoid, qTus, k), qTus[0])
 *             if typecode == cnp.NPY_CFLOAT:
 */
        __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_n, 1.0, ((double *)__pyx_v_qvoid), __pyx_v_n, ((double *)__pyx_v_uvoid), (__pyx_v_us[0]), 0.0, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(((double *)__pyx_v_qTuvoid), __pyx_v_qTus, __pyx_v_k), (__pyx_v_qTus[0]));

        /* "scipy/linalg/_decomp_update.pyx":1311
 *                 gemv(N, m, n, 1, <float*>qvoid, n,
 *                         <float*>uvoid, us[0], 0, col(<float*>qTuvoid, qTus, k), qTus[0])
 *             if typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                 gemv(N, m, n, 1, <double*>qvoid, n,
 *                         <double*>uvoid, us[0], 0, col(<double*>qTuvoid, qTus, k), qTus[0])
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":1314
 *                 gemv(N, m, n, 1, <double*>qvoid, n,
 *                         <double*>uvoid, us[0], 0, col(<double*>qTuvoid, qTus, k), qTus[0])
 *             if typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                 blas_t_conj(m, <float_complex*>uvoid, us)
 *                 gemv(N, m, n, 1, <float_complex*>qvoid, n,
 */
      __pyx_t_1 = ((__pyx_v_typecode == NPY_CFLOAT) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":1315
 *                         <double*>uvoid, us[0], 0, col(<double*>qTuvoid, qTus, k), qTus[0])
 *             if typecode == cnp.NPY_CFLOAT:
 *                 blas_t_conj(m, <float_complex*>uvoid, us)             # <<<<<<<<<<<<<<
 *                 gemv(N, m, n, 1, <float_complex*>qvoid, n,
 *                         <float_complex*>uvoid, us[0], 0,
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_m, ((__pyx_t_float_complex *)__pyx_v_uvoid), __pyx_v_us);

        /* "scipy/linalg/_decomp_update.pyx":1316
 *             if typecode == cnp.NPY_CFLOAT:
 *                 blas_t_conj(m, <float_complex*>uvoid, us)
 *                 gemv(N, m, n, 1, <float_complex*>qvoid, n,             # <<<<<<<<<<<<<<
 *                         <float_complex*>uvoid, us[0], 0,
 *                         col(<float_complex*>qTuvoid, qTus, k), qTus[0])
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_n, __pyx_t_float_complex_from_parts(1, 0), ((__pyx_t_float_complex *)__pyx_v_qvoid), __pyx_v_n, ((__pyx_t_float_complex *)__pyx_v_uvoid), (__pyx_v_us[0]), __pyx_t_float_complex_from_parts(0, 0), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(((__pyx_t_float_complex *)__pyx_v_qTuvoid), __pyx_v_qTus, __pyx_v_k), (__pyx_v_qTus[0]));

        /* "scipy/linalg/_decomp_update.pyx":1319
 *                         <float_complex*>uvoid, us[0], 0,
 *                         col(<float_complex*>qTuvoid, qTus, k), qTus[0])
 *                 blas_t_conj(m, col(<float_complex*>qTuvoid, qTus, k), qTus)             # <<<<<<<<<<<<<<
 *             if typecode == cnp.NPY_CDOUBLE:
 *                 blas_t_conj(m, <double_complex*>uvoid, us)
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_m, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(((__pyx_t_float_complex *)__pyx_v_qTuvoid), __pyx_v_qTus, __pyx_v_k), __pyx_v_qTus);

        /* "scipy/linalg/_decomp_update.pyx":1314
 *                 gemv(N, m, n, 1, <double*>qvoid, n,
 *                         <double*>uvoid, us[0], 0, col(<double*>qTuvoid, qTus, k), qTus[0])
 *             if typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                 blas_t_conj(m, <float_complex*>uvoid, us)
 *                 gemv(N, m, n, 1, <float_complex*>qvoid, n,
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":1320
 *                         col(<float_complex*>qTuvoid, qTus, k), qTus[0])
 *                 blas_t_conj(m, col(<float_complex*>qTuvoid, qTus, k), qTus)
 *             if typecode == cnp.NPY_CDOUBLE:             # <<<<<<<<<<<<<<
 *                 blas_t_conj(m, <double_complex*>uvoid, us)
 *                 gemv(N, m, n, 1, <double_complex*>qvoid, n,
 */
      __pyx_t_1 = ((__pyx_v_typecode == NPY_CDOUBLE) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":1321
 *                 blas_t_conj(m, col(<float_complex*>qTuvoid, qTus, k), qTus)
 *             if typecode == cnp.NPY_CDOUBLE:
 *                 blas_t_conj(m, <double_complex*>uvoid, us)             # <<<<<<<<<<<<<<
 *                 gemv(N, m, n, 1, <double_complex*>qvoid, n,
 *                         <double_complex*>uvoid, us[0], 0,
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_m, ((__pyx_t_double_complex *)__pyx_v_uvoid), __pyx_v_us);

        /* "scipy/linalg/_decomp_update.pyx":1322
 *             if typecode == cnp.NPY_CDOUBLE:
 *                 blas_t_conj(m, <double_complex*>uvoid, us)
 *                 gemv(N, m, n, 1, <double_complex*>qvoid, n,             # <<<<<<<<<<<<<<
 *                         <double_complex*>uvoid, us[0], 0,
 *                         col(<double_complex*>qTuvoid, qTus, k), qTus[0])
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemv(__pyx_v_N, __pyx_v_m, __pyx_v_n, __pyx_t_double_complex_from_parts(1, 0), ((__pyx_t_double_complex *)__pyx_v_qvoid), __pyx_v_n, ((__pyx_t_double_complex *)__pyx_v_uvoid), (__pyx_v_us[0]), __pyx_t_double_complex_from_parts(0, 0), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(((__pyx_t_double_complex *)__pyx_v_qTuvoid), __pyx_v_qTus, __pyx_v_k), (__pyx_v_qTus[0]));

        /* "scipy/linalg/_decomp_update.pyx":1325
 *                         <double_complex*>uvoid, us[0], 0,
 *                         col(<double_complex*>qTuvoid, qTus, k), qTus[0])
 *                 blas_t_conj(m, col(<double_complex*>qTuvoid, qTus, k), qTus)             # <<<<<<<<<<<<<<
 *         elif u.ndim == 2:
 *             p = u.shape[1]
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_conj(__pyx_v_m, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(((__pyx_t_double_complex *)__pyx_v_qTuvoid), __pyx_v_qTus, __pyx_v_k), __pyx_v_qTus);

        /* "scipy/linalg/_decomp_update.pyx":1320
 *                         col(<float_complex*>qTuvoid, qTus, k), qTus[0])
 *                 blas_t_conj(m, col(<float_complex*>qTuvoid, qTus, k), qTus)
 *             if typecode == cnp.NPY_CDOUBLE:             # <<<<<<<<<<<<<<
 *                 blas_t_conj(m, <double_complex*>uvoid, us)
 *                 gemv(N, m, n, 1, <double_complex*>qvoid, n,
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":1306
 *     elif cnp.PyArray_CHKFLAGS(q, cnp.NPY_C_CONTIGUOUS):
 *         qvoid = extract(q, qs)
 *         if u.ndim == 1:             # <<<<<<<<<<<<<<
 *             uvoid = extract(u, us)
 *             if typecode == cnp.NPY_FLOAT:
 */
      break;
      case 2:

      /* "scipy/linalg/_decomp_update.pyx":1327
 *                 blas_t_conj(m, col(<double_complex*>qTuvoid, qTus, k), qTus)
 *         elif u.ndim == 2:
 *             p = u.shape[1]             # <<<<<<<<<<<<<<
 *             if cnp.PyArray_CHKFLAGS(u, cnp.NPY_F_CONTIGUOUS):
 *                 utrans = N
 */
      __pyx_v_p = (__pyx_v_u->dimensions[1]);

      /* "scipy/linalg/_decomp_update.pyx":1328
 *         elif u.ndim == 2:
 *             p = u.shape[1]
 *             if cnp.PyArray_CHKFLAGS(u, cnp.NPY_F_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *                 utrans = N
 *                 uvoid = extract(u, us)
 */
      __pyx_t_1 = (PyArray_CHKFLAGS(__pyx_v_u, NPY_F_CONTIGUOUS) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":1329
 *             p = u.shape[1]
 *             if cnp.PyArray_CHKFLAGS(u, cnp.NPY_F_CONTIGUOUS):
 *                 utrans = N             # <<<<<<<<<<<<<<
 *                 uvoid = extract(u, us)
 *                 ldu = u.shape[0]
 */
        __pyx_v_utrans = __pyx_v_N;

        /* "scipy/linalg/_decomp_update.pyx":1330
 *             if cnp.PyArray_CHKFLAGS(u, cnp.NPY_F_CONTIGUOUS):
 *                 utrans = N
 *                 uvoid = extract(u, us)             # <<<<<<<<<<<<<<
 *                 ldu = u.shape[0]
 *             elif cnp.PyArray_CHKFLAGS(u, cnp.NPY_C_CONTIGUOUS):
 */
        __pyx_v_uvoid = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_u, __pyx_v_us);

        /* "scipy/linalg/_decomp_update.pyx":1331
 *                 utrans = N
 *                 uvoid = extract(u, us)
 *                 ldu = u.shape[0]             # <<<<<<<<<<<<<<
 *             elif cnp.PyArray_CHKFLAGS(u, cnp.NPY_C_CONTIGUOUS):
 *                 utrans = T
 */
        __pyx_v_ldu = (__pyx_v_u->dimensions[0]);

        /* "scipy/linalg/_decomp_update.pyx":1328
 *         elif u.ndim == 2:
 *             p = u.shape[1]
 *             if cnp.PyArray_CHKFLAGS(u, cnp.NPY_F_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *                 utrans = N
 *                 uvoid = extract(u, us)
 */
        goto __pyx_L17;
      }

      /* "scipy/linalg/_decomp_update.pyx":1332
 *                 uvoid = extract(u, us)
 *                 ldu = u.shape[0]
 *             elif cnp.PyArray_CHKFLAGS(u, cnp.NPY_C_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *                 utrans = T
 *                 uvoid = extract(u, us)
 */
      __pyx_t_1 = (PyArray_CHKFLAGS(__pyx_v_u, NPY_C_CONTIGUOUS) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":1333
 *                 ldu = u.shape[0]
 *             elif cnp.PyArray_CHKFLAGS(u, cnp.NPY_C_CONTIGUOUS):
 *                 utrans = T             # <<<<<<<<<<<<<<
 *                 uvoid = extract(u, us)
 *                 ldu = u.shape[1]
 */
        __pyx_v_utrans = __pyx_v_T;

        /* "scipy/linalg/_decomp_update.pyx":1334
 *             elif cnp.PyArray_CHKFLAGS(u, cnp.NPY_C_CONTIGUOUS):
 *                 utrans = T
 *                 uvoid = extract(u, us)             # <<<<<<<<<<<<<<
 *                 ldu = u.shape[1]
 *             else:
 */
        __pyx_v_uvoid = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_u, __pyx_v_us);

        /* "scipy/linalg/_decomp_update.pyx":1335
 *                 utrans = T
 *                 uvoid = extract(u, us)
 *                 ldu = u.shape[1]             # <<<<<<<<<<<<<<
 *             else:
 *                 u = PyArray_FromArray(u, NULL, cnp.NPY_F_CONTIGUOUS)
 */
        __pyx_v_ldu = (__pyx_v_u->dimensions[1]);

        /* "scipy/linalg/_decomp_update.pyx":1332
 *                 uvoid = extract(u, us)
 *                 ldu = u.shape[0]
 *             elif cnp.PyArray_CHKFLAGS(u, cnp.NPY_C_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *                 utrans = T
 *                 uvoid = extract(u, us)
 */
        goto __pyx_L17;
      }

      /* "scipy/linalg/_decomp_update.pyx":1337
 *                 ldu = u.shape[1]
 *             else:
 *                 u = PyArray_FromArray(u, NULL, cnp.NPY_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 utrans = N
 *                 uvoid = extract(u, us)
 */
      /*else*/ {
        __pyx_t_2 = ((PyObject *)PyArray_FromArray(__pyx_v_u, NULL, NPY_F_CONTIGUOUS)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1337, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF_SET(__pyx_v_u, ((PyArrayObject *)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "scipy/linalg/_decomp_update.pyx":1338
 *             else:
 *                 u = PyArray_FromArray(u, NULL, cnp.NPY_F_CONTIGUOUS)
 *                 utrans = N             # <<<<<<<<<<<<<<
 *                 uvoid = extract(u, us)
 *                 ldu = u.shape[0]
 */
        __pyx_v_utrans = __pyx_v_N;

        /* "scipy/linalg/_decomp_update.pyx":1339
 *                 u = PyArray_FromArray(u, NULL, cnp.NPY_F_CONTIGUOUS)
 *                 utrans = N
 *                 uvoid = extract(u, us)             # <<<<<<<<<<<<<<
 *                 ldu = u.shape[0]
 *             if typecode == cnp.NPY_FLOAT:
 */
        __pyx_v_uvoid = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_u, __pyx_v_us);

        /* "scipy/linalg/_decomp_update.pyx":1340
 *                 utrans = N
 *                 uvoid = extract(u, us)
 *                 ldu = u.shape[0]             # <<<<<<<<<<<<<<
 *             if typecode == cnp.NPY_FLOAT:
 *                 gemm(N, utrans, m, p, m, 1, <float*>qvoid, n,
 */
        __pyx_v_ldu = (__pyx_v_u->dimensions[0]);
      }
      __pyx_L17:;

      /* "scipy/linalg/_decomp_update.pyx":1341
 *                 uvoid = extract(u, us)
 *                 ldu = u.shape[0]
 *             if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                 gemm(N, utrans, m, p, m, 1, <float*>qvoid, n,
 *                         <float*>uvoid, ldu, 0, col(<float*>qTuvoid, qTus, k), m)
 */
      switch (__pyx_v_typecode) {
        case NPY_FLOAT:

        /* "scipy/linalg/_decomp_update.pyx":1342
 *                 ldu = u.shape[0]
 *             if typecode == cnp.NPY_FLOAT:
 *                 gemm(N, utrans, m, p, m, 1, <float*>qvoid, n,             # <<<<<<<<<<<<<<
 *                         <float*>uvoid, ldu, 0, col(<float*>qTuvoid, qTus, k), m)
 *             elif typecode == cnp.NPY_DOUBLE:
 */
        __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_gemm(__pyx_v_N, __pyx_v_utrans, __pyx_v_m, __pyx_v_p, __pyx_v_m, 1.0, ((float *)__pyx_v_qvoid), __pyx_v_n, ((float *)__pyx_v_uvoid), __pyx_v_ldu, 0.0, __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_col(((float *)__pyx_v_qTuvoid), __pyx_v_qTus, __pyx_v_k), __pyx_v_m);

        /* "scipy/linalg/_decomp_update.pyx":1341
 *                 uvoid = extract(u, us)
 *                 ldu = u.shape[0]
 *             if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                 gemm(N, utrans, m, p, m, 1, <float*>qvoid, n,
 *                         <float*>uvoid, ldu, 0, col(<float*>qTuvoid, qTus, k), m)
 */
        break;
        case NPY_DOUBLE:

        /* "scipy/linalg/_decomp_update.pyx":1345
 *                         <float*>uvoid, ldu, 0, col(<float*>qTuvoid, qTus, k), m)
 *             elif typecode == cnp.NPY_DOUBLE:
 *                 gemm(N, utrans, m, p, m, 1, <double*>qvoid, n,             # <<<<<<<<<<<<<<
 *                         <double*>uvoid, ldu, 0, col(<double*>qTuvoid, qTus, k), m)
 *             elif typecode == cnp.NPY_CFLOAT:
 */
        __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_gemm(__pyx_v_N, __pyx_v_utrans, __pyx_v_m, __pyx_v_p, __pyx_v_m, 1.0, ((double *)__pyx_v_qvoid), __pyx_v_n, ((double *)__pyx_v_uvoid), __pyx_v_ldu, 0.0, __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_col(((double *)__pyx_v_qTuvoid), __pyx_v_qTus, __pyx_v_k), __pyx_v_m);

        /* "scipy/linalg/_decomp_update.pyx":1344
 *                 gemm(N, utrans, m, p, m, 1, <float*>qvoid, n,
 *                         <float*>uvoid, ldu, 0, col(<float*>qTuvoid, qTus, k), m)
 *             elif typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                 gemm(N, utrans, m, p, m, 1, <double*>qvoid, n,
 *                         <double*>uvoid, ldu, 0, col(<double*>qTuvoid, qTus, k), m)
 */
        break;
        case NPY_CFLOAT:

        /* "scipy/linalg/_decomp_update.pyx":1348
 *                         <double*>uvoid, ldu, 0, col(<double*>qTuvoid, qTus, k), m)
 *             elif typecode == cnp.NPY_CFLOAT:
 *                 blas_t_2d_conj(m, p, <float_complex*>uvoid, us)             # <<<<<<<<<<<<<<
 *                 gemm(N, utrans, m, p, m, 1, <float_complex*>qvoid, n,
 *                         <float_complex*>uvoid, ldu, 0,
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_2d_conj(__pyx_v_m, __pyx_v_p, ((__pyx_t_float_complex *)__pyx_v_uvoid), __pyx_v_us);

        /* "scipy/linalg/_decomp_update.pyx":1349
 *             elif typecode == cnp.NPY_CFLOAT:
 *                 blas_t_2d_conj(m, p, <float_complex*>uvoid, us)
 *                 gemm(N, utrans, m, p, m, 1, <float_complex*>qvoid, n,             # <<<<<<<<<<<<<<
 *                         <float_complex*>uvoid, ldu, 0,
 *                         col(<float_complex*>qTuvoid, qTus, k), m)
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_gemm(__pyx_v_N, __pyx_v_utrans, __pyx_v_m, __pyx_v_p, __pyx_v_m, __pyx_t_float_complex_from_parts(1, 0), ((__pyx_t_float_complex *)__pyx_v_qvoid), __pyx_v_n, ((__pyx_t_float_complex *)__pyx_v_uvoid), __pyx_v_ldu, __pyx_t_float_complex_from_parts(0, 0), __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(((__pyx_t_float_complex *)__pyx_v_qTuvoid), __pyx_v_qTus, __pyx_v_k), __pyx_v_m);

        /* "scipy/linalg/_decomp_update.pyx":1352
 *                         <float_complex*>uvoid, ldu, 0,
 *                         col(<float_complex*>qTuvoid, qTus, k), m)
 *                 blas_t_2d_conj(m, p, col(<float_complex*>qTuvoid, qTus, k), qTus)             # <<<<<<<<<<<<<<
 *             elif typecode == cnp.NPY_CDOUBLE:
 *                 blas_t_2d_conj(m, p, <double_complex*>uvoid, us)
 */
        __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_2d_conj(__pyx_v_m, __pyx_v_p, __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_col(((__pyx_t_float_complex *)__pyx_v_qTuvoid), __pyx_v_qTus, __pyx_v_k), __pyx_v_qTus);

        /* "scipy/linalg/_decomp_update.pyx":1347
 *                 gemm(N, utrans, m, p, m, 1, <double*>qvoid, n,
 *                         <double*>uvoid, ldu, 0, col(<double*>qTuvoid, qTus, k), m)
 *             elif typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                 blas_t_2d_conj(m, p, <float_complex*>uvoid, us)
 *                 gemm(N, utrans, m, p, m, 1, <float_complex*>qvoid, n,
 */
        break;
        case NPY_CDOUBLE:

        /* "scipy/linalg/_decomp_update.pyx":1354
 *                 blas_t_2d_conj(m, p, col(<float_complex*>qTuvoid, qTus, k), qTus)
 *             elif typecode == cnp.NPY_CDOUBLE:
 *                 blas_t_2d_conj(m, p, <double_complex*>uvoid, us)             # <<<<<<<<<<<<<<
 *                 gemm(N, utrans, m, p, m, 1, <double_complex*>qvoid, n,
 *                         <double_complex*>uvoid, ldu, 0,
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_2d_conj(__pyx_v_m, __pyx_v_p, ((__pyx_t_double_complex *)__pyx_v_uvoid), __pyx_v_us);

        /* "scipy/linalg/_decomp_update.pyx":1355
 *             elif typecode == cnp.NPY_CDOUBLE:
 *                 blas_t_2d_conj(m, p, <double_complex*>uvoid, us)
 *                 gemm(N, utrans, m, p, m, 1, <double_complex*>qvoid, n,             # <<<<<<<<<<<<<<
 *                         <double_complex*>uvoid, ldu, 0,
 *                         col(<double_complex*>qTuvoid, qTus, k), m)
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_gemm(__pyx_v_N, __pyx_v_utrans, __pyx_v_m, __pyx_v_p, __pyx_v_m, __pyx_t_double_complex_from_parts(1, 0), ((__pyx_t_double_complex *)__pyx_v_qvoid), __pyx_v_n, ((__pyx_t_double_complex *)__pyx_v_uvoid), __pyx_v_ldu, __pyx_t_double_complex_from_parts(0, 0), __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(((__pyx_t_double_complex *)__pyx_v_qTuvoid), __pyx_v_qTus, __pyx_v_k), __pyx_v_m);

        /* "scipy/linalg/_decomp_update.pyx":1358
 *                         <double_complex*>uvoid, ldu, 0,
 *                         col(<double_complex*>qTuvoid, qTus, k), m)
 *                 blas_t_2d_conj(m, p, col(<double_complex*>qTuvoid, qTus, k), qTus)             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError('Shape of u is incorrect, should be 1 <= u.ndim <= 2')
 */
        __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_blas_t_2d_conj(__pyx_v_m, __pyx_v_p, __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_col(((__pyx_t_double_complex *)__pyx_v_qTuvoid), __pyx_v_qTus, __pyx_v_k), __pyx_v_qTus);

        /* "scipy/linalg/_decomp_update.pyx":1353
 *                         col(<float_complex*>qTuvoid, qTus, k), m)
 *                 blas_t_2d_conj(m, p, col(<float_complex*>qTuvoid, qTus, k), qTus)
 *             elif typecode == cnp.NPY_CDOUBLE:             # <<<<<<<<<<<<<<
 *                 blas_t_2d_conj(m, p, <double_complex*>uvoid, us)
 *                 gemm(N, utrans, m, p, m, 1, <double_complex*>qvoid, n,
 */
        break;
        default: break;
      }

      /* "scipy/linalg/_decomp_update.pyx":1326
 *                         col(<double_complex*>qTuvoid, qTus, k), qTus[0])
 *                 blas_t_conj(m, col(<double_complex*>qTuvoid, qTus, k), qTus)
 *         elif u.ndim == 2:             # <<<<<<<<<<<<<<
 *             p = u.shape[1]
 *             if cnp.PyArray_CHKFLAGS(u, cnp.NPY_F_CONTIGUOUS):
 */
      break;
      default:

      /* "scipy/linalg/_decomp_update.pyx":1360
 *                 blas_t_2d_conj(m, p, col(<double_complex*>qTuvoid, qTus, k), qTus)
 *         else:
 *             raise ValueError('Shape of u is incorrect, should be 1 <= u.ndim <= 2')             # <<<<<<<<<<<<<<
 *     else:
 *         raise ValueError('q must be either F or C contiguous')
 */
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1360, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 1360, __pyx_L1_error)
      break;
    }

    /* "scipy/linalg/_decomp_update.pyx":1304
 *                         col(<double_complex*>qTuvoid, qTus, k), m)
 * 
 *     elif cnp.PyArray_CHKFLAGS(q, cnp.NPY_C_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *         qvoid = extract(q, qs)
 *         if u.ndim == 1:
 */
    goto __pyx_L3;
  }

  /* "scipy/linalg/_decomp_update.pyx":1362
 *             raise ValueError('Shape of u is incorrect, should be 1 <= u.ndim <= 2')
 *     else:
 *         raise ValueError('q must be either F or C contiguous')             # <<<<<<<<<<<<<<
 * 
 * cdef validate_array(cnp.ndarray a, bint chkfinite):
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1362, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1362, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "scipy/linalg/_decomp_update.pyx":1232
 *     return qTu
 * 
 * cdef form_qTu(cnp.ndarray q, cnp.ndarray u, void* qTuvoid, int* qTus,             # <<<<<<<<<<<<<<
 *               int k):
 *     """ assuming here that q and u have compatible shapes, and are the same
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("scipy.linalg._decomp_update.form_qTu", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_u);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":1364
 *         raise ValueError('q must be either F or C contiguous')
 * 
 * cdef validate_array(cnp.ndarray a, bint chkfinite):             # <<<<<<<<<<<<<<
 *     # here we check that a has positive strides and that its size is small
 *     # enough to fit in into an int, as BLAS/LAPACK require
 */

static PyObject *__pyx_f_5scipy_6linalg_14_decomp_update_validate_array(PyArrayObject *__pyx_v_a, int __pyx_v_chkfinite) {
  int __pyx_v_copy;
  int __pyx_v_j;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  __Pyx_RefNannySetupContext("validate_array", 0);

  /* "scipy/linalg/_decomp_update.pyx":1367
 *     # here we check that a has positive strides and that its size is small
 *     # enough to fit in into an int, as BLAS/LAPACK require
 *     cdef bint copy = False             # <<<<<<<<<<<<<<
 *     cdef int j
 * 
 */
  __pyx_v_copy = 0;

  /* "scipy/linalg/_decomp_update.pyx":1370
 *     cdef int j
 * 
 *     for j in range(a.ndim):             # <<<<<<<<<<<<<<
 *         if a.strides[j] <= 0:
 *             copy = True
 */
  __pyx_t_1 = __pyx_v_a->nd;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "scipy/linalg/_decomp_update.pyx":1371
 * 
 *     for j in range(a.ndim):
 *         if a.strides[j] <= 0:             # <<<<<<<<<<<<<<
 *             copy = True
 *         if (a.strides[j] / a.descr.itemsize) >= libc.limits.INT_MAX:
 */
    __pyx_t_4 = (((__pyx_v_a->strides[__pyx_v_j]) <= 0) != 0);
    if (__pyx_t_4) {

      /* "scipy/linalg/_decomp_update.pyx":1372
 *     for j in range(a.ndim):
 *         if a.strides[j] <= 0:
 *             copy = True             # <<<<<<<<<<<<<<
 *         if (a.strides[j] / a.descr.itemsize) >= libc.limits.INT_MAX:
 *             copy = True
 */
      __pyx_v_copy = 1;

      /* "scipy/linalg/_decomp_update.pyx":1371
 * 
 *     for j in range(a.ndim):
 *         if a.strides[j] <= 0:             # <<<<<<<<<<<<<<
 *             copy = True
 *         if (a.strides[j] / a.descr.itemsize) >= libc.limits.INT_MAX:
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1373
 *         if a.strides[j] <= 0:
 *             copy = True
 *         if (a.strides[j] / a.descr.itemsize) >= libc.limits.INT_MAX:             # <<<<<<<<<<<<<<
 *             copy = True
 *         if a.shape[j] >= libc.limits.INT_MAX:
 */
    if (unlikely(__pyx_v_a->descr->elsize == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      __PYX_ERR(0, 1373, __pyx_L1_error)
    }
    __pyx_t_4 = (((((double)(__pyx_v_a->strides[__pyx_v_j])) / ((double)__pyx_v_a->descr->elsize)) >= INT_MAX) != 0);
    if (__pyx_t_4) {

      /* "scipy/linalg/_decomp_update.pyx":1374
 *             copy = True
 *         if (a.strides[j] / a.descr.itemsize) >= libc.limits.INT_MAX:
 *             copy = True             # <<<<<<<<<<<<<<
 *         if a.shape[j] >= libc.limits.INT_MAX:
 *             raise ValueError('Input array too large for use with BLAS')
 */
      __pyx_v_copy = 1;

      /* "scipy/linalg/_decomp_update.pyx":1373
 *         if a.strides[j] <= 0:
 *             copy = True
 *         if (a.strides[j] / a.descr.itemsize) >= libc.limits.INT_MAX:             # <<<<<<<<<<<<<<
 *             copy = True
 *         if a.shape[j] >= libc.limits.INT_MAX:
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1375
 *         if (a.strides[j] / a.descr.itemsize) >= libc.limits.INT_MAX:
 *             copy = True
 *         if a.shape[j] >= libc.limits.INT_MAX:             # <<<<<<<<<<<<<<
 *             raise ValueError('Input array too large for use with BLAS')
 * 
 */
    __pyx_t_4 = (((__pyx_v_a->dimensions[__pyx_v_j]) >= INT_MAX) != 0);
    if (unlikely(__pyx_t_4)) {

      /* "scipy/linalg/_decomp_update.pyx":1376
 *             copy = True
 *         if a.shape[j] >= libc.limits.INT_MAX:
 *             raise ValueError('Input array too large for use with BLAS')             # <<<<<<<<<<<<<<
 * 
 *     if chkfinite:
 */
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1376, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_Raise(__pyx_t_5, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __PYX_ERR(0, 1376, __pyx_L1_error)

      /* "scipy/linalg/_decomp_update.pyx":1375
 *         if (a.strides[j] / a.descr.itemsize) >= libc.limits.INT_MAX:
 *             copy = True
 *         if a.shape[j] >= libc.limits.INT_MAX:             # <<<<<<<<<<<<<<
 *             raise ValueError('Input array too large for use with BLAS')
 * 
 */
    }
  }

  /* "scipy/linalg/_decomp_update.pyx":1378
 *             raise ValueError('Input array too large for use with BLAS')
 * 
 *     if chkfinite:             # <<<<<<<<<<<<<<
 *         if not np.isfinite(a).all():
 *             raise ValueError('array must not contain infs or NaNs')
 */
  __pyx_t_4 = (__pyx_v_chkfinite != 0);
  if (__pyx_t_4) {

    /* "scipy/linalg/_decomp_update.pyx":1379
 * 
 *     if chkfinite:
 *         if not np.isfinite(a).all():             # <<<<<<<<<<<<<<
 *             raise ValueError('array must not contain infs or NaNs')
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_np); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1379, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_isfinite); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1379, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_6 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_7, ((PyObject *)__pyx_v_a)) : __Pyx_PyObject_CallOneArg(__pyx_t_8, ((PyObject *)__pyx_v_a));
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1379, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_all); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1379, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_5 = (__pyx_t_6) ? __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallNoArg(__pyx_t_8);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1379, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1379, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_9 = ((!__pyx_t_4) != 0);
    if (unlikely(__pyx_t_9)) {

      /* "scipy/linalg/_decomp_update.pyx":1380
 *     if chkfinite:
 *         if not np.isfinite(a).all():
 *             raise ValueError('array must not contain infs or NaNs')             # <<<<<<<<<<<<<<
 * 
 *     if copy:
 */
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1380, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_Raise(__pyx_t_5, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __PYX_ERR(0, 1380, __pyx_L1_error)

      /* "scipy/linalg/_decomp_update.pyx":1379
 * 
 *     if chkfinite:
 *         if not np.isfinite(a).all():             # <<<<<<<<<<<<<<
 *             raise ValueError('array must not contain infs or NaNs')
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1378
 *             raise ValueError('Input array too large for use with BLAS')
 * 
 *     if chkfinite:             # <<<<<<<<<<<<<<
 *         if not np.isfinite(a).all():
 *             raise ValueError('array must not contain infs or NaNs')
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1382
 *             raise ValueError('array must not contain infs or NaNs')
 * 
 *     if copy:             # <<<<<<<<<<<<<<
 *         return PyArray_FromArray(a, NULL, cnp.NPY_F_CONTIGUOUS)
 *     return a
 */
  __pyx_t_9 = (__pyx_v_copy != 0);
  if (__pyx_t_9) {

    /* "scipy/linalg/_decomp_update.pyx":1383
 * 
 *     if copy:
 *         return PyArray_FromArray(a, NULL, cnp.NPY_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *     return a
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_5 = ((PyObject *)PyArray_FromArray(__pyx_v_a, NULL, NPY_F_CONTIGUOUS)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1383, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":1382
 *             raise ValueError('array must not contain infs or NaNs')
 * 
 *     if copy:             # <<<<<<<<<<<<<<
 *         return PyArray_FromArray(a, NULL, cnp.NPY_F_CONTIGUOUS)
 *     return a
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1384
 *     if copy:
 *         return PyArray_FromArray(a, NULL, cnp.NPY_F_CONTIGUOUS)
 *     return a             # <<<<<<<<<<<<<<
 * 
 * cdef tuple validate_qr(object q0, object r0, bint overwrite_q, int q_order,
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_a));
  __pyx_r = ((PyObject *)__pyx_v_a);
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":1364
 *         raise ValueError('q must be either F or C contiguous')
 * 
 * cdef validate_array(cnp.ndarray a, bint chkfinite):             # <<<<<<<<<<<<<<
 *     # here we check that a has positive strides and that its size is small
 *     # enough to fit in into an int, as BLAS/LAPACK require
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("scipy.linalg._decomp_update.validate_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":1386
 *     return a
 * 
 * cdef tuple validate_qr(object q0, object r0, bint overwrite_q, int q_order,             # <<<<<<<<<<<<<<
 *                        bint overwrite_r, int r_order, bint chkfinite):
 *     cdef cnp.ndarray Q
 */

static PyObject *__pyx_f_5scipy_6linalg_14_decomp_update_validate_qr(PyObject *__pyx_v_q0, PyObject *__pyx_v_r0, int __pyx_v_overwrite_q, int __pyx_v_q_order, int __pyx_v_overwrite_r, int __pyx_v_r_order, int __pyx_v_chkfinite) {
  PyArrayObject *__pyx_v_Q = 0;
  PyArrayObject *__pyx_v_R = 0;
  int __pyx_v_typecode;
  int __pyx_v_economic;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_UCS4 __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("validate_qr", 0);

  /* "scipy/linalg/_decomp_update.pyx":1391
 *     cdef cnp.ndarray R
 *     cdef int typecode
 *     cdef bint economic = False             # <<<<<<<<<<<<<<
 * 
 *     q_order |= cnp.NPY_BEHAVED_NS | cnp.NPY_ELEMENTSTRIDES
 */
  __pyx_v_economic = 0;

  /* "scipy/linalg/_decomp_update.pyx":1393
 *     cdef bint economic = False
 * 
 *     q_order |= cnp.NPY_BEHAVED_NS | cnp.NPY_ELEMENTSTRIDES             # <<<<<<<<<<<<<<
 *     r_order |= cnp.NPY_BEHAVED_NS | cnp.NPY_ELEMENTSTRIDES
 * 
 */
  __pyx_v_q_order = (__pyx_v_q_order | (NPY_BEHAVED_NS | NPY_ELEMENTSTRIDES));

  /* "scipy/linalg/_decomp_update.pyx":1394
 * 
 *     q_order |= cnp.NPY_BEHAVED_NS | cnp.NPY_ELEMENTSTRIDES
 *     r_order |= cnp.NPY_BEHAVED_NS | cnp.NPY_ELEMENTSTRIDES             # <<<<<<<<<<<<<<
 * 
 *     if not overwrite_q:
 */
  __pyx_v_r_order = (__pyx_v_r_order | (NPY_BEHAVED_NS | NPY_ELEMENTSTRIDES));

  /* "scipy/linalg/_decomp_update.pyx":1396
 *     r_order |= cnp.NPY_BEHAVED_NS | cnp.NPY_ELEMENTSTRIDES
 * 
 *     if not overwrite_q:             # <<<<<<<<<<<<<<
 *         q_order |= cnp.NPY_ENSURECOPY
 * 
 */
  __pyx_t_1 = ((!(__pyx_v_overwrite_q != 0)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":1397
 * 
 *     if not overwrite_q:
 *         q_order |= cnp.NPY_ENSURECOPY             # <<<<<<<<<<<<<<
 * 
 *     if not overwrite_r:
 */
    __pyx_v_q_order = (__pyx_v_q_order | NPY_ENSURECOPY);

    /* "scipy/linalg/_decomp_update.pyx":1396
 *     r_order |= cnp.NPY_BEHAVED_NS | cnp.NPY_ELEMENTSTRIDES
 * 
 *     if not overwrite_q:             # <<<<<<<<<<<<<<
 *         q_order |= cnp.NPY_ENSURECOPY
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1399
 *         q_order |= cnp.NPY_ENSURECOPY
 * 
 *     if not overwrite_r:             # <<<<<<<<<<<<<<
 *         r_order |= cnp.NPY_ENSURECOPY
 * 
 */
  __pyx_t_1 = ((!(__pyx_v_overwrite_r != 0)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":1400
 * 
 *     if not overwrite_r:
 *         r_order |= cnp.NPY_ENSURECOPY             # <<<<<<<<<<<<<<
 * 
 *     # in the interests of giving better error messages take any number of
 */
    __pyx_v_r_order = (__pyx_v_r_order | NPY_ENSURECOPY);

    /* "scipy/linalg/_decomp_update.pyx":1399
 *         q_order |= cnp.NPY_ENSURECOPY
 * 
 *     if not overwrite_r:             # <<<<<<<<<<<<<<
 *         r_order |= cnp.NPY_ENSURECOPY
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1404
 *     # in the interests of giving better error messages take any number of
 *     # dimensions here.
 *     Q = PyArray_CheckFromAny(q0, NULL, 0, 0, q_order, NULL)             # <<<<<<<<<<<<<<
 *     R = PyArray_CheckFromAny(r0, NULL, 0, 0, r_order, NULL)
 * 
 */
  __pyx_t_2 = ((PyObject *)PyArray_CheckFromAny(__pyx_v_q0, NULL, 0, 0, __pyx_v_q_order, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1404, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_Q = ((PyArrayObject *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "scipy/linalg/_decomp_update.pyx":1405
 *     # dimensions here.
 *     Q = PyArray_CheckFromAny(q0, NULL, 0, 0, q_order, NULL)
 *     R = PyArray_CheckFromAny(r0, NULL, 0, 0, r_order, NULL)             # <<<<<<<<<<<<<<
 * 
 *     if Q.ndim != 2 or R.ndim != 2:
 */
  __pyx_t_2 = ((PyObject *)PyArray_CheckFromAny(__pyx_v_r0, NULL, 0, 0, __pyx_v_r_order, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1405, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_R = ((PyArrayObject *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "scipy/linalg/_decomp_update.pyx":1407
 *     R = PyArray_CheckFromAny(r0, NULL, 0, 0, r_order, NULL)
 * 
 *     if Q.ndim != 2 or R.ndim != 2:             # <<<<<<<<<<<<<<
 *         raise ValueError('Q and R must be 2-D')
 * 
 */
  __pyx_t_3 = ((__pyx_v_Q->nd != 2) != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_3 = ((__pyx_v_R->nd != 2) != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L6_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "scipy/linalg/_decomp_update.pyx":1408
 * 
 *     if Q.ndim != 2 or R.ndim != 2:
 *         raise ValueError('Q and R must be 2-D')             # <<<<<<<<<<<<<<
 * 
 *     typecode = cnp.PyArray_TYPE(Q)
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1408, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1408, __pyx_L1_error)

    /* "scipy/linalg/_decomp_update.pyx":1407
 *     R = PyArray_CheckFromAny(r0, NULL, 0, 0, r_order, NULL)
 * 
 *     if Q.ndim != 2 or R.ndim != 2:             # <<<<<<<<<<<<<<
 *         raise ValueError('Q and R must be 2-D')
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1410
 *         raise ValueError('Q and R must be 2-D')
 * 
 *     typecode = cnp.PyArray_TYPE(Q)             # <<<<<<<<<<<<<<
 * 
 *     if typecode != cnp.PyArray_TYPE(R):
 */
  __pyx_v_typecode = PyArray_TYPE(__pyx_v_Q);

  /* "scipy/linalg/_decomp_update.pyx":1412
 *     typecode = cnp.PyArray_TYPE(Q)
 * 
 *     if typecode != cnp.PyArray_TYPE(R):             # <<<<<<<<<<<<<<
 *         raise ValueError('Q and R must have the same dtype')
 * 
 */
  __pyx_t_1 = ((__pyx_v_typecode != PyArray_TYPE(__pyx_v_R)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "scipy/linalg/_decomp_update.pyx":1413
 * 
 *     if typecode != cnp.PyArray_TYPE(R):
 *         raise ValueError('Q and R must have the same dtype')             # <<<<<<<<<<<<<<
 * 
 *     if not (typecode == cnp.NPY_FLOAT or typecode == cnp.NPY_DOUBLE
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1413, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1413, __pyx_L1_error)

    /* "scipy/linalg/_decomp_update.pyx":1412
 *     typecode = cnp.PyArray_TYPE(Q)
 * 
 *     if typecode != cnp.PyArray_TYPE(R):             # <<<<<<<<<<<<<<
 *         raise ValueError('Q and R must have the same dtype')
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1415
 *         raise ValueError('Q and R must have the same dtype')
 * 
 *     if not (typecode == cnp.NPY_FLOAT or typecode == cnp.NPY_DOUBLE             # <<<<<<<<<<<<<<
 *             or typecode == cnp.NPY_CFLOAT or typecode == cnp.NPY_CDOUBLE):
 *         raise ValueError('Only arrays with dtypes float32, float64, '
 */
  switch (__pyx_v_typecode) {
    case NPY_FLOAT:
    case NPY_DOUBLE:
    case NPY_CFLOAT:

    /* "scipy/linalg/_decomp_update.pyx":1416
 * 
 *     if not (typecode == cnp.NPY_FLOAT or typecode == cnp.NPY_DOUBLE
 *             or typecode == cnp.NPY_CFLOAT or typecode == cnp.NPY_CDOUBLE):             # <<<<<<<<<<<<<<
 *         raise ValueError('Only arrays with dtypes float32, float64, '
 *                          'complex64, and complex128 are supported.')
 */
    case NPY_CDOUBLE:

    /* "scipy/linalg/_decomp_update.pyx":1415
 *         raise ValueError('Q and R must have the same dtype')
 * 
 *     if not (typecode == cnp.NPY_FLOAT or typecode == cnp.NPY_DOUBLE             # <<<<<<<<<<<<<<
 *             or typecode == cnp.NPY_CFLOAT or typecode == cnp.NPY_CDOUBLE):
 *         raise ValueError('Only arrays with dtypes float32, float64, '
 */
    __pyx_t_1 = 1;
    break;
    default:
    __pyx_t_1 = 0;
    break;
  }
  __pyx_t_3 = ((!__pyx_t_1) != 0);
  if (unlikely(__pyx_t_3)) {

    /* "scipy/linalg/_decomp_update.pyx":1417
 *     if not (typecode == cnp.NPY_FLOAT or typecode == cnp.NPY_DOUBLE
 *             or typecode == cnp.NPY_CFLOAT or typecode == cnp.NPY_CDOUBLE):
 *         raise ValueError('Only arrays with dtypes float32, float64, '             # <<<<<<<<<<<<<<
 *                          'complex64, and complex128 are supported.')
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1417, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1417, __pyx_L1_error)

    /* "scipy/linalg/_decomp_update.pyx":1415
 *         raise ValueError('Q and R must have the same dtype')
 * 
 *     if not (typecode == cnp.NPY_FLOAT or typecode == cnp.NPY_DOUBLE             # <<<<<<<<<<<<<<
 *             or typecode == cnp.NPY_CFLOAT or typecode == cnp.NPY_CDOUBLE):
 *         raise ValueError('Only arrays with dtypes float32, float64, '
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1421
 * 
 *     # we support MxM MxN and MxN NxN
 *     if Q.shape[1] != R.shape[0]:             # <<<<<<<<<<<<<<
 *         raise ValueError('Q and R do not have compatible shapes. Expected '
 *                          '(M,M) (M,N) or (M,N) (N,N) but found %s %s for Q '
 */
  __pyx_t_3 = (((__pyx_v_Q->dimensions[1]) != (__pyx_v_R->dimensions[0])) != 0);
  if (unlikely(__pyx_t_3)) {

    /* "scipy/linalg/_decomp_update.pyx":1422
 *     # we support MxM MxN and MxN NxN
 *     if Q.shape[1] != R.shape[0]:
 *         raise ValueError('Q and R do not have compatible shapes. Expected '             # <<<<<<<<<<<<<<
 *                          '(M,M) (M,N) or (M,N) (N,N) but found %s %s for Q '
 *                          'and R respectively' %
 */
    __pyx_t_2 = PyTuple_New(5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1422, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = 0;
    __pyx_t_5 = 127;
    __Pyx_INCREF(__pyx_kp_u_Q_and_R_do_not_have_compatible_s);
    __pyx_t_4 += 85;
    __Pyx_GIVEREF(__pyx_kp_u_Q_and_R_do_not_have_compatible_s);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_kp_u_Q_and_R_do_not_have_compatible_s);

    /* "scipy/linalg/_decomp_update.pyx":1425
 *                          '(M,M) (M,N) or (M,N) (N,N) but found %s %s for Q '
 *                          'and R respectively' %
 *                          (str(getattr(Q, 'shape')), str(getattr(R, 'shape'))))             # <<<<<<<<<<<<<<
 * 
 *     # so one or the other or both should be square.
 */
    __pyx_t_6 = __Pyx_GetAttr(((PyObject *)__pyx_v_Q), __pyx_n_u_shape); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyUnicode_Type)), __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_5 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) > __pyx_t_5) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) : __pyx_t_5;
    __pyx_t_4 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_7);
    __pyx_t_7 = 0;
    __Pyx_INCREF(__pyx_kp_u__11);
    __pyx_t_4 += 1;
    __Pyx_GIVEREF(__pyx_kp_u__11);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_kp_u__11);
    __pyx_t_7 = __Pyx_GetAttr(((PyObject *)__pyx_v_R), __pyx_n_u_shape); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyUnicode_Type)), __pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_5 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) > __pyx_t_5) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) : __pyx_t_5;
    __pyx_t_4 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_6);
    __pyx_t_6 = 0;
    __Pyx_INCREF(__pyx_kp_u_for_Q_and_R_respectively);
    __pyx_t_4 += 25;
    __Pyx_GIVEREF(__pyx_kp_u_for_Q_and_R_respectively);
    PyTuple_SET_ITEM(__pyx_t_2, 4, __pyx_kp_u_for_Q_and_R_respectively);

    /* "scipy/linalg/_decomp_update.pyx":1422
 *     # we support MxM MxN and MxN NxN
 *     if Q.shape[1] != R.shape[0]:
 *         raise ValueError('Q and R do not have compatible shapes. Expected '             # <<<<<<<<<<<<<<
 *                          '(M,M) (M,N) or (M,N) (N,N) but found %s %s for Q '
 *                          'and R respectively' %
 */
    __pyx_t_6 = __Pyx_PyUnicode_Join(__pyx_t_2, 5, __pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1422, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1422, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1422, __pyx_L1_error)

    /* "scipy/linalg/_decomp_update.pyx":1421
 * 
 *     # we support MxM MxN and MxN NxN
 *     if Q.shape[1] != R.shape[0]:             # <<<<<<<<<<<<<<
 *         raise ValueError('Q and R do not have compatible shapes. Expected '
 *                          '(M,M) (M,N) or (M,N) (N,N) but found %s %s for Q '
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1428
 * 
 *     # so one or the other or both should be square.
 *     if Q.shape[0] != Q.shape[1] and R.shape[0] == R.shape[1]:             # <<<<<<<<<<<<<<
 *         economic = True
 *     elif Q.shape[0] != Q.shape[1]:
 */
  __pyx_t_1 = (((__pyx_v_Q->dimensions[0]) != (__pyx_v_Q->dimensions[1])) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_1 = (((__pyx_v_R->dimensions[0]) == (__pyx_v_R->dimensions[1])) != 0);
  __pyx_t_3 = __pyx_t_1;
  __pyx_L12_bool_binop_done:;
  if (__pyx_t_3) {

    /* "scipy/linalg/_decomp_update.pyx":1429
 *     # so one or the other or both should be square.
 *     if Q.shape[0] != Q.shape[1] and R.shape[0] == R.shape[1]:
 *         economic = True             # <<<<<<<<<<<<<<
 *     elif Q.shape[0] != Q.shape[1]:
 *         raise ValueError('Expected (M,M) (M,N) or (M,N) (N,N) but found %s %s '
 */
    __pyx_v_economic = 1;

    /* "scipy/linalg/_decomp_update.pyx":1428
 * 
 *     # so one or the other or both should be square.
 *     if Q.shape[0] != Q.shape[1] and R.shape[0] == R.shape[1]:             # <<<<<<<<<<<<<<
 *         economic = True
 *     elif Q.shape[0] != Q.shape[1]:
 */
    goto __pyx_L11;
  }

  /* "scipy/linalg/_decomp_update.pyx":1430
 *     if Q.shape[0] != Q.shape[1] and R.shape[0] == R.shape[1]:
 *         economic = True
 *     elif Q.shape[0] != Q.shape[1]:             # <<<<<<<<<<<<<<
 *         raise ValueError('Expected (M,M) (M,N) or (M,N) (N,N) but found %s %s '
 *                          'for Q and R respectively' %
 */
  __pyx_t_3 = (((__pyx_v_Q->dimensions[0]) != (__pyx_v_Q->dimensions[1])) != 0);
  if (unlikely(__pyx_t_3)) {

    /* "scipy/linalg/_decomp_update.pyx":1431
 *         economic = True
 *     elif Q.shape[0] != Q.shape[1]:
 *         raise ValueError('Expected (M,M) (M,N) or (M,N) (N,N) but found %s %s '             # <<<<<<<<<<<<<<
 *                          'for Q and R respectively' %
 *                          (str(getattr(Q, 'shape')), str(getattr(R, 'shape'))))
 */
    __pyx_t_2 = PyTuple_New(5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = 0;
    __pyx_t_5 = 127;
    __Pyx_INCREF(__pyx_kp_u_Expected_M_M_M_N_or_M_N_N_N_but);
    __pyx_t_4 += 46;
    __Pyx_GIVEREF(__pyx_kp_u_Expected_M_M_M_N_or_M_N_N_N_but);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_kp_u_Expected_M_M_M_N_or_M_N_N_N_but);

    /* "scipy/linalg/_decomp_update.pyx":1433
 *         raise ValueError('Expected (M,M) (M,N) or (M,N) (N,N) but found %s %s '
 *                          'for Q and R respectively' %
 *                          (str(getattr(Q, 'shape')), str(getattr(R, 'shape'))))             # <<<<<<<<<<<<<<
 * 
 *     Q = validate_array(Q, chkfinite)
 */
    __pyx_t_6 = __Pyx_GetAttr(((PyObject *)__pyx_v_Q), __pyx_n_u_shape); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1433, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyUnicode_Type)), __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1433, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_5 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) > __pyx_t_5) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) : __pyx_t_5;
    __pyx_t_4 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_7);
    __pyx_t_7 = 0;
    __Pyx_INCREF(__pyx_kp_u__11);
    __pyx_t_4 += 1;
    __Pyx_GIVEREF(__pyx_kp_u__11);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_kp_u__11);
    __pyx_t_7 = __Pyx_GetAttr(((PyObject *)__pyx_v_R), __pyx_n_u_shape); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1433, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyUnicode_Type)), __pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1433, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_5 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) > __pyx_t_5) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) : __pyx_t_5;
    __pyx_t_4 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_6);
    __pyx_t_6 = 0;
    __Pyx_INCREF(__pyx_kp_u_for_Q_and_R_respectively);
    __pyx_t_4 += 25;
    __Pyx_GIVEREF(__pyx_kp_u_for_Q_and_R_respectively);
    PyTuple_SET_ITEM(__pyx_t_2, 4, __pyx_kp_u_for_Q_and_R_respectively);

    /* "scipy/linalg/_decomp_update.pyx":1431
 *         economic = True
 *     elif Q.shape[0] != Q.shape[1]:
 *         raise ValueError('Expected (M,M) (M,N) or (M,N) (N,N) but found %s %s '             # <<<<<<<<<<<<<<
 *                          'for Q and R respectively' %
 *                          (str(getattr(Q, 'shape')), str(getattr(R, 'shape'))))
 */
    __pyx_t_6 = __Pyx_PyUnicode_Join(__pyx_t_2, 5, __pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1431, __pyx_L1_error)

    /* "scipy/linalg/_decomp_update.pyx":1430
 *     if Q.shape[0] != Q.shape[1] and R.shape[0] == R.shape[1]:
 *         economic = True
 *     elif Q.shape[0] != Q.shape[1]:             # <<<<<<<<<<<<<<
 *         raise ValueError('Expected (M,M) (M,N) or (M,N) (N,N) but found %s %s '
 *                          'for Q and R respectively' %
 */
  }
  __pyx_L11:;

  /* "scipy/linalg/_decomp_update.pyx":1435
 *                          (str(getattr(Q, 'shape')), str(getattr(R, 'shape'))))
 * 
 *     Q = validate_array(Q, chkfinite)             # <<<<<<<<<<<<<<
 *     R = validate_array(R, chkfinite)
 * 
 */
  __pyx_t_2 = __pyx_f_5scipy_6linalg_14_decomp_update_validate_array(__pyx_v_Q, __pyx_v_chkfinite); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1435, __pyx_L1_error)
  __Pyx_DECREF_SET(__pyx_v_Q, ((PyArrayObject *)__pyx_t_2));
  __pyx_t_2 = 0;

  /* "scipy/linalg/_decomp_update.pyx":1436
 * 
 *     Q = validate_array(Q, chkfinite)
 *     R = validate_array(R, chkfinite)             # <<<<<<<<<<<<<<
 * 
 *     return Q, R, typecode, Q.shape[0], R.shape[1], economic
 */
  __pyx_t_2 = __pyx_f_5scipy_6linalg_14_decomp_update_validate_array(__pyx_v_R, __pyx_v_chkfinite); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1436, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1436, __pyx_L1_error)
  __Pyx_DECREF_SET(__pyx_v_R, ((PyArrayObject *)__pyx_t_2));
  __pyx_t_2 = 0;

  /* "scipy/linalg/_decomp_update.pyx":1438
 *     R = validate_array(R, chkfinite)
 * 
 *     return Q, R, typecode, Q.shape[0], R.shape[1], economic             # <<<<<<<<<<<<<<
 * 
 * cdef void* extract(cnp.ndarray arr, int* arrs):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_typecode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_Q->dimensions[0])); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_R->dimensions[1])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyBool_FromLong(__pyx_v_economic); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = PyTuple_New(6); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_INCREF(((PyObject *)__pyx_v_Q));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_Q));
  PyTuple_SET_ITEM(__pyx_t_9, 0, ((PyObject *)__pyx_v_Q));
  __Pyx_INCREF(((PyObject *)__pyx_v_R));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_R));
  PyTuple_SET_ITEM(__pyx_t_9, 1, ((PyObject *)__pyx_v_R));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_9, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_9, 3, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_9, 4, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_9, 5, __pyx_t_8);
  __pyx_t_2 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_r = ((PyObject*)__pyx_t_9);
  __pyx_t_9 = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":1386
 *     return a
 * 
 * cdef tuple validate_qr(object q0, object r0, bint overwrite_q, int q_order,             # <<<<<<<<<<<<<<
 *                        bint overwrite_r, int r_order, bint chkfinite):
 *     cdef cnp.ndarray Q
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("scipy.linalg._decomp_update.validate_qr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_Q);
  __Pyx_XDECREF((PyObject *)__pyx_v_R);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":1440
 *     return Q, R, typecode, Q.shape[0], R.shape[1], economic
 * 
 * cdef void* extract(cnp.ndarray arr, int* arrs):             # <<<<<<<<<<<<<<
 *     with cython.cdivision(True):    # Assumes itemsize != 0.
 *         if arr.ndim == 2:
 */

static void *__pyx_f_5scipy_6linalg_14_decomp_update_extract(PyArrayObject *__pyx_v_arr, int *__pyx_v_arrs) {
  void *__pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("extract", 0);

  /* "scipy/linalg/_decomp_update.pyx":1442
 * cdef void* extract(cnp.ndarray arr, int* arrs):
 *     with cython.cdivision(True):    # Assumes itemsize != 0.
 *         if arr.ndim == 2:             # <<<<<<<<<<<<<<
 *             arrs[0] = arr.strides[0] / cnp.PyArray_ITEMSIZE(arr)
 *             arrs[1] = arr.strides[1] / cnp.PyArray_ITEMSIZE(arr)
 */
  switch (__pyx_v_arr->nd) {
    case 2:

    /* "scipy/linalg/_decomp_update.pyx":1443
 *     with cython.cdivision(True):    # Assumes itemsize != 0.
 *         if arr.ndim == 2:
 *             arrs[0] = arr.strides[0] / cnp.PyArray_ITEMSIZE(arr)             # <<<<<<<<<<<<<<
 *             arrs[1] = arr.strides[1] / cnp.PyArray_ITEMSIZE(arr)
 *         elif arr.ndim == 1:
 */
    (__pyx_v_arrs[0]) = ((__pyx_v_arr->strides[0]) / PyArray_ITEMSIZE(__pyx_v_arr));

    /* "scipy/linalg/_decomp_update.pyx":1444
 *         if arr.ndim == 2:
 *             arrs[0] = arr.strides[0] / cnp.PyArray_ITEMSIZE(arr)
 *             arrs[1] = arr.strides[1] / cnp.PyArray_ITEMSIZE(arr)             # <<<<<<<<<<<<<<
 *         elif arr.ndim == 1:
 *             arrs[0] = arr.strides[0] / cnp.PyArray_ITEMSIZE(arr)
 */
    (__pyx_v_arrs[1]) = ((__pyx_v_arr->strides[1]) / PyArray_ITEMSIZE(__pyx_v_arr));

    /* "scipy/linalg/_decomp_update.pyx":1442
 * cdef void* extract(cnp.ndarray arr, int* arrs):
 *     with cython.cdivision(True):    # Assumes itemsize != 0.
 *         if arr.ndim == 2:             # <<<<<<<<<<<<<<
 *             arrs[0] = arr.strides[0] / cnp.PyArray_ITEMSIZE(arr)
 *             arrs[1] = arr.strides[1] / cnp.PyArray_ITEMSIZE(arr)
 */
    break;
    case 1:

    /* "scipy/linalg/_decomp_update.pyx":1446
 *             arrs[1] = arr.strides[1] / cnp.PyArray_ITEMSIZE(arr)
 *         elif arr.ndim == 1:
 *             arrs[0] = arr.strides[0] / cnp.PyArray_ITEMSIZE(arr)             # <<<<<<<<<<<<<<
 *             arrs[1] = 0
 *     return cnp.PyArray_DATA(arr)
 */
    (__pyx_v_arrs[0]) = ((__pyx_v_arr->strides[0]) / PyArray_ITEMSIZE(__pyx_v_arr));

    /* "scipy/linalg/_decomp_update.pyx":1447
 *         elif arr.ndim == 1:
 *             arrs[0] = arr.strides[0] / cnp.PyArray_ITEMSIZE(arr)
 *             arrs[1] = 0             # <<<<<<<<<<<<<<
 *     return cnp.PyArray_DATA(arr)
 * 
 */
    (__pyx_v_arrs[1]) = 0;

    /* "scipy/linalg/_decomp_update.pyx":1445
 *             arrs[0] = arr.strides[0] / cnp.PyArray_ITEMSIZE(arr)
 *             arrs[1] = arr.strides[1] / cnp.PyArray_ITEMSIZE(arr)
 *         elif arr.ndim == 1:             # <<<<<<<<<<<<<<
 *             arrs[0] = arr.strides[0] / cnp.PyArray_ITEMSIZE(arr)
 *             arrs[1] = 0
 */
    break;
    default: break;
  }

  /* "scipy/linalg/_decomp_update.pyx":1448
 *             arrs[0] = arr.strides[0] / cnp.PyArray_ITEMSIZE(arr)
 *             arrs[1] = 0
 *     return cnp.PyArray_DATA(arr)             # <<<<<<<<<<<<<<
 * 
 * @cython.embedsignature(True)
 */
  __pyx_r = PyArray_DATA(__pyx_v_arr);
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":1440
 *     return Q, R, typecode, Q.shape[0], R.shape[1], economic
 * 
 * cdef void* extract(cnp.ndarray arr, int* arrs):             # <<<<<<<<<<<<<<
 *     with cython.cdivision(True):    # Assumes itemsize != 0.
 *         if arr.ndim == 2:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":1451
 * 
 * @cython.embedsignature(True)
 * def qr_delete(Q, R, k, int p=1, which='row', overwrite_qr=False,             # <<<<<<<<<<<<<<
 *               check_finite=True):
 *     """
 */

/* Python wrapper */
static PyObject *__pyx_pw_5scipy_6linalg_14_decomp_update_3qr_delete(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5scipy_6linalg_14_decomp_update_2qr_delete[] = "qr_delete(Q, R, k, int p=1, which=u'row', overwrite_qr=False, check_finite=True)\n\n    QR downdate on row or column deletions\n\n    If ``A = Q R`` is the QR factorization of ``A``, return the QR\n    factorization of ``A`` where ``p`` rows or columns have been removed\n    starting at row or column ``k``.\n\n    Parameters\n    ----------\n    Q : (M, M) or (M, N) array_like\n        Unitary/orthogonal matrix from QR decomposition.\n    R : (M, N) or (N, N) array_like\n        Upper triangular matrix from QR decomposition.\n    k : int\n        Index of the first row or column to delete.\n    p : int, optional\n        Number of rows or columns to delete, defaults to 1.\n    which: {'row', 'col'}, optional\n        Determines if rows or columns will be deleted, defaults to 'row'\n    overwrite_qr : bool, optional\n        If True, consume Q and R, overwriting their contents with their\n        downdated versions, and returning approriately sized views.\n        Defaults to False.\n    check_finite : bool, optional\n        Whether to check that the input matrix contains only finite numbers.\n        Disabling may give a performance gain, but may result in problems\n        (crashes, non-termination) if the inputs do contain infinities or NaNs.\n        Default is True.\n\n    Returns\n    -------\n    Q1 : ndarray\n        Updated unitary/orthogonal factor\n    R1 : ndarray\n        Updated upper triangular factor\n\n    See Also\n    --------\n    qr, qr_multiply, qr_insert, qr_update\n\n    Notes\n    -----\n    This routine does not guarantee that the diagonal entries of ``R1`` are\n    positive.\n\n    .. versionadded:: 0.16.0\n\n    References\n    ----------\n    .. [1] Golub, G. H. & Van Loan, C. F. Matrix Computations, 3rd Ed.\n           (Johns Hopkins University Press, 1996).\n\n    .. [2] Daniel, J. W., Gragg, W. B., Kaufman, L. & Stewart, G. W.\n           Reorthogonalization and stable algorithms for updating the\n           Gram-Schmidt QR factorizat""ion. Math. Comput. 30, 772-795 (1976).\n\n    .. [3] Reichel, L. & Gragg, W. B. Algorithm 686: FORTRAN Subroutines for\n           Updating the QR Decomposition. ACM Trans. Math. Softw. 16, 369-377\n           (1990).\n\n    Examples\n    --------\n    >>> from scipy import linalg\n    >>> a = np.array([[  3.,  -2.,  -2.],\n    ...               [  6.,  -9.,  -3.],\n    ...               [ -3.,  10.,   1.],\n    ...               [  6.,  -7.,   4.],\n    ...               [  7.,   8.,  -6.]])\n    >>> q, r = linalg.qr(a)\n\n    Given this QR decomposition, update q and r when 2 rows are removed.\n\n    >>> q1, r1 = linalg.qr_delete(q, r, 2, 2, 'row', False)\n    >>> q1\n    array([[ 0.30942637,  0.15347579,  0.93845645],  # may vary (signs)\n           [ 0.61885275,  0.71680171, -0.32127338],\n           [ 0.72199487, -0.68017681, -0.12681844]])\n    >>> r1\n    array([[  9.69535971,  -0.4125685 ,  -6.80738023],  # may vary (signs)\n           [  0.        , -12.19958144,   1.62370412],\n           [  0.        ,   0.        ,  -0.15218213]])\n\n    The update is equivalent, but faster than the following.\n\n    >>> a1 = np.delete(a, slice(2,4), 0)\n    >>> a1\n    array([[ 3., -2., -2.],\n           [ 6., -9., -3.],\n           [ 7.,  8., -6.]])\n    >>> q_direct, r_direct = linalg.qr(a1)\n\n    Check that we have equivalent results:\n\n    >>> np.dot(q1, r1)\n    array([[ 3., -2., -2.],\n           [ 6., -9., -3.],\n           [ 7.,  8., -6.]])\n    >>> np.allclose(np.dot(q1, r1), a1)\n    True\n\n    And the updated Q is still unitary:\n\n    >>> np.allclose(np.dot(q1.T, q1), np.eye(3))\n    True\n\n    ";
static PyMethodDef __pyx_mdef_5scipy_6linalg_14_decomp_update_3qr_delete = {"qr_delete", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5scipy_6linalg_14_decomp_update_3qr_delete, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5scipy_6linalg_14_decomp_update_2qr_delete};
static PyObject *__pyx_pw_5scipy_6linalg_14_decomp_update_3qr_delete(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_Q = 0;
  PyObject *__pyx_v_R = 0;
  PyObject *__pyx_v_k = 0;
  int __pyx_v_p;
  PyObject *__pyx_v_which = 0;
  PyObject *__pyx_v_overwrite_qr = 0;
  PyObject *__pyx_v_check_finite = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("qr_delete (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_Q,&__pyx_n_s_R,&__pyx_n_s_k,&__pyx_n_s_p,&__pyx_n_s_which,&__pyx_n_s_overwrite_qr,&__pyx_n_s_check_finite,0};
    PyObject* values[7] = {0,0,0,0,0,0,0};
    values[4] = ((PyObject *)__pyx_n_u_row);
    values[5] = ((PyObject *)Py_False);

    /* "scipy/linalg/_decomp_update.pyx":1452
 * @cython.embedsignature(True)
 * def qr_delete(Q, R, k, int p=1, which='row', overwrite_qr=False,
 *               check_finite=True):             # <<<<<<<<<<<<<<
 *     """
 *     QR downdate on row or column deletions
 */
    values[6] = ((PyObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_Q)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_R)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("qr_delete", 0, 3, 7, 1); __PYX_ERR(0, 1451, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_k)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("qr_delete", 0, 3, 7, 2); __PYX_ERR(0, 1451, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_p);
          if (value) { values[3] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_which);
          if (value) { values[4] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_overwrite_qr);
          if (value) { values[5] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_check_finite);
          if (value) { values[6] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "qr_delete") < 0)) __PYX_ERR(0, 1451, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_Q = values[0];
    __pyx_v_R = values[1];
    __pyx_v_k = values[2];
    if (values[3]) {
      __pyx_v_p = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_p == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1451, __pyx_L3_error)
    } else {
      __pyx_v_p = ((int)1);
    }
    __pyx_v_which = values[4];
    __pyx_v_overwrite_qr = values[5];
    __pyx_v_check_finite = values[6];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("qr_delete", 0, 3, 7, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1451, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("scipy.linalg._decomp_update.qr_delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5scipy_6linalg_14_decomp_update_2qr_delete(__pyx_self, __pyx_v_Q, __pyx_v_R, __pyx_v_k, __pyx_v_p, __pyx_v_which, __pyx_v_overwrite_qr, __pyx_v_check_finite);

  /* "scipy/linalg/_decomp_update.pyx":1451
 * 
 * @cython.embedsignature(True)
 * def qr_delete(Q, R, k, int p=1, which='row', overwrite_qr=False,             # <<<<<<<<<<<<<<
 *               check_finite=True):
 *     """
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5scipy_6linalg_14_decomp_update_2qr_delete(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_Q, PyObject *__pyx_v_R, PyObject *__pyx_v_k, int __pyx_v_p, PyObject *__pyx_v_which, PyObject *__pyx_v_overwrite_qr, PyObject *__pyx_v_check_finite) {
  PyArrayObject *__pyx_v_q1 = 0;
  PyArrayObject *__pyx_v_r1 = 0;
  int __pyx_v_k1;
  int __pyx_v_p1;
  int __pyx_v_p_eco;
  int __pyx_v_p_full;
  int __pyx_v_typecode;
  int __pyx_v_m;
  int __pyx_v_n;
  int __pyx_v_info;
  void *__pyx_v_qptr;
  void *__pyx_v_rptr;
  int __pyx_v_qs[2];
  int __pyx_v_rs[2];
  int __pyx_v_economic;
  int __pyx_v_qisF;
  int __pyx_v_chkfinite;
  int __pyx_v_overwrite;
  PyObject *__pyx_v_qnew = NULL;
  PyObject *__pyx_v_norm = NULL;
  int __pyx_v_o;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  __Pyx_RefNannySetupContext("qr_delete", 0);

  /* "scipy/linalg/_decomp_update.pyx":1560
 *     """
 *     cdef cnp.ndarray q1, r1
 *     cdef int k1 = k             # <<<<<<<<<<<<<<
 *     cdef int p1 = p
 *     cdef int p_eco, p_full
 */
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_k); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1560, __pyx_L1_error)
  __pyx_v_k1 = __pyx_t_1;

  /* "scipy/linalg/_decomp_update.pyx":1561
 *     cdef cnp.ndarray q1, r1
 *     cdef int k1 = k
 *     cdef int p1 = p             # <<<<<<<<<<<<<<
 *     cdef int p_eco, p_full
 *     cdef int typecode, m, n, info
 */
  __pyx_v_p1 = __pyx_v_p;

  /* "scipy/linalg/_decomp_update.pyx":1568
 *     cdef int qs[2]
 *     cdef int rs[2]
 *     cdef bint economic, qisF = False             # <<<<<<<<<<<<<<
 *     cdef bint chkfinite = check_finite, overwrite = overwrite_qr
 * 
 */
  __pyx_v_qisF = 0;

  /* "scipy/linalg/_decomp_update.pyx":1569
 *     cdef int rs[2]
 *     cdef bint economic, qisF = False
 *     cdef bint chkfinite = check_finite, overwrite = overwrite_qr             # <<<<<<<<<<<<<<
 * 
 *     if which == 'row':
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_check_finite); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1569, __pyx_L1_error)
  __pyx_v_chkfinite = __pyx_t_2;
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_overwrite_qr); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1569, __pyx_L1_error)
  __pyx_v_overwrite = __pyx_t_2;

  /* "scipy/linalg/_decomp_update.pyx":1571
 *     cdef bint chkfinite = check_finite, overwrite = overwrite_qr
 * 
 *     if which == 'row':             # <<<<<<<<<<<<<<
 *         q1, r1, typecode, m, n, economic = validate_qr(Q, R, overwrite,
 *                 ARRAY_ANYORDER, overwrite, ARRAY_ANYORDER, chkfinite)
 */
  __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_v_which, __pyx_n_u_row, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 1571, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "scipy/linalg/_decomp_update.pyx":1572
 * 
 *     if which == 'row':
 *         q1, r1, typecode, m, n, economic = validate_qr(Q, R, overwrite,             # <<<<<<<<<<<<<<
 *                 ARRAY_ANYORDER, overwrite, ARRAY_ANYORDER, chkfinite)
 *         if not (-m <= k1 < m):
 */
    __pyx_t_3 = __pyx_f_5scipy_6linalg_14_decomp_update_validate_qr(__pyx_v_Q, __pyx_v_R, __pyx_v_overwrite, __pyx_v_5scipy_6linalg_14_decomp_update_ARRAY_ANYORDER, __pyx_v_overwrite, __pyx_v_5scipy_6linalg_14_decomp_update_ARRAY_ANYORDER, __pyx_v_chkfinite); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1572, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (likely(__pyx_t_3 != Py_None)) {
      PyObject* sequence = __pyx_t_3;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 6)) {
        if (size > 6) __Pyx_RaiseTooManyValuesError(6);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 1572, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_5 = PyTuple_GET_ITEM(sequence, 1); 
      __pyx_t_6 = PyTuple_GET_ITEM(sequence, 2); 
      __pyx_t_7 = PyTuple_GET_ITEM(sequence, 3); 
      __pyx_t_8 = PyTuple_GET_ITEM(sequence, 4); 
      __pyx_t_9 = PyTuple_GET_ITEM(sequence, 5); 
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_9);
      #else
      {
        Py_ssize_t i;
        PyObject** temps[6] = {&__pyx_t_4,&__pyx_t_5,&__pyx_t_6,&__pyx_t_7,&__pyx_t_8,&__pyx_t_9};
        for (i=0; i < 6; i++) {
          PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 1572, __pyx_L1_error)
          __Pyx_GOTREF(item);
          *(temps[i]) = item;
        }
      }
      #endif
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else {
      __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(0, 1572, __pyx_L1_error)
    }
    if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1572, __pyx_L1_error)
    if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1572, __pyx_L1_error)
    __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_t_6); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1572, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_10 = __Pyx_PyInt_As_int(__pyx_t_7); if (unlikely((__pyx_t_10 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1572, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_11 = __Pyx_PyInt_As_int(__pyx_t_8); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1572, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1572, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_v_q1 = ((PyArrayObject *)__pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_v_r1 = ((PyArrayObject *)__pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_v_typecode = __pyx_t_1;
    __pyx_v_m = __pyx_t_10;
    __pyx_v_n = __pyx_t_11;
    __pyx_v_economic = __pyx_t_2;

    /* "scipy/linalg/_decomp_update.pyx":1574
 *         q1, r1, typecode, m, n, economic = validate_qr(Q, R, overwrite,
 *                 ARRAY_ANYORDER, overwrite, ARRAY_ANYORDER, chkfinite)
 *         if not (-m <= k1 < m):             # <<<<<<<<<<<<<<
 *             raise ValueError("'k' is out of bounds")
 *         if k1 < 0:
 */
    __pyx_t_2 = ((-__pyx_v_m) <= __pyx_v_k1);
    if (__pyx_t_2) {
      __pyx_t_2 = (__pyx_v_k1 < __pyx_v_m);
    }
    __pyx_t_12 = ((!(__pyx_t_2 != 0)) != 0);
    if (unlikely(__pyx_t_12)) {

      /* "scipy/linalg/_decomp_update.pyx":1575
 *                 ARRAY_ANYORDER, overwrite, ARRAY_ANYORDER, chkfinite)
 *         if not (-m <= k1 < m):
 *             raise ValueError("'k' is out of bounds")             # <<<<<<<<<<<<<<
 *         if k1 < 0:
 *             k1 += m
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1575, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 1575, __pyx_L1_error)

      /* "scipy/linalg/_decomp_update.pyx":1574
 *         q1, r1, typecode, m, n, economic = validate_qr(Q, R, overwrite,
 *                 ARRAY_ANYORDER, overwrite, ARRAY_ANYORDER, chkfinite)
 *         if not (-m <= k1 < m):             # <<<<<<<<<<<<<<
 *             raise ValueError("'k' is out of bounds")
 *         if k1 < 0:
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1576
 *         if not (-m <= k1 < m):
 *             raise ValueError("'k' is out of bounds")
 *         if k1 < 0:             # <<<<<<<<<<<<<<
 *             k1 += m
 *         if k1 + p1 > m or p1 <= 0:
 */
    __pyx_t_12 = ((__pyx_v_k1 < 0) != 0);
    if (__pyx_t_12) {

      /* "scipy/linalg/_decomp_update.pyx":1577
 *             raise ValueError("'k' is out of bounds")
 *         if k1 < 0:
 *             k1 += m             # <<<<<<<<<<<<<<
 *         if k1 + p1 > m or p1 <= 0:
 *             raise ValueError("'p' is out of range")
 */
      __pyx_v_k1 = (__pyx_v_k1 + __pyx_v_m);

      /* "scipy/linalg/_decomp_update.pyx":1576
 *         if not (-m <= k1 < m):
 *             raise ValueError("'k' is out of bounds")
 *         if k1 < 0:             # <<<<<<<<<<<<<<
 *             k1 += m
 *         if k1 + p1 > m or p1 <= 0:
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1578
 *         if k1 < 0:
 *             k1 += m
 *         if k1 + p1 > m or p1 <= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("'p' is out of range")
 *         if economic:
 */
    __pyx_t_2 = (((__pyx_v_k1 + __pyx_v_p1) > __pyx_v_m) != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_12 = __pyx_t_2;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_p1 <= 0) != 0);
    __pyx_t_12 = __pyx_t_2;
    __pyx_L7_bool_binop_done:;
    if (unlikely(__pyx_t_12)) {

      /* "scipy/linalg/_decomp_update.pyx":1579
 *             k1 += m
 *         if k1 + p1 > m or p1 <= 0:
 *             raise ValueError("'p' is out of range")             # <<<<<<<<<<<<<<
 *         if economic:
 *             # handle the special case of (M,1), (1,1)
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1579, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 1579, __pyx_L1_error)

      /* "scipy/linalg/_decomp_update.pyx":1578
 *         if k1 < 0:
 *             k1 += m
 *         if k1 + p1 > m or p1 <= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("'p' is out of range")
 *         if economic:
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1580
 *         if k1 + p1 > m or p1 <= 0:
 *             raise ValueError("'p' is out of range")
 *         if economic:             # <<<<<<<<<<<<<<
 *             # handle the special case of (M,1), (1,1)
 *             if n == 1:
 */
    __pyx_t_12 = (__pyx_v_economic != 0);
    if (__pyx_t_12) {

      /* "scipy/linalg/_decomp_update.pyx":1582
 *         if economic:
 *             # handle the special case of (M,1), (1,1)
 *             if n == 1:             # <<<<<<<<<<<<<<
 *                 qnew = np.delete(q1, slice(k1, k1 + p1), 0)
 *                 norm = np.linalg.norm(qnew)
 */
      __pyx_t_12 = ((__pyx_v_n == 1) != 0);
      if (__pyx_t_12) {

        /* "scipy/linalg/_decomp_update.pyx":1583
 *             # handle the special case of (M,1), (1,1)
 *             if n == 1:
 *                 qnew = np.delete(q1, slice(k1, k1 + p1), 0)             # <<<<<<<<<<<<<<
 *                 norm = np.linalg.norm(qnew)
 *                 return qnew / norm, r1 * norm
 */
        __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_np); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1583, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_delete); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1583, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __pyx_t_9 = __Pyx_PyInt_From_int(__pyx_v_k1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1583, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_7 = __Pyx_PyInt_From_int((__pyx_v_k1 + __pyx_v_p1)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1583, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_6 = PySlice_New(__pyx_t_9, __pyx_t_7, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1583, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        __pyx_t_11 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_8))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_8);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_8, function);
            __pyx_t_11 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_8)) {
          PyObject *__pyx_temp[4] = {__pyx_t_7, ((PyObject *)__pyx_v_q1), __pyx_t_6, __pyx_int_0};
          __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_11, 3+__pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1583, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_8)) {
          PyObject *__pyx_temp[4] = {__pyx_t_7, ((PyObject *)__pyx_v_q1), __pyx_t_6, __pyx_int_0};
          __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_11, 3+__pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1583, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        {
          __pyx_t_9 = PyTuple_New(3+__pyx_t_11); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1583, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          if (__pyx_t_7) {
            __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
          }
          __Pyx_INCREF(((PyObject *)__pyx_v_q1));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_q1));
          PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_11, ((PyObject *)__pyx_v_q1));
          __Pyx_GIVEREF(__pyx_t_6);
          PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_11, __pyx_t_6);
          __Pyx_INCREF(__pyx_int_0);
          __Pyx_GIVEREF(__pyx_int_0);
          PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_11, __pyx_int_0);
          __pyx_t_6 = 0;
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_9, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1583, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_qnew = __pyx_t_3;
        __pyx_t_3 = 0;

        /* "scipy/linalg/_decomp_update.pyx":1584
 *             if n == 1:
 *                 qnew = np.delete(q1, slice(k1, k1 + p1), 0)
 *                 norm = np.linalg.norm(qnew)             # <<<<<<<<<<<<<<
 *                 return qnew / norm, r1 * norm
 *             if not cnp.PyArray_ISONESEGMENT(q1):
 */
        __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_np); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1584, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_linalg); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1584, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_norm); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1584, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __pyx_t_9 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
          __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
          if (likely(__pyx_t_9)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
            __Pyx_INCREF(__pyx_t_9);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_8, function);
          }
        }
        __pyx_t_3 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_v_qnew) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_v_qnew);
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1584, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_norm = __pyx_t_3;
        __pyx_t_3 = 0;

        /* "scipy/linalg/_decomp_update.pyx":1585
 *                 qnew = np.delete(q1, slice(k1, k1 + p1), 0)
 *                 norm = np.linalg.norm(qnew)
 *                 return qnew / norm, r1 * norm             # <<<<<<<<<<<<<<
 *             if not cnp.PyArray_ISONESEGMENT(q1):
 *                 q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 */
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = __Pyx_PyNumber_Divide(__pyx_v_qnew, __pyx_v_norm); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1585, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_8 = PyNumber_Multiply(((PyObject *)__pyx_v_r1), __pyx_v_norm); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1585, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1585, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_8);
        PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_8);
        __pyx_t_3 = 0;
        __pyx_t_8 = 0;
        __pyx_r = __pyx_t_9;
        __pyx_t_9 = 0;
        goto __pyx_L0;

        /* "scipy/linalg/_decomp_update.pyx":1582
 *         if economic:
 *             # handle the special case of (M,1), (1,1)
 *             if n == 1:             # <<<<<<<<<<<<<<
 *                 qnew = np.delete(q1, slice(k1, k1 + p1), 0)
 *                 norm = np.linalg.norm(qnew)
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":1586
 *                 norm = np.linalg.norm(qnew)
 *                 return qnew / norm, r1 * norm
 *             if not cnp.PyArray_ISONESEGMENT(q1):             # <<<<<<<<<<<<<<
 *                 q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *                 qisF = True
 */
      __pyx_t_12 = ((!(PyArray_ISONESEGMENT(__pyx_v_q1) != 0)) != 0);
      if (__pyx_t_12) {

        /* "scipy/linalg/_decomp_update.pyx":1587
 *                 return qnew / norm, r1 * norm
 *             if not cnp.PyArray_ISONESEGMENT(q1):
 *                 q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 qisF = True
 *             elif cnp.PyArray_CHKFLAGS(q1, cnp.NPY_F_CONTIGUOUS):
 */
        __pyx_t_9 = ((PyObject *)PyArray_FromArray(__pyx_v_q1, NULL, NPY_F_CONTIGUOUS)); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1587, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF_SET(__pyx_v_q1, ((PyArrayObject *)__pyx_t_9));
        __pyx_t_9 = 0;

        /* "scipy/linalg/_decomp_update.pyx":1588
 *             if not cnp.PyArray_ISONESEGMENT(q1):
 *                 q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *                 qisF = True             # <<<<<<<<<<<<<<
 *             elif cnp.PyArray_CHKFLAGS(q1, cnp.NPY_F_CONTIGUOUS):
 *                 qisF = True
 */
        __pyx_v_qisF = 1;

        /* "scipy/linalg/_decomp_update.pyx":1586
 *                 norm = np.linalg.norm(qnew)
 *                 return qnew / norm, r1 * norm
 *             if not cnp.PyArray_ISONESEGMENT(q1):             # <<<<<<<<<<<<<<
 *                 q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *                 qisF = True
 */
        goto __pyx_L11;
      }

      /* "scipy/linalg/_decomp_update.pyx":1589
 *                 q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *                 qisF = True
 *             elif cnp.PyArray_CHKFLAGS(q1, cnp.NPY_F_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *                 qisF = True
 *             else:
 */
      __pyx_t_12 = (PyArray_CHKFLAGS(__pyx_v_q1, NPY_F_CONTIGUOUS) != 0);
      if (__pyx_t_12) {

        /* "scipy/linalg/_decomp_update.pyx":1590
 *                 qisF = True
 *             elif cnp.PyArray_CHKFLAGS(q1, cnp.NPY_F_CONTIGUOUS):
 *                 qisF = True             # <<<<<<<<<<<<<<
 *             else:
 *                 qisF = False
 */
        __pyx_v_qisF = 1;

        /* "scipy/linalg/_decomp_update.pyx":1589
 *                 q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *                 qisF = True
 *             elif cnp.PyArray_CHKFLAGS(q1, cnp.NPY_F_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *                 qisF = True
 *             else:
 */
        goto __pyx_L11;
      }

      /* "scipy/linalg/_decomp_update.pyx":1592
 *                 qisF = True
 *             else:
 *                 qisF = False             # <<<<<<<<<<<<<<
 *             if m-p >= n:
 *                 p_eco = p1
 */
      /*else*/ {
        __pyx_v_qisF = 0;
      }
      __pyx_L11:;

      /* "scipy/linalg/_decomp_update.pyx":1593
 *             else:
 *                 qisF = False
 *             if m-p >= n:             # <<<<<<<<<<<<<<
 *                 p_eco = p1
 *                 p_full = 0
 */
      __pyx_t_12 = (((__pyx_v_m - __pyx_v_p) >= __pyx_v_n) != 0);
      if (__pyx_t_12) {

        /* "scipy/linalg/_decomp_update.pyx":1594
 *                 qisF = False
 *             if m-p >= n:
 *                 p_eco = p1             # <<<<<<<<<<<<<<
 *                 p_full = 0
 *             else:
 */
        __pyx_v_p_eco = __pyx_v_p1;

        /* "scipy/linalg/_decomp_update.pyx":1595
 *             if m-p >= n:
 *                 p_eco = p1
 *                 p_full = 0             # <<<<<<<<<<<<<<
 *             else:
 *                 p_eco = m-n
 */
        __pyx_v_p_full = 0;

        /* "scipy/linalg/_decomp_update.pyx":1593
 *             else:
 *                 qisF = False
 *             if m-p >= n:             # <<<<<<<<<<<<<<
 *                 p_eco = p1
 *                 p_full = 0
 */
        goto __pyx_L12;
      }

      /* "scipy/linalg/_decomp_update.pyx":1597
 *                 p_full = 0
 *             else:
 *                 p_eco = m-n             # <<<<<<<<<<<<<<
 *                 p_full = p1 - p_eco
 *             qptr = extract(q1, qs)
 */
      /*else*/ {
        __pyx_v_p_eco = (__pyx_v_m - __pyx_v_n);

        /* "scipy/linalg/_decomp_update.pyx":1598
 *             else:
 *                 p_eco = m-n
 *                 p_full = p1 - p_eco             # <<<<<<<<<<<<<<
 *             qptr = extract(q1, qs)
 *             rptr = extract(r1, rs)
 */
        __pyx_v_p_full = (__pyx_v_p1 - __pyx_v_p_eco);
      }
      __pyx_L12:;

      /* "scipy/linalg/_decomp_update.pyx":1599
 *                 p_eco = m-n
 *                 p_full = p1 - p_eco
 *             qptr = extract(q1, qs)             # <<<<<<<<<<<<<<
 *             rptr = extract(r1, rs)
 *             with nogil:
 */
      __pyx_v_qptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_q1, __pyx_v_qs);

      /* "scipy/linalg/_decomp_update.pyx":1600
 *                 p_full = p1 - p_eco
 *             qptr = extract(q1, qs)
 *             rptr = extract(r1, rs)             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 */
      __pyx_v_rptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_r1, __pyx_v_rs);

      /* "scipy/linalg/_decomp_update.pyx":1601
 *             qptr = extract(q1, qs)
 *             rptr = extract(r1, rs)
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     info = thin_qr_row_delete(m, n, <float*>qptr, qs, qisF,
 */
      {
          #ifdef WITH_THREAD
          PyThreadState *_save;
          Py_UNBLOCK_THREADS
          __Pyx_FastGIL_Remember();
          #endif
          /*try:*/ {

            /* "scipy/linalg/_decomp_update.pyx":1602
 *             rptr = extract(r1, rs)
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     info = thin_qr_row_delete(m, n, <float*>qptr, qs, qisF,
 *                         <float*>rptr, rs, k1, p_eco, p_full)
 */
            switch (__pyx_v_typecode) {
              case NPY_FLOAT:

              /* "scipy/linalg/_decomp_update.pyx":1603
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 *                     info = thin_qr_row_delete(m, n, <float*>qptr, qs, qisF,             # <<<<<<<<<<<<<<
 *                         <float*>rptr, rs, k1, p_eco, p_full)
 *                 elif typecode == cnp.NPY_DOUBLE:
 */
              __pyx_v_info = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_delete(__pyx_v_m, __pyx_v_n, ((float *)__pyx_v_qptr), __pyx_v_qs, __pyx_v_qisF, ((float *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k1, __pyx_v_p_eco, __pyx_v_p_full);

              /* "scipy/linalg/_decomp_update.pyx":1602
 *             rptr = extract(r1, rs)
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     info = thin_qr_row_delete(m, n, <float*>qptr, qs, qisF,
 *                         <float*>rptr, rs, k1, p_eco, p_full)
 */
              break;
              case NPY_DOUBLE:

              /* "scipy/linalg/_decomp_update.pyx":1606
 *                         <float*>rptr, rs, k1, p_eco, p_full)
 *                 elif typecode == cnp.NPY_DOUBLE:
 *                     info = thin_qr_row_delete(m, n, <double*>qptr, qs, qisF,             # <<<<<<<<<<<<<<
 *                         <double*>rptr, rs, k1, p_eco, p_full)
 *                 elif typecode == cnp.NPY_CFLOAT:
 */
              __pyx_v_info = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_delete(__pyx_v_m, __pyx_v_n, ((double *)__pyx_v_qptr), __pyx_v_qs, __pyx_v_qisF, ((double *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k1, __pyx_v_p_eco, __pyx_v_p_full);

              /* "scipy/linalg/_decomp_update.pyx":1605
 *                     info = thin_qr_row_delete(m, n, <float*>qptr, qs, qisF,
 *                         <float*>rptr, rs, k1, p_eco, p_full)
 *                 elif typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                     info = thin_qr_row_delete(m, n, <double*>qptr, qs, qisF,
 *                         <double*>rptr, rs, k1, p_eco, p_full)
 */
              break;
              case NPY_CFLOAT:

              /* "scipy/linalg/_decomp_update.pyx":1609
 *                         <double*>rptr, rs, k1, p_eco, p_full)
 *                 elif typecode == cnp.NPY_CFLOAT:
 *                     info = thin_qr_row_delete(m, n, <float_complex*>qptr, qs, qisF,             # <<<<<<<<<<<<<<
 *                         <float_complex*>rptr, rs, k1, p_eco, p_full)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 */
              __pyx_v_info = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_delete(__pyx_v_m, __pyx_v_n, ((__pyx_t_float_complex *)__pyx_v_qptr), __pyx_v_qs, __pyx_v_qisF, ((__pyx_t_float_complex *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k1, __pyx_v_p_eco, __pyx_v_p_full);

              /* "scipy/linalg/_decomp_update.pyx":1608
 *                     info = thin_qr_row_delete(m, n, <double*>qptr, qs, qisF,
 *                         <double*>rptr, rs, k1, p_eco, p_full)
 *                 elif typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                     info = thin_qr_row_delete(m, n, <float_complex*>qptr, qs, qisF,
 *                         <float_complex*>rptr, rs, k1, p_eco, p_full)
 */
              break;
              default:

              /* "scipy/linalg/_decomp_update.pyx":1612
 *                         <float_complex*>rptr, rs, k1, p_eco, p_full)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 *                     info = thin_qr_row_delete(m, n, <double_complex*>qptr, qs, qisF,             # <<<<<<<<<<<<<<
 *                         <double_complex*>rptr, rs, k1, p_eco, p_full)
 *             if info == 1:
 */
              __pyx_v_info = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_delete(__pyx_v_m, __pyx_v_n, ((__pyx_t_double_complex *)__pyx_v_qptr), __pyx_v_qs, __pyx_v_qisF, ((__pyx_t_double_complex *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k1, __pyx_v_p_eco, __pyx_v_p_full);
              break;
            }
          }

          /* "scipy/linalg/_decomp_update.pyx":1601
 *             qptr = extract(q1, qs)
 *             rptr = extract(r1, rs)
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     info = thin_qr_row_delete(m, n, <float*>qptr, qs, qisF,
 */
          /*finally:*/ {
            /*normal exit:*/{
              #ifdef WITH_THREAD
              __Pyx_FastGIL_Forget();
              Py_BLOCK_THREADS
              #endif
              goto __pyx_L15;
            }
            __pyx_L15:;
          }
      }

      /* "scipy/linalg/_decomp_update.pyx":1614
 *                     info = thin_qr_row_delete(m, n, <double_complex*>qptr, qs, qisF,
 *                         <double_complex*>rptr, rs, k1, p_eco, p_full)
 *             if info == 1:             # <<<<<<<<<<<<<<
 *                 return q1[p_full:-p_eco, p_full:], r1[p_full:,:]
 *             elif info == MEMORY_ERROR:
 */
      __pyx_t_12 = ((__pyx_v_info == 1) != 0);
      if (__pyx_t_12) {

        /* "scipy/linalg/_decomp_update.pyx":1615
 *                         <double_complex*>rptr, rs, k1, p_eco, p_full)
 *             if info == 1:
 *                 return q1[p_full:-p_eco, p_full:], r1[p_full:,:]             # <<<<<<<<<<<<<<
 *             elif info == MEMORY_ERROR:
 *                 raise MemoryError('Unable to allocate memory for array.')
 */
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_9 = __Pyx_PyInt_From_int(__pyx_v_p_full); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1615, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_8 = __Pyx_PyInt_From_int((-__pyx_v_p_eco)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1615, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_3 = PySlice_New(__pyx_t_9, __pyx_t_8, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1615, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_v_p_full); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1615, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_9 = PySlice_New(__pyx_t_8, Py_None, Py_None); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1615, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1615, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_9);
        PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_9);
        __pyx_t_3 = 0;
        __pyx_t_9 = 0;
        __pyx_t_9 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_q1), __pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1615, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_v_p_full); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1615, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_3 = PySlice_New(__pyx_t_8, Py_None, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1615, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1615, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_3);
        __Pyx_INCREF(__pyx_slice__14);
        __Pyx_GIVEREF(__pyx_slice__14);
        PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_slice__14);
        __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_r1), __pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1615, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1615, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_GIVEREF(__pyx_t_9);
        PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_9);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_3);
        __pyx_t_9 = 0;
        __pyx_t_3 = 0;
        __pyx_r = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L0;

        /* "scipy/linalg/_decomp_update.pyx":1614
 *                     info = thin_qr_row_delete(m, n, <double_complex*>qptr, qs, qisF,
 *                         <double_complex*>rptr, rs, k1, p_eco, p_full)
 *             if info == 1:             # <<<<<<<<<<<<<<
 *                 return q1[p_full:-p_eco, p_full:], r1[p_full:,:]
 *             elif info == MEMORY_ERROR:
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":1616
 *             if info == 1:
 *                 return q1[p_full:-p_eco, p_full:], r1[p_full:,:]
 *             elif info == MEMORY_ERROR:             # <<<<<<<<<<<<<<
 *                 raise MemoryError('Unable to allocate memory for array.')
 *             else:
 */
      __pyx_t_12 = ((__pyx_v_info == __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR) != 0);
      if (unlikely(__pyx_t_12)) {

        /* "scipy/linalg/_decomp_update.pyx":1617
 *                 return q1[p_full:-p_eco, p_full:], r1[p_full:,:]
 *             elif info == MEMORY_ERROR:
 *                 raise MemoryError('Unable to allocate memory for array.')             # <<<<<<<<<<<<<<
 *             else:
 *                 raise ValueError('Reorthogonalization Failed, unable to perform row deletion.')
 */
        __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1617, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_Raise(__pyx_t_8, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __PYX_ERR(0, 1617, __pyx_L1_error)

        /* "scipy/linalg/_decomp_update.pyx":1616
 *             if info == 1:
 *                 return q1[p_full:-p_eco, p_full:], r1[p_full:,:]
 *             elif info == MEMORY_ERROR:             # <<<<<<<<<<<<<<
 *                 raise MemoryError('Unable to allocate memory for array.')
 *             else:
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":1619
 *                 raise MemoryError('Unable to allocate memory for array.')
 *             else:
 *                 raise ValueError('Reorthogonalization Failed, unable to perform row deletion.')             # <<<<<<<<<<<<<<
 *         else:
 *             qptr = extract(q1, qs)
 */
      /*else*/ {
        __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1619, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_Raise(__pyx_t_8, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __PYX_ERR(0, 1619, __pyx_L1_error)
      }

      /* "scipy/linalg/_decomp_update.pyx":1580
 *         if k1 + p1 > m or p1 <= 0:
 *             raise ValueError("'p' is out of range")
 *         if economic:             # <<<<<<<<<<<<<<
 *             # handle the special case of (M,1), (1,1)
 *             if n == 1:
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1621
 *                 raise ValueError('Reorthogonalization Failed, unable to perform row deletion.')
 *         else:
 *             qptr = extract(q1, qs)             # <<<<<<<<<<<<<<
 *             rptr = extract(r1, rs)
 *             with nogil:
 */
    /*else*/ {
      __pyx_v_qptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_q1, __pyx_v_qs);

      /* "scipy/linalg/_decomp_update.pyx":1622
 *         else:
 *             qptr = extract(q1, qs)
 *             rptr = extract(r1, rs)             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 */
      __pyx_v_rptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_r1, __pyx_v_rs);

      /* "scipy/linalg/_decomp_update.pyx":1623
 *             qptr = extract(q1, qs)
 *             rptr = extract(r1, rs)
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     qr_block_row_delete(m, n, <float*>qptr, qs,
 */
      {
          #ifdef WITH_THREAD
          PyThreadState *_save;
          Py_UNBLOCK_THREADS
          __Pyx_FastGIL_Remember();
          #endif
          /*try:*/ {

            /* "scipy/linalg/_decomp_update.pyx":1624
 *             rptr = extract(r1, rs)
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     qr_block_row_delete(m, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, k1, p1)
 */
            switch (__pyx_v_typecode) {
              case NPY_FLOAT:

              /* "scipy/linalg/_decomp_update.pyx":1625
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 *                     qr_block_row_delete(m, n, <float*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float*>rptr, rs, k1, p1)
 *                 elif typecode == cnp.NPY_DOUBLE:
 */
              __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_delete(__pyx_v_m, __pyx_v_n, ((float *)__pyx_v_qptr), __pyx_v_qs, ((float *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k1, __pyx_v_p1);

              /* "scipy/linalg/_decomp_update.pyx":1624
 *             rptr = extract(r1, rs)
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     qr_block_row_delete(m, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, k1, p1)
 */
              break;
              case NPY_DOUBLE:

              /* "scipy/linalg/_decomp_update.pyx":1628
 *                         <float*>rptr, rs, k1, p1)
 *                 elif typecode == cnp.NPY_DOUBLE:
 *                     qr_block_row_delete(m, n, <double*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double*>rptr, rs, k1, p1)
 *                 elif typecode == cnp.NPY_CFLOAT:
 */
              __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_delete(__pyx_v_m, __pyx_v_n, ((double *)__pyx_v_qptr), __pyx_v_qs, ((double *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k1, __pyx_v_p1);

              /* "scipy/linalg/_decomp_update.pyx":1627
 *                     qr_block_row_delete(m, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, k1, p1)
 *                 elif typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                     qr_block_row_delete(m, n, <double*>qptr, qs,
 *                         <double*>rptr, rs, k1, p1)
 */
              break;
              case NPY_CFLOAT:

              /* "scipy/linalg/_decomp_update.pyx":1631
 *                         <double*>rptr, rs, k1, p1)
 *                 elif typecode == cnp.NPY_CFLOAT:
 *                     qr_block_row_delete(m, n, <float_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float_complex*>rptr, rs, k1, p1)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 */
              __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_delete(__pyx_v_m, __pyx_v_n, ((__pyx_t_float_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_float_complex *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k1, __pyx_v_p1);

              /* "scipy/linalg/_decomp_update.pyx":1630
 *                     qr_block_row_delete(m, n, <double*>qptr, qs,
 *                         <double*>rptr, rs, k1, p1)
 *                 elif typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                     qr_block_row_delete(m, n, <float_complex*>qptr, qs,
 *                         <float_complex*>rptr, rs, k1, p1)
 */
              break;
              default:

              /* "scipy/linalg/_decomp_update.pyx":1634
 *                         <float_complex*>rptr, rs, k1, p1)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 *                     qr_block_row_delete(m, n, <double_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double_complex*>rptr, rs, k1, p1)
 *             return q1[p1:, p1:], r1[p1:, :]
 */
              __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_delete(__pyx_v_m, __pyx_v_n, ((__pyx_t_double_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_double_complex *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k1, __pyx_v_p1);
              break;
            }
          }

          /* "scipy/linalg/_decomp_update.pyx":1623
 *             qptr = extract(q1, qs)
 *             rptr = extract(r1, rs)
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     qr_block_row_delete(m, n, <float*>qptr, qs,
 */
          /*finally:*/ {
            /*normal exit:*/{
              #ifdef WITH_THREAD
              __Pyx_FastGIL_Forget();
              Py_BLOCK_THREADS
              #endif
              goto __pyx_L19;
            }
            __pyx_L19:;
          }
      }

      /* "scipy/linalg/_decomp_update.pyx":1636
 *                     qr_block_row_delete(m, n, <double_complex*>qptr, qs,
 *                         <double_complex*>rptr, rs, k1, p1)
 *             return q1[p1:, p1:], r1[p1:, :]             # <<<<<<<<<<<<<<
 *     elif which == 'col':
 *         # Special case single column removal to be more accepting of C ordered
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_v_p1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1636, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_3 = PySlice_New(__pyx_t_8, Py_None, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1636, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_v_p1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1636, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_9 = PySlice_New(__pyx_t_8, Py_None, Py_None); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1636, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1636, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_9);
      PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_9);
      __pyx_t_3 = 0;
      __pyx_t_9 = 0;
      __pyx_t_9 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_q1), __pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1636, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_v_p1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1636, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_3 = PySlice_New(__pyx_t_8, Py_None, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1636, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1636, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_3);
      __Pyx_INCREF(__pyx_slice__14);
      __Pyx_GIVEREF(__pyx_slice__14);
      PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_slice__14);
      __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_r1), __pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1636, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1636, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_9);
      PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_9);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_3);
      __pyx_t_9 = 0;
      __pyx_t_3 = 0;
      __pyx_r = __pyx_t_8;
      __pyx_t_8 = 0;
      goto __pyx_L0;
    }

    /* "scipy/linalg/_decomp_update.pyx":1571
 *     cdef bint chkfinite = check_finite, overwrite = overwrite_qr
 * 
 *     if which == 'row':             # <<<<<<<<<<<<<<
 *         q1, r1, typecode, m, n, economic = validate_qr(Q, R, overwrite,
 *                 ARRAY_ANYORDER, overwrite, ARRAY_ANYORDER, chkfinite)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1637
 *                         <double_complex*>rptr, rs, k1, p1)
 *             return q1[p1:, p1:], r1[p1:, :]
 *     elif which == 'col':             # <<<<<<<<<<<<<<
 *         # Special case single column removal to be more accepting of C ordered
 *         # inputs and to avoid allocating a work array for that case.
 */
  __pyx_t_12 = (__Pyx_PyUnicode_Equals(__pyx_v_which, __pyx_n_u_col, Py_EQ)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 1637, __pyx_L1_error)
  if (likely(__pyx_t_12)) {

    /* "scipy/linalg/_decomp_update.pyx":1640
 *         # Special case single column removal to be more accepting of C ordered
 *         # inputs and to avoid allocating a work array for that case.
 *         if p1 > 1:             # <<<<<<<<<<<<<<
 *             q1, r1, typecode, m, n, economic = validate_qr(Q, R, overwrite,
 *                     cnp.NPY_F_CONTIGUOUS, overwrite, cnp.NPY_F_CONTIGUOUS,
 */
    __pyx_t_12 = ((__pyx_v_p1 > 1) != 0);
    if (__pyx_t_12) {

      /* "scipy/linalg/_decomp_update.pyx":1641
 *         # inputs and to avoid allocating a work array for that case.
 *         if p1 > 1:
 *             q1, r1, typecode, m, n, economic = validate_qr(Q, R, overwrite,             # <<<<<<<<<<<<<<
 *                     cnp.NPY_F_CONTIGUOUS, overwrite, cnp.NPY_F_CONTIGUOUS,
 *                     chkfinite)
 */
      __pyx_t_8 = __pyx_f_5scipy_6linalg_14_decomp_update_validate_qr(__pyx_v_Q, __pyx_v_R, __pyx_v_overwrite, NPY_F_CONTIGUOUS, __pyx_v_overwrite, NPY_F_CONTIGUOUS, __pyx_v_chkfinite); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1641, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (likely(__pyx_t_8 != Py_None)) {
        PyObject* sequence = __pyx_t_8;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 6)) {
          if (size > 6) __Pyx_RaiseTooManyValuesError(6);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 1641, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_9 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 2); 
        __pyx_t_7 = PyTuple_GET_ITEM(sequence, 3); 
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 4); 
        __pyx_t_4 = PyTuple_GET_ITEM(sequence, 5); 
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        #else
        {
          Py_ssize_t i;
          PyObject** temps[6] = {&__pyx_t_3,&__pyx_t_9,&__pyx_t_6,&__pyx_t_7,&__pyx_t_5,&__pyx_t_4};
          for (i=0; i < 6; i++) {
            PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 1641, __pyx_L1_error)
            __Pyx_GOTREF(item);
            *(temps[i]) = item;
          }
        }
        #endif
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else {
        __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(0, 1641, __pyx_L1_error)
      }
      if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1641, __pyx_L1_error)
      if (!(likely(((__pyx_t_9) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_9, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1641, __pyx_L1_error)
      __pyx_t_11 = __Pyx_PyInt_As_int(__pyx_t_6); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1641, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_10 = __Pyx_PyInt_As_int(__pyx_t_7); if (unlikely((__pyx_t_10 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1641, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_t_5); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1641, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_12 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_12 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1641, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_v_q1 = ((PyArrayObject *)__pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_v_r1 = ((PyArrayObject *)__pyx_t_9);
      __pyx_t_9 = 0;
      __pyx_v_typecode = __pyx_t_11;
      __pyx_v_m = __pyx_t_10;
      __pyx_v_n = __pyx_t_1;
      __pyx_v_economic = __pyx_t_12;

      /* "scipy/linalg/_decomp_update.pyx":1640
 *         # Special case single column removal to be more accepting of C ordered
 *         # inputs and to avoid allocating a work array for that case.
 *         if p1 > 1:             # <<<<<<<<<<<<<<
 *             q1, r1, typecode, m, n, economic = validate_qr(Q, R, overwrite,
 *                     cnp.NPY_F_CONTIGUOUS, overwrite, cnp.NPY_F_CONTIGUOUS,
 */
      goto __pyx_L20;
    }

    /* "scipy/linalg/_decomp_update.pyx":1645
 *                     chkfinite)
 *         else:
 *             q1, r1, typecode, m, n, economic = validate_qr(Q, R, overwrite,             # <<<<<<<<<<<<<<
 *                     ARRAY_ANYORDER, overwrite, ARRAY_ANYORDER, chkfinite)
 *         o = n if economic else m
 */
    /*else*/ {

      /* "scipy/linalg/_decomp_update.pyx":1646
 *         else:
 *             q1, r1, typecode, m, n, economic = validate_qr(Q, R, overwrite,
 *                     ARRAY_ANYORDER, overwrite, ARRAY_ANYORDER, chkfinite)             # <<<<<<<<<<<<<<
 *         o = n if economic else m
 *         if not (-n <= k1 < n):
 */
      __pyx_t_8 = __pyx_f_5scipy_6linalg_14_decomp_update_validate_qr(__pyx_v_Q, __pyx_v_R, __pyx_v_overwrite, __pyx_v_5scipy_6linalg_14_decomp_update_ARRAY_ANYORDER, __pyx_v_overwrite, __pyx_v_5scipy_6linalg_14_decomp_update_ARRAY_ANYORDER, __pyx_v_chkfinite); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1645, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (likely(__pyx_t_8 != Py_None)) {
        PyObject* sequence = __pyx_t_8;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 6)) {
          if (size > 6) __Pyx_RaiseTooManyValuesError(6);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 1645, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_7 = PyTuple_GET_ITEM(sequence, 2); 
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 3); 
        __pyx_t_9 = PyTuple_GET_ITEM(sequence, 4); 
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 5); 
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_3);
        #else
        {
          Py_ssize_t i;
          PyObject** temps[6] = {&__pyx_t_4,&__pyx_t_5,&__pyx_t_7,&__pyx_t_6,&__pyx_t_9,&__pyx_t_3};
          for (i=0; i < 6; i++) {
            PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 1645, __pyx_L1_error)
            __Pyx_GOTREF(item);
            *(temps[i]) = item;
          }
        }
        #endif
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else {
        __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(0, 1645, __pyx_L1_error)
      }

      /* "scipy/linalg/_decomp_update.pyx":1645
 *                     chkfinite)
 *         else:
 *             q1, r1, typecode, m, n, economic = validate_qr(Q, R, overwrite,             # <<<<<<<<<<<<<<
 *                     ARRAY_ANYORDER, overwrite, ARRAY_ANYORDER, chkfinite)
 *         o = n if economic else m
 */
      if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1645, __pyx_L1_error)
      if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1645, __pyx_L1_error)
      __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_t_7); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1645, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_10 = __Pyx_PyInt_As_int(__pyx_t_6); if (unlikely((__pyx_t_10 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1645, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_11 = __Pyx_PyInt_As_int(__pyx_t_9); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1645, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_12 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_12 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1645, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_v_q1 = ((PyArrayObject *)__pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_v_r1 = ((PyArrayObject *)__pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_v_typecode = __pyx_t_1;
      __pyx_v_m = __pyx_t_10;
      __pyx_v_n = __pyx_t_11;
      __pyx_v_economic = __pyx_t_12;
    }
    __pyx_L20:;

    /* "scipy/linalg/_decomp_update.pyx":1647
 *             q1, r1, typecode, m, n, economic = validate_qr(Q, R, overwrite,
 *                     ARRAY_ANYORDER, overwrite, ARRAY_ANYORDER, chkfinite)
 *         o = n if economic else m             # <<<<<<<<<<<<<<
 *         if not (-n <= k1 < n):
 *             raise ValueError("'k' is out of bounds")
 */
    if ((__pyx_v_economic != 0)) {
      __pyx_t_11 = __pyx_v_n;
    } else {
      __pyx_t_11 = __pyx_v_m;
    }
    __pyx_v_o = __pyx_t_11;

    /* "scipy/linalg/_decomp_update.pyx":1648
 *                     ARRAY_ANYORDER, overwrite, ARRAY_ANYORDER, chkfinite)
 *         o = n if economic else m
 *         if not (-n <= k1 < n):             # <<<<<<<<<<<<<<
 *             raise ValueError("'k' is out of bounds")
 *         if k1 < 0:
 */
    __pyx_t_12 = ((-__pyx_v_n) <= __pyx_v_k1);
    if (__pyx_t_12) {
      __pyx_t_12 = (__pyx_v_k1 < __pyx_v_n);
    }
    __pyx_t_2 = ((!(__pyx_t_12 != 0)) != 0);
    if (unlikely(__pyx_t_2)) {

      /* "scipy/linalg/_decomp_update.pyx":1649
 *         o = n if economic else m
 *         if not (-n <= k1 < n):
 *             raise ValueError("'k' is out of bounds")             # <<<<<<<<<<<<<<
 *         if k1 < 0:
 *             k1 += n
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1649, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(0, 1649, __pyx_L1_error)

      /* "scipy/linalg/_decomp_update.pyx":1648
 *                     ARRAY_ANYORDER, overwrite, ARRAY_ANYORDER, chkfinite)
 *         o = n if economic else m
 *         if not (-n <= k1 < n):             # <<<<<<<<<<<<<<
 *             raise ValueError("'k' is out of bounds")
 *         if k1 < 0:
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1650
 *         if not (-n <= k1 < n):
 *             raise ValueError("'k' is out of bounds")
 *         if k1 < 0:             # <<<<<<<<<<<<<<
 *             k1 += n
 *         if k1 + p1 > n or p1 <= 0:
 */
    __pyx_t_2 = ((__pyx_v_k1 < 0) != 0);
    if (__pyx_t_2) {

      /* "scipy/linalg/_decomp_update.pyx":1651
 *             raise ValueError("'k' is out of bounds")
 *         if k1 < 0:
 *             k1 += n             # <<<<<<<<<<<<<<
 *         if k1 + p1 > n or p1 <= 0:
 *             raise ValueError("'p' is out of range")
 */
      __pyx_v_k1 = (__pyx_v_k1 + __pyx_v_n);

      /* "scipy/linalg/_decomp_update.pyx":1650
 *         if not (-n <= k1 < n):
 *             raise ValueError("'k' is out of bounds")
 *         if k1 < 0:             # <<<<<<<<<<<<<<
 *             k1 += n
 *         if k1 + p1 > n or p1 <= 0:
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1652
 *         if k1 < 0:
 *             k1 += n
 *         if k1 + p1 > n or p1 <= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("'p' is out of range")
 * 
 */
    __pyx_t_12 = (((__pyx_v_k1 + __pyx_v_p1) > __pyx_v_n) != 0);
    if (!__pyx_t_12) {
    } else {
      __pyx_t_2 = __pyx_t_12;
      goto __pyx_L24_bool_binop_done;
    }
    __pyx_t_12 = ((__pyx_v_p1 <= 0) != 0);
    __pyx_t_2 = __pyx_t_12;
    __pyx_L24_bool_binop_done:;
    if (unlikely(__pyx_t_2)) {

      /* "scipy/linalg/_decomp_update.pyx":1653
 *             k1 += n
 *         if k1 + p1 > n or p1 <= 0:
 *             raise ValueError("'p' is out of range")             # <<<<<<<<<<<<<<
 * 
 *         qptr = extract(q1, qs)
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1653, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(0, 1653, __pyx_L1_error)

      /* "scipy/linalg/_decomp_update.pyx":1652
 *         if k1 < 0:
 *             k1 += n
 *         if k1 + p1 > n or p1 <= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("'p' is out of range")
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1655
 *             raise ValueError("'p' is out of range")
 * 
 *         qptr = extract(q1, qs)             # <<<<<<<<<<<<<<
 *         rptr = extract(r1, rs)
 *         if p1 == 1:
 */
    __pyx_v_qptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_q1, __pyx_v_qs);

    /* "scipy/linalg/_decomp_update.pyx":1656
 * 
 *         qptr = extract(q1, qs)
 *         rptr = extract(r1, rs)             # <<<<<<<<<<<<<<
 *         if p1 == 1:
 *             with nogil:
 */
    __pyx_v_rptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_r1, __pyx_v_rs);

    /* "scipy/linalg/_decomp_update.pyx":1657
 *         qptr = extract(q1, qs)
 *         rptr = extract(r1, rs)
 *         if p1 == 1:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 */
    __pyx_t_2 = ((__pyx_v_p1 == 1) != 0);
    if (__pyx_t_2) {

      /* "scipy/linalg/_decomp_update.pyx":1658
 *         rptr = extract(r1, rs)
 *         if p1 == 1:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     qr_col_delete(m, o, n, <float*>qptr, qs,
 */
      {
          #ifdef WITH_THREAD
          PyThreadState *_save;
          Py_UNBLOCK_THREADS
          __Pyx_FastGIL_Remember();
          #endif
          /*try:*/ {

            /* "scipy/linalg/_decomp_update.pyx":1659
 *         if p1 == 1:
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     qr_col_delete(m, o, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, k1)
 */
            switch (__pyx_v_typecode) {
              case NPY_FLOAT:

              /* "scipy/linalg/_decomp_update.pyx":1660
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 *                     qr_col_delete(m, o, n, <float*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float*>rptr, rs, k1)
 *                 elif typecode == cnp.NPY_DOUBLE:
 */
              __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_delete(__pyx_v_m, __pyx_v_o, __pyx_v_n, ((float *)__pyx_v_qptr), __pyx_v_qs, ((float *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k1);

              /* "scipy/linalg/_decomp_update.pyx":1659
 *         if p1 == 1:
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     qr_col_delete(m, o, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, k1)
 */
              break;
              case NPY_DOUBLE:

              /* "scipy/linalg/_decomp_update.pyx":1663
 *                         <float*>rptr, rs, k1)
 *                 elif typecode == cnp.NPY_DOUBLE:
 *                     qr_col_delete(m, o, n, <double*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double*>rptr, rs, k1)
 *                 elif typecode == cnp.NPY_CFLOAT:
 */
              __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_delete(__pyx_v_m, __pyx_v_o, __pyx_v_n, ((double *)__pyx_v_qptr), __pyx_v_qs, ((double *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k1);

              /* "scipy/linalg/_decomp_update.pyx":1662
 *                     qr_col_delete(m, o, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, k1)
 *                 elif typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                     qr_col_delete(m, o, n, <double*>qptr, qs,
 *                         <double*>rptr, rs, k1)
 */
              break;
              case NPY_CFLOAT:

              /* "scipy/linalg/_decomp_update.pyx":1666
 *                         <double*>rptr, rs, k1)
 *                 elif typecode == cnp.NPY_CFLOAT:
 *                     qr_col_delete(m, o, n, <float_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float_complex*>rptr, rs, k1)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 */
              __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_delete(__pyx_v_m, __pyx_v_o, __pyx_v_n, ((__pyx_t_float_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_float_complex *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k1);

              /* "scipy/linalg/_decomp_update.pyx":1665
 *                     qr_col_delete(m, o, n, <double*>qptr, qs,
 *                         <double*>rptr, rs, k1)
 *                 elif typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                     qr_col_delete(m, o, n, <float_complex*>qptr, qs,
 *                         <float_complex*>rptr, rs, k1)
 */
              break;
              default:

              /* "scipy/linalg/_decomp_update.pyx":1669
 *                         <float_complex*>rptr, rs, k1)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 *                     qr_col_delete(m, o, n, <double_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double_complex*>rptr, rs, k1)
 *         else:
 */
              __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_delete(__pyx_v_m, __pyx_v_o, __pyx_v_n, ((__pyx_t_double_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_double_complex *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k1);
              break;
            }
          }

          /* "scipy/linalg/_decomp_update.pyx":1658
 *         rptr = extract(r1, rs)
 *         if p1 == 1:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     qr_col_delete(m, o, n, <float*>qptr, qs,
 */
          /*finally:*/ {
            /*normal exit:*/{
              #ifdef WITH_THREAD
              __Pyx_FastGIL_Forget();
              Py_BLOCK_THREADS
              #endif
              goto __pyx_L29;
            }
            __pyx_L29:;
          }
      }

      /* "scipy/linalg/_decomp_update.pyx":1657
 *         qptr = extract(q1, qs)
 *         rptr = extract(r1, rs)
 *         if p1 == 1:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 */
      goto __pyx_L26;
    }

    /* "scipy/linalg/_decomp_update.pyx":1672
 *                         <double_complex*>rptr, rs, k1)
 *         else:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     info = qr_block_col_delete(m, o, n, <float*>qptr, qs,
 */
    /*else*/ {
      {
          #ifdef WITH_THREAD
          PyThreadState *_save;
          Py_UNBLOCK_THREADS
          __Pyx_FastGIL_Remember();
          #endif
          /*try:*/ {

            /* "scipy/linalg/_decomp_update.pyx":1673
 *         else:
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     info = qr_block_col_delete(m, o, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, k1, p1)
 */
            switch (__pyx_v_typecode) {
              case NPY_FLOAT:

              /* "scipy/linalg/_decomp_update.pyx":1674
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 *                     info = qr_block_col_delete(m, o, n, <float*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float*>rptr, rs, k1, p1)
 *                 elif typecode == cnp.NPY_DOUBLE:
 */
              __pyx_v_info = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_delete(__pyx_v_m, __pyx_v_o, __pyx_v_n, ((float *)__pyx_v_qptr), __pyx_v_qs, ((float *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k1, __pyx_v_p1);

              /* "scipy/linalg/_decomp_update.pyx":1673
 *         else:
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     info = qr_block_col_delete(m, o, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, k1, p1)
 */
              break;
              case NPY_DOUBLE:

              /* "scipy/linalg/_decomp_update.pyx":1677
 *                         <float*>rptr, rs, k1, p1)
 *                 elif typecode == cnp.NPY_DOUBLE:
 *                     info = qr_block_col_delete(m, o, n, <double*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double*>rptr, rs, k1, p1)
 *                 elif typecode == cnp.NPY_CFLOAT:
 */
              __pyx_v_info = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_delete(__pyx_v_m, __pyx_v_o, __pyx_v_n, ((double *)__pyx_v_qptr), __pyx_v_qs, ((double *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k1, __pyx_v_p1);

              /* "scipy/linalg/_decomp_update.pyx":1676
 *                     info = qr_block_col_delete(m, o, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, k1, p1)
 *                 elif typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                     info = qr_block_col_delete(m, o, n, <double*>qptr, qs,
 *                         <double*>rptr, rs, k1, p1)
 */
              break;
              case NPY_CFLOAT:

              /* "scipy/linalg/_decomp_update.pyx":1680
 *                         <double*>rptr, rs, k1, p1)
 *                 elif typecode == cnp.NPY_CFLOAT:
 *                     info = qr_block_col_delete(m, o, n, <float_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float_complex*>rptr, rs, k1, p1)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 */
              __pyx_v_info = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_delete(__pyx_v_m, __pyx_v_o, __pyx_v_n, ((__pyx_t_float_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_float_complex *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k1, __pyx_v_p1);

              /* "scipy/linalg/_decomp_update.pyx":1679
 *                     info = qr_block_col_delete(m, o, n, <double*>qptr, qs,
 *                         <double*>rptr, rs, k1, p1)
 *                 elif typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                     info = qr_block_col_delete(m, o, n, <float_complex*>qptr, qs,
 *                         <float_complex*>rptr, rs, k1, p1)
 */
              break;
              default:

              /* "scipy/linalg/_decomp_update.pyx":1683
 *                         <float_complex*>rptr, rs, k1, p1)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 *                     info = qr_block_col_delete(m, o, n, <double_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double_complex*>rptr, rs, k1, p1)
 *             if info == MEMORY_ERROR:
 */
              __pyx_v_info = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_delete(__pyx_v_m, __pyx_v_o, __pyx_v_n, ((__pyx_t_double_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_double_complex *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k1, __pyx_v_p1);
              break;
            }
          }

          /* "scipy/linalg/_decomp_update.pyx":1672
 *                         <double_complex*>rptr, rs, k1)
 *         else:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     info = qr_block_col_delete(m, o, n, <float*>qptr, qs,
 */
          /*finally:*/ {
            /*normal exit:*/{
              #ifdef WITH_THREAD
              __Pyx_FastGIL_Forget();
              Py_BLOCK_THREADS
              #endif
              goto __pyx_L32;
            }
            __pyx_L32:;
          }
      }

      /* "scipy/linalg/_decomp_update.pyx":1685
 *                     info = qr_block_col_delete(m, o, n, <double_complex*>qptr, qs,
 *                         <double_complex*>rptr, rs, k1, p1)
 *             if info == MEMORY_ERROR:             # <<<<<<<<<<<<<<
 *                 raise MemoryError('Unable to allocate memory for array')
 *         if economic:
 */
      __pyx_t_2 = ((__pyx_v_info == __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR) != 0);
      if (unlikely(__pyx_t_2)) {

        /* "scipy/linalg/_decomp_update.pyx":1686
 *                         <double_complex*>rptr, rs, k1, p1)
 *             if info == MEMORY_ERROR:
 *                 raise MemoryError('Unable to allocate memory for array')             # <<<<<<<<<<<<<<
 *         if economic:
 *             return q1[:, :-p], r1[:-p, :-p]
 */
        __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1686, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_Raise(__pyx_t_8, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __PYX_ERR(0, 1686, __pyx_L1_error)

        /* "scipy/linalg/_decomp_update.pyx":1685
 *                     info = qr_block_col_delete(m, o, n, <double_complex*>qptr, qs,
 *                         <double_complex*>rptr, rs, k1, p1)
 *             if info == MEMORY_ERROR:             # <<<<<<<<<<<<<<
 *                 raise MemoryError('Unable to allocate memory for array')
 *         if economic:
 */
      }
    }
    __pyx_L26:;

    /* "scipy/linalg/_decomp_update.pyx":1687
 *             if info == MEMORY_ERROR:
 *                 raise MemoryError('Unable to allocate memory for array')
 *         if economic:             # <<<<<<<<<<<<<<
 *             return q1[:, :-p], r1[:-p, :-p]
 *         else:
 */
    __pyx_t_2 = (__pyx_v_economic != 0);
    if (__pyx_t_2) {

      /* "scipy/linalg/_decomp_update.pyx":1688
 *                 raise MemoryError('Unable to allocate memory for array')
 *         if economic:
 *             return q1[:, :-p], r1[:-p, :-p]             # <<<<<<<<<<<<<<
 *         else:
 *             return q1, r1[:, :-p]
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_8 = __Pyx_PyInt_From_int((-__pyx_v_p)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_3 = PySlice_New(Py_None, __pyx_t_8, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_INCREF(__pyx_slice__14);
      __Pyx_GIVEREF(__pyx_slice__14);
      PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_slice__14);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_q1), __pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyInt_From_int((-__pyx_v_p)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_9 = PySlice_New(Py_None, __pyx_t_8, Py_None); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyInt_From_int((-__pyx_v_p)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_6 = PySlice_New(Py_None, __pyx_t_8, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_9);
      PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_9);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_6);
      __pyx_t_9 = 0;
      __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_r1), __pyx_t_8); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_6);
      __pyx_t_3 = 0;
      __pyx_t_6 = 0;
      __pyx_r = __pyx_t_8;
      __pyx_t_8 = 0;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":1687
 *             if info == MEMORY_ERROR:
 *                 raise MemoryError('Unable to allocate memory for array')
 *         if economic:             # <<<<<<<<<<<<<<
 *             return q1[:, :-p], r1[:-p, :-p]
 *         else:
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1690
 *             return q1[:, :-p], r1[:-p, :-p]
 *         else:
 *             return q1, r1[:, :-p]             # <<<<<<<<<<<<<<
 *     else:
 *         raise ValueError("'which' must be either 'row' or 'col'")
 */
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_8 = __Pyx_PyInt_From_int((-__pyx_v_p)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1690, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_6 = PySlice_New(Py_None, __pyx_t_8, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1690, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1690, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_INCREF(__pyx_slice__14);
      __Pyx_GIVEREF(__pyx_slice__14);
      PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_slice__14);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_r1), __pyx_t_8); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1690, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1690, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_INCREF(((PyObject *)__pyx_v_q1));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_q1));
      PyTuple_SET_ITEM(__pyx_t_8, 0, ((PyObject *)__pyx_v_q1));
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_r = __pyx_t_8;
      __pyx_t_8 = 0;
      goto __pyx_L0;
    }

    /* "scipy/linalg/_decomp_update.pyx":1637
 *                         <double_complex*>rptr, rs, k1, p1)
 *             return q1[p1:, p1:], r1[p1:, :]
 *     elif which == 'col':             # <<<<<<<<<<<<<<
 *         # Special case single column removal to be more accepting of C ordered
 *         # inputs and to avoid allocating a work array for that case.
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1692
 *             return q1, r1[:, :-p]
 *     else:
 *         raise ValueError("'which' must be either 'row' or 'col'")             # <<<<<<<<<<<<<<
 * 
 * @cython.embedsignature(True)
 */
  /*else*/ {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1692, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __PYX_ERR(0, 1692, __pyx_L1_error)
  }

  /* "scipy/linalg/_decomp_update.pyx":1451
 * 
 * @cython.embedsignature(True)
 * def qr_delete(Q, R, k, int p=1, which='row', overwrite_qr=False,             # <<<<<<<<<<<<<<
 *               check_finite=True):
 *     """
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("scipy.linalg._decomp_update.qr_delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_q1);
  __Pyx_XDECREF((PyObject *)__pyx_v_r1);
  __Pyx_XDECREF(__pyx_v_qnew);
  __Pyx_XDECREF(__pyx_v_norm);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":1695
 * 
 * @cython.embedsignature(True)
 * def qr_insert(Q, R, u, k, which='row', rcond=None, overwrite_qru=False, check_finite=True):             # <<<<<<<<<<<<<<
 *     """
 *     QR update on row or column insertions
 */

/* Python wrapper */
static PyObject *__pyx_pw_5scipy_6linalg_14_decomp_update_5qr_insert(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5scipy_6linalg_14_decomp_update_4qr_insert[] = "qr_insert(Q, R, u, k, which=u'row', rcond=None, overwrite_qru=False, check_finite=True)\n\n    QR update on row or column insertions\n\n    If ``A = Q R`` is the QR factorization of ``A``, return the QR\n    factorization of ``A`` where rows or columns have been inserted starting\n    at row or column ``k``.\n\n    Parameters\n    ----------\n    Q : (M, M) array_like\n        Unitary/orthogonal matrix from the QR decomposition of A.\n    R : (M, N) array_like\n        Upper triangular matrix from the QR decomposition of A.\n    u : (N,), (p, N), (M,), or (M, p) array_like\n        Rows or columns to insert\n    k : int\n        Index before which `u` is to be inserted.\n    which: {'row', 'col'}, optional\n        Determines if rows or columns will be inserted, defaults to 'row'\n    rcond : float\n        Lower bound on the reciprocal condition number of ``Q`` augmented with\n        ``u/||u||`` Only used when updating economic mode (thin, (M,N) (N,N))\n        decompositions.  If None, machine precision is used.  Defaults to\n        None.\n    overwrite_qru : bool, optional\n        If True, consume Q, R, and u, if possible, while performing the update,\n        otherwise make copies as necessary. Defaults to False.\n    check_finite : bool, optional\n        Whether to check that the input matrices contain only finite numbers.\n        Disabling may give a performance gain, but may result in problems\n        (crashes, non-termination) if the inputs do contain infinities or NaNs.\n        Default is True.\n\n    Returns\n    -------\n    Q1 : ndarray\n        Updated unitary/orthogonal factor\n    R1 : ndarray\n        Updated upper triangular factor\n\n    Raises\n    ------\n    LinAlgError :\n        If updating a (M,N) (N,N) factorization and the reciprocal condition\n        number of Q augmented with u/||u|| is smaller than rcond.\n\n    See Also\n    --------\n    qr, qr_multiply, qr_delete, qr_update\n\n    Notes\n    -----\n    This routine does not gu""arantee that the diagonal entries of ``R1`` are\n    positive.\n\n    .. versionadded:: 0.16.0\n\n    References\n    ----------\n\n    .. [1] Golub, G. H. & Van Loan, C. F. Matrix Computations, 3rd Ed.\n           (Johns Hopkins University Press, 1996).\n\n    .. [2] Daniel, J. W., Gragg, W. B., Kaufman, L. & Stewart, G. W.\n           Reorthogonalization and stable algorithms for updating the\n           Gram-Schmidt QR factorization. Math. Comput. 30, 772-795 (1976).\n\n    .. [3] Reichel, L. & Gragg, W. B. Algorithm 686: FORTRAN Subroutines for\n           Updating the QR Decomposition. ACM Trans. Math. Softw. 16, 369-377\n           (1990).\n\n    Examples\n    --------\n    >>> from scipy import linalg\n    >>> a = np.array([[  3.,  -2.,  -2.],\n    ...               [  6.,  -7.,   4.],\n    ...               [  7.,   8.,  -6.]])\n    >>> q, r = linalg.qr(a)\n\n    Given this QR decomposition, update q and r when 2 rows are inserted.\n\n    >>> u = np.array([[  6.,  -9.,  -3.],\n    ...               [ -3.,  10.,   1.]])\n    >>> q1, r1 = linalg.qr_insert(q, r, u, 2, 'row')\n    >>> q1\n    array([[-0.25445668,  0.02246245,  0.18146236, -0.72798806,  0.60979671],  # may vary (signs)\n           [-0.50891336,  0.23226178, -0.82836478, -0.02837033, -0.00828114],\n           [-0.50891336,  0.35715302,  0.38937158,  0.58110733,  0.35235345],\n           [ 0.25445668, -0.52202743, -0.32165498,  0.36263239,  0.65404509],\n           [-0.59373225, -0.73856549,  0.16065817, -0.0063658 , -0.27595554]])\n    >>> r1\n    array([[-11.78982612,   6.44623587,   3.81685018],  # may vary (signs)\n           [  0.        , -16.01393278,   3.72202865],\n           [  0.        ,   0.        ,  -6.13010256],\n           [  0.        ,   0.        ,   0.        ],\n           [  0.        ,   0.        ,   0.        ]])\n\n    The update is equivalent, but faster than the following.\n\n    >>> a1 = np.insert(a, 2, u, 0)\n    >>> a1\n    array([[  3.,  -2.,  -2.],\n           [  6"".,  -7.,   4.],\n           [  6.,  -9.,  -3.],\n           [ -3.,  10.,   1.],\n           [  7.,   8.,  -6.]])\n    >>> q_direct, r_direct = linalg.qr(a1)\n\n    Check that we have equivalent results:\n\n    >>> np.dot(q1, r1)\n    array([[  3.,  -2.,  -2.],\n           [  6.,  -7.,   4.],\n           [  6.,  -9.,  -3.],\n           [ -3.,  10.,   1.],\n           [  7.,   8.,  -6.]])\n\n    >>> np.allclose(np.dot(q1, r1), a1)\n    True\n\n    And the updated Q is still unitary:\n\n    >>> np.allclose(np.dot(q1.T, q1), np.eye(5))\n    True\n\n    ";
static PyMethodDef __pyx_mdef_5scipy_6linalg_14_decomp_update_5qr_insert = {"qr_insert", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5scipy_6linalg_14_decomp_update_5qr_insert, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5scipy_6linalg_14_decomp_update_4qr_insert};
static PyObject *__pyx_pw_5scipy_6linalg_14_decomp_update_5qr_insert(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_Q = 0;
  PyObject *__pyx_v_R = 0;
  PyObject *__pyx_v_u = 0;
  PyObject *__pyx_v_k = 0;
  PyObject *__pyx_v_which = 0;
  PyObject *__pyx_v_rcond = 0;
  PyObject *__pyx_v_overwrite_qru = 0;
  PyObject *__pyx_v_check_finite = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("qr_insert (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_Q,&__pyx_n_s_R,&__pyx_n_s_u,&__pyx_n_s_k,&__pyx_n_s_which,&__pyx_n_s_rcond,&__pyx_n_s_overwrite_qru,&__pyx_n_s_check_finite,0};
    PyObject* values[8] = {0,0,0,0,0,0,0,0};
    values[4] = ((PyObject *)__pyx_n_u_row);
    values[5] = ((PyObject *)Py_None);
    values[6] = ((PyObject *)Py_False);
    values[7] = ((PyObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_Q)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_R)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("qr_insert", 0, 4, 8, 1); __PYX_ERR(0, 1695, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_u)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("qr_insert", 0, 4, 8, 2); __PYX_ERR(0, 1695, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_k)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("qr_insert", 0, 4, 8, 3); __PYX_ERR(0, 1695, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_which);
          if (value) { values[4] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_rcond);
          if (value) { values[5] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_overwrite_qru);
          if (value) { values[6] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_check_finite);
          if (value) { values[7] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "qr_insert") < 0)) __PYX_ERR(0, 1695, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_Q = values[0];
    __pyx_v_R = values[1];
    __pyx_v_u = values[2];
    __pyx_v_k = values[3];
    __pyx_v_which = values[4];
    __pyx_v_rcond = values[5];
    __pyx_v_overwrite_qru = values[6];
    __pyx_v_check_finite = values[7];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("qr_insert", 0, 4, 8, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1695, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("scipy.linalg._decomp_update.qr_insert", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5scipy_6linalg_14_decomp_update_4qr_insert(__pyx_self, __pyx_v_Q, __pyx_v_R, __pyx_v_u, __pyx_v_k, __pyx_v_which, __pyx_v_rcond, __pyx_v_overwrite_qru, __pyx_v_check_finite);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5scipy_6linalg_14_decomp_update_4qr_insert(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_Q, PyObject *__pyx_v_R, PyObject *__pyx_v_u, PyObject *__pyx_v_k, PyObject *__pyx_v_which, PyObject *__pyx_v_rcond, PyObject *__pyx_v_overwrite_qru, PyObject *__pyx_v_check_finite) {
  int __pyx_v_chkfinite;
  int __pyx_v_overwrite;
  int __pyx_v_k1;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("qr_insert", 0);

  /* "scipy/linalg/_decomp_update.pyx":1822
 * 
 *     """
 *     cdef bint chkfinite = check_finite, overwrite = overwrite_qru             # <<<<<<<<<<<<<<
 *     cdef int k1 = k
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_check_finite); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1822, __pyx_L1_error)
  __pyx_v_chkfinite = __pyx_t_1;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_overwrite_qru); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1822, __pyx_L1_error)
  __pyx_v_overwrite = __pyx_t_1;

  /* "scipy/linalg/_decomp_update.pyx":1823
 *     """
 *     cdef bint chkfinite = check_finite, overwrite = overwrite_qru
 *     cdef int k1 = k             # <<<<<<<<<<<<<<
 * 
 *     if which == 'row':
 */
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_v_k); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1823, __pyx_L1_error)
  __pyx_v_k1 = __pyx_t_2;

  /* "scipy/linalg/_decomp_update.pyx":1825
 *     cdef int k1 = k
 * 
 *     if which == 'row':             # <<<<<<<<<<<<<<
 *         if rcond is not None:
 *             raise ValueError("'rcond' is unused when inserting rows and "
 */
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_which, __pyx_n_u_row, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 1825, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":1826
 * 
 *     if which == 'row':
 *         if rcond is not None:             # <<<<<<<<<<<<<<
 *             raise ValueError("'rcond' is unused when inserting rows and "
 *                              "must be None")
 */
    __pyx_t_1 = (__pyx_v_rcond != Py_None);
    __pyx_t_3 = (__pyx_t_1 != 0);
    if (unlikely(__pyx_t_3)) {

      /* "scipy/linalg/_decomp_update.pyx":1827
 *     if which == 'row':
 *         if rcond is not None:
 *             raise ValueError("'rcond' is unused when inserting rows and "             # <<<<<<<<<<<<<<
 *                              "must be None")
 *         return qr_insert_row(Q, R, u, k1, overwrite, chkfinite)
 */
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1827, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_Raise(__pyx_t_4, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __PYX_ERR(0, 1827, __pyx_L1_error)

      /* "scipy/linalg/_decomp_update.pyx":1826
 * 
 *     if which == 'row':
 *         if rcond is not None:             # <<<<<<<<<<<<<<
 *             raise ValueError("'rcond' is unused when inserting rows and "
 *                              "must be None")
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1829
 *             raise ValueError("'rcond' is unused when inserting rows and "
 *                              "must be None")
 *         return qr_insert_row(Q, R, u, k1, overwrite, chkfinite)             # <<<<<<<<<<<<<<
 *     elif which == 'col':
 *         return qr_insert_col(Q, R, u, k1, rcond, overwrite, chkfinite)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __pyx_f_5scipy_6linalg_14_decomp_update_qr_insert_row(__pyx_v_Q, __pyx_v_R, __pyx_v_u, __pyx_v_k1, __pyx_v_overwrite, __pyx_v_chkfinite); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1829, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":1825
 *     cdef int k1 = k
 * 
 *     if which == 'row':             # <<<<<<<<<<<<<<
 *         if rcond is not None:
 *             raise ValueError("'rcond' is unused when inserting rows and "
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1830
 *                              "must be None")
 *         return qr_insert_row(Q, R, u, k1, overwrite, chkfinite)
 *     elif which == 'col':             # <<<<<<<<<<<<<<
 *         return qr_insert_col(Q, R, u, k1, rcond, overwrite, chkfinite)
 *     else:
 */
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_v_which, __pyx_n_u_col, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 1830, __pyx_L1_error)
  if (likely(__pyx_t_3)) {

    /* "scipy/linalg/_decomp_update.pyx":1831
 *         return qr_insert_row(Q, R, u, k1, overwrite, chkfinite)
 *     elif which == 'col':
 *         return qr_insert_col(Q, R, u, k1, rcond, overwrite, chkfinite)             # <<<<<<<<<<<<<<
 *     else:
 *         raise ValueError("'which' must be either 'row' or 'col'")
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __pyx_f_5scipy_6linalg_14_decomp_update_qr_insert_col(__pyx_v_Q, __pyx_v_R, __pyx_v_u, __pyx_v_k1, __pyx_v_rcond, __pyx_v_overwrite, __pyx_v_chkfinite); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1831, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":1830
 *                              "must be None")
 *         return qr_insert_row(Q, R, u, k1, overwrite, chkfinite)
 *     elif which == 'col':             # <<<<<<<<<<<<<<
 *         return qr_insert_col(Q, R, u, k1, rcond, overwrite, chkfinite)
 *     else:
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1833
 *         return qr_insert_col(Q, R, u, k1, rcond, overwrite, chkfinite)
 *     else:
 *         raise ValueError("'which' must be either 'row' or 'col'")             # <<<<<<<<<<<<<<
 * 
 * cdef qr_insert_row(Q, R, u, int k, bint overwrite_qru, bint check_finite):
 */
  /*else*/ {
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1833, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 1833, __pyx_L1_error)
  }

  /* "scipy/linalg/_decomp_update.pyx":1695
 * 
 * @cython.embedsignature(True)
 * def qr_insert(Q, R, u, k, which='row', rcond=None, overwrite_qru=False, check_finite=True):             # <<<<<<<<<<<<<<
 *     """
 *     QR update on row or column insertions
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("scipy.linalg._decomp_update.qr_insert", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":1835
 *         raise ValueError("'which' must be either 'row' or 'col'")
 * 
 * cdef qr_insert_row(Q, R, u, int k, bint overwrite_qru, bint check_finite):             # <<<<<<<<<<<<<<
 *     cdef cnp.ndarray q1, r1, u1, qnew, rnew
 *     cdef int j
 */

static PyObject *__pyx_f_5scipy_6linalg_14_decomp_update_qr_insert_row(PyObject *__pyx_v_Q, PyObject *__pyx_v_R, PyObject *__pyx_v_u, int __pyx_v_k, int __pyx_v_overwrite_qru, int __pyx_v_check_finite) {
  PyArrayObject *__pyx_v_q1 = 0;
  PyArrayObject *__pyx_v_r1 = 0;
  PyArrayObject *__pyx_v_u1 = 0;
  PyArrayObject *__pyx_v_qnew = 0;
  PyArrayObject *__pyx_v_rnew = 0;
  int __pyx_v_j;
  int __pyx_v_u_flags;
  int __pyx_v_typecode;
  int __pyx_v_m;
  int __pyx_v_n;
  int __pyx_v_p;
  int __pyx_v_info;
  void *__pyx_v_qptr;
  void *__pyx_v_rptr;
  void *__pyx_v_uptr;
  int __pyx_v_qs[2];
  int __pyx_v_rs[2];
  int __pyx_v_us[2];
  npy_intp __pyx_v_shape[2];
  int __pyx_v_economic;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  __Pyx_RefNannySetupContext("qr_insert_row", 0);

  /* "scipy/linalg/_decomp_update.pyx":1838
 *     cdef cnp.ndarray q1, r1, u1, qnew, rnew
 *     cdef int j
 *     cdef int u_flags = cnp.NPY_BEHAVED_NS | cnp.NPY_ELEMENTSTRIDES             # <<<<<<<<<<<<<<
 *     cdef int typecode, m, n, p, info
 *     cdef void* qptr
 */
  __pyx_v_u_flags = (NPY_BEHAVED_NS | NPY_ELEMENTSTRIDES);

  /* "scipy/linalg/_decomp_update.pyx":1854
 *     # p full q alloc, r alloc, u any
 * 
 *     q1, r1, typecode, m, n, economic = validate_qr(Q, R, True, ARRAY_ANYORDER,             # <<<<<<<<<<<<<<
 *             True, ARRAY_ANYORDER, check_finite)
 *     u1 = PyArray_CheckFromAny(u, NULL, 0, 0, u_flags, NULL)
 */
  __pyx_t_1 = __pyx_f_5scipy_6linalg_14_decomp_update_validate_qr(__pyx_v_Q, __pyx_v_R, 1, __pyx_v_5scipy_6linalg_14_decomp_update_ARRAY_ANYORDER, 1, __pyx_v_5scipy_6linalg_14_decomp_update_ARRAY_ANYORDER, __pyx_v_check_finite); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1854, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(__pyx_t_1 != Py_None)) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 6)) {
      if (size > 6) __Pyx_RaiseTooManyValuesError(6);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 1854, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
    __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
    __pyx_t_4 = PyTuple_GET_ITEM(sequence, 2); 
    __pyx_t_5 = PyTuple_GET_ITEM(sequence, 3); 
    __pyx_t_6 = PyTuple_GET_ITEM(sequence, 4); 
    __pyx_t_7 = PyTuple_GET_ITEM(sequence, 5); 
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_t_7);
    #else
    {
      Py_ssize_t i;
      PyObject** temps[6] = {&__pyx_t_2,&__pyx_t_3,&__pyx_t_4,&__pyx_t_5,&__pyx_t_6,&__pyx_t_7};
      for (i=0; i < 6; i++) {
        PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 1854, __pyx_L1_error)
        __Pyx_GOTREF(item);
        *(temps[i]) = item;
      }
    }
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(0, 1854, __pyx_L1_error)
  }
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1854, __pyx_L1_error)
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1854, __pyx_L1_error)
  __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1854, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_9 = __Pyx_PyInt_As_int(__pyx_t_5); if (unlikely((__pyx_t_9 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1854, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_10 = __Pyx_PyInt_As_int(__pyx_t_6); if (unlikely((__pyx_t_10 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1854, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1854, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_q1 = ((PyArrayObject *)__pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_v_r1 = ((PyArrayObject *)__pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_v_typecode = __pyx_t_8;
  __pyx_v_m = __pyx_t_9;
  __pyx_v_n = __pyx_t_10;
  __pyx_v_economic = __pyx_t_11;

  /* "scipy/linalg/_decomp_update.pyx":1856
 *     q1, r1, typecode, m, n, economic = validate_qr(Q, R, True, ARRAY_ANYORDER,
 *             True, ARRAY_ANYORDER, check_finite)
 *     u1 = PyArray_CheckFromAny(u, NULL, 0, 0, u_flags, NULL)             # <<<<<<<<<<<<<<
 * 
 *     if cnp.PyArray_TYPE(u1) != typecode:
 */
  __pyx_t_1 = ((PyObject *)PyArray_CheckFromAny(__pyx_v_u, NULL, 0, 0, __pyx_v_u_flags, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1856, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_u1 = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "scipy/linalg/_decomp_update.pyx":1858
 *     u1 = PyArray_CheckFromAny(u, NULL, 0, 0, u_flags, NULL)
 * 
 *     if cnp.PyArray_TYPE(u1) != typecode:             # <<<<<<<<<<<<<<
 *         raise ValueError("'u' must have the same type as 'Q' and 'R'")
 * 
 */
  __pyx_t_11 = ((PyArray_TYPE(__pyx_v_u1) != __pyx_v_typecode) != 0);
  if (unlikely(__pyx_t_11)) {

    /* "scipy/linalg/_decomp_update.pyx":1859
 * 
 *     if cnp.PyArray_TYPE(u1) != typecode:
 *         raise ValueError("'u' must have the same type as 'Q' and 'R'")             # <<<<<<<<<<<<<<
 * 
 *     if not (-m <= k <= m):
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__20, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1859, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1859, __pyx_L1_error)

    /* "scipy/linalg/_decomp_update.pyx":1858
 *     u1 = PyArray_CheckFromAny(u, NULL, 0, 0, u_flags, NULL)
 * 
 *     if cnp.PyArray_TYPE(u1) != typecode:             # <<<<<<<<<<<<<<
 *         raise ValueError("'u' must have the same type as 'Q' and 'R'")
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1861
 *         raise ValueError("'u' must have the same type as 'Q' and 'R'")
 * 
 *     if not (-m <= k <= m):             # <<<<<<<<<<<<<<
 *         raise ValueError("'k' is out of bounds")
 * 
 */
  __pyx_t_11 = ((-__pyx_v_m) <= __pyx_v_k);
  if (__pyx_t_11) {
    __pyx_t_11 = (__pyx_v_k <= __pyx_v_m);
  }
  __pyx_t_12 = ((!(__pyx_t_11 != 0)) != 0);
  if (unlikely(__pyx_t_12)) {

    /* "scipy/linalg/_decomp_update.pyx":1862
 * 
 *     if not (-m <= k <= m):
 *         raise ValueError("'k' is out of bounds")             # <<<<<<<<<<<<<<
 * 
 *     if k < 0:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1862, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1862, __pyx_L1_error)

    /* "scipy/linalg/_decomp_update.pyx":1861
 *         raise ValueError("'u' must have the same type as 'Q' and 'R'")
 * 
 *     if not (-m <= k <= m):             # <<<<<<<<<<<<<<
 *         raise ValueError("'k' is out of bounds")
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1864
 *         raise ValueError("'k' is out of bounds")
 * 
 *     if k < 0:             # <<<<<<<<<<<<<<
 *         k += m
 * 
 */
  __pyx_t_12 = ((__pyx_v_k < 0) != 0);
  if (__pyx_t_12) {

    /* "scipy/linalg/_decomp_update.pyx":1865
 * 
 *     if k < 0:
 *         k += m             # <<<<<<<<<<<<<<
 * 
 *     if u1.ndim == 2:
 */
    __pyx_v_k = (__pyx_v_k + __pyx_v_m);

    /* "scipy/linalg/_decomp_update.pyx":1864
 *         raise ValueError("'k' is out of bounds")
 * 
 *     if k < 0:             # <<<<<<<<<<<<<<
 *         k += m
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1867
 *         k += m
 * 
 *     if u1.ndim == 2:             # <<<<<<<<<<<<<<
 *         p = u1.shape[0]
 *         if u1.shape[1] != n:
 */
  switch (__pyx_v_u1->nd) {
    case 2:

    /* "scipy/linalg/_decomp_update.pyx":1868
 * 
 *     if u1.ndim == 2:
 *         p = u1.shape[0]             # <<<<<<<<<<<<<<
 *         if u1.shape[1] != n:
 *             raise ValueError("'u' should be either (N,) or (p,N) when "
 */
    __pyx_v_p = (__pyx_v_u1->dimensions[0]);

    /* "scipy/linalg/_decomp_update.pyx":1869
 *     if u1.ndim == 2:
 *         p = u1.shape[0]
 *         if u1.shape[1] != n:             # <<<<<<<<<<<<<<
 *             raise ValueError("'u' should be either (N,) or (p,N) when "
 *                              "inserting rows. Found %s." %
 */
    __pyx_t_12 = (((__pyx_v_u1->dimensions[1]) != __pyx_v_n) != 0);
    if (unlikely(__pyx_t_12)) {

      /* "scipy/linalg/_decomp_update.pyx":1872
 *             raise ValueError("'u' should be either (N,) or (p,N) when "
 *                              "inserting rows. Found %s." %
 *                              str(getattr(u1, 'shape')))             # <<<<<<<<<<<<<<
 *     elif u1.ndim == 1:
 *         p = 1
 */
      __pyx_t_1 = __Pyx_GetAttr(((PyObject *)__pyx_v_u1), __pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1872, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyUnicode_Type)), __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1872, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "scipy/linalg/_decomp_update.pyx":1871
 *         if u1.shape[1] != n:
 *             raise ValueError("'u' should be either (N,) or (p,N) when "
 *                              "inserting rows. Found %s." %             # <<<<<<<<<<<<<<
 *                              str(getattr(u1, 'shape')))
 *     elif u1.ndim == 1:
 */
      __pyx_t_1 = PyUnicode_Format(__pyx_kp_u_u_should_be_either_N_or_p_N_whe, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1871, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "scipy/linalg/_decomp_update.pyx":1870
 *         p = u1.shape[0]
 *         if u1.shape[1] != n:
 *             raise ValueError("'u' should be either (N,) or (p,N) when "             # <<<<<<<<<<<<<<
 *                              "inserting rows. Found %s." %
 *                              str(getattr(u1, 'shape')))
 */
      __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1870, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_Raise(__pyx_t_7, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __PYX_ERR(0, 1870, __pyx_L1_error)

      /* "scipy/linalg/_decomp_update.pyx":1869
 *     if u1.ndim == 2:
 *         p = u1.shape[0]
 *         if u1.shape[1] != n:             # <<<<<<<<<<<<<<
 *             raise ValueError("'u' should be either (N,) or (p,N) when "
 *                              "inserting rows. Found %s." %
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1867
 *         k += m
 * 
 *     if u1.ndim == 2:             # <<<<<<<<<<<<<<
 *         p = u1.shape[0]
 *         if u1.shape[1] != n:
 */
    break;
    case 1:

    /* "scipy/linalg/_decomp_update.pyx":1874
 *                              str(getattr(u1, 'shape')))
 *     elif u1.ndim == 1:
 *         p = 1             # <<<<<<<<<<<<<<
 *         if u1.shape[0] != n:
 *             raise ValueError("'u' should be either (N,) or (p,N) when "
 */
    __pyx_v_p = 1;

    /* "scipy/linalg/_decomp_update.pyx":1875
 *     elif u1.ndim == 1:
 *         p = 1
 *         if u1.shape[0] != n:             # <<<<<<<<<<<<<<
 *             raise ValueError("'u' should be either (N,) or (p,N) when "
 *                              "inserting rows. Found %s." %
 */
    __pyx_t_12 = (((__pyx_v_u1->dimensions[0]) != __pyx_v_n) != 0);
    if (unlikely(__pyx_t_12)) {

      /* "scipy/linalg/_decomp_update.pyx":1878
 *             raise ValueError("'u' should be either (N,) or (p,N) when "
 *                              "inserting rows. Found %s." %
 *                              str(getattr(u1, 'shape')))             # <<<<<<<<<<<<<<
 *     else:
 *         raise ValueError("'u' must be either 1- or 2-D")
 */
      __pyx_t_7 = __Pyx_GetAttr(((PyObject *)__pyx_v_u1), __pyx_n_u_shape); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1878, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyUnicode_Type)), __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1878, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "scipy/linalg/_decomp_update.pyx":1877
 *         if u1.shape[0] != n:
 *             raise ValueError("'u' should be either (N,) or (p,N) when "
 *                              "inserting rows. Found %s." %             # <<<<<<<<<<<<<<
 *                              str(getattr(u1, 'shape')))
 *     else:
 */
      __pyx_t_7 = PyUnicode_Format(__pyx_kp_u_u_should_be_either_N_or_p_N_whe, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1877, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "scipy/linalg/_decomp_update.pyx":1876
 *         p = 1
 *         if u1.shape[0] != n:
 *             raise ValueError("'u' should be either (N,) or (p,N) when "             # <<<<<<<<<<<<<<
 *                              "inserting rows. Found %s." %
 *                              str(getattr(u1, 'shape')))
 */
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1876, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 1876, __pyx_L1_error)

      /* "scipy/linalg/_decomp_update.pyx":1875
 *     elif u1.ndim == 1:
 *         p = 1
 *         if u1.shape[0] != n:             # <<<<<<<<<<<<<<
 *             raise ValueError("'u' should be either (N,) or (p,N) when "
 *                              "inserting rows. Found %s." %
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1873
 *                              "inserting rows. Found %s." %
 *                              str(getattr(u1, 'shape')))
 *     elif u1.ndim == 1:             # <<<<<<<<<<<<<<
 *         p = 1
 *         if u1.shape[0] != n:
 */
    break;
    default:

    /* "scipy/linalg/_decomp_update.pyx":1880
 *                              str(getattr(u1, 'shape')))
 *     else:
 *         raise ValueError("'u' must be either 1- or 2-D")             # <<<<<<<<<<<<<<
 * 
 *     u1 = validate_array(u1, check_finite)
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1880, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1880, __pyx_L1_error)
    break;
  }

  /* "scipy/linalg/_decomp_update.pyx":1882
 *         raise ValueError("'u' must be either 1- or 2-D")
 * 
 *     u1 = validate_array(u1, check_finite)             # <<<<<<<<<<<<<<
 * 
 *     if economic:
 */
  __pyx_t_1 = __pyx_f_5scipy_6linalg_14_decomp_update_validate_array(__pyx_v_u1, __pyx_v_check_finite); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1882, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1882, __pyx_L1_error)
  __Pyx_DECREF_SET(__pyx_v_u1, ((PyArrayObject *)__pyx_t_1));
  __pyx_t_1 = 0;

  /* "scipy/linalg/_decomp_update.pyx":1884
 *     u1 = validate_array(u1, check_finite)
 * 
 *     if economic:             # <<<<<<<<<<<<<<
 *         if not overwrite_qru:
 *             r1 = PyArray_FromArray(r1, NULL,
 */
  __pyx_t_12 = (__pyx_v_economic != 0);
  if (__pyx_t_12) {

    /* "scipy/linalg/_decomp_update.pyx":1885
 * 
 *     if economic:
 *         if not overwrite_qru:             # <<<<<<<<<<<<<<
 *             r1 = PyArray_FromArray(r1, NULL,
 *                     cnp.NPY_F_CONTIGUOUS | cnp.NPY_ENSURECOPY)
 */
    __pyx_t_12 = ((!(__pyx_v_overwrite_qru != 0)) != 0);
    if (__pyx_t_12) {

      /* "scipy/linalg/_decomp_update.pyx":1886
 *     if economic:
 *         if not overwrite_qru:
 *             r1 = PyArray_FromArray(r1, NULL,             # <<<<<<<<<<<<<<
 *                     cnp.NPY_F_CONTIGUOUS | cnp.NPY_ENSURECOPY)
 *             u1 = PyArray_FromArray(u1, NULL,
 */
      __pyx_t_1 = ((PyObject *)PyArray_FromArray(__pyx_v_r1, NULL, (NPY_F_CONTIGUOUS | NPY_ENSURECOPY))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1886, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF_SET(__pyx_v_r1, ((PyArrayObject *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "scipy/linalg/_decomp_update.pyx":1888
 *             r1 = PyArray_FromArray(r1, NULL,
 *                     cnp.NPY_F_CONTIGUOUS | cnp.NPY_ENSURECOPY)
 *             u1 = PyArray_FromArray(u1, NULL,             # <<<<<<<<<<<<<<
 *                     cnp.NPY_F_CONTIGUOUS | cnp.NPY_ENSURECOPY)
 * 
 */
      __pyx_t_1 = ((PyObject *)PyArray_FromArray(__pyx_v_u1, NULL, (NPY_F_CONTIGUOUS | NPY_ENSURECOPY))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1888, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF_SET(__pyx_v_u1, ((PyArrayObject *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "scipy/linalg/_decomp_update.pyx":1885
 * 
 *     if economic:
 *         if not overwrite_qru:             # <<<<<<<<<<<<<<
 *             r1 = PyArray_FromArray(r1, NULL,
 *                     cnp.NPY_F_CONTIGUOUS | cnp.NPY_ENSURECOPY)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":1891
 *                     cnp.NPY_F_CONTIGUOUS | cnp.NPY_ENSURECOPY)
 * 
 *         shape[0] = m + p             # <<<<<<<<<<<<<<
 *         shape[1] = n + p
 *         qnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 */
    (__pyx_v_shape[0]) = (__pyx_v_m + __pyx_v_p);

    /* "scipy/linalg/_decomp_update.pyx":1892
 * 
 *         shape[0] = m + p
 *         shape[1] = n + p             # <<<<<<<<<<<<<<
 *         qnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 *         qnew[:-p,:-p] = q1
 */
    (__pyx_v_shape[1]) = (__pyx_v_n + __pyx_v_p);

    /* "scipy/linalg/_decomp_update.pyx":1893
 *         shape[0] = m + p
 *         shape[1] = n + p
 *         qnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)             # <<<<<<<<<<<<<<
 *         qnew[:-p,:-p] = q1
 *         for j in range(p):
 */
    __pyx_t_1 = PyArray_ZEROS(2, __pyx_v_shape, __pyx_v_typecode, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1893, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1893, __pyx_L1_error)
    __pyx_v_qnew = ((PyArrayObject *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "scipy/linalg/_decomp_update.pyx":1894
 *         shape[1] = n + p
 *         qnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 *         qnew[:-p,:-p] = q1             # <<<<<<<<<<<<<<
 *         for j in range(p):
 *             qnew[m+j, n+j] = 1
 */
    __pyx_t_1 = __Pyx_PyInt_From_int((-__pyx_v_p)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1894, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = PySlice_New(Py_None, __pyx_t_1, Py_None); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1894, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyInt_From_int((-__pyx_v_p)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1894, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = PySlice_New(Py_None, __pyx_t_1, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1894, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1894, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_6);
    __pyx_t_7 = 0;
    __pyx_t_6 = 0;
    if (unlikely(PyObject_SetItem(((PyObject *)__pyx_v_qnew), __pyx_t_1, ((PyObject *)__pyx_v_q1)) < 0)) __PYX_ERR(0, 1894, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "scipy/linalg/_decomp_update.pyx":1895
 *         qnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 *         qnew[:-p,:-p] = q1
 *         for j in range(p):             # <<<<<<<<<<<<<<
 *             qnew[m+j, n+j] = 1
 * 
 */
    __pyx_t_10 = __pyx_v_p;
    __pyx_t_9 = __pyx_t_10;
    for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_9; __pyx_t_8+=1) {
      __pyx_v_j = __pyx_t_8;

      /* "scipy/linalg/_decomp_update.pyx":1896
 *         qnew[:-p,:-p] = q1
 *         for j in range(p):
 *             qnew[m+j, n+j] = 1             # <<<<<<<<<<<<<<
 * 
 *         if p == 1:
 */
      __pyx_t_1 = __Pyx_PyInt_From_int((__pyx_v_m + __pyx_v_j)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1896, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = __Pyx_PyInt_From_int((__pyx_v_n + __pyx_v_j)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1896, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1896, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_6);
      __pyx_t_1 = 0;
      __pyx_t_6 = 0;
      if (unlikely(PyObject_SetItem(((PyObject *)__pyx_v_qnew), __pyx_t_7, __pyx_int_1) < 0)) __PYX_ERR(0, 1896, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }

    /* "scipy/linalg/_decomp_update.pyx":1898
 *             qnew[m+j, n+j] = 1
 * 
 *         if p == 1:             # <<<<<<<<<<<<<<
 *             qptr = extract(qnew, qs)
 *             rptr = extract(r1, rs)
 */
    __pyx_t_12 = ((__pyx_v_p == 1) != 0);
    if (__pyx_t_12) {

      /* "scipy/linalg/_decomp_update.pyx":1899
 * 
 *         if p == 1:
 *             qptr = extract(qnew, qs)             # <<<<<<<<<<<<<<
 *             rptr = extract(r1, rs)
 *             uptr = extract(u1, us)
 */
      __pyx_v_qptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_qnew, __pyx_v_qs);

      /* "scipy/linalg/_decomp_update.pyx":1900
 *         if p == 1:
 *             qptr = extract(qnew, qs)
 *             rptr = extract(r1, rs)             # <<<<<<<<<<<<<<
 *             uptr = extract(u1, us)
 *             with nogil:
 */
      __pyx_v_rptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_r1, __pyx_v_rs);

      /* "scipy/linalg/_decomp_update.pyx":1901
 *             qptr = extract(qnew, qs)
 *             rptr = extract(r1, rs)
 *             uptr = extract(u1, us)             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 */
      __pyx_v_uptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_u1, __pyx_v_us);

      /* "scipy/linalg/_decomp_update.pyx":1902
 *             rptr = extract(r1, rs)
 *             uptr = extract(u1, us)
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     thin_qr_row_insert(m+p, n, <float*>qptr, qs,
 */
      {
          #ifdef WITH_THREAD
          PyThreadState *_save;
          Py_UNBLOCK_THREADS
          __Pyx_FastGIL_Remember();
          #endif
          /*try:*/ {

            /* "scipy/linalg/_decomp_update.pyx":1903
 *             uptr = extract(u1, us)
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     thin_qr_row_insert(m+p, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, <float*>uptr, us, k)
 */
            switch (__pyx_v_typecode) {
              case NPY_FLOAT:

              /* "scipy/linalg/_decomp_update.pyx":1904
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 *                     thin_qr_row_insert(m+p, n, <float*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float*>rptr, rs, <float*>uptr, us, k)
 *                 elif typecode == cnp.NPY_DOUBLE:
 */
              __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_insert((__pyx_v_m + __pyx_v_p), __pyx_v_n, ((float *)__pyx_v_qptr), __pyx_v_qs, ((float *)__pyx_v_rptr), __pyx_v_rs, ((float *)__pyx_v_uptr), __pyx_v_us, __pyx_v_k);

              /* "scipy/linalg/_decomp_update.pyx":1903
 *             uptr = extract(u1, us)
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     thin_qr_row_insert(m+p, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, <float*>uptr, us, k)
 */
              break;
              case NPY_DOUBLE:

              /* "scipy/linalg/_decomp_update.pyx":1907
 *                         <float*>rptr, rs, <float*>uptr, us, k)
 *                 elif typecode == cnp.NPY_DOUBLE:
 *                     thin_qr_row_insert(m+p, n, <double*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double*>rptr, rs, <double*>uptr, us, k)
 *                 elif typecode == cnp.NPY_CFLOAT:
 */
              __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_insert((__pyx_v_m + __pyx_v_p), __pyx_v_n, ((double *)__pyx_v_qptr), __pyx_v_qs, ((double *)__pyx_v_rptr), __pyx_v_rs, ((double *)__pyx_v_uptr), __pyx_v_us, __pyx_v_k);

              /* "scipy/linalg/_decomp_update.pyx":1906
 *                     thin_qr_row_insert(m+p, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, <float*>uptr, us, k)
 *                 elif typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                     thin_qr_row_insert(m+p, n, <double*>qptr, qs,
 *                         <double*>rptr, rs, <double*>uptr, us, k)
 */
              break;
              case NPY_CFLOAT:

              /* "scipy/linalg/_decomp_update.pyx":1910
 *                         <double*>rptr, rs, <double*>uptr, us, k)
 *                 elif typecode == cnp.NPY_CFLOAT:
 *                     thin_qr_row_insert(m+p, n, <float_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float_complex*>rptr, rs, <float_complex*>uptr, us, k)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 */
              __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_insert((__pyx_v_m + __pyx_v_p), __pyx_v_n, ((__pyx_t_float_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_float_complex *)__pyx_v_rptr), __pyx_v_rs, ((__pyx_t_float_complex *)__pyx_v_uptr), __pyx_v_us, __pyx_v_k);

              /* "scipy/linalg/_decomp_update.pyx":1909
 *                     thin_qr_row_insert(m+p, n, <double*>qptr, qs,
 *                         <double*>rptr, rs, <double*>uptr, us, k)
 *                 elif typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                     thin_qr_row_insert(m+p, n, <float_complex*>qptr, qs,
 *                         <float_complex*>rptr, rs, <float_complex*>uptr, us, k)
 */
              break;
              default:

              /* "scipy/linalg/_decomp_update.pyx":1913
 *                         <float_complex*>rptr, rs, <float_complex*>uptr, us, k)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 *                     thin_qr_row_insert(m+p, n, <double_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double_complex*>rptr, rs, <double_complex*>uptr, us, k)
 *         else:
 */
              __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_row_insert((__pyx_v_m + __pyx_v_p), __pyx_v_n, ((__pyx_t_double_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_double_complex *)__pyx_v_rptr), __pyx_v_rs, ((__pyx_t_double_complex *)__pyx_v_uptr), __pyx_v_us, __pyx_v_k);
              break;
            }
          }

          /* "scipy/linalg/_decomp_update.pyx":1902
 *             rptr = extract(r1, rs)
 *             uptr = extract(u1, us)
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     thin_qr_row_insert(m+p, n, <float*>qptr, qs,
 */
          /*finally:*/ {
            /*normal exit:*/{
              #ifdef WITH_THREAD
              __Pyx_FastGIL_Forget();
              Py_BLOCK_THREADS
              #endif
              goto __pyx_L15;
            }
            __pyx_L15:;
          }
      }

      /* "scipy/linalg/_decomp_update.pyx":1898
 *             qnew[m+j, n+j] = 1
 * 
 *         if p == 1:             # <<<<<<<<<<<<<<
 *             qptr = extract(qnew, qs)
 *             rptr = extract(r1, rs)
 */
      goto __pyx_L12;
    }

    /* "scipy/linalg/_decomp_update.pyx":1917
 *         else:
 *             # only copies if necessary.
 *             r1 = PyArray_FromArray(r1, NULL, cnp.NPY_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *             u1 = PyArray_FromArray(u1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             qptr = extract(qnew, qs)
 */
    /*else*/ {
      __pyx_t_7 = ((PyObject *)PyArray_FromArray(__pyx_v_r1, NULL, NPY_F_CONTIGUOUS)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1917, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF_SET(__pyx_v_r1, ((PyArrayObject *)__pyx_t_7));
      __pyx_t_7 = 0;

      /* "scipy/linalg/_decomp_update.pyx":1918
 *             # only copies if necessary.
 *             r1 = PyArray_FromArray(r1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             u1 = PyArray_FromArray(u1, NULL, cnp.NPY_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *             qptr = extract(qnew, qs)
 *             rptr = extract(r1, rs)
 */
      __pyx_t_7 = ((PyObject *)PyArray_FromArray(__pyx_v_u1, NULL, NPY_F_CONTIGUOUS)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1918, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF_SET(__pyx_v_u1, ((PyArrayObject *)__pyx_t_7));
      __pyx_t_7 = 0;

      /* "scipy/linalg/_decomp_update.pyx":1919
 *             r1 = PyArray_FromArray(r1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             u1 = PyArray_FromArray(u1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             qptr = extract(qnew, qs)             # <<<<<<<<<<<<<<
 *             rptr = extract(r1, rs)
 *             uptr = extract(u1, us)
 */
      __pyx_v_qptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_qnew, __pyx_v_qs);

      /* "scipy/linalg/_decomp_update.pyx":1920
 *             u1 = PyArray_FromArray(u1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             qptr = extract(qnew, qs)
 *             rptr = extract(r1, rs)             # <<<<<<<<<<<<<<
 *             uptr = extract(u1, us)
 *             with nogil:
 */
      __pyx_v_rptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_r1, __pyx_v_rs);

      /* "scipy/linalg/_decomp_update.pyx":1921
 *             qptr = extract(qnew, qs)
 *             rptr = extract(r1, rs)
 *             uptr = extract(u1, us)             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 */
      __pyx_v_uptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_u1, __pyx_v_us);

      /* "scipy/linalg/_decomp_update.pyx":1922
 *             rptr = extract(r1, rs)
 *             uptr = extract(u1, us)
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     thin_qr_block_row_insert(m+p, n, <float*>qptr, qs,
 */
      {
          #ifdef WITH_THREAD
          PyThreadState *_save;
          Py_UNBLOCK_THREADS
          __Pyx_FastGIL_Remember();
          #endif
          /*try:*/ {

            /* "scipy/linalg/_decomp_update.pyx":1923
 *             uptr = extract(u1, us)
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     thin_qr_block_row_insert(m+p, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, <float*>uptr, us, k, p)
 */
            switch (__pyx_v_typecode) {
              case NPY_FLOAT:

              /* "scipy/linalg/_decomp_update.pyx":1924
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 *                     thin_qr_block_row_insert(m+p, n, <float*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float*>rptr, rs, <float*>uptr, us, k, p)
 *                 elif typecode == cnp.NPY_DOUBLE:
 */
              (void)(__pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_block_row_insert((__pyx_v_m + __pyx_v_p), __pyx_v_n, ((float *)__pyx_v_qptr), __pyx_v_qs, ((float *)__pyx_v_rptr), __pyx_v_rs, ((float *)__pyx_v_uptr), __pyx_v_us, __pyx_v_k, __pyx_v_p));

              /* "scipy/linalg/_decomp_update.pyx":1923
 *             uptr = extract(u1, us)
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     thin_qr_block_row_insert(m+p, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, <float*>uptr, us, k, p)
 */
              break;
              case NPY_DOUBLE:

              /* "scipy/linalg/_decomp_update.pyx":1927
 *                         <float*>rptr, rs, <float*>uptr, us, k, p)
 *                 elif typecode == cnp.NPY_DOUBLE:
 *                     thin_qr_block_row_insert(m+p, n, <double*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double*>rptr, rs, <double*>uptr, us, k, p)
 *                 elif typecode == cnp.NPY_CFLOAT:
 */
              (void)(__pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_block_row_insert((__pyx_v_m + __pyx_v_p), __pyx_v_n, ((double *)__pyx_v_qptr), __pyx_v_qs, ((double *)__pyx_v_rptr), __pyx_v_rs, ((double *)__pyx_v_uptr), __pyx_v_us, __pyx_v_k, __pyx_v_p));

              /* "scipy/linalg/_decomp_update.pyx":1926
 *                     thin_qr_block_row_insert(m+p, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, <float*>uptr, us, k, p)
 *                 elif typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                     thin_qr_block_row_insert(m+p, n, <double*>qptr, qs,
 *                         <double*>rptr, rs, <double*>uptr, us, k, p)
 */
              break;
              case NPY_CFLOAT:

              /* "scipy/linalg/_decomp_update.pyx":1930
 *                         <double*>rptr, rs, <double*>uptr, us, k, p)
 *                 elif typecode == cnp.NPY_CFLOAT:
 *                     thin_qr_block_row_insert(m+p, n, <float_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float_complex*>rptr, rs, <float_complex*>uptr, us, k, p)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 */
              (void)(__pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_block_row_insert((__pyx_v_m + __pyx_v_p), __pyx_v_n, ((__pyx_t_float_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_float_complex *)__pyx_v_rptr), __pyx_v_rs, ((__pyx_t_float_complex *)__pyx_v_uptr), __pyx_v_us, __pyx_v_k, __pyx_v_p));

              /* "scipy/linalg/_decomp_update.pyx":1929
 *                     thin_qr_block_row_insert(m+p, n, <double*>qptr, qs,
 *                         <double*>rptr, rs, <double*>uptr, us, k, p)
 *                 elif typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                     thin_qr_block_row_insert(m+p, n, <float_complex*>qptr, qs,
 *                         <float_complex*>rptr, rs, <float_complex*>uptr, us, k, p)
 */
              break;
              default:

              /* "scipy/linalg/_decomp_update.pyx":1933
 *                         <float_complex*>rptr, rs, <float_complex*>uptr, us, k, p)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 *                     thin_qr_block_row_insert(m+p, n, <double_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double_complex*>rptr, rs, <double_complex*>uptr, us, k, p)
 *         return qnew[:, :-p], r1
 */
              (void)(__pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_block_row_insert((__pyx_v_m + __pyx_v_p), __pyx_v_n, ((__pyx_t_double_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_double_complex *)__pyx_v_rptr), __pyx_v_rs, ((__pyx_t_double_complex *)__pyx_v_uptr), __pyx_v_us, __pyx_v_k, __pyx_v_p));
              break;
            }
          }

          /* "scipy/linalg/_decomp_update.pyx":1922
 *             rptr = extract(r1, rs)
 *             uptr = extract(u1, us)
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     thin_qr_block_row_insert(m+p, n, <float*>qptr, qs,
 */
          /*finally:*/ {
            /*normal exit:*/{
              #ifdef WITH_THREAD
              __Pyx_FastGIL_Forget();
              Py_BLOCK_THREADS
              #endif
              goto __pyx_L18;
            }
            __pyx_L18:;
          }
      }
    }
    __pyx_L12:;

    /* "scipy/linalg/_decomp_update.pyx":1935
 *                     thin_qr_block_row_insert(m+p, n, <double_complex*>qptr, qs,
 *                         <double_complex*>rptr, rs, <double_complex*>uptr, us, k, p)
 *         return qnew[:, :-p], r1             # <<<<<<<<<<<<<<
 *     else:
 *         shape[0] = m + p
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_7 = __Pyx_PyInt_From_int((-__pyx_v_p)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1935, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = PySlice_New(Py_None, __pyx_t_7, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1935, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1935, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_INCREF(__pyx_slice__14);
    __Pyx_GIVEREF(__pyx_slice__14);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_slice__14);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_qnew), __pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1935, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1935, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6);
    __Pyx_INCREF(((PyObject *)__pyx_v_r1));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_r1));
    PyTuple_SET_ITEM(__pyx_t_7, 1, ((PyObject *)__pyx_v_r1));
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_7;
    __pyx_t_7 = 0;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":1884
 *     u1 = validate_array(u1, check_finite)
 * 
 *     if economic:             # <<<<<<<<<<<<<<
 *         if not overwrite_qru:
 *             r1 = PyArray_FromArray(r1, NULL,
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":1937
 *         return qnew[:, :-p], r1
 *     else:
 *         shape[0] = m + p             # <<<<<<<<<<<<<<
 *         shape[1] = m + p
 *         qnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 */
  /*else*/ {
    (__pyx_v_shape[0]) = (__pyx_v_m + __pyx_v_p);

    /* "scipy/linalg/_decomp_update.pyx":1938
 *     else:
 *         shape[0] = m + p
 *         shape[1] = m + p             # <<<<<<<<<<<<<<
 *         qnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 *         shape[1] = n
 */
    (__pyx_v_shape[1]) = (__pyx_v_m + __pyx_v_p);

    /* "scipy/linalg/_decomp_update.pyx":1939
 *         shape[0] = m + p
 *         shape[1] = m + p
 *         qnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)             # <<<<<<<<<<<<<<
 *         shape[1] = n
 *         rnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 */
    __pyx_t_7 = PyArray_ZEROS(2, __pyx_v_shape, __pyx_v_typecode, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1939, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (!(likely(((__pyx_t_7) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_7, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1939, __pyx_L1_error)
    __pyx_v_qnew = ((PyArrayObject *)__pyx_t_7);
    __pyx_t_7 = 0;

    /* "scipy/linalg/_decomp_update.pyx":1940
 *         shape[1] = m + p
 *         qnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 *         shape[1] = n             # <<<<<<<<<<<<<<
 *         rnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 * 
 */
    (__pyx_v_shape[1]) = __pyx_v_n;

    /* "scipy/linalg/_decomp_update.pyx":1941
 *         qnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 *         shape[1] = n
 *         rnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)             # <<<<<<<<<<<<<<
 * 
 *         rnew[:m,:] = r1
 */
    __pyx_t_7 = PyArray_ZEROS(2, __pyx_v_shape, __pyx_v_typecode, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1941, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (!(likely(((__pyx_t_7) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_7, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 1941, __pyx_L1_error)
    __pyx_v_rnew = ((PyArrayObject *)__pyx_t_7);
    __pyx_t_7 = 0;

    /* "scipy/linalg/_decomp_update.pyx":1943
 *         rnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 * 
 *         rnew[:m,:] = r1             # <<<<<<<<<<<<<<
 *         rnew[m:,:] = u1
 *         qnew[:-p,:-p] = q1;
 */
    __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_m); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1943, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = PySlice_New(Py_None, __pyx_t_7, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1943, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1943, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6);
    __Pyx_INCREF(__pyx_slice__14);
    __Pyx_GIVEREF(__pyx_slice__14);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_slice__14);
    __pyx_t_6 = 0;
    if (unlikely(PyObject_SetItem(((PyObject *)__pyx_v_rnew), __pyx_t_7, ((PyObject *)__pyx_v_r1)) < 0)) __PYX_ERR(0, 1943, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "scipy/linalg/_decomp_update.pyx":1944
 * 
 *         rnew[:m,:] = r1
 *         rnew[m:,:] = u1             # <<<<<<<<<<<<<<
 *         qnew[:-p,:-p] = q1;
 *         for j in range(m, m+p):
 */
    __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_m); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1944, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = PySlice_New(__pyx_t_7, Py_None, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1944, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1944, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6);
    __Pyx_INCREF(__pyx_slice__14);
    __Pyx_GIVEREF(__pyx_slice__14);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_slice__14);
    __pyx_t_6 = 0;
    if (unlikely(PyObject_SetItem(((PyObject *)__pyx_v_rnew), __pyx_t_7, ((PyObject *)__pyx_v_u1)) < 0)) __PYX_ERR(0, 1944, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "scipy/linalg/_decomp_update.pyx":1945
 *         rnew[:m,:] = r1
 *         rnew[m:,:] = u1
 *         qnew[:-p,:-p] = q1;             # <<<<<<<<<<<<<<
 *         for j in range(m, m+p):
 *             qnew[j, j] = 1
 */
    __pyx_t_7 = __Pyx_PyInt_From_int((-__pyx_v_p)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1945, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = PySlice_New(Py_None, __pyx_t_7, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1945, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyInt_From_int((-__pyx_v_p)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1945, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_1 = PySlice_New(Py_None, __pyx_t_7, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1945, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1945, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_1);
    __pyx_t_6 = 0;
    __pyx_t_1 = 0;
    if (unlikely(PyObject_SetItem(((PyObject *)__pyx_v_qnew), __pyx_t_7, ((PyObject *)__pyx_v_q1)) < 0)) __PYX_ERR(0, 1945, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "scipy/linalg/_decomp_update.pyx":1946
 *         rnew[m:,:] = u1
 *         qnew[:-p,:-p] = q1;
 *         for j in range(m, m+p):             # <<<<<<<<<<<<<<
 *             qnew[j, j] = 1
 * 
 */
    __pyx_t_10 = (__pyx_v_m + __pyx_v_p);
    __pyx_t_9 = __pyx_t_10;
    for (__pyx_t_8 = __pyx_v_m; __pyx_t_8 < __pyx_t_9; __pyx_t_8+=1) {
      __pyx_v_j = __pyx_t_8;

      /* "scipy/linalg/_decomp_update.pyx":1947
 *         qnew[:-p,:-p] = q1;
 *         for j in range(m, m+p):
 *             qnew[j, j] = 1             # <<<<<<<<<<<<<<
 * 
 *         qptr = extract(qnew, qs)
 */
      __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_j); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1947, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_j); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1947, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1947, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_7);
      PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
      __pyx_t_7 = 0;
      __pyx_t_1 = 0;
      if (unlikely(PyObject_SetItem(((PyObject *)__pyx_v_qnew), __pyx_t_6, __pyx_int_1) < 0)) __PYX_ERR(0, 1947, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }

    /* "scipy/linalg/_decomp_update.pyx":1949
 *             qnew[j, j] = 1
 * 
 *         qptr = extract(qnew, qs)             # <<<<<<<<<<<<<<
 *         rptr = extract(rnew, rs)
 *         if p == 1:
 */
    __pyx_v_qptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_qnew, __pyx_v_qs);

    /* "scipy/linalg/_decomp_update.pyx":1950
 * 
 *         qptr = extract(qnew, qs)
 *         rptr = extract(rnew, rs)             # <<<<<<<<<<<<<<
 *         if p == 1:
 *             with nogil:
 */
    __pyx_v_rptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_rnew, __pyx_v_rs);

    /* "scipy/linalg/_decomp_update.pyx":1951
 *         qptr = extract(qnew, qs)
 *         rptr = extract(rnew, rs)
 *         if p == 1:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 */
    __pyx_t_12 = ((__pyx_v_p == 1) != 0);
    if (__pyx_t_12) {

      /* "scipy/linalg/_decomp_update.pyx":1952
 *         rptr = extract(rnew, rs)
 *         if p == 1:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     qr_row_insert(m+p, n, <float*>qptr, qs,
 */
      {
          #ifdef WITH_THREAD
          PyThreadState *_save;
          Py_UNBLOCK_THREADS
          __Pyx_FastGIL_Remember();
          #endif
          /*try:*/ {

            /* "scipy/linalg/_decomp_update.pyx":1953
 *         if p == 1:
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     qr_row_insert(m+p, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, k)
 */
            switch (__pyx_v_typecode) {
              case NPY_FLOAT:

              /* "scipy/linalg/_decomp_update.pyx":1954
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 *                     qr_row_insert(m+p, n, <float*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float*>rptr, rs, k)
 *                 elif typecode == cnp.NPY_DOUBLE:
 */
              __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_row_insert((__pyx_v_m + __pyx_v_p), __pyx_v_n, ((float *)__pyx_v_qptr), __pyx_v_qs, ((float *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k);

              /* "scipy/linalg/_decomp_update.pyx":1953
 *         if p == 1:
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     qr_row_insert(m+p, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, k)
 */
              break;
              case NPY_DOUBLE:

              /* "scipy/linalg/_decomp_update.pyx":1957
 *                         <float*>rptr, rs, k)
 *                 elif typecode == cnp.NPY_DOUBLE:
 *                     qr_row_insert(m+p, n, <double*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double*>rptr, rs, k)
 *                 elif typecode == cnp.NPY_CFLOAT:
 */
              __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_row_insert((__pyx_v_m + __pyx_v_p), __pyx_v_n, ((double *)__pyx_v_qptr), __pyx_v_qs, ((double *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k);

              /* "scipy/linalg/_decomp_update.pyx":1956
 *                     qr_row_insert(m+p, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, k)
 *                 elif typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                     qr_row_insert(m+p, n, <double*>qptr, qs,
 *                         <double*>rptr, rs, k)
 */
              break;
              case NPY_CFLOAT:

              /* "scipy/linalg/_decomp_update.pyx":1960
 *                         <double*>rptr, rs, k)
 *                 elif typecode == cnp.NPY_CFLOAT:
 *                     qr_row_insert(m+p, n, <float_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float_complex*>rptr, rs, k)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 */
              __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_row_insert((__pyx_v_m + __pyx_v_p), __pyx_v_n, ((__pyx_t_float_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_float_complex *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k);

              /* "scipy/linalg/_decomp_update.pyx":1959
 *                     qr_row_insert(m+p, n, <double*>qptr, qs,
 *                         <double*>rptr, rs, k)
 *                 elif typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                     qr_row_insert(m+p, n, <float_complex*>qptr, qs,
 *                         <float_complex*>rptr, rs, k)
 */
              break;
              default:

              /* "scipy/linalg/_decomp_update.pyx":1963
 *                         <float_complex*>rptr, rs, k)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 *                     qr_row_insert(m+p, n, <double_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double_complex*>rptr, rs, k)
 *         else:
 */
              __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_row_insert((__pyx_v_m + __pyx_v_p), __pyx_v_n, ((__pyx_t_double_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_double_complex *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k);
              break;
            }
          }

          /* "scipy/linalg/_decomp_update.pyx":1952
 *         rptr = extract(rnew, rs)
 *         if p == 1:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     qr_row_insert(m+p, n, <float*>qptr, qs,
 */
          /*finally:*/ {
            /*normal exit:*/{
              #ifdef WITH_THREAD
              __Pyx_FastGIL_Forget();
              Py_BLOCK_THREADS
              #endif
              goto __pyx_L24;
            }
            __pyx_L24:;
          }
      }

      /* "scipy/linalg/_decomp_update.pyx":1951
 *         qptr = extract(qnew, qs)
 *         rptr = extract(rnew, rs)
 *         if p == 1:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 */
      goto __pyx_L21;
    }

    /* "scipy/linalg/_decomp_update.pyx":1966
 *                         <double_complex*>rptr, rs, k)
 *         else:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     info = qr_block_row_insert(m+p, n, <float*>qptr, qs,
 */
    /*else*/ {
      {
          #ifdef WITH_THREAD
          PyThreadState *_save;
          Py_UNBLOCK_THREADS
          __Pyx_FastGIL_Remember();
          #endif
          /*try:*/ {

            /* "scipy/linalg/_decomp_update.pyx":1967
 *         else:
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     info = qr_block_row_insert(m+p, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, k, p)
 */
            switch (__pyx_v_typecode) {
              case NPY_FLOAT:

              /* "scipy/linalg/_decomp_update.pyx":1968
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 *                     info = qr_block_row_insert(m+p, n, <float*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float*>rptr, rs, k, p)
 *                 elif typecode == cnp.NPY_DOUBLE:
 */
              __pyx_v_info = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_insert((__pyx_v_m + __pyx_v_p), __pyx_v_n, ((float *)__pyx_v_qptr), __pyx_v_qs, ((float *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k, __pyx_v_p);

              /* "scipy/linalg/_decomp_update.pyx":1967
 *         else:
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     info = qr_block_row_insert(m+p, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, k, p)
 */
              break;
              case NPY_DOUBLE:

              /* "scipy/linalg/_decomp_update.pyx":1971
 *                         <float*>rptr, rs, k, p)
 *                 elif typecode == cnp.NPY_DOUBLE:
 *                     info = qr_block_row_insert(m+p, n, <double*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double*>rptr, rs, k, p)
 *                 elif typecode == cnp.NPY_CFLOAT:
 */
              __pyx_v_info = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_insert((__pyx_v_m + __pyx_v_p), __pyx_v_n, ((double *)__pyx_v_qptr), __pyx_v_qs, ((double *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k, __pyx_v_p);

              /* "scipy/linalg/_decomp_update.pyx":1970
 *                     info = qr_block_row_insert(m+p, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, k, p)
 *                 elif typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                     info = qr_block_row_insert(m+p, n, <double*>qptr, qs,
 *                         <double*>rptr, rs, k, p)
 */
              break;
              case NPY_CFLOAT:

              /* "scipy/linalg/_decomp_update.pyx":1974
 *                         <double*>rptr, rs, k, p)
 *                 elif typecode == cnp.NPY_CFLOAT:
 *                     info = qr_block_row_insert(m+p, n, <float_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float_complex*>rptr, rs, k, p)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 */
              __pyx_v_info = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_insert((__pyx_v_m + __pyx_v_p), __pyx_v_n, ((__pyx_t_float_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_float_complex *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k, __pyx_v_p);

              /* "scipy/linalg/_decomp_update.pyx":1973
 *                     info = qr_block_row_insert(m+p, n, <double*>qptr, qs,
 *                         <double*>rptr, rs, k, p)
 *                 elif typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                     info = qr_block_row_insert(m+p, n, <float_complex*>qptr, qs,
 *                         <float_complex*>rptr, rs, k, p)
 */
              break;
              default:

              /* "scipy/linalg/_decomp_update.pyx":1977
 *                         <float_complex*>rptr, rs, k, p)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 *                     info = qr_block_row_insert(m+p, n, <double_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double_complex*>rptr, rs, k, p)
 *             if info == MEMORY_ERROR:
 */
              __pyx_v_info = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_row_insert((__pyx_v_m + __pyx_v_p), __pyx_v_n, ((__pyx_t_double_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_double_complex *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k, __pyx_v_p);
              break;
            }
          }

          /* "scipy/linalg/_decomp_update.pyx":1966
 *                         <double_complex*>rptr, rs, k)
 *         else:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     info = qr_block_row_insert(m+p, n, <float*>qptr, qs,
 */
          /*finally:*/ {
            /*normal exit:*/{
              #ifdef WITH_THREAD
              __Pyx_FastGIL_Forget();
              Py_BLOCK_THREADS
              #endif
              goto __pyx_L27;
            }
            __pyx_L27:;
          }
      }

      /* "scipy/linalg/_decomp_update.pyx":1979
 *                     info = qr_block_row_insert(m+p, n, <double_complex*>qptr, qs,
 *                         <double_complex*>rptr, rs, k, p)
 *             if info == MEMORY_ERROR:             # <<<<<<<<<<<<<<
 *                 raise MemoryError('Unable to allocate memory for array')
 *         return qnew, rnew
 */
      __pyx_t_12 = ((__pyx_v_info == __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR) != 0);
      if (unlikely(__pyx_t_12)) {

        /* "scipy/linalg/_decomp_update.pyx":1980
 *                         <double_complex*>rptr, rs, k, p)
 *             if info == MEMORY_ERROR:
 *                 raise MemoryError('Unable to allocate memory for array')             # <<<<<<<<<<<<<<
 *         return qnew, rnew
 * 
 */
        __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1980, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_Raise(__pyx_t_6, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __PYX_ERR(0, 1980, __pyx_L1_error)

        /* "scipy/linalg/_decomp_update.pyx":1979
 *                     info = qr_block_row_insert(m+p, n, <double_complex*>qptr, qs,
 *                         <double_complex*>rptr, rs, k, p)
 *             if info == MEMORY_ERROR:             # <<<<<<<<<<<<<<
 *                 raise MemoryError('Unable to allocate memory for array')
 *         return qnew, rnew
 */
      }
    }
    __pyx_L21:;

    /* "scipy/linalg/_decomp_update.pyx":1981
 *             if info == MEMORY_ERROR:
 *                 raise MemoryError('Unable to allocate memory for array')
 *         return qnew, rnew             # <<<<<<<<<<<<<<
 * 
 * cdef qr_insert_col(Q, R, u, int k, rcond, bint overwrite_qru, bint check_finite):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1981, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)__pyx_v_qnew));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_qnew));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)__pyx_v_qnew));
    __Pyx_INCREF(((PyObject *)__pyx_v_rnew));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_rnew));
    PyTuple_SET_ITEM(__pyx_t_6, 1, ((PyObject *)__pyx_v_rnew));
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;
  }

  /* "scipy/linalg/_decomp_update.pyx":1835
 *         raise ValueError("'which' must be either 'row' or 'col'")
 * 
 * cdef qr_insert_row(Q, R, u, int k, bint overwrite_qru, bint check_finite):             # <<<<<<<<<<<<<<
 *     cdef cnp.ndarray q1, r1, u1, qnew, rnew
 *     cdef int j
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("scipy.linalg._decomp_update.qr_insert_row", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_q1);
  __Pyx_XDECREF((PyObject *)__pyx_v_r1);
  __Pyx_XDECREF((PyObject *)__pyx_v_u1);
  __Pyx_XDECREF((PyObject *)__pyx_v_qnew);
  __Pyx_XDECREF((PyObject *)__pyx_v_rnew);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":1983
 *         return qnew, rnew
 * 
 * cdef qr_insert_col(Q, R, u, int k, rcond, bint overwrite_qru, bint check_finite):             # <<<<<<<<<<<<<<
 *     cdef cnp.ndarray q1, r1, u1, qnew, rnew
 *     cdef int j
 */

static PyObject *__pyx_f_5scipy_6linalg_14_decomp_update_qr_insert_col(PyObject *__pyx_v_Q, PyObject *__pyx_v_R, PyObject *__pyx_v_u, int __pyx_v_k, PyObject *__pyx_v_rcond, int __pyx_v_overwrite_qru, int __pyx_v_check_finite) {
  PyArrayObject *__pyx_v_q1 = 0;
  PyArrayObject *__pyx_v_r1 = 0;
  PyArrayObject *__pyx_v_u1 = 0;
  PyArrayObject *__pyx_v_qnew = 0;
  PyArrayObject *__pyx_v_rnew = 0;
  CYTHON_UNUSED int __pyx_v_q_flags;
  int __pyx_v_u_flags;
  int __pyx_v_typecode;
  int __pyx_v_m;
  int __pyx_v_n;
  int __pyx_v_p;
  int __pyx_v_info;
  int __pyx_v_p_eco;
  int __pyx_v_p_full;
  void *__pyx_v_qptr;
  void *__pyx_v_rptr;
  void *__pyx_v_uptr;
  int __pyx_v_qs[2];
  int __pyx_v_rs[2];
  int __pyx_v_us[2];
  npy_intp __pyx_v_shape[2];
  int __pyx_v_economic;
  float __pyx_v_frc;
  double __pyx_v_drc;
  __pyx_t_float_complex __pyx_v_cfrc;
  __pyx_t_double_complex __pyx_v_cdrc;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  double __pyx_t_14;
  float __pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  __Pyx_RefNannySetupContext("qr_insert_col", 0);

  /* "scipy/linalg/_decomp_update.pyx":1986
 *     cdef cnp.ndarray q1, r1, u1, qnew, rnew
 *     cdef int j
 *     cdef int q_flags = ARRAY_ANYORDER             # <<<<<<<<<<<<<<
 *     cdef int u_flags = cnp.NPY_BEHAVED_NS | cnp.NPY_ELEMENTSTRIDES
 *     cdef int typecode, m, n, p, info, p_eco, p_full
 */
  __pyx_v_q_flags = __pyx_v_5scipy_6linalg_14_decomp_update_ARRAY_ANYORDER;

  /* "scipy/linalg/_decomp_update.pyx":1987
 *     cdef int j
 *     cdef int q_flags = ARRAY_ANYORDER
 *     cdef int u_flags = cnp.NPY_BEHAVED_NS | cnp.NPY_ELEMENTSTRIDES             # <<<<<<<<<<<<<<
 *     cdef int typecode, m, n, p, info, p_eco, p_full
 *     cdef void* qptr
 */
  __pyx_v_u_flags = (NPY_BEHAVED_NS | NPY_ELEMENTSTRIDES);

  /* "scipy/linalg/_decomp_update.pyx":1997
 *     cdef cnp.npy_intp shape[2]
 *     cdef bint economic
 *     cdef float frc = libc.float.FLT_EPSILON             # <<<<<<<<<<<<<<
 *     cdef double drc = libc.float.DBL_EPSILON
 *     cdef float_complex cfrc = frc
 */
  __pyx_v_frc = FLT_EPSILON;

  /* "scipy/linalg/_decomp_update.pyx":1998
 *     cdef bint economic
 *     cdef float frc = libc.float.FLT_EPSILON
 *     cdef double drc = libc.float.DBL_EPSILON             # <<<<<<<<<<<<<<
 *     cdef float_complex cfrc = frc
 *     cdef double_complex cdrc = drc
 */
  __pyx_v_drc = DBL_EPSILON;

  /* "scipy/linalg/_decomp_update.pyx":1999
 *     cdef float frc = libc.float.FLT_EPSILON
 *     cdef double drc = libc.float.DBL_EPSILON
 *     cdef float_complex cfrc = frc             # <<<<<<<<<<<<<<
 *     cdef double_complex cdrc = drc
 * 
 */
  __pyx_v_cfrc = __pyx_t_float_complex_from_parts(__pyx_v_frc, 0);

  /* "scipy/linalg/_decomp_update.pyx":2000
 *     cdef double drc = libc.float.DBL_EPSILON
 *     cdef float_complex cfrc = frc
 *     cdef double_complex cdrc = drc             # <<<<<<<<<<<<<<
 * 
 *     # 1 eco q alloc, r alloc, u any
 */
  __pyx_v_cdrc = __pyx_t_double_complex_from_parts(__pyx_v_drc, 0);

  /* "scipy/linalg/_decomp_update.pyx":2007
 *     # p full, q F, r alloc, u any (handled in form_qTu)
 * 
 *     q1, r1, typecode, m, n, economic = validate_qr(Q, R, True, ARRAY_ANYORDER,             # <<<<<<<<<<<<<<
 *             True, ARRAY_ANYORDER, check_finite)
 *     if not overwrite_qru:
 */
  __pyx_t_1 = __pyx_f_5scipy_6linalg_14_decomp_update_validate_qr(__pyx_v_Q, __pyx_v_R, 1, __pyx_v_5scipy_6linalg_14_decomp_update_ARRAY_ANYORDER, 1, __pyx_v_5scipy_6linalg_14_decomp_update_ARRAY_ANYORDER, __pyx_v_check_finite); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2007, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(__pyx_t_1 != Py_None)) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 6)) {
      if (size > 6) __Pyx_RaiseTooManyValuesError(6);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 2007, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
    __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
    __pyx_t_4 = PyTuple_GET_ITEM(sequence, 2); 
    __pyx_t_5 = PyTuple_GET_ITEM(sequence, 3); 
    __pyx_t_6 = PyTuple_GET_ITEM(sequence, 4); 
    __pyx_t_7 = PyTuple_GET_ITEM(sequence, 5); 
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_t_7);
    #else
    {
      Py_ssize_t i;
      PyObject** temps[6] = {&__pyx_t_2,&__pyx_t_3,&__pyx_t_4,&__pyx_t_5,&__pyx_t_6,&__pyx_t_7};
      for (i=0; i < 6; i++) {
        PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 2007, __pyx_L1_error)
        __Pyx_GOTREF(item);
        *(temps[i]) = item;
      }
    }
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(0, 2007, __pyx_L1_error)
  }
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2007, __pyx_L1_error)
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2007, __pyx_L1_error)
  __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2007, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_9 = __Pyx_PyInt_As_int(__pyx_t_5); if (unlikely((__pyx_t_9 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2007, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_10 = __Pyx_PyInt_As_int(__pyx_t_6); if (unlikely((__pyx_t_10 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2007, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2007, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_q1 = ((PyArrayObject *)__pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_v_r1 = ((PyArrayObject *)__pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_v_typecode = __pyx_t_8;
  __pyx_v_m = __pyx_t_9;
  __pyx_v_n = __pyx_t_10;
  __pyx_v_economic = __pyx_t_11;

  /* "scipy/linalg/_decomp_update.pyx":2009
 *     q1, r1, typecode, m, n, economic = validate_qr(Q, R, True, ARRAY_ANYORDER,
 *             True, ARRAY_ANYORDER, check_finite)
 *     if not overwrite_qru:             # <<<<<<<<<<<<<<
 *         u_flags |= cnp.NPY_ENSURECOPY | cnp.NPY_F_CONTIGUOUS
 *     u1 = PyArray_CheckFromAny(u, NULL, 0, 0, u_flags, NULL)
 */
  __pyx_t_11 = ((!(__pyx_v_overwrite_qru != 0)) != 0);
  if (__pyx_t_11) {

    /* "scipy/linalg/_decomp_update.pyx":2010
 *             True, ARRAY_ANYORDER, check_finite)
 *     if not overwrite_qru:
 *         u_flags |= cnp.NPY_ENSURECOPY | cnp.NPY_F_CONTIGUOUS             # <<<<<<<<<<<<<<
 *     u1 = PyArray_CheckFromAny(u, NULL, 0, 0, u_flags, NULL)
 * 
 */
    __pyx_v_u_flags = (__pyx_v_u_flags | (NPY_ENSURECOPY | NPY_F_CONTIGUOUS));

    /* "scipy/linalg/_decomp_update.pyx":2009
 *     q1, r1, typecode, m, n, economic = validate_qr(Q, R, True, ARRAY_ANYORDER,
 *             True, ARRAY_ANYORDER, check_finite)
 *     if not overwrite_qru:             # <<<<<<<<<<<<<<
 *         u_flags |= cnp.NPY_ENSURECOPY | cnp.NPY_F_CONTIGUOUS
 *     u1 = PyArray_CheckFromAny(u, NULL, 0, 0, u_flags, NULL)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":2011
 *     if not overwrite_qru:
 *         u_flags |= cnp.NPY_ENSURECOPY | cnp.NPY_F_CONTIGUOUS
 *     u1 = PyArray_CheckFromAny(u, NULL, 0, 0, u_flags, NULL)             # <<<<<<<<<<<<<<
 * 
 *     if cnp.PyArray_TYPE(u1) != typecode:
 */
  __pyx_t_1 = ((PyObject *)PyArray_CheckFromAny(__pyx_v_u, NULL, 0, 0, __pyx_v_u_flags, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2011, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_u1 = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "scipy/linalg/_decomp_update.pyx":2013
 *     u1 = PyArray_CheckFromAny(u, NULL, 0, 0, u_flags, NULL)
 * 
 *     if cnp.PyArray_TYPE(u1) != typecode:             # <<<<<<<<<<<<<<
 *         raise ValueError("'u' must have the same type as Q and R")
 *     if not (-n <= k <= n):
 */
  __pyx_t_11 = ((PyArray_TYPE(__pyx_v_u1) != __pyx_v_typecode) != 0);
  if (unlikely(__pyx_t_11)) {

    /* "scipy/linalg/_decomp_update.pyx":2014
 * 
 *     if cnp.PyArray_TYPE(u1) != typecode:
 *         raise ValueError("'u' must have the same type as Q and R")             # <<<<<<<<<<<<<<
 *     if not (-n <= k <= n):
 *         raise ValueError("'k' is out of bounds")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__22, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2014, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 2014, __pyx_L1_error)

    /* "scipy/linalg/_decomp_update.pyx":2013
 *     u1 = PyArray_CheckFromAny(u, NULL, 0, 0, u_flags, NULL)
 * 
 *     if cnp.PyArray_TYPE(u1) != typecode:             # <<<<<<<<<<<<<<
 *         raise ValueError("'u' must have the same type as Q and R")
 *     if not (-n <= k <= n):
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":2015
 *     if cnp.PyArray_TYPE(u1) != typecode:
 *         raise ValueError("'u' must have the same type as Q and R")
 *     if not (-n <= k <= n):             # <<<<<<<<<<<<<<
 *         raise ValueError("'k' is out of bounds")
 *     if k < 0:
 */
  __pyx_t_11 = ((-__pyx_v_n) <= __pyx_v_k);
  if (__pyx_t_11) {
    __pyx_t_11 = (__pyx_v_k <= __pyx_v_n);
  }
  __pyx_t_12 = ((!(__pyx_t_11 != 0)) != 0);
  if (unlikely(__pyx_t_12)) {

    /* "scipy/linalg/_decomp_update.pyx":2016
 *         raise ValueError("'u' must have the same type as Q and R")
 *     if not (-n <= k <= n):
 *         raise ValueError("'k' is out of bounds")             # <<<<<<<<<<<<<<
 *     if k < 0:
 *         k += n
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2016, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 2016, __pyx_L1_error)

    /* "scipy/linalg/_decomp_update.pyx":2015
 *     if cnp.PyArray_TYPE(u1) != typecode:
 *         raise ValueError("'u' must have the same type as Q and R")
 *     if not (-n <= k <= n):             # <<<<<<<<<<<<<<
 *         raise ValueError("'k' is out of bounds")
 *     if k < 0:
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":2017
 *     if not (-n <= k <= n):
 *         raise ValueError("'k' is out of bounds")
 *     if k < 0:             # <<<<<<<<<<<<<<
 *         k += n
 * 
 */
  __pyx_t_12 = ((__pyx_v_k < 0) != 0);
  if (__pyx_t_12) {

    /* "scipy/linalg/_decomp_update.pyx":2018
 *         raise ValueError("'k' is out of bounds")
 *     if k < 0:
 *         k += n             # <<<<<<<<<<<<<<
 * 
 *     if u1.ndim == 2:
 */
    __pyx_v_k = (__pyx_v_k + __pyx_v_n);

    /* "scipy/linalg/_decomp_update.pyx":2017
 *     if not (-n <= k <= n):
 *         raise ValueError("'k' is out of bounds")
 *     if k < 0:             # <<<<<<<<<<<<<<
 *         k += n
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":2020
 *         k += n
 * 
 *     if u1.ndim == 2:             # <<<<<<<<<<<<<<
 *         p = u1.shape[1]
 *         if u1.shape[0] != m:
 */
  switch (__pyx_v_u1->nd) {
    case 2:

    /* "scipy/linalg/_decomp_update.pyx":2021
 * 
 *     if u1.ndim == 2:
 *         p = u1.shape[1]             # <<<<<<<<<<<<<<
 *         if u1.shape[0] != m:
 *             raise ValueError("'u' should be either (M,) or (M,p) when "
 */
    __pyx_v_p = (__pyx_v_u1->dimensions[1]);

    /* "scipy/linalg/_decomp_update.pyx":2022
 *     if u1.ndim == 2:
 *         p = u1.shape[1]
 *         if u1.shape[0] != m:             # <<<<<<<<<<<<<<
 *             raise ValueError("'u' should be either (M,) or (M,p) when "
 *                              "inserting columns. Found %s." %
 */
    __pyx_t_12 = (((__pyx_v_u1->dimensions[0]) != __pyx_v_m) != 0);
    if (unlikely(__pyx_t_12)) {

      /* "scipy/linalg/_decomp_update.pyx":2025
 *             raise ValueError("'u' should be either (M,) or (M,p) when "
 *                              "inserting columns. Found %s." %
 *                              str(getattr(u1, 'shape')))             # <<<<<<<<<<<<<<
 *     elif u1.ndim == 1:
 *         p = 1
 */
      __pyx_t_1 = __Pyx_GetAttr(((PyObject *)__pyx_v_u1), __pyx_n_u_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2025, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyUnicode_Type)), __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2025, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "scipy/linalg/_decomp_update.pyx":2024
 *         if u1.shape[0] != m:
 *             raise ValueError("'u' should be either (M,) or (M,p) when "
 *                              "inserting columns. Found %s." %             # <<<<<<<<<<<<<<
 *                              str(getattr(u1, 'shape')))
 *     elif u1.ndim == 1:
 */
      __pyx_t_1 = PyUnicode_Format(__pyx_kp_u_u_should_be_either_M_or_M_p_whe, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2024, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "scipy/linalg/_decomp_update.pyx":2023
 *         p = u1.shape[1]
 *         if u1.shape[0] != m:
 *             raise ValueError("'u' should be either (M,) or (M,p) when "             # <<<<<<<<<<<<<<
 *                              "inserting columns. Found %s." %
 *                              str(getattr(u1, 'shape')))
 */
      __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2023, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_Raise(__pyx_t_7, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __PYX_ERR(0, 2023, __pyx_L1_error)

      /* "scipy/linalg/_decomp_update.pyx":2022
 *     if u1.ndim == 2:
 *         p = u1.shape[1]
 *         if u1.shape[0] != m:             # <<<<<<<<<<<<<<
 *             raise ValueError("'u' should be either (M,) or (M,p) when "
 *                              "inserting columns. Found %s." %
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":2020
 *         k += n
 * 
 *     if u1.ndim == 2:             # <<<<<<<<<<<<<<
 *         p = u1.shape[1]
 *         if u1.shape[0] != m:
 */
    break;
    case 1:

    /* "scipy/linalg/_decomp_update.pyx":2027
 *                              str(getattr(u1, 'shape')))
 *     elif u1.ndim == 1:
 *         p = 1             # <<<<<<<<<<<<<<
 *         if u1.shape[0] != m:
 *             raise ValueError("'u' should be either (M,) or (M,p) when "
 */
    __pyx_v_p = 1;

    /* "scipy/linalg/_decomp_update.pyx":2028
 *     elif u1.ndim == 1:
 *         p = 1
 *         if u1.shape[0] != m:             # <<<<<<<<<<<<<<
 *             raise ValueError("'u' should be either (M,) or (M,p) when "
 *                              "inserting columns. Found %s." %
 */
    __pyx_t_12 = (((__pyx_v_u1->dimensions[0]) != __pyx_v_m) != 0);
    if (unlikely(__pyx_t_12)) {

      /* "scipy/linalg/_decomp_update.pyx":2031
 *             raise ValueError("'u' should be either (M,) or (M,p) when "
 *                              "inserting columns. Found %s." %
 *                              str(getattr(u1, 'shape')))             # <<<<<<<<<<<<<<
 *     else:
 *         raise ValueError("'u' must be either 1- or 2-D")
 */
      __pyx_t_7 = __Pyx_GetAttr(((PyObject *)__pyx_v_u1), __pyx_n_u_shape); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2031, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyUnicode_Type)), __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2031, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "scipy/linalg/_decomp_update.pyx":2030
 *         if u1.shape[0] != m:
 *             raise ValueError("'u' should be either (M,) or (M,p) when "
 *                              "inserting columns. Found %s." %             # <<<<<<<<<<<<<<
 *                              str(getattr(u1, 'shape')))
 *     else:
 */
      __pyx_t_7 = PyUnicode_Format(__pyx_kp_u_u_should_be_either_M_or_M_p_whe, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2030, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "scipy/linalg/_decomp_update.pyx":2029
 *         p = 1
 *         if u1.shape[0] != m:
 *             raise ValueError("'u' should be either (M,) or (M,p) when "             # <<<<<<<<<<<<<<
 *                              "inserting columns. Found %s." %
 *                              str(getattr(u1, 'shape')))
 */
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2029, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 2029, __pyx_L1_error)

      /* "scipy/linalg/_decomp_update.pyx":2028
 *     elif u1.ndim == 1:
 *         p = 1
 *         if u1.shape[0] != m:             # <<<<<<<<<<<<<<
 *             raise ValueError("'u' should be either (M,) or (M,p) when "
 *                              "inserting columns. Found %s." %
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":2026
 *                              "inserting columns. Found %s." %
 *                              str(getattr(u1, 'shape')))
 *     elif u1.ndim == 1:             # <<<<<<<<<<<<<<
 *         p = 1
 *         if u1.shape[0] != m:
 */
    break;
    default:

    /* "scipy/linalg/_decomp_update.pyx":2033
 *                              str(getattr(u1, 'shape')))
 *     else:
 *         raise ValueError("'u' must be either 1- or 2-D")             # <<<<<<<<<<<<<<
 * 
 *     if rcond is not None and economic:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2033, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 2033, __pyx_L1_error)
    break;
  }

  /* "scipy/linalg/_decomp_update.pyx":2035
 *         raise ValueError("'u' must be either 1- or 2-D")
 * 
 *     if rcond is not None and economic:             # <<<<<<<<<<<<<<
 *         if typecode == cnp.NPY_DOUBLE or typecode == cnp.NPY_CDOUBLE:
 *             drc = rcond
 */
  __pyx_t_11 = (__pyx_v_rcond != Py_None);
  __pyx_t_13 = (__pyx_t_11 != 0);
  if (__pyx_t_13) {
  } else {
    __pyx_t_12 = __pyx_t_13;
    goto __pyx_L10_bool_binop_done;
  }
  __pyx_t_13 = (__pyx_v_economic != 0);
  __pyx_t_12 = __pyx_t_13;
  __pyx_L10_bool_binop_done:;
  if (__pyx_t_12) {

    /* "scipy/linalg/_decomp_update.pyx":2036
 * 
 *     if rcond is not None and economic:
 *         if typecode == cnp.NPY_DOUBLE or typecode == cnp.NPY_CDOUBLE:             # <<<<<<<<<<<<<<
 *             drc = rcond
 *         else:
 */
    switch (__pyx_v_typecode) {
      case NPY_DOUBLE:
      case NPY_CDOUBLE:

      /* "scipy/linalg/_decomp_update.pyx":2037
 *     if rcond is not None and economic:
 *         if typecode == cnp.NPY_DOUBLE or typecode == cnp.NPY_CDOUBLE:
 *             drc = rcond             # <<<<<<<<<<<<<<
 *         else:
 *             frc = rcond
 */
      __pyx_t_14 = __pyx_PyFloat_AsDouble(__pyx_v_rcond); if (unlikely((__pyx_t_14 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 2037, __pyx_L1_error)
      __pyx_v_drc = __pyx_t_14;

      /* "scipy/linalg/_decomp_update.pyx":2036
 * 
 *     if rcond is not None and economic:
 *         if typecode == cnp.NPY_DOUBLE or typecode == cnp.NPY_CDOUBLE:             # <<<<<<<<<<<<<<
 *             drc = rcond
 *         else:
 */
      break;
      default:

      /* "scipy/linalg/_decomp_update.pyx":2039
 *             drc = rcond
 *         else:
 *             frc = rcond             # <<<<<<<<<<<<<<
 *     elif rcond is not None:
 *         raise ValueError("'rcond' is not used when updating full, (M,M) (M,N) "
 */
      __pyx_t_15 = __pyx_PyFloat_AsFloat(__pyx_v_rcond); if (unlikely((__pyx_t_15 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 2039, __pyx_L1_error)
      __pyx_v_frc = __pyx_t_15;
      break;
    }

    /* "scipy/linalg/_decomp_update.pyx":2035
 *         raise ValueError("'u' must be either 1- or 2-D")
 * 
 *     if rcond is not None and economic:             # <<<<<<<<<<<<<<
 *         if typecode == cnp.NPY_DOUBLE or typecode == cnp.NPY_CDOUBLE:
 *             drc = rcond
 */
    goto __pyx_L9;
  }

  /* "scipy/linalg/_decomp_update.pyx":2040
 *         else:
 *             frc = rcond
 *     elif rcond is not None:             # <<<<<<<<<<<<<<
 *         raise ValueError("'rcond' is not used when updating full, (M,M) (M,N) "
 *                          "decompositions and must be None.")
 */
  __pyx_t_12 = (__pyx_v_rcond != Py_None);
  __pyx_t_13 = (__pyx_t_12 != 0);
  if (unlikely(__pyx_t_13)) {

    /* "scipy/linalg/_decomp_update.pyx":2041
 *             frc = rcond
 *     elif rcond is not None:
 *         raise ValueError("'rcond' is not used when updating full, (M,M) (M,N) "             # <<<<<<<<<<<<<<
 *                          "decompositions and must be None.")
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__23, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2041, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 2041, __pyx_L1_error)

    /* "scipy/linalg/_decomp_update.pyx":2040
 *         else:
 *             frc = rcond
 *     elif rcond is not None:             # <<<<<<<<<<<<<<
 *         raise ValueError("'rcond' is not used when updating full, (M,M) (M,N) "
 *                          "decompositions and must be None.")
 */
  }
  __pyx_L9:;

  /* "scipy/linalg/_decomp_update.pyx":2046
 *     # special case 1xN
 *     # if m == 1, Q is always 1x1 and abs(Q[0,0]) == 1.0
 *     if m == 1:             # <<<<<<<<<<<<<<
 *         rnew = np.insert(r1, np.full(p, k, np.intp), q1.conjugate()*u1, 1)
 *         return q1.copy(), rnew
 */
  __pyx_t_13 = ((__pyx_v_m == 1) != 0);
  if (__pyx_t_13) {

    /* "scipy/linalg/_decomp_update.pyx":2047
 *     # if m == 1, Q is always 1x1 and abs(Q[0,0]) == 1.0
 *     if m == 1:
 *         rnew = np.insert(r1, np.full(p, k, np.intp), q1.conjugate()*u1, 1)             # <<<<<<<<<<<<<<
 *         return q1.copy(), rnew
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_np); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2047, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_insert); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2047, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2047, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_full); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2047, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_p); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2047, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_k); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2047, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2047, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_16 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_intp); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 2047, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_t_5, __pyx_t_3, __pyx_t_16};
      __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2047, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_t_5, __pyx_t_3, __pyx_t_16};
      __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2047, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
    } else
    #endif
    {
      __pyx_t_17 = PyTuple_New(3+__pyx_t_10); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 2047, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_17);
      if (__pyx_t_2) {
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_17, 0, __pyx_t_2); __pyx_t_2 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_17, 0+__pyx_t_10, __pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_17, 1+__pyx_t_10, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_16);
      PyTuple_SET_ITEM(__pyx_t_17, 2+__pyx_t_10, __pyx_t_16);
      __pyx_t_5 = 0;
      __pyx_t_3 = 0;
      __pyx_t_16 = 0;
      __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_17, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2047, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_17 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_q1), __pyx_n_s_conjugate); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 2047, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_17);
    __pyx_t_16 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_17))) {
      __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_17);
      if (likely(__pyx_t_16)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_17);
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_17, function);
      }
    }
    __pyx_t_4 = (__pyx_t_16) ? __Pyx_PyObject_CallOneArg(__pyx_t_17, __pyx_t_16) : __Pyx_PyObject_CallNoArg(__pyx_t_17);
    __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2047, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
    __pyx_t_17 = PyNumber_Multiply(__pyx_t_4, ((PyObject *)__pyx_v_u1)); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 2047, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_17);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_6)) {
      PyObject *__pyx_temp[5] = {__pyx_t_4, ((PyObject *)__pyx_v_r1), __pyx_t_7, __pyx_t_17, __pyx_int_1};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_10, 4+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2047, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
      PyObject *__pyx_temp[5] = {__pyx_t_4, ((PyObject *)__pyx_v_r1), __pyx_t_7, __pyx_t_17, __pyx_int_1};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_10, 4+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2047, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
    } else
    #endif
    {
      __pyx_t_16 = PyTuple_New(4+__pyx_t_10); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 2047, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_16);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(((PyObject *)__pyx_v_r1));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_r1));
      PyTuple_SET_ITEM(__pyx_t_16, 0+__pyx_t_10, ((PyObject *)__pyx_v_r1));
      __Pyx_GIVEREF(__pyx_t_7);
      PyTuple_SET_ITEM(__pyx_t_16, 1+__pyx_t_10, __pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_17);
      PyTuple_SET_ITEM(__pyx_t_16, 2+__pyx_t_10, __pyx_t_17);
      __Pyx_INCREF(__pyx_int_1);
      __Pyx_GIVEREF(__pyx_int_1);
      PyTuple_SET_ITEM(__pyx_t_16, 3+__pyx_t_10, __pyx_int_1);
      __pyx_t_7 = 0;
      __pyx_t_17 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_16, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2047, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2047, __pyx_L1_error)
    __pyx_v_rnew = ((PyArrayObject *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "scipy/linalg/_decomp_update.pyx":2048
 *     if m == 1:
 *         rnew = np.insert(r1, np.full(p, k, np.intp), q1.conjugate()*u1, 1)
 *         return q1.copy(), rnew             # <<<<<<<<<<<<<<
 * 
 *     if economic:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_q1), __pyx_n_s_copy); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2048, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_16 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_16)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
      }
    }
    __pyx_t_1 = (__pyx_t_16) ? __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_16) : __Pyx_PyObject_CallNoArg(__pyx_t_6);
    __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2048, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2048, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
    __Pyx_INCREF(((PyObject *)__pyx_v_rnew));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_rnew));
    PyTuple_SET_ITEM(__pyx_t_6, 1, ((PyObject *)__pyx_v_rnew));
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":2046
 *     # special case 1xN
 *     # if m == 1, Q is always 1x1 and abs(Q[0,0]) == 1.0
 *     if m == 1:             # <<<<<<<<<<<<<<
 *         rnew = np.insert(r1, np.full(p, k, np.intp), q1.conjugate()*u1, 1)
 *         return q1.copy(), rnew
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":2050
 *         return q1.copy(), rnew
 * 
 *     if economic:             # <<<<<<<<<<<<<<
 *         if n+p <= m:
 *             p_eco = p
 */
  __pyx_t_13 = (__pyx_v_economic != 0);
  if (__pyx_t_13) {

    /* "scipy/linalg/_decomp_update.pyx":2051
 * 
 *     if economic:
 *         if n+p <= m:             # <<<<<<<<<<<<<<
 *             p_eco = p
 *             p_full = 0
 */
    __pyx_t_13 = (((__pyx_v_n + __pyx_v_p) <= __pyx_v_m) != 0);
    if (__pyx_t_13) {

      /* "scipy/linalg/_decomp_update.pyx":2052
 *     if economic:
 *         if n+p <= m:
 *             p_eco = p             # <<<<<<<<<<<<<<
 *             p_full = 0
 *         else:
 */
      __pyx_v_p_eco = __pyx_v_p;

      /* "scipy/linalg/_decomp_update.pyx":2053
 *         if n+p <= m:
 *             p_eco = p
 *             p_full = 0             # <<<<<<<<<<<<<<
 *         else:
 *             p_eco = m-n
 */
      __pyx_v_p_full = 0;

      /* "scipy/linalg/_decomp_update.pyx":2051
 * 
 *     if economic:
 *         if n+p <= m:             # <<<<<<<<<<<<<<
 *             p_eco = p
 *             p_full = 0
 */
      goto __pyx_L14;
    }

    /* "scipy/linalg/_decomp_update.pyx":2055
 *             p_full = 0
 *         else:
 *             p_eco = m-n             # <<<<<<<<<<<<<<
 *             p_full = p - p_eco
 *             if not cnp.PyArray_CHKFLAGS(u1, cnp.NPY_F_CONTIGUOUS):
 */
    /*else*/ {
      __pyx_v_p_eco = (__pyx_v_m - __pyx_v_n);

      /* "scipy/linalg/_decomp_update.pyx":2056
 *         else:
 *             p_eco = m-n
 *             p_full = p - p_eco             # <<<<<<<<<<<<<<
 *             if not cnp.PyArray_CHKFLAGS(u1, cnp.NPY_F_CONTIGUOUS):
 *                 u1 = PyArray_FromArray(u1, NULL, cnp.NPY_F_CONTIGUOUS)
 */
      __pyx_v_p_full = (__pyx_v_p - __pyx_v_p_eco);

      /* "scipy/linalg/_decomp_update.pyx":2057
 *             p_eco = m-n
 *             p_full = p - p_eco
 *             if not cnp.PyArray_CHKFLAGS(u1, cnp.NPY_F_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *                 u1 = PyArray_FromArray(u1, NULL, cnp.NPY_F_CONTIGUOUS)
 *         shape[0] = m
 */
      __pyx_t_13 = ((!(PyArray_CHKFLAGS(__pyx_v_u1, NPY_F_CONTIGUOUS) != 0)) != 0);
      if (__pyx_t_13) {

        /* "scipy/linalg/_decomp_update.pyx":2058
 *             p_full = p - p_eco
 *             if not cnp.PyArray_CHKFLAGS(u1, cnp.NPY_F_CONTIGUOUS):
 *                 u1 = PyArray_FromArray(u1, NULL, cnp.NPY_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *         shape[0] = m
 *         shape[1] = n+p_eco
 */
        __pyx_t_6 = ((PyObject *)PyArray_FromArray(__pyx_v_u1, NULL, NPY_F_CONTIGUOUS)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2058, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF_SET(__pyx_v_u1, ((PyArrayObject *)__pyx_t_6));
        __pyx_t_6 = 0;

        /* "scipy/linalg/_decomp_update.pyx":2057
 *             p_eco = m-n
 *             p_full = p - p_eco
 *             if not cnp.PyArray_CHKFLAGS(u1, cnp.NPY_F_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *                 u1 = PyArray_FromArray(u1, NULL, cnp.NPY_F_CONTIGUOUS)
 *         shape[0] = m
 */
      }
    }
    __pyx_L14:;

    /* "scipy/linalg/_decomp_update.pyx":2059
 *             if not cnp.PyArray_CHKFLAGS(u1, cnp.NPY_F_CONTIGUOUS):
 *                 u1 = PyArray_FromArray(u1, NULL, cnp.NPY_F_CONTIGUOUS)
 *         shape[0] = m             # <<<<<<<<<<<<<<
 *         shape[1] = n+p_eco
 *         qnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 */
    (__pyx_v_shape[0]) = __pyx_v_m;

    /* "scipy/linalg/_decomp_update.pyx":2060
 *                 u1 = PyArray_FromArray(u1, NULL, cnp.NPY_F_CONTIGUOUS)
 *         shape[0] = m
 *         shape[1] = n+p_eco             # <<<<<<<<<<<<<<
 *         qnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 *         qnew[:,:-p_eco] = q1
 */
    (__pyx_v_shape[1]) = (__pyx_v_n + __pyx_v_p_eco);

    /* "scipy/linalg/_decomp_update.pyx":2061
 *         shape[0] = m
 *         shape[1] = n+p_eco
 *         qnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)             # <<<<<<<<<<<<<<
 *         qnew[:,:-p_eco] = q1
 * 
 */
    __pyx_t_6 = PyArray_ZEROS(2, __pyx_v_shape, __pyx_v_typecode, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2061, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2061, __pyx_L1_error)
    __pyx_v_qnew = ((PyArrayObject *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "scipy/linalg/_decomp_update.pyx":2062
 *         shape[1] = n+p_eco
 *         qnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 *         qnew[:,:-p_eco] = q1             # <<<<<<<<<<<<<<
 * 
 *         shape[0] = n+p_eco
 */
    __pyx_t_6 = __Pyx_PyInt_From_int((-__pyx_v_p_eco)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2062, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = PySlice_New(Py_None, __pyx_t_6, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2062, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2062, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_slice__14);
    __Pyx_GIVEREF(__pyx_slice__14);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_slice__14);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
    __pyx_t_1 = 0;
    if (unlikely(PyObject_SetItem(((PyObject *)__pyx_v_qnew), __pyx_t_6, ((PyObject *)__pyx_v_q1)) < 0)) __PYX_ERR(0, 2062, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "scipy/linalg/_decomp_update.pyx":2064
 *         qnew[:,:-p_eco] = q1
 * 
 *         shape[0] = n+p_eco             # <<<<<<<<<<<<<<
 *         shape[1] = n+p
 *         rnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 */
    (__pyx_v_shape[0]) = (__pyx_v_n + __pyx_v_p_eco);

    /* "scipy/linalg/_decomp_update.pyx":2065
 * 
 *         shape[0] = n+p_eco
 *         shape[1] = n+p             # <<<<<<<<<<<<<<
 *         rnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 *         rnew[:n,:k] = r1[:,:k]
 */
    (__pyx_v_shape[1]) = (__pyx_v_n + __pyx_v_p);

    /* "scipy/linalg/_decomp_update.pyx":2066
 *         shape[0] = n+p_eco
 *         shape[1] = n+p
 *         rnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)             # <<<<<<<<<<<<<<
 *         rnew[:n,:k] = r1[:,:k]
 *         rnew[:n,k+p:] = r1[:,k:]
 */
    __pyx_t_6 = PyArray_ZEROS(2, __pyx_v_shape, __pyx_v_typecode, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2066, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2066, __pyx_L1_error)
    __pyx_v_rnew = ((PyArrayObject *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "scipy/linalg/_decomp_update.pyx":2067
 *         shape[1] = n+p
 *         rnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 *         rnew[:n,:k] = r1[:,:k]             # <<<<<<<<<<<<<<
 *         rnew[:n,k+p:] = r1[:,k:]
 *         u1 = validate_array(u1, check_finite)
 */
    __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_k); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2067, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = PySlice_New(Py_None, __pyx_t_6, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2067, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2067, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_slice__14);
    __Pyx_GIVEREF(__pyx_slice__14);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_slice__14);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_r1), __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2067, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_n); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2067, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_16 = PySlice_New(Py_None, __pyx_t_6, Py_None); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 2067, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_k); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2067, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_17 = PySlice_New(Py_None, __pyx_t_6, Py_None); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 2067, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_17);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2067, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_16);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_16);
    __Pyx_GIVEREF(__pyx_t_17);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_17);
    __pyx_t_16 = 0;
    __pyx_t_17 = 0;
    if (unlikely(PyObject_SetItem(((PyObject *)__pyx_v_rnew), __pyx_t_6, __pyx_t_1) < 0)) __PYX_ERR(0, 2067, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "scipy/linalg/_decomp_update.pyx":2068
 *         rnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 *         rnew[:n,:k] = r1[:,:k]
 *         rnew[:n,k+p:] = r1[:,k:]             # <<<<<<<<<<<<<<
 *         u1 = validate_array(u1, check_finite)
 *         qptr = extract(qnew, qs)
 */
    __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_k); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2068, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = PySlice_New(__pyx_t_1, Py_None, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2068, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2068, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_slice__14);
    __Pyx_GIVEREF(__pyx_slice__14);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_slice__14);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_r1), __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2068, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_n); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2068, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_17 = PySlice_New(Py_None, __pyx_t_1, Py_None); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 2068, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_17);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyInt_From_int((__pyx_v_k + __pyx_v_p)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2068, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_16 = PySlice_New(__pyx_t_1, Py_None, Py_None); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 2068, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2068, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_17);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_17);
    __Pyx_GIVEREF(__pyx_t_16);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_16);
    __pyx_t_17 = 0;
    __pyx_t_16 = 0;
    if (unlikely(PyObject_SetItem(((PyObject *)__pyx_v_rnew), __pyx_t_1, __pyx_t_6) < 0)) __PYX_ERR(0, 2068, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "scipy/linalg/_decomp_update.pyx":2069
 *         rnew[:n,:k] = r1[:,:k]
 *         rnew[:n,k+p:] = r1[:,k:]
 *         u1 = validate_array(u1, check_finite)             # <<<<<<<<<<<<<<
 *         qptr = extract(qnew, qs)
 *         rptr = extract(rnew, rs)
 */
    __pyx_t_6 = __pyx_f_5scipy_6linalg_14_decomp_update_validate_array(__pyx_v_u1, __pyx_v_check_finite); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2069, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2069, __pyx_L1_error)
    __Pyx_DECREF_SET(__pyx_v_u1, ((PyArrayObject *)__pyx_t_6));
    __pyx_t_6 = 0;

    /* "scipy/linalg/_decomp_update.pyx":2070
 *         rnew[:n,k+p:] = r1[:,k:]
 *         u1 = validate_array(u1, check_finite)
 *         qptr = extract(qnew, qs)             # <<<<<<<<<<<<<<
 *         rptr = extract(rnew, rs)
 *         uptr = extract(u1, us)
 */
    __pyx_v_qptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_qnew, __pyx_v_qs);

    /* "scipy/linalg/_decomp_update.pyx":2071
 *         u1 = validate_array(u1, check_finite)
 *         qptr = extract(qnew, qs)
 *         rptr = extract(rnew, rs)             # <<<<<<<<<<<<<<
 *         uptr = extract(u1, us)
 *         with nogil:
 */
    __pyx_v_rptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_rnew, __pyx_v_rs);

    /* "scipy/linalg/_decomp_update.pyx":2072
 *         qptr = extract(qnew, qs)
 *         rptr = extract(rnew, rs)
 *         uptr = extract(u1, us)             # <<<<<<<<<<<<<<
 *         with nogil:
 *             if typecode == cnp.NPY_FLOAT:
 */
    __pyx_v_uptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_u1, __pyx_v_us);

    /* "scipy/linalg/_decomp_update.pyx":2073
 *         rptr = extract(rnew, rs)
 *         uptr = extract(u1, us)
 *         with nogil:             # <<<<<<<<<<<<<<
 *             if typecode == cnp.NPY_FLOAT:
 *                 info = thin_qr_col_insert(m, n, <float*>qptr, qs,
 */
    {
        #ifdef WITH_THREAD
        PyThreadState *_save;
        Py_UNBLOCK_THREADS
        __Pyx_FastGIL_Remember();
        #endif
        /*try:*/ {

          /* "scipy/linalg/_decomp_update.pyx":2074
 *         uptr = extract(u1, us)
 *         with nogil:
 *             if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                 info = thin_qr_col_insert(m, n, <float*>qptr, qs,
 *                     <float*>rptr, rs, <float*>uptr, us, k, p_eco,
 */
          switch (__pyx_v_typecode) {
            case NPY_FLOAT:

            /* "scipy/linalg/_decomp_update.pyx":2075
 *         with nogil:
 *             if typecode == cnp.NPY_FLOAT:
 *                 info = thin_qr_col_insert(m, n, <float*>qptr, qs,             # <<<<<<<<<<<<<<
 *                     <float*>rptr, rs, <float*>uptr, us, k, p_eco,
 *                     p_full, &frc)
 */
            __pyx_v_info = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_col_insert(__pyx_v_m, __pyx_v_n, ((float *)__pyx_v_qptr), __pyx_v_qs, ((float *)__pyx_v_rptr), __pyx_v_rs, ((float *)__pyx_v_uptr), __pyx_v_us, __pyx_v_k, __pyx_v_p_eco, __pyx_v_p_full, (&__pyx_v_frc));

            /* "scipy/linalg/_decomp_update.pyx":2074
 *         uptr = extract(u1, us)
 *         with nogil:
 *             if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                 info = thin_qr_col_insert(m, n, <float*>qptr, qs,
 *                     <float*>rptr, rs, <float*>uptr, us, k, p_eco,
 */
            break;
            case NPY_DOUBLE:

            /* "scipy/linalg/_decomp_update.pyx":2079
 *                     p_full, &frc)
 *             elif typecode == cnp.NPY_DOUBLE:
 *                 info = thin_qr_col_insert(m, n, <double*>qptr, qs,             # <<<<<<<<<<<<<<
 *                     <double*>rptr, rs, <double*>uptr, us, k, p_eco,
 *                     p_full, &drc)
 */
            __pyx_v_info = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_col_insert(__pyx_v_m, __pyx_v_n, ((double *)__pyx_v_qptr), __pyx_v_qs, ((double *)__pyx_v_rptr), __pyx_v_rs, ((double *)__pyx_v_uptr), __pyx_v_us, __pyx_v_k, __pyx_v_p_eco, __pyx_v_p_full, (&__pyx_v_drc));

            /* "scipy/linalg/_decomp_update.pyx":2078
 *                     <float*>rptr, rs, <float*>uptr, us, k, p_eco,
 *                     p_full, &frc)
 *             elif typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                 info = thin_qr_col_insert(m, n, <double*>qptr, qs,
 *                     <double*>rptr, rs, <double*>uptr, us, k, p_eco,
 */
            break;
            case NPY_CFLOAT:

            /* "scipy/linalg/_decomp_update.pyx":2083
 *                     p_full, &drc)
 *             elif typecode == cnp.NPY_CFLOAT:
 *                 info = thin_qr_col_insert(m, n, <float_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                     <float_complex*>rptr, rs, <float_complex*>uptr, us, k, p_eco,
 *                     p_full, &cfrc)
 */
            __pyx_v_info = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_col_insert(__pyx_v_m, __pyx_v_n, ((__pyx_t_float_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_float_complex *)__pyx_v_rptr), __pyx_v_rs, ((__pyx_t_float_complex *)__pyx_v_uptr), __pyx_v_us, __pyx_v_k, __pyx_v_p_eco, __pyx_v_p_full, (&__pyx_v_cfrc));

            /* "scipy/linalg/_decomp_update.pyx":2082
 *                     <double*>rptr, rs, <double*>uptr, us, k, p_eco,
 *                     p_full, &drc)
 *             elif typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                 info = thin_qr_col_insert(m, n, <float_complex*>qptr, qs,
 *                     <float_complex*>rptr, rs, <float_complex*>uptr, us, k, p_eco,
 */
            break;
            default:

            /* "scipy/linalg/_decomp_update.pyx":2087
 *                     p_full, &cfrc)
 *             else:  # typecode == cnp.NPY_CDOUBLE:
 *                 info = thin_qr_col_insert(m, n, <double_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                     <double_complex*>rptr, rs, <double_complex*>uptr, us, k, p_eco,
 *                     p_full, &cdrc)
 */
            __pyx_v_info = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_col_insert(__pyx_v_m, __pyx_v_n, ((__pyx_t_double_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_double_complex *)__pyx_v_rptr), __pyx_v_rs, ((__pyx_t_double_complex *)__pyx_v_uptr), __pyx_v_us, __pyx_v_k, __pyx_v_p_eco, __pyx_v_p_full, (&__pyx_v_cdrc));
            break;
          }
        }

        /* "scipy/linalg/_decomp_update.pyx":2073
 *         rptr = extract(rnew, rs)
 *         uptr = extract(u1, us)
 *         with nogil:             # <<<<<<<<<<<<<<
 *             if typecode == cnp.NPY_FLOAT:
 *                 info = thin_qr_col_insert(m, n, <float*>qptr, qs,
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_FastGIL_Forget();
            Py_BLOCK_THREADS
            #endif
            goto __pyx_L18;
          }
          __pyx_L18:;
        }
    }

    /* "scipy/linalg/_decomp_update.pyx":2090
 *                     <double_complex*>rptr, rs, <double_complex*>uptr, us, k, p_eco,
 *                     p_full, &cdrc)
 *         if info == 2:             # <<<<<<<<<<<<<<
 *             raise LinAlgError("One of the columns of u lies in the span of Q. "
 *                               "Found reciprocal condition number of %s for Q "
 */
    __pyx_t_13 = ((__pyx_v_info == 2) != 0);
    if (unlikely(__pyx_t_13)) {

      /* "scipy/linalg/_decomp_update.pyx":2091
 *                     p_full, &cdrc)
 *         if info == 2:
 *             raise LinAlgError("One of the columns of u lies in the span of Q. "             # <<<<<<<<<<<<<<
 *                               "Found reciprocal condition number of %s for Q "
 *                               "augmented with u/||u||." % str(drc))
 */
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_LinAlgError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2091, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);

      /* "scipy/linalg/_decomp_update.pyx":2093
 *             raise LinAlgError("One of the columns of u lies in the span of Q. "
 *                               "Found reciprocal condition number of %s for Q "
 *                               "augmented with u/||u||." % str(drc))             # <<<<<<<<<<<<<<
 *         elif info == MEMORY_ERROR:
 *             raise MemoryError("Unable to allocate memory for array")
 */
      __pyx_t_16 = PyFloat_FromDouble(__pyx_v_drc); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 2093, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_16);
      __pyx_t_17 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyUnicode_Type)), __pyx_t_16); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 2093, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_17);
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
      __pyx_t_16 = PyUnicode_Format(__pyx_kp_u_One_of_the_columns_of_u_lies_in, __pyx_t_17); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 2093, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_16);
      __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
      __pyx_t_17 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_17 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_17)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_17);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
        }
      }
      __pyx_t_6 = (__pyx_t_17) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_17, __pyx_t_16) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_16);
      __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2091, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __PYX_ERR(0, 2091, __pyx_L1_error)

      /* "scipy/linalg/_decomp_update.pyx":2090
 *                     <double_complex*>rptr, rs, <double_complex*>uptr, us, k, p_eco,
 *                     p_full, &cdrc)
 *         if info == 2:             # <<<<<<<<<<<<<<
 *             raise LinAlgError("One of the columns of u lies in the span of Q. "
 *                               "Found reciprocal condition number of %s for Q "
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":2094
 *                               "Found reciprocal condition number of %s for Q "
 *                               "augmented with u/||u||." % str(drc))
 *         elif info == MEMORY_ERROR:             # <<<<<<<<<<<<<<
 *             raise MemoryError("Unable to allocate memory for array")
 *         return qnew, rnew
 */
    __pyx_t_13 = ((__pyx_v_info == __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR) != 0);
    if (unlikely(__pyx_t_13)) {

      /* "scipy/linalg/_decomp_update.pyx":2095
 *                               "augmented with u/||u||." % str(drc))
 *         elif info == MEMORY_ERROR:
 *             raise MemoryError("Unable to allocate memory for array")             # <<<<<<<<<<<<<<
 *         return qnew, rnew
 *     else:
 */
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2095, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __PYX_ERR(0, 2095, __pyx_L1_error)

      /* "scipy/linalg/_decomp_update.pyx":2094
 *                               "Found reciprocal condition number of %s for Q "
 *                               "augmented with u/||u||." % str(drc))
 *         elif info == MEMORY_ERROR:             # <<<<<<<<<<<<<<
 *             raise MemoryError("Unable to allocate memory for array")
 *         return qnew, rnew
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":2096
 *         elif info == MEMORY_ERROR:
 *             raise MemoryError("Unable to allocate memory for array")
 *         return qnew, rnew             # <<<<<<<<<<<<<<
 *     else:
 *         if (not cnp.PyArray_ISONESEGMENT(q1)) or u1.ndim == 2:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2096, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)__pyx_v_qnew));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_qnew));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)__pyx_v_qnew));
    __Pyx_INCREF(((PyObject *)__pyx_v_rnew));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_rnew));
    PyTuple_SET_ITEM(__pyx_t_6, 1, ((PyObject *)__pyx_v_rnew));
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":2050
 *         return q1.copy(), rnew
 * 
 *     if economic:             # <<<<<<<<<<<<<<
 *         if n+p <= m:
 *             p_eco = p
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":2098
 *         return qnew, rnew
 *     else:
 *         if (not cnp.PyArray_ISONESEGMENT(q1)) or u1.ndim == 2:             # <<<<<<<<<<<<<<
 *             q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *         if (not overwrite_qru and cnp.PyArray_CHKFLAGS(q1, cnp.NPY_C_CONTIGUOUS)
 */
  /*else*/ {
    __pyx_t_12 = ((!(PyArray_ISONESEGMENT(__pyx_v_q1) != 0)) != 0);
    if (!__pyx_t_12) {
    } else {
      __pyx_t_13 = __pyx_t_12;
      goto __pyx_L21_bool_binop_done;
    }
    __pyx_t_12 = ((__pyx_v_u1->nd == 2) != 0);
    __pyx_t_13 = __pyx_t_12;
    __pyx_L21_bool_binop_done:;
    if (__pyx_t_13) {

      /* "scipy/linalg/_decomp_update.pyx":2099
 *     else:
 *         if (not cnp.PyArray_ISONESEGMENT(q1)) or u1.ndim == 2:
 *             q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *         if (not overwrite_qru and cnp.PyArray_CHKFLAGS(q1, cnp.NPY_C_CONTIGUOUS)
 *             and (typecode == cnp.NPY_CFLOAT or typecode == cnp.NPY_CDOUBLE)):
 */
      __pyx_t_6 = ((PyObject *)PyArray_FromArray(__pyx_v_q1, NULL, NPY_F_CONTIGUOUS)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2099, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF_SET(__pyx_v_q1, ((PyArrayObject *)__pyx_t_6));
      __pyx_t_6 = 0;

      /* "scipy/linalg/_decomp_update.pyx":2098
 *         return qnew, rnew
 *     else:
 *         if (not cnp.PyArray_ISONESEGMENT(q1)) or u1.ndim == 2:             # <<<<<<<<<<<<<<
 *             q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *         if (not overwrite_qru and cnp.PyArray_CHKFLAGS(q1, cnp.NPY_C_CONTIGUOUS)
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":2100
 *         if (not cnp.PyArray_ISONESEGMENT(q1)) or u1.ndim == 2:
 *             q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *         if (not overwrite_qru and cnp.PyArray_CHKFLAGS(q1, cnp.NPY_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *             and (typecode == cnp.NPY_CFLOAT or typecode == cnp.NPY_CDOUBLE)):
 *             u_flags |= cnp.NPY_ENSURECOPY
 */
    __pyx_t_12 = ((!(__pyx_v_overwrite_qru != 0)) != 0);
    if (__pyx_t_12) {
    } else {
      __pyx_t_13 = __pyx_t_12;
      goto __pyx_L24_bool_binop_done;
    }

    /* "scipy/linalg/_decomp_update.pyx":2101
 *             q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *         if (not overwrite_qru and cnp.PyArray_CHKFLAGS(q1, cnp.NPY_C_CONTIGUOUS)
 *             and (typecode == cnp.NPY_CFLOAT or typecode == cnp.NPY_CDOUBLE)):             # <<<<<<<<<<<<<<
 *             u_flags |= cnp.NPY_ENSURECOPY
 *             u1 = PyArray_FromArray(u1, NULL, u_flags)
 */
    __pyx_t_12 = (PyArray_CHKFLAGS(__pyx_v_q1, NPY_C_CONTIGUOUS) != 0);
    if (__pyx_t_12) {
    } else {
      __pyx_t_13 = __pyx_t_12;
      goto __pyx_L24_bool_binop_done;
    }
    switch (__pyx_v_typecode) {
      case NPY_CFLOAT:
      case NPY_CDOUBLE:
      __pyx_t_12 = 1;
      break;
      default:
      __pyx_t_12 = 0;
      break;
    }
    __pyx_t_13 = __pyx_t_12;
    __pyx_L24_bool_binop_done:;

    /* "scipy/linalg/_decomp_update.pyx":2100
 *         if (not cnp.PyArray_ISONESEGMENT(q1)) or u1.ndim == 2:
 *             q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *         if (not overwrite_qru and cnp.PyArray_CHKFLAGS(q1, cnp.NPY_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *             and (typecode == cnp.NPY_CFLOAT or typecode == cnp.NPY_CDOUBLE)):
 *             u_flags |= cnp.NPY_ENSURECOPY
 */
    if (__pyx_t_13) {

      /* "scipy/linalg/_decomp_update.pyx":2102
 *         if (not overwrite_qru and cnp.PyArray_CHKFLAGS(q1, cnp.NPY_C_CONTIGUOUS)
 *             and (typecode == cnp.NPY_CFLOAT or typecode == cnp.NPY_CDOUBLE)):
 *             u_flags |= cnp.NPY_ENSURECOPY             # <<<<<<<<<<<<<<
 *             u1 = PyArray_FromArray(u1, NULL, u_flags)
 * 
 */
      __pyx_v_u_flags = (__pyx_v_u_flags | NPY_ENSURECOPY);

      /* "scipy/linalg/_decomp_update.pyx":2103
 *             and (typecode == cnp.NPY_CFLOAT or typecode == cnp.NPY_CDOUBLE)):
 *             u_flags |= cnp.NPY_ENSURECOPY
 *             u1 = PyArray_FromArray(u1, NULL, u_flags)             # <<<<<<<<<<<<<<
 * 
 *         shape[0] = m
 */
      __pyx_t_6 = ((PyObject *)PyArray_FromArray(__pyx_v_u1, NULL, __pyx_v_u_flags)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2103, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF_SET(__pyx_v_u1, ((PyArrayObject *)__pyx_t_6));
      __pyx_t_6 = 0;

      /* "scipy/linalg/_decomp_update.pyx":2100
 *         if (not cnp.PyArray_ISONESEGMENT(q1)) or u1.ndim == 2:
 *             q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *         if (not overwrite_qru and cnp.PyArray_CHKFLAGS(q1, cnp.NPY_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *             and (typecode == cnp.NPY_CFLOAT or typecode == cnp.NPY_CDOUBLE)):
 *             u_flags |= cnp.NPY_ENSURECOPY
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":2105
 *             u1 = PyArray_FromArray(u1, NULL, u_flags)
 * 
 *         shape[0] = m             # <<<<<<<<<<<<<<
 *         shape[1] = n+p
 *         rnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 */
    (__pyx_v_shape[0]) = __pyx_v_m;

    /* "scipy/linalg/_decomp_update.pyx":2106
 * 
 *         shape[0] = m
 *         shape[1] = n+p             # <<<<<<<<<<<<<<
 *         rnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 * 
 */
    (__pyx_v_shape[1]) = (__pyx_v_n + __pyx_v_p);

    /* "scipy/linalg/_decomp_update.pyx":2107
 *         shape[0] = m
 *         shape[1] = n+p
 *         rnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)             # <<<<<<<<<<<<<<
 * 
 *         rnew[:,:k] = r1[:,:k]
 */
    __pyx_t_6 = PyArray_ZEROS(2, __pyx_v_shape, __pyx_v_typecode, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2107, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2107, __pyx_L1_error)
    __pyx_v_rnew = ((PyArrayObject *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "scipy/linalg/_decomp_update.pyx":2109
 *         rnew = cnp.PyArray_ZEROS(2, shape, typecode, 1)
 * 
 *         rnew[:,:k] = r1[:,:k]             # <<<<<<<<<<<<<<
 *         rnew[:,k+p:] = r1[:,k:]
 * 
 */
    __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_k); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2109, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = PySlice_New(Py_None, __pyx_t_6, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2109, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2109, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_slice__14);
    __Pyx_GIVEREF(__pyx_slice__14);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_slice__14);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_r1), __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2109, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_k); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2109, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_16 = PySlice_New(Py_None, __pyx_t_6, Py_None); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 2109, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2109, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_slice__14);
    __Pyx_GIVEREF(__pyx_slice__14);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_slice__14);
    __Pyx_GIVEREF(__pyx_t_16);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_16);
    __pyx_t_16 = 0;
    if (unlikely(PyObject_SetItem(((PyObject *)__pyx_v_rnew), __pyx_t_6, __pyx_t_1) < 0)) __PYX_ERR(0, 2109, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "scipy/linalg/_decomp_update.pyx":2110
 * 
 *         rnew[:,:k] = r1[:,:k]
 *         rnew[:,k+p:] = r1[:,k:]             # <<<<<<<<<<<<<<
 * 
 *         u1 = validate_array(u1, check_finite)
 */
    __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_k); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2110, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = PySlice_New(__pyx_t_1, Py_None, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2110, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2110, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_slice__14);
    __Pyx_GIVEREF(__pyx_slice__14);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_slice__14);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_r1), __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2110, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyInt_From_int((__pyx_v_k + __pyx_v_p)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2110, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_16 = PySlice_New(__pyx_t_1, Py_None, Py_None); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 2110, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2110, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_slice__14);
    __Pyx_GIVEREF(__pyx_slice__14);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_slice__14);
    __Pyx_GIVEREF(__pyx_t_16);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_16);
    __pyx_t_16 = 0;
    if (unlikely(PyObject_SetItem(((PyObject *)__pyx_v_rnew), __pyx_t_1, __pyx_t_6) < 0)) __PYX_ERR(0, 2110, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "scipy/linalg/_decomp_update.pyx":2112
 *         rnew[:,k+p:] = r1[:,k:]
 * 
 *         u1 = validate_array(u1, check_finite)             # <<<<<<<<<<<<<<
 *         rptr = extract(rnew, rs)
 *         form_qTu(q1, u1, rptr, rs, k)
 */
    __pyx_t_6 = __pyx_f_5scipy_6linalg_14_decomp_update_validate_array(__pyx_v_u1, __pyx_v_check_finite); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2112, __pyx_L1_error)
    __Pyx_DECREF_SET(__pyx_v_u1, ((PyArrayObject *)__pyx_t_6));
    __pyx_t_6 = 0;

    /* "scipy/linalg/_decomp_update.pyx":2113
 * 
 *         u1 = validate_array(u1, check_finite)
 *         rptr = extract(rnew, rs)             # <<<<<<<<<<<<<<
 *         form_qTu(q1, u1, rptr, rs, k)
 *         if not overwrite_qru:
 */
    __pyx_v_rptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_rnew, __pyx_v_rs);

    /* "scipy/linalg/_decomp_update.pyx":2114
 *         u1 = validate_array(u1, check_finite)
 *         rptr = extract(rnew, rs)
 *         form_qTu(q1, u1, rptr, rs, k)             # <<<<<<<<<<<<<<
 *         if not overwrite_qru:
 *             q1 = q1.copy('F')
 */
    __pyx_t_6 = __pyx_f_5scipy_6linalg_14_decomp_update_form_qTu(__pyx_v_q1, __pyx_v_u1, __pyx_v_rptr, __pyx_v_rs, __pyx_v_k); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2114, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "scipy/linalg/_decomp_update.pyx":2115
 *         rptr = extract(rnew, rs)
 *         form_qTu(q1, u1, rptr, rs, k)
 *         if not overwrite_qru:             # <<<<<<<<<<<<<<
 *             q1 = q1.copy('F')
 * 
 */
    __pyx_t_13 = ((!(__pyx_v_overwrite_qru != 0)) != 0);
    if (__pyx_t_13) {

      /* "scipy/linalg/_decomp_update.pyx":2116
 *         form_qTu(q1, u1, rptr, rs, k)
 *         if not overwrite_qru:
 *             q1 = q1.copy('F')             # <<<<<<<<<<<<<<
 * 
 *         qptr = extract(q1, qs)
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_q1), __pyx_n_s_copy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2116, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_16 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_16)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_16);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
        }
      }
      __pyx_t_6 = (__pyx_t_16) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_16, __pyx_n_u_F) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_n_u_F);
      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2116, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2116, __pyx_L1_error)
      __Pyx_DECREF_SET(__pyx_v_q1, ((PyArrayObject *)__pyx_t_6));
      __pyx_t_6 = 0;

      /* "scipy/linalg/_decomp_update.pyx":2115
 *         rptr = extract(rnew, rs)
 *         form_qTu(q1, u1, rptr, rs, k)
 *         if not overwrite_qru:             # <<<<<<<<<<<<<<
 *             q1 = q1.copy('F')
 * 
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":2118
 *             q1 = q1.copy('F')
 * 
 *         qptr = extract(q1, qs)             # <<<<<<<<<<<<<<
 *         if p == 1:
 *             with nogil:
 */
    __pyx_v_qptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_q1, __pyx_v_qs);

    /* "scipy/linalg/_decomp_update.pyx":2119
 * 
 *         qptr = extract(q1, qs)
 *         if p == 1:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 */
    __pyx_t_13 = ((__pyx_v_p == 1) != 0);
    if (__pyx_t_13) {

      /* "scipy/linalg/_decomp_update.pyx":2120
 *         qptr = extract(q1, qs)
 *         if p == 1:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     qr_col_insert(m, n+p, <float*>qptr, qs,
 */
      {
          #ifdef WITH_THREAD
          PyThreadState *_save;
          Py_UNBLOCK_THREADS
          __Pyx_FastGIL_Remember();
          #endif
          /*try:*/ {

            /* "scipy/linalg/_decomp_update.pyx":2121
 *         if p == 1:
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     qr_col_insert(m, n+p, <float*>qptr, qs,
 *                         <float*>rptr, rs, k)
 */
            switch (__pyx_v_typecode) {
              case NPY_FLOAT:

              /* "scipy/linalg/_decomp_update.pyx":2122
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 *                     qr_col_insert(m, n+p, <float*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float*>rptr, rs, k)
 *                 elif typecode == cnp.NPY_DOUBLE:
 */
              __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_insert(__pyx_v_m, (__pyx_v_n + __pyx_v_p), ((float *)__pyx_v_qptr), __pyx_v_qs, ((float *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k);

              /* "scipy/linalg/_decomp_update.pyx":2121
 *         if p == 1:
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     qr_col_insert(m, n+p, <float*>qptr, qs,
 *                         <float*>rptr, rs, k)
 */
              break;
              case NPY_DOUBLE:

              /* "scipy/linalg/_decomp_update.pyx":2125
 *                         <float*>rptr, rs, k)
 *                 elif typecode == cnp.NPY_DOUBLE:
 *                     qr_col_insert(m, n+p, <double*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double*>rptr, rs, k)
 *                 elif typecode == cnp.NPY_CFLOAT:
 */
              __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_insert(__pyx_v_m, (__pyx_v_n + __pyx_v_p), ((double *)__pyx_v_qptr), __pyx_v_qs, ((double *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k);

              /* "scipy/linalg/_decomp_update.pyx":2124
 *                     qr_col_insert(m, n+p, <float*>qptr, qs,
 *                         <float*>rptr, rs, k)
 *                 elif typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                     qr_col_insert(m, n+p, <double*>qptr, qs,
 *                         <double*>rptr, rs, k)
 */
              break;
              case NPY_CFLOAT:

              /* "scipy/linalg/_decomp_update.pyx":2128
 *                         <double*>rptr, rs, k)
 *                 elif typecode == cnp.NPY_CFLOAT:
 *                     qr_col_insert(m, n+p, <float_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float_complex*>rptr, rs, k)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 */
              __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_insert(__pyx_v_m, (__pyx_v_n + __pyx_v_p), ((__pyx_t_float_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_float_complex *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k);

              /* "scipy/linalg/_decomp_update.pyx":2127
 *                     qr_col_insert(m, n+p, <double*>qptr, qs,
 *                         <double*>rptr, rs, k)
 *                 elif typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                     qr_col_insert(m, n+p, <float_complex*>qptr, qs,
 *                         <float_complex*>rptr, rs, k)
 */
              break;
              default:

              /* "scipy/linalg/_decomp_update.pyx":2131
 *                         <float_complex*>rptr, rs, k)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 *                     qr_col_insert(m, n+p, <double_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double_complex*>rptr, rs, k)
 *         else:
 */
              __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_col_insert(__pyx_v_m, (__pyx_v_n + __pyx_v_p), ((__pyx_t_double_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_double_complex *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k);
              break;
            }
          }

          /* "scipy/linalg/_decomp_update.pyx":2120
 *         qptr = extract(q1, qs)
 *         if p == 1:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     qr_col_insert(m, n+p, <float*>qptr, qs,
 */
          /*finally:*/ {
            /*normal exit:*/{
              #ifdef WITH_THREAD
              __Pyx_FastGIL_Forget();
              Py_BLOCK_THREADS
              #endif
              goto __pyx_L31;
            }
            __pyx_L31:;
          }
      }

      /* "scipy/linalg/_decomp_update.pyx":2119
 * 
 *         qptr = extract(q1, qs)
 *         if p == 1:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 */
      goto __pyx_L28;
    }

    /* "scipy/linalg/_decomp_update.pyx":2134
 *                         <double_complex*>rptr, rs, k)
 *         else:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     info = qr_block_col_insert(m, n+p, <float*>qptr, qs,
 */
    /*else*/ {
      {
          #ifdef WITH_THREAD
          PyThreadState *_save;
          Py_UNBLOCK_THREADS
          __Pyx_FastGIL_Remember();
          #endif
          /*try:*/ {

            /* "scipy/linalg/_decomp_update.pyx":2135
 *         else:
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     info = qr_block_col_insert(m, n+p, <float*>qptr, qs,
 *                         <float*>rptr, rs, k, p)
 */
            switch (__pyx_v_typecode) {
              case NPY_FLOAT:

              /* "scipy/linalg/_decomp_update.pyx":2136
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 *                     info = qr_block_col_insert(m, n+p, <float*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float*>rptr, rs, k, p)
 *                 elif typecode == cnp.NPY_DOUBLE:
 */
              __pyx_v_info = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_insert(__pyx_v_m, (__pyx_v_n + __pyx_v_p), ((float *)__pyx_v_qptr), __pyx_v_qs, ((float *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k, __pyx_v_p);

              /* "scipy/linalg/_decomp_update.pyx":2135
 *         else:
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     info = qr_block_col_insert(m, n+p, <float*>qptr, qs,
 *                         <float*>rptr, rs, k, p)
 */
              break;
              case NPY_DOUBLE:

              /* "scipy/linalg/_decomp_update.pyx":2139
 *                         <float*>rptr, rs, k, p)
 *                 elif typecode == cnp.NPY_DOUBLE:
 *                     info = qr_block_col_insert(m, n+p, <double*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double*>rptr, rs, k, p)
 *                 elif typecode == cnp.NPY_CFLOAT:
 */
              __pyx_v_info = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_insert(__pyx_v_m, (__pyx_v_n + __pyx_v_p), ((double *)__pyx_v_qptr), __pyx_v_qs, ((double *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k, __pyx_v_p);

              /* "scipy/linalg/_decomp_update.pyx":2138
 *                     info = qr_block_col_insert(m, n+p, <float*>qptr, qs,
 *                         <float*>rptr, rs, k, p)
 *                 elif typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                     info = qr_block_col_insert(m, n+p, <double*>qptr, qs,
 *                         <double*>rptr, rs, k, p)
 */
              break;
              case NPY_CFLOAT:

              /* "scipy/linalg/_decomp_update.pyx":2142
 *                         <double*>rptr, rs, k, p)
 *                 elif typecode == cnp.NPY_CFLOAT:
 *                     info = qr_block_col_insert(m, n+p, <float_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float_complex*>rptr, rs, k, p)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 */
              __pyx_v_info = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_insert(__pyx_v_m, (__pyx_v_n + __pyx_v_p), ((__pyx_t_float_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_float_complex *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k, __pyx_v_p);

              /* "scipy/linalg/_decomp_update.pyx":2141
 *                     info = qr_block_col_insert(m, n+p, <double*>qptr, qs,
 *                         <double*>rptr, rs, k, p)
 *                 elif typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                     info = qr_block_col_insert(m, n+p, <float_complex*>qptr, qs,
 *                         <float_complex*>rptr, rs, k, p)
 */
              break;
              default:

              /* "scipy/linalg/_decomp_update.pyx":2145
 *                         <float_complex*>rptr, rs, k, p)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 *                     info = qr_block_col_insert(m, n+p, <double_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double_complex*>rptr, rs, k, p)
 *             if info != 0:
 */
              __pyx_v_info = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_block_col_insert(__pyx_v_m, (__pyx_v_n + __pyx_v_p), ((__pyx_t_double_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_double_complex *)__pyx_v_rptr), __pyx_v_rs, __pyx_v_k, __pyx_v_p);
              break;
            }
          }

          /* "scipy/linalg/_decomp_update.pyx":2134
 *                         <double_complex*>rptr, rs, k)
 *         else:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     info = qr_block_col_insert(m, n+p, <float*>qptr, qs,
 */
          /*finally:*/ {
            /*normal exit:*/{
              #ifdef WITH_THREAD
              __Pyx_FastGIL_Forget();
              Py_BLOCK_THREADS
              #endif
              goto __pyx_L34;
            }
            __pyx_L34:;
          }
      }

      /* "scipy/linalg/_decomp_update.pyx":2147
 *                     info = qr_block_col_insert(m, n+p, <double_complex*>qptr, qs,
 *                         <double_complex*>rptr, rs, k, p)
 *             if info != 0:             # <<<<<<<<<<<<<<
 *                 if info > 0:
 *                     raise ValueError('The {0}th argument to ?geqrf was '
 */
      __pyx_t_13 = ((__pyx_v_info != 0) != 0);
      if (__pyx_t_13) {

        /* "scipy/linalg/_decomp_update.pyx":2148
 *                         <double_complex*>rptr, rs, k, p)
 *             if info != 0:
 *                 if info > 0:             # <<<<<<<<<<<<<<
 *                     raise ValueError('The {0}th argument to ?geqrf was '
 *                             'invalid'.format(info))
 */
        __pyx_t_13 = ((__pyx_v_info > 0) != 0);
        if (unlikely(__pyx_t_13)) {

          /* "scipy/linalg/_decomp_update.pyx":2150
 *                 if info > 0:
 *                     raise ValueError('The {0}th argument to ?geqrf was '
 *                             'invalid'.format(info))             # <<<<<<<<<<<<<<
 *                 elif info < 0:
 *                     raise ValueError('The {0}th argument to ?ormqr/?unmqr was '
 */
          __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_The_0_th_argument_to_geqrf_was_i, __pyx_n_s_format); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2150, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_16 = __Pyx_PyInt_From_int(__pyx_v_info); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 2150, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_16);
          __pyx_t_17 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
            __pyx_t_17 = PyMethod_GET_SELF(__pyx_t_1);
            if (likely(__pyx_t_17)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
              __Pyx_INCREF(__pyx_t_17);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_1, function);
            }
          }
          __pyx_t_6 = (__pyx_t_17) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_17, __pyx_t_16) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_16);
          __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2150, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "scipy/linalg/_decomp_update.pyx":2149
 *             if info != 0:
 *                 if info > 0:
 *                     raise ValueError('The {0}th argument to ?geqrf was '             # <<<<<<<<<<<<<<
 *                             'invalid'.format(info))
 *                 elif info < 0:
 */
          __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2149, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_Raise(__pyx_t_1, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __PYX_ERR(0, 2149, __pyx_L1_error)

          /* "scipy/linalg/_decomp_update.pyx":2148
 *                         <double_complex*>rptr, rs, k, p)
 *             if info != 0:
 *                 if info > 0:             # <<<<<<<<<<<<<<
 *                     raise ValueError('The {0}th argument to ?geqrf was '
 *                             'invalid'.format(info))
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":2151
 *                     raise ValueError('The {0}th argument to ?geqrf was '
 *                             'invalid'.format(info))
 *                 elif info < 0:             # <<<<<<<<<<<<<<
 *                     raise ValueError('The {0}th argument to ?ormqr/?unmqr was '
 *                             'invalid'.format(abs(info)))
 */
        __pyx_t_13 = ((__pyx_v_info < 0) != 0);
        if (unlikely(__pyx_t_13)) {

          /* "scipy/linalg/_decomp_update.pyx":2153
 *                 elif info < 0:
 *                     raise ValueError('The {0}th argument to ?ormqr/?unmqr was '
 *                             'invalid'.format(abs(info)))             # <<<<<<<<<<<<<<
 *                 elif info == MEMORY_ERROR:
 *                     raise MemoryError("Unable to allocate memory for array")
 */
          __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_The_0_th_argument_to_ormqr_unmqr, __pyx_n_s_format); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2153, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_10 = abs(__pyx_v_info); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(0, 2153, __pyx_L1_error)
          __pyx_t_16 = __Pyx_PyInt_From_int(__pyx_t_10); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 2153, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_16);
          __pyx_t_17 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
            __pyx_t_17 = PyMethod_GET_SELF(__pyx_t_6);
            if (likely(__pyx_t_17)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
              __Pyx_INCREF(__pyx_t_17);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_6, function);
            }
          }
          __pyx_t_1 = (__pyx_t_17) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_17, __pyx_t_16) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_16);
          __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
          if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2153, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

          /* "scipy/linalg/_decomp_update.pyx":2152
 *                             'invalid'.format(info))
 *                 elif info < 0:
 *                     raise ValueError('The {0}th argument to ?ormqr/?unmqr was '             # <<<<<<<<<<<<<<
 *                             'invalid'.format(abs(info)))
 *                 elif info == MEMORY_ERROR:
 */
          __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2152, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_Raise(__pyx_t_6, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __PYX_ERR(0, 2152, __pyx_L1_error)

          /* "scipy/linalg/_decomp_update.pyx":2151
 *                     raise ValueError('The {0}th argument to ?geqrf was '
 *                             'invalid'.format(info))
 *                 elif info < 0:             # <<<<<<<<<<<<<<
 *                     raise ValueError('The {0}th argument to ?ormqr/?unmqr was '
 *                             'invalid'.format(abs(info)))
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":2154
 *                     raise ValueError('The {0}th argument to ?ormqr/?unmqr was '
 *                             'invalid'.format(abs(info)))
 *                 elif info == MEMORY_ERROR:             # <<<<<<<<<<<<<<
 *                     raise MemoryError("Unable to allocate memory for array")
 *         return q1, rnew
 */
        __pyx_t_13 = ((__pyx_v_info == __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR) != 0);
        if (unlikely(__pyx_t_13)) {

          /* "scipy/linalg/_decomp_update.pyx":2155
 *                             'invalid'.format(abs(info)))
 *                 elif info == MEMORY_ERROR:
 *                     raise MemoryError("Unable to allocate memory for array")             # <<<<<<<<<<<<<<
 *         return q1, rnew
 * 
 */
          __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2155, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_Raise(__pyx_t_6, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __PYX_ERR(0, 2155, __pyx_L1_error)

          /* "scipy/linalg/_decomp_update.pyx":2154
 *                     raise ValueError('The {0}th argument to ?ormqr/?unmqr was '
 *                             'invalid'.format(abs(info)))
 *                 elif info == MEMORY_ERROR:             # <<<<<<<<<<<<<<
 *                     raise MemoryError("Unable to allocate memory for array")
 *         return q1, rnew
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":2147
 *                     info = qr_block_col_insert(m, n+p, <double_complex*>qptr, qs,
 *                         <double_complex*>rptr, rs, k, p)
 *             if info != 0:             # <<<<<<<<<<<<<<
 *                 if info > 0:
 *                     raise ValueError('The {0}th argument to ?geqrf was '
 */
      }
    }
    __pyx_L28:;

    /* "scipy/linalg/_decomp_update.pyx":2156
 *                 elif info == MEMORY_ERROR:
 *                     raise MemoryError("Unable to allocate memory for array")
 *         return q1, rnew             # <<<<<<<<<<<<<<
 * 
 * @cython.embedsignature(True)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2156, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(((PyObject *)__pyx_v_q1));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_q1));
    PyTuple_SET_ITEM(__pyx_t_6, 0, ((PyObject *)__pyx_v_q1));
    __Pyx_INCREF(((PyObject *)__pyx_v_rnew));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_rnew));
    PyTuple_SET_ITEM(__pyx_t_6, 1, ((PyObject *)__pyx_v_rnew));
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;
  }

  /* "scipy/linalg/_decomp_update.pyx":1983
 *         return qnew, rnew
 * 
 * cdef qr_insert_col(Q, R, u, int k, rcond, bint overwrite_qru, bint check_finite):             # <<<<<<<<<<<<<<
 *     cdef cnp.ndarray q1, r1, u1, qnew, rnew
 *     cdef int j
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_AddTraceback("scipy.linalg._decomp_update.qr_insert_col", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_q1);
  __Pyx_XDECREF((PyObject *)__pyx_v_r1);
  __Pyx_XDECREF((PyObject *)__pyx_v_u1);
  __Pyx_XDECREF((PyObject *)__pyx_v_qnew);
  __Pyx_XDECREF((PyObject *)__pyx_v_rnew);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/linalg/_decomp_update.pyx":2159
 * 
 * @cython.embedsignature(True)
 * def qr_update(Q, R, u, v, overwrite_qruv=False, check_finite=True):             # <<<<<<<<<<<<<<
 *     """
 *     Rank-k QR update
 */

/* Python wrapper */
static PyObject *__pyx_pw_5scipy_6linalg_14_decomp_update_7qr_update(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5scipy_6linalg_14_decomp_update_6qr_update[] = "qr_update(Q, R, u, v, overwrite_qruv=False, check_finite=True)\n\n    Rank-k QR update\n\n    If ``A = Q R`` is the QR factorization of ``A``, return the QR\n    factorization of ``A + u v**T`` for real ``A`` or ``A + u v**H``\n    for complex ``A``.\n\n    Parameters\n    ----------\n    Q : (M, M) or (M, N) array_like\n        Unitary/orthogonal matrix from the qr decomposition of A.\n    R : (M, N) or (N, N) array_like\n        Upper triangular matrix from the qr decomposition of A.\n    u : (M,) or (M, k) array_like\n        Left update vector\n    v : (N,) or (N, k) array_like\n        Right update vector\n    overwrite_qruv : bool, optional\n        If True, consume Q, R, u, and v, if possible, while performing the\n        update, otherwise make copies as necessary. Defaults to False.\n    check_finite : bool, optional\n        Whether to check that the input matrix contains only finite numbers.\n        Disabling may give a performance gain, but may result in problems\n        (crashes, non-termination) if the inputs do contain infinities or NaNs.\n        Default is True.\n\n    Returns\n    -------\n    Q1 : ndarray\n        Updated unitary/orthogonal factor\n    R1 : ndarray\n        Updated upper triangular factor\n\n    See Also\n    --------\n    qr, qr_multiply, qr_delete, qr_insert\n\n    Notes\n    -----\n    This routine does not guarantee that the diagonal entries of `R1` are\n    real or positive.\n\n    .. versionadded:: 0.16.0\n\n    References\n    ----------\n    .. [1] Golub, G. H. & Van Loan, C. F. Matrix Computations, 3rd Ed.\n           (Johns Hopkins University Press, 1996).\n\n    .. [2] Daniel, J. W., Gragg, W. B., Kaufman, L. & Stewart, G. W.\n           Reorthogonalization and stable algorithms for updating the\n           Gram-Schmidt QR factorization. Math. Comput. 30, 772-795 (1976).\n\n    .. [3] Reichel, L. & Gragg, W. B. Algorithm 686: FORTRAN Subroutines for\n           Updating the QR Decomposition. ACM Trans. Math. Softw. 16"", 369-377\n           (1990).\n\n    Examples\n    --------\n    >>> from scipy import linalg\n    >>> a = np.array([[  3.,  -2.,  -2.],\n    ...               [  6.,  -9.,  -3.],\n    ...               [ -3.,  10.,   1.],\n    ...               [  6.,  -7.,   4.],\n    ...               [  7.,   8.,  -6.]])\n    >>> q, r = linalg.qr(a)\n\n    Given this q, r decomposition, perform a rank 1 update.\n\n    >>> u = np.array([7., -2., 4., 3., 5.])\n    >>> v = np.array([1., 3., -5.])\n    >>> q_up, r_up = linalg.qr_update(q, r, u, v, False)\n    >>> q_up\n    array([[ 0.54073807,  0.18645997,  0.81707661, -0.02136616,  0.06902409],  # may vary (signs)\n           [ 0.21629523, -0.63257324,  0.06567893,  0.34125904, -0.65749222],\n           [ 0.05407381,  0.64757787, -0.12781284, -0.20031219, -0.72198188],\n           [ 0.48666426, -0.30466718, -0.27487277, -0.77079214,  0.0256951 ],\n           [ 0.64888568,  0.23001   , -0.4859845 ,  0.49883891,  0.20253783]])\n    >>> r_up\n    array([[ 18.49324201,  24.11691794, -44.98940746],  # may vary (signs)\n           [  0.        ,  31.95894662, -27.40998201],\n           [  0.        ,   0.        ,  -9.25451794],\n           [  0.        ,   0.        ,   0.        ],\n           [  0.        ,   0.        ,   0.        ]])\n\n    The update is equivalent, but faster than the following.\n\n    >>> a_up = a + np.outer(u, v)\n    >>> q_direct, r_direct = linalg.qr(a_up)\n\n    Check that we have equivalent results:\n\n    >>> np.allclose(np.dot(q_up, r_up), a_up)\n    True\n\n    And the updated Q is still unitary:\n\n    >>> np.allclose(np.dot(q_up.T, q_up), np.eye(5))\n    True\n\n    Updating economic (reduced, thin) decompositions is also possible:\n\n    >>> qe, re = linalg.qr(a, mode='economic')\n    >>> qe_up, re_up = linalg.qr_update(qe, re, u, v, False)\n    >>> qe_up\n    array([[ 0.54073807,  0.18645997,  0.81707661],  # may vary (signs)\n           [ 0.21629523, -0.63257324,  0.06567893],\n           [ 0.0540738""1,  0.64757787, -0.12781284],\n           [ 0.48666426, -0.30466718, -0.27487277],\n           [ 0.64888568,  0.23001   , -0.4859845 ]])\n    >>> re_up\n    array([[ 18.49324201,  24.11691794, -44.98940746],  # may vary (signs)\n           [  0.        ,  31.95894662, -27.40998201],\n           [  0.        ,   0.        ,  -9.25451794]])\n    >>> np.allclose(np.dot(qe_up, re_up), a_up)\n    True\n    >>> np.allclose(np.dot(qe_up.T, qe_up), np.eye(3))\n    True\n\n    Similarly to the above, perform a rank 2 update.\n\n    >>> u2 = np.array([[ 7., -1,],\n    ...                [-2.,  4.],\n    ...                [ 4.,  2.],\n    ...                [ 3., -6.],\n    ...                [ 5.,  3.]])\n    >>> v2 = np.array([[ 1., 2.],\n    ...                [ 3., 4.],\n    ...                [-5., 2]])\n    >>> q_up2, r_up2 = linalg.qr_update(q, r, u2, v2, False)\n    >>> q_up2\n    array([[-0.33626508, -0.03477253,  0.61956287, -0.64352987, -0.29618884],  # may vary (signs)\n           [-0.50439762,  0.58319694, -0.43010077, -0.33395279,  0.33008064],\n           [-0.21016568, -0.63123106,  0.0582249 , -0.13675572,  0.73163206],\n           [ 0.12609941,  0.49694436,  0.64590024,  0.31191919,  0.47187344],\n           [-0.75659643, -0.11517748,  0.10284903,  0.5986227 , -0.21299983]])\n    >>> r_up2\n    array([[-23.79075451, -41.1084062 ,  24.71548348],  # may vary (signs)\n           [  0.        , -33.83931057,  11.02226551],\n           [  0.        ,   0.        ,  48.91476811],\n           [  0.        ,   0.        ,   0.        ],\n           [  0.        ,   0.        ,   0.        ]])\n\n    This update is also a valid qr decomposition of ``A + U V**T``.\n\n    >>> a_up2 = a + np.dot(u2, v2.T)\n    >>> np.allclose(a_up2, np.dot(q_up2, r_up2))\n    True\n    >>> np.allclose(np.dot(q_up2.T, q_up2), np.eye(5))\n    True\n\n    ";
static PyMethodDef __pyx_mdef_5scipy_6linalg_14_decomp_update_7qr_update = {"qr_update", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5scipy_6linalg_14_decomp_update_7qr_update, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5scipy_6linalg_14_decomp_update_6qr_update};
static PyObject *__pyx_pw_5scipy_6linalg_14_decomp_update_7qr_update(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_Q = 0;
  PyObject *__pyx_v_R = 0;
  PyObject *__pyx_v_u = 0;
  PyObject *__pyx_v_v = 0;
  PyObject *__pyx_v_overwrite_qruv = 0;
  PyObject *__pyx_v_check_finite = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("qr_update (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_Q,&__pyx_n_s_R,&__pyx_n_s_u,&__pyx_n_s_v,&__pyx_n_s_overwrite_qruv,&__pyx_n_s_check_finite,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    values[4] = ((PyObject *)Py_False);
    values[5] = ((PyObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_Q)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_R)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("qr_update", 0, 4, 6, 1); __PYX_ERR(0, 2159, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_u)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("qr_update", 0, 4, 6, 2); __PYX_ERR(0, 2159, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_v)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("qr_update", 0, 4, 6, 3); __PYX_ERR(0, 2159, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_overwrite_qruv);
          if (value) { values[4] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_check_finite);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "qr_update") < 0)) __PYX_ERR(0, 2159, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_Q = values[0];
    __pyx_v_R = values[1];
    __pyx_v_u = values[2];
    __pyx_v_v = values[3];
    __pyx_v_overwrite_qruv = values[4];
    __pyx_v_check_finite = values[5];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("qr_update", 0, 4, 6, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 2159, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("scipy.linalg._decomp_update.qr_update", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5scipy_6linalg_14_decomp_update_6qr_update(__pyx_self, __pyx_v_Q, __pyx_v_R, __pyx_v_u, __pyx_v_v, __pyx_v_overwrite_qruv, __pyx_v_check_finite);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5scipy_6linalg_14_decomp_update_6qr_update(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_Q, PyObject *__pyx_v_R, PyObject *__pyx_v_u, PyObject *__pyx_v_v, PyObject *__pyx_v_overwrite_qruv, PyObject *__pyx_v_check_finite) {
  PyArrayObject *__pyx_v_q1 = 0;
  PyArrayObject *__pyx_v_r1 = 0;
  PyArrayObject *__pyx_v_u1 = 0;
  PyArrayObject *__pyx_v_v1 = 0;
  PyArrayObject *__pyx_v_qTu = 0;
  PyArrayObject *__pyx_v_s = 0;
  int __pyx_v_uv_flags;
  int __pyx_v_typecode;
  int __pyx_v_p;
  int __pyx_v_m;
  int __pyx_v_n;
  int __pyx_v_info;
  void *__pyx_v_qptr;
  void *__pyx_v_rptr;
  void *__pyx_v_uptr;
  void *__pyx_v_vptr;
  void *__pyx_v_sptr;
  void *__pyx_v_qTuptr;
  int __pyx_v_qs[2];
  int __pyx_v_rs[2];
  int __pyx_v_qTus[2];
  int __pyx_v_us[2];
  int __pyx_v_vs[2];
  int __pyx_v_ss[2];
  int __pyx_v_economic;
  int __pyx_v_qisF;
  int __pyx_v_chkfinite;
  int __pyx_v_overwrite;
  npy_intp __pyx_v_ndim;
  npy_intp __pyx_v_length;
  PyObject *__pyx_v_rnew = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  __Pyx_RefNannySetupContext("qr_update", 0);

  /* "scipy/linalg/_decomp_update.pyx":2313
 *     """
 *     cdef cnp.ndarray q1, r1, u1, v1, qTu, s
 *     cdef int uv_flags = cnp.NPY_BEHAVED_NS | cnp.NPY_ELEMENTSTRIDES             # <<<<<<<<<<<<<<
 *     cdef int typecode, p, m, n, info
 *     cdef void* qptr
 */
  __pyx_v_uv_flags = (NPY_BEHAVED_NS | NPY_ELEMENTSTRIDES);

  /* "scipy/linalg/_decomp_update.pyx":2327
 *     cdef int vs[2]
 *     cdef int ss[2]
 *     cdef bint economic, qisF = False             # <<<<<<<<<<<<<<
 *     cdef bint chkfinite = check_finite, overwrite = overwrite_qruv
 *     cdef cnp.npy_intp ndim, length
 */
  __pyx_v_qisF = 0;

  /* "scipy/linalg/_decomp_update.pyx":2328
 *     cdef int ss[2]
 *     cdef bint economic, qisF = False
 *     cdef bint chkfinite = check_finite, overwrite = overwrite_qruv             # <<<<<<<<<<<<<<
 *     cdef cnp.npy_intp ndim, length
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_check_finite); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2328, __pyx_L1_error)
  __pyx_v_chkfinite = __pyx_t_1;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_overwrite_qruv); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2328, __pyx_L1_error)
  __pyx_v_overwrite = __pyx_t_1;

  /* "scipy/linalg/_decomp_update.pyx":2333
 *     # Rather than overspecify our order requirements on Q and R, let anything
 *     # through then adjust.
 *     q1, r1, typecode, m, n, economic = validate_qr(Q, R, overwrite, ARRAY_ANYORDER,             # <<<<<<<<<<<<<<
 *             overwrite, ARRAY_ANYORDER, chkfinite)
 * 
 */
  __pyx_t_2 = __pyx_f_5scipy_6linalg_14_decomp_update_validate_qr(__pyx_v_Q, __pyx_v_R, __pyx_v_overwrite, __pyx_v_5scipy_6linalg_14_decomp_update_ARRAY_ANYORDER, __pyx_v_overwrite, __pyx_v_5scipy_6linalg_14_decomp_update_ARRAY_ANYORDER, __pyx_v_chkfinite); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2333, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (likely(__pyx_t_2 != Py_None)) {
    PyObject* sequence = __pyx_t_2;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 6)) {
      if (size > 6) __Pyx_RaiseTooManyValuesError(6);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 2333, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
    __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
    __pyx_t_5 = PyTuple_GET_ITEM(sequence, 2); 
    __pyx_t_6 = PyTuple_GET_ITEM(sequence, 3); 
    __pyx_t_7 = PyTuple_GET_ITEM(sequence, 4); 
    __pyx_t_8 = PyTuple_GET_ITEM(sequence, 5); 
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_8);
    #else
    {
      Py_ssize_t i;
      PyObject** temps[6] = {&__pyx_t_3,&__pyx_t_4,&__pyx_t_5,&__pyx_t_6,&__pyx_t_7,&__pyx_t_8};
      for (i=0; i < 6; i++) {
        PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 2333, __pyx_L1_error)
        __Pyx_GOTREF(item);
        *(temps[i]) = item;
      }
    }
    #endif
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(0, 2333, __pyx_L1_error)
  }
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2333, __pyx_L1_error)
  if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2333, __pyx_L1_error)
  __pyx_t_9 = __Pyx_PyInt_As_int(__pyx_t_5); if (unlikely((__pyx_t_9 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2333, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_10 = __Pyx_PyInt_As_int(__pyx_t_6); if (unlikely((__pyx_t_10 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2333, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_11 = __Pyx_PyInt_As_int(__pyx_t_7); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2333, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2333, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_v_q1 = ((PyArrayObject *)__pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_v_r1 = ((PyArrayObject *)__pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_v_typecode = __pyx_t_9;
  __pyx_v_m = __pyx_t_10;
  __pyx_v_n = __pyx_t_11;
  __pyx_v_economic = __pyx_t_1;

  /* "scipy/linalg/_decomp_update.pyx":2336
 *             overwrite, ARRAY_ANYORDER, chkfinite)
 * 
 *     if not overwrite:             # <<<<<<<<<<<<<<
 *         uv_flags |= cnp.NPY_ENSURECOPY
 *     u1 = PyArray_CheckFromAny(u, NULL, 0, 0, uv_flags, NULL)
 */
  __pyx_t_1 = ((!(__pyx_v_overwrite != 0)) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":2337
 * 
 *     if not overwrite:
 *         uv_flags |= cnp.NPY_ENSURECOPY             # <<<<<<<<<<<<<<
 *     u1 = PyArray_CheckFromAny(u, NULL, 0, 0, uv_flags, NULL)
 *     v1 = PyArray_CheckFromAny(v, NULL, 0, 0, uv_flags, NULL)
 */
    __pyx_v_uv_flags = (__pyx_v_uv_flags | NPY_ENSURECOPY);

    /* "scipy/linalg/_decomp_update.pyx":2336
 *             overwrite, ARRAY_ANYORDER, chkfinite)
 * 
 *     if not overwrite:             # <<<<<<<<<<<<<<
 *         uv_flags |= cnp.NPY_ENSURECOPY
 *     u1 = PyArray_CheckFromAny(u, NULL, 0, 0, uv_flags, NULL)
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":2338
 *     if not overwrite:
 *         uv_flags |= cnp.NPY_ENSURECOPY
 *     u1 = PyArray_CheckFromAny(u, NULL, 0, 0, uv_flags, NULL)             # <<<<<<<<<<<<<<
 *     v1 = PyArray_CheckFromAny(v, NULL, 0, 0, uv_flags, NULL)
 * 
 */
  __pyx_t_2 = ((PyObject *)PyArray_CheckFromAny(__pyx_v_u, NULL, 0, 0, __pyx_v_uv_flags, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2338, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_u1 = ((PyArrayObject *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "scipy/linalg/_decomp_update.pyx":2339
 *         uv_flags |= cnp.NPY_ENSURECOPY
 *     u1 = PyArray_CheckFromAny(u, NULL, 0, 0, uv_flags, NULL)
 *     v1 = PyArray_CheckFromAny(v, NULL, 0, 0, uv_flags, NULL)             # <<<<<<<<<<<<<<
 * 
 *     if cnp.PyArray_TYPE(u1) != typecode or cnp.PyArray_TYPE(v1) != typecode:
 */
  __pyx_t_2 = ((PyObject *)PyArray_CheckFromAny(__pyx_v_v, NULL, 0, 0, __pyx_v_uv_flags, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2339, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_v1 = ((PyArrayObject *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "scipy/linalg/_decomp_update.pyx":2341
 *     v1 = PyArray_CheckFromAny(v, NULL, 0, 0, uv_flags, NULL)
 * 
 *     if cnp.PyArray_TYPE(u1) != typecode or cnp.PyArray_TYPE(v1) != typecode:             # <<<<<<<<<<<<<<
 *         raise ValueError('u and v must have the same type as Q and R')
 * 
 */
  __pyx_t_12 = ((PyArray_TYPE(__pyx_v_u1) != __pyx_v_typecode) != 0);
  if (!__pyx_t_12) {
  } else {
    __pyx_t_1 = __pyx_t_12;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_12 = ((PyArray_TYPE(__pyx_v_v1) != __pyx_v_typecode) != 0);
  __pyx_t_1 = __pyx_t_12;
  __pyx_L5_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "scipy/linalg/_decomp_update.pyx":2342
 * 
 *     if cnp.PyArray_TYPE(u1) != typecode or cnp.PyArray_TYPE(v1) != typecode:
 *         raise ValueError('u and v must have the same type as Q and R')             # <<<<<<<<<<<<<<
 * 
 *     if u1.ndim < 1 or u1.ndim > 2:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__24, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2342, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 2342, __pyx_L1_error)

    /* "scipy/linalg/_decomp_update.pyx":2341
 *     v1 = PyArray_CheckFromAny(v, NULL, 0, 0, uv_flags, NULL)
 * 
 *     if cnp.PyArray_TYPE(u1) != typecode or cnp.PyArray_TYPE(v1) != typecode:             # <<<<<<<<<<<<<<
 *         raise ValueError('u and v must have the same type as Q and R')
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":2344
 *         raise ValueError('u and v must have the same type as Q and R')
 * 
 *     if u1.ndim < 1 or u1.ndim > 2:             # <<<<<<<<<<<<<<
 *         raise ValueError('u must be either 1- or 2-D.')
 * 
 */
  __pyx_t_12 = ((__pyx_v_u1->nd < 1) != 0);
  if (!__pyx_t_12) {
  } else {
    __pyx_t_1 = __pyx_t_12;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_12 = ((__pyx_v_u1->nd > 2) != 0);
  __pyx_t_1 = __pyx_t_12;
  __pyx_L8_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "scipy/linalg/_decomp_update.pyx":2345
 * 
 *     if u1.ndim < 1 or u1.ndim > 2:
 *         raise ValueError('u must be either 1- or 2-D.')             # <<<<<<<<<<<<<<
 * 
 *     if v1.ndim < 1 or v1.ndim > 2:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__25, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2345, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 2345, __pyx_L1_error)

    /* "scipy/linalg/_decomp_update.pyx":2344
 *         raise ValueError('u and v must have the same type as Q and R')
 * 
 *     if u1.ndim < 1 or u1.ndim > 2:             # <<<<<<<<<<<<<<
 *         raise ValueError('u must be either 1- or 2-D.')
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":2347
 *         raise ValueError('u must be either 1- or 2-D.')
 * 
 *     if v1.ndim < 1 or v1.ndim > 2:             # <<<<<<<<<<<<<<
 *         raise ValueError('v must be either 1- or 2-D.')
 * 
 */
  __pyx_t_12 = ((__pyx_v_v1->nd < 1) != 0);
  if (!__pyx_t_12) {
  } else {
    __pyx_t_1 = __pyx_t_12;
    goto __pyx_L11_bool_binop_done;
  }
  __pyx_t_12 = ((__pyx_v_v1->nd > 2) != 0);
  __pyx_t_1 = __pyx_t_12;
  __pyx_L11_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "scipy/linalg/_decomp_update.pyx":2348
 * 
 *     if v1.ndim < 1 or v1.ndim > 2:
 *         raise ValueError('v must be either 1- or 2-D.')             # <<<<<<<<<<<<<<
 * 
 *     if u1.shape[0] != m:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2348, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 2348, __pyx_L1_error)

    /* "scipy/linalg/_decomp_update.pyx":2347
 *         raise ValueError('u must be either 1- or 2-D.')
 * 
 *     if v1.ndim < 1 or v1.ndim > 2:             # <<<<<<<<<<<<<<
 *         raise ValueError('v must be either 1- or 2-D.')
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":2350
 *         raise ValueError('v must be either 1- or 2-D.')
 * 
 *     if u1.shape[0] != m:             # <<<<<<<<<<<<<<
 *         raise ValueError('u.shape[0] must equal Q.shape[0]')
 * 
 */
  __pyx_t_1 = (((__pyx_v_u1->dimensions[0]) != __pyx_v_m) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "scipy/linalg/_decomp_update.pyx":2351
 * 
 *     if u1.shape[0] != m:
 *         raise ValueError('u.shape[0] must equal Q.shape[0]')             # <<<<<<<<<<<<<<
 * 
 *     if v1.shape[0] != n:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__27, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2351, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 2351, __pyx_L1_error)

    /* "scipy/linalg/_decomp_update.pyx":2350
 *         raise ValueError('v must be either 1- or 2-D.')
 * 
 *     if u1.shape[0] != m:             # <<<<<<<<<<<<<<
 *         raise ValueError('u.shape[0] must equal Q.shape[0]')
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":2353
 *         raise ValueError('u.shape[0] must equal Q.shape[0]')
 * 
 *     if v1.shape[0] != n:             # <<<<<<<<<<<<<<
 *         raise ValueError('v.shape[0] must equal R.shape[1]')
 * 
 */
  __pyx_t_1 = (((__pyx_v_v1->dimensions[0]) != __pyx_v_n) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "scipy/linalg/_decomp_update.pyx":2354
 * 
 *     if v1.shape[0] != n:
 *         raise ValueError('v.shape[0] must equal R.shape[1]')             # <<<<<<<<<<<<<<
 * 
 *     if u1.ndim != v1.ndim:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__28, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2354, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 2354, __pyx_L1_error)

    /* "scipy/linalg/_decomp_update.pyx":2353
 *         raise ValueError('u.shape[0] must equal Q.shape[0]')
 * 
 *     if v1.shape[0] != n:             # <<<<<<<<<<<<<<
 *         raise ValueError('v.shape[0] must equal R.shape[1]')
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":2356
 *         raise ValueError('v.shape[0] must equal R.shape[1]')
 * 
 *     if u1.ndim != v1.ndim:             # <<<<<<<<<<<<<<
 *         raise ValueError('u and v must have the same number of dimensions')
 * 
 */
  __pyx_t_1 = ((__pyx_v_u1->nd != __pyx_v_v1->nd) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "scipy/linalg/_decomp_update.pyx":2357
 * 
 *     if u1.ndim != v1.ndim:
 *         raise ValueError('u and v must have the same number of dimensions')             # <<<<<<<<<<<<<<
 * 
 *     if u1.ndim == 2:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__29, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2357, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 2357, __pyx_L1_error)

    /* "scipy/linalg/_decomp_update.pyx":2356
 *         raise ValueError('v.shape[0] must equal R.shape[1]')
 * 
 *     if u1.ndim != v1.ndim:             # <<<<<<<<<<<<<<
 *         raise ValueError('u and v must have the same number of dimensions')
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":2359
 *         raise ValueError('u and v must have the same number of dimensions')
 * 
 *     if u1.ndim == 2:             # <<<<<<<<<<<<<<
 *         if u1.shape[1] != v1.shape[1]:
 *             raise ValueError('Second dimension of u and v must be the same')
 */
  __pyx_t_1 = ((__pyx_v_u1->nd == 2) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":2360
 * 
 *     if u1.ndim == 2:
 *         if u1.shape[1] != v1.shape[1]:             # <<<<<<<<<<<<<<
 *             raise ValueError('Second dimension of u and v must be the same')
 *         elif u1.shape[1] == 1:
 */
    __pyx_t_1 = (((__pyx_v_u1->dimensions[1]) != (__pyx_v_v1->dimensions[1])) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "scipy/linalg/_decomp_update.pyx":2361
 *     if u1.ndim == 2:
 *         if u1.shape[1] != v1.shape[1]:
 *             raise ValueError('Second dimension of u and v must be the same')             # <<<<<<<<<<<<<<
 *         elif u1.shape[1] == 1:
 *             u1 = u1[:,0]
 */
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__30, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2361, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 2361, __pyx_L1_error)

      /* "scipy/linalg/_decomp_update.pyx":2360
 * 
 *     if u1.ndim == 2:
 *         if u1.shape[1] != v1.shape[1]:             # <<<<<<<<<<<<<<
 *             raise ValueError('Second dimension of u and v must be the same')
 *         elif u1.shape[1] == 1:
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":2362
 *         if u1.shape[1] != v1.shape[1]:
 *             raise ValueError('Second dimension of u and v must be the same')
 *         elif u1.shape[1] == 1:             # <<<<<<<<<<<<<<
 *             u1 = u1[:,0]
 *             v1 = v1[:,0]
 */
    __pyx_t_1 = (((__pyx_v_u1->dimensions[1]) == 1) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":2363
 *             raise ValueError('Second dimension of u and v must be the same')
 *         elif u1.shape[1] == 1:
 *             u1 = u1[:,0]             # <<<<<<<<<<<<<<
 *             v1 = v1[:,0]
 * 
 */
      __pyx_t_2 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_u1), __pyx_tuple__31); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2363, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2363, __pyx_L1_error)
      __Pyx_DECREF_SET(__pyx_v_u1, ((PyArrayObject *)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "scipy/linalg/_decomp_update.pyx":2364
 *         elif u1.shape[1] == 1:
 *             u1 = u1[:,0]
 *             v1 = v1[:,0]             # <<<<<<<<<<<<<<
 * 
 *     if u1.ndim == 1:
 */
      __pyx_t_2 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_v1), __pyx_tuple__31); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2364, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2364, __pyx_L1_error)
      __Pyx_DECREF_SET(__pyx_v_v1, ((PyArrayObject *)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "scipy/linalg/_decomp_update.pyx":2362
 *         if u1.shape[1] != v1.shape[1]:
 *             raise ValueError('Second dimension of u and v must be the same')
 *         elif u1.shape[1] == 1:             # <<<<<<<<<<<<<<
 *             u1 = u1[:,0]
 *             v1 = v1[:,0]
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":2359
 *         raise ValueError('u and v must have the same number of dimensions')
 * 
 *     if u1.ndim == 2:             # <<<<<<<<<<<<<<
 *         if u1.shape[1] != v1.shape[1]:
 *             raise ValueError('Second dimension of u and v must be the same')
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":2366
 *             v1 = v1[:,0]
 * 
 *     if u1.ndim == 1:             # <<<<<<<<<<<<<<
 *         p = 1
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_u1->nd == 1) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":2367
 * 
 *     if u1.ndim == 1:
 *         p = 1             # <<<<<<<<<<<<<<
 *     else:
 *         p = u1.shape[1]
 */
    __pyx_v_p = 1;

    /* "scipy/linalg/_decomp_update.pyx":2366
 *             v1 = v1[:,0]
 * 
 *     if u1.ndim == 1:             # <<<<<<<<<<<<<<
 *         p = 1
 *     else:
 */
    goto __pyx_L18;
  }

  /* "scipy/linalg/_decomp_update.pyx":2369
 *         p = 1
 *     else:
 *         p = u1.shape[1]             # <<<<<<<<<<<<<<
 * 
 *     # limit p to at most max(n, m)
 */
  /*else*/ {
    __pyx_v_p = (__pyx_v_u1->dimensions[1]);
  }
  __pyx_L18:;

  /* "scipy/linalg/_decomp_update.pyx":2372
 * 
 *     # limit p to at most max(n, m)
 *     if p > n or p > m:             # <<<<<<<<<<<<<<
 *         raise ValueError('Update rank larger than np.dot(Q, R).')
 * 
 */
  __pyx_t_12 = ((__pyx_v_p > __pyx_v_n) != 0);
  if (!__pyx_t_12) {
  } else {
    __pyx_t_1 = __pyx_t_12;
    goto __pyx_L20_bool_binop_done;
  }
  __pyx_t_12 = ((__pyx_v_p > __pyx_v_m) != 0);
  __pyx_t_1 = __pyx_t_12;
  __pyx_L20_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "scipy/linalg/_decomp_update.pyx":2373
 *     # limit p to at most max(n, m)
 *     if p > n or p > m:
 *         raise ValueError('Update rank larger than np.dot(Q, R).')             # <<<<<<<<<<<<<<
 * 
 *     u1 = validate_array(u1, chkfinite)
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__32, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2373, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 2373, __pyx_L1_error)

    /* "scipy/linalg/_decomp_update.pyx":2372
 * 
 *     # limit p to at most max(n, m)
 *     if p > n or p > m:             # <<<<<<<<<<<<<<
 *         raise ValueError('Update rank larger than np.dot(Q, R).')
 * 
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":2375
 *         raise ValueError('Update rank larger than np.dot(Q, R).')
 * 
 *     u1 = validate_array(u1, chkfinite)             # <<<<<<<<<<<<<<
 *     v1 = validate_array(v1, chkfinite)
 * 
 */
  __pyx_t_2 = __pyx_f_5scipy_6linalg_14_decomp_update_validate_array(__pyx_v_u1, __pyx_v_chkfinite); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2375, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2375, __pyx_L1_error)
  __Pyx_DECREF_SET(__pyx_v_u1, ((PyArrayObject *)__pyx_t_2));
  __pyx_t_2 = 0;

  /* "scipy/linalg/_decomp_update.pyx":2376
 * 
 *     u1 = validate_array(u1, chkfinite)
 *     v1 = validate_array(v1, chkfinite)             # <<<<<<<<<<<<<<
 * 
 *     # special case 1xN
 */
  __pyx_t_2 = __pyx_f_5scipy_6linalg_14_decomp_update_validate_array(__pyx_v_v1, __pyx_v_chkfinite); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2376, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2376, __pyx_L1_error)
  __Pyx_DECREF_SET(__pyx_v_v1, ((PyArrayObject *)__pyx_t_2));
  __pyx_t_2 = 0;

  /* "scipy/linalg/_decomp_update.pyx":2382
 *     # we only need consider rank 1 updates, since we have
 *     # limited p to max(m,n) above.
 *     if m == 1:             # <<<<<<<<<<<<<<
 *         rnew = r1 + q1.conjugate()*u1.ravel()*v1.conjugate().ravel()
 *         return q1.copy(), rnew
 */
  __pyx_t_1 = ((__pyx_v_m == 1) != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":2383
 *     # limited p to max(m,n) above.
 *     if m == 1:
 *         rnew = r1 + q1.conjugate()*u1.ravel()*v1.conjugate().ravel()             # <<<<<<<<<<<<<<
 *         return q1.copy(), rnew
 * 
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_q1), __pyx_n_s_conjugate); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2383, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7) : __Pyx_PyObject_CallNoArg(__pyx_t_8);
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2383, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_u1), __pyx_n_s_ravel); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2383, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_8 = (__pyx_t_6) ? __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6) : __Pyx_PyObject_CallNoArg(__pyx_t_7);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2383, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = PyNumber_Multiply(__pyx_t_2, __pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2383, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_v1), __pyx_n_s_conjugate); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2383, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
      }
    }
    __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_6);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2383, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_ravel); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2383, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
      }
    }
    __pyx_t_8 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_6);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2383, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyNumber_Multiply(__pyx_t_7, __pyx_t_8); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2383, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = PyNumber_Add(((PyObject *)__pyx_v_r1), __pyx_t_6); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2383, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_rnew = __pyx_t_8;
    __pyx_t_8 = 0;

    /* "scipy/linalg/_decomp_update.pyx":2384
 *     if m == 1:
 *         rnew = r1 + q1.conjugate()*u1.ravel()*v1.conjugate().ravel()
 *         return q1.copy(), rnew             # <<<<<<<<<<<<<<
 * 
 *     if economic:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_q1), __pyx_n_s_copy); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2384, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
      }
    }
    __pyx_t_8 = (__pyx_t_7) ? __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_7) : __Pyx_PyObject_CallNoArg(__pyx_t_6);
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2384, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2384, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_8);
    __Pyx_INCREF(__pyx_v_rnew);
    __Pyx_GIVEREF(__pyx_v_rnew);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_rnew);
    __pyx_t_8 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "scipy/linalg/_decomp_update.pyx":2382
 *     # we only need consider rank 1 updates, since we have
 *     # limited p to max(m,n) above.
 *     if m == 1:             # <<<<<<<<<<<<<<
 *         rnew = r1 + q1.conjugate()*u1.ravel()*v1.conjugate().ravel()
 *         return q1.copy(), rnew
 */
  }

  /* "scipy/linalg/_decomp_update.pyx":2386
 *         return q1.copy(), rnew
 * 
 *     if economic:             # <<<<<<<<<<<<<<
 *         ndim = 1
 *         length = 2*n
 */
  __pyx_t_1 = (__pyx_v_economic != 0);
  if (__pyx_t_1) {

    /* "scipy/linalg/_decomp_update.pyx":2387
 * 
 *     if economic:
 *         ndim = 1             # <<<<<<<<<<<<<<
 *         length = 2*n
 *         s = cnp.PyArray_ZEROS(ndim, &length, typecode, 1)
 */
    __pyx_v_ndim = 1;

    /* "scipy/linalg/_decomp_update.pyx":2388
 *     if economic:
 *         ndim = 1
 *         length = 2*n             # <<<<<<<<<<<<<<
 *         s = cnp.PyArray_ZEROS(ndim, &length, typecode, 1)
 *         if not cnp.PyArray_ISONESEGMENT(q1):
 */
    __pyx_v_length = (2 * __pyx_v_n);

    /* "scipy/linalg/_decomp_update.pyx":2389
 *         ndim = 1
 *         length = 2*n
 *         s = cnp.PyArray_ZEROS(ndim, &length, typecode, 1)             # <<<<<<<<<<<<<<
 *         if not cnp.PyArray_ISONESEGMENT(q1):
 *             q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 */
    __pyx_t_6 = PyArray_ZEROS(__pyx_v_ndim, (&__pyx_v_length), __pyx_v_typecode, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2389, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2389, __pyx_L1_error)
    __pyx_v_s = ((PyArrayObject *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "scipy/linalg/_decomp_update.pyx":2390
 *         length = 2*n
 *         s = cnp.PyArray_ZEROS(ndim, &length, typecode, 1)
 *         if not cnp.PyArray_ISONESEGMENT(q1):             # <<<<<<<<<<<<<<
 *             q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             qisF = True
 */
    __pyx_t_1 = ((!(PyArray_ISONESEGMENT(__pyx_v_q1) != 0)) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":2391
 *         s = cnp.PyArray_ZEROS(ndim, &length, typecode, 1)
 *         if not cnp.PyArray_ISONESEGMENT(q1):
 *             q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *             qisF = True
 *         elif cnp.PyArray_CHKFLAGS(q1, cnp.NPY_F_CONTIGUOUS):
 */
      __pyx_t_6 = ((PyObject *)PyArray_FromArray(__pyx_v_q1, NULL, NPY_F_CONTIGUOUS)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2391, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF_SET(__pyx_v_q1, ((PyArrayObject *)__pyx_t_6));
      __pyx_t_6 = 0;

      /* "scipy/linalg/_decomp_update.pyx":2392
 *         if not cnp.PyArray_ISONESEGMENT(q1):
 *             q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             qisF = True             # <<<<<<<<<<<<<<
 *         elif cnp.PyArray_CHKFLAGS(q1, cnp.NPY_F_CONTIGUOUS):
 *             qisF = True
 */
      __pyx_v_qisF = 1;

      /* "scipy/linalg/_decomp_update.pyx":2390
 *         length = 2*n
 *         s = cnp.PyArray_ZEROS(ndim, &length, typecode, 1)
 *         if not cnp.PyArray_ISONESEGMENT(q1):             # <<<<<<<<<<<<<<
 *             q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             qisF = True
 */
      goto __pyx_L24;
    }

    /* "scipy/linalg/_decomp_update.pyx":2393
 *             q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             qisF = True
 *         elif cnp.PyArray_CHKFLAGS(q1, cnp.NPY_F_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *             qisF = True
 *         else:
 */
    __pyx_t_1 = (PyArray_CHKFLAGS(__pyx_v_q1, NPY_F_CONTIGUOUS) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":2394
 *             qisF = True
 *         elif cnp.PyArray_CHKFLAGS(q1, cnp.NPY_F_CONTIGUOUS):
 *             qisF = True             # <<<<<<<<<<<<<<
 *         else:
 *             qisF = False
 */
      __pyx_v_qisF = 1;

      /* "scipy/linalg/_decomp_update.pyx":2393
 *             q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             qisF = True
 *         elif cnp.PyArray_CHKFLAGS(q1, cnp.NPY_F_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *             qisF = True
 *         else:
 */
      goto __pyx_L24;
    }

    /* "scipy/linalg/_decomp_update.pyx":2396
 *             qisF = True
 *         else:
 *             qisF = False             # <<<<<<<<<<<<<<
 *         qptr = extract(q1, qs)
 *         rptr = extract(r1, rs)
 */
    /*else*/ {
      __pyx_v_qisF = 0;
    }
    __pyx_L24:;

    /* "scipy/linalg/_decomp_update.pyx":2397
 *         else:
 *             qisF = False
 *         qptr = extract(q1, qs)             # <<<<<<<<<<<<<<
 *         rptr = extract(r1, rs)
 *         uptr = extract(u1, us)
 */
    __pyx_v_qptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_q1, __pyx_v_qs);

    /* "scipy/linalg/_decomp_update.pyx":2398
 *             qisF = False
 *         qptr = extract(q1, qs)
 *         rptr = extract(r1, rs)             # <<<<<<<<<<<<<<
 *         uptr = extract(u1, us)
 *         vptr = extract(v1, vs)
 */
    __pyx_v_rptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_r1, __pyx_v_rs);

    /* "scipy/linalg/_decomp_update.pyx":2399
 *         qptr = extract(q1, qs)
 *         rptr = extract(r1, rs)
 *         uptr = extract(u1, us)             # <<<<<<<<<<<<<<
 *         vptr = extract(v1, vs)
 *         sptr = extract(s, ss)
 */
    __pyx_v_uptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_u1, __pyx_v_us);

    /* "scipy/linalg/_decomp_update.pyx":2400
 *         rptr = extract(r1, rs)
 *         uptr = extract(u1, us)
 *         vptr = extract(v1, vs)             # <<<<<<<<<<<<<<
 *         sptr = extract(s, ss)
 *         with nogil:
 */
    __pyx_v_vptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_v1, __pyx_v_vs);

    /* "scipy/linalg/_decomp_update.pyx":2401
 *         uptr = extract(u1, us)
 *         vptr = extract(v1, vs)
 *         sptr = extract(s, ss)             # <<<<<<<<<<<<<<
 *         with nogil:
 *             if p == 1:
 */
    __pyx_v_sptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_s, __pyx_v_ss);

    /* "scipy/linalg/_decomp_update.pyx":2402
 *         vptr = extract(v1, vs)
 *         sptr = extract(s, ss)
 *         with nogil:             # <<<<<<<<<<<<<<
 *             if p == 1:
 *                 if typecode == cnp.NPY_FLOAT:
 */
    {
        #ifdef WITH_THREAD
        PyThreadState *_save;
        Py_UNBLOCK_THREADS
        __Pyx_FastGIL_Remember();
        #endif
        /*try:*/ {

          /* "scipy/linalg/_decomp_update.pyx":2403
 *         sptr = extract(s, ss)
 *         with nogil:
 *             if p == 1:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     thin_qr_rank_1_update(m, n, <float*>qptr, qs, qisF,
 */
          __pyx_t_1 = ((__pyx_v_p == 1) != 0);
          if (__pyx_t_1) {

            /* "scipy/linalg/_decomp_update.pyx":2404
 *         with nogil:
 *             if p == 1:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     thin_qr_rank_1_update(m, n, <float*>qptr, qs, qisF,
 *                         <float*>rptr, rs, <float*>uptr, us,
 */
            switch (__pyx_v_typecode) {
              case NPY_FLOAT:

              /* "scipy/linalg/_decomp_update.pyx":2405
 *             if p == 1:
 *                 if typecode == cnp.NPY_FLOAT:
 *                     thin_qr_rank_1_update(m, n, <float*>qptr, qs, qisF,             # <<<<<<<<<<<<<<
 *                         <float*>rptr, rs, <float*>uptr, us,
 *                         <float*>vptr, vs, <float*>sptr, ss)
 */
              __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_1_update(__pyx_v_m, __pyx_v_n, ((float *)__pyx_v_qptr), __pyx_v_qs, __pyx_v_qisF, ((float *)__pyx_v_rptr), __pyx_v_rs, ((float *)__pyx_v_uptr), __pyx_v_us, ((float *)__pyx_v_vptr), __pyx_v_vs, ((float *)__pyx_v_sptr), __pyx_v_ss);

              /* "scipy/linalg/_decomp_update.pyx":2404
 *         with nogil:
 *             if p == 1:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     thin_qr_rank_1_update(m, n, <float*>qptr, qs, qisF,
 *                         <float*>rptr, rs, <float*>uptr, us,
 */
              break;
              case NPY_DOUBLE:

              /* "scipy/linalg/_decomp_update.pyx":2409
 *                         <float*>vptr, vs, <float*>sptr, ss)
 *                 elif typecode == cnp.NPY_DOUBLE:
 *                     thin_qr_rank_1_update(m, n, <double*>qptr, qs, qisF,             # <<<<<<<<<<<<<<
 *                         <double*>rptr, rs, <double*>uptr, us,
 *                         <double*>vptr, vs, <double*>sptr, ss)
 */
              __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_1_update(__pyx_v_m, __pyx_v_n, ((double *)__pyx_v_qptr), __pyx_v_qs, __pyx_v_qisF, ((double *)__pyx_v_rptr), __pyx_v_rs, ((double *)__pyx_v_uptr), __pyx_v_us, ((double *)__pyx_v_vptr), __pyx_v_vs, ((double *)__pyx_v_sptr), __pyx_v_ss);

              /* "scipy/linalg/_decomp_update.pyx":2408
 *                         <float*>rptr, rs, <float*>uptr, us,
 *                         <float*>vptr, vs, <float*>sptr, ss)
 *                 elif typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                     thin_qr_rank_1_update(m, n, <double*>qptr, qs, qisF,
 *                         <double*>rptr, rs, <double*>uptr, us,
 */
              break;
              case NPY_CFLOAT:

              /* "scipy/linalg/_decomp_update.pyx":2413
 *                         <double*>vptr, vs, <double*>sptr, ss)
 *                 elif typecode == cnp.NPY_CFLOAT:
 *                     thin_qr_rank_1_update(m, n, <float_complex*>qptr, qs, qisF,             # <<<<<<<<<<<<<<
 *                         <float_complex*>rptr, rs, <float_complex*>uptr, us,
 *                         <float_complex*>vptr, vs, <float_complex*>sptr, ss)
 */
              __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_1_update(__pyx_v_m, __pyx_v_n, ((__pyx_t_float_complex *)__pyx_v_qptr), __pyx_v_qs, __pyx_v_qisF, ((__pyx_t_float_complex *)__pyx_v_rptr), __pyx_v_rs, ((__pyx_t_float_complex *)__pyx_v_uptr), __pyx_v_us, ((__pyx_t_float_complex *)__pyx_v_vptr), __pyx_v_vs, ((__pyx_t_float_complex *)__pyx_v_sptr), __pyx_v_ss);

              /* "scipy/linalg/_decomp_update.pyx":2412
 *                         <double*>rptr, rs, <double*>uptr, us,
 *                         <double*>vptr, vs, <double*>sptr, ss)
 *                 elif typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                     thin_qr_rank_1_update(m, n, <float_complex*>qptr, qs, qisF,
 *                         <float_complex*>rptr, rs, <float_complex*>uptr, us,
 */
              break;
              default:

              /* "scipy/linalg/_decomp_update.pyx":2417
 *                         <float_complex*>vptr, vs, <float_complex*>sptr, ss)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 *                     thin_qr_rank_1_update(m, n, <double_complex*>qptr, qs, qisF,             # <<<<<<<<<<<<<<
 *                         <double_complex*>rptr, rs, <double_complex*>uptr, us,
 *                         <double_complex*>vptr, vs, <double_complex*>sptr, ss)
 */
              __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_1_update(__pyx_v_m, __pyx_v_n, ((__pyx_t_double_complex *)__pyx_v_qptr), __pyx_v_qs, __pyx_v_qisF, ((__pyx_t_double_complex *)__pyx_v_rptr), __pyx_v_rs, ((__pyx_t_double_complex *)__pyx_v_uptr), __pyx_v_us, ((__pyx_t_double_complex *)__pyx_v_vptr), __pyx_v_vs, ((__pyx_t_double_complex *)__pyx_v_sptr), __pyx_v_ss);
              break;
            }

            /* "scipy/linalg/_decomp_update.pyx":2403
 *         sptr = extract(s, ss)
 *         with nogil:
 *             if p == 1:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     thin_qr_rank_1_update(m, n, <float*>qptr, qs, qisF,
 */
            goto __pyx_L28;
          }

          /* "scipy/linalg/_decomp_update.pyx":2421
 *                         <double_complex*>vptr, vs, <double_complex*>sptr, ss)
 *             else:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     thin_qr_rank_p_update(m, n, p, <float*>qptr, qs, qisF,
 *                         <float*>rptr, rs, <float*>uptr, us,
 */
          /*else*/ {

            /* "scipy/linalg/_decomp_update.pyx":2429
 *                         <double*>rptr, rs, <double*>uptr, us,
 *                         <double*>vptr, vs, <double*>sptr, ss)
 *                 elif typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                     thin_qr_rank_p_update(m, n, p, <float_complex*>qptr, qs, qisF,
 *                         <float_complex*>rptr, rs, <float_complex*>uptr, us,
 */
            switch (__pyx_v_typecode) {
              case NPY_FLOAT:

              /* "scipy/linalg/_decomp_update.pyx":2422
 *             else:
 *                 if typecode == cnp.NPY_FLOAT:
 *                     thin_qr_rank_p_update(m, n, p, <float*>qptr, qs, qisF,             # <<<<<<<<<<<<<<
 *                         <float*>rptr, rs, <float*>uptr, us,
 *                         <float*>vptr, vs, <float*>sptr, ss)
 */
              __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_p_update(__pyx_v_m, __pyx_v_n, __pyx_v_p, ((float *)__pyx_v_qptr), __pyx_v_qs, __pyx_v_qisF, ((float *)__pyx_v_rptr), __pyx_v_rs, ((float *)__pyx_v_uptr), __pyx_v_us, ((float *)__pyx_v_vptr), __pyx_v_vs, ((float *)__pyx_v_sptr), __pyx_v_ss);

              /* "scipy/linalg/_decomp_update.pyx":2421
 *                         <double_complex*>vptr, vs, <double_complex*>sptr, ss)
 *             else:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     thin_qr_rank_p_update(m, n, p, <float*>qptr, qs, qisF,
 *                         <float*>rptr, rs, <float*>uptr, us,
 */
              break;
              case NPY_DOUBLE:

              /* "scipy/linalg/_decomp_update.pyx":2426
 *                         <float*>vptr, vs, <float*>sptr, ss)
 *                 elif typecode == cnp.NPY_DOUBLE:
 *                     thin_qr_rank_p_update(m, n, p, <double*>qptr, qs, qisF,             # <<<<<<<<<<<<<<
 *                         <double*>rptr, rs, <double*>uptr, us,
 *                         <double*>vptr, vs, <double*>sptr, ss)
 */
              __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_p_update(__pyx_v_m, __pyx_v_n, __pyx_v_p, ((double *)__pyx_v_qptr), __pyx_v_qs, __pyx_v_qisF, ((double *)__pyx_v_rptr), __pyx_v_rs, ((double *)__pyx_v_uptr), __pyx_v_us, ((double *)__pyx_v_vptr), __pyx_v_vs, ((double *)__pyx_v_sptr), __pyx_v_ss);

              /* "scipy/linalg/_decomp_update.pyx":2425
 *                         <float*>rptr, rs, <float*>uptr, us,
 *                         <float*>vptr, vs, <float*>sptr, ss)
 *                 elif typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                     thin_qr_rank_p_update(m, n, p, <double*>qptr, qs, qisF,
 *                         <double*>rptr, rs, <double*>uptr, us,
 */
              break;
              case NPY_CFLOAT:

              /* "scipy/linalg/_decomp_update.pyx":2430
 *                         <double*>vptr, vs, <double*>sptr, ss)
 *                 elif typecode == cnp.NPY_CFLOAT:
 *                     thin_qr_rank_p_update(m, n, p, <float_complex*>qptr, qs, qisF,             # <<<<<<<<<<<<<<
 *                         <float_complex*>rptr, rs, <float_complex*>uptr, us,
 *                         <float_complex*>vptr, vs, <float_complex*>sptr, ss)
 */
              __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_p_update(__pyx_v_m, __pyx_v_n, __pyx_v_p, ((__pyx_t_float_complex *)__pyx_v_qptr), __pyx_v_qs, __pyx_v_qisF, ((__pyx_t_float_complex *)__pyx_v_rptr), __pyx_v_rs, ((__pyx_t_float_complex *)__pyx_v_uptr), __pyx_v_us, ((__pyx_t_float_complex *)__pyx_v_vptr), __pyx_v_vs, ((__pyx_t_float_complex *)__pyx_v_sptr), __pyx_v_ss);

              /* "scipy/linalg/_decomp_update.pyx":2429
 *                         <double*>rptr, rs, <double*>uptr, us,
 *                         <double*>vptr, vs, <double*>sptr, ss)
 *                 elif typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                     thin_qr_rank_p_update(m, n, p, <float_complex*>qptr, qs, qisF,
 *                         <float_complex*>rptr, rs, <float_complex*>uptr, us,
 */
              break;
              default:

              /* "scipy/linalg/_decomp_update.pyx":2434
 *                         <float_complex*>vptr, vs, <float_complex*>sptr, ss)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 *                     thin_qr_rank_p_update(m, n, p, <double_complex*>qptr, qs, qisF,             # <<<<<<<<<<<<<<
 *                         <double_complex*>rptr, rs, <double_complex*>uptr, us,
 *                         <double_complex*>vptr, vs, <double_complex*>sptr, ss)
 */
              __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_thin_qr_rank_p_update(__pyx_v_m, __pyx_v_n, __pyx_v_p, ((__pyx_t_double_complex *)__pyx_v_qptr), __pyx_v_qs, __pyx_v_qisF, ((__pyx_t_double_complex *)__pyx_v_rptr), __pyx_v_rs, ((__pyx_t_double_complex *)__pyx_v_uptr), __pyx_v_us, ((__pyx_t_double_complex *)__pyx_v_vptr), __pyx_v_vs, ((__pyx_t_double_complex *)__pyx_v_sptr), __pyx_v_ss);
              break;
            }
          }
          __pyx_L28:;
        }

        /* "scipy/linalg/_decomp_update.pyx":2402
 *         vptr = extract(v1, vs)
 *         sptr = extract(s, ss)
 *         with nogil:             # <<<<<<<<<<<<<<
 *             if p == 1:
 *                 if typecode == cnp.NPY_FLOAT:
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_FastGIL_Forget();
            Py_BLOCK_THREADS
            #endif
            goto __pyx_L27;
          }
          __pyx_L27:;
        }
    }

    /* "scipy/linalg/_decomp_update.pyx":2386
 *         return q1.copy(), rnew
 * 
 *     if economic:             # <<<<<<<<<<<<<<
 *         ndim = 1
 *         length = 2*n
 */
    goto __pyx_L23;
  }

  /* "scipy/linalg/_decomp_update.pyx":2438
 *                         <double_complex*>vptr, vs, <double_complex*>sptr, ss)
 *     else:
 *         if m == 1:             # <<<<<<<<<<<<<<
 *             if p == 1:
 *                 r1 += q1.conj() * u1 * v1.T.conj()
 */
  /*else*/ {
    __pyx_t_1 = ((__pyx_v_m == 1) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":2439
 *     else:
 *         if m == 1:
 *             if p == 1:             # <<<<<<<<<<<<<<
 *                 r1 += q1.conj() * u1 * v1.T.conj()
 *             else:
 */
      __pyx_t_1 = ((__pyx_v_p == 1) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":2440
 *         if m == 1:
 *             if p == 1:
 *                 r1 += q1.conj() * u1 * v1.T.conj()             # <<<<<<<<<<<<<<
 *             else:
 *                 r1 += q1.conj() * np.dot(u1,  v1.T.conj())
 */
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_q1), __pyx_n_s_conj); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2440, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_8);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_8, function);
          }
        }
        __pyx_t_6 = (__pyx_t_7) ? __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7) : __Pyx_PyObject_CallNoArg(__pyx_t_8);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2440, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_8 = PyNumber_Multiply(__pyx_t_6, ((PyObject *)__pyx_v_u1)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2440, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_v1), __pyx_n_s_T); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2440, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_conj); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2440, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_2, function);
          }
        }
        __pyx_t_6 = (__pyx_t_7) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_7) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2440, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_2 = PyNumber_Multiply(__pyx_t_8, __pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2440, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_6 = PyNumber_InPlaceAdd(((PyObject *)__pyx_v_r1), __pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2440, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2440, __pyx_L1_error)
        __Pyx_DECREF_SET(__pyx_v_r1, ((PyArrayObject *)__pyx_t_6));
        __pyx_t_6 = 0;

        /* "scipy/linalg/_decomp_update.pyx":2439
 *     else:
 *         if m == 1:
 *             if p == 1:             # <<<<<<<<<<<<<<
 *                 r1 += q1.conj() * u1 * v1.T.conj()
 *             else:
 */
        goto __pyx_L30;
      }

      /* "scipy/linalg/_decomp_update.pyx":2442
 *                 r1 += q1.conj() * u1 * v1.T.conj()
 *             else:
 *                 r1 += q1.conj() * np.dot(u1,  v1.T.conj())             # <<<<<<<<<<<<<<
 *             return q1, r1
 *         qTu = cnp.PyArray_ZEROS(u1.ndim, u1.shape, typecode, 1)
 */
      /*else*/ {
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_q1), __pyx_n_s_conj); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2442, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_8 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
          __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_2);
          if (likely(__pyx_t_8)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
            __Pyx_INCREF(__pyx_t_8);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_2, function);
          }
        }
        __pyx_t_6 = (__pyx_t_8) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_8) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2442, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_np); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2442, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_dot); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2442, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_v1), __pyx_n_s_T); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2442, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_conj); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2442, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        __pyx_t_8 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2442, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = NULL;
        __pyx_t_11 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
            __pyx_t_11 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_u1), __pyx_t_8};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2442, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_u1), __pyx_t_8};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2442, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2442, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(((PyObject *)__pyx_v_u1));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_u1));
          PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_11, ((PyObject *)__pyx_v_u1));
          __Pyx_GIVEREF(__pyx_t_8);
          PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_11, __pyx_t_8);
          __pyx_t_8 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2442, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = PyNumber_Multiply(__pyx_t_6, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2442, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_2 = PyNumber_InPlaceAdd(((PyObject *)__pyx_v_r1), __pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2442, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2442, __pyx_L1_error)
        __Pyx_DECREF_SET(__pyx_v_r1, ((PyArrayObject *)__pyx_t_2));
        __pyx_t_2 = 0;
      }
      __pyx_L30:;

      /* "scipy/linalg/_decomp_update.pyx":2443
 *             else:
 *                 r1 += q1.conj() * np.dot(u1,  v1.T.conj())
 *             return q1, r1             # <<<<<<<<<<<<<<
 *         qTu = cnp.PyArray_ZEROS(u1.ndim, u1.shape, typecode, 1)
 *         qTuptr = extract(qTu, qTus)
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2443, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_INCREF(((PyObject *)__pyx_v_q1));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_q1));
      PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_q1));
      __Pyx_INCREF(((PyObject *)__pyx_v_r1));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_r1));
      PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_r1));
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      goto __pyx_L0;

      /* "scipy/linalg/_decomp_update.pyx":2438
 *                         <double_complex*>vptr, vs, <double_complex*>sptr, ss)
 *     else:
 *         if m == 1:             # <<<<<<<<<<<<<<
 *             if p == 1:
 *                 r1 += q1.conj() * u1 * v1.T.conj()
 */
    }

    /* "scipy/linalg/_decomp_update.pyx":2444
 *                 r1 += q1.conj() * np.dot(u1,  v1.T.conj())
 *             return q1, r1
 *         qTu = cnp.PyArray_ZEROS(u1.ndim, u1.shape, typecode, 1)             # <<<<<<<<<<<<<<
 *         qTuptr = extract(qTu, qTus)
 *         if p == 1:
 */
    __pyx_t_2 = PyArray_ZEROS(__pyx_v_u1->nd, __pyx_v_u1->dimensions, __pyx_v_typecode, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2444, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2444, __pyx_L1_error)
    __pyx_v_qTu = ((PyArrayObject *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "scipy/linalg/_decomp_update.pyx":2445
 *             return q1, r1
 *         qTu = cnp.PyArray_ZEROS(u1.ndim, u1.shape, typecode, 1)
 *         qTuptr = extract(qTu, qTus)             # <<<<<<<<<<<<<<
 *         if p == 1:
 *             if not cnp.PyArray_ISONESEGMENT(q1):
 */
    __pyx_v_qTuptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_qTu, __pyx_v_qTus);

    /* "scipy/linalg/_decomp_update.pyx":2446
 *         qTu = cnp.PyArray_ZEROS(u1.ndim, u1.shape, typecode, 1)
 *         qTuptr = extract(qTu, qTus)
 *         if p == 1:             # <<<<<<<<<<<<<<
 *             if not cnp.PyArray_ISONESEGMENT(q1):
 *                 q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 */
    __pyx_t_1 = ((__pyx_v_p == 1) != 0);
    if (__pyx_t_1) {

      /* "scipy/linalg/_decomp_update.pyx":2447
 *         qTuptr = extract(qTu, qTus)
 *         if p == 1:
 *             if not cnp.PyArray_ISONESEGMENT(q1):             # <<<<<<<<<<<<<<
 *                 q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             form_qTu(q1, u1, qTuptr, qTus, 0)
 */
      __pyx_t_1 = ((!(PyArray_ISONESEGMENT(__pyx_v_q1) != 0)) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":2448
 *         if p == 1:
 *             if not cnp.PyArray_ISONESEGMENT(q1):
 *                 q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *             form_qTu(q1, u1, qTuptr, qTus, 0)
 *             qptr = extract(q1, qs)
 */
        __pyx_t_2 = ((PyObject *)PyArray_FromArray(__pyx_v_q1, NULL, NPY_F_CONTIGUOUS)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2448, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF_SET(__pyx_v_q1, ((PyArrayObject *)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "scipy/linalg/_decomp_update.pyx":2447
 *         qTuptr = extract(qTu, qTus)
 *         if p == 1:
 *             if not cnp.PyArray_ISONESEGMENT(q1):             # <<<<<<<<<<<<<<
 *                 q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             form_qTu(q1, u1, qTuptr, qTus, 0)
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":2449
 *             if not cnp.PyArray_ISONESEGMENT(q1):
 *                 q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             form_qTu(q1, u1, qTuptr, qTus, 0)             # <<<<<<<<<<<<<<
 *             qptr = extract(q1, qs)
 *             rptr = extract(r1, rs)
 */
      __pyx_t_2 = __pyx_f_5scipy_6linalg_14_decomp_update_form_qTu(__pyx_v_q1, __pyx_v_u1, __pyx_v_qTuptr, __pyx_v_qTus, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2449, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "scipy/linalg/_decomp_update.pyx":2450
 *                 q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             form_qTu(q1, u1, qTuptr, qTus, 0)
 *             qptr = extract(q1, qs)             # <<<<<<<<<<<<<<
 *             rptr = extract(r1, rs)
 *             vptr = extract(v1, vs)
 */
      __pyx_v_qptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_q1, __pyx_v_qs);

      /* "scipy/linalg/_decomp_update.pyx":2451
 *             form_qTu(q1, u1, qTuptr, qTus, 0)
 *             qptr = extract(q1, qs)
 *             rptr = extract(r1, rs)             # <<<<<<<<<<<<<<
 *             vptr = extract(v1, vs)
 *             with nogil:
 */
      __pyx_v_rptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_r1, __pyx_v_rs);

      /* "scipy/linalg/_decomp_update.pyx":2452
 *             qptr = extract(q1, qs)
 *             rptr = extract(r1, rs)
 *             vptr = extract(v1, vs)             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 */
      __pyx_v_vptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_v1, __pyx_v_vs);

      /* "scipy/linalg/_decomp_update.pyx":2453
 *             rptr = extract(r1, rs)
 *             vptr = extract(v1, vs)
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     qr_rank_1_update(m, n, <float*>qptr, qs,
 */
      {
          #ifdef WITH_THREAD
          PyThreadState *_save;
          Py_UNBLOCK_THREADS
          __Pyx_FastGIL_Remember();
          #endif
          /*try:*/ {

            /* "scipy/linalg/_decomp_update.pyx":2454
 *             vptr = extract(v1, vs)
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     qr_rank_1_update(m, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, <float*>qTuptr, qTus,
 */
            switch (__pyx_v_typecode) {
              case NPY_FLOAT:

              /* "scipy/linalg/_decomp_update.pyx":2455
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 *                     qr_rank_1_update(m, n, <float*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float*>rptr, rs, <float*>qTuptr, qTus,
 *                         <float*>vptr, vs)
 */
              __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_1_update(__pyx_v_m, __pyx_v_n, ((float *)__pyx_v_qptr), __pyx_v_qs, ((float *)__pyx_v_rptr), __pyx_v_rs, ((float *)__pyx_v_qTuptr), __pyx_v_qTus, ((float *)__pyx_v_vptr), __pyx_v_vs);

              /* "scipy/linalg/_decomp_update.pyx":2454
 *             vptr = extract(v1, vs)
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     qr_rank_1_update(m, n, <float*>qptr, qs,
 *                         <float*>rptr, rs, <float*>qTuptr, qTus,
 */
              break;
              case NPY_DOUBLE:

              /* "scipy/linalg/_decomp_update.pyx":2459
 *                         <float*>vptr, vs)
 *                 elif typecode == cnp.NPY_DOUBLE:
 *                     qr_rank_1_update(m, n, <double*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double*>rptr, rs, <double*>qTuptr, qTus,
 *                         <double*>vptr, vs)
 */
              __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_1_update(__pyx_v_m, __pyx_v_n, ((double *)__pyx_v_qptr), __pyx_v_qs, ((double *)__pyx_v_rptr), __pyx_v_rs, ((double *)__pyx_v_qTuptr), __pyx_v_qTus, ((double *)__pyx_v_vptr), __pyx_v_vs);

              /* "scipy/linalg/_decomp_update.pyx":2458
 *                         <float*>rptr, rs, <float*>qTuptr, qTus,
 *                         <float*>vptr, vs)
 *                 elif typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                     qr_rank_1_update(m, n, <double*>qptr, qs,
 *                         <double*>rptr, rs, <double*>qTuptr, qTus,
 */
              break;
              case NPY_CFLOAT:

              /* "scipy/linalg/_decomp_update.pyx":2463
 *                         <double*>vptr, vs)
 *                 elif typecode == cnp.NPY_CFLOAT:
 *                     qr_rank_1_update(m, n, <float_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float_complex*>rptr, rs, <float_complex*>qTuptr, qTus,
 *                         <float_complex*>vptr, vs)
 */
              __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_1_update(__pyx_v_m, __pyx_v_n, ((__pyx_t_float_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_float_complex *)__pyx_v_rptr), __pyx_v_rs, ((__pyx_t_float_complex *)__pyx_v_qTuptr), __pyx_v_qTus, ((__pyx_t_float_complex *)__pyx_v_vptr), __pyx_v_vs);

              /* "scipy/linalg/_decomp_update.pyx":2462
 *                         <double*>rptr, rs, <double*>qTuptr, qTus,
 *                         <double*>vptr, vs)
 *                 elif typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                     qr_rank_1_update(m, n, <float_complex*>qptr, qs,
 *                         <float_complex*>rptr, rs, <float_complex*>qTuptr, qTus,
 */
              break;
              default:

              /* "scipy/linalg/_decomp_update.pyx":2467
 *                         <float_complex*>vptr, vs)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 *                     qr_rank_1_update(m, n, <double_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double_complex*>rptr, rs, <double_complex*>qTuptr, qTus,
 *                         <double_complex*>vptr, vs)
 */
              __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_1_update(__pyx_v_m, __pyx_v_n, ((__pyx_t_double_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_double_complex *)__pyx_v_rptr), __pyx_v_rs, ((__pyx_t_double_complex *)__pyx_v_qTuptr), __pyx_v_qTus, ((__pyx_t_double_complex *)__pyx_v_vptr), __pyx_v_vs);
              break;
            }
          }

          /* "scipy/linalg/_decomp_update.pyx":2453
 *             rptr = extract(r1, rs)
 *             vptr = extract(v1, vs)
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     qr_rank_1_update(m, n, <float*>qptr, qs,
 */
          /*finally:*/ {
            /*normal exit:*/{
              #ifdef WITH_THREAD
              __Pyx_FastGIL_Forget();
              Py_BLOCK_THREADS
              #endif
              goto __pyx_L35;
            }
            __pyx_L35:;
          }
      }

      /* "scipy/linalg/_decomp_update.pyx":2446
 *         qTu = cnp.PyArray_ZEROS(u1.ndim, u1.shape, typecode, 1)
 *         qTuptr = extract(qTu, qTus)
 *         if p == 1:             # <<<<<<<<<<<<<<
 *             if not cnp.PyArray_ISONESEGMENT(q1):
 *                 q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 */
      goto __pyx_L31;
    }

    /* "scipy/linalg/_decomp_update.pyx":2471
 *                         <double_complex*>vptr, vs)
 *         else:
 *             if not cnp.PyArray_CHKFLAGS(q1, cnp.NPY_F_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *                 q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             if not cnp.PyArray_CHKFLAGS(r1, cnp.NPY_F_CONTIGUOUS):
 */
    /*else*/ {
      __pyx_t_1 = ((!(PyArray_CHKFLAGS(__pyx_v_q1, NPY_F_CONTIGUOUS) != 0)) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":2472
 *         else:
 *             if not cnp.PyArray_CHKFLAGS(q1, cnp.NPY_F_CONTIGUOUS):
 *                 q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *             if not cnp.PyArray_CHKFLAGS(r1, cnp.NPY_F_CONTIGUOUS):
 *                 r1 = PyArray_FromArray(r1, NULL, cnp.NPY_F_CONTIGUOUS)
 */
        __pyx_t_2 = ((PyObject *)PyArray_FromArray(__pyx_v_q1, NULL, NPY_F_CONTIGUOUS)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2472, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF_SET(__pyx_v_q1, ((PyArrayObject *)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "scipy/linalg/_decomp_update.pyx":2471
 *                         <double_complex*>vptr, vs)
 *         else:
 *             if not cnp.PyArray_CHKFLAGS(q1, cnp.NPY_F_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *                 q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             if not cnp.PyArray_CHKFLAGS(r1, cnp.NPY_F_CONTIGUOUS):
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":2473
 *             if not cnp.PyArray_CHKFLAGS(q1, cnp.NPY_F_CONTIGUOUS):
 *                 q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             if not cnp.PyArray_CHKFLAGS(r1, cnp.NPY_F_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *                 r1 = PyArray_FromArray(r1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             if not cnp.PyArray_ISONESEGMENT(u1):
 */
      __pyx_t_1 = ((!(PyArray_CHKFLAGS(__pyx_v_r1, NPY_F_CONTIGUOUS) != 0)) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":2474
 *                 q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             if not cnp.PyArray_CHKFLAGS(r1, cnp.NPY_F_CONTIGUOUS):
 *                 r1 = PyArray_FromArray(r1, NULL, cnp.NPY_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *             if not cnp.PyArray_ISONESEGMENT(u1):
 *                 u1 = PyArray_FromArray(u1, NULL, cnp.NPY_F_CONTIGUOUS)
 */
        __pyx_t_2 = ((PyObject *)PyArray_FromArray(__pyx_v_r1, NULL, NPY_F_CONTIGUOUS)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2474, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF_SET(__pyx_v_r1, ((PyArrayObject *)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "scipy/linalg/_decomp_update.pyx":2473
 *             if not cnp.PyArray_CHKFLAGS(q1, cnp.NPY_F_CONTIGUOUS):
 *                 q1 = PyArray_FromArray(q1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             if not cnp.PyArray_CHKFLAGS(r1, cnp.NPY_F_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *                 r1 = PyArray_FromArray(r1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             if not cnp.PyArray_ISONESEGMENT(u1):
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":2475
 *             if not cnp.PyArray_CHKFLAGS(r1, cnp.NPY_F_CONTIGUOUS):
 *                 r1 = PyArray_FromArray(r1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             if not cnp.PyArray_ISONESEGMENT(u1):             # <<<<<<<<<<<<<<
 *                 u1 = PyArray_FromArray(u1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             # v.T must be F contiguous --> v must be C contiguous
 */
      __pyx_t_1 = ((!(PyArray_ISONESEGMENT(__pyx_v_u1) != 0)) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":2476
 *                 r1 = PyArray_FromArray(r1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             if not cnp.PyArray_ISONESEGMENT(u1):
 *                 u1 = PyArray_FromArray(u1, NULL, cnp.NPY_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *             # v.T must be F contiguous --> v must be C contiguous
 *             if not cnp.PyArray_CHKFLAGS(v1, cnp.NPY_C_CONTIGUOUS):
 */
        __pyx_t_2 = ((PyObject *)PyArray_FromArray(__pyx_v_u1, NULL, NPY_F_CONTIGUOUS)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2476, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF_SET(__pyx_v_u1, ((PyArrayObject *)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "scipy/linalg/_decomp_update.pyx":2475
 *             if not cnp.PyArray_CHKFLAGS(r1, cnp.NPY_F_CONTIGUOUS):
 *                 r1 = PyArray_FromArray(r1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             if not cnp.PyArray_ISONESEGMENT(u1):             # <<<<<<<<<<<<<<
 *                 u1 = PyArray_FromArray(u1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             # v.T must be F contiguous --> v must be C contiguous
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":2478
 *                 u1 = PyArray_FromArray(u1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             # v.T must be F contiguous --> v must be C contiguous
 *             if not cnp.PyArray_CHKFLAGS(v1, cnp.NPY_C_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *                 v1 = PyArray_FromArray(v1, NULL, cnp.NPY_C_CONTIGUOUS)
 *             v1 = v1.T
 */
      __pyx_t_1 = ((!(PyArray_CHKFLAGS(__pyx_v_v1, NPY_C_CONTIGUOUS) != 0)) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":2479
 *             # v.T must be F contiguous --> v must be C contiguous
 *             if not cnp.PyArray_CHKFLAGS(v1, cnp.NPY_C_CONTIGUOUS):
 *                 v1 = PyArray_FromArray(v1, NULL, cnp.NPY_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *             v1 = v1.T
 *             form_qTu(q1, u1, qTuptr, qTus, 0)
 */
        __pyx_t_2 = ((PyObject *)PyArray_FromArray(__pyx_v_v1, NULL, NPY_C_CONTIGUOUS)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2479, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF_SET(__pyx_v_v1, ((PyArrayObject *)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "scipy/linalg/_decomp_update.pyx":2478
 *                 u1 = PyArray_FromArray(u1, NULL, cnp.NPY_F_CONTIGUOUS)
 *             # v.T must be F contiguous --> v must be C contiguous
 *             if not cnp.PyArray_CHKFLAGS(v1, cnp.NPY_C_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *                 v1 = PyArray_FromArray(v1, NULL, cnp.NPY_C_CONTIGUOUS)
 *             v1 = v1.T
 */
      }

      /* "scipy/linalg/_decomp_update.pyx":2480
 *             if not cnp.PyArray_CHKFLAGS(v1, cnp.NPY_C_CONTIGUOUS):
 *                 v1 = PyArray_FromArray(v1, NULL, cnp.NPY_C_CONTIGUOUS)
 *             v1 = v1.T             # <<<<<<<<<<<<<<
 *             form_qTu(q1, u1, qTuptr, qTus, 0)
 *             qptr = extract(q1, qs)
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_v1), __pyx_n_s_T); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2480, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(0, 2480, __pyx_L1_error)
      __Pyx_DECREF_SET(__pyx_v_v1, ((PyArrayObject *)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "scipy/linalg/_decomp_update.pyx":2481
 *                 v1 = PyArray_FromArray(v1, NULL, cnp.NPY_C_CONTIGUOUS)
 *             v1 = v1.T
 *             form_qTu(q1, u1, qTuptr, qTus, 0)             # <<<<<<<<<<<<<<
 *             qptr = extract(q1, qs)
 *             rptr = extract(r1, rs)
 */
      __pyx_t_2 = __pyx_f_5scipy_6linalg_14_decomp_update_form_qTu(__pyx_v_q1, __pyx_v_u1, __pyx_v_qTuptr, __pyx_v_qTus, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2481, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "scipy/linalg/_decomp_update.pyx":2482
 *             v1 = v1.T
 *             form_qTu(q1, u1, qTuptr, qTus, 0)
 *             qptr = extract(q1, qs)             # <<<<<<<<<<<<<<
 *             rptr = extract(r1, rs)
 *             vptr = extract(v1, vs)
 */
      __pyx_v_qptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_q1, __pyx_v_qs);

      /* "scipy/linalg/_decomp_update.pyx":2483
 *             form_qTu(q1, u1, qTuptr, qTus, 0)
 *             qptr = extract(q1, qs)
 *             rptr = extract(r1, rs)             # <<<<<<<<<<<<<<
 *             vptr = extract(v1, vs)
 *             with nogil:
 */
      __pyx_v_rptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_r1, __pyx_v_rs);

      /* "scipy/linalg/_decomp_update.pyx":2484
 *             qptr = extract(q1, qs)
 *             rptr = extract(r1, rs)
 *             vptr = extract(v1, vs)             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 */
      __pyx_v_vptr = __pyx_f_5scipy_6linalg_14_decomp_update_extract(__pyx_v_v1, __pyx_v_vs);

      /* "scipy/linalg/_decomp_update.pyx":2485
 *             rptr = extract(r1, rs)
 *             vptr = extract(v1, vs)
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     info = qr_rank_p_update(m, n, p, <float*>qptr, qs,
 */
      {
          #ifdef WITH_THREAD
          PyThreadState *_save;
          Py_UNBLOCK_THREADS
          __Pyx_FastGIL_Remember();
          #endif
          /*try:*/ {

            /* "scipy/linalg/_decomp_update.pyx":2486
 *             vptr = extract(v1, vs)
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     info = qr_rank_p_update(m, n, p, <float*>qptr, qs,
 *                         <float*>rptr, rs, <float*>qTuptr, qTus,
 */
            switch (__pyx_v_typecode) {
              case NPY_FLOAT:

              /* "scipy/linalg/_decomp_update.pyx":2487
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:
 *                     info = qr_rank_p_update(m, n, p, <float*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float*>rptr, rs, <float*>qTuptr, qTus,
 *                         <float*>vptr, vs)
 */
              __pyx_v_info = __pyx_fuse_0__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_p_update(__pyx_v_m, __pyx_v_n, __pyx_v_p, ((float *)__pyx_v_qptr), __pyx_v_qs, ((float *)__pyx_v_rptr), __pyx_v_rs, ((float *)__pyx_v_qTuptr), __pyx_v_qTus, ((float *)__pyx_v_vptr), __pyx_v_vs);

              /* "scipy/linalg/_decomp_update.pyx":2486
 *             vptr = extract(v1, vs)
 *             with nogil:
 *                 if typecode == cnp.NPY_FLOAT:             # <<<<<<<<<<<<<<
 *                     info = qr_rank_p_update(m, n, p, <float*>qptr, qs,
 *                         <float*>rptr, rs, <float*>qTuptr, qTus,
 */
              break;
              case NPY_DOUBLE:

              /* "scipy/linalg/_decomp_update.pyx":2491
 *                         <float*>vptr, vs)
 *                 elif typecode == cnp.NPY_DOUBLE:
 *                     info = qr_rank_p_update(m, n, p, <double*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double*>rptr, rs, <double*>qTuptr, qTus,
 *                         <double*>vptr, vs)
 */
              __pyx_v_info = __pyx_fuse_1__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_p_update(__pyx_v_m, __pyx_v_n, __pyx_v_p, ((double *)__pyx_v_qptr), __pyx_v_qs, ((double *)__pyx_v_rptr), __pyx_v_rs, ((double *)__pyx_v_qTuptr), __pyx_v_qTus, ((double *)__pyx_v_vptr), __pyx_v_vs);

              /* "scipy/linalg/_decomp_update.pyx":2490
 *                         <float*>rptr, rs, <float*>qTuptr, qTus,
 *                         <float*>vptr, vs)
 *                 elif typecode == cnp.NPY_DOUBLE:             # <<<<<<<<<<<<<<
 *                     info = qr_rank_p_update(m, n, p, <double*>qptr, qs,
 *                         <double*>rptr, rs, <double*>qTuptr, qTus,
 */
              break;
              case NPY_CFLOAT:

              /* "scipy/linalg/_decomp_update.pyx":2495
 *                         <double*>vptr, vs)
 *                 elif typecode == cnp.NPY_CFLOAT:
 *                     info = qr_rank_p_update(m, n, p, <float_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <float_complex*>rptr, rs, <float_complex*>qTuptr, qTus,
 *                         <float_complex*>vptr, vs)
 */
              __pyx_v_info = __pyx_fuse_2__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_p_update(__pyx_v_m, __pyx_v_n, __pyx_v_p, ((__pyx_t_float_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_float_complex *)__pyx_v_rptr), __pyx_v_rs, ((__pyx_t_float_complex *)__pyx_v_qTuptr), __pyx_v_qTus, ((__pyx_t_float_complex *)__pyx_v_vptr), __pyx_v_vs);

              /* "scipy/linalg/_decomp_update.pyx":2494
 *                         <double*>rptr, rs, <double*>qTuptr, qTus,
 *                         <double*>vptr, vs)
 *                 elif typecode == cnp.NPY_CFLOAT:             # <<<<<<<<<<<<<<
 *                     info = qr_rank_p_update(m, n, p, <float_complex*>qptr, qs,
 *                         <float_complex*>rptr, rs, <float_complex*>qTuptr, qTus,
 */
              break;
              default:

              /* "scipy/linalg/_decomp_update.pyx":2499
 *                         <float_complex*>vptr, vs)
 *                 else:  # typecode == cnp.NPY_CDOUBLE:
 *                     info = qr_rank_p_update(m, n, p, <double_complex*>qptr, qs,             # <<<<<<<<<<<<<<
 *                         <double_complex*>rptr, rs, <double_complex*>qTuptr, qTus,
 *                         <double_complex*>vptr, vs)
 */
              __pyx_v_info = __pyx_fuse_3__pyx_f_5scipy_6linalg_14_decomp_update_qr_rank_p_update(__pyx_v_m, __pyx_v_n, __pyx_v_p, ((__pyx_t_double_complex *)__pyx_v_qptr), __pyx_v_qs, ((__pyx_t_double_complex *)__pyx_v_rptr), __pyx_v_rs, ((__pyx_t_double_complex *)__pyx_v_qTuptr), __pyx_v_qTus, ((__pyx_t_double_complex *)__pyx_v_vptr), __pyx_v_vs);
              break;
            }
          }

          /* "scipy/linalg/_decomp_update.pyx":2485
 *             rptr = extract(r1, rs)
 *             vptr = extract(v1, vs)
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 if typecode == cnp.NPY_FLOAT:
 *                     info = qr_rank_p_update(m, n, p, <float*>qptr, qs,
 */
          /*finally:*/ {
            /*normal exit:*/{
              #ifdef WITH_THREAD
              __Pyx_FastGIL_Forget();
              Py_BLOCK_THREADS
              #endif
              goto __pyx_L42;
            }
            __pyx_L42:;
          }
      }

      /* "scipy/linalg/_decomp_update.pyx":2502
 *                         <double_complex*>rptr, rs, <double_complex*>qTuptr, qTus,
 *                         <double_complex*>vptr, vs)
 *             if info != 0:             # <<<<<<<<<<<<<<
 *                 if info > 0:
 *                     raise ValueError('The {0}th argument to ?geqrf was '
 */
      __pyx_t_1 = ((__pyx_v_info != 0) != 0);
      if (__pyx_t_1) {

        /* "scipy/linalg/_decomp_update.pyx":2503
 *                         <double_complex*>vptr, vs)
 *             if info != 0:
 *                 if info > 0:             # <<<<<<<<<<<<<<
 *                     raise ValueError('The {0}th argument to ?geqrf was '
 *                             'invalid'.format(info))
 */
        __pyx_t_1 = ((__pyx_v_info > 0) != 0);
        if (unlikely(__pyx_t_1)) {

          /* "scipy/linalg/_decomp_update.pyx":2505
 *                 if info > 0:
 *                     raise ValueError('The {0}th argument to ?geqrf was '
 *                             'invalid'.format(info))             # <<<<<<<<<<<<<<
 *                 elif info < 0:
 *                     raise ValueError('The {0}th argument to ?ormqr/?unmqr was '
 */
          __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_The_0_th_argument_to_geqrf_was_i, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2505, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_info); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2505, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_5 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
            __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_7);
            if (likely(__pyx_t_5)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
              __Pyx_INCREF(__pyx_t_5);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_7, function);
            }
          }
          __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_5, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2505, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

          /* "scipy/linalg/_decomp_update.pyx":2504
 *             if info != 0:
 *                 if info > 0:
 *                     raise ValueError('The {0}th argument to ?geqrf was '             # <<<<<<<<<<<<<<
 *                             'invalid'.format(info))
 *                 elif info < 0:
 */
          __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2504, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_Raise(__pyx_t_7, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __PYX_ERR(0, 2504, __pyx_L1_error)

          /* "scipy/linalg/_decomp_update.pyx":2503
 *                         <double_complex*>vptr, vs)
 *             if info != 0:
 *                 if info > 0:             # <<<<<<<<<<<<<<
 *                     raise ValueError('The {0}th argument to ?geqrf was '
 *                             'invalid'.format(info))
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":2506
 *                     raise ValueError('The {0}th argument to ?geqrf was '
 *                             'invalid'.format(info))
 *                 elif info < 0:             # <<<<<<<<<<<<<<
 *                     raise ValueError('The {0}th argument to ?ormqr/?unmqr was '
 *                             'invalid'.format(abs(info)))
 */
        __pyx_t_1 = ((__pyx_v_info < 0) != 0);
        if (unlikely(__pyx_t_1)) {

          /* "scipy/linalg/_decomp_update.pyx":2508
 *                 elif info < 0:
 *                     raise ValueError('The {0}th argument to ?ormqr/?unmqr was '
 *                             'invalid'.format(abs(info)))             # <<<<<<<<<<<<<<
 *                 elif info == MEMORY_ERROR:
 *                     raise MemoryError('Unable to allocate memory for array.')
 */
          __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_The_0_th_argument_to_ormqr_unmqr, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2508, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_11 = abs(__pyx_v_info); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 2508, __pyx_L1_error)
          __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_t_11); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2508, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_5 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
            __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
            if (likely(__pyx_t_5)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
              __Pyx_INCREF(__pyx_t_5);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_2, function);
            }
          }
          __pyx_t_7 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_6);
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2508, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "scipy/linalg/_decomp_update.pyx":2507
 *                             'invalid'.format(info))
 *                 elif info < 0:
 *                     raise ValueError('The {0}th argument to ?ormqr/?unmqr was '             # <<<<<<<<<<<<<<
 *                             'invalid'.format(abs(info)))
 *                 elif info == MEMORY_ERROR:
 */
          __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2507, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_Raise(__pyx_t_2, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __PYX_ERR(0, 2507, __pyx_L1_error)

          /* "scipy/linalg/_decomp_update.pyx":2506
 *                     raise ValueError('The {0}th argument to ?geqrf was '
 *                             'invalid'.format(info))
 *                 elif info < 0:             # <<<<<<<<<<<<<<
 *                     raise ValueError('The {0}th argument to ?ormqr/?unmqr was '
 *                             'invalid'.format(abs(info)))
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":2509
 *                     raise ValueError('The {0}th argument to ?ormqr/?unmqr was '
 *                             'invalid'.format(abs(info)))
 *                 elif info == MEMORY_ERROR:             # <<<<<<<<<<<<<<
 *                     raise MemoryError('Unable to allocate memory for array.')
 *     return q1, r1
 */
        __pyx_t_1 = ((__pyx_v_info == __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR) != 0);
        if (unlikely(__pyx_t_1)) {

          /* "scipy/linalg/_decomp_update.pyx":2510
 *                             'invalid'.format(abs(info)))
 *                 elif info == MEMORY_ERROR:
 *                     raise MemoryError('Unable to allocate memory for array.')             # <<<<<<<<<<<<<<
 *     return q1, r1
 * 
 */
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2510, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_Raise(__pyx_t_2, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __PYX_ERR(0, 2510, __pyx_L1_error)

          /* "scipy/linalg/_decomp_update.pyx":2509
 *                     raise ValueError('The {0}th argument to ?ormqr/?unmqr was '
 *                             'invalid'.format(abs(info)))
 *                 elif info == MEMORY_ERROR:             # <<<<<<<<<<<<<<
 *                     raise MemoryError('Unable to allocate memory for array.')
 *     return q1, r1
 */
        }

        /* "scipy/linalg/_decomp_update.pyx":2502
 *                         <double_complex*>rptr, rs, <double_complex*>qTuptr, qTus,
 *                         <double_complex*>vptr, vs)
 *             if info != 0:             # <<<<<<<<<<<<<<
 *                 if info > 0:
 *                     raise ValueError('The {0}th argument to ?geqrf was '
 */
      }
    }
    __pyx_L31:;
  }
  __pyx_L23:;

  /* "scipy/linalg/_decomp_update.pyx":2511
 *                 elif info == MEMORY_ERROR:
 *                     raise MemoryError('Unable to allocate memory for array.')
 *     return q1, r1             # <<<<<<<<<<<<<<
 * 
 * cnp.import_array()
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2511, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_q1));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_q1));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_q1));
  __Pyx_INCREF(((PyObject *)__pyx_v_r1));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_r1));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_r1));
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "scipy/linalg/_decomp_update.pyx":2159
 * 
 * @cython.embedsignature(True)
 * def qr_update(Q, R, u, v, overwrite_qruv=False, check_finite=True):             # <<<<<<<<<<<<<<
 *     """
 *     Rank-k QR update
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("scipy.linalg._decomp_update.qr_update", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_q1);
  __Pyx_XDECREF((PyObject *)__pyx_v_r1);
  __Pyx_XDECREF((PyObject *)__pyx_v_u1);
  __Pyx_XDECREF((PyObject *)__pyx_v_v1);
  __Pyx_XDECREF((PyObject *)__pyx_v_qTu);
  __Pyx_XDECREF((PyObject *)__pyx_v_s);
  __Pyx_XDECREF(__pyx_v_rnew);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":258
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fulfill the PEP.
 */

/* Python wrapper */
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5numpy_7ndarray___getbuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_v_i;
  int __pyx_v_ndim;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  int __pyx_v_t;
  char *__pyx_v_f;
  PyArray_Descr *__pyx_v_descr = 0;
  int __pyx_v_offset;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyArray_Descr *__pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  char *__pyx_t_9;
  if (__pyx_v_info == NULL) {
    PyErr_SetString(PyExc_BufferError, "PyObject_GetBuffer: view==NULL argument is obsolete");
    return -1;
  }
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(__pyx_v_info->obj);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":265
 * 
 *             cdef int i, ndim
 *             cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 */
  __pyx_v_endian_detector = 1;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":266
 *             cdef int i, ndim
 *             cdef int endian_detector = 1
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 * 
 *             ndim = PyArray_NDIM(self)
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":268
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 *             ndim = PyArray_NDIM(self)             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 */
  __pyx_v_ndim = PyArray_NDIM(__pyx_v_self);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":270
 *             ndim = PyArray_NDIM(self)
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_C_CONTIGUOUS) == PyBUF_C_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":271
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_C_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_ARRAY_C_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":270
 *             ndim = PyArray_NDIM(self)
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  if (unlikely(__pyx_t_1)) {

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":272
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__33, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 272, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 272, __pyx_L1_error)

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":270
 *             ndim = PyArray_NDIM(self)
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":274
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_F_CONTIGUOUS) == PyBUF_F_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L7_bool_binop_done;
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":275
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_F_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_ARRAY_F_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L7_bool_binop_done:;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":274
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  if (unlikely(__pyx_t_1)) {

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":276
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__34, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 276, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 276, __pyx_L1_error)

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":274
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":278
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 *             info.buf = PyArray_DATA(self)             # <<<<<<<<<<<<<<
 *             info.ndim = ndim
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_v_info->buf = PyArray_DATA(__pyx_v_self);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":279
 * 
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim             # <<<<<<<<<<<<<<
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 # Allocate new buffer for strides and shape info.
 */
  __pyx_v_info->ndim = __pyx_v_ndim;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":280
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":283
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>PyObject_Malloc(sizeof(Py_ssize_t) * 2 * <size_t>ndim)             # <<<<<<<<<<<<<<
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 */
    __pyx_v_info->strides = ((Py_ssize_t *)PyObject_Malloc((((sizeof(Py_ssize_t)) * 2) * ((size_t)__pyx_v_ndim))));

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":284
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>PyObject_Malloc(sizeof(Py_ssize_t) * 2 * <size_t>ndim)
 *                 info.shape = info.strides + ndim             # <<<<<<<<<<<<<<
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 */
    __pyx_v_info->shape = (__pyx_v_info->strides + __pyx_v_ndim);

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":285
 *                 info.strides = <Py_ssize_t*>PyObject_Malloc(sizeof(Py_ssize_t) * 2 * <size_t>ndim)
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):             # <<<<<<<<<<<<<<
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 */
    __pyx_t_4 = __pyx_v_ndim;
    __pyx_t_5 = __pyx_t_4;
    for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {
      __pyx_v_i = __pyx_t_6;

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":286
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]             # <<<<<<<<<<<<<<
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 */
      (__pyx_v_info->strides[__pyx_v_i]) = (PyArray_STRIDES(__pyx_v_self)[__pyx_v_i]);

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":287
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]             # <<<<<<<<<<<<<<
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 */
      (__pyx_v_info->shape[__pyx_v_i]) = (PyArray_DIMS(__pyx_v_self)[__pyx_v_i]);
    }

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":280
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 */
    goto __pyx_L9;
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":289
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)             # <<<<<<<<<<<<<<
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 */
  /*else*/ {
    __pyx_v_info->strides = ((Py_ssize_t *)PyArray_STRIDES(__pyx_v_self));

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":290
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)             # <<<<<<<<<<<<<<
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 */
    __pyx_v_info->shape = ((Py_ssize_t *)PyArray_DIMS(__pyx_v_self));
  }
  __pyx_L9:;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":291
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL             # <<<<<<<<<<<<<<
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 */
  __pyx_v_info->suboffsets = NULL;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":292
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)             # <<<<<<<<<<<<<<
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 * 
 */
  __pyx_v_info->itemsize = PyArray_ITEMSIZE(__pyx_v_self);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":293
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)             # <<<<<<<<<<<<<<
 * 
 *             cdef int t
 */
  __pyx_v_info->readonly = (!(PyArray_ISWRITEABLE(__pyx_v_self) != 0));

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":296
 * 
 *             cdef int t
 *             cdef char* f = NULL             # <<<<<<<<<<<<<<
 *             cdef dtype descr = <dtype>PyArray_DESCR(self)
 *             cdef int offset
 */
  __pyx_v_f = NULL;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":297
 *             cdef int t
 *             cdef char* f = NULL
 *             cdef dtype descr = <dtype>PyArray_DESCR(self)             # <<<<<<<<<<<<<<
 *             cdef int offset
 * 
 */
  __pyx_t_7 = PyArray_DESCR(__pyx_v_self);
  __pyx_t_3 = ((PyObject *)__pyx_t_7);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_v_descr = ((PyArray_Descr *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":300
 *             cdef int offset
 * 
 *             info.obj = self             # <<<<<<<<<<<<<<
 * 
 *             if not PyDataType_HASFIELDS(descr):
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  __Pyx_GOTREF(__pyx_v_info->obj);
  __Pyx_DECREF(__pyx_v_info->obj);
  __pyx_v_info->obj = ((PyObject *)__pyx_v_self);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":302
 *             info.obj = self
 * 
 *             if not PyDataType_HASFIELDS(descr):             # <<<<<<<<<<<<<<
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 */
  __pyx_t_1 = ((!(PyDataType_HASFIELDS(__pyx_v_descr) != 0)) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":303
 * 
 *             if not PyDataType_HASFIELDS(descr):
 *                 t = descr.type_num             # <<<<<<<<<<<<<<
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 */
    __pyx_t_4 = __pyx_v_descr->type_num;
    __pyx_v_t = __pyx_t_4;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":304
 *             if not PyDataType_HASFIELDS(descr):
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '>') != 0);
    if (!__pyx_t_2) {
      goto __pyx_L15_next_or;
    } else {
    }
    __pyx_t_2 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L14_bool_binop_done;
    }
    __pyx_L15_next_or:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":305
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '<') != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L14_bool_binop_done;
    }
    __pyx_t_2 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L14_bool_binop_done:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":304
 *             if not PyDataType_HASFIELDS(descr):
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    if (unlikely(__pyx_t_1)) {

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":306
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__35, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 306, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(1, 306, __pyx_L1_error)

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":304
 *             if not PyDataType_HASFIELDS(descr):
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    }

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":307
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 */
    switch (__pyx_v_t) {
      case NPY_BYTE:
      __pyx_v_f = ((char *)"b");
      break;
      case NPY_UBYTE:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":308
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 */
      __pyx_v_f = ((char *)"B");
      break;
      case NPY_SHORT:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":309
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 */
      __pyx_v_f = ((char *)"h");
      break;
      case NPY_USHORT:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":310
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 */
      __pyx_v_f = ((char *)"H");
      break;
      case NPY_INT:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":311
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 */
      __pyx_v_f = ((char *)"i");
      break;
      case NPY_UINT:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":312
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 */
      __pyx_v_f = ((char *)"I");
      break;
      case NPY_LONG:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":313
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 */
      __pyx_v_f = ((char *)"l");
      break;
      case NPY_ULONG:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":314
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 */
      __pyx_v_f = ((char *)"L");
      break;
      case NPY_LONGLONG:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":315
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 */
      __pyx_v_f = ((char *)"q");
      break;
      case NPY_ULONGLONG:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":316
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 */
      __pyx_v_f = ((char *)"Q");
      break;
      case NPY_FLOAT:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":317
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 */
      __pyx_v_f = ((char *)"f");
      break;
      case NPY_DOUBLE:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":318
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 */
      __pyx_v_f = ((char *)"d");
      break;
      case NPY_LONGDOUBLE:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":319
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 */
      __pyx_v_f = ((char *)"g");
      break;
      case NPY_CFLOAT:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":320
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 */
      __pyx_v_f = ((char *)"Zf");
      break;
      case NPY_CDOUBLE:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":321
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"
 */
      __pyx_v_f = ((char *)"Zd");
      break;
      case NPY_CLONGDOUBLE:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":322
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 */
      __pyx_v_f = ((char *)"Zg");
      break;
      case NPY_OBJECT:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":323
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"             # <<<<<<<<<<<<<<
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      __pyx_v_f = ((char *)"O");
      break;
      default:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":325
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *                 info.format = f
 *                 return
 */
      __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_t); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 325, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_8 = PyUnicode_Format(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_t_3); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 325, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 325, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(1, 325, __pyx_L1_error)
      break;
    }

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":326
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f             # <<<<<<<<<<<<<<
 *                 return
 *             else:
 */
    __pyx_v_info->format = __pyx_v_f;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":327
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f
 *                 return             # <<<<<<<<<<<<<<
 *             else:
 *                 info.format = <char*>PyObject_Malloc(_buffer_format_string_len)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":302
 *             info.obj = self
 * 
 *             if not PyDataType_HASFIELDS(descr):             # <<<<<<<<<<<<<<
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 */
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":329
 *                 return
 *             else:
 *                 info.format = <char*>PyObject_Malloc(_buffer_format_string_len)             # <<<<<<<<<<<<<<
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 */
  /*else*/ {
    __pyx_v_info->format = ((char *)PyObject_Malloc(0xFF));

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":330
 *             else:
 *                 info.format = <char*>PyObject_Malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment             # <<<<<<<<<<<<<<
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,
 */
    (__pyx_v_info->format[0]) = '^';

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":331
 *                 info.format = <char*>PyObject_Malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0             # <<<<<<<<<<<<<<
 *                 f = _util_dtypestring(descr, info.format + 1,
 *                                       info.format + _buffer_format_string_len,
 */
    __pyx_v_offset = 0;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":332
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,             # <<<<<<<<<<<<<<
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 */
    __pyx_t_9 = __pyx_f_5numpy__util_dtypestring(__pyx_v_descr, (__pyx_v_info->format + 1), (__pyx_v_info->format + 0xFF), (&__pyx_v_offset)); if (unlikely(__pyx_t_9 == ((char *)NULL))) __PYX_ERR(1, 332, __pyx_L1_error)
    __pyx_v_f = __pyx_t_9;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":335
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 *                 f[0] = c'\0' # Terminate format string             # <<<<<<<<<<<<<<
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 */
    (__pyx_v_f[0]) = '\x00';
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":258
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fulfill the PEP.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.ndarray.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  if (__pyx_v_info->obj != NULL) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;
  }
  goto __pyx_L2;
  __pyx_L0:;
  if (__pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;
  }
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_descr);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":337
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 PyObject_Free(info.format)
 */

/* Python wrapper */
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info); /*proto*/
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__releasebuffer__ (wrapper)", 0);
  __pyx_pf_5numpy_7ndarray_2__releasebuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__releasebuffer__", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":338
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):             # <<<<<<<<<<<<<<
 *                 PyObject_Free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_t_1 = (PyArray_HASFIELDS(__pyx_v_self) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":339
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):
 *                 PyObject_Free(info.format)             # <<<<<<<<<<<<<<
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 PyObject_Free(info.strides)
 */
    PyObject_Free(__pyx_v_info->format);

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":338
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):             # <<<<<<<<<<<<<<
 *                 PyObject_Free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":340
 *             if PyArray_HASFIELDS(self):
 *                 PyObject_Free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 PyObject_Free(info.strides)
 *                 # info.shape was stored after info.strides in the same block
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":341
 *                 PyObject_Free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 PyObject_Free(info.strides)             # <<<<<<<<<<<<<<
 *                 # info.shape was stored after info.strides in the same block
 * 
 */
    PyObject_Free(__pyx_v_info->strides);

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":340
 *             if PyArray_HASFIELDS(self):
 *                 PyObject_Free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 PyObject_Free(info.strides)
 *                 # info.shape was stored after info.strides in the same block
 */
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":337
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 PyObject_Free(info.format)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":821
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":822
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 822, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":821
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":824
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":825
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 825, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":824
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":827
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":828
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 828, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":827
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":830
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":831
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":830
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":833
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":834
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 834, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":833
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":836
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyDataType_SHAPE(PyArray_Descr *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("PyDataType_SHAPE", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":837
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  __pyx_t_1 = (PyDataType_HASSUBARRAY(__pyx_v_d) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":838
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape             # <<<<<<<<<<<<<<
 *     else:
 *         return ()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject*)__pyx_v_d->subarray->shape));
    __pyx_r = ((PyObject*)__pyx_v_d->subarray->shape);
    goto __pyx_L0;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":837
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":840
 *         return <tuple>d.subarray.shape
 *     else:
 *         return ()             # <<<<<<<<<<<<<<
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_empty_tuple);
    __pyx_r = __pyx_empty_tuple;
    goto __pyx_L0;
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":836
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":842
 *         return ()
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *__pyx_v_descr, char *__pyx_v_f, char *__pyx_v_end, int *__pyx_v_offset) {
  PyArray_Descr *__pyx_v_child = 0;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  PyObject *__pyx_v_fields = 0;
  PyObject *__pyx_v_childname = NULL;
  PyObject *__pyx_v_new_offset = NULL;
  PyObject *__pyx_v_t = NULL;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  long __pyx_t_8;
  char *__pyx_t_9;
  __Pyx_RefNannySetupContext("_util_dtypestring", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":847
 * 
 *     cdef dtype child
 *     cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 *     cdef tuple fields
 */
  __pyx_v_endian_detector = 1;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":848
 *     cdef dtype child
 *     cdef int endian_detector = 1
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 *     cdef tuple fields
 * 
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":851
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  if (unlikely(__pyx_v_descr->names == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(1, 851, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_descr->names; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(1, 851, __pyx_L1_error)
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 851, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_childname, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":852
 * 
 *     for childname in descr.names:
 *         fields = descr.fields[childname]             # <<<<<<<<<<<<<<
 *         child, new_offset = fields
 * 
 */
    if (unlikely(__pyx_v_descr->fields == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 852, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_descr->fields, __pyx_v_childname); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 852, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyTuple_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(1, 852, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_fields, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":853
 *     for childname in descr.names:
 *         fields = descr.fields[childname]
 *         child, new_offset = fields             # <<<<<<<<<<<<<<
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 */
    if (likely(__pyx_v_fields != Py_None)) {
      PyObject* sequence = __pyx_v_fields;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(1, 853, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      #else
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 853, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 853, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      #endif
    } else {
      __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(1, 853, __pyx_L1_error)
    }
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_dtype))))) __PYX_ERR(1, 853, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_child, ((PyArray_Descr *)__pyx_t_3));
    __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_new_offset, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":855
 *         child, new_offset = fields
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 */
    __pyx_t_4 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 855, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = PyNumber_Subtract(__pyx_v_new_offset, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 855, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 855, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = ((((__pyx_v_end - __pyx_v_f) - ((int)__pyx_t_5)) < 15) != 0);
    if (unlikely(__pyx_t_6)) {

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":856
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__36, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 856, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(1, 856, __pyx_L1_error)

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":855
 *         child, new_offset = fields
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 */
    }

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":858
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '>') != 0);
    if (!__pyx_t_7) {
      goto __pyx_L8_next_or;
    } else {
    }
    __pyx_t_7 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_L8_next_or:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":859
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *             raise ValueError(u"Non-native byte order not supported")
 *             # One could encode it in the format string and have Cython
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '<') != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_7 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L7_bool_binop_done:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":858
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    if (unlikely(__pyx_t_6)) {

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":860
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *             # One could encode it in the format string and have Cython
 *             # complain instead, BUT: < and > in format strings also imply
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__35, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 860, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(1, 860, __pyx_L1_error)

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":858
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    }

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":870
 * 
 *         # Output padding bytes
 *         while offset[0] < new_offset:             # <<<<<<<<<<<<<<
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 */
    while (1) {
      __pyx_t_3 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 870, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_v_new_offset, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 870, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(1, 870, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (!__pyx_t_6) break;

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":871
 *         # Output padding bytes
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte             # <<<<<<<<<<<<<<
 *             f += 1
 *             offset[0] += 1
 */
      (__pyx_v_f[0]) = 0x78;

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":872
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte
 *             f += 1             # <<<<<<<<<<<<<<
 *             offset[0] += 1
 * 
 */
      __pyx_v_f = (__pyx_v_f + 1);

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":873
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 *             offset[0] += 1             # <<<<<<<<<<<<<<
 * 
 *         offset[0] += child.itemsize
 */
      __pyx_t_8 = 0;
      (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + 1);
    }

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":875
 *             offset[0] += 1
 * 
 *         offset[0] += child.itemsize             # <<<<<<<<<<<<<<
 * 
 *         if not PyDataType_HASFIELDS(child):
 */
    __pyx_t_8 = 0;
    (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + __pyx_v_child->elsize);

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":877
 *         offset[0] += child.itemsize
 * 
 *         if not PyDataType_HASFIELDS(child):             # <<<<<<<<<<<<<<
 *             t = child.type_num
 *             if end - f < 5:
 */
    __pyx_t_6 = ((!(PyDataType_HASFIELDS(__pyx_v_child) != 0)) != 0);
    if (__pyx_t_6) {

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":878
 * 
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num             # <<<<<<<<<<<<<<
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")
 */
      __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_child->type_num); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 878, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_XDECREF_SET(__pyx_v_t, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":879
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num
 *             if end - f < 5:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError(u"Format string allocated too short.")
 * 
 */
      __pyx_t_6 = (((__pyx_v_end - __pyx_v_f) < 5) != 0);
      if (unlikely(__pyx_t_6)) {

        /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":880
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__37, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 880, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_Raise(__pyx_t_4, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __PYX_ERR(1, 880, __pyx_L1_error)

        /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":879
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num
 *             if end - f < 5:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError(u"Format string allocated too short.")
 * 
 */
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":883
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_BYTE); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 883, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 883, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(1, 883, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 98;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":884
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"             # <<<<<<<<<<<<<<
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_UBYTE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 884, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 884, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(1, 884, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 66;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":885
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"             # <<<<<<<<<<<<<<
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_SHORT); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 885, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 885, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(1, 885, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x68;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":886
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"             # <<<<<<<<<<<<<<
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_USHORT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 886, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 886, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(1, 886, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 72;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":887
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_INT); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 887, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 887, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(1, 887, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x69;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":888
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_UINT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 888, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 888, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(1, 888, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 73;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":889
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONG); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 889, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 889, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(1, 889, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x6C;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":890
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_ULONG); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 890, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 890, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(1, 890, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 76;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":891
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONGLONG); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 891, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 891, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(1, 891, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x71;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":892
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_ULONGLONG); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 892, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 892, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(1, 892, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 81;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":893
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"             # <<<<<<<<<<<<<<
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_FLOAT); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 893, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 893, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(1, 893, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x66;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":894
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_DOUBLE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 894, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 894, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(1, 894, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x64;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":895
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"             # <<<<<<<<<<<<<<
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONGDOUBLE); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 895, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 895, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(1, 895, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x67;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":896
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf             # <<<<<<<<<<<<<<
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CFLOAT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 896, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 896, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(1, 896, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x66;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":897
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd             # <<<<<<<<<<<<<<
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CDOUBLE); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 897, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 897, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(1, 897, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x64;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":898
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg             # <<<<<<<<<<<<<<
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CLONGDOUBLE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 898, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 898, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(1, 898, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x67;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":899
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"             # <<<<<<<<<<<<<<
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_OBJECT); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 899, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 899, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(1, 899, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (likely(__pyx_t_6)) {
        (__pyx_v_f[0]) = 79;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":901
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *             f += 1
 *         else:
 */
      /*else*/ {
        __pyx_t_3 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_v_t); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 901, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 901, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_Raise(__pyx_t_4, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __PYX_ERR(1, 901, __pyx_L1_error)
      }
      __pyx_L15:;

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":902
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *             f += 1             # <<<<<<<<<<<<<<
 *         else:
 *             # Cython ignores struct boundary information ("T{...}"),
 */
      __pyx_v_f = (__pyx_v_f + 1);

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":877
 *         offset[0] += child.itemsize
 * 
 *         if not PyDataType_HASFIELDS(child):             # <<<<<<<<<<<<<<
 *             t = child.type_num
 *             if end - f < 5:
 */
      goto __pyx_L13;
    }

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":906
 *             # Cython ignores struct boundary information ("T{...}"),
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)             # <<<<<<<<<<<<<<
 *     return f
 * 
 */
    /*else*/ {
      __pyx_t_9 = __pyx_f_5numpy__util_dtypestring(__pyx_v_child, __pyx_v_f, __pyx_v_end, __pyx_v_offset); if (unlikely(__pyx_t_9 == ((char *)NULL))) __PYX_ERR(1, 906, __pyx_L1_error)
      __pyx_v_f = __pyx_t_9;
    }
    __pyx_L13:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":851
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":907
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)
 *     return f             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_f;
  goto __pyx_L0;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":842
 *         return ()
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("numpy._util_dtypestring", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child);
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XDECREF(__pyx_v_childname);
  __Pyx_XDECREF(__pyx_v_new_offset);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1022
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1023
 * 
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!             # <<<<<<<<<<<<<<
 *     PyArray_SetBaseObject(arr, base)
 * 
 */
  Py_INCREF(__pyx_v_base);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1024
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  (void)(PyArray_SetBaseObject(__pyx_v_arr, __pyx_v_base));

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1022
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1026
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_v_base;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1027
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)             # <<<<<<<<<<<<<<
 *     if base is NULL:
 *         return None
 */
  __pyx_v_base = PyArray_BASE(__pyx_v_arr);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1028
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  __pyx_t_1 = ((__pyx_v_base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1029
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     return <object>base
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1028
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1030
 *     if base is NULL:
 *         return None
 *     return <object>base             # <<<<<<<<<<<<<<
 * 
 * # Versions of the import_* functions which are more suitable for
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_base));
  __pyx_r = ((PyObject *)__pyx_v_base);
  goto __pyx_L0;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1026
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1034
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_array()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_array(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("import_array", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1035
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_array()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1036
 * cdef inline int import_array() except -1:
 *     try:
 *         _import_array()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")
 */
      __pyx_t_4 = _import_array(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 1036, __pyx_L3_error)

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1035
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_array()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1037
 *     try:
 *         _import_array()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(1, 1037, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1038
 *         _import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__38, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 1038, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(1, 1038, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1035
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_array()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1034
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_array()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1040
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_umath(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("import_umath", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1041
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1042
 * cdef inline int import_umath() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 1042, __pyx_L3_error)

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1041
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1043
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(1, 1043, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1044
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__39, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 1044, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(1, 1044, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1041
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1040
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1046
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_ufunc(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("import_ufunc", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1047
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1048
 * cdef inline int import_ufunc() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 1048, __pyx_L3_error)

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1047
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1049
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(1, 1049, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1050
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__39, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 1050, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(1, 1050, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1047
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1046
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec__decomp_update(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec__decomp_update},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "_decomp_update",
    __pyx_k_Routines_for_updating_QR_decomp, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_u_Expected_M_M_M_N_or_M_N_N_N_but, __pyx_k_Expected_M_M_M_N_or_M_N_N_N_but, sizeof(__pyx_k_Expected_M_M_M_N_or_M_N_N_N_but), 0, 1, 0, 0},
  {&__pyx_n_u_F, __pyx_k_F, sizeof(__pyx_k_F), 0, 1, 0, 1},
  {&__pyx_kp_u_Format_string_allocated_too_shor, __pyx_k_Format_string_allocated_too_shor, sizeof(__pyx_k_Format_string_allocated_too_shor), 0, 1, 0, 0},
  {&__pyx_kp_u_Format_string_allocated_too_shor_2, __pyx_k_Format_string_allocated_too_shor_2, sizeof(__pyx_k_Format_string_allocated_too_shor_2), 0, 1, 0, 0},
  {&__pyx_n_s_ImportError, __pyx_k_ImportError, sizeof(__pyx_k_ImportError), 0, 0, 1, 1},
  {&__pyx_kp_u_Input_array_too_large_for_use_wi, __pyx_k_Input_array_too_large_for_use_wi, sizeof(__pyx_k_Input_array_too_large_for_use_wi), 0, 1, 0, 0},
  {&__pyx_kp_u_Inputs_must_be_arrays, __pyx_k_Inputs_must_be_arrays, sizeof(__pyx_k_Inputs_must_be_arrays), 0, 1, 0, 0},
  {&__pyx_n_s_LinAlgError, __pyx_k_LinAlgError, sizeof(__pyx_k_LinAlgError), 0, 0, 1, 1},
  {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
  {&__pyx_kp_u_Non_native_byte_order_not_suppor, __pyx_k_Non_native_byte_order_not_suppor, sizeof(__pyx_k_Non_native_byte_order_not_suppor), 0, 1, 0, 0},
  {&__pyx_kp_u_One_of_the_columns_of_u_lies_in, __pyx_k_One_of_the_columns_of_u_lies_in, sizeof(__pyx_k_One_of_the_columns_of_u_lies_in), 0, 1, 0, 0},
  {&__pyx_kp_u_Only_arrays_with_dtypes_float32, __pyx_k_Only_arrays_with_dtypes_float32, sizeof(__pyx_k_Only_arrays_with_dtypes_float32), 0, 1, 0, 0},
  {&__pyx_n_s_Q, __pyx_k_Q, sizeof(__pyx_k_Q), 0, 0, 1, 1},
  {&__pyx_kp_u_QR_downdate_on_row_or_column_de, __pyx_k_QR_downdate_on_row_or_column_de, sizeof(__pyx_k_QR_downdate_on_row_or_column_de), 0, 1, 0, 0},
  {&__pyx_kp_u_QR_update_on_row_or_column_inse, __pyx_k_QR_update_on_row_or_column_inse, sizeof(__pyx_k_QR_update_on_row_or_column_inse), 0, 1, 0, 0},
  {&__pyx_kp_u_Q_and_R_do_not_have_compatible_s, __pyx_k_Q_and_R_do_not_have_compatible_s, sizeof(__pyx_k_Q_and_R_do_not_have_compatible_s), 0, 1, 0, 0},
  {&__pyx_kp_u_Q_and_R_must_be_2_D, __pyx_k_Q_and_R_must_be_2_D, sizeof(__pyx_k_Q_and_R_must_be_2_D), 0, 1, 0, 0},
  {&__pyx_kp_u_Q_and_R_must_have_the_same_dtype, __pyx_k_Q_and_R_must_have_the_same_dtype, sizeof(__pyx_k_Q_and_R_must_have_the_same_dtype), 0, 1, 0, 0},
  {&__pyx_n_s_R, __pyx_k_R, sizeof(__pyx_k_R), 0, 0, 1, 1},
  {&__pyx_kp_u_Rank_k_QR_update_If_A_Q_R_is_th, __pyx_k_Rank_k_QR_update_If_A_Q_R_is_th, sizeof(__pyx_k_Rank_k_QR_update_If_A_Q_R_is_th), 0, 1, 0, 0},
  {&__pyx_kp_u_Reorthogonalization_Failed_unabl, __pyx_k_Reorthogonalization_Failed_unabl, sizeof(__pyx_k_Reorthogonalization_Failed_unabl), 0, 1, 0, 0},
  {&__pyx_n_s_RuntimeError, __pyx_k_RuntimeError, sizeof(__pyx_k_RuntimeError), 0, 0, 1, 1},
  {&__pyx_kp_u_Second_dimension_of_u_and_v_must, __pyx_k_Second_dimension_of_u_and_v_must, sizeof(__pyx_k_Second_dimension_of_u_and_v_must), 0, 1, 0, 0},
  {&__pyx_kp_u_Shape_of_u_is_incorrect_should_b, __pyx_k_Shape_of_u_is_incorrect_should_b, sizeof(__pyx_k_Shape_of_u_is_incorrect_should_b), 0, 1, 0, 0},
  {&__pyx_n_s_T, __pyx_k_T, sizeof(__pyx_k_T), 0, 0, 1, 1},
  {&__pyx_kp_u_The_0_th_argument_to_geqrf_was_i, __pyx_k_The_0_th_argument_to_geqrf_was_i, sizeof(__pyx_k_The_0_th_argument_to_geqrf_was_i), 0, 1, 0, 0},
  {&__pyx_kp_u_The_0_th_argument_to_ormqr_unmqr, __pyx_k_The_0_th_argument_to_ormqr_unmqr, sizeof(__pyx_k_The_0_th_argument_to_ormqr_unmqr), 0, 1, 0, 0},
  {&__pyx_kp_u_Unable_to_allocate_memory_for_ar, __pyx_k_Unable_to_allocate_memory_for_ar, sizeof(__pyx_k_Unable_to_allocate_memory_for_ar), 0, 1, 0, 0},
  {&__pyx_kp_u_Unable_to_allocate_memory_for_ar_2, __pyx_k_Unable_to_allocate_memory_for_ar_2, sizeof(__pyx_k_Unable_to_allocate_memory_for_ar_2), 0, 1, 0, 0},
  {&__pyx_kp_u_Update_rank_larger_than_np_dot_Q, __pyx_k_Update_rank_larger_than_np_dot_Q, sizeof(__pyx_k_Update_rank_larger_than_np_dot_Q), 0, 1, 0, 0},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_kp_u__11, __pyx_k__11, sizeof(__pyx_k__11), 0, 1, 0, 0},
  {&__pyx_n_s_a, __pyx_k_a, sizeof(__pyx_k_a), 0, 0, 1, 1},
  {&__pyx_n_s_all, __pyx_k_all, sizeof(__pyx_k_all), 0, 0, 1, 1},
  {&__pyx_n_s_all_2, __pyx_k_all_2, sizeof(__pyx_k_all_2), 0, 0, 1, 1},
  {&__pyx_kp_u_array_must_not_contain_infs_or_N, __pyx_k_array_must_not_contain_infs_or_N, sizeof(__pyx_k_array_must_not_contain_infs_or_N), 0, 1, 0, 0},
  {&__pyx_n_s_b, __pyx_k_b, sizeof(__pyx_k_b), 0, 0, 1, 1},
  {&__pyx_n_s_check_finite, __pyx_k_check_finite, sizeof(__pyx_k_check_finite), 0, 0, 1, 1},
  {&__pyx_n_s_chkfinite, __pyx_k_chkfinite, sizeof(__pyx_k_chkfinite), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_u_col, __pyx_k_col, sizeof(__pyx_k_col), 0, 1, 0, 1},
  {&__pyx_n_s_conj, __pyx_k_conj, sizeof(__pyx_k_conj), 0, 0, 1, 1},
  {&__pyx_n_s_conjugate, __pyx_k_conjugate, sizeof(__pyx_k_conjugate), 0, 0, 1, 1},
  {&__pyx_n_s_copy, __pyx_k_copy, sizeof(__pyx_k_copy), 0, 0, 1, 1},
  {&__pyx_kp_s_decomp_update_pyx, __pyx_k_decomp_update_pyx, sizeof(__pyx_k_decomp_update_pyx), 0, 0, 1, 0},
  {&__pyx_n_s_delete, __pyx_k_delete, sizeof(__pyx_k_delete), 0, 0, 1, 1},
  {&__pyx_n_s_dot, __pyx_k_dot, sizeof(__pyx_k_dot), 0, 0, 1, 1},
  {&__pyx_n_s_economic, __pyx_k_economic, sizeof(__pyx_k_economic), 0, 0, 1, 1},
  {&__pyx_kp_u_for_Q_and_R_respectively, __pyx_k_for_Q_and_R_respectively, sizeof(__pyx_k_for_Q_and_R_respectively), 0, 1, 0, 0},
  {&__pyx_n_s_form_qTu, __pyx_k_form_qTu, sizeof(__pyx_k_form_qTu), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_full, __pyx_k_full, sizeof(__pyx_k_full), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_info, __pyx_k_info, sizeof(__pyx_k_info), 0, 0, 1, 1},
  {&__pyx_n_s_insert, __pyx_k_insert, sizeof(__pyx_k_insert), 0, 0, 1, 1},
  {&__pyx_n_s_intp, __pyx_k_intp, sizeof(__pyx_k_intp), 0, 0, 1, 1},
  {&__pyx_n_s_isfinite, __pyx_k_isfinite, sizeof(__pyx_k_isfinite), 0, 0, 1, 1},
  {&__pyx_n_s_k, __pyx_k_k, sizeof(__pyx_k_k), 0, 0, 1, 1},
  {&__pyx_n_s_k1, __pyx_k_k1, sizeof(__pyx_k_k1), 0, 0, 1, 1},
  {&__pyx_kp_u_k_is_out_of_bounds, __pyx_k_k_is_out_of_bounds, sizeof(__pyx_k_k_is_out_of_bounds), 0, 1, 0, 0},
  {&__pyx_n_s_length, __pyx_k_length, sizeof(__pyx_k_length), 0, 0, 1, 1},
  {&__pyx_n_s_linalg, __pyx_k_linalg, sizeof(__pyx_k_linalg), 0, 0, 1, 1},
  {&__pyx_n_s_m, __pyx_k_m, sizeof(__pyx_k_m), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_n, __pyx_k_n, sizeof(__pyx_k_n), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_kp_u_ndarray_is_not_C_contiguous, __pyx_k_ndarray_is_not_C_contiguous, sizeof(__pyx_k_ndarray_is_not_C_contiguous), 0, 1, 0, 0},
  {&__pyx_kp_u_ndarray_is_not_Fortran_contiguou, __pyx_k_ndarray_is_not_Fortran_contiguou, sizeof(__pyx_k_ndarray_is_not_Fortran_contiguou), 0, 1, 0, 0},
  {&__pyx_n_s_ndim, __pyx_k_ndim, sizeof(__pyx_k_ndim), 0, 0, 1, 1},
  {&__pyx_n_s_norm, __pyx_k_norm, sizeof(__pyx_k_norm), 0, 0, 1, 1},
  {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_kp_u_numpy_core_multiarray_failed_to, __pyx_k_numpy_core_multiarray_failed_to, sizeof(__pyx_k_numpy_core_multiarray_failed_to), 0, 1, 0, 0},
  {&__pyx_kp_u_numpy_core_umath_failed_to_impor, __pyx_k_numpy_core_umath_failed_to_impor, sizeof(__pyx_k_numpy_core_umath_failed_to_impor), 0, 1, 0, 0},
  {&__pyx_n_s_numpy_linalg, __pyx_k_numpy_linalg, sizeof(__pyx_k_numpy_linalg), 0, 0, 1, 1},
  {&__pyx_n_s_o, __pyx_k_o, sizeof(__pyx_k_o), 0, 0, 1, 1},
  {&__pyx_n_s_overwrite, __pyx_k_overwrite, sizeof(__pyx_k_overwrite), 0, 0, 1, 1},
  {&__pyx_n_s_overwrite_qr, __pyx_k_overwrite_qr, sizeof(__pyx_k_overwrite_qr), 0, 0, 1, 1},
  {&__pyx_n_s_overwrite_qru, __pyx_k_overwrite_qru, sizeof(__pyx_k_overwrite_qru), 0, 0, 1, 1},
  {&__pyx_n_s_overwrite_qruv, __pyx_k_overwrite_qruv, sizeof(__pyx_k_overwrite_qruv), 0, 0, 1, 1},
  {&__pyx_n_s_p, __pyx_k_p, sizeof(__pyx_k_p), 0, 0, 1, 1},
  {&__pyx_n_s_p1, __pyx_k_p1, sizeof(__pyx_k_p1), 0, 0, 1, 1},
  {&__pyx_n_s_p_eco, __pyx_k_p_eco, sizeof(__pyx_k_p_eco), 0, 0, 1, 1},
  {&__pyx_n_s_p_full, __pyx_k_p_full, sizeof(__pyx_k_p_full), 0, 0, 1, 1},
  {&__pyx_kp_u_p_is_out_of_range, __pyx_k_p_is_out_of_range, sizeof(__pyx_k_p_is_out_of_range), 0, 1, 0, 0},
  {&__pyx_n_s_q, __pyx_k_q, sizeof(__pyx_k_q), 0, 0, 1, 1},
  {&__pyx_n_s_q1, __pyx_k_q1, sizeof(__pyx_k_q1), 0, 0, 1, 1},
  {&__pyx_n_s_qTu, __pyx_k_qTu, sizeof(__pyx_k_qTu), 0, 0, 1, 1},
  {&__pyx_n_s_qTuptr, __pyx_k_qTuptr, sizeof(__pyx_k_qTuptr), 0, 0, 1, 1},
  {&__pyx_n_s_qTus, __pyx_k_qTus, sizeof(__pyx_k_qTus), 0, 0, 1, 1},
  {&__pyx_n_s_qTuvoid, __pyx_k_qTuvoid, sizeof(__pyx_k_qTuvoid), 0, 0, 1, 1},
  {&__pyx_kp_u_q_and_u_must_be_a_blas_compatibl, __pyx_k_q_and_u_must_be_a_blas_compatibl, sizeof(__pyx_k_q_and_u_must_be_a_blas_compatibl), 0, 1, 0, 0},
  {&__pyx_kp_u_q_and_u_must_have_the_same_type, __pyx_k_q_and_u_must_have_the_same_type, sizeof(__pyx_k_q_and_u_must_have_the_same_type), 0, 1, 0, 0},
  {&__pyx_kp_u_q_must_be_either_F_or_C_contiguo, __pyx_k_q_must_be_either_F_or_C_contiguo, sizeof(__pyx_k_q_must_be_either_F_or_C_contiguo), 0, 1, 0, 0},
  {&__pyx_n_s_qisF, __pyx_k_qisF, sizeof(__pyx_k_qisF), 0, 0, 1, 1},
  {&__pyx_n_s_qnew, __pyx_k_qnew, sizeof(__pyx_k_qnew), 0, 0, 1, 1},
  {&__pyx_n_s_qptr, __pyx_k_qptr, sizeof(__pyx_k_qptr), 0, 0, 1, 1},
  {&__pyx_n_s_qr_delete, __pyx_k_qr_delete, sizeof(__pyx_k_qr_delete), 0, 0, 1, 1},
  {&__pyx_n_u_qr_delete, __pyx_k_qr_delete, sizeof(__pyx_k_qr_delete), 0, 1, 0, 1},
  {&__pyx_kp_u_qr_delete_line_1451, __pyx_k_qr_delete_line_1451, sizeof(__pyx_k_qr_delete_line_1451), 0, 1, 0, 0},
  {&__pyx_n_s_qr_insert, __pyx_k_qr_insert, sizeof(__pyx_k_qr_insert), 0, 0, 1, 1},
  {&__pyx_n_u_qr_insert, __pyx_k_qr_insert, sizeof(__pyx_k_qr_insert), 0, 1, 0, 1},
  {&__pyx_kp_u_qr_insert_line_1695, __pyx_k_qr_insert_line_1695, sizeof(__pyx_k_qr_insert_line_1695), 0, 1, 0, 0},
  {&__pyx_n_s_qr_update, __pyx_k_qr_update, sizeof(__pyx_k_qr_update), 0, 0, 1, 1},
  {&__pyx_n_u_qr_update, __pyx_k_qr_update, sizeof(__pyx_k_qr_update), 0, 1, 0, 1},
  {&__pyx_kp_u_qr_update_line_2159, __pyx_k_qr_update_line_2159, sizeof(__pyx_k_qr_update_line_2159), 0, 1, 0, 0},
  {&__pyx_n_s_qs, __pyx_k_qs, sizeof(__pyx_k_qs), 0, 0, 1, 1},
  {&__pyx_n_s_r1, __pyx_k_r1, sizeof(__pyx_k_r1), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_ravel, __pyx_k_ravel, sizeof(__pyx_k_ravel), 0, 0, 1, 1},
  {&__pyx_n_s_rcond, __pyx_k_rcond, sizeof(__pyx_k_rcond), 0, 0, 1, 1},
  {&__pyx_kp_u_rcond_is_not_used_when_updating, __pyx_k_rcond_is_not_used_when_updating, sizeof(__pyx_k_rcond_is_not_used_when_updating), 0, 1, 0, 0},
  {&__pyx_kp_u_rcond_is_unused_when_inserting, __pyx_k_rcond_is_unused_when_inserting, sizeof(__pyx_k_rcond_is_unused_when_inserting), 0, 1, 0, 0},
  {&__pyx_n_s_rnew, __pyx_k_rnew, sizeof(__pyx_k_rnew), 0, 0, 1, 1},
  {&__pyx_n_u_row, __pyx_k_row, sizeof(__pyx_k_row), 0, 1, 0, 1},
  {&__pyx_n_s_rptr, __pyx_k_rptr, sizeof(__pyx_k_rptr), 0, 0, 1, 1},
  {&__pyx_n_s_rs, __pyx_k_rs, sizeof(__pyx_k_rs), 0, 0, 1, 1},
  {&__pyx_n_s_s, __pyx_k_s, sizeof(__pyx_k_s), 0, 0, 1, 1},
  {&__pyx_n_s_scipy_linalg__decomp_update, __pyx_k_scipy_linalg__decomp_update, sizeof(__pyx_k_scipy_linalg__decomp_update), 0, 0, 1, 1},
  {&__pyx_n_u_shape, __pyx_k_shape, sizeof(__pyx_k_shape), 0, 1, 0, 1},
  {&__pyx_n_s_sptr, __pyx_k_sptr, sizeof(__pyx_k_sptr), 0, 0, 1, 1},
  {&__pyx_n_s_ss, __pyx_k_ss, sizeof(__pyx_k_ss), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_typecode, __pyx_k_typecode, sizeof(__pyx_k_typecode), 0, 0, 1, 1},
  {&__pyx_n_s_u, __pyx_k_u, sizeof(__pyx_k_u), 0, 0, 1, 1},
  {&__pyx_n_s_u1, __pyx_k_u1, sizeof(__pyx_k_u1), 0, 0, 1, 1},
  {&__pyx_kp_u_u_and_v_must_have_the_same_numbe, __pyx_k_u_and_v_must_have_the_same_numbe, sizeof(__pyx_k_u_and_v_must_have_the_same_numbe), 0, 1, 0, 0},
  {&__pyx_kp_u_u_and_v_must_have_the_same_type, __pyx_k_u_and_v_must_have_the_same_type, sizeof(__pyx_k_u_and_v_must_have_the_same_type), 0, 1, 0, 0},
  {&__pyx_kp_u_u_must_be_either_1_or_2_D, __pyx_k_u_must_be_either_1_or_2_D, sizeof(__pyx_k_u_must_be_either_1_or_2_D), 0, 1, 0, 0},
  {&__pyx_kp_u_u_must_be_either_1_or_2_D_2, __pyx_k_u_must_be_either_1_or_2_D_2, sizeof(__pyx_k_u_must_be_either_1_or_2_D_2), 0, 1, 0, 0},
  {&__pyx_kp_u_u_must_have_the_same_type_as_Q, __pyx_k_u_must_have_the_same_type_as_Q, sizeof(__pyx_k_u_must_have_the_same_type_as_Q), 0, 1, 0, 0},
  {&__pyx_kp_u_u_must_have_the_same_type_as_Q_2, __pyx_k_u_must_have_the_same_type_as_Q_2, sizeof(__pyx_k_u_must_have_the_same_type_as_Q_2), 0, 1, 0, 0},
  {&__pyx_kp_u_u_shape_0_must_equal_Q_shape_0, __pyx_k_u_shape_0_must_equal_Q_shape_0, sizeof(__pyx_k_u_shape_0_must_equal_Q_shape_0), 0, 1, 0, 0},
  {&__pyx_kp_u_u_should_be_either_M_or_M_p_whe, __pyx_k_u_should_be_either_M_or_M_p_whe, sizeof(__pyx_k_u_should_be_either_M_or_M_p_whe), 0, 1, 0, 0},
  {&__pyx_kp_u_u_should_be_either_N_or_p_N_whe, __pyx_k_u_should_be_either_N_or_p_N_whe, sizeof(__pyx_k_u_should_be_either_N_or_p_N_whe), 0, 1, 0, 0},
  {&__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_k_unknown_dtype_code_in_numpy_pxd, sizeof(__pyx_k_unknown_dtype_code_in_numpy_pxd), 0, 1, 0, 0},
  {&__pyx_n_s_uptr, __pyx_k_uptr, sizeof(__pyx_k_uptr), 0, 0, 1, 1},
  {&__pyx_n_s_us, __pyx_k_us, sizeof(__pyx_k_us), 0, 0, 1, 1},
  {&__pyx_n_s_uv_flags, __pyx_k_uv_flags, sizeof(__pyx_k_uv_flags), 0, 0, 1, 1},
  {&__pyx_n_s_v, __pyx_k_v, sizeof(__pyx_k_v), 0, 0, 1, 1},
  {&__pyx_n_s_v1, __pyx_k_v1, sizeof(__pyx_k_v1), 0, 0, 1, 1},
  {&__pyx_kp_u_v_must_be_either_1_or_2_D, __pyx_k_v_must_be_either_1_or_2_D, sizeof(__pyx_k_v_must_be_either_1_or_2_D), 0, 1, 0, 0},
  {&__pyx_kp_u_v_shape_0_must_equal_R_shape_1, __pyx_k_v_shape_0_must_equal_R_shape_1, sizeof(__pyx_k_v_shape_0_must_equal_R_shape_1), 0, 1, 0, 0},
  {&__pyx_n_s_vptr, __pyx_k_vptr, sizeof(__pyx_k_vptr), 0, 0, 1, 1},
  {&__pyx_n_s_vs, __pyx_k_vs, sizeof(__pyx_k_vs), 0, 0, 1, 1},
  {&__pyx_n_s_which, __pyx_k_which, sizeof(__pyx_k_which), 0, 0, 1, 1},
  {&__pyx_kp_u_which_must_be_either_row_or_col, __pyx_k_which_must_be_either_row_or_col, sizeof(__pyx_k_which_must_be_either_row_or_col), 0, 1, 0, 0},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 289, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 1211, __pyx_L1_error)
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) __PYX_ERR(0, 1617, __pyx_L1_error)
  __pyx_builtin_RuntimeError = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeError); if (!__pyx_builtin_RuntimeError) __PYX_ERR(1, 856, __pyx_L1_error)
  __pyx_builtin_ImportError = __Pyx_GetBuiltinName(__pyx_n_s_ImportError); if (!__pyx_builtin_ImportError) __PYX_ERR(1, 1038, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "scipy/linalg/_decomp_update.pyx":1211
 * 
 *     if not cnp.PyArray_Check(a) or not cnp.PyArray_Check(b):
 *         raise ValueError('Inputs must be arrays')             # <<<<<<<<<<<<<<
 * 
 *     q = a
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_u_Inputs_must_be_arrays); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 1211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "scipy/linalg/_decomp_update.pyx":1218
 *     typecode = cnp.PyArray_TYPE(q)
 *     if cnp.PyArray_TYPE(u) != typecode:
 *         raise ValueError('q and u must have the same type.')             # <<<<<<<<<<<<<<
 * 
 *     if not (typecode == cnp.NPY_FLOAT or typecode == cnp.NPY_DOUBLE \
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_u_q_and_u_must_have_the_same_type); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(0, 1218, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "scipy/linalg/_decomp_update.pyx":1222
 *     if not (typecode == cnp.NPY_FLOAT or typecode == cnp.NPY_DOUBLE \
 *             or typecode == cnp.NPY_CFLOAT or typecode == cnp.NPY_CDOUBLE):
 *         raise ValueError('q and u must be a blas compatible type: f d F or D')             # <<<<<<<<<<<<<<
 * 
 *     q = validate_array(q, True)
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_u_q_and_u_must_be_a_blas_compatibl); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 1222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "scipy/linalg/_decomp_update.pyx":1360
 *                 blas_t_2d_conj(m, p, col(<double_complex*>qTuvoid, qTus, k), qTus)
 *         else:
 *             raise ValueError('Shape of u is incorrect, should be 1 <= u.ndim <= 2')             # <<<<<<<<<<<<<<
 *     else:
 *         raise ValueError('q must be either F or C contiguous')
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_u_Shape_of_u_is_incorrect_should_b); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 1360, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "scipy/linalg/_decomp_update.pyx":1362
 *             raise ValueError('Shape of u is incorrect, should be 1 <= u.ndim <= 2')
 *     else:
 *         raise ValueError('q must be either F or C contiguous')             # <<<<<<<<<<<<<<
 * 
 * cdef validate_array(cnp.ndarray a, bint chkfinite):
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_u_q_must_be_either_F_or_C_contiguo); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 1362, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "scipy/linalg/_decomp_update.pyx":1376
 *             copy = True
 *         if a.shape[j] >= libc.limits.INT_MAX:
 *             raise ValueError('Input array too large for use with BLAS')             # <<<<<<<<<<<<<<
 * 
 *     if chkfinite:
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_u_Input_array_too_large_for_use_wi); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 1376, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "scipy/linalg/_decomp_update.pyx":1380
 *     if chkfinite:
 *         if not np.isfinite(a).all():
 *             raise ValueError('array must not contain infs or NaNs')             # <<<<<<<<<<<<<<
 * 
 *     if copy:
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_u_array_must_not_contain_infs_or_N); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(0, 1380, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "scipy/linalg/_decomp_update.pyx":1408
 * 
 *     if Q.ndim != 2 or R.ndim != 2:
 *         raise ValueError('Q and R must be 2-D')             # <<<<<<<<<<<<<<
 * 
 *     typecode = cnp.PyArray_TYPE(Q)
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_u_Q_and_R_must_be_2_D); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 1408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "scipy/linalg/_decomp_update.pyx":1413
 * 
 *     if typecode != cnp.PyArray_TYPE(R):
 *         raise ValueError('Q and R must have the same dtype')             # <<<<<<<<<<<<<<
 * 
 *     if not (typecode == cnp.NPY_FLOAT or typecode == cnp.NPY_DOUBLE
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_u_Q_and_R_must_have_the_same_dtype); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 1413, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "scipy/linalg/_decomp_update.pyx":1417
 *     if not (typecode == cnp.NPY_FLOAT or typecode == cnp.NPY_DOUBLE
 *             or typecode == cnp.NPY_CFLOAT or typecode == cnp.NPY_CDOUBLE):
 *         raise ValueError('Only arrays with dtypes float32, float64, '             # <<<<<<<<<<<<<<
 *                          'complex64, and complex128 are supported.')
 * 
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_u_Only_arrays_with_dtypes_float32); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(0, 1417, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "scipy/linalg/_decomp_update.pyx":1575
 *                 ARRAY_ANYORDER, overwrite, ARRAY_ANYORDER, chkfinite)
 *         if not (-m <= k1 < m):
 *             raise ValueError("'k' is out of bounds")             # <<<<<<<<<<<<<<
 *         if k1 < 0:
 *             k1 += m
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_kp_u_k_is_out_of_bounds); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(0, 1575, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "scipy/linalg/_decomp_update.pyx":1579
 *             k1 += m
 *         if k1 + p1 > m or p1 <= 0:
 *             raise ValueError("'p' is out of range")             # <<<<<<<<<<<<<<
 *         if economic:
 *             # handle the special case of (M,1), (1,1)
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_u_p_is_out_of_range); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 1579, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "scipy/linalg/_decomp_update.pyx":1615
 *                         <double_complex*>rptr, rs, k1, p_eco, p_full)
 *             if info == 1:
 *                 return q1[p_full:-p_eco, p_full:], r1[p_full:,:]             # <<<<<<<<<<<<<<
 *             elif info == MEMORY_ERROR:
 *                 raise MemoryError('Unable to allocate memory for array.')
 */
  __pyx_slice__14 = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_slice__14)) __PYX_ERR(0, 1615, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__14);
  __Pyx_GIVEREF(__pyx_slice__14);

  /* "scipy/linalg/_decomp_update.pyx":1617
 *                 return q1[p_full:-p_eco, p_full:], r1[p_full:,:]
 *             elif info == MEMORY_ERROR:
 *                 raise MemoryError('Unable to allocate memory for array.')             # <<<<<<<<<<<<<<
 *             else:
 *                 raise ValueError('Reorthogonalization Failed, unable to perform row deletion.')
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_kp_u_Unable_to_allocate_memory_for_ar); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 1617, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "scipy/linalg/_decomp_update.pyx":1619
 *                 raise MemoryError('Unable to allocate memory for array.')
 *             else:
 *                 raise ValueError('Reorthogonalization Failed, unable to perform row deletion.')             # <<<<<<<<<<<<<<
 *         else:
 *             qptr = extract(q1, qs)
 */
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_kp_u_Reorthogonalization_Failed_unabl); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(0, 1619, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "scipy/linalg/_decomp_update.pyx":1686
 *                         <double_complex*>rptr, rs, k1, p1)
 *             if info == MEMORY_ERROR:
 *                 raise MemoryError('Unable to allocate memory for array')             # <<<<<<<<<<<<<<
 *         if economic:
 *             return q1[:, :-p], r1[:-p, :-p]
 */
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_kp_u_Unable_to_allocate_memory_for_ar_2); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 1686, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "scipy/linalg/_decomp_update.pyx":1692
 *             return q1, r1[:, :-p]
 *     else:
 *         raise ValueError("'which' must be either 'row' or 'col'")             # <<<<<<<<<<<<<<
 * 
 * @cython.embedsignature(True)
 */
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_kp_u_which_must_be_either_row_or_col); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(0, 1692, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "scipy/linalg/_decomp_update.pyx":1827
 *     if which == 'row':
 *         if rcond is not None:
 *             raise ValueError("'rcond' is unused when inserting rows and "             # <<<<<<<<<<<<<<
 *                              "must be None")
 *         return qr_insert_row(Q, R, u, k1, overwrite, chkfinite)
 */
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_kp_u_rcond_is_unused_when_inserting); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(0, 1827, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "scipy/linalg/_decomp_update.pyx":1859
 * 
 *     if cnp.PyArray_TYPE(u1) != typecode:
 *         raise ValueError("'u' must have the same type as 'Q' and 'R'")             # <<<<<<<<<<<<<<
 * 
 *     if not (-m <= k <= m):
 */
  __pyx_tuple__20 = PyTuple_Pack(1, __pyx_kp_u_u_must_have_the_same_type_as_Q); if (unlikely(!__pyx_tuple__20)) __PYX_ERR(0, 1859, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);

  /* "scipy/linalg/_decomp_update.pyx":1880
 *                              str(getattr(u1, 'shape')))
 *     else:
 *         raise ValueError("'u' must be either 1- or 2-D")             # <<<<<<<<<<<<<<
 * 
 *     u1 = validate_array(u1, check_finite)
 */
  __pyx_tuple__21 = PyTuple_Pack(1, __pyx_kp_u_u_must_be_either_1_or_2_D); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(0, 1880, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);

  /* "scipy/linalg/_decomp_update.pyx":2014
 * 
 *     if cnp.PyArray_TYPE(u1) != typecode:
 *         raise ValueError("'u' must have the same type as Q and R")             # <<<<<<<<<<<<<<
 *     if not (-n <= k <= n):
 *         raise ValueError("'k' is out of bounds")
 */
  __pyx_tuple__22 = PyTuple_Pack(1, __pyx_kp_u_u_must_have_the_same_type_as_Q_2); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(0, 2014, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);

  /* "scipy/linalg/_decomp_update.pyx":2041
 *             frc = rcond
 *     elif rcond is not None:
 *         raise ValueError("'rcond' is not used when updating full, (M,M) (M,N) "             # <<<<<<<<<<<<<<
 *                          "decompositions and must be None.")
 * 
 */
  __pyx_tuple__23 = PyTuple_Pack(1, __pyx_kp_u_rcond_is_not_used_when_updating); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(0, 2041, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);

  /* "scipy/linalg/_decomp_update.pyx":2342
 * 
 *     if cnp.PyArray_TYPE(u1) != typecode or cnp.PyArray_TYPE(v1) != typecode:
 *         raise ValueError('u and v must have the same type as Q and R')             # <<<<<<<<<<<<<<
 * 
 *     if u1.ndim < 1 or u1.ndim > 2:
 */
  __pyx_tuple__24 = PyTuple_Pack(1, __pyx_kp_u_u_and_v_must_have_the_same_type); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(0, 2342, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);

  /* "scipy/linalg/_decomp_update.pyx":2345
 * 
 *     if u1.ndim < 1 or u1.ndim > 2:
 *         raise ValueError('u must be either 1- or 2-D.')             # <<<<<<<<<<<<<<
 * 
 *     if v1.ndim < 1 or v1.ndim > 2:
 */
  __pyx_tuple__25 = PyTuple_Pack(1, __pyx_kp_u_u_must_be_either_1_or_2_D_2); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(0, 2345, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);

  /* "scipy/linalg/_decomp_update.pyx":2348
 * 
 *     if v1.ndim < 1 or v1.ndim > 2:
 *         raise ValueError('v must be either 1- or 2-D.')             # <<<<<<<<<<<<<<
 * 
 *     if u1.shape[0] != m:
 */
  __pyx_tuple__26 = PyTuple_Pack(1, __pyx_kp_u_v_must_be_either_1_or_2_D); if (unlikely(!__pyx_tuple__26)) __PYX_ERR(0, 2348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);

  /* "scipy/linalg/_decomp_update.pyx":2351
 * 
 *     if u1.shape[0] != m:
 *         raise ValueError('u.shape[0] must equal Q.shape[0]')             # <<<<<<<<<<<<<<
 * 
 *     if v1.shape[0] != n:
 */
  __pyx_tuple__27 = PyTuple_Pack(1, __pyx_kp_u_u_shape_0_must_equal_Q_shape_0); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(0, 2351, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);

  /* "scipy/linalg/_decomp_update.pyx":2354
 * 
 *     if v1.shape[0] != n:
 *         raise ValueError('v.shape[0] must equal R.shape[1]')             # <<<<<<<<<<<<<<
 * 
 *     if u1.ndim != v1.ndim:
 */
  __pyx_tuple__28 = PyTuple_Pack(1, __pyx_kp_u_v_shape_0_must_equal_R_shape_1); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(0, 2354, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);

  /* "scipy/linalg/_decomp_update.pyx":2357
 * 
 *     if u1.ndim != v1.ndim:
 *         raise ValueError('u and v must have the same number of dimensions')             # <<<<<<<<<<<<<<
 * 
 *     if u1.ndim == 2:
 */
  __pyx_tuple__29 = PyTuple_Pack(1, __pyx_kp_u_u_and_v_must_have_the_same_numbe); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(0, 2357, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);

  /* "scipy/linalg/_decomp_update.pyx":2361
 *     if u1.ndim == 2:
 *         if u1.shape[1] != v1.shape[1]:
 *             raise ValueError('Second dimension of u and v must be the same')             # <<<<<<<<<<<<<<
 *         elif u1.shape[1] == 1:
 *             u1 = u1[:,0]
 */
  __pyx_tuple__30 = PyTuple_Pack(1, __pyx_kp_u_Second_dimension_of_u_and_v_must); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(0, 2361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);

  /* "scipy/linalg/_decomp_update.pyx":2363
 *             raise ValueError('Second dimension of u and v must be the same')
 *         elif u1.shape[1] == 1:
 *             u1 = u1[:,0]             # <<<<<<<<<<<<<<
 *             v1 = v1[:,0]
 * 
 */
  __pyx_tuple__31 = PyTuple_Pack(2, __pyx_slice__14, __pyx_int_0); if (unlikely(!__pyx_tuple__31)) __PYX_ERR(0, 2363, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);

  /* "scipy/linalg/_decomp_update.pyx":2373
 *     # limit p to at most max(n, m)
 *     if p > n or p > m:
 *         raise ValueError('Update rank larger than np.dot(Q, R).')             # <<<<<<<<<<<<<<
 * 
 *     u1 = validate_array(u1, chkfinite)
 */
  __pyx_tuple__32 = PyTuple_Pack(1, __pyx_kp_u_Update_rank_larger_than_np_dot_Q); if (unlikely(!__pyx_tuple__32)) __PYX_ERR(0, 2373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__32);
  __Pyx_GIVEREF(__pyx_tuple__32);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":272
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
  __pyx_tuple__33 = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_C_contiguous); if (unlikely(!__pyx_tuple__33)) __PYX_ERR(1, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__33);
  __Pyx_GIVEREF(__pyx_tuple__33);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":276
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
  __pyx_tuple__34 = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_Fortran_contiguou); if (unlikely(!__pyx_tuple__34)) __PYX_ERR(1, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":306
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
  __pyx_tuple__35 = PyTuple_Pack(1, __pyx_kp_u_Non_native_byte_order_not_suppor); if (unlikely(!__pyx_tuple__35)) __PYX_ERR(1, 306, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__35);
  __Pyx_GIVEREF(__pyx_tuple__35);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":856
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
  __pyx_tuple__36 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor); if (unlikely(!__pyx_tuple__36)) __PYX_ERR(1, 856, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__36);
  __Pyx_GIVEREF(__pyx_tuple__36);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":880
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
  __pyx_tuple__37 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor_2); if (unlikely(!__pyx_tuple__37)) __PYX_ERR(1, 880, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__37);
  __Pyx_GIVEREF(__pyx_tuple__37);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1038
 *         _import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
  __pyx_tuple__38 = PyTuple_Pack(1, __pyx_kp_u_numpy_core_multiarray_failed_to); if (unlikely(!__pyx_tuple__38)) __PYX_ERR(1, 1038, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__38);
  __Pyx_GIVEREF(__pyx_tuple__38);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1044
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
  __pyx_tuple__39 = PyTuple_Pack(1, __pyx_kp_u_numpy_core_umath_failed_to_impor); if (unlikely(!__pyx_tuple__39)) __PYX_ERR(1, 1044, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__39);
  __Pyx_GIVEREF(__pyx_tuple__39);

  /* "scipy/linalg/_decomp_update.pyx":1200
 *     return 0
 * 
 * def _form_qTu(object a, object b):             # <<<<<<<<<<<<<<
 *     """ this function only exists to expose the cdef version below for testing
 *         purposes. Here we perform minimal input validation to ensure that the
 */
  __pyx_tuple__40 = PyTuple_Pack(8, __pyx_n_s_a, __pyx_n_s_b, __pyx_n_s_q, __pyx_n_s_u, __pyx_n_s_qTu, __pyx_n_s_typecode, __pyx_n_s_qTuvoid, __pyx_n_s_qTus); if (unlikely(!__pyx_tuple__40)) __PYX_ERR(0, 1200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__40);
  __Pyx_GIVEREF(__pyx_tuple__40);
  __pyx_codeobj__41 = (PyObject*)__Pyx_PyCode_New(2, 0, 8, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__40, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_decomp_update_pyx, __pyx_n_s_form_qTu, 1200, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__41)) __PYX_ERR(0, 1200, __pyx_L1_error)

  /* "scipy/linalg/_decomp_update.pyx":1451
 * 
 * @cython.embedsignature(True)
 * def qr_delete(Q, R, k, int p=1, which='row', overwrite_qr=False,             # <<<<<<<<<<<<<<
 *               check_finite=True):
 *     """
 */
  __pyx_tuple__42 = PyTuple_Pack(28, __pyx_n_s_Q, __pyx_n_s_R, __pyx_n_s_k, __pyx_n_s_p, __pyx_n_s_which, __pyx_n_s_overwrite_qr, __pyx_n_s_check_finite, __pyx_n_s_q1, __pyx_n_s_r1, __pyx_n_s_k1, __pyx_n_s_p1, __pyx_n_s_p_eco, __pyx_n_s_p_full, __pyx_n_s_typecode, __pyx_n_s_m, __pyx_n_s_n, __pyx_n_s_info, __pyx_n_s_qptr, __pyx_n_s_rptr, __pyx_n_s_qs, __pyx_n_s_rs, __pyx_n_s_economic, __pyx_n_s_qisF, __pyx_n_s_chkfinite, __pyx_n_s_overwrite, __pyx_n_s_qnew, __pyx_n_s_norm, __pyx_n_s_o); if (unlikely(!__pyx_tuple__42)) __PYX_ERR(0, 1451, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__42);
  __Pyx_GIVEREF(__pyx_tuple__42);
  __pyx_codeobj__43 = (PyObject*)__Pyx_PyCode_New(7, 0, 28, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__42, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_decomp_update_pyx, __pyx_n_s_qr_delete, 1451, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__43)) __PYX_ERR(0, 1451, __pyx_L1_error)

  /* "scipy/linalg/_decomp_update.pyx":1695
 * 
 * @cython.embedsignature(True)
 * def qr_insert(Q, R, u, k, which='row', rcond=None, overwrite_qru=False, check_finite=True):             # <<<<<<<<<<<<<<
 *     """
 *     QR update on row or column insertions
 */
  __pyx_tuple__44 = PyTuple_Pack(11, __pyx_n_s_Q, __pyx_n_s_R, __pyx_n_s_u, __pyx_n_s_k, __pyx_n_s_which, __pyx_n_s_rcond, __pyx_n_s_overwrite_qru, __pyx_n_s_check_finite, __pyx_n_s_chkfinite, __pyx_n_s_overwrite, __pyx_n_s_k1); if (unlikely(!__pyx_tuple__44)) __PYX_ERR(0, 1695, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__44);
  __Pyx_GIVEREF(__pyx_tuple__44);
  __pyx_codeobj__45 = (PyObject*)__Pyx_PyCode_New(8, 0, 11, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__44, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_decomp_update_pyx, __pyx_n_s_qr_insert, 1695, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__45)) __PYX_ERR(0, 1695, __pyx_L1_error)

  /* "scipy/linalg/_decomp_update.pyx":2159
 * 
 * @cython.embedsignature(True)
 * def qr_update(Q, R, u, v, overwrite_qruv=False, check_finite=True):             # <<<<<<<<<<<<<<
 *     """
 *     Rank-k QR update
 */
  __pyx_tuple__46 = PyTuple_Pack(37, __pyx_n_s_Q, __pyx_n_s_R, __pyx_n_s_u, __pyx_n_s_v, __pyx_n_s_overwrite_qruv, __pyx_n_s_check_finite, __pyx_n_s_q1, __pyx_n_s_r1, __pyx_n_s_u1, __pyx_n_s_v1, __pyx_n_s_qTu, __pyx_n_s_s, __pyx_n_s_uv_flags, __pyx_n_s_typecode, __pyx_n_s_p, __pyx_n_s_m, __pyx_n_s_n, __pyx_n_s_info, __pyx_n_s_qptr, __pyx_n_s_rptr, __pyx_n_s_uptr, __pyx_n_s_vptr, __pyx_n_s_sptr, __pyx_n_s_qTuptr, __pyx_n_s_qs, __pyx_n_s_rs, __pyx_n_s_qTus, __pyx_n_s_us, __pyx_n_s_vs, __pyx_n_s_ss, __pyx_n_s_economic, __pyx_n_s_qisF, __pyx_n_s_chkfinite, __pyx_n_s_overwrite, __pyx_n_s_ndim, __pyx_n_s_length, __pyx_n_s_rnew); if (unlikely(!__pyx_tuple__46)) __PYX_ERR(0, 2159, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__46);
  __Pyx_GIVEREF(__pyx_tuple__46);
  __pyx_codeobj__47 = (PyObject*)__Pyx_PyCode_New(6, 0, 37, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__46, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_decomp_update_pyx, __pyx_n_s_qr_update, 2159, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__47)) __PYX_ERR(0, 2159, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(2, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("numpy"); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType(__pyx_t_1, "numpy", "dtype", sizeof(PyArray_Descr), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_dtype) __PYX_ERR(1, 206, __pyx_L1_error)
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType(__pyx_t_1, "numpy", "flatiter", sizeof(PyArrayIterObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_flatiter) __PYX_ERR(1, 229, __pyx_L1_error)
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType(__pyx_t_1, "numpy", "broadcast", sizeof(PyArrayMultiIterObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_broadcast) __PYX_ERR(1, 233, __pyx_L1_error)
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType(__pyx_t_1, "numpy", "ndarray", sizeof(PyArrayObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ndarray) __PYX_ERR(1, 242, __pyx_L1_error)
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType(__pyx_t_1, "numpy", "ufunc", sizeof(PyUFuncObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_ufunc) __PYX_ERR(1, 918, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __pyx_t_1 = PyImport_ImportModule("scipy.linalg.cython_blas"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "caxpy", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_caxpy, "void (int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "ccopy", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_ccopy, "void (int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "cgemm", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_cgemm, "void (char *, char *, int *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "cgemv", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_cgemv, "void (char *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "cgeru", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_cgeru, "void (int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "cscal", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_cscal, "void (int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "cswap", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_cswap, "void (int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "ctrmm", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_ctrmm, "void (char *, char *, char *, char *, int *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "daxpy", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_daxpy, "void (int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "dcopy", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_dcopy, "void (int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "dgemm", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_dgemm, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "dgemv", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_dgemv, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "dger", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_dger, "void (int *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "dnrm2", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_dnrm2, "__pyx_t_5scipy_6linalg_11cython_blas_d (int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "drot", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_drot, "void (int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, __pyx_t_5scipy_6linalg_11cython_blas_d *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "dscal", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_dscal, "void (int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "dswap", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_dswap, "void (int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "dtrmm", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_dtrmm, "void (char *, char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *, __pyx_t_5scipy_6linalg_11cython_blas_d *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "dznrm2", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_dznrm2, "__pyx_t_5scipy_6linalg_11cython_blas_d (int *, __pyx_t_double_complex *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "saxpy", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_saxpy, "void (int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "scnrm2", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_scnrm2, "__pyx_t_5scipy_6linalg_11cython_blas_s (int *, __pyx_t_float_complex *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "scopy", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_scopy, "void (int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "sgemm", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_sgemm, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "sgemv", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_sgemv, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "sger", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_sger, "void (int *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "snrm2", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_snrm2, "__pyx_t_5scipy_6linalg_11cython_blas_s (int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "srot", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_srot, "void (int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, __pyx_t_5scipy_6linalg_11cython_blas_s *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "sscal", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_sscal, "void (int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "sswap", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_sswap, "void (int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "strmm", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_strmm, "void (char *, char *, char *, char *, int *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *, __pyx_t_5scipy_6linalg_11cython_blas_s *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "zaxpy", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_zaxpy, "void (int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "zcopy", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_zcopy, "void (int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "zgemm", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_zgemm, "void (char *, char *, int *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "zgemv", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_zgemv, "void (char *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "zgeru", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_zgeru, "void (int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "zscal", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_zscal, "void (int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "zswap", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_zswap, "void (int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "ztrmm", (void (**)(void))&__pyx_f_5scipy_6linalg_11cython_blas_ztrmm, "void (char *, char *, char *, char *, int *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = PyImport_ImportModule("scipy.linalg.cython_lapack"); if (!__pyx_t_2) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "cgeqrf", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_cgeqrf, "void (int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "clarf", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_clarf, "void (char *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "clarfg", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_clarfg, "void (int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "clartg", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_clartg, "void (__pyx_t_float_complex *, __pyx_t_float_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *, __pyx_t_float_complex *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "crot", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_crot, "void (int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_float_complex *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "cunmqr", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_cunmqr, "void (char *, char *, int *, int *, int *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, __pyx_t_float_complex *, int *, __pyx_t_float_complex *, int *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "dgeqrf", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_dgeqrf, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "dlarf", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_dlarf, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "dlarfg", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_dlarfg, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "dlartg", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_dlartg, "void (__pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "dormqr", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_dormqr, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, int *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "sgeqrf", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_sgeqrf, "void (int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "slarf", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_slarf, "void (char *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "slarfg", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_slarfg, "void (int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "slartg", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_slartg, "void (__pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "sormqr", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_sormqr, "void (char *, char *, int *, int *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_s *, int *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "zgeqrf", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_zgeqrf, "void (int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "zlarf", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_zlarf, "void (char *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "zlarfg", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_zlarfg, "void (int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "zlartg", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_zlartg, "void (__pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *, __pyx_t_double_complex *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "zrot", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_zrot, "void (int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, __pyx_t_5scipy_6linalg_13cython_lapack_d *, __pyx_t_double_complex *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "zunmqr", (void (**)(void))&__pyx_f_5scipy_6linalg_13cython_lapack_zunmqr, "void (char *, char *, int *, int *, int *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, __pyx_t_double_complex *, int *, __pyx_t_double_complex *, int *, int *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_RefNannyFinishContext();
  return -1;
}


#if PY_MAJOR_VERSION < 3
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC void
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#else
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC init_decomp_update(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC init_decomp_update(void)
#else
__Pyx_PyMODINIT_FUNC PyInit__decomp_update(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit__decomp_update(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec__decomp_update(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module '_decomp_update' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit__decomp_update(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("_decomp_update", __pyx_methods, __pyx_k_Routines_for_updating_QR_decomp, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_scipy__linalg___decomp_update) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "scipy.linalg._decomp_update")) {
      if (unlikely(PyDict_SetItemString(modules, "scipy.linalg._decomp_update", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  (void)__Pyx_modinit_type_init_code();
  if (unlikely(__Pyx_modinit_type_import_code() != 0)) goto __pyx_L1_error;
  (void)__Pyx_modinit_variable_import_code();
  if (unlikely(__Pyx_modinit_function_import_code() != 0)) goto __pyx_L1_error;
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "scipy/linalg/_decomp_update.pyx":37
 * from __future__ import absolute_import
 * 
 * __all__ = ['qr_delete', 'qr_insert', 'qr_update']             # <<<<<<<<<<<<<<
 * 
 * cimport cython
 */
  __pyx_t_1 = PyList_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_u_qr_delete);
  __Pyx_GIVEREF(__pyx_n_u_qr_delete);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_u_qr_delete);
  __Pyx_INCREF(__pyx_n_u_qr_insert);
  __Pyx_GIVEREF(__pyx_n_u_qr_insert);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_u_qr_insert);
  __Pyx_INCREF(__pyx_n_u_qr_update);
  __Pyx_GIVEREF(__pyx_n_u_qr_update);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_u_qr_update);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_all_2, __pyx_t_1) < 0) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "scipy/linalg/_decomp_update.pyx":51
 *     double NPY_SQRT1_2
 * 
 * from numpy.linalg import LinAlgError             # <<<<<<<<<<<<<<
 * 
 * # This is used in place of, e.g., cnp.NPY_C_CONTIGUOUS, to indicate that a C
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_LinAlgError);
  __Pyx_GIVEREF(__pyx_n_s_LinAlgError);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_LinAlgError);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_numpy_linalg, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_LinAlgError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_LinAlgError, __pyx_t_1) < 0) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/linalg/_decomp_update.pyx":55
 * # This is used in place of, e.g., cnp.NPY_C_CONTIGUOUS, to indicate that a C
 * # F or non contiguous array is acceptable.
 * cdef int ARRAY_ANYORDER = 0             # <<<<<<<<<<<<<<
 * 
 * cdef int MEMORY_ERROR = libc.limits.INT_MAX
 */
  __pyx_v_5scipy_6linalg_14_decomp_update_ARRAY_ANYORDER = 0;

  /* "scipy/linalg/_decomp_update.pyx":57
 * cdef int ARRAY_ANYORDER = 0
 * 
 * cdef int MEMORY_ERROR = libc.limits.INT_MAX             # <<<<<<<<<<<<<<
 * 
 * # These are commented out in the numpy support we cimported above.
 */
  __pyx_v_5scipy_6linalg_14_decomp_update_MEMORY_ERROR = INT_MAX;

  /* "scipy/linalg/_decomp_update.pyx":69
 * from . cimport cython_lapack as lapack_pointers
 * 
 * import numpy as np             # <<<<<<<<<<<<<<
 * 
 * #------------------------------------------------------------------------------
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_2) < 0) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/linalg/_decomp_update.pyx":1200
 *     return 0
 * 
 * def _form_qTu(object a, object b):             # <<<<<<<<<<<<<<
 *     """ this function only exists to expose the cdef version below for testing
 *         purposes. Here we perform minimal input validation to ensure that the
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5scipy_6linalg_14_decomp_update_1_form_qTu, NULL, __pyx_n_s_scipy_linalg__decomp_update); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_form_qTu, __pyx_t_2) < 0) __PYX_ERR(0, 1200, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/linalg/_decomp_update.pyx":1451
 * 
 * @cython.embedsignature(True)
 * def qr_delete(Q, R, k, int p=1, which='row', overwrite_qr=False,             # <<<<<<<<<<<<<<
 *               check_finite=True):
 *     """
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5scipy_6linalg_14_decomp_update_3qr_delete, NULL, __pyx_n_s_scipy_linalg__decomp_update); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1451, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_qr_delete, __pyx_t_2) < 0) __PYX_ERR(0, 1451, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/linalg/_decomp_update.pyx":1695
 * 
 * @cython.embedsignature(True)
 * def qr_insert(Q, R, u, k, which='row', rcond=None, overwrite_qru=False, check_finite=True):             # <<<<<<<<<<<<<<
 *     """
 *     QR update on row or column insertions
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5scipy_6linalg_14_decomp_update_5qr_insert, NULL, __pyx_n_s_scipy_linalg__decomp_update); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1695, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_qr_insert, __pyx_t_2) < 0) __PYX_ERR(0, 1695, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/linalg/_decomp_update.pyx":2159
 * 
 * @cython.embedsignature(True)
 * def qr_update(Q, R, u, v, overwrite_qruv=False, check_finite=True):             # <<<<<<<<<<<<<<
 *     """
 *     Rank-k QR update
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5scipy_6linalg_14_decomp_update_7qr_update, NULL, __pyx_n_s_scipy_linalg__decomp_update); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2159, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_qr_update, __pyx_t_2) < 0) __PYX_ERR(0, 2159, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/linalg/_decomp_update.pyx":2513
 *     return q1, r1
 * 
 * cnp.import_array()             # <<<<<<<<<<<<<<
 */
  __pyx_t_3 = __pyx_f_5numpy_import_array(); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 2513, __pyx_L1_error)

  /* "scipy/linalg/_decomp_update.pyx":1
 * """             # <<<<<<<<<<<<<<
 * Routines for updating QR decompositions
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_kp_u_qr_delete_line_1451, __pyx_kp_u_QR_downdate_on_row_or_column_de) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_kp_u_qr_insert_line_1695, __pyx_kp_u_QR_update_on_row_or_column_inse) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_kp_u_qr_update_line_2159, __pyx_kp_u_Rank_k_QR_update_If_A_Q_R_is_th) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1046
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init scipy.linalg._decomp_update", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init scipy.linalg._decomp_update");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* JoinPyUnicode */
static PyObject* __Pyx_PyUnicode_Join(PyObject* value_tuple, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      CYTHON_UNUSED Py_UCS4 max_char) {
#if CYTHON_USE_UNICODE_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyObject *result_uval;
    int result_ukind;
    Py_ssize_t i, char_pos;
    void *result_udata;
#if CYTHON_PEP393_ENABLED
    result_uval = PyUnicode_New(result_ulength, max_char);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = (max_char <= 255) ? PyUnicode_1BYTE_KIND : (max_char <= 65535) ? PyUnicode_2BYTE_KIND : PyUnicode_4BYTE_KIND;
    result_udata = PyUnicode_DATA(result_uval);
#else
    result_uval = PyUnicode_FromUnicode(NULL, result_ulength);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = sizeof(Py_UNICODE);
    result_udata = PyUnicode_AS_UNICODE(result_uval);
#endif
    char_pos = 0;
    for (i=0; i < value_count; i++) {
        int ukind;
        Py_ssize_t ulength;
        void *udata;
        PyObject *uval = PyTuple_GET_ITEM(value_tuple, i);
        if (unlikely(__Pyx_PyUnicode_READY(uval)))
            goto bad;
        ulength = __Pyx_PyUnicode_GET_LENGTH(uval);
        if (unlikely(!ulength))
            continue;
        if (unlikely(char_pos + ulength < 0))
            goto overflow;
        ukind = __Pyx_PyUnicode_KIND(uval);
        udata = __Pyx_PyUnicode_DATA(uval);
        if (!CYTHON_PEP393_ENABLED || ukind == result_ukind) {
            memcpy((char *)result_udata + char_pos * result_ukind, udata, (size_t) (ulength * result_ukind));
        } else {
            #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030300F0 || defined(_PyUnicode_FastCopyCharacters)
            _PyUnicode_FastCopyCharacters(result_uval, char_pos, uval, 0, ulength);
            #else
            Py_ssize_t j;
            for (j=0; j < ulength; j++) {
                Py_UCS4 uchar = __Pyx_PyUnicode_READ(ukind, udata, j);
                __Pyx_PyUnicode_WRITE(result_ukind, result_udata, char_pos+j, uchar);
            }
            #endif
        }
        char_pos += ulength;
    }
    return result_uval;
overflow:
    PyErr_SetString(PyExc_OverflowError, "join() result is too long for a Python string");
bad:
    Py_DECREF(result_uval);
    return NULL;
#else
    result_ulength++;
    value_count++;
    return PyUnicode_Join(__pyx_empty_unicode, value_tuple);
#endif
}

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseNoneIterError */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* DictGetItem */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            if (unlikely(PyTuple_Check(key))) {
                PyObject* args = PyTuple_Pack(1, key);
                if (likely(args)) {
                    PyErr_SetObject(PyExc_KeyError, args);
                    Py_DECREF(args);
                }
            } else {
                PyErr_SetObject(PyExc_KeyError, key);
            }
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* Declarations */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsf(b.real) >= fabsf(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                float r = b.imag / b.real;
                float s = (float)(1.0) / (b.real + b.imag * r);
                return __pyx_t_float_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            float r = b.real / b.imag;
            float s = (float)(1.0) / (b.imag + b.real * r);
            return __pyx_t_float_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            float denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_float_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(a, a);
                    case 3:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, a);
                    case 4:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = powf(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2f(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_float(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabs(b.real) >= fabs(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                double r = b.imag / b.real;
                double s = (double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            double r = b.real / b.imag;
            double s = (double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(a, a);
                    case 3:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, a);
                    case 4:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = pow(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_double(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Py_intptr_t(Py_intptr_t value) {
    const Py_intptr_t neg_one = (Py_intptr_t) ((Py_intptr_t) 0 - (Py_intptr_t) 1), const_zero = (Py_intptr_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(Py_intptr_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(Py_intptr_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(Py_intptr_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(Py_intptr_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(Py_intptr_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(Py_intptr_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__NPY_TYPES(enum NPY_TYPES value) {
    const enum NPY_TYPES neg_one = (enum NPY_TYPES) ((enum NPY_TYPES) 0 - (enum NPY_TYPES) 1), const_zero = (enum NPY_TYPES) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum NPY_TYPES) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum NPY_TYPES) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum NPY_TYPES) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum NPY_TYPES) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum NPY_TYPES),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* FunctionImport */
#ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
