/* Generated by Cython 0.29.6 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_6"
#define CYTHON_HEX_VERSION 0x001D06F0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
  #define PyObject_Unicode             PyObject_Str
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : (Py_INCREF(func), func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__scipy__special___ufuncs
#define __PYX_HAVE_API__scipy__special___ufuncs
/* Early includes */
#include "_complexstuff.h"
#include "numpy/npy_math.h"
#include <string.h>
#include <stdio.h>
#include "numpy/arrayobject.h"
#include "numpy/ufuncobject.h"
#include "sf_error.h"
#include <math.h>
#include <stdlib.h>
#include "lapack_defs.h"
#include "specfun_wrappers.h"
#include "c_misc/misc.h"
#include "cephes.h"
#include "amos_wrappers.h"
#include "_ufuncs_defs.h"
#include "_c99compat.h"
#include "math.h"
#include "cephes/dd_real.h"
#include "float.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

/* Header.proto */
#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "_ufuncs_extra_code.pxi",
  "_ufuncs.pyx",
  "__init__.pxd",
  "sph_harm.pxd",
  "_legacy.pxd",
  "_boxcox.pxd",
  "_convex_analysis.pxd",
  "_cunity.pxd",
  "_exprel.pxd",
  "_hyp0f1.pxd",
  "type.pxd",
  "_ufuncs_extra_code_common.pxi",
};
/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* NoFastGil.proto */
#define __Pyx_PyGILState_Ensure PyGILState_Ensure
#define __Pyx_PyGILState_Release PyGILState_Release
#define __Pyx_FastGIL_Remember()
#define __Pyx_FastGIL_Forget()
#define __Pyx_FastGilFuncInit()


/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":776
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":777
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":778
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":779
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":783
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":784
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":785
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":786
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":790
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":791
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":800
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":801
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":802
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":804
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":805
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":806
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":808
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":809
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":811
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":812
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":813
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;
/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif
static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);

/* Declarations.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif
static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);


/*--- Type declarations ---*/

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":815
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":816
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":817
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":819
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;
union __pyx_t_5scipy_7special_13_complexstuff__complex_pun;
typedef union __pyx_t_5scipy_7special_13_complexstuff__complex_pun __pyx_t_5scipy_7special_13_complexstuff__complex_pun;

/* "_complexstuff.pxd":35
 *     double_complex
 * 
 * ctypedef union _complex_pun:             # <<<<<<<<<<<<<<
 *     np.npy_cdouble npy
 *     double_complex c99
 */
union __pyx_t_5scipy_7special_13_complexstuff__complex_pun {
  npy_cdouble npy;
  __pyx_t_double_complex c99;
};

/* "scipy/special/_ufuncs.pyx":1195
 * 
 * from ._trig cimport ccospi as _func_ccospi
 * ctypedef double complex _proto_ccospi_t(double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_ccospi_t *_proto_ccospi_t_var = &_func_ccospi
 * from ._trig cimport dcospi as _func_dcospi
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_ccospi_t(__pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1198
 * cdef _proto_ccospi_t *_proto_ccospi_t_var = &_func_ccospi
 * from ._trig cimport dcospi as _func_dcospi
 * ctypedef double _proto_dcospi_t(double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_dcospi_t *_proto_dcospi_t_var = &_func_dcospi
 * from ._ellip_harm cimport ellip_harmonic as _func_ellip_harmonic
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_dcospi_t(double);

/* "scipy/special/_ufuncs.pyx":1201
 * cdef _proto_dcospi_t *_proto_dcospi_t_var = &_func_dcospi
 * from ._ellip_harm cimport ellip_harmonic as _func_ellip_harmonic
 * ctypedef double _proto_ellip_harmonic_t(double, double, int, int, double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_ellip_harmonic_t *_proto_ellip_harmonic_t_var = &_func_ellip_harmonic
 * from ._legacy cimport ellip_harmonic_unsafe as _func_ellip_harmonic_unsafe
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_ellip_harmonic_t(double, double, int, int, double, double, double);

/* "scipy/special/_ufuncs.pyx":1204
 * cdef _proto_ellip_harmonic_t *_proto_ellip_harmonic_t_var = &_func_ellip_harmonic
 * from ._legacy cimport ellip_harmonic_unsafe as _func_ellip_harmonic_unsafe
 * ctypedef double _proto_ellip_harmonic_unsafe_t(double, double, double, double, double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_ellip_harmonic_unsafe_t *_proto_ellip_harmonic_unsafe_t_var = &_func_ellip_harmonic_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_ellip_harmonic_unsafe_t(double, double, double, double, double, double, double);

/* "scipy/special/_ufuncs.pyx":1215
 *     cdef double _func_kolmogp "kolmogp"(double) nogil
 * from .lambertw cimport lambertw_scalar as _func_lambertw_scalar
 * ctypedef double complex _proto_lambertw_scalar_t(double complex, long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_lambertw_scalar_t *_proto_lambertw_scalar_t_var = &_func_lambertw_scalar
 * cdef extern from "_ufuncs_defs.h":
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_lambertw_scalar_t(__pyx_t_double_complex, long, double);

/* "scipy/special/_ufuncs.pyx":1224
 *     cdef double _func_log1pmx "log1pmx"(double) nogil
 * from .sf_error cimport _sf_error_test_function as _func__sf_error_test_function
 * ctypedef int _proto__sf_error_test_function_t(int) nogil             # <<<<<<<<<<<<<<
 * cdef _proto__sf_error_test_function_t *_proto__sf_error_test_function_t_var = &_func__sf_error_test_function
 * from ._trig cimport csinpi as _func_csinpi
 */
typedef int __pyx_t_5scipy_7special_7_ufuncs__proto__sf_error_test_function_t(int);

/* "scipy/special/_ufuncs.pyx":1227
 * cdef _proto__sf_error_test_function_t *_proto__sf_error_test_function_t_var = &_func__sf_error_test_function
 * from ._trig cimport csinpi as _func_csinpi
 * ctypedef double complex _proto_csinpi_t(double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_csinpi_t *_proto_csinpi_t_var = &_func_csinpi
 * from ._trig cimport dsinpi as _func_dsinpi
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_csinpi_t(__pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1230
 * cdef _proto_csinpi_t *_proto_csinpi_t_var = &_func_csinpi
 * from ._trig cimport dsinpi as _func_dsinpi
 * ctypedef double _proto_dsinpi_t(double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_dsinpi_t *_proto_dsinpi_t_var = &_func_dsinpi
 * from ._legacy cimport smirnovc_unsafe as _func_smirnovc_unsafe
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_dsinpi_t(double);

/* "scipy/special/_ufuncs.pyx":1233
 * cdef _proto_dsinpi_t *_proto_dsinpi_t_var = &_func_dsinpi
 * from ._legacy cimport smirnovc_unsafe as _func_smirnovc_unsafe
 * ctypedef double _proto_smirnovc_unsafe_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_smirnovc_unsafe_t *_proto_smirnovc_unsafe_t_var = &_func_smirnovc_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_smirnovc_unsafe_t(double, double);

/* "scipy/special/_ufuncs.pyx":1238
 *     cdef double _func_smirnovc "smirnovc"(int, double) nogil
 * from ._legacy cimport smirnovci_unsafe as _func_smirnovci_unsafe
 * ctypedef double _proto_smirnovci_unsafe_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_smirnovci_unsafe_t *_proto_smirnovci_unsafe_t_var = &_func_smirnovci_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_smirnovci_unsafe_t(double, double);

/* "scipy/special/_ufuncs.pyx":1243
 *     cdef double _func_smirnovci "smirnovci"(int, double) nogil
 * from ._legacy cimport smirnovp_unsafe as _func_smirnovp_unsafe
 * ctypedef double _proto_smirnovp_unsafe_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_smirnovp_unsafe_t *_proto_smirnovp_unsafe_t_var = &_func_smirnovp_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_smirnovp_unsafe_t(double, double);

/* "scipy/special/_ufuncs.pyx":1248
 *     cdef double _func_smirnovp "smirnovp"(int, double) nogil
 * from ._spherical_bessel cimport spherical_in_complex as _func_spherical_in_complex
 * ctypedef double complex _proto_spherical_in_complex_t(long, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_spherical_in_complex_t *_proto_spherical_in_complex_t_var = &_func_spherical_in_complex
 * from ._spherical_bessel cimport spherical_in_real as _func_spherical_in_real
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_in_complex_t(long, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1251
 * cdef _proto_spherical_in_complex_t *_proto_spherical_in_complex_t_var = &_func_spherical_in_complex
 * from ._spherical_bessel cimport spherical_in_real as _func_spherical_in_real
 * ctypedef double _proto_spherical_in_real_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_spherical_in_real_t *_proto_spherical_in_real_t_var = &_func_spherical_in_real
 * from ._spherical_bessel cimport spherical_in_d_complex as _func_spherical_in_d_complex
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_in_real_t(long, double);

/* "scipy/special/_ufuncs.pyx":1254
 * cdef _proto_spherical_in_real_t *_proto_spherical_in_real_t_var = &_func_spherical_in_real
 * from ._spherical_bessel cimport spherical_in_d_complex as _func_spherical_in_d_complex
 * ctypedef double complex _proto_spherical_in_d_complex_t(long, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_spherical_in_d_complex_t *_proto_spherical_in_d_complex_t_var = &_func_spherical_in_d_complex
 * from ._spherical_bessel cimport spherical_in_d_real as _func_spherical_in_d_real
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_in_d_complex_t(long, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1257
 * cdef _proto_spherical_in_d_complex_t *_proto_spherical_in_d_complex_t_var = &_func_spherical_in_d_complex
 * from ._spherical_bessel cimport spherical_in_d_real as _func_spherical_in_d_real
 * ctypedef double _proto_spherical_in_d_real_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_spherical_in_d_real_t *_proto_spherical_in_d_real_t_var = &_func_spherical_in_d_real
 * from ._spherical_bessel cimport spherical_jn_complex as _func_spherical_jn_complex
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_in_d_real_t(long, double);

/* "scipy/special/_ufuncs.pyx":1260
 * cdef _proto_spherical_in_d_real_t *_proto_spherical_in_d_real_t_var = &_func_spherical_in_d_real
 * from ._spherical_bessel cimport spherical_jn_complex as _func_spherical_jn_complex
 * ctypedef double complex _proto_spherical_jn_complex_t(long, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_spherical_jn_complex_t *_proto_spherical_jn_complex_t_var = &_func_spherical_jn_complex
 * from ._spherical_bessel cimport spherical_jn_real as _func_spherical_jn_real
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_jn_complex_t(long, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1263
 * cdef _proto_spherical_jn_complex_t *_proto_spherical_jn_complex_t_var = &_func_spherical_jn_complex
 * from ._spherical_bessel cimport spherical_jn_real as _func_spherical_jn_real
 * ctypedef double _proto_spherical_jn_real_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_spherical_jn_real_t *_proto_spherical_jn_real_t_var = &_func_spherical_jn_real
 * from ._spherical_bessel cimport spherical_jn_d_complex as _func_spherical_jn_d_complex
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_jn_real_t(long, double);

/* "scipy/special/_ufuncs.pyx":1266
 * cdef _proto_spherical_jn_real_t *_proto_spherical_jn_real_t_var = &_func_spherical_jn_real
 * from ._spherical_bessel cimport spherical_jn_d_complex as _func_spherical_jn_d_complex
 * ctypedef double complex _proto_spherical_jn_d_complex_t(long, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_spherical_jn_d_complex_t *_proto_spherical_jn_d_complex_t_var = &_func_spherical_jn_d_complex
 * from ._spherical_bessel cimport spherical_jn_d_real as _func_spherical_jn_d_real
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_jn_d_complex_t(long, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1269
 * cdef _proto_spherical_jn_d_complex_t *_proto_spherical_jn_d_complex_t_var = &_func_spherical_jn_d_complex
 * from ._spherical_bessel cimport spherical_jn_d_real as _func_spherical_jn_d_real
 * ctypedef double _proto_spherical_jn_d_real_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_spherical_jn_d_real_t *_proto_spherical_jn_d_real_t_var = &_func_spherical_jn_d_real
 * from ._spherical_bessel cimport spherical_kn_complex as _func_spherical_kn_complex
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_jn_d_real_t(long, double);

/* "scipy/special/_ufuncs.pyx":1272
 * cdef _proto_spherical_jn_d_real_t *_proto_spherical_jn_d_real_t_var = &_func_spherical_jn_d_real
 * from ._spherical_bessel cimport spherical_kn_complex as _func_spherical_kn_complex
 * ctypedef double complex _proto_spherical_kn_complex_t(long, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_spherical_kn_complex_t *_proto_spherical_kn_complex_t_var = &_func_spherical_kn_complex
 * from ._spherical_bessel cimport spherical_kn_real as _func_spherical_kn_real
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_kn_complex_t(long, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1275
 * cdef _proto_spherical_kn_complex_t *_proto_spherical_kn_complex_t_var = &_func_spherical_kn_complex
 * from ._spherical_bessel cimport spherical_kn_real as _func_spherical_kn_real
 * ctypedef double _proto_spherical_kn_real_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_spherical_kn_real_t *_proto_spherical_kn_real_t_var = &_func_spherical_kn_real
 * from ._spherical_bessel cimport spherical_kn_d_complex as _func_spherical_kn_d_complex
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_kn_real_t(long, double);

/* "scipy/special/_ufuncs.pyx":1278
 * cdef _proto_spherical_kn_real_t *_proto_spherical_kn_real_t_var = &_func_spherical_kn_real
 * from ._spherical_bessel cimport spherical_kn_d_complex as _func_spherical_kn_d_complex
 * ctypedef double complex _proto_spherical_kn_d_complex_t(long, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_spherical_kn_d_complex_t *_proto_spherical_kn_d_complex_t_var = &_func_spherical_kn_d_complex
 * from ._spherical_bessel cimport spherical_kn_d_real as _func_spherical_kn_d_real
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_kn_d_complex_t(long, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1281
 * cdef _proto_spherical_kn_d_complex_t *_proto_spherical_kn_d_complex_t_var = &_func_spherical_kn_d_complex
 * from ._spherical_bessel cimport spherical_kn_d_real as _func_spherical_kn_d_real
 * ctypedef double _proto_spherical_kn_d_real_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_spherical_kn_d_real_t *_proto_spherical_kn_d_real_t_var = &_func_spherical_kn_d_real
 * from ._spherical_bessel cimport spherical_yn_complex as _func_spherical_yn_complex
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_kn_d_real_t(long, double);

/* "scipy/special/_ufuncs.pyx":1284
 * cdef _proto_spherical_kn_d_real_t *_proto_spherical_kn_d_real_t_var = &_func_spherical_kn_d_real
 * from ._spherical_bessel cimport spherical_yn_complex as _func_spherical_yn_complex
 * ctypedef double complex _proto_spherical_yn_complex_t(long, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_spherical_yn_complex_t *_proto_spherical_yn_complex_t_var = &_func_spherical_yn_complex
 * from ._spherical_bessel cimport spherical_yn_real as _func_spherical_yn_real
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_yn_complex_t(long, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1287
 * cdef _proto_spherical_yn_complex_t *_proto_spherical_yn_complex_t_var = &_func_spherical_yn_complex
 * from ._spherical_bessel cimport spherical_yn_real as _func_spherical_yn_real
 * ctypedef double _proto_spherical_yn_real_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_spherical_yn_real_t *_proto_spherical_yn_real_t_var = &_func_spherical_yn_real
 * from ._spherical_bessel cimport spherical_yn_d_complex as _func_spherical_yn_d_complex
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_yn_real_t(long, double);

/* "scipy/special/_ufuncs.pyx":1290
 * cdef _proto_spherical_yn_real_t *_proto_spherical_yn_real_t_var = &_func_spherical_yn_real
 * from ._spherical_bessel cimport spherical_yn_d_complex as _func_spherical_yn_d_complex
 * ctypedef double complex _proto_spherical_yn_d_complex_t(long, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_spherical_yn_d_complex_t *_proto_spherical_yn_d_complex_t_var = &_func_spherical_yn_d_complex
 * from ._spherical_bessel cimport spherical_yn_d_real as _func_spherical_yn_d_real
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_yn_d_complex_t(long, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1293
 * cdef _proto_spherical_yn_d_complex_t *_proto_spherical_yn_d_complex_t_var = &_func_spherical_yn_d_complex
 * from ._spherical_bessel cimport spherical_yn_d_real as _func_spherical_yn_d_real
 * ctypedef double _proto_spherical_yn_d_real_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_spherical_yn_d_real_t *_proto_spherical_yn_d_real_t_var = &_func_spherical_yn_d_real
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_yn_d_real_t(long, double);

/* "scipy/special/_ufuncs.pyx":1304
 *     cdef double _func_zeta "zeta"(double, double) nogil
 * from ._agm cimport agm as _func_agm
 * ctypedef double _proto_agm_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_agm_t *_proto_agm_t_var = &_func_agm
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_agm_t(double, double);

/* "scipy/special/_ufuncs.pyx":1315
 *     cdef int _func_cairy_wrap_e_real "cairy_wrap_e_real"(double, double *, double *, double *, double *) nogil
 * from ._legacy cimport bdtr_unsafe as _func_bdtr_unsafe
 * ctypedef double _proto_bdtr_unsafe_t(double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_bdtr_unsafe_t *_proto_bdtr_unsafe_t_var = &_func_bdtr_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_bdtr_unsafe_t(double, double, double);

/* "scipy/special/_ufuncs.pyx":1320
 *     cdef double _func_bdtr "bdtr"(int, int, double) nogil
 * from ._legacy cimport bdtrc_unsafe as _func_bdtrc_unsafe
 * ctypedef double _proto_bdtrc_unsafe_t(double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_bdtrc_unsafe_t *_proto_bdtrc_unsafe_t_var = &_func_bdtrc_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_bdtrc_unsafe_t(double, double, double);

/* "scipy/special/_ufuncs.pyx":1325
 *     cdef double _func_bdtrc "bdtrc"(int, int, double) nogil
 * from ._legacy cimport bdtri_unsafe as _func_bdtri_unsafe
 * ctypedef double _proto_bdtri_unsafe_t(double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_bdtri_unsafe_t *_proto_bdtri_unsafe_t_var = &_func_bdtri_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_bdtri_unsafe_t(double, double, double);

/* "scipy/special/_ufuncs.pyx":1352
 *     cdef double _func_lbeta "lbeta"(double, double) nogil
 * from .orthogonal_eval cimport binom as _func_binom
 * ctypedef double _proto_binom_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_binom_t *_proto_binom_t_var = &_func_binom
 * from ._boxcox cimport boxcox as _func_boxcox
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_binom_t(double, double);

/* "scipy/special/_ufuncs.pyx":1355
 * cdef _proto_binom_t *_proto_binom_t_var = &_func_binom
 * from ._boxcox cimport boxcox as _func_boxcox
 * ctypedef double _proto_boxcox_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_boxcox_t *_proto_boxcox_t_var = &_func_boxcox
 * from ._boxcox cimport boxcox1p as _func_boxcox1p
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_boxcox_t(double, double);

/* "scipy/special/_ufuncs.pyx":1358
 * cdef _proto_boxcox_t *_proto_boxcox_t_var = &_func_boxcox
 * from ._boxcox cimport boxcox1p as _func_boxcox1p
 * ctypedef double _proto_boxcox1p_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_boxcox1p_t *_proto_boxcox1p_t_var = &_func_boxcox1p
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_boxcox1p_t(double, double);

/* "scipy/special/_ufuncs.pyx":1403
 *     cdef double _func_ellpk "ellpk"(double) nogil
 * from ._convex_analysis cimport entr as _func_entr
 * ctypedef double _proto_entr_t(double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_entr_t *_proto_entr_t_var = &_func_entr
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_entr_t(double);

/* "scipy/special/_ufuncs.pyx":1410
 *     cdef double _func_erfc "erfc"(double) nogil
 * from .orthogonal_eval cimport eval_chebyc as _func_eval_chebyc
 * ctypedef double complex _proto_eval_chebyc_double_complex__t(double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebyc_double_complex__t *_proto_eval_chebyc_double_complex__t_var = &_func_eval_chebyc[double_complex]
 * from .orthogonal_eval cimport eval_chebyc as _func_eval_chebyc
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyc_double_complex__t(double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1413
 * cdef _proto_eval_chebyc_double_complex__t *_proto_eval_chebyc_double_complex__t_var = &_func_eval_chebyc[double_complex]
 * from .orthogonal_eval cimport eval_chebyc as _func_eval_chebyc
 * ctypedef double _proto_eval_chebyc_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebyc_double__t *_proto_eval_chebyc_double__t_var = &_func_eval_chebyc[double]
 * from .orthogonal_eval cimport eval_chebyc_l as _func_eval_chebyc_l
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyc_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1416
 * cdef _proto_eval_chebyc_double__t *_proto_eval_chebyc_double__t_var = &_func_eval_chebyc[double]
 * from .orthogonal_eval cimport eval_chebyc_l as _func_eval_chebyc_l
 * ctypedef double _proto_eval_chebyc_l_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebyc_l_t *_proto_eval_chebyc_l_t_var = &_func_eval_chebyc_l
 * from .orthogonal_eval cimport eval_chebys as _func_eval_chebys
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyc_l_t(long, double);

/* "scipy/special/_ufuncs.pyx":1419
 * cdef _proto_eval_chebyc_l_t *_proto_eval_chebyc_l_t_var = &_func_eval_chebyc_l
 * from .orthogonal_eval cimport eval_chebys as _func_eval_chebys
 * ctypedef double complex _proto_eval_chebys_double_complex__t(double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebys_double_complex__t *_proto_eval_chebys_double_complex__t_var = &_func_eval_chebys[double_complex]
 * from .orthogonal_eval cimport eval_chebys as _func_eval_chebys
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebys_double_complex__t(double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1422
 * cdef _proto_eval_chebys_double_complex__t *_proto_eval_chebys_double_complex__t_var = &_func_eval_chebys[double_complex]
 * from .orthogonal_eval cimport eval_chebys as _func_eval_chebys
 * ctypedef double _proto_eval_chebys_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebys_double__t *_proto_eval_chebys_double__t_var = &_func_eval_chebys[double]
 * from .orthogonal_eval cimport eval_chebys_l as _func_eval_chebys_l
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebys_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1425
 * cdef _proto_eval_chebys_double__t *_proto_eval_chebys_double__t_var = &_func_eval_chebys[double]
 * from .orthogonal_eval cimport eval_chebys_l as _func_eval_chebys_l
 * ctypedef double _proto_eval_chebys_l_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebys_l_t *_proto_eval_chebys_l_t_var = &_func_eval_chebys_l
 * from .orthogonal_eval cimport eval_chebyt as _func_eval_chebyt
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebys_l_t(long, double);

/* "scipy/special/_ufuncs.pyx":1428
 * cdef _proto_eval_chebys_l_t *_proto_eval_chebys_l_t_var = &_func_eval_chebys_l
 * from .orthogonal_eval cimport eval_chebyt as _func_eval_chebyt
 * ctypedef double complex _proto_eval_chebyt_double_complex__t(double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebyt_double_complex__t *_proto_eval_chebyt_double_complex__t_var = &_func_eval_chebyt[double_complex]
 * from .orthogonal_eval cimport eval_chebyt as _func_eval_chebyt
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyt_double_complex__t(double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1431
 * cdef _proto_eval_chebyt_double_complex__t *_proto_eval_chebyt_double_complex__t_var = &_func_eval_chebyt[double_complex]
 * from .orthogonal_eval cimport eval_chebyt as _func_eval_chebyt
 * ctypedef double _proto_eval_chebyt_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebyt_double__t *_proto_eval_chebyt_double__t_var = &_func_eval_chebyt[double]
 * from .orthogonal_eval cimport eval_chebyt_l as _func_eval_chebyt_l
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyt_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1434
 * cdef _proto_eval_chebyt_double__t *_proto_eval_chebyt_double__t_var = &_func_eval_chebyt[double]
 * from .orthogonal_eval cimport eval_chebyt_l as _func_eval_chebyt_l
 * ctypedef double _proto_eval_chebyt_l_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebyt_l_t *_proto_eval_chebyt_l_t_var = &_func_eval_chebyt_l
 * from .orthogonal_eval cimport eval_chebyu as _func_eval_chebyu
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyt_l_t(long, double);

/* "scipy/special/_ufuncs.pyx":1437
 * cdef _proto_eval_chebyt_l_t *_proto_eval_chebyt_l_t_var = &_func_eval_chebyt_l
 * from .orthogonal_eval cimport eval_chebyu as _func_eval_chebyu
 * ctypedef double complex _proto_eval_chebyu_double_complex__t(double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebyu_double_complex__t *_proto_eval_chebyu_double_complex__t_var = &_func_eval_chebyu[double_complex]
 * from .orthogonal_eval cimport eval_chebyu as _func_eval_chebyu
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyu_double_complex__t(double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1440
 * cdef _proto_eval_chebyu_double_complex__t *_proto_eval_chebyu_double_complex__t_var = &_func_eval_chebyu[double_complex]
 * from .orthogonal_eval cimport eval_chebyu as _func_eval_chebyu
 * ctypedef double _proto_eval_chebyu_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebyu_double__t *_proto_eval_chebyu_double__t_var = &_func_eval_chebyu[double]
 * from .orthogonal_eval cimport eval_chebyu_l as _func_eval_chebyu_l
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyu_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1443
 * cdef _proto_eval_chebyu_double__t *_proto_eval_chebyu_double__t_var = &_func_eval_chebyu[double]
 * from .orthogonal_eval cimport eval_chebyu_l as _func_eval_chebyu_l
 * ctypedef double _proto_eval_chebyu_l_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_chebyu_l_t *_proto_eval_chebyu_l_t_var = &_func_eval_chebyu_l
 * from .orthogonal_eval cimport eval_gegenbauer as _func_eval_gegenbauer
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyu_l_t(long, double);

/* "scipy/special/_ufuncs.pyx":1446
 * cdef _proto_eval_chebyu_l_t *_proto_eval_chebyu_l_t_var = &_func_eval_chebyu_l
 * from .orthogonal_eval cimport eval_gegenbauer as _func_eval_gegenbauer
 * ctypedef double complex _proto_eval_gegenbauer_double_complex__t(double, double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_gegenbauer_double_complex__t *_proto_eval_gegenbauer_double_complex__t_var = &_func_eval_gegenbauer[double_complex]
 * from .orthogonal_eval cimport eval_gegenbauer as _func_eval_gegenbauer
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_double_complex__t(double, double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1449
 * cdef _proto_eval_gegenbauer_double_complex__t *_proto_eval_gegenbauer_double_complex__t_var = &_func_eval_gegenbauer[double_complex]
 * from .orthogonal_eval cimport eval_gegenbauer as _func_eval_gegenbauer
 * ctypedef double _proto_eval_gegenbauer_double__t(double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_gegenbauer_double__t *_proto_eval_gegenbauer_double__t_var = &_func_eval_gegenbauer[double]
 * from .orthogonal_eval cimport eval_gegenbauer_l as _func_eval_gegenbauer_l
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_double__t(double, double, double);

/* "scipy/special/_ufuncs.pyx":1452
 * cdef _proto_eval_gegenbauer_double__t *_proto_eval_gegenbauer_double__t_var = &_func_eval_gegenbauer[double]
 * from .orthogonal_eval cimport eval_gegenbauer_l as _func_eval_gegenbauer_l
 * ctypedef double _proto_eval_gegenbauer_l_t(long, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_gegenbauer_l_t *_proto_eval_gegenbauer_l_t_var = &_func_eval_gegenbauer_l
 * from .orthogonal_eval cimport eval_genlaguerre as _func_eval_genlaguerre
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_l_t(long, double, double);

/* "scipy/special/_ufuncs.pyx":1455
 * cdef _proto_eval_gegenbauer_l_t *_proto_eval_gegenbauer_l_t_var = &_func_eval_gegenbauer_l
 * from .orthogonal_eval cimport eval_genlaguerre as _func_eval_genlaguerre
 * ctypedef double complex _proto_eval_genlaguerre_double_complex__t(double, double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_genlaguerre_double_complex__t *_proto_eval_genlaguerre_double_complex__t_var = &_func_eval_genlaguerre[double_complex]
 * from .orthogonal_eval cimport eval_genlaguerre as _func_eval_genlaguerre
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_double_complex__t(double, double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1458
 * cdef _proto_eval_genlaguerre_double_complex__t *_proto_eval_genlaguerre_double_complex__t_var = &_func_eval_genlaguerre[double_complex]
 * from .orthogonal_eval cimport eval_genlaguerre as _func_eval_genlaguerre
 * ctypedef double _proto_eval_genlaguerre_double__t(double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_genlaguerre_double__t *_proto_eval_genlaguerre_double__t_var = &_func_eval_genlaguerre[double]
 * from .orthogonal_eval cimport eval_genlaguerre_l as _func_eval_genlaguerre_l
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_double__t(double, double, double);

/* "scipy/special/_ufuncs.pyx":1461
 * cdef _proto_eval_genlaguerre_double__t *_proto_eval_genlaguerre_double__t_var = &_func_eval_genlaguerre[double]
 * from .orthogonal_eval cimport eval_genlaguerre_l as _func_eval_genlaguerre_l
 * ctypedef double _proto_eval_genlaguerre_l_t(long, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_genlaguerre_l_t *_proto_eval_genlaguerre_l_t_var = &_func_eval_genlaguerre_l
 * from .orthogonal_eval cimport eval_hermite as _func_eval_hermite
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_l_t(long, double, double);

/* "scipy/special/_ufuncs.pyx":1464
 * cdef _proto_eval_genlaguerre_l_t *_proto_eval_genlaguerre_l_t_var = &_func_eval_genlaguerre_l
 * from .orthogonal_eval cimport eval_hermite as _func_eval_hermite
 * ctypedef double _proto_eval_hermite_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_hermite_t *_proto_eval_hermite_t_var = &_func_eval_hermite
 * from .orthogonal_eval cimport eval_hermitenorm as _func_eval_hermitenorm
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_hermite_t(long, double);

/* "scipy/special/_ufuncs.pyx":1467
 * cdef _proto_eval_hermite_t *_proto_eval_hermite_t_var = &_func_eval_hermite
 * from .orthogonal_eval cimport eval_hermitenorm as _func_eval_hermitenorm
 * ctypedef double _proto_eval_hermitenorm_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_hermitenorm_t *_proto_eval_hermitenorm_t_var = &_func_eval_hermitenorm
 * from .orthogonal_eval cimport eval_jacobi as _func_eval_jacobi
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_hermitenorm_t(long, double);

/* "scipy/special/_ufuncs.pyx":1470
 * cdef _proto_eval_hermitenorm_t *_proto_eval_hermitenorm_t_var = &_func_eval_hermitenorm
 * from .orthogonal_eval cimport eval_jacobi as _func_eval_jacobi
 * ctypedef double complex _proto_eval_jacobi_double_complex__t(double, double, double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_jacobi_double_complex__t *_proto_eval_jacobi_double_complex__t_var = &_func_eval_jacobi[double_complex]
 * from .orthogonal_eval cimport eval_jacobi as _func_eval_jacobi
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_jacobi_double_complex__t(double, double, double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1473
 * cdef _proto_eval_jacobi_double_complex__t *_proto_eval_jacobi_double_complex__t_var = &_func_eval_jacobi[double_complex]
 * from .orthogonal_eval cimport eval_jacobi as _func_eval_jacobi
 * ctypedef double _proto_eval_jacobi_double__t(double, double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_jacobi_double__t *_proto_eval_jacobi_double__t_var = &_func_eval_jacobi[double]
 * from .orthogonal_eval cimport eval_jacobi_l as _func_eval_jacobi_l
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_jacobi_double__t(double, double, double, double);

/* "scipy/special/_ufuncs.pyx":1476
 * cdef _proto_eval_jacobi_double__t *_proto_eval_jacobi_double__t_var = &_func_eval_jacobi[double]
 * from .orthogonal_eval cimport eval_jacobi_l as _func_eval_jacobi_l
 * ctypedef double _proto_eval_jacobi_l_t(long, double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_jacobi_l_t *_proto_eval_jacobi_l_t_var = &_func_eval_jacobi_l
 * from .orthogonal_eval cimport eval_laguerre as _func_eval_laguerre
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_jacobi_l_t(long, double, double, double);

/* "scipy/special/_ufuncs.pyx":1479
 * cdef _proto_eval_jacobi_l_t *_proto_eval_jacobi_l_t_var = &_func_eval_jacobi_l
 * from .orthogonal_eval cimport eval_laguerre as _func_eval_laguerre
 * ctypedef double complex _proto_eval_laguerre_double_complex__t(double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_laguerre_double_complex__t *_proto_eval_laguerre_double_complex__t_var = &_func_eval_laguerre[double_complex]
 * from .orthogonal_eval cimport eval_laguerre as _func_eval_laguerre
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_laguerre_double_complex__t(double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1482
 * cdef _proto_eval_laguerre_double_complex__t *_proto_eval_laguerre_double_complex__t_var = &_func_eval_laguerre[double_complex]
 * from .orthogonal_eval cimport eval_laguerre as _func_eval_laguerre
 * ctypedef double _proto_eval_laguerre_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_laguerre_double__t *_proto_eval_laguerre_double__t_var = &_func_eval_laguerre[double]
 * from .orthogonal_eval cimport eval_laguerre_l as _func_eval_laguerre_l
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_laguerre_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1485
 * cdef _proto_eval_laguerre_double__t *_proto_eval_laguerre_double__t_var = &_func_eval_laguerre[double]
 * from .orthogonal_eval cimport eval_laguerre_l as _func_eval_laguerre_l
 * ctypedef double _proto_eval_laguerre_l_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_laguerre_l_t *_proto_eval_laguerre_l_t_var = &_func_eval_laguerre_l
 * from .orthogonal_eval cimport eval_legendre as _func_eval_legendre
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_laguerre_l_t(long, double);

/* "scipy/special/_ufuncs.pyx":1488
 * cdef _proto_eval_laguerre_l_t *_proto_eval_laguerre_l_t_var = &_func_eval_laguerre_l
 * from .orthogonal_eval cimport eval_legendre as _func_eval_legendre
 * ctypedef double complex _proto_eval_legendre_double_complex__t(double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_legendre_double_complex__t *_proto_eval_legendre_double_complex__t_var = &_func_eval_legendre[double_complex]
 * from .orthogonal_eval cimport eval_legendre as _func_eval_legendre
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_legendre_double_complex__t(double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1491
 * cdef _proto_eval_legendre_double_complex__t *_proto_eval_legendre_double_complex__t_var = &_func_eval_legendre[double_complex]
 * from .orthogonal_eval cimport eval_legendre as _func_eval_legendre
 * ctypedef double _proto_eval_legendre_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_legendre_double__t *_proto_eval_legendre_double__t_var = &_func_eval_legendre[double]
 * from .orthogonal_eval cimport eval_legendre_l as _func_eval_legendre_l
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_legendre_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1494
 * cdef _proto_eval_legendre_double__t *_proto_eval_legendre_double__t_var = &_func_eval_legendre[double]
 * from .orthogonal_eval cimport eval_legendre_l as _func_eval_legendre_l
 * ctypedef double _proto_eval_legendre_l_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_legendre_l_t *_proto_eval_legendre_l_t_var = &_func_eval_legendre_l
 * from .orthogonal_eval cimport eval_sh_chebyt as _func_eval_sh_chebyt
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_legendre_l_t(long, double);

/* "scipy/special/_ufuncs.pyx":1497
 * cdef _proto_eval_legendre_l_t *_proto_eval_legendre_l_t_var = &_func_eval_legendre_l
 * from .orthogonal_eval cimport eval_sh_chebyt as _func_eval_sh_chebyt
 * ctypedef double complex _proto_eval_sh_chebyt_double_complex__t(double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_chebyt_double_complex__t *_proto_eval_sh_chebyt_double_complex__t_var = &_func_eval_sh_chebyt[double_complex]
 * from .orthogonal_eval cimport eval_sh_chebyt as _func_eval_sh_chebyt
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_double_complex__t(double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1500
 * cdef _proto_eval_sh_chebyt_double_complex__t *_proto_eval_sh_chebyt_double_complex__t_var = &_func_eval_sh_chebyt[double_complex]
 * from .orthogonal_eval cimport eval_sh_chebyt as _func_eval_sh_chebyt
 * ctypedef double _proto_eval_sh_chebyt_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_chebyt_double__t *_proto_eval_sh_chebyt_double__t_var = &_func_eval_sh_chebyt[double]
 * from .orthogonal_eval cimport eval_sh_chebyt_l as _func_eval_sh_chebyt_l
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1503
 * cdef _proto_eval_sh_chebyt_double__t *_proto_eval_sh_chebyt_double__t_var = &_func_eval_sh_chebyt[double]
 * from .orthogonal_eval cimport eval_sh_chebyt_l as _func_eval_sh_chebyt_l
 * ctypedef double _proto_eval_sh_chebyt_l_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_chebyt_l_t *_proto_eval_sh_chebyt_l_t_var = &_func_eval_sh_chebyt_l
 * from .orthogonal_eval cimport eval_sh_chebyu as _func_eval_sh_chebyu
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_l_t(long, double);

/* "scipy/special/_ufuncs.pyx":1506
 * cdef _proto_eval_sh_chebyt_l_t *_proto_eval_sh_chebyt_l_t_var = &_func_eval_sh_chebyt_l
 * from .orthogonal_eval cimport eval_sh_chebyu as _func_eval_sh_chebyu
 * ctypedef double complex _proto_eval_sh_chebyu_double_complex__t(double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_chebyu_double_complex__t *_proto_eval_sh_chebyu_double_complex__t_var = &_func_eval_sh_chebyu[double_complex]
 * from .orthogonal_eval cimport eval_sh_chebyu as _func_eval_sh_chebyu
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_double_complex__t(double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1509
 * cdef _proto_eval_sh_chebyu_double_complex__t *_proto_eval_sh_chebyu_double_complex__t_var = &_func_eval_sh_chebyu[double_complex]
 * from .orthogonal_eval cimport eval_sh_chebyu as _func_eval_sh_chebyu
 * ctypedef double _proto_eval_sh_chebyu_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_chebyu_double__t *_proto_eval_sh_chebyu_double__t_var = &_func_eval_sh_chebyu[double]
 * from .orthogonal_eval cimport eval_sh_chebyu_l as _func_eval_sh_chebyu_l
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1512
 * cdef _proto_eval_sh_chebyu_double__t *_proto_eval_sh_chebyu_double__t_var = &_func_eval_sh_chebyu[double]
 * from .orthogonal_eval cimport eval_sh_chebyu_l as _func_eval_sh_chebyu_l
 * ctypedef double _proto_eval_sh_chebyu_l_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_chebyu_l_t *_proto_eval_sh_chebyu_l_t_var = &_func_eval_sh_chebyu_l
 * from .orthogonal_eval cimport eval_sh_jacobi as _func_eval_sh_jacobi
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_l_t(long, double);

/* "scipy/special/_ufuncs.pyx":1515
 * cdef _proto_eval_sh_chebyu_l_t *_proto_eval_sh_chebyu_l_t_var = &_func_eval_sh_chebyu_l
 * from .orthogonal_eval cimport eval_sh_jacobi as _func_eval_sh_jacobi
 * ctypedef double complex _proto_eval_sh_jacobi_double_complex__t(double, double, double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_jacobi_double_complex__t *_proto_eval_sh_jacobi_double_complex__t_var = &_func_eval_sh_jacobi[double_complex]
 * from .orthogonal_eval cimport eval_sh_jacobi as _func_eval_sh_jacobi
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_double_complex__t(double, double, double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1518
 * cdef _proto_eval_sh_jacobi_double_complex__t *_proto_eval_sh_jacobi_double_complex__t_var = &_func_eval_sh_jacobi[double_complex]
 * from .orthogonal_eval cimport eval_sh_jacobi as _func_eval_sh_jacobi
 * ctypedef double _proto_eval_sh_jacobi_double__t(double, double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_jacobi_double__t *_proto_eval_sh_jacobi_double__t_var = &_func_eval_sh_jacobi[double]
 * from .orthogonal_eval cimport eval_sh_jacobi_l as _func_eval_sh_jacobi_l
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_double__t(double, double, double, double);

/* "scipy/special/_ufuncs.pyx":1521
 * cdef _proto_eval_sh_jacobi_double__t *_proto_eval_sh_jacobi_double__t_var = &_func_eval_sh_jacobi[double]
 * from .orthogonal_eval cimport eval_sh_jacobi_l as _func_eval_sh_jacobi_l
 * ctypedef double _proto_eval_sh_jacobi_l_t(long, double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_jacobi_l_t *_proto_eval_sh_jacobi_l_t_var = &_func_eval_sh_jacobi_l
 * from .orthogonal_eval cimport eval_sh_legendre as _func_eval_sh_legendre
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_l_t(long, double, double, double);

/* "scipy/special/_ufuncs.pyx":1524
 * cdef _proto_eval_sh_jacobi_l_t *_proto_eval_sh_jacobi_l_t_var = &_func_eval_sh_jacobi_l
 * from .orthogonal_eval cimport eval_sh_legendre as _func_eval_sh_legendre
 * ctypedef double complex _proto_eval_sh_legendre_double_complex__t(double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_legendre_double_complex__t *_proto_eval_sh_legendre_double_complex__t_var = &_func_eval_sh_legendre[double_complex]
 * from .orthogonal_eval cimport eval_sh_legendre as _func_eval_sh_legendre
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_double_complex__t(double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1527
 * cdef _proto_eval_sh_legendre_double_complex__t *_proto_eval_sh_legendre_double_complex__t_var = &_func_eval_sh_legendre[double_complex]
 * from .orthogonal_eval cimport eval_sh_legendre as _func_eval_sh_legendre
 * ctypedef double _proto_eval_sh_legendre_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_legendre_double__t *_proto_eval_sh_legendre_double__t_var = &_func_eval_sh_legendre[double]
 * from .orthogonal_eval cimport eval_sh_legendre_l as _func_eval_sh_legendre_l
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1530
 * cdef _proto_eval_sh_legendre_double__t *_proto_eval_sh_legendre_double__t_var = &_func_eval_sh_legendre[double]
 * from .orthogonal_eval cimport eval_sh_legendre_l as _func_eval_sh_legendre_l
 * ctypedef double _proto_eval_sh_legendre_l_t(long, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_eval_sh_legendre_l_t *_proto_eval_sh_legendre_l_t_var = &_func_eval_sh_legendre_l
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_l_t(long, double);

/* "scipy/special/_ufuncs.pyx":1551
 *     cdef long double _func_expitl "expitl"(long double) nogil
 * from ._cunity cimport cexpm1 as _func_cexpm1
 * ctypedef double complex _proto_cexpm1_t(double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_cexpm1_t *_proto_cexpm1_t_var = &_func_cexpm1
 * cdef extern from "_ufuncs_defs.h":
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_cexpm1_t(__pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1556
 *     cdef double _func_expm1 "expm1"(double) nogil
 * from ._legacy cimport expn_unsafe as _func_expn_unsafe
 * ctypedef double _proto_expn_unsafe_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_expn_unsafe_t *_proto_expn_unsafe_t_var = &_func_expn_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_expn_unsafe_t(double, double);

/* "scipy/special/_ufuncs.pyx":1561
 *     cdef double _func_expn "expn"(int, double) nogil
 * from ._exprel cimport exprel as _func_exprel
 * ctypedef double _proto_exprel_t(double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_exprel_t *_proto_exprel_t_var = &_func_exprel
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_exprel_t(double);

/* "scipy/special/_ufuncs.pyx":1576
 *     cdef int _func_cfresnl_wrap "cfresnl_wrap"(double complex, double complex *, double complex *) nogil
 * from ._loggamma cimport cgamma as _func_cgamma
 * ctypedef double complex _proto_cgamma_t(double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_cgamma_t *_proto_cgamma_t_var = &_func_cgamma
 * cdef extern from "_ufuncs_defs.h":
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_cgamma_t(__pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1611
 *     cdef double complex _func_cbesh_wrap2_e "cbesh_wrap2_e"(double, double complex) nogil
 * from ._convex_analysis cimport huber as _func_huber
 * ctypedef double _proto_huber_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_huber_t *_proto_huber_t_var = &_func_huber
 * from ._hyp0f1 cimport _hyp0f1_cmplx as _func__hyp0f1_cmplx
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_huber_t(double, double);

/* "scipy/special/_ufuncs.pyx":1614
 * cdef _proto_huber_t *_proto_huber_t_var = &_func_huber
 * from ._hyp0f1 cimport _hyp0f1_cmplx as _func__hyp0f1_cmplx
 * ctypedef double complex _proto__hyp0f1_cmplx_t(double, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto__hyp0f1_cmplx_t *_proto__hyp0f1_cmplx_t_var = &_func__hyp0f1_cmplx
 * from ._hyp0f1 cimport _hyp0f1_real as _func__hyp0f1_real
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto__hyp0f1_cmplx_t(double, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1617
 * cdef _proto__hyp0f1_cmplx_t *_proto__hyp0f1_cmplx_t_var = &_func__hyp0f1_cmplx
 * from ._hyp0f1 cimport _hyp0f1_real as _func__hyp0f1_real
 * ctypedef double _proto__hyp0f1_real_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto__hyp0f1_real_t *_proto__hyp0f1_real_t_var = &_func__hyp0f1_real
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto__hyp0f1_real_t(double, double);

/* "scipy/special/_ufuncs.pyx":1626
 *     cdef double _func_onef2 "onef2"(double, double, double, double, double *) nogil
 * from ._legacy cimport hyp2f0_unsafe as _func_hyp2f0_unsafe
 * ctypedef double _proto_hyp2f0_unsafe_t(double, double, double, double, double *) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_hyp2f0_unsafe_t *_proto_hyp2f0_unsafe_t_var = &_func_hyp2f0_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_hyp2f0_unsafe_t(double, double, double, double, double *);

/* "scipy/special/_ufuncs.pyx":1647
 *     cdef double _func_i1e "i1e"(double) nogil
 * from ._boxcox cimport inv_boxcox as _func_inv_boxcox
 * ctypedef double _proto_inv_boxcox_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_inv_boxcox_t *_proto_inv_boxcox_t_var = &_func_inv_boxcox
 * from ._boxcox cimport inv_boxcox1p as _func_inv_boxcox1p
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_inv_boxcox_t(double, double);

/* "scipy/special/_ufuncs.pyx":1650
 * cdef _proto_inv_boxcox_t *_proto_inv_boxcox_t_var = &_func_inv_boxcox
 * from ._boxcox cimport inv_boxcox1p as _func_inv_boxcox1p
 * ctypedef double _proto_inv_boxcox1p_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_inv_boxcox1p_t *_proto_inv_boxcox1p_t_var = &_func_inv_boxcox1p
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_inv_boxcox1p_t(double, double);

/* "scipy/special/_ufuncs.pyx":1707
 *     cdef double _func_kerp_wrap "kerp_wrap"(double) nogil
 * from ._convex_analysis cimport kl_div as _func_kl_div
 * ctypedef double _proto_kl_div_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_kl_div_t *_proto_kl_div_t_var = &_func_kl_div
 * from ._legacy cimport kn_unsafe as _func_kn_unsafe
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_kl_div_t(double, double);

/* "scipy/special/_ufuncs.pyx":1710
 * cdef _proto_kl_div_t *_proto_kl_div_t_var = &_func_kl_div
 * from ._legacy cimport kn_unsafe as _func_kn_unsafe
 * ctypedef double _proto_kn_unsafe_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_kn_unsafe_t *_proto_kn_unsafe_t_var = &_func_kn_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_kn_unsafe_t(double, double);

/* "scipy/special/_ufuncs.pyx":1727
 *     cdef double _func_cbesk_wrap_e_real "cbesk_wrap_e_real"(double, double) nogil
 * from ._cunity cimport clog1p as _func_clog1p
 * ctypedef double complex _proto_clog1p_t(double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_clog1p_t *_proto_clog1p_t_var = &_func_clog1p
 * cdef extern from "_ufuncs_defs.h":
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_clog1p_t(__pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1734
 *     cdef double _func_log_ndtr "log_ndtr"(double) nogil
 * from ._loggamma cimport loggamma_real as _func_loggamma_real
 * ctypedef double _proto_loggamma_real_t(double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_loggamma_real_t *_proto_loggamma_real_t_var = &_func_loggamma_real
 * from ._loggamma cimport loggamma as _func_loggamma
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_loggamma_real_t(double);

/* "scipy/special/_ufuncs.pyx":1737
 * cdef _proto_loggamma_real_t *_proto_loggamma_real_t_var = &_func_loggamma_real
 * from ._loggamma cimport loggamma as _func_loggamma
 * ctypedef double complex _proto_loggamma_t(double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_loggamma_t *_proto_loggamma_t_var = &_func_loggamma
 * cdef extern from "_ufuncs_defs.h":
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_loggamma_t(__pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1770
 *     cdef double _func_struve_l "struve_l"(double, double) nogil
 * from ._legacy cimport nbdtr_unsafe as _func_nbdtr_unsafe
 * ctypedef double _proto_nbdtr_unsafe_t(double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_nbdtr_unsafe_t *_proto_nbdtr_unsafe_t_var = &_func_nbdtr_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_nbdtr_unsafe_t(double, double, double);

/* "scipy/special/_ufuncs.pyx":1775
 *     cdef double _func_nbdtr "nbdtr"(int, int, double) nogil
 * from ._legacy cimport nbdtrc_unsafe as _func_nbdtrc_unsafe
 * ctypedef double _proto_nbdtrc_unsafe_t(double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_nbdtrc_unsafe_t *_proto_nbdtrc_unsafe_t_var = &_func_nbdtrc_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_nbdtrc_unsafe_t(double, double, double);

/* "scipy/special/_ufuncs.pyx":1780
 *     cdef double _func_nbdtrc "nbdtrc"(int, int, double) nogil
 * from ._legacy cimport nbdtri_unsafe as _func_nbdtri_unsafe
 * ctypedef double _proto_nbdtri_unsafe_t(double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_nbdtri_unsafe_t *_proto_nbdtri_unsafe_t_var = &_func_nbdtri_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_nbdtri_unsafe_t(double, double, double);

/* "scipy/special/_ufuncs.pyx":1837
 *     cdef int _func_pbwa_wrap "pbwa_wrap"(double, double, double *, double *) nogil
 * from ._legacy cimport pdtr_unsafe as _func_pdtr_unsafe
 * ctypedef double _proto_pdtr_unsafe_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_pdtr_unsafe_t *_proto_pdtr_unsafe_t_var = &_func_pdtr_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_pdtr_unsafe_t(double, double);

/* "scipy/special/_ufuncs.pyx":1842
 *     cdef double _func_pdtr "pdtr"(int, double) nogil
 * from ._legacy cimport pdtrc_unsafe as _func_pdtrc_unsafe
 * ctypedef double _proto_pdtrc_unsafe_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_pdtrc_unsafe_t *_proto_pdtrc_unsafe_t_var = &_func_pdtrc_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_pdtrc_unsafe_t(double, double);

/* "scipy/special/_ufuncs.pyx":1847
 *     cdef double _func_pdtrc "pdtrc"(int, double) nogil
 * from ._legacy cimport pdtri_unsafe as _func_pdtri_unsafe
 * ctypedef double _proto_pdtri_unsafe_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_pdtri_unsafe_t *_proto_pdtri_unsafe_t_var = &_func_pdtri_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_pdtri_unsafe_t(double, double);

/* "scipy/special/_ufuncs.pyx":1870
 *     cdef int _func_prolate_radial2_wrap "prolate_radial2_wrap"(double, double, double, double, double, double *, double *) nogil
 * from ._convex_analysis cimport pseudo_huber as _func_pseudo_huber
 * ctypedef double _proto_pseudo_huber_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_pseudo_huber_t *_proto_pseudo_huber_t_var = &_func_pseudo_huber
 * from ._digamma cimport cdigamma as _func_cdigamma
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_pseudo_huber_t(double, double);

/* "scipy/special/_ufuncs.pyx":1873
 * cdef _proto_pseudo_huber_t *_proto_pseudo_huber_t_var = &_func_pseudo_huber
 * from ._digamma cimport cdigamma as _func_cdigamma
 * ctypedef double complex _proto_cdigamma_t(double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_cdigamma_t *_proto_cdigamma_t_var = &_func_cdigamma
 * from ._digamma cimport digamma as _func_digamma
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_cdigamma_t(__pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1876
 * cdef _proto_cdigamma_t *_proto_cdigamma_t_var = &_func_cdigamma
 * from ._digamma cimport digamma as _func_digamma
 * ctypedef double _proto_digamma_t(double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_digamma_t *_proto_digamma_t_var = &_func_digamma
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_digamma_t(double);

/* "scipy/special/_ufuncs.pyx":1881
 *     cdef double _func_radian "radian"(double, double, double) nogil
 * from ._convex_analysis cimport rel_entr as _func_rel_entr
 * ctypedef double _proto_rel_entr_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_rel_entr_t *_proto_rel_entr_t_var = &_func_rel_entr
 * from ._loggamma cimport crgamma as _func_crgamma
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_rel_entr_t(double, double);

/* "scipy/special/_ufuncs.pyx":1884
 * cdef _proto_rel_entr_t *_proto_rel_entr_t_var = &_func_rel_entr
 * from ._loggamma cimport crgamma as _func_crgamma
 * ctypedef double complex _proto_crgamma_t(double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_crgamma_t *_proto_crgamma_t_var = &_func_crgamma
 * cdef extern from "_ufuncs_defs.h":
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_crgamma_t(__pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1891
 *     cdef double _func_round "round"(double) nogil
 * from ._sici cimport cshichi as _func_cshichi
 * ctypedef int _proto_cshichi_t(double complex, double complex *, double complex *) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_cshichi_t *_proto_cshichi_t_var = &_func_cshichi
 * cdef extern from "_ufuncs_defs.h":
 */
typedef int __pyx_t_5scipy_7special_7_ufuncs__proto_cshichi_t(__pyx_t_double_complex, __pyx_t_double_complex *, __pyx_t_double_complex *);

/* "scipy/special/_ufuncs.pyx":1896
 *     cdef int _func_shichi "shichi"(double, double *, double *) nogil
 * from ._sici cimport csici as _func_csici
 * ctypedef int _proto_csici_t(double complex, double complex *, double complex *) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_csici_t *_proto_csici_t_var = &_func_csici
 * cdef extern from "_ufuncs_defs.h":
 */
typedef int __pyx_t_5scipy_7special_7_ufuncs__proto_csici_t(__pyx_t_double_complex, __pyx_t_double_complex *, __pyx_t_double_complex *);

/* "scipy/special/_ufuncs.pyx":1903
 *     cdef double _func_sindg "sindg"(double) nogil
 * from ._legacy cimport smirnov_unsafe as _func_smirnov_unsafe
 * ctypedef double _proto_smirnov_unsafe_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_smirnov_unsafe_t *_proto_smirnov_unsafe_t_var = &_func_smirnov_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_smirnov_unsafe_t(double, double);

/* "scipy/special/_ufuncs.pyx":1908
 *     cdef double _func_smirnov "smirnov"(int, double) nogil
 * from ._legacy cimport smirnovi_unsafe as _func_smirnovi_unsafe
 * ctypedef double _proto_smirnovi_unsafe_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_smirnovi_unsafe_t *_proto_smirnovi_unsafe_t_var = &_func_smirnovi_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_smirnovi_unsafe_t(double, double);

/* "scipy/special/_ufuncs.pyx":1913
 *     cdef double _func_smirnovi "smirnovi"(int, double) nogil
 * from ._spence cimport cspence as _func_cspence
 * ctypedef double complex _proto_cspence_t(double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_cspence_t *_proto_cspence_t_var = &_func_cspence
 * cdef extern from "_ufuncs_defs.h":
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_cspence_t(__pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1918
 *     cdef double _func_spence "spence"(double) nogil
 * from ._legacy cimport sph_harmonic_unsafe as _func_sph_harmonic_unsafe
 * ctypedef double complex _proto_sph_harmonic_unsafe_t(double, double, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_sph_harmonic_unsafe_t *_proto_sph_harmonic_unsafe_t_var = &_func_sph_harmonic_unsafe
 * from .sph_harm cimport sph_harmonic as _func_sph_harmonic
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_sph_harmonic_unsafe_t(double, double, double, double);

/* "scipy/special/_ufuncs.pyx":1921
 * cdef _proto_sph_harmonic_unsafe_t *_proto_sph_harmonic_unsafe_t_var = &_func_sph_harmonic_unsafe
 * from .sph_harm cimport sph_harmonic as _func_sph_harmonic
 * ctypedef double complex _proto_sph_harmonic_t(int, int, double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_sph_harmonic_t *_proto_sph_harmonic_t_var = &_func_sph_harmonic
 * cdef extern from "_ufuncs_defs.h":
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_sph_harmonic_t(int, int, double, double);

/* "scipy/special/_ufuncs.pyx":1936
 *     cdef double _func_tukeylambdacdf "tukeylambdacdf"(double, double) nogil
 * from ._xlogy cimport xlog1py as _func_xlog1py
 * ctypedef double _proto_xlog1py_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_xlog1py_double__t *_proto_xlog1py_double__t_var = &_func_xlog1py[double]
 * from ._xlogy cimport xlog1py as _func_xlog1py
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_xlog1py_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1939
 * cdef _proto_xlog1py_double__t *_proto_xlog1py_double__t_var = &_func_xlog1py[double]
 * from ._xlogy cimport xlog1py as _func_xlog1py
 * ctypedef double complex _proto_xlog1py_double_complex__t(double complex, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_xlog1py_double_complex__t *_proto_xlog1py_double_complex__t_var = &_func_xlog1py[double_complex]
 * from ._xlogy cimport xlogy as _func_xlogy
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_xlog1py_double_complex__t(__pyx_t_double_complex, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1942
 * cdef _proto_xlog1py_double_complex__t *_proto_xlog1py_double_complex__t_var = &_func_xlog1py[double_complex]
 * from ._xlogy cimport xlogy as _func_xlogy
 * ctypedef double _proto_xlogy_double__t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_xlogy_double__t *_proto_xlogy_double__t_var = &_func_xlogy[double]
 * from ._xlogy cimport xlogy as _func_xlogy
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_xlogy_double__t(double, double);

/* "scipy/special/_ufuncs.pyx":1945
 * cdef _proto_xlogy_double__t *_proto_xlogy_double__t_var = &_func_xlogy[double]
 * from ._xlogy cimport xlogy as _func_xlogy
 * ctypedef double complex _proto_xlogy_double_complex__t(double complex, double complex) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_xlogy_double_complex__t *_proto_xlogy_double_complex__t_var = &_func_xlogy[double_complex]
 * cdef extern from "_ufuncs_defs.h":
 */
typedef __pyx_t_double_complex __pyx_t_5scipy_7special_7_ufuncs__proto_xlogy_double_complex__t(__pyx_t_double_complex, __pyx_t_double_complex);

/* "scipy/special/_ufuncs.pyx":1952
 *     cdef double _func_y1 "y1"(double) nogil
 * from ._legacy cimport yn_unsafe as _func_yn_unsafe
 * ctypedef double _proto_yn_unsafe_t(double, double) nogil             # <<<<<<<<<<<<<<
 * cdef _proto_yn_unsafe_t *_proto_yn_unsafe_t_var = &_func_yn_unsafe
 * cdef extern from "_ufuncs_defs.h":
 */
typedef double __pyx_t_5scipy_7special_7_ufuncs__proto_yn_unsafe_t(double, double);

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod0.proto */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* UnpackTupleError.proto */
static void __Pyx_UnpackTupleError(PyObject *, Py_ssize_t index);

/* UnpackTuple2.proto */
#define __Pyx_unpack_tuple2(tuple, value1, value2, is_tuple, has_known_size, decref_tuple)\
    (likely(is_tuple || PyTuple_Check(tuple)) ?\
        (likely(has_known_size || PyTuple_GET_SIZE(tuple) == 2) ?\
            __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple) :\
            (__Pyx_UnpackTupleError(tuple, 2), -1)) :\
        __Pyx_unpack_tuple2_generic(tuple, value1, value2, has_known_size, decref_tuple))
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
    PyObject* tuple, PyObject** value1, PyObject** value2, int decref_tuple);
static int __Pyx_unpack_tuple2_generic(
    PyObject* tuple, PyObject** value1, PyObject** value2, int has_known_size, int decref_tuple);

/* dict_iter.proto */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* dict, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_is_dict);
static CYTHON_INLINE int __Pyx_dict_iter_next(PyObject* dict_or_iter, Py_ssize_t orig_length, Py_ssize_t* ppos,
                                              PyObject** pkey, PyObject** pvalue, PyObject** pitem, int is_dict);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

/* py_dict_keys.proto */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Keys(PyObject* d);

/* UnpackUnboundCMethod.proto */
typedef struct {
    PyObject *type;
    PyObject **method_name;
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;

/* CallUnboundCMethod0.proto */
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CallUnboundCMethod0(cfunc, self)\
    (likely((cfunc)->func) ?\
        (likely((cfunc)->flag == METH_NOARGS) ?  (*((cfunc)->func))(self, NULL) :\
         (PY_VERSION_HEX >= 0x030600B1 && likely((cfunc)->flag == METH_FASTCALL) ?\
            (PY_VERSION_HEX >= 0x030700A0 ?\
                (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0) :\
                (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0, NULL)) :\
          (PY_VERSION_HEX >= 0x030700A0 && (cfunc)->flag == (METH_FASTCALL | METH_KEYWORDS) ?\
            (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0, NULL) :\
            (likely((cfunc)->flag == (METH_VARARGS | METH_KEYWORDS)) ?  ((*(PyCFunctionWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, __pyx_empty_tuple, NULL)) :\
               ((cfunc)->flag == METH_VARARGS ?  (*((cfunc)->func))(self, __pyx_empty_tuple) :\
               __Pyx__CallUnboundCMethod0(cfunc, self)))))) :\
        __Pyx__CallUnboundCMethod0(cfunc, self))
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* py_dict_pop.proto */
static CYTHON_INLINE PyObject *__Pyx_PyDict_Pop(PyObject *d, PyObject *key, PyObject *default_value);

/* CallUnboundCMethod2.proto */
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030600B1
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2);
#else
#define __Pyx_CallUnboundCMethod2(cfunc, self, arg1, arg2)  __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2)
#endif

/* CallUnboundCMethod1.proto */
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#else
#define __Pyx_CallUnboundCMethod1(cfunc, self, arg)  __Pyx__CallUnboundCMethod1(cfunc, self, arg)
#endif

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* None.proto */
static CYTHON_INLINE long __Pyx_div_long(long, long);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunction.proto */
#define __Pyx_CyFunction_USED 1
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_Check(obj)  (__Pyx_TypeCheck(obj, __pyx_CyFunctionType))
#define __Pyx_CyFunction_NewEx(ml, flags, qualname, self, module, globals, code)\
    __Pyx_CyFunction_New(__pyx_CyFunctionType, ml, flags, qualname, self, module, globals, code)
static PyObject *__Pyx_CyFunction_New(PyTypeObject *, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* SetNameInClass.proto */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? _PyDict_SetItem_KnownHash(ns, name, value, ((PyASCIIObject *) name)->hash) : PyObject_SetItem(ns, name, value))
#elif CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? PyDict_SetItem(ns, name, value) : PyObject_SetItem(ns, name, value))
#else
#define __Pyx_SetNameInClass(ns, name, value)  PyObject_SetItem(ns, name, value)
#endif

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* RealImag.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if defined(__cplusplus) && CYTHON_CCOMPLEX\
        && (defined(_WIN32) || defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5 || __GNUC__ == 4 && __GNUC_MINOR__ >= 4 )) || __cplusplus >= 201103)
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_double(a, b) ((a)-(b))
    #define __Pyx_c_prod_double(a, b) ((a)*(b))
    #define __Pyx_c_quot_double(a, b) ((a)/(b))
    #define __Pyx_c_neg_double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_double(z) ((z)==(double)0)
    #define __Pyx_c_conj_double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (::std::abs(z))
        #define __Pyx_c_pow_double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_double(z) ((z)==0)
    #define __Pyx_c_conj_double(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (cabs(z))
        #define __Pyx_c_pow_double(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_sf_action_t(sf_action_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Py_intptr_t(Py_intptr_t value);

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq_float(a, b)   ((a)==(b))
    #define __Pyx_c_sum_float(a, b)  ((a)+(b))
    #define __Pyx_c_diff_float(a, b) ((a)-(b))
    #define __Pyx_c_prod_float(a, b) ((a)*(b))
    #define __Pyx_c_quot_float(a, b) ((a)/(b))
    #define __Pyx_c_neg_float(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_float(z) ((z)==(float)0)
    #define __Pyx_c_conj_float(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (::std::abs(z))
        #define __Pyx_c_pow_float(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_float(z) ((z)==0)
    #define __Pyx_c_conj_float(z)    (conjf(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (cabsf(z))
        #define __Pyx_c_pow_float(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__NPY_TYPES(enum NPY_TYPES value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* None.proto */
static CYTHON_INLINE long __Pyx_pow_long(long, long);

/* CIntFromPy.proto */
static CYTHON_INLINE sf_error_t __Pyx_PyInt_As_sf_error_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE sf_action_t __Pyx_PyInt_As_sf_action_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE Py_intptr_t __Pyx_PyInt_As_Py_intptr_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* VoidPtrImport.proto */
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig);

/* FunctionImport.proto */
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;
static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *, char *, char *, int *); /*proto*/
static CYTHON_INLINE int __pyx_f_5numpy_import_array(void); /*proto*/
static CYTHON_INLINE int __pyx_f_5numpy_import_ufunc(void); /*proto*/

/* Module declarations from 'libc.math' */

/* Module declarations from 'libc' */

/* Module declarations from 'scipy.special.cython_special' */

/* Module declarations from 'scipy.special.sf_error' */
static CYTHON_INLINE int __pyx_f_5scipy_7special_8sf_error__sf_error_test_function(int); /*proto*/

/* Module declarations from 'scipy.special._ufuncs_cxx' */
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn)
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn_complex = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn_complex (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn_complex)
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erf = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erf (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erf)
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfc = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfc (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfc)
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx)
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx_complex = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx_complex (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx_complex)
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi)
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi_complex = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi_complex (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi_complex)
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_log_ndtr = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_log_ndtr (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_log_ndtr)
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_ndtr = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_ndtr (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_ndtr)
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_w = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_w (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_w)
static void **__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_wrightomega = 0;
#define __pyx_v_5scipy_7special_11_ufuncs_cxx__export_wrightomega (*__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_wrightomega)
static void (*__pyx_f_5scipy_7special_11_ufuncs_cxx__set_action)(sf_error_t, sf_action_t); /*proto*/

/* Module declarations from 'scipy.special' */

/* Module declarations from 'scipy' */

/* Module declarations from 'cython' */

/* Module declarations from 'scipy.special._complexstuff' */
static CYTHON_INLINE npy_cdouble __pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_13_complexstuff_double_complex_from_npy_cdouble(npy_cdouble); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_13_complexstuff_zpack(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_13_complexstuff_zlog1(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE int __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zisnan(double); /*proto*/
static CYTHON_INLINE int __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisnan(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE int __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisfinite(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE int __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisinf(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zabs(double); /*proto*/
static CYTHON_INLINE double __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zlog(double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zlog(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zexp(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zsqrt(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zpow(__pyx_t_double_complex, double); /*proto*/

/* Module declarations from 'scipy.special._trig' */
static CYTHON_INLINE double __pyx_f_5scipy_7special_5_trig_dsinpi(double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_5_trig_dcospi(double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_5_trig_csinpi(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_5_trig_ccospi(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_5_trig_sinpi(double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_5_trig_sinpi(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_5_trig_cospi(double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_5_trig_cospi(__pyx_t_double_complex); /*proto*/

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'numpy.math' */

/* Module declarations from 'scipy.special._ellip_harm' */
static CYTHON_INLINE double *__pyx_f_5scipy_7special_11_ellip_harm_lame_coefficients(double, double, int, int, void **, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_11_ellip_harm_ellip_harm_eval(double, double, int, int, double, double *, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_11_ellip_harm_ellip_harmonic(double, double, int, int, double, double, double); /*proto*/

/* Module declarations from 'scipy.special.sph_harm' */
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8sph_harm_sph_harmonic(int, int, double, double); /*proto*/

/* Module declarations from 'scipy.special._cephes' */

/* Module declarations from 'scipy.special._legacy' */
static CYTHON_INLINE void __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(char *, double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_7_legacy_sph_harmonic_unsafe(double, double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_ellip_harmonic_unsafe(double, double, double, double, double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_bdtrc_unsafe(double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_bdtr_unsafe(double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_bdtri_unsafe(double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_expn_unsafe(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_hyp2f0_unsafe(double, double, double, double, double *); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_nbdtrc_unsafe(double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_nbdtr_unsafe(double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_nbdtri_unsafe(double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_pdtrc_unsafe(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_pdtr_unsafe(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_pdtri_unsafe(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_kn_unsafe(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_yn_unsafe(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_smirnov_unsafe(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_smirnovc_unsafe(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_smirnovp_unsafe(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_smirnovi_unsafe(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_smirnovci_unsafe(double, double); /*proto*/

/* Module declarations from 'scipy.special._evalpoly' */
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_9_evalpoly_cevalpoly(double *, int, __pyx_t_double_complex); /*proto*/

/* Module declarations from 'scipy.special.lambertw' */
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8lambertw_lambertw_scalar(__pyx_t_double_complex, long, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8lambertw_lambertw_branchpt(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8lambertw_lambertw_pade0(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8lambertw_lambertw_asy(__pyx_t_double_complex, long); /*proto*/

/* Module declarations from 'scipy.special._spherical_bessel' */
static CYTHON_INLINE double __pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_real(long, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_complex(long, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_real(long, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_complex(long, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_real(long, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_complex(long, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_real(long, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_complex(long, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_d_real(long, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_d_complex(long, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_d_real(long, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_d_complex(long, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_d_real(long, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_d_complex(long, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_d_real(long, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_d_complex(long, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_17_spherical_bessel_cbesj(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_17_spherical_bessel_cbesj(double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_17_spherical_bessel_cbesy(double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_17_spherical_bessel_cbesk(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_17_spherical_bessel_cbesk(double, __pyx_t_double_complex); /*proto*/

/* Module declarations from 'scipy.special._agm' */
static CYTHON_INLINE double __pyx_f_5scipy_7special_4_agm__agm_iter(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_4_agm_agm(double, double); /*proto*/

/* Module declarations from 'scipy.special.orthogonal_eval' */
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_binom(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi_l(long, double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi_l(long, double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer_l(long, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt_l(long, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu_l(long, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys_l(long, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc_l(long, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt_l(long, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu_l(long, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre_l(long, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre_l(long, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre_l(long, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre_l(long, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_hermitenorm(long, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_hermite(long, double); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(double, double, double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(double, double, double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_hyp1f1(double, double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_hyp1f1(double, double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi(double, double, double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi(double, double, double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi(double, double, double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi(double, double, double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer(double, double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer(double, double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt(double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu(double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys(double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc(double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt(double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu(double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre(double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre(double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre(double, double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre(double, double, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre(double, __pyx_t_double_complex); /*proto*/

/* Module declarations from 'scipy.special._boxcox' */
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_boxcox_boxcox(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_boxcox_boxcox1p(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_boxcox_inv_boxcox(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_boxcox_inv_boxcox1p(double, double); /*proto*/

/* Module declarations from 'scipy.special._convex_analysis' */
static CYTHON_INLINE double __pyx_f_5scipy_7special_16_convex_analysis_entr(double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_16_convex_analysis_kl_div(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_16_convex_analysis_rel_entr(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_16_convex_analysis_huber(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_16_convex_analysis_pseudo_huber(double, double); /*proto*/

/* Module declarations from 'scipy.special._cunity' */
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_7_cunity_clog1p(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_7_cunity_clog1p_ddouble(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_7_cunity_cexpm1(__pyx_t_double_complex); /*proto*/

/* Module declarations from 'scipy.special._exprel' */
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_exprel_exprel(double); /*proto*/

/* Module declarations from 'scipy.special._loggamma' */
static CYTHON_INLINE double __pyx_f_5scipy_7special_9_loggamma_loggamma_real(double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_9_loggamma_loggamma(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_9_loggamma_loggamma_recurrence(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_9_loggamma_loggamma_stirling(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_9_loggamma_loggamma_taylor(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_9_loggamma_cgamma(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_9_loggamma_crgamma(__pyx_t_double_complex); /*proto*/

/* Module declarations from 'scipy.special._xlogy' */
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_6_xlogy_xlogy(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_6_xlogy_xlogy(__pyx_t_double_complex, __pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_6_xlogy_xlog1py(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_6_xlogy_xlog1py(__pyx_t_double_complex, __pyx_t_double_complex); /*proto*/

/* Module declarations from 'scipy.special._hyp0f1' */
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_hyp0f1__hyp0f1_real(double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5scipy_7special_7_hyp0f1__hyp0f1_asy(double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_7_hyp0f1__hyp0f1_cmplx(double, __pyx_t_double_complex); /*proto*/

/* Module declarations from 'scipy.special._digamma' */
static CYTHON_INLINE double __pyx_f_5scipy_7special_8_digamma_digamma(double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8_digamma_cdigamma(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8_digamma_forward_recurrence(__pyx_t_double_complex, __pyx_t_double_complex, int); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8_digamma_backward_recurrence(__pyx_t_double_complex, __pyx_t_double_complex, int); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8_digamma_asymptotic_series(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_8_digamma_zeta_series(double, double, double); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_8_digamma_zeta_series(__pyx_t_double_complex, double, double); /*proto*/

/* Module declarations from 'scipy.special._sici' */
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_5_sici_zexpi(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE void __pyx_f_5scipy_7special_5_sici_power_series(int, __pyx_t_double_complex, __pyx_t_double_complex *, __pyx_t_double_complex *); /*proto*/
static CYTHON_INLINE int __pyx_f_5scipy_7special_5_sici_csici(__pyx_t_double_complex, __pyx_t_double_complex *, __pyx_t_double_complex *); /*proto*/
static CYTHON_INLINE int __pyx_f_5scipy_7special_5_sici_cshichi(__pyx_t_double_complex, __pyx_t_double_complex *, __pyx_t_double_complex *); /*proto*/

/* Module declarations from 'scipy.special._spence' */
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_7_spence_cspence(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_7_spence_cspence_series0(__pyx_t_double_complex); /*proto*/
static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_7_spence_cspence_series1(__pyx_t_double_complex); /*proto*/

/* Module declarations from 'scipy.special._ufuncs' */
static __pyx_t_5scipy_7special_7_ufuncs__proto_ccospi_t *__pyx_v_5scipy_7special_7_ufuncs__proto_ccospi_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_dcospi_t *__pyx_v_5scipy_7special_7_ufuncs__proto_dcospi_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_ellip_harmonic_t *__pyx_v_5scipy_7special_7_ufuncs__proto_ellip_harmonic_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_ellip_harmonic_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_ellip_harmonic_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_lambertw_scalar_t *__pyx_v_5scipy_7special_7_ufuncs__proto_lambertw_scalar_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto__sf_error_test_function_t *__pyx_v_5scipy_7special_7_ufuncs__proto__sf_error_test_function_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_csinpi_t *__pyx_v_5scipy_7special_7_ufuncs__proto_csinpi_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_dsinpi_t *__pyx_v_5scipy_7special_7_ufuncs__proto_dsinpi_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_smirnovc_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_smirnovc_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_smirnovci_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_smirnovci_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_smirnovp_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_smirnovp_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_in_complex_t *__pyx_v_5scipy_7special_7_ufuncs__proto_spherical_in_complex_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_in_real_t *__pyx_v_5scipy_7special_7_ufuncs__proto_spherical_in_real_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_in_d_complex_t *__pyx_v_5scipy_7special_7_ufuncs__proto_spherical_in_d_complex_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_in_d_real_t *__pyx_v_5scipy_7special_7_ufuncs__proto_spherical_in_d_real_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_jn_complex_t *__pyx_v_5scipy_7special_7_ufuncs__proto_spherical_jn_complex_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_jn_real_t *__pyx_v_5scipy_7special_7_ufuncs__proto_spherical_jn_real_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_jn_d_complex_t *__pyx_v_5scipy_7special_7_ufuncs__proto_spherical_jn_d_complex_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_jn_d_real_t *__pyx_v_5scipy_7special_7_ufuncs__proto_spherical_jn_d_real_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_kn_complex_t *__pyx_v_5scipy_7special_7_ufuncs__proto_spherical_kn_complex_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_kn_real_t *__pyx_v_5scipy_7special_7_ufuncs__proto_spherical_kn_real_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_kn_d_complex_t *__pyx_v_5scipy_7special_7_ufuncs__proto_spherical_kn_d_complex_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_kn_d_real_t *__pyx_v_5scipy_7special_7_ufuncs__proto_spherical_kn_d_real_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_yn_complex_t *__pyx_v_5scipy_7special_7_ufuncs__proto_spherical_yn_complex_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_yn_real_t *__pyx_v_5scipy_7special_7_ufuncs__proto_spherical_yn_real_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_yn_d_complex_t *__pyx_v_5scipy_7special_7_ufuncs__proto_spherical_yn_d_complex_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_spherical_yn_d_real_t *__pyx_v_5scipy_7special_7_ufuncs__proto_spherical_yn_d_real_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_agm_t *__pyx_v_5scipy_7special_7_ufuncs__proto_agm_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_bdtr_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_bdtr_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_bdtrc_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_bdtrc_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_bdtri_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_bdtri_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_binom_t *__pyx_v_5scipy_7special_7_ufuncs__proto_binom_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_boxcox_t *__pyx_v_5scipy_7special_7_ufuncs__proto_boxcox_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_boxcox1p_t *__pyx_v_5scipy_7special_7_ufuncs__proto_boxcox1p_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_entr_t *__pyx_v_5scipy_7special_7_ufuncs__proto_entr_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyc_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyc_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyc_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyc_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyc_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyc_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebys_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebys_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebys_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebys_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebys_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebys_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyt_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyt_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyt_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyt_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyt_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyt_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyu_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyu_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyu_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyu_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_chebyu_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyu_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_hermite_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_hermite_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_hermitenorm_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_hermitenorm_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_jacobi_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_jacobi_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_jacobi_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_jacobi_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_jacobi_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_jacobi_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_laguerre_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_laguerre_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_laguerre_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_laguerre_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_laguerre_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_laguerre_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_legendre_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_legendre_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_legendre_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_legendre_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_legendre_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_legendre_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_l_t *__pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_l_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_cexpm1_t *__pyx_v_5scipy_7special_7_ufuncs__proto_cexpm1_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_expn_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_expn_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_exprel_t *__pyx_v_5scipy_7special_7_ufuncs__proto_exprel_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_cgamma_t *__pyx_v_5scipy_7special_7_ufuncs__proto_cgamma_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_huber_t *__pyx_v_5scipy_7special_7_ufuncs__proto_huber_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto__hyp0f1_cmplx_t *__pyx_v_5scipy_7special_7_ufuncs__proto__hyp0f1_cmplx_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto__hyp0f1_real_t *__pyx_v_5scipy_7special_7_ufuncs__proto__hyp0f1_real_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_hyp2f0_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_hyp2f0_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_inv_boxcox_t *__pyx_v_5scipy_7special_7_ufuncs__proto_inv_boxcox_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_inv_boxcox1p_t *__pyx_v_5scipy_7special_7_ufuncs__proto_inv_boxcox1p_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_kl_div_t *__pyx_v_5scipy_7special_7_ufuncs__proto_kl_div_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_kn_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_kn_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_clog1p_t *__pyx_v_5scipy_7special_7_ufuncs__proto_clog1p_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_loggamma_real_t *__pyx_v_5scipy_7special_7_ufuncs__proto_loggamma_real_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_loggamma_t *__pyx_v_5scipy_7special_7_ufuncs__proto_loggamma_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_nbdtr_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_nbdtr_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_nbdtrc_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_nbdtrc_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_nbdtri_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_nbdtri_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_pdtr_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_pdtr_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_pdtrc_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_pdtrc_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_pdtri_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_pdtri_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_pseudo_huber_t *__pyx_v_5scipy_7special_7_ufuncs__proto_pseudo_huber_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_cdigamma_t *__pyx_v_5scipy_7special_7_ufuncs__proto_cdigamma_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_digamma_t *__pyx_v_5scipy_7special_7_ufuncs__proto_digamma_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_rel_entr_t *__pyx_v_5scipy_7special_7_ufuncs__proto_rel_entr_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_crgamma_t *__pyx_v_5scipy_7special_7_ufuncs__proto_crgamma_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_cshichi_t *__pyx_v_5scipy_7special_7_ufuncs__proto_cshichi_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_csici_t *__pyx_v_5scipy_7special_7_ufuncs__proto_csici_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_smirnov_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_smirnov_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_smirnovi_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_smirnovi_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_cspence_t *__pyx_v_5scipy_7special_7_ufuncs__proto_cspence_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_sph_harmonic_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_sph_harmonic_unsafe_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_sph_harmonic_t *__pyx_v_5scipy_7special_7_ufuncs__proto_sph_harmonic_t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_xlog1py_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_xlog1py_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_xlog1py_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_xlog1py_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_xlogy_double__t *__pyx_v_5scipy_7special_7_ufuncs__proto_xlogy_double__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_xlogy_double_complex__t *__pyx_v_5scipy_7special_7_ufuncs__proto_xlogy_double_complex__t_var;
static __pyx_t_5scipy_7special_7_ufuncs__proto_yn_unsafe_t *__pyx_v_5scipy_7special_7_ufuncs__proto_yn_unsafe_t_var;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[24];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_loops[1];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_ptr[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_data[1];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__sf_error_test_function_loops[1];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__sf_error_test_function_ptr[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__sf_error_test_function_data[1];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__sf_error_test_function_types[2];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__sf_error_test_function_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_types[9];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_types[9];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_types[9];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_loops[1];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_ptr[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_data[1];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_types[5];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_loops[1];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_ptr[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_data[1];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_types[5];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_loops[1];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_ptr[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_data[1];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_types[5];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[20];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[20];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[15];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[15];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[15];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[15];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[20];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[20];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_loops[1];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_ptr[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_data[1];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_types[3];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_loops[1];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_ptr[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_data[1];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_types[3];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[25];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[15];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[15];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[15];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[15];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[25];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_loops[5];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[10];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_data[5];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[15];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types[9];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[16];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[18];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[20];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types[9];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[10];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[14];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[14];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[14];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types[9];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types[9];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types[9];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[14];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[14];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[14];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_round_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_round_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types[9];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types[9];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_types[8];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[15];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_types[6];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_loops[3];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_ptr[6];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_data[3];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types[9];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_loops[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[8];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_data[4];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[12];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_doc;
static PyUFuncGenericFunction __pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_loops[2];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_ptr[4];
static void *__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_data[2];
static char __pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_types[4];
static char *__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_doc;
__PYX_EXTERN_C int wrap_PyUFunc_getfperr(void); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs__set_action(sf_error_t, sf_action_t); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_DD__As_DD_D(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_DD__As_FF_F(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_Dld__As_Dld_D(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_ddD__As_ddD_D(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_ddD__As_ffF_F(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dddD__As_dddD_D(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dddD__As_fffF_F(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dddd__As_dddd_D(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dddd__As_ffff_F(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_iidd__As_lldd_D(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_lD__As_lD_D(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_dddd_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_ffff_f(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddddddd__As_ddddddd_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddddddd__As_fffffff_f(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dddi_d_As_dddl_dd(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddi_d_As_ddl_dd(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddiiddd__As_ddllddd_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_iid__As_lld_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ldd__As_ldd_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_lddd__As_lddd_d(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_f_f__As_f_f(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_g_g__As_g_g(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DDDD_As_D_DDDD(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DDDD_As_F_FFFF(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DD_As_D_DD(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DD_As_F_FF(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DDDD_As_d_DDDD(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DDDD_As_f_FFFF(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DD_As_d_DD(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DD_As_f_FF(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_d_dd(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_f_ff(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dddd_As_d_dddd(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dddd_As_f_ffff(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dd_As_dd_dd(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dd_As_ff_ff(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dddd_As_dd_dddd(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dddd_As_ff_ffff(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_ddd_dd(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_fff_ff(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_ddddd_dd(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_fffff_ff(char **, npy_intp *, npy_intp *, void *); /*proto*/
static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_i__As_l_l(char **, npy_intp *, npy_intp *, void *); /*proto*/
#define __Pyx_MODULE_NAME "scipy.special._ufuncs"
extern int __pyx_module_is_main_scipy__special___ufuncs;
int __pyx_module_is_main_scipy__special___ufuncs = 0;

/* Implementation of 'scipy.special._ufuncs' */
static PyObject *__pyx_builtin_object;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_RuntimeError;
static PyObject *__pyx_builtin_ImportError;
static PyObject *__pyx_builtin_RuntimeWarning;
static const char __pyx_k_i0[] = "i0";
static const char __pyx_k_i1[] = "i1";
static const char __pyx_k_iv[] = "iv";
static const char __pyx_k_j0[] = "j0";
static const char __pyx_k_j1[] = "j1";
static const char __pyx_k_jn[] = "jn";
static const char __pyx_k_jv[] = "jv";
static const char __pyx_k_k0[] = "k0";
static const char __pyx_k_k1[] = "k1";
static const char __pyx_k_kn[] = "kn";
static const char __pyx_k_kv[] = "kv";
static const char __pyx_k_np[] = "np";
static const char __pyx_k_y0[] = "y0";
static const char __pyx_k_y1[] = "y1";
static const char __pyx_k_yn[] = "yn";
static const char __pyx_k_yv[] = "yv";
static const char __pyx_k_agm[] = "agm";
static const char __pyx_k_all[] = "all";
static const char __pyx_k_arg[] = "arg";
static const char __pyx_k_bei[] = "bei";
static const char __pyx_k_ber[] = "ber";
static const char __pyx_k_doc[] = "__doc__";
static const char __pyx_k_erf[] = "erf";
static const char __pyx_k_err[] = "err";
static const char __pyx_k_i0e[] = "i0e";
static const char __pyx_k_i1e[] = "i1e";
static const char __pyx_k_ive[] = "ive";
static const char __pyx_k_jve[] = "jve";
static const char __pyx_k_k0e[] = "k0e";
static const char __pyx_k_k1e[] = "k1e";
static const char __pyx_k_kei[] = "kei";
static const char __pyx_k_ker[] = "ker";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_kve[] = "kve";
static const char __pyx_k_pop[] = "pop";
static const char __pyx_k_psi[] = "psi";
static const char __pyx_k_yve[] = "yve";
static const char __pyx_k_airy[] = "airy";
static const char __pyx_k_bdtr[] = "bdtr";
static const char __pyx_k_beip[] = "beip";
static const char __pyx_k_berp[] = "berp";
static const char __pyx_k_beta[] = "beta";
static const char __pyx_k_cbrt[] = "cbrt";
static const char __pyx_k_code[] = "code";
static const char __pyx_k_entr[] = "entr";
static const char __pyx_k_erfc[] = "erfc";
static const char __pyx_k_erfi[] = "erfi";
static const char __pyx_k_exit[] = "__exit__";
static const char __pyx_k_exp1[] = "exp1";
static const char __pyx_k_exp2[] = "exp2";
static const char __pyx_k_expi[] = "expi";
static const char __pyx_k_expn[] = "expn";
static const char __pyx_k_fdtr[] = "fdtr";
static const char __pyx_k_gdtr[] = "gdtr";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_keip[] = "keip";
static const char __pyx_k_kerp[] = "kerp";
static const char __pyx_k_keys[] = "keys";
static const char __pyx_k_loss[] = "loss";
static const char __pyx_k_lpmv[] = "lpmv";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_ndtr[] = "ndtr";
static const char __pyx_k_pbdv[] = "pbdv";
static const char __pyx_k_pbvv[] = "pbvv";
static const char __pyx_k_pbwa[] = "pbwa";
static const char __pyx_k_pdtr[] = "pdtr";
static const char __pyx_k_poch[] = "poch";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_sici[] = "sici";
static const char __pyx_k_slow[] = "slow";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_warn[] = "warn";
static const char __pyx_k_wofz[] = "wofz";
static const char __pyx_k_zeta[] = "_zeta";
static const char __pyx_k_airye[] = "airye";
static const char __pyx_k_bdtrc[] = "bdtrc";
static const char __pyx_k_bdtri[] = "bdtri";
static const char __pyx_k_binom[] = "binom";
static const char __pyx_k_btdtr[] = "btdtr";
static const char __pyx_k_chdtr[] = "chdtr";
static const char __pyx_k_cosdg[] = "cosdg";
static const char __pyx_k_cosm1[] = "cosm1";
static const char __pyx_k_cospi[] = "_cospi";
static const char __pyx_k_cotdg[] = "cotdg";
static const char __pyx_k_dawsn[] = "dawsn";
static const char __pyx_k_enter[] = "__enter__";
static const char __pyx_k_erfcx[] = "erfcx";
static const char __pyx_k_error[] = "error";
static const char __pyx_k_exp10[] = "exp10";
static const char __pyx_k_expit[] = "expit";
static const char __pyx_k_expm1[] = "expm1";
static const char __pyx_k_fdtrc[] = "fdtrc";
static const char __pyx_k_fdtri[] = "fdtri";
static const char __pyx_k_gamma[] = "gamma";
static const char __pyx_k_gdtrc[] = "gdtrc";
static const char __pyx_k_huber[] = "huber";
static const char __pyx_k_items[] = "items";
static const char __pyx_k_log1p[] = "log1p";
static const char __pyx_k_logit[] = "logit";
static const char __pyx_k_nbdtr[] = "nbdtr";
static const char __pyx_k_ndtri[] = "ndtri";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_other[] = "other";
static const char __pyx_k_pdtrc[] = "pdtrc";
static const char __pyx_k_pdtri[] = "pdtri";
static const char __pyx_k_raise[] = "raise";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_round[] = "round";
static const char __pyx_k_sindg[] = "sindg";
static const char __pyx_k_sinpi[] = "_sinpi";
static const char __pyx_k_stdtr[] = "stdtr";
static const char __pyx_k_tandg[] = "tandg";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_xlogy[] = "xlogy";
static const char __pyx_k_zetac[] = "zetac";
static const char __pyx_k_action[] = "action";
static const char __pyx_k_bdtrik[] = "bdtrik";
static const char __pyx_k_bdtrin[] = "bdtrin";
static const char __pyx_k_betaln[] = "betaln";
static const char __pyx_k_boxcox[] = "boxcox";
static const char __pyx_k_btdtri[] = "btdtri";
static const char __pyx_k_chdtrc[] = "chdtrc";
static const char __pyx_k_chdtri[] = "chdtri";
static const char __pyx_k_chndtr[] = "chndtr";
static const char __pyx_k_domain[] = "domain";
static const char __pyx_k_ellipe[] = "ellipe";
static const char __pyx_k_ellipj[] = "ellipj";
static const char __pyx_k_exprel[] = "exprel";
static const char __pyx_k_gdtria[] = "gdtria";
static const char __pyx_k_gdtrib[] = "gdtrib";
static const char __pyx_k_gdtrix[] = "gdtrix";
static const char __pyx_k_geterr[] = "geterr";
static const char __pyx_k_hyp0f1[] = "hyp0f1";
static const char __pyx_k_hyp1f1[] = "hyp1f1";
static const char __pyx_k_hyp1f2[] = "hyp1f2";
static const char __pyx_k_hyp2f0[] = "hyp2f0";
static const char __pyx_k_hyp2f1[] = "hyp2f1";
static const char __pyx_k_hyp3f0[] = "hyp3f0";
static const char __pyx_k_hyperu[] = "hyperu";
static const char __pyx_k_ignore[] = "ignore";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_itairy[] = "itairy";
static const char __pyx_k_iti0k0[] = "iti0k0";
static const char __pyx_k_itj0y0[] = "itj0y0";
static const char __pyx_k_kelvin[] = "kelvin";
static const char __pyx_k_kl_div[] = "kl_div";
static const char __pyx_k_kwargs[] = "kwargs";
static const char __pyx_k_lgam1p[] = "_lgam1p";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_nbdtrc[] = "nbdtrc";
static const char __pyx_k_nbdtri[] = "nbdtri";
static const char __pyx_k_ncfdtr[] = "ncfdtr";
static const char __pyx_k_nctdtr[] = "nctdtr";
static const char __pyx_k_object[] = "object";
static const char __pyx_k_obl_cv[] = "obl_cv";
static const char __pyx_k_olderr[] = "olderr";
static const char __pyx_k_pdtrik[] = "pdtrik";
static const char __pyx_k_pro_cv[] = "pro_cv";
static const char __pyx_k_radian[] = "radian";
static const char __pyx_k_rgamma[] = "rgamma";
static const char __pyx_k_seterr[] = "seterr";
static const char __pyx_k_shichi[] = "shichi";
static const char __pyx_k_spence[] = "spence";
static const char __pyx_k_struve[] = "struve";
static const char __pyx_k_betainc[] = "betainc";
static const char __pyx_k_btdtria[] = "btdtria";
static const char __pyx_k_btdtrib[] = "btdtrib";
static const char __pyx_k_chdtriv[] = "chdtriv";
static const char __pyx_k_fresnel[] = "fresnel";
static const char __pyx_k_gammaln[] = "gammaln";
static const char __pyx_k_hankel1[] = "hankel1";
static const char __pyx_k_hankel2[] = "hankel2";
static const char __pyx_k_it2i0k0[] = "it2i0k0";
static const char __pyx_k_it2j0y0[] = "it2j0y0";
static const char __pyx_k_kolmogc[] = "_kolmogc";
static const char __pyx_k_kolmogi[] = "kolmogi";
static const char __pyx_k_kolmogp[] = "_kolmogp";
static const char __pyx_k_log1pmx[] = "_log1pmx";
static const char __pyx_k_nbdtrik[] = "nbdtrik";
static const char __pyx_k_nbdtrin[] = "nbdtrin";
static const char __pyx_k_ncfdtri[] = "ncfdtri";
static const char __pyx_k_owens_t[] = "owens_t";
static const char __pyx_k_prepare[] = "__prepare__";
static const char __pyx_k_smirnov[] = "smirnov";
static const char __pyx_k_stdtrit[] = "stdtrit";
static const char __pyx_k_tklmbda[] = "tklmbda";
static const char __pyx_k_xlog1py[] = "xlog1py";
static const char __pyx_k_boxcox1p[] = "boxcox1p";
static const char __pyx_k_chndtrix[] = "chndtrix";
static const char __pyx_k_ellipkm1[] = "ellipkm1";
static const char __pyx_k_errstate[] = "errstate";
static const char __pyx_k_exc_type[] = "exc_type";
static const char __pyx_k_fdtridfd[] = "fdtridfd";
static const char __pyx_k_gammainc[] = "gammainc";
static const char __pyx_k_gammasgn[] = "gammasgn";
static const char __pyx_k_hankel1e[] = "hankel1e";
static const char __pyx_k_hankel2e[] = "hankel2e";
static const char __pyx_k_igam_fac[] = "_igam_fac";
static const char __pyx_k_kolmogci[] = "_kolmogci";
static const char __pyx_k_lambertw[] = "_lambertw";
static const char __pyx_k_log_ndtr[] = "log_ndtr";
static const char __pyx_k_loggamma[] = "loggamma";
static const char __pyx_k_nctdtrit[] = "nctdtrit";
static const char __pyx_k_nrdtrimn[] = "nrdtrimn";
static const char __pyx_k_nrdtrisd[] = "nrdtrisd";
static const char __pyx_k_obl_ang1[] = "obl_ang1";
static const char __pyx_k_obl_rad1[] = "obl_rad1";
static const char __pyx_k_obl_rad2[] = "obl_rad2";
static const char __pyx_k_oldstate[] = "oldstate";
static const char __pyx_k_overflow[] = "overflow";
static const char __pyx_k_pro_ang1[] = "pro_ang1";
static const char __pyx_k_pro_rad1[] = "pro_rad1";
static const char __pyx_k_pro_rad2[] = "pro_rad2";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_rel_entr[] = "rel_entr";
static const char __pyx_k_singular[] = "singular";
static const char __pyx_k_smirnovc[] = "_smirnovc";
static const char __pyx_k_smirnovi[] = "smirnovi";
static const char __pyx_k_smirnovp[] = "_smirnovp";
static const char __pyx_k_sph_harm[] = "sph_harm";
static const char __pyx_k_stdtridf[] = "stdtridf";
static const char __pyx_k_chndtridf[] = "chndtridf";
static const char __pyx_k_chndtrinc[] = "chndtrinc";
static const char __pyx_k_ellipeinc[] = "ellipeinc";
static const char __pyx_k_ellipkinc[] = "ellipkinc";
static const char __pyx_k_exc_value[] = "exc_value";
static const char __pyx_k_gammaincc[] = "gammaincc";
static const char __pyx_k_itstruve0[] = "itstruve0";
static const char __pyx_k_mathieu_a[] = "mathieu_a";
static const char __pyx_k_mathieu_b[] = "mathieu_b";
static const char __pyx_k_metaclass[] = "__metaclass__";
static const char __pyx_k_modstruve[] = "modstruve";
static const char __pyx_k_ncfdtrinc[] = "ncfdtrinc";
static const char __pyx_k_nctdtridf[] = "nctdtridf";
static const char __pyx_k_nctdtrinc[] = "nctdtrinc";
static const char __pyx_k_newkwargs[] = "newkwargs";
static const char __pyx_k_no_result[] = "no_result";
static const char __pyx_k_smirnovci[] = "_smirnovci";
static const char __pyx_k_traceback[] = "traceback";
static const char __pyx_k_underflow[] = "underflow";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_besselpoly[] = "besselpoly";
static const char __pyx_k_betaincinv[] = "betaincinv";
static const char __pyx_k_ellip_harm[] = "_ellip_harm";
static const char __pyx_k_inv_boxcox[] = "inv_boxcox";
static const char __pyx_k_it2struve0[] = "it2struve0";
static const char __pyx_k_kolmogorov[] = "kolmogorov";
static const char __pyx_k_ncfdtridfd[] = "ncfdtridfd";
static const char __pyx_k_ncfdtridfn[] = "ncfdtridfn";
static const char __pyx_k_ImportError[] = "ImportError";
static const char __pyx_k_eval_chebyc[] = "eval_chebyc";
static const char __pyx_k_eval_chebys[] = "eval_chebys";
static const char __pyx_k_eval_chebyt[] = "eval_chebyt";
static const char __pyx_k_eval_chebyu[] = "eval_chebyu";
static const char __pyx_k_eval_jacobi[] = "eval_jacobi";
static const char __pyx_k_gammaincinv[] = "gammaincinv";
static const char __pyx_k_mathieu_cem[] = "mathieu_cem";
static const char __pyx_k_mathieu_sem[] = "mathieu_sem";
static const char __pyx_k_modfresnelm[] = "modfresnelm";
static const char __pyx_k_modfresnelp[] = "modfresnelp";
static const char __pyx_k_obl_ang1_cv[] = "obl_ang1_cv";
static const char __pyx_k_obl_rad1_cv[] = "obl_rad1_cv";
static const char __pyx_k_obl_rad2_cv[] = "obl_rad2_cv";
static const char __pyx_k_pro_ang1_cv[] = "pro_ang1_cv";
static const char __pyx_k_pro_rad1_cv[] = "pro_rad1_cv";
static const char __pyx_k_pro_rad2_cv[] = "pro_rad2_cv";
static const char __pyx_k_wrightomega[] = "wrightomega";
static const char __pyx_k_RuntimeError[] = "RuntimeError";
static const char __pyx_k_eval_hermite[] = "eval_hermite";
static const char __pyx_k_gammainccinv[] = "gammainccinv";
static const char __pyx_k_inv_boxcox1p[] = "inv_boxcox1p";
static const char __pyx_k_itmodstruve0[] = "itmodstruve0";
static const char __pyx_k_pseudo_huber[] = "pseudo_huber";
static const char __pyx_k_spherical_in[] = "_spherical_in";
static const char __pyx_k_spherical_jn[] = "_spherical_jn";
static const char __pyx_k_spherical_kn[] = "_spherical_kn";
static const char __pyx_k_spherical_yn[] = "_spherical_yn";
static const char __pyx_k_eval_laguerre[] = "eval_laguerre";
static const char __pyx_k_eval_legendre[] = "eval_legendre";
static const char __pyx_k_RuntimeWarning[] = "RuntimeWarning";
static const char __pyx_k_eval_sh_chebyt[] = "eval_sh_chebyt";
static const char __pyx_k_eval_sh_chebyu[] = "eval_sh_chebyu";
static const char __pyx_k_eval_sh_jacobi[] = "eval_sh_jacobi";
static const char __pyx_k_geterr_line_28[] = "geterr (line 28)";
static const char __pyx_k_seterr_line_77[] = "seterr (line 77)";
static const char __pyx_k_spherical_in_d[] = "_spherical_in_d";
static const char __pyx_k_spherical_jn_d[] = "_spherical_jn_d";
static const char __pyx_k_spherical_kn_d[] = "_spherical_kn_d";
static const char __pyx_k_spherical_yn_d[] = "_spherical_yn_d";
static const char __pyx_k_errstate___exit[] = "errstate.__exit__";
static const char __pyx_k_errstate___init[] = "errstate.__init__";
static const char __pyx_k_eval_gegenbauer[] = "eval_gegenbauer";
static const char __pyx_k_mathieu_modcem1[] = "mathieu_modcem1";
static const char __pyx_k_mathieu_modcem2[] = "mathieu_modcem2";
static const char __pyx_k_mathieu_modsem1[] = "mathieu_modsem1";
static const char __pyx_k_mathieu_modsem2[] = "mathieu_modsem2";
static const char __pyx_k_errstate___enter[] = "errstate.__enter__";
static const char __pyx_k_eval_genlaguerre[] = "eval_genlaguerre";
static const char __pyx_k_eval_hermitenorm[] = "eval_hermitenorm";
static const char __pyx_k_eval_sh_legendre[] = "eval_sh_legendre";
static const char __pyx_k_sf_error_code_map[] = "_sf_error_code_map";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_sf_error_action_map[] = "_sf_error_action_map";
static const char __pyx_k_struve_power_series[] = "_struve_power_series";
static const char __pyx_k_struve_asymp_large_z[] = "_struve_asymp_large_z";
static const char __pyx_k_struve_bessel_series[] = "_struve_bessel_series";
static const char __pyx_k_scipy_special__ufuncs[] = "scipy.special._ufuncs";
static const char __pyx_k_ufuncs_extra_code_pxi[] = "_ufuncs_extra_code.pxi";
static const char __pyx_k_sf_error_test_function[] = "_sf_error_test_function";
static const char __pyx_k_lanczos_sum_expg_scaled[] = "_lanczos_sum_expg_scaled";
static const char __pyx_k_ndarray_is_not_C_contiguous[] = "ndarray is not C contiguous";
static const char __pyx_k_Get_the_current_way_of_handling[] = "Get the current way of handling special-function errors.\n\n    Returns\n    -------\n    err : dict\n        A dictionary with keys \"singular\", \"underflow\", \"overflow\",\n        \"slow\", \"loss\", \"no_result\", \"domain\", \"arg\", and \"other\",\n        whose values are from the strings \"ignore\", \"warn\", and\n        \"raise\". The keys represent possible special-function errors,\n        and the values define how these errors are handled.\n\n    See Also\n    --------\n    seterr : set how special-function errors are handled\n    errstate : context manager for special-function error handling\n    numpy.geterr : similar numpy function for floating-point errors\n\n    Notes\n    -----\n    For complete documentation of the types of special-function errors\n    and treatment options, see `seterr`.\n\n    Examples\n    --------\n    By default all errors are ignored.\n\n    >>> import scipy.special as sc\n    >>> for key, value in sorted(sc.geterr().items()):\n    ...     print(\"{}: {}\".format(key, value))\n    ...\n    arg: ignore\n    domain: ignore\n    loss: ignore\n    no_result: ignore\n    other: ignore\n    overflow: ignore\n    singular: ignore\n    slow: ignore\n    underflow: ignore\n\n    ";
static const char __pyx_k_Set_how_special_function_errors[] = "Set how special-function errors are handled.\n\n    Parameters\n    ----------\n    all : {'ignore', 'warn' 'raise'}, optional\n        Set treatment for all type of special-function errors at\n        once. The options are:\n\n        - 'ignore' Take no action when the error occurs\n        - 'warn' Print a `SpecialFunctionWarning` when the error\n          occurs (via the Python `warnings` module)\n        - 'raise' Raise a `SpecialFunctionError` when the error\n          occurs.\n\n        The default is to not change the current behavior. If\n        behaviors for additional categories of special-function errors\n        are specified, then ``all`` is applied first, followed by the\n        additional categories.\n    singular : {'ignore', 'warn', 'raise'}, optional\n        Treatment for singularities.\n    underflow : {'ignore', 'warn', 'raise'}, optional\n        Treatment for underflow.\n    overflow : {'ignore', 'warn', 'raise'}, optional\n        Treatment for overflow.\n    slow : {'ignore', 'warn', 'raise'}, optional\n        Treatment for slow convergence.\n    loss : {'ignore', 'warn', 'raise'}, optional\n        Treatment for loss of accuracy.\n    no_result : {'ignore', 'warn', 'raise'}, optional\n        Treatment for failing to find a result.\n    domain : {'ignore', 'warn', 'raise'}, optional\n        Treatment for an invalid argument to a function.\n    arg : {'ignore', 'warn', 'raise'}, optional\n        Treatment for an invalid parameter to a function.\n    other : {'ignore', 'warn', 'raise'}, optional\n        Treatment for an unknown error.\n\n    Returns\n    -------\n    olderr : dict\n        Dictionary containing the old settings.\n\n    See Also\n    --------\n    geterr : get the current way of handling special-function errors\n    errstate : context manager for special-function error handling\n    numpy.seterr : similar numpy function for floating-point errors\n\n    Examples\n    --------\n    >>> import scipy.special as sc\n    >>> f""rom pytest import raises\n    >>> sc.gammaln(0)\n    inf\n    >>> olderr = sc.seterr(singular='raise')\n    >>> with raises(sc.SpecialFunctionError):\n    ...     sc.gammaln(0)\n    ...\n    >>> _ = sc.seterr(**olderr)\n\n    We can also raise for every category except one.\n\n    >>> olderr = sc.seterr(all='raise', singular='ignore')\n    >>> sc.gammaln(0)\n    inf\n    >>> with raises(sc.SpecialFunctionError):\n    ...     sc.spence(-1)\n    ...\n    >>> _ = sc.seterr(**olderr)\n\n    ";
static const char __pyx_k_numpy_core_multiarray_failed_to[] = "numpy.core.multiarray failed to import";
static const char __pyx_k_unknown_dtype_code_in_numpy_pxd[] = "unknown dtype code in numpy.pxd (%d)";
static const char __pyx_k_Context_manager_for_special_func[] = "Context manager for special-function error handling.\n\n    Using an instance of `errstate` as a context manager allows\n    statements in that context to execute with a known error handling\n    behavior. Upon entering the context the error handling is set with\n    `seterr`, and upon exiting it is restored to what it was before.\n\n    Parameters\n    ----------\n    kwargs : {all, singular, underflow, overflow, slow, loss, no_result, domain, arg, other}\n        Keyword arguments. The valid keywords are possible\n        special-function errors. Each keyword should have a string\n        value that defines the treatement for the particular type of\n        error. Values must be 'ignore', 'warn', or 'other'. See\n        `seterr` for details.\n\n    See Also\n    --------\n    geterr : get the current way of handling special-function errors\n    seterr : set how special-function errors are handled\n    numpy.errstate : similar numpy function for floating-point errors\n\n    Examples\n    --------\n    >>> import scipy.special as sc\n    >>> from pytest import raises\n    >>> sc.gammaln(0)\n    inf\n    >>> with sc.errstate(singular='raise'):\n    ...     with raises(sc.SpecialFunctionError):\n    ...         sc.gammaln(0)\n    ...\n    >>> sc.gammaln(0)\n    inf\n\n    We can also raise on every category except one.\n\n    >>> with sc.errstate(all='raise', singular='ignore'):\n    ...     sc.gammaln(0)\n    ...     with raises(sc.SpecialFunctionError):\n    ...         sc.spence(-1)\n    ...\n    inf\n\n    ";
static const char __pyx_k_Format_string_allocated_too_shor[] = "Format string allocated too short, see comment in numpy.pxd";
static const char __pyx_k_Non_native_byte_order_not_suppor[] = "Non-native byte order not supported";
static const char __pyx_k_ndarray_is_not_Fortran_contiguou[] = "ndarray is not Fortran contiguous";
static const char __pyx_k_numpy_core_umath_failed_to_impor[] = "numpy.core.umath failed to import";
static const char __pyx_k_Format_string_allocated_too_shor_2[] = "Format string allocated too short.";
static PyObject *__pyx_kp_s_Context_manager_for_special_func;
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor;
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor_2;
static PyObject *__pyx_kp_u_Get_the_current_way_of_handling;
static PyObject *__pyx_n_s_ImportError;
static PyObject *__pyx_kp_u_Non_native_byte_order_not_suppor;
static PyObject *__pyx_n_s_RuntimeError;
static PyObject *__pyx_n_s_RuntimeWarning;
static PyObject *__pyx_kp_u_Set_how_special_function_errors;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_action;
static PyObject *__pyx_n_s_agm;
static PyObject *__pyx_n_s_airy;
static PyObject *__pyx_n_s_airye;
static PyObject *__pyx_n_u_all;
static PyObject *__pyx_n_u_arg;
static PyObject *__pyx_n_s_bdtr;
static PyObject *__pyx_n_s_bdtrc;
static PyObject *__pyx_n_s_bdtri;
static PyObject *__pyx_n_s_bdtrik;
static PyObject *__pyx_n_s_bdtrin;
static PyObject *__pyx_n_s_bei;
static PyObject *__pyx_n_s_beip;
static PyObject *__pyx_n_s_ber;
static PyObject *__pyx_n_s_berp;
static PyObject *__pyx_n_s_besselpoly;
static PyObject *__pyx_n_s_beta;
static PyObject *__pyx_n_s_betainc;
static PyObject *__pyx_n_s_betaincinv;
static PyObject *__pyx_n_s_betaln;
static PyObject *__pyx_n_s_binom;
static PyObject *__pyx_n_s_boxcox;
static PyObject *__pyx_n_s_boxcox1p;
static PyObject *__pyx_n_s_btdtr;
static PyObject *__pyx_n_s_btdtri;
static PyObject *__pyx_n_s_btdtria;
static PyObject *__pyx_n_s_btdtrib;
static PyObject *__pyx_n_s_cbrt;
static PyObject *__pyx_n_s_chdtr;
static PyObject *__pyx_n_s_chdtrc;
static PyObject *__pyx_n_s_chdtri;
static PyObject *__pyx_n_s_chdtriv;
static PyObject *__pyx_n_s_chndtr;
static PyObject *__pyx_n_s_chndtridf;
static PyObject *__pyx_n_s_chndtrinc;
static PyObject *__pyx_n_s_chndtrix;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_code;
static PyObject *__pyx_n_s_cosdg;
static PyObject *__pyx_n_s_cosm1;
static PyObject *__pyx_n_s_cospi;
static PyObject *__pyx_n_s_cotdg;
static PyObject *__pyx_n_s_dawsn;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_u_domain;
static PyObject *__pyx_n_s_ellip_harm;
static PyObject *__pyx_n_s_ellipe;
static PyObject *__pyx_n_s_ellipeinc;
static PyObject *__pyx_n_s_ellipj;
static PyObject *__pyx_n_s_ellipkinc;
static PyObject *__pyx_n_s_ellipkm1;
static PyObject *__pyx_n_s_enter;
static PyObject *__pyx_n_s_entr;
static PyObject *__pyx_n_s_erf;
static PyObject *__pyx_n_s_erfc;
static PyObject *__pyx_n_s_erfcx;
static PyObject *__pyx_n_s_erfi;
static PyObject *__pyx_n_s_err;
static PyObject *__pyx_n_s_error;
static PyObject *__pyx_n_s_errstate;
static PyObject *__pyx_n_s_errstate___enter;
static PyObject *__pyx_n_s_errstate___exit;
static PyObject *__pyx_n_s_errstate___init;
static PyObject *__pyx_n_s_eval_chebyc;
static PyObject *__pyx_n_s_eval_chebys;
static PyObject *__pyx_n_s_eval_chebyt;
static PyObject *__pyx_n_s_eval_chebyu;
static PyObject *__pyx_n_s_eval_gegenbauer;
static PyObject *__pyx_n_s_eval_genlaguerre;
static PyObject *__pyx_n_s_eval_hermite;
static PyObject *__pyx_n_s_eval_hermitenorm;
static PyObject *__pyx_n_s_eval_jacobi;
static PyObject *__pyx_n_s_eval_laguerre;
static PyObject *__pyx_n_s_eval_legendre;
static PyObject *__pyx_n_s_eval_sh_chebyt;
static PyObject *__pyx_n_s_eval_sh_chebyu;
static PyObject *__pyx_n_s_eval_sh_jacobi;
static PyObject *__pyx_n_s_eval_sh_legendre;
static PyObject *__pyx_n_s_exc_type;
static PyObject *__pyx_n_s_exc_value;
static PyObject *__pyx_n_s_exit;
static PyObject *__pyx_n_s_exp1;
static PyObject *__pyx_n_s_exp10;
static PyObject *__pyx_n_s_exp2;
static PyObject *__pyx_n_s_expi;
static PyObject *__pyx_n_s_expit;
static PyObject *__pyx_n_s_expm1;
static PyObject *__pyx_n_s_expn;
static PyObject *__pyx_n_s_exprel;
static PyObject *__pyx_n_s_fdtr;
static PyObject *__pyx_n_s_fdtrc;
static PyObject *__pyx_n_s_fdtri;
static PyObject *__pyx_n_s_fdtridfd;
static PyObject *__pyx_n_s_fresnel;
static PyObject *__pyx_n_s_gamma;
static PyObject *__pyx_n_s_gammainc;
static PyObject *__pyx_n_s_gammaincc;
static PyObject *__pyx_n_s_gammainccinv;
static PyObject *__pyx_n_s_gammaincinv;
static PyObject *__pyx_n_s_gammaln;
static PyObject *__pyx_n_s_gammasgn;
static PyObject *__pyx_n_s_gdtr;
static PyObject *__pyx_n_s_gdtrc;
static PyObject *__pyx_n_s_gdtria;
static PyObject *__pyx_n_s_gdtrib;
static PyObject *__pyx_n_s_gdtrix;
static PyObject *__pyx_n_s_geterr;
static PyObject *__pyx_kp_u_geterr_line_28;
static PyObject *__pyx_n_s_hankel1;
static PyObject *__pyx_n_s_hankel1e;
static PyObject *__pyx_n_s_hankel2;
static PyObject *__pyx_n_s_hankel2e;
static PyObject *__pyx_n_s_huber;
static PyObject *__pyx_n_s_hyp0f1;
static PyObject *__pyx_n_s_hyp1f1;
static PyObject *__pyx_n_s_hyp1f2;
static PyObject *__pyx_n_s_hyp2f0;
static PyObject *__pyx_n_s_hyp2f1;
static PyObject *__pyx_n_s_hyp3f0;
static PyObject *__pyx_n_s_hyperu;
static PyObject *__pyx_n_s_i0;
static PyObject *__pyx_n_s_i0e;
static PyObject *__pyx_n_s_i1;
static PyObject *__pyx_n_s_i1e;
static PyObject *__pyx_n_s_igam_fac;
static PyObject *__pyx_n_u_ignore;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_inv_boxcox;
static PyObject *__pyx_n_s_inv_boxcox1p;
static PyObject *__pyx_n_s_it2i0k0;
static PyObject *__pyx_n_s_it2j0y0;
static PyObject *__pyx_n_s_it2struve0;
static PyObject *__pyx_n_s_itairy;
static PyObject *__pyx_n_s_items;
static PyObject *__pyx_n_s_iti0k0;
static PyObject *__pyx_n_s_itj0y0;
static PyObject *__pyx_n_s_itmodstruve0;
static PyObject *__pyx_n_s_itstruve0;
static PyObject *__pyx_n_s_iv;
static PyObject *__pyx_n_s_ive;
static PyObject *__pyx_n_s_j0;
static PyObject *__pyx_n_s_j1;
static PyObject *__pyx_n_s_jn;
static PyObject *__pyx_n_s_jv;
static PyObject *__pyx_n_s_jve;
static PyObject *__pyx_n_s_k0;
static PyObject *__pyx_n_s_k0e;
static PyObject *__pyx_n_s_k1;
static PyObject *__pyx_n_s_k1e;
static PyObject *__pyx_n_s_kei;
static PyObject *__pyx_n_s_keip;
static PyObject *__pyx_n_s_kelvin;
static PyObject *__pyx_n_s_ker;
static PyObject *__pyx_n_s_kerp;
static PyObject *__pyx_n_s_key;
static PyObject *__pyx_n_s_keys;
static PyObject *__pyx_n_s_kl_div;
static PyObject *__pyx_n_s_kn;
static PyObject *__pyx_n_s_kolmogc;
static PyObject *__pyx_n_s_kolmogci;
static PyObject *__pyx_n_s_kolmogi;
static PyObject *__pyx_n_s_kolmogorov;
static PyObject *__pyx_n_s_kolmogp;
static PyObject *__pyx_n_s_kv;
static PyObject *__pyx_n_s_kve;
static PyObject *__pyx_n_s_kwargs;
static PyObject *__pyx_n_s_lambertw;
static PyObject *__pyx_n_s_lanczos_sum_expg_scaled;
static PyObject *__pyx_n_s_lgam1p;
static PyObject *__pyx_n_s_log1p;
static PyObject *__pyx_n_s_log1pmx;
static PyObject *__pyx_n_s_log_ndtr;
static PyObject *__pyx_n_s_loggamma;
static PyObject *__pyx_n_s_logit;
static PyObject *__pyx_n_u_loss;
static PyObject *__pyx_n_s_lpmv;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_mathieu_a;
static PyObject *__pyx_n_s_mathieu_b;
static PyObject *__pyx_n_s_mathieu_cem;
static PyObject *__pyx_n_s_mathieu_modcem1;
static PyObject *__pyx_n_s_mathieu_modcem2;
static PyObject *__pyx_n_s_mathieu_modsem1;
static PyObject *__pyx_n_s_mathieu_modsem2;
static PyObject *__pyx_n_s_mathieu_sem;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_modfresnelm;
static PyObject *__pyx_n_s_modfresnelp;
static PyObject *__pyx_n_s_modstruve;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_nbdtr;
static PyObject *__pyx_n_s_nbdtrc;
static PyObject *__pyx_n_s_nbdtri;
static PyObject *__pyx_n_s_nbdtrik;
static PyObject *__pyx_n_s_nbdtrin;
static PyObject *__pyx_n_s_ncfdtr;
static PyObject *__pyx_n_s_ncfdtri;
static PyObject *__pyx_n_s_ncfdtridfd;
static PyObject *__pyx_n_s_ncfdtridfn;
static PyObject *__pyx_n_s_ncfdtrinc;
static PyObject *__pyx_n_s_nctdtr;
static PyObject *__pyx_n_s_nctdtridf;
static PyObject *__pyx_n_s_nctdtrinc;
static PyObject *__pyx_n_s_nctdtrit;
static PyObject *__pyx_kp_u_ndarray_is_not_C_contiguous;
static PyObject *__pyx_kp_u_ndarray_is_not_Fortran_contiguou;
static PyObject *__pyx_n_s_ndtr;
static PyObject *__pyx_n_s_ndtri;
static PyObject *__pyx_n_s_newkwargs;
static PyObject *__pyx_n_u_no_result;
static PyObject *__pyx_n_s_np;
static PyObject *__pyx_n_s_nrdtrimn;
static PyObject *__pyx_n_s_nrdtrisd;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_kp_u_numpy_core_multiarray_failed_to;
static PyObject *__pyx_kp_u_numpy_core_umath_failed_to_impor;
static PyObject *__pyx_n_s_object;
static PyObject *__pyx_n_s_obl_ang1;
static PyObject *__pyx_n_s_obl_ang1_cv;
static PyObject *__pyx_n_s_obl_cv;
static PyObject *__pyx_n_s_obl_rad1;
static PyObject *__pyx_n_s_obl_rad1_cv;
static PyObject *__pyx_n_s_obl_rad2;
static PyObject *__pyx_n_s_obl_rad2_cv;
static PyObject *__pyx_n_s_olderr;
static PyObject *__pyx_n_s_oldstate;
static PyObject *__pyx_n_u_other;
static PyObject *__pyx_n_u_overflow;
static PyObject *__pyx_n_s_owens_t;
static PyObject *__pyx_n_s_pbdv;
static PyObject *__pyx_n_s_pbvv;
static PyObject *__pyx_n_s_pbwa;
static PyObject *__pyx_n_s_pdtr;
static PyObject *__pyx_n_s_pdtrc;
static PyObject *__pyx_n_s_pdtri;
static PyObject *__pyx_n_s_pdtrik;
static PyObject *__pyx_n_s_poch;
static PyObject *__pyx_n_s_pop;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_n_s_pro_ang1;
static PyObject *__pyx_n_s_pro_ang1_cv;
static PyObject *__pyx_n_s_pro_cv;
static PyObject *__pyx_n_s_pro_rad1;
static PyObject *__pyx_n_s_pro_rad1_cv;
static PyObject *__pyx_n_s_pro_rad2;
static PyObject *__pyx_n_s_pro_rad2_cv;
static PyObject *__pyx_n_s_pseudo_huber;
static PyObject *__pyx_n_s_psi;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_n_s_radian;
static PyObject *__pyx_n_u_raise;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_rel_entr;
static PyObject *__pyx_n_s_rgamma;
static PyObject *__pyx_n_s_round;
static PyObject *__pyx_n_s_scipy_special__ufuncs;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_n_s_seterr;
static PyObject *__pyx_kp_u_seterr_line_77;
static PyObject *__pyx_n_s_sf_error_action_map;
static PyObject *__pyx_n_s_sf_error_code_map;
static PyObject *__pyx_n_s_sf_error_test_function;
static PyObject *__pyx_n_s_shichi;
static PyObject *__pyx_n_s_sici;
static PyObject *__pyx_n_s_sindg;
static PyObject *__pyx_n_u_singular;
static PyObject *__pyx_n_s_sinpi;
static PyObject *__pyx_n_u_slow;
static PyObject *__pyx_n_s_smirnov;
static PyObject *__pyx_n_s_smirnovc;
static PyObject *__pyx_n_s_smirnovci;
static PyObject *__pyx_n_s_smirnovi;
static PyObject *__pyx_n_s_smirnovp;
static PyObject *__pyx_n_s_spence;
static PyObject *__pyx_n_s_sph_harm;
static PyObject *__pyx_n_s_spherical_in;
static PyObject *__pyx_n_s_spherical_in_d;
static PyObject *__pyx_n_s_spherical_jn;
static PyObject *__pyx_n_s_spherical_jn_d;
static PyObject *__pyx_n_s_spherical_kn;
static PyObject *__pyx_n_s_spherical_kn_d;
static PyObject *__pyx_n_s_spherical_yn;
static PyObject *__pyx_n_s_spherical_yn_d;
static PyObject *__pyx_n_s_stdtr;
static PyObject *__pyx_n_s_stdtridf;
static PyObject *__pyx_n_s_stdtrit;
static PyObject *__pyx_n_s_struve;
static PyObject *__pyx_n_s_struve_asymp_large_z;
static PyObject *__pyx_n_s_struve_bessel_series;
static PyObject *__pyx_n_s_struve_power_series;
static PyObject *__pyx_n_s_tandg;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_tklmbda;
static PyObject *__pyx_n_s_traceback;
static PyObject *__pyx_kp_s_ufuncs_extra_code_pxi;
static PyObject *__pyx_n_u_underflow;
static PyObject *__pyx_kp_u_unknown_dtype_code_in_numpy_pxd;
static PyObject *__pyx_n_s_value;
static PyObject *__pyx_n_u_warn;
static PyObject *__pyx_n_s_wofz;
static PyObject *__pyx_n_s_wrightomega;
static PyObject *__pyx_n_s_xlog1py;
static PyObject *__pyx_n_s_xlogy;
static PyObject *__pyx_n_s_y0;
static PyObject *__pyx_n_s_y1;
static PyObject *__pyx_n_s_yn;
static PyObject *__pyx_n_s_yv;
static PyObject *__pyx_n_s_yve;
static PyObject *__pyx_n_s_zeta;
static PyObject *__pyx_n_s_zetac;
static PyObject *__pyx_pf_5scipy_7special_7_ufuncs_geterr(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_5scipy_7special_7_ufuncs_2seterr(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_5scipy_7special_7_ufuncs_8errstate___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_5scipy_7special_7_ufuncs_8errstate_2__enter__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5scipy_7special_7_ufuncs_8errstate_4__exit__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_exc_type, CYTHON_UNUSED PyObject *__pyx_v_exc_value, CYTHON_UNUSED PyObject *__pyx_v_traceback); /* proto */
static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info); /* proto */
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_keys = {0, &__pyx_n_s_keys, 0, 0, 0};
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_pop = {0, &__pyx_n_s_pop, 0, 0, 0};
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_3;
static PyObject *__pyx_int_4;
static PyObject *__pyx_int_5;
static PyObject *__pyx_int_6;
static PyObject *__pyx_int_7;
static PyObject *__pyx_int_8;
static PyObject *__pyx_int_9;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_codeobj__9;
static PyObject *__pyx_codeobj__11;
static PyObject *__pyx_codeobj__14;
static PyObject *__pyx_codeobj__16;
static PyObject *__pyx_codeobj__18;
/* Late includes */

/* "_ufuncs_extra_code_common.pxi":23
 *     int PyUFunc_getfperr() nogil
 * 
 * cdef public int wrap_PyUFunc_getfperr() nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Call PyUFunc_getfperr in a context where PyUFunc_API array is initialized;
 */

int wrap_PyUFunc_getfperr(void) {
  int __pyx_r;

  /* "_ufuncs_extra_code_common.pxi":28
 *     this avoids messing with the UNIQUE_SYMBOL #defines
 *     """
 *     return PyUFunc_getfperr()             # <<<<<<<<<<<<<<
 * 
 * cimport libc
 */
  __pyx_r = PyUFunc_getfperr();
  goto __pyx_L0;

  /* "_ufuncs_extra_code_common.pxi":23
 *     int PyUFunc_getfperr() nogil
 * 
 * cdef public int wrap_PyUFunc_getfperr() nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Call PyUFunc_getfperr in a context where PyUFunc_API array is initialized;
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_ufuncs_extra_code_common.pxi":37
 * np.import_ufunc()
 * 
 * cdef void _set_action(sf_error.sf_error_t code,             # <<<<<<<<<<<<<<
 *                       sf_error.sf_action_t action) nogil:
 *     sf_error.set_action(code, action)
 */

static void __pyx_f_5scipy_7special_7_ufuncs__set_action(sf_error_t __pyx_v_code, sf_action_t __pyx_v_action) {

  /* "_ufuncs_extra_code_common.pxi":39
 * cdef void _set_action(sf_error.sf_error_t code,
 *                       sf_error.sf_action_t action) nogil:
 *     sf_error.set_action(code, action)             # <<<<<<<<<<<<<<
 */
  sf_error_set_action(__pyx_v_code, __pyx_v_action);

  /* "_ufuncs_extra_code_common.pxi":37
 * np.import_ufunc()
 * 
 * cdef void _set_action(sf_error.sf_error_t code,             # <<<<<<<<<<<<<<
 *                       sf_error.sf_action_t action) nogil:
 *     sf_error.set_action(code, action)
 */

  /* function exit code */
}

/* "_ufuncs_extra_code.pxi":28
 * 
 * 
 * def geterr():             # <<<<<<<<<<<<<<
 *     """Get the current way of handling special-function errors.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5scipy_7special_7_ufuncs_1geterr(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5scipy_7special_7_ufuncs_geterr[] = "Get the current way of handling special-function errors.\n\n    Returns\n    -------\n    err : dict\n        A dictionary with keys \"singular\", \"underflow\", \"overflow\",\n        \"slow\", \"loss\", \"no_result\", \"domain\", \"arg\", and \"other\",\n        whose values are from the strings \"ignore\", \"warn\", and\n        \"raise\". The keys represent possible special-function errors,\n        and the values define how these errors are handled.\n\n    See Also\n    --------\n    seterr : set how special-function errors are handled\n    errstate : context manager for special-function error handling\n    numpy.geterr : similar numpy function for floating-point errors\n\n    Notes\n    -----\n    For complete documentation of the types of special-function errors\n    and treatment options, see `seterr`.\n\n    Examples\n    --------\n    By default all errors are ignored.\n\n    >>> import scipy.special as sc\n    >>> for key, value in sorted(sc.geterr().items()):\n    ...     print(\"{}: {}\".format(key, value))\n    ...\n    arg: ignore\n    domain: ignore\n    loss: ignore\n    no_result: ignore\n    other: ignore\n    overflow: ignore\n    singular: ignore\n    slow: ignore\n    underflow: ignore\n\n    ";
static PyMethodDef __pyx_mdef_5scipy_7special_7_ufuncs_1geterr = {"geterr", (PyCFunction)__pyx_pw_5scipy_7special_7_ufuncs_1geterr, METH_NOARGS, __pyx_doc_5scipy_7special_7_ufuncs_geterr};
static PyObject *__pyx_pw_5scipy_7special_7_ufuncs_1geterr(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("geterr (wrapper)", 0);
  __pyx_r = __pyx_pf_5scipy_7special_7_ufuncs_geterr(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5scipy_7special_7_ufuncs_geterr(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_v_err = NULL;
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_v_code = NULL;
  sf_action_t __pyx_v_action;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  sf_error_t __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("geterr", 0);

  /* "_ufuncs_extra_code.pxi":70
 * 
 *     """
 *     err = {}             # <<<<<<<<<<<<<<
 *     for key, code in _sf_error_code_map.items():
 *         action = sf_error.get_action(code)
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_err = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "_ufuncs_extra_code.pxi":71
 *     """
 *     err = {}
 *     for key, code in _sf_error_code_map.items():             # <<<<<<<<<<<<<<
 *         action = sf_error.get_action(code)
 *         err[key] = _sf_error_action_map[action]
 */
  __pyx_t_2 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_sf_error_code_map); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (unlikely(__pyx_t_5 == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
    __PYX_ERR(0, 71, __pyx_L1_error)
  }
  __pyx_t_6 = __Pyx_dict_iterator(__pyx_t_5, 0, __pyx_n_s_items, (&__pyx_t_3), (&__pyx_t_4)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_XDECREF(__pyx_t_1);
  __pyx_t_1 = __pyx_t_6;
  __pyx_t_6 = 0;
  while (1) {
    __pyx_t_7 = __Pyx_dict_iter_next(__pyx_t_1, __pyx_t_3, &__pyx_t_2, &__pyx_t_6, &__pyx_t_5, NULL, __pyx_t_4);
    if (unlikely(__pyx_t_7 == 0)) break;
    if (unlikely(__pyx_t_7 == -1)) __PYX_ERR(0, 71, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_6);
    __pyx_t_6 = 0;
    __Pyx_XDECREF_SET(__pyx_v_code, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "_ufuncs_extra_code.pxi":72
 *     err = {}
 *     for key, code in _sf_error_code_map.items():
 *         action = sf_error.get_action(code)             # <<<<<<<<<<<<<<
 *         err[key] = _sf_error_action_map[action]
 *     return err
 */
    __pyx_t_8 = ((sf_error_t)__Pyx_PyInt_As_sf_error_t(__pyx_v_code)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 72, __pyx_L1_error)
    __pyx_v_action = sf_error_get_action(__pyx_t_8);

    /* "_ufuncs_extra_code.pxi":73
 *     for key, code in _sf_error_code_map.items():
 *         action = sf_error.get_action(code)
 *         err[key] = _sf_error_action_map[action]             # <<<<<<<<<<<<<<
 *     return err
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_sf_error_action_map); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 73, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyInt_From_sf_action_t(__pyx_v_action); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 73, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_9 = __Pyx_PyObject_GetItem(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 73, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(PyDict_SetItem(__pyx_v_err, __pyx_v_key, __pyx_t_9) < 0)) __PYX_ERR(0, 73, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "_ufuncs_extra_code.pxi":74
 *         action = sf_error.get_action(code)
 *         err[key] = _sf_error_action_map[action]
 *     return err             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_err);
  __pyx_r = __pyx_v_err;
  goto __pyx_L0;

  /* "_ufuncs_extra_code.pxi":28
 * 
 * 
 * def geterr():             # <<<<<<<<<<<<<<
 *     """Get the current way of handling special-function errors.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("scipy.special._ufuncs.geterr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_err);
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF(__pyx_v_code);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_ufuncs_extra_code.pxi":77
 * 
 * 
 * def seterr(**kwargs):             # <<<<<<<<<<<<<<
 *     """Set how special-function errors are handled.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5scipy_7special_7_ufuncs_3seterr(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5scipy_7special_7_ufuncs_2seterr[] = "Set how special-function errors are handled.\n\n    Parameters\n    ----------\n    all : {'ignore', 'warn' 'raise'}, optional\n        Set treatment for all type of special-function errors at\n        once. The options are:\n\n        - 'ignore' Take no action when the error occurs\n        - 'warn' Print a `SpecialFunctionWarning` when the error\n          occurs (via the Python `warnings` module)\n        - 'raise' Raise a `SpecialFunctionError` when the error\n          occurs.\n\n        The default is to not change the current behavior. If\n        behaviors for additional categories of special-function errors\n        are specified, then ``all`` is applied first, followed by the\n        additional categories.\n    singular : {'ignore', 'warn', 'raise'}, optional\n        Treatment for singularities.\n    underflow : {'ignore', 'warn', 'raise'}, optional\n        Treatment for underflow.\n    overflow : {'ignore', 'warn', 'raise'}, optional\n        Treatment for overflow.\n    slow : {'ignore', 'warn', 'raise'}, optional\n        Treatment for slow convergence.\n    loss : {'ignore', 'warn', 'raise'}, optional\n        Treatment for loss of accuracy.\n    no_result : {'ignore', 'warn', 'raise'}, optional\n        Treatment for failing to find a result.\n    domain : {'ignore', 'warn', 'raise'}, optional\n        Treatment for an invalid argument to a function.\n    arg : {'ignore', 'warn', 'raise'}, optional\n        Treatment for an invalid parameter to a function.\n    other : {'ignore', 'warn', 'raise'}, optional\n        Treatment for an unknown error.\n\n    Returns\n    -------\n    olderr : dict\n        Dictionary containing the old settings.\n\n    See Also\n    --------\n    geterr : get the current way of handling special-function errors\n    errstate : context manager for special-function error handling\n    numpy.seterr : similar numpy function for floating-point errors\n\n    Examples\n    --------\n    >>> import scipy.special as sc\n    >>> f""rom pytest import raises\n    >>> sc.gammaln(0)\n    inf\n    >>> olderr = sc.seterr(singular='raise')\n    >>> with raises(sc.SpecialFunctionError):\n    ...     sc.gammaln(0)\n    ...\n    >>> _ = sc.seterr(**olderr)\n\n    We can also raise for every category except one.\n\n    >>> olderr = sc.seterr(all='raise', singular='ignore')\n    >>> sc.gammaln(0)\n    inf\n    >>> with raises(sc.SpecialFunctionError):\n    ...     sc.spence(-1)\n    ...\n    >>> _ = sc.seterr(**olderr)\n\n    ";
static PyMethodDef __pyx_mdef_5scipy_7special_7_ufuncs_3seterr = {"seterr", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5scipy_7special_7_ufuncs_3seterr, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5scipy_7special_7_ufuncs_2seterr};
static PyObject *__pyx_pw_5scipy_7special_7_ufuncs_3seterr(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_kwargs = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("seterr (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("seterr", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return NULL;}
  if (__pyx_kwds && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "seterr", 1))) return NULL;
  __pyx_v_kwargs = (__pyx_kwds) ? PyDict_Copy(__pyx_kwds) : PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return NULL;
  __Pyx_GOTREF(__pyx_v_kwargs);
  __pyx_r = __pyx_pf_5scipy_7special_7_ufuncs_2seterr(__pyx_self, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5scipy_7special_7_ufuncs_2seterr(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_kwargs) {
  PyObject *__pyx_v_olderr = NULL;
  PyObject *__pyx_v_action = NULL;
  PyObject *__pyx_v_newkwargs = NULL;
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_v_error = NULL;
  PyObject *__pyx_v_code = NULL;
  PyObject *__pyx_7genexpr__pyx_v_key = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  sf_error_t __pyx_t_11;
  sf_action_t __pyx_t_12;
  __Pyx_RefNannySetupContext("seterr", 0);
  __Pyx_INCREF(__pyx_v_kwargs);

  /* "_ufuncs_extra_code.pxi":149
 * 
 *     """
 *     olderr = geterr()             # <<<<<<<<<<<<<<
 * 
 *     if 'all' in kwargs.keys():
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_geterr); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_olderr = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "_ufuncs_extra_code.pxi":151
 *     olderr = geterr()
 * 
 *     if 'all' in kwargs.keys():             # <<<<<<<<<<<<<<
 *         action = kwargs.pop('all')
 *         newkwargs = {key: action for key in _sf_error_code_map.keys()}
 */
  __pyx_t_1 = __Pyx_PyDict_Keys(__pyx_v_kwargs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = (__Pyx_PySequence_ContainsTF(__pyx_n_u_all, __pyx_t_1, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 151, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "_ufuncs_extra_code.pxi":152
 * 
 *     if 'all' in kwargs.keys():
 *         action = kwargs.pop('all')             # <<<<<<<<<<<<<<
 *         newkwargs = {key: action for key in _sf_error_code_map.keys()}
 *         for key, value in kwargs.items():
 */
    __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_kwargs, __pyx_n_u_all, ((PyObject *)NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 152, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_action = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "_ufuncs_extra_code.pxi":153
 *     if 'all' in kwargs.keys():
 *         action = kwargs.pop('all')
 *         newkwargs = {key: action for key in _sf_error_code_map.keys()}             # <<<<<<<<<<<<<<
 *         for key, value in kwargs.items():
 *             newkwargs[key] = value
 */
    { /* enter inner scope */
      __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 153, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = 0;
      __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_sf_error_code_map); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 153, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (unlikely(__pyx_t_3 == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "keys");
        __PYX_ERR(0, 153, __pyx_L6_error)
      }
      __pyx_t_9 = __Pyx_dict_iterator(__pyx_t_3, 0, __pyx_n_s_keys, (&__pyx_t_7), (&__pyx_t_8)); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 153, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_2);
      __pyx_t_2 = __pyx_t_9;
      __pyx_t_9 = 0;
      while (1) {
        __pyx_t_10 = __Pyx_dict_iter_next(__pyx_t_2, __pyx_t_7, &__pyx_t_6, &__pyx_t_9, NULL, NULL, __pyx_t_8);
        if (unlikely(__pyx_t_10 == 0)) break;
        if (unlikely(__pyx_t_10 == -1)) __PYX_ERR(0, 153, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_XDECREF_SET(__pyx_7genexpr__pyx_v_key, __pyx_t_9);
        __pyx_t_9 = 0;
        if (unlikely(PyDict_SetItem(__pyx_t_1, (PyObject*)__pyx_7genexpr__pyx_v_key, (PyObject*)__pyx_v_action))) __PYX_ERR(0, 153, __pyx_L6_error)
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_7genexpr__pyx_v_key); __pyx_7genexpr__pyx_v_key = 0;
      goto __pyx_L9_exit_scope;
      __pyx_L6_error:;
      __Pyx_XDECREF(__pyx_7genexpr__pyx_v_key); __pyx_7genexpr__pyx_v_key = 0;
      goto __pyx_L1_error;
      __pyx_L9_exit_scope:;
    } /* exit inner scope */
    __pyx_v_newkwargs = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "_ufuncs_extra_code.pxi":154
 *         action = kwargs.pop('all')
 *         newkwargs = {key: action for key in _sf_error_code_map.keys()}
 *         for key, value in kwargs.items():             # <<<<<<<<<<<<<<
 *             newkwargs[key] = value
 *         kwargs = newkwargs
 */
    __pyx_t_7 = 0;
    __pyx_t_2 = __Pyx_dict_iterator(__pyx_v_kwargs, 1, __pyx_n_s_items, (&__pyx_t_6), (&__pyx_t_8)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 154, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF(__pyx_t_1);
    __pyx_t_1 = __pyx_t_2;
    __pyx_t_2 = 0;
    while (1) {
      __pyx_t_10 = __Pyx_dict_iter_next(__pyx_t_1, __pyx_t_6, &__pyx_t_7, &__pyx_t_2, &__pyx_t_9, NULL, __pyx_t_8);
      if (unlikely(__pyx_t_10 == 0)) break;
      if (unlikely(__pyx_t_10 == -1)) __PYX_ERR(0, 154, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_9);
      __pyx_t_9 = 0;

      /* "_ufuncs_extra_code.pxi":155
 *         newkwargs = {key: action for key in _sf_error_code_map.keys()}
 *         for key, value in kwargs.items():
 *             newkwargs[key] = value             # <<<<<<<<<<<<<<
 *         kwargs = newkwargs
 * 
 */
      if (unlikely(PyDict_SetItem(__pyx_v_newkwargs, __pyx_v_key, __pyx_v_value) < 0)) __PYX_ERR(0, 155, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "_ufuncs_extra_code.pxi":156
 *         for key, value in kwargs.items():
 *             newkwargs[key] = value
 *         kwargs = newkwargs             # <<<<<<<<<<<<<<
 * 
 *     for error, action in kwargs.items():
 */
    __Pyx_INCREF(__pyx_v_newkwargs);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_v_newkwargs);

    /* "_ufuncs_extra_code.pxi":151
 *     olderr = geterr()
 * 
 *     if 'all' in kwargs.keys():             # <<<<<<<<<<<<<<
 *         action = kwargs.pop('all')
 *         newkwargs = {key: action for key in _sf_error_code_map.keys()}
 */
  }

  /* "_ufuncs_extra_code.pxi":158
 *         kwargs = newkwargs
 * 
 *     for error, action in kwargs.items():             # <<<<<<<<<<<<<<
 *         action = _sf_error_action_map[action]
 *         code = _sf_error_code_map[error]
 */
  __pyx_t_6 = 0;
  __pyx_t_9 = __Pyx_dict_iterator(__pyx_v_kwargs, 1, __pyx_n_s_items, (&__pyx_t_7), (&__pyx_t_8)); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_1);
  __pyx_t_1 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_10 = __Pyx_dict_iter_next(__pyx_t_1, __pyx_t_7, &__pyx_t_6, &__pyx_t_9, &__pyx_t_2, NULL, __pyx_t_8);
    if (unlikely(__pyx_t_10 == 0)) break;
    if (unlikely(__pyx_t_10 == -1)) __PYX_ERR(0, 158, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_error, __pyx_t_9);
    __pyx_t_9 = 0;
    __Pyx_XDECREF_SET(__pyx_v_action, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "_ufuncs_extra_code.pxi":159
 * 
 *     for error, action in kwargs.items():
 *         action = _sf_error_action_map[action]             # <<<<<<<<<<<<<<
 *         code = _sf_error_code_map[error]
 *         _set_action(code, action)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_sf_error_action_map); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 159, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = __Pyx_PyObject_GetItem(__pyx_t_2, __pyx_v_action); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 159, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_action, __pyx_t_9);
    __pyx_t_9 = 0;

    /* "_ufuncs_extra_code.pxi":160
 *     for error, action in kwargs.items():
 *         action = _sf_error_action_map[action]
 *         code = _sf_error_code_map[error]             # <<<<<<<<<<<<<<
 *         _set_action(code, action)
 *         scipy.special._ufuncs_cxx._set_action(code, action)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_sf_error_code_map); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_t_9, __pyx_v_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_XDECREF_SET(__pyx_v_code, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "_ufuncs_extra_code.pxi":161
 *         action = _sf_error_action_map[action]
 *         code = _sf_error_code_map[error]
 *         _set_action(code, action)             # <<<<<<<<<<<<<<
 *         scipy.special._ufuncs_cxx._set_action(code, action)
 * 
 */
    __pyx_t_11 = ((sf_error_t)__Pyx_PyInt_As_sf_error_t(__pyx_v_code)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 161, __pyx_L1_error)
    __pyx_t_12 = ((sf_action_t)__Pyx_PyInt_As_sf_action_t(__pyx_v_action)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 161, __pyx_L1_error)
    __pyx_f_5scipy_7special_7_ufuncs__set_action(__pyx_t_11, __pyx_t_12);

    /* "_ufuncs_extra_code.pxi":162
 *         code = _sf_error_code_map[error]
 *         _set_action(code, action)
 *         scipy.special._ufuncs_cxx._set_action(code, action)             # <<<<<<<<<<<<<<
 * 
 *     return olderr
 */
    __pyx_t_11 = ((sf_error_t)__Pyx_PyInt_As_sf_error_t(__pyx_v_code)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 162, __pyx_L1_error)
    __pyx_t_12 = ((sf_action_t)__Pyx_PyInt_As_sf_action_t(__pyx_v_action)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 162, __pyx_L1_error)
    __pyx_f_5scipy_7special_11_ufuncs_cxx__set_action(__pyx_t_11, __pyx_t_12);
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "_ufuncs_extra_code.pxi":164
 *         scipy.special._ufuncs_cxx._set_action(code, action)
 * 
 *     return olderr             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_olderr);
  __pyx_r = __pyx_v_olderr;
  goto __pyx_L0;

  /* "_ufuncs_extra_code.pxi":77
 * 
 * 
 * def seterr(**kwargs):             # <<<<<<<<<<<<<<
 *     """Set how special-function errors are handled.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("scipy.special._ufuncs.seterr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XDECREF(__pyx_v_olderr);
  __Pyx_XDECREF(__pyx_v_action);
  __Pyx_XDECREF(__pyx_v_newkwargs);
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XDECREF(__pyx_v_error);
  __Pyx_XDECREF(__pyx_v_code);
  __Pyx_XDECREF(__pyx_7genexpr__pyx_v_key);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_ufuncs_extra_code.pxi":213
 * 
 *     """
 *     def __init__(self, **kwargs):             # <<<<<<<<<<<<<<
 *         self.kwargs = kwargs
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5scipy_7special_7_ufuncs_8errstate_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5scipy_7special_7_ufuncs_8errstate_1__init__ = {"__init__", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5scipy_7special_7_ufuncs_8errstate_1__init__, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5scipy_7special_7_ufuncs_8errstate_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_kwargs = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return NULL;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 213, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_self = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 213, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("scipy.special._ufuncs.errstate.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5scipy_7special_7_ufuncs_8errstate___init__(__pyx_self, __pyx_v_self, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5scipy_7special_7_ufuncs_8errstate___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_kwargs) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "_ufuncs_extra_code.pxi":214
 *     """
 *     def __init__(self, **kwargs):
 *         self.kwargs = kwargs             # <<<<<<<<<<<<<<
 * 
 *     def __enter__(self):
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_kwargs, __pyx_v_kwargs) < 0) __PYX_ERR(0, 214, __pyx_L1_error)

  /* "_ufuncs_extra_code.pxi":213
 * 
 *     """
 *     def __init__(self, **kwargs):             # <<<<<<<<<<<<<<
 *         self.kwargs = kwargs
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("scipy.special._ufuncs.errstate.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_ufuncs_extra_code.pxi":216
 *         self.kwargs = kwargs
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         self.oldstate = seterr(**self.kwargs)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5scipy_7special_7_ufuncs_8errstate_3__enter__(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_5scipy_7special_7_ufuncs_8errstate_3__enter__ = {"__enter__", (PyCFunction)__pyx_pw_5scipy_7special_7_ufuncs_8errstate_3__enter__, METH_O, 0};
static PyObject *__pyx_pw_5scipy_7special_7_ufuncs_8errstate_3__enter__(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__enter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5scipy_7special_7_ufuncs_8errstate_2__enter__(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5scipy_7special_7_ufuncs_8errstate_2__enter__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__enter__", 0);

  /* "_ufuncs_extra_code.pxi":217
 * 
 *     def __enter__(self):
 *         self.oldstate = seterr(**self.kwargs)             # <<<<<<<<<<<<<<
 * 
 *     def __exit__(self, exc_type, exc_value, traceback):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_seterr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 217, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_kwargs); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 217, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(__pyx_t_3 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "argument after ** must be a mapping, not NoneType");
    __PYX_ERR(0, 217, __pyx_L1_error)
  }
  if (likely(PyDict_CheckExact(__pyx_t_3))) {
    __pyx_t_2 = PyDict_Copy(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 217, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_2 = PyObject_CallFunctionObjArgs((PyObject*)&PyDict_Type, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 217, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 217, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_oldstate, __pyx_t_3) < 0) __PYX_ERR(0, 217, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "_ufuncs_extra_code.pxi":216
 *         self.kwargs = kwargs
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         self.oldstate = seterr(**self.kwargs)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("scipy.special._ufuncs.errstate.__enter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_ufuncs_extra_code.pxi":219
 *         self.oldstate = seterr(**self.kwargs)
 * 
 *     def __exit__(self, exc_type, exc_value, traceback):             # <<<<<<<<<<<<<<
 *         seterr(**self.oldstate)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5scipy_7special_7_ufuncs_8errstate_5__exit__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5scipy_7special_7_ufuncs_8errstate_5__exit__ = {"__exit__", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_5scipy_7special_7_ufuncs_8errstate_5__exit__, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5scipy_7special_7_ufuncs_8errstate_5__exit__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  CYTHON_UNUSED PyObject *__pyx_v_exc_type = 0;
  CYTHON_UNUSED PyObject *__pyx_v_exc_value = 0;
  CYTHON_UNUSED PyObject *__pyx_v_traceback = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__exit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_exc_type,&__pyx_n_s_exc_value,&__pyx_n_s_traceback,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_exc_type)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__exit__", 1, 4, 4, 1); __PYX_ERR(0, 219, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_exc_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__exit__", 1, 4, 4, 2); __PYX_ERR(0, 219, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_traceback)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__exit__", 1, 4, 4, 3); __PYX_ERR(0, 219, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__exit__") < 0)) __PYX_ERR(0, 219, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_self = values[0];
    __pyx_v_exc_type = values[1];
    __pyx_v_exc_value = values[2];
    __pyx_v_traceback = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__exit__", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 219, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("scipy.special._ufuncs.errstate.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5scipy_7special_7_ufuncs_8errstate_4__exit__(__pyx_self, __pyx_v_self, __pyx_v_exc_type, __pyx_v_exc_value, __pyx_v_traceback);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5scipy_7special_7_ufuncs_8errstate_4__exit__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_exc_type, CYTHON_UNUSED PyObject *__pyx_v_exc_value, CYTHON_UNUSED PyObject *__pyx_v_traceback) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__exit__", 0);

  /* "_ufuncs_extra_code.pxi":220
 * 
 *     def __exit__(self, exc_type, exc_value, traceback):
 *         seterr(**self.oldstate)             # <<<<<<<<<<<<<<
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_seterr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_oldstate); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(__pyx_t_3 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "argument after ** must be a mapping, not NoneType");
    __PYX_ERR(0, 220, __pyx_L1_error)
  }
  if (likely(PyDict_CheckExact(__pyx_t_3))) {
    __pyx_t_2 = PyDict_Copy(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 220, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_2 = PyObject_CallFunctionObjArgs((PyObject*)&PyDict_Type, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 220, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "_ufuncs_extra_code.pxi":219
 *         self.oldstate = seterr(**self.kwargs)
 * 
 *     def __exit__(self, exc_type, exc_value, traceback):             # <<<<<<<<<<<<<<
 *         seterr(**self.oldstate)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("scipy.special._ufuncs.errstate.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "scipy/special/_ufuncs.pyx":8
 * include "_ufuncs_extra_code.pxi"
 * 
 * cdef void loop_D_DD__As_DD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_DD__As_DD_D(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":9
 * 
 * cdef void loop_D_DD__As_DD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":10
 * cdef void loop_D_DD__As_DD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":11
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":12
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":13
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef double complex ov0
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":14
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":16
 *     cdef char *op0 = args[2]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double complex, double complex) nogil>func)(<double complex>(<double complex*>ip0)[0], <double complex>(<double complex*>ip1)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":17
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double complex, double complex) nogil>func)(<double complex>(<double complex*>ip0)[0], <double complex>(<double complex*>ip1)[0])             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(__pyx_t_double_complex, __pyx_t_double_complex))__pyx_v_func)(__pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_double_complex *)__pyx_v_ip0)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_double_complex *)__pyx_v_ip0)[0])))), __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_double_complex *)__pyx_v_ip1)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_double_complex *)__pyx_v_ip1)[0])))));

    /* "scipy/special/_ufuncs.pyx":18
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double complex, double complex) nogil>func)(<double complex>(<double complex*>ip0)[0], <double complex>(<double complex*>ip1)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":19
 *         ov0 = (<double complex(*)(double complex, double complex) nogil>func)(<double complex>(<double complex*>ip0)[0], <double complex>(<double complex*>ip1)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":20
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":21
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":22
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_DD__As_FF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":8
 * include "_ufuncs_extra_code.pxi"
 * 
 * cdef void loop_D_DD__As_DD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":24
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_DD__As_FF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_DD__As_FF_F(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":25
 * 
 * cdef void loop_D_DD__As_FF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":26
 * cdef void loop_D_DD__As_FF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":27
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":28
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":29
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef double complex ov0
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":30
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":32
 *     cdef char *op0 = args[2]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double complex, double complex) nogil>func)(<double complex>(<float complex*>ip0)[0], <double complex>(<float complex*>ip1)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":33
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double complex, double complex) nogil>func)(<double complex>(<float complex*>ip0)[0], <double complex>(<float complex*>ip1)[0])             # <<<<<<<<<<<<<<
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(__pyx_t_double_complex, __pyx_t_double_complex))__pyx_v_func)(__pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_float_complex *)__pyx_v_ip0)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_float_complex *)__pyx_v_ip0)[0])))), __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_float_complex *)__pyx_v_ip1)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_float_complex *)__pyx_v_ip1)[0])))));

    /* "scipy/special/_ufuncs.pyx":34
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double complex, double complex) nogil>func)(<double complex>(<float complex*>ip0)[0], <double complex>(<float complex*>ip1)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_float_complex *)__pyx_v_op0)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov0)), ((float)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":35
 *         ov0 = (<double complex(*)(double complex, double complex) nogil>func)(<double complex>(<float complex*>ip0)[0], <double complex>(<float complex*>ip1)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":36
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":37
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":38
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_D__As_D_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":24
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_DD__As_FF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":40
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_D__As_D_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":41
 * 
 * cdef void loop_D_D__As_D_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":42
 * cdef void loop_D_D__As_D_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":43
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":44
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef double complex ov0
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":45
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":47
 *     cdef char *op0 = args[1]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double complex) nogil>func)(<double complex>(<double complex*>ip0)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":48
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double complex) nogil>func)(<double complex>(<double complex*>ip0)[0])             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(__pyx_t_double_complex))__pyx_v_func)(__pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_double_complex *)__pyx_v_ip0)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_double_complex *)__pyx_v_ip0)[0])))));

    /* "scipy/special/_ufuncs.pyx":49
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double complex) nogil>func)(<double complex>(<double complex*>ip0)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":50
 *         ov0 = (<double complex(*)(double complex) nogil>func)(<double complex>(<double complex*>ip0)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":51
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));
  }

  /* "scipy/special/_ufuncs.pyx":52
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_D__As_F_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":40
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_D__As_D_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":54
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_D__As_F_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":55
 * 
 * cdef void loop_D_D__As_F_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":56
 * cdef void loop_D_D__As_F_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":57
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":58
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef double complex ov0
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":59
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":61
 *     cdef char *op0 = args[1]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double complex) nogil>func)(<double complex>(<float complex*>ip0)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":62
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double complex) nogil>func)(<double complex>(<float complex*>ip0)[0])             # <<<<<<<<<<<<<<
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(__pyx_t_double_complex))__pyx_v_func)(__pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_float_complex *)__pyx_v_ip0)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_float_complex *)__pyx_v_ip0)[0])))));

    /* "scipy/special/_ufuncs.pyx":63
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double complex) nogil>func)(<double complex>(<float complex*>ip0)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((__pyx_t_float_complex *)__pyx_v_op0)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov0)), ((float)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":64
 *         ov0 = (<double complex(*)(double complex) nogil>func)(<double complex>(<float complex*>ip0)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":65
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));
  }

  /* "scipy/special/_ufuncs.pyx":66
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_Dld__As_Dld_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":54
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_D__As_F_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":68
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_Dld__As_Dld_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_Dld__As_Dld_D(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":69
 * 
 * cdef void loop_D_Dld__As_Dld_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":70
 * cdef void loop_D_Dld__As_Dld_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":71
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":72
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":73
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":74
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[3]
 *     cdef double complex ov0
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":75
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":77
 *     cdef char *op0 = args[3]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double complex, long, double) nogil>func)(<double complex>(<double complex*>ip0)[0], <long>(<long*>ip1)[0], <double>(<double*>ip2)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":78
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double complex, long, double) nogil>func)(<double complex>(<double complex*>ip0)[0], <long>(<long*>ip1)[0], <double>(<double*>ip2)[0])             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(__pyx_t_double_complex, long, double))__pyx_v_func)(__pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_double_complex *)__pyx_v_ip0)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_double_complex *)__pyx_v_ip0)[0])))), ((long)(((long *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])));

    /* "scipy/special/_ufuncs.pyx":79
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double complex, long, double) nogil>func)(<double complex>(<double complex*>ip0)[0], <long>(<long*>ip1)[0], <double>(<double*>ip2)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":80
 *         ov0 = (<double complex(*)(double complex, long, double) nogil>func)(<double complex>(<double complex*>ip0)[0], <long>(<long*>ip1)[0], <double>(<double*>ip2)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":81
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         op0 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":82
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":83
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         op0 += steps[3]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[3]));
  }

  /* "scipy/special/_ufuncs.pyx":84
 *         ip2 += steps[2]
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_dD__As_dD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":68
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_Dld__As_Dld_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":86
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dD__As_dD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":87
 * 
 * cdef void loop_D_dD__As_dD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":88
 * cdef void loop_D_dD__As_dD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":89
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":90
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":91
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef double complex ov0
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":92
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":94
 *     cdef char *op0 = args[2]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double complex>(<double complex*>ip1)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":95
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double complex>(<double complex*>ip1)[0])             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(double, __pyx_t_double_complex))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_double_complex *)__pyx_v_ip1)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_double_complex *)__pyx_v_ip1)[0])))));

    /* "scipy/special/_ufuncs.pyx":96
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double complex>(<double complex*>ip1)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":97
 *         ov0 = (<double complex(*)(double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double complex>(<double complex*>ip1)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":98
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":99
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":100
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_dD__As_fF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":86
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dD__As_dD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":102
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dD__As_fF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":103
 * 
 * cdef void loop_D_dD__As_fF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":104
 * cdef void loop_D_dD__As_fF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":105
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":106
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":107
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef double complex ov0
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":108
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":110
 *     cdef char *op0 = args[2]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double complex>(<float complex*>ip1)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":111
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double complex>(<float complex*>ip1)[0])             # <<<<<<<<<<<<<<
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(double, __pyx_t_double_complex))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_float_complex *)__pyx_v_ip1)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_float_complex *)__pyx_v_ip1)[0])))));

    /* "scipy/special/_ufuncs.pyx":112
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double complex>(<float complex*>ip1)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_float_complex *)__pyx_v_op0)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov0)), ((float)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":113
 *         ov0 = (<double complex(*)(double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double complex>(<float complex*>ip1)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":114
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":115
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":116
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_ddD__As_ddD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":102
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dD__As_fF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":118
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_ddD__As_ddD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_ddD__As_ddD_D(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":119
 * 
 * cdef void loop_D_ddD__As_ddD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":120
 * cdef void loop_D_ddD__As_ddD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":121
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":122
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":123
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":124
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[3]
 *     cdef double complex ov0
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":125
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":127
 *     cdef char *op0 = args[3]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double, double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double complex>(<double complex*>ip2)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":128
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double complex>(<double complex*>ip2)[0])             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(double, double, __pyx_t_double_complex))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_double_complex *)__pyx_v_ip2)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_double_complex *)__pyx_v_ip2)[0])))));

    /* "scipy/special/_ufuncs.pyx":129
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double complex>(<double complex*>ip2)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":130
 *         ov0 = (<double complex(*)(double, double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double complex>(<double complex*>ip2)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":131
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         op0 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":132
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":133
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         op0 += steps[3]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[3]));
  }

  /* "scipy/special/_ufuncs.pyx":134
 *         ip2 += steps[2]
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_ddD__As_ffF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":118
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_ddD__As_ddD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":136
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_ddD__As_ffF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_ddD__As_ffF_F(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":137
 * 
 * cdef void loop_D_ddD__As_ffF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":138
 * cdef void loop_D_ddD__As_ffF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":139
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":140
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":141
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":142
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[3]
 *     cdef double complex ov0
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":143
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":145
 *     cdef char *op0 = args[3]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double, double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double complex>(<float complex*>ip2)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":146
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double complex>(<float complex*>ip2)[0])             # <<<<<<<<<<<<<<
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(double, double, __pyx_t_double_complex))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_float_complex *)__pyx_v_ip2)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_float_complex *)__pyx_v_ip2)[0])))));

    /* "scipy/special/_ufuncs.pyx":147
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double complex>(<float complex*>ip2)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_float_complex *)__pyx_v_op0)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov0)), ((float)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":148
 *         ov0 = (<double complex(*)(double, double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double complex>(<float complex*>ip2)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":149
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         op0 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":150
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":151
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         op0 += steps[3]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[3]));
  }

  /* "scipy/special/_ufuncs.pyx":152
 *         ip2 += steps[2]
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_dddD__As_dddD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":136
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_ddD__As_ffF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":154
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dddD__As_dddD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dddD__As_dddD_D(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":155
 * 
 * cdef void loop_D_dddD__As_dddD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":156
 * cdef void loop_D_dddD__As_dddD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":157
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":158
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":159
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":160
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":161
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef double complex ov0
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":162
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":164
 *     cdef char *op0 = args[4]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double, double, double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double complex>(<double complex*>ip3)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":165
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double complex>(<double complex*>ip3)[0])             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(double, double, double, __pyx_t_double_complex))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])), __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_double_complex *)__pyx_v_ip3)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_double_complex *)__pyx_v_ip3)[0])))));

    /* "scipy/special/_ufuncs.pyx":166
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double complex>(<double complex*>ip3)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":167
 *         ov0 = (<double complex(*)(double, double, double, double complex) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double complex>(<double complex*>ip3)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":168
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":169
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":170
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":171
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":172
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_dddD__As_fffF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":154
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dddD__As_dddD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":174
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dddD__As_fffF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dddD__As_fffF_F(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":175
 * 
 * cdef void loop_D_dddD__As_fffF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":176
 * cdef void loop_D_dddD__As_fffF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":177
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":178
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":179
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":180
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":181
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef double complex ov0
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":182
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":184
 *     cdef char *op0 = args[4]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double, double, double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double complex>(<float complex*>ip3)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":185
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double complex>(<float complex*>ip3)[0])             # <<<<<<<<<<<<<<
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(double, double, double, __pyx_t_double_complex))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), ((double)(((float *)__pyx_v_ip2)[0])), __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_float_complex *)__pyx_v_ip3)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_float_complex *)__pyx_v_ip3)[0])))));

    /* "scipy/special/_ufuncs.pyx":186
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double complex>(<float complex*>ip3)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_float_complex *)__pyx_v_op0)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov0)), ((float)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":187
 *         ov0 = (<double complex(*)(double, double, double, double complex) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double complex>(<float complex*>ip3)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":188
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":189
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":190
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":191
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":192
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_dddd__As_dddd_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":174
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dddD__As_fffF_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":194
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dddd__As_dddd_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dddd__As_dddd_D(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":195
 * 
 * cdef void loop_D_dddd__As_dddd_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":196
 * cdef void loop_D_dddd__As_dddd_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":197
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":198
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":199
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":200
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":201
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef double complex ov0
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":202
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":204
 *     cdef char *op0 = args[4]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":205
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(double, double, double, double))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])), ((double)(((double *)__pyx_v_ip3)[0])));

    /* "scipy/special/_ufuncs.pyx":206
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":207
 *         ov0 = (<double complex(*)(double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":208
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":209
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":210
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":211
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":212
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_dddd__As_ffff_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":194
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dddd__As_dddd_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":214
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dddd__As_ffff_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_dddd__As_ffff_F(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":215
 * 
 * cdef void loop_D_dddd__As_ffff_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":216
 * cdef void loop_D_dddd__As_ffff_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":217
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":218
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":219
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":220
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":221
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef double complex ov0
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":222
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":224
 *     cdef char *op0 = args[4]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":225
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0])             # <<<<<<<<<<<<<<
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(double, double, double, double))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), ((double)(((float *)__pyx_v_ip2)[0])), ((double)(((float *)__pyx_v_ip3)[0])));

    /* "scipy/special/_ufuncs.pyx":226
 *     for i in range(n):
 *         ov0 = (<double complex(*)(double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_float_complex *)__pyx_v_op0)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov0)), ((float)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":227
 *         ov0 = (<double complex(*)(double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0])
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":228
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":229
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":230
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":231
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":232
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_iidd__As_lldd_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":214
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_dddd__As_ffff_F(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":234
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_iidd__As_lldd_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_iidd__As_lldd_D(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;

  /* "scipy/special/_ufuncs.pyx":235
 * 
 * cdef void loop_D_iidd__As_lldd_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":236
 * cdef void loop_D_iidd__As_lldd_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":237
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":238
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":239
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":240
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":241
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef double complex ov0
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":242
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":244
 *     cdef char *op0 = args[4]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0] and <int>(<long*>ip1)[0] == (<long*>ip1)[0]:
 *             ov0 = (<double complex(*)(int, int, double, double) nogil>func)(<int>(<long*>ip0)[0], <int>(<long*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":245
 *     cdef double complex ov0
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0] and <int>(<long*>ip1)[0] == (<long*>ip1)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double complex(*)(int, int, double, double) nogil>func)(<int>(<long*>ip0)[0], <int>(<long*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         else:
 */
    __pyx_t_5 = ((((int)(((long *)__pyx_v_ip0)[0])) == (((long *)__pyx_v_ip0)[0])) != 0);
    if (__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_5 = ((((int)(((long *)__pyx_v_ip1)[0])) == (((long *)__pyx_v_ip1)[0])) != 0);
    __pyx_t_4 = __pyx_t_5;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_4) {

      /* "scipy/special/_ufuncs.pyx":246
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0] and <int>(<long*>ip1)[0] == (<long*>ip1)[0]:
 *             ov0 = (<double complex(*)(int, int, double, double) nogil>func)(<int>(<long*>ip0)[0], <int>(<long*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])             # <<<<<<<<<<<<<<
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 */
      __pyx_v_ov0 = ((__pyx_t_double_complex (*)(int, int, double, double))__pyx_v_func)(((int)(((long *)__pyx_v_ip0)[0])), ((int)(((long *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])), ((double)(((double *)__pyx_v_ip3)[0])));

      /* "scipy/special/_ufuncs.pyx":245
 *     cdef double complex ov0
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0] and <int>(<long*>ip1)[0] == (<long*>ip1)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double complex(*)(int, int, double, double) nogil>func)(<int>(<long*>ip0)[0], <int>(<long*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         else:
 */
      goto __pyx_L5;
    }

    /* "scipy/special/_ufuncs.pyx":248
 *             ov0 = (<double complex(*)(int, int, double, double) nogil>func)(<int>(<long*>ip0)[0], <int>(<long*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")             # <<<<<<<<<<<<<<
 *             ov0 = <double complex>NPY_NAN
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
    /*else*/ {
      sf_error(__pyx_v_func_name, SF_ERROR_DOMAIN, ((char *)"invalid input argument"));

      /* "scipy/special/_ufuncs.pyx":249
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double complex>NPY_NAN             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 */
      __pyx_v_ov0 = __pyx_t_double_complex_from_parts(((double)NPY_NAN), 0);
    }
    __pyx_L5:;

    /* "scipy/special/_ufuncs.pyx":250
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double complex>NPY_NAN
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":251
 *             ov0 = <double complex>NPY_NAN
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":252
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":253
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":254
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":255
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":256
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_D_lD__As_lD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":234
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_iidd__As_lldd_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":258
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_lD__As_lD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_D_lD__As_lD_D(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  __pyx_t_double_complex __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":259
 * 
 * cdef void loop_D_lD__As_lD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":260
 * cdef void loop_D_lD__As_lD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":261
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":262
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":263
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef double complex ov0
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":264
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":266
 *     cdef char *op0 = args[2]
 *     cdef double complex ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double complex(*)(long, double complex) nogil>func)(<long>(<long*>ip0)[0], <double complex>(<double complex*>ip1)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":267
 *     cdef double complex ov0
 *     for i in range(n):
 *         ov0 = (<double complex(*)(long, double complex) nogil>func)(<long>(<long*>ip0)[0], <double complex>(<double complex*>ip1)[0])             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((__pyx_t_double_complex (*)(long, __pyx_t_double_complex))__pyx_v_func)(((long)(((long *)__pyx_v_ip0)[0])), __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_double_complex *)__pyx_v_ip1)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_double_complex *)__pyx_v_ip1)[0])))));

    /* "scipy/special/_ufuncs.pyx":268
 *     for i in range(n):
 *         ov0 = (<double complex(*)(long, double complex) nogil>func)(<long>(<long*>ip0)[0], <double complex>(<double complex*>ip1)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":269
 *         ov0 = (<double complex(*)(long, double complex) nogil>func)(<long>(<long*>ip0)[0], <double complex>(<double complex*>ip1)[0])
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":270
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":271
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":272
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_d__As_d_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":258
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_D_lD__As_lD_D(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":274
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_d__As_d_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":275
 * 
 * cdef void loop_d_d__As_d_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":276
 * cdef void loop_d_d__As_d_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":277
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":278
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef double ov0
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":279
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":281
 *     cdef char *op0 = args[1]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double) nogil>func)(<double>(<double*>ip0)[0])
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":282
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(double) nogil>func)(<double>(<double*>ip0)[0])             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(double))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])));

    /* "scipy/special/_ufuncs.pyx":283
 *     for i in range(n):
 *         ov0 = (<double(*)(double) nogil>func)(<double>(<double*>ip0)[0])
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":284
 *         ov0 = (<double(*)(double) nogil>func)(<double>(<double*>ip0)[0])
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":285
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));
  }

  /* "scipy/special/_ufuncs.pyx":286
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_d__As_f_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":274
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_d__As_d_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":288
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_d__As_f_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":289
 * 
 * cdef void loop_d_d__As_f_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":290
 * cdef void loop_d_d__As_f_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":291
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":292
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef double ov0
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":293
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":295
 *     cdef char *op0 = args[1]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double) nogil>func)(<double>(<float*>ip0)[0])
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":296
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(double) nogil>func)(<double>(<float*>ip0)[0])             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(double))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])));

    /* "scipy/special/_ufuncs.pyx":297
 *     for i in range(n):
 *         ov0 = (<double(*)(double) nogil>func)(<double>(<float*>ip0)[0])
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":298
 *         ov0 = (<double(*)(double) nogil>func)(<double>(<float*>ip0)[0])
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":299
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));
  }

  /* "scipy/special/_ufuncs.pyx":300
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_dd__As_dd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":288
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_d__As_f_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":302
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dd__As_dd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":303
 * 
 * cdef void loop_d_dd__As_dd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":304
 * cdef void loop_d_dd__As_dd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":305
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":306
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":307
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef double ov0
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":308
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":310
 *     cdef char *op0 = args[2]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0])
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":311
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0])             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(double, double))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])));

    /* "scipy/special/_ufuncs.pyx":312
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0])
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":313
 *         ov0 = (<double(*)(double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0])
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":314
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":315
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":316
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_dd__As_ff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":302
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dd__As_dd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":318
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dd__As_ff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":319
 * 
 * cdef void loop_d_dd__As_ff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":320
 * cdef void loop_d_dd__As_ff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":321
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":322
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":323
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef double ov0
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":324
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":326
 *     cdef char *op0 = args[2]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0])
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":327
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0])             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(double, double))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])));

    /* "scipy/special/_ufuncs.pyx":328
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0])
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":329
 *         ov0 = (<double(*)(double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0])
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":330
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":331
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":332
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_ddd__As_ddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":318
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dd__As_ff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":334
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddd__As_ddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":335
 * 
 * cdef void loop_d_ddd__As_ddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":336
 * cdef void loop_d_ddd__As_ddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":337
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":338
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":339
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":340
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[3]
 *     cdef double ov0
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":341
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":343
 *     cdef char *op0 = args[3]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0])
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":344
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0])             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(double, double, double))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])));

    /* "scipy/special/_ufuncs.pyx":345
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0])
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":346
 *         ov0 = (<double(*)(double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0])
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":347
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         op0 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":348
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":349
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         op0 += steps[3]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[3]));
  }

  /* "scipy/special/_ufuncs.pyx":350
 *         ip2 += steps[2]
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_ddd__As_fff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":334
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddd__As_ddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":352
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddd__As_fff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":353
 * 
 * cdef void loop_d_ddd__As_fff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":354
 * cdef void loop_d_ddd__As_fff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":355
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":356
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":357
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":358
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[3]
 *     cdef double ov0
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":359
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":361
 *     cdef char *op0 = args[3]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0])
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":362
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0])             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(double, double, double))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), ((double)(((float *)__pyx_v_ip2)[0])));

    /* "scipy/special/_ufuncs.pyx":363
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0])
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":364
 *         ov0 = (<double(*)(double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0])
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":365
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         op0 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":366
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":367
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         op0 += steps[3]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[3]));
  }

  /* "scipy/special/_ufuncs.pyx":368
 *         ip2 += steps[2]
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_dddd__As_dddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":352
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddd__As_fff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":370
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dddd__As_dddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_dddd_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":371
 * 
 * cdef void loop_d_dddd__As_dddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":372
 * cdef void loop_d_dddd__As_dddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":373
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":374
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":375
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":376
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":377
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef double ov0
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":378
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":380
 *     cdef char *op0 = args[4]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":381
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(double, double, double, double))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])), ((double)(((double *)__pyx_v_ip3)[0])));

    /* "scipy/special/_ufuncs.pyx":382
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":383
 *         ov0 = (<double(*)(double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":384
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":385
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":386
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":387
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":388
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_dddd__As_ffff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":370
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dddd__As_dddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":390
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dddd__As_ffff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_ffff_f(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":391
 * 
 * cdef void loop_d_dddd__As_ffff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":392
 * cdef void loop_d_dddd__As_ffff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":393
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":394
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":395
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":396
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":397
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef double ov0
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":398
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":400
 *     cdef char *op0 = args[4]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0])
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":401
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0])             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(double, double, double, double))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), ((double)(((float *)__pyx_v_ip2)[0])), ((double)(((float *)__pyx_v_ip3)[0])));

    /* "scipy/special/_ufuncs.pyx":402
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0])
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":403
 *         ov0 = (<double(*)(double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0])
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":404
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":405
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":406
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":407
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":408
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_dddd_d_As_dddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":390
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dddd__As_ffff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":410
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dddd_d_As_dddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":411
 * 
 * cdef void loop_d_dddd_d_As_dddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":412
 * cdef void loop_d_dddd_d_As_dddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":413
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":414
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":415
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":416
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":417
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef char *op1 = args[5]
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":418
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[5]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":419
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 *     cdef char *op1 = args[5]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[5]);

  /* "scipy/special/_ufuncs.pyx":422
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double, double, double, double, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], &ov1)
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":423
 *     cdef double ov1
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], &ov1)             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 */
    __pyx_v_ov0 = ((double (*)(double, double, double, double, double *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])), ((double)(((double *)__pyx_v_ip3)[0])), (&__pyx_v_ov1));

    /* "scipy/special/_ufuncs.pyx":424
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], &ov1)
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":425
 *         ov0 = (<double(*)(double, double, double, double, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], &ov1)
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op1)[0]) = ((double)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":426
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":427
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":428
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":429
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *         op1 += steps[5]
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":430
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *         op1 += steps[5]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));

    /* "scipy/special/_ufuncs.pyx":431
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *         op1 += steps[5]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[5]));
  }

  /* "scipy/special/_ufuncs.pyx":432
 *         op0 += steps[4]
 *         op1 += steps[5]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_dddd_d_As_ffff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":410
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dddd_d_As_dddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":434
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dddd_d_As_ffff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":435
 * 
 * cdef void loop_d_dddd_d_As_ffff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":436
 * cdef void loop_d_dddd_d_As_ffff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":437
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":438
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":439
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":440
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":441
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef char *op1 = args[5]
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":442
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[5]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":443
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 *     cdef char *op1 = args[5]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[5]);

  /* "scipy/special/_ufuncs.pyx":446
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double, double, double, double, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], &ov1)
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":447
 *     cdef double ov1
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], &ov1)             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 */
    __pyx_v_ov0 = ((double (*)(double, double, double, double, double *))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), ((double)(((float *)__pyx_v_ip2)[0])), ((double)(((float *)__pyx_v_ip3)[0])), (&__pyx_v_ov1));

    /* "scipy/special/_ufuncs.pyx":448
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], &ov1)
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":449
 *         ov0 = (<double(*)(double, double, double, double, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], &ov1)
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((float *)__pyx_v_op1)[0]) = ((float)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":450
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":451
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":452
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":453
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *         op1 += steps[5]
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":454
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *         op1 += steps[5]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));

    /* "scipy/special/_ufuncs.pyx":455
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *         op1 += steps[5]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[5]));
  }

  /* "scipy/special/_ufuncs.pyx":456
 *         op0 += steps[4]
 *         op1 += steps[5]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_ddddddd__As_ddddddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":434
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dddd_d_As_ffff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":458
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddddddd__As_ddddddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddddddd__As_ddddddd_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_ip4;
  char *__pyx_v_ip5;
  char *__pyx_v_ip6;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":459
 * 
 * cdef void loop_d_ddddddd__As_ddddddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":460
 * cdef void loop_d_ddddddd__As_ddddddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":461
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":462
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":463
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":464
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":465
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *ip4 = args[4]
 *     cdef char *ip5 = args[5]
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":466
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]             # <<<<<<<<<<<<<<
 *     cdef char *ip5 = args[5]
 *     cdef char *ip6 = args[6]
 */
  __pyx_v_ip4 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":467
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]
 *     cdef char *ip5 = args[5]             # <<<<<<<<<<<<<<
 *     cdef char *ip6 = args[6]
 *     cdef char *op0 = args[7]
 */
  __pyx_v_ip5 = (__pyx_v_args[5]);

  /* "scipy/special/_ufuncs.pyx":468
 *     cdef char *ip4 = args[4]
 *     cdef char *ip5 = args[5]
 *     cdef char *ip6 = args[6]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[7]
 *     cdef double ov0
 */
  __pyx_v_ip6 = (__pyx_v_args[6]);

  /* "scipy/special/_ufuncs.pyx":469
 *     cdef char *ip5 = args[5]
 *     cdef char *ip6 = args[6]
 *     cdef char *op0 = args[7]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[7]);

  /* "scipy/special/_ufuncs.pyx":471
 *     cdef char *op0 = args[7]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double, double, double, double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], <double>(<double*>ip4)[0], <double>(<double*>ip5)[0], <double>(<double*>ip6)[0])
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":472
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], <double>(<double*>ip4)[0], <double>(<double*>ip5)[0], <double>(<double*>ip6)[0])             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(double, double, double, double, double, double, double))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])), ((double)(((double *)__pyx_v_ip3)[0])), ((double)(((double *)__pyx_v_ip4)[0])), ((double)(((double *)__pyx_v_ip5)[0])), ((double)(((double *)__pyx_v_ip6)[0])));

    /* "scipy/special/_ufuncs.pyx":473
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], <double>(<double*>ip4)[0], <double>(<double*>ip5)[0], <double>(<double*>ip6)[0])
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":474
 *         ov0 = (<double(*)(double, double, double, double, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], <double>(<double*>ip4)[0], <double>(<double*>ip5)[0], <double>(<double*>ip6)[0])
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":475
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":476
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         ip4 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":477
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         ip4 += steps[4]
 *         ip5 += steps[5]
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":478
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         ip4 += steps[4]             # <<<<<<<<<<<<<<
 *         ip5 += steps[5]
 *         ip6 += steps[6]
 */
    __pyx_v_ip4 = (__pyx_v_ip4 + (__pyx_v_steps[4]));

    /* "scipy/special/_ufuncs.pyx":479
 *         ip3 += steps[3]
 *         ip4 += steps[4]
 *         ip5 += steps[5]             # <<<<<<<<<<<<<<
 *         ip6 += steps[6]
 *         op0 += steps[7]
 */
    __pyx_v_ip5 = (__pyx_v_ip5 + (__pyx_v_steps[5]));

    /* "scipy/special/_ufuncs.pyx":480
 *         ip4 += steps[4]
 *         ip5 += steps[5]
 *         ip6 += steps[6]             # <<<<<<<<<<<<<<
 *         op0 += steps[7]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip6 = (__pyx_v_ip6 + (__pyx_v_steps[6]));

    /* "scipy/special/_ufuncs.pyx":481
 *         ip5 += steps[5]
 *         ip6 += steps[6]
 *         op0 += steps[7]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[7]));
  }

  /* "scipy/special/_ufuncs.pyx":482
 *         ip6 += steps[6]
 *         op0 += steps[7]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_ddddddd__As_fffffff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":458
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddddddd__As_ddddddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":484
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddddddd__As_fffffff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddddddd__As_fffffff_f(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_ip4;
  char *__pyx_v_ip5;
  char *__pyx_v_ip6;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":485
 * 
 * cdef void loop_d_ddddddd__As_fffffff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":486
 * cdef void loop_d_ddddddd__As_fffffff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":487
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":488
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":489
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":490
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":491
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *ip4 = args[4]
 *     cdef char *ip5 = args[5]
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":492
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]             # <<<<<<<<<<<<<<
 *     cdef char *ip5 = args[5]
 *     cdef char *ip6 = args[6]
 */
  __pyx_v_ip4 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":493
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]
 *     cdef char *ip5 = args[5]             # <<<<<<<<<<<<<<
 *     cdef char *ip6 = args[6]
 *     cdef char *op0 = args[7]
 */
  __pyx_v_ip5 = (__pyx_v_args[5]);

  /* "scipy/special/_ufuncs.pyx":494
 *     cdef char *ip4 = args[4]
 *     cdef char *ip5 = args[5]
 *     cdef char *ip6 = args[6]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[7]
 *     cdef double ov0
 */
  __pyx_v_ip6 = (__pyx_v_args[6]);

  /* "scipy/special/_ufuncs.pyx":495
 *     cdef char *ip5 = args[5]
 *     cdef char *ip6 = args[6]
 *     cdef char *op0 = args[7]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[7]);

  /* "scipy/special/_ufuncs.pyx":497
 *     cdef char *op0 = args[7]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(double, double, double, double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], <double>(<float*>ip4)[0], <double>(<float*>ip5)[0], <double>(<float*>ip6)[0])
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":498
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], <double>(<float*>ip4)[0], <double>(<float*>ip5)[0], <double>(<float*>ip6)[0])             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(double, double, double, double, double, double, double))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), ((double)(((float *)__pyx_v_ip2)[0])), ((double)(((float *)__pyx_v_ip3)[0])), ((double)(((float *)__pyx_v_ip4)[0])), ((double)(((float *)__pyx_v_ip5)[0])), ((double)(((float *)__pyx_v_ip6)[0])));

    /* "scipy/special/_ufuncs.pyx":499
 *     for i in range(n):
 *         ov0 = (<double(*)(double, double, double, double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], <double>(<float*>ip4)[0], <double>(<float*>ip5)[0], <double>(<float*>ip6)[0])
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":500
 *         ov0 = (<double(*)(double, double, double, double, double, double, double) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], <double>(<float*>ip4)[0], <double>(<float*>ip5)[0], <double>(<float*>ip6)[0])
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":501
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":502
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         ip4 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":503
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         ip4 += steps[4]
 *         ip5 += steps[5]
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":504
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         ip4 += steps[4]             # <<<<<<<<<<<<<<
 *         ip5 += steps[5]
 *         ip6 += steps[6]
 */
    __pyx_v_ip4 = (__pyx_v_ip4 + (__pyx_v_steps[4]));

    /* "scipy/special/_ufuncs.pyx":505
 *         ip3 += steps[3]
 *         ip4 += steps[4]
 *         ip5 += steps[5]             # <<<<<<<<<<<<<<
 *         ip6 += steps[6]
 *         op0 += steps[7]
 */
    __pyx_v_ip5 = (__pyx_v_ip5 + (__pyx_v_steps[5]));

    /* "scipy/special/_ufuncs.pyx":506
 *         ip4 += steps[4]
 *         ip5 += steps[5]
 *         ip6 += steps[6]             # <<<<<<<<<<<<<<
 *         op0 += steps[7]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip6 = (__pyx_v_ip6 + (__pyx_v_steps[6]));

    /* "scipy/special/_ufuncs.pyx":507
 *         ip5 += steps[5]
 *         ip6 += steps[6]
 *         op0 += steps[7]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[7]));
  }

  /* "scipy/special/_ufuncs.pyx":508
 *         ip6 += steps[6]
 *         op0 += steps[7]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_dddi_d_As_dddl_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":484
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddddddd__As_fffffff_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":510
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dddi_d_As_dddl_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_dddi_d_As_dddl_dd(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/special/_ufuncs.pyx":511
 * 
 * cdef void loop_d_dddi_d_As_dddl_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":512
 * cdef void loop_d_dddi_d_As_dddl_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":513
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":514
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":515
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":516
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":517
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef char *op1 = args[5]
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":518
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[5]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":519
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 *     cdef char *op1 = args[5]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[5]);

  /* "scipy/special/_ufuncs.pyx":522
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         if <int>(<long*>ip3)[0] == (<long*>ip3)[0]:
 *             ov0 = (<double(*)(double, double, double, int, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <int>(<long*>ip3)[0], &ov1)
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":523
 *     cdef double ov1
 *     for i in range(n):
 *         if <int>(<long*>ip3)[0] == (<long*>ip3)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double(*)(double, double, double, int, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <int>(<long*>ip3)[0], &ov1)
 *         else:
 */
    __pyx_t_4 = ((((int)(((long *)__pyx_v_ip3)[0])) == (((long *)__pyx_v_ip3)[0])) != 0);
    if (__pyx_t_4) {

      /* "scipy/special/_ufuncs.pyx":524
 *     for i in range(n):
 *         if <int>(<long*>ip3)[0] == (<long*>ip3)[0]:
 *             ov0 = (<double(*)(double, double, double, int, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <int>(<long*>ip3)[0], &ov1)             # <<<<<<<<<<<<<<
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 */
      __pyx_v_ov0 = ((double (*)(double, double, double, int, double *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])), ((int)(((long *)__pyx_v_ip3)[0])), (&__pyx_v_ov1));

      /* "scipy/special/_ufuncs.pyx":523
 *     cdef double ov1
 *     for i in range(n):
 *         if <int>(<long*>ip3)[0] == (<long*>ip3)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double(*)(double, double, double, int, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <int>(<long*>ip3)[0], &ov1)
 *         else:
 */
      goto __pyx_L5;
    }

    /* "scipy/special/_ufuncs.pyx":526
 *             ov0 = (<double(*)(double, double, double, int, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <int>(<long*>ip3)[0], &ov1)
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")             # <<<<<<<<<<<<<<
 *             ov0 = <double>NPY_NAN
 *             ov1 = <double>NPY_NAN
 */
    /*else*/ {
      sf_error(__pyx_v_func_name, SF_ERROR_DOMAIN, ((char *)"invalid input argument"));

      /* "scipy/special/_ufuncs.pyx":527
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double>NPY_NAN             # <<<<<<<<<<<<<<
 *             ov1 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0
 */
      __pyx_v_ov0 = ((double)NPY_NAN);

      /* "scipy/special/_ufuncs.pyx":528
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double>NPY_NAN
 *             ov1 = <double>NPY_NAN             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 */
      __pyx_v_ov1 = ((double)NPY_NAN);
    }
    __pyx_L5:;

    /* "scipy/special/_ufuncs.pyx":529
 *             ov0 = <double>NPY_NAN
 *             ov1 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":530
 *             ov1 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op1)[0]) = ((double)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":531
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":532
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":533
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":534
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *         op1 += steps[5]
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":535
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *         op1 += steps[5]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));

    /* "scipy/special/_ufuncs.pyx":536
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *         op1 += steps[5]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[5]));
  }

  /* "scipy/special/_ufuncs.pyx":537
 *         op0 += steps[4]
 *         op1 += steps[5]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_ddi_d_As_ddl_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":510
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_dddi_d_As_dddl_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":539
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddi_d_As_ddl_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddi_d_As_ddl_dd(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/special/_ufuncs.pyx":540
 * 
 * cdef void loop_d_ddi_d_As_ddl_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":541
 * cdef void loop_d_ddi_d_As_ddl_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":542
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":543
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":544
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":545
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[3]
 *     cdef char *op1 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":546
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[4]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":547
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 *     cdef char *op1 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":550
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         if <int>(<long*>ip2)[0] == (<long*>ip2)[0]:
 *             ov0 = (<double(*)(double, double, int, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <int>(<long*>ip2)[0], &ov1)
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":551
 *     cdef double ov1
 *     for i in range(n):
 *         if <int>(<long*>ip2)[0] == (<long*>ip2)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double(*)(double, double, int, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <int>(<long*>ip2)[0], &ov1)
 *         else:
 */
    __pyx_t_4 = ((((int)(((long *)__pyx_v_ip2)[0])) == (((long *)__pyx_v_ip2)[0])) != 0);
    if (__pyx_t_4) {

      /* "scipy/special/_ufuncs.pyx":552
 *     for i in range(n):
 *         if <int>(<long*>ip2)[0] == (<long*>ip2)[0]:
 *             ov0 = (<double(*)(double, double, int, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <int>(<long*>ip2)[0], &ov1)             # <<<<<<<<<<<<<<
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 */
      __pyx_v_ov0 = ((double (*)(double, double, int, double *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((int)(((long *)__pyx_v_ip2)[0])), (&__pyx_v_ov1));

      /* "scipy/special/_ufuncs.pyx":551
 *     cdef double ov1
 *     for i in range(n):
 *         if <int>(<long*>ip2)[0] == (<long*>ip2)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double(*)(double, double, int, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <int>(<long*>ip2)[0], &ov1)
 *         else:
 */
      goto __pyx_L5;
    }

    /* "scipy/special/_ufuncs.pyx":554
 *             ov0 = (<double(*)(double, double, int, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <int>(<long*>ip2)[0], &ov1)
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")             # <<<<<<<<<<<<<<
 *             ov0 = <double>NPY_NAN
 *             ov1 = <double>NPY_NAN
 */
    /*else*/ {
      sf_error(__pyx_v_func_name, SF_ERROR_DOMAIN, ((char *)"invalid input argument"));

      /* "scipy/special/_ufuncs.pyx":555
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double>NPY_NAN             # <<<<<<<<<<<<<<
 *             ov1 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0
 */
      __pyx_v_ov0 = ((double)NPY_NAN);

      /* "scipy/special/_ufuncs.pyx":556
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double>NPY_NAN
 *             ov1 = <double>NPY_NAN             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 */
      __pyx_v_ov1 = ((double)NPY_NAN);
    }
    __pyx_L5:;

    /* "scipy/special/_ufuncs.pyx":557
 *             ov0 = <double>NPY_NAN
 *             ov1 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":558
 *             ov1 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op1)[0]) = ((double)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":559
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":560
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         op0 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":561
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         op0 += steps[3]
 *         op1 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":562
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         op0 += steps[3]             # <<<<<<<<<<<<<<
 *         op1 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":563
 *         ip2 += steps[2]
 *         op0 += steps[3]
 *         op1 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":564
 *         op0 += steps[3]
 *         op1 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_ddiiddd__As_ddllddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":539
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddi_d_As_ddl_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":566
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddiiddd__As_ddllddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ddiiddd__As_ddllddd_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_ip4;
  char *__pyx_v_ip5;
  char *__pyx_v_ip6;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;

  /* "scipy/special/_ufuncs.pyx":567
 * 
 * cdef void loop_d_ddiiddd__As_ddllddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":568
 * cdef void loop_d_ddiiddd__As_ddllddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":569
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":570
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":571
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":572
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":573
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *ip4 = args[4]
 *     cdef char *ip5 = args[5]
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":574
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]             # <<<<<<<<<<<<<<
 *     cdef char *ip5 = args[5]
 *     cdef char *ip6 = args[6]
 */
  __pyx_v_ip4 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":575
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]
 *     cdef char *ip5 = args[5]             # <<<<<<<<<<<<<<
 *     cdef char *ip6 = args[6]
 *     cdef char *op0 = args[7]
 */
  __pyx_v_ip5 = (__pyx_v_args[5]);

  /* "scipy/special/_ufuncs.pyx":576
 *     cdef char *ip4 = args[4]
 *     cdef char *ip5 = args[5]
 *     cdef char *ip6 = args[6]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[7]
 *     cdef double ov0
 */
  __pyx_v_ip6 = (__pyx_v_args[6]);

  /* "scipy/special/_ufuncs.pyx":577
 *     cdef char *ip5 = args[5]
 *     cdef char *ip6 = args[6]
 *     cdef char *op0 = args[7]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[7]);

  /* "scipy/special/_ufuncs.pyx":579
 *     cdef char *op0 = args[7]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         if <int>(<long*>ip2)[0] == (<long*>ip2)[0] and <int>(<long*>ip3)[0] == (<long*>ip3)[0]:
 *             ov0 = (<double(*)(double, double, int, int, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <int>(<long*>ip2)[0], <int>(<long*>ip3)[0], <double>(<double*>ip4)[0], <double>(<double*>ip5)[0], <double>(<double*>ip6)[0])
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":580
 *     cdef double ov0
 *     for i in range(n):
 *         if <int>(<long*>ip2)[0] == (<long*>ip2)[0] and <int>(<long*>ip3)[0] == (<long*>ip3)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double(*)(double, double, int, int, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <int>(<long*>ip2)[0], <int>(<long*>ip3)[0], <double>(<double*>ip4)[0], <double>(<double*>ip5)[0], <double>(<double*>ip6)[0])
 *         else:
 */
    __pyx_t_5 = ((((int)(((long *)__pyx_v_ip2)[0])) == (((long *)__pyx_v_ip2)[0])) != 0);
    if (__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_5 = ((((int)(((long *)__pyx_v_ip3)[0])) == (((long *)__pyx_v_ip3)[0])) != 0);
    __pyx_t_4 = __pyx_t_5;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_4) {

      /* "scipy/special/_ufuncs.pyx":581
 *     for i in range(n):
 *         if <int>(<long*>ip2)[0] == (<long*>ip2)[0] and <int>(<long*>ip3)[0] == (<long*>ip3)[0]:
 *             ov0 = (<double(*)(double, double, int, int, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <int>(<long*>ip2)[0], <int>(<long*>ip3)[0], <double>(<double*>ip4)[0], <double>(<double*>ip5)[0], <double>(<double*>ip6)[0])             # <<<<<<<<<<<<<<
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 */
      __pyx_v_ov0 = ((double (*)(double, double, int, int, double, double, double))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((int)(((long *)__pyx_v_ip2)[0])), ((int)(((long *)__pyx_v_ip3)[0])), ((double)(((double *)__pyx_v_ip4)[0])), ((double)(((double *)__pyx_v_ip5)[0])), ((double)(((double *)__pyx_v_ip6)[0])));

      /* "scipy/special/_ufuncs.pyx":580
 *     cdef double ov0
 *     for i in range(n):
 *         if <int>(<long*>ip2)[0] == (<long*>ip2)[0] and <int>(<long*>ip3)[0] == (<long*>ip3)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double(*)(double, double, int, int, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <int>(<long*>ip2)[0], <int>(<long*>ip3)[0], <double>(<double*>ip4)[0], <double>(<double*>ip5)[0], <double>(<double*>ip6)[0])
 *         else:
 */
      goto __pyx_L5;
    }

    /* "scipy/special/_ufuncs.pyx":583
 *             ov0 = (<double(*)(double, double, int, int, double, double, double) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <int>(<long*>ip2)[0], <int>(<long*>ip3)[0], <double>(<double*>ip4)[0], <double>(<double*>ip5)[0], <double>(<double*>ip6)[0])
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")             # <<<<<<<<<<<<<<
 *             ov0 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0
 */
    /*else*/ {
      sf_error(__pyx_v_func_name, SF_ERROR_DOMAIN, ((char *)"invalid input argument"));

      /* "scipy/special/_ufuncs.pyx":584
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double>NPY_NAN             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
      __pyx_v_ov0 = ((double)NPY_NAN);
    }
    __pyx_L5:;

    /* "scipy/special/_ufuncs.pyx":585
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":586
 *             ov0 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":587
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":588
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         ip4 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":589
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         ip4 += steps[4]
 *         ip5 += steps[5]
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":590
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         ip4 += steps[4]             # <<<<<<<<<<<<<<
 *         ip5 += steps[5]
 *         ip6 += steps[6]
 */
    __pyx_v_ip4 = (__pyx_v_ip4 + (__pyx_v_steps[4]));

    /* "scipy/special/_ufuncs.pyx":591
 *         ip3 += steps[3]
 *         ip4 += steps[4]
 *         ip5 += steps[5]             # <<<<<<<<<<<<<<
 *         ip6 += steps[6]
 *         op0 += steps[7]
 */
    __pyx_v_ip5 = (__pyx_v_ip5 + (__pyx_v_steps[5]));

    /* "scipy/special/_ufuncs.pyx":592
 *         ip4 += steps[4]
 *         ip5 += steps[5]
 *         ip6 += steps[6]             # <<<<<<<<<<<<<<
 *         op0 += steps[7]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip6 = (__pyx_v_ip6 + (__pyx_v_steps[6]));

    /* "scipy/special/_ufuncs.pyx":593
 *         ip5 += steps[5]
 *         ip6 += steps[6]
 *         op0 += steps[7]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[7]));
  }

  /* "scipy/special/_ufuncs.pyx":594
 *         ip6 += steps[6]
 *         op0 += steps[7]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_id__As_ld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":566
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ddiiddd__As_ddllddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":596
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_id__As_ld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/special/_ufuncs.pyx":597
 * 
 * cdef void loop_d_id__As_ld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":598
 * cdef void loop_d_id__As_ld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":599
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":600
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":601
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef double ov0
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":602
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":604
 *     cdef char *op0 = args[2]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0]:
 *             ov0 = (<double(*)(int, double) nogil>func)(<int>(<long*>ip0)[0], <double>(<double*>ip1)[0])
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":605
 *     cdef double ov0
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double(*)(int, double) nogil>func)(<int>(<long*>ip0)[0], <double>(<double*>ip1)[0])
 *         else:
 */
    __pyx_t_4 = ((((int)(((long *)__pyx_v_ip0)[0])) == (((long *)__pyx_v_ip0)[0])) != 0);
    if (__pyx_t_4) {

      /* "scipy/special/_ufuncs.pyx":606
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0]:
 *             ov0 = (<double(*)(int, double) nogil>func)(<int>(<long*>ip0)[0], <double>(<double*>ip1)[0])             # <<<<<<<<<<<<<<
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 */
      __pyx_v_ov0 = ((double (*)(int, double))__pyx_v_func)(((int)(((long *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])));

      /* "scipy/special/_ufuncs.pyx":605
 *     cdef double ov0
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double(*)(int, double) nogil>func)(<int>(<long*>ip0)[0], <double>(<double*>ip1)[0])
 *         else:
 */
      goto __pyx_L5;
    }

    /* "scipy/special/_ufuncs.pyx":608
 *             ov0 = (<double(*)(int, double) nogil>func)(<int>(<long*>ip0)[0], <double>(<double*>ip1)[0])
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")             # <<<<<<<<<<<<<<
 *             ov0 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0
 */
    /*else*/ {
      sf_error(__pyx_v_func_name, SF_ERROR_DOMAIN, ((char *)"invalid input argument"));

      /* "scipy/special/_ufuncs.pyx":609
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double>NPY_NAN             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
      __pyx_v_ov0 = ((double)NPY_NAN);
    }
    __pyx_L5:;

    /* "scipy/special/_ufuncs.pyx":610
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":611
 *             ov0 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":612
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":613
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":614
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_iid__As_lld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":596
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_id__As_ld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":616
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_iid__As_lld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_iid__As_lld_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;

  /* "scipy/special/_ufuncs.pyx":617
 * 
 * cdef void loop_d_iid__As_lld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":618
 * cdef void loop_d_iid__As_lld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":619
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":620
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":621
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":622
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[3]
 *     cdef double ov0
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":623
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":625
 *     cdef char *op0 = args[3]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0] and <int>(<long*>ip1)[0] == (<long*>ip1)[0]:
 *             ov0 = (<double(*)(int, int, double) nogil>func)(<int>(<long*>ip0)[0], <int>(<long*>ip1)[0], <double>(<double*>ip2)[0])
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":626
 *     cdef double ov0
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0] and <int>(<long*>ip1)[0] == (<long*>ip1)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double(*)(int, int, double) nogil>func)(<int>(<long*>ip0)[0], <int>(<long*>ip1)[0], <double>(<double*>ip2)[0])
 *         else:
 */
    __pyx_t_5 = ((((int)(((long *)__pyx_v_ip0)[0])) == (((long *)__pyx_v_ip0)[0])) != 0);
    if (__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_5 = ((((int)(((long *)__pyx_v_ip1)[0])) == (((long *)__pyx_v_ip1)[0])) != 0);
    __pyx_t_4 = __pyx_t_5;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_4) {

      /* "scipy/special/_ufuncs.pyx":627
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0] and <int>(<long*>ip1)[0] == (<long*>ip1)[0]:
 *             ov0 = (<double(*)(int, int, double) nogil>func)(<int>(<long*>ip0)[0], <int>(<long*>ip1)[0], <double>(<double*>ip2)[0])             # <<<<<<<<<<<<<<
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 */
      __pyx_v_ov0 = ((double (*)(int, int, double))__pyx_v_func)(((int)(((long *)__pyx_v_ip0)[0])), ((int)(((long *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])));

      /* "scipy/special/_ufuncs.pyx":626
 *     cdef double ov0
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0] and <int>(<long*>ip1)[0] == (<long*>ip1)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<double(*)(int, int, double) nogil>func)(<int>(<long*>ip0)[0], <int>(<long*>ip1)[0], <double>(<double*>ip2)[0])
 *         else:
 */
      goto __pyx_L5;
    }

    /* "scipy/special/_ufuncs.pyx":629
 *             ov0 = (<double(*)(int, int, double) nogil>func)(<int>(<long*>ip0)[0], <int>(<long*>ip1)[0], <double>(<double*>ip2)[0])
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")             # <<<<<<<<<<<<<<
 *             ov0 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0
 */
    /*else*/ {
      sf_error(__pyx_v_func_name, SF_ERROR_DOMAIN, ((char *)"invalid input argument"));

      /* "scipy/special/_ufuncs.pyx":630
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double>NPY_NAN             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
      __pyx_v_ov0 = ((double)NPY_NAN);
    }
    __pyx_L5:;

    /* "scipy/special/_ufuncs.pyx":631
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":632
 *             ov0 = <double>NPY_NAN
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":633
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         op0 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":634
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":635
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         op0 += steps[3]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[3]));
  }

  /* "scipy/special/_ufuncs.pyx":636
 *         ip2 += steps[2]
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_ld__As_ld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":616
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_iid__As_lld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":638
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ld__As_ld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":639
 * 
 * cdef void loop_d_ld__As_ld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":640
 * cdef void loop_d_ld__As_ld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":641
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":642
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":643
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef double ov0
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":644
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":646
 *     cdef char *op0 = args[2]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(long, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0])
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":647
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(long, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0])             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(long, double))__pyx_v_func)(((long)(((long *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])));

    /* "scipy/special/_ufuncs.pyx":648
 *     for i in range(n):
 *         ov0 = (<double(*)(long, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0])
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":649
 *         ov0 = (<double(*)(long, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0])
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":650
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":651
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":652
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_ldd__As_ldd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":638
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ld__As_ld_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":654
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ldd__As_ldd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_ldd__As_ldd_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":655
 * 
 * cdef void loop_d_ldd__As_ldd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":656
 * cdef void loop_d_ldd__As_ldd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":657
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":658
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":659
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":660
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[3]
 *     cdef double ov0
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":661
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":663
 *     cdef char *op0 = args[3]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(long, double, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0])
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":664
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(long, double, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0])             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(long, double, double))__pyx_v_func)(((long)(((long *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])));

    /* "scipy/special/_ufuncs.pyx":665
 *     for i in range(n):
 *         ov0 = (<double(*)(long, double, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0])
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":666
 *         ov0 = (<double(*)(long, double, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0])
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":667
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         op0 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":668
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":669
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         op0 += steps[3]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[3]));
  }

  /* "scipy/special/_ufuncs.pyx":670
 *         ip2 += steps[2]
 *         op0 += steps[3]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_d_lddd__As_lddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":654
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_ldd__As_ldd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":672
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_lddd__As_lddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_d_lddd__As_lddd_d(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_op0;
  double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":673
 * 
 * cdef void loop_d_lddd__As_lddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":674
 * cdef void loop_d_lddd__As_lddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":675
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":676
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":677
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":678
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":679
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[4]
 *     cdef double ov0
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":680
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *op0 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":682
 *     cdef char *op0 = args[4]
 *     cdef double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<double(*)(long, double, double, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":683
 *     cdef double ov0
 *     for i in range(n):
 *         ov0 = (<double(*)(long, double, double, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((double (*)(long, double, double, double))__pyx_v_func)(((long)(((long *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])), ((double)(((double *)__pyx_v_ip3)[0])));

    /* "scipy/special/_ufuncs.pyx":684
 *     for i in range(n):
 *         ov0 = (<double(*)(long, double, double, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":685
 *         ov0 = (<double(*)(long, double, double, double) nogil>func)(<long>(<long*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0])
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":686
 *         (<double *>op0)[0] = <double>ov0
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":687
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         op0 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":688
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":689
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         op0 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":690
 *         ip3 += steps[3]
 *         op0 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_f_f__As_f_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":672
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_d_lddd__As_lddd_d(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":692
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_f_f__As_f_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_f_f__As_f_f(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  float __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":693
 * 
 * cdef void loop_f_f__As_f_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":694
 * cdef void loop_f_f__As_f_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":695
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":696
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef float ov0
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":697
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef float ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":699
 *     cdef char *op0 = args[1]
 *     cdef float ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<float(*)(float) nogil>func)(<float>(<float*>ip0)[0])
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":700
 *     cdef float ov0
 *     for i in range(n):
 *         ov0 = (<float(*)(float) nogil>func)(<float>(<float*>ip0)[0])             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((float (*)(float))__pyx_v_func)(((float)(((float *)__pyx_v_ip0)[0])));

    /* "scipy/special/_ufuncs.pyx":701
 *     for i in range(n):
 *         ov0 = (<float(*)(float) nogil>func)(<float>(<float*>ip0)[0])
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":702
 *         ov0 = (<float(*)(float) nogil>func)(<float>(<float*>ip0)[0])
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":703
 *         (<float *>op0)[0] = <float>ov0
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));
  }

  /* "scipy/special/_ufuncs.pyx":704
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_g_g__As_g_g(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":692
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_f_f__As_f_f(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":706
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_g_g__As_g_g(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_g_g__As_g_g(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  long double __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":707
 * 
 * cdef void loop_g_g__As_g_g(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":708
 * cdef void loop_g_g__As_g_g(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":709
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":710
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef long double ov0
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":711
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef long double ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":713
 *     cdef char *op0 = args[1]
 *     cdef long double ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         ov0 = (<long double(*)(long double) nogil>func)(<long double>(<long double*>ip0)[0])
 *         (<long double *>op0)[0] = <long double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":714
 *     cdef long double ov0
 *     for i in range(n):
 *         ov0 = (<long double(*)(long double) nogil>func)(<long double>(<long double*>ip0)[0])             # <<<<<<<<<<<<<<
 *         (<long double *>op0)[0] = <long double>ov0
 *         ip0 += steps[0]
 */
    __pyx_v_ov0 = ((long double (*)(long double))__pyx_v_func)(((long double)(((long double *)__pyx_v_ip0)[0])));

    /* "scipy/special/_ufuncs.pyx":715
 *     for i in range(n):
 *         ov0 = (<long double(*)(long double) nogil>func)(<long double>(<long double*>ip0)[0])
 *         (<long double *>op0)[0] = <long double>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((long double *)__pyx_v_op0)[0]) = ((long double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":716
 *         ov0 = (<long double(*)(long double) nogil>func)(<long double>(<long double*>ip0)[0])
 *         (<long double *>op0)[0] = <long double>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":717
 *         (<long double *>op0)[0] = <long double>ov0
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));
  }

  /* "scipy/special/_ufuncs.pyx":718
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_D_DDDD_As_D_DDDD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":706
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_g_g__As_g_g(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":720
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_D_DDDD_As_D_DDDD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DDDD_As_D_DDDD(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  char *__pyx_v_op2;
  char *__pyx_v_op3;
  __pyx_t_double_complex __pyx_v_ov0;
  __pyx_t_double_complex __pyx_v_ov1;
  __pyx_t_double_complex __pyx_v_ov2;
  __pyx_t_double_complex __pyx_v_ov3;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":721
 * 
 * cdef void loop_i_D_DDDD_As_D_DDDD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":722
 * cdef void loop_i_D_DDDD_As_D_DDDD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":723
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":724
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":725
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":726
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":727
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op3 = args[4]
 *     cdef double complex ov0
 */
  __pyx_v_op2 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":728
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     cdef double complex ov1
 */
  __pyx_v_op3 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":733
 *     cdef double complex ov2
 *     cdef double complex ov3
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double complex, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double complex>(<double complex*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":734
 *     cdef double complex ov3
 *     for i in range(n):
 *         (<int(*)(double complex, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double complex>(<double complex*>ip0)[0], &ov0, &ov1, &ov2, &ov3)             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1
 */
    (void)(((int (*)(__pyx_t_double_complex, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *))__pyx_v_func)(__pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_double_complex *)__pyx_v_ip0)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_double_complex *)__pyx_v_ip0)[0])))), (&__pyx_v_ov0), (&__pyx_v_ov1), (&__pyx_v_ov2), (&__pyx_v_ov3)));

    /* "scipy/special/_ufuncs.pyx":735
 *     for i in range(n):
 *         (<int(*)(double complex, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double complex>(<double complex*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         (<double complex *>op2)[0] = <double complex>ov2
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":736
 *         (<int(*)(double complex, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double complex>(<double complex*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1             # <<<<<<<<<<<<<<
 *         (<double complex *>op2)[0] = <double complex>ov2
 *         (<double complex *>op3)[0] = <double complex>ov3
 */
    (((__pyx_t_double_complex *)__pyx_v_op1)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov1)), ((double)__Pyx_CIMAG(__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":737
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         (<double complex *>op2)[0] = <double complex>ov2             # <<<<<<<<<<<<<<
 *         (<double complex *>op3)[0] = <double complex>ov3
 *         ip0 += steps[0]
 */
    (((__pyx_t_double_complex *)__pyx_v_op2)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov2)), ((double)__Pyx_CIMAG(__pyx_v_ov2)));

    /* "scipy/special/_ufuncs.pyx":738
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         (<double complex *>op2)[0] = <double complex>ov2
 *         (<double complex *>op3)[0] = <double complex>ov3             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op3)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov3)), ((double)__Pyx_CIMAG(__pyx_v_ov3)));

    /* "scipy/special/_ufuncs.pyx":739
 *         (<double complex *>op2)[0] = <double complex>ov2
 *         (<double complex *>op3)[0] = <double complex>ov3
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":740
 *         (<double complex *>op3)[0] = <double complex>ov3
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *         op2 += steps[3]
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":741
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *         op2 += steps[3]
 *         op3 += steps[4]
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":742
 *         op0 += steps[1]
 *         op1 += steps[2]
 *         op2 += steps[3]             # <<<<<<<<<<<<<<
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op2 = (__pyx_v_op2 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":743
 *         op1 += steps[2]
 *         op2 += steps[3]
 *         op3 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op3 = (__pyx_v_op3 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":744
 *         op2 += steps[3]
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_D_DDDD_As_F_FFFF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":720
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_D_DDDD_As_D_DDDD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":746
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_D_DDDD_As_F_FFFF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DDDD_As_F_FFFF(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  char *__pyx_v_op2;
  char *__pyx_v_op3;
  __pyx_t_double_complex __pyx_v_ov0;
  __pyx_t_double_complex __pyx_v_ov1;
  __pyx_t_double_complex __pyx_v_ov2;
  __pyx_t_double_complex __pyx_v_ov3;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":747
 * 
 * cdef void loop_i_D_DDDD_As_F_FFFF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":748
 * cdef void loop_i_D_DDDD_As_F_FFFF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":749
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":750
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":751
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":752
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":753
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op3 = args[4]
 *     cdef double complex ov0
 */
  __pyx_v_op2 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":754
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     cdef double complex ov1
 */
  __pyx_v_op3 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":759
 *     cdef double complex ov2
 *     cdef double complex ov3
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double complex, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double complex>(<float complex*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float complex *>op0)[0] = <float complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":760
 *     cdef double complex ov3
 *     for i in range(n):
 *         (<int(*)(double complex, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double complex>(<float complex*>ip0)[0], &ov0, &ov1, &ov2, &ov3)             # <<<<<<<<<<<<<<
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1
 */
    (void)(((int (*)(__pyx_t_double_complex, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *))__pyx_v_func)(__pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_float_complex *)__pyx_v_ip0)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_float_complex *)__pyx_v_ip0)[0])))), (&__pyx_v_ov0), (&__pyx_v_ov1), (&__pyx_v_ov2), (&__pyx_v_ov3)));

    /* "scipy/special/_ufuncs.pyx":761
 *     for i in range(n):
 *         (<int(*)(double complex, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double complex>(<float complex*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float complex *>op0)[0] = <float complex>ov0             # <<<<<<<<<<<<<<
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         (<float complex *>op2)[0] = <float complex>ov2
 */
    (((__pyx_t_float_complex *)__pyx_v_op0)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov0)), ((float)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":762
 *         (<int(*)(double complex, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double complex>(<float complex*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1             # <<<<<<<<<<<<<<
 *         (<float complex *>op2)[0] = <float complex>ov2
 *         (<float complex *>op3)[0] = <float complex>ov3
 */
    (((__pyx_t_float_complex *)__pyx_v_op1)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov1)), ((float)__Pyx_CIMAG(__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":763
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         (<float complex *>op2)[0] = <float complex>ov2             # <<<<<<<<<<<<<<
 *         (<float complex *>op3)[0] = <float complex>ov3
 *         ip0 += steps[0]
 */
    (((__pyx_t_float_complex *)__pyx_v_op2)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov2)), ((float)__Pyx_CIMAG(__pyx_v_ov2)));

    /* "scipy/special/_ufuncs.pyx":764
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         (<float complex *>op2)[0] = <float complex>ov2
 *         (<float complex *>op3)[0] = <float complex>ov3             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((__pyx_t_float_complex *)__pyx_v_op3)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov3)), ((float)__Pyx_CIMAG(__pyx_v_ov3)));

    /* "scipy/special/_ufuncs.pyx":765
 *         (<float complex *>op2)[0] = <float complex>ov2
 *         (<float complex *>op3)[0] = <float complex>ov3
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":766
 *         (<float complex *>op3)[0] = <float complex>ov3
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *         op2 += steps[3]
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":767
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *         op2 += steps[3]
 *         op3 += steps[4]
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":768
 *         op0 += steps[1]
 *         op1 += steps[2]
 *         op2 += steps[3]             # <<<<<<<<<<<<<<
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op2 = (__pyx_v_op2 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":769
 *         op1 += steps[2]
 *         op2 += steps[3]
 *         op3 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op3 = (__pyx_v_op3 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":770
 *         op2 += steps[3]
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_D_DD_As_D_DD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":746
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_D_DDDD_As_F_FFFF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":772
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_D_DD_As_D_DD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DD_As_D_DD(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  __pyx_t_double_complex __pyx_v_ov0;
  __pyx_t_double_complex __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":773
 * 
 * cdef void loop_i_D_DD_As_D_DD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":774
 * cdef void loop_i_D_DD_As_D_DD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":775
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":776
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":777
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef double complex ov0
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":778
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     cdef double complex ov1
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":781
 *     cdef double complex ov0
 *     cdef double complex ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double complex, double complex *, double complex *) nogil>func)(<double complex>(<double complex*>ip0)[0], &ov0, &ov1)
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":782
 *     cdef double complex ov1
 *     for i in range(n):
 *         (<int(*)(double complex, double complex *, double complex *) nogil>func)(<double complex>(<double complex*>ip0)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1
 */
    (void)(((int (*)(__pyx_t_double_complex, __pyx_t_double_complex *, __pyx_t_double_complex *))__pyx_v_func)(__pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_double_complex *)__pyx_v_ip0)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_double_complex *)__pyx_v_ip0)[0])))), (&__pyx_v_ov0), (&__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":783
 *     for i in range(n):
 *         (<int(*)(double complex, double complex *, double complex *) nogil>func)(<double complex>(<double complex*>ip0)[0], &ov0, &ov1)
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         ip0 += steps[0]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":784
 *         (<int(*)(double complex, double complex *, double complex *) nogil>func)(<double complex>(<double complex*>ip0)[0], &ov0, &ov1)
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op1)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov1)), ((double)__Pyx_CIMAG(__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":785
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":786
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":787
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":788
 *         op0 += steps[1]
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_D_DD_As_F_FF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":772
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_D_DD_As_D_DD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":790
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_D_DD_As_F_FF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DD_As_F_FF(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  __pyx_t_double_complex __pyx_v_ov0;
  __pyx_t_double_complex __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":791
 * 
 * cdef void loop_i_D_DD_As_F_FF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":792
 * cdef void loop_i_D_DD_As_F_FF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":793
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":794
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":795
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef double complex ov0
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":796
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     cdef double complex ov1
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":799
 *     cdef double complex ov0
 *     cdef double complex ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double complex, double complex *, double complex *) nogil>func)(<double complex>(<float complex*>ip0)[0], &ov0, &ov1)
 *         (<float complex *>op0)[0] = <float complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":800
 *     cdef double complex ov1
 *     for i in range(n):
 *         (<int(*)(double complex, double complex *, double complex *) nogil>func)(<double complex>(<float complex*>ip0)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1
 */
    (void)(((int (*)(__pyx_t_double_complex, __pyx_t_double_complex *, __pyx_t_double_complex *))__pyx_v_func)(__pyx_t_double_complex_from_parts(((double)__Pyx_CREAL((((__pyx_t_float_complex *)__pyx_v_ip0)[0]))), ((double)__Pyx_CIMAG((((__pyx_t_float_complex *)__pyx_v_ip0)[0])))), (&__pyx_v_ov0), (&__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":801
 *     for i in range(n):
 *         (<int(*)(double complex, double complex *, double complex *) nogil>func)(<double complex>(<float complex*>ip0)[0], &ov0, &ov1)
 *         (<float complex *>op0)[0] = <float complex>ov0             # <<<<<<<<<<<<<<
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         ip0 += steps[0]
 */
    (((__pyx_t_float_complex *)__pyx_v_op0)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov0)), ((float)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":802
 *         (<int(*)(double complex, double complex *, double complex *) nogil>func)(<double complex>(<float complex*>ip0)[0], &ov0, &ov1)
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((__pyx_t_float_complex *)__pyx_v_op1)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov1)), ((float)__Pyx_CIMAG(__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":803
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":804
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":805
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":806
 *         op0 += steps[1]
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_d_DDDD_As_d_DDDD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":790
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_D_DD_As_F_FF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":808
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_DDDD_As_d_DDDD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DDDD_As_d_DDDD(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  char *__pyx_v_op2;
  char *__pyx_v_op3;
  __pyx_t_double_complex __pyx_v_ov0;
  __pyx_t_double_complex __pyx_v_ov1;
  __pyx_t_double_complex __pyx_v_ov2;
  __pyx_t_double_complex __pyx_v_ov3;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":809
 * 
 * cdef void loop_i_d_DDDD_As_d_DDDD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":810
 * cdef void loop_i_d_DDDD_As_d_DDDD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":811
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":812
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":813
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":814
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":815
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op3 = args[4]
 *     cdef double complex ov0
 */
  __pyx_v_op2 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":816
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     cdef double complex ov1
 */
  __pyx_v_op3 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":821
 *     cdef double complex ov2
 *     cdef double complex ov3
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":822
 *     cdef double complex ov3
 *     for i in range(n):
 *         (<int(*)(double, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1, &ov2, &ov3)             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1
 */
    (void)(((int (*)(double, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1), (&__pyx_v_ov2), (&__pyx_v_ov3)));

    /* "scipy/special/_ufuncs.pyx":823
 *     for i in range(n):
 *         (<int(*)(double, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         (<double complex *>op2)[0] = <double complex>ov2
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":824
 *         (<int(*)(double, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1             # <<<<<<<<<<<<<<
 *         (<double complex *>op2)[0] = <double complex>ov2
 *         (<double complex *>op3)[0] = <double complex>ov3
 */
    (((__pyx_t_double_complex *)__pyx_v_op1)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov1)), ((double)__Pyx_CIMAG(__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":825
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         (<double complex *>op2)[0] = <double complex>ov2             # <<<<<<<<<<<<<<
 *         (<double complex *>op3)[0] = <double complex>ov3
 *         ip0 += steps[0]
 */
    (((__pyx_t_double_complex *)__pyx_v_op2)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov2)), ((double)__Pyx_CIMAG(__pyx_v_ov2)));

    /* "scipy/special/_ufuncs.pyx":826
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         (<double complex *>op2)[0] = <double complex>ov2
 *         (<double complex *>op3)[0] = <double complex>ov3             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op3)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov3)), ((double)__Pyx_CIMAG(__pyx_v_ov3)));

    /* "scipy/special/_ufuncs.pyx":827
 *         (<double complex *>op2)[0] = <double complex>ov2
 *         (<double complex *>op3)[0] = <double complex>ov3
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":828
 *         (<double complex *>op3)[0] = <double complex>ov3
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *         op2 += steps[3]
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":829
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *         op2 += steps[3]
 *         op3 += steps[4]
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":830
 *         op0 += steps[1]
 *         op1 += steps[2]
 *         op2 += steps[3]             # <<<<<<<<<<<<<<
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op2 = (__pyx_v_op2 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":831
 *         op1 += steps[2]
 *         op2 += steps[3]
 *         op3 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op3 = (__pyx_v_op3 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":832
 *         op2 += steps[3]
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_d_DDDD_As_f_FFFF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":808
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_DDDD_As_d_DDDD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":834
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_DDDD_As_f_FFFF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DDDD_As_f_FFFF(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  char *__pyx_v_op2;
  char *__pyx_v_op3;
  __pyx_t_double_complex __pyx_v_ov0;
  __pyx_t_double_complex __pyx_v_ov1;
  __pyx_t_double_complex __pyx_v_ov2;
  __pyx_t_double_complex __pyx_v_ov3;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":835
 * 
 * cdef void loop_i_d_DDDD_As_f_FFFF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":836
 * cdef void loop_i_d_DDDD_As_f_FFFF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":837
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":838
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":839
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":840
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":841
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op3 = args[4]
 *     cdef double complex ov0
 */
  __pyx_v_op2 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":842
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     cdef double complex ov1
 */
  __pyx_v_op3 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":847
 *     cdef double complex ov2
 *     cdef double complex ov3
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float complex *>op0)[0] = <float complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":848
 *     cdef double complex ov3
 *     for i in range(n):
 *         (<int(*)(double, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1, &ov2, &ov3)             # <<<<<<<<<<<<<<
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1
 */
    (void)(((int (*)(double, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *, __pyx_t_double_complex *))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1), (&__pyx_v_ov2), (&__pyx_v_ov3)));

    /* "scipy/special/_ufuncs.pyx":849
 *     for i in range(n):
 *         (<int(*)(double, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float complex *>op0)[0] = <float complex>ov0             # <<<<<<<<<<<<<<
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         (<float complex *>op2)[0] = <float complex>ov2
 */
    (((__pyx_t_float_complex *)__pyx_v_op0)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov0)), ((float)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":850
 *         (<int(*)(double, double complex *, double complex *, double complex *, double complex *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1             # <<<<<<<<<<<<<<
 *         (<float complex *>op2)[0] = <float complex>ov2
 *         (<float complex *>op3)[0] = <float complex>ov3
 */
    (((__pyx_t_float_complex *)__pyx_v_op1)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov1)), ((float)__Pyx_CIMAG(__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":851
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         (<float complex *>op2)[0] = <float complex>ov2             # <<<<<<<<<<<<<<
 *         (<float complex *>op3)[0] = <float complex>ov3
 *         ip0 += steps[0]
 */
    (((__pyx_t_float_complex *)__pyx_v_op2)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov2)), ((float)__Pyx_CIMAG(__pyx_v_ov2)));

    /* "scipy/special/_ufuncs.pyx":852
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         (<float complex *>op2)[0] = <float complex>ov2
 *         (<float complex *>op3)[0] = <float complex>ov3             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((__pyx_t_float_complex *)__pyx_v_op3)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov3)), ((float)__Pyx_CIMAG(__pyx_v_ov3)));

    /* "scipy/special/_ufuncs.pyx":853
 *         (<float complex *>op2)[0] = <float complex>ov2
 *         (<float complex *>op3)[0] = <float complex>ov3
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":854
 *         (<float complex *>op3)[0] = <float complex>ov3
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *         op2 += steps[3]
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":855
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *         op2 += steps[3]
 *         op3 += steps[4]
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":856
 *         op0 += steps[1]
 *         op1 += steps[2]
 *         op2 += steps[3]             # <<<<<<<<<<<<<<
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op2 = (__pyx_v_op2 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":857
 *         op1 += steps[2]
 *         op2 += steps[3]
 *         op3 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op3 = (__pyx_v_op3 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":858
 *         op2 += steps[3]
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_d_DD_As_d_DD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":834
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_DDDD_As_f_FFFF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":860
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_DD_As_d_DD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DD_As_d_DD(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  __pyx_t_double_complex __pyx_v_ov0;
  __pyx_t_double_complex __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":861
 * 
 * cdef void loop_i_d_DD_As_d_DD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":862
 * cdef void loop_i_d_DD_As_d_DD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":863
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":864
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":865
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef double complex ov0
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":866
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     cdef double complex ov1
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":869
 *     cdef double complex ov0
 *     cdef double complex ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double complex *, double complex *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1)
 *         (<double complex *>op0)[0] = <double complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":870
 *     cdef double complex ov1
 *     for i in range(n):
 *         (<int(*)(double, double complex *, double complex *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1
 */
    (void)(((int (*)(double, __pyx_t_double_complex *, __pyx_t_double_complex *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":871
 *     for i in range(n):
 *         (<int(*)(double, double complex *, double complex *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1)
 *         (<double complex *>op0)[0] = <double complex>ov0             # <<<<<<<<<<<<<<
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         ip0 += steps[0]
 */
    (((__pyx_t_double_complex *)__pyx_v_op0)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov0)), ((double)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":872
 *         (<int(*)(double, double complex *, double complex *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1)
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((__pyx_t_double_complex *)__pyx_v_op1)[0]) = __pyx_t_double_complex_from_parts(((double)__Pyx_CREAL(__pyx_v_ov1)), ((double)__Pyx_CIMAG(__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":873
 *         (<double complex *>op0)[0] = <double complex>ov0
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":874
 *         (<double complex *>op1)[0] = <double complex>ov1
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":875
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":876
 *         op0 += steps[1]
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_d_DD_As_f_FF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":860
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_DD_As_d_DD(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":878
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_DD_As_f_FF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DD_As_f_FF(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  __pyx_t_double_complex __pyx_v_ov0;
  __pyx_t_double_complex __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":879
 * 
 * cdef void loop_i_d_DD_As_f_FF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":880
 * cdef void loop_i_d_DD_As_f_FF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":881
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":882
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":883
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef double complex ov0
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":884
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double complex ov0
 *     cdef double complex ov1
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":887
 *     cdef double complex ov0
 *     cdef double complex ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double complex *, double complex *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1)
 *         (<float complex *>op0)[0] = <float complex>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":888
 *     cdef double complex ov1
 *     for i in range(n):
 *         (<int(*)(double, double complex *, double complex *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1
 */
    (void)(((int (*)(double, __pyx_t_double_complex *, __pyx_t_double_complex *))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":889
 *     for i in range(n):
 *         (<int(*)(double, double complex *, double complex *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1)
 *         (<float complex *>op0)[0] = <float complex>ov0             # <<<<<<<<<<<<<<
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         ip0 += steps[0]
 */
    (((__pyx_t_float_complex *)__pyx_v_op0)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov0)), ((float)__Pyx_CIMAG(__pyx_v_ov0)));

    /* "scipy/special/_ufuncs.pyx":890
 *         (<int(*)(double, double complex *, double complex *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1)
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((__pyx_t_float_complex *)__pyx_v_op1)[0]) = __pyx_t_float_complex_from_parts(((float)__Pyx_CREAL(__pyx_v_ov1)), ((float)__Pyx_CIMAG(__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":891
 *         (<float complex *>op0)[0] = <float complex>ov0
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":892
 *         (<float complex *>op1)[0] = <float complex>ov1
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":893
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":894
 *         op0 += steps[1]
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_d_dd_As_d_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":878
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_DD_As_f_FF(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":896
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_dd_As_d_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_d_dd(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":897
 * 
 * cdef void loop_i_d_dd_As_d_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":898
 * cdef void loop_i_d_dd_As_d_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":899
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":900
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":901
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":902
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":905
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":906
 *     cdef double ov1
 *     for i in range(n):
 *         (<int(*)(double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 */
    (void)(((int (*)(double, double *, double *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":907
 *     for i in range(n):
 *         (<int(*)(double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":908
 *         (<int(*)(double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((double *)__pyx_v_op1)[0]) = ((double)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":909
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":910
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":911
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":912
 *         op0 += steps[1]
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_d_dd_As_f_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":896
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_dd_As_d_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":914
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_dd_As_f_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_f_ff(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":915
 * 
 * cdef void loop_i_d_dd_As_f_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":916
 * cdef void loop_i_d_dd_As_f_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":917
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":918
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":919
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":920
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":923
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":924
 *     cdef double ov1
 *     for i in range(n):
 *         (<int(*)(double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 */
    (void)(((int (*)(double, double *, double *))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":925
 *     for i in range(n):
 *         (<int(*)(double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":926
 *         (<int(*)(double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((float *)__pyx_v_op1)[0]) = ((float)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":927
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":928
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":929
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));
  }

  /* "scipy/special/_ufuncs.pyx":930
 *         op0 += steps[1]
 *         op1 += steps[2]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_d_dddd_As_d_dddd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":914
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_dd_As_f_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":932
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_dddd_As_d_dddd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dddd_As_d_dddd(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  char *__pyx_v_op2;
  char *__pyx_v_op3;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  double __pyx_v_ov2;
  double __pyx_v_ov3;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":933
 * 
 * cdef void loop_i_d_dddd_As_d_dddd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":934
 * cdef void loop_i_d_dddd_As_d_dddd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":935
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":936
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":937
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":938
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":939
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op3 = args[4]
 *     cdef double ov0
 */
  __pyx_v_op2 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":940
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op3 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":945
 *     cdef double ov2
 *     cdef double ov3
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double *, double *, double *, double *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":946
 *     cdef double ov3
 *     for i in range(n):
 *         (<int(*)(double, double *, double *, double *, double *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1, &ov2, &ov3)             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 */
    (void)(((int (*)(double, double *, double *, double *, double *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1), (&__pyx_v_ov2), (&__pyx_v_ov3)));

    /* "scipy/special/_ufuncs.pyx":947
 *     for i in range(n):
 *         (<int(*)(double, double *, double *, double *, double *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         (<double *>op1)[0] = <double>ov1
 *         (<double *>op2)[0] = <double>ov2
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":948
 *         (<int(*)(double, double *, double *, double *, double *) nogil>func)(<double>(<double*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1             # <<<<<<<<<<<<<<
 *         (<double *>op2)[0] = <double>ov2
 *         (<double *>op3)[0] = <double>ov3
 */
    (((double *)__pyx_v_op1)[0]) = ((double)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":949
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 *         (<double *>op2)[0] = <double>ov2             # <<<<<<<<<<<<<<
 *         (<double *>op3)[0] = <double>ov3
 *         ip0 += steps[0]
 */
    (((double *)__pyx_v_op2)[0]) = ((double)__pyx_v_ov2);

    /* "scipy/special/_ufuncs.pyx":950
 *         (<double *>op1)[0] = <double>ov1
 *         (<double *>op2)[0] = <double>ov2
 *         (<double *>op3)[0] = <double>ov3             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((double *)__pyx_v_op3)[0]) = ((double)__pyx_v_ov3);

    /* "scipy/special/_ufuncs.pyx":951
 *         (<double *>op2)[0] = <double>ov2
 *         (<double *>op3)[0] = <double>ov3
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":952
 *         (<double *>op3)[0] = <double>ov3
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *         op2 += steps[3]
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":953
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *         op2 += steps[3]
 *         op3 += steps[4]
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":954
 *         op0 += steps[1]
 *         op1 += steps[2]
 *         op2 += steps[3]             # <<<<<<<<<<<<<<
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op2 = (__pyx_v_op2 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":955
 *         op1 += steps[2]
 *         op2 += steps[3]
 *         op3 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op3 = (__pyx_v_op3 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":956
 *         op2 += steps[3]
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_d_dddd_As_f_ffff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":932
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_dddd_As_d_dddd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":958
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_dddd_As_f_ffff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dddd_As_f_ffff(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  char *__pyx_v_op2;
  char *__pyx_v_op3;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  double __pyx_v_ov2;
  double __pyx_v_ov3;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":959
 * 
 * cdef void loop_i_d_dddd_As_f_ffff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":960
 * cdef void loop_i_d_dddd_As_f_ffff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":961
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":962
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":963
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":964
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]
 */
  __pyx_v_op1 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":965
 *     cdef char *op0 = args[1]
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op3 = args[4]
 *     cdef double ov0
 */
  __pyx_v_op2 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":966
 *     cdef char *op1 = args[2]
 *     cdef char *op2 = args[3]
 *     cdef char *op3 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op3 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":971
 *     cdef double ov2
 *     cdef double ov3
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double *, double *, double *, double *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":972
 *     cdef double ov3
 *     for i in range(n):
 *         (<int(*)(double, double *, double *, double *, double *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1, &ov2, &ov3)             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 */
    (void)(((int (*)(double, double *, double *, double *, double *))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1), (&__pyx_v_ov2), (&__pyx_v_ov3)));

    /* "scipy/special/_ufuncs.pyx":973
 *     for i in range(n):
 *         (<int(*)(double, double *, double *, double *, double *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         (<float *>op1)[0] = <float>ov1
 *         (<float *>op2)[0] = <float>ov2
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":974
 *         (<int(*)(double, double *, double *, double *, double *) nogil>func)(<double>(<float*>ip0)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1             # <<<<<<<<<<<<<<
 *         (<float *>op2)[0] = <float>ov2
 *         (<float *>op3)[0] = <float>ov3
 */
    (((float *)__pyx_v_op1)[0]) = ((float)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":975
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 *         (<float *>op2)[0] = <float>ov2             # <<<<<<<<<<<<<<
 *         (<float *>op3)[0] = <float>ov3
 *         ip0 += steps[0]
 */
    (((float *)__pyx_v_op2)[0]) = ((float)__pyx_v_ov2);

    /* "scipy/special/_ufuncs.pyx":976
 *         (<float *>op1)[0] = <float>ov1
 *         (<float *>op2)[0] = <float>ov2
 *         (<float *>op3)[0] = <float>ov3             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((float *)__pyx_v_op3)[0]) = ((float)__pyx_v_ov3);

    /* "scipy/special/_ufuncs.pyx":977
 *         (<float *>op2)[0] = <float>ov2
 *         (<float *>op3)[0] = <float>ov3
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *         op1 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":978
 *         (<float *>op3)[0] = <float>ov3
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *         op1 += steps[2]
 *         op2 += steps[3]
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":979
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *         op1 += steps[2]             # <<<<<<<<<<<<<<
 *         op2 += steps[3]
 *         op3 += steps[4]
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":980
 *         op0 += steps[1]
 *         op1 += steps[2]
 *         op2 += steps[3]             # <<<<<<<<<<<<<<
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op2 = (__pyx_v_op2 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":981
 *         op1 += steps[2]
 *         op2 += steps[3]
 *         op3 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op3 = (__pyx_v_op3 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":982
 *         op2 += steps[3]
 *         op3 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_dd_dd_As_dd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":958
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_d_dddd_As_f_ffff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":984
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_dd_dd_As_dd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dd_As_dd_dd(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":985
 * 
 * cdef void loop_i_dd_dd_As_dd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":986
 * cdef void loop_i_dd_dd_As_dd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":987
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":988
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":989
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef char *op1 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":990
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[3]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":991
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 *     cdef char *op1 = args[3]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":994
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":995
 *     cdef double ov1
 *     for i in range(n):
 *         (<int(*)(double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 */
    (void)(((int (*)(double, double, double *, double *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":996
 *     for i in range(n):
 *         (<int(*)(double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":997
 *         (<int(*)(double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op1)[0]) = ((double)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":998
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":999
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *         op1 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":1000
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *         op1 += steps[3]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":1001
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *         op1 += steps[3]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[3]));
  }

  /* "scipy/special/_ufuncs.pyx":1002
 *         op0 += steps[2]
 *         op1 += steps[3]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_dd_dd_As_ff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":984
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_dd_dd_As_dd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":1004
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_dd_dd_As_ff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dd_As_ff_ff(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":1005
 * 
 * cdef void loop_i_dd_dd_As_ff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":1006
 * cdef void loop_i_dd_dd_As_ff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":1007
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":1008
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":1009
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef char *op1 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":1010
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[3]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":1011
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 *     cdef char *op1 = args[3]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":1014
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":1015
 *     cdef double ov1
 *     for i in range(n):
 *         (<int(*)(double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 */
    (void)(((int (*)(double, double, double *, double *))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":1016
 *     for i in range(n):
 *         (<int(*)(double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":1017
 *         (<int(*)(double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((float *)__pyx_v_op1)[0]) = ((float)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":1018
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":1019
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *         op1 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":1020
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *         op1 += steps[3]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":1021
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *         op1 += steps[3]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[3]));
  }

  /* "scipy/special/_ufuncs.pyx":1022
 *         op0 += steps[2]
 *         op1 += steps[3]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_dd_dddd_As_dd_dddd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":1004
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_dd_dd_As_ff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":1024
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_dd_dddd_As_dd_dddd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dddd_As_dd_dddd(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  char *__pyx_v_op2;
  char *__pyx_v_op3;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  double __pyx_v_ov2;
  double __pyx_v_ov3;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":1025
 * 
 * cdef void loop_i_dd_dddd_As_dd_dddd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":1026
 * cdef void loop_i_dd_dddd_As_dd_dddd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":1027
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":1028
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":1029
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef char *op1 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":1030
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[3]
 *     cdef char *op2 = args[4]
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":1031
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 *     cdef char *op1 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op2 = args[4]
 *     cdef char *op3 = args[5]
 */
  __pyx_v_op1 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":1032
 *     cdef char *op0 = args[2]
 *     cdef char *op1 = args[3]
 *     cdef char *op2 = args[4]             # <<<<<<<<<<<<<<
 *     cdef char *op3 = args[5]
 *     cdef double ov0
 */
  __pyx_v_op2 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":1033
 *     cdef char *op1 = args[3]
 *     cdef char *op2 = args[4]
 *     cdef char *op3 = args[5]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op3 = (__pyx_v_args[5]);

  /* "scipy/special/_ufuncs.pyx":1038
 *     cdef double ov2
 *     cdef double ov3
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double, double *, double *, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":1039
 *     cdef double ov3
 *     for i in range(n):
 *         (<int(*)(double, double, double *, double *, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], &ov0, &ov1, &ov2, &ov3)             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 */
    (void)(((int (*)(double, double, double *, double *, double *, double *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1), (&__pyx_v_ov2), (&__pyx_v_ov3)));

    /* "scipy/special/_ufuncs.pyx":1040
 *     for i in range(n):
 *         (<int(*)(double, double, double *, double *, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         (<double *>op1)[0] = <double>ov1
 *         (<double *>op2)[0] = <double>ov2
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":1041
 *         (<int(*)(double, double, double *, double *, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1             # <<<<<<<<<<<<<<
 *         (<double *>op2)[0] = <double>ov2
 *         (<double *>op3)[0] = <double>ov3
 */
    (((double *)__pyx_v_op1)[0]) = ((double)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":1042
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 *         (<double *>op2)[0] = <double>ov2             # <<<<<<<<<<<<<<
 *         (<double *>op3)[0] = <double>ov3
 *         ip0 += steps[0]
 */
    (((double *)__pyx_v_op2)[0]) = ((double)__pyx_v_ov2);

    /* "scipy/special/_ufuncs.pyx":1043
 *         (<double *>op1)[0] = <double>ov1
 *         (<double *>op2)[0] = <double>ov2
 *         (<double *>op3)[0] = <double>ov3             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op3)[0]) = ((double)__pyx_v_ov3);

    /* "scipy/special/_ufuncs.pyx":1044
 *         (<double *>op2)[0] = <double>ov2
 *         (<double *>op3)[0] = <double>ov3
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":1045
 *         (<double *>op3)[0] = <double>ov3
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *         op1 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":1046
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *         op1 += steps[3]
 *         op2 += steps[4]
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":1047
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *         op1 += steps[3]             # <<<<<<<<<<<<<<
 *         op2 += steps[4]
 *         op3 += steps[5]
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":1048
 *         op0 += steps[2]
 *         op1 += steps[3]
 *         op2 += steps[4]             # <<<<<<<<<<<<<<
 *         op3 += steps[5]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op2 = (__pyx_v_op2 + (__pyx_v_steps[4]));

    /* "scipy/special/_ufuncs.pyx":1049
 *         op1 += steps[3]
 *         op2 += steps[4]
 *         op3 += steps[5]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op3 = (__pyx_v_op3 + (__pyx_v_steps[5]));
  }

  /* "scipy/special/_ufuncs.pyx":1050
 *         op2 += steps[4]
 *         op3 += steps[5]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_dd_dddd_As_ff_ffff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":1024
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_dd_dddd_As_dd_dddd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":1052
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_dd_dddd_As_ff_ffff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dddd_As_ff_ffff(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  char *__pyx_v_op2;
  char *__pyx_v_op3;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  double __pyx_v_ov2;
  double __pyx_v_ov3;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":1053
 * 
 * cdef void loop_i_dd_dddd_As_ff_ffff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":1054
 * cdef void loop_i_dd_dddd_As_ff_ffff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":1055
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":1056
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":1057
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[2]
 *     cdef char *op1 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":1058
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[3]
 *     cdef char *op2 = args[4]
 */
  __pyx_v_op0 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":1059
 *     cdef char *ip1 = args[1]
 *     cdef char *op0 = args[2]
 *     cdef char *op1 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op2 = args[4]
 *     cdef char *op3 = args[5]
 */
  __pyx_v_op1 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":1060
 *     cdef char *op0 = args[2]
 *     cdef char *op1 = args[3]
 *     cdef char *op2 = args[4]             # <<<<<<<<<<<<<<
 *     cdef char *op3 = args[5]
 *     cdef double ov0
 */
  __pyx_v_op2 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":1061
 *     cdef char *op1 = args[3]
 *     cdef char *op2 = args[4]
 *     cdef char *op3 = args[5]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op3 = (__pyx_v_args[5]);

  /* "scipy/special/_ufuncs.pyx":1066
 *     cdef double ov2
 *     cdef double ov3
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double, double *, double *, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":1067
 *     cdef double ov3
 *     for i in range(n):
 *         (<int(*)(double, double, double *, double *, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], &ov0, &ov1, &ov2, &ov3)             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 */
    (void)(((int (*)(double, double, double *, double *, double *, double *))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1), (&__pyx_v_ov2), (&__pyx_v_ov3)));

    /* "scipy/special/_ufuncs.pyx":1068
 *     for i in range(n):
 *         (<int(*)(double, double, double *, double *, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         (<float *>op1)[0] = <float>ov1
 *         (<float *>op2)[0] = <float>ov2
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":1069
 *         (<int(*)(double, double, double *, double *, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], &ov0, &ov1, &ov2, &ov3)
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1             # <<<<<<<<<<<<<<
 *         (<float *>op2)[0] = <float>ov2
 *         (<float *>op3)[0] = <float>ov3
 */
    (((float *)__pyx_v_op1)[0]) = ((float)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":1070
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 *         (<float *>op2)[0] = <float>ov2             # <<<<<<<<<<<<<<
 *         (<float *>op3)[0] = <float>ov3
 *         ip0 += steps[0]
 */
    (((float *)__pyx_v_op2)[0]) = ((float)__pyx_v_ov2);

    /* "scipy/special/_ufuncs.pyx":1071
 *         (<float *>op1)[0] = <float>ov1
 *         (<float *>op2)[0] = <float>ov2
 *         (<float *>op3)[0] = <float>ov3             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((float *)__pyx_v_op3)[0]) = ((float)__pyx_v_ov3);

    /* "scipy/special/_ufuncs.pyx":1072
 *         (<float *>op2)[0] = <float>ov2
 *         (<float *>op3)[0] = <float>ov3
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         op0 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":1073
 *         (<float *>op3)[0] = <float>ov3
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         op0 += steps[2]
 *         op1 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":1074
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         op0 += steps[2]             # <<<<<<<<<<<<<<
 *         op1 += steps[3]
 *         op2 += steps[4]
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":1075
 *         ip1 += steps[1]
 *         op0 += steps[2]
 *         op1 += steps[3]             # <<<<<<<<<<<<<<
 *         op2 += steps[4]
 *         op3 += steps[5]
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":1076
 *         op0 += steps[2]
 *         op1 += steps[3]
 *         op2 += steps[4]             # <<<<<<<<<<<<<<
 *         op3 += steps[5]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op2 = (__pyx_v_op2 + (__pyx_v_steps[4]));

    /* "scipy/special/_ufuncs.pyx":1077
 *         op1 += steps[3]
 *         op2 += steps[4]
 *         op3 += steps[5]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op3 = (__pyx_v_op3 + (__pyx_v_steps[5]));
  }

  /* "scipy/special/_ufuncs.pyx":1078
 *         op2 += steps[4]
 *         op3 += steps[5]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_ddd_dd_As_ddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":1052
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_dd_dddd_As_ff_ffff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":1080
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_ddd_dd_As_ddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_ddd_dd(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":1081
 * 
 * cdef void loop_i_ddd_dd_As_ddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":1082
 * cdef void loop_i_ddd_dd_As_ddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":1083
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":1084
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":1085
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":1086
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[3]
 *     cdef char *op1 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":1087
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[4]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":1088
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 *     cdef char *op1 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":1091
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":1092
 *     cdef double ov1
 *     for i in range(n):
 *         (<int(*)(double, double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 */
    (void)(((int (*)(double, double, double, double *, double *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":1093
 *     for i in range(n):
 *         (<int(*)(double, double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":1094
 *         (<int(*)(double, double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op1)[0]) = ((double)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":1095
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":1096
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         op0 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":1097
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         op0 += steps[3]
 *         op1 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":1098
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         op0 += steps[3]             # <<<<<<<<<<<<<<
 *         op1 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":1099
 *         ip2 += steps[2]
 *         op0 += steps[3]
 *         op1 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":1100
 *         op0 += steps[3]
 *         op1 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_ddd_dd_As_fff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":1080
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_ddd_dd_As_ddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":1102
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_ddd_dd_As_fff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_fff_ff(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":1103
 * 
 * cdef void loop_i_ddd_dd_As_fff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":1104
 * cdef void loop_i_ddd_dd_As_fff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":1105
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":1106
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":1107
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":1108
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[3]
 *     cdef char *op1 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":1109
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[4]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":1110
 *     cdef char *ip2 = args[2]
 *     cdef char *op0 = args[3]
 *     cdef char *op1 = args[4]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":1113
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":1114
 *     cdef double ov1
 *     for i in range(n):
 *         (<int(*)(double, double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 */
    (void)(((int (*)(double, double, double, double *, double *))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), ((double)(((float *)__pyx_v_ip2)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":1115
 *     for i in range(n):
 *         (<int(*)(double, double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":1116
 *         (<int(*)(double, double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((float *)__pyx_v_op1)[0]) = ((float)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":1117
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":1118
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         op0 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":1119
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         op0 += steps[3]
 *         op1 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":1120
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         op0 += steps[3]             # <<<<<<<<<<<<<<
 *         op1 += steps[4]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":1121
 *         ip2 += steps[2]
 *         op0 += steps[3]
 *         op1 += steps[4]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[4]));
  }

  /* "scipy/special/_ufuncs.pyx":1122
 *         op0 += steps[3]
 *         op1 += steps[4]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_ddddd_dd_As_ddddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":1102
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_ddd_dd_As_fff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":1124
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_ddddd_dd_As_ddddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_ddddd_dd(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_ip4;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":1125
 * 
 * cdef void loop_i_ddddd_dd_As_ddddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":1126
 * cdef void loop_i_ddddd_dd_As_ddddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":1127
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":1128
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":1129
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":1130
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":1131
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *ip4 = args[4]
 *     cdef char *op0 = args[5]
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":1132
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[5]
 *     cdef char *op1 = args[6]
 */
  __pyx_v_ip4 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":1133
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]
 *     cdef char *op0 = args[5]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[6]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[5]);

  /* "scipy/special/_ufuncs.pyx":1134
 *     cdef char *ip4 = args[4]
 *     cdef char *op0 = args[5]
 *     cdef char *op1 = args[6]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[6]);

  /* "scipy/special/_ufuncs.pyx":1137
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double, double, double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], <double>(<double*>ip4)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":1138
 *     cdef double ov1
 *     for i in range(n):
 *         (<int(*)(double, double, double, double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], <double>(<double*>ip4)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 */
    (void)(((int (*)(double, double, double, double, double, double *, double *))__pyx_v_func)(((double)(((double *)__pyx_v_ip0)[0])), ((double)(((double *)__pyx_v_ip1)[0])), ((double)(((double *)__pyx_v_ip2)[0])), ((double)(((double *)__pyx_v_ip3)[0])), ((double)(((double *)__pyx_v_ip4)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":1139
 *     for i in range(n):
 *         (<int(*)(double, double, double, double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], <double>(<double*>ip4)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0             # <<<<<<<<<<<<<<
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 */
    (((double *)__pyx_v_op0)[0]) = ((double)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":1140
 *         (<int(*)(double, double, double, double, double, double *, double *) nogil>func)(<double>(<double*>ip0)[0], <double>(<double*>ip1)[0], <double>(<double*>ip2)[0], <double>(<double*>ip3)[0], <double>(<double*>ip4)[0], &ov0, &ov1)
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((double *)__pyx_v_op1)[0]) = ((double)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":1141
 *         (<double *>op0)[0] = <double>ov0
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":1142
 *         (<double *>op1)[0] = <double>ov1
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":1143
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         ip4 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":1144
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         ip4 += steps[4]
 *         op0 += steps[5]
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":1145
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         ip4 += steps[4]             # <<<<<<<<<<<<<<
 *         op0 += steps[5]
 *         op1 += steps[6]
 */
    __pyx_v_ip4 = (__pyx_v_ip4 + (__pyx_v_steps[4]));

    /* "scipy/special/_ufuncs.pyx":1146
 *         ip3 += steps[3]
 *         ip4 += steps[4]
 *         op0 += steps[5]             # <<<<<<<<<<<<<<
 *         op1 += steps[6]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[5]));

    /* "scipy/special/_ufuncs.pyx":1147
 *         ip4 += steps[4]
 *         op0 += steps[5]
 *         op1 += steps[6]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[6]));
  }

  /* "scipy/special/_ufuncs.pyx":1148
 *         op0 += steps[5]
 *         op1 += steps[6]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_ddddd_dd_As_fffff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":1124
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_ddddd_dd_As_ddddd_dd(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":1150
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_ddddd_dd_As_fffff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_fffff_ff(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_ip1;
  char *__pyx_v_ip2;
  char *__pyx_v_ip3;
  char *__pyx_v_ip4;
  char *__pyx_v_op0;
  char *__pyx_v_op1;
  double __pyx_v_ov0;
  double __pyx_v_ov1;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;

  /* "scipy/special/_ufuncs.pyx":1151
 * 
 * cdef void loop_i_ddddd_dd_As_fffff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":1152
 * cdef void loop_i_ddddd_dd_As_fffff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":1153
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":1154
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":1155
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 */
  __pyx_v_ip1 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":1156
 *     cdef char *ip0 = args[0]
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]             # <<<<<<<<<<<<<<
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]
 */
  __pyx_v_ip2 = (__pyx_v_args[2]);

  /* "scipy/special/_ufuncs.pyx":1157
 *     cdef char *ip1 = args[1]
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]             # <<<<<<<<<<<<<<
 *     cdef char *ip4 = args[4]
 *     cdef char *op0 = args[5]
 */
  __pyx_v_ip3 = (__pyx_v_args[3]);

  /* "scipy/special/_ufuncs.pyx":1158
 *     cdef char *ip2 = args[2]
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[5]
 *     cdef char *op1 = args[6]
 */
  __pyx_v_ip4 = (__pyx_v_args[4]);

  /* "scipy/special/_ufuncs.pyx":1159
 *     cdef char *ip3 = args[3]
 *     cdef char *ip4 = args[4]
 *     cdef char *op0 = args[5]             # <<<<<<<<<<<<<<
 *     cdef char *op1 = args[6]
 *     cdef double ov0
 */
  __pyx_v_op0 = (__pyx_v_args[5]);

  /* "scipy/special/_ufuncs.pyx":1160
 *     cdef char *ip4 = args[4]
 *     cdef char *op0 = args[5]
 *     cdef char *op1 = args[6]             # <<<<<<<<<<<<<<
 *     cdef double ov0
 *     cdef double ov1
 */
  __pyx_v_op1 = (__pyx_v_args[6]);

  /* "scipy/special/_ufuncs.pyx":1163
 *     cdef double ov0
 *     cdef double ov1
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         (<int(*)(double, double, double, double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], <double>(<float*>ip4)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":1164
 *     cdef double ov1
 *     for i in range(n):
 *         (<int(*)(double, double, double, double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], <double>(<float*>ip4)[0], &ov0, &ov1)             # <<<<<<<<<<<<<<
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 */
    (void)(((int (*)(double, double, double, double, double, double *, double *))__pyx_v_func)(((double)(((float *)__pyx_v_ip0)[0])), ((double)(((float *)__pyx_v_ip1)[0])), ((double)(((float *)__pyx_v_ip2)[0])), ((double)(((float *)__pyx_v_ip3)[0])), ((double)(((float *)__pyx_v_ip4)[0])), (&__pyx_v_ov0), (&__pyx_v_ov1)));

    /* "scipy/special/_ufuncs.pyx":1165
 *     for i in range(n):
 *         (<int(*)(double, double, double, double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], <double>(<float*>ip4)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0             # <<<<<<<<<<<<<<
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]
 */
    (((float *)__pyx_v_op0)[0]) = ((float)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":1166
 *         (<int(*)(double, double, double, double, double, double *, double *) nogil>func)(<double>(<float*>ip0)[0], <double>(<float*>ip1)[0], <double>(<float*>ip2)[0], <double>(<float*>ip3)[0], <double>(<float*>ip4)[0], &ov0, &ov1)
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 */
    (((float *)__pyx_v_op1)[0]) = ((float)__pyx_v_ov1);

    /* "scipy/special/_ufuncs.pyx":1167
 *         (<float *>op0)[0] = <float>ov0
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":1168
 *         (<float *>op1)[0] = <float>ov1
 *         ip0 += steps[0]
 *         ip1 += steps[1]             # <<<<<<<<<<<<<<
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 */
    __pyx_v_ip1 = (__pyx_v_ip1 + (__pyx_v_steps[1]));

    /* "scipy/special/_ufuncs.pyx":1169
 *         ip0 += steps[0]
 *         ip1 += steps[1]
 *         ip2 += steps[2]             # <<<<<<<<<<<<<<
 *         ip3 += steps[3]
 *         ip4 += steps[4]
 */
    __pyx_v_ip2 = (__pyx_v_ip2 + (__pyx_v_steps[2]));

    /* "scipy/special/_ufuncs.pyx":1170
 *         ip1 += steps[1]
 *         ip2 += steps[2]
 *         ip3 += steps[3]             # <<<<<<<<<<<<<<
 *         ip4 += steps[4]
 *         op0 += steps[5]
 */
    __pyx_v_ip3 = (__pyx_v_ip3 + (__pyx_v_steps[3]));

    /* "scipy/special/_ufuncs.pyx":1171
 *         ip2 += steps[2]
 *         ip3 += steps[3]
 *         ip4 += steps[4]             # <<<<<<<<<<<<<<
 *         op0 += steps[5]
 *         op1 += steps[6]
 */
    __pyx_v_ip4 = (__pyx_v_ip4 + (__pyx_v_steps[4]));

    /* "scipy/special/_ufuncs.pyx":1172
 *         ip3 += steps[3]
 *         ip4 += steps[4]
 *         op0 += steps[5]             # <<<<<<<<<<<<<<
 *         op1 += steps[6]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[5]));

    /* "scipy/special/_ufuncs.pyx":1173
 *         ip4 += steps[4]
 *         op0 += steps[5]
 *         op1 += steps[6]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op1 = (__pyx_v_op1 + (__pyx_v_steps[6]));
  }

  /* "scipy/special/_ufuncs.pyx":1174
 *         op0 += steps[5]
 *         op1 += steps[6]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * cdef void loop_i_i__As_l_l(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":1150
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_ddddd_dd_As_fffff_ff(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "scipy/special/_ufuncs.pyx":1176
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_i__As_l_l(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

static void __pyx_f_5scipy_7special_7_ufuncs_loop_i_i__As_l_l(char **__pyx_v_args, npy_intp *__pyx_v_dims, npy_intp *__pyx_v_steps, void *__pyx_v_data) {
  CYTHON_UNUSED npy_intp __pyx_v_i;
  npy_intp __pyx_v_n;
  void *__pyx_v_func;
  char *__pyx_v_func_name;
  char *__pyx_v_ip0;
  char *__pyx_v_op0;
  int __pyx_v_ov0;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;
  int __pyx_t_4;

  /* "scipy/special/_ufuncs.pyx":1177
 * 
 * cdef void loop_i_i__As_l_l(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]             # <<<<<<<<<<<<<<
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 */
  __pyx_v_n = (__pyx_v_dims[0]);

  /* "scipy/special/_ufuncs.pyx":1178
 * cdef void loop_i_i__As_l_l(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]             # <<<<<<<<<<<<<<
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 */
  __pyx_v_func = (((void **)__pyx_v_data)[0]);

  /* "scipy/special/_ufuncs.pyx":1179
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]             # <<<<<<<<<<<<<<
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]
 */
  __pyx_v_func_name = ((char *)(((void **)__pyx_v_data)[1]));

  /* "scipy/special/_ufuncs.pyx":1180
 *     cdef void *func = (<void**>data)[0]
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]             # <<<<<<<<<<<<<<
 *     cdef char *op0 = args[1]
 *     cdef int ov0
 */
  __pyx_v_ip0 = (__pyx_v_args[0]);

  /* "scipy/special/_ufuncs.pyx":1181
 *     cdef char *func_name = <char*>(<void**>data)[1]
 *     cdef char *ip0 = args[0]
 *     cdef char *op0 = args[1]             # <<<<<<<<<<<<<<
 *     cdef int ov0
 *     for i in range(n):
 */
  __pyx_v_op0 = (__pyx_v_args[1]);

  /* "scipy/special/_ufuncs.pyx":1183
 *     cdef char *op0 = args[1]
 *     cdef int ov0
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0]:
 *             ov0 = (<int(*)(int) nogil>func)(<int>(<long*>ip0)[0])
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "scipy/special/_ufuncs.pyx":1184
 *     cdef int ov0
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<int(*)(int) nogil>func)(<int>(<long*>ip0)[0])
 *         else:
 */
    __pyx_t_4 = ((((int)(((long *)__pyx_v_ip0)[0])) == (((long *)__pyx_v_ip0)[0])) != 0);
    if (__pyx_t_4) {

      /* "scipy/special/_ufuncs.pyx":1185
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0]:
 *             ov0 = (<int(*)(int) nogil>func)(<int>(<long*>ip0)[0])             # <<<<<<<<<<<<<<
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 */
      __pyx_v_ov0 = ((int (*)(int))__pyx_v_func)(((int)(((long *)__pyx_v_ip0)[0])));

      /* "scipy/special/_ufuncs.pyx":1184
 *     cdef int ov0
 *     for i in range(n):
 *         if <int>(<long*>ip0)[0] == (<long*>ip0)[0]:             # <<<<<<<<<<<<<<
 *             ov0 = (<int(*)(int) nogil>func)(<int>(<long*>ip0)[0])
 *         else:
 */
      goto __pyx_L5;
    }

    /* "scipy/special/_ufuncs.pyx":1187
 *             ov0 = (<int(*)(int) nogil>func)(<int>(<long*>ip0)[0])
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")             # <<<<<<<<<<<<<<
 *             ov0 = <int>0xbad0bad0
 *         (<long *>op0)[0] = <long>ov0
 */
    /*else*/ {
      sf_error(__pyx_v_func_name, SF_ERROR_DOMAIN, ((char *)"invalid input argument"));

      /* "scipy/special/_ufuncs.pyx":1188
 *         else:
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <int>0xbad0bad0             # <<<<<<<<<<<<<<
 *         (<long *>op0)[0] = <long>ov0
 *         ip0 += steps[0]
 */
      __pyx_v_ov0 = ((int)0xbad0bad0);
    }
    __pyx_L5:;

    /* "scipy/special/_ufuncs.pyx":1189
 *             sf_error.error(func_name, sf_error.DOMAIN, "invalid input argument")
 *             ov0 = <int>0xbad0bad0
 *         (<long *>op0)[0] = <long>ov0             # <<<<<<<<<<<<<<
 *         ip0 += steps[0]
 *         op0 += steps[1]
 */
    (((long *)__pyx_v_op0)[0]) = ((long)__pyx_v_ov0);

    /* "scipy/special/_ufuncs.pyx":1190
 *             ov0 = <int>0xbad0bad0
 *         (<long *>op0)[0] = <long>ov0
 *         ip0 += steps[0]             # <<<<<<<<<<<<<<
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)
 */
    __pyx_v_ip0 = (__pyx_v_ip0 + (__pyx_v_steps[0]));

    /* "scipy/special/_ufuncs.pyx":1191
 *         (<long *>op0)[0] = <long>ov0
 *         ip0 += steps[0]
 *         op0 += steps[1]             # <<<<<<<<<<<<<<
 *     sf_error.check_fpe(func_name)
 * 
 */
    __pyx_v_op0 = (__pyx_v_op0 + (__pyx_v_steps[1]));
  }

  /* "scipy/special/_ufuncs.pyx":1192
 *         ip0 += steps[0]
 *         op0 += steps[1]
 *     sf_error.check_fpe(func_name)             # <<<<<<<<<<<<<<
 * 
 * from ._trig cimport ccospi as _func_ccospi
 */
  sf_error_check_fpe(__pyx_v_func_name);

  /* "scipy/special/_ufuncs.pyx":1176
 *     sf_error.check_fpe(func_name)
 * 
 * cdef void loop_i_i__As_l_l(char **args, np.npy_intp *dims, np.npy_intp *steps, void *data) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_intp i, n = dims[0]
 *     cdef void *func = (<void**>data)[0]
 */

  /* function exit code */
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":258
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fulfill the PEP.
 */

/* Python wrapper */
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5numpy_7ndarray___getbuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_v_i;
  int __pyx_v_ndim;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  int __pyx_v_t;
  char *__pyx_v_f;
  PyArray_Descr *__pyx_v_descr = 0;
  int __pyx_v_offset;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyArray_Descr *__pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  char *__pyx_t_9;
  if (__pyx_v_info == NULL) {
    PyErr_SetString(PyExc_BufferError, "PyObject_GetBuffer: view==NULL argument is obsolete");
    return -1;
  }
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(__pyx_v_info->obj);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":265
 * 
 *             cdef int i, ndim
 *             cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 */
  __pyx_v_endian_detector = 1;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":266
 *             cdef int i, ndim
 *             cdef int endian_detector = 1
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 * 
 *             ndim = PyArray_NDIM(self)
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":268
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 *             ndim = PyArray_NDIM(self)             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 */
  __pyx_v_ndim = PyArray_NDIM(__pyx_v_self);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":270
 *             ndim = PyArray_NDIM(self)
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_C_CONTIGUOUS) == PyBUF_C_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":271
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_C_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_ARRAY_C_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":270
 *             ndim = PyArray_NDIM(self)
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  if (unlikely(__pyx_t_1)) {

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":272
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 272, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 272, __pyx_L1_error)

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":270
 *             ndim = PyArray_NDIM(self)
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":274
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_F_CONTIGUOUS) == PyBUF_F_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L7_bool_binop_done;
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":275
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_F_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_ARRAY_F_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L7_bool_binop_done:;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":274
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  if (unlikely(__pyx_t_1)) {

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":276
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 276, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 276, __pyx_L1_error)

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":274
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":278
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 *             info.buf = PyArray_DATA(self)             # <<<<<<<<<<<<<<
 *             info.ndim = ndim
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_v_info->buf = PyArray_DATA(__pyx_v_self);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":279
 * 
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim             # <<<<<<<<<<<<<<
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 # Allocate new buffer for strides and shape info.
 */
  __pyx_v_info->ndim = __pyx_v_ndim;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":280
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":283
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>PyObject_Malloc(sizeof(Py_ssize_t) * 2 * <size_t>ndim)             # <<<<<<<<<<<<<<
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 */
    __pyx_v_info->strides = ((Py_ssize_t *)PyObject_Malloc((((sizeof(Py_ssize_t)) * 2) * ((size_t)__pyx_v_ndim))));

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":284
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>PyObject_Malloc(sizeof(Py_ssize_t) * 2 * <size_t>ndim)
 *                 info.shape = info.strides + ndim             # <<<<<<<<<<<<<<
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 */
    __pyx_v_info->shape = (__pyx_v_info->strides + __pyx_v_ndim);

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":285
 *                 info.strides = <Py_ssize_t*>PyObject_Malloc(sizeof(Py_ssize_t) * 2 * <size_t>ndim)
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):             # <<<<<<<<<<<<<<
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 */
    __pyx_t_4 = __pyx_v_ndim;
    __pyx_t_5 = __pyx_t_4;
    for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {
      __pyx_v_i = __pyx_t_6;

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":286
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]             # <<<<<<<<<<<<<<
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 */
      (__pyx_v_info->strides[__pyx_v_i]) = (PyArray_STRIDES(__pyx_v_self)[__pyx_v_i]);

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":287
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]             # <<<<<<<<<<<<<<
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 */
      (__pyx_v_info->shape[__pyx_v_i]) = (PyArray_DIMS(__pyx_v_self)[__pyx_v_i]);
    }

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":280
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 */
    goto __pyx_L9;
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":289
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)             # <<<<<<<<<<<<<<
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 */
  /*else*/ {
    __pyx_v_info->strides = ((Py_ssize_t *)PyArray_STRIDES(__pyx_v_self));

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":290
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)             # <<<<<<<<<<<<<<
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 */
    __pyx_v_info->shape = ((Py_ssize_t *)PyArray_DIMS(__pyx_v_self));
  }
  __pyx_L9:;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":291
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL             # <<<<<<<<<<<<<<
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 */
  __pyx_v_info->suboffsets = NULL;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":292
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)             # <<<<<<<<<<<<<<
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 * 
 */
  __pyx_v_info->itemsize = PyArray_ITEMSIZE(__pyx_v_self);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":293
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)             # <<<<<<<<<<<<<<
 * 
 *             cdef int t
 */
  __pyx_v_info->readonly = (!(PyArray_ISWRITEABLE(__pyx_v_self) != 0));

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":296
 * 
 *             cdef int t
 *             cdef char* f = NULL             # <<<<<<<<<<<<<<
 *             cdef dtype descr = <dtype>PyArray_DESCR(self)
 *             cdef int offset
 */
  __pyx_v_f = NULL;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":297
 *             cdef int t
 *             cdef char* f = NULL
 *             cdef dtype descr = <dtype>PyArray_DESCR(self)             # <<<<<<<<<<<<<<
 *             cdef int offset
 * 
 */
  __pyx_t_7 = PyArray_DESCR(__pyx_v_self);
  __pyx_t_3 = ((PyObject *)__pyx_t_7);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_v_descr = ((PyArray_Descr *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":300
 *             cdef int offset
 * 
 *             info.obj = self             # <<<<<<<<<<<<<<
 * 
 *             if not PyDataType_HASFIELDS(descr):
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  __Pyx_GOTREF(__pyx_v_info->obj);
  __Pyx_DECREF(__pyx_v_info->obj);
  __pyx_v_info->obj = ((PyObject *)__pyx_v_self);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":302
 *             info.obj = self
 * 
 *             if not PyDataType_HASFIELDS(descr):             # <<<<<<<<<<<<<<
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 */
  __pyx_t_1 = ((!(PyDataType_HASFIELDS(__pyx_v_descr) != 0)) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":303
 * 
 *             if not PyDataType_HASFIELDS(descr):
 *                 t = descr.type_num             # <<<<<<<<<<<<<<
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 */
    __pyx_t_4 = __pyx_v_descr->type_num;
    __pyx_v_t = __pyx_t_4;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":304
 *             if not PyDataType_HASFIELDS(descr):
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '>') != 0);
    if (!__pyx_t_2) {
      goto __pyx_L15_next_or;
    } else {
    }
    __pyx_t_2 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L14_bool_binop_done;
    }
    __pyx_L15_next_or:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":305
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '<') != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L14_bool_binop_done;
    }
    __pyx_t_2 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L14_bool_binop_done:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":304
 *             if not PyDataType_HASFIELDS(descr):
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    if (unlikely(__pyx_t_1)) {

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":306
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 306, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(2, 306, __pyx_L1_error)

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":304
 *             if not PyDataType_HASFIELDS(descr):
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    }

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":307
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 */
    switch (__pyx_v_t) {
      case NPY_BYTE:
      __pyx_v_f = ((char *)"b");
      break;
      case NPY_UBYTE:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":308
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 */
      __pyx_v_f = ((char *)"B");
      break;
      case NPY_SHORT:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":309
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 */
      __pyx_v_f = ((char *)"h");
      break;
      case NPY_USHORT:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":310
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 */
      __pyx_v_f = ((char *)"H");
      break;
      case NPY_INT:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":311
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 */
      __pyx_v_f = ((char *)"i");
      break;
      case NPY_UINT:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":312
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 */
      __pyx_v_f = ((char *)"I");
      break;
      case NPY_LONG:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":313
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 */
      __pyx_v_f = ((char *)"l");
      break;
      case NPY_ULONG:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":314
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 */
      __pyx_v_f = ((char *)"L");
      break;
      case NPY_LONGLONG:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":315
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 */
      __pyx_v_f = ((char *)"q");
      break;
      case NPY_ULONGLONG:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":316
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 */
      __pyx_v_f = ((char *)"Q");
      break;
      case NPY_FLOAT:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":317
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 */
      __pyx_v_f = ((char *)"f");
      break;
      case NPY_DOUBLE:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":318
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 */
      __pyx_v_f = ((char *)"d");
      break;
      case NPY_LONGDOUBLE:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":319
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 */
      __pyx_v_f = ((char *)"g");
      break;
      case NPY_CFLOAT:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":320
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 */
      __pyx_v_f = ((char *)"Zf");
      break;
      case NPY_CDOUBLE:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":321
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"
 */
      __pyx_v_f = ((char *)"Zd");
      break;
      case NPY_CLONGDOUBLE:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":322
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 */
      __pyx_v_f = ((char *)"Zg");
      break;
      case NPY_OBJECT:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":323
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"             # <<<<<<<<<<<<<<
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      __pyx_v_f = ((char *)"O");
      break;
      default:

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":325
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *                 info.format = f
 *                 return
 */
      __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_t); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 325, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_8 = PyUnicode_Format(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_t_3); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 325, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 325, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(2, 325, __pyx_L1_error)
      break;
    }

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":326
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f             # <<<<<<<<<<<<<<
 *                 return
 *             else:
 */
    __pyx_v_info->format = __pyx_v_f;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":327
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f
 *                 return             # <<<<<<<<<<<<<<
 *             else:
 *                 info.format = <char*>PyObject_Malloc(_buffer_format_string_len)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":302
 *             info.obj = self
 * 
 *             if not PyDataType_HASFIELDS(descr):             # <<<<<<<<<<<<<<
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 */
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":329
 *                 return
 *             else:
 *                 info.format = <char*>PyObject_Malloc(_buffer_format_string_len)             # <<<<<<<<<<<<<<
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 */
  /*else*/ {
    __pyx_v_info->format = ((char *)PyObject_Malloc(0xFF));

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":330
 *             else:
 *                 info.format = <char*>PyObject_Malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment             # <<<<<<<<<<<<<<
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,
 */
    (__pyx_v_info->format[0]) = '^';

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":331
 *                 info.format = <char*>PyObject_Malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0             # <<<<<<<<<<<<<<
 *                 f = _util_dtypestring(descr, info.format + 1,
 *                                       info.format + _buffer_format_string_len,
 */
    __pyx_v_offset = 0;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":332
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,             # <<<<<<<<<<<<<<
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 */
    __pyx_t_9 = __pyx_f_5numpy__util_dtypestring(__pyx_v_descr, (__pyx_v_info->format + 1), (__pyx_v_info->format + 0xFF), (&__pyx_v_offset)); if (unlikely(__pyx_t_9 == ((char *)NULL))) __PYX_ERR(2, 332, __pyx_L1_error)
    __pyx_v_f = __pyx_t_9;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":335
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 *                 f[0] = c'\0' # Terminate format string             # <<<<<<<<<<<<<<
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 */
    (__pyx_v_f[0]) = '\x00';
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":258
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fulfill the PEP.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.ndarray.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  if (__pyx_v_info->obj != NULL) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;
  }
  goto __pyx_L2;
  __pyx_L0:;
  if (__pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;
  }
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_descr);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":337
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 PyObject_Free(info.format)
 */

/* Python wrapper */
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info); /*proto*/
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__releasebuffer__ (wrapper)", 0);
  __pyx_pf_5numpy_7ndarray_2__releasebuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__releasebuffer__", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":338
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):             # <<<<<<<<<<<<<<
 *                 PyObject_Free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_t_1 = (PyArray_HASFIELDS(__pyx_v_self) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":339
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):
 *                 PyObject_Free(info.format)             # <<<<<<<<<<<<<<
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 PyObject_Free(info.strides)
 */
    PyObject_Free(__pyx_v_info->format);

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":338
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):             # <<<<<<<<<<<<<<
 *                 PyObject_Free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":340
 *             if PyArray_HASFIELDS(self):
 *                 PyObject_Free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 PyObject_Free(info.strides)
 *                 # info.shape was stored after info.strides in the same block
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":341
 *                 PyObject_Free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 PyObject_Free(info.strides)             # <<<<<<<<<<<<<<
 *                 # info.shape was stored after info.strides in the same block
 * 
 */
    PyObject_Free(__pyx_v_info->strides);

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":340
 *             if PyArray_HASFIELDS(self):
 *                 PyObject_Free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 PyObject_Free(info.strides)
 *                 # info.shape was stored after info.strides in the same block
 */
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":337
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 PyObject_Free(info.format)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":821
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":822
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 822, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":821
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":824
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":825
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 825, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":824
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":827
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":828
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 828, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":827
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":830
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":831
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":830
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":833
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":834
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 834, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":833
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":836
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyDataType_SHAPE(PyArray_Descr *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("PyDataType_SHAPE", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":837
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  __pyx_t_1 = (PyDataType_HASSUBARRAY(__pyx_v_d) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":838
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape             # <<<<<<<<<<<<<<
 *     else:
 *         return ()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject*)__pyx_v_d->subarray->shape));
    __pyx_r = ((PyObject*)__pyx_v_d->subarray->shape);
    goto __pyx_L0;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":837
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":840
 *         return <tuple>d.subarray.shape
 *     else:
 *         return ()             # <<<<<<<<<<<<<<
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_empty_tuple);
    __pyx_r = __pyx_empty_tuple;
    goto __pyx_L0;
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":836
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":842
 *         return ()
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *__pyx_v_descr, char *__pyx_v_f, char *__pyx_v_end, int *__pyx_v_offset) {
  PyArray_Descr *__pyx_v_child = 0;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  PyObject *__pyx_v_fields = 0;
  PyObject *__pyx_v_childname = NULL;
  PyObject *__pyx_v_new_offset = NULL;
  PyObject *__pyx_v_t = NULL;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  long __pyx_t_8;
  char *__pyx_t_9;
  __Pyx_RefNannySetupContext("_util_dtypestring", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":847
 * 
 *     cdef dtype child
 *     cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 *     cdef tuple fields
 */
  __pyx_v_endian_detector = 1;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":848
 *     cdef dtype child
 *     cdef int endian_detector = 1
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 *     cdef tuple fields
 * 
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":851
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  if (unlikely(__pyx_v_descr->names == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(2, 851, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_descr->names; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(2, 851, __pyx_L1_error)
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 851, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_childname, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":852
 * 
 *     for childname in descr.names:
 *         fields = descr.fields[childname]             # <<<<<<<<<<<<<<
 *         child, new_offset = fields
 * 
 */
    if (unlikely(__pyx_v_descr->fields == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 852, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_descr->fields, __pyx_v_childname); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 852, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyTuple_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(2, 852, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_fields, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":853
 *     for childname in descr.names:
 *         fields = descr.fields[childname]
 *         child, new_offset = fields             # <<<<<<<<<<<<<<
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 */
    if (likely(__pyx_v_fields != Py_None)) {
      PyObject* sequence = __pyx_v_fields;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(2, 853, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      #else
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 853, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 853, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      #endif
    } else {
      __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(2, 853, __pyx_L1_error)
    }
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_dtype))))) __PYX_ERR(2, 853, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_child, ((PyArray_Descr *)__pyx_t_3));
    __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_new_offset, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":855
 *         child, new_offset = fields
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 */
    __pyx_t_4 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 855, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = PyNumber_Subtract(__pyx_v_new_offset, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 855, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 855, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = ((((__pyx_v_end - __pyx_v_f) - ((int)__pyx_t_5)) < 15) != 0);
    if (unlikely(__pyx_t_6)) {

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":856
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 856, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(2, 856, __pyx_L1_error)

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":855
 *         child, new_offset = fields
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 */
    }

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":858
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '>') != 0);
    if (!__pyx_t_7) {
      goto __pyx_L8_next_or;
    } else {
    }
    __pyx_t_7 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_L8_next_or:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":859
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *             raise ValueError(u"Non-native byte order not supported")
 *             # One could encode it in the format string and have Cython
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '<') != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_7 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L7_bool_binop_done:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":858
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    if (unlikely(__pyx_t_6)) {

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":860
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *             # One could encode it in the format string and have Cython
 *             # complain instead, BUT: < and > in format strings also imply
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 860, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(2, 860, __pyx_L1_error)

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":858
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    }

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":870
 * 
 *         # Output padding bytes
 *         while offset[0] < new_offset:             # <<<<<<<<<<<<<<
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 */
    while (1) {
      __pyx_t_3 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 870, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_v_new_offset, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 870, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(2, 870, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (!__pyx_t_6) break;

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":871
 *         # Output padding bytes
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte             # <<<<<<<<<<<<<<
 *             f += 1
 *             offset[0] += 1
 */
      (__pyx_v_f[0]) = 0x78;

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":872
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte
 *             f += 1             # <<<<<<<<<<<<<<
 *             offset[0] += 1
 * 
 */
      __pyx_v_f = (__pyx_v_f + 1);

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":873
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 *             offset[0] += 1             # <<<<<<<<<<<<<<
 * 
 *         offset[0] += child.itemsize
 */
      __pyx_t_8 = 0;
      (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + 1);
    }

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":875
 *             offset[0] += 1
 * 
 *         offset[0] += child.itemsize             # <<<<<<<<<<<<<<
 * 
 *         if not PyDataType_HASFIELDS(child):
 */
    __pyx_t_8 = 0;
    (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + __pyx_v_child->elsize);

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":877
 *         offset[0] += child.itemsize
 * 
 *         if not PyDataType_HASFIELDS(child):             # <<<<<<<<<<<<<<
 *             t = child.type_num
 *             if end - f < 5:
 */
    __pyx_t_6 = ((!(PyDataType_HASFIELDS(__pyx_v_child) != 0)) != 0);
    if (__pyx_t_6) {

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":878
 * 
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num             # <<<<<<<<<<<<<<
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")
 */
      __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_child->type_num); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 878, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_XDECREF_SET(__pyx_v_t, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":879
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num
 *             if end - f < 5:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError(u"Format string allocated too short.")
 * 
 */
      __pyx_t_6 = (((__pyx_v_end - __pyx_v_f) < 5) != 0);
      if (unlikely(__pyx_t_6)) {

        /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":880
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 880, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_Raise(__pyx_t_4, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __PYX_ERR(2, 880, __pyx_L1_error)

        /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":879
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num
 *             if end - f < 5:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError(u"Format string allocated too short.")
 * 
 */
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":883
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_BYTE); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 883, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 883, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(2, 883, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 98;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":884
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"             # <<<<<<<<<<<<<<
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_UBYTE); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 884, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 884, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(2, 884, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 66;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":885
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"             # <<<<<<<<<<<<<<
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_SHORT); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 885, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 885, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(2, 885, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x68;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":886
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"             # <<<<<<<<<<<<<<
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_USHORT); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 886, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 886, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(2, 886, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 72;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":887
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_INT); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 887, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 887, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(2, 887, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x69;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":888
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_UINT); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 888, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 888, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(2, 888, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 73;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":889
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONG); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 889, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 889, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(2, 889, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x6C;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":890
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_ULONG); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 890, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 890, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(2, 890, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 76;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":891
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONGLONG); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 891, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 891, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(2, 891, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x71;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":892
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_ULONGLONG); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 892, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 892, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(2, 892, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 81;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":893
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"             # <<<<<<<<<<<<<<
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_FLOAT); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 893, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 893, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(2, 893, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x66;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":894
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_DOUBLE); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 894, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 894, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(2, 894, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x64;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":895
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"             # <<<<<<<<<<<<<<
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONGDOUBLE); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 895, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 895, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(2, 895, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x67;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":896
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf             # <<<<<<<<<<<<<<
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CFLOAT); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 896, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 896, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(2, 896, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x66;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":897
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd             # <<<<<<<<<<<<<<
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CDOUBLE); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 897, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 897, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(2, 897, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x64;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":898
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg             # <<<<<<<<<<<<<<
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CLONGDOUBLE); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 898, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 898, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(2, 898, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x67;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":899
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"             # <<<<<<<<<<<<<<
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_OBJECT); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 899, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 899, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(2, 899, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (likely(__pyx_t_6)) {
        (__pyx_v_f[0]) = 79;
        goto __pyx_L15;
      }

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":901
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *             f += 1
 *         else:
 */
      /*else*/ {
        __pyx_t_3 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_v_t); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 901, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 901, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_Raise(__pyx_t_4, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __PYX_ERR(2, 901, __pyx_L1_error)
      }
      __pyx_L15:;

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":902
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *             f += 1             # <<<<<<<<<<<<<<
 *         else:
 *             # Cython ignores struct boundary information ("T{...}"),
 */
      __pyx_v_f = (__pyx_v_f + 1);

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":877
 *         offset[0] += child.itemsize
 * 
 *         if not PyDataType_HASFIELDS(child):             # <<<<<<<<<<<<<<
 *             t = child.type_num
 *             if end - f < 5:
 */
      goto __pyx_L13;
    }

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":906
 *             # Cython ignores struct boundary information ("T{...}"),
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)             # <<<<<<<<<<<<<<
 *     return f
 * 
 */
    /*else*/ {
      __pyx_t_9 = __pyx_f_5numpy__util_dtypestring(__pyx_v_child, __pyx_v_f, __pyx_v_end, __pyx_v_offset); if (unlikely(__pyx_t_9 == ((char *)NULL))) __PYX_ERR(2, 906, __pyx_L1_error)
      __pyx_v_f = __pyx_t_9;
    }
    __pyx_L13:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":851
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":907
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)
 *     return f             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_f;
  goto __pyx_L0;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":842
 *         return ()
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("numpy._util_dtypestring", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child);
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XDECREF(__pyx_v_childname);
  __Pyx_XDECREF(__pyx_v_new_offset);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1022
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1023
 * 
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!             # <<<<<<<<<<<<<<
 *     PyArray_SetBaseObject(arr, base)
 * 
 */
  Py_INCREF(__pyx_v_base);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1024
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  (void)(PyArray_SetBaseObject(__pyx_v_arr, __pyx_v_base));

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1022
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1026
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_v_base;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1027
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)             # <<<<<<<<<<<<<<
 *     if base is NULL:
 *         return None
 */
  __pyx_v_base = PyArray_BASE(__pyx_v_arr);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1028
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  __pyx_t_1 = ((__pyx_v_base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1029
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     return <object>base
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1028
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1030
 *     if base is NULL:
 *         return None
 *     return <object>base             # <<<<<<<<<<<<<<
 * 
 * # Versions of the import_* functions which are more suitable for
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_base));
  __pyx_r = ((PyObject *)__pyx_v_base);
  goto __pyx_L0;

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1026
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1034
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_array()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_array(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("import_array", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1035
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_array()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1036
 * cdef inline int import_array() except -1:
 *     try:
 *         _import_array()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")
 */
      __pyx_t_4 = _import_array(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 1036, __pyx_L3_error)

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1035
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_array()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1037
 *     try:
 *         _import_array()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 1037, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1038
 *         _import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 1038, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 1038, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1035
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_array()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1034
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_array()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1040
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_umath(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("import_umath", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1041
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1042
 * cdef inline int import_umath() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 1042, __pyx_L3_error)

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1041
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1043
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 1043, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1044
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 1044, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 1044, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1041
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1040
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1046
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_ufunc(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("import_ufunc", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1047
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1048
 * cdef inline int import_ufunc() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(2, 1048, __pyx_L3_error)

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1047
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1049
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(2, 1049, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);

      /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1050
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 1050, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(2, 1050, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1047
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1046
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "sf_error.pxd":28
 * 
 * 
 * cdef inline int _sf_error_test_function(int code) nogil:             # <<<<<<<<<<<<<<
 *     """Function that can raise every sf_error category for testing
 *     purposes.
 */

static CYTHON_INLINE int __pyx_f_5scipy_7special_8sf_error__sf_error_test_function(int __pyx_v_code) {
  sf_error_t __pyx_v_sf_error_code;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "sf_error.pxd":35
 *     cdef sf_error_t sf_error_code
 * 
 *     if code < 0 or code >= 10:             # <<<<<<<<<<<<<<
 *         sf_error_code = OTHER
 *     else:
 */
  __pyx_t_2 = ((__pyx_v_code < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_code >= 10) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "sf_error.pxd":36
 * 
 *     if code < 0 or code >= 10:
 *         sf_error_code = OTHER             # <<<<<<<<<<<<<<
 *     else:
 *         sf_error_code = <sf_error_t>code
 */
    __pyx_v_sf_error_code = SF_ERROR_OTHER;

    /* "sf_error.pxd":35
 *     cdef sf_error_t sf_error_code
 * 
 *     if code < 0 or code >= 10:             # <<<<<<<<<<<<<<
 *         sf_error_code = OTHER
 *     else:
 */
    goto __pyx_L3;
  }

  /* "sf_error.pxd":38
 *         sf_error_code = OTHER
 *     else:
 *         sf_error_code = <sf_error_t>code             # <<<<<<<<<<<<<<
 *     error('_err_test_function', sf_error_code, NULL)
 *     return 0
 */
  /*else*/ {
    __pyx_v_sf_error_code = ((sf_error_t)__pyx_v_code);
  }
  __pyx_L3:;

  /* "sf_error.pxd":39
 *     else:
 *         sf_error_code = <sf_error_t>code
 *     error('_err_test_function', sf_error_code, NULL)             # <<<<<<<<<<<<<<
 *     return 0
 */
  sf_error(((char *)"_err_test_function"), __pyx_v_sf_error_code, NULL);

  /* "sf_error.pxd":40
 *         sf_error_code = <sf_error_t>code
 *     error('_err_test_function', sf_error_code, NULL)
 *     return 0             # <<<<<<<<<<<<<<
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "sf_error.pxd":28
 * 
 * 
 * cdef inline int _sf_error_test_function(int code) nogil:             # <<<<<<<<<<<<<<
 *     """Function that can raise every sf_error category for testing
 *     purposes.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":39
 *     double_complex c99
 * 
 * cdef inline np.npy_cdouble npy_cdouble_from_double_complex(             # <<<<<<<<<<<<<<
 *         double_complex x) nogil:
 *     cdef _complex_pun z
 */

static CYTHON_INLINE npy_cdouble __pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_t_double_complex __pyx_v_x) {
  __pyx_t_5scipy_7special_13_complexstuff__complex_pun __pyx_v_z;
  npy_cdouble __pyx_r;

  /* "_complexstuff.pxd":42
 *         double_complex x) nogil:
 *     cdef _complex_pun z
 *     z.c99 = x             # <<<<<<<<<<<<<<
 *     return z.npy
 * 
 */
  __pyx_v_z.c99 = __pyx_v_x;

  /* "_complexstuff.pxd":43
 *     cdef _complex_pun z
 *     z.c99 = x
 *     return z.npy             # <<<<<<<<<<<<<<
 * 
 * cdef inline double_complex double_complex_from_npy_cdouble(
 */
  __pyx_r = __pyx_v_z.npy;
  goto __pyx_L0;

  /* "_complexstuff.pxd":39
 *     double_complex c99
 * 
 * cdef inline np.npy_cdouble npy_cdouble_from_double_complex(             # <<<<<<<<<<<<<<
 *         double_complex x) nogil:
 *     cdef _complex_pun z
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":45
 *     return z.npy
 * 
 * cdef inline double_complex double_complex_from_npy_cdouble(             # <<<<<<<<<<<<<<
 *         np.npy_cdouble x) nogil:
 *     cdef _complex_pun z
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_13_complexstuff_double_complex_from_npy_cdouble(npy_cdouble __pyx_v_x) {
  __pyx_t_5scipy_7special_13_complexstuff__complex_pun __pyx_v_z;
  __pyx_t_double_complex __pyx_r;

  /* "_complexstuff.pxd":48
 *         np.npy_cdouble x) nogil:
 *     cdef _complex_pun z
 *     z.npy = x             # <<<<<<<<<<<<<<
 *     return z.c99
 * 
 */
  __pyx_v_z.npy = __pyx_v_x;

  /* "_complexstuff.pxd":49
 *     cdef _complex_pun z
 *     z.npy = x
 *     return z.c99             # <<<<<<<<<<<<<<
 * 
 * cdef inline bint zisnan(number_t x) nogil:
 */
  __pyx_r = __pyx_v_z.c99;
  goto __pyx_L0;

  /* "_complexstuff.pxd":45
 *     return z.npy
 * 
 * cdef inline double_complex double_complex_from_npy_cdouble(             # <<<<<<<<<<<<<<
 *         np.npy_cdouble x) nogil:
 *     cdef _complex_pun z
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":51
 *     return z.c99
 * 
 * cdef inline bint zisnan(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return npy_isnan(x.real) or npy_isnan(x.imag)
 */

static CYTHON_INLINE int __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zisnan(double __pyx_v_x) {
  int __pyx_r;

  /* "_complexstuff.pxd":55
 *         return npy_isnan(x.real) or npy_isnan(x.imag)
 *     else:
 *         return npy_isnan(x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline bint zisfinite(number_t x) nogil:
 */
  __pyx_r = npy_isnan(__pyx_v_x);
  goto __pyx_L0;

  /* "_complexstuff.pxd":51
 *     return z.c99
 * 
 * cdef inline bint zisnan(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return npy_isnan(x.real) or npy_isnan(x.imag)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisnan(__pyx_t_double_complex __pyx_v_x) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_complexstuff.pxd":53
 * cdef inline bint zisnan(number_t x) nogil:
 *     if number_t is double_complex:
 *         return npy_isnan(x.real) or npy_isnan(x.imag)             # <<<<<<<<<<<<<<
 *     else:
 *         return npy_isnan(x)
 */
  __pyx_t_2 = (npy_isnan(__Pyx_CREAL(__pyx_v_x)) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = (npy_isnan(__Pyx_CIMAG(__pyx_v_x)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "_complexstuff.pxd":51
 *     return z.c99
 * 
 * cdef inline bint zisnan(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return npy_isnan(x.real) or npy_isnan(x.imag)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":57
 *         return npy_isnan(x)
 * 
 * cdef inline bint zisfinite(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return npy_isfinite(x.real) and npy_isfinite(x.imag)
 */

static CYTHON_INLINE int __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zisfinite(double __pyx_v_x) {
  int __pyx_r;

  /* "_complexstuff.pxd":61
 *         return npy_isfinite(x.real) and npy_isfinite(x.imag)
 *     else:
 *         return npy_isfinite(x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline bint zisinf(number_t x) nogil:
 */
  __pyx_r = npy_isfinite(__pyx_v_x);
  goto __pyx_L0;

  /* "_complexstuff.pxd":57
 *         return npy_isnan(x)
 * 
 * cdef inline bint zisfinite(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return npy_isfinite(x.real) and npy_isfinite(x.imag)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisfinite(__pyx_t_double_complex __pyx_v_x) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_complexstuff.pxd":59
 * cdef inline bint zisfinite(number_t x) nogil:
 *     if number_t is double_complex:
 *         return npy_isfinite(x.real) and npy_isfinite(x.imag)             # <<<<<<<<<<<<<<
 *     else:
 *         return npy_isfinite(x)
 */
  __pyx_t_2 = (npy_isfinite(__Pyx_CREAL(__pyx_v_x)) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = (npy_isfinite(__Pyx_CIMAG(__pyx_v_x)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "_complexstuff.pxd":57
 *         return npy_isnan(x)
 * 
 * cdef inline bint zisfinite(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return npy_isfinite(x.real) and npy_isfinite(x.imag)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":63
 *         return npy_isfinite(x)
 * 
 * cdef inline bint zisinf(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return not zisnan(x) and not zisfinite(x)
 */

static CYTHON_INLINE int __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zisinf(double __pyx_v_x) {
  int __pyx_r;

  /* "_complexstuff.pxd":67
 *         return not zisnan(x) and not zisfinite(x)
 *     else:
 *         return npy_isinf(x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double zreal(number_t x) nogil:
 */
  __pyx_r = npy_isinf(__pyx_v_x);
  goto __pyx_L0;

  /* "_complexstuff.pxd":63
 *         return npy_isfinite(x)
 * 
 * cdef inline bint zisinf(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return not zisnan(x) and not zisfinite(x)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisinf(__pyx_t_double_complex __pyx_v_x) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_complexstuff.pxd":65
 * cdef inline bint zisinf(number_t x) nogil:
 *     if number_t is double_complex:
 *         return not zisnan(x) and not zisfinite(x)             # <<<<<<<<<<<<<<
 *     else:
 *         return npy_isinf(x)
 */
  __pyx_t_2 = ((!(__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisnan(__pyx_v_x) != 0)) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = ((!(__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisfinite(__pyx_v_x) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "_complexstuff.pxd":63
 *         return npy_isfinite(x)
 * 
 * cdef inline bint zisinf(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return not zisnan(x) and not zisfinite(x)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":69
 *         return npy_isinf(x)
 * 
 * cdef inline double zreal(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return x.real
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zreal(double __pyx_v_x) {
  double __pyx_r;

  /* "_complexstuff.pxd":73
 *         return x.real
 *     else:
 *         return x             # <<<<<<<<<<<<<<
 * 
 * cdef inline double zabs(number_t x) nogil:
 */
  __pyx_r = __pyx_v_x;
  goto __pyx_L0;

  /* "_complexstuff.pxd":69
 *         return npy_isinf(x)
 * 
 * cdef inline double zreal(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return x.real
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE double __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zreal(__pyx_t_double_complex __pyx_v_x) {
  double __pyx_r;

  /* "_complexstuff.pxd":71
 * cdef inline double zreal(number_t x) nogil:
 *     if number_t is double_complex:
 *         return x.real             # <<<<<<<<<<<<<<
 *     else:
 *         return x
 */
  __pyx_r = __Pyx_CREAL(__pyx_v_x);
  goto __pyx_L0;

  /* "_complexstuff.pxd":69
 *         return npy_isinf(x)
 * 
 * cdef inline double zreal(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return x.real
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":75
 *         return x
 * 
 * cdef inline double zabs(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return npy_cabs(npy_cdouble_from_double_complex(x))
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zabs(double __pyx_v_x) {
  double __pyx_r;

  /* "_complexstuff.pxd":79
 *         return npy_cabs(npy_cdouble_from_double_complex(x))
 *     else:
 *         return libc.math.fabs(x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double zarg(double complex x) nogil:
 */
  __pyx_r = fabs(__pyx_v_x);
  goto __pyx_L0;

  /* "_complexstuff.pxd":75
 *         return x
 * 
 * cdef inline double zabs(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return npy_cabs(npy_cdouble_from_double_complex(x))
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE double __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_t_double_complex __pyx_v_x) {
  double __pyx_r;

  /* "_complexstuff.pxd":77
 * cdef inline double zabs(number_t x) nogil:
 *     if number_t is double_complex:
 *         return npy_cabs(npy_cdouble_from_double_complex(x))             # <<<<<<<<<<<<<<
 *     else:
 *         return libc.math.fabs(x)
 */
  __pyx_r = npy_cabs(__pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_x));
  goto __pyx_L0;

  /* "_complexstuff.pxd":75
 *         return x
 * 
 * cdef inline double zabs(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double_complex:
 *         return npy_cabs(npy_cdouble_from_double_complex(x))
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":81
 *         return libc.math.fabs(x)
 * 
 * cdef inline double zarg(double complex x) nogil:             # <<<<<<<<<<<<<<
 *     return npy_carg(npy_cdouble_from_double_complex(x))
 * 
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_13_complexstuff_zarg(__pyx_t_double_complex __pyx_v_x) {
  double __pyx_r;

  /* "_complexstuff.pxd":82
 * 
 * cdef inline double zarg(double complex x) nogil:
 *     return npy_carg(npy_cdouble_from_double_complex(x))             # <<<<<<<<<<<<<<
 * 
 * cdef inline number_t zlog(number_t x) nogil:
 */
  __pyx_r = npy_carg(__pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_x));
  goto __pyx_L0;

  /* "_complexstuff.pxd":81
 *         return libc.math.fabs(x)
 * 
 * cdef inline double zarg(double complex x) nogil:             # <<<<<<<<<<<<<<
 *     return npy_carg(npy_cdouble_from_double_complex(x))
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":84
 *     return npy_carg(npy_cdouble_from_double_complex(x))
 * 
 * cdef inline number_t zlog(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zlog(double __pyx_v_x) {
  double __pyx_r;

  /* "_complexstuff.pxd":90
 *         return double_complex_from_npy_cdouble(r)
 *     else:
 *         return libc.math.log(x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline number_t zexp(number_t x) nogil:
 */
  __pyx_r = log(__pyx_v_x);
  goto __pyx_L0;

  /* "_complexstuff.pxd":84
 *     return npy_carg(npy_cdouble_from_double_complex(x))
 * 
 * cdef inline number_t zlog(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zlog(__pyx_t_double_complex __pyx_v_x) {
  npy_cdouble __pyx_v_r;
  __pyx_t_double_complex __pyx_r;

  /* "_complexstuff.pxd":87
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 *         r = npy_clog(npy_cdouble_from_double_complex(x))             # <<<<<<<<<<<<<<
 *         return double_complex_from_npy_cdouble(r)
 *     else:
 */
  __pyx_v_r = npy_clog(__pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_x));

  /* "_complexstuff.pxd":88
 *     if number_t is double_complex:
 *         r = npy_clog(npy_cdouble_from_double_complex(x))
 *         return double_complex_from_npy_cdouble(r)             # <<<<<<<<<<<<<<
 *     else:
 *         return libc.math.log(x)
 */
  __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_double_complex_from_npy_cdouble(__pyx_v_r);
  goto __pyx_L0;

  /* "_complexstuff.pxd":84
 *     return npy_carg(npy_cdouble_from_double_complex(x))
 * 
 * cdef inline number_t zlog(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":92
 *         return libc.math.log(x)
 * 
 * cdef inline number_t zexp(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zexp(double __pyx_v_x) {
  double __pyx_r;

  /* "_complexstuff.pxd":98
 *         return double_complex_from_npy_cdouble(r)
 *     else:
 *         return libc.math.exp(x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline number_t zsin(number_t x) nogil:
 */
  __pyx_r = exp(__pyx_v_x);
  goto __pyx_L0;

  /* "_complexstuff.pxd":92
 *         return libc.math.log(x)
 * 
 * cdef inline number_t zexp(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zexp(__pyx_t_double_complex __pyx_v_x) {
  npy_cdouble __pyx_v_r;
  __pyx_t_double_complex __pyx_r;

  /* "_complexstuff.pxd":95
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 *         r = npy_cexp(npy_cdouble_from_double_complex(x))             # <<<<<<<<<<<<<<
 *         return double_complex_from_npy_cdouble(r)
 *     else:
 */
  __pyx_v_r = npy_cexp(__pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_x));

  /* "_complexstuff.pxd":96
 *     if number_t is double_complex:
 *         r = npy_cexp(npy_cdouble_from_double_complex(x))
 *         return double_complex_from_npy_cdouble(r)             # <<<<<<<<<<<<<<
 *     else:
 *         return libc.math.exp(x)
 */
  __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_double_complex_from_npy_cdouble(__pyx_v_r);
  goto __pyx_L0;

  /* "_complexstuff.pxd":92
 *         return libc.math.log(x)
 * 
 * cdef inline number_t zexp(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":100
 *         return libc.math.exp(x)
 * 
 * cdef inline number_t zsin(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zsin(double __pyx_v_x) {
  double __pyx_r;

  /* "_complexstuff.pxd":106
 *         return double_complex_from_npy_cdouble(r)
 *     else:
 *         return libc.math.sin(x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline number_t zcos(number_t x) nogil:
 */
  __pyx_r = sin(__pyx_v_x);
  goto __pyx_L0;

  /* "_complexstuff.pxd":100
 *         return libc.math.exp(x)
 * 
 * cdef inline number_t zsin(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zsin(__pyx_t_double_complex __pyx_v_x) {
  npy_cdouble __pyx_v_r;
  __pyx_t_double_complex __pyx_r;

  /* "_complexstuff.pxd":103
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 *         r = npy_csin(npy_cdouble_from_double_complex(x))             # <<<<<<<<<<<<<<
 *         return double_complex_from_npy_cdouble(r)
 *     else:
 */
  __pyx_v_r = npy_csin(__pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_x));

  /* "_complexstuff.pxd":104
 *     if number_t is double_complex:
 *         r = npy_csin(npy_cdouble_from_double_complex(x))
 *         return double_complex_from_npy_cdouble(r)             # <<<<<<<<<<<<<<
 *     else:
 *         return libc.math.sin(x)
 */
  __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_double_complex_from_npy_cdouble(__pyx_v_r);
  goto __pyx_L0;

  /* "_complexstuff.pxd":100
 *         return libc.math.exp(x)
 * 
 * cdef inline number_t zsin(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":108
 *         return libc.math.sin(x)
 * 
 * cdef inline number_t zcos(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zcos(double __pyx_v_x) {
  double __pyx_r;

  /* "_complexstuff.pxd":114
 *         return double_complex_from_npy_cdouble(r)
 *     else:
 *         return libc.math.cos(x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline number_t zsqrt(number_t x) nogil:
 */
  __pyx_r = cos(__pyx_v_x);
  goto __pyx_L0;

  /* "_complexstuff.pxd":108
 *         return libc.math.sin(x)
 * 
 * cdef inline number_t zcos(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zcos(__pyx_t_double_complex __pyx_v_x) {
  npy_cdouble __pyx_v_r;
  __pyx_t_double_complex __pyx_r;

  /* "_complexstuff.pxd":111
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 *         r = npy_ccos(npy_cdouble_from_double_complex(x))             # <<<<<<<<<<<<<<
 *         return double_complex_from_npy_cdouble(r)
 *     else:
 */
  __pyx_v_r = npy_ccos(__pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_x));

  /* "_complexstuff.pxd":112
 *     if number_t is double_complex:
 *         r = npy_ccos(npy_cdouble_from_double_complex(x))
 *         return double_complex_from_npy_cdouble(r)             # <<<<<<<<<<<<<<
 *     else:
 *         return libc.math.cos(x)
 */
  __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_double_complex_from_npy_cdouble(__pyx_v_r);
  goto __pyx_L0;

  /* "_complexstuff.pxd":108
 *         return libc.math.sin(x)
 * 
 * cdef inline number_t zcos(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":116
 *         return libc.math.cos(x)
 * 
 * cdef inline number_t zsqrt(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zsqrt(double __pyx_v_x) {
  double __pyx_r;

  /* "_complexstuff.pxd":122
 *         return double_complex_from_npy_cdouble(r)
 *     else:
 *         return libc.math.sqrt(x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline number_t zpow(number_t x, double y) nogil:
 */
  __pyx_r = sqrt(__pyx_v_x);
  goto __pyx_L0;

  /* "_complexstuff.pxd":116
 *         return libc.math.cos(x)
 * 
 * cdef inline number_t zsqrt(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zsqrt(__pyx_t_double_complex __pyx_v_x) {
  npy_cdouble __pyx_v_r;
  __pyx_t_double_complex __pyx_r;

  /* "_complexstuff.pxd":119
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 *         r = npy_csqrt(npy_cdouble_from_double_complex(x))             # <<<<<<<<<<<<<<
 *         return double_complex_from_npy_cdouble(r)
 *     else:
 */
  __pyx_v_r = npy_csqrt(__pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_x));

  /* "_complexstuff.pxd":120
 *     if number_t is double_complex:
 *         r = npy_csqrt(npy_cdouble_from_double_complex(x))
 *         return double_complex_from_npy_cdouble(r)             # <<<<<<<<<<<<<<
 *     else:
 *         return libc.math.sqrt(x)
 */
  __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_double_complex_from_npy_cdouble(__pyx_v_r);
  goto __pyx_L0;

  /* "_complexstuff.pxd":116
 *         return libc.math.cos(x)
 * 
 * cdef inline number_t zsqrt(number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     if number_t is double_complex:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":124
 *         return libc.math.sqrt(x)
 * 
 * cdef inline number_t zpow(number_t x, double y) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r, z
 *     # FIXME
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zpow(double __pyx_v_x, double __pyx_v_y) {
  double __pyx_r;

  /* "_complexstuff.pxd":133
 *         return double_complex_from_npy_cdouble(r)
 *     else:
 *         return libc.math.pow(x, y)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double_complex zpack(double zr, double zi) nogil:
 */
  __pyx_r = pow(__pyx_v_x, __pyx_v_y);
  goto __pyx_L0;

  /* "_complexstuff.pxd":124
 *         return libc.math.sqrt(x)
 * 
 * cdef inline number_t zpow(number_t x, double y) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r, z
 *     # FIXME
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zpow(__pyx_t_double_complex __pyx_v_x, double __pyx_v_y) {
  npy_cdouble __pyx_v_r;
  npy_cdouble __pyx_v_z;
  __pyx_t_double_complex __pyx_r;

  /* "_complexstuff.pxd":128
 *     # FIXME
 *     if number_t is double_complex:
 *         z.real = y             # <<<<<<<<<<<<<<
 *         z.imag = 0.0
 *         r = npy_cpow(npy_cdouble_from_double_complex(x), z)
 */
  __pyx_v_z.real = __pyx_v_y;

  /* "_complexstuff.pxd":129
 *     if number_t is double_complex:
 *         z.real = y
 *         z.imag = 0.0             # <<<<<<<<<<<<<<
 *         r = npy_cpow(npy_cdouble_from_double_complex(x), z)
 *         return double_complex_from_npy_cdouble(r)
 */
  __pyx_v_z.imag = 0.0;

  /* "_complexstuff.pxd":130
 *         z.real = y
 *         z.imag = 0.0
 *         r = npy_cpow(npy_cdouble_from_double_complex(x), z)             # <<<<<<<<<<<<<<
 *         return double_complex_from_npy_cdouble(r)
 *     else:
 */
  __pyx_v_r = npy_cpow(__pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_x), __pyx_v_z);

  /* "_complexstuff.pxd":131
 *         z.imag = 0.0
 *         r = npy_cpow(npy_cdouble_from_double_complex(x), z)
 *         return double_complex_from_npy_cdouble(r)             # <<<<<<<<<<<<<<
 *     else:
 *         return libc.math.pow(x, y)
 */
  __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_double_complex_from_npy_cdouble(__pyx_v_r);
  goto __pyx_L0;

  /* "_complexstuff.pxd":124
 *         return libc.math.sqrt(x)
 * 
 * cdef inline number_t zpow(number_t x, double y) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r, z
 *     # FIXME
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":135
 *         return libc.math.pow(x, y)
 * 
 * cdef inline double_complex zpack(double zr, double zi) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble z
 *     z.real = zr
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_13_complexstuff_zpack(double __pyx_v_zr, double __pyx_v_zi) {
  npy_cdouble __pyx_v_z;
  __pyx_t_double_complex __pyx_r;

  /* "_complexstuff.pxd":137
 * cdef inline double_complex zpack(double zr, double zi) nogil:
 *     cdef np.npy_cdouble z
 *     z.real = zr             # <<<<<<<<<<<<<<
 *     z.imag = zi
 *     return double_complex_from_npy_cdouble(z)
 */
  __pyx_v_z.real = __pyx_v_zr;

  /* "_complexstuff.pxd":138
 *     cdef np.npy_cdouble z
 *     z.real = zr
 *     z.imag = zi             # <<<<<<<<<<<<<<
 *     return double_complex_from_npy_cdouble(z)
 * 
 */
  __pyx_v_z.imag = __pyx_v_zi;

  /* "_complexstuff.pxd":139
 *     z.real = zr
 *     z.imag = zi
 *     return double_complex_from_npy_cdouble(z)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_double_complex_from_npy_cdouble(__pyx_v_z);
  goto __pyx_L0;

  /* "_complexstuff.pxd":135
 *         return libc.math.pow(x, y)
 * 
 * cdef inline double_complex zpack(double zr, double zi) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble z
 *     z.real = zr
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_complexstuff.pxd":142
 * 
 * @cython.cdivision(True)
 * cdef inline double complex zlog1(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Compute log, paying special attention to accuracy around 1. We
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_13_complexstuff_zlog1(__pyx_t_double_complex __pyx_v_z) {
  int __pyx_v_n;
  __pyx_t_double_complex __pyx_v_coeff;
  __pyx_t_double_complex __pyx_v_res;
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_complexstuff.pxd":151
 *     cdef:
 *         int n
 *         double complex coeff = -1             # <<<<<<<<<<<<<<
 *         double complex res = 0
 * 
 */
  __pyx_v_coeff = __pyx_t_double_complex_from_parts(-1L, 0);

  /* "_complexstuff.pxd":152
 *         int n
 *         double complex coeff = -1
 *         double complex res = 0             # <<<<<<<<<<<<<<
 * 
 *     if zabs(z - 1) > 0.1:
 */
  __pyx_v_res = __pyx_t_double_complex_from_parts(0, 0);

  /* "_complexstuff.pxd":154
 *         double complex res = 0
 * 
 *     if zabs(z - 1) > 0.1:             # <<<<<<<<<<<<<<
 *         return zlog(z)
 *     z = z - 1
 */
  __pyx_t_1 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__Pyx_c_diff_double(__pyx_v_z, __pyx_t_double_complex_from_parts(1, 0))) > 0.1) != 0);
  if (__pyx_t_1) {

    /* "_complexstuff.pxd":155
 * 
 *     if zabs(z - 1) > 0.1:
 *         return zlog(z)             # <<<<<<<<<<<<<<
 *     z = z - 1
 *     if z == 0:
 */
    __pyx_r = __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zlog(__pyx_v_z);
    goto __pyx_L0;

    /* "_complexstuff.pxd":154
 *         double complex res = 0
 * 
 *     if zabs(z - 1) > 0.1:             # <<<<<<<<<<<<<<
 *         return zlog(z)
 *     z = z - 1
 */
  }

  /* "_complexstuff.pxd":156
 *     if zabs(z - 1) > 0.1:
 *         return zlog(z)
 *     z = z - 1             # <<<<<<<<<<<<<<
 *     if z == 0:
 *         return 0
 */
  __pyx_v_z = __Pyx_c_diff_double(__pyx_v_z, __pyx_t_double_complex_from_parts(1, 0));

  /* "_complexstuff.pxd":157
 *         return zlog(z)
 *     z = z - 1
 *     if z == 0:             # <<<<<<<<<<<<<<
 *         return 0
 *     for n in range(1, 17):
 */
  __pyx_t_1 = ((__Pyx_c_eq_double(__pyx_v_z, __pyx_t_double_complex_from_parts(0, 0))) != 0);
  if (__pyx_t_1) {

    /* "_complexstuff.pxd":158
 *     z = z - 1
 *     if z == 0:
 *         return 0             # <<<<<<<<<<<<<<
 *     for n in range(1, 17):
 *         coeff *= -z
 */
    __pyx_r = __pyx_t_double_complex_from_parts(0, 0);
    goto __pyx_L0;

    /* "_complexstuff.pxd":157
 *         return zlog(z)
 *     z = z - 1
 *     if z == 0:             # <<<<<<<<<<<<<<
 *         return 0
 *     for n in range(1, 17):
 */
  }

  /* "_complexstuff.pxd":159
 *     if z == 0:
 *         return 0
 *     for n in range(1, 17):             # <<<<<<<<<<<<<<
 *         coeff *= -z
 *         res += coeff/n
 */
  for (__pyx_t_2 = 1; __pyx_t_2 < 17; __pyx_t_2+=1) {
    __pyx_v_n = __pyx_t_2;

    /* "_complexstuff.pxd":160
 *         return 0
 *     for n in range(1, 17):
 *         coeff *= -z             # <<<<<<<<<<<<<<
 *         res += coeff/n
 *         if zabs(res/coeff) < tol:
 */
    __pyx_v_coeff = __Pyx_c_prod_double(__pyx_v_coeff, __Pyx_c_neg_double(__pyx_v_z));

    /* "_complexstuff.pxd":161
 *     for n in range(1, 17):
 *         coeff *= -z
 *         res += coeff/n             # <<<<<<<<<<<<<<
 *         if zabs(res/coeff) < tol:
 *             break
 */
    __pyx_v_res = __Pyx_c_sum_double(__pyx_v_res, __Pyx_c_quot_double(__pyx_v_coeff, __pyx_t_double_complex_from_parts(__pyx_v_n, 0)));

    /* "_complexstuff.pxd":162
 *         coeff *= -z
 *         res += coeff/n
 *         if zabs(res/coeff) < tol:             # <<<<<<<<<<<<<<
 *             break
 *     return res
 */
    __pyx_t_1 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__Pyx_c_quot_double(__pyx_v_res, __pyx_v_coeff)) < 2.220446092504131e-16) != 0);
    if (__pyx_t_1) {

      /* "_complexstuff.pxd":163
 *         res += coeff/n
 *         if zabs(res/coeff) < tol:
 *             break             # <<<<<<<<<<<<<<
 *     return res
 */
      goto __pyx_L6_break;

      /* "_complexstuff.pxd":162
 *         coeff *= -z
 *         res += coeff/n
 *         if zabs(res/coeff) < tol:             # <<<<<<<<<<<<<<
 *             break
 *     return res
 */
    }
  }
  __pyx_L6_break:;

  /* "_complexstuff.pxd":164
 *         if zabs(res/coeff) < tol:
 *             break
 *     return res             # <<<<<<<<<<<<<<
 */
  __pyx_r = __pyx_v_res;
  goto __pyx_L0;

  /* "_complexstuff.pxd":142
 * 
 * @cython.cdivision(True)
 * cdef inline double complex zlog1(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Compute log, paying special attention to accuracy around 1. We
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_trig.pxd":16
 * 
 * 
 * cdef inline double dsinpi(double x) nogil:             # <<<<<<<<<<<<<<
 *     """Compute sin(pi*x) for real arguments."""
 *     cdef:
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_5_trig_dsinpi(double __pyx_v_x) {
  double __pyx_v_s;
  double __pyx_v_r;
  double __pyx_r;
  int __pyx_t_1;

  /* "_trig.pxd":19
 *     """Compute sin(pi*x) for real arguments."""
 *     cdef:
 *         double s = 1.0             # <<<<<<<<<<<<<<
 *         double r
 * 
 */
  __pyx_v_s = 1.0;

  /* "_trig.pxd":22
 *         double r
 * 
 *     if x < 0.0:             # <<<<<<<<<<<<<<
 *         x = -x
 *         s = -1.0
 */
  __pyx_t_1 = ((__pyx_v_x < 0.0) != 0);
  if (__pyx_t_1) {

    /* "_trig.pxd":23
 * 
 *     if x < 0.0:
 *         x = -x             # <<<<<<<<<<<<<<
 *         s = -1.0
 * 
 */
    __pyx_v_x = (-__pyx_v_x);

    /* "_trig.pxd":24
 *     if x < 0.0:
 *         x = -x
 *         s = -1.0             # <<<<<<<<<<<<<<
 * 
 *     r = fmod(x, 2.0)
 */
    __pyx_v_s = -1.0;

    /* "_trig.pxd":22
 *         double r
 * 
 *     if x < 0.0:             # <<<<<<<<<<<<<<
 *         x = -x
 *         s = -1.0
 */
  }

  /* "_trig.pxd":26
 *         s = -1.0
 * 
 *     r = fmod(x, 2.0)             # <<<<<<<<<<<<<<
 *     if r < 0.5:
 *         return s*sin(M_PI*r)
 */
  __pyx_v_r = fmod(__pyx_v_x, 2.0);

  /* "_trig.pxd":27
 * 
 *     r = fmod(x, 2.0)
 *     if r < 0.5:             # <<<<<<<<<<<<<<
 *         return s*sin(M_PI*r)
 *     elif r > 1.5:
 */
  __pyx_t_1 = ((__pyx_v_r < 0.5) != 0);
  if (__pyx_t_1) {

    /* "_trig.pxd":28
 *     r = fmod(x, 2.0)
 *     if r < 0.5:
 *         return s*sin(M_PI*r)             # <<<<<<<<<<<<<<
 *     elif r > 1.5:
 *         return s*sin(M_PI*(r - 2.0))
 */
    __pyx_r = (__pyx_v_s * sin((M_PI * __pyx_v_r)));
    goto __pyx_L0;

    /* "_trig.pxd":27
 * 
 *     r = fmod(x, 2.0)
 *     if r < 0.5:             # <<<<<<<<<<<<<<
 *         return s*sin(M_PI*r)
 *     elif r > 1.5:
 */
  }

  /* "_trig.pxd":29
 *     if r < 0.5:
 *         return s*sin(M_PI*r)
 *     elif r > 1.5:             # <<<<<<<<<<<<<<
 *         return s*sin(M_PI*(r - 2.0))
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_r > 1.5) != 0);
  if (__pyx_t_1) {

    /* "_trig.pxd":30
 *         return s*sin(M_PI*r)
 *     elif r > 1.5:
 *         return s*sin(M_PI*(r - 2.0))             # <<<<<<<<<<<<<<
 *     else:
 *         return -s*sin(M_PI*(r - 1.0))
 */
    __pyx_r = (__pyx_v_s * sin((M_PI * (__pyx_v_r - 2.0))));
    goto __pyx_L0;

    /* "_trig.pxd":29
 *     if r < 0.5:
 *         return s*sin(M_PI*r)
 *     elif r > 1.5:             # <<<<<<<<<<<<<<
 *         return s*sin(M_PI*(r - 2.0))
 *     else:
 */
  }

  /* "_trig.pxd":32
 *         return s*sin(M_PI*(r - 2.0))
 *     else:
 *         return -s*sin(M_PI*(r - 1.0))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_r = ((-__pyx_v_s) * sin((M_PI * (__pyx_v_r - 1.0))));
    goto __pyx_L0;
  }

  /* "_trig.pxd":16
 * 
 * 
 * cdef inline double dsinpi(double x) nogil:             # <<<<<<<<<<<<<<
 *     """Compute sin(pi*x) for real arguments."""
 *     cdef:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_trig.pxd":35
 * 
 * 
 * cdef inline double dcospi(double x) nogil:             # <<<<<<<<<<<<<<
 *     """Compute cos(pi*x) for real arguments."""
 *     cdef double r
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_5_trig_dcospi(double __pyx_v_x) {
  double __pyx_v_r;
  double __pyx_r;
  int __pyx_t_1;

  /* "_trig.pxd":39
 *     cdef double r
 * 
 *     if x < 0.0:             # <<<<<<<<<<<<<<
 *         x = -x
 * 
 */
  __pyx_t_1 = ((__pyx_v_x < 0.0) != 0);
  if (__pyx_t_1) {

    /* "_trig.pxd":40
 * 
 *     if x < 0.0:
 *         x = -x             # <<<<<<<<<<<<<<
 * 
 *     r = fmod(x, 2.0)
 */
    __pyx_v_x = (-__pyx_v_x);

    /* "_trig.pxd":39
 *     cdef double r
 * 
 *     if x < 0.0:             # <<<<<<<<<<<<<<
 *         x = -x
 * 
 */
  }

  /* "_trig.pxd":42
 *         x = -x
 * 
 *     r = fmod(x, 2.0)             # <<<<<<<<<<<<<<
 *     if r == 0.5:
 *         # We don't want to return -0.0
 */
  __pyx_v_r = fmod(__pyx_v_x, 2.0);

  /* "_trig.pxd":43
 * 
 *     r = fmod(x, 2.0)
 *     if r == 0.5:             # <<<<<<<<<<<<<<
 *         # We don't want to return -0.0
 *         return 0.0
 */
  __pyx_t_1 = ((__pyx_v_r == 0.5) != 0);
  if (__pyx_t_1) {

    /* "_trig.pxd":45
 *     if r == 0.5:
 *         # We don't want to return -0.0
 *         return 0.0             # <<<<<<<<<<<<<<
 *     if r < 1.0:
 *         return -sin(M_PI*(r - 0.5))
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "_trig.pxd":43
 * 
 *     r = fmod(x, 2.0)
 *     if r == 0.5:             # <<<<<<<<<<<<<<
 *         # We don't want to return -0.0
 *         return 0.0
 */
  }

  /* "_trig.pxd":46
 *         # We don't want to return -0.0
 *         return 0.0
 *     if r < 1.0:             # <<<<<<<<<<<<<<
 *         return -sin(M_PI*(r - 0.5))
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_r < 1.0) != 0);
  if (__pyx_t_1) {

    /* "_trig.pxd":47
 *         return 0.0
 *     if r < 1.0:
 *         return -sin(M_PI*(r - 0.5))             # <<<<<<<<<<<<<<
 *     else:
 *         return sin(M_PI*(r - 1.5))
 */
    __pyx_r = (-sin((M_PI * (__pyx_v_r - 0.5))));
    goto __pyx_L0;

    /* "_trig.pxd":46
 *         # We don't want to return -0.0
 *         return 0.0
 *     if r < 1.0:             # <<<<<<<<<<<<<<
 *         return -sin(M_PI*(r - 0.5))
 *     else:
 */
  }

  /* "_trig.pxd":49
 *         return -sin(M_PI*(r - 0.5))
 *     else:
 *         return sin(M_PI*(r - 1.5))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_r = sin((M_PI * (__pyx_v_r - 1.5)));
    goto __pyx_L0;
  }

  /* "_trig.pxd":35
 * 
 * 
 * cdef inline double dcospi(double x) nogil:             # <<<<<<<<<<<<<<
 *     """Compute cos(pi*x) for real arguments."""
 *     cdef double r
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_trig.pxd":52
 * 
 * 
 * cdef inline double complex csinpi(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """Compute sin(pi*z) for complex arguments."""
 *     cdef:
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_5_trig_csinpi(__pyx_t_double_complex __pyx_v_z) {
  double __pyx_v_x;
  double __pyx_v_piy;
  double __pyx_v_abspiy;
  double __pyx_v_sinpix;
  double __pyx_v_cospix;
  double __pyx_v_exphpiy;
  double __pyx_v_coshfac;
  double __pyx_v_sinhfac;
  __pyx_t_double_complex __pyx_r;
  double __pyx_t_1;
  int __pyx_t_2;

  /* "_trig.pxd":55
 *     """Compute sin(pi*z) for complex arguments."""
 *     cdef:
 *         double x = z.real             # <<<<<<<<<<<<<<
 *         double piy = M_PI*z.imag
 *         double abspiy = fabs(piy)
 */
  __pyx_t_1 = __Pyx_CREAL(__pyx_v_z);
  __pyx_v_x = __pyx_t_1;

  /* "_trig.pxd":56
 *     cdef:
 *         double x = z.real
 *         double piy = M_PI*z.imag             # <<<<<<<<<<<<<<
 *         double abspiy = fabs(piy)
 *         double sinpix = sinpi(x)
 */
  __pyx_v_piy = (M_PI * __Pyx_CIMAG(__pyx_v_z));

  /* "_trig.pxd":57
 *         double x = z.real
 *         double piy = M_PI*z.imag
 *         double abspiy = fabs(piy)             # <<<<<<<<<<<<<<
 *         double sinpix = sinpi(x)
 *         double cospix = cospi(x)
 */
  __pyx_v_abspiy = fabs(__pyx_v_piy);

  /* "_trig.pxd":58
 *         double piy = M_PI*z.imag
 *         double abspiy = fabs(piy)
 *         double sinpix = sinpi(x)             # <<<<<<<<<<<<<<
 *         double cospix = cospi(x)
 *         double exphpiy, coshfac, sinhfac
 */
  __pyx_v_sinpix = __pyx_fuse_0__pyx_f_5scipy_7special_5_trig_sinpi(__pyx_v_x);

  /* "_trig.pxd":59
 *         double abspiy = fabs(piy)
 *         double sinpix = sinpi(x)
 *         double cospix = cospi(x)             # <<<<<<<<<<<<<<
 *         double exphpiy, coshfac, sinhfac
 * 
 */
  __pyx_v_cospix = __pyx_fuse_0__pyx_f_5scipy_7special_5_trig_cospi(__pyx_v_x);

  /* "_trig.pxd":62
 *         double exphpiy, coshfac, sinhfac
 * 
 *     if abspiy < 700:             # <<<<<<<<<<<<<<
 *         return zpack(sinpix*cosh(piy), cospix*sinh(piy))
 * 
 */
  __pyx_t_2 = ((__pyx_v_abspiy < 700.0) != 0);
  if (__pyx_t_2) {

    /* "_trig.pxd":63
 * 
 *     if abspiy < 700:
 *         return zpack(sinpix*cosh(piy), cospix*sinh(piy))             # <<<<<<<<<<<<<<
 * 
 *     # Have to be careful--sinh/cosh could overflow while cos/sin are
 */
    __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_zpack((__pyx_v_sinpix * cosh(__pyx_v_piy)), (__pyx_v_cospix * sinh(__pyx_v_piy)));
    goto __pyx_L0;

    /* "_trig.pxd":62
 *         double exphpiy, coshfac, sinhfac
 * 
 *     if abspiy < 700:             # <<<<<<<<<<<<<<
 *         return zpack(sinpix*cosh(piy), cospix*sinh(piy))
 * 
 */
  }

  /* "_trig.pxd":73
 *     # so we can compute exp(y/2), scale by the right factor of sin/cos
 *     # and then multiply by exp(y/2) to avoid overflow.
 *     exphpiy = exp(abspiy/2)             # <<<<<<<<<<<<<<
 *     if exphpiy == NPY_INFINITY:
 *         if sinpix == 0:
 */
  __pyx_v_exphpiy = exp((__pyx_v_abspiy / 2.0));

  /* "_trig.pxd":74
 *     # and then multiply by exp(y/2) to avoid overflow.
 *     exphpiy = exp(abspiy/2)
 *     if exphpiy == NPY_INFINITY:             # <<<<<<<<<<<<<<
 *         if sinpix == 0:
 *             # Preserve the sign of zero
 */
  __pyx_t_2 = ((__pyx_v_exphpiy == NPY_INFINITY) != 0);
  if (__pyx_t_2) {

    /* "_trig.pxd":75
 *     exphpiy = exp(abspiy/2)
 *     if exphpiy == NPY_INFINITY:
 *         if sinpix == 0:             # <<<<<<<<<<<<<<
 *             # Preserve the sign of zero
 *             coshfac = npy_copysign(0.0, sinpix)
 */
    __pyx_t_2 = ((__pyx_v_sinpix == 0.0) != 0);
    if (__pyx_t_2) {

      /* "_trig.pxd":77
 *         if sinpix == 0:
 *             # Preserve the sign of zero
 *             coshfac = npy_copysign(0.0, sinpix)             # <<<<<<<<<<<<<<
 *         else:
 *             coshfac = npy_copysign(NPY_INFINITY, sinpix)
 */
      __pyx_v_coshfac = npy_copysign(0.0, __pyx_v_sinpix);

      /* "_trig.pxd":75
 *     exphpiy = exp(abspiy/2)
 *     if exphpiy == NPY_INFINITY:
 *         if sinpix == 0:             # <<<<<<<<<<<<<<
 *             # Preserve the sign of zero
 *             coshfac = npy_copysign(0.0, sinpix)
 */
      goto __pyx_L5;
    }

    /* "_trig.pxd":79
 *             coshfac = npy_copysign(0.0, sinpix)
 *         else:
 *             coshfac = npy_copysign(NPY_INFINITY, sinpix)             # <<<<<<<<<<<<<<
 *         if cospix == 0:
 *             sinhfac = npy_copysign(0.0, cospix)
 */
    /*else*/ {
      __pyx_v_coshfac = npy_copysign(NPY_INFINITY, __pyx_v_sinpix);
    }
    __pyx_L5:;

    /* "_trig.pxd":80
 *         else:
 *             coshfac = npy_copysign(NPY_INFINITY, sinpix)
 *         if cospix == 0:             # <<<<<<<<<<<<<<
 *             sinhfac = npy_copysign(0.0, cospix)
 *         else:
 */
    __pyx_t_2 = ((__pyx_v_cospix == 0.0) != 0);
    if (__pyx_t_2) {

      /* "_trig.pxd":81
 *             coshfac = npy_copysign(NPY_INFINITY, sinpix)
 *         if cospix == 0:
 *             sinhfac = npy_copysign(0.0, cospix)             # <<<<<<<<<<<<<<
 *         else:
 *             sinhfac = npy_copysign(NPY_INFINITY, cospix)
 */
      __pyx_v_sinhfac = npy_copysign(0.0, __pyx_v_cospix);

      /* "_trig.pxd":80
 *         else:
 *             coshfac = npy_copysign(NPY_INFINITY, sinpix)
 *         if cospix == 0:             # <<<<<<<<<<<<<<
 *             sinhfac = npy_copysign(0.0, cospix)
 *         else:
 */
      goto __pyx_L6;
    }

    /* "_trig.pxd":83
 *             sinhfac = npy_copysign(0.0, cospix)
 *         else:
 *             sinhfac = npy_copysign(NPY_INFINITY, cospix)             # <<<<<<<<<<<<<<
 *         return zpack(coshfac, sinhfac)
 * 
 */
    /*else*/ {
      __pyx_v_sinhfac = npy_copysign(NPY_INFINITY, __pyx_v_cospix);
    }
    __pyx_L6:;

    /* "_trig.pxd":84
 *         else:
 *             sinhfac = npy_copysign(NPY_INFINITY, cospix)
 *         return zpack(coshfac, sinhfac)             # <<<<<<<<<<<<<<
 * 
 *     coshfac = 0.5*sinpix*exphpiy
 */
    __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_zpack(__pyx_v_coshfac, __pyx_v_sinhfac);
    goto __pyx_L0;

    /* "_trig.pxd":74
 *     # and then multiply by exp(y/2) to avoid overflow.
 *     exphpiy = exp(abspiy/2)
 *     if exphpiy == NPY_INFINITY:             # <<<<<<<<<<<<<<
 *         if sinpix == 0:
 *             # Preserve the sign of zero
 */
  }

  /* "_trig.pxd":86
 *         return zpack(coshfac, sinhfac)
 * 
 *     coshfac = 0.5*sinpix*exphpiy             # <<<<<<<<<<<<<<
 *     sinhfac = 0.5*cospix*exphpiy
 *     return zpack(coshfac*exphpiy, sinhfac*exphpiy)
 */
  __pyx_v_coshfac = ((0.5 * __pyx_v_sinpix) * __pyx_v_exphpiy);

  /* "_trig.pxd":87
 * 
 *     coshfac = 0.5*sinpix*exphpiy
 *     sinhfac = 0.5*cospix*exphpiy             # <<<<<<<<<<<<<<
 *     return zpack(coshfac*exphpiy, sinhfac*exphpiy)
 * 
 */
  __pyx_v_sinhfac = ((0.5 * __pyx_v_cospix) * __pyx_v_exphpiy);

  /* "_trig.pxd":88
 *     coshfac = 0.5*sinpix*exphpiy
 *     sinhfac = 0.5*cospix*exphpiy
 *     return zpack(coshfac*exphpiy, sinhfac*exphpiy)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_zpack((__pyx_v_coshfac * __pyx_v_exphpiy), (__pyx_v_sinhfac * __pyx_v_exphpiy));
  goto __pyx_L0;

  /* "_trig.pxd":52
 * 
 * 
 * cdef inline double complex csinpi(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """Compute sin(pi*z) for complex arguments."""
 *     cdef:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_trig.pxd":91
 * 
 * 
 * cdef inline double complex ccospi(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """Compute cos(pi*z) for complex arguments."""
 *     cdef:
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_5_trig_ccospi(__pyx_t_double_complex __pyx_v_z) {
  double __pyx_v_x;
  double __pyx_v_piy;
  double __pyx_v_abspiy;
  double __pyx_v_sinpix;
  double __pyx_v_cospix;
  double __pyx_v_exphpiy;
  double __pyx_v_coshfac;
  double __pyx_v_sinhfac;
  __pyx_t_double_complex __pyx_r;
  double __pyx_t_1;
  int __pyx_t_2;

  /* "_trig.pxd":94
 *     """Compute cos(pi*z) for complex arguments."""
 *     cdef:
 *         double x = z.real             # <<<<<<<<<<<<<<
 *         double piy = M_PI*z.imag
 *         double abspiy = fabs(piy)
 */
  __pyx_t_1 = __Pyx_CREAL(__pyx_v_z);
  __pyx_v_x = __pyx_t_1;

  /* "_trig.pxd":95
 *     cdef:
 *         double x = z.real
 *         double piy = M_PI*z.imag             # <<<<<<<<<<<<<<
 *         double abspiy = fabs(piy)
 *         double sinpix = sinpi(x)
 */
  __pyx_v_piy = (M_PI * __Pyx_CIMAG(__pyx_v_z));

  /* "_trig.pxd":96
 *         double x = z.real
 *         double piy = M_PI*z.imag
 *         double abspiy = fabs(piy)             # <<<<<<<<<<<<<<
 *         double sinpix = sinpi(x)
 *         double cospix = cospi(x)
 */
  __pyx_v_abspiy = fabs(__pyx_v_piy);

  /* "_trig.pxd":97
 *         double piy = M_PI*z.imag
 *         double abspiy = fabs(piy)
 *         double sinpix = sinpi(x)             # <<<<<<<<<<<<<<
 *         double cospix = cospi(x)
 *         double exphpiy, coshfac, sinhfac
 */
  __pyx_v_sinpix = __pyx_fuse_0__pyx_f_5scipy_7special_5_trig_sinpi(__pyx_v_x);

  /* "_trig.pxd":98
 *         double abspiy = fabs(piy)
 *         double sinpix = sinpi(x)
 *         double cospix = cospi(x)             # <<<<<<<<<<<<<<
 *         double exphpiy, coshfac, sinhfac
 * 
 */
  __pyx_v_cospix = __pyx_fuse_0__pyx_f_5scipy_7special_5_trig_cospi(__pyx_v_x);

  /* "_trig.pxd":101
 *         double exphpiy, coshfac, sinhfac
 * 
 *     if abspiy < 700:             # <<<<<<<<<<<<<<
 *         return zpack(cospix*cosh(piy), -sinpix*sinh(piy))
 * 
 */
  __pyx_t_2 = ((__pyx_v_abspiy < 700.0) != 0);
  if (__pyx_t_2) {

    /* "_trig.pxd":102
 * 
 *     if abspiy < 700:
 *         return zpack(cospix*cosh(piy), -sinpix*sinh(piy))             # <<<<<<<<<<<<<<
 * 
 *     # See csinpi(z) for an idea of what's going on here
 */
    __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_zpack((__pyx_v_cospix * cosh(__pyx_v_piy)), ((-__pyx_v_sinpix) * sinh(__pyx_v_piy)));
    goto __pyx_L0;

    /* "_trig.pxd":101
 *         double exphpiy, coshfac, sinhfac
 * 
 *     if abspiy < 700:             # <<<<<<<<<<<<<<
 *         return zpack(cospix*cosh(piy), -sinpix*sinh(piy))
 * 
 */
  }

  /* "_trig.pxd":105
 * 
 *     # See csinpi(z) for an idea of what's going on here
 *     exphpiy = exp(abspiy/2)             # <<<<<<<<<<<<<<
 *     if exphpiy == NPY_INFINITY:
 *         if sinpix == 0:
 */
  __pyx_v_exphpiy = exp((__pyx_v_abspiy / 2.0));

  /* "_trig.pxd":106
 *     # See csinpi(z) for an idea of what's going on here
 *     exphpiy = exp(abspiy/2)
 *     if exphpiy == NPY_INFINITY:             # <<<<<<<<<<<<<<
 *         if sinpix == 0:
 *             coshfac = npy_copysign(0.0, cospix)
 */
  __pyx_t_2 = ((__pyx_v_exphpiy == NPY_INFINITY) != 0);
  if (__pyx_t_2) {

    /* "_trig.pxd":107
 *     exphpiy = exp(abspiy/2)
 *     if exphpiy == NPY_INFINITY:
 *         if sinpix == 0:             # <<<<<<<<<<<<<<
 *             coshfac = npy_copysign(0.0, cospix)
 *         else:
 */
    __pyx_t_2 = ((__pyx_v_sinpix == 0.0) != 0);
    if (__pyx_t_2) {

      /* "_trig.pxd":108
 *     if exphpiy == NPY_INFINITY:
 *         if sinpix == 0:
 *             coshfac = npy_copysign(0.0, cospix)             # <<<<<<<<<<<<<<
 *         else:
 *             coshfac = npy_copysign(NPY_INFINITY, cospix)
 */
      __pyx_v_coshfac = npy_copysign(0.0, __pyx_v_cospix);

      /* "_trig.pxd":107
 *     exphpiy = exp(abspiy/2)
 *     if exphpiy == NPY_INFINITY:
 *         if sinpix == 0:             # <<<<<<<<<<<<<<
 *             coshfac = npy_copysign(0.0, cospix)
 *         else:
 */
      goto __pyx_L5;
    }

    /* "_trig.pxd":110
 *             coshfac = npy_copysign(0.0, cospix)
 *         else:
 *             coshfac = npy_copysign(NPY_INFINITY, cospix)             # <<<<<<<<<<<<<<
 *         if cospix == 0:
 *             sinhfac = npy_copysign(0.0, sinpix)
 */
    /*else*/ {
      __pyx_v_coshfac = npy_copysign(NPY_INFINITY, __pyx_v_cospix);
    }
    __pyx_L5:;

    /* "_trig.pxd":111
 *         else:
 *             coshfac = npy_copysign(NPY_INFINITY, cospix)
 *         if cospix == 0:             # <<<<<<<<<<<<<<
 *             sinhfac = npy_copysign(0.0, sinpix)
 *         else:
 */
    __pyx_t_2 = ((__pyx_v_cospix == 0.0) != 0);
    if (__pyx_t_2) {

      /* "_trig.pxd":112
 *             coshfac = npy_copysign(NPY_INFINITY, cospix)
 *         if cospix == 0:
 *             sinhfac = npy_copysign(0.0, sinpix)             # <<<<<<<<<<<<<<
 *         else:
 *             sinhfac = npy_copysign(NPY_INFINITY, sinpix)
 */
      __pyx_v_sinhfac = npy_copysign(0.0, __pyx_v_sinpix);

      /* "_trig.pxd":111
 *         else:
 *             coshfac = npy_copysign(NPY_INFINITY, cospix)
 *         if cospix == 0:             # <<<<<<<<<<<<<<
 *             sinhfac = npy_copysign(0.0, sinpix)
 *         else:
 */
      goto __pyx_L6;
    }

    /* "_trig.pxd":114
 *             sinhfac = npy_copysign(0.0, sinpix)
 *         else:
 *             sinhfac = npy_copysign(NPY_INFINITY, sinpix)             # <<<<<<<<<<<<<<
 *         return zpack(coshfac, sinhfac)
 * 
 */
    /*else*/ {
      __pyx_v_sinhfac = npy_copysign(NPY_INFINITY, __pyx_v_sinpix);
    }
    __pyx_L6:;

    /* "_trig.pxd":115
 *         else:
 *             sinhfac = npy_copysign(NPY_INFINITY, sinpix)
 *         return zpack(coshfac, sinhfac)             # <<<<<<<<<<<<<<
 * 
 *     coshfac = 0.5*cospix*exphpiy
 */
    __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_zpack(__pyx_v_coshfac, __pyx_v_sinhfac);
    goto __pyx_L0;

    /* "_trig.pxd":106
 *     # See csinpi(z) for an idea of what's going on here
 *     exphpiy = exp(abspiy/2)
 *     if exphpiy == NPY_INFINITY:             # <<<<<<<<<<<<<<
 *         if sinpix == 0:
 *             coshfac = npy_copysign(0.0, cospix)
 */
  }

  /* "_trig.pxd":117
 *         return zpack(coshfac, sinhfac)
 * 
 *     coshfac = 0.5*cospix*exphpiy             # <<<<<<<<<<<<<<
 *     sinhfac = 0.5*sinpix*exphpiy
 *     return zpack(coshfac*exphpiy, sinhfac*exphpiy)
 */
  __pyx_v_coshfac = ((0.5 * __pyx_v_cospix) * __pyx_v_exphpiy);

  /* "_trig.pxd":118
 * 
 *     coshfac = 0.5*cospix*exphpiy
 *     sinhfac = 0.5*sinpix*exphpiy             # <<<<<<<<<<<<<<
 *     return zpack(coshfac*exphpiy, sinhfac*exphpiy)
 * 
 */
  __pyx_v_sinhfac = ((0.5 * __pyx_v_sinpix) * __pyx_v_exphpiy);

  /* "_trig.pxd":119
 *     coshfac = 0.5*cospix*exphpiy
 *     sinhfac = 0.5*sinpix*exphpiy
 *     return zpack(coshfac*exphpiy, sinhfac*exphpiy)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_zpack((__pyx_v_coshfac * __pyx_v_exphpiy), (__pyx_v_sinhfac * __pyx_v_exphpiy));
  goto __pyx_L0;

  /* "_trig.pxd":91
 * 
 * 
 * cdef inline double complex ccospi(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """Compute cos(pi*z) for complex arguments."""
 *     cdef:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_trig.pxd":122
 * 
 * 
 * cdef inline number_t sinpi(number_t z) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double:
 *         return dsinpi(z)
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_5_trig_sinpi(double __pyx_v_z) {
  double __pyx_r;

  /* "_trig.pxd":124
 * cdef inline number_t sinpi(number_t z) nogil:
 *     if number_t is double:
 *         return dsinpi(z)             # <<<<<<<<<<<<<<
 *     else:
 *         return csinpi(z)
 */
  __pyx_r = __pyx_f_5scipy_7special_5_trig_dsinpi(__pyx_v_z);
  goto __pyx_L0;

  /* "_trig.pxd":122
 * 
 * 
 * cdef inline number_t sinpi(number_t z) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double:
 *         return dsinpi(z)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_5_trig_sinpi(__pyx_t_double_complex __pyx_v_z) {
  __pyx_t_double_complex __pyx_r;

  /* "_trig.pxd":126
 *         return dsinpi(z)
 *     else:
 *         return csinpi(z)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_f_5scipy_7special_5_trig_csinpi(__pyx_v_z);
  goto __pyx_L0;

  /* "_trig.pxd":122
 * 
 * 
 * cdef inline number_t sinpi(number_t z) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double:
 *         return dsinpi(z)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_trig.pxd":129
 * 
 * 
 * cdef inline number_t cospi(number_t z) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double:
 *         return dcospi(z)
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_5_trig_cospi(double __pyx_v_z) {
  double __pyx_r;

  /* "_trig.pxd":131
 * cdef inline number_t cospi(number_t z) nogil:
 *     if number_t is double:
 *         return dcospi(z)             # <<<<<<<<<<<<<<
 *     else:
 *         return ccospi(z)
 */
  __pyx_r = __pyx_f_5scipy_7special_5_trig_dcospi(__pyx_v_z);
  goto __pyx_L0;

  /* "_trig.pxd":129
 * 
 * 
 * cdef inline number_t cospi(number_t z) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double:
 *         return dcospi(z)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_5_trig_cospi(__pyx_t_double_complex __pyx_v_z) {
  __pyx_t_double_complex __pyx_r;

  /* "_trig.pxd":133
 *         return dcospi(z)
 *     else:
 *         return ccospi(z)             # <<<<<<<<<<<<<<
 */
  __pyx_r = __pyx_f_5scipy_7special_5_trig_ccospi(__pyx_v_z);
  goto __pyx_L0;

  /* "_trig.pxd":129
 * 
 * 
 * cdef inline number_t cospi(number_t z) nogil:             # <<<<<<<<<<<<<<
 *     if number_t is double:
 *         return dcospi(z)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_ellip_harm.pxd":49
 * @cython.boundscheck(False)
 * @cython.cdivision(True)
 * cdef inline double* lame_coefficients(double h2, double k2, int n, int p,             # <<<<<<<<<<<<<<
 *                                       void **bufferp, double signm,
 *                                       double signn) nogil:
 */

static CYTHON_INLINE double *__pyx_f_5scipy_7special_11_ellip_harm_lame_coefficients(double __pyx_v_h2, double __pyx_v_k2, int __pyx_v_n, int __pyx_v_p, void **__pyx_v_bufferp, double __pyx_v_signm, double __pyx_v_signn) {
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_gamma;
  double __pyx_v_tol;
  double __pyx_v_vl;
  double __pyx_v_vu;
  int __pyx_v_r;
  int __pyx_v_tp;
  int __pyx_v_j;
  int __pyx_v_size;
  int __pyx_v_i;
  int __pyx_v_info;
  int __pyx_v_lwork;
  int __pyx_v_liwork;
  int __pyx_v_c;
  Py_UNICODE __pyx_v_t;
  void *__pyx_v_buffer;
  double *__pyx_v_g;
  double *__pyx_v_d;
  double *__pyx_v_f;
  double *__pyx_v_ss;
  double *__pyx_v_w;
  double *__pyx_v_dd;
  double *__pyx_v_eigv;
  double *__pyx_v_work;
  int *__pyx_v_iwork;
  int *__pyx_v_isuppz;
  double *__pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  Py_UNICODE __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  long __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;

  /* "_ellip_harm.pxd":52
 *                                       void **bufferp, double signm,
 *                                       double signn) nogil:
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid value for n")
 *         return NULL
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":53
 *                                       double signn) nogil:
 *     if n < 0:
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid value for n")             # <<<<<<<<<<<<<<
 *         return NULL
 * 
 */
    sf_error(((char *)"ellip_harm"), SF_ERROR_ARG, ((char *)"invalid value for n"));

    /* "_ellip_harm.pxd":54
 *     if n < 0:
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid value for n")
 *         return NULL             # <<<<<<<<<<<<<<
 * 
 *     if p < 1 or p > 2*n + 1:
 */
    __pyx_r = NULL;
    goto __pyx_L0;

    /* "_ellip_harm.pxd":52
 *                                       void **bufferp, double signm,
 *                                       double signn) nogil:
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid value for n")
 *         return NULL
 */
  }

  /* "_ellip_harm.pxd":56
 *         return NULL
 * 
 *     if p < 1 or p > 2*n + 1:             # <<<<<<<<<<<<<<
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid value for p")
 *         return NULL
 */
  __pyx_t_2 = ((__pyx_v_p < 1) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_p > ((2 * __pyx_v_n) + 1)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":57
 * 
 *     if p < 1 or p > 2*n + 1:
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid value for p")             # <<<<<<<<<<<<<<
 *         return NULL
 * 
 */
    sf_error(((char *)"ellip_harm"), SF_ERROR_ARG, ((char *)"invalid value for p"));

    /* "_ellip_harm.pxd":58
 *     if p < 1 or p > 2*n + 1:
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid value for p")
 *         return NULL             # <<<<<<<<<<<<<<
 * 
 *     if fabs(signm) != 1 or fabs(signn) != 1:
 */
    __pyx_r = NULL;
    goto __pyx_L0;

    /* "_ellip_harm.pxd":56
 *         return NULL
 * 
 *     if p < 1 or p > 2*n + 1:             # <<<<<<<<<<<<<<
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid value for p")
 *         return NULL
 */
  }

  /* "_ellip_harm.pxd":60
 *         return NULL
 * 
 *     if fabs(signm) != 1 or fabs(signn) != 1:             # <<<<<<<<<<<<<<
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid signm or signn")
 *         return NULL
 */
  __pyx_t_2 = ((fabs(__pyx_v_signm) != 1.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_2 = ((fabs(__pyx_v_signn) != 1.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L8_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":61
 * 
 *     if fabs(signm) != 1 or fabs(signn) != 1:
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid signm or signn")             # <<<<<<<<<<<<<<
 *         return NULL
 * 
 */
    sf_error(((char *)"ellip_harm"), SF_ERROR_ARG, ((char *)"invalid signm or signn"));

    /* "_ellip_harm.pxd":62
 *     if fabs(signm) != 1 or fabs(signn) != 1:
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid signm or signn")
 *         return NULL             # <<<<<<<<<<<<<<
 * 
 *     cdef double s2, alpha, beta, gamma, lamba_romain, pp, psi, t1, tol, vl, vu
 */
    __pyx_r = NULL;
    goto __pyx_L0;

    /* "_ellip_harm.pxd":60
 *         return NULL
 * 
 *     if fabs(signm) != 1 or fabs(signn) != 1:             # <<<<<<<<<<<<<<
 *         sf_error.error("ellip_harm", sf_error.ARG, "invalid signm or signn")
 *         return NULL
 */
  }

  /* "_ellip_harm.pxd":68
 *     cdef Py_UNICODE t
 * 
 *     r = n/2             # <<<<<<<<<<<<<<
 *     alpha = h2
 *     beta = k2 - h2
 */
  __pyx_v_r = (((long)__pyx_v_n) / 2);

  /* "_ellip_harm.pxd":69
 * 
 *     r = n/2
 *     alpha = h2             # <<<<<<<<<<<<<<
 *     beta = k2 - h2
 *     gamma = alpha - beta
 */
  __pyx_v_alpha = __pyx_v_h2;

  /* "_ellip_harm.pxd":70
 *     r = n/2
 *     alpha = h2
 *     beta = k2 - h2             # <<<<<<<<<<<<<<
 *     gamma = alpha - beta
 * 
 */
  __pyx_v_beta = (__pyx_v_k2 - __pyx_v_h2);

  /* "_ellip_harm.pxd":71
 *     alpha = h2
 *     beta = k2 - h2
 *     gamma = alpha - beta             # <<<<<<<<<<<<<<
 * 
 *     if p - 1 < r + 1:
 */
  __pyx_v_gamma = (__pyx_v_alpha - __pyx_v_beta);

  /* "_ellip_harm.pxd":73
 *     gamma = alpha - beta
 * 
 *     if p - 1 < r + 1:             # <<<<<<<<<<<<<<
 *         t, tp, size = 'K', p, r + 1
 *     elif p - 1 < (n - r) + (r + 1):
 */
  __pyx_t_1 = (((__pyx_v_p - 1) < (__pyx_v_r + 1)) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":74
 * 
 *     if p - 1 < r + 1:
 *         t, tp, size = 'K', p, r + 1             # <<<<<<<<<<<<<<
 *     elif p - 1 < (n - r) + (r + 1):
 *         t, tp, size = 'L', p - (r + 1), n - r
 */
    __pyx_t_3 = 75;
    __pyx_t_4 = __pyx_v_p;
    __pyx_t_5 = (__pyx_v_r + 1);
    __pyx_v_t = __pyx_t_3;
    __pyx_v_tp = __pyx_t_4;
    __pyx_v_size = __pyx_t_5;

    /* "_ellip_harm.pxd":73
 *     gamma = alpha - beta
 * 
 *     if p - 1 < r + 1:             # <<<<<<<<<<<<<<
 *         t, tp, size = 'K', p, r + 1
 *     elif p - 1 < (n - r) + (r + 1):
 */
    goto __pyx_L10;
  }

  /* "_ellip_harm.pxd":75
 *     if p - 1 < r + 1:
 *         t, tp, size = 'K', p, r + 1
 *     elif p - 1 < (n - r) + (r + 1):             # <<<<<<<<<<<<<<
 *         t, tp, size = 'L', p - (r + 1), n - r
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 */
  __pyx_t_1 = (((__pyx_v_p - 1) < ((__pyx_v_n - __pyx_v_r) + (__pyx_v_r + 1))) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":76
 *         t, tp, size = 'K', p, r + 1
 *     elif p - 1 < (n - r) + (r + 1):
 *         t, tp, size = 'L', p - (r + 1), n - r             # <<<<<<<<<<<<<<
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 *         t, tp, size = 'M', p - (n - r) - (r + 1), n - r
 */
    __pyx_t_3 = 76;
    __pyx_t_5 = (__pyx_v_p - (__pyx_v_r + 1));
    __pyx_t_4 = (__pyx_v_n - __pyx_v_r);
    __pyx_v_t = __pyx_t_3;
    __pyx_v_tp = __pyx_t_5;
    __pyx_v_size = __pyx_t_4;

    /* "_ellip_harm.pxd":75
 *     if p - 1 < r + 1:
 *         t, tp, size = 'K', p, r + 1
 *     elif p - 1 < (n - r) + (r + 1):             # <<<<<<<<<<<<<<
 *         t, tp, size = 'L', p - (r + 1), n - r
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 */
    goto __pyx_L10;
  }

  /* "_ellip_harm.pxd":77
 *     elif p - 1 < (n - r) + (r + 1):
 *         t, tp, size = 'L', p - (r + 1), n - r
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):             # <<<<<<<<<<<<<<
 *         t, tp, size = 'M', p - (n - r) - (r + 1), n - r
 *     elif p - 1 < 2*n + 1:
 */
  __pyx_t_1 = (((__pyx_v_p - 1) < (((__pyx_v_n - __pyx_v_r) + (__pyx_v_n - __pyx_v_r)) + (__pyx_v_r + 1))) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":78
 *         t, tp, size = 'L', p - (r + 1), n - r
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 *         t, tp, size = 'M', p - (n - r) - (r + 1), n - r             # <<<<<<<<<<<<<<
 *     elif p - 1 < 2*n + 1:
 *         t, tp, size = 'N', p - (n - r) - (n - r) - (r + 1), r
 */
    __pyx_t_3 = 77;
    __pyx_t_5 = ((__pyx_v_p - (__pyx_v_n - __pyx_v_r)) - (__pyx_v_r + 1));
    __pyx_t_4 = (__pyx_v_n - __pyx_v_r);
    __pyx_v_t = __pyx_t_3;
    __pyx_v_tp = __pyx_t_5;
    __pyx_v_size = __pyx_t_4;

    /* "_ellip_harm.pxd":77
 *     elif p - 1 < (n - r) + (r + 1):
 *         t, tp, size = 'L', p - (r + 1), n - r
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):             # <<<<<<<<<<<<<<
 *         t, tp, size = 'M', p - (n - r) - (r + 1), n - r
 *     elif p - 1 < 2*n + 1:
 */
    goto __pyx_L10;
  }

  /* "_ellip_harm.pxd":79
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 *         t, tp, size = 'M', p - (n - r) - (r + 1), n - r
 *     elif p - 1 < 2*n + 1:             # <<<<<<<<<<<<<<
 *         t, tp, size = 'N', p - (n - r) - (n - r) - (r + 1), r
 * 
 */
  __pyx_t_1 = (((__pyx_v_p - 1) < ((2 * __pyx_v_n) + 1)) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":80
 *         t, tp, size = 'M', p - (n - r) - (r + 1), n - r
 *     elif p - 1 < 2*n + 1:
 *         t, tp, size = 'N', p - (n - r) - (n - r) - (r + 1), r             # <<<<<<<<<<<<<<
 * 
 *     lwork = 60*size
 */
    __pyx_t_3 = 78;
    __pyx_t_5 = (((__pyx_v_p - (__pyx_v_n - __pyx_v_r)) - (__pyx_v_n - __pyx_v_r)) - (__pyx_v_r + 1));
    __pyx_t_4 = __pyx_v_r;
    __pyx_v_t = __pyx_t_3;
    __pyx_v_tp = __pyx_t_5;
    __pyx_v_size = __pyx_t_4;

    /* "_ellip_harm.pxd":79
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 *         t, tp, size = 'M', p - (n - r) - (r + 1), n - r
 *     elif p - 1 < 2*n + 1:             # <<<<<<<<<<<<<<
 *         t, tp, size = 'N', p - (n - r) - (n - r) - (r + 1), r
 * 
 */
  }
  __pyx_L10:;

  /* "_ellip_harm.pxd":82
 *         t, tp, size = 'N', p - (n - r) - (n - r) - (r + 1), r
 * 
 *     lwork = 60*size             # <<<<<<<<<<<<<<
 *     liwork = 30*size
 *     tol = 0.0
 */
  __pyx_v_lwork = (60 * __pyx_v_size);

  /* "_ellip_harm.pxd":83
 * 
 *     lwork = 60*size
 *     liwork = 30*size             # <<<<<<<<<<<<<<
 *     tol = 0.0
 *     vl = 0
 */
  __pyx_v_liwork = (30 * __pyx_v_size);

  /* "_ellip_harm.pxd":84
 *     lwork = 60*size
 *     liwork = 30*size
 *     tol = 0.0             # <<<<<<<<<<<<<<
 *     vl = 0
 *     vu = 0
 */
  __pyx_v_tol = 0.0;

  /* "_ellip_harm.pxd":85
 *     liwork = 30*size
 *     tol = 0.0
 *     vl = 0             # <<<<<<<<<<<<<<
 *     vu = 0
 * 
 */
  __pyx_v_vl = 0.0;

  /* "_ellip_harm.pxd":86
 *     tol = 0.0
 *     vl = 0
 *     vu = 0             # <<<<<<<<<<<<<<
 * 
 *     cdef void *buffer = malloc((sizeof(double)*(7*size + lwork))
 */
  __pyx_v_vu = 0.0;

  /* "_ellip_harm.pxd":88
 *     vu = 0
 * 
 *     cdef void *buffer = malloc((sizeof(double)*(7*size + lwork))             # <<<<<<<<<<<<<<
 *                                + (sizeof(int)*(2*size + liwork)))
 *     bufferp[0] = buffer
 */
  __pyx_v_buffer = malloc((((sizeof(double)) * ((7 * __pyx_v_size) + __pyx_v_lwork)) + ((sizeof(int)) * ((2 * __pyx_v_size) + __pyx_v_liwork))));

  /* "_ellip_harm.pxd":90
 *     cdef void *buffer = malloc((sizeof(double)*(7*size + lwork))
 *                                + (sizeof(int)*(2*size + liwork)))
 *     bufferp[0] = buffer             # <<<<<<<<<<<<<<
 *     if not buffer:
 *         sf_error.error("ellip_harm", sf_error.NO_RESULT, "failed to allocate memory")
 */
  (__pyx_v_bufferp[0]) = __pyx_v_buffer;

  /* "_ellip_harm.pxd":91
 *                                + (sizeof(int)*(2*size + liwork)))
 *     bufferp[0] = buffer
 *     if not buffer:             # <<<<<<<<<<<<<<
 *         sf_error.error("ellip_harm", sf_error.NO_RESULT, "failed to allocate memory")
 *         return NULL
 */
  __pyx_t_1 = ((!(__pyx_v_buffer != 0)) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":92
 *     bufferp[0] = buffer
 *     if not buffer:
 *         sf_error.error("ellip_harm", sf_error.NO_RESULT, "failed to allocate memory")             # <<<<<<<<<<<<<<
 *         return NULL
 * 
 */
    sf_error(((char *)"ellip_harm"), SF_ERROR_NO_RESULT, ((char *)"failed to allocate memory"));

    /* "_ellip_harm.pxd":93
 *     if not buffer:
 *         sf_error.error("ellip_harm", sf_error.NO_RESULT, "failed to allocate memory")
 *         return NULL             # <<<<<<<<<<<<<<
 * 
 *     cdef double *g = <double *>buffer
 */
    __pyx_r = NULL;
    goto __pyx_L0;

    /* "_ellip_harm.pxd":91
 *                                + (sizeof(int)*(2*size + liwork)))
 *     bufferp[0] = buffer
 *     if not buffer:             # <<<<<<<<<<<<<<
 *         sf_error.error("ellip_harm", sf_error.NO_RESULT, "failed to allocate memory")
 *         return NULL
 */
  }

  /* "_ellip_harm.pxd":95
 *         return NULL
 * 
 *     cdef double *g = <double *>buffer             # <<<<<<<<<<<<<<
 *     cdef double *d = g + size
 *     cdef double *f = d + size
 */
  __pyx_v_g = ((double *)__pyx_v_buffer);

  /* "_ellip_harm.pxd":96
 * 
 *     cdef double *g = <double *>buffer
 *     cdef double *d = g + size             # <<<<<<<<<<<<<<
 *     cdef double *f = d + size
 *     cdef double *ss =  f + size
 */
  __pyx_v_d = (__pyx_v_g + __pyx_v_size);

  /* "_ellip_harm.pxd":97
 *     cdef double *g = <double *>buffer
 *     cdef double *d = g + size
 *     cdef double *f = d + size             # <<<<<<<<<<<<<<
 *     cdef double *ss =  f + size
 *     cdef double *w =  ss + size
 */
  __pyx_v_f = (__pyx_v_d + __pyx_v_size);

  /* "_ellip_harm.pxd":98
 *     cdef double *d = g + size
 *     cdef double *f = d + size
 *     cdef double *ss =  f + size             # <<<<<<<<<<<<<<
 *     cdef double *w =  ss + size
 *     cdef double *dd = w + size
 */
  __pyx_v_ss = (__pyx_v_f + __pyx_v_size);

  /* "_ellip_harm.pxd":99
 *     cdef double *f = d + size
 *     cdef double *ss =  f + size
 *     cdef double *w =  ss + size             # <<<<<<<<<<<<<<
 *     cdef double *dd = w + size
 *     cdef double *eigv = dd + size
 */
  __pyx_v_w = (__pyx_v_ss + __pyx_v_size);

  /* "_ellip_harm.pxd":100
 *     cdef double *ss =  f + size
 *     cdef double *w =  ss + size
 *     cdef double *dd = w + size             # <<<<<<<<<<<<<<
 *     cdef double *eigv = dd + size
 *     cdef double *work = eigv + size
 */
  __pyx_v_dd = (__pyx_v_w + __pyx_v_size);

  /* "_ellip_harm.pxd":101
 *     cdef double *w =  ss + size
 *     cdef double *dd = w + size
 *     cdef double *eigv = dd + size             # <<<<<<<<<<<<<<
 *     cdef double *work = eigv + size
 * 
 */
  __pyx_v_eigv = (__pyx_v_dd + __pyx_v_size);

  /* "_ellip_harm.pxd":102
 *     cdef double *dd = w + size
 *     cdef double *eigv = dd + size
 *     cdef double *work = eigv + size             # <<<<<<<<<<<<<<
 * 
 *     cdef int *iwork = <int *>(work + lwork)
 */
  __pyx_v_work = (__pyx_v_eigv + __pyx_v_size);

  /* "_ellip_harm.pxd":104
 *     cdef double *work = eigv + size
 * 
 *     cdef int *iwork = <int *>(work + lwork)             # <<<<<<<<<<<<<<
 *     cdef int *isuppz = iwork + liwork
 * 
 */
  __pyx_v_iwork = ((int *)(__pyx_v_work + __pyx_v_lwork));

  /* "_ellip_harm.pxd":105
 * 
 *     cdef int *iwork = <int *>(work + lwork)
 *     cdef int *isuppz = iwork + liwork             # <<<<<<<<<<<<<<
 * 
 *     if t == 'K':
 */
  __pyx_v_isuppz = (__pyx_v_iwork + __pyx_v_liwork);

  /* "_ellip_harm.pxd":107
 *     cdef int *isuppz = iwork + liwork
 * 
 *     if t == 'K':             # <<<<<<<<<<<<<<
 *         for j in range(0, r + 1):
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 */
  switch (__pyx_v_t) {
    case 75:

    /* "_ellip_harm.pxd":108
 * 
 *     if t == 'K':
 *         for j in range(0, r + 1):             # <<<<<<<<<<<<<<
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 *            if n%2:
 */
    __pyx_t_5 = (__pyx_v_r + 1);
    __pyx_t_6 = __pyx_t_5;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_6; __pyx_t_4+=1) {
      __pyx_v_j = __pyx_t_4;

      /* "_ellip_harm.pxd":109
 *     if t == 'K':
 *         for j in range(0, r + 1):
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)             # <<<<<<<<<<<<<<
 *            if n%2:
 *                f[j] = (-alpha*(2*(r- (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 */
      (__pyx_v_g[__pyx_v_j]) = (((-((2 * __pyx_v_j) + 2)) * ((2 * __pyx_v_j) + 1)) * __pyx_v_beta);

      /* "_ellip_harm.pxd":110
 *         for j in range(0, r + 1):
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 *            if n%2:             # <<<<<<<<<<<<<<
 *                f[j] = (-alpha*(2*(r- (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 *                d[j] = ((2*r + 1)*(2*r + 2) - 4*j*j)*alpha + (2*j + 1)*(2*j + 1)*beta
 */
      __pyx_t_1 = ((__pyx_v_n % 2) != 0);
      if (__pyx_t_1) {

        /* "_ellip_harm.pxd":111
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 *            if n%2:
 *                f[j] = (-alpha*(2*(r- (j + 1)) + 2)*(2*((j + 1) + r) + 1))             # <<<<<<<<<<<<<<
 *                d[j] = ((2*r + 1)*(2*r + 2) - 4*j*j)*alpha + (2*j + 1)*(2*j + 1)*beta
 *            else:
 */
        (__pyx_v_f[__pyx_v_j]) = (((-__pyx_v_alpha) * ((2 * (__pyx_v_r - (__pyx_v_j + 1))) + 2)) * ((2 * ((__pyx_v_j + 1) + __pyx_v_r)) + 1));

        /* "_ellip_harm.pxd":112
 *            if n%2:
 *                f[j] = (-alpha*(2*(r- (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 *                d[j] = ((2*r + 1)*(2*r + 2) - 4*j*j)*alpha + (2*j + 1)*(2*j + 1)*beta             # <<<<<<<<<<<<<<
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)) + 2)*(2*(r + (j + 1)) - 1))
 */
        (__pyx_v_d[__pyx_v_j]) = ((((((2 * __pyx_v_r) + 1) * ((2 * __pyx_v_r) + 2)) - ((4 * __pyx_v_j) * __pyx_v_j)) * __pyx_v_alpha) + ((((2 * __pyx_v_j) + 1) * ((2 * __pyx_v_j) + 1)) * __pyx_v_beta));

        /* "_ellip_harm.pxd":110
 *         for j in range(0, r + 1):
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 *            if n%2:             # <<<<<<<<<<<<<<
 *                f[j] = (-alpha*(2*(r- (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 *                d[j] = ((2*r + 1)*(2*r + 2) - 4*j*j)*alpha + (2*j + 1)*(2*j + 1)*beta
 */
        goto __pyx_L14;
      }

      /* "_ellip_harm.pxd":114
 *                d[j] = ((2*r + 1)*(2*r + 2) - 4*j*j)*alpha + (2*j + 1)*(2*j + 1)*beta
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)) + 2)*(2*(r + (j + 1)) - 1))             # <<<<<<<<<<<<<<
 *                d[j] = 2*r*(2*r + 1)*alpha - 4*j*j*gamma
 * 
 */
      /*else*/ {
        (__pyx_v_f[__pyx_v_j]) = (((-__pyx_v_alpha) * ((2 * (__pyx_v_r - (__pyx_v_j + 1))) + 2)) * ((2 * (__pyx_v_r + (__pyx_v_j + 1))) - 1));

        /* "_ellip_harm.pxd":115
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)) + 2)*(2*(r + (j + 1)) - 1))
 *                d[j] = 2*r*(2*r + 1)*alpha - 4*j*j*gamma             # <<<<<<<<<<<<<<
 * 
 *     elif t == 'L':
 */
        (__pyx_v_d[__pyx_v_j]) = ((((2 * __pyx_v_r) * ((2 * __pyx_v_r) + 1)) * __pyx_v_alpha) - (((4 * __pyx_v_j) * __pyx_v_j) * __pyx_v_gamma));
      }
      __pyx_L14:;
    }

    /* "_ellip_harm.pxd":107
 *     cdef int *isuppz = iwork + liwork
 * 
 *     if t == 'K':             # <<<<<<<<<<<<<<
 *         for j in range(0, r + 1):
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 */
    break;
    case 76:

    /* "_ellip_harm.pxd":118
 * 
 *     elif t == 'L':
 *         for j in range(0, n - r):             # <<<<<<<<<<<<<<
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 *            if n%2:
 */
    __pyx_t_4 = (__pyx_v_n - __pyx_v_r);
    __pyx_t_7 = __pyx_t_4;
    for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
      __pyx_v_j = __pyx_t_8;

      /* "_ellip_harm.pxd":119
 *     elif t == 'L':
 *         for j in range(0, n - r):
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)             # <<<<<<<<<<<<<<
 *            if n%2:
 *                f[j] = (-alpha*(2*(r- (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 */
      (__pyx_v_g[__pyx_v_j]) = (((-((2 * __pyx_v_j) + 2)) * ((2 * __pyx_v_j) + 3)) * __pyx_v_beta);

      /* "_ellip_harm.pxd":120
 *         for j in range(0, n - r):
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 *            if n%2:             # <<<<<<<<<<<<<<
 *                f[j] = (-alpha*(2*(r- (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 *                d[j] = (2*r + 1)*(2*r + 2)*alpha - (2*j + 1)*(2*j + 1)*gamma
 */
      __pyx_t_1 = ((__pyx_v_n % 2) != 0);
      if (__pyx_t_1) {

        /* "_ellip_harm.pxd":121
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 *            if n%2:
 *                f[j] = (-alpha*(2*(r- (j + 1)) + 2)*(2*((j + 1) + r) + 1))             # <<<<<<<<<<<<<<
 *                d[j] = (2*r + 1)*(2*r + 2)*alpha - (2*j + 1)*(2*j + 1)*gamma
 *            else:
 */
        (__pyx_v_f[__pyx_v_j]) = (((-__pyx_v_alpha) * ((2 * (__pyx_v_r - (__pyx_v_j + 1))) + 2)) * ((2 * ((__pyx_v_j + 1) + __pyx_v_r)) + 1));

        /* "_ellip_harm.pxd":122
 *            if n%2:
 *                f[j] = (-alpha*(2*(r- (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 *                d[j] = (2*r + 1)*(2*r + 2)*alpha - (2*j + 1)*(2*j + 1)*gamma             # <<<<<<<<<<<<<<
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)))*(2*(r+(j + 1)) + 1))
 */
        (__pyx_v_d[__pyx_v_j]) = (((((2 * __pyx_v_r) + 1) * ((2 * __pyx_v_r) + 2)) * __pyx_v_alpha) - ((((2 * __pyx_v_j) + 1) * ((2 * __pyx_v_j) + 1)) * __pyx_v_gamma));

        /* "_ellip_harm.pxd":120
 *         for j in range(0, n - r):
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 *            if n%2:             # <<<<<<<<<<<<<<
 *                f[j] = (-alpha*(2*(r- (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 *                d[j] = (2*r + 1)*(2*r + 2)*alpha - (2*j + 1)*(2*j + 1)*gamma
 */
        goto __pyx_L17;
      }

      /* "_ellip_harm.pxd":124
 *                d[j] = (2*r + 1)*(2*r + 2)*alpha - (2*j + 1)*(2*j + 1)*gamma
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)))*(2*(r+(j + 1)) + 1))             # <<<<<<<<<<<<<<
 *                d[j] = (2*r*(2*r + 1) - (2*j + 1)*(2*j + 1))*alpha + (2*j + 2)*(2*j + 2)*beta
 * 
 */
      /*else*/ {
        (__pyx_v_f[__pyx_v_j]) = (((-__pyx_v_alpha) * (2 * (__pyx_v_r - (__pyx_v_j + 1)))) * ((2 * (__pyx_v_r + (__pyx_v_j + 1))) + 1));

        /* "_ellip_harm.pxd":125
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)))*(2*(r+(j + 1)) + 1))
 *                d[j] = (2*r*(2*r + 1) - (2*j + 1)*(2*j + 1))*alpha + (2*j + 2)*(2*j + 2)*beta             # <<<<<<<<<<<<<<
 * 
 *     elif t == 'M':
 */
        (__pyx_v_d[__pyx_v_j]) = (((((2 * __pyx_v_r) * ((2 * __pyx_v_r) + 1)) - (((2 * __pyx_v_j) + 1) * ((2 * __pyx_v_j) + 1))) * __pyx_v_alpha) + ((((2 * __pyx_v_j) + 2) * ((2 * __pyx_v_j) + 2)) * __pyx_v_beta));
      }
      __pyx_L17:;
    }

    /* "_ellip_harm.pxd":117
 *                d[j] = 2*r*(2*r + 1)*alpha - 4*j*j*gamma
 * 
 *     elif t == 'L':             # <<<<<<<<<<<<<<
 *         for j in range(0, n - r):
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 */
    break;
    case 77:

    /* "_ellip_harm.pxd":128
 * 
 *     elif t == 'M':
 *         for j in range(0, n - r):             # <<<<<<<<<<<<<<
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 *            if n%2:
 */
    __pyx_t_4 = (__pyx_v_n - __pyx_v_r);
    __pyx_t_7 = __pyx_t_4;
    for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
      __pyx_v_j = __pyx_t_8;

      /* "_ellip_harm.pxd":129
 *     elif t == 'M':
 *         for j in range(0, n - r):
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)             # <<<<<<<<<<<<<<
 *            if n%2:
 *                f[j] = (-alpha*(2*(r - (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 */
      (__pyx_v_g[__pyx_v_j]) = (((-((2 * __pyx_v_j) + 2)) * ((2 * __pyx_v_j) + 1)) * __pyx_v_beta);

      /* "_ellip_harm.pxd":130
 *         for j in range(0, n - r):
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 *            if n%2:             # <<<<<<<<<<<<<<
 *                f[j] = (-alpha*(2*(r - (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 *                d[j] = ((2*r + 1)*(2*r + 2) - (2*j + 1)*(2*j + 1))*alpha + 4*j*j*beta
 */
      __pyx_t_1 = ((__pyx_v_n % 2) != 0);
      if (__pyx_t_1) {

        /* "_ellip_harm.pxd":131
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 *            if n%2:
 *                f[j] = (-alpha*(2*(r - (j + 1)) + 2)*(2*((j + 1) + r) + 1))             # <<<<<<<<<<<<<<
 *                d[j] = ((2*r + 1)*(2*r + 2) - (2*j + 1)*(2*j + 1))*alpha + 4*j*j*beta
 *            else:
 */
        (__pyx_v_f[__pyx_v_j]) = (((-__pyx_v_alpha) * ((2 * (__pyx_v_r - (__pyx_v_j + 1))) + 2)) * ((2 * ((__pyx_v_j + 1) + __pyx_v_r)) + 1));

        /* "_ellip_harm.pxd":132
 *            if n%2:
 *                f[j] = (-alpha*(2*(r - (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 *                d[j] = ((2*r + 1)*(2*r + 2) - (2*j + 1)*(2*j + 1))*alpha + 4*j*j*beta             # <<<<<<<<<<<<<<
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)))*(2*(r+(j + 1)) + 1))
 */
        (__pyx_v_d[__pyx_v_j]) = ((((((2 * __pyx_v_r) + 1) * ((2 * __pyx_v_r) + 2)) - (((2 * __pyx_v_j) + 1) * ((2 * __pyx_v_j) + 1))) * __pyx_v_alpha) + (((4 * __pyx_v_j) * __pyx_v_j) * __pyx_v_beta));

        /* "_ellip_harm.pxd":130
 *         for j in range(0, n - r):
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 *            if n%2:             # <<<<<<<<<<<<<<
 *                f[j] = (-alpha*(2*(r - (j + 1)) + 2)*(2*((j + 1) + r) + 1))
 *                d[j] = ((2*r + 1)*(2*r + 2) - (2*j + 1)*(2*j + 1))*alpha + 4*j*j*beta
 */
        goto __pyx_L20;
      }

      /* "_ellip_harm.pxd":134
 *                d[j] = ((2*r + 1)*(2*r + 2) - (2*j + 1)*(2*j + 1))*alpha + 4*j*j*beta
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)))*(2*(r+(j + 1)) + 1))             # <<<<<<<<<<<<<<
 *                d[j] = 2*r*(2*r + 1)*alpha - (2*j + 1)*(2*j + 1)*gamma
 * 
 */
      /*else*/ {
        (__pyx_v_f[__pyx_v_j]) = (((-__pyx_v_alpha) * (2 * (__pyx_v_r - (__pyx_v_j + 1)))) * ((2 * (__pyx_v_r + (__pyx_v_j + 1))) + 1));

        /* "_ellip_harm.pxd":135
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)))*(2*(r+(j + 1)) + 1))
 *                d[j] = 2*r*(2*r + 1)*alpha - (2*j + 1)*(2*j + 1)*gamma             # <<<<<<<<<<<<<<
 * 
 *     elif t == 'N':
 */
        (__pyx_v_d[__pyx_v_j]) = ((((2 * __pyx_v_r) * ((2 * __pyx_v_r) + 1)) * __pyx_v_alpha) - ((((2 * __pyx_v_j) + 1) * ((2 * __pyx_v_j) + 1)) * __pyx_v_gamma));
      }
      __pyx_L20:;
    }

    /* "_ellip_harm.pxd":127
 *                d[j] = (2*r*(2*r + 1) - (2*j + 1)*(2*j + 1))*alpha + (2*j + 2)*(2*j + 2)*beta
 * 
 *     elif t == 'M':             # <<<<<<<<<<<<<<
 *         for j in range(0, n - r):
 *            g[j] = (-(2*j + 2)*(2*j + 1)*beta)
 */
    break;
    case 78:

    /* "_ellip_harm.pxd":138
 * 
 *     elif t == 'N':
 *         for j in range(0, r):             # <<<<<<<<<<<<<<
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 *            if n%2:
 */
    __pyx_t_4 = __pyx_v_r;
    __pyx_t_7 = __pyx_t_4;
    for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
      __pyx_v_j = __pyx_t_8;

      /* "_ellip_harm.pxd":139
 *     elif t == 'N':
 *         for j in range(0, r):
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)             # <<<<<<<<<<<<<<
 *            if n%2:
 *                f[j] = (-alpha*(2*(r- (j + 1)))*(2*((j + 1) + r) + 3))
 */
      (__pyx_v_g[__pyx_v_j]) = (((-((2 * __pyx_v_j) + 2)) * ((2 * __pyx_v_j) + 3)) * __pyx_v_beta);

      /* "_ellip_harm.pxd":140
 *         for j in range(0, r):
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 *            if n%2:             # <<<<<<<<<<<<<<
 *                f[j] = (-alpha*(2*(r- (j + 1)))*(2*((j + 1) + r) + 3))
 *                d[j] = (2*r + 1)*(2*r + 2)*alpha - (2*j + 2)*(2*j + 2)*gamma
 */
      __pyx_t_1 = ((__pyx_v_n % 2) != 0);
      if (__pyx_t_1) {

        /* "_ellip_harm.pxd":141
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 *            if n%2:
 *                f[j] = (-alpha*(2*(r- (j + 1)))*(2*((j + 1) + r) + 3))             # <<<<<<<<<<<<<<
 *                d[j] = (2*r + 1)*(2*r + 2)*alpha - (2*j + 2)*(2*j + 2)*gamma
 *            else:
 */
        (__pyx_v_f[__pyx_v_j]) = (((-__pyx_v_alpha) * (2 * (__pyx_v_r - (__pyx_v_j + 1)))) * ((2 * ((__pyx_v_j + 1) + __pyx_v_r)) + 3));

        /* "_ellip_harm.pxd":142
 *            if n%2:
 *                f[j] = (-alpha*(2*(r- (j + 1)))*(2*((j + 1) + r) + 3))
 *                d[j] = (2*r + 1)*(2*r + 2)*alpha - (2*j + 2)*(2*j + 2)*gamma             # <<<<<<<<<<<<<<
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)))*(2*(r+(j + 1)) + 1))
 */
        (__pyx_v_d[__pyx_v_j]) = (((((2 * __pyx_v_r) + 1) * ((2 * __pyx_v_r) + 2)) * __pyx_v_alpha) - ((((2 * __pyx_v_j) + 2) * ((2 * __pyx_v_j) + 2)) * __pyx_v_gamma));

        /* "_ellip_harm.pxd":140
 *         for j in range(0, r):
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 *            if n%2:             # <<<<<<<<<<<<<<
 *                f[j] = (-alpha*(2*(r- (j + 1)))*(2*((j + 1) + r) + 3))
 *                d[j] = (2*r + 1)*(2*r + 2)*alpha - (2*j + 2)*(2*j + 2)*gamma
 */
        goto __pyx_L23;
      }

      /* "_ellip_harm.pxd":144
 *                d[j] = (2*r + 1)*(2*r + 2)*alpha - (2*j + 2)*(2*j + 2)*gamma
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)))*(2*(r+(j + 1)) + 1))             # <<<<<<<<<<<<<<
 *                d[j] = 2*r*(2*r + 1)*alpha - (2*j + 2)*(2*j +2)*alpha + (2*j + 1)*(2*j + 1)*beta
 * 
 */
      /*else*/ {
        (__pyx_v_f[__pyx_v_j]) = (((-__pyx_v_alpha) * (2 * (__pyx_v_r - (__pyx_v_j + 1)))) * ((2 * (__pyx_v_r + (__pyx_v_j + 1))) + 1));

        /* "_ellip_harm.pxd":145
 *            else:
 *                f[j] = (-alpha*(2*(r - (j + 1)))*(2*(r+(j + 1)) + 1))
 *                d[j] = 2*r*(2*r + 1)*alpha - (2*j + 2)*(2*j +2)*alpha + (2*j + 1)*(2*j + 1)*beta             # <<<<<<<<<<<<<<
 * 
 *     for i in range(0, size):
 */
        (__pyx_v_d[__pyx_v_j]) = (((((2 * __pyx_v_r) * ((2 * __pyx_v_r) + 1)) * __pyx_v_alpha) - ((((2 * __pyx_v_j) + 2) * ((2 * __pyx_v_j) + 2)) * __pyx_v_alpha)) + ((((2 * __pyx_v_j) + 1) * ((2 * __pyx_v_j) + 1)) * __pyx_v_beta));
      }
      __pyx_L23:;
    }

    /* "_ellip_harm.pxd":137
 *                d[j] = 2*r*(2*r + 1)*alpha - (2*j + 1)*(2*j + 1)*gamma
 * 
 *     elif t == 'N':             # <<<<<<<<<<<<<<
 *         for j in range(0, r):
 *            g[j] = (-(2*j + 2)*(2*j + 3)*beta)
 */
    break;
    default: break;
  }

  /* "_ellip_harm.pxd":147
 *                d[j] = 2*r*(2*r + 1)*alpha - (2*j + 2)*(2*j +2)*alpha + (2*j + 1)*(2*j + 1)*beta
 * 
 *     for i in range(0, size):             # <<<<<<<<<<<<<<
 *         if i == 0:
 *            ss[i] = 1
 */
  __pyx_t_4 = __pyx_v_size;
  __pyx_t_7 = __pyx_t_4;
  for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
    __pyx_v_i = __pyx_t_8;

    /* "_ellip_harm.pxd":148
 * 
 *     for i in range(0, size):
 *         if i == 0:             # <<<<<<<<<<<<<<
 *            ss[i] = 1
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_i == 0) != 0);
    if (__pyx_t_1) {

      /* "_ellip_harm.pxd":149
 *     for i in range(0, size):
 *         if i == 0:
 *            ss[i] = 1             # <<<<<<<<<<<<<<
 *         else:
 *            ss[i] = sqrt(g[i - 1]/f[i - 1])*ss[i - 1]
 */
      (__pyx_v_ss[__pyx_v_i]) = 1.0;

      /* "_ellip_harm.pxd":148
 * 
 *     for i in range(0, size):
 *         if i == 0:             # <<<<<<<<<<<<<<
 *            ss[i] = 1
 *         else:
 */
      goto __pyx_L26;
    }

    /* "_ellip_harm.pxd":151
 *            ss[i] = 1
 *         else:
 *            ss[i] = sqrt(g[i - 1]/f[i - 1])*ss[i - 1]             # <<<<<<<<<<<<<<
 * 
 *     for i in range(0, size-1):
 */
    /*else*/ {
      (__pyx_v_ss[__pyx_v_i]) = (sqrt(((__pyx_v_g[(__pyx_v_i - 1)]) / (__pyx_v_f[(__pyx_v_i - 1)]))) * (__pyx_v_ss[(__pyx_v_i - 1)]));
    }
    __pyx_L26:;
  }

  /* "_ellip_harm.pxd":153
 *            ss[i] = sqrt(g[i - 1]/f[i - 1])*ss[i - 1]
 * 
 *     for i in range(0, size-1):             # <<<<<<<<<<<<<<
 *         dd[i] = g[i]*ss[i]/ss[i+1]
 * 
 */
  __pyx_t_5 = (__pyx_v_size - 1);
  __pyx_t_6 = __pyx_t_5;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_6; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "_ellip_harm.pxd":154
 * 
 *     for i in range(0, size-1):
 *         dd[i] = g[i]*ss[i]/ss[i+1]             # <<<<<<<<<<<<<<
 * 
 *     c_dstevr("V", "I", &size, d, dd, &vl, &vu, &tp, &tp, &tol, &c, w, eigv,
 */
    (__pyx_v_dd[__pyx_v_i]) = (((__pyx_v_g[__pyx_v_i]) * (__pyx_v_ss[__pyx_v_i])) / (__pyx_v_ss[(__pyx_v_i + 1)]));
  }

  /* "_ellip_harm.pxd":156
 *         dd[i] = g[i]*ss[i]/ss[i+1]
 * 
 *     c_dstevr("V", "I", &size, d, dd, &vl, &vu, &tp, &tp, &tol, &c, w, eigv,             # <<<<<<<<<<<<<<
 *              &size, isuppz, work, &lwork, iwork, &liwork, &info)
 * 
 */
  c_dstevr(((char *)"V"), ((char *)"I"), (&__pyx_v_size), __pyx_v_d, __pyx_v_dd, (&__pyx_v_vl), (&__pyx_v_vu), (&__pyx_v_tp), (&__pyx_v_tp), (&__pyx_v_tol), (&__pyx_v_c), __pyx_v_w, __pyx_v_eigv, (&__pyx_v_size), __pyx_v_isuppz, __pyx_v_work, (&__pyx_v_lwork), __pyx_v_iwork, (&__pyx_v_liwork), (&__pyx_v_info));

  /* "_ellip_harm.pxd":159
 *              &size, isuppz, work, &lwork, iwork, &liwork, &info)
 * 
 *     if info != 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("ellip_harm", sf_error.NO_RESULT, "failed to allocate memory")
 *         return NULL
 */
  __pyx_t_1 = ((__pyx_v_info != 0) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":160
 * 
 *     if info != 0:
 *         sf_error.error("ellip_harm", sf_error.NO_RESULT, "failed to allocate memory")             # <<<<<<<<<<<<<<
 *         return NULL
 * 
 */
    sf_error(((char *)"ellip_harm"), SF_ERROR_NO_RESULT, ((char *)"failed to allocate memory"));

    /* "_ellip_harm.pxd":161
 *     if info != 0:
 *         sf_error.error("ellip_harm", sf_error.NO_RESULT, "failed to allocate memory")
 *         return NULL             # <<<<<<<<<<<<<<
 * 
 *     for i in range(0, size):
 */
    __pyx_r = NULL;
    goto __pyx_L0;

    /* "_ellip_harm.pxd":159
 *              &size, isuppz, work, &lwork, iwork, &liwork, &info)
 * 
 *     if info != 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("ellip_harm", sf_error.NO_RESULT, "failed to allocate memory")
 *         return NULL
 */
  }

  /* "_ellip_harm.pxd":163
 *         return NULL
 * 
 *     for i in range(0, size):             # <<<<<<<<<<<<<<
 *         eigv[i] /= ss[i]
 * 
 */
  __pyx_t_4 = __pyx_v_size;
  __pyx_t_7 = __pyx_t_4;
  for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
    __pyx_v_i = __pyx_t_8;

    /* "_ellip_harm.pxd":164
 * 
 *     for i in range(0, size):
 *         eigv[i] /= ss[i]             # <<<<<<<<<<<<<<
 * 
 *     for i in range(0, size):
 */
    __pyx_t_9 = __pyx_v_i;
    (__pyx_v_eigv[__pyx_t_9]) = ((__pyx_v_eigv[__pyx_t_9]) / (__pyx_v_ss[__pyx_v_i]));
  }

  /* "_ellip_harm.pxd":166
 *         eigv[i] /= ss[i]
 * 
 *     for i in range(0, size):             # <<<<<<<<<<<<<<
 *         eigv[i] = eigv[i]/(eigv[size - 1]/pow(-h2, size - 1))
 *     return eigv
 */
  __pyx_t_4 = __pyx_v_size;
  __pyx_t_7 = __pyx_t_4;
  for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
    __pyx_v_i = __pyx_t_8;

    /* "_ellip_harm.pxd":167
 * 
 *     for i in range(0, size):
 *         eigv[i] = eigv[i]/(eigv[size - 1]/pow(-h2, size - 1))             # <<<<<<<<<<<<<<
 *     return eigv
 * 
 */
    (__pyx_v_eigv[__pyx_v_i]) = ((__pyx_v_eigv[__pyx_v_i]) / ((__pyx_v_eigv[(__pyx_v_size - 1)]) / pow((-__pyx_v_h2), (__pyx_v_size - 1))));
  }

  /* "_ellip_harm.pxd":168
 *     for i in range(0, size):
 *         eigv[i] = eigv[i]/(eigv[size - 1]/pow(-h2, size - 1))
 *     return eigv             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_eigv;
  goto __pyx_L0;

  /* "_ellip_harm.pxd":49
 * @cython.boundscheck(False)
 * @cython.cdivision(True)
 * cdef inline double* lame_coefficients(double h2, double k2, int n, int p,             # <<<<<<<<<<<<<<
 *                                       void **bufferp, double signm,
 *                                       double signn) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_ellip_harm.pxd":174
 * @cython.boundscheck(False)
 * @cython.cdivision(True)
 * cdef inline double ellip_harm_eval(double h2, double k2, int n, int p,             # <<<<<<<<<<<<<<
 *                                    double s, double *eigv, double signm,
 *                                    double signn) nogil:
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_11_ellip_harm_ellip_harm_eval(double __pyx_v_h2, double __pyx_v_k2, int __pyx_v_n, int __pyx_v_p, double __pyx_v_s, double *__pyx_v_eigv, double __pyx_v_signm, double __pyx_v_signn) {
  int __pyx_v_size;
  int __pyx_v_r;
  int __pyx_v_j;
  double __pyx_v_s2;
  double __pyx_v_pp;
  double __pyx_v_lambda_romain;
  double __pyx_v_psi;
  double __pyx_r;
  int __pyx_t_1;
  long __pyx_t_2;
  double __pyx_t_3;
  int __pyx_t_4;

  /* "_ellip_harm.pxd":179
 *     cdef int size, tp, r, j
 *     cdef double s2, pp, lambda_romain, psi
 *     s2 = s*s             # <<<<<<<<<<<<<<
 *     r = n/2
 *     if p - 1 < r + 1:
 */
  __pyx_v_s2 = (__pyx_v_s * __pyx_v_s);

  /* "_ellip_harm.pxd":180
 *     cdef double s2, pp, lambda_romain, psi
 *     s2 = s*s
 *     r = n/2             # <<<<<<<<<<<<<<
 *     if p - 1 < r + 1:
 *         size, psi = r + 1, pow(s, n - 2*r)
 */
  __pyx_v_r = (((long)__pyx_v_n) / 2);

  /* "_ellip_harm.pxd":181
 *     s2 = s*s
 *     r = n/2
 *     if p - 1 < r + 1:             # <<<<<<<<<<<<<<
 *         size, psi = r + 1, pow(s, n - 2*r)
 *     elif p - 1 < (n - r) + (r + 1):
 */
  __pyx_t_1 = (((__pyx_v_p - 1) < (__pyx_v_r + 1)) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":182
 *     r = n/2
 *     if p - 1 < r + 1:
 *         size, psi = r + 1, pow(s, n - 2*r)             # <<<<<<<<<<<<<<
 *     elif p - 1 < (n - r) + (r + 1):
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signm*sqrt(fabs(s2 - h2))
 */
    __pyx_t_2 = (__pyx_v_r + 1);
    __pyx_t_3 = pow(__pyx_v_s, (__pyx_v_n - (2 * __pyx_v_r)));
    __pyx_v_size = __pyx_t_2;
    __pyx_v_psi = __pyx_t_3;

    /* "_ellip_harm.pxd":181
 *     s2 = s*s
 *     r = n/2
 *     if p - 1 < r + 1:             # <<<<<<<<<<<<<<
 *         size, psi = r + 1, pow(s, n - 2*r)
 *     elif p - 1 < (n - r) + (r + 1):
 */
    goto __pyx_L3;
  }

  /* "_ellip_harm.pxd":183
 *     if p - 1 < r + 1:
 *         size, psi = r + 1, pow(s, n - 2*r)
 *     elif p - 1 < (n - r) + (r + 1):             # <<<<<<<<<<<<<<
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signm*sqrt(fabs(s2 - h2))
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 */
  __pyx_t_1 = (((__pyx_v_p - 1) < ((__pyx_v_n - __pyx_v_r) + (__pyx_v_r + 1))) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":184
 *         size, psi = r + 1, pow(s, n - 2*r)
 *     elif p - 1 < (n - r) + (r + 1):
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signm*sqrt(fabs(s2 - h2))             # <<<<<<<<<<<<<<
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signn*sqrt(fabs(s2 - k2))
 */
    __pyx_t_4 = (__pyx_v_n - __pyx_v_r);
    __pyx_t_3 = ((pow(__pyx_v_s, ((1 - __pyx_v_n) + (2 * __pyx_v_r))) * __pyx_v_signm) * sqrt(fabs((__pyx_v_s2 - __pyx_v_h2))));
    __pyx_v_size = __pyx_t_4;
    __pyx_v_psi = __pyx_t_3;

    /* "_ellip_harm.pxd":183
 *     if p - 1 < r + 1:
 *         size, psi = r + 1, pow(s, n - 2*r)
 *     elif p - 1 < (n - r) + (r + 1):             # <<<<<<<<<<<<<<
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signm*sqrt(fabs(s2 - h2))
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 */
    goto __pyx_L3;
  }

  /* "_ellip_harm.pxd":185
 *     elif p - 1 < (n - r) + (r + 1):
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signm*sqrt(fabs(s2 - h2))
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):             # <<<<<<<<<<<<<<
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signn*sqrt(fabs(s2 - k2))
 *     elif p - 1 < 2*n + 1:
 */
  __pyx_t_1 = (((__pyx_v_p - 1) < (((__pyx_v_n - __pyx_v_r) + (__pyx_v_n - __pyx_v_r)) + (__pyx_v_r + 1))) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":186
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signm*sqrt(fabs(s2 - h2))
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signn*sqrt(fabs(s2 - k2))             # <<<<<<<<<<<<<<
 *     elif p - 1 < 2*n + 1:
 *         size, psi = r, pow(s,  n - 2*r)*signm*signn*sqrt(fabs((s2 - h2)*(s2 - k2)))
 */
    __pyx_t_4 = (__pyx_v_n - __pyx_v_r);
    __pyx_t_3 = ((pow(__pyx_v_s, ((1 - __pyx_v_n) + (2 * __pyx_v_r))) * __pyx_v_signn) * sqrt(fabs((__pyx_v_s2 - __pyx_v_k2))));
    __pyx_v_size = __pyx_t_4;
    __pyx_v_psi = __pyx_t_3;

    /* "_ellip_harm.pxd":185
 *     elif p - 1 < (n - r) + (r + 1):
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signm*sqrt(fabs(s2 - h2))
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):             # <<<<<<<<<<<<<<
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signn*sqrt(fabs(s2 - k2))
 *     elif p - 1 < 2*n + 1:
 */
    goto __pyx_L3;
  }

  /* "_ellip_harm.pxd":187
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signn*sqrt(fabs(s2 - k2))
 *     elif p - 1 < 2*n + 1:             # <<<<<<<<<<<<<<
 *         size, psi = r, pow(s,  n - 2*r)*signm*signn*sqrt(fabs((s2 - h2)*(s2 - k2)))
 *     lambda_romain = 1.0 - <double>s2/<double>h2
 */
  __pyx_t_1 = (((__pyx_v_p - 1) < ((2 * __pyx_v_n) + 1)) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":188
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signn*sqrt(fabs(s2 - k2))
 *     elif p - 1 < 2*n + 1:
 *         size, psi = r, pow(s,  n - 2*r)*signm*signn*sqrt(fabs((s2 - h2)*(s2 - k2)))             # <<<<<<<<<<<<<<
 *     lambda_romain = 1.0 - <double>s2/<double>h2
 *     pp = eigv[size - 1]
 */
    __pyx_t_4 = __pyx_v_r;
    __pyx_t_3 = (((pow(__pyx_v_s, (__pyx_v_n - (2 * __pyx_v_r))) * __pyx_v_signm) * __pyx_v_signn) * sqrt(fabs(((__pyx_v_s2 - __pyx_v_h2) * (__pyx_v_s2 - __pyx_v_k2)))));
    __pyx_v_size = __pyx_t_4;
    __pyx_v_psi = __pyx_t_3;

    /* "_ellip_harm.pxd":187
 *     elif p - 1 < (n - r) + (n - r) + (r + 1):
 *         size, psi = n - r, pow(s, 1 - n + 2*r)*signn*sqrt(fabs(s2 - k2))
 *     elif p - 1 < 2*n + 1:             # <<<<<<<<<<<<<<
 *         size, psi = r, pow(s,  n - 2*r)*signm*signn*sqrt(fabs((s2 - h2)*(s2 - k2)))
 *     lambda_romain = 1.0 - <double>s2/<double>h2
 */
  }
  __pyx_L3:;

  /* "_ellip_harm.pxd":189
 *     elif p - 1 < 2*n + 1:
 *         size, psi = r, pow(s,  n - 2*r)*signm*signn*sqrt(fabs((s2 - h2)*(s2 - k2)))
 *     lambda_romain = 1.0 - <double>s2/<double>h2             # <<<<<<<<<<<<<<
 *     pp = eigv[size - 1]
 * 
 */
  __pyx_v_lambda_romain = (1.0 - (((double)__pyx_v_s2) / ((double)__pyx_v_h2)));

  /* "_ellip_harm.pxd":190
 *         size, psi = r, pow(s,  n - 2*r)*signm*signn*sqrt(fabs((s2 - h2)*(s2 - k2)))
 *     lambda_romain = 1.0 - <double>s2/<double>h2
 *     pp = eigv[size - 1]             # <<<<<<<<<<<<<<
 * 
 *     for j in range(size - 2, -1, -1):
 */
  __pyx_v_pp = (__pyx_v_eigv[(__pyx_v_size - 1)]);

  /* "_ellip_harm.pxd":192
 *     pp = eigv[size - 1]
 * 
 *     for j in range(size - 2, -1, -1):             # <<<<<<<<<<<<<<
 *         pp = pp*lambda_romain + eigv[j]
 *     pp = pp*psi
 */
  for (__pyx_t_4 = (__pyx_v_size - 2); __pyx_t_4 > -1; __pyx_t_4-=1) {
    __pyx_v_j = __pyx_t_4;

    /* "_ellip_harm.pxd":193
 * 
 *     for j in range(size - 2, -1, -1):
 *         pp = pp*lambda_romain + eigv[j]             # <<<<<<<<<<<<<<
 *     pp = pp*psi
 *     return pp
 */
    __pyx_v_pp = ((__pyx_v_pp * __pyx_v_lambda_romain) + (__pyx_v_eigv[__pyx_v_j]));
  }

  /* "_ellip_harm.pxd":194
 *     for j in range(size - 2, -1, -1):
 *         pp = pp*lambda_romain + eigv[j]
 *     pp = pp*psi             # <<<<<<<<<<<<<<
 *     return pp
 * 
 */
  __pyx_v_pp = (__pyx_v_pp * __pyx_v_psi);

  /* "_ellip_harm.pxd":195
 *         pp = pp*lambda_romain + eigv[j]
 *     pp = pp*psi
 *     return pp             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_pp;
  goto __pyx_L0;

  /* "_ellip_harm.pxd":174
 * @cython.boundscheck(False)
 * @cython.cdivision(True)
 * cdef inline double ellip_harm_eval(double h2, double k2, int n, int p,             # <<<<<<<<<<<<<<
 *                                    double s, double *eigv, double signm,
 *                                    double signn) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_ellip_harm.pxd":198
 * 
 * 
 * cdef inline double ellip_harmonic(double h2, double k2, int n, int p, double s,             # <<<<<<<<<<<<<<
 *                                   double signm, double signn) nogil:
 *     cdef double result
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_11_ellip_harm_ellip_harmonic(double __pyx_v_h2, double __pyx_v_k2, int __pyx_v_n, int __pyx_v_p, double __pyx_v_s, double __pyx_v_signm, double __pyx_v_signn) {
  double __pyx_v_result;
  double *__pyx_v_eigv;
  void *__pyx_v_bufferp;
  double __pyx_r;
  int __pyx_t_1;

  /* "_ellip_harm.pxd":203
 *     cdef double *eigv
 *     cdef void *bufferp
 *     eigv = lame_coefficients(h2, k2, n, p, &bufferp, signm, signn)             # <<<<<<<<<<<<<<
 *     if not eigv:
 *         free(bufferp)
 */
  __pyx_v_eigv = __pyx_f_5scipy_7special_11_ellip_harm_lame_coefficients(__pyx_v_h2, __pyx_v_k2, __pyx_v_n, __pyx_v_p, (&__pyx_v_bufferp), __pyx_v_signm, __pyx_v_signn);

  /* "_ellip_harm.pxd":204
 *     cdef void *bufferp
 *     eigv = lame_coefficients(h2, k2, n, p, &bufferp, signm, signn)
 *     if not eigv:             # <<<<<<<<<<<<<<
 *         free(bufferp)
 *         return NAN
 */
  __pyx_t_1 = ((!(__pyx_v_eigv != 0)) != 0);
  if (__pyx_t_1) {

    /* "_ellip_harm.pxd":205
 *     eigv = lame_coefficients(h2, k2, n, p, &bufferp, signm, signn)
 *     if not eigv:
 *         free(bufferp)             # <<<<<<<<<<<<<<
 *         return NAN
 *     result = ellip_harm_eval(h2, k2, n, p, s, eigv, signm, signn)
 */
    free(__pyx_v_bufferp);

    /* "_ellip_harm.pxd":206
 *     if not eigv:
 *         free(bufferp)
 *         return NAN             # <<<<<<<<<<<<<<
 *     result = ellip_harm_eval(h2, k2, n, p, s, eigv, signm, signn)
 *     free(bufferp)
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "_ellip_harm.pxd":204
 *     cdef void *bufferp
 *     eigv = lame_coefficients(h2, k2, n, p, &bufferp, signm, signn)
 *     if not eigv:             # <<<<<<<<<<<<<<
 *         free(bufferp)
 *         return NAN
 */
  }

  /* "_ellip_harm.pxd":207
 *         free(bufferp)
 *         return NAN
 *     result = ellip_harm_eval(h2, k2, n, p, s, eigv, signm, signn)             # <<<<<<<<<<<<<<
 *     free(bufferp)
 *     return result
 */
  __pyx_v_result = __pyx_f_5scipy_7special_11_ellip_harm_ellip_harm_eval(__pyx_v_h2, __pyx_v_k2, __pyx_v_n, __pyx_v_p, __pyx_v_s, __pyx_v_eigv, __pyx_v_signm, __pyx_v_signn);

  /* "_ellip_harm.pxd":208
 *         return NAN
 *     result = ellip_harm_eval(h2, k2, n, p, s, eigv, signm, signn)
 *     free(bufferp)             # <<<<<<<<<<<<<<
 *     return result
 */
  free(__pyx_v_bufferp);

  /* "_ellip_harm.pxd":209
 *     result = ellip_harm_eval(h2, k2, n, p, s, eigv, signm, signn)
 *     free(bufferp)
 *     return result             # <<<<<<<<<<<<<<
 */
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "_ellip_harm.pxd":198
 * 
 * 
 * cdef inline double ellip_harmonic(double h2, double k2, int n, int p, double s,             # <<<<<<<<<<<<<<
 *                                   double signm, double signn) nogil:
 *     cdef double result
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "sph_harm.pxd":13
 * from libc.stdlib cimport abs
 * 
 * cdef inline double complex sph_harmonic(int m, int n, double theta, double phi) nogil:             # <<<<<<<<<<<<<<
 *     cdef double x, prefactor
 *     cdef double complex val
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8sph_harm_sph_harmonic(int __pyx_v_m, int __pyx_v_n, double __pyx_v_theta, double __pyx_v_phi) {
  double __pyx_v_x;
  double __pyx_v_prefactor;
  __pyx_t_double_complex __pyx_v_val;
  int __pyx_v_mp;
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  double __pyx_t_2;

  /* "sph_harm.pxd":17
 *     cdef double complex val
 *     cdef int mp
 *     x = cos(phi)             # <<<<<<<<<<<<<<
 *     if abs(m) > n :
 *         sf_error.error("sph_harm", sf_error.ARG, "m should not be greater than n")
 */
  __pyx_v_x = cos(__pyx_v_phi);

  /* "sph_harm.pxd":18
 *     cdef int mp
 *     x = cos(phi)
 *     if abs(m) > n :             # <<<<<<<<<<<<<<
 *         sf_error.error("sph_harm", sf_error.ARG, "m should not be greater than n")
 *         return nan
 */
  __pyx_t_1 = ((abs(__pyx_v_m) > __pyx_v_n) != 0);
  if (__pyx_t_1) {

    /* "sph_harm.pxd":19
 *     x = cos(phi)
 *     if abs(m) > n :
 *         sf_error.error("sph_harm", sf_error.ARG, "m should not be greater than n")             # <<<<<<<<<<<<<<
 *         return nan
 *     if n < 0:
 */
    sf_error(((char *)"sph_harm"), SF_ERROR_ARG, ((char *)"m should not be greater than n"));

    /* "sph_harm.pxd":20
 *     if abs(m) > n :
 *         sf_error.error("sph_harm", sf_error.ARG, "m should not be greater than n")
 *         return nan             # <<<<<<<<<<<<<<
 *     if n < 0:
 *         sf_error.error("sph_harm", sf_error.ARG, "n should not be negative")
 */
    __pyx_r = __pyx_t_double_complex_from_parts(NPY_NAN, 0);
    goto __pyx_L0;

    /* "sph_harm.pxd":18
 *     cdef int mp
 *     x = cos(phi)
 *     if abs(m) > n :             # <<<<<<<<<<<<<<
 *         sf_error.error("sph_harm", sf_error.ARG, "m should not be greater than n")
 *         return nan
 */
  }

  /* "sph_harm.pxd":21
 *         sf_error.error("sph_harm", sf_error.ARG, "m should not be greater than n")
 *         return nan
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("sph_harm", sf_error.ARG, "n should not be negative")
 *         return nan
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "sph_harm.pxd":22
 *         return nan
 *     if n < 0:
 *         sf_error.error("sph_harm", sf_error.ARG, "n should not be negative")             # <<<<<<<<<<<<<<
 *         return nan
 *     if m < 0:
 */
    sf_error(((char *)"sph_harm"), SF_ERROR_ARG, ((char *)"n should not be negative"));

    /* "sph_harm.pxd":23
 *     if n < 0:
 *         sf_error.error("sph_harm", sf_error.ARG, "n should not be negative")
 *         return nan             # <<<<<<<<<<<<<<
 *     if m < 0:
 *         mp = -m
 */
    __pyx_r = __pyx_t_double_complex_from_parts(NPY_NAN, 0);
    goto __pyx_L0;

    /* "sph_harm.pxd":21
 *         sf_error.error("sph_harm", sf_error.ARG, "m should not be greater than n")
 *         return nan
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("sph_harm", sf_error.ARG, "n should not be negative")
 *         return nan
 */
  }

  /* "sph_harm.pxd":24
 *         sf_error.error("sph_harm", sf_error.ARG, "n should not be negative")
 *         return nan
 *     if m < 0:             # <<<<<<<<<<<<<<
 *         mp = -m
 *         prefactor = (-1)**mp * poch(n + mp + 1, -2 * mp)
 */
  __pyx_t_1 = ((__pyx_v_m < 0) != 0);
  if (__pyx_t_1) {

    /* "sph_harm.pxd":25
 *         return nan
 *     if m < 0:
 *         mp = -m             # <<<<<<<<<<<<<<
 *         prefactor = (-1)**mp * poch(n + mp + 1, -2 * mp)
 *     else:
 */
    __pyx_v_mp = (-__pyx_v_m);

    /* "sph_harm.pxd":26
 *     if m < 0:
 *         mp = -m
 *         prefactor = (-1)**mp * poch(n + mp + 1, -2 * mp)             # <<<<<<<<<<<<<<
 *     else:
 *         mp = m
 */
    __pyx_v_prefactor = (__Pyx_pow_long(-1L, ((long)__pyx_v_mp)) * poch(((__pyx_v_n + __pyx_v_mp) + 1), (-2L * __pyx_v_mp)));

    /* "sph_harm.pxd":24
 *         sf_error.error("sph_harm", sf_error.ARG, "n should not be negative")
 *         return nan
 *     if m < 0:             # <<<<<<<<<<<<<<
 *         mp = -m
 *         prefactor = (-1)**mp * poch(n + mp + 1, -2 * mp)
 */
    goto __pyx_L5;
  }

  /* "sph_harm.pxd":28
 *         prefactor = (-1)**mp * poch(n + mp + 1, -2 * mp)
 *     else:
 *         mp = m             # <<<<<<<<<<<<<<
 *     val = pmv_wrap(mp, n, x)
 *     if  m < 0:
 */
  /*else*/ {
    __pyx_v_mp = __pyx_v_m;
  }
  __pyx_L5:;

  /* "sph_harm.pxd":29
 *     else:
 *         mp = m
 *     val = pmv_wrap(mp, n, x)             # <<<<<<<<<<<<<<
 *     if  m < 0:
 *         val *= prefactor
 */
  __pyx_v_val = __pyx_t_double_complex_from_parts(pmv_wrap(__pyx_v_mp, __pyx_v_n, __pyx_v_x), 0);

  /* "sph_harm.pxd":30
 *         mp = m
 *     val = pmv_wrap(mp, n, x)
 *     if  m < 0:             # <<<<<<<<<<<<<<
 *         val *= prefactor
 *     val *= sqrt((2*n + 1) / 4.0 / pi)
 */
  __pyx_t_1 = ((__pyx_v_m < 0) != 0);
  if (__pyx_t_1) {

    /* "sph_harm.pxd":31
 *     val = pmv_wrap(mp, n, x)
 *     if  m < 0:
 *         val *= prefactor             # <<<<<<<<<<<<<<
 *     val *= sqrt((2*n + 1) / 4.0 / pi)
 *     val *= sqrt(poch(n + m + 1, -2 * m))
 */
    __pyx_v_val = __Pyx_c_prod_double(__pyx_v_val, __pyx_t_double_complex_from_parts(__pyx_v_prefactor, 0));

    /* "sph_harm.pxd":30
 *         mp = m
 *     val = pmv_wrap(mp, n, x)
 *     if  m < 0:             # <<<<<<<<<<<<<<
 *         val *= prefactor
 *     val *= sqrt((2*n + 1) / 4.0 / pi)
 */
  }

  /* "sph_harm.pxd":32
 *     if  m < 0:
 *         val *= prefactor
 *     val *= sqrt((2*n + 1) / 4.0 / pi)             # <<<<<<<<<<<<<<
 *     val *= sqrt(poch(n + m + 1, -2 * m))
 *     val *= zexp(1j * m * theta)
 */
  __pyx_t_2 = (((double)((2 * __pyx_v_n) + 1)) / 4.0);
  if (unlikely(NPY_PI == 0)) {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
    #endif
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    #ifdef WITH_THREAD
    __Pyx_PyGILState_Release(__pyx_gilstate_save);
    #endif
    __PYX_ERR(3, 32, __pyx_L1_error)
  }
  __pyx_v_val = __Pyx_c_prod_double(__pyx_v_val, __pyx_t_double_complex_from_parts(sqrt((__pyx_t_2 / NPY_PI)), 0));

  /* "sph_harm.pxd":33
 *         val *= prefactor
 *     val *= sqrt((2*n + 1) / 4.0 / pi)
 *     val *= sqrt(poch(n + m + 1, -2 * m))             # <<<<<<<<<<<<<<
 *     val *= zexp(1j * m * theta)
 *     return val
 */
  __pyx_v_val = __Pyx_c_prod_double(__pyx_v_val, __pyx_t_double_complex_from_parts(sqrt(poch(((__pyx_v_n + __pyx_v_m) + 1), (-2L * __pyx_v_m))), 0));

  /* "sph_harm.pxd":34
 *     val *= sqrt((2*n + 1) / 4.0 / pi)
 *     val *= sqrt(poch(n + m + 1, -2 * m))
 *     val *= zexp(1j * m * theta)             # <<<<<<<<<<<<<<
 *     return val
 */
  __pyx_v_val = __Pyx_c_prod_double(__pyx_v_val, __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zexp(__Pyx_c_prod_double(__Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0, 1.0), __pyx_t_double_complex_from_parts(__pyx_v_m, 0)), __pyx_t_double_complex_from_parts(__pyx_v_theta, 0))));

  /* "sph_harm.pxd":35
 *     val *= sqrt(poch(n + m + 1, -2 * m))
 *     val *= zexp(1j * m * theta)
 *     return val             # <<<<<<<<<<<<<<
 */
  __pyx_r = __pyx_v_val;
  goto __pyx_L0;

  /* "sph_harm.pxd":13
 * from libc.stdlib cimport abs
 * 
 * cdef inline double complex sph_harmonic(int m, int n, double theta, double phi) nogil:             # <<<<<<<<<<<<<<
 *     cdef double x, prefactor
 *     cdef double complex val
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special.sph_harm.sph_harmonic", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = __pyx_t_double_complex_from_parts(0, 0);
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":31
 *     int PyErr_WarnEx_noerr "PyErr_WarnEx" (object, char *, int)
 * 
 * cdef inline void _legacy_cast_check(char *func_name, double x, double y) nogil:             # <<<<<<<<<<<<<<
 *     if <int>x != x or <int>y != y:
 *         with gil:
 */

static CYTHON_INLINE void __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(CYTHON_UNUSED char *__pyx_v_func_name, double __pyx_v_x, double __pyx_v_y) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save;
  #endif
  __Pyx_RefNannySetupContext("_legacy_cast_check", 1);

  /* "_legacy.pxd":32
 * 
 * cdef inline void _legacy_cast_check(char *func_name, double x, double y) nogil:
 *     if <int>x != x or <int>y != y:             # <<<<<<<<<<<<<<
 *         with gil:
 *             PyErr_WarnEx_noerr(RuntimeWarning,
 */
  /*try:*/ {
    __pyx_t_2 = ((((int)__pyx_v_x) != __pyx_v_x) != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_2 = ((((int)__pyx_v_y) != __pyx_v_y) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L7_bool_binop_done:;
    if (__pyx_t_1) {

      /* "_legacy.pxd":33
 * cdef inline void _legacy_cast_check(char *func_name, double x, double y) nogil:
 *     if <int>x != x or <int>y != y:
 *         with gil:             # <<<<<<<<<<<<<<
 *             PyErr_WarnEx_noerr(RuntimeWarning,
 *                                "floating point number truncated to an integer",
 */
      {
          #ifdef WITH_THREAD
          PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
          #endif
          /*try:*/ {

            /* "_legacy.pxd":34
 *     if <int>x != x or <int>y != y:
 *         with gil:
 *             PyErr_WarnEx_noerr(RuntimeWarning,             # <<<<<<<<<<<<<<
 *                                "floating point number truncated to an integer",
 *                                1)
 */
            (void)(PyErr_WarnEx(__pyx_builtin_RuntimeWarning, ((char *)"floating point number truncated to an integer"), 1));
          }

          /* "_legacy.pxd":33
 * cdef inline void _legacy_cast_check(char *func_name, double x, double y) nogil:
 *     if <int>x != x or <int>y != y:
 *         with gil:             # <<<<<<<<<<<<<<
 *             PyErr_WarnEx_noerr(RuntimeWarning,
 *                                "floating point number truncated to an integer",
 */
          /*finally:*/ {
            /*normal exit:*/{
              #ifdef WITH_THREAD
              __Pyx_PyGILState_Release(__pyx_gilstate_save);
              #endif
              goto __pyx_L11;
            }
            __pyx_L11:;
          }
      }

      /* "_legacy.pxd":32
 * 
 * cdef inline void _legacy_cast_check(char *func_name, double x, double y) nogil:
 *     if <int>x != x or <int>y != y:             # <<<<<<<<<<<<<<
 *         with gil:
 *             PyErr_WarnEx_noerr(RuntimeWarning,
 */
    }
  }
  /*finally:*/ {
    /*normal exit:*/{
      #ifdef WITH_THREAD
      __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      goto __pyx_L5;
    }
    __pyx_L5:;
  }

  /* "_legacy.pxd":31
 *     int PyErr_WarnEx_noerr "PyErr_WarnEx" (object, char *, int)
 * 
 * cdef inline void _legacy_cast_check(char *func_name, double x, double y) nogil:             # <<<<<<<<<<<<<<
 *     if <int>x != x or <int>y != y:
 *         with gil:
 */

  /* function exit code */
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "_legacy.pxd":38
 *                                1)
 * 
 * cdef inline double complex sph_harmonic_unsafe(double m, double n,             # <<<<<<<<<<<<<<
 *                                                double theta, double phi) nogil:
 *     if npy_isnan(m) or npy_isnan(n):
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_7_legacy_sph_harmonic_unsafe(double __pyx_v_m, double __pyx_v_n, double __pyx_v_theta, double __pyx_v_phi) {
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_legacy.pxd":40
 * cdef inline double complex sph_harmonic_unsafe(double m, double n,
 *                                                double theta, double phi) nogil:
 *     if npy_isnan(m) or npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return nan
 *     _legacy_cast_check("sph_harm", m, n)
 */
  __pyx_t_2 = (npy_isnan(__pyx_v_m) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (npy_isnan(__pyx_v_n) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_legacy.pxd":41
 *                                                double theta, double phi) nogil:
 *     if npy_isnan(m) or npy_isnan(n):
 *         return nan             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("sph_harm", m, n)
 *     return sph_harmonic(<int>m, <int> n, theta, phi)
 */
    __pyx_r = __pyx_t_double_complex_from_parts(NPY_NAN, 0);
    goto __pyx_L0;

    /* "_legacy.pxd":40
 * cdef inline double complex sph_harmonic_unsafe(double m, double n,
 *                                                double theta, double phi) nogil:
 *     if npy_isnan(m) or npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return nan
 *     _legacy_cast_check("sph_harm", m, n)
 */
  }

  /* "_legacy.pxd":42
 *     if npy_isnan(m) or npy_isnan(n):
 *         return nan
 *     _legacy_cast_check("sph_harm", m, n)             # <<<<<<<<<<<<<<
 *     return sph_harmonic(<int>m, <int> n, theta, phi)
 * 
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(((char *)"sph_harm"), __pyx_v_m, __pyx_v_n);

  /* "_legacy.pxd":43
 *         return nan
 *     _legacy_cast_check("sph_harm", m, n)
 *     return sph_harmonic(<int>m, <int> n, theta, phi)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double ellip_harmonic_unsafe(double h2, double k2, double n,
 */
  __pyx_r = __pyx_f_5scipy_7special_8sph_harm_sph_harmonic(((int)__pyx_v_m), ((int)__pyx_v_n), __pyx_v_theta, __pyx_v_phi);
  goto __pyx_L0;

  /* "_legacy.pxd":38
 *                                1)
 * 
 * cdef inline double complex sph_harmonic_unsafe(double m, double n,             # <<<<<<<<<<<<<<
 *                                                double theta, double phi) nogil:
 *     if npy_isnan(m) or npy_isnan(n):
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":45
 *     return sph_harmonic(<int>m, <int> n, theta, phi)
 * 
 * cdef inline double ellip_harmonic_unsafe(double h2, double k2, double n,             # <<<<<<<<<<<<<<
 *                                          double p, double l, double signm,
 *                                          double signn) nogil:
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_ellip_harmonic_unsafe(double __pyx_v_h2, double __pyx_v_k2, double __pyx_v_n, double __pyx_v_p, double __pyx_v_l, double __pyx_v_signm, double __pyx_v_signn) {
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_legacy.pxd":48
 *                                          double p, double l, double signm,
 *                                          double signn) nogil:
 *     if npy_isnan(n) or npy_isnan(p):             # <<<<<<<<<<<<<<
 *         return nan
 *     _legacy_cast_check("_ellip_harm", n, p)
 */
  __pyx_t_2 = (npy_isnan(__pyx_v_n) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (npy_isnan(__pyx_v_p) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_legacy.pxd":49
 *                                          double signn) nogil:
 *     if npy_isnan(n) or npy_isnan(p):
 *         return nan             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("_ellip_harm", n, p)
 *     return ellip_harmonic(h2, k2, <int>n, <int>p, l, signm, signn)
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "_legacy.pxd":48
 *                                          double p, double l, double signm,
 *                                          double signn) nogil:
 *     if npy_isnan(n) or npy_isnan(p):             # <<<<<<<<<<<<<<
 *         return nan
 *     _legacy_cast_check("_ellip_harm", n, p)
 */
  }

  /* "_legacy.pxd":50
 *     if npy_isnan(n) or npy_isnan(p):
 *         return nan
 *     _legacy_cast_check("_ellip_harm", n, p)             # <<<<<<<<<<<<<<
 *     return ellip_harmonic(h2, k2, <int>n, <int>p, l, signm, signn)
 * 
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(((char *)"_ellip_harm"), __pyx_v_n, __pyx_v_p);

  /* "_legacy.pxd":51
 *         return nan
 *     _legacy_cast_check("_ellip_harm", n, p)
 *     return ellip_harmonic(h2, k2, <int>n, <int>p, l, signm, signn)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double bdtrc_unsafe(double k, double n, double p) nogil:
 */
  __pyx_r = __pyx_f_5scipy_7special_11_ellip_harm_ellip_harmonic(__pyx_v_h2, __pyx_v_k2, ((int)__pyx_v_n), ((int)__pyx_v_p), __pyx_v_l, __pyx_v_signm, __pyx_v_signn);
  goto __pyx_L0;

  /* "_legacy.pxd":45
 *     return sph_harmonic(<int>m, <int> n, theta, phi)
 * 
 * cdef inline double ellip_harmonic_unsafe(double h2, double k2, double n,             # <<<<<<<<<<<<<<
 *                                          double p, double l, double signm,
 *                                          double signn) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":53
 *     return ellip_harmonic(h2, k2, <int>n, <int>p, l, signm, signn)
 * 
 * cdef inline double bdtrc_unsafe(double k, double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_bdtrc_unsafe(double __pyx_v_k, double __pyx_v_n, double __pyx_v_p) {
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_legacy.pxd":54
 * 
 * cdef inline double bdtrc_unsafe(double k, double n, double p) nogil:
 *     if npy_isnan(k) or npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return nan
 *     _legacy_cast_check("bdtrc", k, n)
 */
  __pyx_t_2 = (npy_isnan(__pyx_v_k) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (npy_isnan(__pyx_v_n) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_legacy.pxd":55
 * cdef inline double bdtrc_unsafe(double k, double n, double p) nogil:
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("bdtrc", k, n)
 *     return bdtrc(<int>k, <int>n, p)
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "_legacy.pxd":54
 * 
 * cdef inline double bdtrc_unsafe(double k, double n, double p) nogil:
 *     if npy_isnan(k) or npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return nan
 *     _legacy_cast_check("bdtrc", k, n)
 */
  }

  /* "_legacy.pxd":56
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan
 *     _legacy_cast_check("bdtrc", k, n)             # <<<<<<<<<<<<<<
 *     return bdtrc(<int>k, <int>n, p)
 * 
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(((char *)"bdtrc"), __pyx_v_k, __pyx_v_n);

  /* "_legacy.pxd":57
 *         return nan
 *     _legacy_cast_check("bdtrc", k, n)
 *     return bdtrc(<int>k, <int>n, p)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double bdtr_unsafe(double k, double n, double p) nogil:
 */
  __pyx_r = bdtrc(((int)__pyx_v_k), ((int)__pyx_v_n), __pyx_v_p);
  goto __pyx_L0;

  /* "_legacy.pxd":53
 *     return ellip_harmonic(h2, k2, <int>n, <int>p, l, signm, signn)
 * 
 * cdef inline double bdtrc_unsafe(double k, double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":59
 *     return bdtrc(<int>k, <int>n, p)
 * 
 * cdef inline double bdtr_unsafe(double k, double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_bdtr_unsafe(double __pyx_v_k, double __pyx_v_n, double __pyx_v_p) {
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_legacy.pxd":60
 * 
 * cdef inline double bdtr_unsafe(double k, double n, double p) nogil:
 *     if npy_isnan(k) or npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return nan
 *     _legacy_cast_check("bdtr", k, n)
 */
  __pyx_t_2 = (npy_isnan(__pyx_v_k) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (npy_isnan(__pyx_v_n) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_legacy.pxd":61
 * cdef inline double bdtr_unsafe(double k, double n, double p) nogil:
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("bdtr", k, n)
 *     return bdtr(<int>k, <int>n, p)
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "_legacy.pxd":60
 * 
 * cdef inline double bdtr_unsafe(double k, double n, double p) nogil:
 *     if npy_isnan(k) or npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return nan
 *     _legacy_cast_check("bdtr", k, n)
 */
  }

  /* "_legacy.pxd":62
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan
 *     _legacy_cast_check("bdtr", k, n)             # <<<<<<<<<<<<<<
 *     return bdtr(<int>k, <int>n, p)
 * 
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(((char *)"bdtr"), __pyx_v_k, __pyx_v_n);

  /* "_legacy.pxd":63
 *         return nan
 *     _legacy_cast_check("bdtr", k, n)
 *     return bdtr(<int>k, <int>n, p)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double bdtri_unsafe(double k, double n, double y) nogil:
 */
  __pyx_r = bdtr(((int)__pyx_v_k), ((int)__pyx_v_n), __pyx_v_p);
  goto __pyx_L0;

  /* "_legacy.pxd":59
 *     return bdtrc(<int>k, <int>n, p)
 * 
 * cdef inline double bdtr_unsafe(double k, double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":65
 *     return bdtr(<int>k, <int>n, p)
 * 
 * cdef inline double bdtri_unsafe(double k, double n, double y) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_bdtri_unsafe(double __pyx_v_k, double __pyx_v_n, double __pyx_v_y) {
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_legacy.pxd":66
 * 
 * cdef inline double bdtri_unsafe(double k, double n, double y) nogil:
 *     if npy_isnan(k) or npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return nan
 *     _legacy_cast_check("bdtri", k, n)
 */
  __pyx_t_2 = (npy_isnan(__pyx_v_k) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (npy_isnan(__pyx_v_n) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_legacy.pxd":67
 * cdef inline double bdtri_unsafe(double k, double n, double y) nogil:
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("bdtri", k, n)
 *     return bdtri(<int>k, <int>n, y)
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "_legacy.pxd":66
 * 
 * cdef inline double bdtri_unsafe(double k, double n, double y) nogil:
 *     if npy_isnan(k) or npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return nan
 *     _legacy_cast_check("bdtri", k, n)
 */
  }

  /* "_legacy.pxd":68
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan
 *     _legacy_cast_check("bdtri", k, n)             # <<<<<<<<<<<<<<
 *     return bdtri(<int>k, <int>n, y)
 * 
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(((char *)"bdtri"), __pyx_v_k, __pyx_v_n);

  /* "_legacy.pxd":69
 *         return nan
 *     _legacy_cast_check("bdtri", k, n)
 *     return bdtri(<int>k, <int>n, y)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double expn_unsafe(double n, double x) nogil:
 */
  __pyx_r = bdtri(((int)__pyx_v_k), ((int)__pyx_v_n), __pyx_v_y);
  goto __pyx_L0;

  /* "_legacy.pxd":65
 *     return bdtr(<int>k, <int>n, p)
 * 
 * cdef inline double bdtri_unsafe(double k, double n, double y) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":71
 *     return bdtri(<int>k, <int>n, y)
 * 
 * cdef inline double expn_unsafe(double n, double x) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(n):
 *         return n
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_expn_unsafe(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_legacy.pxd":72
 * 
 * cdef inline double expn_unsafe(double n, double x) nogil:
 *     if npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return n
 *     _legacy_cast_check("expn", n, 0)
 */
  __pyx_t_1 = (npy_isnan(__pyx_v_n) != 0);
  if (__pyx_t_1) {

    /* "_legacy.pxd":73
 * cdef inline double expn_unsafe(double n, double x) nogil:
 *     if npy_isnan(n):
 *         return n             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("expn", n, 0)
 *     return expn(<int>n, x)
 */
    __pyx_r = __pyx_v_n;
    goto __pyx_L0;

    /* "_legacy.pxd":72
 * 
 * cdef inline double expn_unsafe(double n, double x) nogil:
 *     if npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return n
 *     _legacy_cast_check("expn", n, 0)
 */
  }

  /* "_legacy.pxd":74
 *     if npy_isnan(n):
 *         return n
 *     _legacy_cast_check("expn", n, 0)             # <<<<<<<<<<<<<<
 *     return expn(<int>n, x)
 * 
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(((char *)"expn"), __pyx_v_n, 0.0);

  /* "_legacy.pxd":75
 *         return n
 *     _legacy_cast_check("expn", n, 0)
 *     return expn(<int>n, x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double hyp2f0_unsafe(double a, double b, double x,
 */
  __pyx_r = expn(((int)__pyx_v_n), __pyx_v_x);
  goto __pyx_L0;

  /* "_legacy.pxd":71
 *     return bdtri(<int>k, <int>n, y)
 * 
 * cdef inline double expn_unsafe(double n, double x) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(n):
 *         return n
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":77
 *     return expn(<int>n, x)
 * 
 * cdef inline double hyp2f0_unsafe(double a, double b, double x,             # <<<<<<<<<<<<<<
 *                                  double type, double *err) nogil:
 *     if npy_isnan(type):
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_hyp2f0_unsafe(double __pyx_v_a, double __pyx_v_b, double __pyx_v_x, double __pyx_v_type, double *__pyx_v_err) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_legacy.pxd":79
 * cdef inline double hyp2f0_unsafe(double a, double b, double x,
 *                                  double type, double *err) nogil:
 *     if npy_isnan(type):             # <<<<<<<<<<<<<<
 *         return type
 *     _legacy_cast_check("hyp2f0", type, 0)
 */
  __pyx_t_1 = (npy_isnan(__pyx_v_type) != 0);
  if (__pyx_t_1) {

    /* "_legacy.pxd":80
 *                                  double type, double *err) nogil:
 *     if npy_isnan(type):
 *         return type             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("hyp2f0", type, 0)
 *     return hyp2f0(a, b, x, <int>type, err)
 */
    __pyx_r = __pyx_v_type;
    goto __pyx_L0;

    /* "_legacy.pxd":79
 * cdef inline double hyp2f0_unsafe(double a, double b, double x,
 *                                  double type, double *err) nogil:
 *     if npy_isnan(type):             # <<<<<<<<<<<<<<
 *         return type
 *     _legacy_cast_check("hyp2f0", type, 0)
 */
  }

  /* "_legacy.pxd":81
 *     if npy_isnan(type):
 *         return type
 *     _legacy_cast_check("hyp2f0", type, 0)             # <<<<<<<<<<<<<<
 *     return hyp2f0(a, b, x, <int>type, err)
 * 
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(((char *)"hyp2f0"), __pyx_v_type, 0.0);

  /* "_legacy.pxd":82
 *         return type
 *     _legacy_cast_check("hyp2f0", type, 0)
 *     return hyp2f0(a, b, x, <int>type, err)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double nbdtrc_unsafe(double k, double n, double p) nogil:
 */
  __pyx_r = hyp2f0(__pyx_v_a, __pyx_v_b, __pyx_v_x, ((int)__pyx_v_type), __pyx_v_err);
  goto __pyx_L0;

  /* "_legacy.pxd":77
 *     return expn(<int>n, x)
 * 
 * cdef inline double hyp2f0_unsafe(double a, double b, double x,             # <<<<<<<<<<<<<<
 *                                  double type, double *err) nogil:
 *     if npy_isnan(type):
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":84
 *     return hyp2f0(a, b, x, <int>type, err)
 * 
 * cdef inline double nbdtrc_unsafe(double k, double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_nbdtrc_unsafe(double __pyx_v_k, double __pyx_v_n, double __pyx_v_p) {
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_legacy.pxd":85
 * 
 * cdef inline double nbdtrc_unsafe(double k, double n, double p) nogil:
 *     if npy_isnan(k) or npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return nan
 *     _legacy_cast_check("nbdtrc", k, n)
 */
  __pyx_t_2 = (npy_isnan(__pyx_v_k) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (npy_isnan(__pyx_v_n) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_legacy.pxd":86
 * cdef inline double nbdtrc_unsafe(double k, double n, double p) nogil:
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("nbdtrc", k, n)
 *     return nbdtrc(<int>k, <int>n, p)
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "_legacy.pxd":85
 * 
 * cdef inline double nbdtrc_unsafe(double k, double n, double p) nogil:
 *     if npy_isnan(k) or npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return nan
 *     _legacy_cast_check("nbdtrc", k, n)
 */
  }

  /* "_legacy.pxd":87
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan
 *     _legacy_cast_check("nbdtrc", k, n)             # <<<<<<<<<<<<<<
 *     return nbdtrc(<int>k, <int>n, p)
 * 
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(((char *)"nbdtrc"), __pyx_v_k, __pyx_v_n);

  /* "_legacy.pxd":88
 *         return nan
 *     _legacy_cast_check("nbdtrc", k, n)
 *     return nbdtrc(<int>k, <int>n, p)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double nbdtr_unsafe(double k, double n, double p)  nogil:
 */
  __pyx_r = nbdtrc(((int)__pyx_v_k), ((int)__pyx_v_n), __pyx_v_p);
  goto __pyx_L0;

  /* "_legacy.pxd":84
 *     return hyp2f0(a, b, x, <int>type, err)
 * 
 * cdef inline double nbdtrc_unsafe(double k, double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":90
 *     return nbdtrc(<int>k, <int>n, p)
 * 
 * cdef inline double nbdtr_unsafe(double k, double n, double p)  nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_nbdtr_unsafe(double __pyx_v_k, double __pyx_v_n, double __pyx_v_p) {
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_legacy.pxd":91
 * 
 * cdef inline double nbdtr_unsafe(double k, double n, double p)  nogil:
 *     if npy_isnan(k) or npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return nan
 *     _legacy_cast_check("nbdtr", k, n)
 */
  __pyx_t_2 = (npy_isnan(__pyx_v_k) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (npy_isnan(__pyx_v_n) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_legacy.pxd":92
 * cdef inline double nbdtr_unsafe(double k, double n, double p)  nogil:
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("nbdtr", k, n)
 *     return nbdtr(<int>k, <int>n, p)
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "_legacy.pxd":91
 * 
 * cdef inline double nbdtr_unsafe(double k, double n, double p)  nogil:
 *     if npy_isnan(k) or npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return nan
 *     _legacy_cast_check("nbdtr", k, n)
 */
  }

  /* "_legacy.pxd":93
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan
 *     _legacy_cast_check("nbdtr", k, n)             # <<<<<<<<<<<<<<
 *     return nbdtr(<int>k, <int>n, p)
 * 
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(((char *)"nbdtr"), __pyx_v_k, __pyx_v_n);

  /* "_legacy.pxd":94
 *         return nan
 *     _legacy_cast_check("nbdtr", k, n)
 *     return nbdtr(<int>k, <int>n, p)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double nbdtri_unsafe(double k, double n, double p) nogil:
 */
  __pyx_r = nbdtr(((int)__pyx_v_k), ((int)__pyx_v_n), __pyx_v_p);
  goto __pyx_L0;

  /* "_legacy.pxd":90
 *     return nbdtrc(<int>k, <int>n, p)
 * 
 * cdef inline double nbdtr_unsafe(double k, double n, double p)  nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":96
 *     return nbdtr(<int>k, <int>n, p)
 * 
 * cdef inline double nbdtri_unsafe(double k, double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_nbdtri_unsafe(double __pyx_v_k, double __pyx_v_n, double __pyx_v_p) {
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_legacy.pxd":97
 * 
 * cdef inline double nbdtri_unsafe(double k, double n, double p) nogil:
 *     if npy_isnan(k) or npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return nan
 *     _legacy_cast_check("nbdtri", k, n)
 */
  __pyx_t_2 = (npy_isnan(__pyx_v_k) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (npy_isnan(__pyx_v_n) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_legacy.pxd":98
 * cdef inline double nbdtri_unsafe(double k, double n, double p) nogil:
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("nbdtri", k, n)
 *     return nbdtri(<int>k, <int>n, p)
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "_legacy.pxd":97
 * 
 * cdef inline double nbdtri_unsafe(double k, double n, double p) nogil:
 *     if npy_isnan(k) or npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return nan
 *     _legacy_cast_check("nbdtri", k, n)
 */
  }

  /* "_legacy.pxd":99
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan
 *     _legacy_cast_check("nbdtri", k, n)             # <<<<<<<<<<<<<<
 *     return nbdtri(<int>k, <int>n, p)
 * 
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(((char *)"nbdtri"), __pyx_v_k, __pyx_v_n);

  /* "_legacy.pxd":100
 *         return nan
 *     _legacy_cast_check("nbdtri", k, n)
 *     return nbdtri(<int>k, <int>n, p)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double pdtrc_unsafe(double k, double m) nogil:
 */
  __pyx_r = nbdtri(((int)__pyx_v_k), ((int)__pyx_v_n), __pyx_v_p);
  goto __pyx_L0;

  /* "_legacy.pxd":96
 *     return nbdtr(<int>k, <int>n, p)
 * 
 * cdef inline double nbdtri_unsafe(double k, double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(k) or npy_isnan(n):
 *         return nan
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":102
 *     return nbdtri(<int>k, <int>n, p)
 * 
 * cdef inline double pdtrc_unsafe(double k, double m) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(k):
 *         return k
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_pdtrc_unsafe(double __pyx_v_k, double __pyx_v_m) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_legacy.pxd":103
 * 
 * cdef inline double pdtrc_unsafe(double k, double m) nogil:
 *     if npy_isnan(k):             # <<<<<<<<<<<<<<
 *         return k
 *     _legacy_cast_check("pdtrc", k, 0)
 */
  __pyx_t_1 = (npy_isnan(__pyx_v_k) != 0);
  if (__pyx_t_1) {

    /* "_legacy.pxd":104
 * cdef inline double pdtrc_unsafe(double k, double m) nogil:
 *     if npy_isnan(k):
 *         return k             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("pdtrc", k, 0)
 *     return pdtrc(<int>k, m)
 */
    __pyx_r = __pyx_v_k;
    goto __pyx_L0;

    /* "_legacy.pxd":103
 * 
 * cdef inline double pdtrc_unsafe(double k, double m) nogil:
 *     if npy_isnan(k):             # <<<<<<<<<<<<<<
 *         return k
 *     _legacy_cast_check("pdtrc", k, 0)
 */
  }

  /* "_legacy.pxd":105
 *     if npy_isnan(k):
 *         return k
 *     _legacy_cast_check("pdtrc", k, 0)             # <<<<<<<<<<<<<<
 *     return pdtrc(<int>k, m)
 * 
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(((char *)"pdtrc"), __pyx_v_k, 0.0);

  /* "_legacy.pxd":106
 *         return k
 *     _legacy_cast_check("pdtrc", k, 0)
 *     return pdtrc(<int>k, m)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double pdtr_unsafe(double k, double m) nogil:
 */
  __pyx_r = pdtrc(((int)__pyx_v_k), __pyx_v_m);
  goto __pyx_L0;

  /* "_legacy.pxd":102
 *     return nbdtri(<int>k, <int>n, p)
 * 
 * cdef inline double pdtrc_unsafe(double k, double m) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(k):
 *         return k
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":108
 *     return pdtrc(<int>k, m)
 * 
 * cdef inline double pdtr_unsafe(double k, double m) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(k):
 *         return k
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_pdtr_unsafe(double __pyx_v_k, double __pyx_v_m) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_legacy.pxd":109
 * 
 * cdef inline double pdtr_unsafe(double k, double m) nogil:
 *     if npy_isnan(k):             # <<<<<<<<<<<<<<
 *         return k
 *     _legacy_cast_check("pdtr", k, 0)
 */
  __pyx_t_1 = (npy_isnan(__pyx_v_k) != 0);
  if (__pyx_t_1) {

    /* "_legacy.pxd":110
 * cdef inline double pdtr_unsafe(double k, double m) nogil:
 *     if npy_isnan(k):
 *         return k             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("pdtr", k, 0)
 *     return pdtr(<int>k, m)
 */
    __pyx_r = __pyx_v_k;
    goto __pyx_L0;

    /* "_legacy.pxd":109
 * 
 * cdef inline double pdtr_unsafe(double k, double m) nogil:
 *     if npy_isnan(k):             # <<<<<<<<<<<<<<
 *         return k
 *     _legacy_cast_check("pdtr", k, 0)
 */
  }

  /* "_legacy.pxd":111
 *     if npy_isnan(k):
 *         return k
 *     _legacy_cast_check("pdtr", k, 0)             # <<<<<<<<<<<<<<
 *     return pdtr(<int>k, m)
 * 
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(((char *)"pdtr"), __pyx_v_k, 0.0);

  /* "_legacy.pxd":112
 *         return k
 *     _legacy_cast_check("pdtr", k, 0)
 *     return pdtr(<int>k, m)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double pdtri_unsafe(double k, double y) nogil:
 */
  __pyx_r = pdtr(((int)__pyx_v_k), __pyx_v_m);
  goto __pyx_L0;

  /* "_legacy.pxd":108
 *     return pdtrc(<int>k, m)
 * 
 * cdef inline double pdtr_unsafe(double k, double m) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(k):
 *         return k
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":114
 *     return pdtr(<int>k, m)
 * 
 * cdef inline double pdtri_unsafe(double k, double y) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(k):
 *         return k
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_pdtri_unsafe(double __pyx_v_k, double __pyx_v_y) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_legacy.pxd":115
 * 
 * cdef inline double pdtri_unsafe(double k, double y) nogil:
 *     if npy_isnan(k):             # <<<<<<<<<<<<<<
 *         return k
 *     _legacy_cast_check("pdtri", k, 0)
 */
  __pyx_t_1 = (npy_isnan(__pyx_v_k) != 0);
  if (__pyx_t_1) {

    /* "_legacy.pxd":116
 * cdef inline double pdtri_unsafe(double k, double y) nogil:
 *     if npy_isnan(k):
 *         return k             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("pdtri", k, 0)
 *     return pdtri(<int>k, y)
 */
    __pyx_r = __pyx_v_k;
    goto __pyx_L0;

    /* "_legacy.pxd":115
 * 
 * cdef inline double pdtri_unsafe(double k, double y) nogil:
 *     if npy_isnan(k):             # <<<<<<<<<<<<<<
 *         return k
 *     _legacy_cast_check("pdtri", k, 0)
 */
  }

  /* "_legacy.pxd":117
 *     if npy_isnan(k):
 *         return k
 *     _legacy_cast_check("pdtri", k, 0)             # <<<<<<<<<<<<<<
 *     return pdtri(<int>k, y)
 * 
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(((char *)"pdtri"), __pyx_v_k, 0.0);

  /* "_legacy.pxd":118
 *         return k
 *     _legacy_cast_check("pdtri", k, 0)
 *     return pdtri(<int>k, y)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double kn_unsafe(double n, double x) nogil:
 */
  __pyx_r = pdtri(((int)__pyx_v_k), __pyx_v_y);
  goto __pyx_L0;

  /* "_legacy.pxd":114
 *     return pdtr(<int>k, m)
 * 
 * cdef inline double pdtri_unsafe(double k, double y) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(k):
 *         return k
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":120
 *     return pdtri(<int>k, y)
 * 
 * cdef inline double kn_unsafe(double n, double x) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(n):
 *         return n
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_kn_unsafe(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_legacy.pxd":121
 * 
 * cdef inline double kn_unsafe(double n, double x) nogil:
 *     if npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return n
 *     _legacy_cast_check("kn", n, 0)
 */
  __pyx_t_1 = (npy_isnan(__pyx_v_n) != 0);
  if (__pyx_t_1) {

    /* "_legacy.pxd":122
 * cdef inline double kn_unsafe(double n, double x) nogil:
 *     if npy_isnan(n):
 *         return n             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("kn", n, 0)
 *     return cbesk_wrap_real_int(<int>n, x)
 */
    __pyx_r = __pyx_v_n;
    goto __pyx_L0;

    /* "_legacy.pxd":121
 * 
 * cdef inline double kn_unsafe(double n, double x) nogil:
 *     if npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return n
 *     _legacy_cast_check("kn", n, 0)
 */
  }

  /* "_legacy.pxd":123
 *     if npy_isnan(n):
 *         return n
 *     _legacy_cast_check("kn", n, 0)             # <<<<<<<<<<<<<<
 *     return cbesk_wrap_real_int(<int>n, x)
 * 
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(((char *)"kn"), __pyx_v_n, 0.0);

  /* "_legacy.pxd":124
 *         return n
 *     _legacy_cast_check("kn", n, 0)
 *     return cbesk_wrap_real_int(<int>n, x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double yn_unsafe(double n, double x) nogil:
 */
  __pyx_r = cbesk_wrap_real_int(((int)__pyx_v_n), __pyx_v_x);
  goto __pyx_L0;

  /* "_legacy.pxd":120
 *     return pdtri(<int>k, y)
 * 
 * cdef inline double kn_unsafe(double n, double x) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(n):
 *         return n
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":126
 *     return cbesk_wrap_real_int(<int>n, x)
 * 
 * cdef inline double yn_unsafe(double n, double x) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(n):
 *         return n
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_yn_unsafe(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_legacy.pxd":127
 * 
 * cdef inline double yn_unsafe(double n, double x) nogil:
 *     if npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return n
 *     _legacy_cast_check("yn", n, 0)
 */
  __pyx_t_1 = (npy_isnan(__pyx_v_n) != 0);
  if (__pyx_t_1) {

    /* "_legacy.pxd":128
 * cdef inline double yn_unsafe(double n, double x) nogil:
 *     if npy_isnan(n):
 *         return n             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("yn", n, 0)
 *     return yn(<int>n, x)
 */
    __pyx_r = __pyx_v_n;
    goto __pyx_L0;

    /* "_legacy.pxd":127
 * 
 * cdef inline double yn_unsafe(double n, double x) nogil:
 *     if npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return n
 *     _legacy_cast_check("yn", n, 0)
 */
  }

  /* "_legacy.pxd":129
 *     if npy_isnan(n):
 *         return n
 *     _legacy_cast_check("yn", n, 0)             # <<<<<<<<<<<<<<
 *     return yn(<int>n, x)
 * 
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(((char *)"yn"), __pyx_v_n, 0.0);

  /* "_legacy.pxd":130
 *         return n
 *     _legacy_cast_check("yn", n, 0)
 *     return yn(<int>n, x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double smirnov_unsafe(double n, double e) nogil:
 */
  __pyx_r = yn(((int)__pyx_v_n), __pyx_v_x);
  goto __pyx_L0;

  /* "_legacy.pxd":126
 *     return cbesk_wrap_real_int(<int>n, x)
 * 
 * cdef inline double yn_unsafe(double n, double x) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(n):
 *         return n
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":132
 *     return yn(<int>n, x)
 * 
 * cdef inline double smirnov_unsafe(double n, double e) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(n):
 *         return n
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_smirnov_unsafe(double __pyx_v_n, double __pyx_v_e) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_legacy.pxd":133
 * 
 * cdef inline double smirnov_unsafe(double n, double e) nogil:
 *     if npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return n
 *     _legacy_cast_check("smirnov", n, 0)
 */
  __pyx_t_1 = (npy_isnan(__pyx_v_n) != 0);
  if (__pyx_t_1) {

    /* "_legacy.pxd":134
 * cdef inline double smirnov_unsafe(double n, double e) nogil:
 *     if npy_isnan(n):
 *         return n             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("smirnov", n, 0)
 *     return smirnov(<int>n, e)
 */
    __pyx_r = __pyx_v_n;
    goto __pyx_L0;

    /* "_legacy.pxd":133
 * 
 * cdef inline double smirnov_unsafe(double n, double e) nogil:
 *     if npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return n
 *     _legacy_cast_check("smirnov", n, 0)
 */
  }

  /* "_legacy.pxd":135
 *     if npy_isnan(n):
 *         return n
 *     _legacy_cast_check("smirnov", n, 0)             # <<<<<<<<<<<<<<
 *     return smirnov(<int>n, e)
 * 
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(((char *)"smirnov"), __pyx_v_n, 0.0);

  /* "_legacy.pxd":136
 *         return n
 *     _legacy_cast_check("smirnov", n, 0)
 *     return smirnov(<int>n, e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double smirnovc_unsafe(double n, double e) nogil:
 */
  __pyx_r = smirnov(((int)__pyx_v_n), __pyx_v_e);
  goto __pyx_L0;

  /* "_legacy.pxd":132
 *     return yn(<int>n, x)
 * 
 * cdef inline double smirnov_unsafe(double n, double e) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(n):
 *         return n
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":138
 *     return smirnov(<int>n, e)
 * 
 * cdef inline double smirnovc_unsafe(double n, double e) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(n):
 *         return n
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_smirnovc_unsafe(double __pyx_v_n, double __pyx_v_e) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_legacy.pxd":139
 * 
 * cdef inline double smirnovc_unsafe(double n, double e) nogil:
 *     if npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return n
 *     _legacy_cast_check("smirnovc", n, 0)
 */
  __pyx_t_1 = (npy_isnan(__pyx_v_n) != 0);
  if (__pyx_t_1) {

    /* "_legacy.pxd":140
 * cdef inline double smirnovc_unsafe(double n, double e) nogil:
 *     if npy_isnan(n):
 *         return n             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("smirnovc", n, 0)
 *     return smirnovc(<int>n, e)
 */
    __pyx_r = __pyx_v_n;
    goto __pyx_L0;

    /* "_legacy.pxd":139
 * 
 * cdef inline double smirnovc_unsafe(double n, double e) nogil:
 *     if npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return n
 *     _legacy_cast_check("smirnovc", n, 0)
 */
  }

  /* "_legacy.pxd":141
 *     if npy_isnan(n):
 *         return n
 *     _legacy_cast_check("smirnovc", n, 0)             # <<<<<<<<<<<<<<
 *     return smirnovc(<int>n, e)
 * 
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(((char *)"smirnovc"), __pyx_v_n, 0.0);

  /* "_legacy.pxd":142
 *         return n
 *     _legacy_cast_check("smirnovc", n, 0)
 *     return smirnovc(<int>n, e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double smirnovp_unsafe(double n, double e) nogil:
 */
  __pyx_r = smirnovc(((int)__pyx_v_n), __pyx_v_e);
  goto __pyx_L0;

  /* "_legacy.pxd":138
 *     return smirnov(<int>n, e)
 * 
 * cdef inline double smirnovc_unsafe(double n, double e) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(n):
 *         return n
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":144
 *     return smirnovc(<int>n, e)
 * 
 * cdef inline double smirnovp_unsafe(double n, double e) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(n):
 *         return n
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_smirnovp_unsafe(double __pyx_v_n, double __pyx_v_e) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_legacy.pxd":145
 * 
 * cdef inline double smirnovp_unsafe(double n, double e) nogil:
 *     if npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return n
 *     _legacy_cast_check("smirnovp", n, 0)
 */
  __pyx_t_1 = (npy_isnan(__pyx_v_n) != 0);
  if (__pyx_t_1) {

    /* "_legacy.pxd":146
 * cdef inline double smirnovp_unsafe(double n, double e) nogil:
 *     if npy_isnan(n):
 *         return n             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("smirnovp", n, 0)
 *     return smirnovp(<int>n, e)
 */
    __pyx_r = __pyx_v_n;
    goto __pyx_L0;

    /* "_legacy.pxd":145
 * 
 * cdef inline double smirnovp_unsafe(double n, double e) nogil:
 *     if npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return n
 *     _legacy_cast_check("smirnovp", n, 0)
 */
  }

  /* "_legacy.pxd":147
 *     if npy_isnan(n):
 *         return n
 *     _legacy_cast_check("smirnovp", n, 0)             # <<<<<<<<<<<<<<
 *     return smirnovp(<int>n, e)
 * 
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(((char *)"smirnovp"), __pyx_v_n, 0.0);

  /* "_legacy.pxd":148
 *         return n
 *     _legacy_cast_check("smirnovp", n, 0)
 *     return smirnovp(<int>n, e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double smirnovi_unsafe(double n, double p) nogil:
 */
  __pyx_r = smirnovp(((int)__pyx_v_n), __pyx_v_e);
  goto __pyx_L0;

  /* "_legacy.pxd":144
 *     return smirnovc(<int>n, e)
 * 
 * cdef inline double smirnovp_unsafe(double n, double e) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(n):
 *         return n
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":150
 *     return smirnovp(<int>n, e)
 * 
 * cdef inline double smirnovi_unsafe(double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(n):
 *         return n
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_smirnovi_unsafe(double __pyx_v_n, double __pyx_v_p) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_legacy.pxd":151
 * 
 * cdef inline double smirnovi_unsafe(double n, double p) nogil:
 *     if npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return n
 *     _legacy_cast_check("smirnovi", n, 0)
 */
  __pyx_t_1 = (npy_isnan(__pyx_v_n) != 0);
  if (__pyx_t_1) {

    /* "_legacy.pxd":152
 * cdef inline double smirnovi_unsafe(double n, double p) nogil:
 *     if npy_isnan(n):
 *         return n             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("smirnovi", n, 0)
 *     return smirnovi(<int>n, p)
 */
    __pyx_r = __pyx_v_n;
    goto __pyx_L0;

    /* "_legacy.pxd":151
 * 
 * cdef inline double smirnovi_unsafe(double n, double p) nogil:
 *     if npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return n
 *     _legacy_cast_check("smirnovi", n, 0)
 */
  }

  /* "_legacy.pxd":153
 *     if npy_isnan(n):
 *         return n
 *     _legacy_cast_check("smirnovi", n, 0)             # <<<<<<<<<<<<<<
 *     return smirnovi(<int>n, p)
 * 
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(((char *)"smirnovi"), __pyx_v_n, 0.0);

  /* "_legacy.pxd":154
 *         return n
 *     _legacy_cast_check("smirnovi", n, 0)
 *     return smirnovi(<int>n, p)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double smirnovci_unsafe(double n, double p) nogil:
 */
  __pyx_r = smirnovi(((int)__pyx_v_n), __pyx_v_p);
  goto __pyx_L0;

  /* "_legacy.pxd":150
 *     return smirnovp(<int>n, e)
 * 
 * cdef inline double smirnovi_unsafe(double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(n):
 *         return n
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_legacy.pxd":156
 *     return smirnovi(<int>n, p)
 * 
 * cdef inline double smirnovci_unsafe(double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(n):
 *         return n
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_legacy_smirnovci_unsafe(double __pyx_v_n, double __pyx_v_p) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_legacy.pxd":157
 * 
 * cdef inline double smirnovci_unsafe(double n, double p) nogil:
 *     if npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return n
 *     _legacy_cast_check("smirnovci", n, 0)
 */
  __pyx_t_1 = (npy_isnan(__pyx_v_n) != 0);
  if (__pyx_t_1) {

    /* "_legacy.pxd":158
 * cdef inline double smirnovci_unsafe(double n, double p) nogil:
 *     if npy_isnan(n):
 *         return n             # <<<<<<<<<<<<<<
 *     _legacy_cast_check("smirnovci", n, 0)
 *     return smirnovci(<int>n, p)
 */
    __pyx_r = __pyx_v_n;
    goto __pyx_L0;

    /* "_legacy.pxd":157
 * 
 * cdef inline double smirnovci_unsafe(double n, double p) nogil:
 *     if npy_isnan(n):             # <<<<<<<<<<<<<<
 *         return n
 *     _legacy_cast_check("smirnovci", n, 0)
 */
  }

  /* "_legacy.pxd":159
 *     if npy_isnan(n):
 *         return n
 *     _legacy_cast_check("smirnovci", n, 0)             # <<<<<<<<<<<<<<
 *     return smirnovci(<int>n, p)
 */
  __pyx_f_5scipy_7special_7_legacy__legacy_cast_check(((char *)"smirnovci"), __pyx_v_n, 0.0);

  /* "_legacy.pxd":160
 *         return n
 *     _legacy_cast_check("smirnovci", n, 0)
 *     return smirnovci(<int>n, p)             # <<<<<<<<<<<<<<
 */
  __pyx_r = smirnovci(((int)__pyx_v_n), __pyx_v_p);
  goto __pyx_L0;

  /* "_legacy.pxd":156
 *     return smirnovi(<int>n, p)
 * 
 * cdef inline double smirnovci_unsafe(double n, double p) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(n):
 *         return n
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_evalpoly.pxd":21
 * 
 * 
 * cdef inline double complex cevalpoly(double *coeffs, int degree,             # <<<<<<<<<<<<<<
 *                                      double complex z) nogil:
 *     """Evaluate a polynomial with real coefficients at a complex point.
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_9_evalpoly_cevalpoly(double *__pyx_v_coeffs, int __pyx_v_degree, __pyx_t_double_complex __pyx_v_z) {
  int __pyx_v_j;
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_r;
  double __pyx_v_s;
  double __pyx_v_tmp;
  __pyx_t_double_complex __pyx_r;
  long __pyx_t_1;
  long __pyx_t_2;
  int __pyx_t_3;

  /* "_evalpoly.pxd":31
 *     cdef:
 *         int j
 *         double a = coeffs[0]             # <<<<<<<<<<<<<<
 *         double b = coeffs[1]
 *         double r = 2*z.real
 */
  __pyx_v_a = (__pyx_v_coeffs[0]);

  /* "_evalpoly.pxd":32
 *         int j
 *         double a = coeffs[0]
 *         double b = coeffs[1]             # <<<<<<<<<<<<<<
 *         double r = 2*z.real
 *         double s = z.real*z.real + z.imag*z.imag
 */
  __pyx_v_b = (__pyx_v_coeffs[1]);

  /* "_evalpoly.pxd":33
 *         double a = coeffs[0]
 *         double b = coeffs[1]
 *         double r = 2*z.real             # <<<<<<<<<<<<<<
 *         double s = z.real*z.real + z.imag*z.imag
 *         double tmp
 */
  __pyx_v_r = (2.0 * __Pyx_CREAL(__pyx_v_z));

  /* "_evalpoly.pxd":34
 *         double b = coeffs[1]
 *         double r = 2*z.real
 *         double s = z.real*z.real + z.imag*z.imag             # <<<<<<<<<<<<<<
 *         double tmp
 * 
 */
  __pyx_v_s = ((__Pyx_CREAL(__pyx_v_z) * __Pyx_CREAL(__pyx_v_z)) + (__Pyx_CIMAG(__pyx_v_z) * __Pyx_CIMAG(__pyx_v_z)));

  /* "_evalpoly.pxd":37
 *         double tmp
 * 
 *     for j in range(2, degree + 1):             # <<<<<<<<<<<<<<
 *         tmp = b
 *         b = sc_fma(-s, a, coeffs[j])
 */
  __pyx_t_1 = (__pyx_v_degree + 1);
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 2; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "_evalpoly.pxd":38
 * 
 *     for j in range(2, degree + 1):
 *         tmp = b             # <<<<<<<<<<<<<<
 *         b = sc_fma(-s, a, coeffs[j])
 *         a = sc_fma(r, a, tmp)
 */
    __pyx_v_tmp = __pyx_v_b;

    /* "_evalpoly.pxd":39
 *     for j in range(2, degree + 1):
 *         tmp = b
 *         b = sc_fma(-s, a, coeffs[j])             # <<<<<<<<<<<<<<
 *         a = sc_fma(r, a, tmp)
 *     return z*a + b
 */
    __pyx_v_b = sc_fma((-__pyx_v_s), __pyx_v_a, (__pyx_v_coeffs[__pyx_v_j]));

    /* "_evalpoly.pxd":40
 *         tmp = b
 *         b = sc_fma(-s, a, coeffs[j])
 *         a = sc_fma(r, a, tmp)             # <<<<<<<<<<<<<<
 *     return z*a + b
 */
    __pyx_v_a = sc_fma(__pyx_v_r, __pyx_v_a, __pyx_v_tmp);
  }

  /* "_evalpoly.pxd":41
 *         b = sc_fma(-s, a, coeffs[j])
 *         a = sc_fma(r, a, tmp)
 *     return z*a + b             # <<<<<<<<<<<<<<
 */
  __pyx_r = __Pyx_c_sum_double(__Pyx_c_prod_double(__pyx_v_z, __pyx_t_double_complex_from_parts(__pyx_v_a, 0)), __pyx_t_double_complex_from_parts(__pyx_v_b, 0));
  goto __pyx_L0;

  /* "_evalpoly.pxd":21
 * 
 * 
 * cdef inline double complex cevalpoly(double *coeffs, int degree,             # <<<<<<<<<<<<<<
 *                                      double complex z) nogil:
 *     """Evaluate a polynomial with real coefficients at a complex point.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "lambertw.pxd":42
 * 
 * @cython.cdivision(True)
 * cdef inline double complex lambertw_scalar(double complex z, long k, double tol) nogil:             # <<<<<<<<<<<<<<
 *     cdef int i
 *     cdef double absz, p
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8lambertw_lambertw_scalar(__pyx_t_double_complex __pyx_v_z, long __pyx_v_k, double __pyx_v_tol) {
  CYTHON_UNUSED int __pyx_v_i;
  double __pyx_v_absz;
  __pyx_t_double_complex __pyx_v_w;
  __pyx_t_double_complex __pyx_v_ew;
  __pyx_t_double_complex __pyx_v_wew;
  __pyx_t_double_complex __pyx_v_wewz;
  __pyx_t_double_complex __pyx_v_wn;
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "lambertw.pxd":48
 *     cdef double complex ew, wew, wewz, wn
 * 
 *     if zisnan(z):             # <<<<<<<<<<<<<<
 *         return z
 *     elif z.real == inf:
 */
  __pyx_t_1 = (__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisnan(__pyx_v_z) != 0);
  if (__pyx_t_1) {

    /* "lambertw.pxd":49
 * 
 *     if zisnan(z):
 *         return z             # <<<<<<<<<<<<<<
 *     elif z.real == inf:
 *         return z + twopi*k*1j
 */
    __pyx_r = __pyx_v_z;
    goto __pyx_L0;

    /* "lambertw.pxd":48
 *     cdef double complex ew, wew, wewz, wn
 * 
 *     if zisnan(z):             # <<<<<<<<<<<<<<
 *         return z
 *     elif z.real == inf:
 */
  }

  /* "lambertw.pxd":50
 *     if zisnan(z):
 *         return z
 *     elif z.real == inf:             # <<<<<<<<<<<<<<
 *         return z + twopi*k*1j
 *     elif z.real == -inf:
 */
  __pyx_t_1 = ((__Pyx_CREAL(__pyx_v_z) == NPY_INFINITY) != 0);
  if (__pyx_t_1) {

    /* "lambertw.pxd":51
 *         return z
 *     elif z.real == inf:
 *         return z + twopi*k*1j             # <<<<<<<<<<<<<<
 *     elif z.real == -inf:
 *         return -z + (twopi*k+pi)*1j
 */
    __pyx_r = __Pyx_c_sum_double(__pyx_v_z, __Pyx_c_prod_double(__pyx_t_double_complex_from_parts((6.283185307179586 * __pyx_v_k), 0), __pyx_t_double_complex_from_parts(0, 1.0)));
    goto __pyx_L0;

    /* "lambertw.pxd":50
 *     if zisnan(z):
 *         return z
 *     elif z.real == inf:             # <<<<<<<<<<<<<<
 *         return z + twopi*k*1j
 *     elif z.real == -inf:
 */
  }

  /* "lambertw.pxd":52
 *     elif z.real == inf:
 *         return z + twopi*k*1j
 *     elif z.real == -inf:             # <<<<<<<<<<<<<<
 *         return -z + (twopi*k+pi)*1j
 *     elif z == 0:
 */
  __pyx_t_1 = ((__Pyx_CREAL(__pyx_v_z) == (-NPY_INFINITY)) != 0);
  if (__pyx_t_1) {

    /* "lambertw.pxd":53
 *         return z + twopi*k*1j
 *     elif z.real == -inf:
 *         return -z + (twopi*k+pi)*1j             # <<<<<<<<<<<<<<
 *     elif z == 0:
 *         if k == 0:
 */
    __pyx_r = __Pyx_c_sum_double(__Pyx_c_neg_double(__pyx_v_z), __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(((6.283185307179586 * __pyx_v_k) + NPY_PI), 0), __pyx_t_double_complex_from_parts(0, 1.0)));
    goto __pyx_L0;

    /* "lambertw.pxd":52
 *     elif z.real == inf:
 *         return z + twopi*k*1j
 *     elif z.real == -inf:             # <<<<<<<<<<<<<<
 *         return -z + (twopi*k+pi)*1j
 *     elif z == 0:
 */
  }

  /* "lambertw.pxd":54
 *     elif z.real == -inf:
 *         return -z + (twopi*k+pi)*1j
 *     elif z == 0:             # <<<<<<<<<<<<<<
 *         if k == 0:
 *             return z
 */
  __pyx_t_1 = ((__Pyx_c_eq_double(__pyx_v_z, __pyx_t_double_complex_from_parts(0, 0))) != 0);
  if (__pyx_t_1) {

    /* "lambertw.pxd":55
 *         return -z + (twopi*k+pi)*1j
 *     elif z == 0:
 *         if k == 0:             # <<<<<<<<<<<<<<
 *             return z
 *         sf_error.error("lambertw", sf_error.SINGULAR, NULL)
 */
    __pyx_t_1 = ((__pyx_v_k == 0) != 0);
    if (__pyx_t_1) {

      /* "lambertw.pxd":56
 *     elif z == 0:
 *         if k == 0:
 *             return z             # <<<<<<<<<<<<<<
 *         sf_error.error("lambertw", sf_error.SINGULAR, NULL)
 *         return -inf
 */
      __pyx_r = __pyx_v_z;
      goto __pyx_L0;

      /* "lambertw.pxd":55
 *         return -z + (twopi*k+pi)*1j
 *     elif z == 0:
 *         if k == 0:             # <<<<<<<<<<<<<<
 *             return z
 *         sf_error.error("lambertw", sf_error.SINGULAR, NULL)
 */
    }

    /* "lambertw.pxd":57
 *         if k == 0:
 *             return z
 *         sf_error.error("lambertw", sf_error.SINGULAR, NULL)             # <<<<<<<<<<<<<<
 *         return -inf
 *     elif z == 1 and k == 0:
 */
    sf_error(((char *)"lambertw"), SF_ERROR_SINGULAR, NULL);

    /* "lambertw.pxd":58
 *             return z
 *         sf_error.error("lambertw", sf_error.SINGULAR, NULL)
 *         return -inf             # <<<<<<<<<<<<<<
 *     elif z == 1 and k == 0:
 *         # Split out this case because the asymptotic series blows up
 */
    __pyx_r = __pyx_t_double_complex_from_parts((-NPY_INFINITY), 0);
    goto __pyx_L0;

    /* "lambertw.pxd":54
 *     elif z.real == -inf:
 *         return -z + (twopi*k+pi)*1j
 *     elif z == 0:             # <<<<<<<<<<<<<<
 *         if k == 0:
 *             return z
 */
  }

  /* "lambertw.pxd":59
 *         sf_error.error("lambertw", sf_error.SINGULAR, NULL)
 *         return -inf
 *     elif z == 1 and k == 0:             # <<<<<<<<<<<<<<
 *         # Split out this case because the asymptotic series blows up
 *         return OMEGA
 */
  __pyx_t_2 = ((__Pyx_c_eq_double(__pyx_v_z, __pyx_t_double_complex_from_parts(1, 0))) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_k == 0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_1) {

    /* "lambertw.pxd":61
 *     elif z == 1 and k == 0:
 *         # Split out this case because the asymptotic series blows up
 *         return OMEGA             # <<<<<<<<<<<<<<
 * 
 *     absz = zabs(z)
 */
    __pyx_r = __pyx_t_double_complex_from_parts(0.5671432904097838, 0);
    goto __pyx_L0;

    /* "lambertw.pxd":59
 *         sf_error.error("lambertw", sf_error.SINGULAR, NULL)
 *         return -inf
 *     elif z == 1 and k == 0:             # <<<<<<<<<<<<<<
 *         # Split out this case because the asymptotic series blows up
 *         return OMEGA
 */
  }

  /* "lambertw.pxd":63
 *         return OMEGA
 * 
 *     absz = zabs(z)             # <<<<<<<<<<<<<<
 *     # Get an initial guess for Halley's method
 *     if k == 0:
 */
  __pyx_v_absz = __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_z);

  /* "lambertw.pxd":65
 *     absz = zabs(z)
 *     # Get an initial guess for Halley's method
 *     if k == 0:             # <<<<<<<<<<<<<<
 *         if zabs(z + EXPN1) < 0.3:
 *             w = lambertw_branchpt(z)
 */
  switch (__pyx_v_k) {
    case 0:

    /* "lambertw.pxd":66
 *     # Get an initial guess for Halley's method
 *     if k == 0:
 *         if zabs(z + EXPN1) < 0.3:             # <<<<<<<<<<<<<<
 *             w = lambertw_branchpt(z)
 *         elif (-1.0 < z.real < 1.5 and zabs(z.imag) < 1.0
 */
    __pyx_t_1 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__Pyx_c_sum_double(__pyx_v_z, __pyx_t_double_complex_from_parts(0.36787944117144233, 0))) < 0.3) != 0);
    if (__pyx_t_1) {

      /* "lambertw.pxd":67
 *     if k == 0:
 *         if zabs(z + EXPN1) < 0.3:
 *             w = lambertw_branchpt(z)             # <<<<<<<<<<<<<<
 *         elif (-1.0 < z.real < 1.5 and zabs(z.imag) < 1.0
 *               and -2.5*zabs(z.imag) - 0.2 < z.real):
 */
      __pyx_v_w = __pyx_f_5scipy_7special_8lambertw_lambertw_branchpt(__pyx_v_z);

      /* "lambertw.pxd":66
 *     # Get an initial guess for Halley's method
 *     if k == 0:
 *         if zabs(z + EXPN1) < 0.3:             # <<<<<<<<<<<<<<
 *             w = lambertw_branchpt(z)
 *         elif (-1.0 < z.real < 1.5 and zabs(z.imag) < 1.0
 */
      goto __pyx_L7;
    }

    /* "lambertw.pxd":68
 *         if zabs(z + EXPN1) < 0.3:
 *             w = lambertw_branchpt(z)
 *         elif (-1.0 < z.real < 1.5 and zabs(z.imag) < 1.0             # <<<<<<<<<<<<<<
 *               and -2.5*zabs(z.imag) - 0.2 < z.real):
 *             # Empirically determined decision boundary where the Pade
 */
    __pyx_t_2 = (-1.0 < __Pyx_CREAL(__pyx_v_z));
    if (__pyx_t_2) {
      __pyx_t_2 = (__Pyx_CREAL(__pyx_v_z) < 1.5);
    }
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
    } else {
      __pyx_t_1 = __pyx_t_3;
      goto __pyx_L8_bool_binop_done;
    }

    /* "lambertw.pxd":69
 *             w = lambertw_branchpt(z)
 *         elif (-1.0 < z.real < 1.5 and zabs(z.imag) < 1.0
 *               and -2.5*zabs(z.imag) - 0.2 < z.real):             # <<<<<<<<<<<<<<
 *             # Empirically determined decision boundary where the Pade
 *             # approximation is more accurate.
 */
    __pyx_t_3 = ((__pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zabs(__Pyx_CIMAG(__pyx_v_z)) < 1.0) != 0);
    if (__pyx_t_3) {
    } else {
      __pyx_t_1 = __pyx_t_3;
      goto __pyx_L8_bool_binop_done;
    }
    __pyx_t_3 = ((((-2.5 * __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zabs(__Pyx_CIMAG(__pyx_v_z))) - 0.2) < __Pyx_CREAL(__pyx_v_z)) != 0);
    __pyx_t_1 = __pyx_t_3;
    __pyx_L8_bool_binop_done:;

    /* "lambertw.pxd":68
 *         if zabs(z + EXPN1) < 0.3:
 *             w = lambertw_branchpt(z)
 *         elif (-1.0 < z.real < 1.5 and zabs(z.imag) < 1.0             # <<<<<<<<<<<<<<
 *               and -2.5*zabs(z.imag) - 0.2 < z.real):
 *             # Empirically determined decision boundary where the Pade
 */
    if (__pyx_t_1) {

      /* "lambertw.pxd":72
 *             # Empirically determined decision boundary where the Pade
 *             # approximation is more accurate.
 *             w = lambertw_pade0(z)             # <<<<<<<<<<<<<<
 *         else:
 *             w = lambertw_asy(z, k)
 */
      __pyx_v_w = __pyx_f_5scipy_7special_8lambertw_lambertw_pade0(__pyx_v_z);

      /* "lambertw.pxd":68
 *         if zabs(z + EXPN1) < 0.3:
 *             w = lambertw_branchpt(z)
 *         elif (-1.0 < z.real < 1.5 and zabs(z.imag) < 1.0             # <<<<<<<<<<<<<<
 *               and -2.5*zabs(z.imag) - 0.2 < z.real):
 *             # Empirically determined decision boundary where the Pade
 */
      goto __pyx_L7;
    }

    /* "lambertw.pxd":74
 *             w = lambertw_pade0(z)
 *         else:
 *             w = lambertw_asy(z, k)             # <<<<<<<<<<<<<<
 *     elif k == -1:
 *         if absz <= EXPN1 and z.imag == 0 and z.real < 0:
 */
    /*else*/ {
      __pyx_v_w = __pyx_f_5scipy_7special_8lambertw_lambertw_asy(__pyx_v_z, __pyx_v_k);
    }
    __pyx_L7:;

    /* "lambertw.pxd":65
 *     absz = zabs(z)
 *     # Get an initial guess for Halley's method
 *     if k == 0:             # <<<<<<<<<<<<<<
 *         if zabs(z + EXPN1) < 0.3:
 *             w = lambertw_branchpt(z)
 */
    break;
    case -1L:

    /* "lambertw.pxd":76
 *             w = lambertw_asy(z, k)
 *     elif k == -1:
 *         if absz <= EXPN1 and z.imag == 0 and z.real < 0:             # <<<<<<<<<<<<<<
 *             w = log(-z.real)
 *         else:
 */
    __pyx_t_3 = ((__pyx_v_absz <= 0.36787944117144233) != 0);
    if (__pyx_t_3) {
    } else {
      __pyx_t_1 = __pyx_t_3;
      goto __pyx_L12_bool_binop_done;
    }
    __pyx_t_3 = ((__Pyx_CIMAG(__pyx_v_z) == 0.0) != 0);
    if (__pyx_t_3) {
    } else {
      __pyx_t_1 = __pyx_t_3;
      goto __pyx_L12_bool_binop_done;
    }
    __pyx_t_3 = ((__Pyx_CREAL(__pyx_v_z) < 0.0) != 0);
    __pyx_t_1 = __pyx_t_3;
    __pyx_L12_bool_binop_done:;
    if (__pyx_t_1) {

      /* "lambertw.pxd":77
 *     elif k == -1:
 *         if absz <= EXPN1 and z.imag == 0 and z.real < 0:
 *             w = log(-z.real)             # <<<<<<<<<<<<<<
 *         else:
 *             w = lambertw_asy(z, k)
 */
      __pyx_v_w = __pyx_t_double_complex_from_parts(log((-__Pyx_CREAL(__pyx_v_z))), 0);

      /* "lambertw.pxd":76
 *             w = lambertw_asy(z, k)
 *     elif k == -1:
 *         if absz <= EXPN1 and z.imag == 0 and z.real < 0:             # <<<<<<<<<<<<<<
 *             w = log(-z.real)
 *         else:
 */
      goto __pyx_L11;
    }

    /* "lambertw.pxd":79
 *             w = log(-z.real)
 *         else:
 *             w = lambertw_asy(z, k)             # <<<<<<<<<<<<<<
 *     else:
 *         w = lambertw_asy(z, k)
 */
    /*else*/ {
      __pyx_v_w = __pyx_f_5scipy_7special_8lambertw_lambertw_asy(__pyx_v_z, __pyx_v_k);
    }
    __pyx_L11:;

    /* "lambertw.pxd":75
 *         else:
 *             w = lambertw_asy(z, k)
 *     elif k == -1:             # <<<<<<<<<<<<<<
 *         if absz <= EXPN1 and z.imag == 0 and z.real < 0:
 *             w = log(-z.real)
 */
    break;
    default:

    /* "lambertw.pxd":81
 *             w = lambertw_asy(z, k)
 *     else:
 *         w = lambertw_asy(z, k)             # <<<<<<<<<<<<<<
 * 
 *     # Halley's method; see 5.9 in [1]
 */
    __pyx_v_w = __pyx_f_5scipy_7special_8lambertw_lambertw_asy(__pyx_v_z, __pyx_v_k);
    break;
  }

  /* "lambertw.pxd":84
 * 
 *     # Halley's method; see 5.9 in [1]
 *     if w.real >= 0:             # <<<<<<<<<<<<<<
 *         # Rearrange the formula to avoid overflow in exp
 *         for i in range(100):
 */
  __pyx_t_1 = ((__Pyx_CREAL(__pyx_v_w) >= 0.0) != 0);
  if (__pyx_t_1) {

    /* "lambertw.pxd":86
 *     if w.real >= 0:
 *         # Rearrange the formula to avoid overflow in exp
 *         for i in range(100):             # <<<<<<<<<<<<<<
 *             ew = zexp(-w)
 *             wewz = w - z*ew
 */
    for (__pyx_t_4 = 0; __pyx_t_4 < 0x64; __pyx_t_4+=1) {
      __pyx_v_i = __pyx_t_4;

      /* "lambertw.pxd":87
 *         # Rearrange the formula to avoid overflow in exp
 *         for i in range(100):
 *             ew = zexp(-w)             # <<<<<<<<<<<<<<
 *             wewz = w - z*ew
 *             wn = w - wewz/(w + 1 - (w + 2)*wewz/(2*w + 2))
 */
      __pyx_v_ew = __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zexp(__Pyx_c_neg_double(__pyx_v_w));

      /* "lambertw.pxd":88
 *         for i in range(100):
 *             ew = zexp(-w)
 *             wewz = w - z*ew             # <<<<<<<<<<<<<<
 *             wn = w - wewz/(w + 1 - (w + 2)*wewz/(2*w + 2))
 *             if zabs(wn - w) < tol*zabs(wn):
 */
      __pyx_v_wewz = __Pyx_c_diff_double(__pyx_v_w, __Pyx_c_prod_double(__pyx_v_z, __pyx_v_ew));

      /* "lambertw.pxd":89
 *             ew = zexp(-w)
 *             wewz = w - z*ew
 *             wn = w - wewz/(w + 1 - (w + 2)*wewz/(2*w + 2))             # <<<<<<<<<<<<<<
 *             if zabs(wn - w) < tol*zabs(wn):
 *                 return wn
 */
      __pyx_v_wn = __Pyx_c_diff_double(__pyx_v_w, __Pyx_c_quot_double(__pyx_v_wewz, __Pyx_c_diff_double(__Pyx_c_sum_double(__pyx_v_w, __pyx_t_double_complex_from_parts(1, 0)), __Pyx_c_quot_double(__Pyx_c_prod_double(__Pyx_c_sum_double(__pyx_v_w, __pyx_t_double_complex_from_parts(2, 0)), __pyx_v_wewz), __Pyx_c_sum_double(__Pyx_c_prod_double(__pyx_t_double_complex_from_parts(2, 0), __pyx_v_w), __pyx_t_double_complex_from_parts(2, 0))))));

      /* "lambertw.pxd":90
 *             wewz = w - z*ew
 *             wn = w - wewz/(w + 1 - (w + 2)*wewz/(2*w + 2))
 *             if zabs(wn - w) < tol*zabs(wn):             # <<<<<<<<<<<<<<
 *                 return wn
 *             else:
 */
      __pyx_t_1 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__Pyx_c_diff_double(__pyx_v_wn, __pyx_v_w)) < (__pyx_v_tol * __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_wn))) != 0);
      if (__pyx_t_1) {

        /* "lambertw.pxd":91
 *             wn = w - wewz/(w + 1 - (w + 2)*wewz/(2*w + 2))
 *             if zabs(wn - w) < tol*zabs(wn):
 *                 return wn             # <<<<<<<<<<<<<<
 *             else:
 *                 w = wn
 */
        __pyx_r = __pyx_v_wn;
        goto __pyx_L0;

        /* "lambertw.pxd":90
 *             wewz = w - z*ew
 *             wn = w - wewz/(w + 1 - (w + 2)*wewz/(2*w + 2))
 *             if zabs(wn - w) < tol*zabs(wn):             # <<<<<<<<<<<<<<
 *                 return wn
 *             else:
 */
      }

      /* "lambertw.pxd":93
 *                 return wn
 *             else:
 *                 w = wn             # <<<<<<<<<<<<<<
 *     else:
 *         for i in range(100):
 */
      /*else*/ {
        __pyx_v_w = __pyx_v_wn;
      }
    }

    /* "lambertw.pxd":84
 * 
 *     # Halley's method; see 5.9 in [1]
 *     if w.real >= 0:             # <<<<<<<<<<<<<<
 *         # Rearrange the formula to avoid overflow in exp
 *         for i in range(100):
 */
    goto __pyx_L15;
  }

  /* "lambertw.pxd":95
 *                 w = wn
 *     else:
 *         for i in range(100):             # <<<<<<<<<<<<<<
 *             ew = zexp(w)
 *             wew = w*ew
 */
  /*else*/ {
    for (__pyx_t_4 = 0; __pyx_t_4 < 0x64; __pyx_t_4+=1) {
      __pyx_v_i = __pyx_t_4;

      /* "lambertw.pxd":96
 *     else:
 *         for i in range(100):
 *             ew = zexp(w)             # <<<<<<<<<<<<<<
 *             wew = w*ew
 *             wewz = wew - z
 */
      __pyx_v_ew = __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zexp(__pyx_v_w);

      /* "lambertw.pxd":97
 *         for i in range(100):
 *             ew = zexp(w)
 *             wew = w*ew             # <<<<<<<<<<<<<<
 *             wewz = wew - z
 *             wn = w - wewz/(wew + ew - (w + 2)*wewz/(2*w + 2))
 */
      __pyx_v_wew = __Pyx_c_prod_double(__pyx_v_w, __pyx_v_ew);

      /* "lambertw.pxd":98
 *             ew = zexp(w)
 *             wew = w*ew
 *             wewz = wew - z             # <<<<<<<<<<<<<<
 *             wn = w - wewz/(wew + ew - (w + 2)*wewz/(2*w + 2))
 *             if zabs(wn - w) < tol*zabs(wn):
 */
      __pyx_v_wewz = __Pyx_c_diff_double(__pyx_v_wew, __pyx_v_z);

      /* "lambertw.pxd":99
 *             wew = w*ew
 *             wewz = wew - z
 *             wn = w - wewz/(wew + ew - (w + 2)*wewz/(2*w + 2))             # <<<<<<<<<<<<<<
 *             if zabs(wn - w) < tol*zabs(wn):
 *                 return wn
 */
      __pyx_v_wn = __Pyx_c_diff_double(__pyx_v_w, __Pyx_c_quot_double(__pyx_v_wewz, __Pyx_c_diff_double(__Pyx_c_sum_double(__pyx_v_wew, __pyx_v_ew), __Pyx_c_quot_double(__Pyx_c_prod_double(__Pyx_c_sum_double(__pyx_v_w, __pyx_t_double_complex_from_parts(2, 0)), __pyx_v_wewz), __Pyx_c_sum_double(__Pyx_c_prod_double(__pyx_t_double_complex_from_parts(2, 0), __pyx_v_w), __pyx_t_double_complex_from_parts(2, 0))))));

      /* "lambertw.pxd":100
 *             wewz = wew - z
 *             wn = w - wewz/(wew + ew - (w + 2)*wewz/(2*w + 2))
 *             if zabs(wn - w) < tol*zabs(wn):             # <<<<<<<<<<<<<<
 *                 return wn
 *             else:
 */
      __pyx_t_1 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__Pyx_c_diff_double(__pyx_v_wn, __pyx_v_w)) < (__pyx_v_tol * __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_wn))) != 0);
      if (__pyx_t_1) {

        /* "lambertw.pxd":101
 *             wn = w - wewz/(wew + ew - (w + 2)*wewz/(2*w + 2))
 *             if zabs(wn - w) < tol*zabs(wn):
 *                 return wn             # <<<<<<<<<<<<<<
 *             else:
 *                 w = wn
 */
        __pyx_r = __pyx_v_wn;
        goto __pyx_L0;

        /* "lambertw.pxd":100
 *             wewz = wew - z
 *             wn = w - wewz/(wew + ew - (w + 2)*wewz/(2*w + 2))
 *             if zabs(wn - w) < tol*zabs(wn):             # <<<<<<<<<<<<<<
 *                 return wn
 *             else:
 */
      }

      /* "lambertw.pxd":103
 *                 return wn
 *             else:
 *                 w = wn             # <<<<<<<<<<<<<<
 * 
 *     sf_error.error("lambertw", sf_error.SLOW,
 */
      /*else*/ {
        __pyx_v_w = __pyx_v_wn;
      }
    }
  }
  __pyx_L15:;

  /* "lambertw.pxd":105
 *                 w = wn
 * 
 *     sf_error.error("lambertw", sf_error.SLOW,             # <<<<<<<<<<<<<<
 *                    "iteration failed to converge: %g + %gj",
 *                    <double>z.real, <double>z.imag)
 */
  sf_error(((char *)"lambertw"), SF_ERROR_SLOW, ((char *)"iteration failed to converge: %g + %gj"), ((double)__Pyx_CREAL(__pyx_v_z)), ((double)__Pyx_CIMAG(__pyx_v_z)));

  /* "lambertw.pxd":108
 *                    "iteration failed to converge: %g + %gj",
 *                    <double>z.real, <double>z.imag)
 *     return zpack(nan, nan)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_zpack(NPY_NAN, NPY_NAN);
  goto __pyx_L0;

  /* "lambertw.pxd":42
 * 
 * @cython.cdivision(True)
 * cdef inline double complex lambertw_scalar(double complex z, long k, double tol) nogil:             # <<<<<<<<<<<<<<
 *     cdef int i
 *     cdef double absz, p
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "lambertw.pxd":112
 * 
 * @cython.cdivision(True)
 * cdef inline double complex lambertw_branchpt(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """Series for W(z, 0) around the branch point; see 4.22 in [1]."""
 *     cdef double *coeffs = [-1.0/3.0, 1.0, -1.0]
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8lambertw_lambertw_branchpt(__pyx_t_double_complex __pyx_v_z) {
  double *__pyx_v_coeffs;
  __pyx_t_double_complex __pyx_v_p;
  __pyx_t_double_complex __pyx_r;
  double __pyx_t_1[3];

  /* "lambertw.pxd":114
 * cdef inline double complex lambertw_branchpt(double complex z) nogil:
 *     """Series for W(z, 0) around the branch point; see 4.22 in [1]."""
 *     cdef double *coeffs = [-1.0/3.0, 1.0, -1.0]             # <<<<<<<<<<<<<<
 *     cdef double complex p = zsqrt(2*(NPY_E*z + 1))
 * 
 */
  __pyx_t_1[0] = (-1.0 / 3.0);
  __pyx_t_1[1] = 1.0;
  __pyx_t_1[2] = -1.0;
  __pyx_v_coeffs = __pyx_t_1;

  /* "lambertw.pxd":115
 *     """Series for W(z, 0) around the branch point; see 4.22 in [1]."""
 *     cdef double *coeffs = [-1.0/3.0, 1.0, -1.0]
 *     cdef double complex p = zsqrt(2*(NPY_E*z + 1))             # <<<<<<<<<<<<<<
 * 
 *     return cevalpoly(coeffs, 2, p)
 */
  __pyx_v_p = __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zsqrt(__Pyx_c_prod_double(__pyx_t_double_complex_from_parts(2, 0), __Pyx_c_sum_double(__Pyx_c_prod_double(__pyx_t_double_complex_from_parts(NPY_E, 0), __pyx_v_z), __pyx_t_double_complex_from_parts(1, 0))));

  /* "lambertw.pxd":117
 *     cdef double complex p = zsqrt(2*(NPY_E*z + 1))
 * 
 *     return cevalpoly(coeffs, 2, p)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_f_5scipy_7special_9_evalpoly_cevalpoly(__pyx_v_coeffs, 2, __pyx_v_p);
  goto __pyx_L0;

  /* "lambertw.pxd":112
 * 
 * @cython.cdivision(True)
 * cdef inline double complex lambertw_branchpt(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """Series for W(z, 0) around the branch point; see 4.22 in [1]."""
 *     cdef double *coeffs = [-1.0/3.0, 1.0, -1.0]
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "lambertw.pxd":121
 * 
 * @cython.cdivision(True)
 * cdef inline double complex lambertw_pade0(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """(3, 2) Pade approximation for W(z, 0) around 0."""
 *     cdef:
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8lambertw_lambertw_pade0(__pyx_t_double_complex __pyx_v_z) {
  double *__pyx_v_num;
  double *__pyx_v_denom;
  __pyx_t_double_complex __pyx_r;
  double __pyx_t_1[3];
  double __pyx_t_2[3];

  /* "lambertw.pxd":124
 *     """(3, 2) Pade approximation for W(z, 0) around 0."""
 *     cdef:
 *         double *num = [             # <<<<<<<<<<<<<<
 *             12.85106382978723404255,
 *             12.34042553191489361902,
 */
  __pyx_t_1[0] = 12.85106382978723404255;
  __pyx_t_1[1] = 12.34042553191489361902;
  __pyx_t_1[2] = 1.0;
  __pyx_v_num = __pyx_t_1;

  /* "lambertw.pxd":129
 *             1.0
 *         ]
 *         double *denom = [             # <<<<<<<<<<<<<<
 *             32.53191489361702127660,
 *             14.34042553191489361702,
 */
  __pyx_t_2[0] = 32.53191489361702127660;
  __pyx_t_2[1] = 14.34042553191489361702;
  __pyx_t_2[2] = 1.0;
  __pyx_v_denom = __pyx_t_2;

  /* "lambertw.pxd":138
 *     # careful algorithm that avoids overflow in the numerator for
 *     # large z.
 *     return z*cevalpoly(num, 2, z)/cevalpoly(denom, 2, z)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __Pyx_c_quot_double(__Pyx_c_prod_double(__pyx_v_z, __pyx_f_5scipy_7special_9_evalpoly_cevalpoly(__pyx_v_num, 2, __pyx_v_z)), __pyx_f_5scipy_7special_9_evalpoly_cevalpoly(__pyx_v_denom, 2, __pyx_v_z));
  goto __pyx_L0;

  /* "lambertw.pxd":121
 * 
 * @cython.cdivision(True)
 * cdef inline double complex lambertw_pade0(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """(3, 2) Pade approximation for W(z, 0) around 0."""
 *     cdef:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "lambertw.pxd":142
 * 
 * @cython.cdivision(True)
 * cdef inline double complex lambertw_asy(double complex z, long k) nogil:             # <<<<<<<<<<<<<<
 *     """Compute the W function using the first two terms of the
 *     asymptotic series. See 4.20 in [1].
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8lambertw_lambertw_asy(__pyx_t_double_complex __pyx_v_z, long __pyx_v_k) {
  __pyx_t_double_complex __pyx_v_w;
  __pyx_t_double_complex __pyx_r;

  /* "lambertw.pxd":149
 *     cdef double complex w
 * 
 *     w = zlog(z) + twopi*k*1j             # <<<<<<<<<<<<<<
 *     return w - zlog(w)
 */
  __pyx_v_w = __Pyx_c_sum_double(__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zlog(__pyx_v_z), __Pyx_c_prod_double(__pyx_t_double_complex_from_parts((6.283185307179586 * __pyx_v_k), 0), __pyx_t_double_complex_from_parts(0, 1.0)));

  /* "lambertw.pxd":150
 * 
 *     w = zlog(z) + twopi*k*1j
 *     return w - zlog(w)             # <<<<<<<<<<<<<<
 */
  __pyx_r = __Pyx_c_diff_double(__pyx_v_w, __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zlog(__pyx_v_w));
  goto __pyx_L0;

  /* "lambertw.pxd":142
 * 
 * @cython.cdivision(True)
 * cdef inline double complex lambertw_asy(double complex z, long k) nogil:             # <<<<<<<<<<<<<<
 *     """Compute the W function using the first two terms of the
 *     asymptotic series. See 4.20 in [1].
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spherical_bessel.pxd":46
 * # Fused type wrappers
 * 
 * cdef inline number_t cbesj(double v, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_17_spherical_bessel_cbesj(double __pyx_v_v, double __pyx_v_z) {
  double __pyx_r;

  /* "_spherical_bessel.pxd":49
 *     cdef npy_cdouble r
 *     if number_t is double:
 *         return cbesj_wrap_real(v, z)             # <<<<<<<<<<<<<<
 *     else:
 *         r = cbesj_wrap(v, npy_cdouble_from_double_complex(z))
 */
  __pyx_r = cbesj_wrap_real(__pyx_v_v, __pyx_v_z);
  goto __pyx_L0;

  /* "_spherical_bessel.pxd":46
 * # Fused type wrappers
 * 
 * cdef inline number_t cbesj(double v, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_17_spherical_bessel_cbesj(double __pyx_v_v, __pyx_t_double_complex __pyx_v_z) {
  npy_cdouble __pyx_v_r;
  __pyx_t_double_complex __pyx_r;

  /* "_spherical_bessel.pxd":51
 *         return cbesj_wrap_real(v, z)
 *     else:
 *         r = cbesj_wrap(v, npy_cdouble_from_double_complex(z))             # <<<<<<<<<<<<<<
 *         return double_complex_from_npy_cdouble(r)
 * 
 */
  __pyx_v_r = cbesj_wrap(__pyx_v_v, __pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_z));

  /* "_spherical_bessel.pxd":52
 *     else:
 *         r = cbesj_wrap(v, npy_cdouble_from_double_complex(z))
 *         return double_complex_from_npy_cdouble(r)             # <<<<<<<<<<<<<<
 * 
 * cdef inline number_t cbesy(double v, number_t z) nogil:
 */
  __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_double_complex_from_npy_cdouble(__pyx_v_r);
  goto __pyx_L0;

  /* "_spherical_bessel.pxd":46
 * # Fused type wrappers
 * 
 * cdef inline number_t cbesj(double v, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spherical_bessel.pxd":54
 *         return double_complex_from_npy_cdouble(r)
 * 
 * cdef inline number_t cbesy(double v, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_17_spherical_bessel_cbesy(double __pyx_v_v, double __pyx_v_z) {
  double __pyx_r;

  /* "_spherical_bessel.pxd":57
 *     cdef npy_cdouble r
 *     if number_t is double:
 *         return cbesy_wrap_real(v, z)             # <<<<<<<<<<<<<<
 *     else:
 *         r = cbesy_wrap(v, npy_cdouble_from_double_complex(z))
 */
  __pyx_r = cbesy_wrap_real(__pyx_v_v, __pyx_v_z);
  goto __pyx_L0;

  /* "_spherical_bessel.pxd":54
 *         return double_complex_from_npy_cdouble(r)
 * 
 * cdef inline number_t cbesy(double v, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_17_spherical_bessel_cbesy(double __pyx_v_v, __pyx_t_double_complex __pyx_v_z) {
  npy_cdouble __pyx_v_r;
  __pyx_t_double_complex __pyx_r;

  /* "_spherical_bessel.pxd":59
 *         return cbesy_wrap_real(v, z)
 *     else:
 *         r = cbesy_wrap(v, npy_cdouble_from_double_complex(z))             # <<<<<<<<<<<<<<
 *         return double_complex_from_npy_cdouble(r)
 * 
 */
  __pyx_v_r = cbesy_wrap(__pyx_v_v, __pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_z));

  /* "_spherical_bessel.pxd":60
 *     else:
 *         r = cbesy_wrap(v, npy_cdouble_from_double_complex(z))
 *         return double_complex_from_npy_cdouble(r)             # <<<<<<<<<<<<<<
 * 
 * cdef inline number_t cbesk(double v, number_t z) nogil:
 */
  __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_double_complex_from_npy_cdouble(__pyx_v_r);
  goto __pyx_L0;

  /* "_spherical_bessel.pxd":54
 *         return double_complex_from_npy_cdouble(r)
 * 
 * cdef inline number_t cbesy(double v, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spherical_bessel.pxd":62
 *         return double_complex_from_npy_cdouble(r)
 * 
 * cdef inline number_t cbesk(double v, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_17_spherical_bessel_cbesk(double __pyx_v_v, double __pyx_v_z) {
  double __pyx_r;

  /* "_spherical_bessel.pxd":65
 *     cdef npy_cdouble r
 *     if number_t is double:
 *         return cbesk_wrap_real(v, z)             # <<<<<<<<<<<<<<
 *     else:
 *         r = cbesk_wrap(v, npy_cdouble_from_double_complex(z))
 */
  __pyx_r = cbesk_wrap_real(__pyx_v_v, __pyx_v_z);
  goto __pyx_L0;

  /* "_spherical_bessel.pxd":62
 *         return double_complex_from_npy_cdouble(r)
 * 
 * cdef inline number_t cbesk(double v, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_17_spherical_bessel_cbesk(double __pyx_v_v, __pyx_t_double_complex __pyx_v_z) {
  npy_cdouble __pyx_v_r;
  __pyx_t_double_complex __pyx_r;

  /* "_spherical_bessel.pxd":67
 *         return cbesk_wrap_real(v, z)
 *     else:
 *         r = cbesk_wrap(v, npy_cdouble_from_double_complex(z))             # <<<<<<<<<<<<<<
 *         return double_complex_from_npy_cdouble(r)
 * 
 */
  __pyx_v_r = cbesk_wrap(__pyx_v_v, __pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_z));

  /* "_spherical_bessel.pxd":68
 *     else:
 *         r = cbesk_wrap(v, npy_cdouble_from_double_complex(z))
 *         return double_complex_from_npy_cdouble(r)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_double_complex_from_npy_cdouble(__pyx_v_r);
  goto __pyx_L0;

  /* "_spherical_bessel.pxd":62
 *         return double_complex_from_npy_cdouble(r)
 * 
 * cdef inline number_t cbesk(double v, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spherical_bessel.pxd":74
 * 
 * @cython.cdivision(True)
 * cdef inline double spherical_jn_real(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double s0, s1, sn
 *     cdef int idx
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_real(long __pyx_v_n, double __pyx_v_x) {
  double __pyx_v_s0;
  double __pyx_v_s1;
  double __pyx_v_sn;
  int __pyx_v_idx;
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  long __pyx_t_3;
  long __pyx_t_4;
  int __pyx_t_5;

  /* "_spherical_bessel.pxd":78
 *     cdef int idx
 * 
 *     if npy_isnan(x):             # <<<<<<<<<<<<<<
 *         return x
 *     if n < 0:
 */
  __pyx_t_1 = (npy_isnan(__pyx_v_x) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":79
 * 
 *     if npy_isnan(x):
 *         return x             # <<<<<<<<<<<<<<
 *     if n < 0:
 *         sf_error.error("spherical_jn", sf_error.DOMAIN, NULL)
 */
    __pyx_r = __pyx_v_x;
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":78
 *     cdef int idx
 * 
 *     if npy_isnan(x):             # <<<<<<<<<<<<<<
 *         return x
 *     if n < 0:
 */
  }

  /* "_spherical_bessel.pxd":80
 *     if npy_isnan(x):
 *         return x
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("spherical_jn", sf_error.DOMAIN, NULL)
 *         return nan
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":81
 *         return x
 *     if n < 0:
 *         sf_error.error("spherical_jn", sf_error.DOMAIN, NULL)             # <<<<<<<<<<<<<<
 *         return nan
 *     if x == inf or x == -inf:
 */
    sf_error(((char *)"spherical_jn"), SF_ERROR_DOMAIN, NULL);

    /* "_spherical_bessel.pxd":82
 *     if n < 0:
 *         sf_error.error("spherical_jn", sf_error.DOMAIN, NULL)
 *         return nan             # <<<<<<<<<<<<<<
 *     if x == inf or x == -inf:
 *         return 0
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":80
 *     if npy_isnan(x):
 *         return x
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("spherical_jn", sf_error.DOMAIN, NULL)
 *         return nan
 */
  }

  /* "_spherical_bessel.pxd":83
 *         sf_error.error("spherical_jn", sf_error.DOMAIN, NULL)
 *         return nan
 *     if x == inf or x == -inf:             # <<<<<<<<<<<<<<
 *         return 0
 *     if x == 0:
 */
  __pyx_t_2 = ((__pyx_v_x == NPY_INFINITY) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_x == (-NPY_INFINITY)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":84
 *         return nan
 *     if x == inf or x == -inf:
 *         return 0             # <<<<<<<<<<<<<<
 *     if x == 0:
 *         if n == 0:
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":83
 *         sf_error.error("spherical_jn", sf_error.DOMAIN, NULL)
 *         return nan
 *     if x == inf or x == -inf:             # <<<<<<<<<<<<<<
 *         return 0
 *     if x == 0:
 */
  }

  /* "_spherical_bessel.pxd":85
 *     if x == inf or x == -inf:
 *         return 0
 *     if x == 0:             # <<<<<<<<<<<<<<
 *         if n == 0:
 *             return 1
 */
  __pyx_t_1 = ((__pyx_v_x == 0.0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":86
 *         return 0
 *     if x == 0:
 *         if n == 0:             # <<<<<<<<<<<<<<
 *             return 1
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_n == 0) != 0);
    if (__pyx_t_1) {

      /* "_spherical_bessel.pxd":87
 *     if x == 0:
 *         if n == 0:
 *             return 1             # <<<<<<<<<<<<<<
 *         else:
 *             return 0
 */
      __pyx_r = 1.0;
      goto __pyx_L0;

      /* "_spherical_bessel.pxd":86
 *         return 0
 *     if x == 0:
 *         if n == 0:             # <<<<<<<<<<<<<<
 *             return 1
 *         else:
 */
    }

    /* "_spherical_bessel.pxd":89
 *             return 1
 *         else:
 *             return 0             # <<<<<<<<<<<<<<
 * 
 *     if n > 0 and n >= x:
 */
    /*else*/ {
      __pyx_r = 0.0;
      goto __pyx_L0;
    }

    /* "_spherical_bessel.pxd":85
 *     if x == inf or x == -inf:
 *         return 0
 *     if x == 0:             # <<<<<<<<<<<<<<
 *         if n == 0:
 *             return 1
 */
  }

  /* "_spherical_bessel.pxd":91
 *             return 0
 * 
 *     if n > 0 and n >= x:             # <<<<<<<<<<<<<<
 *         return sqrt(M_PI_2/x)*cbesj(n + 0.5, x)
 * 
 */
  __pyx_t_2 = ((__pyx_v_n > 0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L11_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_n >= __pyx_v_x) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L11_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":92
 * 
 *     if n > 0 and n >= x:
 *         return sqrt(M_PI_2/x)*cbesj(n + 0.5, x)             # <<<<<<<<<<<<<<
 * 
 *     s0 = sin(x)/x
 */
    __pyx_r = (sqrt((((double)M_PI_2) / __pyx_v_x)) * __pyx_fuse_0__pyx_f_5scipy_7special_17_spherical_bessel_cbesj((__pyx_v_n + 0.5), __pyx_v_x));
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":91
 *             return 0
 * 
 *     if n > 0 and n >= x:             # <<<<<<<<<<<<<<
 *         return sqrt(M_PI_2/x)*cbesj(n + 0.5, x)
 * 
 */
  }

  /* "_spherical_bessel.pxd":94
 *         return sqrt(M_PI_2/x)*cbesj(n + 0.5, x)
 * 
 *     s0 = sin(x)/x             # <<<<<<<<<<<<<<
 *     if n == 0:
 *         return s0
 */
  __pyx_v_s0 = (sin(__pyx_v_x) / __pyx_v_x);

  /* "_spherical_bessel.pxd":95
 * 
 *     s0 = sin(x)/x
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return s0
 *     s1 = (s0 - cos(x))/x
 */
  __pyx_t_1 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":96
 *     s0 = sin(x)/x
 *     if n == 0:
 *         return s0             # <<<<<<<<<<<<<<
 *     s1 = (s0 - cos(x))/x
 *     if n == 1:
 */
    __pyx_r = __pyx_v_s0;
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":95
 * 
 *     s0 = sin(x)/x
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return s0
 *     s1 = (s0 - cos(x))/x
 */
  }

  /* "_spherical_bessel.pxd":97
 *     if n == 0:
 *         return s0
 *     s1 = (s0 - cos(x))/x             # <<<<<<<<<<<<<<
 *     if n == 1:
 *         return s1
 */
  __pyx_v_s1 = ((__pyx_v_s0 - cos(__pyx_v_x)) / __pyx_v_x);

  /* "_spherical_bessel.pxd":98
 *         return s0
 *     s1 = (s0 - cos(x))/x
 *     if n == 1:             # <<<<<<<<<<<<<<
 *         return s1
 * 
 */
  __pyx_t_1 = ((__pyx_v_n == 1) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":99
 *     s1 = (s0 - cos(x))/x
 *     if n == 1:
 *         return s1             # <<<<<<<<<<<<<<
 * 
 *     for idx in range(n - 1):
 */
    __pyx_r = __pyx_v_s1;
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":98
 *         return s0
 *     s1 = (s0 - cos(x))/x
 *     if n == 1:             # <<<<<<<<<<<<<<
 *         return s1
 * 
 */
  }

  /* "_spherical_bessel.pxd":101
 *         return s1
 * 
 *     for idx in range(n - 1):             # <<<<<<<<<<<<<<
 *         sn = (2*idx + 3)*s1/x - s0
 *         s0 = s1
 */
  __pyx_t_3 = (__pyx_v_n - 1);
  __pyx_t_4 = __pyx_t_3;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_idx = __pyx_t_5;

    /* "_spherical_bessel.pxd":102
 * 
 *     for idx in range(n - 1):
 *         sn = (2*idx + 3)*s1/x - s0             # <<<<<<<<<<<<<<
 *         s0 = s1
 *         s1 = sn
 */
    __pyx_v_sn = (((((2 * __pyx_v_idx) + 3) * __pyx_v_s1) / __pyx_v_x) - __pyx_v_s0);

    /* "_spherical_bessel.pxd":103
 *     for idx in range(n - 1):
 *         sn = (2*idx + 3)*s1/x - s0
 *         s0 = s1             # <<<<<<<<<<<<<<
 *         s1 = sn
 *         if npy_isinf(sn):
 */
    __pyx_v_s0 = __pyx_v_s1;

    /* "_spherical_bessel.pxd":104
 *         sn = (2*idx + 3)*s1/x - s0
 *         s0 = s1
 *         s1 = sn             # <<<<<<<<<<<<<<
 *         if npy_isinf(sn):
 *             # Overflow occurred already: terminate recurrence.
 */
    __pyx_v_s1 = __pyx_v_sn;

    /* "_spherical_bessel.pxd":105
 *         s0 = s1
 *         s1 = sn
 *         if npy_isinf(sn):             # <<<<<<<<<<<<<<
 *             # Overflow occurred already: terminate recurrence.
 *             return sn
 */
    __pyx_t_1 = (npy_isinf(__pyx_v_sn) != 0);
    if (__pyx_t_1) {

      /* "_spherical_bessel.pxd":107
 *         if npy_isinf(sn):
 *             # Overflow occurred already: terminate recurrence.
 *             return sn             # <<<<<<<<<<<<<<
 * 
 *     return sn
 */
      __pyx_r = __pyx_v_sn;
      goto __pyx_L0;

      /* "_spherical_bessel.pxd":105
 *         s0 = s1
 *         s1 = sn
 *         if npy_isinf(sn):             # <<<<<<<<<<<<<<
 *             # Overflow occurred already: terminate recurrence.
 *             return sn
 */
    }
  }

  /* "_spherical_bessel.pxd":109
 *             return sn
 * 
 *     return sn             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_sn;
  goto __pyx_L0;

  /* "_spherical_bessel.pxd":74
 * 
 * @cython.cdivision(True)
 * cdef inline double spherical_jn_real(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double s0, s1, sn
 *     cdef int idx
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spherical_bessel.pxd":113
 * 
 * @cython.cdivision(True)
 * cdef inline double complex spherical_jn_complex(long n, double complex z) nogil:             # <<<<<<<<<<<<<<
 *     cdef double complex out
 *     if zisnan(z):
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_complex(long __pyx_v_n, __pyx_t_double_complex __pyx_v_z) {
  __pyx_t_double_complex __pyx_v_out;
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_spherical_bessel.pxd":115
 * cdef inline double complex spherical_jn_complex(long n, double complex z) nogil:
 *     cdef double complex out
 *     if zisnan(z):             # <<<<<<<<<<<<<<
 *         return z
 *     if n < 0:
 */
  __pyx_t_1 = (__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisnan(__pyx_v_z) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":116
 *     cdef double complex out
 *     if zisnan(z):
 *         return z             # <<<<<<<<<<<<<<
 *     if n < 0:
 *         sf_error.error("spherical_jn", sf_error.DOMAIN, NULL)
 */
    __pyx_r = __pyx_v_z;
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":115
 * cdef inline double complex spherical_jn_complex(long n, double complex z) nogil:
 *     cdef double complex out
 *     if zisnan(z):             # <<<<<<<<<<<<<<
 *         return z
 *     if n < 0:
 */
  }

  /* "_spherical_bessel.pxd":117
 *     if zisnan(z):
 *         return z
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("spherical_jn", sf_error.DOMAIN, NULL)
 *         return nan
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":118
 *         return z
 *     if n < 0:
 *         sf_error.error("spherical_jn", sf_error.DOMAIN, NULL)             # <<<<<<<<<<<<<<
 *         return nan
 *     if z.real == inf or z.real == -inf:
 */
    sf_error(((char *)"spherical_jn"), SF_ERROR_DOMAIN, NULL);

    /* "_spherical_bessel.pxd":119
 *     if n < 0:
 *         sf_error.error("spherical_jn", sf_error.DOMAIN, NULL)
 *         return nan             # <<<<<<<<<<<<<<
 *     if z.real == inf or z.real == -inf:
 *         # https://dlmf.nist.gov/10.52.E3
 */
    __pyx_r = __pyx_t_double_complex_from_parts(NPY_NAN, 0);
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":117
 *     if zisnan(z):
 *         return z
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("spherical_jn", sf_error.DOMAIN, NULL)
 *         return nan
 */
  }

  /* "_spherical_bessel.pxd":120
 *         sf_error.error("spherical_jn", sf_error.DOMAIN, NULL)
 *         return nan
 *     if z.real == inf or z.real == -inf:             # <<<<<<<<<<<<<<
 *         # https://dlmf.nist.gov/10.52.E3
 *         if z.imag == 0:
 */
  __pyx_t_2 = ((__Pyx_CREAL(__pyx_v_z) == NPY_INFINITY) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_2 = ((__Pyx_CREAL(__pyx_v_z) == (-NPY_INFINITY)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":122
 *     if z.real == inf or z.real == -inf:
 *         # https://dlmf.nist.gov/10.52.E3
 *         if z.imag == 0:             # <<<<<<<<<<<<<<
 *             return 0
 *         else:
 */
    __pyx_t_1 = ((__Pyx_CIMAG(__pyx_v_z) == 0.0) != 0);
    if (__pyx_t_1) {

      /* "_spherical_bessel.pxd":123
 *         # https://dlmf.nist.gov/10.52.E3
 *         if z.imag == 0:
 *             return 0             # <<<<<<<<<<<<<<
 *         else:
 *             return (1+1j)*inf
 */
      __pyx_r = __pyx_t_double_complex_from_parts(0, 0);
      goto __pyx_L0;

      /* "_spherical_bessel.pxd":122
 *     if z.real == inf or z.real == -inf:
 *         # https://dlmf.nist.gov/10.52.E3
 *         if z.imag == 0:             # <<<<<<<<<<<<<<
 *             return 0
 *         else:
 */
    }

    /* "_spherical_bessel.pxd":125
 *             return 0
 *         else:
 *             return (1+1j)*inf             # <<<<<<<<<<<<<<
 *     if z.real == 0 and z.imag == 0:
 *         if n == 0:
 */
    /*else*/ {
      __pyx_r = __Pyx_c_prod_double(__Pyx_c_sum_double(__pyx_t_double_complex_from_parts(1, 0), __pyx_t_double_complex_from_parts(0, 1.0)), __pyx_t_double_complex_from_parts(NPY_INFINITY, 0));
      goto __pyx_L0;
    }

    /* "_spherical_bessel.pxd":120
 *         sf_error.error("spherical_jn", sf_error.DOMAIN, NULL)
 *         return nan
 *     if z.real == inf or z.real == -inf:             # <<<<<<<<<<<<<<
 *         # https://dlmf.nist.gov/10.52.E3
 *         if z.imag == 0:
 */
  }

  /* "_spherical_bessel.pxd":126
 *         else:
 *             return (1+1j)*inf
 *     if z.real == 0 and z.imag == 0:             # <<<<<<<<<<<<<<
 *         if n == 0:
 *             return 1
 */
  __pyx_t_2 = ((__Pyx_CREAL(__pyx_v_z) == 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L10_bool_binop_done;
  }
  __pyx_t_2 = ((__Pyx_CIMAG(__pyx_v_z) == 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L10_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":127
 *             return (1+1j)*inf
 *     if z.real == 0 and z.imag == 0:
 *         if n == 0:             # <<<<<<<<<<<<<<
 *             return 1
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_n == 0) != 0);
    if (__pyx_t_1) {

      /* "_spherical_bessel.pxd":128
 *     if z.real == 0 and z.imag == 0:
 *         if n == 0:
 *             return 1             # <<<<<<<<<<<<<<
 *         else:
 *             return 0
 */
      __pyx_r = __pyx_t_double_complex_from_parts(1, 0);
      goto __pyx_L0;

      /* "_spherical_bessel.pxd":127
 *             return (1+1j)*inf
 *     if z.real == 0 and z.imag == 0:
 *         if n == 0:             # <<<<<<<<<<<<<<
 *             return 1
 *         else:
 */
    }

    /* "_spherical_bessel.pxd":130
 *             return 1
 *         else:
 *             return 0             # <<<<<<<<<<<<<<
 * 
 *     out = zsqrt(M_PI_2/z)*cbesj(n + 0.5, z)
 */
    /*else*/ {
      __pyx_r = __pyx_t_double_complex_from_parts(0, 0);
      goto __pyx_L0;
    }

    /* "_spherical_bessel.pxd":126
 *         else:
 *             return (1+1j)*inf
 *     if z.real == 0 and z.imag == 0:             # <<<<<<<<<<<<<<
 *         if n == 0:
 *             return 1
 */
  }

  /* "_spherical_bessel.pxd":132
 *             return 0
 * 
 *     out = zsqrt(M_PI_2/z)*cbesj(n + 0.5, z)             # <<<<<<<<<<<<<<
 * 
 *     if z.imag == 0:
 */
  __pyx_v_out = __Pyx_c_prod_double(__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zsqrt(__Pyx_c_quot_double(__pyx_t_double_complex_from_parts(M_PI_2, 0), __pyx_v_z)), __pyx_fuse_1__pyx_f_5scipy_7special_17_spherical_bessel_cbesj((__pyx_v_n + 0.5), __pyx_v_z));

  /* "_spherical_bessel.pxd":134
 *     out = zsqrt(M_PI_2/z)*cbesj(n + 0.5, z)
 * 
 *     if z.imag == 0:             # <<<<<<<<<<<<<<
 *         # Small imaginary part is spurious
 *         return out.real
 */
  __pyx_t_1 = ((__Pyx_CIMAG(__pyx_v_z) == 0.0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":136
 *     if z.imag == 0:
 *         # Small imaginary part is spurious
 *         return out.real             # <<<<<<<<<<<<<<
 *     else:
 *         return out
 */
    __pyx_r = __pyx_t_double_complex_from_parts(__Pyx_CREAL(__pyx_v_out), 0);
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":134
 *     out = zsqrt(M_PI_2/z)*cbesj(n + 0.5, z)
 * 
 *     if z.imag == 0:             # <<<<<<<<<<<<<<
 *         # Small imaginary part is spurious
 *         return out.real
 */
  }

  /* "_spherical_bessel.pxd":138
 *         return out.real
 *     else:
 *         return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_r = __pyx_v_out;
    goto __pyx_L0;
  }

  /* "_spherical_bessel.pxd":113
 * 
 * @cython.cdivision(True)
 * cdef inline double complex spherical_jn_complex(long n, double complex z) nogil:             # <<<<<<<<<<<<<<
 *     cdef double complex out
 *     if zisnan(z):
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spherical_bessel.pxd":142
 * 
 * @cython.cdivision(True)
 * cdef inline double spherical_yn_real(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double s0, s1, sn
 *     cdef int idx
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_real(long __pyx_v_n, double __pyx_v_x) {
  double __pyx_v_s0;
  double __pyx_v_s1;
  double __pyx_v_sn;
  int __pyx_v_idx;
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  long __pyx_t_3;
  long __pyx_t_4;
  int __pyx_t_5;

  /* "_spherical_bessel.pxd":146
 *     cdef int idx
 * 
 *     if npy_isnan(x):             # <<<<<<<<<<<<<<
 *         return x
 *     if n < 0:
 */
  __pyx_t_1 = (npy_isnan(__pyx_v_x) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":147
 * 
 *     if npy_isnan(x):
 *         return x             # <<<<<<<<<<<<<<
 *     if n < 0:
 *         sf_error.error("spherical_yn", sf_error.DOMAIN, NULL)
 */
    __pyx_r = __pyx_v_x;
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":146
 *     cdef int idx
 * 
 *     if npy_isnan(x):             # <<<<<<<<<<<<<<
 *         return x
 *     if n < 0:
 */
  }

  /* "_spherical_bessel.pxd":148
 *     if npy_isnan(x):
 *         return x
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("spherical_yn", sf_error.DOMAIN, NULL)
 *         return nan
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":149
 *         return x
 *     if n < 0:
 *         sf_error.error("spherical_yn", sf_error.DOMAIN, NULL)             # <<<<<<<<<<<<<<
 *         return nan
 *     if x < 0:
 */
    sf_error(((char *)"spherical_yn"), SF_ERROR_DOMAIN, NULL);

    /* "_spherical_bessel.pxd":150
 *     if n < 0:
 *         sf_error.error("spherical_yn", sf_error.DOMAIN, NULL)
 *         return nan             # <<<<<<<<<<<<<<
 *     if x < 0:
 *         return (-1)**(n+1)*spherical_yn_real(n, -x)
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":148
 *     if npy_isnan(x):
 *         return x
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("spherical_yn", sf_error.DOMAIN, NULL)
 *         return nan
 */
  }

  /* "_spherical_bessel.pxd":151
 *         sf_error.error("spherical_yn", sf_error.DOMAIN, NULL)
 *         return nan
 *     if x < 0:             # <<<<<<<<<<<<<<
 *         return (-1)**(n+1)*spherical_yn_real(n, -x)
 *     if x == inf or x == -inf:
 */
  __pyx_t_1 = ((__pyx_v_x < 0.0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":152
 *         return nan
 *     if x < 0:
 *         return (-1)**(n+1)*spherical_yn_real(n, -x)             # <<<<<<<<<<<<<<
 *     if x == inf or x == -inf:
 *         return 0
 */
    __pyx_r = (__Pyx_pow_long(-1L, (__pyx_v_n + 1)) * __pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_real(__pyx_v_n, (-__pyx_v_x)));
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":151
 *         sf_error.error("spherical_yn", sf_error.DOMAIN, NULL)
 *         return nan
 *     if x < 0:             # <<<<<<<<<<<<<<
 *         return (-1)**(n+1)*spherical_yn_real(n, -x)
 *     if x == inf or x == -inf:
 */
  }

  /* "_spherical_bessel.pxd":153
 *     if x < 0:
 *         return (-1)**(n+1)*spherical_yn_real(n, -x)
 *     if x == inf or x == -inf:             # <<<<<<<<<<<<<<
 *         return 0
 *     if x == 0:
 */
  __pyx_t_2 = ((__pyx_v_x == NPY_INFINITY) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_x == (-NPY_INFINITY)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L7_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":154
 *         return (-1)**(n+1)*spherical_yn_real(n, -x)
 *     if x == inf or x == -inf:
 *         return 0             # <<<<<<<<<<<<<<
 *     if x == 0:
 *         return -inf
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":153
 *     if x < 0:
 *         return (-1)**(n+1)*spherical_yn_real(n, -x)
 *     if x == inf or x == -inf:             # <<<<<<<<<<<<<<
 *         return 0
 *     if x == 0:
 */
  }

  /* "_spherical_bessel.pxd":155
 *     if x == inf or x == -inf:
 *         return 0
 *     if x == 0:             # <<<<<<<<<<<<<<
 *         return -inf
 * 
 */
  __pyx_t_1 = ((__pyx_v_x == 0.0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":156
 *         return 0
 *     if x == 0:
 *         return -inf             # <<<<<<<<<<<<<<
 * 
 *     s0 = -cos(x)/x
 */
    __pyx_r = (-NPY_INFINITY);
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":155
 *     if x == inf or x == -inf:
 *         return 0
 *     if x == 0:             # <<<<<<<<<<<<<<
 *         return -inf
 * 
 */
  }

  /* "_spherical_bessel.pxd":158
 *         return -inf
 * 
 *     s0 = -cos(x)/x             # <<<<<<<<<<<<<<
 *     if n == 0:
 *         return s0
 */
  __pyx_v_s0 = ((-cos(__pyx_v_x)) / __pyx_v_x);

  /* "_spherical_bessel.pxd":159
 * 
 *     s0 = -cos(x)/x
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return s0
 *     s1 = (s0 - sin(x))/x
 */
  __pyx_t_1 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":160
 *     s0 = -cos(x)/x
 *     if n == 0:
 *         return s0             # <<<<<<<<<<<<<<
 *     s1 = (s0 - sin(x))/x
 *     if n == 1:
 */
    __pyx_r = __pyx_v_s0;
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":159
 * 
 *     s0 = -cos(x)/x
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return s0
 *     s1 = (s0 - sin(x))/x
 */
  }

  /* "_spherical_bessel.pxd":161
 *     if n == 0:
 *         return s0
 *     s1 = (s0 - sin(x))/x             # <<<<<<<<<<<<<<
 *     if n == 1:
 *         return s1
 */
  __pyx_v_s1 = ((__pyx_v_s0 - sin(__pyx_v_x)) / __pyx_v_x);

  /* "_spherical_bessel.pxd":162
 *         return s0
 *     s1 = (s0 - sin(x))/x
 *     if n == 1:             # <<<<<<<<<<<<<<
 *         return s1
 * 
 */
  __pyx_t_1 = ((__pyx_v_n == 1) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":163
 *     s1 = (s0 - sin(x))/x
 *     if n == 1:
 *         return s1             # <<<<<<<<<<<<<<
 * 
 *     for idx in range(n - 1):
 */
    __pyx_r = __pyx_v_s1;
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":162
 *         return s0
 *     s1 = (s0 - sin(x))/x
 *     if n == 1:             # <<<<<<<<<<<<<<
 *         return s1
 * 
 */
  }

  /* "_spherical_bessel.pxd":165
 *         return s1
 * 
 *     for idx in range(n - 1):             # <<<<<<<<<<<<<<
 *         sn = (2*idx + 3)*s1/x - s0
 *         s0 = s1
 */
  __pyx_t_3 = (__pyx_v_n - 1);
  __pyx_t_4 = __pyx_t_3;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_idx = __pyx_t_5;

    /* "_spherical_bessel.pxd":166
 * 
 *     for idx in range(n - 1):
 *         sn = (2*idx + 3)*s1/x - s0             # <<<<<<<<<<<<<<
 *         s0 = s1
 *         s1 = sn
 */
    __pyx_v_sn = (((((2 * __pyx_v_idx) + 3) * __pyx_v_s1) / __pyx_v_x) - __pyx_v_s0);

    /* "_spherical_bessel.pxd":167
 *     for idx in range(n - 1):
 *         sn = (2*idx + 3)*s1/x - s0
 *         s0 = s1             # <<<<<<<<<<<<<<
 *         s1 = sn
 *         if npy_isinf(sn):
 */
    __pyx_v_s0 = __pyx_v_s1;

    /* "_spherical_bessel.pxd":168
 *         sn = (2*idx + 3)*s1/x - s0
 *         s0 = s1
 *         s1 = sn             # <<<<<<<<<<<<<<
 *         if npy_isinf(sn):
 *             # Overflow occurred already: terminate recurrence.
 */
    __pyx_v_s1 = __pyx_v_sn;

    /* "_spherical_bessel.pxd":169
 *         s0 = s1
 *         s1 = sn
 *         if npy_isinf(sn):             # <<<<<<<<<<<<<<
 *             # Overflow occurred already: terminate recurrence.
 *             return sn
 */
    __pyx_t_1 = (npy_isinf(__pyx_v_sn) != 0);
    if (__pyx_t_1) {

      /* "_spherical_bessel.pxd":171
 *         if npy_isinf(sn):
 *             # Overflow occurred already: terminate recurrence.
 *             return sn             # <<<<<<<<<<<<<<
 * 
 *     return sn
 */
      __pyx_r = __pyx_v_sn;
      goto __pyx_L0;

      /* "_spherical_bessel.pxd":169
 *         s0 = s1
 *         s1 = sn
 *         if npy_isinf(sn):             # <<<<<<<<<<<<<<
 *             # Overflow occurred already: terminate recurrence.
 *             return sn
 */
    }
  }

  /* "_spherical_bessel.pxd":173
 *             return sn
 * 
 *     return sn             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_sn;
  goto __pyx_L0;

  /* "_spherical_bessel.pxd":142
 * 
 * @cython.cdivision(True)
 * cdef inline double spherical_yn_real(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double s0, s1, sn
 *     cdef int idx
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spherical_bessel.pxd":177
 * 
 * @cython.cdivision(True)
 * cdef inline double complex spherical_yn_complex(long n, double complex z) nogil:             # <<<<<<<<<<<<<<
 * 
 *     if zisnan(z):
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_complex(long __pyx_v_n, __pyx_t_double_complex __pyx_v_z) {
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_spherical_bessel.pxd":179
 * cdef inline double complex spherical_yn_complex(long n, double complex z) nogil:
 * 
 *     if zisnan(z):             # <<<<<<<<<<<<<<
 *         return z
 *     if n < 0:
 */
  __pyx_t_1 = (__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisnan(__pyx_v_z) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":180
 * 
 *     if zisnan(z):
 *         return z             # <<<<<<<<<<<<<<
 *     if n < 0:
 *         sf_error.error("spherical_yn", sf_error.DOMAIN, NULL)
 */
    __pyx_r = __pyx_v_z;
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":179
 * cdef inline double complex spherical_yn_complex(long n, double complex z) nogil:
 * 
 *     if zisnan(z):             # <<<<<<<<<<<<<<
 *         return z
 *     if n < 0:
 */
  }

  /* "_spherical_bessel.pxd":181
 *     if zisnan(z):
 *         return z
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("spherical_yn", sf_error.DOMAIN, NULL)
 *         return nan
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":182
 *         return z
 *     if n < 0:
 *         sf_error.error("spherical_yn", sf_error.DOMAIN, NULL)             # <<<<<<<<<<<<<<
 *         return nan
 *     if z.real == 0 and z.imag == 0:
 */
    sf_error(((char *)"spherical_yn"), SF_ERROR_DOMAIN, NULL);

    /* "_spherical_bessel.pxd":183
 *     if n < 0:
 *         sf_error.error("spherical_yn", sf_error.DOMAIN, NULL)
 *         return nan             # <<<<<<<<<<<<<<
 *     if z.real == 0 and z.imag == 0:
 *         # https://dlmf.nist.gov/10.52.E2
 */
    __pyx_r = __pyx_t_double_complex_from_parts(NPY_NAN, 0);
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":181
 *     if zisnan(z):
 *         return z
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("spherical_yn", sf_error.DOMAIN, NULL)
 *         return nan
 */
  }

  /* "_spherical_bessel.pxd":184
 *         sf_error.error("spherical_yn", sf_error.DOMAIN, NULL)
 *         return nan
 *     if z.real == 0 and z.imag == 0:             # <<<<<<<<<<<<<<
 *         # https://dlmf.nist.gov/10.52.E2
 *         return nan
 */
  __pyx_t_2 = ((__Pyx_CREAL(__pyx_v_z) == 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_2 = ((__Pyx_CIMAG(__pyx_v_z) == 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":186
 *     if z.real == 0 and z.imag == 0:
 *         # https://dlmf.nist.gov/10.52.E2
 *         return nan             # <<<<<<<<<<<<<<
 *     if z.real == inf or z.real == -inf:
 *         # https://dlmf.nist.gov/10.52.E3
 */
    __pyx_r = __pyx_t_double_complex_from_parts(NPY_NAN, 0);
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":184
 *         sf_error.error("spherical_yn", sf_error.DOMAIN, NULL)
 *         return nan
 *     if z.real == 0 and z.imag == 0:             # <<<<<<<<<<<<<<
 *         # https://dlmf.nist.gov/10.52.E2
 *         return nan
 */
  }

  /* "_spherical_bessel.pxd":187
 *         # https://dlmf.nist.gov/10.52.E2
 *         return nan
 *     if z.real == inf or z.real == -inf:             # <<<<<<<<<<<<<<
 *         # https://dlmf.nist.gov/10.52.E3
 *         if z.imag == 0:
 */
  __pyx_t_2 = ((__Pyx_CREAL(__pyx_v_z) == NPY_INFINITY) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__Pyx_CREAL(__pyx_v_z) == (-NPY_INFINITY)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":189
 *     if z.real == inf or z.real == -inf:
 *         # https://dlmf.nist.gov/10.52.E3
 *         if z.imag == 0:             # <<<<<<<<<<<<<<
 *             return 0
 *         else:
 */
    __pyx_t_1 = ((__Pyx_CIMAG(__pyx_v_z) == 0.0) != 0);
    if (__pyx_t_1) {

      /* "_spherical_bessel.pxd":190
 *         # https://dlmf.nist.gov/10.52.E3
 *         if z.imag == 0:
 *             return 0             # <<<<<<<<<<<<<<
 *         else:
 *             return (1+1j)*inf
 */
      __pyx_r = __pyx_t_double_complex_from_parts(0, 0);
      goto __pyx_L0;

      /* "_spherical_bessel.pxd":189
 *     if z.real == inf or z.real == -inf:
 *         # https://dlmf.nist.gov/10.52.E3
 *         if z.imag == 0:             # <<<<<<<<<<<<<<
 *             return 0
 *         else:
 */
    }

    /* "_spherical_bessel.pxd":192
 *             return 0
 *         else:
 *             return (1+1j)*inf             # <<<<<<<<<<<<<<
 * 
 *     return zsqrt(M_PI_2/z)*cbesy(n + 0.5, z)
 */
    /*else*/ {
      __pyx_r = __Pyx_c_prod_double(__Pyx_c_sum_double(__pyx_t_double_complex_from_parts(1, 0), __pyx_t_double_complex_from_parts(0, 1.0)), __pyx_t_double_complex_from_parts(NPY_INFINITY, 0));
      goto __pyx_L0;
    }

    /* "_spherical_bessel.pxd":187
 *         # https://dlmf.nist.gov/10.52.E2
 *         return nan
 *     if z.real == inf or z.real == -inf:             # <<<<<<<<<<<<<<
 *         # https://dlmf.nist.gov/10.52.E3
 *         if z.imag == 0:
 */
  }

  /* "_spherical_bessel.pxd":194
 *             return (1+1j)*inf
 * 
 *     return zsqrt(M_PI_2/z)*cbesy(n + 0.5, z)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __Pyx_c_prod_double(__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zsqrt(__Pyx_c_quot_double(__pyx_t_double_complex_from_parts(M_PI_2, 0), __pyx_v_z)), __pyx_fuse_1__pyx_f_5scipy_7special_17_spherical_bessel_cbesy((__pyx_v_n + 0.5), __pyx_v_z));
  goto __pyx_L0;

  /* "_spherical_bessel.pxd":177
 * 
 * @cython.cdivision(True)
 * cdef inline double complex spherical_yn_complex(long n, double complex z) nogil:             # <<<<<<<<<<<<<<
 * 
 *     if zisnan(z):
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spherical_bessel.pxd":198
 * 
 * @cython.cdivision(True)
 * cdef inline double spherical_in_real(long n, double z) nogil:             # <<<<<<<<<<<<<<
 * 
 *     if npy_isnan(z):
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_real(long __pyx_v_n, double __pyx_v_z) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_spherical_bessel.pxd":200
 * cdef inline double spherical_in_real(long n, double z) nogil:
 * 
 *     if npy_isnan(z):             # <<<<<<<<<<<<<<
 *         return z
 *     if n < 0:
 */
  __pyx_t_1 = (npy_isnan(__pyx_v_z) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":201
 * 
 *     if npy_isnan(z):
 *         return z             # <<<<<<<<<<<<<<
 *     if n < 0:
 *         sf_error.error("spherical_in", sf_error.DOMAIN, NULL)
 */
    __pyx_r = __pyx_v_z;
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":200
 * cdef inline double spherical_in_real(long n, double z) nogil:
 * 
 *     if npy_isnan(z):             # <<<<<<<<<<<<<<
 *         return z
 *     if n < 0:
 */
  }

  /* "_spherical_bessel.pxd":202
 *     if npy_isnan(z):
 *         return z
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("spherical_in", sf_error.DOMAIN, NULL)
 *         return nan
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":203
 *         return z
 *     if n < 0:
 *         sf_error.error("spherical_in", sf_error.DOMAIN, NULL)             # <<<<<<<<<<<<<<
 *         return nan
 *     if z == 0:
 */
    sf_error(((char *)"spherical_in"), SF_ERROR_DOMAIN, NULL);

    /* "_spherical_bessel.pxd":204
 *     if n < 0:
 *         sf_error.error("spherical_in", sf_error.DOMAIN, NULL)
 *         return nan             # <<<<<<<<<<<<<<
 *     if z == 0:
 *         # https://dlmf.nist.gov/10.52.E1
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":202
 *     if npy_isnan(z):
 *         return z
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("spherical_in", sf_error.DOMAIN, NULL)
 *         return nan
 */
  }

  /* "_spherical_bessel.pxd":205
 *         sf_error.error("spherical_in", sf_error.DOMAIN, NULL)
 *         return nan
 *     if z == 0:             # <<<<<<<<<<<<<<
 *         # https://dlmf.nist.gov/10.52.E1
 *         if n == 0:
 */
  __pyx_t_1 = ((__pyx_v_z == 0.0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":207
 *     if z == 0:
 *         # https://dlmf.nist.gov/10.52.E1
 *         if n == 0:             # <<<<<<<<<<<<<<
 *             return 1
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_n == 0) != 0);
    if (__pyx_t_1) {

      /* "_spherical_bessel.pxd":208
 *         # https://dlmf.nist.gov/10.52.E1
 *         if n == 0:
 *             return 1             # <<<<<<<<<<<<<<
 *         else:
 *             return 0
 */
      __pyx_r = 1.0;
      goto __pyx_L0;

      /* "_spherical_bessel.pxd":207
 *     if z == 0:
 *         # https://dlmf.nist.gov/10.52.E1
 *         if n == 0:             # <<<<<<<<<<<<<<
 *             return 1
 *         else:
 */
    }

    /* "_spherical_bessel.pxd":210
 *             return 1
 *         else:
 *             return 0             # <<<<<<<<<<<<<<
 *     if npy_isinf(z):
 *         # https://dlmf.nist.gov/10.49.E8
 */
    /*else*/ {
      __pyx_r = 0.0;
      goto __pyx_L0;
    }

    /* "_spherical_bessel.pxd":205
 *         sf_error.error("spherical_in", sf_error.DOMAIN, NULL)
 *         return nan
 *     if z == 0:             # <<<<<<<<<<<<<<
 *         # https://dlmf.nist.gov/10.52.E1
 *         if n == 0:
 */
  }

  /* "_spherical_bessel.pxd":211
 *         else:
 *             return 0
 *     if npy_isinf(z):             # <<<<<<<<<<<<<<
 *         # https://dlmf.nist.gov/10.49.E8
 *         if z == -inf:
 */
  __pyx_t_1 = (npy_isinf(__pyx_v_z) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":213
 *     if npy_isinf(z):
 *         # https://dlmf.nist.gov/10.49.E8
 *         if z == -inf:             # <<<<<<<<<<<<<<
 *             return (-1)**n*inf
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_z == (-NPY_INFINITY)) != 0);
    if (__pyx_t_1) {

      /* "_spherical_bessel.pxd":214
 *         # https://dlmf.nist.gov/10.49.E8
 *         if z == -inf:
 *             return (-1)**n*inf             # <<<<<<<<<<<<<<
 *         else:
 *             return inf
 */
      __pyx_r = (__Pyx_pow_long(-1L, __pyx_v_n) * NPY_INFINITY);
      goto __pyx_L0;

      /* "_spherical_bessel.pxd":213
 *     if npy_isinf(z):
 *         # https://dlmf.nist.gov/10.49.E8
 *         if z == -inf:             # <<<<<<<<<<<<<<
 *             return (-1)**n*inf
 *         else:
 */
    }

    /* "_spherical_bessel.pxd":216
 *             return (-1)**n*inf
 *         else:
 *             return inf             # <<<<<<<<<<<<<<
 * 
 *     return sqrt(M_PI_2/z)*iv(n + 0.5, z)
 */
    /*else*/ {
      __pyx_r = NPY_INFINITY;
      goto __pyx_L0;
    }

    /* "_spherical_bessel.pxd":211
 *         else:
 *             return 0
 *     if npy_isinf(z):             # <<<<<<<<<<<<<<
 *         # https://dlmf.nist.gov/10.49.E8
 *         if z == -inf:
 */
  }

  /* "_spherical_bessel.pxd":218
 *             return inf
 * 
 *     return sqrt(M_PI_2/z)*iv(n + 0.5, z)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = (sqrt((((double)M_PI_2) / __pyx_v_z)) * iv((__pyx_v_n + 0.5), __pyx_v_z));
  goto __pyx_L0;

  /* "_spherical_bessel.pxd":198
 * 
 * @cython.cdivision(True)
 * cdef inline double spherical_in_real(long n, double z) nogil:             # <<<<<<<<<<<<<<
 * 
 *     if npy_isnan(z):
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spherical_bessel.pxd":222
 * 
 * @cython.cdivision(True)
 * cdef inline double complex spherical_in_complex(long n, double complex z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble s
 * 
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_complex(long __pyx_v_n, __pyx_t_double_complex __pyx_v_z) {
  npy_cdouble __pyx_v_s;
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;

  /* "_spherical_bessel.pxd":225
 *     cdef npy_cdouble s
 * 
 *     if zisnan(z):             # <<<<<<<<<<<<<<
 *         return z
 *     if n < 0:
 */
  __pyx_t_1 = (__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisnan(__pyx_v_z) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":226
 * 
 *     if zisnan(z):
 *         return z             # <<<<<<<<<<<<<<
 *     if n < 0:
 *         sf_error.error("spherical_in", sf_error.DOMAIN, NULL)
 */
    __pyx_r = __pyx_v_z;
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":225
 *     cdef npy_cdouble s
 * 
 *     if zisnan(z):             # <<<<<<<<<<<<<<
 *         return z
 *     if n < 0:
 */
  }

  /* "_spherical_bessel.pxd":227
 *     if zisnan(z):
 *         return z
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("spherical_in", sf_error.DOMAIN, NULL)
 *         return nan
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":228
 *         return z
 *     if n < 0:
 *         sf_error.error("spherical_in", sf_error.DOMAIN, NULL)             # <<<<<<<<<<<<<<
 *         return nan
 *     if zabs(z) == 0:
 */
    sf_error(((char *)"spherical_in"), SF_ERROR_DOMAIN, NULL);

    /* "_spherical_bessel.pxd":229
 *     if n < 0:
 *         sf_error.error("spherical_in", sf_error.DOMAIN, NULL)
 *         return nan             # <<<<<<<<<<<<<<
 *     if zabs(z) == 0:
 *         # https://dlmf.nist.gov/10.52.E1
 */
    __pyx_r = __pyx_t_double_complex_from_parts(NPY_NAN, 0);
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":227
 *     if zisnan(z):
 *         return z
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("spherical_in", sf_error.DOMAIN, NULL)
 *         return nan
 */
  }

  /* "_spherical_bessel.pxd":230
 *         sf_error.error("spherical_in", sf_error.DOMAIN, NULL)
 *         return nan
 *     if zabs(z) == 0:             # <<<<<<<<<<<<<<
 *         # https://dlmf.nist.gov/10.52.E1
 *         if n == 0:
 */
  __pyx_t_1 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_z) == 0.0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":232
 *     if zabs(z) == 0:
 *         # https://dlmf.nist.gov/10.52.E1
 *         if n == 0:             # <<<<<<<<<<<<<<
 *             return 1
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_n == 0) != 0);
    if (__pyx_t_1) {

      /* "_spherical_bessel.pxd":233
 *         # https://dlmf.nist.gov/10.52.E1
 *         if n == 0:
 *             return 1             # <<<<<<<<<<<<<<
 *         else:
 *             return 0
 */
      __pyx_r = __pyx_t_double_complex_from_parts(1, 0);
      goto __pyx_L0;

      /* "_spherical_bessel.pxd":232
 *     if zabs(z) == 0:
 *         # https://dlmf.nist.gov/10.52.E1
 *         if n == 0:             # <<<<<<<<<<<<<<
 *             return 1
 *         else:
 */
    }

    /* "_spherical_bessel.pxd":235
 *             return 1
 *         else:
 *             return 0             # <<<<<<<<<<<<<<
 *     if zisinf(z):
 *         # https://dlmf.nist.gov/10.52.E5
 */
    /*else*/ {
      __pyx_r = __pyx_t_double_complex_from_parts(0, 0);
      goto __pyx_L0;
    }

    /* "_spherical_bessel.pxd":230
 *         sf_error.error("spherical_in", sf_error.DOMAIN, NULL)
 *         return nan
 *     if zabs(z) == 0:             # <<<<<<<<<<<<<<
 *         # https://dlmf.nist.gov/10.52.E1
 *         if n == 0:
 */
  }

  /* "_spherical_bessel.pxd":236
 *         else:
 *             return 0
 *     if zisinf(z):             # <<<<<<<<<<<<<<
 *         # https://dlmf.nist.gov/10.52.E5
 *         if z.imag == 0:
 */
  __pyx_t_1 = (__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisinf(__pyx_v_z) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":238
 *     if zisinf(z):
 *         # https://dlmf.nist.gov/10.52.E5
 *         if z.imag == 0:             # <<<<<<<<<<<<<<
 *             if z.real == -inf:
 *                 return (-1)**n*inf
 */
    __pyx_t_1 = ((__Pyx_CIMAG(__pyx_v_z) == 0.0) != 0);
    if (__pyx_t_1) {

      /* "_spherical_bessel.pxd":239
 *         # https://dlmf.nist.gov/10.52.E5
 *         if z.imag == 0:
 *             if z.real == -inf:             # <<<<<<<<<<<<<<
 *                 return (-1)**n*inf
 *             else:
 */
      __pyx_t_1 = ((__Pyx_CREAL(__pyx_v_z) == (-NPY_INFINITY)) != 0);
      if (__pyx_t_1) {

        /* "_spherical_bessel.pxd":240
 *         if z.imag == 0:
 *             if z.real == -inf:
 *                 return (-1)**n*inf             # <<<<<<<<<<<<<<
 *             else:
 *                 return inf
 */
        __pyx_r = __pyx_t_double_complex_from_parts((__Pyx_pow_long(-1L, __pyx_v_n) * NPY_INFINITY), 0);
        goto __pyx_L0;

        /* "_spherical_bessel.pxd":239
 *         # https://dlmf.nist.gov/10.52.E5
 *         if z.imag == 0:
 *             if z.real == -inf:             # <<<<<<<<<<<<<<
 *                 return (-1)**n*inf
 *             else:
 */
      }

      /* "_spherical_bessel.pxd":242
 *                 return (-1)**n*inf
 *             else:
 *                 return inf             # <<<<<<<<<<<<<<
 *         else:
 *             return nan
 */
      /*else*/ {
        __pyx_r = __pyx_t_double_complex_from_parts(NPY_INFINITY, 0);
        goto __pyx_L0;
      }

      /* "_spherical_bessel.pxd":238
 *     if zisinf(z):
 *         # https://dlmf.nist.gov/10.52.E5
 *         if z.imag == 0:             # <<<<<<<<<<<<<<
 *             if z.real == -inf:
 *                 return (-1)**n*inf
 */
    }

    /* "_spherical_bessel.pxd":244
 *                 return inf
 *         else:
 *             return nan             # <<<<<<<<<<<<<<
 * 
 *     s = cbesi_wrap(n + 0.5, npy_cdouble_from_double_complex(z))
 */
    /*else*/ {
      __pyx_r = __pyx_t_double_complex_from_parts(NPY_NAN, 0);
      goto __pyx_L0;
    }

    /* "_spherical_bessel.pxd":236
 *         else:
 *             return 0
 *     if zisinf(z):             # <<<<<<<<<<<<<<
 *         # https://dlmf.nist.gov/10.52.E5
 *         if z.imag == 0:
 */
  }

  /* "_spherical_bessel.pxd":246
 *             return nan
 * 
 *     s = cbesi_wrap(n + 0.5, npy_cdouble_from_double_complex(z))             # <<<<<<<<<<<<<<
 *     return zsqrt(M_PI_2/z)*double_complex_from_npy_cdouble(s)
 * 
 */
  __pyx_v_s = cbesi_wrap((__pyx_v_n + 0.5), __pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_z));

  /* "_spherical_bessel.pxd":247
 * 
 *     s = cbesi_wrap(n + 0.5, npy_cdouble_from_double_complex(z))
 *     return zsqrt(M_PI_2/z)*double_complex_from_npy_cdouble(s)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __Pyx_c_prod_double(__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zsqrt(__Pyx_c_quot_double(__pyx_t_double_complex_from_parts(M_PI_2, 0), __pyx_v_z)), __pyx_f_5scipy_7special_13_complexstuff_double_complex_from_npy_cdouble(__pyx_v_s));
  goto __pyx_L0;

  /* "_spherical_bessel.pxd":222
 * 
 * @cython.cdivision(True)
 * cdef inline double complex spherical_in_complex(long n, double complex z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble s
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spherical_bessel.pxd":251
 * 
 * @cython.cdivision(True)
 * cdef inline double spherical_kn_real(long n, double z) nogil:             # <<<<<<<<<<<<<<
 * 
 *     if npy_isnan(z):
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_real(long __pyx_v_n, double __pyx_v_z) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_spherical_bessel.pxd":253
 * cdef inline double spherical_kn_real(long n, double z) nogil:
 * 
 *     if npy_isnan(z):             # <<<<<<<<<<<<<<
 *         return z
 *     if n < 0:
 */
  __pyx_t_1 = (npy_isnan(__pyx_v_z) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":254
 * 
 *     if npy_isnan(z):
 *         return z             # <<<<<<<<<<<<<<
 *     if n < 0:
 *         sf_error.error("spherical_kn", sf_error.DOMAIN, NULL)
 */
    __pyx_r = __pyx_v_z;
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":253
 * cdef inline double spherical_kn_real(long n, double z) nogil:
 * 
 *     if npy_isnan(z):             # <<<<<<<<<<<<<<
 *         return z
 *     if n < 0:
 */
  }

  /* "_spherical_bessel.pxd":255
 *     if npy_isnan(z):
 *         return z
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("spherical_kn", sf_error.DOMAIN, NULL)
 *         return nan
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":256
 *         return z
 *     if n < 0:
 *         sf_error.error("spherical_kn", sf_error.DOMAIN, NULL)             # <<<<<<<<<<<<<<
 *         return nan
 *     if z == 0:
 */
    sf_error(((char *)"spherical_kn"), SF_ERROR_DOMAIN, NULL);

    /* "_spherical_bessel.pxd":257
 *     if n < 0:
 *         sf_error.error("spherical_kn", sf_error.DOMAIN, NULL)
 *         return nan             # <<<<<<<<<<<<<<
 *     if z == 0:
 *         return inf
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":255
 *     if npy_isnan(z):
 *         return z
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("spherical_kn", sf_error.DOMAIN, NULL)
 *         return nan
 */
  }

  /* "_spherical_bessel.pxd":258
 *         sf_error.error("spherical_kn", sf_error.DOMAIN, NULL)
 *         return nan
 *     if z == 0:             # <<<<<<<<<<<<<<
 *         return inf
 *     if npy_isinf(z):
 */
  __pyx_t_1 = ((__pyx_v_z == 0.0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":259
 *         return nan
 *     if z == 0:
 *         return inf             # <<<<<<<<<<<<<<
 *     if npy_isinf(z):
 *         # https://dlmf.nist.gov/10.52.E6
 */
    __pyx_r = NPY_INFINITY;
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":258
 *         sf_error.error("spherical_kn", sf_error.DOMAIN, NULL)
 *         return nan
 *     if z == 0:             # <<<<<<<<<<<<<<
 *         return inf
 *     if npy_isinf(z):
 */
  }

  /* "_spherical_bessel.pxd":260
 *     if z == 0:
 *         return inf
 *     if npy_isinf(z):             # <<<<<<<<<<<<<<
 *         # https://dlmf.nist.gov/10.52.E6
 *         if z == inf:
 */
  __pyx_t_1 = (npy_isinf(__pyx_v_z) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":262
 *     if npy_isinf(z):
 *         # https://dlmf.nist.gov/10.52.E6
 *         if z == inf:             # <<<<<<<<<<<<<<
 *             return 0
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_z == NPY_INFINITY) != 0);
    if (__pyx_t_1) {

      /* "_spherical_bessel.pxd":263
 *         # https://dlmf.nist.gov/10.52.E6
 *         if z == inf:
 *             return 0             # <<<<<<<<<<<<<<
 *         else:
 *             return -inf
 */
      __pyx_r = 0.0;
      goto __pyx_L0;

      /* "_spherical_bessel.pxd":262
 *     if npy_isinf(z):
 *         # https://dlmf.nist.gov/10.52.E6
 *         if z == inf:             # <<<<<<<<<<<<<<
 *             return 0
 *         else:
 */
    }

    /* "_spherical_bessel.pxd":265
 *             return 0
 *         else:
 *             return -inf             # <<<<<<<<<<<<<<
 * 
 *     return sqrt(M_PI_2/z)*cbesk(n + 0.5, z)
 */
    /*else*/ {
      __pyx_r = (-NPY_INFINITY);
      goto __pyx_L0;
    }

    /* "_spherical_bessel.pxd":260
 *     if z == 0:
 *         return inf
 *     if npy_isinf(z):             # <<<<<<<<<<<<<<
 *         # https://dlmf.nist.gov/10.52.E6
 *         if z == inf:
 */
  }

  /* "_spherical_bessel.pxd":267
 *             return -inf
 * 
 *     return sqrt(M_PI_2/z)*cbesk(n + 0.5, z)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = (sqrt((((double)M_PI_2) / __pyx_v_z)) * __pyx_fuse_0__pyx_f_5scipy_7special_17_spherical_bessel_cbesk((__pyx_v_n + 0.5), __pyx_v_z));
  goto __pyx_L0;

  /* "_spherical_bessel.pxd":251
 * 
 * @cython.cdivision(True)
 * cdef inline double spherical_kn_real(long n, double z) nogil:             # <<<<<<<<<<<<<<
 * 
 *     if npy_isnan(z):
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spherical_bessel.pxd":271
 * 
 * @cython.cdivision(True)
 * cdef inline double complex spherical_kn_complex(long n, double complex z) nogil:             # <<<<<<<<<<<<<<
 * 
 *     if zisnan(z):
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_complex(long __pyx_v_n, __pyx_t_double_complex __pyx_v_z) {
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;

  /* "_spherical_bessel.pxd":273
 * cdef inline double complex spherical_kn_complex(long n, double complex z) nogil:
 * 
 *     if zisnan(z):             # <<<<<<<<<<<<<<
 *         return z
 *     if n < 0:
 */
  __pyx_t_1 = (__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisnan(__pyx_v_z) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":274
 * 
 *     if zisnan(z):
 *         return z             # <<<<<<<<<<<<<<
 *     if n < 0:
 *         sf_error.error("spherical_kn", sf_error.DOMAIN, NULL)
 */
    __pyx_r = __pyx_v_z;
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":273
 * cdef inline double complex spherical_kn_complex(long n, double complex z) nogil:
 * 
 *     if zisnan(z):             # <<<<<<<<<<<<<<
 *         return z
 *     if n < 0:
 */
  }

  /* "_spherical_bessel.pxd":275
 *     if zisnan(z):
 *         return z
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("spherical_kn", sf_error.DOMAIN, NULL)
 *         return nan
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":276
 *         return z
 *     if n < 0:
 *         sf_error.error("spherical_kn", sf_error.DOMAIN, NULL)             # <<<<<<<<<<<<<<
 *         return nan
 *     if zabs(z) == 0:
 */
    sf_error(((char *)"spherical_kn"), SF_ERROR_DOMAIN, NULL);

    /* "_spherical_bessel.pxd":277
 *     if n < 0:
 *         sf_error.error("spherical_kn", sf_error.DOMAIN, NULL)
 *         return nan             # <<<<<<<<<<<<<<
 *     if zabs(z) == 0:
 *         return nan
 */
    __pyx_r = __pyx_t_double_complex_from_parts(NPY_NAN, 0);
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":275
 *     if zisnan(z):
 *         return z
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         sf_error.error("spherical_kn", sf_error.DOMAIN, NULL)
 *         return nan
 */
  }

  /* "_spherical_bessel.pxd":278
 *         sf_error.error("spherical_kn", sf_error.DOMAIN, NULL)
 *         return nan
 *     if zabs(z) == 0:             # <<<<<<<<<<<<<<
 *         return nan
 *     if zisinf(z):
 */
  __pyx_t_1 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_z) == 0.0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":279
 *         return nan
 *     if zabs(z) == 0:
 *         return nan             # <<<<<<<<<<<<<<
 *     if zisinf(z):
 *         # https://dlmf.nist.gov/10.52.E6
 */
    __pyx_r = __pyx_t_double_complex_from_parts(NPY_NAN, 0);
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":278
 *         sf_error.error("spherical_kn", sf_error.DOMAIN, NULL)
 *         return nan
 *     if zabs(z) == 0:             # <<<<<<<<<<<<<<
 *         return nan
 *     if zisinf(z):
 */
  }

  /* "_spherical_bessel.pxd":280
 *     if zabs(z) == 0:
 *         return nan
 *     if zisinf(z):             # <<<<<<<<<<<<<<
 *         # https://dlmf.nist.gov/10.52.E6
 *         if z.imag == 0:
 */
  __pyx_t_1 = (__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisinf(__pyx_v_z) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":282
 *     if zisinf(z):
 *         # https://dlmf.nist.gov/10.52.E6
 *         if z.imag == 0:             # <<<<<<<<<<<<<<
 *             if z.real == inf:
 *                 return 0
 */
    __pyx_t_1 = ((__Pyx_CIMAG(__pyx_v_z) == 0.0) != 0);
    if (__pyx_t_1) {

      /* "_spherical_bessel.pxd":283
 *         # https://dlmf.nist.gov/10.52.E6
 *         if z.imag == 0:
 *             if z.real == inf:             # <<<<<<<<<<<<<<
 *                 return 0
 *             else:
 */
      __pyx_t_1 = ((__Pyx_CREAL(__pyx_v_z) == NPY_INFINITY) != 0);
      if (__pyx_t_1) {

        /* "_spherical_bessel.pxd":284
 *         if z.imag == 0:
 *             if z.real == inf:
 *                 return 0             # <<<<<<<<<<<<<<
 *             else:
 *                 return -inf
 */
        __pyx_r = __pyx_t_double_complex_from_parts(0, 0);
        goto __pyx_L0;

        /* "_spherical_bessel.pxd":283
 *         # https://dlmf.nist.gov/10.52.E6
 *         if z.imag == 0:
 *             if z.real == inf:             # <<<<<<<<<<<<<<
 *                 return 0
 *             else:
 */
      }

      /* "_spherical_bessel.pxd":286
 *                 return 0
 *             else:
 *                 return -inf             # <<<<<<<<<<<<<<
 *         else:
 *             return nan
 */
      /*else*/ {
        __pyx_r = __pyx_t_double_complex_from_parts((-NPY_INFINITY), 0);
        goto __pyx_L0;
      }

      /* "_spherical_bessel.pxd":282
 *     if zisinf(z):
 *         # https://dlmf.nist.gov/10.52.E6
 *         if z.imag == 0:             # <<<<<<<<<<<<<<
 *             if z.real == inf:
 *                 return 0
 */
    }

    /* "_spherical_bessel.pxd":288
 *                 return -inf
 *         else:
 *             return nan             # <<<<<<<<<<<<<<
 * 
 *     return zsqrt(M_PI_2/z)*cbesk(n + 0.5, z)
 */
    /*else*/ {
      __pyx_r = __pyx_t_double_complex_from_parts(NPY_NAN, 0);
      goto __pyx_L0;
    }

    /* "_spherical_bessel.pxd":280
 *     if zabs(z) == 0:
 *         return nan
 *     if zisinf(z):             # <<<<<<<<<<<<<<
 *         # https://dlmf.nist.gov/10.52.E6
 *         if z.imag == 0:
 */
  }

  /* "_spherical_bessel.pxd":290
 *             return nan
 * 
 *     return zsqrt(M_PI_2/z)*cbesk(n + 0.5, z)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __Pyx_c_prod_double(__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zsqrt(__Pyx_c_quot_double(__pyx_t_double_complex_from_parts(M_PI_2, 0), __pyx_v_z)), __pyx_fuse_1__pyx_f_5scipy_7special_17_spherical_bessel_cbesk((__pyx_v_n + 0.5), __pyx_v_z));
  goto __pyx_L0;

  /* "_spherical_bessel.pxd":271
 * 
 * @cython.cdivision(True)
 * cdef inline double complex spherical_kn_complex(long n, double complex z) nogil:             # <<<<<<<<<<<<<<
 * 
 *     if zisnan(z):
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spherical_bessel.pxd":296
 * 
 * @cython.cdivision(True)
 * cdef inline double spherical_jn_d_real(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     if n == 0:
 *         return -spherical_jn_real(1, x)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_d_real(long __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_spherical_bessel.pxd":297
 * @cython.cdivision(True)
 * cdef inline double spherical_jn_d_real(long n, double x) nogil:
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return -spherical_jn_real(1, x)
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":298
 * cdef inline double spherical_jn_d_real(long n, double x) nogil:
 *     if n == 0:
 *         return -spherical_jn_real(1, x)             # <<<<<<<<<<<<<<
 *     else:
 *         if x == 0:
 */
    __pyx_r = (-__pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_real(1, __pyx_v_x));
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":297
 * @cython.cdivision(True)
 * cdef inline double spherical_jn_d_real(long n, double x) nogil:
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return -spherical_jn_real(1, x)
 *     else:
 */
  }

  /* "_spherical_bessel.pxd":300
 *         return -spherical_jn_real(1, x)
 *     else:
 *         if x == 0:             # <<<<<<<<<<<<<<
 *             # DLMF 10.51.2 doesn't work, so use 10.51.1 to get the
 *             # exact value
 */
  /*else*/ {
    __pyx_t_1 = ((__pyx_v_x == 0.0) != 0);
    if (__pyx_t_1) {

      /* "_spherical_bessel.pxd":303
 *             # DLMF 10.51.2 doesn't work, so use 10.51.1 to get the
 *             # exact value
 *             if n == 1:             # <<<<<<<<<<<<<<
 *                 return 1.0/3
 *             else:
 */
      __pyx_t_1 = ((__pyx_v_n == 1) != 0);
      if (__pyx_t_1) {

        /* "_spherical_bessel.pxd":304
 *             # exact value
 *             if n == 1:
 *                 return 1.0/3             # <<<<<<<<<<<<<<
 *             else:
 *                 return 0
 */
        __pyx_r = (1.0 / 3.0);
        goto __pyx_L0;

        /* "_spherical_bessel.pxd":303
 *             # DLMF 10.51.2 doesn't work, so use 10.51.1 to get the
 *             # exact value
 *             if n == 1:             # <<<<<<<<<<<<<<
 *                 return 1.0/3
 *             else:
 */
      }

      /* "_spherical_bessel.pxd":306
 *                 return 1.0/3
 *             else:
 *                 return 0             # <<<<<<<<<<<<<<
 *         # DLMF 10.51.2
 *         return (spherical_jn_real(n - 1, x) -
 */
      /*else*/ {
        __pyx_r = 0.0;
        goto __pyx_L0;
      }

      /* "_spherical_bessel.pxd":300
 *         return -spherical_jn_real(1, x)
 *     else:
 *         if x == 0:             # <<<<<<<<<<<<<<
 *             # DLMF 10.51.2 doesn't work, so use 10.51.1 to get the
 *             # exact value
 */
    }

    /* "_spherical_bessel.pxd":308
 *                 return 0
 *         # DLMF 10.51.2
 *         return (spherical_jn_real(n - 1, x) -             # <<<<<<<<<<<<<<
 *                 (n + 1)*spherical_jn_real(n, x)/x)
 * 
 */
    __pyx_r = (__pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_real((__pyx_v_n - 1), __pyx_v_x) - (((__pyx_v_n + 1) * __pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_real(__pyx_v_n, __pyx_v_x)) / __pyx_v_x));
    goto __pyx_L0;
  }

  /* "_spherical_bessel.pxd":296
 * 
 * @cython.cdivision(True)
 * cdef inline double spherical_jn_d_real(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     if n == 0:
 *         return -spherical_jn_real(1, x)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spherical_bessel.pxd":313
 * 
 * @cython.cdivision(True)
 * cdef inline double complex spherical_jn_d_complex(long n, double complex x) nogil:             # <<<<<<<<<<<<<<
 *     if n == 0:
 *         return -spherical_jn_complex(1, x)
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_d_complex(long __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;

  /* "_spherical_bessel.pxd":314
 * @cython.cdivision(True)
 * cdef inline double complex spherical_jn_d_complex(long n, double complex x) nogil:
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return -spherical_jn_complex(1, x)
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":315
 * cdef inline double complex spherical_jn_d_complex(long n, double complex x) nogil:
 *     if n == 0:
 *         return -spherical_jn_complex(1, x)             # <<<<<<<<<<<<<<
 *     else:
 *         return (spherical_jn_complex(n - 1, x) -
 */
    __pyx_r = __Pyx_c_neg_double(__pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_complex(1, __pyx_v_x));
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":314
 * @cython.cdivision(True)
 * cdef inline double complex spherical_jn_d_complex(long n, double complex x) nogil:
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return -spherical_jn_complex(1, x)
 *     else:
 */
  }

  /* "_spherical_bessel.pxd":317
 *         return -spherical_jn_complex(1, x)
 *     else:
 *         return (spherical_jn_complex(n - 1, x) -             # <<<<<<<<<<<<<<
 *                 (n + 1)*spherical_jn_complex(n, x)/x)
 * 
 */
  /*else*/ {

    /* "_spherical_bessel.pxd":318
 *     else:
 *         return (spherical_jn_complex(n - 1, x) -
 *                 (n + 1)*spherical_jn_complex(n, x)/x)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_r = __Pyx_c_diff_double(__pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_complex((__pyx_v_n - 1), __pyx_v_x), __Pyx_c_quot_double(__Pyx_c_prod_double(__pyx_t_double_complex_from_parts((__pyx_v_n + 1), 0), __pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_complex(__pyx_v_n, __pyx_v_x)), __pyx_v_x));
    goto __pyx_L0;
  }

  /* "_spherical_bessel.pxd":313
 * 
 * @cython.cdivision(True)
 * cdef inline double complex spherical_jn_d_complex(long n, double complex x) nogil:             # <<<<<<<<<<<<<<
 *     if n == 0:
 *         return -spherical_jn_complex(1, x)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spherical_bessel.pxd":322
 * 
 * @cython.cdivision(True)
 * cdef inline double spherical_yn_d_real(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     if n == 0:
 *         return -spherical_yn_real(1, x)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_d_real(long __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_spherical_bessel.pxd":323
 * @cython.cdivision(True)
 * cdef inline double spherical_yn_d_real(long n, double x) nogil:
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return -spherical_yn_real(1, x)
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":324
 * cdef inline double spherical_yn_d_real(long n, double x) nogil:
 *     if n == 0:
 *         return -spherical_yn_real(1, x)             # <<<<<<<<<<<<<<
 *     else:
 *         return (spherical_yn_real(n - 1, x) -
 */
    __pyx_r = (-__pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_real(1, __pyx_v_x));
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":323
 * @cython.cdivision(True)
 * cdef inline double spherical_yn_d_real(long n, double x) nogil:
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return -spherical_yn_real(1, x)
 *     else:
 */
  }

  /* "_spherical_bessel.pxd":326
 *         return -spherical_yn_real(1, x)
 *     else:
 *         return (spherical_yn_real(n - 1, x) -             # <<<<<<<<<<<<<<
 *                 (n + 1)*spherical_yn_real(n, x)/x)
 * 
 */
  /*else*/ {

    /* "_spherical_bessel.pxd":327
 *     else:
 *         return (spherical_yn_real(n - 1, x) -
 *                 (n + 1)*spherical_yn_real(n, x)/x)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_r = (__pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_real((__pyx_v_n - 1), __pyx_v_x) - (((__pyx_v_n + 1) * __pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_real(__pyx_v_n, __pyx_v_x)) / __pyx_v_x));
    goto __pyx_L0;
  }

  /* "_spherical_bessel.pxd":322
 * 
 * @cython.cdivision(True)
 * cdef inline double spherical_yn_d_real(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     if n == 0:
 *         return -spherical_yn_real(1, x)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spherical_bessel.pxd":331
 * 
 * @cython.cdivision(True)
 * cdef inline double complex spherical_yn_d_complex(long n, double complex x) nogil:             # <<<<<<<<<<<<<<
 *     if n == 0:
 *         return -spherical_yn_complex(1, x)
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_d_complex(long __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;

  /* "_spherical_bessel.pxd":332
 * @cython.cdivision(True)
 * cdef inline double complex spherical_yn_d_complex(long n, double complex x) nogil:
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return -spherical_yn_complex(1, x)
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":333
 * cdef inline double complex spherical_yn_d_complex(long n, double complex x) nogil:
 *     if n == 0:
 *         return -spherical_yn_complex(1, x)             # <<<<<<<<<<<<<<
 *     else:
 *         return (spherical_yn_complex(n - 1, x) -
 */
    __pyx_r = __Pyx_c_neg_double(__pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_complex(1, __pyx_v_x));
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":332
 * @cython.cdivision(True)
 * cdef inline double complex spherical_yn_d_complex(long n, double complex x) nogil:
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return -spherical_yn_complex(1, x)
 *     else:
 */
  }

  /* "_spherical_bessel.pxd":335
 *         return -spherical_yn_complex(1, x)
 *     else:
 *         return (spherical_yn_complex(n - 1, x) -             # <<<<<<<<<<<<<<
 *                 (n + 1)*spherical_yn_complex(n, x)/x)
 * 
 */
  /*else*/ {

    /* "_spherical_bessel.pxd":336
 *     else:
 *         return (spherical_yn_complex(n - 1, x) -
 *                 (n + 1)*spherical_yn_complex(n, x)/x)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_r = __Pyx_c_diff_double(__pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_complex((__pyx_v_n - 1), __pyx_v_x), __Pyx_c_quot_double(__Pyx_c_prod_double(__pyx_t_double_complex_from_parts((__pyx_v_n + 1), 0), __pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_complex(__pyx_v_n, __pyx_v_x)), __pyx_v_x));
    goto __pyx_L0;
  }

  /* "_spherical_bessel.pxd":331
 * 
 * @cython.cdivision(True)
 * cdef inline double complex spherical_yn_d_complex(long n, double complex x) nogil:             # <<<<<<<<<<<<<<
 *     if n == 0:
 *         return -spherical_yn_complex(1, x)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spherical_bessel.pxd":340
 * 
 * @cython.cdivision(True)
 * cdef inline double spherical_in_d_real(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     if n == 0:
 *         return spherical_in_real(1, x)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_d_real(long __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_spherical_bessel.pxd":341
 * @cython.cdivision(True)
 * cdef inline double spherical_in_d_real(long n, double x) nogil:
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return spherical_in_real(1, x)
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":342
 * cdef inline double spherical_in_d_real(long n, double x) nogil:
 *     if n == 0:
 *         return spherical_in_real(1, x)             # <<<<<<<<<<<<<<
 *     else:
 *         if x == 0:
 */
    __pyx_r = __pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_real(1, __pyx_v_x);
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":341
 * @cython.cdivision(True)
 * cdef inline double spherical_in_d_real(long n, double x) nogil:
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return spherical_in_real(1, x)
 *     else:
 */
  }

  /* "_spherical_bessel.pxd":344
 *         return spherical_in_real(1, x)
 *     else:
 *         if x == 0:             # <<<<<<<<<<<<<<
 *             return 0
 *         return (spherical_in_real(n - 1, x) -
 */
  /*else*/ {
    __pyx_t_1 = ((__pyx_v_x == 0.0) != 0);
    if (__pyx_t_1) {

      /* "_spherical_bessel.pxd":345
 *     else:
 *         if x == 0:
 *             return 0             # <<<<<<<<<<<<<<
 *         return (spherical_in_real(n - 1, x) -
 *                 (n + 1)*spherical_in_real(n, x)/x)
 */
      __pyx_r = 0.0;
      goto __pyx_L0;

      /* "_spherical_bessel.pxd":344
 *         return spherical_in_real(1, x)
 *     else:
 *         if x == 0:             # <<<<<<<<<<<<<<
 *             return 0
 *         return (spherical_in_real(n - 1, x) -
 */
    }

    /* "_spherical_bessel.pxd":346
 *         if x == 0:
 *             return 0
 *         return (spherical_in_real(n - 1, x) -             # <<<<<<<<<<<<<<
 *                 (n + 1)*spherical_in_real(n, x)/x)
 * 
 */
    __pyx_r = (__pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_real((__pyx_v_n - 1), __pyx_v_x) - (((__pyx_v_n + 1) * __pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_real(__pyx_v_n, __pyx_v_x)) / __pyx_v_x));
    goto __pyx_L0;
  }

  /* "_spherical_bessel.pxd":340
 * 
 * @cython.cdivision(True)
 * cdef inline double spherical_in_d_real(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     if n == 0:
 *         return spherical_in_real(1, x)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spherical_bessel.pxd":351
 * 
 * @cython.cdivision(True)
 * cdef inline double complex spherical_in_d_complex(long n, double complex x) nogil:             # <<<<<<<<<<<<<<
 *     if n == 0:
 *         return spherical_in_complex(1, x)
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_d_complex(long __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;

  /* "_spherical_bessel.pxd":352
 * @cython.cdivision(True)
 * cdef inline double complex spherical_in_d_complex(long n, double complex x) nogil:
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return spherical_in_complex(1, x)
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":353
 * cdef inline double complex spherical_in_d_complex(long n, double complex x) nogil:
 *     if n == 0:
 *         return spherical_in_complex(1, x)             # <<<<<<<<<<<<<<
 *     else:
 *         if x == 0:
 */
    __pyx_r = __pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_complex(1, __pyx_v_x);
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":352
 * @cython.cdivision(True)
 * cdef inline double complex spherical_in_d_complex(long n, double complex x) nogil:
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return spherical_in_complex(1, x)
 *     else:
 */
  }

  /* "_spherical_bessel.pxd":355
 *         return spherical_in_complex(1, x)
 *     else:
 *         if x == 0:             # <<<<<<<<<<<<<<
 *             return 0
 *         return (spherical_in_complex(n - 1, x) -
 */
  /*else*/ {
    __pyx_t_1 = ((__Pyx_c_eq_double(__pyx_v_x, __pyx_t_double_complex_from_parts(0, 0))) != 0);
    if (__pyx_t_1) {

      /* "_spherical_bessel.pxd":356
 *     else:
 *         if x == 0:
 *             return 0             # <<<<<<<<<<<<<<
 *         return (spherical_in_complex(n - 1, x) -
 *                 (n + 1)*spherical_in_complex(n, x)/x)
 */
      __pyx_r = __pyx_t_double_complex_from_parts(0, 0);
      goto __pyx_L0;

      /* "_spherical_bessel.pxd":355
 *         return spherical_in_complex(1, x)
 *     else:
 *         if x == 0:             # <<<<<<<<<<<<<<
 *             return 0
 *         return (spherical_in_complex(n - 1, x) -
 */
    }

    /* "_spherical_bessel.pxd":357
 *         if x == 0:
 *             return 0
 *         return (spherical_in_complex(n - 1, x) -             # <<<<<<<<<<<<<<
 *                 (n + 1)*spherical_in_complex(n, x)/x)
 * 
 */
    __pyx_r = __Pyx_c_diff_double(__pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_complex((__pyx_v_n - 1), __pyx_v_x), __Pyx_c_quot_double(__Pyx_c_prod_double(__pyx_t_double_complex_from_parts((__pyx_v_n + 1), 0), __pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_complex(__pyx_v_n, __pyx_v_x)), __pyx_v_x));
    goto __pyx_L0;
  }

  /* "_spherical_bessel.pxd":351
 * 
 * @cython.cdivision(True)
 * cdef inline double complex spherical_in_d_complex(long n, double complex x) nogil:             # <<<<<<<<<<<<<<
 *     if n == 0:
 *         return spherical_in_complex(1, x)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spherical_bessel.pxd":362
 * 
 * @cython.cdivision(True)
 * cdef inline double spherical_kn_d_real(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     if n == 0:
 *         return -spherical_kn_real(1, x)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_d_real(long __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_spherical_bessel.pxd":363
 * @cython.cdivision(True)
 * cdef inline double spherical_kn_d_real(long n, double x) nogil:
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return -spherical_kn_real(1, x)
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":364
 * cdef inline double spherical_kn_d_real(long n, double x) nogil:
 *     if n == 0:
 *         return -spherical_kn_real(1, x)             # <<<<<<<<<<<<<<
 *     else:
 *         return (-spherical_kn_real(n - 1, x) -
 */
    __pyx_r = (-__pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_real(1, __pyx_v_x));
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":363
 * @cython.cdivision(True)
 * cdef inline double spherical_kn_d_real(long n, double x) nogil:
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return -spherical_kn_real(1, x)
 *     else:
 */
  }

  /* "_spherical_bessel.pxd":366
 *         return -spherical_kn_real(1, x)
 *     else:
 *         return (-spherical_kn_real(n - 1, x) -             # <<<<<<<<<<<<<<
 *                 (n + 1)*spherical_kn_real(n, x)/x)
 * 
 */
  /*else*/ {

    /* "_spherical_bessel.pxd":367
 *     else:
 *         return (-spherical_kn_real(n - 1, x) -
 *                 (n + 1)*spherical_kn_real(n, x)/x)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_r = ((-__pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_real((__pyx_v_n - 1), __pyx_v_x)) - (((__pyx_v_n + 1) * __pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_real(__pyx_v_n, __pyx_v_x)) / __pyx_v_x));
    goto __pyx_L0;
  }

  /* "_spherical_bessel.pxd":362
 * 
 * @cython.cdivision(True)
 * cdef inline double spherical_kn_d_real(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     if n == 0:
 *         return -spherical_kn_real(1, x)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spherical_bessel.pxd":371
 * 
 * @cython.cdivision(True)
 * cdef inline double complex spherical_kn_d_complex(long n, double complex x) nogil:             # <<<<<<<<<<<<<<
 *     if n == 0:
 *         return -spherical_kn_complex(1, x)
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_d_complex(long __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;

  /* "_spherical_bessel.pxd":372
 * @cython.cdivision(True)
 * cdef inline double complex spherical_kn_d_complex(long n, double complex x) nogil:
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return -spherical_kn_complex(1, x)
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_1) {

    /* "_spherical_bessel.pxd":373
 * cdef inline double complex spherical_kn_d_complex(long n, double complex x) nogil:
 *     if n == 0:
 *         return -spherical_kn_complex(1, x)             # <<<<<<<<<<<<<<
 *     else:
 *         return (-spherical_kn_complex(n - 1, x) -
 */
    __pyx_r = __Pyx_c_neg_double(__pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_complex(1, __pyx_v_x));
    goto __pyx_L0;

    /* "_spherical_bessel.pxd":372
 * @cython.cdivision(True)
 * cdef inline double complex spherical_kn_d_complex(long n, double complex x) nogil:
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return -spherical_kn_complex(1, x)
 *     else:
 */
  }

  /* "_spherical_bessel.pxd":375
 *         return -spherical_kn_complex(1, x)
 *     else:
 *         return (-spherical_kn_complex(n - 1, x) -             # <<<<<<<<<<<<<<
 *                 (n + 1)*spherical_kn_complex(n, x)/x)
 */
  /*else*/ {

    /* "_spherical_bessel.pxd":376
 *     else:
 *         return (-spherical_kn_complex(n - 1, x) -
 *                 (n + 1)*spherical_kn_complex(n, x)/x)             # <<<<<<<<<<<<<<
 */
    __pyx_r = __Pyx_c_diff_double(__Pyx_c_neg_double(__pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_complex((__pyx_v_n - 1), __pyx_v_x)), __Pyx_c_quot_double(__Pyx_c_prod_double(__pyx_t_double_complex_from_parts((__pyx_v_n + 1), 0), __pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_complex(__pyx_v_n, __pyx_v_x)), __pyx_v_x));
    goto __pyx_L0;
  }

  /* "_spherical_bessel.pxd":371
 * 
 * @cython.cdivision(True)
 * cdef inline double complex spherical_kn_d_complex(long n, double complex x) nogil:             # <<<<<<<<<<<<<<
 *     if n == 0:
 *         return -spherical_kn_complex(1, x)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_agm.pxd":14
 * 
 * 
 * cdef inline double _agm_iter(double a, double b) nogil:             # <<<<<<<<<<<<<<
 *     # Arithmetic-geometric mean, iterative implementation
 *     # a and b must be positive (not zero, not nan).
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_4_agm__agm_iter(double __pyx_v_a, double __pyx_v_b) {
  double __pyx_v_amean;
  double __pyx_v_gmean;
  int __pyx_v_count;
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_agm.pxd":20
 *     cdef double amean, gmean
 * 
 *     cdef int count = 20             # <<<<<<<<<<<<<<
 *     amean = 0.5*a + 0.5*b
 *     while (count > 0) and (amean != a and amean != b):
 */
  __pyx_v_count = 20;

  /* "_agm.pxd":21
 * 
 *     cdef int count = 20
 *     amean = 0.5*a + 0.5*b             # <<<<<<<<<<<<<<
 *     while (count > 0) and (amean != a and amean != b):
 *         gmean = sqrt(a)*sqrt(b)
 */
  __pyx_v_amean = ((0.5 * __pyx_v_a) + (0.5 * __pyx_v_b));

  /* "_agm.pxd":22
 *     cdef int count = 20
 *     amean = 0.5*a + 0.5*b
 *     while (count > 0) and (amean != a and amean != b):             # <<<<<<<<<<<<<<
 *         gmean = sqrt(a)*sqrt(b)
 *         a = amean
 */
  while (1) {
    __pyx_t_2 = ((__pyx_v_count > 0) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_amean != __pyx_v_a) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_amean != __pyx_v_b) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L5_bool_binop_done:;
    if (!__pyx_t_1) break;

    /* "_agm.pxd":23
 *     amean = 0.5*a + 0.5*b
 *     while (count > 0) and (amean != a and amean != b):
 *         gmean = sqrt(a)*sqrt(b)             # <<<<<<<<<<<<<<
 *         a = amean
 *         b = gmean
 */
    __pyx_v_gmean = (sqrt(__pyx_v_a) * sqrt(__pyx_v_b));

    /* "_agm.pxd":24
 *     while (count > 0) and (amean != a and amean != b):
 *         gmean = sqrt(a)*sqrt(b)
 *         a = amean             # <<<<<<<<<<<<<<
 *         b = gmean
 *         amean = 0.5*a + 0.5*b
 */
    __pyx_v_a = __pyx_v_amean;

    /* "_agm.pxd":25
 *         gmean = sqrt(a)*sqrt(b)
 *         a = amean
 *         b = gmean             # <<<<<<<<<<<<<<
 *         amean = 0.5*a + 0.5*b
 *         count -= 1
 */
    __pyx_v_b = __pyx_v_gmean;

    /* "_agm.pxd":26
 *         a = amean
 *         b = gmean
 *         amean = 0.5*a + 0.5*b             # <<<<<<<<<<<<<<
 *         count -= 1
 *     return amean
 */
    __pyx_v_amean = ((0.5 * __pyx_v_a) + (0.5 * __pyx_v_b));

    /* "_agm.pxd":27
 *         b = gmean
 *         amean = 0.5*a + 0.5*b
 *         count -= 1             # <<<<<<<<<<<<<<
 *     return amean
 * 
 */
    __pyx_v_count = (__pyx_v_count - 1);
  }

  /* "_agm.pxd":28
 *         amean = 0.5*a + 0.5*b
 *         count -= 1
 *     return amean             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_amean;
  goto __pyx_L0;

  /* "_agm.pxd":14
 * 
 * 
 * cdef inline double _agm_iter(double a, double b) nogil:             # <<<<<<<<<<<<<<
 *     # Arithmetic-geometric mean, iterative implementation
 *     # a and b must be positive (not zero, not nan).
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_agm.pxd":32
 * 
 * @cython.cdivision(True)
 * cdef inline double agm(double a, double b) nogil:             # <<<<<<<<<<<<<<
 *     # Arithmetic-geometric mean
 * 
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_4_agm_agm(double __pyx_v_a, double __pyx_v_b) {
  double __pyx_v_sqrthalfmax;
  double __pyx_v_invsqrthalfmax;
  double __pyx_v_e;
  int __pyx_v_sgn;
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "_agm.pxd":37
 *     # sqrthalfmax is sqrt(np.finfo(1.0).max/2)
 *     # invsqrthalfmax is 1/sqrthalfmax
 *     cdef double sqrthalfmax = 9.480751908109176e+153             # <<<<<<<<<<<<<<
 *     cdef double invsqrthalfmax = 1.0547686614863e-154
 * 
 */
  __pyx_v_sqrthalfmax = 9.480751908109176e+153;

  /* "_agm.pxd":38
 *     # invsqrthalfmax is 1/sqrthalfmax
 *     cdef double sqrthalfmax = 9.480751908109176e+153
 *     cdef double invsqrthalfmax = 1.0547686614863e-154             # <<<<<<<<<<<<<<
 * 
 *     cdef double e
 */
  __pyx_v_invsqrthalfmax = 1.0547686614863e-154;

  /* "_agm.pxd":43
 *     cdef int sgn
 * 
 *     if sc_isnan(a) or sc_isnan(b):             # <<<<<<<<<<<<<<
 *         return nan
 * 
 */
  __pyx_t_2 = (sc_isnan(__pyx_v_a) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (sc_isnan(__pyx_v_b) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_agm.pxd":44
 * 
 *     if sc_isnan(a) or sc_isnan(b):
 *         return nan             # <<<<<<<<<<<<<<
 * 
 *     if (a < 0 and b > 0) or (a > 0 and b < 0):
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "_agm.pxd":43
 *     cdef int sgn
 * 
 *     if sc_isnan(a) or sc_isnan(b):             # <<<<<<<<<<<<<<
 *         return nan
 * 
 */
  }

  /* "_agm.pxd":46
 *         return nan
 * 
 *     if (a < 0 and b > 0) or (a > 0 and b < 0):             # <<<<<<<<<<<<<<
 *         # a and b have opposite sign.
 *         return nan
 */
  __pyx_t_2 = ((__pyx_v_a < 0.0) != 0);
  if (!__pyx_t_2) {
    goto __pyx_L8_next_or;
  } else {
  }
  __pyx_t_2 = ((__pyx_v_b > 0.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_L8_next_or:;
  __pyx_t_2 = ((__pyx_v_a > 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_b < 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L7_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_agm.pxd":48
 *     if (a < 0 and b > 0) or (a > 0 and b < 0):
 *         # a and b have opposite sign.
 *         return nan             # <<<<<<<<<<<<<<
 * 
 *     if (sc_isinf(a) or sc_isinf(b)) and (a == 0 or b == 0):
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "_agm.pxd":46
 *         return nan
 * 
 *     if (a < 0 and b > 0) or (a > 0 and b < 0):             # <<<<<<<<<<<<<<
 *         # a and b have opposite sign.
 *         return nan
 */
  }

  /* "_agm.pxd":50
 *         return nan
 * 
 *     if (sc_isinf(a) or sc_isinf(b)) and (a == 0 or b == 0):             # <<<<<<<<<<<<<<
 *         # One value is inf and the other is 0.
 *         return nan
 */
  __pyx_t_2 = (sc_isinf(__pyx_v_a) != 0);
  if (!__pyx_t_2) {
  } else {
    goto __pyx_L13_next_and;
  }
  __pyx_t_2 = (sc_isinf(__pyx_v_b) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_L13_next_and:;
  __pyx_t_2 = ((__pyx_v_a == 0.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_b == 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L12_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_agm.pxd":52
 *     if (sc_isinf(a) or sc_isinf(b)) and (a == 0 or b == 0):
 *         # One value is inf and the other is 0.
 *         return nan             # <<<<<<<<<<<<<<
 * 
 *     if a == 0 or b == 0:
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "_agm.pxd":50
 *         return nan
 * 
 *     if (sc_isinf(a) or sc_isinf(b)) and (a == 0 or b == 0):             # <<<<<<<<<<<<<<
 *         # One value is inf and the other is 0.
 *         return nan
 */
  }

  /* "_agm.pxd":54
 *         return nan
 * 
 *     if a == 0 or b == 0:             # <<<<<<<<<<<<<<
 *         # At least one of the arguments is 0.
 *         return 0.0
 */
  __pyx_t_2 = ((__pyx_v_a == 0.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L17_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_b == 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L17_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_agm.pxd":56
 *     if a == 0 or b == 0:
 *         # At least one of the arguments is 0.
 *         return 0.0             # <<<<<<<<<<<<<<
 * 
 *     if a == b:
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "_agm.pxd":54
 *         return nan
 * 
 *     if a == 0 or b == 0:             # <<<<<<<<<<<<<<
 *         # At least one of the arguments is 0.
 *         return 0.0
 */
  }

  /* "_agm.pxd":58
 *         return 0.0
 * 
 *     if a == b:             # <<<<<<<<<<<<<<
 *         return a
 * 
 */
  __pyx_t_1 = ((__pyx_v_a == __pyx_v_b) != 0);
  if (__pyx_t_1) {

    /* "_agm.pxd":59
 * 
 *     if a == b:
 *         return a             # <<<<<<<<<<<<<<
 * 
 *     sgn = 1
 */
    __pyx_r = __pyx_v_a;
    goto __pyx_L0;

    /* "_agm.pxd":58
 *         return 0.0
 * 
 *     if a == b:             # <<<<<<<<<<<<<<
 *         return a
 * 
 */
  }

  /* "_agm.pxd":61
 *         return a
 * 
 *     sgn = 1             # <<<<<<<<<<<<<<
 *     if a < 0:
 *         sgn = -1
 */
  __pyx_v_sgn = 1;

  /* "_agm.pxd":62
 * 
 *     sgn = 1
 *     if a < 0:             # <<<<<<<<<<<<<<
 *         sgn = -1
 *         a = -a
 */
  __pyx_t_1 = ((__pyx_v_a < 0.0) != 0);
  if (__pyx_t_1) {

    /* "_agm.pxd":63
 *     sgn = 1
 *     if a < 0:
 *         sgn = -1             # <<<<<<<<<<<<<<
 *         a = -a
 *         b = -b
 */
    __pyx_v_sgn = -1;

    /* "_agm.pxd":64
 *     if a < 0:
 *         sgn = -1
 *         a = -a             # <<<<<<<<<<<<<<
 *         b = -b
 * 
 */
    __pyx_v_a = (-__pyx_v_a);

    /* "_agm.pxd":65
 *         sgn = -1
 *         a = -a
 *         b = -b             # <<<<<<<<<<<<<<
 * 
 *     # At this point, a and b are both positive and not nan.
 */
    __pyx_v_b = (-__pyx_v_b);

    /* "_agm.pxd":62
 * 
 *     sgn = 1
 *     if a < 0:             # <<<<<<<<<<<<<<
 *         sgn = -1
 *         a = -a
 */
  }

  /* "_agm.pxd":69
 *     # At this point, a and b are both positive and not nan.
 * 
 *     if (invsqrthalfmax < a < sqrthalfmax) and (invsqrthalfmax < b < sqrthalfmax):             # <<<<<<<<<<<<<<
 *         e = 4*a*b/(a + b)**2
 *         return sgn*(pi/4)*(a + b)/ellpk(e)
 */
  __pyx_t_2 = (__pyx_v_invsqrthalfmax < __pyx_v_a);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_a < __pyx_v_sqrthalfmax);
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L22_bool_binop_done;
  }
  __pyx_t_3 = (__pyx_v_invsqrthalfmax < __pyx_v_b);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_b < __pyx_v_sqrthalfmax);
  }
  __pyx_t_2 = (__pyx_t_3 != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L22_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_agm.pxd":70
 * 
 *     if (invsqrthalfmax < a < sqrthalfmax) and (invsqrthalfmax < b < sqrthalfmax):
 *         e = 4*a*b/(a + b)**2             # <<<<<<<<<<<<<<
 *         return sgn*(pi/4)*(a + b)/ellpk(e)
 *     else:
 */
    __pyx_v_e = (((4.0 * __pyx_v_a) * __pyx_v_b) / pow((__pyx_v_a + __pyx_v_b), 2.0));

    /* "_agm.pxd":71
 *     if (invsqrthalfmax < a < sqrthalfmax) and (invsqrthalfmax < b < sqrthalfmax):
 *         e = 4*a*b/(a + b)**2
 *         return sgn*(pi/4)*(a + b)/ellpk(e)             # <<<<<<<<<<<<<<
 *     else:
 *         # At least one value is "extreme" (very big or very small).
 */
    __pyx_r = (((__pyx_v_sgn * (NPY_PI / 4.0)) * (__pyx_v_a + __pyx_v_b)) / ellpk(__pyx_v_e));
    goto __pyx_L0;

    /* "_agm.pxd":69
 *     # At this point, a and b are both positive and not nan.
 * 
 *     if (invsqrthalfmax < a < sqrthalfmax) and (invsqrthalfmax < b < sqrthalfmax):             # <<<<<<<<<<<<<<
 *         e = 4*a*b/(a + b)**2
 *         return sgn*(pi/4)*(a + b)/ellpk(e)
 */
  }

  /* "_agm.pxd":75
 *         # At least one value is "extreme" (very big or very small).
 *         # Use the iteration to avoid overflow or underflow.
 *         return sgn*_agm_iter(a, b)             # <<<<<<<<<<<<<<
 */
  /*else*/ {
    __pyx_r = (__pyx_v_sgn * __pyx_f_5scipy_7special_4_agm__agm_iter(__pyx_v_a, __pyx_v_b));
    goto __pyx_L0;
  }

  /* "_agm.pxd":32
 * 
 * @cython.cdivision(True)
 * cdef inline double agm(double a, double b) nogil:             # <<<<<<<<<<<<<<
 *     # Arithmetic-geometric mean
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":47
 * # Fused type wrappers
 * 
 * cdef inline number_t hyp2f1(double a, double b, double c, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(double __pyx_v_a, double __pyx_v_b, double __pyx_v_c, double __pyx_v_z) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":50
 *     cdef npy_cdouble r
 *     if number_t is double:
 *         return hyp2f1_wrap(a, b, c, z)             # <<<<<<<<<<<<<<
 *     else:
 *         r = chyp2f1_wrap(a, b, c, npy_cdouble_from_double_complex(z))
 */
  __pyx_r = hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_z);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":47
 * # Fused type wrappers
 * 
 * cdef inline number_t hyp2f1(double a, double b, double c, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(double __pyx_v_a, double __pyx_v_b, double __pyx_v_c, __pyx_t_double_complex __pyx_v_z) {
  npy_cdouble __pyx_v_r;
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":52
 *         return hyp2f1_wrap(a, b, c, z)
 *     else:
 *         r = chyp2f1_wrap(a, b, c, npy_cdouble_from_double_complex(z))             # <<<<<<<<<<<<<<
 *         return double_complex_from_npy_cdouble(r)
 * 
 */
  __pyx_v_r = chyp2f1_wrap(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_z));

  /* "orthogonal_eval.pxd":53
 *     else:
 *         r = chyp2f1_wrap(a, b, c, npy_cdouble_from_double_complex(z))
 *         return double_complex_from_npy_cdouble(r)             # <<<<<<<<<<<<<<
 * 
 * cdef inline number_t hyp1f1(double a, double b, number_t z) nogil:
 */
  __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_double_complex_from_npy_cdouble(__pyx_v_r);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":47
 * # Fused type wrappers
 * 
 * cdef inline number_t hyp2f1(double a, double b, double c, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":55
 *         return double_complex_from_npy_cdouble(r)
 * 
 * cdef inline number_t hyp1f1(double a, double b, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_hyp1f1(double __pyx_v_a, double __pyx_v_b, double __pyx_v_z) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":58
 *     cdef npy_cdouble r
 *     if number_t is double:
 *         return hyp1f1_wrap(a, b, z)             # <<<<<<<<<<<<<<
 *     else:
 *         r = chyp1f1_wrap(a, b, npy_cdouble_from_double_complex(z))
 */
  __pyx_r = hyp1f1_wrap(__pyx_v_a, __pyx_v_b, __pyx_v_z);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":55
 *         return double_complex_from_npy_cdouble(r)
 * 
 * cdef inline number_t hyp1f1(double a, double b, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_hyp1f1(double __pyx_v_a, double __pyx_v_b, __pyx_t_double_complex __pyx_v_z) {
  npy_cdouble __pyx_v_r;
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":60
 *         return hyp1f1_wrap(a, b, z)
 *     else:
 *         r = chyp1f1_wrap(a, b, npy_cdouble_from_double_complex(z))             # <<<<<<<<<<<<<<
 *         return double_complex_from_npy_cdouble(r)
 * 
 */
  __pyx_v_r = chyp1f1_wrap(__pyx_v_a, __pyx_v_b, __pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_z));

  /* "orthogonal_eval.pxd":61
 *     else:
 *         r = chyp1f1_wrap(a, b, npy_cdouble_from_double_complex(z))
 *         return double_complex_from_npy_cdouble(r)             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_double_complex_from_npy_cdouble(__pyx_v_r);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":55
 *         return double_complex_from_npy_cdouble(r)
 * 
 * cdef inline number_t hyp1f1(double a, double b, number_t z) nogil:             # <<<<<<<<<<<<<<
 *     cdef npy_cdouble r
 *     if number_t is double:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":68
 * 
 * @cython.cdivision(True)
 * cdef inline double binom(double n, double k) nogil:             # <<<<<<<<<<<<<<
 *     cdef double kx, nx, num, den, dk, sgn
 *     cdef int i
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_binom(double __pyx_v_n, double __pyx_v_k) {
  double __pyx_v_kx;
  double __pyx_v_nx;
  double __pyx_v_num;
  double __pyx_v_den;
  double __pyx_v_dk;
  double __pyx_v_sgn;
  int __pyx_v_i;
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  long __pyx_t_3;
  long __pyx_t_4;
  int __pyx_t_5;
  double __pyx_t_6;

  /* "orthogonal_eval.pxd":72
 *     cdef int i
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         nx = floor(n)
 *         if n == nx:
 */
  __pyx_t_1 = ((__pyx_v_n < 0.0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":73
 * 
 *     if n < 0:
 *         nx = floor(n)             # <<<<<<<<<<<<<<
 *         if n == nx:
 *             # undefined
 */
    __pyx_v_nx = floor(__pyx_v_n);

    /* "orthogonal_eval.pxd":74
 *     if n < 0:
 *         nx = floor(n)
 *         if n == nx:             # <<<<<<<<<<<<<<
 *             # undefined
 *             return nan
 */
    __pyx_t_1 = ((__pyx_v_n == __pyx_v_nx) != 0);
    if (__pyx_t_1) {

      /* "orthogonal_eval.pxd":76
 *         if n == nx:
 *             # undefined
 *             return nan             # <<<<<<<<<<<<<<
 * 
 *     kx = floor(k)
 */
      __pyx_r = NPY_NAN;
      goto __pyx_L0;

      /* "orthogonal_eval.pxd":74
 *     if n < 0:
 *         nx = floor(n)
 *         if n == nx:             # <<<<<<<<<<<<<<
 *             # undefined
 *             return nan
 */
    }

    /* "orthogonal_eval.pxd":72
 *     cdef int i
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         nx = floor(n)
 *         if n == nx:
 */
  }

  /* "orthogonal_eval.pxd":78
 *             return nan
 * 
 *     kx = floor(k)             # <<<<<<<<<<<<<<
 *     if k == kx and (fabs(n) > 1e-8 or n == 0):
 *         # Integer case: use multiplication formula for less rounding error
 */
  __pyx_v_kx = floor(__pyx_v_k);

  /* "orthogonal_eval.pxd":79
 * 
 *     kx = floor(k)
 *     if k == kx and (fabs(n) > 1e-8 or n == 0):             # <<<<<<<<<<<<<<
 *         # Integer case: use multiplication formula for less rounding error
 *         # for cases where the result is an integer.
 */
  __pyx_t_2 = ((__pyx_v_k == __pyx_v_kx) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_2 = ((fabs(__pyx_v_n) > 1e-8) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_n == 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":86
 *         # precision.
 * 
 *         nx = floor(n)             # <<<<<<<<<<<<<<
 *         if nx == n and kx > nx/2 and nx > 0:
 *             # Reduce kx by symmetry
 */
    __pyx_v_nx = floor(__pyx_v_n);

    /* "orthogonal_eval.pxd":87
 * 
 *         nx = floor(n)
 *         if nx == n and kx > nx/2 and nx > 0:             # <<<<<<<<<<<<<<
 *             # Reduce kx by symmetry
 *             kx = nx - kx
 */
    __pyx_t_2 = ((__pyx_v_nx == __pyx_v_n) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L10_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_kx > (__pyx_v_nx / 2.0)) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L10_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_nx > 0.0) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L10_bool_binop_done:;
    if (__pyx_t_1) {

      /* "orthogonal_eval.pxd":89
 *         if nx == n and kx > nx/2 and nx > 0:
 *             # Reduce kx by symmetry
 *             kx = nx - kx             # <<<<<<<<<<<<<<
 * 
 *         if kx >= 0 and kx < 20:
 */
      __pyx_v_kx = (__pyx_v_nx - __pyx_v_kx);

      /* "orthogonal_eval.pxd":87
 * 
 *         nx = floor(n)
 *         if nx == n and kx > nx/2 and nx > 0:             # <<<<<<<<<<<<<<
 *             # Reduce kx by symmetry
 *             kx = nx - kx
 */
    }

    /* "orthogonal_eval.pxd":91
 *             kx = nx - kx
 * 
 *         if kx >= 0 and kx < 20:             # <<<<<<<<<<<<<<
 *             num = 1.0
 *             den = 1.0
 */
    __pyx_t_2 = ((__pyx_v_kx >= 0.0) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L14_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_kx < 20.0) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L14_bool_binop_done:;
    if (__pyx_t_1) {

      /* "orthogonal_eval.pxd":92
 * 
 *         if kx >= 0 and kx < 20:
 *             num = 1.0             # <<<<<<<<<<<<<<
 *             den = 1.0
 *             for i in range(1, 1 + <int>kx):
 */
      __pyx_v_num = 1.0;

      /* "orthogonal_eval.pxd":93
 *         if kx >= 0 and kx < 20:
 *             num = 1.0
 *             den = 1.0             # <<<<<<<<<<<<<<
 *             for i in range(1, 1 + <int>kx):
 *                 num *= i + n - kx
 */
      __pyx_v_den = 1.0;

      /* "orthogonal_eval.pxd":94
 *             num = 1.0
 *             den = 1.0
 *             for i in range(1, 1 + <int>kx):             # <<<<<<<<<<<<<<
 *                 num *= i + n - kx
 *                 den *= i
 */
      __pyx_t_3 = (1 + ((int)__pyx_v_kx));
      __pyx_t_4 = __pyx_t_3;
      for (__pyx_t_5 = 1; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
        __pyx_v_i = __pyx_t_5;

        /* "orthogonal_eval.pxd":95
 *             den = 1.0
 *             for i in range(1, 1 + <int>kx):
 *                 num *= i + n - kx             # <<<<<<<<<<<<<<
 *                 den *= i
 *                 if fabs(num) > 1e50:
 */
        __pyx_v_num = (__pyx_v_num * ((__pyx_v_i + __pyx_v_n) - __pyx_v_kx));

        /* "orthogonal_eval.pxd":96
 *             for i in range(1, 1 + <int>kx):
 *                 num *= i + n - kx
 *                 den *= i             # <<<<<<<<<<<<<<
 *                 if fabs(num) > 1e50:
 *                     num /= den
 */
        __pyx_v_den = (__pyx_v_den * __pyx_v_i);

        /* "orthogonal_eval.pxd":97
 *                 num *= i + n - kx
 *                 den *= i
 *                 if fabs(num) > 1e50:             # <<<<<<<<<<<<<<
 *                     num /= den
 *                     den = 1.0
 */
        __pyx_t_1 = ((fabs(__pyx_v_num) > 1e50) != 0);
        if (__pyx_t_1) {

          /* "orthogonal_eval.pxd":98
 *                 den *= i
 *                 if fabs(num) > 1e50:
 *                     num /= den             # <<<<<<<<<<<<<<
 *                     den = 1.0
 *             return num/den
 */
          __pyx_v_num = (__pyx_v_num / __pyx_v_den);

          /* "orthogonal_eval.pxd":99
 *                 if fabs(num) > 1e50:
 *                     num /= den
 *                     den = 1.0             # <<<<<<<<<<<<<<
 *             return num/den
 * 
 */
          __pyx_v_den = 1.0;

          /* "orthogonal_eval.pxd":97
 *                 num *= i + n - kx
 *                 den *= i
 *                 if fabs(num) > 1e50:             # <<<<<<<<<<<<<<
 *                     num /= den
 *                     den = 1.0
 */
        }
      }

      /* "orthogonal_eval.pxd":100
 *                     num /= den
 *                     den = 1.0
 *             return num/den             # <<<<<<<<<<<<<<
 * 
 *     # general case:
 */
      __pyx_r = (__pyx_v_num / __pyx_v_den);
      goto __pyx_L0;

      /* "orthogonal_eval.pxd":91
 *             kx = nx - kx
 * 
 *         if kx >= 0 and kx < 20:             # <<<<<<<<<<<<<<
 *             num = 1.0
 *             den = 1.0
 */
    }

    /* "orthogonal_eval.pxd":79
 * 
 *     kx = floor(k)
 *     if k == kx and (fabs(n) > 1e-8 or n == 0):             # <<<<<<<<<<<<<<
 *         # Integer case: use multiplication formula for less rounding error
 *         # for cases where the result is an integer.
 */
  }

  /* "orthogonal_eval.pxd":103
 * 
 *     # general case:
 *     if n >= 1e10*k and k > 0:             # <<<<<<<<<<<<<<
 *         # avoid under/overflows in intermediate results
 *         return exp(-lbeta(1 + n - k, 1 + k) - log(n + 1))
 */
  __pyx_t_2 = ((__pyx_v_n >= (1e10 * __pyx_v_k)) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L20_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_k > 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L20_bool_binop_done:;
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":105
 *     if n >= 1e10*k and k > 0:
 *         # avoid under/overflows in intermediate results
 *         return exp(-lbeta(1 + n - k, 1 + k) - log(n + 1))             # <<<<<<<<<<<<<<
 *     elif k > 1e8*fabs(n):
 *         # avoid loss of precision
 */
    __pyx_r = exp(((-lbeta(((1.0 + __pyx_v_n) - __pyx_v_k), (1.0 + __pyx_v_k))) - log((__pyx_v_n + 1.0))));
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":103
 * 
 *     # general case:
 *     if n >= 1e10*k and k > 0:             # <<<<<<<<<<<<<<
 *         # avoid under/overflows in intermediate results
 *         return exp(-lbeta(1 + n - k, 1 + k) - log(n + 1))
 */
  }

  /* "orthogonal_eval.pxd":106
 *         # avoid under/overflows in intermediate results
 *         return exp(-lbeta(1 + n - k, 1 + k) - log(n + 1))
 *     elif k > 1e8*fabs(n):             # <<<<<<<<<<<<<<
 *         # avoid loss of precision
 *         num = Gamma(1 + n) / fabs(k) + Gamma(1 + n) * n / (2*k**2) # + ...
 */
  __pyx_t_1 = ((__pyx_v_k > (1e8 * fabs(__pyx_v_n))) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":108
 *     elif k > 1e8*fabs(n):
 *         # avoid loss of precision
 *         num = Gamma(1 + n) / fabs(k) + Gamma(1 + n) * n / (2*k**2) # + ...             # <<<<<<<<<<<<<<
 *         num /= pi * fabs(k)**n
 *         if k > 0:
 */
    __pyx_v_num = ((Gamma((1.0 + __pyx_v_n)) / fabs(__pyx_v_k)) + ((Gamma((1.0 + __pyx_v_n)) * __pyx_v_n) / (2.0 * pow(__pyx_v_k, 2.0))));

    /* "orthogonal_eval.pxd":109
 *         # avoid loss of precision
 *         num = Gamma(1 + n) / fabs(k) + Gamma(1 + n) * n / (2*k**2) # + ...
 *         num /= pi * fabs(k)**n             # <<<<<<<<<<<<<<
 *         if k > 0:
 *             kx = floor(k)
 */
    __pyx_v_num = (__pyx_v_num / (M_PI * pow(fabs(__pyx_v_k), __pyx_v_n)));

    /* "orthogonal_eval.pxd":110
 *         num = Gamma(1 + n) / fabs(k) + Gamma(1 + n) * n / (2*k**2) # + ...
 *         num /= pi * fabs(k)**n
 *         if k > 0:             # <<<<<<<<<<<<<<
 *             kx = floor(k)
 *             if <int>kx == kx:
 */
    __pyx_t_1 = ((__pyx_v_k > 0.0) != 0);
    if (__pyx_t_1) {

      /* "orthogonal_eval.pxd":111
 *         num /= pi * fabs(k)**n
 *         if k > 0:
 *             kx = floor(k)             # <<<<<<<<<<<<<<
 *             if <int>kx == kx:
 *                 dk = k - kx
 */
      __pyx_v_kx = floor(__pyx_v_k);

      /* "orthogonal_eval.pxd":112
 *         if k > 0:
 *             kx = floor(k)
 *             if <int>kx == kx:             # <<<<<<<<<<<<<<
 *                 dk = k - kx
 *                 sgn = 1 if (<int>kx) % 2 == 0 else -1
 */
      __pyx_t_1 = ((((int)__pyx_v_kx) == __pyx_v_kx) != 0);
      if (__pyx_t_1) {

        /* "orthogonal_eval.pxd":113
 *             kx = floor(k)
 *             if <int>kx == kx:
 *                 dk = k - kx             # <<<<<<<<<<<<<<
 *                 sgn = 1 if (<int>kx) % 2 == 0 else -1
 *             else:
 */
        __pyx_v_dk = (__pyx_v_k - __pyx_v_kx);

        /* "orthogonal_eval.pxd":114
 *             if <int>kx == kx:
 *                 dk = k - kx
 *                 sgn = 1 if (<int>kx) % 2 == 0 else -1             # <<<<<<<<<<<<<<
 *             else:
 *                 dk = k
 */
        if ((((((int)__pyx_v_kx) % 2) == 0) != 0)) {
          __pyx_t_6 = 1.0;
        } else {
          __pyx_t_6 = -1.0;
        }
        __pyx_v_sgn = __pyx_t_6;

        /* "orthogonal_eval.pxd":112
 *         if k > 0:
 *             kx = floor(k)
 *             if <int>kx == kx:             # <<<<<<<<<<<<<<
 *                 dk = k - kx
 *                 sgn = 1 if (<int>kx) % 2 == 0 else -1
 */
        goto __pyx_L23;
      }

      /* "orthogonal_eval.pxd":116
 *                 sgn = 1 if (<int>kx) % 2 == 0 else -1
 *             else:
 *                 dk = k             # <<<<<<<<<<<<<<
 *                 sgn = 1
 *             return num * sin((dk-n)*pi) * sgn
 */
      /*else*/ {
        __pyx_v_dk = __pyx_v_k;

        /* "orthogonal_eval.pxd":117
 *             else:
 *                 dk = k
 *                 sgn = 1             # <<<<<<<<<<<<<<
 *             return num * sin((dk-n)*pi) * sgn
 *         else:
 */
        __pyx_v_sgn = 1.0;
      }
      __pyx_L23:;

      /* "orthogonal_eval.pxd":118
 *                 dk = k
 *                 sgn = 1
 *             return num * sin((dk-n)*pi) * sgn             # <<<<<<<<<<<<<<
 *         else:
 *             kx = floor(k)
 */
      __pyx_r = ((__pyx_v_num * sin(((__pyx_v_dk - __pyx_v_n) * M_PI))) * __pyx_v_sgn);
      goto __pyx_L0;

      /* "orthogonal_eval.pxd":110
 *         num = Gamma(1 + n) / fabs(k) + Gamma(1 + n) * n / (2*k**2) # + ...
 *         num /= pi * fabs(k)**n
 *         if k > 0:             # <<<<<<<<<<<<<<
 *             kx = floor(k)
 *             if <int>kx == kx:
 */
    }

    /* "orthogonal_eval.pxd":120
 *             return num * sin((dk-n)*pi) * sgn
 *         else:
 *             kx = floor(k)             # <<<<<<<<<<<<<<
 *             if <int>kx == kx:
 *                 return 0
 */
    /*else*/ {
      __pyx_v_kx = floor(__pyx_v_k);

      /* "orthogonal_eval.pxd":121
 *         else:
 *             kx = floor(k)
 *             if <int>kx == kx:             # <<<<<<<<<<<<<<
 *                 return 0
 *             else:
 */
      __pyx_t_1 = ((((int)__pyx_v_kx) == __pyx_v_kx) != 0);
      if (__pyx_t_1) {

        /* "orthogonal_eval.pxd":122
 *             kx = floor(k)
 *             if <int>kx == kx:
 *                 return 0             # <<<<<<<<<<<<<<
 *             else:
 *                 return num * sin(k*pi)
 */
        __pyx_r = 0.0;
        goto __pyx_L0;

        /* "orthogonal_eval.pxd":121
 *         else:
 *             kx = floor(k)
 *             if <int>kx == kx:             # <<<<<<<<<<<<<<
 *                 return 0
 *             else:
 */
      }

      /* "orthogonal_eval.pxd":124
 *                 return 0
 *             else:
 *                 return num * sin(k*pi)             # <<<<<<<<<<<<<<
 *     else:
 *         return 1/(n + 1)/beta(1 + n - k, 1 + k)
 */
      /*else*/ {
        __pyx_r = (__pyx_v_num * sin((__pyx_v_k * M_PI)));
        goto __pyx_L0;
      }
    }

    /* "orthogonal_eval.pxd":106
 *         # avoid under/overflows in intermediate results
 *         return exp(-lbeta(1 + n - k, 1 + k) - log(n + 1))
 *     elif k > 1e8*fabs(n):             # <<<<<<<<<<<<<<
 *         # avoid loss of precision
 *         num = Gamma(1 + n) / fabs(k) + Gamma(1 + n) * n / (2*k**2) # + ...
 */
  }

  /* "orthogonal_eval.pxd":126
 *                 return num * sin(k*pi)
 *     else:
 *         return 1/(n + 1)/beta(1 + n - k, 1 + k)             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  /*else*/ {
    __pyx_r = ((1.0 / (__pyx_v_n + 1.0)) / beta(((1.0 + __pyx_v_n) - __pyx_v_k), (1.0 + __pyx_v_k)));
    goto __pyx_L0;
  }

  /* "orthogonal_eval.pxd":68
 * 
 * @cython.cdivision(True)
 * cdef inline double binom(double n, double k) nogil:             # <<<<<<<<<<<<<<
 *     cdef double kx, nx, num, den, dk, sgn
 *     cdef int i
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":132
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_jacobi(double n, double alpha, double beta, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi(double __pyx_v_n, double __pyx_v_alpha, double __pyx_v_beta, double __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_c;
  double __pyx_v_d;
  double __pyx_v_g;
  double __pyx_r;

  /* "orthogonal_eval.pxd":136
 *     cdef number_t g
 * 
 *     d = binom(n+alpha, n)             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = n + alpha + beta + 1
 */
  __pyx_v_d = __pyx_f_5scipy_7special_15orthogonal_eval_binom((__pyx_v_n + __pyx_v_alpha), __pyx_v_n);

  /* "orthogonal_eval.pxd":137
 * 
 *     d = binom(n+alpha, n)
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = n + alpha + beta + 1
 *     c = alpha + 1
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":138
 *     d = binom(n+alpha, n)
 *     a = -n
 *     b = n + alpha + beta + 1             # <<<<<<<<<<<<<<
 *     c = alpha + 1
 *     g = 0.5*(1-x)
 */
  __pyx_v_b = (((__pyx_v_n + __pyx_v_alpha) + __pyx_v_beta) + 1.0);

  /* "orthogonal_eval.pxd":139
 *     a = -n
 *     b = n + alpha + beta + 1
 *     c = alpha + 1             # <<<<<<<<<<<<<<
 *     g = 0.5*(1-x)
 *     return d * hyp2f1(a, b, c, g)
 */
  __pyx_v_c = (__pyx_v_alpha + 1.0);

  /* "orthogonal_eval.pxd":140
 *     b = n + alpha + beta + 1
 *     c = alpha + 1
 *     g = 0.5*(1-x)             # <<<<<<<<<<<<<<
 *     return d * hyp2f1(a, b, c, g)
 * 
 */
  __pyx_v_g = (0.5 * (1.0 - __pyx_v_x));

  /* "orthogonal_eval.pxd":141
 *     c = alpha + 1
 *     g = 0.5*(1-x)
 *     return d * hyp2f1(a, b, c, g)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = (__pyx_v_d * __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_g));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":132
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_jacobi(double n, double alpha, double beta, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi(double __pyx_v_n, double __pyx_v_alpha, double __pyx_v_beta, __pyx_t_double_complex __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_c;
  double __pyx_v_d;
  __pyx_t_double_complex __pyx_v_g;
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":136
 *     cdef number_t g
 * 
 *     d = binom(n+alpha, n)             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = n + alpha + beta + 1
 */
  __pyx_v_d = __pyx_f_5scipy_7special_15orthogonal_eval_binom((__pyx_v_n + __pyx_v_alpha), __pyx_v_n);

  /* "orthogonal_eval.pxd":137
 * 
 *     d = binom(n+alpha, n)
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = n + alpha + beta + 1
 *     c = alpha + 1
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":138
 *     d = binom(n+alpha, n)
 *     a = -n
 *     b = n + alpha + beta + 1             # <<<<<<<<<<<<<<
 *     c = alpha + 1
 *     g = 0.5*(1-x)
 */
  __pyx_v_b = (((__pyx_v_n + __pyx_v_alpha) + __pyx_v_beta) + 1.0);

  /* "orthogonal_eval.pxd":139
 *     a = -n
 *     b = n + alpha + beta + 1
 *     c = alpha + 1             # <<<<<<<<<<<<<<
 *     g = 0.5*(1-x)
 *     return d * hyp2f1(a, b, c, g)
 */
  __pyx_v_c = (__pyx_v_alpha + 1.0);

  /* "orthogonal_eval.pxd":140
 *     b = n + alpha + beta + 1
 *     c = alpha + 1
 *     g = 0.5*(1-x)             # <<<<<<<<<<<<<<
 *     return d * hyp2f1(a, b, c, g)
 * 
 */
  __pyx_v_g = __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0.5, 0), __Pyx_c_diff_double(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_x));

  /* "orthogonal_eval.pxd":141
 *     c = alpha + 1
 *     g = 0.5*(1-x)
 *     return d * hyp2f1(a, b, c, g)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(__pyx_v_d, 0), __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_g));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":132
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_jacobi(double n, double alpha, double beta, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":144
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_jacobi_l(long n, double alpha, double beta, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long kk
 *     cdef double p, d
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi_l(long __pyx_v_n, double __pyx_v_alpha, double __pyx_v_beta, double __pyx_v_x) {
  long __pyx_v_kk;
  double __pyx_v_p;
  double __pyx_v_d;
  double __pyx_v_k;
  double __pyx_v_t;
  double __pyx_r;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;
  long __pyx_t_4;

  /* "orthogonal_eval.pxd":149
 *     cdef double k, t
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         return eval_jacobi(n, alpha, beta, x)
 *     elif n == 0:
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":150
 * 
 *     if n < 0:
 *         return eval_jacobi(n, alpha, beta, x)             # <<<<<<<<<<<<<<
 *     elif n == 0:
 *         return 1.0
 */
    __pyx_r = __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi(__pyx_v_n, __pyx_v_alpha, __pyx_v_beta, __pyx_v_x);
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":149
 *     cdef double k, t
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         return eval_jacobi(n, alpha, beta, x)
 *     elif n == 0:
 */
  }

  /* "orthogonal_eval.pxd":151
 *     if n < 0:
 *         return eval_jacobi(n, alpha, beta, x)
 *     elif n == 0:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif n == 1:
 */
  __pyx_t_1 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":152
 *         return eval_jacobi(n, alpha, beta, x)
 *     elif n == 0:
 *         return 1.0             # <<<<<<<<<<<<<<
 *     elif n == 1:
 *         return 0.5*(2*(alpha+1)+(alpha+beta+2)*(x-1))
 */
    __pyx_r = 1.0;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":151
 *     if n < 0:
 *         return eval_jacobi(n, alpha, beta, x)
 *     elif n == 0:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif n == 1:
 */
  }

  /* "orthogonal_eval.pxd":153
 *     elif n == 0:
 *         return 1.0
 *     elif n == 1:             # <<<<<<<<<<<<<<
 *         return 0.5*(2*(alpha+1)+(alpha+beta+2)*(x-1))
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_n == 1) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":154
 *         return 1.0
 *     elif n == 1:
 *         return 0.5*(2*(alpha+1)+(alpha+beta+2)*(x-1))             # <<<<<<<<<<<<<<
 *     else:
 *         d = (alpha+beta+2)*(x - 1) / (2*(alpha+1))
 */
    __pyx_r = (0.5 * ((2.0 * (__pyx_v_alpha + 1.0)) + (((__pyx_v_alpha + __pyx_v_beta) + 2.0) * (__pyx_v_x - 1.0))));
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":153
 *     elif n == 0:
 *         return 1.0
 *     elif n == 1:             # <<<<<<<<<<<<<<
 *         return 0.5*(2*(alpha+1)+(alpha+beta+2)*(x-1))
 *     else:
 */
  }

  /* "orthogonal_eval.pxd":156
 *         return 0.5*(2*(alpha+1)+(alpha+beta+2)*(x-1))
 *     else:
 *         d = (alpha+beta+2)*(x - 1) / (2*(alpha+1))             # <<<<<<<<<<<<<<
 *         p = d + 1
 *         for kk in range(n-1):
 */
  /*else*/ {
    __pyx_v_d = ((((__pyx_v_alpha + __pyx_v_beta) + 2.0) * (__pyx_v_x - 1.0)) / (2.0 * (__pyx_v_alpha + 1.0)));

    /* "orthogonal_eval.pxd":157
 *     else:
 *         d = (alpha+beta+2)*(x - 1) / (2*(alpha+1))
 *         p = d + 1             # <<<<<<<<<<<<<<
 *         for kk in range(n-1):
 *             k = kk+1.0
 */
    __pyx_v_p = (__pyx_v_d + 1.0);

    /* "orthogonal_eval.pxd":158
 *         d = (alpha+beta+2)*(x - 1) / (2*(alpha+1))
 *         p = d + 1
 *         for kk in range(n-1):             # <<<<<<<<<<<<<<
 *             k = kk+1.0
 *             t = 2*k+alpha+beta
 */
    __pyx_t_2 = (__pyx_v_n - 1);
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_kk = __pyx_t_4;

      /* "orthogonal_eval.pxd":159
 *         p = d + 1
 *         for kk in range(n-1):
 *             k = kk+1.0             # <<<<<<<<<<<<<<
 *             t = 2*k+alpha+beta
 *             d = ((t*(t+1)*(t+2))*(x-1)*p + 2*k*(k+beta)*(t+2)*d) / (2*(k+alpha+1)*(k+alpha+beta+1)*t)
 */
      __pyx_v_k = (__pyx_v_kk + 1.0);

      /* "orthogonal_eval.pxd":160
 *         for kk in range(n-1):
 *             k = kk+1.0
 *             t = 2*k+alpha+beta             # <<<<<<<<<<<<<<
 *             d = ((t*(t+1)*(t+2))*(x-1)*p + 2*k*(k+beta)*(t+2)*d) / (2*(k+alpha+1)*(k+alpha+beta+1)*t)
 *             p = d + p
 */
      __pyx_v_t = (((2.0 * __pyx_v_k) + __pyx_v_alpha) + __pyx_v_beta);

      /* "orthogonal_eval.pxd":161
 *             k = kk+1.0
 *             t = 2*k+alpha+beta
 *             d = ((t*(t+1)*(t+2))*(x-1)*p + 2*k*(k+beta)*(t+2)*d) / (2*(k+alpha+1)*(k+alpha+beta+1)*t)             # <<<<<<<<<<<<<<
 *             p = d + p
 *         return binom(n+alpha, n)*p
 */
      __pyx_v_d = ((((((__pyx_v_t * (__pyx_v_t + 1.0)) * (__pyx_v_t + 2.0)) * (__pyx_v_x - 1.0)) * __pyx_v_p) + ((((2.0 * __pyx_v_k) * (__pyx_v_k + __pyx_v_beta)) * (__pyx_v_t + 2.0)) * __pyx_v_d)) / (((2.0 * ((__pyx_v_k + __pyx_v_alpha) + 1.0)) * (((__pyx_v_k + __pyx_v_alpha) + __pyx_v_beta) + 1.0)) * __pyx_v_t));

      /* "orthogonal_eval.pxd":162
 *             t = 2*k+alpha+beta
 *             d = ((t*(t+1)*(t+2))*(x-1)*p + 2*k*(k+beta)*(t+2)*d) / (2*(k+alpha+1)*(k+alpha+beta+1)*t)
 *             p = d + p             # <<<<<<<<<<<<<<
 *         return binom(n+alpha, n)*p
 * 
 */
      __pyx_v_p = (__pyx_v_d + __pyx_v_p);
    }

    /* "orthogonal_eval.pxd":163
 *             d = ((t*(t+1)*(t+2))*(x-1)*p + 2*k*(k+beta)*(t+2)*d) / (2*(k+alpha+1)*(k+alpha+beta+1)*t)
 *             p = d + p
 *         return binom(n+alpha, n)*p             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
    __pyx_r = (__pyx_f_5scipy_7special_15orthogonal_eval_binom((__pyx_v_n + __pyx_v_alpha), __pyx_v_n) * __pyx_v_p);
    goto __pyx_L0;
  }

  /* "orthogonal_eval.pxd":144
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_jacobi_l(long n, double alpha, double beta, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long kk
 *     cdef double p, d
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":170
 * 
 * @cython.cdivision(True)
 * cdef inline number_t eval_sh_jacobi(double n, double p, double q, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_jacobi(n, p-q, q-1, 2*x-1) / binom(2*n + p - 1, n)
 * 
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi(double __pyx_v_n, double __pyx_v_p, double __pyx_v_q, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":171
 * @cython.cdivision(True)
 * cdef inline number_t eval_sh_jacobi(double n, double p, double q, number_t x) nogil:
 *     return eval_jacobi(n, p-q, q-1, 2*x-1) / binom(2*n + p - 1, n)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = (__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi(__pyx_v_n, (__pyx_v_p - __pyx_v_q), (__pyx_v_q - 1.0), ((2.0 * __pyx_v_x) - 1.0)) / __pyx_f_5scipy_7special_15orthogonal_eval_binom((((2.0 * __pyx_v_n) + __pyx_v_p) - 1.0), __pyx_v_n));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":170
 * 
 * @cython.cdivision(True)
 * cdef inline number_t eval_sh_jacobi(double n, double p, double q, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_jacobi(n, p-q, q-1, 2*x-1) / binom(2*n + p - 1, n)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi(double __pyx_v_n, double __pyx_v_p, double __pyx_v_q, __pyx_t_double_complex __pyx_v_x) {
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":171
 * @cython.cdivision(True)
 * cdef inline number_t eval_sh_jacobi(double n, double p, double q, number_t x) nogil:
 *     return eval_jacobi(n, p-q, q-1, 2*x-1) / binom(2*n + p - 1, n)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = __Pyx_c_quot_double(__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi(__pyx_v_n, (__pyx_v_p - __pyx_v_q), (__pyx_v_q - 1.0), __Pyx_c_diff_double(__Pyx_c_prod_double(__pyx_t_double_complex_from_parts(2, 0), __pyx_v_x), __pyx_t_double_complex_from_parts(1, 0))), __pyx_t_double_complex_from_parts(__pyx_f_5scipy_7special_15orthogonal_eval_binom((((2.0 * __pyx_v_n) + __pyx_v_p) - 1.0), __pyx_v_n), 0));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":170
 * 
 * @cython.cdivision(True)
 * cdef inline number_t eval_sh_jacobi(double n, double p, double q, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_jacobi(n, p-q, q-1, 2*x-1) / binom(2*n + p - 1, n)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":174
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_sh_jacobi_l(long n, double p, double q, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_jacobi_l(n, p-q, q-1, 2*x-1) / binom(2*n + p - 1, n)
 * 
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi_l(long __pyx_v_n, double __pyx_v_p, double __pyx_v_q, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":175
 * @cython.cdivision(True)
 * cdef inline double eval_sh_jacobi_l(long n, double p, double q, double x) nogil:
 *     return eval_jacobi_l(n, p-q, q-1, 2*x-1) / binom(2*n + p - 1, n)             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  __pyx_r = (__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi_l(__pyx_v_n, (__pyx_v_p - __pyx_v_q), (__pyx_v_q - 1.0), ((2.0 * __pyx_v_x) - 1.0)) / __pyx_f_5scipy_7special_15orthogonal_eval_binom((((2 * __pyx_v_n) + __pyx_v_p) - 1.0), __pyx_v_n));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":174
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_sh_jacobi_l(long n, double p, double q, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_jacobi_l(n, p-q, q-1, 2*x-1) / binom(2*n + p - 1, n)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":182
 * 
 * @cython.cdivision(True)
 * cdef inline number_t eval_gegenbauer(double n, double alpha, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer(double __pyx_v_n, double __pyx_v_alpha, double __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_c;
  double __pyx_v_d;
  double __pyx_v_g;
  double __pyx_r;

  /* "orthogonal_eval.pxd":186
 *     cdef number_t g
 * 
 *     d = Gamma(n+2*alpha)/Gamma(1+n)/Gamma(2*alpha)             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = n + 2*alpha
 */
  __pyx_v_d = ((Gamma((__pyx_v_n + (2.0 * __pyx_v_alpha))) / Gamma((1.0 + __pyx_v_n))) / Gamma((2.0 * __pyx_v_alpha)));

  /* "orthogonal_eval.pxd":187
 * 
 *     d = Gamma(n+2*alpha)/Gamma(1+n)/Gamma(2*alpha)
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = n + 2*alpha
 *     c = alpha + 0.5
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":188
 *     d = Gamma(n+2*alpha)/Gamma(1+n)/Gamma(2*alpha)
 *     a = -n
 *     b = n + 2*alpha             # <<<<<<<<<<<<<<
 *     c = alpha + 0.5
 *     g = (1-x)/2.0
 */
  __pyx_v_b = (__pyx_v_n + (2.0 * __pyx_v_alpha));

  /* "orthogonal_eval.pxd":189
 *     a = -n
 *     b = n + 2*alpha
 *     c = alpha + 0.5             # <<<<<<<<<<<<<<
 *     g = (1-x)/2.0
 *     return d * hyp2f1(a, b, c, g)
 */
  __pyx_v_c = (__pyx_v_alpha + 0.5);

  /* "orthogonal_eval.pxd":190
 *     b = n + 2*alpha
 *     c = alpha + 0.5
 *     g = (1-x)/2.0             # <<<<<<<<<<<<<<
 *     return d * hyp2f1(a, b, c, g)
 * 
 */
  __pyx_v_g = ((1.0 - __pyx_v_x) / 2.0);

  /* "orthogonal_eval.pxd":191
 *     c = alpha + 0.5
 *     g = (1-x)/2.0
 *     return d * hyp2f1(a, b, c, g)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = (__pyx_v_d * __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_g));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":182
 * 
 * @cython.cdivision(True)
 * cdef inline number_t eval_gegenbauer(double n, double alpha, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer(double __pyx_v_n, double __pyx_v_alpha, __pyx_t_double_complex __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_c;
  double __pyx_v_d;
  __pyx_t_double_complex __pyx_v_g;
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":186
 *     cdef number_t g
 * 
 *     d = Gamma(n+2*alpha)/Gamma(1+n)/Gamma(2*alpha)             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = n + 2*alpha
 */
  __pyx_v_d = ((Gamma((__pyx_v_n + (2.0 * __pyx_v_alpha))) / Gamma((1.0 + __pyx_v_n))) / Gamma((2.0 * __pyx_v_alpha)));

  /* "orthogonal_eval.pxd":187
 * 
 *     d = Gamma(n+2*alpha)/Gamma(1+n)/Gamma(2*alpha)
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = n + 2*alpha
 *     c = alpha + 0.5
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":188
 *     d = Gamma(n+2*alpha)/Gamma(1+n)/Gamma(2*alpha)
 *     a = -n
 *     b = n + 2*alpha             # <<<<<<<<<<<<<<
 *     c = alpha + 0.5
 *     g = (1-x)/2.0
 */
  __pyx_v_b = (__pyx_v_n + (2.0 * __pyx_v_alpha));

  /* "orthogonal_eval.pxd":189
 *     a = -n
 *     b = n + 2*alpha
 *     c = alpha + 0.5             # <<<<<<<<<<<<<<
 *     g = (1-x)/2.0
 *     return d * hyp2f1(a, b, c, g)
 */
  __pyx_v_c = (__pyx_v_alpha + 0.5);

  /* "orthogonal_eval.pxd":190
 *     b = n + 2*alpha
 *     c = alpha + 0.5
 *     g = (1-x)/2.0             # <<<<<<<<<<<<<<
 *     return d * hyp2f1(a, b, c, g)
 * 
 */
  __pyx_v_g = __Pyx_c_quot_double(__Pyx_c_diff_double(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_x), __pyx_t_double_complex_from_parts(2.0, 0));

  /* "orthogonal_eval.pxd":191
 *     c = alpha + 0.5
 *     g = (1-x)/2.0
 *     return d * hyp2f1(a, b, c, g)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(__pyx_v_d, 0), __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_g));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":182
 * 
 * @cython.cdivision(True)
 * cdef inline number_t eval_gegenbauer(double n, double alpha, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":194
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_gegenbauer_l(long n, double alpha, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long kk
 *     cdef long a, b
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer_l(long __pyx_v_n, double __pyx_v_alpha, double __pyx_v_x) {
  long __pyx_v_kk;
  long __pyx_v_a;
  double __pyx_v_p;
  double __pyx_v_d;
  double __pyx_v_k;
  double __pyx_r;
  int __pyx_t_1;
  double __pyx_t_2;
  long __pyx_t_3;
  long __pyx_t_4;
  long __pyx_t_5;

  /* "orthogonal_eval.pxd":200
 *     cdef double k
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         return 0.0
 *     elif n == 0:
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":201
 * 
 *     if n < 0:
 *         return 0.0             # <<<<<<<<<<<<<<
 *     elif n == 0:
 *         return 1.0
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":200
 *     cdef double k
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         return 0.0
 *     elif n == 0:
 */
  }

  /* "orthogonal_eval.pxd":202
 *     if n < 0:
 *         return 0.0
 *     elif n == 0:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif n == 1:
 */
  __pyx_t_1 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":203
 *         return 0.0
 *     elif n == 0:
 *         return 1.0             # <<<<<<<<<<<<<<
 *     elif n == 1:
 *         return 2*alpha*x
 */
    __pyx_r = 1.0;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":202
 *     if n < 0:
 *         return 0.0
 *     elif n == 0:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif n == 1:
 */
  }

  /* "orthogonal_eval.pxd":204
 *     elif n == 0:
 *         return 1.0
 *     elif n == 1:             # <<<<<<<<<<<<<<
 *         return 2*alpha*x
 *     elif alpha == 0.0:
 */
  __pyx_t_1 = ((__pyx_v_n == 1) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":205
 *         return 1.0
 *     elif n == 1:
 *         return 2*alpha*x             # <<<<<<<<<<<<<<
 *     elif alpha == 0.0:
 *         return eval_gegenbauer(n, alpha, x)
 */
    __pyx_r = ((2.0 * __pyx_v_alpha) * __pyx_v_x);
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":204
 *     elif n == 0:
 *         return 1.0
 *     elif n == 1:             # <<<<<<<<<<<<<<
 *         return 2*alpha*x
 *     elif alpha == 0.0:
 */
  }

  /* "orthogonal_eval.pxd":206
 *     elif n == 1:
 *         return 2*alpha*x
 *     elif alpha == 0.0:             # <<<<<<<<<<<<<<
 *         return eval_gegenbauer(n, alpha, x)
 *     elif fabs(x) < 1e-5:
 */
  __pyx_t_1 = ((__pyx_v_alpha == 0.0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":207
 *         return 2*alpha*x
 *     elif alpha == 0.0:
 *         return eval_gegenbauer(n, alpha, x)             # <<<<<<<<<<<<<<
 *     elif fabs(x) < 1e-5:
 *         # Power series rather than recurrence due to loss of precision
 */
    __pyx_r = __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer(__pyx_v_n, __pyx_v_alpha, __pyx_v_x);
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":206
 *     elif n == 1:
 *         return 2*alpha*x
 *     elif alpha == 0.0:             # <<<<<<<<<<<<<<
 *         return eval_gegenbauer(n, alpha, x)
 *     elif fabs(x) < 1e-5:
 */
  }

  /* "orthogonal_eval.pxd":208
 *     elif alpha == 0.0:
 *         return eval_gegenbauer(n, alpha, x)
 *     elif fabs(x) < 1e-5:             # <<<<<<<<<<<<<<
 *         # Power series rather than recurrence due to loss of precision
 *         # http://functions.wolfram.com/Polynomials/GegenbauerC3/02/
 */
  __pyx_t_1 = ((fabs(__pyx_v_x) < 1e-5) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":211
 *         # Power series rather than recurrence due to loss of precision
 *         # http://functions.wolfram.com/Polynomials/GegenbauerC3/02/
 *         a = n//2             # <<<<<<<<<<<<<<
 * 
 *         d = 1 if a % 2 == 0 else -1
 */
    __pyx_v_a = __Pyx_div_long(__pyx_v_n, 2);

    /* "orthogonal_eval.pxd":213
 *         a = n//2
 * 
 *         d = 1 if a % 2 == 0 else -1             # <<<<<<<<<<<<<<
 *         d /= beta(alpha, 1 + a)
 *         if n == 2*a:
 */
    if ((((__pyx_v_a % 2) == 0) != 0)) {
      __pyx_t_2 = 1.0;
    } else {
      __pyx_t_2 = -1.0;
    }
    __pyx_v_d = __pyx_t_2;

    /* "orthogonal_eval.pxd":214
 * 
 *         d = 1 if a % 2 == 0 else -1
 *         d /= beta(alpha, 1 + a)             # <<<<<<<<<<<<<<
 *         if n == 2*a:
 *             d /= (a + alpha)
 */
    __pyx_v_d = (__pyx_v_d / beta(__pyx_v_alpha, (1 + __pyx_v_a)));

    /* "orthogonal_eval.pxd":215
 *         d = 1 if a % 2 == 0 else -1
 *         d /= beta(alpha, 1 + a)
 *         if n == 2*a:             # <<<<<<<<<<<<<<
 *             d /= (a + alpha)
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_n == (2 * __pyx_v_a)) != 0);
    if (__pyx_t_1) {

      /* "orthogonal_eval.pxd":216
 *         d /= beta(alpha, 1 + a)
 *         if n == 2*a:
 *             d /= (a + alpha)             # <<<<<<<<<<<<<<
 *         else:
 *             d *= 2*x
 */
      __pyx_v_d = (__pyx_v_d / (__pyx_v_a + __pyx_v_alpha));

      /* "orthogonal_eval.pxd":215
 *         d = 1 if a % 2 == 0 else -1
 *         d /= beta(alpha, 1 + a)
 *         if n == 2*a:             # <<<<<<<<<<<<<<
 *             d /= (a + alpha)
 *         else:
 */
      goto __pyx_L4;
    }

    /* "orthogonal_eval.pxd":218
 *             d /= (a + alpha)
 *         else:
 *             d *= 2*x             # <<<<<<<<<<<<<<
 * 
 *         p = 0
 */
    /*else*/ {
      __pyx_v_d = (__pyx_v_d * (2.0 * __pyx_v_x));
    }
    __pyx_L4:;

    /* "orthogonal_eval.pxd":220
 *             d *= 2*x
 * 
 *         p = 0             # <<<<<<<<<<<<<<
 *         for kk in range(a+1):
 *             p += d
 */
    __pyx_v_p = 0.0;

    /* "orthogonal_eval.pxd":221
 * 
 *         p = 0
 *         for kk in range(a+1):             # <<<<<<<<<<<<<<
 *             p += d
 *             d *= -4*x**2 * (a - kk) * (-a + alpha + kk + n) / (
 */
    __pyx_t_3 = (__pyx_v_a + 1);
    __pyx_t_4 = __pyx_t_3;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_kk = __pyx_t_5;

      /* "orthogonal_eval.pxd":222
 *         p = 0
 *         for kk in range(a+1):
 *             p += d             # <<<<<<<<<<<<<<
 *             d *= -4*x**2 * (a - kk) * (-a + alpha + kk + n) / (
 *                 (n + 1 - 2*a + 2*kk) * (n + 2 - 2*a + 2*kk))
 */
      __pyx_v_p = (__pyx_v_p + __pyx_v_d);

      /* "orthogonal_eval.pxd":223
 *         for kk in range(a+1):
 *             p += d
 *             d *= -4*x**2 * (a - kk) * (-a + alpha + kk + n) / (             # <<<<<<<<<<<<<<
 *                 (n + 1 - 2*a + 2*kk) * (n + 2 - 2*a + 2*kk))
 *             if fabs(d) == 1e-20*fabs(p):
 */
      __pyx_v_d = (__pyx_v_d * ((((-4.0 * pow(__pyx_v_x, 2.0)) * (__pyx_v_a - __pyx_v_kk)) * ((((-__pyx_v_a) + __pyx_v_alpha) + __pyx_v_kk) + __pyx_v_n)) / ((double)((((__pyx_v_n + 1) - (2 * __pyx_v_a)) + (2 * __pyx_v_kk)) * (((__pyx_v_n + 2) - (2 * __pyx_v_a)) + (2 * __pyx_v_kk))))));

      /* "orthogonal_eval.pxd":225
 *             d *= -4*x**2 * (a - kk) * (-a + alpha + kk + n) / (
 *                 (n + 1 - 2*a + 2*kk) * (n + 2 - 2*a + 2*kk))
 *             if fabs(d) == 1e-20*fabs(p):             # <<<<<<<<<<<<<<
 *                 # converged
 *                 break
 */
      __pyx_t_1 = ((fabs(__pyx_v_d) == (1e-20 * fabs(__pyx_v_p))) != 0);
      if (__pyx_t_1) {

        /* "orthogonal_eval.pxd":227
 *             if fabs(d) == 1e-20*fabs(p):
 *                 # converged
 *                 break             # <<<<<<<<<<<<<<
 *         return p
 *     else:
 */
        goto __pyx_L6_break;

        /* "orthogonal_eval.pxd":225
 *             d *= -4*x**2 * (a - kk) * (-a + alpha + kk + n) / (
 *                 (n + 1 - 2*a + 2*kk) * (n + 2 - 2*a + 2*kk))
 *             if fabs(d) == 1e-20*fabs(p):             # <<<<<<<<<<<<<<
 *                 # converged
 *                 break
 */
      }
    }
    __pyx_L6_break:;

    /* "orthogonal_eval.pxd":228
 *                 # converged
 *                 break
 *         return p             # <<<<<<<<<<<<<<
 *     else:
 *         d = x - 1
 */
    __pyx_r = __pyx_v_p;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":208
 *     elif alpha == 0.0:
 *         return eval_gegenbauer(n, alpha, x)
 *     elif fabs(x) < 1e-5:             # <<<<<<<<<<<<<<
 *         # Power series rather than recurrence due to loss of precision
 *         # http://functions.wolfram.com/Polynomials/GegenbauerC3/02/
 */
  }

  /* "orthogonal_eval.pxd":230
 *         return p
 *     else:
 *         d = x - 1             # <<<<<<<<<<<<<<
 *         p = x
 *         for kk in range(n-1):
 */
  /*else*/ {
    __pyx_v_d = (__pyx_v_x - 1.0);

    /* "orthogonal_eval.pxd":231
 *     else:
 *         d = x - 1
 *         p = x             # <<<<<<<<<<<<<<
 *         for kk in range(n-1):
 *             k = kk+1.0
 */
    __pyx_v_p = __pyx_v_x;

    /* "orthogonal_eval.pxd":232
 *         d = x - 1
 *         p = x
 *         for kk in range(n-1):             # <<<<<<<<<<<<<<
 *             k = kk+1.0
 *             d = (2*(k+alpha)/(k+2*alpha))*(x-1)*p + (k/(k+2*alpha)) * d
 */
    __pyx_t_3 = (__pyx_v_n - 1);
    __pyx_t_4 = __pyx_t_3;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_kk = __pyx_t_5;

      /* "orthogonal_eval.pxd":233
 *         p = x
 *         for kk in range(n-1):
 *             k = kk+1.0             # <<<<<<<<<<<<<<
 *             d = (2*(k+alpha)/(k+2*alpha))*(x-1)*p + (k/(k+2*alpha)) * d
 *             p = d + p
 */
      __pyx_v_k = (__pyx_v_kk + 1.0);

      /* "orthogonal_eval.pxd":234
 *         for kk in range(n-1):
 *             k = kk+1.0
 *             d = (2*(k+alpha)/(k+2*alpha))*(x-1)*p + (k/(k+2*alpha)) * d             # <<<<<<<<<<<<<<
 *             p = d + p
 * 
 */
      __pyx_v_d = (((((2.0 * (__pyx_v_k + __pyx_v_alpha)) / (__pyx_v_k + (2.0 * __pyx_v_alpha))) * (__pyx_v_x - 1.0)) * __pyx_v_p) + ((__pyx_v_k / (__pyx_v_k + (2.0 * __pyx_v_alpha))) * __pyx_v_d));

      /* "orthogonal_eval.pxd":235
 *             k = kk+1.0
 *             d = (2*(k+alpha)/(k+2*alpha))*(x-1)*p + (k/(k+2*alpha)) * d
 *             p = d + p             # <<<<<<<<<<<<<<
 * 
 *         if fabs(alpha/n) < 1e-8:
 */
      __pyx_v_p = (__pyx_v_d + __pyx_v_p);
    }

    /* "orthogonal_eval.pxd":237
 *             p = d + p
 * 
 *         if fabs(alpha/n) < 1e-8:             # <<<<<<<<<<<<<<
 *             # avoid loss of precision
 *             return 2*alpha/n * p
 */
    __pyx_t_1 = ((fabs((__pyx_v_alpha / ((double)__pyx_v_n))) < 1e-8) != 0);
    if (__pyx_t_1) {

      /* "orthogonal_eval.pxd":239
 *         if fabs(alpha/n) < 1e-8:
 *             # avoid loss of precision
 *             return 2*alpha/n * p             # <<<<<<<<<<<<<<
 *         else:
 *             return binom(n+2*alpha-1, n)*p
 */
      __pyx_r = (((2.0 * __pyx_v_alpha) / ((double)__pyx_v_n)) * __pyx_v_p);
      goto __pyx_L0;

      /* "orthogonal_eval.pxd":237
 *             p = d + p
 * 
 *         if fabs(alpha/n) < 1e-8:             # <<<<<<<<<<<<<<
 *             # avoid loss of precision
 *             return 2*alpha/n * p
 */
    }

    /* "orthogonal_eval.pxd":241
 *             return 2*alpha/n * p
 *         else:
 *             return binom(n+2*alpha-1, n)*p             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
    /*else*/ {
      __pyx_r = (__pyx_f_5scipy_7special_15orthogonal_eval_binom(((__pyx_v_n + (2.0 * __pyx_v_alpha)) - 1.0), __pyx_v_n) * __pyx_v_p);
      goto __pyx_L0;
    }
  }

  /* "orthogonal_eval.pxd":194
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_gegenbauer_l(long n, double alpha, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long kk
 *     cdef long a, b
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":247
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebyt(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_c;
  CYTHON_UNUSED double __pyx_v_d;
  double __pyx_v_g;
  double __pyx_r;

  /* "orthogonal_eval.pxd":251
 *     cdef number_t g
 * 
 *     d = 1.0             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = n
 */
  __pyx_v_d = 1.0;

  /* "orthogonal_eval.pxd":252
 * 
 *     d = 1.0
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = n
 *     c = 0.5
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":253
 *     d = 1.0
 *     a = -n
 *     b = n             # <<<<<<<<<<<<<<
 *     c = 0.5
 *     g = 0.5*(1-x)
 */
  __pyx_v_b = __pyx_v_n;

  /* "orthogonal_eval.pxd":254
 *     a = -n
 *     b = n
 *     c = 0.5             # <<<<<<<<<<<<<<
 *     g = 0.5*(1-x)
 *     return hyp2f1(a, b, c, g)
 */
  __pyx_v_c = 0.5;

  /* "orthogonal_eval.pxd":255
 *     b = n
 *     c = 0.5
 *     g = 0.5*(1-x)             # <<<<<<<<<<<<<<
 *     return hyp2f1(a, b, c, g)
 * 
 */
  __pyx_v_g = (0.5 * (1.0 - __pyx_v_x));

  /* "orthogonal_eval.pxd":256
 *     c = 0.5
 *     g = 0.5*(1-x)
 *     return hyp2f1(a, b, c, g)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_chebyt_l(long k, double x) nogil:
 */
  __pyx_r = __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_g);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":247
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebyt(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt(double __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_c;
  CYTHON_UNUSED double __pyx_v_d;
  __pyx_t_double_complex __pyx_v_g;
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":251
 *     cdef number_t g
 * 
 *     d = 1.0             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = n
 */
  __pyx_v_d = 1.0;

  /* "orthogonal_eval.pxd":252
 * 
 *     d = 1.0
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = n
 *     c = 0.5
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":253
 *     d = 1.0
 *     a = -n
 *     b = n             # <<<<<<<<<<<<<<
 *     c = 0.5
 *     g = 0.5*(1-x)
 */
  __pyx_v_b = __pyx_v_n;

  /* "orthogonal_eval.pxd":254
 *     a = -n
 *     b = n
 *     c = 0.5             # <<<<<<<<<<<<<<
 *     g = 0.5*(1-x)
 *     return hyp2f1(a, b, c, g)
 */
  __pyx_v_c = 0.5;

  /* "orthogonal_eval.pxd":255
 *     b = n
 *     c = 0.5
 *     g = 0.5*(1-x)             # <<<<<<<<<<<<<<
 *     return hyp2f1(a, b, c, g)
 * 
 */
  __pyx_v_g = __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0.5, 0), __Pyx_c_diff_double(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_x));

  /* "orthogonal_eval.pxd":256
 *     c = 0.5
 *     g = 0.5*(1-x)
 *     return hyp2f1(a, b, c, g)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_chebyt_l(long k, double x) nogil:
 */
  __pyx_r = __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_g);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":247
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebyt(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":258
 *     return hyp2f1(a, b, c, g)
 * 
 * cdef inline double eval_chebyt_l(long k, double x) nogil:             # <<<<<<<<<<<<<<
 *     # Use Chebyshev T recurrence directly, see [MH]
 *     cdef long m
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt_l(long __pyx_v_k, double __pyx_v_x) {
  CYTHON_UNUSED long __pyx_v_m;
  double __pyx_v_b2;
  double __pyx_v_b1;
  double __pyx_v_b0;
  double __pyx_r;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;
  long __pyx_t_4;

  /* "orthogonal_eval.pxd":263
 *     cdef double b2, b1, b0
 * 
 *     if k < 0:             # <<<<<<<<<<<<<<
 *         # symmetry
 *         k = -k
 */
  __pyx_t_1 = ((__pyx_v_k < 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":265
 *     if k < 0:
 *         # symmetry
 *         k = -k             # <<<<<<<<<<<<<<
 * 
 *     b2 = 0
 */
    __pyx_v_k = (-__pyx_v_k);

    /* "orthogonal_eval.pxd":263
 *     cdef double b2, b1, b0
 * 
 *     if k < 0:             # <<<<<<<<<<<<<<
 *         # symmetry
 *         k = -k
 */
  }

  /* "orthogonal_eval.pxd":267
 *         k = -k
 * 
 *     b2 = 0             # <<<<<<<<<<<<<<
 *     b1 = -1
 *     b0 = 0
 */
  __pyx_v_b2 = 0.0;

  /* "orthogonal_eval.pxd":268
 * 
 *     b2 = 0
 *     b1 = -1             # <<<<<<<<<<<<<<
 *     b0 = 0
 *     x = 2*x
 */
  __pyx_v_b1 = -1.0;

  /* "orthogonal_eval.pxd":269
 *     b2 = 0
 *     b1 = -1
 *     b0 = 0             # <<<<<<<<<<<<<<
 *     x = 2*x
 *     for m in range(k+1):
 */
  __pyx_v_b0 = 0.0;

  /* "orthogonal_eval.pxd":270
 *     b1 = -1
 *     b0 = 0
 *     x = 2*x             # <<<<<<<<<<<<<<
 *     for m in range(k+1):
 *         b2 = b1
 */
  __pyx_v_x = (2.0 * __pyx_v_x);

  /* "orthogonal_eval.pxd":271
 *     b0 = 0
 *     x = 2*x
 *     for m in range(k+1):             # <<<<<<<<<<<<<<
 *         b2 = b1
 *         b1 = b0
 */
  __pyx_t_2 = (__pyx_v_k + 1);
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_m = __pyx_t_4;

    /* "orthogonal_eval.pxd":272
 *     x = 2*x
 *     for m in range(k+1):
 *         b2 = b1             # <<<<<<<<<<<<<<
 *         b1 = b0
 *         b0 = x*b1 - b2
 */
    __pyx_v_b2 = __pyx_v_b1;

    /* "orthogonal_eval.pxd":273
 *     for m in range(k+1):
 *         b2 = b1
 *         b1 = b0             # <<<<<<<<<<<<<<
 *         b0 = x*b1 - b2
 *     return (b0 - b2)/2.0
 */
    __pyx_v_b1 = __pyx_v_b0;

    /* "orthogonal_eval.pxd":274
 *         b2 = b1
 *         b1 = b0
 *         b0 = x*b1 - b2             # <<<<<<<<<<<<<<
 *     return (b0 - b2)/2.0
 * 
 */
    __pyx_v_b0 = ((__pyx_v_x * __pyx_v_b1) - __pyx_v_b2);
  }

  /* "orthogonal_eval.pxd":275
 *         b1 = b0
 *         b0 = x*b1 - b2
 *     return (b0 - b2)/2.0             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  __pyx_r = ((__pyx_v_b0 - __pyx_v_b2) / 2.0);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":258
 *     return hyp2f1(a, b, c, g)
 * 
 * cdef inline double eval_chebyt_l(long k, double x) nogil:             # <<<<<<<<<<<<<<
 *     # Use Chebyshev T recurrence directly, see [MH]
 *     cdef long m
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":281
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebyu(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_c;
  double __pyx_v_d;
  double __pyx_v_g;
  double __pyx_r;

  /* "orthogonal_eval.pxd":285
 *     cdef number_t g
 * 
 *     d = n+1             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = n+2
 */
  __pyx_v_d = (__pyx_v_n + 1.0);

  /* "orthogonal_eval.pxd":286
 * 
 *     d = n+1
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = n+2
 *     c = 1.5
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":287
 *     d = n+1
 *     a = -n
 *     b = n+2             # <<<<<<<<<<<<<<
 *     c = 1.5
 *     g = 0.5*(1-x)
 */
  __pyx_v_b = (__pyx_v_n + 2.0);

  /* "orthogonal_eval.pxd":288
 *     a = -n
 *     b = n+2
 *     c = 1.5             # <<<<<<<<<<<<<<
 *     g = 0.5*(1-x)
 *     return d*hyp2f1(a, b, c, g)
 */
  __pyx_v_c = 1.5;

  /* "orthogonal_eval.pxd":289
 *     b = n+2
 *     c = 1.5
 *     g = 0.5*(1-x)             # <<<<<<<<<<<<<<
 *     return d*hyp2f1(a, b, c, g)
 * 
 */
  __pyx_v_g = (0.5 * (1.0 - __pyx_v_x));

  /* "orthogonal_eval.pxd":290
 *     c = 1.5
 *     g = 0.5*(1-x)
 *     return d*hyp2f1(a, b, c, g)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_chebyu_l(long k, double x) nogil:
 */
  __pyx_r = (__pyx_v_d * __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_g));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":281
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebyu(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu(double __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_c;
  double __pyx_v_d;
  __pyx_t_double_complex __pyx_v_g;
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":285
 *     cdef number_t g
 * 
 *     d = n+1             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = n+2
 */
  __pyx_v_d = (__pyx_v_n + 1.0);

  /* "orthogonal_eval.pxd":286
 * 
 *     d = n+1
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = n+2
 *     c = 1.5
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":287
 *     d = n+1
 *     a = -n
 *     b = n+2             # <<<<<<<<<<<<<<
 *     c = 1.5
 *     g = 0.5*(1-x)
 */
  __pyx_v_b = (__pyx_v_n + 2.0);

  /* "orthogonal_eval.pxd":288
 *     a = -n
 *     b = n+2
 *     c = 1.5             # <<<<<<<<<<<<<<
 *     g = 0.5*(1-x)
 *     return d*hyp2f1(a, b, c, g)
 */
  __pyx_v_c = 1.5;

  /* "orthogonal_eval.pxd":289
 *     b = n+2
 *     c = 1.5
 *     g = 0.5*(1-x)             # <<<<<<<<<<<<<<
 *     return d*hyp2f1(a, b, c, g)
 * 
 */
  __pyx_v_g = __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0.5, 0), __Pyx_c_diff_double(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_x));

  /* "orthogonal_eval.pxd":290
 *     c = 1.5
 *     g = 0.5*(1-x)
 *     return d*hyp2f1(a, b, c, g)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_chebyu_l(long k, double x) nogil:
 */
  __pyx_r = __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(__pyx_v_d, 0), __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_g));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":281
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebyu(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":292
 *     return d*hyp2f1(a, b, c, g)
 * 
 * cdef inline double eval_chebyu_l(long k, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long m
 *     cdef int sign
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu_l(long __pyx_v_k, double __pyx_v_x) {
  CYTHON_UNUSED long __pyx_v_m;
  int __pyx_v_sign;
  double __pyx_v_b2;
  double __pyx_v_b1;
  double __pyx_v_b0;
  double __pyx_r;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;
  long __pyx_t_4;

  /* "orthogonal_eval.pxd":297
 *     cdef double b2, b1, b0
 * 
 *     if k == -1:             # <<<<<<<<<<<<<<
 *         return 0
 *     elif k < -1:
 */
  __pyx_t_1 = ((__pyx_v_k == -1L) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":298
 * 
 *     if k == -1:
 *         return 0             # <<<<<<<<<<<<<<
 *     elif k < -1:
 *         # symmetry
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":297
 *     cdef double b2, b1, b0
 * 
 *     if k == -1:             # <<<<<<<<<<<<<<
 *         return 0
 *     elif k < -1:
 */
  }

  /* "orthogonal_eval.pxd":299
 *     if k == -1:
 *         return 0
 *     elif k < -1:             # <<<<<<<<<<<<<<
 *         # symmetry
 *         k = -k - 2
 */
  __pyx_t_1 = ((__pyx_v_k < -1L) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":301
 *     elif k < -1:
 *         # symmetry
 *         k = -k - 2             # <<<<<<<<<<<<<<
 *         sign = -1
 *     else:
 */
    __pyx_v_k = ((-__pyx_v_k) - 2);

    /* "orthogonal_eval.pxd":302
 *         # symmetry
 *         k = -k - 2
 *         sign = -1             # <<<<<<<<<<<<<<
 *     else:
 *         sign = 1
 */
    __pyx_v_sign = -1;

    /* "orthogonal_eval.pxd":299
 *     if k == -1:
 *         return 0
 *     elif k < -1:             # <<<<<<<<<<<<<<
 *         # symmetry
 *         k = -k - 2
 */
    goto __pyx_L3;
  }

  /* "orthogonal_eval.pxd":304
 *         sign = -1
 *     else:
 *         sign = 1             # <<<<<<<<<<<<<<
 * 
 *     b2 = 0
 */
  /*else*/ {
    __pyx_v_sign = 1;
  }
  __pyx_L3:;

  /* "orthogonal_eval.pxd":306
 *         sign = 1
 * 
 *     b2 = 0             # <<<<<<<<<<<<<<
 *     b1 = -1
 *     b0 = 0
 */
  __pyx_v_b2 = 0.0;

  /* "orthogonal_eval.pxd":307
 * 
 *     b2 = 0
 *     b1 = -1             # <<<<<<<<<<<<<<
 *     b0 = 0
 *     x = 2*x
 */
  __pyx_v_b1 = -1.0;

  /* "orthogonal_eval.pxd":308
 *     b2 = 0
 *     b1 = -1
 *     b0 = 0             # <<<<<<<<<<<<<<
 *     x = 2*x
 *     for m in range(k+1):
 */
  __pyx_v_b0 = 0.0;

  /* "orthogonal_eval.pxd":309
 *     b1 = -1
 *     b0 = 0
 *     x = 2*x             # <<<<<<<<<<<<<<
 *     for m in range(k+1):
 *         b2 = b1
 */
  __pyx_v_x = (2.0 * __pyx_v_x);

  /* "orthogonal_eval.pxd":310
 *     b0 = 0
 *     x = 2*x
 *     for m in range(k+1):             # <<<<<<<<<<<<<<
 *         b2 = b1
 *         b1 = b0
 */
  __pyx_t_2 = (__pyx_v_k + 1);
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_m = __pyx_t_4;

    /* "orthogonal_eval.pxd":311
 *     x = 2*x
 *     for m in range(k+1):
 *         b2 = b1             # <<<<<<<<<<<<<<
 *         b1 = b0
 *         b0 = x*b1 - b2
 */
    __pyx_v_b2 = __pyx_v_b1;

    /* "orthogonal_eval.pxd":312
 *     for m in range(k+1):
 *         b2 = b1
 *         b1 = b0             # <<<<<<<<<<<<<<
 *         b0 = x*b1 - b2
 *     return b0 * sign
 */
    __pyx_v_b1 = __pyx_v_b0;

    /* "orthogonal_eval.pxd":313
 *         b2 = b1
 *         b1 = b0
 *         b0 = x*b1 - b2             # <<<<<<<<<<<<<<
 *     return b0 * sign
 * 
 */
    __pyx_v_b0 = ((__pyx_v_x * __pyx_v_b1) - __pyx_v_b2);
  }

  /* "orthogonal_eval.pxd":314
 *         b1 = b0
 *         b0 = x*b1 - b2
 *     return b0 * sign             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  __pyx_r = (__pyx_v_b0 * __pyx_v_sign);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":292
 *     return d*hyp2f1(a, b, c, g)
 * 
 * cdef inline double eval_chebyu_l(long k, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long m
 *     cdef int sign
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":320
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebys(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyu(n, 0.5*x)
 * 
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":321
 * 
 * cdef inline number_t eval_chebys(double n, number_t x) nogil:
 *     return eval_chebyu(n, 0.5*x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_chebys_l(long n, double x) nogil:
 */
  __pyx_r = __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu(__pyx_v_n, (0.5 * __pyx_v_x));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":320
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebys(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyu(n, 0.5*x)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys(double __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":321
 * 
 * cdef inline number_t eval_chebys(double n, number_t x) nogil:
 *     return eval_chebyu(n, 0.5*x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_chebys_l(long n, double x) nogil:
 */
  __pyx_r = __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu(__pyx_v_n, __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0.5, 0), __pyx_v_x));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":320
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebys(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyu(n, 0.5*x)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":323
 *     return eval_chebyu(n, 0.5*x)
 * 
 * cdef inline double eval_chebys_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyu_l(n, 0.5*x)
 * 
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys_l(long __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":324
 * 
 * cdef inline double eval_chebys_l(long n, double x) nogil:
 *     return eval_chebyu_l(n, 0.5*x)             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  __pyx_r = __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu_l(__pyx_v_n, (0.5 * __pyx_v_x));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":323
 *     return eval_chebyu(n, 0.5*x)
 * 
 * cdef inline double eval_chebys_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyu_l(n, 0.5*x)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":330
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebyc(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return 2*eval_chebyt(n, 0.5*x)
 * 
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":331
 * 
 * cdef inline number_t eval_chebyc(double n, number_t x) nogil:
 *     return 2*eval_chebyt(n, 0.5*x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_chebyc_l(long n, double x) nogil:
 */
  __pyx_r = (2.0 * __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt(__pyx_v_n, (0.5 * __pyx_v_x)));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":330
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebyc(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return 2*eval_chebyt(n, 0.5*x)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc(double __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":331
 * 
 * cdef inline number_t eval_chebyc(double n, number_t x) nogil:
 *     return 2*eval_chebyt(n, 0.5*x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_chebyc_l(long n, double x) nogil:
 */
  __pyx_r = __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(2, 0), __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt(__pyx_v_n, __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0.5, 0), __pyx_v_x)));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":330
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_chebyc(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return 2*eval_chebyt(n, 0.5*x)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":333
 *     return 2*eval_chebyt(n, 0.5*x)
 * 
 * cdef inline double eval_chebyc_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return 2*eval_chebyt_l(n, 0.5*x)
 * 
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc_l(long __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":334
 * 
 * cdef inline double eval_chebyc_l(long n, double x) nogil:
 *     return 2*eval_chebyt_l(n, 0.5*x)             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  __pyx_r = (2.0 * __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt_l(__pyx_v_n, (0.5 * __pyx_v_x)));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":333
 *     return 2*eval_chebyt(n, 0.5*x)
 * 
 * cdef inline double eval_chebyc_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return 2*eval_chebyt_l(n, 0.5*x)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":340
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_sh_chebyt(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyt(n, 2*x-1)
 * 
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":341
 * 
 * cdef inline number_t eval_sh_chebyt(double n, number_t x) nogil:
 *     return eval_chebyt(n, 2*x-1)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_sh_chebyt_l(long n, double x) nogil:
 */
  __pyx_r = __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt(__pyx_v_n, ((2.0 * __pyx_v_x) - 1.0));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":340
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_sh_chebyt(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyt(n, 2*x-1)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt(double __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":341
 * 
 * cdef inline number_t eval_sh_chebyt(double n, number_t x) nogil:
 *     return eval_chebyt(n, 2*x-1)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_sh_chebyt_l(long n, double x) nogil:
 */
  __pyx_r = __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt(__pyx_v_n, __Pyx_c_diff_double(__Pyx_c_prod_double(__pyx_t_double_complex_from_parts(2, 0), __pyx_v_x), __pyx_t_double_complex_from_parts(1, 0)));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":340
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_sh_chebyt(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyt(n, 2*x-1)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":343
 *     return eval_chebyt(n, 2*x-1)
 * 
 * cdef inline double eval_sh_chebyt_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyt_l(n, 2*x-1)
 * 
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt_l(long __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":344
 * 
 * cdef inline double eval_sh_chebyt_l(long n, double x) nogil:
 *     return eval_chebyt_l(n, 2*x-1)             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  __pyx_r = __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt_l(__pyx_v_n, ((2.0 * __pyx_v_x) - 1.0));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":343
 *     return eval_chebyt(n, 2*x-1)
 * 
 * cdef inline double eval_sh_chebyt_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyt_l(n, 2*x-1)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":350
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_sh_chebyu(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyu(n, 2*x-1)
 * 
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":351
 * 
 * cdef inline number_t eval_sh_chebyu(double n, number_t x) nogil:
 *     return eval_chebyu(n, 2*x-1)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_sh_chebyu_l(long n, double x) nogil:
 */
  __pyx_r = __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu(__pyx_v_n, ((2.0 * __pyx_v_x) - 1.0));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":350
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_sh_chebyu(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyu(n, 2*x-1)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu(double __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":351
 * 
 * cdef inline number_t eval_sh_chebyu(double n, number_t x) nogil:
 *     return eval_chebyu(n, 2*x-1)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_sh_chebyu_l(long n, double x) nogil:
 */
  __pyx_r = __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu(__pyx_v_n, __Pyx_c_diff_double(__Pyx_c_prod_double(__pyx_t_double_complex_from_parts(2, 0), __pyx_v_x), __pyx_t_double_complex_from_parts(1, 0)));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":350
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_sh_chebyu(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyu(n, 2*x-1)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":353
 *     return eval_chebyu(n, 2*x-1)
 * 
 * cdef inline double eval_sh_chebyu_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyu_l(n, 2*x-1)
 * 
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu_l(long __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":354
 * 
 * cdef inline double eval_sh_chebyu_l(long n, double x) nogil:
 *     return eval_chebyu_l(n, 2*x-1)             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  __pyx_r = __pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu_l(__pyx_v_n, ((2.0 * __pyx_v_x) - 1.0));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":353
 *     return eval_chebyu(n, 2*x-1)
 * 
 * cdef inline double eval_sh_chebyu_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_chebyu_l(n, 2*x-1)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":360
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_legendre(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_c;
  double __pyx_v_d;
  double __pyx_v_g;
  double __pyx_r;

  /* "orthogonal_eval.pxd":364
 *     cdef number_t g
 * 
 *     d = 1             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = n+1
 */
  __pyx_v_d = 1.0;

  /* "orthogonal_eval.pxd":365
 * 
 *     d = 1
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = n+1
 *     c = 1
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":366
 *     d = 1
 *     a = -n
 *     b = n+1             # <<<<<<<<<<<<<<
 *     c = 1
 *     g = 0.5*(1-x)
 */
  __pyx_v_b = (__pyx_v_n + 1.0);

  /* "orthogonal_eval.pxd":367
 *     a = -n
 *     b = n+1
 *     c = 1             # <<<<<<<<<<<<<<
 *     g = 0.5*(1-x)
 *     return d*hyp2f1(a, b, c, g)
 */
  __pyx_v_c = 1.0;

  /* "orthogonal_eval.pxd":368
 *     b = n+1
 *     c = 1
 *     g = 0.5*(1-x)             # <<<<<<<<<<<<<<
 *     return d*hyp2f1(a, b, c, g)
 * 
 */
  __pyx_v_g = (0.5 * (1.0 - __pyx_v_x));

  /* "orthogonal_eval.pxd":369
 *     c = 1
 *     g = 0.5*(1-x)
 *     return d*hyp2f1(a, b, c, g)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = (__pyx_v_d * __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_g));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":360
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_legendre(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre(double __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_c;
  double __pyx_v_d;
  __pyx_t_double_complex __pyx_v_g;
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":364
 *     cdef number_t g
 * 
 *     d = 1             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = n+1
 */
  __pyx_v_d = 1.0;

  /* "orthogonal_eval.pxd":365
 * 
 *     d = 1
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = n+1
 *     c = 1
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":366
 *     d = 1
 *     a = -n
 *     b = n+1             # <<<<<<<<<<<<<<
 *     c = 1
 *     g = 0.5*(1-x)
 */
  __pyx_v_b = (__pyx_v_n + 1.0);

  /* "orthogonal_eval.pxd":367
 *     a = -n
 *     b = n+1
 *     c = 1             # <<<<<<<<<<<<<<
 *     g = 0.5*(1-x)
 *     return d*hyp2f1(a, b, c, g)
 */
  __pyx_v_c = 1.0;

  /* "orthogonal_eval.pxd":368
 *     b = n+1
 *     c = 1
 *     g = 0.5*(1-x)             # <<<<<<<<<<<<<<
 *     return d*hyp2f1(a, b, c, g)
 * 
 */
  __pyx_v_g = __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0.5, 0), __Pyx_c_diff_double(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_x));

  /* "orthogonal_eval.pxd":369
 *     c = 1
 *     g = 0.5*(1-x)
 *     return d*hyp2f1(a, b, c, g)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(__pyx_v_d, 0), __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_hyp2f1(__pyx_v_a, __pyx_v_b, __pyx_v_c, __pyx_v_g));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":360
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_legendre(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, c, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":372
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_legendre_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long kk, a
 *     cdef double p, d
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre_l(long __pyx_v_n, double __pyx_v_x) {
  long __pyx_v_kk;
  long __pyx_v_a;
  double __pyx_v_p;
  double __pyx_v_d;
  double __pyx_v_k;
  double __pyx_r;
  int __pyx_t_1;
  double __pyx_t_2;
  long __pyx_t_3;
  long __pyx_t_4;
  long __pyx_t_5;

  /* "orthogonal_eval.pxd":377
 *     cdef double k
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         # symmetry
 *         n = -n - 1
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":379
 *     if n < 0:
 *         # symmetry
 *         n = -n - 1             # <<<<<<<<<<<<<<
 * 
 *     if n == 0:
 */
    __pyx_v_n = ((-__pyx_v_n) - 1);

    /* "orthogonal_eval.pxd":377
 *     cdef double k
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         # symmetry
 *         n = -n - 1
 */
  }

  /* "orthogonal_eval.pxd":381
 *         n = -n - 1
 * 
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif n == 1:
 */
  __pyx_t_1 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":382
 * 
 *     if n == 0:
 *         return 1.0             # <<<<<<<<<<<<<<
 *     elif n == 1:
 *         return x
 */
    __pyx_r = 1.0;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":381
 *         n = -n - 1
 * 
 *     if n == 0:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif n == 1:
 */
  }

  /* "orthogonal_eval.pxd":383
 *     if n == 0:
 *         return 1.0
 *     elif n == 1:             # <<<<<<<<<<<<<<
 *         return x
 *     elif fabs(x) < 1e-5:
 */
  __pyx_t_1 = ((__pyx_v_n == 1) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":384
 *         return 1.0
 *     elif n == 1:
 *         return x             # <<<<<<<<<<<<<<
 *     elif fabs(x) < 1e-5:
 *         # Power series rather than recurrence due to loss of precision
 */
    __pyx_r = __pyx_v_x;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":383
 *     if n == 0:
 *         return 1.0
 *     elif n == 1:             # <<<<<<<<<<<<<<
 *         return x
 *     elif fabs(x) < 1e-5:
 */
  }

  /* "orthogonal_eval.pxd":385
 *     elif n == 1:
 *         return x
 *     elif fabs(x) < 1e-5:             # <<<<<<<<<<<<<<
 *         # Power series rather than recurrence due to loss of precision
 *         # http://functions.wolfram.com/Polynomials/LegendreP/02/
 */
  __pyx_t_1 = ((fabs(__pyx_v_x) < 1e-5) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":388
 *         # Power series rather than recurrence due to loss of precision
 *         # http://functions.wolfram.com/Polynomials/LegendreP/02/
 *         a = n//2             # <<<<<<<<<<<<<<
 * 
 *         d = 1 if a % 2 == 0 else -1
 */
    __pyx_v_a = __Pyx_div_long(__pyx_v_n, 2);

    /* "orthogonal_eval.pxd":390
 *         a = n//2
 * 
 *         d = 1 if a % 2 == 0 else -1             # <<<<<<<<<<<<<<
 *         if n == 2*a:
 *             d *= -2 / beta(a + 1, -0.5)
 */
    if ((((__pyx_v_a % 2) == 0) != 0)) {
      __pyx_t_2 = 1.0;
    } else {
      __pyx_t_2 = -1.0;
    }
    __pyx_v_d = __pyx_t_2;

    /* "orthogonal_eval.pxd":391
 * 
 *         d = 1 if a % 2 == 0 else -1
 *         if n == 2*a:             # <<<<<<<<<<<<<<
 *             d *= -2 / beta(a + 1, -0.5)
 *         else:
 */
    __pyx_t_1 = ((__pyx_v_n == (2 * __pyx_v_a)) != 0);
    if (__pyx_t_1) {

      /* "orthogonal_eval.pxd":392
 *         d = 1 if a % 2 == 0 else -1
 *         if n == 2*a:
 *             d *= -2 / beta(a + 1, -0.5)             # <<<<<<<<<<<<<<
 *         else:
 *             d *= 2 * x / beta(a + 1, 0.5)
 */
      __pyx_v_d = (__pyx_v_d * (-2.0 / beta((__pyx_v_a + 1), -0.5)));

      /* "orthogonal_eval.pxd":391
 * 
 *         d = 1 if a % 2 == 0 else -1
 *         if n == 2*a:             # <<<<<<<<<<<<<<
 *             d *= -2 / beta(a + 1, -0.5)
 *         else:
 */
      goto __pyx_L5;
    }

    /* "orthogonal_eval.pxd":394
 *             d *= -2 / beta(a + 1, -0.5)
 *         else:
 *             d *= 2 * x / beta(a + 1, 0.5)             # <<<<<<<<<<<<<<
 * 
 *         p = 0
 */
    /*else*/ {
      __pyx_v_d = (__pyx_v_d * ((2.0 * __pyx_v_x) / beta((__pyx_v_a + 1), 0.5)));
    }
    __pyx_L5:;

    /* "orthogonal_eval.pxd":396
 *             d *= 2 * x / beta(a + 1, 0.5)
 * 
 *         p = 0             # <<<<<<<<<<<<<<
 *         for kk in range(a+1):
 *             p += d
 */
    __pyx_v_p = 0.0;

    /* "orthogonal_eval.pxd":397
 * 
 *         p = 0
 *         for kk in range(a+1):             # <<<<<<<<<<<<<<
 *             p += d
 *             d *= -2 * x**2 * (a - kk) * (2*n + 1 - 2*a + 2*kk) / (
 */
    __pyx_t_3 = (__pyx_v_a + 1);
    __pyx_t_4 = __pyx_t_3;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_kk = __pyx_t_5;

      /* "orthogonal_eval.pxd":398
 *         p = 0
 *         for kk in range(a+1):
 *             p += d             # <<<<<<<<<<<<<<
 *             d *= -2 * x**2 * (a - kk) * (2*n + 1 - 2*a + 2*kk) / (
 *                 (n + 1 - 2*a + 2*kk) * (n + 2 - 2*a + 2*kk))
 */
      __pyx_v_p = (__pyx_v_p + __pyx_v_d);

      /* "orthogonal_eval.pxd":399
 *         for kk in range(a+1):
 *             p += d
 *             d *= -2 * x**2 * (a - kk) * (2*n + 1 - 2*a + 2*kk) / (             # <<<<<<<<<<<<<<
 *                 (n + 1 - 2*a + 2*kk) * (n + 2 - 2*a + 2*kk))
 *             if fabs(d) == 1e-20*fabs(p):
 */
      __pyx_v_d = (__pyx_v_d * ((((-2.0 * pow(__pyx_v_x, 2.0)) * (__pyx_v_a - __pyx_v_kk)) * ((((2 * __pyx_v_n) + 1) - (2 * __pyx_v_a)) + (2 * __pyx_v_kk))) / ((double)((((__pyx_v_n + 1) - (2 * __pyx_v_a)) + (2 * __pyx_v_kk)) * (((__pyx_v_n + 2) - (2 * __pyx_v_a)) + (2 * __pyx_v_kk))))));

      /* "orthogonal_eval.pxd":401
 *             d *= -2 * x**2 * (a - kk) * (2*n + 1 - 2*a + 2*kk) / (
 *                 (n + 1 - 2*a + 2*kk) * (n + 2 - 2*a + 2*kk))
 *             if fabs(d) == 1e-20*fabs(p):             # <<<<<<<<<<<<<<
 *                 # converged
 *                 break
 */
      __pyx_t_1 = ((fabs(__pyx_v_d) == (1e-20 * fabs(__pyx_v_p))) != 0);
      if (__pyx_t_1) {

        /* "orthogonal_eval.pxd":403
 *             if fabs(d) == 1e-20*fabs(p):
 *                 # converged
 *                 break             # <<<<<<<<<<<<<<
 *         return p
 *     else:
 */
        goto __pyx_L7_break;

        /* "orthogonal_eval.pxd":401
 *             d *= -2 * x**2 * (a - kk) * (2*n + 1 - 2*a + 2*kk) / (
 *                 (n + 1 - 2*a + 2*kk) * (n + 2 - 2*a + 2*kk))
 *             if fabs(d) == 1e-20*fabs(p):             # <<<<<<<<<<<<<<
 *                 # converged
 *                 break
 */
      }
    }
    __pyx_L7_break:;

    /* "orthogonal_eval.pxd":404
 *                 # converged
 *                 break
 *         return p             # <<<<<<<<<<<<<<
 *     else:
 *         d = x - 1
 */
    __pyx_r = __pyx_v_p;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":385
 *     elif n == 1:
 *         return x
 *     elif fabs(x) < 1e-5:             # <<<<<<<<<<<<<<
 *         # Power series rather than recurrence due to loss of precision
 *         # http://functions.wolfram.com/Polynomials/LegendreP/02/
 */
  }

  /* "orthogonal_eval.pxd":406
 *         return p
 *     else:
 *         d = x - 1             # <<<<<<<<<<<<<<
 *         p = x
 *         for kk in range(n-1):
 */
  /*else*/ {
    __pyx_v_d = (__pyx_v_x - 1.0);

    /* "orthogonal_eval.pxd":407
 *     else:
 *         d = x - 1
 *         p = x             # <<<<<<<<<<<<<<
 *         for kk in range(n-1):
 *             k = kk+1.0
 */
    __pyx_v_p = __pyx_v_x;

    /* "orthogonal_eval.pxd":408
 *         d = x - 1
 *         p = x
 *         for kk in range(n-1):             # <<<<<<<<<<<<<<
 *             k = kk+1.0
 *             d = ((2*k+1)/(k+1))*(x-1)*p + (k/(k+1)) * d
 */
    __pyx_t_3 = (__pyx_v_n - 1);
    __pyx_t_4 = __pyx_t_3;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_kk = __pyx_t_5;

      /* "orthogonal_eval.pxd":409
 *         p = x
 *         for kk in range(n-1):
 *             k = kk+1.0             # <<<<<<<<<<<<<<
 *             d = ((2*k+1)/(k+1))*(x-1)*p + (k/(k+1)) * d
 *             p = d + p
 */
      __pyx_v_k = (__pyx_v_kk + 1.0);

      /* "orthogonal_eval.pxd":410
 *         for kk in range(n-1):
 *             k = kk+1.0
 *             d = ((2*k+1)/(k+1))*(x-1)*p + (k/(k+1)) * d             # <<<<<<<<<<<<<<
 *             p = d + p
 *         return p
 */
      __pyx_v_d = ((((((2.0 * __pyx_v_k) + 1.0) / (__pyx_v_k + 1.0)) * (__pyx_v_x - 1.0)) * __pyx_v_p) + ((__pyx_v_k / (__pyx_v_k + 1.0)) * __pyx_v_d));

      /* "orthogonal_eval.pxd":411
 *             k = kk+1.0
 *             d = ((2*k+1)/(k+1))*(x-1)*p + (k/(k+1)) * d
 *             p = d + p             # <<<<<<<<<<<<<<
 *         return p
 * 
 */
      __pyx_v_p = (__pyx_v_d + __pyx_v_p);
    }

    /* "orthogonal_eval.pxd":412
 *             d = ((2*k+1)/(k+1))*(x-1)*p + (k/(k+1)) * d
 *             p = d + p
 *         return p             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
    __pyx_r = __pyx_v_p;
    goto __pyx_L0;
  }

  /* "orthogonal_eval.pxd":372
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_legendre_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long kk, a
 *     cdef double p, d
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":418
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_sh_legendre(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_legendre(n, 2*x-1)
 * 
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":419
 * 
 * cdef inline number_t eval_sh_legendre(double n, number_t x) nogil:
 *     return eval_legendre(n, 2*x-1)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_sh_legendre_l(long n, double x) nogil:
 */
  __pyx_r = __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre(__pyx_v_n, ((2.0 * __pyx_v_x) - 1.0));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":418
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_sh_legendre(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_legendre(n, 2*x-1)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre(double __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":419
 * 
 * cdef inline number_t eval_sh_legendre(double n, number_t x) nogil:
 *     return eval_legendre(n, 2*x-1)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_sh_legendre_l(long n, double x) nogil:
 */
  __pyx_r = __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre(__pyx_v_n, __Pyx_c_diff_double(__Pyx_c_prod_double(__pyx_t_double_complex_from_parts(2, 0), __pyx_v_x), __pyx_t_double_complex_from_parts(1, 0)));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":418
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_sh_legendre(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_legendre(n, 2*x-1)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":421
 *     return eval_legendre(n, 2*x-1)
 * 
 * cdef inline double eval_sh_legendre_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_legendre_l(n, 2*x-1)
 * 
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre_l(long __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":422
 * 
 * cdef inline double eval_sh_legendre_l(long n, double x) nogil:
 *     return eval_legendre_l(n, 2*x-1)             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  __pyx_r = __pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre_l(__pyx_v_n, ((2.0 * __pyx_v_x) - 1.0));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":421
 *     return eval_legendre(n, 2*x-1)
 * 
 * cdef inline double eval_sh_legendre_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_legendre_l(n, 2*x-1)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":428
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_genlaguerre(double n, double alpha, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, d
 *     cdef number_t g
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre(double __pyx_v_n, double __pyx_v_alpha, double __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_d;
  double __pyx_v_g;
  double __pyx_r;
  int __pyx_t_1;

  /* "orthogonal_eval.pxd":432
 *     cdef number_t g
 * 
 *     if alpha <= -1:             # <<<<<<<<<<<<<<
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,
 *                        "polynomial defined only for alpha > -1")
 */
  __pyx_t_1 = ((__pyx_v_alpha <= -1.0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":433
 * 
 *     if alpha <= -1:
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,             # <<<<<<<<<<<<<<
 *                        "polynomial defined only for alpha > -1")
 *         return nan
 */
    sf_error(((char *)"eval_genlaguerre"), SF_ERROR_DOMAIN, ((char *)"polynomial defined only for alpha > -1"));

    /* "orthogonal_eval.pxd":435
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,
 *                        "polynomial defined only for alpha > -1")
 *         return nan             # <<<<<<<<<<<<<<
 * 
 *     d = binom(n+alpha, n)
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":432
 *     cdef number_t g
 * 
 *     if alpha <= -1:             # <<<<<<<<<<<<<<
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,
 *                        "polynomial defined only for alpha > -1")
 */
  }

  /* "orthogonal_eval.pxd":437
 *         return nan
 * 
 *     d = binom(n+alpha, n)             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = alpha + 1
 */
  __pyx_v_d = __pyx_f_5scipy_7special_15orthogonal_eval_binom((__pyx_v_n + __pyx_v_alpha), __pyx_v_n);

  /* "orthogonal_eval.pxd":438
 * 
 *     d = binom(n+alpha, n)
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = alpha + 1
 *     g = x
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":439
 *     d = binom(n+alpha, n)
 *     a = -n
 *     b = alpha + 1             # <<<<<<<<<<<<<<
 *     g = x
 *     return d * hyp1f1(a, b, g)
 */
  __pyx_v_b = (__pyx_v_alpha + 1.0);

  /* "orthogonal_eval.pxd":440
 *     a = -n
 *     b = alpha + 1
 *     g = x             # <<<<<<<<<<<<<<
 *     return d * hyp1f1(a, b, g)
 * 
 */
  __pyx_v_g = __pyx_v_x;

  /* "orthogonal_eval.pxd":441
 *     b = alpha + 1
 *     g = x
 *     return d * hyp1f1(a, b, g)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = (__pyx_v_d * __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_hyp1f1(__pyx_v_a, __pyx_v_b, __pyx_v_g));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":428
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_genlaguerre(double n, double alpha, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre(double __pyx_v_n, double __pyx_v_alpha, __pyx_t_double_complex __pyx_v_x) {
  double __pyx_v_a;
  double __pyx_v_b;
  double __pyx_v_d;
  __pyx_t_double_complex __pyx_v_g;
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;

  /* "orthogonal_eval.pxd":432
 *     cdef number_t g
 * 
 *     if alpha <= -1:             # <<<<<<<<<<<<<<
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,
 *                        "polynomial defined only for alpha > -1")
 */
  __pyx_t_1 = ((__pyx_v_alpha <= -1.0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":433
 * 
 *     if alpha <= -1:
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,             # <<<<<<<<<<<<<<
 *                        "polynomial defined only for alpha > -1")
 *         return nan
 */
    sf_error(((char *)"eval_genlaguerre"), SF_ERROR_DOMAIN, ((char *)"polynomial defined only for alpha > -1"));

    /* "orthogonal_eval.pxd":435
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,
 *                        "polynomial defined only for alpha > -1")
 *         return nan             # <<<<<<<<<<<<<<
 * 
 *     d = binom(n+alpha, n)
 */
    __pyx_r = __pyx_t_double_complex_from_parts(NPY_NAN, 0);
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":432
 *     cdef number_t g
 * 
 *     if alpha <= -1:             # <<<<<<<<<<<<<<
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,
 *                        "polynomial defined only for alpha > -1")
 */
  }

  /* "orthogonal_eval.pxd":437
 *         return nan
 * 
 *     d = binom(n+alpha, n)             # <<<<<<<<<<<<<<
 *     a = -n
 *     b = alpha + 1
 */
  __pyx_v_d = __pyx_f_5scipy_7special_15orthogonal_eval_binom((__pyx_v_n + __pyx_v_alpha), __pyx_v_n);

  /* "orthogonal_eval.pxd":438
 * 
 *     d = binom(n+alpha, n)
 *     a = -n             # <<<<<<<<<<<<<<
 *     b = alpha + 1
 *     g = x
 */
  __pyx_v_a = (-__pyx_v_n);

  /* "orthogonal_eval.pxd":439
 *     d = binom(n+alpha, n)
 *     a = -n
 *     b = alpha + 1             # <<<<<<<<<<<<<<
 *     g = x
 *     return d * hyp1f1(a, b, g)
 */
  __pyx_v_b = (__pyx_v_alpha + 1.0);

  /* "orthogonal_eval.pxd":440
 *     a = -n
 *     b = alpha + 1
 *     g = x             # <<<<<<<<<<<<<<
 *     return d * hyp1f1(a, b, g)
 * 
 */
  __pyx_v_g = __pyx_v_x;

  /* "orthogonal_eval.pxd":441
 *     b = alpha + 1
 *     g = x
 *     return d * hyp1f1(a, b, g)             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  __pyx_r = __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(__pyx_v_d, 0), __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_hyp1f1(__pyx_v_a, __pyx_v_b, __pyx_v_g));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":428
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_genlaguerre(double n, double alpha, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     cdef double a, b, d
 *     cdef number_t g
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":444
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_genlaguerre_l(long n, double alpha, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long kk
 *     cdef double p, d
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre_l(long __pyx_v_n, double __pyx_v_alpha, double __pyx_v_x) {
  long __pyx_v_kk;
  double __pyx_v_p;
  double __pyx_v_d;
  double __pyx_v_k;
  double __pyx_r;
  int __pyx_t_1;
  long __pyx_t_2;
  long __pyx_t_3;
  long __pyx_t_4;

  /* "orthogonal_eval.pxd":449
 *     cdef double k
 * 
 *     if alpha <= -1:             # <<<<<<<<<<<<<<
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,
 *                        "polynomial defined only for alpha > -1")
 */
  __pyx_t_1 = ((__pyx_v_alpha <= -1.0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":450
 * 
 *     if alpha <= -1:
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,             # <<<<<<<<<<<<<<
 *                        "polynomial defined only for alpha > -1")
 *         return nan
 */
    sf_error(((char *)"eval_genlaguerre"), SF_ERROR_DOMAIN, ((char *)"polynomial defined only for alpha > -1"));

    /* "orthogonal_eval.pxd":452
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,
 *                        "polynomial defined only for alpha > -1")
 *         return nan             # <<<<<<<<<<<<<<
 * 
 *     if n < 0:
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":449
 *     cdef double k
 * 
 *     if alpha <= -1:             # <<<<<<<<<<<<<<
 *         sf_error.error("eval_genlaguerre", sf_error.DOMAIN,
 *                        "polynomial defined only for alpha > -1")
 */
  }

  /* "orthogonal_eval.pxd":454
 *         return nan
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         return 0.0
 *     elif n == 0:
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":455
 * 
 *     if n < 0:
 *         return 0.0             # <<<<<<<<<<<<<<
 *     elif n == 0:
 *         return 1.0
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":454
 *         return nan
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         return 0.0
 *     elif n == 0:
 */
  }

  /* "orthogonal_eval.pxd":456
 *     if n < 0:
 *         return 0.0
 *     elif n == 0:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif n == 1:
 */
  __pyx_t_1 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":457
 *         return 0.0
 *     elif n == 0:
 *         return 1.0             # <<<<<<<<<<<<<<
 *     elif n == 1:
 *         return -x+alpha+1
 */
    __pyx_r = 1.0;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":456
 *     if n < 0:
 *         return 0.0
 *     elif n == 0:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif n == 1:
 */
  }

  /* "orthogonal_eval.pxd":458
 *     elif n == 0:
 *         return 1.0
 *     elif n == 1:             # <<<<<<<<<<<<<<
 *         return -x+alpha+1
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_n == 1) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":459
 *         return 1.0
 *     elif n == 1:
 *         return -x+alpha+1             # <<<<<<<<<<<<<<
 *     else:
 *         d = -x/(alpha+1)
 */
    __pyx_r = (((-__pyx_v_x) + __pyx_v_alpha) + 1.0);
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":458
 *     elif n == 0:
 *         return 1.0
 *     elif n == 1:             # <<<<<<<<<<<<<<
 *         return -x+alpha+1
 *     else:
 */
  }

  /* "orthogonal_eval.pxd":461
 *         return -x+alpha+1
 *     else:
 *         d = -x/(alpha+1)             # <<<<<<<<<<<<<<
 *         p = d + 1
 *         for kk in range(n-1):
 */
  /*else*/ {
    __pyx_v_d = ((-__pyx_v_x) / (__pyx_v_alpha + 1.0));

    /* "orthogonal_eval.pxd":462
 *     else:
 *         d = -x/(alpha+1)
 *         p = d + 1             # <<<<<<<<<<<<<<
 *         for kk in range(n-1):
 *             k = kk+1.0
 */
    __pyx_v_p = (__pyx_v_d + 1.0);

    /* "orthogonal_eval.pxd":463
 *         d = -x/(alpha+1)
 *         p = d + 1
 *         for kk in range(n-1):             # <<<<<<<<<<<<<<
 *             k = kk+1.0
 *             d = -x/(k+alpha+1)*p + (k/(k+alpha+1)) * d
 */
    __pyx_t_2 = (__pyx_v_n - 1);
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_kk = __pyx_t_4;

      /* "orthogonal_eval.pxd":464
 *         p = d + 1
 *         for kk in range(n-1):
 *             k = kk+1.0             # <<<<<<<<<<<<<<
 *             d = -x/(k+alpha+1)*p + (k/(k+alpha+1)) * d
 *             p = d + p
 */
      __pyx_v_k = (__pyx_v_kk + 1.0);

      /* "orthogonal_eval.pxd":465
 *         for kk in range(n-1):
 *             k = kk+1.0
 *             d = -x/(k+alpha+1)*p + (k/(k+alpha+1)) * d             # <<<<<<<<<<<<<<
 *             p = d + p
 *         return binom(n+alpha, n)*p
 */
      __pyx_v_d = ((((-__pyx_v_x) / ((__pyx_v_k + __pyx_v_alpha) + 1.0)) * __pyx_v_p) + ((__pyx_v_k / ((__pyx_v_k + __pyx_v_alpha) + 1.0)) * __pyx_v_d));

      /* "orthogonal_eval.pxd":466
 *             k = kk+1.0
 *             d = -x/(k+alpha+1)*p + (k/(k+alpha+1)) * d
 *             p = d + p             # <<<<<<<<<<<<<<
 *         return binom(n+alpha, n)*p
 * 
 */
      __pyx_v_p = (__pyx_v_d + __pyx_v_p);
    }

    /* "orthogonal_eval.pxd":467
 *             d = -x/(k+alpha+1)*p + (k/(k+alpha+1)) * d
 *             p = d + p
 *         return binom(n+alpha, n)*p             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
    __pyx_r = (__pyx_f_5scipy_7special_15orthogonal_eval_binom((__pyx_v_n + __pyx_v_alpha), __pyx_v_n) * __pyx_v_p);
    goto __pyx_L0;
  }

  /* "orthogonal_eval.pxd":444
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_genlaguerre_l(long n, double alpha, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long kk
 *     cdef double p, d
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":473
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_laguerre(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_genlaguerre(n, 0., x)
 * 
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre(double __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":474
 * 
 * cdef inline number_t eval_laguerre(double n, number_t x) nogil:
 *     return eval_genlaguerre(n, 0., x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_laguerre_l(long n, double x) nogil:
 */
  __pyx_r = __pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre(__pyx_v_n, 0., __pyx_v_x);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":473
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_laguerre(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_genlaguerre(n, 0., x)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre(double __pyx_v_n, __pyx_t_double_complex __pyx_v_x) {
  __pyx_t_double_complex __pyx_r;

  /* "orthogonal_eval.pxd":474
 * 
 * cdef inline number_t eval_laguerre(double n, number_t x) nogil:
 *     return eval_genlaguerre(n, 0., x)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double eval_laguerre_l(long n, double x) nogil:
 */
  __pyx_r = __pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre(__pyx_v_n, 0., __pyx_v_x);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":473
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline number_t eval_laguerre(double n, number_t x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_genlaguerre(n, 0., x)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":476
 *     return eval_genlaguerre(n, 0., x)
 * 
 * cdef inline double eval_laguerre_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_genlaguerre_l(n, 0., x)
 * 
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre_l(long __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":477
 * 
 * cdef inline double eval_laguerre_l(long n, double x) nogil:
 *     return eval_genlaguerre_l(n, 0., x)             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
  __pyx_r = __pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre_l(__pyx_v_n, 0., __pyx_v_x);
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":476
 *     return eval_genlaguerre(n, 0., x)
 * 
 * cdef inline double eval_laguerre_l(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_genlaguerre_l(n, 0., x)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":483
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline double eval_hermitenorm(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long k
 *     cdef double y1, y2, y3
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_hermitenorm(long __pyx_v_n, double __pyx_v_x) {
  long __pyx_v_k;
  double __pyx_v_y1;
  double __pyx_v_y2;
  double __pyx_v_y3;
  double __pyx_r;
  int __pyx_t_1;
  long __pyx_t_2;

  /* "orthogonal_eval.pxd":487
 *     cdef double y1, y2, y3
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         return 0.0
 *     elif n == 0:
 */
  __pyx_t_1 = ((__pyx_v_n < 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":488
 * 
 *     if n < 0:
 *         return 0.0             # <<<<<<<<<<<<<<
 *     elif n == 0:
 *         return 1.0
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":487
 *     cdef double y1, y2, y3
 * 
 *     if n < 0:             # <<<<<<<<<<<<<<
 *         return 0.0
 *     elif n == 0:
 */
  }

  /* "orthogonal_eval.pxd":489
 *     if n < 0:
 *         return 0.0
 *     elif n == 0:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif n == 1:
 */
  __pyx_t_1 = ((__pyx_v_n == 0) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":490
 *         return 0.0
 *     elif n == 0:
 *         return 1.0             # <<<<<<<<<<<<<<
 *     elif n == 1:
 *         return x
 */
    __pyx_r = 1.0;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":489
 *     if n < 0:
 *         return 0.0
 *     elif n == 0:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif n == 1:
 */
  }

  /* "orthogonal_eval.pxd":491
 *     elif n == 0:
 *         return 1.0
 *     elif n == 1:             # <<<<<<<<<<<<<<
 *         return x
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_n == 1) != 0);
  if (__pyx_t_1) {

    /* "orthogonal_eval.pxd":492
 *         return 1.0
 *     elif n == 1:
 *         return x             # <<<<<<<<<<<<<<
 *     else:
 *         y3 = 0.0
 */
    __pyx_r = __pyx_v_x;
    goto __pyx_L0;

    /* "orthogonal_eval.pxd":491
 *     elif n == 0:
 *         return 1.0
 *     elif n == 1:             # <<<<<<<<<<<<<<
 *         return x
 *     else:
 */
  }

  /* "orthogonal_eval.pxd":494
 *         return x
 *     else:
 *         y3 = 0.0             # <<<<<<<<<<<<<<
 *         y2 = 1.0
 *         for k in range(n, 1, -1):
 */
  /*else*/ {
    __pyx_v_y3 = 0.0;

    /* "orthogonal_eval.pxd":495
 *     else:
 *         y3 = 0.0
 *         y2 = 1.0             # <<<<<<<<<<<<<<
 *         for k in range(n, 1, -1):
 *             y1 = x*y2 - k*y3
 */
    __pyx_v_y2 = 1.0;

    /* "orthogonal_eval.pxd":496
 *         y3 = 0.0
 *         y2 = 1.0
 *         for k in range(n, 1, -1):             # <<<<<<<<<<<<<<
 *             y1 = x*y2 - k*y3
 *             y3 = y2
 */
    for (__pyx_t_2 = __pyx_v_n; __pyx_t_2 > 1; __pyx_t_2-=1) {
      __pyx_v_k = __pyx_t_2;

      /* "orthogonal_eval.pxd":497
 *         y2 = 1.0
 *         for k in range(n, 1, -1):
 *             y1 = x*y2 - k*y3             # <<<<<<<<<<<<<<
 *             y3 = y2
 *             y2 = y1
 */
      __pyx_v_y1 = ((__pyx_v_x * __pyx_v_y2) - (__pyx_v_k * __pyx_v_y3));

      /* "orthogonal_eval.pxd":498
 *         for k in range(n, 1, -1):
 *             y1 = x*y2 - k*y3
 *             y3 = y2             # <<<<<<<<<<<<<<
 *             y2 = y1
 *         return x*y2 - y3
 */
      __pyx_v_y3 = __pyx_v_y2;

      /* "orthogonal_eval.pxd":499
 *             y1 = x*y2 - k*y3
 *             y3 = y2
 *             y2 = y1             # <<<<<<<<<<<<<<
 *         return x*y2 - y3
 * 
 */
      __pyx_v_y2 = __pyx_v_y1;
    }

    /* "orthogonal_eval.pxd":500
 *             y3 = y2
 *             y2 = y1
 *         return x*y2 - y3             # <<<<<<<<<<<<<<
 * 
 * #-----------------------------------------------------------------------------
 */
    __pyx_r = ((__pyx_v_x * __pyx_v_y2) - __pyx_v_y3);
    goto __pyx_L0;
  }

  /* "orthogonal_eval.pxd":483
 * #-----------------------------------------------------------------------------
 * 
 * cdef inline double eval_hermitenorm(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     cdef long k
 *     cdef double y1, y2, y3
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "orthogonal_eval.pxd":507
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_hermite(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_hermitenorm(n, sqrt(2)*x) * 2**(n/2.0)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_15orthogonal_eval_eval_hermite(long __pyx_v_n, double __pyx_v_x) {
  double __pyx_r;

  /* "orthogonal_eval.pxd":508
 * @cython.cdivision(True)
 * cdef inline double eval_hermite(long n, double x) nogil:
 *     return eval_hermitenorm(n, sqrt(2)*x) * 2**(n/2.0)             # <<<<<<<<<<<<<<
 */
  __pyx_r = (__pyx_f_5scipy_7special_15orthogonal_eval_eval_hermitenorm(__pyx_v_n, (sqrt(2.0) * __pyx_v_x)) * pow(2.0, (((double)__pyx_v_n) / 2.0)));
  goto __pyx_L0;

  /* "orthogonal_eval.pxd":507
 * 
 * @cython.cdivision(True)
 * cdef inline double eval_hermite(long n, double x) nogil:             # <<<<<<<<<<<<<<
 *     return eval_hermitenorm(n, sqrt(2)*x) * 2**(n/2.0)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_boxcox.pxd":5
 * 
 * 
 * cdef inline double boxcox(double x, double lmbda) nogil:             # <<<<<<<<<<<<<<
 *     # if lmbda << 1 and log(x) < 1.0, the lmbda*log(x) product can lose
 *     # precision, furthermore, expm1(x) == x for x < eps.
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_boxcox_boxcox(double __pyx_v_x, double __pyx_v_lmbda) {
  double __pyx_r;
  int __pyx_t_1;
  double __pyx_t_2;

  /* "_boxcox.pxd":12
 *     # abs(lmbda)*log(x) < eps whenever abs(lmbda) <= eps/-log(min double)
 *     # which is ~2.98e-19.
 *     if fabs(lmbda) < 1e-19:             # <<<<<<<<<<<<<<
 *         return log(x)
 *     else:
 */
  __pyx_t_1 = ((fabs(__pyx_v_lmbda) < 1e-19) != 0);
  if (__pyx_t_1) {

    /* "_boxcox.pxd":13
 *     # which is ~2.98e-19.
 *     if fabs(lmbda) < 1e-19:
 *         return log(x)             # <<<<<<<<<<<<<<
 *     else:
 *         return expm1(lmbda * log(x)) / lmbda
 */
    __pyx_r = log(__pyx_v_x);
    goto __pyx_L0;

    /* "_boxcox.pxd":12
 *     # abs(lmbda)*log(x) < eps whenever abs(lmbda) <= eps/-log(min double)
 *     # which is ~2.98e-19.
 *     if fabs(lmbda) < 1e-19:             # <<<<<<<<<<<<<<
 *         return log(x)
 *     else:
 */
  }

  /* "_boxcox.pxd":15
 *         return log(x)
 *     else:
 *         return expm1(lmbda * log(x)) / lmbda             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_t_2 = expm1((__pyx_v_lmbda * log(__pyx_v_x)));
    if (unlikely(__pyx_v_lmbda == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      __Pyx_PyGILState_Release(__pyx_gilstate_save);
      #endif
      __PYX_ERR(5, 15, __pyx_L1_error)
    }
    __pyx_r = (__pyx_t_2 / __pyx_v_lmbda);
    goto __pyx_L0;
  }

  /* "_boxcox.pxd":5
 * 
 * 
 * cdef inline double boxcox(double x, double lmbda) nogil:             # <<<<<<<<<<<<<<
 *     # if lmbda << 1 and log(x) < 1.0, the lmbda*log(x) product can lose
 *     # precision, furthermore, expm1(x) == x for x < eps.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special._boxcox.boxcox", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "_boxcox.pxd":18
 * 
 * 
 * cdef inline double boxcox1p(double x, double lmbda) nogil:             # <<<<<<<<<<<<<<
 *     # The argument given above in boxcox applies here with the modification
 *     # that the smallest value produced by log1p is the minimum representable
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_boxcox_boxcox1p(double __pyx_v_x, double __pyx_v_lmbda) {
  double __pyx_v_lgx;
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  double __pyx_t_3;

  /* "_boxcox.pxd":23
 *     # value, rather than eps.  The second condition here prevents unflow
 *     # when log1p(x) is < eps.
 *     cdef double lgx = log1p(x)             # <<<<<<<<<<<<<<
 *     if fabs(lmbda) < 1e-19 or (fabs(lgx) < 1e-289 and fabs(lmbda) < 1e273):
 *         return lgx
 */
  __pyx_v_lgx = log1p(__pyx_v_x);

  /* "_boxcox.pxd":24
 *     # when log1p(x) is < eps.
 *     cdef double lgx = log1p(x)
 *     if fabs(lmbda) < 1e-19 or (fabs(lgx) < 1e-289 and fabs(lmbda) < 1e273):             # <<<<<<<<<<<<<<
 *         return lgx
 *     else:
 */
  __pyx_t_2 = ((fabs(__pyx_v_lmbda) < 1e-19) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((fabs(__pyx_v_lgx) < 1e-289) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((fabs(__pyx_v_lmbda) < 1e273) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_boxcox.pxd":25
 *     cdef double lgx = log1p(x)
 *     if fabs(lmbda) < 1e-19 or (fabs(lgx) < 1e-289 and fabs(lmbda) < 1e273):
 *         return lgx             # <<<<<<<<<<<<<<
 *     else:
 *         return expm1(lmbda * lgx) / lmbda
 */
    __pyx_r = __pyx_v_lgx;
    goto __pyx_L0;

    /* "_boxcox.pxd":24
 *     # when log1p(x) is < eps.
 *     cdef double lgx = log1p(x)
 *     if fabs(lmbda) < 1e-19 or (fabs(lgx) < 1e-289 and fabs(lmbda) < 1e273):             # <<<<<<<<<<<<<<
 *         return lgx
 *     else:
 */
  }

  /* "_boxcox.pxd":27
 *         return lgx
 *     else:
 *         return expm1(lmbda * lgx) / lmbda             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_t_3 = expm1((__pyx_v_lmbda * __pyx_v_lgx));
    if (unlikely(__pyx_v_lmbda == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      __Pyx_PyGILState_Release(__pyx_gilstate_save);
      #endif
      __PYX_ERR(5, 27, __pyx_L1_error)
    }
    __pyx_r = (__pyx_t_3 / __pyx_v_lmbda);
    goto __pyx_L0;
  }

  /* "_boxcox.pxd":18
 * 
 * 
 * cdef inline double boxcox1p(double x, double lmbda) nogil:             # <<<<<<<<<<<<<<
 *     # The argument given above in boxcox applies here with the modification
 *     # that the smallest value produced by log1p is the minimum representable
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special._boxcox.boxcox1p", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "_boxcox.pxd":30
 * 
 * 
 * cdef inline double inv_boxcox(double x, double lmbda) nogil:             # <<<<<<<<<<<<<<
 *     if lmbda == 0:
 *         return exp(x)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_boxcox_inv_boxcox(double __pyx_v_x, double __pyx_v_lmbda) {
  double __pyx_r;
  int __pyx_t_1;
  double __pyx_t_2;

  /* "_boxcox.pxd":31
 * 
 * cdef inline double inv_boxcox(double x, double lmbda) nogil:
 *     if lmbda == 0:             # <<<<<<<<<<<<<<
 *         return exp(x)
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_lmbda == 0.0) != 0);
  if (__pyx_t_1) {

    /* "_boxcox.pxd":32
 * cdef inline double inv_boxcox(double x, double lmbda) nogil:
 *     if lmbda == 0:
 *         return exp(x)             # <<<<<<<<<<<<<<
 *     else:
 *         return exp(log1p(lmbda * x) / lmbda)
 */
    __pyx_r = exp(__pyx_v_x);
    goto __pyx_L0;

    /* "_boxcox.pxd":31
 * 
 * cdef inline double inv_boxcox(double x, double lmbda) nogil:
 *     if lmbda == 0:             # <<<<<<<<<<<<<<
 *         return exp(x)
 *     else:
 */
  }

  /* "_boxcox.pxd":34
 *         return exp(x)
 *     else:
 *         return exp(log1p(lmbda * x) / lmbda)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_t_2 = log1p((__pyx_v_lmbda * __pyx_v_x));
    if (unlikely(__pyx_v_lmbda == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      __Pyx_PyGILState_Release(__pyx_gilstate_save);
      #endif
      __PYX_ERR(5, 34, __pyx_L1_error)
    }
    __pyx_r = exp((__pyx_t_2 / __pyx_v_lmbda));
    goto __pyx_L0;
  }

  /* "_boxcox.pxd":30
 * 
 * 
 * cdef inline double inv_boxcox(double x, double lmbda) nogil:             # <<<<<<<<<<<<<<
 *     if lmbda == 0:
 *         return exp(x)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special._boxcox.inv_boxcox", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "_boxcox.pxd":37
 * 
 * 
 * cdef inline double inv_boxcox1p(double x, double lmbda) nogil:             # <<<<<<<<<<<<<<
 *     if lmbda == 0:
 *         return expm1(x)
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_boxcox_inv_boxcox1p(double __pyx_v_x, double __pyx_v_lmbda) {
  double __pyx_r;
  int __pyx_t_1;
  double __pyx_t_2;

  /* "_boxcox.pxd":38
 * 
 * cdef inline double inv_boxcox1p(double x, double lmbda) nogil:
 *     if lmbda == 0:             # <<<<<<<<<<<<<<
 *         return expm1(x)
 *     elif fabs(lmbda * x) < 1e-154:
 */
  __pyx_t_1 = ((__pyx_v_lmbda == 0.0) != 0);
  if (__pyx_t_1) {

    /* "_boxcox.pxd":39
 * cdef inline double inv_boxcox1p(double x, double lmbda) nogil:
 *     if lmbda == 0:
 *         return expm1(x)             # <<<<<<<<<<<<<<
 *     elif fabs(lmbda * x) < 1e-154:
 *         return x
 */
    __pyx_r = expm1(__pyx_v_x);
    goto __pyx_L0;

    /* "_boxcox.pxd":38
 * 
 * cdef inline double inv_boxcox1p(double x, double lmbda) nogil:
 *     if lmbda == 0:             # <<<<<<<<<<<<<<
 *         return expm1(x)
 *     elif fabs(lmbda * x) < 1e-154:
 */
  }

  /* "_boxcox.pxd":40
 *     if lmbda == 0:
 *         return expm1(x)
 *     elif fabs(lmbda * x) < 1e-154:             # <<<<<<<<<<<<<<
 *         return x
 *     else:
 */
  __pyx_t_1 = ((fabs((__pyx_v_lmbda * __pyx_v_x)) < 1e-154) != 0);
  if (__pyx_t_1) {

    /* "_boxcox.pxd":41
 *         return expm1(x)
 *     elif fabs(lmbda * x) < 1e-154:
 *         return x             # <<<<<<<<<<<<<<
 *     else:
 *         return expm1(log1p(lmbda * x) / lmbda)
 */
    __pyx_r = __pyx_v_x;
    goto __pyx_L0;

    /* "_boxcox.pxd":40
 *     if lmbda == 0:
 *         return expm1(x)
 *     elif fabs(lmbda * x) < 1e-154:             # <<<<<<<<<<<<<<
 *         return x
 *     else:
 */
  }

  /* "_boxcox.pxd":43
 *         return x
 *     else:
 *         return expm1(log1p(lmbda * x) / lmbda)             # <<<<<<<<<<<<<<
 */
  /*else*/ {
    __pyx_t_2 = log1p((__pyx_v_lmbda * __pyx_v_x));
    if (unlikely(__pyx_v_lmbda == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      __Pyx_PyGILState_Release(__pyx_gilstate_save);
      #endif
      __PYX_ERR(5, 43, __pyx_L1_error)
    }
    __pyx_r = expm1((__pyx_t_2 / __pyx_v_lmbda));
    goto __pyx_L0;
  }

  /* "_boxcox.pxd":37
 * 
 * 
 * cdef inline double inv_boxcox1p(double x, double lmbda) nogil:             # <<<<<<<<<<<<<<
 *     if lmbda == 0:
 *         return expm1(x)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special._boxcox.inv_boxcox1p", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "_convex_analysis.pxd":8
 * from libc.math cimport log, sqrt, fabs
 * 
 * cdef inline double entr(double x) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(x):
 *         return x
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_16_convex_analysis_entr(double __pyx_v_x) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_convex_analysis.pxd":9
 * 
 * cdef inline double entr(double x) nogil:
 *     if npy_isnan(x):             # <<<<<<<<<<<<<<
 *         return x
 *     elif x > 0:
 */
  __pyx_t_1 = (npy_isnan(__pyx_v_x) != 0);
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":10
 * cdef inline double entr(double x) nogil:
 *     if npy_isnan(x):
 *         return x             # <<<<<<<<<<<<<<
 *     elif x > 0:
 *         return -x * log(x)
 */
    __pyx_r = __pyx_v_x;
    goto __pyx_L0;

    /* "_convex_analysis.pxd":9
 * 
 * cdef inline double entr(double x) nogil:
 *     if npy_isnan(x):             # <<<<<<<<<<<<<<
 *         return x
 *     elif x > 0:
 */
  }

  /* "_convex_analysis.pxd":11
 *     if npy_isnan(x):
 *         return x
 *     elif x > 0:             # <<<<<<<<<<<<<<
 *         return -x * log(x)
 *     elif x == 0:
 */
  __pyx_t_1 = ((__pyx_v_x > 0.0) != 0);
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":12
 *         return x
 *     elif x > 0:
 *         return -x * log(x)             # <<<<<<<<<<<<<<
 *     elif x == 0:
 *         return 0
 */
    __pyx_r = ((-__pyx_v_x) * log(__pyx_v_x));
    goto __pyx_L0;

    /* "_convex_analysis.pxd":11
 *     if npy_isnan(x):
 *         return x
 *     elif x > 0:             # <<<<<<<<<<<<<<
 *         return -x * log(x)
 *     elif x == 0:
 */
  }

  /* "_convex_analysis.pxd":13
 *     elif x > 0:
 *         return -x * log(x)
 *     elif x == 0:             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_x == 0.0) != 0);
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":14
 *         return -x * log(x)
 *     elif x == 0:
 *         return 0             # <<<<<<<<<<<<<<
 *     else:
 *         return -inf
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "_convex_analysis.pxd":13
 *     elif x > 0:
 *         return -x * log(x)
 *     elif x == 0:             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  }

  /* "_convex_analysis.pxd":16
 *         return 0
 *     else:
 *         return -inf             # <<<<<<<<<<<<<<
 * 
 * cdef inline double kl_div(double x, double y) nogil:
 */
  /*else*/ {
    __pyx_r = (-NPY_INFINITY);
    goto __pyx_L0;
  }

  /* "_convex_analysis.pxd":8
 * from libc.math cimport log, sqrt, fabs
 * 
 * cdef inline double entr(double x) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(x):
 *         return x
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_convex_analysis.pxd":18
 *         return -inf
 * 
 * cdef inline double kl_div(double x, double y) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(x) or npy_isnan(y):
 *         return nan
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_16_convex_analysis_kl_div(double __pyx_v_x, double __pyx_v_y) {
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_convex_analysis.pxd":19
 * 
 * cdef inline double kl_div(double x, double y) nogil:
 *     if npy_isnan(x) or npy_isnan(y):             # <<<<<<<<<<<<<<
 *         return nan
 *     elif x > 0 and y > 0:
 */
  __pyx_t_2 = (npy_isnan(__pyx_v_x) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (npy_isnan(__pyx_v_y) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":20
 * cdef inline double kl_div(double x, double y) nogil:
 *     if npy_isnan(x) or npy_isnan(y):
 *         return nan             # <<<<<<<<<<<<<<
 *     elif x > 0 and y > 0:
 *         return x * log(x / y) - x + y
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "_convex_analysis.pxd":19
 * 
 * cdef inline double kl_div(double x, double y) nogil:
 *     if npy_isnan(x) or npy_isnan(y):             # <<<<<<<<<<<<<<
 *         return nan
 *     elif x > 0 and y > 0:
 */
  }

  /* "_convex_analysis.pxd":21
 *     if npy_isnan(x) or npy_isnan(y):
 *         return nan
 *     elif x > 0 and y > 0:             # <<<<<<<<<<<<<<
 *         return x * log(x / y) - x + y
 *     elif x == 0 and y >= 0:
 */
  __pyx_t_2 = ((__pyx_v_x > 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_y > 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":22
 *         return nan
 *     elif x > 0 and y > 0:
 *         return x * log(x / y) - x + y             # <<<<<<<<<<<<<<
 *     elif x == 0 and y >= 0:
 *         return y
 */
    if (unlikely(__pyx_v_y == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      __Pyx_PyGILState_Release(__pyx_gilstate_save);
      #endif
      __PYX_ERR(6, 22, __pyx_L1_error)
    }
    __pyx_r = (((__pyx_v_x * log((__pyx_v_x / __pyx_v_y))) - __pyx_v_x) + __pyx_v_y);
    goto __pyx_L0;

    /* "_convex_analysis.pxd":21
 *     if npy_isnan(x) or npy_isnan(y):
 *         return nan
 *     elif x > 0 and y > 0:             # <<<<<<<<<<<<<<
 *         return x * log(x / y) - x + y
 *     elif x == 0 and y >= 0:
 */
  }

  /* "_convex_analysis.pxd":23
 *     elif x > 0 and y > 0:
 *         return x * log(x / y) - x + y
 *     elif x == 0 and y >= 0:             # <<<<<<<<<<<<<<
 *         return y
 *     else:
 */
  __pyx_t_2 = ((__pyx_v_x == 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_y >= 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L8_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":24
 *         return x * log(x / y) - x + y
 *     elif x == 0 and y >= 0:
 *         return y             # <<<<<<<<<<<<<<
 *     else:
 *         return inf
 */
    __pyx_r = __pyx_v_y;
    goto __pyx_L0;

    /* "_convex_analysis.pxd":23
 *     elif x > 0 and y > 0:
 *         return x * log(x / y) - x + y
 *     elif x == 0 and y >= 0:             # <<<<<<<<<<<<<<
 *         return y
 *     else:
 */
  }

  /* "_convex_analysis.pxd":26
 *         return y
 *     else:
 *         return inf             # <<<<<<<<<<<<<<
 * 
 * cdef inline double rel_entr(double x, double y) nogil:
 */
  /*else*/ {
    __pyx_r = NPY_INFINITY;
    goto __pyx_L0;
  }

  /* "_convex_analysis.pxd":18
 *         return -inf
 * 
 * cdef inline double kl_div(double x, double y) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(x) or npy_isnan(y):
 *         return nan
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special._convex_analysis.kl_div", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "_convex_analysis.pxd":28
 *         return inf
 * 
 * cdef inline double rel_entr(double x, double y) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(x) or npy_isnan(y):
 *         return nan
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_16_convex_analysis_rel_entr(double __pyx_v_x, double __pyx_v_y) {
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_convex_analysis.pxd":29
 * 
 * cdef inline double rel_entr(double x, double y) nogil:
 *     if npy_isnan(x) or npy_isnan(y):             # <<<<<<<<<<<<<<
 *         return nan
 *     elif x > 0 and y > 0:
 */
  __pyx_t_2 = (npy_isnan(__pyx_v_x) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (npy_isnan(__pyx_v_y) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":30
 * cdef inline double rel_entr(double x, double y) nogil:
 *     if npy_isnan(x) or npy_isnan(y):
 *         return nan             # <<<<<<<<<<<<<<
 *     elif x > 0 and y > 0:
 *         return x * log(x / y)
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "_convex_analysis.pxd":29
 * 
 * cdef inline double rel_entr(double x, double y) nogil:
 *     if npy_isnan(x) or npy_isnan(y):             # <<<<<<<<<<<<<<
 *         return nan
 *     elif x > 0 and y > 0:
 */
  }

  /* "_convex_analysis.pxd":31
 *     if npy_isnan(x) or npy_isnan(y):
 *         return nan
 *     elif x > 0 and y > 0:             # <<<<<<<<<<<<<<
 *         return x * log(x / y)
 *     elif x == 0 and y >= 0:
 */
  __pyx_t_2 = ((__pyx_v_x > 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_y > 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":32
 *         return nan
 *     elif x > 0 and y > 0:
 *         return x * log(x / y)             # <<<<<<<<<<<<<<
 *     elif x == 0 and y >= 0:
 *         return 0
 */
    if (unlikely(__pyx_v_y == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      __Pyx_PyGILState_Release(__pyx_gilstate_save);
      #endif
      __PYX_ERR(6, 32, __pyx_L1_error)
    }
    __pyx_r = (__pyx_v_x * log((__pyx_v_x / __pyx_v_y)));
    goto __pyx_L0;

    /* "_convex_analysis.pxd":31
 *     if npy_isnan(x) or npy_isnan(y):
 *         return nan
 *     elif x > 0 and y > 0:             # <<<<<<<<<<<<<<
 *         return x * log(x / y)
 *     elif x == 0 and y >= 0:
 */
  }

  /* "_convex_analysis.pxd":33
 *     elif x > 0 and y > 0:
 *         return x * log(x / y)
 *     elif x == 0 and y >= 0:             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  __pyx_t_2 = ((__pyx_v_x == 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_y >= 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L8_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":34
 *         return x * log(x / y)
 *     elif x == 0 and y >= 0:
 *         return 0             # <<<<<<<<<<<<<<
 *     else:
 *         return inf
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "_convex_analysis.pxd":33
 *     elif x > 0 and y > 0:
 *         return x * log(x / y)
 *     elif x == 0 and y >= 0:             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  }

  /* "_convex_analysis.pxd":36
 *         return 0
 *     else:
 *         return inf             # <<<<<<<<<<<<<<
 * 
 * cdef inline double huber(double delta, double r) nogil:
 */
  /*else*/ {
    __pyx_r = NPY_INFINITY;
    goto __pyx_L0;
  }

  /* "_convex_analysis.pxd":28
 *         return inf
 * 
 * cdef inline double rel_entr(double x, double y) nogil:             # <<<<<<<<<<<<<<
 *     if npy_isnan(x) or npy_isnan(y):
 *         return nan
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special._convex_analysis.rel_entr", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "_convex_analysis.pxd":38
 *         return inf
 * 
 * cdef inline double huber(double delta, double r) nogil:             # <<<<<<<<<<<<<<
 *     if delta < 0:
 *         return inf
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_16_convex_analysis_huber(double __pyx_v_delta, double __pyx_v_r) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_convex_analysis.pxd":39
 * 
 * cdef inline double huber(double delta, double r) nogil:
 *     if delta < 0:             # <<<<<<<<<<<<<<
 *         return inf
 *     elif fabs(r) <= delta:
 */
  __pyx_t_1 = ((__pyx_v_delta < 0.0) != 0);
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":40
 * cdef inline double huber(double delta, double r) nogil:
 *     if delta < 0:
 *         return inf             # <<<<<<<<<<<<<<
 *     elif fabs(r) <= delta:
 *         return 0.5 * r * r;
 */
    __pyx_r = NPY_INFINITY;
    goto __pyx_L0;

    /* "_convex_analysis.pxd":39
 * 
 * cdef inline double huber(double delta, double r) nogil:
 *     if delta < 0:             # <<<<<<<<<<<<<<
 *         return inf
 *     elif fabs(r) <= delta:
 */
  }

  /* "_convex_analysis.pxd":41
 *     if delta < 0:
 *         return inf
 *     elif fabs(r) <= delta:             # <<<<<<<<<<<<<<
 *         return 0.5 * r * r;
 *     else:
 */
  __pyx_t_1 = ((fabs(__pyx_v_r) <= __pyx_v_delta) != 0);
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":42
 *         return inf
 *     elif fabs(r) <= delta:
 *         return 0.5 * r * r;             # <<<<<<<<<<<<<<
 *     else:
 *         return delta * (fabs(r) - 0.5 * delta);
 */
    __pyx_r = ((0.5 * __pyx_v_r) * __pyx_v_r);
    goto __pyx_L0;

    /* "_convex_analysis.pxd":41
 *     if delta < 0:
 *         return inf
 *     elif fabs(r) <= delta:             # <<<<<<<<<<<<<<
 *         return 0.5 * r * r;
 *     else:
 */
  }

  /* "_convex_analysis.pxd":44
 *         return 0.5 * r * r;
 *     else:
 *         return delta * (fabs(r) - 0.5 * delta);             # <<<<<<<<<<<<<<
 * 
 * cdef inline double pseudo_huber(double delta, double r) nogil:
 */
  /*else*/ {
    __pyx_r = (__pyx_v_delta * (fabs(__pyx_v_r) - (0.5 * __pyx_v_delta)));
    goto __pyx_L0;
  }

  /* "_convex_analysis.pxd":38
 *         return inf
 * 
 * cdef inline double huber(double delta, double r) nogil:             # <<<<<<<<<<<<<<
 *     if delta < 0:
 *         return inf
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_convex_analysis.pxd":46
 *         return delta * (fabs(r) - 0.5 * delta);
 * 
 * cdef inline double pseudo_huber(double delta, double r) nogil:             # <<<<<<<<<<<<<<
 *     cdef double u, v
 *     if delta < 0:
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_16_convex_analysis_pseudo_huber(double __pyx_v_delta, double __pyx_v_r) {
  double __pyx_v_u;
  double __pyx_v_v;
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_convex_analysis.pxd":48
 * cdef inline double pseudo_huber(double delta, double r) nogil:
 *     cdef double u, v
 *     if delta < 0:             # <<<<<<<<<<<<<<
 *         return inf
 *     elif delta == 0 or r == 0:
 */
  __pyx_t_1 = ((__pyx_v_delta < 0.0) != 0);
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":49
 *     cdef double u, v
 *     if delta < 0:
 *         return inf             # <<<<<<<<<<<<<<
 *     elif delta == 0 or r == 0:
 *         return 0
 */
    __pyx_r = NPY_INFINITY;
    goto __pyx_L0;

    /* "_convex_analysis.pxd":48
 * cdef inline double pseudo_huber(double delta, double r) nogil:
 *     cdef double u, v
 *     if delta < 0:             # <<<<<<<<<<<<<<
 *         return inf
 *     elif delta == 0 or r == 0:
 */
  }

  /* "_convex_analysis.pxd":50
 *     if delta < 0:
 *         return inf
 *     elif delta == 0 or r == 0:             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  __pyx_t_2 = ((__pyx_v_delta == 0.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_r == 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_convex_analysis.pxd":51
 *         return inf
 *     elif delta == 0 or r == 0:
 *         return 0             # <<<<<<<<<<<<<<
 *     else:
 *         u = delta
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "_convex_analysis.pxd":50
 *     if delta < 0:
 *         return inf
 *     elif delta == 0 or r == 0:             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  }

  /* "_convex_analysis.pxd":53
 *         return 0
 *     else:
 *         u = delta             # <<<<<<<<<<<<<<
 *         v = r / delta
 *         return u*u*(sqrt(1 + v*v) - 1)
 */
  /*else*/ {
    __pyx_v_u = __pyx_v_delta;

    /* "_convex_analysis.pxd":54
 *     else:
 *         u = delta
 *         v = r / delta             # <<<<<<<<<<<<<<
 *         return u*u*(sqrt(1 + v*v) - 1)
 */
    if (unlikely(__pyx_v_delta == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      __Pyx_PyGILState_Release(__pyx_gilstate_save);
      #endif
      __PYX_ERR(6, 54, __pyx_L1_error)
    }
    __pyx_v_v = (__pyx_v_r / __pyx_v_delta);

    /* "_convex_analysis.pxd":55
 *         u = delta
 *         v = r / delta
 *         return u*u*(sqrt(1 + v*v) - 1)             # <<<<<<<<<<<<<<
 */
    __pyx_r = ((__pyx_v_u * __pyx_v_u) * (sqrt((1.0 + (__pyx_v_v * __pyx_v_v))) - 1.0));
    goto __pyx_L0;
  }

  /* "_convex_analysis.pxd":46
 *         return delta * (fabs(r) - 0.5 * delta);
 * 
 * cdef inline double pseudo_huber(double delta, double r) nogil:             # <<<<<<<<<<<<<<
 *     cdef double u, v
 *     if delta < 0:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special._convex_analysis.pseudo_huber", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "_cunity.pxd":39
 * # y = +/-sqrt(2*fabs(x)). To get around this cancellation problem, we use
 * # double-double precision when necessary.
 * cdef inline double complex clog1p(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     cdef double zr, zi, x, y, az, azi
 *     cdef np.npy_cdouble ret
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_7_cunity_clog1p(__pyx_t_double_complex __pyx_v_z) {
  double __pyx_v_zr;
  double __pyx_v_zi;
  double __pyx_v_x;
  double __pyx_v_y;
  double __pyx_v_az;
  double __pyx_v_azi;
  npy_cdouble __pyx_v_ret;
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  double __pyx_t_2;
  int __pyx_t_3;
  double __pyx_t_4;

  /* "_cunity.pxd":43
 *     cdef np.npy_cdouble ret
 * 
 *     if not zisfinite(z):             # <<<<<<<<<<<<<<
 *         z = z + 1
 *         ret = npy_clog(npy_cdouble_from_double_complex(z))
 */
  __pyx_t_1 = ((!(__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisfinite(__pyx_v_z) != 0)) != 0);
  if (__pyx_t_1) {

    /* "_cunity.pxd":44
 * 
 *     if not zisfinite(z):
 *         z = z + 1             # <<<<<<<<<<<<<<
 *         ret = npy_clog(npy_cdouble_from_double_complex(z))
 *         return double_complex_from_npy_cdouble(ret)
 */
    __pyx_v_z = __Pyx_c_sum_double(__pyx_v_z, __pyx_t_double_complex_from_parts(1, 0));

    /* "_cunity.pxd":45
 *     if not zisfinite(z):
 *         z = z + 1
 *         ret = npy_clog(npy_cdouble_from_double_complex(z))             # <<<<<<<<<<<<<<
 *         return double_complex_from_npy_cdouble(ret)
 * 
 */
    __pyx_v_ret = npy_clog(__pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_z));

    /* "_cunity.pxd":46
 *         z = z + 1
 *         ret = npy_clog(npy_cdouble_from_double_complex(z))
 *         return double_complex_from_npy_cdouble(ret)             # <<<<<<<<<<<<<<
 * 
 *     zr = z.real
 */
    __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_double_complex_from_npy_cdouble(__pyx_v_ret);
    goto __pyx_L0;

    /* "_cunity.pxd":43
 *     cdef np.npy_cdouble ret
 * 
 *     if not zisfinite(z):             # <<<<<<<<<<<<<<
 *         z = z + 1
 *         ret = npy_clog(npy_cdouble_from_double_complex(z))
 */
  }

  /* "_cunity.pxd":48
 *         return double_complex_from_npy_cdouble(ret)
 * 
 *     zr = z.real             # <<<<<<<<<<<<<<
 *     zi = z.imag
 * 
 */
  __pyx_t_2 = __Pyx_CREAL(__pyx_v_z);
  __pyx_v_zr = __pyx_t_2;

  /* "_cunity.pxd":49
 * 
 *     zr = z.real
 *     zi = z.imag             # <<<<<<<<<<<<<<
 * 
 *     if zi == 0.0 and zr >= -1.0:
 */
  __pyx_t_2 = __Pyx_CIMAG(__pyx_v_z);
  __pyx_v_zi = __pyx_t_2;

  /* "_cunity.pxd":51
 *     zi = z.imag
 * 
 *     if zi == 0.0 and zr >= -1.0:             # <<<<<<<<<<<<<<
 *         return zpack(log1p(zr), 0.0)
 * 
 */
  __pyx_t_3 = ((__pyx_v_zi == 0.0) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_3 = ((__pyx_v_zr >= -1.0) != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_cunity.pxd":52
 * 
 *     if zi == 0.0 and zr >= -1.0:
 *         return zpack(log1p(zr), 0.0)             # <<<<<<<<<<<<<<
 * 
 *     az = zabs(z)
 */
    __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_zpack(log1p(__pyx_v_zr), 0.0);
    goto __pyx_L0;

    /* "_cunity.pxd":51
 *     zi = z.imag
 * 
 *     if zi == 0.0 and zr >= -1.0:             # <<<<<<<<<<<<<<
 *         return zpack(log1p(zr), 0.0)
 * 
 */
  }

  /* "_cunity.pxd":54
 *         return zpack(log1p(zr), 0.0)
 * 
 *     az = zabs(z)             # <<<<<<<<<<<<<<
 *     if az < 0.707:
 *         azi = fabs(zi)
 */
  __pyx_v_az = __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_z);

  /* "_cunity.pxd":55
 * 
 *     az = zabs(z)
 *     if az < 0.707:             # <<<<<<<<<<<<<<
 *         azi = fabs(zi)
 *         if zr < 0 and fabs(-zr - azi*azi/2)/(-zr) < 0.5:
 */
  __pyx_t_1 = ((__pyx_v_az < 0.707) != 0);
  if (__pyx_t_1) {

    /* "_cunity.pxd":56
 *     az = zabs(z)
 *     if az < 0.707:
 *         azi = fabs(zi)             # <<<<<<<<<<<<<<
 *         if zr < 0 and fabs(-zr - azi*azi/2)/(-zr) < 0.5:
 *             return clog1p_ddouble(zr, zi)
 */
    __pyx_v_azi = fabs(__pyx_v_zi);

    /* "_cunity.pxd":57
 *     if az < 0.707:
 *         azi = fabs(zi)
 *         if zr < 0 and fabs(-zr - azi*azi/2)/(-zr) < 0.5:             # <<<<<<<<<<<<<<
 *             return clog1p_ddouble(zr, zi)
 *         else:
 */
    __pyx_t_3 = ((__pyx_v_zr < 0.0) != 0);
    if (__pyx_t_3) {
    } else {
      __pyx_t_1 = __pyx_t_3;
      goto __pyx_L9_bool_binop_done;
    }
    __pyx_t_2 = fabs(((-__pyx_v_zr) - ((__pyx_v_azi * __pyx_v_azi) / 2.0)));
    __pyx_t_4 = (-__pyx_v_zr);
    if (unlikely(__pyx_t_4 == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      __Pyx_PyGILState_Release(__pyx_gilstate_save);
      #endif
      __PYX_ERR(7, 57, __pyx_L1_error)
    }
    __pyx_t_3 = (((__pyx_t_2 / __pyx_t_4) < 0.5) != 0);
    __pyx_t_1 = __pyx_t_3;
    __pyx_L9_bool_binop_done:;
    if (__pyx_t_1) {

      /* "_cunity.pxd":58
 *         azi = fabs(zi)
 *         if zr < 0 and fabs(-zr - azi*azi/2)/(-zr) < 0.5:
 *             return clog1p_ddouble(zr, zi)             # <<<<<<<<<<<<<<
 *         else:
 *             x = 0.5 * log1p(az*(az + 2*zr/az))
 */
      __pyx_r = __pyx_f_5scipy_7special_7_cunity_clog1p_ddouble(__pyx_v_zr, __pyx_v_zi);
      goto __pyx_L0;

      /* "_cunity.pxd":57
 *     if az < 0.707:
 *         azi = fabs(zi)
 *         if zr < 0 and fabs(-zr - azi*azi/2)/(-zr) < 0.5:             # <<<<<<<<<<<<<<
 *             return clog1p_ddouble(zr, zi)
 *         else:
 */
    }

    /* "_cunity.pxd":60
 *             return clog1p_ddouble(zr, zi)
 *         else:
 *             x = 0.5 * log1p(az*(az + 2*zr/az))             # <<<<<<<<<<<<<<
 *             y = npy_atan2(zi, zr + 1.0)
 *             return zpack(x, y)
 */
    /*else*/ {
      __pyx_t_4 = (2.0 * __pyx_v_zr);
      if (unlikely(__pyx_v_az == 0)) {
        #ifdef WITH_THREAD
        PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
        #endif
        PyErr_SetString(PyExc_ZeroDivisionError, "float division");
        #ifdef WITH_THREAD
        __Pyx_PyGILState_Release(__pyx_gilstate_save);
        #endif
        __PYX_ERR(7, 60, __pyx_L1_error)
      }
      __pyx_v_x = (0.5 * log1p((__pyx_v_az * (__pyx_v_az + (__pyx_t_4 / __pyx_v_az)))));

      /* "_cunity.pxd":61
 *         else:
 *             x = 0.5 * log1p(az*(az + 2*zr/az))
 *             y = npy_atan2(zi, zr + 1.0)             # <<<<<<<<<<<<<<
 *             return zpack(x, y)
 * 
 */
      __pyx_v_y = npy_atan2(__pyx_v_zi, (__pyx_v_zr + 1.0));

      /* "_cunity.pxd":62
 *             x = 0.5 * log1p(az*(az + 2*zr/az))
 *             y = npy_atan2(zi, zr + 1.0)
 *             return zpack(x, y)             # <<<<<<<<<<<<<<
 * 
 *     z = z + 1.0
 */
      __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_zpack(__pyx_v_x, __pyx_v_y);
      goto __pyx_L0;
    }

    /* "_cunity.pxd":55
 * 
 *     az = zabs(z)
 *     if az < 0.707:             # <<<<<<<<<<<<<<
 *         azi = fabs(zi)
 *         if zr < 0 and fabs(-zr - azi*azi/2)/(-zr) < 0.5:
 */
  }

  /* "_cunity.pxd":64
 *             return zpack(x, y)
 * 
 *     z = z + 1.0             # <<<<<<<<<<<<<<
 *     ret = npy_clog(npy_cdouble_from_double_complex(z))
 *     return double_complex_from_npy_cdouble(ret)
 */
  __pyx_v_z = __Pyx_c_sum_double(__pyx_v_z, __pyx_t_double_complex_from_parts(1.0, 0));

  /* "_cunity.pxd":65
 * 
 *     z = z + 1.0
 *     ret = npy_clog(npy_cdouble_from_double_complex(z))             # <<<<<<<<<<<<<<
 *     return double_complex_from_npy_cdouble(ret)
 * 
 */
  __pyx_v_ret = npy_clog(__pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_z));

  /* "_cunity.pxd":66
 *     z = z + 1.0
 *     ret = npy_clog(npy_cdouble_from_double_complex(z))
 *     return double_complex_from_npy_cdouble(ret)             # <<<<<<<<<<<<<<
 * 
 * cdef inline double complex clog1p_ddouble(double zr, double zi) nogil:
 */
  __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_double_complex_from_npy_cdouble(__pyx_v_ret);
  goto __pyx_L0;

  /* "_cunity.pxd":39
 * # y = +/-sqrt(2*fabs(x)). To get around this cancellation problem, we use
 * # double-double precision when necessary.
 * cdef inline double complex clog1p(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     cdef double zr, zi, x, y, az, azi
 *     cdef np.npy_cdouble ret
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special._cunity.clog1p", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = __pyx_t_double_complex_from_parts(0, 0);
  __pyx_L0:;
  return __pyx_r;
}

/* "_cunity.pxd":68
 *     return double_complex_from_npy_cdouble(ret)
 * 
 * cdef inline double complex clog1p_ddouble(double zr, double zi) nogil:             # <<<<<<<<<<<<<<
 *     cdef double x, y
 *     cdef double2 r, i, two, rsqr, isqr, rtwo, absm1
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_7_cunity_clog1p_ddouble(double __pyx_v_zr, double __pyx_v_zi) {
  double __pyx_v_x;
  double __pyx_v_y;
  double2 __pyx_v_r;
  double2 __pyx_v_i;
  double2 __pyx_v_two;
  double2 __pyx_v_rsqr;
  double2 __pyx_v_isqr;
  double2 __pyx_v_rtwo;
  double2 __pyx_v_absm1;
  __pyx_t_double_complex __pyx_r;

  /* "_cunity.pxd":72
 *     cdef double2 r, i, two, rsqr, isqr, rtwo, absm1
 * 
 *     r = dd_create_d(zr)             # <<<<<<<<<<<<<<
 *     i = dd_create_d(zi)
 *     two = dd_create_d(2.0)
 */
  __pyx_v_r = dd_create_d(__pyx_v_zr);

  /* "_cunity.pxd":73
 * 
 *     r = dd_create_d(zr)
 *     i = dd_create_d(zi)             # <<<<<<<<<<<<<<
 *     two = dd_create_d(2.0)
 * 
 */
  __pyx_v_i = dd_create_d(__pyx_v_zi);

  /* "_cunity.pxd":74
 *     r = dd_create_d(zr)
 *     i = dd_create_d(zi)
 *     two = dd_create_d(2.0)             # <<<<<<<<<<<<<<
 * 
 *     rsqr = dd_mul(r, r)
 */
  __pyx_v_two = dd_create_d(2.0);

  /* "_cunity.pxd":76
 *     two = dd_create_d(2.0)
 * 
 *     rsqr = dd_mul(r, r)             # <<<<<<<<<<<<<<
 *     isqr = dd_mul(i, i)
 *     rtwo = dd_mul(two, r)
 */
  __pyx_v_rsqr = dd_mul(__pyx_v_r, __pyx_v_r);

  /* "_cunity.pxd":77
 * 
 *     rsqr = dd_mul(r, r)
 *     isqr = dd_mul(i, i)             # <<<<<<<<<<<<<<
 *     rtwo = dd_mul(two, r)
 *     absm1 = dd_add(rsqr, isqr)
 */
  __pyx_v_isqr = dd_mul(__pyx_v_i, __pyx_v_i);

  /* "_cunity.pxd":78
 *     rsqr = dd_mul(r, r)
 *     isqr = dd_mul(i, i)
 *     rtwo = dd_mul(two, r)             # <<<<<<<<<<<<<<
 *     absm1 = dd_add(rsqr, isqr)
 *     absm1 = dd_add(absm1, rtwo)
 */
  __pyx_v_rtwo = dd_mul(__pyx_v_two, __pyx_v_r);

  /* "_cunity.pxd":79
 *     isqr = dd_mul(i, i)
 *     rtwo = dd_mul(two, r)
 *     absm1 = dd_add(rsqr, isqr)             # <<<<<<<<<<<<<<
 *     absm1 = dd_add(absm1, rtwo)
 * 
 */
  __pyx_v_absm1 = dd_add(__pyx_v_rsqr, __pyx_v_isqr);

  /* "_cunity.pxd":80
 *     rtwo = dd_mul(two, r)
 *     absm1 = dd_add(rsqr, isqr)
 *     absm1 = dd_add(absm1, rtwo)             # <<<<<<<<<<<<<<
 * 
 *     x = 0.5 * log1p(dd_to_double(absm1))
 */
  __pyx_v_absm1 = dd_add(__pyx_v_absm1, __pyx_v_rtwo);

  /* "_cunity.pxd":82
 *     absm1 = dd_add(absm1, rtwo)
 * 
 *     x = 0.5 * log1p(dd_to_double(absm1))             # <<<<<<<<<<<<<<
 *     y = npy_atan2(zi, zr+1.0)
 *     return zpack(x, y)
 */
  __pyx_v_x = (0.5 * log1p(dd_to_double(__pyx_v_absm1)));

  /* "_cunity.pxd":83
 * 
 *     x = 0.5 * log1p(dd_to_double(absm1))
 *     y = npy_atan2(zi, zr+1.0)             # <<<<<<<<<<<<<<
 *     return zpack(x, y)
 * 
 */
  __pyx_v_y = npy_atan2(__pyx_v_zi, (__pyx_v_zr + 1.0));

  /* "_cunity.pxd":84
 *     x = 0.5 * log1p(dd_to_double(absm1))
 *     y = npy_atan2(zi, zr+1.0)
 *     return zpack(x, y)             # <<<<<<<<<<<<<<
 * 
 * # cexpm1(z) = cexp(z) - 1
 */
  __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_zpack(__pyx_v_x, __pyx_v_y);
  goto __pyx_L0;

  /* "_cunity.pxd":68
 *     return double_complex_from_npy_cdouble(ret)
 * 
 * cdef inline double complex clog1p_ddouble(double zr, double zi) nogil:             # <<<<<<<<<<<<<<
 *     cdef double x, y
 *     cdef double2 r, i, two, rsqr, isqr, rtwo, absm1
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_cunity.pxd":93
 * # doesn't involve computing exp(z.real) and/or cos(z.imag) to higher
 * # precision.
 * cdef inline double complex cexpm1(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     cdef double zr, zi, ezr, x, y
 *     cdef np.npy_cdouble ret
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_7_cunity_cexpm1(__pyx_t_double_complex __pyx_v_z) {
  double __pyx_v_zr;
  double __pyx_v_zi;
  double __pyx_v_ezr;
  double __pyx_v_x;
  double __pyx_v_y;
  npy_cdouble __pyx_v_ret;
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  double __pyx_t_2;

  /* "_cunity.pxd":97
 *     cdef np.npy_cdouble ret
 * 
 *     if not zisfinite(z):             # <<<<<<<<<<<<<<
 *         ret = npy_cexp(npy_cdouble_from_double_complex(z))
 *         return double_complex_from_npy_cdouble(ret) - 1.0
 */
  __pyx_t_1 = ((!(__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisfinite(__pyx_v_z) != 0)) != 0);
  if (__pyx_t_1) {

    /* "_cunity.pxd":98
 * 
 *     if not zisfinite(z):
 *         ret = npy_cexp(npy_cdouble_from_double_complex(z))             # <<<<<<<<<<<<<<
 *         return double_complex_from_npy_cdouble(ret) - 1.0
 * 
 */
    __pyx_v_ret = npy_cexp(__pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_z));

    /* "_cunity.pxd":99
 *     if not zisfinite(z):
 *         ret = npy_cexp(npy_cdouble_from_double_complex(z))
 *         return double_complex_from_npy_cdouble(ret) - 1.0             # <<<<<<<<<<<<<<
 * 
 *     zr = z.real
 */
    __pyx_r = __Pyx_c_diff_double(__pyx_f_5scipy_7special_13_complexstuff_double_complex_from_npy_cdouble(__pyx_v_ret), __pyx_t_double_complex_from_parts(1.0, 0));
    goto __pyx_L0;

    /* "_cunity.pxd":97
 *     cdef np.npy_cdouble ret
 * 
 *     if not zisfinite(z):             # <<<<<<<<<<<<<<
 *         ret = npy_cexp(npy_cdouble_from_double_complex(z))
 *         return double_complex_from_npy_cdouble(ret) - 1.0
 */
  }

  /* "_cunity.pxd":101
 *         return double_complex_from_npy_cdouble(ret) - 1.0
 * 
 *     zr = z.real             # <<<<<<<<<<<<<<
 *     zi = z.imag
 * 
 */
  __pyx_t_2 = __Pyx_CREAL(__pyx_v_z);
  __pyx_v_zr = __pyx_t_2;

  /* "_cunity.pxd":102
 * 
 *     zr = z.real
 *     zi = z.imag             # <<<<<<<<<<<<<<
 * 
 *     if zr <= -40:
 */
  __pyx_t_2 = __Pyx_CIMAG(__pyx_v_z);
  __pyx_v_zi = __pyx_t_2;

  /* "_cunity.pxd":104
 *     zi = z.imag
 * 
 *     if zr <= -40:             # <<<<<<<<<<<<<<
 *         x = -1.0
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_zr <= -40.0) != 0);
  if (__pyx_t_1) {

    /* "_cunity.pxd":105
 * 
 *     if zr <= -40:
 *         x = -1.0             # <<<<<<<<<<<<<<
 *     else:
 *         ezr = expm1(zr)
 */
    __pyx_v_x = -1.0;

    /* "_cunity.pxd":104
 *     zi = z.imag
 * 
 *     if zr <= -40:             # <<<<<<<<<<<<<<
 *         x = -1.0
 *     else:
 */
    goto __pyx_L4;
  }

  /* "_cunity.pxd":107
 *         x = -1.0
 *     else:
 *         ezr = expm1(zr)             # <<<<<<<<<<<<<<
 *         x = ezr*cos(zi) + cosm1(zi)
 *     # don't compute exp(zr) too, unless necessary
 */
  /*else*/ {
    __pyx_v_ezr = expm1(__pyx_v_zr);

    /* "_cunity.pxd":108
 *     else:
 *         ezr = expm1(zr)
 *         x = ezr*cos(zi) + cosm1(zi)             # <<<<<<<<<<<<<<
 *     # don't compute exp(zr) too, unless necessary
 *     if zr > -1.0:
 */
    __pyx_v_x = ((__pyx_v_ezr * cos(__pyx_v_zi)) + cosm1(__pyx_v_zi));
  }
  __pyx_L4:;

  /* "_cunity.pxd":110
 *         x = ezr*cos(zi) + cosm1(zi)
 *     # don't compute exp(zr) too, unless necessary
 *     if zr > -1.0:             # <<<<<<<<<<<<<<
 *         y = (ezr + 1.0)*sin(zi)
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_zr > -1.0) != 0);
  if (__pyx_t_1) {

    /* "_cunity.pxd":111
 *     # don't compute exp(zr) too, unless necessary
 *     if zr > -1.0:
 *         y = (ezr + 1.0)*sin(zi)             # <<<<<<<<<<<<<<
 *     else:
 *         y = exp(zr)*sin(zi)
 */
    __pyx_v_y = ((__pyx_v_ezr + 1.0) * sin(__pyx_v_zi));

    /* "_cunity.pxd":110
 *         x = ezr*cos(zi) + cosm1(zi)
 *     # don't compute exp(zr) too, unless necessary
 *     if zr > -1.0:             # <<<<<<<<<<<<<<
 *         y = (ezr + 1.0)*sin(zi)
 *     else:
 */
    goto __pyx_L5;
  }

  /* "_cunity.pxd":113
 *         y = (ezr + 1.0)*sin(zi)
 *     else:
 *         y = exp(zr)*sin(zi)             # <<<<<<<<<<<<<<
 * 
 *     return zpack(x, y)
 */
  /*else*/ {
    __pyx_v_y = (exp(__pyx_v_zr) * sin(__pyx_v_zi));
  }
  __pyx_L5:;

  /* "_cunity.pxd":115
 *         y = exp(zr)*sin(zi)
 * 
 *     return zpack(x, y)             # <<<<<<<<<<<<<<
 */
  __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_zpack(__pyx_v_x, __pyx_v_y);
  goto __pyx_L0;

  /* "_cunity.pxd":93
 * # doesn't involve computing exp(z.real) and/or cos(z.imag) to higher
 * # precision.
 * cdef inline double complex cexpm1(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     cdef double zr, zi, ezr, x, y
 *     cdef np.npy_cdouble ret
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_exprel.pxd":7
 * from libc.math cimport expm1, fabs, log
 * 
 * cdef inline double exprel(double x) nogil:             # <<<<<<<<<<<<<<
 *     if fabs(x) < 1e-16:
 *         return 1.0
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_exprel_exprel(double __pyx_v_x) {
  double __pyx_r;
  int __pyx_t_1;
  double __pyx_t_2;

  /* "_exprel.pxd":8
 * 
 * cdef inline double exprel(double x) nogil:
 *     if fabs(x) < 1e-16:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif x > 717:  # near log(DBL_MAX)
 */
  __pyx_t_1 = ((fabs(__pyx_v_x) < 1e-16) != 0);
  if (__pyx_t_1) {

    /* "_exprel.pxd":9
 * cdef inline double exprel(double x) nogil:
 *     if fabs(x) < 1e-16:
 *         return 1.0             # <<<<<<<<<<<<<<
 *     elif x > 717:  # near log(DBL_MAX)
 *         return NPY_INFINITY
 */
    __pyx_r = 1.0;
    goto __pyx_L0;

    /* "_exprel.pxd":8
 * 
 * cdef inline double exprel(double x) nogil:
 *     if fabs(x) < 1e-16:             # <<<<<<<<<<<<<<
 *         return 1.0
 *     elif x > 717:  # near log(DBL_MAX)
 */
  }

  /* "_exprel.pxd":10
 *     if fabs(x) < 1e-16:
 *         return 1.0
 *     elif x > 717:  # near log(DBL_MAX)             # <<<<<<<<<<<<<<
 *         return NPY_INFINITY
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_x > 717.0) != 0);
  if (__pyx_t_1) {

    /* "_exprel.pxd":11
 *         return 1.0
 *     elif x > 717:  # near log(DBL_MAX)
 *         return NPY_INFINITY             # <<<<<<<<<<<<<<
 *     else:
 *         return expm1(x) / x
 */
    __pyx_r = NPY_INFINITY;
    goto __pyx_L0;

    /* "_exprel.pxd":10
 *     if fabs(x) < 1e-16:
 *         return 1.0
 *     elif x > 717:  # near log(DBL_MAX)             # <<<<<<<<<<<<<<
 *         return NPY_INFINITY
 *     else:
 */
  }

  /* "_exprel.pxd":13
 *         return NPY_INFINITY
 *     else:
 *         return expm1(x) / x             # <<<<<<<<<<<<<<
 * 
 */
  /*else*/ {
    __pyx_t_2 = expm1(__pyx_v_x);
    if (unlikely(__pyx_v_x == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      __Pyx_PyGILState_Release(__pyx_gilstate_save);
      #endif
      __PYX_ERR(8, 13, __pyx_L1_error)
    }
    __pyx_r = (__pyx_t_2 / __pyx_v_x);
    goto __pyx_L0;
  }

  /* "_exprel.pxd":7
 * from libc.math cimport expm1, fabs, log
 * 
 * cdef inline double exprel(double x) nogil:             # <<<<<<<<<<<<<<
 *     if fabs(x) < 1e-16:
 *         return 1.0
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special._exprel.exprel", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "_loggamma.pxd":42
 * 
 * 
 * cdef inline double loggamma_real(double x) nogil:             # <<<<<<<<<<<<<<
 *     if x < 0.0:
 *         return nan
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_9_loggamma_loggamma_real(double __pyx_v_x) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_loggamma.pxd":43
 * 
 * cdef inline double loggamma_real(double x) nogil:
 *     if x < 0.0:             # <<<<<<<<<<<<<<
 *         return nan
 * 
 */
  __pyx_t_1 = ((__pyx_v_x < 0.0) != 0);
  if (__pyx_t_1) {

    /* "_loggamma.pxd":44
 * cdef inline double loggamma_real(double x) nogil:
 *     if x < 0.0:
 *         return nan             # <<<<<<<<<<<<<<
 * 
 *     return lgam(x)
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "_loggamma.pxd":43
 * 
 * cdef inline double loggamma_real(double x) nogil:
 *     if x < 0.0:             # <<<<<<<<<<<<<<
 *         return nan
 * 
 */
  }

  /* "_loggamma.pxd":46
 *         return nan
 * 
 *     return lgam(x)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = lgam(__pyx_v_x);
  goto __pyx_L0;

  /* "_loggamma.pxd":42
 * 
 * 
 * cdef inline double loggamma_real(double x) nogil:             # <<<<<<<<<<<<<<
 *     if x < 0.0:
 *         return nan
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_loggamma.pxd":50
 * 
 * @cython.cdivision(True)
 * cdef inline double complex loggamma(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """Compute the principal branch of log-Gamma."""
 *     cdef double tmp
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_9_loggamma_loggamma(__pyx_t_double_complex __pyx_v_z) {
  double __pyx_v_tmp;
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_loggamma.pxd":54
 *     cdef double tmp
 * 
 *     if zisnan(z):             # <<<<<<<<<<<<<<
 *         return zpack(nan, nan)
 *     elif z.real <= 0 and z == floor(z.real):
 */
  __pyx_t_1 = (__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisnan(__pyx_v_z) != 0);
  if (__pyx_t_1) {

    /* "_loggamma.pxd":55
 * 
 *     if zisnan(z):
 *         return zpack(nan, nan)             # <<<<<<<<<<<<<<
 *     elif z.real <= 0 and z == floor(z.real):
 *         sf_error.error("loggamma", sf_error.SINGULAR, NULL)
 */
    __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_zpack(NPY_NAN, NPY_NAN);
    goto __pyx_L0;

    /* "_loggamma.pxd":54
 *     cdef double tmp
 * 
 *     if zisnan(z):             # <<<<<<<<<<<<<<
 *         return zpack(nan, nan)
 *     elif z.real <= 0 and z == floor(z.real):
 */
  }

  /* "_loggamma.pxd":56
 *     if zisnan(z):
 *         return zpack(nan, nan)
 *     elif z.real <= 0 and z == floor(z.real):             # <<<<<<<<<<<<<<
 *         sf_error.error("loggamma", sf_error.SINGULAR, NULL)
 *         return zpack(nan, nan)
 */
  __pyx_t_2 = ((__Pyx_CREAL(__pyx_v_z) <= 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__Pyx_c_eq_double(__pyx_v_z, __pyx_t_double_complex_from_parts(floor(__Pyx_CREAL(__pyx_v_z)), 0))) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_loggamma.pxd":57
 *         return zpack(nan, nan)
 *     elif z.real <= 0 and z == floor(z.real):
 *         sf_error.error("loggamma", sf_error.SINGULAR, NULL)             # <<<<<<<<<<<<<<
 *         return zpack(nan, nan)
 *     elif z.real > SMALLX or fabs(z.imag) > SMALLY:
 */
    sf_error(((char *)"loggamma"), SF_ERROR_SINGULAR, NULL);

    /* "_loggamma.pxd":58
 *     elif z.real <= 0 and z == floor(z.real):
 *         sf_error.error("loggamma", sf_error.SINGULAR, NULL)
 *         return zpack(nan, nan)             # <<<<<<<<<<<<<<
 *     elif z.real > SMALLX or fabs(z.imag) > SMALLY:
 *         return loggamma_stirling(z)
 */
    __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_zpack(NPY_NAN, NPY_NAN);
    goto __pyx_L0;

    /* "_loggamma.pxd":56
 *     if zisnan(z):
 *         return zpack(nan, nan)
 *     elif z.real <= 0 and z == floor(z.real):             # <<<<<<<<<<<<<<
 *         sf_error.error("loggamma", sf_error.SINGULAR, NULL)
 *         return zpack(nan, nan)
 */
  }

  /* "_loggamma.pxd":59
 *         sf_error.error("loggamma", sf_error.SINGULAR, NULL)
 *         return zpack(nan, nan)
 *     elif z.real > SMALLX or fabs(z.imag) > SMALLY:             # <<<<<<<<<<<<<<
 *         return loggamma_stirling(z)
 *     elif zabs(z - 1) <= TAYLOR_RADIUS:
 */
  __pyx_t_2 = ((__Pyx_CREAL(__pyx_v_z) > 7.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_2 = ((fabs(__Pyx_CIMAG(__pyx_v_z)) > 7.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_loggamma.pxd":60
 *         return zpack(nan, nan)
 *     elif z.real > SMALLX or fabs(z.imag) > SMALLY:
 *         return loggamma_stirling(z)             # <<<<<<<<<<<<<<
 *     elif zabs(z - 1) <= TAYLOR_RADIUS:
 *         return loggamma_taylor(z)
 */
    __pyx_r = __pyx_f_5scipy_7special_9_loggamma_loggamma_stirling(__pyx_v_z);
    goto __pyx_L0;

    /* "_loggamma.pxd":59
 *         sf_error.error("loggamma", sf_error.SINGULAR, NULL)
 *         return zpack(nan, nan)
 *     elif z.real > SMALLX or fabs(z.imag) > SMALLY:             # <<<<<<<<<<<<<<
 *         return loggamma_stirling(z)
 *     elif zabs(z - 1) <= TAYLOR_RADIUS:
 */
  }

  /* "_loggamma.pxd":61
 *     elif z.real > SMALLX or fabs(z.imag) > SMALLY:
 *         return loggamma_stirling(z)
 *     elif zabs(z - 1) <= TAYLOR_RADIUS:             # <<<<<<<<<<<<<<
 *         return loggamma_taylor(z)
 *     elif zabs(z - 2) <= TAYLOR_RADIUS:
 */
  __pyx_t_1 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__Pyx_c_diff_double(__pyx_v_z, __pyx_t_double_complex_from_parts(1, 0))) <= 0.2) != 0);
  if (__pyx_t_1) {

    /* "_loggamma.pxd":62
 *         return loggamma_stirling(z)
 *     elif zabs(z - 1) <= TAYLOR_RADIUS:
 *         return loggamma_taylor(z)             # <<<<<<<<<<<<<<
 *     elif zabs(z - 2) <= TAYLOR_RADIUS:
 *         # Recurrence relation and the Taylor series around 1
 */
    __pyx_r = __pyx_f_5scipy_7special_9_loggamma_loggamma_taylor(__pyx_v_z);
    goto __pyx_L0;

    /* "_loggamma.pxd":61
 *     elif z.real > SMALLX or fabs(z.imag) > SMALLY:
 *         return loggamma_stirling(z)
 *     elif zabs(z - 1) <= TAYLOR_RADIUS:             # <<<<<<<<<<<<<<
 *         return loggamma_taylor(z)
 *     elif zabs(z - 2) <= TAYLOR_RADIUS:
 */
  }

  /* "_loggamma.pxd":63
 *     elif zabs(z - 1) <= TAYLOR_RADIUS:
 *         return loggamma_taylor(z)
 *     elif zabs(z - 2) <= TAYLOR_RADIUS:             # <<<<<<<<<<<<<<
 *         # Recurrence relation and the Taylor series around 1
 *         return zlog1(z - 1) + loggamma_taylor(z - 1)
 */
  __pyx_t_1 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__Pyx_c_diff_double(__pyx_v_z, __pyx_t_double_complex_from_parts(2, 0))) <= 0.2) != 0);
  if (__pyx_t_1) {

    /* "_loggamma.pxd":65
 *     elif zabs(z - 2) <= TAYLOR_RADIUS:
 *         # Recurrence relation and the Taylor series around 1
 *         return zlog1(z - 1) + loggamma_taylor(z - 1)             # <<<<<<<<<<<<<<
 *     elif z.real < 0.1:
 *         # Reflection formula; see Proposition 3.1 in [1]
 */
    __pyx_r = __Pyx_c_sum_double(__pyx_f_5scipy_7special_13_complexstuff_zlog1(__Pyx_c_diff_double(__pyx_v_z, __pyx_t_double_complex_from_parts(1, 0))), __pyx_f_5scipy_7special_9_loggamma_loggamma_taylor(__Pyx_c_diff_double(__pyx_v_z, __pyx_t_double_complex_from_parts(1, 0))));
    goto __pyx_L0;

    /* "_loggamma.pxd":63
 *     elif zabs(z - 1) <= TAYLOR_RADIUS:
 *         return loggamma_taylor(z)
 *     elif zabs(z - 2) <= TAYLOR_RADIUS:             # <<<<<<<<<<<<<<
 *         # Recurrence relation and the Taylor series around 1
 *         return zlog1(z - 1) + loggamma_taylor(z - 1)
 */
  }

  /* "_loggamma.pxd":66
 *         # Recurrence relation and the Taylor series around 1
 *         return zlog1(z - 1) + loggamma_taylor(z - 1)
 *     elif z.real < 0.1:             # <<<<<<<<<<<<<<
 *         # Reflection formula; see Proposition 3.1 in [1]
 *         tmp = npy_copysign(TWOPI, z.imag)*floor(0.5*z.real + 0.25)
 */
  __pyx_t_1 = ((__Pyx_CREAL(__pyx_v_z) < 0.1) != 0);
  if (__pyx_t_1) {

    /* "_loggamma.pxd":68
 *     elif z.real < 0.1:
 *         # Reflection formula; see Proposition 3.1 in [1]
 *         tmp = npy_copysign(TWOPI, z.imag)*floor(0.5*z.real + 0.25)             # <<<<<<<<<<<<<<
 *         return zpack(LOGPI, tmp) - zlog(sinpi(z)) - loggamma(1 - z)
 *     elif npy_signbit(z.imag) == 0:
 */
    __pyx_v_tmp = (npy_copysign(6.283185307179586, __Pyx_CIMAG(__pyx_v_z)) * floor(((0.5 * __Pyx_CREAL(__pyx_v_z)) + 0.25)));

    /* "_loggamma.pxd":69
 *         # Reflection formula; see Proposition 3.1 in [1]
 *         tmp = npy_copysign(TWOPI, z.imag)*floor(0.5*z.real + 0.25)
 *         return zpack(LOGPI, tmp) - zlog(sinpi(z)) - loggamma(1 - z)             # <<<<<<<<<<<<<<
 *     elif npy_signbit(z.imag) == 0:
 *         # z.imag >= 0 but is not -0.0
 */
    __pyx_r = __Pyx_c_diff_double(__Pyx_c_diff_double(__pyx_f_5scipy_7special_13_complexstuff_zpack(1.1447298858494002, __pyx_v_tmp), __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zlog(__pyx_fuse_1__pyx_f_5scipy_7special_5_trig_sinpi(__pyx_v_z))), __pyx_f_5scipy_7special_9_loggamma_loggamma(__Pyx_c_diff_double(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_z)));
    goto __pyx_L0;

    /* "_loggamma.pxd":66
 *         # Recurrence relation and the Taylor series around 1
 *         return zlog1(z - 1) + loggamma_taylor(z - 1)
 *     elif z.real < 0.1:             # <<<<<<<<<<<<<<
 *         # Reflection formula; see Proposition 3.1 in [1]
 *         tmp = npy_copysign(TWOPI, z.imag)*floor(0.5*z.real + 0.25)
 */
  }

  /* "_loggamma.pxd":70
 *         tmp = npy_copysign(TWOPI, z.imag)*floor(0.5*z.real + 0.25)
 *         return zpack(LOGPI, tmp) - zlog(sinpi(z)) - loggamma(1 - z)
 *     elif npy_signbit(z.imag) == 0:             # <<<<<<<<<<<<<<
 *         # z.imag >= 0 but is not -0.0
 *         return loggamma_recurrence(z)
 */
  __pyx_t_1 = ((npy_signbit(__Pyx_CIMAG(__pyx_v_z)) == 0) != 0);
  if (__pyx_t_1) {

    /* "_loggamma.pxd":72
 *     elif npy_signbit(z.imag) == 0:
 *         # z.imag >= 0 but is not -0.0
 *         return loggamma_recurrence(z)             # <<<<<<<<<<<<<<
 *     else:
 *         return loggamma_recurrence(z.conjugate()).conjugate()
 */
    __pyx_r = __pyx_f_5scipy_7special_9_loggamma_loggamma_recurrence(__pyx_v_z);
    goto __pyx_L0;

    /* "_loggamma.pxd":70
 *         tmp = npy_copysign(TWOPI, z.imag)*floor(0.5*z.real + 0.25)
 *         return zpack(LOGPI, tmp) - zlog(sinpi(z)) - loggamma(1 - z)
 *     elif npy_signbit(z.imag) == 0:             # <<<<<<<<<<<<<<
 *         # z.imag >= 0 but is not -0.0
 *         return loggamma_recurrence(z)
 */
  }

  /* "_loggamma.pxd":74
 *         return loggamma_recurrence(z)
 *     else:
 *         return loggamma_recurrence(z.conjugate()).conjugate()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_r = __Pyx_c_conj_double(__pyx_f_5scipy_7special_9_loggamma_loggamma_recurrence(__Pyx_c_conj_double(__pyx_v_z)));
    goto __pyx_L0;
  }

  /* "_loggamma.pxd":50
 * 
 * @cython.cdivision(True)
 * cdef inline double complex loggamma(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """Compute the principal branch of log-Gamma."""
 *     cdef double tmp
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_loggamma.pxd":78
 * 
 * @cython.cdivision(True)
 * cdef inline double complex loggamma_recurrence(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """Backward recurrence relation.
 * 
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_9_loggamma_loggamma_recurrence(__pyx_t_double_complex __pyx_v_z) {
  int __pyx_v_signflips;
  int __pyx_v_sb;
  int __pyx_v_nsb;
  __pyx_t_double_complex __pyx_v_shiftprod;
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  long __pyx_t_2;
  int __pyx_t_3;

  /* "_loggamma.pxd":85
 *     """
 *     cdef:
 *         int signflips = 0             # <<<<<<<<<<<<<<
 *         int sb = 0
 *         int nsb
 */
  __pyx_v_signflips = 0;

  /* "_loggamma.pxd":86
 *     cdef:
 *         int signflips = 0
 *         int sb = 0             # <<<<<<<<<<<<<<
 *         int nsb
 *         double complex shiftprod = z
 */
  __pyx_v_sb = 0;

  /* "_loggamma.pxd":88
 *         int sb = 0
 *         int nsb
 *         double complex shiftprod = z             # <<<<<<<<<<<<<<
 * 
 *     z.real += 1
 */
  __pyx_v_shiftprod = __pyx_v_z;

  /* "_loggamma.pxd":90
 *         double complex shiftprod = z
 * 
 *     z.real += 1             # <<<<<<<<<<<<<<
 *     while z.real <= SMALLX:
 *         shiftprod *= z
 */
  __Pyx_SET_CREAL(__pyx_v_z, (__Pyx_CREAL(__pyx_v_z) + 1.0));

  /* "_loggamma.pxd":91
 * 
 *     z.real += 1
 *     while z.real <= SMALLX:             # <<<<<<<<<<<<<<
 *         shiftprod *= z
 *         nsb = npy_signbit(shiftprod.imag)
 */
  while (1) {
    __pyx_t_1 = ((__Pyx_CREAL(__pyx_v_z) <= 7.0) != 0);
    if (!__pyx_t_1) break;

    /* "_loggamma.pxd":92
 *     z.real += 1
 *     while z.real <= SMALLX:
 *         shiftprod *= z             # <<<<<<<<<<<<<<
 *         nsb = npy_signbit(shiftprod.imag)
 *         signflips += 1 if nsb != 0 and sb == 0 else 0
 */
    __pyx_v_shiftprod = __Pyx_c_prod_double(__pyx_v_shiftprod, __pyx_v_z);

    /* "_loggamma.pxd":93
 *     while z.real <= SMALLX:
 *         shiftprod *= z
 *         nsb = npy_signbit(shiftprod.imag)             # <<<<<<<<<<<<<<
 *         signflips += 1 if nsb != 0 and sb == 0 else 0
 *         sb = nsb
 */
    __pyx_v_nsb = npy_signbit(__Pyx_CIMAG(__pyx_v_shiftprod));

    /* "_loggamma.pxd":94
 *         shiftprod *= z
 *         nsb = npy_signbit(shiftprod.imag)
 *         signflips += 1 if nsb != 0 and sb == 0 else 0             # <<<<<<<<<<<<<<
 *         sb = nsb
 *         z.real += 1
 */
    __pyx_t_3 = ((__pyx_v_nsb != 0) != 0);
    if (__pyx_t_3) {
    } else {
      __pyx_t_1 = __pyx_t_3;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_sb == 0) != 0);
    __pyx_t_1 = __pyx_t_3;
    __pyx_L5_bool_binop_done:;
    if (__pyx_t_1) {
      __pyx_t_2 = 1;
    } else {
      __pyx_t_2 = 0;
    }
    __pyx_v_signflips = (__pyx_v_signflips + __pyx_t_2);

    /* "_loggamma.pxd":95
 *         nsb = npy_signbit(shiftprod.imag)
 *         signflips += 1 if nsb != 0 and sb == 0 else 0
 *         sb = nsb             # <<<<<<<<<<<<<<
 *         z.real += 1
 *     return loggamma_stirling(z) - zlog(shiftprod) - signflips*TWOPI*1J
 */
    __pyx_v_sb = __pyx_v_nsb;

    /* "_loggamma.pxd":96
 *         signflips += 1 if nsb != 0 and sb == 0 else 0
 *         sb = nsb
 *         z.real += 1             # <<<<<<<<<<<<<<
 *     return loggamma_stirling(z) - zlog(shiftprod) - signflips*TWOPI*1J
 * 
 */
    __Pyx_SET_CREAL(__pyx_v_z, (__Pyx_CREAL(__pyx_v_z) + 1.0));
  }

  /* "_loggamma.pxd":97
 *         sb = nsb
 *         z.real += 1
 *     return loggamma_stirling(z) - zlog(shiftprod) - signflips*TWOPI*1J             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __Pyx_c_diff_double(__Pyx_c_diff_double(__pyx_f_5scipy_7special_9_loggamma_loggamma_stirling(__pyx_v_z), __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zlog(__pyx_v_shiftprod)), __Pyx_c_prod_double(__pyx_t_double_complex_from_parts((__pyx_v_signflips * 6.283185307179586), 0), __pyx_t_double_complex_from_parts(0, 1.0)));
  goto __pyx_L0;

  /* "_loggamma.pxd":78
 * 
 * @cython.cdivision(True)
 * cdef inline double complex loggamma_recurrence(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """Backward recurrence relation.
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_loggamma.pxd":101
 * 
 * @cython.cdivision(True)
 * cdef inline double complex loggamma_stirling(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """Stirling series for log-Gamma.
 * 
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_9_loggamma_loggamma_stirling(__pyx_t_double_complex __pyx_v_z) {
  double *__pyx_v_coeffs;
  __pyx_t_double_complex __pyx_v_rz;
  __pyx_t_double_complex __pyx_v_rzz;
  __pyx_t_double_complex __pyx_r;
  double __pyx_t_1[8];

  /* "_loggamma.pxd":109
 *     """
 *     cdef:
 *         double *coeffs = [             # <<<<<<<<<<<<<<
 *             -2.955065359477124183e-2, 6.4102564102564102564e-3,
 *             -1.9175269175269175269e-3, 8.4175084175084175084e-4,
 */
  __pyx_t_1[0] = -2.955065359477124183e-2;
  __pyx_t_1[1] = 6.4102564102564102564e-3;
  __pyx_t_1[2] = -1.9175269175269175269e-3;
  __pyx_t_1[3] = 8.4175084175084175084e-4;
  __pyx_t_1[4] = -5.952380952380952381e-4;
  __pyx_t_1[5] = 7.9365079365079365079e-4;
  __pyx_t_1[6] = -2.7777777777777777778e-3;
  __pyx_t_1[7] = 8.3333333333333333333e-2;
  __pyx_v_coeffs = __pyx_t_1;

  /* "_loggamma.pxd":115
 *             -2.7777777777777777778e-3, 8.3333333333333333333e-2
 *         ]
 *         double complex rz = 1.0/z             # <<<<<<<<<<<<<<
 *         double complex rzz = rz/z
 * 
 */
  __pyx_v_rz = __Pyx_c_quot_double(__pyx_t_double_complex_from_parts(1.0, 0), __pyx_v_z);

  /* "_loggamma.pxd":116
 *         ]
 *         double complex rz = 1.0/z
 *         double complex rzz = rz/z             # <<<<<<<<<<<<<<
 * 
 *     return (z - 0.5)*zlog(z) - z + HLOG2PI + rz*cevalpoly(coeffs, 7, rzz)
 */
  __pyx_v_rzz = __Pyx_c_quot_double(__pyx_v_rz, __pyx_v_z);

  /* "_loggamma.pxd":118
 *         double complex rzz = rz/z
 * 
 *     return (z - 0.5)*zlog(z) - z + HLOG2PI + rz*cevalpoly(coeffs, 7, rzz)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __Pyx_c_sum_double(__Pyx_c_sum_double(__Pyx_c_diff_double(__Pyx_c_prod_double(__Pyx_c_diff_double(__pyx_v_z, __pyx_t_double_complex_from_parts(0.5, 0)), __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zlog(__pyx_v_z)), __pyx_v_z), __pyx_t_double_complex_from_parts(0.9189385332046728, 0)), __Pyx_c_prod_double(__pyx_v_rz, __pyx_f_5scipy_7special_9_evalpoly_cevalpoly(__pyx_v_coeffs, 7, __pyx_v_rzz)));
  goto __pyx_L0;

  /* "_loggamma.pxd":101
 * 
 * @cython.cdivision(True)
 * cdef inline double complex loggamma_stirling(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """Stirling series for log-Gamma.
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_loggamma.pxd":122
 * 
 * @cython.cdivision(True)
 * cdef inline double complex loggamma_taylor(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """Taylor series for log-Gamma around z = 1.
 * 
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_9_loggamma_loggamma_taylor(__pyx_t_double_complex __pyx_v_z) {
  double *__pyx_v_coeffs;
  __pyx_t_double_complex __pyx_r;
  double __pyx_t_1[23];

  /* "_loggamma.pxd":133
 *     """
 *     cdef:
 *         double *coeffs = [             # <<<<<<<<<<<<<<
 *             -4.3478266053040259361e-2, 4.5454556293204669442e-2,
 *             -4.7619070330142227991e-2, 5.000004769810169364e-2,
 */
  __pyx_t_1[0] = -4.3478266053040259361e-2;
  __pyx_t_1[1] = 4.5454556293204669442e-2;
  __pyx_t_1[2] = -4.7619070330142227991e-2;
  __pyx_t_1[3] = 5.000004769810169364e-2;
  __pyx_t_1[4] = -5.2631679379616660734e-2;
  __pyx_t_1[5] = 5.5555767627403611102e-2;
  __pyx_t_1[6] = -5.8823978658684582339e-2;
  __pyx_t_1[7] = 6.2500955141213040742e-2;
  __pyx_t_1[8] = -6.6668705882420468033e-2;
  __pyx_t_1[9] = 7.1432946295361336059e-2;
  __pyx_t_1[10] = -7.6932516411352191473e-2;
  __pyx_t_1[11] = 8.3353840546109004025e-2;
  __pyx_t_1[12] = -9.0954017145829042233e-2;
  __pyx_t_1[13] = 1.0009945751278180853e-1;
  __pyx_t_1[14] = -1.1133426586956469049e-1;
  __pyx_t_1[15] = 1.2550966952474304242e-1;
  __pyx_t_1[16] = -1.4404989676884611812e-1;
  __pyx_t_1[17] = 1.6955717699740818995e-1;
  __pyx_t_1[18] = -2.0738555102867398527e-1;
  __pyx_t_1[19] = 2.7058080842778454788e-1;
  __pyx_t_1[20] = -4.0068563438653142847e-1;
  __pyx_t_1[21] = 8.2246703342411321824e-1;
  __pyx_t_1[22] = -5.7721566490153286061e-1;
  __pyx_v_coeffs = __pyx_t_1;

  /* "_loggamma.pxd":148
 *         ]
 * 
 *     z = z - 1             # <<<<<<<<<<<<<<
 *     return z*cevalpoly(coeffs, 22, z)
 * 
 */
  __pyx_v_z = __Pyx_c_diff_double(__pyx_v_z, __pyx_t_double_complex_from_parts(1, 0));

  /* "_loggamma.pxd":149
 * 
 *     z = z - 1
 *     return z*cevalpoly(coeffs, 22, z)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __Pyx_c_prod_double(__pyx_v_z, __pyx_f_5scipy_7special_9_evalpoly_cevalpoly(__pyx_v_coeffs, 22, __pyx_v_z));
  goto __pyx_L0;

  /* "_loggamma.pxd":122
 * 
 * @cython.cdivision(True)
 * cdef inline double complex loggamma_taylor(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """Taylor series for log-Gamma around z = 1.
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_loggamma.pxd":152
 * 
 * 
 * cdef inline double complex cgamma(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """Compute Gamma(z) using loggamma."""
 *     if z.real <= 0 and z == floor(z.real):
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_9_loggamma_cgamma(__pyx_t_double_complex __pyx_v_z) {
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_loggamma.pxd":154
 * cdef inline double complex cgamma(double complex z) nogil:
 *     """Compute Gamma(z) using loggamma."""
 *     if z.real <= 0 and z == floor(z.real):             # <<<<<<<<<<<<<<
 *         # Poles
 *         sf_error.error("gamma", sf_error.SINGULAR, NULL)
 */
  __pyx_t_2 = ((__Pyx_CREAL(__pyx_v_z) <= 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__Pyx_c_eq_double(__pyx_v_z, __pyx_t_double_complex_from_parts(floor(__Pyx_CREAL(__pyx_v_z)), 0))) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_loggamma.pxd":156
 *     if z.real <= 0 and z == floor(z.real):
 *         # Poles
 *         sf_error.error("gamma", sf_error.SINGULAR, NULL)             # <<<<<<<<<<<<<<
 *         return nan + 1J*nan
 *     return zexp(loggamma(z))
 */
    sf_error(((char *)"gamma"), SF_ERROR_SINGULAR, NULL);

    /* "_loggamma.pxd":157
 *         # Poles
 *         sf_error.error("gamma", sf_error.SINGULAR, NULL)
 *         return nan + 1J*nan             # <<<<<<<<<<<<<<
 *     return zexp(loggamma(z))
 * 
 */
    __pyx_r = __Pyx_c_sum_double(__pyx_t_double_complex_from_parts(NPY_NAN, 0), __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0, 1.0), __pyx_t_double_complex_from_parts(NPY_NAN, 0)));
    goto __pyx_L0;

    /* "_loggamma.pxd":154
 * cdef inline double complex cgamma(double complex z) nogil:
 *     """Compute Gamma(z) using loggamma."""
 *     if z.real <= 0 and z == floor(z.real):             # <<<<<<<<<<<<<<
 *         # Poles
 *         sf_error.error("gamma", sf_error.SINGULAR, NULL)
 */
  }

  /* "_loggamma.pxd":158
 *         sf_error.error("gamma", sf_error.SINGULAR, NULL)
 *         return nan + 1J*nan
 *     return zexp(loggamma(z))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zexp(__pyx_f_5scipy_7special_9_loggamma_loggamma(__pyx_v_z));
  goto __pyx_L0;

  /* "_loggamma.pxd":152
 * 
 * 
 * cdef inline double complex cgamma(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """Compute Gamma(z) using loggamma."""
 *     if z.real <= 0 and z == floor(z.real):
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_loggamma.pxd":161
 * 
 * 
 * cdef inline double complex crgamma(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """Compute 1/Gamma(z) using loggamma."""
 *     if z.real <= 0 and z == floor(z.real):
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_9_loggamma_crgamma(__pyx_t_double_complex __pyx_v_z) {
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_loggamma.pxd":163
 * cdef inline double complex crgamma(double complex z) nogil:
 *     """Compute 1/Gamma(z) using loggamma."""
 *     if z.real <= 0 and z == floor(z.real):             # <<<<<<<<<<<<<<
 *         # Zeros at 0, -1, -2, ...
 *         return 0
 */
  __pyx_t_2 = ((__Pyx_CREAL(__pyx_v_z) <= 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__Pyx_c_eq_double(__pyx_v_z, __pyx_t_double_complex_from_parts(floor(__Pyx_CREAL(__pyx_v_z)), 0))) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_loggamma.pxd":165
 *     if z.real <= 0 and z == floor(z.real):
 *         # Zeros at 0, -1, -2, ...
 *         return 0             # <<<<<<<<<<<<<<
 *     return zexp(-loggamma(z))
 */
    __pyx_r = __pyx_t_double_complex_from_parts(0, 0);
    goto __pyx_L0;

    /* "_loggamma.pxd":163
 * cdef inline double complex crgamma(double complex z) nogil:
 *     """Compute 1/Gamma(z) using loggamma."""
 *     if z.real <= 0 and z == floor(z.real):             # <<<<<<<<<<<<<<
 *         # Zeros at 0, -1, -2, ...
 *         return 0
 */
  }

  /* "_loggamma.pxd":166
 *         # Zeros at 0, -1, -2, ...
 *         return 0
 *     return zexp(-loggamma(z))             # <<<<<<<<<<<<<<
 */
  __pyx_r = __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zexp(__Pyx_c_neg_double(__pyx_f_5scipy_7special_9_loggamma_loggamma(__pyx_v_z)));
  goto __pyx_L0;

  /* "_loggamma.pxd":161
 * 
 * 
 * cdef inline double complex crgamma(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """Compute 1/Gamma(z) using loggamma."""
 *     if z.real <= 0 and z == floor(z.real):
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_xlogy.pxd":6
 * from ._cunity cimport clog1p
 * 
 * cdef inline number_t xlogy(number_t x, number_t y) nogil:             # <<<<<<<<<<<<<<
 *     if x == 0 and not zisnan(y):
 *         return 0
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_6_xlogy_xlogy(double __pyx_v_x, double __pyx_v_y) {
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_xlogy.pxd":7
 * 
 * cdef inline number_t xlogy(number_t x, number_t y) nogil:
 *     if x == 0 and not zisnan(y):             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  __pyx_t_2 = ((__pyx_v_x == 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((!(__pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zisnan(__pyx_v_y) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_xlogy.pxd":8
 * cdef inline number_t xlogy(number_t x, number_t y) nogil:
 *     if x == 0 and not zisnan(y):
 *         return 0             # <<<<<<<<<<<<<<
 *     else:
 *         return x * zlog(y)
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "_xlogy.pxd":7
 * 
 * cdef inline number_t xlogy(number_t x, number_t y) nogil:
 *     if x == 0 and not zisnan(y):             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  }

  /* "_xlogy.pxd":10
 *         return 0
 *     else:
 *         return x * zlog(y)             # <<<<<<<<<<<<<<
 * 
 * cdef inline number_t xlog1py(number_t x, number_t y) nogil:
 */
  /*else*/ {
    __pyx_r = (__pyx_v_x * __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zlog(__pyx_v_y));
    goto __pyx_L0;
  }

  /* "_xlogy.pxd":6
 * from ._cunity cimport clog1p
 * 
 * cdef inline number_t xlogy(number_t x, number_t y) nogil:             # <<<<<<<<<<<<<<
 *     if x == 0 and not zisnan(y):
 *         return 0
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_6_xlogy_xlogy(__pyx_t_double_complex __pyx_v_x, __pyx_t_double_complex __pyx_v_y) {
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_xlogy.pxd":7
 * 
 * cdef inline number_t xlogy(number_t x, number_t y) nogil:
 *     if x == 0 and not zisnan(y):             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  __pyx_t_2 = ((__Pyx_c_eq_double(__pyx_v_x, __pyx_t_double_complex_from_parts(0, 0))) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((!(__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisnan(__pyx_v_y) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_xlogy.pxd":8
 * cdef inline number_t xlogy(number_t x, number_t y) nogil:
 *     if x == 0 and not zisnan(y):
 *         return 0             # <<<<<<<<<<<<<<
 *     else:
 *         return x * zlog(y)
 */
    __pyx_r = __pyx_t_double_complex_from_parts(0, 0);
    goto __pyx_L0;

    /* "_xlogy.pxd":7
 * 
 * cdef inline number_t xlogy(number_t x, number_t y) nogil:
 *     if x == 0 and not zisnan(y):             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  }

  /* "_xlogy.pxd":10
 *         return 0
 *     else:
 *         return x * zlog(y)             # <<<<<<<<<<<<<<
 * 
 * cdef inline number_t xlog1py(number_t x, number_t y) nogil:
 */
  /*else*/ {
    __pyx_r = __Pyx_c_prod_double(__pyx_v_x, __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zlog(__pyx_v_y));
    goto __pyx_L0;
  }

  /* "_xlogy.pxd":6
 * from ._cunity cimport clog1p
 * 
 * cdef inline number_t xlogy(number_t x, number_t y) nogil:             # <<<<<<<<<<<<<<
 *     if x == 0 and not zisnan(y):
 *         return 0
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_xlogy.pxd":12
 *         return x * zlog(y)
 * 
 * cdef inline number_t xlog1py(number_t x, number_t y) nogil:             # <<<<<<<<<<<<<<
 *     if x == 0 and not zisnan(y):
 *         return 0
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_6_xlogy_xlog1py(double __pyx_v_x, double __pyx_v_y) {
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_xlogy.pxd":13
 * 
 * cdef inline number_t xlog1py(number_t x, number_t y) nogil:
 *     if x == 0 and not zisnan(y):             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  __pyx_t_2 = ((__pyx_v_x == 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((!(__pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zisnan(__pyx_v_y) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_xlogy.pxd":14
 * cdef inline number_t xlog1py(number_t x, number_t y) nogil:
 *     if x == 0 and not zisnan(y):
 *         return 0             # <<<<<<<<<<<<<<
 *     else:
 *         if number_t is double:
 */
    __pyx_r = 0.0;
    goto __pyx_L0;

    /* "_xlogy.pxd":13
 * 
 * cdef inline number_t xlog1py(number_t x, number_t y) nogil:
 *     if x == 0 and not zisnan(y):             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  }

  /* "_xlogy.pxd":16
 *         return 0
 *     else:
 *         if number_t is double:             # <<<<<<<<<<<<<<
 *             return x * npy_log1p(y)
 *         else:
 */
  /*else*/ {

    /* "_xlogy.pxd":17
 *     else:
 *         if number_t is double:
 *             return x * npy_log1p(y)             # <<<<<<<<<<<<<<
 *         else:
 *             return x * clog1p(y)
 */
    __pyx_r = (__pyx_v_x * npy_log1p(__pyx_v_y));
    goto __pyx_L0;
  }

  /* "_xlogy.pxd":12
 *         return x * zlog(y)
 * 
 * cdef inline number_t xlog1py(number_t x, number_t y) nogil:             # <<<<<<<<<<<<<<
 *     if x == 0 and not zisnan(y):
 *         return 0
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_6_xlogy_xlog1py(__pyx_t_double_complex __pyx_v_x, __pyx_t_double_complex __pyx_v_y) {
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_xlogy.pxd":13
 * 
 * cdef inline number_t xlog1py(number_t x, number_t y) nogil:
 *     if x == 0 and not zisnan(y):             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  __pyx_t_2 = ((__Pyx_c_eq_double(__pyx_v_x, __pyx_t_double_complex_from_parts(0, 0))) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((!(__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zisnan(__pyx_v_y) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_xlogy.pxd":14
 * cdef inline number_t xlog1py(number_t x, number_t y) nogil:
 *     if x == 0 and not zisnan(y):
 *         return 0             # <<<<<<<<<<<<<<
 *     else:
 *         if number_t is double:
 */
    __pyx_r = __pyx_t_double_complex_from_parts(0, 0);
    goto __pyx_L0;

    /* "_xlogy.pxd":13
 * 
 * cdef inline number_t xlog1py(number_t x, number_t y) nogil:
 *     if x == 0 and not zisnan(y):             # <<<<<<<<<<<<<<
 *         return 0
 *     else:
 */
  }

  /* "_xlogy.pxd":16
 *         return 0
 *     else:
 *         if number_t is double:             # <<<<<<<<<<<<<<
 *             return x * npy_log1p(y)
 *         else:
 */
  /*else*/ {

    /* "_xlogy.pxd":19
 *             return x * npy_log1p(y)
 *         else:
 *             return x * clog1p(y)             # <<<<<<<<<<<<<<
 */
    __pyx_r = __Pyx_c_prod_double(__pyx_v_x, __pyx_f_5scipy_7special_7_cunity_clog1p(__pyx_v_y));
    goto __pyx_L0;
  }

  /* "_xlogy.pxd":12
 *         return x * zlog(y)
 * 
 * cdef inline number_t xlog1py(number_t x, number_t y) nogil:             # <<<<<<<<<<<<<<
 *     if x == 0 and not zisnan(y):
 *         return 0
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_hyp0f1.pxd":26
 * # Real-valued kernel
 * #
 * cdef inline double _hyp0f1_real(double v, double z) nogil:             # <<<<<<<<<<<<<<
 *     cdef double arg, v1, arg_exp, bess_val
 * 
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_hyp0f1__hyp0f1_real(double __pyx_v_v, double __pyx_v_z) {
  double __pyx_v_arg;
  double __pyx_v_arg_exp;
  double __pyx_v_bess_val;
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  double __pyx_t_3;
  double __pyx_t_4;

  /* "_hyp0f1.pxd":30
 * 
 *     # handle poles, zeros
 *     if v <= 0.0 and v == floor(v):             # <<<<<<<<<<<<<<
 *         return NAN
 *     if z == 0.0 and v != 0.0:
 */
  __pyx_t_2 = ((__pyx_v_v <= 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_v == floor(__pyx_v_v)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_hyp0f1.pxd":31
 *     # handle poles, zeros
 *     if v <= 0.0 and v == floor(v):
 *         return NAN             # <<<<<<<<<<<<<<
 *     if z == 0.0 and v != 0.0:
 *         return 1.0
 */
    __pyx_r = NPY_NAN;
    goto __pyx_L0;

    /* "_hyp0f1.pxd":30
 * 
 *     # handle poles, zeros
 *     if v <= 0.0 and v == floor(v):             # <<<<<<<<<<<<<<
 *         return NAN
 *     if z == 0.0 and v != 0.0:
 */
  }

  /* "_hyp0f1.pxd":32
 *     if v <= 0.0 and v == floor(v):
 *         return NAN
 *     if z == 0.0 and v != 0.0:             # <<<<<<<<<<<<<<
 *         return 1.0
 * 
 */
  __pyx_t_2 = ((__pyx_v_z == 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_v != 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L7_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_hyp0f1.pxd":33
 *         return NAN
 *     if z == 0.0 and v != 0.0:
 *         return 1.0             # <<<<<<<<<<<<<<
 * 
 *     # both v and z small: truncate the Taylor series at O(z**2)
 */
    __pyx_r = 1.0;
    goto __pyx_L0;

    /* "_hyp0f1.pxd":32
 *     if v <= 0.0 and v == floor(v):
 *         return NAN
 *     if z == 0.0 and v != 0.0:             # <<<<<<<<<<<<<<
 *         return 1.0
 * 
 */
  }

  /* "_hyp0f1.pxd":36
 * 
 *     # both v and z small: truncate the Taylor series at O(z**2)
 *     if fabs(z) < 1e-6*(1.0 + fabs(v)):             # <<<<<<<<<<<<<<
 *         return 1.0 + z/v + z*z/(2.0*v*(v+1.0))
 * 
 */
  __pyx_t_1 = ((fabs(__pyx_v_z) < (1e-6 * (1.0 + fabs(__pyx_v_v)))) != 0);
  if (__pyx_t_1) {

    /* "_hyp0f1.pxd":37
 *     # both v and z small: truncate the Taylor series at O(z**2)
 *     if fabs(z) < 1e-6*(1.0 + fabs(v)):
 *         return 1.0 + z/v + z*z/(2.0*v*(v+1.0))             # <<<<<<<<<<<<<<
 * 
 *     if z > 0:
 */
    if (unlikely(__pyx_v_v == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      __Pyx_PyGILState_Release(__pyx_gilstate_save);
      #endif
      __PYX_ERR(9, 37, __pyx_L1_error)
    }
    __pyx_t_3 = (__pyx_v_z * __pyx_v_z);
    __pyx_t_4 = ((2.0 * __pyx_v_v) * (__pyx_v_v + 1.0));
    if (unlikely(__pyx_t_4 == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      __Pyx_PyGILState_Release(__pyx_gilstate_save);
      #endif
      __PYX_ERR(9, 37, __pyx_L1_error)
    }
    __pyx_r = ((1.0 + (__pyx_v_z / __pyx_v_v)) + (__pyx_t_3 / __pyx_t_4));
    goto __pyx_L0;

    /* "_hyp0f1.pxd":36
 * 
 *     # both v and z small: truncate the Taylor series at O(z**2)
 *     if fabs(z) < 1e-6*(1.0 + fabs(v)):             # <<<<<<<<<<<<<<
 *         return 1.0 + z/v + z*z/(2.0*v*(v+1.0))
 * 
 */
  }

  /* "_hyp0f1.pxd":39
 *         return 1.0 + z/v + z*z/(2.0*v*(v+1.0))
 * 
 *     if z > 0:             # <<<<<<<<<<<<<<
 *         arg = sqrt(z)
 *         arg_exp = xlogy(1.0-v, arg) + lgam(v)
 */
  __pyx_t_1 = ((__pyx_v_z > 0.0) != 0);
  if (__pyx_t_1) {

    /* "_hyp0f1.pxd":40
 * 
 *     if z > 0:
 *         arg = sqrt(z)             # <<<<<<<<<<<<<<
 *         arg_exp = xlogy(1.0-v, arg) + lgam(v)
 *         bess_val = iv(v-1, 2.0*arg)
 */
    __pyx_v_arg = sqrt(__pyx_v_z);

    /* "_hyp0f1.pxd":41
 *     if z > 0:
 *         arg = sqrt(z)
 *         arg_exp = xlogy(1.0-v, arg) + lgam(v)             # <<<<<<<<<<<<<<
 *         bess_val = iv(v-1, 2.0*arg)
 * 
 */
    __pyx_v_arg_exp = (__pyx_fuse_0__pyx_f_5scipy_7special_6_xlogy_xlogy((1.0 - __pyx_v_v), __pyx_v_arg) + lgam(__pyx_v_v));

    /* "_hyp0f1.pxd":42
 *         arg = sqrt(z)
 *         arg_exp = xlogy(1.0-v, arg) + lgam(v)
 *         bess_val = iv(v-1, 2.0*arg)             # <<<<<<<<<<<<<<
 * 
 *         if (arg_exp > log(DBL_MAX) or bess_val == 0 or   # overflow
 */
    __pyx_v_bess_val = iv((__pyx_v_v - 1.0), (2.0 * __pyx_v_arg));

    /* "_hyp0f1.pxd":44
 *         bess_val = iv(v-1, 2.0*arg)
 * 
 *         if (arg_exp > log(DBL_MAX) or bess_val == 0 or   # overflow             # <<<<<<<<<<<<<<
 *             arg_exp < log(DBL_MIN) or isinf(bess_val)):  # underflow
 *             return _hyp0f1_asy(v, z)
 */
    __pyx_t_2 = ((__pyx_v_arg_exp > log(DBL_MAX)) != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L12_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_bess_val == 0.0) != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L12_bool_binop_done;
    }

    /* "_hyp0f1.pxd":45
 * 
 *         if (arg_exp > log(DBL_MAX) or bess_val == 0 or   # overflow
 *             arg_exp < log(DBL_MIN) or isinf(bess_val)):  # underflow             # <<<<<<<<<<<<<<
 *             return _hyp0f1_asy(v, z)
 *         else:
 */
    __pyx_t_2 = ((__pyx_v_arg_exp < log(DBL_MIN)) != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L12_bool_binop_done;
    }
    __pyx_t_2 = (npy_isinf(__pyx_v_bess_val) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L12_bool_binop_done:;

    /* "_hyp0f1.pxd":44
 *         bess_val = iv(v-1, 2.0*arg)
 * 
 *         if (arg_exp > log(DBL_MAX) or bess_val == 0 or   # overflow             # <<<<<<<<<<<<<<
 *             arg_exp < log(DBL_MIN) or isinf(bess_val)):  # underflow
 *             return _hyp0f1_asy(v, z)
 */
    if (__pyx_t_1) {

      /* "_hyp0f1.pxd":46
 *         if (arg_exp > log(DBL_MAX) or bess_val == 0 or   # overflow
 *             arg_exp < log(DBL_MIN) or isinf(bess_val)):  # underflow
 *             return _hyp0f1_asy(v, z)             # <<<<<<<<<<<<<<
 *         else:
 *             return exp(arg_exp) * gammasgn(v) * bess_val
 */
      __pyx_r = __pyx_f_5scipy_7special_7_hyp0f1__hyp0f1_asy(__pyx_v_v, __pyx_v_z);
      goto __pyx_L0;

      /* "_hyp0f1.pxd":44
 *         bess_val = iv(v-1, 2.0*arg)
 * 
 *         if (arg_exp > log(DBL_MAX) or bess_val == 0 or   # overflow             # <<<<<<<<<<<<<<
 *             arg_exp < log(DBL_MIN) or isinf(bess_val)):  # underflow
 *             return _hyp0f1_asy(v, z)
 */
    }

    /* "_hyp0f1.pxd":48
 *             return _hyp0f1_asy(v, z)
 *         else:
 *             return exp(arg_exp) * gammasgn(v) * bess_val             # <<<<<<<<<<<<<<
 *     else:
 *         arg = sqrt(-z)
 */
    /*else*/ {
      __pyx_r = ((exp(__pyx_v_arg_exp) * gammasgn(__pyx_v_v)) * __pyx_v_bess_val);
      goto __pyx_L0;
    }

    /* "_hyp0f1.pxd":39
 *         return 1.0 + z/v + z*z/(2.0*v*(v+1.0))
 * 
 *     if z > 0:             # <<<<<<<<<<<<<<
 *         arg = sqrt(z)
 *         arg_exp = xlogy(1.0-v, arg) + lgam(v)
 */
  }

  /* "_hyp0f1.pxd":50
 *             return exp(arg_exp) * gammasgn(v) * bess_val
 *     else:
 *         arg = sqrt(-z)             # <<<<<<<<<<<<<<
 *         return pow(arg, 1.0 - v) * Gamma(v) * jv(v - 1, 2*arg)
 * 
 */
  /*else*/ {
    __pyx_v_arg = sqrt((-__pyx_v_z));

    /* "_hyp0f1.pxd":51
 *     else:
 *         arg = sqrt(-z)
 *         return pow(arg, 1.0 - v) * Gamma(v) * jv(v - 1, 2*arg)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_r = ((pow(__pyx_v_arg, (1.0 - __pyx_v_v)) * Gamma(__pyx_v_v)) * jv((__pyx_v_v - 1.0), (2.0 * __pyx_v_arg)));
    goto __pyx_L0;
  }

  /* "_hyp0f1.pxd":26
 * # Real-valued kernel
 * #
 * cdef inline double _hyp0f1_real(double v, double z) nogil:             # <<<<<<<<<<<<<<
 *     cdef double arg, v1, arg_exp, bess_val
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special._hyp0f1._hyp0f1_real", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "_hyp0f1.pxd":54
 * 
 * 
 * cdef inline double _hyp0f1_asy(double v, double z) nogil:             # <<<<<<<<<<<<<<
 *     r"""Asymptotic expansion for I_{v-1}(2*sqrt(z)) * Gamma(v)
 *     for real $z > 0$ and $v\to +\infty$.
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_7_hyp0f1__hyp0f1_asy(double __pyx_v_v, double __pyx_v_z) {
  double __pyx_v_arg;
  double __pyx_v_v1;
  double __pyx_v_x;
  double __pyx_v_p1;
  double __pyx_v_eta;
  double __pyx_v_arg_exp_i;
  double __pyx_v_arg_exp_k;
  double __pyx_v_pp;
  double __pyx_v_p2;
  double __pyx_v_p4;
  double __pyx_v_p6;
  double __pyx_v_u1;
  double __pyx_v_u2;
  double __pyx_v_u3;
  double __pyx_v_u_corr_i;
  double __pyx_v_u_corr_k;
  double __pyx_v_result;
  double __pyx_v_gs;
  double __pyx_r;
  double __pyx_t_1;
  double __pyx_t_2;
  int __pyx_t_3;

  /* "_hyp0f1.pxd":61
 *     """
 *     cdef:
 *         double arg = sqrt(z)             # <<<<<<<<<<<<<<
 *         double v1 = fabs(v - 1)
 *         double x = 2.0 * arg / v1
 */
  __pyx_v_arg = sqrt(__pyx_v_z);

  /* "_hyp0f1.pxd":62
 *     cdef:
 *         double arg = sqrt(z)
 *         double v1 = fabs(v - 1)             # <<<<<<<<<<<<<<
 *         double x = 2.0 * arg / v1
 *         double p1 = sqrt(1.0 + x*x)
 */
  __pyx_v_v1 = fabs((__pyx_v_v - 1.0));

  /* "_hyp0f1.pxd":63
 *         double arg = sqrt(z)
 *         double v1 = fabs(v - 1)
 *         double x = 2.0 * arg / v1             # <<<<<<<<<<<<<<
 *         double p1 = sqrt(1.0 + x*x)
 *         double eta = p1 + log(x) - log1p(p1)
 */
  __pyx_t_1 = (2.0 * __pyx_v_arg);
  if (unlikely(__pyx_v_v1 == 0)) {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
    #endif
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    #ifdef WITH_THREAD
    __Pyx_PyGILState_Release(__pyx_gilstate_save);
    #endif
    __PYX_ERR(9, 63, __pyx_L1_error)
  }
  __pyx_v_x = (__pyx_t_1 / __pyx_v_v1);

  /* "_hyp0f1.pxd":64
 *         double v1 = fabs(v - 1)
 *         double x = 2.0 * arg / v1
 *         double p1 = sqrt(1.0 + x*x)             # <<<<<<<<<<<<<<
 *         double eta = p1 + log(x) - log1p(p1)
 *         double arg_exp_i, arg_exp_k
 */
  __pyx_v_p1 = sqrt((1.0 + (__pyx_v_x * __pyx_v_x)));

  /* "_hyp0f1.pxd":65
 *         double x = 2.0 * arg / v1
 *         double p1 = sqrt(1.0 + x*x)
 *         double eta = p1 + log(x) - log1p(p1)             # <<<<<<<<<<<<<<
 *         double arg_exp_i, arg_exp_k
 *         double pp, p2, p4, p6, u1, u2, u3, u_corr_i, u_corr_k
 */
  __pyx_v_eta = ((__pyx_v_p1 + log(__pyx_v_x)) - log1p(__pyx_v_p1));

  /* "_hyp0f1.pxd":70
 *         double result, gs
 * 
 *     arg_exp_i = -0.5*log(p1)             # <<<<<<<<<<<<<<
 *     arg_exp_i -= 0.5*log(2.0*M_PI*v1)
 *     arg_exp_i += lgam(v)
 */
  __pyx_v_arg_exp_i = (-0.5 * log(__pyx_v_p1));

  /* "_hyp0f1.pxd":71
 * 
 *     arg_exp_i = -0.5*log(p1)
 *     arg_exp_i -= 0.5*log(2.0*M_PI*v1)             # <<<<<<<<<<<<<<
 *     arg_exp_i += lgam(v)
 *     gs = gammasgn(v)
 */
  __pyx_v_arg_exp_i = (__pyx_v_arg_exp_i - (0.5 * log(((2.0 * M_PI) * __pyx_v_v1))));

  /* "_hyp0f1.pxd":72
 *     arg_exp_i = -0.5*log(p1)
 *     arg_exp_i -= 0.5*log(2.0*M_PI*v1)
 *     arg_exp_i += lgam(v)             # <<<<<<<<<<<<<<
 *     gs = gammasgn(v)
 * 
 */
  __pyx_v_arg_exp_i = (__pyx_v_arg_exp_i + lgam(__pyx_v_v));

  /* "_hyp0f1.pxd":73
 *     arg_exp_i -= 0.5*log(2.0*M_PI*v1)
 *     arg_exp_i += lgam(v)
 *     gs = gammasgn(v)             # <<<<<<<<<<<<<<
 * 
 *     arg_exp_k = arg_exp_i
 */
  __pyx_v_gs = gammasgn(__pyx_v_v);

  /* "_hyp0f1.pxd":75
 *     gs = gammasgn(v)
 * 
 *     arg_exp_k = arg_exp_i             # <<<<<<<<<<<<<<
 *     arg_exp_i += v1 * eta
 *     arg_exp_k -= v1 * eta
 */
  __pyx_v_arg_exp_k = __pyx_v_arg_exp_i;

  /* "_hyp0f1.pxd":76
 * 
 *     arg_exp_k = arg_exp_i
 *     arg_exp_i += v1 * eta             # <<<<<<<<<<<<<<
 *     arg_exp_k -= v1 * eta
 * 
 */
  __pyx_v_arg_exp_i = (__pyx_v_arg_exp_i + (__pyx_v_v1 * __pyx_v_eta));

  /* "_hyp0f1.pxd":77
 *     arg_exp_k = arg_exp_i
 *     arg_exp_i += v1 * eta
 *     arg_exp_k -= v1 * eta             # <<<<<<<<<<<<<<
 * 
 *     # large-v asymptotic correction, DLMF 10.41.10
 */
  __pyx_v_arg_exp_k = (__pyx_v_arg_exp_k - (__pyx_v_v1 * __pyx_v_eta));

  /* "_hyp0f1.pxd":80
 * 
 *     # large-v asymptotic correction, DLMF 10.41.10
 *     pp = 1.0/p1             # <<<<<<<<<<<<<<
 *     p2 = pp*pp
 *     p4 = p2*p2
 */
  if (unlikely(__pyx_v_p1 == 0)) {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
    #endif
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    #ifdef WITH_THREAD
    __Pyx_PyGILState_Release(__pyx_gilstate_save);
    #endif
    __PYX_ERR(9, 80, __pyx_L1_error)
  }
  __pyx_v_pp = (1.0 / __pyx_v_p1);

  /* "_hyp0f1.pxd":81
 *     # large-v asymptotic correction, DLMF 10.41.10
 *     pp = 1.0/p1
 *     p2 = pp*pp             # <<<<<<<<<<<<<<
 *     p4 = p2*p2
 *     p6 = p4*p2
 */
  __pyx_v_p2 = (__pyx_v_pp * __pyx_v_pp);

  /* "_hyp0f1.pxd":82
 *     pp = 1.0/p1
 *     p2 = pp*pp
 *     p4 = p2*p2             # <<<<<<<<<<<<<<
 *     p6 = p4*p2
 *     u1 = (3.0 - 5.0*p2) * pp / 24.0
 */
  __pyx_v_p4 = (__pyx_v_p2 * __pyx_v_p2);

  /* "_hyp0f1.pxd":83
 *     p2 = pp*pp
 *     p4 = p2*p2
 *     p6 = p4*p2             # <<<<<<<<<<<<<<
 *     u1 = (3.0 - 5.0*p2) * pp / 24.0
 *     u2 = (81.0 - 462.0*p2 + 385.0*p4) * p2 / 1152.0
 */
  __pyx_v_p6 = (__pyx_v_p4 * __pyx_v_p2);

  /* "_hyp0f1.pxd":84
 *     p4 = p2*p2
 *     p6 = p4*p2
 *     u1 = (3.0 - 5.0*p2) * pp / 24.0             # <<<<<<<<<<<<<<
 *     u2 = (81.0 - 462.0*p2 + 385.0*p4) * p2 / 1152.0
 *     u3 = (30375.0 - 369603.0*p2 + 765765.0*p4 - 425425.0*p6) * pp * p2 / 414720.0
 */
  __pyx_v_u1 = (((3.0 - (5.0 * __pyx_v_p2)) * __pyx_v_pp) / 24.0);

  /* "_hyp0f1.pxd":85
 *     p6 = p4*p2
 *     u1 = (3.0 - 5.0*p2) * pp / 24.0
 *     u2 = (81.0 - 462.0*p2 + 385.0*p4) * p2 / 1152.0             # <<<<<<<<<<<<<<
 *     u3 = (30375.0 - 369603.0*p2 + 765765.0*p4 - 425425.0*p6) * pp * p2 / 414720.0
 *     u_corr_i = 1.0 + u1/v1 + u2/(v1*v1) + u3/(v1*v1*v1)
 */
  __pyx_v_u2 = ((((81.0 - (462.0 * __pyx_v_p2)) + (385.0 * __pyx_v_p4)) * __pyx_v_p2) / 1152.0);

  /* "_hyp0f1.pxd":86
 *     u1 = (3.0 - 5.0*p2) * pp / 24.0
 *     u2 = (81.0 - 462.0*p2 + 385.0*p4) * p2 / 1152.0
 *     u3 = (30375.0 - 369603.0*p2 + 765765.0*p4 - 425425.0*p6) * pp * p2 / 414720.0             # <<<<<<<<<<<<<<
 *     u_corr_i = 1.0 + u1/v1 + u2/(v1*v1) + u3/(v1*v1*v1)
 * 
 */
  __pyx_v_u3 = ((((((30375.0 - (369603.0 * __pyx_v_p2)) + (765765.0 * __pyx_v_p4)) - (425425.0 * __pyx_v_p6)) * __pyx_v_pp) * __pyx_v_p2) / 414720.0);

  /* "_hyp0f1.pxd":87
 *     u2 = (81.0 - 462.0*p2 + 385.0*p4) * p2 / 1152.0
 *     u3 = (30375.0 - 369603.0*p2 + 765765.0*p4 - 425425.0*p6) * pp * p2 / 414720.0
 *     u_corr_i = 1.0 + u1/v1 + u2/(v1*v1) + u3/(v1*v1*v1)             # <<<<<<<<<<<<<<
 * 
 *     result = exp(arg_exp_i - xlogy(v1, arg)) * gs * u_corr_i
 */
  if (unlikely(__pyx_v_v1 == 0)) {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
    #endif
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    #ifdef WITH_THREAD
    __Pyx_PyGILState_Release(__pyx_gilstate_save);
    #endif
    __PYX_ERR(9, 87, __pyx_L1_error)
  }
  __pyx_t_1 = (__pyx_v_v1 * __pyx_v_v1);
  if (unlikely(__pyx_t_1 == 0)) {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
    #endif
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    #ifdef WITH_THREAD
    __Pyx_PyGILState_Release(__pyx_gilstate_save);
    #endif
    __PYX_ERR(9, 87, __pyx_L1_error)
  }
  __pyx_t_2 = ((__pyx_v_v1 * __pyx_v_v1) * __pyx_v_v1);
  if (unlikely(__pyx_t_2 == 0)) {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
    #endif
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    #ifdef WITH_THREAD
    __Pyx_PyGILState_Release(__pyx_gilstate_save);
    #endif
    __PYX_ERR(9, 87, __pyx_L1_error)
  }
  __pyx_v_u_corr_i = (((1.0 + (__pyx_v_u1 / __pyx_v_v1)) + (__pyx_v_u2 / __pyx_t_1)) + (__pyx_v_u3 / __pyx_t_2));

  /* "_hyp0f1.pxd":89
 *     u_corr_i = 1.0 + u1/v1 + u2/(v1*v1) + u3/(v1*v1*v1)
 * 
 *     result = exp(arg_exp_i - xlogy(v1, arg)) * gs * u_corr_i             # <<<<<<<<<<<<<<
 *     if v - 1 < 0:
 *         # DLMF 10.27.2: I_{-v} = I_{v} + (2/pi) sin(pi*v) K_v
 */
  __pyx_v_result = ((exp((__pyx_v_arg_exp_i - __pyx_fuse_0__pyx_f_5scipy_7special_6_xlogy_xlogy(__pyx_v_v1, __pyx_v_arg))) * __pyx_v_gs) * __pyx_v_u_corr_i);

  /* "_hyp0f1.pxd":90
 * 
 *     result = exp(arg_exp_i - xlogy(v1, arg)) * gs * u_corr_i
 *     if v - 1 < 0:             # <<<<<<<<<<<<<<
 *         # DLMF 10.27.2: I_{-v} = I_{v} + (2/pi) sin(pi*v) K_v
 *         u_corr_k = 1.0 - u1/v1 + u2/(v1*v1) - u3/(v1*v1*v1)
 */
  __pyx_t_3 = (((__pyx_v_v - 1.0) < 0.0) != 0);
  if (__pyx_t_3) {

    /* "_hyp0f1.pxd":92
 *     if v - 1 < 0:
 *         # DLMF 10.27.2: I_{-v} = I_{v} + (2/pi) sin(pi*v) K_v
 *         u_corr_k = 1.0 - u1/v1 + u2/(v1*v1) - u3/(v1*v1*v1)             # <<<<<<<<<<<<<<
 *         result += exp(arg_exp_k + xlogy(v1, arg)) * gs * 2.0 * sin_pi(v1) * u_corr_k
 * 
 */
    if (unlikely(__pyx_v_v1 == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      __Pyx_PyGILState_Release(__pyx_gilstate_save);
      #endif
      __PYX_ERR(9, 92, __pyx_L1_error)
    }
    __pyx_t_2 = (__pyx_v_v1 * __pyx_v_v1);
    if (unlikely(__pyx_t_2 == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      __Pyx_PyGILState_Release(__pyx_gilstate_save);
      #endif
      __PYX_ERR(9, 92, __pyx_L1_error)
    }
    __pyx_t_1 = ((__pyx_v_v1 * __pyx_v_v1) * __pyx_v_v1);
    if (unlikely(__pyx_t_1 == 0)) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      __Pyx_PyGILState_Release(__pyx_gilstate_save);
      #endif
      __PYX_ERR(9, 92, __pyx_L1_error)
    }
    __pyx_v_u_corr_k = (((1.0 - (__pyx_v_u1 / __pyx_v_v1)) + (__pyx_v_u2 / __pyx_t_2)) - (__pyx_v_u3 / __pyx_t_1));

    /* "_hyp0f1.pxd":93
 *         # DLMF 10.27.2: I_{-v} = I_{v} + (2/pi) sin(pi*v) K_v
 *         u_corr_k = 1.0 - u1/v1 + u2/(v1*v1) - u3/(v1*v1*v1)
 *         result += exp(arg_exp_k + xlogy(v1, arg)) * gs * 2.0 * sin_pi(v1) * u_corr_k             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
    __pyx_v_result = (__pyx_v_result + ((((exp((__pyx_v_arg_exp_k + __pyx_fuse_0__pyx_f_5scipy_7special_6_xlogy_xlogy(__pyx_v_v1, __pyx_v_arg))) * __pyx_v_gs) * 2.0) * sin_pi(__pyx_v_v1)) * __pyx_v_u_corr_k));

    /* "_hyp0f1.pxd":90
 * 
 *     result = exp(arg_exp_i - xlogy(v1, arg)) * gs * u_corr_i
 *     if v - 1 < 0:             # <<<<<<<<<<<<<<
 *         # DLMF 10.27.2: I_{-v} = I_{v} + (2/pi) sin(pi*v) K_v
 *         u_corr_k = 1.0 - u1/v1 + u2/(v1*v1) - u3/(v1*v1*v1)
 */
  }

  /* "_hyp0f1.pxd":95
 *         result += exp(arg_exp_k + xlogy(v1, arg)) * gs * 2.0 * sin_pi(v1) * u_corr_k
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "_hyp0f1.pxd":54
 * 
 * 
 * cdef inline double _hyp0f1_asy(double v, double z) nogil:             # <<<<<<<<<<<<<<
 *     r"""Asymptotic expansion for I_{v-1}(2*sqrt(z)) * Gamma(v)
 *     for real $z > 0$ and $v\to +\infty$.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special._hyp0f1._hyp0f1_asy", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "_hyp0f1.pxd":101
 * # Complex valued kernel
 * #
 * cdef inline double complex _hyp0f1_cmplx(double v, double complex z) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         np.npy_cdouble zz = npy_cdouble_from_double_complex(z)
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_7_hyp0f1__hyp0f1_cmplx(double __pyx_v_v, __pyx_t_double_complex __pyx_v_z) {
  npy_cdouble __pyx_v_zz;
  npy_cdouble __pyx_v_r;
  __pyx_t_double_complex __pyx_v_arg;
  __pyx_t_double_complex __pyx_v_s;
  __pyx_t_double_complex __pyx_v_t1;
  __pyx_t_double_complex __pyx_v_t2;
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  __pyx_t_double_complex __pyx_t_3;
  __pyx_t_double_complex __pyx_t_4;

  /* "_hyp0f1.pxd":103
 * cdef inline double complex _hyp0f1_cmplx(double v, double complex z) nogil:
 *     cdef:
 *         np.npy_cdouble zz = npy_cdouble_from_double_complex(z)             # <<<<<<<<<<<<<<
 *         np.npy_cdouble r
 *         double complex arg, s
 */
  __pyx_v_zz = __pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_z);

  /* "_hyp0f1.pxd":109
 * 
 *     # handle poles, zeros
 *     if v <= 0.0 and v == floor(v):             # <<<<<<<<<<<<<<
 *         return NAN
 *     if z.real == 0.0 and z.imag == 0.0 and v != 0.0:
 */
  __pyx_t_2 = ((__pyx_v_v <= 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_v == floor(__pyx_v_v)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_hyp0f1.pxd":110
 *     # handle poles, zeros
 *     if v <= 0.0 and v == floor(v):
 *         return NAN             # <<<<<<<<<<<<<<
 *     if z.real == 0.0 and z.imag == 0.0 and v != 0.0:
 *         return 1.0
 */
    __pyx_r = __pyx_t_double_complex_from_parts(NPY_NAN, 0);
    goto __pyx_L0;

    /* "_hyp0f1.pxd":109
 * 
 *     # handle poles, zeros
 *     if v <= 0.0 and v == floor(v):             # <<<<<<<<<<<<<<
 *         return NAN
 *     if z.real == 0.0 and z.imag == 0.0 and v != 0.0:
 */
  }

  /* "_hyp0f1.pxd":111
 *     if v <= 0.0 and v == floor(v):
 *         return NAN
 *     if z.real == 0.0 and z.imag == 0.0 and v != 0.0:             # <<<<<<<<<<<<<<
 *         return 1.0
 * 
 */
  __pyx_t_2 = ((__Pyx_CREAL(__pyx_v_z) == 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_2 = ((__Pyx_CIMAG(__pyx_v_z) == 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_v != 0.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L7_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_hyp0f1.pxd":112
 *         return NAN
 *     if z.real == 0.0 and z.imag == 0.0 and v != 0.0:
 *         return 1.0             # <<<<<<<<<<<<<<
 * 
 *     # both v and z small: truncate the Taylor series at O(z**2)
 */
    __pyx_r = __pyx_t_double_complex_from_parts(1.0, 0);
    goto __pyx_L0;

    /* "_hyp0f1.pxd":111
 *     if v <= 0.0 and v == floor(v):
 *         return NAN
 *     if z.real == 0.0 and z.imag == 0.0 and v != 0.0:             # <<<<<<<<<<<<<<
 *         return 1.0
 * 
 */
  }

  /* "_hyp0f1.pxd":115
 * 
 *     # both v and z small: truncate the Taylor series at O(z**2)
 *     if zabs(z) < 1e-6*(1.0 + zabs(v)):             # <<<<<<<<<<<<<<
 *         # need to do computations in this order, for otherwise $v\approx -z \ll 1$
 *         # it can lose precision (as was reported for 32-bit linux, see gh-6365)
 */
  __pyx_t_1 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_z) < (1e-6 * (1.0 + __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_v)))) != 0);
  if (__pyx_t_1) {

    /* "_hyp0f1.pxd":118
 *         # need to do computations in this order, for otherwise $v\approx -z \ll 1$
 *         # it can lose precision (as was reported for 32-bit linux, see gh-6365)
 *         t1 = 1.0 + z/v             # <<<<<<<<<<<<<<
 *         t2 = z*z / (2.0*v*(v+1.0))
 *         return t1 + t2
 */
    __pyx_t_3 = __pyx_t_double_complex_from_parts(__pyx_v_v, 0);
    if (unlikely(__Pyx_c_is_zero_double(__pyx_t_3))) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      __Pyx_PyGILState_Release(__pyx_gilstate_save);
      #endif
      __PYX_ERR(9, 118, __pyx_L1_error)
    }
    __pyx_v_t1 = __Pyx_c_sum_double(__pyx_t_double_complex_from_parts(1.0, 0), __Pyx_c_quot_double(__pyx_v_z, __pyx_t_3));

    /* "_hyp0f1.pxd":119
 *         # it can lose precision (as was reported for 32-bit linux, see gh-6365)
 *         t1 = 1.0 + z/v
 *         t2 = z*z / (2.0*v*(v+1.0))             # <<<<<<<<<<<<<<
 *         return t1 + t2
 * 
 */
    __pyx_t_3 = __Pyx_c_prod_double(__pyx_v_z, __pyx_v_z);
    __pyx_t_4 = __pyx_t_double_complex_from_parts(((2.0 * __pyx_v_v) * (__pyx_v_v + 1.0)), 0);
    if (unlikely(__Pyx_c_is_zero_double(__pyx_t_4))) {
      #ifdef WITH_THREAD
      PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      #ifdef WITH_THREAD
      __Pyx_PyGILState_Release(__pyx_gilstate_save);
      #endif
      __PYX_ERR(9, 119, __pyx_L1_error)
    }
    __pyx_v_t2 = __Pyx_c_quot_double(__pyx_t_3, __pyx_t_4);

    /* "_hyp0f1.pxd":120
 *         t1 = 1.0 + z/v
 *         t2 = z*z / (2.0*v*(v+1.0))
 *         return t1 + t2             # <<<<<<<<<<<<<<
 * 
 *     if zz.real > 0:
 */
    __pyx_r = __Pyx_c_sum_double(__pyx_v_t1, __pyx_v_t2);
    goto __pyx_L0;

    /* "_hyp0f1.pxd":115
 * 
 *     # both v and z small: truncate the Taylor series at O(z**2)
 *     if zabs(z) < 1e-6*(1.0 + zabs(v)):             # <<<<<<<<<<<<<<
 *         # need to do computations in this order, for otherwise $v\approx -z \ll 1$
 *         # it can lose precision (as was reported for 32-bit linux, see gh-6365)
 */
  }

  /* "_hyp0f1.pxd":122
 *         return t1 + t2
 * 
 *     if zz.real > 0:             # <<<<<<<<<<<<<<
 *         arg = zsqrt(z)
 *         s = 2.0 * arg
 */
  __pyx_t_1 = ((__pyx_v_zz.real > 0.0) != 0);
  if (__pyx_t_1) {

    /* "_hyp0f1.pxd":123
 * 
 *     if zz.real > 0:
 *         arg = zsqrt(z)             # <<<<<<<<<<<<<<
 *         s = 2.0 * arg
 *         r = cbesi_wrap(v-1.0, npy_cdouble_from_double_complex(s))
 */
    __pyx_v_arg = __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zsqrt(__pyx_v_z);

    /* "_hyp0f1.pxd":124
 *     if zz.real > 0:
 *         arg = zsqrt(z)
 *         s = 2.0 * arg             # <<<<<<<<<<<<<<
 *         r = cbesi_wrap(v-1.0, npy_cdouble_from_double_complex(s))
 *     else:
 */
    __pyx_v_s = __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(2.0, 0), __pyx_v_arg);

    /* "_hyp0f1.pxd":125
 *         arg = zsqrt(z)
 *         s = 2.0 * arg
 *         r = cbesi_wrap(v-1.0, npy_cdouble_from_double_complex(s))             # <<<<<<<<<<<<<<
 *     else:
 *         arg = zsqrt(-z)
 */
    __pyx_v_r = cbesi_wrap((__pyx_v_v - 1.0), __pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_s));

    /* "_hyp0f1.pxd":122
 *         return t1 + t2
 * 
 *     if zz.real > 0:             # <<<<<<<<<<<<<<
 *         arg = zsqrt(z)
 *         s = 2.0 * arg
 */
    goto __pyx_L11;
  }

  /* "_hyp0f1.pxd":127
 *         r = cbesi_wrap(v-1.0, npy_cdouble_from_double_complex(s))
 *     else:
 *         arg = zsqrt(-z)             # <<<<<<<<<<<<<<
 *         s = 2.0 * arg
 *         r = cbesj_wrap(v-1.0, npy_cdouble_from_double_complex(s))
 */
  /*else*/ {
    __pyx_v_arg = __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zsqrt(__Pyx_c_neg_double(__pyx_v_z));

    /* "_hyp0f1.pxd":128
 *     else:
 *         arg = zsqrt(-z)
 *         s = 2.0 * arg             # <<<<<<<<<<<<<<
 *         r = cbesj_wrap(v-1.0, npy_cdouble_from_double_complex(s))
 * 
 */
    __pyx_v_s = __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(2.0, 0), __pyx_v_arg);

    /* "_hyp0f1.pxd":129
 *         arg = zsqrt(-z)
 *         s = 2.0 * arg
 *         r = cbesj_wrap(v-1.0, npy_cdouble_from_double_complex(s))             # <<<<<<<<<<<<<<
 * 
 *     return double_complex_from_npy_cdouble(r) * Gamma(v) * zpow(arg, 1.0 - v)
 */
    __pyx_v_r = cbesj_wrap((__pyx_v_v - 1.0), __pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_s));
  }
  __pyx_L11:;

  /* "_hyp0f1.pxd":131
 *         r = cbesj_wrap(v-1.0, npy_cdouble_from_double_complex(s))
 * 
 *     return double_complex_from_npy_cdouble(r) * Gamma(v) * zpow(arg, 1.0 - v)             # <<<<<<<<<<<<<<
 */
  __pyx_r = __Pyx_c_prod_double(__Pyx_c_prod_double(__pyx_f_5scipy_7special_13_complexstuff_double_complex_from_npy_cdouble(__pyx_v_r), __pyx_t_double_complex_from_parts(Gamma(__pyx_v_v), 0)), __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zpow(__pyx_v_arg, (1.0 - __pyx_v_v)));
  goto __pyx_L0;

  /* "_hyp0f1.pxd":101
 * # Complex valued kernel
 * #
 * cdef inline double complex _hyp0f1_cmplx(double v, double complex z) nogil:             # <<<<<<<<<<<<<<
 *     cdef:
 *         np.npy_cdouble zz = npy_cdouble_from_double_complex(z)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("scipy.special._hyp0f1._hyp0f1_cmplx", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = __pyx_t_double_complex_from_parts(0, 0);
  __pyx_L0:;
  return __pyx_r;
}

/* "_digamma.pxd":39
 * 
 * 
 * cdef inline double digamma(double z) nogil:             # <<<<<<<<<<<<<<
 *     """Wrap Cephes' psi to take advantage of the series expansion around
 *     the smallest negative zero.
 */

static CYTHON_INLINE double __pyx_f_5scipy_7special_8_digamma_digamma(double __pyx_v_z) {
  double __pyx_r;
  int __pyx_t_1;

  /* "_digamma.pxd":44
 * 
 *     """
 *     if zabs(z - negroot) < 0.3:             # <<<<<<<<<<<<<<
 *         return zeta_series(z, negroot, negrootval)
 *     else:
 */
  __pyx_t_1 = ((__pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zabs((__pyx_v_z - -0.5040830082644554)) < 0.3) != 0);
  if (__pyx_t_1) {

    /* "_digamma.pxd":45
 *     """
 *     if zabs(z - negroot) < 0.3:
 *         return zeta_series(z, negroot, negrootval)             # <<<<<<<<<<<<<<
 *     else:
 *         return psi(z)
 */
    __pyx_r = __pyx_fuse_0__pyx_f_5scipy_7special_8_digamma_zeta_series(__pyx_v_z, -0.5040830082644554, 7.289763902976895e-17);
    goto __pyx_L0;

    /* "_digamma.pxd":44
 * 
 *     """
 *     if zabs(z - negroot) < 0.3:             # <<<<<<<<<<<<<<
 *         return zeta_series(z, negroot, negrootval)
 *     else:
 */
  }

  /* "_digamma.pxd":47
 *         return zeta_series(z, negroot, negrootval)
 *     else:
 *         return psi(z)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_r = psi(__pyx_v_z);
    goto __pyx_L0;
  }

  /* "_digamma.pxd":39
 * 
 * 
 * cdef inline double digamma(double z) nogil:             # <<<<<<<<<<<<<<
 *     """Wrap Cephes' psi to take advantage of the series expansion around
 *     the smallest negative zero.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_digamma.pxd":51
 * 
 * @cython.cdivision(True)
 * cdef inline double complex cdigamma(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Compute the digamma function for complex arguments. The strategy
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8_digamma_cdigamma(__pyx_t_double_complex __pyx_v_z) {
  int __pyx_v_n;
  double __pyx_v_absz;
  __pyx_t_double_complex __pyx_v_res;
  __pyx_t_double_complex __pyx_v_init;
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_digamma.pxd":69
 *     cdef:
 *         int n
 *         double absz = zabs(z)             # <<<<<<<<<<<<<<
 *         double complex res = 0
 *         double complex init
 */
  __pyx_v_absz = __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_z);

  /* "_digamma.pxd":70
 *         int n
 *         double absz = zabs(z)
 *         double complex res = 0             # <<<<<<<<<<<<<<
 *         double complex init
 * 
 */
  __pyx_v_res = __pyx_t_double_complex_from_parts(0, 0);

  /* "_digamma.pxd":73
 *         double complex init
 * 
 *     if z.real <= 0 and ceil(z.real) == z:             # <<<<<<<<<<<<<<
 *         # Poles
 *         sf_error.error("digamma", sf_error.SINGULAR, NULL)
 */
  __pyx_t_2 = ((__Pyx_CREAL(__pyx_v_z) <= 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__Pyx_c_eq_double(__pyx_t_double_complex_from_parts(ceil(__Pyx_CREAL(__pyx_v_z)), 0), __pyx_v_z)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_digamma.pxd":75
 *     if z.real <= 0 and ceil(z.real) == z:
 *         # Poles
 *         sf_error.error("digamma", sf_error.SINGULAR, NULL)             # <<<<<<<<<<<<<<
 *         return nan + 1j*nan
 *     elif zabs(z - negroot) < 0.3:
 */
    sf_error(((char *)"digamma"), SF_ERROR_SINGULAR, NULL);

    /* "_digamma.pxd":76
 *         # Poles
 *         sf_error.error("digamma", sf_error.SINGULAR, NULL)
 *         return nan + 1j*nan             # <<<<<<<<<<<<<<
 *     elif zabs(z - negroot) < 0.3:
 *         # First negative root
 */
    __pyx_r = __Pyx_c_sum_double(__pyx_t_double_complex_from_parts(NPY_NAN, 0), __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0, 1.0), __pyx_t_double_complex_from_parts(NPY_NAN, 0)));
    goto __pyx_L0;

    /* "_digamma.pxd":73
 *         double complex init
 * 
 *     if z.real <= 0 and ceil(z.real) == z:             # <<<<<<<<<<<<<<
 *         # Poles
 *         sf_error.error("digamma", sf_error.SINGULAR, NULL)
 */
  }

  /* "_digamma.pxd":77
 *         sf_error.error("digamma", sf_error.SINGULAR, NULL)
 *         return nan + 1j*nan
 *     elif zabs(z - negroot) < 0.3:             # <<<<<<<<<<<<<<
 *         # First negative root
 *         return zeta_series(z, negroot, negrootval)
 */
  __pyx_t_1 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__Pyx_c_diff_double(__pyx_v_z, __pyx_t_double_complex_from_parts(-0.5040830082644554, 0))) < 0.3) != 0);
  if (__pyx_t_1) {

    /* "_digamma.pxd":79
 *     elif zabs(z - negroot) < 0.3:
 *         # First negative root
 *         return zeta_series(z, negroot, negrootval)             # <<<<<<<<<<<<<<
 * 
 *     if z.real < 0 and fabs(z.imag) < smallabsz:
 */
    __pyx_r = __pyx_fuse_1__pyx_f_5scipy_7special_8_digamma_zeta_series(__pyx_v_z, -0.5040830082644554, 7.289763902976895e-17);
    goto __pyx_L0;

    /* "_digamma.pxd":77
 *         sf_error.error("digamma", sf_error.SINGULAR, NULL)
 *         return nan + 1j*nan
 *     elif zabs(z - negroot) < 0.3:             # <<<<<<<<<<<<<<
 *         # First negative root
 *         return zeta_series(z, negroot, negrootval)
 */
  }

  /* "_digamma.pxd":81
 *         return zeta_series(z, negroot, negrootval)
 * 
 *     if z.real < 0 and fabs(z.imag) < smallabsz:             # <<<<<<<<<<<<<<
 *         # Reflection formula for digamma. See
 *         #
 */
  __pyx_t_2 = ((__Pyx_CREAL(__pyx_v_z) < 0.0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_2 = ((fabs(__Pyx_CIMAG(__pyx_v_z)) < 16.0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L7_bool_binop_done:;
  if (__pyx_t_1) {

    /* "_digamma.pxd":86
 *         # https://dlmf.nist.gov/5.5#E4
 *         #
 *         res -= M_PI*cospi(z)/sinpi(z)             # <<<<<<<<<<<<<<
 *         z = 1 - z
 *         absz = zabs(z)
 */
    __pyx_v_res = __Pyx_c_diff_double(__pyx_v_res, __Pyx_c_quot_double(__Pyx_c_prod_double(__pyx_t_double_complex_from_parts(M_PI, 0), __pyx_fuse_1__pyx_f_5scipy_7special_5_trig_cospi(__pyx_v_z)), __pyx_fuse_1__pyx_f_5scipy_7special_5_trig_sinpi(__pyx_v_z)));

    /* "_digamma.pxd":87
 *         #
 *         res -= M_PI*cospi(z)/sinpi(z)
 *         z = 1 - z             # <<<<<<<<<<<<<<
 *         absz = zabs(z)
 * 
 */
    __pyx_v_z = __Pyx_c_diff_double(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_z);

    /* "_digamma.pxd":88
 *         res -= M_PI*cospi(z)/sinpi(z)
 *         z = 1 - z
 *         absz = zabs(z)             # <<<<<<<<<<<<<<
 * 
 *     if absz < 0.5:
 */
    __pyx_v_absz = __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_z);

    /* "_digamma.pxd":81
 *         return zeta_series(z, negroot, negrootval)
 * 
 *     if z.real < 0 and fabs(z.imag) < smallabsz:             # <<<<<<<<<<<<<<
 *         # Reflection formula for digamma. See
 *         #
 */
  }

  /* "_digamma.pxd":90
 *         absz = zabs(z)
 * 
 *     if absz < 0.5:             # <<<<<<<<<<<<<<
 *         # Use one step of the recurrence relation to step away from
 *         # the pole.
 */
  __pyx_t_1 = ((__pyx_v_absz < 0.5) != 0);
  if (__pyx_t_1) {

    /* "_digamma.pxd":93
 *         # Use one step of the recurrence relation to step away from
 *         # the pole.
 *         res -= 1/z             # <<<<<<<<<<<<<<
 *         z += 1
 *         absz = zabs(z)
 */
    __pyx_v_res = __Pyx_c_diff_double(__pyx_v_res, __Pyx_c_quot_double(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_z));

    /* "_digamma.pxd":94
 *         # the pole.
 *         res -= 1/z
 *         z += 1             # <<<<<<<<<<<<<<
 *         absz = zabs(z)
 * 
 */
    __pyx_v_z = __Pyx_c_sum_double(__pyx_v_z, __pyx_t_double_complex_from_parts(1, 0));

    /* "_digamma.pxd":95
 *         res -= 1/z
 *         z += 1
 *         absz = zabs(z)             # <<<<<<<<<<<<<<
 * 
 *     if zabs(z - posroot) < 0.5:
 */
    __pyx_v_absz = __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_z);

    /* "_digamma.pxd":90
 *         absz = zabs(z)
 * 
 *     if absz < 0.5:             # <<<<<<<<<<<<<<
 *         # Use one step of the recurrence relation to step away from
 *         # the pole.
 */
  }

  /* "_digamma.pxd":97
 *         absz = zabs(z)
 * 
 *     if zabs(z - posroot) < 0.5:             # <<<<<<<<<<<<<<
 *         res += zeta_series(z, posroot, posrootval)
 *     elif absz > smallabsz:
 */
  __pyx_t_1 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__Pyx_c_diff_double(__pyx_v_z, __pyx_t_double_complex_from_parts(1.4616321449683622, 0))) < 0.5) != 0);
  if (__pyx_t_1) {

    /* "_digamma.pxd":98
 * 
 *     if zabs(z - posroot) < 0.5:
 *         res += zeta_series(z, posroot, posrootval)             # <<<<<<<<<<<<<<
 *     elif absz > smallabsz:
 *         res += asymptotic_series(z)
 */
    __pyx_v_res = __Pyx_c_sum_double(__pyx_v_res, __pyx_fuse_1__pyx_f_5scipy_7special_8_digamma_zeta_series(__pyx_v_z, 1.4616321449683622, -9.241265521729427e-17));

    /* "_digamma.pxd":97
 *         absz = zabs(z)
 * 
 *     if zabs(z - posroot) < 0.5:             # <<<<<<<<<<<<<<
 *         res += zeta_series(z, posroot, posrootval)
 *     elif absz > smallabsz:
 */
    goto __pyx_L10;
  }

  /* "_digamma.pxd":99
 *     if zabs(z - posroot) < 0.5:
 *         res += zeta_series(z, posroot, posrootval)
 *     elif absz > smallabsz:             # <<<<<<<<<<<<<<
 *         res += asymptotic_series(z)
 *     elif z.real >= 0:
 */
  __pyx_t_1 = ((__pyx_v_absz > 16.0) != 0);
  if (__pyx_t_1) {

    /* "_digamma.pxd":100
 *         res += zeta_series(z, posroot, posrootval)
 *     elif absz > smallabsz:
 *         res += asymptotic_series(z)             # <<<<<<<<<<<<<<
 *     elif z.real >= 0:
 *         n = <int>(smallabsz - absz) + 1
 */
    __pyx_v_res = __Pyx_c_sum_double(__pyx_v_res, __pyx_f_5scipy_7special_8_digamma_asymptotic_series(__pyx_v_z));

    /* "_digamma.pxd":99
 *     if zabs(z - posroot) < 0.5:
 *         res += zeta_series(z, posroot, posrootval)
 *     elif absz > smallabsz:             # <<<<<<<<<<<<<<
 *         res += asymptotic_series(z)
 *     elif z.real >= 0:
 */
    goto __pyx_L10;
  }

  /* "_digamma.pxd":101
 *     elif absz > smallabsz:
 *         res += asymptotic_series(z)
 *     elif z.real >= 0:             # <<<<<<<<<<<<<<
 *         n = <int>(smallabsz - absz) + 1
 *         init = asymptotic_series(z + n)
 */
  __pyx_t_1 = ((__Pyx_CREAL(__pyx_v_z) >= 0.0) != 0);
  if (__pyx_t_1) {

    /* "_digamma.pxd":102
 *         res += asymptotic_series(z)
 *     elif z.real >= 0:
 *         n = <int>(smallabsz - absz) + 1             # <<<<<<<<<<<<<<
 *         init = asymptotic_series(z + n)
 *         res += backward_recurrence(z + n, init, n)
 */
    __pyx_v_n = (((int)(16.0 - __pyx_v_absz)) + 1);

    /* "_digamma.pxd":103
 *     elif z.real >= 0:
 *         n = <int>(smallabsz - absz) + 1
 *         init = asymptotic_series(z + n)             # <<<<<<<<<<<<<<
 *         res += backward_recurrence(z + n, init, n)
 *     else:
 */
    __pyx_v_init = __pyx_f_5scipy_7special_8_digamma_asymptotic_series(__Pyx_c_sum_double(__pyx_v_z, __pyx_t_double_complex_from_parts(__pyx_v_n, 0)));

    /* "_digamma.pxd":104
 *         n = <int>(smallabsz - absz) + 1
 *         init = asymptotic_series(z + n)
 *         res += backward_recurrence(z + n, init, n)             # <<<<<<<<<<<<<<
 *     else:
 *         # z.real < 0, absz < smallabsz, and z.imag > smallimag
 */
    __pyx_v_res = __Pyx_c_sum_double(__pyx_v_res, __pyx_f_5scipy_7special_8_digamma_backward_recurrence(__Pyx_c_sum_double(__pyx_v_z, __pyx_t_double_complex_from_parts(__pyx_v_n, 0)), __pyx_v_init, __pyx_v_n));

    /* "_digamma.pxd":101
 *     elif absz > smallabsz:
 *         res += asymptotic_series(z)
 *     elif z.real >= 0:             # <<<<<<<<<<<<<<
 *         n = <int>(smallabsz - absz) + 1
 *         init = asymptotic_series(z + n)
 */
    goto __pyx_L10;
  }

  /* "_digamma.pxd":107
 *     else:
 *         # z.real < 0, absz < smallabsz, and z.imag > smallimag
 *         n = <int>(smallabsz - absz) - 1             # <<<<<<<<<<<<<<
 *         init = asymptotic_series(z - n)
 *         res += forward_recurrence(z - n, init, n)
 */
  /*else*/ {
    __pyx_v_n = (((int)(16.0 - __pyx_v_absz)) - 1);

    /* "_digamma.pxd":108
 *         # z.real < 0, absz < smallabsz, and z.imag > smallimag
 *         n = <int>(smallabsz - absz) - 1
 *         init = asymptotic_series(z - n)             # <<<<<<<<<<<<<<
 *         res += forward_recurrence(z - n, init, n)
 *     return res
 */
    __pyx_v_init = __pyx_f_5scipy_7special_8_digamma_asymptotic_series(__Pyx_c_diff_double(__pyx_v_z, __pyx_t_double_complex_from_parts(__pyx_v_n, 0)));

    /* "_digamma.pxd":109
 *         n = <int>(smallabsz - absz) - 1
 *         init = asymptotic_series(z - n)
 *         res += forward_recurrence(z - n, init, n)             # <<<<<<<<<<<<<<
 *     return res
 * 
 */
    __pyx_v_res = __Pyx_c_sum_double(__pyx_v_res, __pyx_f_5scipy_7special_8_digamma_forward_recurrence(__Pyx_c_diff_double(__pyx_v_z, __pyx_t_double_complex_from_parts(__pyx_v_n, 0)), __pyx_v_init, __pyx_v_n));
  }
  __pyx_L10:;

  /* "_digamma.pxd":110
 *         init = asymptotic_series(z - n)
 *         res += forward_recurrence(z - n, init, n)
 *     return res             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_res;
  goto __pyx_L0;

  /* "_digamma.pxd":51
 * 
 * @cython.cdivision(True)
 * cdef inline double complex cdigamma(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Compute the digamma function for complex arguments. The strategy
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_digamma.pxd":114
 * 
 * @cython.cdivision(True)
 * cdef inline double complex forward_recurrence(double complex z,             # <<<<<<<<<<<<<<
 *                                               double complex psiz,
 *                                                int n) nogil:
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8_digamma_forward_recurrence(__pyx_t_double_complex __pyx_v_z, __pyx_t_double_complex __pyx_v_psiz, int __pyx_v_n) {
  int __pyx_v_k;
  __pyx_t_double_complex __pyx_v_res;
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "_digamma.pxd":128
 *     cdef:
 *         int k
 *         double complex res = psiz             # <<<<<<<<<<<<<<
 * 
 *     for k in range(n):
 */
  __pyx_v_res = __pyx_v_psiz;

  /* "_digamma.pxd":130
 *         double complex res = psiz
 * 
 *     for k in range(n):             # <<<<<<<<<<<<<<
 *         res += 1/(z + k)
 *     return res
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_k = __pyx_t_3;

    /* "_digamma.pxd":131
 * 
 *     for k in range(n):
 *         res += 1/(z + k)             # <<<<<<<<<<<<<<
 *     return res
 * 
 */
    __pyx_v_res = __Pyx_c_sum_double(__pyx_v_res, __Pyx_c_quot_double(__pyx_t_double_complex_from_parts(1, 0), __Pyx_c_sum_double(__pyx_v_z, __pyx_t_double_complex_from_parts(__pyx_v_k, 0))));
  }

  /* "_digamma.pxd":132
 *     for k in range(n):
 *         res += 1/(z + k)
 *     return res             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_res;
  goto __pyx_L0;

  /* "_digamma.pxd":114
 * 
 * @cython.cdivision(True)
 * cdef inline double complex forward_recurrence(double complex z,             # <<<<<<<<<<<<<<
 *                                               double complex psiz,
 *                                                int n) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_digamma.pxd":136
 * 
 * @cython.cdivision(True)
 * cdef inline double complex backward_recurrence(double complex z,             # <<<<<<<<<<<<<<
 *                                                double complex psiz,
 *                                                int n) nogil:
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8_digamma_backward_recurrence(__pyx_t_double_complex __pyx_v_z, __pyx_t_double_complex __pyx_v_psiz, int __pyx_v_n) {
  int __pyx_v_k;
  __pyx_t_double_complex __pyx_v_res;
  __pyx_t_double_complex __pyx_r;
  long __pyx_t_1;
  long __pyx_t_2;
  int __pyx_t_3;

  /* "_digamma.pxd":146
 *     cdef:
 *         int k
 *         double complex res = psiz             # <<<<<<<<<<<<<<
 * 
 *     for k in range(1, n + 1):
 */
  __pyx_v_res = __pyx_v_psiz;

  /* "_digamma.pxd":148
 *         double complex res = psiz
 * 
 *     for k in range(1, n + 1):             # <<<<<<<<<<<<<<
 *         res -= 1/(z - k)
 *     return res
 */
  __pyx_t_1 = (__pyx_v_n + 1);
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 1; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_k = __pyx_t_3;

    /* "_digamma.pxd":149
 * 
 *     for k in range(1, n + 1):
 *         res -= 1/(z - k)             # <<<<<<<<<<<<<<
 *     return res
 * 
 */
    __pyx_v_res = __Pyx_c_diff_double(__pyx_v_res, __Pyx_c_quot_double(__pyx_t_double_complex_from_parts(1, 0), __Pyx_c_diff_double(__pyx_v_z, __pyx_t_double_complex_from_parts(__pyx_v_k, 0))));
  }

  /* "_digamma.pxd":150
 *     for k in range(1, n + 1):
 *         res -= 1/(z - k)
 *     return res             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_res;
  goto __pyx_L0;

  /* "_digamma.pxd":136
 * 
 * @cython.cdivision(True)
 * cdef inline double complex backward_recurrence(double complex z,             # <<<<<<<<<<<<<<
 *                                                double complex psiz,
 *                                                int n) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_digamma.pxd":154
 * 
 * @cython.cdivision(True)
 * cdef inline double complex asymptotic_series(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Evaluate digamma using an asymptotic series. See
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_8_digamma_asymptotic_series(__pyx_t_double_complex __pyx_v_z) {
  int __pyx_v_k;
  double *__pyx_v_bernoulli2k;
  __pyx_t_double_complex __pyx_v_rzz;
  __pyx_t_double_complex __pyx_v_zfac;
  __pyx_t_double_complex __pyx_v_term;
  __pyx_t_double_complex __pyx_v_res;
  __pyx_t_double_complex __pyx_r;
  double __pyx_t_1[16];
  int __pyx_t_2;
  int __pyx_t_3;

  /* "_digamma.pxd":162
 *     """
 *     cdef:
 *         int k = 1             # <<<<<<<<<<<<<<
 *         # The Bernoulli numbers B_2k for 1 <= k <= 16.
 *         double *bernoulli2k = [
 */
  __pyx_v_k = 1;

  /* "_digamma.pxd":164
 *         int k = 1
 *         # The Bernoulli numbers B_2k for 1 <= k <= 16.
 *         double *bernoulli2k = [             # <<<<<<<<<<<<<<
 *             0.166666666666666667, -0.0333333333333333333,
 *             0.0238095238095238095, -0.0333333333333333333,
 */
  __pyx_t_1[0] = 0.166666666666666667;
  __pyx_t_1[1] = -0.0333333333333333333;
  __pyx_t_1[2] = 0.0238095238095238095;
  __pyx_t_1[3] = -0.0333333333333333333;
  __pyx_t_1[4] = 0.0757575757575757576;
  __pyx_t_1[5] = -0.253113553113553114;
  __pyx_t_1[6] = 1.16666666666666667;
  __pyx_t_1[7] = -7.09215686274509804;
  __pyx_t_1[8] = 54.9711779448621554;
  __pyx_t_1[9] = -529.124242424242424;
  __pyx_t_1[10] = 6192.12318840579710;
  __pyx_t_1[11] = -86580.2531135531136;
  __pyx_t_1[12] = 1425517.16666666667;
  __pyx_t_1[13] = -27298231.0678160920;
  __pyx_t_1[14] = 601580873.900642368;
  __pyx_t_1[15] = -15116315767.0921569;
  __pyx_v_bernoulli2k = __pyx_t_1;

  /* "_digamma.pxd":173
 *             1425517.16666666667, -27298231.0678160920,
 *             601580873.900642368, -15116315767.0921569]
 *         double complex rzz = 1/z/z             # <<<<<<<<<<<<<<
 *         double complex zfac = 1
 *         double complex term
 */
  __pyx_v_rzz = __Pyx_c_quot_double(__Pyx_c_quot_double(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_z), __pyx_v_z);

  /* "_digamma.pxd":174
 *             601580873.900642368, -15116315767.0921569]
 *         double complex rzz = 1/z/z
 *         double complex zfac = 1             # <<<<<<<<<<<<<<
 *         double complex term
 *         double complex res
 */
  __pyx_v_zfac = __pyx_t_double_complex_from_parts(1, 0);

  /* "_digamma.pxd":178
 *         double complex res
 * 
 *     res = zlog(z) - 0.5/z             # <<<<<<<<<<<<<<
 *     for k in range(1, 17):
 *         zfac *= rzz
 */
  __pyx_v_res = __Pyx_c_diff_double(__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zlog(__pyx_v_z), __Pyx_c_quot_double(__pyx_t_double_complex_from_parts(0.5, 0), __pyx_v_z));

  /* "_digamma.pxd":179
 * 
 *     res = zlog(z) - 0.5/z
 *     for k in range(1, 17):             # <<<<<<<<<<<<<<
 *         zfac *= rzz
 *         term = -bernoulli2k[k-1]*zfac/(2*k)
 */
  for (__pyx_t_2 = 1; __pyx_t_2 < 17; __pyx_t_2+=1) {
    __pyx_v_k = __pyx_t_2;

    /* "_digamma.pxd":180
 *     res = zlog(z) - 0.5/z
 *     for k in range(1, 17):
 *         zfac *= rzz             # <<<<<<<<<<<<<<
 *         term = -bernoulli2k[k-1]*zfac/(2*k)
 *         res += term
 */
    __pyx_v_zfac = __Pyx_c_prod_double(__pyx_v_zfac, __pyx_v_rzz);

    /* "_digamma.pxd":181
 *     for k in range(1, 17):
 *         zfac *= rzz
 *         term = -bernoulli2k[k-1]*zfac/(2*k)             # <<<<<<<<<<<<<<
 *         res += term
 *         if zabs(term) < tol*zabs(res):
 */
    __pyx_v_term = __Pyx_c_quot_double(__Pyx_c_prod_double(__pyx_t_double_complex_from_parts((-(__pyx_v_bernoulli2k[(__pyx_v_k - 1)])), 0), __pyx_v_zfac), __pyx_t_double_complex_from_parts((2 * __pyx_v_k), 0));

    /* "_digamma.pxd":182
 *         zfac *= rzz
 *         term = -bernoulli2k[k-1]*zfac/(2*k)
 *         res += term             # <<<<<<<<<<<<<<
 *         if zabs(term) < tol*zabs(res):
 *             break
 */
    __pyx_v_res = __Pyx_c_sum_double(__pyx_v_res, __pyx_v_term);

    /* "_digamma.pxd":183
 *         term = -bernoulli2k[k-1]*zfac/(2*k)
 *         res += term
 *         if zabs(term) < tol*zabs(res):             # <<<<<<<<<<<<<<
 *             break
 *     return res
 */
    __pyx_t_3 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_term) < (2.220446092504131e-16 * __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_res))) != 0);
    if (__pyx_t_3) {

      /* "_digamma.pxd":184
 *         res += term
 *         if zabs(term) < tol*zabs(res):
 *             break             # <<<<<<<<<<<<<<
 *     return res
 * 
 */
      goto __pyx_L4_break;

      /* "_digamma.pxd":183
 *         term = -bernoulli2k[k-1]*zfac/(2*k)
 *         res += term
 *         if zabs(term) < tol*zabs(res):             # <<<<<<<<<<<<<<
 *             break
 *     return res
 */
    }
  }
  __pyx_L4_break:;

  /* "_digamma.pxd":185
 *         if zabs(term) < tol*zabs(res):
 *             break
 *     return res             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_res;
  goto __pyx_L0;

  /* "_digamma.pxd":154
 * 
 * @cython.cdivision(True)
 * cdef inline double complex asymptotic_series(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Evaluate digamma using an asymptotic series. See
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_digamma.pxd":188
 * 
 * 
 * cdef inline number_t zeta_series(number_t z, double root, double rootval) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     The coefficients of the Taylor series for digamma at any point can
 */

static CYTHON_INLINE double __pyx_fuse_0__pyx_f_5scipy_7special_8_digamma_zeta_series(double __pyx_v_z, double __pyx_v_root, double __pyx_v_rootval) {
  int __pyx_v_n;
  double __pyx_v_res;
  double __pyx_v_coeff;
  double __pyx_v_term;
  double __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_digamma.pxd":200
 *     cdef:
 *         int n
 *         number_t res = rootval             # <<<<<<<<<<<<<<
 *         number_t coeff = -1
 *         number_t term
 */
  __pyx_v_res = __pyx_v_rootval;

  /* "_digamma.pxd":201
 *         int n
 *         number_t res = rootval
 *         number_t coeff = -1             # <<<<<<<<<<<<<<
 *         number_t term
 * 
 */
  __pyx_v_coeff = -1.0;

  /* "_digamma.pxd":204
 *         number_t term
 * 
 *     z = z - root             # <<<<<<<<<<<<<<
 *     for n in range(1, 100):
 *         coeff *= -z
 */
  __pyx_v_z = (__pyx_v_z - __pyx_v_root);

  /* "_digamma.pxd":205
 * 
 *     z = z - root
 *     for n in range(1, 100):             # <<<<<<<<<<<<<<
 *         coeff *= -z
 *         term = coeff*zeta(n + 1, root)
 */
  for (__pyx_t_1 = 1; __pyx_t_1 < 0x64; __pyx_t_1+=1) {
    __pyx_v_n = __pyx_t_1;

    /* "_digamma.pxd":206
 *     z = z - root
 *     for n in range(1, 100):
 *         coeff *= -z             # <<<<<<<<<<<<<<
 *         term = coeff*zeta(n + 1, root)
 *         res += term
 */
    __pyx_v_coeff = (__pyx_v_coeff * (-__pyx_v_z));

    /* "_digamma.pxd":207
 *     for n in range(1, 100):
 *         coeff *= -z
 *         term = coeff*zeta(n + 1, root)             # <<<<<<<<<<<<<<
 *         res += term
 *         if zabs(term) < tol*zabs(res):
 */
    __pyx_v_term = (__pyx_v_coeff * zeta((__pyx_v_n + 1), __pyx_v_root));

    /* "_digamma.pxd":208
 *         coeff *= -z
 *         term = coeff*zeta(n + 1, root)
 *         res += term             # <<<<<<<<<<<<<<
 *         if zabs(term) < tol*zabs(res):
 *             break
 */
    __pyx_v_res = (__pyx_v_res + __pyx_v_term);

    /* "_digamma.pxd":209
 *         term = coeff*zeta(n + 1, root)
 *         res += term
 *         if zabs(term) < tol*zabs(res):             # <<<<<<<<<<<<<<
 *             break
 *     return res
 */
    __pyx_t_2 = ((__pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_term) < (2.220446092504131e-16 * __pyx_fuse_0__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_res))) != 0);
    if (__pyx_t_2) {

      /* "_digamma.pxd":210
 *         res += term
 *         if zabs(term) < tol*zabs(res):
 *             break             # <<<<<<<<<<<<<<
 *     return res
 */
      goto __pyx_L4_break;

      /* "_digamma.pxd":209
 *         term = coeff*zeta(n + 1, root)
 *         res += term
 *         if zabs(term) < tol*zabs(res):             # <<<<<<<<<<<<<<
 *             break
 *     return res
 */
    }
  }
  __pyx_L4_break:;

  /* "_digamma.pxd":211
 *         if zabs(term) < tol*zabs(res):
 *             break
 *     return res             # <<<<<<<<<<<<<<
 */
  __pyx_r = __pyx_v_res;
  goto __pyx_L0;

  /* "_digamma.pxd":188
 * 
 * 
 * cdef inline number_t zeta_series(number_t z, double root, double rootval) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     The coefficients of the Taylor series for digamma at any point can
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE __pyx_t_double_complex __pyx_fuse_1__pyx_f_5scipy_7special_8_digamma_zeta_series(__pyx_t_double_complex __pyx_v_z, double __pyx_v_root, double __pyx_v_rootval) {
  int __pyx_v_n;
  __pyx_t_double_complex __pyx_v_res;
  __pyx_t_double_complex __pyx_v_coeff;
  __pyx_t_double_complex __pyx_v_term;
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_digamma.pxd":200
 *     cdef:
 *         int n
 *         number_t res = rootval             # <<<<<<<<<<<<<<
 *         number_t coeff = -1
 *         number_t term
 */
  __pyx_v_res = __pyx_t_double_complex_from_parts(__pyx_v_rootval, 0);

  /* "_digamma.pxd":201
 *         int n
 *         number_t res = rootval
 *         number_t coeff = -1             # <<<<<<<<<<<<<<
 *         number_t term
 * 
 */
  __pyx_v_coeff = __pyx_t_double_complex_from_parts(-1L, 0);

  /* "_digamma.pxd":204
 *         number_t term
 * 
 *     z = z - root             # <<<<<<<<<<<<<<
 *     for n in range(1, 100):
 *         coeff *= -z
 */
  __pyx_v_z = __Pyx_c_diff_double(__pyx_v_z, __pyx_t_double_complex_from_parts(__pyx_v_root, 0));

  /* "_digamma.pxd":205
 * 
 *     z = z - root
 *     for n in range(1, 100):             # <<<<<<<<<<<<<<
 *         coeff *= -z
 *         term = coeff*zeta(n + 1, root)
 */
  for (__pyx_t_1 = 1; __pyx_t_1 < 0x64; __pyx_t_1+=1) {
    __pyx_v_n = __pyx_t_1;

    /* "_digamma.pxd":206
 *     z = z - root
 *     for n in range(1, 100):
 *         coeff *= -z             # <<<<<<<<<<<<<<
 *         term = coeff*zeta(n + 1, root)
 *         res += term
 */
    __pyx_v_coeff = __Pyx_c_prod_double(__pyx_v_coeff, __Pyx_c_neg_double(__pyx_v_z));

    /* "_digamma.pxd":207
 *     for n in range(1, 100):
 *         coeff *= -z
 *         term = coeff*zeta(n + 1, root)             # <<<<<<<<<<<<<<
 *         res += term
 *         if zabs(term) < tol*zabs(res):
 */
    __pyx_v_term = __Pyx_c_prod_double(__pyx_v_coeff, __pyx_t_double_complex_from_parts(zeta((__pyx_v_n + 1), __pyx_v_root), 0));

    /* "_digamma.pxd":208
 *         coeff *= -z
 *         term = coeff*zeta(n + 1, root)
 *         res += term             # <<<<<<<<<<<<<<
 *         if zabs(term) < tol*zabs(res):
 *             break
 */
    __pyx_v_res = __Pyx_c_sum_double(__pyx_v_res, __pyx_v_term);

    /* "_digamma.pxd":209
 *         term = coeff*zeta(n + 1, root)
 *         res += term
 *         if zabs(term) < tol*zabs(res):             # <<<<<<<<<<<<<<
 *             break
 *     return res
 */
    __pyx_t_2 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_term) < (2.220446092504131e-16 * __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_res))) != 0);
    if (__pyx_t_2) {

      /* "_digamma.pxd":210
 *         res += term
 *         if zabs(term) < tol*zabs(res):
 *             break             # <<<<<<<<<<<<<<
 *     return res
 */
      goto __pyx_L4_break;

      /* "_digamma.pxd":209
 *         term = coeff*zeta(n + 1, root)
 *         res += term
 *         if zabs(term) < tol*zabs(res):             # <<<<<<<<<<<<<<
 *             break
 *     return res
 */
    }
  }
  __pyx_L4_break:;

  /* "_digamma.pxd":211
 *         if zabs(term) < tol*zabs(res):
 *             break
 *     return res             # <<<<<<<<<<<<<<
 */
  __pyx_r = __pyx_v_res;
  goto __pyx_L0;

  /* "_digamma.pxd":188
 * 
 * 
 * cdef inline number_t zeta_series(number_t z, double root, double rootval) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     The coefficients of the Taylor series for digamma at any point can
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_sici.pxd":30
 * 
 * 
 * cdef inline double complex zexpi(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     r = cexpi_wrap(npy_cdouble_from_double_complex(z))
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_5_sici_zexpi(__pyx_t_double_complex __pyx_v_z) {
  npy_cdouble __pyx_v_r;
  __pyx_t_double_complex __pyx_r;

  /* "_sici.pxd":32
 * cdef inline double complex zexpi(double complex z) nogil:
 *     cdef np.npy_cdouble r
 *     r = cexpi_wrap(npy_cdouble_from_double_complex(z))             # <<<<<<<<<<<<<<
 *     return double_complex_from_npy_cdouble(r)
 * 
 */
  __pyx_v_r = cexpi_wrap(__pyx_f_5scipy_7special_13_complexstuff_npy_cdouble_from_double_complex(__pyx_v_z));

  /* "_sici.pxd":33
 *     cdef np.npy_cdouble r
 *     r = cexpi_wrap(npy_cdouble_from_double_complex(z))
 *     return double_complex_from_npy_cdouble(r)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_f_5scipy_7special_13_complexstuff_double_complex_from_npy_cdouble(__pyx_v_r);
  goto __pyx_L0;

  /* "_sici.pxd":30
 * 
 * 
 * cdef inline double complex zexpi(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     cdef np.npy_cdouble r
 *     r = cexpi_wrap(npy_cdouble_from_double_complex(z))
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_sici.pxd":37
 * 
 * @cython.cdivision(True)
 * cdef inline void power_series(int sgn, double complex z,             # <<<<<<<<<<<<<<
 *                              double complex *s, double complex *c) nogil:
 *     """DLMF 6.6.5 and 6.6.6. If sgn = -1 computes si/ci, and if sgn = 1
 */

static CYTHON_INLINE void __pyx_f_5scipy_7special_5_sici_power_series(int __pyx_v_sgn, __pyx_t_double_complex __pyx_v_z, __pyx_t_double_complex *__pyx_v_s, __pyx_t_double_complex *__pyx_v_c) {
  int __pyx_v_n;
  __pyx_t_double_complex __pyx_v_fac;
  __pyx_t_double_complex __pyx_v_term1;
  __pyx_t_double_complex __pyx_v_term2;
  int __pyx_t_1;
  long __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "_sici.pxd":47
 *         double complex fac, term1, term2
 * 
 *     fac = z             # <<<<<<<<<<<<<<
 *     s[0] = fac
 *     c[0] = 0
 */
  __pyx_v_fac = __pyx_v_z;

  /* "_sici.pxd":48
 * 
 *     fac = z
 *     s[0] = fac             # <<<<<<<<<<<<<<
 *     c[0] = 0
 *     for n in range(1, MAXITER):
 */
  (__pyx_v_s[0]) = __pyx_v_fac;

  /* "_sici.pxd":49
 *     fac = z
 *     s[0] = fac
 *     c[0] = 0             # <<<<<<<<<<<<<<
 *     for n in range(1, MAXITER):
 *         fac *= sgn*z/(2*n)
 */
  (__pyx_v_c[0]) = __pyx_t_double_complex_from_parts(0, 0);

  /* "_sici.pxd":50
 *     s[0] = fac
 *     c[0] = 0
 *     for n in range(1, MAXITER):             # <<<<<<<<<<<<<<
 *         fac *= sgn*z/(2*n)
 *         term2 = fac/(2*n)
 */
  for (__pyx_t_1 = 1; __pyx_t_1 < 0x64; __pyx_t_1+=1) {
    __pyx_v_n = __pyx_t_1;

    /* "_sici.pxd":51
 *     c[0] = 0
 *     for n in range(1, MAXITER):
 *         fac *= sgn*z/(2*n)             # <<<<<<<<<<<<<<
 *         term2 = fac/(2*n)
 *         c[0] += term2
 */
    __pyx_v_fac = __Pyx_c_prod_double(__pyx_v_fac, __Pyx_c_quot_double(__Pyx_c_prod_double(__pyx_t_double_complex_from_parts(__pyx_v_sgn, 0), __pyx_v_z), __pyx_t_double_complex_from_parts((2 * __pyx_v_n), 0)));

    /* "_sici.pxd":52
 *     for n in range(1, MAXITER):
 *         fac *= sgn*z/(2*n)
 *         term2 = fac/(2*n)             # <<<<<<<<<<<<<<
 *         c[0] += term2
 *         fac *= z/(2*n + 1)
 */
    __pyx_v_term2 = __Pyx_c_quot_double(__pyx_v_fac, __pyx_t_double_complex_from_parts((2 * __pyx_v_n), 0));

    /* "_sici.pxd":53
 *         fac *= sgn*z/(2*n)
 *         term2 = fac/(2*n)
 *         c[0] += term2             # <<<<<<<<<<<<<<
 *         fac *= z/(2*n + 1)
 *         term1 = fac/(2*n + 1)
 */
    __pyx_t_2 = 0;
    (__pyx_v_c[__pyx_t_2]) = __Pyx_c_sum_double((__pyx_v_c[__pyx_t_2]), __pyx_v_term2);

    /* "_sici.pxd":54
 *         term2 = fac/(2*n)
 *         c[0] += term2
 *         fac *= z/(2*n + 1)             # <<<<<<<<<<<<<<
 *         term1 = fac/(2*n + 1)
 *         s[0] += term1
 */
    __pyx_v_fac = __Pyx_c_prod_double(__pyx_v_fac, __Pyx_c_quot_double(__pyx_v_z, __pyx_t_double_complex_from_parts(((2 * __pyx_v_n) + 1), 0)));

    /* "_sici.pxd":55
 *         c[0] += term2
 *         fac *= z/(2*n + 1)
 *         term1 = fac/(2*n + 1)             # <<<<<<<<<<<<<<
 *         s[0] += term1
 *         if zabs(term1) < TOL*zabs(s[0]) and zabs(term2) < TOL*zabs(c[0]):
 */
    __pyx_v_term1 = __Pyx_c_quot_double(__pyx_v_fac, __pyx_t_double_complex_from_parts(((2 * __pyx_v_n) + 1), 0));

    /* "_sici.pxd":56
 *         fac *= z/(2*n + 1)
 *         term1 = fac/(2*n + 1)
 *         s[0] += term1             # <<<<<<<<<<<<<<
 *         if zabs(term1) < TOL*zabs(s[0]) and zabs(term2) < TOL*zabs(c[0]):
 *             break
 */
    __pyx_t_2 = 0;
    (__pyx_v_s[__pyx_t_2]) = __Pyx_c_sum_double((__pyx_v_s[__pyx_t_2]), __pyx_v_term1);

    /* "_sici.pxd":57
 *         term1 = fac/(2*n + 1)
 *         s[0] += term1
 *         if zabs(term1) < TOL*zabs(s[0]) and zabs(term2) < TOL*zabs(c[0]):             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    __pyx_t_4 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_term1) < (2.220446092504131e-16 * __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs((__pyx_v_s[0])))) != 0);
    if (__pyx_t_4) {
    } else {
      __pyx_t_3 = __pyx_t_4;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_4 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_term2) < (2.220446092504131e-16 * __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs((__pyx_v_c[0])))) != 0);
    __pyx_t_3 = __pyx_t_4;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_3) {

      /* "_sici.pxd":58
 *         s[0] += term1
 *         if zabs(term1) < TOL*zabs(s[0]) and zabs(term2) < TOL*zabs(c[0]):
 *             break             # <<<<<<<<<<<<<<
 * 
 * 
 */
      goto __pyx_L4_break;

      /* "_sici.pxd":57
 *         term1 = fac/(2*n + 1)
 *         s[0] += term1
 *         if zabs(term1) < TOL*zabs(s[0]) and zabs(term2) < TOL*zabs(c[0]):             # <<<<<<<<<<<<<<
 *             break
 * 
 */
    }
  }
  __pyx_L4_break:;

  /* "_sici.pxd":37
 * 
 * @cython.cdivision(True)
 * cdef inline void power_series(int sgn, double complex z,             # <<<<<<<<<<<<<<
 *                              double complex *s, double complex *c) nogil:
 *     """DLMF 6.6.5 and 6.6.6. If sgn = -1 computes si/ci, and if sgn = 1
 */

  /* function exit code */
}

/* "_sici.pxd":61
 * 
 * 
 * cdef inline int csici(double complex z,             # <<<<<<<<<<<<<<
 *                       double complex *si, double complex *ci) nogil:
 *     """Compute sin/cos integrals at complex arguments. The algorithm
 */

static CYTHON_INLINE int __pyx_f_5scipy_7special_5_sici_csici(__pyx_t_double_complex __pyx_v_z, __pyx_t_double_complex *__pyx_v_si, __pyx_t_double_complex *__pyx_v_ci) {
  __pyx_t_double_complex __pyx_v_jz;
  __pyx_t_double_complex __pyx_v_term1;
  __pyx_t_double_complex __pyx_v_term2;
  int __pyx_r;
  int __pyx_t_1;
  long __pyx_t_2;

  /* "_sici.pxd":69
 *     cdef double complex jz, term1, term2
 * 
 *     if z == inf:             # <<<<<<<<<<<<<<
 *         si[0] = NPY_PI_2
 *         ci[0] = 0
 */
  __pyx_t_1 = ((__Pyx_c_eq_double(__pyx_v_z, __pyx_t_double_complex_from_parts(NPY_INFINITY, 0))) != 0);
  if (__pyx_t_1) {

    /* "_sici.pxd":70
 * 
 *     if z == inf:
 *         si[0] = NPY_PI_2             # <<<<<<<<<<<<<<
 *         ci[0] = 0
 *         return 0
 */
    (__pyx_v_si[0]) = __pyx_t_double_complex_from_parts(NPY_PI_2, 0);

    /* "_sici.pxd":71
 *     if z == inf:
 *         si[0] = NPY_PI_2
 *         ci[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     elif z == -inf:
 */
    (__pyx_v_ci[0]) = __pyx_t_double_complex_from_parts(0, 0);

    /* "_sici.pxd":72
 *         si[0] = NPY_PI_2
 *         ci[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     elif z == -inf:
 *         si[0] = -NPY_PI_2
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "_sici.pxd":69
 *     cdef double complex jz, term1, term2
 * 
 *     if z == inf:             # <<<<<<<<<<<<<<
 *         si[0] = NPY_PI_2
 *         ci[0] = 0
 */
  }

  /* "_sici.pxd":73
 *         ci[0] = 0
 *         return 0
 *     elif z == -inf:             # <<<<<<<<<<<<<<
 *         si[0] = -NPY_PI_2
 *         ci[0] = 1j*NPY_PI
 */
  __pyx_t_1 = ((__Pyx_c_eq_double(__pyx_v_z, __pyx_t_double_complex_from_parts((-NPY_INFINITY), 0))) != 0);
  if (__pyx_t_1) {

    /* "_sici.pxd":74
 *         return 0
 *     elif z == -inf:
 *         si[0] = -NPY_PI_2             # <<<<<<<<<<<<<<
 *         ci[0] = 1j*NPY_PI
 *         return 0
 */
    (__pyx_v_si[0]) = __pyx_t_double_complex_from_parts((-NPY_PI_2), 0);

    /* "_sici.pxd":75
 *     elif z == -inf:
 *         si[0] = -NPY_PI_2
 *         ci[0] = 1j*NPY_PI             # <<<<<<<<<<<<<<
 *         return 0
 *     elif zabs(z) < 0.8:
 */
    (__pyx_v_ci[0]) = __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0, 1.0), __pyx_t_double_complex_from_parts(NPY_PI, 0));

    /* "_sici.pxd":76
 *         si[0] = -NPY_PI_2
 *         ci[0] = 1j*NPY_PI
 *         return 0             # <<<<<<<<<<<<<<
 *     elif zabs(z) < 0.8:
 *         # Use the series to avoid cancellation in si
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "_sici.pxd":73
 *         ci[0] = 0
 *         return 0
 *     elif z == -inf:             # <<<<<<<<<<<<<<
 *         si[0] = -NPY_PI_2
 *         ci[0] = 1j*NPY_PI
 */
  }

  /* "_sici.pxd":77
 *         ci[0] = 1j*NPY_PI
 *         return 0
 *     elif zabs(z) < 0.8:             # <<<<<<<<<<<<<<
 *         # Use the series to avoid cancellation in si
 *         power_series(-1, z, si, ci)
 */
  __pyx_t_1 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_z) < 0.8) != 0);
  if (__pyx_t_1) {

    /* "_sici.pxd":79
 *     elif zabs(z) < 0.8:
 *         # Use the series to avoid cancellation in si
 *         power_series(-1, z, si, ci)             # <<<<<<<<<<<<<<
 *         if z == 0:
 *             sf_error.error("sici", sf_error.DOMAIN, NULL)
 */
    __pyx_f_5scipy_7special_5_sici_power_series(-1, __pyx_v_z, __pyx_v_si, __pyx_v_ci);

    /* "_sici.pxd":80
 *         # Use the series to avoid cancellation in si
 *         power_series(-1, z, si, ci)
 *         if z == 0:             # <<<<<<<<<<<<<<
 *             sf_error.error("sici", sf_error.DOMAIN, NULL)
 *             ci[0] = zpack(-inf, nan)
 */
    __pyx_t_1 = ((__Pyx_c_eq_double(__pyx_v_z, __pyx_t_double_complex_from_parts(0, 0))) != 0);
    if (__pyx_t_1) {

      /* "_sici.pxd":81
 *         power_series(-1, z, si, ci)
 *         if z == 0:
 *             sf_error.error("sici", sf_error.DOMAIN, NULL)             # <<<<<<<<<<<<<<
 *             ci[0] = zpack(-inf, nan)
 *         else:
 */
      sf_error(((char *)"sici"), SF_ERROR_DOMAIN, NULL);

      /* "_sici.pxd":82
 *         if z == 0:
 *             sf_error.error("sici", sf_error.DOMAIN, NULL)
 *             ci[0] = zpack(-inf, nan)             # <<<<<<<<<<<<<<
 *         else:
 *             ci[0] += NPY_EULER + zlog(z)
 */
      (__pyx_v_ci[0]) = __pyx_f_5scipy_7special_13_complexstuff_zpack((-NPY_INFINITY), NPY_NAN);

      /* "_sici.pxd":80
 *         # Use the series to avoid cancellation in si
 *         power_series(-1, z, si, ci)
 *         if z == 0:             # <<<<<<<<<<<<<<
 *             sf_error.error("sici", sf_error.DOMAIN, NULL)
 *             ci[0] = zpack(-inf, nan)
 */
      goto __pyx_L4;
    }

    /* "_sici.pxd":84
 *             ci[0] = zpack(-inf, nan)
 *         else:
 *             ci[0] += NPY_EULER + zlog(z)             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
    /*else*/ {
      __pyx_t_2 = 0;
      (__pyx_v_ci[__pyx_t_2]) = __Pyx_c_sum_double((__pyx_v_ci[__pyx_t_2]), __Pyx_c_sum_double(__pyx_t_double_complex_from_parts(NPY_EULER, 0), __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zlog(__pyx_v_z)));
    }
    __pyx_L4:;

    /* "_sici.pxd":85
 *         else:
 *             ci[0] += NPY_EULER + zlog(z)
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     # DLMF 6.5.5/6.5.6 plus DLMF 6.4.4/6.4.6/6.4.7
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "_sici.pxd":77
 *         ci[0] = 1j*NPY_PI
 *         return 0
 *     elif zabs(z) < 0.8:             # <<<<<<<<<<<<<<
 *         # Use the series to avoid cancellation in si
 *         power_series(-1, z, si, ci)
 */
  }

  /* "_sici.pxd":88
 * 
 *     # DLMF 6.5.5/6.5.6 plus DLMF 6.4.4/6.4.6/6.4.7
 *     jz = 1j*z             # <<<<<<<<<<<<<<
 *     term1 = zexpi(jz)
 *     term2 = zexpi(-jz)
 */
  __pyx_v_jz = __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0, 1.0), __pyx_v_z);

  /* "_sici.pxd":89
 *     # DLMF 6.5.5/6.5.6 plus DLMF 6.4.4/6.4.6/6.4.7
 *     jz = 1j*z
 *     term1 = zexpi(jz)             # <<<<<<<<<<<<<<
 *     term2 = zexpi(-jz)
 *     si[0] = -0.5j*(term1 - term2)
 */
  __pyx_v_term1 = __pyx_f_5scipy_7special_5_sici_zexpi(__pyx_v_jz);

  /* "_sici.pxd":90
 *     jz = 1j*z
 *     term1 = zexpi(jz)
 *     term2 = zexpi(-jz)             # <<<<<<<<<<<<<<
 *     si[0] = -0.5j*(term1 - term2)
 *     ci[0] = 0.5*(term1 + term2)
 */
  __pyx_v_term2 = __pyx_f_5scipy_7special_5_sici_zexpi(__Pyx_c_neg_double(__pyx_v_jz));

  /* "_sici.pxd":91
 *     term1 = zexpi(jz)
 *     term2 = zexpi(-jz)
 *     si[0] = -0.5j*(term1 - term2)             # <<<<<<<<<<<<<<
 *     ci[0] = 0.5*(term1 + term2)
 *     if z.real == 0:
 */
  (__pyx_v_si[0]) = __Pyx_c_prod_double(__Pyx_c_neg_double(__pyx_t_double_complex_from_parts(0, 0.5)), __Pyx_c_diff_double(__pyx_v_term1, __pyx_v_term2));

  /* "_sici.pxd":92
 *     term2 = zexpi(-jz)
 *     si[0] = -0.5j*(term1 - term2)
 *     ci[0] = 0.5*(term1 + term2)             # <<<<<<<<<<<<<<
 *     if z.real == 0:
 *         if z.imag > 0:
 */
  (__pyx_v_ci[0]) = __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0.5, 0), __Pyx_c_sum_double(__pyx_v_term1, __pyx_v_term2));

  /* "_sici.pxd":93
 *     si[0] = -0.5j*(term1 - term2)
 *     ci[0] = 0.5*(term1 + term2)
 *     if z.real == 0:             # <<<<<<<<<<<<<<
 *         if z.imag > 0:
 *             ci[0] += 1j*NPY_PI_2
 */
  __pyx_t_1 = ((__Pyx_CREAL(__pyx_v_z) == 0.0) != 0);
  if (__pyx_t_1) {

    /* "_sici.pxd":94
 *     ci[0] = 0.5*(term1 + term2)
 *     if z.real == 0:
 *         if z.imag > 0:             # <<<<<<<<<<<<<<
 *             ci[0] += 1j*NPY_PI_2
 *         elif z.imag < 0:
 */
    __pyx_t_1 = ((__Pyx_CIMAG(__pyx_v_z) > 0.0) != 0);
    if (__pyx_t_1) {

      /* "_sici.pxd":95
 *     if z.real == 0:
 *         if z.imag > 0:
 *             ci[0] += 1j*NPY_PI_2             # <<<<<<<<<<<<<<
 *         elif z.imag < 0:
 *             ci[0] -= 1j*NPY_PI_2
 */
      __pyx_t_2 = 0;
      (__pyx_v_ci[__pyx_t_2]) = __Pyx_c_sum_double((__pyx_v_ci[__pyx_t_2]), __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0, 1.0), __pyx_t_double_complex_from_parts(NPY_PI_2, 0)));

      /* "_sici.pxd":94
 *     ci[0] = 0.5*(term1 + term2)
 *     if z.real == 0:
 *         if z.imag > 0:             # <<<<<<<<<<<<<<
 *             ci[0] += 1j*NPY_PI_2
 *         elif z.imag < 0:
 */
      goto __pyx_L6;
    }

    /* "_sici.pxd":96
 *         if z.imag > 0:
 *             ci[0] += 1j*NPY_PI_2
 *         elif z.imag < 0:             # <<<<<<<<<<<<<<
 *             ci[0] -= 1j*NPY_PI_2
 *     elif z.real > 0:
 */
    __pyx_t_1 = ((__Pyx_CIMAG(__pyx_v_z) < 0.0) != 0);
    if (__pyx_t_1) {

      /* "_sici.pxd":97
 *             ci[0] += 1j*NPY_PI_2
 *         elif z.imag < 0:
 *             ci[0] -= 1j*NPY_PI_2             # <<<<<<<<<<<<<<
 *     elif z.real > 0:
 *         si[0] -= NPY_PI_2
 */
      __pyx_t_2 = 0;
      (__pyx_v_ci[__pyx_t_2]) = __Pyx_c_diff_double((__pyx_v_ci[__pyx_t_2]), __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0, 1.0), __pyx_t_double_complex_from_parts(NPY_PI_2, 0)));

      /* "_sici.pxd":96
 *         if z.imag > 0:
 *             ci[0] += 1j*NPY_PI_2
 *         elif z.imag < 0:             # <<<<<<<<<<<<<<
 *             ci[0] -= 1j*NPY_PI_2
 *     elif z.real > 0:
 */
    }
    __pyx_L6:;

    /* "_sici.pxd":93
 *     si[0] = -0.5j*(term1 - term2)
 *     ci[0] = 0.5*(term1 + term2)
 *     if z.real == 0:             # <<<<<<<<<<<<<<
 *         if z.imag > 0:
 *             ci[0] += 1j*NPY_PI_2
 */
    goto __pyx_L5;
  }

  /* "_sici.pxd":98
 *         elif z.imag < 0:
 *             ci[0] -= 1j*NPY_PI_2
 *     elif z.real > 0:             # <<<<<<<<<<<<<<
 *         si[0] -= NPY_PI_2
 *     else:
 */
  __pyx_t_1 = ((__Pyx_CREAL(__pyx_v_z) > 0.0) != 0);
  if (__pyx_t_1) {

    /* "_sici.pxd":99
 *             ci[0] -= 1j*NPY_PI_2
 *     elif z.real > 0:
 *         si[0] -= NPY_PI_2             # <<<<<<<<<<<<<<
 *     else:
 *         si[0] += NPY_PI_2
 */
    __pyx_t_2 = 0;
    (__pyx_v_si[__pyx_t_2]) = __Pyx_c_diff_double((__pyx_v_si[__pyx_t_2]), __pyx_t_double_complex_from_parts(NPY_PI_2, 0));

    /* "_sici.pxd":98
 *         elif z.imag < 0:
 *             ci[0] -= 1j*NPY_PI_2
 *     elif z.real > 0:             # <<<<<<<<<<<<<<
 *         si[0] -= NPY_PI_2
 *     else:
 */
    goto __pyx_L5;
  }

  /* "_sici.pxd":101
 *         si[0] -= NPY_PI_2
 *     else:
 *         si[0] += NPY_PI_2             # <<<<<<<<<<<<<<
 *         if z.imag >= 0:
 *             ci[0] += 1j*NPY_PI
 */
  /*else*/ {
    __pyx_t_2 = 0;
    (__pyx_v_si[__pyx_t_2]) = __Pyx_c_sum_double((__pyx_v_si[__pyx_t_2]), __pyx_t_double_complex_from_parts(NPY_PI_2, 0));

    /* "_sici.pxd":102
 *     else:
 *         si[0] += NPY_PI_2
 *         if z.imag >= 0:             # <<<<<<<<<<<<<<
 *             ci[0] += 1j*NPY_PI
 *         else:
 */
    __pyx_t_1 = ((__Pyx_CIMAG(__pyx_v_z) >= 0.0) != 0);
    if (__pyx_t_1) {

      /* "_sici.pxd":103
 *         si[0] += NPY_PI_2
 *         if z.imag >= 0:
 *             ci[0] += 1j*NPY_PI             # <<<<<<<<<<<<<<
 *         else:
 *             ci[0] -= 1j*NPY_PI
 */
      __pyx_t_2 = 0;
      (__pyx_v_ci[__pyx_t_2]) = __Pyx_c_sum_double((__pyx_v_ci[__pyx_t_2]), __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0, 1.0), __pyx_t_double_complex_from_parts(NPY_PI, 0)));

      /* "_sici.pxd":102
 *     else:
 *         si[0] += NPY_PI_2
 *         if z.imag >= 0:             # <<<<<<<<<<<<<<
 *             ci[0] += 1j*NPY_PI
 *         else:
 */
      goto __pyx_L7;
    }

    /* "_sici.pxd":105
 *             ci[0] += 1j*NPY_PI
 *         else:
 *             ci[0] -= 1j*NPY_PI             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
    /*else*/ {
      __pyx_t_2 = 0;
      (__pyx_v_ci[__pyx_t_2]) = __Pyx_c_diff_double((__pyx_v_ci[__pyx_t_2]), __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0, 1.0), __pyx_t_double_complex_from_parts(NPY_PI, 0)));
    }
    __pyx_L7:;
  }
  __pyx_L5:;

  /* "_sici.pxd":107
 *             ci[0] -= 1j*NPY_PI
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "_sici.pxd":61
 * 
 * 
 * cdef inline int csici(double complex z,             # <<<<<<<<<<<<<<
 *                       double complex *si, double complex *ci) nogil:
 *     """Compute sin/cos integrals at complex arguments. The algorithm
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_sici.pxd":110
 * 
 * 
 * cdef inline int cshichi(double complex z,             # <<<<<<<<<<<<<<
 *                         double complex *shi, double complex *chi) nogil:
 *     """Compute sinh/cosh integrals at complex arguments. The algorithm
 */

static CYTHON_INLINE int __pyx_f_5scipy_7special_5_sici_cshichi(__pyx_t_double_complex __pyx_v_z, __pyx_t_double_complex *__pyx_v_shi, __pyx_t_double_complex *__pyx_v_chi) {
  __pyx_t_double_complex __pyx_v_term1;
  __pyx_t_double_complex __pyx_v_term2;
  int __pyx_r;
  int __pyx_t_1;
  long __pyx_t_2;

  /* "_sici.pxd":118
 *     cdef double complex term1, term2
 * 
 *     if z == inf:             # <<<<<<<<<<<<<<
 *         shi[0] = inf
 *         chi[0] = inf
 */
  __pyx_t_1 = ((__Pyx_c_eq_double(__pyx_v_z, __pyx_t_double_complex_from_parts(NPY_INFINITY, 0))) != 0);
  if (__pyx_t_1) {

    /* "_sici.pxd":119
 * 
 *     if z == inf:
 *         shi[0] = inf             # <<<<<<<<<<<<<<
 *         chi[0] = inf
 *         return 0
 */
    (__pyx_v_shi[0]) = __pyx_t_double_complex_from_parts(NPY_INFINITY, 0);

    /* "_sici.pxd":120
 *     if z == inf:
 *         shi[0] = inf
 *         chi[0] = inf             # <<<<<<<<<<<<<<
 *         return 0
 *     elif z == -inf:
 */
    (__pyx_v_chi[0]) = __pyx_t_double_complex_from_parts(NPY_INFINITY, 0);

    /* "_sici.pxd":121
 *         shi[0] = inf
 *         chi[0] = inf
 *         return 0             # <<<<<<<<<<<<<<
 *     elif z == -inf:
 *         shi[0] = -inf
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "_sici.pxd":118
 *     cdef double complex term1, term2
 * 
 *     if z == inf:             # <<<<<<<<<<<<<<
 *         shi[0] = inf
 *         chi[0] = inf
 */
  }

  /* "_sici.pxd":122
 *         chi[0] = inf
 *         return 0
 *     elif z == -inf:             # <<<<<<<<<<<<<<
 *         shi[0] = -inf
 *         chi[0] = inf
 */
  __pyx_t_1 = ((__Pyx_c_eq_double(__pyx_v_z, __pyx_t_double_complex_from_parts((-NPY_INFINITY), 0))) != 0);
  if (__pyx_t_1) {

    /* "_sici.pxd":123
 *         return 0
 *     elif z == -inf:
 *         shi[0] = -inf             # <<<<<<<<<<<<<<
 *         chi[0] = inf
 *         return 0
 */
    (__pyx_v_shi[0]) = __pyx_t_double_complex_from_parts((-NPY_INFINITY), 0);

    /* "_sici.pxd":124
 *     elif z == -inf:
 *         shi[0] = -inf
 *         chi[0] = inf             # <<<<<<<<<<<<<<
 *         return 0
 *     elif zabs(z) < 0.8:
 */
    (__pyx_v_chi[0]) = __pyx_t_double_complex_from_parts(NPY_INFINITY, 0);

    /* "_sici.pxd":125
 *         shi[0] = -inf
 *         chi[0] = inf
 *         return 0             # <<<<<<<<<<<<<<
 *     elif zabs(z) < 0.8:
 *         # Use the series to avoid cancellation in shi
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "_sici.pxd":122
 *         chi[0] = inf
 *         return 0
 *     elif z == -inf:             # <<<<<<<<<<<<<<
 *         shi[0] = -inf
 *         chi[0] = inf
 */
  }

  /* "_sici.pxd":126
 *         chi[0] = inf
 *         return 0
 *     elif zabs(z) < 0.8:             # <<<<<<<<<<<<<<
 *         # Use the series to avoid cancellation in shi
 *         power_series(1, z, shi, chi)
 */
  __pyx_t_1 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_z) < 0.8) != 0);
  if (__pyx_t_1) {

    /* "_sici.pxd":128
 *     elif zabs(z) < 0.8:
 *         # Use the series to avoid cancellation in shi
 *         power_series(1, z, shi, chi)             # <<<<<<<<<<<<<<
 *         if z == 0:
 *             sf_error.error("shichi", sf_error.DOMAIN, NULL)
 */
    __pyx_f_5scipy_7special_5_sici_power_series(1, __pyx_v_z, __pyx_v_shi, __pyx_v_chi);

    /* "_sici.pxd":129
 *         # Use the series to avoid cancellation in shi
 *         power_series(1, z, shi, chi)
 *         if z == 0:             # <<<<<<<<<<<<<<
 *             sf_error.error("shichi", sf_error.DOMAIN, NULL)
 *             chi[0] = zpack(-inf, nan)
 */
    __pyx_t_1 = ((__Pyx_c_eq_double(__pyx_v_z, __pyx_t_double_complex_from_parts(0, 0))) != 0);
    if (__pyx_t_1) {

      /* "_sici.pxd":130
 *         power_series(1, z, shi, chi)
 *         if z == 0:
 *             sf_error.error("shichi", sf_error.DOMAIN, NULL)             # <<<<<<<<<<<<<<
 *             chi[0] = zpack(-inf, nan)
 *         else:
 */
      sf_error(((char *)"shichi"), SF_ERROR_DOMAIN, NULL);

      /* "_sici.pxd":131
 *         if z == 0:
 *             sf_error.error("shichi", sf_error.DOMAIN, NULL)
 *             chi[0] = zpack(-inf, nan)             # <<<<<<<<<<<<<<
 *         else:
 *             chi[0] += NPY_EULER + zlog(z)
 */
      (__pyx_v_chi[0]) = __pyx_f_5scipy_7special_13_complexstuff_zpack((-NPY_INFINITY), NPY_NAN);

      /* "_sici.pxd":129
 *         # Use the series to avoid cancellation in shi
 *         power_series(1, z, shi, chi)
 *         if z == 0:             # <<<<<<<<<<<<<<
 *             sf_error.error("shichi", sf_error.DOMAIN, NULL)
 *             chi[0] = zpack(-inf, nan)
 */
      goto __pyx_L4;
    }

    /* "_sici.pxd":133
 *             chi[0] = zpack(-inf, nan)
 *         else:
 *             chi[0] += NPY_EULER + zlog(z)             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
    /*else*/ {
      __pyx_t_2 = 0;
      (__pyx_v_chi[__pyx_t_2]) = __Pyx_c_sum_double((__pyx_v_chi[__pyx_t_2]), __Pyx_c_sum_double(__pyx_t_double_complex_from_parts(NPY_EULER, 0), __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zlog(__pyx_v_z)));
    }
    __pyx_L4:;

    /* "_sici.pxd":134
 *         else:
 *             chi[0] += NPY_EULER + zlog(z)
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     term1 = zexpi(z)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "_sici.pxd":126
 *         chi[0] = inf
 *         return 0
 *     elif zabs(z) < 0.8:             # <<<<<<<<<<<<<<
 *         # Use the series to avoid cancellation in shi
 *         power_series(1, z, shi, chi)
 */
  }

  /* "_sici.pxd":136
 *         return 0
 * 
 *     term1 = zexpi(z)             # <<<<<<<<<<<<<<
 *     term2 = zexpi(-z)
 *     shi[0] = 0.5*(term1 - term2)
 */
  __pyx_v_term1 = __pyx_f_5scipy_7special_5_sici_zexpi(__pyx_v_z);

  /* "_sici.pxd":137
 * 
 *     term1 = zexpi(z)
 *     term2 = zexpi(-z)             # <<<<<<<<<<<<<<
 *     shi[0] = 0.5*(term1 - term2)
 *     chi[0] = 0.5*(term1 + term2)
 */
  __pyx_v_term2 = __pyx_f_5scipy_7special_5_sici_zexpi(__Pyx_c_neg_double(__pyx_v_z));

  /* "_sici.pxd":138
 *     term1 = zexpi(z)
 *     term2 = zexpi(-z)
 *     shi[0] = 0.5*(term1 - term2)             # <<<<<<<<<<<<<<
 *     chi[0] = 0.5*(term1 + term2)
 *     if z.imag > 0:
 */
  (__pyx_v_shi[0]) = __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0.5, 0), __Pyx_c_diff_double(__pyx_v_term1, __pyx_v_term2));

  /* "_sici.pxd":139
 *     term2 = zexpi(-z)
 *     shi[0] = 0.5*(term1 - term2)
 *     chi[0] = 0.5*(term1 + term2)             # <<<<<<<<<<<<<<
 *     if z.imag > 0:
 *         shi[0] -= 0.5j*NPY_PI
 */
  (__pyx_v_chi[0]) = __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0.5, 0), __Pyx_c_sum_double(__pyx_v_term1, __pyx_v_term2));

  /* "_sici.pxd":140
 *     shi[0] = 0.5*(term1 - term2)
 *     chi[0] = 0.5*(term1 + term2)
 *     if z.imag > 0:             # <<<<<<<<<<<<<<
 *         shi[0] -= 0.5j*NPY_PI
 *         chi[0] += 0.5j*NPY_PI
 */
  __pyx_t_1 = ((__Pyx_CIMAG(__pyx_v_z) > 0.0) != 0);
  if (__pyx_t_1) {

    /* "_sici.pxd":141
 *     chi[0] = 0.5*(term1 + term2)
 *     if z.imag > 0:
 *         shi[0] -= 0.5j*NPY_PI             # <<<<<<<<<<<<<<
 *         chi[0] += 0.5j*NPY_PI
 *     elif z.imag < 0:
 */
    __pyx_t_2 = 0;
    (__pyx_v_shi[__pyx_t_2]) = __Pyx_c_diff_double((__pyx_v_shi[__pyx_t_2]), __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0, 0.5), __pyx_t_double_complex_from_parts(NPY_PI, 0)));

    /* "_sici.pxd":142
 *     if z.imag > 0:
 *         shi[0] -= 0.5j*NPY_PI
 *         chi[0] += 0.5j*NPY_PI             # <<<<<<<<<<<<<<
 *     elif z.imag < 0:
 *         shi[0] += 0.5j*NPY_PI
 */
    __pyx_t_2 = 0;
    (__pyx_v_chi[__pyx_t_2]) = __Pyx_c_sum_double((__pyx_v_chi[__pyx_t_2]), __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0, 0.5), __pyx_t_double_complex_from_parts(NPY_PI, 0)));

    /* "_sici.pxd":140
 *     shi[0] = 0.5*(term1 - term2)
 *     chi[0] = 0.5*(term1 + term2)
 *     if z.imag > 0:             # <<<<<<<<<<<<<<
 *         shi[0] -= 0.5j*NPY_PI
 *         chi[0] += 0.5j*NPY_PI
 */
    goto __pyx_L5;
  }

  /* "_sici.pxd":143
 *         shi[0] -= 0.5j*NPY_PI
 *         chi[0] += 0.5j*NPY_PI
 *     elif z.imag < 0:             # <<<<<<<<<<<<<<
 *         shi[0] += 0.5j*NPY_PI
 *         chi[0] -= 0.5j*NPY_PI
 */
  __pyx_t_1 = ((__Pyx_CIMAG(__pyx_v_z) < 0.0) != 0);
  if (__pyx_t_1) {

    /* "_sici.pxd":144
 *         chi[0] += 0.5j*NPY_PI
 *     elif z.imag < 0:
 *         shi[0] += 0.5j*NPY_PI             # <<<<<<<<<<<<<<
 *         chi[0] -= 0.5j*NPY_PI
 *     elif z.real < 0:
 */
    __pyx_t_2 = 0;
    (__pyx_v_shi[__pyx_t_2]) = __Pyx_c_sum_double((__pyx_v_shi[__pyx_t_2]), __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0, 0.5), __pyx_t_double_complex_from_parts(NPY_PI, 0)));

    /* "_sici.pxd":145
 *     elif z.imag < 0:
 *         shi[0] += 0.5j*NPY_PI
 *         chi[0] -= 0.5j*NPY_PI             # <<<<<<<<<<<<<<
 *     elif z.real < 0:
 *         chi[0] += 1j*NPY_PI
 */
    __pyx_t_2 = 0;
    (__pyx_v_chi[__pyx_t_2]) = __Pyx_c_diff_double((__pyx_v_chi[__pyx_t_2]), __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0, 0.5), __pyx_t_double_complex_from_parts(NPY_PI, 0)));

    /* "_sici.pxd":143
 *         shi[0] -= 0.5j*NPY_PI
 *         chi[0] += 0.5j*NPY_PI
 *     elif z.imag < 0:             # <<<<<<<<<<<<<<
 *         shi[0] += 0.5j*NPY_PI
 *         chi[0] -= 0.5j*NPY_PI
 */
    goto __pyx_L5;
  }

  /* "_sici.pxd":146
 *         shi[0] += 0.5j*NPY_PI
 *         chi[0] -= 0.5j*NPY_PI
 *     elif z.real < 0:             # <<<<<<<<<<<<<<
 *         chi[0] += 1j*NPY_PI
 * 
 */
  __pyx_t_1 = ((__Pyx_CREAL(__pyx_v_z) < 0.0) != 0);
  if (__pyx_t_1) {

    /* "_sici.pxd":147
 *         chi[0] -= 0.5j*NPY_PI
 *     elif z.real < 0:
 *         chi[0] += 1j*NPY_PI             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
    __pyx_t_2 = 0;
    (__pyx_v_chi[__pyx_t_2]) = __Pyx_c_sum_double((__pyx_v_chi[__pyx_t_2]), __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0, 1.0), __pyx_t_double_complex_from_parts(NPY_PI, 0)));

    /* "_sici.pxd":146
 *         shi[0] += 0.5j*NPY_PI
 *         chi[0] -= 0.5j*NPY_PI
 *     elif z.real < 0:             # <<<<<<<<<<<<<<
 *         chi[0] += 1j*NPY_PI
 * 
 */
  }
  __pyx_L5:;

  /* "_sici.pxd":149
 *         chi[0] += 1j*NPY_PI
 * 
 *     return 0             # <<<<<<<<<<<<<<
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "_sici.pxd":110
 * 
 * 
 * cdef inline int cshichi(double complex z,             # <<<<<<<<<<<<<<
 *                         double complex *shi, double complex *chi) nogil:
 *     """Compute sinh/cosh integrals at complex arguments. The algorithm
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spence.pxd":23
 * 
 * @cython.cdivision(True)
 * cdef inline double complex cspence(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Compute Spence's function for complex arguments. The strategy is:
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_7_spence_cspence(__pyx_t_double_complex __pyx_v_z) {
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;

  /* "_spence.pxd":35
 * 
 *     """
 *     if zabs(z) < 0.5:             # <<<<<<<<<<<<<<
 *         # This step isn't necessary, but this series converges faster.
 *         return cspence_series0(z)
 */
  __pyx_t_1 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_z) < 0.5) != 0);
  if (__pyx_t_1) {

    /* "_spence.pxd":37
 *     if zabs(z) < 0.5:
 *         # This step isn't necessary, but this series converges faster.
 *         return cspence_series0(z)             # <<<<<<<<<<<<<<
 *     elif zabs(1 - z) > 1:
 *         return -cspence_series1(z/(z - 1)) - PISQ_6 - 0.5*zlog1(z - 1)**2
 */
    __pyx_r = __pyx_f_5scipy_7special_7_spence_cspence_series0(__pyx_v_z);
    goto __pyx_L0;

    /* "_spence.pxd":35
 * 
 *     """
 *     if zabs(z) < 0.5:             # <<<<<<<<<<<<<<
 *         # This step isn't necessary, but this series converges faster.
 *         return cspence_series0(z)
 */
  }

  /* "_spence.pxd":38
 *         # This step isn't necessary, but this series converges faster.
 *         return cspence_series0(z)
 *     elif zabs(1 - z) > 1:             # <<<<<<<<<<<<<<
 *         return -cspence_series1(z/(z - 1)) - PISQ_6 - 0.5*zlog1(z - 1)**2
 *     else:
 */
  __pyx_t_1 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__Pyx_c_diff_double(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_z)) > 1.0) != 0);
  if (__pyx_t_1) {

    /* "_spence.pxd":39
 *         return cspence_series0(z)
 *     elif zabs(1 - z) > 1:
 *         return -cspence_series1(z/(z - 1)) - PISQ_6 - 0.5*zlog1(z - 1)**2             # <<<<<<<<<<<<<<
 *     else:
 *         return cspence_series1(z)
 */
    __pyx_r = __Pyx_c_diff_double(__Pyx_c_diff_double(__Pyx_c_neg_double(__pyx_f_5scipy_7special_7_spence_cspence_series1(__Pyx_c_quot_double(__pyx_v_z, __Pyx_c_diff_double(__pyx_v_z, __pyx_t_double_complex_from_parts(1, 0))))), __pyx_t_double_complex_from_parts(1.6449340668482264, 0)), __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(0.5, 0), __Pyx_c_pow_double(__pyx_f_5scipy_7special_13_complexstuff_zlog1(__Pyx_c_diff_double(__pyx_v_z, __pyx_t_double_complex_from_parts(1, 0))), __pyx_t_double_complex_from_parts(2, 0))));
    goto __pyx_L0;

    /* "_spence.pxd":38
 *         # This step isn't necessary, but this series converges faster.
 *         return cspence_series0(z)
 *     elif zabs(1 - z) > 1:             # <<<<<<<<<<<<<<
 *         return -cspence_series1(z/(z - 1)) - PISQ_6 - 0.5*zlog1(z - 1)**2
 *     else:
 */
  }

  /* "_spence.pxd":41
 *         return -cspence_series1(z/(z - 1)) - PISQ_6 - 0.5*zlog1(z - 1)**2
 *     else:
 *         return cspence_series1(z)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_r = __pyx_f_5scipy_7special_7_spence_cspence_series1(__pyx_v_z);
    goto __pyx_L0;
  }

  /* "_spence.pxd":23
 * 
 * @cython.cdivision(True)
 * cdef inline double complex cspence(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Compute Spence's function for complex arguments. The strategy is:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spence.pxd":45
 * 
 * @cython.cdivision(True)
 * cdef inline double complex cspence_series0(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     A series centered at z = 0; see
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_7_spence_cspence_series0(__pyx_t_double_complex __pyx_v_z) {
  int __pyx_v_n;
  __pyx_t_double_complex __pyx_v_zfac;
  __pyx_t_double_complex __pyx_v_sum1;
  __pyx_t_double_complex __pyx_v_sum2;
  __pyx_t_double_complex __pyx_v_term1;
  __pyx_t_double_complex __pyx_v_term2;
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "_spence.pxd":54
 *     cdef:
 *         int n
 *         double complex zfac = 1             # <<<<<<<<<<<<<<
 *         double complex sum1 = 0
 *         double complex sum2 = 0
 */
  __pyx_v_zfac = __pyx_t_double_complex_from_parts(1, 0);

  /* "_spence.pxd":55
 *         int n
 *         double complex zfac = 1
 *         double complex sum1 = 0             # <<<<<<<<<<<<<<
 *         double complex sum2 = 0
 *         double complex term1, term2
 */
  __pyx_v_sum1 = __pyx_t_double_complex_from_parts(0, 0);

  /* "_spence.pxd":56
 *         double complex zfac = 1
 *         double complex sum1 = 0
 *         double complex sum2 = 0             # <<<<<<<<<<<<<<
 *         double complex term1, term2
 * 
 */
  __pyx_v_sum2 = __pyx_t_double_complex_from_parts(0, 0);

  /* "_spence.pxd":59
 *         double complex term1, term2
 * 
 *     if z == 0:             # <<<<<<<<<<<<<<
 *         return PISQ_6
 * 
 */
  __pyx_t_1 = ((__Pyx_c_eq_double(__pyx_v_z, __pyx_t_double_complex_from_parts(0, 0))) != 0);
  if (__pyx_t_1) {

    /* "_spence.pxd":60
 * 
 *     if z == 0:
 *         return PISQ_6             # <<<<<<<<<<<<<<
 * 
 *     for n in range(1, 500):
 */
    __pyx_r = __pyx_t_double_complex_from_parts(1.6449340668482264, 0);
    goto __pyx_L0;

    /* "_spence.pxd":59
 *         double complex term1, term2
 * 
 *     if z == 0:             # <<<<<<<<<<<<<<
 *         return PISQ_6
 * 
 */
  }

  /* "_spence.pxd":62
 *         return PISQ_6
 * 
 *     for n in range(1, 500):             # <<<<<<<<<<<<<<
 *         zfac *= z
 *         term1 = zfac/n**2
 */
  for (__pyx_t_2 = 1; __pyx_t_2 < 0x1F4; __pyx_t_2+=1) {
    __pyx_v_n = __pyx_t_2;

    /* "_spence.pxd":63
 * 
 *     for n in range(1, 500):
 *         zfac *= z             # <<<<<<<<<<<<<<
 *         term1 = zfac/n**2
 *         sum1 += term1
 */
    __pyx_v_zfac = __Pyx_c_prod_double(__pyx_v_zfac, __pyx_v_z);

    /* "_spence.pxd":64
 *     for n in range(1, 500):
 *         zfac *= z
 *         term1 = zfac/n**2             # <<<<<<<<<<<<<<
 *         sum1 += term1
 *         term2 = zfac/n
 */
    __pyx_v_term1 = __Pyx_c_quot_double(__pyx_v_zfac, __pyx_t_double_complex_from_parts(__Pyx_pow_long(((long)__pyx_v_n), 2), 0));

    /* "_spence.pxd":65
 *         zfac *= z
 *         term1 = zfac/n**2
 *         sum1 += term1             # <<<<<<<<<<<<<<
 *         term2 = zfac/n
 *         sum2 += term2
 */
    __pyx_v_sum1 = __Pyx_c_sum_double(__pyx_v_sum1, __pyx_v_term1);

    /* "_spence.pxd":66
 *         term1 = zfac/n**2
 *         sum1 += term1
 *         term2 = zfac/n             # <<<<<<<<<<<<<<
 *         sum2 += term2
 *         if zabs(term1) <= TOL*zabs(sum1) and zabs(term2) <= TOL*zabs(sum2):
 */
    __pyx_v_term2 = __Pyx_c_quot_double(__pyx_v_zfac, __pyx_t_double_complex_from_parts(__pyx_v_n, 0));

    /* "_spence.pxd":67
 *         sum1 += term1
 *         term2 = zfac/n
 *         sum2 += term2             # <<<<<<<<<<<<<<
 *         if zabs(term1) <= TOL*zabs(sum1) and zabs(term2) <= TOL*zabs(sum2):
 *             break
 */
    __pyx_v_sum2 = __Pyx_c_sum_double(__pyx_v_sum2, __pyx_v_term2);

    /* "_spence.pxd":68
 *         term2 = zfac/n
 *         sum2 += term2
 *         if zabs(term1) <= TOL*zabs(sum1) and zabs(term2) <= TOL*zabs(sum2):             # <<<<<<<<<<<<<<
 *             break
 *     return PISQ_6 - sum1 + zlog1(z)*sum2
 */
    __pyx_t_3 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_term1) <= (2.220446092504131e-16 * __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_sum1))) != 0);
    if (__pyx_t_3) {
    } else {
      __pyx_t_1 = __pyx_t_3;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_term2) <= (2.220446092504131e-16 * __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_sum2))) != 0);
    __pyx_t_1 = __pyx_t_3;
    __pyx_L7_bool_binop_done:;
    if (__pyx_t_1) {

      /* "_spence.pxd":69
 *         sum2 += term2
 *         if zabs(term1) <= TOL*zabs(sum1) and zabs(term2) <= TOL*zabs(sum2):
 *             break             # <<<<<<<<<<<<<<
 *     return PISQ_6 - sum1 + zlog1(z)*sum2
 * 
 */
      goto __pyx_L5_break;

      /* "_spence.pxd":68
 *         term2 = zfac/n
 *         sum2 += term2
 *         if zabs(term1) <= TOL*zabs(sum1) and zabs(term2) <= TOL*zabs(sum2):             # <<<<<<<<<<<<<<
 *             break
 *     return PISQ_6 - sum1 + zlog1(z)*sum2
 */
    }
  }
  __pyx_L5_break:;

  /* "_spence.pxd":70
 *         if zabs(term1) <= TOL*zabs(sum1) and zabs(term2) <= TOL*zabs(sum2):
 *             break
 *     return PISQ_6 - sum1 + zlog1(z)*sum2             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __Pyx_c_sum_double(__Pyx_c_diff_double(__pyx_t_double_complex_from_parts(1.6449340668482264, 0), __pyx_v_sum1), __Pyx_c_prod_double(__pyx_f_5scipy_7special_13_complexstuff_zlog1(__pyx_v_z), __pyx_v_sum2));
  goto __pyx_L0;

  /* "_spence.pxd":45
 * 
 * @cython.cdivision(True)
 * cdef inline double complex cspence_series0(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     A series centered at z = 0; see
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "_spence.pxd":74
 * 
 * @cython.cdivision(True)
 * cdef inline double complex cspence_series1(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     A series centered at z = 1 which enjoys faster convergence than
 */

static CYTHON_INLINE __pyx_t_double_complex __pyx_f_5scipy_7special_7_spence_cspence_series1(__pyx_t_double_complex __pyx_v_z) {
  int __pyx_v_n;
  __pyx_t_double_complex __pyx_v_zfac;
  __pyx_t_double_complex __pyx_v_res;
  __pyx_t_double_complex __pyx_v_term;
  __pyx_t_double_complex __pyx_v_zz;
  __pyx_t_double_complex __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_spence.pxd":84
 *     cdef:
 *         int n
 *         double complex zfac = 1             # <<<<<<<<<<<<<<
 *         double complex res = 0
 *         double complex term, zz
 */
  __pyx_v_zfac = __pyx_t_double_complex_from_parts(1, 0);

  /* "_spence.pxd":85
 *         int n
 *         double complex zfac = 1
 *         double complex res = 0             # <<<<<<<<<<<<<<
 *         double complex term, zz
 * 
 */
  __pyx_v_res = __pyx_t_double_complex_from_parts(0, 0);

  /* "_spence.pxd":88
 *         double complex term, zz
 * 
 *     if z == 1:             # <<<<<<<<<<<<<<
 *         return 0
 *     z = 1 - z
 */
  __pyx_t_1 = ((__Pyx_c_eq_double(__pyx_v_z, __pyx_t_double_complex_from_parts(1, 0))) != 0);
  if (__pyx_t_1) {

    /* "_spence.pxd":89
 * 
 *     if z == 1:
 *         return 0             # <<<<<<<<<<<<<<
 *     z = 1 - z
 *     zz = z**2
 */
    __pyx_r = __pyx_t_double_complex_from_parts(0, 0);
    goto __pyx_L0;

    /* "_spence.pxd":88
 *         double complex term, zz
 * 
 *     if z == 1:             # <<<<<<<<<<<<<<
 *         return 0
 *     z = 1 - z
 */
  }

  /* "_spence.pxd":90
 *     if z == 1:
 *         return 0
 *     z = 1 - z             # <<<<<<<<<<<<<<
 *     zz = z**2
 *     for n in range(1, 500):
 */
  __pyx_v_z = __Pyx_c_diff_double(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_z);

  /* "_spence.pxd":91
 *         return 0
 *     z = 1 - z
 *     zz = z**2             # <<<<<<<<<<<<<<
 *     for n in range(1, 500):
 *         zfac *= z
 */
  __pyx_v_zz = __Pyx_c_pow_double(__pyx_v_z, __pyx_t_double_complex_from_parts(2, 0));

  /* "_spence.pxd":92
 *     z = 1 - z
 *     zz = z**2
 *     for n in range(1, 500):             # <<<<<<<<<<<<<<
 *         zfac *= z
 *         # Do the divisions one at a time to guard against overflow
 */
  for (__pyx_t_2 = 1; __pyx_t_2 < 0x1F4; __pyx_t_2+=1) {
    __pyx_v_n = __pyx_t_2;

    /* "_spence.pxd":93
 *     zz = z**2
 *     for n in range(1, 500):
 *         zfac *= z             # <<<<<<<<<<<<<<
 *         # Do the divisions one at a time to guard against overflow
 *         term = ((zfac/n**2)/(n + 1)**2)/(n + 2)**2
 */
    __pyx_v_zfac = __Pyx_c_prod_double(__pyx_v_zfac, __pyx_v_z);

    /* "_spence.pxd":95
 *         zfac *= z
 *         # Do the divisions one at a time to guard against overflow
 *         term = ((zfac/n**2)/(n + 1)**2)/(n + 2)**2             # <<<<<<<<<<<<<<
 *         res += term
 *         if zabs(term) <= TOL*zabs(res):
 */
    __pyx_v_term = __Pyx_c_quot_double(__Pyx_c_quot_double(__Pyx_c_quot_double(__pyx_v_zfac, __pyx_t_double_complex_from_parts(__Pyx_pow_long(((long)__pyx_v_n), 2), 0)), __pyx_t_double_complex_from_parts(__Pyx_pow_long((__pyx_v_n + 1), 2), 0)), __pyx_t_double_complex_from_parts(__Pyx_pow_long((__pyx_v_n + 2), 2), 0));

    /* "_spence.pxd":96
 *         # Do the divisions one at a time to guard against overflow
 *         term = ((zfac/n**2)/(n + 1)**2)/(n + 2)**2
 *         res += term             # <<<<<<<<<<<<<<
 *         if zabs(term) <= TOL*zabs(res):
 *             break
 */
    __pyx_v_res = __Pyx_c_sum_double(__pyx_v_res, __pyx_v_term);

    /* "_spence.pxd":97
 *         term = ((zfac/n**2)/(n + 1)**2)/(n + 2)**2
 *         res += term
 *         if zabs(term) <= TOL*zabs(res):             # <<<<<<<<<<<<<<
 *             break
 *     res *= 4*zz
 */
    __pyx_t_1 = ((__pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_term) <= (2.220446092504131e-16 * __pyx_fuse_1__pyx_f_5scipy_7special_13_complexstuff_zabs(__pyx_v_res))) != 0);
    if (__pyx_t_1) {

      /* "_spence.pxd":98
 *         res += term
 *         if zabs(term) <= TOL*zabs(res):
 *             break             # <<<<<<<<<<<<<<
 *     res *= 4*zz
 *     res += 4*z + 5.75*zz + 3*(1 - zz)*zlog1(1 - z)
 */
      goto __pyx_L5_break;

      /* "_spence.pxd":97
 *         term = ((zfac/n**2)/(n + 1)**2)/(n + 2)**2
 *         res += term
 *         if zabs(term) <= TOL*zabs(res):             # <<<<<<<<<<<<<<
 *             break
 *     res *= 4*zz
 */
    }
  }
  __pyx_L5_break:;

  /* "_spence.pxd":99
 *         if zabs(term) <= TOL*zabs(res):
 *             break
 *     res *= 4*zz             # <<<<<<<<<<<<<<
 *     res += 4*z + 5.75*zz + 3*(1 - zz)*zlog1(1 - z)
 *     res /= 1 + 4*z + zz
 */
  __pyx_v_res = __Pyx_c_prod_double(__pyx_v_res, __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(4, 0), __pyx_v_zz));

  /* "_spence.pxd":100
 *             break
 *     res *= 4*zz
 *     res += 4*z + 5.75*zz + 3*(1 - zz)*zlog1(1 - z)             # <<<<<<<<<<<<<<
 *     res /= 1 + 4*z + zz
 *     return res
 */
  __pyx_v_res = __Pyx_c_sum_double(__pyx_v_res, __Pyx_c_sum_double(__Pyx_c_sum_double(__Pyx_c_prod_double(__pyx_t_double_complex_from_parts(4, 0), __pyx_v_z), __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(5.75, 0), __pyx_v_zz)), __Pyx_c_prod_double(__Pyx_c_prod_double(__pyx_t_double_complex_from_parts(3, 0), __Pyx_c_diff_double(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_zz)), __pyx_f_5scipy_7special_13_complexstuff_zlog1(__Pyx_c_diff_double(__pyx_t_double_complex_from_parts(1, 0), __pyx_v_z)))));

  /* "_spence.pxd":101
 *     res *= 4*zz
 *     res += 4*z + 5.75*zz + 3*(1 - zz)*zlog1(1 - z)
 *     res /= 1 + 4*z + zz             # <<<<<<<<<<<<<<
 *     return res
 */
  __pyx_v_res = __Pyx_c_quot_double(__pyx_v_res, __Pyx_c_sum_double(__Pyx_c_sum_double(__pyx_t_double_complex_from_parts(1, 0), __Pyx_c_prod_double(__pyx_t_double_complex_from_parts(4, 0), __pyx_v_z)), __pyx_v_zz));

  /* "_spence.pxd":102
 *     res += 4*z + 5.75*zz + 3*(1 - zz)*zlog1(1 - z)
 *     res /= 1 + 4*z + zz
 *     return res             # <<<<<<<<<<<<<<
 */
  __pyx_r = __pyx_v_res;
  goto __pyx_L0;

  /* "_spence.pxd":74
 * 
 * @cython.cdivision(True)
 * cdef inline double complex cspence_series1(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     A series centered at z = 1 which enjoys faster convergence than
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec__ufuncs(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec__ufuncs},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "_ufuncs",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_Context_manager_for_special_func, __pyx_k_Context_manager_for_special_func, sizeof(__pyx_k_Context_manager_for_special_func), 0, 0, 1, 0},
  {&__pyx_kp_u_Format_string_allocated_too_shor, __pyx_k_Format_string_allocated_too_shor, sizeof(__pyx_k_Format_string_allocated_too_shor), 0, 1, 0, 0},
  {&__pyx_kp_u_Format_string_allocated_too_shor_2, __pyx_k_Format_string_allocated_too_shor_2, sizeof(__pyx_k_Format_string_allocated_too_shor_2), 0, 1, 0, 0},
  {&__pyx_kp_u_Get_the_current_way_of_handling, __pyx_k_Get_the_current_way_of_handling, sizeof(__pyx_k_Get_the_current_way_of_handling), 0, 1, 0, 0},
  {&__pyx_n_s_ImportError, __pyx_k_ImportError, sizeof(__pyx_k_ImportError), 0, 0, 1, 1},
  {&__pyx_kp_u_Non_native_byte_order_not_suppor, __pyx_k_Non_native_byte_order_not_suppor, sizeof(__pyx_k_Non_native_byte_order_not_suppor), 0, 1, 0, 0},
  {&__pyx_n_s_RuntimeError, __pyx_k_RuntimeError, sizeof(__pyx_k_RuntimeError), 0, 0, 1, 1},
  {&__pyx_n_s_RuntimeWarning, __pyx_k_RuntimeWarning, sizeof(__pyx_k_RuntimeWarning), 0, 0, 1, 1},
  {&__pyx_kp_u_Set_how_special_function_errors, __pyx_k_Set_how_special_function_errors, sizeof(__pyx_k_Set_how_special_function_errors), 0, 1, 0, 0},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_action, __pyx_k_action, sizeof(__pyx_k_action), 0, 0, 1, 1},
  {&__pyx_n_s_agm, __pyx_k_agm, sizeof(__pyx_k_agm), 0, 0, 1, 1},
  {&__pyx_n_s_airy, __pyx_k_airy, sizeof(__pyx_k_airy), 0, 0, 1, 1},
  {&__pyx_n_s_airye, __pyx_k_airye, sizeof(__pyx_k_airye), 0, 0, 1, 1},
  {&__pyx_n_u_all, __pyx_k_all, sizeof(__pyx_k_all), 0, 1, 0, 1},
  {&__pyx_n_u_arg, __pyx_k_arg, sizeof(__pyx_k_arg), 0, 1, 0, 1},
  {&__pyx_n_s_bdtr, __pyx_k_bdtr, sizeof(__pyx_k_bdtr), 0, 0, 1, 1},
  {&__pyx_n_s_bdtrc, __pyx_k_bdtrc, sizeof(__pyx_k_bdtrc), 0, 0, 1, 1},
  {&__pyx_n_s_bdtri, __pyx_k_bdtri, sizeof(__pyx_k_bdtri), 0, 0, 1, 1},
  {&__pyx_n_s_bdtrik, __pyx_k_bdtrik, sizeof(__pyx_k_bdtrik), 0, 0, 1, 1},
  {&__pyx_n_s_bdtrin, __pyx_k_bdtrin, sizeof(__pyx_k_bdtrin), 0, 0, 1, 1},
  {&__pyx_n_s_bei, __pyx_k_bei, sizeof(__pyx_k_bei), 0, 0, 1, 1},
  {&__pyx_n_s_beip, __pyx_k_beip, sizeof(__pyx_k_beip), 0, 0, 1, 1},
  {&__pyx_n_s_ber, __pyx_k_ber, sizeof(__pyx_k_ber), 0, 0, 1, 1},
  {&__pyx_n_s_berp, __pyx_k_berp, sizeof(__pyx_k_berp), 0, 0, 1, 1},
  {&__pyx_n_s_besselpoly, __pyx_k_besselpoly, sizeof(__pyx_k_besselpoly), 0, 0, 1, 1},
  {&__pyx_n_s_beta, __pyx_k_beta, sizeof(__pyx_k_beta), 0, 0, 1, 1},
  {&__pyx_n_s_betainc, __pyx_k_betainc, sizeof(__pyx_k_betainc), 0, 0, 1, 1},
  {&__pyx_n_s_betaincinv, __pyx_k_betaincinv, sizeof(__pyx_k_betaincinv), 0, 0, 1, 1},
  {&__pyx_n_s_betaln, __pyx_k_betaln, sizeof(__pyx_k_betaln), 0, 0, 1, 1},
  {&__pyx_n_s_binom, __pyx_k_binom, sizeof(__pyx_k_binom), 0, 0, 1, 1},
  {&__pyx_n_s_boxcox, __pyx_k_boxcox, sizeof(__pyx_k_boxcox), 0, 0, 1, 1},
  {&__pyx_n_s_boxcox1p, __pyx_k_boxcox1p, sizeof(__pyx_k_boxcox1p), 0, 0, 1, 1},
  {&__pyx_n_s_btdtr, __pyx_k_btdtr, sizeof(__pyx_k_btdtr), 0, 0, 1, 1},
  {&__pyx_n_s_btdtri, __pyx_k_btdtri, sizeof(__pyx_k_btdtri), 0, 0, 1, 1},
  {&__pyx_n_s_btdtria, __pyx_k_btdtria, sizeof(__pyx_k_btdtria), 0, 0, 1, 1},
  {&__pyx_n_s_btdtrib, __pyx_k_btdtrib, sizeof(__pyx_k_btdtrib), 0, 0, 1, 1},
  {&__pyx_n_s_cbrt, __pyx_k_cbrt, sizeof(__pyx_k_cbrt), 0, 0, 1, 1},
  {&__pyx_n_s_chdtr, __pyx_k_chdtr, sizeof(__pyx_k_chdtr), 0, 0, 1, 1},
  {&__pyx_n_s_chdtrc, __pyx_k_chdtrc, sizeof(__pyx_k_chdtrc), 0, 0, 1, 1},
  {&__pyx_n_s_chdtri, __pyx_k_chdtri, sizeof(__pyx_k_chdtri), 0, 0, 1, 1},
  {&__pyx_n_s_chdtriv, __pyx_k_chdtriv, sizeof(__pyx_k_chdtriv), 0, 0, 1, 1},
  {&__pyx_n_s_chndtr, __pyx_k_chndtr, sizeof(__pyx_k_chndtr), 0, 0, 1, 1},
  {&__pyx_n_s_chndtridf, __pyx_k_chndtridf, sizeof(__pyx_k_chndtridf), 0, 0, 1, 1},
  {&__pyx_n_s_chndtrinc, __pyx_k_chndtrinc, sizeof(__pyx_k_chndtrinc), 0, 0, 1, 1},
  {&__pyx_n_s_chndtrix, __pyx_k_chndtrix, sizeof(__pyx_k_chndtrix), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_code, __pyx_k_code, sizeof(__pyx_k_code), 0, 0, 1, 1},
  {&__pyx_n_s_cosdg, __pyx_k_cosdg, sizeof(__pyx_k_cosdg), 0, 0, 1, 1},
  {&__pyx_n_s_cosm1, __pyx_k_cosm1, sizeof(__pyx_k_cosm1), 0, 0, 1, 1},
  {&__pyx_n_s_cospi, __pyx_k_cospi, sizeof(__pyx_k_cospi), 0, 0, 1, 1},
  {&__pyx_n_s_cotdg, __pyx_k_cotdg, sizeof(__pyx_k_cotdg), 0, 0, 1, 1},
  {&__pyx_n_s_dawsn, __pyx_k_dawsn, sizeof(__pyx_k_dawsn), 0, 0, 1, 1},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_u_domain, __pyx_k_domain, sizeof(__pyx_k_domain), 0, 1, 0, 1},
  {&__pyx_n_s_ellip_harm, __pyx_k_ellip_harm, sizeof(__pyx_k_ellip_harm), 0, 0, 1, 1},
  {&__pyx_n_s_ellipe, __pyx_k_ellipe, sizeof(__pyx_k_ellipe), 0, 0, 1, 1},
  {&__pyx_n_s_ellipeinc, __pyx_k_ellipeinc, sizeof(__pyx_k_ellipeinc), 0, 0, 1, 1},
  {&__pyx_n_s_ellipj, __pyx_k_ellipj, sizeof(__pyx_k_ellipj), 0, 0, 1, 1},
  {&__pyx_n_s_ellipkinc, __pyx_k_ellipkinc, sizeof(__pyx_k_ellipkinc), 0, 0, 1, 1},
  {&__pyx_n_s_ellipkm1, __pyx_k_ellipkm1, sizeof(__pyx_k_ellipkm1), 0, 0, 1, 1},
  {&__pyx_n_s_enter, __pyx_k_enter, sizeof(__pyx_k_enter), 0, 0, 1, 1},
  {&__pyx_n_s_entr, __pyx_k_entr, sizeof(__pyx_k_entr), 0, 0, 1, 1},
  {&__pyx_n_s_erf, __pyx_k_erf, sizeof(__pyx_k_erf), 0, 0, 1, 1},
  {&__pyx_n_s_erfc, __pyx_k_erfc, sizeof(__pyx_k_erfc), 0, 0, 1, 1},
  {&__pyx_n_s_erfcx, __pyx_k_erfcx, sizeof(__pyx_k_erfcx), 0, 0, 1, 1},
  {&__pyx_n_s_erfi, __pyx_k_erfi, sizeof(__pyx_k_erfi), 0, 0, 1, 1},
  {&__pyx_n_s_err, __pyx_k_err, sizeof(__pyx_k_err), 0, 0, 1, 1},
  {&__pyx_n_s_error, __pyx_k_error, sizeof(__pyx_k_error), 0, 0, 1, 1},
  {&__pyx_n_s_errstate, __pyx_k_errstate, sizeof(__pyx_k_errstate), 0, 0, 1, 1},
  {&__pyx_n_s_errstate___enter, __pyx_k_errstate___enter, sizeof(__pyx_k_errstate___enter), 0, 0, 1, 1},
  {&__pyx_n_s_errstate___exit, __pyx_k_errstate___exit, sizeof(__pyx_k_errstate___exit), 0, 0, 1, 1},
  {&__pyx_n_s_errstate___init, __pyx_k_errstate___init, sizeof(__pyx_k_errstate___init), 0, 0, 1, 1},
  {&__pyx_n_s_eval_chebyc, __pyx_k_eval_chebyc, sizeof(__pyx_k_eval_chebyc), 0, 0, 1, 1},
  {&__pyx_n_s_eval_chebys, __pyx_k_eval_chebys, sizeof(__pyx_k_eval_chebys), 0, 0, 1, 1},
  {&__pyx_n_s_eval_chebyt, __pyx_k_eval_chebyt, sizeof(__pyx_k_eval_chebyt), 0, 0, 1, 1},
  {&__pyx_n_s_eval_chebyu, __pyx_k_eval_chebyu, sizeof(__pyx_k_eval_chebyu), 0, 0, 1, 1},
  {&__pyx_n_s_eval_gegenbauer, __pyx_k_eval_gegenbauer, sizeof(__pyx_k_eval_gegenbauer), 0, 0, 1, 1},
  {&__pyx_n_s_eval_genlaguerre, __pyx_k_eval_genlaguerre, sizeof(__pyx_k_eval_genlaguerre), 0, 0, 1, 1},
  {&__pyx_n_s_eval_hermite, __pyx_k_eval_hermite, sizeof(__pyx_k_eval_hermite), 0, 0, 1, 1},
  {&__pyx_n_s_eval_hermitenorm, __pyx_k_eval_hermitenorm, sizeof(__pyx_k_eval_hermitenorm), 0, 0, 1, 1},
  {&__pyx_n_s_eval_jacobi, __pyx_k_eval_jacobi, sizeof(__pyx_k_eval_jacobi), 0, 0, 1, 1},
  {&__pyx_n_s_eval_laguerre, __pyx_k_eval_laguerre, sizeof(__pyx_k_eval_laguerre), 0, 0, 1, 1},
  {&__pyx_n_s_eval_legendre, __pyx_k_eval_legendre, sizeof(__pyx_k_eval_legendre), 0, 0, 1, 1},
  {&__pyx_n_s_eval_sh_chebyt, __pyx_k_eval_sh_chebyt, sizeof(__pyx_k_eval_sh_chebyt), 0, 0, 1, 1},
  {&__pyx_n_s_eval_sh_chebyu, __pyx_k_eval_sh_chebyu, sizeof(__pyx_k_eval_sh_chebyu), 0, 0, 1, 1},
  {&__pyx_n_s_eval_sh_jacobi, __pyx_k_eval_sh_jacobi, sizeof(__pyx_k_eval_sh_jacobi), 0, 0, 1, 1},
  {&__pyx_n_s_eval_sh_legendre, __pyx_k_eval_sh_legendre, sizeof(__pyx_k_eval_sh_legendre), 0, 0, 1, 1},
  {&__pyx_n_s_exc_type, __pyx_k_exc_type, sizeof(__pyx_k_exc_type), 0, 0, 1, 1},
  {&__pyx_n_s_exc_value, __pyx_k_exc_value, sizeof(__pyx_k_exc_value), 0, 0, 1, 1},
  {&__pyx_n_s_exit, __pyx_k_exit, sizeof(__pyx_k_exit), 0, 0, 1, 1},
  {&__pyx_n_s_exp1, __pyx_k_exp1, sizeof(__pyx_k_exp1), 0, 0, 1, 1},
  {&__pyx_n_s_exp10, __pyx_k_exp10, sizeof(__pyx_k_exp10), 0, 0, 1, 1},
  {&__pyx_n_s_exp2, __pyx_k_exp2, sizeof(__pyx_k_exp2), 0, 0, 1, 1},
  {&__pyx_n_s_expi, __pyx_k_expi, sizeof(__pyx_k_expi), 0, 0, 1, 1},
  {&__pyx_n_s_expit, __pyx_k_expit, sizeof(__pyx_k_expit), 0, 0, 1, 1},
  {&__pyx_n_s_expm1, __pyx_k_expm1, sizeof(__pyx_k_expm1), 0, 0, 1, 1},
  {&__pyx_n_s_expn, __pyx_k_expn, sizeof(__pyx_k_expn), 0, 0, 1, 1},
  {&__pyx_n_s_exprel, __pyx_k_exprel, sizeof(__pyx_k_exprel), 0, 0, 1, 1},
  {&__pyx_n_s_fdtr, __pyx_k_fdtr, sizeof(__pyx_k_fdtr), 0, 0, 1, 1},
  {&__pyx_n_s_fdtrc, __pyx_k_fdtrc, sizeof(__pyx_k_fdtrc), 0, 0, 1, 1},
  {&__pyx_n_s_fdtri, __pyx_k_fdtri, sizeof(__pyx_k_fdtri), 0, 0, 1, 1},
  {&__pyx_n_s_fdtridfd, __pyx_k_fdtridfd, sizeof(__pyx_k_fdtridfd), 0, 0, 1, 1},
  {&__pyx_n_s_fresnel, __pyx_k_fresnel, sizeof(__pyx_k_fresnel), 0, 0, 1, 1},
  {&__pyx_n_s_gamma, __pyx_k_gamma, sizeof(__pyx_k_gamma), 0, 0, 1, 1},
  {&__pyx_n_s_gammainc, __pyx_k_gammainc, sizeof(__pyx_k_gammainc), 0, 0, 1, 1},
  {&__pyx_n_s_gammaincc, __pyx_k_gammaincc, sizeof(__pyx_k_gammaincc), 0, 0, 1, 1},
  {&__pyx_n_s_gammainccinv, __pyx_k_gammainccinv, sizeof(__pyx_k_gammainccinv), 0, 0, 1, 1},
  {&__pyx_n_s_gammaincinv, __pyx_k_gammaincinv, sizeof(__pyx_k_gammaincinv), 0, 0, 1, 1},
  {&__pyx_n_s_gammaln, __pyx_k_gammaln, sizeof(__pyx_k_gammaln), 0, 0, 1, 1},
  {&__pyx_n_s_gammasgn, __pyx_k_gammasgn, sizeof(__pyx_k_gammasgn), 0, 0, 1, 1},
  {&__pyx_n_s_gdtr, __pyx_k_gdtr, sizeof(__pyx_k_gdtr), 0, 0, 1, 1},
  {&__pyx_n_s_gdtrc, __pyx_k_gdtrc, sizeof(__pyx_k_gdtrc), 0, 0, 1, 1},
  {&__pyx_n_s_gdtria, __pyx_k_gdtria, sizeof(__pyx_k_gdtria), 0, 0, 1, 1},
  {&__pyx_n_s_gdtrib, __pyx_k_gdtrib, sizeof(__pyx_k_gdtrib), 0, 0, 1, 1},
  {&__pyx_n_s_gdtrix, __pyx_k_gdtrix, sizeof(__pyx_k_gdtrix), 0, 0, 1, 1},
  {&__pyx_n_s_geterr, __pyx_k_geterr, sizeof(__pyx_k_geterr), 0, 0, 1, 1},
  {&__pyx_kp_u_geterr_line_28, __pyx_k_geterr_line_28, sizeof(__pyx_k_geterr_line_28), 0, 1, 0, 0},
  {&__pyx_n_s_hankel1, __pyx_k_hankel1, sizeof(__pyx_k_hankel1), 0, 0, 1, 1},
  {&__pyx_n_s_hankel1e, __pyx_k_hankel1e, sizeof(__pyx_k_hankel1e), 0, 0, 1, 1},
  {&__pyx_n_s_hankel2, __pyx_k_hankel2, sizeof(__pyx_k_hankel2), 0, 0, 1, 1},
  {&__pyx_n_s_hankel2e, __pyx_k_hankel2e, sizeof(__pyx_k_hankel2e), 0, 0, 1, 1},
  {&__pyx_n_s_huber, __pyx_k_huber, sizeof(__pyx_k_huber), 0, 0, 1, 1},
  {&__pyx_n_s_hyp0f1, __pyx_k_hyp0f1, sizeof(__pyx_k_hyp0f1), 0, 0, 1, 1},
  {&__pyx_n_s_hyp1f1, __pyx_k_hyp1f1, sizeof(__pyx_k_hyp1f1), 0, 0, 1, 1},
  {&__pyx_n_s_hyp1f2, __pyx_k_hyp1f2, sizeof(__pyx_k_hyp1f2), 0, 0, 1, 1},
  {&__pyx_n_s_hyp2f0, __pyx_k_hyp2f0, sizeof(__pyx_k_hyp2f0), 0, 0, 1, 1},
  {&__pyx_n_s_hyp2f1, __pyx_k_hyp2f1, sizeof(__pyx_k_hyp2f1), 0, 0, 1, 1},
  {&__pyx_n_s_hyp3f0, __pyx_k_hyp3f0, sizeof(__pyx_k_hyp3f0), 0, 0, 1, 1},
  {&__pyx_n_s_hyperu, __pyx_k_hyperu, sizeof(__pyx_k_hyperu), 0, 0, 1, 1},
  {&__pyx_n_s_i0, __pyx_k_i0, sizeof(__pyx_k_i0), 0, 0, 1, 1},
  {&__pyx_n_s_i0e, __pyx_k_i0e, sizeof(__pyx_k_i0e), 0, 0, 1, 1},
  {&__pyx_n_s_i1, __pyx_k_i1, sizeof(__pyx_k_i1), 0, 0, 1, 1},
  {&__pyx_n_s_i1e, __pyx_k_i1e, sizeof(__pyx_k_i1e), 0, 0, 1, 1},
  {&__pyx_n_s_igam_fac, __pyx_k_igam_fac, sizeof(__pyx_k_igam_fac), 0, 0, 1, 1},
  {&__pyx_n_u_ignore, __pyx_k_ignore, sizeof(__pyx_k_ignore), 0, 1, 0, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_inv_boxcox, __pyx_k_inv_boxcox, sizeof(__pyx_k_inv_boxcox), 0, 0, 1, 1},
  {&__pyx_n_s_inv_boxcox1p, __pyx_k_inv_boxcox1p, sizeof(__pyx_k_inv_boxcox1p), 0, 0, 1, 1},
  {&__pyx_n_s_it2i0k0, __pyx_k_it2i0k0, sizeof(__pyx_k_it2i0k0), 0, 0, 1, 1},
  {&__pyx_n_s_it2j0y0, __pyx_k_it2j0y0, sizeof(__pyx_k_it2j0y0), 0, 0, 1, 1},
  {&__pyx_n_s_it2struve0, __pyx_k_it2struve0, sizeof(__pyx_k_it2struve0), 0, 0, 1, 1},
  {&__pyx_n_s_itairy, __pyx_k_itairy, sizeof(__pyx_k_itairy), 0, 0, 1, 1},
  {&__pyx_n_s_items, __pyx_k_items, sizeof(__pyx_k_items), 0, 0, 1, 1},
  {&__pyx_n_s_iti0k0, __pyx_k_iti0k0, sizeof(__pyx_k_iti0k0), 0, 0, 1, 1},
  {&__pyx_n_s_itj0y0, __pyx_k_itj0y0, sizeof(__pyx_k_itj0y0), 0, 0, 1, 1},
  {&__pyx_n_s_itmodstruve0, __pyx_k_itmodstruve0, sizeof(__pyx_k_itmodstruve0), 0, 0, 1, 1},
  {&__pyx_n_s_itstruve0, __pyx_k_itstruve0, sizeof(__pyx_k_itstruve0), 0, 0, 1, 1},
  {&__pyx_n_s_iv, __pyx_k_iv, sizeof(__pyx_k_iv), 0, 0, 1, 1},
  {&__pyx_n_s_ive, __pyx_k_ive, sizeof(__pyx_k_ive), 0, 0, 1, 1},
  {&__pyx_n_s_j0, __pyx_k_j0, sizeof(__pyx_k_j0), 0, 0, 1, 1},
  {&__pyx_n_s_j1, __pyx_k_j1, sizeof(__pyx_k_j1), 0, 0, 1, 1},
  {&__pyx_n_s_jn, __pyx_k_jn, sizeof(__pyx_k_jn), 0, 0, 1, 1},
  {&__pyx_n_s_jv, __pyx_k_jv, sizeof(__pyx_k_jv), 0, 0, 1, 1},
  {&__pyx_n_s_jve, __pyx_k_jve, sizeof(__pyx_k_jve), 0, 0, 1, 1},
  {&__pyx_n_s_k0, __pyx_k_k0, sizeof(__pyx_k_k0), 0, 0, 1, 1},
  {&__pyx_n_s_k0e, __pyx_k_k0e, sizeof(__pyx_k_k0e), 0, 0, 1, 1},
  {&__pyx_n_s_k1, __pyx_k_k1, sizeof(__pyx_k_k1), 0, 0, 1, 1},
  {&__pyx_n_s_k1e, __pyx_k_k1e, sizeof(__pyx_k_k1e), 0, 0, 1, 1},
  {&__pyx_n_s_kei, __pyx_k_kei, sizeof(__pyx_k_kei), 0, 0, 1, 1},
  {&__pyx_n_s_keip, __pyx_k_keip, sizeof(__pyx_k_keip), 0, 0, 1, 1},
  {&__pyx_n_s_kelvin, __pyx_k_kelvin, sizeof(__pyx_k_kelvin), 0, 0, 1, 1},
  {&__pyx_n_s_ker, __pyx_k_ker, sizeof(__pyx_k_ker), 0, 0, 1, 1},
  {&__pyx_n_s_kerp, __pyx_k_kerp, sizeof(__pyx_k_kerp), 0, 0, 1, 1},
  {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
  {&__pyx_n_s_keys, __pyx_k_keys, sizeof(__pyx_k_keys), 0, 0, 1, 1},
  {&__pyx_n_s_kl_div, __pyx_k_kl_div, sizeof(__pyx_k_kl_div), 0, 0, 1, 1},
  {&__pyx_n_s_kn, __pyx_k_kn, sizeof(__pyx_k_kn), 0, 0, 1, 1},
  {&__pyx_n_s_kolmogc, __pyx_k_kolmogc, sizeof(__pyx_k_kolmogc), 0, 0, 1, 1},
  {&__pyx_n_s_kolmogci, __pyx_k_kolmogci, sizeof(__pyx_k_kolmogci), 0, 0, 1, 1},
  {&__pyx_n_s_kolmogi, __pyx_k_kolmogi, sizeof(__pyx_k_kolmogi), 0, 0, 1, 1},
  {&__pyx_n_s_kolmogorov, __pyx_k_kolmogorov, sizeof(__pyx_k_kolmogorov), 0, 0, 1, 1},
  {&__pyx_n_s_kolmogp, __pyx_k_kolmogp, sizeof(__pyx_k_kolmogp), 0, 0, 1, 1},
  {&__pyx_n_s_kv, __pyx_k_kv, sizeof(__pyx_k_kv), 0, 0, 1, 1},
  {&__pyx_n_s_kve, __pyx_k_kve, sizeof(__pyx_k_kve), 0, 0, 1, 1},
  {&__pyx_n_s_kwargs, __pyx_k_kwargs, sizeof(__pyx_k_kwargs), 0, 0, 1, 1},
  {&__pyx_n_s_lambertw, __pyx_k_lambertw, sizeof(__pyx_k_lambertw), 0, 0, 1, 1},
  {&__pyx_n_s_lanczos_sum_expg_scaled, __pyx_k_lanczos_sum_expg_scaled, sizeof(__pyx_k_lanczos_sum_expg_scaled), 0, 0, 1, 1},
  {&__pyx_n_s_lgam1p, __pyx_k_lgam1p, sizeof(__pyx_k_lgam1p), 0, 0, 1, 1},
  {&__pyx_n_s_log1p, __pyx_k_log1p, sizeof(__pyx_k_log1p), 0, 0, 1, 1},
  {&__pyx_n_s_log1pmx, __pyx_k_log1pmx, sizeof(__pyx_k_log1pmx), 0, 0, 1, 1},
  {&__pyx_n_s_log_ndtr, __pyx_k_log_ndtr, sizeof(__pyx_k_log_ndtr), 0, 0, 1, 1},
  {&__pyx_n_s_loggamma, __pyx_k_loggamma, sizeof(__pyx_k_loggamma), 0, 0, 1, 1},
  {&__pyx_n_s_logit, __pyx_k_logit, sizeof(__pyx_k_logit), 0, 0, 1, 1},
  {&__pyx_n_u_loss, __pyx_k_loss, sizeof(__pyx_k_loss), 0, 1, 0, 1},
  {&__pyx_n_s_lpmv, __pyx_k_lpmv, sizeof(__pyx_k_lpmv), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_mathieu_a, __pyx_k_mathieu_a, sizeof(__pyx_k_mathieu_a), 0, 0, 1, 1},
  {&__pyx_n_s_mathieu_b, __pyx_k_mathieu_b, sizeof(__pyx_k_mathieu_b), 0, 0, 1, 1},
  {&__pyx_n_s_mathieu_cem, __pyx_k_mathieu_cem, sizeof(__pyx_k_mathieu_cem), 0, 0, 1, 1},
  {&__pyx_n_s_mathieu_modcem1, __pyx_k_mathieu_modcem1, sizeof(__pyx_k_mathieu_modcem1), 0, 0, 1, 1},
  {&__pyx_n_s_mathieu_modcem2, __pyx_k_mathieu_modcem2, sizeof(__pyx_k_mathieu_modcem2), 0, 0, 1, 1},
  {&__pyx_n_s_mathieu_modsem1, __pyx_k_mathieu_modsem1, sizeof(__pyx_k_mathieu_modsem1), 0, 0, 1, 1},
  {&__pyx_n_s_mathieu_modsem2, __pyx_k_mathieu_modsem2, sizeof(__pyx_k_mathieu_modsem2), 0, 0, 1, 1},
  {&__pyx_n_s_mathieu_sem, __pyx_k_mathieu_sem, sizeof(__pyx_k_mathieu_sem), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_modfresnelm, __pyx_k_modfresnelm, sizeof(__pyx_k_modfresnelm), 0, 0, 1, 1},
  {&__pyx_n_s_modfresnelp, __pyx_k_modfresnelp, sizeof(__pyx_k_modfresnelp), 0, 0, 1, 1},
  {&__pyx_n_s_modstruve, __pyx_k_modstruve, sizeof(__pyx_k_modstruve), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_nbdtr, __pyx_k_nbdtr, sizeof(__pyx_k_nbdtr), 0, 0, 1, 1},
  {&__pyx_n_s_nbdtrc, __pyx_k_nbdtrc, sizeof(__pyx_k_nbdtrc), 0, 0, 1, 1},
  {&__pyx_n_s_nbdtri, __pyx_k_nbdtri, sizeof(__pyx_k_nbdtri), 0, 0, 1, 1},
  {&__pyx_n_s_nbdtrik, __pyx_k_nbdtrik, sizeof(__pyx_k_nbdtrik), 0, 0, 1, 1},
  {&__pyx_n_s_nbdtrin, __pyx_k_nbdtrin, sizeof(__pyx_k_nbdtrin), 0, 0, 1, 1},
  {&__pyx_n_s_ncfdtr, __pyx_k_ncfdtr, sizeof(__pyx_k_ncfdtr), 0, 0, 1, 1},
  {&__pyx_n_s_ncfdtri, __pyx_k_ncfdtri, sizeof(__pyx_k_ncfdtri), 0, 0, 1, 1},
  {&__pyx_n_s_ncfdtridfd, __pyx_k_ncfdtridfd, sizeof(__pyx_k_ncfdtridfd), 0, 0, 1, 1},
  {&__pyx_n_s_ncfdtridfn, __pyx_k_ncfdtridfn, sizeof(__pyx_k_ncfdtridfn), 0, 0, 1, 1},
  {&__pyx_n_s_ncfdtrinc, __pyx_k_ncfdtrinc, sizeof(__pyx_k_ncfdtrinc), 0, 0, 1, 1},
  {&__pyx_n_s_nctdtr, __pyx_k_nctdtr, sizeof(__pyx_k_nctdtr), 0, 0, 1, 1},
  {&__pyx_n_s_nctdtridf, __pyx_k_nctdtridf, sizeof(__pyx_k_nctdtridf), 0, 0, 1, 1},
  {&__pyx_n_s_nctdtrinc, __pyx_k_nctdtrinc, sizeof(__pyx_k_nctdtrinc), 0, 0, 1, 1},
  {&__pyx_n_s_nctdtrit, __pyx_k_nctdtrit, sizeof(__pyx_k_nctdtrit), 0, 0, 1, 1},
  {&__pyx_kp_u_ndarray_is_not_C_contiguous, __pyx_k_ndarray_is_not_C_contiguous, sizeof(__pyx_k_ndarray_is_not_C_contiguous), 0, 1, 0, 0},
  {&__pyx_kp_u_ndarray_is_not_Fortran_contiguou, __pyx_k_ndarray_is_not_Fortran_contiguou, sizeof(__pyx_k_ndarray_is_not_Fortran_contiguou), 0, 1, 0, 0},
  {&__pyx_n_s_ndtr, __pyx_k_ndtr, sizeof(__pyx_k_ndtr), 0, 0, 1, 1},
  {&__pyx_n_s_ndtri, __pyx_k_ndtri, sizeof(__pyx_k_ndtri), 0, 0, 1, 1},
  {&__pyx_n_s_newkwargs, __pyx_k_newkwargs, sizeof(__pyx_k_newkwargs), 0, 0, 1, 1},
  {&__pyx_n_u_no_result, __pyx_k_no_result, sizeof(__pyx_k_no_result), 0, 1, 0, 1},
  {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
  {&__pyx_n_s_nrdtrimn, __pyx_k_nrdtrimn, sizeof(__pyx_k_nrdtrimn), 0, 0, 1, 1},
  {&__pyx_n_s_nrdtrisd, __pyx_k_nrdtrisd, sizeof(__pyx_k_nrdtrisd), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_kp_u_numpy_core_multiarray_failed_to, __pyx_k_numpy_core_multiarray_failed_to, sizeof(__pyx_k_numpy_core_multiarray_failed_to), 0, 1, 0, 0},
  {&__pyx_kp_u_numpy_core_umath_failed_to_impor, __pyx_k_numpy_core_umath_failed_to_impor, sizeof(__pyx_k_numpy_core_umath_failed_to_impor), 0, 1, 0, 0},
  {&__pyx_n_s_object, __pyx_k_object, sizeof(__pyx_k_object), 0, 0, 1, 1},
  {&__pyx_n_s_obl_ang1, __pyx_k_obl_ang1, sizeof(__pyx_k_obl_ang1), 0, 0, 1, 1},
  {&__pyx_n_s_obl_ang1_cv, __pyx_k_obl_ang1_cv, sizeof(__pyx_k_obl_ang1_cv), 0, 0, 1, 1},
  {&__pyx_n_s_obl_cv, __pyx_k_obl_cv, sizeof(__pyx_k_obl_cv), 0, 0, 1, 1},
  {&__pyx_n_s_obl_rad1, __pyx_k_obl_rad1, sizeof(__pyx_k_obl_rad1), 0, 0, 1, 1},
  {&__pyx_n_s_obl_rad1_cv, __pyx_k_obl_rad1_cv, sizeof(__pyx_k_obl_rad1_cv), 0, 0, 1, 1},
  {&__pyx_n_s_obl_rad2, __pyx_k_obl_rad2, sizeof(__pyx_k_obl_rad2), 0, 0, 1, 1},
  {&__pyx_n_s_obl_rad2_cv, __pyx_k_obl_rad2_cv, sizeof(__pyx_k_obl_rad2_cv), 0, 0, 1, 1},
  {&__pyx_n_s_olderr, __pyx_k_olderr, sizeof(__pyx_k_olderr), 0, 0, 1, 1},
  {&__pyx_n_s_oldstate, __pyx_k_oldstate, sizeof(__pyx_k_oldstate), 0, 0, 1, 1},
  {&__pyx_n_u_other, __pyx_k_other, sizeof(__pyx_k_other), 0, 1, 0, 1},
  {&__pyx_n_u_overflow, __pyx_k_overflow, sizeof(__pyx_k_overflow), 0, 1, 0, 1},
  {&__pyx_n_s_owens_t, __pyx_k_owens_t, sizeof(__pyx_k_owens_t), 0, 0, 1, 1},
  {&__pyx_n_s_pbdv, __pyx_k_pbdv, sizeof(__pyx_k_pbdv), 0, 0, 1, 1},
  {&__pyx_n_s_pbvv, __pyx_k_pbvv, sizeof(__pyx_k_pbvv), 0, 0, 1, 1},
  {&__pyx_n_s_pbwa, __pyx_k_pbwa, sizeof(__pyx_k_pbwa), 0, 0, 1, 1},
  {&__pyx_n_s_pdtr, __pyx_k_pdtr, sizeof(__pyx_k_pdtr), 0, 0, 1, 1},
  {&__pyx_n_s_pdtrc, __pyx_k_pdtrc, sizeof(__pyx_k_pdtrc), 0, 0, 1, 1},
  {&__pyx_n_s_pdtri, __pyx_k_pdtri, sizeof(__pyx_k_pdtri), 0, 0, 1, 1},
  {&__pyx_n_s_pdtrik, __pyx_k_pdtrik, sizeof(__pyx_k_pdtrik), 0, 0, 1, 1},
  {&__pyx_n_s_poch, __pyx_k_poch, sizeof(__pyx_k_poch), 0, 0, 1, 1},
  {&__pyx_n_s_pop, __pyx_k_pop, sizeof(__pyx_k_pop), 0, 0, 1, 1},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_n_s_pro_ang1, __pyx_k_pro_ang1, sizeof(__pyx_k_pro_ang1), 0, 0, 1, 1},
  {&__pyx_n_s_pro_ang1_cv, __pyx_k_pro_ang1_cv, sizeof(__pyx_k_pro_ang1_cv), 0, 0, 1, 1},
  {&__pyx_n_s_pro_cv, __pyx_k_pro_cv, sizeof(__pyx_k_pro_cv), 0, 0, 1, 1},
  {&__pyx_n_s_pro_rad1, __pyx_k_pro_rad1, sizeof(__pyx_k_pro_rad1), 0, 0, 1, 1},
  {&__pyx_n_s_pro_rad1_cv, __pyx_k_pro_rad1_cv, sizeof(__pyx_k_pro_rad1_cv), 0, 0, 1, 1},
  {&__pyx_n_s_pro_rad2, __pyx_k_pro_rad2, sizeof(__pyx_k_pro_rad2), 0, 0, 1, 1},
  {&__pyx_n_s_pro_rad2_cv, __pyx_k_pro_rad2_cv, sizeof(__pyx_k_pro_rad2_cv), 0, 0, 1, 1},
  {&__pyx_n_s_pseudo_huber, __pyx_k_pseudo_huber, sizeof(__pyx_k_pseudo_huber), 0, 0, 1, 1},
  {&__pyx_n_s_psi, __pyx_k_psi, sizeof(__pyx_k_psi), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_n_s_radian, __pyx_k_radian, sizeof(__pyx_k_radian), 0, 0, 1, 1},
  {&__pyx_n_u_raise, __pyx_k_raise, sizeof(__pyx_k_raise), 0, 1, 0, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_rel_entr, __pyx_k_rel_entr, sizeof(__pyx_k_rel_entr), 0, 0, 1, 1},
  {&__pyx_n_s_rgamma, __pyx_k_rgamma, sizeof(__pyx_k_rgamma), 0, 0, 1, 1},
  {&__pyx_n_s_round, __pyx_k_round, sizeof(__pyx_k_round), 0, 0, 1, 1},
  {&__pyx_n_s_scipy_special__ufuncs, __pyx_k_scipy_special__ufuncs, sizeof(__pyx_k_scipy_special__ufuncs), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_n_s_seterr, __pyx_k_seterr, sizeof(__pyx_k_seterr), 0, 0, 1, 1},
  {&__pyx_kp_u_seterr_line_77, __pyx_k_seterr_line_77, sizeof(__pyx_k_seterr_line_77), 0, 1, 0, 0},
  {&__pyx_n_s_sf_error_action_map, __pyx_k_sf_error_action_map, sizeof(__pyx_k_sf_error_action_map), 0, 0, 1, 1},
  {&__pyx_n_s_sf_error_code_map, __pyx_k_sf_error_code_map, sizeof(__pyx_k_sf_error_code_map), 0, 0, 1, 1},
  {&__pyx_n_s_sf_error_test_function, __pyx_k_sf_error_test_function, sizeof(__pyx_k_sf_error_test_function), 0, 0, 1, 1},
  {&__pyx_n_s_shichi, __pyx_k_shichi, sizeof(__pyx_k_shichi), 0, 0, 1, 1},
  {&__pyx_n_s_sici, __pyx_k_sici, sizeof(__pyx_k_sici), 0, 0, 1, 1},
  {&__pyx_n_s_sindg, __pyx_k_sindg, sizeof(__pyx_k_sindg), 0, 0, 1, 1},
  {&__pyx_n_u_singular, __pyx_k_singular, sizeof(__pyx_k_singular), 0, 1, 0, 1},
  {&__pyx_n_s_sinpi, __pyx_k_sinpi, sizeof(__pyx_k_sinpi), 0, 0, 1, 1},
  {&__pyx_n_u_slow, __pyx_k_slow, sizeof(__pyx_k_slow), 0, 1, 0, 1},
  {&__pyx_n_s_smirnov, __pyx_k_smirnov, sizeof(__pyx_k_smirnov), 0, 0, 1, 1},
  {&__pyx_n_s_smirnovc, __pyx_k_smirnovc, sizeof(__pyx_k_smirnovc), 0, 0, 1, 1},
  {&__pyx_n_s_smirnovci, __pyx_k_smirnovci, sizeof(__pyx_k_smirnovci), 0, 0, 1, 1},
  {&__pyx_n_s_smirnovi, __pyx_k_smirnovi, sizeof(__pyx_k_smirnovi), 0, 0, 1, 1},
  {&__pyx_n_s_smirnovp, __pyx_k_smirnovp, sizeof(__pyx_k_smirnovp), 0, 0, 1, 1},
  {&__pyx_n_s_spence, __pyx_k_spence, sizeof(__pyx_k_spence), 0, 0, 1, 1},
  {&__pyx_n_s_sph_harm, __pyx_k_sph_harm, sizeof(__pyx_k_sph_harm), 0, 0, 1, 1},
  {&__pyx_n_s_spherical_in, __pyx_k_spherical_in, sizeof(__pyx_k_spherical_in), 0, 0, 1, 1},
  {&__pyx_n_s_spherical_in_d, __pyx_k_spherical_in_d, sizeof(__pyx_k_spherical_in_d), 0, 0, 1, 1},
  {&__pyx_n_s_spherical_jn, __pyx_k_spherical_jn, sizeof(__pyx_k_spherical_jn), 0, 0, 1, 1},
  {&__pyx_n_s_spherical_jn_d, __pyx_k_spherical_jn_d, sizeof(__pyx_k_spherical_jn_d), 0, 0, 1, 1},
  {&__pyx_n_s_spherical_kn, __pyx_k_spherical_kn, sizeof(__pyx_k_spherical_kn), 0, 0, 1, 1},
  {&__pyx_n_s_spherical_kn_d, __pyx_k_spherical_kn_d, sizeof(__pyx_k_spherical_kn_d), 0, 0, 1, 1},
  {&__pyx_n_s_spherical_yn, __pyx_k_spherical_yn, sizeof(__pyx_k_spherical_yn), 0, 0, 1, 1},
  {&__pyx_n_s_spherical_yn_d, __pyx_k_spherical_yn_d, sizeof(__pyx_k_spherical_yn_d), 0, 0, 1, 1},
  {&__pyx_n_s_stdtr, __pyx_k_stdtr, sizeof(__pyx_k_stdtr), 0, 0, 1, 1},
  {&__pyx_n_s_stdtridf, __pyx_k_stdtridf, sizeof(__pyx_k_stdtridf), 0, 0, 1, 1},
  {&__pyx_n_s_stdtrit, __pyx_k_stdtrit, sizeof(__pyx_k_stdtrit), 0, 0, 1, 1},
  {&__pyx_n_s_struve, __pyx_k_struve, sizeof(__pyx_k_struve), 0, 0, 1, 1},
  {&__pyx_n_s_struve_asymp_large_z, __pyx_k_struve_asymp_large_z, sizeof(__pyx_k_struve_asymp_large_z), 0, 0, 1, 1},
  {&__pyx_n_s_struve_bessel_series, __pyx_k_struve_bessel_series, sizeof(__pyx_k_struve_bessel_series), 0, 0, 1, 1},
  {&__pyx_n_s_struve_power_series, __pyx_k_struve_power_series, sizeof(__pyx_k_struve_power_series), 0, 0, 1, 1},
  {&__pyx_n_s_tandg, __pyx_k_tandg, sizeof(__pyx_k_tandg), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_tklmbda, __pyx_k_tklmbda, sizeof(__pyx_k_tklmbda), 0, 0, 1, 1},
  {&__pyx_n_s_traceback, __pyx_k_traceback, sizeof(__pyx_k_traceback), 0, 0, 1, 1},
  {&__pyx_kp_s_ufuncs_extra_code_pxi, __pyx_k_ufuncs_extra_code_pxi, sizeof(__pyx_k_ufuncs_extra_code_pxi), 0, 0, 1, 0},
  {&__pyx_n_u_underflow, __pyx_k_underflow, sizeof(__pyx_k_underflow), 0, 1, 0, 1},
  {&__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_k_unknown_dtype_code_in_numpy_pxd, sizeof(__pyx_k_unknown_dtype_code_in_numpy_pxd), 0, 1, 0, 0},
  {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
  {&__pyx_n_u_warn, __pyx_k_warn, sizeof(__pyx_k_warn), 0, 1, 0, 1},
  {&__pyx_n_s_wofz, __pyx_k_wofz, sizeof(__pyx_k_wofz), 0, 0, 1, 1},
  {&__pyx_n_s_wrightomega, __pyx_k_wrightomega, sizeof(__pyx_k_wrightomega), 0, 0, 1, 1},
  {&__pyx_n_s_xlog1py, __pyx_k_xlog1py, sizeof(__pyx_k_xlog1py), 0, 0, 1, 1},
  {&__pyx_n_s_xlogy, __pyx_k_xlogy, sizeof(__pyx_k_xlogy), 0, 0, 1, 1},
  {&__pyx_n_s_y0, __pyx_k_y0, sizeof(__pyx_k_y0), 0, 0, 1, 1},
  {&__pyx_n_s_y1, __pyx_k_y1, sizeof(__pyx_k_y1), 0, 0, 1, 1},
  {&__pyx_n_s_yn, __pyx_k_yn, sizeof(__pyx_k_yn), 0, 0, 1, 1},
  {&__pyx_n_s_yv, __pyx_k_yv, sizeof(__pyx_k_yv), 0, 0, 1, 1},
  {&__pyx_n_s_yve, __pyx_k_yve, sizeof(__pyx_k_yve), 0, 0, 1, 1},
  {&__pyx_n_s_zeta, __pyx_k_zeta, sizeof(__pyx_k_zeta), 0, 0, 1, 1},
  {&__pyx_n_s_zetac, __pyx_k_zetac, sizeof(__pyx_k_zetac), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_object = __Pyx_GetBuiltinName(__pyx_n_s_object); if (!__pyx_builtin_object) __PYX_ERR(0, 167, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(1, 16, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(2, 272, __pyx_L1_error)
  __pyx_builtin_RuntimeError = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeError); if (!__pyx_builtin_RuntimeError) __PYX_ERR(2, 856, __pyx_L1_error)
  __pyx_builtin_ImportError = __Pyx_GetBuiltinName(__pyx_n_s_ImportError); if (!__pyx_builtin_ImportError) __PYX_ERR(2, 1038, __pyx_L1_error)
  __pyx_builtin_RuntimeWarning = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeWarning); if (!__pyx_builtin_RuntimeWarning) __PYX_ERR(4, 34, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":272
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_C_contiguous); if (unlikely(!__pyx_tuple_)) __PYX_ERR(2, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":276
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_ARRAY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_Fortran_contiguou); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(2, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":306
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_u_Non_native_byte_order_not_suppor); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(2, 306, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":856
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(2, 856, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":880
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor_2); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(2, 880, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1038
 *         _import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_u_numpy_core_multiarray_failed_to); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(2, 1038, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "../../../../../../../../home/treddy/miniconda2/envs/numpy_dev_py3/lib/python3.6/site-packages/Cython/Includes/numpy/__init__.pxd":1044
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_u_numpy_core_umath_failed_to_impor); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(2, 1044, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "_ufuncs_extra_code.pxi":28
 * 
 * 
 * def geterr():             # <<<<<<<<<<<<<<
 *     """Get the current way of handling special-function errors.
 * 
 */
  __pyx_tuple__8 = PyTuple_Pack(4, __pyx_n_s_err, __pyx_n_s_key, __pyx_n_s_code, __pyx_n_s_action); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);
  __pyx_codeobj__9 = (PyObject*)__Pyx_PyCode_New(0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__8, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ufuncs_extra_code_pxi, __pyx_n_s_geterr, 28, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__9)) __PYX_ERR(0, 28, __pyx_L1_error)

  /* "_ufuncs_extra_code.pxi":77
 * 
 * 
 * def seterr(**kwargs):             # <<<<<<<<<<<<<<
 *     """Set how special-function errors are handled.
 * 
 */
  __pyx_tuple__10 = PyTuple_Pack(9, __pyx_n_s_kwargs, __pyx_n_s_olderr, __pyx_n_s_action, __pyx_n_s_newkwargs, __pyx_n_s_key, __pyx_n_s_value, __pyx_n_s_error, __pyx_n_s_code, __pyx_n_s_key); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);
  __pyx_codeobj__11 = (PyObject*)__Pyx_PyCode_New(0, 0, 9, 0, CO_OPTIMIZED|CO_NEWLOCALS|CO_VARKEYWORDS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__10, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ufuncs_extra_code_pxi, __pyx_n_s_seterr, 77, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__11)) __PYX_ERR(0, 77, __pyx_L1_error)

  /* "_ufuncs_extra_code.pxi":167
 * 
 * 
 * class errstate(object):             # <<<<<<<<<<<<<<
 *     """Context manager for special-function error handling.
 * 
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_builtin_object); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "_ufuncs_extra_code.pxi":213
 * 
 *     """
 *     def __init__(self, **kwargs):             # <<<<<<<<<<<<<<
 *         self.kwargs = kwargs
 * 
 */
  __pyx_tuple__13 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_kwargs); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);
  __pyx_codeobj__14 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS|CO_VARKEYWORDS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__13, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ufuncs_extra_code_pxi, __pyx_n_s_init, 213, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__14)) __PYX_ERR(0, 213, __pyx_L1_error)

  /* "_ufuncs_extra_code.pxi":216
 *         self.kwargs = kwargs
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         self.oldstate = seterr(**self.kwargs)
 * 
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 216, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);
  __pyx_codeobj__16 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__15, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ufuncs_extra_code_pxi, __pyx_n_s_enter, 216, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__16)) __PYX_ERR(0, 216, __pyx_L1_error)

  /* "_ufuncs_extra_code.pxi":219
 *         self.oldstate = seterr(**self.kwargs)
 * 
 *     def __exit__(self, exc_type, exc_value, traceback):             # <<<<<<<<<<<<<<
 *         seterr(**self.oldstate)
 */
  __pyx_tuple__17 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_exc_type, __pyx_n_s_exc_value, __pyx_n_s_traceback); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);
  __pyx_codeobj__18 = (PyObject*)__Pyx_PyCode_New(4, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__17, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ufuncs_extra_code_pxi, __pyx_n_s_exit, 219, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__18)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  __pyx_umethod_PyDict_Type_keys.type = (PyObject*)&PyDict_Type;
  __pyx_umethod_PyDict_Type_pop.type = (PyObject*)&PyDict_Type;
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(1, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_3 = PyInt_FromLong(3); if (unlikely(!__pyx_int_3)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_4 = PyInt_FromLong(4); if (unlikely(!__pyx_int_4)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_5 = PyInt_FromLong(5); if (unlikely(!__pyx_int_5)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_6 = PyInt_FromLong(6); if (unlikely(!__pyx_int_6)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_7 = PyInt_FromLong(7); if (unlikely(!__pyx_int_7)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_8 = PyInt_FromLong(8); if (unlikely(!__pyx_int_8)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_9 = PyInt_FromLong(9); if (unlikely(!__pyx_int_9)) __PYX_ERR(1, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(10, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("numpy"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType(__pyx_t_1, "numpy", "dtype", sizeof(PyArray_Descr), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_dtype) __PYX_ERR(2, 206, __pyx_L1_error)
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType(__pyx_t_1, "numpy", "flatiter", sizeof(PyArrayIterObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_flatiter) __PYX_ERR(2, 229, __pyx_L1_error)
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType(__pyx_t_1, "numpy", "broadcast", sizeof(PyArrayMultiIterObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_broadcast) __PYX_ERR(2, 233, __pyx_L1_error)
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType(__pyx_t_1, "numpy", "ndarray", sizeof(PyArrayObject), __Pyx_ImportType_CheckSize_Ignore);
   if (!__pyx_ptype_5numpy_ndarray) __PYX_ERR(2, 242, __pyx_L1_error)
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType(__pyx_t_1, "numpy", "ufunc", sizeof(PyUFuncObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5numpy_ufunc) __PYX_ERR(2, 918, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __pyx_t_1 = PyImport_ImportModule("scipy.special._ufuncs_cxx"); if (!__pyx_t_1) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_faddeeva_dawsn", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn, "void *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_faddeeva_dawsn_complex", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn_complex, "void *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_faddeeva_erf", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erf, "void *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_faddeeva_erfc", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfc, "void *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_faddeeva_erfcx", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx, "void *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_faddeeva_erfcx_complex", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx_complex, "void *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_faddeeva_erfi", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi, "void *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_faddeeva_erfi_complex", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi_complex, "void *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_faddeeva_log_ndtr", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_log_ndtr, "void *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_faddeeva_ndtr", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_ndtr, "void *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_faddeeva_w", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_faddeeva_w, "void *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_export_wrightomega", (void **)&__pyx_vp_5scipy_7special_11_ufuncs_cxx__export_wrightomega, "void *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __pyx_t_1 = PyImport_ImportModule("scipy.special._ufuncs_cxx"); if (!__pyx_t_1) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "_set_action", (void (**)(void))&__pyx_f_5scipy_7special_11_ufuncs_cxx__set_action, "void (sf_error_t, sf_action_t)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}


#if PY_MAJOR_VERSION < 3
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC void
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#else
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC init_ufuncs(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC init_ufuncs(void)
#else
__Pyx_PyMODINIT_FUNC PyInit__ufuncs(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit__ufuncs(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec__ufuncs(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module '_ufuncs' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit__ufuncs(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(1, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("_ufuncs", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(1, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(1, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(1, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_scipy__special___ufuncs) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(1, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "scipy.special._ufuncs")) {
      if (unlikely(PyDict_SetItemString(modules, "scipy.special._ufuncs", __pyx_m) < 0)) __PYX_ERR(1, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  (void)__Pyx_modinit_type_init_code();
  if (unlikely(__Pyx_modinit_type_import_code() != 0)) goto __pyx_L1_error;
  if (unlikely(__Pyx_modinit_variable_import_code() != 0)) goto __pyx_L1_error;
  if (unlikely(__Pyx_modinit_function_import_code() != 0)) goto __pyx_L1_error;
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif

  /* "_ufuncs_extra_code_common.pxi":34
 * from . cimport sf_error
 * 
 * np.import_array()             # <<<<<<<<<<<<<<
 * np.import_ufunc()
 * 
 */
  __pyx_t_1 = __pyx_f_5numpy_import_array(); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(11, 34, __pyx_L1_error)

  /* "_ufuncs_extra_code_common.pxi":35
 * 
 * np.import_array()
 * np.import_ufunc()             # <<<<<<<<<<<<<<
 * 
 * cdef void _set_action(sf_error.sf_error_t code,
 */
  __pyx_t_1 = __pyx_f_5numpy_import_ufunc(); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(11, 35, __pyx_L1_error)

  /* "_ufuncs_extra_code.pxi":2
 * cimport scipy.special._ufuncs_cxx
 * import numpy as np             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_2) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "_ufuncs_extra_code.pxi":7
 * _sf_error_code_map = {
 *     # skip 'ok'
 *     'singular': 1,             # <<<<<<<<<<<<<<
 *     'underflow': 2,
 *     'overflow': 3,
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(9); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_singular, __pyx_int_1) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_underflow, __pyx_int_2) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_overflow, __pyx_int_3) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_slow, __pyx_int_4) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_loss, __pyx_int_5) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_no_result, __pyx_int_6) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_domain, __pyx_int_7) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_arg, __pyx_int_8) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_other, __pyx_int_9) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sf_error_code_map, __pyx_t_2) < 0) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "_ufuncs_extra_code.pxi":19
 * 
 * _sf_error_action_map = {
 *     'ignore': 0,             # <<<<<<<<<<<<<<
 *     'warn': 1,
 *     'raise': 2,
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_ignore, __pyx_int_0) < 0) __PYX_ERR(0, 19, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_warn, __pyx_int_1) < 0) __PYX_ERR(0, 19, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_u_raise, __pyx_int_2) < 0) __PYX_ERR(0, 19, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_int_0, __pyx_n_u_ignore) < 0) __PYX_ERR(0, 19, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_int_1, __pyx_n_u_warn) < 0) __PYX_ERR(0, 19, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_int_2, __pyx_n_u_raise) < 0) __PYX_ERR(0, 19, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sf_error_action_map, __pyx_t_2) < 0) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "_ufuncs_extra_code.pxi":28
 * 
 * 
 * def geterr():             # <<<<<<<<<<<<<<
 *     """Get the current way of handling special-function errors.
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5scipy_7special_7_ufuncs_1geterr, NULL, __pyx_n_s_scipy_special__ufuncs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_geterr, __pyx_t_2) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "_ufuncs_extra_code.pxi":77
 * 
 * 
 * def seterr(**kwargs):             # <<<<<<<<<<<<<<
 *     """Set how special-function errors are handled.
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5scipy_7special_7_ufuncs_3seterr, NULL, __pyx_n_s_scipy_special__ufuncs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_seterr, __pyx_t_2) < 0) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "_ufuncs_extra_code.pxi":167
 * 
 * 
 * class errstate(object):             # <<<<<<<<<<<<<<
 *     """Context manager for special-function error handling.
 * 
 */
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_tuple__12); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_tuple__12, __pyx_n_s_errstate, __pyx_n_s_errstate, (PyObject *) NULL, __pyx_n_s_scipy_special__ufuncs, __pyx_kp_s_Context_manager_for_special_func); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "_ufuncs_extra_code.pxi":213
 * 
 *     """
 *     def __init__(self, **kwargs):             # <<<<<<<<<<<<<<
 *         self.kwargs = kwargs
 * 
 */
  __pyx_t_4 = __Pyx_CyFunction_NewEx(&__pyx_mdef_5scipy_7special_7_ufuncs_8errstate_1__init__, 0, __pyx_n_s_errstate___init, NULL, __pyx_n_s_scipy_special__ufuncs, __pyx_d, ((PyObject *)__pyx_codeobj__14)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_init, __pyx_t_4) < 0) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "_ufuncs_extra_code.pxi":216
 *         self.kwargs = kwargs
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         self.oldstate = seterr(**self.kwargs)
 * 
 */
  __pyx_t_4 = __Pyx_CyFunction_NewEx(&__pyx_mdef_5scipy_7special_7_ufuncs_8errstate_3__enter__, 0, __pyx_n_s_errstate___enter, NULL, __pyx_n_s_scipy_special__ufuncs, __pyx_d, ((PyObject *)__pyx_codeobj__16)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 216, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_enter, __pyx_t_4) < 0) __PYX_ERR(0, 216, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "_ufuncs_extra_code.pxi":219
 *         self.oldstate = seterr(**self.kwargs)
 * 
 *     def __exit__(self, exc_type, exc_value, traceback):             # <<<<<<<<<<<<<<
 *         seterr(**self.oldstate)
 */
  __pyx_t_4 = __Pyx_CyFunction_NewEx(&__pyx_mdef_5scipy_7special_7_ufuncs_8errstate_5__exit__, 0, __pyx_n_s_errstate___exit, NULL, __pyx_n_s_scipy_special__ufuncs, __pyx_d, ((PyObject *)__pyx_codeobj__18)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_exit, __pyx_t_4) < 0) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "_ufuncs_extra_code.pxi":167
 * 
 * 
 * class errstate(object):             # <<<<<<<<<<<<<<
 *     """Context manager for special-function error handling.
 * 
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_errstate, __pyx_tuple__12, __pyx_t_3, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_errstate, __pyx_t_4) < 0) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":1196
 * from ._trig cimport ccospi as _func_ccospi
 * ctypedef double complex _proto_ccospi_t(double complex) nogil
 * cdef _proto_ccospi_t *_proto_ccospi_t_var = &_func_ccospi             # <<<<<<<<<<<<<<
 * from ._trig cimport dcospi as _func_dcospi
 * ctypedef double _proto_dcospi_t(double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_ccospi_t_var = (&__pyx_f_5scipy_7special_5_trig_ccospi);

  /* "scipy/special/_ufuncs.pyx":1199
 * from ._trig cimport dcospi as _func_dcospi
 * ctypedef double _proto_dcospi_t(double) nogil
 * cdef _proto_dcospi_t *_proto_dcospi_t_var = &_func_dcospi             # <<<<<<<<<<<<<<
 * from ._ellip_harm cimport ellip_harmonic as _func_ellip_harmonic
 * ctypedef double _proto_ellip_harmonic_t(double, double, int, int, double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_dcospi_t_var = (&__pyx_f_5scipy_7special_5_trig_dcospi);

  /* "scipy/special/_ufuncs.pyx":1202
 * from ._ellip_harm cimport ellip_harmonic as _func_ellip_harmonic
 * ctypedef double _proto_ellip_harmonic_t(double, double, int, int, double, double, double) nogil
 * cdef _proto_ellip_harmonic_t *_proto_ellip_harmonic_t_var = &_func_ellip_harmonic             # <<<<<<<<<<<<<<
 * from ._legacy cimport ellip_harmonic_unsafe as _func_ellip_harmonic_unsafe
 * ctypedef double _proto_ellip_harmonic_unsafe_t(double, double, double, double, double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_ellip_harmonic_t_var = (&__pyx_f_5scipy_7special_11_ellip_harm_ellip_harmonic);

  /* "scipy/special/_ufuncs.pyx":1205
 * from ._legacy cimport ellip_harmonic_unsafe as _func_ellip_harmonic_unsafe
 * ctypedef double _proto_ellip_harmonic_unsafe_t(double, double, double, double, double, double, double) nogil
 * cdef _proto_ellip_harmonic_unsafe_t *_proto_ellip_harmonic_unsafe_t_var = &_func_ellip_harmonic_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_igam_fac "igam_fac"(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_ellip_harmonic_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_ellip_harmonic_unsafe);

  /* "scipy/special/_ufuncs.pyx":1216
 * from .lambertw cimport lambertw_scalar as _func_lambertw_scalar
 * ctypedef double complex _proto_lambertw_scalar_t(double complex, long, double) nogil
 * cdef _proto_lambertw_scalar_t *_proto_lambertw_scalar_t_var = &_func_lambertw_scalar             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_lanczos_sum_expg_scaled "lanczos_sum_expg_scaled"(double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_lambertw_scalar_t_var = (&__pyx_f_5scipy_7special_8lambertw_lambertw_scalar);

  /* "scipy/special/_ufuncs.pyx":1225
 * from .sf_error cimport _sf_error_test_function as _func__sf_error_test_function
 * ctypedef int _proto__sf_error_test_function_t(int) nogil
 * cdef _proto__sf_error_test_function_t *_proto__sf_error_test_function_t_var = &_func__sf_error_test_function             # <<<<<<<<<<<<<<
 * from ._trig cimport csinpi as _func_csinpi
 * ctypedef double complex _proto_csinpi_t(double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto__sf_error_test_function_t_var = (&__pyx_f_5scipy_7special_8sf_error__sf_error_test_function);

  /* "scipy/special/_ufuncs.pyx":1228
 * from ._trig cimport csinpi as _func_csinpi
 * ctypedef double complex _proto_csinpi_t(double complex) nogil
 * cdef _proto_csinpi_t *_proto_csinpi_t_var = &_func_csinpi             # <<<<<<<<<<<<<<
 * from ._trig cimport dsinpi as _func_dsinpi
 * ctypedef double _proto_dsinpi_t(double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_csinpi_t_var = (&__pyx_f_5scipy_7special_5_trig_csinpi);

  /* "scipy/special/_ufuncs.pyx":1231
 * from ._trig cimport dsinpi as _func_dsinpi
 * ctypedef double _proto_dsinpi_t(double) nogil
 * cdef _proto_dsinpi_t *_proto_dsinpi_t_var = &_func_dsinpi             # <<<<<<<<<<<<<<
 * from ._legacy cimport smirnovc_unsafe as _func_smirnovc_unsafe
 * ctypedef double _proto_smirnovc_unsafe_t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_dsinpi_t_var = (&__pyx_f_5scipy_7special_5_trig_dsinpi);

  /* "scipy/special/_ufuncs.pyx":1234
 * from ._legacy cimport smirnovc_unsafe as _func_smirnovc_unsafe
 * ctypedef double _proto_smirnovc_unsafe_t(double, double) nogil
 * cdef _proto_smirnovc_unsafe_t *_proto_smirnovc_unsafe_t_var = &_func_smirnovc_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_smirnovc "smirnovc"(int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_smirnovc_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_smirnovc_unsafe);

  /* "scipy/special/_ufuncs.pyx":1239
 * from ._legacy cimport smirnovci_unsafe as _func_smirnovci_unsafe
 * ctypedef double _proto_smirnovci_unsafe_t(double, double) nogil
 * cdef _proto_smirnovci_unsafe_t *_proto_smirnovci_unsafe_t_var = &_func_smirnovci_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_smirnovci "smirnovci"(int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_smirnovci_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_smirnovci_unsafe);

  /* "scipy/special/_ufuncs.pyx":1244
 * from ._legacy cimport smirnovp_unsafe as _func_smirnovp_unsafe
 * ctypedef double _proto_smirnovp_unsafe_t(double, double) nogil
 * cdef _proto_smirnovp_unsafe_t *_proto_smirnovp_unsafe_t_var = &_func_smirnovp_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_smirnovp "smirnovp"(int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_smirnovp_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_smirnovp_unsafe);

  /* "scipy/special/_ufuncs.pyx":1249
 * from ._spherical_bessel cimport spherical_in_complex as _func_spherical_in_complex
 * ctypedef double complex _proto_spherical_in_complex_t(long, double complex) nogil
 * cdef _proto_spherical_in_complex_t *_proto_spherical_in_complex_t_var = &_func_spherical_in_complex             # <<<<<<<<<<<<<<
 * from ._spherical_bessel cimport spherical_in_real as _func_spherical_in_real
 * ctypedef double _proto_spherical_in_real_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_spherical_in_complex_t_var = (&__pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_complex);

  /* "scipy/special/_ufuncs.pyx":1252
 * from ._spherical_bessel cimport spherical_in_real as _func_spherical_in_real
 * ctypedef double _proto_spherical_in_real_t(long, double) nogil
 * cdef _proto_spherical_in_real_t *_proto_spherical_in_real_t_var = &_func_spherical_in_real             # <<<<<<<<<<<<<<
 * from ._spherical_bessel cimport spherical_in_d_complex as _func_spherical_in_d_complex
 * ctypedef double complex _proto_spherical_in_d_complex_t(long, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_spherical_in_real_t_var = (&__pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_real);

  /* "scipy/special/_ufuncs.pyx":1255
 * from ._spherical_bessel cimport spherical_in_d_complex as _func_spherical_in_d_complex
 * ctypedef double complex _proto_spherical_in_d_complex_t(long, double complex) nogil
 * cdef _proto_spherical_in_d_complex_t *_proto_spherical_in_d_complex_t_var = &_func_spherical_in_d_complex             # <<<<<<<<<<<<<<
 * from ._spherical_bessel cimport spherical_in_d_real as _func_spherical_in_d_real
 * ctypedef double _proto_spherical_in_d_real_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_spherical_in_d_complex_t_var = (&__pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_d_complex);

  /* "scipy/special/_ufuncs.pyx":1258
 * from ._spherical_bessel cimport spherical_in_d_real as _func_spherical_in_d_real
 * ctypedef double _proto_spherical_in_d_real_t(long, double) nogil
 * cdef _proto_spherical_in_d_real_t *_proto_spherical_in_d_real_t_var = &_func_spherical_in_d_real             # <<<<<<<<<<<<<<
 * from ._spherical_bessel cimport spherical_jn_complex as _func_spherical_jn_complex
 * ctypedef double complex _proto_spherical_jn_complex_t(long, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_spherical_in_d_real_t_var = (&__pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_d_real);

  /* "scipy/special/_ufuncs.pyx":1261
 * from ._spherical_bessel cimport spherical_jn_complex as _func_spherical_jn_complex
 * ctypedef double complex _proto_spherical_jn_complex_t(long, double complex) nogil
 * cdef _proto_spherical_jn_complex_t *_proto_spherical_jn_complex_t_var = &_func_spherical_jn_complex             # <<<<<<<<<<<<<<
 * from ._spherical_bessel cimport spherical_jn_real as _func_spherical_jn_real
 * ctypedef double _proto_spherical_jn_real_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_spherical_jn_complex_t_var = (&__pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_complex);

  /* "scipy/special/_ufuncs.pyx":1264
 * from ._spherical_bessel cimport spherical_jn_real as _func_spherical_jn_real
 * ctypedef double _proto_spherical_jn_real_t(long, double) nogil
 * cdef _proto_spherical_jn_real_t *_proto_spherical_jn_real_t_var = &_func_spherical_jn_real             # <<<<<<<<<<<<<<
 * from ._spherical_bessel cimport spherical_jn_d_complex as _func_spherical_jn_d_complex
 * ctypedef double complex _proto_spherical_jn_d_complex_t(long, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_spherical_jn_real_t_var = (&__pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_real);

  /* "scipy/special/_ufuncs.pyx":1267
 * from ._spherical_bessel cimport spherical_jn_d_complex as _func_spherical_jn_d_complex
 * ctypedef double complex _proto_spherical_jn_d_complex_t(long, double complex) nogil
 * cdef _proto_spherical_jn_d_complex_t *_proto_spherical_jn_d_complex_t_var = &_func_spherical_jn_d_complex             # <<<<<<<<<<<<<<
 * from ._spherical_bessel cimport spherical_jn_d_real as _func_spherical_jn_d_real
 * ctypedef double _proto_spherical_jn_d_real_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_spherical_jn_d_complex_t_var = (&__pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_d_complex);

  /* "scipy/special/_ufuncs.pyx":1270
 * from ._spherical_bessel cimport spherical_jn_d_real as _func_spherical_jn_d_real
 * ctypedef double _proto_spherical_jn_d_real_t(long, double) nogil
 * cdef _proto_spherical_jn_d_real_t *_proto_spherical_jn_d_real_t_var = &_func_spherical_jn_d_real             # <<<<<<<<<<<<<<
 * from ._spherical_bessel cimport spherical_kn_complex as _func_spherical_kn_complex
 * ctypedef double complex _proto_spherical_kn_complex_t(long, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_spherical_jn_d_real_t_var = (&__pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_d_real);

  /* "scipy/special/_ufuncs.pyx":1273
 * from ._spherical_bessel cimport spherical_kn_complex as _func_spherical_kn_complex
 * ctypedef double complex _proto_spherical_kn_complex_t(long, double complex) nogil
 * cdef _proto_spherical_kn_complex_t *_proto_spherical_kn_complex_t_var = &_func_spherical_kn_complex             # <<<<<<<<<<<<<<
 * from ._spherical_bessel cimport spherical_kn_real as _func_spherical_kn_real
 * ctypedef double _proto_spherical_kn_real_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_spherical_kn_complex_t_var = (&__pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_complex);

  /* "scipy/special/_ufuncs.pyx":1276
 * from ._spherical_bessel cimport spherical_kn_real as _func_spherical_kn_real
 * ctypedef double _proto_spherical_kn_real_t(long, double) nogil
 * cdef _proto_spherical_kn_real_t *_proto_spherical_kn_real_t_var = &_func_spherical_kn_real             # <<<<<<<<<<<<<<
 * from ._spherical_bessel cimport spherical_kn_d_complex as _func_spherical_kn_d_complex
 * ctypedef double complex _proto_spherical_kn_d_complex_t(long, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_spherical_kn_real_t_var = (&__pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_real);

  /* "scipy/special/_ufuncs.pyx":1279
 * from ._spherical_bessel cimport spherical_kn_d_complex as _func_spherical_kn_d_complex
 * ctypedef double complex _proto_spherical_kn_d_complex_t(long, double complex) nogil
 * cdef _proto_spherical_kn_d_complex_t *_proto_spherical_kn_d_complex_t_var = &_func_spherical_kn_d_complex             # <<<<<<<<<<<<<<
 * from ._spherical_bessel cimport spherical_kn_d_real as _func_spherical_kn_d_real
 * ctypedef double _proto_spherical_kn_d_real_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_spherical_kn_d_complex_t_var = (&__pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_d_complex);

  /* "scipy/special/_ufuncs.pyx":1282
 * from ._spherical_bessel cimport spherical_kn_d_real as _func_spherical_kn_d_real
 * ctypedef double _proto_spherical_kn_d_real_t(long, double) nogil
 * cdef _proto_spherical_kn_d_real_t *_proto_spherical_kn_d_real_t_var = &_func_spherical_kn_d_real             # <<<<<<<<<<<<<<
 * from ._spherical_bessel cimport spherical_yn_complex as _func_spherical_yn_complex
 * ctypedef double complex _proto_spherical_yn_complex_t(long, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_spherical_kn_d_real_t_var = (&__pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_d_real);

  /* "scipy/special/_ufuncs.pyx":1285
 * from ._spherical_bessel cimport spherical_yn_complex as _func_spherical_yn_complex
 * ctypedef double complex _proto_spherical_yn_complex_t(long, double complex) nogil
 * cdef _proto_spherical_yn_complex_t *_proto_spherical_yn_complex_t_var = &_func_spherical_yn_complex             # <<<<<<<<<<<<<<
 * from ._spherical_bessel cimport spherical_yn_real as _func_spherical_yn_real
 * ctypedef double _proto_spherical_yn_real_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_spherical_yn_complex_t_var = (&__pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_complex);

  /* "scipy/special/_ufuncs.pyx":1288
 * from ._spherical_bessel cimport spherical_yn_real as _func_spherical_yn_real
 * ctypedef double _proto_spherical_yn_real_t(long, double) nogil
 * cdef _proto_spherical_yn_real_t *_proto_spherical_yn_real_t_var = &_func_spherical_yn_real             # <<<<<<<<<<<<<<
 * from ._spherical_bessel cimport spherical_yn_d_complex as _func_spherical_yn_d_complex
 * ctypedef double complex _proto_spherical_yn_d_complex_t(long, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_spherical_yn_real_t_var = (&__pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_real);

  /* "scipy/special/_ufuncs.pyx":1291
 * from ._spherical_bessel cimport spherical_yn_d_complex as _func_spherical_yn_d_complex
 * ctypedef double complex _proto_spherical_yn_d_complex_t(long, double complex) nogil
 * cdef _proto_spherical_yn_d_complex_t *_proto_spherical_yn_d_complex_t_var = &_func_spherical_yn_d_complex             # <<<<<<<<<<<<<<
 * from ._spherical_bessel cimport spherical_yn_d_real as _func_spherical_yn_d_real
 * ctypedef double _proto_spherical_yn_d_real_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_spherical_yn_d_complex_t_var = (&__pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_d_complex);

  /* "scipy/special/_ufuncs.pyx":1294
 * from ._spherical_bessel cimport spherical_yn_d_real as _func_spherical_yn_d_real
 * ctypedef double _proto_spherical_yn_d_real_t(long, double) nogil
 * cdef _proto_spherical_yn_d_real_t *_proto_spherical_yn_d_real_t_var = &_func_spherical_yn_d_real             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_struve_asymp_large_z "struve_asymp_large_z"(double, double, int, double *) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_spherical_yn_d_real_t_var = (&__pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_d_real);

  /* "scipy/special/_ufuncs.pyx":1305
 * from ._agm cimport agm as _func_agm
 * ctypedef double _proto_agm_t(double, double) nogil
 * cdef _proto_agm_t *_proto_agm_t_var = &_func_agm             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef int _func_airy_wrap "airy_wrap"(double, double *, double *, double *, double *) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_agm_t_var = (&__pyx_f_5scipy_7special_4_agm_agm);

  /* "scipy/special/_ufuncs.pyx":1316
 * from ._legacy cimport bdtr_unsafe as _func_bdtr_unsafe
 * ctypedef double _proto_bdtr_unsafe_t(double, double, double) nogil
 * cdef _proto_bdtr_unsafe_t *_proto_bdtr_unsafe_t_var = &_func_bdtr_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_bdtr "bdtr"(int, int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_bdtr_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_bdtr_unsafe);

  /* "scipy/special/_ufuncs.pyx":1321
 * from ._legacy cimport bdtrc_unsafe as _func_bdtrc_unsafe
 * ctypedef double _proto_bdtrc_unsafe_t(double, double, double) nogil
 * cdef _proto_bdtrc_unsafe_t *_proto_bdtrc_unsafe_t_var = &_func_bdtrc_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_bdtrc "bdtrc"(int, int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_bdtrc_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_bdtrc_unsafe);

  /* "scipy/special/_ufuncs.pyx":1326
 * from ._legacy cimport bdtri_unsafe as _func_bdtri_unsafe
 * ctypedef double _proto_bdtri_unsafe_t(double, double, double) nogil
 * cdef _proto_bdtri_unsafe_t *_proto_bdtri_unsafe_t_var = &_func_bdtri_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_bdtri "bdtri"(int, int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_bdtri_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_bdtri_unsafe);

  /* "scipy/special/_ufuncs.pyx":1353
 * from .orthogonal_eval cimport binom as _func_binom
 * ctypedef double _proto_binom_t(double, double) nogil
 * cdef _proto_binom_t *_proto_binom_t_var = &_func_binom             # <<<<<<<<<<<<<<
 * from ._boxcox cimport boxcox as _func_boxcox
 * ctypedef double _proto_boxcox_t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_binom_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_binom);

  /* "scipy/special/_ufuncs.pyx":1356
 * from ._boxcox cimport boxcox as _func_boxcox
 * ctypedef double _proto_boxcox_t(double, double) nogil
 * cdef _proto_boxcox_t *_proto_boxcox_t_var = &_func_boxcox             # <<<<<<<<<<<<<<
 * from ._boxcox cimport boxcox1p as _func_boxcox1p
 * ctypedef double _proto_boxcox1p_t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_boxcox_t_var = (&__pyx_f_5scipy_7special_7_boxcox_boxcox);

  /* "scipy/special/_ufuncs.pyx":1359
 * from ._boxcox cimport boxcox1p as _func_boxcox1p
 * ctypedef double _proto_boxcox1p_t(double, double) nogil
 * cdef _proto_boxcox1p_t *_proto_boxcox1p_t_var = &_func_boxcox1p             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_btdtr "btdtr"(double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_boxcox1p_t_var = (&__pyx_f_5scipy_7special_7_boxcox_boxcox1p);

  /* "scipy/special/_ufuncs.pyx":1404
 * from ._convex_analysis cimport entr as _func_entr
 * ctypedef double _proto_entr_t(double) nogil
 * cdef _proto_entr_t *_proto_entr_t_var = &_func_entr             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_erf "erf"(double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_entr_t_var = (&__pyx_f_5scipy_7special_16_convex_analysis_entr);

  /* "scipy/special/_ufuncs.pyx":1411
 * from .orthogonal_eval cimport eval_chebyc as _func_eval_chebyc
 * ctypedef double complex _proto_eval_chebyc_double_complex__t(double, double complex) nogil
 * cdef _proto_eval_chebyc_double_complex__t *_proto_eval_chebyc_double_complex__t_var = &_func_eval_chebyc[double_complex]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_chebyc as _func_eval_chebyc
 * ctypedef double _proto_eval_chebyc_double__t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyc_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc);

  /* "scipy/special/_ufuncs.pyx":1414
 * from .orthogonal_eval cimport eval_chebyc as _func_eval_chebyc
 * ctypedef double _proto_eval_chebyc_double__t(double, double) nogil
 * cdef _proto_eval_chebyc_double__t *_proto_eval_chebyc_double__t_var = &_func_eval_chebyc[double]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_chebyc_l as _func_eval_chebyc_l
 * ctypedef double _proto_eval_chebyc_l_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyc_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc);

  /* "scipy/special/_ufuncs.pyx":1417
 * from .orthogonal_eval cimport eval_chebyc_l as _func_eval_chebyc_l
 * ctypedef double _proto_eval_chebyc_l_t(long, double) nogil
 * cdef _proto_eval_chebyc_l_t *_proto_eval_chebyc_l_t_var = &_func_eval_chebyc_l             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_chebys as _func_eval_chebys
 * ctypedef double complex _proto_eval_chebys_double_complex__t(double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyc_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc_l);

  /* "scipy/special/_ufuncs.pyx":1420
 * from .orthogonal_eval cimport eval_chebys as _func_eval_chebys
 * ctypedef double complex _proto_eval_chebys_double_complex__t(double, double complex) nogil
 * cdef _proto_eval_chebys_double_complex__t *_proto_eval_chebys_double_complex__t_var = &_func_eval_chebys[double_complex]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_chebys as _func_eval_chebys
 * ctypedef double _proto_eval_chebys_double__t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebys_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys);

  /* "scipy/special/_ufuncs.pyx":1423
 * from .orthogonal_eval cimport eval_chebys as _func_eval_chebys
 * ctypedef double _proto_eval_chebys_double__t(double, double) nogil
 * cdef _proto_eval_chebys_double__t *_proto_eval_chebys_double__t_var = &_func_eval_chebys[double]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_chebys_l as _func_eval_chebys_l
 * ctypedef double _proto_eval_chebys_l_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebys_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys);

  /* "scipy/special/_ufuncs.pyx":1426
 * from .orthogonal_eval cimport eval_chebys_l as _func_eval_chebys_l
 * ctypedef double _proto_eval_chebys_l_t(long, double) nogil
 * cdef _proto_eval_chebys_l_t *_proto_eval_chebys_l_t_var = &_func_eval_chebys_l             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_chebyt as _func_eval_chebyt
 * ctypedef double complex _proto_eval_chebyt_double_complex__t(double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebys_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys_l);

  /* "scipy/special/_ufuncs.pyx":1429
 * from .orthogonal_eval cimport eval_chebyt as _func_eval_chebyt
 * ctypedef double complex _proto_eval_chebyt_double_complex__t(double, double complex) nogil
 * cdef _proto_eval_chebyt_double_complex__t *_proto_eval_chebyt_double_complex__t_var = &_func_eval_chebyt[double_complex]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_chebyt as _func_eval_chebyt
 * ctypedef double _proto_eval_chebyt_double__t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyt_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt);

  /* "scipy/special/_ufuncs.pyx":1432
 * from .orthogonal_eval cimport eval_chebyt as _func_eval_chebyt
 * ctypedef double _proto_eval_chebyt_double__t(double, double) nogil
 * cdef _proto_eval_chebyt_double__t *_proto_eval_chebyt_double__t_var = &_func_eval_chebyt[double]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_chebyt_l as _func_eval_chebyt_l
 * ctypedef double _proto_eval_chebyt_l_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyt_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt);

  /* "scipy/special/_ufuncs.pyx":1435
 * from .orthogonal_eval cimport eval_chebyt_l as _func_eval_chebyt_l
 * ctypedef double _proto_eval_chebyt_l_t(long, double) nogil
 * cdef _proto_eval_chebyt_l_t *_proto_eval_chebyt_l_t_var = &_func_eval_chebyt_l             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_chebyu as _func_eval_chebyu
 * ctypedef double complex _proto_eval_chebyu_double_complex__t(double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyt_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt_l);

  /* "scipy/special/_ufuncs.pyx":1438
 * from .orthogonal_eval cimport eval_chebyu as _func_eval_chebyu
 * ctypedef double complex _proto_eval_chebyu_double_complex__t(double, double complex) nogil
 * cdef _proto_eval_chebyu_double_complex__t *_proto_eval_chebyu_double_complex__t_var = &_func_eval_chebyu[double_complex]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_chebyu as _func_eval_chebyu
 * ctypedef double _proto_eval_chebyu_double__t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyu_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu);

  /* "scipy/special/_ufuncs.pyx":1441
 * from .orthogonal_eval cimport eval_chebyu as _func_eval_chebyu
 * ctypedef double _proto_eval_chebyu_double__t(double, double) nogil
 * cdef _proto_eval_chebyu_double__t *_proto_eval_chebyu_double__t_var = &_func_eval_chebyu[double]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_chebyu_l as _func_eval_chebyu_l
 * ctypedef double _proto_eval_chebyu_l_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyu_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu);

  /* "scipy/special/_ufuncs.pyx":1444
 * from .orthogonal_eval cimport eval_chebyu_l as _func_eval_chebyu_l
 * ctypedef double _proto_eval_chebyu_l_t(long, double) nogil
 * cdef _proto_eval_chebyu_l_t *_proto_eval_chebyu_l_t_var = &_func_eval_chebyu_l             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_gegenbauer as _func_eval_gegenbauer
 * ctypedef double complex _proto_eval_gegenbauer_double_complex__t(double, double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_chebyu_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu_l);

  /* "scipy/special/_ufuncs.pyx":1447
 * from .orthogonal_eval cimport eval_gegenbauer as _func_eval_gegenbauer
 * ctypedef double complex _proto_eval_gegenbauer_double_complex__t(double, double, double complex) nogil
 * cdef _proto_eval_gegenbauer_double_complex__t *_proto_eval_gegenbauer_double_complex__t_var = &_func_eval_gegenbauer[double_complex]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_gegenbauer as _func_eval_gegenbauer
 * ctypedef double _proto_eval_gegenbauer_double__t(double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer);

  /* "scipy/special/_ufuncs.pyx":1450
 * from .orthogonal_eval cimport eval_gegenbauer as _func_eval_gegenbauer
 * ctypedef double _proto_eval_gegenbauer_double__t(double, double, double) nogil
 * cdef _proto_eval_gegenbauer_double__t *_proto_eval_gegenbauer_double__t_var = &_func_eval_gegenbauer[double]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_gegenbauer_l as _func_eval_gegenbauer_l
 * ctypedef double _proto_eval_gegenbauer_l_t(long, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer);

  /* "scipy/special/_ufuncs.pyx":1453
 * from .orthogonal_eval cimport eval_gegenbauer_l as _func_eval_gegenbauer_l
 * ctypedef double _proto_eval_gegenbauer_l_t(long, double, double) nogil
 * cdef _proto_eval_gegenbauer_l_t *_proto_eval_gegenbauer_l_t_var = &_func_eval_gegenbauer_l             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_genlaguerre as _func_eval_genlaguerre
 * ctypedef double complex _proto_eval_genlaguerre_double_complex__t(double, double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_gegenbauer_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer_l);

  /* "scipy/special/_ufuncs.pyx":1456
 * from .orthogonal_eval cimport eval_genlaguerre as _func_eval_genlaguerre
 * ctypedef double complex _proto_eval_genlaguerre_double_complex__t(double, double, double complex) nogil
 * cdef _proto_eval_genlaguerre_double_complex__t *_proto_eval_genlaguerre_double_complex__t_var = &_func_eval_genlaguerre[double_complex]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_genlaguerre as _func_eval_genlaguerre
 * ctypedef double _proto_eval_genlaguerre_double__t(double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre);

  /* "scipy/special/_ufuncs.pyx":1459
 * from .orthogonal_eval cimport eval_genlaguerre as _func_eval_genlaguerre
 * ctypedef double _proto_eval_genlaguerre_double__t(double, double, double) nogil
 * cdef _proto_eval_genlaguerre_double__t *_proto_eval_genlaguerre_double__t_var = &_func_eval_genlaguerre[double]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_genlaguerre_l as _func_eval_genlaguerre_l
 * ctypedef double _proto_eval_genlaguerre_l_t(long, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre);

  /* "scipy/special/_ufuncs.pyx":1462
 * from .orthogonal_eval cimport eval_genlaguerre_l as _func_eval_genlaguerre_l
 * ctypedef double _proto_eval_genlaguerre_l_t(long, double, double) nogil
 * cdef _proto_eval_genlaguerre_l_t *_proto_eval_genlaguerre_l_t_var = &_func_eval_genlaguerre_l             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_hermite as _func_eval_hermite
 * ctypedef double _proto_eval_hermite_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_genlaguerre_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre_l);

  /* "scipy/special/_ufuncs.pyx":1465
 * from .orthogonal_eval cimport eval_hermite as _func_eval_hermite
 * ctypedef double _proto_eval_hermite_t(long, double) nogil
 * cdef _proto_eval_hermite_t *_proto_eval_hermite_t_var = &_func_eval_hermite             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_hermitenorm as _func_eval_hermitenorm
 * ctypedef double _proto_eval_hermitenorm_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_hermite_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_hermite);

  /* "scipy/special/_ufuncs.pyx":1468
 * from .orthogonal_eval cimport eval_hermitenorm as _func_eval_hermitenorm
 * ctypedef double _proto_eval_hermitenorm_t(long, double) nogil
 * cdef _proto_eval_hermitenorm_t *_proto_eval_hermitenorm_t_var = &_func_eval_hermitenorm             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_jacobi as _func_eval_jacobi
 * ctypedef double complex _proto_eval_jacobi_double_complex__t(double, double, double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_hermitenorm_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_hermitenorm);

  /* "scipy/special/_ufuncs.pyx":1471
 * from .orthogonal_eval cimport eval_jacobi as _func_eval_jacobi
 * ctypedef double complex _proto_eval_jacobi_double_complex__t(double, double, double, double complex) nogil
 * cdef _proto_eval_jacobi_double_complex__t *_proto_eval_jacobi_double_complex__t_var = &_func_eval_jacobi[double_complex]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_jacobi as _func_eval_jacobi
 * ctypedef double _proto_eval_jacobi_double__t(double, double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_jacobi_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi);

  /* "scipy/special/_ufuncs.pyx":1474
 * from .orthogonal_eval cimport eval_jacobi as _func_eval_jacobi
 * ctypedef double _proto_eval_jacobi_double__t(double, double, double, double) nogil
 * cdef _proto_eval_jacobi_double__t *_proto_eval_jacobi_double__t_var = &_func_eval_jacobi[double]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_jacobi_l as _func_eval_jacobi_l
 * ctypedef double _proto_eval_jacobi_l_t(long, double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_jacobi_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi);

  /* "scipy/special/_ufuncs.pyx":1477
 * from .orthogonal_eval cimport eval_jacobi_l as _func_eval_jacobi_l
 * ctypedef double _proto_eval_jacobi_l_t(long, double, double, double) nogil
 * cdef _proto_eval_jacobi_l_t *_proto_eval_jacobi_l_t_var = &_func_eval_jacobi_l             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_laguerre as _func_eval_laguerre
 * ctypedef double complex _proto_eval_laguerre_double_complex__t(double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_jacobi_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi_l);

  /* "scipy/special/_ufuncs.pyx":1480
 * from .orthogonal_eval cimport eval_laguerre as _func_eval_laguerre
 * ctypedef double complex _proto_eval_laguerre_double_complex__t(double, double complex) nogil
 * cdef _proto_eval_laguerre_double_complex__t *_proto_eval_laguerre_double_complex__t_var = &_func_eval_laguerre[double_complex]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_laguerre as _func_eval_laguerre
 * ctypedef double _proto_eval_laguerre_double__t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_laguerre_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre);

  /* "scipy/special/_ufuncs.pyx":1483
 * from .orthogonal_eval cimport eval_laguerre as _func_eval_laguerre
 * ctypedef double _proto_eval_laguerre_double__t(double, double) nogil
 * cdef _proto_eval_laguerre_double__t *_proto_eval_laguerre_double__t_var = &_func_eval_laguerre[double]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_laguerre_l as _func_eval_laguerre_l
 * ctypedef double _proto_eval_laguerre_l_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_laguerre_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre);

  /* "scipy/special/_ufuncs.pyx":1486
 * from .orthogonal_eval cimport eval_laguerre_l as _func_eval_laguerre_l
 * ctypedef double _proto_eval_laguerre_l_t(long, double) nogil
 * cdef _proto_eval_laguerre_l_t *_proto_eval_laguerre_l_t_var = &_func_eval_laguerre_l             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_legendre as _func_eval_legendre
 * ctypedef double complex _proto_eval_legendre_double_complex__t(double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_laguerre_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre_l);

  /* "scipy/special/_ufuncs.pyx":1489
 * from .orthogonal_eval cimport eval_legendre as _func_eval_legendre
 * ctypedef double complex _proto_eval_legendre_double_complex__t(double, double complex) nogil
 * cdef _proto_eval_legendre_double_complex__t *_proto_eval_legendre_double_complex__t_var = &_func_eval_legendre[double_complex]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_legendre as _func_eval_legendre
 * ctypedef double _proto_eval_legendre_double__t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_legendre_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre);

  /* "scipy/special/_ufuncs.pyx":1492
 * from .orthogonal_eval cimport eval_legendre as _func_eval_legendre
 * ctypedef double _proto_eval_legendre_double__t(double, double) nogil
 * cdef _proto_eval_legendre_double__t *_proto_eval_legendre_double__t_var = &_func_eval_legendre[double]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_legendre_l as _func_eval_legendre_l
 * ctypedef double _proto_eval_legendre_l_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_legendre_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre);

  /* "scipy/special/_ufuncs.pyx":1495
 * from .orthogonal_eval cimport eval_legendre_l as _func_eval_legendre_l
 * ctypedef double _proto_eval_legendre_l_t(long, double) nogil
 * cdef _proto_eval_legendre_l_t *_proto_eval_legendre_l_t_var = &_func_eval_legendre_l             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_sh_chebyt as _func_eval_sh_chebyt
 * ctypedef double complex _proto_eval_sh_chebyt_double_complex__t(double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_legendre_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre_l);

  /* "scipy/special/_ufuncs.pyx":1498
 * from .orthogonal_eval cimport eval_sh_chebyt as _func_eval_sh_chebyt
 * ctypedef double complex _proto_eval_sh_chebyt_double_complex__t(double, double complex) nogil
 * cdef _proto_eval_sh_chebyt_double_complex__t *_proto_eval_sh_chebyt_double_complex__t_var = &_func_eval_sh_chebyt[double_complex]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_sh_chebyt as _func_eval_sh_chebyt
 * ctypedef double _proto_eval_sh_chebyt_double__t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt);

  /* "scipy/special/_ufuncs.pyx":1501
 * from .orthogonal_eval cimport eval_sh_chebyt as _func_eval_sh_chebyt
 * ctypedef double _proto_eval_sh_chebyt_double__t(double, double) nogil
 * cdef _proto_eval_sh_chebyt_double__t *_proto_eval_sh_chebyt_double__t_var = &_func_eval_sh_chebyt[double]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_sh_chebyt_l as _func_eval_sh_chebyt_l
 * ctypedef double _proto_eval_sh_chebyt_l_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt);

  /* "scipy/special/_ufuncs.pyx":1504
 * from .orthogonal_eval cimport eval_sh_chebyt_l as _func_eval_sh_chebyt_l
 * ctypedef double _proto_eval_sh_chebyt_l_t(long, double) nogil
 * cdef _proto_eval_sh_chebyt_l_t *_proto_eval_sh_chebyt_l_t_var = &_func_eval_sh_chebyt_l             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_sh_chebyu as _func_eval_sh_chebyu
 * ctypedef double complex _proto_eval_sh_chebyu_double_complex__t(double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyt_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt_l);

  /* "scipy/special/_ufuncs.pyx":1507
 * from .orthogonal_eval cimport eval_sh_chebyu as _func_eval_sh_chebyu
 * ctypedef double complex _proto_eval_sh_chebyu_double_complex__t(double, double complex) nogil
 * cdef _proto_eval_sh_chebyu_double_complex__t *_proto_eval_sh_chebyu_double_complex__t_var = &_func_eval_sh_chebyu[double_complex]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_sh_chebyu as _func_eval_sh_chebyu
 * ctypedef double _proto_eval_sh_chebyu_double__t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu);

  /* "scipy/special/_ufuncs.pyx":1510
 * from .orthogonal_eval cimport eval_sh_chebyu as _func_eval_sh_chebyu
 * ctypedef double _proto_eval_sh_chebyu_double__t(double, double) nogil
 * cdef _proto_eval_sh_chebyu_double__t *_proto_eval_sh_chebyu_double__t_var = &_func_eval_sh_chebyu[double]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_sh_chebyu_l as _func_eval_sh_chebyu_l
 * ctypedef double _proto_eval_sh_chebyu_l_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu);

  /* "scipy/special/_ufuncs.pyx":1513
 * from .orthogonal_eval cimport eval_sh_chebyu_l as _func_eval_sh_chebyu_l
 * ctypedef double _proto_eval_sh_chebyu_l_t(long, double) nogil
 * cdef _proto_eval_sh_chebyu_l_t *_proto_eval_sh_chebyu_l_t_var = &_func_eval_sh_chebyu_l             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_sh_jacobi as _func_eval_sh_jacobi
 * ctypedef double complex _proto_eval_sh_jacobi_double_complex__t(double, double, double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_chebyu_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu_l);

  /* "scipy/special/_ufuncs.pyx":1516
 * from .orthogonal_eval cimport eval_sh_jacobi as _func_eval_sh_jacobi
 * ctypedef double complex _proto_eval_sh_jacobi_double_complex__t(double, double, double, double complex) nogil
 * cdef _proto_eval_sh_jacobi_double_complex__t *_proto_eval_sh_jacobi_double_complex__t_var = &_func_eval_sh_jacobi[double_complex]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_sh_jacobi as _func_eval_sh_jacobi
 * ctypedef double _proto_eval_sh_jacobi_double__t(double, double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi);

  /* "scipy/special/_ufuncs.pyx":1519
 * from .orthogonal_eval cimport eval_sh_jacobi as _func_eval_sh_jacobi
 * ctypedef double _proto_eval_sh_jacobi_double__t(double, double, double, double) nogil
 * cdef _proto_eval_sh_jacobi_double__t *_proto_eval_sh_jacobi_double__t_var = &_func_eval_sh_jacobi[double]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_sh_jacobi_l as _func_eval_sh_jacobi_l
 * ctypedef double _proto_eval_sh_jacobi_l_t(long, double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi);

  /* "scipy/special/_ufuncs.pyx":1522
 * from .orthogonal_eval cimport eval_sh_jacobi_l as _func_eval_sh_jacobi_l
 * ctypedef double _proto_eval_sh_jacobi_l_t(long, double, double, double) nogil
 * cdef _proto_eval_sh_jacobi_l_t *_proto_eval_sh_jacobi_l_t_var = &_func_eval_sh_jacobi_l             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_sh_legendre as _func_eval_sh_legendre
 * ctypedef double complex _proto_eval_sh_legendre_double_complex__t(double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_jacobi_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi_l);

  /* "scipy/special/_ufuncs.pyx":1525
 * from .orthogonal_eval cimport eval_sh_legendre as _func_eval_sh_legendre
 * ctypedef double complex _proto_eval_sh_legendre_double_complex__t(double, double complex) nogil
 * cdef _proto_eval_sh_legendre_double_complex__t *_proto_eval_sh_legendre_double_complex__t_var = &_func_eval_sh_legendre[double_complex]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_sh_legendre as _func_eval_sh_legendre
 * ctypedef double _proto_eval_sh_legendre_double__t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre);

  /* "scipy/special/_ufuncs.pyx":1528
 * from .orthogonal_eval cimport eval_sh_legendre as _func_eval_sh_legendre
 * ctypedef double _proto_eval_sh_legendre_double__t(double, double) nogil
 * cdef _proto_eval_sh_legendre_double__t *_proto_eval_sh_legendre_double__t_var = &_func_eval_sh_legendre[double]             # <<<<<<<<<<<<<<
 * from .orthogonal_eval cimport eval_sh_legendre_l as _func_eval_sh_legendre_l
 * ctypedef double _proto_eval_sh_legendre_l_t(long, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre);

  /* "scipy/special/_ufuncs.pyx":1531
 * from .orthogonal_eval cimport eval_sh_legendre_l as _func_eval_sh_legendre_l
 * ctypedef double _proto_eval_sh_legendre_l_t(long, double) nogil
 * cdef _proto_eval_sh_legendre_l_t *_proto_eval_sh_legendre_l_t_var = &_func_eval_sh_legendre_l             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double complex _func_cexp1_wrap "cexp1_wrap"(double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_eval_sh_legendre_l_t_var = (&__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre_l);

  /* "scipy/special/_ufuncs.pyx":1552
 * from ._cunity cimport cexpm1 as _func_cexpm1
 * ctypedef double complex _proto_cexpm1_t(double complex) nogil
 * cdef _proto_cexpm1_t *_proto_cexpm1_t_var = &_func_cexpm1             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_expm1 "expm1"(double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_cexpm1_t_var = (&__pyx_f_5scipy_7special_7_cunity_cexpm1);

  /* "scipy/special/_ufuncs.pyx":1557
 * from ._legacy cimport expn_unsafe as _func_expn_unsafe
 * ctypedef double _proto_expn_unsafe_t(double, double) nogil
 * cdef _proto_expn_unsafe_t *_proto_expn_unsafe_t_var = &_func_expn_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_expn "expn"(int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_expn_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_expn_unsafe);

  /* "scipy/special/_ufuncs.pyx":1562
 * from ._exprel cimport exprel as _func_exprel
 * ctypedef double _proto_exprel_t(double) nogil
 * cdef _proto_exprel_t *_proto_exprel_t_var = &_func_exprel             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_fdtr "fdtr"(double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_exprel_t_var = (&__pyx_f_5scipy_7special_7_exprel_exprel);

  /* "scipy/special/_ufuncs.pyx":1577
 * from ._loggamma cimport cgamma as _func_cgamma
 * ctypedef double complex _proto_cgamma_t(double complex) nogil
 * cdef _proto_cgamma_t *_proto_cgamma_t_var = &_func_cgamma             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_Gamma "Gamma"(double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_cgamma_t_var = (&__pyx_f_5scipy_7special_9_loggamma_cgamma);

  /* "scipy/special/_ufuncs.pyx":1612
 * from ._convex_analysis cimport huber as _func_huber
 * ctypedef double _proto_huber_t(double, double) nogil
 * cdef _proto_huber_t *_proto_huber_t_var = &_func_huber             # <<<<<<<<<<<<<<
 * from ._hyp0f1 cimport _hyp0f1_cmplx as _func__hyp0f1_cmplx
 * ctypedef double complex _proto__hyp0f1_cmplx_t(double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_huber_t_var = (&__pyx_f_5scipy_7special_16_convex_analysis_huber);

  /* "scipy/special/_ufuncs.pyx":1615
 * from ._hyp0f1 cimport _hyp0f1_cmplx as _func__hyp0f1_cmplx
 * ctypedef double complex _proto__hyp0f1_cmplx_t(double, double complex) nogil
 * cdef _proto__hyp0f1_cmplx_t *_proto__hyp0f1_cmplx_t_var = &_func__hyp0f1_cmplx             # <<<<<<<<<<<<<<
 * from ._hyp0f1 cimport _hyp0f1_real as _func__hyp0f1_real
 * ctypedef double _proto__hyp0f1_real_t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto__hyp0f1_cmplx_t_var = (&__pyx_f_5scipy_7special_7_hyp0f1__hyp0f1_cmplx);

  /* "scipy/special/_ufuncs.pyx":1618
 * from ._hyp0f1 cimport _hyp0f1_real as _func__hyp0f1_real
 * ctypedef double _proto__hyp0f1_real_t(double, double) nogil
 * cdef _proto__hyp0f1_real_t *_proto__hyp0f1_real_t_var = &_func__hyp0f1_real             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double complex _func_chyp1f1_wrap "chyp1f1_wrap"(double, double, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto__hyp0f1_real_t_var = (&__pyx_f_5scipy_7special_7_hyp0f1__hyp0f1_real);

  /* "scipy/special/_ufuncs.pyx":1627
 * from ._legacy cimport hyp2f0_unsafe as _func_hyp2f0_unsafe
 * ctypedef double _proto_hyp2f0_unsafe_t(double, double, double, double, double *) nogil
 * cdef _proto_hyp2f0_unsafe_t *_proto_hyp2f0_unsafe_t_var = &_func_hyp2f0_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_hyp2f0 "hyp2f0"(double, double, double, int, double *) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_hyp2f0_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_hyp2f0_unsafe);

  /* "scipy/special/_ufuncs.pyx":1648
 * from ._boxcox cimport inv_boxcox as _func_inv_boxcox
 * ctypedef double _proto_inv_boxcox_t(double, double) nogil
 * cdef _proto_inv_boxcox_t *_proto_inv_boxcox_t_var = &_func_inv_boxcox             # <<<<<<<<<<<<<<
 * from ._boxcox cimport inv_boxcox1p as _func_inv_boxcox1p
 * ctypedef double _proto_inv_boxcox1p_t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_inv_boxcox_t_var = (&__pyx_f_5scipy_7special_7_boxcox_inv_boxcox);

  /* "scipy/special/_ufuncs.pyx":1651
 * from ._boxcox cimport inv_boxcox1p as _func_inv_boxcox1p
 * ctypedef double _proto_inv_boxcox1p_t(double, double) nogil
 * cdef _proto_inv_boxcox1p_t *_proto_inv_boxcox1p_t_var = &_func_inv_boxcox1p             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef int _func_it2i0k0_wrap "it2i0k0_wrap"(double, double *, double *) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_inv_boxcox1p_t_var = (&__pyx_f_5scipy_7special_7_boxcox_inv_boxcox1p);

  /* "scipy/special/_ufuncs.pyx":1708
 * from ._convex_analysis cimport kl_div as _func_kl_div
 * ctypedef double _proto_kl_div_t(double, double) nogil
 * cdef _proto_kl_div_t *_proto_kl_div_t_var = &_func_kl_div             # <<<<<<<<<<<<<<
 * from ._legacy cimport kn_unsafe as _func_kn_unsafe
 * ctypedef double _proto_kn_unsafe_t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_kl_div_t_var = (&__pyx_f_5scipy_7special_16_convex_analysis_kl_div);

  /* "scipy/special/_ufuncs.pyx":1711
 * from ._legacy cimport kn_unsafe as _func_kn_unsafe
 * ctypedef double _proto_kn_unsafe_t(double, double) nogil
 * cdef _proto_kn_unsafe_t *_proto_kn_unsafe_t_var = &_func_kn_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_cbesk_wrap_real_int "cbesk_wrap_real_int"(int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_kn_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_kn_unsafe);

  /* "scipy/special/_ufuncs.pyx":1728
 * from ._cunity cimport clog1p as _func_clog1p
 * ctypedef double complex _proto_clog1p_t(double complex) nogil
 * cdef _proto_clog1p_t *_proto_clog1p_t_var = &_func_clog1p             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_log1p "log1p"(double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_clog1p_t_var = (&__pyx_f_5scipy_7special_7_cunity_clog1p);

  /* "scipy/special/_ufuncs.pyx":1735
 * from ._loggamma cimport loggamma_real as _func_loggamma_real
 * ctypedef double _proto_loggamma_real_t(double) nogil
 * cdef _proto_loggamma_real_t *_proto_loggamma_real_t_var = &_func_loggamma_real             # <<<<<<<<<<<<<<
 * from ._loggamma cimport loggamma as _func_loggamma
 * ctypedef double complex _proto_loggamma_t(double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_loggamma_real_t_var = (&__pyx_f_5scipy_7special_9_loggamma_loggamma_real);

  /* "scipy/special/_ufuncs.pyx":1738
 * from ._loggamma cimport loggamma as _func_loggamma
 * ctypedef double complex _proto_loggamma_t(double complex) nogil
 * cdef _proto_loggamma_t *_proto_loggamma_t_var = &_func_loggamma             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_logit "logit"(double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_loggamma_t_var = (&__pyx_f_5scipy_7special_9_loggamma_loggamma);

  /* "scipy/special/_ufuncs.pyx":1771
 * from ._legacy cimport nbdtr_unsafe as _func_nbdtr_unsafe
 * ctypedef double _proto_nbdtr_unsafe_t(double, double, double) nogil
 * cdef _proto_nbdtr_unsafe_t *_proto_nbdtr_unsafe_t_var = &_func_nbdtr_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_nbdtr "nbdtr"(int, int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_nbdtr_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_nbdtr_unsafe);

  /* "scipy/special/_ufuncs.pyx":1776
 * from ._legacy cimport nbdtrc_unsafe as _func_nbdtrc_unsafe
 * ctypedef double _proto_nbdtrc_unsafe_t(double, double, double) nogil
 * cdef _proto_nbdtrc_unsafe_t *_proto_nbdtrc_unsafe_t_var = &_func_nbdtrc_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_nbdtrc "nbdtrc"(int, int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_nbdtrc_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_nbdtrc_unsafe);

  /* "scipy/special/_ufuncs.pyx":1781
 * from ._legacy cimport nbdtri_unsafe as _func_nbdtri_unsafe
 * ctypedef double _proto_nbdtri_unsafe_t(double, double, double) nogil
 * cdef _proto_nbdtri_unsafe_t *_proto_nbdtri_unsafe_t_var = &_func_nbdtri_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_nbdtri "nbdtri"(int, int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_nbdtri_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_nbdtri_unsafe);

  /* "scipy/special/_ufuncs.pyx":1838
 * from ._legacy cimport pdtr_unsafe as _func_pdtr_unsafe
 * ctypedef double _proto_pdtr_unsafe_t(double, double) nogil
 * cdef _proto_pdtr_unsafe_t *_proto_pdtr_unsafe_t_var = &_func_pdtr_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_pdtr "pdtr"(int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_pdtr_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_pdtr_unsafe);

  /* "scipy/special/_ufuncs.pyx":1843
 * from ._legacy cimport pdtrc_unsafe as _func_pdtrc_unsafe
 * ctypedef double _proto_pdtrc_unsafe_t(double, double) nogil
 * cdef _proto_pdtrc_unsafe_t *_proto_pdtrc_unsafe_t_var = &_func_pdtrc_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_pdtrc "pdtrc"(int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_pdtrc_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_pdtrc_unsafe);

  /* "scipy/special/_ufuncs.pyx":1848
 * from ._legacy cimport pdtri_unsafe as _func_pdtri_unsafe
 * ctypedef double _proto_pdtri_unsafe_t(double, double) nogil
 * cdef _proto_pdtri_unsafe_t *_proto_pdtri_unsafe_t_var = &_func_pdtri_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_pdtri "pdtri"(int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_pdtri_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_pdtri_unsafe);

  /* "scipy/special/_ufuncs.pyx":1871
 * from ._convex_analysis cimport pseudo_huber as _func_pseudo_huber
 * ctypedef double _proto_pseudo_huber_t(double, double) nogil
 * cdef _proto_pseudo_huber_t *_proto_pseudo_huber_t_var = &_func_pseudo_huber             # <<<<<<<<<<<<<<
 * from ._digamma cimport cdigamma as _func_cdigamma
 * ctypedef double complex _proto_cdigamma_t(double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_pseudo_huber_t_var = (&__pyx_f_5scipy_7special_16_convex_analysis_pseudo_huber);

  /* "scipy/special/_ufuncs.pyx":1874
 * from ._digamma cimport cdigamma as _func_cdigamma
 * ctypedef double complex _proto_cdigamma_t(double complex) nogil
 * cdef _proto_cdigamma_t *_proto_cdigamma_t_var = &_func_cdigamma             # <<<<<<<<<<<<<<
 * from ._digamma cimport digamma as _func_digamma
 * ctypedef double _proto_digamma_t(double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_cdigamma_t_var = (&__pyx_f_5scipy_7special_8_digamma_cdigamma);

  /* "scipy/special/_ufuncs.pyx":1877
 * from ._digamma cimport digamma as _func_digamma
 * ctypedef double _proto_digamma_t(double) nogil
 * cdef _proto_digamma_t *_proto_digamma_t_var = &_func_digamma             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_radian "radian"(double, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_digamma_t_var = (&__pyx_f_5scipy_7special_8_digamma_digamma);

  /* "scipy/special/_ufuncs.pyx":1882
 * from ._convex_analysis cimport rel_entr as _func_rel_entr
 * ctypedef double _proto_rel_entr_t(double, double) nogil
 * cdef _proto_rel_entr_t *_proto_rel_entr_t_var = &_func_rel_entr             # <<<<<<<<<<<<<<
 * from ._loggamma cimport crgamma as _func_crgamma
 * ctypedef double complex _proto_crgamma_t(double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_rel_entr_t_var = (&__pyx_f_5scipy_7special_16_convex_analysis_rel_entr);

  /* "scipy/special/_ufuncs.pyx":1885
 * from ._loggamma cimport crgamma as _func_crgamma
 * ctypedef double complex _proto_crgamma_t(double complex) nogil
 * cdef _proto_crgamma_t *_proto_crgamma_t_var = &_func_crgamma             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_rgamma "rgamma"(double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_crgamma_t_var = (&__pyx_f_5scipy_7special_9_loggamma_crgamma);

  /* "scipy/special/_ufuncs.pyx":1892
 * from ._sici cimport cshichi as _func_cshichi
 * ctypedef int _proto_cshichi_t(double complex, double complex *, double complex *) nogil
 * cdef _proto_cshichi_t *_proto_cshichi_t_var = &_func_cshichi             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef int _func_shichi "shichi"(double, double *, double *) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_cshichi_t_var = (&__pyx_f_5scipy_7special_5_sici_cshichi);

  /* "scipy/special/_ufuncs.pyx":1897
 * from ._sici cimport csici as _func_csici
 * ctypedef int _proto_csici_t(double complex, double complex *, double complex *) nogil
 * cdef _proto_csici_t *_proto_csici_t_var = &_func_csici             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef int _func_sici "sici"(double, double *, double *) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_csici_t_var = (&__pyx_f_5scipy_7special_5_sici_csici);

  /* "scipy/special/_ufuncs.pyx":1904
 * from ._legacy cimport smirnov_unsafe as _func_smirnov_unsafe
 * ctypedef double _proto_smirnov_unsafe_t(double, double) nogil
 * cdef _proto_smirnov_unsafe_t *_proto_smirnov_unsafe_t_var = &_func_smirnov_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_smirnov "smirnov"(int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_smirnov_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_smirnov_unsafe);

  /* "scipy/special/_ufuncs.pyx":1909
 * from ._legacy cimport smirnovi_unsafe as _func_smirnovi_unsafe
 * ctypedef double _proto_smirnovi_unsafe_t(double, double) nogil
 * cdef _proto_smirnovi_unsafe_t *_proto_smirnovi_unsafe_t_var = &_func_smirnovi_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_smirnovi "smirnovi"(int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_smirnovi_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_smirnovi_unsafe);

  /* "scipy/special/_ufuncs.pyx":1914
 * from ._spence cimport cspence as _func_cspence
 * ctypedef double complex _proto_cspence_t(double complex) nogil
 * cdef _proto_cspence_t *_proto_cspence_t_var = &_func_cspence             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_spence "spence"(double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_cspence_t_var = (&__pyx_f_5scipy_7special_7_spence_cspence);

  /* "scipy/special/_ufuncs.pyx":1919
 * from ._legacy cimport sph_harmonic_unsafe as _func_sph_harmonic_unsafe
 * ctypedef double complex _proto_sph_harmonic_unsafe_t(double, double, double, double) nogil
 * cdef _proto_sph_harmonic_unsafe_t *_proto_sph_harmonic_unsafe_t_var = &_func_sph_harmonic_unsafe             # <<<<<<<<<<<<<<
 * from .sph_harm cimport sph_harmonic as _func_sph_harmonic
 * ctypedef double complex _proto_sph_harmonic_t(int, int, double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_sph_harmonic_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_sph_harmonic_unsafe);

  /* "scipy/special/_ufuncs.pyx":1922
 * from .sph_harm cimport sph_harmonic as _func_sph_harmonic
 * ctypedef double complex _proto_sph_harmonic_t(int, int, double, double) nogil
 * cdef _proto_sph_harmonic_t *_proto_sph_harmonic_t_var = &_func_sph_harmonic             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_cdft1_wrap "cdft1_wrap"(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_sph_harmonic_t_var = (&__pyx_f_5scipy_7special_8sph_harm_sph_harmonic);

  /* "scipy/special/_ufuncs.pyx":1937
 * from ._xlogy cimport xlog1py as _func_xlog1py
 * ctypedef double _proto_xlog1py_double__t(double, double) nogil
 * cdef _proto_xlog1py_double__t *_proto_xlog1py_double__t_var = &_func_xlog1py[double]             # <<<<<<<<<<<<<<
 * from ._xlogy cimport xlog1py as _func_xlog1py
 * ctypedef double complex _proto_xlog1py_double_complex__t(double complex, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_xlog1py_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_6_xlogy_xlog1py);

  /* "scipy/special/_ufuncs.pyx":1940
 * from ._xlogy cimport xlog1py as _func_xlog1py
 * ctypedef double complex _proto_xlog1py_double_complex__t(double complex, double complex) nogil
 * cdef _proto_xlog1py_double_complex__t *_proto_xlog1py_double_complex__t_var = &_func_xlog1py[double_complex]             # <<<<<<<<<<<<<<
 * from ._xlogy cimport xlogy as _func_xlogy
 * ctypedef double _proto_xlogy_double__t(double, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_xlog1py_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_6_xlogy_xlog1py);

  /* "scipy/special/_ufuncs.pyx":1943
 * from ._xlogy cimport xlogy as _func_xlogy
 * ctypedef double _proto_xlogy_double__t(double, double) nogil
 * cdef _proto_xlogy_double__t *_proto_xlogy_double__t_var = &_func_xlogy[double]             # <<<<<<<<<<<<<<
 * from ._xlogy cimport xlogy as _func_xlogy
 * ctypedef double complex _proto_xlogy_double_complex__t(double complex, double complex) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_xlogy_double__t_var = (&__pyx_fuse_0__pyx_f_5scipy_7special_6_xlogy_xlogy);

  /* "scipy/special/_ufuncs.pyx":1946
 * from ._xlogy cimport xlogy as _func_xlogy
 * ctypedef double complex _proto_xlogy_double_complex__t(double complex, double complex) nogil
 * cdef _proto_xlogy_double_complex__t *_proto_xlogy_double_complex__t_var = &_func_xlogy[double_complex]             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_y0 "y0"(double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_xlogy_double_complex__t_var = (&__pyx_fuse_1__pyx_f_5scipy_7special_6_xlogy_xlogy);

  /* "scipy/special/_ufuncs.pyx":1953
 * from ._legacy cimport yn_unsafe as _func_yn_unsafe
 * ctypedef double _proto_yn_unsafe_t(double, double) nogil
 * cdef _proto_yn_unsafe_t *_proto_yn_unsafe_t_var = &_func_yn_unsafe             # <<<<<<<<<<<<<<
 * cdef extern from "_ufuncs_defs.h":
 *     cdef double _func_yn "yn"(int, double) nogil
 */
  __pyx_v_5scipy_7special_7_ufuncs__proto_yn_unsafe_t_var = (&__pyx_f_5scipy_7special_7_legacy_yn_unsafe);

  /* "scipy/special/_ufuncs.pyx":1970
 * cdef void *ufunc__cospi_data[4]
 * cdef char ufunc__cospi_types[8]
 * cdef char *ufunc__cospi_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, do not use.")
 * ufunc__cospi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_doc = ((char *)"Internal function, do not use.");

  /* "scipy/special/_ufuncs.pyx":1972
 * cdef char *ufunc__cospi_doc = (
 *     "Internal function, do not use.")
 * ufunc__cospi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc__cospi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__cospi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":1973
 *     "Internal function, do not use.")
 * ufunc__cospi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc__cospi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc__cospi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc__cospi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":1974
 * ufunc__cospi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc__cospi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__cospi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc__cospi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc__cospi_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":1975
 * ufunc__cospi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__cospi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc__cospi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc__cospi_types[0] = <char>NPY_FLOAT
 * ufunc__cospi_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":1976
 * ufunc__cospi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc__cospi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc__cospi_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__cospi_types[1] = <char>NPY_FLOAT
 * ufunc__cospi_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":1977
 * ufunc__cospi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc__cospi_types[0] = <char>NPY_FLOAT
 * ufunc__cospi_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__cospi_types[2] = <char>NPY_DOUBLE
 * ufunc__cospi_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":1978
 * ufunc__cospi_types[0] = <char>NPY_FLOAT
 * ufunc__cospi_types[1] = <char>NPY_FLOAT
 * ufunc__cospi_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__cospi_types[3] = <char>NPY_DOUBLE
 * ufunc__cospi_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1979
 * ufunc__cospi_types[1] = <char>NPY_FLOAT
 * ufunc__cospi_types[2] = <char>NPY_DOUBLE
 * ufunc__cospi_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__cospi_types[4] = <char>NPY_CFLOAT
 * ufunc__cospi_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":1980
 * ufunc__cospi_types[2] = <char>NPY_DOUBLE
 * ufunc__cospi_types[3] = <char>NPY_DOUBLE
 * ufunc__cospi_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc__cospi_types[5] = <char>NPY_CFLOAT
 * ufunc__cospi_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":1981
 * ufunc__cospi_types[3] = <char>NPY_DOUBLE
 * ufunc__cospi_types[4] = <char>NPY_CFLOAT
 * ufunc__cospi_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc__cospi_types[6] = <char>NPY_CDOUBLE
 * ufunc__cospi_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":1982
 * ufunc__cospi_types[4] = <char>NPY_CFLOAT
 * ufunc__cospi_types[5] = <char>NPY_CFLOAT
 * ufunc__cospi_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__cospi_types[7] = <char>NPY_CDOUBLE
 * ufunc__cospi_ptr[2*0] = <void*>_func_dcospi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":1983
 * ufunc__cospi_types[5] = <char>NPY_CFLOAT
 * ufunc__cospi_types[6] = <char>NPY_CDOUBLE
 * ufunc__cospi_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__cospi_ptr[2*0] = <void*>_func_dcospi
 * ufunc__cospi_ptr[2*0+1] = <void*>(<char*>"_cospi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":1984
 * ufunc__cospi_types[6] = <char>NPY_CDOUBLE
 * ufunc__cospi_types[7] = <char>NPY_CDOUBLE
 * ufunc__cospi_ptr[2*0] = <void*>_func_dcospi             # <<<<<<<<<<<<<<
 * ufunc__cospi_ptr[2*0+1] = <void*>(<char*>"_cospi")
 * ufunc__cospi_ptr[2*1] = <void*>_func_dcospi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_ptr[0]) = ((void *)__pyx_f_5scipy_7special_5_trig_dcospi);

  /* "scipy/special/_ufuncs.pyx":1985
 * ufunc__cospi_types[7] = <char>NPY_CDOUBLE
 * ufunc__cospi_ptr[2*0] = <void*>_func_dcospi
 * ufunc__cospi_ptr[2*0+1] = <void*>(<char*>"_cospi")             # <<<<<<<<<<<<<<
 * ufunc__cospi_ptr[2*1] = <void*>_func_dcospi
 * ufunc__cospi_ptr[2*1+1] = <void*>(<char*>"_cospi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_ptr[1]) = ((void *)((char *)((char *)"_cospi")));

  /* "scipy/special/_ufuncs.pyx":1986
 * ufunc__cospi_ptr[2*0] = <void*>_func_dcospi
 * ufunc__cospi_ptr[2*0+1] = <void*>(<char*>"_cospi")
 * ufunc__cospi_ptr[2*1] = <void*>_func_dcospi             # <<<<<<<<<<<<<<
 * ufunc__cospi_ptr[2*1+1] = <void*>(<char*>"_cospi")
 * ufunc__cospi_ptr[2*2] = <void*>_func_ccospi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_ptr[2]) = ((void *)__pyx_f_5scipy_7special_5_trig_dcospi);

  /* "scipy/special/_ufuncs.pyx":1987
 * ufunc__cospi_ptr[2*0+1] = <void*>(<char*>"_cospi")
 * ufunc__cospi_ptr[2*1] = <void*>_func_dcospi
 * ufunc__cospi_ptr[2*1+1] = <void*>(<char*>"_cospi")             # <<<<<<<<<<<<<<
 * ufunc__cospi_ptr[2*2] = <void*>_func_ccospi
 * ufunc__cospi_ptr[2*2+1] = <void*>(<char*>"_cospi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_ptr[3]) = ((void *)((char *)((char *)"_cospi")));

  /* "scipy/special/_ufuncs.pyx":1988
 * ufunc__cospi_ptr[2*1] = <void*>_func_dcospi
 * ufunc__cospi_ptr[2*1+1] = <void*>(<char*>"_cospi")
 * ufunc__cospi_ptr[2*2] = <void*>_func_ccospi             # <<<<<<<<<<<<<<
 * ufunc__cospi_ptr[2*2+1] = <void*>(<char*>"_cospi")
 * ufunc__cospi_ptr[2*3] = <void*>_func_ccospi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_ptr[4]) = ((void *)__pyx_f_5scipy_7special_5_trig_ccospi);

  /* "scipy/special/_ufuncs.pyx":1989
 * ufunc__cospi_ptr[2*1+1] = <void*>(<char*>"_cospi")
 * ufunc__cospi_ptr[2*2] = <void*>_func_ccospi
 * ufunc__cospi_ptr[2*2+1] = <void*>(<char*>"_cospi")             # <<<<<<<<<<<<<<
 * ufunc__cospi_ptr[2*3] = <void*>_func_ccospi
 * ufunc__cospi_ptr[2*3+1] = <void*>(<char*>"_cospi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_ptr[5]) = ((void *)((char *)((char *)"_cospi")));

  /* "scipy/special/_ufuncs.pyx":1990
 * ufunc__cospi_ptr[2*2] = <void*>_func_ccospi
 * ufunc__cospi_ptr[2*2+1] = <void*>(<char*>"_cospi")
 * ufunc__cospi_ptr[2*3] = <void*>_func_ccospi             # <<<<<<<<<<<<<<
 * ufunc__cospi_ptr[2*3+1] = <void*>(<char*>"_cospi")
 * ufunc__cospi_data[0] = &ufunc__cospi_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_ptr[6]) = ((void *)__pyx_f_5scipy_7special_5_trig_ccospi);

  /* "scipy/special/_ufuncs.pyx":1991
 * ufunc__cospi_ptr[2*2+1] = <void*>(<char*>"_cospi")
 * ufunc__cospi_ptr[2*3] = <void*>_func_ccospi
 * ufunc__cospi_ptr[2*3+1] = <void*>(<char*>"_cospi")             # <<<<<<<<<<<<<<
 * ufunc__cospi_data[0] = &ufunc__cospi_ptr[2*0]
 * ufunc__cospi_data[1] = &ufunc__cospi_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_ptr[7]) = ((void *)((char *)((char *)"_cospi")));

  /* "scipy/special/_ufuncs.pyx":1992
 * ufunc__cospi_ptr[2*3] = <void*>_func_ccospi
 * ufunc__cospi_ptr[2*3+1] = <void*>(<char*>"_cospi")
 * ufunc__cospi_data[0] = &ufunc__cospi_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__cospi_data[1] = &ufunc__cospi_ptr[2*1]
 * ufunc__cospi_data[2] = &ufunc__cospi_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":1993
 * ufunc__cospi_ptr[2*3+1] = <void*>(<char*>"_cospi")
 * ufunc__cospi_data[0] = &ufunc__cospi_ptr[2*0]
 * ufunc__cospi_data[1] = &ufunc__cospi_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc__cospi_data[2] = &ufunc__cospi_ptr[2*2]
 * ufunc__cospi_data[3] = &ufunc__cospi_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":1994
 * ufunc__cospi_data[0] = &ufunc__cospi_ptr[2*0]
 * ufunc__cospi_data[1] = &ufunc__cospi_ptr[2*1]
 * ufunc__cospi_data[2] = &ufunc__cospi_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc__cospi_data[3] = &ufunc__cospi_ptr[2*3]
 * _cospi = np.PyUFunc_FromFuncAndData(ufunc__cospi_loops, ufunc__cospi_data, ufunc__cospi_types, 4, 1, 1, 0, "_cospi", ufunc__cospi_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":1995
 * ufunc__cospi_data[1] = &ufunc__cospi_ptr[2*1]
 * ufunc__cospi_data[2] = &ufunc__cospi_ptr[2*2]
 * ufunc__cospi_data[3] = &ufunc__cospi_ptr[2*3]             # <<<<<<<<<<<<<<
 * _cospi = np.PyUFunc_FromFuncAndData(ufunc__cospi_loops, ufunc__cospi_data, ufunc__cospi_types, 4, 1, 1, 0, "_cospi", ufunc__cospi_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":1996
 * ufunc__cospi_data[2] = &ufunc__cospi_ptr[2*2]
 * ufunc__cospi_data[3] = &ufunc__cospi_ptr[2*3]
 * _cospi = np.PyUFunc_FromFuncAndData(ufunc__cospi_loops, ufunc__cospi_data, ufunc__cospi_types, 4, 1, 1, 0, "_cospi", ufunc__cospi_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__ellip_harm_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_types, 4, 1, 1, 0, ((char *)"_cospi"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__cospi_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1996, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_cospi, __pyx_t_2) < 0) __PYX_ERR(1, 1996, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2002
 * cdef void *ufunc__ellip_harm_data[3]
 * cdef char ufunc__ellip_harm_types[24]
 * cdef char *ufunc__ellip_harm_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, use `ellip_harm` instead.")
 * ufunc__ellip_harm_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddddddd__As_fffffff_f
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_doc = ((char *)"Internal function, use `ellip_harm` instead.");

  /* "scipy/special/_ufuncs.pyx":2004
 * cdef char *ufunc__ellip_harm_doc = (
 *     "Internal function, use `ellip_harm` instead.")
 * ufunc__ellip_harm_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddddddd__As_fffffff_f             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddiiddd__As_ddllddd_d
 * ufunc__ellip_harm_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddddddd__As_ddddddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddddddd__As_fffffff_f);

  /* "scipy/special/_ufuncs.pyx":2005
 *     "Internal function, use `ellip_harm` instead.")
 * ufunc__ellip_harm_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddddddd__As_fffffff_f
 * ufunc__ellip_harm_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddiiddd__As_ddllddd_d             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddddddd__As_ddddddd_d
 * ufunc__ellip_harm_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddiiddd__As_ddllddd_d);

  /* "scipy/special/_ufuncs.pyx":2006
 * ufunc__ellip_harm_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddddddd__As_fffffff_f
 * ufunc__ellip_harm_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddiiddd__As_ddllddd_d
 * ufunc__ellip_harm_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddddddd__As_ddddddd_d             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[0] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddddddd__As_ddddddd_d);

  /* "scipy/special/_ufuncs.pyx":2007
 * ufunc__ellip_harm_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddiiddd__As_ddllddd_d
 * ufunc__ellip_harm_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddddddd__As_ddddddd_d
 * ufunc__ellip_harm_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[1] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2008
 * ufunc__ellip_harm_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddddddd__As_ddddddd_d
 * ufunc__ellip_harm_types[0] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[2] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2009
 * ufunc__ellip_harm_types[0] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[1] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[3] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2010
 * ufunc__ellip_harm_types[1] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[2] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[4] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2011
 * ufunc__ellip_harm_types[2] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[3] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[5] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2012
 * ufunc__ellip_harm_types[3] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[4] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[6] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2013
 * ufunc__ellip_harm_types[4] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[5] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[7] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2014
 * ufunc__ellip_harm_types[5] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[6] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[8] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2015
 * ufunc__ellip_harm_types[6] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[7] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[9] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[10] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2016
 * ufunc__ellip_harm_types[7] = <char>NPY_FLOAT
 * ufunc__ellip_harm_types[8] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[10] = <char>NPY_LONG
 * ufunc__ellip_harm_types[11] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2017
 * ufunc__ellip_harm_types[8] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[9] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[10] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[11] = <char>NPY_LONG
 * ufunc__ellip_harm_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[10]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2018
 * ufunc__ellip_harm_types[9] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[10] = <char>NPY_LONG
 * ufunc__ellip_harm_types[11] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[12] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[11]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2019
 * ufunc__ellip_harm_types[10] = <char>NPY_LONG
 * ufunc__ellip_harm_types[11] = <char>NPY_LONG
 * ufunc__ellip_harm_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[13] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[14] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2020
 * ufunc__ellip_harm_types[11] = <char>NPY_LONG
 * ufunc__ellip_harm_types[12] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[14] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[15] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2021
 * ufunc__ellip_harm_types[12] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[13] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[14] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[15] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[16] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[14]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2022
 * ufunc__ellip_harm_types[13] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[14] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[15] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[16] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[17] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[15]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2023
 * ufunc__ellip_harm_types[14] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[15] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[16] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[17] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[18] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[16]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2024
 * ufunc__ellip_harm_types[15] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[16] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[17] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[18] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[19] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[17]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2025
 * ufunc__ellip_harm_types[16] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[17] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[18] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[19] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[20] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[18]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2026
 * ufunc__ellip_harm_types[17] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[18] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[19] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[20] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[21] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[19]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2027
 * ufunc__ellip_harm_types[18] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[19] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[20] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[21] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[22] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[20]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2028
 * ufunc__ellip_harm_types[19] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[20] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[21] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[22] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[23] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[21]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2029
 * ufunc__ellip_harm_types[20] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[21] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[22] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_types[23] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_ptr[2*0] = <void*>_func_ellip_harmonic_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[22]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2030
 * ufunc__ellip_harm_types[21] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[22] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[23] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_ptr[2*0] = <void*>_func_ellip_harmonic_unsafe
 * ufunc__ellip_harm_ptr[2*0+1] = <void*>(<char*>"_ellip_harm")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types[23]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2031
 * ufunc__ellip_harm_types[22] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_types[23] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_ptr[2*0] = <void*>_func_ellip_harmonic_unsafe             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_ptr[2*0+1] = <void*>(<char*>"_ellip_harm")
 * ufunc__ellip_harm_ptr[2*1] = <void*>_func_ellip_harmonic
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_ptr[0]) = ((void *)__pyx_f_5scipy_7special_7_legacy_ellip_harmonic_unsafe);

  /* "scipy/special/_ufuncs.pyx":2032
 * ufunc__ellip_harm_types[23] = <char>NPY_DOUBLE
 * ufunc__ellip_harm_ptr[2*0] = <void*>_func_ellip_harmonic_unsafe
 * ufunc__ellip_harm_ptr[2*0+1] = <void*>(<char*>"_ellip_harm")             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_ptr[2*1] = <void*>_func_ellip_harmonic
 * ufunc__ellip_harm_ptr[2*1+1] = <void*>(<char*>"_ellip_harm")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_ptr[1]) = ((void *)((char *)((char *)"_ellip_harm")));

  /* "scipy/special/_ufuncs.pyx":2033
 * ufunc__ellip_harm_ptr[2*0] = <void*>_func_ellip_harmonic_unsafe
 * ufunc__ellip_harm_ptr[2*0+1] = <void*>(<char*>"_ellip_harm")
 * ufunc__ellip_harm_ptr[2*1] = <void*>_func_ellip_harmonic             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_ptr[2*1+1] = <void*>(<char*>"_ellip_harm")
 * ufunc__ellip_harm_ptr[2*2] = <void*>_func_ellip_harmonic_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_ptr[2]) = ((void *)__pyx_f_5scipy_7special_11_ellip_harm_ellip_harmonic);

  /* "scipy/special/_ufuncs.pyx":2034
 * ufunc__ellip_harm_ptr[2*0+1] = <void*>(<char*>"_ellip_harm")
 * ufunc__ellip_harm_ptr[2*1] = <void*>_func_ellip_harmonic
 * ufunc__ellip_harm_ptr[2*1+1] = <void*>(<char*>"_ellip_harm")             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_ptr[2*2] = <void*>_func_ellip_harmonic_unsafe
 * ufunc__ellip_harm_ptr[2*2+1] = <void*>(<char*>"_ellip_harm")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_ptr[3]) = ((void *)((char *)((char *)"_ellip_harm")));

  /* "scipy/special/_ufuncs.pyx":2035
 * ufunc__ellip_harm_ptr[2*1] = <void*>_func_ellip_harmonic
 * ufunc__ellip_harm_ptr[2*1+1] = <void*>(<char*>"_ellip_harm")
 * ufunc__ellip_harm_ptr[2*2] = <void*>_func_ellip_harmonic_unsafe             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_ptr[2*2+1] = <void*>(<char*>"_ellip_harm")
 * ufunc__ellip_harm_data[0] = &ufunc__ellip_harm_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_ellip_harmonic_unsafe);

  /* "scipy/special/_ufuncs.pyx":2036
 * ufunc__ellip_harm_ptr[2*1+1] = <void*>(<char*>"_ellip_harm")
 * ufunc__ellip_harm_ptr[2*2] = <void*>_func_ellip_harmonic_unsafe
 * ufunc__ellip_harm_ptr[2*2+1] = <void*>(<char*>"_ellip_harm")             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_data[0] = &ufunc__ellip_harm_ptr[2*0]
 * ufunc__ellip_harm_data[1] = &ufunc__ellip_harm_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_ptr[5]) = ((void *)((char *)((char *)"_ellip_harm")));

  /* "scipy/special/_ufuncs.pyx":2037
 * ufunc__ellip_harm_ptr[2*2] = <void*>_func_ellip_harmonic_unsafe
 * ufunc__ellip_harm_ptr[2*2+1] = <void*>(<char*>"_ellip_harm")
 * ufunc__ellip_harm_data[0] = &ufunc__ellip_harm_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_data[1] = &ufunc__ellip_harm_ptr[2*1]
 * ufunc__ellip_harm_data[2] = &ufunc__ellip_harm_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2038
 * ufunc__ellip_harm_ptr[2*2+1] = <void*>(<char*>"_ellip_harm")
 * ufunc__ellip_harm_data[0] = &ufunc__ellip_harm_ptr[2*0]
 * ufunc__ellip_harm_data[1] = &ufunc__ellip_harm_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc__ellip_harm_data[2] = &ufunc__ellip_harm_ptr[2*2]
 * _ellip_harm = np.PyUFunc_FromFuncAndData(ufunc__ellip_harm_loops, ufunc__ellip_harm_data, ufunc__ellip_harm_types, 3, 7, 1, 0, "_ellip_harm", ufunc__ellip_harm_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2039
 * ufunc__ellip_harm_data[0] = &ufunc__ellip_harm_ptr[2*0]
 * ufunc__ellip_harm_data[1] = &ufunc__ellip_harm_ptr[2*1]
 * ufunc__ellip_harm_data[2] = &ufunc__ellip_harm_ptr[2*2]             # <<<<<<<<<<<<<<
 * _ellip_harm = np.PyUFunc_FromFuncAndData(ufunc__ellip_harm_loops, ufunc__ellip_harm_data, ufunc__ellip_harm_types, 3, 7, 1, 0, "_ellip_harm", ufunc__ellip_harm_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":2040
 * ufunc__ellip_harm_data[1] = &ufunc__ellip_harm_ptr[2*1]
 * ufunc__ellip_harm_data[2] = &ufunc__ellip_harm_ptr[2*2]
 * _ellip_harm = np.PyUFunc_FromFuncAndData(ufunc__ellip_harm_loops, ufunc__ellip_harm_data, ufunc__ellip_harm_types, 3, 7, 1, 0, "_ellip_harm", ufunc__ellip_harm_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__igam_fac_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_types, 3, 7, 1, 0, ((char *)"_ellip_harm"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__ellip_harm_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2040, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ellip_harm, __pyx_t_2) < 0) __PYX_ERR(1, 2040, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2046
 * cdef void *ufunc__igam_fac_data[2]
 * cdef char ufunc__igam_fac_types[6]
 * cdef char *ufunc__igam_fac_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, do not use.")
 * ufunc__igam_fac_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_doc = ((char *)"Internal function, do not use.");

  /* "scipy/special/_ufuncs.pyx":2048
 * cdef char *ufunc__igam_fac_doc = (
 *     "Internal function, do not use.")
 * ufunc__igam_fac_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc__igam_fac_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc__igam_fac_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":2049
 *     "Internal function, do not use.")
 * ufunc__igam_fac_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc__igam_fac_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc__igam_fac_types[0] = <char>NPY_FLOAT
 * ufunc__igam_fac_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":2050
 * ufunc__igam_fac_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc__igam_fac_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc__igam_fac_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__igam_fac_types[1] = <char>NPY_FLOAT
 * ufunc__igam_fac_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2051
 * ufunc__igam_fac_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc__igam_fac_types[0] = <char>NPY_FLOAT
 * ufunc__igam_fac_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__igam_fac_types[2] = <char>NPY_FLOAT
 * ufunc__igam_fac_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2052
 * ufunc__igam_fac_types[0] = <char>NPY_FLOAT
 * ufunc__igam_fac_types[1] = <char>NPY_FLOAT
 * ufunc__igam_fac_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__igam_fac_types[3] = <char>NPY_DOUBLE
 * ufunc__igam_fac_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2053
 * ufunc__igam_fac_types[1] = <char>NPY_FLOAT
 * ufunc__igam_fac_types[2] = <char>NPY_FLOAT
 * ufunc__igam_fac_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__igam_fac_types[4] = <char>NPY_DOUBLE
 * ufunc__igam_fac_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2054
 * ufunc__igam_fac_types[2] = <char>NPY_FLOAT
 * ufunc__igam_fac_types[3] = <char>NPY_DOUBLE
 * ufunc__igam_fac_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__igam_fac_types[5] = <char>NPY_DOUBLE
 * ufunc__igam_fac_ptr[2*0] = <void*>_func_igam_fac
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2055
 * ufunc__igam_fac_types[3] = <char>NPY_DOUBLE
 * ufunc__igam_fac_types[4] = <char>NPY_DOUBLE
 * ufunc__igam_fac_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__igam_fac_ptr[2*0] = <void*>_func_igam_fac
 * ufunc__igam_fac_ptr[2*0+1] = <void*>(<char*>"_igam_fac")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2056
 * ufunc__igam_fac_types[4] = <char>NPY_DOUBLE
 * ufunc__igam_fac_types[5] = <char>NPY_DOUBLE
 * ufunc__igam_fac_ptr[2*0] = <void*>_func_igam_fac             # <<<<<<<<<<<<<<
 * ufunc__igam_fac_ptr[2*0+1] = <void*>(<char*>"_igam_fac")
 * ufunc__igam_fac_ptr[2*1] = <void*>_func_igam_fac
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_ptr[0]) = ((void *)igam_fac);

  /* "scipy/special/_ufuncs.pyx":2057
 * ufunc__igam_fac_types[5] = <char>NPY_DOUBLE
 * ufunc__igam_fac_ptr[2*0] = <void*>_func_igam_fac
 * ufunc__igam_fac_ptr[2*0+1] = <void*>(<char*>"_igam_fac")             # <<<<<<<<<<<<<<
 * ufunc__igam_fac_ptr[2*1] = <void*>_func_igam_fac
 * ufunc__igam_fac_ptr[2*1+1] = <void*>(<char*>"_igam_fac")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_ptr[1]) = ((void *)((char *)((char *)"_igam_fac")));

  /* "scipy/special/_ufuncs.pyx":2058
 * ufunc__igam_fac_ptr[2*0] = <void*>_func_igam_fac
 * ufunc__igam_fac_ptr[2*0+1] = <void*>(<char*>"_igam_fac")
 * ufunc__igam_fac_ptr[2*1] = <void*>_func_igam_fac             # <<<<<<<<<<<<<<
 * ufunc__igam_fac_ptr[2*1+1] = <void*>(<char*>"_igam_fac")
 * ufunc__igam_fac_data[0] = &ufunc__igam_fac_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_ptr[2]) = ((void *)igam_fac);

  /* "scipy/special/_ufuncs.pyx":2059
 * ufunc__igam_fac_ptr[2*0+1] = <void*>(<char*>"_igam_fac")
 * ufunc__igam_fac_ptr[2*1] = <void*>_func_igam_fac
 * ufunc__igam_fac_ptr[2*1+1] = <void*>(<char*>"_igam_fac")             # <<<<<<<<<<<<<<
 * ufunc__igam_fac_data[0] = &ufunc__igam_fac_ptr[2*0]
 * ufunc__igam_fac_data[1] = &ufunc__igam_fac_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_ptr[3]) = ((void *)((char *)((char *)"_igam_fac")));

  /* "scipy/special/_ufuncs.pyx":2060
 * ufunc__igam_fac_ptr[2*1] = <void*>_func_igam_fac
 * ufunc__igam_fac_ptr[2*1+1] = <void*>(<char*>"_igam_fac")
 * ufunc__igam_fac_data[0] = &ufunc__igam_fac_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__igam_fac_data[1] = &ufunc__igam_fac_ptr[2*1]
 * _igam_fac = np.PyUFunc_FromFuncAndData(ufunc__igam_fac_loops, ufunc__igam_fac_data, ufunc__igam_fac_types, 2, 2, 1, 0, "_igam_fac", ufunc__igam_fac_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2061
 * ufunc__igam_fac_ptr[2*1+1] = <void*>(<char*>"_igam_fac")
 * ufunc__igam_fac_data[0] = &ufunc__igam_fac_ptr[2*0]
 * ufunc__igam_fac_data[1] = &ufunc__igam_fac_ptr[2*1]             # <<<<<<<<<<<<<<
 * _igam_fac = np.PyUFunc_FromFuncAndData(ufunc__igam_fac_loops, ufunc__igam_fac_data, ufunc__igam_fac_types, 2, 2, 1, 0, "_igam_fac", ufunc__igam_fac_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2062
 * ufunc__igam_fac_data[0] = &ufunc__igam_fac_ptr[2*0]
 * ufunc__igam_fac_data[1] = &ufunc__igam_fac_ptr[2*1]
 * _igam_fac = np.PyUFunc_FromFuncAndData(ufunc__igam_fac_loops, ufunc__igam_fac_data, ufunc__igam_fac_types, 2, 2, 1, 0, "_igam_fac", ufunc__igam_fac_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__kolmogc_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_types, 2, 2, 1, 0, ((char *)"_igam_fac"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__igam_fac_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2062, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_igam_fac, __pyx_t_2) < 0) __PYX_ERR(1, 2062, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2068
 * cdef void *ufunc__kolmogc_data[2]
 * cdef char ufunc__kolmogc_types[4]
 * cdef char *ufunc__kolmogc_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, do not use.")
 * ufunc__kolmogc_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_doc = ((char *)"Internal function, do not use.");

  /* "scipy/special/_ufuncs.pyx":2070
 * cdef char *ufunc__kolmogc_doc = (
 *     "Internal function, do not use.")
 * ufunc__kolmogc_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc__kolmogc_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__kolmogc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":2071
 *     "Internal function, do not use.")
 * ufunc__kolmogc_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc__kolmogc_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc__kolmogc_types[0] = <char>NPY_FLOAT
 * ufunc__kolmogc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":2072
 * ufunc__kolmogc_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc__kolmogc_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__kolmogc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__kolmogc_types[1] = <char>NPY_FLOAT
 * ufunc__kolmogc_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2073
 * ufunc__kolmogc_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__kolmogc_types[0] = <char>NPY_FLOAT
 * ufunc__kolmogc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__kolmogc_types[2] = <char>NPY_DOUBLE
 * ufunc__kolmogc_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2074
 * ufunc__kolmogc_types[0] = <char>NPY_FLOAT
 * ufunc__kolmogc_types[1] = <char>NPY_FLOAT
 * ufunc__kolmogc_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__kolmogc_types[3] = <char>NPY_DOUBLE
 * ufunc__kolmogc_ptr[2*0] = <void*>_func_kolmogc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2075
 * ufunc__kolmogc_types[1] = <char>NPY_FLOAT
 * ufunc__kolmogc_types[2] = <char>NPY_DOUBLE
 * ufunc__kolmogc_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__kolmogc_ptr[2*0] = <void*>_func_kolmogc
 * ufunc__kolmogc_ptr[2*0+1] = <void*>(<char*>"_kolmogc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2076
 * ufunc__kolmogc_types[2] = <char>NPY_DOUBLE
 * ufunc__kolmogc_types[3] = <char>NPY_DOUBLE
 * ufunc__kolmogc_ptr[2*0] = <void*>_func_kolmogc             # <<<<<<<<<<<<<<
 * ufunc__kolmogc_ptr[2*0+1] = <void*>(<char*>"_kolmogc")
 * ufunc__kolmogc_ptr[2*1] = <void*>_func_kolmogc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_ptr[0]) = ((void *)kolmogc);

  /* "scipy/special/_ufuncs.pyx":2077
 * ufunc__kolmogc_types[3] = <char>NPY_DOUBLE
 * ufunc__kolmogc_ptr[2*0] = <void*>_func_kolmogc
 * ufunc__kolmogc_ptr[2*0+1] = <void*>(<char*>"_kolmogc")             # <<<<<<<<<<<<<<
 * ufunc__kolmogc_ptr[2*1] = <void*>_func_kolmogc
 * ufunc__kolmogc_ptr[2*1+1] = <void*>(<char*>"_kolmogc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_ptr[1]) = ((void *)((char *)((char *)"_kolmogc")));

  /* "scipy/special/_ufuncs.pyx":2078
 * ufunc__kolmogc_ptr[2*0] = <void*>_func_kolmogc
 * ufunc__kolmogc_ptr[2*0+1] = <void*>(<char*>"_kolmogc")
 * ufunc__kolmogc_ptr[2*1] = <void*>_func_kolmogc             # <<<<<<<<<<<<<<
 * ufunc__kolmogc_ptr[2*1+1] = <void*>(<char*>"_kolmogc")
 * ufunc__kolmogc_data[0] = &ufunc__kolmogc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_ptr[2]) = ((void *)kolmogc);

  /* "scipy/special/_ufuncs.pyx":2079
 * ufunc__kolmogc_ptr[2*0+1] = <void*>(<char*>"_kolmogc")
 * ufunc__kolmogc_ptr[2*1] = <void*>_func_kolmogc
 * ufunc__kolmogc_ptr[2*1+1] = <void*>(<char*>"_kolmogc")             # <<<<<<<<<<<<<<
 * ufunc__kolmogc_data[0] = &ufunc__kolmogc_ptr[2*0]
 * ufunc__kolmogc_data[1] = &ufunc__kolmogc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_ptr[3]) = ((void *)((char *)((char *)"_kolmogc")));

  /* "scipy/special/_ufuncs.pyx":2080
 * ufunc__kolmogc_ptr[2*1] = <void*>_func_kolmogc
 * ufunc__kolmogc_ptr[2*1+1] = <void*>(<char*>"_kolmogc")
 * ufunc__kolmogc_data[0] = &ufunc__kolmogc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__kolmogc_data[1] = &ufunc__kolmogc_ptr[2*1]
 * _kolmogc = np.PyUFunc_FromFuncAndData(ufunc__kolmogc_loops, ufunc__kolmogc_data, ufunc__kolmogc_types, 2, 1, 1, 0, "_kolmogc", ufunc__kolmogc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2081
 * ufunc__kolmogc_ptr[2*1+1] = <void*>(<char*>"_kolmogc")
 * ufunc__kolmogc_data[0] = &ufunc__kolmogc_ptr[2*0]
 * ufunc__kolmogc_data[1] = &ufunc__kolmogc_ptr[2*1]             # <<<<<<<<<<<<<<
 * _kolmogc = np.PyUFunc_FromFuncAndData(ufunc__kolmogc_loops, ufunc__kolmogc_data, ufunc__kolmogc_types, 2, 1, 1, 0, "_kolmogc", ufunc__kolmogc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2082
 * ufunc__kolmogc_data[0] = &ufunc__kolmogc_ptr[2*0]
 * ufunc__kolmogc_data[1] = &ufunc__kolmogc_ptr[2*1]
 * _kolmogc = np.PyUFunc_FromFuncAndData(ufunc__kolmogc_loops, ufunc__kolmogc_data, ufunc__kolmogc_types, 2, 1, 1, 0, "_kolmogc", ufunc__kolmogc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__kolmogci_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_types, 2, 1, 1, 0, ((char *)"_kolmogc"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogc_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2082, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kolmogc, __pyx_t_2) < 0) __PYX_ERR(1, 2082, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2088
 * cdef void *ufunc__kolmogci_data[2]
 * cdef char ufunc__kolmogci_types[4]
 * cdef char *ufunc__kolmogci_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, do not use.")
 * ufunc__kolmogci_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_doc = ((char *)"Internal function, do not use.");

  /* "scipy/special/_ufuncs.pyx":2090
 * cdef char *ufunc__kolmogci_doc = (
 *     "Internal function, do not use.")
 * ufunc__kolmogci_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc__kolmogci_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__kolmogci_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":2091
 *     "Internal function, do not use.")
 * ufunc__kolmogci_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc__kolmogci_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc__kolmogci_types[0] = <char>NPY_FLOAT
 * ufunc__kolmogci_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":2092
 * ufunc__kolmogci_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc__kolmogci_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__kolmogci_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__kolmogci_types[1] = <char>NPY_FLOAT
 * ufunc__kolmogci_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2093
 * ufunc__kolmogci_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__kolmogci_types[0] = <char>NPY_FLOAT
 * ufunc__kolmogci_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__kolmogci_types[2] = <char>NPY_DOUBLE
 * ufunc__kolmogci_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2094
 * ufunc__kolmogci_types[0] = <char>NPY_FLOAT
 * ufunc__kolmogci_types[1] = <char>NPY_FLOAT
 * ufunc__kolmogci_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__kolmogci_types[3] = <char>NPY_DOUBLE
 * ufunc__kolmogci_ptr[2*0] = <void*>_func_kolmogci
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2095
 * ufunc__kolmogci_types[1] = <char>NPY_FLOAT
 * ufunc__kolmogci_types[2] = <char>NPY_DOUBLE
 * ufunc__kolmogci_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__kolmogci_ptr[2*0] = <void*>_func_kolmogci
 * ufunc__kolmogci_ptr[2*0+1] = <void*>(<char*>"_kolmogci")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2096
 * ufunc__kolmogci_types[2] = <char>NPY_DOUBLE
 * ufunc__kolmogci_types[3] = <char>NPY_DOUBLE
 * ufunc__kolmogci_ptr[2*0] = <void*>_func_kolmogci             # <<<<<<<<<<<<<<
 * ufunc__kolmogci_ptr[2*0+1] = <void*>(<char*>"_kolmogci")
 * ufunc__kolmogci_ptr[2*1] = <void*>_func_kolmogci
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_ptr[0]) = ((void *)kolmogci);

  /* "scipy/special/_ufuncs.pyx":2097
 * ufunc__kolmogci_types[3] = <char>NPY_DOUBLE
 * ufunc__kolmogci_ptr[2*0] = <void*>_func_kolmogci
 * ufunc__kolmogci_ptr[2*0+1] = <void*>(<char*>"_kolmogci")             # <<<<<<<<<<<<<<
 * ufunc__kolmogci_ptr[2*1] = <void*>_func_kolmogci
 * ufunc__kolmogci_ptr[2*1+1] = <void*>(<char*>"_kolmogci")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_ptr[1]) = ((void *)((char *)((char *)"_kolmogci")));

  /* "scipy/special/_ufuncs.pyx":2098
 * ufunc__kolmogci_ptr[2*0] = <void*>_func_kolmogci
 * ufunc__kolmogci_ptr[2*0+1] = <void*>(<char*>"_kolmogci")
 * ufunc__kolmogci_ptr[2*1] = <void*>_func_kolmogci             # <<<<<<<<<<<<<<
 * ufunc__kolmogci_ptr[2*1+1] = <void*>(<char*>"_kolmogci")
 * ufunc__kolmogci_data[0] = &ufunc__kolmogci_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_ptr[2]) = ((void *)kolmogci);

  /* "scipy/special/_ufuncs.pyx":2099
 * ufunc__kolmogci_ptr[2*0+1] = <void*>(<char*>"_kolmogci")
 * ufunc__kolmogci_ptr[2*1] = <void*>_func_kolmogci
 * ufunc__kolmogci_ptr[2*1+1] = <void*>(<char*>"_kolmogci")             # <<<<<<<<<<<<<<
 * ufunc__kolmogci_data[0] = &ufunc__kolmogci_ptr[2*0]
 * ufunc__kolmogci_data[1] = &ufunc__kolmogci_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_ptr[3]) = ((void *)((char *)((char *)"_kolmogci")));

  /* "scipy/special/_ufuncs.pyx":2100
 * ufunc__kolmogci_ptr[2*1] = <void*>_func_kolmogci
 * ufunc__kolmogci_ptr[2*1+1] = <void*>(<char*>"_kolmogci")
 * ufunc__kolmogci_data[0] = &ufunc__kolmogci_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__kolmogci_data[1] = &ufunc__kolmogci_ptr[2*1]
 * _kolmogci = np.PyUFunc_FromFuncAndData(ufunc__kolmogci_loops, ufunc__kolmogci_data, ufunc__kolmogci_types, 2, 1, 1, 0, "_kolmogci", ufunc__kolmogci_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2101
 * ufunc__kolmogci_ptr[2*1+1] = <void*>(<char*>"_kolmogci")
 * ufunc__kolmogci_data[0] = &ufunc__kolmogci_ptr[2*0]
 * ufunc__kolmogci_data[1] = &ufunc__kolmogci_ptr[2*1]             # <<<<<<<<<<<<<<
 * _kolmogci = np.PyUFunc_FromFuncAndData(ufunc__kolmogci_loops, ufunc__kolmogci_data, ufunc__kolmogci_types, 2, 1, 1, 0, "_kolmogci", ufunc__kolmogci_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2102
 * ufunc__kolmogci_data[0] = &ufunc__kolmogci_ptr[2*0]
 * ufunc__kolmogci_data[1] = &ufunc__kolmogci_ptr[2*1]
 * _kolmogci = np.PyUFunc_FromFuncAndData(ufunc__kolmogci_loops, ufunc__kolmogci_data, ufunc__kolmogci_types, 2, 1, 1, 0, "_kolmogci", ufunc__kolmogci_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__kolmogp_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_types, 2, 1, 1, 0, ((char *)"_kolmogci"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogci_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kolmogci, __pyx_t_2) < 0) __PYX_ERR(1, 2102, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2108
 * cdef void *ufunc__kolmogp_data[2]
 * cdef char ufunc__kolmogp_types[4]
 * cdef char *ufunc__kolmogp_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, do not use.")
 * ufunc__kolmogp_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_doc = ((char *)"Internal function, do not use.");

  /* "scipy/special/_ufuncs.pyx":2110
 * cdef char *ufunc__kolmogp_doc = (
 *     "Internal function, do not use.")
 * ufunc__kolmogp_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc__kolmogp_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__kolmogp_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":2111
 *     "Internal function, do not use.")
 * ufunc__kolmogp_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc__kolmogp_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc__kolmogp_types[0] = <char>NPY_FLOAT
 * ufunc__kolmogp_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":2112
 * ufunc__kolmogp_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc__kolmogp_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__kolmogp_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__kolmogp_types[1] = <char>NPY_FLOAT
 * ufunc__kolmogp_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2113
 * ufunc__kolmogp_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__kolmogp_types[0] = <char>NPY_FLOAT
 * ufunc__kolmogp_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__kolmogp_types[2] = <char>NPY_DOUBLE
 * ufunc__kolmogp_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2114
 * ufunc__kolmogp_types[0] = <char>NPY_FLOAT
 * ufunc__kolmogp_types[1] = <char>NPY_FLOAT
 * ufunc__kolmogp_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__kolmogp_types[3] = <char>NPY_DOUBLE
 * ufunc__kolmogp_ptr[2*0] = <void*>_func_kolmogp
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2115
 * ufunc__kolmogp_types[1] = <char>NPY_FLOAT
 * ufunc__kolmogp_types[2] = <char>NPY_DOUBLE
 * ufunc__kolmogp_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__kolmogp_ptr[2*0] = <void*>_func_kolmogp
 * ufunc__kolmogp_ptr[2*0+1] = <void*>(<char*>"_kolmogp")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2116
 * ufunc__kolmogp_types[2] = <char>NPY_DOUBLE
 * ufunc__kolmogp_types[3] = <char>NPY_DOUBLE
 * ufunc__kolmogp_ptr[2*0] = <void*>_func_kolmogp             # <<<<<<<<<<<<<<
 * ufunc__kolmogp_ptr[2*0+1] = <void*>(<char*>"_kolmogp")
 * ufunc__kolmogp_ptr[2*1] = <void*>_func_kolmogp
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_ptr[0]) = ((void *)kolmogp);

  /* "scipy/special/_ufuncs.pyx":2117
 * ufunc__kolmogp_types[3] = <char>NPY_DOUBLE
 * ufunc__kolmogp_ptr[2*0] = <void*>_func_kolmogp
 * ufunc__kolmogp_ptr[2*0+1] = <void*>(<char*>"_kolmogp")             # <<<<<<<<<<<<<<
 * ufunc__kolmogp_ptr[2*1] = <void*>_func_kolmogp
 * ufunc__kolmogp_ptr[2*1+1] = <void*>(<char*>"_kolmogp")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_ptr[1]) = ((void *)((char *)((char *)"_kolmogp")));

  /* "scipy/special/_ufuncs.pyx":2118
 * ufunc__kolmogp_ptr[2*0] = <void*>_func_kolmogp
 * ufunc__kolmogp_ptr[2*0+1] = <void*>(<char*>"_kolmogp")
 * ufunc__kolmogp_ptr[2*1] = <void*>_func_kolmogp             # <<<<<<<<<<<<<<
 * ufunc__kolmogp_ptr[2*1+1] = <void*>(<char*>"_kolmogp")
 * ufunc__kolmogp_data[0] = &ufunc__kolmogp_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_ptr[2]) = ((void *)kolmogp);

  /* "scipy/special/_ufuncs.pyx":2119
 * ufunc__kolmogp_ptr[2*0+1] = <void*>(<char*>"_kolmogp")
 * ufunc__kolmogp_ptr[2*1] = <void*>_func_kolmogp
 * ufunc__kolmogp_ptr[2*1+1] = <void*>(<char*>"_kolmogp")             # <<<<<<<<<<<<<<
 * ufunc__kolmogp_data[0] = &ufunc__kolmogp_ptr[2*0]
 * ufunc__kolmogp_data[1] = &ufunc__kolmogp_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_ptr[3]) = ((void *)((char *)((char *)"_kolmogp")));

  /* "scipy/special/_ufuncs.pyx":2120
 * ufunc__kolmogp_ptr[2*1] = <void*>_func_kolmogp
 * ufunc__kolmogp_ptr[2*1+1] = <void*>(<char*>"_kolmogp")
 * ufunc__kolmogp_data[0] = &ufunc__kolmogp_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__kolmogp_data[1] = &ufunc__kolmogp_ptr[2*1]
 * _kolmogp = np.PyUFunc_FromFuncAndData(ufunc__kolmogp_loops, ufunc__kolmogp_data, ufunc__kolmogp_types, 2, 1, 1, 0, "_kolmogp", ufunc__kolmogp_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2121
 * ufunc__kolmogp_ptr[2*1+1] = <void*>(<char*>"_kolmogp")
 * ufunc__kolmogp_data[0] = &ufunc__kolmogp_ptr[2*0]
 * ufunc__kolmogp_data[1] = &ufunc__kolmogp_ptr[2*1]             # <<<<<<<<<<<<<<
 * _kolmogp = np.PyUFunc_FromFuncAndData(ufunc__kolmogp_loops, ufunc__kolmogp_data, ufunc__kolmogp_types, 2, 1, 1, 0, "_kolmogp", ufunc__kolmogp_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2122
 * ufunc__kolmogp_data[0] = &ufunc__kolmogp_ptr[2*0]
 * ufunc__kolmogp_data[1] = &ufunc__kolmogp_ptr[2*1]
 * _kolmogp = np.PyUFunc_FromFuncAndData(ufunc__kolmogp_loops, ufunc__kolmogp_data, ufunc__kolmogp_types, 2, 1, 1, 0, "_kolmogp", ufunc__kolmogp_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__lambertw_loops[1]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_types, 2, 1, 1, 0, ((char *)"_kolmogp"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__kolmogp_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kolmogp, __pyx_t_2) < 0) __PYX_ERR(1, 2122, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2128
 * cdef void *ufunc__lambertw_data[1]
 * cdef char ufunc__lambertw_types[4]
 * cdef char *ufunc__lambertw_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, use `lambertw` instead.")
 * ufunc__lambertw_loops[0] = <np.PyUFuncGenericFunction>loop_D_Dld__As_Dld_D
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_doc = ((char *)"Internal function, use `lambertw` instead.");

  /* "scipy/special/_ufuncs.pyx":2130
 * cdef char *ufunc__lambertw_doc = (
 *     "Internal function, use `lambertw` instead.")
 * ufunc__lambertw_loops[0] = <np.PyUFuncGenericFunction>loop_D_Dld__As_Dld_D             # <<<<<<<<<<<<<<
 * ufunc__lambertw_types[0] = <char>NPY_CDOUBLE
 * ufunc__lambertw_types[1] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_Dld__As_Dld_D);

  /* "scipy/special/_ufuncs.pyx":2131
 *     "Internal function, use `lambertw` instead.")
 * ufunc__lambertw_loops[0] = <np.PyUFuncGenericFunction>loop_D_Dld__As_Dld_D
 * ufunc__lambertw_types[0] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__lambertw_types[1] = <char>NPY_LONG
 * ufunc__lambertw_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_types[0]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2132
 * ufunc__lambertw_loops[0] = <np.PyUFuncGenericFunction>loop_D_Dld__As_Dld_D
 * ufunc__lambertw_types[0] = <char>NPY_CDOUBLE
 * ufunc__lambertw_types[1] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__lambertw_types[2] = <char>NPY_DOUBLE
 * ufunc__lambertw_types[3] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_types[1]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2133
 * ufunc__lambertw_types[0] = <char>NPY_CDOUBLE
 * ufunc__lambertw_types[1] = <char>NPY_LONG
 * ufunc__lambertw_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__lambertw_types[3] = <char>NPY_CDOUBLE
 * ufunc__lambertw_ptr[2*0] = <void*>_func_lambertw_scalar
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2134
 * ufunc__lambertw_types[1] = <char>NPY_LONG
 * ufunc__lambertw_types[2] = <char>NPY_DOUBLE
 * ufunc__lambertw_types[3] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__lambertw_ptr[2*0] = <void*>_func_lambertw_scalar
 * ufunc__lambertw_ptr[2*0+1] = <void*>(<char*>"_lambertw")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_types[3]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2135
 * ufunc__lambertw_types[2] = <char>NPY_DOUBLE
 * ufunc__lambertw_types[3] = <char>NPY_CDOUBLE
 * ufunc__lambertw_ptr[2*0] = <void*>_func_lambertw_scalar             # <<<<<<<<<<<<<<
 * ufunc__lambertw_ptr[2*0+1] = <void*>(<char*>"_lambertw")
 * ufunc__lambertw_data[0] = &ufunc__lambertw_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_ptr[0]) = ((void *)__pyx_f_5scipy_7special_8lambertw_lambertw_scalar);

  /* "scipy/special/_ufuncs.pyx":2136
 * ufunc__lambertw_types[3] = <char>NPY_CDOUBLE
 * ufunc__lambertw_ptr[2*0] = <void*>_func_lambertw_scalar
 * ufunc__lambertw_ptr[2*0+1] = <void*>(<char*>"_lambertw")             # <<<<<<<<<<<<<<
 * ufunc__lambertw_data[0] = &ufunc__lambertw_ptr[2*0]
 * _lambertw = np.PyUFunc_FromFuncAndData(ufunc__lambertw_loops, ufunc__lambertw_data, ufunc__lambertw_types, 1, 3, 1, 0, "_lambertw", ufunc__lambertw_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_ptr[1]) = ((void *)((char *)((char *)"_lambertw")));

  /* "scipy/special/_ufuncs.pyx":2137
 * ufunc__lambertw_ptr[2*0] = <void*>_func_lambertw_scalar
 * ufunc__lambertw_ptr[2*0+1] = <void*>(<char*>"_lambertw")
 * ufunc__lambertw_data[0] = &ufunc__lambertw_ptr[2*0]             # <<<<<<<<<<<<<<
 * _lambertw = np.PyUFunc_FromFuncAndData(ufunc__lambertw_loops, ufunc__lambertw_data, ufunc__lambertw_types, 1, 3, 1, 0, "_lambertw", ufunc__lambertw_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2138
 * ufunc__lambertw_ptr[2*0+1] = <void*>(<char*>"_lambertw")
 * ufunc__lambertw_data[0] = &ufunc__lambertw_ptr[2*0]
 * _lambertw = np.PyUFunc_FromFuncAndData(ufunc__lambertw_loops, ufunc__lambertw_data, ufunc__lambertw_types, 1, 3, 1, 0, "_lambertw", ufunc__lambertw_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__lanczos_sum_expg_scaled_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_types, 1, 3, 1, 0, ((char *)"_lambertw"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__lambertw_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_lambertw, __pyx_t_2) < 0) __PYX_ERR(1, 2138, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2144
 * cdef void *ufunc__lanczos_sum_expg_scaled_data[2]
 * cdef char ufunc__lanczos_sum_expg_scaled_types[4]
 * cdef char *ufunc__lanczos_sum_expg_scaled_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, do not use.")
 * ufunc__lanczos_sum_expg_scaled_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_doc = ((char *)"Internal function, do not use.");

  /* "scipy/special/_ufuncs.pyx":2146
 * cdef char *ufunc__lanczos_sum_expg_scaled_doc = (
 *     "Internal function, do not use.")
 * ufunc__lanczos_sum_expg_scaled_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc__lanczos_sum_expg_scaled_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__lanczos_sum_expg_scaled_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":2147
 *     "Internal function, do not use.")
 * ufunc__lanczos_sum_expg_scaled_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc__lanczos_sum_expg_scaled_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc__lanczos_sum_expg_scaled_types[0] = <char>NPY_FLOAT
 * ufunc__lanczos_sum_expg_scaled_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":2148
 * ufunc__lanczos_sum_expg_scaled_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc__lanczos_sum_expg_scaled_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__lanczos_sum_expg_scaled_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__lanczos_sum_expg_scaled_types[1] = <char>NPY_FLOAT
 * ufunc__lanczos_sum_expg_scaled_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2149
 * ufunc__lanczos_sum_expg_scaled_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__lanczos_sum_expg_scaled_types[0] = <char>NPY_FLOAT
 * ufunc__lanczos_sum_expg_scaled_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__lanczos_sum_expg_scaled_types[2] = <char>NPY_DOUBLE
 * ufunc__lanczos_sum_expg_scaled_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2150
 * ufunc__lanczos_sum_expg_scaled_types[0] = <char>NPY_FLOAT
 * ufunc__lanczos_sum_expg_scaled_types[1] = <char>NPY_FLOAT
 * ufunc__lanczos_sum_expg_scaled_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__lanczos_sum_expg_scaled_types[3] = <char>NPY_DOUBLE
 * ufunc__lanczos_sum_expg_scaled_ptr[2*0] = <void*>_func_lanczos_sum_expg_scaled
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2151
 * ufunc__lanczos_sum_expg_scaled_types[1] = <char>NPY_FLOAT
 * ufunc__lanczos_sum_expg_scaled_types[2] = <char>NPY_DOUBLE
 * ufunc__lanczos_sum_expg_scaled_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__lanczos_sum_expg_scaled_ptr[2*0] = <void*>_func_lanczos_sum_expg_scaled
 * ufunc__lanczos_sum_expg_scaled_ptr[2*0+1] = <void*>(<char*>"_lanczos_sum_expg_scaled")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2152
 * ufunc__lanczos_sum_expg_scaled_types[2] = <char>NPY_DOUBLE
 * ufunc__lanczos_sum_expg_scaled_types[3] = <char>NPY_DOUBLE
 * ufunc__lanczos_sum_expg_scaled_ptr[2*0] = <void*>_func_lanczos_sum_expg_scaled             # <<<<<<<<<<<<<<
 * ufunc__lanczos_sum_expg_scaled_ptr[2*0+1] = <void*>(<char*>"_lanczos_sum_expg_scaled")
 * ufunc__lanczos_sum_expg_scaled_ptr[2*1] = <void*>_func_lanczos_sum_expg_scaled
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_ptr[0]) = ((void *)lanczos_sum_expg_scaled);

  /* "scipy/special/_ufuncs.pyx":2153
 * ufunc__lanczos_sum_expg_scaled_types[3] = <char>NPY_DOUBLE
 * ufunc__lanczos_sum_expg_scaled_ptr[2*0] = <void*>_func_lanczos_sum_expg_scaled
 * ufunc__lanczos_sum_expg_scaled_ptr[2*0+1] = <void*>(<char*>"_lanczos_sum_expg_scaled")             # <<<<<<<<<<<<<<
 * ufunc__lanczos_sum_expg_scaled_ptr[2*1] = <void*>_func_lanczos_sum_expg_scaled
 * ufunc__lanczos_sum_expg_scaled_ptr[2*1+1] = <void*>(<char*>"_lanczos_sum_expg_scaled")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_ptr[1]) = ((void *)((char *)((char *)"_lanczos_sum_expg_scaled")));

  /* "scipy/special/_ufuncs.pyx":2154
 * ufunc__lanczos_sum_expg_scaled_ptr[2*0] = <void*>_func_lanczos_sum_expg_scaled
 * ufunc__lanczos_sum_expg_scaled_ptr[2*0+1] = <void*>(<char*>"_lanczos_sum_expg_scaled")
 * ufunc__lanczos_sum_expg_scaled_ptr[2*1] = <void*>_func_lanczos_sum_expg_scaled             # <<<<<<<<<<<<<<
 * ufunc__lanczos_sum_expg_scaled_ptr[2*1+1] = <void*>(<char*>"_lanczos_sum_expg_scaled")
 * ufunc__lanczos_sum_expg_scaled_data[0] = &ufunc__lanczos_sum_expg_scaled_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_ptr[2]) = ((void *)lanczos_sum_expg_scaled);

  /* "scipy/special/_ufuncs.pyx":2155
 * ufunc__lanczos_sum_expg_scaled_ptr[2*0+1] = <void*>(<char*>"_lanczos_sum_expg_scaled")
 * ufunc__lanczos_sum_expg_scaled_ptr[2*1] = <void*>_func_lanczos_sum_expg_scaled
 * ufunc__lanczos_sum_expg_scaled_ptr[2*1+1] = <void*>(<char*>"_lanczos_sum_expg_scaled")             # <<<<<<<<<<<<<<
 * ufunc__lanczos_sum_expg_scaled_data[0] = &ufunc__lanczos_sum_expg_scaled_ptr[2*0]
 * ufunc__lanczos_sum_expg_scaled_data[1] = &ufunc__lanczos_sum_expg_scaled_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_ptr[3]) = ((void *)((char *)((char *)"_lanczos_sum_expg_scaled")));

  /* "scipy/special/_ufuncs.pyx":2156
 * ufunc__lanczos_sum_expg_scaled_ptr[2*1] = <void*>_func_lanczos_sum_expg_scaled
 * ufunc__lanczos_sum_expg_scaled_ptr[2*1+1] = <void*>(<char*>"_lanczos_sum_expg_scaled")
 * ufunc__lanczos_sum_expg_scaled_data[0] = &ufunc__lanczos_sum_expg_scaled_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__lanczos_sum_expg_scaled_data[1] = &ufunc__lanczos_sum_expg_scaled_ptr[2*1]
 * _lanczos_sum_expg_scaled = np.PyUFunc_FromFuncAndData(ufunc__lanczos_sum_expg_scaled_loops, ufunc__lanczos_sum_expg_scaled_data, ufunc__lanczos_sum_expg_scaled_types, 2, 1, 1, 0, "_lanczos_sum_expg_scaled", ufunc__lanczos_sum_expg_scaled_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2157
 * ufunc__lanczos_sum_expg_scaled_ptr[2*1+1] = <void*>(<char*>"_lanczos_sum_expg_scaled")
 * ufunc__lanczos_sum_expg_scaled_data[0] = &ufunc__lanczos_sum_expg_scaled_ptr[2*0]
 * ufunc__lanczos_sum_expg_scaled_data[1] = &ufunc__lanczos_sum_expg_scaled_ptr[2*1]             # <<<<<<<<<<<<<<
 * _lanczos_sum_expg_scaled = np.PyUFunc_FromFuncAndData(ufunc__lanczos_sum_expg_scaled_loops, ufunc__lanczos_sum_expg_scaled_data, ufunc__lanczos_sum_expg_scaled_types, 2, 1, 1, 0, "_lanczos_sum_expg_scaled", ufunc__lanczos_sum_expg_scaled_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2158
 * ufunc__lanczos_sum_expg_scaled_data[0] = &ufunc__lanczos_sum_expg_scaled_ptr[2*0]
 * ufunc__lanczos_sum_expg_scaled_data[1] = &ufunc__lanczos_sum_expg_scaled_ptr[2*1]
 * _lanczos_sum_expg_scaled = np.PyUFunc_FromFuncAndData(ufunc__lanczos_sum_expg_scaled_loops, ufunc__lanczos_sum_expg_scaled_data, ufunc__lanczos_sum_expg_scaled_types, 2, 1, 1, 0, "_lanczos_sum_expg_scaled", ufunc__lanczos_sum_expg_scaled_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__lgam1p_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_types, 2, 1, 1, 0, ((char *)"_lanczos_sum_expg_scaled"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__lanczos_sum_expg_scaled_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_lanczos_sum_expg_scaled, __pyx_t_2) < 0) __PYX_ERR(1, 2158, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2164
 * cdef void *ufunc__lgam1p_data[2]
 * cdef char ufunc__lgam1p_types[4]
 * cdef char *ufunc__lgam1p_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, do not use.")
 * ufunc__lgam1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_doc = ((char *)"Internal function, do not use.");

  /* "scipy/special/_ufuncs.pyx":2166
 * cdef char *ufunc__lgam1p_doc = (
 *     "Internal function, do not use.")
 * ufunc__lgam1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc__lgam1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__lgam1p_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":2167
 *     "Internal function, do not use.")
 * ufunc__lgam1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc__lgam1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc__lgam1p_types[0] = <char>NPY_FLOAT
 * ufunc__lgam1p_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":2168
 * ufunc__lgam1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc__lgam1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__lgam1p_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__lgam1p_types[1] = <char>NPY_FLOAT
 * ufunc__lgam1p_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2169
 * ufunc__lgam1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__lgam1p_types[0] = <char>NPY_FLOAT
 * ufunc__lgam1p_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__lgam1p_types[2] = <char>NPY_DOUBLE
 * ufunc__lgam1p_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2170
 * ufunc__lgam1p_types[0] = <char>NPY_FLOAT
 * ufunc__lgam1p_types[1] = <char>NPY_FLOAT
 * ufunc__lgam1p_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__lgam1p_types[3] = <char>NPY_DOUBLE
 * ufunc__lgam1p_ptr[2*0] = <void*>_func_lgam1p
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2171
 * ufunc__lgam1p_types[1] = <char>NPY_FLOAT
 * ufunc__lgam1p_types[2] = <char>NPY_DOUBLE
 * ufunc__lgam1p_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__lgam1p_ptr[2*0] = <void*>_func_lgam1p
 * ufunc__lgam1p_ptr[2*0+1] = <void*>(<char*>"_lgam1p")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2172
 * ufunc__lgam1p_types[2] = <char>NPY_DOUBLE
 * ufunc__lgam1p_types[3] = <char>NPY_DOUBLE
 * ufunc__lgam1p_ptr[2*0] = <void*>_func_lgam1p             # <<<<<<<<<<<<<<
 * ufunc__lgam1p_ptr[2*0+1] = <void*>(<char*>"_lgam1p")
 * ufunc__lgam1p_ptr[2*1] = <void*>_func_lgam1p
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_ptr[0]) = ((void *)lgam1p);

  /* "scipy/special/_ufuncs.pyx":2173
 * ufunc__lgam1p_types[3] = <char>NPY_DOUBLE
 * ufunc__lgam1p_ptr[2*0] = <void*>_func_lgam1p
 * ufunc__lgam1p_ptr[2*0+1] = <void*>(<char*>"_lgam1p")             # <<<<<<<<<<<<<<
 * ufunc__lgam1p_ptr[2*1] = <void*>_func_lgam1p
 * ufunc__lgam1p_ptr[2*1+1] = <void*>(<char*>"_lgam1p")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_ptr[1]) = ((void *)((char *)((char *)"_lgam1p")));

  /* "scipy/special/_ufuncs.pyx":2174
 * ufunc__lgam1p_ptr[2*0] = <void*>_func_lgam1p
 * ufunc__lgam1p_ptr[2*0+1] = <void*>(<char*>"_lgam1p")
 * ufunc__lgam1p_ptr[2*1] = <void*>_func_lgam1p             # <<<<<<<<<<<<<<
 * ufunc__lgam1p_ptr[2*1+1] = <void*>(<char*>"_lgam1p")
 * ufunc__lgam1p_data[0] = &ufunc__lgam1p_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_ptr[2]) = ((void *)lgam1p);

  /* "scipy/special/_ufuncs.pyx":2175
 * ufunc__lgam1p_ptr[2*0+1] = <void*>(<char*>"_lgam1p")
 * ufunc__lgam1p_ptr[2*1] = <void*>_func_lgam1p
 * ufunc__lgam1p_ptr[2*1+1] = <void*>(<char*>"_lgam1p")             # <<<<<<<<<<<<<<
 * ufunc__lgam1p_data[0] = &ufunc__lgam1p_ptr[2*0]
 * ufunc__lgam1p_data[1] = &ufunc__lgam1p_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_ptr[3]) = ((void *)((char *)((char *)"_lgam1p")));

  /* "scipy/special/_ufuncs.pyx":2176
 * ufunc__lgam1p_ptr[2*1] = <void*>_func_lgam1p
 * ufunc__lgam1p_ptr[2*1+1] = <void*>(<char*>"_lgam1p")
 * ufunc__lgam1p_data[0] = &ufunc__lgam1p_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__lgam1p_data[1] = &ufunc__lgam1p_ptr[2*1]
 * _lgam1p = np.PyUFunc_FromFuncAndData(ufunc__lgam1p_loops, ufunc__lgam1p_data, ufunc__lgam1p_types, 2, 1, 1, 0, "_lgam1p", ufunc__lgam1p_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2177
 * ufunc__lgam1p_ptr[2*1+1] = <void*>(<char*>"_lgam1p")
 * ufunc__lgam1p_data[0] = &ufunc__lgam1p_ptr[2*0]
 * ufunc__lgam1p_data[1] = &ufunc__lgam1p_ptr[2*1]             # <<<<<<<<<<<<<<
 * _lgam1p = np.PyUFunc_FromFuncAndData(ufunc__lgam1p_loops, ufunc__lgam1p_data, ufunc__lgam1p_types, 2, 1, 1, 0, "_lgam1p", ufunc__lgam1p_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2178
 * ufunc__lgam1p_data[0] = &ufunc__lgam1p_ptr[2*0]
 * ufunc__lgam1p_data[1] = &ufunc__lgam1p_ptr[2*1]
 * _lgam1p = np.PyUFunc_FromFuncAndData(ufunc__lgam1p_loops, ufunc__lgam1p_data, ufunc__lgam1p_types, 2, 1, 1, 0, "_lgam1p", ufunc__lgam1p_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__log1pmx_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_types, 2, 1, 1, 0, ((char *)"_lgam1p"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__lgam1p_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_lgam1p, __pyx_t_2) < 0) __PYX_ERR(1, 2178, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2184
 * cdef void *ufunc__log1pmx_data[2]
 * cdef char ufunc__log1pmx_types[4]
 * cdef char *ufunc__log1pmx_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, do not use.")
 * ufunc__log1pmx_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_doc = ((char *)"Internal function, do not use.");

  /* "scipy/special/_ufuncs.pyx":2186
 * cdef char *ufunc__log1pmx_doc = (
 *     "Internal function, do not use.")
 * ufunc__log1pmx_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc__log1pmx_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__log1pmx_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":2187
 *     "Internal function, do not use.")
 * ufunc__log1pmx_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc__log1pmx_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc__log1pmx_types[0] = <char>NPY_FLOAT
 * ufunc__log1pmx_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":2188
 * ufunc__log1pmx_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc__log1pmx_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__log1pmx_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__log1pmx_types[1] = <char>NPY_FLOAT
 * ufunc__log1pmx_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2189
 * ufunc__log1pmx_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__log1pmx_types[0] = <char>NPY_FLOAT
 * ufunc__log1pmx_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__log1pmx_types[2] = <char>NPY_DOUBLE
 * ufunc__log1pmx_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2190
 * ufunc__log1pmx_types[0] = <char>NPY_FLOAT
 * ufunc__log1pmx_types[1] = <char>NPY_FLOAT
 * ufunc__log1pmx_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__log1pmx_types[3] = <char>NPY_DOUBLE
 * ufunc__log1pmx_ptr[2*0] = <void*>_func_log1pmx
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2191
 * ufunc__log1pmx_types[1] = <char>NPY_FLOAT
 * ufunc__log1pmx_types[2] = <char>NPY_DOUBLE
 * ufunc__log1pmx_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__log1pmx_ptr[2*0] = <void*>_func_log1pmx
 * ufunc__log1pmx_ptr[2*0+1] = <void*>(<char*>"_log1pmx")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2192
 * ufunc__log1pmx_types[2] = <char>NPY_DOUBLE
 * ufunc__log1pmx_types[3] = <char>NPY_DOUBLE
 * ufunc__log1pmx_ptr[2*0] = <void*>_func_log1pmx             # <<<<<<<<<<<<<<
 * ufunc__log1pmx_ptr[2*0+1] = <void*>(<char*>"_log1pmx")
 * ufunc__log1pmx_ptr[2*1] = <void*>_func_log1pmx
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_ptr[0]) = ((void *)log1pmx);

  /* "scipy/special/_ufuncs.pyx":2193
 * ufunc__log1pmx_types[3] = <char>NPY_DOUBLE
 * ufunc__log1pmx_ptr[2*0] = <void*>_func_log1pmx
 * ufunc__log1pmx_ptr[2*0+1] = <void*>(<char*>"_log1pmx")             # <<<<<<<<<<<<<<
 * ufunc__log1pmx_ptr[2*1] = <void*>_func_log1pmx
 * ufunc__log1pmx_ptr[2*1+1] = <void*>(<char*>"_log1pmx")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_ptr[1]) = ((void *)((char *)((char *)"_log1pmx")));

  /* "scipy/special/_ufuncs.pyx":2194
 * ufunc__log1pmx_ptr[2*0] = <void*>_func_log1pmx
 * ufunc__log1pmx_ptr[2*0+1] = <void*>(<char*>"_log1pmx")
 * ufunc__log1pmx_ptr[2*1] = <void*>_func_log1pmx             # <<<<<<<<<<<<<<
 * ufunc__log1pmx_ptr[2*1+1] = <void*>(<char*>"_log1pmx")
 * ufunc__log1pmx_data[0] = &ufunc__log1pmx_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_ptr[2]) = ((void *)log1pmx);

  /* "scipy/special/_ufuncs.pyx":2195
 * ufunc__log1pmx_ptr[2*0+1] = <void*>(<char*>"_log1pmx")
 * ufunc__log1pmx_ptr[2*1] = <void*>_func_log1pmx
 * ufunc__log1pmx_ptr[2*1+1] = <void*>(<char*>"_log1pmx")             # <<<<<<<<<<<<<<
 * ufunc__log1pmx_data[0] = &ufunc__log1pmx_ptr[2*0]
 * ufunc__log1pmx_data[1] = &ufunc__log1pmx_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_ptr[3]) = ((void *)((char *)((char *)"_log1pmx")));

  /* "scipy/special/_ufuncs.pyx":2196
 * ufunc__log1pmx_ptr[2*1] = <void*>_func_log1pmx
 * ufunc__log1pmx_ptr[2*1+1] = <void*>(<char*>"_log1pmx")
 * ufunc__log1pmx_data[0] = &ufunc__log1pmx_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__log1pmx_data[1] = &ufunc__log1pmx_ptr[2*1]
 * _log1pmx = np.PyUFunc_FromFuncAndData(ufunc__log1pmx_loops, ufunc__log1pmx_data, ufunc__log1pmx_types, 2, 1, 1, 0, "_log1pmx", ufunc__log1pmx_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2197
 * ufunc__log1pmx_ptr[2*1+1] = <void*>(<char*>"_log1pmx")
 * ufunc__log1pmx_data[0] = &ufunc__log1pmx_ptr[2*0]
 * ufunc__log1pmx_data[1] = &ufunc__log1pmx_ptr[2*1]             # <<<<<<<<<<<<<<
 * _log1pmx = np.PyUFunc_FromFuncAndData(ufunc__log1pmx_loops, ufunc__log1pmx_data, ufunc__log1pmx_types, 2, 1, 1, 0, "_log1pmx", ufunc__log1pmx_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2198
 * ufunc__log1pmx_data[0] = &ufunc__log1pmx_ptr[2*0]
 * ufunc__log1pmx_data[1] = &ufunc__log1pmx_ptr[2*1]
 * _log1pmx = np.PyUFunc_FromFuncAndData(ufunc__log1pmx_loops, ufunc__log1pmx_data, ufunc__log1pmx_types, 2, 1, 1, 0, "_log1pmx", ufunc__log1pmx_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__sf_error_test_function_loops[1]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_types, 2, 1, 1, 0, ((char *)"_log1pmx"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__log1pmx_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_log1pmx, __pyx_t_2) < 0) __PYX_ERR(1, 2198, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2204
 * cdef void *ufunc__sf_error_test_function_data[1]
 * cdef char ufunc__sf_error_test_function_types[2]
 * cdef char *ufunc__sf_error_test_function_doc = (             # <<<<<<<<<<<<<<
 *     "Private function; do not use.")
 * ufunc__sf_error_test_function_loops[0] = <np.PyUFuncGenericFunction>loop_i_i__As_l_l
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__sf_error_test_function_doc = ((char *)"Private function; do not use.");

  /* "scipy/special/_ufuncs.pyx":2206
 * cdef char *ufunc__sf_error_test_function_doc = (
 *     "Private function; do not use.")
 * ufunc__sf_error_test_function_loops[0] = <np.PyUFuncGenericFunction>loop_i_i__As_l_l             # <<<<<<<<<<<<<<
 * ufunc__sf_error_test_function_types[0] = <char>NPY_LONG
 * ufunc__sf_error_test_function_types[1] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sf_error_test_function_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_i__As_l_l);

  /* "scipy/special/_ufuncs.pyx":2207
 *     "Private function; do not use.")
 * ufunc__sf_error_test_function_loops[0] = <np.PyUFuncGenericFunction>loop_i_i__As_l_l
 * ufunc__sf_error_test_function_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__sf_error_test_function_types[1] = <char>NPY_LONG
 * ufunc__sf_error_test_function_ptr[2*0] = <void*>_func__sf_error_test_function
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sf_error_test_function_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2208
 * ufunc__sf_error_test_function_loops[0] = <np.PyUFuncGenericFunction>loop_i_i__As_l_l
 * ufunc__sf_error_test_function_types[0] = <char>NPY_LONG
 * ufunc__sf_error_test_function_types[1] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__sf_error_test_function_ptr[2*0] = <void*>_func__sf_error_test_function
 * ufunc__sf_error_test_function_ptr[2*0+1] = <void*>(<char*>"_sf_error_test_function")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sf_error_test_function_types[1]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2209
 * ufunc__sf_error_test_function_types[0] = <char>NPY_LONG
 * ufunc__sf_error_test_function_types[1] = <char>NPY_LONG
 * ufunc__sf_error_test_function_ptr[2*0] = <void*>_func__sf_error_test_function             # <<<<<<<<<<<<<<
 * ufunc__sf_error_test_function_ptr[2*0+1] = <void*>(<char*>"_sf_error_test_function")
 * ufunc__sf_error_test_function_data[0] = &ufunc__sf_error_test_function_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sf_error_test_function_ptr[0]) = ((void *)__pyx_f_5scipy_7special_8sf_error__sf_error_test_function);

  /* "scipy/special/_ufuncs.pyx":2210
 * ufunc__sf_error_test_function_types[1] = <char>NPY_LONG
 * ufunc__sf_error_test_function_ptr[2*0] = <void*>_func__sf_error_test_function
 * ufunc__sf_error_test_function_ptr[2*0+1] = <void*>(<char*>"_sf_error_test_function")             # <<<<<<<<<<<<<<
 * ufunc__sf_error_test_function_data[0] = &ufunc__sf_error_test_function_ptr[2*0]
 * _sf_error_test_function = np.PyUFunc_FromFuncAndData(ufunc__sf_error_test_function_loops, ufunc__sf_error_test_function_data, ufunc__sf_error_test_function_types, 1, 1, 1, 0, "_sf_error_test_function", ufunc__sf_error_test_function_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sf_error_test_function_ptr[1]) = ((void *)((char *)((char *)"_sf_error_test_function")));

  /* "scipy/special/_ufuncs.pyx":2211
 * ufunc__sf_error_test_function_ptr[2*0] = <void*>_func__sf_error_test_function
 * ufunc__sf_error_test_function_ptr[2*0+1] = <void*>(<char*>"_sf_error_test_function")
 * ufunc__sf_error_test_function_data[0] = &ufunc__sf_error_test_function_ptr[2*0]             # <<<<<<<<<<<<<<
 * _sf_error_test_function = np.PyUFunc_FromFuncAndData(ufunc__sf_error_test_function_loops, ufunc__sf_error_test_function_data, ufunc__sf_error_test_function_types, 1, 1, 1, 0, "_sf_error_test_function", ufunc__sf_error_test_function_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sf_error_test_function_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__sf_error_test_function_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2212
 * ufunc__sf_error_test_function_ptr[2*0+1] = <void*>(<char*>"_sf_error_test_function")
 * ufunc__sf_error_test_function_data[0] = &ufunc__sf_error_test_function_ptr[2*0]
 * _sf_error_test_function = np.PyUFunc_FromFuncAndData(ufunc__sf_error_test_function_loops, ufunc__sf_error_test_function_data, ufunc__sf_error_test_function_types, 1, 1, 1, 0, "_sf_error_test_function", ufunc__sf_error_test_function_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__sinpi_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__sf_error_test_function_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__sf_error_test_function_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__sf_error_test_function_types, 1, 1, 1, 0, ((char *)"_sf_error_test_function"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__sf_error_test_function_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sf_error_test_function, __pyx_t_2) < 0) __PYX_ERR(1, 2212, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2218
 * cdef void *ufunc__sinpi_data[4]
 * cdef char ufunc__sinpi_types[8]
 * cdef char *ufunc__sinpi_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, do not use.")
 * ufunc__sinpi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_doc = ((char *)"Internal function, do not use.");

  /* "scipy/special/_ufuncs.pyx":2220
 * cdef char *ufunc__sinpi_doc = (
 *     "Internal function, do not use.")
 * ufunc__sinpi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc__sinpi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__sinpi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":2221
 *     "Internal function, do not use.")
 * ufunc__sinpi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc__sinpi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc__sinpi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc__sinpi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":2222
 * ufunc__sinpi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc__sinpi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__sinpi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc__sinpi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc__sinpi_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":2223
 * ufunc__sinpi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc__sinpi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc__sinpi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc__sinpi_types[0] = <char>NPY_FLOAT
 * ufunc__sinpi_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":2224
 * ufunc__sinpi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc__sinpi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc__sinpi_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__sinpi_types[1] = <char>NPY_FLOAT
 * ufunc__sinpi_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2225
 * ufunc__sinpi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc__sinpi_types[0] = <char>NPY_FLOAT
 * ufunc__sinpi_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__sinpi_types[2] = <char>NPY_DOUBLE
 * ufunc__sinpi_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2226
 * ufunc__sinpi_types[0] = <char>NPY_FLOAT
 * ufunc__sinpi_types[1] = <char>NPY_FLOAT
 * ufunc__sinpi_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__sinpi_types[3] = <char>NPY_DOUBLE
 * ufunc__sinpi_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2227
 * ufunc__sinpi_types[1] = <char>NPY_FLOAT
 * ufunc__sinpi_types[2] = <char>NPY_DOUBLE
 * ufunc__sinpi_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__sinpi_types[4] = <char>NPY_CFLOAT
 * ufunc__sinpi_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2228
 * ufunc__sinpi_types[2] = <char>NPY_DOUBLE
 * ufunc__sinpi_types[3] = <char>NPY_DOUBLE
 * ufunc__sinpi_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc__sinpi_types[5] = <char>NPY_CFLOAT
 * ufunc__sinpi_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2229
 * ufunc__sinpi_types[3] = <char>NPY_DOUBLE
 * ufunc__sinpi_types[4] = <char>NPY_CFLOAT
 * ufunc__sinpi_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc__sinpi_types[6] = <char>NPY_CDOUBLE
 * ufunc__sinpi_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2230
 * ufunc__sinpi_types[4] = <char>NPY_CFLOAT
 * ufunc__sinpi_types[5] = <char>NPY_CFLOAT
 * ufunc__sinpi_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__sinpi_types[7] = <char>NPY_CDOUBLE
 * ufunc__sinpi_ptr[2*0] = <void*>_func_dsinpi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2231
 * ufunc__sinpi_types[5] = <char>NPY_CFLOAT
 * ufunc__sinpi_types[6] = <char>NPY_CDOUBLE
 * ufunc__sinpi_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__sinpi_ptr[2*0] = <void*>_func_dsinpi
 * ufunc__sinpi_ptr[2*0+1] = <void*>(<char*>"_sinpi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2232
 * ufunc__sinpi_types[6] = <char>NPY_CDOUBLE
 * ufunc__sinpi_types[7] = <char>NPY_CDOUBLE
 * ufunc__sinpi_ptr[2*0] = <void*>_func_dsinpi             # <<<<<<<<<<<<<<
 * ufunc__sinpi_ptr[2*0+1] = <void*>(<char*>"_sinpi")
 * ufunc__sinpi_ptr[2*1] = <void*>_func_dsinpi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_ptr[0]) = ((void *)__pyx_f_5scipy_7special_5_trig_dsinpi);

  /* "scipy/special/_ufuncs.pyx":2233
 * ufunc__sinpi_types[7] = <char>NPY_CDOUBLE
 * ufunc__sinpi_ptr[2*0] = <void*>_func_dsinpi
 * ufunc__sinpi_ptr[2*0+1] = <void*>(<char*>"_sinpi")             # <<<<<<<<<<<<<<
 * ufunc__sinpi_ptr[2*1] = <void*>_func_dsinpi
 * ufunc__sinpi_ptr[2*1+1] = <void*>(<char*>"_sinpi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_ptr[1]) = ((void *)((char *)((char *)"_sinpi")));

  /* "scipy/special/_ufuncs.pyx":2234
 * ufunc__sinpi_ptr[2*0] = <void*>_func_dsinpi
 * ufunc__sinpi_ptr[2*0+1] = <void*>(<char*>"_sinpi")
 * ufunc__sinpi_ptr[2*1] = <void*>_func_dsinpi             # <<<<<<<<<<<<<<
 * ufunc__sinpi_ptr[2*1+1] = <void*>(<char*>"_sinpi")
 * ufunc__sinpi_ptr[2*2] = <void*>_func_csinpi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_ptr[2]) = ((void *)__pyx_f_5scipy_7special_5_trig_dsinpi);

  /* "scipy/special/_ufuncs.pyx":2235
 * ufunc__sinpi_ptr[2*0+1] = <void*>(<char*>"_sinpi")
 * ufunc__sinpi_ptr[2*1] = <void*>_func_dsinpi
 * ufunc__sinpi_ptr[2*1+1] = <void*>(<char*>"_sinpi")             # <<<<<<<<<<<<<<
 * ufunc__sinpi_ptr[2*2] = <void*>_func_csinpi
 * ufunc__sinpi_ptr[2*2+1] = <void*>(<char*>"_sinpi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_ptr[3]) = ((void *)((char *)((char *)"_sinpi")));

  /* "scipy/special/_ufuncs.pyx":2236
 * ufunc__sinpi_ptr[2*1] = <void*>_func_dsinpi
 * ufunc__sinpi_ptr[2*1+1] = <void*>(<char*>"_sinpi")
 * ufunc__sinpi_ptr[2*2] = <void*>_func_csinpi             # <<<<<<<<<<<<<<
 * ufunc__sinpi_ptr[2*2+1] = <void*>(<char*>"_sinpi")
 * ufunc__sinpi_ptr[2*3] = <void*>_func_csinpi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_ptr[4]) = ((void *)__pyx_f_5scipy_7special_5_trig_csinpi);

  /* "scipy/special/_ufuncs.pyx":2237
 * ufunc__sinpi_ptr[2*1+1] = <void*>(<char*>"_sinpi")
 * ufunc__sinpi_ptr[2*2] = <void*>_func_csinpi
 * ufunc__sinpi_ptr[2*2+1] = <void*>(<char*>"_sinpi")             # <<<<<<<<<<<<<<
 * ufunc__sinpi_ptr[2*3] = <void*>_func_csinpi
 * ufunc__sinpi_ptr[2*3+1] = <void*>(<char*>"_sinpi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_ptr[5]) = ((void *)((char *)((char *)"_sinpi")));

  /* "scipy/special/_ufuncs.pyx":2238
 * ufunc__sinpi_ptr[2*2] = <void*>_func_csinpi
 * ufunc__sinpi_ptr[2*2+1] = <void*>(<char*>"_sinpi")
 * ufunc__sinpi_ptr[2*3] = <void*>_func_csinpi             # <<<<<<<<<<<<<<
 * ufunc__sinpi_ptr[2*3+1] = <void*>(<char*>"_sinpi")
 * ufunc__sinpi_data[0] = &ufunc__sinpi_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_ptr[6]) = ((void *)__pyx_f_5scipy_7special_5_trig_csinpi);

  /* "scipy/special/_ufuncs.pyx":2239
 * ufunc__sinpi_ptr[2*2+1] = <void*>(<char*>"_sinpi")
 * ufunc__sinpi_ptr[2*3] = <void*>_func_csinpi
 * ufunc__sinpi_ptr[2*3+1] = <void*>(<char*>"_sinpi")             # <<<<<<<<<<<<<<
 * ufunc__sinpi_data[0] = &ufunc__sinpi_ptr[2*0]
 * ufunc__sinpi_data[1] = &ufunc__sinpi_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_ptr[7]) = ((void *)((char *)((char *)"_sinpi")));

  /* "scipy/special/_ufuncs.pyx":2240
 * ufunc__sinpi_ptr[2*3] = <void*>_func_csinpi
 * ufunc__sinpi_ptr[2*3+1] = <void*>(<char*>"_sinpi")
 * ufunc__sinpi_data[0] = &ufunc__sinpi_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__sinpi_data[1] = &ufunc__sinpi_ptr[2*1]
 * ufunc__sinpi_data[2] = &ufunc__sinpi_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2241
 * ufunc__sinpi_ptr[2*3+1] = <void*>(<char*>"_sinpi")
 * ufunc__sinpi_data[0] = &ufunc__sinpi_ptr[2*0]
 * ufunc__sinpi_data[1] = &ufunc__sinpi_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc__sinpi_data[2] = &ufunc__sinpi_ptr[2*2]
 * ufunc__sinpi_data[3] = &ufunc__sinpi_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2242
 * ufunc__sinpi_data[0] = &ufunc__sinpi_ptr[2*0]
 * ufunc__sinpi_data[1] = &ufunc__sinpi_ptr[2*1]
 * ufunc__sinpi_data[2] = &ufunc__sinpi_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc__sinpi_data[3] = &ufunc__sinpi_ptr[2*3]
 * _sinpi = np.PyUFunc_FromFuncAndData(ufunc__sinpi_loops, ufunc__sinpi_data, ufunc__sinpi_types, 4, 1, 1, 0, "_sinpi", ufunc__sinpi_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":2243
 * ufunc__sinpi_data[1] = &ufunc__sinpi_ptr[2*1]
 * ufunc__sinpi_data[2] = &ufunc__sinpi_ptr[2*2]
 * ufunc__sinpi_data[3] = &ufunc__sinpi_ptr[2*3]             # <<<<<<<<<<<<<<
 * _sinpi = np.PyUFunc_FromFuncAndData(ufunc__sinpi_loops, ufunc__sinpi_data, ufunc__sinpi_types, 4, 1, 1, 0, "_sinpi", ufunc__sinpi_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":2244
 * ufunc__sinpi_data[2] = &ufunc__sinpi_ptr[2*2]
 * ufunc__sinpi_data[3] = &ufunc__sinpi_ptr[2*3]
 * _sinpi = np.PyUFunc_FromFuncAndData(ufunc__sinpi_loops, ufunc__sinpi_data, ufunc__sinpi_types, 4, 1, 1, 0, "_sinpi", ufunc__sinpi_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__smirnovc_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_types, 4, 1, 1, 0, ((char *)"_sinpi"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__sinpi_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sinpi, __pyx_t_2) < 0) __PYX_ERR(1, 2244, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2250
 * cdef void *ufunc__smirnovc_data[3]
 * cdef char ufunc__smirnovc_types[9]
 * cdef char *ufunc__smirnovc_doc = (             # <<<<<<<<<<<<<<
 *     "_smirnovc(n, d)\n"
 *     " Internal function, do not use.")
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_doc = ((char *)"_smirnovc(n, d)\n Internal function, do not use.");

  /* "scipy/special/_ufuncs.pyx":2253
 *     "_smirnovc(n, d)\n"
 *     " Internal function, do not use.")
 * ufunc__smirnovc_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc__smirnovc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc__smirnovc_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":2254
 *     " Internal function, do not use.")
 * ufunc__smirnovc_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc__smirnovc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc__smirnovc_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc__smirnovc_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":2255
 * ufunc__smirnovc_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc__smirnovc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc__smirnovc_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc__smirnovc_types[0] = <char>NPY_LONG
 * ufunc__smirnovc_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":2256
 * ufunc__smirnovc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc__smirnovc_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc__smirnovc_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__smirnovc_types[1] = <char>NPY_DOUBLE
 * ufunc__smirnovc_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2257
 * ufunc__smirnovc_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc__smirnovc_types[0] = <char>NPY_LONG
 * ufunc__smirnovc_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__smirnovc_types[2] = <char>NPY_DOUBLE
 * ufunc__smirnovc_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2258
 * ufunc__smirnovc_types[0] = <char>NPY_LONG
 * ufunc__smirnovc_types[1] = <char>NPY_DOUBLE
 * ufunc__smirnovc_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__smirnovc_types[3] = <char>NPY_FLOAT
 * ufunc__smirnovc_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2259
 * ufunc__smirnovc_types[1] = <char>NPY_DOUBLE
 * ufunc__smirnovc_types[2] = <char>NPY_DOUBLE
 * ufunc__smirnovc_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__smirnovc_types[4] = <char>NPY_FLOAT
 * ufunc__smirnovc_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2260
 * ufunc__smirnovc_types[2] = <char>NPY_DOUBLE
 * ufunc__smirnovc_types[3] = <char>NPY_FLOAT
 * ufunc__smirnovc_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__smirnovc_types[5] = <char>NPY_FLOAT
 * ufunc__smirnovc_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2261
 * ufunc__smirnovc_types[3] = <char>NPY_FLOAT
 * ufunc__smirnovc_types[4] = <char>NPY_FLOAT
 * ufunc__smirnovc_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__smirnovc_types[6] = <char>NPY_DOUBLE
 * ufunc__smirnovc_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2262
 * ufunc__smirnovc_types[4] = <char>NPY_FLOAT
 * ufunc__smirnovc_types[5] = <char>NPY_FLOAT
 * ufunc__smirnovc_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__smirnovc_types[7] = <char>NPY_DOUBLE
 * ufunc__smirnovc_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2263
 * ufunc__smirnovc_types[5] = <char>NPY_FLOAT
 * ufunc__smirnovc_types[6] = <char>NPY_DOUBLE
 * ufunc__smirnovc_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__smirnovc_types[8] = <char>NPY_DOUBLE
 * ufunc__smirnovc_ptr[2*0] = <void*>_func_smirnovc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2264
 * ufunc__smirnovc_types[6] = <char>NPY_DOUBLE
 * ufunc__smirnovc_types[7] = <char>NPY_DOUBLE
 * ufunc__smirnovc_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__smirnovc_ptr[2*0] = <void*>_func_smirnovc
 * ufunc__smirnovc_ptr[2*0+1] = <void*>(<char*>"_smirnovc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2265
 * ufunc__smirnovc_types[7] = <char>NPY_DOUBLE
 * ufunc__smirnovc_types[8] = <char>NPY_DOUBLE
 * ufunc__smirnovc_ptr[2*0] = <void*>_func_smirnovc             # <<<<<<<<<<<<<<
 * ufunc__smirnovc_ptr[2*0+1] = <void*>(<char*>"_smirnovc")
 * ufunc__smirnovc_ptr[2*1] = <void*>_func_smirnovc_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_ptr[0]) = ((void *)smirnovc);

  /* "scipy/special/_ufuncs.pyx":2266
 * ufunc__smirnovc_types[8] = <char>NPY_DOUBLE
 * ufunc__smirnovc_ptr[2*0] = <void*>_func_smirnovc
 * ufunc__smirnovc_ptr[2*0+1] = <void*>(<char*>"_smirnovc")             # <<<<<<<<<<<<<<
 * ufunc__smirnovc_ptr[2*1] = <void*>_func_smirnovc_unsafe
 * ufunc__smirnovc_ptr[2*1+1] = <void*>(<char*>"_smirnovc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_ptr[1]) = ((void *)((char *)((char *)"_smirnovc")));

  /* "scipy/special/_ufuncs.pyx":2267
 * ufunc__smirnovc_ptr[2*0] = <void*>_func_smirnovc
 * ufunc__smirnovc_ptr[2*0+1] = <void*>(<char*>"_smirnovc")
 * ufunc__smirnovc_ptr[2*1] = <void*>_func_smirnovc_unsafe             # <<<<<<<<<<<<<<
 * ufunc__smirnovc_ptr[2*1+1] = <void*>(<char*>"_smirnovc")
 * ufunc__smirnovc_ptr[2*2] = <void*>_func_smirnovc_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_smirnovc_unsafe);

  /* "scipy/special/_ufuncs.pyx":2268
 * ufunc__smirnovc_ptr[2*0+1] = <void*>(<char*>"_smirnovc")
 * ufunc__smirnovc_ptr[2*1] = <void*>_func_smirnovc_unsafe
 * ufunc__smirnovc_ptr[2*1+1] = <void*>(<char*>"_smirnovc")             # <<<<<<<<<<<<<<
 * ufunc__smirnovc_ptr[2*2] = <void*>_func_smirnovc_unsafe
 * ufunc__smirnovc_ptr[2*2+1] = <void*>(<char*>"_smirnovc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_ptr[3]) = ((void *)((char *)((char *)"_smirnovc")));

  /* "scipy/special/_ufuncs.pyx":2269
 * ufunc__smirnovc_ptr[2*1] = <void*>_func_smirnovc_unsafe
 * ufunc__smirnovc_ptr[2*1+1] = <void*>(<char*>"_smirnovc")
 * ufunc__smirnovc_ptr[2*2] = <void*>_func_smirnovc_unsafe             # <<<<<<<<<<<<<<
 * ufunc__smirnovc_ptr[2*2+1] = <void*>(<char*>"_smirnovc")
 * ufunc__smirnovc_data[0] = &ufunc__smirnovc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_smirnovc_unsafe);

  /* "scipy/special/_ufuncs.pyx":2270
 * ufunc__smirnovc_ptr[2*1+1] = <void*>(<char*>"_smirnovc")
 * ufunc__smirnovc_ptr[2*2] = <void*>_func_smirnovc_unsafe
 * ufunc__smirnovc_ptr[2*2+1] = <void*>(<char*>"_smirnovc")             # <<<<<<<<<<<<<<
 * ufunc__smirnovc_data[0] = &ufunc__smirnovc_ptr[2*0]
 * ufunc__smirnovc_data[1] = &ufunc__smirnovc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_ptr[5]) = ((void *)((char *)((char *)"_smirnovc")));

  /* "scipy/special/_ufuncs.pyx":2271
 * ufunc__smirnovc_ptr[2*2] = <void*>_func_smirnovc_unsafe
 * ufunc__smirnovc_ptr[2*2+1] = <void*>(<char*>"_smirnovc")
 * ufunc__smirnovc_data[0] = &ufunc__smirnovc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__smirnovc_data[1] = &ufunc__smirnovc_ptr[2*1]
 * ufunc__smirnovc_data[2] = &ufunc__smirnovc_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2272
 * ufunc__smirnovc_ptr[2*2+1] = <void*>(<char*>"_smirnovc")
 * ufunc__smirnovc_data[0] = &ufunc__smirnovc_ptr[2*0]
 * ufunc__smirnovc_data[1] = &ufunc__smirnovc_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc__smirnovc_data[2] = &ufunc__smirnovc_ptr[2*2]
 * _smirnovc = np.PyUFunc_FromFuncAndData(ufunc__smirnovc_loops, ufunc__smirnovc_data, ufunc__smirnovc_types, 3, 2, 1, 0, "_smirnovc", ufunc__smirnovc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2273
 * ufunc__smirnovc_data[0] = &ufunc__smirnovc_ptr[2*0]
 * ufunc__smirnovc_data[1] = &ufunc__smirnovc_ptr[2*1]
 * ufunc__smirnovc_data[2] = &ufunc__smirnovc_ptr[2*2]             # <<<<<<<<<<<<<<
 * _smirnovc = np.PyUFunc_FromFuncAndData(ufunc__smirnovc_loops, ufunc__smirnovc_data, ufunc__smirnovc_types, 3, 2, 1, 0, "_smirnovc", ufunc__smirnovc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":2274
 * ufunc__smirnovc_data[1] = &ufunc__smirnovc_ptr[2*1]
 * ufunc__smirnovc_data[2] = &ufunc__smirnovc_ptr[2*2]
 * _smirnovc = np.PyUFunc_FromFuncAndData(ufunc__smirnovc_loops, ufunc__smirnovc_data, ufunc__smirnovc_types, 3, 2, 1, 0, "_smirnovc", ufunc__smirnovc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__smirnovci_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_types, 3, 2, 1, 0, ((char *)"_smirnovc"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovc_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2274, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_smirnovc, __pyx_t_2) < 0) __PYX_ERR(1, 2274, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2280
 * cdef void *ufunc__smirnovci_data[3]
 * cdef char ufunc__smirnovci_types[9]
 * cdef char *ufunc__smirnovci_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, do not use.")
 * ufunc__smirnovci_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_doc = ((char *)"Internal function, do not use.");

  /* "scipy/special/_ufuncs.pyx":2282
 * cdef char *ufunc__smirnovci_doc = (
 *     "Internal function, do not use.")
 * ufunc__smirnovci_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc__smirnovci_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc__smirnovci_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":2283
 *     "Internal function, do not use.")
 * ufunc__smirnovci_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc__smirnovci_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc__smirnovci_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc__smirnovci_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":2284
 * ufunc__smirnovci_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc__smirnovci_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc__smirnovci_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc__smirnovci_types[0] = <char>NPY_LONG
 * ufunc__smirnovci_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":2285
 * ufunc__smirnovci_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc__smirnovci_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc__smirnovci_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__smirnovci_types[1] = <char>NPY_DOUBLE
 * ufunc__smirnovci_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2286
 * ufunc__smirnovci_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc__smirnovci_types[0] = <char>NPY_LONG
 * ufunc__smirnovci_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__smirnovci_types[2] = <char>NPY_DOUBLE
 * ufunc__smirnovci_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2287
 * ufunc__smirnovci_types[0] = <char>NPY_LONG
 * ufunc__smirnovci_types[1] = <char>NPY_DOUBLE
 * ufunc__smirnovci_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__smirnovci_types[3] = <char>NPY_FLOAT
 * ufunc__smirnovci_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2288
 * ufunc__smirnovci_types[1] = <char>NPY_DOUBLE
 * ufunc__smirnovci_types[2] = <char>NPY_DOUBLE
 * ufunc__smirnovci_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__smirnovci_types[4] = <char>NPY_FLOAT
 * ufunc__smirnovci_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2289
 * ufunc__smirnovci_types[2] = <char>NPY_DOUBLE
 * ufunc__smirnovci_types[3] = <char>NPY_FLOAT
 * ufunc__smirnovci_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__smirnovci_types[5] = <char>NPY_FLOAT
 * ufunc__smirnovci_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2290
 * ufunc__smirnovci_types[3] = <char>NPY_FLOAT
 * ufunc__smirnovci_types[4] = <char>NPY_FLOAT
 * ufunc__smirnovci_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__smirnovci_types[6] = <char>NPY_DOUBLE
 * ufunc__smirnovci_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2291
 * ufunc__smirnovci_types[4] = <char>NPY_FLOAT
 * ufunc__smirnovci_types[5] = <char>NPY_FLOAT
 * ufunc__smirnovci_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__smirnovci_types[7] = <char>NPY_DOUBLE
 * ufunc__smirnovci_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2292
 * ufunc__smirnovci_types[5] = <char>NPY_FLOAT
 * ufunc__smirnovci_types[6] = <char>NPY_DOUBLE
 * ufunc__smirnovci_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__smirnovci_types[8] = <char>NPY_DOUBLE
 * ufunc__smirnovci_ptr[2*0] = <void*>_func_smirnovci
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2293
 * ufunc__smirnovci_types[6] = <char>NPY_DOUBLE
 * ufunc__smirnovci_types[7] = <char>NPY_DOUBLE
 * ufunc__smirnovci_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__smirnovci_ptr[2*0] = <void*>_func_smirnovci
 * ufunc__smirnovci_ptr[2*0+1] = <void*>(<char*>"_smirnovci")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2294
 * ufunc__smirnovci_types[7] = <char>NPY_DOUBLE
 * ufunc__smirnovci_types[8] = <char>NPY_DOUBLE
 * ufunc__smirnovci_ptr[2*0] = <void*>_func_smirnovci             # <<<<<<<<<<<<<<
 * ufunc__smirnovci_ptr[2*0+1] = <void*>(<char*>"_smirnovci")
 * ufunc__smirnovci_ptr[2*1] = <void*>_func_smirnovci_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_ptr[0]) = ((void *)smirnovci);

  /* "scipy/special/_ufuncs.pyx":2295
 * ufunc__smirnovci_types[8] = <char>NPY_DOUBLE
 * ufunc__smirnovci_ptr[2*0] = <void*>_func_smirnovci
 * ufunc__smirnovci_ptr[2*0+1] = <void*>(<char*>"_smirnovci")             # <<<<<<<<<<<<<<
 * ufunc__smirnovci_ptr[2*1] = <void*>_func_smirnovci_unsafe
 * ufunc__smirnovci_ptr[2*1+1] = <void*>(<char*>"_smirnovci")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_ptr[1]) = ((void *)((char *)((char *)"_smirnovci")));

  /* "scipy/special/_ufuncs.pyx":2296
 * ufunc__smirnovci_ptr[2*0] = <void*>_func_smirnovci
 * ufunc__smirnovci_ptr[2*0+1] = <void*>(<char*>"_smirnovci")
 * ufunc__smirnovci_ptr[2*1] = <void*>_func_smirnovci_unsafe             # <<<<<<<<<<<<<<
 * ufunc__smirnovci_ptr[2*1+1] = <void*>(<char*>"_smirnovci")
 * ufunc__smirnovci_ptr[2*2] = <void*>_func_smirnovci_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_smirnovci_unsafe);

  /* "scipy/special/_ufuncs.pyx":2297
 * ufunc__smirnovci_ptr[2*0+1] = <void*>(<char*>"_smirnovci")
 * ufunc__smirnovci_ptr[2*1] = <void*>_func_smirnovci_unsafe
 * ufunc__smirnovci_ptr[2*1+1] = <void*>(<char*>"_smirnovci")             # <<<<<<<<<<<<<<
 * ufunc__smirnovci_ptr[2*2] = <void*>_func_smirnovci_unsafe
 * ufunc__smirnovci_ptr[2*2+1] = <void*>(<char*>"_smirnovci")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_ptr[3]) = ((void *)((char *)((char *)"_smirnovci")));

  /* "scipy/special/_ufuncs.pyx":2298
 * ufunc__smirnovci_ptr[2*1] = <void*>_func_smirnovci_unsafe
 * ufunc__smirnovci_ptr[2*1+1] = <void*>(<char*>"_smirnovci")
 * ufunc__smirnovci_ptr[2*2] = <void*>_func_smirnovci_unsafe             # <<<<<<<<<<<<<<
 * ufunc__smirnovci_ptr[2*2+1] = <void*>(<char*>"_smirnovci")
 * ufunc__smirnovci_data[0] = &ufunc__smirnovci_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_smirnovci_unsafe);

  /* "scipy/special/_ufuncs.pyx":2299
 * ufunc__smirnovci_ptr[2*1+1] = <void*>(<char*>"_smirnovci")
 * ufunc__smirnovci_ptr[2*2] = <void*>_func_smirnovci_unsafe
 * ufunc__smirnovci_ptr[2*2+1] = <void*>(<char*>"_smirnovci")             # <<<<<<<<<<<<<<
 * ufunc__smirnovci_data[0] = &ufunc__smirnovci_ptr[2*0]
 * ufunc__smirnovci_data[1] = &ufunc__smirnovci_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_ptr[5]) = ((void *)((char *)((char *)"_smirnovci")));

  /* "scipy/special/_ufuncs.pyx":2300
 * ufunc__smirnovci_ptr[2*2] = <void*>_func_smirnovci_unsafe
 * ufunc__smirnovci_ptr[2*2+1] = <void*>(<char*>"_smirnovci")
 * ufunc__smirnovci_data[0] = &ufunc__smirnovci_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__smirnovci_data[1] = &ufunc__smirnovci_ptr[2*1]
 * ufunc__smirnovci_data[2] = &ufunc__smirnovci_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2301
 * ufunc__smirnovci_ptr[2*2+1] = <void*>(<char*>"_smirnovci")
 * ufunc__smirnovci_data[0] = &ufunc__smirnovci_ptr[2*0]
 * ufunc__smirnovci_data[1] = &ufunc__smirnovci_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc__smirnovci_data[2] = &ufunc__smirnovci_ptr[2*2]
 * _smirnovci = np.PyUFunc_FromFuncAndData(ufunc__smirnovci_loops, ufunc__smirnovci_data, ufunc__smirnovci_types, 3, 2, 1, 0, "_smirnovci", ufunc__smirnovci_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2302
 * ufunc__smirnovci_data[0] = &ufunc__smirnovci_ptr[2*0]
 * ufunc__smirnovci_data[1] = &ufunc__smirnovci_ptr[2*1]
 * ufunc__smirnovci_data[2] = &ufunc__smirnovci_ptr[2*2]             # <<<<<<<<<<<<<<
 * _smirnovci = np.PyUFunc_FromFuncAndData(ufunc__smirnovci_loops, ufunc__smirnovci_data, ufunc__smirnovci_types, 3, 2, 1, 0, "_smirnovci", ufunc__smirnovci_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":2303
 * ufunc__smirnovci_data[1] = &ufunc__smirnovci_ptr[2*1]
 * ufunc__smirnovci_data[2] = &ufunc__smirnovci_ptr[2*2]
 * _smirnovci = np.PyUFunc_FromFuncAndData(ufunc__smirnovci_loops, ufunc__smirnovci_data, ufunc__smirnovci_types, 3, 2, 1, 0, "_smirnovci", ufunc__smirnovci_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__smirnovp_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_types, 3, 2, 1, 0, ((char *)"_smirnovci"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovci_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2303, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_smirnovci, __pyx_t_2) < 0) __PYX_ERR(1, 2303, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2309
 * cdef void *ufunc__smirnovp_data[3]
 * cdef char ufunc__smirnovp_types[9]
 * cdef char *ufunc__smirnovp_doc = (             # <<<<<<<<<<<<<<
 *     "_smirnovp(n, p)\n"
 *     " Internal function, do not use.")
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_doc = ((char *)"_smirnovp(n, p)\n Internal function, do not use.");

  /* "scipy/special/_ufuncs.pyx":2312
 *     "_smirnovp(n, p)\n"
 *     " Internal function, do not use.")
 * ufunc__smirnovp_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc__smirnovp_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc__smirnovp_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":2313
 *     " Internal function, do not use.")
 * ufunc__smirnovp_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc__smirnovp_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc__smirnovp_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc__smirnovp_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":2314
 * ufunc__smirnovp_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc__smirnovp_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc__smirnovp_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc__smirnovp_types[0] = <char>NPY_LONG
 * ufunc__smirnovp_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":2315
 * ufunc__smirnovp_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc__smirnovp_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc__smirnovp_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__smirnovp_types[1] = <char>NPY_DOUBLE
 * ufunc__smirnovp_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2316
 * ufunc__smirnovp_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc__smirnovp_types[0] = <char>NPY_LONG
 * ufunc__smirnovp_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__smirnovp_types[2] = <char>NPY_DOUBLE
 * ufunc__smirnovp_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2317
 * ufunc__smirnovp_types[0] = <char>NPY_LONG
 * ufunc__smirnovp_types[1] = <char>NPY_DOUBLE
 * ufunc__smirnovp_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__smirnovp_types[3] = <char>NPY_FLOAT
 * ufunc__smirnovp_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2318
 * ufunc__smirnovp_types[1] = <char>NPY_DOUBLE
 * ufunc__smirnovp_types[2] = <char>NPY_DOUBLE
 * ufunc__smirnovp_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__smirnovp_types[4] = <char>NPY_FLOAT
 * ufunc__smirnovp_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2319
 * ufunc__smirnovp_types[2] = <char>NPY_DOUBLE
 * ufunc__smirnovp_types[3] = <char>NPY_FLOAT
 * ufunc__smirnovp_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__smirnovp_types[5] = <char>NPY_FLOAT
 * ufunc__smirnovp_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2320
 * ufunc__smirnovp_types[3] = <char>NPY_FLOAT
 * ufunc__smirnovp_types[4] = <char>NPY_FLOAT
 * ufunc__smirnovp_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__smirnovp_types[6] = <char>NPY_DOUBLE
 * ufunc__smirnovp_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2321
 * ufunc__smirnovp_types[4] = <char>NPY_FLOAT
 * ufunc__smirnovp_types[5] = <char>NPY_FLOAT
 * ufunc__smirnovp_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__smirnovp_types[7] = <char>NPY_DOUBLE
 * ufunc__smirnovp_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2322
 * ufunc__smirnovp_types[5] = <char>NPY_FLOAT
 * ufunc__smirnovp_types[6] = <char>NPY_DOUBLE
 * ufunc__smirnovp_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__smirnovp_types[8] = <char>NPY_DOUBLE
 * ufunc__smirnovp_ptr[2*0] = <void*>_func_smirnovp
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2323
 * ufunc__smirnovp_types[6] = <char>NPY_DOUBLE
 * ufunc__smirnovp_types[7] = <char>NPY_DOUBLE
 * ufunc__smirnovp_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__smirnovp_ptr[2*0] = <void*>_func_smirnovp
 * ufunc__smirnovp_ptr[2*0+1] = <void*>(<char*>"_smirnovp")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2324
 * ufunc__smirnovp_types[7] = <char>NPY_DOUBLE
 * ufunc__smirnovp_types[8] = <char>NPY_DOUBLE
 * ufunc__smirnovp_ptr[2*0] = <void*>_func_smirnovp             # <<<<<<<<<<<<<<
 * ufunc__smirnovp_ptr[2*0+1] = <void*>(<char*>"_smirnovp")
 * ufunc__smirnovp_ptr[2*1] = <void*>_func_smirnovp_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_ptr[0]) = ((void *)smirnovp);

  /* "scipy/special/_ufuncs.pyx":2325
 * ufunc__smirnovp_types[8] = <char>NPY_DOUBLE
 * ufunc__smirnovp_ptr[2*0] = <void*>_func_smirnovp
 * ufunc__smirnovp_ptr[2*0+1] = <void*>(<char*>"_smirnovp")             # <<<<<<<<<<<<<<
 * ufunc__smirnovp_ptr[2*1] = <void*>_func_smirnovp_unsafe
 * ufunc__smirnovp_ptr[2*1+1] = <void*>(<char*>"_smirnovp")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_ptr[1]) = ((void *)((char *)((char *)"_smirnovp")));

  /* "scipy/special/_ufuncs.pyx":2326
 * ufunc__smirnovp_ptr[2*0] = <void*>_func_smirnovp
 * ufunc__smirnovp_ptr[2*0+1] = <void*>(<char*>"_smirnovp")
 * ufunc__smirnovp_ptr[2*1] = <void*>_func_smirnovp_unsafe             # <<<<<<<<<<<<<<
 * ufunc__smirnovp_ptr[2*1+1] = <void*>(<char*>"_smirnovp")
 * ufunc__smirnovp_ptr[2*2] = <void*>_func_smirnovp_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_smirnovp_unsafe);

  /* "scipy/special/_ufuncs.pyx":2327
 * ufunc__smirnovp_ptr[2*0+1] = <void*>(<char*>"_smirnovp")
 * ufunc__smirnovp_ptr[2*1] = <void*>_func_smirnovp_unsafe
 * ufunc__smirnovp_ptr[2*1+1] = <void*>(<char*>"_smirnovp")             # <<<<<<<<<<<<<<
 * ufunc__smirnovp_ptr[2*2] = <void*>_func_smirnovp_unsafe
 * ufunc__smirnovp_ptr[2*2+1] = <void*>(<char*>"_smirnovp")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_ptr[3]) = ((void *)((char *)((char *)"_smirnovp")));

  /* "scipy/special/_ufuncs.pyx":2328
 * ufunc__smirnovp_ptr[2*1] = <void*>_func_smirnovp_unsafe
 * ufunc__smirnovp_ptr[2*1+1] = <void*>(<char*>"_smirnovp")
 * ufunc__smirnovp_ptr[2*2] = <void*>_func_smirnovp_unsafe             # <<<<<<<<<<<<<<
 * ufunc__smirnovp_ptr[2*2+1] = <void*>(<char*>"_smirnovp")
 * ufunc__smirnovp_data[0] = &ufunc__smirnovp_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_smirnovp_unsafe);

  /* "scipy/special/_ufuncs.pyx":2329
 * ufunc__smirnovp_ptr[2*1+1] = <void*>(<char*>"_smirnovp")
 * ufunc__smirnovp_ptr[2*2] = <void*>_func_smirnovp_unsafe
 * ufunc__smirnovp_ptr[2*2+1] = <void*>(<char*>"_smirnovp")             # <<<<<<<<<<<<<<
 * ufunc__smirnovp_data[0] = &ufunc__smirnovp_ptr[2*0]
 * ufunc__smirnovp_data[1] = &ufunc__smirnovp_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_ptr[5]) = ((void *)((char *)((char *)"_smirnovp")));

  /* "scipy/special/_ufuncs.pyx":2330
 * ufunc__smirnovp_ptr[2*2] = <void*>_func_smirnovp_unsafe
 * ufunc__smirnovp_ptr[2*2+1] = <void*>(<char*>"_smirnovp")
 * ufunc__smirnovp_data[0] = &ufunc__smirnovp_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__smirnovp_data[1] = &ufunc__smirnovp_ptr[2*1]
 * ufunc__smirnovp_data[2] = &ufunc__smirnovp_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2331
 * ufunc__smirnovp_ptr[2*2+1] = <void*>(<char*>"_smirnovp")
 * ufunc__smirnovp_data[0] = &ufunc__smirnovp_ptr[2*0]
 * ufunc__smirnovp_data[1] = &ufunc__smirnovp_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc__smirnovp_data[2] = &ufunc__smirnovp_ptr[2*2]
 * _smirnovp = np.PyUFunc_FromFuncAndData(ufunc__smirnovp_loops, ufunc__smirnovp_data, ufunc__smirnovp_types, 3, 2, 1, 0, "_smirnovp", ufunc__smirnovp_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2332
 * ufunc__smirnovp_data[0] = &ufunc__smirnovp_ptr[2*0]
 * ufunc__smirnovp_data[1] = &ufunc__smirnovp_ptr[2*1]
 * ufunc__smirnovp_data[2] = &ufunc__smirnovp_ptr[2*2]             # <<<<<<<<<<<<<<
 * _smirnovp = np.PyUFunc_FromFuncAndData(ufunc__smirnovp_loops, ufunc__smirnovp_data, ufunc__smirnovp_types, 3, 2, 1, 0, "_smirnovp", ufunc__smirnovp_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":2333
 * ufunc__smirnovp_data[1] = &ufunc__smirnovp_ptr[2*1]
 * ufunc__smirnovp_data[2] = &ufunc__smirnovp_ptr[2*2]
 * _smirnovp = np.PyUFunc_FromFuncAndData(ufunc__smirnovp_loops, ufunc__smirnovp_data, ufunc__smirnovp_types, 3, 2, 1, 0, "_smirnovp", ufunc__smirnovp_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__spherical_in_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_types, 3, 2, 1, 0, ((char *)"_smirnovp"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__smirnovp_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2333, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_smirnovp, __pyx_t_2) < 0) __PYX_ERR(1, 2333, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2339
 * cdef void *ufunc__spherical_in_data[2]
 * cdef char ufunc__spherical_in_types[6]
 * cdef char *ufunc__spherical_in_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, use `spherical_in` instead.")
 * ufunc__spherical_in_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_doc = ((char *)"Internal function, use `spherical_in` instead.");

  /* "scipy/special/_ufuncs.pyx":2341
 * cdef char *ufunc__spherical_in_doc = (
 *     "Internal function, use `spherical_in` instead.")
 * ufunc__spherical_in_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_in_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":2342
 *     "Internal function, use `spherical_in` instead.")
 * ufunc__spherical_in_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc__spherical_in_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_types[0] = <char>NPY_LONG
 * ufunc__spherical_in_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_lD__As_lD_D);

  /* "scipy/special/_ufuncs.pyx":2343
 * ufunc__spherical_in_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc__spherical_in_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_in_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_in_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2344
 * ufunc__spherical_in_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_in_types[0] = <char>NPY_LONG
 * ufunc__spherical_in_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_in_types[3] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2345
 * ufunc__spherical_in_types[0] = <char>NPY_LONG
 * ufunc__spherical_in_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_in_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_types[3] = <char>NPY_LONG
 * ufunc__spherical_in_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2346
 * ufunc__spherical_in_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_in_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_in_types[3] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_in_types[5] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_types[3]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2347
 * ufunc__spherical_in_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_in_types[3] = <char>NPY_LONG
 * ufunc__spherical_in_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_in_ptr[2*0] = <void*>_func_spherical_in_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2348
 * ufunc__spherical_in_types[3] = <char>NPY_LONG
 * ufunc__spherical_in_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_in_types[5] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_ptr[2*0] = <void*>_func_spherical_in_real
 * ufunc__spherical_in_ptr[2*0+1] = <void*>(<char*>"_spherical_in")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_types[5]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2349
 * ufunc__spherical_in_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_in_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_in_ptr[2*0] = <void*>_func_spherical_in_real             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_ptr[2*0+1] = <void*>(<char*>"_spherical_in")
 * ufunc__spherical_in_ptr[2*1] = <void*>_func_spherical_in_complex
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_ptr[0]) = ((void *)__pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_real);

  /* "scipy/special/_ufuncs.pyx":2350
 * ufunc__spherical_in_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_in_ptr[2*0] = <void*>_func_spherical_in_real
 * ufunc__spherical_in_ptr[2*0+1] = <void*>(<char*>"_spherical_in")             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_ptr[2*1] = <void*>_func_spherical_in_complex
 * ufunc__spherical_in_ptr[2*1+1] = <void*>(<char*>"_spherical_in")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_ptr[1]) = ((void *)((char *)((char *)"_spherical_in")));

  /* "scipy/special/_ufuncs.pyx":2351
 * ufunc__spherical_in_ptr[2*0] = <void*>_func_spherical_in_real
 * ufunc__spherical_in_ptr[2*0+1] = <void*>(<char*>"_spherical_in")
 * ufunc__spherical_in_ptr[2*1] = <void*>_func_spherical_in_complex             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_ptr[2*1+1] = <void*>(<char*>"_spherical_in")
 * ufunc__spherical_in_data[0] = &ufunc__spherical_in_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_ptr[2]) = ((void *)__pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_complex);

  /* "scipy/special/_ufuncs.pyx":2352
 * ufunc__spherical_in_ptr[2*0+1] = <void*>(<char*>"_spherical_in")
 * ufunc__spherical_in_ptr[2*1] = <void*>_func_spherical_in_complex
 * ufunc__spherical_in_ptr[2*1+1] = <void*>(<char*>"_spherical_in")             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_data[0] = &ufunc__spherical_in_ptr[2*0]
 * ufunc__spherical_in_data[1] = &ufunc__spherical_in_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_ptr[3]) = ((void *)((char *)((char *)"_spherical_in")));

  /* "scipy/special/_ufuncs.pyx":2353
 * ufunc__spherical_in_ptr[2*1] = <void*>_func_spherical_in_complex
 * ufunc__spherical_in_ptr[2*1+1] = <void*>(<char*>"_spherical_in")
 * ufunc__spherical_in_data[0] = &ufunc__spherical_in_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_data[1] = &ufunc__spherical_in_ptr[2*1]
 * _spherical_in = np.PyUFunc_FromFuncAndData(ufunc__spherical_in_loops, ufunc__spherical_in_data, ufunc__spherical_in_types, 2, 2, 1, 0, "_spherical_in", ufunc__spherical_in_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2354
 * ufunc__spherical_in_ptr[2*1+1] = <void*>(<char*>"_spherical_in")
 * ufunc__spherical_in_data[0] = &ufunc__spherical_in_ptr[2*0]
 * ufunc__spherical_in_data[1] = &ufunc__spherical_in_ptr[2*1]             # <<<<<<<<<<<<<<
 * _spherical_in = np.PyUFunc_FromFuncAndData(ufunc__spherical_in_loops, ufunc__spherical_in_data, ufunc__spherical_in_types, 2, 2, 1, 0, "_spherical_in", ufunc__spherical_in_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2355
 * ufunc__spherical_in_data[0] = &ufunc__spherical_in_ptr[2*0]
 * ufunc__spherical_in_data[1] = &ufunc__spherical_in_ptr[2*1]
 * _spherical_in = np.PyUFunc_FromFuncAndData(ufunc__spherical_in_loops, ufunc__spherical_in_data, ufunc__spherical_in_types, 2, 2, 1, 0, "_spherical_in", ufunc__spherical_in_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__spherical_in_d_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_types, 2, 2, 1, 0, ((char *)"_spherical_in"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2355, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_spherical_in, __pyx_t_2) < 0) __PYX_ERR(1, 2355, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2361
 * cdef void *ufunc__spherical_in_d_data[2]
 * cdef char ufunc__spherical_in_d_types[6]
 * cdef char *ufunc__spherical_in_d_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, use `spherical_in` instead.")
 * ufunc__spherical_in_d_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_doc = ((char *)"Internal function, use `spherical_in` instead.");

  /* "scipy/special/_ufuncs.pyx":2363
 * cdef char *ufunc__spherical_in_d_doc = (
 *     "Internal function, use `spherical_in` instead.")
 * ufunc__spherical_in_d_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_d_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_in_d_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":2364
 *     "Internal function, use `spherical_in` instead.")
 * ufunc__spherical_in_d_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc__spherical_in_d_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_d_types[0] = <char>NPY_LONG
 * ufunc__spherical_in_d_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_lD__As_lD_D);

  /* "scipy/special/_ufuncs.pyx":2365
 * ufunc__spherical_in_d_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc__spherical_in_d_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_in_d_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_d_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_in_d_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2366
 * ufunc__spherical_in_d_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_in_d_types[0] = <char>NPY_LONG
 * ufunc__spherical_in_d_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_d_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_in_d_types[3] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2367
 * ufunc__spherical_in_d_types[0] = <char>NPY_LONG
 * ufunc__spherical_in_d_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_in_d_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_d_types[3] = <char>NPY_LONG
 * ufunc__spherical_in_d_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2368
 * ufunc__spherical_in_d_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_in_d_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_in_d_types[3] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_d_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_in_d_types[5] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_types[3]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2369
 * ufunc__spherical_in_d_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_in_d_types[3] = <char>NPY_LONG
 * ufunc__spherical_in_d_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_d_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_in_d_ptr[2*0] = <void*>_func_spherical_in_d_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2370
 * ufunc__spherical_in_d_types[3] = <char>NPY_LONG
 * ufunc__spherical_in_d_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_in_d_types[5] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_d_ptr[2*0] = <void*>_func_spherical_in_d_real
 * ufunc__spherical_in_d_ptr[2*0+1] = <void*>(<char*>"_spherical_in_d")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_types[5]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2371
 * ufunc__spherical_in_d_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_in_d_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_in_d_ptr[2*0] = <void*>_func_spherical_in_d_real             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_d_ptr[2*0+1] = <void*>(<char*>"_spherical_in_d")
 * ufunc__spherical_in_d_ptr[2*1] = <void*>_func_spherical_in_d_complex
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_ptr[0]) = ((void *)__pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_d_real);

  /* "scipy/special/_ufuncs.pyx":2372
 * ufunc__spherical_in_d_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_in_d_ptr[2*0] = <void*>_func_spherical_in_d_real
 * ufunc__spherical_in_d_ptr[2*0+1] = <void*>(<char*>"_spherical_in_d")             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_d_ptr[2*1] = <void*>_func_spherical_in_d_complex
 * ufunc__spherical_in_d_ptr[2*1+1] = <void*>(<char*>"_spherical_in_d")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_ptr[1]) = ((void *)((char *)((char *)"_spherical_in_d")));

  /* "scipy/special/_ufuncs.pyx":2373
 * ufunc__spherical_in_d_ptr[2*0] = <void*>_func_spherical_in_d_real
 * ufunc__spherical_in_d_ptr[2*0+1] = <void*>(<char*>"_spherical_in_d")
 * ufunc__spherical_in_d_ptr[2*1] = <void*>_func_spherical_in_d_complex             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_d_ptr[2*1+1] = <void*>(<char*>"_spherical_in_d")
 * ufunc__spherical_in_d_data[0] = &ufunc__spherical_in_d_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_ptr[2]) = ((void *)__pyx_f_5scipy_7special_17_spherical_bessel_spherical_in_d_complex);

  /* "scipy/special/_ufuncs.pyx":2374
 * ufunc__spherical_in_d_ptr[2*0+1] = <void*>(<char*>"_spherical_in_d")
 * ufunc__spherical_in_d_ptr[2*1] = <void*>_func_spherical_in_d_complex
 * ufunc__spherical_in_d_ptr[2*1+1] = <void*>(<char*>"_spherical_in_d")             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_d_data[0] = &ufunc__spherical_in_d_ptr[2*0]
 * ufunc__spherical_in_d_data[1] = &ufunc__spherical_in_d_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_ptr[3]) = ((void *)((char *)((char *)"_spherical_in_d")));

  /* "scipy/special/_ufuncs.pyx":2375
 * ufunc__spherical_in_d_ptr[2*1] = <void*>_func_spherical_in_d_complex
 * ufunc__spherical_in_d_ptr[2*1+1] = <void*>(<char*>"_spherical_in_d")
 * ufunc__spherical_in_d_data[0] = &ufunc__spherical_in_d_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__spherical_in_d_data[1] = &ufunc__spherical_in_d_ptr[2*1]
 * _spherical_in_d = np.PyUFunc_FromFuncAndData(ufunc__spherical_in_d_loops, ufunc__spherical_in_d_data, ufunc__spherical_in_d_types, 2, 2, 1, 0, "_spherical_in_d", ufunc__spherical_in_d_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2376
 * ufunc__spherical_in_d_ptr[2*1+1] = <void*>(<char*>"_spherical_in_d")
 * ufunc__spherical_in_d_data[0] = &ufunc__spherical_in_d_ptr[2*0]
 * ufunc__spherical_in_d_data[1] = &ufunc__spherical_in_d_ptr[2*1]             # <<<<<<<<<<<<<<
 * _spherical_in_d = np.PyUFunc_FromFuncAndData(ufunc__spherical_in_d_loops, ufunc__spherical_in_d_data, ufunc__spherical_in_d_types, 2, 2, 1, 0, "_spherical_in_d", ufunc__spherical_in_d_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2377
 * ufunc__spherical_in_d_data[0] = &ufunc__spherical_in_d_ptr[2*0]
 * ufunc__spherical_in_d_data[1] = &ufunc__spherical_in_d_ptr[2*1]
 * _spherical_in_d = np.PyUFunc_FromFuncAndData(ufunc__spherical_in_d_loops, ufunc__spherical_in_d_data, ufunc__spherical_in_d_types, 2, 2, 1, 0, "_spherical_in_d", ufunc__spherical_in_d_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__spherical_jn_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_types, 2, 2, 1, 0, ((char *)"_spherical_in_d"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_in_d_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2377, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_spherical_in_d, __pyx_t_2) < 0) __PYX_ERR(1, 2377, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2383
 * cdef void *ufunc__spherical_jn_data[2]
 * cdef char ufunc__spherical_jn_types[6]
 * cdef char *ufunc__spherical_jn_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, use `spherical_jn` instead.")
 * ufunc__spherical_jn_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_doc = ((char *)"Internal function, use `spherical_jn` instead.");

  /* "scipy/special/_ufuncs.pyx":2385
 * cdef char *ufunc__spherical_jn_doc = (
 *     "Internal function, use `spherical_jn` instead.")
 * ufunc__spherical_jn_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_jn_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":2386
 *     "Internal function, use `spherical_jn` instead.")
 * ufunc__spherical_jn_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc__spherical_jn_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_types[0] = <char>NPY_LONG
 * ufunc__spherical_jn_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_lD__As_lD_D);

  /* "scipy/special/_ufuncs.pyx":2387
 * ufunc__spherical_jn_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc__spherical_jn_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_jn_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_jn_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2388
 * ufunc__spherical_jn_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_jn_types[0] = <char>NPY_LONG
 * ufunc__spherical_jn_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_jn_types[3] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2389
 * ufunc__spherical_jn_types[0] = <char>NPY_LONG
 * ufunc__spherical_jn_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_jn_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_types[3] = <char>NPY_LONG
 * ufunc__spherical_jn_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2390
 * ufunc__spherical_jn_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_jn_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_jn_types[3] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_jn_types[5] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_types[3]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2391
 * ufunc__spherical_jn_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_jn_types[3] = <char>NPY_LONG
 * ufunc__spherical_jn_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_jn_ptr[2*0] = <void*>_func_spherical_jn_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2392
 * ufunc__spherical_jn_types[3] = <char>NPY_LONG
 * ufunc__spherical_jn_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_jn_types[5] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_ptr[2*0] = <void*>_func_spherical_jn_real
 * ufunc__spherical_jn_ptr[2*0+1] = <void*>(<char*>"_spherical_jn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_types[5]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2393
 * ufunc__spherical_jn_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_jn_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_jn_ptr[2*0] = <void*>_func_spherical_jn_real             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_ptr[2*0+1] = <void*>(<char*>"_spherical_jn")
 * ufunc__spherical_jn_ptr[2*1] = <void*>_func_spherical_jn_complex
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_ptr[0]) = ((void *)__pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_real);

  /* "scipy/special/_ufuncs.pyx":2394
 * ufunc__spherical_jn_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_jn_ptr[2*0] = <void*>_func_spherical_jn_real
 * ufunc__spherical_jn_ptr[2*0+1] = <void*>(<char*>"_spherical_jn")             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_ptr[2*1] = <void*>_func_spherical_jn_complex
 * ufunc__spherical_jn_ptr[2*1+1] = <void*>(<char*>"_spherical_jn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_ptr[1]) = ((void *)((char *)((char *)"_spherical_jn")));

  /* "scipy/special/_ufuncs.pyx":2395
 * ufunc__spherical_jn_ptr[2*0] = <void*>_func_spherical_jn_real
 * ufunc__spherical_jn_ptr[2*0+1] = <void*>(<char*>"_spherical_jn")
 * ufunc__spherical_jn_ptr[2*1] = <void*>_func_spherical_jn_complex             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_ptr[2*1+1] = <void*>(<char*>"_spherical_jn")
 * ufunc__spherical_jn_data[0] = &ufunc__spherical_jn_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_ptr[2]) = ((void *)__pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_complex);

  /* "scipy/special/_ufuncs.pyx":2396
 * ufunc__spherical_jn_ptr[2*0+1] = <void*>(<char*>"_spherical_jn")
 * ufunc__spherical_jn_ptr[2*1] = <void*>_func_spherical_jn_complex
 * ufunc__spherical_jn_ptr[2*1+1] = <void*>(<char*>"_spherical_jn")             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_data[0] = &ufunc__spherical_jn_ptr[2*0]
 * ufunc__spherical_jn_data[1] = &ufunc__spherical_jn_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_ptr[3]) = ((void *)((char *)((char *)"_spherical_jn")));

  /* "scipy/special/_ufuncs.pyx":2397
 * ufunc__spherical_jn_ptr[2*1] = <void*>_func_spherical_jn_complex
 * ufunc__spherical_jn_ptr[2*1+1] = <void*>(<char*>"_spherical_jn")
 * ufunc__spherical_jn_data[0] = &ufunc__spherical_jn_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_data[1] = &ufunc__spherical_jn_ptr[2*1]
 * _spherical_jn = np.PyUFunc_FromFuncAndData(ufunc__spherical_jn_loops, ufunc__spherical_jn_data, ufunc__spherical_jn_types, 2, 2, 1, 0, "_spherical_jn", ufunc__spherical_jn_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2398
 * ufunc__spherical_jn_ptr[2*1+1] = <void*>(<char*>"_spherical_jn")
 * ufunc__spherical_jn_data[0] = &ufunc__spherical_jn_ptr[2*0]
 * ufunc__spherical_jn_data[1] = &ufunc__spherical_jn_ptr[2*1]             # <<<<<<<<<<<<<<
 * _spherical_jn = np.PyUFunc_FromFuncAndData(ufunc__spherical_jn_loops, ufunc__spherical_jn_data, ufunc__spherical_jn_types, 2, 2, 1, 0, "_spherical_jn", ufunc__spherical_jn_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2399
 * ufunc__spherical_jn_data[0] = &ufunc__spherical_jn_ptr[2*0]
 * ufunc__spherical_jn_data[1] = &ufunc__spherical_jn_ptr[2*1]
 * _spherical_jn = np.PyUFunc_FromFuncAndData(ufunc__spherical_jn_loops, ufunc__spherical_jn_data, ufunc__spherical_jn_types, 2, 2, 1, 0, "_spherical_jn", ufunc__spherical_jn_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__spherical_jn_d_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_types, 2, 2, 1, 0, ((char *)"_spherical_jn"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2399, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_spherical_jn, __pyx_t_2) < 0) __PYX_ERR(1, 2399, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2405
 * cdef void *ufunc__spherical_jn_d_data[2]
 * cdef char ufunc__spherical_jn_d_types[6]
 * cdef char *ufunc__spherical_jn_d_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, use `spherical_jn` instead.")
 * ufunc__spherical_jn_d_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_doc = ((char *)"Internal function, use `spherical_jn` instead.");

  /* "scipy/special/_ufuncs.pyx":2407
 * cdef char *ufunc__spherical_jn_d_doc = (
 *     "Internal function, use `spherical_jn` instead.")
 * ufunc__spherical_jn_d_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_d_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_jn_d_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":2408
 *     "Internal function, use `spherical_jn` instead.")
 * ufunc__spherical_jn_d_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc__spherical_jn_d_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_d_types[0] = <char>NPY_LONG
 * ufunc__spherical_jn_d_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_lD__As_lD_D);

  /* "scipy/special/_ufuncs.pyx":2409
 * ufunc__spherical_jn_d_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc__spherical_jn_d_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_jn_d_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_d_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_jn_d_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2410
 * ufunc__spherical_jn_d_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_jn_d_types[0] = <char>NPY_LONG
 * ufunc__spherical_jn_d_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_d_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_jn_d_types[3] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2411
 * ufunc__spherical_jn_d_types[0] = <char>NPY_LONG
 * ufunc__spherical_jn_d_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_jn_d_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_d_types[3] = <char>NPY_LONG
 * ufunc__spherical_jn_d_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2412
 * ufunc__spherical_jn_d_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_jn_d_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_jn_d_types[3] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_d_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_jn_d_types[5] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_types[3]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2413
 * ufunc__spherical_jn_d_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_jn_d_types[3] = <char>NPY_LONG
 * ufunc__spherical_jn_d_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_d_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_jn_d_ptr[2*0] = <void*>_func_spherical_jn_d_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2414
 * ufunc__spherical_jn_d_types[3] = <char>NPY_LONG
 * ufunc__spherical_jn_d_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_jn_d_types[5] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_d_ptr[2*0] = <void*>_func_spherical_jn_d_real
 * ufunc__spherical_jn_d_ptr[2*0+1] = <void*>(<char*>"_spherical_jn_d")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_types[5]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2415
 * ufunc__spherical_jn_d_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_jn_d_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_jn_d_ptr[2*0] = <void*>_func_spherical_jn_d_real             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_d_ptr[2*0+1] = <void*>(<char*>"_spherical_jn_d")
 * ufunc__spherical_jn_d_ptr[2*1] = <void*>_func_spherical_jn_d_complex
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_ptr[0]) = ((void *)__pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_d_real);

  /* "scipy/special/_ufuncs.pyx":2416
 * ufunc__spherical_jn_d_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_jn_d_ptr[2*0] = <void*>_func_spherical_jn_d_real
 * ufunc__spherical_jn_d_ptr[2*0+1] = <void*>(<char*>"_spherical_jn_d")             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_d_ptr[2*1] = <void*>_func_spherical_jn_d_complex
 * ufunc__spherical_jn_d_ptr[2*1+1] = <void*>(<char*>"_spherical_jn_d")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_ptr[1]) = ((void *)((char *)((char *)"_spherical_jn_d")));

  /* "scipy/special/_ufuncs.pyx":2417
 * ufunc__spherical_jn_d_ptr[2*0] = <void*>_func_spherical_jn_d_real
 * ufunc__spherical_jn_d_ptr[2*0+1] = <void*>(<char*>"_spherical_jn_d")
 * ufunc__spherical_jn_d_ptr[2*1] = <void*>_func_spherical_jn_d_complex             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_d_ptr[2*1+1] = <void*>(<char*>"_spherical_jn_d")
 * ufunc__spherical_jn_d_data[0] = &ufunc__spherical_jn_d_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_ptr[2]) = ((void *)__pyx_f_5scipy_7special_17_spherical_bessel_spherical_jn_d_complex);

  /* "scipy/special/_ufuncs.pyx":2418
 * ufunc__spherical_jn_d_ptr[2*0+1] = <void*>(<char*>"_spherical_jn_d")
 * ufunc__spherical_jn_d_ptr[2*1] = <void*>_func_spherical_jn_d_complex
 * ufunc__spherical_jn_d_ptr[2*1+1] = <void*>(<char*>"_spherical_jn_d")             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_d_data[0] = &ufunc__spherical_jn_d_ptr[2*0]
 * ufunc__spherical_jn_d_data[1] = &ufunc__spherical_jn_d_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_ptr[3]) = ((void *)((char *)((char *)"_spherical_jn_d")));

  /* "scipy/special/_ufuncs.pyx":2419
 * ufunc__spherical_jn_d_ptr[2*1] = <void*>_func_spherical_jn_d_complex
 * ufunc__spherical_jn_d_ptr[2*1+1] = <void*>(<char*>"_spherical_jn_d")
 * ufunc__spherical_jn_d_data[0] = &ufunc__spherical_jn_d_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__spherical_jn_d_data[1] = &ufunc__spherical_jn_d_ptr[2*1]
 * _spherical_jn_d = np.PyUFunc_FromFuncAndData(ufunc__spherical_jn_d_loops, ufunc__spherical_jn_d_data, ufunc__spherical_jn_d_types, 2, 2, 1, 0, "_spherical_jn_d", ufunc__spherical_jn_d_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2420
 * ufunc__spherical_jn_d_ptr[2*1+1] = <void*>(<char*>"_spherical_jn_d")
 * ufunc__spherical_jn_d_data[0] = &ufunc__spherical_jn_d_ptr[2*0]
 * ufunc__spherical_jn_d_data[1] = &ufunc__spherical_jn_d_ptr[2*1]             # <<<<<<<<<<<<<<
 * _spherical_jn_d = np.PyUFunc_FromFuncAndData(ufunc__spherical_jn_d_loops, ufunc__spherical_jn_d_data, ufunc__spherical_jn_d_types, 2, 2, 1, 0, "_spherical_jn_d", ufunc__spherical_jn_d_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2421
 * ufunc__spherical_jn_d_data[0] = &ufunc__spherical_jn_d_ptr[2*0]
 * ufunc__spherical_jn_d_data[1] = &ufunc__spherical_jn_d_ptr[2*1]
 * _spherical_jn_d = np.PyUFunc_FromFuncAndData(ufunc__spherical_jn_d_loops, ufunc__spherical_jn_d_data, ufunc__spherical_jn_d_types, 2, 2, 1, 0, "_spherical_jn_d", ufunc__spherical_jn_d_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__spherical_kn_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_types, 2, 2, 1, 0, ((char *)"_spherical_jn_d"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_jn_d_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2421, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_spherical_jn_d, __pyx_t_2) < 0) __PYX_ERR(1, 2421, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2427
 * cdef void *ufunc__spherical_kn_data[2]
 * cdef char ufunc__spherical_kn_types[6]
 * cdef char *ufunc__spherical_kn_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, use `spherical_kn` instead.")
 * ufunc__spherical_kn_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_doc = ((char *)"Internal function, use `spherical_kn` instead.");

  /* "scipy/special/_ufuncs.pyx":2429
 * cdef char *ufunc__spherical_kn_doc = (
 *     "Internal function, use `spherical_kn` instead.")
 * ufunc__spherical_kn_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_kn_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":2430
 *     "Internal function, use `spherical_kn` instead.")
 * ufunc__spherical_kn_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc__spherical_kn_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_types[0] = <char>NPY_LONG
 * ufunc__spherical_kn_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_lD__As_lD_D);

  /* "scipy/special/_ufuncs.pyx":2431
 * ufunc__spherical_kn_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc__spherical_kn_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_kn_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_kn_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2432
 * ufunc__spherical_kn_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_kn_types[0] = <char>NPY_LONG
 * ufunc__spherical_kn_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_kn_types[3] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2433
 * ufunc__spherical_kn_types[0] = <char>NPY_LONG
 * ufunc__spherical_kn_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_kn_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_types[3] = <char>NPY_LONG
 * ufunc__spherical_kn_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2434
 * ufunc__spherical_kn_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_kn_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_kn_types[3] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_kn_types[5] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_types[3]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2435
 * ufunc__spherical_kn_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_kn_types[3] = <char>NPY_LONG
 * ufunc__spherical_kn_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_kn_ptr[2*0] = <void*>_func_spherical_kn_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2436
 * ufunc__spherical_kn_types[3] = <char>NPY_LONG
 * ufunc__spherical_kn_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_kn_types[5] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_ptr[2*0] = <void*>_func_spherical_kn_real
 * ufunc__spherical_kn_ptr[2*0+1] = <void*>(<char*>"_spherical_kn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_types[5]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2437
 * ufunc__spherical_kn_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_kn_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_kn_ptr[2*0] = <void*>_func_spherical_kn_real             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_ptr[2*0+1] = <void*>(<char*>"_spherical_kn")
 * ufunc__spherical_kn_ptr[2*1] = <void*>_func_spherical_kn_complex
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_ptr[0]) = ((void *)__pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_real);

  /* "scipy/special/_ufuncs.pyx":2438
 * ufunc__spherical_kn_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_kn_ptr[2*0] = <void*>_func_spherical_kn_real
 * ufunc__spherical_kn_ptr[2*0+1] = <void*>(<char*>"_spherical_kn")             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_ptr[2*1] = <void*>_func_spherical_kn_complex
 * ufunc__spherical_kn_ptr[2*1+1] = <void*>(<char*>"_spherical_kn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_ptr[1]) = ((void *)((char *)((char *)"_spherical_kn")));

  /* "scipy/special/_ufuncs.pyx":2439
 * ufunc__spherical_kn_ptr[2*0] = <void*>_func_spherical_kn_real
 * ufunc__spherical_kn_ptr[2*0+1] = <void*>(<char*>"_spherical_kn")
 * ufunc__spherical_kn_ptr[2*1] = <void*>_func_spherical_kn_complex             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_ptr[2*1+1] = <void*>(<char*>"_spherical_kn")
 * ufunc__spherical_kn_data[0] = &ufunc__spherical_kn_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_ptr[2]) = ((void *)__pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_complex);

  /* "scipy/special/_ufuncs.pyx":2440
 * ufunc__spherical_kn_ptr[2*0+1] = <void*>(<char*>"_spherical_kn")
 * ufunc__spherical_kn_ptr[2*1] = <void*>_func_spherical_kn_complex
 * ufunc__spherical_kn_ptr[2*1+1] = <void*>(<char*>"_spherical_kn")             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_data[0] = &ufunc__spherical_kn_ptr[2*0]
 * ufunc__spherical_kn_data[1] = &ufunc__spherical_kn_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_ptr[3]) = ((void *)((char *)((char *)"_spherical_kn")));

  /* "scipy/special/_ufuncs.pyx":2441
 * ufunc__spherical_kn_ptr[2*1] = <void*>_func_spherical_kn_complex
 * ufunc__spherical_kn_ptr[2*1+1] = <void*>(<char*>"_spherical_kn")
 * ufunc__spherical_kn_data[0] = &ufunc__spherical_kn_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_data[1] = &ufunc__spherical_kn_ptr[2*1]
 * _spherical_kn = np.PyUFunc_FromFuncAndData(ufunc__spherical_kn_loops, ufunc__spherical_kn_data, ufunc__spherical_kn_types, 2, 2, 1, 0, "_spherical_kn", ufunc__spherical_kn_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2442
 * ufunc__spherical_kn_ptr[2*1+1] = <void*>(<char*>"_spherical_kn")
 * ufunc__spherical_kn_data[0] = &ufunc__spherical_kn_ptr[2*0]
 * ufunc__spherical_kn_data[1] = &ufunc__spherical_kn_ptr[2*1]             # <<<<<<<<<<<<<<
 * _spherical_kn = np.PyUFunc_FromFuncAndData(ufunc__spherical_kn_loops, ufunc__spherical_kn_data, ufunc__spherical_kn_types, 2, 2, 1, 0, "_spherical_kn", ufunc__spherical_kn_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2443
 * ufunc__spherical_kn_data[0] = &ufunc__spherical_kn_ptr[2*0]
 * ufunc__spherical_kn_data[1] = &ufunc__spherical_kn_ptr[2*1]
 * _spherical_kn = np.PyUFunc_FromFuncAndData(ufunc__spherical_kn_loops, ufunc__spherical_kn_data, ufunc__spherical_kn_types, 2, 2, 1, 0, "_spherical_kn", ufunc__spherical_kn_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__spherical_kn_d_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_types, 2, 2, 1, 0, ((char *)"_spherical_kn"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_spherical_kn, __pyx_t_2) < 0) __PYX_ERR(1, 2443, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2449
 * cdef void *ufunc__spherical_kn_d_data[2]
 * cdef char ufunc__spherical_kn_d_types[6]
 * cdef char *ufunc__spherical_kn_d_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, use `spherical_kn` instead.")
 * ufunc__spherical_kn_d_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_doc = ((char *)"Internal function, use `spherical_kn` instead.");

  /* "scipy/special/_ufuncs.pyx":2451
 * cdef char *ufunc__spherical_kn_d_doc = (
 *     "Internal function, use `spherical_kn` instead.")
 * ufunc__spherical_kn_d_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_d_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_kn_d_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":2452
 *     "Internal function, use `spherical_kn` instead.")
 * ufunc__spherical_kn_d_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc__spherical_kn_d_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_d_types[0] = <char>NPY_LONG
 * ufunc__spherical_kn_d_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_lD__As_lD_D);

  /* "scipy/special/_ufuncs.pyx":2453
 * ufunc__spherical_kn_d_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc__spherical_kn_d_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_kn_d_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_d_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_kn_d_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2454
 * ufunc__spherical_kn_d_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_kn_d_types[0] = <char>NPY_LONG
 * ufunc__spherical_kn_d_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_d_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_kn_d_types[3] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2455
 * ufunc__spherical_kn_d_types[0] = <char>NPY_LONG
 * ufunc__spherical_kn_d_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_kn_d_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_d_types[3] = <char>NPY_LONG
 * ufunc__spherical_kn_d_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2456
 * ufunc__spherical_kn_d_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_kn_d_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_kn_d_types[3] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_d_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_kn_d_types[5] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_types[3]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2457
 * ufunc__spherical_kn_d_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_kn_d_types[3] = <char>NPY_LONG
 * ufunc__spherical_kn_d_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_d_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_kn_d_ptr[2*0] = <void*>_func_spherical_kn_d_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2458
 * ufunc__spherical_kn_d_types[3] = <char>NPY_LONG
 * ufunc__spherical_kn_d_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_kn_d_types[5] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_d_ptr[2*0] = <void*>_func_spherical_kn_d_real
 * ufunc__spherical_kn_d_ptr[2*0+1] = <void*>(<char*>"_spherical_kn_d")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_types[5]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2459
 * ufunc__spherical_kn_d_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_kn_d_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_kn_d_ptr[2*0] = <void*>_func_spherical_kn_d_real             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_d_ptr[2*0+1] = <void*>(<char*>"_spherical_kn_d")
 * ufunc__spherical_kn_d_ptr[2*1] = <void*>_func_spherical_kn_d_complex
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_ptr[0]) = ((void *)__pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_d_real);

  /* "scipy/special/_ufuncs.pyx":2460
 * ufunc__spherical_kn_d_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_kn_d_ptr[2*0] = <void*>_func_spherical_kn_d_real
 * ufunc__spherical_kn_d_ptr[2*0+1] = <void*>(<char*>"_spherical_kn_d")             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_d_ptr[2*1] = <void*>_func_spherical_kn_d_complex
 * ufunc__spherical_kn_d_ptr[2*1+1] = <void*>(<char*>"_spherical_kn_d")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_ptr[1]) = ((void *)((char *)((char *)"_spherical_kn_d")));

  /* "scipy/special/_ufuncs.pyx":2461
 * ufunc__spherical_kn_d_ptr[2*0] = <void*>_func_spherical_kn_d_real
 * ufunc__spherical_kn_d_ptr[2*0+1] = <void*>(<char*>"_spherical_kn_d")
 * ufunc__spherical_kn_d_ptr[2*1] = <void*>_func_spherical_kn_d_complex             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_d_ptr[2*1+1] = <void*>(<char*>"_spherical_kn_d")
 * ufunc__spherical_kn_d_data[0] = &ufunc__spherical_kn_d_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_ptr[2]) = ((void *)__pyx_f_5scipy_7special_17_spherical_bessel_spherical_kn_d_complex);

  /* "scipy/special/_ufuncs.pyx":2462
 * ufunc__spherical_kn_d_ptr[2*0+1] = <void*>(<char*>"_spherical_kn_d")
 * ufunc__spherical_kn_d_ptr[2*1] = <void*>_func_spherical_kn_d_complex
 * ufunc__spherical_kn_d_ptr[2*1+1] = <void*>(<char*>"_spherical_kn_d")             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_d_data[0] = &ufunc__spherical_kn_d_ptr[2*0]
 * ufunc__spherical_kn_d_data[1] = &ufunc__spherical_kn_d_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_ptr[3]) = ((void *)((char *)((char *)"_spherical_kn_d")));

  /* "scipy/special/_ufuncs.pyx":2463
 * ufunc__spherical_kn_d_ptr[2*1] = <void*>_func_spherical_kn_d_complex
 * ufunc__spherical_kn_d_ptr[2*1+1] = <void*>(<char*>"_spherical_kn_d")
 * ufunc__spherical_kn_d_data[0] = &ufunc__spherical_kn_d_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__spherical_kn_d_data[1] = &ufunc__spherical_kn_d_ptr[2*1]
 * _spherical_kn_d = np.PyUFunc_FromFuncAndData(ufunc__spherical_kn_d_loops, ufunc__spherical_kn_d_data, ufunc__spherical_kn_d_types, 2, 2, 1, 0, "_spherical_kn_d", ufunc__spherical_kn_d_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2464
 * ufunc__spherical_kn_d_ptr[2*1+1] = <void*>(<char*>"_spherical_kn_d")
 * ufunc__spherical_kn_d_data[0] = &ufunc__spherical_kn_d_ptr[2*0]
 * ufunc__spherical_kn_d_data[1] = &ufunc__spherical_kn_d_ptr[2*1]             # <<<<<<<<<<<<<<
 * _spherical_kn_d = np.PyUFunc_FromFuncAndData(ufunc__spherical_kn_d_loops, ufunc__spherical_kn_d_data, ufunc__spherical_kn_d_types, 2, 2, 1, 0, "_spherical_kn_d", ufunc__spherical_kn_d_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2465
 * ufunc__spherical_kn_d_data[0] = &ufunc__spherical_kn_d_ptr[2*0]
 * ufunc__spherical_kn_d_data[1] = &ufunc__spherical_kn_d_ptr[2*1]
 * _spherical_kn_d = np.PyUFunc_FromFuncAndData(ufunc__spherical_kn_d_loops, ufunc__spherical_kn_d_data, ufunc__spherical_kn_d_types, 2, 2, 1, 0, "_spherical_kn_d", ufunc__spherical_kn_d_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__spherical_yn_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_types, 2, 2, 1, 0, ((char *)"_spherical_kn_d"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_kn_d_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2465, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_spherical_kn_d, __pyx_t_2) < 0) __PYX_ERR(1, 2465, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2471
 * cdef void *ufunc__spherical_yn_data[2]
 * cdef char ufunc__spherical_yn_types[6]
 * cdef char *ufunc__spherical_yn_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, use `spherical_yn` instead.")
 * ufunc__spherical_yn_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_doc = ((char *)"Internal function, use `spherical_yn` instead.");

  /* "scipy/special/_ufuncs.pyx":2473
 * cdef char *ufunc__spherical_yn_doc = (
 *     "Internal function, use `spherical_yn` instead.")
 * ufunc__spherical_yn_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_yn_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":2474
 *     "Internal function, use `spherical_yn` instead.")
 * ufunc__spherical_yn_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc__spherical_yn_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_types[0] = <char>NPY_LONG
 * ufunc__spherical_yn_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_lD__As_lD_D);

  /* "scipy/special/_ufuncs.pyx":2475
 * ufunc__spherical_yn_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc__spherical_yn_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_yn_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_yn_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2476
 * ufunc__spherical_yn_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_yn_types[0] = <char>NPY_LONG
 * ufunc__spherical_yn_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_yn_types[3] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2477
 * ufunc__spherical_yn_types[0] = <char>NPY_LONG
 * ufunc__spherical_yn_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_yn_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_types[3] = <char>NPY_LONG
 * ufunc__spherical_yn_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2478
 * ufunc__spherical_yn_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_yn_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_yn_types[3] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_yn_types[5] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_types[3]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2479
 * ufunc__spherical_yn_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_yn_types[3] = <char>NPY_LONG
 * ufunc__spherical_yn_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_yn_ptr[2*0] = <void*>_func_spherical_yn_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2480
 * ufunc__spherical_yn_types[3] = <char>NPY_LONG
 * ufunc__spherical_yn_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_yn_types[5] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_ptr[2*0] = <void*>_func_spherical_yn_real
 * ufunc__spherical_yn_ptr[2*0+1] = <void*>(<char*>"_spherical_yn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_types[5]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2481
 * ufunc__spherical_yn_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_yn_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_yn_ptr[2*0] = <void*>_func_spherical_yn_real             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_ptr[2*0+1] = <void*>(<char*>"_spherical_yn")
 * ufunc__spherical_yn_ptr[2*1] = <void*>_func_spherical_yn_complex
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_ptr[0]) = ((void *)__pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_real);

  /* "scipy/special/_ufuncs.pyx":2482
 * ufunc__spherical_yn_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_yn_ptr[2*0] = <void*>_func_spherical_yn_real
 * ufunc__spherical_yn_ptr[2*0+1] = <void*>(<char*>"_spherical_yn")             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_ptr[2*1] = <void*>_func_spherical_yn_complex
 * ufunc__spherical_yn_ptr[2*1+1] = <void*>(<char*>"_spherical_yn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_ptr[1]) = ((void *)((char *)((char *)"_spherical_yn")));

  /* "scipy/special/_ufuncs.pyx":2483
 * ufunc__spherical_yn_ptr[2*0] = <void*>_func_spherical_yn_real
 * ufunc__spherical_yn_ptr[2*0+1] = <void*>(<char*>"_spherical_yn")
 * ufunc__spherical_yn_ptr[2*1] = <void*>_func_spherical_yn_complex             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_ptr[2*1+1] = <void*>(<char*>"_spherical_yn")
 * ufunc__spherical_yn_data[0] = &ufunc__spherical_yn_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_ptr[2]) = ((void *)__pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_complex);

  /* "scipy/special/_ufuncs.pyx":2484
 * ufunc__spherical_yn_ptr[2*0+1] = <void*>(<char*>"_spherical_yn")
 * ufunc__spherical_yn_ptr[2*1] = <void*>_func_spherical_yn_complex
 * ufunc__spherical_yn_ptr[2*1+1] = <void*>(<char*>"_spherical_yn")             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_data[0] = &ufunc__spherical_yn_ptr[2*0]
 * ufunc__spherical_yn_data[1] = &ufunc__spherical_yn_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_ptr[3]) = ((void *)((char *)((char *)"_spherical_yn")));

  /* "scipy/special/_ufuncs.pyx":2485
 * ufunc__spherical_yn_ptr[2*1] = <void*>_func_spherical_yn_complex
 * ufunc__spherical_yn_ptr[2*1+1] = <void*>(<char*>"_spherical_yn")
 * ufunc__spherical_yn_data[0] = &ufunc__spherical_yn_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_data[1] = &ufunc__spherical_yn_ptr[2*1]
 * _spherical_yn = np.PyUFunc_FromFuncAndData(ufunc__spherical_yn_loops, ufunc__spherical_yn_data, ufunc__spherical_yn_types, 2, 2, 1, 0, "_spherical_yn", ufunc__spherical_yn_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2486
 * ufunc__spherical_yn_ptr[2*1+1] = <void*>(<char*>"_spherical_yn")
 * ufunc__spherical_yn_data[0] = &ufunc__spherical_yn_ptr[2*0]
 * ufunc__spherical_yn_data[1] = &ufunc__spherical_yn_ptr[2*1]             # <<<<<<<<<<<<<<
 * _spherical_yn = np.PyUFunc_FromFuncAndData(ufunc__spherical_yn_loops, ufunc__spherical_yn_data, ufunc__spherical_yn_types, 2, 2, 1, 0, "_spherical_yn", ufunc__spherical_yn_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2487
 * ufunc__spherical_yn_data[0] = &ufunc__spherical_yn_ptr[2*0]
 * ufunc__spherical_yn_data[1] = &ufunc__spherical_yn_ptr[2*1]
 * _spherical_yn = np.PyUFunc_FromFuncAndData(ufunc__spherical_yn_loops, ufunc__spherical_yn_data, ufunc__spherical_yn_types, 2, 2, 1, 0, "_spherical_yn", ufunc__spherical_yn_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__spherical_yn_d_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_types, 2, 2, 1, 0, ((char *)"_spherical_yn"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_spherical_yn, __pyx_t_2) < 0) __PYX_ERR(1, 2487, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2493
 * cdef void *ufunc__spherical_yn_d_data[2]
 * cdef char ufunc__spherical_yn_d_types[6]
 * cdef char *ufunc__spherical_yn_d_doc = (             # <<<<<<<<<<<<<<
 *     "Internal function, use `spherical_yn` instead.")
 * ufunc__spherical_yn_d_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_doc = ((char *)"Internal function, use `spherical_yn` instead.");

  /* "scipy/special/_ufuncs.pyx":2495
 * cdef char *ufunc__spherical_yn_d_doc = (
 *     "Internal function, use `spherical_yn` instead.")
 * ufunc__spherical_yn_d_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_d_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_yn_d_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":2496
 *     "Internal function, use `spherical_yn` instead.")
 * ufunc__spherical_yn_d_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc__spherical_yn_d_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_d_types[0] = <char>NPY_LONG
 * ufunc__spherical_yn_d_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_lD__As_lD_D);

  /* "scipy/special/_ufuncs.pyx":2497
 * ufunc__spherical_yn_d_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc__spherical_yn_d_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_yn_d_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_d_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_yn_d_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2498
 * ufunc__spherical_yn_d_loops[1] = <np.PyUFuncGenericFunction>loop_D_lD__As_lD_D
 * ufunc__spherical_yn_d_types[0] = <char>NPY_LONG
 * ufunc__spherical_yn_d_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_d_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_yn_d_types[3] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2499
 * ufunc__spherical_yn_d_types[0] = <char>NPY_LONG
 * ufunc__spherical_yn_d_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_yn_d_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_d_types[3] = <char>NPY_LONG
 * ufunc__spherical_yn_d_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2500
 * ufunc__spherical_yn_d_types[1] = <char>NPY_DOUBLE
 * ufunc__spherical_yn_d_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_yn_d_types[3] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_d_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_yn_d_types[5] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_types[3]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2501
 * ufunc__spherical_yn_d_types[2] = <char>NPY_DOUBLE
 * ufunc__spherical_yn_d_types[3] = <char>NPY_LONG
 * ufunc__spherical_yn_d_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_d_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_yn_d_ptr[2*0] = <void*>_func_spherical_yn_d_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2502
 * ufunc__spherical_yn_d_types[3] = <char>NPY_LONG
 * ufunc__spherical_yn_d_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_yn_d_types[5] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_d_ptr[2*0] = <void*>_func_spherical_yn_d_real
 * ufunc__spherical_yn_d_ptr[2*0+1] = <void*>(<char*>"_spherical_yn_d")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_types[5]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2503
 * ufunc__spherical_yn_d_types[4] = <char>NPY_CDOUBLE
 * ufunc__spherical_yn_d_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_yn_d_ptr[2*0] = <void*>_func_spherical_yn_d_real             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_d_ptr[2*0+1] = <void*>(<char*>"_spherical_yn_d")
 * ufunc__spherical_yn_d_ptr[2*1] = <void*>_func_spherical_yn_d_complex
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_ptr[0]) = ((void *)__pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_d_real);

  /* "scipy/special/_ufuncs.pyx":2504
 * ufunc__spherical_yn_d_types[5] = <char>NPY_CDOUBLE
 * ufunc__spherical_yn_d_ptr[2*0] = <void*>_func_spherical_yn_d_real
 * ufunc__spherical_yn_d_ptr[2*0+1] = <void*>(<char*>"_spherical_yn_d")             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_d_ptr[2*1] = <void*>_func_spherical_yn_d_complex
 * ufunc__spherical_yn_d_ptr[2*1+1] = <void*>(<char*>"_spherical_yn_d")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_ptr[1]) = ((void *)((char *)((char *)"_spherical_yn_d")));

  /* "scipy/special/_ufuncs.pyx":2505
 * ufunc__spherical_yn_d_ptr[2*0] = <void*>_func_spherical_yn_d_real
 * ufunc__spherical_yn_d_ptr[2*0+1] = <void*>(<char*>"_spherical_yn_d")
 * ufunc__spherical_yn_d_ptr[2*1] = <void*>_func_spherical_yn_d_complex             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_d_ptr[2*1+1] = <void*>(<char*>"_spherical_yn_d")
 * ufunc__spherical_yn_d_data[0] = &ufunc__spherical_yn_d_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_ptr[2]) = ((void *)__pyx_f_5scipy_7special_17_spherical_bessel_spherical_yn_d_complex);

  /* "scipy/special/_ufuncs.pyx":2506
 * ufunc__spherical_yn_d_ptr[2*0+1] = <void*>(<char*>"_spherical_yn_d")
 * ufunc__spherical_yn_d_ptr[2*1] = <void*>_func_spherical_yn_d_complex
 * ufunc__spherical_yn_d_ptr[2*1+1] = <void*>(<char*>"_spherical_yn_d")             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_d_data[0] = &ufunc__spherical_yn_d_ptr[2*0]
 * ufunc__spherical_yn_d_data[1] = &ufunc__spherical_yn_d_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_ptr[3]) = ((void *)((char *)((char *)"_spherical_yn_d")));

  /* "scipy/special/_ufuncs.pyx":2507
 * ufunc__spherical_yn_d_ptr[2*1] = <void*>_func_spherical_yn_d_complex
 * ufunc__spherical_yn_d_ptr[2*1+1] = <void*>(<char*>"_spherical_yn_d")
 * ufunc__spherical_yn_d_data[0] = &ufunc__spherical_yn_d_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__spherical_yn_d_data[1] = &ufunc__spherical_yn_d_ptr[2*1]
 * _spherical_yn_d = np.PyUFunc_FromFuncAndData(ufunc__spherical_yn_d_loops, ufunc__spherical_yn_d_data, ufunc__spherical_yn_d_types, 2, 2, 1, 0, "_spherical_yn_d", ufunc__spherical_yn_d_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2508
 * ufunc__spherical_yn_d_ptr[2*1+1] = <void*>(<char*>"_spherical_yn_d")
 * ufunc__spherical_yn_d_data[0] = &ufunc__spherical_yn_d_ptr[2*0]
 * ufunc__spherical_yn_d_data[1] = &ufunc__spherical_yn_d_ptr[2*1]             # <<<<<<<<<<<<<<
 * _spherical_yn_d = np.PyUFunc_FromFuncAndData(ufunc__spherical_yn_d_loops, ufunc__spherical_yn_d_data, ufunc__spherical_yn_d_types, 2, 2, 1, 0, "_spherical_yn_d", ufunc__spherical_yn_d_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2509
 * ufunc__spherical_yn_d_data[0] = &ufunc__spherical_yn_d_ptr[2*0]
 * ufunc__spherical_yn_d_data[1] = &ufunc__spherical_yn_d_ptr[2*1]
 * _spherical_yn_d = np.PyUFunc_FromFuncAndData(ufunc__spherical_yn_d_loops, ufunc__spherical_yn_d_data, ufunc__spherical_yn_d_types, 2, 2, 1, 0, "_spherical_yn_d", ufunc__spherical_yn_d_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__struve_asymp_large_z_loops[1]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_types, 2, 2, 1, 0, ((char *)"_spherical_yn_d"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__spherical_yn_d_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2509, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_spherical_yn_d, __pyx_t_2) < 0) __PYX_ERR(1, 2509, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2515
 * cdef void *ufunc__struve_asymp_large_z_data[1]
 * cdef char ufunc__struve_asymp_large_z_types[5]
 * cdef char *ufunc__struve_asymp_large_z_doc = (             # <<<<<<<<<<<<<<
 *     "_struve_asymp_large_z(v, z, is_h)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_doc = ((char *)"_struve_asymp_large_z(v, z, is_h)\n\nInternal function for testing `struve` & `modstruve`\n\nEvaluates using asymptotic expansion\n\nReturns\n-------\nv, err");

  /* "scipy/special/_ufuncs.pyx":2525
 *     "-------\n"
 *     "v, err")
 * ufunc__struve_asymp_large_z_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddi_d_As_ddl_dd             # <<<<<<<<<<<<<<
 * ufunc__struve_asymp_large_z_types[0] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddi_d_As_ddl_dd);

  /* "scipy/special/_ufuncs.pyx":2526
 *     "v, err")
 * ufunc__struve_asymp_large_z_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddi_d_As_ddl_dd
 * ufunc__struve_asymp_large_z_types[0] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_asymp_large_z_types[1] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_types[2] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_types[0]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2527
 * ufunc__struve_asymp_large_z_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddi_d_As_ddl_dd
 * ufunc__struve_asymp_large_z_types[0] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_asymp_large_z_types[2] = <char>NPY_LONG
 * ufunc__struve_asymp_large_z_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2528
 * ufunc__struve_asymp_large_z_types[0] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_types[1] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_types[2] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__struve_asymp_large_z_types[3] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_types[2]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2529
 * ufunc__struve_asymp_large_z_types[1] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_types[2] = <char>NPY_LONG
 * ufunc__struve_asymp_large_z_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_asymp_large_z_types[4] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_ptr[2*0] = <void*>_func_struve_asymp_large_z
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2530
 * ufunc__struve_asymp_large_z_types[2] = <char>NPY_LONG
 * ufunc__struve_asymp_large_z_types[3] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_asymp_large_z_ptr[2*0] = <void*>_func_struve_asymp_large_z
 * ufunc__struve_asymp_large_z_ptr[2*0+1] = <void*>(<char*>"_struve_asymp_large_z")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2531
 * ufunc__struve_asymp_large_z_types[3] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_types[4] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_ptr[2*0] = <void*>_func_struve_asymp_large_z             # <<<<<<<<<<<<<<
 * ufunc__struve_asymp_large_z_ptr[2*0+1] = <void*>(<char*>"_struve_asymp_large_z")
 * ufunc__struve_asymp_large_z_data[0] = &ufunc__struve_asymp_large_z_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_ptr[0]) = ((void *)struve_asymp_large_z);

  /* "scipy/special/_ufuncs.pyx":2532
 * ufunc__struve_asymp_large_z_types[4] = <char>NPY_DOUBLE
 * ufunc__struve_asymp_large_z_ptr[2*0] = <void*>_func_struve_asymp_large_z
 * ufunc__struve_asymp_large_z_ptr[2*0+1] = <void*>(<char*>"_struve_asymp_large_z")             # <<<<<<<<<<<<<<
 * ufunc__struve_asymp_large_z_data[0] = &ufunc__struve_asymp_large_z_ptr[2*0]
 * _struve_asymp_large_z = np.PyUFunc_FromFuncAndData(ufunc__struve_asymp_large_z_loops, ufunc__struve_asymp_large_z_data, ufunc__struve_asymp_large_z_types, 1, 3, 2, 0, "_struve_asymp_large_z", ufunc__struve_asymp_large_z_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_ptr[1]) = ((void *)((char *)((char *)"_struve_asymp_large_z")));

  /* "scipy/special/_ufuncs.pyx":2533
 * ufunc__struve_asymp_large_z_ptr[2*0] = <void*>_func_struve_asymp_large_z
 * ufunc__struve_asymp_large_z_ptr[2*0+1] = <void*>(<char*>"_struve_asymp_large_z")
 * ufunc__struve_asymp_large_z_data[0] = &ufunc__struve_asymp_large_z_ptr[2*0]             # <<<<<<<<<<<<<<
 * _struve_asymp_large_z = np.PyUFunc_FromFuncAndData(ufunc__struve_asymp_large_z_loops, ufunc__struve_asymp_large_z_data, ufunc__struve_asymp_large_z_types, 1, 3, 2, 0, "_struve_asymp_large_z", ufunc__struve_asymp_large_z_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2534
 * ufunc__struve_asymp_large_z_ptr[2*0+1] = <void*>(<char*>"_struve_asymp_large_z")
 * ufunc__struve_asymp_large_z_data[0] = &ufunc__struve_asymp_large_z_ptr[2*0]
 * _struve_asymp_large_z = np.PyUFunc_FromFuncAndData(ufunc__struve_asymp_large_z_loops, ufunc__struve_asymp_large_z_data, ufunc__struve_asymp_large_z_types, 1, 3, 2, 0, "_struve_asymp_large_z", ufunc__struve_asymp_large_z_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__struve_bessel_series_loops[1]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_types, 1, 3, 2, 0, ((char *)"_struve_asymp_large_z"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_asymp_large_z_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2534, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_struve_asymp_large_z, __pyx_t_2) < 0) __PYX_ERR(1, 2534, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2540
 * cdef void *ufunc__struve_bessel_series_data[1]
 * cdef char ufunc__struve_bessel_series_types[5]
 * cdef char *ufunc__struve_bessel_series_doc = (             # <<<<<<<<<<<<<<
 *     "_struve_bessel_series(v, z, is_h)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_doc = ((char *)"_struve_bessel_series(v, z, is_h)\n\nInternal function for testing `struve` & `modstruve`\n\nEvaluates using Bessel function series\n\nReturns\n-------\nv, err");

  /* "scipy/special/_ufuncs.pyx":2550
 *     "-------\n"
 *     "v, err")
 * ufunc__struve_bessel_series_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddi_d_As_ddl_dd             # <<<<<<<<<<<<<<
 * ufunc__struve_bessel_series_types[0] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddi_d_As_ddl_dd);

  /* "scipy/special/_ufuncs.pyx":2551
 *     "v, err")
 * ufunc__struve_bessel_series_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddi_d_As_ddl_dd
 * ufunc__struve_bessel_series_types[0] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_bessel_series_types[1] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_types[2] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_types[0]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2552
 * ufunc__struve_bessel_series_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddi_d_As_ddl_dd
 * ufunc__struve_bessel_series_types[0] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_bessel_series_types[2] = <char>NPY_LONG
 * ufunc__struve_bessel_series_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2553
 * ufunc__struve_bessel_series_types[0] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_types[1] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_types[2] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__struve_bessel_series_types[3] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_types[2]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2554
 * ufunc__struve_bessel_series_types[1] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_types[2] = <char>NPY_LONG
 * ufunc__struve_bessel_series_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_bessel_series_types[4] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_ptr[2*0] = <void*>_func_struve_bessel_series
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2555
 * ufunc__struve_bessel_series_types[2] = <char>NPY_LONG
 * ufunc__struve_bessel_series_types[3] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_bessel_series_ptr[2*0] = <void*>_func_struve_bessel_series
 * ufunc__struve_bessel_series_ptr[2*0+1] = <void*>(<char*>"_struve_bessel_series")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2556
 * ufunc__struve_bessel_series_types[3] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_types[4] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_ptr[2*0] = <void*>_func_struve_bessel_series             # <<<<<<<<<<<<<<
 * ufunc__struve_bessel_series_ptr[2*0+1] = <void*>(<char*>"_struve_bessel_series")
 * ufunc__struve_bessel_series_data[0] = &ufunc__struve_bessel_series_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_ptr[0]) = ((void *)struve_bessel_series);

  /* "scipy/special/_ufuncs.pyx":2557
 * ufunc__struve_bessel_series_types[4] = <char>NPY_DOUBLE
 * ufunc__struve_bessel_series_ptr[2*0] = <void*>_func_struve_bessel_series
 * ufunc__struve_bessel_series_ptr[2*0+1] = <void*>(<char*>"_struve_bessel_series")             # <<<<<<<<<<<<<<
 * ufunc__struve_bessel_series_data[0] = &ufunc__struve_bessel_series_ptr[2*0]
 * _struve_bessel_series = np.PyUFunc_FromFuncAndData(ufunc__struve_bessel_series_loops, ufunc__struve_bessel_series_data, ufunc__struve_bessel_series_types, 1, 3, 2, 0, "_struve_bessel_series", ufunc__struve_bessel_series_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_ptr[1]) = ((void *)((char *)((char *)"_struve_bessel_series")));

  /* "scipy/special/_ufuncs.pyx":2558
 * ufunc__struve_bessel_series_ptr[2*0] = <void*>_func_struve_bessel_series
 * ufunc__struve_bessel_series_ptr[2*0+1] = <void*>(<char*>"_struve_bessel_series")
 * ufunc__struve_bessel_series_data[0] = &ufunc__struve_bessel_series_ptr[2*0]             # <<<<<<<<<<<<<<
 * _struve_bessel_series = np.PyUFunc_FromFuncAndData(ufunc__struve_bessel_series_loops, ufunc__struve_bessel_series_data, ufunc__struve_bessel_series_types, 1, 3, 2, 0, "_struve_bessel_series", ufunc__struve_bessel_series_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2559
 * ufunc__struve_bessel_series_ptr[2*0+1] = <void*>(<char*>"_struve_bessel_series")
 * ufunc__struve_bessel_series_data[0] = &ufunc__struve_bessel_series_ptr[2*0]
 * _struve_bessel_series = np.PyUFunc_FromFuncAndData(ufunc__struve_bessel_series_loops, ufunc__struve_bessel_series_data, ufunc__struve_bessel_series_types, 1, 3, 2, 0, "_struve_bessel_series", ufunc__struve_bessel_series_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__struve_power_series_loops[1]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_types, 1, 3, 2, 0, ((char *)"_struve_bessel_series"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_bessel_series_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2559, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_struve_bessel_series, __pyx_t_2) < 0) __PYX_ERR(1, 2559, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2565
 * cdef void *ufunc__struve_power_series_data[1]
 * cdef char ufunc__struve_power_series_types[5]
 * cdef char *ufunc__struve_power_series_doc = (             # <<<<<<<<<<<<<<
 *     "_struve_power_series(v, z, is_h)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_doc = ((char *)"_struve_power_series(v, z, is_h)\n\nInternal function for testing `struve` & `modstruve`\n\nEvaluates using power series\n\nReturns\n-------\nv, err");

  /* "scipy/special/_ufuncs.pyx":2575
 *     "-------\n"
 *     "v, err")
 * ufunc__struve_power_series_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddi_d_As_ddl_dd             # <<<<<<<<<<<<<<
 * ufunc__struve_power_series_types[0] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddi_d_As_ddl_dd);

  /* "scipy/special/_ufuncs.pyx":2576
 *     "v, err")
 * ufunc__struve_power_series_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddi_d_As_ddl_dd
 * ufunc__struve_power_series_types[0] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_power_series_types[1] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_types[2] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_types[0]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2577
 * ufunc__struve_power_series_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddi_d_As_ddl_dd
 * ufunc__struve_power_series_types[0] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_power_series_types[2] = <char>NPY_LONG
 * ufunc__struve_power_series_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2578
 * ufunc__struve_power_series_types[0] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_types[1] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_types[2] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc__struve_power_series_types[3] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_types[2]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2579
 * ufunc__struve_power_series_types[1] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_types[2] = <char>NPY_LONG
 * ufunc__struve_power_series_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_power_series_types[4] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_ptr[2*0] = <void*>_func_struve_power_series
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2580
 * ufunc__struve_power_series_types[2] = <char>NPY_LONG
 * ufunc__struve_power_series_types[3] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__struve_power_series_ptr[2*0] = <void*>_func_struve_power_series
 * ufunc__struve_power_series_ptr[2*0+1] = <void*>(<char*>"_struve_power_series")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2581
 * ufunc__struve_power_series_types[3] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_types[4] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_ptr[2*0] = <void*>_func_struve_power_series             # <<<<<<<<<<<<<<
 * ufunc__struve_power_series_ptr[2*0+1] = <void*>(<char*>"_struve_power_series")
 * ufunc__struve_power_series_data[0] = &ufunc__struve_power_series_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_ptr[0]) = ((void *)struve_power_series);

  /* "scipy/special/_ufuncs.pyx":2582
 * ufunc__struve_power_series_types[4] = <char>NPY_DOUBLE
 * ufunc__struve_power_series_ptr[2*0] = <void*>_func_struve_power_series
 * ufunc__struve_power_series_ptr[2*0+1] = <void*>(<char*>"_struve_power_series")             # <<<<<<<<<<<<<<
 * ufunc__struve_power_series_data[0] = &ufunc__struve_power_series_ptr[2*0]
 * _struve_power_series = np.PyUFunc_FromFuncAndData(ufunc__struve_power_series_loops, ufunc__struve_power_series_data, ufunc__struve_power_series_types, 1, 3, 2, 0, "_struve_power_series", ufunc__struve_power_series_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_ptr[1]) = ((void *)((char *)((char *)"_struve_power_series")));

  /* "scipy/special/_ufuncs.pyx":2583
 * ufunc__struve_power_series_ptr[2*0] = <void*>_func_struve_power_series
 * ufunc__struve_power_series_ptr[2*0+1] = <void*>(<char*>"_struve_power_series")
 * ufunc__struve_power_series_data[0] = &ufunc__struve_power_series_ptr[2*0]             # <<<<<<<<<<<<<<
 * _struve_power_series = np.PyUFunc_FromFuncAndData(ufunc__struve_power_series_loops, ufunc__struve_power_series_data, ufunc__struve_power_series_types, 1, 3, 2, 0, "_struve_power_series", ufunc__struve_power_series_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2584
 * ufunc__struve_power_series_ptr[2*0+1] = <void*>(<char*>"_struve_power_series")
 * ufunc__struve_power_series_data[0] = &ufunc__struve_power_series_ptr[2*0]
 * _struve_power_series = np.PyUFunc_FromFuncAndData(ufunc__struve_power_series_loops, ufunc__struve_power_series_data, ufunc__struve_power_series_types, 1, 3, 2, 0, "_struve_power_series", ufunc__struve_power_series_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc__zeta_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_types, 1, 3, 2, 0, ((char *)"_struve_power_series"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__struve_power_series_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_struve_power_series, __pyx_t_2) < 0) __PYX_ERR(1, 2584, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2590
 * cdef void *ufunc__zeta_data[2]
 * cdef char ufunc__zeta_types[6]
 * cdef char *ufunc__zeta_doc = (             # <<<<<<<<<<<<<<
 *     "_zeta(x, q)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_doc = ((char *)"_zeta(x, q)\n\nInternal function, Hurwitz zeta.");

  /* "scipy/special/_ufuncs.pyx":2594
 *     "\n"
 *     "Internal function, Hurwitz zeta.")
 * ufunc__zeta_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc__zeta_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc__zeta_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":2595
 *     "Internal function, Hurwitz zeta.")
 * ufunc__zeta_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc__zeta_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc__zeta_types[0] = <char>NPY_FLOAT
 * ufunc__zeta_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":2596
 * ufunc__zeta_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc__zeta_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc__zeta_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__zeta_types[1] = <char>NPY_FLOAT
 * ufunc__zeta_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2597
 * ufunc__zeta_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc__zeta_types[0] = <char>NPY_FLOAT
 * ufunc__zeta_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__zeta_types[2] = <char>NPY_FLOAT
 * ufunc__zeta_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2598
 * ufunc__zeta_types[0] = <char>NPY_FLOAT
 * ufunc__zeta_types[1] = <char>NPY_FLOAT
 * ufunc__zeta_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc__zeta_types[3] = <char>NPY_DOUBLE
 * ufunc__zeta_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2599
 * ufunc__zeta_types[1] = <char>NPY_FLOAT
 * ufunc__zeta_types[2] = <char>NPY_FLOAT
 * ufunc__zeta_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__zeta_types[4] = <char>NPY_DOUBLE
 * ufunc__zeta_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2600
 * ufunc__zeta_types[2] = <char>NPY_FLOAT
 * ufunc__zeta_types[3] = <char>NPY_DOUBLE
 * ufunc__zeta_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__zeta_types[5] = <char>NPY_DOUBLE
 * ufunc__zeta_ptr[2*0] = <void*>_func_zeta
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2601
 * ufunc__zeta_types[3] = <char>NPY_DOUBLE
 * ufunc__zeta_types[4] = <char>NPY_DOUBLE
 * ufunc__zeta_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc__zeta_ptr[2*0] = <void*>_func_zeta
 * ufunc__zeta_ptr[2*0+1] = <void*>(<char*>"_zeta")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2602
 * ufunc__zeta_types[4] = <char>NPY_DOUBLE
 * ufunc__zeta_types[5] = <char>NPY_DOUBLE
 * ufunc__zeta_ptr[2*0] = <void*>_func_zeta             # <<<<<<<<<<<<<<
 * ufunc__zeta_ptr[2*0+1] = <void*>(<char*>"_zeta")
 * ufunc__zeta_ptr[2*1] = <void*>_func_zeta
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_ptr[0]) = ((void *)zeta);

  /* "scipy/special/_ufuncs.pyx":2603
 * ufunc__zeta_types[5] = <char>NPY_DOUBLE
 * ufunc__zeta_ptr[2*0] = <void*>_func_zeta
 * ufunc__zeta_ptr[2*0+1] = <void*>(<char*>"_zeta")             # <<<<<<<<<<<<<<
 * ufunc__zeta_ptr[2*1] = <void*>_func_zeta
 * ufunc__zeta_ptr[2*1+1] = <void*>(<char*>"_zeta")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_ptr[1]) = ((void *)((char *)((char *)"_zeta")));

  /* "scipy/special/_ufuncs.pyx":2604
 * ufunc__zeta_ptr[2*0] = <void*>_func_zeta
 * ufunc__zeta_ptr[2*0+1] = <void*>(<char*>"_zeta")
 * ufunc__zeta_ptr[2*1] = <void*>_func_zeta             # <<<<<<<<<<<<<<
 * ufunc__zeta_ptr[2*1+1] = <void*>(<char*>"_zeta")
 * ufunc__zeta_data[0] = &ufunc__zeta_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_ptr[2]) = ((void *)zeta);

  /* "scipy/special/_ufuncs.pyx":2605
 * ufunc__zeta_ptr[2*0+1] = <void*>(<char*>"_zeta")
 * ufunc__zeta_ptr[2*1] = <void*>_func_zeta
 * ufunc__zeta_ptr[2*1+1] = <void*>(<char*>"_zeta")             # <<<<<<<<<<<<<<
 * ufunc__zeta_data[0] = &ufunc__zeta_ptr[2*0]
 * ufunc__zeta_data[1] = &ufunc__zeta_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_ptr[3]) = ((void *)((char *)((char *)"_zeta")));

  /* "scipy/special/_ufuncs.pyx":2606
 * ufunc__zeta_ptr[2*1] = <void*>_func_zeta
 * ufunc__zeta_ptr[2*1+1] = <void*>(<char*>"_zeta")
 * ufunc__zeta_data[0] = &ufunc__zeta_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc__zeta_data[1] = &ufunc__zeta_ptr[2*1]
 * _zeta = np.PyUFunc_FromFuncAndData(ufunc__zeta_loops, ufunc__zeta_data, ufunc__zeta_types, 2, 2, 1, 0, "_zeta", ufunc__zeta_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2607
 * ufunc__zeta_ptr[2*1+1] = <void*>(<char*>"_zeta")
 * ufunc__zeta_data[0] = &ufunc__zeta_ptr[2*0]
 * ufunc__zeta_data[1] = &ufunc__zeta_ptr[2*1]             # <<<<<<<<<<<<<<
 * _zeta = np.PyUFunc_FromFuncAndData(ufunc__zeta_loops, ufunc__zeta_data, ufunc__zeta_types, 2, 2, 1, 0, "_zeta", ufunc__zeta_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2608
 * ufunc__zeta_data[0] = &ufunc__zeta_ptr[2*0]
 * ufunc__zeta_data[1] = &ufunc__zeta_ptr[2*1]
 * _zeta = np.PyUFunc_FromFuncAndData(ufunc__zeta_loops, ufunc__zeta_data, ufunc__zeta_types, 2, 2, 1, 0, "_zeta", ufunc__zeta_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_agm_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_types, 2, 2, 1, 0, ((char *)"_zeta"), __pyx_v_5scipy_7special_7_ufuncs_ufunc__zeta_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2608, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_zeta, __pyx_t_2) < 0) __PYX_ERR(1, 2608, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2614
 * cdef void *ufunc_agm_data[2]
 * cdef char ufunc_agm_types[6]
 * cdef char *ufunc_agm_doc = (             # <<<<<<<<<<<<<<
 *     "agm(a, b)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_doc = ((char *)"agm(a, b)\n\nCompute the arithmetic-geometric mean of `a` and `b`.\n\nStart with a_0 = a and b_0 = b and iteratively compute::\n\n    a_{n+1} = (a_n + b_n)/2\n    b_{n+1} = sqrt(a_n*b_n)\n\na_n and b_n converge to the same limit as n increases; their common\nlimit is agm(a, b).\n\nParameters\n----------\na, b : array_like\n    Real values only.  If the values are both negative, the result\n    is negative.  If one value is negative and the other is positive,\n    `nan` is returned.\n\nReturns\n-------\nfloat\n    The arithmetic-geometric mean of `a` and `b`.\n\nExamples\n--------\n>>> from scipy.special import agm\n>>> a, b = 24.0, 6.0\n>>> agm(a, b)\n13.458171481725614\n\nCompare that result to the iteration:\n\n>>> while a != b:\n...     a, b = (a + b)/2, np.sqrt(a*b)\n...     print(\"a = %19.16f  b=%19.16f\" % (a, b))\n...\na = 15.0000000000000000  b=12.0000000000000000\na = 13.5000000000000000  b=13.4164078649987388\na = 13.4582039324993694  b=13.4581390309909850\na = 13.4581714817451772  b=13.4581714817060547\na = 13.4581714817256159  b=13.4581714817256159\n\nWhen array-like arguments are given, broadcasting applies:\n\n>>> a = np.array([[1.5], [3], [6]])  # a has shape (3, 1).\n>>> b = np.array([6, 12, 24, 48])    # b has shape (4,).\n>>> agm(a, b)\narray([[  3.36454287,   5.42363427,   9.05798751,  15.53650756],\n       [  4.37037309,   6.72908574,  10.84726853,  18.11597502],\n       [  6.        ,   8.74074619,  13.45817148,  21.69453707]])");

  /* "scipy/special/_ufuncs.pyx":2666
 *     "       [  4.37037309,   6.72908574,  10.84726853,  18.11597502],\n"
 *     "       [  6.        ,   8.74074619,  13.45817148,  21.69453707]])")
 * ufunc_agm_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_agm_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_agm_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":2667
 *     "       [  6.        ,   8.74074619,  13.45817148,  21.69453707]])")
 * ufunc_agm_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_agm_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_agm_types[0] = <char>NPY_FLOAT
 * ufunc_agm_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":2668
 * ufunc_agm_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_agm_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_agm_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_agm_types[1] = <char>NPY_FLOAT
 * ufunc_agm_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2669
 * ufunc_agm_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_agm_types[0] = <char>NPY_FLOAT
 * ufunc_agm_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_agm_types[2] = <char>NPY_FLOAT
 * ufunc_agm_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2670
 * ufunc_agm_types[0] = <char>NPY_FLOAT
 * ufunc_agm_types[1] = <char>NPY_FLOAT
 * ufunc_agm_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_agm_types[3] = <char>NPY_DOUBLE
 * ufunc_agm_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2671
 * ufunc_agm_types[1] = <char>NPY_FLOAT
 * ufunc_agm_types[2] = <char>NPY_FLOAT
 * ufunc_agm_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_agm_types[4] = <char>NPY_DOUBLE
 * ufunc_agm_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2672
 * ufunc_agm_types[2] = <char>NPY_FLOAT
 * ufunc_agm_types[3] = <char>NPY_DOUBLE
 * ufunc_agm_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_agm_types[5] = <char>NPY_DOUBLE
 * ufunc_agm_ptr[2*0] = <void*>_func_agm
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2673
 * ufunc_agm_types[3] = <char>NPY_DOUBLE
 * ufunc_agm_types[4] = <char>NPY_DOUBLE
 * ufunc_agm_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_agm_ptr[2*0] = <void*>_func_agm
 * ufunc_agm_ptr[2*0+1] = <void*>(<char*>"agm")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2674
 * ufunc_agm_types[4] = <char>NPY_DOUBLE
 * ufunc_agm_types[5] = <char>NPY_DOUBLE
 * ufunc_agm_ptr[2*0] = <void*>_func_agm             # <<<<<<<<<<<<<<
 * ufunc_agm_ptr[2*0+1] = <void*>(<char*>"agm")
 * ufunc_agm_ptr[2*1] = <void*>_func_agm
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_ptr[0]) = ((void *)__pyx_f_5scipy_7special_4_agm_agm);

  /* "scipy/special/_ufuncs.pyx":2675
 * ufunc_agm_types[5] = <char>NPY_DOUBLE
 * ufunc_agm_ptr[2*0] = <void*>_func_agm
 * ufunc_agm_ptr[2*0+1] = <void*>(<char*>"agm")             # <<<<<<<<<<<<<<
 * ufunc_agm_ptr[2*1] = <void*>_func_agm
 * ufunc_agm_ptr[2*1+1] = <void*>(<char*>"agm")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_ptr[1]) = ((void *)((char *)((char *)"agm")));

  /* "scipy/special/_ufuncs.pyx":2676
 * ufunc_agm_ptr[2*0] = <void*>_func_agm
 * ufunc_agm_ptr[2*0+1] = <void*>(<char*>"agm")
 * ufunc_agm_ptr[2*1] = <void*>_func_agm             # <<<<<<<<<<<<<<
 * ufunc_agm_ptr[2*1+1] = <void*>(<char*>"agm")
 * ufunc_agm_data[0] = &ufunc_agm_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_ptr[2]) = ((void *)__pyx_f_5scipy_7special_4_agm_agm);

  /* "scipy/special/_ufuncs.pyx":2677
 * ufunc_agm_ptr[2*0+1] = <void*>(<char*>"agm")
 * ufunc_agm_ptr[2*1] = <void*>_func_agm
 * ufunc_agm_ptr[2*1+1] = <void*>(<char*>"agm")             # <<<<<<<<<<<<<<
 * ufunc_agm_data[0] = &ufunc_agm_ptr[2*0]
 * ufunc_agm_data[1] = &ufunc_agm_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_ptr[3]) = ((void *)((char *)((char *)"agm")));

  /* "scipy/special/_ufuncs.pyx":2678
 * ufunc_agm_ptr[2*1] = <void*>_func_agm
 * ufunc_agm_ptr[2*1+1] = <void*>(<char*>"agm")
 * ufunc_agm_data[0] = &ufunc_agm_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_agm_data[1] = &ufunc_agm_ptr[2*1]
 * agm = np.PyUFunc_FromFuncAndData(ufunc_agm_loops, ufunc_agm_data, ufunc_agm_types, 2, 2, 1, 0, "agm", ufunc_agm_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2679
 * ufunc_agm_ptr[2*1+1] = <void*>(<char*>"agm")
 * ufunc_agm_data[0] = &ufunc_agm_ptr[2*0]
 * ufunc_agm_data[1] = &ufunc_agm_ptr[2*1]             # <<<<<<<<<<<<<<
 * agm = np.PyUFunc_FromFuncAndData(ufunc_agm_loops, ufunc_agm_data, ufunc_agm_types, 2, 2, 1, 0, "agm", ufunc_agm_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2680
 * ufunc_agm_data[0] = &ufunc_agm_ptr[2*0]
 * ufunc_agm_data[1] = &ufunc_agm_ptr[2*1]
 * agm = np.PyUFunc_FromFuncAndData(ufunc_agm_loops, ufunc_agm_data, ufunc_agm_types, 2, 2, 1, 0, "agm", ufunc_agm_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_airy_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_types, 2, 2, 1, 0, ((char *)"agm"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_agm_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2680, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_agm, __pyx_t_2) < 0) __PYX_ERR(1, 2680, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2686
 * cdef void *ufunc_airy_data[4]
 * cdef char ufunc_airy_types[20]
 * cdef char *ufunc_airy_doc = (             # <<<<<<<<<<<<<<
 *     "airy(z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_doc = ((char *)"airy(z)\n\nAiry functions and their derivatives.\n\nParameters\n----------\nz : array_like\n    Real or complex argument.\n\nReturns\n-------\nAi, Aip, Bi, Bip : ndarrays\n    Airy functions Ai and Bi, and their derivatives Aip and Bip.\n\nNotes\n-----\nThe Airy functions Ai and Bi are two independent solutions of\n\n.. math:: y''(x) = x y(x).\n\nFor real `z` in [-10, 10], the computation is carried out by calling\nthe Cephes [1]_ `airy` routine, which uses power series summation\nfor small `z` and rational minimax approximations for large `z`.\n\nOutside this range, the AMOS [2]_ `zairy` and `zbiry` routines are\nemployed.  They are computed using power series for :math:`|z| < 1` and\nthe following relations to modified Bessel functions for larger `z`\n(where :math:`t \\equiv 2 z^{3/2}/3`):\n\n.. math::\n\n    Ai(z) = \\frac{1}{\\pi \\sqrt{3}} K_{1/3}(t)\n\n    Ai'(z) = -\\frac{z}{\\pi \\sqrt{3}} K_{2/3}(t)\n\n    Bi(z) = \\sqrt{\\frac{z}{3}} \\left(I_{-1/3}(t) + I_{1/3}(t) \\right)\n\n    Bi'(z) = \\frac{z}{\\sqrt{3}} \\left(I_{-2/3}(t) + I_{2/3}(t)\\right)\n\nSee also\n--------\nairye : exponentially scaled Airy functions.\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/\n.. [2] Donald E. Amos, \"AMOS, A Portable Package for Bessel Functions\n       of a Complex Argument and Nonnegative Order\",\n       http://netlib.org/amos/\n\nExamples\n--------\nCompute the Airy functions on the interval [-15, 5].\n\n>>> from scipy import special\n>>> x = np.linspace(-15, 5, 201)\n>>> ai, aip, bi, bip = special.airy(x)\n\nPlot Ai(x) and Bi(x).\n\n>>> import matplotlib.pyplot as plt\n>>> plt.plot(x, ai, 'r', label='Ai(x)')\n>>> plt.plot(x, bi, 'b--', label='Bi(x)')\n>>> plt.ylim(-0.5, 1.0)\n>>> plt.grid()\n>>> plt.legend(loc='upper left')\n>>> plt.show()");

  /* "scipy/special/_ufuncs.pyx":2755
 *     ">>> plt.legend(loc='upper left')\n"
 *     ">>> plt.show()")
 * ufunc_airy_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_f_ffff             # <<<<<<<<<<<<<<
 * ufunc_airy_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd
 * ufunc_airy_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_F_FFFF
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dddd_As_f_ffff);

  /* "scipy/special/_ufuncs.pyx":2756
 *     ">>> plt.show()")
 * ufunc_airy_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_f_ffff
 * ufunc_airy_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd             # <<<<<<<<<<<<<<
 * ufunc_airy_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_F_FFFF
 * ufunc_airy_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_D_DDDD
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dddd_As_d_dddd);

  /* "scipy/special/_ufuncs.pyx":2757
 * ufunc_airy_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_f_ffff
 * ufunc_airy_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd
 * ufunc_airy_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_F_FFFF             # <<<<<<<<<<<<<<
 * ufunc_airy_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_D_DDDD
 * ufunc_airy_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DDDD_As_F_FFFF);

  /* "scipy/special/_ufuncs.pyx":2758
 * ufunc_airy_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd
 * ufunc_airy_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_F_FFFF
 * ufunc_airy_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_D_DDDD             # <<<<<<<<<<<<<<
 * ufunc_airy_types[0] = <char>NPY_FLOAT
 * ufunc_airy_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DDDD_As_D_DDDD);

  /* "scipy/special/_ufuncs.pyx":2759
 * ufunc_airy_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_F_FFFF
 * ufunc_airy_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_D_DDDD
 * ufunc_airy_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_airy_types[1] = <char>NPY_FLOAT
 * ufunc_airy_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2760
 * ufunc_airy_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_D_DDDD
 * ufunc_airy_types[0] = <char>NPY_FLOAT
 * ufunc_airy_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_airy_types[2] = <char>NPY_FLOAT
 * ufunc_airy_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2761
 * ufunc_airy_types[0] = <char>NPY_FLOAT
 * ufunc_airy_types[1] = <char>NPY_FLOAT
 * ufunc_airy_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_airy_types[3] = <char>NPY_FLOAT
 * ufunc_airy_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2762
 * ufunc_airy_types[1] = <char>NPY_FLOAT
 * ufunc_airy_types[2] = <char>NPY_FLOAT
 * ufunc_airy_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_airy_types[4] = <char>NPY_FLOAT
 * ufunc_airy_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2763
 * ufunc_airy_types[2] = <char>NPY_FLOAT
 * ufunc_airy_types[3] = <char>NPY_FLOAT
 * ufunc_airy_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_airy_types[5] = <char>NPY_DOUBLE
 * ufunc_airy_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2764
 * ufunc_airy_types[3] = <char>NPY_FLOAT
 * ufunc_airy_types[4] = <char>NPY_FLOAT
 * ufunc_airy_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airy_types[6] = <char>NPY_DOUBLE
 * ufunc_airy_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2765
 * ufunc_airy_types[4] = <char>NPY_FLOAT
 * ufunc_airy_types[5] = <char>NPY_DOUBLE
 * ufunc_airy_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airy_types[7] = <char>NPY_DOUBLE
 * ufunc_airy_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2766
 * ufunc_airy_types[5] = <char>NPY_DOUBLE
 * ufunc_airy_types[6] = <char>NPY_DOUBLE
 * ufunc_airy_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airy_types[8] = <char>NPY_DOUBLE
 * ufunc_airy_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2767
 * ufunc_airy_types[6] = <char>NPY_DOUBLE
 * ufunc_airy_types[7] = <char>NPY_DOUBLE
 * ufunc_airy_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airy_types[9] = <char>NPY_DOUBLE
 * ufunc_airy_types[10] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2768
 * ufunc_airy_types[7] = <char>NPY_DOUBLE
 * ufunc_airy_types[8] = <char>NPY_DOUBLE
 * ufunc_airy_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airy_types[10] = <char>NPY_CFLOAT
 * ufunc_airy_types[11] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2769
 * ufunc_airy_types[8] = <char>NPY_DOUBLE
 * ufunc_airy_types[9] = <char>NPY_DOUBLE
 * ufunc_airy_types[10] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_airy_types[11] = <char>NPY_CFLOAT
 * ufunc_airy_types[12] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[10]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2770
 * ufunc_airy_types[9] = <char>NPY_DOUBLE
 * ufunc_airy_types[10] = <char>NPY_CFLOAT
 * ufunc_airy_types[11] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_airy_types[12] = <char>NPY_CFLOAT
 * ufunc_airy_types[13] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[11]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2771
 * ufunc_airy_types[10] = <char>NPY_CFLOAT
 * ufunc_airy_types[11] = <char>NPY_CFLOAT
 * ufunc_airy_types[12] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_airy_types[13] = <char>NPY_CFLOAT
 * ufunc_airy_types[14] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[12]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2772
 * ufunc_airy_types[11] = <char>NPY_CFLOAT
 * ufunc_airy_types[12] = <char>NPY_CFLOAT
 * ufunc_airy_types[13] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_airy_types[14] = <char>NPY_CFLOAT
 * ufunc_airy_types[15] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[13]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2773
 * ufunc_airy_types[12] = <char>NPY_CFLOAT
 * ufunc_airy_types[13] = <char>NPY_CFLOAT
 * ufunc_airy_types[14] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_airy_types[15] = <char>NPY_CDOUBLE
 * ufunc_airy_types[16] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[14]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2774
 * ufunc_airy_types[13] = <char>NPY_CFLOAT
 * ufunc_airy_types[14] = <char>NPY_CFLOAT
 * ufunc_airy_types[15] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airy_types[16] = <char>NPY_CDOUBLE
 * ufunc_airy_types[17] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[15]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2775
 * ufunc_airy_types[14] = <char>NPY_CFLOAT
 * ufunc_airy_types[15] = <char>NPY_CDOUBLE
 * ufunc_airy_types[16] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airy_types[17] = <char>NPY_CDOUBLE
 * ufunc_airy_types[18] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[16]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2776
 * ufunc_airy_types[15] = <char>NPY_CDOUBLE
 * ufunc_airy_types[16] = <char>NPY_CDOUBLE
 * ufunc_airy_types[17] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airy_types[18] = <char>NPY_CDOUBLE
 * ufunc_airy_types[19] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[17]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2777
 * ufunc_airy_types[16] = <char>NPY_CDOUBLE
 * ufunc_airy_types[17] = <char>NPY_CDOUBLE
 * ufunc_airy_types[18] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airy_types[19] = <char>NPY_CDOUBLE
 * ufunc_airy_ptr[2*0] = <void*>_func_airy_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[18]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2778
 * ufunc_airy_types[17] = <char>NPY_CDOUBLE
 * ufunc_airy_types[18] = <char>NPY_CDOUBLE
 * ufunc_airy_types[19] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airy_ptr[2*0] = <void*>_func_airy_wrap
 * ufunc_airy_ptr[2*0+1] = <void*>(<char*>"airy")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types[19]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2779
 * ufunc_airy_types[18] = <char>NPY_CDOUBLE
 * ufunc_airy_types[19] = <char>NPY_CDOUBLE
 * ufunc_airy_ptr[2*0] = <void*>_func_airy_wrap             # <<<<<<<<<<<<<<
 * ufunc_airy_ptr[2*0+1] = <void*>(<char*>"airy")
 * ufunc_airy_ptr[2*1] = <void*>_func_airy_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[0]) = ((void *)airy_wrap);

  /* "scipy/special/_ufuncs.pyx":2780
 * ufunc_airy_types[19] = <char>NPY_CDOUBLE
 * ufunc_airy_ptr[2*0] = <void*>_func_airy_wrap
 * ufunc_airy_ptr[2*0+1] = <void*>(<char*>"airy")             # <<<<<<<<<<<<<<
 * ufunc_airy_ptr[2*1] = <void*>_func_airy_wrap
 * ufunc_airy_ptr[2*1+1] = <void*>(<char*>"airy")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[1]) = ((void *)((char *)((char *)"airy")));

  /* "scipy/special/_ufuncs.pyx":2781
 * ufunc_airy_ptr[2*0] = <void*>_func_airy_wrap
 * ufunc_airy_ptr[2*0+1] = <void*>(<char*>"airy")
 * ufunc_airy_ptr[2*1] = <void*>_func_airy_wrap             # <<<<<<<<<<<<<<
 * ufunc_airy_ptr[2*1+1] = <void*>(<char*>"airy")
 * ufunc_airy_ptr[2*2] = <void*>_func_cairy_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[2]) = ((void *)airy_wrap);

  /* "scipy/special/_ufuncs.pyx":2782
 * ufunc_airy_ptr[2*0+1] = <void*>(<char*>"airy")
 * ufunc_airy_ptr[2*1] = <void*>_func_airy_wrap
 * ufunc_airy_ptr[2*1+1] = <void*>(<char*>"airy")             # <<<<<<<<<<<<<<
 * ufunc_airy_ptr[2*2] = <void*>_func_cairy_wrap
 * ufunc_airy_ptr[2*2+1] = <void*>(<char*>"airy")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[3]) = ((void *)((char *)((char *)"airy")));

  /* "scipy/special/_ufuncs.pyx":2783
 * ufunc_airy_ptr[2*1] = <void*>_func_airy_wrap
 * ufunc_airy_ptr[2*1+1] = <void*>(<char*>"airy")
 * ufunc_airy_ptr[2*2] = <void*>_func_cairy_wrap             # <<<<<<<<<<<<<<
 * ufunc_airy_ptr[2*2+1] = <void*>(<char*>"airy")
 * ufunc_airy_ptr[2*3] = <void*>_func_cairy_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[4]) = ((void *)cairy_wrap);

  /* "scipy/special/_ufuncs.pyx":2784
 * ufunc_airy_ptr[2*1+1] = <void*>(<char*>"airy")
 * ufunc_airy_ptr[2*2] = <void*>_func_cairy_wrap
 * ufunc_airy_ptr[2*2+1] = <void*>(<char*>"airy")             # <<<<<<<<<<<<<<
 * ufunc_airy_ptr[2*3] = <void*>_func_cairy_wrap
 * ufunc_airy_ptr[2*3+1] = <void*>(<char*>"airy")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[5]) = ((void *)((char *)((char *)"airy")));

  /* "scipy/special/_ufuncs.pyx":2785
 * ufunc_airy_ptr[2*2] = <void*>_func_cairy_wrap
 * ufunc_airy_ptr[2*2+1] = <void*>(<char*>"airy")
 * ufunc_airy_ptr[2*3] = <void*>_func_cairy_wrap             # <<<<<<<<<<<<<<
 * ufunc_airy_ptr[2*3+1] = <void*>(<char*>"airy")
 * ufunc_airy_data[0] = &ufunc_airy_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[6]) = ((void *)cairy_wrap);

  /* "scipy/special/_ufuncs.pyx":2786
 * ufunc_airy_ptr[2*2+1] = <void*>(<char*>"airy")
 * ufunc_airy_ptr[2*3] = <void*>_func_cairy_wrap
 * ufunc_airy_ptr[2*3+1] = <void*>(<char*>"airy")             # <<<<<<<<<<<<<<
 * ufunc_airy_data[0] = &ufunc_airy_ptr[2*0]
 * ufunc_airy_data[1] = &ufunc_airy_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[7]) = ((void *)((char *)((char *)"airy")));

  /* "scipy/special/_ufuncs.pyx":2787
 * ufunc_airy_ptr[2*3] = <void*>_func_cairy_wrap
 * ufunc_airy_ptr[2*3+1] = <void*>(<char*>"airy")
 * ufunc_airy_data[0] = &ufunc_airy_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_airy_data[1] = &ufunc_airy_ptr[2*1]
 * ufunc_airy_data[2] = &ufunc_airy_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2788
 * ufunc_airy_ptr[2*3+1] = <void*>(<char*>"airy")
 * ufunc_airy_data[0] = &ufunc_airy_ptr[2*0]
 * ufunc_airy_data[1] = &ufunc_airy_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_airy_data[2] = &ufunc_airy_ptr[2*2]
 * ufunc_airy_data[3] = &ufunc_airy_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2789
 * ufunc_airy_data[0] = &ufunc_airy_ptr[2*0]
 * ufunc_airy_data[1] = &ufunc_airy_ptr[2*1]
 * ufunc_airy_data[2] = &ufunc_airy_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_airy_data[3] = &ufunc_airy_ptr[2*3]
 * airy = np.PyUFunc_FromFuncAndData(ufunc_airy_loops, ufunc_airy_data, ufunc_airy_types, 4, 1, 4, 0, "airy", ufunc_airy_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":2790
 * ufunc_airy_data[1] = &ufunc_airy_ptr[2*1]
 * ufunc_airy_data[2] = &ufunc_airy_ptr[2*2]
 * ufunc_airy_data[3] = &ufunc_airy_ptr[2*3]             # <<<<<<<<<<<<<<
 * airy = np.PyUFunc_FromFuncAndData(ufunc_airy_loops, ufunc_airy_data, ufunc_airy_types, 4, 1, 4, 0, "airy", ufunc_airy_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":2791
 * ufunc_airy_data[2] = &ufunc_airy_ptr[2*2]
 * ufunc_airy_data[3] = &ufunc_airy_ptr[2*3]
 * airy = np.PyUFunc_FromFuncAndData(ufunc_airy_loops, ufunc_airy_data, ufunc_airy_types, 4, 1, 4, 0, "airy", ufunc_airy_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_airye_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_types, 4, 1, 4, 0, ((char *)"airy"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_airy_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2791, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_airy, __pyx_t_2) < 0) __PYX_ERR(1, 2791, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2797
 * cdef void *ufunc_airye_data[4]
 * cdef char ufunc_airye_types[20]
 * cdef char *ufunc_airye_doc = (             # <<<<<<<<<<<<<<
 *     "airye(z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_doc = ((char *)"airye(z)\n\nExponentially scaled Airy functions and their derivatives.\n\nScaling::\n\n    eAi  = Ai  * exp(2.0/3.0*z*sqrt(z))\n    eAip = Aip * exp(2.0/3.0*z*sqrt(z))\n    eBi  = Bi  * exp(-abs(2.0/3.0*(z*sqrt(z)).real))\n    eBip = Bip * exp(-abs(2.0/3.0*(z*sqrt(z)).real))\n\nParameters\n----------\nz : array_like\n    Real or complex argument.\n\nReturns\n-------\neAi, eAip, eBi, eBip : array_like\n    Airy functions Ai and Bi, and their derivatives Aip and Bip\n\nNotes\n-----\nWrapper for the AMOS [1]_ routines `zairy` and `zbiry`.\n\nSee also\n--------\nairy\n\nReferences\n----------\n.. [1] Donald E. Amos, \"AMOS, A Portable Package for Bessel Functions\n       of a Complex Argument and Nonnegative Order\",\n       http://netlib.org/amos/");

  /* "scipy/special/_ufuncs.pyx":2832
 *     "       of a Complex Argument and Nonnegative Order\",\n"
 *     "       http://netlib.org/amos/")
 * ufunc_airye_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_f_ffff             # <<<<<<<<<<<<<<
 * ufunc_airye_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd
 * ufunc_airye_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_F_FFFF
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dddd_As_f_ffff);

  /* "scipy/special/_ufuncs.pyx":2833
 *     "       http://netlib.org/amos/")
 * ufunc_airye_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_f_ffff
 * ufunc_airye_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd             # <<<<<<<<<<<<<<
 * ufunc_airye_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_F_FFFF
 * ufunc_airye_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_D_DDDD
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dddd_As_d_dddd);

  /* "scipy/special/_ufuncs.pyx":2834
 * ufunc_airye_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_f_ffff
 * ufunc_airye_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd
 * ufunc_airye_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_F_FFFF             # <<<<<<<<<<<<<<
 * ufunc_airye_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_D_DDDD
 * ufunc_airye_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DDDD_As_F_FFFF);

  /* "scipy/special/_ufuncs.pyx":2835
 * ufunc_airye_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd
 * ufunc_airye_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_F_FFFF
 * ufunc_airye_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_D_DDDD             # <<<<<<<<<<<<<<
 * ufunc_airye_types[0] = <char>NPY_FLOAT
 * ufunc_airye_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DDDD_As_D_DDDD);

  /* "scipy/special/_ufuncs.pyx":2836
 * ufunc_airye_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_F_FFFF
 * ufunc_airye_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_D_DDDD
 * ufunc_airye_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_airye_types[1] = <char>NPY_FLOAT
 * ufunc_airye_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2837
 * ufunc_airye_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DDDD_As_D_DDDD
 * ufunc_airye_types[0] = <char>NPY_FLOAT
 * ufunc_airye_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_airye_types[2] = <char>NPY_FLOAT
 * ufunc_airye_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2838
 * ufunc_airye_types[0] = <char>NPY_FLOAT
 * ufunc_airye_types[1] = <char>NPY_FLOAT
 * ufunc_airye_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_airye_types[3] = <char>NPY_FLOAT
 * ufunc_airye_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2839
 * ufunc_airye_types[1] = <char>NPY_FLOAT
 * ufunc_airye_types[2] = <char>NPY_FLOAT
 * ufunc_airye_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_airye_types[4] = <char>NPY_FLOAT
 * ufunc_airye_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2840
 * ufunc_airye_types[2] = <char>NPY_FLOAT
 * ufunc_airye_types[3] = <char>NPY_FLOAT
 * ufunc_airye_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_airye_types[5] = <char>NPY_DOUBLE
 * ufunc_airye_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2841
 * ufunc_airye_types[3] = <char>NPY_FLOAT
 * ufunc_airye_types[4] = <char>NPY_FLOAT
 * ufunc_airye_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airye_types[6] = <char>NPY_DOUBLE
 * ufunc_airye_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2842
 * ufunc_airye_types[4] = <char>NPY_FLOAT
 * ufunc_airye_types[5] = <char>NPY_DOUBLE
 * ufunc_airye_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airye_types[7] = <char>NPY_DOUBLE
 * ufunc_airye_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2843
 * ufunc_airye_types[5] = <char>NPY_DOUBLE
 * ufunc_airye_types[6] = <char>NPY_DOUBLE
 * ufunc_airye_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airye_types[8] = <char>NPY_DOUBLE
 * ufunc_airye_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2844
 * ufunc_airye_types[6] = <char>NPY_DOUBLE
 * ufunc_airye_types[7] = <char>NPY_DOUBLE
 * ufunc_airye_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airye_types[9] = <char>NPY_DOUBLE
 * ufunc_airye_types[10] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2845
 * ufunc_airye_types[7] = <char>NPY_DOUBLE
 * ufunc_airye_types[8] = <char>NPY_DOUBLE
 * ufunc_airye_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airye_types[10] = <char>NPY_CFLOAT
 * ufunc_airye_types[11] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2846
 * ufunc_airye_types[8] = <char>NPY_DOUBLE
 * ufunc_airye_types[9] = <char>NPY_DOUBLE
 * ufunc_airye_types[10] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_airye_types[11] = <char>NPY_CFLOAT
 * ufunc_airye_types[12] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[10]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2847
 * ufunc_airye_types[9] = <char>NPY_DOUBLE
 * ufunc_airye_types[10] = <char>NPY_CFLOAT
 * ufunc_airye_types[11] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_airye_types[12] = <char>NPY_CFLOAT
 * ufunc_airye_types[13] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[11]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2848
 * ufunc_airye_types[10] = <char>NPY_CFLOAT
 * ufunc_airye_types[11] = <char>NPY_CFLOAT
 * ufunc_airye_types[12] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_airye_types[13] = <char>NPY_CFLOAT
 * ufunc_airye_types[14] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[12]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2849
 * ufunc_airye_types[11] = <char>NPY_CFLOAT
 * ufunc_airye_types[12] = <char>NPY_CFLOAT
 * ufunc_airye_types[13] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_airye_types[14] = <char>NPY_CFLOAT
 * ufunc_airye_types[15] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[13]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2850
 * ufunc_airye_types[12] = <char>NPY_CFLOAT
 * ufunc_airye_types[13] = <char>NPY_CFLOAT
 * ufunc_airye_types[14] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_airye_types[15] = <char>NPY_CDOUBLE
 * ufunc_airye_types[16] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[14]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":2851
 * ufunc_airye_types[13] = <char>NPY_CFLOAT
 * ufunc_airye_types[14] = <char>NPY_CFLOAT
 * ufunc_airye_types[15] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airye_types[16] = <char>NPY_CDOUBLE
 * ufunc_airye_types[17] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[15]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2852
 * ufunc_airye_types[14] = <char>NPY_CFLOAT
 * ufunc_airye_types[15] = <char>NPY_CDOUBLE
 * ufunc_airye_types[16] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airye_types[17] = <char>NPY_CDOUBLE
 * ufunc_airye_types[18] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[16]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2853
 * ufunc_airye_types[15] = <char>NPY_CDOUBLE
 * ufunc_airye_types[16] = <char>NPY_CDOUBLE
 * ufunc_airye_types[17] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airye_types[18] = <char>NPY_CDOUBLE
 * ufunc_airye_types[19] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[17]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2854
 * ufunc_airye_types[16] = <char>NPY_CDOUBLE
 * ufunc_airye_types[17] = <char>NPY_CDOUBLE
 * ufunc_airye_types[18] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airye_types[19] = <char>NPY_CDOUBLE
 * ufunc_airye_ptr[2*0] = <void*>_func_cairy_wrap_e_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[18]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2855
 * ufunc_airye_types[17] = <char>NPY_CDOUBLE
 * ufunc_airye_types[18] = <char>NPY_CDOUBLE
 * ufunc_airye_types[19] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_airye_ptr[2*0] = <void*>_func_cairy_wrap_e_real
 * ufunc_airye_ptr[2*0+1] = <void*>(<char*>"airye")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types[19]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":2856
 * ufunc_airye_types[18] = <char>NPY_CDOUBLE
 * ufunc_airye_types[19] = <char>NPY_CDOUBLE
 * ufunc_airye_ptr[2*0] = <void*>_func_cairy_wrap_e_real             # <<<<<<<<<<<<<<
 * ufunc_airye_ptr[2*0+1] = <void*>(<char*>"airye")
 * ufunc_airye_ptr[2*1] = <void*>_func_cairy_wrap_e_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[0]) = ((void *)cairy_wrap_e_real);

  /* "scipy/special/_ufuncs.pyx":2857
 * ufunc_airye_types[19] = <char>NPY_CDOUBLE
 * ufunc_airye_ptr[2*0] = <void*>_func_cairy_wrap_e_real
 * ufunc_airye_ptr[2*0+1] = <void*>(<char*>"airye")             # <<<<<<<<<<<<<<
 * ufunc_airye_ptr[2*1] = <void*>_func_cairy_wrap_e_real
 * ufunc_airye_ptr[2*1+1] = <void*>(<char*>"airye")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[1]) = ((void *)((char *)((char *)"airye")));

  /* "scipy/special/_ufuncs.pyx":2858
 * ufunc_airye_ptr[2*0] = <void*>_func_cairy_wrap_e_real
 * ufunc_airye_ptr[2*0+1] = <void*>(<char*>"airye")
 * ufunc_airye_ptr[2*1] = <void*>_func_cairy_wrap_e_real             # <<<<<<<<<<<<<<
 * ufunc_airye_ptr[2*1+1] = <void*>(<char*>"airye")
 * ufunc_airye_ptr[2*2] = <void*>_func_cairy_wrap_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[2]) = ((void *)cairy_wrap_e_real);

  /* "scipy/special/_ufuncs.pyx":2859
 * ufunc_airye_ptr[2*0+1] = <void*>(<char*>"airye")
 * ufunc_airye_ptr[2*1] = <void*>_func_cairy_wrap_e_real
 * ufunc_airye_ptr[2*1+1] = <void*>(<char*>"airye")             # <<<<<<<<<<<<<<
 * ufunc_airye_ptr[2*2] = <void*>_func_cairy_wrap_e
 * ufunc_airye_ptr[2*2+1] = <void*>(<char*>"airye")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[3]) = ((void *)((char *)((char *)"airye")));

  /* "scipy/special/_ufuncs.pyx":2860
 * ufunc_airye_ptr[2*1] = <void*>_func_cairy_wrap_e_real
 * ufunc_airye_ptr[2*1+1] = <void*>(<char*>"airye")
 * ufunc_airye_ptr[2*2] = <void*>_func_cairy_wrap_e             # <<<<<<<<<<<<<<
 * ufunc_airye_ptr[2*2+1] = <void*>(<char*>"airye")
 * ufunc_airye_ptr[2*3] = <void*>_func_cairy_wrap_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[4]) = ((void *)cairy_wrap_e);

  /* "scipy/special/_ufuncs.pyx":2861
 * ufunc_airye_ptr[2*1+1] = <void*>(<char*>"airye")
 * ufunc_airye_ptr[2*2] = <void*>_func_cairy_wrap_e
 * ufunc_airye_ptr[2*2+1] = <void*>(<char*>"airye")             # <<<<<<<<<<<<<<
 * ufunc_airye_ptr[2*3] = <void*>_func_cairy_wrap_e
 * ufunc_airye_ptr[2*3+1] = <void*>(<char*>"airye")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[5]) = ((void *)((char *)((char *)"airye")));

  /* "scipy/special/_ufuncs.pyx":2862
 * ufunc_airye_ptr[2*2] = <void*>_func_cairy_wrap_e
 * ufunc_airye_ptr[2*2+1] = <void*>(<char*>"airye")
 * ufunc_airye_ptr[2*3] = <void*>_func_cairy_wrap_e             # <<<<<<<<<<<<<<
 * ufunc_airye_ptr[2*3+1] = <void*>(<char*>"airye")
 * ufunc_airye_data[0] = &ufunc_airye_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[6]) = ((void *)cairy_wrap_e);

  /* "scipy/special/_ufuncs.pyx":2863
 * ufunc_airye_ptr[2*2+1] = <void*>(<char*>"airye")
 * ufunc_airye_ptr[2*3] = <void*>_func_cairy_wrap_e
 * ufunc_airye_ptr[2*3+1] = <void*>(<char*>"airye")             # <<<<<<<<<<<<<<
 * ufunc_airye_data[0] = &ufunc_airye_ptr[2*0]
 * ufunc_airye_data[1] = &ufunc_airye_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[7]) = ((void *)((char *)((char *)"airye")));

  /* "scipy/special/_ufuncs.pyx":2864
 * ufunc_airye_ptr[2*3] = <void*>_func_cairy_wrap_e
 * ufunc_airye_ptr[2*3+1] = <void*>(<char*>"airye")
 * ufunc_airye_data[0] = &ufunc_airye_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_airye_data[1] = &ufunc_airye_ptr[2*1]
 * ufunc_airye_data[2] = &ufunc_airye_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2865
 * ufunc_airye_ptr[2*3+1] = <void*>(<char*>"airye")
 * ufunc_airye_data[0] = &ufunc_airye_ptr[2*0]
 * ufunc_airye_data[1] = &ufunc_airye_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_airye_data[2] = &ufunc_airye_ptr[2*2]
 * ufunc_airye_data[3] = &ufunc_airye_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2866
 * ufunc_airye_data[0] = &ufunc_airye_ptr[2*0]
 * ufunc_airye_data[1] = &ufunc_airye_ptr[2*1]
 * ufunc_airye_data[2] = &ufunc_airye_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_airye_data[3] = &ufunc_airye_ptr[2*3]
 * airye = np.PyUFunc_FromFuncAndData(ufunc_airye_loops, ufunc_airye_data, ufunc_airye_types, 4, 1, 4, 0, "airye", ufunc_airye_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":2867
 * ufunc_airye_data[1] = &ufunc_airye_ptr[2*1]
 * ufunc_airye_data[2] = &ufunc_airye_ptr[2*2]
 * ufunc_airye_data[3] = &ufunc_airye_ptr[2*3]             # <<<<<<<<<<<<<<
 * airye = np.PyUFunc_FromFuncAndData(ufunc_airye_loops, ufunc_airye_data, ufunc_airye_types, 4, 1, 4, 0, "airye", ufunc_airye_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":2868
 * ufunc_airye_data[2] = &ufunc_airye_ptr[2*2]
 * ufunc_airye_data[3] = &ufunc_airye_ptr[2*3]
 * airye = np.PyUFunc_FromFuncAndData(ufunc_airye_loops, ufunc_airye_data, ufunc_airye_types, 4, 1, 4, 0, "airye", ufunc_airye_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_bdtr_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_types, 4, 1, 4, 0, ((char *)"airye"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_airye_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2868, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_airye, __pyx_t_2) < 0) __PYX_ERR(1, 2868, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2874
 * cdef void *ufunc_bdtr_data[3]
 * cdef char ufunc_bdtr_types[12]
 * cdef char *ufunc_bdtr_doc = (             # <<<<<<<<<<<<<<
 *     "bdtr(k, n, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_doc = ((char *)"bdtr(k, n, p)\n\nBinomial distribution cumulative distribution function.\n\nSum of the terms 0 through `k` of the Binomial probability density.\n\n.. math::\n    \\mathrm{bdtr}(k, n, p) = \\sum_{j=0}^k {{n}\\choose{j}} p^j (1-p)^{n-j}\n\nParameters\n----------\nk : array_like\n    Number of successes (int).\nn : array_like\n    Number of events (int).\np : array_like\n    Probability of success in a single event (float).\n\nReturns\n-------\ny : ndarray\n    Probability of `k` or fewer successes in `n` independent events with\n    success probabilities of `p`.\n\nNotes\n-----\nThe terms are not summed directly; instead the regularized incomplete beta\nfunction is employed, according to the formula,\n\n.. math::\n    \\mathrm{bdtr}(k, n, p) = I_{1 - p}(n - k, k + 1).\n\nWrapper for the Cephes [1]_ routine `bdtr`.\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":2913
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_bdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d             # <<<<<<<<<<<<<<
 * ufunc_bdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_iid__As_lld_d);

  /* "scipy/special/_ufuncs.pyx":2914
 *     "       http://www.netlib.org/cephes/")
 * ufunc_bdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_bdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_bdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtr_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":2915
 * ufunc_bdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_bdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[0] = <char>NPY_LONG
 * ufunc_bdtr_types[1] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":2916
 * ufunc_bdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtr_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[1] = <char>NPY_LONG
 * ufunc_bdtr_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2917
 * ufunc_bdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtr_types[0] = <char>NPY_LONG
 * ufunc_bdtr_types[1] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[2] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[1]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2918
 * ufunc_bdtr_types[0] = <char>NPY_LONG
 * ufunc_bdtr_types[1] = <char>NPY_LONG
 * ufunc_bdtr_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2919
 * ufunc_bdtr_types[1] = <char>NPY_LONG
 * ufunc_bdtr_types[2] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[4] = <char>NPY_FLOAT
 * ufunc_bdtr_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2920
 * ufunc_bdtr_types[2] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[5] = <char>NPY_FLOAT
 * ufunc_bdtr_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2921
 * ufunc_bdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[4] = <char>NPY_FLOAT
 * ufunc_bdtr_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[6] = <char>NPY_FLOAT
 * ufunc_bdtr_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2922
 * ufunc_bdtr_types[4] = <char>NPY_FLOAT
 * ufunc_bdtr_types[5] = <char>NPY_FLOAT
 * ufunc_bdtr_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[7] = <char>NPY_FLOAT
 * ufunc_bdtr_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2923
 * ufunc_bdtr_types[5] = <char>NPY_FLOAT
 * ufunc_bdtr_types[6] = <char>NPY_FLOAT
 * ufunc_bdtr_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2924
 * ufunc_bdtr_types[6] = <char>NPY_FLOAT
 * ufunc_bdtr_types[7] = <char>NPY_FLOAT
 * ufunc_bdtr_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[9] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2925
 * ufunc_bdtr_types[7] = <char>NPY_FLOAT
 * ufunc_bdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[10] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2926
 * ufunc_bdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[9] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtr_types[11] = <char>NPY_DOUBLE
 * ufunc_bdtr_ptr[2*0] = <void*>_func_bdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2927
 * ufunc_bdtr_types[9] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[10] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtr_ptr[2*0] = <void*>_func_bdtr
 * ufunc_bdtr_ptr[2*0+1] = <void*>(<char*>"bdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2928
 * ufunc_bdtr_types[10] = <char>NPY_DOUBLE
 * ufunc_bdtr_types[11] = <char>NPY_DOUBLE
 * ufunc_bdtr_ptr[2*0] = <void*>_func_bdtr             # <<<<<<<<<<<<<<
 * ufunc_bdtr_ptr[2*0+1] = <void*>(<char*>"bdtr")
 * ufunc_bdtr_ptr[2*1] = <void*>_func_bdtr_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_ptr[0]) = ((void *)bdtr);

  /* "scipy/special/_ufuncs.pyx":2929
 * ufunc_bdtr_types[11] = <char>NPY_DOUBLE
 * ufunc_bdtr_ptr[2*0] = <void*>_func_bdtr
 * ufunc_bdtr_ptr[2*0+1] = <void*>(<char*>"bdtr")             # <<<<<<<<<<<<<<
 * ufunc_bdtr_ptr[2*1] = <void*>_func_bdtr_unsafe
 * ufunc_bdtr_ptr[2*1+1] = <void*>(<char*>"bdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_ptr[1]) = ((void *)((char *)((char *)"bdtr")));

  /* "scipy/special/_ufuncs.pyx":2930
 * ufunc_bdtr_ptr[2*0] = <void*>_func_bdtr
 * ufunc_bdtr_ptr[2*0+1] = <void*>(<char*>"bdtr")
 * ufunc_bdtr_ptr[2*1] = <void*>_func_bdtr_unsafe             # <<<<<<<<<<<<<<
 * ufunc_bdtr_ptr[2*1+1] = <void*>(<char*>"bdtr")
 * ufunc_bdtr_ptr[2*2] = <void*>_func_bdtr_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_bdtr_unsafe);

  /* "scipy/special/_ufuncs.pyx":2931
 * ufunc_bdtr_ptr[2*0+1] = <void*>(<char*>"bdtr")
 * ufunc_bdtr_ptr[2*1] = <void*>_func_bdtr_unsafe
 * ufunc_bdtr_ptr[2*1+1] = <void*>(<char*>"bdtr")             # <<<<<<<<<<<<<<
 * ufunc_bdtr_ptr[2*2] = <void*>_func_bdtr_unsafe
 * ufunc_bdtr_ptr[2*2+1] = <void*>(<char*>"bdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_ptr[3]) = ((void *)((char *)((char *)"bdtr")));

  /* "scipy/special/_ufuncs.pyx":2932
 * ufunc_bdtr_ptr[2*1] = <void*>_func_bdtr_unsafe
 * ufunc_bdtr_ptr[2*1+1] = <void*>(<char*>"bdtr")
 * ufunc_bdtr_ptr[2*2] = <void*>_func_bdtr_unsafe             # <<<<<<<<<<<<<<
 * ufunc_bdtr_ptr[2*2+1] = <void*>(<char*>"bdtr")
 * ufunc_bdtr_data[0] = &ufunc_bdtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_bdtr_unsafe);

  /* "scipy/special/_ufuncs.pyx":2933
 * ufunc_bdtr_ptr[2*1+1] = <void*>(<char*>"bdtr")
 * ufunc_bdtr_ptr[2*2] = <void*>_func_bdtr_unsafe
 * ufunc_bdtr_ptr[2*2+1] = <void*>(<char*>"bdtr")             # <<<<<<<<<<<<<<
 * ufunc_bdtr_data[0] = &ufunc_bdtr_ptr[2*0]
 * ufunc_bdtr_data[1] = &ufunc_bdtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_ptr[5]) = ((void *)((char *)((char *)"bdtr")));

  /* "scipy/special/_ufuncs.pyx":2934
 * ufunc_bdtr_ptr[2*2] = <void*>_func_bdtr_unsafe
 * ufunc_bdtr_ptr[2*2+1] = <void*>(<char*>"bdtr")
 * ufunc_bdtr_data[0] = &ufunc_bdtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_bdtr_data[1] = &ufunc_bdtr_ptr[2*1]
 * ufunc_bdtr_data[2] = &ufunc_bdtr_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":2935
 * ufunc_bdtr_ptr[2*2+1] = <void*>(<char*>"bdtr")
 * ufunc_bdtr_data[0] = &ufunc_bdtr_ptr[2*0]
 * ufunc_bdtr_data[1] = &ufunc_bdtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_bdtr_data[2] = &ufunc_bdtr_ptr[2*2]
 * bdtr = np.PyUFunc_FromFuncAndData(ufunc_bdtr_loops, ufunc_bdtr_data, ufunc_bdtr_types, 3, 3, 1, 0, "bdtr", ufunc_bdtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":2936
 * ufunc_bdtr_data[0] = &ufunc_bdtr_ptr[2*0]
 * ufunc_bdtr_data[1] = &ufunc_bdtr_ptr[2*1]
 * ufunc_bdtr_data[2] = &ufunc_bdtr_ptr[2*2]             # <<<<<<<<<<<<<<
 * bdtr = np.PyUFunc_FromFuncAndData(ufunc_bdtr_loops, ufunc_bdtr_data, ufunc_bdtr_types, 3, 3, 1, 0, "bdtr", ufunc_bdtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":2937
 * ufunc_bdtr_data[1] = &ufunc_bdtr_ptr[2*1]
 * ufunc_bdtr_data[2] = &ufunc_bdtr_ptr[2*2]
 * bdtr = np.PyUFunc_FromFuncAndData(ufunc_bdtr_loops, ufunc_bdtr_data, ufunc_bdtr_types, 3, 3, 1, 0, "bdtr", ufunc_bdtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_bdtrc_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_types, 3, 3, 1, 0, ((char *)"bdtr"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtr_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 2937, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_bdtr, __pyx_t_2) < 0) __PYX_ERR(1, 2937, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":2943
 * cdef void *ufunc_bdtrc_data[3]
 * cdef char ufunc_bdtrc_types[12]
 * cdef char *ufunc_bdtrc_doc = (             # <<<<<<<<<<<<<<
 *     "bdtrc(k, n, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_doc = ((char *)"bdtrc(k, n, p)\n\nBinomial distribution survival function.\n\nSum of the terms `k + 1` through `n` of the binomial probability density,\n\n.. math::\n    \\mathrm{bdtrc}(k, n, p) = \\sum_{j=k+1}^n {{n}\\choose{j}} p^j (1-p)^{n-j}\n\nParameters\n----------\nk : array_like\n    Number of successes (int).\nn : array_like\n    Number of events (int)\np : array_like\n    Probability of success in a single event.\n\nReturns\n-------\ny : ndarray\n    Probability of `k + 1` or more successes in `n` independent events\n    with success probabilities of `p`.\n\nSee also\n--------\nbdtr\nbetainc\n\nNotes\n-----\nThe terms are not summed directly; instead the regularized incomplete beta\nfunction is employed, according to the formula,\n\n.. math::\n    \\mathrm{bdtrc}(k, n, p) = I_{p}(k + 1, n - k).\n\nWrapper for the Cephes [1]_ routine `bdtrc`.\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":2987
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_bdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_iid__As_lld_d);

  /* "scipy/special/_ufuncs.pyx":2988
 *     "       http://www.netlib.org/cephes/")
 * ufunc_bdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_bdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtrc_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":2989
 * ufunc_bdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_bdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[0] = <char>NPY_LONG
 * ufunc_bdtrc_types[1] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":2990
 * ufunc_bdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtrc_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[1] = <char>NPY_LONG
 * ufunc_bdtrc_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2991
 * ufunc_bdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtrc_types[0] = <char>NPY_LONG
 * ufunc_bdtrc_types[1] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[2] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[1]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":2992
 * ufunc_bdtrc_types[0] = <char>NPY_LONG
 * ufunc_bdtrc_types[1] = <char>NPY_LONG
 * ufunc_bdtrc_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[3] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2993
 * ufunc_bdtrc_types[1] = <char>NPY_LONG
 * ufunc_bdtrc_types[2] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[4] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2994
 * ufunc_bdtrc_types[2] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[3] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[5] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2995
 * ufunc_bdtrc_types[3] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[4] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[6] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2996
 * ufunc_bdtrc_types[4] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[5] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[7] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2997
 * ufunc_bdtrc_types[5] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[6] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[8] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":2998
 * ufunc_bdtrc_types[6] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[7] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[9] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":2999
 * ufunc_bdtrc_types[7] = <char>NPY_FLOAT
 * ufunc_bdtrc_types[8] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[10] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3000
 * ufunc_bdtrc_types[8] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[9] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_types[11] = <char>NPY_DOUBLE
 * ufunc_bdtrc_ptr[2*0] = <void*>_func_bdtrc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3001
 * ufunc_bdtrc_types[9] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[10] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_ptr[2*0] = <void*>_func_bdtrc
 * ufunc_bdtrc_ptr[2*0+1] = <void*>(<char*>"bdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3002
 * ufunc_bdtrc_types[10] = <char>NPY_DOUBLE
 * ufunc_bdtrc_types[11] = <char>NPY_DOUBLE
 * ufunc_bdtrc_ptr[2*0] = <void*>_func_bdtrc             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_ptr[2*0+1] = <void*>(<char*>"bdtrc")
 * ufunc_bdtrc_ptr[2*1] = <void*>_func_bdtrc_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_ptr[0]) = ((void *)bdtrc);

  /* "scipy/special/_ufuncs.pyx":3003
 * ufunc_bdtrc_types[11] = <char>NPY_DOUBLE
 * ufunc_bdtrc_ptr[2*0] = <void*>_func_bdtrc
 * ufunc_bdtrc_ptr[2*0+1] = <void*>(<char*>"bdtrc")             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_ptr[2*1] = <void*>_func_bdtrc_unsafe
 * ufunc_bdtrc_ptr[2*1+1] = <void*>(<char*>"bdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_ptr[1]) = ((void *)((char *)((char *)"bdtrc")));

  /* "scipy/special/_ufuncs.pyx":3004
 * ufunc_bdtrc_ptr[2*0] = <void*>_func_bdtrc
 * ufunc_bdtrc_ptr[2*0+1] = <void*>(<char*>"bdtrc")
 * ufunc_bdtrc_ptr[2*1] = <void*>_func_bdtrc_unsafe             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_ptr[2*1+1] = <void*>(<char*>"bdtrc")
 * ufunc_bdtrc_ptr[2*2] = <void*>_func_bdtrc_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_bdtrc_unsafe);

  /* "scipy/special/_ufuncs.pyx":3005
 * ufunc_bdtrc_ptr[2*0+1] = <void*>(<char*>"bdtrc")
 * ufunc_bdtrc_ptr[2*1] = <void*>_func_bdtrc_unsafe
 * ufunc_bdtrc_ptr[2*1+1] = <void*>(<char*>"bdtrc")             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_ptr[2*2] = <void*>_func_bdtrc_unsafe
 * ufunc_bdtrc_ptr[2*2+1] = <void*>(<char*>"bdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_ptr[3]) = ((void *)((char *)((char *)"bdtrc")));

  /* "scipy/special/_ufuncs.pyx":3006
 * ufunc_bdtrc_ptr[2*1] = <void*>_func_bdtrc_unsafe
 * ufunc_bdtrc_ptr[2*1+1] = <void*>(<char*>"bdtrc")
 * ufunc_bdtrc_ptr[2*2] = <void*>_func_bdtrc_unsafe             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_ptr[2*2+1] = <void*>(<char*>"bdtrc")
 * ufunc_bdtrc_data[0] = &ufunc_bdtrc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_bdtrc_unsafe);

  /* "scipy/special/_ufuncs.pyx":3007
 * ufunc_bdtrc_ptr[2*1+1] = <void*>(<char*>"bdtrc")
 * ufunc_bdtrc_ptr[2*2] = <void*>_func_bdtrc_unsafe
 * ufunc_bdtrc_ptr[2*2+1] = <void*>(<char*>"bdtrc")             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_data[0] = &ufunc_bdtrc_ptr[2*0]
 * ufunc_bdtrc_data[1] = &ufunc_bdtrc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_ptr[5]) = ((void *)((char *)((char *)"bdtrc")));

  /* "scipy/special/_ufuncs.pyx":3008
 * ufunc_bdtrc_ptr[2*2] = <void*>_func_bdtrc_unsafe
 * ufunc_bdtrc_ptr[2*2+1] = <void*>(<char*>"bdtrc")
 * ufunc_bdtrc_data[0] = &ufunc_bdtrc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_data[1] = &ufunc_bdtrc_ptr[2*1]
 * ufunc_bdtrc_data[2] = &ufunc_bdtrc_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3009
 * ufunc_bdtrc_ptr[2*2+1] = <void*>(<char*>"bdtrc")
 * ufunc_bdtrc_data[0] = &ufunc_bdtrc_ptr[2*0]
 * ufunc_bdtrc_data[1] = &ufunc_bdtrc_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_bdtrc_data[2] = &ufunc_bdtrc_ptr[2*2]
 * bdtrc = np.PyUFunc_FromFuncAndData(ufunc_bdtrc_loops, ufunc_bdtrc_data, ufunc_bdtrc_types, 3, 3, 1, 0, "bdtrc", ufunc_bdtrc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3010
 * ufunc_bdtrc_data[0] = &ufunc_bdtrc_ptr[2*0]
 * ufunc_bdtrc_data[1] = &ufunc_bdtrc_ptr[2*1]
 * ufunc_bdtrc_data[2] = &ufunc_bdtrc_ptr[2*2]             # <<<<<<<<<<<<<<
 * bdtrc = np.PyUFunc_FromFuncAndData(ufunc_bdtrc_loops, ufunc_bdtrc_data, ufunc_bdtrc_types, 3, 3, 1, 0, "bdtrc", ufunc_bdtrc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":3011
 * ufunc_bdtrc_data[1] = &ufunc_bdtrc_ptr[2*1]
 * ufunc_bdtrc_data[2] = &ufunc_bdtrc_ptr[2*2]
 * bdtrc = np.PyUFunc_FromFuncAndData(ufunc_bdtrc_loops, ufunc_bdtrc_data, ufunc_bdtrc_types, 3, 3, 1, 0, "bdtrc", ufunc_bdtrc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_bdtri_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_types, 3, 3, 1, 0, ((char *)"bdtrc"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrc_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3011, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_bdtrc, __pyx_t_2) < 0) __PYX_ERR(1, 3011, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3017
 * cdef void *ufunc_bdtri_data[3]
 * cdef char ufunc_bdtri_types[12]
 * cdef char *ufunc_bdtri_doc = (             # <<<<<<<<<<<<<<
 *     "bdtri(k, n, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_doc = ((char *)"bdtri(k, n, y)\n\nInverse function to `bdtr` with respect to `p`.\n\nFinds the event probability `p` such that the sum of the terms 0 through\n`k` of the binomial probability density is equal to the given cumulative\nprobability `y`.\n\nParameters\n----------\nk : array_like\n    Number of successes (float).\nn : array_like\n    Number of events (float)\ny : array_like\n    Cumulative probability (probability of `k` or fewer successes in `n`\n    events).\n\nReturns\n-------\np : ndarray\n    The event probability such that `bdtr(k, n, p) = y`.\n\nSee also\n--------\nbdtr\nbetaincinv\n\nNotes\n-----\nThe computation is carried out using the inverse beta integral function\nand the relation,::\n\n    1 - p = betaincinv(n - k, k + 1, y).\n\nWrapper for the Cephes [1]_ routine `bdtri`.\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":3059
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_bdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d             # <<<<<<<<<<<<<<
 * ufunc_bdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_iid__As_lld_d);

  /* "scipy/special/_ufuncs.pyx":3060
 *     "       http://www.netlib.org/cephes/")
 * ufunc_bdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_bdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_bdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtri_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":3061
 * ufunc_bdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_bdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[0] = <char>NPY_LONG
 * ufunc_bdtri_types[1] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":3062
 * ufunc_bdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtri_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[1] = <char>NPY_LONG
 * ufunc_bdtri_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":3063
 * ufunc_bdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtri_types[0] = <char>NPY_LONG
 * ufunc_bdtri_types[1] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[2] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[1]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":3064
 * ufunc_bdtri_types[0] = <char>NPY_LONG
 * ufunc_bdtri_types[1] = <char>NPY_LONG
 * ufunc_bdtri_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[3] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3065
 * ufunc_bdtri_types[1] = <char>NPY_LONG
 * ufunc_bdtri_types[2] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[4] = <char>NPY_FLOAT
 * ufunc_bdtri_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3066
 * ufunc_bdtri_types[2] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[3] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[5] = <char>NPY_FLOAT
 * ufunc_bdtri_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3067
 * ufunc_bdtri_types[3] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[4] = <char>NPY_FLOAT
 * ufunc_bdtri_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[6] = <char>NPY_FLOAT
 * ufunc_bdtri_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3068
 * ufunc_bdtri_types[4] = <char>NPY_FLOAT
 * ufunc_bdtri_types[5] = <char>NPY_FLOAT
 * ufunc_bdtri_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[7] = <char>NPY_FLOAT
 * ufunc_bdtri_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3069
 * ufunc_bdtri_types[5] = <char>NPY_FLOAT
 * ufunc_bdtri_types[6] = <char>NPY_FLOAT
 * ufunc_bdtri_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3070
 * ufunc_bdtri_types[6] = <char>NPY_FLOAT
 * ufunc_bdtri_types[7] = <char>NPY_FLOAT
 * ufunc_bdtri_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[9] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3071
 * ufunc_bdtri_types[7] = <char>NPY_FLOAT
 * ufunc_bdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[10] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3072
 * ufunc_bdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[9] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtri_types[11] = <char>NPY_DOUBLE
 * ufunc_bdtri_ptr[2*0] = <void*>_func_bdtri
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3073
 * ufunc_bdtri_types[9] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[10] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtri_ptr[2*0] = <void*>_func_bdtri
 * ufunc_bdtri_ptr[2*0+1] = <void*>(<char*>"bdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3074
 * ufunc_bdtri_types[10] = <char>NPY_DOUBLE
 * ufunc_bdtri_types[11] = <char>NPY_DOUBLE
 * ufunc_bdtri_ptr[2*0] = <void*>_func_bdtri             # <<<<<<<<<<<<<<
 * ufunc_bdtri_ptr[2*0+1] = <void*>(<char*>"bdtri")
 * ufunc_bdtri_ptr[2*1] = <void*>_func_bdtri_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_ptr[0]) = ((void *)bdtri);

  /* "scipy/special/_ufuncs.pyx":3075
 * ufunc_bdtri_types[11] = <char>NPY_DOUBLE
 * ufunc_bdtri_ptr[2*0] = <void*>_func_bdtri
 * ufunc_bdtri_ptr[2*0+1] = <void*>(<char*>"bdtri")             # <<<<<<<<<<<<<<
 * ufunc_bdtri_ptr[2*1] = <void*>_func_bdtri_unsafe
 * ufunc_bdtri_ptr[2*1+1] = <void*>(<char*>"bdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_ptr[1]) = ((void *)((char *)((char *)"bdtri")));

  /* "scipy/special/_ufuncs.pyx":3076
 * ufunc_bdtri_ptr[2*0] = <void*>_func_bdtri
 * ufunc_bdtri_ptr[2*0+1] = <void*>(<char*>"bdtri")
 * ufunc_bdtri_ptr[2*1] = <void*>_func_bdtri_unsafe             # <<<<<<<<<<<<<<
 * ufunc_bdtri_ptr[2*1+1] = <void*>(<char*>"bdtri")
 * ufunc_bdtri_ptr[2*2] = <void*>_func_bdtri_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_bdtri_unsafe);

  /* "scipy/special/_ufuncs.pyx":3077
 * ufunc_bdtri_ptr[2*0+1] = <void*>(<char*>"bdtri")
 * ufunc_bdtri_ptr[2*1] = <void*>_func_bdtri_unsafe
 * ufunc_bdtri_ptr[2*1+1] = <void*>(<char*>"bdtri")             # <<<<<<<<<<<<<<
 * ufunc_bdtri_ptr[2*2] = <void*>_func_bdtri_unsafe
 * ufunc_bdtri_ptr[2*2+1] = <void*>(<char*>"bdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_ptr[3]) = ((void *)((char *)((char *)"bdtri")));

  /* "scipy/special/_ufuncs.pyx":3078
 * ufunc_bdtri_ptr[2*1] = <void*>_func_bdtri_unsafe
 * ufunc_bdtri_ptr[2*1+1] = <void*>(<char*>"bdtri")
 * ufunc_bdtri_ptr[2*2] = <void*>_func_bdtri_unsafe             # <<<<<<<<<<<<<<
 * ufunc_bdtri_ptr[2*2+1] = <void*>(<char*>"bdtri")
 * ufunc_bdtri_data[0] = &ufunc_bdtri_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_bdtri_unsafe);

  /* "scipy/special/_ufuncs.pyx":3079
 * ufunc_bdtri_ptr[2*1+1] = <void*>(<char*>"bdtri")
 * ufunc_bdtri_ptr[2*2] = <void*>_func_bdtri_unsafe
 * ufunc_bdtri_ptr[2*2+1] = <void*>(<char*>"bdtri")             # <<<<<<<<<<<<<<
 * ufunc_bdtri_data[0] = &ufunc_bdtri_ptr[2*0]
 * ufunc_bdtri_data[1] = &ufunc_bdtri_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_ptr[5]) = ((void *)((char *)((char *)"bdtri")));

  /* "scipy/special/_ufuncs.pyx":3080
 * ufunc_bdtri_ptr[2*2] = <void*>_func_bdtri_unsafe
 * ufunc_bdtri_ptr[2*2+1] = <void*>(<char*>"bdtri")
 * ufunc_bdtri_data[0] = &ufunc_bdtri_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_bdtri_data[1] = &ufunc_bdtri_ptr[2*1]
 * ufunc_bdtri_data[2] = &ufunc_bdtri_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3081
 * ufunc_bdtri_ptr[2*2+1] = <void*>(<char*>"bdtri")
 * ufunc_bdtri_data[0] = &ufunc_bdtri_ptr[2*0]
 * ufunc_bdtri_data[1] = &ufunc_bdtri_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_bdtri_data[2] = &ufunc_bdtri_ptr[2*2]
 * bdtri = np.PyUFunc_FromFuncAndData(ufunc_bdtri_loops, ufunc_bdtri_data, ufunc_bdtri_types, 3, 3, 1, 0, "bdtri", ufunc_bdtri_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3082
 * ufunc_bdtri_data[0] = &ufunc_bdtri_ptr[2*0]
 * ufunc_bdtri_data[1] = &ufunc_bdtri_ptr[2*1]
 * ufunc_bdtri_data[2] = &ufunc_bdtri_ptr[2*2]             # <<<<<<<<<<<<<<
 * bdtri = np.PyUFunc_FromFuncAndData(ufunc_bdtri_loops, ufunc_bdtri_data, ufunc_bdtri_types, 3, 3, 1, 0, "bdtri", ufunc_bdtri_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":3083
 * ufunc_bdtri_data[1] = &ufunc_bdtri_ptr[2*1]
 * ufunc_bdtri_data[2] = &ufunc_bdtri_ptr[2*2]
 * bdtri = np.PyUFunc_FromFuncAndData(ufunc_bdtri_loops, ufunc_bdtri_data, ufunc_bdtri_types, 3, 3, 1, 0, "bdtri", ufunc_bdtri_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_bdtrik_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_types, 3, 3, 1, 0, ((char *)"bdtri"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtri_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3083, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_bdtri, __pyx_t_2) < 0) __PYX_ERR(1, 3083, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3089
 * cdef void *ufunc_bdtrik_data[2]
 * cdef char ufunc_bdtrik_types[8]
 * cdef char *ufunc_bdtrik_doc = (             # <<<<<<<<<<<<<<
 *     "bdtrik(y, n, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_doc = ((char *)"bdtrik(y, n, p)\n\nInverse function to `bdtr` with respect to `k`.\n\nFinds the number of successes `k` such that the sum of the terms 0 through\n`k` of the Binomial probability density for `n` events with probability\n`p` is equal to the given cumulative probability `y`.\n\nParameters\n----------\ny : array_like\n    Cumulative probability (probability of `k` or fewer successes in `n`\n    events).\nn : array_like\n    Number of events (float).\np : array_like\n    Success probability (float).\n\nReturns\n-------\nk : ndarray\n    The number of successes `k` such that `bdtr(k, n, p) = y`.\n\nSee also\n--------\nbdtr\n\nNotes\n-----\nFormula 26.5.24 of [1]_ is used to reduce the binomial distribution to the\ncumulative incomplete beta distribution.\n\nComputation of `k` involves a search for a value that produces the desired\nvalue of `y`.  The search relies on the monotonicity of `y` with `k`.\n\nWrapper for the CDFLIB [2]_ Fortran routine `cdfbin`.\n\nReferences\n----------\n.. [1] Milton Abramowitz and Irene A. Stegun, eds.\n       Handbook of Mathematical Functions with Formulas,\n       Graphs, and Mathematical Tables. New York: Dover, 1972.\n.. [2] Barry Brown, James Lovato, and Kathy Russell,\n       CDFLIB: Library of Fortran Routines for Cumulative Distribution\n       Functions, Inverses, and Other Parameters.");

  /* "scipy/special/_ufuncs.pyx":3135
 *     "       CDFLIB: Library of Fortran Routines for Cumulative Distribution\n"
 *     "       Functions, Inverses, and Other Parameters.")
 * ufunc_bdtrik_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtrik_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":3136
 *     "       Functions, Inverses, and Other Parameters.")
 * ufunc_bdtrik_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_types[0] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":3137
 * ufunc_bdtrik_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtrik_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_types[1] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3138
 * ufunc_bdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtrik_types[0] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_types[2] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3139
 * ufunc_bdtrik_types[0] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[1] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_types[3] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3140
 * ufunc_bdtrik_types[1] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[2] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_types[4] = <char>NPY_DOUBLE
 * ufunc_bdtrik_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3141
 * ufunc_bdtrik_types[2] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[3] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_types[5] = <char>NPY_DOUBLE
 * ufunc_bdtrik_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3142
 * ufunc_bdtrik_types[3] = <char>NPY_FLOAT
 * ufunc_bdtrik_types[4] = <char>NPY_DOUBLE
 * ufunc_bdtrik_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_types[6] = <char>NPY_DOUBLE
 * ufunc_bdtrik_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3143
 * ufunc_bdtrik_types[4] = <char>NPY_DOUBLE
 * ufunc_bdtrik_types[5] = <char>NPY_DOUBLE
 * ufunc_bdtrik_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_types[7] = <char>NPY_DOUBLE
 * ufunc_bdtrik_ptr[2*0] = <void*>_func_cdfbin2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3144
 * ufunc_bdtrik_types[5] = <char>NPY_DOUBLE
 * ufunc_bdtrik_types[6] = <char>NPY_DOUBLE
 * ufunc_bdtrik_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_ptr[2*0] = <void*>_func_cdfbin2_wrap
 * ufunc_bdtrik_ptr[2*0+1] = <void*>(<char*>"bdtrik")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3145
 * ufunc_bdtrik_types[6] = <char>NPY_DOUBLE
 * ufunc_bdtrik_types[7] = <char>NPY_DOUBLE
 * ufunc_bdtrik_ptr[2*0] = <void*>_func_cdfbin2_wrap             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_ptr[2*0+1] = <void*>(<char*>"bdtrik")
 * ufunc_bdtrik_ptr[2*1] = <void*>_func_cdfbin2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_ptr[0]) = ((void *)cdfbin2_wrap);

  /* "scipy/special/_ufuncs.pyx":3146
 * ufunc_bdtrik_types[7] = <char>NPY_DOUBLE
 * ufunc_bdtrik_ptr[2*0] = <void*>_func_cdfbin2_wrap
 * ufunc_bdtrik_ptr[2*0+1] = <void*>(<char*>"bdtrik")             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_ptr[2*1] = <void*>_func_cdfbin2_wrap
 * ufunc_bdtrik_ptr[2*1+1] = <void*>(<char*>"bdtrik")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_ptr[1]) = ((void *)((char *)((char *)"bdtrik")));

  /* "scipy/special/_ufuncs.pyx":3147
 * ufunc_bdtrik_ptr[2*0] = <void*>_func_cdfbin2_wrap
 * ufunc_bdtrik_ptr[2*0+1] = <void*>(<char*>"bdtrik")
 * ufunc_bdtrik_ptr[2*1] = <void*>_func_cdfbin2_wrap             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_ptr[2*1+1] = <void*>(<char*>"bdtrik")
 * ufunc_bdtrik_data[0] = &ufunc_bdtrik_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_ptr[2]) = ((void *)cdfbin2_wrap);

  /* "scipy/special/_ufuncs.pyx":3148
 * ufunc_bdtrik_ptr[2*0+1] = <void*>(<char*>"bdtrik")
 * ufunc_bdtrik_ptr[2*1] = <void*>_func_cdfbin2_wrap
 * ufunc_bdtrik_ptr[2*1+1] = <void*>(<char*>"bdtrik")             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_data[0] = &ufunc_bdtrik_ptr[2*0]
 * ufunc_bdtrik_data[1] = &ufunc_bdtrik_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_ptr[3]) = ((void *)((char *)((char *)"bdtrik")));

  /* "scipy/special/_ufuncs.pyx":3149
 * ufunc_bdtrik_ptr[2*1] = <void*>_func_cdfbin2_wrap
 * ufunc_bdtrik_ptr[2*1+1] = <void*>(<char*>"bdtrik")
 * ufunc_bdtrik_data[0] = &ufunc_bdtrik_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_bdtrik_data[1] = &ufunc_bdtrik_ptr[2*1]
 * bdtrik = np.PyUFunc_FromFuncAndData(ufunc_bdtrik_loops, ufunc_bdtrik_data, ufunc_bdtrik_types, 2, 3, 1, 0, "bdtrik", ufunc_bdtrik_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3150
 * ufunc_bdtrik_ptr[2*1+1] = <void*>(<char*>"bdtrik")
 * ufunc_bdtrik_data[0] = &ufunc_bdtrik_ptr[2*0]
 * ufunc_bdtrik_data[1] = &ufunc_bdtrik_ptr[2*1]             # <<<<<<<<<<<<<<
 * bdtrik = np.PyUFunc_FromFuncAndData(ufunc_bdtrik_loops, ufunc_bdtrik_data, ufunc_bdtrik_types, 2, 3, 1, 0, "bdtrik", ufunc_bdtrik_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3151
 * ufunc_bdtrik_data[0] = &ufunc_bdtrik_ptr[2*0]
 * ufunc_bdtrik_data[1] = &ufunc_bdtrik_ptr[2*1]
 * bdtrik = np.PyUFunc_FromFuncAndData(ufunc_bdtrik_loops, ufunc_bdtrik_data, ufunc_bdtrik_types, 2, 3, 1, 0, "bdtrik", ufunc_bdtrik_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_bdtrin_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_types, 2, 3, 1, 0, ((char *)"bdtrik"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrik_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_bdtrik, __pyx_t_2) < 0) __PYX_ERR(1, 3151, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3157
 * cdef void *ufunc_bdtrin_data[2]
 * cdef char ufunc_bdtrin_types[8]
 * cdef char *ufunc_bdtrin_doc = (             # <<<<<<<<<<<<<<
 *     "bdtrin(k, y, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_doc = ((char *)"bdtrin(k, y, p)\n\nInverse function to `bdtr` with respect to `n`.\n\nFinds the number of events `n` such that the sum of the terms 0 through\n`k` of the Binomial probability density for events with probability `p` is\nequal to the given cumulative probability `y`.\n\nParameters\n----------\nk : array_like\n    Number of successes (float).\ny : array_like\n    Cumulative probability (probability of `k` or fewer successes in `n`\n    events).\np : array_like\n    Success probability (float).\n\nReturns\n-------\nn : ndarray\n    The number of events `n` such that `bdtr(k, n, p) = y`.\n\nSee also\n--------\nbdtr\n\nNotes\n-----\nFormula 26.5.24 of [1]_ is used to reduce the binomial distribution to the\ncumulative incomplete beta distribution.\n\nComputation of `n` involves a search for a value that produces the desired\nvalue of `y`.  The search relies on the monotonicity of `y` with `n`.\n\nWrapper for the CDFLIB [2]_ Fortran routine `cdfbin`.\n\nReferences\n----------\n.. [1] Milton Abramowitz and Irene A. Stegun, eds.\n       Handbook of Mathematical Functions with Formulas,\n       Graphs, and Mathematical Tables. New York: Dover, 1972.\n.. [2] Barry Brown, James Lovato, and Kathy Russell,\n       CDFLIB: Library of Fortran Routines for Cumulative Distribution\n       Functions, Inverses, and Other Parameters.");

  /* "scipy/special/_ufuncs.pyx":3203
 *     "       CDFLIB: Library of Fortran Routines for Cumulative Distribution\n"
 *     "       Functions, Inverses, and Other Parameters.")
 * ufunc_bdtrin_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtrin_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":3204
 *     "       Functions, Inverses, and Other Parameters.")
 * ufunc_bdtrin_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtrin_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_types[0] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":3205
 * ufunc_bdtrin_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_bdtrin_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtrin_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_types[1] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3206
 * ufunc_bdtrin_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_bdtrin_types[0] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_types[2] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3207
 * ufunc_bdtrin_types[0] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[1] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_types[3] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3208
 * ufunc_bdtrin_types[1] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[2] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_types[4] = <char>NPY_DOUBLE
 * ufunc_bdtrin_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3209
 * ufunc_bdtrin_types[2] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[3] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_types[5] = <char>NPY_DOUBLE
 * ufunc_bdtrin_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3210
 * ufunc_bdtrin_types[3] = <char>NPY_FLOAT
 * ufunc_bdtrin_types[4] = <char>NPY_DOUBLE
 * ufunc_bdtrin_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_types[6] = <char>NPY_DOUBLE
 * ufunc_bdtrin_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3211
 * ufunc_bdtrin_types[4] = <char>NPY_DOUBLE
 * ufunc_bdtrin_types[5] = <char>NPY_DOUBLE
 * ufunc_bdtrin_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_types[7] = <char>NPY_DOUBLE
 * ufunc_bdtrin_ptr[2*0] = <void*>_func_cdfbin3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3212
 * ufunc_bdtrin_types[5] = <char>NPY_DOUBLE
 * ufunc_bdtrin_types[6] = <char>NPY_DOUBLE
 * ufunc_bdtrin_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_ptr[2*0] = <void*>_func_cdfbin3_wrap
 * ufunc_bdtrin_ptr[2*0+1] = <void*>(<char*>"bdtrin")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3213
 * ufunc_bdtrin_types[6] = <char>NPY_DOUBLE
 * ufunc_bdtrin_types[7] = <char>NPY_DOUBLE
 * ufunc_bdtrin_ptr[2*0] = <void*>_func_cdfbin3_wrap             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_ptr[2*0+1] = <void*>(<char*>"bdtrin")
 * ufunc_bdtrin_ptr[2*1] = <void*>_func_cdfbin3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_ptr[0]) = ((void *)cdfbin3_wrap);

  /* "scipy/special/_ufuncs.pyx":3214
 * ufunc_bdtrin_types[7] = <char>NPY_DOUBLE
 * ufunc_bdtrin_ptr[2*0] = <void*>_func_cdfbin3_wrap
 * ufunc_bdtrin_ptr[2*0+1] = <void*>(<char*>"bdtrin")             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_ptr[2*1] = <void*>_func_cdfbin3_wrap
 * ufunc_bdtrin_ptr[2*1+1] = <void*>(<char*>"bdtrin")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_ptr[1]) = ((void *)((char *)((char *)"bdtrin")));

  /* "scipy/special/_ufuncs.pyx":3215
 * ufunc_bdtrin_ptr[2*0] = <void*>_func_cdfbin3_wrap
 * ufunc_bdtrin_ptr[2*0+1] = <void*>(<char*>"bdtrin")
 * ufunc_bdtrin_ptr[2*1] = <void*>_func_cdfbin3_wrap             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_ptr[2*1+1] = <void*>(<char*>"bdtrin")
 * ufunc_bdtrin_data[0] = &ufunc_bdtrin_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_ptr[2]) = ((void *)cdfbin3_wrap);

  /* "scipy/special/_ufuncs.pyx":3216
 * ufunc_bdtrin_ptr[2*0+1] = <void*>(<char*>"bdtrin")
 * ufunc_bdtrin_ptr[2*1] = <void*>_func_cdfbin3_wrap
 * ufunc_bdtrin_ptr[2*1+1] = <void*>(<char*>"bdtrin")             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_data[0] = &ufunc_bdtrin_ptr[2*0]
 * ufunc_bdtrin_data[1] = &ufunc_bdtrin_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_ptr[3]) = ((void *)((char *)((char *)"bdtrin")));

  /* "scipy/special/_ufuncs.pyx":3217
 * ufunc_bdtrin_ptr[2*1] = <void*>_func_cdfbin3_wrap
 * ufunc_bdtrin_ptr[2*1+1] = <void*>(<char*>"bdtrin")
 * ufunc_bdtrin_data[0] = &ufunc_bdtrin_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_bdtrin_data[1] = &ufunc_bdtrin_ptr[2*1]
 * bdtrin = np.PyUFunc_FromFuncAndData(ufunc_bdtrin_loops, ufunc_bdtrin_data, ufunc_bdtrin_types, 2, 3, 1, 0, "bdtrin", ufunc_bdtrin_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3218
 * ufunc_bdtrin_ptr[2*1+1] = <void*>(<char*>"bdtrin")
 * ufunc_bdtrin_data[0] = &ufunc_bdtrin_ptr[2*0]
 * ufunc_bdtrin_data[1] = &ufunc_bdtrin_ptr[2*1]             # <<<<<<<<<<<<<<
 * bdtrin = np.PyUFunc_FromFuncAndData(ufunc_bdtrin_loops, ufunc_bdtrin_data, ufunc_bdtrin_types, 2, 3, 1, 0, "bdtrin", ufunc_bdtrin_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3219
 * ufunc_bdtrin_data[0] = &ufunc_bdtrin_ptr[2*0]
 * ufunc_bdtrin_data[1] = &ufunc_bdtrin_ptr[2*1]
 * bdtrin = np.PyUFunc_FromFuncAndData(ufunc_bdtrin_loops, ufunc_bdtrin_data, ufunc_bdtrin_types, 2, 3, 1, 0, "bdtrin", ufunc_bdtrin_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_bei_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_types, 2, 3, 1, 0, ((char *)"bdtrin"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_bdtrin_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_bdtrin, __pyx_t_2) < 0) __PYX_ERR(1, 3219, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3225
 * cdef void *ufunc_bei_data[2]
 * cdef char ufunc_bei_types[4]
 * cdef char *ufunc_bei_doc = (             # <<<<<<<<<<<<<<
 *     "bei(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_doc = ((char *)"bei(x)\n\nKelvin function bei");

  /* "scipy/special/_ufuncs.pyx":3229
 *     "\n"
 *     "Kelvin function bei")
 * ufunc_bei_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_bei_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_bei_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":3230
 *     "Kelvin function bei")
 * ufunc_bei_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_bei_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_bei_types[0] = <char>NPY_FLOAT
 * ufunc_bei_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":3231
 * ufunc_bei_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_bei_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_bei_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bei_types[1] = <char>NPY_FLOAT
 * ufunc_bei_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3232
 * ufunc_bei_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_bei_types[0] = <char>NPY_FLOAT
 * ufunc_bei_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_bei_types[2] = <char>NPY_DOUBLE
 * ufunc_bei_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3233
 * ufunc_bei_types[0] = <char>NPY_FLOAT
 * ufunc_bei_types[1] = <char>NPY_FLOAT
 * ufunc_bei_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bei_types[3] = <char>NPY_DOUBLE
 * ufunc_bei_ptr[2*0] = <void*>_func_bei_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3234
 * ufunc_bei_types[1] = <char>NPY_FLOAT
 * ufunc_bei_types[2] = <char>NPY_DOUBLE
 * ufunc_bei_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_bei_ptr[2*0] = <void*>_func_bei_wrap
 * ufunc_bei_ptr[2*0+1] = <void*>(<char*>"bei")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3235
 * ufunc_bei_types[2] = <char>NPY_DOUBLE
 * ufunc_bei_types[3] = <char>NPY_DOUBLE
 * ufunc_bei_ptr[2*0] = <void*>_func_bei_wrap             # <<<<<<<<<<<<<<
 * ufunc_bei_ptr[2*0+1] = <void*>(<char*>"bei")
 * ufunc_bei_ptr[2*1] = <void*>_func_bei_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_ptr[0]) = ((void *)bei_wrap);

  /* "scipy/special/_ufuncs.pyx":3236
 * ufunc_bei_types[3] = <char>NPY_DOUBLE
 * ufunc_bei_ptr[2*0] = <void*>_func_bei_wrap
 * ufunc_bei_ptr[2*0+1] = <void*>(<char*>"bei")             # <<<<<<<<<<<<<<
 * ufunc_bei_ptr[2*1] = <void*>_func_bei_wrap
 * ufunc_bei_ptr[2*1+1] = <void*>(<char*>"bei")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_ptr[1]) = ((void *)((char *)((char *)"bei")));

  /* "scipy/special/_ufuncs.pyx":3237
 * ufunc_bei_ptr[2*0] = <void*>_func_bei_wrap
 * ufunc_bei_ptr[2*0+1] = <void*>(<char*>"bei")
 * ufunc_bei_ptr[2*1] = <void*>_func_bei_wrap             # <<<<<<<<<<<<<<
 * ufunc_bei_ptr[2*1+1] = <void*>(<char*>"bei")
 * ufunc_bei_data[0] = &ufunc_bei_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_ptr[2]) = ((void *)bei_wrap);

  /* "scipy/special/_ufuncs.pyx":3238
 * ufunc_bei_ptr[2*0+1] = <void*>(<char*>"bei")
 * ufunc_bei_ptr[2*1] = <void*>_func_bei_wrap
 * ufunc_bei_ptr[2*1+1] = <void*>(<char*>"bei")             # <<<<<<<<<<<<<<
 * ufunc_bei_data[0] = &ufunc_bei_ptr[2*0]
 * ufunc_bei_data[1] = &ufunc_bei_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_ptr[3]) = ((void *)((char *)((char *)"bei")));

  /* "scipy/special/_ufuncs.pyx":3239
 * ufunc_bei_ptr[2*1] = <void*>_func_bei_wrap
 * ufunc_bei_ptr[2*1+1] = <void*>(<char*>"bei")
 * ufunc_bei_data[0] = &ufunc_bei_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_bei_data[1] = &ufunc_bei_ptr[2*1]
 * bei = np.PyUFunc_FromFuncAndData(ufunc_bei_loops, ufunc_bei_data, ufunc_bei_types, 2, 1, 1, 0, "bei", ufunc_bei_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3240
 * ufunc_bei_ptr[2*1+1] = <void*>(<char*>"bei")
 * ufunc_bei_data[0] = &ufunc_bei_ptr[2*0]
 * ufunc_bei_data[1] = &ufunc_bei_ptr[2*1]             # <<<<<<<<<<<<<<
 * bei = np.PyUFunc_FromFuncAndData(ufunc_bei_loops, ufunc_bei_data, ufunc_bei_types, 2, 1, 1, 0, "bei", ufunc_bei_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3241
 * ufunc_bei_data[0] = &ufunc_bei_ptr[2*0]
 * ufunc_bei_data[1] = &ufunc_bei_ptr[2*1]
 * bei = np.PyUFunc_FromFuncAndData(ufunc_bei_loops, ufunc_bei_data, ufunc_bei_types, 2, 1, 1, 0, "bei", ufunc_bei_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_beip_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_types, 2, 1, 1, 0, ((char *)"bei"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_bei_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_bei, __pyx_t_2) < 0) __PYX_ERR(1, 3241, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3247
 * cdef void *ufunc_beip_data[2]
 * cdef char ufunc_beip_types[4]
 * cdef char *ufunc_beip_doc = (             # <<<<<<<<<<<<<<
 *     "beip(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_doc = ((char *)"beip(x)\n\nDerivative of the Kelvin function `bei`");

  /* "scipy/special/_ufuncs.pyx":3251
 *     "\n"
 *     "Derivative of the Kelvin function `bei`")
 * ufunc_beip_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_beip_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_beip_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":3252
 *     "Derivative of the Kelvin function `bei`")
 * ufunc_beip_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_beip_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_beip_types[0] = <char>NPY_FLOAT
 * ufunc_beip_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":3253
 * ufunc_beip_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_beip_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_beip_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_beip_types[1] = <char>NPY_FLOAT
 * ufunc_beip_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3254
 * ufunc_beip_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_beip_types[0] = <char>NPY_FLOAT
 * ufunc_beip_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_beip_types[2] = <char>NPY_DOUBLE
 * ufunc_beip_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3255
 * ufunc_beip_types[0] = <char>NPY_FLOAT
 * ufunc_beip_types[1] = <char>NPY_FLOAT
 * ufunc_beip_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_beip_types[3] = <char>NPY_DOUBLE
 * ufunc_beip_ptr[2*0] = <void*>_func_beip_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3256
 * ufunc_beip_types[1] = <char>NPY_FLOAT
 * ufunc_beip_types[2] = <char>NPY_DOUBLE
 * ufunc_beip_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_beip_ptr[2*0] = <void*>_func_beip_wrap
 * ufunc_beip_ptr[2*0+1] = <void*>(<char*>"beip")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3257
 * ufunc_beip_types[2] = <char>NPY_DOUBLE
 * ufunc_beip_types[3] = <char>NPY_DOUBLE
 * ufunc_beip_ptr[2*0] = <void*>_func_beip_wrap             # <<<<<<<<<<<<<<
 * ufunc_beip_ptr[2*0+1] = <void*>(<char*>"beip")
 * ufunc_beip_ptr[2*1] = <void*>_func_beip_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_ptr[0]) = ((void *)beip_wrap);

  /* "scipy/special/_ufuncs.pyx":3258
 * ufunc_beip_types[3] = <char>NPY_DOUBLE
 * ufunc_beip_ptr[2*0] = <void*>_func_beip_wrap
 * ufunc_beip_ptr[2*0+1] = <void*>(<char*>"beip")             # <<<<<<<<<<<<<<
 * ufunc_beip_ptr[2*1] = <void*>_func_beip_wrap
 * ufunc_beip_ptr[2*1+1] = <void*>(<char*>"beip")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_ptr[1]) = ((void *)((char *)((char *)"beip")));

  /* "scipy/special/_ufuncs.pyx":3259
 * ufunc_beip_ptr[2*0] = <void*>_func_beip_wrap
 * ufunc_beip_ptr[2*0+1] = <void*>(<char*>"beip")
 * ufunc_beip_ptr[2*1] = <void*>_func_beip_wrap             # <<<<<<<<<<<<<<
 * ufunc_beip_ptr[2*1+1] = <void*>(<char*>"beip")
 * ufunc_beip_data[0] = &ufunc_beip_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_ptr[2]) = ((void *)beip_wrap);

  /* "scipy/special/_ufuncs.pyx":3260
 * ufunc_beip_ptr[2*0+1] = <void*>(<char*>"beip")
 * ufunc_beip_ptr[2*1] = <void*>_func_beip_wrap
 * ufunc_beip_ptr[2*1+1] = <void*>(<char*>"beip")             # <<<<<<<<<<<<<<
 * ufunc_beip_data[0] = &ufunc_beip_ptr[2*0]
 * ufunc_beip_data[1] = &ufunc_beip_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_ptr[3]) = ((void *)((char *)((char *)"beip")));

  /* "scipy/special/_ufuncs.pyx":3261
 * ufunc_beip_ptr[2*1] = <void*>_func_beip_wrap
 * ufunc_beip_ptr[2*1+1] = <void*>(<char*>"beip")
 * ufunc_beip_data[0] = &ufunc_beip_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_beip_data[1] = &ufunc_beip_ptr[2*1]
 * beip = np.PyUFunc_FromFuncAndData(ufunc_beip_loops, ufunc_beip_data, ufunc_beip_types, 2, 1, 1, 0, "beip", ufunc_beip_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3262
 * ufunc_beip_ptr[2*1+1] = <void*>(<char*>"beip")
 * ufunc_beip_data[0] = &ufunc_beip_ptr[2*0]
 * ufunc_beip_data[1] = &ufunc_beip_ptr[2*1]             # <<<<<<<<<<<<<<
 * beip = np.PyUFunc_FromFuncAndData(ufunc_beip_loops, ufunc_beip_data, ufunc_beip_types, 2, 1, 1, 0, "beip", ufunc_beip_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3263
 * ufunc_beip_data[0] = &ufunc_beip_ptr[2*0]
 * ufunc_beip_data[1] = &ufunc_beip_ptr[2*1]
 * beip = np.PyUFunc_FromFuncAndData(ufunc_beip_loops, ufunc_beip_data, ufunc_beip_types, 2, 1, 1, 0, "beip", ufunc_beip_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ber_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_types, 2, 1, 1, 0, ((char *)"beip"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_beip_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_beip, __pyx_t_2) < 0) __PYX_ERR(1, 3263, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3269
 * cdef void *ufunc_ber_data[2]
 * cdef char ufunc_ber_types[4]
 * cdef char *ufunc_ber_doc = (             # <<<<<<<<<<<<<<
 *     "ber(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_doc = ((char *)"ber(x)\n\nKelvin function ber.");

  /* "scipy/special/_ufuncs.pyx":3273
 *     "\n"
 *     "Kelvin function ber.")
 * ufunc_ber_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_ber_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ber_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":3274
 *     "Kelvin function ber.")
 * ufunc_ber_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ber_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_ber_types[0] = <char>NPY_FLOAT
 * ufunc_ber_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":3275
 * ufunc_ber_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ber_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ber_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ber_types[1] = <char>NPY_FLOAT
 * ufunc_ber_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3276
 * ufunc_ber_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ber_types[0] = <char>NPY_FLOAT
 * ufunc_ber_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ber_types[2] = <char>NPY_DOUBLE
 * ufunc_ber_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3277
 * ufunc_ber_types[0] = <char>NPY_FLOAT
 * ufunc_ber_types[1] = <char>NPY_FLOAT
 * ufunc_ber_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ber_types[3] = <char>NPY_DOUBLE
 * ufunc_ber_ptr[2*0] = <void*>_func_ber_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3278
 * ufunc_ber_types[1] = <char>NPY_FLOAT
 * ufunc_ber_types[2] = <char>NPY_DOUBLE
 * ufunc_ber_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ber_ptr[2*0] = <void*>_func_ber_wrap
 * ufunc_ber_ptr[2*0+1] = <void*>(<char*>"ber")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3279
 * ufunc_ber_types[2] = <char>NPY_DOUBLE
 * ufunc_ber_types[3] = <char>NPY_DOUBLE
 * ufunc_ber_ptr[2*0] = <void*>_func_ber_wrap             # <<<<<<<<<<<<<<
 * ufunc_ber_ptr[2*0+1] = <void*>(<char*>"ber")
 * ufunc_ber_ptr[2*1] = <void*>_func_ber_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_ptr[0]) = ((void *)ber_wrap);

  /* "scipy/special/_ufuncs.pyx":3280
 * ufunc_ber_types[3] = <char>NPY_DOUBLE
 * ufunc_ber_ptr[2*0] = <void*>_func_ber_wrap
 * ufunc_ber_ptr[2*0+1] = <void*>(<char*>"ber")             # <<<<<<<<<<<<<<
 * ufunc_ber_ptr[2*1] = <void*>_func_ber_wrap
 * ufunc_ber_ptr[2*1+1] = <void*>(<char*>"ber")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_ptr[1]) = ((void *)((char *)((char *)"ber")));

  /* "scipy/special/_ufuncs.pyx":3281
 * ufunc_ber_ptr[2*0] = <void*>_func_ber_wrap
 * ufunc_ber_ptr[2*0+1] = <void*>(<char*>"ber")
 * ufunc_ber_ptr[2*1] = <void*>_func_ber_wrap             # <<<<<<<<<<<<<<
 * ufunc_ber_ptr[2*1+1] = <void*>(<char*>"ber")
 * ufunc_ber_data[0] = &ufunc_ber_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_ptr[2]) = ((void *)ber_wrap);

  /* "scipy/special/_ufuncs.pyx":3282
 * ufunc_ber_ptr[2*0+1] = <void*>(<char*>"ber")
 * ufunc_ber_ptr[2*1] = <void*>_func_ber_wrap
 * ufunc_ber_ptr[2*1+1] = <void*>(<char*>"ber")             # <<<<<<<<<<<<<<
 * ufunc_ber_data[0] = &ufunc_ber_ptr[2*0]
 * ufunc_ber_data[1] = &ufunc_ber_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_ptr[3]) = ((void *)((char *)((char *)"ber")));

  /* "scipy/special/_ufuncs.pyx":3283
 * ufunc_ber_ptr[2*1] = <void*>_func_ber_wrap
 * ufunc_ber_ptr[2*1+1] = <void*>(<char*>"ber")
 * ufunc_ber_data[0] = &ufunc_ber_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ber_data[1] = &ufunc_ber_ptr[2*1]
 * ber = np.PyUFunc_FromFuncAndData(ufunc_ber_loops, ufunc_ber_data, ufunc_ber_types, 2, 1, 1, 0, "ber", ufunc_ber_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3284
 * ufunc_ber_ptr[2*1+1] = <void*>(<char*>"ber")
 * ufunc_ber_data[0] = &ufunc_ber_ptr[2*0]
 * ufunc_ber_data[1] = &ufunc_ber_ptr[2*1]             # <<<<<<<<<<<<<<
 * ber = np.PyUFunc_FromFuncAndData(ufunc_ber_loops, ufunc_ber_data, ufunc_ber_types, 2, 1, 1, 0, "ber", ufunc_ber_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3285
 * ufunc_ber_data[0] = &ufunc_ber_ptr[2*0]
 * ufunc_ber_data[1] = &ufunc_ber_ptr[2*1]
 * ber = np.PyUFunc_FromFuncAndData(ufunc_ber_loops, ufunc_ber_data, ufunc_ber_types, 2, 1, 1, 0, "ber", ufunc_ber_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_berp_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_types, 2, 1, 1, 0, ((char *)"ber"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_ber_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ber, __pyx_t_2) < 0) __PYX_ERR(1, 3285, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3291
 * cdef void *ufunc_berp_data[2]
 * cdef char ufunc_berp_types[4]
 * cdef char *ufunc_berp_doc = (             # <<<<<<<<<<<<<<
 *     "berp(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_doc = ((char *)"berp(x)\n\nDerivative of the Kelvin function `ber`");

  /* "scipy/special/_ufuncs.pyx":3295
 *     "\n"
 *     "Derivative of the Kelvin function `ber`")
 * ufunc_berp_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_berp_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_berp_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":3296
 *     "Derivative of the Kelvin function `ber`")
 * ufunc_berp_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_berp_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_berp_types[0] = <char>NPY_FLOAT
 * ufunc_berp_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":3297
 * ufunc_berp_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_berp_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_berp_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_berp_types[1] = <char>NPY_FLOAT
 * ufunc_berp_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3298
 * ufunc_berp_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_berp_types[0] = <char>NPY_FLOAT
 * ufunc_berp_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_berp_types[2] = <char>NPY_DOUBLE
 * ufunc_berp_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3299
 * ufunc_berp_types[0] = <char>NPY_FLOAT
 * ufunc_berp_types[1] = <char>NPY_FLOAT
 * ufunc_berp_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_berp_types[3] = <char>NPY_DOUBLE
 * ufunc_berp_ptr[2*0] = <void*>_func_berp_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3300
 * ufunc_berp_types[1] = <char>NPY_FLOAT
 * ufunc_berp_types[2] = <char>NPY_DOUBLE
 * ufunc_berp_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_berp_ptr[2*0] = <void*>_func_berp_wrap
 * ufunc_berp_ptr[2*0+1] = <void*>(<char*>"berp")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3301
 * ufunc_berp_types[2] = <char>NPY_DOUBLE
 * ufunc_berp_types[3] = <char>NPY_DOUBLE
 * ufunc_berp_ptr[2*0] = <void*>_func_berp_wrap             # <<<<<<<<<<<<<<
 * ufunc_berp_ptr[2*0+1] = <void*>(<char*>"berp")
 * ufunc_berp_ptr[2*1] = <void*>_func_berp_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_ptr[0]) = ((void *)berp_wrap);

  /* "scipy/special/_ufuncs.pyx":3302
 * ufunc_berp_types[3] = <char>NPY_DOUBLE
 * ufunc_berp_ptr[2*0] = <void*>_func_berp_wrap
 * ufunc_berp_ptr[2*0+1] = <void*>(<char*>"berp")             # <<<<<<<<<<<<<<
 * ufunc_berp_ptr[2*1] = <void*>_func_berp_wrap
 * ufunc_berp_ptr[2*1+1] = <void*>(<char*>"berp")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_ptr[1]) = ((void *)((char *)((char *)"berp")));

  /* "scipy/special/_ufuncs.pyx":3303
 * ufunc_berp_ptr[2*0] = <void*>_func_berp_wrap
 * ufunc_berp_ptr[2*0+1] = <void*>(<char*>"berp")
 * ufunc_berp_ptr[2*1] = <void*>_func_berp_wrap             # <<<<<<<<<<<<<<
 * ufunc_berp_ptr[2*1+1] = <void*>(<char*>"berp")
 * ufunc_berp_data[0] = &ufunc_berp_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_ptr[2]) = ((void *)berp_wrap);

  /* "scipy/special/_ufuncs.pyx":3304
 * ufunc_berp_ptr[2*0+1] = <void*>(<char*>"berp")
 * ufunc_berp_ptr[2*1] = <void*>_func_berp_wrap
 * ufunc_berp_ptr[2*1+1] = <void*>(<char*>"berp")             # <<<<<<<<<<<<<<
 * ufunc_berp_data[0] = &ufunc_berp_ptr[2*0]
 * ufunc_berp_data[1] = &ufunc_berp_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_ptr[3]) = ((void *)((char *)((char *)"berp")));

  /* "scipy/special/_ufuncs.pyx":3305
 * ufunc_berp_ptr[2*1] = <void*>_func_berp_wrap
 * ufunc_berp_ptr[2*1+1] = <void*>(<char*>"berp")
 * ufunc_berp_data[0] = &ufunc_berp_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_berp_data[1] = &ufunc_berp_ptr[2*1]
 * berp = np.PyUFunc_FromFuncAndData(ufunc_berp_loops, ufunc_berp_data, ufunc_berp_types, 2, 1, 1, 0, "berp", ufunc_berp_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3306
 * ufunc_berp_ptr[2*1+1] = <void*>(<char*>"berp")
 * ufunc_berp_data[0] = &ufunc_berp_ptr[2*0]
 * ufunc_berp_data[1] = &ufunc_berp_ptr[2*1]             # <<<<<<<<<<<<<<
 * berp = np.PyUFunc_FromFuncAndData(ufunc_berp_loops, ufunc_berp_data, ufunc_berp_types, 2, 1, 1, 0, "berp", ufunc_berp_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3307
 * ufunc_berp_data[0] = &ufunc_berp_ptr[2*0]
 * ufunc_berp_data[1] = &ufunc_berp_ptr[2*1]
 * berp = np.PyUFunc_FromFuncAndData(ufunc_berp_loops, ufunc_berp_data, ufunc_berp_types, 2, 1, 1, 0, "berp", ufunc_berp_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_besselpoly_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_types, 2, 1, 1, 0, ((char *)"berp"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_berp_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_berp, __pyx_t_2) < 0) __PYX_ERR(1, 3307, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3313
 * cdef void *ufunc_besselpoly_data[2]
 * cdef char ufunc_besselpoly_types[8]
 * cdef char *ufunc_besselpoly_doc = (             # <<<<<<<<<<<<<<
 *     "besselpoly(a, lmb, nu)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_doc = ((char *)"besselpoly(a, lmb, nu)\n\nWeighted integral of a Bessel function.\n\n.. math::\n\n   \\int_0^1 x^\\lambda J_\\nu(2 a x) \\, dx\n\nwhere :math:`J_\\nu` is a Bessel function and :math:`\\lambda=lmb`,\n:math:`\\nu=nu`.");

  /* "scipy/special/_ufuncs.pyx":3324
 *     "where :math:`J_\\nu` is a Bessel function and :math:`\\lambda=lmb`,\n"
 *     ":math:`\\nu=nu`.")
 * ufunc_besselpoly_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_besselpoly_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":3325
 *     ":math:`\\nu=nu`.")
 * ufunc_besselpoly_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_besselpoly_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_types[0] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":3326
 * ufunc_besselpoly_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_besselpoly_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_besselpoly_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_types[1] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3327
 * ufunc_besselpoly_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_besselpoly_types[0] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_types[2] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3328
 * ufunc_besselpoly_types[0] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[1] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_types[3] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3329
 * ufunc_besselpoly_types[1] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[2] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_types[4] = <char>NPY_DOUBLE
 * ufunc_besselpoly_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3330
 * ufunc_besselpoly_types[2] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[3] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_types[5] = <char>NPY_DOUBLE
 * ufunc_besselpoly_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3331
 * ufunc_besselpoly_types[3] = <char>NPY_FLOAT
 * ufunc_besselpoly_types[4] = <char>NPY_DOUBLE
 * ufunc_besselpoly_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_types[6] = <char>NPY_DOUBLE
 * ufunc_besselpoly_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3332
 * ufunc_besselpoly_types[4] = <char>NPY_DOUBLE
 * ufunc_besselpoly_types[5] = <char>NPY_DOUBLE
 * ufunc_besselpoly_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_types[7] = <char>NPY_DOUBLE
 * ufunc_besselpoly_ptr[2*0] = <void*>_func_besselpoly
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3333
 * ufunc_besselpoly_types[5] = <char>NPY_DOUBLE
 * ufunc_besselpoly_types[6] = <char>NPY_DOUBLE
 * ufunc_besselpoly_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_ptr[2*0] = <void*>_func_besselpoly
 * ufunc_besselpoly_ptr[2*0+1] = <void*>(<char*>"besselpoly")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3334
 * ufunc_besselpoly_types[6] = <char>NPY_DOUBLE
 * ufunc_besselpoly_types[7] = <char>NPY_DOUBLE
 * ufunc_besselpoly_ptr[2*0] = <void*>_func_besselpoly             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_ptr[2*0+1] = <void*>(<char*>"besselpoly")
 * ufunc_besselpoly_ptr[2*1] = <void*>_func_besselpoly
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_ptr[0]) = ((void *)besselpoly);

  /* "scipy/special/_ufuncs.pyx":3335
 * ufunc_besselpoly_types[7] = <char>NPY_DOUBLE
 * ufunc_besselpoly_ptr[2*0] = <void*>_func_besselpoly
 * ufunc_besselpoly_ptr[2*0+1] = <void*>(<char*>"besselpoly")             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_ptr[2*1] = <void*>_func_besselpoly
 * ufunc_besselpoly_ptr[2*1+1] = <void*>(<char*>"besselpoly")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_ptr[1]) = ((void *)((char *)((char *)"besselpoly")));

  /* "scipy/special/_ufuncs.pyx":3336
 * ufunc_besselpoly_ptr[2*0] = <void*>_func_besselpoly
 * ufunc_besselpoly_ptr[2*0+1] = <void*>(<char*>"besselpoly")
 * ufunc_besselpoly_ptr[2*1] = <void*>_func_besselpoly             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_ptr[2*1+1] = <void*>(<char*>"besselpoly")
 * ufunc_besselpoly_data[0] = &ufunc_besselpoly_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_ptr[2]) = ((void *)besselpoly);

  /* "scipy/special/_ufuncs.pyx":3337
 * ufunc_besselpoly_ptr[2*0+1] = <void*>(<char*>"besselpoly")
 * ufunc_besselpoly_ptr[2*1] = <void*>_func_besselpoly
 * ufunc_besselpoly_ptr[2*1+1] = <void*>(<char*>"besselpoly")             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_data[0] = &ufunc_besselpoly_ptr[2*0]
 * ufunc_besselpoly_data[1] = &ufunc_besselpoly_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_ptr[3]) = ((void *)((char *)((char *)"besselpoly")));

  /* "scipy/special/_ufuncs.pyx":3338
 * ufunc_besselpoly_ptr[2*1] = <void*>_func_besselpoly
 * ufunc_besselpoly_ptr[2*1+1] = <void*>(<char*>"besselpoly")
 * ufunc_besselpoly_data[0] = &ufunc_besselpoly_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_besselpoly_data[1] = &ufunc_besselpoly_ptr[2*1]
 * besselpoly = np.PyUFunc_FromFuncAndData(ufunc_besselpoly_loops, ufunc_besselpoly_data, ufunc_besselpoly_types, 2, 3, 1, 0, "besselpoly", ufunc_besselpoly_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3339
 * ufunc_besselpoly_ptr[2*1+1] = <void*>(<char*>"besselpoly")
 * ufunc_besselpoly_data[0] = &ufunc_besselpoly_ptr[2*0]
 * ufunc_besselpoly_data[1] = &ufunc_besselpoly_ptr[2*1]             # <<<<<<<<<<<<<<
 * besselpoly = np.PyUFunc_FromFuncAndData(ufunc_besselpoly_loops, ufunc_besselpoly_data, ufunc_besselpoly_types, 2, 3, 1, 0, "besselpoly", ufunc_besselpoly_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3340
 * ufunc_besselpoly_data[0] = &ufunc_besselpoly_ptr[2*0]
 * ufunc_besselpoly_data[1] = &ufunc_besselpoly_ptr[2*1]
 * besselpoly = np.PyUFunc_FromFuncAndData(ufunc_besselpoly_loops, ufunc_besselpoly_data, ufunc_besselpoly_types, 2, 3, 1, 0, "besselpoly", ufunc_besselpoly_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_beta_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_types, 2, 3, 1, 0, ((char *)"besselpoly"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_besselpoly_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_besselpoly, __pyx_t_2) < 0) __PYX_ERR(1, 3340, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3346
 * cdef void *ufunc_beta_data[2]
 * cdef char ufunc_beta_types[6]
 * cdef char *ufunc_beta_doc = (             # <<<<<<<<<<<<<<
 *     "beta(a, b)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_doc = ((char *)"beta(a, b)\n\nBeta function.\n\n::\n\n    beta(a, b) =  gamma(a) * gamma(b) / gamma(a+b)");

  /* "scipy/special/_ufuncs.pyx":3354
 *     "\n"
 *     "    beta(a, b) =  gamma(a) * gamma(b) / gamma(a+b)")
 * ufunc_beta_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_beta_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_beta_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":3355
 *     "    beta(a, b) =  gamma(a) * gamma(b) / gamma(a+b)")
 * ufunc_beta_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_beta_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_beta_types[0] = <char>NPY_FLOAT
 * ufunc_beta_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":3356
 * ufunc_beta_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_beta_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_beta_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_beta_types[1] = <char>NPY_FLOAT
 * ufunc_beta_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3357
 * ufunc_beta_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_beta_types[0] = <char>NPY_FLOAT
 * ufunc_beta_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_beta_types[2] = <char>NPY_FLOAT
 * ufunc_beta_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3358
 * ufunc_beta_types[0] = <char>NPY_FLOAT
 * ufunc_beta_types[1] = <char>NPY_FLOAT
 * ufunc_beta_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_beta_types[3] = <char>NPY_DOUBLE
 * ufunc_beta_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3359
 * ufunc_beta_types[1] = <char>NPY_FLOAT
 * ufunc_beta_types[2] = <char>NPY_FLOAT
 * ufunc_beta_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_beta_types[4] = <char>NPY_DOUBLE
 * ufunc_beta_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3360
 * ufunc_beta_types[2] = <char>NPY_FLOAT
 * ufunc_beta_types[3] = <char>NPY_DOUBLE
 * ufunc_beta_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_beta_types[5] = <char>NPY_DOUBLE
 * ufunc_beta_ptr[2*0] = <void*>_func_beta
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3361
 * ufunc_beta_types[3] = <char>NPY_DOUBLE
 * ufunc_beta_types[4] = <char>NPY_DOUBLE
 * ufunc_beta_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_beta_ptr[2*0] = <void*>_func_beta
 * ufunc_beta_ptr[2*0+1] = <void*>(<char*>"beta")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3362
 * ufunc_beta_types[4] = <char>NPY_DOUBLE
 * ufunc_beta_types[5] = <char>NPY_DOUBLE
 * ufunc_beta_ptr[2*0] = <void*>_func_beta             # <<<<<<<<<<<<<<
 * ufunc_beta_ptr[2*0+1] = <void*>(<char*>"beta")
 * ufunc_beta_ptr[2*1] = <void*>_func_beta
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_ptr[0]) = ((void *)beta);

  /* "scipy/special/_ufuncs.pyx":3363
 * ufunc_beta_types[5] = <char>NPY_DOUBLE
 * ufunc_beta_ptr[2*0] = <void*>_func_beta
 * ufunc_beta_ptr[2*0+1] = <void*>(<char*>"beta")             # <<<<<<<<<<<<<<
 * ufunc_beta_ptr[2*1] = <void*>_func_beta
 * ufunc_beta_ptr[2*1+1] = <void*>(<char*>"beta")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_ptr[1]) = ((void *)((char *)((char *)"beta")));

  /* "scipy/special/_ufuncs.pyx":3364
 * ufunc_beta_ptr[2*0] = <void*>_func_beta
 * ufunc_beta_ptr[2*0+1] = <void*>(<char*>"beta")
 * ufunc_beta_ptr[2*1] = <void*>_func_beta             # <<<<<<<<<<<<<<
 * ufunc_beta_ptr[2*1+1] = <void*>(<char*>"beta")
 * ufunc_beta_data[0] = &ufunc_beta_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_ptr[2]) = ((void *)beta);

  /* "scipy/special/_ufuncs.pyx":3365
 * ufunc_beta_ptr[2*0+1] = <void*>(<char*>"beta")
 * ufunc_beta_ptr[2*1] = <void*>_func_beta
 * ufunc_beta_ptr[2*1+1] = <void*>(<char*>"beta")             # <<<<<<<<<<<<<<
 * ufunc_beta_data[0] = &ufunc_beta_ptr[2*0]
 * ufunc_beta_data[1] = &ufunc_beta_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_ptr[3]) = ((void *)((char *)((char *)"beta")));

  /* "scipy/special/_ufuncs.pyx":3366
 * ufunc_beta_ptr[2*1] = <void*>_func_beta
 * ufunc_beta_ptr[2*1+1] = <void*>(<char*>"beta")
 * ufunc_beta_data[0] = &ufunc_beta_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_beta_data[1] = &ufunc_beta_ptr[2*1]
 * beta = np.PyUFunc_FromFuncAndData(ufunc_beta_loops, ufunc_beta_data, ufunc_beta_types, 2, 2, 1, 0, "beta", ufunc_beta_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3367
 * ufunc_beta_ptr[2*1+1] = <void*>(<char*>"beta")
 * ufunc_beta_data[0] = &ufunc_beta_ptr[2*0]
 * ufunc_beta_data[1] = &ufunc_beta_ptr[2*1]             # <<<<<<<<<<<<<<
 * beta = np.PyUFunc_FromFuncAndData(ufunc_beta_loops, ufunc_beta_data, ufunc_beta_types, 2, 2, 1, 0, "beta", ufunc_beta_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3368
 * ufunc_beta_data[0] = &ufunc_beta_ptr[2*0]
 * ufunc_beta_data[1] = &ufunc_beta_ptr[2*1]
 * beta = np.PyUFunc_FromFuncAndData(ufunc_beta_loops, ufunc_beta_data, ufunc_beta_types, 2, 2, 1, 0, "beta", ufunc_beta_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_betainc_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_types, 2, 2, 1, 0, ((char *)"beta"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_beta_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3368, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_beta, __pyx_t_2) < 0) __PYX_ERR(1, 3368, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3374
 * cdef void *ufunc_betainc_data[2]
 * cdef char ufunc_betainc_types[8]
 * cdef char *ufunc_betainc_doc = (             # <<<<<<<<<<<<<<
 *     "betainc(a, b, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_doc = ((char *)"betainc(a, b, x)\n\nIncomplete beta integral.\n\nCompute the incomplete beta integral of the arguments, evaluated\nfrom zero to `x`::\n\n    gamma(a+b) / (gamma(a)*gamma(b)) * integral(t**(a-1) (1-t)**(b-1), t=0..x).\n\nNotes\n-----\nThe incomplete beta is also sometimes defined without the terms\nin gamma, in which case the above definition is the so-called regularized\nincomplete beta. Under this definition, you can get the incomplete beta by\nmultiplying the result of the scipy function by beta(a, b).");

  /* "scipy/special/_ufuncs.pyx":3390
 *     "incomplete beta. Under this definition, you can get the incomplete beta by\n"
 *     "multiplying the result of the scipy function by beta(a, b).")
 * ufunc_betainc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_betainc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_betainc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":3391
 *     "multiplying the result of the scipy function by beta(a, b).")
 * ufunc_betainc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_betainc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_betainc_types[0] = <char>NPY_FLOAT
 * ufunc_betainc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":3392
 * ufunc_betainc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_betainc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_betainc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betainc_types[1] = <char>NPY_FLOAT
 * ufunc_betainc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3393
 * ufunc_betainc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_betainc_types[0] = <char>NPY_FLOAT
 * ufunc_betainc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betainc_types[2] = <char>NPY_FLOAT
 * ufunc_betainc_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3394
 * ufunc_betainc_types[0] = <char>NPY_FLOAT
 * ufunc_betainc_types[1] = <char>NPY_FLOAT
 * ufunc_betainc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betainc_types[3] = <char>NPY_FLOAT
 * ufunc_betainc_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3395
 * ufunc_betainc_types[1] = <char>NPY_FLOAT
 * ufunc_betainc_types[2] = <char>NPY_FLOAT
 * ufunc_betainc_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betainc_types[4] = <char>NPY_DOUBLE
 * ufunc_betainc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3396
 * ufunc_betainc_types[2] = <char>NPY_FLOAT
 * ufunc_betainc_types[3] = <char>NPY_FLOAT
 * ufunc_betainc_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betainc_types[5] = <char>NPY_DOUBLE
 * ufunc_betainc_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3397
 * ufunc_betainc_types[3] = <char>NPY_FLOAT
 * ufunc_betainc_types[4] = <char>NPY_DOUBLE
 * ufunc_betainc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betainc_types[6] = <char>NPY_DOUBLE
 * ufunc_betainc_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3398
 * ufunc_betainc_types[4] = <char>NPY_DOUBLE
 * ufunc_betainc_types[5] = <char>NPY_DOUBLE
 * ufunc_betainc_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betainc_types[7] = <char>NPY_DOUBLE
 * ufunc_betainc_ptr[2*0] = <void*>_func_incbet
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3399
 * ufunc_betainc_types[5] = <char>NPY_DOUBLE
 * ufunc_betainc_types[6] = <char>NPY_DOUBLE
 * ufunc_betainc_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betainc_ptr[2*0] = <void*>_func_incbet
 * ufunc_betainc_ptr[2*0+1] = <void*>(<char*>"betainc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3400
 * ufunc_betainc_types[6] = <char>NPY_DOUBLE
 * ufunc_betainc_types[7] = <char>NPY_DOUBLE
 * ufunc_betainc_ptr[2*0] = <void*>_func_incbet             # <<<<<<<<<<<<<<
 * ufunc_betainc_ptr[2*0+1] = <void*>(<char*>"betainc")
 * ufunc_betainc_ptr[2*1] = <void*>_func_incbet
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_ptr[0]) = ((void *)incbet);

  /* "scipy/special/_ufuncs.pyx":3401
 * ufunc_betainc_types[7] = <char>NPY_DOUBLE
 * ufunc_betainc_ptr[2*0] = <void*>_func_incbet
 * ufunc_betainc_ptr[2*0+1] = <void*>(<char*>"betainc")             # <<<<<<<<<<<<<<
 * ufunc_betainc_ptr[2*1] = <void*>_func_incbet
 * ufunc_betainc_ptr[2*1+1] = <void*>(<char*>"betainc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_ptr[1]) = ((void *)((char *)((char *)"betainc")));

  /* "scipy/special/_ufuncs.pyx":3402
 * ufunc_betainc_ptr[2*0] = <void*>_func_incbet
 * ufunc_betainc_ptr[2*0+1] = <void*>(<char*>"betainc")
 * ufunc_betainc_ptr[2*1] = <void*>_func_incbet             # <<<<<<<<<<<<<<
 * ufunc_betainc_ptr[2*1+1] = <void*>(<char*>"betainc")
 * ufunc_betainc_data[0] = &ufunc_betainc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_ptr[2]) = ((void *)incbet);

  /* "scipy/special/_ufuncs.pyx":3403
 * ufunc_betainc_ptr[2*0+1] = <void*>(<char*>"betainc")
 * ufunc_betainc_ptr[2*1] = <void*>_func_incbet
 * ufunc_betainc_ptr[2*1+1] = <void*>(<char*>"betainc")             # <<<<<<<<<<<<<<
 * ufunc_betainc_data[0] = &ufunc_betainc_ptr[2*0]
 * ufunc_betainc_data[1] = &ufunc_betainc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_ptr[3]) = ((void *)((char *)((char *)"betainc")));

  /* "scipy/special/_ufuncs.pyx":3404
 * ufunc_betainc_ptr[2*1] = <void*>_func_incbet
 * ufunc_betainc_ptr[2*1+1] = <void*>(<char*>"betainc")
 * ufunc_betainc_data[0] = &ufunc_betainc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_betainc_data[1] = &ufunc_betainc_ptr[2*1]
 * betainc = np.PyUFunc_FromFuncAndData(ufunc_betainc_loops, ufunc_betainc_data, ufunc_betainc_types, 2, 3, 1, 0, "betainc", ufunc_betainc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3405
 * ufunc_betainc_ptr[2*1+1] = <void*>(<char*>"betainc")
 * ufunc_betainc_data[0] = &ufunc_betainc_ptr[2*0]
 * ufunc_betainc_data[1] = &ufunc_betainc_ptr[2*1]             # <<<<<<<<<<<<<<
 * betainc = np.PyUFunc_FromFuncAndData(ufunc_betainc_loops, ufunc_betainc_data, ufunc_betainc_types, 2, 3, 1, 0, "betainc", ufunc_betainc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3406
 * ufunc_betainc_data[0] = &ufunc_betainc_ptr[2*0]
 * ufunc_betainc_data[1] = &ufunc_betainc_ptr[2*1]
 * betainc = np.PyUFunc_FromFuncAndData(ufunc_betainc_loops, ufunc_betainc_data, ufunc_betainc_types, 2, 3, 1, 0, "betainc", ufunc_betainc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_betaincinv_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_types, 2, 3, 1, 0, ((char *)"betainc"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_betainc_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3406, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_betainc, __pyx_t_2) < 0) __PYX_ERR(1, 3406, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3412
 * cdef void *ufunc_betaincinv_data[2]
 * cdef char ufunc_betaincinv_types[8]
 * cdef char *ufunc_betaincinv_doc = (             # <<<<<<<<<<<<<<
 *     "betaincinv(a, b, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_doc = ((char *)"betaincinv(a, b, y)\n\nInverse function to beta integral.\n\nCompute `x` such that betainc(a, b, x) = y.");

  /* "scipy/special/_ufuncs.pyx":3418
 *     "\n"
 *     "Compute `x` such that betainc(a, b, x) = y.")
 * ufunc_betaincinv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_betaincinv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":3419
 *     "Compute `x` such that betainc(a, b, x) = y.")
 * ufunc_betaincinv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_betaincinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_types[0] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":3420
 * ufunc_betaincinv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_betaincinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_betaincinv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_types[1] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3421
 * ufunc_betaincinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_betaincinv_types[0] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_types[2] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3422
 * ufunc_betaincinv_types[0] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[1] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_types[3] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3423
 * ufunc_betaincinv_types[1] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[2] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_types[4] = <char>NPY_DOUBLE
 * ufunc_betaincinv_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3424
 * ufunc_betaincinv_types[2] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[3] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_types[5] = <char>NPY_DOUBLE
 * ufunc_betaincinv_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3425
 * ufunc_betaincinv_types[3] = <char>NPY_FLOAT
 * ufunc_betaincinv_types[4] = <char>NPY_DOUBLE
 * ufunc_betaincinv_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_types[6] = <char>NPY_DOUBLE
 * ufunc_betaincinv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3426
 * ufunc_betaincinv_types[4] = <char>NPY_DOUBLE
 * ufunc_betaincinv_types[5] = <char>NPY_DOUBLE
 * ufunc_betaincinv_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_types[7] = <char>NPY_DOUBLE
 * ufunc_betaincinv_ptr[2*0] = <void*>_func_incbi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3427
 * ufunc_betaincinv_types[5] = <char>NPY_DOUBLE
 * ufunc_betaincinv_types[6] = <char>NPY_DOUBLE
 * ufunc_betaincinv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_ptr[2*0] = <void*>_func_incbi
 * ufunc_betaincinv_ptr[2*0+1] = <void*>(<char*>"betaincinv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3428
 * ufunc_betaincinv_types[6] = <char>NPY_DOUBLE
 * ufunc_betaincinv_types[7] = <char>NPY_DOUBLE
 * ufunc_betaincinv_ptr[2*0] = <void*>_func_incbi             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_ptr[2*0+1] = <void*>(<char*>"betaincinv")
 * ufunc_betaincinv_ptr[2*1] = <void*>_func_incbi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_ptr[0]) = ((void *)incbi);

  /* "scipy/special/_ufuncs.pyx":3429
 * ufunc_betaincinv_types[7] = <char>NPY_DOUBLE
 * ufunc_betaincinv_ptr[2*0] = <void*>_func_incbi
 * ufunc_betaincinv_ptr[2*0+1] = <void*>(<char*>"betaincinv")             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_ptr[2*1] = <void*>_func_incbi
 * ufunc_betaincinv_ptr[2*1+1] = <void*>(<char*>"betaincinv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_ptr[1]) = ((void *)((char *)((char *)"betaincinv")));

  /* "scipy/special/_ufuncs.pyx":3430
 * ufunc_betaincinv_ptr[2*0] = <void*>_func_incbi
 * ufunc_betaincinv_ptr[2*0+1] = <void*>(<char*>"betaincinv")
 * ufunc_betaincinv_ptr[2*1] = <void*>_func_incbi             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_ptr[2*1+1] = <void*>(<char*>"betaincinv")
 * ufunc_betaincinv_data[0] = &ufunc_betaincinv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_ptr[2]) = ((void *)incbi);

  /* "scipy/special/_ufuncs.pyx":3431
 * ufunc_betaincinv_ptr[2*0+1] = <void*>(<char*>"betaincinv")
 * ufunc_betaincinv_ptr[2*1] = <void*>_func_incbi
 * ufunc_betaincinv_ptr[2*1+1] = <void*>(<char*>"betaincinv")             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_data[0] = &ufunc_betaincinv_ptr[2*0]
 * ufunc_betaincinv_data[1] = &ufunc_betaincinv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_ptr[3]) = ((void *)((char *)((char *)"betaincinv")));

  /* "scipy/special/_ufuncs.pyx":3432
 * ufunc_betaincinv_ptr[2*1] = <void*>_func_incbi
 * ufunc_betaincinv_ptr[2*1+1] = <void*>(<char*>"betaincinv")
 * ufunc_betaincinv_data[0] = &ufunc_betaincinv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_betaincinv_data[1] = &ufunc_betaincinv_ptr[2*1]
 * betaincinv = np.PyUFunc_FromFuncAndData(ufunc_betaincinv_loops, ufunc_betaincinv_data, ufunc_betaincinv_types, 2, 3, 1, 0, "betaincinv", ufunc_betaincinv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3433
 * ufunc_betaincinv_ptr[2*1+1] = <void*>(<char*>"betaincinv")
 * ufunc_betaincinv_data[0] = &ufunc_betaincinv_ptr[2*0]
 * ufunc_betaincinv_data[1] = &ufunc_betaincinv_ptr[2*1]             # <<<<<<<<<<<<<<
 * betaincinv = np.PyUFunc_FromFuncAndData(ufunc_betaincinv_loops, ufunc_betaincinv_data, ufunc_betaincinv_types, 2, 3, 1, 0, "betaincinv", ufunc_betaincinv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3434
 * ufunc_betaincinv_data[0] = &ufunc_betaincinv_ptr[2*0]
 * ufunc_betaincinv_data[1] = &ufunc_betaincinv_ptr[2*1]
 * betaincinv = np.PyUFunc_FromFuncAndData(ufunc_betaincinv_loops, ufunc_betaincinv_data, ufunc_betaincinv_types, 2, 3, 1, 0, "betaincinv", ufunc_betaincinv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_betaln_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_types, 2, 3, 1, 0, ((char *)"betaincinv"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaincinv_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3434, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_betaincinv, __pyx_t_2) < 0) __PYX_ERR(1, 3434, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3440
 * cdef void *ufunc_betaln_data[2]
 * cdef char ufunc_betaln_types[6]
 * cdef char *ufunc_betaln_doc = (             # <<<<<<<<<<<<<<
 *     "betaln(a, b)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_doc = ((char *)"betaln(a, b)\n\nNatural logarithm of absolute value of beta function.\n\nComputes ``ln(abs(beta(a, b)))``.");

  /* "scipy/special/_ufuncs.pyx":3446
 *     "\n"
 *     "Computes ``ln(abs(beta(a, b)))``.")
 * ufunc_betaln_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_betaln_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_betaln_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":3447
 *     "Computes ``ln(abs(beta(a, b)))``.")
 * ufunc_betaln_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_betaln_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_betaln_types[0] = <char>NPY_FLOAT
 * ufunc_betaln_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":3448
 * ufunc_betaln_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_betaln_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_betaln_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betaln_types[1] = <char>NPY_FLOAT
 * ufunc_betaln_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3449
 * ufunc_betaln_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_betaln_types[0] = <char>NPY_FLOAT
 * ufunc_betaln_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betaln_types[2] = <char>NPY_FLOAT
 * ufunc_betaln_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3450
 * ufunc_betaln_types[0] = <char>NPY_FLOAT
 * ufunc_betaln_types[1] = <char>NPY_FLOAT
 * ufunc_betaln_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_betaln_types[3] = <char>NPY_DOUBLE
 * ufunc_betaln_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3451
 * ufunc_betaln_types[1] = <char>NPY_FLOAT
 * ufunc_betaln_types[2] = <char>NPY_FLOAT
 * ufunc_betaln_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betaln_types[4] = <char>NPY_DOUBLE
 * ufunc_betaln_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3452
 * ufunc_betaln_types[2] = <char>NPY_FLOAT
 * ufunc_betaln_types[3] = <char>NPY_DOUBLE
 * ufunc_betaln_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betaln_types[5] = <char>NPY_DOUBLE
 * ufunc_betaln_ptr[2*0] = <void*>_func_lbeta
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3453
 * ufunc_betaln_types[3] = <char>NPY_DOUBLE
 * ufunc_betaln_types[4] = <char>NPY_DOUBLE
 * ufunc_betaln_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_betaln_ptr[2*0] = <void*>_func_lbeta
 * ufunc_betaln_ptr[2*0+1] = <void*>(<char*>"betaln")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3454
 * ufunc_betaln_types[4] = <char>NPY_DOUBLE
 * ufunc_betaln_types[5] = <char>NPY_DOUBLE
 * ufunc_betaln_ptr[2*0] = <void*>_func_lbeta             # <<<<<<<<<<<<<<
 * ufunc_betaln_ptr[2*0+1] = <void*>(<char*>"betaln")
 * ufunc_betaln_ptr[2*1] = <void*>_func_lbeta
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_ptr[0]) = ((void *)lbeta);

  /* "scipy/special/_ufuncs.pyx":3455
 * ufunc_betaln_types[5] = <char>NPY_DOUBLE
 * ufunc_betaln_ptr[2*0] = <void*>_func_lbeta
 * ufunc_betaln_ptr[2*0+1] = <void*>(<char*>"betaln")             # <<<<<<<<<<<<<<
 * ufunc_betaln_ptr[2*1] = <void*>_func_lbeta
 * ufunc_betaln_ptr[2*1+1] = <void*>(<char*>"betaln")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_ptr[1]) = ((void *)((char *)((char *)"betaln")));

  /* "scipy/special/_ufuncs.pyx":3456
 * ufunc_betaln_ptr[2*0] = <void*>_func_lbeta
 * ufunc_betaln_ptr[2*0+1] = <void*>(<char*>"betaln")
 * ufunc_betaln_ptr[2*1] = <void*>_func_lbeta             # <<<<<<<<<<<<<<
 * ufunc_betaln_ptr[2*1+1] = <void*>(<char*>"betaln")
 * ufunc_betaln_data[0] = &ufunc_betaln_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_ptr[2]) = ((void *)lbeta);

  /* "scipy/special/_ufuncs.pyx":3457
 * ufunc_betaln_ptr[2*0+1] = <void*>(<char*>"betaln")
 * ufunc_betaln_ptr[2*1] = <void*>_func_lbeta
 * ufunc_betaln_ptr[2*1+1] = <void*>(<char*>"betaln")             # <<<<<<<<<<<<<<
 * ufunc_betaln_data[0] = &ufunc_betaln_ptr[2*0]
 * ufunc_betaln_data[1] = &ufunc_betaln_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_ptr[3]) = ((void *)((char *)((char *)"betaln")));

  /* "scipy/special/_ufuncs.pyx":3458
 * ufunc_betaln_ptr[2*1] = <void*>_func_lbeta
 * ufunc_betaln_ptr[2*1+1] = <void*>(<char*>"betaln")
 * ufunc_betaln_data[0] = &ufunc_betaln_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_betaln_data[1] = &ufunc_betaln_ptr[2*1]
 * betaln = np.PyUFunc_FromFuncAndData(ufunc_betaln_loops, ufunc_betaln_data, ufunc_betaln_types, 2, 2, 1, 0, "betaln", ufunc_betaln_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3459
 * ufunc_betaln_ptr[2*1+1] = <void*>(<char*>"betaln")
 * ufunc_betaln_data[0] = &ufunc_betaln_ptr[2*0]
 * ufunc_betaln_data[1] = &ufunc_betaln_ptr[2*1]             # <<<<<<<<<<<<<<
 * betaln = np.PyUFunc_FromFuncAndData(ufunc_betaln_loops, ufunc_betaln_data, ufunc_betaln_types, 2, 2, 1, 0, "betaln", ufunc_betaln_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3460
 * ufunc_betaln_data[0] = &ufunc_betaln_ptr[2*0]
 * ufunc_betaln_data[1] = &ufunc_betaln_ptr[2*1]
 * betaln = np.PyUFunc_FromFuncAndData(ufunc_betaln_loops, ufunc_betaln_data, ufunc_betaln_types, 2, 2, 1, 0, "betaln", ufunc_betaln_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_binom_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_types, 2, 2, 1, 0, ((char *)"betaln"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_betaln_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_betaln, __pyx_t_2) < 0) __PYX_ERR(1, 3460, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3466
 * cdef void *ufunc_binom_data[2]
 * cdef char ufunc_binom_types[6]
 * cdef char *ufunc_binom_doc = (             # <<<<<<<<<<<<<<
 *     "binom(n, k)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_doc = ((char *)"binom(n, k)\n\nBinomial coefficient\n\nSee Also\n--------\ncomb : The number of combinations of N things taken k at a time.");

  /* "scipy/special/_ufuncs.pyx":3474
 *     "--------\n"
 *     "comb : The number of combinations of N things taken k at a time.")
 * ufunc_binom_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_binom_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_binom_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":3475
 *     "comb : The number of combinations of N things taken k at a time.")
 * ufunc_binom_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_binom_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_binom_types[0] = <char>NPY_FLOAT
 * ufunc_binom_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":3476
 * ufunc_binom_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_binom_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_binom_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_binom_types[1] = <char>NPY_FLOAT
 * ufunc_binom_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3477
 * ufunc_binom_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_binom_types[0] = <char>NPY_FLOAT
 * ufunc_binom_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_binom_types[2] = <char>NPY_FLOAT
 * ufunc_binom_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3478
 * ufunc_binom_types[0] = <char>NPY_FLOAT
 * ufunc_binom_types[1] = <char>NPY_FLOAT
 * ufunc_binom_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_binom_types[3] = <char>NPY_DOUBLE
 * ufunc_binom_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3479
 * ufunc_binom_types[1] = <char>NPY_FLOAT
 * ufunc_binom_types[2] = <char>NPY_FLOAT
 * ufunc_binom_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_binom_types[4] = <char>NPY_DOUBLE
 * ufunc_binom_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3480
 * ufunc_binom_types[2] = <char>NPY_FLOAT
 * ufunc_binom_types[3] = <char>NPY_DOUBLE
 * ufunc_binom_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_binom_types[5] = <char>NPY_DOUBLE
 * ufunc_binom_ptr[2*0] = <void*>_func_binom
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3481
 * ufunc_binom_types[3] = <char>NPY_DOUBLE
 * ufunc_binom_types[4] = <char>NPY_DOUBLE
 * ufunc_binom_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_binom_ptr[2*0] = <void*>_func_binom
 * ufunc_binom_ptr[2*0+1] = <void*>(<char*>"binom")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3482
 * ufunc_binom_types[4] = <char>NPY_DOUBLE
 * ufunc_binom_types[5] = <char>NPY_DOUBLE
 * ufunc_binom_ptr[2*0] = <void*>_func_binom             # <<<<<<<<<<<<<<
 * ufunc_binom_ptr[2*0+1] = <void*>(<char*>"binom")
 * ufunc_binom_ptr[2*1] = <void*>_func_binom
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_binom);

  /* "scipy/special/_ufuncs.pyx":3483
 * ufunc_binom_types[5] = <char>NPY_DOUBLE
 * ufunc_binom_ptr[2*0] = <void*>_func_binom
 * ufunc_binom_ptr[2*0+1] = <void*>(<char*>"binom")             # <<<<<<<<<<<<<<
 * ufunc_binom_ptr[2*1] = <void*>_func_binom
 * ufunc_binom_ptr[2*1+1] = <void*>(<char*>"binom")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_ptr[1]) = ((void *)((char *)((char *)"binom")));

  /* "scipy/special/_ufuncs.pyx":3484
 * ufunc_binom_ptr[2*0] = <void*>_func_binom
 * ufunc_binom_ptr[2*0+1] = <void*>(<char*>"binom")
 * ufunc_binom_ptr[2*1] = <void*>_func_binom             # <<<<<<<<<<<<<<
 * ufunc_binom_ptr[2*1+1] = <void*>(<char*>"binom")
 * ufunc_binom_data[0] = &ufunc_binom_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_ptr[2]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_binom);

  /* "scipy/special/_ufuncs.pyx":3485
 * ufunc_binom_ptr[2*0+1] = <void*>(<char*>"binom")
 * ufunc_binom_ptr[2*1] = <void*>_func_binom
 * ufunc_binom_ptr[2*1+1] = <void*>(<char*>"binom")             # <<<<<<<<<<<<<<
 * ufunc_binom_data[0] = &ufunc_binom_ptr[2*0]
 * ufunc_binom_data[1] = &ufunc_binom_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_ptr[3]) = ((void *)((char *)((char *)"binom")));

  /* "scipy/special/_ufuncs.pyx":3486
 * ufunc_binom_ptr[2*1] = <void*>_func_binom
 * ufunc_binom_ptr[2*1+1] = <void*>(<char*>"binom")
 * ufunc_binom_data[0] = &ufunc_binom_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_binom_data[1] = &ufunc_binom_ptr[2*1]
 * binom = np.PyUFunc_FromFuncAndData(ufunc_binom_loops, ufunc_binom_data, ufunc_binom_types, 2, 2, 1, 0, "binom", ufunc_binom_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3487
 * ufunc_binom_ptr[2*1+1] = <void*>(<char*>"binom")
 * ufunc_binom_data[0] = &ufunc_binom_ptr[2*0]
 * ufunc_binom_data[1] = &ufunc_binom_ptr[2*1]             # <<<<<<<<<<<<<<
 * binom = np.PyUFunc_FromFuncAndData(ufunc_binom_loops, ufunc_binom_data, ufunc_binom_types, 2, 2, 1, 0, "binom", ufunc_binom_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3488
 * ufunc_binom_data[0] = &ufunc_binom_ptr[2*0]
 * ufunc_binom_data[1] = &ufunc_binom_ptr[2*1]
 * binom = np.PyUFunc_FromFuncAndData(ufunc_binom_loops, ufunc_binom_data, ufunc_binom_types, 2, 2, 1, 0, "binom", ufunc_binom_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_boxcox_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_types, 2, 2, 1, 0, ((char *)"binom"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_binom_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3488, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_binom, __pyx_t_2) < 0) __PYX_ERR(1, 3488, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3494
 * cdef void *ufunc_boxcox_data[2]
 * cdef char ufunc_boxcox_types[6]
 * cdef char *ufunc_boxcox_doc = (             # <<<<<<<<<<<<<<
 *     "boxcox(x, lmbda)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_doc = ((char *)"boxcox(x, lmbda)\n\nCompute the Box-Cox transformation.\n\nThe Box-Cox transformation is::\n\n    y = (x**lmbda - 1) / lmbda  if lmbda != 0\n        log(x)                  if lmbda == 0\n\nReturns `nan` if ``x < 0``.\nReturns `-inf` if ``x == 0`` and ``lmbda < 0``.\n\nParameters\n----------\nx : array_like\n    Data to be transformed.\nlmbda : array_like\n    Power parameter of the Box-Cox transform.\n\nReturns\n-------\ny : array\n    Transformed data.\n\nNotes\n-----\n\n.. versionadded:: 0.14.0\n\nExamples\n--------\n>>> from scipy.special import boxcox\n>>> boxcox([1, 4, 10], 2.5)\narray([   0.        ,   12.4       ,  126.09110641])\n>>> boxcox(2, [0, 1, 2])\narray([ 0.69314718,  1.        ,  1.5       ])");

  /* "scipy/special/_ufuncs.pyx":3531
 *     ">>> boxcox(2, [0, 1, 2])\n"
 *     "array([ 0.69314718,  1.        ,  1.5       ])")
 * ufunc_boxcox_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_boxcox_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_boxcox_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":3532
 *     "array([ 0.69314718,  1.        ,  1.5       ])")
 * ufunc_boxcox_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_boxcox_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_boxcox_types[0] = <char>NPY_FLOAT
 * ufunc_boxcox_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":3533
 * ufunc_boxcox_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_boxcox_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_boxcox_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_boxcox_types[1] = <char>NPY_FLOAT
 * ufunc_boxcox_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3534
 * ufunc_boxcox_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_boxcox_types[0] = <char>NPY_FLOAT
 * ufunc_boxcox_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_boxcox_types[2] = <char>NPY_FLOAT
 * ufunc_boxcox_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3535
 * ufunc_boxcox_types[0] = <char>NPY_FLOAT
 * ufunc_boxcox_types[1] = <char>NPY_FLOAT
 * ufunc_boxcox_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_boxcox_types[3] = <char>NPY_DOUBLE
 * ufunc_boxcox_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3536
 * ufunc_boxcox_types[1] = <char>NPY_FLOAT
 * ufunc_boxcox_types[2] = <char>NPY_FLOAT
 * ufunc_boxcox_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_boxcox_types[4] = <char>NPY_DOUBLE
 * ufunc_boxcox_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3537
 * ufunc_boxcox_types[2] = <char>NPY_FLOAT
 * ufunc_boxcox_types[3] = <char>NPY_DOUBLE
 * ufunc_boxcox_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_boxcox_types[5] = <char>NPY_DOUBLE
 * ufunc_boxcox_ptr[2*0] = <void*>_func_boxcox
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3538
 * ufunc_boxcox_types[3] = <char>NPY_DOUBLE
 * ufunc_boxcox_types[4] = <char>NPY_DOUBLE
 * ufunc_boxcox_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_boxcox_ptr[2*0] = <void*>_func_boxcox
 * ufunc_boxcox_ptr[2*0+1] = <void*>(<char*>"boxcox")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3539
 * ufunc_boxcox_types[4] = <char>NPY_DOUBLE
 * ufunc_boxcox_types[5] = <char>NPY_DOUBLE
 * ufunc_boxcox_ptr[2*0] = <void*>_func_boxcox             # <<<<<<<<<<<<<<
 * ufunc_boxcox_ptr[2*0+1] = <void*>(<char*>"boxcox")
 * ufunc_boxcox_ptr[2*1] = <void*>_func_boxcox
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_ptr[0]) = ((void *)__pyx_f_5scipy_7special_7_boxcox_boxcox);

  /* "scipy/special/_ufuncs.pyx":3540
 * ufunc_boxcox_types[5] = <char>NPY_DOUBLE
 * ufunc_boxcox_ptr[2*0] = <void*>_func_boxcox
 * ufunc_boxcox_ptr[2*0+1] = <void*>(<char*>"boxcox")             # <<<<<<<<<<<<<<
 * ufunc_boxcox_ptr[2*1] = <void*>_func_boxcox
 * ufunc_boxcox_ptr[2*1+1] = <void*>(<char*>"boxcox")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_ptr[1]) = ((void *)((char *)((char *)"boxcox")));

  /* "scipy/special/_ufuncs.pyx":3541
 * ufunc_boxcox_ptr[2*0] = <void*>_func_boxcox
 * ufunc_boxcox_ptr[2*0+1] = <void*>(<char*>"boxcox")
 * ufunc_boxcox_ptr[2*1] = <void*>_func_boxcox             # <<<<<<<<<<<<<<
 * ufunc_boxcox_ptr[2*1+1] = <void*>(<char*>"boxcox")
 * ufunc_boxcox_data[0] = &ufunc_boxcox_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_boxcox_boxcox);

  /* "scipy/special/_ufuncs.pyx":3542
 * ufunc_boxcox_ptr[2*0+1] = <void*>(<char*>"boxcox")
 * ufunc_boxcox_ptr[2*1] = <void*>_func_boxcox
 * ufunc_boxcox_ptr[2*1+1] = <void*>(<char*>"boxcox")             # <<<<<<<<<<<<<<
 * ufunc_boxcox_data[0] = &ufunc_boxcox_ptr[2*0]
 * ufunc_boxcox_data[1] = &ufunc_boxcox_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_ptr[3]) = ((void *)((char *)((char *)"boxcox")));

  /* "scipy/special/_ufuncs.pyx":3543
 * ufunc_boxcox_ptr[2*1] = <void*>_func_boxcox
 * ufunc_boxcox_ptr[2*1+1] = <void*>(<char*>"boxcox")
 * ufunc_boxcox_data[0] = &ufunc_boxcox_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_boxcox_data[1] = &ufunc_boxcox_ptr[2*1]
 * boxcox = np.PyUFunc_FromFuncAndData(ufunc_boxcox_loops, ufunc_boxcox_data, ufunc_boxcox_types, 2, 2, 1, 0, "boxcox", ufunc_boxcox_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3544
 * ufunc_boxcox_ptr[2*1+1] = <void*>(<char*>"boxcox")
 * ufunc_boxcox_data[0] = &ufunc_boxcox_ptr[2*0]
 * ufunc_boxcox_data[1] = &ufunc_boxcox_ptr[2*1]             # <<<<<<<<<<<<<<
 * boxcox = np.PyUFunc_FromFuncAndData(ufunc_boxcox_loops, ufunc_boxcox_data, ufunc_boxcox_types, 2, 2, 1, 0, "boxcox", ufunc_boxcox_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3545
 * ufunc_boxcox_data[0] = &ufunc_boxcox_ptr[2*0]
 * ufunc_boxcox_data[1] = &ufunc_boxcox_ptr[2*1]
 * boxcox = np.PyUFunc_FromFuncAndData(ufunc_boxcox_loops, ufunc_boxcox_data, ufunc_boxcox_types, 2, 2, 1, 0, "boxcox", ufunc_boxcox_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_boxcox1p_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_types, 2, 2, 1, 0, ((char *)"boxcox"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3545, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_boxcox, __pyx_t_2) < 0) __PYX_ERR(1, 3545, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3551
 * cdef void *ufunc_boxcox1p_data[2]
 * cdef char ufunc_boxcox1p_types[6]
 * cdef char *ufunc_boxcox1p_doc = (             # <<<<<<<<<<<<<<
 *     "boxcox1p(x, lmbda)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_doc = ((char *)"boxcox1p(x, lmbda)\n\nCompute the Box-Cox transformation of 1 + `x`.\n\nThe Box-Cox transformation computed by `boxcox1p` is::\n\n    y = ((1+x)**lmbda - 1) / lmbda  if lmbda != 0\n        log(1+x)                    if lmbda == 0\n\nReturns `nan` if ``x < -1``.\nReturns `-inf` if ``x == -1`` and ``lmbda < 0``.\n\nParameters\n----------\nx : array_like\n    Data to be transformed.\nlmbda : array_like\n    Power parameter of the Box-Cox transform.\n\nReturns\n-------\ny : array\n    Transformed data.\n\nNotes\n-----\n\n.. versionadded:: 0.14.0\n\nExamples\n--------\n>>> from scipy.special import boxcox1p\n>>> boxcox1p(1e-4, [0, 0.5, 1])\narray([  9.99950003e-05,   9.99975001e-05,   1.00000000e-04])\n>>> boxcox1p([0.01, 0.1], 0.25)\narray([ 0.00996272,  0.09645476])");

  /* "scipy/special/_ufuncs.pyx":3588
 *     ">>> boxcox1p([0.01, 0.1], 0.25)\n"
 *     "array([ 0.00996272,  0.09645476])")
 * ufunc_boxcox1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_boxcox1p_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":3589
 *     "array([ 0.00996272,  0.09645476])")
 * ufunc_boxcox1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_boxcox1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_types[0] = <char>NPY_FLOAT
 * ufunc_boxcox1p_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":3590
 * ufunc_boxcox1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_boxcox1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_boxcox1p_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_types[1] = <char>NPY_FLOAT
 * ufunc_boxcox1p_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3591
 * ufunc_boxcox1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_boxcox1p_types[0] = <char>NPY_FLOAT
 * ufunc_boxcox1p_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_types[2] = <char>NPY_FLOAT
 * ufunc_boxcox1p_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3592
 * ufunc_boxcox1p_types[0] = <char>NPY_FLOAT
 * ufunc_boxcox1p_types[1] = <char>NPY_FLOAT
 * ufunc_boxcox1p_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_types[3] = <char>NPY_DOUBLE
 * ufunc_boxcox1p_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3593
 * ufunc_boxcox1p_types[1] = <char>NPY_FLOAT
 * ufunc_boxcox1p_types[2] = <char>NPY_FLOAT
 * ufunc_boxcox1p_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_types[4] = <char>NPY_DOUBLE
 * ufunc_boxcox1p_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3594
 * ufunc_boxcox1p_types[2] = <char>NPY_FLOAT
 * ufunc_boxcox1p_types[3] = <char>NPY_DOUBLE
 * ufunc_boxcox1p_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_types[5] = <char>NPY_DOUBLE
 * ufunc_boxcox1p_ptr[2*0] = <void*>_func_boxcox1p
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3595
 * ufunc_boxcox1p_types[3] = <char>NPY_DOUBLE
 * ufunc_boxcox1p_types[4] = <char>NPY_DOUBLE
 * ufunc_boxcox1p_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_ptr[2*0] = <void*>_func_boxcox1p
 * ufunc_boxcox1p_ptr[2*0+1] = <void*>(<char*>"boxcox1p")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3596
 * ufunc_boxcox1p_types[4] = <char>NPY_DOUBLE
 * ufunc_boxcox1p_types[5] = <char>NPY_DOUBLE
 * ufunc_boxcox1p_ptr[2*0] = <void*>_func_boxcox1p             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_ptr[2*0+1] = <void*>(<char*>"boxcox1p")
 * ufunc_boxcox1p_ptr[2*1] = <void*>_func_boxcox1p
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_ptr[0]) = ((void *)__pyx_f_5scipy_7special_7_boxcox_boxcox1p);

  /* "scipy/special/_ufuncs.pyx":3597
 * ufunc_boxcox1p_types[5] = <char>NPY_DOUBLE
 * ufunc_boxcox1p_ptr[2*0] = <void*>_func_boxcox1p
 * ufunc_boxcox1p_ptr[2*0+1] = <void*>(<char*>"boxcox1p")             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_ptr[2*1] = <void*>_func_boxcox1p
 * ufunc_boxcox1p_ptr[2*1+1] = <void*>(<char*>"boxcox1p")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_ptr[1]) = ((void *)((char *)((char *)"boxcox1p")));

  /* "scipy/special/_ufuncs.pyx":3598
 * ufunc_boxcox1p_ptr[2*0] = <void*>_func_boxcox1p
 * ufunc_boxcox1p_ptr[2*0+1] = <void*>(<char*>"boxcox1p")
 * ufunc_boxcox1p_ptr[2*1] = <void*>_func_boxcox1p             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_ptr[2*1+1] = <void*>(<char*>"boxcox1p")
 * ufunc_boxcox1p_data[0] = &ufunc_boxcox1p_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_boxcox_boxcox1p);

  /* "scipy/special/_ufuncs.pyx":3599
 * ufunc_boxcox1p_ptr[2*0+1] = <void*>(<char*>"boxcox1p")
 * ufunc_boxcox1p_ptr[2*1] = <void*>_func_boxcox1p
 * ufunc_boxcox1p_ptr[2*1+1] = <void*>(<char*>"boxcox1p")             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_data[0] = &ufunc_boxcox1p_ptr[2*0]
 * ufunc_boxcox1p_data[1] = &ufunc_boxcox1p_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_ptr[3]) = ((void *)((char *)((char *)"boxcox1p")));

  /* "scipy/special/_ufuncs.pyx":3600
 * ufunc_boxcox1p_ptr[2*1] = <void*>_func_boxcox1p
 * ufunc_boxcox1p_ptr[2*1+1] = <void*>(<char*>"boxcox1p")
 * ufunc_boxcox1p_data[0] = &ufunc_boxcox1p_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_boxcox1p_data[1] = &ufunc_boxcox1p_ptr[2*1]
 * boxcox1p = np.PyUFunc_FromFuncAndData(ufunc_boxcox1p_loops, ufunc_boxcox1p_data, ufunc_boxcox1p_types, 2, 2, 1, 0, "boxcox1p", ufunc_boxcox1p_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3601
 * ufunc_boxcox1p_ptr[2*1+1] = <void*>(<char*>"boxcox1p")
 * ufunc_boxcox1p_data[0] = &ufunc_boxcox1p_ptr[2*0]
 * ufunc_boxcox1p_data[1] = &ufunc_boxcox1p_ptr[2*1]             # <<<<<<<<<<<<<<
 * boxcox1p = np.PyUFunc_FromFuncAndData(ufunc_boxcox1p_loops, ufunc_boxcox1p_data, ufunc_boxcox1p_types, 2, 2, 1, 0, "boxcox1p", ufunc_boxcox1p_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3602
 * ufunc_boxcox1p_data[0] = &ufunc_boxcox1p_ptr[2*0]
 * ufunc_boxcox1p_data[1] = &ufunc_boxcox1p_ptr[2*1]
 * boxcox1p = np.PyUFunc_FromFuncAndData(ufunc_boxcox1p_loops, ufunc_boxcox1p_data, ufunc_boxcox1p_types, 2, 2, 1, 0, "boxcox1p", ufunc_boxcox1p_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_btdtr_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_types, 2, 2, 1, 0, ((char *)"boxcox1p"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_boxcox1p_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3602, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_boxcox1p, __pyx_t_2) < 0) __PYX_ERR(1, 3602, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3608
 * cdef void *ufunc_btdtr_data[2]
 * cdef char ufunc_btdtr_types[8]
 * cdef char *ufunc_btdtr_doc = (             # <<<<<<<<<<<<<<
 *     "btdtr(a, b, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_doc = ((char *)"btdtr(a, b, x)\n\nCumulative distribution function of the beta distribution.\n\nReturns the integral from zero to `x` of the beta probability density\nfunction,\n\n.. math::\n    I = \\int_0^x \\frac{\\Gamma(a + b)}{\\Gamma(a)\\Gamma(b)} t^{a-1} (1-t)^{b-1}\\,dt\n\nwhere :math:`\\Gamma` is the gamma function.\n\nParameters\n----------\na : array_like\n    Shape parameter (a > 0).\nb : array_like\n    Shape parameter (b > 0).\nx : array_like\n    Upper limit of integration, in [0, 1].\n\nReturns\n-------\nI : ndarray\n    Cumulative distribution function of the beta distribution with\n    parameters `a` and `b` at `x`.\n\nSee Also\n--------\nbetainc\n\nNotes\n-----\nThis function is identical to the incomplete beta integral function\n`betainc`.\n\nWrapper for the Cephes [1]_ routine `btdtr`.\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":3651
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_btdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_btdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":3652
 *     "       http://www.netlib.org/cephes/")
 * ufunc_btdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_btdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_btdtr_types[0] = <char>NPY_FLOAT
 * ufunc_btdtr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":3653
 * ufunc_btdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_btdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtr_types[1] = <char>NPY_FLOAT
 * ufunc_btdtr_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3654
 * ufunc_btdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtr_types[0] = <char>NPY_FLOAT
 * ufunc_btdtr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtr_types[2] = <char>NPY_FLOAT
 * ufunc_btdtr_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3655
 * ufunc_btdtr_types[0] = <char>NPY_FLOAT
 * ufunc_btdtr_types[1] = <char>NPY_FLOAT
 * ufunc_btdtr_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtr_types[3] = <char>NPY_FLOAT
 * ufunc_btdtr_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3656
 * ufunc_btdtr_types[1] = <char>NPY_FLOAT
 * ufunc_btdtr_types[2] = <char>NPY_FLOAT
 * ufunc_btdtr_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtr_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3657
 * ufunc_btdtr_types[2] = <char>NPY_FLOAT
 * ufunc_btdtr_types[3] = <char>NPY_FLOAT
 * ufunc_btdtr_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtr_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3658
 * ufunc_btdtr_types[3] = <char>NPY_FLOAT
 * ufunc_btdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtr_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtr_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3659
 * ufunc_btdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtr_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtr_ptr[2*0] = <void*>_func_btdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3660
 * ufunc_btdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtr_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtr_ptr[2*0] = <void*>_func_btdtr
 * ufunc_btdtr_ptr[2*0+1] = <void*>(<char*>"btdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3661
 * ufunc_btdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtr_ptr[2*0] = <void*>_func_btdtr             # <<<<<<<<<<<<<<
 * ufunc_btdtr_ptr[2*0+1] = <void*>(<char*>"btdtr")
 * ufunc_btdtr_ptr[2*1] = <void*>_func_btdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_ptr[0]) = ((void *)btdtr);

  /* "scipy/special/_ufuncs.pyx":3662
 * ufunc_btdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtr_ptr[2*0] = <void*>_func_btdtr
 * ufunc_btdtr_ptr[2*0+1] = <void*>(<char*>"btdtr")             # <<<<<<<<<<<<<<
 * ufunc_btdtr_ptr[2*1] = <void*>_func_btdtr
 * ufunc_btdtr_ptr[2*1+1] = <void*>(<char*>"btdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_ptr[1]) = ((void *)((char *)((char *)"btdtr")));

  /* "scipy/special/_ufuncs.pyx":3663
 * ufunc_btdtr_ptr[2*0] = <void*>_func_btdtr
 * ufunc_btdtr_ptr[2*0+1] = <void*>(<char*>"btdtr")
 * ufunc_btdtr_ptr[2*1] = <void*>_func_btdtr             # <<<<<<<<<<<<<<
 * ufunc_btdtr_ptr[2*1+1] = <void*>(<char*>"btdtr")
 * ufunc_btdtr_data[0] = &ufunc_btdtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_ptr[2]) = ((void *)btdtr);

  /* "scipy/special/_ufuncs.pyx":3664
 * ufunc_btdtr_ptr[2*0+1] = <void*>(<char*>"btdtr")
 * ufunc_btdtr_ptr[2*1] = <void*>_func_btdtr
 * ufunc_btdtr_ptr[2*1+1] = <void*>(<char*>"btdtr")             # <<<<<<<<<<<<<<
 * ufunc_btdtr_data[0] = &ufunc_btdtr_ptr[2*0]
 * ufunc_btdtr_data[1] = &ufunc_btdtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_ptr[3]) = ((void *)((char *)((char *)"btdtr")));

  /* "scipy/special/_ufuncs.pyx":3665
 * ufunc_btdtr_ptr[2*1] = <void*>_func_btdtr
 * ufunc_btdtr_ptr[2*1+1] = <void*>(<char*>"btdtr")
 * ufunc_btdtr_data[0] = &ufunc_btdtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_btdtr_data[1] = &ufunc_btdtr_ptr[2*1]
 * btdtr = np.PyUFunc_FromFuncAndData(ufunc_btdtr_loops, ufunc_btdtr_data, ufunc_btdtr_types, 2, 3, 1, 0, "btdtr", ufunc_btdtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3666
 * ufunc_btdtr_ptr[2*1+1] = <void*>(<char*>"btdtr")
 * ufunc_btdtr_data[0] = &ufunc_btdtr_ptr[2*0]
 * ufunc_btdtr_data[1] = &ufunc_btdtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * btdtr = np.PyUFunc_FromFuncAndData(ufunc_btdtr_loops, ufunc_btdtr_data, ufunc_btdtr_types, 2, 3, 1, 0, "btdtr", ufunc_btdtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3667
 * ufunc_btdtr_data[0] = &ufunc_btdtr_ptr[2*0]
 * ufunc_btdtr_data[1] = &ufunc_btdtr_ptr[2*1]
 * btdtr = np.PyUFunc_FromFuncAndData(ufunc_btdtr_loops, ufunc_btdtr_data, ufunc_btdtr_types, 2, 3, 1, 0, "btdtr", ufunc_btdtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_btdtri_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_types, 2, 3, 1, 0, ((char *)"btdtr"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtr_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3667, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_btdtr, __pyx_t_2) < 0) __PYX_ERR(1, 3667, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3673
 * cdef void *ufunc_btdtri_data[2]
 * cdef char ufunc_btdtri_types[8]
 * cdef char *ufunc_btdtri_doc = (             # <<<<<<<<<<<<<<
 *     "btdtri(a, b, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_doc = ((char *)"btdtri(a, b, p)\n\nThe `p`-th quantile of the beta distribution.\n\nThis function is the inverse of the beta cumulative distribution function,\n`btdtr`, returning the value of `x` for which `btdtr(a, b, x) = p`, or\n\n.. math::\n    p = \\int_0^x \\frac{\\Gamma(a + b)}{\\Gamma(a)\\Gamma(b)} t^{a-1} (1-t)^{b-1}\\,dt\n\nParameters\n----------\na : array_like\n    Shape parameter (`a` > 0).\nb : array_like\n    Shape parameter (`b` > 0).\np : array_like\n    Cumulative probability, in [0, 1].\n\nReturns\n-------\nx : ndarray\n    The quantile corresponding to `p`.\n\nSee Also\n--------\nbetaincinv\nbtdtr\n\nNotes\n-----\nThe value of `x` is found by interval halving or Newton iterations.\n\nWrapper for the Cephes [1]_ routine `incbi`, which solves the equivalent\nproblem of finding the inverse of the incomplete beta integral.\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":3714
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_btdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_btdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtri_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":3715
 *     "       http://www.netlib.org/cephes/")
 * ufunc_btdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_btdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_btdtri_types[0] = <char>NPY_FLOAT
 * ufunc_btdtri_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":3716
 * ufunc_btdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_btdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtri_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtri_types[1] = <char>NPY_FLOAT
 * ufunc_btdtri_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3717
 * ufunc_btdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtri_types[0] = <char>NPY_FLOAT
 * ufunc_btdtri_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtri_types[2] = <char>NPY_FLOAT
 * ufunc_btdtri_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3718
 * ufunc_btdtri_types[0] = <char>NPY_FLOAT
 * ufunc_btdtri_types[1] = <char>NPY_FLOAT
 * ufunc_btdtri_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtri_types[3] = <char>NPY_FLOAT
 * ufunc_btdtri_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3719
 * ufunc_btdtri_types[1] = <char>NPY_FLOAT
 * ufunc_btdtri_types[2] = <char>NPY_FLOAT
 * ufunc_btdtri_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtri_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtri_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3720
 * ufunc_btdtri_types[2] = <char>NPY_FLOAT
 * ufunc_btdtri_types[3] = <char>NPY_FLOAT
 * ufunc_btdtri_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtri_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3721
 * ufunc_btdtri_types[3] = <char>NPY_FLOAT
 * ufunc_btdtri_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtri_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtri_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3722
 * ufunc_btdtri_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtri_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtri_ptr[2*0] = <void*>_func_incbi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3723
 * ufunc_btdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtri_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtri_ptr[2*0] = <void*>_func_incbi
 * ufunc_btdtri_ptr[2*0+1] = <void*>(<char*>"btdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3724
 * ufunc_btdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtri_ptr[2*0] = <void*>_func_incbi             # <<<<<<<<<<<<<<
 * ufunc_btdtri_ptr[2*0+1] = <void*>(<char*>"btdtri")
 * ufunc_btdtri_ptr[2*1] = <void*>_func_incbi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_ptr[0]) = ((void *)incbi);

  /* "scipy/special/_ufuncs.pyx":3725
 * ufunc_btdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtri_ptr[2*0] = <void*>_func_incbi
 * ufunc_btdtri_ptr[2*0+1] = <void*>(<char*>"btdtri")             # <<<<<<<<<<<<<<
 * ufunc_btdtri_ptr[2*1] = <void*>_func_incbi
 * ufunc_btdtri_ptr[2*1+1] = <void*>(<char*>"btdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_ptr[1]) = ((void *)((char *)((char *)"btdtri")));

  /* "scipy/special/_ufuncs.pyx":3726
 * ufunc_btdtri_ptr[2*0] = <void*>_func_incbi
 * ufunc_btdtri_ptr[2*0+1] = <void*>(<char*>"btdtri")
 * ufunc_btdtri_ptr[2*1] = <void*>_func_incbi             # <<<<<<<<<<<<<<
 * ufunc_btdtri_ptr[2*1+1] = <void*>(<char*>"btdtri")
 * ufunc_btdtri_data[0] = &ufunc_btdtri_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_ptr[2]) = ((void *)incbi);

  /* "scipy/special/_ufuncs.pyx":3727
 * ufunc_btdtri_ptr[2*0+1] = <void*>(<char*>"btdtri")
 * ufunc_btdtri_ptr[2*1] = <void*>_func_incbi
 * ufunc_btdtri_ptr[2*1+1] = <void*>(<char*>"btdtri")             # <<<<<<<<<<<<<<
 * ufunc_btdtri_data[0] = &ufunc_btdtri_ptr[2*0]
 * ufunc_btdtri_data[1] = &ufunc_btdtri_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_ptr[3]) = ((void *)((char *)((char *)"btdtri")));

  /* "scipy/special/_ufuncs.pyx":3728
 * ufunc_btdtri_ptr[2*1] = <void*>_func_incbi
 * ufunc_btdtri_ptr[2*1+1] = <void*>(<char*>"btdtri")
 * ufunc_btdtri_data[0] = &ufunc_btdtri_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_btdtri_data[1] = &ufunc_btdtri_ptr[2*1]
 * btdtri = np.PyUFunc_FromFuncAndData(ufunc_btdtri_loops, ufunc_btdtri_data, ufunc_btdtri_types, 2, 3, 1, 0, "btdtri", ufunc_btdtri_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3729
 * ufunc_btdtri_ptr[2*1+1] = <void*>(<char*>"btdtri")
 * ufunc_btdtri_data[0] = &ufunc_btdtri_ptr[2*0]
 * ufunc_btdtri_data[1] = &ufunc_btdtri_ptr[2*1]             # <<<<<<<<<<<<<<
 * btdtri = np.PyUFunc_FromFuncAndData(ufunc_btdtri_loops, ufunc_btdtri_data, ufunc_btdtri_types, 2, 3, 1, 0, "btdtri", ufunc_btdtri_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3730
 * ufunc_btdtri_data[0] = &ufunc_btdtri_ptr[2*0]
 * ufunc_btdtri_data[1] = &ufunc_btdtri_ptr[2*1]
 * btdtri = np.PyUFunc_FromFuncAndData(ufunc_btdtri_loops, ufunc_btdtri_data, ufunc_btdtri_types, 2, 3, 1, 0, "btdtri", ufunc_btdtri_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_btdtria_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_types, 2, 3, 1, 0, ((char *)"btdtri"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtri_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3730, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_btdtri, __pyx_t_2) < 0) __PYX_ERR(1, 3730, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3736
 * cdef void *ufunc_btdtria_data[2]
 * cdef char ufunc_btdtria_types[8]
 * cdef char *ufunc_btdtria_doc = (             # <<<<<<<<<<<<<<
 *     "btdtria(p, b, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_doc = ((char *)"btdtria(p, b, x)\n\nInverse of `btdtr` with respect to `a`.\n\nThis is the inverse of the beta cumulative distribution function, `btdtr`,\nconsidered as a function of `a`, returning the value of `a` for which\n`btdtr(a, b, x) = p`, or\n\n.. math::\n    p = \\int_0^x \\frac{\\Gamma(a + b)}{\\Gamma(a)\\Gamma(b)} t^{a-1} (1-t)^{b-1}\\,dt\n\nParameters\n----------\np : array_like\n    Cumulative probability, in [0, 1].\nb : array_like\n    Shape parameter (`b` > 0).\nx : array_like\n    The quantile, in [0, 1].\n\nReturns\n-------\na : ndarray\n    The value of the shape parameter `a` such that `btdtr(a, b, x) = p`.\n\nSee Also\n--------\nbtdtr : Cumulative distribution function of the beta distribution.\nbtdtri : Inverse with respect to `x`.\nbtdtrib : Inverse with respect to `b`.\n\nNotes\n-----\nWrapper for the CDFLIB [1]_ Fortran routine `cdfbet`.\n\nThe cumulative distribution function `p` is computed using a routine by\nDiDinato and Morris [2]_.  Computation of `a` involves a search for a value\nthat produces the desired value of `p`.  The search relies on the\nmonotonicity of `p` with `a`.\n\nReferences\n----------\n.. [1] Barry Brown, James Lovato, and Kathy Russell,\n       CDFLIB: Library of Fortran Routines for Cumulative Distribution\n       Functions, Inverses, and Other Parameters.\n.. [2] DiDinato, A. R. and Morris, A. H.,\n       Algorithm 708: Significant Digit Computation of the Incomplete Beta\n       Function Ratios. ACM Trans. Math. Softw. 18 (1993), 360-373.");

  /* "scipy/special/_ufuncs.pyx":3785
 *     "       Algorithm 708: Significant Digit Computation of the Incomplete Beta\n"
 *     "       Function Ratios. ACM Trans. Math. Softw. 18 (1993), 360-373.")
 * ufunc_btdtria_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_btdtria_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtria_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":3786
 *     "       Function Ratios. ACM Trans. Math. Softw. 18 (1993), 360-373.")
 * ufunc_btdtria_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_btdtria_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_btdtria_types[0] = <char>NPY_FLOAT
 * ufunc_btdtria_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":3787
 * ufunc_btdtria_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_btdtria_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtria_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtria_types[1] = <char>NPY_FLOAT
 * ufunc_btdtria_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3788
 * ufunc_btdtria_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtria_types[0] = <char>NPY_FLOAT
 * ufunc_btdtria_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtria_types[2] = <char>NPY_FLOAT
 * ufunc_btdtria_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3789
 * ufunc_btdtria_types[0] = <char>NPY_FLOAT
 * ufunc_btdtria_types[1] = <char>NPY_FLOAT
 * ufunc_btdtria_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtria_types[3] = <char>NPY_FLOAT
 * ufunc_btdtria_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3790
 * ufunc_btdtria_types[1] = <char>NPY_FLOAT
 * ufunc_btdtria_types[2] = <char>NPY_FLOAT
 * ufunc_btdtria_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtria_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtria_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3791
 * ufunc_btdtria_types[2] = <char>NPY_FLOAT
 * ufunc_btdtria_types[3] = <char>NPY_FLOAT
 * ufunc_btdtria_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtria_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtria_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3792
 * ufunc_btdtria_types[3] = <char>NPY_FLOAT
 * ufunc_btdtria_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtria_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtria_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtria_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3793
 * ufunc_btdtria_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtria_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtria_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtria_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtria_ptr[2*0] = <void*>_func_cdfbet3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3794
 * ufunc_btdtria_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtria_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtria_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtria_ptr[2*0] = <void*>_func_cdfbet3_wrap
 * ufunc_btdtria_ptr[2*0+1] = <void*>(<char*>"btdtria")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3795
 * ufunc_btdtria_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtria_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtria_ptr[2*0] = <void*>_func_cdfbet3_wrap             # <<<<<<<<<<<<<<
 * ufunc_btdtria_ptr[2*0+1] = <void*>(<char*>"btdtria")
 * ufunc_btdtria_ptr[2*1] = <void*>_func_cdfbet3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_ptr[0]) = ((void *)cdfbet3_wrap);

  /* "scipy/special/_ufuncs.pyx":3796
 * ufunc_btdtria_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtria_ptr[2*0] = <void*>_func_cdfbet3_wrap
 * ufunc_btdtria_ptr[2*0+1] = <void*>(<char*>"btdtria")             # <<<<<<<<<<<<<<
 * ufunc_btdtria_ptr[2*1] = <void*>_func_cdfbet3_wrap
 * ufunc_btdtria_ptr[2*1+1] = <void*>(<char*>"btdtria")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_ptr[1]) = ((void *)((char *)((char *)"btdtria")));

  /* "scipy/special/_ufuncs.pyx":3797
 * ufunc_btdtria_ptr[2*0] = <void*>_func_cdfbet3_wrap
 * ufunc_btdtria_ptr[2*0+1] = <void*>(<char*>"btdtria")
 * ufunc_btdtria_ptr[2*1] = <void*>_func_cdfbet3_wrap             # <<<<<<<<<<<<<<
 * ufunc_btdtria_ptr[2*1+1] = <void*>(<char*>"btdtria")
 * ufunc_btdtria_data[0] = &ufunc_btdtria_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_ptr[2]) = ((void *)cdfbet3_wrap);

  /* "scipy/special/_ufuncs.pyx":3798
 * ufunc_btdtria_ptr[2*0+1] = <void*>(<char*>"btdtria")
 * ufunc_btdtria_ptr[2*1] = <void*>_func_cdfbet3_wrap
 * ufunc_btdtria_ptr[2*1+1] = <void*>(<char*>"btdtria")             # <<<<<<<<<<<<<<
 * ufunc_btdtria_data[0] = &ufunc_btdtria_ptr[2*0]
 * ufunc_btdtria_data[1] = &ufunc_btdtria_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_ptr[3]) = ((void *)((char *)((char *)"btdtria")));

  /* "scipy/special/_ufuncs.pyx":3799
 * ufunc_btdtria_ptr[2*1] = <void*>_func_cdfbet3_wrap
 * ufunc_btdtria_ptr[2*1+1] = <void*>(<char*>"btdtria")
 * ufunc_btdtria_data[0] = &ufunc_btdtria_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_btdtria_data[1] = &ufunc_btdtria_ptr[2*1]
 * btdtria = np.PyUFunc_FromFuncAndData(ufunc_btdtria_loops, ufunc_btdtria_data, ufunc_btdtria_types, 2, 3, 1, 0, "btdtria", ufunc_btdtria_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3800
 * ufunc_btdtria_ptr[2*1+1] = <void*>(<char*>"btdtria")
 * ufunc_btdtria_data[0] = &ufunc_btdtria_ptr[2*0]
 * ufunc_btdtria_data[1] = &ufunc_btdtria_ptr[2*1]             # <<<<<<<<<<<<<<
 * btdtria = np.PyUFunc_FromFuncAndData(ufunc_btdtria_loops, ufunc_btdtria_data, ufunc_btdtria_types, 2, 3, 1, 0, "btdtria", ufunc_btdtria_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3801
 * ufunc_btdtria_data[0] = &ufunc_btdtria_ptr[2*0]
 * ufunc_btdtria_data[1] = &ufunc_btdtria_ptr[2*1]
 * btdtria = np.PyUFunc_FromFuncAndData(ufunc_btdtria_loops, ufunc_btdtria_data, ufunc_btdtria_types, 2, 3, 1, 0, "btdtria", ufunc_btdtria_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_btdtrib_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_types, 2, 3, 1, 0, ((char *)"btdtria"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtria_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3801, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_btdtria, __pyx_t_2) < 0) __PYX_ERR(1, 3801, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3807
 * cdef void *ufunc_btdtrib_data[2]
 * cdef char ufunc_btdtrib_types[8]
 * cdef char *ufunc_btdtrib_doc = (             # <<<<<<<<<<<<<<
 *     "btdtria(a, p, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_doc = ((char *)"btdtria(a, p, x)\n\nInverse of `btdtr` with respect to `b`.\n\nThis is the inverse of the beta cumulative distribution function, `btdtr`,\nconsidered as a function of `b`, returning the value of `b` for which\n`btdtr(a, b, x) = p`, or\n\n.. math::\n    p = \\int_0^x \\frac{\\Gamma(a + b)}{\\Gamma(a)\\Gamma(b)} t^{a-1} (1-t)^{b-1}\\,dt\n\nParameters\n----------\na : array_like\n    Shape parameter (`a` > 0).\np : array_like\n    Cumulative probability, in [0, 1].\nx : array_like\n    The quantile, in [0, 1].\n\nReturns\n-------\nb : ndarray\n    The value of the shape parameter `b` such that `btdtr(a, b, x) = p`.\n\nSee Also\n--------\nbtdtr : Cumulative distribution function of the beta distribution.\nbtdtri : Inverse with respect to `x`.\nbtdtria : Inverse with respect to `a`.\n\nNotes\n-----\nWrapper for the CDFLIB [1]_ Fortran routine `cdfbet`.\n\nThe cumulative distribution function `p` is computed using a routine by\nDiDinato and Morris [2]_.  Computation of `b` involves a search for a value\nthat produces the desired value of `p`.  The search relies on the\nmonotonicity of `p` with `b`.\n\nReferences\n----------\n.. [1] Barry Brown, James Lovato, and Kathy Russell,\n       CDFLIB: Library of Fortran Routines for Cumulative Distribution\n       Functions, Inverses, and Other Parameters.\n.. [2] DiDinato, A. R. and Morris, A. H.,\n       Algorithm 708: Significant Digit Computation of the Incomplete Beta\n       Function Ratios. ACM Trans. Math. Softw. 18 (1993), 360-373.");

  /* "scipy/special/_ufuncs.pyx":3856
 *     "       Algorithm 708: Significant Digit Computation of the Incomplete Beta\n"
 *     "       Function Ratios. ACM Trans. Math. Softw. 18 (1993), 360-373.")
 * ufunc_btdtrib_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtrib_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":3857
 *     "       Function Ratios. ACM Trans. Math. Softw. 18 (1993), 360-373.")
 * ufunc_btdtrib_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_btdtrib_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_types[0] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":3858
 * ufunc_btdtrib_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_btdtrib_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtrib_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_types[1] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3859
 * ufunc_btdtrib_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_btdtrib_types[0] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_types[2] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3860
 * ufunc_btdtrib_types[0] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[1] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_types[3] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3861
 * ufunc_btdtrib_types[1] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[2] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtrib_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3862
 * ufunc_btdtrib_types[2] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[3] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtrib_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3863
 * ufunc_btdtrib_types[3] = <char>NPY_FLOAT
 * ufunc_btdtrib_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtrib_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtrib_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3864
 * ufunc_btdtrib_types[4] = <char>NPY_DOUBLE
 * ufunc_btdtrib_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtrib_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtrib_ptr[2*0] = <void*>_func_cdfbet4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3865
 * ufunc_btdtrib_types[5] = <char>NPY_DOUBLE
 * ufunc_btdtrib_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtrib_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_ptr[2*0] = <void*>_func_cdfbet4_wrap
 * ufunc_btdtrib_ptr[2*0+1] = <void*>(<char*>"btdtrib")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3866
 * ufunc_btdtrib_types[6] = <char>NPY_DOUBLE
 * ufunc_btdtrib_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtrib_ptr[2*0] = <void*>_func_cdfbet4_wrap             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_ptr[2*0+1] = <void*>(<char*>"btdtrib")
 * ufunc_btdtrib_ptr[2*1] = <void*>_func_cdfbet4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_ptr[0]) = ((void *)cdfbet4_wrap);

  /* "scipy/special/_ufuncs.pyx":3867
 * ufunc_btdtrib_types[7] = <char>NPY_DOUBLE
 * ufunc_btdtrib_ptr[2*0] = <void*>_func_cdfbet4_wrap
 * ufunc_btdtrib_ptr[2*0+1] = <void*>(<char*>"btdtrib")             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_ptr[2*1] = <void*>_func_cdfbet4_wrap
 * ufunc_btdtrib_ptr[2*1+1] = <void*>(<char*>"btdtrib")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_ptr[1]) = ((void *)((char *)((char *)"btdtrib")));

  /* "scipy/special/_ufuncs.pyx":3868
 * ufunc_btdtrib_ptr[2*0] = <void*>_func_cdfbet4_wrap
 * ufunc_btdtrib_ptr[2*0+1] = <void*>(<char*>"btdtrib")
 * ufunc_btdtrib_ptr[2*1] = <void*>_func_cdfbet4_wrap             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_ptr[2*1+1] = <void*>(<char*>"btdtrib")
 * ufunc_btdtrib_data[0] = &ufunc_btdtrib_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_ptr[2]) = ((void *)cdfbet4_wrap);

  /* "scipy/special/_ufuncs.pyx":3869
 * ufunc_btdtrib_ptr[2*0+1] = <void*>(<char*>"btdtrib")
 * ufunc_btdtrib_ptr[2*1] = <void*>_func_cdfbet4_wrap
 * ufunc_btdtrib_ptr[2*1+1] = <void*>(<char*>"btdtrib")             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_data[0] = &ufunc_btdtrib_ptr[2*0]
 * ufunc_btdtrib_data[1] = &ufunc_btdtrib_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_ptr[3]) = ((void *)((char *)((char *)"btdtrib")));

  /* "scipy/special/_ufuncs.pyx":3870
 * ufunc_btdtrib_ptr[2*1] = <void*>_func_cdfbet4_wrap
 * ufunc_btdtrib_ptr[2*1+1] = <void*>(<char*>"btdtrib")
 * ufunc_btdtrib_data[0] = &ufunc_btdtrib_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_btdtrib_data[1] = &ufunc_btdtrib_ptr[2*1]
 * btdtrib = np.PyUFunc_FromFuncAndData(ufunc_btdtrib_loops, ufunc_btdtrib_data, ufunc_btdtrib_types, 2, 3, 1, 0, "btdtrib", ufunc_btdtrib_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3871
 * ufunc_btdtrib_ptr[2*1+1] = <void*>(<char*>"btdtrib")
 * ufunc_btdtrib_data[0] = &ufunc_btdtrib_ptr[2*0]
 * ufunc_btdtrib_data[1] = &ufunc_btdtrib_ptr[2*1]             # <<<<<<<<<<<<<<
 * btdtrib = np.PyUFunc_FromFuncAndData(ufunc_btdtrib_loops, ufunc_btdtrib_data, ufunc_btdtrib_types, 2, 3, 1, 0, "btdtrib", ufunc_btdtrib_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3872
 * ufunc_btdtrib_data[0] = &ufunc_btdtrib_ptr[2*0]
 * ufunc_btdtrib_data[1] = &ufunc_btdtrib_ptr[2*1]
 * btdtrib = np.PyUFunc_FromFuncAndData(ufunc_btdtrib_loops, ufunc_btdtrib_data, ufunc_btdtrib_types, 2, 3, 1, 0, "btdtrib", ufunc_btdtrib_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_cbrt_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_types, 2, 3, 1, 0, ((char *)"btdtrib"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_btdtrib_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3872, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_btdtrib, __pyx_t_2) < 0) __PYX_ERR(1, 3872, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3878
 * cdef void *ufunc_cbrt_data[2]
 * cdef char ufunc_cbrt_types[4]
 * cdef char *ufunc_cbrt_doc = (             # <<<<<<<<<<<<<<
 *     "cbrt(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_doc = ((char *)"cbrt(x)\n\nElement-wise cube root of `x`.\n\nParameters\n----------\nx : array_like\n    `x` must contain real numbers.\n\nReturns\n-------\nfloat\n    The cube root of each value in `x`.\n\nExamples\n--------\n>>> from scipy.special import cbrt\n\n>>> cbrt(8)\n2.0\n>>> cbrt([-8, -3, 0.125, 1.331])\narray([-2.        , -1.44224957,  0.5       ,  1.1       ])");

  /* "scipy/special/_ufuncs.pyx":3901
 *     ">>> cbrt([-8, -3, 0.125, 1.331])\n"
 *     "array([-2.        , -1.44224957,  0.5       ,  1.1       ])")
 * ufunc_cbrt_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_cbrt_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cbrt_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":3902
 *     "array([-2.        , -1.44224957,  0.5       ,  1.1       ])")
 * ufunc_cbrt_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_cbrt_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_cbrt_types[0] = <char>NPY_FLOAT
 * ufunc_cbrt_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":3903
 * ufunc_cbrt_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_cbrt_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cbrt_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_cbrt_types[1] = <char>NPY_FLOAT
 * ufunc_cbrt_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3904
 * ufunc_cbrt_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cbrt_types[0] = <char>NPY_FLOAT
 * ufunc_cbrt_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_cbrt_types[2] = <char>NPY_DOUBLE
 * ufunc_cbrt_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3905
 * ufunc_cbrt_types[0] = <char>NPY_FLOAT
 * ufunc_cbrt_types[1] = <char>NPY_FLOAT
 * ufunc_cbrt_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_cbrt_types[3] = <char>NPY_DOUBLE
 * ufunc_cbrt_ptr[2*0] = <void*>_func_cbrt
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3906
 * ufunc_cbrt_types[1] = <char>NPY_FLOAT
 * ufunc_cbrt_types[2] = <char>NPY_DOUBLE
 * ufunc_cbrt_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_cbrt_ptr[2*0] = <void*>_func_cbrt
 * ufunc_cbrt_ptr[2*0+1] = <void*>(<char*>"cbrt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3907
 * ufunc_cbrt_types[2] = <char>NPY_DOUBLE
 * ufunc_cbrt_types[3] = <char>NPY_DOUBLE
 * ufunc_cbrt_ptr[2*0] = <void*>_func_cbrt             # <<<<<<<<<<<<<<
 * ufunc_cbrt_ptr[2*0+1] = <void*>(<char*>"cbrt")
 * ufunc_cbrt_ptr[2*1] = <void*>_func_cbrt
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_ptr[0]) = ((void *)cbrt);

  /* "scipy/special/_ufuncs.pyx":3908
 * ufunc_cbrt_types[3] = <char>NPY_DOUBLE
 * ufunc_cbrt_ptr[2*0] = <void*>_func_cbrt
 * ufunc_cbrt_ptr[2*0+1] = <void*>(<char*>"cbrt")             # <<<<<<<<<<<<<<
 * ufunc_cbrt_ptr[2*1] = <void*>_func_cbrt
 * ufunc_cbrt_ptr[2*1+1] = <void*>(<char*>"cbrt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_ptr[1]) = ((void *)((char *)((char *)"cbrt")));

  /* "scipy/special/_ufuncs.pyx":3909
 * ufunc_cbrt_ptr[2*0] = <void*>_func_cbrt
 * ufunc_cbrt_ptr[2*0+1] = <void*>(<char*>"cbrt")
 * ufunc_cbrt_ptr[2*1] = <void*>_func_cbrt             # <<<<<<<<<<<<<<
 * ufunc_cbrt_ptr[2*1+1] = <void*>(<char*>"cbrt")
 * ufunc_cbrt_data[0] = &ufunc_cbrt_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_ptr[2]) = ((void *)cbrt);

  /* "scipy/special/_ufuncs.pyx":3910
 * ufunc_cbrt_ptr[2*0+1] = <void*>(<char*>"cbrt")
 * ufunc_cbrt_ptr[2*1] = <void*>_func_cbrt
 * ufunc_cbrt_ptr[2*1+1] = <void*>(<char*>"cbrt")             # <<<<<<<<<<<<<<
 * ufunc_cbrt_data[0] = &ufunc_cbrt_ptr[2*0]
 * ufunc_cbrt_data[1] = &ufunc_cbrt_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_ptr[3]) = ((void *)((char *)((char *)"cbrt")));

  /* "scipy/special/_ufuncs.pyx":3911
 * ufunc_cbrt_ptr[2*1] = <void*>_func_cbrt
 * ufunc_cbrt_ptr[2*1+1] = <void*>(<char*>"cbrt")
 * ufunc_cbrt_data[0] = &ufunc_cbrt_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_cbrt_data[1] = &ufunc_cbrt_ptr[2*1]
 * cbrt = np.PyUFunc_FromFuncAndData(ufunc_cbrt_loops, ufunc_cbrt_data, ufunc_cbrt_types, 2, 1, 1, 0, "cbrt", ufunc_cbrt_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3912
 * ufunc_cbrt_ptr[2*1+1] = <void*>(<char*>"cbrt")
 * ufunc_cbrt_data[0] = &ufunc_cbrt_ptr[2*0]
 * ufunc_cbrt_data[1] = &ufunc_cbrt_ptr[2*1]             # <<<<<<<<<<<<<<
 * cbrt = np.PyUFunc_FromFuncAndData(ufunc_cbrt_loops, ufunc_cbrt_data, ufunc_cbrt_types, 2, 1, 1, 0, "cbrt", ufunc_cbrt_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3913
 * ufunc_cbrt_data[0] = &ufunc_cbrt_ptr[2*0]
 * ufunc_cbrt_data[1] = &ufunc_cbrt_ptr[2*1]
 * cbrt = np.PyUFunc_FromFuncAndData(ufunc_cbrt_loops, ufunc_cbrt_data, ufunc_cbrt_types, 2, 1, 1, 0, "cbrt", ufunc_cbrt_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_chdtr_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_types, 2, 1, 1, 0, ((char *)"cbrt"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_cbrt_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3913, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_cbrt, __pyx_t_2) < 0) __PYX_ERR(1, 3913, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3919
 * cdef void *ufunc_chdtr_data[2]
 * cdef char ufunc_chdtr_types[6]
 * cdef char *ufunc_chdtr_doc = (             # <<<<<<<<<<<<<<
 *     "chdtr(v, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_doc = ((char *)"chdtr(v, x)\n\nChi square cumulative distribution function\n\nReturns the area under the left hand tail (from 0 to `x`) of the Chi\nsquare probability density function with `v` degrees of freedom::\n\n    1/(2**(v/2) * gamma(v/2)) * integral(t**(v/2-1) * exp(-t/2), t=0..x)");

  /* "scipy/special/_ufuncs.pyx":3928
 *     "\n"
 *     "    1/(2**(v/2) * gamma(v/2)) * integral(t**(v/2-1) * exp(-t/2), t=0..x)")
 * ufunc_chdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_chdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":3929
 *     "    1/(2**(v/2) * gamma(v/2)) * integral(t**(v/2-1) * exp(-t/2), t=0..x)")
 * ufunc_chdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_chdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_chdtr_types[0] = <char>NPY_FLOAT
 * ufunc_chdtr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":3930
 * ufunc_chdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_chdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtr_types[1] = <char>NPY_FLOAT
 * ufunc_chdtr_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3931
 * ufunc_chdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtr_types[0] = <char>NPY_FLOAT
 * ufunc_chdtr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtr_types[2] = <char>NPY_FLOAT
 * ufunc_chdtr_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3932
 * ufunc_chdtr_types[0] = <char>NPY_FLOAT
 * ufunc_chdtr_types[1] = <char>NPY_FLOAT
 * ufunc_chdtr_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtr_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3933
 * ufunc_chdtr_types[1] = <char>NPY_FLOAT
 * ufunc_chdtr_types[2] = <char>NPY_FLOAT
 * ufunc_chdtr_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtr_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3934
 * ufunc_chdtr_types[2] = <char>NPY_FLOAT
 * ufunc_chdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtr_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtr_ptr[2*0] = <void*>_func_chdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3935
 * ufunc_chdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtr_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtr_ptr[2*0] = <void*>_func_chdtr
 * ufunc_chdtr_ptr[2*0+1] = <void*>(<char*>"chdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3936
 * ufunc_chdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtr_ptr[2*0] = <void*>_func_chdtr             # <<<<<<<<<<<<<<
 * ufunc_chdtr_ptr[2*0+1] = <void*>(<char*>"chdtr")
 * ufunc_chdtr_ptr[2*1] = <void*>_func_chdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_ptr[0]) = ((void *)chdtr);

  /* "scipy/special/_ufuncs.pyx":3937
 * ufunc_chdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtr_ptr[2*0] = <void*>_func_chdtr
 * ufunc_chdtr_ptr[2*0+1] = <void*>(<char*>"chdtr")             # <<<<<<<<<<<<<<
 * ufunc_chdtr_ptr[2*1] = <void*>_func_chdtr
 * ufunc_chdtr_ptr[2*1+1] = <void*>(<char*>"chdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_ptr[1]) = ((void *)((char *)((char *)"chdtr")));

  /* "scipy/special/_ufuncs.pyx":3938
 * ufunc_chdtr_ptr[2*0] = <void*>_func_chdtr
 * ufunc_chdtr_ptr[2*0+1] = <void*>(<char*>"chdtr")
 * ufunc_chdtr_ptr[2*1] = <void*>_func_chdtr             # <<<<<<<<<<<<<<
 * ufunc_chdtr_ptr[2*1+1] = <void*>(<char*>"chdtr")
 * ufunc_chdtr_data[0] = &ufunc_chdtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_ptr[2]) = ((void *)chdtr);

  /* "scipy/special/_ufuncs.pyx":3939
 * ufunc_chdtr_ptr[2*0+1] = <void*>(<char*>"chdtr")
 * ufunc_chdtr_ptr[2*1] = <void*>_func_chdtr
 * ufunc_chdtr_ptr[2*1+1] = <void*>(<char*>"chdtr")             # <<<<<<<<<<<<<<
 * ufunc_chdtr_data[0] = &ufunc_chdtr_ptr[2*0]
 * ufunc_chdtr_data[1] = &ufunc_chdtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_ptr[3]) = ((void *)((char *)((char *)"chdtr")));

  /* "scipy/special/_ufuncs.pyx":3940
 * ufunc_chdtr_ptr[2*1] = <void*>_func_chdtr
 * ufunc_chdtr_ptr[2*1+1] = <void*>(<char*>"chdtr")
 * ufunc_chdtr_data[0] = &ufunc_chdtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_chdtr_data[1] = &ufunc_chdtr_ptr[2*1]
 * chdtr = np.PyUFunc_FromFuncAndData(ufunc_chdtr_loops, ufunc_chdtr_data, ufunc_chdtr_types, 2, 2, 1, 0, "chdtr", ufunc_chdtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3941
 * ufunc_chdtr_ptr[2*1+1] = <void*>(<char*>"chdtr")
 * ufunc_chdtr_data[0] = &ufunc_chdtr_ptr[2*0]
 * ufunc_chdtr_data[1] = &ufunc_chdtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * chdtr = np.PyUFunc_FromFuncAndData(ufunc_chdtr_loops, ufunc_chdtr_data, ufunc_chdtr_types, 2, 2, 1, 0, "chdtr", ufunc_chdtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3942
 * ufunc_chdtr_data[0] = &ufunc_chdtr_ptr[2*0]
 * ufunc_chdtr_data[1] = &ufunc_chdtr_ptr[2*1]
 * chdtr = np.PyUFunc_FromFuncAndData(ufunc_chdtr_loops, ufunc_chdtr_data, ufunc_chdtr_types, 2, 2, 1, 0, "chdtr", ufunc_chdtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_chdtrc_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_types, 2, 2, 1, 0, ((char *)"chdtr"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtr_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3942, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_chdtr, __pyx_t_2) < 0) __PYX_ERR(1, 3942, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3948
 * cdef void *ufunc_chdtrc_data[2]
 * cdef char ufunc_chdtrc_types[6]
 * cdef char *ufunc_chdtrc_doc = (             # <<<<<<<<<<<<<<
 *     "chdtrc(v, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_doc = ((char *)"chdtrc(v, x)\n\nChi square survival function\n\nReturns the area under the right hand tail (from `x` to\ninfinity) of the Chi square probability density function with `v`\ndegrees of freedom::\n\n    1/(2**(v/2) * gamma(v/2)) * integral(t**(v/2-1) * exp(-t/2), t=x..inf)");

  /* "scipy/special/_ufuncs.pyx":3958
 *     "\n"
 *     "    1/(2**(v/2) * gamma(v/2)) * integral(t**(v/2-1) * exp(-t/2), t=x..inf)")
 * ufunc_chdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtrc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":3959
 *     "    1/(2**(v/2) * gamma(v/2)) * integral(t**(v/2-1) * exp(-t/2), t=x..inf)")
 * ufunc_chdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_chdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_types[0] = <char>NPY_FLOAT
 * ufunc_chdtrc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":3960
 * ufunc_chdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_chdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtrc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_types[1] = <char>NPY_FLOAT
 * ufunc_chdtrc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3961
 * ufunc_chdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtrc_types[0] = <char>NPY_FLOAT
 * ufunc_chdtrc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_types[2] = <char>NPY_FLOAT
 * ufunc_chdtrc_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3962
 * ufunc_chdtrc_types[0] = <char>NPY_FLOAT
 * ufunc_chdtrc_types[1] = <char>NPY_FLOAT
 * ufunc_chdtrc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtrc_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3963
 * ufunc_chdtrc_types[1] = <char>NPY_FLOAT
 * ufunc_chdtrc_types[2] = <char>NPY_FLOAT
 * ufunc_chdtrc_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtrc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3964
 * ufunc_chdtrc_types[2] = <char>NPY_FLOAT
 * ufunc_chdtrc_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtrc_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtrc_ptr[2*0] = <void*>_func_chdtrc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3965
 * ufunc_chdtrc_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtrc_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtrc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_ptr[2*0] = <void*>_func_chdtrc
 * ufunc_chdtrc_ptr[2*0+1] = <void*>(<char*>"chdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3966
 * ufunc_chdtrc_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtrc_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtrc_ptr[2*0] = <void*>_func_chdtrc             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_ptr[2*0+1] = <void*>(<char*>"chdtrc")
 * ufunc_chdtrc_ptr[2*1] = <void*>_func_chdtrc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_ptr[0]) = ((void *)chdtrc);

  /* "scipy/special/_ufuncs.pyx":3967
 * ufunc_chdtrc_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtrc_ptr[2*0] = <void*>_func_chdtrc
 * ufunc_chdtrc_ptr[2*0+1] = <void*>(<char*>"chdtrc")             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_ptr[2*1] = <void*>_func_chdtrc
 * ufunc_chdtrc_ptr[2*1+1] = <void*>(<char*>"chdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_ptr[1]) = ((void *)((char *)((char *)"chdtrc")));

  /* "scipy/special/_ufuncs.pyx":3968
 * ufunc_chdtrc_ptr[2*0] = <void*>_func_chdtrc
 * ufunc_chdtrc_ptr[2*0+1] = <void*>(<char*>"chdtrc")
 * ufunc_chdtrc_ptr[2*1] = <void*>_func_chdtrc             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_ptr[2*1+1] = <void*>(<char*>"chdtrc")
 * ufunc_chdtrc_data[0] = &ufunc_chdtrc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_ptr[2]) = ((void *)chdtrc);

  /* "scipy/special/_ufuncs.pyx":3969
 * ufunc_chdtrc_ptr[2*0+1] = <void*>(<char*>"chdtrc")
 * ufunc_chdtrc_ptr[2*1] = <void*>_func_chdtrc
 * ufunc_chdtrc_ptr[2*1+1] = <void*>(<char*>"chdtrc")             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_data[0] = &ufunc_chdtrc_ptr[2*0]
 * ufunc_chdtrc_data[1] = &ufunc_chdtrc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_ptr[3]) = ((void *)((char *)((char *)"chdtrc")));

  /* "scipy/special/_ufuncs.pyx":3970
 * ufunc_chdtrc_ptr[2*1] = <void*>_func_chdtrc
 * ufunc_chdtrc_ptr[2*1+1] = <void*>(<char*>"chdtrc")
 * ufunc_chdtrc_data[0] = &ufunc_chdtrc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_chdtrc_data[1] = &ufunc_chdtrc_ptr[2*1]
 * chdtrc = np.PyUFunc_FromFuncAndData(ufunc_chdtrc_loops, ufunc_chdtrc_data, ufunc_chdtrc_types, 2, 2, 1, 0, "chdtrc", ufunc_chdtrc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3971
 * ufunc_chdtrc_ptr[2*1+1] = <void*>(<char*>"chdtrc")
 * ufunc_chdtrc_data[0] = &ufunc_chdtrc_ptr[2*0]
 * ufunc_chdtrc_data[1] = &ufunc_chdtrc_ptr[2*1]             # <<<<<<<<<<<<<<
 * chdtrc = np.PyUFunc_FromFuncAndData(ufunc_chdtrc_loops, ufunc_chdtrc_data, ufunc_chdtrc_types, 2, 2, 1, 0, "chdtrc", ufunc_chdtrc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3972
 * ufunc_chdtrc_data[0] = &ufunc_chdtrc_ptr[2*0]
 * ufunc_chdtrc_data[1] = &ufunc_chdtrc_ptr[2*1]
 * chdtrc = np.PyUFunc_FromFuncAndData(ufunc_chdtrc_loops, ufunc_chdtrc_data, ufunc_chdtrc_types, 2, 2, 1, 0, "chdtrc", ufunc_chdtrc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_chdtri_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_types, 2, 2, 1, 0, ((char *)"chdtrc"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtrc_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3972, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_chdtrc, __pyx_t_2) < 0) __PYX_ERR(1, 3972, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":3978
 * cdef void *ufunc_chdtri_data[2]
 * cdef char ufunc_chdtri_types[6]
 * cdef char *ufunc_chdtri_doc = (             # <<<<<<<<<<<<<<
 *     "chdtri(v, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_doc = ((char *)"chdtri(v, p)\n\nInverse to `chdtrc`\n\nReturns the argument x such that ``chdtrc(v, x) == p``.");

  /* "scipy/special/_ufuncs.pyx":3984
 *     "\n"
 *     "Returns the argument x such that ``chdtrc(v, x) == p``.")
 * ufunc_chdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_chdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtri_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":3985
 *     "Returns the argument x such that ``chdtrc(v, x) == p``.")
 * ufunc_chdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_chdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_chdtri_types[0] = <char>NPY_FLOAT
 * ufunc_chdtri_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":3986
 * ufunc_chdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_chdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtri_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtri_types[1] = <char>NPY_FLOAT
 * ufunc_chdtri_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3987
 * ufunc_chdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtri_types[0] = <char>NPY_FLOAT
 * ufunc_chdtri_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtri_types[2] = <char>NPY_FLOAT
 * ufunc_chdtri_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3988
 * ufunc_chdtri_types[0] = <char>NPY_FLOAT
 * ufunc_chdtri_types[1] = <char>NPY_FLOAT
 * ufunc_chdtri_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtri_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtri_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":3989
 * ufunc_chdtri_types[1] = <char>NPY_FLOAT
 * ufunc_chdtri_types[2] = <char>NPY_FLOAT
 * ufunc_chdtri_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtri_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtri_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3990
 * ufunc_chdtri_types[2] = <char>NPY_FLOAT
 * ufunc_chdtri_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtri_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtri_ptr[2*0] = <void*>_func_chdtri
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3991
 * ufunc_chdtri_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtri_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtri_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtri_ptr[2*0] = <void*>_func_chdtri
 * ufunc_chdtri_ptr[2*0+1] = <void*>(<char*>"chdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":3992
 * ufunc_chdtri_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtri_ptr[2*0] = <void*>_func_chdtri             # <<<<<<<<<<<<<<
 * ufunc_chdtri_ptr[2*0+1] = <void*>(<char*>"chdtri")
 * ufunc_chdtri_ptr[2*1] = <void*>_func_chdtri
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_ptr[0]) = ((void *)chdtri);

  /* "scipy/special/_ufuncs.pyx":3993
 * ufunc_chdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtri_ptr[2*0] = <void*>_func_chdtri
 * ufunc_chdtri_ptr[2*0+1] = <void*>(<char*>"chdtri")             # <<<<<<<<<<<<<<
 * ufunc_chdtri_ptr[2*1] = <void*>_func_chdtri
 * ufunc_chdtri_ptr[2*1+1] = <void*>(<char*>"chdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_ptr[1]) = ((void *)((char *)((char *)"chdtri")));

  /* "scipy/special/_ufuncs.pyx":3994
 * ufunc_chdtri_ptr[2*0] = <void*>_func_chdtri
 * ufunc_chdtri_ptr[2*0+1] = <void*>(<char*>"chdtri")
 * ufunc_chdtri_ptr[2*1] = <void*>_func_chdtri             # <<<<<<<<<<<<<<
 * ufunc_chdtri_ptr[2*1+1] = <void*>(<char*>"chdtri")
 * ufunc_chdtri_data[0] = &ufunc_chdtri_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_ptr[2]) = ((void *)chdtri);

  /* "scipy/special/_ufuncs.pyx":3995
 * ufunc_chdtri_ptr[2*0+1] = <void*>(<char*>"chdtri")
 * ufunc_chdtri_ptr[2*1] = <void*>_func_chdtri
 * ufunc_chdtri_ptr[2*1+1] = <void*>(<char*>"chdtri")             # <<<<<<<<<<<<<<
 * ufunc_chdtri_data[0] = &ufunc_chdtri_ptr[2*0]
 * ufunc_chdtri_data[1] = &ufunc_chdtri_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_ptr[3]) = ((void *)((char *)((char *)"chdtri")));

  /* "scipy/special/_ufuncs.pyx":3996
 * ufunc_chdtri_ptr[2*1] = <void*>_func_chdtri
 * ufunc_chdtri_ptr[2*1+1] = <void*>(<char*>"chdtri")
 * ufunc_chdtri_data[0] = &ufunc_chdtri_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_chdtri_data[1] = &ufunc_chdtri_ptr[2*1]
 * chdtri = np.PyUFunc_FromFuncAndData(ufunc_chdtri_loops, ufunc_chdtri_data, ufunc_chdtri_types, 2, 2, 1, 0, "chdtri", ufunc_chdtri_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":3997
 * ufunc_chdtri_ptr[2*1+1] = <void*>(<char*>"chdtri")
 * ufunc_chdtri_data[0] = &ufunc_chdtri_ptr[2*0]
 * ufunc_chdtri_data[1] = &ufunc_chdtri_ptr[2*1]             # <<<<<<<<<<<<<<
 * chdtri = np.PyUFunc_FromFuncAndData(ufunc_chdtri_loops, ufunc_chdtri_data, ufunc_chdtri_types, 2, 2, 1, 0, "chdtri", ufunc_chdtri_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":3998
 * ufunc_chdtri_data[0] = &ufunc_chdtri_ptr[2*0]
 * ufunc_chdtri_data[1] = &ufunc_chdtri_ptr[2*1]
 * chdtri = np.PyUFunc_FromFuncAndData(ufunc_chdtri_loops, ufunc_chdtri_data, ufunc_chdtri_types, 2, 2, 1, 0, "chdtri", ufunc_chdtri_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_chdtriv_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_types, 2, 2, 1, 0, ((char *)"chdtri"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtri_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3998, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_chdtri, __pyx_t_2) < 0) __PYX_ERR(1, 3998, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4004
 * cdef void *ufunc_chdtriv_data[2]
 * cdef char ufunc_chdtriv_types[6]
 * cdef char *ufunc_chdtriv_doc = (             # <<<<<<<<<<<<<<
 *     "chdtriv(p, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_doc = ((char *)"chdtriv(p, x)\n\nInverse to `chdtr` vs `v`\n\nReturns the argument v such that ``chdtr(v, x) == p``.");

  /* "scipy/special/_ufuncs.pyx":4010
 *     "\n"
 *     "Returns the argument v such that ``chdtr(v, x) == p``.")
 * ufunc_chdtriv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtriv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":4011
 *     "Returns the argument v such that ``chdtr(v, x) == p``.")
 * ufunc_chdtriv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_chdtriv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_types[0] = <char>NPY_FLOAT
 * ufunc_chdtriv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":4012
 * ufunc_chdtriv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_chdtriv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtriv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_types[1] = <char>NPY_FLOAT
 * ufunc_chdtriv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4013
 * ufunc_chdtriv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_chdtriv_types[0] = <char>NPY_FLOAT
 * ufunc_chdtriv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_types[2] = <char>NPY_FLOAT
 * ufunc_chdtriv_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4014
 * ufunc_chdtriv_types[0] = <char>NPY_FLOAT
 * ufunc_chdtriv_types[1] = <char>NPY_FLOAT
 * ufunc_chdtriv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtriv_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4015
 * ufunc_chdtriv_types[1] = <char>NPY_FLOAT
 * ufunc_chdtriv_types[2] = <char>NPY_FLOAT
 * ufunc_chdtriv_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtriv_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4016
 * ufunc_chdtriv_types[2] = <char>NPY_FLOAT
 * ufunc_chdtriv_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtriv_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtriv_ptr[2*0] = <void*>_func_cdfchi3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4017
 * ufunc_chdtriv_types[3] = <char>NPY_DOUBLE
 * ufunc_chdtriv_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtriv_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_ptr[2*0] = <void*>_func_cdfchi3_wrap
 * ufunc_chdtriv_ptr[2*0+1] = <void*>(<char*>"chdtriv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4018
 * ufunc_chdtriv_types[4] = <char>NPY_DOUBLE
 * ufunc_chdtriv_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtriv_ptr[2*0] = <void*>_func_cdfchi3_wrap             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_ptr[2*0+1] = <void*>(<char*>"chdtriv")
 * ufunc_chdtriv_ptr[2*1] = <void*>_func_cdfchi3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_ptr[0]) = ((void *)cdfchi3_wrap);

  /* "scipy/special/_ufuncs.pyx":4019
 * ufunc_chdtriv_types[5] = <char>NPY_DOUBLE
 * ufunc_chdtriv_ptr[2*0] = <void*>_func_cdfchi3_wrap
 * ufunc_chdtriv_ptr[2*0+1] = <void*>(<char*>"chdtriv")             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_ptr[2*1] = <void*>_func_cdfchi3_wrap
 * ufunc_chdtriv_ptr[2*1+1] = <void*>(<char*>"chdtriv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_ptr[1]) = ((void *)((char *)((char *)"chdtriv")));

  /* "scipy/special/_ufuncs.pyx":4020
 * ufunc_chdtriv_ptr[2*0] = <void*>_func_cdfchi3_wrap
 * ufunc_chdtriv_ptr[2*0+1] = <void*>(<char*>"chdtriv")
 * ufunc_chdtriv_ptr[2*1] = <void*>_func_cdfchi3_wrap             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_ptr[2*1+1] = <void*>(<char*>"chdtriv")
 * ufunc_chdtriv_data[0] = &ufunc_chdtriv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_ptr[2]) = ((void *)cdfchi3_wrap);

  /* "scipy/special/_ufuncs.pyx":4021
 * ufunc_chdtriv_ptr[2*0+1] = <void*>(<char*>"chdtriv")
 * ufunc_chdtriv_ptr[2*1] = <void*>_func_cdfchi3_wrap
 * ufunc_chdtriv_ptr[2*1+1] = <void*>(<char*>"chdtriv")             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_data[0] = &ufunc_chdtriv_ptr[2*0]
 * ufunc_chdtriv_data[1] = &ufunc_chdtriv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_ptr[3]) = ((void *)((char *)((char *)"chdtriv")));

  /* "scipy/special/_ufuncs.pyx":4022
 * ufunc_chdtriv_ptr[2*1] = <void*>_func_cdfchi3_wrap
 * ufunc_chdtriv_ptr[2*1+1] = <void*>(<char*>"chdtriv")
 * ufunc_chdtriv_data[0] = &ufunc_chdtriv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_chdtriv_data[1] = &ufunc_chdtriv_ptr[2*1]
 * chdtriv = np.PyUFunc_FromFuncAndData(ufunc_chdtriv_loops, ufunc_chdtriv_data, ufunc_chdtriv_types, 2, 2, 1, 0, "chdtriv", ufunc_chdtriv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4023
 * ufunc_chdtriv_ptr[2*1+1] = <void*>(<char*>"chdtriv")
 * ufunc_chdtriv_data[0] = &ufunc_chdtriv_ptr[2*0]
 * ufunc_chdtriv_data[1] = &ufunc_chdtriv_ptr[2*1]             # <<<<<<<<<<<<<<
 * chdtriv = np.PyUFunc_FromFuncAndData(ufunc_chdtriv_loops, ufunc_chdtriv_data, ufunc_chdtriv_types, 2, 2, 1, 0, "chdtriv", ufunc_chdtriv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4024
 * ufunc_chdtriv_data[0] = &ufunc_chdtriv_ptr[2*0]
 * ufunc_chdtriv_data[1] = &ufunc_chdtriv_ptr[2*1]
 * chdtriv = np.PyUFunc_FromFuncAndData(ufunc_chdtriv_loops, ufunc_chdtriv_data, ufunc_chdtriv_types, 2, 2, 1, 0, "chdtriv", ufunc_chdtriv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_chndtr_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_types, 2, 2, 1, 0, ((char *)"chdtriv"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_chdtriv_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4024, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_chdtriv, __pyx_t_2) < 0) __PYX_ERR(1, 4024, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4030
 * cdef void *ufunc_chndtr_data[2]
 * cdef char ufunc_chndtr_types[8]
 * cdef char *ufunc_chndtr_doc = (             # <<<<<<<<<<<<<<
 *     "chndtr(x, df, nc)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_doc = ((char *)"chndtr(x, df, nc)\n\nNon-central chi square cumulative distribution function");

  /* "scipy/special/_ufuncs.pyx":4034
 *     "\n"
 *     "Non-central chi square cumulative distribution function")
 * ufunc_chndtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_chndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":4035
 *     "Non-central chi square cumulative distribution function")
 * ufunc_chndtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_chndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_chndtr_types[0] = <char>NPY_FLOAT
 * ufunc_chndtr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":4036
 * ufunc_chndtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_chndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtr_types[1] = <char>NPY_FLOAT
 * ufunc_chndtr_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4037
 * ufunc_chndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtr_types[0] = <char>NPY_FLOAT
 * ufunc_chndtr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtr_types[2] = <char>NPY_FLOAT
 * ufunc_chndtr_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4038
 * ufunc_chndtr_types[0] = <char>NPY_FLOAT
 * ufunc_chndtr_types[1] = <char>NPY_FLOAT
 * ufunc_chndtr_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtr_types[3] = <char>NPY_FLOAT
 * ufunc_chndtr_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4039
 * ufunc_chndtr_types[1] = <char>NPY_FLOAT
 * ufunc_chndtr_types[2] = <char>NPY_FLOAT
 * ufunc_chndtr_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtr_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtr_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4040
 * ufunc_chndtr_types[2] = <char>NPY_FLOAT
 * ufunc_chndtr_types[3] = <char>NPY_FLOAT
 * ufunc_chndtr_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtr_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtr_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4041
 * ufunc_chndtr_types[3] = <char>NPY_FLOAT
 * ufunc_chndtr_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtr_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtr_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtr_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4042
 * ufunc_chndtr_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtr_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtr_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtr_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtr_ptr[2*0] = <void*>_func_cdfchn1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4043
 * ufunc_chndtr_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtr_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtr_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtr_ptr[2*0] = <void*>_func_cdfchn1_wrap
 * ufunc_chndtr_ptr[2*0+1] = <void*>(<char*>"chndtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4044
 * ufunc_chndtr_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtr_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtr_ptr[2*0] = <void*>_func_cdfchn1_wrap             # <<<<<<<<<<<<<<
 * ufunc_chndtr_ptr[2*0+1] = <void*>(<char*>"chndtr")
 * ufunc_chndtr_ptr[2*1] = <void*>_func_cdfchn1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_ptr[0]) = ((void *)cdfchn1_wrap);

  /* "scipy/special/_ufuncs.pyx":4045
 * ufunc_chndtr_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtr_ptr[2*0] = <void*>_func_cdfchn1_wrap
 * ufunc_chndtr_ptr[2*0+1] = <void*>(<char*>"chndtr")             # <<<<<<<<<<<<<<
 * ufunc_chndtr_ptr[2*1] = <void*>_func_cdfchn1_wrap
 * ufunc_chndtr_ptr[2*1+1] = <void*>(<char*>"chndtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_ptr[1]) = ((void *)((char *)((char *)"chndtr")));

  /* "scipy/special/_ufuncs.pyx":4046
 * ufunc_chndtr_ptr[2*0] = <void*>_func_cdfchn1_wrap
 * ufunc_chndtr_ptr[2*0+1] = <void*>(<char*>"chndtr")
 * ufunc_chndtr_ptr[2*1] = <void*>_func_cdfchn1_wrap             # <<<<<<<<<<<<<<
 * ufunc_chndtr_ptr[2*1+1] = <void*>(<char*>"chndtr")
 * ufunc_chndtr_data[0] = &ufunc_chndtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_ptr[2]) = ((void *)cdfchn1_wrap);

  /* "scipy/special/_ufuncs.pyx":4047
 * ufunc_chndtr_ptr[2*0+1] = <void*>(<char*>"chndtr")
 * ufunc_chndtr_ptr[2*1] = <void*>_func_cdfchn1_wrap
 * ufunc_chndtr_ptr[2*1+1] = <void*>(<char*>"chndtr")             # <<<<<<<<<<<<<<
 * ufunc_chndtr_data[0] = &ufunc_chndtr_ptr[2*0]
 * ufunc_chndtr_data[1] = &ufunc_chndtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_ptr[3]) = ((void *)((char *)((char *)"chndtr")));

  /* "scipy/special/_ufuncs.pyx":4048
 * ufunc_chndtr_ptr[2*1] = <void*>_func_cdfchn1_wrap
 * ufunc_chndtr_ptr[2*1+1] = <void*>(<char*>"chndtr")
 * ufunc_chndtr_data[0] = &ufunc_chndtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_chndtr_data[1] = &ufunc_chndtr_ptr[2*1]
 * chndtr = np.PyUFunc_FromFuncAndData(ufunc_chndtr_loops, ufunc_chndtr_data, ufunc_chndtr_types, 2, 3, 1, 0, "chndtr", ufunc_chndtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4049
 * ufunc_chndtr_ptr[2*1+1] = <void*>(<char*>"chndtr")
 * ufunc_chndtr_data[0] = &ufunc_chndtr_ptr[2*0]
 * ufunc_chndtr_data[1] = &ufunc_chndtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * chndtr = np.PyUFunc_FromFuncAndData(ufunc_chndtr_loops, ufunc_chndtr_data, ufunc_chndtr_types, 2, 3, 1, 0, "chndtr", ufunc_chndtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4050
 * ufunc_chndtr_data[0] = &ufunc_chndtr_ptr[2*0]
 * ufunc_chndtr_data[1] = &ufunc_chndtr_ptr[2*1]
 * chndtr = np.PyUFunc_FromFuncAndData(ufunc_chndtr_loops, ufunc_chndtr_data, ufunc_chndtr_types, 2, 3, 1, 0, "chndtr", ufunc_chndtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_chndtridf_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_types, 2, 3, 1, 0, ((char *)"chndtr"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtr_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4050, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_chndtr, __pyx_t_2) < 0) __PYX_ERR(1, 4050, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4056
 * cdef void *ufunc_chndtridf_data[2]
 * cdef char ufunc_chndtridf_types[8]
 * cdef char *ufunc_chndtridf_doc = (             # <<<<<<<<<<<<<<
 *     "chndtridf(x, p, nc)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_doc = ((char *)"chndtridf(x, p, nc)\n\nInverse to `chndtr` vs `df`");

  /* "scipy/special/_ufuncs.pyx":4060
 *     "\n"
 *     "Inverse to `chndtr` vs `df`")
 * ufunc_chndtridf_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtridf_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":4061
 *     "Inverse to `chndtr` vs `df`")
 * ufunc_chndtridf_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_chndtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_types[0] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":4062
 * ufunc_chndtridf_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_chndtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtridf_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_types[1] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4063
 * ufunc_chndtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtridf_types[0] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_types[2] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4064
 * ufunc_chndtridf_types[0] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[1] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_types[3] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4065
 * ufunc_chndtridf_types[1] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[2] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtridf_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4066
 * ufunc_chndtridf_types[2] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[3] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtridf_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4067
 * ufunc_chndtridf_types[3] = <char>NPY_FLOAT
 * ufunc_chndtridf_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtridf_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtridf_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4068
 * ufunc_chndtridf_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtridf_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtridf_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtridf_ptr[2*0] = <void*>_func_cdfchn3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4069
 * ufunc_chndtridf_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtridf_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtridf_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_ptr[2*0] = <void*>_func_cdfchn3_wrap
 * ufunc_chndtridf_ptr[2*0+1] = <void*>(<char*>"chndtridf")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4070
 * ufunc_chndtridf_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtridf_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtridf_ptr[2*0] = <void*>_func_cdfchn3_wrap             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_ptr[2*0+1] = <void*>(<char*>"chndtridf")
 * ufunc_chndtridf_ptr[2*1] = <void*>_func_cdfchn3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_ptr[0]) = ((void *)cdfchn3_wrap);

  /* "scipy/special/_ufuncs.pyx":4071
 * ufunc_chndtridf_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtridf_ptr[2*0] = <void*>_func_cdfchn3_wrap
 * ufunc_chndtridf_ptr[2*0+1] = <void*>(<char*>"chndtridf")             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_ptr[2*1] = <void*>_func_cdfchn3_wrap
 * ufunc_chndtridf_ptr[2*1+1] = <void*>(<char*>"chndtridf")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_ptr[1]) = ((void *)((char *)((char *)"chndtridf")));

  /* "scipy/special/_ufuncs.pyx":4072
 * ufunc_chndtridf_ptr[2*0] = <void*>_func_cdfchn3_wrap
 * ufunc_chndtridf_ptr[2*0+1] = <void*>(<char*>"chndtridf")
 * ufunc_chndtridf_ptr[2*1] = <void*>_func_cdfchn3_wrap             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_ptr[2*1+1] = <void*>(<char*>"chndtridf")
 * ufunc_chndtridf_data[0] = &ufunc_chndtridf_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_ptr[2]) = ((void *)cdfchn3_wrap);

  /* "scipy/special/_ufuncs.pyx":4073
 * ufunc_chndtridf_ptr[2*0+1] = <void*>(<char*>"chndtridf")
 * ufunc_chndtridf_ptr[2*1] = <void*>_func_cdfchn3_wrap
 * ufunc_chndtridf_ptr[2*1+1] = <void*>(<char*>"chndtridf")             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_data[0] = &ufunc_chndtridf_ptr[2*0]
 * ufunc_chndtridf_data[1] = &ufunc_chndtridf_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_ptr[3]) = ((void *)((char *)((char *)"chndtridf")));

  /* "scipy/special/_ufuncs.pyx":4074
 * ufunc_chndtridf_ptr[2*1] = <void*>_func_cdfchn3_wrap
 * ufunc_chndtridf_ptr[2*1+1] = <void*>(<char*>"chndtridf")
 * ufunc_chndtridf_data[0] = &ufunc_chndtridf_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_chndtridf_data[1] = &ufunc_chndtridf_ptr[2*1]
 * chndtridf = np.PyUFunc_FromFuncAndData(ufunc_chndtridf_loops, ufunc_chndtridf_data, ufunc_chndtridf_types, 2, 3, 1, 0, "chndtridf", ufunc_chndtridf_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4075
 * ufunc_chndtridf_ptr[2*1+1] = <void*>(<char*>"chndtridf")
 * ufunc_chndtridf_data[0] = &ufunc_chndtridf_ptr[2*0]
 * ufunc_chndtridf_data[1] = &ufunc_chndtridf_ptr[2*1]             # <<<<<<<<<<<<<<
 * chndtridf = np.PyUFunc_FromFuncAndData(ufunc_chndtridf_loops, ufunc_chndtridf_data, ufunc_chndtridf_types, 2, 3, 1, 0, "chndtridf", ufunc_chndtridf_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4076
 * ufunc_chndtridf_data[0] = &ufunc_chndtridf_ptr[2*0]
 * ufunc_chndtridf_data[1] = &ufunc_chndtridf_ptr[2*1]
 * chndtridf = np.PyUFunc_FromFuncAndData(ufunc_chndtridf_loops, ufunc_chndtridf_data, ufunc_chndtridf_types, 2, 3, 1, 0, "chndtridf", ufunc_chndtridf_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_chndtrinc_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_types, 2, 3, 1, 0, ((char *)"chndtridf"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtridf_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4076, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_chndtridf, __pyx_t_2) < 0) __PYX_ERR(1, 4076, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4082
 * cdef void *ufunc_chndtrinc_data[2]
 * cdef char ufunc_chndtrinc_types[8]
 * cdef char *ufunc_chndtrinc_doc = (             # <<<<<<<<<<<<<<
 *     "chndtrinc(x, df, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_doc = ((char *)"chndtrinc(x, df, p)\n\nInverse to `chndtr` vs `nc`");

  /* "scipy/special/_ufuncs.pyx":4086
 *     "\n"
 *     "Inverse to `chndtr` vs `nc`")
 * ufunc_chndtrinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtrinc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":4087
 *     "Inverse to `chndtr` vs `nc`")
 * ufunc_chndtrinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_chndtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_types[0] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":4088
 * ufunc_chndtrinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_chndtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtrinc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_types[1] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4089
 * ufunc_chndtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtrinc_types[0] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_types[2] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4090
 * ufunc_chndtrinc_types[0] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[1] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_types[3] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4091
 * ufunc_chndtrinc_types[1] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[2] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4092
 * ufunc_chndtrinc_types[2] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[3] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4093
 * ufunc_chndtrinc_types[3] = <char>NPY_FLOAT
 * ufunc_chndtrinc_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4094
 * ufunc_chndtrinc_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_ptr[2*0] = <void*>_func_cdfchn4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4095
 * ufunc_chndtrinc_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_ptr[2*0] = <void*>_func_cdfchn4_wrap
 * ufunc_chndtrinc_ptr[2*0+1] = <void*>(<char*>"chndtrinc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4096
 * ufunc_chndtrinc_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_ptr[2*0] = <void*>_func_cdfchn4_wrap             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_ptr[2*0+1] = <void*>(<char*>"chndtrinc")
 * ufunc_chndtrinc_ptr[2*1] = <void*>_func_cdfchn4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_ptr[0]) = ((void *)cdfchn4_wrap);

  /* "scipy/special/_ufuncs.pyx":4097
 * ufunc_chndtrinc_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtrinc_ptr[2*0] = <void*>_func_cdfchn4_wrap
 * ufunc_chndtrinc_ptr[2*0+1] = <void*>(<char*>"chndtrinc")             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_ptr[2*1] = <void*>_func_cdfchn4_wrap
 * ufunc_chndtrinc_ptr[2*1+1] = <void*>(<char*>"chndtrinc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_ptr[1]) = ((void *)((char *)((char *)"chndtrinc")));

  /* "scipy/special/_ufuncs.pyx":4098
 * ufunc_chndtrinc_ptr[2*0] = <void*>_func_cdfchn4_wrap
 * ufunc_chndtrinc_ptr[2*0+1] = <void*>(<char*>"chndtrinc")
 * ufunc_chndtrinc_ptr[2*1] = <void*>_func_cdfchn4_wrap             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_ptr[2*1+1] = <void*>(<char*>"chndtrinc")
 * ufunc_chndtrinc_data[0] = &ufunc_chndtrinc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_ptr[2]) = ((void *)cdfchn4_wrap);

  /* "scipy/special/_ufuncs.pyx":4099
 * ufunc_chndtrinc_ptr[2*0+1] = <void*>(<char*>"chndtrinc")
 * ufunc_chndtrinc_ptr[2*1] = <void*>_func_cdfchn4_wrap
 * ufunc_chndtrinc_ptr[2*1+1] = <void*>(<char*>"chndtrinc")             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_data[0] = &ufunc_chndtrinc_ptr[2*0]
 * ufunc_chndtrinc_data[1] = &ufunc_chndtrinc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_ptr[3]) = ((void *)((char *)((char *)"chndtrinc")));

  /* "scipy/special/_ufuncs.pyx":4100
 * ufunc_chndtrinc_ptr[2*1] = <void*>_func_cdfchn4_wrap
 * ufunc_chndtrinc_ptr[2*1+1] = <void*>(<char*>"chndtrinc")
 * ufunc_chndtrinc_data[0] = &ufunc_chndtrinc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_chndtrinc_data[1] = &ufunc_chndtrinc_ptr[2*1]
 * chndtrinc = np.PyUFunc_FromFuncAndData(ufunc_chndtrinc_loops, ufunc_chndtrinc_data, ufunc_chndtrinc_types, 2, 3, 1, 0, "chndtrinc", ufunc_chndtrinc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4101
 * ufunc_chndtrinc_ptr[2*1+1] = <void*>(<char*>"chndtrinc")
 * ufunc_chndtrinc_data[0] = &ufunc_chndtrinc_ptr[2*0]
 * ufunc_chndtrinc_data[1] = &ufunc_chndtrinc_ptr[2*1]             # <<<<<<<<<<<<<<
 * chndtrinc = np.PyUFunc_FromFuncAndData(ufunc_chndtrinc_loops, ufunc_chndtrinc_data, ufunc_chndtrinc_types, 2, 3, 1, 0, "chndtrinc", ufunc_chndtrinc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4102
 * ufunc_chndtrinc_data[0] = &ufunc_chndtrinc_ptr[2*0]
 * ufunc_chndtrinc_data[1] = &ufunc_chndtrinc_ptr[2*1]
 * chndtrinc = np.PyUFunc_FromFuncAndData(ufunc_chndtrinc_loops, ufunc_chndtrinc_data, ufunc_chndtrinc_types, 2, 3, 1, 0, "chndtrinc", ufunc_chndtrinc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_chndtrix_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_types, 2, 3, 1, 0, ((char *)"chndtrinc"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrinc_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_chndtrinc, __pyx_t_2) < 0) __PYX_ERR(1, 4102, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4108
 * cdef void *ufunc_chndtrix_data[2]
 * cdef char ufunc_chndtrix_types[8]
 * cdef char *ufunc_chndtrix_doc = (             # <<<<<<<<<<<<<<
 *     "chndtrix(p, df, nc)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_doc = ((char *)"chndtrix(p, df, nc)\n\nInverse to `chndtr` vs `x`");

  /* "scipy/special/_ufuncs.pyx":4112
 *     "\n"
 *     "Inverse to `chndtr` vs `x`")
 * ufunc_chndtrix_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtrix_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":4113
 *     "Inverse to `chndtr` vs `x`")
 * ufunc_chndtrix_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_chndtrix_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_types[0] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":4114
 * ufunc_chndtrix_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_chndtrix_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtrix_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_types[1] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4115
 * ufunc_chndtrix_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_chndtrix_types[0] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_types[2] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4116
 * ufunc_chndtrix_types[0] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[1] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_types[3] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4117
 * ufunc_chndtrix_types[1] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[2] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtrix_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4118
 * ufunc_chndtrix_types[2] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[3] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtrix_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4119
 * ufunc_chndtrix_types[3] = <char>NPY_FLOAT
 * ufunc_chndtrix_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtrix_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtrix_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4120
 * ufunc_chndtrix_types[4] = <char>NPY_DOUBLE
 * ufunc_chndtrix_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtrix_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtrix_ptr[2*0] = <void*>_func_cdfchn2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4121
 * ufunc_chndtrix_types[5] = <char>NPY_DOUBLE
 * ufunc_chndtrix_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtrix_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_ptr[2*0] = <void*>_func_cdfchn2_wrap
 * ufunc_chndtrix_ptr[2*0+1] = <void*>(<char*>"chndtrix")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4122
 * ufunc_chndtrix_types[6] = <char>NPY_DOUBLE
 * ufunc_chndtrix_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtrix_ptr[2*0] = <void*>_func_cdfchn2_wrap             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_ptr[2*0+1] = <void*>(<char*>"chndtrix")
 * ufunc_chndtrix_ptr[2*1] = <void*>_func_cdfchn2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_ptr[0]) = ((void *)cdfchn2_wrap);

  /* "scipy/special/_ufuncs.pyx":4123
 * ufunc_chndtrix_types[7] = <char>NPY_DOUBLE
 * ufunc_chndtrix_ptr[2*0] = <void*>_func_cdfchn2_wrap
 * ufunc_chndtrix_ptr[2*0+1] = <void*>(<char*>"chndtrix")             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_ptr[2*1] = <void*>_func_cdfchn2_wrap
 * ufunc_chndtrix_ptr[2*1+1] = <void*>(<char*>"chndtrix")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_ptr[1]) = ((void *)((char *)((char *)"chndtrix")));

  /* "scipy/special/_ufuncs.pyx":4124
 * ufunc_chndtrix_ptr[2*0] = <void*>_func_cdfchn2_wrap
 * ufunc_chndtrix_ptr[2*0+1] = <void*>(<char*>"chndtrix")
 * ufunc_chndtrix_ptr[2*1] = <void*>_func_cdfchn2_wrap             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_ptr[2*1+1] = <void*>(<char*>"chndtrix")
 * ufunc_chndtrix_data[0] = &ufunc_chndtrix_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_ptr[2]) = ((void *)cdfchn2_wrap);

  /* "scipy/special/_ufuncs.pyx":4125
 * ufunc_chndtrix_ptr[2*0+1] = <void*>(<char*>"chndtrix")
 * ufunc_chndtrix_ptr[2*1] = <void*>_func_cdfchn2_wrap
 * ufunc_chndtrix_ptr[2*1+1] = <void*>(<char*>"chndtrix")             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_data[0] = &ufunc_chndtrix_ptr[2*0]
 * ufunc_chndtrix_data[1] = &ufunc_chndtrix_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_ptr[3]) = ((void *)((char *)((char *)"chndtrix")));

  /* "scipy/special/_ufuncs.pyx":4126
 * ufunc_chndtrix_ptr[2*1] = <void*>_func_cdfchn2_wrap
 * ufunc_chndtrix_ptr[2*1+1] = <void*>(<char*>"chndtrix")
 * ufunc_chndtrix_data[0] = &ufunc_chndtrix_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_chndtrix_data[1] = &ufunc_chndtrix_ptr[2*1]
 * chndtrix = np.PyUFunc_FromFuncAndData(ufunc_chndtrix_loops, ufunc_chndtrix_data, ufunc_chndtrix_types, 2, 3, 1, 0, "chndtrix", ufunc_chndtrix_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4127
 * ufunc_chndtrix_ptr[2*1+1] = <void*>(<char*>"chndtrix")
 * ufunc_chndtrix_data[0] = &ufunc_chndtrix_ptr[2*0]
 * ufunc_chndtrix_data[1] = &ufunc_chndtrix_ptr[2*1]             # <<<<<<<<<<<<<<
 * chndtrix = np.PyUFunc_FromFuncAndData(ufunc_chndtrix_loops, ufunc_chndtrix_data, ufunc_chndtrix_types, 2, 3, 1, 0, "chndtrix", ufunc_chndtrix_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4128
 * ufunc_chndtrix_data[0] = &ufunc_chndtrix_ptr[2*0]
 * ufunc_chndtrix_data[1] = &ufunc_chndtrix_ptr[2*1]
 * chndtrix = np.PyUFunc_FromFuncAndData(ufunc_chndtrix_loops, ufunc_chndtrix_data, ufunc_chndtrix_types, 2, 3, 1, 0, "chndtrix", ufunc_chndtrix_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_cosdg_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_types, 2, 3, 1, 0, ((char *)"chndtrix"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_chndtrix_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_chndtrix, __pyx_t_2) < 0) __PYX_ERR(1, 4128, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4134
 * cdef void *ufunc_cosdg_data[2]
 * cdef char ufunc_cosdg_types[4]
 * cdef char *ufunc_cosdg_doc = (             # <<<<<<<<<<<<<<
 *     "cosdg(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_doc = ((char *)"cosdg(x)\n\nCosine of the angle `x` given in degrees.");

  /* "scipy/special/_ufuncs.pyx":4138
 *     "\n"
 *     "Cosine of the angle `x` given in degrees.")
 * ufunc_cosdg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_cosdg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cosdg_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4139
 *     "Cosine of the angle `x` given in degrees.")
 * ufunc_cosdg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_cosdg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_cosdg_types[0] = <char>NPY_FLOAT
 * ufunc_cosdg_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4140
 * ufunc_cosdg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_cosdg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cosdg_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_cosdg_types[1] = <char>NPY_FLOAT
 * ufunc_cosdg_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4141
 * ufunc_cosdg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cosdg_types[0] = <char>NPY_FLOAT
 * ufunc_cosdg_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_cosdg_types[2] = <char>NPY_DOUBLE
 * ufunc_cosdg_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4142
 * ufunc_cosdg_types[0] = <char>NPY_FLOAT
 * ufunc_cosdg_types[1] = <char>NPY_FLOAT
 * ufunc_cosdg_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_cosdg_types[3] = <char>NPY_DOUBLE
 * ufunc_cosdg_ptr[2*0] = <void*>_func_cosdg
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4143
 * ufunc_cosdg_types[1] = <char>NPY_FLOAT
 * ufunc_cosdg_types[2] = <char>NPY_DOUBLE
 * ufunc_cosdg_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_cosdg_ptr[2*0] = <void*>_func_cosdg
 * ufunc_cosdg_ptr[2*0+1] = <void*>(<char*>"cosdg")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4144
 * ufunc_cosdg_types[2] = <char>NPY_DOUBLE
 * ufunc_cosdg_types[3] = <char>NPY_DOUBLE
 * ufunc_cosdg_ptr[2*0] = <void*>_func_cosdg             # <<<<<<<<<<<<<<
 * ufunc_cosdg_ptr[2*0+1] = <void*>(<char*>"cosdg")
 * ufunc_cosdg_ptr[2*1] = <void*>_func_cosdg
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_ptr[0]) = ((void *)cosdg);

  /* "scipy/special/_ufuncs.pyx":4145
 * ufunc_cosdg_types[3] = <char>NPY_DOUBLE
 * ufunc_cosdg_ptr[2*0] = <void*>_func_cosdg
 * ufunc_cosdg_ptr[2*0+1] = <void*>(<char*>"cosdg")             # <<<<<<<<<<<<<<
 * ufunc_cosdg_ptr[2*1] = <void*>_func_cosdg
 * ufunc_cosdg_ptr[2*1+1] = <void*>(<char*>"cosdg")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_ptr[1]) = ((void *)((char *)((char *)"cosdg")));

  /* "scipy/special/_ufuncs.pyx":4146
 * ufunc_cosdg_ptr[2*0] = <void*>_func_cosdg
 * ufunc_cosdg_ptr[2*0+1] = <void*>(<char*>"cosdg")
 * ufunc_cosdg_ptr[2*1] = <void*>_func_cosdg             # <<<<<<<<<<<<<<
 * ufunc_cosdg_ptr[2*1+1] = <void*>(<char*>"cosdg")
 * ufunc_cosdg_data[0] = &ufunc_cosdg_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_ptr[2]) = ((void *)cosdg);

  /* "scipy/special/_ufuncs.pyx":4147
 * ufunc_cosdg_ptr[2*0+1] = <void*>(<char*>"cosdg")
 * ufunc_cosdg_ptr[2*1] = <void*>_func_cosdg
 * ufunc_cosdg_ptr[2*1+1] = <void*>(<char*>"cosdg")             # <<<<<<<<<<<<<<
 * ufunc_cosdg_data[0] = &ufunc_cosdg_ptr[2*0]
 * ufunc_cosdg_data[1] = &ufunc_cosdg_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_ptr[3]) = ((void *)((char *)((char *)"cosdg")));

  /* "scipy/special/_ufuncs.pyx":4148
 * ufunc_cosdg_ptr[2*1] = <void*>_func_cosdg
 * ufunc_cosdg_ptr[2*1+1] = <void*>(<char*>"cosdg")
 * ufunc_cosdg_data[0] = &ufunc_cosdg_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_cosdg_data[1] = &ufunc_cosdg_ptr[2*1]
 * cosdg = np.PyUFunc_FromFuncAndData(ufunc_cosdg_loops, ufunc_cosdg_data, ufunc_cosdg_types, 2, 1, 1, 0, "cosdg", ufunc_cosdg_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4149
 * ufunc_cosdg_ptr[2*1+1] = <void*>(<char*>"cosdg")
 * ufunc_cosdg_data[0] = &ufunc_cosdg_ptr[2*0]
 * ufunc_cosdg_data[1] = &ufunc_cosdg_ptr[2*1]             # <<<<<<<<<<<<<<
 * cosdg = np.PyUFunc_FromFuncAndData(ufunc_cosdg_loops, ufunc_cosdg_data, ufunc_cosdg_types, 2, 1, 1, 0, "cosdg", ufunc_cosdg_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4150
 * ufunc_cosdg_data[0] = &ufunc_cosdg_ptr[2*0]
 * ufunc_cosdg_data[1] = &ufunc_cosdg_ptr[2*1]
 * cosdg = np.PyUFunc_FromFuncAndData(ufunc_cosdg_loops, ufunc_cosdg_data, ufunc_cosdg_types, 2, 1, 1, 0, "cosdg", ufunc_cosdg_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_cosm1_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_types, 2, 1, 1, 0, ((char *)"cosdg"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosdg_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_cosdg, __pyx_t_2) < 0) __PYX_ERR(1, 4150, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4156
 * cdef void *ufunc_cosm1_data[2]
 * cdef char ufunc_cosm1_types[4]
 * cdef char *ufunc_cosm1_doc = (             # <<<<<<<<<<<<<<
 *     "cosm1(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_doc = ((char *)"cosm1(x)\n\ncos(x) - 1 for use when `x` is near zero.");

  /* "scipy/special/_ufuncs.pyx":4160
 *     "\n"
 *     "cos(x) - 1 for use when `x` is near zero.")
 * ufunc_cosm1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_cosm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cosm1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4161
 *     "cos(x) - 1 for use when `x` is near zero.")
 * ufunc_cosm1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_cosm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_cosm1_types[0] = <char>NPY_FLOAT
 * ufunc_cosm1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4162
 * ufunc_cosm1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_cosm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cosm1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_cosm1_types[1] = <char>NPY_FLOAT
 * ufunc_cosm1_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4163
 * ufunc_cosm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cosm1_types[0] = <char>NPY_FLOAT
 * ufunc_cosm1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_cosm1_types[2] = <char>NPY_DOUBLE
 * ufunc_cosm1_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4164
 * ufunc_cosm1_types[0] = <char>NPY_FLOAT
 * ufunc_cosm1_types[1] = <char>NPY_FLOAT
 * ufunc_cosm1_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_cosm1_types[3] = <char>NPY_DOUBLE
 * ufunc_cosm1_ptr[2*0] = <void*>_func_cosm1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4165
 * ufunc_cosm1_types[1] = <char>NPY_FLOAT
 * ufunc_cosm1_types[2] = <char>NPY_DOUBLE
 * ufunc_cosm1_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_cosm1_ptr[2*0] = <void*>_func_cosm1
 * ufunc_cosm1_ptr[2*0+1] = <void*>(<char*>"cosm1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4166
 * ufunc_cosm1_types[2] = <char>NPY_DOUBLE
 * ufunc_cosm1_types[3] = <char>NPY_DOUBLE
 * ufunc_cosm1_ptr[2*0] = <void*>_func_cosm1             # <<<<<<<<<<<<<<
 * ufunc_cosm1_ptr[2*0+1] = <void*>(<char*>"cosm1")
 * ufunc_cosm1_ptr[2*1] = <void*>_func_cosm1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_ptr[0]) = ((void *)cosm1);

  /* "scipy/special/_ufuncs.pyx":4167
 * ufunc_cosm1_types[3] = <char>NPY_DOUBLE
 * ufunc_cosm1_ptr[2*0] = <void*>_func_cosm1
 * ufunc_cosm1_ptr[2*0+1] = <void*>(<char*>"cosm1")             # <<<<<<<<<<<<<<
 * ufunc_cosm1_ptr[2*1] = <void*>_func_cosm1
 * ufunc_cosm1_ptr[2*1+1] = <void*>(<char*>"cosm1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_ptr[1]) = ((void *)((char *)((char *)"cosm1")));

  /* "scipy/special/_ufuncs.pyx":4168
 * ufunc_cosm1_ptr[2*0] = <void*>_func_cosm1
 * ufunc_cosm1_ptr[2*0+1] = <void*>(<char*>"cosm1")
 * ufunc_cosm1_ptr[2*1] = <void*>_func_cosm1             # <<<<<<<<<<<<<<
 * ufunc_cosm1_ptr[2*1+1] = <void*>(<char*>"cosm1")
 * ufunc_cosm1_data[0] = &ufunc_cosm1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_ptr[2]) = ((void *)cosm1);

  /* "scipy/special/_ufuncs.pyx":4169
 * ufunc_cosm1_ptr[2*0+1] = <void*>(<char*>"cosm1")
 * ufunc_cosm1_ptr[2*1] = <void*>_func_cosm1
 * ufunc_cosm1_ptr[2*1+1] = <void*>(<char*>"cosm1")             # <<<<<<<<<<<<<<
 * ufunc_cosm1_data[0] = &ufunc_cosm1_ptr[2*0]
 * ufunc_cosm1_data[1] = &ufunc_cosm1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_ptr[3]) = ((void *)((char *)((char *)"cosm1")));

  /* "scipy/special/_ufuncs.pyx":4170
 * ufunc_cosm1_ptr[2*1] = <void*>_func_cosm1
 * ufunc_cosm1_ptr[2*1+1] = <void*>(<char*>"cosm1")
 * ufunc_cosm1_data[0] = &ufunc_cosm1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_cosm1_data[1] = &ufunc_cosm1_ptr[2*1]
 * cosm1 = np.PyUFunc_FromFuncAndData(ufunc_cosm1_loops, ufunc_cosm1_data, ufunc_cosm1_types, 2, 1, 1, 0, "cosm1", ufunc_cosm1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4171
 * ufunc_cosm1_ptr[2*1+1] = <void*>(<char*>"cosm1")
 * ufunc_cosm1_data[0] = &ufunc_cosm1_ptr[2*0]
 * ufunc_cosm1_data[1] = &ufunc_cosm1_ptr[2*1]             # <<<<<<<<<<<<<<
 * cosm1 = np.PyUFunc_FromFuncAndData(ufunc_cosm1_loops, ufunc_cosm1_data, ufunc_cosm1_types, 2, 1, 1, 0, "cosm1", ufunc_cosm1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4172
 * ufunc_cosm1_data[0] = &ufunc_cosm1_ptr[2*0]
 * ufunc_cosm1_data[1] = &ufunc_cosm1_ptr[2*1]
 * cosm1 = np.PyUFunc_FromFuncAndData(ufunc_cosm1_loops, ufunc_cosm1_data, ufunc_cosm1_types, 2, 1, 1, 0, "cosm1", ufunc_cosm1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_cotdg_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_types, 2, 1, 1, 0, ((char *)"cosm1"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_cosm1_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_cosm1, __pyx_t_2) < 0) __PYX_ERR(1, 4172, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4178
 * cdef void *ufunc_cotdg_data[2]
 * cdef char ufunc_cotdg_types[4]
 * cdef char *ufunc_cotdg_doc = (             # <<<<<<<<<<<<<<
 *     "cotdg(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_doc = ((char *)"cotdg(x)\n\nCotangent of the angle `x` given in degrees.");

  /* "scipy/special/_ufuncs.pyx":4182
 *     "\n"
 *     "Cotangent of the angle `x` given in degrees.")
 * ufunc_cotdg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_cotdg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cotdg_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4183
 *     "Cotangent of the angle `x` given in degrees.")
 * ufunc_cotdg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_cotdg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_cotdg_types[0] = <char>NPY_FLOAT
 * ufunc_cotdg_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4184
 * ufunc_cotdg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_cotdg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cotdg_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_cotdg_types[1] = <char>NPY_FLOAT
 * ufunc_cotdg_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4185
 * ufunc_cotdg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_cotdg_types[0] = <char>NPY_FLOAT
 * ufunc_cotdg_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_cotdg_types[2] = <char>NPY_DOUBLE
 * ufunc_cotdg_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4186
 * ufunc_cotdg_types[0] = <char>NPY_FLOAT
 * ufunc_cotdg_types[1] = <char>NPY_FLOAT
 * ufunc_cotdg_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_cotdg_types[3] = <char>NPY_DOUBLE
 * ufunc_cotdg_ptr[2*0] = <void*>_func_cotdg
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4187
 * ufunc_cotdg_types[1] = <char>NPY_FLOAT
 * ufunc_cotdg_types[2] = <char>NPY_DOUBLE
 * ufunc_cotdg_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_cotdg_ptr[2*0] = <void*>_func_cotdg
 * ufunc_cotdg_ptr[2*0+1] = <void*>(<char*>"cotdg")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4188
 * ufunc_cotdg_types[2] = <char>NPY_DOUBLE
 * ufunc_cotdg_types[3] = <char>NPY_DOUBLE
 * ufunc_cotdg_ptr[2*0] = <void*>_func_cotdg             # <<<<<<<<<<<<<<
 * ufunc_cotdg_ptr[2*0+1] = <void*>(<char*>"cotdg")
 * ufunc_cotdg_ptr[2*1] = <void*>_func_cotdg
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_ptr[0]) = ((void *)cotdg);

  /* "scipy/special/_ufuncs.pyx":4189
 * ufunc_cotdg_types[3] = <char>NPY_DOUBLE
 * ufunc_cotdg_ptr[2*0] = <void*>_func_cotdg
 * ufunc_cotdg_ptr[2*0+1] = <void*>(<char*>"cotdg")             # <<<<<<<<<<<<<<
 * ufunc_cotdg_ptr[2*1] = <void*>_func_cotdg
 * ufunc_cotdg_ptr[2*1+1] = <void*>(<char*>"cotdg")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_ptr[1]) = ((void *)((char *)((char *)"cotdg")));

  /* "scipy/special/_ufuncs.pyx":4190
 * ufunc_cotdg_ptr[2*0] = <void*>_func_cotdg
 * ufunc_cotdg_ptr[2*0+1] = <void*>(<char*>"cotdg")
 * ufunc_cotdg_ptr[2*1] = <void*>_func_cotdg             # <<<<<<<<<<<<<<
 * ufunc_cotdg_ptr[2*1+1] = <void*>(<char*>"cotdg")
 * ufunc_cotdg_data[0] = &ufunc_cotdg_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_ptr[2]) = ((void *)cotdg);

  /* "scipy/special/_ufuncs.pyx":4191
 * ufunc_cotdg_ptr[2*0+1] = <void*>(<char*>"cotdg")
 * ufunc_cotdg_ptr[2*1] = <void*>_func_cotdg
 * ufunc_cotdg_ptr[2*1+1] = <void*>(<char*>"cotdg")             # <<<<<<<<<<<<<<
 * ufunc_cotdg_data[0] = &ufunc_cotdg_ptr[2*0]
 * ufunc_cotdg_data[1] = &ufunc_cotdg_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_ptr[3]) = ((void *)((char *)((char *)"cotdg")));

  /* "scipy/special/_ufuncs.pyx":4192
 * ufunc_cotdg_ptr[2*1] = <void*>_func_cotdg
 * ufunc_cotdg_ptr[2*1+1] = <void*>(<char*>"cotdg")
 * ufunc_cotdg_data[0] = &ufunc_cotdg_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_cotdg_data[1] = &ufunc_cotdg_ptr[2*1]
 * cotdg = np.PyUFunc_FromFuncAndData(ufunc_cotdg_loops, ufunc_cotdg_data, ufunc_cotdg_types, 2, 1, 1, 0, "cotdg", ufunc_cotdg_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4193
 * ufunc_cotdg_ptr[2*1+1] = <void*>(<char*>"cotdg")
 * ufunc_cotdg_data[0] = &ufunc_cotdg_ptr[2*0]
 * ufunc_cotdg_data[1] = &ufunc_cotdg_ptr[2*1]             # <<<<<<<<<<<<<<
 * cotdg = np.PyUFunc_FromFuncAndData(ufunc_cotdg_loops, ufunc_cotdg_data, ufunc_cotdg_types, 2, 1, 1, 0, "cotdg", ufunc_cotdg_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4194
 * ufunc_cotdg_data[0] = &ufunc_cotdg_ptr[2*0]
 * ufunc_cotdg_data[1] = &ufunc_cotdg_ptr[2*1]
 * cotdg = np.PyUFunc_FromFuncAndData(ufunc_cotdg_loops, ufunc_cotdg_data, ufunc_cotdg_types, 2, 1, 1, 0, "cotdg", ufunc_cotdg_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_dawsn_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_types, 2, 1, 1, 0, ((char *)"cotdg"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_cotdg_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_cotdg, __pyx_t_2) < 0) __PYX_ERR(1, 4194, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4200
 * cdef void *ufunc_dawsn_data[4]
 * cdef char ufunc_dawsn_types[8]
 * cdef char *ufunc_dawsn_doc = (             # <<<<<<<<<<<<<<
 *     "dawsn(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_doc = ((char *)"dawsn(x)\n\nDawson's integral.\n\nComputes::\n\n    exp(-x**2) * integral(exp(t**2), t=0..x).\n\nSee Also\n--------\nwofz, erf, erfc, erfcx, erfi\n\nReferences\n----------\n.. [1] Steven G. Johnson, Faddeeva W function implementation.\n   http://ab-initio.mit.edu/Faddeeva\n\nExamples\n--------\n>>> from scipy import special\n>>> import matplotlib.pyplot as plt\n>>> x = np.linspace(-15, 15, num=1000)\n>>> plt.plot(x, special.dawsn(x))\n>>> plt.xlabel('$x$')\n>>> plt.ylabel('$dawsn(x)$')\n>>> plt.show()");

  /* "scipy/special/_ufuncs.pyx":4227
 *     ">>> plt.ylabel('$dawsn(x)$')\n"
 *     ">>> plt.show()")
 * ufunc_dawsn_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_dawsn_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_dawsn_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4228
 *     ">>> plt.show()")
 * ufunc_dawsn_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_dawsn_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_dawsn_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_dawsn_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4229
 * ufunc_dawsn_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_dawsn_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_dawsn_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_dawsn_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_dawsn_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":4230
 * ufunc_dawsn_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_dawsn_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_dawsn_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_dawsn_types[0] = <char>NPY_FLOAT
 * ufunc_dawsn_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":4231
 * ufunc_dawsn_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_dawsn_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_dawsn_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_dawsn_types[1] = <char>NPY_FLOAT
 * ufunc_dawsn_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4232
 * ufunc_dawsn_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_dawsn_types[0] = <char>NPY_FLOAT
 * ufunc_dawsn_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_dawsn_types[2] = <char>NPY_DOUBLE
 * ufunc_dawsn_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4233
 * ufunc_dawsn_types[0] = <char>NPY_FLOAT
 * ufunc_dawsn_types[1] = <char>NPY_FLOAT
 * ufunc_dawsn_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_dawsn_types[3] = <char>NPY_DOUBLE
 * ufunc_dawsn_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4234
 * ufunc_dawsn_types[1] = <char>NPY_FLOAT
 * ufunc_dawsn_types[2] = <char>NPY_DOUBLE
 * ufunc_dawsn_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_dawsn_types[4] = <char>NPY_CFLOAT
 * ufunc_dawsn_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4235
 * ufunc_dawsn_types[2] = <char>NPY_DOUBLE
 * ufunc_dawsn_types[3] = <char>NPY_DOUBLE
 * ufunc_dawsn_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_dawsn_types[5] = <char>NPY_CFLOAT
 * ufunc_dawsn_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4236
 * ufunc_dawsn_types[3] = <char>NPY_DOUBLE
 * ufunc_dawsn_types[4] = <char>NPY_CFLOAT
 * ufunc_dawsn_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_dawsn_types[6] = <char>NPY_CDOUBLE
 * ufunc_dawsn_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4237
 * ufunc_dawsn_types[4] = <char>NPY_CFLOAT
 * ufunc_dawsn_types[5] = <char>NPY_CFLOAT
 * ufunc_dawsn_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_dawsn_types[7] = <char>NPY_CDOUBLE
 * ufunc_dawsn_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4238
 * ufunc_dawsn_types[5] = <char>NPY_CFLOAT
 * ufunc_dawsn_types[6] = <char>NPY_CDOUBLE
 * ufunc_dawsn_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_dawsn_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn
 * ufunc_dawsn_ptr[2*0+1] = <void*>(<char*>"dawsn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4239
 * ufunc_dawsn_types[6] = <char>NPY_CDOUBLE
 * ufunc_dawsn_types[7] = <char>NPY_CDOUBLE
 * ufunc_dawsn_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn             # <<<<<<<<<<<<<<
 * ufunc_dawsn_ptr[2*0+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[0]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn);

  /* "scipy/special/_ufuncs.pyx":4240
 * ufunc_dawsn_types[7] = <char>NPY_CDOUBLE
 * ufunc_dawsn_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn
 * ufunc_dawsn_ptr[2*0+1] = <void*>(<char*>"dawsn")             # <<<<<<<<<<<<<<
 * ufunc_dawsn_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn
 * ufunc_dawsn_ptr[2*1+1] = <void*>(<char*>"dawsn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[1]) = ((void *)((char *)((char *)"dawsn")));

  /* "scipy/special/_ufuncs.pyx":4241
 * ufunc_dawsn_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn
 * ufunc_dawsn_ptr[2*0+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn             # <<<<<<<<<<<<<<
 * ufunc_dawsn_ptr[2*1+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn_complex
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[2]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn);

  /* "scipy/special/_ufuncs.pyx":4242
 * ufunc_dawsn_ptr[2*0+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn
 * ufunc_dawsn_ptr[2*1+1] = <void*>(<char*>"dawsn")             # <<<<<<<<<<<<<<
 * ufunc_dawsn_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn_complex
 * ufunc_dawsn_ptr[2*2+1] = <void*>(<char*>"dawsn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[3]) = ((void *)((char *)((char *)"dawsn")));

  /* "scipy/special/_ufuncs.pyx":4243
 * ufunc_dawsn_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn
 * ufunc_dawsn_ptr[2*1+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn_complex             # <<<<<<<<<<<<<<
 * ufunc_dawsn_ptr[2*2+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn_complex
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[4]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn_complex);

  /* "scipy/special/_ufuncs.pyx":4244
 * ufunc_dawsn_ptr[2*1+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn_complex
 * ufunc_dawsn_ptr[2*2+1] = <void*>(<char*>"dawsn")             # <<<<<<<<<<<<<<
 * ufunc_dawsn_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn_complex
 * ufunc_dawsn_ptr[2*3+1] = <void*>(<char*>"dawsn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[5]) = ((void *)((char *)((char *)"dawsn")));

  /* "scipy/special/_ufuncs.pyx":4245
 * ufunc_dawsn_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn_complex
 * ufunc_dawsn_ptr[2*2+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn_complex             # <<<<<<<<<<<<<<
 * ufunc_dawsn_ptr[2*3+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_data[0] = &ufunc_dawsn_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[6]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_dawsn_complex);

  /* "scipy/special/_ufuncs.pyx":4246
 * ufunc_dawsn_ptr[2*2+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn_complex
 * ufunc_dawsn_ptr[2*3+1] = <void*>(<char*>"dawsn")             # <<<<<<<<<<<<<<
 * ufunc_dawsn_data[0] = &ufunc_dawsn_ptr[2*0]
 * ufunc_dawsn_data[1] = &ufunc_dawsn_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[7]) = ((void *)((char *)((char *)"dawsn")));

  /* "scipy/special/_ufuncs.pyx":4247
 * ufunc_dawsn_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_dawsn_complex
 * ufunc_dawsn_ptr[2*3+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_data[0] = &ufunc_dawsn_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_dawsn_data[1] = &ufunc_dawsn_ptr[2*1]
 * ufunc_dawsn_data[2] = &ufunc_dawsn_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4248
 * ufunc_dawsn_ptr[2*3+1] = <void*>(<char*>"dawsn")
 * ufunc_dawsn_data[0] = &ufunc_dawsn_ptr[2*0]
 * ufunc_dawsn_data[1] = &ufunc_dawsn_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_dawsn_data[2] = &ufunc_dawsn_ptr[2*2]
 * ufunc_dawsn_data[3] = &ufunc_dawsn_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4249
 * ufunc_dawsn_data[0] = &ufunc_dawsn_ptr[2*0]
 * ufunc_dawsn_data[1] = &ufunc_dawsn_ptr[2*1]
 * ufunc_dawsn_data[2] = &ufunc_dawsn_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_dawsn_data[3] = &ufunc_dawsn_ptr[2*3]
 * dawsn = np.PyUFunc_FromFuncAndData(ufunc_dawsn_loops, ufunc_dawsn_data, ufunc_dawsn_types, 4, 1, 1, 0, "dawsn", ufunc_dawsn_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":4250
 * ufunc_dawsn_data[1] = &ufunc_dawsn_ptr[2*1]
 * ufunc_dawsn_data[2] = &ufunc_dawsn_ptr[2*2]
 * ufunc_dawsn_data[3] = &ufunc_dawsn_ptr[2*3]             # <<<<<<<<<<<<<<
 * dawsn = np.PyUFunc_FromFuncAndData(ufunc_dawsn_loops, ufunc_dawsn_data, ufunc_dawsn_types, 4, 1, 1, 0, "dawsn", ufunc_dawsn_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":4251
 * ufunc_dawsn_data[2] = &ufunc_dawsn_ptr[2*2]
 * ufunc_dawsn_data[3] = &ufunc_dawsn_ptr[2*3]
 * dawsn = np.PyUFunc_FromFuncAndData(ufunc_dawsn_loops, ufunc_dawsn_data, ufunc_dawsn_types, 4, 1, 1, 0, "dawsn", ufunc_dawsn_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ellipe_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_types, 4, 1, 1, 0, ((char *)"dawsn"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_dawsn_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_dawsn, __pyx_t_2) < 0) __PYX_ERR(1, 4251, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4257
 * cdef void *ufunc_ellipe_data[2]
 * cdef char ufunc_ellipe_types[4]
 * cdef char *ufunc_ellipe_doc = (             # <<<<<<<<<<<<<<
 *     "ellipe(m)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_doc = ((char *)"ellipe(m)\n\nComplete elliptic integral of the second kind\n\nThis function is defined as\n\n.. math:: E(m) = \\int_0^{\\pi/2} [1 - m \\sin(t)^2]^{1/2} dt\n\nParameters\n----------\nm : array_like\n    Defines the parameter of the elliptic integral.\n\nReturns\n-------\nE : ndarray\n    Value of the elliptic integral.\n\nNotes\n-----\nWrapper for the Cephes [1]_ routine `ellpe`.\n\nFor `m > 0` the computation uses the approximation,\n\n.. math:: E(m) \\approx P(1-m) - (1-m) \\log(1-m) Q(1-m),\n\nwhere :math:`P` and :math:`Q` are tenth-order polynomials.  For\n`m < 0`, the relation\n\n.. math:: E(m) = E(m/(m - 1)) \\sqrt(1-m)\n\nis used.\n\nThe parameterization in terms of :math:`m` follows that of section\n17.2 in [2]_. Other parameterizations in terms of the\ncomplementary parameter :math:`1 - m`, modular angle\n:math:`\\sin^2(\\alpha) = m`, or modulus :math:`k^2 = m` are also\nused, so be careful that you choose the correct parameter.\n\nSee Also\n--------\nellipkm1 : Complete elliptic integral of the first kind, near `m` = 1\nellipk : Complete elliptic integral of the first kind\nellipkinc : Incomplete elliptic integral of the first kind\nellipeinc : Incomplete elliptic integral of the second kind\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/\n.. [2] Milton Abramowitz and Irene A. Stegun, eds.\n       Handbook of Mathematical Functions with Formulas,\n       Graphs, and Mathematical Tables. New York: Dover, 1972.\n\nExamples\n--------\nThis function is used in finding the circumference of an\nellipse with semi-major axis `a` and semi-minor axis `b`.\n\n>>> from scipy import special\n\n>>> a = 3.5\n>>> b = 2.1\n>>> e_sq = 1.0 - b**2/a**2  # eccentricity squared\n\nThen the circumference is found using the following:\n\n>>> C = 4*a*special.ellipe(e_sq)  # circumference formula\n>>> C\n17.868899204378693\n\nWhen `a` and `b` are the same (meaning eccentricity is 0),\nthis reduces to the circumference of"" a circle.\n\n>>> 4*a*special.ellipe(0.0)  # formula for ellipse with a = b\n21.991148575128552\n>>> 2*np.pi*a  # formula for circle of radius a\n21.991148575128552");

  /* "scipy/special/_ufuncs.pyx":4336
 *     ">>> 2*np.pi*a  # formula for circle of radius a\n"
 *     "21.991148575128552")
 * ufunc_ellipe_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_ellipe_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ellipe_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4337
 *     "21.991148575128552")
 * ufunc_ellipe_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ellipe_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_ellipe_types[0] = <char>NPY_FLOAT
 * ufunc_ellipe_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4338
 * ufunc_ellipe_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ellipe_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ellipe_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipe_types[1] = <char>NPY_FLOAT
 * ufunc_ellipe_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4339
 * ufunc_ellipe_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ellipe_types[0] = <char>NPY_FLOAT
 * ufunc_ellipe_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipe_types[2] = <char>NPY_DOUBLE
 * ufunc_ellipe_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4340
 * ufunc_ellipe_types[0] = <char>NPY_FLOAT
 * ufunc_ellipe_types[1] = <char>NPY_FLOAT
 * ufunc_ellipe_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipe_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipe_ptr[2*0] = <void*>_func_ellpe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4341
 * ufunc_ellipe_types[1] = <char>NPY_FLOAT
 * ufunc_ellipe_types[2] = <char>NPY_DOUBLE
 * ufunc_ellipe_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipe_ptr[2*0] = <void*>_func_ellpe
 * ufunc_ellipe_ptr[2*0+1] = <void*>(<char*>"ellipe")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4342
 * ufunc_ellipe_types[2] = <char>NPY_DOUBLE
 * ufunc_ellipe_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipe_ptr[2*0] = <void*>_func_ellpe             # <<<<<<<<<<<<<<
 * ufunc_ellipe_ptr[2*0+1] = <void*>(<char*>"ellipe")
 * ufunc_ellipe_ptr[2*1] = <void*>_func_ellpe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_ptr[0]) = ((void *)ellpe);

  /* "scipy/special/_ufuncs.pyx":4343
 * ufunc_ellipe_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipe_ptr[2*0] = <void*>_func_ellpe
 * ufunc_ellipe_ptr[2*0+1] = <void*>(<char*>"ellipe")             # <<<<<<<<<<<<<<
 * ufunc_ellipe_ptr[2*1] = <void*>_func_ellpe
 * ufunc_ellipe_ptr[2*1+1] = <void*>(<char*>"ellipe")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_ptr[1]) = ((void *)((char *)((char *)"ellipe")));

  /* "scipy/special/_ufuncs.pyx":4344
 * ufunc_ellipe_ptr[2*0] = <void*>_func_ellpe
 * ufunc_ellipe_ptr[2*0+1] = <void*>(<char*>"ellipe")
 * ufunc_ellipe_ptr[2*1] = <void*>_func_ellpe             # <<<<<<<<<<<<<<
 * ufunc_ellipe_ptr[2*1+1] = <void*>(<char*>"ellipe")
 * ufunc_ellipe_data[0] = &ufunc_ellipe_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_ptr[2]) = ((void *)ellpe);

  /* "scipy/special/_ufuncs.pyx":4345
 * ufunc_ellipe_ptr[2*0+1] = <void*>(<char*>"ellipe")
 * ufunc_ellipe_ptr[2*1] = <void*>_func_ellpe
 * ufunc_ellipe_ptr[2*1+1] = <void*>(<char*>"ellipe")             # <<<<<<<<<<<<<<
 * ufunc_ellipe_data[0] = &ufunc_ellipe_ptr[2*0]
 * ufunc_ellipe_data[1] = &ufunc_ellipe_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_ptr[3]) = ((void *)((char *)((char *)"ellipe")));

  /* "scipy/special/_ufuncs.pyx":4346
 * ufunc_ellipe_ptr[2*1] = <void*>_func_ellpe
 * ufunc_ellipe_ptr[2*1+1] = <void*>(<char*>"ellipe")
 * ufunc_ellipe_data[0] = &ufunc_ellipe_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ellipe_data[1] = &ufunc_ellipe_ptr[2*1]
 * ellipe = np.PyUFunc_FromFuncAndData(ufunc_ellipe_loops, ufunc_ellipe_data, ufunc_ellipe_types, 2, 1, 1, 0, "ellipe", ufunc_ellipe_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4347
 * ufunc_ellipe_ptr[2*1+1] = <void*>(<char*>"ellipe")
 * ufunc_ellipe_data[0] = &ufunc_ellipe_ptr[2*0]
 * ufunc_ellipe_data[1] = &ufunc_ellipe_ptr[2*1]             # <<<<<<<<<<<<<<
 * ellipe = np.PyUFunc_FromFuncAndData(ufunc_ellipe_loops, ufunc_ellipe_data, ufunc_ellipe_types, 2, 1, 1, 0, "ellipe", ufunc_ellipe_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4348
 * ufunc_ellipe_data[0] = &ufunc_ellipe_ptr[2*0]
 * ufunc_ellipe_data[1] = &ufunc_ellipe_ptr[2*1]
 * ellipe = np.PyUFunc_FromFuncAndData(ufunc_ellipe_loops, ufunc_ellipe_data, ufunc_ellipe_types, 2, 1, 1, 0, "ellipe", ufunc_ellipe_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ellipeinc_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_types, 2, 1, 1, 0, ((char *)"ellipe"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipe_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ellipe, __pyx_t_2) < 0) __PYX_ERR(1, 4348, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4354
 * cdef void *ufunc_ellipeinc_data[2]
 * cdef char ufunc_ellipeinc_types[6]
 * cdef char *ufunc_ellipeinc_doc = (             # <<<<<<<<<<<<<<
 *     "ellipeinc(phi, m)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_doc = ((char *)"ellipeinc(phi, m)\n\nIncomplete elliptic integral of the second kind\n\nThis function is defined as\n\n.. math:: E(\\phi, m) = \\int_0^{\\phi} [1 - m \\sin(t)^2]^{1/2} dt\n\nParameters\n----------\nphi : array_like\n    amplitude of the elliptic integral.\n\nm : array_like\n    parameter of the elliptic integral.\n\nReturns\n-------\nE : ndarray\n    Value of the elliptic integral.\n\nNotes\n-----\nWrapper for the Cephes [1]_ routine `ellie`.\n\nComputation uses arithmetic-geometric means algorithm.\n\nThe parameterization in terms of :math:`m` follows that of section\n17.2 in [2]_. Other parameterizations in terms of the\ncomplementary parameter :math:`1 - m`, modular angle\n:math:`\\sin^2(\\alpha) = m`, or modulus :math:`k^2 = m` are also\nused, so be careful that you choose the correct parameter.\n\nSee Also\n--------\nellipkm1 : Complete elliptic integral of the first kind, near `m` = 1\nellipk : Complete elliptic integral of the first kind\nellipkinc : Incomplete elliptic integral of the first kind\nellipe : Complete elliptic integral of the second kind\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/\n.. [2] Milton Abramowitz and Irene A. Stegun, eds.\n       Handbook of Mathematical Functions with Formulas,\n       Graphs, and Mathematical Tables. New York: Dover, 1972.");

  /* "scipy/special/_ufuncs.pyx":4402
 *     "       Handbook of Mathematical Functions with Formulas,\n"
 *     "       Graphs, and Mathematical Tables. New York: Dover, 1972.")
 * ufunc_ellipeinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_ellipeinc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":4403
 *     "       Graphs, and Mathematical Tables. New York: Dover, 1972.")
 * ufunc_ellipeinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_ellipeinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_types[0] = <char>NPY_FLOAT
 * ufunc_ellipeinc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":4404
 * ufunc_ellipeinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_ellipeinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_ellipeinc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_types[1] = <char>NPY_FLOAT
 * ufunc_ellipeinc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4405
 * ufunc_ellipeinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_ellipeinc_types[0] = <char>NPY_FLOAT
 * ufunc_ellipeinc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_types[2] = <char>NPY_FLOAT
 * ufunc_ellipeinc_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4406
 * ufunc_ellipeinc_types[0] = <char>NPY_FLOAT
 * ufunc_ellipeinc_types[1] = <char>NPY_FLOAT
 * ufunc_ellipeinc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipeinc_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4407
 * ufunc_ellipeinc_types[1] = <char>NPY_FLOAT
 * ufunc_ellipeinc_types[2] = <char>NPY_FLOAT
 * ufunc_ellipeinc_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_types[4] = <char>NPY_DOUBLE
 * ufunc_ellipeinc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4408
 * ufunc_ellipeinc_types[2] = <char>NPY_FLOAT
 * ufunc_ellipeinc_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipeinc_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_types[5] = <char>NPY_DOUBLE
 * ufunc_ellipeinc_ptr[2*0] = <void*>_func_ellie
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4409
 * ufunc_ellipeinc_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipeinc_types[4] = <char>NPY_DOUBLE
 * ufunc_ellipeinc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_ptr[2*0] = <void*>_func_ellie
 * ufunc_ellipeinc_ptr[2*0+1] = <void*>(<char*>"ellipeinc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4410
 * ufunc_ellipeinc_types[4] = <char>NPY_DOUBLE
 * ufunc_ellipeinc_types[5] = <char>NPY_DOUBLE
 * ufunc_ellipeinc_ptr[2*0] = <void*>_func_ellie             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_ptr[2*0+1] = <void*>(<char*>"ellipeinc")
 * ufunc_ellipeinc_ptr[2*1] = <void*>_func_ellie
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_ptr[0]) = ((void *)ellie);

  /* "scipy/special/_ufuncs.pyx":4411
 * ufunc_ellipeinc_types[5] = <char>NPY_DOUBLE
 * ufunc_ellipeinc_ptr[2*0] = <void*>_func_ellie
 * ufunc_ellipeinc_ptr[2*0+1] = <void*>(<char*>"ellipeinc")             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_ptr[2*1] = <void*>_func_ellie
 * ufunc_ellipeinc_ptr[2*1+1] = <void*>(<char*>"ellipeinc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_ptr[1]) = ((void *)((char *)((char *)"ellipeinc")));

  /* "scipy/special/_ufuncs.pyx":4412
 * ufunc_ellipeinc_ptr[2*0] = <void*>_func_ellie
 * ufunc_ellipeinc_ptr[2*0+1] = <void*>(<char*>"ellipeinc")
 * ufunc_ellipeinc_ptr[2*1] = <void*>_func_ellie             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_ptr[2*1+1] = <void*>(<char*>"ellipeinc")
 * ufunc_ellipeinc_data[0] = &ufunc_ellipeinc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_ptr[2]) = ((void *)ellie);

  /* "scipy/special/_ufuncs.pyx":4413
 * ufunc_ellipeinc_ptr[2*0+1] = <void*>(<char*>"ellipeinc")
 * ufunc_ellipeinc_ptr[2*1] = <void*>_func_ellie
 * ufunc_ellipeinc_ptr[2*1+1] = <void*>(<char*>"ellipeinc")             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_data[0] = &ufunc_ellipeinc_ptr[2*0]
 * ufunc_ellipeinc_data[1] = &ufunc_ellipeinc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_ptr[3]) = ((void *)((char *)((char *)"ellipeinc")));

  /* "scipy/special/_ufuncs.pyx":4414
 * ufunc_ellipeinc_ptr[2*1] = <void*>_func_ellie
 * ufunc_ellipeinc_ptr[2*1+1] = <void*>(<char*>"ellipeinc")
 * ufunc_ellipeinc_data[0] = &ufunc_ellipeinc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ellipeinc_data[1] = &ufunc_ellipeinc_ptr[2*1]
 * ellipeinc = np.PyUFunc_FromFuncAndData(ufunc_ellipeinc_loops, ufunc_ellipeinc_data, ufunc_ellipeinc_types, 2, 2, 1, 0, "ellipeinc", ufunc_ellipeinc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4415
 * ufunc_ellipeinc_ptr[2*1+1] = <void*>(<char*>"ellipeinc")
 * ufunc_ellipeinc_data[0] = &ufunc_ellipeinc_ptr[2*0]
 * ufunc_ellipeinc_data[1] = &ufunc_ellipeinc_ptr[2*1]             # <<<<<<<<<<<<<<
 * ellipeinc = np.PyUFunc_FromFuncAndData(ufunc_ellipeinc_loops, ufunc_ellipeinc_data, ufunc_ellipeinc_types, 2, 2, 1, 0, "ellipeinc", ufunc_ellipeinc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4416
 * ufunc_ellipeinc_data[0] = &ufunc_ellipeinc_ptr[2*0]
 * ufunc_ellipeinc_data[1] = &ufunc_ellipeinc_ptr[2*1]
 * ellipeinc = np.PyUFunc_FromFuncAndData(ufunc_ellipeinc_loops, ufunc_ellipeinc_data, ufunc_ellipeinc_types, 2, 2, 1, 0, "ellipeinc", ufunc_ellipeinc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ellipj_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_types, 2, 2, 1, 0, ((char *)"ellipeinc"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipeinc_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4416, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ellipeinc, __pyx_t_2) < 0) __PYX_ERR(1, 4416, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4422
 * cdef void *ufunc_ellipj_data[2]
 * cdef char ufunc_ellipj_types[12]
 * cdef char *ufunc_ellipj_doc = (             # <<<<<<<<<<<<<<
 *     "ellipj(u, m)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_doc = ((char *)"ellipj(u, m)\n\nJacobian elliptic functions\n\nCalculates the Jacobian elliptic functions of parameter `m` between\n0 and 1, and real argument `u`.\n\nParameters\n----------\nm : array_like\n    Parameter.\nu : array_like\n    Argument.\n\nReturns\n-------\nsn, cn, dn, ph : ndarrays\n    The returned functions::\n\n        sn(u|m), cn(u|m), dn(u|m)\n\n    The value `ph` is such that if `u = ellipkinc(ph, m)`,\n    then `sn(u|m) = sin(ph)` and `cn(u|m) = cos(ph)`.\n\nNotes\n-----\nWrapper for the Cephes [1]_ routine `ellpj`.\n\nThese functions are periodic, with quarter-period on the real axis\nequal to the complete elliptic integral `ellipk(m)`.\n\nRelation to incomplete elliptic integral: If `u = ellipkinc(phi,m)`, then\n`sn(u|m) = sin(phi)`, and `cn(u|m) = cos(phi)`.  The `phi` is called\nthe amplitude of `u`.\n\nComputation is by means of the arithmetic-geometric mean algorithm,\nexcept when `m` is within 1e-9 of 0 or 1.  In the latter case with `m`\nclose to 1, the approximation applies only for `phi < pi/2`.\n\nSee also\n--------\nellipk : Complete elliptic integral of the first kind\nellipkinc : Incomplete elliptic integral of the first kind\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":4471
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_ellipj_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dddd_As_ff_ffff             # <<<<<<<<<<<<<<
 * ufunc_ellipj_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dddd_As_dd_dddd
 * ufunc_ellipj_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dddd_As_ff_ffff);

  /* "scipy/special/_ufuncs.pyx":4472
 *     "       http://www.netlib.org/cephes/")
 * ufunc_ellipj_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dddd_As_ff_ffff
 * ufunc_ellipj_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dddd_As_dd_dddd             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[0] = <char>NPY_FLOAT
 * ufunc_ellipj_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dddd_As_dd_dddd);

  /* "scipy/special/_ufuncs.pyx":4473
 * ufunc_ellipj_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dddd_As_ff_ffff
 * ufunc_ellipj_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dddd_As_dd_dddd
 * ufunc_ellipj_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[1] = <char>NPY_FLOAT
 * ufunc_ellipj_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4474
 * ufunc_ellipj_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dddd_As_dd_dddd
 * ufunc_ellipj_types[0] = <char>NPY_FLOAT
 * ufunc_ellipj_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[2] = <char>NPY_FLOAT
 * ufunc_ellipj_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4475
 * ufunc_ellipj_types[0] = <char>NPY_FLOAT
 * ufunc_ellipj_types[1] = <char>NPY_FLOAT
 * ufunc_ellipj_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[3] = <char>NPY_FLOAT
 * ufunc_ellipj_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4476
 * ufunc_ellipj_types[1] = <char>NPY_FLOAT
 * ufunc_ellipj_types[2] = <char>NPY_FLOAT
 * ufunc_ellipj_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[4] = <char>NPY_FLOAT
 * ufunc_ellipj_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4477
 * ufunc_ellipj_types[2] = <char>NPY_FLOAT
 * ufunc_ellipj_types[3] = <char>NPY_FLOAT
 * ufunc_ellipj_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[5] = <char>NPY_FLOAT
 * ufunc_ellipj_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4478
 * ufunc_ellipj_types[3] = <char>NPY_FLOAT
 * ufunc_ellipj_types[4] = <char>NPY_FLOAT
 * ufunc_ellipj_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[6] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4479
 * ufunc_ellipj_types[4] = <char>NPY_FLOAT
 * ufunc_ellipj_types[5] = <char>NPY_FLOAT
 * ufunc_ellipj_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[7] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4480
 * ufunc_ellipj_types[5] = <char>NPY_FLOAT
 * ufunc_ellipj_types[6] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[8] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4481
 * ufunc_ellipj_types[6] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[7] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[9] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4482
 * ufunc_ellipj_types[7] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[8] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[10] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4483
 * ufunc_ellipj_types[8] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[9] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipj_types[11] = <char>NPY_DOUBLE
 * ufunc_ellipj_ptr[2*0] = <void*>_func_ellpj
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4484
 * ufunc_ellipj_types[9] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[10] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipj_ptr[2*0] = <void*>_func_ellpj
 * ufunc_ellipj_ptr[2*0+1] = <void*>(<char*>"ellipj")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4485
 * ufunc_ellipj_types[10] = <char>NPY_DOUBLE
 * ufunc_ellipj_types[11] = <char>NPY_DOUBLE
 * ufunc_ellipj_ptr[2*0] = <void*>_func_ellpj             # <<<<<<<<<<<<<<
 * ufunc_ellipj_ptr[2*0+1] = <void*>(<char*>"ellipj")
 * ufunc_ellipj_ptr[2*1] = <void*>_func_ellpj
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_ptr[0]) = ((void *)ellpj);

  /* "scipy/special/_ufuncs.pyx":4486
 * ufunc_ellipj_types[11] = <char>NPY_DOUBLE
 * ufunc_ellipj_ptr[2*0] = <void*>_func_ellpj
 * ufunc_ellipj_ptr[2*0+1] = <void*>(<char*>"ellipj")             # <<<<<<<<<<<<<<
 * ufunc_ellipj_ptr[2*1] = <void*>_func_ellpj
 * ufunc_ellipj_ptr[2*1+1] = <void*>(<char*>"ellipj")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_ptr[1]) = ((void *)((char *)((char *)"ellipj")));

  /* "scipy/special/_ufuncs.pyx":4487
 * ufunc_ellipj_ptr[2*0] = <void*>_func_ellpj
 * ufunc_ellipj_ptr[2*0+1] = <void*>(<char*>"ellipj")
 * ufunc_ellipj_ptr[2*1] = <void*>_func_ellpj             # <<<<<<<<<<<<<<
 * ufunc_ellipj_ptr[2*1+1] = <void*>(<char*>"ellipj")
 * ufunc_ellipj_data[0] = &ufunc_ellipj_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_ptr[2]) = ((void *)ellpj);

  /* "scipy/special/_ufuncs.pyx":4488
 * ufunc_ellipj_ptr[2*0+1] = <void*>(<char*>"ellipj")
 * ufunc_ellipj_ptr[2*1] = <void*>_func_ellpj
 * ufunc_ellipj_ptr[2*1+1] = <void*>(<char*>"ellipj")             # <<<<<<<<<<<<<<
 * ufunc_ellipj_data[0] = &ufunc_ellipj_ptr[2*0]
 * ufunc_ellipj_data[1] = &ufunc_ellipj_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_ptr[3]) = ((void *)((char *)((char *)"ellipj")));

  /* "scipy/special/_ufuncs.pyx":4489
 * ufunc_ellipj_ptr[2*1] = <void*>_func_ellpj
 * ufunc_ellipj_ptr[2*1+1] = <void*>(<char*>"ellipj")
 * ufunc_ellipj_data[0] = &ufunc_ellipj_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ellipj_data[1] = &ufunc_ellipj_ptr[2*1]
 * ellipj = np.PyUFunc_FromFuncAndData(ufunc_ellipj_loops, ufunc_ellipj_data, ufunc_ellipj_types, 2, 2, 4, 0, "ellipj", ufunc_ellipj_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4490
 * ufunc_ellipj_ptr[2*1+1] = <void*>(<char*>"ellipj")
 * ufunc_ellipj_data[0] = &ufunc_ellipj_ptr[2*0]
 * ufunc_ellipj_data[1] = &ufunc_ellipj_ptr[2*1]             # <<<<<<<<<<<<<<
 * ellipj = np.PyUFunc_FromFuncAndData(ufunc_ellipj_loops, ufunc_ellipj_data, ufunc_ellipj_types, 2, 2, 4, 0, "ellipj", ufunc_ellipj_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4491
 * ufunc_ellipj_data[0] = &ufunc_ellipj_ptr[2*0]
 * ufunc_ellipj_data[1] = &ufunc_ellipj_ptr[2*1]
 * ellipj = np.PyUFunc_FromFuncAndData(ufunc_ellipj_loops, ufunc_ellipj_data, ufunc_ellipj_types, 2, 2, 4, 0, "ellipj", ufunc_ellipj_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ellipkinc_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_types, 2, 2, 4, 0, ((char *)"ellipj"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipj_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4491, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ellipj, __pyx_t_2) < 0) __PYX_ERR(1, 4491, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4497
 * cdef void *ufunc_ellipkinc_data[2]
 * cdef char ufunc_ellipkinc_types[6]
 * cdef char *ufunc_ellipkinc_doc = (             # <<<<<<<<<<<<<<
 *     "ellipkinc(phi, m)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_doc = ((char *)"ellipkinc(phi, m)\n\nIncomplete elliptic integral of the first kind\n\nThis function is defined as\n\n.. math:: K(\\phi, m) = \\int_0^{\\phi} [1 - m \\sin(t)^2]^{-1/2} dt\n\nThis function is also called `F(phi, m)`.\n\nParameters\n----------\nphi : array_like\n    amplitude of the elliptic integral\n\nm : array_like\n    parameter of the elliptic integral\n\nReturns\n-------\nK : ndarray\n    Value of the elliptic integral\n\nNotes\n-----\nWrapper for the Cephes [1]_ routine `ellik`.  The computation is\ncarried out using the arithmetic-geometric mean algorithm.\n\nThe parameterization in terms of :math:`m` follows that of section\n17.2 in [2]_. Other parameterizations in terms of the\ncomplementary parameter :math:`1 - m`, modular angle\n:math:`\\sin^2(\\alpha) = m`, or modulus :math:`k^2 = m` are also\nused, so be careful that you choose the correct parameter.\n\nSee Also\n--------\nellipkm1 : Complete elliptic integral of the first kind, near `m` = 1\nellipk : Complete elliptic integral of the first kind\nellipe : Complete elliptic integral of the second kind\nellipeinc : Incomplete elliptic integral of the second kind\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/\n.. [2] Milton Abramowitz and Irene A. Stegun, eds.\n       Handbook of Mathematical Functions with Formulas,\n       Graphs, and Mathematical Tables. New York: Dover, 1972.");

  /* "scipy/special/_ufuncs.pyx":4546
 *     "       Handbook of Mathematical Functions with Formulas,\n"
 *     "       Graphs, and Mathematical Tables. New York: Dover, 1972.")
 * ufunc_ellipkinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_ellipkinc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":4547
 *     "       Graphs, and Mathematical Tables. New York: Dover, 1972.")
 * ufunc_ellipkinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_ellipkinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_types[0] = <char>NPY_FLOAT
 * ufunc_ellipkinc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":4548
 * ufunc_ellipkinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_ellipkinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_ellipkinc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_types[1] = <char>NPY_FLOAT
 * ufunc_ellipkinc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4549
 * ufunc_ellipkinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_ellipkinc_types[0] = <char>NPY_FLOAT
 * ufunc_ellipkinc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_types[2] = <char>NPY_FLOAT
 * ufunc_ellipkinc_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4550
 * ufunc_ellipkinc_types[0] = <char>NPY_FLOAT
 * ufunc_ellipkinc_types[1] = <char>NPY_FLOAT
 * ufunc_ellipkinc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipkinc_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4551
 * ufunc_ellipkinc_types[1] = <char>NPY_FLOAT
 * ufunc_ellipkinc_types[2] = <char>NPY_FLOAT
 * ufunc_ellipkinc_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_types[4] = <char>NPY_DOUBLE
 * ufunc_ellipkinc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4552
 * ufunc_ellipkinc_types[2] = <char>NPY_FLOAT
 * ufunc_ellipkinc_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipkinc_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_types[5] = <char>NPY_DOUBLE
 * ufunc_ellipkinc_ptr[2*0] = <void*>_func_ellik
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4553
 * ufunc_ellipkinc_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipkinc_types[4] = <char>NPY_DOUBLE
 * ufunc_ellipkinc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_ptr[2*0] = <void*>_func_ellik
 * ufunc_ellipkinc_ptr[2*0+1] = <void*>(<char*>"ellipkinc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4554
 * ufunc_ellipkinc_types[4] = <char>NPY_DOUBLE
 * ufunc_ellipkinc_types[5] = <char>NPY_DOUBLE
 * ufunc_ellipkinc_ptr[2*0] = <void*>_func_ellik             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_ptr[2*0+1] = <void*>(<char*>"ellipkinc")
 * ufunc_ellipkinc_ptr[2*1] = <void*>_func_ellik
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_ptr[0]) = ((void *)ellik);

  /* "scipy/special/_ufuncs.pyx":4555
 * ufunc_ellipkinc_types[5] = <char>NPY_DOUBLE
 * ufunc_ellipkinc_ptr[2*0] = <void*>_func_ellik
 * ufunc_ellipkinc_ptr[2*0+1] = <void*>(<char*>"ellipkinc")             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_ptr[2*1] = <void*>_func_ellik
 * ufunc_ellipkinc_ptr[2*1+1] = <void*>(<char*>"ellipkinc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_ptr[1]) = ((void *)((char *)((char *)"ellipkinc")));

  /* "scipy/special/_ufuncs.pyx":4556
 * ufunc_ellipkinc_ptr[2*0] = <void*>_func_ellik
 * ufunc_ellipkinc_ptr[2*0+1] = <void*>(<char*>"ellipkinc")
 * ufunc_ellipkinc_ptr[2*1] = <void*>_func_ellik             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_ptr[2*1+1] = <void*>(<char*>"ellipkinc")
 * ufunc_ellipkinc_data[0] = &ufunc_ellipkinc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_ptr[2]) = ((void *)ellik);

  /* "scipy/special/_ufuncs.pyx":4557
 * ufunc_ellipkinc_ptr[2*0+1] = <void*>(<char*>"ellipkinc")
 * ufunc_ellipkinc_ptr[2*1] = <void*>_func_ellik
 * ufunc_ellipkinc_ptr[2*1+1] = <void*>(<char*>"ellipkinc")             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_data[0] = &ufunc_ellipkinc_ptr[2*0]
 * ufunc_ellipkinc_data[1] = &ufunc_ellipkinc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_ptr[3]) = ((void *)((char *)((char *)"ellipkinc")));

  /* "scipy/special/_ufuncs.pyx":4558
 * ufunc_ellipkinc_ptr[2*1] = <void*>_func_ellik
 * ufunc_ellipkinc_ptr[2*1+1] = <void*>(<char*>"ellipkinc")
 * ufunc_ellipkinc_data[0] = &ufunc_ellipkinc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ellipkinc_data[1] = &ufunc_ellipkinc_ptr[2*1]
 * ellipkinc = np.PyUFunc_FromFuncAndData(ufunc_ellipkinc_loops, ufunc_ellipkinc_data, ufunc_ellipkinc_types, 2, 2, 1, 0, "ellipkinc", ufunc_ellipkinc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4559
 * ufunc_ellipkinc_ptr[2*1+1] = <void*>(<char*>"ellipkinc")
 * ufunc_ellipkinc_data[0] = &ufunc_ellipkinc_ptr[2*0]
 * ufunc_ellipkinc_data[1] = &ufunc_ellipkinc_ptr[2*1]             # <<<<<<<<<<<<<<
 * ellipkinc = np.PyUFunc_FromFuncAndData(ufunc_ellipkinc_loops, ufunc_ellipkinc_data, ufunc_ellipkinc_types, 2, 2, 1, 0, "ellipkinc", ufunc_ellipkinc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4560
 * ufunc_ellipkinc_data[0] = &ufunc_ellipkinc_ptr[2*0]
 * ufunc_ellipkinc_data[1] = &ufunc_ellipkinc_ptr[2*1]
 * ellipkinc = np.PyUFunc_FromFuncAndData(ufunc_ellipkinc_loops, ufunc_ellipkinc_data, ufunc_ellipkinc_types, 2, 2, 1, 0, "ellipkinc", ufunc_ellipkinc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ellipkm1_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_types, 2, 2, 1, 0, ((char *)"ellipkinc"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkinc_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4560, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ellipkinc, __pyx_t_2) < 0) __PYX_ERR(1, 4560, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4566
 * cdef void *ufunc_ellipkm1_data[2]
 * cdef char ufunc_ellipkm1_types[4]
 * cdef char *ufunc_ellipkm1_doc = (             # <<<<<<<<<<<<<<
 *     "ellipkm1(p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_doc = ((char *)"ellipkm1(p)\n\nComplete elliptic integral of the first kind around `m` = 1\n\nThis function is defined as\n\n.. math:: K(p) = \\int_0^{\\pi/2} [1 - m \\sin(t)^2]^{-1/2} dt\n\nwhere `m = 1 - p`.\n\nParameters\n----------\np : array_like\n    Defines the parameter of the elliptic integral as `m = 1 - p`.\n\nReturns\n-------\nK : ndarray\n    Value of the elliptic integral.\n\nNotes\n-----\nWrapper for the Cephes [1]_ routine `ellpk`.\n\nFor `p <= 1`, computation uses the approximation,\n\n.. math:: K(p) \\approx P(p) - \\log(p) Q(p),\n\nwhere :math:`P` and :math:`Q` are tenth-order polynomials.  The\nargument `p` is used internally rather than `m` so that the logarithmic\nsingularity at `m = 1` will be shifted to the origin; this preserves\nmaximum accuracy.  For `p > 1`, the identity\n\n.. math:: K(p) = K(1/p)/\\sqrt(p)\n\nis used.\n\nSee Also\n--------\nellipk : Complete elliptic integral of the first kind\nellipkinc : Incomplete elliptic integral of the first kind\nellipe : Complete elliptic integral of the second kind\nellipeinc : Incomplete elliptic integral of the second kind\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":4615
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_ellipkm1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ellipkm1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4616
 *     "       http://www.netlib.org/cephes/")
 * ufunc_ellipkm1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ellipkm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_types[0] = <char>NPY_FLOAT
 * ufunc_ellipkm1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4617
 * ufunc_ellipkm1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ellipkm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ellipkm1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_types[1] = <char>NPY_FLOAT
 * ufunc_ellipkm1_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4618
 * ufunc_ellipkm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ellipkm1_types[0] = <char>NPY_FLOAT
 * ufunc_ellipkm1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_types[2] = <char>NPY_DOUBLE
 * ufunc_ellipkm1_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4619
 * ufunc_ellipkm1_types[0] = <char>NPY_FLOAT
 * ufunc_ellipkm1_types[1] = <char>NPY_FLOAT
 * ufunc_ellipkm1_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipkm1_ptr[2*0] = <void*>_func_ellpk
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4620
 * ufunc_ellipkm1_types[1] = <char>NPY_FLOAT
 * ufunc_ellipkm1_types[2] = <char>NPY_DOUBLE
 * ufunc_ellipkm1_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_ptr[2*0] = <void*>_func_ellpk
 * ufunc_ellipkm1_ptr[2*0+1] = <void*>(<char*>"ellipkm1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4621
 * ufunc_ellipkm1_types[2] = <char>NPY_DOUBLE
 * ufunc_ellipkm1_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipkm1_ptr[2*0] = <void*>_func_ellpk             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_ptr[2*0+1] = <void*>(<char*>"ellipkm1")
 * ufunc_ellipkm1_ptr[2*1] = <void*>_func_ellpk
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_ptr[0]) = ((void *)ellpk);

  /* "scipy/special/_ufuncs.pyx":4622
 * ufunc_ellipkm1_types[3] = <char>NPY_DOUBLE
 * ufunc_ellipkm1_ptr[2*0] = <void*>_func_ellpk
 * ufunc_ellipkm1_ptr[2*0+1] = <void*>(<char*>"ellipkm1")             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_ptr[2*1] = <void*>_func_ellpk
 * ufunc_ellipkm1_ptr[2*1+1] = <void*>(<char*>"ellipkm1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_ptr[1]) = ((void *)((char *)((char *)"ellipkm1")));

  /* "scipy/special/_ufuncs.pyx":4623
 * ufunc_ellipkm1_ptr[2*0] = <void*>_func_ellpk
 * ufunc_ellipkm1_ptr[2*0+1] = <void*>(<char*>"ellipkm1")
 * ufunc_ellipkm1_ptr[2*1] = <void*>_func_ellpk             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_ptr[2*1+1] = <void*>(<char*>"ellipkm1")
 * ufunc_ellipkm1_data[0] = &ufunc_ellipkm1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_ptr[2]) = ((void *)ellpk);

  /* "scipy/special/_ufuncs.pyx":4624
 * ufunc_ellipkm1_ptr[2*0+1] = <void*>(<char*>"ellipkm1")
 * ufunc_ellipkm1_ptr[2*1] = <void*>_func_ellpk
 * ufunc_ellipkm1_ptr[2*1+1] = <void*>(<char*>"ellipkm1")             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_data[0] = &ufunc_ellipkm1_ptr[2*0]
 * ufunc_ellipkm1_data[1] = &ufunc_ellipkm1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_ptr[3]) = ((void *)((char *)((char *)"ellipkm1")));

  /* "scipy/special/_ufuncs.pyx":4625
 * ufunc_ellipkm1_ptr[2*1] = <void*>_func_ellpk
 * ufunc_ellipkm1_ptr[2*1+1] = <void*>(<char*>"ellipkm1")
 * ufunc_ellipkm1_data[0] = &ufunc_ellipkm1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ellipkm1_data[1] = &ufunc_ellipkm1_ptr[2*1]
 * ellipkm1 = np.PyUFunc_FromFuncAndData(ufunc_ellipkm1_loops, ufunc_ellipkm1_data, ufunc_ellipkm1_types, 2, 1, 1, 0, "ellipkm1", ufunc_ellipkm1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4626
 * ufunc_ellipkm1_ptr[2*1+1] = <void*>(<char*>"ellipkm1")
 * ufunc_ellipkm1_data[0] = &ufunc_ellipkm1_ptr[2*0]
 * ufunc_ellipkm1_data[1] = &ufunc_ellipkm1_ptr[2*1]             # <<<<<<<<<<<<<<
 * ellipkm1 = np.PyUFunc_FromFuncAndData(ufunc_ellipkm1_loops, ufunc_ellipkm1_data, ufunc_ellipkm1_types, 2, 1, 1, 0, "ellipkm1", ufunc_ellipkm1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4627
 * ufunc_ellipkm1_data[0] = &ufunc_ellipkm1_ptr[2*0]
 * ufunc_ellipkm1_data[1] = &ufunc_ellipkm1_ptr[2*1]
 * ellipkm1 = np.PyUFunc_FromFuncAndData(ufunc_ellipkm1_loops, ufunc_ellipkm1_data, ufunc_ellipkm1_types, 2, 1, 1, 0, "ellipkm1", ufunc_ellipkm1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_entr_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_types, 2, 1, 1, 0, ((char *)"ellipkm1"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_ellipkm1_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ellipkm1, __pyx_t_2) < 0) __PYX_ERR(1, 4627, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4633
 * cdef void *ufunc_entr_data[2]
 * cdef char ufunc_entr_types[4]
 * cdef char *ufunc_entr_doc = (             # <<<<<<<<<<<<<<
 *     "entr(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_doc = ((char *)"entr(x)\n\nElementwise function for computing entropy.\n\n.. math:: \\text{entr}(x) = \\begin{cases} - x \\log(x) & x > 0  \\\\ 0 & x = 0 \\\\ -\\infty & \\text{otherwise} \\end{cases}\n\nParameters\n----------\nx : ndarray\n    Input array.\n\nReturns\n-------\nres : ndarray\n    The value of the elementwise entropy function at the given points `x`.\n\nSee Also\n--------\nkl_div, rel_entr\n\nNotes\n-----\nThis function is concave.\n\n.. versionadded:: 0.15.0");

  /* "scipy/special/_ufuncs.pyx":4659
 *     "\n"
 *     ".. versionadded:: 0.15.0")
 * ufunc_entr_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_entr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_entr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4660
 *     ".. versionadded:: 0.15.0")
 * ufunc_entr_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_entr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_entr_types[0] = <char>NPY_FLOAT
 * ufunc_entr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4661
 * ufunc_entr_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_entr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_entr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_entr_types[1] = <char>NPY_FLOAT
 * ufunc_entr_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4662
 * ufunc_entr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_entr_types[0] = <char>NPY_FLOAT
 * ufunc_entr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_entr_types[2] = <char>NPY_DOUBLE
 * ufunc_entr_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4663
 * ufunc_entr_types[0] = <char>NPY_FLOAT
 * ufunc_entr_types[1] = <char>NPY_FLOAT
 * ufunc_entr_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_entr_types[3] = <char>NPY_DOUBLE
 * ufunc_entr_ptr[2*0] = <void*>_func_entr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4664
 * ufunc_entr_types[1] = <char>NPY_FLOAT
 * ufunc_entr_types[2] = <char>NPY_DOUBLE
 * ufunc_entr_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_entr_ptr[2*0] = <void*>_func_entr
 * ufunc_entr_ptr[2*0+1] = <void*>(<char*>"entr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4665
 * ufunc_entr_types[2] = <char>NPY_DOUBLE
 * ufunc_entr_types[3] = <char>NPY_DOUBLE
 * ufunc_entr_ptr[2*0] = <void*>_func_entr             # <<<<<<<<<<<<<<
 * ufunc_entr_ptr[2*0+1] = <void*>(<char*>"entr")
 * ufunc_entr_ptr[2*1] = <void*>_func_entr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_ptr[0]) = ((void *)__pyx_f_5scipy_7special_16_convex_analysis_entr);

  /* "scipy/special/_ufuncs.pyx":4666
 * ufunc_entr_types[3] = <char>NPY_DOUBLE
 * ufunc_entr_ptr[2*0] = <void*>_func_entr
 * ufunc_entr_ptr[2*0+1] = <void*>(<char*>"entr")             # <<<<<<<<<<<<<<
 * ufunc_entr_ptr[2*1] = <void*>_func_entr
 * ufunc_entr_ptr[2*1+1] = <void*>(<char*>"entr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_ptr[1]) = ((void *)((char *)((char *)"entr")));

  /* "scipy/special/_ufuncs.pyx":4667
 * ufunc_entr_ptr[2*0] = <void*>_func_entr
 * ufunc_entr_ptr[2*0+1] = <void*>(<char*>"entr")
 * ufunc_entr_ptr[2*1] = <void*>_func_entr             # <<<<<<<<<<<<<<
 * ufunc_entr_ptr[2*1+1] = <void*>(<char*>"entr")
 * ufunc_entr_data[0] = &ufunc_entr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_ptr[2]) = ((void *)__pyx_f_5scipy_7special_16_convex_analysis_entr);

  /* "scipy/special/_ufuncs.pyx":4668
 * ufunc_entr_ptr[2*0+1] = <void*>(<char*>"entr")
 * ufunc_entr_ptr[2*1] = <void*>_func_entr
 * ufunc_entr_ptr[2*1+1] = <void*>(<char*>"entr")             # <<<<<<<<<<<<<<
 * ufunc_entr_data[0] = &ufunc_entr_ptr[2*0]
 * ufunc_entr_data[1] = &ufunc_entr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_ptr[3]) = ((void *)((char *)((char *)"entr")));

  /* "scipy/special/_ufuncs.pyx":4669
 * ufunc_entr_ptr[2*1] = <void*>_func_entr
 * ufunc_entr_ptr[2*1+1] = <void*>(<char*>"entr")
 * ufunc_entr_data[0] = &ufunc_entr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_entr_data[1] = &ufunc_entr_ptr[2*1]
 * entr = np.PyUFunc_FromFuncAndData(ufunc_entr_loops, ufunc_entr_data, ufunc_entr_types, 2, 1, 1, 0, "entr", ufunc_entr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4670
 * ufunc_entr_ptr[2*1+1] = <void*>(<char*>"entr")
 * ufunc_entr_data[0] = &ufunc_entr_ptr[2*0]
 * ufunc_entr_data[1] = &ufunc_entr_ptr[2*1]             # <<<<<<<<<<<<<<
 * entr = np.PyUFunc_FromFuncAndData(ufunc_entr_loops, ufunc_entr_data, ufunc_entr_types, 2, 1, 1, 0, "entr", ufunc_entr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4671
 * ufunc_entr_data[0] = &ufunc_entr_ptr[2*0]
 * ufunc_entr_data[1] = &ufunc_entr_ptr[2*1]
 * entr = np.PyUFunc_FromFuncAndData(ufunc_entr_loops, ufunc_entr_data, ufunc_entr_types, 2, 1, 1, 0, "entr", ufunc_entr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_erf_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_types, 2, 1, 1, 0, ((char *)"entr"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_entr_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4671, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_entr, __pyx_t_2) < 0) __PYX_ERR(1, 4671, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4677
 * cdef void *ufunc_erf_data[4]
 * cdef char ufunc_erf_types[8]
 * cdef char *ufunc_erf_doc = (             # <<<<<<<<<<<<<<
 *     "erf(z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_doc = ((char *)"erf(z)\n\nReturns the error function of complex argument.\n\nIt is defined as ``2/sqrt(pi)*integral(exp(-t**2), t=0..z)``.\n\nParameters\n----------\nx : ndarray\n    Input array.\n\nReturns\n-------\nres : ndarray\n    The values of the error function at the given points `x`.\n\nSee Also\n--------\nerfc, erfinv, erfcinv, wofz, erfcx, erfi\n\nNotes\n-----\nThe cumulative of the unit normal distribution is given by\n``Phi(z) = 1/2[1 + erf(z/sqrt(2))]``.\n\nReferences\n----------\n.. [1] https://en.wikipedia.org/wiki/Error_function\n.. [2] Milton Abramowitz and Irene A. Stegun, eds.\n    Handbook of Mathematical Functions with Formulas,\n    Graphs, and Mathematical Tables. New York: Dover,\n    1972. http://www.math.sfu.ca/~cbm/aands/page_297.htm\n.. [3] Steven G. Johnson, Faddeeva W function implementation.\n   http://ab-initio.mit.edu/Faddeeva\n\nExamples\n--------\n>>> from scipy import special\n>>> import matplotlib.pyplot as plt\n>>> x = np.linspace(-3, 3)\n>>> plt.plot(x, special.erf(x))\n>>> plt.xlabel('$x$')\n>>> plt.ylabel('$erf(x)$')\n>>> plt.show()");

  /* "scipy/special/_ufuncs.pyx":4722
 *     ">>> plt.ylabel('$erf(x)$')\n"
 *     ">>> plt.show()")
 * ufunc_erf_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_erf_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erf_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4723
 *     ">>> plt.show()")
 * ufunc_erf_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_erf_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_erf_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erf_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4724
 * ufunc_erf_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_erf_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erf_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_erf_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erf_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":4725
 * ufunc_erf_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erf_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erf_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_erf_types[0] = <char>NPY_FLOAT
 * ufunc_erf_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":4726
 * ufunc_erf_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erf_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erf_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_erf_types[1] = <char>NPY_FLOAT
 * ufunc_erf_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4727
 * ufunc_erf_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erf_types[0] = <char>NPY_FLOAT
 * ufunc_erf_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_erf_types[2] = <char>NPY_DOUBLE
 * ufunc_erf_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4728
 * ufunc_erf_types[0] = <char>NPY_FLOAT
 * ufunc_erf_types[1] = <char>NPY_FLOAT
 * ufunc_erf_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erf_types[3] = <char>NPY_DOUBLE
 * ufunc_erf_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4729
 * ufunc_erf_types[1] = <char>NPY_FLOAT
 * ufunc_erf_types[2] = <char>NPY_DOUBLE
 * ufunc_erf_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erf_types[4] = <char>NPY_CFLOAT
 * ufunc_erf_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4730
 * ufunc_erf_types[2] = <char>NPY_DOUBLE
 * ufunc_erf_types[3] = <char>NPY_DOUBLE
 * ufunc_erf_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_erf_types[5] = <char>NPY_CFLOAT
 * ufunc_erf_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4731
 * ufunc_erf_types[3] = <char>NPY_DOUBLE
 * ufunc_erf_types[4] = <char>NPY_CFLOAT
 * ufunc_erf_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_erf_types[6] = <char>NPY_CDOUBLE
 * ufunc_erf_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4732
 * ufunc_erf_types[4] = <char>NPY_CFLOAT
 * ufunc_erf_types[5] = <char>NPY_CFLOAT
 * ufunc_erf_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erf_types[7] = <char>NPY_CDOUBLE
 * ufunc_erf_ptr[2*0] = <void*>_func_erf
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4733
 * ufunc_erf_types[5] = <char>NPY_CFLOAT
 * ufunc_erf_types[6] = <char>NPY_CDOUBLE
 * ufunc_erf_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erf_ptr[2*0] = <void*>_func_erf
 * ufunc_erf_ptr[2*0+1] = <void*>(<char*>"erf")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4734
 * ufunc_erf_types[6] = <char>NPY_CDOUBLE
 * ufunc_erf_types[7] = <char>NPY_CDOUBLE
 * ufunc_erf_ptr[2*0] = <void*>_func_erf             # <<<<<<<<<<<<<<
 * ufunc_erf_ptr[2*0+1] = <void*>(<char*>"erf")
 * ufunc_erf_ptr[2*1] = <void*>_func_erf
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[0]) = ((void *)erf);

  /* "scipy/special/_ufuncs.pyx":4735
 * ufunc_erf_types[7] = <char>NPY_CDOUBLE
 * ufunc_erf_ptr[2*0] = <void*>_func_erf
 * ufunc_erf_ptr[2*0+1] = <void*>(<char*>"erf")             # <<<<<<<<<<<<<<
 * ufunc_erf_ptr[2*1] = <void*>_func_erf
 * ufunc_erf_ptr[2*1+1] = <void*>(<char*>"erf")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[1]) = ((void *)((char *)((char *)"erf")));

  /* "scipy/special/_ufuncs.pyx":4736
 * ufunc_erf_ptr[2*0] = <void*>_func_erf
 * ufunc_erf_ptr[2*0+1] = <void*>(<char*>"erf")
 * ufunc_erf_ptr[2*1] = <void*>_func_erf             # <<<<<<<<<<<<<<
 * ufunc_erf_ptr[2*1+1] = <void*>(<char*>"erf")
 * ufunc_erf_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erf
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[2]) = ((void *)erf);

  /* "scipy/special/_ufuncs.pyx":4737
 * ufunc_erf_ptr[2*0+1] = <void*>(<char*>"erf")
 * ufunc_erf_ptr[2*1] = <void*>_func_erf
 * ufunc_erf_ptr[2*1+1] = <void*>(<char*>"erf")             # <<<<<<<<<<<<<<
 * ufunc_erf_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erf
 * ufunc_erf_ptr[2*2+1] = <void*>(<char*>"erf")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[3]) = ((void *)((char *)((char *)"erf")));

  /* "scipy/special/_ufuncs.pyx":4738
 * ufunc_erf_ptr[2*1] = <void*>_func_erf
 * ufunc_erf_ptr[2*1+1] = <void*>(<char*>"erf")
 * ufunc_erf_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erf             # <<<<<<<<<<<<<<
 * ufunc_erf_ptr[2*2+1] = <void*>(<char*>"erf")
 * ufunc_erf_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erf
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[4]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erf);

  /* "scipy/special/_ufuncs.pyx":4739
 * ufunc_erf_ptr[2*1+1] = <void*>(<char*>"erf")
 * ufunc_erf_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erf
 * ufunc_erf_ptr[2*2+1] = <void*>(<char*>"erf")             # <<<<<<<<<<<<<<
 * ufunc_erf_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erf
 * ufunc_erf_ptr[2*3+1] = <void*>(<char*>"erf")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[5]) = ((void *)((char *)((char *)"erf")));

  /* "scipy/special/_ufuncs.pyx":4740
 * ufunc_erf_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erf
 * ufunc_erf_ptr[2*2+1] = <void*>(<char*>"erf")
 * ufunc_erf_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erf             # <<<<<<<<<<<<<<
 * ufunc_erf_ptr[2*3+1] = <void*>(<char*>"erf")
 * ufunc_erf_data[0] = &ufunc_erf_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[6]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erf);

  /* "scipy/special/_ufuncs.pyx":4741
 * ufunc_erf_ptr[2*2+1] = <void*>(<char*>"erf")
 * ufunc_erf_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erf
 * ufunc_erf_ptr[2*3+1] = <void*>(<char*>"erf")             # <<<<<<<<<<<<<<
 * ufunc_erf_data[0] = &ufunc_erf_ptr[2*0]
 * ufunc_erf_data[1] = &ufunc_erf_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[7]) = ((void *)((char *)((char *)"erf")));

  /* "scipy/special/_ufuncs.pyx":4742
 * ufunc_erf_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erf
 * ufunc_erf_ptr[2*3+1] = <void*>(<char*>"erf")
 * ufunc_erf_data[0] = &ufunc_erf_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_erf_data[1] = &ufunc_erf_ptr[2*1]
 * ufunc_erf_data[2] = &ufunc_erf_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4743
 * ufunc_erf_ptr[2*3+1] = <void*>(<char*>"erf")
 * ufunc_erf_data[0] = &ufunc_erf_ptr[2*0]
 * ufunc_erf_data[1] = &ufunc_erf_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_erf_data[2] = &ufunc_erf_ptr[2*2]
 * ufunc_erf_data[3] = &ufunc_erf_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4744
 * ufunc_erf_data[0] = &ufunc_erf_ptr[2*0]
 * ufunc_erf_data[1] = &ufunc_erf_ptr[2*1]
 * ufunc_erf_data[2] = &ufunc_erf_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_erf_data[3] = &ufunc_erf_ptr[2*3]
 * erf = np.PyUFunc_FromFuncAndData(ufunc_erf_loops, ufunc_erf_data, ufunc_erf_types, 4, 1, 1, 0, "erf", ufunc_erf_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":4745
 * ufunc_erf_data[1] = &ufunc_erf_ptr[2*1]
 * ufunc_erf_data[2] = &ufunc_erf_ptr[2*2]
 * ufunc_erf_data[3] = &ufunc_erf_ptr[2*3]             # <<<<<<<<<<<<<<
 * erf = np.PyUFunc_FromFuncAndData(ufunc_erf_loops, ufunc_erf_data, ufunc_erf_types, 4, 1, 1, 0, "erf", ufunc_erf_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":4746
 * ufunc_erf_data[2] = &ufunc_erf_ptr[2*2]
 * ufunc_erf_data[3] = &ufunc_erf_ptr[2*3]
 * erf = np.PyUFunc_FromFuncAndData(ufunc_erf_loops, ufunc_erf_data, ufunc_erf_types, 4, 1, 1, 0, "erf", ufunc_erf_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_erfc_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_types, 4, 1, 1, 0, ((char *)"erf"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_erf_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4746, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_erf, __pyx_t_2) < 0) __PYX_ERR(1, 4746, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4752
 * cdef void *ufunc_erfc_data[4]
 * cdef char ufunc_erfc_types[8]
 * cdef char *ufunc_erfc_doc = (             # <<<<<<<<<<<<<<
 *     "erfc(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_doc = ((char *)"erfc(x)\n\nComplementary error function, ``1 - erf(x)``.\n\nSee Also\n--------\nerf, erfi, erfcx, dawsn, wofz\n\nReferences\n----------\n.. [1] Steven G. Johnson, Faddeeva W function implementation.\n   http://ab-initio.mit.edu/Faddeeva\n\nExamples\n--------\n>>> from scipy import special\n>>> import matplotlib.pyplot as plt\n>>> x = np.linspace(-3, 3)\n>>> plt.plot(x, special.erfc(x))\n>>> plt.xlabel('$x$')\n>>> plt.ylabel('$erfc(x)$')\n>>> plt.show()");

  /* "scipy/special/_ufuncs.pyx":4775
 *     ">>> plt.ylabel('$erfc(x)$')\n"
 *     ">>> plt.show()")
 * ufunc_erfc_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_erfc_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erfc_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4776
 *     ">>> plt.show()")
 * ufunc_erfc_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_erfc_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_erfc_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erfc_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4777
 * ufunc_erfc_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_erfc_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erfc_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_erfc_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erfc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":4778
 * ufunc_erfc_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erfc_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erfc_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_erfc_types[0] = <char>NPY_FLOAT
 * ufunc_erfc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":4779
 * ufunc_erfc_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erfc_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erfc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfc_types[1] = <char>NPY_FLOAT
 * ufunc_erfc_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4780
 * ufunc_erfc_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erfc_types[0] = <char>NPY_FLOAT
 * ufunc_erfc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfc_types[2] = <char>NPY_DOUBLE
 * ufunc_erfc_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4781
 * ufunc_erfc_types[0] = <char>NPY_FLOAT
 * ufunc_erfc_types[1] = <char>NPY_FLOAT
 * ufunc_erfc_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfc_types[3] = <char>NPY_DOUBLE
 * ufunc_erfc_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4782
 * ufunc_erfc_types[1] = <char>NPY_FLOAT
 * ufunc_erfc_types[2] = <char>NPY_DOUBLE
 * ufunc_erfc_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfc_types[4] = <char>NPY_CFLOAT
 * ufunc_erfc_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4783
 * ufunc_erfc_types[2] = <char>NPY_DOUBLE
 * ufunc_erfc_types[3] = <char>NPY_DOUBLE
 * ufunc_erfc_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfc_types[5] = <char>NPY_CFLOAT
 * ufunc_erfc_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4784
 * ufunc_erfc_types[3] = <char>NPY_DOUBLE
 * ufunc_erfc_types[4] = <char>NPY_CFLOAT
 * ufunc_erfc_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfc_types[6] = <char>NPY_CDOUBLE
 * ufunc_erfc_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4785
 * ufunc_erfc_types[4] = <char>NPY_CFLOAT
 * ufunc_erfc_types[5] = <char>NPY_CFLOAT
 * ufunc_erfc_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfc_types[7] = <char>NPY_CDOUBLE
 * ufunc_erfc_ptr[2*0] = <void*>_func_erfc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4786
 * ufunc_erfc_types[5] = <char>NPY_CFLOAT
 * ufunc_erfc_types[6] = <char>NPY_CDOUBLE
 * ufunc_erfc_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfc_ptr[2*0] = <void*>_func_erfc
 * ufunc_erfc_ptr[2*0+1] = <void*>(<char*>"erfc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4787
 * ufunc_erfc_types[6] = <char>NPY_CDOUBLE
 * ufunc_erfc_types[7] = <char>NPY_CDOUBLE
 * ufunc_erfc_ptr[2*0] = <void*>_func_erfc             # <<<<<<<<<<<<<<
 * ufunc_erfc_ptr[2*0+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_ptr[2*1] = <void*>_func_erfc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[0]) = ((void *)erfc);

  /* "scipy/special/_ufuncs.pyx":4788
 * ufunc_erfc_types[7] = <char>NPY_CDOUBLE
 * ufunc_erfc_ptr[2*0] = <void*>_func_erfc
 * ufunc_erfc_ptr[2*0+1] = <void*>(<char*>"erfc")             # <<<<<<<<<<<<<<
 * ufunc_erfc_ptr[2*1] = <void*>_func_erfc
 * ufunc_erfc_ptr[2*1+1] = <void*>(<char*>"erfc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[1]) = ((void *)((char *)((char *)"erfc")));

  /* "scipy/special/_ufuncs.pyx":4789
 * ufunc_erfc_ptr[2*0] = <void*>_func_erfc
 * ufunc_erfc_ptr[2*0+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_ptr[2*1] = <void*>_func_erfc             # <<<<<<<<<<<<<<
 * ufunc_erfc_ptr[2*1+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[2]) = ((void *)erfc);

  /* "scipy/special/_ufuncs.pyx":4790
 * ufunc_erfc_ptr[2*0+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_ptr[2*1] = <void*>_func_erfc
 * ufunc_erfc_ptr[2*1+1] = <void*>(<char*>"erfc")             # <<<<<<<<<<<<<<
 * ufunc_erfc_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfc
 * ufunc_erfc_ptr[2*2+1] = <void*>(<char*>"erfc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[3]) = ((void *)((char *)((char *)"erfc")));

  /* "scipy/special/_ufuncs.pyx":4791
 * ufunc_erfc_ptr[2*1] = <void*>_func_erfc
 * ufunc_erfc_ptr[2*1+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfc             # <<<<<<<<<<<<<<
 * ufunc_erfc_ptr[2*2+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[4]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfc);

  /* "scipy/special/_ufuncs.pyx":4792
 * ufunc_erfc_ptr[2*1+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfc
 * ufunc_erfc_ptr[2*2+1] = <void*>(<char*>"erfc")             # <<<<<<<<<<<<<<
 * ufunc_erfc_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfc
 * ufunc_erfc_ptr[2*3+1] = <void*>(<char*>"erfc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[5]) = ((void *)((char *)((char *)"erfc")));

  /* "scipy/special/_ufuncs.pyx":4793
 * ufunc_erfc_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfc
 * ufunc_erfc_ptr[2*2+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfc             # <<<<<<<<<<<<<<
 * ufunc_erfc_ptr[2*3+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_data[0] = &ufunc_erfc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[6]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfc);

  /* "scipy/special/_ufuncs.pyx":4794
 * ufunc_erfc_ptr[2*2+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfc
 * ufunc_erfc_ptr[2*3+1] = <void*>(<char*>"erfc")             # <<<<<<<<<<<<<<
 * ufunc_erfc_data[0] = &ufunc_erfc_ptr[2*0]
 * ufunc_erfc_data[1] = &ufunc_erfc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[7]) = ((void *)((char *)((char *)"erfc")));

  /* "scipy/special/_ufuncs.pyx":4795
 * ufunc_erfc_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfc
 * ufunc_erfc_ptr[2*3+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_data[0] = &ufunc_erfc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_erfc_data[1] = &ufunc_erfc_ptr[2*1]
 * ufunc_erfc_data[2] = &ufunc_erfc_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4796
 * ufunc_erfc_ptr[2*3+1] = <void*>(<char*>"erfc")
 * ufunc_erfc_data[0] = &ufunc_erfc_ptr[2*0]
 * ufunc_erfc_data[1] = &ufunc_erfc_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_erfc_data[2] = &ufunc_erfc_ptr[2*2]
 * ufunc_erfc_data[3] = &ufunc_erfc_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4797
 * ufunc_erfc_data[0] = &ufunc_erfc_ptr[2*0]
 * ufunc_erfc_data[1] = &ufunc_erfc_ptr[2*1]
 * ufunc_erfc_data[2] = &ufunc_erfc_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_erfc_data[3] = &ufunc_erfc_ptr[2*3]
 * erfc = np.PyUFunc_FromFuncAndData(ufunc_erfc_loops, ufunc_erfc_data, ufunc_erfc_types, 4, 1, 1, 0, "erfc", ufunc_erfc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":4798
 * ufunc_erfc_data[1] = &ufunc_erfc_ptr[2*1]
 * ufunc_erfc_data[2] = &ufunc_erfc_ptr[2*2]
 * ufunc_erfc_data[3] = &ufunc_erfc_ptr[2*3]             # <<<<<<<<<<<<<<
 * erfc = np.PyUFunc_FromFuncAndData(ufunc_erfc_loops, ufunc_erfc_data, ufunc_erfc_types, 4, 1, 1, 0, "erfc", ufunc_erfc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":4799
 * ufunc_erfc_data[2] = &ufunc_erfc_ptr[2*2]
 * ufunc_erfc_data[3] = &ufunc_erfc_ptr[2*3]
 * erfc = np.PyUFunc_FromFuncAndData(ufunc_erfc_loops, ufunc_erfc_data, ufunc_erfc_types, 4, 1, 1, 0, "erfc", ufunc_erfc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_erfcx_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_types, 4, 1, 1, 0, ((char *)"erfc"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfc_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4799, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_erfc, __pyx_t_2) < 0) __PYX_ERR(1, 4799, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4805
 * cdef void *ufunc_erfcx_data[4]
 * cdef char ufunc_erfcx_types[8]
 * cdef char *ufunc_erfcx_doc = (             # <<<<<<<<<<<<<<
 *     "erfcx(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_doc = ((char *)"erfcx(x)\n\nScaled complementary error function, ``exp(x**2) * erfc(x)``.\n\nSee Also\n--------\nerf, erfc, erfi, dawsn, wofz\n\nNotes\n-----\n\n.. versionadded:: 0.12.0\n\nReferences\n----------\n.. [1] Steven G. Johnson, Faddeeva W function implementation.\n   http://ab-initio.mit.edu/Faddeeva\n\nExamples\n--------\n>>> from scipy import special\n>>> import matplotlib.pyplot as plt\n>>> x = np.linspace(-3, 3)\n>>> plt.plot(x, special.erfcx(x))\n>>> plt.xlabel('$x$')\n>>> plt.ylabel('$erfcx(x)$')\n>>> plt.show()");

  /* "scipy/special/_ufuncs.pyx":4833
 *     ">>> plt.ylabel('$erfcx(x)$')\n"
 *     ">>> plt.show()")
 * ufunc_erfcx_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_erfcx_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erfcx_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4834
 *     ">>> plt.show()")
 * ufunc_erfcx_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_erfcx_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_erfcx_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erfcx_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4835
 * ufunc_erfcx_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_erfcx_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erfcx_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_erfcx_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erfcx_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":4836
 * ufunc_erfcx_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erfcx_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erfcx_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_erfcx_types[0] = <char>NPY_FLOAT
 * ufunc_erfcx_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":4837
 * ufunc_erfcx_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erfcx_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erfcx_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfcx_types[1] = <char>NPY_FLOAT
 * ufunc_erfcx_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4838
 * ufunc_erfcx_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erfcx_types[0] = <char>NPY_FLOAT
 * ufunc_erfcx_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfcx_types[2] = <char>NPY_DOUBLE
 * ufunc_erfcx_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4839
 * ufunc_erfcx_types[0] = <char>NPY_FLOAT
 * ufunc_erfcx_types[1] = <char>NPY_FLOAT
 * ufunc_erfcx_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfcx_types[3] = <char>NPY_DOUBLE
 * ufunc_erfcx_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4840
 * ufunc_erfcx_types[1] = <char>NPY_FLOAT
 * ufunc_erfcx_types[2] = <char>NPY_DOUBLE
 * ufunc_erfcx_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfcx_types[4] = <char>NPY_CFLOAT
 * ufunc_erfcx_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4841
 * ufunc_erfcx_types[2] = <char>NPY_DOUBLE
 * ufunc_erfcx_types[3] = <char>NPY_DOUBLE
 * ufunc_erfcx_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfcx_types[5] = <char>NPY_CFLOAT
 * ufunc_erfcx_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4842
 * ufunc_erfcx_types[3] = <char>NPY_DOUBLE
 * ufunc_erfcx_types[4] = <char>NPY_CFLOAT
 * ufunc_erfcx_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfcx_types[6] = <char>NPY_CDOUBLE
 * ufunc_erfcx_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4843
 * ufunc_erfcx_types[4] = <char>NPY_CFLOAT
 * ufunc_erfcx_types[5] = <char>NPY_CFLOAT
 * ufunc_erfcx_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfcx_types[7] = <char>NPY_CDOUBLE
 * ufunc_erfcx_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4844
 * ufunc_erfcx_types[5] = <char>NPY_CFLOAT
 * ufunc_erfcx_types[6] = <char>NPY_CDOUBLE
 * ufunc_erfcx_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfcx_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx
 * ufunc_erfcx_ptr[2*0+1] = <void*>(<char*>"erfcx")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4845
 * ufunc_erfcx_types[6] = <char>NPY_CDOUBLE
 * ufunc_erfcx_types[7] = <char>NPY_CDOUBLE
 * ufunc_erfcx_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx             # <<<<<<<<<<<<<<
 * ufunc_erfcx_ptr[2*0+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[0]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx);

  /* "scipy/special/_ufuncs.pyx":4846
 * ufunc_erfcx_types[7] = <char>NPY_CDOUBLE
 * ufunc_erfcx_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx
 * ufunc_erfcx_ptr[2*0+1] = <void*>(<char*>"erfcx")             # <<<<<<<<<<<<<<
 * ufunc_erfcx_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx
 * ufunc_erfcx_ptr[2*1+1] = <void*>(<char*>"erfcx")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[1]) = ((void *)((char *)((char *)"erfcx")));

  /* "scipy/special/_ufuncs.pyx":4847
 * ufunc_erfcx_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx
 * ufunc_erfcx_ptr[2*0+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx             # <<<<<<<<<<<<<<
 * ufunc_erfcx_ptr[2*1+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx_complex
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[2]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx);

  /* "scipy/special/_ufuncs.pyx":4848
 * ufunc_erfcx_ptr[2*0+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx
 * ufunc_erfcx_ptr[2*1+1] = <void*>(<char*>"erfcx")             # <<<<<<<<<<<<<<
 * ufunc_erfcx_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx_complex
 * ufunc_erfcx_ptr[2*2+1] = <void*>(<char*>"erfcx")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[3]) = ((void *)((char *)((char *)"erfcx")));

  /* "scipy/special/_ufuncs.pyx":4849
 * ufunc_erfcx_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx
 * ufunc_erfcx_ptr[2*1+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx_complex             # <<<<<<<<<<<<<<
 * ufunc_erfcx_ptr[2*2+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx_complex
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[4]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx_complex);

  /* "scipy/special/_ufuncs.pyx":4850
 * ufunc_erfcx_ptr[2*1+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx_complex
 * ufunc_erfcx_ptr[2*2+1] = <void*>(<char*>"erfcx")             # <<<<<<<<<<<<<<
 * ufunc_erfcx_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx_complex
 * ufunc_erfcx_ptr[2*3+1] = <void*>(<char*>"erfcx")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[5]) = ((void *)((char *)((char *)"erfcx")));

  /* "scipy/special/_ufuncs.pyx":4851
 * ufunc_erfcx_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx_complex
 * ufunc_erfcx_ptr[2*2+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx_complex             # <<<<<<<<<<<<<<
 * ufunc_erfcx_ptr[2*3+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_data[0] = &ufunc_erfcx_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[6]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfcx_complex);

  /* "scipy/special/_ufuncs.pyx":4852
 * ufunc_erfcx_ptr[2*2+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx_complex
 * ufunc_erfcx_ptr[2*3+1] = <void*>(<char*>"erfcx")             # <<<<<<<<<<<<<<
 * ufunc_erfcx_data[0] = &ufunc_erfcx_ptr[2*0]
 * ufunc_erfcx_data[1] = &ufunc_erfcx_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[7]) = ((void *)((char *)((char *)"erfcx")));

  /* "scipy/special/_ufuncs.pyx":4853
 * ufunc_erfcx_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfcx_complex
 * ufunc_erfcx_ptr[2*3+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_data[0] = &ufunc_erfcx_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_erfcx_data[1] = &ufunc_erfcx_ptr[2*1]
 * ufunc_erfcx_data[2] = &ufunc_erfcx_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4854
 * ufunc_erfcx_ptr[2*3+1] = <void*>(<char*>"erfcx")
 * ufunc_erfcx_data[0] = &ufunc_erfcx_ptr[2*0]
 * ufunc_erfcx_data[1] = &ufunc_erfcx_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_erfcx_data[2] = &ufunc_erfcx_ptr[2*2]
 * ufunc_erfcx_data[3] = &ufunc_erfcx_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4855
 * ufunc_erfcx_data[0] = &ufunc_erfcx_ptr[2*0]
 * ufunc_erfcx_data[1] = &ufunc_erfcx_ptr[2*1]
 * ufunc_erfcx_data[2] = &ufunc_erfcx_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_erfcx_data[3] = &ufunc_erfcx_ptr[2*3]
 * erfcx = np.PyUFunc_FromFuncAndData(ufunc_erfcx_loops, ufunc_erfcx_data, ufunc_erfcx_types, 4, 1, 1, 0, "erfcx", ufunc_erfcx_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":4856
 * ufunc_erfcx_data[1] = &ufunc_erfcx_ptr[2*1]
 * ufunc_erfcx_data[2] = &ufunc_erfcx_ptr[2*2]
 * ufunc_erfcx_data[3] = &ufunc_erfcx_ptr[2*3]             # <<<<<<<<<<<<<<
 * erfcx = np.PyUFunc_FromFuncAndData(ufunc_erfcx_loops, ufunc_erfcx_data, ufunc_erfcx_types, 4, 1, 1, 0, "erfcx", ufunc_erfcx_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":4857
 * ufunc_erfcx_data[2] = &ufunc_erfcx_ptr[2*2]
 * ufunc_erfcx_data[3] = &ufunc_erfcx_ptr[2*3]
 * erfcx = np.PyUFunc_FromFuncAndData(ufunc_erfcx_loops, ufunc_erfcx_data, ufunc_erfcx_types, 4, 1, 1, 0, "erfcx", ufunc_erfcx_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_erfi_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_types, 4, 1, 1, 0, ((char *)"erfcx"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfcx_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4857, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_erfcx, __pyx_t_2) < 0) __PYX_ERR(1, 4857, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4863
 * cdef void *ufunc_erfi_data[4]
 * cdef char ufunc_erfi_types[8]
 * cdef char *ufunc_erfi_doc = (             # <<<<<<<<<<<<<<
 *     "erfi(z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_doc = ((char *)"erfi(z)\n\nImaginary error function, ``-i erf(i z)``.\n\nSee Also\n--------\nerf, erfc, erfcx, dawsn, wofz\n\nNotes\n-----\n\n.. versionadded:: 0.12.0\n\nReferences\n----------\n.. [1] Steven G. Johnson, Faddeeva W function implementation.\n   http://ab-initio.mit.edu/Faddeeva\n\nExamples\n--------\n>>> from scipy import special\n>>> import matplotlib.pyplot as plt\n>>> x = np.linspace(-3, 3)\n>>> plt.plot(x, special.erfi(x))\n>>> plt.xlabel('$x$')\n>>> plt.ylabel('$erfi(x)$')\n>>> plt.show()");

  /* "scipy/special/_ufuncs.pyx":4891
 *     ">>> plt.ylabel('$erfi(x)$')\n"
 *     ">>> plt.show()")
 * ufunc_erfi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_erfi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erfi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":4892
 *     ">>> plt.show()")
 * ufunc_erfi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_erfi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_erfi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erfi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":4893
 * ufunc_erfi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_erfi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erfi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_erfi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erfi_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":4894
 * ufunc_erfi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_erfi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erfi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_erfi_types[0] = <char>NPY_FLOAT
 * ufunc_erfi_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":4895
 * ufunc_erfi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_erfi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erfi_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfi_types[1] = <char>NPY_FLOAT
 * ufunc_erfi_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4896
 * ufunc_erfi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_erfi_types[0] = <char>NPY_FLOAT
 * ufunc_erfi_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfi_types[2] = <char>NPY_DOUBLE
 * ufunc_erfi_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4897
 * ufunc_erfi_types[0] = <char>NPY_FLOAT
 * ufunc_erfi_types[1] = <char>NPY_FLOAT
 * ufunc_erfi_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfi_types[3] = <char>NPY_DOUBLE
 * ufunc_erfi_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4898
 * ufunc_erfi_types[1] = <char>NPY_FLOAT
 * ufunc_erfi_types[2] = <char>NPY_DOUBLE
 * ufunc_erfi_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfi_types[4] = <char>NPY_CFLOAT
 * ufunc_erfi_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4899
 * ufunc_erfi_types[2] = <char>NPY_DOUBLE
 * ufunc_erfi_types[3] = <char>NPY_DOUBLE
 * ufunc_erfi_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfi_types[5] = <char>NPY_CFLOAT
 * ufunc_erfi_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4900
 * ufunc_erfi_types[3] = <char>NPY_DOUBLE
 * ufunc_erfi_types[4] = <char>NPY_CFLOAT
 * ufunc_erfi_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_erfi_types[6] = <char>NPY_CDOUBLE
 * ufunc_erfi_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4901
 * ufunc_erfi_types[4] = <char>NPY_CFLOAT
 * ufunc_erfi_types[5] = <char>NPY_CFLOAT
 * ufunc_erfi_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfi_types[7] = <char>NPY_CDOUBLE
 * ufunc_erfi_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4902
 * ufunc_erfi_types[5] = <char>NPY_CFLOAT
 * ufunc_erfi_types[6] = <char>NPY_CDOUBLE
 * ufunc_erfi_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_erfi_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi
 * ufunc_erfi_ptr[2*0+1] = <void*>(<char*>"erfi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4903
 * ufunc_erfi_types[6] = <char>NPY_CDOUBLE
 * ufunc_erfi_types[7] = <char>NPY_CDOUBLE
 * ufunc_erfi_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi             # <<<<<<<<<<<<<<
 * ufunc_erfi_ptr[2*0+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[0]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi);

  /* "scipy/special/_ufuncs.pyx":4904
 * ufunc_erfi_types[7] = <char>NPY_CDOUBLE
 * ufunc_erfi_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi
 * ufunc_erfi_ptr[2*0+1] = <void*>(<char*>"erfi")             # <<<<<<<<<<<<<<
 * ufunc_erfi_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi
 * ufunc_erfi_ptr[2*1+1] = <void*>(<char*>"erfi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[1]) = ((void *)((char *)((char *)"erfi")));

  /* "scipy/special/_ufuncs.pyx":4905
 * ufunc_erfi_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi
 * ufunc_erfi_ptr[2*0+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi             # <<<<<<<<<<<<<<
 * ufunc_erfi_ptr[2*1+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi_complex
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[2]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi);

  /* "scipy/special/_ufuncs.pyx":4906
 * ufunc_erfi_ptr[2*0+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi
 * ufunc_erfi_ptr[2*1+1] = <void*>(<char*>"erfi")             # <<<<<<<<<<<<<<
 * ufunc_erfi_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi_complex
 * ufunc_erfi_ptr[2*2+1] = <void*>(<char*>"erfi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[3]) = ((void *)((char *)((char *)"erfi")));

  /* "scipy/special/_ufuncs.pyx":4907
 * ufunc_erfi_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi
 * ufunc_erfi_ptr[2*1+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi_complex             # <<<<<<<<<<<<<<
 * ufunc_erfi_ptr[2*2+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi_complex
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[4]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi_complex);

  /* "scipy/special/_ufuncs.pyx":4908
 * ufunc_erfi_ptr[2*1+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi_complex
 * ufunc_erfi_ptr[2*2+1] = <void*>(<char*>"erfi")             # <<<<<<<<<<<<<<
 * ufunc_erfi_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi_complex
 * ufunc_erfi_ptr[2*3+1] = <void*>(<char*>"erfi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[5]) = ((void *)((char *)((char *)"erfi")));

  /* "scipy/special/_ufuncs.pyx":4909
 * ufunc_erfi_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi_complex
 * ufunc_erfi_ptr[2*2+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi_complex             # <<<<<<<<<<<<<<
 * ufunc_erfi_ptr[2*3+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_data[0] = &ufunc_erfi_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[6]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_erfi_complex);

  /* "scipy/special/_ufuncs.pyx":4910
 * ufunc_erfi_ptr[2*2+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi_complex
 * ufunc_erfi_ptr[2*3+1] = <void*>(<char*>"erfi")             # <<<<<<<<<<<<<<
 * ufunc_erfi_data[0] = &ufunc_erfi_ptr[2*0]
 * ufunc_erfi_data[1] = &ufunc_erfi_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[7]) = ((void *)((char *)((char *)"erfi")));

  /* "scipy/special/_ufuncs.pyx":4911
 * ufunc_erfi_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_erfi_complex
 * ufunc_erfi_ptr[2*3+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_data[0] = &ufunc_erfi_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_erfi_data[1] = &ufunc_erfi_ptr[2*1]
 * ufunc_erfi_data[2] = &ufunc_erfi_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4912
 * ufunc_erfi_ptr[2*3+1] = <void*>(<char*>"erfi")
 * ufunc_erfi_data[0] = &ufunc_erfi_ptr[2*0]
 * ufunc_erfi_data[1] = &ufunc_erfi_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_erfi_data[2] = &ufunc_erfi_ptr[2*2]
 * ufunc_erfi_data[3] = &ufunc_erfi_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4913
 * ufunc_erfi_data[0] = &ufunc_erfi_ptr[2*0]
 * ufunc_erfi_data[1] = &ufunc_erfi_ptr[2*1]
 * ufunc_erfi_data[2] = &ufunc_erfi_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_erfi_data[3] = &ufunc_erfi_ptr[2*3]
 * erfi = np.PyUFunc_FromFuncAndData(ufunc_erfi_loops, ufunc_erfi_data, ufunc_erfi_types, 4, 1, 1, 0, "erfi", ufunc_erfi_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":4914
 * ufunc_erfi_data[1] = &ufunc_erfi_ptr[2*1]
 * ufunc_erfi_data[2] = &ufunc_erfi_ptr[2*2]
 * ufunc_erfi_data[3] = &ufunc_erfi_ptr[2*3]             # <<<<<<<<<<<<<<
 * erfi = np.PyUFunc_FromFuncAndData(ufunc_erfi_loops, ufunc_erfi_data, ufunc_erfi_types, 4, 1, 1, 0, "erfi", ufunc_erfi_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":4915
 * ufunc_erfi_data[2] = &ufunc_erfi_ptr[2*2]
 * ufunc_erfi_data[3] = &ufunc_erfi_ptr[2*3]
 * erfi = np.PyUFunc_FromFuncAndData(ufunc_erfi_loops, ufunc_erfi_data, ufunc_erfi_types, 4, 1, 1, 0, "erfi", ufunc_erfi_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_chebyc_loops[5]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_types, 4, 1, 1, 0, ((char *)"erfi"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_erfi_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4915, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_erfi, __pyx_t_2) < 0) __PYX_ERR(1, 4915, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4921
 * cdef void *ufunc_eval_chebyc_data[5]
 * cdef char ufunc_eval_chebyc_types[15]
 * cdef char *ufunc_eval_chebyc_doc = (             # <<<<<<<<<<<<<<
 *     "eval_chebyc(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_doc = ((char *)"eval_chebyc(n, x, out=None)\n\nEvaluate Chebyshev polynomial of the first kind on [-2, 2] at a\npoint.\n\nThese polynomials are defined as\n\n.. math::\n\n    S_n(x) = T_n(x/2)\n\nwhere :math:`T_n` is a Chebyshev polynomial of the first kind.\n\nParameters\n----------\nn : array_like\n    Degree of the polynomial. If not an integer, the result is\n    determined via the relation to `eval_chebyt`.\nx : array_like\n    Points at which to evaluate the Chebyshev polynomial\n\nReturns\n-------\nC : ndarray\n    Values of the Chebyshev polynomial\n\nSee Also\n--------\nroots_chebyc : roots and quadrature weights of Chebyshev\n               polynomials of the first kind on [-2, 2]\nchebyc : Chebyshev polynomial object\nnumpy.polynomial.chebyshev.Chebyshev : Chebyshev series\neval_chebyt : evaluate Chebycshev polynomials of the first kind");

  /* "scipy/special/_ufuncs.pyx":4955
 *     "numpy.polynomial.chebyshev.Chebyshev : Chebyshev series\n"
 *     "eval_chebyt : evaluate Chebycshev polynomials of the first kind")
 * ufunc_eval_chebyc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebyc_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":4956
 *     "eval_chebyt : evaluate Chebycshev polynomials of the first kind")
 * ufunc_eval_chebyc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_chebyc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebyc_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":4957
 * ufunc_eval_chebyc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_chebyc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebyc_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebyc_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":4958
 * ufunc_eval_chebyc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebyc_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebyc_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebyc_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":4959
 * ufunc_eval_chebyc_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebyc_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebyc_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[0] = <char>NPY_LONG
 * ufunc_eval_chebyc_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":4960
 * ufunc_eval_chebyc_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebyc_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebyc_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":4961
 * ufunc_eval_chebyc_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebyc_types[0] = <char>NPY_LONG
 * ufunc_eval_chebyc_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4962
 * ufunc_eval_chebyc_types[0] = <char>NPY_LONG
 * ufunc_eval_chebyc_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4963
 * ufunc_eval_chebyc_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4964
 * ufunc_eval_chebyc_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4965
 * ufunc_eval_chebyc_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4966
 * ufunc_eval_chebyc_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebyc_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":4967
 * ufunc_eval_chebyc_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebyc_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4968
 * ufunc_eval_chebyc_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebyc_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebyc_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":4969
 * ufunc_eval_chebyc_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebyc_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebyc_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4970
 * ufunc_eval_chebyc_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebyc_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4971
 * ufunc_eval_chebyc_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[13] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4972
 * ufunc_eval_chebyc_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyc_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":4973
 * ufunc_eval_chebyc_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[13] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyc_ptr[2*0] = <void*>_func_eval_chebyc_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[13]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4974
 * ufunc_eval_chebyc_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebyc_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyc_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_ptr[2*0] = <void*>_func_eval_chebyc_l
 * ufunc_eval_chebyc_ptr[2*0+1] = <void*>(<char*>"eval_chebyc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":4975
 * ufunc_eval_chebyc_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyc_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyc_ptr[2*0] = <void*>_func_eval_chebyc_l             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_ptr[2*0+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*1] = <void*>_func_eval_chebyc[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc_l);

  /* "scipy/special/_ufuncs.pyx":4976
 * ufunc_eval_chebyc_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyc_ptr[2*0] = <void*>_func_eval_chebyc_l
 * ufunc_eval_chebyc_ptr[2*0+1] = <void*>(<char*>"eval_chebyc")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_ptr[2*1] = <void*>_func_eval_chebyc[double]
 * ufunc_eval_chebyc_ptr[2*1+1] = <void*>(<char*>"eval_chebyc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[1]) = ((void *)((char *)((char *)"eval_chebyc")));

  /* "scipy/special/_ufuncs.pyx":4977
 * ufunc_eval_chebyc_ptr[2*0] = <void*>_func_eval_chebyc_l
 * ufunc_eval_chebyc_ptr[2*0+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*1] = <void*>_func_eval_chebyc[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_ptr[2*1+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*2] = <void*>_func_eval_chebyc[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc);

  /* "scipy/special/_ufuncs.pyx":4978
 * ufunc_eval_chebyc_ptr[2*0+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*1] = <void*>_func_eval_chebyc[double]
 * ufunc_eval_chebyc_ptr[2*1+1] = <void*>(<char*>"eval_chebyc")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_ptr[2*2] = <void*>_func_eval_chebyc[double_complex]
 * ufunc_eval_chebyc_ptr[2*2+1] = <void*>(<char*>"eval_chebyc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[3]) = ((void *)((char *)((char *)"eval_chebyc")));

  /* "scipy/special/_ufuncs.pyx":4979
 * ufunc_eval_chebyc_ptr[2*1] = <void*>_func_eval_chebyc[double]
 * ufunc_eval_chebyc_ptr[2*1+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*2] = <void*>_func_eval_chebyc[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_ptr[2*2+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*3] = <void*>_func_eval_chebyc[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc);

  /* "scipy/special/_ufuncs.pyx":4980
 * ufunc_eval_chebyc_ptr[2*1+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*2] = <void*>_func_eval_chebyc[double_complex]
 * ufunc_eval_chebyc_ptr[2*2+1] = <void*>(<char*>"eval_chebyc")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_ptr[2*3] = <void*>_func_eval_chebyc[double]
 * ufunc_eval_chebyc_ptr[2*3+1] = <void*>(<char*>"eval_chebyc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[5]) = ((void *)((char *)((char *)"eval_chebyc")));

  /* "scipy/special/_ufuncs.pyx":4981
 * ufunc_eval_chebyc_ptr[2*2] = <void*>_func_eval_chebyc[double_complex]
 * ufunc_eval_chebyc_ptr[2*2+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*3] = <void*>_func_eval_chebyc[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_ptr[2*3+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*4] = <void*>_func_eval_chebyc[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc);

  /* "scipy/special/_ufuncs.pyx":4982
 * ufunc_eval_chebyc_ptr[2*2+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*3] = <void*>_func_eval_chebyc[double]
 * ufunc_eval_chebyc_ptr[2*3+1] = <void*>(<char*>"eval_chebyc")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_ptr[2*4] = <void*>_func_eval_chebyc[double_complex]
 * ufunc_eval_chebyc_ptr[2*4+1] = <void*>(<char*>"eval_chebyc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[7]) = ((void *)((char *)((char *)"eval_chebyc")));

  /* "scipy/special/_ufuncs.pyx":4983
 * ufunc_eval_chebyc_ptr[2*3] = <void*>_func_eval_chebyc[double]
 * ufunc_eval_chebyc_ptr[2*3+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*4] = <void*>_func_eval_chebyc[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_ptr[2*4+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_data[0] = &ufunc_eval_chebyc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyc);

  /* "scipy/special/_ufuncs.pyx":4984
 * ufunc_eval_chebyc_ptr[2*3+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_ptr[2*4] = <void*>_func_eval_chebyc[double_complex]
 * ufunc_eval_chebyc_ptr[2*4+1] = <void*>(<char*>"eval_chebyc")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_data[0] = &ufunc_eval_chebyc_ptr[2*0]
 * ufunc_eval_chebyc_data[1] = &ufunc_eval_chebyc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[9]) = ((void *)((char *)((char *)"eval_chebyc")));

  /* "scipy/special/_ufuncs.pyx":4985
 * ufunc_eval_chebyc_ptr[2*4] = <void*>_func_eval_chebyc[double_complex]
 * ufunc_eval_chebyc_ptr[2*4+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_data[0] = &ufunc_eval_chebyc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_data[1] = &ufunc_eval_chebyc_ptr[2*1]
 * ufunc_eval_chebyc_data[2] = &ufunc_eval_chebyc_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":4986
 * ufunc_eval_chebyc_ptr[2*4+1] = <void*>(<char*>"eval_chebyc")
 * ufunc_eval_chebyc_data[0] = &ufunc_eval_chebyc_ptr[2*0]
 * ufunc_eval_chebyc_data[1] = &ufunc_eval_chebyc_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_data[2] = &ufunc_eval_chebyc_ptr[2*2]
 * ufunc_eval_chebyc_data[3] = &ufunc_eval_chebyc_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":4987
 * ufunc_eval_chebyc_data[0] = &ufunc_eval_chebyc_ptr[2*0]
 * ufunc_eval_chebyc_data[1] = &ufunc_eval_chebyc_ptr[2*1]
 * ufunc_eval_chebyc_data[2] = &ufunc_eval_chebyc_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_data[3] = &ufunc_eval_chebyc_ptr[2*3]
 * ufunc_eval_chebyc_data[4] = &ufunc_eval_chebyc_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":4988
 * ufunc_eval_chebyc_data[1] = &ufunc_eval_chebyc_ptr[2*1]
 * ufunc_eval_chebyc_data[2] = &ufunc_eval_chebyc_ptr[2*2]
 * ufunc_eval_chebyc_data[3] = &ufunc_eval_chebyc_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyc_data[4] = &ufunc_eval_chebyc_ptr[2*4]
 * eval_chebyc = np.PyUFunc_FromFuncAndData(ufunc_eval_chebyc_loops, ufunc_eval_chebyc_data, ufunc_eval_chebyc_types, 5, 2, 1, 0, "eval_chebyc", ufunc_eval_chebyc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":4989
 * ufunc_eval_chebyc_data[2] = &ufunc_eval_chebyc_ptr[2*2]
 * ufunc_eval_chebyc_data[3] = &ufunc_eval_chebyc_ptr[2*3]
 * ufunc_eval_chebyc_data[4] = &ufunc_eval_chebyc_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_chebyc = np.PyUFunc_FromFuncAndData(ufunc_eval_chebyc_loops, ufunc_eval_chebyc_data, ufunc_eval_chebyc_types, 5, 2, 1, 0, "eval_chebyc", ufunc_eval_chebyc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":4990
 * ufunc_eval_chebyc_data[3] = &ufunc_eval_chebyc_ptr[2*3]
 * ufunc_eval_chebyc_data[4] = &ufunc_eval_chebyc_ptr[2*4]
 * eval_chebyc = np.PyUFunc_FromFuncAndData(ufunc_eval_chebyc_loops, ufunc_eval_chebyc_data, ufunc_eval_chebyc_types, 5, 2, 1, 0, "eval_chebyc", ufunc_eval_chebyc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_chebys_loops[5]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_types, 5, 2, 1, 0, ((char *)"eval_chebyc"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyc_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4990, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_chebyc, __pyx_t_2) < 0) __PYX_ERR(1, 4990, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":4996
 * cdef void *ufunc_eval_chebys_data[5]
 * cdef char ufunc_eval_chebys_types[15]
 * cdef char *ufunc_eval_chebys_doc = (             # <<<<<<<<<<<<<<
 *     "eval_chebys(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_doc = ((char *)"eval_chebys(n, x, out=None)\n\nEvaluate Chebyshev polynomial of the second kind on [-2, 2] at a\npoint.\n\nThese polynomials are defined as\n\n.. math::\n\n    S_n(x) = U_n(x/2)\n\nwhere :math:`U_n` is a Chebyshev polynomial of the second kind.\n\nParameters\n----------\nn : array_like\n    Degree of the polynomial. If not an integer, the result is\n    determined via the relation to `eval_chebyu`.\nx : array_like\n    Points at which to evaluate the Chebyshev polynomial\n\nReturns\n-------\nS : ndarray\n    Values of the Chebyshev polynomial\n\nSee Also\n--------\nroots_chebys : roots and quadrature weights of Chebyshev\n               polynomials of the second kind on [-2, 2]\nchebys : Chebyshev polynomial object\neval_chebyu : evaluate Chebyshev polynomials of the second kind");

  /* "scipy/special/_ufuncs.pyx":5029
 *     "chebys : Chebyshev polynomial object\n"
 *     "eval_chebyu : evaluate Chebyshev polynomials of the second kind")
 * ufunc_eval_chebys_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebys_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":5030
 *     "eval_chebyu : evaluate Chebyshev polynomials of the second kind")
 * ufunc_eval_chebys_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_chebys_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebys_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":5031
 * ufunc_eval_chebys_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_chebys_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebys_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebys_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":5032
 * ufunc_eval_chebys_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebys_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebys_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebys_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":5033
 * ufunc_eval_chebys_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebys_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebys_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[0] = <char>NPY_LONG
 * ufunc_eval_chebys_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":5034
 * ufunc_eval_chebys_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebys_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebys_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":5035
 * ufunc_eval_chebys_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebys_types[0] = <char>NPY_LONG
 * ufunc_eval_chebys_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5036
 * ufunc_eval_chebys_types[0] = <char>NPY_LONG
 * ufunc_eval_chebys_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5037
 * ufunc_eval_chebys_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5038
 * ufunc_eval_chebys_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5039
 * ufunc_eval_chebys_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5040
 * ufunc_eval_chebys_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebys_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5041
 * ufunc_eval_chebys_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebys_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5042
 * ufunc_eval_chebys_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebys_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebys_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5043
 * ufunc_eval_chebys_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebys_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebys_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5044
 * ufunc_eval_chebys_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebys_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5045
 * ufunc_eval_chebys_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[13] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5046
 * ufunc_eval_chebys_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebys_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5047
 * ufunc_eval_chebys_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[13] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebys_ptr[2*0] = <void*>_func_eval_chebys_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[13]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5048
 * ufunc_eval_chebys_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebys_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebys_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_ptr[2*0] = <void*>_func_eval_chebys_l
 * ufunc_eval_chebys_ptr[2*0+1] = <void*>(<char*>"eval_chebys")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5049
 * ufunc_eval_chebys_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebys_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebys_ptr[2*0] = <void*>_func_eval_chebys_l             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_ptr[2*0+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*1] = <void*>_func_eval_chebys[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys_l);

  /* "scipy/special/_ufuncs.pyx":5050
 * ufunc_eval_chebys_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebys_ptr[2*0] = <void*>_func_eval_chebys_l
 * ufunc_eval_chebys_ptr[2*0+1] = <void*>(<char*>"eval_chebys")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_ptr[2*1] = <void*>_func_eval_chebys[double]
 * ufunc_eval_chebys_ptr[2*1+1] = <void*>(<char*>"eval_chebys")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[1]) = ((void *)((char *)((char *)"eval_chebys")));

  /* "scipy/special/_ufuncs.pyx":5051
 * ufunc_eval_chebys_ptr[2*0] = <void*>_func_eval_chebys_l
 * ufunc_eval_chebys_ptr[2*0+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*1] = <void*>_func_eval_chebys[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_ptr[2*1+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*2] = <void*>_func_eval_chebys[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys);

  /* "scipy/special/_ufuncs.pyx":5052
 * ufunc_eval_chebys_ptr[2*0+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*1] = <void*>_func_eval_chebys[double]
 * ufunc_eval_chebys_ptr[2*1+1] = <void*>(<char*>"eval_chebys")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_ptr[2*2] = <void*>_func_eval_chebys[double_complex]
 * ufunc_eval_chebys_ptr[2*2+1] = <void*>(<char*>"eval_chebys")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[3]) = ((void *)((char *)((char *)"eval_chebys")));

  /* "scipy/special/_ufuncs.pyx":5053
 * ufunc_eval_chebys_ptr[2*1] = <void*>_func_eval_chebys[double]
 * ufunc_eval_chebys_ptr[2*1+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*2] = <void*>_func_eval_chebys[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_ptr[2*2+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*3] = <void*>_func_eval_chebys[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys);

  /* "scipy/special/_ufuncs.pyx":5054
 * ufunc_eval_chebys_ptr[2*1+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*2] = <void*>_func_eval_chebys[double_complex]
 * ufunc_eval_chebys_ptr[2*2+1] = <void*>(<char*>"eval_chebys")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_ptr[2*3] = <void*>_func_eval_chebys[double]
 * ufunc_eval_chebys_ptr[2*3+1] = <void*>(<char*>"eval_chebys")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[5]) = ((void *)((char *)((char *)"eval_chebys")));

  /* "scipy/special/_ufuncs.pyx":5055
 * ufunc_eval_chebys_ptr[2*2] = <void*>_func_eval_chebys[double_complex]
 * ufunc_eval_chebys_ptr[2*2+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*3] = <void*>_func_eval_chebys[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_ptr[2*3+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*4] = <void*>_func_eval_chebys[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys);

  /* "scipy/special/_ufuncs.pyx":5056
 * ufunc_eval_chebys_ptr[2*2+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*3] = <void*>_func_eval_chebys[double]
 * ufunc_eval_chebys_ptr[2*3+1] = <void*>(<char*>"eval_chebys")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_ptr[2*4] = <void*>_func_eval_chebys[double_complex]
 * ufunc_eval_chebys_ptr[2*4+1] = <void*>(<char*>"eval_chebys")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[7]) = ((void *)((char *)((char *)"eval_chebys")));

  /* "scipy/special/_ufuncs.pyx":5057
 * ufunc_eval_chebys_ptr[2*3] = <void*>_func_eval_chebys[double]
 * ufunc_eval_chebys_ptr[2*3+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*4] = <void*>_func_eval_chebys[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_ptr[2*4+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_data[0] = &ufunc_eval_chebys_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebys);

  /* "scipy/special/_ufuncs.pyx":5058
 * ufunc_eval_chebys_ptr[2*3+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_ptr[2*4] = <void*>_func_eval_chebys[double_complex]
 * ufunc_eval_chebys_ptr[2*4+1] = <void*>(<char*>"eval_chebys")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_data[0] = &ufunc_eval_chebys_ptr[2*0]
 * ufunc_eval_chebys_data[1] = &ufunc_eval_chebys_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[9]) = ((void *)((char *)((char *)"eval_chebys")));

  /* "scipy/special/_ufuncs.pyx":5059
 * ufunc_eval_chebys_ptr[2*4] = <void*>_func_eval_chebys[double_complex]
 * ufunc_eval_chebys_ptr[2*4+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_data[0] = &ufunc_eval_chebys_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_data[1] = &ufunc_eval_chebys_ptr[2*1]
 * ufunc_eval_chebys_data[2] = &ufunc_eval_chebys_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5060
 * ufunc_eval_chebys_ptr[2*4+1] = <void*>(<char*>"eval_chebys")
 * ufunc_eval_chebys_data[0] = &ufunc_eval_chebys_ptr[2*0]
 * ufunc_eval_chebys_data[1] = &ufunc_eval_chebys_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_data[2] = &ufunc_eval_chebys_ptr[2*2]
 * ufunc_eval_chebys_data[3] = &ufunc_eval_chebys_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5061
 * ufunc_eval_chebys_data[0] = &ufunc_eval_chebys_ptr[2*0]
 * ufunc_eval_chebys_data[1] = &ufunc_eval_chebys_ptr[2*1]
 * ufunc_eval_chebys_data[2] = &ufunc_eval_chebys_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_data[3] = &ufunc_eval_chebys_ptr[2*3]
 * ufunc_eval_chebys_data[4] = &ufunc_eval_chebys_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":5062
 * ufunc_eval_chebys_data[1] = &ufunc_eval_chebys_ptr[2*1]
 * ufunc_eval_chebys_data[2] = &ufunc_eval_chebys_ptr[2*2]
 * ufunc_eval_chebys_data[3] = &ufunc_eval_chebys_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebys_data[4] = &ufunc_eval_chebys_ptr[2*4]
 * eval_chebys = np.PyUFunc_FromFuncAndData(ufunc_eval_chebys_loops, ufunc_eval_chebys_data, ufunc_eval_chebys_types, 5, 2, 1, 0, "eval_chebys", ufunc_eval_chebys_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":5063
 * ufunc_eval_chebys_data[2] = &ufunc_eval_chebys_ptr[2*2]
 * ufunc_eval_chebys_data[3] = &ufunc_eval_chebys_ptr[2*3]
 * ufunc_eval_chebys_data[4] = &ufunc_eval_chebys_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_chebys = np.PyUFunc_FromFuncAndData(ufunc_eval_chebys_loops, ufunc_eval_chebys_data, ufunc_eval_chebys_types, 5, 2, 1, 0, "eval_chebys", ufunc_eval_chebys_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":5064
 * ufunc_eval_chebys_data[3] = &ufunc_eval_chebys_ptr[2*3]
 * ufunc_eval_chebys_data[4] = &ufunc_eval_chebys_ptr[2*4]
 * eval_chebys = np.PyUFunc_FromFuncAndData(ufunc_eval_chebys_loops, ufunc_eval_chebys_data, ufunc_eval_chebys_types, 5, 2, 1, 0, "eval_chebys", ufunc_eval_chebys_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_chebyt_loops[5]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_types, 5, 2, 1, 0, ((char *)"eval_chebys"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebys_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5064, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_chebys, __pyx_t_2) < 0) __PYX_ERR(1, 5064, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":5070
 * cdef void *ufunc_eval_chebyt_data[5]
 * cdef char ufunc_eval_chebyt_types[15]
 * cdef char *ufunc_eval_chebyt_doc = (             # <<<<<<<<<<<<<<
 *     "eval_chebyt(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_doc = ((char *)"eval_chebyt(n, x, out=None)\n\nEvaluate Chebyshev polynomial of the first kind at a point.\n\nThe Chebyshev polynomials of the first kind can be defined via the\nGauss hypergeometric function :math:`{}_2F_1` as\n\n.. math::\n\n    T_n(x) = {}_2F_1(n, -n; 1/2; (1 - x)/2).\n\nWhen :math:`n` is an integer the result is a polynomial of degree\n:math:`n`.\n\nParameters\n----------\nn : array_like\n    Degree of the polynomial. If not an integer, the result is\n    determined via the relation to the Gauss hypergeometric\n    function.\nx : array_like\n    Points at which to evaluate the Chebyshev polynomial\n\nReturns\n-------\nT : ndarray\n    Values of the Chebyshev polynomial\n\nSee Also\n--------\nroots_chebyt : roots and quadrature weights of Chebyshev\n               polynomials of the first kind\nchebyu : Chebychev polynomial object\neval_chebyu : evaluate Chebyshev polynomials of the second kind\nhyp2f1 : Gauss hypergeometric function\nnumpy.polynomial.chebyshev.Chebyshev : Chebyshev series\n\nNotes\n-----\nThis routine is numerically stable for `x` in ``[-1, 1]`` at least\nup to order ``10000``.");

  /* "scipy/special/_ufuncs.pyx":5112
 *     "This routine is numerically stable for `x` in ``[-1, 1]`` at least\n"
 *     "up to order ``10000``.")
 * ufunc_eval_chebyt_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebyt_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":5113
 *     "up to order ``10000``.")
 * ufunc_eval_chebyt_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_chebyt_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebyt_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":5114
 * ufunc_eval_chebyt_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_chebyt_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebyt_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebyt_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":5115
 * ufunc_eval_chebyt_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebyt_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebyt_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebyt_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":5116
 * ufunc_eval_chebyt_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebyt_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebyt_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[0] = <char>NPY_LONG
 * ufunc_eval_chebyt_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":5117
 * ufunc_eval_chebyt_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebyt_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebyt_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":5118
 * ufunc_eval_chebyt_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebyt_types[0] = <char>NPY_LONG
 * ufunc_eval_chebyt_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5119
 * ufunc_eval_chebyt_types[0] = <char>NPY_LONG
 * ufunc_eval_chebyt_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5120
 * ufunc_eval_chebyt_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5121
 * ufunc_eval_chebyt_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5122
 * ufunc_eval_chebyt_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5123
 * ufunc_eval_chebyt_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebyt_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5124
 * ufunc_eval_chebyt_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebyt_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5125
 * ufunc_eval_chebyt_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebyt_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebyt_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5126
 * ufunc_eval_chebyt_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebyt_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebyt_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5127
 * ufunc_eval_chebyt_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebyt_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5128
 * ufunc_eval_chebyt_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[13] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5129
 * ufunc_eval_chebyt_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyt_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5130
 * ufunc_eval_chebyt_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[13] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyt_ptr[2*0] = <void*>_func_eval_chebyt_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[13]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5131
 * ufunc_eval_chebyt_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebyt_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyt_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_ptr[2*0] = <void*>_func_eval_chebyt_l
 * ufunc_eval_chebyt_ptr[2*0+1] = <void*>(<char*>"eval_chebyt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5132
 * ufunc_eval_chebyt_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyt_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyt_ptr[2*0] = <void*>_func_eval_chebyt_l             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_ptr[2*0+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*1] = <void*>_func_eval_chebyt[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt_l);

  /* "scipy/special/_ufuncs.pyx":5133
 * ufunc_eval_chebyt_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyt_ptr[2*0] = <void*>_func_eval_chebyt_l
 * ufunc_eval_chebyt_ptr[2*0+1] = <void*>(<char*>"eval_chebyt")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_ptr[2*1] = <void*>_func_eval_chebyt[double]
 * ufunc_eval_chebyt_ptr[2*1+1] = <void*>(<char*>"eval_chebyt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[1]) = ((void *)((char *)((char *)"eval_chebyt")));

  /* "scipy/special/_ufuncs.pyx":5134
 * ufunc_eval_chebyt_ptr[2*0] = <void*>_func_eval_chebyt_l
 * ufunc_eval_chebyt_ptr[2*0+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*1] = <void*>_func_eval_chebyt[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_ptr[2*1+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*2] = <void*>_func_eval_chebyt[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt);

  /* "scipy/special/_ufuncs.pyx":5135
 * ufunc_eval_chebyt_ptr[2*0+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*1] = <void*>_func_eval_chebyt[double]
 * ufunc_eval_chebyt_ptr[2*1+1] = <void*>(<char*>"eval_chebyt")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_ptr[2*2] = <void*>_func_eval_chebyt[double_complex]
 * ufunc_eval_chebyt_ptr[2*2+1] = <void*>(<char*>"eval_chebyt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[3]) = ((void *)((char *)((char *)"eval_chebyt")));

  /* "scipy/special/_ufuncs.pyx":5136
 * ufunc_eval_chebyt_ptr[2*1] = <void*>_func_eval_chebyt[double]
 * ufunc_eval_chebyt_ptr[2*1+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*2] = <void*>_func_eval_chebyt[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_ptr[2*2+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*3] = <void*>_func_eval_chebyt[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt);

  /* "scipy/special/_ufuncs.pyx":5137
 * ufunc_eval_chebyt_ptr[2*1+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*2] = <void*>_func_eval_chebyt[double_complex]
 * ufunc_eval_chebyt_ptr[2*2+1] = <void*>(<char*>"eval_chebyt")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_ptr[2*3] = <void*>_func_eval_chebyt[double]
 * ufunc_eval_chebyt_ptr[2*3+1] = <void*>(<char*>"eval_chebyt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[5]) = ((void *)((char *)((char *)"eval_chebyt")));

  /* "scipy/special/_ufuncs.pyx":5138
 * ufunc_eval_chebyt_ptr[2*2] = <void*>_func_eval_chebyt[double_complex]
 * ufunc_eval_chebyt_ptr[2*2+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*3] = <void*>_func_eval_chebyt[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_ptr[2*3+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*4] = <void*>_func_eval_chebyt[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt);

  /* "scipy/special/_ufuncs.pyx":5139
 * ufunc_eval_chebyt_ptr[2*2+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*3] = <void*>_func_eval_chebyt[double]
 * ufunc_eval_chebyt_ptr[2*3+1] = <void*>(<char*>"eval_chebyt")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_ptr[2*4] = <void*>_func_eval_chebyt[double_complex]
 * ufunc_eval_chebyt_ptr[2*4+1] = <void*>(<char*>"eval_chebyt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[7]) = ((void *)((char *)((char *)"eval_chebyt")));

  /* "scipy/special/_ufuncs.pyx":5140
 * ufunc_eval_chebyt_ptr[2*3] = <void*>_func_eval_chebyt[double]
 * ufunc_eval_chebyt_ptr[2*3+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*4] = <void*>_func_eval_chebyt[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_ptr[2*4+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_data[0] = &ufunc_eval_chebyt_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyt);

  /* "scipy/special/_ufuncs.pyx":5141
 * ufunc_eval_chebyt_ptr[2*3+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_ptr[2*4] = <void*>_func_eval_chebyt[double_complex]
 * ufunc_eval_chebyt_ptr[2*4+1] = <void*>(<char*>"eval_chebyt")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_data[0] = &ufunc_eval_chebyt_ptr[2*0]
 * ufunc_eval_chebyt_data[1] = &ufunc_eval_chebyt_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[9]) = ((void *)((char *)((char *)"eval_chebyt")));

  /* "scipy/special/_ufuncs.pyx":5142
 * ufunc_eval_chebyt_ptr[2*4] = <void*>_func_eval_chebyt[double_complex]
 * ufunc_eval_chebyt_ptr[2*4+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_data[0] = &ufunc_eval_chebyt_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_data[1] = &ufunc_eval_chebyt_ptr[2*1]
 * ufunc_eval_chebyt_data[2] = &ufunc_eval_chebyt_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5143
 * ufunc_eval_chebyt_ptr[2*4+1] = <void*>(<char*>"eval_chebyt")
 * ufunc_eval_chebyt_data[0] = &ufunc_eval_chebyt_ptr[2*0]
 * ufunc_eval_chebyt_data[1] = &ufunc_eval_chebyt_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_data[2] = &ufunc_eval_chebyt_ptr[2*2]
 * ufunc_eval_chebyt_data[3] = &ufunc_eval_chebyt_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5144
 * ufunc_eval_chebyt_data[0] = &ufunc_eval_chebyt_ptr[2*0]
 * ufunc_eval_chebyt_data[1] = &ufunc_eval_chebyt_ptr[2*1]
 * ufunc_eval_chebyt_data[2] = &ufunc_eval_chebyt_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_data[3] = &ufunc_eval_chebyt_ptr[2*3]
 * ufunc_eval_chebyt_data[4] = &ufunc_eval_chebyt_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":5145
 * ufunc_eval_chebyt_data[1] = &ufunc_eval_chebyt_ptr[2*1]
 * ufunc_eval_chebyt_data[2] = &ufunc_eval_chebyt_ptr[2*2]
 * ufunc_eval_chebyt_data[3] = &ufunc_eval_chebyt_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyt_data[4] = &ufunc_eval_chebyt_ptr[2*4]
 * eval_chebyt = np.PyUFunc_FromFuncAndData(ufunc_eval_chebyt_loops, ufunc_eval_chebyt_data, ufunc_eval_chebyt_types, 5, 2, 1, 0, "eval_chebyt", ufunc_eval_chebyt_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":5146
 * ufunc_eval_chebyt_data[2] = &ufunc_eval_chebyt_ptr[2*2]
 * ufunc_eval_chebyt_data[3] = &ufunc_eval_chebyt_ptr[2*3]
 * ufunc_eval_chebyt_data[4] = &ufunc_eval_chebyt_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_chebyt = np.PyUFunc_FromFuncAndData(ufunc_eval_chebyt_loops, ufunc_eval_chebyt_data, ufunc_eval_chebyt_types, 5, 2, 1, 0, "eval_chebyt", ufunc_eval_chebyt_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":5147
 * ufunc_eval_chebyt_data[3] = &ufunc_eval_chebyt_ptr[2*3]
 * ufunc_eval_chebyt_data[4] = &ufunc_eval_chebyt_ptr[2*4]
 * eval_chebyt = np.PyUFunc_FromFuncAndData(ufunc_eval_chebyt_loops, ufunc_eval_chebyt_data, ufunc_eval_chebyt_types, 5, 2, 1, 0, "eval_chebyt", ufunc_eval_chebyt_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_chebyu_loops[5]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_types, 5, 2, 1, 0, ((char *)"eval_chebyt"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyt_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5147, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_chebyt, __pyx_t_2) < 0) __PYX_ERR(1, 5147, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":5153
 * cdef void *ufunc_eval_chebyu_data[5]
 * cdef char ufunc_eval_chebyu_types[15]
 * cdef char *ufunc_eval_chebyu_doc = (             # <<<<<<<<<<<<<<
 *     "eval_chebyu(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_doc = ((char *)"eval_chebyu(n, x, out=None)\n\nEvaluate Chebyshev polynomial of the second kind at a point.\n\nThe Chebyshev polynomials of the second kind can be defined via\nthe Gauss hypergeometric function :math:`{}_2F_1` as\n\n.. math::\n\n    U_n(x) = (n + 1) {}_2F_1(-n, n + 2; 3/2; (1 - x)/2).\n\nWhen :math:`n` is an integer the result is a polynomial of degree\n:math:`n`.\n\nParameters\n----------\nn : array_like\n    Degree of the polynomial. If not an integer, the result is\n    determined via the relation to the Gauss hypergeometric\n    function.\nx : array_like\n    Points at which to evaluate the Chebyshev polynomial\n\nReturns\n-------\nU : ndarray\n    Values of the Chebyshev polynomial\n\nSee Also\n--------\nroots_chebyu : roots and quadrature weights of Chebyshev\n               polynomials of the second kind\nchebyu : Chebyshev polynomial object\neval_chebyt : evaluate Chebyshev polynomials of the first kind\nhyp2f1 : Gauss hypergeometric function");

  /* "scipy/special/_ufuncs.pyx":5189
 *     "eval_chebyt : evaluate Chebyshev polynomials of the first kind\n"
 *     "hyp2f1 : Gauss hypergeometric function")
 * ufunc_eval_chebyu_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebyu_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":5190
 *     "hyp2f1 : Gauss hypergeometric function")
 * ufunc_eval_chebyu_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_chebyu_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebyu_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":5191
 * ufunc_eval_chebyu_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_chebyu_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebyu_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebyu_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":5192
 * ufunc_eval_chebyu_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_chebyu_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebyu_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebyu_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":5193
 * ufunc_eval_chebyu_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_chebyu_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebyu_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[0] = <char>NPY_LONG
 * ufunc_eval_chebyu_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":5194
 * ufunc_eval_chebyu_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_chebyu_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebyu_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":5195
 * ufunc_eval_chebyu_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_chebyu_types[0] = <char>NPY_LONG
 * ufunc_eval_chebyu_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5196
 * ufunc_eval_chebyu_types[0] = <char>NPY_LONG
 * ufunc_eval_chebyu_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5197
 * ufunc_eval_chebyu_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5198
 * ufunc_eval_chebyu_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5199
 * ufunc_eval_chebyu_types[3] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5200
 * ufunc_eval_chebyu_types[4] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebyu_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5201
 * ufunc_eval_chebyu_types[5] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebyu_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5202
 * ufunc_eval_chebyu_types[6] = <char>NPY_FLOAT
 * ufunc_eval_chebyu_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebyu_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5203
 * ufunc_eval_chebyu_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_chebyu_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebyu_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5204
 * ufunc_eval_chebyu_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_chebyu_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5205
 * ufunc_eval_chebyu_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[13] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5206
 * ufunc_eval_chebyu_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyu_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5207
 * ufunc_eval_chebyu_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[13] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyu_ptr[2*0] = <void*>_func_eval_chebyu_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[13]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5208
 * ufunc_eval_chebyu_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_chebyu_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyu_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_ptr[2*0] = <void*>_func_eval_chebyu_l
 * ufunc_eval_chebyu_ptr[2*0+1] = <void*>(<char*>"eval_chebyu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5209
 * ufunc_eval_chebyu_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyu_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyu_ptr[2*0] = <void*>_func_eval_chebyu_l             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_ptr[2*0+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*1] = <void*>_func_eval_chebyu[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu_l);

  /* "scipy/special/_ufuncs.pyx":5210
 * ufunc_eval_chebyu_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_chebyu_ptr[2*0] = <void*>_func_eval_chebyu_l
 * ufunc_eval_chebyu_ptr[2*0+1] = <void*>(<char*>"eval_chebyu")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_ptr[2*1] = <void*>_func_eval_chebyu[double]
 * ufunc_eval_chebyu_ptr[2*1+1] = <void*>(<char*>"eval_chebyu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[1]) = ((void *)((char *)((char *)"eval_chebyu")));

  /* "scipy/special/_ufuncs.pyx":5211
 * ufunc_eval_chebyu_ptr[2*0] = <void*>_func_eval_chebyu_l
 * ufunc_eval_chebyu_ptr[2*0+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*1] = <void*>_func_eval_chebyu[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_ptr[2*1+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*2] = <void*>_func_eval_chebyu[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu);

  /* "scipy/special/_ufuncs.pyx":5212
 * ufunc_eval_chebyu_ptr[2*0+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*1] = <void*>_func_eval_chebyu[double]
 * ufunc_eval_chebyu_ptr[2*1+1] = <void*>(<char*>"eval_chebyu")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_ptr[2*2] = <void*>_func_eval_chebyu[double_complex]
 * ufunc_eval_chebyu_ptr[2*2+1] = <void*>(<char*>"eval_chebyu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[3]) = ((void *)((char *)((char *)"eval_chebyu")));

  /* "scipy/special/_ufuncs.pyx":5213
 * ufunc_eval_chebyu_ptr[2*1] = <void*>_func_eval_chebyu[double]
 * ufunc_eval_chebyu_ptr[2*1+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*2] = <void*>_func_eval_chebyu[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_ptr[2*2+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*3] = <void*>_func_eval_chebyu[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu);

  /* "scipy/special/_ufuncs.pyx":5214
 * ufunc_eval_chebyu_ptr[2*1+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*2] = <void*>_func_eval_chebyu[double_complex]
 * ufunc_eval_chebyu_ptr[2*2+1] = <void*>(<char*>"eval_chebyu")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_ptr[2*3] = <void*>_func_eval_chebyu[double]
 * ufunc_eval_chebyu_ptr[2*3+1] = <void*>(<char*>"eval_chebyu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[5]) = ((void *)((char *)((char *)"eval_chebyu")));

  /* "scipy/special/_ufuncs.pyx":5215
 * ufunc_eval_chebyu_ptr[2*2] = <void*>_func_eval_chebyu[double_complex]
 * ufunc_eval_chebyu_ptr[2*2+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*3] = <void*>_func_eval_chebyu[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_ptr[2*3+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*4] = <void*>_func_eval_chebyu[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu);

  /* "scipy/special/_ufuncs.pyx":5216
 * ufunc_eval_chebyu_ptr[2*2+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*3] = <void*>_func_eval_chebyu[double]
 * ufunc_eval_chebyu_ptr[2*3+1] = <void*>(<char*>"eval_chebyu")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_ptr[2*4] = <void*>_func_eval_chebyu[double_complex]
 * ufunc_eval_chebyu_ptr[2*4+1] = <void*>(<char*>"eval_chebyu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[7]) = ((void *)((char *)((char *)"eval_chebyu")));

  /* "scipy/special/_ufuncs.pyx":5217
 * ufunc_eval_chebyu_ptr[2*3] = <void*>_func_eval_chebyu[double]
 * ufunc_eval_chebyu_ptr[2*3+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*4] = <void*>_func_eval_chebyu[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_ptr[2*4+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_data[0] = &ufunc_eval_chebyu_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_chebyu);

  /* "scipy/special/_ufuncs.pyx":5218
 * ufunc_eval_chebyu_ptr[2*3+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_ptr[2*4] = <void*>_func_eval_chebyu[double_complex]
 * ufunc_eval_chebyu_ptr[2*4+1] = <void*>(<char*>"eval_chebyu")             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_data[0] = &ufunc_eval_chebyu_ptr[2*0]
 * ufunc_eval_chebyu_data[1] = &ufunc_eval_chebyu_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[9]) = ((void *)((char *)((char *)"eval_chebyu")));

  /* "scipy/special/_ufuncs.pyx":5219
 * ufunc_eval_chebyu_ptr[2*4] = <void*>_func_eval_chebyu[double_complex]
 * ufunc_eval_chebyu_ptr[2*4+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_data[0] = &ufunc_eval_chebyu_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_data[1] = &ufunc_eval_chebyu_ptr[2*1]
 * ufunc_eval_chebyu_data[2] = &ufunc_eval_chebyu_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5220
 * ufunc_eval_chebyu_ptr[2*4+1] = <void*>(<char*>"eval_chebyu")
 * ufunc_eval_chebyu_data[0] = &ufunc_eval_chebyu_ptr[2*0]
 * ufunc_eval_chebyu_data[1] = &ufunc_eval_chebyu_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_data[2] = &ufunc_eval_chebyu_ptr[2*2]
 * ufunc_eval_chebyu_data[3] = &ufunc_eval_chebyu_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5221
 * ufunc_eval_chebyu_data[0] = &ufunc_eval_chebyu_ptr[2*0]
 * ufunc_eval_chebyu_data[1] = &ufunc_eval_chebyu_ptr[2*1]
 * ufunc_eval_chebyu_data[2] = &ufunc_eval_chebyu_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_data[3] = &ufunc_eval_chebyu_ptr[2*3]
 * ufunc_eval_chebyu_data[4] = &ufunc_eval_chebyu_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":5222
 * ufunc_eval_chebyu_data[1] = &ufunc_eval_chebyu_ptr[2*1]
 * ufunc_eval_chebyu_data[2] = &ufunc_eval_chebyu_ptr[2*2]
 * ufunc_eval_chebyu_data[3] = &ufunc_eval_chebyu_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_chebyu_data[4] = &ufunc_eval_chebyu_ptr[2*4]
 * eval_chebyu = np.PyUFunc_FromFuncAndData(ufunc_eval_chebyu_loops, ufunc_eval_chebyu_data, ufunc_eval_chebyu_types, 5, 2, 1, 0, "eval_chebyu", ufunc_eval_chebyu_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":5223
 * ufunc_eval_chebyu_data[2] = &ufunc_eval_chebyu_ptr[2*2]
 * ufunc_eval_chebyu_data[3] = &ufunc_eval_chebyu_ptr[2*3]
 * ufunc_eval_chebyu_data[4] = &ufunc_eval_chebyu_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_chebyu = np.PyUFunc_FromFuncAndData(ufunc_eval_chebyu_loops, ufunc_eval_chebyu_data, ufunc_eval_chebyu_types, 5, 2, 1, 0, "eval_chebyu", ufunc_eval_chebyu_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":5224
 * ufunc_eval_chebyu_data[3] = &ufunc_eval_chebyu_ptr[2*3]
 * ufunc_eval_chebyu_data[4] = &ufunc_eval_chebyu_ptr[2*4]
 * eval_chebyu = np.PyUFunc_FromFuncAndData(ufunc_eval_chebyu_loops, ufunc_eval_chebyu_data, ufunc_eval_chebyu_types, 5, 2, 1, 0, "eval_chebyu", ufunc_eval_chebyu_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_gegenbauer_loops[5]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_types, 5, 2, 1, 0, ((char *)"eval_chebyu"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_chebyu_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_chebyu, __pyx_t_2) < 0) __PYX_ERR(1, 5224, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":5230
 * cdef void *ufunc_eval_gegenbauer_data[5]
 * cdef char ufunc_eval_gegenbauer_types[20]
 * cdef char *ufunc_eval_gegenbauer_doc = (             # <<<<<<<<<<<<<<
 *     "eval_gegenbauer(n, alpha, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_doc = ((char *)"eval_gegenbauer(n, alpha, x, out=None)\n\nEvaluate Gegenbauer polynomial at a point.\n\nThe Gegenbauer polynomials can be defined via the Gauss\nhypergeometric function :math:`{}_2F_1` as\n\n.. math::\n\n    C_n^{(\\alpha)} = \\frac{(2\\alpha)_n}{\\Gamma(n + 1)}\n      {}_2F_1(-n, 2\\alpha + n; \\alpha + 1/2; (1 - z)/2).\n\nWhen :math:`n` is an integer the result is a polynomial of degree\n:math:`n`.\n\nParameters\n----------\nn : array_like\n    Degree of the polynomial. If not an integer, the result is\n    determined via the relation to the Gauss hypergeometric\n    function.\nalpha : array_like\n    Parameter\nx : array_like\n    Points at which to evaluate the Gegenbauer polynomial\n\nReturns\n-------\nC : ndarray\n    Values of the Gegenbauer polynomial\n\nSee Also\n--------\nroots_gegenbauer : roots and quadrature weights of Gegenbauer\n                   polynomials\ngegenbauer : Gegenbauer polynomial object\nhyp2f1 : Gauss hypergeometric function");

  /* "scipy/special/_ufuncs.pyx":5268
 *     "gegenbauer : Gegenbauer polynomial object\n"
 *     "hyp2f1 : Gauss hypergeometric function")
 * ufunc_eval_gegenbauer_loops[0] = <np.PyUFuncGenericFunction>loop_d_ldd__As_ldd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_eval_gegenbauer_loops[2] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ldd__As_ldd_d);

  /* "scipy/special/_ufuncs.pyx":5269
 *     "hyp2f1 : Gauss hypergeometric function")
 * ufunc_eval_gegenbauer_loops[0] = <np.PyUFuncGenericFunction>loop_d_ldd__As_ldd_d
 * ufunc_eval_gegenbauer_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_loops[2] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 * ufunc_eval_gegenbauer_loops[3] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":5270
 * ufunc_eval_gegenbauer_loops[0] = <np.PyUFuncGenericFunction>loop_d_ldd__As_ldd_d
 * ufunc_eval_gegenbauer_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_eval_gegenbauer_loops[2] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_loops[3] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_eval_gegenbauer_loops[4] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_ddD__As_ffF_F);

  /* "scipy/special/_ufuncs.pyx":5271
 * ufunc_eval_gegenbauer_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_eval_gegenbauer_loops[2] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 * ufunc_eval_gegenbauer_loops[3] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_loops[4] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 * ufunc_eval_gegenbauer_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":5272
 * ufunc_eval_gegenbauer_loops[2] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 * ufunc_eval_gegenbauer_loops[3] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_eval_gegenbauer_loops[4] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[0] = <char>NPY_LONG
 * ufunc_eval_gegenbauer_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_ddD__As_ddD_D);

  /* "scipy/special/_ufuncs.pyx":5273
 * ufunc_eval_gegenbauer_loops[3] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_eval_gegenbauer_loops[4] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 * ufunc_eval_gegenbauer_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":5274
 * ufunc_eval_gegenbauer_loops[4] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 * ufunc_eval_gegenbauer_types[0] = <char>NPY_LONG
 * ufunc_eval_gegenbauer_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5275
 * ufunc_eval_gegenbauer_types[0] = <char>NPY_LONG
 * ufunc_eval_gegenbauer_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5276
 * ufunc_eval_gegenbauer_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[4] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5277
 * ufunc_eval_gegenbauer_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[5] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5278
 * ufunc_eval_gegenbauer_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[4] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[6] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5279
 * ufunc_eval_gegenbauer_types[4] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[5] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[7] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[8] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5280
 * ufunc_eval_gegenbauer_types[5] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[6] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[8] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[9] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5281
 * ufunc_eval_gegenbauer_types[6] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[7] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[8] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[9] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[10] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[8]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5282
 * ufunc_eval_gegenbauer_types[7] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[8] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[9] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[10] = <char>NPY_CFLOAT
 * ufunc_eval_gegenbauer_types[11] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[9]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5283
 * ufunc_eval_gegenbauer_types[8] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[9] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[10] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[11] = <char>NPY_CFLOAT
 * ufunc_eval_gegenbauer_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[10]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5284
 * ufunc_eval_gegenbauer_types[9] = <char>NPY_FLOAT
 * ufunc_eval_gegenbauer_types[10] = <char>NPY_CFLOAT
 * ufunc_eval_gegenbauer_types[11] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[11]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5285
 * ufunc_eval_gegenbauer_types[10] = <char>NPY_CFLOAT
 * ufunc_eval_gegenbauer_types[11] = <char>NPY_CFLOAT
 * ufunc_eval_gegenbauer_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[13] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[14] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5286
 * ufunc_eval_gegenbauer_types[11] = <char>NPY_CFLOAT
 * ufunc_eval_gegenbauer_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[14] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[15] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5287
 * ufunc_eval_gegenbauer_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[13] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[14] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[16] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[14]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5288
 * ufunc_eval_gegenbauer_types[13] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[14] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[15] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[17] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[15]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5289
 * ufunc_eval_gegenbauer_types[14] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[16] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[18] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[16]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5290
 * ufunc_eval_gegenbauer_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[17] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[18] = <char>NPY_CDOUBLE
 * ufunc_eval_gegenbauer_types[19] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[17]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5291
 * ufunc_eval_gegenbauer_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[18] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_types[19] = <char>NPY_CDOUBLE
 * ufunc_eval_gegenbauer_ptr[2*0] = <void*>_func_eval_gegenbauer_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[18]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5292
 * ufunc_eval_gegenbauer_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_gegenbauer_types[18] = <char>NPY_CDOUBLE
 * ufunc_eval_gegenbauer_types[19] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_ptr[2*0] = <void*>_func_eval_gegenbauer_l
 * ufunc_eval_gegenbauer_ptr[2*0+1] = <void*>(<char*>"eval_gegenbauer")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types[19]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5293
 * ufunc_eval_gegenbauer_types[18] = <char>NPY_CDOUBLE
 * ufunc_eval_gegenbauer_types[19] = <char>NPY_CDOUBLE
 * ufunc_eval_gegenbauer_ptr[2*0] = <void*>_func_eval_gegenbauer_l             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_ptr[2*0+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*1] = <void*>_func_eval_gegenbauer[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer_l);

  /* "scipy/special/_ufuncs.pyx":5294
 * ufunc_eval_gegenbauer_types[19] = <char>NPY_CDOUBLE
 * ufunc_eval_gegenbauer_ptr[2*0] = <void*>_func_eval_gegenbauer_l
 * ufunc_eval_gegenbauer_ptr[2*0+1] = <void*>(<char*>"eval_gegenbauer")             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_ptr[2*1] = <void*>_func_eval_gegenbauer[double]
 * ufunc_eval_gegenbauer_ptr[2*1+1] = <void*>(<char*>"eval_gegenbauer")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[1]) = ((void *)((char *)((char *)"eval_gegenbauer")));

  /* "scipy/special/_ufuncs.pyx":5295
 * ufunc_eval_gegenbauer_ptr[2*0] = <void*>_func_eval_gegenbauer_l
 * ufunc_eval_gegenbauer_ptr[2*0+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*1] = <void*>_func_eval_gegenbauer[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_ptr[2*1+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*2] = <void*>_func_eval_gegenbauer[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer);

  /* "scipy/special/_ufuncs.pyx":5296
 * ufunc_eval_gegenbauer_ptr[2*0+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*1] = <void*>_func_eval_gegenbauer[double]
 * ufunc_eval_gegenbauer_ptr[2*1+1] = <void*>(<char*>"eval_gegenbauer")             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_ptr[2*2] = <void*>_func_eval_gegenbauer[double_complex]
 * ufunc_eval_gegenbauer_ptr[2*2+1] = <void*>(<char*>"eval_gegenbauer")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[3]) = ((void *)((char *)((char *)"eval_gegenbauer")));

  /* "scipy/special/_ufuncs.pyx":5297
 * ufunc_eval_gegenbauer_ptr[2*1] = <void*>_func_eval_gegenbauer[double]
 * ufunc_eval_gegenbauer_ptr[2*1+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*2] = <void*>_func_eval_gegenbauer[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_ptr[2*2+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*3] = <void*>_func_eval_gegenbauer[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer);

  /* "scipy/special/_ufuncs.pyx":5298
 * ufunc_eval_gegenbauer_ptr[2*1+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*2] = <void*>_func_eval_gegenbauer[double_complex]
 * ufunc_eval_gegenbauer_ptr[2*2+1] = <void*>(<char*>"eval_gegenbauer")             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_ptr[2*3] = <void*>_func_eval_gegenbauer[double]
 * ufunc_eval_gegenbauer_ptr[2*3+1] = <void*>(<char*>"eval_gegenbauer")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[5]) = ((void *)((char *)((char *)"eval_gegenbauer")));

  /* "scipy/special/_ufuncs.pyx":5299
 * ufunc_eval_gegenbauer_ptr[2*2] = <void*>_func_eval_gegenbauer[double_complex]
 * ufunc_eval_gegenbauer_ptr[2*2+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*3] = <void*>_func_eval_gegenbauer[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_ptr[2*3+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*4] = <void*>_func_eval_gegenbauer[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer);

  /* "scipy/special/_ufuncs.pyx":5300
 * ufunc_eval_gegenbauer_ptr[2*2+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*3] = <void*>_func_eval_gegenbauer[double]
 * ufunc_eval_gegenbauer_ptr[2*3+1] = <void*>(<char*>"eval_gegenbauer")             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_ptr[2*4] = <void*>_func_eval_gegenbauer[double_complex]
 * ufunc_eval_gegenbauer_ptr[2*4+1] = <void*>(<char*>"eval_gegenbauer")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[7]) = ((void *)((char *)((char *)"eval_gegenbauer")));

  /* "scipy/special/_ufuncs.pyx":5301
 * ufunc_eval_gegenbauer_ptr[2*3] = <void*>_func_eval_gegenbauer[double]
 * ufunc_eval_gegenbauer_ptr[2*3+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*4] = <void*>_func_eval_gegenbauer[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_ptr[2*4+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_data[0] = &ufunc_eval_gegenbauer_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_gegenbauer);

  /* "scipy/special/_ufuncs.pyx":5302
 * ufunc_eval_gegenbauer_ptr[2*3+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_ptr[2*4] = <void*>_func_eval_gegenbauer[double_complex]
 * ufunc_eval_gegenbauer_ptr[2*4+1] = <void*>(<char*>"eval_gegenbauer")             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_data[0] = &ufunc_eval_gegenbauer_ptr[2*0]
 * ufunc_eval_gegenbauer_data[1] = &ufunc_eval_gegenbauer_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[9]) = ((void *)((char *)((char *)"eval_gegenbauer")));

  /* "scipy/special/_ufuncs.pyx":5303
 * ufunc_eval_gegenbauer_ptr[2*4] = <void*>_func_eval_gegenbauer[double_complex]
 * ufunc_eval_gegenbauer_ptr[2*4+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_data[0] = &ufunc_eval_gegenbauer_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_data[1] = &ufunc_eval_gegenbauer_ptr[2*1]
 * ufunc_eval_gegenbauer_data[2] = &ufunc_eval_gegenbauer_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5304
 * ufunc_eval_gegenbauer_ptr[2*4+1] = <void*>(<char*>"eval_gegenbauer")
 * ufunc_eval_gegenbauer_data[0] = &ufunc_eval_gegenbauer_ptr[2*0]
 * ufunc_eval_gegenbauer_data[1] = &ufunc_eval_gegenbauer_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_data[2] = &ufunc_eval_gegenbauer_ptr[2*2]
 * ufunc_eval_gegenbauer_data[3] = &ufunc_eval_gegenbauer_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5305
 * ufunc_eval_gegenbauer_data[0] = &ufunc_eval_gegenbauer_ptr[2*0]
 * ufunc_eval_gegenbauer_data[1] = &ufunc_eval_gegenbauer_ptr[2*1]
 * ufunc_eval_gegenbauer_data[2] = &ufunc_eval_gegenbauer_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_data[3] = &ufunc_eval_gegenbauer_ptr[2*3]
 * ufunc_eval_gegenbauer_data[4] = &ufunc_eval_gegenbauer_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":5306
 * ufunc_eval_gegenbauer_data[1] = &ufunc_eval_gegenbauer_ptr[2*1]
 * ufunc_eval_gegenbauer_data[2] = &ufunc_eval_gegenbauer_ptr[2*2]
 * ufunc_eval_gegenbauer_data[3] = &ufunc_eval_gegenbauer_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_gegenbauer_data[4] = &ufunc_eval_gegenbauer_ptr[2*4]
 * eval_gegenbauer = np.PyUFunc_FromFuncAndData(ufunc_eval_gegenbauer_loops, ufunc_eval_gegenbauer_data, ufunc_eval_gegenbauer_types, 5, 3, 1, 0, "eval_gegenbauer", ufunc_eval_gegenbauer_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":5307
 * ufunc_eval_gegenbauer_data[2] = &ufunc_eval_gegenbauer_ptr[2*2]
 * ufunc_eval_gegenbauer_data[3] = &ufunc_eval_gegenbauer_ptr[2*3]
 * ufunc_eval_gegenbauer_data[4] = &ufunc_eval_gegenbauer_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_gegenbauer = np.PyUFunc_FromFuncAndData(ufunc_eval_gegenbauer_loops, ufunc_eval_gegenbauer_data, ufunc_eval_gegenbauer_types, 5, 3, 1, 0, "eval_gegenbauer", ufunc_eval_gegenbauer_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":5308
 * ufunc_eval_gegenbauer_data[3] = &ufunc_eval_gegenbauer_ptr[2*3]
 * ufunc_eval_gegenbauer_data[4] = &ufunc_eval_gegenbauer_ptr[2*4]
 * eval_gegenbauer = np.PyUFunc_FromFuncAndData(ufunc_eval_gegenbauer_loops, ufunc_eval_gegenbauer_data, ufunc_eval_gegenbauer_types, 5, 3, 1, 0, "eval_gegenbauer", ufunc_eval_gegenbauer_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_genlaguerre_loops[5]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_types, 5, 3, 1, 0, ((char *)"eval_gegenbauer"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_gegenbauer_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5308, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_gegenbauer, __pyx_t_2) < 0) __PYX_ERR(1, 5308, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":5314
 * cdef void *ufunc_eval_genlaguerre_data[5]
 * cdef char ufunc_eval_genlaguerre_types[20]
 * cdef char *ufunc_eval_genlaguerre_doc = (             # <<<<<<<<<<<<<<
 *     "eval_genlaguerre(n, alpha, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_doc = ((char *)"eval_genlaguerre(n, alpha, x, out=None)\n\nEvaluate generalized Laguerre polynomial at a point.\n\nThe generalized Laguerre polynomials can be defined via the\nconfluent hypergeometric function :math:`{}_1F_1` as\n\n.. math::\n\n    L_n^{(\\alpha)}(x) = \\binom{n + \\alpha}{n}\n      {}_1F_1(-n, \\alpha + 1, x).\n\nWhen :math:`n` is an integer the result is a polynomial of degree\n:math:`n`. The Laguerre polynomials are the special case where\n:math:`\\alpha = 0`.\n\nParameters\n----------\nn : array_like\n    Degree of the polynomial. If not an integer the result is\n    determined via the relation to the confluent hypergeometric\n    function.\nalpha : array_like\n    Parameter; must have ``alpha > -1``\nx : array_like\n    Points at which to evaluate the generalized Laguerre\n    polynomial\n\nReturns\n-------\nL : ndarray\n    Values of the generalized Laguerre polynomial\n\nSee Also\n--------\nroots_genlaguerre : roots and quadrature weights of generalized\n                    Laguerre polynomials\ngenlaguerre : generalized Laguerre polynomial object\nhyp1f1 : confluent hypergeometric function\neval_laguerre : evaluate Laguerre polynomials");

  /* "scipy/special/_ufuncs.pyx":5355
 *     "hyp1f1 : confluent hypergeometric function\n"
 *     "eval_laguerre : evaluate Laguerre polynomials")
 * ufunc_eval_genlaguerre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ldd__As_ldd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_eval_genlaguerre_loops[2] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ldd__As_ldd_d);

  /* "scipy/special/_ufuncs.pyx":5356
 *     "eval_laguerre : evaluate Laguerre polynomials")
 * ufunc_eval_genlaguerre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ldd__As_ldd_d
 * ufunc_eval_genlaguerre_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_loops[2] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 * ufunc_eval_genlaguerre_loops[3] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":5357
 * ufunc_eval_genlaguerre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ldd__As_ldd_d
 * ufunc_eval_genlaguerre_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_eval_genlaguerre_loops[2] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_loops[3] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_eval_genlaguerre_loops[4] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_ddD__As_ffF_F);

  /* "scipy/special/_ufuncs.pyx":5358
 * ufunc_eval_genlaguerre_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_eval_genlaguerre_loops[2] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 * ufunc_eval_genlaguerre_loops[3] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_loops[4] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 * ufunc_eval_genlaguerre_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":5359
 * ufunc_eval_genlaguerre_loops[2] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 * ufunc_eval_genlaguerre_loops[3] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_eval_genlaguerre_loops[4] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[0] = <char>NPY_LONG
 * ufunc_eval_genlaguerre_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_ddD__As_ddD_D);

  /* "scipy/special/_ufuncs.pyx":5360
 * ufunc_eval_genlaguerre_loops[3] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_eval_genlaguerre_loops[4] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 * ufunc_eval_genlaguerre_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":5361
 * ufunc_eval_genlaguerre_loops[4] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 * ufunc_eval_genlaguerre_types[0] = <char>NPY_LONG
 * ufunc_eval_genlaguerre_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5362
 * ufunc_eval_genlaguerre_types[0] = <char>NPY_LONG
 * ufunc_eval_genlaguerre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5363
 * ufunc_eval_genlaguerre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5364
 * ufunc_eval_genlaguerre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5365
 * ufunc_eval_genlaguerre_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5366
 * ufunc_eval_genlaguerre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[7] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[8] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5367
 * ufunc_eval_genlaguerre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[8] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[9] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5368
 * ufunc_eval_genlaguerre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[7] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[8] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[9] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[10] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[8]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5369
 * ufunc_eval_genlaguerre_types[7] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[8] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[9] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[10] = <char>NPY_CFLOAT
 * ufunc_eval_genlaguerre_types[11] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[9]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5370
 * ufunc_eval_genlaguerre_types[8] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[9] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[10] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[11] = <char>NPY_CFLOAT
 * ufunc_eval_genlaguerre_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[10]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5371
 * ufunc_eval_genlaguerre_types[9] = <char>NPY_FLOAT
 * ufunc_eval_genlaguerre_types[10] = <char>NPY_CFLOAT
 * ufunc_eval_genlaguerre_types[11] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[11]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5372
 * ufunc_eval_genlaguerre_types[10] = <char>NPY_CFLOAT
 * ufunc_eval_genlaguerre_types[11] = <char>NPY_CFLOAT
 * ufunc_eval_genlaguerre_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[13] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[14] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5373
 * ufunc_eval_genlaguerre_types[11] = <char>NPY_CFLOAT
 * ufunc_eval_genlaguerre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[14] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[15] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5374
 * ufunc_eval_genlaguerre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[13] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[14] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[16] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[14]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5375
 * ufunc_eval_genlaguerre_types[13] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[14] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[15] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[17] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[15]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5376
 * ufunc_eval_genlaguerre_types[14] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[16] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[18] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[16]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5377
 * ufunc_eval_genlaguerre_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[17] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[18] = <char>NPY_CDOUBLE
 * ufunc_eval_genlaguerre_types[19] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[17]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5378
 * ufunc_eval_genlaguerre_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[18] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_types[19] = <char>NPY_CDOUBLE
 * ufunc_eval_genlaguerre_ptr[2*0] = <void*>_func_eval_genlaguerre_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[18]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5379
 * ufunc_eval_genlaguerre_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_genlaguerre_types[18] = <char>NPY_CDOUBLE
 * ufunc_eval_genlaguerre_types[19] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_ptr[2*0] = <void*>_func_eval_genlaguerre_l
 * ufunc_eval_genlaguerre_ptr[2*0+1] = <void*>(<char*>"eval_genlaguerre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types[19]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5380
 * ufunc_eval_genlaguerre_types[18] = <char>NPY_CDOUBLE
 * ufunc_eval_genlaguerre_types[19] = <char>NPY_CDOUBLE
 * ufunc_eval_genlaguerre_ptr[2*0] = <void*>_func_eval_genlaguerre_l             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_ptr[2*0+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*1] = <void*>_func_eval_genlaguerre[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre_l);

  /* "scipy/special/_ufuncs.pyx":5381
 * ufunc_eval_genlaguerre_types[19] = <char>NPY_CDOUBLE
 * ufunc_eval_genlaguerre_ptr[2*0] = <void*>_func_eval_genlaguerre_l
 * ufunc_eval_genlaguerre_ptr[2*0+1] = <void*>(<char*>"eval_genlaguerre")             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_ptr[2*1] = <void*>_func_eval_genlaguerre[double]
 * ufunc_eval_genlaguerre_ptr[2*1+1] = <void*>(<char*>"eval_genlaguerre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[1]) = ((void *)((char *)((char *)"eval_genlaguerre")));

  /* "scipy/special/_ufuncs.pyx":5382
 * ufunc_eval_genlaguerre_ptr[2*0] = <void*>_func_eval_genlaguerre_l
 * ufunc_eval_genlaguerre_ptr[2*0+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*1] = <void*>_func_eval_genlaguerre[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_ptr[2*1+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*2] = <void*>_func_eval_genlaguerre[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre);

  /* "scipy/special/_ufuncs.pyx":5383
 * ufunc_eval_genlaguerre_ptr[2*0+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*1] = <void*>_func_eval_genlaguerre[double]
 * ufunc_eval_genlaguerre_ptr[2*1+1] = <void*>(<char*>"eval_genlaguerre")             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_ptr[2*2] = <void*>_func_eval_genlaguerre[double_complex]
 * ufunc_eval_genlaguerre_ptr[2*2+1] = <void*>(<char*>"eval_genlaguerre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[3]) = ((void *)((char *)((char *)"eval_genlaguerre")));

  /* "scipy/special/_ufuncs.pyx":5384
 * ufunc_eval_genlaguerre_ptr[2*1] = <void*>_func_eval_genlaguerre[double]
 * ufunc_eval_genlaguerre_ptr[2*1+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*2] = <void*>_func_eval_genlaguerre[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_ptr[2*2+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*3] = <void*>_func_eval_genlaguerre[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre);

  /* "scipy/special/_ufuncs.pyx":5385
 * ufunc_eval_genlaguerre_ptr[2*1+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*2] = <void*>_func_eval_genlaguerre[double_complex]
 * ufunc_eval_genlaguerre_ptr[2*2+1] = <void*>(<char*>"eval_genlaguerre")             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_ptr[2*3] = <void*>_func_eval_genlaguerre[double]
 * ufunc_eval_genlaguerre_ptr[2*3+1] = <void*>(<char*>"eval_genlaguerre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[5]) = ((void *)((char *)((char *)"eval_genlaguerre")));

  /* "scipy/special/_ufuncs.pyx":5386
 * ufunc_eval_genlaguerre_ptr[2*2] = <void*>_func_eval_genlaguerre[double_complex]
 * ufunc_eval_genlaguerre_ptr[2*2+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*3] = <void*>_func_eval_genlaguerre[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_ptr[2*3+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*4] = <void*>_func_eval_genlaguerre[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre);

  /* "scipy/special/_ufuncs.pyx":5387
 * ufunc_eval_genlaguerre_ptr[2*2+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*3] = <void*>_func_eval_genlaguerre[double]
 * ufunc_eval_genlaguerre_ptr[2*3+1] = <void*>(<char*>"eval_genlaguerre")             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_ptr[2*4] = <void*>_func_eval_genlaguerre[double_complex]
 * ufunc_eval_genlaguerre_ptr[2*4+1] = <void*>(<char*>"eval_genlaguerre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[7]) = ((void *)((char *)((char *)"eval_genlaguerre")));

  /* "scipy/special/_ufuncs.pyx":5388
 * ufunc_eval_genlaguerre_ptr[2*3] = <void*>_func_eval_genlaguerre[double]
 * ufunc_eval_genlaguerre_ptr[2*3+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*4] = <void*>_func_eval_genlaguerre[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_ptr[2*4+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_data[0] = &ufunc_eval_genlaguerre_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_genlaguerre);

  /* "scipy/special/_ufuncs.pyx":5389
 * ufunc_eval_genlaguerre_ptr[2*3+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_ptr[2*4] = <void*>_func_eval_genlaguerre[double_complex]
 * ufunc_eval_genlaguerre_ptr[2*4+1] = <void*>(<char*>"eval_genlaguerre")             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_data[0] = &ufunc_eval_genlaguerre_ptr[2*0]
 * ufunc_eval_genlaguerre_data[1] = &ufunc_eval_genlaguerre_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[9]) = ((void *)((char *)((char *)"eval_genlaguerre")));

  /* "scipy/special/_ufuncs.pyx":5390
 * ufunc_eval_genlaguerre_ptr[2*4] = <void*>_func_eval_genlaguerre[double_complex]
 * ufunc_eval_genlaguerre_ptr[2*4+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_data[0] = &ufunc_eval_genlaguerre_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_data[1] = &ufunc_eval_genlaguerre_ptr[2*1]
 * ufunc_eval_genlaguerre_data[2] = &ufunc_eval_genlaguerre_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5391
 * ufunc_eval_genlaguerre_ptr[2*4+1] = <void*>(<char*>"eval_genlaguerre")
 * ufunc_eval_genlaguerre_data[0] = &ufunc_eval_genlaguerre_ptr[2*0]
 * ufunc_eval_genlaguerre_data[1] = &ufunc_eval_genlaguerre_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_data[2] = &ufunc_eval_genlaguerre_ptr[2*2]
 * ufunc_eval_genlaguerre_data[3] = &ufunc_eval_genlaguerre_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5392
 * ufunc_eval_genlaguerre_data[0] = &ufunc_eval_genlaguerre_ptr[2*0]
 * ufunc_eval_genlaguerre_data[1] = &ufunc_eval_genlaguerre_ptr[2*1]
 * ufunc_eval_genlaguerre_data[2] = &ufunc_eval_genlaguerre_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_data[3] = &ufunc_eval_genlaguerre_ptr[2*3]
 * ufunc_eval_genlaguerre_data[4] = &ufunc_eval_genlaguerre_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":5393
 * ufunc_eval_genlaguerre_data[1] = &ufunc_eval_genlaguerre_ptr[2*1]
 * ufunc_eval_genlaguerre_data[2] = &ufunc_eval_genlaguerre_ptr[2*2]
 * ufunc_eval_genlaguerre_data[3] = &ufunc_eval_genlaguerre_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_genlaguerre_data[4] = &ufunc_eval_genlaguerre_ptr[2*4]
 * eval_genlaguerre = np.PyUFunc_FromFuncAndData(ufunc_eval_genlaguerre_loops, ufunc_eval_genlaguerre_data, ufunc_eval_genlaguerre_types, 5, 3, 1, 0, "eval_genlaguerre", ufunc_eval_genlaguerre_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":5394
 * ufunc_eval_genlaguerre_data[2] = &ufunc_eval_genlaguerre_ptr[2*2]
 * ufunc_eval_genlaguerre_data[3] = &ufunc_eval_genlaguerre_ptr[2*3]
 * ufunc_eval_genlaguerre_data[4] = &ufunc_eval_genlaguerre_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_genlaguerre = np.PyUFunc_FromFuncAndData(ufunc_eval_genlaguerre_loops, ufunc_eval_genlaguerre_data, ufunc_eval_genlaguerre_types, 5, 3, 1, 0, "eval_genlaguerre", ufunc_eval_genlaguerre_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":5395
 * ufunc_eval_genlaguerre_data[3] = &ufunc_eval_genlaguerre_ptr[2*3]
 * ufunc_eval_genlaguerre_data[4] = &ufunc_eval_genlaguerre_ptr[2*4]
 * eval_genlaguerre = np.PyUFunc_FromFuncAndData(ufunc_eval_genlaguerre_loops, ufunc_eval_genlaguerre_data, ufunc_eval_genlaguerre_types, 5, 3, 1, 0, "eval_genlaguerre", ufunc_eval_genlaguerre_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_hermite_loops[1]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_types, 5, 3, 1, 0, ((char *)"eval_genlaguerre"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_genlaguerre_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5395, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_genlaguerre, __pyx_t_2) < 0) __PYX_ERR(1, 5395, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":5401
 * cdef void *ufunc_eval_hermite_data[1]
 * cdef char ufunc_eval_hermite_types[3]
 * cdef char *ufunc_eval_hermite_doc = (             # <<<<<<<<<<<<<<
 *     "eval_hermite(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_doc = ((char *)"eval_hermite(n, x, out=None)\n\nEvaluate physicist's Hermite polynomial at a point.\n\nDefined by\n\n.. math::\n\n    H_n(x) = (-1)^n e^{x^2} \\frac{d^n}{dx^n} e^{-x^2};\n\n:math:`H_n` is a polynomial of degree :math:`n`.\n\nParameters\n----------\nn : array_like\n    Degree of the polynomial\nx : array_like\n    Points at which to evaluate the Hermite polynomial\n\nReturns\n-------\nH : ndarray\n    Values of the Hermite polynomial\n\nSee Also\n--------\nroots_hermite : roots and quadrature weights of physicist's\n                Hermite polynomials\nhermite : physicist's Hermite polynomial object\nnumpy.polynomial.hermite.Hermite : Physicist's Hermite series\neval_hermitenorm : evaluate Probabilist's Hermite polynomials");

  /* "scipy/special/_ufuncs.pyx":5433
 *     "numpy.polynomial.hermite.Hermite : Physicist's Hermite series\n"
 *     "eval_hermitenorm : evaluate Probabilist's Hermite polynomials")
 * ufunc_eval_hermite_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_hermite_types[0] = <char>NPY_LONG
 * ufunc_eval_hermite_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":5434
 *     "eval_hermitenorm : evaluate Probabilist's Hermite polynomials")
 * ufunc_eval_hermite_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_hermite_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_hermite_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_hermite_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":5435
 * ufunc_eval_hermite_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_hermite_types[0] = <char>NPY_LONG
 * ufunc_eval_hermite_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_hermite_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_hermite_ptr[2*0] = <void*>_func_eval_hermite
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5436
 * ufunc_eval_hermite_types[0] = <char>NPY_LONG
 * ufunc_eval_hermite_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_hermite_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_hermite_ptr[2*0] = <void*>_func_eval_hermite
 * ufunc_eval_hermite_ptr[2*0+1] = <void*>(<char*>"eval_hermite")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5437
 * ufunc_eval_hermite_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_hermite_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_hermite_ptr[2*0] = <void*>_func_eval_hermite             # <<<<<<<<<<<<<<
 * ufunc_eval_hermite_ptr[2*0+1] = <void*>(<char*>"eval_hermite")
 * ufunc_eval_hermite_data[0] = &ufunc_eval_hermite_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_hermite);

  /* "scipy/special/_ufuncs.pyx":5438
 * ufunc_eval_hermite_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_hermite_ptr[2*0] = <void*>_func_eval_hermite
 * ufunc_eval_hermite_ptr[2*0+1] = <void*>(<char*>"eval_hermite")             # <<<<<<<<<<<<<<
 * ufunc_eval_hermite_data[0] = &ufunc_eval_hermite_ptr[2*0]
 * eval_hermite = np.PyUFunc_FromFuncAndData(ufunc_eval_hermite_loops, ufunc_eval_hermite_data, ufunc_eval_hermite_types, 1, 2, 1, 0, "eval_hermite", ufunc_eval_hermite_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_ptr[1]) = ((void *)((char *)((char *)"eval_hermite")));

  /* "scipy/special/_ufuncs.pyx":5439
 * ufunc_eval_hermite_ptr[2*0] = <void*>_func_eval_hermite
 * ufunc_eval_hermite_ptr[2*0+1] = <void*>(<char*>"eval_hermite")
 * ufunc_eval_hermite_data[0] = &ufunc_eval_hermite_ptr[2*0]             # <<<<<<<<<<<<<<
 * eval_hermite = np.PyUFunc_FromFuncAndData(ufunc_eval_hermite_loops, ufunc_eval_hermite_data, ufunc_eval_hermite_types, 1, 2, 1, 0, "eval_hermite", ufunc_eval_hermite_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5440
 * ufunc_eval_hermite_ptr[2*0+1] = <void*>(<char*>"eval_hermite")
 * ufunc_eval_hermite_data[0] = &ufunc_eval_hermite_ptr[2*0]
 * eval_hermite = np.PyUFunc_FromFuncAndData(ufunc_eval_hermite_loops, ufunc_eval_hermite_data, ufunc_eval_hermite_types, 1, 2, 1, 0, "eval_hermite", ufunc_eval_hermite_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_hermitenorm_loops[1]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_types, 1, 2, 1, 0, ((char *)"eval_hermite"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermite_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5440, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_hermite, __pyx_t_2) < 0) __PYX_ERR(1, 5440, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":5446
 * cdef void *ufunc_eval_hermitenorm_data[1]
 * cdef char ufunc_eval_hermitenorm_types[3]
 * cdef char *ufunc_eval_hermitenorm_doc = (             # <<<<<<<<<<<<<<
 *     "eval_hermitenorm(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_doc = ((char *)"eval_hermitenorm(n, x, out=None)\n\nEvaluate probabilist's (normalized) Hermite polynomial at a\npoint.\n\nDefined by\n\n.. math::\n\n    He_n(x) = (-1)^n e^{x^2/2} \\frac{d^n}{dx^n} e^{-x^2/2};\n\n:math:`He_n` is a polynomial of degree :math:`n`.\n\nParameters\n----------\nn : array_like\n    Degree of the polynomial\nx : array_like\n    Points at which to evaluate the Hermite polynomial\n\nReturns\n-------\nHe : ndarray\n    Values of the Hermite polynomial\n\nSee Also\n--------\nroots_hermitenorm : roots and quadrature weights of probabilist's\n                    Hermite polynomials\nhermitenorm : probabilist's Hermite polynomial object\nnumpy.polynomial.hermite_e.HermiteE : Probabilist's Hermite series\neval_hermite : evaluate physicist's Hermite polynomials");

  /* "scipy/special/_ufuncs.pyx":5479
 *     "numpy.polynomial.hermite_e.HermiteE : Probabilist's Hermite series\n"
 *     "eval_hermite : evaluate physicist's Hermite polynomials")
 * ufunc_eval_hermitenorm_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_hermitenorm_types[0] = <char>NPY_LONG
 * ufunc_eval_hermitenorm_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":5480
 *     "eval_hermite : evaluate physicist's Hermite polynomials")
 * ufunc_eval_hermitenorm_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_hermitenorm_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_hermitenorm_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_hermitenorm_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":5481
 * ufunc_eval_hermitenorm_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_hermitenorm_types[0] = <char>NPY_LONG
 * ufunc_eval_hermitenorm_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_hermitenorm_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_hermitenorm_ptr[2*0] = <void*>_func_eval_hermitenorm
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5482
 * ufunc_eval_hermitenorm_types[0] = <char>NPY_LONG
 * ufunc_eval_hermitenorm_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_hermitenorm_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_hermitenorm_ptr[2*0] = <void*>_func_eval_hermitenorm
 * ufunc_eval_hermitenorm_ptr[2*0+1] = <void*>(<char*>"eval_hermitenorm")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5483
 * ufunc_eval_hermitenorm_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_hermitenorm_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_hermitenorm_ptr[2*0] = <void*>_func_eval_hermitenorm             # <<<<<<<<<<<<<<
 * ufunc_eval_hermitenorm_ptr[2*0+1] = <void*>(<char*>"eval_hermitenorm")
 * ufunc_eval_hermitenorm_data[0] = &ufunc_eval_hermitenorm_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_hermitenorm);

  /* "scipy/special/_ufuncs.pyx":5484
 * ufunc_eval_hermitenorm_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_hermitenorm_ptr[2*0] = <void*>_func_eval_hermitenorm
 * ufunc_eval_hermitenorm_ptr[2*0+1] = <void*>(<char*>"eval_hermitenorm")             # <<<<<<<<<<<<<<
 * ufunc_eval_hermitenorm_data[0] = &ufunc_eval_hermitenorm_ptr[2*0]
 * eval_hermitenorm = np.PyUFunc_FromFuncAndData(ufunc_eval_hermitenorm_loops, ufunc_eval_hermitenorm_data, ufunc_eval_hermitenorm_types, 1, 2, 1, 0, "eval_hermitenorm", ufunc_eval_hermitenorm_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_ptr[1]) = ((void *)((char *)((char *)"eval_hermitenorm")));

  /* "scipy/special/_ufuncs.pyx":5485
 * ufunc_eval_hermitenorm_ptr[2*0] = <void*>_func_eval_hermitenorm
 * ufunc_eval_hermitenorm_ptr[2*0+1] = <void*>(<char*>"eval_hermitenorm")
 * ufunc_eval_hermitenorm_data[0] = &ufunc_eval_hermitenorm_ptr[2*0]             # <<<<<<<<<<<<<<
 * eval_hermitenorm = np.PyUFunc_FromFuncAndData(ufunc_eval_hermitenorm_loops, ufunc_eval_hermitenorm_data, ufunc_eval_hermitenorm_types, 1, 2, 1, 0, "eval_hermitenorm", ufunc_eval_hermitenorm_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5486
 * ufunc_eval_hermitenorm_ptr[2*0+1] = <void*>(<char*>"eval_hermitenorm")
 * ufunc_eval_hermitenorm_data[0] = &ufunc_eval_hermitenorm_ptr[2*0]
 * eval_hermitenorm = np.PyUFunc_FromFuncAndData(ufunc_eval_hermitenorm_loops, ufunc_eval_hermitenorm_data, ufunc_eval_hermitenorm_types, 1, 2, 1, 0, "eval_hermitenorm", ufunc_eval_hermitenorm_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_jacobi_loops[5]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_types, 1, 2, 1, 0, ((char *)"eval_hermitenorm"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_hermitenorm_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_hermitenorm, __pyx_t_2) < 0) __PYX_ERR(1, 5486, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":5492
 * cdef void *ufunc_eval_jacobi_data[5]
 * cdef char ufunc_eval_jacobi_types[25]
 * cdef char *ufunc_eval_jacobi_doc = (             # <<<<<<<<<<<<<<
 *     "eval_jacobi(n, alpha, beta, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_doc = ((char *)"eval_jacobi(n, alpha, beta, x, out=None)\n\nEvaluate Jacobi polynomial at a point.\n\nThe Jacobi polynomials can be defined via the Gauss hypergeometric\nfunction :math:`{}_2F_1` as\n\n.. math::\n\n    P_n^{(\\alpha, \\beta)}(x) = \\frac{(\\alpha + 1)_n}{\\Gamma(n + 1)}\n      {}_2F_1(-n, 1 + \\alpha + \\beta + n; \\alpha + 1; (1 - z)/2)\n\nwhere :math:`(\\cdot)_n` is the Pochhammer symbol; see `poch`. When\n:math:`n` is an integer the result is a polynomial of degree\n:math:`n`.\n\nParameters\n----------\nn : array_like\n    Degree of the polynomial. If not an integer the result is\n    determined via the relation to the Gauss hypergeometric\n    function.\nalpha : array_like\n    Parameter\nbeta : array_like\n    Parameter\nx : array_like\n    Points at which to evaluate the polynomial\n\nReturns\n-------\nP : ndarray\n    Values of the Jacobi polynomial\n\nSee Also\n--------\nroots_jacobi : roots and quadrature weights of Jacobi polynomials\njacobi : Jacobi polynomial object\nhyp2f1 : Gauss hypergeometric function");

  /* "scipy/special/_ufuncs.pyx":5532
 *     "jacobi : Jacobi polynomial object\n"
 *     "hyp2f1 : Gauss hypergeometric function")
 * ufunc_eval_jacobi_loops[0] = <np.PyUFuncGenericFunction>loop_d_lddd__As_lddd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_eval_jacobi_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_lddd__As_lddd_d);

  /* "scipy/special/_ufuncs.pyx":5533
 *     "hyp2f1 : Gauss hypergeometric function")
 * ufunc_eval_jacobi_loops[0] = <np.PyUFuncGenericFunction>loop_d_lddd__As_lddd_d
 * ufunc_eval_jacobi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 * ufunc_eval_jacobi_loops[3] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_ffff_f);

  /* "scipy/special/_ufuncs.pyx":5534
 * ufunc_eval_jacobi_loops[0] = <np.PyUFuncGenericFunction>loop_d_lddd__As_lddd_d
 * ufunc_eval_jacobi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_eval_jacobi_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_loops[3] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_eval_jacobi_loops[4] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dddD__As_fffF_F);

  /* "scipy/special/_ufuncs.pyx":5535
 * ufunc_eval_jacobi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_eval_jacobi_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 * ufunc_eval_jacobi_loops[3] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_loops[4] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 * ufunc_eval_jacobi_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_dddd_d);

  /* "scipy/special/_ufuncs.pyx":5536
 * ufunc_eval_jacobi_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 * ufunc_eval_jacobi_loops[3] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_eval_jacobi_loops[4] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[0] = <char>NPY_LONG
 * ufunc_eval_jacobi_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dddD__As_dddD_D);

  /* "scipy/special/_ufuncs.pyx":5537
 * ufunc_eval_jacobi_loops[3] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_eval_jacobi_loops[4] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 * ufunc_eval_jacobi_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":5538
 * ufunc_eval_jacobi_loops[4] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 * ufunc_eval_jacobi_types[0] = <char>NPY_LONG
 * ufunc_eval_jacobi_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5539
 * ufunc_eval_jacobi_types[0] = <char>NPY_LONG
 * ufunc_eval_jacobi_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5540
 * ufunc_eval_jacobi_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[4] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5541
 * ufunc_eval_jacobi_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[5] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5542
 * ufunc_eval_jacobi_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[4] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[6] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5543
 * ufunc_eval_jacobi_types[4] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[5] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[7] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[8] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5544
 * ufunc_eval_jacobi_types[5] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[6] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[8] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[9] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5545
 * ufunc_eval_jacobi_types[6] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[7] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[8] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[9] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[10] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[8]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5546
 * ufunc_eval_jacobi_types[7] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[8] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[9] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[10] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[11] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[9]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5547
 * ufunc_eval_jacobi_types[8] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[9] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[10] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[11] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[12] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[10]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5548
 * ufunc_eval_jacobi_types[9] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[10] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[11] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[12] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[13] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[11]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5549
 * ufunc_eval_jacobi_types[10] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[11] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[12] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[13] = <char>NPY_CFLOAT
 * ufunc_eval_jacobi_types[14] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[12]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5550
 * ufunc_eval_jacobi_types[11] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[12] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[13] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[14] = <char>NPY_CFLOAT
 * ufunc_eval_jacobi_types[15] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[13]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5551
 * ufunc_eval_jacobi_types[12] = <char>NPY_FLOAT
 * ufunc_eval_jacobi_types[13] = <char>NPY_CFLOAT
 * ufunc_eval_jacobi_types[14] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[16] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[14]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5552
 * ufunc_eval_jacobi_types[13] = <char>NPY_CFLOAT
 * ufunc_eval_jacobi_types[14] = <char>NPY_CFLOAT
 * ufunc_eval_jacobi_types[15] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[17] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[15]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5553
 * ufunc_eval_jacobi_types[14] = <char>NPY_CFLOAT
 * ufunc_eval_jacobi_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[16] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[18] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[16]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5554
 * ufunc_eval_jacobi_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[17] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[18] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[19] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[17]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5555
 * ufunc_eval_jacobi_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[18] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[19] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[20] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[18]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5556
 * ufunc_eval_jacobi_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[18] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[19] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[20] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[21] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[19]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5557
 * ufunc_eval_jacobi_types[18] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[19] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[20] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[21] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[22] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[20]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5558
 * ufunc_eval_jacobi_types[19] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[20] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[21] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[22] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[23] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[21]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5559
 * ufunc_eval_jacobi_types[20] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[21] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[22] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[23] = <char>NPY_CDOUBLE
 * ufunc_eval_jacobi_types[24] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[22]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5560
 * ufunc_eval_jacobi_types[21] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[22] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[23] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_types[24] = <char>NPY_CDOUBLE
 * ufunc_eval_jacobi_ptr[2*0] = <void*>_func_eval_jacobi_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[23]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5561
 * ufunc_eval_jacobi_types[22] = <char>NPY_DOUBLE
 * ufunc_eval_jacobi_types[23] = <char>NPY_CDOUBLE
 * ufunc_eval_jacobi_types[24] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_ptr[2*0] = <void*>_func_eval_jacobi_l
 * ufunc_eval_jacobi_ptr[2*0+1] = <void*>(<char*>"eval_jacobi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types[24]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5562
 * ufunc_eval_jacobi_types[23] = <char>NPY_CDOUBLE
 * ufunc_eval_jacobi_types[24] = <char>NPY_CDOUBLE
 * ufunc_eval_jacobi_ptr[2*0] = <void*>_func_eval_jacobi_l             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_ptr[2*0+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*1] = <void*>_func_eval_jacobi[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi_l);

  /* "scipy/special/_ufuncs.pyx":5563
 * ufunc_eval_jacobi_types[24] = <char>NPY_CDOUBLE
 * ufunc_eval_jacobi_ptr[2*0] = <void*>_func_eval_jacobi_l
 * ufunc_eval_jacobi_ptr[2*0+1] = <void*>(<char*>"eval_jacobi")             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_ptr[2*1] = <void*>_func_eval_jacobi[double]
 * ufunc_eval_jacobi_ptr[2*1+1] = <void*>(<char*>"eval_jacobi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[1]) = ((void *)((char *)((char *)"eval_jacobi")));

  /* "scipy/special/_ufuncs.pyx":5564
 * ufunc_eval_jacobi_ptr[2*0] = <void*>_func_eval_jacobi_l
 * ufunc_eval_jacobi_ptr[2*0+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*1] = <void*>_func_eval_jacobi[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_ptr[2*1+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*2] = <void*>_func_eval_jacobi[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi);

  /* "scipy/special/_ufuncs.pyx":5565
 * ufunc_eval_jacobi_ptr[2*0+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*1] = <void*>_func_eval_jacobi[double]
 * ufunc_eval_jacobi_ptr[2*1+1] = <void*>(<char*>"eval_jacobi")             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_ptr[2*2] = <void*>_func_eval_jacobi[double_complex]
 * ufunc_eval_jacobi_ptr[2*2+1] = <void*>(<char*>"eval_jacobi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[3]) = ((void *)((char *)((char *)"eval_jacobi")));

  /* "scipy/special/_ufuncs.pyx":5566
 * ufunc_eval_jacobi_ptr[2*1] = <void*>_func_eval_jacobi[double]
 * ufunc_eval_jacobi_ptr[2*1+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*2] = <void*>_func_eval_jacobi[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_ptr[2*2+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*3] = <void*>_func_eval_jacobi[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi);

  /* "scipy/special/_ufuncs.pyx":5567
 * ufunc_eval_jacobi_ptr[2*1+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*2] = <void*>_func_eval_jacobi[double_complex]
 * ufunc_eval_jacobi_ptr[2*2+1] = <void*>(<char*>"eval_jacobi")             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_ptr[2*3] = <void*>_func_eval_jacobi[double]
 * ufunc_eval_jacobi_ptr[2*3+1] = <void*>(<char*>"eval_jacobi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[5]) = ((void *)((char *)((char *)"eval_jacobi")));

  /* "scipy/special/_ufuncs.pyx":5568
 * ufunc_eval_jacobi_ptr[2*2] = <void*>_func_eval_jacobi[double_complex]
 * ufunc_eval_jacobi_ptr[2*2+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*3] = <void*>_func_eval_jacobi[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_ptr[2*3+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*4] = <void*>_func_eval_jacobi[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi);

  /* "scipy/special/_ufuncs.pyx":5569
 * ufunc_eval_jacobi_ptr[2*2+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*3] = <void*>_func_eval_jacobi[double]
 * ufunc_eval_jacobi_ptr[2*3+1] = <void*>(<char*>"eval_jacobi")             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_ptr[2*4] = <void*>_func_eval_jacobi[double_complex]
 * ufunc_eval_jacobi_ptr[2*4+1] = <void*>(<char*>"eval_jacobi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[7]) = ((void *)((char *)((char *)"eval_jacobi")));

  /* "scipy/special/_ufuncs.pyx":5570
 * ufunc_eval_jacobi_ptr[2*3] = <void*>_func_eval_jacobi[double]
 * ufunc_eval_jacobi_ptr[2*3+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*4] = <void*>_func_eval_jacobi[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_ptr[2*4+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_data[0] = &ufunc_eval_jacobi_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_jacobi);

  /* "scipy/special/_ufuncs.pyx":5571
 * ufunc_eval_jacobi_ptr[2*3+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_ptr[2*4] = <void*>_func_eval_jacobi[double_complex]
 * ufunc_eval_jacobi_ptr[2*4+1] = <void*>(<char*>"eval_jacobi")             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_data[0] = &ufunc_eval_jacobi_ptr[2*0]
 * ufunc_eval_jacobi_data[1] = &ufunc_eval_jacobi_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[9]) = ((void *)((char *)((char *)"eval_jacobi")));

  /* "scipy/special/_ufuncs.pyx":5572
 * ufunc_eval_jacobi_ptr[2*4] = <void*>_func_eval_jacobi[double_complex]
 * ufunc_eval_jacobi_ptr[2*4+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_data[0] = &ufunc_eval_jacobi_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_data[1] = &ufunc_eval_jacobi_ptr[2*1]
 * ufunc_eval_jacobi_data[2] = &ufunc_eval_jacobi_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5573
 * ufunc_eval_jacobi_ptr[2*4+1] = <void*>(<char*>"eval_jacobi")
 * ufunc_eval_jacobi_data[0] = &ufunc_eval_jacobi_ptr[2*0]
 * ufunc_eval_jacobi_data[1] = &ufunc_eval_jacobi_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_data[2] = &ufunc_eval_jacobi_ptr[2*2]
 * ufunc_eval_jacobi_data[3] = &ufunc_eval_jacobi_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5574
 * ufunc_eval_jacobi_data[0] = &ufunc_eval_jacobi_ptr[2*0]
 * ufunc_eval_jacobi_data[1] = &ufunc_eval_jacobi_ptr[2*1]
 * ufunc_eval_jacobi_data[2] = &ufunc_eval_jacobi_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_data[3] = &ufunc_eval_jacobi_ptr[2*3]
 * ufunc_eval_jacobi_data[4] = &ufunc_eval_jacobi_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":5575
 * ufunc_eval_jacobi_data[1] = &ufunc_eval_jacobi_ptr[2*1]
 * ufunc_eval_jacobi_data[2] = &ufunc_eval_jacobi_ptr[2*2]
 * ufunc_eval_jacobi_data[3] = &ufunc_eval_jacobi_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_jacobi_data[4] = &ufunc_eval_jacobi_ptr[2*4]
 * eval_jacobi = np.PyUFunc_FromFuncAndData(ufunc_eval_jacobi_loops, ufunc_eval_jacobi_data, ufunc_eval_jacobi_types, 5, 4, 1, 0, "eval_jacobi", ufunc_eval_jacobi_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":5576
 * ufunc_eval_jacobi_data[2] = &ufunc_eval_jacobi_ptr[2*2]
 * ufunc_eval_jacobi_data[3] = &ufunc_eval_jacobi_ptr[2*3]
 * ufunc_eval_jacobi_data[4] = &ufunc_eval_jacobi_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_jacobi = np.PyUFunc_FromFuncAndData(ufunc_eval_jacobi_loops, ufunc_eval_jacobi_data, ufunc_eval_jacobi_types, 5, 4, 1, 0, "eval_jacobi", ufunc_eval_jacobi_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":5577
 * ufunc_eval_jacobi_data[3] = &ufunc_eval_jacobi_ptr[2*3]
 * ufunc_eval_jacobi_data[4] = &ufunc_eval_jacobi_ptr[2*4]
 * eval_jacobi = np.PyUFunc_FromFuncAndData(ufunc_eval_jacobi_loops, ufunc_eval_jacobi_data, ufunc_eval_jacobi_types, 5, 4, 1, 0, "eval_jacobi", ufunc_eval_jacobi_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_laguerre_loops[5]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_types, 5, 4, 1, 0, ((char *)"eval_jacobi"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_jacobi_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5577, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_jacobi, __pyx_t_2) < 0) __PYX_ERR(1, 5577, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":5583
 * cdef void *ufunc_eval_laguerre_data[5]
 * cdef char ufunc_eval_laguerre_types[15]
 * cdef char *ufunc_eval_laguerre_doc = (             # <<<<<<<<<<<<<<
 *     "eval_laguerre(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_doc = ((char *)"eval_laguerre(n, x, out=None)\n\nEvaluate Laguerre polynomial at a point.\n\nThe Laguerre polynomials can be defined via the confluent\nhypergeometric function :math:`{}_1F_1` as\n\n.. math::\n\n    L_n(x) = {}_1F_1(-n, 1, x).\n\nWhen :math:`n` is an integer the result is a polynomial of degree\n:math:`n`.\n\nParameters\n----------\nn : array_like\n    Degree of the polynomial. If not an integer the result is\n    determined via the relation to the confluent hypergeometric\n    function.\nx : array_like\n    Points at which to evaluate the Laguerre polynomial\n\nReturns\n-------\nL : ndarray\n    Values of the Laguerre polynomial\n\nSee Also\n--------\nroots_laguerre : roots and quadrature weights of Laguerre\n                 polynomials\nlaguerre : Laguerre polynomial object\nnumpy.polynomial.laguerre.Laguerre : Laguerre series\neval_genlaguerre : evaluate generalized Laguerre polynomials");

  /* "scipy/special/_ufuncs.pyx":5619
 *     "numpy.polynomial.laguerre.Laguerre : Laguerre series\n"
 *     "eval_genlaguerre : evaluate generalized Laguerre polynomials")
 * ufunc_eval_laguerre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_laguerre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":5620
 *     "eval_genlaguerre : evaluate generalized Laguerre polynomials")
 * ufunc_eval_laguerre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_laguerre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_laguerre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":5621
 * ufunc_eval_laguerre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_laguerre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_laguerre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_laguerre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":5622
 * ufunc_eval_laguerre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_laguerre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_laguerre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_laguerre_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":5623
 * ufunc_eval_laguerre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_laguerre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_laguerre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[0] = <char>NPY_LONG
 * ufunc_eval_laguerre_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":5624
 * ufunc_eval_laguerre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_laguerre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_laguerre_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":5625
 * ufunc_eval_laguerre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_laguerre_types[0] = <char>NPY_LONG
 * ufunc_eval_laguerre_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5626
 * ufunc_eval_laguerre_types[0] = <char>NPY_LONG
 * ufunc_eval_laguerre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[3] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5627
 * ufunc_eval_laguerre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5628
 * ufunc_eval_laguerre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[3] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5629
 * ufunc_eval_laguerre_types[3] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5630
 * ufunc_eval_laguerre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_laguerre_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5631
 * ufunc_eval_laguerre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_laguerre_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5632
 * ufunc_eval_laguerre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_laguerre_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_laguerre_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5633
 * ufunc_eval_laguerre_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_laguerre_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_laguerre_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5634
 * ufunc_eval_laguerre_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_laguerre_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5635
 * ufunc_eval_laguerre_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[13] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5636
 * ufunc_eval_laguerre_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_laguerre_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5637
 * ufunc_eval_laguerre_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[13] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_laguerre_ptr[2*0] = <void*>_func_eval_laguerre_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[13]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5638
 * ufunc_eval_laguerre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_laguerre_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_laguerre_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_ptr[2*0] = <void*>_func_eval_laguerre_l
 * ufunc_eval_laguerre_ptr[2*0+1] = <void*>(<char*>"eval_laguerre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5639
 * ufunc_eval_laguerre_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_laguerre_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_laguerre_ptr[2*0] = <void*>_func_eval_laguerre_l             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_ptr[2*0+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*1] = <void*>_func_eval_laguerre[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre_l);

  /* "scipy/special/_ufuncs.pyx":5640
 * ufunc_eval_laguerre_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_laguerre_ptr[2*0] = <void*>_func_eval_laguerre_l
 * ufunc_eval_laguerre_ptr[2*0+1] = <void*>(<char*>"eval_laguerre")             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_ptr[2*1] = <void*>_func_eval_laguerre[double]
 * ufunc_eval_laguerre_ptr[2*1+1] = <void*>(<char*>"eval_laguerre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[1]) = ((void *)((char *)((char *)"eval_laguerre")));

  /* "scipy/special/_ufuncs.pyx":5641
 * ufunc_eval_laguerre_ptr[2*0] = <void*>_func_eval_laguerre_l
 * ufunc_eval_laguerre_ptr[2*0+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*1] = <void*>_func_eval_laguerre[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_ptr[2*1+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*2] = <void*>_func_eval_laguerre[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre);

  /* "scipy/special/_ufuncs.pyx":5642
 * ufunc_eval_laguerre_ptr[2*0+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*1] = <void*>_func_eval_laguerre[double]
 * ufunc_eval_laguerre_ptr[2*1+1] = <void*>(<char*>"eval_laguerre")             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_ptr[2*2] = <void*>_func_eval_laguerre[double_complex]
 * ufunc_eval_laguerre_ptr[2*2+1] = <void*>(<char*>"eval_laguerre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[3]) = ((void *)((char *)((char *)"eval_laguerre")));

  /* "scipy/special/_ufuncs.pyx":5643
 * ufunc_eval_laguerre_ptr[2*1] = <void*>_func_eval_laguerre[double]
 * ufunc_eval_laguerre_ptr[2*1+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*2] = <void*>_func_eval_laguerre[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_ptr[2*2+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*3] = <void*>_func_eval_laguerre[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre);

  /* "scipy/special/_ufuncs.pyx":5644
 * ufunc_eval_laguerre_ptr[2*1+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*2] = <void*>_func_eval_laguerre[double_complex]
 * ufunc_eval_laguerre_ptr[2*2+1] = <void*>(<char*>"eval_laguerre")             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_ptr[2*3] = <void*>_func_eval_laguerre[double]
 * ufunc_eval_laguerre_ptr[2*3+1] = <void*>(<char*>"eval_laguerre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[5]) = ((void *)((char *)((char *)"eval_laguerre")));

  /* "scipy/special/_ufuncs.pyx":5645
 * ufunc_eval_laguerre_ptr[2*2] = <void*>_func_eval_laguerre[double_complex]
 * ufunc_eval_laguerre_ptr[2*2+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*3] = <void*>_func_eval_laguerre[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_ptr[2*3+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*4] = <void*>_func_eval_laguerre[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre);

  /* "scipy/special/_ufuncs.pyx":5646
 * ufunc_eval_laguerre_ptr[2*2+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*3] = <void*>_func_eval_laguerre[double]
 * ufunc_eval_laguerre_ptr[2*3+1] = <void*>(<char*>"eval_laguerre")             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_ptr[2*4] = <void*>_func_eval_laguerre[double_complex]
 * ufunc_eval_laguerre_ptr[2*4+1] = <void*>(<char*>"eval_laguerre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[7]) = ((void *)((char *)((char *)"eval_laguerre")));

  /* "scipy/special/_ufuncs.pyx":5647
 * ufunc_eval_laguerre_ptr[2*3] = <void*>_func_eval_laguerre[double]
 * ufunc_eval_laguerre_ptr[2*3+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*4] = <void*>_func_eval_laguerre[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_ptr[2*4+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_data[0] = &ufunc_eval_laguerre_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_laguerre);

  /* "scipy/special/_ufuncs.pyx":5648
 * ufunc_eval_laguerre_ptr[2*3+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_ptr[2*4] = <void*>_func_eval_laguerre[double_complex]
 * ufunc_eval_laguerre_ptr[2*4+1] = <void*>(<char*>"eval_laguerre")             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_data[0] = &ufunc_eval_laguerre_ptr[2*0]
 * ufunc_eval_laguerre_data[1] = &ufunc_eval_laguerre_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[9]) = ((void *)((char *)((char *)"eval_laguerre")));

  /* "scipy/special/_ufuncs.pyx":5649
 * ufunc_eval_laguerre_ptr[2*4] = <void*>_func_eval_laguerre[double_complex]
 * ufunc_eval_laguerre_ptr[2*4+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_data[0] = &ufunc_eval_laguerre_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_data[1] = &ufunc_eval_laguerre_ptr[2*1]
 * ufunc_eval_laguerre_data[2] = &ufunc_eval_laguerre_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5650
 * ufunc_eval_laguerre_ptr[2*4+1] = <void*>(<char*>"eval_laguerre")
 * ufunc_eval_laguerre_data[0] = &ufunc_eval_laguerre_ptr[2*0]
 * ufunc_eval_laguerre_data[1] = &ufunc_eval_laguerre_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_data[2] = &ufunc_eval_laguerre_ptr[2*2]
 * ufunc_eval_laguerre_data[3] = &ufunc_eval_laguerre_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5651
 * ufunc_eval_laguerre_data[0] = &ufunc_eval_laguerre_ptr[2*0]
 * ufunc_eval_laguerre_data[1] = &ufunc_eval_laguerre_ptr[2*1]
 * ufunc_eval_laguerre_data[2] = &ufunc_eval_laguerre_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_data[3] = &ufunc_eval_laguerre_ptr[2*3]
 * ufunc_eval_laguerre_data[4] = &ufunc_eval_laguerre_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":5652
 * ufunc_eval_laguerre_data[1] = &ufunc_eval_laguerre_ptr[2*1]
 * ufunc_eval_laguerre_data[2] = &ufunc_eval_laguerre_ptr[2*2]
 * ufunc_eval_laguerre_data[3] = &ufunc_eval_laguerre_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_laguerre_data[4] = &ufunc_eval_laguerre_ptr[2*4]
 * eval_laguerre = np.PyUFunc_FromFuncAndData(ufunc_eval_laguerre_loops, ufunc_eval_laguerre_data, ufunc_eval_laguerre_types, 5, 2, 1, 0, "eval_laguerre", ufunc_eval_laguerre_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":5653
 * ufunc_eval_laguerre_data[2] = &ufunc_eval_laguerre_ptr[2*2]
 * ufunc_eval_laguerre_data[3] = &ufunc_eval_laguerre_ptr[2*3]
 * ufunc_eval_laguerre_data[4] = &ufunc_eval_laguerre_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_laguerre = np.PyUFunc_FromFuncAndData(ufunc_eval_laguerre_loops, ufunc_eval_laguerre_data, ufunc_eval_laguerre_types, 5, 2, 1, 0, "eval_laguerre", ufunc_eval_laguerre_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":5654
 * ufunc_eval_laguerre_data[3] = &ufunc_eval_laguerre_ptr[2*3]
 * ufunc_eval_laguerre_data[4] = &ufunc_eval_laguerre_ptr[2*4]
 * eval_laguerre = np.PyUFunc_FromFuncAndData(ufunc_eval_laguerre_loops, ufunc_eval_laguerre_data, ufunc_eval_laguerre_types, 5, 2, 1, 0, "eval_laguerre", ufunc_eval_laguerre_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_legendre_loops[5]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_types, 5, 2, 1, 0, ((char *)"eval_laguerre"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_laguerre_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5654, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_laguerre, __pyx_t_2) < 0) __PYX_ERR(1, 5654, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":5660
 * cdef void *ufunc_eval_legendre_data[5]
 * cdef char ufunc_eval_legendre_types[15]
 * cdef char *ufunc_eval_legendre_doc = (             # <<<<<<<<<<<<<<
 *     "eval_legendre(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_doc = ((char *)"eval_legendre(n, x, out=None)\n\nEvaluate Legendre polynomial at a point.\n\nThe Legendre polynomials can be defined via the Gauss\nhypergeometric function :math:`{}_2F_1` as\n\n.. math::\n\n    P_n(x) = {}_2F_1(-n, n + 1; 1; (1 - x)/2).\n\nWhen :math:`n` is an integer the result is a polynomial of degree\n:math:`n`.\n\nParameters\n----------\nn : array_like\n    Degree of the polynomial. If not an integer, the result is\n    determined via the relation to the Gauss hypergeometric\n    function.\nx : array_like\n    Points at which to evaluate the Legendre polynomial\n\nReturns\n-------\nP : ndarray\n    Values of the Legendre polynomial\n\nSee Also\n--------\nroots_legendre : roots and quadrature weights of Legendre\n                 polynomials\nlegendre : Legendre polynomial object\nhyp2f1 : Gauss hypergeometric function\nnumpy.polynomial.legendre.Legendre : Legendre series");

  /* "scipy/special/_ufuncs.pyx":5696
 *     "hyp2f1 : Gauss hypergeometric function\n"
 *     "numpy.polynomial.legendre.Legendre : Legendre series")
 * ufunc_eval_legendre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_legendre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":5697
 *     "numpy.polynomial.legendre.Legendre : Legendre series")
 * ufunc_eval_legendre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_legendre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_legendre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":5698
 * ufunc_eval_legendre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_legendre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_legendre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_legendre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":5699
 * ufunc_eval_legendre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_legendre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_legendre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_legendre_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":5700
 * ufunc_eval_legendre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_legendre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_legendre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[0] = <char>NPY_LONG
 * ufunc_eval_legendre_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":5701
 * ufunc_eval_legendre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_legendre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_legendre_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":5702
 * ufunc_eval_legendre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_legendre_types[0] = <char>NPY_LONG
 * ufunc_eval_legendre_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5703
 * ufunc_eval_legendre_types[0] = <char>NPY_LONG
 * ufunc_eval_legendre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[3] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5704
 * ufunc_eval_legendre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5705
 * ufunc_eval_legendre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[3] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5706
 * ufunc_eval_legendre_types[3] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5707
 * ufunc_eval_legendre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_legendre_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5708
 * ufunc_eval_legendre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_legendre_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5709
 * ufunc_eval_legendre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_legendre_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_legendre_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5710
 * ufunc_eval_legendre_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_legendre_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_legendre_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5711
 * ufunc_eval_legendre_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_legendre_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5712
 * ufunc_eval_legendre_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[13] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5713
 * ufunc_eval_legendre_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_legendre_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5714
 * ufunc_eval_legendre_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[13] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_legendre_ptr[2*0] = <void*>_func_eval_legendre_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[13]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5715
 * ufunc_eval_legendre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_legendre_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_legendre_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_ptr[2*0] = <void*>_func_eval_legendre_l
 * ufunc_eval_legendre_ptr[2*0+1] = <void*>(<char*>"eval_legendre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5716
 * ufunc_eval_legendre_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_legendre_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_legendre_ptr[2*0] = <void*>_func_eval_legendre_l             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_ptr[2*0+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*1] = <void*>_func_eval_legendre[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre_l);

  /* "scipy/special/_ufuncs.pyx":5717
 * ufunc_eval_legendre_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_legendre_ptr[2*0] = <void*>_func_eval_legendre_l
 * ufunc_eval_legendre_ptr[2*0+1] = <void*>(<char*>"eval_legendre")             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_ptr[2*1] = <void*>_func_eval_legendre[double]
 * ufunc_eval_legendre_ptr[2*1+1] = <void*>(<char*>"eval_legendre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[1]) = ((void *)((char *)((char *)"eval_legendre")));

  /* "scipy/special/_ufuncs.pyx":5718
 * ufunc_eval_legendre_ptr[2*0] = <void*>_func_eval_legendre_l
 * ufunc_eval_legendre_ptr[2*0+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*1] = <void*>_func_eval_legendre[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_ptr[2*1+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*2] = <void*>_func_eval_legendre[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre);

  /* "scipy/special/_ufuncs.pyx":5719
 * ufunc_eval_legendre_ptr[2*0+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*1] = <void*>_func_eval_legendre[double]
 * ufunc_eval_legendre_ptr[2*1+1] = <void*>(<char*>"eval_legendre")             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_ptr[2*2] = <void*>_func_eval_legendre[double_complex]
 * ufunc_eval_legendre_ptr[2*2+1] = <void*>(<char*>"eval_legendre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[3]) = ((void *)((char *)((char *)"eval_legendre")));

  /* "scipy/special/_ufuncs.pyx":5720
 * ufunc_eval_legendre_ptr[2*1] = <void*>_func_eval_legendre[double]
 * ufunc_eval_legendre_ptr[2*1+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*2] = <void*>_func_eval_legendre[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_ptr[2*2+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*3] = <void*>_func_eval_legendre[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre);

  /* "scipy/special/_ufuncs.pyx":5721
 * ufunc_eval_legendre_ptr[2*1+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*2] = <void*>_func_eval_legendre[double_complex]
 * ufunc_eval_legendre_ptr[2*2+1] = <void*>(<char*>"eval_legendre")             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_ptr[2*3] = <void*>_func_eval_legendre[double]
 * ufunc_eval_legendre_ptr[2*3+1] = <void*>(<char*>"eval_legendre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[5]) = ((void *)((char *)((char *)"eval_legendre")));

  /* "scipy/special/_ufuncs.pyx":5722
 * ufunc_eval_legendre_ptr[2*2] = <void*>_func_eval_legendre[double_complex]
 * ufunc_eval_legendre_ptr[2*2+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*3] = <void*>_func_eval_legendre[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_ptr[2*3+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*4] = <void*>_func_eval_legendre[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre);

  /* "scipy/special/_ufuncs.pyx":5723
 * ufunc_eval_legendre_ptr[2*2+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*3] = <void*>_func_eval_legendre[double]
 * ufunc_eval_legendre_ptr[2*3+1] = <void*>(<char*>"eval_legendre")             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_ptr[2*4] = <void*>_func_eval_legendre[double_complex]
 * ufunc_eval_legendre_ptr[2*4+1] = <void*>(<char*>"eval_legendre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[7]) = ((void *)((char *)((char *)"eval_legendre")));

  /* "scipy/special/_ufuncs.pyx":5724
 * ufunc_eval_legendre_ptr[2*3] = <void*>_func_eval_legendre[double]
 * ufunc_eval_legendre_ptr[2*3+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*4] = <void*>_func_eval_legendre[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_ptr[2*4+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_data[0] = &ufunc_eval_legendre_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_legendre);

  /* "scipy/special/_ufuncs.pyx":5725
 * ufunc_eval_legendre_ptr[2*3+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_ptr[2*4] = <void*>_func_eval_legendre[double_complex]
 * ufunc_eval_legendre_ptr[2*4+1] = <void*>(<char*>"eval_legendre")             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_data[0] = &ufunc_eval_legendre_ptr[2*0]
 * ufunc_eval_legendre_data[1] = &ufunc_eval_legendre_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[9]) = ((void *)((char *)((char *)"eval_legendre")));

  /* "scipy/special/_ufuncs.pyx":5726
 * ufunc_eval_legendre_ptr[2*4] = <void*>_func_eval_legendre[double_complex]
 * ufunc_eval_legendre_ptr[2*4+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_data[0] = &ufunc_eval_legendre_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_data[1] = &ufunc_eval_legendre_ptr[2*1]
 * ufunc_eval_legendre_data[2] = &ufunc_eval_legendre_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5727
 * ufunc_eval_legendre_ptr[2*4+1] = <void*>(<char*>"eval_legendre")
 * ufunc_eval_legendre_data[0] = &ufunc_eval_legendre_ptr[2*0]
 * ufunc_eval_legendre_data[1] = &ufunc_eval_legendre_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_data[2] = &ufunc_eval_legendre_ptr[2*2]
 * ufunc_eval_legendre_data[3] = &ufunc_eval_legendre_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5728
 * ufunc_eval_legendre_data[0] = &ufunc_eval_legendre_ptr[2*0]
 * ufunc_eval_legendre_data[1] = &ufunc_eval_legendre_ptr[2*1]
 * ufunc_eval_legendre_data[2] = &ufunc_eval_legendre_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_data[3] = &ufunc_eval_legendre_ptr[2*3]
 * ufunc_eval_legendre_data[4] = &ufunc_eval_legendre_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":5729
 * ufunc_eval_legendre_data[1] = &ufunc_eval_legendre_ptr[2*1]
 * ufunc_eval_legendre_data[2] = &ufunc_eval_legendre_ptr[2*2]
 * ufunc_eval_legendre_data[3] = &ufunc_eval_legendre_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_legendre_data[4] = &ufunc_eval_legendre_ptr[2*4]
 * eval_legendre = np.PyUFunc_FromFuncAndData(ufunc_eval_legendre_loops, ufunc_eval_legendre_data, ufunc_eval_legendre_types, 5, 2, 1, 0, "eval_legendre", ufunc_eval_legendre_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":5730
 * ufunc_eval_legendre_data[2] = &ufunc_eval_legendre_ptr[2*2]
 * ufunc_eval_legendre_data[3] = &ufunc_eval_legendre_ptr[2*3]
 * ufunc_eval_legendre_data[4] = &ufunc_eval_legendre_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_legendre = np.PyUFunc_FromFuncAndData(ufunc_eval_legendre_loops, ufunc_eval_legendre_data, ufunc_eval_legendre_types, 5, 2, 1, 0, "eval_legendre", ufunc_eval_legendre_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":5731
 * ufunc_eval_legendre_data[3] = &ufunc_eval_legendre_ptr[2*3]
 * ufunc_eval_legendre_data[4] = &ufunc_eval_legendre_ptr[2*4]
 * eval_legendre = np.PyUFunc_FromFuncAndData(ufunc_eval_legendre_loops, ufunc_eval_legendre_data, ufunc_eval_legendre_types, 5, 2, 1, 0, "eval_legendre", ufunc_eval_legendre_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_sh_chebyt_loops[5]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_types, 5, 2, 1, 0, ((char *)"eval_legendre"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_legendre_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5731, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_legendre, __pyx_t_2) < 0) __PYX_ERR(1, 5731, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":5737
 * cdef void *ufunc_eval_sh_chebyt_data[5]
 * cdef char ufunc_eval_sh_chebyt_types[15]
 * cdef char *ufunc_eval_sh_chebyt_doc = (             # <<<<<<<<<<<<<<
 *     "eval_sh_chebyt(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_doc = ((char *)"eval_sh_chebyt(n, x, out=None)\n\nEvaluate shifted Chebyshev polynomial of the first kind at a\npoint.\n\nThese polynomials are defined as\n\n.. math::\n\n    T_n^*(x) = T_n(2x - 1)\n\nwhere :math:`T_n` is a Chebyshev polynomial of the first kind.\n\nParameters\n----------\nn : array_like\n    Degree of the polynomial. If not an integer, the result is\n    determined via the relation to `eval_chebyt`.\nx : array_like\n    Points at which to evaluate the shifted Chebyshev polynomial\n\nReturns\n-------\nT : ndarray\n    Values of the shifted Chebyshev polynomial\n\nSee Also\n--------\nroots_sh_chebyt : roots and quadrature weights of shifted\n                  Chebyshev polynomials of the first kind\nsh_chebyt : shifted Chebyshev polynomial object\neval_chebyt : evaluate Chebyshev polynomials of the first kind\nnumpy.polynomial.chebyshev.Chebyshev : Chebyshev series");

  /* "scipy/special/_ufuncs.pyx":5771
 *     "eval_chebyt : evaluate Chebyshev polynomials of the first kind\n"
 *     "numpy.polynomial.chebyshev.Chebyshev : Chebyshev series")
 * ufunc_eval_sh_chebyt_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_sh_chebyt_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":5772
 *     "numpy.polynomial.chebyshev.Chebyshev : Chebyshev series")
 * ufunc_eval_sh_chebyt_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_sh_chebyt_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_sh_chebyt_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":5773
 * ufunc_eval_sh_chebyt_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_sh_chebyt_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_sh_chebyt_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_sh_chebyt_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":5774
 * ufunc_eval_sh_chebyt_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_sh_chebyt_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_sh_chebyt_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_sh_chebyt_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":5775
 * ufunc_eval_sh_chebyt_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_sh_chebyt_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_sh_chebyt_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_chebyt_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":5776
 * ufunc_eval_sh_chebyt_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_sh_chebyt_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_sh_chebyt_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":5777
 * ufunc_eval_sh_chebyt_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_sh_chebyt_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_chebyt_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5778
 * ufunc_eval_sh_chebyt_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_chebyt_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[3] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5779
 * ufunc_eval_sh_chebyt_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[4] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5780
 * ufunc_eval_sh_chebyt_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[3] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5781
 * ufunc_eval_sh_chebyt_types[3] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[4] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5782
 * ufunc_eval_sh_chebyt_types[4] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyt_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5783
 * ufunc_eval_sh_chebyt_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyt_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5784
 * ufunc_eval_sh_chebyt_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyt_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyt_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5785
 * ufunc_eval_sh_chebyt_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyt_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyt_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5786
 * ufunc_eval_sh_chebyt_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyt_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5787
 * ufunc_eval_sh_chebyt_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[13] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5788
 * ufunc_eval_sh_chebyt_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyt_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5789
 * ufunc_eval_sh_chebyt_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[13] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyt_ptr[2*0] = <void*>_func_eval_sh_chebyt_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[13]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5790
 * ufunc_eval_sh_chebyt_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyt_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyt_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_ptr[2*0] = <void*>_func_eval_sh_chebyt_l
 * ufunc_eval_sh_chebyt_ptr[2*0+1] = <void*>(<char*>"eval_sh_chebyt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5791
 * ufunc_eval_sh_chebyt_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyt_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyt_ptr[2*0] = <void*>_func_eval_sh_chebyt_l             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_ptr[2*0+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*1] = <void*>_func_eval_sh_chebyt[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt_l);

  /* "scipy/special/_ufuncs.pyx":5792
 * ufunc_eval_sh_chebyt_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyt_ptr[2*0] = <void*>_func_eval_sh_chebyt_l
 * ufunc_eval_sh_chebyt_ptr[2*0+1] = <void*>(<char*>"eval_sh_chebyt")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_ptr[2*1] = <void*>_func_eval_sh_chebyt[double]
 * ufunc_eval_sh_chebyt_ptr[2*1+1] = <void*>(<char*>"eval_sh_chebyt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[1]) = ((void *)((char *)((char *)"eval_sh_chebyt")));

  /* "scipy/special/_ufuncs.pyx":5793
 * ufunc_eval_sh_chebyt_ptr[2*0] = <void*>_func_eval_sh_chebyt_l
 * ufunc_eval_sh_chebyt_ptr[2*0+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*1] = <void*>_func_eval_sh_chebyt[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_ptr[2*1+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*2] = <void*>_func_eval_sh_chebyt[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt);

  /* "scipy/special/_ufuncs.pyx":5794
 * ufunc_eval_sh_chebyt_ptr[2*0+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*1] = <void*>_func_eval_sh_chebyt[double]
 * ufunc_eval_sh_chebyt_ptr[2*1+1] = <void*>(<char*>"eval_sh_chebyt")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_ptr[2*2] = <void*>_func_eval_sh_chebyt[double_complex]
 * ufunc_eval_sh_chebyt_ptr[2*2+1] = <void*>(<char*>"eval_sh_chebyt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[3]) = ((void *)((char *)((char *)"eval_sh_chebyt")));

  /* "scipy/special/_ufuncs.pyx":5795
 * ufunc_eval_sh_chebyt_ptr[2*1] = <void*>_func_eval_sh_chebyt[double]
 * ufunc_eval_sh_chebyt_ptr[2*1+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*2] = <void*>_func_eval_sh_chebyt[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_ptr[2*2+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*3] = <void*>_func_eval_sh_chebyt[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt);

  /* "scipy/special/_ufuncs.pyx":5796
 * ufunc_eval_sh_chebyt_ptr[2*1+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*2] = <void*>_func_eval_sh_chebyt[double_complex]
 * ufunc_eval_sh_chebyt_ptr[2*2+1] = <void*>(<char*>"eval_sh_chebyt")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_ptr[2*3] = <void*>_func_eval_sh_chebyt[double]
 * ufunc_eval_sh_chebyt_ptr[2*3+1] = <void*>(<char*>"eval_sh_chebyt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[5]) = ((void *)((char *)((char *)"eval_sh_chebyt")));

  /* "scipy/special/_ufuncs.pyx":5797
 * ufunc_eval_sh_chebyt_ptr[2*2] = <void*>_func_eval_sh_chebyt[double_complex]
 * ufunc_eval_sh_chebyt_ptr[2*2+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*3] = <void*>_func_eval_sh_chebyt[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_ptr[2*3+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*4] = <void*>_func_eval_sh_chebyt[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt);

  /* "scipy/special/_ufuncs.pyx":5798
 * ufunc_eval_sh_chebyt_ptr[2*2+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*3] = <void*>_func_eval_sh_chebyt[double]
 * ufunc_eval_sh_chebyt_ptr[2*3+1] = <void*>(<char*>"eval_sh_chebyt")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_ptr[2*4] = <void*>_func_eval_sh_chebyt[double_complex]
 * ufunc_eval_sh_chebyt_ptr[2*4+1] = <void*>(<char*>"eval_sh_chebyt")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[7]) = ((void *)((char *)((char *)"eval_sh_chebyt")));

  /* "scipy/special/_ufuncs.pyx":5799
 * ufunc_eval_sh_chebyt_ptr[2*3] = <void*>_func_eval_sh_chebyt[double]
 * ufunc_eval_sh_chebyt_ptr[2*3+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*4] = <void*>_func_eval_sh_chebyt[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_ptr[2*4+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_data[0] = &ufunc_eval_sh_chebyt_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyt);

  /* "scipy/special/_ufuncs.pyx":5800
 * ufunc_eval_sh_chebyt_ptr[2*3+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_ptr[2*4] = <void*>_func_eval_sh_chebyt[double_complex]
 * ufunc_eval_sh_chebyt_ptr[2*4+1] = <void*>(<char*>"eval_sh_chebyt")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_data[0] = &ufunc_eval_sh_chebyt_ptr[2*0]
 * ufunc_eval_sh_chebyt_data[1] = &ufunc_eval_sh_chebyt_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[9]) = ((void *)((char *)((char *)"eval_sh_chebyt")));

  /* "scipy/special/_ufuncs.pyx":5801
 * ufunc_eval_sh_chebyt_ptr[2*4] = <void*>_func_eval_sh_chebyt[double_complex]
 * ufunc_eval_sh_chebyt_ptr[2*4+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_data[0] = &ufunc_eval_sh_chebyt_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_data[1] = &ufunc_eval_sh_chebyt_ptr[2*1]
 * ufunc_eval_sh_chebyt_data[2] = &ufunc_eval_sh_chebyt_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5802
 * ufunc_eval_sh_chebyt_ptr[2*4+1] = <void*>(<char*>"eval_sh_chebyt")
 * ufunc_eval_sh_chebyt_data[0] = &ufunc_eval_sh_chebyt_ptr[2*0]
 * ufunc_eval_sh_chebyt_data[1] = &ufunc_eval_sh_chebyt_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_data[2] = &ufunc_eval_sh_chebyt_ptr[2*2]
 * ufunc_eval_sh_chebyt_data[3] = &ufunc_eval_sh_chebyt_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5803
 * ufunc_eval_sh_chebyt_data[0] = &ufunc_eval_sh_chebyt_ptr[2*0]
 * ufunc_eval_sh_chebyt_data[1] = &ufunc_eval_sh_chebyt_ptr[2*1]
 * ufunc_eval_sh_chebyt_data[2] = &ufunc_eval_sh_chebyt_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_data[3] = &ufunc_eval_sh_chebyt_ptr[2*3]
 * ufunc_eval_sh_chebyt_data[4] = &ufunc_eval_sh_chebyt_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":5804
 * ufunc_eval_sh_chebyt_data[1] = &ufunc_eval_sh_chebyt_ptr[2*1]
 * ufunc_eval_sh_chebyt_data[2] = &ufunc_eval_sh_chebyt_ptr[2*2]
 * ufunc_eval_sh_chebyt_data[3] = &ufunc_eval_sh_chebyt_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyt_data[4] = &ufunc_eval_sh_chebyt_ptr[2*4]
 * eval_sh_chebyt = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_chebyt_loops, ufunc_eval_sh_chebyt_data, ufunc_eval_sh_chebyt_types, 5, 2, 1, 0, "eval_sh_chebyt", ufunc_eval_sh_chebyt_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":5805
 * ufunc_eval_sh_chebyt_data[2] = &ufunc_eval_sh_chebyt_ptr[2*2]
 * ufunc_eval_sh_chebyt_data[3] = &ufunc_eval_sh_chebyt_ptr[2*3]
 * ufunc_eval_sh_chebyt_data[4] = &ufunc_eval_sh_chebyt_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_sh_chebyt = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_chebyt_loops, ufunc_eval_sh_chebyt_data, ufunc_eval_sh_chebyt_types, 5, 2, 1, 0, "eval_sh_chebyt", ufunc_eval_sh_chebyt_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":5806
 * ufunc_eval_sh_chebyt_data[3] = &ufunc_eval_sh_chebyt_ptr[2*3]
 * ufunc_eval_sh_chebyt_data[4] = &ufunc_eval_sh_chebyt_ptr[2*4]
 * eval_sh_chebyt = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_chebyt_loops, ufunc_eval_sh_chebyt_data, ufunc_eval_sh_chebyt_types, 5, 2, 1, 0, "eval_sh_chebyt", ufunc_eval_sh_chebyt_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_sh_chebyu_loops[5]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_types, 5, 2, 1, 0, ((char *)"eval_sh_chebyt"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyt_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5806, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_sh_chebyt, __pyx_t_2) < 0) __PYX_ERR(1, 5806, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":5812
 * cdef void *ufunc_eval_sh_chebyu_data[5]
 * cdef char ufunc_eval_sh_chebyu_types[15]
 * cdef char *ufunc_eval_sh_chebyu_doc = (             # <<<<<<<<<<<<<<
 *     "eval_sh_chebyu(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_doc = ((char *)"eval_sh_chebyu(n, x, out=None)\n\nEvaluate shifted Chebyshev polynomial of the second kind at a\npoint.\n\nThese polynomials are defined as\n\n.. math::\n\n    U_n^*(x) = U_n(2x - 1)\n\nwhere :math:`U_n` is a Chebyshev polynomial of the first kind.\n\nParameters\n----------\nn : array_like\n    Degree of the polynomial. If not an integer, the result is\n    determined via the relation to `eval_chebyu`.\nx : array_like\n    Points at which to evaluate the shifted Chebyshev polynomial\n\nReturns\n-------\nU : ndarray\n    Values of the shifted Chebyshev polynomial\n\nSee Also\n--------\nroots_sh_chebyu : roots and quadrature weights of shifted\n                  Chebychev polynomials of the second kind\nsh_chebyu : shifted Chebyshev polynomial object\neval_chebyu : evaluate Chebyshev polynomials of the second kind");

  /* "scipy/special/_ufuncs.pyx":5845
 *     "sh_chebyu : shifted Chebyshev polynomial object\n"
 *     "eval_chebyu : evaluate Chebyshev polynomials of the second kind")
 * ufunc_eval_sh_chebyu_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_sh_chebyu_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":5846
 *     "eval_chebyu : evaluate Chebyshev polynomials of the second kind")
 * ufunc_eval_sh_chebyu_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_sh_chebyu_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_sh_chebyu_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":5847
 * ufunc_eval_sh_chebyu_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_sh_chebyu_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_sh_chebyu_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_sh_chebyu_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":5848
 * ufunc_eval_sh_chebyu_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_sh_chebyu_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_sh_chebyu_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_sh_chebyu_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":5849
 * ufunc_eval_sh_chebyu_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_sh_chebyu_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_sh_chebyu_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_chebyu_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":5850
 * ufunc_eval_sh_chebyu_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_sh_chebyu_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_sh_chebyu_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":5851
 * ufunc_eval_sh_chebyu_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_sh_chebyu_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_chebyu_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5852
 * ufunc_eval_sh_chebyu_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_chebyu_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[3] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5853
 * ufunc_eval_sh_chebyu_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[4] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5854
 * ufunc_eval_sh_chebyu_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[3] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5855
 * ufunc_eval_sh_chebyu_types[3] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[4] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5856
 * ufunc_eval_sh_chebyu_types[4] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyu_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5857
 * ufunc_eval_sh_chebyu_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyu_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5858
 * ufunc_eval_sh_chebyu_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_chebyu_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyu_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5859
 * ufunc_eval_sh_chebyu_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyu_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyu_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5860
 * ufunc_eval_sh_chebyu_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_sh_chebyu_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5861
 * ufunc_eval_sh_chebyu_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[13] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5862
 * ufunc_eval_sh_chebyu_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyu_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5863
 * ufunc_eval_sh_chebyu_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[13] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyu_ptr[2*0] = <void*>_func_eval_sh_chebyu_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[13]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5864
 * ufunc_eval_sh_chebyu_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_sh_chebyu_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyu_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_ptr[2*0] = <void*>_func_eval_sh_chebyu_l
 * ufunc_eval_sh_chebyu_ptr[2*0+1] = <void*>(<char*>"eval_sh_chebyu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5865
 * ufunc_eval_sh_chebyu_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyu_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyu_ptr[2*0] = <void*>_func_eval_sh_chebyu_l             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_ptr[2*0+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*1] = <void*>_func_eval_sh_chebyu[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu_l);

  /* "scipy/special/_ufuncs.pyx":5866
 * ufunc_eval_sh_chebyu_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_chebyu_ptr[2*0] = <void*>_func_eval_sh_chebyu_l
 * ufunc_eval_sh_chebyu_ptr[2*0+1] = <void*>(<char*>"eval_sh_chebyu")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_ptr[2*1] = <void*>_func_eval_sh_chebyu[double]
 * ufunc_eval_sh_chebyu_ptr[2*1+1] = <void*>(<char*>"eval_sh_chebyu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[1]) = ((void *)((char *)((char *)"eval_sh_chebyu")));

  /* "scipy/special/_ufuncs.pyx":5867
 * ufunc_eval_sh_chebyu_ptr[2*0] = <void*>_func_eval_sh_chebyu_l
 * ufunc_eval_sh_chebyu_ptr[2*0+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*1] = <void*>_func_eval_sh_chebyu[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_ptr[2*1+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*2] = <void*>_func_eval_sh_chebyu[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu);

  /* "scipy/special/_ufuncs.pyx":5868
 * ufunc_eval_sh_chebyu_ptr[2*0+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*1] = <void*>_func_eval_sh_chebyu[double]
 * ufunc_eval_sh_chebyu_ptr[2*1+1] = <void*>(<char*>"eval_sh_chebyu")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_ptr[2*2] = <void*>_func_eval_sh_chebyu[double_complex]
 * ufunc_eval_sh_chebyu_ptr[2*2+1] = <void*>(<char*>"eval_sh_chebyu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[3]) = ((void *)((char *)((char *)"eval_sh_chebyu")));

  /* "scipy/special/_ufuncs.pyx":5869
 * ufunc_eval_sh_chebyu_ptr[2*1] = <void*>_func_eval_sh_chebyu[double]
 * ufunc_eval_sh_chebyu_ptr[2*1+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*2] = <void*>_func_eval_sh_chebyu[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_ptr[2*2+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*3] = <void*>_func_eval_sh_chebyu[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu);

  /* "scipy/special/_ufuncs.pyx":5870
 * ufunc_eval_sh_chebyu_ptr[2*1+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*2] = <void*>_func_eval_sh_chebyu[double_complex]
 * ufunc_eval_sh_chebyu_ptr[2*2+1] = <void*>(<char*>"eval_sh_chebyu")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_ptr[2*3] = <void*>_func_eval_sh_chebyu[double]
 * ufunc_eval_sh_chebyu_ptr[2*3+1] = <void*>(<char*>"eval_sh_chebyu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[5]) = ((void *)((char *)((char *)"eval_sh_chebyu")));

  /* "scipy/special/_ufuncs.pyx":5871
 * ufunc_eval_sh_chebyu_ptr[2*2] = <void*>_func_eval_sh_chebyu[double_complex]
 * ufunc_eval_sh_chebyu_ptr[2*2+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*3] = <void*>_func_eval_sh_chebyu[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_ptr[2*3+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*4] = <void*>_func_eval_sh_chebyu[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu);

  /* "scipy/special/_ufuncs.pyx":5872
 * ufunc_eval_sh_chebyu_ptr[2*2+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*3] = <void*>_func_eval_sh_chebyu[double]
 * ufunc_eval_sh_chebyu_ptr[2*3+1] = <void*>(<char*>"eval_sh_chebyu")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_ptr[2*4] = <void*>_func_eval_sh_chebyu[double_complex]
 * ufunc_eval_sh_chebyu_ptr[2*4+1] = <void*>(<char*>"eval_sh_chebyu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[7]) = ((void *)((char *)((char *)"eval_sh_chebyu")));

  /* "scipy/special/_ufuncs.pyx":5873
 * ufunc_eval_sh_chebyu_ptr[2*3] = <void*>_func_eval_sh_chebyu[double]
 * ufunc_eval_sh_chebyu_ptr[2*3+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*4] = <void*>_func_eval_sh_chebyu[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_ptr[2*4+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_data[0] = &ufunc_eval_sh_chebyu_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_chebyu);

  /* "scipy/special/_ufuncs.pyx":5874
 * ufunc_eval_sh_chebyu_ptr[2*3+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_ptr[2*4] = <void*>_func_eval_sh_chebyu[double_complex]
 * ufunc_eval_sh_chebyu_ptr[2*4+1] = <void*>(<char*>"eval_sh_chebyu")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_data[0] = &ufunc_eval_sh_chebyu_ptr[2*0]
 * ufunc_eval_sh_chebyu_data[1] = &ufunc_eval_sh_chebyu_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[9]) = ((void *)((char *)((char *)"eval_sh_chebyu")));

  /* "scipy/special/_ufuncs.pyx":5875
 * ufunc_eval_sh_chebyu_ptr[2*4] = <void*>_func_eval_sh_chebyu[double_complex]
 * ufunc_eval_sh_chebyu_ptr[2*4+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_data[0] = &ufunc_eval_sh_chebyu_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_data[1] = &ufunc_eval_sh_chebyu_ptr[2*1]
 * ufunc_eval_sh_chebyu_data[2] = &ufunc_eval_sh_chebyu_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5876
 * ufunc_eval_sh_chebyu_ptr[2*4+1] = <void*>(<char*>"eval_sh_chebyu")
 * ufunc_eval_sh_chebyu_data[0] = &ufunc_eval_sh_chebyu_ptr[2*0]
 * ufunc_eval_sh_chebyu_data[1] = &ufunc_eval_sh_chebyu_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_data[2] = &ufunc_eval_sh_chebyu_ptr[2*2]
 * ufunc_eval_sh_chebyu_data[3] = &ufunc_eval_sh_chebyu_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5877
 * ufunc_eval_sh_chebyu_data[0] = &ufunc_eval_sh_chebyu_ptr[2*0]
 * ufunc_eval_sh_chebyu_data[1] = &ufunc_eval_sh_chebyu_ptr[2*1]
 * ufunc_eval_sh_chebyu_data[2] = &ufunc_eval_sh_chebyu_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_data[3] = &ufunc_eval_sh_chebyu_ptr[2*3]
 * ufunc_eval_sh_chebyu_data[4] = &ufunc_eval_sh_chebyu_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":5878
 * ufunc_eval_sh_chebyu_data[1] = &ufunc_eval_sh_chebyu_ptr[2*1]
 * ufunc_eval_sh_chebyu_data[2] = &ufunc_eval_sh_chebyu_ptr[2*2]
 * ufunc_eval_sh_chebyu_data[3] = &ufunc_eval_sh_chebyu_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_chebyu_data[4] = &ufunc_eval_sh_chebyu_ptr[2*4]
 * eval_sh_chebyu = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_chebyu_loops, ufunc_eval_sh_chebyu_data, ufunc_eval_sh_chebyu_types, 5, 2, 1, 0, "eval_sh_chebyu", ufunc_eval_sh_chebyu_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":5879
 * ufunc_eval_sh_chebyu_data[2] = &ufunc_eval_sh_chebyu_ptr[2*2]
 * ufunc_eval_sh_chebyu_data[3] = &ufunc_eval_sh_chebyu_ptr[2*3]
 * ufunc_eval_sh_chebyu_data[4] = &ufunc_eval_sh_chebyu_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_sh_chebyu = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_chebyu_loops, ufunc_eval_sh_chebyu_data, ufunc_eval_sh_chebyu_types, 5, 2, 1, 0, "eval_sh_chebyu", ufunc_eval_sh_chebyu_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":5880
 * ufunc_eval_sh_chebyu_data[3] = &ufunc_eval_sh_chebyu_ptr[2*3]
 * ufunc_eval_sh_chebyu_data[4] = &ufunc_eval_sh_chebyu_ptr[2*4]
 * eval_sh_chebyu = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_chebyu_loops, ufunc_eval_sh_chebyu_data, ufunc_eval_sh_chebyu_types, 5, 2, 1, 0, "eval_sh_chebyu", ufunc_eval_sh_chebyu_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_sh_jacobi_loops[5]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_types, 5, 2, 1, 0, ((char *)"eval_sh_chebyu"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_chebyu_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5880, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_sh_chebyu, __pyx_t_2) < 0) __PYX_ERR(1, 5880, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":5886
 * cdef void *ufunc_eval_sh_jacobi_data[5]
 * cdef char ufunc_eval_sh_jacobi_types[25]
 * cdef char *ufunc_eval_sh_jacobi_doc = (             # <<<<<<<<<<<<<<
 *     "eval_sh_jacobi(n, p, q, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_doc = ((char *)"eval_sh_jacobi(n, p, q, x, out=None)\n\nEvaluate shifted Jacobi polynomial at a point.\n\nDefined by\n\n.. math::\n\n    G_n^{(p, q)}(x)\n      = \\binom{2n + p - 1}{n}^{-1} P_n^{(p - q, q - 1)}(2x - 1),\n\nwhere :math:`P_n^{(\\cdot, \\cdot)}` is the n-th Jacobi polynomial.\n\nParameters\n----------\nn : int\n    Degree of the polynomial. If not an integer, the result is\n    determined via the relation to `binom` and `eval_jacobi`.\np : float\n    Parameter\nq : float\n    Parameter\n\nReturns\n-------\nG : ndarray\n    Values of the shifted Jacobi polynomial.\n\nSee Also\n--------\nroots_sh_jacobi : roots and quadrature weights of shifted Jacobi\n                  polynomials\nsh_jacobi : shifted Jacobi polynomial object\neval_jacobi : evaluate Jacobi polynomials");

  /* "scipy/special/_ufuncs.pyx":5921
 *     "sh_jacobi : shifted Jacobi polynomial object\n"
 *     "eval_jacobi : evaluate Jacobi polynomials")
 * ufunc_eval_sh_jacobi_loops[0] = <np.PyUFuncGenericFunction>loop_d_lddd__As_lddd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_eval_sh_jacobi_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_lddd__As_lddd_d);

  /* "scipy/special/_ufuncs.pyx":5922
 *     "eval_jacobi : evaluate Jacobi polynomials")
 * ufunc_eval_sh_jacobi_loops[0] = <np.PyUFuncGenericFunction>loop_d_lddd__As_lddd_d
 * ufunc_eval_sh_jacobi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 * ufunc_eval_sh_jacobi_loops[3] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_ffff_f);

  /* "scipy/special/_ufuncs.pyx":5923
 * ufunc_eval_sh_jacobi_loops[0] = <np.PyUFuncGenericFunction>loop_d_lddd__As_lddd_d
 * ufunc_eval_sh_jacobi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_eval_sh_jacobi_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_loops[3] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_eval_sh_jacobi_loops[4] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dddD__As_fffF_F);

  /* "scipy/special/_ufuncs.pyx":5924
 * ufunc_eval_sh_jacobi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_eval_sh_jacobi_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 * ufunc_eval_sh_jacobi_loops[3] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_loops[4] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 * ufunc_eval_sh_jacobi_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_dddd_d);

  /* "scipy/special/_ufuncs.pyx":5925
 * ufunc_eval_sh_jacobi_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 * ufunc_eval_sh_jacobi_loops[3] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_eval_sh_jacobi_loops[4] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_jacobi_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dddD__As_dddD_D);

  /* "scipy/special/_ufuncs.pyx":5926
 * ufunc_eval_sh_jacobi_loops[3] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_eval_sh_jacobi_loops[4] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 * ufunc_eval_sh_jacobi_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":5927
 * ufunc_eval_sh_jacobi_loops[4] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 * ufunc_eval_sh_jacobi_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_jacobi_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5928
 * ufunc_eval_sh_jacobi_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_jacobi_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5929
 * ufunc_eval_sh_jacobi_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[4] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5930
 * ufunc_eval_sh_jacobi_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5931
 * ufunc_eval_sh_jacobi_types[3] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[4] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5932
 * ufunc_eval_sh_jacobi_types[4] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[7] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[8] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5933
 * ufunc_eval_sh_jacobi_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[8] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[9] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5934
 * ufunc_eval_sh_jacobi_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[7] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[8] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[9] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[10] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[8]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5935
 * ufunc_eval_sh_jacobi_types[7] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[8] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[9] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[10] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[11] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[9]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5936
 * ufunc_eval_sh_jacobi_types[8] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[9] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[10] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[11] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[12] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[10]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5937
 * ufunc_eval_sh_jacobi_types[9] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[10] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[11] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[12] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[13] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[11]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5938
 * ufunc_eval_sh_jacobi_types[10] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[11] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[12] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[13] = <char>NPY_CFLOAT
 * ufunc_eval_sh_jacobi_types[14] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[12]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":5939
 * ufunc_eval_sh_jacobi_types[11] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[12] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[13] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[14] = <char>NPY_CFLOAT
 * ufunc_eval_sh_jacobi_types[15] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[13]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5940
 * ufunc_eval_sh_jacobi_types[12] = <char>NPY_FLOAT
 * ufunc_eval_sh_jacobi_types[13] = <char>NPY_CFLOAT
 * ufunc_eval_sh_jacobi_types[14] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[16] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[14]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":5941
 * ufunc_eval_sh_jacobi_types[13] = <char>NPY_CFLOAT
 * ufunc_eval_sh_jacobi_types[14] = <char>NPY_CFLOAT
 * ufunc_eval_sh_jacobi_types[15] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[17] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[15]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5942
 * ufunc_eval_sh_jacobi_types[14] = <char>NPY_CFLOAT
 * ufunc_eval_sh_jacobi_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[16] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[18] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[16]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5943
 * ufunc_eval_sh_jacobi_types[15] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[17] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[18] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[19] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[17]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5944
 * ufunc_eval_sh_jacobi_types[16] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[18] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[19] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[20] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[18]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5945
 * ufunc_eval_sh_jacobi_types[17] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[18] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[19] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[20] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[21] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[19]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5946
 * ufunc_eval_sh_jacobi_types[18] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[19] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[20] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[21] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[22] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[20]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5947
 * ufunc_eval_sh_jacobi_types[19] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[20] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[21] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[22] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[23] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[21]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5948
 * ufunc_eval_sh_jacobi_types[20] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[21] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[22] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[23] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_jacobi_types[24] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[22]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":5949
 * ufunc_eval_sh_jacobi_types[21] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[22] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[23] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_types[24] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_jacobi_ptr[2*0] = <void*>_func_eval_sh_jacobi_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[23]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5950
 * ufunc_eval_sh_jacobi_types[22] = <char>NPY_DOUBLE
 * ufunc_eval_sh_jacobi_types[23] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_jacobi_types[24] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_ptr[2*0] = <void*>_func_eval_sh_jacobi_l
 * ufunc_eval_sh_jacobi_ptr[2*0+1] = <void*>(<char*>"eval_sh_jacobi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types[24]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":5951
 * ufunc_eval_sh_jacobi_types[23] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_jacobi_types[24] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_jacobi_ptr[2*0] = <void*>_func_eval_sh_jacobi_l             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_ptr[2*0+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*1] = <void*>_func_eval_sh_jacobi[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi_l);

  /* "scipy/special/_ufuncs.pyx":5952
 * ufunc_eval_sh_jacobi_types[24] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_jacobi_ptr[2*0] = <void*>_func_eval_sh_jacobi_l
 * ufunc_eval_sh_jacobi_ptr[2*0+1] = <void*>(<char*>"eval_sh_jacobi")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_ptr[2*1] = <void*>_func_eval_sh_jacobi[double]
 * ufunc_eval_sh_jacobi_ptr[2*1+1] = <void*>(<char*>"eval_sh_jacobi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[1]) = ((void *)((char *)((char *)"eval_sh_jacobi")));

  /* "scipy/special/_ufuncs.pyx":5953
 * ufunc_eval_sh_jacobi_ptr[2*0] = <void*>_func_eval_sh_jacobi_l
 * ufunc_eval_sh_jacobi_ptr[2*0+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*1] = <void*>_func_eval_sh_jacobi[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_ptr[2*1+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*2] = <void*>_func_eval_sh_jacobi[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi);

  /* "scipy/special/_ufuncs.pyx":5954
 * ufunc_eval_sh_jacobi_ptr[2*0+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*1] = <void*>_func_eval_sh_jacobi[double]
 * ufunc_eval_sh_jacobi_ptr[2*1+1] = <void*>(<char*>"eval_sh_jacobi")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_ptr[2*2] = <void*>_func_eval_sh_jacobi[double_complex]
 * ufunc_eval_sh_jacobi_ptr[2*2+1] = <void*>(<char*>"eval_sh_jacobi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[3]) = ((void *)((char *)((char *)"eval_sh_jacobi")));

  /* "scipy/special/_ufuncs.pyx":5955
 * ufunc_eval_sh_jacobi_ptr[2*1] = <void*>_func_eval_sh_jacobi[double]
 * ufunc_eval_sh_jacobi_ptr[2*1+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*2] = <void*>_func_eval_sh_jacobi[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_ptr[2*2+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*3] = <void*>_func_eval_sh_jacobi[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi);

  /* "scipy/special/_ufuncs.pyx":5956
 * ufunc_eval_sh_jacobi_ptr[2*1+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*2] = <void*>_func_eval_sh_jacobi[double_complex]
 * ufunc_eval_sh_jacobi_ptr[2*2+1] = <void*>(<char*>"eval_sh_jacobi")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_ptr[2*3] = <void*>_func_eval_sh_jacobi[double]
 * ufunc_eval_sh_jacobi_ptr[2*3+1] = <void*>(<char*>"eval_sh_jacobi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[5]) = ((void *)((char *)((char *)"eval_sh_jacobi")));

  /* "scipy/special/_ufuncs.pyx":5957
 * ufunc_eval_sh_jacobi_ptr[2*2] = <void*>_func_eval_sh_jacobi[double_complex]
 * ufunc_eval_sh_jacobi_ptr[2*2+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*3] = <void*>_func_eval_sh_jacobi[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_ptr[2*3+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*4] = <void*>_func_eval_sh_jacobi[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi);

  /* "scipy/special/_ufuncs.pyx":5958
 * ufunc_eval_sh_jacobi_ptr[2*2+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*3] = <void*>_func_eval_sh_jacobi[double]
 * ufunc_eval_sh_jacobi_ptr[2*3+1] = <void*>(<char*>"eval_sh_jacobi")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_ptr[2*4] = <void*>_func_eval_sh_jacobi[double_complex]
 * ufunc_eval_sh_jacobi_ptr[2*4+1] = <void*>(<char*>"eval_sh_jacobi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[7]) = ((void *)((char *)((char *)"eval_sh_jacobi")));

  /* "scipy/special/_ufuncs.pyx":5959
 * ufunc_eval_sh_jacobi_ptr[2*3] = <void*>_func_eval_sh_jacobi[double]
 * ufunc_eval_sh_jacobi_ptr[2*3+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*4] = <void*>_func_eval_sh_jacobi[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_ptr[2*4+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_data[0] = &ufunc_eval_sh_jacobi_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_jacobi);

  /* "scipy/special/_ufuncs.pyx":5960
 * ufunc_eval_sh_jacobi_ptr[2*3+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_ptr[2*4] = <void*>_func_eval_sh_jacobi[double_complex]
 * ufunc_eval_sh_jacobi_ptr[2*4+1] = <void*>(<char*>"eval_sh_jacobi")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_data[0] = &ufunc_eval_sh_jacobi_ptr[2*0]
 * ufunc_eval_sh_jacobi_data[1] = &ufunc_eval_sh_jacobi_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[9]) = ((void *)((char *)((char *)"eval_sh_jacobi")));

  /* "scipy/special/_ufuncs.pyx":5961
 * ufunc_eval_sh_jacobi_ptr[2*4] = <void*>_func_eval_sh_jacobi[double_complex]
 * ufunc_eval_sh_jacobi_ptr[2*4+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_data[0] = &ufunc_eval_sh_jacobi_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_data[1] = &ufunc_eval_sh_jacobi_ptr[2*1]
 * ufunc_eval_sh_jacobi_data[2] = &ufunc_eval_sh_jacobi_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":5962
 * ufunc_eval_sh_jacobi_ptr[2*4+1] = <void*>(<char*>"eval_sh_jacobi")
 * ufunc_eval_sh_jacobi_data[0] = &ufunc_eval_sh_jacobi_ptr[2*0]
 * ufunc_eval_sh_jacobi_data[1] = &ufunc_eval_sh_jacobi_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_data[2] = &ufunc_eval_sh_jacobi_ptr[2*2]
 * ufunc_eval_sh_jacobi_data[3] = &ufunc_eval_sh_jacobi_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":5963
 * ufunc_eval_sh_jacobi_data[0] = &ufunc_eval_sh_jacobi_ptr[2*0]
 * ufunc_eval_sh_jacobi_data[1] = &ufunc_eval_sh_jacobi_ptr[2*1]
 * ufunc_eval_sh_jacobi_data[2] = &ufunc_eval_sh_jacobi_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_data[3] = &ufunc_eval_sh_jacobi_ptr[2*3]
 * ufunc_eval_sh_jacobi_data[4] = &ufunc_eval_sh_jacobi_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":5964
 * ufunc_eval_sh_jacobi_data[1] = &ufunc_eval_sh_jacobi_ptr[2*1]
 * ufunc_eval_sh_jacobi_data[2] = &ufunc_eval_sh_jacobi_ptr[2*2]
 * ufunc_eval_sh_jacobi_data[3] = &ufunc_eval_sh_jacobi_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_jacobi_data[4] = &ufunc_eval_sh_jacobi_ptr[2*4]
 * eval_sh_jacobi = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_jacobi_loops, ufunc_eval_sh_jacobi_data, ufunc_eval_sh_jacobi_types, 5, 4, 1, 0, "eval_sh_jacobi", ufunc_eval_sh_jacobi_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":5965
 * ufunc_eval_sh_jacobi_data[2] = &ufunc_eval_sh_jacobi_ptr[2*2]
 * ufunc_eval_sh_jacobi_data[3] = &ufunc_eval_sh_jacobi_ptr[2*3]
 * ufunc_eval_sh_jacobi_data[4] = &ufunc_eval_sh_jacobi_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_sh_jacobi = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_jacobi_loops, ufunc_eval_sh_jacobi_data, ufunc_eval_sh_jacobi_types, 5, 4, 1, 0, "eval_sh_jacobi", ufunc_eval_sh_jacobi_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":5966
 * ufunc_eval_sh_jacobi_data[3] = &ufunc_eval_sh_jacobi_ptr[2*3]
 * ufunc_eval_sh_jacobi_data[4] = &ufunc_eval_sh_jacobi_ptr[2*4]
 * eval_sh_jacobi = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_jacobi_loops, ufunc_eval_sh_jacobi_data, ufunc_eval_sh_jacobi_types, 5, 4, 1, 0, "eval_sh_jacobi", ufunc_eval_sh_jacobi_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_eval_sh_legendre_loops[5]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_types, 5, 4, 1, 0, ((char *)"eval_sh_jacobi"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_jacobi_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5966, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_sh_jacobi, __pyx_t_2) < 0) __PYX_ERR(1, 5966, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":5972
 * cdef void *ufunc_eval_sh_legendre_data[5]
 * cdef char ufunc_eval_sh_legendre_types[15]
 * cdef char *ufunc_eval_sh_legendre_doc = (             # <<<<<<<<<<<<<<
 *     "eval_sh_legendre(n, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_doc = ((char *)"eval_sh_legendre(n, x, out=None)\n\nEvaluate shifted Legendre polynomial at a point.\n\nThese polynomials are defined as\n\n.. math::\n\n    P_n^*(x) = P_n(2x - 1)\n\nwhere :math:`P_n` is a Legendre polynomial.\n\nParameters\n----------\nn : array_like\n    Degree of the polynomial. If not an integer, the value is\n    determined via the relation to `eval_legendre`.\nx : array_like\n    Points at which to evaluate the shifted Legendre polynomial\n\nReturns\n-------\nP : ndarray\n    Values of the shifted Legendre polynomial\n\nSee Also\n--------\nroots_sh_legendre : roots and quadrature weights of shifted\n                    Legendre polynomials\nsh_legendre : shifted Legendre polynomial object\neval_legendre : evaluate Legendre polynomials\nnumpy.polynomial.legendre.Legendre : Legendre series");

  /* "scipy/special/_ufuncs.pyx":6005
 *     "eval_legendre : evaluate Legendre polynomials\n"
 *     "numpy.polynomial.legendre.Legendre : Legendre series")
 * ufunc_eval_sh_legendre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_sh_legendre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ld__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":6006
 *     "numpy.polynomial.legendre.Legendre : Legendre series")
 * ufunc_eval_sh_legendre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_sh_legendre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_sh_legendre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":6007
 * ufunc_eval_sh_legendre_loops[0] = <np.PyUFuncGenericFunction>loop_d_ld__As_ld_d
 * ufunc_eval_sh_legendre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_sh_legendre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_sh_legendre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":6008
 * ufunc_eval_sh_legendre_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_eval_sh_legendre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_sh_legendre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_sh_legendre_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":6009
 * ufunc_eval_sh_legendre_loops[2] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_eval_sh_legendre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_sh_legendre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_legendre_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_loops[4]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":6010
 * ufunc_eval_sh_legendre_loops[3] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_eval_sh_legendre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_sh_legendre_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":6011
 * ufunc_eval_sh_legendre_loops[4] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_eval_sh_legendre_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_legendre_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6012
 * ufunc_eval_sh_legendre_types[0] = <char>NPY_LONG
 * ufunc_eval_sh_legendre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[3] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6013
 * ufunc_eval_sh_legendre_types[1] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6014
 * ufunc_eval_sh_legendre_types[2] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[3] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6015
 * ufunc_eval_sh_legendre_types[3] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6016
 * ufunc_eval_sh_legendre_types[4] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_sh_legendre_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6017
 * ufunc_eval_sh_legendre_types[5] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_sh_legendre_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6018
 * ufunc_eval_sh_legendre_types[6] = <char>NPY_FLOAT
 * ufunc_eval_sh_legendre_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_sh_legendre_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6019
 * ufunc_eval_sh_legendre_types[7] = <char>NPY_CFLOAT
 * ufunc_eval_sh_legendre_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_sh_legendre_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6020
 * ufunc_eval_sh_legendre_types[8] = <char>NPY_CFLOAT
 * ufunc_eval_sh_legendre_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6021
 * ufunc_eval_sh_legendre_types[9] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[13] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6022
 * ufunc_eval_sh_legendre_types[10] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_legendre_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6023
 * ufunc_eval_sh_legendre_types[11] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[13] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_legendre_ptr[2*0] = <void*>_func_eval_sh_legendre_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[13]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6024
 * ufunc_eval_sh_legendre_types[12] = <char>NPY_DOUBLE
 * ufunc_eval_sh_legendre_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_legendre_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_ptr[2*0] = <void*>_func_eval_sh_legendre_l
 * ufunc_eval_sh_legendre_ptr[2*0+1] = <void*>(<char*>"eval_sh_legendre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6025
 * ufunc_eval_sh_legendre_types[13] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_legendre_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_legendre_ptr[2*0] = <void*>_func_eval_sh_legendre_l             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_ptr[2*0+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*1] = <void*>_func_eval_sh_legendre[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[0]) = ((void *)__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre_l);

  /* "scipy/special/_ufuncs.pyx":6026
 * ufunc_eval_sh_legendre_types[14] = <char>NPY_CDOUBLE
 * ufunc_eval_sh_legendre_ptr[2*0] = <void*>_func_eval_sh_legendre_l
 * ufunc_eval_sh_legendre_ptr[2*0+1] = <void*>(<char*>"eval_sh_legendre")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_ptr[2*1] = <void*>_func_eval_sh_legendre[double]
 * ufunc_eval_sh_legendre_ptr[2*1+1] = <void*>(<char*>"eval_sh_legendre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[1]) = ((void *)((char *)((char *)"eval_sh_legendre")));

  /* "scipy/special/_ufuncs.pyx":6027
 * ufunc_eval_sh_legendre_ptr[2*0] = <void*>_func_eval_sh_legendre_l
 * ufunc_eval_sh_legendre_ptr[2*0+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*1] = <void*>_func_eval_sh_legendre[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_ptr[2*1+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*2] = <void*>_func_eval_sh_legendre[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre);

  /* "scipy/special/_ufuncs.pyx":6028
 * ufunc_eval_sh_legendre_ptr[2*0+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*1] = <void*>_func_eval_sh_legendre[double]
 * ufunc_eval_sh_legendre_ptr[2*1+1] = <void*>(<char*>"eval_sh_legendre")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_ptr[2*2] = <void*>_func_eval_sh_legendre[double_complex]
 * ufunc_eval_sh_legendre_ptr[2*2+1] = <void*>(<char*>"eval_sh_legendre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[3]) = ((void *)((char *)((char *)"eval_sh_legendre")));

  /* "scipy/special/_ufuncs.pyx":6029
 * ufunc_eval_sh_legendre_ptr[2*1] = <void*>_func_eval_sh_legendre[double]
 * ufunc_eval_sh_legendre_ptr[2*1+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*2] = <void*>_func_eval_sh_legendre[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_ptr[2*2+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*3] = <void*>_func_eval_sh_legendre[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre);

  /* "scipy/special/_ufuncs.pyx":6030
 * ufunc_eval_sh_legendre_ptr[2*1+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*2] = <void*>_func_eval_sh_legendre[double_complex]
 * ufunc_eval_sh_legendre_ptr[2*2+1] = <void*>(<char*>"eval_sh_legendre")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_ptr[2*3] = <void*>_func_eval_sh_legendre[double]
 * ufunc_eval_sh_legendre_ptr[2*3+1] = <void*>(<char*>"eval_sh_legendre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[5]) = ((void *)((char *)((char *)"eval_sh_legendre")));

  /* "scipy/special/_ufuncs.pyx":6031
 * ufunc_eval_sh_legendre_ptr[2*2] = <void*>_func_eval_sh_legendre[double_complex]
 * ufunc_eval_sh_legendre_ptr[2*2+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*3] = <void*>_func_eval_sh_legendre[double]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_ptr[2*3+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*4] = <void*>_func_eval_sh_legendre[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[6]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre);

  /* "scipy/special/_ufuncs.pyx":6032
 * ufunc_eval_sh_legendre_ptr[2*2+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*3] = <void*>_func_eval_sh_legendre[double]
 * ufunc_eval_sh_legendre_ptr[2*3+1] = <void*>(<char*>"eval_sh_legendre")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_ptr[2*4] = <void*>_func_eval_sh_legendre[double_complex]
 * ufunc_eval_sh_legendre_ptr[2*4+1] = <void*>(<char*>"eval_sh_legendre")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[7]) = ((void *)((char *)((char *)"eval_sh_legendre")));

  /* "scipy/special/_ufuncs.pyx":6033
 * ufunc_eval_sh_legendre_ptr[2*3] = <void*>_func_eval_sh_legendre[double]
 * ufunc_eval_sh_legendre_ptr[2*3+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*4] = <void*>_func_eval_sh_legendre[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_ptr[2*4+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_data[0] = &ufunc_eval_sh_legendre_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[8]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_15orthogonal_eval_eval_sh_legendre);

  /* "scipy/special/_ufuncs.pyx":6034
 * ufunc_eval_sh_legendre_ptr[2*3+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_ptr[2*4] = <void*>_func_eval_sh_legendre[double_complex]
 * ufunc_eval_sh_legendre_ptr[2*4+1] = <void*>(<char*>"eval_sh_legendre")             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_data[0] = &ufunc_eval_sh_legendre_ptr[2*0]
 * ufunc_eval_sh_legendre_data[1] = &ufunc_eval_sh_legendre_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[9]) = ((void *)((char *)((char *)"eval_sh_legendre")));

  /* "scipy/special/_ufuncs.pyx":6035
 * ufunc_eval_sh_legendre_ptr[2*4] = <void*>_func_eval_sh_legendre[double_complex]
 * ufunc_eval_sh_legendre_ptr[2*4+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_data[0] = &ufunc_eval_sh_legendre_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_data[1] = &ufunc_eval_sh_legendre_ptr[2*1]
 * ufunc_eval_sh_legendre_data[2] = &ufunc_eval_sh_legendre_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6036
 * ufunc_eval_sh_legendre_ptr[2*4+1] = <void*>(<char*>"eval_sh_legendre")
 * ufunc_eval_sh_legendre_data[0] = &ufunc_eval_sh_legendre_ptr[2*0]
 * ufunc_eval_sh_legendre_data[1] = &ufunc_eval_sh_legendre_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_data[2] = &ufunc_eval_sh_legendre_ptr[2*2]
 * ufunc_eval_sh_legendre_data[3] = &ufunc_eval_sh_legendre_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6037
 * ufunc_eval_sh_legendre_data[0] = &ufunc_eval_sh_legendre_ptr[2*0]
 * ufunc_eval_sh_legendre_data[1] = &ufunc_eval_sh_legendre_ptr[2*1]
 * ufunc_eval_sh_legendre_data[2] = &ufunc_eval_sh_legendre_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_data[3] = &ufunc_eval_sh_legendre_ptr[2*3]
 * ufunc_eval_sh_legendre_data[4] = &ufunc_eval_sh_legendre_ptr[2*4]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":6038
 * ufunc_eval_sh_legendre_data[1] = &ufunc_eval_sh_legendre_ptr[2*1]
 * ufunc_eval_sh_legendre_data[2] = &ufunc_eval_sh_legendre_ptr[2*2]
 * ufunc_eval_sh_legendre_data[3] = &ufunc_eval_sh_legendre_ptr[2*3]             # <<<<<<<<<<<<<<
 * ufunc_eval_sh_legendre_data[4] = &ufunc_eval_sh_legendre_ptr[2*4]
 * eval_sh_legendre = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_legendre_loops, ufunc_eval_sh_legendre_data, ufunc_eval_sh_legendre_types, 5, 2, 1, 0, "eval_sh_legendre", ufunc_eval_sh_legendre_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":6039
 * ufunc_eval_sh_legendre_data[2] = &ufunc_eval_sh_legendre_ptr[2*2]
 * ufunc_eval_sh_legendre_data[3] = &ufunc_eval_sh_legendre_ptr[2*3]
 * ufunc_eval_sh_legendre_data[4] = &ufunc_eval_sh_legendre_ptr[2*4]             # <<<<<<<<<<<<<<
 * eval_sh_legendre = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_legendre_loops, ufunc_eval_sh_legendre_data, ufunc_eval_sh_legendre_types, 5, 2, 1, 0, "eval_sh_legendre", ufunc_eval_sh_legendre_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_data[4]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_ptr[8]));

  /* "scipy/special/_ufuncs.pyx":6040
 * ufunc_eval_sh_legendre_data[3] = &ufunc_eval_sh_legendre_ptr[2*3]
 * ufunc_eval_sh_legendre_data[4] = &ufunc_eval_sh_legendre_ptr[2*4]
 * eval_sh_legendre = np.PyUFunc_FromFuncAndData(ufunc_eval_sh_legendre_loops, ufunc_eval_sh_legendre_data, ufunc_eval_sh_legendre_types, 5, 2, 1, 0, "eval_sh_legendre", ufunc_eval_sh_legendre_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_exp1_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_types, 5, 2, 1, 0, ((char *)"eval_sh_legendre"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_eval_sh_legendre_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6040, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_eval_sh_legendre, __pyx_t_2) < 0) __PYX_ERR(1, 6040, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":6046
 * cdef void *ufunc_exp1_data[4]
 * cdef char ufunc_exp1_types[8]
 * cdef char *ufunc_exp1_doc = (             # <<<<<<<<<<<<<<
 *     "exp1(z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_doc = ((char *)"exp1(z)\n\nExponential integral E_1 of complex argument z\n\n::\n\n    integral(exp(-z*t)/t, t=1..inf).");

  /* "scipy/special/_ufuncs.pyx":6054
 *     "\n"
 *     "    integral(exp(-z*t)/t, t=1..inf).")
 * ufunc_exp1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_exp1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exp1_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6055
 *     "    integral(exp(-z*t)/t, t=1..inf).")
 * ufunc_exp1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_exp1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_exp1_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_exp1_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6056
 * ufunc_exp1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_exp1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exp1_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_exp1_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_exp1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":6057
 * ufunc_exp1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exp1_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_exp1_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_exp1_types[0] = <char>NPY_FLOAT
 * ufunc_exp1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":6058
 * ufunc_exp1_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_exp1_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_exp1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_exp1_types[1] = <char>NPY_FLOAT
 * ufunc_exp1_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6059
 * ufunc_exp1_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_exp1_types[0] = <char>NPY_FLOAT
 * ufunc_exp1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_exp1_types[2] = <char>NPY_DOUBLE
 * ufunc_exp1_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6060
 * ufunc_exp1_types[0] = <char>NPY_FLOAT
 * ufunc_exp1_types[1] = <char>NPY_FLOAT
 * ufunc_exp1_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_exp1_types[3] = <char>NPY_DOUBLE
 * ufunc_exp1_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6061
 * ufunc_exp1_types[1] = <char>NPY_FLOAT
 * ufunc_exp1_types[2] = <char>NPY_DOUBLE
 * ufunc_exp1_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_exp1_types[4] = <char>NPY_CFLOAT
 * ufunc_exp1_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6062
 * ufunc_exp1_types[2] = <char>NPY_DOUBLE
 * ufunc_exp1_types[3] = <char>NPY_DOUBLE
 * ufunc_exp1_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_exp1_types[5] = <char>NPY_CFLOAT
 * ufunc_exp1_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6063
 * ufunc_exp1_types[3] = <char>NPY_DOUBLE
 * ufunc_exp1_types[4] = <char>NPY_CFLOAT
 * ufunc_exp1_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_exp1_types[6] = <char>NPY_CDOUBLE
 * ufunc_exp1_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6064
 * ufunc_exp1_types[4] = <char>NPY_CFLOAT
 * ufunc_exp1_types[5] = <char>NPY_CFLOAT
 * ufunc_exp1_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_exp1_types[7] = <char>NPY_CDOUBLE
 * ufunc_exp1_ptr[2*0] = <void*>_func_exp1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6065
 * ufunc_exp1_types[5] = <char>NPY_CFLOAT
 * ufunc_exp1_types[6] = <char>NPY_CDOUBLE
 * ufunc_exp1_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_exp1_ptr[2*0] = <void*>_func_exp1_wrap
 * ufunc_exp1_ptr[2*0+1] = <void*>(<char*>"exp1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6066
 * ufunc_exp1_types[6] = <char>NPY_CDOUBLE
 * ufunc_exp1_types[7] = <char>NPY_CDOUBLE
 * ufunc_exp1_ptr[2*0] = <void*>_func_exp1_wrap             # <<<<<<<<<<<<<<
 * ufunc_exp1_ptr[2*0+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_ptr[2*1] = <void*>_func_exp1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[0]) = ((void *)exp1_wrap);

  /* "scipy/special/_ufuncs.pyx":6067
 * ufunc_exp1_types[7] = <char>NPY_CDOUBLE
 * ufunc_exp1_ptr[2*0] = <void*>_func_exp1_wrap
 * ufunc_exp1_ptr[2*0+1] = <void*>(<char*>"exp1")             # <<<<<<<<<<<<<<
 * ufunc_exp1_ptr[2*1] = <void*>_func_exp1_wrap
 * ufunc_exp1_ptr[2*1+1] = <void*>(<char*>"exp1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[1]) = ((void *)((char *)((char *)"exp1")));

  /* "scipy/special/_ufuncs.pyx":6068
 * ufunc_exp1_ptr[2*0] = <void*>_func_exp1_wrap
 * ufunc_exp1_ptr[2*0+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_ptr[2*1] = <void*>_func_exp1_wrap             # <<<<<<<<<<<<<<
 * ufunc_exp1_ptr[2*1+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_ptr[2*2] = <void*>_func_cexp1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[2]) = ((void *)exp1_wrap);

  /* "scipy/special/_ufuncs.pyx":6069
 * ufunc_exp1_ptr[2*0+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_ptr[2*1] = <void*>_func_exp1_wrap
 * ufunc_exp1_ptr[2*1+1] = <void*>(<char*>"exp1")             # <<<<<<<<<<<<<<
 * ufunc_exp1_ptr[2*2] = <void*>_func_cexp1_wrap
 * ufunc_exp1_ptr[2*2+1] = <void*>(<char*>"exp1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[3]) = ((void *)((char *)((char *)"exp1")));

  /* "scipy/special/_ufuncs.pyx":6070
 * ufunc_exp1_ptr[2*1] = <void*>_func_exp1_wrap
 * ufunc_exp1_ptr[2*1+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_ptr[2*2] = <void*>_func_cexp1_wrap             # <<<<<<<<<<<<<<
 * ufunc_exp1_ptr[2*2+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_ptr[2*3] = <void*>_func_cexp1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[4]) = ((void *)cexp1_wrap);

  /* "scipy/special/_ufuncs.pyx":6071
 * ufunc_exp1_ptr[2*1+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_ptr[2*2] = <void*>_func_cexp1_wrap
 * ufunc_exp1_ptr[2*2+1] = <void*>(<char*>"exp1")             # <<<<<<<<<<<<<<
 * ufunc_exp1_ptr[2*3] = <void*>_func_cexp1_wrap
 * ufunc_exp1_ptr[2*3+1] = <void*>(<char*>"exp1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[5]) = ((void *)((char *)((char *)"exp1")));

  /* "scipy/special/_ufuncs.pyx":6072
 * ufunc_exp1_ptr[2*2] = <void*>_func_cexp1_wrap
 * ufunc_exp1_ptr[2*2+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_ptr[2*3] = <void*>_func_cexp1_wrap             # <<<<<<<<<<<<<<
 * ufunc_exp1_ptr[2*3+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_data[0] = &ufunc_exp1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[6]) = ((void *)cexp1_wrap);

  /* "scipy/special/_ufuncs.pyx":6073
 * ufunc_exp1_ptr[2*2+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_ptr[2*3] = <void*>_func_cexp1_wrap
 * ufunc_exp1_ptr[2*3+1] = <void*>(<char*>"exp1")             # <<<<<<<<<<<<<<
 * ufunc_exp1_data[0] = &ufunc_exp1_ptr[2*0]
 * ufunc_exp1_data[1] = &ufunc_exp1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[7]) = ((void *)((char *)((char *)"exp1")));

  /* "scipy/special/_ufuncs.pyx":6074
 * ufunc_exp1_ptr[2*3] = <void*>_func_cexp1_wrap
 * ufunc_exp1_ptr[2*3+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_data[0] = &ufunc_exp1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_exp1_data[1] = &ufunc_exp1_ptr[2*1]
 * ufunc_exp1_data[2] = &ufunc_exp1_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6075
 * ufunc_exp1_ptr[2*3+1] = <void*>(<char*>"exp1")
 * ufunc_exp1_data[0] = &ufunc_exp1_ptr[2*0]
 * ufunc_exp1_data[1] = &ufunc_exp1_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_exp1_data[2] = &ufunc_exp1_ptr[2*2]
 * ufunc_exp1_data[3] = &ufunc_exp1_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6076
 * ufunc_exp1_data[0] = &ufunc_exp1_ptr[2*0]
 * ufunc_exp1_data[1] = &ufunc_exp1_ptr[2*1]
 * ufunc_exp1_data[2] = &ufunc_exp1_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_exp1_data[3] = &ufunc_exp1_ptr[2*3]
 * exp1 = np.PyUFunc_FromFuncAndData(ufunc_exp1_loops, ufunc_exp1_data, ufunc_exp1_types, 4, 1, 1, 0, "exp1", ufunc_exp1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":6077
 * ufunc_exp1_data[1] = &ufunc_exp1_ptr[2*1]
 * ufunc_exp1_data[2] = &ufunc_exp1_ptr[2*2]
 * ufunc_exp1_data[3] = &ufunc_exp1_ptr[2*3]             # <<<<<<<<<<<<<<
 * exp1 = np.PyUFunc_FromFuncAndData(ufunc_exp1_loops, ufunc_exp1_data, ufunc_exp1_types, 4, 1, 1, 0, "exp1", ufunc_exp1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":6078
 * ufunc_exp1_data[2] = &ufunc_exp1_ptr[2*2]
 * ufunc_exp1_data[3] = &ufunc_exp1_ptr[2*3]
 * exp1 = np.PyUFunc_FromFuncAndData(ufunc_exp1_loops, ufunc_exp1_data, ufunc_exp1_types, 4, 1, 1, 0, "exp1", ufunc_exp1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_exp10_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_types, 4, 1, 1, 0, ((char *)"exp1"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp1_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6078, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_exp1, __pyx_t_2) < 0) __PYX_ERR(1, 6078, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":6084
 * cdef void *ufunc_exp10_data[2]
 * cdef char ufunc_exp10_types[4]
 * cdef char *ufunc_exp10_doc = (             # <<<<<<<<<<<<<<
 *     "exp10(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_doc = ((char *)"exp10(x)\n\nCompute ``10**x`` element-wise.\n\nParameters\n----------\nx : array_like\n    `x` must contain real numbers.\n\nReturns\n-------\nfloat\n    ``10**x``, computed element-wise.\n\nExamples\n--------\n>>> from scipy.special import exp10\n\n>>> exp10(3)\n1000.0\n>>> x = np.array([[-1, -0.5, 0], [0.5, 1, 1.5]])\n>>> exp10(x)\narray([[  0.1       ,   0.31622777,   1.        ],\n       [  3.16227766,  10.        ,  31.6227766 ]])");

  /* "scipy/special/_ufuncs.pyx":6109
 *     "array([[  0.1       ,   0.31622777,   1.        ],\n"
 *     "       [  3.16227766,  10.        ,  31.6227766 ]])")
 * ufunc_exp10_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_exp10_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exp10_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6110
 *     "       [  3.16227766,  10.        ,  31.6227766 ]])")
 * ufunc_exp10_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_exp10_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_exp10_types[0] = <char>NPY_FLOAT
 * ufunc_exp10_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6111
 * ufunc_exp10_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_exp10_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exp10_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_exp10_types[1] = <char>NPY_FLOAT
 * ufunc_exp10_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6112
 * ufunc_exp10_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exp10_types[0] = <char>NPY_FLOAT
 * ufunc_exp10_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_exp10_types[2] = <char>NPY_DOUBLE
 * ufunc_exp10_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6113
 * ufunc_exp10_types[0] = <char>NPY_FLOAT
 * ufunc_exp10_types[1] = <char>NPY_FLOAT
 * ufunc_exp10_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_exp10_types[3] = <char>NPY_DOUBLE
 * ufunc_exp10_ptr[2*0] = <void*>_func_exp10
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6114
 * ufunc_exp10_types[1] = <char>NPY_FLOAT
 * ufunc_exp10_types[2] = <char>NPY_DOUBLE
 * ufunc_exp10_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_exp10_ptr[2*0] = <void*>_func_exp10
 * ufunc_exp10_ptr[2*0+1] = <void*>(<char*>"exp10")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6115
 * ufunc_exp10_types[2] = <char>NPY_DOUBLE
 * ufunc_exp10_types[3] = <char>NPY_DOUBLE
 * ufunc_exp10_ptr[2*0] = <void*>_func_exp10             # <<<<<<<<<<<<<<
 * ufunc_exp10_ptr[2*0+1] = <void*>(<char*>"exp10")
 * ufunc_exp10_ptr[2*1] = <void*>_func_exp10
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_ptr[0]) = ((void *)exp10);

  /* "scipy/special/_ufuncs.pyx":6116
 * ufunc_exp10_types[3] = <char>NPY_DOUBLE
 * ufunc_exp10_ptr[2*0] = <void*>_func_exp10
 * ufunc_exp10_ptr[2*0+1] = <void*>(<char*>"exp10")             # <<<<<<<<<<<<<<
 * ufunc_exp10_ptr[2*1] = <void*>_func_exp10
 * ufunc_exp10_ptr[2*1+1] = <void*>(<char*>"exp10")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_ptr[1]) = ((void *)((char *)((char *)"exp10")));

  /* "scipy/special/_ufuncs.pyx":6117
 * ufunc_exp10_ptr[2*0] = <void*>_func_exp10
 * ufunc_exp10_ptr[2*0+1] = <void*>(<char*>"exp10")
 * ufunc_exp10_ptr[2*1] = <void*>_func_exp10             # <<<<<<<<<<<<<<
 * ufunc_exp10_ptr[2*1+1] = <void*>(<char*>"exp10")
 * ufunc_exp10_data[0] = &ufunc_exp10_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_ptr[2]) = ((void *)exp10);

  /* "scipy/special/_ufuncs.pyx":6118
 * ufunc_exp10_ptr[2*0+1] = <void*>(<char*>"exp10")
 * ufunc_exp10_ptr[2*1] = <void*>_func_exp10
 * ufunc_exp10_ptr[2*1+1] = <void*>(<char*>"exp10")             # <<<<<<<<<<<<<<
 * ufunc_exp10_data[0] = &ufunc_exp10_ptr[2*0]
 * ufunc_exp10_data[1] = &ufunc_exp10_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_ptr[3]) = ((void *)((char *)((char *)"exp10")));

  /* "scipy/special/_ufuncs.pyx":6119
 * ufunc_exp10_ptr[2*1] = <void*>_func_exp10
 * ufunc_exp10_ptr[2*1+1] = <void*>(<char*>"exp10")
 * ufunc_exp10_data[0] = &ufunc_exp10_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_exp10_data[1] = &ufunc_exp10_ptr[2*1]
 * exp10 = np.PyUFunc_FromFuncAndData(ufunc_exp10_loops, ufunc_exp10_data, ufunc_exp10_types, 2, 1, 1, 0, "exp10", ufunc_exp10_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6120
 * ufunc_exp10_ptr[2*1+1] = <void*>(<char*>"exp10")
 * ufunc_exp10_data[0] = &ufunc_exp10_ptr[2*0]
 * ufunc_exp10_data[1] = &ufunc_exp10_ptr[2*1]             # <<<<<<<<<<<<<<
 * exp10 = np.PyUFunc_FromFuncAndData(ufunc_exp10_loops, ufunc_exp10_data, ufunc_exp10_types, 2, 1, 1, 0, "exp10", ufunc_exp10_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6121
 * ufunc_exp10_data[0] = &ufunc_exp10_ptr[2*0]
 * ufunc_exp10_data[1] = &ufunc_exp10_ptr[2*1]
 * exp10 = np.PyUFunc_FromFuncAndData(ufunc_exp10_loops, ufunc_exp10_data, ufunc_exp10_types, 2, 1, 1, 0, "exp10", ufunc_exp10_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_exp2_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_types, 2, 1, 1, 0, ((char *)"exp10"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp10_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_exp10, __pyx_t_2) < 0) __PYX_ERR(1, 6121, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":6127
 * cdef void *ufunc_exp2_data[2]
 * cdef char ufunc_exp2_types[4]
 * cdef char *ufunc_exp2_doc = (             # <<<<<<<<<<<<<<
 *     "exp2(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_doc = ((char *)"exp2(x)\n\nCompute ``2**x`` element-wise.\n\nParameters\n----------\nx : array_like\n    `x` must contain real numbers.\n\nReturns\n-------\nfloat\n    ``2**x``, computed element-wise.\n\nExamples\n--------\n>>> from scipy.special import exp2\n\n>>> exp2(3)\n8.0\n>>> x = np.array([[-1, -0.5, 0], [0.5, 1, 1.5]])\n>>> exp2(x)\narray([[ 0.5       ,  0.70710678,  1.        ],\n       [ 1.41421356,  2.        ,  2.82842712]])");

  /* "scipy/special/_ufuncs.pyx":6152
 *     "array([[ 0.5       ,  0.70710678,  1.        ],\n"
 *     "       [ 1.41421356,  2.        ,  2.82842712]])")
 * ufunc_exp2_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_exp2_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exp2_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6153
 *     "       [ 1.41421356,  2.        ,  2.82842712]])")
 * ufunc_exp2_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_exp2_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_exp2_types[0] = <char>NPY_FLOAT
 * ufunc_exp2_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6154
 * ufunc_exp2_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_exp2_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exp2_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_exp2_types[1] = <char>NPY_FLOAT
 * ufunc_exp2_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6155
 * ufunc_exp2_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exp2_types[0] = <char>NPY_FLOAT
 * ufunc_exp2_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_exp2_types[2] = <char>NPY_DOUBLE
 * ufunc_exp2_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6156
 * ufunc_exp2_types[0] = <char>NPY_FLOAT
 * ufunc_exp2_types[1] = <char>NPY_FLOAT
 * ufunc_exp2_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_exp2_types[3] = <char>NPY_DOUBLE
 * ufunc_exp2_ptr[2*0] = <void*>_func_exp2
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6157
 * ufunc_exp2_types[1] = <char>NPY_FLOAT
 * ufunc_exp2_types[2] = <char>NPY_DOUBLE
 * ufunc_exp2_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_exp2_ptr[2*0] = <void*>_func_exp2
 * ufunc_exp2_ptr[2*0+1] = <void*>(<char*>"exp2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6158
 * ufunc_exp2_types[2] = <char>NPY_DOUBLE
 * ufunc_exp2_types[3] = <char>NPY_DOUBLE
 * ufunc_exp2_ptr[2*0] = <void*>_func_exp2             # <<<<<<<<<<<<<<
 * ufunc_exp2_ptr[2*0+1] = <void*>(<char*>"exp2")
 * ufunc_exp2_ptr[2*1] = <void*>_func_exp2
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_ptr[0]) = ((void *)exp2);

  /* "scipy/special/_ufuncs.pyx":6159
 * ufunc_exp2_types[3] = <char>NPY_DOUBLE
 * ufunc_exp2_ptr[2*0] = <void*>_func_exp2
 * ufunc_exp2_ptr[2*0+1] = <void*>(<char*>"exp2")             # <<<<<<<<<<<<<<
 * ufunc_exp2_ptr[2*1] = <void*>_func_exp2
 * ufunc_exp2_ptr[2*1+1] = <void*>(<char*>"exp2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_ptr[1]) = ((void *)((char *)((char *)"exp2")));

  /* "scipy/special/_ufuncs.pyx":6160
 * ufunc_exp2_ptr[2*0] = <void*>_func_exp2
 * ufunc_exp2_ptr[2*0+1] = <void*>(<char*>"exp2")
 * ufunc_exp2_ptr[2*1] = <void*>_func_exp2             # <<<<<<<<<<<<<<
 * ufunc_exp2_ptr[2*1+1] = <void*>(<char*>"exp2")
 * ufunc_exp2_data[0] = &ufunc_exp2_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_ptr[2]) = ((void *)exp2);

  /* "scipy/special/_ufuncs.pyx":6161
 * ufunc_exp2_ptr[2*0+1] = <void*>(<char*>"exp2")
 * ufunc_exp2_ptr[2*1] = <void*>_func_exp2
 * ufunc_exp2_ptr[2*1+1] = <void*>(<char*>"exp2")             # <<<<<<<<<<<<<<
 * ufunc_exp2_data[0] = &ufunc_exp2_ptr[2*0]
 * ufunc_exp2_data[1] = &ufunc_exp2_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_ptr[3]) = ((void *)((char *)((char *)"exp2")));

  /* "scipy/special/_ufuncs.pyx":6162
 * ufunc_exp2_ptr[2*1] = <void*>_func_exp2
 * ufunc_exp2_ptr[2*1+1] = <void*>(<char*>"exp2")
 * ufunc_exp2_data[0] = &ufunc_exp2_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_exp2_data[1] = &ufunc_exp2_ptr[2*1]
 * exp2 = np.PyUFunc_FromFuncAndData(ufunc_exp2_loops, ufunc_exp2_data, ufunc_exp2_types, 2, 1, 1, 0, "exp2", ufunc_exp2_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6163
 * ufunc_exp2_ptr[2*1+1] = <void*>(<char*>"exp2")
 * ufunc_exp2_data[0] = &ufunc_exp2_ptr[2*0]
 * ufunc_exp2_data[1] = &ufunc_exp2_ptr[2*1]             # <<<<<<<<<<<<<<
 * exp2 = np.PyUFunc_FromFuncAndData(ufunc_exp2_loops, ufunc_exp2_data, ufunc_exp2_types, 2, 1, 1, 0, "exp2", ufunc_exp2_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6164
 * ufunc_exp2_data[0] = &ufunc_exp2_ptr[2*0]
 * ufunc_exp2_data[1] = &ufunc_exp2_ptr[2*1]
 * exp2 = np.PyUFunc_FromFuncAndData(ufunc_exp2_loops, ufunc_exp2_data, ufunc_exp2_types, 2, 1, 1, 0, "exp2", ufunc_exp2_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_expi_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_types, 2, 1, 1, 0, ((char *)"exp2"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_exp2_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_exp2, __pyx_t_2) < 0) __PYX_ERR(1, 6164, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":6170
 * cdef void *ufunc_expi_data[4]
 * cdef char ufunc_expi_types[8]
 * cdef char *ufunc_expi_doc = (             # <<<<<<<<<<<<<<
 *     "expi(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_doc = ((char *)"expi(x)\n\nExponential integral Ei\n\nDefined as::\n\n    integral(exp(t)/t, t=-inf..x)\n\nSee `expn` for a different exponential integral.");

  /* "scipy/special/_ufuncs.pyx":6180
 *     "\n"
 *     "See `expn` for a different exponential integral.")
 * ufunc_expi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_expi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_expi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6181
 *     "See `expn` for a different exponential integral.")
 * ufunc_expi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_expi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_expi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_expi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6182
 * ufunc_expi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_expi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_expi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_expi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_expi_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":6183
 * ufunc_expi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_expi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_expi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_expi_types[0] = <char>NPY_FLOAT
 * ufunc_expi_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":6184
 * ufunc_expi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_expi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_expi_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_expi_types[1] = <char>NPY_FLOAT
 * ufunc_expi_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6185
 * ufunc_expi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_expi_types[0] = <char>NPY_FLOAT
 * ufunc_expi_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_expi_types[2] = <char>NPY_DOUBLE
 * ufunc_expi_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6186
 * ufunc_expi_types[0] = <char>NPY_FLOAT
 * ufunc_expi_types[1] = <char>NPY_FLOAT
 * ufunc_expi_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expi_types[3] = <char>NPY_DOUBLE
 * ufunc_expi_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6187
 * ufunc_expi_types[1] = <char>NPY_FLOAT
 * ufunc_expi_types[2] = <char>NPY_DOUBLE
 * ufunc_expi_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expi_types[4] = <char>NPY_CFLOAT
 * ufunc_expi_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6188
 * ufunc_expi_types[2] = <char>NPY_DOUBLE
 * ufunc_expi_types[3] = <char>NPY_DOUBLE
 * ufunc_expi_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_expi_types[5] = <char>NPY_CFLOAT
 * ufunc_expi_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6189
 * ufunc_expi_types[3] = <char>NPY_DOUBLE
 * ufunc_expi_types[4] = <char>NPY_CFLOAT
 * ufunc_expi_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_expi_types[6] = <char>NPY_CDOUBLE
 * ufunc_expi_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6190
 * ufunc_expi_types[4] = <char>NPY_CFLOAT
 * ufunc_expi_types[5] = <char>NPY_CFLOAT
 * ufunc_expi_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expi_types[7] = <char>NPY_CDOUBLE
 * ufunc_expi_ptr[2*0] = <void*>_func_expi_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6191
 * ufunc_expi_types[5] = <char>NPY_CFLOAT
 * ufunc_expi_types[6] = <char>NPY_CDOUBLE
 * ufunc_expi_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expi_ptr[2*0] = <void*>_func_expi_wrap
 * ufunc_expi_ptr[2*0+1] = <void*>(<char*>"expi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6192
 * ufunc_expi_types[6] = <char>NPY_CDOUBLE
 * ufunc_expi_types[7] = <char>NPY_CDOUBLE
 * ufunc_expi_ptr[2*0] = <void*>_func_expi_wrap             # <<<<<<<<<<<<<<
 * ufunc_expi_ptr[2*0+1] = <void*>(<char*>"expi")
 * ufunc_expi_ptr[2*1] = <void*>_func_expi_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[0]) = ((void *)expi_wrap);

  /* "scipy/special/_ufuncs.pyx":6193
 * ufunc_expi_types[7] = <char>NPY_CDOUBLE
 * ufunc_expi_ptr[2*0] = <void*>_func_expi_wrap
 * ufunc_expi_ptr[2*0+1] = <void*>(<char*>"expi")             # <<<<<<<<<<<<<<
 * ufunc_expi_ptr[2*1] = <void*>_func_expi_wrap
 * ufunc_expi_ptr[2*1+1] = <void*>(<char*>"expi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[1]) = ((void *)((char *)((char *)"expi")));

  /* "scipy/special/_ufuncs.pyx":6194
 * ufunc_expi_ptr[2*0] = <void*>_func_expi_wrap
 * ufunc_expi_ptr[2*0+1] = <void*>(<char*>"expi")
 * ufunc_expi_ptr[2*1] = <void*>_func_expi_wrap             # <<<<<<<<<<<<<<
 * ufunc_expi_ptr[2*1+1] = <void*>(<char*>"expi")
 * ufunc_expi_ptr[2*2] = <void*>_func_cexpi_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[2]) = ((void *)expi_wrap);

  /* "scipy/special/_ufuncs.pyx":6195
 * ufunc_expi_ptr[2*0+1] = <void*>(<char*>"expi")
 * ufunc_expi_ptr[2*1] = <void*>_func_expi_wrap
 * ufunc_expi_ptr[2*1+1] = <void*>(<char*>"expi")             # <<<<<<<<<<<<<<
 * ufunc_expi_ptr[2*2] = <void*>_func_cexpi_wrap
 * ufunc_expi_ptr[2*2+1] = <void*>(<char*>"expi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[3]) = ((void *)((char *)((char *)"expi")));

  /* "scipy/special/_ufuncs.pyx":6196
 * ufunc_expi_ptr[2*1] = <void*>_func_expi_wrap
 * ufunc_expi_ptr[2*1+1] = <void*>(<char*>"expi")
 * ufunc_expi_ptr[2*2] = <void*>_func_cexpi_wrap             # <<<<<<<<<<<<<<
 * ufunc_expi_ptr[2*2+1] = <void*>(<char*>"expi")
 * ufunc_expi_ptr[2*3] = <void*>_func_cexpi_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[4]) = ((void *)cexpi_wrap);

  /* "scipy/special/_ufuncs.pyx":6197
 * ufunc_expi_ptr[2*1+1] = <void*>(<char*>"expi")
 * ufunc_expi_ptr[2*2] = <void*>_func_cexpi_wrap
 * ufunc_expi_ptr[2*2+1] = <void*>(<char*>"expi")             # <<<<<<<<<<<<<<
 * ufunc_expi_ptr[2*3] = <void*>_func_cexpi_wrap
 * ufunc_expi_ptr[2*3+1] = <void*>(<char*>"expi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[5]) = ((void *)((char *)((char *)"expi")));

  /* "scipy/special/_ufuncs.pyx":6198
 * ufunc_expi_ptr[2*2] = <void*>_func_cexpi_wrap
 * ufunc_expi_ptr[2*2+1] = <void*>(<char*>"expi")
 * ufunc_expi_ptr[2*3] = <void*>_func_cexpi_wrap             # <<<<<<<<<<<<<<
 * ufunc_expi_ptr[2*3+1] = <void*>(<char*>"expi")
 * ufunc_expi_data[0] = &ufunc_expi_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[6]) = ((void *)cexpi_wrap);

  /* "scipy/special/_ufuncs.pyx":6199
 * ufunc_expi_ptr[2*2+1] = <void*>(<char*>"expi")
 * ufunc_expi_ptr[2*3] = <void*>_func_cexpi_wrap
 * ufunc_expi_ptr[2*3+1] = <void*>(<char*>"expi")             # <<<<<<<<<<<<<<
 * ufunc_expi_data[0] = &ufunc_expi_ptr[2*0]
 * ufunc_expi_data[1] = &ufunc_expi_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[7]) = ((void *)((char *)((char *)"expi")));

  /* "scipy/special/_ufuncs.pyx":6200
 * ufunc_expi_ptr[2*3] = <void*>_func_cexpi_wrap
 * ufunc_expi_ptr[2*3+1] = <void*>(<char*>"expi")
 * ufunc_expi_data[0] = &ufunc_expi_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_expi_data[1] = &ufunc_expi_ptr[2*1]
 * ufunc_expi_data[2] = &ufunc_expi_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6201
 * ufunc_expi_ptr[2*3+1] = <void*>(<char*>"expi")
 * ufunc_expi_data[0] = &ufunc_expi_ptr[2*0]
 * ufunc_expi_data[1] = &ufunc_expi_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_expi_data[2] = &ufunc_expi_ptr[2*2]
 * ufunc_expi_data[3] = &ufunc_expi_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6202
 * ufunc_expi_data[0] = &ufunc_expi_ptr[2*0]
 * ufunc_expi_data[1] = &ufunc_expi_ptr[2*1]
 * ufunc_expi_data[2] = &ufunc_expi_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_expi_data[3] = &ufunc_expi_ptr[2*3]
 * expi = np.PyUFunc_FromFuncAndData(ufunc_expi_loops, ufunc_expi_data, ufunc_expi_types, 4, 1, 1, 0, "expi", ufunc_expi_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":6203
 * ufunc_expi_data[1] = &ufunc_expi_ptr[2*1]
 * ufunc_expi_data[2] = &ufunc_expi_ptr[2*2]
 * ufunc_expi_data[3] = &ufunc_expi_ptr[2*3]             # <<<<<<<<<<<<<<
 * expi = np.PyUFunc_FromFuncAndData(ufunc_expi_loops, ufunc_expi_data, ufunc_expi_types, 4, 1, 1, 0, "expi", ufunc_expi_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":6204
 * ufunc_expi_data[2] = &ufunc_expi_ptr[2*2]
 * ufunc_expi_data[3] = &ufunc_expi_ptr[2*3]
 * expi = np.PyUFunc_FromFuncAndData(ufunc_expi_loops, ufunc_expi_data, ufunc_expi_types, 4, 1, 1, 0, "expi", ufunc_expi_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_expit_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_types, 4, 1, 1, 0, ((char *)"expi"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_expi_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_expi, __pyx_t_2) < 0) __PYX_ERR(1, 6204, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":6210
 * cdef void *ufunc_expit_data[3]
 * cdef char ufunc_expit_types[6]
 * cdef char *ufunc_expit_doc = (             # <<<<<<<<<<<<<<
 *     "expit(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_doc = ((char *)"expit(x)\n\nExpit (a.k.a. logistic sigmoid) ufunc for ndarrays.\n\nThe expit function, also known as the logistic sigmoid function, is\ndefined as ``expit(x) = 1/(1+exp(-x))``.  It is the inverse of the\nlogit function.\n\nParameters\n----------\nx : ndarray\n    The ndarray to apply expit to element-wise.\n\nReturns\n-------\nout : ndarray\n    An ndarray of the same shape as x. Its entries\n    are `expit` of the corresponding entry of x.\n\nSee Also\n--------\nlogit\n\nNotes\n-----\nAs a ufunc expit takes a number of optional\nkeyword arguments. For more information\nsee `ufuncs <https://docs.scipy.org/doc/numpy/reference/ufuncs.html>`_\n\n.. versionadded:: 0.10.0\n\nExamples\n--------\n>>> from scipy.special import expit, logit\n\n>>> expit([-np.inf, -1.5, 0, 1.5, np.inf])\narray([ 0.        ,  0.18242552,  0.5       ,  0.81757448,  1.        ])\n\n`logit` is the inverse of `expit`:\n\n>>> logit(expit([-2.5, 0, 3.1, 5.0]))\narray([-2.5,  0. ,  3.1,  5. ])\n\nPlot expit(x) for x in [-6, 6]:\n\n>>> import matplotlib.pyplot as plt\n>>> x = np.linspace(-6, 6, 121)\n>>> y = expit(x)\n>>> plt.plot(x, y)\n>>> plt.grid()\n>>> plt.xlim(-6, 6)\n>>> plt.xlabel('x')\n>>> plt.title('expit(x)')\n>>> plt.show()");

  /* "scipy/special/_ufuncs.pyx":6265
 *     ">>> plt.title('expit(x)')\n"
 *     ">>> plt.show()")
 * ufunc_expit_loops[0] = <np.PyUFuncGenericFunction>loop_f_f__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_expit_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_expit_loops[2] = <np.PyUFuncGenericFunction>loop_g_g__As_g_g
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_f_f__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6266
 *     ">>> plt.show()")
 * ufunc_expit_loops[0] = <np.PyUFuncGenericFunction>loop_f_f__As_f_f
 * ufunc_expit_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_expit_loops[2] = <np.PyUFuncGenericFunction>loop_g_g__As_g_g
 * ufunc_expit_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6267
 * ufunc_expit_loops[0] = <np.PyUFuncGenericFunction>loop_f_f__As_f_f
 * ufunc_expit_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_expit_loops[2] = <np.PyUFuncGenericFunction>loop_g_g__As_g_g             # <<<<<<<<<<<<<<
 * ufunc_expit_types[0] = <char>NPY_FLOAT
 * ufunc_expit_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_g_g__As_g_g);

  /* "scipy/special/_ufuncs.pyx":6268
 * ufunc_expit_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_expit_loops[2] = <np.PyUFuncGenericFunction>loop_g_g__As_g_g
 * ufunc_expit_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_expit_types[1] = <char>NPY_FLOAT
 * ufunc_expit_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6269
 * ufunc_expit_loops[2] = <np.PyUFuncGenericFunction>loop_g_g__As_g_g
 * ufunc_expit_types[0] = <char>NPY_FLOAT
 * ufunc_expit_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_expit_types[2] = <char>NPY_DOUBLE
 * ufunc_expit_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6270
 * ufunc_expit_types[0] = <char>NPY_FLOAT
 * ufunc_expit_types[1] = <char>NPY_FLOAT
 * ufunc_expit_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expit_types[3] = <char>NPY_DOUBLE
 * ufunc_expit_types[4] = <char>NPY_LONGDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6271
 * ufunc_expit_types[1] = <char>NPY_FLOAT
 * ufunc_expit_types[2] = <char>NPY_DOUBLE
 * ufunc_expit_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expit_types[4] = <char>NPY_LONGDOUBLE
 * ufunc_expit_types[5] = <char>NPY_LONGDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6272
 * ufunc_expit_types[2] = <char>NPY_DOUBLE
 * ufunc_expit_types[3] = <char>NPY_DOUBLE
 * ufunc_expit_types[4] = <char>NPY_LONGDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expit_types[5] = <char>NPY_LONGDOUBLE
 * ufunc_expit_ptr[2*0] = <void*>_func_expitf
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_types[4]) = ((char)NPY_LONGDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6273
 * ufunc_expit_types[3] = <char>NPY_DOUBLE
 * ufunc_expit_types[4] = <char>NPY_LONGDOUBLE
 * ufunc_expit_types[5] = <char>NPY_LONGDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expit_ptr[2*0] = <void*>_func_expitf
 * ufunc_expit_ptr[2*0+1] = <void*>(<char*>"expit")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_types[5]) = ((char)NPY_LONGDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6274
 * ufunc_expit_types[4] = <char>NPY_LONGDOUBLE
 * ufunc_expit_types[5] = <char>NPY_LONGDOUBLE
 * ufunc_expit_ptr[2*0] = <void*>_func_expitf             # <<<<<<<<<<<<<<
 * ufunc_expit_ptr[2*0+1] = <void*>(<char*>"expit")
 * ufunc_expit_ptr[2*1] = <void*>_func_expit
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_ptr[0]) = ((void *)expitf);

  /* "scipy/special/_ufuncs.pyx":6275
 * ufunc_expit_types[5] = <char>NPY_LONGDOUBLE
 * ufunc_expit_ptr[2*0] = <void*>_func_expitf
 * ufunc_expit_ptr[2*0+1] = <void*>(<char*>"expit")             # <<<<<<<<<<<<<<
 * ufunc_expit_ptr[2*1] = <void*>_func_expit
 * ufunc_expit_ptr[2*1+1] = <void*>(<char*>"expit")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_ptr[1]) = ((void *)((char *)((char *)"expit")));

  /* "scipy/special/_ufuncs.pyx":6276
 * ufunc_expit_ptr[2*0] = <void*>_func_expitf
 * ufunc_expit_ptr[2*0+1] = <void*>(<char*>"expit")
 * ufunc_expit_ptr[2*1] = <void*>_func_expit             # <<<<<<<<<<<<<<
 * ufunc_expit_ptr[2*1+1] = <void*>(<char*>"expit")
 * ufunc_expit_ptr[2*2] = <void*>_func_expitl
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_ptr[2]) = ((void *)expit);

  /* "scipy/special/_ufuncs.pyx":6277
 * ufunc_expit_ptr[2*0+1] = <void*>(<char*>"expit")
 * ufunc_expit_ptr[2*1] = <void*>_func_expit
 * ufunc_expit_ptr[2*1+1] = <void*>(<char*>"expit")             # <<<<<<<<<<<<<<
 * ufunc_expit_ptr[2*2] = <void*>_func_expitl
 * ufunc_expit_ptr[2*2+1] = <void*>(<char*>"expit")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_ptr[3]) = ((void *)((char *)((char *)"expit")));

  /* "scipy/special/_ufuncs.pyx":6278
 * ufunc_expit_ptr[2*1] = <void*>_func_expit
 * ufunc_expit_ptr[2*1+1] = <void*>(<char*>"expit")
 * ufunc_expit_ptr[2*2] = <void*>_func_expitl             # <<<<<<<<<<<<<<
 * ufunc_expit_ptr[2*2+1] = <void*>(<char*>"expit")
 * ufunc_expit_data[0] = &ufunc_expit_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_ptr[4]) = ((void *)expitl);

  /* "scipy/special/_ufuncs.pyx":6279
 * ufunc_expit_ptr[2*1+1] = <void*>(<char*>"expit")
 * ufunc_expit_ptr[2*2] = <void*>_func_expitl
 * ufunc_expit_ptr[2*2+1] = <void*>(<char*>"expit")             # <<<<<<<<<<<<<<
 * ufunc_expit_data[0] = &ufunc_expit_ptr[2*0]
 * ufunc_expit_data[1] = &ufunc_expit_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_ptr[5]) = ((void *)((char *)((char *)"expit")));

  /* "scipy/special/_ufuncs.pyx":6280
 * ufunc_expit_ptr[2*2] = <void*>_func_expitl
 * ufunc_expit_ptr[2*2+1] = <void*>(<char*>"expit")
 * ufunc_expit_data[0] = &ufunc_expit_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_expit_data[1] = &ufunc_expit_ptr[2*1]
 * ufunc_expit_data[2] = &ufunc_expit_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6281
 * ufunc_expit_ptr[2*2+1] = <void*>(<char*>"expit")
 * ufunc_expit_data[0] = &ufunc_expit_ptr[2*0]
 * ufunc_expit_data[1] = &ufunc_expit_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_expit_data[2] = &ufunc_expit_ptr[2*2]
 * expit = np.PyUFunc_FromFuncAndData(ufunc_expit_loops, ufunc_expit_data, ufunc_expit_types, 3, 1, 1, 0, "expit", ufunc_expit_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6282
 * ufunc_expit_data[0] = &ufunc_expit_ptr[2*0]
 * ufunc_expit_data[1] = &ufunc_expit_ptr[2*1]
 * ufunc_expit_data[2] = &ufunc_expit_ptr[2*2]             # <<<<<<<<<<<<<<
 * expit = np.PyUFunc_FromFuncAndData(ufunc_expit_loops, ufunc_expit_data, ufunc_expit_types, 3, 1, 1, 0, "expit", ufunc_expit_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":6283
 * ufunc_expit_data[1] = &ufunc_expit_ptr[2*1]
 * ufunc_expit_data[2] = &ufunc_expit_ptr[2*2]
 * expit = np.PyUFunc_FromFuncAndData(ufunc_expit_loops, ufunc_expit_data, ufunc_expit_types, 3, 1, 1, 0, "expit", ufunc_expit_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_expm1_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_types, 3, 1, 1, 0, ((char *)"expit"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_expit_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_expit, __pyx_t_2) < 0) __PYX_ERR(1, 6283, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":6289
 * cdef void *ufunc_expm1_data[4]
 * cdef char ufunc_expm1_types[8]
 * cdef char *ufunc_expm1_doc = (             # <<<<<<<<<<<<<<
 *     "expm1(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_doc = ((char *)"expm1(x)\n\nCompute ``exp(x) - 1``.\n\nWhen `x` is near zero, ``exp(x)`` is near 1, so the numerical calculation\nof ``exp(x) - 1`` can suffer from catastrophic loss of precision.\n``expm1(x)`` is implemented to avoid the loss of precision that occurs when\n`x` is near zero.\n\nParameters\n----------\nx : array_like\n    `x` must contain real numbers.\n\nReturns\n-------\nfloat\n    ``exp(x) - 1`` computed element-wise.\n\nExamples\n--------\n>>> from scipy.special import expm1\n\n>>> expm1(1.0)\n1.7182818284590451\n>>> expm1([-0.2, -0.1, 0, 0.1, 0.2])\narray([-0.18126925, -0.09516258,  0.        ,  0.10517092,  0.22140276])\n\nThe exact value of ``exp(7.5e-13) - 1`` is::\n\n    7.5000000000028125000000007031250000001318...*10**-13.\n\nHere is what ``expm1(7.5e-13)`` gives:\n\n>>> expm1(7.5e-13)\n7.5000000000028135e-13\n\nCompare that to ``exp(7.5e-13) - 1``, where the subtraction results in\na \"catastrophic\" loss of precision:\n\n>>> np.exp(7.5e-13) - 1\n7.5006667543675576e-13");

  /* "scipy/special/_ufuncs.pyx":6332
 *     ">>> np.exp(7.5e-13) - 1\n"
 *     "7.5006667543675576e-13")
 * ufunc_expm1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_expm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_expm1_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6333
 *     "7.5006667543675576e-13")
 * ufunc_expm1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_expm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_expm1_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_expm1_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6334
 * ufunc_expm1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_expm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_expm1_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_expm1_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_expm1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":6335
 * ufunc_expm1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_expm1_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_expm1_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_expm1_types[0] = <char>NPY_FLOAT
 * ufunc_expm1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":6336
 * ufunc_expm1_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_expm1_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_expm1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_expm1_types[1] = <char>NPY_FLOAT
 * ufunc_expm1_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6337
 * ufunc_expm1_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_expm1_types[0] = <char>NPY_FLOAT
 * ufunc_expm1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_expm1_types[2] = <char>NPY_DOUBLE
 * ufunc_expm1_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6338
 * ufunc_expm1_types[0] = <char>NPY_FLOAT
 * ufunc_expm1_types[1] = <char>NPY_FLOAT
 * ufunc_expm1_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expm1_types[3] = <char>NPY_DOUBLE
 * ufunc_expm1_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6339
 * ufunc_expm1_types[1] = <char>NPY_FLOAT
 * ufunc_expm1_types[2] = <char>NPY_DOUBLE
 * ufunc_expm1_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expm1_types[4] = <char>NPY_CFLOAT
 * ufunc_expm1_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6340
 * ufunc_expm1_types[2] = <char>NPY_DOUBLE
 * ufunc_expm1_types[3] = <char>NPY_DOUBLE
 * ufunc_expm1_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_expm1_types[5] = <char>NPY_CFLOAT
 * ufunc_expm1_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6341
 * ufunc_expm1_types[3] = <char>NPY_DOUBLE
 * ufunc_expm1_types[4] = <char>NPY_CFLOAT
 * ufunc_expm1_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_expm1_types[6] = <char>NPY_CDOUBLE
 * ufunc_expm1_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6342
 * ufunc_expm1_types[4] = <char>NPY_CFLOAT
 * ufunc_expm1_types[5] = <char>NPY_CFLOAT
 * ufunc_expm1_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expm1_types[7] = <char>NPY_CDOUBLE
 * ufunc_expm1_ptr[2*0] = <void*>_func_expm1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6343
 * ufunc_expm1_types[5] = <char>NPY_CFLOAT
 * ufunc_expm1_types[6] = <char>NPY_CDOUBLE
 * ufunc_expm1_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expm1_ptr[2*0] = <void*>_func_expm1
 * ufunc_expm1_ptr[2*0+1] = <void*>(<char*>"expm1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6344
 * ufunc_expm1_types[6] = <char>NPY_CDOUBLE
 * ufunc_expm1_types[7] = <char>NPY_CDOUBLE
 * ufunc_expm1_ptr[2*0] = <void*>_func_expm1             # <<<<<<<<<<<<<<
 * ufunc_expm1_ptr[2*0+1] = <void*>(<char*>"expm1")
 * ufunc_expm1_ptr[2*1] = <void*>_func_expm1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_ptr[0]) = ((void *)expm1);

  /* "scipy/special/_ufuncs.pyx":6345
 * ufunc_expm1_types[7] = <char>NPY_CDOUBLE
 * ufunc_expm1_ptr[2*0] = <void*>_func_expm1
 * ufunc_expm1_ptr[2*0+1] = <void*>(<char*>"expm1")             # <<<<<<<<<<<<<<
 * ufunc_expm1_ptr[2*1] = <void*>_func_expm1
 * ufunc_expm1_ptr[2*1+1] = <void*>(<char*>"expm1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_ptr[1]) = ((void *)((char *)((char *)"expm1")));

  /* "scipy/special/_ufuncs.pyx":6346
 * ufunc_expm1_ptr[2*0] = <void*>_func_expm1
 * ufunc_expm1_ptr[2*0+1] = <void*>(<char*>"expm1")
 * ufunc_expm1_ptr[2*1] = <void*>_func_expm1             # <<<<<<<<<<<<<<
 * ufunc_expm1_ptr[2*1+1] = <void*>(<char*>"expm1")
 * ufunc_expm1_ptr[2*2] = <void*>_func_cexpm1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_ptr[2]) = ((void *)expm1);

  /* "scipy/special/_ufuncs.pyx":6347
 * ufunc_expm1_ptr[2*0+1] = <void*>(<char*>"expm1")
 * ufunc_expm1_ptr[2*1] = <void*>_func_expm1
 * ufunc_expm1_ptr[2*1+1] = <void*>(<char*>"expm1")             # <<<<<<<<<<<<<<
 * ufunc_expm1_ptr[2*2] = <void*>_func_cexpm1
 * ufunc_expm1_ptr[2*2+1] = <void*>(<char*>"expm1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_ptr[3]) = ((void *)((char *)((char *)"expm1")));

  /* "scipy/special/_ufuncs.pyx":6348
 * ufunc_expm1_ptr[2*1] = <void*>_func_expm1
 * ufunc_expm1_ptr[2*1+1] = <void*>(<char*>"expm1")
 * ufunc_expm1_ptr[2*2] = <void*>_func_cexpm1             # <<<<<<<<<<<<<<
 * ufunc_expm1_ptr[2*2+1] = <void*>(<char*>"expm1")
 * ufunc_expm1_ptr[2*3] = <void*>_func_cexpm1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_cunity_cexpm1);

  /* "scipy/special/_ufuncs.pyx":6349
 * ufunc_expm1_ptr[2*1+1] = <void*>(<char*>"expm1")
 * ufunc_expm1_ptr[2*2] = <void*>_func_cexpm1
 * ufunc_expm1_ptr[2*2+1] = <void*>(<char*>"expm1")             # <<<<<<<<<<<<<<
 * ufunc_expm1_ptr[2*3] = <void*>_func_cexpm1
 * ufunc_expm1_ptr[2*3+1] = <void*>(<char*>"expm1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_ptr[5]) = ((void *)((char *)((char *)"expm1")));

  /* "scipy/special/_ufuncs.pyx":6350
 * ufunc_expm1_ptr[2*2] = <void*>_func_cexpm1
 * ufunc_expm1_ptr[2*2+1] = <void*>(<char*>"expm1")
 * ufunc_expm1_ptr[2*3] = <void*>_func_cexpm1             # <<<<<<<<<<<<<<
 * ufunc_expm1_ptr[2*3+1] = <void*>(<char*>"expm1")
 * ufunc_expm1_data[0] = &ufunc_expm1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_ptr[6]) = ((void *)__pyx_f_5scipy_7special_7_cunity_cexpm1);

  /* "scipy/special/_ufuncs.pyx":6351
 * ufunc_expm1_ptr[2*2+1] = <void*>(<char*>"expm1")
 * ufunc_expm1_ptr[2*3] = <void*>_func_cexpm1
 * ufunc_expm1_ptr[2*3+1] = <void*>(<char*>"expm1")             # <<<<<<<<<<<<<<
 * ufunc_expm1_data[0] = &ufunc_expm1_ptr[2*0]
 * ufunc_expm1_data[1] = &ufunc_expm1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_ptr[7]) = ((void *)((char *)((char *)"expm1")));

  /* "scipy/special/_ufuncs.pyx":6352
 * ufunc_expm1_ptr[2*3] = <void*>_func_cexpm1
 * ufunc_expm1_ptr[2*3+1] = <void*>(<char*>"expm1")
 * ufunc_expm1_data[0] = &ufunc_expm1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_expm1_data[1] = &ufunc_expm1_ptr[2*1]
 * ufunc_expm1_data[2] = &ufunc_expm1_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6353
 * ufunc_expm1_ptr[2*3+1] = <void*>(<char*>"expm1")
 * ufunc_expm1_data[0] = &ufunc_expm1_ptr[2*0]
 * ufunc_expm1_data[1] = &ufunc_expm1_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_expm1_data[2] = &ufunc_expm1_ptr[2*2]
 * ufunc_expm1_data[3] = &ufunc_expm1_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6354
 * ufunc_expm1_data[0] = &ufunc_expm1_ptr[2*0]
 * ufunc_expm1_data[1] = &ufunc_expm1_ptr[2*1]
 * ufunc_expm1_data[2] = &ufunc_expm1_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_expm1_data[3] = &ufunc_expm1_ptr[2*3]
 * expm1 = np.PyUFunc_FromFuncAndData(ufunc_expm1_loops, ufunc_expm1_data, ufunc_expm1_types, 4, 1, 1, 0, "expm1", ufunc_expm1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":6355
 * ufunc_expm1_data[1] = &ufunc_expm1_ptr[2*1]
 * ufunc_expm1_data[2] = &ufunc_expm1_ptr[2*2]
 * ufunc_expm1_data[3] = &ufunc_expm1_ptr[2*3]             # <<<<<<<<<<<<<<
 * expm1 = np.PyUFunc_FromFuncAndData(ufunc_expm1_loops, ufunc_expm1_data, ufunc_expm1_types, 4, 1, 1, 0, "expm1", ufunc_expm1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":6356
 * ufunc_expm1_data[2] = &ufunc_expm1_ptr[2*2]
 * ufunc_expm1_data[3] = &ufunc_expm1_ptr[2*3]
 * expm1 = np.PyUFunc_FromFuncAndData(ufunc_expm1_loops, ufunc_expm1_data, ufunc_expm1_types, 4, 1, 1, 0, "expm1", ufunc_expm1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_expn_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_types, 4, 1, 1, 0, ((char *)"expm1"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_expm1_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6356, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_expm1, __pyx_t_2) < 0) __PYX_ERR(1, 6356, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":6362
 * cdef void *ufunc_expn_data[3]
 * cdef char ufunc_expn_types[9]
 * cdef char *ufunc_expn_doc = (             # <<<<<<<<<<<<<<
 *     "expn(n, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_doc = ((char *)"expn(n, x)\n\nExponential integral E_n\n\nReturns the exponential integral for integer `n` and non-negative `x` and\n`n`::\n\n    integral(exp(-x*t) / t**n, t=1..inf).");

  /* "scipy/special/_ufuncs.pyx":6371
 *     "\n"
 *     "    integral(exp(-x*t) / t**n, t=1..inf).")
 * ufunc_expn_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_expn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_expn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":6372
 *     "    integral(exp(-x*t) / t**n, t=1..inf).")
 * ufunc_expn_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_expn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_expn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_expn_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":6373
 * ufunc_expn_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_expn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_expn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_expn_types[0] = <char>NPY_LONG
 * ufunc_expn_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":6374
 * ufunc_expn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_expn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_expn_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_expn_types[1] = <char>NPY_DOUBLE
 * ufunc_expn_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":6375
 * ufunc_expn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_expn_types[0] = <char>NPY_LONG
 * ufunc_expn_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expn_types[2] = <char>NPY_DOUBLE
 * ufunc_expn_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6376
 * ufunc_expn_types[0] = <char>NPY_LONG
 * ufunc_expn_types[1] = <char>NPY_DOUBLE
 * ufunc_expn_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expn_types[3] = <char>NPY_FLOAT
 * ufunc_expn_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6377
 * ufunc_expn_types[1] = <char>NPY_DOUBLE
 * ufunc_expn_types[2] = <char>NPY_DOUBLE
 * ufunc_expn_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_expn_types[4] = <char>NPY_FLOAT
 * ufunc_expn_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6378
 * ufunc_expn_types[2] = <char>NPY_DOUBLE
 * ufunc_expn_types[3] = <char>NPY_FLOAT
 * ufunc_expn_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_expn_types[5] = <char>NPY_FLOAT
 * ufunc_expn_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6379
 * ufunc_expn_types[3] = <char>NPY_FLOAT
 * ufunc_expn_types[4] = <char>NPY_FLOAT
 * ufunc_expn_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_expn_types[6] = <char>NPY_DOUBLE
 * ufunc_expn_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6380
 * ufunc_expn_types[4] = <char>NPY_FLOAT
 * ufunc_expn_types[5] = <char>NPY_FLOAT
 * ufunc_expn_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expn_types[7] = <char>NPY_DOUBLE
 * ufunc_expn_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6381
 * ufunc_expn_types[5] = <char>NPY_FLOAT
 * ufunc_expn_types[6] = <char>NPY_DOUBLE
 * ufunc_expn_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expn_types[8] = <char>NPY_DOUBLE
 * ufunc_expn_ptr[2*0] = <void*>_func_expn
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6382
 * ufunc_expn_types[6] = <char>NPY_DOUBLE
 * ufunc_expn_types[7] = <char>NPY_DOUBLE
 * ufunc_expn_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_expn_ptr[2*0] = <void*>_func_expn
 * ufunc_expn_ptr[2*0+1] = <void*>(<char*>"expn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6383
 * ufunc_expn_types[7] = <char>NPY_DOUBLE
 * ufunc_expn_types[8] = <char>NPY_DOUBLE
 * ufunc_expn_ptr[2*0] = <void*>_func_expn             # <<<<<<<<<<<<<<
 * ufunc_expn_ptr[2*0+1] = <void*>(<char*>"expn")
 * ufunc_expn_ptr[2*1] = <void*>_func_expn_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_ptr[0]) = ((void *)expn);

  /* "scipy/special/_ufuncs.pyx":6384
 * ufunc_expn_types[8] = <char>NPY_DOUBLE
 * ufunc_expn_ptr[2*0] = <void*>_func_expn
 * ufunc_expn_ptr[2*0+1] = <void*>(<char*>"expn")             # <<<<<<<<<<<<<<
 * ufunc_expn_ptr[2*1] = <void*>_func_expn_unsafe
 * ufunc_expn_ptr[2*1+1] = <void*>(<char*>"expn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_ptr[1]) = ((void *)((char *)((char *)"expn")));

  /* "scipy/special/_ufuncs.pyx":6385
 * ufunc_expn_ptr[2*0] = <void*>_func_expn
 * ufunc_expn_ptr[2*0+1] = <void*>(<char*>"expn")
 * ufunc_expn_ptr[2*1] = <void*>_func_expn_unsafe             # <<<<<<<<<<<<<<
 * ufunc_expn_ptr[2*1+1] = <void*>(<char*>"expn")
 * ufunc_expn_ptr[2*2] = <void*>_func_expn_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_expn_unsafe);

  /* "scipy/special/_ufuncs.pyx":6386
 * ufunc_expn_ptr[2*0+1] = <void*>(<char*>"expn")
 * ufunc_expn_ptr[2*1] = <void*>_func_expn_unsafe
 * ufunc_expn_ptr[2*1+1] = <void*>(<char*>"expn")             # <<<<<<<<<<<<<<
 * ufunc_expn_ptr[2*2] = <void*>_func_expn_unsafe
 * ufunc_expn_ptr[2*2+1] = <void*>(<char*>"expn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_ptr[3]) = ((void *)((char *)((char *)"expn")));

  /* "scipy/special/_ufuncs.pyx":6387
 * ufunc_expn_ptr[2*1] = <void*>_func_expn_unsafe
 * ufunc_expn_ptr[2*1+1] = <void*>(<char*>"expn")
 * ufunc_expn_ptr[2*2] = <void*>_func_expn_unsafe             # <<<<<<<<<<<<<<
 * ufunc_expn_ptr[2*2+1] = <void*>(<char*>"expn")
 * ufunc_expn_data[0] = &ufunc_expn_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_expn_unsafe);

  /* "scipy/special/_ufuncs.pyx":6388
 * ufunc_expn_ptr[2*1+1] = <void*>(<char*>"expn")
 * ufunc_expn_ptr[2*2] = <void*>_func_expn_unsafe
 * ufunc_expn_ptr[2*2+1] = <void*>(<char*>"expn")             # <<<<<<<<<<<<<<
 * ufunc_expn_data[0] = &ufunc_expn_ptr[2*0]
 * ufunc_expn_data[1] = &ufunc_expn_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_ptr[5]) = ((void *)((char *)((char *)"expn")));

  /* "scipy/special/_ufuncs.pyx":6389
 * ufunc_expn_ptr[2*2] = <void*>_func_expn_unsafe
 * ufunc_expn_ptr[2*2+1] = <void*>(<char*>"expn")
 * ufunc_expn_data[0] = &ufunc_expn_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_expn_data[1] = &ufunc_expn_ptr[2*1]
 * ufunc_expn_data[2] = &ufunc_expn_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6390
 * ufunc_expn_ptr[2*2+1] = <void*>(<char*>"expn")
 * ufunc_expn_data[0] = &ufunc_expn_ptr[2*0]
 * ufunc_expn_data[1] = &ufunc_expn_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_expn_data[2] = &ufunc_expn_ptr[2*2]
 * expn = np.PyUFunc_FromFuncAndData(ufunc_expn_loops, ufunc_expn_data, ufunc_expn_types, 3, 2, 1, 0, "expn", ufunc_expn_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6391
 * ufunc_expn_data[0] = &ufunc_expn_ptr[2*0]
 * ufunc_expn_data[1] = &ufunc_expn_ptr[2*1]
 * ufunc_expn_data[2] = &ufunc_expn_ptr[2*2]             # <<<<<<<<<<<<<<
 * expn = np.PyUFunc_FromFuncAndData(ufunc_expn_loops, ufunc_expn_data, ufunc_expn_types, 3, 2, 1, 0, "expn", ufunc_expn_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":6392
 * ufunc_expn_data[1] = &ufunc_expn_ptr[2*1]
 * ufunc_expn_data[2] = &ufunc_expn_ptr[2*2]
 * expn = np.PyUFunc_FromFuncAndData(ufunc_expn_loops, ufunc_expn_data, ufunc_expn_types, 3, 2, 1, 0, "expn", ufunc_expn_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_exprel_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_types, 3, 2, 1, 0, ((char *)"expn"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_expn_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6392, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_expn, __pyx_t_2) < 0) __PYX_ERR(1, 6392, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":6398
 * cdef void *ufunc_exprel_data[2]
 * cdef char ufunc_exprel_types[4]
 * cdef char *ufunc_exprel_doc = (             # <<<<<<<<<<<<<<
 *     "exprel(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_doc = ((char *)"exprel(x)\n\nRelative error exponential, ``(exp(x) - 1)/x``.\n\nWhen `x` is near zero, ``exp(x)`` is near 1, so the numerical calculation\nof ``exp(x) - 1`` can suffer from catastrophic loss of precision.\n``exprel(x)`` is implemented to avoid the loss of precision that occurs when\n`x` is near zero.\n\nParameters\n----------\nx : ndarray\n    Input array.  `x` must contain real numbers.\n\nReturns\n-------\nfloat\n    ``(exp(x) - 1)/x``, computed element-wise.\n\nSee Also\n--------\nexpm1\n\nNotes\n-----\n.. versionadded:: 0.17.0\n\nExamples\n--------\n>>> from scipy.special import exprel\n\n>>> exprel(0.01)\n1.0050167084168056\n>>> exprel([-0.25, -0.1, 0, 0.1, 0.25])\narray([ 0.88479687,  0.95162582,  1.        ,  1.05170918,  1.13610167])\n\nCompare ``exprel(5e-9)`` to the naive calculation.  The exact value\nis ``1.00000000250000000416...``.\n\n>>> exprel(5e-9)\n1.0000000025\n\n>>> (np.exp(5e-9) - 1)/5e-9\n0.99999999392252903");

  /* "scipy/special/_ufuncs.pyx":6443
 *     ">>> (np.exp(5e-9) - 1)/5e-9\n"
 *     "0.99999999392252903")
 * ufunc_exprel_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_exprel_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exprel_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6444
 *     "0.99999999392252903")
 * ufunc_exprel_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_exprel_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_exprel_types[0] = <char>NPY_FLOAT
 * ufunc_exprel_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6445
 * ufunc_exprel_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_exprel_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exprel_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_exprel_types[1] = <char>NPY_FLOAT
 * ufunc_exprel_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6446
 * ufunc_exprel_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_exprel_types[0] = <char>NPY_FLOAT
 * ufunc_exprel_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_exprel_types[2] = <char>NPY_DOUBLE
 * ufunc_exprel_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6447
 * ufunc_exprel_types[0] = <char>NPY_FLOAT
 * ufunc_exprel_types[1] = <char>NPY_FLOAT
 * ufunc_exprel_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_exprel_types[3] = <char>NPY_DOUBLE
 * ufunc_exprel_ptr[2*0] = <void*>_func_exprel
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6448
 * ufunc_exprel_types[1] = <char>NPY_FLOAT
 * ufunc_exprel_types[2] = <char>NPY_DOUBLE
 * ufunc_exprel_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_exprel_ptr[2*0] = <void*>_func_exprel
 * ufunc_exprel_ptr[2*0+1] = <void*>(<char*>"exprel")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6449
 * ufunc_exprel_types[2] = <char>NPY_DOUBLE
 * ufunc_exprel_types[3] = <char>NPY_DOUBLE
 * ufunc_exprel_ptr[2*0] = <void*>_func_exprel             # <<<<<<<<<<<<<<
 * ufunc_exprel_ptr[2*0+1] = <void*>(<char*>"exprel")
 * ufunc_exprel_ptr[2*1] = <void*>_func_exprel
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_ptr[0]) = ((void *)__pyx_f_5scipy_7special_7_exprel_exprel);

  /* "scipy/special/_ufuncs.pyx":6450
 * ufunc_exprel_types[3] = <char>NPY_DOUBLE
 * ufunc_exprel_ptr[2*0] = <void*>_func_exprel
 * ufunc_exprel_ptr[2*0+1] = <void*>(<char*>"exprel")             # <<<<<<<<<<<<<<
 * ufunc_exprel_ptr[2*1] = <void*>_func_exprel
 * ufunc_exprel_ptr[2*1+1] = <void*>(<char*>"exprel")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_ptr[1]) = ((void *)((char *)((char *)"exprel")));

  /* "scipy/special/_ufuncs.pyx":6451
 * ufunc_exprel_ptr[2*0] = <void*>_func_exprel
 * ufunc_exprel_ptr[2*0+1] = <void*>(<char*>"exprel")
 * ufunc_exprel_ptr[2*1] = <void*>_func_exprel             # <<<<<<<<<<<<<<
 * ufunc_exprel_ptr[2*1+1] = <void*>(<char*>"exprel")
 * ufunc_exprel_data[0] = &ufunc_exprel_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_exprel_exprel);

  /* "scipy/special/_ufuncs.pyx":6452
 * ufunc_exprel_ptr[2*0+1] = <void*>(<char*>"exprel")
 * ufunc_exprel_ptr[2*1] = <void*>_func_exprel
 * ufunc_exprel_ptr[2*1+1] = <void*>(<char*>"exprel")             # <<<<<<<<<<<<<<
 * ufunc_exprel_data[0] = &ufunc_exprel_ptr[2*0]
 * ufunc_exprel_data[1] = &ufunc_exprel_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_ptr[3]) = ((void *)((char *)((char *)"exprel")));

  /* "scipy/special/_ufuncs.pyx":6453
 * ufunc_exprel_ptr[2*1] = <void*>_func_exprel
 * ufunc_exprel_ptr[2*1+1] = <void*>(<char*>"exprel")
 * ufunc_exprel_data[0] = &ufunc_exprel_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_exprel_data[1] = &ufunc_exprel_ptr[2*1]
 * exprel = np.PyUFunc_FromFuncAndData(ufunc_exprel_loops, ufunc_exprel_data, ufunc_exprel_types, 2, 1, 1, 0, "exprel", ufunc_exprel_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6454
 * ufunc_exprel_ptr[2*1+1] = <void*>(<char*>"exprel")
 * ufunc_exprel_data[0] = &ufunc_exprel_ptr[2*0]
 * ufunc_exprel_data[1] = &ufunc_exprel_ptr[2*1]             # <<<<<<<<<<<<<<
 * exprel = np.PyUFunc_FromFuncAndData(ufunc_exprel_loops, ufunc_exprel_data, ufunc_exprel_types, 2, 1, 1, 0, "exprel", ufunc_exprel_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6455
 * ufunc_exprel_data[0] = &ufunc_exprel_ptr[2*0]
 * ufunc_exprel_data[1] = &ufunc_exprel_ptr[2*1]
 * exprel = np.PyUFunc_FromFuncAndData(ufunc_exprel_loops, ufunc_exprel_data, ufunc_exprel_types, 2, 1, 1, 0, "exprel", ufunc_exprel_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_fdtr_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_types, 2, 1, 1, 0, ((char *)"exprel"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_exprel_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_exprel, __pyx_t_2) < 0) __PYX_ERR(1, 6455, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":6461
 * cdef void *ufunc_fdtr_data[2]
 * cdef char ufunc_fdtr_types[8]
 * cdef char *ufunc_fdtr_doc = (             # <<<<<<<<<<<<<<
 *     "fdtr(dfn, dfd, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_doc = ((char *)"fdtr(dfn, dfd, x)\n\nF cumulative distribution function.\n\nReturns the value of the cumulative distribution function of the\nF-distribution, also known as Snedecor's F-distribution or the\nFisher-Snedecor distribution.\n\nThe F-distribution with parameters :math:`d_n` and :math:`d_d` is the\ndistribution of the random variable,\n\n.. math::\n    X = \\frac{U_n/d_n}{U_d/d_d},\n\nwhere :math:`U_n` and :math:`U_d` are random variables distributed\n:math:`\\chi^2`, with :math:`d_n` and :math:`d_d` degrees of freedom,\nrespectively.\n\nParameters\n----------\ndfn : array_like\n    First parameter (positive float).\ndfd : array_like\n    Second parameter (positive float).\nx : array_like\n    Argument (nonnegative float).\n\nReturns\n-------\ny : ndarray\n    The CDF of the F-distribution with parameters `dfn` and `dfd` at `x`.\n\nNotes\n-----\nThe regularized incomplete beta function is used, according to the\nformula,\n\n.. math::\n    F(d_n, d_d; x) = I_{xd_n/(d_d + xd_n)}(d_n/2, d_d/2).\n\nWrapper for the Cephes [1]_ routine `fdtr`.\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":6508
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_fdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_fdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":6509
 *     "       http://www.netlib.org/cephes/")
 * ufunc_fdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_fdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_fdtr_types[0] = <char>NPY_FLOAT
 * ufunc_fdtr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":6510
 * ufunc_fdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_fdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtr_types[1] = <char>NPY_FLOAT
 * ufunc_fdtr_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6511
 * ufunc_fdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtr_types[0] = <char>NPY_FLOAT
 * ufunc_fdtr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtr_types[2] = <char>NPY_FLOAT
 * ufunc_fdtr_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6512
 * ufunc_fdtr_types[0] = <char>NPY_FLOAT
 * ufunc_fdtr_types[1] = <char>NPY_FLOAT
 * ufunc_fdtr_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtr_types[3] = <char>NPY_FLOAT
 * ufunc_fdtr_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6513
 * ufunc_fdtr_types[1] = <char>NPY_FLOAT
 * ufunc_fdtr_types[2] = <char>NPY_FLOAT
 * ufunc_fdtr_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtr_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6514
 * ufunc_fdtr_types[2] = <char>NPY_FLOAT
 * ufunc_fdtr_types[3] = <char>NPY_FLOAT
 * ufunc_fdtr_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtr_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6515
 * ufunc_fdtr_types[3] = <char>NPY_FLOAT
 * ufunc_fdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtr_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtr_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6516
 * ufunc_fdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtr_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtr_ptr[2*0] = <void*>_func_fdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6517
 * ufunc_fdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtr_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtr_ptr[2*0] = <void*>_func_fdtr
 * ufunc_fdtr_ptr[2*0+1] = <void*>(<char*>"fdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6518
 * ufunc_fdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtr_ptr[2*0] = <void*>_func_fdtr             # <<<<<<<<<<<<<<
 * ufunc_fdtr_ptr[2*0+1] = <void*>(<char*>"fdtr")
 * ufunc_fdtr_ptr[2*1] = <void*>_func_fdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_ptr[0]) = ((void *)fdtr);

  /* "scipy/special/_ufuncs.pyx":6519
 * ufunc_fdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtr_ptr[2*0] = <void*>_func_fdtr
 * ufunc_fdtr_ptr[2*0+1] = <void*>(<char*>"fdtr")             # <<<<<<<<<<<<<<
 * ufunc_fdtr_ptr[2*1] = <void*>_func_fdtr
 * ufunc_fdtr_ptr[2*1+1] = <void*>(<char*>"fdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_ptr[1]) = ((void *)((char *)((char *)"fdtr")));

  /* "scipy/special/_ufuncs.pyx":6520
 * ufunc_fdtr_ptr[2*0] = <void*>_func_fdtr
 * ufunc_fdtr_ptr[2*0+1] = <void*>(<char*>"fdtr")
 * ufunc_fdtr_ptr[2*1] = <void*>_func_fdtr             # <<<<<<<<<<<<<<
 * ufunc_fdtr_ptr[2*1+1] = <void*>(<char*>"fdtr")
 * ufunc_fdtr_data[0] = &ufunc_fdtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_ptr[2]) = ((void *)fdtr);

  /* "scipy/special/_ufuncs.pyx":6521
 * ufunc_fdtr_ptr[2*0+1] = <void*>(<char*>"fdtr")
 * ufunc_fdtr_ptr[2*1] = <void*>_func_fdtr
 * ufunc_fdtr_ptr[2*1+1] = <void*>(<char*>"fdtr")             # <<<<<<<<<<<<<<
 * ufunc_fdtr_data[0] = &ufunc_fdtr_ptr[2*0]
 * ufunc_fdtr_data[1] = &ufunc_fdtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_ptr[3]) = ((void *)((char *)((char *)"fdtr")));

  /* "scipy/special/_ufuncs.pyx":6522
 * ufunc_fdtr_ptr[2*1] = <void*>_func_fdtr
 * ufunc_fdtr_ptr[2*1+1] = <void*>(<char*>"fdtr")
 * ufunc_fdtr_data[0] = &ufunc_fdtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_fdtr_data[1] = &ufunc_fdtr_ptr[2*1]
 * fdtr = np.PyUFunc_FromFuncAndData(ufunc_fdtr_loops, ufunc_fdtr_data, ufunc_fdtr_types, 2, 3, 1, 0, "fdtr", ufunc_fdtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6523
 * ufunc_fdtr_ptr[2*1+1] = <void*>(<char*>"fdtr")
 * ufunc_fdtr_data[0] = &ufunc_fdtr_ptr[2*0]
 * ufunc_fdtr_data[1] = &ufunc_fdtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * fdtr = np.PyUFunc_FromFuncAndData(ufunc_fdtr_loops, ufunc_fdtr_data, ufunc_fdtr_types, 2, 3, 1, 0, "fdtr", ufunc_fdtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6524
 * ufunc_fdtr_data[0] = &ufunc_fdtr_ptr[2*0]
 * ufunc_fdtr_data[1] = &ufunc_fdtr_ptr[2*1]
 * fdtr = np.PyUFunc_FromFuncAndData(ufunc_fdtr_loops, ufunc_fdtr_data, ufunc_fdtr_types, 2, 3, 1, 0, "fdtr", ufunc_fdtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_fdtrc_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_types, 2, 3, 1, 0, ((char *)"fdtr"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtr_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6524, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_fdtr, __pyx_t_2) < 0) __PYX_ERR(1, 6524, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":6530
 * cdef void *ufunc_fdtrc_data[2]
 * cdef char ufunc_fdtrc_types[8]
 * cdef char *ufunc_fdtrc_doc = (             # <<<<<<<<<<<<<<
 *     "fdtrc(dfn, dfd, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_doc = ((char *)"fdtrc(dfn, dfd, x)\n\nF survival function.\n\nReturns the complemented F-distribution function (the integral of the\ndensity from `x` to infinity).\n\nParameters\n----------\ndfn : array_like\n    First parameter (positive float).\ndfd : array_like\n    Second parameter (positive float).\nx : array_like\n    Argument (nonnegative float).\n\nReturns\n-------\ny : ndarray\n    The complemented F-distribution function with parameters `dfn` and\n    `dfd` at `x`.\n\nSee also\n--------\nfdtr\n\nNotes\n-----\nThe regularized incomplete beta function is used, according to the\nformula,\n\n.. math::\n    F(d_n, d_d; x) = I_{d_d/(d_d + xd_n)}(d_d/2, d_n/2).\n\nWrapper for the Cephes [1]_ routine `fdtrc`.\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":6571
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_fdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtrc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":6572
 *     "       http://www.netlib.org/cephes/")
 * ufunc_fdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_fdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_types[0] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":6573
 * ufunc_fdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_fdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtrc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_types[1] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6574
 * ufunc_fdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtrc_types[0] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_types[2] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6575
 * ufunc_fdtrc_types[0] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[1] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_types[3] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6576
 * ufunc_fdtrc_types[1] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[2] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtrc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6577
 * ufunc_fdtrc_types[2] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[3] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtrc_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6578
 * ufunc_fdtrc_types[3] = <char>NPY_FLOAT
 * ufunc_fdtrc_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtrc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtrc_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6579
 * ufunc_fdtrc_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtrc_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtrc_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtrc_ptr[2*0] = <void*>_func_fdtrc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6580
 * ufunc_fdtrc_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtrc_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtrc_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_ptr[2*0] = <void*>_func_fdtrc
 * ufunc_fdtrc_ptr[2*0+1] = <void*>(<char*>"fdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6581
 * ufunc_fdtrc_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtrc_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtrc_ptr[2*0] = <void*>_func_fdtrc             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_ptr[2*0+1] = <void*>(<char*>"fdtrc")
 * ufunc_fdtrc_ptr[2*1] = <void*>_func_fdtrc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_ptr[0]) = ((void *)fdtrc);

  /* "scipy/special/_ufuncs.pyx":6582
 * ufunc_fdtrc_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtrc_ptr[2*0] = <void*>_func_fdtrc
 * ufunc_fdtrc_ptr[2*0+1] = <void*>(<char*>"fdtrc")             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_ptr[2*1] = <void*>_func_fdtrc
 * ufunc_fdtrc_ptr[2*1+1] = <void*>(<char*>"fdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_ptr[1]) = ((void *)((char *)((char *)"fdtrc")));

  /* "scipy/special/_ufuncs.pyx":6583
 * ufunc_fdtrc_ptr[2*0] = <void*>_func_fdtrc
 * ufunc_fdtrc_ptr[2*0+1] = <void*>(<char*>"fdtrc")
 * ufunc_fdtrc_ptr[2*1] = <void*>_func_fdtrc             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_ptr[2*1+1] = <void*>(<char*>"fdtrc")
 * ufunc_fdtrc_data[0] = &ufunc_fdtrc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_ptr[2]) = ((void *)fdtrc);

  /* "scipy/special/_ufuncs.pyx":6584
 * ufunc_fdtrc_ptr[2*0+1] = <void*>(<char*>"fdtrc")
 * ufunc_fdtrc_ptr[2*1] = <void*>_func_fdtrc
 * ufunc_fdtrc_ptr[2*1+1] = <void*>(<char*>"fdtrc")             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_data[0] = &ufunc_fdtrc_ptr[2*0]
 * ufunc_fdtrc_data[1] = &ufunc_fdtrc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_ptr[3]) = ((void *)((char *)((char *)"fdtrc")));

  /* "scipy/special/_ufuncs.pyx":6585
 * ufunc_fdtrc_ptr[2*1] = <void*>_func_fdtrc
 * ufunc_fdtrc_ptr[2*1+1] = <void*>(<char*>"fdtrc")
 * ufunc_fdtrc_data[0] = &ufunc_fdtrc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_fdtrc_data[1] = &ufunc_fdtrc_ptr[2*1]
 * fdtrc = np.PyUFunc_FromFuncAndData(ufunc_fdtrc_loops, ufunc_fdtrc_data, ufunc_fdtrc_types, 2, 3, 1, 0, "fdtrc", ufunc_fdtrc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6586
 * ufunc_fdtrc_ptr[2*1+1] = <void*>(<char*>"fdtrc")
 * ufunc_fdtrc_data[0] = &ufunc_fdtrc_ptr[2*0]
 * ufunc_fdtrc_data[1] = &ufunc_fdtrc_ptr[2*1]             # <<<<<<<<<<<<<<
 * fdtrc = np.PyUFunc_FromFuncAndData(ufunc_fdtrc_loops, ufunc_fdtrc_data, ufunc_fdtrc_types, 2, 3, 1, 0, "fdtrc", ufunc_fdtrc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6587
 * ufunc_fdtrc_data[0] = &ufunc_fdtrc_ptr[2*0]
 * ufunc_fdtrc_data[1] = &ufunc_fdtrc_ptr[2*1]
 * fdtrc = np.PyUFunc_FromFuncAndData(ufunc_fdtrc_loops, ufunc_fdtrc_data, ufunc_fdtrc_types, 2, 3, 1, 0, "fdtrc", ufunc_fdtrc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_fdtri_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_types, 2, 3, 1, 0, ((char *)"fdtrc"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtrc_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6587, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_fdtrc, __pyx_t_2) < 0) __PYX_ERR(1, 6587, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":6593
 * cdef void *ufunc_fdtri_data[2]
 * cdef char ufunc_fdtri_types[8]
 * cdef char *ufunc_fdtri_doc = (             # <<<<<<<<<<<<<<
 *     "fdtri(dfn, dfd, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_doc = ((char *)"fdtri(dfn, dfd, p)\n\nThe `p`-th quantile of the F-distribution.\n\nThis function is the inverse of the F-distribution CDF, `fdtr`, returning\nthe `x` such that `fdtr(dfn, dfd, x) = p`.\n\nParameters\n----------\ndfn : array_like\n    First parameter (positive float).\ndfd : array_like\n    Second parameter (positive float).\np : array_like\n    Cumulative probability, in [0, 1].\n\nReturns\n-------\nx : ndarray\n    The quantile corresponding to `p`.\n\nNotes\n-----\nThe computation is carried out using the relation to the inverse\nregularized beta function, :math:`I^{-1}_x(a, b)`.  Let\n:math:`z = I^{-1}_p(d_d/2, d_n/2).`  Then,\n\n.. math::\n    x = \\frac{d_d (1 - z)}{d_n z}.\n\nIf `p` is such that :math:`x < 0.5`, the following relation is used\ninstead for improved stability: let\n:math:`z' = I^{-1}_{1 - p}(d_n/2, d_d/2).` Then,\n\n.. math::\n    x = \\frac{d_d z'}{d_n (1 - z')}.\n\nWrapper for the Cephes [1]_ routine `fdtri`.\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":6637
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_fdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_fdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtri_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":6638
 *     "       http://www.netlib.org/cephes/")
 * ufunc_fdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_fdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_fdtri_types[0] = <char>NPY_FLOAT
 * ufunc_fdtri_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":6639
 * ufunc_fdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_fdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtri_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtri_types[1] = <char>NPY_FLOAT
 * ufunc_fdtri_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6640
 * ufunc_fdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtri_types[0] = <char>NPY_FLOAT
 * ufunc_fdtri_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtri_types[2] = <char>NPY_FLOAT
 * ufunc_fdtri_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6641
 * ufunc_fdtri_types[0] = <char>NPY_FLOAT
 * ufunc_fdtri_types[1] = <char>NPY_FLOAT
 * ufunc_fdtri_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtri_types[3] = <char>NPY_FLOAT
 * ufunc_fdtri_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6642
 * ufunc_fdtri_types[1] = <char>NPY_FLOAT
 * ufunc_fdtri_types[2] = <char>NPY_FLOAT
 * ufunc_fdtri_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtri_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtri_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6643
 * ufunc_fdtri_types[2] = <char>NPY_FLOAT
 * ufunc_fdtri_types[3] = <char>NPY_FLOAT
 * ufunc_fdtri_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtri_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6644
 * ufunc_fdtri_types[3] = <char>NPY_FLOAT
 * ufunc_fdtri_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtri_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtri_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6645
 * ufunc_fdtri_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtri_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtri_ptr[2*0] = <void*>_func_fdtri
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6646
 * ufunc_fdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtri_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtri_ptr[2*0] = <void*>_func_fdtri
 * ufunc_fdtri_ptr[2*0+1] = <void*>(<char*>"fdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6647
 * ufunc_fdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtri_ptr[2*0] = <void*>_func_fdtri             # <<<<<<<<<<<<<<
 * ufunc_fdtri_ptr[2*0+1] = <void*>(<char*>"fdtri")
 * ufunc_fdtri_ptr[2*1] = <void*>_func_fdtri
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_ptr[0]) = ((void *)fdtri);

  /* "scipy/special/_ufuncs.pyx":6648
 * ufunc_fdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtri_ptr[2*0] = <void*>_func_fdtri
 * ufunc_fdtri_ptr[2*0+1] = <void*>(<char*>"fdtri")             # <<<<<<<<<<<<<<
 * ufunc_fdtri_ptr[2*1] = <void*>_func_fdtri
 * ufunc_fdtri_ptr[2*1+1] = <void*>(<char*>"fdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_ptr[1]) = ((void *)((char *)((char *)"fdtri")));

  /* "scipy/special/_ufuncs.pyx":6649
 * ufunc_fdtri_ptr[2*0] = <void*>_func_fdtri
 * ufunc_fdtri_ptr[2*0+1] = <void*>(<char*>"fdtri")
 * ufunc_fdtri_ptr[2*1] = <void*>_func_fdtri             # <<<<<<<<<<<<<<
 * ufunc_fdtri_ptr[2*1+1] = <void*>(<char*>"fdtri")
 * ufunc_fdtri_data[0] = &ufunc_fdtri_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_ptr[2]) = ((void *)fdtri);

  /* "scipy/special/_ufuncs.pyx":6650
 * ufunc_fdtri_ptr[2*0+1] = <void*>(<char*>"fdtri")
 * ufunc_fdtri_ptr[2*1] = <void*>_func_fdtri
 * ufunc_fdtri_ptr[2*1+1] = <void*>(<char*>"fdtri")             # <<<<<<<<<<<<<<
 * ufunc_fdtri_data[0] = &ufunc_fdtri_ptr[2*0]
 * ufunc_fdtri_data[1] = &ufunc_fdtri_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_ptr[3]) = ((void *)((char *)((char *)"fdtri")));

  /* "scipy/special/_ufuncs.pyx":6651
 * ufunc_fdtri_ptr[2*1] = <void*>_func_fdtri
 * ufunc_fdtri_ptr[2*1+1] = <void*>(<char*>"fdtri")
 * ufunc_fdtri_data[0] = &ufunc_fdtri_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_fdtri_data[1] = &ufunc_fdtri_ptr[2*1]
 * fdtri = np.PyUFunc_FromFuncAndData(ufunc_fdtri_loops, ufunc_fdtri_data, ufunc_fdtri_types, 2, 3, 1, 0, "fdtri", ufunc_fdtri_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6652
 * ufunc_fdtri_ptr[2*1+1] = <void*>(<char*>"fdtri")
 * ufunc_fdtri_data[0] = &ufunc_fdtri_ptr[2*0]
 * ufunc_fdtri_data[1] = &ufunc_fdtri_ptr[2*1]             # <<<<<<<<<<<<<<
 * fdtri = np.PyUFunc_FromFuncAndData(ufunc_fdtri_loops, ufunc_fdtri_data, ufunc_fdtri_types, 2, 3, 1, 0, "fdtri", ufunc_fdtri_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6653
 * ufunc_fdtri_data[0] = &ufunc_fdtri_ptr[2*0]
 * ufunc_fdtri_data[1] = &ufunc_fdtri_ptr[2*1]
 * fdtri = np.PyUFunc_FromFuncAndData(ufunc_fdtri_loops, ufunc_fdtri_data, ufunc_fdtri_types, 2, 3, 1, 0, "fdtri", ufunc_fdtri_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_fdtridfd_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_types, 2, 3, 1, 0, ((char *)"fdtri"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtri_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6653, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_fdtri, __pyx_t_2) < 0) __PYX_ERR(1, 6653, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":6659
 * cdef void *ufunc_fdtridfd_data[2]
 * cdef char ufunc_fdtridfd_types[8]
 * cdef char *ufunc_fdtridfd_doc = (             # <<<<<<<<<<<<<<
 *     "fdtridfd(dfn, p, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_doc = ((char *)"fdtridfd(dfn, p, x)\n\nInverse to `fdtr` vs dfd\n\nFinds the F density argument dfd such that ``fdtr(dfn, dfd, x) == p``.");

  /* "scipy/special/_ufuncs.pyx":6665
 *     "\n"
 *     "Finds the F density argument dfd such that ``fdtr(dfn, dfd, x) == p``.")
 * ufunc_fdtridfd_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtridfd_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":6666
 *     "Finds the F density argument dfd such that ``fdtr(dfn, dfd, x) == p``.")
 * ufunc_fdtridfd_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_fdtridfd_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_types[0] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":6667
 * ufunc_fdtridfd_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_fdtridfd_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtridfd_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_types[1] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6668
 * ufunc_fdtridfd_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_fdtridfd_types[0] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_types[2] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6669
 * ufunc_fdtridfd_types[0] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[1] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_types[3] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6670
 * ufunc_fdtridfd_types[1] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[2] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6671
 * ufunc_fdtridfd_types[2] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[3] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6672
 * ufunc_fdtridfd_types[3] = <char>NPY_FLOAT
 * ufunc_fdtridfd_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6673
 * ufunc_fdtridfd_types[4] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_ptr[2*0] = <void*>_func_cdff4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6674
 * ufunc_fdtridfd_types[5] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_ptr[2*0] = <void*>_func_cdff4_wrap
 * ufunc_fdtridfd_ptr[2*0+1] = <void*>(<char*>"fdtridfd")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6675
 * ufunc_fdtridfd_types[6] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_ptr[2*0] = <void*>_func_cdff4_wrap             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_ptr[2*0+1] = <void*>(<char*>"fdtridfd")
 * ufunc_fdtridfd_ptr[2*1] = <void*>_func_cdff4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_ptr[0]) = ((void *)cdff4_wrap);

  /* "scipy/special/_ufuncs.pyx":6676
 * ufunc_fdtridfd_types[7] = <char>NPY_DOUBLE
 * ufunc_fdtridfd_ptr[2*0] = <void*>_func_cdff4_wrap
 * ufunc_fdtridfd_ptr[2*0+1] = <void*>(<char*>"fdtridfd")             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_ptr[2*1] = <void*>_func_cdff4_wrap
 * ufunc_fdtridfd_ptr[2*1+1] = <void*>(<char*>"fdtridfd")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_ptr[1]) = ((void *)((char *)((char *)"fdtridfd")));

  /* "scipy/special/_ufuncs.pyx":6677
 * ufunc_fdtridfd_ptr[2*0] = <void*>_func_cdff4_wrap
 * ufunc_fdtridfd_ptr[2*0+1] = <void*>(<char*>"fdtridfd")
 * ufunc_fdtridfd_ptr[2*1] = <void*>_func_cdff4_wrap             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_ptr[2*1+1] = <void*>(<char*>"fdtridfd")
 * ufunc_fdtridfd_data[0] = &ufunc_fdtridfd_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_ptr[2]) = ((void *)cdff4_wrap);

  /* "scipy/special/_ufuncs.pyx":6678
 * ufunc_fdtridfd_ptr[2*0+1] = <void*>(<char*>"fdtridfd")
 * ufunc_fdtridfd_ptr[2*1] = <void*>_func_cdff4_wrap
 * ufunc_fdtridfd_ptr[2*1+1] = <void*>(<char*>"fdtridfd")             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_data[0] = &ufunc_fdtridfd_ptr[2*0]
 * ufunc_fdtridfd_data[1] = &ufunc_fdtridfd_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_ptr[3]) = ((void *)((char *)((char *)"fdtridfd")));

  /* "scipy/special/_ufuncs.pyx":6679
 * ufunc_fdtridfd_ptr[2*1] = <void*>_func_cdff4_wrap
 * ufunc_fdtridfd_ptr[2*1+1] = <void*>(<char*>"fdtridfd")
 * ufunc_fdtridfd_data[0] = &ufunc_fdtridfd_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_fdtridfd_data[1] = &ufunc_fdtridfd_ptr[2*1]
 * fdtridfd = np.PyUFunc_FromFuncAndData(ufunc_fdtridfd_loops, ufunc_fdtridfd_data, ufunc_fdtridfd_types, 2, 3, 1, 0, "fdtridfd", ufunc_fdtridfd_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6680
 * ufunc_fdtridfd_ptr[2*1+1] = <void*>(<char*>"fdtridfd")
 * ufunc_fdtridfd_data[0] = &ufunc_fdtridfd_ptr[2*0]
 * ufunc_fdtridfd_data[1] = &ufunc_fdtridfd_ptr[2*1]             # <<<<<<<<<<<<<<
 * fdtridfd = np.PyUFunc_FromFuncAndData(ufunc_fdtridfd_loops, ufunc_fdtridfd_data, ufunc_fdtridfd_types, 2, 3, 1, 0, "fdtridfd", ufunc_fdtridfd_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6681
 * ufunc_fdtridfd_data[0] = &ufunc_fdtridfd_ptr[2*0]
 * ufunc_fdtridfd_data[1] = &ufunc_fdtridfd_ptr[2*1]
 * fdtridfd = np.PyUFunc_FromFuncAndData(ufunc_fdtridfd_loops, ufunc_fdtridfd_data, ufunc_fdtridfd_types, 2, 3, 1, 0, "fdtridfd", ufunc_fdtridfd_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_fresnel_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_types, 2, 3, 1, 0, ((char *)"fdtridfd"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_fdtridfd_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6681, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_fdtridfd, __pyx_t_2) < 0) __PYX_ERR(1, 6681, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":6687
 * cdef void *ufunc_fresnel_data[4]
 * cdef char ufunc_fresnel_types[12]
 * cdef char *ufunc_fresnel_doc = (             # <<<<<<<<<<<<<<
 *     "fresnel(z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_doc = ((char *)"fresnel(z)\n\nFresnel sin and cos integrals\n\nDefined as::\n\n    ssa = integral(sin(pi/2 * t**2), t=0..z)\n    csa = integral(cos(pi/2 * t**2), t=0..z)\n\nParameters\n----------\nz : float or complex array_like\n    Argument\n\nReturns\n-------\nssa, csa\n    Fresnel sin and cos integral values");

  /* "scipy/special/_ufuncs.pyx":6706
 *     "ssa, csa\n"
 *     "    Fresnel sin and cos integral values")
 * ufunc_fresnel_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff             # <<<<<<<<<<<<<<
 * ufunc_fresnel_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_fresnel_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_F_FF
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_f_ff);

  /* "scipy/special/_ufuncs.pyx":6707
 *     "    Fresnel sin and cos integral values")
 * ufunc_fresnel_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_fresnel_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd             # <<<<<<<<<<<<<<
 * ufunc_fresnel_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_F_FF
 * ufunc_fresnel_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_D_DD
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_d_dd);

  /* "scipy/special/_ufuncs.pyx":6708
 * ufunc_fresnel_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_fresnel_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_fresnel_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_F_FF             # <<<<<<<<<<<<<<
 * ufunc_fresnel_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_D_DD
 * ufunc_fresnel_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DD_As_F_FF);

  /* "scipy/special/_ufuncs.pyx":6709
 * ufunc_fresnel_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_fresnel_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_F_FF
 * ufunc_fresnel_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_D_DD             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[0] = <char>NPY_FLOAT
 * ufunc_fresnel_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DD_As_D_DD);

  /* "scipy/special/_ufuncs.pyx":6710
 * ufunc_fresnel_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_F_FF
 * ufunc_fresnel_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_D_DD
 * ufunc_fresnel_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[1] = <char>NPY_FLOAT
 * ufunc_fresnel_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6711
 * ufunc_fresnel_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_D_DD
 * ufunc_fresnel_types[0] = <char>NPY_FLOAT
 * ufunc_fresnel_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[2] = <char>NPY_FLOAT
 * ufunc_fresnel_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6712
 * ufunc_fresnel_types[0] = <char>NPY_FLOAT
 * ufunc_fresnel_types[1] = <char>NPY_FLOAT
 * ufunc_fresnel_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[3] = <char>NPY_DOUBLE
 * ufunc_fresnel_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6713
 * ufunc_fresnel_types[1] = <char>NPY_FLOAT
 * ufunc_fresnel_types[2] = <char>NPY_FLOAT
 * ufunc_fresnel_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[4] = <char>NPY_DOUBLE
 * ufunc_fresnel_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6714
 * ufunc_fresnel_types[2] = <char>NPY_FLOAT
 * ufunc_fresnel_types[3] = <char>NPY_DOUBLE
 * ufunc_fresnel_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[5] = <char>NPY_DOUBLE
 * ufunc_fresnel_types[6] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6715
 * ufunc_fresnel_types[3] = <char>NPY_DOUBLE
 * ufunc_fresnel_types[4] = <char>NPY_DOUBLE
 * ufunc_fresnel_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[6] = <char>NPY_CFLOAT
 * ufunc_fresnel_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6716
 * ufunc_fresnel_types[4] = <char>NPY_DOUBLE
 * ufunc_fresnel_types[5] = <char>NPY_DOUBLE
 * ufunc_fresnel_types[6] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[7] = <char>NPY_CFLOAT
 * ufunc_fresnel_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[6]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6717
 * ufunc_fresnel_types[5] = <char>NPY_DOUBLE
 * ufunc_fresnel_types[6] = <char>NPY_CFLOAT
 * ufunc_fresnel_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[8] = <char>NPY_CFLOAT
 * ufunc_fresnel_types[9] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6718
 * ufunc_fresnel_types[6] = <char>NPY_CFLOAT
 * ufunc_fresnel_types[7] = <char>NPY_CFLOAT
 * ufunc_fresnel_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[9] = <char>NPY_CDOUBLE
 * ufunc_fresnel_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6719
 * ufunc_fresnel_types[7] = <char>NPY_CFLOAT
 * ufunc_fresnel_types[8] = <char>NPY_CFLOAT
 * ufunc_fresnel_types[9] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[10] = <char>NPY_CDOUBLE
 * ufunc_fresnel_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[9]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6720
 * ufunc_fresnel_types[8] = <char>NPY_CFLOAT
 * ufunc_fresnel_types[9] = <char>NPY_CDOUBLE
 * ufunc_fresnel_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fresnel_types[11] = <char>NPY_CDOUBLE
 * ufunc_fresnel_ptr[2*0] = <void*>_func_fresnl
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6721
 * ufunc_fresnel_types[9] = <char>NPY_CDOUBLE
 * ufunc_fresnel_types[10] = <char>NPY_CDOUBLE
 * ufunc_fresnel_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_fresnel_ptr[2*0] = <void*>_func_fresnl
 * ufunc_fresnel_ptr[2*0+1] = <void*>(<char*>"fresnel")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6722
 * ufunc_fresnel_types[10] = <char>NPY_CDOUBLE
 * ufunc_fresnel_types[11] = <char>NPY_CDOUBLE
 * ufunc_fresnel_ptr[2*0] = <void*>_func_fresnl             # <<<<<<<<<<<<<<
 * ufunc_fresnel_ptr[2*0+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_ptr[2*1] = <void*>_func_fresnl
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[0]) = ((void *)fresnl);

  /* "scipy/special/_ufuncs.pyx":6723
 * ufunc_fresnel_types[11] = <char>NPY_CDOUBLE
 * ufunc_fresnel_ptr[2*0] = <void*>_func_fresnl
 * ufunc_fresnel_ptr[2*0+1] = <void*>(<char*>"fresnel")             # <<<<<<<<<<<<<<
 * ufunc_fresnel_ptr[2*1] = <void*>_func_fresnl
 * ufunc_fresnel_ptr[2*1+1] = <void*>(<char*>"fresnel")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[1]) = ((void *)((char *)((char *)"fresnel")));

  /* "scipy/special/_ufuncs.pyx":6724
 * ufunc_fresnel_ptr[2*0] = <void*>_func_fresnl
 * ufunc_fresnel_ptr[2*0+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_ptr[2*1] = <void*>_func_fresnl             # <<<<<<<<<<<<<<
 * ufunc_fresnel_ptr[2*1+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_ptr[2*2] = <void*>_func_cfresnl_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[2]) = ((void *)fresnl);

  /* "scipy/special/_ufuncs.pyx":6725
 * ufunc_fresnel_ptr[2*0+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_ptr[2*1] = <void*>_func_fresnl
 * ufunc_fresnel_ptr[2*1+1] = <void*>(<char*>"fresnel")             # <<<<<<<<<<<<<<
 * ufunc_fresnel_ptr[2*2] = <void*>_func_cfresnl_wrap
 * ufunc_fresnel_ptr[2*2+1] = <void*>(<char*>"fresnel")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[3]) = ((void *)((char *)((char *)"fresnel")));

  /* "scipy/special/_ufuncs.pyx":6726
 * ufunc_fresnel_ptr[2*1] = <void*>_func_fresnl
 * ufunc_fresnel_ptr[2*1+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_ptr[2*2] = <void*>_func_cfresnl_wrap             # <<<<<<<<<<<<<<
 * ufunc_fresnel_ptr[2*2+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_ptr[2*3] = <void*>_func_cfresnl_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[4]) = ((void *)cfresnl_wrap);

  /* "scipy/special/_ufuncs.pyx":6727
 * ufunc_fresnel_ptr[2*1+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_ptr[2*2] = <void*>_func_cfresnl_wrap
 * ufunc_fresnel_ptr[2*2+1] = <void*>(<char*>"fresnel")             # <<<<<<<<<<<<<<
 * ufunc_fresnel_ptr[2*3] = <void*>_func_cfresnl_wrap
 * ufunc_fresnel_ptr[2*3+1] = <void*>(<char*>"fresnel")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[5]) = ((void *)((char *)((char *)"fresnel")));

  /* "scipy/special/_ufuncs.pyx":6728
 * ufunc_fresnel_ptr[2*2] = <void*>_func_cfresnl_wrap
 * ufunc_fresnel_ptr[2*2+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_ptr[2*3] = <void*>_func_cfresnl_wrap             # <<<<<<<<<<<<<<
 * ufunc_fresnel_ptr[2*3+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_data[0] = &ufunc_fresnel_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[6]) = ((void *)cfresnl_wrap);

  /* "scipy/special/_ufuncs.pyx":6729
 * ufunc_fresnel_ptr[2*2+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_ptr[2*3] = <void*>_func_cfresnl_wrap
 * ufunc_fresnel_ptr[2*3+1] = <void*>(<char*>"fresnel")             # <<<<<<<<<<<<<<
 * ufunc_fresnel_data[0] = &ufunc_fresnel_ptr[2*0]
 * ufunc_fresnel_data[1] = &ufunc_fresnel_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[7]) = ((void *)((char *)((char *)"fresnel")));

  /* "scipy/special/_ufuncs.pyx":6730
 * ufunc_fresnel_ptr[2*3] = <void*>_func_cfresnl_wrap
 * ufunc_fresnel_ptr[2*3+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_data[0] = &ufunc_fresnel_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_fresnel_data[1] = &ufunc_fresnel_ptr[2*1]
 * ufunc_fresnel_data[2] = &ufunc_fresnel_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6731
 * ufunc_fresnel_ptr[2*3+1] = <void*>(<char*>"fresnel")
 * ufunc_fresnel_data[0] = &ufunc_fresnel_ptr[2*0]
 * ufunc_fresnel_data[1] = &ufunc_fresnel_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_fresnel_data[2] = &ufunc_fresnel_ptr[2*2]
 * ufunc_fresnel_data[3] = &ufunc_fresnel_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6732
 * ufunc_fresnel_data[0] = &ufunc_fresnel_ptr[2*0]
 * ufunc_fresnel_data[1] = &ufunc_fresnel_ptr[2*1]
 * ufunc_fresnel_data[2] = &ufunc_fresnel_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_fresnel_data[3] = &ufunc_fresnel_ptr[2*3]
 * fresnel = np.PyUFunc_FromFuncAndData(ufunc_fresnel_loops, ufunc_fresnel_data, ufunc_fresnel_types, 4, 1, 2, 0, "fresnel", ufunc_fresnel_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":6733
 * ufunc_fresnel_data[1] = &ufunc_fresnel_ptr[2*1]
 * ufunc_fresnel_data[2] = &ufunc_fresnel_ptr[2*2]
 * ufunc_fresnel_data[3] = &ufunc_fresnel_ptr[2*3]             # <<<<<<<<<<<<<<
 * fresnel = np.PyUFunc_FromFuncAndData(ufunc_fresnel_loops, ufunc_fresnel_data, ufunc_fresnel_types, 4, 1, 2, 0, "fresnel", ufunc_fresnel_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":6734
 * ufunc_fresnel_data[2] = &ufunc_fresnel_ptr[2*2]
 * ufunc_fresnel_data[3] = &ufunc_fresnel_ptr[2*3]
 * fresnel = np.PyUFunc_FromFuncAndData(ufunc_fresnel_loops, ufunc_fresnel_data, ufunc_fresnel_types, 4, 1, 2, 0, "fresnel", ufunc_fresnel_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gamma_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_types, 4, 1, 2, 0, ((char *)"fresnel"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_fresnel_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6734, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_fresnel, __pyx_t_2) < 0) __PYX_ERR(1, 6734, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":6740
 * cdef void *ufunc_gamma_data[4]
 * cdef char ufunc_gamma_types[8]
 * cdef char *ufunc_gamma_doc = (             # <<<<<<<<<<<<<<
 *     "gamma(z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_doc = ((char *)"gamma(z)\n\nGamma function.\n\n.. math::\n\n      \\Gamma(z) = \\int_0^\\infty x^{z-1} e^{-x} dx = (z - 1)!\n\nThe gamma function is often referred to as the generalized\nfactorial since ``z*gamma(z) = gamma(z+1)`` and ``gamma(n+1) =\nn!`` for natural number *n*.\n\nParameters\n----------\nz : float or complex array_like\n\nReturns\n-------\nfloat or complex\n    The value(s) of gamma(z)\n\nExamples\n--------\n>>> from scipy.special import gamma, factorial\n\n>>> gamma([0, 0.5, 1, 5])\narray([         inf,   1.77245385,   1.        ,  24.        ])\n\n>>> z = 2.5 + 1j\n>>> gamma(z)\n(0.77476210455108352+0.70763120437959293j)\n>>> gamma(z+1), z*gamma(z)  # Recurrence property\n((1.2292740569981171+2.5438401155000685j),\n (1.2292740569981158+2.5438401155000658j))\n\n>>> gamma(0.5)**2  # gamma(0.5) = sqrt(pi)\n3.1415926535897927\n\nPlot gamma(x) for real x\n\n>>> x = np.linspace(-3.5, 5.5, 2251)\n>>> y = gamma(x)\n\n>>> import matplotlib.pyplot as plt\n>>> plt.plot(x, y, 'b', alpha=0.6, label='gamma(x)')\n>>> k = np.arange(1, 7)\n>>> plt.plot(k, factorial(k-1), 'k*', alpha=0.6,\n...          label='(x-1)!, x = 1, 2, ...')\n>>> plt.xlim(-3.5, 5.5)\n>>> plt.ylim(-10, 25)\n>>> plt.grid()\n>>> plt.xlabel('x')\n>>> plt.legend(loc='lower right')\n>>> plt.show()");

  /* "scipy/special/_ufuncs.pyx":6795
 *     ">>> plt.legend(loc='lower right')\n"
 *     ">>> plt.show()")
 * ufunc_gamma_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_gamma_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_gamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":6796
 *     ">>> plt.show()")
 * ufunc_gamma_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_gamma_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_gamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_gamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":6797
 * ufunc_gamma_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_gamma_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_gamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_gamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_gamma_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":6798
 * ufunc_gamma_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_gamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_gamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_gamma_types[0] = <char>NPY_FLOAT
 * ufunc_gamma_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":6799
 * ufunc_gamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_gamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_gamma_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gamma_types[1] = <char>NPY_FLOAT
 * ufunc_gamma_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6800
 * ufunc_gamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_gamma_types[0] = <char>NPY_FLOAT
 * ufunc_gamma_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gamma_types[2] = <char>NPY_DOUBLE
 * ufunc_gamma_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6801
 * ufunc_gamma_types[0] = <char>NPY_FLOAT
 * ufunc_gamma_types[1] = <char>NPY_FLOAT
 * ufunc_gamma_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gamma_types[3] = <char>NPY_DOUBLE
 * ufunc_gamma_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6802
 * ufunc_gamma_types[1] = <char>NPY_FLOAT
 * ufunc_gamma_types[2] = <char>NPY_DOUBLE
 * ufunc_gamma_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gamma_types[4] = <char>NPY_CFLOAT
 * ufunc_gamma_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6803
 * ufunc_gamma_types[2] = <char>NPY_DOUBLE
 * ufunc_gamma_types[3] = <char>NPY_DOUBLE
 * ufunc_gamma_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_gamma_types[5] = <char>NPY_CFLOAT
 * ufunc_gamma_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6804
 * ufunc_gamma_types[3] = <char>NPY_DOUBLE
 * ufunc_gamma_types[4] = <char>NPY_CFLOAT
 * ufunc_gamma_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_gamma_types[6] = <char>NPY_CDOUBLE
 * ufunc_gamma_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":6805
 * ufunc_gamma_types[4] = <char>NPY_CFLOAT
 * ufunc_gamma_types[5] = <char>NPY_CFLOAT
 * ufunc_gamma_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gamma_types[7] = <char>NPY_CDOUBLE
 * ufunc_gamma_ptr[2*0] = <void*>_func_Gamma
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6806
 * ufunc_gamma_types[5] = <char>NPY_CFLOAT
 * ufunc_gamma_types[6] = <char>NPY_CDOUBLE
 * ufunc_gamma_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gamma_ptr[2*0] = <void*>_func_Gamma
 * ufunc_gamma_ptr[2*0+1] = <void*>(<char*>"gamma")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":6807
 * ufunc_gamma_types[6] = <char>NPY_CDOUBLE
 * ufunc_gamma_types[7] = <char>NPY_CDOUBLE
 * ufunc_gamma_ptr[2*0] = <void*>_func_Gamma             # <<<<<<<<<<<<<<
 * ufunc_gamma_ptr[2*0+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_ptr[2*1] = <void*>_func_Gamma
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[0]) = ((void *)Gamma);

  /* "scipy/special/_ufuncs.pyx":6808
 * ufunc_gamma_types[7] = <char>NPY_CDOUBLE
 * ufunc_gamma_ptr[2*0] = <void*>_func_Gamma
 * ufunc_gamma_ptr[2*0+1] = <void*>(<char*>"gamma")             # <<<<<<<<<<<<<<
 * ufunc_gamma_ptr[2*1] = <void*>_func_Gamma
 * ufunc_gamma_ptr[2*1+1] = <void*>(<char*>"gamma")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[1]) = ((void *)((char *)((char *)"gamma")));

  /* "scipy/special/_ufuncs.pyx":6809
 * ufunc_gamma_ptr[2*0] = <void*>_func_Gamma
 * ufunc_gamma_ptr[2*0+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_ptr[2*1] = <void*>_func_Gamma             # <<<<<<<<<<<<<<
 * ufunc_gamma_ptr[2*1+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_ptr[2*2] = <void*>_func_cgamma
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[2]) = ((void *)Gamma);

  /* "scipy/special/_ufuncs.pyx":6810
 * ufunc_gamma_ptr[2*0+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_ptr[2*1] = <void*>_func_Gamma
 * ufunc_gamma_ptr[2*1+1] = <void*>(<char*>"gamma")             # <<<<<<<<<<<<<<
 * ufunc_gamma_ptr[2*2] = <void*>_func_cgamma
 * ufunc_gamma_ptr[2*2+1] = <void*>(<char*>"gamma")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[3]) = ((void *)((char *)((char *)"gamma")));

  /* "scipy/special/_ufuncs.pyx":6811
 * ufunc_gamma_ptr[2*1] = <void*>_func_Gamma
 * ufunc_gamma_ptr[2*1+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_ptr[2*2] = <void*>_func_cgamma             # <<<<<<<<<<<<<<
 * ufunc_gamma_ptr[2*2+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_ptr[2*3] = <void*>_func_cgamma
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[4]) = ((void *)__pyx_f_5scipy_7special_9_loggamma_cgamma);

  /* "scipy/special/_ufuncs.pyx":6812
 * ufunc_gamma_ptr[2*1+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_ptr[2*2] = <void*>_func_cgamma
 * ufunc_gamma_ptr[2*2+1] = <void*>(<char*>"gamma")             # <<<<<<<<<<<<<<
 * ufunc_gamma_ptr[2*3] = <void*>_func_cgamma
 * ufunc_gamma_ptr[2*3+1] = <void*>(<char*>"gamma")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[5]) = ((void *)((char *)((char *)"gamma")));

  /* "scipy/special/_ufuncs.pyx":6813
 * ufunc_gamma_ptr[2*2] = <void*>_func_cgamma
 * ufunc_gamma_ptr[2*2+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_ptr[2*3] = <void*>_func_cgamma             # <<<<<<<<<<<<<<
 * ufunc_gamma_ptr[2*3+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_data[0] = &ufunc_gamma_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[6]) = ((void *)__pyx_f_5scipy_7special_9_loggamma_cgamma);

  /* "scipy/special/_ufuncs.pyx":6814
 * ufunc_gamma_ptr[2*2+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_ptr[2*3] = <void*>_func_cgamma
 * ufunc_gamma_ptr[2*3+1] = <void*>(<char*>"gamma")             # <<<<<<<<<<<<<<
 * ufunc_gamma_data[0] = &ufunc_gamma_ptr[2*0]
 * ufunc_gamma_data[1] = &ufunc_gamma_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[7]) = ((void *)((char *)((char *)"gamma")));

  /* "scipy/special/_ufuncs.pyx":6815
 * ufunc_gamma_ptr[2*3] = <void*>_func_cgamma
 * ufunc_gamma_ptr[2*3+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_data[0] = &ufunc_gamma_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gamma_data[1] = &ufunc_gamma_ptr[2*1]
 * ufunc_gamma_data[2] = &ufunc_gamma_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6816
 * ufunc_gamma_ptr[2*3+1] = <void*>(<char*>"gamma")
 * ufunc_gamma_data[0] = &ufunc_gamma_ptr[2*0]
 * ufunc_gamma_data[1] = &ufunc_gamma_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_gamma_data[2] = &ufunc_gamma_ptr[2*2]
 * ufunc_gamma_data[3] = &ufunc_gamma_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6817
 * ufunc_gamma_data[0] = &ufunc_gamma_ptr[2*0]
 * ufunc_gamma_data[1] = &ufunc_gamma_ptr[2*1]
 * ufunc_gamma_data[2] = &ufunc_gamma_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_gamma_data[3] = &ufunc_gamma_ptr[2*3]
 * gamma = np.PyUFunc_FromFuncAndData(ufunc_gamma_loops, ufunc_gamma_data, ufunc_gamma_types, 4, 1, 1, 0, "gamma", ufunc_gamma_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":6818
 * ufunc_gamma_data[1] = &ufunc_gamma_ptr[2*1]
 * ufunc_gamma_data[2] = &ufunc_gamma_ptr[2*2]
 * ufunc_gamma_data[3] = &ufunc_gamma_ptr[2*3]             # <<<<<<<<<<<<<<
 * gamma = np.PyUFunc_FromFuncAndData(ufunc_gamma_loops, ufunc_gamma_data, ufunc_gamma_types, 4, 1, 1, 0, "gamma", ufunc_gamma_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":6819
 * ufunc_gamma_data[2] = &ufunc_gamma_ptr[2*2]
 * ufunc_gamma_data[3] = &ufunc_gamma_ptr[2*3]
 * gamma = np.PyUFunc_FromFuncAndData(ufunc_gamma_loops, ufunc_gamma_data, ufunc_gamma_types, 4, 1, 1, 0, "gamma", ufunc_gamma_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gammainc_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_types, 4, 1, 1, 0, ((char *)"gamma"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_gamma_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6819, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gamma, __pyx_t_2) < 0) __PYX_ERR(1, 6819, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":6825
 * cdef void *ufunc_gammainc_data[2]
 * cdef char ufunc_gammainc_types[6]
 * cdef char *ufunc_gammainc_doc = (             # <<<<<<<<<<<<<<
 *     "gammainc(a, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_doc = ((char *)"gammainc(a, x)\n\nRegularized lower incomplete gamma function.\n\nDefined as\n\n.. math::\n\n    \\frac{1}{\\Gamma(a)} \\int_0^x t^{a - 1}e^{-t} dt\n\nfor :math:`a > 0` and :math:`x \\geq 0`. The function satisfies the\nrelation ``gammainc(a, x) + gammaincc(a, x) = 1`` where\n`gammaincc` is the regularized upper incomplete gamma function.\n\nNotes\n-----\nThe implementation largely follows that of [1]_.\n\nSee also\n--------\ngammaincc : regularized upper incomplete gamma function\ngammaincinv : inverse to ``gammainc`` versus ``x``\ngammainccinv : inverse to ``gammaincc`` versus ``x``\n\nReferences\n----------\n.. [1] Maddock et. al., \"Incomplete Gamma Functions\",\n   https://www.boost.org/doc/libs/1_61_0/libs/math/doc/html/math_toolkit/sf_gamma/igamma.html");

  /* "scipy/special/_ufuncs.pyx":6854
 *     ".. [1] Maddock et. al., \"Incomplete Gamma Functions\",\n"
 *     "   https://www.boost.org/doc/libs/1_61_0/libs/math/doc/html/math_toolkit/sf_gamma/igamma.html")
 * ufunc_gammainc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_gammainc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammainc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":6855
 *     "   https://www.boost.org/doc/libs/1_61_0/libs/math/doc/html/math_toolkit/sf_gamma/igamma.html")
 * ufunc_gammainc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_gammainc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_gammainc_types[0] = <char>NPY_FLOAT
 * ufunc_gammainc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":6856
 * ufunc_gammainc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_gammainc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammainc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammainc_types[1] = <char>NPY_FLOAT
 * ufunc_gammainc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6857
 * ufunc_gammainc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammainc_types[0] = <char>NPY_FLOAT
 * ufunc_gammainc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammainc_types[2] = <char>NPY_FLOAT
 * ufunc_gammainc_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6858
 * ufunc_gammainc_types[0] = <char>NPY_FLOAT
 * ufunc_gammainc_types[1] = <char>NPY_FLOAT
 * ufunc_gammainc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammainc_types[3] = <char>NPY_DOUBLE
 * ufunc_gammainc_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6859
 * ufunc_gammainc_types[1] = <char>NPY_FLOAT
 * ufunc_gammainc_types[2] = <char>NPY_FLOAT
 * ufunc_gammainc_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammainc_types[4] = <char>NPY_DOUBLE
 * ufunc_gammainc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6860
 * ufunc_gammainc_types[2] = <char>NPY_FLOAT
 * ufunc_gammainc_types[3] = <char>NPY_DOUBLE
 * ufunc_gammainc_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammainc_types[5] = <char>NPY_DOUBLE
 * ufunc_gammainc_ptr[2*0] = <void*>_func_igam
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6861
 * ufunc_gammainc_types[3] = <char>NPY_DOUBLE
 * ufunc_gammainc_types[4] = <char>NPY_DOUBLE
 * ufunc_gammainc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammainc_ptr[2*0] = <void*>_func_igam
 * ufunc_gammainc_ptr[2*0+1] = <void*>(<char*>"gammainc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6862
 * ufunc_gammainc_types[4] = <char>NPY_DOUBLE
 * ufunc_gammainc_types[5] = <char>NPY_DOUBLE
 * ufunc_gammainc_ptr[2*0] = <void*>_func_igam             # <<<<<<<<<<<<<<
 * ufunc_gammainc_ptr[2*0+1] = <void*>(<char*>"gammainc")
 * ufunc_gammainc_ptr[2*1] = <void*>_func_igam
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_ptr[0]) = ((void *)igam);

  /* "scipy/special/_ufuncs.pyx":6863
 * ufunc_gammainc_types[5] = <char>NPY_DOUBLE
 * ufunc_gammainc_ptr[2*0] = <void*>_func_igam
 * ufunc_gammainc_ptr[2*0+1] = <void*>(<char*>"gammainc")             # <<<<<<<<<<<<<<
 * ufunc_gammainc_ptr[2*1] = <void*>_func_igam
 * ufunc_gammainc_ptr[2*1+1] = <void*>(<char*>"gammainc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_ptr[1]) = ((void *)((char *)((char *)"gammainc")));

  /* "scipy/special/_ufuncs.pyx":6864
 * ufunc_gammainc_ptr[2*0] = <void*>_func_igam
 * ufunc_gammainc_ptr[2*0+1] = <void*>(<char*>"gammainc")
 * ufunc_gammainc_ptr[2*1] = <void*>_func_igam             # <<<<<<<<<<<<<<
 * ufunc_gammainc_ptr[2*1+1] = <void*>(<char*>"gammainc")
 * ufunc_gammainc_data[0] = &ufunc_gammainc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_ptr[2]) = ((void *)igam);

  /* "scipy/special/_ufuncs.pyx":6865
 * ufunc_gammainc_ptr[2*0+1] = <void*>(<char*>"gammainc")
 * ufunc_gammainc_ptr[2*1] = <void*>_func_igam
 * ufunc_gammainc_ptr[2*1+1] = <void*>(<char*>"gammainc")             # <<<<<<<<<<<<<<
 * ufunc_gammainc_data[0] = &ufunc_gammainc_ptr[2*0]
 * ufunc_gammainc_data[1] = &ufunc_gammainc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_ptr[3]) = ((void *)((char *)((char *)"gammainc")));

  /* "scipy/special/_ufuncs.pyx":6866
 * ufunc_gammainc_ptr[2*1] = <void*>_func_igam
 * ufunc_gammainc_ptr[2*1+1] = <void*>(<char*>"gammainc")
 * ufunc_gammainc_data[0] = &ufunc_gammainc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gammainc_data[1] = &ufunc_gammainc_ptr[2*1]
 * gammainc = np.PyUFunc_FromFuncAndData(ufunc_gammainc_loops, ufunc_gammainc_data, ufunc_gammainc_types, 2, 2, 1, 0, "gammainc", ufunc_gammainc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6867
 * ufunc_gammainc_ptr[2*1+1] = <void*>(<char*>"gammainc")
 * ufunc_gammainc_data[0] = &ufunc_gammainc_ptr[2*0]
 * ufunc_gammainc_data[1] = &ufunc_gammainc_ptr[2*1]             # <<<<<<<<<<<<<<
 * gammainc = np.PyUFunc_FromFuncAndData(ufunc_gammainc_loops, ufunc_gammainc_data, ufunc_gammainc_types, 2, 2, 1, 0, "gammainc", ufunc_gammainc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6868
 * ufunc_gammainc_data[0] = &ufunc_gammainc_ptr[2*0]
 * ufunc_gammainc_data[1] = &ufunc_gammainc_ptr[2*1]
 * gammainc = np.PyUFunc_FromFuncAndData(ufunc_gammainc_loops, ufunc_gammainc_data, ufunc_gammainc_types, 2, 2, 1, 0, "gammainc", ufunc_gammainc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gammaincc_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_types, 2, 2, 1, 0, ((char *)"gammainc"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainc_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6868, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gammainc, __pyx_t_2) < 0) __PYX_ERR(1, 6868, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":6874
 * cdef void *ufunc_gammaincc_data[2]
 * cdef char ufunc_gammaincc_types[6]
 * cdef char *ufunc_gammaincc_doc = (             # <<<<<<<<<<<<<<
 *     "gammaincc(a, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_doc = ((char *)"gammaincc(a, x)\n\nRegularized upper incomplete gamma function.\n\nDefined as\n\n.. math::\n\n    \\frac{1}{\\Gamma(a)} \\int_x^\\infty t^{a - 1}e^{-t} dt\n\nfor :math:`a > 0` and :math:`x \\geq 0`. The function satisfies the\nrelation ``gammainc(a, x) + gammaincc(a, x) = 1`` where `gammainc`\nis the regularized lower incomplete gamma function.\n\nNotes\n-----\nThe implementation largely follows that of [1]_.\n\nSee also\n--------\ngammainc : regularized lower incomplete gamma function\ngammaincinv : inverse to ``gammainc`` versus ``x``\ngammainccinv : inverse to ``gammaincc`` versus ``x``\n\nReferences\n----------\n.. [1] Maddock et. al., \"Incomplete Gamma Functions\",\n   https://www.boost.org/doc/libs/1_61_0/libs/math/doc/html/math_toolkit/sf_gamma/igamma.html");

  /* "scipy/special/_ufuncs.pyx":6903
 *     ".. [1] Maddock et. al., \"Incomplete Gamma Functions\",\n"
 *     "   https://www.boost.org/doc/libs/1_61_0/libs/math/doc/html/math_toolkit/sf_gamma/igamma.html")
 * ufunc_gammaincc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammaincc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":6904
 *     "   https://www.boost.org/doc/libs/1_61_0/libs/math/doc/html/math_toolkit/sf_gamma/igamma.html")
 * ufunc_gammaincc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_gammaincc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_types[0] = <char>NPY_FLOAT
 * ufunc_gammaincc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":6905
 * ufunc_gammaincc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_gammaincc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammaincc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_types[1] = <char>NPY_FLOAT
 * ufunc_gammaincc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6906
 * ufunc_gammaincc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammaincc_types[0] = <char>NPY_FLOAT
 * ufunc_gammaincc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_types[2] = <char>NPY_FLOAT
 * ufunc_gammaincc_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6907
 * ufunc_gammaincc_types[0] = <char>NPY_FLOAT
 * ufunc_gammaincc_types[1] = <char>NPY_FLOAT
 * ufunc_gammaincc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_types[3] = <char>NPY_DOUBLE
 * ufunc_gammaincc_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6908
 * ufunc_gammaincc_types[1] = <char>NPY_FLOAT
 * ufunc_gammaincc_types[2] = <char>NPY_FLOAT
 * ufunc_gammaincc_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_types[4] = <char>NPY_DOUBLE
 * ufunc_gammaincc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6909
 * ufunc_gammaincc_types[2] = <char>NPY_FLOAT
 * ufunc_gammaincc_types[3] = <char>NPY_DOUBLE
 * ufunc_gammaincc_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_types[5] = <char>NPY_DOUBLE
 * ufunc_gammaincc_ptr[2*0] = <void*>_func_igamc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6910
 * ufunc_gammaincc_types[3] = <char>NPY_DOUBLE
 * ufunc_gammaincc_types[4] = <char>NPY_DOUBLE
 * ufunc_gammaincc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_ptr[2*0] = <void*>_func_igamc
 * ufunc_gammaincc_ptr[2*0+1] = <void*>(<char*>"gammaincc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6911
 * ufunc_gammaincc_types[4] = <char>NPY_DOUBLE
 * ufunc_gammaincc_types[5] = <char>NPY_DOUBLE
 * ufunc_gammaincc_ptr[2*0] = <void*>_func_igamc             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_ptr[2*0+1] = <void*>(<char*>"gammaincc")
 * ufunc_gammaincc_ptr[2*1] = <void*>_func_igamc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_ptr[0]) = ((void *)igamc);

  /* "scipy/special/_ufuncs.pyx":6912
 * ufunc_gammaincc_types[5] = <char>NPY_DOUBLE
 * ufunc_gammaincc_ptr[2*0] = <void*>_func_igamc
 * ufunc_gammaincc_ptr[2*0+1] = <void*>(<char*>"gammaincc")             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_ptr[2*1] = <void*>_func_igamc
 * ufunc_gammaincc_ptr[2*1+1] = <void*>(<char*>"gammaincc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_ptr[1]) = ((void *)((char *)((char *)"gammaincc")));

  /* "scipy/special/_ufuncs.pyx":6913
 * ufunc_gammaincc_ptr[2*0] = <void*>_func_igamc
 * ufunc_gammaincc_ptr[2*0+1] = <void*>(<char*>"gammaincc")
 * ufunc_gammaincc_ptr[2*1] = <void*>_func_igamc             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_ptr[2*1+1] = <void*>(<char*>"gammaincc")
 * ufunc_gammaincc_data[0] = &ufunc_gammaincc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_ptr[2]) = ((void *)igamc);

  /* "scipy/special/_ufuncs.pyx":6914
 * ufunc_gammaincc_ptr[2*0+1] = <void*>(<char*>"gammaincc")
 * ufunc_gammaincc_ptr[2*1] = <void*>_func_igamc
 * ufunc_gammaincc_ptr[2*1+1] = <void*>(<char*>"gammaincc")             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_data[0] = &ufunc_gammaincc_ptr[2*0]
 * ufunc_gammaincc_data[1] = &ufunc_gammaincc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_ptr[3]) = ((void *)((char *)((char *)"gammaincc")));

  /* "scipy/special/_ufuncs.pyx":6915
 * ufunc_gammaincc_ptr[2*1] = <void*>_func_igamc
 * ufunc_gammaincc_ptr[2*1+1] = <void*>(<char*>"gammaincc")
 * ufunc_gammaincc_data[0] = &ufunc_gammaincc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gammaincc_data[1] = &ufunc_gammaincc_ptr[2*1]
 * gammaincc = np.PyUFunc_FromFuncAndData(ufunc_gammaincc_loops, ufunc_gammaincc_data, ufunc_gammaincc_types, 2, 2, 1, 0, "gammaincc", ufunc_gammaincc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6916
 * ufunc_gammaincc_ptr[2*1+1] = <void*>(<char*>"gammaincc")
 * ufunc_gammaincc_data[0] = &ufunc_gammaincc_ptr[2*0]
 * ufunc_gammaincc_data[1] = &ufunc_gammaincc_ptr[2*1]             # <<<<<<<<<<<<<<
 * gammaincc = np.PyUFunc_FromFuncAndData(ufunc_gammaincc_loops, ufunc_gammaincc_data, ufunc_gammaincc_types, 2, 2, 1, 0, "gammaincc", ufunc_gammaincc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6917
 * ufunc_gammaincc_data[0] = &ufunc_gammaincc_ptr[2*0]
 * ufunc_gammaincc_data[1] = &ufunc_gammaincc_ptr[2*1]
 * gammaincc = np.PyUFunc_FromFuncAndData(ufunc_gammaincc_loops, ufunc_gammaincc_data, ufunc_gammaincc_types, 2, 2, 1, 0, "gammaincc", ufunc_gammaincc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gammainccinv_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_types, 2, 2, 1, 0, ((char *)"gammaincc"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincc_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6917, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gammaincc, __pyx_t_2) < 0) __PYX_ERR(1, 6917, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":6923
 * cdef void *ufunc_gammainccinv_data[2]
 * cdef char ufunc_gammainccinv_types[6]
 * cdef char *ufunc_gammainccinv_doc = (             # <<<<<<<<<<<<<<
 *     "gammainccinv(a, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_doc = ((char *)"gammainccinv(a, y)\n\nInverse to `gammaincc`\n\nReturns `x` such that ``gammaincc(a, x) == y``.");

  /* "scipy/special/_ufuncs.pyx":6929
 *     "\n"
 *     "Returns `x` such that ``gammaincc(a, x) == y``.")
 * ufunc_gammainccinv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammainccinv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":6930
 *     "Returns `x` such that ``gammaincc(a, x) == y``.")
 * ufunc_gammainccinv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_gammainccinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_types[0] = <char>NPY_FLOAT
 * ufunc_gammainccinv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":6931
 * ufunc_gammainccinv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_gammainccinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammainccinv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_types[1] = <char>NPY_FLOAT
 * ufunc_gammainccinv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6932
 * ufunc_gammainccinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammainccinv_types[0] = <char>NPY_FLOAT
 * ufunc_gammainccinv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_types[2] = <char>NPY_FLOAT
 * ufunc_gammainccinv_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6933
 * ufunc_gammainccinv_types[0] = <char>NPY_FLOAT
 * ufunc_gammainccinv_types[1] = <char>NPY_FLOAT
 * ufunc_gammainccinv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_types[3] = <char>NPY_DOUBLE
 * ufunc_gammainccinv_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6934
 * ufunc_gammainccinv_types[1] = <char>NPY_FLOAT
 * ufunc_gammainccinv_types[2] = <char>NPY_FLOAT
 * ufunc_gammainccinv_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_types[4] = <char>NPY_DOUBLE
 * ufunc_gammainccinv_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6935
 * ufunc_gammainccinv_types[2] = <char>NPY_FLOAT
 * ufunc_gammainccinv_types[3] = <char>NPY_DOUBLE
 * ufunc_gammainccinv_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_types[5] = <char>NPY_DOUBLE
 * ufunc_gammainccinv_ptr[2*0] = <void*>_func_igamci
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6936
 * ufunc_gammainccinv_types[3] = <char>NPY_DOUBLE
 * ufunc_gammainccinv_types[4] = <char>NPY_DOUBLE
 * ufunc_gammainccinv_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_ptr[2*0] = <void*>_func_igamci
 * ufunc_gammainccinv_ptr[2*0+1] = <void*>(<char*>"gammainccinv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6937
 * ufunc_gammainccinv_types[4] = <char>NPY_DOUBLE
 * ufunc_gammainccinv_types[5] = <char>NPY_DOUBLE
 * ufunc_gammainccinv_ptr[2*0] = <void*>_func_igamci             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_ptr[2*0+1] = <void*>(<char*>"gammainccinv")
 * ufunc_gammainccinv_ptr[2*1] = <void*>_func_igamci
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_ptr[0]) = ((void *)igamci);

  /* "scipy/special/_ufuncs.pyx":6938
 * ufunc_gammainccinv_types[5] = <char>NPY_DOUBLE
 * ufunc_gammainccinv_ptr[2*0] = <void*>_func_igamci
 * ufunc_gammainccinv_ptr[2*0+1] = <void*>(<char*>"gammainccinv")             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_ptr[2*1] = <void*>_func_igamci
 * ufunc_gammainccinv_ptr[2*1+1] = <void*>(<char*>"gammainccinv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_ptr[1]) = ((void *)((char *)((char *)"gammainccinv")));

  /* "scipy/special/_ufuncs.pyx":6939
 * ufunc_gammainccinv_ptr[2*0] = <void*>_func_igamci
 * ufunc_gammainccinv_ptr[2*0+1] = <void*>(<char*>"gammainccinv")
 * ufunc_gammainccinv_ptr[2*1] = <void*>_func_igamci             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_ptr[2*1+1] = <void*>(<char*>"gammainccinv")
 * ufunc_gammainccinv_data[0] = &ufunc_gammainccinv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_ptr[2]) = ((void *)igamci);

  /* "scipy/special/_ufuncs.pyx":6940
 * ufunc_gammainccinv_ptr[2*0+1] = <void*>(<char*>"gammainccinv")
 * ufunc_gammainccinv_ptr[2*1] = <void*>_func_igamci
 * ufunc_gammainccinv_ptr[2*1+1] = <void*>(<char*>"gammainccinv")             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_data[0] = &ufunc_gammainccinv_ptr[2*0]
 * ufunc_gammainccinv_data[1] = &ufunc_gammainccinv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_ptr[3]) = ((void *)((char *)((char *)"gammainccinv")));

  /* "scipy/special/_ufuncs.pyx":6941
 * ufunc_gammainccinv_ptr[2*1] = <void*>_func_igamci
 * ufunc_gammainccinv_ptr[2*1+1] = <void*>(<char*>"gammainccinv")
 * ufunc_gammainccinv_data[0] = &ufunc_gammainccinv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gammainccinv_data[1] = &ufunc_gammainccinv_ptr[2*1]
 * gammainccinv = np.PyUFunc_FromFuncAndData(ufunc_gammainccinv_loops, ufunc_gammainccinv_data, ufunc_gammainccinv_types, 2, 2, 1, 0, "gammainccinv", ufunc_gammainccinv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6942
 * ufunc_gammainccinv_ptr[2*1+1] = <void*>(<char*>"gammainccinv")
 * ufunc_gammainccinv_data[0] = &ufunc_gammainccinv_ptr[2*0]
 * ufunc_gammainccinv_data[1] = &ufunc_gammainccinv_ptr[2*1]             # <<<<<<<<<<<<<<
 * gammainccinv = np.PyUFunc_FromFuncAndData(ufunc_gammainccinv_loops, ufunc_gammainccinv_data, ufunc_gammainccinv_types, 2, 2, 1, 0, "gammainccinv", ufunc_gammainccinv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6943
 * ufunc_gammainccinv_data[0] = &ufunc_gammainccinv_ptr[2*0]
 * ufunc_gammainccinv_data[1] = &ufunc_gammainccinv_ptr[2*1]
 * gammainccinv = np.PyUFunc_FromFuncAndData(ufunc_gammainccinv_loops, ufunc_gammainccinv_data, ufunc_gammainccinv_types, 2, 2, 1, 0, "gammainccinv", ufunc_gammainccinv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gammaincinv_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_types, 2, 2, 1, 0, ((char *)"gammainccinv"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammainccinv_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6943, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gammainccinv, __pyx_t_2) < 0) __PYX_ERR(1, 6943, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":6949
 * cdef void *ufunc_gammaincinv_data[2]
 * cdef char ufunc_gammaincinv_types[6]
 * cdef char *ufunc_gammaincinv_doc = (             # <<<<<<<<<<<<<<
 *     "gammaincinv(a, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_doc = ((char *)"gammaincinv(a, y)\n\nInverse to `gammainc`\n\nReturns `x` such that ``gammainc(a, x) = y``.");

  /* "scipy/special/_ufuncs.pyx":6955
 *     "\n"
 *     "Returns `x` such that ``gammainc(a, x) = y``.")
 * ufunc_gammaincinv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammaincinv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":6956
 *     "Returns `x` such that ``gammainc(a, x) = y``.")
 * ufunc_gammaincinv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_gammaincinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_types[0] = <char>NPY_FLOAT
 * ufunc_gammaincinv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":6957
 * ufunc_gammaincinv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_gammaincinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammaincinv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_types[1] = <char>NPY_FLOAT
 * ufunc_gammaincinv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6958
 * ufunc_gammaincinv_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_gammaincinv_types[0] = <char>NPY_FLOAT
 * ufunc_gammaincinv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_types[2] = <char>NPY_FLOAT
 * ufunc_gammaincinv_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6959
 * ufunc_gammaincinv_types[0] = <char>NPY_FLOAT
 * ufunc_gammaincinv_types[1] = <char>NPY_FLOAT
 * ufunc_gammaincinv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_types[3] = <char>NPY_DOUBLE
 * ufunc_gammaincinv_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":6960
 * ufunc_gammaincinv_types[1] = <char>NPY_FLOAT
 * ufunc_gammaincinv_types[2] = <char>NPY_FLOAT
 * ufunc_gammaincinv_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_types[4] = <char>NPY_DOUBLE
 * ufunc_gammaincinv_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6961
 * ufunc_gammaincinv_types[2] = <char>NPY_FLOAT
 * ufunc_gammaincinv_types[3] = <char>NPY_DOUBLE
 * ufunc_gammaincinv_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_types[5] = <char>NPY_DOUBLE
 * ufunc_gammaincinv_ptr[2*0] = <void*>_func_igami
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6962
 * ufunc_gammaincinv_types[3] = <char>NPY_DOUBLE
 * ufunc_gammaincinv_types[4] = <char>NPY_DOUBLE
 * ufunc_gammaincinv_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_ptr[2*0] = <void*>_func_igami
 * ufunc_gammaincinv_ptr[2*0+1] = <void*>(<char*>"gammaincinv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":6963
 * ufunc_gammaincinv_types[4] = <char>NPY_DOUBLE
 * ufunc_gammaincinv_types[5] = <char>NPY_DOUBLE
 * ufunc_gammaincinv_ptr[2*0] = <void*>_func_igami             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_ptr[2*0+1] = <void*>(<char*>"gammaincinv")
 * ufunc_gammaincinv_ptr[2*1] = <void*>_func_igami
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_ptr[0]) = ((void *)igami);

  /* "scipy/special/_ufuncs.pyx":6964
 * ufunc_gammaincinv_types[5] = <char>NPY_DOUBLE
 * ufunc_gammaincinv_ptr[2*0] = <void*>_func_igami
 * ufunc_gammaincinv_ptr[2*0+1] = <void*>(<char*>"gammaincinv")             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_ptr[2*1] = <void*>_func_igami
 * ufunc_gammaincinv_ptr[2*1+1] = <void*>(<char*>"gammaincinv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_ptr[1]) = ((void *)((char *)((char *)"gammaincinv")));

  /* "scipy/special/_ufuncs.pyx":6965
 * ufunc_gammaincinv_ptr[2*0] = <void*>_func_igami
 * ufunc_gammaincinv_ptr[2*0+1] = <void*>(<char*>"gammaincinv")
 * ufunc_gammaincinv_ptr[2*1] = <void*>_func_igami             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_ptr[2*1+1] = <void*>(<char*>"gammaincinv")
 * ufunc_gammaincinv_data[0] = &ufunc_gammaincinv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_ptr[2]) = ((void *)igami);

  /* "scipy/special/_ufuncs.pyx":6966
 * ufunc_gammaincinv_ptr[2*0+1] = <void*>(<char*>"gammaincinv")
 * ufunc_gammaincinv_ptr[2*1] = <void*>_func_igami
 * ufunc_gammaincinv_ptr[2*1+1] = <void*>(<char*>"gammaincinv")             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_data[0] = &ufunc_gammaincinv_ptr[2*0]
 * ufunc_gammaincinv_data[1] = &ufunc_gammaincinv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_ptr[3]) = ((void *)((char *)((char *)"gammaincinv")));

  /* "scipy/special/_ufuncs.pyx":6967
 * ufunc_gammaincinv_ptr[2*1] = <void*>_func_igami
 * ufunc_gammaincinv_ptr[2*1+1] = <void*>(<char*>"gammaincinv")
 * ufunc_gammaincinv_data[0] = &ufunc_gammaincinv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gammaincinv_data[1] = &ufunc_gammaincinv_ptr[2*1]
 * gammaincinv = np.PyUFunc_FromFuncAndData(ufunc_gammaincinv_loops, ufunc_gammaincinv_data, ufunc_gammaincinv_types, 2, 2, 1, 0, "gammaincinv", ufunc_gammaincinv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":6968
 * ufunc_gammaincinv_ptr[2*1+1] = <void*>(<char*>"gammaincinv")
 * ufunc_gammaincinv_data[0] = &ufunc_gammaincinv_ptr[2*0]
 * ufunc_gammaincinv_data[1] = &ufunc_gammaincinv_ptr[2*1]             # <<<<<<<<<<<<<<
 * gammaincinv = np.PyUFunc_FromFuncAndData(ufunc_gammaincinv_loops, ufunc_gammaincinv_data, ufunc_gammaincinv_types, 2, 2, 1, 0, "gammaincinv", ufunc_gammaincinv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":6969
 * ufunc_gammaincinv_data[0] = &ufunc_gammaincinv_ptr[2*0]
 * ufunc_gammaincinv_data[1] = &ufunc_gammaincinv_ptr[2*1]
 * gammaincinv = np.PyUFunc_FromFuncAndData(ufunc_gammaincinv_loops, ufunc_gammaincinv_data, ufunc_gammaincinv_types, 2, 2, 1, 0, "gammaincinv", ufunc_gammaincinv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gammaln_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_types, 2, 2, 1, 0, ((char *)"gammaincinv"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaincinv_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6969, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gammaincinv, __pyx_t_2) < 0) __PYX_ERR(1, 6969, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":6975
 * cdef void *ufunc_gammaln_data[2]
 * cdef char ufunc_gammaln_types[4]
 * cdef char *ufunc_gammaln_doc = (             # <<<<<<<<<<<<<<
 *     "Logarithm of the absolute value of the Gamma function.\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_doc = ((char *)"Logarithm of the absolute value of the Gamma function.\n\nParameters\n----------\nx : array-like\n    Values on the real line at which to compute ``gammaln``\n\nReturns\n-------\ngammaln : ndarray\n    Values of ``gammaln`` at x.\n\nSee Also\n--------\ngammasgn : sign of the gamma function\nloggamma : principal branch of the logarithm of the gamma function\n\nNotes\n-----\nWhen used in conjunction with `gammasgn`, this function is useful\nfor working in logspace on the real axis without having to deal with\ncomplex numbers, via the relation ``exp(gammaln(x)) = gammasgn(x)*gamma(x)``.\n\nFor complex-valued log-gamma, use `loggamma` instead of `gammaln`.");

  /* "scipy/special/_ufuncs.pyx":7000
 *     "\n"
 *     "For complex-valued log-gamma, use `loggamma` instead of `gammaln`.")
 * ufunc_gammaln_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_gammaln_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_gammaln_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":7001
 *     "For complex-valued log-gamma, use `loggamma` instead of `gammaln`.")
 * ufunc_gammaln_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_gammaln_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_gammaln_types[0] = <char>NPY_FLOAT
 * ufunc_gammaln_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":7002
 * ufunc_gammaln_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_gammaln_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_gammaln_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammaln_types[1] = <char>NPY_FLOAT
 * ufunc_gammaln_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7003
 * ufunc_gammaln_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_gammaln_types[0] = <char>NPY_FLOAT
 * ufunc_gammaln_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammaln_types[2] = <char>NPY_DOUBLE
 * ufunc_gammaln_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7004
 * ufunc_gammaln_types[0] = <char>NPY_FLOAT
 * ufunc_gammaln_types[1] = <char>NPY_FLOAT
 * ufunc_gammaln_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammaln_types[3] = <char>NPY_DOUBLE
 * ufunc_gammaln_ptr[2*0] = <void*>_func_lgam
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7005
 * ufunc_gammaln_types[1] = <char>NPY_FLOAT
 * ufunc_gammaln_types[2] = <char>NPY_DOUBLE
 * ufunc_gammaln_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammaln_ptr[2*0] = <void*>_func_lgam
 * ufunc_gammaln_ptr[2*0+1] = <void*>(<char*>"gammaln")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7006
 * ufunc_gammaln_types[2] = <char>NPY_DOUBLE
 * ufunc_gammaln_types[3] = <char>NPY_DOUBLE
 * ufunc_gammaln_ptr[2*0] = <void*>_func_lgam             # <<<<<<<<<<<<<<
 * ufunc_gammaln_ptr[2*0+1] = <void*>(<char*>"gammaln")
 * ufunc_gammaln_ptr[2*1] = <void*>_func_lgam
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_ptr[0]) = ((void *)lgam);

  /* "scipy/special/_ufuncs.pyx":7007
 * ufunc_gammaln_types[3] = <char>NPY_DOUBLE
 * ufunc_gammaln_ptr[2*0] = <void*>_func_lgam
 * ufunc_gammaln_ptr[2*0+1] = <void*>(<char*>"gammaln")             # <<<<<<<<<<<<<<
 * ufunc_gammaln_ptr[2*1] = <void*>_func_lgam
 * ufunc_gammaln_ptr[2*1+1] = <void*>(<char*>"gammaln")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_ptr[1]) = ((void *)((char *)((char *)"gammaln")));

  /* "scipy/special/_ufuncs.pyx":7008
 * ufunc_gammaln_ptr[2*0] = <void*>_func_lgam
 * ufunc_gammaln_ptr[2*0+1] = <void*>(<char*>"gammaln")
 * ufunc_gammaln_ptr[2*1] = <void*>_func_lgam             # <<<<<<<<<<<<<<
 * ufunc_gammaln_ptr[2*1+1] = <void*>(<char*>"gammaln")
 * ufunc_gammaln_data[0] = &ufunc_gammaln_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_ptr[2]) = ((void *)lgam);

  /* "scipy/special/_ufuncs.pyx":7009
 * ufunc_gammaln_ptr[2*0+1] = <void*>(<char*>"gammaln")
 * ufunc_gammaln_ptr[2*1] = <void*>_func_lgam
 * ufunc_gammaln_ptr[2*1+1] = <void*>(<char*>"gammaln")             # <<<<<<<<<<<<<<
 * ufunc_gammaln_data[0] = &ufunc_gammaln_ptr[2*0]
 * ufunc_gammaln_data[1] = &ufunc_gammaln_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_ptr[3]) = ((void *)((char *)((char *)"gammaln")));

  /* "scipy/special/_ufuncs.pyx":7010
 * ufunc_gammaln_ptr[2*1] = <void*>_func_lgam
 * ufunc_gammaln_ptr[2*1+1] = <void*>(<char*>"gammaln")
 * ufunc_gammaln_data[0] = &ufunc_gammaln_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gammaln_data[1] = &ufunc_gammaln_ptr[2*1]
 * gammaln = np.PyUFunc_FromFuncAndData(ufunc_gammaln_loops, ufunc_gammaln_data, ufunc_gammaln_types, 2, 1, 1, 0, "gammaln", ufunc_gammaln_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7011
 * ufunc_gammaln_ptr[2*1+1] = <void*>(<char*>"gammaln")
 * ufunc_gammaln_data[0] = &ufunc_gammaln_ptr[2*0]
 * ufunc_gammaln_data[1] = &ufunc_gammaln_ptr[2*1]             # <<<<<<<<<<<<<<
 * gammaln = np.PyUFunc_FromFuncAndData(ufunc_gammaln_loops, ufunc_gammaln_data, ufunc_gammaln_types, 2, 1, 1, 0, "gammaln", ufunc_gammaln_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7012
 * ufunc_gammaln_data[0] = &ufunc_gammaln_ptr[2*0]
 * ufunc_gammaln_data[1] = &ufunc_gammaln_ptr[2*1]
 * gammaln = np.PyUFunc_FromFuncAndData(ufunc_gammaln_loops, ufunc_gammaln_data, ufunc_gammaln_types, 2, 1, 1, 0, "gammaln", ufunc_gammaln_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gammasgn_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_types, 2, 1, 1, 0, ((char *)"gammaln"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammaln_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7012, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gammaln, __pyx_t_2) < 0) __PYX_ERR(1, 7012, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":7018
 * cdef void *ufunc_gammasgn_data[2]
 * cdef char ufunc_gammasgn_types[4]
 * cdef char *ufunc_gammasgn_doc = (             # <<<<<<<<<<<<<<
 *     "gammasgn(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_doc = ((char *)"gammasgn(x)\n\nSign of the gamma function.\n\nSee Also\n--------\ngammaln\nloggamma");

  /* "scipy/special/_ufuncs.pyx":7027
 *     "gammaln\n"
 *     "loggamma")
 * ufunc_gammasgn_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_gammasgn_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":7028
 *     "loggamma")
 * ufunc_gammasgn_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_gammasgn_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_types[0] = <char>NPY_FLOAT
 * ufunc_gammasgn_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":7029
 * ufunc_gammasgn_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_gammasgn_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_gammasgn_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_types[1] = <char>NPY_FLOAT
 * ufunc_gammasgn_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7030
 * ufunc_gammasgn_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_gammasgn_types[0] = <char>NPY_FLOAT
 * ufunc_gammasgn_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_types[2] = <char>NPY_DOUBLE
 * ufunc_gammasgn_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7031
 * ufunc_gammasgn_types[0] = <char>NPY_FLOAT
 * ufunc_gammasgn_types[1] = <char>NPY_FLOAT
 * ufunc_gammasgn_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_types[3] = <char>NPY_DOUBLE
 * ufunc_gammasgn_ptr[2*0] = <void*>_func_gammasgn
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7032
 * ufunc_gammasgn_types[1] = <char>NPY_FLOAT
 * ufunc_gammasgn_types[2] = <char>NPY_DOUBLE
 * ufunc_gammasgn_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_ptr[2*0] = <void*>_func_gammasgn
 * ufunc_gammasgn_ptr[2*0+1] = <void*>(<char*>"gammasgn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7033
 * ufunc_gammasgn_types[2] = <char>NPY_DOUBLE
 * ufunc_gammasgn_types[3] = <char>NPY_DOUBLE
 * ufunc_gammasgn_ptr[2*0] = <void*>_func_gammasgn             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_ptr[2*0+1] = <void*>(<char*>"gammasgn")
 * ufunc_gammasgn_ptr[2*1] = <void*>_func_gammasgn
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_ptr[0]) = ((void *)gammasgn);

  /* "scipy/special/_ufuncs.pyx":7034
 * ufunc_gammasgn_types[3] = <char>NPY_DOUBLE
 * ufunc_gammasgn_ptr[2*0] = <void*>_func_gammasgn
 * ufunc_gammasgn_ptr[2*0+1] = <void*>(<char*>"gammasgn")             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_ptr[2*1] = <void*>_func_gammasgn
 * ufunc_gammasgn_ptr[2*1+1] = <void*>(<char*>"gammasgn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_ptr[1]) = ((void *)((char *)((char *)"gammasgn")));

  /* "scipy/special/_ufuncs.pyx":7035
 * ufunc_gammasgn_ptr[2*0] = <void*>_func_gammasgn
 * ufunc_gammasgn_ptr[2*0+1] = <void*>(<char*>"gammasgn")
 * ufunc_gammasgn_ptr[2*1] = <void*>_func_gammasgn             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_ptr[2*1+1] = <void*>(<char*>"gammasgn")
 * ufunc_gammasgn_data[0] = &ufunc_gammasgn_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_ptr[2]) = ((void *)gammasgn);

  /* "scipy/special/_ufuncs.pyx":7036
 * ufunc_gammasgn_ptr[2*0+1] = <void*>(<char*>"gammasgn")
 * ufunc_gammasgn_ptr[2*1] = <void*>_func_gammasgn
 * ufunc_gammasgn_ptr[2*1+1] = <void*>(<char*>"gammasgn")             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_data[0] = &ufunc_gammasgn_ptr[2*0]
 * ufunc_gammasgn_data[1] = &ufunc_gammasgn_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_ptr[3]) = ((void *)((char *)((char *)"gammasgn")));

  /* "scipy/special/_ufuncs.pyx":7037
 * ufunc_gammasgn_ptr[2*1] = <void*>_func_gammasgn
 * ufunc_gammasgn_ptr[2*1+1] = <void*>(<char*>"gammasgn")
 * ufunc_gammasgn_data[0] = &ufunc_gammasgn_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gammasgn_data[1] = &ufunc_gammasgn_ptr[2*1]
 * gammasgn = np.PyUFunc_FromFuncAndData(ufunc_gammasgn_loops, ufunc_gammasgn_data, ufunc_gammasgn_types, 2, 1, 1, 0, "gammasgn", ufunc_gammasgn_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7038
 * ufunc_gammasgn_ptr[2*1+1] = <void*>(<char*>"gammasgn")
 * ufunc_gammasgn_data[0] = &ufunc_gammasgn_ptr[2*0]
 * ufunc_gammasgn_data[1] = &ufunc_gammasgn_ptr[2*1]             # <<<<<<<<<<<<<<
 * gammasgn = np.PyUFunc_FromFuncAndData(ufunc_gammasgn_loops, ufunc_gammasgn_data, ufunc_gammasgn_types, 2, 1, 1, 0, "gammasgn", ufunc_gammasgn_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7039
 * ufunc_gammasgn_data[0] = &ufunc_gammasgn_ptr[2*0]
 * ufunc_gammasgn_data[1] = &ufunc_gammasgn_ptr[2*1]
 * gammasgn = np.PyUFunc_FromFuncAndData(ufunc_gammasgn_loops, ufunc_gammasgn_data, ufunc_gammasgn_types, 2, 1, 1, 0, "gammasgn", ufunc_gammasgn_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gdtr_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_types, 2, 1, 1, 0, ((char *)"gammasgn"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_gammasgn_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7039, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gammasgn, __pyx_t_2) < 0) __PYX_ERR(1, 7039, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":7045
 * cdef void *ufunc_gdtr_data[2]
 * cdef char ufunc_gdtr_types[8]
 * cdef char *ufunc_gdtr_doc = (             # <<<<<<<<<<<<<<
 *     "gdtr(a, b, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_doc = ((char *)"gdtr(a, b, x)\n\nGamma distribution cumulative distribution function.\n\nReturns the integral from zero to `x` of the gamma probability density\nfunction,\n\n.. math::\n\n    F = \\int_0^x \\frac{a^b}{\\Gamma(b)} t^{b-1} e^{-at}\\,dt,\n\nwhere :math:`\\Gamma` is the gamma function.\n\nParameters\n----------\na : array_like\n    The rate parameter of the gamma distribution, sometimes denoted\n    :math:`\\beta` (float).  It is also the reciprocal of the scale\n    parameter :math:`\\theta`.\nb : array_like\n    The shape parameter of the gamma distribution, sometimes denoted\n    :math:`\\alpha` (float).\nx : array_like\n    The quantile (upper limit of integration; float).\n\nSee also\n--------\ngdtrc : 1 - CDF of the gamma distribution.\n\nReturns\n-------\nF : ndarray\n    The CDF of the gamma distribution with parameters `a` and `b`\n    evaluated at `x`.\n\nNotes\n-----\nThe evaluation is carried out using the relation to the incomplete gamma\nintegral (regularized gamma function).\n\nWrapper for the Cephes [1]_ routine `gdtr`.\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":7092
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_gdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_gdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":7093
 *     "       http://www.netlib.org/cephes/")
 * ufunc_gdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_gdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_gdtr_types[0] = <char>NPY_FLOAT
 * ufunc_gdtr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":7094
 * ufunc_gdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_gdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtr_types[1] = <char>NPY_FLOAT
 * ufunc_gdtr_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7095
 * ufunc_gdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtr_types[0] = <char>NPY_FLOAT
 * ufunc_gdtr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtr_types[2] = <char>NPY_FLOAT
 * ufunc_gdtr_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7096
 * ufunc_gdtr_types[0] = <char>NPY_FLOAT
 * ufunc_gdtr_types[1] = <char>NPY_FLOAT
 * ufunc_gdtr_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtr_types[3] = <char>NPY_FLOAT
 * ufunc_gdtr_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7097
 * ufunc_gdtr_types[1] = <char>NPY_FLOAT
 * ufunc_gdtr_types[2] = <char>NPY_FLOAT
 * ufunc_gdtr_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtr_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7098
 * ufunc_gdtr_types[2] = <char>NPY_FLOAT
 * ufunc_gdtr_types[3] = <char>NPY_FLOAT
 * ufunc_gdtr_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtr_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7099
 * ufunc_gdtr_types[3] = <char>NPY_FLOAT
 * ufunc_gdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtr_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtr_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7100
 * ufunc_gdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtr_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtr_ptr[2*0] = <void*>_func_gdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7101
 * ufunc_gdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtr_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtr_ptr[2*0] = <void*>_func_gdtr
 * ufunc_gdtr_ptr[2*0+1] = <void*>(<char*>"gdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7102
 * ufunc_gdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtr_ptr[2*0] = <void*>_func_gdtr             # <<<<<<<<<<<<<<
 * ufunc_gdtr_ptr[2*0+1] = <void*>(<char*>"gdtr")
 * ufunc_gdtr_ptr[2*1] = <void*>_func_gdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_ptr[0]) = ((void *)gdtr);

  /* "scipy/special/_ufuncs.pyx":7103
 * ufunc_gdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtr_ptr[2*0] = <void*>_func_gdtr
 * ufunc_gdtr_ptr[2*0+1] = <void*>(<char*>"gdtr")             # <<<<<<<<<<<<<<
 * ufunc_gdtr_ptr[2*1] = <void*>_func_gdtr
 * ufunc_gdtr_ptr[2*1+1] = <void*>(<char*>"gdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_ptr[1]) = ((void *)((char *)((char *)"gdtr")));

  /* "scipy/special/_ufuncs.pyx":7104
 * ufunc_gdtr_ptr[2*0] = <void*>_func_gdtr
 * ufunc_gdtr_ptr[2*0+1] = <void*>(<char*>"gdtr")
 * ufunc_gdtr_ptr[2*1] = <void*>_func_gdtr             # <<<<<<<<<<<<<<
 * ufunc_gdtr_ptr[2*1+1] = <void*>(<char*>"gdtr")
 * ufunc_gdtr_data[0] = &ufunc_gdtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_ptr[2]) = ((void *)gdtr);

  /* "scipy/special/_ufuncs.pyx":7105
 * ufunc_gdtr_ptr[2*0+1] = <void*>(<char*>"gdtr")
 * ufunc_gdtr_ptr[2*1] = <void*>_func_gdtr
 * ufunc_gdtr_ptr[2*1+1] = <void*>(<char*>"gdtr")             # <<<<<<<<<<<<<<
 * ufunc_gdtr_data[0] = &ufunc_gdtr_ptr[2*0]
 * ufunc_gdtr_data[1] = &ufunc_gdtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_ptr[3]) = ((void *)((char *)((char *)"gdtr")));

  /* "scipy/special/_ufuncs.pyx":7106
 * ufunc_gdtr_ptr[2*1] = <void*>_func_gdtr
 * ufunc_gdtr_ptr[2*1+1] = <void*>(<char*>"gdtr")
 * ufunc_gdtr_data[0] = &ufunc_gdtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gdtr_data[1] = &ufunc_gdtr_ptr[2*1]
 * gdtr = np.PyUFunc_FromFuncAndData(ufunc_gdtr_loops, ufunc_gdtr_data, ufunc_gdtr_types, 2, 3, 1, 0, "gdtr", ufunc_gdtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7107
 * ufunc_gdtr_ptr[2*1+1] = <void*>(<char*>"gdtr")
 * ufunc_gdtr_data[0] = &ufunc_gdtr_ptr[2*0]
 * ufunc_gdtr_data[1] = &ufunc_gdtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * gdtr = np.PyUFunc_FromFuncAndData(ufunc_gdtr_loops, ufunc_gdtr_data, ufunc_gdtr_types, 2, 3, 1, 0, "gdtr", ufunc_gdtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7108
 * ufunc_gdtr_data[0] = &ufunc_gdtr_ptr[2*0]
 * ufunc_gdtr_data[1] = &ufunc_gdtr_ptr[2*1]
 * gdtr = np.PyUFunc_FromFuncAndData(ufunc_gdtr_loops, ufunc_gdtr_data, ufunc_gdtr_types, 2, 3, 1, 0, "gdtr", ufunc_gdtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gdtrc_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_types, 2, 3, 1, 0, ((char *)"gdtr"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtr_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gdtr, __pyx_t_2) < 0) __PYX_ERR(1, 7108, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":7114
 * cdef void *ufunc_gdtrc_data[2]
 * cdef char ufunc_gdtrc_types[8]
 * cdef char *ufunc_gdtrc_doc = (             # <<<<<<<<<<<<<<
 *     "gdtrc(a, b, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_doc = ((char *)"gdtrc(a, b, x)\n\nGamma distribution survival function.\n\nIntegral from `x` to infinity of the gamma probability density function,\n\n.. math::\n\n    F = \\int_x^\\infty \\frac{a^b}{\\Gamma(b)} t^{b-1} e^{-at}\\,dt,\n\nwhere :math:`\\Gamma` is the gamma function.\n\nParameters\n----------\na : array_like\n    The rate parameter of the gamma distribution, sometimes denoted\n    :math:`\\beta` (float).  It is also the reciprocal of the scale\n    parameter :math:`\\theta`.\nb : array_like\n    The shape parameter of the gamma distribution, sometimes denoted\n    :math:`\\alpha` (float).\nx : array_like\n    The quantile (lower limit of integration; float).\n\nReturns\n-------\nF : ndarray\n    The survival function of the gamma distribution with parameters `a`\n    and `b` evaluated at `x`.\n\nSee Also\n--------\ngdtr, gdtrix\n\nNotes\n-----\nThe evaluation is carried out using the relation to the incomplete gamma\nintegral (regularized gamma function).\n\nWrapper for the Cephes [1]_ routine `gdtrc`.\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":7160
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_gdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtrc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":7161
 *     "       http://www.netlib.org/cephes/")
 * ufunc_gdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_gdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_types[0] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":7162
 * ufunc_gdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_gdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtrc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_types[1] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7163
 * ufunc_gdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtrc_types[0] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_types[2] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7164
 * ufunc_gdtrc_types[0] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[1] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_types[3] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7165
 * ufunc_gdtrc_types[1] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[2] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtrc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7166
 * ufunc_gdtrc_types[2] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[3] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtrc_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7167
 * ufunc_gdtrc_types[3] = <char>NPY_FLOAT
 * ufunc_gdtrc_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtrc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtrc_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7168
 * ufunc_gdtrc_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtrc_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtrc_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtrc_ptr[2*0] = <void*>_func_gdtrc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7169
 * ufunc_gdtrc_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtrc_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtrc_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_ptr[2*0] = <void*>_func_gdtrc
 * ufunc_gdtrc_ptr[2*0+1] = <void*>(<char*>"gdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7170
 * ufunc_gdtrc_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtrc_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtrc_ptr[2*0] = <void*>_func_gdtrc             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_ptr[2*0+1] = <void*>(<char*>"gdtrc")
 * ufunc_gdtrc_ptr[2*1] = <void*>_func_gdtrc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_ptr[0]) = ((void *)gdtrc);

  /* "scipy/special/_ufuncs.pyx":7171
 * ufunc_gdtrc_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtrc_ptr[2*0] = <void*>_func_gdtrc
 * ufunc_gdtrc_ptr[2*0+1] = <void*>(<char*>"gdtrc")             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_ptr[2*1] = <void*>_func_gdtrc
 * ufunc_gdtrc_ptr[2*1+1] = <void*>(<char*>"gdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_ptr[1]) = ((void *)((char *)((char *)"gdtrc")));

  /* "scipy/special/_ufuncs.pyx":7172
 * ufunc_gdtrc_ptr[2*0] = <void*>_func_gdtrc
 * ufunc_gdtrc_ptr[2*0+1] = <void*>(<char*>"gdtrc")
 * ufunc_gdtrc_ptr[2*1] = <void*>_func_gdtrc             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_ptr[2*1+1] = <void*>(<char*>"gdtrc")
 * ufunc_gdtrc_data[0] = &ufunc_gdtrc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_ptr[2]) = ((void *)gdtrc);

  /* "scipy/special/_ufuncs.pyx":7173
 * ufunc_gdtrc_ptr[2*0+1] = <void*>(<char*>"gdtrc")
 * ufunc_gdtrc_ptr[2*1] = <void*>_func_gdtrc
 * ufunc_gdtrc_ptr[2*1+1] = <void*>(<char*>"gdtrc")             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_data[0] = &ufunc_gdtrc_ptr[2*0]
 * ufunc_gdtrc_data[1] = &ufunc_gdtrc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_ptr[3]) = ((void *)((char *)((char *)"gdtrc")));

  /* "scipy/special/_ufuncs.pyx":7174
 * ufunc_gdtrc_ptr[2*1] = <void*>_func_gdtrc
 * ufunc_gdtrc_ptr[2*1+1] = <void*>(<char*>"gdtrc")
 * ufunc_gdtrc_data[0] = &ufunc_gdtrc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gdtrc_data[1] = &ufunc_gdtrc_ptr[2*1]
 * gdtrc = np.PyUFunc_FromFuncAndData(ufunc_gdtrc_loops, ufunc_gdtrc_data, ufunc_gdtrc_types, 2, 3, 1, 0, "gdtrc", ufunc_gdtrc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7175
 * ufunc_gdtrc_ptr[2*1+1] = <void*>(<char*>"gdtrc")
 * ufunc_gdtrc_data[0] = &ufunc_gdtrc_ptr[2*0]
 * ufunc_gdtrc_data[1] = &ufunc_gdtrc_ptr[2*1]             # <<<<<<<<<<<<<<
 * gdtrc = np.PyUFunc_FromFuncAndData(ufunc_gdtrc_loops, ufunc_gdtrc_data, ufunc_gdtrc_types, 2, 3, 1, 0, "gdtrc", ufunc_gdtrc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7176
 * ufunc_gdtrc_data[0] = &ufunc_gdtrc_ptr[2*0]
 * ufunc_gdtrc_data[1] = &ufunc_gdtrc_ptr[2*1]
 * gdtrc = np.PyUFunc_FromFuncAndData(ufunc_gdtrc_loops, ufunc_gdtrc_data, ufunc_gdtrc_types, 2, 3, 1, 0, "gdtrc", ufunc_gdtrc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gdtria_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_types, 2, 3, 1, 0, ((char *)"gdtrc"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrc_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gdtrc, __pyx_t_2) < 0) __PYX_ERR(1, 7176, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":7182
 * cdef void *ufunc_gdtria_data[2]
 * cdef char ufunc_gdtria_types[8]
 * cdef char *ufunc_gdtria_doc = (             # <<<<<<<<<<<<<<
 *     "gdtria(p, b, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_doc = ((char *)"gdtria(p, b, x, out=None)\n\nInverse of `gdtr` vs a.\n\nReturns the inverse with respect to the parameter `a` of ``p =\ngdtr(a, b, x)``, the cumulative distribution function of the gamma\ndistribution.\n\nParameters\n----------\np : array_like\n    Probability values.\nb : array_like\n    `b` parameter values of `gdtr(a, b, x)`.  `b` is the \"shape\" parameter\n    of the gamma distribution.\nx : array_like\n    Nonnegative real values, from the domain of the gamma distribution.\nout : ndarray, optional\n    If a fourth argument is given, it must be a numpy.ndarray whose size\n    matches the broadcast result of `a`, `b` and `x`.  `out` is then the\n    array returned by the function.\n\nReturns\n-------\na : ndarray\n    Values of the `a` parameter such that `p = gdtr(a, b, x)`.  `1/a`\n    is the \"scale\" parameter of the gamma distribution.\n\nSee Also\n--------\ngdtr : CDF of the gamma distribution.\ngdtrib : Inverse with respect to `b` of `gdtr(a, b, x)`.\ngdtrix : Inverse with respect to `x` of `gdtr(a, b, x)`.\n\nNotes\n-----\nWrapper for the CDFLIB [1]_ Fortran routine `cdfgam`.\n\nThe cumulative distribution function `p` is computed using a routine by\nDiDinato and Morris [2]_.  Computation of `a` involves a search for a value\nthat produces the desired value of `p`.  The search relies on the\nmonotonicity of `p` with `a`.\n\nReferences\n----------\n.. [1] Barry Brown, James Lovato, and Kathy Russell,\n       CDFLIB: Library of Fortran Routines for Cumulative Distribution\n       Functions, Inverses, and Other Parameters.\n.. [2] DiDinato, A. R. and Morris, A. H.,\n       Computation of the incomplete gamma function ratios and their\n       inverse.  ACM Trans. Math. Softw. 12 (1986), 377-393.\n\nExamples\n--------\nFirst evaluate `gdtr`.\n\n>>> from scipy.special import gdtr, gdtria\n>>> p = gdtr(1.2, 3.4, 5.6)\n>>> print(p)\n0.94378087442\n\nVerify the inverse.\n\n>>> gdtria(p, 3.4, 5.6)\n1.2");

  /* "scipy/special/_ufuncs.pyx":7248
 *     ">>> gdtria(p, 3.4, 5.6)\n"
 *     "1.2")
 * ufunc_gdtria_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_gdtria_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtria_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":7249
 *     "1.2")
 * ufunc_gdtria_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_gdtria_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_gdtria_types[0] = <char>NPY_FLOAT
 * ufunc_gdtria_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":7250
 * ufunc_gdtria_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_gdtria_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtria_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtria_types[1] = <char>NPY_FLOAT
 * ufunc_gdtria_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7251
 * ufunc_gdtria_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtria_types[0] = <char>NPY_FLOAT
 * ufunc_gdtria_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtria_types[2] = <char>NPY_FLOAT
 * ufunc_gdtria_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7252
 * ufunc_gdtria_types[0] = <char>NPY_FLOAT
 * ufunc_gdtria_types[1] = <char>NPY_FLOAT
 * ufunc_gdtria_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtria_types[3] = <char>NPY_FLOAT
 * ufunc_gdtria_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7253
 * ufunc_gdtria_types[1] = <char>NPY_FLOAT
 * ufunc_gdtria_types[2] = <char>NPY_FLOAT
 * ufunc_gdtria_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtria_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtria_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7254
 * ufunc_gdtria_types[2] = <char>NPY_FLOAT
 * ufunc_gdtria_types[3] = <char>NPY_FLOAT
 * ufunc_gdtria_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtria_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtria_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7255
 * ufunc_gdtria_types[3] = <char>NPY_FLOAT
 * ufunc_gdtria_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtria_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtria_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtria_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7256
 * ufunc_gdtria_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtria_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtria_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtria_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtria_ptr[2*0] = <void*>_func_cdfgam4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7257
 * ufunc_gdtria_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtria_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtria_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtria_ptr[2*0] = <void*>_func_cdfgam4_wrap
 * ufunc_gdtria_ptr[2*0+1] = <void*>(<char*>"gdtria")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7258
 * ufunc_gdtria_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtria_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtria_ptr[2*0] = <void*>_func_cdfgam4_wrap             # <<<<<<<<<<<<<<
 * ufunc_gdtria_ptr[2*0+1] = <void*>(<char*>"gdtria")
 * ufunc_gdtria_ptr[2*1] = <void*>_func_cdfgam4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_ptr[0]) = ((void *)cdfgam4_wrap);

  /* "scipy/special/_ufuncs.pyx":7259
 * ufunc_gdtria_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtria_ptr[2*0] = <void*>_func_cdfgam4_wrap
 * ufunc_gdtria_ptr[2*0+1] = <void*>(<char*>"gdtria")             # <<<<<<<<<<<<<<
 * ufunc_gdtria_ptr[2*1] = <void*>_func_cdfgam4_wrap
 * ufunc_gdtria_ptr[2*1+1] = <void*>(<char*>"gdtria")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_ptr[1]) = ((void *)((char *)((char *)"gdtria")));

  /* "scipy/special/_ufuncs.pyx":7260
 * ufunc_gdtria_ptr[2*0] = <void*>_func_cdfgam4_wrap
 * ufunc_gdtria_ptr[2*0+1] = <void*>(<char*>"gdtria")
 * ufunc_gdtria_ptr[2*1] = <void*>_func_cdfgam4_wrap             # <<<<<<<<<<<<<<
 * ufunc_gdtria_ptr[2*1+1] = <void*>(<char*>"gdtria")
 * ufunc_gdtria_data[0] = &ufunc_gdtria_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_ptr[2]) = ((void *)cdfgam4_wrap);

  /* "scipy/special/_ufuncs.pyx":7261
 * ufunc_gdtria_ptr[2*0+1] = <void*>(<char*>"gdtria")
 * ufunc_gdtria_ptr[2*1] = <void*>_func_cdfgam4_wrap
 * ufunc_gdtria_ptr[2*1+1] = <void*>(<char*>"gdtria")             # <<<<<<<<<<<<<<
 * ufunc_gdtria_data[0] = &ufunc_gdtria_ptr[2*0]
 * ufunc_gdtria_data[1] = &ufunc_gdtria_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_ptr[3]) = ((void *)((char *)((char *)"gdtria")));

  /* "scipy/special/_ufuncs.pyx":7262
 * ufunc_gdtria_ptr[2*1] = <void*>_func_cdfgam4_wrap
 * ufunc_gdtria_ptr[2*1+1] = <void*>(<char*>"gdtria")
 * ufunc_gdtria_data[0] = &ufunc_gdtria_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gdtria_data[1] = &ufunc_gdtria_ptr[2*1]
 * gdtria = np.PyUFunc_FromFuncAndData(ufunc_gdtria_loops, ufunc_gdtria_data, ufunc_gdtria_types, 2, 3, 1, 0, "gdtria", ufunc_gdtria_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7263
 * ufunc_gdtria_ptr[2*1+1] = <void*>(<char*>"gdtria")
 * ufunc_gdtria_data[0] = &ufunc_gdtria_ptr[2*0]
 * ufunc_gdtria_data[1] = &ufunc_gdtria_ptr[2*1]             # <<<<<<<<<<<<<<
 * gdtria = np.PyUFunc_FromFuncAndData(ufunc_gdtria_loops, ufunc_gdtria_data, ufunc_gdtria_types, 2, 3, 1, 0, "gdtria", ufunc_gdtria_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7264
 * ufunc_gdtria_data[0] = &ufunc_gdtria_ptr[2*0]
 * ufunc_gdtria_data[1] = &ufunc_gdtria_ptr[2*1]
 * gdtria = np.PyUFunc_FromFuncAndData(ufunc_gdtria_loops, ufunc_gdtria_data, ufunc_gdtria_types, 2, 3, 1, 0, "gdtria", ufunc_gdtria_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gdtrib_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_types, 2, 3, 1, 0, ((char *)"gdtria"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtria_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gdtria, __pyx_t_2) < 0) __PYX_ERR(1, 7264, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":7270
 * cdef void *ufunc_gdtrib_data[2]
 * cdef char ufunc_gdtrib_types[8]
 * cdef char *ufunc_gdtrib_doc = (             # <<<<<<<<<<<<<<
 *     "gdtrib(a, p, x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_doc = ((char *)"gdtrib(a, p, x, out=None)\n\nInverse of `gdtr` vs b.\n\nReturns the inverse with respect to the parameter `b` of ``p =\ngdtr(a, b, x)``, the cumulative distribution function of the gamma\ndistribution.\n\nParameters\n----------\na : array_like\n    `a` parameter values of `gdtr(a, b, x)`. `1/a` is the \"scale\"\n    parameter of the gamma distribution.\np : array_like\n    Probability values.\nx : array_like\n    Nonnegative real values, from the domain of the gamma distribution.\nout : ndarray, optional\n    If a fourth argument is given, it must be a numpy.ndarray whose size\n    matches the broadcast result of `a`, `b` and `x`.  `out` is then the\n    array returned by the function.\n\nReturns\n-------\nb : ndarray\n    Values of the `b` parameter such that `p = gdtr(a, b, x)`.  `b` is\n    the \"shape\" parameter of the gamma distribution.\n\nSee Also\n--------\ngdtr : CDF of the gamma distribution.\ngdtria : Inverse with respect to `a` of `gdtr(a, b, x)`.\ngdtrix : Inverse with respect to `x` of `gdtr(a, b, x)`.\n\nNotes\n-----\nWrapper for the CDFLIB [1]_ Fortran routine `cdfgam`.\n\nThe cumulative distribution function `p` is computed using a routine by\nDiDinato and Morris [2]_.  Computation of `b` involves a search for a value\nthat produces the desired value of `p`.  The search relies on the\nmonotonicity of `p` with `b`.\n\nReferences\n----------\n.. [1] Barry Brown, James Lovato, and Kathy Russell,\n       CDFLIB: Library of Fortran Routines for Cumulative Distribution\n       Functions, Inverses, and Other Parameters.\n.. [2] DiDinato, A. R. and Morris, A. H.,\n       Computation of the incomplete gamma function ratios and their\n       inverse.  ACM Trans. Math. Softw. 12 (1986), 377-393.\n\nExamples\n--------\nFirst evaluate `gdtr`.\n\n>>> from scipy.special import gdtr, gdtrib\n>>> p = gdtr(1.2, 3.4, 5.6)\n>>> print(p)\n0.94378087442\n\nVerify the inverse.\n\n>>> gdtrib(1.2, p, 5.6)\n3.3999999999723882");

  /* "scipy/special/_ufuncs.pyx":7336
 *     ">>> gdtrib(1.2, p, 5.6)\n"
 *     "3.3999999999723882")
 * ufunc_gdtrib_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtrib_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":7337
 *     "3.3999999999723882")
 * ufunc_gdtrib_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_gdtrib_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_types[0] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":7338
 * ufunc_gdtrib_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_gdtrib_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtrib_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_types[1] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7339
 * ufunc_gdtrib_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtrib_types[0] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_types[2] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7340
 * ufunc_gdtrib_types[0] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[1] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_types[3] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7341
 * ufunc_gdtrib_types[1] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[2] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtrib_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7342
 * ufunc_gdtrib_types[2] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[3] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtrib_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7343
 * ufunc_gdtrib_types[3] = <char>NPY_FLOAT
 * ufunc_gdtrib_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtrib_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtrib_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7344
 * ufunc_gdtrib_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtrib_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtrib_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtrib_ptr[2*0] = <void*>_func_cdfgam3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7345
 * ufunc_gdtrib_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtrib_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtrib_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_ptr[2*0] = <void*>_func_cdfgam3_wrap
 * ufunc_gdtrib_ptr[2*0+1] = <void*>(<char*>"gdtrib")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7346
 * ufunc_gdtrib_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtrib_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtrib_ptr[2*0] = <void*>_func_cdfgam3_wrap             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_ptr[2*0+1] = <void*>(<char*>"gdtrib")
 * ufunc_gdtrib_ptr[2*1] = <void*>_func_cdfgam3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_ptr[0]) = ((void *)cdfgam3_wrap);

  /* "scipy/special/_ufuncs.pyx":7347
 * ufunc_gdtrib_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtrib_ptr[2*0] = <void*>_func_cdfgam3_wrap
 * ufunc_gdtrib_ptr[2*0+1] = <void*>(<char*>"gdtrib")             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_ptr[2*1] = <void*>_func_cdfgam3_wrap
 * ufunc_gdtrib_ptr[2*1+1] = <void*>(<char*>"gdtrib")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_ptr[1]) = ((void *)((char *)((char *)"gdtrib")));

  /* "scipy/special/_ufuncs.pyx":7348
 * ufunc_gdtrib_ptr[2*0] = <void*>_func_cdfgam3_wrap
 * ufunc_gdtrib_ptr[2*0+1] = <void*>(<char*>"gdtrib")
 * ufunc_gdtrib_ptr[2*1] = <void*>_func_cdfgam3_wrap             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_ptr[2*1+1] = <void*>(<char*>"gdtrib")
 * ufunc_gdtrib_data[0] = &ufunc_gdtrib_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_ptr[2]) = ((void *)cdfgam3_wrap);

  /* "scipy/special/_ufuncs.pyx":7349
 * ufunc_gdtrib_ptr[2*0+1] = <void*>(<char*>"gdtrib")
 * ufunc_gdtrib_ptr[2*1] = <void*>_func_cdfgam3_wrap
 * ufunc_gdtrib_ptr[2*1+1] = <void*>(<char*>"gdtrib")             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_data[0] = &ufunc_gdtrib_ptr[2*0]
 * ufunc_gdtrib_data[1] = &ufunc_gdtrib_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_ptr[3]) = ((void *)((char *)((char *)"gdtrib")));

  /* "scipy/special/_ufuncs.pyx":7350
 * ufunc_gdtrib_ptr[2*1] = <void*>_func_cdfgam3_wrap
 * ufunc_gdtrib_ptr[2*1+1] = <void*>(<char*>"gdtrib")
 * ufunc_gdtrib_data[0] = &ufunc_gdtrib_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gdtrib_data[1] = &ufunc_gdtrib_ptr[2*1]
 * gdtrib = np.PyUFunc_FromFuncAndData(ufunc_gdtrib_loops, ufunc_gdtrib_data, ufunc_gdtrib_types, 2, 3, 1, 0, "gdtrib", ufunc_gdtrib_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7351
 * ufunc_gdtrib_ptr[2*1+1] = <void*>(<char*>"gdtrib")
 * ufunc_gdtrib_data[0] = &ufunc_gdtrib_ptr[2*0]
 * ufunc_gdtrib_data[1] = &ufunc_gdtrib_ptr[2*1]             # <<<<<<<<<<<<<<
 * gdtrib = np.PyUFunc_FromFuncAndData(ufunc_gdtrib_loops, ufunc_gdtrib_data, ufunc_gdtrib_types, 2, 3, 1, 0, "gdtrib", ufunc_gdtrib_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7352
 * ufunc_gdtrib_data[0] = &ufunc_gdtrib_ptr[2*0]
 * ufunc_gdtrib_data[1] = &ufunc_gdtrib_ptr[2*1]
 * gdtrib = np.PyUFunc_FromFuncAndData(ufunc_gdtrib_loops, ufunc_gdtrib_data, ufunc_gdtrib_types, 2, 3, 1, 0, "gdtrib", ufunc_gdtrib_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_gdtrix_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_types, 2, 3, 1, 0, ((char *)"gdtrib"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrib_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7352, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gdtrib, __pyx_t_2) < 0) __PYX_ERR(1, 7352, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":7358
 * cdef void *ufunc_gdtrix_data[2]
 * cdef char ufunc_gdtrix_types[8]
 * cdef char *ufunc_gdtrix_doc = (             # <<<<<<<<<<<<<<
 *     "gdtrix(a, b, p, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_doc = ((char *)"gdtrix(a, b, p, out=None)\n\nInverse of `gdtr` vs x.\n\nReturns the inverse with respect to the parameter `x` of ``p =\ngdtr(a, b, x)``, the cumulative distribution function of the gamma\ndistribution. This is also known as the p'th quantile of the\ndistribution.\n\nParameters\n----------\na : array_like\n    `a` parameter values of `gdtr(a, b, x)`.  `1/a` is the \"scale\"\n    parameter of the gamma distribution.\nb : array_like\n    `b` parameter values of `gdtr(a, b, x)`.  `b` is the \"shape\" parameter\n    of the gamma distribution.\np : array_like\n    Probability values.\nout : ndarray, optional\n    If a fourth argument is given, it must be a numpy.ndarray whose size\n    matches the broadcast result of `a`, `b` and `x`.  `out` is then the\n    array returned by the function.\n\nReturns\n-------\nx : ndarray\n    Values of the `x` parameter such that `p = gdtr(a, b, x)`.\n\nSee Also\n--------\ngdtr : CDF of the gamma distribution.\ngdtria : Inverse with respect to `a` of `gdtr(a, b, x)`.\ngdtrib : Inverse with respect to `b` of `gdtr(a, b, x)`.\n\nNotes\n-----\nWrapper for the CDFLIB [1]_ Fortran routine `cdfgam`.\n\nThe cumulative distribution function `p` is computed using a routine by\nDiDinato and Morris [2]_.  Computation of `x` involves a search for a value\nthat produces the desired value of `p`.  The search relies on the\nmonotonicity of `p` with `x`.\n\nReferences\n----------\n.. [1] Barry Brown, James Lovato, and Kathy Russell,\n       CDFLIB: Library of Fortran Routines for Cumulative Distribution\n       Functions, Inverses, and Other Parameters.\n.. [2] DiDinato, A. R. and Morris, A. H.,\n       Computation of the incomplete gamma function ratios and their\n       inverse.  ACM Trans. Math. Softw. 12 (1986), 377-393.\n\nExamples\n--------\nFirst evaluate `gdtr`.\n\n>>> from scipy.special import gdtr, gdtrix\n>>> p = gdtr(1.2, 3.4, 5.6)\n>>> print(p)\n0.94378087442\n\nVerify the inverse.\n\n>>> gdtrix(1.2, 3.4, p)\n5.5999999999999996");

  /* "scipy/special/_ufuncs.pyx":7425
 *     ">>> gdtrix(1.2, 3.4, p)\n"
 *     "5.5999999999999996")
 * ufunc_gdtrix_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtrix_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":7426
 *     "5.5999999999999996")
 * ufunc_gdtrix_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_gdtrix_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_types[0] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":7427
 * ufunc_gdtrix_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_gdtrix_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtrix_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_types[1] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7428
 * ufunc_gdtrix_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_gdtrix_types[0] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_types[2] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7429
 * ufunc_gdtrix_types[0] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[1] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_types[3] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7430
 * ufunc_gdtrix_types[1] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[2] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtrix_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7431
 * ufunc_gdtrix_types[2] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[3] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtrix_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7432
 * ufunc_gdtrix_types[3] = <char>NPY_FLOAT
 * ufunc_gdtrix_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtrix_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtrix_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7433
 * ufunc_gdtrix_types[4] = <char>NPY_DOUBLE
 * ufunc_gdtrix_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtrix_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtrix_ptr[2*0] = <void*>_func_cdfgam2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7434
 * ufunc_gdtrix_types[5] = <char>NPY_DOUBLE
 * ufunc_gdtrix_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtrix_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_ptr[2*0] = <void*>_func_cdfgam2_wrap
 * ufunc_gdtrix_ptr[2*0+1] = <void*>(<char*>"gdtrix")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7435
 * ufunc_gdtrix_types[6] = <char>NPY_DOUBLE
 * ufunc_gdtrix_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtrix_ptr[2*0] = <void*>_func_cdfgam2_wrap             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_ptr[2*0+1] = <void*>(<char*>"gdtrix")
 * ufunc_gdtrix_ptr[2*1] = <void*>_func_cdfgam2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_ptr[0]) = ((void *)cdfgam2_wrap);

  /* "scipy/special/_ufuncs.pyx":7436
 * ufunc_gdtrix_types[7] = <char>NPY_DOUBLE
 * ufunc_gdtrix_ptr[2*0] = <void*>_func_cdfgam2_wrap
 * ufunc_gdtrix_ptr[2*0+1] = <void*>(<char*>"gdtrix")             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_ptr[2*1] = <void*>_func_cdfgam2_wrap
 * ufunc_gdtrix_ptr[2*1+1] = <void*>(<char*>"gdtrix")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_ptr[1]) = ((void *)((char *)((char *)"gdtrix")));

  /* "scipy/special/_ufuncs.pyx":7437
 * ufunc_gdtrix_ptr[2*0] = <void*>_func_cdfgam2_wrap
 * ufunc_gdtrix_ptr[2*0+1] = <void*>(<char*>"gdtrix")
 * ufunc_gdtrix_ptr[2*1] = <void*>_func_cdfgam2_wrap             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_ptr[2*1+1] = <void*>(<char*>"gdtrix")
 * ufunc_gdtrix_data[0] = &ufunc_gdtrix_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_ptr[2]) = ((void *)cdfgam2_wrap);

  /* "scipy/special/_ufuncs.pyx":7438
 * ufunc_gdtrix_ptr[2*0+1] = <void*>(<char*>"gdtrix")
 * ufunc_gdtrix_ptr[2*1] = <void*>_func_cdfgam2_wrap
 * ufunc_gdtrix_ptr[2*1+1] = <void*>(<char*>"gdtrix")             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_data[0] = &ufunc_gdtrix_ptr[2*0]
 * ufunc_gdtrix_data[1] = &ufunc_gdtrix_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_ptr[3]) = ((void *)((char *)((char *)"gdtrix")));

  /* "scipy/special/_ufuncs.pyx":7439
 * ufunc_gdtrix_ptr[2*1] = <void*>_func_cdfgam2_wrap
 * ufunc_gdtrix_ptr[2*1+1] = <void*>(<char*>"gdtrix")
 * ufunc_gdtrix_data[0] = &ufunc_gdtrix_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_gdtrix_data[1] = &ufunc_gdtrix_ptr[2*1]
 * gdtrix = np.PyUFunc_FromFuncAndData(ufunc_gdtrix_loops, ufunc_gdtrix_data, ufunc_gdtrix_types, 2, 3, 1, 0, "gdtrix", ufunc_gdtrix_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7440
 * ufunc_gdtrix_ptr[2*1+1] = <void*>(<char*>"gdtrix")
 * ufunc_gdtrix_data[0] = &ufunc_gdtrix_ptr[2*0]
 * ufunc_gdtrix_data[1] = &ufunc_gdtrix_ptr[2*1]             # <<<<<<<<<<<<<<
 * gdtrix = np.PyUFunc_FromFuncAndData(ufunc_gdtrix_loops, ufunc_gdtrix_data, ufunc_gdtrix_types, 2, 3, 1, 0, "gdtrix", ufunc_gdtrix_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7441
 * ufunc_gdtrix_data[0] = &ufunc_gdtrix_ptr[2*0]
 * ufunc_gdtrix_data[1] = &ufunc_gdtrix_ptr[2*1]
 * gdtrix = np.PyUFunc_FromFuncAndData(ufunc_gdtrix_loops, ufunc_gdtrix_data, ufunc_gdtrix_types, 2, 3, 1, 0, "gdtrix", ufunc_gdtrix_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hankel1_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_types, 2, 3, 1, 0, ((char *)"gdtrix"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_gdtrix_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7441, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gdtrix, __pyx_t_2) < 0) __PYX_ERR(1, 7441, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":7447
 * cdef void *ufunc_hankel1_data[2]
 * cdef char ufunc_hankel1_types[6]
 * cdef char *ufunc_hankel1_doc = (             # <<<<<<<<<<<<<<
 *     "hankel1(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_doc = ((char *)"hankel1(v, z)\n\nHankel function of the first kind\n\nParameters\n----------\nv : array_like\n    Order (float).\nz : array_like\n    Argument (float or complex).\n\nReturns\n-------\nout : Values of the Hankel function of the first kind.\n\nNotes\n-----\nA wrapper for the AMOS [1]_ routine `zbesh`, which carries out the\ncomputation using the relation,\n\n.. math:: H^{(1)}_v(z) = \\frac{2}{\\imath\\pi} \\exp(-\\imath \\pi v/2) K_v(z \\exp(-\\imath\\pi/2))\n\nwhere :math:`K_v` is the modified Bessel function of the second kind.\nFor negative orders, the relation\n\n.. math:: H^{(1)}_{-v}(z) = H^{(1)}_v(z) \\exp(\\imath\\pi v)\n\nis used.\n\nSee also\n--------\nhankel1e : this function with leading exponential behavior stripped off.\n\nReferences\n----------\n.. [1] Donald E. Amos, \"AMOS, A Portable Package for Bessel Functions\n       of a Complex Argument and Nonnegative Order\",\n       http://netlib.org/amos/");

  /* "scipy/special/_ufuncs.pyx":7486
 *     "       of a Complex Argument and Nonnegative Order\",\n"
 *     "       http://netlib.org/amos/")
 * ufunc_hankel1_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_hankel1_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":7487
 *     "       http://netlib.org/amos/")
 * ufunc_hankel1_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_hankel1_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_hankel1_types[0] = <char>NPY_FLOAT
 * ufunc_hankel1_types[1] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":7488
 * ufunc_hankel1_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_hankel1_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel1_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel1_types[2] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7489
 * ufunc_hankel1_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel1_types[0] = <char>NPY_FLOAT
 * ufunc_hankel1_types[1] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel1_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel1_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_types[1]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":7490
 * ufunc_hankel1_types[0] = <char>NPY_FLOAT
 * ufunc_hankel1_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel1_types[2] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel1_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel1_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_types[2]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":7491
 * ufunc_hankel1_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel1_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel1_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel1_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel1_types[5] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7492
 * ufunc_hankel1_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel1_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel1_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel1_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel1_ptr[2*0] = <void*>_func_cbesh_wrap1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":7493
 * ufunc_hankel1_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel1_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel1_types[5] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel1_ptr[2*0] = <void*>_func_cbesh_wrap1
 * ufunc_hankel1_ptr[2*0+1] = <void*>(<char*>"hankel1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_types[5]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":7494
 * ufunc_hankel1_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel1_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel1_ptr[2*0] = <void*>_func_cbesh_wrap1             # <<<<<<<<<<<<<<
 * ufunc_hankel1_ptr[2*0+1] = <void*>(<char*>"hankel1")
 * ufunc_hankel1_ptr[2*1] = <void*>_func_cbesh_wrap1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_ptr[0]) = ((void *)cbesh_wrap1);

  /* "scipy/special/_ufuncs.pyx":7495
 * ufunc_hankel1_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel1_ptr[2*0] = <void*>_func_cbesh_wrap1
 * ufunc_hankel1_ptr[2*0+1] = <void*>(<char*>"hankel1")             # <<<<<<<<<<<<<<
 * ufunc_hankel1_ptr[2*1] = <void*>_func_cbesh_wrap1
 * ufunc_hankel1_ptr[2*1+1] = <void*>(<char*>"hankel1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_ptr[1]) = ((void *)((char *)((char *)"hankel1")));

  /* "scipy/special/_ufuncs.pyx":7496
 * ufunc_hankel1_ptr[2*0] = <void*>_func_cbesh_wrap1
 * ufunc_hankel1_ptr[2*0+1] = <void*>(<char*>"hankel1")
 * ufunc_hankel1_ptr[2*1] = <void*>_func_cbesh_wrap1             # <<<<<<<<<<<<<<
 * ufunc_hankel1_ptr[2*1+1] = <void*>(<char*>"hankel1")
 * ufunc_hankel1_data[0] = &ufunc_hankel1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_ptr[2]) = ((void *)cbesh_wrap1);

  /* "scipy/special/_ufuncs.pyx":7497
 * ufunc_hankel1_ptr[2*0+1] = <void*>(<char*>"hankel1")
 * ufunc_hankel1_ptr[2*1] = <void*>_func_cbesh_wrap1
 * ufunc_hankel1_ptr[2*1+1] = <void*>(<char*>"hankel1")             # <<<<<<<<<<<<<<
 * ufunc_hankel1_data[0] = &ufunc_hankel1_ptr[2*0]
 * ufunc_hankel1_data[1] = &ufunc_hankel1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_ptr[3]) = ((void *)((char *)((char *)"hankel1")));

  /* "scipy/special/_ufuncs.pyx":7498
 * ufunc_hankel1_ptr[2*1] = <void*>_func_cbesh_wrap1
 * ufunc_hankel1_ptr[2*1+1] = <void*>(<char*>"hankel1")
 * ufunc_hankel1_data[0] = &ufunc_hankel1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hankel1_data[1] = &ufunc_hankel1_ptr[2*1]
 * hankel1 = np.PyUFunc_FromFuncAndData(ufunc_hankel1_loops, ufunc_hankel1_data, ufunc_hankel1_types, 2, 2, 1, 0, "hankel1", ufunc_hankel1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7499
 * ufunc_hankel1_ptr[2*1+1] = <void*>(<char*>"hankel1")
 * ufunc_hankel1_data[0] = &ufunc_hankel1_ptr[2*0]
 * ufunc_hankel1_data[1] = &ufunc_hankel1_ptr[2*1]             # <<<<<<<<<<<<<<
 * hankel1 = np.PyUFunc_FromFuncAndData(ufunc_hankel1_loops, ufunc_hankel1_data, ufunc_hankel1_types, 2, 2, 1, 0, "hankel1", ufunc_hankel1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7500
 * ufunc_hankel1_data[0] = &ufunc_hankel1_ptr[2*0]
 * ufunc_hankel1_data[1] = &ufunc_hankel1_ptr[2*1]
 * hankel1 = np.PyUFunc_FromFuncAndData(ufunc_hankel1_loops, ufunc_hankel1_data, ufunc_hankel1_types, 2, 2, 1, 0, "hankel1", ufunc_hankel1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hankel1e_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_types, 2, 2, 1, 0, ((char *)"hankel1"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7500, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hankel1, __pyx_t_2) < 0) __PYX_ERR(1, 7500, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":7506
 * cdef void *ufunc_hankel1e_data[2]
 * cdef char ufunc_hankel1e_types[6]
 * cdef char *ufunc_hankel1e_doc = (             # <<<<<<<<<<<<<<
 *     "hankel1e(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_doc = ((char *)"hankel1e(v, z)\n\nExponentially scaled Hankel function of the first kind\n\nDefined as::\n\n    hankel1e(v, z) = hankel1(v, z) * exp(-1j * z)\n\nParameters\n----------\nv : array_like\n    Order (float).\nz : array_like\n    Argument (float or complex).\n\nReturns\n-------\nout : Values of the exponentially scaled Hankel function.\n\nNotes\n-----\nA wrapper for the AMOS [1]_ routine `zbesh`, which carries out the\ncomputation using the relation,\n\n.. math:: H^{(1)}_v(z) = \\frac{2}{\\imath\\pi} \\exp(-\\imath \\pi v/2) K_v(z \\exp(-\\imath\\pi/2))\n\nwhere :math:`K_v` is the modified Bessel function of the second kind.\nFor negative orders, the relation\n\n.. math:: H^{(1)}_{-v}(z) = H^{(1)}_v(z) \\exp(\\imath\\pi v)\n\nis used.\n\nReferences\n----------\n.. [1] Donald E. Amos, \"AMOS, A Portable Package for Bessel Functions\n       of a Complex Argument and Nonnegative Order\",\n       http://netlib.org/amos/");

  /* "scipy/special/_ufuncs.pyx":7545
 *     "       of a Complex Argument and Nonnegative Order\",\n"
 *     "       http://netlib.org/amos/")
 * ufunc_hankel1e_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel1e_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":7546
 *     "       http://netlib.org/amos/")
 * ufunc_hankel1e_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_hankel1e_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_types[0] = <char>NPY_FLOAT
 * ufunc_hankel1e_types[1] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":7547
 * ufunc_hankel1e_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_hankel1e_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel1e_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel1e_types[2] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7548
 * ufunc_hankel1e_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel1e_types[0] = <char>NPY_FLOAT
 * ufunc_hankel1e_types[1] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel1e_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_types[1]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":7549
 * ufunc_hankel1e_types[0] = <char>NPY_FLOAT
 * ufunc_hankel1e_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel1e_types[2] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel1e_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_types[2]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":7550
 * ufunc_hankel1e_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel1e_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel1e_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel1e_types[5] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7551
 * ufunc_hankel1e_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel1e_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel1e_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel1e_ptr[2*0] = <void*>_func_cbesh_wrap1_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":7552
 * ufunc_hankel1e_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel1e_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel1e_types[5] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_ptr[2*0] = <void*>_func_cbesh_wrap1_e
 * ufunc_hankel1e_ptr[2*0+1] = <void*>(<char*>"hankel1e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_types[5]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":7553
 * ufunc_hankel1e_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel1e_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel1e_ptr[2*0] = <void*>_func_cbesh_wrap1_e             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_ptr[2*0+1] = <void*>(<char*>"hankel1e")
 * ufunc_hankel1e_ptr[2*1] = <void*>_func_cbesh_wrap1_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_ptr[0]) = ((void *)cbesh_wrap1_e);

  /* "scipy/special/_ufuncs.pyx":7554
 * ufunc_hankel1e_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel1e_ptr[2*0] = <void*>_func_cbesh_wrap1_e
 * ufunc_hankel1e_ptr[2*0+1] = <void*>(<char*>"hankel1e")             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_ptr[2*1] = <void*>_func_cbesh_wrap1_e
 * ufunc_hankel1e_ptr[2*1+1] = <void*>(<char*>"hankel1e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_ptr[1]) = ((void *)((char *)((char *)"hankel1e")));

  /* "scipy/special/_ufuncs.pyx":7555
 * ufunc_hankel1e_ptr[2*0] = <void*>_func_cbesh_wrap1_e
 * ufunc_hankel1e_ptr[2*0+1] = <void*>(<char*>"hankel1e")
 * ufunc_hankel1e_ptr[2*1] = <void*>_func_cbesh_wrap1_e             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_ptr[2*1+1] = <void*>(<char*>"hankel1e")
 * ufunc_hankel1e_data[0] = &ufunc_hankel1e_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_ptr[2]) = ((void *)cbesh_wrap1_e);

  /* "scipy/special/_ufuncs.pyx":7556
 * ufunc_hankel1e_ptr[2*0+1] = <void*>(<char*>"hankel1e")
 * ufunc_hankel1e_ptr[2*1] = <void*>_func_cbesh_wrap1_e
 * ufunc_hankel1e_ptr[2*1+1] = <void*>(<char*>"hankel1e")             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_data[0] = &ufunc_hankel1e_ptr[2*0]
 * ufunc_hankel1e_data[1] = &ufunc_hankel1e_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_ptr[3]) = ((void *)((char *)((char *)"hankel1e")));

  /* "scipy/special/_ufuncs.pyx":7557
 * ufunc_hankel1e_ptr[2*1] = <void*>_func_cbesh_wrap1_e
 * ufunc_hankel1e_ptr[2*1+1] = <void*>(<char*>"hankel1e")
 * ufunc_hankel1e_data[0] = &ufunc_hankel1e_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hankel1e_data[1] = &ufunc_hankel1e_ptr[2*1]
 * hankel1e = np.PyUFunc_FromFuncAndData(ufunc_hankel1e_loops, ufunc_hankel1e_data, ufunc_hankel1e_types, 2, 2, 1, 0, "hankel1e", ufunc_hankel1e_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7558
 * ufunc_hankel1e_ptr[2*1+1] = <void*>(<char*>"hankel1e")
 * ufunc_hankel1e_data[0] = &ufunc_hankel1e_ptr[2*0]
 * ufunc_hankel1e_data[1] = &ufunc_hankel1e_ptr[2*1]             # <<<<<<<<<<<<<<
 * hankel1e = np.PyUFunc_FromFuncAndData(ufunc_hankel1e_loops, ufunc_hankel1e_data, ufunc_hankel1e_types, 2, 2, 1, 0, "hankel1e", ufunc_hankel1e_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7559
 * ufunc_hankel1e_data[0] = &ufunc_hankel1e_ptr[2*0]
 * ufunc_hankel1e_data[1] = &ufunc_hankel1e_ptr[2*1]
 * hankel1e = np.PyUFunc_FromFuncAndData(ufunc_hankel1e_loops, ufunc_hankel1e_data, ufunc_hankel1e_types, 2, 2, 1, 0, "hankel1e", ufunc_hankel1e_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hankel2_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_types, 2, 2, 1, 0, ((char *)"hankel1e"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel1e_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7559, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hankel1e, __pyx_t_2) < 0) __PYX_ERR(1, 7559, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":7565
 * cdef void *ufunc_hankel2_data[2]
 * cdef char ufunc_hankel2_types[6]
 * cdef char *ufunc_hankel2_doc = (             # <<<<<<<<<<<<<<
 *     "hankel2(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_doc = ((char *)"hankel2(v, z)\n\nHankel function of the second kind\n\nParameters\n----------\nv : array_like\n    Order (float).\nz : array_like\n    Argument (float or complex).\n\nReturns\n-------\nout : Values of the Hankel function of the second kind.\n\nNotes\n-----\nA wrapper for the AMOS [1]_ routine `zbesh`, which carries out the\ncomputation using the relation,\n\n.. math:: H^{(2)}_v(z) = -\\frac{2}{\\imath\\pi} \\exp(\\imath \\pi v/2) K_v(z \\exp(\\imath\\pi/2))\n\nwhere :math:`K_v` is the modified Bessel function of the second kind.\nFor negative orders, the relation\n\n.. math:: H^{(2)}_{-v}(z) = H^{(2)}_v(z) \\exp(-\\imath\\pi v)\n\nis used.\n\nSee also\n--------\nhankel2e : this function with leading exponential behavior stripped off.\n\nReferences\n----------\n.. [1] Donald E. Amos, \"AMOS, A Portable Package for Bessel Functions\n       of a Complex Argument and Nonnegative Order\",\n       http://netlib.org/amos/");

  /* "scipy/special/_ufuncs.pyx":7604
 *     "       of a Complex Argument and Nonnegative Order\",\n"
 *     "       http://netlib.org/amos/")
 * ufunc_hankel2_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_hankel2_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel2_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":7605
 *     "       http://netlib.org/amos/")
 * ufunc_hankel2_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_hankel2_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_hankel2_types[0] = <char>NPY_FLOAT
 * ufunc_hankel2_types[1] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":7606
 * ufunc_hankel2_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_hankel2_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel2_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel2_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel2_types[2] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7607
 * ufunc_hankel2_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel2_types[0] = <char>NPY_FLOAT
 * ufunc_hankel2_types[1] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel2_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel2_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_types[1]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":7608
 * ufunc_hankel2_types[0] = <char>NPY_FLOAT
 * ufunc_hankel2_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel2_types[2] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel2_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel2_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_types[2]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":7609
 * ufunc_hankel2_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel2_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel2_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel2_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel2_types[5] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7610
 * ufunc_hankel2_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel2_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel2_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel2_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel2_ptr[2*0] = <void*>_func_cbesh_wrap2
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":7611
 * ufunc_hankel2_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel2_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel2_types[5] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel2_ptr[2*0] = <void*>_func_cbesh_wrap2
 * ufunc_hankel2_ptr[2*0+1] = <void*>(<char*>"hankel2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_types[5]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":7612
 * ufunc_hankel2_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel2_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel2_ptr[2*0] = <void*>_func_cbesh_wrap2             # <<<<<<<<<<<<<<
 * ufunc_hankel2_ptr[2*0+1] = <void*>(<char*>"hankel2")
 * ufunc_hankel2_ptr[2*1] = <void*>_func_cbesh_wrap2
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_ptr[0]) = ((void *)cbesh_wrap2);

  /* "scipy/special/_ufuncs.pyx":7613
 * ufunc_hankel2_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel2_ptr[2*0] = <void*>_func_cbesh_wrap2
 * ufunc_hankel2_ptr[2*0+1] = <void*>(<char*>"hankel2")             # <<<<<<<<<<<<<<
 * ufunc_hankel2_ptr[2*1] = <void*>_func_cbesh_wrap2
 * ufunc_hankel2_ptr[2*1+1] = <void*>(<char*>"hankel2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_ptr[1]) = ((void *)((char *)((char *)"hankel2")));

  /* "scipy/special/_ufuncs.pyx":7614
 * ufunc_hankel2_ptr[2*0] = <void*>_func_cbesh_wrap2
 * ufunc_hankel2_ptr[2*0+1] = <void*>(<char*>"hankel2")
 * ufunc_hankel2_ptr[2*1] = <void*>_func_cbesh_wrap2             # <<<<<<<<<<<<<<
 * ufunc_hankel2_ptr[2*1+1] = <void*>(<char*>"hankel2")
 * ufunc_hankel2_data[0] = &ufunc_hankel2_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_ptr[2]) = ((void *)cbesh_wrap2);

  /* "scipy/special/_ufuncs.pyx":7615
 * ufunc_hankel2_ptr[2*0+1] = <void*>(<char*>"hankel2")
 * ufunc_hankel2_ptr[2*1] = <void*>_func_cbesh_wrap2
 * ufunc_hankel2_ptr[2*1+1] = <void*>(<char*>"hankel2")             # <<<<<<<<<<<<<<
 * ufunc_hankel2_data[0] = &ufunc_hankel2_ptr[2*0]
 * ufunc_hankel2_data[1] = &ufunc_hankel2_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_ptr[3]) = ((void *)((char *)((char *)"hankel2")));

  /* "scipy/special/_ufuncs.pyx":7616
 * ufunc_hankel2_ptr[2*1] = <void*>_func_cbesh_wrap2
 * ufunc_hankel2_ptr[2*1+1] = <void*>(<char*>"hankel2")
 * ufunc_hankel2_data[0] = &ufunc_hankel2_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hankel2_data[1] = &ufunc_hankel2_ptr[2*1]
 * hankel2 = np.PyUFunc_FromFuncAndData(ufunc_hankel2_loops, ufunc_hankel2_data, ufunc_hankel2_types, 2, 2, 1, 0, "hankel2", ufunc_hankel2_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7617
 * ufunc_hankel2_ptr[2*1+1] = <void*>(<char*>"hankel2")
 * ufunc_hankel2_data[0] = &ufunc_hankel2_ptr[2*0]
 * ufunc_hankel2_data[1] = &ufunc_hankel2_ptr[2*1]             # <<<<<<<<<<<<<<
 * hankel2 = np.PyUFunc_FromFuncAndData(ufunc_hankel2_loops, ufunc_hankel2_data, ufunc_hankel2_types, 2, 2, 1, 0, "hankel2", ufunc_hankel2_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7618
 * ufunc_hankel2_data[0] = &ufunc_hankel2_ptr[2*0]
 * ufunc_hankel2_data[1] = &ufunc_hankel2_ptr[2*1]
 * hankel2 = np.PyUFunc_FromFuncAndData(ufunc_hankel2_loops, ufunc_hankel2_data, ufunc_hankel2_types, 2, 2, 1, 0, "hankel2", ufunc_hankel2_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hankel2e_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_types, 2, 2, 1, 0, ((char *)"hankel2"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hankel2, __pyx_t_2) < 0) __PYX_ERR(1, 7618, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":7624
 * cdef void *ufunc_hankel2e_data[2]
 * cdef char ufunc_hankel2e_types[6]
 * cdef char *ufunc_hankel2e_doc = (             # <<<<<<<<<<<<<<
 *     "hankel2e(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_doc = ((char *)"hankel2e(v, z)\n\nExponentially scaled Hankel function of the second kind\n\nDefined as::\n\n    hankel2e(v, z) = hankel2(v, z) * exp(1j * z)\n\nParameters\n----------\nv : array_like\n    Order (float).\nz : array_like\n    Argument (float or complex).\n\nReturns\n-------\nout : Values of the exponentially scaled Hankel function of the second kind.\n\nNotes\n-----\nA wrapper for the AMOS [1]_ routine `zbesh`, which carries out the\ncomputation using the relation,\n\n.. math:: H^{(2)}_v(z) = -\\frac{2}{\\imath\\pi} \\exp(\\frac{\\imath \\pi v}{2}) K_v(z exp(\\frac{\\imath\\pi}{2}))\n\nwhere :math:`K_v` is the modified Bessel function of the second kind.\nFor negative orders, the relation\n\n.. math:: H^{(2)}_{-v}(z) = H^{(2)}_v(z) \\exp(-\\imath\\pi v)\n\nis used.\n\nReferences\n----------\n.. [1] Donald E. Amos, \"AMOS, A Portable Package for Bessel Functions\n       of a Complex Argument and Nonnegative Order\",\n       http://netlib.org/amos/");

  /* "scipy/special/_ufuncs.pyx":7663
 *     "       of a Complex Argument and Nonnegative Order\",\n"
 *     "       http://netlib.org/amos/")
 * ufunc_hankel2e_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel2e_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":7664
 *     "       http://netlib.org/amos/")
 * ufunc_hankel2e_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_hankel2e_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_types[0] = <char>NPY_FLOAT
 * ufunc_hankel2e_types[1] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":7665
 * ufunc_hankel2e_loops[0] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_hankel2e_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel2e_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel2e_types[2] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7666
 * ufunc_hankel2e_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hankel2e_types[0] = <char>NPY_FLOAT
 * ufunc_hankel2e_types[1] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel2e_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_types[1]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":7667
 * ufunc_hankel2e_types[0] = <char>NPY_FLOAT
 * ufunc_hankel2e_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel2e_types[2] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel2e_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_types[2]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":7668
 * ufunc_hankel2e_types[1] = <char>NPY_CFLOAT
 * ufunc_hankel2e_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel2e_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel2e_types[5] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7669
 * ufunc_hankel2e_types[2] = <char>NPY_CFLOAT
 * ufunc_hankel2e_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel2e_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel2e_ptr[2*0] = <void*>_func_cbesh_wrap2_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":7670
 * ufunc_hankel2e_types[3] = <char>NPY_DOUBLE
 * ufunc_hankel2e_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel2e_types[5] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_ptr[2*0] = <void*>_func_cbesh_wrap2_e
 * ufunc_hankel2e_ptr[2*0+1] = <void*>(<char*>"hankel2e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_types[5]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":7671
 * ufunc_hankel2e_types[4] = <char>NPY_CDOUBLE
 * ufunc_hankel2e_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel2e_ptr[2*0] = <void*>_func_cbesh_wrap2_e             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_ptr[2*0+1] = <void*>(<char*>"hankel2e")
 * ufunc_hankel2e_ptr[2*1] = <void*>_func_cbesh_wrap2_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_ptr[0]) = ((void *)cbesh_wrap2_e);

  /* "scipy/special/_ufuncs.pyx":7672
 * ufunc_hankel2e_types[5] = <char>NPY_CDOUBLE
 * ufunc_hankel2e_ptr[2*0] = <void*>_func_cbesh_wrap2_e
 * ufunc_hankel2e_ptr[2*0+1] = <void*>(<char*>"hankel2e")             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_ptr[2*1] = <void*>_func_cbesh_wrap2_e
 * ufunc_hankel2e_ptr[2*1+1] = <void*>(<char*>"hankel2e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_ptr[1]) = ((void *)((char *)((char *)"hankel2e")));

  /* "scipy/special/_ufuncs.pyx":7673
 * ufunc_hankel2e_ptr[2*0] = <void*>_func_cbesh_wrap2_e
 * ufunc_hankel2e_ptr[2*0+1] = <void*>(<char*>"hankel2e")
 * ufunc_hankel2e_ptr[2*1] = <void*>_func_cbesh_wrap2_e             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_ptr[2*1+1] = <void*>(<char*>"hankel2e")
 * ufunc_hankel2e_data[0] = &ufunc_hankel2e_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_ptr[2]) = ((void *)cbesh_wrap2_e);

  /* "scipy/special/_ufuncs.pyx":7674
 * ufunc_hankel2e_ptr[2*0+1] = <void*>(<char*>"hankel2e")
 * ufunc_hankel2e_ptr[2*1] = <void*>_func_cbesh_wrap2_e
 * ufunc_hankel2e_ptr[2*1+1] = <void*>(<char*>"hankel2e")             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_data[0] = &ufunc_hankel2e_ptr[2*0]
 * ufunc_hankel2e_data[1] = &ufunc_hankel2e_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_ptr[3]) = ((void *)((char *)((char *)"hankel2e")));

  /* "scipy/special/_ufuncs.pyx":7675
 * ufunc_hankel2e_ptr[2*1] = <void*>_func_cbesh_wrap2_e
 * ufunc_hankel2e_ptr[2*1+1] = <void*>(<char*>"hankel2e")
 * ufunc_hankel2e_data[0] = &ufunc_hankel2e_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hankel2e_data[1] = &ufunc_hankel2e_ptr[2*1]
 * hankel2e = np.PyUFunc_FromFuncAndData(ufunc_hankel2e_loops, ufunc_hankel2e_data, ufunc_hankel2e_types, 2, 2, 1, 0, "hankel2e", ufunc_hankel2e_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7676
 * ufunc_hankel2e_ptr[2*1+1] = <void*>(<char*>"hankel2e")
 * ufunc_hankel2e_data[0] = &ufunc_hankel2e_ptr[2*0]
 * ufunc_hankel2e_data[1] = &ufunc_hankel2e_ptr[2*1]             # <<<<<<<<<<<<<<
 * hankel2e = np.PyUFunc_FromFuncAndData(ufunc_hankel2e_loops, ufunc_hankel2e_data, ufunc_hankel2e_types, 2, 2, 1, 0, "hankel2e", ufunc_hankel2e_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7677
 * ufunc_hankel2e_data[0] = &ufunc_hankel2e_ptr[2*0]
 * ufunc_hankel2e_data[1] = &ufunc_hankel2e_ptr[2*1]
 * hankel2e = np.PyUFunc_FromFuncAndData(ufunc_hankel2e_loops, ufunc_hankel2e_data, ufunc_hankel2e_types, 2, 2, 1, 0, "hankel2e", ufunc_hankel2e_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_huber_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_types, 2, 2, 1, 0, ((char *)"hankel2e"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_hankel2e_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7677, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hankel2e, __pyx_t_2) < 0) __PYX_ERR(1, 7677, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":7683
 * cdef void *ufunc_huber_data[2]
 * cdef char ufunc_huber_types[6]
 * cdef char *ufunc_huber_doc = (             # <<<<<<<<<<<<<<
 *     "huber(delta, r)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_doc = ((char *)"huber(delta, r)\n\nHuber loss function.\n\n.. math:: \\text{huber}(\\delta, r) = \\begin{cases} \\infty & \\delta < 0  \\\\ \\frac{1}{2}r^2 & 0 \\le \\delta, | r | \\le \\delta \\\\ \\delta ( |r| - \\frac{1}{2}\\delta ) & \\text{otherwise} \\end{cases}\n\nParameters\n----------\ndelta : ndarray\n    Input array, indicating the quadratic vs. linear loss changepoint.\nr : ndarray\n    Input array, possibly representing residuals.\n\nReturns\n-------\nres : ndarray\n    The computed Huber loss function values.\n\nNotes\n-----\nThis function is convex in r.\n\n.. versionadded:: 0.15.0");

  /* "scipy/special/_ufuncs.pyx":7707
 *     "\n"
 *     ".. versionadded:: 0.15.0")
 * ufunc_huber_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_huber_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_huber_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":7708
 *     ".. versionadded:: 0.15.0")
 * ufunc_huber_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_huber_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_huber_types[0] = <char>NPY_FLOAT
 * ufunc_huber_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":7709
 * ufunc_huber_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_huber_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_huber_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_huber_types[1] = <char>NPY_FLOAT
 * ufunc_huber_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7710
 * ufunc_huber_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_huber_types[0] = <char>NPY_FLOAT
 * ufunc_huber_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_huber_types[2] = <char>NPY_FLOAT
 * ufunc_huber_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7711
 * ufunc_huber_types[0] = <char>NPY_FLOAT
 * ufunc_huber_types[1] = <char>NPY_FLOAT
 * ufunc_huber_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_huber_types[3] = <char>NPY_DOUBLE
 * ufunc_huber_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7712
 * ufunc_huber_types[1] = <char>NPY_FLOAT
 * ufunc_huber_types[2] = <char>NPY_FLOAT
 * ufunc_huber_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_huber_types[4] = <char>NPY_DOUBLE
 * ufunc_huber_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7713
 * ufunc_huber_types[2] = <char>NPY_FLOAT
 * ufunc_huber_types[3] = <char>NPY_DOUBLE
 * ufunc_huber_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_huber_types[5] = <char>NPY_DOUBLE
 * ufunc_huber_ptr[2*0] = <void*>_func_huber
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7714
 * ufunc_huber_types[3] = <char>NPY_DOUBLE
 * ufunc_huber_types[4] = <char>NPY_DOUBLE
 * ufunc_huber_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_huber_ptr[2*0] = <void*>_func_huber
 * ufunc_huber_ptr[2*0+1] = <void*>(<char*>"huber")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7715
 * ufunc_huber_types[4] = <char>NPY_DOUBLE
 * ufunc_huber_types[5] = <char>NPY_DOUBLE
 * ufunc_huber_ptr[2*0] = <void*>_func_huber             # <<<<<<<<<<<<<<
 * ufunc_huber_ptr[2*0+1] = <void*>(<char*>"huber")
 * ufunc_huber_ptr[2*1] = <void*>_func_huber
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_ptr[0]) = ((void *)__pyx_f_5scipy_7special_16_convex_analysis_huber);

  /* "scipy/special/_ufuncs.pyx":7716
 * ufunc_huber_types[5] = <char>NPY_DOUBLE
 * ufunc_huber_ptr[2*0] = <void*>_func_huber
 * ufunc_huber_ptr[2*0+1] = <void*>(<char*>"huber")             # <<<<<<<<<<<<<<
 * ufunc_huber_ptr[2*1] = <void*>_func_huber
 * ufunc_huber_ptr[2*1+1] = <void*>(<char*>"huber")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_ptr[1]) = ((void *)((char *)((char *)"huber")));

  /* "scipy/special/_ufuncs.pyx":7717
 * ufunc_huber_ptr[2*0] = <void*>_func_huber
 * ufunc_huber_ptr[2*0+1] = <void*>(<char*>"huber")
 * ufunc_huber_ptr[2*1] = <void*>_func_huber             # <<<<<<<<<<<<<<
 * ufunc_huber_ptr[2*1+1] = <void*>(<char*>"huber")
 * ufunc_huber_data[0] = &ufunc_huber_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_ptr[2]) = ((void *)__pyx_f_5scipy_7special_16_convex_analysis_huber);

  /* "scipy/special/_ufuncs.pyx":7718
 * ufunc_huber_ptr[2*0+1] = <void*>(<char*>"huber")
 * ufunc_huber_ptr[2*1] = <void*>_func_huber
 * ufunc_huber_ptr[2*1+1] = <void*>(<char*>"huber")             # <<<<<<<<<<<<<<
 * ufunc_huber_data[0] = &ufunc_huber_ptr[2*0]
 * ufunc_huber_data[1] = &ufunc_huber_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_ptr[3]) = ((void *)((char *)((char *)"huber")));

  /* "scipy/special/_ufuncs.pyx":7719
 * ufunc_huber_ptr[2*1] = <void*>_func_huber
 * ufunc_huber_ptr[2*1+1] = <void*>(<char*>"huber")
 * ufunc_huber_data[0] = &ufunc_huber_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_huber_data[1] = &ufunc_huber_ptr[2*1]
 * huber = np.PyUFunc_FromFuncAndData(ufunc_huber_loops, ufunc_huber_data, ufunc_huber_types, 2, 2, 1, 0, "huber", ufunc_huber_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7720
 * ufunc_huber_ptr[2*1+1] = <void*>(<char*>"huber")
 * ufunc_huber_data[0] = &ufunc_huber_ptr[2*0]
 * ufunc_huber_data[1] = &ufunc_huber_ptr[2*1]             # <<<<<<<<<<<<<<
 * huber = np.PyUFunc_FromFuncAndData(ufunc_huber_loops, ufunc_huber_data, ufunc_huber_types, 2, 2, 1, 0, "huber", ufunc_huber_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7721
 * ufunc_huber_data[0] = &ufunc_huber_ptr[2*0]
 * ufunc_huber_data[1] = &ufunc_huber_ptr[2*1]
 * huber = np.PyUFunc_FromFuncAndData(ufunc_huber_loops, ufunc_huber_data, ufunc_huber_types, 2, 2, 1, 0, "huber", ufunc_huber_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hyp0f1_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_types, 2, 2, 1, 0, ((char *)"huber"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_huber_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7721, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_huber, __pyx_t_2) < 0) __PYX_ERR(1, 7721, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":7727
 * cdef void *ufunc_hyp0f1_data[4]
 * cdef char ufunc_hyp0f1_types[12]
 * cdef char *ufunc_hyp0f1_doc = (             # <<<<<<<<<<<<<<
 *     "hyp0f1(v, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_doc = ((char *)"hyp0f1(v, x)\n\nConfluent hypergeometric limit function 0F1.\n\nParameters\n----------\nv, z : array_like\n    Input values.\n\nReturns\n-------\nhyp0f1 : ndarray\n    The confluent hypergeometric limit function.\n\nNotes\n-----\nThis function is defined as:\n\n.. math:: _0F_1(v, z) = \\sum_{k=0}^{\\infty}\\frac{z^k}{(v)_k k!}.\n\nIt's also the limit as :math:`q \\to \\infty` of :math:`_1F_1(q; v; z/q)`,\nand satisfies the differential equation :math:`f''(z) + vf'(z) = f(z)`.");

  /* "scipy/special/_ufuncs.pyx":7750
 *     "It's also the limit as :math:`q \\to \\infty` of :math:`_1F_1(q; v; z/q)`,\n"
 *     "and satisfies the differential equation :math:`f''(z) + vf'(z) = f(z)`.")
 * ufunc_hyp0f1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_hyp0f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":7751
 *     "and satisfies the differential equation :math:`f''(z) + vf'(z) = f(z)`.")
 * ufunc_hyp0f1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_hyp0f1_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_hyp0f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":7752
 * ufunc_hyp0f1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_hyp0f1_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_hyp0f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hyp0f1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":7753
 * ufunc_hyp0f1_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_hyp0f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_hyp0f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_types[0] = <char>NPY_FLOAT
 * ufunc_hyp0f1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":7754
 * ufunc_hyp0f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_hyp0f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hyp0f1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_types[1] = <char>NPY_FLOAT
 * ufunc_hyp0f1_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7755
 * ufunc_hyp0f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_hyp0f1_types[0] = <char>NPY_FLOAT
 * ufunc_hyp0f1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_types[2] = <char>NPY_FLOAT
 * ufunc_hyp0f1_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7756
 * ufunc_hyp0f1_types[0] = <char>NPY_FLOAT
 * ufunc_hyp0f1_types[1] = <char>NPY_FLOAT
 * ufunc_hyp0f1_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_types[3] = <char>NPY_FLOAT
 * ufunc_hyp0f1_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7757
 * ufunc_hyp0f1_types[1] = <char>NPY_FLOAT
 * ufunc_hyp0f1_types[2] = <char>NPY_FLOAT
 * ufunc_hyp0f1_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_types[4] = <char>NPY_CFLOAT
 * ufunc_hyp0f1_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7758
 * ufunc_hyp0f1_types[2] = <char>NPY_FLOAT
 * ufunc_hyp0f1_types[3] = <char>NPY_FLOAT
 * ufunc_hyp0f1_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_types[5] = <char>NPY_CFLOAT
 * ufunc_hyp0f1_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":7759
 * ufunc_hyp0f1_types[3] = <char>NPY_FLOAT
 * ufunc_hyp0f1_types[4] = <char>NPY_CFLOAT
 * ufunc_hyp0f1_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp0f1_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":7760
 * ufunc_hyp0f1_types[4] = <char>NPY_CFLOAT
 * ufunc_hyp0f1_types[5] = <char>NPY_CFLOAT
 * ufunc_hyp0f1_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp0f1_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7761
 * ufunc_hyp0f1_types[5] = <char>NPY_CFLOAT
 * ufunc_hyp0f1_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp0f1_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp0f1_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7762
 * ufunc_hyp0f1_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp0f1_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp0f1_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp0f1_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7763
 * ufunc_hyp0f1_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp0f1_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp0f1_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_types[10] = <char>NPY_CDOUBLE
 * ufunc_hyp0f1_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7764
 * ufunc_hyp0f1_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp0f1_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp0f1_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_types[11] = <char>NPY_CDOUBLE
 * ufunc_hyp0f1_ptr[2*0] = <void*>_func__hyp0f1_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":7765
 * ufunc_hyp0f1_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp0f1_types[10] = <char>NPY_CDOUBLE
 * ufunc_hyp0f1_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_ptr[2*0] = <void*>_func__hyp0f1_real
 * ufunc_hyp0f1_ptr[2*0+1] = <void*>(<char*>"hyp0f1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":7766
 * ufunc_hyp0f1_types[10] = <char>NPY_CDOUBLE
 * ufunc_hyp0f1_types[11] = <char>NPY_CDOUBLE
 * ufunc_hyp0f1_ptr[2*0] = <void*>_func__hyp0f1_real             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_ptr[2*0+1] = <void*>(<char*>"hyp0f1")
 * ufunc_hyp0f1_ptr[2*1] = <void*>_func__hyp0f1_cmplx
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_ptr[0]) = ((void *)__pyx_f_5scipy_7special_7_hyp0f1__hyp0f1_real);

  /* "scipy/special/_ufuncs.pyx":7767
 * ufunc_hyp0f1_types[11] = <char>NPY_CDOUBLE
 * ufunc_hyp0f1_ptr[2*0] = <void*>_func__hyp0f1_real
 * ufunc_hyp0f1_ptr[2*0+1] = <void*>(<char*>"hyp0f1")             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_ptr[2*1] = <void*>_func__hyp0f1_cmplx
 * ufunc_hyp0f1_ptr[2*1+1] = <void*>(<char*>"hyp0f1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_ptr[1]) = ((void *)((char *)((char *)"hyp0f1")));

  /* "scipy/special/_ufuncs.pyx":7768
 * ufunc_hyp0f1_ptr[2*0] = <void*>_func__hyp0f1_real
 * ufunc_hyp0f1_ptr[2*0+1] = <void*>(<char*>"hyp0f1")
 * ufunc_hyp0f1_ptr[2*1] = <void*>_func__hyp0f1_cmplx             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_ptr[2*1+1] = <void*>(<char*>"hyp0f1")
 * ufunc_hyp0f1_ptr[2*2] = <void*>_func__hyp0f1_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_hyp0f1__hyp0f1_cmplx);

  /* "scipy/special/_ufuncs.pyx":7769
 * ufunc_hyp0f1_ptr[2*0+1] = <void*>(<char*>"hyp0f1")
 * ufunc_hyp0f1_ptr[2*1] = <void*>_func__hyp0f1_cmplx
 * ufunc_hyp0f1_ptr[2*1+1] = <void*>(<char*>"hyp0f1")             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_ptr[2*2] = <void*>_func__hyp0f1_real
 * ufunc_hyp0f1_ptr[2*2+1] = <void*>(<char*>"hyp0f1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_ptr[3]) = ((void *)((char *)((char *)"hyp0f1")));

  /* "scipy/special/_ufuncs.pyx":7770
 * ufunc_hyp0f1_ptr[2*1] = <void*>_func__hyp0f1_cmplx
 * ufunc_hyp0f1_ptr[2*1+1] = <void*>(<char*>"hyp0f1")
 * ufunc_hyp0f1_ptr[2*2] = <void*>_func__hyp0f1_real             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_ptr[2*2+1] = <void*>(<char*>"hyp0f1")
 * ufunc_hyp0f1_ptr[2*3] = <void*>_func__hyp0f1_cmplx
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_hyp0f1__hyp0f1_real);

  /* "scipy/special/_ufuncs.pyx":7771
 * ufunc_hyp0f1_ptr[2*1+1] = <void*>(<char*>"hyp0f1")
 * ufunc_hyp0f1_ptr[2*2] = <void*>_func__hyp0f1_real
 * ufunc_hyp0f1_ptr[2*2+1] = <void*>(<char*>"hyp0f1")             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_ptr[2*3] = <void*>_func__hyp0f1_cmplx
 * ufunc_hyp0f1_ptr[2*3+1] = <void*>(<char*>"hyp0f1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_ptr[5]) = ((void *)((char *)((char *)"hyp0f1")));

  /* "scipy/special/_ufuncs.pyx":7772
 * ufunc_hyp0f1_ptr[2*2] = <void*>_func__hyp0f1_real
 * ufunc_hyp0f1_ptr[2*2+1] = <void*>(<char*>"hyp0f1")
 * ufunc_hyp0f1_ptr[2*3] = <void*>_func__hyp0f1_cmplx             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_ptr[2*3+1] = <void*>(<char*>"hyp0f1")
 * ufunc_hyp0f1_data[0] = &ufunc_hyp0f1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_ptr[6]) = ((void *)__pyx_f_5scipy_7special_7_hyp0f1__hyp0f1_cmplx);

  /* "scipy/special/_ufuncs.pyx":7773
 * ufunc_hyp0f1_ptr[2*2+1] = <void*>(<char*>"hyp0f1")
 * ufunc_hyp0f1_ptr[2*3] = <void*>_func__hyp0f1_cmplx
 * ufunc_hyp0f1_ptr[2*3+1] = <void*>(<char*>"hyp0f1")             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_data[0] = &ufunc_hyp0f1_ptr[2*0]
 * ufunc_hyp0f1_data[1] = &ufunc_hyp0f1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_ptr[7]) = ((void *)((char *)((char *)"hyp0f1")));

  /* "scipy/special/_ufuncs.pyx":7774
 * ufunc_hyp0f1_ptr[2*3] = <void*>_func__hyp0f1_cmplx
 * ufunc_hyp0f1_ptr[2*3+1] = <void*>(<char*>"hyp0f1")
 * ufunc_hyp0f1_data[0] = &ufunc_hyp0f1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_data[1] = &ufunc_hyp0f1_ptr[2*1]
 * ufunc_hyp0f1_data[2] = &ufunc_hyp0f1_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7775
 * ufunc_hyp0f1_ptr[2*3+1] = <void*>(<char*>"hyp0f1")
 * ufunc_hyp0f1_data[0] = &ufunc_hyp0f1_ptr[2*0]
 * ufunc_hyp0f1_data[1] = &ufunc_hyp0f1_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_data[2] = &ufunc_hyp0f1_ptr[2*2]
 * ufunc_hyp0f1_data[3] = &ufunc_hyp0f1_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7776
 * ufunc_hyp0f1_data[0] = &ufunc_hyp0f1_ptr[2*0]
 * ufunc_hyp0f1_data[1] = &ufunc_hyp0f1_ptr[2*1]
 * ufunc_hyp0f1_data[2] = &ufunc_hyp0f1_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_hyp0f1_data[3] = &ufunc_hyp0f1_ptr[2*3]
 * hyp0f1 = np.PyUFunc_FromFuncAndData(ufunc_hyp0f1_loops, ufunc_hyp0f1_data, ufunc_hyp0f1_types, 4, 2, 1, 0, "hyp0f1", ufunc_hyp0f1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":7777
 * ufunc_hyp0f1_data[1] = &ufunc_hyp0f1_ptr[2*1]
 * ufunc_hyp0f1_data[2] = &ufunc_hyp0f1_ptr[2*2]
 * ufunc_hyp0f1_data[3] = &ufunc_hyp0f1_ptr[2*3]             # <<<<<<<<<<<<<<
 * hyp0f1 = np.PyUFunc_FromFuncAndData(ufunc_hyp0f1_loops, ufunc_hyp0f1_data, ufunc_hyp0f1_types, 4, 2, 1, 0, "hyp0f1", ufunc_hyp0f1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":7778
 * ufunc_hyp0f1_data[2] = &ufunc_hyp0f1_ptr[2*2]
 * ufunc_hyp0f1_data[3] = &ufunc_hyp0f1_ptr[2*3]
 * hyp0f1 = np.PyUFunc_FromFuncAndData(ufunc_hyp0f1_loops, ufunc_hyp0f1_data, ufunc_hyp0f1_types, 4, 2, 1, 0, "hyp0f1", ufunc_hyp0f1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hyp1f1_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_types, 4, 2, 1, 0, ((char *)"hyp0f1"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp0f1_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7778, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hyp0f1, __pyx_t_2) < 0) __PYX_ERR(1, 7778, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":7784
 * cdef void *ufunc_hyp1f1_data[4]
 * cdef char ufunc_hyp1f1_types[16]
 * cdef char *ufunc_hyp1f1_doc = (             # <<<<<<<<<<<<<<
 *     "hyp1f1(a, b, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_doc = ((char *)"hyp1f1(a, b, x)\n\nConfluent hypergeometric function 1F1(a, b; x)");

  /* "scipy/special/_ufuncs.pyx":7788
 *     "\n"
 *     "Confluent hypergeometric function 1F1(a, b; x)")
 * ufunc_hyp1f1_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_loops[1] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 * ufunc_hyp1f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":7789
 *     "Confluent hypergeometric function 1F1(a, b; x)")
 * ufunc_hyp1f1_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_hyp1f1_loops[1] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_hyp1f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_ddD__As_ffF_F);

  /* "scipy/special/_ufuncs.pyx":7790
 * ufunc_hyp1f1_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_hyp1f1_loops[1] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 * ufunc_hyp1f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 * ufunc_hyp1f1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":7791
 * ufunc_hyp1f1_loops[1] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ffF_F
 * ufunc_hyp1f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_hyp1f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[0] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_ddD__As_ddD_D);

  /* "scipy/special/_ufuncs.pyx":7792
 * ufunc_hyp1f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_hyp1f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 * ufunc_hyp1f1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[1] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7793
 * ufunc_hyp1f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_ddD__As_ddD_D
 * ufunc_hyp1f1_types[0] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[2] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7794
 * ufunc_hyp1f1_types[0] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[1] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[3] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7795
 * ufunc_hyp1f1_types[1] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[2] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[4] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7796
 * ufunc_hyp1f1_types[2] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[3] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[5] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[6] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7797
 * ufunc_hyp1f1_types[3] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[4] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[6] = <char>NPY_CFLOAT
 * ufunc_hyp1f1_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7798
 * ufunc_hyp1f1_types[4] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[5] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[6] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[7] = <char>NPY_CFLOAT
 * ufunc_hyp1f1_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[6]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":7799
 * ufunc_hyp1f1_types[5] = <char>NPY_FLOAT
 * ufunc_hyp1f1_types[6] = <char>NPY_CFLOAT
 * ufunc_hyp1f1_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":7800
 * ufunc_hyp1f1_types[6] = <char>NPY_CFLOAT
 * ufunc_hyp1f1_types[7] = <char>NPY_CFLOAT
 * ufunc_hyp1f1_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7801
 * ufunc_hyp1f1_types[7] = <char>NPY_CFLOAT
 * ufunc_hyp1f1_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7802
 * ufunc_hyp1f1_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7803
 * ufunc_hyp1f1_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[12] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7804
 * ufunc_hyp1f1_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[13] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7805
 * ufunc_hyp1f1_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[12] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[14] = <char>NPY_CDOUBLE
 * ufunc_hyp1f1_types[15] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7806
 * ufunc_hyp1f1_types[12] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[13] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_types[15] = <char>NPY_CDOUBLE
 * ufunc_hyp1f1_ptr[2*0] = <void*>_func_hyp1f1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":7807
 * ufunc_hyp1f1_types[13] = <char>NPY_DOUBLE
 * ufunc_hyp1f1_types[14] = <char>NPY_CDOUBLE
 * ufunc_hyp1f1_types[15] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_ptr[2*0] = <void*>_func_hyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*0+1] = <void*>(<char*>"hyp1f1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types[15]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":7808
 * ufunc_hyp1f1_types[14] = <char>NPY_CDOUBLE
 * ufunc_hyp1f1_types[15] = <char>NPY_CDOUBLE
 * ufunc_hyp1f1_ptr[2*0] = <void*>_func_hyp1f1_wrap             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_ptr[2*0+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_ptr[2*1] = <void*>_func_chyp1f1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[0]) = ((void *)hyp1f1_wrap);

  /* "scipy/special/_ufuncs.pyx":7809
 * ufunc_hyp1f1_types[15] = <char>NPY_CDOUBLE
 * ufunc_hyp1f1_ptr[2*0] = <void*>_func_hyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*0+1] = <void*>(<char*>"hyp1f1")             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_ptr[2*1] = <void*>_func_chyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*1+1] = <void*>(<char*>"hyp1f1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[1]) = ((void *)((char *)((char *)"hyp1f1")));

  /* "scipy/special/_ufuncs.pyx":7810
 * ufunc_hyp1f1_ptr[2*0] = <void*>_func_hyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*0+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_ptr[2*1] = <void*>_func_chyp1f1_wrap             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_ptr[2*1+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_ptr[2*2] = <void*>_func_hyp1f1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[2]) = ((void *)chyp1f1_wrap);

  /* "scipy/special/_ufuncs.pyx":7811
 * ufunc_hyp1f1_ptr[2*0+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_ptr[2*1] = <void*>_func_chyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*1+1] = <void*>(<char*>"hyp1f1")             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_ptr[2*2] = <void*>_func_hyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*2+1] = <void*>(<char*>"hyp1f1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[3]) = ((void *)((char *)((char *)"hyp1f1")));

  /* "scipy/special/_ufuncs.pyx":7812
 * ufunc_hyp1f1_ptr[2*1] = <void*>_func_chyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*1+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_ptr[2*2] = <void*>_func_hyp1f1_wrap             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_ptr[2*2+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_ptr[2*3] = <void*>_func_chyp1f1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[4]) = ((void *)hyp1f1_wrap);

  /* "scipy/special/_ufuncs.pyx":7813
 * ufunc_hyp1f1_ptr[2*1+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_ptr[2*2] = <void*>_func_hyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*2+1] = <void*>(<char*>"hyp1f1")             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_ptr[2*3] = <void*>_func_chyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*3+1] = <void*>(<char*>"hyp1f1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[5]) = ((void *)((char *)((char *)"hyp1f1")));

  /* "scipy/special/_ufuncs.pyx":7814
 * ufunc_hyp1f1_ptr[2*2] = <void*>_func_hyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*2+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_ptr[2*3] = <void*>_func_chyp1f1_wrap             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_ptr[2*3+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_data[0] = &ufunc_hyp1f1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[6]) = ((void *)chyp1f1_wrap);

  /* "scipy/special/_ufuncs.pyx":7815
 * ufunc_hyp1f1_ptr[2*2+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_ptr[2*3] = <void*>_func_chyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*3+1] = <void*>(<char*>"hyp1f1")             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_data[0] = &ufunc_hyp1f1_ptr[2*0]
 * ufunc_hyp1f1_data[1] = &ufunc_hyp1f1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[7]) = ((void *)((char *)((char *)"hyp1f1")));

  /* "scipy/special/_ufuncs.pyx":7816
 * ufunc_hyp1f1_ptr[2*3] = <void*>_func_chyp1f1_wrap
 * ufunc_hyp1f1_ptr[2*3+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_data[0] = &ufunc_hyp1f1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_data[1] = &ufunc_hyp1f1_ptr[2*1]
 * ufunc_hyp1f1_data[2] = &ufunc_hyp1f1_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7817
 * ufunc_hyp1f1_ptr[2*3+1] = <void*>(<char*>"hyp1f1")
 * ufunc_hyp1f1_data[0] = &ufunc_hyp1f1_ptr[2*0]
 * ufunc_hyp1f1_data[1] = &ufunc_hyp1f1_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_data[2] = &ufunc_hyp1f1_ptr[2*2]
 * ufunc_hyp1f1_data[3] = &ufunc_hyp1f1_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7818
 * ufunc_hyp1f1_data[0] = &ufunc_hyp1f1_ptr[2*0]
 * ufunc_hyp1f1_data[1] = &ufunc_hyp1f1_ptr[2*1]
 * ufunc_hyp1f1_data[2] = &ufunc_hyp1f1_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_hyp1f1_data[3] = &ufunc_hyp1f1_ptr[2*3]
 * hyp1f1 = np.PyUFunc_FromFuncAndData(ufunc_hyp1f1_loops, ufunc_hyp1f1_data, ufunc_hyp1f1_types, 4, 3, 1, 0, "hyp1f1", ufunc_hyp1f1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":7819
 * ufunc_hyp1f1_data[1] = &ufunc_hyp1f1_ptr[2*1]
 * ufunc_hyp1f1_data[2] = &ufunc_hyp1f1_ptr[2*2]
 * ufunc_hyp1f1_data[3] = &ufunc_hyp1f1_ptr[2*3]             # <<<<<<<<<<<<<<
 * hyp1f1 = np.PyUFunc_FromFuncAndData(ufunc_hyp1f1_loops, ufunc_hyp1f1_data, ufunc_hyp1f1_types, 4, 3, 1, 0, "hyp1f1", ufunc_hyp1f1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":7820
 * ufunc_hyp1f1_data[2] = &ufunc_hyp1f1_ptr[2*2]
 * ufunc_hyp1f1_data[3] = &ufunc_hyp1f1_ptr[2*3]
 * hyp1f1 = np.PyUFunc_FromFuncAndData(ufunc_hyp1f1_loops, ufunc_hyp1f1_data, ufunc_hyp1f1_types, 4, 3, 1, 0, "hyp1f1", ufunc_hyp1f1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hyp1f2_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_types, 4, 3, 1, 0, ((char *)"hyp1f1"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f1_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7820, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hyp1f1, __pyx_t_2) < 0) __PYX_ERR(1, 7820, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":7826
 * cdef void *ufunc_hyp1f2_data[2]
 * cdef char ufunc_hyp1f2_types[12]
 * cdef char *ufunc_hyp1f2_doc = (             # <<<<<<<<<<<<<<
 *     "hyp1f2(a, b, c, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_doc = ((char *)"hyp1f2(a, b, c, x)\n\nHypergeometric function 1F2 and error estimate\n\nReturns\n-------\ny\n    Value of the function\nerr\n    Error estimate");

  /* "scipy/special/_ufuncs.pyx":7837
 *     "err\n"
 *     "    Error estimate")
 * ufunc_hyp1f2_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_hyp1f2_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff);

  /* "scipy/special/_ufuncs.pyx":7838
 *     "    Error estimate")
 * ufunc_hyp1f2_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_hyp1f2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[0] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd);

  /* "scipy/special/_ufuncs.pyx":7839
 * ufunc_hyp1f2_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_hyp1f2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_hyp1f2_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[1] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7840
 * ufunc_hyp1f2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_hyp1f2_types[0] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[2] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7841
 * ufunc_hyp1f2_types[0] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[1] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[3] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7842
 * ufunc_hyp1f2_types[1] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[2] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[4] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7843
 * ufunc_hyp1f2_types[2] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[3] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[5] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7844
 * ufunc_hyp1f2_types[3] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[4] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7845
 * ufunc_hyp1f2_types[4] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[5] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7846
 * ufunc_hyp1f2_types[5] = <char>NPY_FLOAT
 * ufunc_hyp1f2_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7847
 * ufunc_hyp1f2_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7848
 * ufunc_hyp1f2_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7849
 * ufunc_hyp1f2_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_ptr[2*0] = <void*>_func_onef2
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7850
 * ufunc_hyp1f2_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_ptr[2*0] = <void*>_func_onef2
 * ufunc_hyp1f2_ptr[2*0+1] = <void*>(<char*>"hyp1f2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7851
 * ufunc_hyp1f2_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_ptr[2*0] = <void*>_func_onef2             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_ptr[2*0+1] = <void*>(<char*>"hyp1f2")
 * ufunc_hyp1f2_ptr[2*1] = <void*>_func_onef2
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_ptr[0]) = ((void *)onef2);

  /* "scipy/special/_ufuncs.pyx":7852
 * ufunc_hyp1f2_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp1f2_ptr[2*0] = <void*>_func_onef2
 * ufunc_hyp1f2_ptr[2*0+1] = <void*>(<char*>"hyp1f2")             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_ptr[2*1] = <void*>_func_onef2
 * ufunc_hyp1f2_ptr[2*1+1] = <void*>(<char*>"hyp1f2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_ptr[1]) = ((void *)((char *)((char *)"hyp1f2")));

  /* "scipy/special/_ufuncs.pyx":7853
 * ufunc_hyp1f2_ptr[2*0] = <void*>_func_onef2
 * ufunc_hyp1f2_ptr[2*0+1] = <void*>(<char*>"hyp1f2")
 * ufunc_hyp1f2_ptr[2*1] = <void*>_func_onef2             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_ptr[2*1+1] = <void*>(<char*>"hyp1f2")
 * ufunc_hyp1f2_data[0] = &ufunc_hyp1f2_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_ptr[2]) = ((void *)onef2);

  /* "scipy/special/_ufuncs.pyx":7854
 * ufunc_hyp1f2_ptr[2*0+1] = <void*>(<char*>"hyp1f2")
 * ufunc_hyp1f2_ptr[2*1] = <void*>_func_onef2
 * ufunc_hyp1f2_ptr[2*1+1] = <void*>(<char*>"hyp1f2")             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_data[0] = &ufunc_hyp1f2_ptr[2*0]
 * ufunc_hyp1f2_data[1] = &ufunc_hyp1f2_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_ptr[3]) = ((void *)((char *)((char *)"hyp1f2")));

  /* "scipy/special/_ufuncs.pyx":7855
 * ufunc_hyp1f2_ptr[2*1] = <void*>_func_onef2
 * ufunc_hyp1f2_ptr[2*1+1] = <void*>(<char*>"hyp1f2")
 * ufunc_hyp1f2_data[0] = &ufunc_hyp1f2_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hyp1f2_data[1] = &ufunc_hyp1f2_ptr[2*1]
 * hyp1f2 = np.PyUFunc_FromFuncAndData(ufunc_hyp1f2_loops, ufunc_hyp1f2_data, ufunc_hyp1f2_types, 2, 4, 2, 0, "hyp1f2", ufunc_hyp1f2_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7856
 * ufunc_hyp1f2_ptr[2*1+1] = <void*>(<char*>"hyp1f2")
 * ufunc_hyp1f2_data[0] = &ufunc_hyp1f2_ptr[2*0]
 * ufunc_hyp1f2_data[1] = &ufunc_hyp1f2_ptr[2*1]             # <<<<<<<<<<<<<<
 * hyp1f2 = np.PyUFunc_FromFuncAndData(ufunc_hyp1f2_loops, ufunc_hyp1f2_data, ufunc_hyp1f2_types, 2, 4, 2, 0, "hyp1f2", ufunc_hyp1f2_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7857
 * ufunc_hyp1f2_data[0] = &ufunc_hyp1f2_ptr[2*0]
 * ufunc_hyp1f2_data[1] = &ufunc_hyp1f2_ptr[2*1]
 * hyp1f2 = np.PyUFunc_FromFuncAndData(ufunc_hyp1f2_loops, ufunc_hyp1f2_data, ufunc_hyp1f2_types, 2, 4, 2, 0, "hyp1f2", ufunc_hyp1f2_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hyp2f0_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_types, 2, 4, 2, 0, ((char *)"hyp1f2"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp1f2_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7857, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hyp1f2, __pyx_t_2) < 0) __PYX_ERR(1, 7857, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":7863
 * cdef void *ufunc_hyp2f0_data[3]
 * cdef char ufunc_hyp2f0_types[18]
 * cdef char *ufunc_hyp2f0_doc = (             # <<<<<<<<<<<<<<
 *     "hyp2f0(a, b, x, type)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_doc = ((char *)"hyp2f0(a, b, x, type)\n\nHypergeometric function 2F0 in y and an error estimate\n\nThe parameter `type` determines a convergence factor and can be\neither 1 or 2.\n\nReturns\n-------\ny\n    Value of the function\nerr\n    Error estimate");

  /* "scipy/special/_ufuncs.pyx":7877
 *     "err\n"
 *     "    Error estimate")
 * ufunc_hyp2f0_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddi_d_As_dddl_dd
 * ufunc_hyp2f0_loops[2] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff);

  /* "scipy/special/_ufuncs.pyx":7878
 *     "    Error estimate")
 * ufunc_hyp2f0_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_hyp2f0_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddi_d_As_dddl_dd             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_loops[2] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_hyp2f0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddi_d_As_dddl_dd);

  /* "scipy/special/_ufuncs.pyx":7879
 * ufunc_hyp2f0_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_hyp2f0_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddi_d_As_dddl_dd
 * ufunc_hyp2f0_loops[2] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[0] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd);

  /* "scipy/special/_ufuncs.pyx":7880
 * ufunc_hyp2f0_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddi_d_As_dddl_dd
 * ufunc_hyp2f0_loops[2] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_hyp2f0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[1] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7881
 * ufunc_hyp2f0_loops[2] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_hyp2f0_types[0] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[2] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7882
 * ufunc_hyp2f0_types[0] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[1] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[3] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7883
 * ufunc_hyp2f0_types[1] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[2] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[4] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7884
 * ufunc_hyp2f0_types[2] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[3] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[5] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7885
 * ufunc_hyp2f0_types[3] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[4] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7886
 * ufunc_hyp2f0_types[4] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[5] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7887
 * ufunc_hyp2f0_types[5] = <char>NPY_FLOAT
 * ufunc_hyp2f0_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[9] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7888
 * ufunc_hyp2f0_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[9] = <char>NPY_LONG
 * ufunc_hyp2f0_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7889
 * ufunc_hyp2f0_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[9] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[9]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":7890
 * ufunc_hyp2f0_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[9] = <char>NPY_LONG
 * ufunc_hyp2f0_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7891
 * ufunc_hyp2f0_types[9] = <char>NPY_LONG
 * ufunc_hyp2f0_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[12] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7892
 * ufunc_hyp2f0_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[13] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[14] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7893
 * ufunc_hyp2f0_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[12] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[14] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[15] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7894
 * ufunc_hyp2f0_types[12] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[13] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[14] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[15] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[16] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[14]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7895
 * ufunc_hyp2f0_types[13] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[14] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[15] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[16] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[17] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[15]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7896
 * ufunc_hyp2f0_types[14] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[15] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[16] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_types[17] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_ptr[2*0] = <void*>_func_hyp2f0_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[16]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7897
 * ufunc_hyp2f0_types[15] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[16] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[17] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_ptr[2*0] = <void*>_func_hyp2f0_unsafe
 * ufunc_hyp2f0_ptr[2*0+1] = <void*>(<char*>"hyp2f0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types[17]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7898
 * ufunc_hyp2f0_types[16] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_types[17] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_ptr[2*0] = <void*>_func_hyp2f0_unsafe             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_ptr[2*0+1] = <void*>(<char*>"hyp2f0")
 * ufunc_hyp2f0_ptr[2*1] = <void*>_func_hyp2f0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_ptr[0]) = ((void *)__pyx_f_5scipy_7special_7_legacy_hyp2f0_unsafe);

  /* "scipy/special/_ufuncs.pyx":7899
 * ufunc_hyp2f0_types[17] = <char>NPY_DOUBLE
 * ufunc_hyp2f0_ptr[2*0] = <void*>_func_hyp2f0_unsafe
 * ufunc_hyp2f0_ptr[2*0+1] = <void*>(<char*>"hyp2f0")             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_ptr[2*1] = <void*>_func_hyp2f0
 * ufunc_hyp2f0_ptr[2*1+1] = <void*>(<char*>"hyp2f0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_ptr[1]) = ((void *)((char *)((char *)"hyp2f0")));

  /* "scipy/special/_ufuncs.pyx":7900
 * ufunc_hyp2f0_ptr[2*0] = <void*>_func_hyp2f0_unsafe
 * ufunc_hyp2f0_ptr[2*0+1] = <void*>(<char*>"hyp2f0")
 * ufunc_hyp2f0_ptr[2*1] = <void*>_func_hyp2f0             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_ptr[2*1+1] = <void*>(<char*>"hyp2f0")
 * ufunc_hyp2f0_ptr[2*2] = <void*>_func_hyp2f0_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_ptr[2]) = ((void *)hyp2f0);

  /* "scipy/special/_ufuncs.pyx":7901
 * ufunc_hyp2f0_ptr[2*0+1] = <void*>(<char*>"hyp2f0")
 * ufunc_hyp2f0_ptr[2*1] = <void*>_func_hyp2f0
 * ufunc_hyp2f0_ptr[2*1+1] = <void*>(<char*>"hyp2f0")             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_ptr[2*2] = <void*>_func_hyp2f0_unsafe
 * ufunc_hyp2f0_ptr[2*2+1] = <void*>(<char*>"hyp2f0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_ptr[3]) = ((void *)((char *)((char *)"hyp2f0")));

  /* "scipy/special/_ufuncs.pyx":7902
 * ufunc_hyp2f0_ptr[2*1] = <void*>_func_hyp2f0
 * ufunc_hyp2f0_ptr[2*1+1] = <void*>(<char*>"hyp2f0")
 * ufunc_hyp2f0_ptr[2*2] = <void*>_func_hyp2f0_unsafe             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_ptr[2*2+1] = <void*>(<char*>"hyp2f0")
 * ufunc_hyp2f0_data[0] = &ufunc_hyp2f0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_hyp2f0_unsafe);

  /* "scipy/special/_ufuncs.pyx":7903
 * ufunc_hyp2f0_ptr[2*1+1] = <void*>(<char*>"hyp2f0")
 * ufunc_hyp2f0_ptr[2*2] = <void*>_func_hyp2f0_unsafe
 * ufunc_hyp2f0_ptr[2*2+1] = <void*>(<char*>"hyp2f0")             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_data[0] = &ufunc_hyp2f0_ptr[2*0]
 * ufunc_hyp2f0_data[1] = &ufunc_hyp2f0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_ptr[5]) = ((void *)((char *)((char *)"hyp2f0")));

  /* "scipy/special/_ufuncs.pyx":7904
 * ufunc_hyp2f0_ptr[2*2] = <void*>_func_hyp2f0_unsafe
 * ufunc_hyp2f0_ptr[2*2+1] = <void*>(<char*>"hyp2f0")
 * ufunc_hyp2f0_data[0] = &ufunc_hyp2f0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_data[1] = &ufunc_hyp2f0_ptr[2*1]
 * ufunc_hyp2f0_data[2] = &ufunc_hyp2f0_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7905
 * ufunc_hyp2f0_ptr[2*2+1] = <void*>(<char*>"hyp2f0")
 * ufunc_hyp2f0_data[0] = &ufunc_hyp2f0_ptr[2*0]
 * ufunc_hyp2f0_data[1] = &ufunc_hyp2f0_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_hyp2f0_data[2] = &ufunc_hyp2f0_ptr[2*2]
 * hyp2f0 = np.PyUFunc_FromFuncAndData(ufunc_hyp2f0_loops, ufunc_hyp2f0_data, ufunc_hyp2f0_types, 3, 4, 2, 0, "hyp2f0", ufunc_hyp2f0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7906
 * ufunc_hyp2f0_data[0] = &ufunc_hyp2f0_ptr[2*0]
 * ufunc_hyp2f0_data[1] = &ufunc_hyp2f0_ptr[2*1]
 * ufunc_hyp2f0_data[2] = &ufunc_hyp2f0_ptr[2*2]             # <<<<<<<<<<<<<<
 * hyp2f0 = np.PyUFunc_FromFuncAndData(ufunc_hyp2f0_loops, ufunc_hyp2f0_data, ufunc_hyp2f0_types, 3, 4, 2, 0, "hyp2f0", ufunc_hyp2f0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":7907
 * ufunc_hyp2f0_data[1] = &ufunc_hyp2f0_ptr[2*1]
 * ufunc_hyp2f0_data[2] = &ufunc_hyp2f0_ptr[2*2]
 * hyp2f0 = np.PyUFunc_FromFuncAndData(ufunc_hyp2f0_loops, ufunc_hyp2f0_data, ufunc_hyp2f0_types, 3, 4, 2, 0, "hyp2f0", ufunc_hyp2f0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hyp2f1_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_types, 3, 4, 2, 0, ((char *)"hyp2f0"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f0_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7907, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hyp2f0, __pyx_t_2) < 0) __PYX_ERR(1, 7907, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":7913
 * cdef void *ufunc_hyp2f1_data[4]
 * cdef char ufunc_hyp2f1_types[20]
 * cdef char *ufunc_hyp2f1_doc = (             # <<<<<<<<<<<<<<
 *     "hyp2f1(a, b, c, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_doc = ((char *)"hyp2f1(a, b, c, z)\n\nGauss hypergeometric function 2F1(a, b; c; z)\n\nParameters\n----------\na, b, c : array_like\n    Arguments, should be real-valued.\nz : array_like\n    Argument, real or complex.\n\nReturns\n-------\nhyp2f1 : scalar or ndarray\n    The values of the gaussian hypergeometric function.\n\nSee also\n--------\nhyp0f1 : confluent hypergeometric limit function.\nhyp1f1 : Kummer's (confluent hypergeometric) function.\n\nNotes\n-----\nThis function is defined for :math:`|z| < 1` as\n\n.. math::\n\n   \\mathrm{hyp2f1}(a, b, c, z) = \\sum_{n=0}^\\infty\n   \\frac{(a)_n (b)_n}{(c)_n}\\frac{z^n}{n!},\n\nand defined on the rest of the complex z-plane by analytic continuation.\nHere :math:`(\\cdot)_n` is the Pochhammer symbol; see `poch`. When\n:math:`n` is an integer the result is a polynomial of degree :math:`n`.\n\nThe implementation for complex values of ``z`` is described in [1]_.\n\nReferences\n----------\n.. [1] S. Zhang and J.M. Jin, \"Computation of Special Functions\", Wiley 1996\n.. [2] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/\n.. [3] NIST Digital Library of Mathematical Functions\n       https://dlmf.nist.gov/");

  /* "scipy/special/_ufuncs.pyx":7957
 *     ".. [3] NIST Digital Library of Mathematical Functions\n"
 *     "       https://dlmf.nist.gov/")
 * ufunc_hyp2f1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_loops[1] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 * ufunc_hyp2f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_ffff_f);

  /* "scipy/special/_ufuncs.pyx":7958
 *     "       https://dlmf.nist.gov/")
 * ufunc_hyp2f1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_hyp2f1_loops[1] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_hyp2f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dddD__As_fffF_F);

  /* "scipy/special/_ufuncs.pyx":7959
 * ufunc_hyp2f1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_hyp2f1_loops[1] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 * ufunc_hyp2f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 * ufunc_hyp2f1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_dddd_d);

  /* "scipy/special/_ufuncs.pyx":7960
 * ufunc_hyp2f1_loops[1] = <np.PyUFuncGenericFunction>loop_D_dddD__As_fffF_F
 * ufunc_hyp2f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_hyp2f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[0] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dddD__As_dddD_D);

  /* "scipy/special/_ufuncs.pyx":7961
 * ufunc_hyp2f1_loops[2] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_hyp2f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 * ufunc_hyp2f1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[1] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7962
 * ufunc_hyp2f1_loops[3] = <np.PyUFuncGenericFunction>loop_D_dddD__As_dddD_D
 * ufunc_hyp2f1_types[0] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[2] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7963
 * ufunc_hyp2f1_types[0] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[1] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[3] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7964
 * ufunc_hyp2f1_types[1] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[2] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[4] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7965
 * ufunc_hyp2f1_types[2] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[3] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[5] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7966
 * ufunc_hyp2f1_types[3] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[4] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[6] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7967
 * ufunc_hyp2f1_types[4] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[5] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[7] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7968
 * ufunc_hyp2f1_types[5] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[6] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[8] = <char>NPY_CFLOAT
 * ufunc_hyp2f1_types[9] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":7969
 * ufunc_hyp2f1_types[6] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[7] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[9] = <char>NPY_CFLOAT
 * ufunc_hyp2f1_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":7970
 * ufunc_hyp2f1_types[7] = <char>NPY_FLOAT
 * ufunc_hyp2f1_types[8] = <char>NPY_CFLOAT
 * ufunc_hyp2f1_types[9] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[9]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":7971
 * ufunc_hyp2f1_types[8] = <char>NPY_CFLOAT
 * ufunc_hyp2f1_types[9] = <char>NPY_CFLOAT
 * ufunc_hyp2f1_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7972
 * ufunc_hyp2f1_types[9] = <char>NPY_CFLOAT
 * ufunc_hyp2f1_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[12] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7973
 * ufunc_hyp2f1_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[13] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[14] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7974
 * ufunc_hyp2f1_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[12] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[14] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[15] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7975
 * ufunc_hyp2f1_types[12] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[13] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[14] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[15] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[16] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[14]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7976
 * ufunc_hyp2f1_types[13] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[14] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[15] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[16] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[17] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[15]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7977
 * ufunc_hyp2f1_types[14] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[15] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[16] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[17] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[18] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[16]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7978
 * ufunc_hyp2f1_types[15] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[16] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[17] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[18] = <char>NPY_CDOUBLE
 * ufunc_hyp2f1_types[19] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[17]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":7979
 * ufunc_hyp2f1_types[16] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[17] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[18] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_types[19] = <char>NPY_CDOUBLE
 * ufunc_hyp2f1_ptr[2*0] = <void*>_func_hyp2f1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[18]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":7980
 * ufunc_hyp2f1_types[17] = <char>NPY_DOUBLE
 * ufunc_hyp2f1_types[18] = <char>NPY_CDOUBLE
 * ufunc_hyp2f1_types[19] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_ptr[2*0] = <void*>_func_hyp2f1
 * ufunc_hyp2f1_ptr[2*0+1] = <void*>(<char*>"hyp2f1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types[19]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":7981
 * ufunc_hyp2f1_types[18] = <char>NPY_CDOUBLE
 * ufunc_hyp2f1_types[19] = <char>NPY_CDOUBLE
 * ufunc_hyp2f1_ptr[2*0] = <void*>_func_hyp2f1             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_ptr[2*0+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_ptr[2*1] = <void*>_func_chyp2f1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[0]) = ((void *)hyp2f1);

  /* "scipy/special/_ufuncs.pyx":7982
 * ufunc_hyp2f1_types[19] = <char>NPY_CDOUBLE
 * ufunc_hyp2f1_ptr[2*0] = <void*>_func_hyp2f1
 * ufunc_hyp2f1_ptr[2*0+1] = <void*>(<char*>"hyp2f1")             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_ptr[2*1] = <void*>_func_chyp2f1_wrap
 * ufunc_hyp2f1_ptr[2*1+1] = <void*>(<char*>"hyp2f1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[1]) = ((void *)((char *)((char *)"hyp2f1")));

  /* "scipy/special/_ufuncs.pyx":7983
 * ufunc_hyp2f1_ptr[2*0] = <void*>_func_hyp2f1
 * ufunc_hyp2f1_ptr[2*0+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_ptr[2*1] = <void*>_func_chyp2f1_wrap             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_ptr[2*1+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_ptr[2*2] = <void*>_func_hyp2f1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[2]) = ((void *)chyp2f1_wrap);

  /* "scipy/special/_ufuncs.pyx":7984
 * ufunc_hyp2f1_ptr[2*0+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_ptr[2*1] = <void*>_func_chyp2f1_wrap
 * ufunc_hyp2f1_ptr[2*1+1] = <void*>(<char*>"hyp2f1")             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_ptr[2*2] = <void*>_func_hyp2f1
 * ufunc_hyp2f1_ptr[2*2+1] = <void*>(<char*>"hyp2f1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[3]) = ((void *)((char *)((char *)"hyp2f1")));

  /* "scipy/special/_ufuncs.pyx":7985
 * ufunc_hyp2f1_ptr[2*1] = <void*>_func_chyp2f1_wrap
 * ufunc_hyp2f1_ptr[2*1+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_ptr[2*2] = <void*>_func_hyp2f1             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_ptr[2*2+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_ptr[2*3] = <void*>_func_chyp2f1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[4]) = ((void *)hyp2f1);

  /* "scipy/special/_ufuncs.pyx":7986
 * ufunc_hyp2f1_ptr[2*1+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_ptr[2*2] = <void*>_func_hyp2f1
 * ufunc_hyp2f1_ptr[2*2+1] = <void*>(<char*>"hyp2f1")             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_ptr[2*3] = <void*>_func_chyp2f1_wrap
 * ufunc_hyp2f1_ptr[2*3+1] = <void*>(<char*>"hyp2f1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[5]) = ((void *)((char *)((char *)"hyp2f1")));

  /* "scipy/special/_ufuncs.pyx":7987
 * ufunc_hyp2f1_ptr[2*2] = <void*>_func_hyp2f1
 * ufunc_hyp2f1_ptr[2*2+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_ptr[2*3] = <void*>_func_chyp2f1_wrap             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_ptr[2*3+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_data[0] = &ufunc_hyp2f1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[6]) = ((void *)chyp2f1_wrap);

  /* "scipy/special/_ufuncs.pyx":7988
 * ufunc_hyp2f1_ptr[2*2+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_ptr[2*3] = <void*>_func_chyp2f1_wrap
 * ufunc_hyp2f1_ptr[2*3+1] = <void*>(<char*>"hyp2f1")             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_data[0] = &ufunc_hyp2f1_ptr[2*0]
 * ufunc_hyp2f1_data[1] = &ufunc_hyp2f1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[7]) = ((void *)((char *)((char *)"hyp2f1")));

  /* "scipy/special/_ufuncs.pyx":7989
 * ufunc_hyp2f1_ptr[2*3] = <void*>_func_chyp2f1_wrap
 * ufunc_hyp2f1_ptr[2*3+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_data[0] = &ufunc_hyp2f1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_data[1] = &ufunc_hyp2f1_ptr[2*1]
 * ufunc_hyp2f1_data[2] = &ufunc_hyp2f1_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":7990
 * ufunc_hyp2f1_ptr[2*3+1] = <void*>(<char*>"hyp2f1")
 * ufunc_hyp2f1_data[0] = &ufunc_hyp2f1_ptr[2*0]
 * ufunc_hyp2f1_data[1] = &ufunc_hyp2f1_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_data[2] = &ufunc_hyp2f1_ptr[2*2]
 * ufunc_hyp2f1_data[3] = &ufunc_hyp2f1_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":7991
 * ufunc_hyp2f1_data[0] = &ufunc_hyp2f1_ptr[2*0]
 * ufunc_hyp2f1_data[1] = &ufunc_hyp2f1_ptr[2*1]
 * ufunc_hyp2f1_data[2] = &ufunc_hyp2f1_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_hyp2f1_data[3] = &ufunc_hyp2f1_ptr[2*3]
 * hyp2f1 = np.PyUFunc_FromFuncAndData(ufunc_hyp2f1_loops, ufunc_hyp2f1_data, ufunc_hyp2f1_types, 4, 4, 1, 0, "hyp2f1", ufunc_hyp2f1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":7992
 * ufunc_hyp2f1_data[1] = &ufunc_hyp2f1_ptr[2*1]
 * ufunc_hyp2f1_data[2] = &ufunc_hyp2f1_ptr[2*2]
 * ufunc_hyp2f1_data[3] = &ufunc_hyp2f1_ptr[2*3]             # <<<<<<<<<<<<<<
 * hyp2f1 = np.PyUFunc_FromFuncAndData(ufunc_hyp2f1_loops, ufunc_hyp2f1_data, ufunc_hyp2f1_types, 4, 4, 1, 0, "hyp2f1", ufunc_hyp2f1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":7993
 * ufunc_hyp2f1_data[2] = &ufunc_hyp2f1_ptr[2*2]
 * ufunc_hyp2f1_data[3] = &ufunc_hyp2f1_ptr[2*3]
 * hyp2f1 = np.PyUFunc_FromFuncAndData(ufunc_hyp2f1_loops, ufunc_hyp2f1_data, ufunc_hyp2f1_types, 4, 4, 1, 0, "hyp2f1", ufunc_hyp2f1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hyp3f0_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_types, 4, 4, 1, 0, ((char *)"hyp2f1"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp2f1_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7993, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hyp2f1, __pyx_t_2) < 0) __PYX_ERR(1, 7993, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":7999
 * cdef void *ufunc_hyp3f0_data[2]
 * cdef char ufunc_hyp3f0_types[12]
 * cdef char *ufunc_hyp3f0_doc = (             # <<<<<<<<<<<<<<
 *     "hyp3f0(a, b, c, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_doc = ((char *)"hyp3f0(a, b, c, x)\n\nHypergeometric function 3F0 in y and an error estimate\n\nReturns\n-------\ny\n    Value of the function\nerr\n    Error estimate");

  /* "scipy/special/_ufuncs.pyx":8010
 *     "err\n"
 *     "    Error estimate")
 * ufunc_hyp3f0_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_hyp3f0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff);

  /* "scipy/special/_ufuncs.pyx":8011
 *     "    Error estimate")
 * ufunc_hyp3f0_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_hyp3f0_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[0] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd);

  /* "scipy/special/_ufuncs.pyx":8012
 * ufunc_hyp3f0_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_hyp3f0_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_hyp3f0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[1] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8013
 * ufunc_hyp3f0_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_hyp3f0_types[0] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[2] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8014
 * ufunc_hyp3f0_types[0] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[1] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[3] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8015
 * ufunc_hyp3f0_types[1] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[2] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[4] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8016
 * ufunc_hyp3f0_types[2] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[3] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[5] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8017
 * ufunc_hyp3f0_types[3] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[4] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8018
 * ufunc_hyp3f0_types[4] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[5] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8019
 * ufunc_hyp3f0_types[5] = <char>NPY_FLOAT
 * ufunc_hyp3f0_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8020
 * ufunc_hyp3f0_types[6] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8021
 * ufunc_hyp3f0_types[7] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8022
 * ufunc_hyp3f0_types[8] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_ptr[2*0] = <void*>_func_threef0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8023
 * ufunc_hyp3f0_types[9] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_ptr[2*0] = <void*>_func_threef0
 * ufunc_hyp3f0_ptr[2*0+1] = <void*>(<char*>"hyp3f0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8024
 * ufunc_hyp3f0_types[10] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_ptr[2*0] = <void*>_func_threef0             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_ptr[2*0+1] = <void*>(<char*>"hyp3f0")
 * ufunc_hyp3f0_ptr[2*1] = <void*>_func_threef0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_ptr[0]) = ((void *)threef0);

  /* "scipy/special/_ufuncs.pyx":8025
 * ufunc_hyp3f0_types[11] = <char>NPY_DOUBLE
 * ufunc_hyp3f0_ptr[2*0] = <void*>_func_threef0
 * ufunc_hyp3f0_ptr[2*0+1] = <void*>(<char*>"hyp3f0")             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_ptr[2*1] = <void*>_func_threef0
 * ufunc_hyp3f0_ptr[2*1+1] = <void*>(<char*>"hyp3f0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_ptr[1]) = ((void *)((char *)((char *)"hyp3f0")));

  /* "scipy/special/_ufuncs.pyx":8026
 * ufunc_hyp3f0_ptr[2*0] = <void*>_func_threef0
 * ufunc_hyp3f0_ptr[2*0+1] = <void*>(<char*>"hyp3f0")
 * ufunc_hyp3f0_ptr[2*1] = <void*>_func_threef0             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_ptr[2*1+1] = <void*>(<char*>"hyp3f0")
 * ufunc_hyp3f0_data[0] = &ufunc_hyp3f0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_ptr[2]) = ((void *)threef0);

  /* "scipy/special/_ufuncs.pyx":8027
 * ufunc_hyp3f0_ptr[2*0+1] = <void*>(<char*>"hyp3f0")
 * ufunc_hyp3f0_ptr[2*1] = <void*>_func_threef0
 * ufunc_hyp3f0_ptr[2*1+1] = <void*>(<char*>"hyp3f0")             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_data[0] = &ufunc_hyp3f0_ptr[2*0]
 * ufunc_hyp3f0_data[1] = &ufunc_hyp3f0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_ptr[3]) = ((void *)((char *)((char *)"hyp3f0")));

  /* "scipy/special/_ufuncs.pyx":8028
 * ufunc_hyp3f0_ptr[2*1] = <void*>_func_threef0
 * ufunc_hyp3f0_ptr[2*1+1] = <void*>(<char*>"hyp3f0")
 * ufunc_hyp3f0_data[0] = &ufunc_hyp3f0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hyp3f0_data[1] = &ufunc_hyp3f0_ptr[2*1]
 * hyp3f0 = np.PyUFunc_FromFuncAndData(ufunc_hyp3f0_loops, ufunc_hyp3f0_data, ufunc_hyp3f0_types, 2, 4, 2, 0, "hyp3f0", ufunc_hyp3f0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8029
 * ufunc_hyp3f0_ptr[2*1+1] = <void*>(<char*>"hyp3f0")
 * ufunc_hyp3f0_data[0] = &ufunc_hyp3f0_ptr[2*0]
 * ufunc_hyp3f0_data[1] = &ufunc_hyp3f0_ptr[2*1]             # <<<<<<<<<<<<<<
 * hyp3f0 = np.PyUFunc_FromFuncAndData(ufunc_hyp3f0_loops, ufunc_hyp3f0_data, ufunc_hyp3f0_types, 2, 4, 2, 0, "hyp3f0", ufunc_hyp3f0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8030
 * ufunc_hyp3f0_data[0] = &ufunc_hyp3f0_ptr[2*0]
 * ufunc_hyp3f0_data[1] = &ufunc_hyp3f0_ptr[2*1]
 * hyp3f0 = np.PyUFunc_FromFuncAndData(ufunc_hyp3f0_loops, ufunc_hyp3f0_data, ufunc_hyp3f0_types, 2, 4, 2, 0, "hyp3f0", ufunc_hyp3f0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_hyperu_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_types, 2, 4, 2, 0, ((char *)"hyp3f0"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyp3f0_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8030, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hyp3f0, __pyx_t_2) < 0) __PYX_ERR(1, 8030, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":8036
 * cdef void *ufunc_hyperu_data[2]
 * cdef char ufunc_hyperu_types[8]
 * cdef char *ufunc_hyperu_doc = (             # <<<<<<<<<<<<<<
 *     "hyperu(a, b, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_doc = ((char *)"hyperu(a, b, x)\n\nConfluent hypergeometric function U(a, b, x) of the second kind");

  /* "scipy/special/_ufuncs.pyx":8040
 *     "\n"
 *     "Confluent hypergeometric function U(a, b, x) of the second kind")
 * ufunc_hyperu_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_hyperu_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_hyperu_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":8041
 *     "Confluent hypergeometric function U(a, b, x) of the second kind")
 * ufunc_hyperu_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_hyperu_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_hyperu_types[0] = <char>NPY_FLOAT
 * ufunc_hyperu_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":8042
 * ufunc_hyperu_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_hyperu_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_hyperu_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyperu_types[1] = <char>NPY_FLOAT
 * ufunc_hyperu_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8043
 * ufunc_hyperu_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_hyperu_types[0] = <char>NPY_FLOAT
 * ufunc_hyperu_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyperu_types[2] = <char>NPY_FLOAT
 * ufunc_hyperu_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8044
 * ufunc_hyperu_types[0] = <char>NPY_FLOAT
 * ufunc_hyperu_types[1] = <char>NPY_FLOAT
 * ufunc_hyperu_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyperu_types[3] = <char>NPY_FLOAT
 * ufunc_hyperu_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8045
 * ufunc_hyperu_types[1] = <char>NPY_FLOAT
 * ufunc_hyperu_types[2] = <char>NPY_FLOAT
 * ufunc_hyperu_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_hyperu_types[4] = <char>NPY_DOUBLE
 * ufunc_hyperu_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8046
 * ufunc_hyperu_types[2] = <char>NPY_FLOAT
 * ufunc_hyperu_types[3] = <char>NPY_FLOAT
 * ufunc_hyperu_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyperu_types[5] = <char>NPY_DOUBLE
 * ufunc_hyperu_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8047
 * ufunc_hyperu_types[3] = <char>NPY_FLOAT
 * ufunc_hyperu_types[4] = <char>NPY_DOUBLE
 * ufunc_hyperu_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyperu_types[6] = <char>NPY_DOUBLE
 * ufunc_hyperu_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8048
 * ufunc_hyperu_types[4] = <char>NPY_DOUBLE
 * ufunc_hyperu_types[5] = <char>NPY_DOUBLE
 * ufunc_hyperu_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyperu_types[7] = <char>NPY_DOUBLE
 * ufunc_hyperu_ptr[2*0] = <void*>_func_hypU_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8049
 * ufunc_hyperu_types[5] = <char>NPY_DOUBLE
 * ufunc_hyperu_types[6] = <char>NPY_DOUBLE
 * ufunc_hyperu_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_hyperu_ptr[2*0] = <void*>_func_hypU_wrap
 * ufunc_hyperu_ptr[2*0+1] = <void*>(<char*>"hyperu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8050
 * ufunc_hyperu_types[6] = <char>NPY_DOUBLE
 * ufunc_hyperu_types[7] = <char>NPY_DOUBLE
 * ufunc_hyperu_ptr[2*0] = <void*>_func_hypU_wrap             # <<<<<<<<<<<<<<
 * ufunc_hyperu_ptr[2*0+1] = <void*>(<char*>"hyperu")
 * ufunc_hyperu_ptr[2*1] = <void*>_func_hypU_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_ptr[0]) = ((void *)hypU_wrap);

  /* "scipy/special/_ufuncs.pyx":8051
 * ufunc_hyperu_types[7] = <char>NPY_DOUBLE
 * ufunc_hyperu_ptr[2*0] = <void*>_func_hypU_wrap
 * ufunc_hyperu_ptr[2*0+1] = <void*>(<char*>"hyperu")             # <<<<<<<<<<<<<<
 * ufunc_hyperu_ptr[2*1] = <void*>_func_hypU_wrap
 * ufunc_hyperu_ptr[2*1+1] = <void*>(<char*>"hyperu")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_ptr[1]) = ((void *)((char *)((char *)"hyperu")));

  /* "scipy/special/_ufuncs.pyx":8052
 * ufunc_hyperu_ptr[2*0] = <void*>_func_hypU_wrap
 * ufunc_hyperu_ptr[2*0+1] = <void*>(<char*>"hyperu")
 * ufunc_hyperu_ptr[2*1] = <void*>_func_hypU_wrap             # <<<<<<<<<<<<<<
 * ufunc_hyperu_ptr[2*1+1] = <void*>(<char*>"hyperu")
 * ufunc_hyperu_data[0] = &ufunc_hyperu_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_ptr[2]) = ((void *)hypU_wrap);

  /* "scipy/special/_ufuncs.pyx":8053
 * ufunc_hyperu_ptr[2*0+1] = <void*>(<char*>"hyperu")
 * ufunc_hyperu_ptr[2*1] = <void*>_func_hypU_wrap
 * ufunc_hyperu_ptr[2*1+1] = <void*>(<char*>"hyperu")             # <<<<<<<<<<<<<<
 * ufunc_hyperu_data[0] = &ufunc_hyperu_ptr[2*0]
 * ufunc_hyperu_data[1] = &ufunc_hyperu_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_ptr[3]) = ((void *)((char *)((char *)"hyperu")));

  /* "scipy/special/_ufuncs.pyx":8054
 * ufunc_hyperu_ptr[2*1] = <void*>_func_hypU_wrap
 * ufunc_hyperu_ptr[2*1+1] = <void*>(<char*>"hyperu")
 * ufunc_hyperu_data[0] = &ufunc_hyperu_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_hyperu_data[1] = &ufunc_hyperu_ptr[2*1]
 * hyperu = np.PyUFunc_FromFuncAndData(ufunc_hyperu_loops, ufunc_hyperu_data, ufunc_hyperu_types, 2, 3, 1, 0, "hyperu", ufunc_hyperu_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8055
 * ufunc_hyperu_ptr[2*1+1] = <void*>(<char*>"hyperu")
 * ufunc_hyperu_data[0] = &ufunc_hyperu_ptr[2*0]
 * ufunc_hyperu_data[1] = &ufunc_hyperu_ptr[2*1]             # <<<<<<<<<<<<<<
 * hyperu = np.PyUFunc_FromFuncAndData(ufunc_hyperu_loops, ufunc_hyperu_data, ufunc_hyperu_types, 2, 3, 1, 0, "hyperu", ufunc_hyperu_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8056
 * ufunc_hyperu_data[0] = &ufunc_hyperu_ptr[2*0]
 * ufunc_hyperu_data[1] = &ufunc_hyperu_ptr[2*1]
 * hyperu = np.PyUFunc_FromFuncAndData(ufunc_hyperu_loops, ufunc_hyperu_data, ufunc_hyperu_types, 2, 3, 1, 0, "hyperu", ufunc_hyperu_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_i0_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_types, 2, 3, 1, 0, ((char *)"hyperu"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_hyperu_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8056, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_hyperu, __pyx_t_2) < 0) __PYX_ERR(1, 8056, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":8062
 * cdef void *ufunc_i0_data[2]
 * cdef char ufunc_i0_types[4]
 * cdef char *ufunc_i0_doc = (             # <<<<<<<<<<<<<<
 *     "i0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_doc = ((char *)"i0(x)\n\nModified Bessel function of order 0.\n\nDefined as,\n\n.. math::\n    I_0(x) = \\sum_{k=0}^\\infty \\frac{(x^2/4)^k}{(k!)^2} = J_0(\\imath x),\n\nwhere :math:`J_0` is the Bessel function of the first kind of order 0.\n\nParameters\n----------\nx : array_like\n    Argument (float)\n\nReturns\n-------\nI : ndarray\n    Value of the modified Bessel function of order 0 at `x`.\n\nNotes\n-----\nThe range is partitioned into the two intervals [0, 8] and (8, infinity).\nChebyshev polynomial expansions are employed in each interval.\n\nThis function is a wrapper for the Cephes [1]_ routine `i0`.\n\nSee also\n--------\niv\ni0e\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":8100
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_i0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_i0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":8101
 *     "       http://www.netlib.org/cephes/")
 * ufunc_i0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_i0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_i0_types[0] = <char>NPY_FLOAT
 * ufunc_i0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":8102
 * ufunc_i0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_i0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_i0_types[1] = <char>NPY_FLOAT
 * ufunc_i0_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8103
 * ufunc_i0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i0_types[0] = <char>NPY_FLOAT
 * ufunc_i0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_i0_types[2] = <char>NPY_DOUBLE
 * ufunc_i0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8104
 * ufunc_i0_types[0] = <char>NPY_FLOAT
 * ufunc_i0_types[1] = <char>NPY_FLOAT
 * ufunc_i0_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_i0_types[3] = <char>NPY_DOUBLE
 * ufunc_i0_ptr[2*0] = <void*>_func_i0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8105
 * ufunc_i0_types[1] = <char>NPY_FLOAT
 * ufunc_i0_types[2] = <char>NPY_DOUBLE
 * ufunc_i0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_i0_ptr[2*0] = <void*>_func_i0
 * ufunc_i0_ptr[2*0+1] = <void*>(<char*>"i0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8106
 * ufunc_i0_types[2] = <char>NPY_DOUBLE
 * ufunc_i0_types[3] = <char>NPY_DOUBLE
 * ufunc_i0_ptr[2*0] = <void*>_func_i0             # <<<<<<<<<<<<<<
 * ufunc_i0_ptr[2*0+1] = <void*>(<char*>"i0")
 * ufunc_i0_ptr[2*1] = <void*>_func_i0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_ptr[0]) = ((void *)i0);

  /* "scipy/special/_ufuncs.pyx":8107
 * ufunc_i0_types[3] = <char>NPY_DOUBLE
 * ufunc_i0_ptr[2*0] = <void*>_func_i0
 * ufunc_i0_ptr[2*0+1] = <void*>(<char*>"i0")             # <<<<<<<<<<<<<<
 * ufunc_i0_ptr[2*1] = <void*>_func_i0
 * ufunc_i0_ptr[2*1+1] = <void*>(<char*>"i0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_ptr[1]) = ((void *)((char *)((char *)"i0")));

  /* "scipy/special/_ufuncs.pyx":8108
 * ufunc_i0_ptr[2*0] = <void*>_func_i0
 * ufunc_i0_ptr[2*0+1] = <void*>(<char*>"i0")
 * ufunc_i0_ptr[2*1] = <void*>_func_i0             # <<<<<<<<<<<<<<
 * ufunc_i0_ptr[2*1+1] = <void*>(<char*>"i0")
 * ufunc_i0_data[0] = &ufunc_i0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_ptr[2]) = ((void *)i0);

  /* "scipy/special/_ufuncs.pyx":8109
 * ufunc_i0_ptr[2*0+1] = <void*>(<char*>"i0")
 * ufunc_i0_ptr[2*1] = <void*>_func_i0
 * ufunc_i0_ptr[2*1+1] = <void*>(<char*>"i0")             # <<<<<<<<<<<<<<
 * ufunc_i0_data[0] = &ufunc_i0_ptr[2*0]
 * ufunc_i0_data[1] = &ufunc_i0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_ptr[3]) = ((void *)((char *)((char *)"i0")));

  /* "scipy/special/_ufuncs.pyx":8110
 * ufunc_i0_ptr[2*1] = <void*>_func_i0
 * ufunc_i0_ptr[2*1+1] = <void*>(<char*>"i0")
 * ufunc_i0_data[0] = &ufunc_i0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_i0_data[1] = &ufunc_i0_ptr[2*1]
 * i0 = np.PyUFunc_FromFuncAndData(ufunc_i0_loops, ufunc_i0_data, ufunc_i0_types, 2, 1, 1, 0, "i0", ufunc_i0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8111
 * ufunc_i0_ptr[2*1+1] = <void*>(<char*>"i0")
 * ufunc_i0_data[0] = &ufunc_i0_ptr[2*0]
 * ufunc_i0_data[1] = &ufunc_i0_ptr[2*1]             # <<<<<<<<<<<<<<
 * i0 = np.PyUFunc_FromFuncAndData(ufunc_i0_loops, ufunc_i0_data, ufunc_i0_types, 2, 1, 1, 0, "i0", ufunc_i0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8112
 * ufunc_i0_data[0] = &ufunc_i0_ptr[2*0]
 * ufunc_i0_data[1] = &ufunc_i0_ptr[2*1]
 * i0 = np.PyUFunc_FromFuncAndData(ufunc_i0_loops, ufunc_i0_data, ufunc_i0_types, 2, 1, 1, 0, "i0", ufunc_i0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_i0e_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_types, 2, 1, 1, 0, ((char *)"i0"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_i0, __pyx_t_2) < 0) __PYX_ERR(1, 8112, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":8118
 * cdef void *ufunc_i0e_data[2]
 * cdef char ufunc_i0e_types[4]
 * cdef char *ufunc_i0e_doc = (             # <<<<<<<<<<<<<<
 *     "i0e(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_doc = ((char *)"i0e(x)\n\nExponentially scaled modified Bessel function of order 0.\n\nDefined as::\n\n    i0e(x) = exp(-abs(x)) * i0(x).\n\nParameters\n----------\nx : array_like\n    Argument (float)\n\nReturns\n-------\nI : ndarray\n    Value of the exponentially scaled modified Bessel function of order 0\n    at `x`.\n\nNotes\n-----\nThe range is partitioned into the two intervals [0, 8] and (8, infinity).\nChebyshev polynomial expansions are employed in each interval.  The\npolynomial expansions used are the same as those in `i0`, but\nthey are not multiplied by the dominant exponential factor.\n\nThis function is a wrapper for the Cephes [1]_ routine `i0e`.\n\nSee also\n--------\niv\ni0\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":8156
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_i0e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_i0e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i0e_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":8157
 *     "       http://www.netlib.org/cephes/")
 * ufunc_i0e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_i0e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_i0e_types[0] = <char>NPY_FLOAT
 * ufunc_i0e_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":8158
 * ufunc_i0e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_i0e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i0e_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_i0e_types[1] = <char>NPY_FLOAT
 * ufunc_i0e_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8159
 * ufunc_i0e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i0e_types[0] = <char>NPY_FLOAT
 * ufunc_i0e_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_i0e_types[2] = <char>NPY_DOUBLE
 * ufunc_i0e_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8160
 * ufunc_i0e_types[0] = <char>NPY_FLOAT
 * ufunc_i0e_types[1] = <char>NPY_FLOAT
 * ufunc_i0e_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_i0e_types[3] = <char>NPY_DOUBLE
 * ufunc_i0e_ptr[2*0] = <void*>_func_i0e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8161
 * ufunc_i0e_types[1] = <char>NPY_FLOAT
 * ufunc_i0e_types[2] = <char>NPY_DOUBLE
 * ufunc_i0e_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_i0e_ptr[2*0] = <void*>_func_i0e
 * ufunc_i0e_ptr[2*0+1] = <void*>(<char*>"i0e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8162
 * ufunc_i0e_types[2] = <char>NPY_DOUBLE
 * ufunc_i0e_types[3] = <char>NPY_DOUBLE
 * ufunc_i0e_ptr[2*0] = <void*>_func_i0e             # <<<<<<<<<<<<<<
 * ufunc_i0e_ptr[2*0+1] = <void*>(<char*>"i0e")
 * ufunc_i0e_ptr[2*1] = <void*>_func_i0e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_ptr[0]) = ((void *)i0e);

  /* "scipy/special/_ufuncs.pyx":8163
 * ufunc_i0e_types[3] = <char>NPY_DOUBLE
 * ufunc_i0e_ptr[2*0] = <void*>_func_i0e
 * ufunc_i0e_ptr[2*0+1] = <void*>(<char*>"i0e")             # <<<<<<<<<<<<<<
 * ufunc_i0e_ptr[2*1] = <void*>_func_i0e
 * ufunc_i0e_ptr[2*1+1] = <void*>(<char*>"i0e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_ptr[1]) = ((void *)((char *)((char *)"i0e")));

  /* "scipy/special/_ufuncs.pyx":8164
 * ufunc_i0e_ptr[2*0] = <void*>_func_i0e
 * ufunc_i0e_ptr[2*0+1] = <void*>(<char*>"i0e")
 * ufunc_i0e_ptr[2*1] = <void*>_func_i0e             # <<<<<<<<<<<<<<
 * ufunc_i0e_ptr[2*1+1] = <void*>(<char*>"i0e")
 * ufunc_i0e_data[0] = &ufunc_i0e_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_ptr[2]) = ((void *)i0e);

  /* "scipy/special/_ufuncs.pyx":8165
 * ufunc_i0e_ptr[2*0+1] = <void*>(<char*>"i0e")
 * ufunc_i0e_ptr[2*1] = <void*>_func_i0e
 * ufunc_i0e_ptr[2*1+1] = <void*>(<char*>"i0e")             # <<<<<<<<<<<<<<
 * ufunc_i0e_data[0] = &ufunc_i0e_ptr[2*0]
 * ufunc_i0e_data[1] = &ufunc_i0e_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_ptr[3]) = ((void *)((char *)((char *)"i0e")));

  /* "scipy/special/_ufuncs.pyx":8166
 * ufunc_i0e_ptr[2*1] = <void*>_func_i0e
 * ufunc_i0e_ptr[2*1+1] = <void*>(<char*>"i0e")
 * ufunc_i0e_data[0] = &ufunc_i0e_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_i0e_data[1] = &ufunc_i0e_ptr[2*1]
 * i0e = np.PyUFunc_FromFuncAndData(ufunc_i0e_loops, ufunc_i0e_data, ufunc_i0e_types, 2, 1, 1, 0, "i0e", ufunc_i0e_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8167
 * ufunc_i0e_ptr[2*1+1] = <void*>(<char*>"i0e")
 * ufunc_i0e_data[0] = &ufunc_i0e_ptr[2*0]
 * ufunc_i0e_data[1] = &ufunc_i0e_ptr[2*1]             # <<<<<<<<<<<<<<
 * i0e = np.PyUFunc_FromFuncAndData(ufunc_i0e_loops, ufunc_i0e_data, ufunc_i0e_types, 2, 1, 1, 0, "i0e", ufunc_i0e_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8168
 * ufunc_i0e_data[0] = &ufunc_i0e_ptr[2*0]
 * ufunc_i0e_data[1] = &ufunc_i0e_ptr[2*1]
 * i0e = np.PyUFunc_FromFuncAndData(ufunc_i0e_loops, ufunc_i0e_data, ufunc_i0e_types, 2, 1, 1, 0, "i0e", ufunc_i0e_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_i1_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_types, 2, 1, 1, 0, ((char *)"i0e"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_i0e_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_i0e, __pyx_t_2) < 0) __PYX_ERR(1, 8168, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":8174
 * cdef void *ufunc_i1_data[2]
 * cdef char ufunc_i1_types[4]
 * cdef char *ufunc_i1_doc = (             # <<<<<<<<<<<<<<
 *     "i1(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_doc = ((char *)"i1(x)\n\nModified Bessel function of order 1.\n\nDefined as,\n\n.. math::\n    I_1(x) = \\frac{1}{2}x \\sum_{k=0}^\\infty \\frac{(x^2/4)^k}{k! (k + 1)!}\n           = -\\imath J_1(\\imath x),\n\nwhere :math:`J_1` is the Bessel function of the first kind of order 1.\n\nParameters\n----------\nx : array_like\n    Argument (float)\n\nReturns\n-------\nI : ndarray\n    Value of the modified Bessel function of order 1 at `x`.\n\nNotes\n-----\nThe range is partitioned into the two intervals [0, 8] and (8, infinity).\nChebyshev polynomial expansions are employed in each interval.\n\nThis function is a wrapper for the Cephes [1]_ routine `i1`.\n\nSee also\n--------\niv\ni1e\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":8213
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_i1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_i1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":8214
 *     "       http://www.netlib.org/cephes/")
 * ufunc_i1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_i1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_i1_types[0] = <char>NPY_FLOAT
 * ufunc_i1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":8215
 * ufunc_i1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_i1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_i1_types[1] = <char>NPY_FLOAT
 * ufunc_i1_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8216
 * ufunc_i1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i1_types[0] = <char>NPY_FLOAT
 * ufunc_i1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_i1_types[2] = <char>NPY_DOUBLE
 * ufunc_i1_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8217
 * ufunc_i1_types[0] = <char>NPY_FLOAT
 * ufunc_i1_types[1] = <char>NPY_FLOAT
 * ufunc_i1_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_i1_types[3] = <char>NPY_DOUBLE
 * ufunc_i1_ptr[2*0] = <void*>_func_i1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8218
 * ufunc_i1_types[1] = <char>NPY_FLOAT
 * ufunc_i1_types[2] = <char>NPY_DOUBLE
 * ufunc_i1_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_i1_ptr[2*0] = <void*>_func_i1
 * ufunc_i1_ptr[2*0+1] = <void*>(<char*>"i1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8219
 * ufunc_i1_types[2] = <char>NPY_DOUBLE
 * ufunc_i1_types[3] = <char>NPY_DOUBLE
 * ufunc_i1_ptr[2*0] = <void*>_func_i1             # <<<<<<<<<<<<<<
 * ufunc_i1_ptr[2*0+1] = <void*>(<char*>"i1")
 * ufunc_i1_ptr[2*1] = <void*>_func_i1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_ptr[0]) = ((void *)i1);

  /* "scipy/special/_ufuncs.pyx":8220
 * ufunc_i1_types[3] = <char>NPY_DOUBLE
 * ufunc_i1_ptr[2*0] = <void*>_func_i1
 * ufunc_i1_ptr[2*0+1] = <void*>(<char*>"i1")             # <<<<<<<<<<<<<<
 * ufunc_i1_ptr[2*1] = <void*>_func_i1
 * ufunc_i1_ptr[2*1+1] = <void*>(<char*>"i1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_ptr[1]) = ((void *)((char *)((char *)"i1")));

  /* "scipy/special/_ufuncs.pyx":8221
 * ufunc_i1_ptr[2*0] = <void*>_func_i1
 * ufunc_i1_ptr[2*0+1] = <void*>(<char*>"i1")
 * ufunc_i1_ptr[2*1] = <void*>_func_i1             # <<<<<<<<<<<<<<
 * ufunc_i1_ptr[2*1+1] = <void*>(<char*>"i1")
 * ufunc_i1_data[0] = &ufunc_i1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_ptr[2]) = ((void *)i1);

  /* "scipy/special/_ufuncs.pyx":8222
 * ufunc_i1_ptr[2*0+1] = <void*>(<char*>"i1")
 * ufunc_i1_ptr[2*1] = <void*>_func_i1
 * ufunc_i1_ptr[2*1+1] = <void*>(<char*>"i1")             # <<<<<<<<<<<<<<
 * ufunc_i1_data[0] = &ufunc_i1_ptr[2*0]
 * ufunc_i1_data[1] = &ufunc_i1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_ptr[3]) = ((void *)((char *)((char *)"i1")));

  /* "scipy/special/_ufuncs.pyx":8223
 * ufunc_i1_ptr[2*1] = <void*>_func_i1
 * ufunc_i1_ptr[2*1+1] = <void*>(<char*>"i1")
 * ufunc_i1_data[0] = &ufunc_i1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_i1_data[1] = &ufunc_i1_ptr[2*1]
 * i1 = np.PyUFunc_FromFuncAndData(ufunc_i1_loops, ufunc_i1_data, ufunc_i1_types, 2, 1, 1, 0, "i1", ufunc_i1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8224
 * ufunc_i1_ptr[2*1+1] = <void*>(<char*>"i1")
 * ufunc_i1_data[0] = &ufunc_i1_ptr[2*0]
 * ufunc_i1_data[1] = &ufunc_i1_ptr[2*1]             # <<<<<<<<<<<<<<
 * i1 = np.PyUFunc_FromFuncAndData(ufunc_i1_loops, ufunc_i1_data, ufunc_i1_types, 2, 1, 1, 0, "i1", ufunc_i1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8225
 * ufunc_i1_data[0] = &ufunc_i1_ptr[2*0]
 * ufunc_i1_data[1] = &ufunc_i1_ptr[2*1]
 * i1 = np.PyUFunc_FromFuncAndData(ufunc_i1_loops, ufunc_i1_data, ufunc_i1_types, 2, 1, 1, 0, "i1", ufunc_i1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_i1e_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_types, 2, 1, 1, 0, ((char *)"i1"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8225, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_i1, __pyx_t_2) < 0) __PYX_ERR(1, 8225, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":8231
 * cdef void *ufunc_i1e_data[2]
 * cdef char ufunc_i1e_types[4]
 * cdef char *ufunc_i1e_doc = (             # <<<<<<<<<<<<<<
 *     "i1e(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_doc = ((char *)"i1e(x)\n\nExponentially scaled modified Bessel function of order 1.\n\nDefined as::\n\n    i1e(x) = exp(-abs(x)) * i1(x)\n\nParameters\n----------\nx : array_like\n    Argument (float)\n\nReturns\n-------\nI : ndarray\n    Value of the exponentially scaled modified Bessel function of order 1\n    at `x`.\n\nNotes\n-----\nThe range is partitioned into the two intervals [0, 8] and (8, infinity).\nChebyshev polynomial expansions are employed in each interval. The\npolynomial expansions used are the same as those in `i1`, but\nthey are not multiplied by the dominant exponential factor.\n\nThis function is a wrapper for the Cephes [1]_ routine `i1e`.\n\nSee also\n--------\niv\ni1\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":8269
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_i1e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_i1e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i1e_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":8270
 *     "       http://www.netlib.org/cephes/")
 * ufunc_i1e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_i1e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_i1e_types[0] = <char>NPY_FLOAT
 * ufunc_i1e_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":8271
 * ufunc_i1e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_i1e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i1e_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_i1e_types[1] = <char>NPY_FLOAT
 * ufunc_i1e_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8272
 * ufunc_i1e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_i1e_types[0] = <char>NPY_FLOAT
 * ufunc_i1e_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_i1e_types[2] = <char>NPY_DOUBLE
 * ufunc_i1e_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8273
 * ufunc_i1e_types[0] = <char>NPY_FLOAT
 * ufunc_i1e_types[1] = <char>NPY_FLOAT
 * ufunc_i1e_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_i1e_types[3] = <char>NPY_DOUBLE
 * ufunc_i1e_ptr[2*0] = <void*>_func_i1e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8274
 * ufunc_i1e_types[1] = <char>NPY_FLOAT
 * ufunc_i1e_types[2] = <char>NPY_DOUBLE
 * ufunc_i1e_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_i1e_ptr[2*0] = <void*>_func_i1e
 * ufunc_i1e_ptr[2*0+1] = <void*>(<char*>"i1e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8275
 * ufunc_i1e_types[2] = <char>NPY_DOUBLE
 * ufunc_i1e_types[3] = <char>NPY_DOUBLE
 * ufunc_i1e_ptr[2*0] = <void*>_func_i1e             # <<<<<<<<<<<<<<
 * ufunc_i1e_ptr[2*0+1] = <void*>(<char*>"i1e")
 * ufunc_i1e_ptr[2*1] = <void*>_func_i1e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_ptr[0]) = ((void *)i1e);

  /* "scipy/special/_ufuncs.pyx":8276
 * ufunc_i1e_types[3] = <char>NPY_DOUBLE
 * ufunc_i1e_ptr[2*0] = <void*>_func_i1e
 * ufunc_i1e_ptr[2*0+1] = <void*>(<char*>"i1e")             # <<<<<<<<<<<<<<
 * ufunc_i1e_ptr[2*1] = <void*>_func_i1e
 * ufunc_i1e_ptr[2*1+1] = <void*>(<char*>"i1e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_ptr[1]) = ((void *)((char *)((char *)"i1e")));

  /* "scipy/special/_ufuncs.pyx":8277
 * ufunc_i1e_ptr[2*0] = <void*>_func_i1e
 * ufunc_i1e_ptr[2*0+1] = <void*>(<char*>"i1e")
 * ufunc_i1e_ptr[2*1] = <void*>_func_i1e             # <<<<<<<<<<<<<<
 * ufunc_i1e_ptr[2*1+1] = <void*>(<char*>"i1e")
 * ufunc_i1e_data[0] = &ufunc_i1e_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_ptr[2]) = ((void *)i1e);

  /* "scipy/special/_ufuncs.pyx":8278
 * ufunc_i1e_ptr[2*0+1] = <void*>(<char*>"i1e")
 * ufunc_i1e_ptr[2*1] = <void*>_func_i1e
 * ufunc_i1e_ptr[2*1+1] = <void*>(<char*>"i1e")             # <<<<<<<<<<<<<<
 * ufunc_i1e_data[0] = &ufunc_i1e_ptr[2*0]
 * ufunc_i1e_data[1] = &ufunc_i1e_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_ptr[3]) = ((void *)((char *)((char *)"i1e")));

  /* "scipy/special/_ufuncs.pyx":8279
 * ufunc_i1e_ptr[2*1] = <void*>_func_i1e
 * ufunc_i1e_ptr[2*1+1] = <void*>(<char*>"i1e")
 * ufunc_i1e_data[0] = &ufunc_i1e_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_i1e_data[1] = &ufunc_i1e_ptr[2*1]
 * i1e = np.PyUFunc_FromFuncAndData(ufunc_i1e_loops, ufunc_i1e_data, ufunc_i1e_types, 2, 1, 1, 0, "i1e", ufunc_i1e_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8280
 * ufunc_i1e_ptr[2*1+1] = <void*>(<char*>"i1e")
 * ufunc_i1e_data[0] = &ufunc_i1e_ptr[2*0]
 * ufunc_i1e_data[1] = &ufunc_i1e_ptr[2*1]             # <<<<<<<<<<<<<<
 * i1e = np.PyUFunc_FromFuncAndData(ufunc_i1e_loops, ufunc_i1e_data, ufunc_i1e_types, 2, 1, 1, 0, "i1e", ufunc_i1e_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8281
 * ufunc_i1e_data[0] = &ufunc_i1e_ptr[2*0]
 * ufunc_i1e_data[1] = &ufunc_i1e_ptr[2*1]
 * i1e = np.PyUFunc_FromFuncAndData(ufunc_i1e_loops, ufunc_i1e_data, ufunc_i1e_types, 2, 1, 1, 0, "i1e", ufunc_i1e_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_inv_boxcox_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_types, 2, 1, 1, 0, ((char *)"i1e"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_i1e_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_i1e, __pyx_t_2) < 0) __PYX_ERR(1, 8281, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":8287
 * cdef void *ufunc_inv_boxcox_data[2]
 * cdef char ufunc_inv_boxcox_types[6]
 * cdef char *ufunc_inv_boxcox_doc = (             # <<<<<<<<<<<<<<
 *     "inv_boxcox(y, lmbda)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_doc = ((char *)"inv_boxcox(y, lmbda)\n\nCompute the inverse of the Box-Cox transformation.\n\nFind ``x`` such that::\n\n    y = (x**lmbda - 1) / lmbda  if lmbda != 0\n        log(x)                  if lmbda == 0\n\nParameters\n----------\ny : array_like\n    Data to be transformed.\nlmbda : array_like\n    Power parameter of the Box-Cox transform.\n\nReturns\n-------\nx : array\n    Transformed data.\n\nNotes\n-----\n\n.. versionadded:: 0.16.0\n\nExamples\n--------\n>>> from scipy.special import boxcox, inv_boxcox\n>>> y = boxcox([1, 4, 10], 2.5)\n>>> inv_boxcox(y, 2.5)\narray([1., 4., 10.])");

  /* "scipy/special/_ufuncs.pyx":8320
 *     ">>> inv_boxcox(y, 2.5)\n"
 *     "array([1., 4., 10.])")
 * ufunc_inv_boxcox_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_inv_boxcox_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":8321
 *     "array([1., 4., 10.])")
 * ufunc_inv_boxcox_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_inv_boxcox_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_types[0] = <char>NPY_FLOAT
 * ufunc_inv_boxcox_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":8322
 * ufunc_inv_boxcox_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_inv_boxcox_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_inv_boxcox_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_types[1] = <char>NPY_FLOAT
 * ufunc_inv_boxcox_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8323
 * ufunc_inv_boxcox_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_inv_boxcox_types[0] = <char>NPY_FLOAT
 * ufunc_inv_boxcox_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_types[2] = <char>NPY_FLOAT
 * ufunc_inv_boxcox_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8324
 * ufunc_inv_boxcox_types[0] = <char>NPY_FLOAT
 * ufunc_inv_boxcox_types[1] = <char>NPY_FLOAT
 * ufunc_inv_boxcox_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_types[3] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8325
 * ufunc_inv_boxcox_types[1] = <char>NPY_FLOAT
 * ufunc_inv_boxcox_types[2] = <char>NPY_FLOAT
 * ufunc_inv_boxcox_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_types[4] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8326
 * ufunc_inv_boxcox_types[2] = <char>NPY_FLOAT
 * ufunc_inv_boxcox_types[3] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_types[5] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox_ptr[2*0] = <void*>_func_inv_boxcox
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8327
 * ufunc_inv_boxcox_types[3] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox_types[4] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_ptr[2*0] = <void*>_func_inv_boxcox
 * ufunc_inv_boxcox_ptr[2*0+1] = <void*>(<char*>"inv_boxcox")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8328
 * ufunc_inv_boxcox_types[4] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox_types[5] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox_ptr[2*0] = <void*>_func_inv_boxcox             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_ptr[2*0+1] = <void*>(<char*>"inv_boxcox")
 * ufunc_inv_boxcox_ptr[2*1] = <void*>_func_inv_boxcox
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_ptr[0]) = ((void *)__pyx_f_5scipy_7special_7_boxcox_inv_boxcox);

  /* "scipy/special/_ufuncs.pyx":8329
 * ufunc_inv_boxcox_types[5] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox_ptr[2*0] = <void*>_func_inv_boxcox
 * ufunc_inv_boxcox_ptr[2*0+1] = <void*>(<char*>"inv_boxcox")             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_ptr[2*1] = <void*>_func_inv_boxcox
 * ufunc_inv_boxcox_ptr[2*1+1] = <void*>(<char*>"inv_boxcox")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_ptr[1]) = ((void *)((char *)((char *)"inv_boxcox")));

  /* "scipy/special/_ufuncs.pyx":8330
 * ufunc_inv_boxcox_ptr[2*0] = <void*>_func_inv_boxcox
 * ufunc_inv_boxcox_ptr[2*0+1] = <void*>(<char*>"inv_boxcox")
 * ufunc_inv_boxcox_ptr[2*1] = <void*>_func_inv_boxcox             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_ptr[2*1+1] = <void*>(<char*>"inv_boxcox")
 * ufunc_inv_boxcox_data[0] = &ufunc_inv_boxcox_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_boxcox_inv_boxcox);

  /* "scipy/special/_ufuncs.pyx":8331
 * ufunc_inv_boxcox_ptr[2*0+1] = <void*>(<char*>"inv_boxcox")
 * ufunc_inv_boxcox_ptr[2*1] = <void*>_func_inv_boxcox
 * ufunc_inv_boxcox_ptr[2*1+1] = <void*>(<char*>"inv_boxcox")             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_data[0] = &ufunc_inv_boxcox_ptr[2*0]
 * ufunc_inv_boxcox_data[1] = &ufunc_inv_boxcox_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_ptr[3]) = ((void *)((char *)((char *)"inv_boxcox")));

  /* "scipy/special/_ufuncs.pyx":8332
 * ufunc_inv_boxcox_ptr[2*1] = <void*>_func_inv_boxcox
 * ufunc_inv_boxcox_ptr[2*1+1] = <void*>(<char*>"inv_boxcox")
 * ufunc_inv_boxcox_data[0] = &ufunc_inv_boxcox_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox_data[1] = &ufunc_inv_boxcox_ptr[2*1]
 * inv_boxcox = np.PyUFunc_FromFuncAndData(ufunc_inv_boxcox_loops, ufunc_inv_boxcox_data, ufunc_inv_boxcox_types, 2, 2, 1, 0, "inv_boxcox", ufunc_inv_boxcox_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8333
 * ufunc_inv_boxcox_ptr[2*1+1] = <void*>(<char*>"inv_boxcox")
 * ufunc_inv_boxcox_data[0] = &ufunc_inv_boxcox_ptr[2*0]
 * ufunc_inv_boxcox_data[1] = &ufunc_inv_boxcox_ptr[2*1]             # <<<<<<<<<<<<<<
 * inv_boxcox = np.PyUFunc_FromFuncAndData(ufunc_inv_boxcox_loops, ufunc_inv_boxcox_data, ufunc_inv_boxcox_types, 2, 2, 1, 0, "inv_boxcox", ufunc_inv_boxcox_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8334
 * ufunc_inv_boxcox_data[0] = &ufunc_inv_boxcox_ptr[2*0]
 * ufunc_inv_boxcox_data[1] = &ufunc_inv_boxcox_ptr[2*1]
 * inv_boxcox = np.PyUFunc_FromFuncAndData(ufunc_inv_boxcox_loops, ufunc_inv_boxcox_data, ufunc_inv_boxcox_types, 2, 2, 1, 0, "inv_boxcox", ufunc_inv_boxcox_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_inv_boxcox1p_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_types, 2, 2, 1, 0, ((char *)"inv_boxcox"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8334, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_inv_boxcox, __pyx_t_2) < 0) __PYX_ERR(1, 8334, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":8340
 * cdef void *ufunc_inv_boxcox1p_data[2]
 * cdef char ufunc_inv_boxcox1p_types[6]
 * cdef char *ufunc_inv_boxcox1p_doc = (             # <<<<<<<<<<<<<<
 *     "inv_boxcox1p(y, lmbda)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_doc = ((char *)"inv_boxcox1p(y, lmbda)\n\nCompute the inverse of the Box-Cox transformation.\n\nFind ``x`` such that::\n\n    y = ((1+x)**lmbda - 1) / lmbda  if lmbda != 0\n        log(1+x)                    if lmbda == 0\n\nParameters\n----------\ny : array_like\n    Data to be transformed.\nlmbda : array_like\n    Power parameter of the Box-Cox transform.\n\nReturns\n-------\nx : array\n    Transformed data.\n\nNotes\n-----\n\n.. versionadded:: 0.16.0\n\nExamples\n--------\n>>> from scipy.special import boxcox1p, inv_boxcox1p\n>>> y = boxcox1p([1, 4, 10], 2.5)\n>>> inv_boxcox1p(y, 2.5)\narray([1., 4., 10.])");

  /* "scipy/special/_ufuncs.pyx":8373
 *     ">>> inv_boxcox1p(y, 2.5)\n"
 *     "array([1., 4., 10.])")
 * ufunc_inv_boxcox1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_inv_boxcox1p_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":8374
 *     "array([1., 4., 10.])")
 * ufunc_inv_boxcox1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_inv_boxcox1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_types[0] = <char>NPY_FLOAT
 * ufunc_inv_boxcox1p_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":8375
 * ufunc_inv_boxcox1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_inv_boxcox1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_inv_boxcox1p_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_types[1] = <char>NPY_FLOAT
 * ufunc_inv_boxcox1p_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8376
 * ufunc_inv_boxcox1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_inv_boxcox1p_types[0] = <char>NPY_FLOAT
 * ufunc_inv_boxcox1p_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_types[2] = <char>NPY_FLOAT
 * ufunc_inv_boxcox1p_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8377
 * ufunc_inv_boxcox1p_types[0] = <char>NPY_FLOAT
 * ufunc_inv_boxcox1p_types[1] = <char>NPY_FLOAT
 * ufunc_inv_boxcox1p_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_types[3] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox1p_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8378
 * ufunc_inv_boxcox1p_types[1] = <char>NPY_FLOAT
 * ufunc_inv_boxcox1p_types[2] = <char>NPY_FLOAT
 * ufunc_inv_boxcox1p_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_types[4] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox1p_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8379
 * ufunc_inv_boxcox1p_types[2] = <char>NPY_FLOAT
 * ufunc_inv_boxcox1p_types[3] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox1p_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_types[5] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox1p_ptr[2*0] = <void*>_func_inv_boxcox1p
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8380
 * ufunc_inv_boxcox1p_types[3] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox1p_types[4] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox1p_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_ptr[2*0] = <void*>_func_inv_boxcox1p
 * ufunc_inv_boxcox1p_ptr[2*0+1] = <void*>(<char*>"inv_boxcox1p")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8381
 * ufunc_inv_boxcox1p_types[4] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox1p_types[5] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox1p_ptr[2*0] = <void*>_func_inv_boxcox1p             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_ptr[2*0+1] = <void*>(<char*>"inv_boxcox1p")
 * ufunc_inv_boxcox1p_ptr[2*1] = <void*>_func_inv_boxcox1p
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_ptr[0]) = ((void *)__pyx_f_5scipy_7special_7_boxcox_inv_boxcox1p);

  /* "scipy/special/_ufuncs.pyx":8382
 * ufunc_inv_boxcox1p_types[5] = <char>NPY_DOUBLE
 * ufunc_inv_boxcox1p_ptr[2*0] = <void*>_func_inv_boxcox1p
 * ufunc_inv_boxcox1p_ptr[2*0+1] = <void*>(<char*>"inv_boxcox1p")             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_ptr[2*1] = <void*>_func_inv_boxcox1p
 * ufunc_inv_boxcox1p_ptr[2*1+1] = <void*>(<char*>"inv_boxcox1p")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_ptr[1]) = ((void *)((char *)((char *)"inv_boxcox1p")));

  /* "scipy/special/_ufuncs.pyx":8383
 * ufunc_inv_boxcox1p_ptr[2*0] = <void*>_func_inv_boxcox1p
 * ufunc_inv_boxcox1p_ptr[2*0+1] = <void*>(<char*>"inv_boxcox1p")
 * ufunc_inv_boxcox1p_ptr[2*1] = <void*>_func_inv_boxcox1p             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_ptr[2*1+1] = <void*>(<char*>"inv_boxcox1p")
 * ufunc_inv_boxcox1p_data[0] = &ufunc_inv_boxcox1p_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_boxcox_inv_boxcox1p);

  /* "scipy/special/_ufuncs.pyx":8384
 * ufunc_inv_boxcox1p_ptr[2*0+1] = <void*>(<char*>"inv_boxcox1p")
 * ufunc_inv_boxcox1p_ptr[2*1] = <void*>_func_inv_boxcox1p
 * ufunc_inv_boxcox1p_ptr[2*1+1] = <void*>(<char*>"inv_boxcox1p")             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_data[0] = &ufunc_inv_boxcox1p_ptr[2*0]
 * ufunc_inv_boxcox1p_data[1] = &ufunc_inv_boxcox1p_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_ptr[3]) = ((void *)((char *)((char *)"inv_boxcox1p")));

  /* "scipy/special/_ufuncs.pyx":8385
 * ufunc_inv_boxcox1p_ptr[2*1] = <void*>_func_inv_boxcox1p
 * ufunc_inv_boxcox1p_ptr[2*1+1] = <void*>(<char*>"inv_boxcox1p")
 * ufunc_inv_boxcox1p_data[0] = &ufunc_inv_boxcox1p_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_inv_boxcox1p_data[1] = &ufunc_inv_boxcox1p_ptr[2*1]
 * inv_boxcox1p = np.PyUFunc_FromFuncAndData(ufunc_inv_boxcox1p_loops, ufunc_inv_boxcox1p_data, ufunc_inv_boxcox1p_types, 2, 2, 1, 0, "inv_boxcox1p", ufunc_inv_boxcox1p_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8386
 * ufunc_inv_boxcox1p_ptr[2*1+1] = <void*>(<char*>"inv_boxcox1p")
 * ufunc_inv_boxcox1p_data[0] = &ufunc_inv_boxcox1p_ptr[2*0]
 * ufunc_inv_boxcox1p_data[1] = &ufunc_inv_boxcox1p_ptr[2*1]             # <<<<<<<<<<<<<<
 * inv_boxcox1p = np.PyUFunc_FromFuncAndData(ufunc_inv_boxcox1p_loops, ufunc_inv_boxcox1p_data, ufunc_inv_boxcox1p_types, 2, 2, 1, 0, "inv_boxcox1p", ufunc_inv_boxcox1p_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8387
 * ufunc_inv_boxcox1p_data[0] = &ufunc_inv_boxcox1p_ptr[2*0]
 * ufunc_inv_boxcox1p_data[1] = &ufunc_inv_boxcox1p_ptr[2*1]
 * inv_boxcox1p = np.PyUFunc_FromFuncAndData(ufunc_inv_boxcox1p_loops, ufunc_inv_boxcox1p_data, ufunc_inv_boxcox1p_types, 2, 2, 1, 0, "inv_boxcox1p", ufunc_inv_boxcox1p_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_it2i0k0_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_types, 2, 2, 1, 0, ((char *)"inv_boxcox1p"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_inv_boxcox1p_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8387, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_inv_boxcox1p, __pyx_t_2) < 0) __PYX_ERR(1, 8387, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":8393
 * cdef void *ufunc_it2i0k0_data[2]
 * cdef char ufunc_it2i0k0_types[6]
 * cdef char *ufunc_it2i0k0_doc = (             # <<<<<<<<<<<<<<
 *     "it2i0k0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_doc = ((char *)"it2i0k0(x)\n\nIntegrals related to modified Bessel functions of order 0\n\nReturns\n-------\nii0\n    ``integral((i0(t)-1)/t, t=0..x)``\nik0\n    ``integral(k0(t)/t, t=x..inf)``");

  /* "scipy/special/_ufuncs.pyx":8404
 *     "ik0\n"
 *     "    ``integral(k0(t)/t, t=x..inf)``")
 * ufunc_it2i0k0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_it2i0k0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_f_ff);

  /* "scipy/special/_ufuncs.pyx":8405
 *     "    ``integral(k0(t)/t, t=x..inf)``")
 * ufunc_it2i0k0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_it2i0k0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_types[0] = <char>NPY_FLOAT
 * ufunc_it2i0k0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_d_dd);

  /* "scipy/special/_ufuncs.pyx":8406
 * ufunc_it2i0k0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_it2i0k0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_it2i0k0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_types[1] = <char>NPY_FLOAT
 * ufunc_it2i0k0_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8407
 * ufunc_it2i0k0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_it2i0k0_types[0] = <char>NPY_FLOAT
 * ufunc_it2i0k0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_types[2] = <char>NPY_FLOAT
 * ufunc_it2i0k0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8408
 * ufunc_it2i0k0_types[0] = <char>NPY_FLOAT
 * ufunc_it2i0k0_types[1] = <char>NPY_FLOAT
 * ufunc_it2i0k0_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_types[3] = <char>NPY_DOUBLE
 * ufunc_it2i0k0_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8409
 * ufunc_it2i0k0_types[1] = <char>NPY_FLOAT
 * ufunc_it2i0k0_types[2] = <char>NPY_FLOAT
 * ufunc_it2i0k0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_types[4] = <char>NPY_DOUBLE
 * ufunc_it2i0k0_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8410
 * ufunc_it2i0k0_types[2] = <char>NPY_FLOAT
 * ufunc_it2i0k0_types[3] = <char>NPY_DOUBLE
 * ufunc_it2i0k0_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_types[5] = <char>NPY_DOUBLE
 * ufunc_it2i0k0_ptr[2*0] = <void*>_func_it2i0k0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8411
 * ufunc_it2i0k0_types[3] = <char>NPY_DOUBLE
 * ufunc_it2i0k0_types[4] = <char>NPY_DOUBLE
 * ufunc_it2i0k0_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_ptr[2*0] = <void*>_func_it2i0k0_wrap
 * ufunc_it2i0k0_ptr[2*0+1] = <void*>(<char*>"it2i0k0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8412
 * ufunc_it2i0k0_types[4] = <char>NPY_DOUBLE
 * ufunc_it2i0k0_types[5] = <char>NPY_DOUBLE
 * ufunc_it2i0k0_ptr[2*0] = <void*>_func_it2i0k0_wrap             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_ptr[2*0+1] = <void*>(<char*>"it2i0k0")
 * ufunc_it2i0k0_ptr[2*1] = <void*>_func_it2i0k0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_ptr[0]) = ((void *)it2i0k0_wrap);

  /* "scipy/special/_ufuncs.pyx":8413
 * ufunc_it2i0k0_types[5] = <char>NPY_DOUBLE
 * ufunc_it2i0k0_ptr[2*0] = <void*>_func_it2i0k0_wrap
 * ufunc_it2i0k0_ptr[2*0+1] = <void*>(<char*>"it2i0k0")             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_ptr[2*1] = <void*>_func_it2i0k0_wrap
 * ufunc_it2i0k0_ptr[2*1+1] = <void*>(<char*>"it2i0k0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_ptr[1]) = ((void *)((char *)((char *)"it2i0k0")));

  /* "scipy/special/_ufuncs.pyx":8414
 * ufunc_it2i0k0_ptr[2*0] = <void*>_func_it2i0k0_wrap
 * ufunc_it2i0k0_ptr[2*0+1] = <void*>(<char*>"it2i0k0")
 * ufunc_it2i0k0_ptr[2*1] = <void*>_func_it2i0k0_wrap             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_ptr[2*1+1] = <void*>(<char*>"it2i0k0")
 * ufunc_it2i0k0_data[0] = &ufunc_it2i0k0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_ptr[2]) = ((void *)it2i0k0_wrap);

  /* "scipy/special/_ufuncs.pyx":8415
 * ufunc_it2i0k0_ptr[2*0+1] = <void*>(<char*>"it2i0k0")
 * ufunc_it2i0k0_ptr[2*1] = <void*>_func_it2i0k0_wrap
 * ufunc_it2i0k0_ptr[2*1+1] = <void*>(<char*>"it2i0k0")             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_data[0] = &ufunc_it2i0k0_ptr[2*0]
 * ufunc_it2i0k0_data[1] = &ufunc_it2i0k0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_ptr[3]) = ((void *)((char *)((char *)"it2i0k0")));

  /* "scipy/special/_ufuncs.pyx":8416
 * ufunc_it2i0k0_ptr[2*1] = <void*>_func_it2i0k0_wrap
 * ufunc_it2i0k0_ptr[2*1+1] = <void*>(<char*>"it2i0k0")
 * ufunc_it2i0k0_data[0] = &ufunc_it2i0k0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_it2i0k0_data[1] = &ufunc_it2i0k0_ptr[2*1]
 * it2i0k0 = np.PyUFunc_FromFuncAndData(ufunc_it2i0k0_loops, ufunc_it2i0k0_data, ufunc_it2i0k0_types, 2, 1, 2, 0, "it2i0k0", ufunc_it2i0k0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8417
 * ufunc_it2i0k0_ptr[2*1+1] = <void*>(<char*>"it2i0k0")
 * ufunc_it2i0k0_data[0] = &ufunc_it2i0k0_ptr[2*0]
 * ufunc_it2i0k0_data[1] = &ufunc_it2i0k0_ptr[2*1]             # <<<<<<<<<<<<<<
 * it2i0k0 = np.PyUFunc_FromFuncAndData(ufunc_it2i0k0_loops, ufunc_it2i0k0_data, ufunc_it2i0k0_types, 2, 1, 2, 0, "it2i0k0", ufunc_it2i0k0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8418
 * ufunc_it2i0k0_data[0] = &ufunc_it2i0k0_ptr[2*0]
 * ufunc_it2i0k0_data[1] = &ufunc_it2i0k0_ptr[2*1]
 * it2i0k0 = np.PyUFunc_FromFuncAndData(ufunc_it2i0k0_loops, ufunc_it2i0k0_data, ufunc_it2i0k0_types, 2, 1, 2, 0, "it2i0k0", ufunc_it2i0k0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_it2j0y0_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_types, 2, 1, 2, 0, ((char *)"it2i0k0"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2i0k0_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8418, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_it2i0k0, __pyx_t_2) < 0) __PYX_ERR(1, 8418, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":8424
 * cdef void *ufunc_it2j0y0_data[2]
 * cdef char ufunc_it2j0y0_types[6]
 * cdef char *ufunc_it2j0y0_doc = (             # <<<<<<<<<<<<<<
 *     "it2j0y0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_doc = ((char *)"it2j0y0(x)\n\nIntegrals related to Bessel functions of order 0\n\nReturns\n-------\nij0\n    ``integral((1-j0(t))/t, t=0..x)``\niy0\n    ``integral(y0(t)/t, t=x..inf)``");

  /* "scipy/special/_ufuncs.pyx":8435
 *     "iy0\n"
 *     "    ``integral(y0(t)/t, t=x..inf)``")
 * ufunc_it2j0y0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_it2j0y0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_f_ff);

  /* "scipy/special/_ufuncs.pyx":8436
 *     "    ``integral(y0(t)/t, t=x..inf)``")
 * ufunc_it2j0y0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_it2j0y0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_types[0] = <char>NPY_FLOAT
 * ufunc_it2j0y0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_d_dd);

  /* "scipy/special/_ufuncs.pyx":8437
 * ufunc_it2j0y0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_it2j0y0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_it2j0y0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_types[1] = <char>NPY_FLOAT
 * ufunc_it2j0y0_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8438
 * ufunc_it2j0y0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_it2j0y0_types[0] = <char>NPY_FLOAT
 * ufunc_it2j0y0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_types[2] = <char>NPY_FLOAT
 * ufunc_it2j0y0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8439
 * ufunc_it2j0y0_types[0] = <char>NPY_FLOAT
 * ufunc_it2j0y0_types[1] = <char>NPY_FLOAT
 * ufunc_it2j0y0_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_types[3] = <char>NPY_DOUBLE
 * ufunc_it2j0y0_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8440
 * ufunc_it2j0y0_types[1] = <char>NPY_FLOAT
 * ufunc_it2j0y0_types[2] = <char>NPY_FLOAT
 * ufunc_it2j0y0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_types[4] = <char>NPY_DOUBLE
 * ufunc_it2j0y0_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8441
 * ufunc_it2j0y0_types[2] = <char>NPY_FLOAT
 * ufunc_it2j0y0_types[3] = <char>NPY_DOUBLE
 * ufunc_it2j0y0_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_types[5] = <char>NPY_DOUBLE
 * ufunc_it2j0y0_ptr[2*0] = <void*>_func_it2j0y0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8442
 * ufunc_it2j0y0_types[3] = <char>NPY_DOUBLE
 * ufunc_it2j0y0_types[4] = <char>NPY_DOUBLE
 * ufunc_it2j0y0_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_ptr[2*0] = <void*>_func_it2j0y0_wrap
 * ufunc_it2j0y0_ptr[2*0+1] = <void*>(<char*>"it2j0y0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8443
 * ufunc_it2j0y0_types[4] = <char>NPY_DOUBLE
 * ufunc_it2j0y0_types[5] = <char>NPY_DOUBLE
 * ufunc_it2j0y0_ptr[2*0] = <void*>_func_it2j0y0_wrap             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_ptr[2*0+1] = <void*>(<char*>"it2j0y0")
 * ufunc_it2j0y0_ptr[2*1] = <void*>_func_it2j0y0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_ptr[0]) = ((void *)it2j0y0_wrap);

  /* "scipy/special/_ufuncs.pyx":8444
 * ufunc_it2j0y0_types[5] = <char>NPY_DOUBLE
 * ufunc_it2j0y0_ptr[2*0] = <void*>_func_it2j0y0_wrap
 * ufunc_it2j0y0_ptr[2*0+1] = <void*>(<char*>"it2j0y0")             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_ptr[2*1] = <void*>_func_it2j0y0_wrap
 * ufunc_it2j0y0_ptr[2*1+1] = <void*>(<char*>"it2j0y0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_ptr[1]) = ((void *)((char *)((char *)"it2j0y0")));

  /* "scipy/special/_ufuncs.pyx":8445
 * ufunc_it2j0y0_ptr[2*0] = <void*>_func_it2j0y0_wrap
 * ufunc_it2j0y0_ptr[2*0+1] = <void*>(<char*>"it2j0y0")
 * ufunc_it2j0y0_ptr[2*1] = <void*>_func_it2j0y0_wrap             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_ptr[2*1+1] = <void*>(<char*>"it2j0y0")
 * ufunc_it2j0y0_data[0] = &ufunc_it2j0y0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_ptr[2]) = ((void *)it2j0y0_wrap);

  /* "scipy/special/_ufuncs.pyx":8446
 * ufunc_it2j0y0_ptr[2*0+1] = <void*>(<char*>"it2j0y0")
 * ufunc_it2j0y0_ptr[2*1] = <void*>_func_it2j0y0_wrap
 * ufunc_it2j0y0_ptr[2*1+1] = <void*>(<char*>"it2j0y0")             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_data[0] = &ufunc_it2j0y0_ptr[2*0]
 * ufunc_it2j0y0_data[1] = &ufunc_it2j0y0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_ptr[3]) = ((void *)((char *)((char *)"it2j0y0")));

  /* "scipy/special/_ufuncs.pyx":8447
 * ufunc_it2j0y0_ptr[2*1] = <void*>_func_it2j0y0_wrap
 * ufunc_it2j0y0_ptr[2*1+1] = <void*>(<char*>"it2j0y0")
 * ufunc_it2j0y0_data[0] = &ufunc_it2j0y0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_it2j0y0_data[1] = &ufunc_it2j0y0_ptr[2*1]
 * it2j0y0 = np.PyUFunc_FromFuncAndData(ufunc_it2j0y0_loops, ufunc_it2j0y0_data, ufunc_it2j0y0_types, 2, 1, 2, 0, "it2j0y0", ufunc_it2j0y0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8448
 * ufunc_it2j0y0_ptr[2*1+1] = <void*>(<char*>"it2j0y0")
 * ufunc_it2j0y0_data[0] = &ufunc_it2j0y0_ptr[2*0]
 * ufunc_it2j0y0_data[1] = &ufunc_it2j0y0_ptr[2*1]             # <<<<<<<<<<<<<<
 * it2j0y0 = np.PyUFunc_FromFuncAndData(ufunc_it2j0y0_loops, ufunc_it2j0y0_data, ufunc_it2j0y0_types, 2, 1, 2, 0, "it2j0y0", ufunc_it2j0y0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8449
 * ufunc_it2j0y0_data[0] = &ufunc_it2j0y0_ptr[2*0]
 * ufunc_it2j0y0_data[1] = &ufunc_it2j0y0_ptr[2*1]
 * it2j0y0 = np.PyUFunc_FromFuncAndData(ufunc_it2j0y0_loops, ufunc_it2j0y0_data, ufunc_it2j0y0_types, 2, 1, 2, 0, "it2j0y0", ufunc_it2j0y0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_it2struve0_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_types, 2, 1, 2, 0, ((char *)"it2j0y0"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2j0y0_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_it2j0y0, __pyx_t_2) < 0) __PYX_ERR(1, 8449, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":8455
 * cdef void *ufunc_it2struve0_data[2]
 * cdef char ufunc_it2struve0_types[4]
 * cdef char *ufunc_it2struve0_doc = (             # <<<<<<<<<<<<<<
 *     "it2struve0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_doc = ((char *)"it2struve0(x)\n\nIntegral related to the Struve function of order 0.\n\nReturns the integral,\n\n.. math::\n    \\int_x^\\infty \\frac{H_0(t)}{t}\\,dt\n\nwhere :math:`H_0` is the Struve function of order 0.\n\nParameters\n----------\nx : array_like\n    Lower limit of integration.\n\nReturns\n-------\nI : ndarray\n    The value of the integral.\n\nSee also\n--------\nstruve\n\nNotes\n-----\nWrapper for a Fortran routine created by Shanjie Zhang and Jianming\nJin [1]_.\n\nReferences\n----------\n.. [1] Zhang, Shanjie and Jin, Jianming. \"Computation of Special\n       Functions\", John Wiley and Sons, 1996.\n       https://people.sc.fsu.edu/~jburkardt/f_src/special_functions/special_functions.html");

  /* "scipy/special/_ufuncs.pyx":8491
 *     "       Functions\", John Wiley and Sons, 1996.\n"
 *     "       https://people.sc.fsu.edu/~jburkardt/f_src/special_functions/special_functions.html")
 * ufunc_it2struve0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_it2struve0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":8492
 *     "       https://people.sc.fsu.edu/~jburkardt/f_src/special_functions/special_functions.html")
 * ufunc_it2struve0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_it2struve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_types[0] = <char>NPY_FLOAT
 * ufunc_it2struve0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":8493
 * ufunc_it2struve0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_it2struve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_it2struve0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_types[1] = <char>NPY_FLOAT
 * ufunc_it2struve0_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8494
 * ufunc_it2struve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_it2struve0_types[0] = <char>NPY_FLOAT
 * ufunc_it2struve0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_types[2] = <char>NPY_DOUBLE
 * ufunc_it2struve0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8495
 * ufunc_it2struve0_types[0] = <char>NPY_FLOAT
 * ufunc_it2struve0_types[1] = <char>NPY_FLOAT
 * ufunc_it2struve0_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_types[3] = <char>NPY_DOUBLE
 * ufunc_it2struve0_ptr[2*0] = <void*>_func_it2struve0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8496
 * ufunc_it2struve0_types[1] = <char>NPY_FLOAT
 * ufunc_it2struve0_types[2] = <char>NPY_DOUBLE
 * ufunc_it2struve0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_ptr[2*0] = <void*>_func_it2struve0_wrap
 * ufunc_it2struve0_ptr[2*0+1] = <void*>(<char*>"it2struve0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8497
 * ufunc_it2struve0_types[2] = <char>NPY_DOUBLE
 * ufunc_it2struve0_types[3] = <char>NPY_DOUBLE
 * ufunc_it2struve0_ptr[2*0] = <void*>_func_it2struve0_wrap             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_ptr[2*0+1] = <void*>(<char*>"it2struve0")
 * ufunc_it2struve0_ptr[2*1] = <void*>_func_it2struve0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_ptr[0]) = ((void *)it2struve0_wrap);

  /* "scipy/special/_ufuncs.pyx":8498
 * ufunc_it2struve0_types[3] = <char>NPY_DOUBLE
 * ufunc_it2struve0_ptr[2*0] = <void*>_func_it2struve0_wrap
 * ufunc_it2struve0_ptr[2*0+1] = <void*>(<char*>"it2struve0")             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_ptr[2*1] = <void*>_func_it2struve0_wrap
 * ufunc_it2struve0_ptr[2*1+1] = <void*>(<char*>"it2struve0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_ptr[1]) = ((void *)((char *)((char *)"it2struve0")));

  /* "scipy/special/_ufuncs.pyx":8499
 * ufunc_it2struve0_ptr[2*0] = <void*>_func_it2struve0_wrap
 * ufunc_it2struve0_ptr[2*0+1] = <void*>(<char*>"it2struve0")
 * ufunc_it2struve0_ptr[2*1] = <void*>_func_it2struve0_wrap             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_ptr[2*1+1] = <void*>(<char*>"it2struve0")
 * ufunc_it2struve0_data[0] = &ufunc_it2struve0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_ptr[2]) = ((void *)it2struve0_wrap);

  /* "scipy/special/_ufuncs.pyx":8500
 * ufunc_it2struve0_ptr[2*0+1] = <void*>(<char*>"it2struve0")
 * ufunc_it2struve0_ptr[2*1] = <void*>_func_it2struve0_wrap
 * ufunc_it2struve0_ptr[2*1+1] = <void*>(<char*>"it2struve0")             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_data[0] = &ufunc_it2struve0_ptr[2*0]
 * ufunc_it2struve0_data[1] = &ufunc_it2struve0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_ptr[3]) = ((void *)((char *)((char *)"it2struve0")));

  /* "scipy/special/_ufuncs.pyx":8501
 * ufunc_it2struve0_ptr[2*1] = <void*>_func_it2struve0_wrap
 * ufunc_it2struve0_ptr[2*1+1] = <void*>(<char*>"it2struve0")
 * ufunc_it2struve0_data[0] = &ufunc_it2struve0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_it2struve0_data[1] = &ufunc_it2struve0_ptr[2*1]
 * it2struve0 = np.PyUFunc_FromFuncAndData(ufunc_it2struve0_loops, ufunc_it2struve0_data, ufunc_it2struve0_types, 2, 1, 1, 0, "it2struve0", ufunc_it2struve0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8502
 * ufunc_it2struve0_ptr[2*1+1] = <void*>(<char*>"it2struve0")
 * ufunc_it2struve0_data[0] = &ufunc_it2struve0_ptr[2*0]
 * ufunc_it2struve0_data[1] = &ufunc_it2struve0_ptr[2*1]             # <<<<<<<<<<<<<<
 * it2struve0 = np.PyUFunc_FromFuncAndData(ufunc_it2struve0_loops, ufunc_it2struve0_data, ufunc_it2struve0_types, 2, 1, 1, 0, "it2struve0", ufunc_it2struve0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8503
 * ufunc_it2struve0_data[0] = &ufunc_it2struve0_ptr[2*0]
 * ufunc_it2struve0_data[1] = &ufunc_it2struve0_ptr[2*1]
 * it2struve0 = np.PyUFunc_FromFuncAndData(ufunc_it2struve0_loops, ufunc_it2struve0_data, ufunc_it2struve0_types, 2, 1, 1, 0, "it2struve0", ufunc_it2struve0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_itairy_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_types, 2, 1, 1, 0, ((char *)"it2struve0"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_it2struve0_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_it2struve0, __pyx_t_2) < 0) __PYX_ERR(1, 8503, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":8509
 * cdef void *ufunc_itairy_data[2]
 * cdef char ufunc_itairy_types[10]
 * cdef char *ufunc_itairy_doc = (             # <<<<<<<<<<<<<<
 *     "itairy(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_doc = ((char *)"itairy(x)\n\nIntegrals of Airy functions\n\nCalculates the integrals of Airy functions from 0 to `x`.\n\nParameters\n----------\n\nx: array_like\n    Upper limit of integration (float).\n\nReturns\n-------\nApt\n    Integral of Ai(t) from 0 to x.\nBpt\n    Integral of Bi(t) from 0 to x.\nAnt\n    Integral of Ai(-t) from 0 to x.\nBnt\n    Integral of Bi(-t) from 0 to x.\n\nNotes\n-----\n\nWrapper for a Fortran routine created by Shanjie Zhang and Jianming\nJin [1]_.\n\nReferences\n----------\n\n.. [1] Zhang, Shanjie and Jin, Jianming. \"Computation of Special\n       Functions\", John Wiley and Sons, 1996.\n       https://people.sc.fsu.edu/~jburkardt/f_src/special_functions/special_functions.html");

  /* "scipy/special/_ufuncs.pyx":8545
 *     "       Functions\", John Wiley and Sons, 1996.\n"
 *     "       https://people.sc.fsu.edu/~jburkardt/f_src/special_functions/special_functions.html")
 * ufunc_itairy_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_f_ffff             # <<<<<<<<<<<<<<
 * ufunc_itairy_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd
 * ufunc_itairy_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dddd_As_f_ffff);

  /* "scipy/special/_ufuncs.pyx":8546
 *     "       https://people.sc.fsu.edu/~jburkardt/f_src/special_functions/special_functions.html")
 * ufunc_itairy_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_f_ffff
 * ufunc_itairy_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd             # <<<<<<<<<<<<<<
 * ufunc_itairy_types[0] = <char>NPY_FLOAT
 * ufunc_itairy_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dddd_As_d_dddd);

  /* "scipy/special/_ufuncs.pyx":8547
 * ufunc_itairy_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_f_ffff
 * ufunc_itairy_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd
 * ufunc_itairy_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itairy_types[1] = <char>NPY_FLOAT
 * ufunc_itairy_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8548
 * ufunc_itairy_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dddd_As_d_dddd
 * ufunc_itairy_types[0] = <char>NPY_FLOAT
 * ufunc_itairy_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itairy_types[2] = <char>NPY_FLOAT
 * ufunc_itairy_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8549
 * ufunc_itairy_types[0] = <char>NPY_FLOAT
 * ufunc_itairy_types[1] = <char>NPY_FLOAT
 * ufunc_itairy_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itairy_types[3] = <char>NPY_FLOAT
 * ufunc_itairy_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8550
 * ufunc_itairy_types[1] = <char>NPY_FLOAT
 * ufunc_itairy_types[2] = <char>NPY_FLOAT
 * ufunc_itairy_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itairy_types[4] = <char>NPY_FLOAT
 * ufunc_itairy_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8551
 * ufunc_itairy_types[2] = <char>NPY_FLOAT
 * ufunc_itairy_types[3] = <char>NPY_FLOAT
 * ufunc_itairy_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itairy_types[5] = <char>NPY_DOUBLE
 * ufunc_itairy_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8552
 * ufunc_itairy_types[3] = <char>NPY_FLOAT
 * ufunc_itairy_types[4] = <char>NPY_FLOAT
 * ufunc_itairy_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itairy_types[6] = <char>NPY_DOUBLE
 * ufunc_itairy_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8553
 * ufunc_itairy_types[4] = <char>NPY_FLOAT
 * ufunc_itairy_types[5] = <char>NPY_DOUBLE
 * ufunc_itairy_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itairy_types[7] = <char>NPY_DOUBLE
 * ufunc_itairy_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8554
 * ufunc_itairy_types[5] = <char>NPY_DOUBLE
 * ufunc_itairy_types[6] = <char>NPY_DOUBLE
 * ufunc_itairy_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itairy_types[8] = <char>NPY_DOUBLE
 * ufunc_itairy_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8555
 * ufunc_itairy_types[6] = <char>NPY_DOUBLE
 * ufunc_itairy_types[7] = <char>NPY_DOUBLE
 * ufunc_itairy_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itairy_types[9] = <char>NPY_DOUBLE
 * ufunc_itairy_ptr[2*0] = <void*>_func_itairy_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8556
 * ufunc_itairy_types[7] = <char>NPY_DOUBLE
 * ufunc_itairy_types[8] = <char>NPY_DOUBLE
 * ufunc_itairy_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itairy_ptr[2*0] = <void*>_func_itairy_wrap
 * ufunc_itairy_ptr[2*0+1] = <void*>(<char*>"itairy")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8557
 * ufunc_itairy_types[8] = <char>NPY_DOUBLE
 * ufunc_itairy_types[9] = <char>NPY_DOUBLE
 * ufunc_itairy_ptr[2*0] = <void*>_func_itairy_wrap             # <<<<<<<<<<<<<<
 * ufunc_itairy_ptr[2*0+1] = <void*>(<char*>"itairy")
 * ufunc_itairy_ptr[2*1] = <void*>_func_itairy_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_ptr[0]) = ((void *)itairy_wrap);

  /* "scipy/special/_ufuncs.pyx":8558
 * ufunc_itairy_types[9] = <char>NPY_DOUBLE
 * ufunc_itairy_ptr[2*0] = <void*>_func_itairy_wrap
 * ufunc_itairy_ptr[2*0+1] = <void*>(<char*>"itairy")             # <<<<<<<<<<<<<<
 * ufunc_itairy_ptr[2*1] = <void*>_func_itairy_wrap
 * ufunc_itairy_ptr[2*1+1] = <void*>(<char*>"itairy")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_ptr[1]) = ((void *)((char *)((char *)"itairy")));

  /* "scipy/special/_ufuncs.pyx":8559
 * ufunc_itairy_ptr[2*0] = <void*>_func_itairy_wrap
 * ufunc_itairy_ptr[2*0+1] = <void*>(<char*>"itairy")
 * ufunc_itairy_ptr[2*1] = <void*>_func_itairy_wrap             # <<<<<<<<<<<<<<
 * ufunc_itairy_ptr[2*1+1] = <void*>(<char*>"itairy")
 * ufunc_itairy_data[0] = &ufunc_itairy_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_ptr[2]) = ((void *)itairy_wrap);

  /* "scipy/special/_ufuncs.pyx":8560
 * ufunc_itairy_ptr[2*0+1] = <void*>(<char*>"itairy")
 * ufunc_itairy_ptr[2*1] = <void*>_func_itairy_wrap
 * ufunc_itairy_ptr[2*1+1] = <void*>(<char*>"itairy")             # <<<<<<<<<<<<<<
 * ufunc_itairy_data[0] = &ufunc_itairy_ptr[2*0]
 * ufunc_itairy_data[1] = &ufunc_itairy_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_ptr[3]) = ((void *)((char *)((char *)"itairy")));

  /* "scipy/special/_ufuncs.pyx":8561
 * ufunc_itairy_ptr[2*1] = <void*>_func_itairy_wrap
 * ufunc_itairy_ptr[2*1+1] = <void*>(<char*>"itairy")
 * ufunc_itairy_data[0] = &ufunc_itairy_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_itairy_data[1] = &ufunc_itairy_ptr[2*1]
 * itairy = np.PyUFunc_FromFuncAndData(ufunc_itairy_loops, ufunc_itairy_data, ufunc_itairy_types, 2, 1, 4, 0, "itairy", ufunc_itairy_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8562
 * ufunc_itairy_ptr[2*1+1] = <void*>(<char*>"itairy")
 * ufunc_itairy_data[0] = &ufunc_itairy_ptr[2*0]
 * ufunc_itairy_data[1] = &ufunc_itairy_ptr[2*1]             # <<<<<<<<<<<<<<
 * itairy = np.PyUFunc_FromFuncAndData(ufunc_itairy_loops, ufunc_itairy_data, ufunc_itairy_types, 2, 1, 4, 0, "itairy", ufunc_itairy_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8563
 * ufunc_itairy_data[0] = &ufunc_itairy_ptr[2*0]
 * ufunc_itairy_data[1] = &ufunc_itairy_ptr[2*1]
 * itairy = np.PyUFunc_FromFuncAndData(ufunc_itairy_loops, ufunc_itairy_data, ufunc_itairy_types, 2, 1, 4, 0, "itairy", ufunc_itairy_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_iti0k0_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_types, 2, 1, 4, 0, ((char *)"itairy"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_itairy_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8563, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_itairy, __pyx_t_2) < 0) __PYX_ERR(1, 8563, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":8569
 * cdef void *ufunc_iti0k0_data[2]
 * cdef char ufunc_iti0k0_types[6]
 * cdef char *ufunc_iti0k0_doc = (             # <<<<<<<<<<<<<<
 *     "iti0k0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_doc = ((char *)"iti0k0(x)\n\nIntegrals of modified Bessel functions of order 0\n\nReturns simple integrals from 0 to `x` of the zeroth order modified\nBessel functions `i0` and `k0`.\n\nReturns\n-------\nii0, ik0");

  /* "scipy/special/_ufuncs.pyx":8580
 *     "-------\n"
 *     "ii0, ik0")
 * ufunc_iti0k0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_iti0k0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_f_ff);

  /* "scipy/special/_ufuncs.pyx":8581
 *     "ii0, ik0")
 * ufunc_iti0k0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_iti0k0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_types[0] = <char>NPY_FLOAT
 * ufunc_iti0k0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_d_dd);

  /* "scipy/special/_ufuncs.pyx":8582
 * ufunc_iti0k0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_iti0k0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_iti0k0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_types[1] = <char>NPY_FLOAT
 * ufunc_iti0k0_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8583
 * ufunc_iti0k0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_iti0k0_types[0] = <char>NPY_FLOAT
 * ufunc_iti0k0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_types[2] = <char>NPY_FLOAT
 * ufunc_iti0k0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8584
 * ufunc_iti0k0_types[0] = <char>NPY_FLOAT
 * ufunc_iti0k0_types[1] = <char>NPY_FLOAT
 * ufunc_iti0k0_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_types[3] = <char>NPY_DOUBLE
 * ufunc_iti0k0_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8585
 * ufunc_iti0k0_types[1] = <char>NPY_FLOAT
 * ufunc_iti0k0_types[2] = <char>NPY_FLOAT
 * ufunc_iti0k0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_types[4] = <char>NPY_DOUBLE
 * ufunc_iti0k0_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8586
 * ufunc_iti0k0_types[2] = <char>NPY_FLOAT
 * ufunc_iti0k0_types[3] = <char>NPY_DOUBLE
 * ufunc_iti0k0_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_types[5] = <char>NPY_DOUBLE
 * ufunc_iti0k0_ptr[2*0] = <void*>_func_it1i0k0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8587
 * ufunc_iti0k0_types[3] = <char>NPY_DOUBLE
 * ufunc_iti0k0_types[4] = <char>NPY_DOUBLE
 * ufunc_iti0k0_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_ptr[2*0] = <void*>_func_it1i0k0_wrap
 * ufunc_iti0k0_ptr[2*0+1] = <void*>(<char*>"iti0k0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8588
 * ufunc_iti0k0_types[4] = <char>NPY_DOUBLE
 * ufunc_iti0k0_types[5] = <char>NPY_DOUBLE
 * ufunc_iti0k0_ptr[2*0] = <void*>_func_it1i0k0_wrap             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_ptr[2*0+1] = <void*>(<char*>"iti0k0")
 * ufunc_iti0k0_ptr[2*1] = <void*>_func_it1i0k0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_ptr[0]) = ((void *)it1i0k0_wrap);

  /* "scipy/special/_ufuncs.pyx":8589
 * ufunc_iti0k0_types[5] = <char>NPY_DOUBLE
 * ufunc_iti0k0_ptr[2*0] = <void*>_func_it1i0k0_wrap
 * ufunc_iti0k0_ptr[2*0+1] = <void*>(<char*>"iti0k0")             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_ptr[2*1] = <void*>_func_it1i0k0_wrap
 * ufunc_iti0k0_ptr[2*1+1] = <void*>(<char*>"iti0k0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_ptr[1]) = ((void *)((char *)((char *)"iti0k0")));

  /* "scipy/special/_ufuncs.pyx":8590
 * ufunc_iti0k0_ptr[2*0] = <void*>_func_it1i0k0_wrap
 * ufunc_iti0k0_ptr[2*0+1] = <void*>(<char*>"iti0k0")
 * ufunc_iti0k0_ptr[2*1] = <void*>_func_it1i0k0_wrap             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_ptr[2*1+1] = <void*>(<char*>"iti0k0")
 * ufunc_iti0k0_data[0] = &ufunc_iti0k0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_ptr[2]) = ((void *)it1i0k0_wrap);

  /* "scipy/special/_ufuncs.pyx":8591
 * ufunc_iti0k0_ptr[2*0+1] = <void*>(<char*>"iti0k0")
 * ufunc_iti0k0_ptr[2*1] = <void*>_func_it1i0k0_wrap
 * ufunc_iti0k0_ptr[2*1+1] = <void*>(<char*>"iti0k0")             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_data[0] = &ufunc_iti0k0_ptr[2*0]
 * ufunc_iti0k0_data[1] = &ufunc_iti0k0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_ptr[3]) = ((void *)((char *)((char *)"iti0k0")));

  /* "scipy/special/_ufuncs.pyx":8592
 * ufunc_iti0k0_ptr[2*1] = <void*>_func_it1i0k0_wrap
 * ufunc_iti0k0_ptr[2*1+1] = <void*>(<char*>"iti0k0")
 * ufunc_iti0k0_data[0] = &ufunc_iti0k0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_iti0k0_data[1] = &ufunc_iti0k0_ptr[2*1]
 * iti0k0 = np.PyUFunc_FromFuncAndData(ufunc_iti0k0_loops, ufunc_iti0k0_data, ufunc_iti0k0_types, 2, 1, 2, 0, "iti0k0", ufunc_iti0k0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8593
 * ufunc_iti0k0_ptr[2*1+1] = <void*>(<char*>"iti0k0")
 * ufunc_iti0k0_data[0] = &ufunc_iti0k0_ptr[2*0]
 * ufunc_iti0k0_data[1] = &ufunc_iti0k0_ptr[2*1]             # <<<<<<<<<<<<<<
 * iti0k0 = np.PyUFunc_FromFuncAndData(ufunc_iti0k0_loops, ufunc_iti0k0_data, ufunc_iti0k0_types, 2, 1, 2, 0, "iti0k0", ufunc_iti0k0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8594
 * ufunc_iti0k0_data[0] = &ufunc_iti0k0_ptr[2*0]
 * ufunc_iti0k0_data[1] = &ufunc_iti0k0_ptr[2*1]
 * iti0k0 = np.PyUFunc_FromFuncAndData(ufunc_iti0k0_loops, ufunc_iti0k0_data, ufunc_iti0k0_types, 2, 1, 2, 0, "iti0k0", ufunc_iti0k0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_itj0y0_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_types, 2, 1, 2, 0, ((char *)"iti0k0"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_iti0k0_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8594, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_iti0k0, __pyx_t_2) < 0) __PYX_ERR(1, 8594, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":8600
 * cdef void *ufunc_itj0y0_data[2]
 * cdef char ufunc_itj0y0_types[6]
 * cdef char *ufunc_itj0y0_doc = (             # <<<<<<<<<<<<<<
 *     "itj0y0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_doc = ((char *)"itj0y0(x)\n\nIntegrals of Bessel functions of order 0\n\nReturns simple integrals from 0 to `x` of the zeroth order Bessel\nfunctions `j0` and `y0`.\n\nReturns\n-------\nij0, iy0");

  /* "scipy/special/_ufuncs.pyx":8611
 *     "-------\n"
 *     "ij0, iy0")
 * ufunc_itj0y0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_itj0y0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_f_ff);

  /* "scipy/special/_ufuncs.pyx":8612
 *     "ij0, iy0")
 * ufunc_itj0y0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_itj0y0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_types[0] = <char>NPY_FLOAT
 * ufunc_itj0y0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_d_dd);

  /* "scipy/special/_ufuncs.pyx":8613
 * ufunc_itj0y0_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_itj0y0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_itj0y0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_types[1] = <char>NPY_FLOAT
 * ufunc_itj0y0_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8614
 * ufunc_itj0y0_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_itj0y0_types[0] = <char>NPY_FLOAT
 * ufunc_itj0y0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_types[2] = <char>NPY_FLOAT
 * ufunc_itj0y0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8615
 * ufunc_itj0y0_types[0] = <char>NPY_FLOAT
 * ufunc_itj0y0_types[1] = <char>NPY_FLOAT
 * ufunc_itj0y0_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_types[3] = <char>NPY_DOUBLE
 * ufunc_itj0y0_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8616
 * ufunc_itj0y0_types[1] = <char>NPY_FLOAT
 * ufunc_itj0y0_types[2] = <char>NPY_FLOAT
 * ufunc_itj0y0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_types[4] = <char>NPY_DOUBLE
 * ufunc_itj0y0_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8617
 * ufunc_itj0y0_types[2] = <char>NPY_FLOAT
 * ufunc_itj0y0_types[3] = <char>NPY_DOUBLE
 * ufunc_itj0y0_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_types[5] = <char>NPY_DOUBLE
 * ufunc_itj0y0_ptr[2*0] = <void*>_func_it1j0y0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8618
 * ufunc_itj0y0_types[3] = <char>NPY_DOUBLE
 * ufunc_itj0y0_types[4] = <char>NPY_DOUBLE
 * ufunc_itj0y0_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_ptr[2*0] = <void*>_func_it1j0y0_wrap
 * ufunc_itj0y0_ptr[2*0+1] = <void*>(<char*>"itj0y0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8619
 * ufunc_itj0y0_types[4] = <char>NPY_DOUBLE
 * ufunc_itj0y0_types[5] = <char>NPY_DOUBLE
 * ufunc_itj0y0_ptr[2*0] = <void*>_func_it1j0y0_wrap             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_ptr[2*0+1] = <void*>(<char*>"itj0y0")
 * ufunc_itj0y0_ptr[2*1] = <void*>_func_it1j0y0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_ptr[0]) = ((void *)it1j0y0_wrap);

  /* "scipy/special/_ufuncs.pyx":8620
 * ufunc_itj0y0_types[5] = <char>NPY_DOUBLE
 * ufunc_itj0y0_ptr[2*0] = <void*>_func_it1j0y0_wrap
 * ufunc_itj0y0_ptr[2*0+1] = <void*>(<char*>"itj0y0")             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_ptr[2*1] = <void*>_func_it1j0y0_wrap
 * ufunc_itj0y0_ptr[2*1+1] = <void*>(<char*>"itj0y0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_ptr[1]) = ((void *)((char *)((char *)"itj0y0")));

  /* "scipy/special/_ufuncs.pyx":8621
 * ufunc_itj0y0_ptr[2*0] = <void*>_func_it1j0y0_wrap
 * ufunc_itj0y0_ptr[2*0+1] = <void*>(<char*>"itj0y0")
 * ufunc_itj0y0_ptr[2*1] = <void*>_func_it1j0y0_wrap             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_ptr[2*1+1] = <void*>(<char*>"itj0y0")
 * ufunc_itj0y0_data[0] = &ufunc_itj0y0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_ptr[2]) = ((void *)it1j0y0_wrap);

  /* "scipy/special/_ufuncs.pyx":8622
 * ufunc_itj0y0_ptr[2*0+1] = <void*>(<char*>"itj0y0")
 * ufunc_itj0y0_ptr[2*1] = <void*>_func_it1j0y0_wrap
 * ufunc_itj0y0_ptr[2*1+1] = <void*>(<char*>"itj0y0")             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_data[0] = &ufunc_itj0y0_ptr[2*0]
 * ufunc_itj0y0_data[1] = &ufunc_itj0y0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_ptr[3]) = ((void *)((char *)((char *)"itj0y0")));

  /* "scipy/special/_ufuncs.pyx":8623
 * ufunc_itj0y0_ptr[2*1] = <void*>_func_it1j0y0_wrap
 * ufunc_itj0y0_ptr[2*1+1] = <void*>(<char*>"itj0y0")
 * ufunc_itj0y0_data[0] = &ufunc_itj0y0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_itj0y0_data[1] = &ufunc_itj0y0_ptr[2*1]
 * itj0y0 = np.PyUFunc_FromFuncAndData(ufunc_itj0y0_loops, ufunc_itj0y0_data, ufunc_itj0y0_types, 2, 1, 2, 0, "itj0y0", ufunc_itj0y0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8624
 * ufunc_itj0y0_ptr[2*1+1] = <void*>(<char*>"itj0y0")
 * ufunc_itj0y0_data[0] = &ufunc_itj0y0_ptr[2*0]
 * ufunc_itj0y0_data[1] = &ufunc_itj0y0_ptr[2*1]             # <<<<<<<<<<<<<<
 * itj0y0 = np.PyUFunc_FromFuncAndData(ufunc_itj0y0_loops, ufunc_itj0y0_data, ufunc_itj0y0_types, 2, 1, 2, 0, "itj0y0", ufunc_itj0y0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8625
 * ufunc_itj0y0_data[0] = &ufunc_itj0y0_ptr[2*0]
 * ufunc_itj0y0_data[1] = &ufunc_itj0y0_ptr[2*1]
 * itj0y0 = np.PyUFunc_FromFuncAndData(ufunc_itj0y0_loops, ufunc_itj0y0_data, ufunc_itj0y0_types, 2, 1, 2, 0, "itj0y0", ufunc_itj0y0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_itmodstruve0_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_types, 2, 1, 2, 0, ((char *)"itj0y0"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_itj0y0_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8625, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_itj0y0, __pyx_t_2) < 0) __PYX_ERR(1, 8625, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":8631
 * cdef void *ufunc_itmodstruve0_data[2]
 * cdef char ufunc_itmodstruve0_types[4]
 * cdef char *ufunc_itmodstruve0_doc = (             # <<<<<<<<<<<<<<
 *     "itmodstruve0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_doc = ((char *)"itmodstruve0(x)\n\nIntegral of the modified Struve function of order 0.\n\n.. math::\n    I = \\int_0^x L_0(t)\\,dt\n\nParameters\n----------\nx : array_like\n    Upper limit of integration (float).\n\nReturns\n-------\nI : ndarray\n    The integral of :math:`L_0` from 0 to `x`.\n\nNotes\n-----\nWrapper for a Fortran routine created by Shanjie Zhang and Jianming\nJin [1]_.\n\nReferences\n----------\n.. [1] Zhang, Shanjie and Jin, Jianming. \"Computation of Special\n       Functions\", John Wiley and Sons, 1996.\n       https://people.sc.fsu.edu/~jburkardt/f_src/special_functions/special_functions.html");

  /* "scipy/special/_ufuncs.pyx":8659
 *     "       Functions\", John Wiley and Sons, 1996.\n"
 *     "       https://people.sc.fsu.edu/~jburkardt/f_src/special_functions/special_functions.html")
 * ufunc_itmodstruve0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_itmodstruve0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":8660
 *     "       https://people.sc.fsu.edu/~jburkardt/f_src/special_functions/special_functions.html")
 * ufunc_itmodstruve0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_itmodstruve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_types[0] = <char>NPY_FLOAT
 * ufunc_itmodstruve0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":8661
 * ufunc_itmodstruve0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_itmodstruve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_itmodstruve0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_types[1] = <char>NPY_FLOAT
 * ufunc_itmodstruve0_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8662
 * ufunc_itmodstruve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_itmodstruve0_types[0] = <char>NPY_FLOAT
 * ufunc_itmodstruve0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_types[2] = <char>NPY_DOUBLE
 * ufunc_itmodstruve0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8663
 * ufunc_itmodstruve0_types[0] = <char>NPY_FLOAT
 * ufunc_itmodstruve0_types[1] = <char>NPY_FLOAT
 * ufunc_itmodstruve0_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_types[3] = <char>NPY_DOUBLE
 * ufunc_itmodstruve0_ptr[2*0] = <void*>_func_itmodstruve0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8664
 * ufunc_itmodstruve0_types[1] = <char>NPY_FLOAT
 * ufunc_itmodstruve0_types[2] = <char>NPY_DOUBLE
 * ufunc_itmodstruve0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_ptr[2*0] = <void*>_func_itmodstruve0_wrap
 * ufunc_itmodstruve0_ptr[2*0+1] = <void*>(<char*>"itmodstruve0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8665
 * ufunc_itmodstruve0_types[2] = <char>NPY_DOUBLE
 * ufunc_itmodstruve0_types[3] = <char>NPY_DOUBLE
 * ufunc_itmodstruve0_ptr[2*0] = <void*>_func_itmodstruve0_wrap             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_ptr[2*0+1] = <void*>(<char*>"itmodstruve0")
 * ufunc_itmodstruve0_ptr[2*1] = <void*>_func_itmodstruve0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_ptr[0]) = ((void *)itmodstruve0_wrap);

  /* "scipy/special/_ufuncs.pyx":8666
 * ufunc_itmodstruve0_types[3] = <char>NPY_DOUBLE
 * ufunc_itmodstruve0_ptr[2*0] = <void*>_func_itmodstruve0_wrap
 * ufunc_itmodstruve0_ptr[2*0+1] = <void*>(<char*>"itmodstruve0")             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_ptr[2*1] = <void*>_func_itmodstruve0_wrap
 * ufunc_itmodstruve0_ptr[2*1+1] = <void*>(<char*>"itmodstruve0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_ptr[1]) = ((void *)((char *)((char *)"itmodstruve0")));

  /* "scipy/special/_ufuncs.pyx":8667
 * ufunc_itmodstruve0_ptr[2*0] = <void*>_func_itmodstruve0_wrap
 * ufunc_itmodstruve0_ptr[2*0+1] = <void*>(<char*>"itmodstruve0")
 * ufunc_itmodstruve0_ptr[2*1] = <void*>_func_itmodstruve0_wrap             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_ptr[2*1+1] = <void*>(<char*>"itmodstruve0")
 * ufunc_itmodstruve0_data[0] = &ufunc_itmodstruve0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_ptr[2]) = ((void *)itmodstruve0_wrap);

  /* "scipy/special/_ufuncs.pyx":8668
 * ufunc_itmodstruve0_ptr[2*0+1] = <void*>(<char*>"itmodstruve0")
 * ufunc_itmodstruve0_ptr[2*1] = <void*>_func_itmodstruve0_wrap
 * ufunc_itmodstruve0_ptr[2*1+1] = <void*>(<char*>"itmodstruve0")             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_data[0] = &ufunc_itmodstruve0_ptr[2*0]
 * ufunc_itmodstruve0_data[1] = &ufunc_itmodstruve0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_ptr[3]) = ((void *)((char *)((char *)"itmodstruve0")));

  /* "scipy/special/_ufuncs.pyx":8669
 * ufunc_itmodstruve0_ptr[2*1] = <void*>_func_itmodstruve0_wrap
 * ufunc_itmodstruve0_ptr[2*1+1] = <void*>(<char*>"itmodstruve0")
 * ufunc_itmodstruve0_data[0] = &ufunc_itmodstruve0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_itmodstruve0_data[1] = &ufunc_itmodstruve0_ptr[2*1]
 * itmodstruve0 = np.PyUFunc_FromFuncAndData(ufunc_itmodstruve0_loops, ufunc_itmodstruve0_data, ufunc_itmodstruve0_types, 2, 1, 1, 0, "itmodstruve0", ufunc_itmodstruve0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8670
 * ufunc_itmodstruve0_ptr[2*1+1] = <void*>(<char*>"itmodstruve0")
 * ufunc_itmodstruve0_data[0] = &ufunc_itmodstruve0_ptr[2*0]
 * ufunc_itmodstruve0_data[1] = &ufunc_itmodstruve0_ptr[2*1]             # <<<<<<<<<<<<<<
 * itmodstruve0 = np.PyUFunc_FromFuncAndData(ufunc_itmodstruve0_loops, ufunc_itmodstruve0_data, ufunc_itmodstruve0_types, 2, 1, 1, 0, "itmodstruve0", ufunc_itmodstruve0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8671
 * ufunc_itmodstruve0_data[0] = &ufunc_itmodstruve0_ptr[2*0]
 * ufunc_itmodstruve0_data[1] = &ufunc_itmodstruve0_ptr[2*1]
 * itmodstruve0 = np.PyUFunc_FromFuncAndData(ufunc_itmodstruve0_loops, ufunc_itmodstruve0_data, ufunc_itmodstruve0_types, 2, 1, 1, 0, "itmodstruve0", ufunc_itmodstruve0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_itstruve0_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_types, 2, 1, 1, 0, ((char *)"itmodstruve0"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_itmodstruve0_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8671, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_itmodstruve0, __pyx_t_2) < 0) __PYX_ERR(1, 8671, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":8677
 * cdef void *ufunc_itstruve0_data[2]
 * cdef char ufunc_itstruve0_types[4]
 * cdef char *ufunc_itstruve0_doc = (             # <<<<<<<<<<<<<<
 *     "itstruve0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_doc = ((char *)"itstruve0(x)\n\nIntegral of the Struve function of order 0.\n\n.. math::\n    I = \\int_0^x H_0(t)\\,dt\n\nParameters\n----------\nx : array_like\n    Upper limit of integration (float).\n\nReturns\n-------\nI : ndarray\n    The integral of :math:`H_0` from 0 to `x`.\n\nSee also\n--------\nstruve\n\nNotes\n-----\nWrapper for a Fortran routine created by Shanjie Zhang and Jianming\nJin [1]_.\n\nReferences\n----------\n.. [1] Zhang, Shanjie and Jin, Jianming. \"Computation of Special\n       Functions\", John Wiley and Sons, 1996.\n       https://people.sc.fsu.edu/~jburkardt/f_src/special_functions/special_functions.html");

  /* "scipy/special/_ufuncs.pyx":8709
 *     "       Functions\", John Wiley and Sons, 1996.\n"
 *     "       https://people.sc.fsu.edu/~jburkardt/f_src/special_functions/special_functions.html")
 * ufunc_itstruve0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_itstruve0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":8710
 *     "       https://people.sc.fsu.edu/~jburkardt/f_src/special_functions/special_functions.html")
 * ufunc_itstruve0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_itstruve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_types[0] = <char>NPY_FLOAT
 * ufunc_itstruve0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":8711
 * ufunc_itstruve0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_itstruve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_itstruve0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_types[1] = <char>NPY_FLOAT
 * ufunc_itstruve0_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8712
 * ufunc_itstruve0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_itstruve0_types[0] = <char>NPY_FLOAT
 * ufunc_itstruve0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_types[2] = <char>NPY_DOUBLE
 * ufunc_itstruve0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8713
 * ufunc_itstruve0_types[0] = <char>NPY_FLOAT
 * ufunc_itstruve0_types[1] = <char>NPY_FLOAT
 * ufunc_itstruve0_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_types[3] = <char>NPY_DOUBLE
 * ufunc_itstruve0_ptr[2*0] = <void*>_func_itstruve0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8714
 * ufunc_itstruve0_types[1] = <char>NPY_FLOAT
 * ufunc_itstruve0_types[2] = <char>NPY_DOUBLE
 * ufunc_itstruve0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_ptr[2*0] = <void*>_func_itstruve0_wrap
 * ufunc_itstruve0_ptr[2*0+1] = <void*>(<char*>"itstruve0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8715
 * ufunc_itstruve0_types[2] = <char>NPY_DOUBLE
 * ufunc_itstruve0_types[3] = <char>NPY_DOUBLE
 * ufunc_itstruve0_ptr[2*0] = <void*>_func_itstruve0_wrap             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_ptr[2*0+1] = <void*>(<char*>"itstruve0")
 * ufunc_itstruve0_ptr[2*1] = <void*>_func_itstruve0_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_ptr[0]) = ((void *)itstruve0_wrap);

  /* "scipy/special/_ufuncs.pyx":8716
 * ufunc_itstruve0_types[3] = <char>NPY_DOUBLE
 * ufunc_itstruve0_ptr[2*0] = <void*>_func_itstruve0_wrap
 * ufunc_itstruve0_ptr[2*0+1] = <void*>(<char*>"itstruve0")             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_ptr[2*1] = <void*>_func_itstruve0_wrap
 * ufunc_itstruve0_ptr[2*1+1] = <void*>(<char*>"itstruve0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_ptr[1]) = ((void *)((char *)((char *)"itstruve0")));

  /* "scipy/special/_ufuncs.pyx":8717
 * ufunc_itstruve0_ptr[2*0] = <void*>_func_itstruve0_wrap
 * ufunc_itstruve0_ptr[2*0+1] = <void*>(<char*>"itstruve0")
 * ufunc_itstruve0_ptr[2*1] = <void*>_func_itstruve0_wrap             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_ptr[2*1+1] = <void*>(<char*>"itstruve0")
 * ufunc_itstruve0_data[0] = &ufunc_itstruve0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_ptr[2]) = ((void *)itstruve0_wrap);

  /* "scipy/special/_ufuncs.pyx":8718
 * ufunc_itstruve0_ptr[2*0+1] = <void*>(<char*>"itstruve0")
 * ufunc_itstruve0_ptr[2*1] = <void*>_func_itstruve0_wrap
 * ufunc_itstruve0_ptr[2*1+1] = <void*>(<char*>"itstruve0")             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_data[0] = &ufunc_itstruve0_ptr[2*0]
 * ufunc_itstruve0_data[1] = &ufunc_itstruve0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_ptr[3]) = ((void *)((char *)((char *)"itstruve0")));

  /* "scipy/special/_ufuncs.pyx":8719
 * ufunc_itstruve0_ptr[2*1] = <void*>_func_itstruve0_wrap
 * ufunc_itstruve0_ptr[2*1+1] = <void*>(<char*>"itstruve0")
 * ufunc_itstruve0_data[0] = &ufunc_itstruve0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_itstruve0_data[1] = &ufunc_itstruve0_ptr[2*1]
 * itstruve0 = np.PyUFunc_FromFuncAndData(ufunc_itstruve0_loops, ufunc_itstruve0_data, ufunc_itstruve0_types, 2, 1, 1, 0, "itstruve0", ufunc_itstruve0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8720
 * ufunc_itstruve0_ptr[2*1+1] = <void*>(<char*>"itstruve0")
 * ufunc_itstruve0_data[0] = &ufunc_itstruve0_ptr[2*0]
 * ufunc_itstruve0_data[1] = &ufunc_itstruve0_ptr[2*1]             # <<<<<<<<<<<<<<
 * itstruve0 = np.PyUFunc_FromFuncAndData(ufunc_itstruve0_loops, ufunc_itstruve0_data, ufunc_itstruve0_types, 2, 1, 1, 0, "itstruve0", ufunc_itstruve0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8721
 * ufunc_itstruve0_data[0] = &ufunc_itstruve0_ptr[2*0]
 * ufunc_itstruve0_data[1] = &ufunc_itstruve0_ptr[2*1]
 * itstruve0 = np.PyUFunc_FromFuncAndData(ufunc_itstruve0_loops, ufunc_itstruve0_data, ufunc_itstruve0_types, 2, 1, 1, 0, "itstruve0", ufunc_itstruve0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_iv_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_types, 2, 1, 1, 0, ((char *)"itstruve0"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_itstruve0_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8721, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_itstruve0, __pyx_t_2) < 0) __PYX_ERR(1, 8721, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":8727
 * cdef void *ufunc_iv_data[4]
 * cdef char ufunc_iv_types[12]
 * cdef char *ufunc_iv_doc = (             # <<<<<<<<<<<<<<
 *     "iv(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_doc = ((char *)"iv(v, z)\n\nModified Bessel function of the first kind of real order.\n\nParameters\n----------\nv : array_like\n    Order. If `z` is of real type and negative, `v` must be integer\n    valued.\nz : array_like of float or complex\n    Argument.\n\nReturns\n-------\nout : ndarray\n    Values of the modified Bessel function.\n\nNotes\n-----\nFor real `z` and :math:`v \\in [-50, 50]`, the evaluation is carried out\nusing Temme's method [1]_.  For larger orders, uniform asymptotic\nexpansions are applied.\n\nFor complex `z` and positive `v`, the AMOS [2]_ `zbesi` routine is\ncalled. It uses a power series for small `z`, the asymptotic expansion\nfor large `abs(z)`, the Miller algorithm normalized by the Wronskian\nand a Neumann series for intermediate magnitudes, and the uniform\nasymptotic expansions for :math:`I_v(z)` and :math:`J_v(z)` for large\norders.  Backward recurrence is used to generate sequences or reduce\norders when necessary.\n\nThe calculations above are done in the right half plane and continued\ninto the left half plane by the formula,\n\n.. math:: I_v(z \\exp(\\pm\\imath\\pi)) = \\exp(\\pm\\pi v) I_v(z)\n\n(valid when the real part of `z` is positive).  For negative `v`, the\nformula\n\n.. math:: I_{-v}(z) = I_v(z) + \\frac{2}{\\pi} \\sin(\\pi v) K_v(z)\n\nis used, where :math:`K_v(z)` is the modified Bessel function of the\nsecond kind, evaluated using the AMOS routine `zbesk`.\n\nSee also\n--------\nkve : This function with leading exponential behavior stripped off.\n\nReferences\n----------\n.. [1] Temme, Journal of Computational Physics, vol 21, 343 (1976)\n.. [2] Donald E. Amos, \"AMOS, A Portable Package for Bessel Functions\n       of a Complex Argument and Nonnegative Order\",\n       http://netlib.org/amos/");

  /* "scipy/special/_ufuncs.pyx":8782
 *     "       of a Complex Argument and Nonnegative Order\",\n"
 *     "       http://netlib.org/amos/")
 * ufunc_iv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_iv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_iv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":8783
 *     "       http://netlib.org/amos/")
 * ufunc_iv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_iv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_iv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_iv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":8784
 * ufunc_iv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_iv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_iv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_iv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_iv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":8785
 * ufunc_iv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_iv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_iv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_iv_types[0] = <char>NPY_FLOAT
 * ufunc_iv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":8786
 * ufunc_iv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_iv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_iv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_iv_types[1] = <char>NPY_FLOAT
 * ufunc_iv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8787
 * ufunc_iv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_iv_types[0] = <char>NPY_FLOAT
 * ufunc_iv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_iv_types[2] = <char>NPY_FLOAT
 * ufunc_iv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8788
 * ufunc_iv_types[0] = <char>NPY_FLOAT
 * ufunc_iv_types[1] = <char>NPY_FLOAT
 * ufunc_iv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_iv_types[3] = <char>NPY_FLOAT
 * ufunc_iv_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8789
 * ufunc_iv_types[1] = <char>NPY_FLOAT
 * ufunc_iv_types[2] = <char>NPY_FLOAT
 * ufunc_iv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_iv_types[4] = <char>NPY_CFLOAT
 * ufunc_iv_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8790
 * ufunc_iv_types[2] = <char>NPY_FLOAT
 * ufunc_iv_types[3] = <char>NPY_FLOAT
 * ufunc_iv_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_iv_types[5] = <char>NPY_CFLOAT
 * ufunc_iv_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":8791
 * ufunc_iv_types[3] = <char>NPY_FLOAT
 * ufunc_iv_types[4] = <char>NPY_CFLOAT
 * ufunc_iv_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_iv_types[6] = <char>NPY_DOUBLE
 * ufunc_iv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":8792
 * ufunc_iv_types[4] = <char>NPY_CFLOAT
 * ufunc_iv_types[5] = <char>NPY_CFLOAT
 * ufunc_iv_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_iv_types[7] = <char>NPY_DOUBLE
 * ufunc_iv_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8793
 * ufunc_iv_types[5] = <char>NPY_CFLOAT
 * ufunc_iv_types[6] = <char>NPY_DOUBLE
 * ufunc_iv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_iv_types[8] = <char>NPY_DOUBLE
 * ufunc_iv_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8794
 * ufunc_iv_types[6] = <char>NPY_DOUBLE
 * ufunc_iv_types[7] = <char>NPY_DOUBLE
 * ufunc_iv_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_iv_types[9] = <char>NPY_DOUBLE
 * ufunc_iv_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8795
 * ufunc_iv_types[7] = <char>NPY_DOUBLE
 * ufunc_iv_types[8] = <char>NPY_DOUBLE
 * ufunc_iv_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_iv_types[10] = <char>NPY_CDOUBLE
 * ufunc_iv_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8796
 * ufunc_iv_types[8] = <char>NPY_DOUBLE
 * ufunc_iv_types[9] = <char>NPY_DOUBLE
 * ufunc_iv_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_iv_types[11] = <char>NPY_CDOUBLE
 * ufunc_iv_ptr[2*0] = <void*>_func_iv
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":8797
 * ufunc_iv_types[9] = <char>NPY_DOUBLE
 * ufunc_iv_types[10] = <char>NPY_CDOUBLE
 * ufunc_iv_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_iv_ptr[2*0] = <void*>_func_iv
 * ufunc_iv_ptr[2*0+1] = <void*>(<char*>"iv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":8798
 * ufunc_iv_types[10] = <char>NPY_CDOUBLE
 * ufunc_iv_types[11] = <char>NPY_CDOUBLE
 * ufunc_iv_ptr[2*0] = <void*>_func_iv             # <<<<<<<<<<<<<<
 * ufunc_iv_ptr[2*0+1] = <void*>(<char*>"iv")
 * ufunc_iv_ptr[2*1] = <void*>_func_cbesi_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[0]) = ((void *)iv);

  /* "scipy/special/_ufuncs.pyx":8799
 * ufunc_iv_types[11] = <char>NPY_CDOUBLE
 * ufunc_iv_ptr[2*0] = <void*>_func_iv
 * ufunc_iv_ptr[2*0+1] = <void*>(<char*>"iv")             # <<<<<<<<<<<<<<
 * ufunc_iv_ptr[2*1] = <void*>_func_cbesi_wrap
 * ufunc_iv_ptr[2*1+1] = <void*>(<char*>"iv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[1]) = ((void *)((char *)((char *)"iv")));

  /* "scipy/special/_ufuncs.pyx":8800
 * ufunc_iv_ptr[2*0] = <void*>_func_iv
 * ufunc_iv_ptr[2*0+1] = <void*>(<char*>"iv")
 * ufunc_iv_ptr[2*1] = <void*>_func_cbesi_wrap             # <<<<<<<<<<<<<<
 * ufunc_iv_ptr[2*1+1] = <void*>(<char*>"iv")
 * ufunc_iv_ptr[2*2] = <void*>_func_iv
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[2]) = ((void *)cbesi_wrap);

  /* "scipy/special/_ufuncs.pyx":8801
 * ufunc_iv_ptr[2*0+1] = <void*>(<char*>"iv")
 * ufunc_iv_ptr[2*1] = <void*>_func_cbesi_wrap
 * ufunc_iv_ptr[2*1+1] = <void*>(<char*>"iv")             # <<<<<<<<<<<<<<
 * ufunc_iv_ptr[2*2] = <void*>_func_iv
 * ufunc_iv_ptr[2*2+1] = <void*>(<char*>"iv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[3]) = ((void *)((char *)((char *)"iv")));

  /* "scipy/special/_ufuncs.pyx":8802
 * ufunc_iv_ptr[2*1] = <void*>_func_cbesi_wrap
 * ufunc_iv_ptr[2*1+1] = <void*>(<char*>"iv")
 * ufunc_iv_ptr[2*2] = <void*>_func_iv             # <<<<<<<<<<<<<<
 * ufunc_iv_ptr[2*2+1] = <void*>(<char*>"iv")
 * ufunc_iv_ptr[2*3] = <void*>_func_cbesi_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[4]) = ((void *)iv);

  /* "scipy/special/_ufuncs.pyx":8803
 * ufunc_iv_ptr[2*1+1] = <void*>(<char*>"iv")
 * ufunc_iv_ptr[2*2] = <void*>_func_iv
 * ufunc_iv_ptr[2*2+1] = <void*>(<char*>"iv")             # <<<<<<<<<<<<<<
 * ufunc_iv_ptr[2*3] = <void*>_func_cbesi_wrap
 * ufunc_iv_ptr[2*3+1] = <void*>(<char*>"iv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[5]) = ((void *)((char *)((char *)"iv")));

  /* "scipy/special/_ufuncs.pyx":8804
 * ufunc_iv_ptr[2*2] = <void*>_func_iv
 * ufunc_iv_ptr[2*2+1] = <void*>(<char*>"iv")
 * ufunc_iv_ptr[2*3] = <void*>_func_cbesi_wrap             # <<<<<<<<<<<<<<
 * ufunc_iv_ptr[2*3+1] = <void*>(<char*>"iv")
 * ufunc_iv_data[0] = &ufunc_iv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[6]) = ((void *)cbesi_wrap);

  /* "scipy/special/_ufuncs.pyx":8805
 * ufunc_iv_ptr[2*2+1] = <void*>(<char*>"iv")
 * ufunc_iv_ptr[2*3] = <void*>_func_cbesi_wrap
 * ufunc_iv_ptr[2*3+1] = <void*>(<char*>"iv")             # <<<<<<<<<<<<<<
 * ufunc_iv_data[0] = &ufunc_iv_ptr[2*0]
 * ufunc_iv_data[1] = &ufunc_iv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[7]) = ((void *)((char *)((char *)"iv")));

  /* "scipy/special/_ufuncs.pyx":8806
 * ufunc_iv_ptr[2*3] = <void*>_func_cbesi_wrap
 * ufunc_iv_ptr[2*3+1] = <void*>(<char*>"iv")
 * ufunc_iv_data[0] = &ufunc_iv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_iv_data[1] = &ufunc_iv_ptr[2*1]
 * ufunc_iv_data[2] = &ufunc_iv_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8807
 * ufunc_iv_ptr[2*3+1] = <void*>(<char*>"iv")
 * ufunc_iv_data[0] = &ufunc_iv_ptr[2*0]
 * ufunc_iv_data[1] = &ufunc_iv_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_iv_data[2] = &ufunc_iv_ptr[2*2]
 * ufunc_iv_data[3] = &ufunc_iv_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8808
 * ufunc_iv_data[0] = &ufunc_iv_ptr[2*0]
 * ufunc_iv_data[1] = &ufunc_iv_ptr[2*1]
 * ufunc_iv_data[2] = &ufunc_iv_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_iv_data[3] = &ufunc_iv_ptr[2*3]
 * iv = np.PyUFunc_FromFuncAndData(ufunc_iv_loops, ufunc_iv_data, ufunc_iv_types, 4, 2, 1, 0, "iv", ufunc_iv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":8809
 * ufunc_iv_data[1] = &ufunc_iv_ptr[2*1]
 * ufunc_iv_data[2] = &ufunc_iv_ptr[2*2]
 * ufunc_iv_data[3] = &ufunc_iv_ptr[2*3]             # <<<<<<<<<<<<<<
 * iv = np.PyUFunc_FromFuncAndData(ufunc_iv_loops, ufunc_iv_data, ufunc_iv_types, 4, 2, 1, 0, "iv", ufunc_iv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":8810
 * ufunc_iv_data[2] = &ufunc_iv_ptr[2*2]
 * ufunc_iv_data[3] = &ufunc_iv_ptr[2*3]
 * iv = np.PyUFunc_FromFuncAndData(ufunc_iv_loops, ufunc_iv_data, ufunc_iv_types, 4, 2, 1, 0, "iv", ufunc_iv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ive_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_types, 4, 2, 1, 0, ((char *)"iv"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_iv_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8810, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_iv, __pyx_t_2) < 0) __PYX_ERR(1, 8810, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":8816
 * cdef void *ufunc_ive_data[4]
 * cdef char ufunc_ive_types[12]
 * cdef char *ufunc_ive_doc = (             # <<<<<<<<<<<<<<
 *     "ive(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_doc = ((char *)"ive(v, z)\n\nExponentially scaled modified Bessel function of the first kind\n\nDefined as::\n\n    ive(v, z) = iv(v, z) * exp(-abs(z.real))\n\nParameters\n----------\nv : array_like of float\n    Order.\nz : array_like of float or complex\n    Argument.\n\nReturns\n-------\nout : ndarray\n    Values of the exponentially scaled modified Bessel function.\n\nNotes\n-----\nFor positive `v`, the AMOS [1]_ `zbesi` routine is called. It uses a\npower series for small `z`, the asymptotic expansion for large\n`abs(z)`, the Miller algorithm normalized by the Wronskian and a\nNeumann series for intermediate magnitudes, and the uniform asymptotic\nexpansions for :math:`I_v(z)` and :math:`J_v(z)` for large orders.\nBackward recurrence is used to generate sequences or reduce orders when\nnecessary.\n\nThe calculations above are done in the right half plane and continued\ninto the left half plane by the formula,\n\n.. math:: I_v(z \\exp(\\pm\\imath\\pi)) = \\exp(\\pm\\pi v) I_v(z)\n\n(valid when the real part of `z` is positive).  For negative `v`, the\nformula\n\n.. math:: I_{-v}(z) = I_v(z) + \\frac{2}{\\pi} \\sin(\\pi v) K_v(z)\n\nis used, where :math:`K_v(z)` is the modified Bessel function of the\nsecond kind, evaluated using the AMOS routine `zbesk`.\n\nReferences\n----------\n.. [1] Donald E. Amos, \"AMOS, A Portable Package for Bessel Functions\n       of a Complex Argument and Nonnegative Order\",\n       http://netlib.org/amos/");

  /* "scipy/special/_ufuncs.pyx":8865
 *     "       of a Complex Argument and Nonnegative Order\",\n"
 *     "       http://netlib.org/amos/")
 * ufunc_ive_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_ive_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_ive_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":8866
 *     "       http://netlib.org/amos/")
 * ufunc_ive_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_ive_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_ive_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_ive_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":8867
 * ufunc_ive_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_ive_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_ive_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_ive_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_ive_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":8868
 * ufunc_ive_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_ive_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_ive_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_ive_types[0] = <char>NPY_FLOAT
 * ufunc_ive_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":8869
 * ufunc_ive_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_ive_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_ive_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ive_types[1] = <char>NPY_FLOAT
 * ufunc_ive_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8870
 * ufunc_ive_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_ive_types[0] = <char>NPY_FLOAT
 * ufunc_ive_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ive_types[2] = <char>NPY_FLOAT
 * ufunc_ive_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8871
 * ufunc_ive_types[0] = <char>NPY_FLOAT
 * ufunc_ive_types[1] = <char>NPY_FLOAT
 * ufunc_ive_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ive_types[3] = <char>NPY_FLOAT
 * ufunc_ive_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8872
 * ufunc_ive_types[1] = <char>NPY_FLOAT
 * ufunc_ive_types[2] = <char>NPY_FLOAT
 * ufunc_ive_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ive_types[4] = <char>NPY_CFLOAT
 * ufunc_ive_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8873
 * ufunc_ive_types[2] = <char>NPY_FLOAT
 * ufunc_ive_types[3] = <char>NPY_FLOAT
 * ufunc_ive_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_ive_types[5] = <char>NPY_CFLOAT
 * ufunc_ive_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":8874
 * ufunc_ive_types[3] = <char>NPY_FLOAT
 * ufunc_ive_types[4] = <char>NPY_CFLOAT
 * ufunc_ive_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_ive_types[6] = <char>NPY_DOUBLE
 * ufunc_ive_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":8875
 * ufunc_ive_types[4] = <char>NPY_CFLOAT
 * ufunc_ive_types[5] = <char>NPY_CFLOAT
 * ufunc_ive_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ive_types[7] = <char>NPY_DOUBLE
 * ufunc_ive_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8876
 * ufunc_ive_types[5] = <char>NPY_CFLOAT
 * ufunc_ive_types[6] = <char>NPY_DOUBLE
 * ufunc_ive_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ive_types[8] = <char>NPY_DOUBLE
 * ufunc_ive_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8877
 * ufunc_ive_types[6] = <char>NPY_DOUBLE
 * ufunc_ive_types[7] = <char>NPY_DOUBLE
 * ufunc_ive_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ive_types[9] = <char>NPY_DOUBLE
 * ufunc_ive_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8878
 * ufunc_ive_types[7] = <char>NPY_DOUBLE
 * ufunc_ive_types[8] = <char>NPY_DOUBLE
 * ufunc_ive_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ive_types[10] = <char>NPY_CDOUBLE
 * ufunc_ive_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8879
 * ufunc_ive_types[8] = <char>NPY_DOUBLE
 * ufunc_ive_types[9] = <char>NPY_DOUBLE
 * ufunc_ive_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ive_types[11] = <char>NPY_CDOUBLE
 * ufunc_ive_ptr[2*0] = <void*>_func_cbesi_wrap_e_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":8880
 * ufunc_ive_types[9] = <char>NPY_DOUBLE
 * ufunc_ive_types[10] = <char>NPY_CDOUBLE
 * ufunc_ive_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ive_ptr[2*0] = <void*>_func_cbesi_wrap_e_real
 * ufunc_ive_ptr[2*0+1] = <void*>(<char*>"ive")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":8881
 * ufunc_ive_types[10] = <char>NPY_CDOUBLE
 * ufunc_ive_types[11] = <char>NPY_CDOUBLE
 * ufunc_ive_ptr[2*0] = <void*>_func_cbesi_wrap_e_real             # <<<<<<<<<<<<<<
 * ufunc_ive_ptr[2*0+1] = <void*>(<char*>"ive")
 * ufunc_ive_ptr[2*1] = <void*>_func_cbesi_wrap_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[0]) = ((void *)cbesi_wrap_e_real);

  /* "scipy/special/_ufuncs.pyx":8882
 * ufunc_ive_types[11] = <char>NPY_CDOUBLE
 * ufunc_ive_ptr[2*0] = <void*>_func_cbesi_wrap_e_real
 * ufunc_ive_ptr[2*0+1] = <void*>(<char*>"ive")             # <<<<<<<<<<<<<<
 * ufunc_ive_ptr[2*1] = <void*>_func_cbesi_wrap_e
 * ufunc_ive_ptr[2*1+1] = <void*>(<char*>"ive")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[1]) = ((void *)((char *)((char *)"ive")));

  /* "scipy/special/_ufuncs.pyx":8883
 * ufunc_ive_ptr[2*0] = <void*>_func_cbesi_wrap_e_real
 * ufunc_ive_ptr[2*0+1] = <void*>(<char*>"ive")
 * ufunc_ive_ptr[2*1] = <void*>_func_cbesi_wrap_e             # <<<<<<<<<<<<<<
 * ufunc_ive_ptr[2*1+1] = <void*>(<char*>"ive")
 * ufunc_ive_ptr[2*2] = <void*>_func_cbesi_wrap_e_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[2]) = ((void *)cbesi_wrap_e);

  /* "scipy/special/_ufuncs.pyx":8884
 * ufunc_ive_ptr[2*0+1] = <void*>(<char*>"ive")
 * ufunc_ive_ptr[2*1] = <void*>_func_cbesi_wrap_e
 * ufunc_ive_ptr[2*1+1] = <void*>(<char*>"ive")             # <<<<<<<<<<<<<<
 * ufunc_ive_ptr[2*2] = <void*>_func_cbesi_wrap_e_real
 * ufunc_ive_ptr[2*2+1] = <void*>(<char*>"ive")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[3]) = ((void *)((char *)((char *)"ive")));

  /* "scipy/special/_ufuncs.pyx":8885
 * ufunc_ive_ptr[2*1] = <void*>_func_cbesi_wrap_e
 * ufunc_ive_ptr[2*1+1] = <void*>(<char*>"ive")
 * ufunc_ive_ptr[2*2] = <void*>_func_cbesi_wrap_e_real             # <<<<<<<<<<<<<<
 * ufunc_ive_ptr[2*2+1] = <void*>(<char*>"ive")
 * ufunc_ive_ptr[2*3] = <void*>_func_cbesi_wrap_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[4]) = ((void *)cbesi_wrap_e_real);

  /* "scipy/special/_ufuncs.pyx":8886
 * ufunc_ive_ptr[2*1+1] = <void*>(<char*>"ive")
 * ufunc_ive_ptr[2*2] = <void*>_func_cbesi_wrap_e_real
 * ufunc_ive_ptr[2*2+1] = <void*>(<char*>"ive")             # <<<<<<<<<<<<<<
 * ufunc_ive_ptr[2*3] = <void*>_func_cbesi_wrap_e
 * ufunc_ive_ptr[2*3+1] = <void*>(<char*>"ive")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[5]) = ((void *)((char *)((char *)"ive")));

  /* "scipy/special/_ufuncs.pyx":8887
 * ufunc_ive_ptr[2*2] = <void*>_func_cbesi_wrap_e_real
 * ufunc_ive_ptr[2*2+1] = <void*>(<char*>"ive")
 * ufunc_ive_ptr[2*3] = <void*>_func_cbesi_wrap_e             # <<<<<<<<<<<<<<
 * ufunc_ive_ptr[2*3+1] = <void*>(<char*>"ive")
 * ufunc_ive_data[0] = &ufunc_ive_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[6]) = ((void *)cbesi_wrap_e);

  /* "scipy/special/_ufuncs.pyx":8888
 * ufunc_ive_ptr[2*2+1] = <void*>(<char*>"ive")
 * ufunc_ive_ptr[2*3] = <void*>_func_cbesi_wrap_e
 * ufunc_ive_ptr[2*3+1] = <void*>(<char*>"ive")             # <<<<<<<<<<<<<<
 * ufunc_ive_data[0] = &ufunc_ive_ptr[2*0]
 * ufunc_ive_data[1] = &ufunc_ive_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[7]) = ((void *)((char *)((char *)"ive")));

  /* "scipy/special/_ufuncs.pyx":8889
 * ufunc_ive_ptr[2*3] = <void*>_func_cbesi_wrap_e
 * ufunc_ive_ptr[2*3+1] = <void*>(<char*>"ive")
 * ufunc_ive_data[0] = &ufunc_ive_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ive_data[1] = &ufunc_ive_ptr[2*1]
 * ufunc_ive_data[2] = &ufunc_ive_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8890
 * ufunc_ive_ptr[2*3+1] = <void*>(<char*>"ive")
 * ufunc_ive_data[0] = &ufunc_ive_ptr[2*0]
 * ufunc_ive_data[1] = &ufunc_ive_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_ive_data[2] = &ufunc_ive_ptr[2*2]
 * ufunc_ive_data[3] = &ufunc_ive_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8891
 * ufunc_ive_data[0] = &ufunc_ive_ptr[2*0]
 * ufunc_ive_data[1] = &ufunc_ive_ptr[2*1]
 * ufunc_ive_data[2] = &ufunc_ive_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_ive_data[3] = &ufunc_ive_ptr[2*3]
 * ive = np.PyUFunc_FromFuncAndData(ufunc_ive_loops, ufunc_ive_data, ufunc_ive_types, 4, 2, 1, 0, "ive", ufunc_ive_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":8892
 * ufunc_ive_data[1] = &ufunc_ive_ptr[2*1]
 * ufunc_ive_data[2] = &ufunc_ive_ptr[2*2]
 * ufunc_ive_data[3] = &ufunc_ive_ptr[2*3]             # <<<<<<<<<<<<<<
 * ive = np.PyUFunc_FromFuncAndData(ufunc_ive_loops, ufunc_ive_data, ufunc_ive_types, 4, 2, 1, 0, "ive", ufunc_ive_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":8893
 * ufunc_ive_data[2] = &ufunc_ive_ptr[2*2]
 * ufunc_ive_data[3] = &ufunc_ive_ptr[2*3]
 * ive = np.PyUFunc_FromFuncAndData(ufunc_ive_loops, ufunc_ive_data, ufunc_ive_types, 4, 2, 1, 0, "ive", ufunc_ive_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_j0_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_types, 4, 2, 1, 0, ((char *)"ive"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_ive_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8893, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ive, __pyx_t_2) < 0) __PYX_ERR(1, 8893, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":8899
 * cdef void *ufunc_j0_data[2]
 * cdef char ufunc_j0_types[4]
 * cdef char *ufunc_j0_doc = (             # <<<<<<<<<<<<<<
 *     "j0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_doc = ((char *)"j0(x)\n\nBessel function of the first kind of order 0.\n\nParameters\n----------\nx : array_like\n    Argument (float).\n\nReturns\n-------\nJ : ndarray\n    Value of the Bessel function of the first kind of order 0 at `x`.\n\nNotes\n-----\nThe domain is divided into the intervals [0, 5] and (5, infinity). In the\nfirst interval the following rational approximation is used:\n\n.. math::\n\n    J_0(x) \\approx (w - r_1^2)(w - r_2^2) \\frac{P_3(w)}{Q_8(w)},\n\nwhere :math:`w = x^2` and :math:`r_1`, :math:`r_2` are the zeros of\n:math:`J_0`, and :math:`P_3` and :math:`Q_8` are polynomials of degrees 3\nand 8, respectively.\n\nIn the second interval, the Hankel asymptotic expansion is employed with\ntwo rational functions of degree 6/6 and 7/7.\n\nThis function is a wrapper for the Cephes [1]_ routine `j0`.\nIt should not be confused with the spherical Bessel functions (see\n`spherical_jn`).\n\nSee also\n--------\njv : Bessel function of real order and complex argument.\nspherical_jn : spherical Bessel functions.\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":8943
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_j0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_j0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_j0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":8944
 *     "       http://www.netlib.org/cephes/")
 * ufunc_j0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_j0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_j0_types[0] = <char>NPY_FLOAT
 * ufunc_j0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":8945
 * ufunc_j0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_j0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_j0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_j0_types[1] = <char>NPY_FLOAT
 * ufunc_j0_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8946
 * ufunc_j0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_j0_types[0] = <char>NPY_FLOAT
 * ufunc_j0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_j0_types[2] = <char>NPY_DOUBLE
 * ufunc_j0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8947
 * ufunc_j0_types[0] = <char>NPY_FLOAT
 * ufunc_j0_types[1] = <char>NPY_FLOAT
 * ufunc_j0_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_j0_types[3] = <char>NPY_DOUBLE
 * ufunc_j0_ptr[2*0] = <void*>_func_j0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8948
 * ufunc_j0_types[1] = <char>NPY_FLOAT
 * ufunc_j0_types[2] = <char>NPY_DOUBLE
 * ufunc_j0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_j0_ptr[2*0] = <void*>_func_j0
 * ufunc_j0_ptr[2*0+1] = <void*>(<char*>"j0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":8949
 * ufunc_j0_types[2] = <char>NPY_DOUBLE
 * ufunc_j0_types[3] = <char>NPY_DOUBLE
 * ufunc_j0_ptr[2*0] = <void*>_func_j0             # <<<<<<<<<<<<<<
 * ufunc_j0_ptr[2*0+1] = <void*>(<char*>"j0")
 * ufunc_j0_ptr[2*1] = <void*>_func_j0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_ptr[0]) = ((void *)j0);

  /* "scipy/special/_ufuncs.pyx":8950
 * ufunc_j0_types[3] = <char>NPY_DOUBLE
 * ufunc_j0_ptr[2*0] = <void*>_func_j0
 * ufunc_j0_ptr[2*0+1] = <void*>(<char*>"j0")             # <<<<<<<<<<<<<<
 * ufunc_j0_ptr[2*1] = <void*>_func_j0
 * ufunc_j0_ptr[2*1+1] = <void*>(<char*>"j0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_ptr[1]) = ((void *)((char *)((char *)"j0")));

  /* "scipy/special/_ufuncs.pyx":8951
 * ufunc_j0_ptr[2*0] = <void*>_func_j0
 * ufunc_j0_ptr[2*0+1] = <void*>(<char*>"j0")
 * ufunc_j0_ptr[2*1] = <void*>_func_j0             # <<<<<<<<<<<<<<
 * ufunc_j0_ptr[2*1+1] = <void*>(<char*>"j0")
 * ufunc_j0_data[0] = &ufunc_j0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_ptr[2]) = ((void *)j0);

  /* "scipy/special/_ufuncs.pyx":8952
 * ufunc_j0_ptr[2*0+1] = <void*>(<char*>"j0")
 * ufunc_j0_ptr[2*1] = <void*>_func_j0
 * ufunc_j0_ptr[2*1+1] = <void*>(<char*>"j0")             # <<<<<<<<<<<<<<
 * ufunc_j0_data[0] = &ufunc_j0_ptr[2*0]
 * ufunc_j0_data[1] = &ufunc_j0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_ptr[3]) = ((void *)((char *)((char *)"j0")));

  /* "scipy/special/_ufuncs.pyx":8953
 * ufunc_j0_ptr[2*1] = <void*>_func_j0
 * ufunc_j0_ptr[2*1+1] = <void*>(<char*>"j0")
 * ufunc_j0_data[0] = &ufunc_j0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_j0_data[1] = &ufunc_j0_ptr[2*1]
 * j0 = np.PyUFunc_FromFuncAndData(ufunc_j0_loops, ufunc_j0_data, ufunc_j0_types, 2, 1, 1, 0, "j0", ufunc_j0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":8954
 * ufunc_j0_ptr[2*1+1] = <void*>(<char*>"j0")
 * ufunc_j0_data[0] = &ufunc_j0_ptr[2*0]
 * ufunc_j0_data[1] = &ufunc_j0_ptr[2*1]             # <<<<<<<<<<<<<<
 * j0 = np.PyUFunc_FromFuncAndData(ufunc_j0_loops, ufunc_j0_data, ufunc_j0_types, 2, 1, 1, 0, "j0", ufunc_j0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":8955
 * ufunc_j0_data[0] = &ufunc_j0_ptr[2*0]
 * ufunc_j0_data[1] = &ufunc_j0_ptr[2*1]
 * j0 = np.PyUFunc_FromFuncAndData(ufunc_j0_loops, ufunc_j0_data, ufunc_j0_types, 2, 1, 1, 0, "j0", ufunc_j0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_j1_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_types, 2, 1, 1, 0, ((char *)"j0"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_j0_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8955, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_j0, __pyx_t_2) < 0) __PYX_ERR(1, 8955, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":8961
 * cdef void *ufunc_j1_data[2]
 * cdef char ufunc_j1_types[4]
 * cdef char *ufunc_j1_doc = (             # <<<<<<<<<<<<<<
 *     "j1(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_doc = ((char *)"j1(x)\n\nBessel function of the first kind of order 1.\n\nParameters\n----------\nx : array_like\n    Argument (float).\n\nReturns\n-------\nJ : ndarray\n    Value of the Bessel function of the first kind of order 1 at `x`.\n\nNotes\n-----\nThe domain is divided into the intervals [0, 8] and (8, infinity). In the\nfirst interval a 24 term Chebyshev expansion is used. In the second, the\nasymptotic trigonometric representation is employed using two rational\nfunctions of degree 5/5.\n\nThis function is a wrapper for the Cephes [1]_ routine `j1`.\nIt should not be confused with the spherical Bessel functions (see\n`spherical_jn`).\n\nSee also\n--------\njv\nspherical_jn : spherical Bessel functions.\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":8996
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_j1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_j1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_j1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":8997
 *     "       http://www.netlib.org/cephes/")
 * ufunc_j1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_j1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_j1_types[0] = <char>NPY_FLOAT
 * ufunc_j1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":8998
 * ufunc_j1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_j1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_j1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_j1_types[1] = <char>NPY_FLOAT
 * ufunc_j1_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":8999
 * ufunc_j1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_j1_types[0] = <char>NPY_FLOAT
 * ufunc_j1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_j1_types[2] = <char>NPY_DOUBLE
 * ufunc_j1_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9000
 * ufunc_j1_types[0] = <char>NPY_FLOAT
 * ufunc_j1_types[1] = <char>NPY_FLOAT
 * ufunc_j1_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_j1_types[3] = <char>NPY_DOUBLE
 * ufunc_j1_ptr[2*0] = <void*>_func_j1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9001
 * ufunc_j1_types[1] = <char>NPY_FLOAT
 * ufunc_j1_types[2] = <char>NPY_DOUBLE
 * ufunc_j1_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_j1_ptr[2*0] = <void*>_func_j1
 * ufunc_j1_ptr[2*0+1] = <void*>(<char*>"j1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9002
 * ufunc_j1_types[2] = <char>NPY_DOUBLE
 * ufunc_j1_types[3] = <char>NPY_DOUBLE
 * ufunc_j1_ptr[2*0] = <void*>_func_j1             # <<<<<<<<<<<<<<
 * ufunc_j1_ptr[2*0+1] = <void*>(<char*>"j1")
 * ufunc_j1_ptr[2*1] = <void*>_func_j1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_ptr[0]) = ((void *)j1);

  /* "scipy/special/_ufuncs.pyx":9003
 * ufunc_j1_types[3] = <char>NPY_DOUBLE
 * ufunc_j1_ptr[2*0] = <void*>_func_j1
 * ufunc_j1_ptr[2*0+1] = <void*>(<char*>"j1")             # <<<<<<<<<<<<<<
 * ufunc_j1_ptr[2*1] = <void*>_func_j1
 * ufunc_j1_ptr[2*1+1] = <void*>(<char*>"j1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_ptr[1]) = ((void *)((char *)((char *)"j1")));

  /* "scipy/special/_ufuncs.pyx":9004
 * ufunc_j1_ptr[2*0] = <void*>_func_j1
 * ufunc_j1_ptr[2*0+1] = <void*>(<char*>"j1")
 * ufunc_j1_ptr[2*1] = <void*>_func_j1             # <<<<<<<<<<<<<<
 * ufunc_j1_ptr[2*1+1] = <void*>(<char*>"j1")
 * ufunc_j1_data[0] = &ufunc_j1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_ptr[2]) = ((void *)j1);

  /* "scipy/special/_ufuncs.pyx":9005
 * ufunc_j1_ptr[2*0+1] = <void*>(<char*>"j1")
 * ufunc_j1_ptr[2*1] = <void*>_func_j1
 * ufunc_j1_ptr[2*1+1] = <void*>(<char*>"j1")             # <<<<<<<<<<<<<<
 * ufunc_j1_data[0] = &ufunc_j1_ptr[2*0]
 * ufunc_j1_data[1] = &ufunc_j1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_ptr[3]) = ((void *)((char *)((char *)"j1")));

  /* "scipy/special/_ufuncs.pyx":9006
 * ufunc_j1_ptr[2*1] = <void*>_func_j1
 * ufunc_j1_ptr[2*1+1] = <void*>(<char*>"j1")
 * ufunc_j1_data[0] = &ufunc_j1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_j1_data[1] = &ufunc_j1_ptr[2*1]
 * j1 = np.PyUFunc_FromFuncAndData(ufunc_j1_loops, ufunc_j1_data, ufunc_j1_types, 2, 1, 1, 0, "j1", ufunc_j1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9007
 * ufunc_j1_ptr[2*1+1] = <void*>(<char*>"j1")
 * ufunc_j1_data[0] = &ufunc_j1_ptr[2*0]
 * ufunc_j1_data[1] = &ufunc_j1_ptr[2*1]             # <<<<<<<<<<<<<<
 * j1 = np.PyUFunc_FromFuncAndData(ufunc_j1_loops, ufunc_j1_data, ufunc_j1_types, 2, 1, 1, 0, "j1", ufunc_j1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9008
 * ufunc_j1_data[0] = &ufunc_j1_ptr[2*0]
 * ufunc_j1_data[1] = &ufunc_j1_ptr[2*1]
 * j1 = np.PyUFunc_FromFuncAndData(ufunc_j1_loops, ufunc_j1_data, ufunc_j1_types, 2, 1, 1, 0, "j1", ufunc_j1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_jv_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_types, 2, 1, 1, 0, ((char *)"j1"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_j1_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9008, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_j1, __pyx_t_2) < 0) __PYX_ERR(1, 9008, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":9014
 * cdef void *ufunc_jv_data[4]
 * cdef char ufunc_jv_types[12]
 * cdef char *ufunc_jv_doc = (             # <<<<<<<<<<<<<<
 *     "jv(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_doc = ((char *)"jv(v, z)\n\nBessel function of the first kind of real order and complex argument.\n\nParameters\n----------\nv : array_like\n    Order (float).\nz : array_like\n    Argument (float or complex).\n\nReturns\n-------\nJ : ndarray\n    Value of the Bessel function, :math:`J_v(z)`.\n\nNotes\n-----\nFor positive `v` values, the computation is carried out using the AMOS\n[1]_ `zbesj` routine, which exploits the connection to the modified\nBessel function :math:`I_v`,\n\n.. math::\n    J_v(z) = \\exp(v\\pi\\imath/2) I_v(-\\imath z)\\qquad (\\Im z > 0)\n\n    J_v(z) = \\exp(-v\\pi\\imath/2) I_v(\\imath z)\\qquad (\\Im z < 0)\n\nFor negative `v` values the formula,\n\n.. math:: J_{-v}(z) = J_v(z) \\cos(\\pi v) - Y_v(z) \\sin(\\pi v)\n\nis used, where :math:`Y_v(z)` is the Bessel function of the second\nkind, computed using the AMOS routine `zbesy`.  Note that the second\nterm is exactly zero for integer `v`; to improve accuracy the second\nterm is explicitly omitted for `v` values such that `v = floor(v)`.\n\nNot to be confused with the spherical Bessel functions (see `spherical_jn`).\n\nSee also\n--------\njve : :math:`J_v` with leading exponential behavior stripped off.\nspherical_jn : spherical Bessel functions.\n\nReferences\n----------\n.. [1] Donald E. Amos, \"AMOS, A Portable Package for Bessel Functions\n       of a Complex Argument and Nonnegative Order\",\n       http://netlib.org/amos/");

  /* "scipy/special/_ufuncs.pyx":9063
 *     "       of a Complex Argument and Nonnegative Order\",\n"
 *     "       http://netlib.org/amos/")
 * ufunc_jv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_jv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_jv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":9064
 *     "       http://netlib.org/amos/")
 * ufunc_jv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_jv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_jv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_jv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":9065
 * ufunc_jv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_jv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_jv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_jv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_jv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":9066
 * ufunc_jv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_jv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_jv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_jv_types[0] = <char>NPY_FLOAT
 * ufunc_jv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":9067
 * ufunc_jv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_jv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_jv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_jv_types[1] = <char>NPY_FLOAT
 * ufunc_jv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9068
 * ufunc_jv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_jv_types[0] = <char>NPY_FLOAT
 * ufunc_jv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_jv_types[2] = <char>NPY_FLOAT
 * ufunc_jv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9069
 * ufunc_jv_types[0] = <char>NPY_FLOAT
 * ufunc_jv_types[1] = <char>NPY_FLOAT
 * ufunc_jv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_jv_types[3] = <char>NPY_FLOAT
 * ufunc_jv_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9070
 * ufunc_jv_types[1] = <char>NPY_FLOAT
 * ufunc_jv_types[2] = <char>NPY_FLOAT
 * ufunc_jv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_jv_types[4] = <char>NPY_CFLOAT
 * ufunc_jv_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9071
 * ufunc_jv_types[2] = <char>NPY_FLOAT
 * ufunc_jv_types[3] = <char>NPY_FLOAT
 * ufunc_jv_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_jv_types[5] = <char>NPY_CFLOAT
 * ufunc_jv_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9072
 * ufunc_jv_types[3] = <char>NPY_FLOAT
 * ufunc_jv_types[4] = <char>NPY_CFLOAT
 * ufunc_jv_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_jv_types[6] = <char>NPY_DOUBLE
 * ufunc_jv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9073
 * ufunc_jv_types[4] = <char>NPY_CFLOAT
 * ufunc_jv_types[5] = <char>NPY_CFLOAT
 * ufunc_jv_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jv_types[7] = <char>NPY_DOUBLE
 * ufunc_jv_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9074
 * ufunc_jv_types[5] = <char>NPY_CFLOAT
 * ufunc_jv_types[6] = <char>NPY_DOUBLE
 * ufunc_jv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jv_types[8] = <char>NPY_DOUBLE
 * ufunc_jv_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9075
 * ufunc_jv_types[6] = <char>NPY_DOUBLE
 * ufunc_jv_types[7] = <char>NPY_DOUBLE
 * ufunc_jv_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jv_types[9] = <char>NPY_DOUBLE
 * ufunc_jv_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9076
 * ufunc_jv_types[7] = <char>NPY_DOUBLE
 * ufunc_jv_types[8] = <char>NPY_DOUBLE
 * ufunc_jv_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jv_types[10] = <char>NPY_CDOUBLE
 * ufunc_jv_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9077
 * ufunc_jv_types[8] = <char>NPY_DOUBLE
 * ufunc_jv_types[9] = <char>NPY_DOUBLE
 * ufunc_jv_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jv_types[11] = <char>NPY_CDOUBLE
 * ufunc_jv_ptr[2*0] = <void*>_func_cbesj_wrap_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9078
 * ufunc_jv_types[9] = <char>NPY_DOUBLE
 * ufunc_jv_types[10] = <char>NPY_CDOUBLE
 * ufunc_jv_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jv_ptr[2*0] = <void*>_func_cbesj_wrap_real
 * ufunc_jv_ptr[2*0+1] = <void*>(<char*>"jv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9079
 * ufunc_jv_types[10] = <char>NPY_CDOUBLE
 * ufunc_jv_types[11] = <char>NPY_CDOUBLE
 * ufunc_jv_ptr[2*0] = <void*>_func_cbesj_wrap_real             # <<<<<<<<<<<<<<
 * ufunc_jv_ptr[2*0+1] = <void*>(<char*>"jv")
 * ufunc_jv_ptr[2*1] = <void*>_func_cbesj_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[0]) = ((void *)cbesj_wrap_real);

  /* "scipy/special/_ufuncs.pyx":9080
 * ufunc_jv_types[11] = <char>NPY_CDOUBLE
 * ufunc_jv_ptr[2*0] = <void*>_func_cbesj_wrap_real
 * ufunc_jv_ptr[2*0+1] = <void*>(<char*>"jv")             # <<<<<<<<<<<<<<
 * ufunc_jv_ptr[2*1] = <void*>_func_cbesj_wrap
 * ufunc_jv_ptr[2*1+1] = <void*>(<char*>"jv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[1]) = ((void *)((char *)((char *)"jv")));

  /* "scipy/special/_ufuncs.pyx":9081
 * ufunc_jv_ptr[2*0] = <void*>_func_cbesj_wrap_real
 * ufunc_jv_ptr[2*0+1] = <void*>(<char*>"jv")
 * ufunc_jv_ptr[2*1] = <void*>_func_cbesj_wrap             # <<<<<<<<<<<<<<
 * ufunc_jv_ptr[2*1+1] = <void*>(<char*>"jv")
 * ufunc_jv_ptr[2*2] = <void*>_func_cbesj_wrap_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[2]) = ((void *)cbesj_wrap);

  /* "scipy/special/_ufuncs.pyx":9082
 * ufunc_jv_ptr[2*0+1] = <void*>(<char*>"jv")
 * ufunc_jv_ptr[2*1] = <void*>_func_cbesj_wrap
 * ufunc_jv_ptr[2*1+1] = <void*>(<char*>"jv")             # <<<<<<<<<<<<<<
 * ufunc_jv_ptr[2*2] = <void*>_func_cbesj_wrap_real
 * ufunc_jv_ptr[2*2+1] = <void*>(<char*>"jv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[3]) = ((void *)((char *)((char *)"jv")));

  /* "scipy/special/_ufuncs.pyx":9083
 * ufunc_jv_ptr[2*1] = <void*>_func_cbesj_wrap
 * ufunc_jv_ptr[2*1+1] = <void*>(<char*>"jv")
 * ufunc_jv_ptr[2*2] = <void*>_func_cbesj_wrap_real             # <<<<<<<<<<<<<<
 * ufunc_jv_ptr[2*2+1] = <void*>(<char*>"jv")
 * ufunc_jv_ptr[2*3] = <void*>_func_cbesj_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[4]) = ((void *)cbesj_wrap_real);

  /* "scipy/special/_ufuncs.pyx":9084
 * ufunc_jv_ptr[2*1+1] = <void*>(<char*>"jv")
 * ufunc_jv_ptr[2*2] = <void*>_func_cbesj_wrap_real
 * ufunc_jv_ptr[2*2+1] = <void*>(<char*>"jv")             # <<<<<<<<<<<<<<
 * ufunc_jv_ptr[2*3] = <void*>_func_cbesj_wrap
 * ufunc_jv_ptr[2*3+1] = <void*>(<char*>"jv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[5]) = ((void *)((char *)((char *)"jv")));

  /* "scipy/special/_ufuncs.pyx":9085
 * ufunc_jv_ptr[2*2] = <void*>_func_cbesj_wrap_real
 * ufunc_jv_ptr[2*2+1] = <void*>(<char*>"jv")
 * ufunc_jv_ptr[2*3] = <void*>_func_cbesj_wrap             # <<<<<<<<<<<<<<
 * ufunc_jv_ptr[2*3+1] = <void*>(<char*>"jv")
 * ufunc_jv_data[0] = &ufunc_jv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[6]) = ((void *)cbesj_wrap);

  /* "scipy/special/_ufuncs.pyx":9086
 * ufunc_jv_ptr[2*2+1] = <void*>(<char*>"jv")
 * ufunc_jv_ptr[2*3] = <void*>_func_cbesj_wrap
 * ufunc_jv_ptr[2*3+1] = <void*>(<char*>"jv")             # <<<<<<<<<<<<<<
 * ufunc_jv_data[0] = &ufunc_jv_ptr[2*0]
 * ufunc_jv_data[1] = &ufunc_jv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[7]) = ((void *)((char *)((char *)"jv")));

  /* "scipy/special/_ufuncs.pyx":9087
 * ufunc_jv_ptr[2*3] = <void*>_func_cbesj_wrap
 * ufunc_jv_ptr[2*3+1] = <void*>(<char*>"jv")
 * ufunc_jv_data[0] = &ufunc_jv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_jv_data[1] = &ufunc_jv_ptr[2*1]
 * ufunc_jv_data[2] = &ufunc_jv_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9088
 * ufunc_jv_ptr[2*3+1] = <void*>(<char*>"jv")
 * ufunc_jv_data[0] = &ufunc_jv_ptr[2*0]
 * ufunc_jv_data[1] = &ufunc_jv_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_jv_data[2] = &ufunc_jv_ptr[2*2]
 * ufunc_jv_data[3] = &ufunc_jv_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9089
 * ufunc_jv_data[0] = &ufunc_jv_ptr[2*0]
 * ufunc_jv_data[1] = &ufunc_jv_ptr[2*1]
 * ufunc_jv_data[2] = &ufunc_jv_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_jv_data[3] = &ufunc_jv_ptr[2*3]
 * jv = np.PyUFunc_FromFuncAndData(ufunc_jv_loops, ufunc_jv_data, ufunc_jv_types, 4, 2, 1, 0, "jv", ufunc_jv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":9090
 * ufunc_jv_data[1] = &ufunc_jv_ptr[2*1]
 * ufunc_jv_data[2] = &ufunc_jv_ptr[2*2]
 * ufunc_jv_data[3] = &ufunc_jv_ptr[2*3]             # <<<<<<<<<<<<<<
 * jv = np.PyUFunc_FromFuncAndData(ufunc_jv_loops, ufunc_jv_data, ufunc_jv_types, 4, 2, 1, 0, "jv", ufunc_jv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":9091
 * ufunc_jv_data[2] = &ufunc_jv_ptr[2*2]
 * ufunc_jv_data[3] = &ufunc_jv_ptr[2*3]
 * jv = np.PyUFunc_FromFuncAndData(ufunc_jv_loops, ufunc_jv_data, ufunc_jv_types, 4, 2, 1, 0, "jv", ufunc_jv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_jve_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_types, 4, 2, 1, 0, ((char *)"jv"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_jv_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9091, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_jv, __pyx_t_2) < 0) __PYX_ERR(1, 9091, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":9097
 * cdef void *ufunc_jve_data[4]
 * cdef char ufunc_jve_types[12]
 * cdef char *ufunc_jve_doc = (             # <<<<<<<<<<<<<<
 *     "jve(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_doc = ((char *)"jve(v, z)\n\nExponentially scaled Bessel function of order `v`.\n\nDefined as::\n\n    jve(v, z) = jv(v, z) * exp(-abs(z.imag))\n\nParameters\n----------\nv : array_like\n    Order (float).\nz : array_like\n    Argument (float or complex).\n\nReturns\n-------\nJ : ndarray\n    Value of the exponentially scaled Bessel function.\n\nNotes\n-----\nFor positive `v` values, the computation is carried out using the AMOS\n[1]_ `zbesj` routine, which exploits the connection to the modified\nBessel function :math:`I_v`,\n\n.. math::\n    J_v(z) = \\exp(v\\pi\\imath/2) I_v(-\\imath z)\\qquad (\\Im z > 0)\n\n    J_v(z) = \\exp(-v\\pi\\imath/2) I_v(\\imath z)\\qquad (\\Im z < 0)\n\nFor negative `v` values the formula,\n\n.. math:: J_{-v}(z) = J_v(z) \\cos(\\pi v) - Y_v(z) \\sin(\\pi v)\n\nis used, where :math:`Y_v(z)` is the Bessel function of the second\nkind, computed using the AMOS routine `zbesy`.  Note that the second\nterm is exactly zero for integer `v`; to improve accuracy the second\nterm is explicitly omitted for `v` values such that `v = floor(v)`.\n\nReferences\n----------\n.. [1] Donald E. Amos, \"AMOS, A Portable Package for Bessel Functions\n       of a Complex Argument and Nonnegative Order\",\n       http://netlib.org/amos/");

  /* "scipy/special/_ufuncs.pyx":9143
 *     "       of a Complex Argument and Nonnegative Order\",\n"
 *     "       http://netlib.org/amos/")
 * ufunc_jve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_jve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_jve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":9144
 *     "       http://netlib.org/amos/")
 * ufunc_jve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_jve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_jve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_jve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":9145
 * ufunc_jve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_jve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_jve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_jve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_jve_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":9146
 * ufunc_jve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_jve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_jve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_jve_types[0] = <char>NPY_FLOAT
 * ufunc_jve_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":9147
 * ufunc_jve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_jve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_jve_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_jve_types[1] = <char>NPY_FLOAT
 * ufunc_jve_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9148
 * ufunc_jve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_jve_types[0] = <char>NPY_FLOAT
 * ufunc_jve_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_jve_types[2] = <char>NPY_FLOAT
 * ufunc_jve_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9149
 * ufunc_jve_types[0] = <char>NPY_FLOAT
 * ufunc_jve_types[1] = <char>NPY_FLOAT
 * ufunc_jve_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_jve_types[3] = <char>NPY_FLOAT
 * ufunc_jve_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9150
 * ufunc_jve_types[1] = <char>NPY_FLOAT
 * ufunc_jve_types[2] = <char>NPY_FLOAT
 * ufunc_jve_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_jve_types[4] = <char>NPY_CFLOAT
 * ufunc_jve_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9151
 * ufunc_jve_types[2] = <char>NPY_FLOAT
 * ufunc_jve_types[3] = <char>NPY_FLOAT
 * ufunc_jve_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_jve_types[5] = <char>NPY_CFLOAT
 * ufunc_jve_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9152
 * ufunc_jve_types[3] = <char>NPY_FLOAT
 * ufunc_jve_types[4] = <char>NPY_CFLOAT
 * ufunc_jve_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_jve_types[6] = <char>NPY_DOUBLE
 * ufunc_jve_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9153
 * ufunc_jve_types[4] = <char>NPY_CFLOAT
 * ufunc_jve_types[5] = <char>NPY_CFLOAT
 * ufunc_jve_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jve_types[7] = <char>NPY_DOUBLE
 * ufunc_jve_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9154
 * ufunc_jve_types[5] = <char>NPY_CFLOAT
 * ufunc_jve_types[6] = <char>NPY_DOUBLE
 * ufunc_jve_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jve_types[8] = <char>NPY_DOUBLE
 * ufunc_jve_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9155
 * ufunc_jve_types[6] = <char>NPY_DOUBLE
 * ufunc_jve_types[7] = <char>NPY_DOUBLE
 * ufunc_jve_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jve_types[9] = <char>NPY_DOUBLE
 * ufunc_jve_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9156
 * ufunc_jve_types[7] = <char>NPY_DOUBLE
 * ufunc_jve_types[8] = <char>NPY_DOUBLE
 * ufunc_jve_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jve_types[10] = <char>NPY_CDOUBLE
 * ufunc_jve_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9157
 * ufunc_jve_types[8] = <char>NPY_DOUBLE
 * ufunc_jve_types[9] = <char>NPY_DOUBLE
 * ufunc_jve_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jve_types[11] = <char>NPY_CDOUBLE
 * ufunc_jve_ptr[2*0] = <void*>_func_cbesj_wrap_e_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9158
 * ufunc_jve_types[9] = <char>NPY_DOUBLE
 * ufunc_jve_types[10] = <char>NPY_CDOUBLE
 * ufunc_jve_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_jve_ptr[2*0] = <void*>_func_cbesj_wrap_e_real
 * ufunc_jve_ptr[2*0+1] = <void*>(<char*>"jve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9159
 * ufunc_jve_types[10] = <char>NPY_CDOUBLE
 * ufunc_jve_types[11] = <char>NPY_CDOUBLE
 * ufunc_jve_ptr[2*0] = <void*>_func_cbesj_wrap_e_real             # <<<<<<<<<<<<<<
 * ufunc_jve_ptr[2*0+1] = <void*>(<char*>"jve")
 * ufunc_jve_ptr[2*1] = <void*>_func_cbesj_wrap_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[0]) = ((void *)cbesj_wrap_e_real);

  /* "scipy/special/_ufuncs.pyx":9160
 * ufunc_jve_types[11] = <char>NPY_CDOUBLE
 * ufunc_jve_ptr[2*0] = <void*>_func_cbesj_wrap_e_real
 * ufunc_jve_ptr[2*0+1] = <void*>(<char*>"jve")             # <<<<<<<<<<<<<<
 * ufunc_jve_ptr[2*1] = <void*>_func_cbesj_wrap_e
 * ufunc_jve_ptr[2*1+1] = <void*>(<char*>"jve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[1]) = ((void *)((char *)((char *)"jve")));

  /* "scipy/special/_ufuncs.pyx":9161
 * ufunc_jve_ptr[2*0] = <void*>_func_cbesj_wrap_e_real
 * ufunc_jve_ptr[2*0+1] = <void*>(<char*>"jve")
 * ufunc_jve_ptr[2*1] = <void*>_func_cbesj_wrap_e             # <<<<<<<<<<<<<<
 * ufunc_jve_ptr[2*1+1] = <void*>(<char*>"jve")
 * ufunc_jve_ptr[2*2] = <void*>_func_cbesj_wrap_e_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[2]) = ((void *)cbesj_wrap_e);

  /* "scipy/special/_ufuncs.pyx":9162
 * ufunc_jve_ptr[2*0+1] = <void*>(<char*>"jve")
 * ufunc_jve_ptr[2*1] = <void*>_func_cbesj_wrap_e
 * ufunc_jve_ptr[2*1+1] = <void*>(<char*>"jve")             # <<<<<<<<<<<<<<
 * ufunc_jve_ptr[2*2] = <void*>_func_cbesj_wrap_e_real
 * ufunc_jve_ptr[2*2+1] = <void*>(<char*>"jve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[3]) = ((void *)((char *)((char *)"jve")));

  /* "scipy/special/_ufuncs.pyx":9163
 * ufunc_jve_ptr[2*1] = <void*>_func_cbesj_wrap_e
 * ufunc_jve_ptr[2*1+1] = <void*>(<char*>"jve")
 * ufunc_jve_ptr[2*2] = <void*>_func_cbesj_wrap_e_real             # <<<<<<<<<<<<<<
 * ufunc_jve_ptr[2*2+1] = <void*>(<char*>"jve")
 * ufunc_jve_ptr[2*3] = <void*>_func_cbesj_wrap_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[4]) = ((void *)cbesj_wrap_e_real);

  /* "scipy/special/_ufuncs.pyx":9164
 * ufunc_jve_ptr[2*1+1] = <void*>(<char*>"jve")
 * ufunc_jve_ptr[2*2] = <void*>_func_cbesj_wrap_e_real
 * ufunc_jve_ptr[2*2+1] = <void*>(<char*>"jve")             # <<<<<<<<<<<<<<
 * ufunc_jve_ptr[2*3] = <void*>_func_cbesj_wrap_e
 * ufunc_jve_ptr[2*3+1] = <void*>(<char*>"jve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[5]) = ((void *)((char *)((char *)"jve")));

  /* "scipy/special/_ufuncs.pyx":9165
 * ufunc_jve_ptr[2*2] = <void*>_func_cbesj_wrap_e_real
 * ufunc_jve_ptr[2*2+1] = <void*>(<char*>"jve")
 * ufunc_jve_ptr[2*3] = <void*>_func_cbesj_wrap_e             # <<<<<<<<<<<<<<
 * ufunc_jve_ptr[2*3+1] = <void*>(<char*>"jve")
 * ufunc_jve_data[0] = &ufunc_jve_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[6]) = ((void *)cbesj_wrap_e);

  /* "scipy/special/_ufuncs.pyx":9166
 * ufunc_jve_ptr[2*2+1] = <void*>(<char*>"jve")
 * ufunc_jve_ptr[2*3] = <void*>_func_cbesj_wrap_e
 * ufunc_jve_ptr[2*3+1] = <void*>(<char*>"jve")             # <<<<<<<<<<<<<<
 * ufunc_jve_data[0] = &ufunc_jve_ptr[2*0]
 * ufunc_jve_data[1] = &ufunc_jve_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[7]) = ((void *)((char *)((char *)"jve")));

  /* "scipy/special/_ufuncs.pyx":9167
 * ufunc_jve_ptr[2*3] = <void*>_func_cbesj_wrap_e
 * ufunc_jve_ptr[2*3+1] = <void*>(<char*>"jve")
 * ufunc_jve_data[0] = &ufunc_jve_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_jve_data[1] = &ufunc_jve_ptr[2*1]
 * ufunc_jve_data[2] = &ufunc_jve_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9168
 * ufunc_jve_ptr[2*3+1] = <void*>(<char*>"jve")
 * ufunc_jve_data[0] = &ufunc_jve_ptr[2*0]
 * ufunc_jve_data[1] = &ufunc_jve_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_jve_data[2] = &ufunc_jve_ptr[2*2]
 * ufunc_jve_data[3] = &ufunc_jve_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9169
 * ufunc_jve_data[0] = &ufunc_jve_ptr[2*0]
 * ufunc_jve_data[1] = &ufunc_jve_ptr[2*1]
 * ufunc_jve_data[2] = &ufunc_jve_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_jve_data[3] = &ufunc_jve_ptr[2*3]
 * jve = np.PyUFunc_FromFuncAndData(ufunc_jve_loops, ufunc_jve_data, ufunc_jve_types, 4, 2, 1, 0, "jve", ufunc_jve_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":9170
 * ufunc_jve_data[1] = &ufunc_jve_ptr[2*1]
 * ufunc_jve_data[2] = &ufunc_jve_ptr[2*2]
 * ufunc_jve_data[3] = &ufunc_jve_ptr[2*3]             # <<<<<<<<<<<<<<
 * jve = np.PyUFunc_FromFuncAndData(ufunc_jve_loops, ufunc_jve_data, ufunc_jve_types, 4, 2, 1, 0, "jve", ufunc_jve_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":9171
 * ufunc_jve_data[2] = &ufunc_jve_ptr[2*2]
 * ufunc_jve_data[3] = &ufunc_jve_ptr[2*3]
 * jve = np.PyUFunc_FromFuncAndData(ufunc_jve_loops, ufunc_jve_data, ufunc_jve_types, 4, 2, 1, 0, "jve", ufunc_jve_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_k0_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_types, 4, 2, 1, 0, ((char *)"jve"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_jve_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_jve, __pyx_t_2) < 0) __PYX_ERR(1, 9171, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":9177
 * cdef void *ufunc_k0_data[2]
 * cdef char ufunc_k0_types[4]
 * cdef char *ufunc_k0_doc = (             # <<<<<<<<<<<<<<
 *     "k0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_doc = ((char *)"k0(x)\n\nModified Bessel function of the second kind of order 0, :math:`K_0`.\n\nThis function is also sometimes referred to as the modified Bessel\nfunction of the third kind of order 0.\n\nParameters\n----------\nx : array_like\n    Argument (float).\n\nReturns\n-------\nK : ndarray\n    Value of the modified Bessel function :math:`K_0` at `x`.\n\nNotes\n-----\nThe range is partitioned into the two intervals [0, 2] and (2, infinity).\nChebyshev polynomial expansions are employed in each interval.\n\nThis function is a wrapper for the Cephes [1]_ routine `k0`.\n\nSee also\n--------\nkv\nk0e\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":9211
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_k0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_k0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":9212
 *     "       http://www.netlib.org/cephes/")
 * ufunc_k0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_k0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_k0_types[0] = <char>NPY_FLOAT
 * ufunc_k0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":9213
 * ufunc_k0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_k0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_k0_types[1] = <char>NPY_FLOAT
 * ufunc_k0_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9214
 * ufunc_k0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k0_types[0] = <char>NPY_FLOAT
 * ufunc_k0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_k0_types[2] = <char>NPY_DOUBLE
 * ufunc_k0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9215
 * ufunc_k0_types[0] = <char>NPY_FLOAT
 * ufunc_k0_types[1] = <char>NPY_FLOAT
 * ufunc_k0_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_k0_types[3] = <char>NPY_DOUBLE
 * ufunc_k0_ptr[2*0] = <void*>_func_k0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9216
 * ufunc_k0_types[1] = <char>NPY_FLOAT
 * ufunc_k0_types[2] = <char>NPY_DOUBLE
 * ufunc_k0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_k0_ptr[2*0] = <void*>_func_k0
 * ufunc_k0_ptr[2*0+1] = <void*>(<char*>"k0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9217
 * ufunc_k0_types[2] = <char>NPY_DOUBLE
 * ufunc_k0_types[3] = <char>NPY_DOUBLE
 * ufunc_k0_ptr[2*0] = <void*>_func_k0             # <<<<<<<<<<<<<<
 * ufunc_k0_ptr[2*0+1] = <void*>(<char*>"k0")
 * ufunc_k0_ptr[2*1] = <void*>_func_k0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_ptr[0]) = ((void *)k0);

  /* "scipy/special/_ufuncs.pyx":9218
 * ufunc_k0_types[3] = <char>NPY_DOUBLE
 * ufunc_k0_ptr[2*0] = <void*>_func_k0
 * ufunc_k0_ptr[2*0+1] = <void*>(<char*>"k0")             # <<<<<<<<<<<<<<
 * ufunc_k0_ptr[2*1] = <void*>_func_k0
 * ufunc_k0_ptr[2*1+1] = <void*>(<char*>"k0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_ptr[1]) = ((void *)((char *)((char *)"k0")));

  /* "scipy/special/_ufuncs.pyx":9219
 * ufunc_k0_ptr[2*0] = <void*>_func_k0
 * ufunc_k0_ptr[2*0+1] = <void*>(<char*>"k0")
 * ufunc_k0_ptr[2*1] = <void*>_func_k0             # <<<<<<<<<<<<<<
 * ufunc_k0_ptr[2*1+1] = <void*>(<char*>"k0")
 * ufunc_k0_data[0] = &ufunc_k0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_ptr[2]) = ((void *)k0);

  /* "scipy/special/_ufuncs.pyx":9220
 * ufunc_k0_ptr[2*0+1] = <void*>(<char*>"k0")
 * ufunc_k0_ptr[2*1] = <void*>_func_k0
 * ufunc_k0_ptr[2*1+1] = <void*>(<char*>"k0")             # <<<<<<<<<<<<<<
 * ufunc_k0_data[0] = &ufunc_k0_ptr[2*0]
 * ufunc_k0_data[1] = &ufunc_k0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_ptr[3]) = ((void *)((char *)((char *)"k0")));

  /* "scipy/special/_ufuncs.pyx":9221
 * ufunc_k0_ptr[2*1] = <void*>_func_k0
 * ufunc_k0_ptr[2*1+1] = <void*>(<char*>"k0")
 * ufunc_k0_data[0] = &ufunc_k0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_k0_data[1] = &ufunc_k0_ptr[2*1]
 * k0 = np.PyUFunc_FromFuncAndData(ufunc_k0_loops, ufunc_k0_data, ufunc_k0_types, 2, 1, 1, 0, "k0", ufunc_k0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9222
 * ufunc_k0_ptr[2*1+1] = <void*>(<char*>"k0")
 * ufunc_k0_data[0] = &ufunc_k0_ptr[2*0]
 * ufunc_k0_data[1] = &ufunc_k0_ptr[2*1]             # <<<<<<<<<<<<<<
 * k0 = np.PyUFunc_FromFuncAndData(ufunc_k0_loops, ufunc_k0_data, ufunc_k0_types, 2, 1, 1, 0, "k0", ufunc_k0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9223
 * ufunc_k0_data[0] = &ufunc_k0_ptr[2*0]
 * ufunc_k0_data[1] = &ufunc_k0_ptr[2*1]
 * k0 = np.PyUFunc_FromFuncAndData(ufunc_k0_loops, ufunc_k0_data, ufunc_k0_types, 2, 1, 1, 0, "k0", ufunc_k0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_k0e_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_types, 2, 1, 1, 0, ((char *)"k0"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9223, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_k0, __pyx_t_2) < 0) __PYX_ERR(1, 9223, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":9229
 * cdef void *ufunc_k0e_data[2]
 * cdef char ufunc_k0e_types[4]
 * cdef char *ufunc_k0e_doc = (             # <<<<<<<<<<<<<<
 *     "k0e(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_doc = ((char *)"k0e(x)\n\nExponentially scaled modified Bessel function K of order 0\n\nDefined as::\n\n    k0e(x) = exp(x) * k0(x).\n\nParameters\n----------\nx : array_like\n    Argument (float)\n\nReturns\n-------\nK : ndarray\n    Value of the exponentially scaled modified Bessel function K of order\n    0 at `x`.\n\nNotes\n-----\nThe range is partitioned into the two intervals [0, 2] and (2, infinity).\nChebyshev polynomial expansions are employed in each interval.\n\nThis function is a wrapper for the Cephes [1]_ routine `k0e`.\n\nSee also\n--------\nkv\nk0\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":9265
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_k0e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_k0e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k0e_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":9266
 *     "       http://www.netlib.org/cephes/")
 * ufunc_k0e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_k0e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_k0e_types[0] = <char>NPY_FLOAT
 * ufunc_k0e_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":9267
 * ufunc_k0e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_k0e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k0e_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_k0e_types[1] = <char>NPY_FLOAT
 * ufunc_k0e_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9268
 * ufunc_k0e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k0e_types[0] = <char>NPY_FLOAT
 * ufunc_k0e_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_k0e_types[2] = <char>NPY_DOUBLE
 * ufunc_k0e_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9269
 * ufunc_k0e_types[0] = <char>NPY_FLOAT
 * ufunc_k0e_types[1] = <char>NPY_FLOAT
 * ufunc_k0e_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_k0e_types[3] = <char>NPY_DOUBLE
 * ufunc_k0e_ptr[2*0] = <void*>_func_k0e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9270
 * ufunc_k0e_types[1] = <char>NPY_FLOAT
 * ufunc_k0e_types[2] = <char>NPY_DOUBLE
 * ufunc_k0e_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_k0e_ptr[2*0] = <void*>_func_k0e
 * ufunc_k0e_ptr[2*0+1] = <void*>(<char*>"k0e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9271
 * ufunc_k0e_types[2] = <char>NPY_DOUBLE
 * ufunc_k0e_types[3] = <char>NPY_DOUBLE
 * ufunc_k0e_ptr[2*0] = <void*>_func_k0e             # <<<<<<<<<<<<<<
 * ufunc_k0e_ptr[2*0+1] = <void*>(<char*>"k0e")
 * ufunc_k0e_ptr[2*1] = <void*>_func_k0e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_ptr[0]) = ((void *)k0e);

  /* "scipy/special/_ufuncs.pyx":9272
 * ufunc_k0e_types[3] = <char>NPY_DOUBLE
 * ufunc_k0e_ptr[2*0] = <void*>_func_k0e
 * ufunc_k0e_ptr[2*0+1] = <void*>(<char*>"k0e")             # <<<<<<<<<<<<<<
 * ufunc_k0e_ptr[2*1] = <void*>_func_k0e
 * ufunc_k0e_ptr[2*1+1] = <void*>(<char*>"k0e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_ptr[1]) = ((void *)((char *)((char *)"k0e")));

  /* "scipy/special/_ufuncs.pyx":9273
 * ufunc_k0e_ptr[2*0] = <void*>_func_k0e
 * ufunc_k0e_ptr[2*0+1] = <void*>(<char*>"k0e")
 * ufunc_k0e_ptr[2*1] = <void*>_func_k0e             # <<<<<<<<<<<<<<
 * ufunc_k0e_ptr[2*1+1] = <void*>(<char*>"k0e")
 * ufunc_k0e_data[0] = &ufunc_k0e_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_ptr[2]) = ((void *)k0e);

  /* "scipy/special/_ufuncs.pyx":9274
 * ufunc_k0e_ptr[2*0+1] = <void*>(<char*>"k0e")
 * ufunc_k0e_ptr[2*1] = <void*>_func_k0e
 * ufunc_k0e_ptr[2*1+1] = <void*>(<char*>"k0e")             # <<<<<<<<<<<<<<
 * ufunc_k0e_data[0] = &ufunc_k0e_ptr[2*0]
 * ufunc_k0e_data[1] = &ufunc_k0e_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_ptr[3]) = ((void *)((char *)((char *)"k0e")));

  /* "scipy/special/_ufuncs.pyx":9275
 * ufunc_k0e_ptr[2*1] = <void*>_func_k0e
 * ufunc_k0e_ptr[2*1+1] = <void*>(<char*>"k0e")
 * ufunc_k0e_data[0] = &ufunc_k0e_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_k0e_data[1] = &ufunc_k0e_ptr[2*1]
 * k0e = np.PyUFunc_FromFuncAndData(ufunc_k0e_loops, ufunc_k0e_data, ufunc_k0e_types, 2, 1, 1, 0, "k0e", ufunc_k0e_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9276
 * ufunc_k0e_ptr[2*1+1] = <void*>(<char*>"k0e")
 * ufunc_k0e_data[0] = &ufunc_k0e_ptr[2*0]
 * ufunc_k0e_data[1] = &ufunc_k0e_ptr[2*1]             # <<<<<<<<<<<<<<
 * k0e = np.PyUFunc_FromFuncAndData(ufunc_k0e_loops, ufunc_k0e_data, ufunc_k0e_types, 2, 1, 1, 0, "k0e", ufunc_k0e_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9277
 * ufunc_k0e_data[0] = &ufunc_k0e_ptr[2*0]
 * ufunc_k0e_data[1] = &ufunc_k0e_ptr[2*1]
 * k0e = np.PyUFunc_FromFuncAndData(ufunc_k0e_loops, ufunc_k0e_data, ufunc_k0e_types, 2, 1, 1, 0, "k0e", ufunc_k0e_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_k1_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_types, 2, 1, 1, 0, ((char *)"k0e"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_k0e_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9277, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_k0e, __pyx_t_2) < 0) __PYX_ERR(1, 9277, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":9283
 * cdef void *ufunc_k1_data[2]
 * cdef char ufunc_k1_types[4]
 * cdef char *ufunc_k1_doc = (             # <<<<<<<<<<<<<<
 *     "k1(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_doc = ((char *)"k1(x)\n\nModified Bessel function of the second kind of order 1, :math:`K_1(x)`.\n\nParameters\n----------\nx : array_like\n    Argument (float)\n\nReturns\n-------\nK : ndarray\n    Value of the modified Bessel function K of order 1 at `x`.\n\nNotes\n-----\nThe range is partitioned into the two intervals [0, 2] and (2, infinity).\nChebyshev polynomial expansions are employed in each interval.\n\nThis function is a wrapper for the Cephes [1]_ routine `k1`.\n\nSee also\n--------\nkv\nk1e\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":9314
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_k1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_k1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":9315
 *     "       http://www.netlib.org/cephes/")
 * ufunc_k1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_k1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_k1_types[0] = <char>NPY_FLOAT
 * ufunc_k1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":9316
 * ufunc_k1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_k1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_k1_types[1] = <char>NPY_FLOAT
 * ufunc_k1_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9317
 * ufunc_k1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k1_types[0] = <char>NPY_FLOAT
 * ufunc_k1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_k1_types[2] = <char>NPY_DOUBLE
 * ufunc_k1_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9318
 * ufunc_k1_types[0] = <char>NPY_FLOAT
 * ufunc_k1_types[1] = <char>NPY_FLOAT
 * ufunc_k1_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_k1_types[3] = <char>NPY_DOUBLE
 * ufunc_k1_ptr[2*0] = <void*>_func_k1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9319
 * ufunc_k1_types[1] = <char>NPY_FLOAT
 * ufunc_k1_types[2] = <char>NPY_DOUBLE
 * ufunc_k1_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_k1_ptr[2*0] = <void*>_func_k1
 * ufunc_k1_ptr[2*0+1] = <void*>(<char*>"k1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9320
 * ufunc_k1_types[2] = <char>NPY_DOUBLE
 * ufunc_k1_types[3] = <char>NPY_DOUBLE
 * ufunc_k1_ptr[2*0] = <void*>_func_k1             # <<<<<<<<<<<<<<
 * ufunc_k1_ptr[2*0+1] = <void*>(<char*>"k1")
 * ufunc_k1_ptr[2*1] = <void*>_func_k1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_ptr[0]) = ((void *)k1);

  /* "scipy/special/_ufuncs.pyx":9321
 * ufunc_k1_types[3] = <char>NPY_DOUBLE
 * ufunc_k1_ptr[2*0] = <void*>_func_k1
 * ufunc_k1_ptr[2*0+1] = <void*>(<char*>"k1")             # <<<<<<<<<<<<<<
 * ufunc_k1_ptr[2*1] = <void*>_func_k1
 * ufunc_k1_ptr[2*1+1] = <void*>(<char*>"k1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_ptr[1]) = ((void *)((char *)((char *)"k1")));

  /* "scipy/special/_ufuncs.pyx":9322
 * ufunc_k1_ptr[2*0] = <void*>_func_k1
 * ufunc_k1_ptr[2*0+1] = <void*>(<char*>"k1")
 * ufunc_k1_ptr[2*1] = <void*>_func_k1             # <<<<<<<<<<<<<<
 * ufunc_k1_ptr[2*1+1] = <void*>(<char*>"k1")
 * ufunc_k1_data[0] = &ufunc_k1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_ptr[2]) = ((void *)k1);

  /* "scipy/special/_ufuncs.pyx":9323
 * ufunc_k1_ptr[2*0+1] = <void*>(<char*>"k1")
 * ufunc_k1_ptr[2*1] = <void*>_func_k1
 * ufunc_k1_ptr[2*1+1] = <void*>(<char*>"k1")             # <<<<<<<<<<<<<<
 * ufunc_k1_data[0] = &ufunc_k1_ptr[2*0]
 * ufunc_k1_data[1] = &ufunc_k1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_ptr[3]) = ((void *)((char *)((char *)"k1")));

  /* "scipy/special/_ufuncs.pyx":9324
 * ufunc_k1_ptr[2*1] = <void*>_func_k1
 * ufunc_k1_ptr[2*1+1] = <void*>(<char*>"k1")
 * ufunc_k1_data[0] = &ufunc_k1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_k1_data[1] = &ufunc_k1_ptr[2*1]
 * k1 = np.PyUFunc_FromFuncAndData(ufunc_k1_loops, ufunc_k1_data, ufunc_k1_types, 2, 1, 1, 0, "k1", ufunc_k1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9325
 * ufunc_k1_ptr[2*1+1] = <void*>(<char*>"k1")
 * ufunc_k1_data[0] = &ufunc_k1_ptr[2*0]
 * ufunc_k1_data[1] = &ufunc_k1_ptr[2*1]             # <<<<<<<<<<<<<<
 * k1 = np.PyUFunc_FromFuncAndData(ufunc_k1_loops, ufunc_k1_data, ufunc_k1_types, 2, 1, 1, 0, "k1", ufunc_k1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9326
 * ufunc_k1_data[0] = &ufunc_k1_ptr[2*0]
 * ufunc_k1_data[1] = &ufunc_k1_ptr[2*1]
 * k1 = np.PyUFunc_FromFuncAndData(ufunc_k1_loops, ufunc_k1_data, ufunc_k1_types, 2, 1, 1, 0, "k1", ufunc_k1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_k1e_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_types, 2, 1, 1, 0, ((char *)"k1"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9326, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_k1, __pyx_t_2) < 0) __PYX_ERR(1, 9326, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":9332
 * cdef void *ufunc_k1e_data[2]
 * cdef char ufunc_k1e_types[4]
 * cdef char *ufunc_k1e_doc = (             # <<<<<<<<<<<<<<
 *     "k1e(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_doc = ((char *)"k1e(x)\n\nExponentially scaled modified Bessel function K of order 1\n\nDefined as::\n\n    k1e(x) = exp(x) * k1(x)\n\nParameters\n----------\nx : array_like\n    Argument (float)\n\nReturns\n-------\nK : ndarray\n    Value of the exponentially scaled modified Bessel function K of order\n    1 at `x`.\n\nNotes\n-----\nThe range is partitioned into the two intervals [0, 2] and (2, infinity).\nChebyshev polynomial expansions are employed in each interval.\n\nThis function is a wrapper for the Cephes [1]_ routine `k1e`.\n\nSee also\n--------\nkv\nk1\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":9368
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_k1e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_k1e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k1e_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":9369
 *     "       http://www.netlib.org/cephes/")
 * ufunc_k1e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_k1e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_k1e_types[0] = <char>NPY_FLOAT
 * ufunc_k1e_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":9370
 * ufunc_k1e_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_k1e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k1e_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_k1e_types[1] = <char>NPY_FLOAT
 * ufunc_k1e_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9371
 * ufunc_k1e_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_k1e_types[0] = <char>NPY_FLOAT
 * ufunc_k1e_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_k1e_types[2] = <char>NPY_DOUBLE
 * ufunc_k1e_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9372
 * ufunc_k1e_types[0] = <char>NPY_FLOAT
 * ufunc_k1e_types[1] = <char>NPY_FLOAT
 * ufunc_k1e_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_k1e_types[3] = <char>NPY_DOUBLE
 * ufunc_k1e_ptr[2*0] = <void*>_func_k1e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9373
 * ufunc_k1e_types[1] = <char>NPY_FLOAT
 * ufunc_k1e_types[2] = <char>NPY_DOUBLE
 * ufunc_k1e_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_k1e_ptr[2*0] = <void*>_func_k1e
 * ufunc_k1e_ptr[2*0+1] = <void*>(<char*>"k1e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9374
 * ufunc_k1e_types[2] = <char>NPY_DOUBLE
 * ufunc_k1e_types[3] = <char>NPY_DOUBLE
 * ufunc_k1e_ptr[2*0] = <void*>_func_k1e             # <<<<<<<<<<<<<<
 * ufunc_k1e_ptr[2*0+1] = <void*>(<char*>"k1e")
 * ufunc_k1e_ptr[2*1] = <void*>_func_k1e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_ptr[0]) = ((void *)k1e);

  /* "scipy/special/_ufuncs.pyx":9375
 * ufunc_k1e_types[3] = <char>NPY_DOUBLE
 * ufunc_k1e_ptr[2*0] = <void*>_func_k1e
 * ufunc_k1e_ptr[2*0+1] = <void*>(<char*>"k1e")             # <<<<<<<<<<<<<<
 * ufunc_k1e_ptr[2*1] = <void*>_func_k1e
 * ufunc_k1e_ptr[2*1+1] = <void*>(<char*>"k1e")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_ptr[1]) = ((void *)((char *)((char *)"k1e")));

  /* "scipy/special/_ufuncs.pyx":9376
 * ufunc_k1e_ptr[2*0] = <void*>_func_k1e
 * ufunc_k1e_ptr[2*0+1] = <void*>(<char*>"k1e")
 * ufunc_k1e_ptr[2*1] = <void*>_func_k1e             # <<<<<<<<<<<<<<
 * ufunc_k1e_ptr[2*1+1] = <void*>(<char*>"k1e")
 * ufunc_k1e_data[0] = &ufunc_k1e_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_ptr[2]) = ((void *)k1e);

  /* "scipy/special/_ufuncs.pyx":9377
 * ufunc_k1e_ptr[2*0+1] = <void*>(<char*>"k1e")
 * ufunc_k1e_ptr[2*1] = <void*>_func_k1e
 * ufunc_k1e_ptr[2*1+1] = <void*>(<char*>"k1e")             # <<<<<<<<<<<<<<
 * ufunc_k1e_data[0] = &ufunc_k1e_ptr[2*0]
 * ufunc_k1e_data[1] = &ufunc_k1e_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_ptr[3]) = ((void *)((char *)((char *)"k1e")));

  /* "scipy/special/_ufuncs.pyx":9378
 * ufunc_k1e_ptr[2*1] = <void*>_func_k1e
 * ufunc_k1e_ptr[2*1+1] = <void*>(<char*>"k1e")
 * ufunc_k1e_data[0] = &ufunc_k1e_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_k1e_data[1] = &ufunc_k1e_ptr[2*1]
 * k1e = np.PyUFunc_FromFuncAndData(ufunc_k1e_loops, ufunc_k1e_data, ufunc_k1e_types, 2, 1, 1, 0, "k1e", ufunc_k1e_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9379
 * ufunc_k1e_ptr[2*1+1] = <void*>(<char*>"k1e")
 * ufunc_k1e_data[0] = &ufunc_k1e_ptr[2*0]
 * ufunc_k1e_data[1] = &ufunc_k1e_ptr[2*1]             # <<<<<<<<<<<<<<
 * k1e = np.PyUFunc_FromFuncAndData(ufunc_k1e_loops, ufunc_k1e_data, ufunc_k1e_types, 2, 1, 1, 0, "k1e", ufunc_k1e_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9380
 * ufunc_k1e_data[0] = &ufunc_k1e_ptr[2*0]
 * ufunc_k1e_data[1] = &ufunc_k1e_ptr[2*1]
 * k1e = np.PyUFunc_FromFuncAndData(ufunc_k1e_loops, ufunc_k1e_data, ufunc_k1e_types, 2, 1, 1, 0, "k1e", ufunc_k1e_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_kei_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_types, 2, 1, 1, 0, ((char *)"k1e"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_k1e_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9380, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_k1e, __pyx_t_2) < 0) __PYX_ERR(1, 9380, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":9386
 * cdef void *ufunc_kei_data[2]
 * cdef char ufunc_kei_types[4]
 * cdef char *ufunc_kei_doc = (             # <<<<<<<<<<<<<<
 *     "kei(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_doc = ((char *)"kei(x)\n\nKelvin function ker");

  /* "scipy/special/_ufuncs.pyx":9390
 *     "\n"
 *     "Kelvin function ker")
 * ufunc_kei_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_kei_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kei_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":9391
 *     "Kelvin function ker")
 * ufunc_kei_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_kei_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_kei_types[0] = <char>NPY_FLOAT
 * ufunc_kei_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":9392
 * ufunc_kei_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_kei_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kei_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kei_types[1] = <char>NPY_FLOAT
 * ufunc_kei_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9393
 * ufunc_kei_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kei_types[0] = <char>NPY_FLOAT
 * ufunc_kei_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kei_types[2] = <char>NPY_DOUBLE
 * ufunc_kei_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9394
 * ufunc_kei_types[0] = <char>NPY_FLOAT
 * ufunc_kei_types[1] = <char>NPY_FLOAT
 * ufunc_kei_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kei_types[3] = <char>NPY_DOUBLE
 * ufunc_kei_ptr[2*0] = <void*>_func_kei_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9395
 * ufunc_kei_types[1] = <char>NPY_FLOAT
 * ufunc_kei_types[2] = <char>NPY_DOUBLE
 * ufunc_kei_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kei_ptr[2*0] = <void*>_func_kei_wrap
 * ufunc_kei_ptr[2*0+1] = <void*>(<char*>"kei")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9396
 * ufunc_kei_types[2] = <char>NPY_DOUBLE
 * ufunc_kei_types[3] = <char>NPY_DOUBLE
 * ufunc_kei_ptr[2*0] = <void*>_func_kei_wrap             # <<<<<<<<<<<<<<
 * ufunc_kei_ptr[2*0+1] = <void*>(<char*>"kei")
 * ufunc_kei_ptr[2*1] = <void*>_func_kei_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_ptr[0]) = ((void *)kei_wrap);

  /* "scipy/special/_ufuncs.pyx":9397
 * ufunc_kei_types[3] = <char>NPY_DOUBLE
 * ufunc_kei_ptr[2*0] = <void*>_func_kei_wrap
 * ufunc_kei_ptr[2*0+1] = <void*>(<char*>"kei")             # <<<<<<<<<<<<<<
 * ufunc_kei_ptr[2*1] = <void*>_func_kei_wrap
 * ufunc_kei_ptr[2*1+1] = <void*>(<char*>"kei")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_ptr[1]) = ((void *)((char *)((char *)"kei")));

  /* "scipy/special/_ufuncs.pyx":9398
 * ufunc_kei_ptr[2*0] = <void*>_func_kei_wrap
 * ufunc_kei_ptr[2*0+1] = <void*>(<char*>"kei")
 * ufunc_kei_ptr[2*1] = <void*>_func_kei_wrap             # <<<<<<<<<<<<<<
 * ufunc_kei_ptr[2*1+1] = <void*>(<char*>"kei")
 * ufunc_kei_data[0] = &ufunc_kei_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_ptr[2]) = ((void *)kei_wrap);

  /* "scipy/special/_ufuncs.pyx":9399
 * ufunc_kei_ptr[2*0+1] = <void*>(<char*>"kei")
 * ufunc_kei_ptr[2*1] = <void*>_func_kei_wrap
 * ufunc_kei_ptr[2*1+1] = <void*>(<char*>"kei")             # <<<<<<<<<<<<<<
 * ufunc_kei_data[0] = &ufunc_kei_ptr[2*0]
 * ufunc_kei_data[1] = &ufunc_kei_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_ptr[3]) = ((void *)((char *)((char *)"kei")));

  /* "scipy/special/_ufuncs.pyx":9400
 * ufunc_kei_ptr[2*1] = <void*>_func_kei_wrap
 * ufunc_kei_ptr[2*1+1] = <void*>(<char*>"kei")
 * ufunc_kei_data[0] = &ufunc_kei_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_kei_data[1] = &ufunc_kei_ptr[2*1]
 * kei = np.PyUFunc_FromFuncAndData(ufunc_kei_loops, ufunc_kei_data, ufunc_kei_types, 2, 1, 1, 0, "kei", ufunc_kei_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9401
 * ufunc_kei_ptr[2*1+1] = <void*>(<char*>"kei")
 * ufunc_kei_data[0] = &ufunc_kei_ptr[2*0]
 * ufunc_kei_data[1] = &ufunc_kei_ptr[2*1]             # <<<<<<<<<<<<<<
 * kei = np.PyUFunc_FromFuncAndData(ufunc_kei_loops, ufunc_kei_data, ufunc_kei_types, 2, 1, 1, 0, "kei", ufunc_kei_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9402
 * ufunc_kei_data[0] = &ufunc_kei_ptr[2*0]
 * ufunc_kei_data[1] = &ufunc_kei_ptr[2*1]
 * kei = np.PyUFunc_FromFuncAndData(ufunc_kei_loops, ufunc_kei_data, ufunc_kei_types, 2, 1, 1, 0, "kei", ufunc_kei_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_keip_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_types, 2, 1, 1, 0, ((char *)"kei"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_kei_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kei, __pyx_t_2) < 0) __PYX_ERR(1, 9402, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":9408
 * cdef void *ufunc_keip_data[2]
 * cdef char ufunc_keip_types[4]
 * cdef char *ufunc_keip_doc = (             # <<<<<<<<<<<<<<
 *     "keip(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_doc = ((char *)"keip(x)\n\nDerivative of the Kelvin function kei");

  /* "scipy/special/_ufuncs.pyx":9412
 *     "\n"
 *     "Derivative of the Kelvin function kei")
 * ufunc_keip_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_keip_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_keip_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":9413
 *     "Derivative of the Kelvin function kei")
 * ufunc_keip_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_keip_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_keip_types[0] = <char>NPY_FLOAT
 * ufunc_keip_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":9414
 * ufunc_keip_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_keip_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_keip_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_keip_types[1] = <char>NPY_FLOAT
 * ufunc_keip_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9415
 * ufunc_keip_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_keip_types[0] = <char>NPY_FLOAT
 * ufunc_keip_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_keip_types[2] = <char>NPY_DOUBLE
 * ufunc_keip_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9416
 * ufunc_keip_types[0] = <char>NPY_FLOAT
 * ufunc_keip_types[1] = <char>NPY_FLOAT
 * ufunc_keip_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_keip_types[3] = <char>NPY_DOUBLE
 * ufunc_keip_ptr[2*0] = <void*>_func_keip_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9417
 * ufunc_keip_types[1] = <char>NPY_FLOAT
 * ufunc_keip_types[2] = <char>NPY_DOUBLE
 * ufunc_keip_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_keip_ptr[2*0] = <void*>_func_keip_wrap
 * ufunc_keip_ptr[2*0+1] = <void*>(<char*>"keip")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9418
 * ufunc_keip_types[2] = <char>NPY_DOUBLE
 * ufunc_keip_types[3] = <char>NPY_DOUBLE
 * ufunc_keip_ptr[2*0] = <void*>_func_keip_wrap             # <<<<<<<<<<<<<<
 * ufunc_keip_ptr[2*0+1] = <void*>(<char*>"keip")
 * ufunc_keip_ptr[2*1] = <void*>_func_keip_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_ptr[0]) = ((void *)keip_wrap);

  /* "scipy/special/_ufuncs.pyx":9419
 * ufunc_keip_types[3] = <char>NPY_DOUBLE
 * ufunc_keip_ptr[2*0] = <void*>_func_keip_wrap
 * ufunc_keip_ptr[2*0+1] = <void*>(<char*>"keip")             # <<<<<<<<<<<<<<
 * ufunc_keip_ptr[2*1] = <void*>_func_keip_wrap
 * ufunc_keip_ptr[2*1+1] = <void*>(<char*>"keip")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_ptr[1]) = ((void *)((char *)((char *)"keip")));

  /* "scipy/special/_ufuncs.pyx":9420
 * ufunc_keip_ptr[2*0] = <void*>_func_keip_wrap
 * ufunc_keip_ptr[2*0+1] = <void*>(<char*>"keip")
 * ufunc_keip_ptr[2*1] = <void*>_func_keip_wrap             # <<<<<<<<<<<<<<
 * ufunc_keip_ptr[2*1+1] = <void*>(<char*>"keip")
 * ufunc_keip_data[0] = &ufunc_keip_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_ptr[2]) = ((void *)keip_wrap);

  /* "scipy/special/_ufuncs.pyx":9421
 * ufunc_keip_ptr[2*0+1] = <void*>(<char*>"keip")
 * ufunc_keip_ptr[2*1] = <void*>_func_keip_wrap
 * ufunc_keip_ptr[2*1+1] = <void*>(<char*>"keip")             # <<<<<<<<<<<<<<
 * ufunc_keip_data[0] = &ufunc_keip_ptr[2*0]
 * ufunc_keip_data[1] = &ufunc_keip_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_ptr[3]) = ((void *)((char *)((char *)"keip")));

  /* "scipy/special/_ufuncs.pyx":9422
 * ufunc_keip_ptr[2*1] = <void*>_func_keip_wrap
 * ufunc_keip_ptr[2*1+1] = <void*>(<char*>"keip")
 * ufunc_keip_data[0] = &ufunc_keip_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_keip_data[1] = &ufunc_keip_ptr[2*1]
 * keip = np.PyUFunc_FromFuncAndData(ufunc_keip_loops, ufunc_keip_data, ufunc_keip_types, 2, 1, 1, 0, "keip", ufunc_keip_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9423
 * ufunc_keip_ptr[2*1+1] = <void*>(<char*>"keip")
 * ufunc_keip_data[0] = &ufunc_keip_ptr[2*0]
 * ufunc_keip_data[1] = &ufunc_keip_ptr[2*1]             # <<<<<<<<<<<<<<
 * keip = np.PyUFunc_FromFuncAndData(ufunc_keip_loops, ufunc_keip_data, ufunc_keip_types, 2, 1, 1, 0, "keip", ufunc_keip_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9424
 * ufunc_keip_data[0] = &ufunc_keip_ptr[2*0]
 * ufunc_keip_data[1] = &ufunc_keip_ptr[2*1]
 * keip = np.PyUFunc_FromFuncAndData(ufunc_keip_loops, ufunc_keip_data, ufunc_keip_types, 2, 1, 1, 0, "keip", ufunc_keip_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_kelvin_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_types, 2, 1, 1, 0, ((char *)"keip"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_keip_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9424, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_keip, __pyx_t_2) < 0) __PYX_ERR(1, 9424, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":9430
 * cdef void *ufunc_kelvin_data[2]
 * cdef char ufunc_kelvin_types[10]
 * cdef char *ufunc_kelvin_doc = (             # <<<<<<<<<<<<<<
 *     "kelvin(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_doc = ((char *)"kelvin(x)\n\nKelvin functions as complex numbers\n\nReturns\n-------\nBe, Ke, Bep, Kep\n    The tuple (Be, Ke, Bep, Kep) contains complex numbers\n    representing the real and imaginary Kelvin functions and their\n    derivatives evaluated at `x`.  For example, kelvin(x)[0].real =\n    ber x and kelvin(x)[0].imag = bei x with similar relationships\n    for ker and kei.");

  /* "scipy/special/_ufuncs.pyx":9443
 *     "    ber x and kelvin(x)[0].imag = bei x with similar relationships\n"
 *     "    for ker and kei.")
 * ufunc_kelvin_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_DDDD_As_f_FFFF             # <<<<<<<<<<<<<<
 * ufunc_kelvin_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DDDD_As_d_DDDD
 * ufunc_kelvin_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DDDD_As_f_FFFF);

  /* "scipy/special/_ufuncs.pyx":9444
 *     "    for ker and kei.")
 * ufunc_kelvin_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_DDDD_As_f_FFFF
 * ufunc_kelvin_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DDDD_As_d_DDDD             # <<<<<<<<<<<<<<
 * ufunc_kelvin_types[0] = <char>NPY_FLOAT
 * ufunc_kelvin_types[1] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DDDD_As_d_DDDD);

  /* "scipy/special/_ufuncs.pyx":9445
 * ufunc_kelvin_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_DDDD_As_f_FFFF
 * ufunc_kelvin_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DDDD_As_d_DDDD
 * ufunc_kelvin_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kelvin_types[1] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[2] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9446
 * ufunc_kelvin_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DDDD_As_d_DDDD
 * ufunc_kelvin_types[0] = <char>NPY_FLOAT
 * ufunc_kelvin_types[1] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_kelvin_types[2] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[3] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[1]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9447
 * ufunc_kelvin_types[0] = <char>NPY_FLOAT
 * ufunc_kelvin_types[1] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[2] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_kelvin_types[3] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[2]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9448
 * ufunc_kelvin_types[1] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[2] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[3] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_kelvin_types[4] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[3]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9449
 * ufunc_kelvin_types[2] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[3] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_kelvin_types[5] = <char>NPY_DOUBLE
 * ufunc_kelvin_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9450
 * ufunc_kelvin_types[3] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[4] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kelvin_types[6] = <char>NPY_CDOUBLE
 * ufunc_kelvin_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9451
 * ufunc_kelvin_types[4] = <char>NPY_CFLOAT
 * ufunc_kelvin_types[5] = <char>NPY_DOUBLE
 * ufunc_kelvin_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kelvin_types[7] = <char>NPY_CDOUBLE
 * ufunc_kelvin_types[8] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9452
 * ufunc_kelvin_types[5] = <char>NPY_DOUBLE
 * ufunc_kelvin_types[6] = <char>NPY_CDOUBLE
 * ufunc_kelvin_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kelvin_types[8] = <char>NPY_CDOUBLE
 * ufunc_kelvin_types[9] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9453
 * ufunc_kelvin_types[6] = <char>NPY_CDOUBLE
 * ufunc_kelvin_types[7] = <char>NPY_CDOUBLE
 * ufunc_kelvin_types[8] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kelvin_types[9] = <char>NPY_CDOUBLE
 * ufunc_kelvin_ptr[2*0] = <void*>_func_kelvin_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[8]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9454
 * ufunc_kelvin_types[7] = <char>NPY_CDOUBLE
 * ufunc_kelvin_types[8] = <char>NPY_CDOUBLE
 * ufunc_kelvin_types[9] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kelvin_ptr[2*0] = <void*>_func_kelvin_wrap
 * ufunc_kelvin_ptr[2*0+1] = <void*>(<char*>"kelvin")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types[9]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9455
 * ufunc_kelvin_types[8] = <char>NPY_CDOUBLE
 * ufunc_kelvin_types[9] = <char>NPY_CDOUBLE
 * ufunc_kelvin_ptr[2*0] = <void*>_func_kelvin_wrap             # <<<<<<<<<<<<<<
 * ufunc_kelvin_ptr[2*0+1] = <void*>(<char*>"kelvin")
 * ufunc_kelvin_ptr[2*1] = <void*>_func_kelvin_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_ptr[0]) = ((void *)kelvin_wrap);

  /* "scipy/special/_ufuncs.pyx":9456
 * ufunc_kelvin_types[9] = <char>NPY_CDOUBLE
 * ufunc_kelvin_ptr[2*0] = <void*>_func_kelvin_wrap
 * ufunc_kelvin_ptr[2*0+1] = <void*>(<char*>"kelvin")             # <<<<<<<<<<<<<<
 * ufunc_kelvin_ptr[2*1] = <void*>_func_kelvin_wrap
 * ufunc_kelvin_ptr[2*1+1] = <void*>(<char*>"kelvin")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_ptr[1]) = ((void *)((char *)((char *)"kelvin")));

  /* "scipy/special/_ufuncs.pyx":9457
 * ufunc_kelvin_ptr[2*0] = <void*>_func_kelvin_wrap
 * ufunc_kelvin_ptr[2*0+1] = <void*>(<char*>"kelvin")
 * ufunc_kelvin_ptr[2*1] = <void*>_func_kelvin_wrap             # <<<<<<<<<<<<<<
 * ufunc_kelvin_ptr[2*1+1] = <void*>(<char*>"kelvin")
 * ufunc_kelvin_data[0] = &ufunc_kelvin_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_ptr[2]) = ((void *)kelvin_wrap);

  /* "scipy/special/_ufuncs.pyx":9458
 * ufunc_kelvin_ptr[2*0+1] = <void*>(<char*>"kelvin")
 * ufunc_kelvin_ptr[2*1] = <void*>_func_kelvin_wrap
 * ufunc_kelvin_ptr[2*1+1] = <void*>(<char*>"kelvin")             # <<<<<<<<<<<<<<
 * ufunc_kelvin_data[0] = &ufunc_kelvin_ptr[2*0]
 * ufunc_kelvin_data[1] = &ufunc_kelvin_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_ptr[3]) = ((void *)((char *)((char *)"kelvin")));

  /* "scipy/special/_ufuncs.pyx":9459
 * ufunc_kelvin_ptr[2*1] = <void*>_func_kelvin_wrap
 * ufunc_kelvin_ptr[2*1+1] = <void*>(<char*>"kelvin")
 * ufunc_kelvin_data[0] = &ufunc_kelvin_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_kelvin_data[1] = &ufunc_kelvin_ptr[2*1]
 * kelvin = np.PyUFunc_FromFuncAndData(ufunc_kelvin_loops, ufunc_kelvin_data, ufunc_kelvin_types, 2, 1, 4, 0, "kelvin", ufunc_kelvin_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9460
 * ufunc_kelvin_ptr[2*1+1] = <void*>(<char*>"kelvin")
 * ufunc_kelvin_data[0] = &ufunc_kelvin_ptr[2*0]
 * ufunc_kelvin_data[1] = &ufunc_kelvin_ptr[2*1]             # <<<<<<<<<<<<<<
 * kelvin = np.PyUFunc_FromFuncAndData(ufunc_kelvin_loops, ufunc_kelvin_data, ufunc_kelvin_types, 2, 1, 4, 0, "kelvin", ufunc_kelvin_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9461
 * ufunc_kelvin_data[0] = &ufunc_kelvin_ptr[2*0]
 * ufunc_kelvin_data[1] = &ufunc_kelvin_ptr[2*1]
 * kelvin = np.PyUFunc_FromFuncAndData(ufunc_kelvin_loops, ufunc_kelvin_data, ufunc_kelvin_types, 2, 1, 4, 0, "kelvin", ufunc_kelvin_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ker_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_types, 2, 1, 4, 0, ((char *)"kelvin"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_kelvin_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kelvin, __pyx_t_2) < 0) __PYX_ERR(1, 9461, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":9467
 * cdef void *ufunc_ker_data[2]
 * cdef char ufunc_ker_types[4]
 * cdef char *ufunc_ker_doc = (             # <<<<<<<<<<<<<<
 *     "ker(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_doc = ((char *)"ker(x)\n\nKelvin function ker");

  /* "scipy/special/_ufuncs.pyx":9471
 *     "\n"
 *     "Kelvin function ker")
 * ufunc_ker_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_ker_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ker_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":9472
 *     "Kelvin function ker")
 * ufunc_ker_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ker_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_ker_types[0] = <char>NPY_FLOAT
 * ufunc_ker_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":9473
 * ufunc_ker_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ker_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ker_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ker_types[1] = <char>NPY_FLOAT
 * ufunc_ker_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9474
 * ufunc_ker_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ker_types[0] = <char>NPY_FLOAT
 * ufunc_ker_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ker_types[2] = <char>NPY_DOUBLE
 * ufunc_ker_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9475
 * ufunc_ker_types[0] = <char>NPY_FLOAT
 * ufunc_ker_types[1] = <char>NPY_FLOAT
 * ufunc_ker_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ker_types[3] = <char>NPY_DOUBLE
 * ufunc_ker_ptr[2*0] = <void*>_func_ker_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9476
 * ufunc_ker_types[1] = <char>NPY_FLOAT
 * ufunc_ker_types[2] = <char>NPY_DOUBLE
 * ufunc_ker_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ker_ptr[2*0] = <void*>_func_ker_wrap
 * ufunc_ker_ptr[2*0+1] = <void*>(<char*>"ker")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9477
 * ufunc_ker_types[2] = <char>NPY_DOUBLE
 * ufunc_ker_types[3] = <char>NPY_DOUBLE
 * ufunc_ker_ptr[2*0] = <void*>_func_ker_wrap             # <<<<<<<<<<<<<<
 * ufunc_ker_ptr[2*0+1] = <void*>(<char*>"ker")
 * ufunc_ker_ptr[2*1] = <void*>_func_ker_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_ptr[0]) = ((void *)ker_wrap);

  /* "scipy/special/_ufuncs.pyx":9478
 * ufunc_ker_types[3] = <char>NPY_DOUBLE
 * ufunc_ker_ptr[2*0] = <void*>_func_ker_wrap
 * ufunc_ker_ptr[2*0+1] = <void*>(<char*>"ker")             # <<<<<<<<<<<<<<
 * ufunc_ker_ptr[2*1] = <void*>_func_ker_wrap
 * ufunc_ker_ptr[2*1+1] = <void*>(<char*>"ker")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_ptr[1]) = ((void *)((char *)((char *)"ker")));

  /* "scipy/special/_ufuncs.pyx":9479
 * ufunc_ker_ptr[2*0] = <void*>_func_ker_wrap
 * ufunc_ker_ptr[2*0+1] = <void*>(<char*>"ker")
 * ufunc_ker_ptr[2*1] = <void*>_func_ker_wrap             # <<<<<<<<<<<<<<
 * ufunc_ker_ptr[2*1+1] = <void*>(<char*>"ker")
 * ufunc_ker_data[0] = &ufunc_ker_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_ptr[2]) = ((void *)ker_wrap);

  /* "scipy/special/_ufuncs.pyx":9480
 * ufunc_ker_ptr[2*0+1] = <void*>(<char*>"ker")
 * ufunc_ker_ptr[2*1] = <void*>_func_ker_wrap
 * ufunc_ker_ptr[2*1+1] = <void*>(<char*>"ker")             # <<<<<<<<<<<<<<
 * ufunc_ker_data[0] = &ufunc_ker_ptr[2*0]
 * ufunc_ker_data[1] = &ufunc_ker_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_ptr[3]) = ((void *)((char *)((char *)"ker")));

  /* "scipy/special/_ufuncs.pyx":9481
 * ufunc_ker_ptr[2*1] = <void*>_func_ker_wrap
 * ufunc_ker_ptr[2*1+1] = <void*>(<char*>"ker")
 * ufunc_ker_data[0] = &ufunc_ker_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ker_data[1] = &ufunc_ker_ptr[2*1]
 * ker = np.PyUFunc_FromFuncAndData(ufunc_ker_loops, ufunc_ker_data, ufunc_ker_types, 2, 1, 1, 0, "ker", ufunc_ker_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9482
 * ufunc_ker_ptr[2*1+1] = <void*>(<char*>"ker")
 * ufunc_ker_data[0] = &ufunc_ker_ptr[2*0]
 * ufunc_ker_data[1] = &ufunc_ker_ptr[2*1]             # <<<<<<<<<<<<<<
 * ker = np.PyUFunc_FromFuncAndData(ufunc_ker_loops, ufunc_ker_data, ufunc_ker_types, 2, 1, 1, 0, "ker", ufunc_ker_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9483
 * ufunc_ker_data[0] = &ufunc_ker_ptr[2*0]
 * ufunc_ker_data[1] = &ufunc_ker_ptr[2*1]
 * ker = np.PyUFunc_FromFuncAndData(ufunc_ker_loops, ufunc_ker_data, ufunc_ker_types, 2, 1, 1, 0, "ker", ufunc_ker_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_kerp_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_types, 2, 1, 1, 0, ((char *)"ker"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_ker_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ker, __pyx_t_2) < 0) __PYX_ERR(1, 9483, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":9489
 * cdef void *ufunc_kerp_data[2]
 * cdef char ufunc_kerp_types[4]
 * cdef char *ufunc_kerp_doc = (             # <<<<<<<<<<<<<<
 *     "kerp(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_doc = ((char *)"kerp(x)\n\nDerivative of the Kelvin function ker");

  /* "scipy/special/_ufuncs.pyx":9493
 *     "\n"
 *     "Derivative of the Kelvin function ker")
 * ufunc_kerp_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_kerp_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kerp_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":9494
 *     "Derivative of the Kelvin function ker")
 * ufunc_kerp_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_kerp_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_kerp_types[0] = <char>NPY_FLOAT
 * ufunc_kerp_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":9495
 * ufunc_kerp_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_kerp_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kerp_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kerp_types[1] = <char>NPY_FLOAT
 * ufunc_kerp_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9496
 * ufunc_kerp_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kerp_types[0] = <char>NPY_FLOAT
 * ufunc_kerp_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kerp_types[2] = <char>NPY_DOUBLE
 * ufunc_kerp_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9497
 * ufunc_kerp_types[0] = <char>NPY_FLOAT
 * ufunc_kerp_types[1] = <char>NPY_FLOAT
 * ufunc_kerp_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kerp_types[3] = <char>NPY_DOUBLE
 * ufunc_kerp_ptr[2*0] = <void*>_func_kerp_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9498
 * ufunc_kerp_types[1] = <char>NPY_FLOAT
 * ufunc_kerp_types[2] = <char>NPY_DOUBLE
 * ufunc_kerp_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kerp_ptr[2*0] = <void*>_func_kerp_wrap
 * ufunc_kerp_ptr[2*0+1] = <void*>(<char*>"kerp")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9499
 * ufunc_kerp_types[2] = <char>NPY_DOUBLE
 * ufunc_kerp_types[3] = <char>NPY_DOUBLE
 * ufunc_kerp_ptr[2*0] = <void*>_func_kerp_wrap             # <<<<<<<<<<<<<<
 * ufunc_kerp_ptr[2*0+1] = <void*>(<char*>"kerp")
 * ufunc_kerp_ptr[2*1] = <void*>_func_kerp_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_ptr[0]) = ((void *)kerp_wrap);

  /* "scipy/special/_ufuncs.pyx":9500
 * ufunc_kerp_types[3] = <char>NPY_DOUBLE
 * ufunc_kerp_ptr[2*0] = <void*>_func_kerp_wrap
 * ufunc_kerp_ptr[2*0+1] = <void*>(<char*>"kerp")             # <<<<<<<<<<<<<<
 * ufunc_kerp_ptr[2*1] = <void*>_func_kerp_wrap
 * ufunc_kerp_ptr[2*1+1] = <void*>(<char*>"kerp")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_ptr[1]) = ((void *)((char *)((char *)"kerp")));

  /* "scipy/special/_ufuncs.pyx":9501
 * ufunc_kerp_ptr[2*0] = <void*>_func_kerp_wrap
 * ufunc_kerp_ptr[2*0+1] = <void*>(<char*>"kerp")
 * ufunc_kerp_ptr[2*1] = <void*>_func_kerp_wrap             # <<<<<<<<<<<<<<
 * ufunc_kerp_ptr[2*1+1] = <void*>(<char*>"kerp")
 * ufunc_kerp_data[0] = &ufunc_kerp_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_ptr[2]) = ((void *)kerp_wrap);

  /* "scipy/special/_ufuncs.pyx":9502
 * ufunc_kerp_ptr[2*0+1] = <void*>(<char*>"kerp")
 * ufunc_kerp_ptr[2*1] = <void*>_func_kerp_wrap
 * ufunc_kerp_ptr[2*1+1] = <void*>(<char*>"kerp")             # <<<<<<<<<<<<<<
 * ufunc_kerp_data[0] = &ufunc_kerp_ptr[2*0]
 * ufunc_kerp_data[1] = &ufunc_kerp_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_ptr[3]) = ((void *)((char *)((char *)"kerp")));

  /* "scipy/special/_ufuncs.pyx":9503
 * ufunc_kerp_ptr[2*1] = <void*>_func_kerp_wrap
 * ufunc_kerp_ptr[2*1+1] = <void*>(<char*>"kerp")
 * ufunc_kerp_data[0] = &ufunc_kerp_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_kerp_data[1] = &ufunc_kerp_ptr[2*1]
 * kerp = np.PyUFunc_FromFuncAndData(ufunc_kerp_loops, ufunc_kerp_data, ufunc_kerp_types, 2, 1, 1, 0, "kerp", ufunc_kerp_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9504
 * ufunc_kerp_ptr[2*1+1] = <void*>(<char*>"kerp")
 * ufunc_kerp_data[0] = &ufunc_kerp_ptr[2*0]
 * ufunc_kerp_data[1] = &ufunc_kerp_ptr[2*1]             # <<<<<<<<<<<<<<
 * kerp = np.PyUFunc_FromFuncAndData(ufunc_kerp_loops, ufunc_kerp_data, ufunc_kerp_types, 2, 1, 1, 0, "kerp", ufunc_kerp_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9505
 * ufunc_kerp_data[0] = &ufunc_kerp_ptr[2*0]
 * ufunc_kerp_data[1] = &ufunc_kerp_ptr[2*1]
 * kerp = np.PyUFunc_FromFuncAndData(ufunc_kerp_loops, ufunc_kerp_data, ufunc_kerp_types, 2, 1, 1, 0, "kerp", ufunc_kerp_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_kl_div_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_types, 2, 1, 1, 0, ((char *)"kerp"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_kerp_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9505, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kerp, __pyx_t_2) < 0) __PYX_ERR(1, 9505, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":9511
 * cdef void *ufunc_kl_div_data[2]
 * cdef char ufunc_kl_div_types[6]
 * cdef char *ufunc_kl_div_doc = (             # <<<<<<<<<<<<<<
 *     "kl_div(x, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_doc = ((char *)"kl_div(x, y)\n\nElementwise function for computing Kullback-Leibler divergence.\n\n.. math:: \\mathrm{kl\\_div}(x, y) = \\begin{cases} x \\log(x / y) - x + y & x > 0, y > 0 \\\\ y & x = 0, y \\ge 0 \\\\ \\infty & \\text{otherwise} \\end{cases}\n\nParameters\n----------\nx : ndarray\n    First input array.\ny : ndarray\n    Second input array.\n\nReturns\n-------\nres : ndarray\n    Output array.\n\nSee Also\n--------\nentr, rel_entr\n\nNotes\n-----\nThis function is non-negative and is jointly convex in `x` and `y`.\n\n.. versionadded:: 0.15.0");

  /* "scipy/special/_ufuncs.pyx":9539
 *     "\n"
 *     ".. versionadded:: 0.15.0")
 * ufunc_kl_div_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_kl_div_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kl_div_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":9540
 *     ".. versionadded:: 0.15.0")
 * ufunc_kl_div_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_kl_div_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_kl_div_types[0] = <char>NPY_FLOAT
 * ufunc_kl_div_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":9541
 * ufunc_kl_div_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_kl_div_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kl_div_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kl_div_types[1] = <char>NPY_FLOAT
 * ufunc_kl_div_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9542
 * ufunc_kl_div_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kl_div_types[0] = <char>NPY_FLOAT
 * ufunc_kl_div_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kl_div_types[2] = <char>NPY_FLOAT
 * ufunc_kl_div_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9543
 * ufunc_kl_div_types[0] = <char>NPY_FLOAT
 * ufunc_kl_div_types[1] = <char>NPY_FLOAT
 * ufunc_kl_div_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kl_div_types[3] = <char>NPY_DOUBLE
 * ufunc_kl_div_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9544
 * ufunc_kl_div_types[1] = <char>NPY_FLOAT
 * ufunc_kl_div_types[2] = <char>NPY_FLOAT
 * ufunc_kl_div_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kl_div_types[4] = <char>NPY_DOUBLE
 * ufunc_kl_div_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9545
 * ufunc_kl_div_types[2] = <char>NPY_FLOAT
 * ufunc_kl_div_types[3] = <char>NPY_DOUBLE
 * ufunc_kl_div_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kl_div_types[5] = <char>NPY_DOUBLE
 * ufunc_kl_div_ptr[2*0] = <void*>_func_kl_div
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9546
 * ufunc_kl_div_types[3] = <char>NPY_DOUBLE
 * ufunc_kl_div_types[4] = <char>NPY_DOUBLE
 * ufunc_kl_div_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kl_div_ptr[2*0] = <void*>_func_kl_div
 * ufunc_kl_div_ptr[2*0+1] = <void*>(<char*>"kl_div")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9547
 * ufunc_kl_div_types[4] = <char>NPY_DOUBLE
 * ufunc_kl_div_types[5] = <char>NPY_DOUBLE
 * ufunc_kl_div_ptr[2*0] = <void*>_func_kl_div             # <<<<<<<<<<<<<<
 * ufunc_kl_div_ptr[2*0+1] = <void*>(<char*>"kl_div")
 * ufunc_kl_div_ptr[2*1] = <void*>_func_kl_div
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_ptr[0]) = ((void *)__pyx_f_5scipy_7special_16_convex_analysis_kl_div);

  /* "scipy/special/_ufuncs.pyx":9548
 * ufunc_kl_div_types[5] = <char>NPY_DOUBLE
 * ufunc_kl_div_ptr[2*0] = <void*>_func_kl_div
 * ufunc_kl_div_ptr[2*0+1] = <void*>(<char*>"kl_div")             # <<<<<<<<<<<<<<
 * ufunc_kl_div_ptr[2*1] = <void*>_func_kl_div
 * ufunc_kl_div_ptr[2*1+1] = <void*>(<char*>"kl_div")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_ptr[1]) = ((void *)((char *)((char *)"kl_div")));

  /* "scipy/special/_ufuncs.pyx":9549
 * ufunc_kl_div_ptr[2*0] = <void*>_func_kl_div
 * ufunc_kl_div_ptr[2*0+1] = <void*>(<char*>"kl_div")
 * ufunc_kl_div_ptr[2*1] = <void*>_func_kl_div             # <<<<<<<<<<<<<<
 * ufunc_kl_div_ptr[2*1+1] = <void*>(<char*>"kl_div")
 * ufunc_kl_div_data[0] = &ufunc_kl_div_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_ptr[2]) = ((void *)__pyx_f_5scipy_7special_16_convex_analysis_kl_div);

  /* "scipy/special/_ufuncs.pyx":9550
 * ufunc_kl_div_ptr[2*0+1] = <void*>(<char*>"kl_div")
 * ufunc_kl_div_ptr[2*1] = <void*>_func_kl_div
 * ufunc_kl_div_ptr[2*1+1] = <void*>(<char*>"kl_div")             # <<<<<<<<<<<<<<
 * ufunc_kl_div_data[0] = &ufunc_kl_div_ptr[2*0]
 * ufunc_kl_div_data[1] = &ufunc_kl_div_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_ptr[3]) = ((void *)((char *)((char *)"kl_div")));

  /* "scipy/special/_ufuncs.pyx":9551
 * ufunc_kl_div_ptr[2*1] = <void*>_func_kl_div
 * ufunc_kl_div_ptr[2*1+1] = <void*>(<char*>"kl_div")
 * ufunc_kl_div_data[0] = &ufunc_kl_div_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_kl_div_data[1] = &ufunc_kl_div_ptr[2*1]
 * kl_div = np.PyUFunc_FromFuncAndData(ufunc_kl_div_loops, ufunc_kl_div_data, ufunc_kl_div_types, 2, 2, 1, 0, "kl_div", ufunc_kl_div_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9552
 * ufunc_kl_div_ptr[2*1+1] = <void*>(<char*>"kl_div")
 * ufunc_kl_div_data[0] = &ufunc_kl_div_ptr[2*0]
 * ufunc_kl_div_data[1] = &ufunc_kl_div_ptr[2*1]             # <<<<<<<<<<<<<<
 * kl_div = np.PyUFunc_FromFuncAndData(ufunc_kl_div_loops, ufunc_kl_div_data, ufunc_kl_div_types, 2, 2, 1, 0, "kl_div", ufunc_kl_div_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9553
 * ufunc_kl_div_data[0] = &ufunc_kl_div_ptr[2*0]
 * ufunc_kl_div_data[1] = &ufunc_kl_div_ptr[2*1]
 * kl_div = np.PyUFunc_FromFuncAndData(ufunc_kl_div_loops, ufunc_kl_div_data, ufunc_kl_div_types, 2, 2, 1, 0, "kl_div", ufunc_kl_div_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_kn_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_types, 2, 2, 1, 0, ((char *)"kl_div"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_kl_div_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kl_div, __pyx_t_2) < 0) __PYX_ERR(1, 9553, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":9559
 * cdef void *ufunc_kn_data[3]
 * cdef char ufunc_kn_types[9]
 * cdef char *ufunc_kn_doc = (             # <<<<<<<<<<<<<<
 *     "kn(n, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_doc = ((char *)"kn(n, x)\n\nModified Bessel function of the second kind of integer order `n`\n\nReturns the modified Bessel function of the second kind for integer order\n`n` at real `z`.\n\nThese are also sometimes called functions of the third kind, Basset\nfunctions, or Macdonald functions.\n\nParameters\n----------\nn : array_like of int\n    Order of Bessel functions (floats will truncate with a warning)\nz : array_like of float\n    Argument at which to evaluate the Bessel functions\n\nReturns\n-------\nout : ndarray\n    The results\n\nNotes\n-----\nWrapper for AMOS [1]_ routine `zbesk`.  For a discussion of the\nalgorithm used, see [2]_ and the references therein.\n\nSee Also\n--------\nkv : Same function, but accepts real order and complex argument\nkvp : Derivative of this function\n\nReferences\n----------\n.. [1] Donald E. Amos, \"AMOS, A Portable Package for Bessel Functions\n       of a Complex Argument and Nonnegative Order\",\n       http://netlib.org/amos/\n.. [2] Donald E. Amos, \"Algorithm 644: A portable package for Bessel\n       functions of a complex argument and nonnegative order\", ACM\n       TOMS Vol. 12 Issue 3, Sept. 1986, p. 265\n\nExamples\n--------\nPlot the function of several orders for real input:\n\n>>> from scipy.special import kn\n>>> import matplotlib.pyplot as plt\n>>> x = np.linspace(0, 5, 1000)\n>>> for N in range(6):\n...     plt.plot(x, kn(N, x), label='$K_{}(x)$'.format(N))\n>>> plt.ylim(0, 10)\n>>> plt.legend()\n>>> plt.title(r'Modified Bessel function of the second kind $K_n(x)$')\n>>> plt.show()\n\nCalculate for a single value at multiple orders:\n\n>>> kn([4, 5, 6], 1)\narray([   44.23241585,   360.9605896 ,  3653.83831186])");

  /* "scipy/special/_ufuncs.pyx":9619
 *     ">>> kn([4, 5, 6], 1)\n"
 *     "array([   44.23241585,   360.9605896 ,  3653.83831186])")
 * ufunc_kn_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_kn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_kn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":9620
 *     "array([   44.23241585,   360.9605896 ,  3653.83831186])")
 * ufunc_kn_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_kn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_kn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kn_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":9621
 * ufunc_kn_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_kn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_kn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_kn_types[0] = <char>NPY_LONG
 * ufunc_kn_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":9622
 * ufunc_kn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_kn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kn_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_kn_types[1] = <char>NPY_DOUBLE
 * ufunc_kn_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":9623
 * ufunc_kn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kn_types[0] = <char>NPY_LONG
 * ufunc_kn_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kn_types[2] = <char>NPY_DOUBLE
 * ufunc_kn_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9624
 * ufunc_kn_types[0] = <char>NPY_LONG
 * ufunc_kn_types[1] = <char>NPY_DOUBLE
 * ufunc_kn_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kn_types[3] = <char>NPY_FLOAT
 * ufunc_kn_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9625
 * ufunc_kn_types[1] = <char>NPY_DOUBLE
 * ufunc_kn_types[2] = <char>NPY_DOUBLE
 * ufunc_kn_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kn_types[4] = <char>NPY_FLOAT
 * ufunc_kn_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9626
 * ufunc_kn_types[2] = <char>NPY_DOUBLE
 * ufunc_kn_types[3] = <char>NPY_FLOAT
 * ufunc_kn_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kn_types[5] = <char>NPY_FLOAT
 * ufunc_kn_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9627
 * ufunc_kn_types[3] = <char>NPY_FLOAT
 * ufunc_kn_types[4] = <char>NPY_FLOAT
 * ufunc_kn_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kn_types[6] = <char>NPY_DOUBLE
 * ufunc_kn_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9628
 * ufunc_kn_types[4] = <char>NPY_FLOAT
 * ufunc_kn_types[5] = <char>NPY_FLOAT
 * ufunc_kn_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kn_types[7] = <char>NPY_DOUBLE
 * ufunc_kn_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9629
 * ufunc_kn_types[5] = <char>NPY_FLOAT
 * ufunc_kn_types[6] = <char>NPY_DOUBLE
 * ufunc_kn_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kn_types[8] = <char>NPY_DOUBLE
 * ufunc_kn_ptr[2*0] = <void*>_func_cbesk_wrap_real_int
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9630
 * ufunc_kn_types[6] = <char>NPY_DOUBLE
 * ufunc_kn_types[7] = <char>NPY_DOUBLE
 * ufunc_kn_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kn_ptr[2*0] = <void*>_func_cbesk_wrap_real_int
 * ufunc_kn_ptr[2*0+1] = <void*>(<char*>"kn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9631
 * ufunc_kn_types[7] = <char>NPY_DOUBLE
 * ufunc_kn_types[8] = <char>NPY_DOUBLE
 * ufunc_kn_ptr[2*0] = <void*>_func_cbesk_wrap_real_int             # <<<<<<<<<<<<<<
 * ufunc_kn_ptr[2*0+1] = <void*>(<char*>"kn")
 * ufunc_kn_ptr[2*1] = <void*>_func_kn_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_ptr[0]) = ((void *)cbesk_wrap_real_int);

  /* "scipy/special/_ufuncs.pyx":9632
 * ufunc_kn_types[8] = <char>NPY_DOUBLE
 * ufunc_kn_ptr[2*0] = <void*>_func_cbesk_wrap_real_int
 * ufunc_kn_ptr[2*0+1] = <void*>(<char*>"kn")             # <<<<<<<<<<<<<<
 * ufunc_kn_ptr[2*1] = <void*>_func_kn_unsafe
 * ufunc_kn_ptr[2*1+1] = <void*>(<char*>"kn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_ptr[1]) = ((void *)((char *)((char *)"kn")));

  /* "scipy/special/_ufuncs.pyx":9633
 * ufunc_kn_ptr[2*0] = <void*>_func_cbesk_wrap_real_int
 * ufunc_kn_ptr[2*0+1] = <void*>(<char*>"kn")
 * ufunc_kn_ptr[2*1] = <void*>_func_kn_unsafe             # <<<<<<<<<<<<<<
 * ufunc_kn_ptr[2*1+1] = <void*>(<char*>"kn")
 * ufunc_kn_ptr[2*2] = <void*>_func_kn_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_kn_unsafe);

  /* "scipy/special/_ufuncs.pyx":9634
 * ufunc_kn_ptr[2*0+1] = <void*>(<char*>"kn")
 * ufunc_kn_ptr[2*1] = <void*>_func_kn_unsafe
 * ufunc_kn_ptr[2*1+1] = <void*>(<char*>"kn")             # <<<<<<<<<<<<<<
 * ufunc_kn_ptr[2*2] = <void*>_func_kn_unsafe
 * ufunc_kn_ptr[2*2+1] = <void*>(<char*>"kn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_ptr[3]) = ((void *)((char *)((char *)"kn")));

  /* "scipy/special/_ufuncs.pyx":9635
 * ufunc_kn_ptr[2*1] = <void*>_func_kn_unsafe
 * ufunc_kn_ptr[2*1+1] = <void*>(<char*>"kn")
 * ufunc_kn_ptr[2*2] = <void*>_func_kn_unsafe             # <<<<<<<<<<<<<<
 * ufunc_kn_ptr[2*2+1] = <void*>(<char*>"kn")
 * ufunc_kn_data[0] = &ufunc_kn_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_kn_unsafe);

  /* "scipy/special/_ufuncs.pyx":9636
 * ufunc_kn_ptr[2*1+1] = <void*>(<char*>"kn")
 * ufunc_kn_ptr[2*2] = <void*>_func_kn_unsafe
 * ufunc_kn_ptr[2*2+1] = <void*>(<char*>"kn")             # <<<<<<<<<<<<<<
 * ufunc_kn_data[0] = &ufunc_kn_ptr[2*0]
 * ufunc_kn_data[1] = &ufunc_kn_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_ptr[5]) = ((void *)((char *)((char *)"kn")));

  /* "scipy/special/_ufuncs.pyx":9637
 * ufunc_kn_ptr[2*2] = <void*>_func_kn_unsafe
 * ufunc_kn_ptr[2*2+1] = <void*>(<char*>"kn")
 * ufunc_kn_data[0] = &ufunc_kn_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_kn_data[1] = &ufunc_kn_ptr[2*1]
 * ufunc_kn_data[2] = &ufunc_kn_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9638
 * ufunc_kn_ptr[2*2+1] = <void*>(<char*>"kn")
 * ufunc_kn_data[0] = &ufunc_kn_ptr[2*0]
 * ufunc_kn_data[1] = &ufunc_kn_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_kn_data[2] = &ufunc_kn_ptr[2*2]
 * kn = np.PyUFunc_FromFuncAndData(ufunc_kn_loops, ufunc_kn_data, ufunc_kn_types, 3, 2, 1, 0, "kn", ufunc_kn_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9639
 * ufunc_kn_data[0] = &ufunc_kn_ptr[2*0]
 * ufunc_kn_data[1] = &ufunc_kn_ptr[2*1]
 * ufunc_kn_data[2] = &ufunc_kn_ptr[2*2]             # <<<<<<<<<<<<<<
 * kn = np.PyUFunc_FromFuncAndData(ufunc_kn_loops, ufunc_kn_data, ufunc_kn_types, 3, 2, 1, 0, "kn", ufunc_kn_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":9640
 * ufunc_kn_data[1] = &ufunc_kn_ptr[2*1]
 * ufunc_kn_data[2] = &ufunc_kn_ptr[2*2]
 * kn = np.PyUFunc_FromFuncAndData(ufunc_kn_loops, ufunc_kn_data, ufunc_kn_types, 3, 2, 1, 0, "kn", ufunc_kn_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_kolmogi_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_types, 3, 2, 1, 0, ((char *)"kn"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_kn_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9640, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kn, __pyx_t_2) < 0) __PYX_ERR(1, 9640, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":9646
 * cdef void *ufunc_kolmogi_data[2]
 * cdef char ufunc_kolmogi_types[4]
 * cdef char *ufunc_kolmogi_doc = (             # <<<<<<<<<<<<<<
 *     "kolmogi(p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_doc = ((char *)"kolmogi(p)\n\nInverse Survival Function of Kolmogorov distribution\n\nIt is the inverse function to `kolmogorov`.\nReturns y such that ``kolmogorov(y) == p``.\n\nParameters\n----------\np : float array_like\n    Probability\n\nReturns\n-------\nfloat\n    The value(s) of kolmogi(p)\n\nNotes\n-----\n`kolmogorov` is used by `stats.kstest` in the application of the\nKolmogorov-Smirnov Goodness of Fit test. For historial reasons this\nfunction is exposed in `scpy.special`, but the recommended way to achieve\nthe most accurate CDF/SF/PDF/PPF/ISF computations is to use the\n`stats.kstwobign` distribution.\n\nSee Also\n--------\nkolmogorov : The Survival Function for the distribution\nscipy.stats.kstwobign : Provides the functionality as a continuous distribution\nsmirnov, smirnovi : Functions for the one-sided distribution\n\nExamples\n--------\n>>> from scipy.special import kolmogi\n>>> kolmogi([0, 0.1, 0.25, 0.5, 0.75, 0.9, 1.0])\narray([        inf,  1.22384787,  1.01918472,  0.82757356,  0.67644769,\n        0.57117327,  0.        ])");

  /* "scipy/special/_ufuncs.pyx":9684
 *     "array([        inf,  1.22384787,  1.01918472,  0.82757356,  0.67644769,\n"
 *     "        0.57117327,  0.        ])")
 * ufunc_kolmogi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kolmogi_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":9685
 *     "        0.57117327,  0.        ])")
 * ufunc_kolmogi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_kolmogi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_types[0] = <char>NPY_FLOAT
 * ufunc_kolmogi_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":9686
 * ufunc_kolmogi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_kolmogi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kolmogi_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_types[1] = <char>NPY_FLOAT
 * ufunc_kolmogi_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9687
 * ufunc_kolmogi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kolmogi_types[0] = <char>NPY_FLOAT
 * ufunc_kolmogi_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_types[2] = <char>NPY_DOUBLE
 * ufunc_kolmogi_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9688
 * ufunc_kolmogi_types[0] = <char>NPY_FLOAT
 * ufunc_kolmogi_types[1] = <char>NPY_FLOAT
 * ufunc_kolmogi_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_types[3] = <char>NPY_DOUBLE
 * ufunc_kolmogi_ptr[2*0] = <void*>_func_kolmogi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9689
 * ufunc_kolmogi_types[1] = <char>NPY_FLOAT
 * ufunc_kolmogi_types[2] = <char>NPY_DOUBLE
 * ufunc_kolmogi_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_ptr[2*0] = <void*>_func_kolmogi
 * ufunc_kolmogi_ptr[2*0+1] = <void*>(<char*>"kolmogi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9690
 * ufunc_kolmogi_types[2] = <char>NPY_DOUBLE
 * ufunc_kolmogi_types[3] = <char>NPY_DOUBLE
 * ufunc_kolmogi_ptr[2*0] = <void*>_func_kolmogi             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_ptr[2*0+1] = <void*>(<char*>"kolmogi")
 * ufunc_kolmogi_ptr[2*1] = <void*>_func_kolmogi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_ptr[0]) = ((void *)kolmogi);

  /* "scipy/special/_ufuncs.pyx":9691
 * ufunc_kolmogi_types[3] = <char>NPY_DOUBLE
 * ufunc_kolmogi_ptr[2*0] = <void*>_func_kolmogi
 * ufunc_kolmogi_ptr[2*0+1] = <void*>(<char*>"kolmogi")             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_ptr[2*1] = <void*>_func_kolmogi
 * ufunc_kolmogi_ptr[2*1+1] = <void*>(<char*>"kolmogi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_ptr[1]) = ((void *)((char *)((char *)"kolmogi")));

  /* "scipy/special/_ufuncs.pyx":9692
 * ufunc_kolmogi_ptr[2*0] = <void*>_func_kolmogi
 * ufunc_kolmogi_ptr[2*0+1] = <void*>(<char*>"kolmogi")
 * ufunc_kolmogi_ptr[2*1] = <void*>_func_kolmogi             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_ptr[2*1+1] = <void*>(<char*>"kolmogi")
 * ufunc_kolmogi_data[0] = &ufunc_kolmogi_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_ptr[2]) = ((void *)kolmogi);

  /* "scipy/special/_ufuncs.pyx":9693
 * ufunc_kolmogi_ptr[2*0+1] = <void*>(<char*>"kolmogi")
 * ufunc_kolmogi_ptr[2*1] = <void*>_func_kolmogi
 * ufunc_kolmogi_ptr[2*1+1] = <void*>(<char*>"kolmogi")             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_data[0] = &ufunc_kolmogi_ptr[2*0]
 * ufunc_kolmogi_data[1] = &ufunc_kolmogi_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_ptr[3]) = ((void *)((char *)((char *)"kolmogi")));

  /* "scipy/special/_ufuncs.pyx":9694
 * ufunc_kolmogi_ptr[2*1] = <void*>_func_kolmogi
 * ufunc_kolmogi_ptr[2*1+1] = <void*>(<char*>"kolmogi")
 * ufunc_kolmogi_data[0] = &ufunc_kolmogi_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_kolmogi_data[1] = &ufunc_kolmogi_ptr[2*1]
 * kolmogi = np.PyUFunc_FromFuncAndData(ufunc_kolmogi_loops, ufunc_kolmogi_data, ufunc_kolmogi_types, 2, 1, 1, 0, "kolmogi", ufunc_kolmogi_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9695
 * ufunc_kolmogi_ptr[2*1+1] = <void*>(<char*>"kolmogi")
 * ufunc_kolmogi_data[0] = &ufunc_kolmogi_ptr[2*0]
 * ufunc_kolmogi_data[1] = &ufunc_kolmogi_ptr[2*1]             # <<<<<<<<<<<<<<
 * kolmogi = np.PyUFunc_FromFuncAndData(ufunc_kolmogi_loops, ufunc_kolmogi_data, ufunc_kolmogi_types, 2, 1, 1, 0, "kolmogi", ufunc_kolmogi_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9696
 * ufunc_kolmogi_data[0] = &ufunc_kolmogi_ptr[2*0]
 * ufunc_kolmogi_data[1] = &ufunc_kolmogi_ptr[2*1]
 * kolmogi = np.PyUFunc_FromFuncAndData(ufunc_kolmogi_loops, ufunc_kolmogi_data, ufunc_kolmogi_types, 2, 1, 1, 0, "kolmogi", ufunc_kolmogi_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_kolmogorov_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_types, 2, 1, 1, 0, ((char *)"kolmogi"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogi_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9696, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kolmogi, __pyx_t_2) < 0) __PYX_ERR(1, 9696, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":9702
 * cdef void *ufunc_kolmogorov_data[2]
 * cdef char ufunc_kolmogorov_types[4]
 * cdef char *ufunc_kolmogorov_doc = (             # <<<<<<<<<<<<<<
 *     "kolmogorov(y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_doc = ((char *)"kolmogorov(y)\n\nComplementary cumulative distribution (Survival Function) function of\nKolmogorov distribution.\n\nReturns the complementary cumulative distribution function of\nKolmogorov's limiting distribution (``D_n*\\sqrt(n)`` as n goes to infinity)\nof a two-sided test for equality between an empirical and a theoretical\ndistribution. It is equal to the (limit as n->infinity of the)\nprobability that ``sqrt(n) * max absolute deviation > y``.\n\nParameters\n----------\ny : float array_like\n  Absolute deviation between the Empirical CDF (ECDF) and the target CDF,\n  multiplied by sqrt(n).\n\nReturns\n-------\nfloat\n    The value(s) of kolmogorov(y)\n\nNotes\n-----\n`kolmogorov` is used by `stats.kstest` in the application of the\nKolmogorov-Smirnov Goodness of Fit test. For historial reasons this\nfunction is exposed in `scpy.special`, but the recommended way to achieve\nthe most accurate CDF/SF/PDF/PPF/ISF computations is to use the\n`stats.kstwobign` distrubution.\n\nSee Also\n--------\nkolmogi : The Inverse Survival Function for the distribution\nscipy.stats.kstwobign : Provides the functionality as a continuous distribution\nsmirnov, smirnovi : Functions for the one-sided distribution\n\nExamples\n--------\nShow the probability of a gap at least as big as 0, 0.5 and 1.0.\n\n>>> from scipy.special import kolmogorov\n>>> from scipy.stats import kstwobign\n>>> kolmogorov([0, 0.5, 1.0])\narray([ 1.        ,  0.96394524,  0.26999967])\n\nCompare a sample of size 1000 drawn from a Laplace(0, 1) distribution against\nthe target distribution, a Normal(0, 1) distribution.\n\n>>> from scipy.stats import norm, laplace\n>>> n = 1000\n>>> np.random.seed(seed=233423)\n>>> lap01 = laplace(0, 1)\n>>> x = np.sort(lap01.rvs(n))\n>>> np.mean(x), np.std(x)\n(-0.083073685397609842, 1.3676426568399822)\n\nConstruct the Empirical CDF and the K-S statistic Dn.\n\n>>> target = norm(0,1)  # Normal mean 0, stddev 1\n>>> cdfs = target.cdf(x)\n>>> ecdfs = np.arange(n+1, dtype=float)/""n\n>>> gaps = np.column_stack([cdfs - ecdfs[:n], ecdfs[1:] - cdfs])\n>>> Dn = np.max(gaps)\n>>> Kn = np.sqrt(n) * Dn\n>>> print('Dn=%f, sqrt(n)*Dn=%f' % (Dn, Kn))\nDn=0.058286, sqrt(n)*Dn=1.843153\n>>> print(chr(10).join(['For a sample of size n drawn from a N(0, 1) distribution:',\n...   ' the approximate Kolmogorov probability that sqrt(n)*Dn>=%f is %f' %  (Kn, kolmogorov(Kn)),\n...   ' the approximate Kolmogorov probability that sqrt(n)*Dn<=%f is %f' %  (Kn, kstwobign.cdf(Kn))]))\nFor a sample of size n drawn from a N(0, 1) distribution:\n the approximate Kolmogorov probability that sqrt(n)*Dn>=1.843153 is 0.002240\n the approximate Kolmogorov probability that sqrt(n)*Dn<=1.843153 is 0.997760\n\nPlot the Empirical CDF against the target N(0, 1) CDF.\n\n>>> import matplotlib.pyplot as plt\n>>> plt.step(np.concatenate([[-3], x]), ecdfs, where='post', label='Empirical CDF')\n>>> x3 = np.linspace(-3, 3, 100)\n>>> plt.plot(x3, target.cdf(x3), label='CDF for N(0, 1)')\n>>> plt.ylim([0, 1]); plt.grid(True); plt.legend();\n>>> # Add vertical lines marking Dn+ and Dn-\n>>> iminus, iplus = np.argmax(gaps, axis=0)\n>>> plt.vlines([x[iminus]], ecdfs[iminus], cdfs[iminus], color='r', linestyle='dashed', lw=4)\n>>> plt.vlines([x[iplus]], cdfs[iplus], ecdfs[iplus+1], color='r', linestyle='dashed', lw=4)\n>>> plt.show()");

  /* "scipy/special/_ufuncs.pyx":9788
 *     ">>> plt.vlines([x[iplus]], cdfs[iplus], ecdfs[iplus+1], color='r', linestyle='dashed', lw=4)\n"
 *     ">>> plt.show()")
 * ufunc_kolmogorov_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kolmogorov_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":9789
 *     ">>> plt.show()")
 * ufunc_kolmogorov_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_kolmogorov_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_types[0] = <char>NPY_FLOAT
 * ufunc_kolmogorov_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":9790
 * ufunc_kolmogorov_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_kolmogorov_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kolmogorov_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_types[1] = <char>NPY_FLOAT
 * ufunc_kolmogorov_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9791
 * ufunc_kolmogorov_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_kolmogorov_types[0] = <char>NPY_FLOAT
 * ufunc_kolmogorov_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_types[2] = <char>NPY_DOUBLE
 * ufunc_kolmogorov_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9792
 * ufunc_kolmogorov_types[0] = <char>NPY_FLOAT
 * ufunc_kolmogorov_types[1] = <char>NPY_FLOAT
 * ufunc_kolmogorov_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_types[3] = <char>NPY_DOUBLE
 * ufunc_kolmogorov_ptr[2*0] = <void*>_func_kolmogorov
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9793
 * ufunc_kolmogorov_types[1] = <char>NPY_FLOAT
 * ufunc_kolmogorov_types[2] = <char>NPY_DOUBLE
 * ufunc_kolmogorov_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_ptr[2*0] = <void*>_func_kolmogorov
 * ufunc_kolmogorov_ptr[2*0+1] = <void*>(<char*>"kolmogorov")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9794
 * ufunc_kolmogorov_types[2] = <char>NPY_DOUBLE
 * ufunc_kolmogorov_types[3] = <char>NPY_DOUBLE
 * ufunc_kolmogorov_ptr[2*0] = <void*>_func_kolmogorov             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_ptr[2*0+1] = <void*>(<char*>"kolmogorov")
 * ufunc_kolmogorov_ptr[2*1] = <void*>_func_kolmogorov
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_ptr[0]) = ((void *)kolmogorov);

  /* "scipy/special/_ufuncs.pyx":9795
 * ufunc_kolmogorov_types[3] = <char>NPY_DOUBLE
 * ufunc_kolmogorov_ptr[2*0] = <void*>_func_kolmogorov
 * ufunc_kolmogorov_ptr[2*0+1] = <void*>(<char*>"kolmogorov")             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_ptr[2*1] = <void*>_func_kolmogorov
 * ufunc_kolmogorov_ptr[2*1+1] = <void*>(<char*>"kolmogorov")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_ptr[1]) = ((void *)((char *)((char *)"kolmogorov")));

  /* "scipy/special/_ufuncs.pyx":9796
 * ufunc_kolmogorov_ptr[2*0] = <void*>_func_kolmogorov
 * ufunc_kolmogorov_ptr[2*0+1] = <void*>(<char*>"kolmogorov")
 * ufunc_kolmogorov_ptr[2*1] = <void*>_func_kolmogorov             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_ptr[2*1+1] = <void*>(<char*>"kolmogorov")
 * ufunc_kolmogorov_data[0] = &ufunc_kolmogorov_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_ptr[2]) = ((void *)kolmogorov);

  /* "scipy/special/_ufuncs.pyx":9797
 * ufunc_kolmogorov_ptr[2*0+1] = <void*>(<char*>"kolmogorov")
 * ufunc_kolmogorov_ptr[2*1] = <void*>_func_kolmogorov
 * ufunc_kolmogorov_ptr[2*1+1] = <void*>(<char*>"kolmogorov")             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_data[0] = &ufunc_kolmogorov_ptr[2*0]
 * ufunc_kolmogorov_data[1] = &ufunc_kolmogorov_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_ptr[3]) = ((void *)((char *)((char *)"kolmogorov")));

  /* "scipy/special/_ufuncs.pyx":9798
 * ufunc_kolmogorov_ptr[2*1] = <void*>_func_kolmogorov
 * ufunc_kolmogorov_ptr[2*1+1] = <void*>(<char*>"kolmogorov")
 * ufunc_kolmogorov_data[0] = &ufunc_kolmogorov_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_kolmogorov_data[1] = &ufunc_kolmogorov_ptr[2*1]
 * kolmogorov = np.PyUFunc_FromFuncAndData(ufunc_kolmogorov_loops, ufunc_kolmogorov_data, ufunc_kolmogorov_types, 2, 1, 1, 0, "kolmogorov", ufunc_kolmogorov_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9799
 * ufunc_kolmogorov_ptr[2*1+1] = <void*>(<char*>"kolmogorov")
 * ufunc_kolmogorov_data[0] = &ufunc_kolmogorov_ptr[2*0]
 * ufunc_kolmogorov_data[1] = &ufunc_kolmogorov_ptr[2*1]             # <<<<<<<<<<<<<<
 * kolmogorov = np.PyUFunc_FromFuncAndData(ufunc_kolmogorov_loops, ufunc_kolmogorov_data, ufunc_kolmogorov_types, 2, 1, 1, 0, "kolmogorov", ufunc_kolmogorov_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9800
 * ufunc_kolmogorov_data[0] = &ufunc_kolmogorov_ptr[2*0]
 * ufunc_kolmogorov_data[1] = &ufunc_kolmogorov_ptr[2*1]
 * kolmogorov = np.PyUFunc_FromFuncAndData(ufunc_kolmogorov_loops, ufunc_kolmogorov_data, ufunc_kolmogorov_types, 2, 1, 1, 0, "kolmogorov", ufunc_kolmogorov_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_kv_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_types, 2, 1, 1, 0, ((char *)"kolmogorov"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_kolmogorov_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9800, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kolmogorov, __pyx_t_2) < 0) __PYX_ERR(1, 9800, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":9806
 * cdef void *ufunc_kv_data[4]
 * cdef char ufunc_kv_types[12]
 * cdef char *ufunc_kv_doc = (             # <<<<<<<<<<<<<<
 *     "kv(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_doc = ((char *)"kv(v, z)\n\nModified Bessel function of the second kind of real order `v`\n\nReturns the modified Bessel function of the second kind for real order\n`v` at complex `z`.\n\nThese are also sometimes called functions of the third kind, Basset\nfunctions, or Macdonald functions.  They are defined as those solutions\nof the modified Bessel equation for which,\n\n.. math::\n    K_v(x) \\sim \\sqrt{\\pi/(2x)} \\exp(-x)\n\nas :math:`x \\to \\infty` [3]_.\n\nParameters\n----------\nv : array_like of float\n    Order of Bessel functions\nz : array_like of complex\n    Argument at which to evaluate the Bessel functions\n\nReturns\n-------\nout : ndarray\n    The results. Note that input must be of complex type to get complex\n    output, e.g. ``kv(3, -2+0j)`` instead of ``kv(3, -2)``.\n\nNotes\n-----\nWrapper for AMOS [1]_ routine `zbesk`.  For a discussion of the\nalgorithm used, see [2]_ and the references therein.\n\nSee Also\n--------\nkve : This function with leading exponential behavior stripped off.\nkvp : Derivative of this function\n\nReferences\n----------\n.. [1] Donald E. Amos, \"AMOS, A Portable Package for Bessel Functions\n       of a Complex Argument and Nonnegative Order\",\n       http://netlib.org/amos/\n.. [2] Donald E. Amos, \"Algorithm 644: A portable package for Bessel\n       functions of a complex argument and nonnegative order\", ACM\n       TOMS Vol. 12 Issue 3, Sept. 1986, p. 265\n.. [3] NIST Digital Library of Mathematical Functions,\n       Eq. 10.25.E3. https://dlmf.nist.gov/10.25.E3\n\nExamples\n--------\nPlot the function of several orders for real input:\n\n>>> from scipy.special import kv\n>>> import matplotlib.pyplot as plt\n>>> x = np.linspace(0, 5, 1000)\n>>> for N in np.linspace(0, 6, 5):\n...     plt.plot(x, kv(N, x), label='$K_{{{}}}(x)$'.format(N))\n>>> plt.ylim(0, 10)\n>>> plt.legend()\n>>> plt.title(r'Modified Bessel function of the second kind $K_\\nu(x)$')\n>>> plt.show()\n\nCalculate for a single value at multiple orders:\n\n>>> k""v([4, 4.5, 5], 1+2j)\narray([ 0.1992+2.3892j,  2.3493+3.6j   ,  7.2827+3.8104j])");

  /* "scipy/special/_ufuncs.pyx":9875
 *     ">>> kv([4, 4.5, 5], 1+2j)\n"
 *     "array([ 0.1992+2.3892j,  2.3493+3.6j   ,  7.2827+3.8104j])")
 * ufunc_kv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_kv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_kv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":9876
 *     "array([ 0.1992+2.3892j,  2.3493+3.6j   ,  7.2827+3.8104j])")
 * ufunc_kv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_kv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_kv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":9877
 * ufunc_kv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_kv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_kv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_kv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_kv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":9878
 * ufunc_kv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_kv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_kv_types[0] = <char>NPY_FLOAT
 * ufunc_kv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":9879
 * ufunc_kv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_kv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kv_types[1] = <char>NPY_FLOAT
 * ufunc_kv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9880
 * ufunc_kv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_kv_types[0] = <char>NPY_FLOAT
 * ufunc_kv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kv_types[2] = <char>NPY_FLOAT
 * ufunc_kv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9881
 * ufunc_kv_types[0] = <char>NPY_FLOAT
 * ufunc_kv_types[1] = <char>NPY_FLOAT
 * ufunc_kv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kv_types[3] = <char>NPY_FLOAT
 * ufunc_kv_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9882
 * ufunc_kv_types[1] = <char>NPY_FLOAT
 * ufunc_kv_types[2] = <char>NPY_FLOAT
 * ufunc_kv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kv_types[4] = <char>NPY_CFLOAT
 * ufunc_kv_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9883
 * ufunc_kv_types[2] = <char>NPY_FLOAT
 * ufunc_kv_types[3] = <char>NPY_FLOAT
 * ufunc_kv_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_kv_types[5] = <char>NPY_CFLOAT
 * ufunc_kv_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9884
 * ufunc_kv_types[3] = <char>NPY_FLOAT
 * ufunc_kv_types[4] = <char>NPY_CFLOAT
 * ufunc_kv_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_kv_types[6] = <char>NPY_DOUBLE
 * ufunc_kv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9885
 * ufunc_kv_types[4] = <char>NPY_CFLOAT
 * ufunc_kv_types[5] = <char>NPY_CFLOAT
 * ufunc_kv_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kv_types[7] = <char>NPY_DOUBLE
 * ufunc_kv_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9886
 * ufunc_kv_types[5] = <char>NPY_CFLOAT
 * ufunc_kv_types[6] = <char>NPY_DOUBLE
 * ufunc_kv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kv_types[8] = <char>NPY_DOUBLE
 * ufunc_kv_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9887
 * ufunc_kv_types[6] = <char>NPY_DOUBLE
 * ufunc_kv_types[7] = <char>NPY_DOUBLE
 * ufunc_kv_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kv_types[9] = <char>NPY_DOUBLE
 * ufunc_kv_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9888
 * ufunc_kv_types[7] = <char>NPY_DOUBLE
 * ufunc_kv_types[8] = <char>NPY_DOUBLE
 * ufunc_kv_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kv_types[10] = <char>NPY_CDOUBLE
 * ufunc_kv_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9889
 * ufunc_kv_types[8] = <char>NPY_DOUBLE
 * ufunc_kv_types[9] = <char>NPY_DOUBLE
 * ufunc_kv_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kv_types[11] = <char>NPY_CDOUBLE
 * ufunc_kv_ptr[2*0] = <void*>_func_cbesk_wrap_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9890
 * ufunc_kv_types[9] = <char>NPY_DOUBLE
 * ufunc_kv_types[10] = <char>NPY_CDOUBLE
 * ufunc_kv_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kv_ptr[2*0] = <void*>_func_cbesk_wrap_real
 * ufunc_kv_ptr[2*0+1] = <void*>(<char*>"kv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9891
 * ufunc_kv_types[10] = <char>NPY_CDOUBLE
 * ufunc_kv_types[11] = <char>NPY_CDOUBLE
 * ufunc_kv_ptr[2*0] = <void*>_func_cbesk_wrap_real             # <<<<<<<<<<<<<<
 * ufunc_kv_ptr[2*0+1] = <void*>(<char*>"kv")
 * ufunc_kv_ptr[2*1] = <void*>_func_cbesk_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[0]) = ((void *)cbesk_wrap_real);

  /* "scipy/special/_ufuncs.pyx":9892
 * ufunc_kv_types[11] = <char>NPY_CDOUBLE
 * ufunc_kv_ptr[2*0] = <void*>_func_cbesk_wrap_real
 * ufunc_kv_ptr[2*0+1] = <void*>(<char*>"kv")             # <<<<<<<<<<<<<<
 * ufunc_kv_ptr[2*1] = <void*>_func_cbesk_wrap
 * ufunc_kv_ptr[2*1+1] = <void*>(<char*>"kv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[1]) = ((void *)((char *)((char *)"kv")));

  /* "scipy/special/_ufuncs.pyx":9893
 * ufunc_kv_ptr[2*0] = <void*>_func_cbesk_wrap_real
 * ufunc_kv_ptr[2*0+1] = <void*>(<char*>"kv")
 * ufunc_kv_ptr[2*1] = <void*>_func_cbesk_wrap             # <<<<<<<<<<<<<<
 * ufunc_kv_ptr[2*1+1] = <void*>(<char*>"kv")
 * ufunc_kv_ptr[2*2] = <void*>_func_cbesk_wrap_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[2]) = ((void *)cbesk_wrap);

  /* "scipy/special/_ufuncs.pyx":9894
 * ufunc_kv_ptr[2*0+1] = <void*>(<char*>"kv")
 * ufunc_kv_ptr[2*1] = <void*>_func_cbesk_wrap
 * ufunc_kv_ptr[2*1+1] = <void*>(<char*>"kv")             # <<<<<<<<<<<<<<
 * ufunc_kv_ptr[2*2] = <void*>_func_cbesk_wrap_real
 * ufunc_kv_ptr[2*2+1] = <void*>(<char*>"kv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[3]) = ((void *)((char *)((char *)"kv")));

  /* "scipy/special/_ufuncs.pyx":9895
 * ufunc_kv_ptr[2*1] = <void*>_func_cbesk_wrap
 * ufunc_kv_ptr[2*1+1] = <void*>(<char*>"kv")
 * ufunc_kv_ptr[2*2] = <void*>_func_cbesk_wrap_real             # <<<<<<<<<<<<<<
 * ufunc_kv_ptr[2*2+1] = <void*>(<char*>"kv")
 * ufunc_kv_ptr[2*3] = <void*>_func_cbesk_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[4]) = ((void *)cbesk_wrap_real);

  /* "scipy/special/_ufuncs.pyx":9896
 * ufunc_kv_ptr[2*1+1] = <void*>(<char*>"kv")
 * ufunc_kv_ptr[2*2] = <void*>_func_cbesk_wrap_real
 * ufunc_kv_ptr[2*2+1] = <void*>(<char*>"kv")             # <<<<<<<<<<<<<<
 * ufunc_kv_ptr[2*3] = <void*>_func_cbesk_wrap
 * ufunc_kv_ptr[2*3+1] = <void*>(<char*>"kv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[5]) = ((void *)((char *)((char *)"kv")));

  /* "scipy/special/_ufuncs.pyx":9897
 * ufunc_kv_ptr[2*2] = <void*>_func_cbesk_wrap_real
 * ufunc_kv_ptr[2*2+1] = <void*>(<char*>"kv")
 * ufunc_kv_ptr[2*3] = <void*>_func_cbesk_wrap             # <<<<<<<<<<<<<<
 * ufunc_kv_ptr[2*3+1] = <void*>(<char*>"kv")
 * ufunc_kv_data[0] = &ufunc_kv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[6]) = ((void *)cbesk_wrap);

  /* "scipy/special/_ufuncs.pyx":9898
 * ufunc_kv_ptr[2*2+1] = <void*>(<char*>"kv")
 * ufunc_kv_ptr[2*3] = <void*>_func_cbesk_wrap
 * ufunc_kv_ptr[2*3+1] = <void*>(<char*>"kv")             # <<<<<<<<<<<<<<
 * ufunc_kv_data[0] = &ufunc_kv_ptr[2*0]
 * ufunc_kv_data[1] = &ufunc_kv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[7]) = ((void *)((char *)((char *)"kv")));

  /* "scipy/special/_ufuncs.pyx":9899
 * ufunc_kv_ptr[2*3] = <void*>_func_cbesk_wrap
 * ufunc_kv_ptr[2*3+1] = <void*>(<char*>"kv")
 * ufunc_kv_data[0] = &ufunc_kv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_kv_data[1] = &ufunc_kv_ptr[2*1]
 * ufunc_kv_data[2] = &ufunc_kv_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9900
 * ufunc_kv_ptr[2*3+1] = <void*>(<char*>"kv")
 * ufunc_kv_data[0] = &ufunc_kv_ptr[2*0]
 * ufunc_kv_data[1] = &ufunc_kv_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_kv_data[2] = &ufunc_kv_ptr[2*2]
 * ufunc_kv_data[3] = &ufunc_kv_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9901
 * ufunc_kv_data[0] = &ufunc_kv_ptr[2*0]
 * ufunc_kv_data[1] = &ufunc_kv_ptr[2*1]
 * ufunc_kv_data[2] = &ufunc_kv_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_kv_data[3] = &ufunc_kv_ptr[2*3]
 * kv = np.PyUFunc_FromFuncAndData(ufunc_kv_loops, ufunc_kv_data, ufunc_kv_types, 4, 2, 1, 0, "kv", ufunc_kv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":9902
 * ufunc_kv_data[1] = &ufunc_kv_ptr[2*1]
 * ufunc_kv_data[2] = &ufunc_kv_ptr[2*2]
 * ufunc_kv_data[3] = &ufunc_kv_ptr[2*3]             # <<<<<<<<<<<<<<
 * kv = np.PyUFunc_FromFuncAndData(ufunc_kv_loops, ufunc_kv_data, ufunc_kv_types, 4, 2, 1, 0, "kv", ufunc_kv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":9903
 * ufunc_kv_data[2] = &ufunc_kv_ptr[2*2]
 * ufunc_kv_data[3] = &ufunc_kv_ptr[2*3]
 * kv = np.PyUFunc_FromFuncAndData(ufunc_kv_loops, ufunc_kv_data, ufunc_kv_types, 4, 2, 1, 0, "kv", ufunc_kv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_kve_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_types, 4, 2, 1, 0, ((char *)"kv"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_kv_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9903, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kv, __pyx_t_2) < 0) __PYX_ERR(1, 9903, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":9909
 * cdef void *ufunc_kve_data[4]
 * cdef char ufunc_kve_types[12]
 * cdef char *ufunc_kve_doc = (             # <<<<<<<<<<<<<<
 *     "kve(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_doc = ((char *)"kve(v, z)\n\nExponentially scaled modified Bessel function of the second kind.\n\nReturns the exponentially scaled, modified Bessel function of the\nsecond kind (sometimes called the third kind) for real order `v` at\ncomplex `z`::\n\n    kve(v, z) = kv(v, z) * exp(z)\n\nParameters\n----------\nv : array_like of float\n    Order of Bessel functions\nz : array_like of complex\n    Argument at which to evaluate the Bessel functions\n\nReturns\n-------\nout : ndarray\n    The exponentially scaled modified Bessel function of the second kind.\n\nNotes\n-----\nWrapper for AMOS [1]_ routine `zbesk`.  For a discussion of the\nalgorithm used, see [2]_ and the references therein.\n\nReferences\n----------\n.. [1] Donald E. Amos, \"AMOS, A Portable Package for Bessel Functions\n       of a Complex Argument and Nonnegative Order\",\n       http://netlib.org/amos/\n.. [2] Donald E. Amos, \"Algorithm 644: A portable package for Bessel\n       functions of a complex argument and nonnegative order\", ACM\n       TOMS Vol. 12 Issue 3, Sept. 1986, p. 265");

  /* "scipy/special/_ufuncs.pyx":9945
 *     "       functions of a complex argument and nonnegative order\", ACM\n"
 *     "       TOMS Vol. 12 Issue 3, Sept. 1986, p. 265")
 * ufunc_kve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_kve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_kve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":9946
 *     "       TOMS Vol. 12 Issue 3, Sept. 1986, p. 265")
 * ufunc_kve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_kve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_kve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":9947
 * ufunc_kve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_kve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_kve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_kve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_kve_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":9948
 * ufunc_kve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_kve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_kve_types[0] = <char>NPY_FLOAT
 * ufunc_kve_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":9949
 * ufunc_kve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_kve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_kve_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kve_types[1] = <char>NPY_FLOAT
 * ufunc_kve_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9950
 * ufunc_kve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_kve_types[0] = <char>NPY_FLOAT
 * ufunc_kve_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kve_types[2] = <char>NPY_FLOAT
 * ufunc_kve_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9951
 * ufunc_kve_types[0] = <char>NPY_FLOAT
 * ufunc_kve_types[1] = <char>NPY_FLOAT
 * ufunc_kve_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kve_types[3] = <char>NPY_FLOAT
 * ufunc_kve_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9952
 * ufunc_kve_types[1] = <char>NPY_FLOAT
 * ufunc_kve_types[2] = <char>NPY_FLOAT
 * ufunc_kve_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_kve_types[4] = <char>NPY_CFLOAT
 * ufunc_kve_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9953
 * ufunc_kve_types[2] = <char>NPY_FLOAT
 * ufunc_kve_types[3] = <char>NPY_FLOAT
 * ufunc_kve_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_kve_types[5] = <char>NPY_CFLOAT
 * ufunc_kve_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9954
 * ufunc_kve_types[3] = <char>NPY_FLOAT
 * ufunc_kve_types[4] = <char>NPY_CFLOAT
 * ufunc_kve_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_kve_types[6] = <char>NPY_DOUBLE
 * ufunc_kve_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9955
 * ufunc_kve_types[4] = <char>NPY_CFLOAT
 * ufunc_kve_types[5] = <char>NPY_CFLOAT
 * ufunc_kve_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kve_types[7] = <char>NPY_DOUBLE
 * ufunc_kve_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9956
 * ufunc_kve_types[5] = <char>NPY_CFLOAT
 * ufunc_kve_types[6] = <char>NPY_DOUBLE
 * ufunc_kve_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kve_types[8] = <char>NPY_DOUBLE
 * ufunc_kve_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9957
 * ufunc_kve_types[6] = <char>NPY_DOUBLE
 * ufunc_kve_types[7] = <char>NPY_DOUBLE
 * ufunc_kve_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kve_types[9] = <char>NPY_DOUBLE
 * ufunc_kve_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9958
 * ufunc_kve_types[7] = <char>NPY_DOUBLE
 * ufunc_kve_types[8] = <char>NPY_DOUBLE
 * ufunc_kve_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kve_types[10] = <char>NPY_CDOUBLE
 * ufunc_kve_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9959
 * ufunc_kve_types[8] = <char>NPY_DOUBLE
 * ufunc_kve_types[9] = <char>NPY_DOUBLE
 * ufunc_kve_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kve_types[11] = <char>NPY_CDOUBLE
 * ufunc_kve_ptr[2*0] = <void*>_func_cbesk_wrap_e_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9960
 * ufunc_kve_types[9] = <char>NPY_DOUBLE
 * ufunc_kve_types[10] = <char>NPY_CDOUBLE
 * ufunc_kve_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_kve_ptr[2*0] = <void*>_func_cbesk_wrap_e_real
 * ufunc_kve_ptr[2*0+1] = <void*>(<char*>"kve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9961
 * ufunc_kve_types[10] = <char>NPY_CDOUBLE
 * ufunc_kve_types[11] = <char>NPY_CDOUBLE
 * ufunc_kve_ptr[2*0] = <void*>_func_cbesk_wrap_e_real             # <<<<<<<<<<<<<<
 * ufunc_kve_ptr[2*0+1] = <void*>(<char*>"kve")
 * ufunc_kve_ptr[2*1] = <void*>_func_cbesk_wrap_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[0]) = ((void *)cbesk_wrap_e_real);

  /* "scipy/special/_ufuncs.pyx":9962
 * ufunc_kve_types[11] = <char>NPY_CDOUBLE
 * ufunc_kve_ptr[2*0] = <void*>_func_cbesk_wrap_e_real
 * ufunc_kve_ptr[2*0+1] = <void*>(<char*>"kve")             # <<<<<<<<<<<<<<
 * ufunc_kve_ptr[2*1] = <void*>_func_cbesk_wrap_e
 * ufunc_kve_ptr[2*1+1] = <void*>(<char*>"kve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[1]) = ((void *)((char *)((char *)"kve")));

  /* "scipy/special/_ufuncs.pyx":9963
 * ufunc_kve_ptr[2*0] = <void*>_func_cbesk_wrap_e_real
 * ufunc_kve_ptr[2*0+1] = <void*>(<char*>"kve")
 * ufunc_kve_ptr[2*1] = <void*>_func_cbesk_wrap_e             # <<<<<<<<<<<<<<
 * ufunc_kve_ptr[2*1+1] = <void*>(<char*>"kve")
 * ufunc_kve_ptr[2*2] = <void*>_func_cbesk_wrap_e_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[2]) = ((void *)cbesk_wrap_e);

  /* "scipy/special/_ufuncs.pyx":9964
 * ufunc_kve_ptr[2*0+1] = <void*>(<char*>"kve")
 * ufunc_kve_ptr[2*1] = <void*>_func_cbesk_wrap_e
 * ufunc_kve_ptr[2*1+1] = <void*>(<char*>"kve")             # <<<<<<<<<<<<<<
 * ufunc_kve_ptr[2*2] = <void*>_func_cbesk_wrap_e_real
 * ufunc_kve_ptr[2*2+1] = <void*>(<char*>"kve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[3]) = ((void *)((char *)((char *)"kve")));

  /* "scipy/special/_ufuncs.pyx":9965
 * ufunc_kve_ptr[2*1] = <void*>_func_cbesk_wrap_e
 * ufunc_kve_ptr[2*1+1] = <void*>(<char*>"kve")
 * ufunc_kve_ptr[2*2] = <void*>_func_cbesk_wrap_e_real             # <<<<<<<<<<<<<<
 * ufunc_kve_ptr[2*2+1] = <void*>(<char*>"kve")
 * ufunc_kve_ptr[2*3] = <void*>_func_cbesk_wrap_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[4]) = ((void *)cbesk_wrap_e_real);

  /* "scipy/special/_ufuncs.pyx":9966
 * ufunc_kve_ptr[2*1+1] = <void*>(<char*>"kve")
 * ufunc_kve_ptr[2*2] = <void*>_func_cbesk_wrap_e_real
 * ufunc_kve_ptr[2*2+1] = <void*>(<char*>"kve")             # <<<<<<<<<<<<<<
 * ufunc_kve_ptr[2*3] = <void*>_func_cbesk_wrap_e
 * ufunc_kve_ptr[2*3+1] = <void*>(<char*>"kve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[5]) = ((void *)((char *)((char *)"kve")));

  /* "scipy/special/_ufuncs.pyx":9967
 * ufunc_kve_ptr[2*2] = <void*>_func_cbesk_wrap_e_real
 * ufunc_kve_ptr[2*2+1] = <void*>(<char*>"kve")
 * ufunc_kve_ptr[2*3] = <void*>_func_cbesk_wrap_e             # <<<<<<<<<<<<<<
 * ufunc_kve_ptr[2*3+1] = <void*>(<char*>"kve")
 * ufunc_kve_data[0] = &ufunc_kve_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[6]) = ((void *)cbesk_wrap_e);

  /* "scipy/special/_ufuncs.pyx":9968
 * ufunc_kve_ptr[2*2+1] = <void*>(<char*>"kve")
 * ufunc_kve_ptr[2*3] = <void*>_func_cbesk_wrap_e
 * ufunc_kve_ptr[2*3+1] = <void*>(<char*>"kve")             # <<<<<<<<<<<<<<
 * ufunc_kve_data[0] = &ufunc_kve_ptr[2*0]
 * ufunc_kve_data[1] = &ufunc_kve_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[7]) = ((void *)((char *)((char *)"kve")));

  /* "scipy/special/_ufuncs.pyx":9969
 * ufunc_kve_ptr[2*3] = <void*>_func_cbesk_wrap_e
 * ufunc_kve_ptr[2*3+1] = <void*>(<char*>"kve")
 * ufunc_kve_data[0] = &ufunc_kve_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_kve_data[1] = &ufunc_kve_ptr[2*1]
 * ufunc_kve_data[2] = &ufunc_kve_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":9970
 * ufunc_kve_ptr[2*3+1] = <void*>(<char*>"kve")
 * ufunc_kve_data[0] = &ufunc_kve_ptr[2*0]
 * ufunc_kve_data[1] = &ufunc_kve_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_kve_data[2] = &ufunc_kve_ptr[2*2]
 * ufunc_kve_data[3] = &ufunc_kve_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":9971
 * ufunc_kve_data[0] = &ufunc_kve_ptr[2*0]
 * ufunc_kve_data[1] = &ufunc_kve_ptr[2*1]
 * ufunc_kve_data[2] = &ufunc_kve_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_kve_data[3] = &ufunc_kve_ptr[2*3]
 * kve = np.PyUFunc_FromFuncAndData(ufunc_kve_loops, ufunc_kve_data, ufunc_kve_types, 4, 2, 1, 0, "kve", ufunc_kve_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":9972
 * ufunc_kve_data[1] = &ufunc_kve_ptr[2*1]
 * ufunc_kve_data[2] = &ufunc_kve_ptr[2*2]
 * ufunc_kve_data[3] = &ufunc_kve_ptr[2*3]             # <<<<<<<<<<<<<<
 * kve = np.PyUFunc_FromFuncAndData(ufunc_kve_loops, ufunc_kve_data, ufunc_kve_types, 4, 2, 1, 0, "kve", ufunc_kve_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":9973
 * ufunc_kve_data[2] = &ufunc_kve_ptr[2*2]
 * ufunc_kve_data[3] = &ufunc_kve_ptr[2*3]
 * kve = np.PyUFunc_FromFuncAndData(ufunc_kve_loops, ufunc_kve_data, ufunc_kve_types, 4, 2, 1, 0, "kve", ufunc_kve_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_log1p_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_types, 4, 2, 1, 0, ((char *)"kve"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_kve_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9973, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_kve, __pyx_t_2) < 0) __PYX_ERR(1, 9973, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":9979
 * cdef void *ufunc_log1p_data[4]
 * cdef char ufunc_log1p_types[8]
 * cdef char *ufunc_log1p_doc = (             # <<<<<<<<<<<<<<
 *     "log1p(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_doc = ((char *)"log1p(x)\n\nCalculates log(1+x) for use when `x` is near zero");

  /* "scipy/special/_ufuncs.pyx":9983
 *     "\n"
 *     "Calculates log(1+x) for use when `x` is near zero")
 * ufunc_log1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_log1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_log1p_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":9984
 *     "Calculates log(1+x) for use when `x` is near zero")
 * ufunc_log1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_log1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_log1p_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_log1p_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":9985
 * ufunc_log1p_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_log1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_log1p_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_log1p_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_log1p_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":9986
 * ufunc_log1p_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_log1p_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_log1p_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_log1p_types[0] = <char>NPY_FLOAT
 * ufunc_log1p_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":9987
 * ufunc_log1p_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_log1p_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_log1p_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_log1p_types[1] = <char>NPY_FLOAT
 * ufunc_log1p_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9988
 * ufunc_log1p_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_log1p_types[0] = <char>NPY_FLOAT
 * ufunc_log1p_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_log1p_types[2] = <char>NPY_DOUBLE
 * ufunc_log1p_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":9989
 * ufunc_log1p_types[0] = <char>NPY_FLOAT
 * ufunc_log1p_types[1] = <char>NPY_FLOAT
 * ufunc_log1p_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_log1p_types[3] = <char>NPY_DOUBLE
 * ufunc_log1p_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9990
 * ufunc_log1p_types[1] = <char>NPY_FLOAT
 * ufunc_log1p_types[2] = <char>NPY_DOUBLE
 * ufunc_log1p_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_log1p_types[4] = <char>NPY_CFLOAT
 * ufunc_log1p_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":9991
 * ufunc_log1p_types[2] = <char>NPY_DOUBLE
 * ufunc_log1p_types[3] = <char>NPY_DOUBLE
 * ufunc_log1p_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_log1p_types[5] = <char>NPY_CFLOAT
 * ufunc_log1p_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9992
 * ufunc_log1p_types[3] = <char>NPY_DOUBLE
 * ufunc_log1p_types[4] = <char>NPY_CFLOAT
 * ufunc_log1p_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_log1p_types[6] = <char>NPY_CDOUBLE
 * ufunc_log1p_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":9993
 * ufunc_log1p_types[4] = <char>NPY_CFLOAT
 * ufunc_log1p_types[5] = <char>NPY_CFLOAT
 * ufunc_log1p_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_log1p_types[7] = <char>NPY_CDOUBLE
 * ufunc_log1p_ptr[2*0] = <void*>_func_log1p
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9994
 * ufunc_log1p_types[5] = <char>NPY_CFLOAT
 * ufunc_log1p_types[6] = <char>NPY_CDOUBLE
 * ufunc_log1p_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_log1p_ptr[2*0] = <void*>_func_log1p
 * ufunc_log1p_ptr[2*0+1] = <void*>(<char*>"log1p")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":9995
 * ufunc_log1p_types[6] = <char>NPY_CDOUBLE
 * ufunc_log1p_types[7] = <char>NPY_CDOUBLE
 * ufunc_log1p_ptr[2*0] = <void*>_func_log1p             # <<<<<<<<<<<<<<
 * ufunc_log1p_ptr[2*0+1] = <void*>(<char*>"log1p")
 * ufunc_log1p_ptr[2*1] = <void*>_func_log1p
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_ptr[0]) = ((void *)log1p);

  /* "scipy/special/_ufuncs.pyx":9996
 * ufunc_log1p_types[7] = <char>NPY_CDOUBLE
 * ufunc_log1p_ptr[2*0] = <void*>_func_log1p
 * ufunc_log1p_ptr[2*0+1] = <void*>(<char*>"log1p")             # <<<<<<<<<<<<<<
 * ufunc_log1p_ptr[2*1] = <void*>_func_log1p
 * ufunc_log1p_ptr[2*1+1] = <void*>(<char*>"log1p")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_ptr[1]) = ((void *)((char *)((char *)"log1p")));

  /* "scipy/special/_ufuncs.pyx":9997
 * ufunc_log1p_ptr[2*0] = <void*>_func_log1p
 * ufunc_log1p_ptr[2*0+1] = <void*>(<char*>"log1p")
 * ufunc_log1p_ptr[2*1] = <void*>_func_log1p             # <<<<<<<<<<<<<<
 * ufunc_log1p_ptr[2*1+1] = <void*>(<char*>"log1p")
 * ufunc_log1p_ptr[2*2] = <void*>_func_clog1p
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_ptr[2]) = ((void *)log1p);

  /* "scipy/special/_ufuncs.pyx":9998
 * ufunc_log1p_ptr[2*0+1] = <void*>(<char*>"log1p")
 * ufunc_log1p_ptr[2*1] = <void*>_func_log1p
 * ufunc_log1p_ptr[2*1+1] = <void*>(<char*>"log1p")             # <<<<<<<<<<<<<<
 * ufunc_log1p_ptr[2*2] = <void*>_func_clog1p
 * ufunc_log1p_ptr[2*2+1] = <void*>(<char*>"log1p")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_ptr[3]) = ((void *)((char *)((char *)"log1p")));

  /* "scipy/special/_ufuncs.pyx":9999
 * ufunc_log1p_ptr[2*1] = <void*>_func_log1p
 * ufunc_log1p_ptr[2*1+1] = <void*>(<char*>"log1p")
 * ufunc_log1p_ptr[2*2] = <void*>_func_clog1p             # <<<<<<<<<<<<<<
 * ufunc_log1p_ptr[2*2+1] = <void*>(<char*>"log1p")
 * ufunc_log1p_ptr[2*3] = <void*>_func_clog1p
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_cunity_clog1p);

  /* "scipy/special/_ufuncs.pyx":10000
 * ufunc_log1p_ptr[2*1+1] = <void*>(<char*>"log1p")
 * ufunc_log1p_ptr[2*2] = <void*>_func_clog1p
 * ufunc_log1p_ptr[2*2+1] = <void*>(<char*>"log1p")             # <<<<<<<<<<<<<<
 * ufunc_log1p_ptr[2*3] = <void*>_func_clog1p
 * ufunc_log1p_ptr[2*3+1] = <void*>(<char*>"log1p")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_ptr[5]) = ((void *)((char *)((char *)"log1p")));

  /* "scipy/special/_ufuncs.pyx":10001
 * ufunc_log1p_ptr[2*2] = <void*>_func_clog1p
 * ufunc_log1p_ptr[2*2+1] = <void*>(<char*>"log1p")
 * ufunc_log1p_ptr[2*3] = <void*>_func_clog1p             # <<<<<<<<<<<<<<
 * ufunc_log1p_ptr[2*3+1] = <void*>(<char*>"log1p")
 * ufunc_log1p_data[0] = &ufunc_log1p_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_ptr[6]) = ((void *)__pyx_f_5scipy_7special_7_cunity_clog1p);

  /* "scipy/special/_ufuncs.pyx":10002
 * ufunc_log1p_ptr[2*2+1] = <void*>(<char*>"log1p")
 * ufunc_log1p_ptr[2*3] = <void*>_func_clog1p
 * ufunc_log1p_ptr[2*3+1] = <void*>(<char*>"log1p")             # <<<<<<<<<<<<<<
 * ufunc_log1p_data[0] = &ufunc_log1p_ptr[2*0]
 * ufunc_log1p_data[1] = &ufunc_log1p_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_ptr[7]) = ((void *)((char *)((char *)"log1p")));

  /* "scipy/special/_ufuncs.pyx":10003
 * ufunc_log1p_ptr[2*3] = <void*>_func_clog1p
 * ufunc_log1p_ptr[2*3+1] = <void*>(<char*>"log1p")
 * ufunc_log1p_data[0] = &ufunc_log1p_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_log1p_data[1] = &ufunc_log1p_ptr[2*1]
 * ufunc_log1p_data[2] = &ufunc_log1p_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":10004
 * ufunc_log1p_ptr[2*3+1] = <void*>(<char*>"log1p")
 * ufunc_log1p_data[0] = &ufunc_log1p_ptr[2*0]
 * ufunc_log1p_data[1] = &ufunc_log1p_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_log1p_data[2] = &ufunc_log1p_ptr[2*2]
 * ufunc_log1p_data[3] = &ufunc_log1p_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":10005
 * ufunc_log1p_data[0] = &ufunc_log1p_ptr[2*0]
 * ufunc_log1p_data[1] = &ufunc_log1p_ptr[2*1]
 * ufunc_log1p_data[2] = &ufunc_log1p_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_log1p_data[3] = &ufunc_log1p_ptr[2*3]
 * log1p = np.PyUFunc_FromFuncAndData(ufunc_log1p_loops, ufunc_log1p_data, ufunc_log1p_types, 4, 1, 1, 0, "log1p", ufunc_log1p_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":10006
 * ufunc_log1p_data[1] = &ufunc_log1p_ptr[2*1]
 * ufunc_log1p_data[2] = &ufunc_log1p_ptr[2*2]
 * ufunc_log1p_data[3] = &ufunc_log1p_ptr[2*3]             # <<<<<<<<<<<<<<
 * log1p = np.PyUFunc_FromFuncAndData(ufunc_log1p_loops, ufunc_log1p_data, ufunc_log1p_types, 4, 1, 1, 0, "log1p", ufunc_log1p_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":10007
 * ufunc_log1p_data[2] = &ufunc_log1p_ptr[2*2]
 * ufunc_log1p_data[3] = &ufunc_log1p_ptr[2*3]
 * log1p = np.PyUFunc_FromFuncAndData(ufunc_log1p_loops, ufunc_log1p_data, ufunc_log1p_types, 4, 1, 1, 0, "log1p", ufunc_log1p_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_log_ndtr_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_types, 4, 1, 1, 0, ((char *)"log1p"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_log1p_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10007, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_log1p, __pyx_t_2) < 0) __PYX_ERR(1, 10007, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":10013
 * cdef void *ufunc_log_ndtr_data[4]
 * cdef char ufunc_log_ndtr_types[8]
 * cdef char *ufunc_log_ndtr_doc = (             # <<<<<<<<<<<<<<
 *     "log_ndtr(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_doc = ((char *)"log_ndtr(x)\n\nLogarithm of Gaussian cumulative distribution function.\n\nReturns the log of the area under the standard Gaussian probability\ndensity function, integrated from minus infinity to `x`::\n\n    log(1/sqrt(2*pi) * integral(exp(-t**2 / 2), t=-inf..x))\n\nParameters\n----------\nx : array_like, real or complex\n    Argument\n\nReturns\n-------\nndarray\n    The value of the log of the normal CDF evaluated at `x`\n\nSee Also\n--------\nerf\nerfc\nscipy.stats.norm\nndtr");

  /* "scipy/special/_ufuncs.pyx":10039
 *     "scipy.stats.norm\n"
 *     "ndtr")
 * ufunc_log_ndtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_log_ndtr_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":10040
 *     "ndtr")
 * ufunc_log_ndtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_log_ndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_log_ndtr_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":10041
 * ufunc_log_ndtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_log_ndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_log_ndtr_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_log_ndtr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":10042
 * ufunc_log_ndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_log_ndtr_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_log_ndtr_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_types[0] = <char>NPY_FLOAT
 * ufunc_log_ndtr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":10043
 * ufunc_log_ndtr_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_log_ndtr_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_log_ndtr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_types[1] = <char>NPY_FLOAT
 * ufunc_log_ndtr_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10044
 * ufunc_log_ndtr_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_log_ndtr_types[0] = <char>NPY_FLOAT
 * ufunc_log_ndtr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_types[2] = <char>NPY_DOUBLE
 * ufunc_log_ndtr_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10045
 * ufunc_log_ndtr_types[0] = <char>NPY_FLOAT
 * ufunc_log_ndtr_types[1] = <char>NPY_FLOAT
 * ufunc_log_ndtr_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_types[3] = <char>NPY_DOUBLE
 * ufunc_log_ndtr_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10046
 * ufunc_log_ndtr_types[1] = <char>NPY_FLOAT
 * ufunc_log_ndtr_types[2] = <char>NPY_DOUBLE
 * ufunc_log_ndtr_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_types[4] = <char>NPY_CFLOAT
 * ufunc_log_ndtr_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10047
 * ufunc_log_ndtr_types[2] = <char>NPY_DOUBLE
 * ufunc_log_ndtr_types[3] = <char>NPY_DOUBLE
 * ufunc_log_ndtr_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_types[5] = <char>NPY_CFLOAT
 * ufunc_log_ndtr_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":10048
 * ufunc_log_ndtr_types[3] = <char>NPY_DOUBLE
 * ufunc_log_ndtr_types[4] = <char>NPY_CFLOAT
 * ufunc_log_ndtr_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_types[6] = <char>NPY_CDOUBLE
 * ufunc_log_ndtr_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":10049
 * ufunc_log_ndtr_types[4] = <char>NPY_CFLOAT
 * ufunc_log_ndtr_types[5] = <char>NPY_CFLOAT
 * ufunc_log_ndtr_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_types[7] = <char>NPY_CDOUBLE
 * ufunc_log_ndtr_ptr[2*0] = <void*>_func_log_ndtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":10050
 * ufunc_log_ndtr_types[5] = <char>NPY_CFLOAT
 * ufunc_log_ndtr_types[6] = <char>NPY_CDOUBLE
 * ufunc_log_ndtr_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_ptr[2*0] = <void*>_func_log_ndtr
 * ufunc_log_ndtr_ptr[2*0+1] = <void*>(<char*>"log_ndtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":10051
 * ufunc_log_ndtr_types[6] = <char>NPY_CDOUBLE
 * ufunc_log_ndtr_types[7] = <char>NPY_CDOUBLE
 * ufunc_log_ndtr_ptr[2*0] = <void*>_func_log_ndtr             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_ptr[2*0+1] = <void*>(<char*>"log_ndtr")
 * ufunc_log_ndtr_ptr[2*1] = <void*>_func_log_ndtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_ptr[0]) = ((void *)log_ndtr);

  /* "scipy/special/_ufuncs.pyx":10052
 * ufunc_log_ndtr_types[7] = <char>NPY_CDOUBLE
 * ufunc_log_ndtr_ptr[2*0] = <void*>_func_log_ndtr
 * ufunc_log_ndtr_ptr[2*0+1] = <void*>(<char*>"log_ndtr")             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_ptr[2*1] = <void*>_func_log_ndtr
 * ufunc_log_ndtr_ptr[2*1+1] = <void*>(<char*>"log_ndtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_ptr[1]) = ((void *)((char *)((char *)"log_ndtr")));

  /* "scipy/special/_ufuncs.pyx":10053
 * ufunc_log_ndtr_ptr[2*0] = <void*>_func_log_ndtr
 * ufunc_log_ndtr_ptr[2*0+1] = <void*>(<char*>"log_ndtr")
 * ufunc_log_ndtr_ptr[2*1] = <void*>_func_log_ndtr             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_ptr[2*1+1] = <void*>(<char*>"log_ndtr")
 * ufunc_log_ndtr_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_log_ndtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_ptr[2]) = ((void *)log_ndtr);

  /* "scipy/special/_ufuncs.pyx":10054
 * ufunc_log_ndtr_ptr[2*0+1] = <void*>(<char*>"log_ndtr")
 * ufunc_log_ndtr_ptr[2*1] = <void*>_func_log_ndtr
 * ufunc_log_ndtr_ptr[2*1+1] = <void*>(<char*>"log_ndtr")             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_log_ndtr
 * ufunc_log_ndtr_ptr[2*2+1] = <void*>(<char*>"log_ndtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_ptr[3]) = ((void *)((char *)((char *)"log_ndtr")));

  /* "scipy/special/_ufuncs.pyx":10055
 * ufunc_log_ndtr_ptr[2*1] = <void*>_func_log_ndtr
 * ufunc_log_ndtr_ptr[2*1+1] = <void*>(<char*>"log_ndtr")
 * ufunc_log_ndtr_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_log_ndtr             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_ptr[2*2+1] = <void*>(<char*>"log_ndtr")
 * ufunc_log_ndtr_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_log_ndtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_ptr[4]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_log_ndtr);

  /* "scipy/special/_ufuncs.pyx":10056
 * ufunc_log_ndtr_ptr[2*1+1] = <void*>(<char*>"log_ndtr")
 * ufunc_log_ndtr_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_log_ndtr
 * ufunc_log_ndtr_ptr[2*2+1] = <void*>(<char*>"log_ndtr")             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_log_ndtr
 * ufunc_log_ndtr_ptr[2*3+1] = <void*>(<char*>"log_ndtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_ptr[5]) = ((void *)((char *)((char *)"log_ndtr")));

  /* "scipy/special/_ufuncs.pyx":10057
 * ufunc_log_ndtr_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_log_ndtr
 * ufunc_log_ndtr_ptr[2*2+1] = <void*>(<char*>"log_ndtr")
 * ufunc_log_ndtr_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_log_ndtr             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_ptr[2*3+1] = <void*>(<char*>"log_ndtr")
 * ufunc_log_ndtr_data[0] = &ufunc_log_ndtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_ptr[6]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_log_ndtr);

  /* "scipy/special/_ufuncs.pyx":10058
 * ufunc_log_ndtr_ptr[2*2+1] = <void*>(<char*>"log_ndtr")
 * ufunc_log_ndtr_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_log_ndtr
 * ufunc_log_ndtr_ptr[2*3+1] = <void*>(<char*>"log_ndtr")             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_data[0] = &ufunc_log_ndtr_ptr[2*0]
 * ufunc_log_ndtr_data[1] = &ufunc_log_ndtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_ptr[7]) = ((void *)((char *)((char *)"log_ndtr")));

  /* "scipy/special/_ufuncs.pyx":10059
 * ufunc_log_ndtr_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_log_ndtr
 * ufunc_log_ndtr_ptr[2*3+1] = <void*>(<char*>"log_ndtr")
 * ufunc_log_ndtr_data[0] = &ufunc_log_ndtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_data[1] = &ufunc_log_ndtr_ptr[2*1]
 * ufunc_log_ndtr_data[2] = &ufunc_log_ndtr_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":10060
 * ufunc_log_ndtr_ptr[2*3+1] = <void*>(<char*>"log_ndtr")
 * ufunc_log_ndtr_data[0] = &ufunc_log_ndtr_ptr[2*0]
 * ufunc_log_ndtr_data[1] = &ufunc_log_ndtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_data[2] = &ufunc_log_ndtr_ptr[2*2]
 * ufunc_log_ndtr_data[3] = &ufunc_log_ndtr_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":10061
 * ufunc_log_ndtr_data[0] = &ufunc_log_ndtr_ptr[2*0]
 * ufunc_log_ndtr_data[1] = &ufunc_log_ndtr_ptr[2*1]
 * ufunc_log_ndtr_data[2] = &ufunc_log_ndtr_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_log_ndtr_data[3] = &ufunc_log_ndtr_ptr[2*3]
 * log_ndtr = np.PyUFunc_FromFuncAndData(ufunc_log_ndtr_loops, ufunc_log_ndtr_data, ufunc_log_ndtr_types, 4, 1, 1, 0, "log_ndtr", ufunc_log_ndtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":10062
 * ufunc_log_ndtr_data[1] = &ufunc_log_ndtr_ptr[2*1]
 * ufunc_log_ndtr_data[2] = &ufunc_log_ndtr_ptr[2*2]
 * ufunc_log_ndtr_data[3] = &ufunc_log_ndtr_ptr[2*3]             # <<<<<<<<<<<<<<
 * log_ndtr = np.PyUFunc_FromFuncAndData(ufunc_log_ndtr_loops, ufunc_log_ndtr_data, ufunc_log_ndtr_types, 4, 1, 1, 0, "log_ndtr", ufunc_log_ndtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":10063
 * ufunc_log_ndtr_data[2] = &ufunc_log_ndtr_ptr[2*2]
 * ufunc_log_ndtr_data[3] = &ufunc_log_ndtr_ptr[2*3]
 * log_ndtr = np.PyUFunc_FromFuncAndData(ufunc_log_ndtr_loops, ufunc_log_ndtr_data, ufunc_log_ndtr_types, 4, 1, 1, 0, "log_ndtr", ufunc_log_ndtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_loggamma_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_types, 4, 1, 1, 0, ((char *)"log_ndtr"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_log_ndtr_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10063, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_log_ndtr, __pyx_t_2) < 0) __PYX_ERR(1, 10063, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":10069
 * cdef void *ufunc_loggamma_data[4]
 * cdef char ufunc_loggamma_types[8]
 * cdef char *ufunc_loggamma_doc = (             # <<<<<<<<<<<<<<
 *     "loggamma(z, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_doc = ((char *)"loggamma(z, out=None)\n\nPrincipal branch of the logarithm of the Gamma function.\n\nDefined to be :math:`\\log(\\Gamma(x))` for :math:`x > 0` and\nextended to the complex plane by analytic continuation. The\nfunction has a single branch cut on the negative real axis.\n\n.. versionadded:: 0.18.0\n\nParameters\n----------\nz : array-like\n    Values in the complex plain at which to compute ``loggamma``\nout : ndarray, optional\n    Output array for computed values of ``loggamma``\n\nReturns\n-------\nloggamma : ndarray\n    Values of ``loggamma`` at z.\n\nNotes\n-----\nIt is not generally true that :math:`\\log\\Gamma(z) =\n\\log(\\Gamma(z))`, though the real parts of the functions do\nagree. The benefit of not defining `loggamma` as\n:math:`\\log(\\Gamma(z))` is that the latter function has a\ncomplicated branch cut structure whereas `loggamma` is analytic\nexcept for on the negative real axis.\n\nThe identities\n\n.. math::\n  \\exp(\\log\\Gamma(z)) &= \\Gamma(z) \\\\\n  \\log\\Gamma(z + 1) &= \\log(z) + \\log\\Gamma(z)\n\nmake `loggamma` useful for working in complex logspace.\n\nOn the real line `loggamma` is related to `gammaln` via\n``exp(loggamma(x + 0j)) = gammasgn(x)*exp(gammaln(x))``, up to\nrounding error.\n\nThe implementation here is based on [hare1997]_.\n\nSee also\n--------\ngammaln : logarithm of the absolute value of the Gamma function\ngammasgn : sign of the gamma function\n\nReferences\n----------\n.. [hare1997] D.E.G. Hare,\n  *Computing the Principal Branch of log-Gamma*,\n  Journal of Algorithms, Volume 25, Issue 2, November 1997, pages 221-236.");

  /* "scipy/special/_ufuncs.pyx":10125
 *     "  *Computing the Principal Branch of log-Gamma*,\n"
 *     "  Journal of Algorithms, Volume 25, Issue 2, November 1997, pages 221-236.")
 * ufunc_loggamma_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_loggamma_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_loggamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":10126
 *     "  Journal of Algorithms, Volume 25, Issue 2, November 1997, pages 221-236.")
 * ufunc_loggamma_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_loggamma_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_loggamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_loggamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":10127
 * ufunc_loggamma_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_loggamma_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_loggamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_loggamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_loggamma_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":10128
 * ufunc_loggamma_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_loggamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_loggamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_loggamma_types[0] = <char>NPY_FLOAT
 * ufunc_loggamma_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":10129
 * ufunc_loggamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_loggamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_loggamma_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_loggamma_types[1] = <char>NPY_FLOAT
 * ufunc_loggamma_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10130
 * ufunc_loggamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_loggamma_types[0] = <char>NPY_FLOAT
 * ufunc_loggamma_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_loggamma_types[2] = <char>NPY_DOUBLE
 * ufunc_loggamma_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10131
 * ufunc_loggamma_types[0] = <char>NPY_FLOAT
 * ufunc_loggamma_types[1] = <char>NPY_FLOAT
 * ufunc_loggamma_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_loggamma_types[3] = <char>NPY_DOUBLE
 * ufunc_loggamma_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10132
 * ufunc_loggamma_types[1] = <char>NPY_FLOAT
 * ufunc_loggamma_types[2] = <char>NPY_DOUBLE
 * ufunc_loggamma_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_loggamma_types[4] = <char>NPY_CFLOAT
 * ufunc_loggamma_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10133
 * ufunc_loggamma_types[2] = <char>NPY_DOUBLE
 * ufunc_loggamma_types[3] = <char>NPY_DOUBLE
 * ufunc_loggamma_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_loggamma_types[5] = <char>NPY_CFLOAT
 * ufunc_loggamma_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":10134
 * ufunc_loggamma_types[3] = <char>NPY_DOUBLE
 * ufunc_loggamma_types[4] = <char>NPY_CFLOAT
 * ufunc_loggamma_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_loggamma_types[6] = <char>NPY_CDOUBLE
 * ufunc_loggamma_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":10135
 * ufunc_loggamma_types[4] = <char>NPY_CFLOAT
 * ufunc_loggamma_types[5] = <char>NPY_CFLOAT
 * ufunc_loggamma_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_loggamma_types[7] = <char>NPY_CDOUBLE
 * ufunc_loggamma_ptr[2*0] = <void*>_func_loggamma_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":10136
 * ufunc_loggamma_types[5] = <char>NPY_CFLOAT
 * ufunc_loggamma_types[6] = <char>NPY_CDOUBLE
 * ufunc_loggamma_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_loggamma_ptr[2*0] = <void*>_func_loggamma_real
 * ufunc_loggamma_ptr[2*0+1] = <void*>(<char*>"loggamma")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":10137
 * ufunc_loggamma_types[6] = <char>NPY_CDOUBLE
 * ufunc_loggamma_types[7] = <char>NPY_CDOUBLE
 * ufunc_loggamma_ptr[2*0] = <void*>_func_loggamma_real             # <<<<<<<<<<<<<<
 * ufunc_loggamma_ptr[2*0+1] = <void*>(<char*>"loggamma")
 * ufunc_loggamma_ptr[2*1] = <void*>_func_loggamma_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_ptr[0]) = ((void *)__pyx_f_5scipy_7special_9_loggamma_loggamma_real);

  /* "scipy/special/_ufuncs.pyx":10138
 * ufunc_loggamma_types[7] = <char>NPY_CDOUBLE
 * ufunc_loggamma_ptr[2*0] = <void*>_func_loggamma_real
 * ufunc_loggamma_ptr[2*0+1] = <void*>(<char*>"loggamma")             # <<<<<<<<<<<<<<
 * ufunc_loggamma_ptr[2*1] = <void*>_func_loggamma_real
 * ufunc_loggamma_ptr[2*1+1] = <void*>(<char*>"loggamma")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_ptr[1]) = ((void *)((char *)((char *)"loggamma")));

  /* "scipy/special/_ufuncs.pyx":10139
 * ufunc_loggamma_ptr[2*0] = <void*>_func_loggamma_real
 * ufunc_loggamma_ptr[2*0+1] = <void*>(<char*>"loggamma")
 * ufunc_loggamma_ptr[2*1] = <void*>_func_loggamma_real             # <<<<<<<<<<<<<<
 * ufunc_loggamma_ptr[2*1+1] = <void*>(<char*>"loggamma")
 * ufunc_loggamma_ptr[2*2] = <void*>_func_loggamma
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_ptr[2]) = ((void *)__pyx_f_5scipy_7special_9_loggamma_loggamma_real);

  /* "scipy/special/_ufuncs.pyx":10140
 * ufunc_loggamma_ptr[2*0+1] = <void*>(<char*>"loggamma")
 * ufunc_loggamma_ptr[2*1] = <void*>_func_loggamma_real
 * ufunc_loggamma_ptr[2*1+1] = <void*>(<char*>"loggamma")             # <<<<<<<<<<<<<<
 * ufunc_loggamma_ptr[2*2] = <void*>_func_loggamma
 * ufunc_loggamma_ptr[2*2+1] = <void*>(<char*>"loggamma")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_ptr[3]) = ((void *)((char *)((char *)"loggamma")));

  /* "scipy/special/_ufuncs.pyx":10141
 * ufunc_loggamma_ptr[2*1] = <void*>_func_loggamma_real
 * ufunc_loggamma_ptr[2*1+1] = <void*>(<char*>"loggamma")
 * ufunc_loggamma_ptr[2*2] = <void*>_func_loggamma             # <<<<<<<<<<<<<<
 * ufunc_loggamma_ptr[2*2+1] = <void*>(<char*>"loggamma")
 * ufunc_loggamma_ptr[2*3] = <void*>_func_loggamma
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_ptr[4]) = ((void *)__pyx_f_5scipy_7special_9_loggamma_loggamma);

  /* "scipy/special/_ufuncs.pyx":10142
 * ufunc_loggamma_ptr[2*1+1] = <void*>(<char*>"loggamma")
 * ufunc_loggamma_ptr[2*2] = <void*>_func_loggamma
 * ufunc_loggamma_ptr[2*2+1] = <void*>(<char*>"loggamma")             # <<<<<<<<<<<<<<
 * ufunc_loggamma_ptr[2*3] = <void*>_func_loggamma
 * ufunc_loggamma_ptr[2*3+1] = <void*>(<char*>"loggamma")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_ptr[5]) = ((void *)((char *)((char *)"loggamma")));

  /* "scipy/special/_ufuncs.pyx":10143
 * ufunc_loggamma_ptr[2*2] = <void*>_func_loggamma
 * ufunc_loggamma_ptr[2*2+1] = <void*>(<char*>"loggamma")
 * ufunc_loggamma_ptr[2*3] = <void*>_func_loggamma             # <<<<<<<<<<<<<<
 * ufunc_loggamma_ptr[2*3+1] = <void*>(<char*>"loggamma")
 * ufunc_loggamma_data[0] = &ufunc_loggamma_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_ptr[6]) = ((void *)__pyx_f_5scipy_7special_9_loggamma_loggamma);

  /* "scipy/special/_ufuncs.pyx":10144
 * ufunc_loggamma_ptr[2*2+1] = <void*>(<char*>"loggamma")
 * ufunc_loggamma_ptr[2*3] = <void*>_func_loggamma
 * ufunc_loggamma_ptr[2*3+1] = <void*>(<char*>"loggamma")             # <<<<<<<<<<<<<<
 * ufunc_loggamma_data[0] = &ufunc_loggamma_ptr[2*0]
 * ufunc_loggamma_data[1] = &ufunc_loggamma_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_ptr[7]) = ((void *)((char *)((char *)"loggamma")));

  /* "scipy/special/_ufuncs.pyx":10145
 * ufunc_loggamma_ptr[2*3] = <void*>_func_loggamma
 * ufunc_loggamma_ptr[2*3+1] = <void*>(<char*>"loggamma")
 * ufunc_loggamma_data[0] = &ufunc_loggamma_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_loggamma_data[1] = &ufunc_loggamma_ptr[2*1]
 * ufunc_loggamma_data[2] = &ufunc_loggamma_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":10146
 * ufunc_loggamma_ptr[2*3+1] = <void*>(<char*>"loggamma")
 * ufunc_loggamma_data[0] = &ufunc_loggamma_ptr[2*0]
 * ufunc_loggamma_data[1] = &ufunc_loggamma_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_loggamma_data[2] = &ufunc_loggamma_ptr[2*2]
 * ufunc_loggamma_data[3] = &ufunc_loggamma_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":10147
 * ufunc_loggamma_data[0] = &ufunc_loggamma_ptr[2*0]
 * ufunc_loggamma_data[1] = &ufunc_loggamma_ptr[2*1]
 * ufunc_loggamma_data[2] = &ufunc_loggamma_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_loggamma_data[3] = &ufunc_loggamma_ptr[2*3]
 * loggamma = np.PyUFunc_FromFuncAndData(ufunc_loggamma_loops, ufunc_loggamma_data, ufunc_loggamma_types, 4, 1, 1, 0, "loggamma", ufunc_loggamma_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":10148
 * ufunc_loggamma_data[1] = &ufunc_loggamma_ptr[2*1]
 * ufunc_loggamma_data[2] = &ufunc_loggamma_ptr[2*2]
 * ufunc_loggamma_data[3] = &ufunc_loggamma_ptr[2*3]             # <<<<<<<<<<<<<<
 * loggamma = np.PyUFunc_FromFuncAndData(ufunc_loggamma_loops, ufunc_loggamma_data, ufunc_loggamma_types, 4, 1, 1, 0, "loggamma", ufunc_loggamma_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":10149
 * ufunc_loggamma_data[2] = &ufunc_loggamma_ptr[2*2]
 * ufunc_loggamma_data[3] = &ufunc_loggamma_ptr[2*3]
 * loggamma = np.PyUFunc_FromFuncAndData(ufunc_loggamma_loops, ufunc_loggamma_data, ufunc_loggamma_types, 4, 1, 1, 0, "loggamma", ufunc_loggamma_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_logit_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_types, 4, 1, 1, 0, ((char *)"loggamma"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_loggamma_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_loggamma, __pyx_t_2) < 0) __PYX_ERR(1, 10149, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":10155
 * cdef void *ufunc_logit_data[3]
 * cdef char ufunc_logit_types[6]
 * cdef char *ufunc_logit_doc = (             # <<<<<<<<<<<<<<
 *     "logit(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_doc = ((char *)"logit(x)\n\nLogit ufunc for ndarrays.\n\nThe logit function is defined as logit(p) = log(p/(1-p)).\nNote that logit(0) = -inf, logit(1) = inf, and logit(p)\nfor p<0 or p>1 yields nan.\n\nParameters\n----------\nx : ndarray\n    The ndarray to apply logit to element-wise.\n\nReturns\n-------\nout : ndarray\n    An ndarray of the same shape as x. Its entries\n    are logit of the corresponding entry of x.\n\nSee Also\n--------\nexpit\n\nNotes\n-----\nAs a ufunc logit takes a number of optional\nkeyword arguments. For more information\nsee `ufuncs <https://docs.scipy.org/doc/numpy/reference/ufuncs.html>`_\n\n.. versionadded:: 0.10.0\n\nExamples\n--------\n>>> from scipy.special import logit, expit\n\n>>> logit([0, 0.25, 0.5, 0.75, 1])\narray([       -inf, -1.09861229,  0.        ,  1.09861229,         inf])\n\n`expit` is the inverse of `logit`:\n\n>>> expit(logit([0.1, 0.75, 0.999]))\narray([ 0.1  ,  0.75 ,  0.999])\n\nPlot logit(x) for x in [0, 1]:\n\n>>> import matplotlib.pyplot as plt\n>>> x = np.linspace(0, 1, 501)\n>>> y = logit(x)\n>>> plt.plot(x, y)\n>>> plt.grid()\n>>> plt.ylim(-6, 6)\n>>> plt.xlabel('x')\n>>> plt.title('logit(x)')\n>>> plt.show()");

  /* "scipy/special/_ufuncs.pyx":10210
 *     ">>> plt.title('logit(x)')\n"
 *     ">>> plt.show()")
 * ufunc_logit_loops[0] = <np.PyUFuncGenericFunction>loop_f_f__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_logit_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_logit_loops[2] = <np.PyUFuncGenericFunction>loop_g_g__As_g_g
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_f_f__As_f_f);

  /* "scipy/special/_ufuncs.pyx":10211
 *     ">>> plt.show()")
 * ufunc_logit_loops[0] = <np.PyUFuncGenericFunction>loop_f_f__As_f_f
 * ufunc_logit_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_logit_loops[2] = <np.PyUFuncGenericFunction>loop_g_g__As_g_g
 * ufunc_logit_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":10212
 * ufunc_logit_loops[0] = <np.PyUFuncGenericFunction>loop_f_f__As_f_f
 * ufunc_logit_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_logit_loops[2] = <np.PyUFuncGenericFunction>loop_g_g__As_g_g             # <<<<<<<<<<<<<<
 * ufunc_logit_types[0] = <char>NPY_FLOAT
 * ufunc_logit_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_g_g__As_g_g);

  /* "scipy/special/_ufuncs.pyx":10213
 * ufunc_logit_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_logit_loops[2] = <np.PyUFuncGenericFunction>loop_g_g__As_g_g
 * ufunc_logit_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_logit_types[1] = <char>NPY_FLOAT
 * ufunc_logit_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10214
 * ufunc_logit_loops[2] = <np.PyUFuncGenericFunction>loop_g_g__As_g_g
 * ufunc_logit_types[0] = <char>NPY_FLOAT
 * ufunc_logit_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_logit_types[2] = <char>NPY_DOUBLE
 * ufunc_logit_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10215
 * ufunc_logit_types[0] = <char>NPY_FLOAT
 * ufunc_logit_types[1] = <char>NPY_FLOAT
 * ufunc_logit_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_logit_types[3] = <char>NPY_DOUBLE
 * ufunc_logit_types[4] = <char>NPY_LONGDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10216
 * ufunc_logit_types[1] = <char>NPY_FLOAT
 * ufunc_logit_types[2] = <char>NPY_DOUBLE
 * ufunc_logit_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_logit_types[4] = <char>NPY_LONGDOUBLE
 * ufunc_logit_types[5] = <char>NPY_LONGDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10217
 * ufunc_logit_types[2] = <char>NPY_DOUBLE
 * ufunc_logit_types[3] = <char>NPY_DOUBLE
 * ufunc_logit_types[4] = <char>NPY_LONGDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_logit_types[5] = <char>NPY_LONGDOUBLE
 * ufunc_logit_ptr[2*0] = <void*>_func_logitf
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_types[4]) = ((char)NPY_LONGDOUBLE);

  /* "scipy/special/_ufuncs.pyx":10218
 * ufunc_logit_types[3] = <char>NPY_DOUBLE
 * ufunc_logit_types[4] = <char>NPY_LONGDOUBLE
 * ufunc_logit_types[5] = <char>NPY_LONGDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_logit_ptr[2*0] = <void*>_func_logitf
 * ufunc_logit_ptr[2*0+1] = <void*>(<char*>"logit")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_types[5]) = ((char)NPY_LONGDOUBLE);

  /* "scipy/special/_ufuncs.pyx":10219
 * ufunc_logit_types[4] = <char>NPY_LONGDOUBLE
 * ufunc_logit_types[5] = <char>NPY_LONGDOUBLE
 * ufunc_logit_ptr[2*0] = <void*>_func_logitf             # <<<<<<<<<<<<<<
 * ufunc_logit_ptr[2*0+1] = <void*>(<char*>"logit")
 * ufunc_logit_ptr[2*1] = <void*>_func_logit
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_ptr[0]) = ((void *)logitf);

  /* "scipy/special/_ufuncs.pyx":10220
 * ufunc_logit_types[5] = <char>NPY_LONGDOUBLE
 * ufunc_logit_ptr[2*0] = <void*>_func_logitf
 * ufunc_logit_ptr[2*0+1] = <void*>(<char*>"logit")             # <<<<<<<<<<<<<<
 * ufunc_logit_ptr[2*1] = <void*>_func_logit
 * ufunc_logit_ptr[2*1+1] = <void*>(<char*>"logit")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_ptr[1]) = ((void *)((char *)((char *)"logit")));

  /* "scipy/special/_ufuncs.pyx":10221
 * ufunc_logit_ptr[2*0] = <void*>_func_logitf
 * ufunc_logit_ptr[2*0+1] = <void*>(<char*>"logit")
 * ufunc_logit_ptr[2*1] = <void*>_func_logit             # <<<<<<<<<<<<<<
 * ufunc_logit_ptr[2*1+1] = <void*>(<char*>"logit")
 * ufunc_logit_ptr[2*2] = <void*>_func_logitl
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_ptr[2]) = ((void *)logit);

  /* "scipy/special/_ufuncs.pyx":10222
 * ufunc_logit_ptr[2*0+1] = <void*>(<char*>"logit")
 * ufunc_logit_ptr[2*1] = <void*>_func_logit
 * ufunc_logit_ptr[2*1+1] = <void*>(<char*>"logit")             # <<<<<<<<<<<<<<
 * ufunc_logit_ptr[2*2] = <void*>_func_logitl
 * ufunc_logit_ptr[2*2+1] = <void*>(<char*>"logit")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_ptr[3]) = ((void *)((char *)((char *)"logit")));

  /* "scipy/special/_ufuncs.pyx":10223
 * ufunc_logit_ptr[2*1] = <void*>_func_logit
 * ufunc_logit_ptr[2*1+1] = <void*>(<char*>"logit")
 * ufunc_logit_ptr[2*2] = <void*>_func_logitl             # <<<<<<<<<<<<<<
 * ufunc_logit_ptr[2*2+1] = <void*>(<char*>"logit")
 * ufunc_logit_data[0] = &ufunc_logit_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_ptr[4]) = ((void *)logitl);

  /* "scipy/special/_ufuncs.pyx":10224
 * ufunc_logit_ptr[2*1+1] = <void*>(<char*>"logit")
 * ufunc_logit_ptr[2*2] = <void*>_func_logitl
 * ufunc_logit_ptr[2*2+1] = <void*>(<char*>"logit")             # <<<<<<<<<<<<<<
 * ufunc_logit_data[0] = &ufunc_logit_ptr[2*0]
 * ufunc_logit_data[1] = &ufunc_logit_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_ptr[5]) = ((void *)((char *)((char *)"logit")));

  /* "scipy/special/_ufuncs.pyx":10225
 * ufunc_logit_ptr[2*2] = <void*>_func_logitl
 * ufunc_logit_ptr[2*2+1] = <void*>(<char*>"logit")
 * ufunc_logit_data[0] = &ufunc_logit_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_logit_data[1] = &ufunc_logit_ptr[2*1]
 * ufunc_logit_data[2] = &ufunc_logit_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":10226
 * ufunc_logit_ptr[2*2+1] = <void*>(<char*>"logit")
 * ufunc_logit_data[0] = &ufunc_logit_ptr[2*0]
 * ufunc_logit_data[1] = &ufunc_logit_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_logit_data[2] = &ufunc_logit_ptr[2*2]
 * logit = np.PyUFunc_FromFuncAndData(ufunc_logit_loops, ufunc_logit_data, ufunc_logit_types, 3, 1, 1, 0, "logit", ufunc_logit_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":10227
 * ufunc_logit_data[0] = &ufunc_logit_ptr[2*0]
 * ufunc_logit_data[1] = &ufunc_logit_ptr[2*1]
 * ufunc_logit_data[2] = &ufunc_logit_ptr[2*2]             # <<<<<<<<<<<<<<
 * logit = np.PyUFunc_FromFuncAndData(ufunc_logit_loops, ufunc_logit_data, ufunc_logit_types, 3, 1, 1, 0, "logit", ufunc_logit_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":10228
 * ufunc_logit_data[1] = &ufunc_logit_ptr[2*1]
 * ufunc_logit_data[2] = &ufunc_logit_ptr[2*2]
 * logit = np.PyUFunc_FromFuncAndData(ufunc_logit_loops, ufunc_logit_data, ufunc_logit_types, 3, 1, 1, 0, "logit", ufunc_logit_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_lpmv_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_types, 3, 1, 1, 0, ((char *)"logit"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_logit_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_logit, __pyx_t_2) < 0) __PYX_ERR(1, 10228, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":10234
 * cdef void *ufunc_lpmv_data[2]
 * cdef char ufunc_lpmv_types[8]
 * cdef char *ufunc_lpmv_doc = (             # <<<<<<<<<<<<<<
 *     "lpmv(m, v, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_doc = ((char *)"lpmv(m, v, x)\n\nAssociated Legendre function of integer order and real degree.\n\nDefined as\n\n.. math::\n\n    P_v^m = (-1)^m (1 - x^2)^{m/2} \\frac{d^m}{dx^m} P_v(x)\n\nwhere\n\n.. math::\n\n    P_v = \\sum_{k = 0}^\\infty \\frac{(-v)_k (v + 1)_k}{(k!)^2}\n            \\left(\\frac{1 - x}{2}\\right)^k\n\nis the Legendre function of the first kind. Here :math:`(\\cdot)_k`\nis the Pochhammer symbol; see `poch`.\n\nParameters\n----------\nm : array_like\n    Order (int or float). If passed a float not equal to an\n    integer the function returns NaN.\nv : array_like\n    Degree (float).\nx : array_like\n    Argument (float). Must have ``|x| <= 1``.\n\nReturns\n-------\npmv : ndarray\n    Value of the associated Legendre function.\n\nSee Also\n--------\nlpmn : Compute the associated Legendre function for all orders\n       ``0, ..., m`` and degrees ``0, ..., n``.\nclpmn : Compute the associated Legendre function at complex\n        arguments.\n\nNotes\n-----\nNote that this implementation includes the Condon-Shortley phase.\n\nReferences\n----------\n.. [1] Zhang, Jin, \"Computation of Special Functions\", John Wiley\n       and Sons, Inc, 1996.");

  /* "scipy/special/_ufuncs.pyx":10285
 *     ".. [1] Zhang, Jin, \"Computation of Special Functions\", John Wiley\n"
 *     "       and Sons, Inc, 1996.")
 * ufunc_lpmv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_lpmv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_lpmv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":10286
 *     "       and Sons, Inc, 1996.")
 * ufunc_lpmv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_lpmv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_lpmv_types[0] = <char>NPY_FLOAT
 * ufunc_lpmv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":10287
 * ufunc_lpmv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_lpmv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_lpmv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_lpmv_types[1] = <char>NPY_FLOAT
 * ufunc_lpmv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10288
 * ufunc_lpmv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_lpmv_types[0] = <char>NPY_FLOAT
 * ufunc_lpmv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_lpmv_types[2] = <char>NPY_FLOAT
 * ufunc_lpmv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10289
 * ufunc_lpmv_types[0] = <char>NPY_FLOAT
 * ufunc_lpmv_types[1] = <char>NPY_FLOAT
 * ufunc_lpmv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_lpmv_types[3] = <char>NPY_FLOAT
 * ufunc_lpmv_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10290
 * ufunc_lpmv_types[1] = <char>NPY_FLOAT
 * ufunc_lpmv_types[2] = <char>NPY_FLOAT
 * ufunc_lpmv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_lpmv_types[4] = <char>NPY_DOUBLE
 * ufunc_lpmv_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10291
 * ufunc_lpmv_types[2] = <char>NPY_FLOAT
 * ufunc_lpmv_types[3] = <char>NPY_FLOAT
 * ufunc_lpmv_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_lpmv_types[5] = <char>NPY_DOUBLE
 * ufunc_lpmv_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10292
 * ufunc_lpmv_types[3] = <char>NPY_FLOAT
 * ufunc_lpmv_types[4] = <char>NPY_DOUBLE
 * ufunc_lpmv_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_lpmv_types[6] = <char>NPY_DOUBLE
 * ufunc_lpmv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10293
 * ufunc_lpmv_types[4] = <char>NPY_DOUBLE
 * ufunc_lpmv_types[5] = <char>NPY_DOUBLE
 * ufunc_lpmv_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_lpmv_types[7] = <char>NPY_DOUBLE
 * ufunc_lpmv_ptr[2*0] = <void*>_func_pmv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10294
 * ufunc_lpmv_types[5] = <char>NPY_DOUBLE
 * ufunc_lpmv_types[6] = <char>NPY_DOUBLE
 * ufunc_lpmv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_lpmv_ptr[2*0] = <void*>_func_pmv_wrap
 * ufunc_lpmv_ptr[2*0+1] = <void*>(<char*>"lpmv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10295
 * ufunc_lpmv_types[6] = <char>NPY_DOUBLE
 * ufunc_lpmv_types[7] = <char>NPY_DOUBLE
 * ufunc_lpmv_ptr[2*0] = <void*>_func_pmv_wrap             # <<<<<<<<<<<<<<
 * ufunc_lpmv_ptr[2*0+1] = <void*>(<char*>"lpmv")
 * ufunc_lpmv_ptr[2*1] = <void*>_func_pmv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_ptr[0]) = ((void *)pmv_wrap);

  /* "scipy/special/_ufuncs.pyx":10296
 * ufunc_lpmv_types[7] = <char>NPY_DOUBLE
 * ufunc_lpmv_ptr[2*0] = <void*>_func_pmv_wrap
 * ufunc_lpmv_ptr[2*0+1] = <void*>(<char*>"lpmv")             # <<<<<<<<<<<<<<
 * ufunc_lpmv_ptr[2*1] = <void*>_func_pmv_wrap
 * ufunc_lpmv_ptr[2*1+1] = <void*>(<char*>"lpmv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_ptr[1]) = ((void *)((char *)((char *)"lpmv")));

  /* "scipy/special/_ufuncs.pyx":10297
 * ufunc_lpmv_ptr[2*0] = <void*>_func_pmv_wrap
 * ufunc_lpmv_ptr[2*0+1] = <void*>(<char*>"lpmv")
 * ufunc_lpmv_ptr[2*1] = <void*>_func_pmv_wrap             # <<<<<<<<<<<<<<
 * ufunc_lpmv_ptr[2*1+1] = <void*>(<char*>"lpmv")
 * ufunc_lpmv_data[0] = &ufunc_lpmv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_ptr[2]) = ((void *)pmv_wrap);

  /* "scipy/special/_ufuncs.pyx":10298
 * ufunc_lpmv_ptr[2*0+1] = <void*>(<char*>"lpmv")
 * ufunc_lpmv_ptr[2*1] = <void*>_func_pmv_wrap
 * ufunc_lpmv_ptr[2*1+1] = <void*>(<char*>"lpmv")             # <<<<<<<<<<<<<<
 * ufunc_lpmv_data[0] = &ufunc_lpmv_ptr[2*0]
 * ufunc_lpmv_data[1] = &ufunc_lpmv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_ptr[3]) = ((void *)((char *)((char *)"lpmv")));

  /* "scipy/special/_ufuncs.pyx":10299
 * ufunc_lpmv_ptr[2*1] = <void*>_func_pmv_wrap
 * ufunc_lpmv_ptr[2*1+1] = <void*>(<char*>"lpmv")
 * ufunc_lpmv_data[0] = &ufunc_lpmv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_lpmv_data[1] = &ufunc_lpmv_ptr[2*1]
 * lpmv = np.PyUFunc_FromFuncAndData(ufunc_lpmv_loops, ufunc_lpmv_data, ufunc_lpmv_types, 2, 3, 1, 0, "lpmv", ufunc_lpmv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":10300
 * ufunc_lpmv_ptr[2*1+1] = <void*>(<char*>"lpmv")
 * ufunc_lpmv_data[0] = &ufunc_lpmv_ptr[2*0]
 * ufunc_lpmv_data[1] = &ufunc_lpmv_ptr[2*1]             # <<<<<<<<<<<<<<
 * lpmv = np.PyUFunc_FromFuncAndData(ufunc_lpmv_loops, ufunc_lpmv_data, ufunc_lpmv_types, 2, 3, 1, 0, "lpmv", ufunc_lpmv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":10301
 * ufunc_lpmv_data[0] = &ufunc_lpmv_ptr[2*0]
 * ufunc_lpmv_data[1] = &ufunc_lpmv_ptr[2*1]
 * lpmv = np.PyUFunc_FromFuncAndData(ufunc_lpmv_loops, ufunc_lpmv_data, ufunc_lpmv_types, 2, 3, 1, 0, "lpmv", ufunc_lpmv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_mathieu_a_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_types, 2, 3, 1, 0, ((char *)"lpmv"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_lpmv_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10301, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_lpmv, __pyx_t_2) < 0) __PYX_ERR(1, 10301, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":10307
 * cdef void *ufunc_mathieu_a_data[2]
 * cdef char ufunc_mathieu_a_types[6]
 * cdef char *ufunc_mathieu_a_doc = (             # <<<<<<<<<<<<<<
 *     "mathieu_a(m, q)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_doc = ((char *)"mathieu_a(m, q)\n\nCharacteristic value of even Mathieu functions\n\nReturns the characteristic value for the even solution,\n``ce_m(z, q)``, of Mathieu's equation.");

  /* "scipy/special/_ufuncs.pyx":10314
 *     "Returns the characteristic value for the even solution,\n"
 *     "``ce_m(z, q)``, of Mathieu's equation.")
 * ufunc_mathieu_a_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_mathieu_a_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":10315
 *     "``ce_m(z, q)``, of Mathieu's equation.")
 * ufunc_mathieu_a_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_mathieu_a_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_a_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":10316
 * ufunc_mathieu_a_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_mathieu_a_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_mathieu_a_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_a_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10317
 * ufunc_mathieu_a_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_mathieu_a_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_a_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_a_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10318
 * ufunc_mathieu_a_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_a_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_a_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_types[3] = <char>NPY_DOUBLE
 * ufunc_mathieu_a_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10319
 * ufunc_mathieu_a_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_a_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_a_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_types[4] = <char>NPY_DOUBLE
 * ufunc_mathieu_a_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10320
 * ufunc_mathieu_a_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_a_types[3] = <char>NPY_DOUBLE
 * ufunc_mathieu_a_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_a_ptr[2*0] = <void*>_func_cem_cva_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10321
 * ufunc_mathieu_a_types[3] = <char>NPY_DOUBLE
 * ufunc_mathieu_a_types[4] = <char>NPY_DOUBLE
 * ufunc_mathieu_a_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_ptr[2*0] = <void*>_func_cem_cva_wrap
 * ufunc_mathieu_a_ptr[2*0+1] = <void*>(<char*>"mathieu_a")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10322
 * ufunc_mathieu_a_types[4] = <char>NPY_DOUBLE
 * ufunc_mathieu_a_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_a_ptr[2*0] = <void*>_func_cem_cva_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_ptr[2*0+1] = <void*>(<char*>"mathieu_a")
 * ufunc_mathieu_a_ptr[2*1] = <void*>_func_cem_cva_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_ptr[0]) = ((void *)cem_cva_wrap);

  /* "scipy/special/_ufuncs.pyx":10323
 * ufunc_mathieu_a_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_a_ptr[2*0] = <void*>_func_cem_cva_wrap
 * ufunc_mathieu_a_ptr[2*0+1] = <void*>(<char*>"mathieu_a")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_ptr[2*1] = <void*>_func_cem_cva_wrap
 * ufunc_mathieu_a_ptr[2*1+1] = <void*>(<char*>"mathieu_a")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_ptr[1]) = ((void *)((char *)((char *)"mathieu_a")));

  /* "scipy/special/_ufuncs.pyx":10324
 * ufunc_mathieu_a_ptr[2*0] = <void*>_func_cem_cva_wrap
 * ufunc_mathieu_a_ptr[2*0+1] = <void*>(<char*>"mathieu_a")
 * ufunc_mathieu_a_ptr[2*1] = <void*>_func_cem_cva_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_ptr[2*1+1] = <void*>(<char*>"mathieu_a")
 * ufunc_mathieu_a_data[0] = &ufunc_mathieu_a_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_ptr[2]) = ((void *)cem_cva_wrap);

  /* "scipy/special/_ufuncs.pyx":10325
 * ufunc_mathieu_a_ptr[2*0+1] = <void*>(<char*>"mathieu_a")
 * ufunc_mathieu_a_ptr[2*1] = <void*>_func_cem_cva_wrap
 * ufunc_mathieu_a_ptr[2*1+1] = <void*>(<char*>"mathieu_a")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_data[0] = &ufunc_mathieu_a_ptr[2*0]
 * ufunc_mathieu_a_data[1] = &ufunc_mathieu_a_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_ptr[3]) = ((void *)((char *)((char *)"mathieu_a")));

  /* "scipy/special/_ufuncs.pyx":10326
 * ufunc_mathieu_a_ptr[2*1] = <void*>_func_cem_cva_wrap
 * ufunc_mathieu_a_ptr[2*1+1] = <void*>(<char*>"mathieu_a")
 * ufunc_mathieu_a_data[0] = &ufunc_mathieu_a_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_mathieu_a_data[1] = &ufunc_mathieu_a_ptr[2*1]
 * mathieu_a = np.PyUFunc_FromFuncAndData(ufunc_mathieu_a_loops, ufunc_mathieu_a_data, ufunc_mathieu_a_types, 2, 2, 1, 0, "mathieu_a", ufunc_mathieu_a_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":10327
 * ufunc_mathieu_a_ptr[2*1+1] = <void*>(<char*>"mathieu_a")
 * ufunc_mathieu_a_data[0] = &ufunc_mathieu_a_ptr[2*0]
 * ufunc_mathieu_a_data[1] = &ufunc_mathieu_a_ptr[2*1]             # <<<<<<<<<<<<<<
 * mathieu_a = np.PyUFunc_FromFuncAndData(ufunc_mathieu_a_loops, ufunc_mathieu_a_data, ufunc_mathieu_a_types, 2, 2, 1, 0, "mathieu_a", ufunc_mathieu_a_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":10328
 * ufunc_mathieu_a_data[0] = &ufunc_mathieu_a_ptr[2*0]
 * ufunc_mathieu_a_data[1] = &ufunc_mathieu_a_ptr[2*1]
 * mathieu_a = np.PyUFunc_FromFuncAndData(ufunc_mathieu_a_loops, ufunc_mathieu_a_data, ufunc_mathieu_a_types, 2, 2, 1, 0, "mathieu_a", ufunc_mathieu_a_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_mathieu_b_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_types, 2, 2, 1, 0, ((char *)"mathieu_a"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_a_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_mathieu_a, __pyx_t_2) < 0) __PYX_ERR(1, 10328, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":10334
 * cdef void *ufunc_mathieu_b_data[2]
 * cdef char ufunc_mathieu_b_types[6]
 * cdef char *ufunc_mathieu_b_doc = (             # <<<<<<<<<<<<<<
 *     "mathieu_b(m, q)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_doc = ((char *)"mathieu_b(m, q)\n\nCharacteristic value of odd Mathieu functions\n\nReturns the characteristic value for the odd solution,\n``se_m(z, q)``, of Mathieu's equation.");

  /* "scipy/special/_ufuncs.pyx":10341
 *     "Returns the characteristic value for the odd solution,\n"
 *     "``se_m(z, q)``, of Mathieu's equation.")
 * ufunc_mathieu_b_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_mathieu_b_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":10342
 *     "``se_m(z, q)``, of Mathieu's equation.")
 * ufunc_mathieu_b_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_mathieu_b_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_b_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":10343
 * ufunc_mathieu_b_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_mathieu_b_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_mathieu_b_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_b_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10344
 * ufunc_mathieu_b_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_mathieu_b_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_b_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_b_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10345
 * ufunc_mathieu_b_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_b_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_b_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_types[3] = <char>NPY_DOUBLE
 * ufunc_mathieu_b_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10346
 * ufunc_mathieu_b_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_b_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_b_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_types[4] = <char>NPY_DOUBLE
 * ufunc_mathieu_b_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10347
 * ufunc_mathieu_b_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_b_types[3] = <char>NPY_DOUBLE
 * ufunc_mathieu_b_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_b_ptr[2*0] = <void*>_func_sem_cva_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10348
 * ufunc_mathieu_b_types[3] = <char>NPY_DOUBLE
 * ufunc_mathieu_b_types[4] = <char>NPY_DOUBLE
 * ufunc_mathieu_b_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_ptr[2*0] = <void*>_func_sem_cva_wrap
 * ufunc_mathieu_b_ptr[2*0+1] = <void*>(<char*>"mathieu_b")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10349
 * ufunc_mathieu_b_types[4] = <char>NPY_DOUBLE
 * ufunc_mathieu_b_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_b_ptr[2*0] = <void*>_func_sem_cva_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_ptr[2*0+1] = <void*>(<char*>"mathieu_b")
 * ufunc_mathieu_b_ptr[2*1] = <void*>_func_sem_cva_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_ptr[0]) = ((void *)sem_cva_wrap);

  /* "scipy/special/_ufuncs.pyx":10350
 * ufunc_mathieu_b_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_b_ptr[2*0] = <void*>_func_sem_cva_wrap
 * ufunc_mathieu_b_ptr[2*0+1] = <void*>(<char*>"mathieu_b")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_ptr[2*1] = <void*>_func_sem_cva_wrap
 * ufunc_mathieu_b_ptr[2*1+1] = <void*>(<char*>"mathieu_b")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_ptr[1]) = ((void *)((char *)((char *)"mathieu_b")));

  /* "scipy/special/_ufuncs.pyx":10351
 * ufunc_mathieu_b_ptr[2*0] = <void*>_func_sem_cva_wrap
 * ufunc_mathieu_b_ptr[2*0+1] = <void*>(<char*>"mathieu_b")
 * ufunc_mathieu_b_ptr[2*1] = <void*>_func_sem_cva_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_ptr[2*1+1] = <void*>(<char*>"mathieu_b")
 * ufunc_mathieu_b_data[0] = &ufunc_mathieu_b_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_ptr[2]) = ((void *)sem_cva_wrap);

  /* "scipy/special/_ufuncs.pyx":10352
 * ufunc_mathieu_b_ptr[2*0+1] = <void*>(<char*>"mathieu_b")
 * ufunc_mathieu_b_ptr[2*1] = <void*>_func_sem_cva_wrap
 * ufunc_mathieu_b_ptr[2*1+1] = <void*>(<char*>"mathieu_b")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_data[0] = &ufunc_mathieu_b_ptr[2*0]
 * ufunc_mathieu_b_data[1] = &ufunc_mathieu_b_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_ptr[3]) = ((void *)((char *)((char *)"mathieu_b")));

  /* "scipy/special/_ufuncs.pyx":10353
 * ufunc_mathieu_b_ptr[2*1] = <void*>_func_sem_cva_wrap
 * ufunc_mathieu_b_ptr[2*1+1] = <void*>(<char*>"mathieu_b")
 * ufunc_mathieu_b_data[0] = &ufunc_mathieu_b_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_mathieu_b_data[1] = &ufunc_mathieu_b_ptr[2*1]
 * mathieu_b = np.PyUFunc_FromFuncAndData(ufunc_mathieu_b_loops, ufunc_mathieu_b_data, ufunc_mathieu_b_types, 2, 2, 1, 0, "mathieu_b", ufunc_mathieu_b_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":10354
 * ufunc_mathieu_b_ptr[2*1+1] = <void*>(<char*>"mathieu_b")
 * ufunc_mathieu_b_data[0] = &ufunc_mathieu_b_ptr[2*0]
 * ufunc_mathieu_b_data[1] = &ufunc_mathieu_b_ptr[2*1]             # <<<<<<<<<<<<<<
 * mathieu_b = np.PyUFunc_FromFuncAndData(ufunc_mathieu_b_loops, ufunc_mathieu_b_data, ufunc_mathieu_b_types, 2, 2, 1, 0, "mathieu_b", ufunc_mathieu_b_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":10355
 * ufunc_mathieu_b_data[0] = &ufunc_mathieu_b_ptr[2*0]
 * ufunc_mathieu_b_data[1] = &ufunc_mathieu_b_ptr[2*1]
 * mathieu_b = np.PyUFunc_FromFuncAndData(ufunc_mathieu_b_loops, ufunc_mathieu_b_data, ufunc_mathieu_b_types, 2, 2, 1, 0, "mathieu_b", ufunc_mathieu_b_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_mathieu_cem_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_types, 2, 2, 1, 0, ((char *)"mathieu_b"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_b_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10355, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_mathieu_b, __pyx_t_2) < 0) __PYX_ERR(1, 10355, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":10361
 * cdef void *ufunc_mathieu_cem_data[2]
 * cdef char ufunc_mathieu_cem_types[10]
 * cdef char *ufunc_mathieu_cem_doc = (             # <<<<<<<<<<<<<<
 *     "mathieu_cem(m, q, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_doc = ((char *)"mathieu_cem(m, q, x)\n\nEven Mathieu function and its derivative\n\nReturns the even Mathieu function, ``ce_m(x, q)``, of order `m` and\nparameter `q` evaluated at `x` (given in degrees).  Also returns the\nderivative with respect to `x` of ce_m(x, q)\n\nParameters\n----------\nm\n    Order of the function\nq\n    Parameter of the function\nx\n    Argument of the function, *given in degrees, not radians*\n\nReturns\n-------\ny\n    Value of the function\nyp\n    Value of the derivative vs x");

  /* "scipy/special/_ufuncs.pyx":10385
 *     "yp\n"
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_cem_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_cem_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_fff_ff);

  /* "scipy/special/_ufuncs.pyx":10386
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_cem_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_cem_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_ddd_dd);

  /* "scipy/special/_ufuncs.pyx":10387
 * ufunc_mathieu_cem_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_cem_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_cem_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10388
 * ufunc_mathieu_cem_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_cem_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10389
 * ufunc_mathieu_cem_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10390
 * ufunc_mathieu_cem_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10391
 * ufunc_mathieu_cem_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10392
 * ufunc_mathieu_cem_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10393
 * ufunc_mathieu_cem_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_cem_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10394
 * ufunc_mathieu_cem_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10395
 * ufunc_mathieu_cem_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_ptr[2*0] = <void*>_func_cem_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10396
 * ufunc_mathieu_cem_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_ptr[2*0] = <void*>_func_cem_wrap
 * ufunc_mathieu_cem_ptr[2*0+1] = <void*>(<char*>"mathieu_cem")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10397
 * ufunc_mathieu_cem_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_ptr[2*0] = <void*>_func_cem_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_ptr[2*0+1] = <void*>(<char*>"mathieu_cem")
 * ufunc_mathieu_cem_ptr[2*1] = <void*>_func_cem_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_ptr[0]) = ((void *)cem_wrap);

  /* "scipy/special/_ufuncs.pyx":10398
 * ufunc_mathieu_cem_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_cem_ptr[2*0] = <void*>_func_cem_wrap
 * ufunc_mathieu_cem_ptr[2*0+1] = <void*>(<char*>"mathieu_cem")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_ptr[2*1] = <void*>_func_cem_wrap
 * ufunc_mathieu_cem_ptr[2*1+1] = <void*>(<char*>"mathieu_cem")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_ptr[1]) = ((void *)((char *)((char *)"mathieu_cem")));

  /* "scipy/special/_ufuncs.pyx":10399
 * ufunc_mathieu_cem_ptr[2*0] = <void*>_func_cem_wrap
 * ufunc_mathieu_cem_ptr[2*0+1] = <void*>(<char*>"mathieu_cem")
 * ufunc_mathieu_cem_ptr[2*1] = <void*>_func_cem_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_ptr[2*1+1] = <void*>(<char*>"mathieu_cem")
 * ufunc_mathieu_cem_data[0] = &ufunc_mathieu_cem_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_ptr[2]) = ((void *)cem_wrap);

  /* "scipy/special/_ufuncs.pyx":10400
 * ufunc_mathieu_cem_ptr[2*0+1] = <void*>(<char*>"mathieu_cem")
 * ufunc_mathieu_cem_ptr[2*1] = <void*>_func_cem_wrap
 * ufunc_mathieu_cem_ptr[2*1+1] = <void*>(<char*>"mathieu_cem")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_data[0] = &ufunc_mathieu_cem_ptr[2*0]
 * ufunc_mathieu_cem_data[1] = &ufunc_mathieu_cem_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_ptr[3]) = ((void *)((char *)((char *)"mathieu_cem")));

  /* "scipy/special/_ufuncs.pyx":10401
 * ufunc_mathieu_cem_ptr[2*1] = <void*>_func_cem_wrap
 * ufunc_mathieu_cem_ptr[2*1+1] = <void*>(<char*>"mathieu_cem")
 * ufunc_mathieu_cem_data[0] = &ufunc_mathieu_cem_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_mathieu_cem_data[1] = &ufunc_mathieu_cem_ptr[2*1]
 * mathieu_cem = np.PyUFunc_FromFuncAndData(ufunc_mathieu_cem_loops, ufunc_mathieu_cem_data, ufunc_mathieu_cem_types, 2, 3, 2, 0, "mathieu_cem", ufunc_mathieu_cem_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":10402
 * ufunc_mathieu_cem_ptr[2*1+1] = <void*>(<char*>"mathieu_cem")
 * ufunc_mathieu_cem_data[0] = &ufunc_mathieu_cem_ptr[2*0]
 * ufunc_mathieu_cem_data[1] = &ufunc_mathieu_cem_ptr[2*1]             # <<<<<<<<<<<<<<
 * mathieu_cem = np.PyUFunc_FromFuncAndData(ufunc_mathieu_cem_loops, ufunc_mathieu_cem_data, ufunc_mathieu_cem_types, 2, 3, 2, 0, "mathieu_cem", ufunc_mathieu_cem_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":10403
 * ufunc_mathieu_cem_data[0] = &ufunc_mathieu_cem_ptr[2*0]
 * ufunc_mathieu_cem_data[1] = &ufunc_mathieu_cem_ptr[2*1]
 * mathieu_cem = np.PyUFunc_FromFuncAndData(ufunc_mathieu_cem_loops, ufunc_mathieu_cem_data, ufunc_mathieu_cem_types, 2, 3, 2, 0, "mathieu_cem", ufunc_mathieu_cem_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_mathieu_modcem1_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_types, 2, 3, 2, 0, ((char *)"mathieu_cem"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_cem_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10403, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_mathieu_cem, __pyx_t_2) < 0) __PYX_ERR(1, 10403, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":10409
 * cdef void *ufunc_mathieu_modcem1_data[2]
 * cdef char ufunc_mathieu_modcem1_types[10]
 * cdef char *ufunc_mathieu_modcem1_doc = (             # <<<<<<<<<<<<<<
 *     "mathieu_modcem1(m, q, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_doc = ((char *)"mathieu_modcem1(m, q, x)\n\nEven modified Mathieu function of the first kind and its derivative\n\nEvaluates the even modified Mathieu function of the first kind,\n``Mc1m(x, q)``, and its derivative at `x` for order `m` and parameter\n`q`.\n\nReturns\n-------\ny\n    Value of the function\nyp\n    Value of the derivative vs x");

  /* "scipy/special/_ufuncs.pyx":10424
 *     "yp\n"
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_modcem1_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modcem1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_fff_ff);

  /* "scipy/special/_ufuncs.pyx":10425
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_modcem1_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_modcem1_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_ddd_dd);

  /* "scipy/special/_ufuncs.pyx":10426
 * ufunc_mathieu_modcem1_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_modcem1_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modcem1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10427
 * ufunc_mathieu_modcem1_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modcem1_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10428
 * ufunc_mathieu_modcem1_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10429
 * ufunc_mathieu_modcem1_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10430
 * ufunc_mathieu_modcem1_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10431
 * ufunc_mathieu_modcem1_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10432
 * ufunc_mathieu_modcem1_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem1_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10433
 * ufunc_mathieu_modcem1_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10434
 * ufunc_mathieu_modcem1_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_ptr[2*0] = <void*>_func_mcm1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10435
 * ufunc_mathieu_modcem1_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_ptr[2*0] = <void*>_func_mcm1_wrap
 * ufunc_mathieu_modcem1_ptr[2*0+1] = <void*>(<char*>"mathieu_modcem1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10436
 * ufunc_mathieu_modcem1_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_ptr[2*0] = <void*>_func_mcm1_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_ptr[2*0+1] = <void*>(<char*>"mathieu_modcem1")
 * ufunc_mathieu_modcem1_ptr[2*1] = <void*>_func_mcm1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_ptr[0]) = ((void *)mcm1_wrap);

  /* "scipy/special/_ufuncs.pyx":10437
 * ufunc_mathieu_modcem1_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem1_ptr[2*0] = <void*>_func_mcm1_wrap
 * ufunc_mathieu_modcem1_ptr[2*0+1] = <void*>(<char*>"mathieu_modcem1")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_ptr[2*1] = <void*>_func_mcm1_wrap
 * ufunc_mathieu_modcem1_ptr[2*1+1] = <void*>(<char*>"mathieu_modcem1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_ptr[1]) = ((void *)((char *)((char *)"mathieu_modcem1")));

  /* "scipy/special/_ufuncs.pyx":10438
 * ufunc_mathieu_modcem1_ptr[2*0] = <void*>_func_mcm1_wrap
 * ufunc_mathieu_modcem1_ptr[2*0+1] = <void*>(<char*>"mathieu_modcem1")
 * ufunc_mathieu_modcem1_ptr[2*1] = <void*>_func_mcm1_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_ptr[2*1+1] = <void*>(<char*>"mathieu_modcem1")
 * ufunc_mathieu_modcem1_data[0] = &ufunc_mathieu_modcem1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_ptr[2]) = ((void *)mcm1_wrap);

  /* "scipy/special/_ufuncs.pyx":10439
 * ufunc_mathieu_modcem1_ptr[2*0+1] = <void*>(<char*>"mathieu_modcem1")
 * ufunc_mathieu_modcem1_ptr[2*1] = <void*>_func_mcm1_wrap
 * ufunc_mathieu_modcem1_ptr[2*1+1] = <void*>(<char*>"mathieu_modcem1")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_data[0] = &ufunc_mathieu_modcem1_ptr[2*0]
 * ufunc_mathieu_modcem1_data[1] = &ufunc_mathieu_modcem1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_ptr[3]) = ((void *)((char *)((char *)"mathieu_modcem1")));

  /* "scipy/special/_ufuncs.pyx":10440
 * ufunc_mathieu_modcem1_ptr[2*1] = <void*>_func_mcm1_wrap
 * ufunc_mathieu_modcem1_ptr[2*1+1] = <void*>(<char*>"mathieu_modcem1")
 * ufunc_mathieu_modcem1_data[0] = &ufunc_mathieu_modcem1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem1_data[1] = &ufunc_mathieu_modcem1_ptr[2*1]
 * mathieu_modcem1 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modcem1_loops, ufunc_mathieu_modcem1_data, ufunc_mathieu_modcem1_types, 2, 3, 2, 0, "mathieu_modcem1", ufunc_mathieu_modcem1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":10441
 * ufunc_mathieu_modcem1_ptr[2*1+1] = <void*>(<char*>"mathieu_modcem1")
 * ufunc_mathieu_modcem1_data[0] = &ufunc_mathieu_modcem1_ptr[2*0]
 * ufunc_mathieu_modcem1_data[1] = &ufunc_mathieu_modcem1_ptr[2*1]             # <<<<<<<<<<<<<<
 * mathieu_modcem1 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modcem1_loops, ufunc_mathieu_modcem1_data, ufunc_mathieu_modcem1_types, 2, 3, 2, 0, "mathieu_modcem1", ufunc_mathieu_modcem1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":10442
 * ufunc_mathieu_modcem1_data[0] = &ufunc_mathieu_modcem1_ptr[2*0]
 * ufunc_mathieu_modcem1_data[1] = &ufunc_mathieu_modcem1_ptr[2*1]
 * mathieu_modcem1 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modcem1_loops, ufunc_mathieu_modcem1_data, ufunc_mathieu_modcem1_types, 2, 3, 2, 0, "mathieu_modcem1", ufunc_mathieu_modcem1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_mathieu_modcem2_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_types, 2, 3, 2, 0, ((char *)"mathieu_modcem1"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem1_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10442, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_mathieu_modcem1, __pyx_t_2) < 0) __PYX_ERR(1, 10442, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":10448
 * cdef void *ufunc_mathieu_modcem2_data[2]
 * cdef char ufunc_mathieu_modcem2_types[10]
 * cdef char *ufunc_mathieu_modcem2_doc = (             # <<<<<<<<<<<<<<
 *     "mathieu_modcem2(m, q, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_doc = ((char *)"mathieu_modcem2(m, q, x)\n\nEven modified Mathieu function of the second kind and its derivative\n\nEvaluates the even modified Mathieu function of the second kind,\nMc2m(x, q), and its derivative at `x` (given in degrees) for order `m`\nand parameter `q`.\n\nReturns\n-------\ny\n    Value of the function\nyp\n    Value of the derivative vs x");

  /* "scipy/special/_ufuncs.pyx":10463
 *     "yp\n"
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_modcem2_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modcem2_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_fff_ff);

  /* "scipy/special/_ufuncs.pyx":10464
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_modcem2_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_modcem2_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_ddd_dd);

  /* "scipy/special/_ufuncs.pyx":10465
 * ufunc_mathieu_modcem2_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_modcem2_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modcem2_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10466
 * ufunc_mathieu_modcem2_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modcem2_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10467
 * ufunc_mathieu_modcem2_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10468
 * ufunc_mathieu_modcem2_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10469
 * ufunc_mathieu_modcem2_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10470
 * ufunc_mathieu_modcem2_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10471
 * ufunc_mathieu_modcem2_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modcem2_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10472
 * ufunc_mathieu_modcem2_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10473
 * ufunc_mathieu_modcem2_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_ptr[2*0] = <void*>_func_mcm2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10474
 * ufunc_mathieu_modcem2_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_ptr[2*0] = <void*>_func_mcm2_wrap
 * ufunc_mathieu_modcem2_ptr[2*0+1] = <void*>(<char*>"mathieu_modcem2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10475
 * ufunc_mathieu_modcem2_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_ptr[2*0] = <void*>_func_mcm2_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_ptr[2*0+1] = <void*>(<char*>"mathieu_modcem2")
 * ufunc_mathieu_modcem2_ptr[2*1] = <void*>_func_mcm2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_ptr[0]) = ((void *)mcm2_wrap);

  /* "scipy/special/_ufuncs.pyx":10476
 * ufunc_mathieu_modcem2_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modcem2_ptr[2*0] = <void*>_func_mcm2_wrap
 * ufunc_mathieu_modcem2_ptr[2*0+1] = <void*>(<char*>"mathieu_modcem2")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_ptr[2*1] = <void*>_func_mcm2_wrap
 * ufunc_mathieu_modcem2_ptr[2*1+1] = <void*>(<char*>"mathieu_modcem2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_ptr[1]) = ((void *)((char *)((char *)"mathieu_modcem2")));

  /* "scipy/special/_ufuncs.pyx":10477
 * ufunc_mathieu_modcem2_ptr[2*0] = <void*>_func_mcm2_wrap
 * ufunc_mathieu_modcem2_ptr[2*0+1] = <void*>(<char*>"mathieu_modcem2")
 * ufunc_mathieu_modcem2_ptr[2*1] = <void*>_func_mcm2_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_ptr[2*1+1] = <void*>(<char*>"mathieu_modcem2")
 * ufunc_mathieu_modcem2_data[0] = &ufunc_mathieu_modcem2_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_ptr[2]) = ((void *)mcm2_wrap);

  /* "scipy/special/_ufuncs.pyx":10478
 * ufunc_mathieu_modcem2_ptr[2*0+1] = <void*>(<char*>"mathieu_modcem2")
 * ufunc_mathieu_modcem2_ptr[2*1] = <void*>_func_mcm2_wrap
 * ufunc_mathieu_modcem2_ptr[2*1+1] = <void*>(<char*>"mathieu_modcem2")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_data[0] = &ufunc_mathieu_modcem2_ptr[2*0]
 * ufunc_mathieu_modcem2_data[1] = &ufunc_mathieu_modcem2_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_ptr[3]) = ((void *)((char *)((char *)"mathieu_modcem2")));

  /* "scipy/special/_ufuncs.pyx":10479
 * ufunc_mathieu_modcem2_ptr[2*1] = <void*>_func_mcm2_wrap
 * ufunc_mathieu_modcem2_ptr[2*1+1] = <void*>(<char*>"mathieu_modcem2")
 * ufunc_mathieu_modcem2_data[0] = &ufunc_mathieu_modcem2_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modcem2_data[1] = &ufunc_mathieu_modcem2_ptr[2*1]
 * mathieu_modcem2 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modcem2_loops, ufunc_mathieu_modcem2_data, ufunc_mathieu_modcem2_types, 2, 3, 2, 0, "mathieu_modcem2", ufunc_mathieu_modcem2_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":10480
 * ufunc_mathieu_modcem2_ptr[2*1+1] = <void*>(<char*>"mathieu_modcem2")
 * ufunc_mathieu_modcem2_data[0] = &ufunc_mathieu_modcem2_ptr[2*0]
 * ufunc_mathieu_modcem2_data[1] = &ufunc_mathieu_modcem2_ptr[2*1]             # <<<<<<<<<<<<<<
 * mathieu_modcem2 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modcem2_loops, ufunc_mathieu_modcem2_data, ufunc_mathieu_modcem2_types, 2, 3, 2, 0, "mathieu_modcem2", ufunc_mathieu_modcem2_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":10481
 * ufunc_mathieu_modcem2_data[0] = &ufunc_mathieu_modcem2_ptr[2*0]
 * ufunc_mathieu_modcem2_data[1] = &ufunc_mathieu_modcem2_ptr[2*1]
 * mathieu_modcem2 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modcem2_loops, ufunc_mathieu_modcem2_data, ufunc_mathieu_modcem2_types, 2, 3, 2, 0, "mathieu_modcem2", ufunc_mathieu_modcem2_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_mathieu_modsem1_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_types, 2, 3, 2, 0, ((char *)"mathieu_modcem2"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modcem2_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10481, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_mathieu_modcem2, __pyx_t_2) < 0) __PYX_ERR(1, 10481, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":10487
 * cdef void *ufunc_mathieu_modsem1_data[2]
 * cdef char ufunc_mathieu_modsem1_types[10]
 * cdef char *ufunc_mathieu_modsem1_doc = (             # <<<<<<<<<<<<<<
 *     "mathieu_modsem1(m, q, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_doc = ((char *)"mathieu_modsem1(m, q, x)\n\nOdd modified Mathieu function of the first kind and its derivative\n\nEvaluates the odd modified Mathieu function of the first kind,\nMs1m(x, q), and its derivative at `x` (given in degrees) for order `m`\nand parameter `q`.\n\nReturns\n-------\ny\n    Value of the function\nyp\n    Value of the derivative vs x");

  /* "scipy/special/_ufuncs.pyx":10502
 *     "yp\n"
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_modsem1_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modsem1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_fff_ff);

  /* "scipy/special/_ufuncs.pyx":10503
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_modsem1_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_modsem1_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_ddd_dd);

  /* "scipy/special/_ufuncs.pyx":10504
 * ufunc_mathieu_modsem1_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_modsem1_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modsem1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10505
 * ufunc_mathieu_modsem1_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modsem1_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10506
 * ufunc_mathieu_modsem1_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10507
 * ufunc_mathieu_modsem1_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10508
 * ufunc_mathieu_modsem1_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10509
 * ufunc_mathieu_modsem1_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10510
 * ufunc_mathieu_modsem1_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem1_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10511
 * ufunc_mathieu_modsem1_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10512
 * ufunc_mathieu_modsem1_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_ptr[2*0] = <void*>_func_msm1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10513
 * ufunc_mathieu_modsem1_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_ptr[2*0] = <void*>_func_msm1_wrap
 * ufunc_mathieu_modsem1_ptr[2*0+1] = <void*>(<char*>"mathieu_modsem1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10514
 * ufunc_mathieu_modsem1_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_ptr[2*0] = <void*>_func_msm1_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_ptr[2*0+1] = <void*>(<char*>"mathieu_modsem1")
 * ufunc_mathieu_modsem1_ptr[2*1] = <void*>_func_msm1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_ptr[0]) = ((void *)msm1_wrap);

  /* "scipy/special/_ufuncs.pyx":10515
 * ufunc_mathieu_modsem1_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem1_ptr[2*0] = <void*>_func_msm1_wrap
 * ufunc_mathieu_modsem1_ptr[2*0+1] = <void*>(<char*>"mathieu_modsem1")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_ptr[2*1] = <void*>_func_msm1_wrap
 * ufunc_mathieu_modsem1_ptr[2*1+1] = <void*>(<char*>"mathieu_modsem1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_ptr[1]) = ((void *)((char *)((char *)"mathieu_modsem1")));

  /* "scipy/special/_ufuncs.pyx":10516
 * ufunc_mathieu_modsem1_ptr[2*0] = <void*>_func_msm1_wrap
 * ufunc_mathieu_modsem1_ptr[2*0+1] = <void*>(<char*>"mathieu_modsem1")
 * ufunc_mathieu_modsem1_ptr[2*1] = <void*>_func_msm1_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_ptr[2*1+1] = <void*>(<char*>"mathieu_modsem1")
 * ufunc_mathieu_modsem1_data[0] = &ufunc_mathieu_modsem1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_ptr[2]) = ((void *)msm1_wrap);

  /* "scipy/special/_ufuncs.pyx":10517
 * ufunc_mathieu_modsem1_ptr[2*0+1] = <void*>(<char*>"mathieu_modsem1")
 * ufunc_mathieu_modsem1_ptr[2*1] = <void*>_func_msm1_wrap
 * ufunc_mathieu_modsem1_ptr[2*1+1] = <void*>(<char*>"mathieu_modsem1")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_data[0] = &ufunc_mathieu_modsem1_ptr[2*0]
 * ufunc_mathieu_modsem1_data[1] = &ufunc_mathieu_modsem1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_ptr[3]) = ((void *)((char *)((char *)"mathieu_modsem1")));

  /* "scipy/special/_ufuncs.pyx":10518
 * ufunc_mathieu_modsem1_ptr[2*1] = <void*>_func_msm1_wrap
 * ufunc_mathieu_modsem1_ptr[2*1+1] = <void*>(<char*>"mathieu_modsem1")
 * ufunc_mathieu_modsem1_data[0] = &ufunc_mathieu_modsem1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem1_data[1] = &ufunc_mathieu_modsem1_ptr[2*1]
 * mathieu_modsem1 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modsem1_loops, ufunc_mathieu_modsem1_data, ufunc_mathieu_modsem1_types, 2, 3, 2, 0, "mathieu_modsem1", ufunc_mathieu_modsem1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":10519
 * ufunc_mathieu_modsem1_ptr[2*1+1] = <void*>(<char*>"mathieu_modsem1")
 * ufunc_mathieu_modsem1_data[0] = &ufunc_mathieu_modsem1_ptr[2*0]
 * ufunc_mathieu_modsem1_data[1] = &ufunc_mathieu_modsem1_ptr[2*1]             # <<<<<<<<<<<<<<
 * mathieu_modsem1 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modsem1_loops, ufunc_mathieu_modsem1_data, ufunc_mathieu_modsem1_types, 2, 3, 2, 0, "mathieu_modsem1", ufunc_mathieu_modsem1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":10520
 * ufunc_mathieu_modsem1_data[0] = &ufunc_mathieu_modsem1_ptr[2*0]
 * ufunc_mathieu_modsem1_data[1] = &ufunc_mathieu_modsem1_ptr[2*1]
 * mathieu_modsem1 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modsem1_loops, ufunc_mathieu_modsem1_data, ufunc_mathieu_modsem1_types, 2, 3, 2, 0, "mathieu_modsem1", ufunc_mathieu_modsem1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_mathieu_modsem2_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_types, 2, 3, 2, 0, ((char *)"mathieu_modsem1"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem1_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10520, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_mathieu_modsem1, __pyx_t_2) < 0) __PYX_ERR(1, 10520, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":10526
 * cdef void *ufunc_mathieu_modsem2_data[2]
 * cdef char ufunc_mathieu_modsem2_types[10]
 * cdef char *ufunc_mathieu_modsem2_doc = (             # <<<<<<<<<<<<<<
 *     "mathieu_modsem2(m, q, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_doc = ((char *)"mathieu_modsem2(m, q, x)\n\nOdd modified Mathieu function of the second kind and its derivative\n\nEvaluates the odd modified Mathieu function of the second kind,\nMs2m(x, q), and its derivative at `x` (given in degrees) for order `m`\nand parameter q.\n\nReturns\n-------\ny\n    Value of the function\nyp\n    Value of the derivative vs x");

  /* "scipy/special/_ufuncs.pyx":10541
 *     "yp\n"
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_modsem2_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modsem2_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_fff_ff);

  /* "scipy/special/_ufuncs.pyx":10542
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_modsem2_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_modsem2_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_ddd_dd);

  /* "scipy/special/_ufuncs.pyx":10543
 * ufunc_mathieu_modsem2_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_modsem2_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modsem2_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10544
 * ufunc_mathieu_modsem2_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_modsem2_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10545
 * ufunc_mathieu_modsem2_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10546
 * ufunc_mathieu_modsem2_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10547
 * ufunc_mathieu_modsem2_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10548
 * ufunc_mathieu_modsem2_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10549
 * ufunc_mathieu_modsem2_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_modsem2_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10550
 * ufunc_mathieu_modsem2_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10551
 * ufunc_mathieu_modsem2_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_ptr[2*0] = <void*>_func_msm2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10552
 * ufunc_mathieu_modsem2_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_ptr[2*0] = <void*>_func_msm2_wrap
 * ufunc_mathieu_modsem2_ptr[2*0+1] = <void*>(<char*>"mathieu_modsem2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10553
 * ufunc_mathieu_modsem2_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_ptr[2*0] = <void*>_func_msm2_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_ptr[2*0+1] = <void*>(<char*>"mathieu_modsem2")
 * ufunc_mathieu_modsem2_ptr[2*1] = <void*>_func_msm2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_ptr[0]) = ((void *)msm2_wrap);

  /* "scipy/special/_ufuncs.pyx":10554
 * ufunc_mathieu_modsem2_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_modsem2_ptr[2*0] = <void*>_func_msm2_wrap
 * ufunc_mathieu_modsem2_ptr[2*0+1] = <void*>(<char*>"mathieu_modsem2")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_ptr[2*1] = <void*>_func_msm2_wrap
 * ufunc_mathieu_modsem2_ptr[2*1+1] = <void*>(<char*>"mathieu_modsem2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_ptr[1]) = ((void *)((char *)((char *)"mathieu_modsem2")));

  /* "scipy/special/_ufuncs.pyx":10555
 * ufunc_mathieu_modsem2_ptr[2*0] = <void*>_func_msm2_wrap
 * ufunc_mathieu_modsem2_ptr[2*0+1] = <void*>(<char*>"mathieu_modsem2")
 * ufunc_mathieu_modsem2_ptr[2*1] = <void*>_func_msm2_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_ptr[2*1+1] = <void*>(<char*>"mathieu_modsem2")
 * ufunc_mathieu_modsem2_data[0] = &ufunc_mathieu_modsem2_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_ptr[2]) = ((void *)msm2_wrap);

  /* "scipy/special/_ufuncs.pyx":10556
 * ufunc_mathieu_modsem2_ptr[2*0+1] = <void*>(<char*>"mathieu_modsem2")
 * ufunc_mathieu_modsem2_ptr[2*1] = <void*>_func_msm2_wrap
 * ufunc_mathieu_modsem2_ptr[2*1+1] = <void*>(<char*>"mathieu_modsem2")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_data[0] = &ufunc_mathieu_modsem2_ptr[2*0]
 * ufunc_mathieu_modsem2_data[1] = &ufunc_mathieu_modsem2_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_ptr[3]) = ((void *)((char *)((char *)"mathieu_modsem2")));

  /* "scipy/special/_ufuncs.pyx":10557
 * ufunc_mathieu_modsem2_ptr[2*1] = <void*>_func_msm2_wrap
 * ufunc_mathieu_modsem2_ptr[2*1+1] = <void*>(<char*>"mathieu_modsem2")
 * ufunc_mathieu_modsem2_data[0] = &ufunc_mathieu_modsem2_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_mathieu_modsem2_data[1] = &ufunc_mathieu_modsem2_ptr[2*1]
 * mathieu_modsem2 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modsem2_loops, ufunc_mathieu_modsem2_data, ufunc_mathieu_modsem2_types, 2, 3, 2, 0, "mathieu_modsem2", ufunc_mathieu_modsem2_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":10558
 * ufunc_mathieu_modsem2_ptr[2*1+1] = <void*>(<char*>"mathieu_modsem2")
 * ufunc_mathieu_modsem2_data[0] = &ufunc_mathieu_modsem2_ptr[2*0]
 * ufunc_mathieu_modsem2_data[1] = &ufunc_mathieu_modsem2_ptr[2*1]             # <<<<<<<<<<<<<<
 * mathieu_modsem2 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modsem2_loops, ufunc_mathieu_modsem2_data, ufunc_mathieu_modsem2_types, 2, 3, 2, 0, "mathieu_modsem2", ufunc_mathieu_modsem2_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":10559
 * ufunc_mathieu_modsem2_data[0] = &ufunc_mathieu_modsem2_ptr[2*0]
 * ufunc_mathieu_modsem2_data[1] = &ufunc_mathieu_modsem2_ptr[2*1]
 * mathieu_modsem2 = np.PyUFunc_FromFuncAndData(ufunc_mathieu_modsem2_loops, ufunc_mathieu_modsem2_data, ufunc_mathieu_modsem2_types, 2, 3, 2, 0, "mathieu_modsem2", ufunc_mathieu_modsem2_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_mathieu_sem_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_types, 2, 3, 2, 0, ((char *)"mathieu_modsem2"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_modsem2_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10559, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_mathieu_modsem2, __pyx_t_2) < 0) __PYX_ERR(1, 10559, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":10565
 * cdef void *ufunc_mathieu_sem_data[2]
 * cdef char ufunc_mathieu_sem_types[10]
 * cdef char *ufunc_mathieu_sem_doc = (             # <<<<<<<<<<<<<<
 *     "mathieu_sem(m, q, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_doc = ((char *)"mathieu_sem(m, q, x)\n\nOdd Mathieu function and its derivative\n\nReturns the odd Mathieu function, se_m(x, q), of order `m` and\nparameter `q` evaluated at `x` (given in degrees).  Also returns the\nderivative with respect to `x` of se_m(x, q).\n\nParameters\n----------\nm\n    Order of the function\nq\n    Parameter of the function\nx\n    Argument of the function, *given in degrees, not radians*.\n\nReturns\n-------\ny\n    Value of the function\nyp\n    Value of the derivative vs x");

  /* "scipy/special/_ufuncs.pyx":10589
 *     "yp\n"
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_sem_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_sem_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_fff_ff);

  /* "scipy/special/_ufuncs.pyx":10590
 *     "    Value of the derivative vs x")
 * ufunc_mathieu_sem_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_sem_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddd_dd_As_ddd_dd);

  /* "scipy/special/_ufuncs.pyx":10591
 * ufunc_mathieu_sem_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_fff_ff
 * ufunc_mathieu_sem_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_sem_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10592
 * ufunc_mathieu_sem_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddd_dd_As_ddd_dd
 * ufunc_mathieu_sem_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10593
 * ufunc_mathieu_sem_types[0] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10594
 * ufunc_mathieu_sem_types[1] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10595
 * ufunc_mathieu_sem_types[2] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10596
 * ufunc_mathieu_sem_types[3] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10597
 * ufunc_mathieu_sem_types[4] = <char>NPY_FLOAT
 * ufunc_mathieu_sem_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10598
 * ufunc_mathieu_sem_types[5] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10599
 * ufunc_mathieu_sem_types[6] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_ptr[2*0] = <void*>_func_sem_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10600
 * ufunc_mathieu_sem_types[7] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_ptr[2*0] = <void*>_func_sem_wrap
 * ufunc_mathieu_sem_ptr[2*0+1] = <void*>(<char*>"mathieu_sem")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10601
 * ufunc_mathieu_sem_types[8] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_ptr[2*0] = <void*>_func_sem_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_ptr[2*0+1] = <void*>(<char*>"mathieu_sem")
 * ufunc_mathieu_sem_ptr[2*1] = <void*>_func_sem_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_ptr[0]) = ((void *)sem_wrap);

  /* "scipy/special/_ufuncs.pyx":10602
 * ufunc_mathieu_sem_types[9] = <char>NPY_DOUBLE
 * ufunc_mathieu_sem_ptr[2*0] = <void*>_func_sem_wrap
 * ufunc_mathieu_sem_ptr[2*0+1] = <void*>(<char*>"mathieu_sem")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_ptr[2*1] = <void*>_func_sem_wrap
 * ufunc_mathieu_sem_ptr[2*1+1] = <void*>(<char*>"mathieu_sem")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_ptr[1]) = ((void *)((char *)((char *)"mathieu_sem")));

  /* "scipy/special/_ufuncs.pyx":10603
 * ufunc_mathieu_sem_ptr[2*0] = <void*>_func_sem_wrap
 * ufunc_mathieu_sem_ptr[2*0+1] = <void*>(<char*>"mathieu_sem")
 * ufunc_mathieu_sem_ptr[2*1] = <void*>_func_sem_wrap             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_ptr[2*1+1] = <void*>(<char*>"mathieu_sem")
 * ufunc_mathieu_sem_data[0] = &ufunc_mathieu_sem_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_ptr[2]) = ((void *)sem_wrap);

  /* "scipy/special/_ufuncs.pyx":10604
 * ufunc_mathieu_sem_ptr[2*0+1] = <void*>(<char*>"mathieu_sem")
 * ufunc_mathieu_sem_ptr[2*1] = <void*>_func_sem_wrap
 * ufunc_mathieu_sem_ptr[2*1+1] = <void*>(<char*>"mathieu_sem")             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_data[0] = &ufunc_mathieu_sem_ptr[2*0]
 * ufunc_mathieu_sem_data[1] = &ufunc_mathieu_sem_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_ptr[3]) = ((void *)((char *)((char *)"mathieu_sem")));

  /* "scipy/special/_ufuncs.pyx":10605
 * ufunc_mathieu_sem_ptr[2*1] = <void*>_func_sem_wrap
 * ufunc_mathieu_sem_ptr[2*1+1] = <void*>(<char*>"mathieu_sem")
 * ufunc_mathieu_sem_data[0] = &ufunc_mathieu_sem_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_mathieu_sem_data[1] = &ufunc_mathieu_sem_ptr[2*1]
 * mathieu_sem = np.PyUFunc_FromFuncAndData(ufunc_mathieu_sem_loops, ufunc_mathieu_sem_data, ufunc_mathieu_sem_types, 2, 3, 2, 0, "mathieu_sem", ufunc_mathieu_sem_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":10606
 * ufunc_mathieu_sem_ptr[2*1+1] = <void*>(<char*>"mathieu_sem")
 * ufunc_mathieu_sem_data[0] = &ufunc_mathieu_sem_ptr[2*0]
 * ufunc_mathieu_sem_data[1] = &ufunc_mathieu_sem_ptr[2*1]             # <<<<<<<<<<<<<<
 * mathieu_sem = np.PyUFunc_FromFuncAndData(ufunc_mathieu_sem_loops, ufunc_mathieu_sem_data, ufunc_mathieu_sem_types, 2, 3, 2, 0, "mathieu_sem", ufunc_mathieu_sem_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":10607
 * ufunc_mathieu_sem_data[0] = &ufunc_mathieu_sem_ptr[2*0]
 * ufunc_mathieu_sem_data[1] = &ufunc_mathieu_sem_ptr[2*1]
 * mathieu_sem = np.PyUFunc_FromFuncAndData(ufunc_mathieu_sem_loops, ufunc_mathieu_sem_data, ufunc_mathieu_sem_types, 2, 3, 2, 0, "mathieu_sem", ufunc_mathieu_sem_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_modfresnelm_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_types, 2, 3, 2, 0, ((char *)"mathieu_sem"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_mathieu_sem_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10607, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_mathieu_sem, __pyx_t_2) < 0) __PYX_ERR(1, 10607, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":10613
 * cdef void *ufunc_modfresnelm_data[2]
 * cdef char ufunc_modfresnelm_types[6]
 * cdef char *ufunc_modfresnelm_doc = (             # <<<<<<<<<<<<<<
 *     "modfresnelm(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_doc = ((char *)"modfresnelm(x)\n\nModified Fresnel negative integrals\n\nReturns\n-------\nfm\n    Integral ``F_-(x)``: ``integral(exp(-1j*t*t), t=x..inf)``\nkm\n    Integral ``K_-(x)``: ``1/sqrt(pi)*exp(1j*(x*x+pi/4))*fp``");

  /* "scipy/special/_ufuncs.pyx":10624
 *     "km\n"
 *     "    Integral ``K_-(x)``: ``1/sqrt(pi)*exp(1j*(x*x+pi/4))*fp``")
 * ufunc_modfresnelm_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_f_FF             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_d_DD
 * ufunc_modfresnelm_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DD_As_f_FF);

  /* "scipy/special/_ufuncs.pyx":10625
 *     "    Integral ``K_-(x)``: ``1/sqrt(pi)*exp(1j*(x*x+pi/4))*fp``")
 * ufunc_modfresnelm_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_f_FF
 * ufunc_modfresnelm_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_d_DD             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_types[0] = <char>NPY_FLOAT
 * ufunc_modfresnelm_types[1] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DD_As_d_DD);

  /* "scipy/special/_ufuncs.pyx":10626
 * ufunc_modfresnelm_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_f_FF
 * ufunc_modfresnelm_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_d_DD
 * ufunc_modfresnelm_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_types[1] = <char>NPY_CFLOAT
 * ufunc_modfresnelm_types[2] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10627
 * ufunc_modfresnelm_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_d_DD
 * ufunc_modfresnelm_types[0] = <char>NPY_FLOAT
 * ufunc_modfresnelm_types[1] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_types[2] = <char>NPY_CFLOAT
 * ufunc_modfresnelm_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_types[1]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":10628
 * ufunc_modfresnelm_types[0] = <char>NPY_FLOAT
 * ufunc_modfresnelm_types[1] = <char>NPY_CFLOAT
 * ufunc_modfresnelm_types[2] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_types[3] = <char>NPY_DOUBLE
 * ufunc_modfresnelm_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_types[2]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":10629
 * ufunc_modfresnelm_types[1] = <char>NPY_CFLOAT
 * ufunc_modfresnelm_types[2] = <char>NPY_CFLOAT
 * ufunc_modfresnelm_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_types[4] = <char>NPY_CDOUBLE
 * ufunc_modfresnelm_types[5] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10630
 * ufunc_modfresnelm_types[2] = <char>NPY_CFLOAT
 * ufunc_modfresnelm_types[3] = <char>NPY_DOUBLE
 * ufunc_modfresnelm_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_types[5] = <char>NPY_CDOUBLE
 * ufunc_modfresnelm_ptr[2*0] = <void*>_func_modified_fresnel_minus_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":10631
 * ufunc_modfresnelm_types[3] = <char>NPY_DOUBLE
 * ufunc_modfresnelm_types[4] = <char>NPY_CDOUBLE
 * ufunc_modfresnelm_types[5] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_ptr[2*0] = <void*>_func_modified_fresnel_minus_wrap
 * ufunc_modfresnelm_ptr[2*0+1] = <void*>(<char*>"modfresnelm")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_types[5]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":10632
 * ufunc_modfresnelm_types[4] = <char>NPY_CDOUBLE
 * ufunc_modfresnelm_types[5] = <char>NPY_CDOUBLE
 * ufunc_modfresnelm_ptr[2*0] = <void*>_func_modified_fresnel_minus_wrap             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_ptr[2*0+1] = <void*>(<char*>"modfresnelm")
 * ufunc_modfresnelm_ptr[2*1] = <void*>_func_modified_fresnel_minus_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_ptr[0]) = ((void *)modified_fresnel_minus_wrap);

  /* "scipy/special/_ufuncs.pyx":10633
 * ufunc_modfresnelm_types[5] = <char>NPY_CDOUBLE
 * ufunc_modfresnelm_ptr[2*0] = <void*>_func_modified_fresnel_minus_wrap
 * ufunc_modfresnelm_ptr[2*0+1] = <void*>(<char*>"modfresnelm")             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_ptr[2*1] = <void*>_func_modified_fresnel_minus_wrap
 * ufunc_modfresnelm_ptr[2*1+1] = <void*>(<char*>"modfresnelm")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_ptr[1]) = ((void *)((char *)((char *)"modfresnelm")));

  /* "scipy/special/_ufuncs.pyx":10634
 * ufunc_modfresnelm_ptr[2*0] = <void*>_func_modified_fresnel_minus_wrap
 * ufunc_modfresnelm_ptr[2*0+1] = <void*>(<char*>"modfresnelm")
 * ufunc_modfresnelm_ptr[2*1] = <void*>_func_modified_fresnel_minus_wrap             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_ptr[2*1+1] = <void*>(<char*>"modfresnelm")
 * ufunc_modfresnelm_data[0] = &ufunc_modfresnelm_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_ptr[2]) = ((void *)modified_fresnel_minus_wrap);

  /* "scipy/special/_ufuncs.pyx":10635
 * ufunc_modfresnelm_ptr[2*0+1] = <void*>(<char*>"modfresnelm")
 * ufunc_modfresnelm_ptr[2*1] = <void*>_func_modified_fresnel_minus_wrap
 * ufunc_modfresnelm_ptr[2*1+1] = <void*>(<char*>"modfresnelm")             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_data[0] = &ufunc_modfresnelm_ptr[2*0]
 * ufunc_modfresnelm_data[1] = &ufunc_modfresnelm_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_ptr[3]) = ((void *)((char *)((char *)"modfresnelm")));

  /* "scipy/special/_ufuncs.pyx":10636
 * ufunc_modfresnelm_ptr[2*1] = <void*>_func_modified_fresnel_minus_wrap
 * ufunc_modfresnelm_ptr[2*1+1] = <void*>(<char*>"modfresnelm")
 * ufunc_modfresnelm_data[0] = &ufunc_modfresnelm_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_modfresnelm_data[1] = &ufunc_modfresnelm_ptr[2*1]
 * modfresnelm = np.PyUFunc_FromFuncAndData(ufunc_modfresnelm_loops, ufunc_modfresnelm_data, ufunc_modfresnelm_types, 2, 1, 2, 0, "modfresnelm", ufunc_modfresnelm_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":10637
 * ufunc_modfresnelm_ptr[2*1+1] = <void*>(<char*>"modfresnelm")
 * ufunc_modfresnelm_data[0] = &ufunc_modfresnelm_ptr[2*0]
 * ufunc_modfresnelm_data[1] = &ufunc_modfresnelm_ptr[2*1]             # <<<<<<<<<<<<<<
 * modfresnelm = np.PyUFunc_FromFuncAndData(ufunc_modfresnelm_loops, ufunc_modfresnelm_data, ufunc_modfresnelm_types, 2, 1, 2, 0, "modfresnelm", ufunc_modfresnelm_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":10638
 * ufunc_modfresnelm_data[0] = &ufunc_modfresnelm_ptr[2*0]
 * ufunc_modfresnelm_data[1] = &ufunc_modfresnelm_ptr[2*1]
 * modfresnelm = np.PyUFunc_FromFuncAndData(ufunc_modfresnelm_loops, ufunc_modfresnelm_data, ufunc_modfresnelm_types, 2, 1, 2, 0, "modfresnelm", ufunc_modfresnelm_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_modfresnelp_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_types, 2, 1, 2, 0, ((char *)"modfresnelm"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelm_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10638, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_modfresnelm, __pyx_t_2) < 0) __PYX_ERR(1, 10638, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":10644
 * cdef void *ufunc_modfresnelp_data[2]
 * cdef char ufunc_modfresnelp_types[6]
 * cdef char *ufunc_modfresnelp_doc = (             # <<<<<<<<<<<<<<
 *     "modfresnelp(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_doc = ((char *)"modfresnelp(x)\n\nModified Fresnel positive integrals\n\nReturns\n-------\nfp\n    Integral ``F_+(x)``: ``integral(exp(1j*t*t), t=x..inf)``\nkp\n    Integral ``K_+(x)``: ``1/sqrt(pi)*exp(-1j*(x*x+pi/4))*fp``");

  /* "scipy/special/_ufuncs.pyx":10655
 *     "kp\n"
 *     "    Integral ``K_+(x)``: ``1/sqrt(pi)*exp(-1j*(x*x+pi/4))*fp``")
 * ufunc_modfresnelp_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_f_FF             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_d_DD
 * ufunc_modfresnelp_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DD_As_f_FF);

  /* "scipy/special/_ufuncs.pyx":10656
 *     "    Integral ``K_+(x)``: ``1/sqrt(pi)*exp(-1j*(x*x+pi/4))*fp``")
 * ufunc_modfresnelp_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_f_FF
 * ufunc_modfresnelp_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_d_DD             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_types[0] = <char>NPY_FLOAT
 * ufunc_modfresnelp_types[1] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_DD_As_d_DD);

  /* "scipy/special/_ufuncs.pyx":10657
 * ufunc_modfresnelp_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_f_FF
 * ufunc_modfresnelp_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_d_DD
 * ufunc_modfresnelp_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_types[1] = <char>NPY_CFLOAT
 * ufunc_modfresnelp_types[2] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10658
 * ufunc_modfresnelp_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_DD_As_d_DD
 * ufunc_modfresnelp_types[0] = <char>NPY_FLOAT
 * ufunc_modfresnelp_types[1] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_types[2] = <char>NPY_CFLOAT
 * ufunc_modfresnelp_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_types[1]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":10659
 * ufunc_modfresnelp_types[0] = <char>NPY_FLOAT
 * ufunc_modfresnelp_types[1] = <char>NPY_CFLOAT
 * ufunc_modfresnelp_types[2] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_types[3] = <char>NPY_DOUBLE
 * ufunc_modfresnelp_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_types[2]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":10660
 * ufunc_modfresnelp_types[1] = <char>NPY_CFLOAT
 * ufunc_modfresnelp_types[2] = <char>NPY_CFLOAT
 * ufunc_modfresnelp_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_types[4] = <char>NPY_CDOUBLE
 * ufunc_modfresnelp_types[5] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10661
 * ufunc_modfresnelp_types[2] = <char>NPY_CFLOAT
 * ufunc_modfresnelp_types[3] = <char>NPY_DOUBLE
 * ufunc_modfresnelp_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_types[5] = <char>NPY_CDOUBLE
 * ufunc_modfresnelp_ptr[2*0] = <void*>_func_modified_fresnel_plus_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":10662
 * ufunc_modfresnelp_types[3] = <char>NPY_DOUBLE
 * ufunc_modfresnelp_types[4] = <char>NPY_CDOUBLE
 * ufunc_modfresnelp_types[5] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_ptr[2*0] = <void*>_func_modified_fresnel_plus_wrap
 * ufunc_modfresnelp_ptr[2*0+1] = <void*>(<char*>"modfresnelp")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_types[5]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":10663
 * ufunc_modfresnelp_types[4] = <char>NPY_CDOUBLE
 * ufunc_modfresnelp_types[5] = <char>NPY_CDOUBLE
 * ufunc_modfresnelp_ptr[2*0] = <void*>_func_modified_fresnel_plus_wrap             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_ptr[2*0+1] = <void*>(<char*>"modfresnelp")
 * ufunc_modfresnelp_ptr[2*1] = <void*>_func_modified_fresnel_plus_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_ptr[0]) = ((void *)modified_fresnel_plus_wrap);

  /* "scipy/special/_ufuncs.pyx":10664
 * ufunc_modfresnelp_types[5] = <char>NPY_CDOUBLE
 * ufunc_modfresnelp_ptr[2*0] = <void*>_func_modified_fresnel_plus_wrap
 * ufunc_modfresnelp_ptr[2*0+1] = <void*>(<char*>"modfresnelp")             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_ptr[2*1] = <void*>_func_modified_fresnel_plus_wrap
 * ufunc_modfresnelp_ptr[2*1+1] = <void*>(<char*>"modfresnelp")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_ptr[1]) = ((void *)((char *)((char *)"modfresnelp")));

  /* "scipy/special/_ufuncs.pyx":10665
 * ufunc_modfresnelp_ptr[2*0] = <void*>_func_modified_fresnel_plus_wrap
 * ufunc_modfresnelp_ptr[2*0+1] = <void*>(<char*>"modfresnelp")
 * ufunc_modfresnelp_ptr[2*1] = <void*>_func_modified_fresnel_plus_wrap             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_ptr[2*1+1] = <void*>(<char*>"modfresnelp")
 * ufunc_modfresnelp_data[0] = &ufunc_modfresnelp_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_ptr[2]) = ((void *)modified_fresnel_plus_wrap);

  /* "scipy/special/_ufuncs.pyx":10666
 * ufunc_modfresnelp_ptr[2*0+1] = <void*>(<char*>"modfresnelp")
 * ufunc_modfresnelp_ptr[2*1] = <void*>_func_modified_fresnel_plus_wrap
 * ufunc_modfresnelp_ptr[2*1+1] = <void*>(<char*>"modfresnelp")             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_data[0] = &ufunc_modfresnelp_ptr[2*0]
 * ufunc_modfresnelp_data[1] = &ufunc_modfresnelp_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_ptr[3]) = ((void *)((char *)((char *)"modfresnelp")));

  /* "scipy/special/_ufuncs.pyx":10667
 * ufunc_modfresnelp_ptr[2*1] = <void*>_func_modified_fresnel_plus_wrap
 * ufunc_modfresnelp_ptr[2*1+1] = <void*>(<char*>"modfresnelp")
 * ufunc_modfresnelp_data[0] = &ufunc_modfresnelp_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_modfresnelp_data[1] = &ufunc_modfresnelp_ptr[2*1]
 * modfresnelp = np.PyUFunc_FromFuncAndData(ufunc_modfresnelp_loops, ufunc_modfresnelp_data, ufunc_modfresnelp_types, 2, 1, 2, 0, "modfresnelp", ufunc_modfresnelp_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":10668
 * ufunc_modfresnelp_ptr[2*1+1] = <void*>(<char*>"modfresnelp")
 * ufunc_modfresnelp_data[0] = &ufunc_modfresnelp_ptr[2*0]
 * ufunc_modfresnelp_data[1] = &ufunc_modfresnelp_ptr[2*1]             # <<<<<<<<<<<<<<
 * modfresnelp = np.PyUFunc_FromFuncAndData(ufunc_modfresnelp_loops, ufunc_modfresnelp_data, ufunc_modfresnelp_types, 2, 1, 2, 0, "modfresnelp", ufunc_modfresnelp_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":10669
 * ufunc_modfresnelp_data[0] = &ufunc_modfresnelp_ptr[2*0]
 * ufunc_modfresnelp_data[1] = &ufunc_modfresnelp_ptr[2*1]
 * modfresnelp = np.PyUFunc_FromFuncAndData(ufunc_modfresnelp_loops, ufunc_modfresnelp_data, ufunc_modfresnelp_types, 2, 1, 2, 0, "modfresnelp", ufunc_modfresnelp_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_modstruve_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_types, 2, 1, 2, 0, ((char *)"modfresnelp"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_modfresnelp_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10669, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_modfresnelp, __pyx_t_2) < 0) __PYX_ERR(1, 10669, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":10675
 * cdef void *ufunc_modstruve_data[2]
 * cdef char ufunc_modstruve_types[6]
 * cdef char *ufunc_modstruve_doc = (             # <<<<<<<<<<<<<<
 *     "modstruve(v, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_doc = ((char *)"modstruve(v, x)\n\nModified Struve function.\n\nReturn the value of the modified Struve function of order `v` at `x`.  The\nmodified Struve function is defined as,\n\n.. math::\n    L_v(x) = -\\imath \\exp(-\\pi\\imath v/2) H_v(x),\n\nwhere :math:`H_v` is the Struve function.\n\nParameters\n----------\nv : array_like\n    Order of the modified Struve function (float).\nx : array_like\n    Argument of the Struve function (float; must be positive unless `v` is\n    an integer).\n\nReturns\n-------\nL : ndarray\n    Value of the modified Struve function of order `v` at `x`.\n\nNotes\n-----\nThree methods discussed in [1]_ are used to evaluate the function:\n\n- power series\n- expansion in Bessel functions (if :math:`|z| < |v| + 20`)\n- asymptotic large-z expansion (if :math:`z \\geq 0.7v + 12`)\n\nRounding errors are estimated based on the largest terms in the sums, and\nthe result associated with the smallest error is returned.\n\nSee also\n--------\nstruve\n\nReferences\n----------\n.. [1] NIST Digital Library of Mathematical Functions\n       https://dlmf.nist.gov/11");

  /* "scipy/special/_ufuncs.pyx":10720
 *     ".. [1] NIST Digital Library of Mathematical Functions\n"
 *     "       https://dlmf.nist.gov/11")
 * ufunc_modstruve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_modstruve_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_modstruve_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":10721
 *     "       https://dlmf.nist.gov/11")
 * ufunc_modstruve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_modstruve_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_modstruve_types[0] = <char>NPY_FLOAT
 * ufunc_modstruve_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":10722
 * ufunc_modstruve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_modstruve_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_modstruve_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_modstruve_types[1] = <char>NPY_FLOAT
 * ufunc_modstruve_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10723
 * ufunc_modstruve_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_modstruve_types[0] = <char>NPY_FLOAT
 * ufunc_modstruve_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_modstruve_types[2] = <char>NPY_FLOAT
 * ufunc_modstruve_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10724
 * ufunc_modstruve_types[0] = <char>NPY_FLOAT
 * ufunc_modstruve_types[1] = <char>NPY_FLOAT
 * ufunc_modstruve_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_modstruve_types[3] = <char>NPY_DOUBLE
 * ufunc_modstruve_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10725
 * ufunc_modstruve_types[1] = <char>NPY_FLOAT
 * ufunc_modstruve_types[2] = <char>NPY_FLOAT
 * ufunc_modstruve_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_modstruve_types[4] = <char>NPY_DOUBLE
 * ufunc_modstruve_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10726
 * ufunc_modstruve_types[2] = <char>NPY_FLOAT
 * ufunc_modstruve_types[3] = <char>NPY_DOUBLE
 * ufunc_modstruve_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_modstruve_types[5] = <char>NPY_DOUBLE
 * ufunc_modstruve_ptr[2*0] = <void*>_func_struve_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10727
 * ufunc_modstruve_types[3] = <char>NPY_DOUBLE
 * ufunc_modstruve_types[4] = <char>NPY_DOUBLE
 * ufunc_modstruve_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_modstruve_ptr[2*0] = <void*>_func_struve_l
 * ufunc_modstruve_ptr[2*0+1] = <void*>(<char*>"modstruve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10728
 * ufunc_modstruve_types[4] = <char>NPY_DOUBLE
 * ufunc_modstruve_types[5] = <char>NPY_DOUBLE
 * ufunc_modstruve_ptr[2*0] = <void*>_func_struve_l             # <<<<<<<<<<<<<<
 * ufunc_modstruve_ptr[2*0+1] = <void*>(<char*>"modstruve")
 * ufunc_modstruve_ptr[2*1] = <void*>_func_struve_l
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_ptr[0]) = ((void *)struve_l);

  /* "scipy/special/_ufuncs.pyx":10729
 * ufunc_modstruve_types[5] = <char>NPY_DOUBLE
 * ufunc_modstruve_ptr[2*0] = <void*>_func_struve_l
 * ufunc_modstruve_ptr[2*0+1] = <void*>(<char*>"modstruve")             # <<<<<<<<<<<<<<
 * ufunc_modstruve_ptr[2*1] = <void*>_func_struve_l
 * ufunc_modstruve_ptr[2*1+1] = <void*>(<char*>"modstruve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_ptr[1]) = ((void *)((char *)((char *)"modstruve")));

  /* "scipy/special/_ufuncs.pyx":10730
 * ufunc_modstruve_ptr[2*0] = <void*>_func_struve_l
 * ufunc_modstruve_ptr[2*0+1] = <void*>(<char*>"modstruve")
 * ufunc_modstruve_ptr[2*1] = <void*>_func_struve_l             # <<<<<<<<<<<<<<
 * ufunc_modstruve_ptr[2*1+1] = <void*>(<char*>"modstruve")
 * ufunc_modstruve_data[0] = &ufunc_modstruve_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_ptr[2]) = ((void *)struve_l);

  /* "scipy/special/_ufuncs.pyx":10731
 * ufunc_modstruve_ptr[2*0+1] = <void*>(<char*>"modstruve")
 * ufunc_modstruve_ptr[2*1] = <void*>_func_struve_l
 * ufunc_modstruve_ptr[2*1+1] = <void*>(<char*>"modstruve")             # <<<<<<<<<<<<<<
 * ufunc_modstruve_data[0] = &ufunc_modstruve_ptr[2*0]
 * ufunc_modstruve_data[1] = &ufunc_modstruve_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_ptr[3]) = ((void *)((char *)((char *)"modstruve")));

  /* "scipy/special/_ufuncs.pyx":10732
 * ufunc_modstruve_ptr[2*1] = <void*>_func_struve_l
 * ufunc_modstruve_ptr[2*1+1] = <void*>(<char*>"modstruve")
 * ufunc_modstruve_data[0] = &ufunc_modstruve_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_modstruve_data[1] = &ufunc_modstruve_ptr[2*1]
 * modstruve = np.PyUFunc_FromFuncAndData(ufunc_modstruve_loops, ufunc_modstruve_data, ufunc_modstruve_types, 2, 2, 1, 0, "modstruve", ufunc_modstruve_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":10733
 * ufunc_modstruve_ptr[2*1+1] = <void*>(<char*>"modstruve")
 * ufunc_modstruve_data[0] = &ufunc_modstruve_ptr[2*0]
 * ufunc_modstruve_data[1] = &ufunc_modstruve_ptr[2*1]             # <<<<<<<<<<<<<<
 * modstruve = np.PyUFunc_FromFuncAndData(ufunc_modstruve_loops, ufunc_modstruve_data, ufunc_modstruve_types, 2, 2, 1, 0, "modstruve", ufunc_modstruve_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":10734
 * ufunc_modstruve_data[0] = &ufunc_modstruve_ptr[2*0]
 * ufunc_modstruve_data[1] = &ufunc_modstruve_ptr[2*1]
 * modstruve = np.PyUFunc_FromFuncAndData(ufunc_modstruve_loops, ufunc_modstruve_data, ufunc_modstruve_types, 2, 2, 1, 0, "modstruve", ufunc_modstruve_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nbdtr_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_types, 2, 2, 1, 0, ((char *)"modstruve"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_modstruve_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10734, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_modstruve, __pyx_t_2) < 0) __PYX_ERR(1, 10734, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":10740
 * cdef void *ufunc_nbdtr_data[3]
 * cdef char ufunc_nbdtr_types[12]
 * cdef char *ufunc_nbdtr_doc = (             # <<<<<<<<<<<<<<
 *     "nbdtr(k, n, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_doc = ((char *)"nbdtr(k, n, p)\n\nNegative binomial cumulative distribution function.\n\nReturns the sum of the terms 0 through `k` of the negative binomial\ndistribution probability mass function,\n\n.. math::\n\n    F = \\sum_{j=0}^k {{n + j - 1}\\choose{j}} p^n (1 - p)^j.\n\nIn a sequence of Bernoulli trials with individual success probabilities\n`p`, this is the probability that `k` or fewer failures precede the nth\nsuccess.\n\nParameters\n----------\nk : array_like\n    The maximum number of allowed failures (nonnegative int).\nn : array_like\n    The target number of successes (positive int).\np : array_like\n    Probability of success in a single event (float).\n\nReturns\n-------\nF : ndarray\n    The probability of `k` or fewer failures before `n` successes in a\n    sequence of events with individual success probability `p`.\n\nSee also\n--------\nnbdtrc\n\nNotes\n-----\nIf floating point values are passed for `k` or `n`, they will be truncated\nto integers.\n\nThe terms are not summed directly; instead the regularized incomplete beta\nfunction is employed, according to the formula,\n\n.. math::\n    \\mathrm{nbdtr}(k, n, p) = I_{p}(n, k + 1).\n\nWrapper for the Cephes [1]_ routine `nbdtr`.\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":10792
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_nbdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_iid__As_lld_d);

  /* "scipy/special/_ufuncs.pyx":10793
 *     "       http://www.netlib.org/cephes/")
 * ufunc_nbdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_nbdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtr_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":10794
 * ufunc_nbdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_nbdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[0] = <char>NPY_LONG
 * ufunc_nbdtr_types[1] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":10795
 * ufunc_nbdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtr_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[1] = <char>NPY_LONG
 * ufunc_nbdtr_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":10796
 * ufunc_nbdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtr_types[0] = <char>NPY_LONG
 * ufunc_nbdtr_types[1] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[2] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[1]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":10797
 * ufunc_nbdtr_types[0] = <char>NPY_LONG
 * ufunc_nbdtr_types[1] = <char>NPY_LONG
 * ufunc_nbdtr_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10798
 * ufunc_nbdtr_types[1] = <char>NPY_LONG
 * ufunc_nbdtr_types[2] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[4] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10799
 * ufunc_nbdtr_types[2] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[5] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10800
 * ufunc_nbdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[4] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[6] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10801
 * ufunc_nbdtr_types[4] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[5] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[7] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10802
 * ufunc_nbdtr_types[5] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[6] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10803
 * ufunc_nbdtr_types[6] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[7] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[9] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10804
 * ufunc_nbdtr_types[7] = <char>NPY_FLOAT
 * ufunc_nbdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[10] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10805
 * ufunc_nbdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[9] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_types[11] = <char>NPY_DOUBLE
 * ufunc_nbdtr_ptr[2*0] = <void*>_func_nbdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10806
 * ufunc_nbdtr_types[9] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[10] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_ptr[2*0] = <void*>_func_nbdtr
 * ufunc_nbdtr_ptr[2*0+1] = <void*>(<char*>"nbdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10807
 * ufunc_nbdtr_types[10] = <char>NPY_DOUBLE
 * ufunc_nbdtr_types[11] = <char>NPY_DOUBLE
 * ufunc_nbdtr_ptr[2*0] = <void*>_func_nbdtr             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_ptr[2*0+1] = <void*>(<char*>"nbdtr")
 * ufunc_nbdtr_ptr[2*1] = <void*>_func_nbdtr_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_ptr[0]) = ((void *)nbdtr);

  /* "scipy/special/_ufuncs.pyx":10808
 * ufunc_nbdtr_types[11] = <char>NPY_DOUBLE
 * ufunc_nbdtr_ptr[2*0] = <void*>_func_nbdtr
 * ufunc_nbdtr_ptr[2*0+1] = <void*>(<char*>"nbdtr")             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_ptr[2*1] = <void*>_func_nbdtr_unsafe
 * ufunc_nbdtr_ptr[2*1+1] = <void*>(<char*>"nbdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_ptr[1]) = ((void *)((char *)((char *)"nbdtr")));

  /* "scipy/special/_ufuncs.pyx":10809
 * ufunc_nbdtr_ptr[2*0] = <void*>_func_nbdtr
 * ufunc_nbdtr_ptr[2*0+1] = <void*>(<char*>"nbdtr")
 * ufunc_nbdtr_ptr[2*1] = <void*>_func_nbdtr_unsafe             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_ptr[2*1+1] = <void*>(<char*>"nbdtr")
 * ufunc_nbdtr_ptr[2*2] = <void*>_func_nbdtr_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_nbdtr_unsafe);

  /* "scipy/special/_ufuncs.pyx":10810
 * ufunc_nbdtr_ptr[2*0+1] = <void*>(<char*>"nbdtr")
 * ufunc_nbdtr_ptr[2*1] = <void*>_func_nbdtr_unsafe
 * ufunc_nbdtr_ptr[2*1+1] = <void*>(<char*>"nbdtr")             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_ptr[2*2] = <void*>_func_nbdtr_unsafe
 * ufunc_nbdtr_ptr[2*2+1] = <void*>(<char*>"nbdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_ptr[3]) = ((void *)((char *)((char *)"nbdtr")));

  /* "scipy/special/_ufuncs.pyx":10811
 * ufunc_nbdtr_ptr[2*1] = <void*>_func_nbdtr_unsafe
 * ufunc_nbdtr_ptr[2*1+1] = <void*>(<char*>"nbdtr")
 * ufunc_nbdtr_ptr[2*2] = <void*>_func_nbdtr_unsafe             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_ptr[2*2+1] = <void*>(<char*>"nbdtr")
 * ufunc_nbdtr_data[0] = &ufunc_nbdtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_nbdtr_unsafe);

  /* "scipy/special/_ufuncs.pyx":10812
 * ufunc_nbdtr_ptr[2*1+1] = <void*>(<char*>"nbdtr")
 * ufunc_nbdtr_ptr[2*2] = <void*>_func_nbdtr_unsafe
 * ufunc_nbdtr_ptr[2*2+1] = <void*>(<char*>"nbdtr")             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_data[0] = &ufunc_nbdtr_ptr[2*0]
 * ufunc_nbdtr_data[1] = &ufunc_nbdtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_ptr[5]) = ((void *)((char *)((char *)"nbdtr")));

  /* "scipy/special/_ufuncs.pyx":10813
 * ufunc_nbdtr_ptr[2*2] = <void*>_func_nbdtr_unsafe
 * ufunc_nbdtr_ptr[2*2+1] = <void*>(<char*>"nbdtr")
 * ufunc_nbdtr_data[0] = &ufunc_nbdtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_data[1] = &ufunc_nbdtr_ptr[2*1]
 * ufunc_nbdtr_data[2] = &ufunc_nbdtr_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":10814
 * ufunc_nbdtr_ptr[2*2+1] = <void*>(<char*>"nbdtr")
 * ufunc_nbdtr_data[0] = &ufunc_nbdtr_ptr[2*0]
 * ufunc_nbdtr_data[1] = &ufunc_nbdtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_nbdtr_data[2] = &ufunc_nbdtr_ptr[2*2]
 * nbdtr = np.PyUFunc_FromFuncAndData(ufunc_nbdtr_loops, ufunc_nbdtr_data, ufunc_nbdtr_types, 3, 3, 1, 0, "nbdtr", ufunc_nbdtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":10815
 * ufunc_nbdtr_data[0] = &ufunc_nbdtr_ptr[2*0]
 * ufunc_nbdtr_data[1] = &ufunc_nbdtr_ptr[2*1]
 * ufunc_nbdtr_data[2] = &ufunc_nbdtr_ptr[2*2]             # <<<<<<<<<<<<<<
 * nbdtr = np.PyUFunc_FromFuncAndData(ufunc_nbdtr_loops, ufunc_nbdtr_data, ufunc_nbdtr_types, 3, 3, 1, 0, "nbdtr", ufunc_nbdtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":10816
 * ufunc_nbdtr_data[1] = &ufunc_nbdtr_ptr[2*1]
 * ufunc_nbdtr_data[2] = &ufunc_nbdtr_ptr[2*2]
 * nbdtr = np.PyUFunc_FromFuncAndData(ufunc_nbdtr_loops, ufunc_nbdtr_data, ufunc_nbdtr_types, 3, 3, 1, 0, "nbdtr", ufunc_nbdtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nbdtrc_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_types, 3, 3, 1, 0, ((char *)"nbdtr"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtr_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10816, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nbdtr, __pyx_t_2) < 0) __PYX_ERR(1, 10816, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":10822
 * cdef void *ufunc_nbdtrc_data[3]
 * cdef char ufunc_nbdtrc_types[12]
 * cdef char *ufunc_nbdtrc_doc = (             # <<<<<<<<<<<<<<
 *     "nbdtrc(k, n, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_doc = ((char *)"nbdtrc(k, n, p)\n\nNegative binomial survival function.\n\nReturns the sum of the terms `k + 1` to infinity of the negative binomial\ndistribution probability mass function,\n\n.. math::\n\n    F = \\sum_{j=k + 1}^\\infty {{n + j - 1}\\choose{j}} p^n (1 - p)^j.\n\nIn a sequence of Bernoulli trials with individual success probabilities\n`p`, this is the probability that more than `k` failures precede the nth\nsuccess.\n\nParameters\n----------\nk : array_like\n    The maximum number of allowed failures (nonnegative int).\nn : array_like\n    The target number of successes (positive int).\np : array_like\n    Probability of success in a single event (float).\n\nReturns\n-------\nF : ndarray\n    The probability of `k + 1` or more failures before `n` successes in a\n    sequence of events with individual success probability `p`.\n\nNotes\n-----\nIf floating point values are passed for `k` or `n`, they will be truncated\nto integers.\n\nThe terms are not summed directly; instead the regularized incomplete beta\nfunction is employed, according to the formula,\n\n.. math::\n    \\mathrm{nbdtrc}(k, n, p) = I_{1 - p}(k + 1, n).\n\nWrapper for the Cephes [1]_ routine `nbdtrc`.\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":10870
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_nbdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_iid__As_lld_d);

  /* "scipy/special/_ufuncs.pyx":10871
 *     "       http://www.netlib.org/cephes/")
 * ufunc_nbdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_nbdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtrc_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":10872
 * ufunc_nbdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_nbdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[0] = <char>NPY_LONG
 * ufunc_nbdtrc_types[1] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":10873
 * ufunc_nbdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtrc_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[1] = <char>NPY_LONG
 * ufunc_nbdtrc_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":10874
 * ufunc_nbdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtrc_types[0] = <char>NPY_LONG
 * ufunc_nbdtrc_types[1] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[2] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[1]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":10875
 * ufunc_nbdtrc_types[0] = <char>NPY_LONG
 * ufunc_nbdtrc_types[1] = <char>NPY_LONG
 * ufunc_nbdtrc_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[3] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10876
 * ufunc_nbdtrc_types[1] = <char>NPY_LONG
 * ufunc_nbdtrc_types[2] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[4] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10877
 * ufunc_nbdtrc_types[2] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[3] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[5] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10878
 * ufunc_nbdtrc_types[3] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[4] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[6] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10879
 * ufunc_nbdtrc_types[4] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[5] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[7] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10880
 * ufunc_nbdtrc_types[5] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[6] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[8] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10881
 * ufunc_nbdtrc_types[6] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[7] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[9] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10882
 * ufunc_nbdtrc_types[7] = <char>NPY_FLOAT
 * ufunc_nbdtrc_types[8] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[10] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10883
 * ufunc_nbdtrc_types[8] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[9] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_types[11] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_ptr[2*0] = <void*>_func_nbdtrc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10884
 * ufunc_nbdtrc_types[9] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[10] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_ptr[2*0] = <void*>_func_nbdtrc
 * ufunc_nbdtrc_ptr[2*0+1] = <void*>(<char*>"nbdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10885
 * ufunc_nbdtrc_types[10] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_types[11] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_ptr[2*0] = <void*>_func_nbdtrc             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_ptr[2*0+1] = <void*>(<char*>"nbdtrc")
 * ufunc_nbdtrc_ptr[2*1] = <void*>_func_nbdtrc_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_ptr[0]) = ((void *)nbdtrc);

  /* "scipy/special/_ufuncs.pyx":10886
 * ufunc_nbdtrc_types[11] = <char>NPY_DOUBLE
 * ufunc_nbdtrc_ptr[2*0] = <void*>_func_nbdtrc
 * ufunc_nbdtrc_ptr[2*0+1] = <void*>(<char*>"nbdtrc")             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_ptr[2*1] = <void*>_func_nbdtrc_unsafe
 * ufunc_nbdtrc_ptr[2*1+1] = <void*>(<char*>"nbdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_ptr[1]) = ((void *)((char *)((char *)"nbdtrc")));

  /* "scipy/special/_ufuncs.pyx":10887
 * ufunc_nbdtrc_ptr[2*0] = <void*>_func_nbdtrc
 * ufunc_nbdtrc_ptr[2*0+1] = <void*>(<char*>"nbdtrc")
 * ufunc_nbdtrc_ptr[2*1] = <void*>_func_nbdtrc_unsafe             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_ptr[2*1+1] = <void*>(<char*>"nbdtrc")
 * ufunc_nbdtrc_ptr[2*2] = <void*>_func_nbdtrc_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_nbdtrc_unsafe);

  /* "scipy/special/_ufuncs.pyx":10888
 * ufunc_nbdtrc_ptr[2*0+1] = <void*>(<char*>"nbdtrc")
 * ufunc_nbdtrc_ptr[2*1] = <void*>_func_nbdtrc_unsafe
 * ufunc_nbdtrc_ptr[2*1+1] = <void*>(<char*>"nbdtrc")             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_ptr[2*2] = <void*>_func_nbdtrc_unsafe
 * ufunc_nbdtrc_ptr[2*2+1] = <void*>(<char*>"nbdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_ptr[3]) = ((void *)((char *)((char *)"nbdtrc")));

  /* "scipy/special/_ufuncs.pyx":10889
 * ufunc_nbdtrc_ptr[2*1] = <void*>_func_nbdtrc_unsafe
 * ufunc_nbdtrc_ptr[2*1+1] = <void*>(<char*>"nbdtrc")
 * ufunc_nbdtrc_ptr[2*2] = <void*>_func_nbdtrc_unsafe             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_ptr[2*2+1] = <void*>(<char*>"nbdtrc")
 * ufunc_nbdtrc_data[0] = &ufunc_nbdtrc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_nbdtrc_unsafe);

  /* "scipy/special/_ufuncs.pyx":10890
 * ufunc_nbdtrc_ptr[2*1+1] = <void*>(<char*>"nbdtrc")
 * ufunc_nbdtrc_ptr[2*2] = <void*>_func_nbdtrc_unsafe
 * ufunc_nbdtrc_ptr[2*2+1] = <void*>(<char*>"nbdtrc")             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_data[0] = &ufunc_nbdtrc_ptr[2*0]
 * ufunc_nbdtrc_data[1] = &ufunc_nbdtrc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_ptr[5]) = ((void *)((char *)((char *)"nbdtrc")));

  /* "scipy/special/_ufuncs.pyx":10891
 * ufunc_nbdtrc_ptr[2*2] = <void*>_func_nbdtrc_unsafe
 * ufunc_nbdtrc_ptr[2*2+1] = <void*>(<char*>"nbdtrc")
 * ufunc_nbdtrc_data[0] = &ufunc_nbdtrc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_data[1] = &ufunc_nbdtrc_ptr[2*1]
 * ufunc_nbdtrc_data[2] = &ufunc_nbdtrc_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":10892
 * ufunc_nbdtrc_ptr[2*2+1] = <void*>(<char*>"nbdtrc")
 * ufunc_nbdtrc_data[0] = &ufunc_nbdtrc_ptr[2*0]
 * ufunc_nbdtrc_data[1] = &ufunc_nbdtrc_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_nbdtrc_data[2] = &ufunc_nbdtrc_ptr[2*2]
 * nbdtrc = np.PyUFunc_FromFuncAndData(ufunc_nbdtrc_loops, ufunc_nbdtrc_data, ufunc_nbdtrc_types, 3, 3, 1, 0, "nbdtrc", ufunc_nbdtrc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":10893
 * ufunc_nbdtrc_data[0] = &ufunc_nbdtrc_ptr[2*0]
 * ufunc_nbdtrc_data[1] = &ufunc_nbdtrc_ptr[2*1]
 * ufunc_nbdtrc_data[2] = &ufunc_nbdtrc_ptr[2*2]             # <<<<<<<<<<<<<<
 * nbdtrc = np.PyUFunc_FromFuncAndData(ufunc_nbdtrc_loops, ufunc_nbdtrc_data, ufunc_nbdtrc_types, 3, 3, 1, 0, "nbdtrc", ufunc_nbdtrc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":10894
 * ufunc_nbdtrc_data[1] = &ufunc_nbdtrc_ptr[2*1]
 * ufunc_nbdtrc_data[2] = &ufunc_nbdtrc_ptr[2*2]
 * nbdtrc = np.PyUFunc_FromFuncAndData(ufunc_nbdtrc_loops, ufunc_nbdtrc_data, ufunc_nbdtrc_types, 3, 3, 1, 0, "nbdtrc", ufunc_nbdtrc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nbdtri_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_types, 3, 3, 1, 0, ((char *)"nbdtrc"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrc_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10894, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nbdtrc, __pyx_t_2) < 0) __PYX_ERR(1, 10894, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":10900
 * cdef void *ufunc_nbdtri_data[3]
 * cdef char ufunc_nbdtri_types[12]
 * cdef char *ufunc_nbdtri_doc = (             # <<<<<<<<<<<<<<
 *     "nbdtri(k, n, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_doc = ((char *)"nbdtri(k, n, y)\n\nInverse of `nbdtr` vs `p`.\n\nReturns the inverse with respect to the parameter `p` of\n`y = nbdtr(k, n, p)`, the negative binomial cumulative distribution\nfunction.\n\nParameters\n----------\nk : array_like\n    The maximum number of allowed failures (nonnegative int).\nn : array_like\n    The target number of successes (positive int).\ny : array_like\n    The probability of `k` or fewer failures before `n` successes (float).\n\nReturns\n-------\np : ndarray\n    Probability of success in a single event (float) such that\n    `nbdtr(k, n, p) = y`.\n\nSee also\n--------\nnbdtr : Cumulative distribution function of the negative binomial.\nnbdtrik : Inverse with respect to `k` of `nbdtr(k, n, p)`.\nnbdtrin : Inverse with respect to `n` of `nbdtr(k, n, p)`.\n\nNotes\n-----\nWrapper for the Cephes [1]_ routine `nbdtri`.\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":10938
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_nbdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_iid__As_lld_d);

  /* "scipy/special/_ufuncs.pyx":10939
 *     "       http://www.netlib.org/cephes/")
 * ufunc_nbdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_nbdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtri_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":10940
 * ufunc_nbdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_iid__As_lld_d
 * ufunc_nbdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[0] = <char>NPY_LONG
 * ufunc_nbdtri_types[1] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":10941
 * ufunc_nbdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtri_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[1] = <char>NPY_LONG
 * ufunc_nbdtri_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":10942
 * ufunc_nbdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtri_types[0] = <char>NPY_LONG
 * ufunc_nbdtri_types[1] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[2] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[1]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":10943
 * ufunc_nbdtri_types[0] = <char>NPY_LONG
 * ufunc_nbdtri_types[1] = <char>NPY_LONG
 * ufunc_nbdtri_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[3] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10944
 * ufunc_nbdtri_types[1] = <char>NPY_LONG
 * ufunc_nbdtri_types[2] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[4] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10945
 * ufunc_nbdtri_types[2] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[3] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[5] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10946
 * ufunc_nbdtri_types[3] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[4] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[6] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10947
 * ufunc_nbdtri_types[4] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[5] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[7] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10948
 * ufunc_nbdtri_types[5] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[6] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":10949
 * ufunc_nbdtri_types[6] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[7] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[9] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10950
 * ufunc_nbdtri_types[7] = <char>NPY_FLOAT
 * ufunc_nbdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[10] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10951
 * ufunc_nbdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[9] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_types[11] = <char>NPY_DOUBLE
 * ufunc_nbdtri_ptr[2*0] = <void*>_func_nbdtri
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10952
 * ufunc_nbdtri_types[9] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[10] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_ptr[2*0] = <void*>_func_nbdtri
 * ufunc_nbdtri_ptr[2*0+1] = <void*>(<char*>"nbdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":10953
 * ufunc_nbdtri_types[10] = <char>NPY_DOUBLE
 * ufunc_nbdtri_types[11] = <char>NPY_DOUBLE
 * ufunc_nbdtri_ptr[2*0] = <void*>_func_nbdtri             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_ptr[2*0+1] = <void*>(<char*>"nbdtri")
 * ufunc_nbdtri_ptr[2*1] = <void*>_func_nbdtri_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_ptr[0]) = ((void *)nbdtri);

  /* "scipy/special/_ufuncs.pyx":10954
 * ufunc_nbdtri_types[11] = <char>NPY_DOUBLE
 * ufunc_nbdtri_ptr[2*0] = <void*>_func_nbdtri
 * ufunc_nbdtri_ptr[2*0+1] = <void*>(<char*>"nbdtri")             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_ptr[2*1] = <void*>_func_nbdtri_unsafe
 * ufunc_nbdtri_ptr[2*1+1] = <void*>(<char*>"nbdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_ptr[1]) = ((void *)((char *)((char *)"nbdtri")));

  /* "scipy/special/_ufuncs.pyx":10955
 * ufunc_nbdtri_ptr[2*0] = <void*>_func_nbdtri
 * ufunc_nbdtri_ptr[2*0+1] = <void*>(<char*>"nbdtri")
 * ufunc_nbdtri_ptr[2*1] = <void*>_func_nbdtri_unsafe             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_ptr[2*1+1] = <void*>(<char*>"nbdtri")
 * ufunc_nbdtri_ptr[2*2] = <void*>_func_nbdtri_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_nbdtri_unsafe);

  /* "scipy/special/_ufuncs.pyx":10956
 * ufunc_nbdtri_ptr[2*0+1] = <void*>(<char*>"nbdtri")
 * ufunc_nbdtri_ptr[2*1] = <void*>_func_nbdtri_unsafe
 * ufunc_nbdtri_ptr[2*1+1] = <void*>(<char*>"nbdtri")             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_ptr[2*2] = <void*>_func_nbdtri_unsafe
 * ufunc_nbdtri_ptr[2*2+1] = <void*>(<char*>"nbdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_ptr[3]) = ((void *)((char *)((char *)"nbdtri")));

  /* "scipy/special/_ufuncs.pyx":10957
 * ufunc_nbdtri_ptr[2*1] = <void*>_func_nbdtri_unsafe
 * ufunc_nbdtri_ptr[2*1+1] = <void*>(<char*>"nbdtri")
 * ufunc_nbdtri_ptr[2*2] = <void*>_func_nbdtri_unsafe             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_ptr[2*2+1] = <void*>(<char*>"nbdtri")
 * ufunc_nbdtri_data[0] = &ufunc_nbdtri_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_nbdtri_unsafe);

  /* "scipy/special/_ufuncs.pyx":10958
 * ufunc_nbdtri_ptr[2*1+1] = <void*>(<char*>"nbdtri")
 * ufunc_nbdtri_ptr[2*2] = <void*>_func_nbdtri_unsafe
 * ufunc_nbdtri_ptr[2*2+1] = <void*>(<char*>"nbdtri")             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_data[0] = &ufunc_nbdtri_ptr[2*0]
 * ufunc_nbdtri_data[1] = &ufunc_nbdtri_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_ptr[5]) = ((void *)((char *)((char *)"nbdtri")));

  /* "scipy/special/_ufuncs.pyx":10959
 * ufunc_nbdtri_ptr[2*2] = <void*>_func_nbdtri_unsafe
 * ufunc_nbdtri_ptr[2*2+1] = <void*>(<char*>"nbdtri")
 * ufunc_nbdtri_data[0] = &ufunc_nbdtri_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_data[1] = &ufunc_nbdtri_ptr[2*1]
 * ufunc_nbdtri_data[2] = &ufunc_nbdtri_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":10960
 * ufunc_nbdtri_ptr[2*2+1] = <void*>(<char*>"nbdtri")
 * ufunc_nbdtri_data[0] = &ufunc_nbdtri_ptr[2*0]
 * ufunc_nbdtri_data[1] = &ufunc_nbdtri_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_nbdtri_data[2] = &ufunc_nbdtri_ptr[2*2]
 * nbdtri = np.PyUFunc_FromFuncAndData(ufunc_nbdtri_loops, ufunc_nbdtri_data, ufunc_nbdtri_types, 3, 3, 1, 0, "nbdtri", ufunc_nbdtri_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":10961
 * ufunc_nbdtri_data[0] = &ufunc_nbdtri_ptr[2*0]
 * ufunc_nbdtri_data[1] = &ufunc_nbdtri_ptr[2*1]
 * ufunc_nbdtri_data[2] = &ufunc_nbdtri_ptr[2*2]             # <<<<<<<<<<<<<<
 * nbdtri = np.PyUFunc_FromFuncAndData(ufunc_nbdtri_loops, ufunc_nbdtri_data, ufunc_nbdtri_types, 3, 3, 1, 0, "nbdtri", ufunc_nbdtri_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":10962
 * ufunc_nbdtri_data[1] = &ufunc_nbdtri_ptr[2*1]
 * ufunc_nbdtri_data[2] = &ufunc_nbdtri_ptr[2*2]
 * nbdtri = np.PyUFunc_FromFuncAndData(ufunc_nbdtri_loops, ufunc_nbdtri_data, ufunc_nbdtri_types, 3, 3, 1, 0, "nbdtri", ufunc_nbdtri_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nbdtrik_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_types, 3, 3, 1, 0, ((char *)"nbdtri"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtri_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10962, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nbdtri, __pyx_t_2) < 0) __PYX_ERR(1, 10962, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":10968
 * cdef void *ufunc_nbdtrik_data[2]
 * cdef char ufunc_nbdtrik_types[8]
 * cdef char *ufunc_nbdtrik_doc = (             # <<<<<<<<<<<<<<
 *     "nbdtrik(y, n, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_doc = ((char *)"nbdtrik(y, n, p)\n\nInverse of `nbdtr` vs `k`.\n\nReturns the inverse with respect to the parameter `k` of\n`y = nbdtr(k, n, p)`, the negative binomial cumulative distribution\nfunction.\n\nParameters\n----------\ny : array_like\n    The probability of `k` or fewer failures before `n` successes (float).\nn : array_like\n    The target number of successes (positive int).\np : array_like\n    Probability of success in a single event (float).\n\nReturns\n-------\nk : ndarray\n    The maximum number of allowed failures such that `nbdtr(k, n, p) = y`.\n\nSee also\n--------\nnbdtr : Cumulative distribution function of the negative binomial.\nnbdtri : Inverse with respect to `p` of `nbdtr(k, n, p)`.\nnbdtrin : Inverse with respect to `n` of `nbdtr(k, n, p)`.\n\nNotes\n-----\nWrapper for the CDFLIB [1]_ Fortran routine `cdfnbn`.\n\nFormula 26.5.26 of [2]_,\n\n.. math::\n    \\sum_{j=k + 1}^\\infty {{n + j - 1}\\choose{j}} p^n (1 - p)^j = I_{1 - p}(k + 1, n),\n\nis used to reduce calculation of the cumulative distribution function to\nthat of a regularized incomplete beta :math:`I`.\n\nComputation of `k` involves a search for a value that produces the desired\nvalue of `y`.  The search relies on the monotonicity of `y` with `k`.\n\nReferences\n----------\n.. [1] Barry Brown, James Lovato, and Kathy Russell,\n       CDFLIB: Library of Fortran Routines for Cumulative Distribution\n       Functions, Inverses, and Other Parameters.\n.. [2] Milton Abramowitz and Irene A. Stegun, eds.\n       Handbook of Mathematical Functions with Formulas,\n       Graphs, and Mathematical Tables. New York: Dover, 1972.");

  /* "scipy/special/_ufuncs.pyx":11020
 *     "       Handbook of Mathematical Functions with Formulas,\n"
 *     "       Graphs, and Mathematical Tables. New York: Dover, 1972.")
 * ufunc_nbdtrik_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtrik_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":11021
 *     "       Graphs, and Mathematical Tables. New York: Dover, 1972.")
 * ufunc_nbdtrik_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_types[0] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":11022
 * ufunc_nbdtrik_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtrik_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_types[1] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11023
 * ufunc_nbdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtrik_types[0] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_types[2] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11024
 * ufunc_nbdtrik_types[0] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[1] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_types[3] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11025
 * ufunc_nbdtrik_types[1] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[2] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_types[4] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11026
 * ufunc_nbdtrik_types[2] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[3] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_types[5] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11027
 * ufunc_nbdtrik_types[3] = <char>NPY_FLOAT
 * ufunc_nbdtrik_types[4] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_types[6] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11028
 * ufunc_nbdtrik_types[4] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_types[5] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_types[7] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_ptr[2*0] = <void*>_func_cdfnbn2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11029
 * ufunc_nbdtrik_types[5] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_types[6] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_ptr[2*0] = <void*>_func_cdfnbn2_wrap
 * ufunc_nbdtrik_ptr[2*0+1] = <void*>(<char*>"nbdtrik")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11030
 * ufunc_nbdtrik_types[6] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_types[7] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_ptr[2*0] = <void*>_func_cdfnbn2_wrap             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_ptr[2*0+1] = <void*>(<char*>"nbdtrik")
 * ufunc_nbdtrik_ptr[2*1] = <void*>_func_cdfnbn2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_ptr[0]) = ((void *)cdfnbn2_wrap);

  /* "scipy/special/_ufuncs.pyx":11031
 * ufunc_nbdtrik_types[7] = <char>NPY_DOUBLE
 * ufunc_nbdtrik_ptr[2*0] = <void*>_func_cdfnbn2_wrap
 * ufunc_nbdtrik_ptr[2*0+1] = <void*>(<char*>"nbdtrik")             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_ptr[2*1] = <void*>_func_cdfnbn2_wrap
 * ufunc_nbdtrik_ptr[2*1+1] = <void*>(<char*>"nbdtrik")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_ptr[1]) = ((void *)((char *)((char *)"nbdtrik")));

  /* "scipy/special/_ufuncs.pyx":11032
 * ufunc_nbdtrik_ptr[2*0] = <void*>_func_cdfnbn2_wrap
 * ufunc_nbdtrik_ptr[2*0+1] = <void*>(<char*>"nbdtrik")
 * ufunc_nbdtrik_ptr[2*1] = <void*>_func_cdfnbn2_wrap             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_ptr[2*1+1] = <void*>(<char*>"nbdtrik")
 * ufunc_nbdtrik_data[0] = &ufunc_nbdtrik_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_ptr[2]) = ((void *)cdfnbn2_wrap);

  /* "scipy/special/_ufuncs.pyx":11033
 * ufunc_nbdtrik_ptr[2*0+1] = <void*>(<char*>"nbdtrik")
 * ufunc_nbdtrik_ptr[2*1] = <void*>_func_cdfnbn2_wrap
 * ufunc_nbdtrik_ptr[2*1+1] = <void*>(<char*>"nbdtrik")             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_data[0] = &ufunc_nbdtrik_ptr[2*0]
 * ufunc_nbdtrik_data[1] = &ufunc_nbdtrik_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_ptr[3]) = ((void *)((char *)((char *)"nbdtrik")));

  /* "scipy/special/_ufuncs.pyx":11034
 * ufunc_nbdtrik_ptr[2*1] = <void*>_func_cdfnbn2_wrap
 * ufunc_nbdtrik_ptr[2*1+1] = <void*>(<char*>"nbdtrik")
 * ufunc_nbdtrik_data[0] = &ufunc_nbdtrik_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nbdtrik_data[1] = &ufunc_nbdtrik_ptr[2*1]
 * nbdtrik = np.PyUFunc_FromFuncAndData(ufunc_nbdtrik_loops, ufunc_nbdtrik_data, ufunc_nbdtrik_types, 2, 3, 1, 0, "nbdtrik", ufunc_nbdtrik_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":11035
 * ufunc_nbdtrik_ptr[2*1+1] = <void*>(<char*>"nbdtrik")
 * ufunc_nbdtrik_data[0] = &ufunc_nbdtrik_ptr[2*0]
 * ufunc_nbdtrik_data[1] = &ufunc_nbdtrik_ptr[2*1]             # <<<<<<<<<<<<<<
 * nbdtrik = np.PyUFunc_FromFuncAndData(ufunc_nbdtrik_loops, ufunc_nbdtrik_data, ufunc_nbdtrik_types, 2, 3, 1, 0, "nbdtrik", ufunc_nbdtrik_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":11036
 * ufunc_nbdtrik_data[0] = &ufunc_nbdtrik_ptr[2*0]
 * ufunc_nbdtrik_data[1] = &ufunc_nbdtrik_ptr[2*1]
 * nbdtrik = np.PyUFunc_FromFuncAndData(ufunc_nbdtrik_loops, ufunc_nbdtrik_data, ufunc_nbdtrik_types, 2, 3, 1, 0, "nbdtrik", ufunc_nbdtrik_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nbdtrin_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_types, 2, 3, 1, 0, ((char *)"nbdtrik"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrik_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11036, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nbdtrik, __pyx_t_2) < 0) __PYX_ERR(1, 11036, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":11042
 * cdef void *ufunc_nbdtrin_data[2]
 * cdef char ufunc_nbdtrin_types[8]
 * cdef char *ufunc_nbdtrin_doc = (             # <<<<<<<<<<<<<<
 *     "nbdtrin(k, y, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_doc = ((char *)"nbdtrin(k, y, p)\n\nInverse of `nbdtr` vs `n`.\n\nReturns the inverse with respect to the parameter `n` of\n`y = nbdtr(k, n, p)`, the negative binomial cumulative distribution\nfunction.\n\nParameters\n----------\nk : array_like\n    The maximum number of allowed failures (nonnegative int).\ny : array_like\n    The probability of `k` or fewer failures before `n` successes (float).\np : array_like\n    Probability of success in a single event (float).\n\nReturns\n-------\nn : ndarray\n    The number of successes `n` such that `nbdtr(k, n, p) = y`.\n\nSee also\n--------\nnbdtr : Cumulative distribution function of the negative binomial.\nnbdtri : Inverse with respect to `p` of `nbdtr(k, n, p)`.\nnbdtrik : Inverse with respect to `k` of `nbdtr(k, n, p)`.\n\nNotes\n-----\nWrapper for the CDFLIB [1]_ Fortran routine `cdfnbn`.\n\nFormula 26.5.26 of [2]_,\n\n.. math::\n    \\sum_{j=k + 1}^\\infty {{n + j - 1}\\choose{j}} p^n (1 - p)^j = I_{1 - p}(k + 1, n),\n\nis used to reduce calculation of the cumulative distribution function to\nthat of a regularized incomplete beta :math:`I`.\n\nComputation of `n` involves a search for a value that produces the desired\nvalue of `y`.  The search relies on the monotonicity of `y` with `n`.\n\nReferences\n----------\n.. [1] Barry Brown, James Lovato, and Kathy Russell,\n       CDFLIB: Library of Fortran Routines for Cumulative Distribution\n       Functions, Inverses, and Other Parameters.\n.. [2] Milton Abramowitz and Irene A. Stegun, eds.\n       Handbook of Mathematical Functions with Formulas,\n       Graphs, and Mathematical Tables. New York: Dover, 1972.");

  /* "scipy/special/_ufuncs.pyx":11094
 *     "       Handbook of Mathematical Functions with Formulas,\n"
 *     "       Graphs, and Mathematical Tables. New York: Dover, 1972.")
 * ufunc_nbdtrin_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtrin_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":11095
 *     "       Graphs, and Mathematical Tables. New York: Dover, 1972.")
 * ufunc_nbdtrin_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtrin_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_types[0] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":11096
 * ufunc_nbdtrin_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nbdtrin_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtrin_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_types[1] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11097
 * ufunc_nbdtrin_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nbdtrin_types[0] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_types[2] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11098
 * ufunc_nbdtrin_types[0] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[1] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_types[3] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11099
 * ufunc_nbdtrin_types[1] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[2] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_types[4] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11100
 * ufunc_nbdtrin_types[2] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[3] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_types[5] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11101
 * ufunc_nbdtrin_types[3] = <char>NPY_FLOAT
 * ufunc_nbdtrin_types[4] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_types[6] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11102
 * ufunc_nbdtrin_types[4] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_types[5] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_types[7] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_ptr[2*0] = <void*>_func_cdfnbn3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11103
 * ufunc_nbdtrin_types[5] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_types[6] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_ptr[2*0] = <void*>_func_cdfnbn3_wrap
 * ufunc_nbdtrin_ptr[2*0+1] = <void*>(<char*>"nbdtrin")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11104
 * ufunc_nbdtrin_types[6] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_types[7] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_ptr[2*0] = <void*>_func_cdfnbn3_wrap             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_ptr[2*0+1] = <void*>(<char*>"nbdtrin")
 * ufunc_nbdtrin_ptr[2*1] = <void*>_func_cdfnbn3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_ptr[0]) = ((void *)cdfnbn3_wrap);

  /* "scipy/special/_ufuncs.pyx":11105
 * ufunc_nbdtrin_types[7] = <char>NPY_DOUBLE
 * ufunc_nbdtrin_ptr[2*0] = <void*>_func_cdfnbn3_wrap
 * ufunc_nbdtrin_ptr[2*0+1] = <void*>(<char*>"nbdtrin")             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_ptr[2*1] = <void*>_func_cdfnbn3_wrap
 * ufunc_nbdtrin_ptr[2*1+1] = <void*>(<char*>"nbdtrin")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_ptr[1]) = ((void *)((char *)((char *)"nbdtrin")));

  /* "scipy/special/_ufuncs.pyx":11106
 * ufunc_nbdtrin_ptr[2*0] = <void*>_func_cdfnbn3_wrap
 * ufunc_nbdtrin_ptr[2*0+1] = <void*>(<char*>"nbdtrin")
 * ufunc_nbdtrin_ptr[2*1] = <void*>_func_cdfnbn3_wrap             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_ptr[2*1+1] = <void*>(<char*>"nbdtrin")
 * ufunc_nbdtrin_data[0] = &ufunc_nbdtrin_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_ptr[2]) = ((void *)cdfnbn3_wrap);

  /* "scipy/special/_ufuncs.pyx":11107
 * ufunc_nbdtrin_ptr[2*0+1] = <void*>(<char*>"nbdtrin")
 * ufunc_nbdtrin_ptr[2*1] = <void*>_func_cdfnbn3_wrap
 * ufunc_nbdtrin_ptr[2*1+1] = <void*>(<char*>"nbdtrin")             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_data[0] = &ufunc_nbdtrin_ptr[2*0]
 * ufunc_nbdtrin_data[1] = &ufunc_nbdtrin_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_ptr[3]) = ((void *)((char *)((char *)"nbdtrin")));

  /* "scipy/special/_ufuncs.pyx":11108
 * ufunc_nbdtrin_ptr[2*1] = <void*>_func_cdfnbn3_wrap
 * ufunc_nbdtrin_ptr[2*1+1] = <void*>(<char*>"nbdtrin")
 * ufunc_nbdtrin_data[0] = &ufunc_nbdtrin_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nbdtrin_data[1] = &ufunc_nbdtrin_ptr[2*1]
 * nbdtrin = np.PyUFunc_FromFuncAndData(ufunc_nbdtrin_loops, ufunc_nbdtrin_data, ufunc_nbdtrin_types, 2, 3, 1, 0, "nbdtrin", ufunc_nbdtrin_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":11109
 * ufunc_nbdtrin_ptr[2*1+1] = <void*>(<char*>"nbdtrin")
 * ufunc_nbdtrin_data[0] = &ufunc_nbdtrin_ptr[2*0]
 * ufunc_nbdtrin_data[1] = &ufunc_nbdtrin_ptr[2*1]             # <<<<<<<<<<<<<<
 * nbdtrin = np.PyUFunc_FromFuncAndData(ufunc_nbdtrin_loops, ufunc_nbdtrin_data, ufunc_nbdtrin_types, 2, 3, 1, 0, "nbdtrin", ufunc_nbdtrin_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":11110
 * ufunc_nbdtrin_data[0] = &ufunc_nbdtrin_ptr[2*0]
 * ufunc_nbdtrin_data[1] = &ufunc_nbdtrin_ptr[2*1]
 * nbdtrin = np.PyUFunc_FromFuncAndData(ufunc_nbdtrin_loops, ufunc_nbdtrin_data, ufunc_nbdtrin_types, 2, 3, 1, 0, "nbdtrin", ufunc_nbdtrin_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ncfdtr_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_types, 2, 3, 1, 0, ((char *)"nbdtrin"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_nbdtrin_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nbdtrin, __pyx_t_2) < 0) __PYX_ERR(1, 11110, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":11116
 * cdef void *ufunc_ncfdtr_data[2]
 * cdef char ufunc_ncfdtr_types[10]
 * cdef char *ufunc_ncfdtr_doc = (             # <<<<<<<<<<<<<<
 *     "ncfdtr(dfn, dfd, nc, f)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_doc = ((char *)"ncfdtr(dfn, dfd, nc, f)\n\nCumulative distribution function of the non-central F distribution.\n\nThe non-central F describes the distribution of,\n\n.. math::\n    Z = \\frac{X/d_n}{Y/d_d}\n\nwhere :math:`X` and :math:`Y` are independently distributed, with\n:math:`X` distributed non-central :math:`\\chi^2` with noncentrality\nparameter `nc` and :math:`d_n` degrees of freedom, and :math:`Y`\ndistributed :math:`\\chi^2` with :math:`d_d` degrees of freedom.\n\nParameters\n----------\ndfn : array_like\n    Degrees of freedom of the numerator sum of squares.  Range (0, inf).\ndfd : array_like\n    Degrees of freedom of the denominator sum of squares.  Range (0, inf).\nnc : array_like\n    Noncentrality parameter.  Should be in range (0, 1e4).\nf : array_like\n    Quantiles, i.e. the upper limit of integration.\n\nReturns\n-------\ncdf : float or ndarray\n    The calculated CDF.  If all inputs are scalar, the return will be a\n    float.  Otherwise it will be an array.\n\nSee Also\n--------\nncfdtri : Quantile function; inverse of `ncfdtr` with respect to `f`.\nncfdtridfd : Inverse of `ncfdtr` with respect to `dfd`.\nncfdtridfn : Inverse of `ncfdtr` with respect to `dfn`.\nncfdtrinc : Inverse of `ncfdtr` with respect to `nc`.\n\nNotes\n-----\nWrapper for the CDFLIB [1]_ Fortran routine `cdffnc`.\n\nThe cumulative distribution function is computed using Formula 26.6.20 of\n[2]_:\n\n.. math::\n    F(d_n, d_d, n_c, f) = \\sum_{j=0}^\\infty e^{-n_c/2} \\frac{(n_c/2)^j}{j!} I_{x}(\\frac{d_n}{2} + j, \\frac{d_d}{2}),\n\nwhere :math:`I` is the regularized incomplete beta function, and\n:math:`x = f d_n/(f d_n + d_d)`.\n\nThe computation time required for this routine is proportional to the\nnoncentrality parameter `nc`.  Very large values of this parameter can\nconsume immense computer resources.  This is why the search range is\nbounded by 10,000.\n\nReferences\n----------\n.. [1] Barry Brown, James Lovato, and Kathy Russell,\n       CDFLIB: Library of Fortran Routines for Cu""mulative Distribution\n       Functions, Inverses, and Other Parameters.\n.. [2] Milton Abramowitz and Irene A. Stegun, eds.\n       Handbook of Mathematical Functions with Formulas,\n       Graphs, and Mathematical Tables. New York: Dover, 1972.\n\nExamples\n--------\n>>> from scipy import special\n>>> from scipy import stats\n>>> import matplotlib.pyplot as plt\n\nPlot the CDF of the non-central F distribution, for nc=0.  Compare with the\nF-distribution from scipy.stats:\n\n>>> x = np.linspace(-1, 8, num=500)\n>>> dfn = 3\n>>> dfd = 2\n>>> ncf_stats = stats.f.cdf(x, dfn, dfd)\n>>> ncf_special = special.ncfdtr(dfn, dfd, 0, x)\n\n>>> fig = plt.figure()\n>>> ax = fig.add_subplot(111)\n>>> ax.plot(x, ncf_stats, 'b-', lw=3)\n>>> ax.plot(x, ncf_special, 'r-')\n>>> plt.show()");

  /* "scipy/special/_ufuncs.pyx":11202
 *     ">>> ax.plot(x, ncf_special, 'r-')\n"
 *     ">>> plt.show()")
 * ufunc_ncfdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_ffff_f);

  /* "scipy/special/_ufuncs.pyx":11203
 *     ">>> plt.show()")
 * ufunc_ncfdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_ncfdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_dddd_d);

  /* "scipy/special/_ufuncs.pyx":11204
 * ufunc_ncfdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_ncfdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11205
 * ufunc_ncfdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtr_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11206
 * ufunc_ncfdtr_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11207
 * ufunc_ncfdtr_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11208
 * ufunc_ncfdtr_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11209
 * ufunc_ncfdtr_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11210
 * ufunc_ncfdtr_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11211
 * ufunc_ncfdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11212
 * ufunc_ncfdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_ptr[2*0] = <void*>_func_cdffnc1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11213
 * ufunc_ncfdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_ptr[2*0] = <void*>_func_cdffnc1_wrap
 * ufunc_ncfdtr_ptr[2*0+1] = <void*>(<char*>"ncfdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11214
 * ufunc_ncfdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_ptr[2*0] = <void*>_func_cdffnc1_wrap             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_ptr[2*0+1] = <void*>(<char*>"ncfdtr")
 * ufunc_ncfdtr_ptr[2*1] = <void*>_func_cdffnc1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_ptr[0]) = ((void *)cdffnc1_wrap);

  /* "scipy/special/_ufuncs.pyx":11215
 * ufunc_ncfdtr_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtr_ptr[2*0] = <void*>_func_cdffnc1_wrap
 * ufunc_ncfdtr_ptr[2*0+1] = <void*>(<char*>"ncfdtr")             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_ptr[2*1] = <void*>_func_cdffnc1_wrap
 * ufunc_ncfdtr_ptr[2*1+1] = <void*>(<char*>"ncfdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_ptr[1]) = ((void *)((char *)((char *)"ncfdtr")));

  /* "scipy/special/_ufuncs.pyx":11216
 * ufunc_ncfdtr_ptr[2*0] = <void*>_func_cdffnc1_wrap
 * ufunc_ncfdtr_ptr[2*0+1] = <void*>(<char*>"ncfdtr")
 * ufunc_ncfdtr_ptr[2*1] = <void*>_func_cdffnc1_wrap             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_ptr[2*1+1] = <void*>(<char*>"ncfdtr")
 * ufunc_ncfdtr_data[0] = &ufunc_ncfdtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_ptr[2]) = ((void *)cdffnc1_wrap);

  /* "scipy/special/_ufuncs.pyx":11217
 * ufunc_ncfdtr_ptr[2*0+1] = <void*>(<char*>"ncfdtr")
 * ufunc_ncfdtr_ptr[2*1] = <void*>_func_cdffnc1_wrap
 * ufunc_ncfdtr_ptr[2*1+1] = <void*>(<char*>"ncfdtr")             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_data[0] = &ufunc_ncfdtr_ptr[2*0]
 * ufunc_ncfdtr_data[1] = &ufunc_ncfdtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_ptr[3]) = ((void *)((char *)((char *)"ncfdtr")));

  /* "scipy/special/_ufuncs.pyx":11218
 * ufunc_ncfdtr_ptr[2*1] = <void*>_func_cdffnc1_wrap
 * ufunc_ncfdtr_ptr[2*1+1] = <void*>(<char*>"ncfdtr")
 * ufunc_ncfdtr_data[0] = &ufunc_ncfdtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ncfdtr_data[1] = &ufunc_ncfdtr_ptr[2*1]
 * ncfdtr = np.PyUFunc_FromFuncAndData(ufunc_ncfdtr_loops, ufunc_ncfdtr_data, ufunc_ncfdtr_types, 2, 4, 1, 0, "ncfdtr", ufunc_ncfdtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":11219
 * ufunc_ncfdtr_ptr[2*1+1] = <void*>(<char*>"ncfdtr")
 * ufunc_ncfdtr_data[0] = &ufunc_ncfdtr_ptr[2*0]
 * ufunc_ncfdtr_data[1] = &ufunc_ncfdtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * ncfdtr = np.PyUFunc_FromFuncAndData(ufunc_ncfdtr_loops, ufunc_ncfdtr_data, ufunc_ncfdtr_types, 2, 4, 1, 0, "ncfdtr", ufunc_ncfdtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":11220
 * ufunc_ncfdtr_data[0] = &ufunc_ncfdtr_ptr[2*0]
 * ufunc_ncfdtr_data[1] = &ufunc_ncfdtr_ptr[2*1]
 * ncfdtr = np.PyUFunc_FromFuncAndData(ufunc_ncfdtr_loops, ufunc_ncfdtr_data, ufunc_ncfdtr_types, 2, 4, 1, 0, "ncfdtr", ufunc_ncfdtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ncfdtri_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_types, 2, 4, 1, 0, ((char *)"ncfdtr"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtr_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ncfdtr, __pyx_t_2) < 0) __PYX_ERR(1, 11220, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":11226
 * cdef void *ufunc_ncfdtri_data[2]
 * cdef char ufunc_ncfdtri_types[10]
 * cdef char *ufunc_ncfdtri_doc = (             # <<<<<<<<<<<<<<
 *     "ncfdtri(dfn, dfd, nc, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_doc = ((char *)"ncfdtri(dfn, dfd, nc, p)\n\nInverse with respect to `f` of the CDF of the non-central F distribution.\n\nSee `ncfdtr` for more details.\n\nParameters\n----------\ndfn : array_like\n    Degrees of freedom of the numerator sum of squares.  Range (0, inf).\ndfd : array_like\n    Degrees of freedom of the denominator sum of squares.  Range (0, inf).\nnc : array_like\n    Noncentrality parameter.  Should be in range (0, 1e4).\np : array_like\n    Value of the cumulative distribution function.  Must be in the\n    range [0, 1].\n\nReturns\n-------\nf : float\n    Quantiles, i.e. the upper limit of integration.\n\nSee Also\n--------\nncfdtr : CDF of the non-central F distribution.\nncfdtridfd : Inverse of `ncfdtr` with respect to `dfd`.\nncfdtridfn : Inverse of `ncfdtr` with respect to `dfn`.\nncfdtrinc : Inverse of `ncfdtr` with respect to `nc`.\n\nExamples\n--------\n>>> from scipy.special import ncfdtr, ncfdtri\n\nCompute the CDF for several values of `f`:\n\n>>> f = [0.5, 1, 1.5]\n>>> p = ncfdtr(2, 3, 1.5, f)\n>>> p\narray([ 0.20782291,  0.36107392,  0.47345752])\n\nCompute the inverse.  We recover the values of `f`, as expected:\n\n>>> ncfdtri(2, 3, 1.5, p)\narray([ 0.5,  1. ,  1.5])");

  /* "scipy/special/_ufuncs.pyx":11272
 *     ">>> ncfdtri(2, 3, 1.5, p)\n"
 *     "array([ 0.5,  1. ,  1.5])")
 * ufunc_ncfdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtri_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_ffff_f);

  /* "scipy/special/_ufuncs.pyx":11273
 *     "array([ 0.5,  1. ,  1.5])")
 * ufunc_ncfdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_ncfdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_dddd_d);

  /* "scipy/special/_ufuncs.pyx":11274
 * ufunc_ncfdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_ncfdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtri_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11275
 * ufunc_ncfdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtri_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11276
 * ufunc_ncfdtri_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11277
 * ufunc_ncfdtri_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11278
 * ufunc_ncfdtri_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11279
 * ufunc_ncfdtri_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11280
 * ufunc_ncfdtri_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11281
 * ufunc_ncfdtri_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11282
 * ufunc_ncfdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_ptr[2*0] = <void*>_func_cdffnc2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11283
 * ufunc_ncfdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_ptr[2*0] = <void*>_func_cdffnc2_wrap
 * ufunc_ncfdtri_ptr[2*0+1] = <void*>(<char*>"ncfdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11284
 * ufunc_ncfdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_ptr[2*0] = <void*>_func_cdffnc2_wrap             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_ptr[2*0+1] = <void*>(<char*>"ncfdtri")
 * ufunc_ncfdtri_ptr[2*1] = <void*>_func_cdffnc2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_ptr[0]) = ((void *)cdffnc2_wrap);

  /* "scipy/special/_ufuncs.pyx":11285
 * ufunc_ncfdtri_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtri_ptr[2*0] = <void*>_func_cdffnc2_wrap
 * ufunc_ncfdtri_ptr[2*0+1] = <void*>(<char*>"ncfdtri")             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_ptr[2*1] = <void*>_func_cdffnc2_wrap
 * ufunc_ncfdtri_ptr[2*1+1] = <void*>(<char*>"ncfdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_ptr[1]) = ((void *)((char *)((char *)"ncfdtri")));

  /* "scipy/special/_ufuncs.pyx":11286
 * ufunc_ncfdtri_ptr[2*0] = <void*>_func_cdffnc2_wrap
 * ufunc_ncfdtri_ptr[2*0+1] = <void*>(<char*>"ncfdtri")
 * ufunc_ncfdtri_ptr[2*1] = <void*>_func_cdffnc2_wrap             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_ptr[2*1+1] = <void*>(<char*>"ncfdtri")
 * ufunc_ncfdtri_data[0] = &ufunc_ncfdtri_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_ptr[2]) = ((void *)cdffnc2_wrap);

  /* "scipy/special/_ufuncs.pyx":11287
 * ufunc_ncfdtri_ptr[2*0+1] = <void*>(<char*>"ncfdtri")
 * ufunc_ncfdtri_ptr[2*1] = <void*>_func_cdffnc2_wrap
 * ufunc_ncfdtri_ptr[2*1+1] = <void*>(<char*>"ncfdtri")             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_data[0] = &ufunc_ncfdtri_ptr[2*0]
 * ufunc_ncfdtri_data[1] = &ufunc_ncfdtri_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_ptr[3]) = ((void *)((char *)((char *)"ncfdtri")));

  /* "scipy/special/_ufuncs.pyx":11288
 * ufunc_ncfdtri_ptr[2*1] = <void*>_func_cdffnc2_wrap
 * ufunc_ncfdtri_ptr[2*1+1] = <void*>(<char*>"ncfdtri")
 * ufunc_ncfdtri_data[0] = &ufunc_ncfdtri_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ncfdtri_data[1] = &ufunc_ncfdtri_ptr[2*1]
 * ncfdtri = np.PyUFunc_FromFuncAndData(ufunc_ncfdtri_loops, ufunc_ncfdtri_data, ufunc_ncfdtri_types, 2, 4, 1, 0, "ncfdtri", ufunc_ncfdtri_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":11289
 * ufunc_ncfdtri_ptr[2*1+1] = <void*>(<char*>"ncfdtri")
 * ufunc_ncfdtri_data[0] = &ufunc_ncfdtri_ptr[2*0]
 * ufunc_ncfdtri_data[1] = &ufunc_ncfdtri_ptr[2*1]             # <<<<<<<<<<<<<<
 * ncfdtri = np.PyUFunc_FromFuncAndData(ufunc_ncfdtri_loops, ufunc_ncfdtri_data, ufunc_ncfdtri_types, 2, 4, 1, 0, "ncfdtri", ufunc_ncfdtri_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":11290
 * ufunc_ncfdtri_data[0] = &ufunc_ncfdtri_ptr[2*0]
 * ufunc_ncfdtri_data[1] = &ufunc_ncfdtri_ptr[2*1]
 * ncfdtri = np.PyUFunc_FromFuncAndData(ufunc_ncfdtri_loops, ufunc_ncfdtri_data, ufunc_ncfdtri_types, 2, 4, 1, 0, "ncfdtri", ufunc_ncfdtri_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ncfdtridfd_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_types, 2, 4, 1, 0, ((char *)"ncfdtri"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtri_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11290, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ncfdtri, __pyx_t_2) < 0) __PYX_ERR(1, 11290, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":11296
 * cdef void *ufunc_ncfdtridfd_data[2]
 * cdef char ufunc_ncfdtridfd_types[10]
 * cdef char *ufunc_ncfdtridfd_doc = (             # <<<<<<<<<<<<<<
 *     "ncfdtridfd(dfn, p, nc, f)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_doc = ((char *)"ncfdtridfd(dfn, p, nc, f)\n\nCalculate degrees of freedom (denominator) for the noncentral F-distribution.\n\nThis is the inverse with respect to `dfd` of `ncfdtr`.\nSee `ncfdtr` for more details.\n\nParameters\n----------\ndfn : array_like\n    Degrees of freedom of the numerator sum of squares.  Range (0, inf).\np : array_like\n    Value of the cumulative distribution function.  Must be in the\n    range [0, 1].\nnc : array_like\n    Noncentrality parameter.  Should be in range (0, 1e4).\nf : array_like\n    Quantiles, i.e. the upper limit of integration.\n\nReturns\n-------\ndfd : float\n    Degrees of freedom of the denominator sum of squares.\n\nSee Also\n--------\nncfdtr : CDF of the non-central F distribution.\nncfdtri : Quantile function; inverse of `ncfdtr` with respect to `f`.\nncfdtridfn : Inverse of `ncfdtr` with respect to `dfn`.\nncfdtrinc : Inverse of `ncfdtr` with respect to `nc`.\n\nNotes\n-----\nThe value of the cumulative noncentral F distribution is not necessarily\nmonotone in either degrees of freedom.  There thus may be two values that\nprovide a given CDF value.  This routine assumes monotonicity and will\nfind an arbitrary one of the two values.\n\nExamples\n--------\n>>> from scipy.special import ncfdtr, ncfdtridfd\n\nCompute the CDF for several values of `dfd`:\n\n>>> dfd = [1, 2, 3]\n>>> p = ncfdtr(2, dfd, 0.25, 15)\n>>> p\narray([ 0.8097138 ,  0.93020416,  0.96787852])\n\nCompute the inverse.  We recover the values of `dfd`, as expected:\n\n>>> ncfdtridfd(2, p, 0.25, 15)\narray([ 1.,  2.,  3.])");

  /* "scipy/special/_ufuncs.pyx":11350
 *     ">>> ncfdtridfd(2, p, 0.25, 15)\n"
 *     "array([ 1.,  2.,  3.])")
 * ufunc_ncfdtridfd_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtridfd_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_ffff_f);

  /* "scipy/special/_ufuncs.pyx":11351
 *     "array([ 1.,  2.,  3.])")
 * ufunc_ncfdtridfd_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_ncfdtridfd_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_dddd_d);

  /* "scipy/special/_ufuncs.pyx":11352
 * ufunc_ncfdtridfd_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_ncfdtridfd_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtridfd_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11353
 * ufunc_ncfdtridfd_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtridfd_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11354
 * ufunc_ncfdtridfd_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11355
 * ufunc_ncfdtridfd_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11356
 * ufunc_ncfdtridfd_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11357
 * ufunc_ncfdtridfd_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11358
 * ufunc_ncfdtridfd_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtridfd_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11359
 * ufunc_ncfdtridfd_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11360
 * ufunc_ncfdtridfd_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_ptr[2*0] = <void*>_func_cdffnc4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11361
 * ufunc_ncfdtridfd_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_ptr[2*0] = <void*>_func_cdffnc4_wrap
 * ufunc_ncfdtridfd_ptr[2*0+1] = <void*>(<char*>"ncfdtridfd")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11362
 * ufunc_ncfdtridfd_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_ptr[2*0] = <void*>_func_cdffnc4_wrap             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_ptr[2*0+1] = <void*>(<char*>"ncfdtridfd")
 * ufunc_ncfdtridfd_ptr[2*1] = <void*>_func_cdffnc4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_ptr[0]) = ((void *)cdffnc4_wrap);

  /* "scipy/special/_ufuncs.pyx":11363
 * ufunc_ncfdtridfd_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfd_ptr[2*0] = <void*>_func_cdffnc4_wrap
 * ufunc_ncfdtridfd_ptr[2*0+1] = <void*>(<char*>"ncfdtridfd")             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_ptr[2*1] = <void*>_func_cdffnc4_wrap
 * ufunc_ncfdtridfd_ptr[2*1+1] = <void*>(<char*>"ncfdtridfd")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_ptr[1]) = ((void *)((char *)((char *)"ncfdtridfd")));

  /* "scipy/special/_ufuncs.pyx":11364
 * ufunc_ncfdtridfd_ptr[2*0] = <void*>_func_cdffnc4_wrap
 * ufunc_ncfdtridfd_ptr[2*0+1] = <void*>(<char*>"ncfdtridfd")
 * ufunc_ncfdtridfd_ptr[2*1] = <void*>_func_cdffnc4_wrap             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_ptr[2*1+1] = <void*>(<char*>"ncfdtridfd")
 * ufunc_ncfdtridfd_data[0] = &ufunc_ncfdtridfd_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_ptr[2]) = ((void *)cdffnc4_wrap);

  /* "scipy/special/_ufuncs.pyx":11365
 * ufunc_ncfdtridfd_ptr[2*0+1] = <void*>(<char*>"ncfdtridfd")
 * ufunc_ncfdtridfd_ptr[2*1] = <void*>_func_cdffnc4_wrap
 * ufunc_ncfdtridfd_ptr[2*1+1] = <void*>(<char*>"ncfdtridfd")             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_data[0] = &ufunc_ncfdtridfd_ptr[2*0]
 * ufunc_ncfdtridfd_data[1] = &ufunc_ncfdtridfd_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_ptr[3]) = ((void *)((char *)((char *)"ncfdtridfd")));

  /* "scipy/special/_ufuncs.pyx":11366
 * ufunc_ncfdtridfd_ptr[2*1] = <void*>_func_cdffnc4_wrap
 * ufunc_ncfdtridfd_ptr[2*1+1] = <void*>(<char*>"ncfdtridfd")
 * ufunc_ncfdtridfd_data[0] = &ufunc_ncfdtridfd_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfd_data[1] = &ufunc_ncfdtridfd_ptr[2*1]
 * ncfdtridfd = np.PyUFunc_FromFuncAndData(ufunc_ncfdtridfd_loops, ufunc_ncfdtridfd_data, ufunc_ncfdtridfd_types, 2, 4, 1, 0, "ncfdtridfd", ufunc_ncfdtridfd_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":11367
 * ufunc_ncfdtridfd_ptr[2*1+1] = <void*>(<char*>"ncfdtridfd")
 * ufunc_ncfdtridfd_data[0] = &ufunc_ncfdtridfd_ptr[2*0]
 * ufunc_ncfdtridfd_data[1] = &ufunc_ncfdtridfd_ptr[2*1]             # <<<<<<<<<<<<<<
 * ncfdtridfd = np.PyUFunc_FromFuncAndData(ufunc_ncfdtridfd_loops, ufunc_ncfdtridfd_data, ufunc_ncfdtridfd_types, 2, 4, 1, 0, "ncfdtridfd", ufunc_ncfdtridfd_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":11368
 * ufunc_ncfdtridfd_data[0] = &ufunc_ncfdtridfd_ptr[2*0]
 * ufunc_ncfdtridfd_data[1] = &ufunc_ncfdtridfd_ptr[2*1]
 * ncfdtridfd = np.PyUFunc_FromFuncAndData(ufunc_ncfdtridfd_loops, ufunc_ncfdtridfd_data, ufunc_ncfdtridfd_types, 2, 4, 1, 0, "ncfdtridfd", ufunc_ncfdtridfd_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ncfdtridfn_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_types, 2, 4, 1, 0, ((char *)"ncfdtridfd"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfd_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11368, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ncfdtridfd, __pyx_t_2) < 0) __PYX_ERR(1, 11368, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":11374
 * cdef void *ufunc_ncfdtridfn_data[2]
 * cdef char ufunc_ncfdtridfn_types[10]
 * cdef char *ufunc_ncfdtridfn_doc = (             # <<<<<<<<<<<<<<
 *     "ncfdtridfn(p, dfd, nc, f)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_doc = ((char *)"ncfdtridfn(p, dfd, nc, f)\n\nCalculate degrees of freedom (numerator) for the noncentral F-distribution.\n\nThis is the inverse with respect to `dfn` of `ncfdtr`.\nSee `ncfdtr` for more details.\n\nParameters\n----------\np : array_like\n    Value of the cumulative distribution function.  Must be in the\n    range [0, 1].\ndfd : array_like\n    Degrees of freedom of the denominator sum of squares.  Range (0, inf).\nnc : array_like\n    Noncentrality parameter.  Should be in range (0, 1e4).\nf : float\n    Quantiles, i.e. the upper limit of integration.\n\nReturns\n-------\ndfn : float\n    Degrees of freedom of the numerator sum of squares.\n\nSee Also\n--------\nncfdtr : CDF of the non-central F distribution.\nncfdtri : Quantile function; inverse of `ncfdtr` with respect to `f`.\nncfdtridfd : Inverse of `ncfdtr` with respect to `dfd`.\nncfdtrinc : Inverse of `ncfdtr` with respect to `nc`.\n\nNotes\n-----\nThe value of the cumulative noncentral F distribution is not necessarily\nmonotone in either degrees of freedom.  There thus may be two values that\nprovide a given CDF value.  This routine assumes monotonicity and will\nfind an arbitrary one of the two values.\n\nExamples\n--------\n>>> from scipy.special import ncfdtr, ncfdtridfn\n\nCompute the CDF for several values of `dfn`:\n\n>>> dfn = [1, 2, 3]\n>>> p = ncfdtr(dfn, 2, 0.25, 15)\n>>> p\narray([ 0.92562363,  0.93020416,  0.93188394])\n\nCompute the inverse.  We recover the values of `dfn`, as expected:\n\n>>> ncfdtridfn(p, 2, 0.25, 15)\narray([ 1.,  2.,  3.])");

  /* "scipy/special/_ufuncs.pyx":11428
 *     ">>> ncfdtridfn(p, 2, 0.25, 15)\n"
 *     "array([ 1.,  2.,  3.])")
 * ufunc_ncfdtridfn_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtridfn_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_ffff_f);

  /* "scipy/special/_ufuncs.pyx":11429
 *     "array([ 1.,  2.,  3.])")
 * ufunc_ncfdtridfn_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_ncfdtridfn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_dddd_d);

  /* "scipy/special/_ufuncs.pyx":11430
 * ufunc_ncfdtridfn_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_ncfdtridfn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtridfn_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11431
 * ufunc_ncfdtridfn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtridfn_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11432
 * ufunc_ncfdtridfn_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11433
 * ufunc_ncfdtridfn_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11434
 * ufunc_ncfdtridfn_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11435
 * ufunc_ncfdtridfn_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11436
 * ufunc_ncfdtridfn_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtridfn_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11437
 * ufunc_ncfdtridfn_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11438
 * ufunc_ncfdtridfn_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_ptr[2*0] = <void*>_func_cdffnc3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11439
 * ufunc_ncfdtridfn_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_ptr[2*0] = <void*>_func_cdffnc3_wrap
 * ufunc_ncfdtridfn_ptr[2*0+1] = <void*>(<char*>"ncfdtridfn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11440
 * ufunc_ncfdtridfn_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_ptr[2*0] = <void*>_func_cdffnc3_wrap             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_ptr[2*0+1] = <void*>(<char*>"ncfdtridfn")
 * ufunc_ncfdtridfn_ptr[2*1] = <void*>_func_cdffnc3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_ptr[0]) = ((void *)cdffnc3_wrap);

  /* "scipy/special/_ufuncs.pyx":11441
 * ufunc_ncfdtridfn_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtridfn_ptr[2*0] = <void*>_func_cdffnc3_wrap
 * ufunc_ncfdtridfn_ptr[2*0+1] = <void*>(<char*>"ncfdtridfn")             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_ptr[2*1] = <void*>_func_cdffnc3_wrap
 * ufunc_ncfdtridfn_ptr[2*1+1] = <void*>(<char*>"ncfdtridfn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_ptr[1]) = ((void *)((char *)((char *)"ncfdtridfn")));

  /* "scipy/special/_ufuncs.pyx":11442
 * ufunc_ncfdtridfn_ptr[2*0] = <void*>_func_cdffnc3_wrap
 * ufunc_ncfdtridfn_ptr[2*0+1] = <void*>(<char*>"ncfdtridfn")
 * ufunc_ncfdtridfn_ptr[2*1] = <void*>_func_cdffnc3_wrap             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_ptr[2*1+1] = <void*>(<char*>"ncfdtridfn")
 * ufunc_ncfdtridfn_data[0] = &ufunc_ncfdtridfn_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_ptr[2]) = ((void *)cdffnc3_wrap);

  /* "scipy/special/_ufuncs.pyx":11443
 * ufunc_ncfdtridfn_ptr[2*0+1] = <void*>(<char*>"ncfdtridfn")
 * ufunc_ncfdtridfn_ptr[2*1] = <void*>_func_cdffnc3_wrap
 * ufunc_ncfdtridfn_ptr[2*1+1] = <void*>(<char*>"ncfdtridfn")             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_data[0] = &ufunc_ncfdtridfn_ptr[2*0]
 * ufunc_ncfdtridfn_data[1] = &ufunc_ncfdtridfn_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_ptr[3]) = ((void *)((char *)((char *)"ncfdtridfn")));

  /* "scipy/special/_ufuncs.pyx":11444
 * ufunc_ncfdtridfn_ptr[2*1] = <void*>_func_cdffnc3_wrap
 * ufunc_ncfdtridfn_ptr[2*1+1] = <void*>(<char*>"ncfdtridfn")
 * ufunc_ncfdtridfn_data[0] = &ufunc_ncfdtridfn_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ncfdtridfn_data[1] = &ufunc_ncfdtridfn_ptr[2*1]
 * ncfdtridfn = np.PyUFunc_FromFuncAndData(ufunc_ncfdtridfn_loops, ufunc_ncfdtridfn_data, ufunc_ncfdtridfn_types, 2, 4, 1, 0, "ncfdtridfn", ufunc_ncfdtridfn_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":11445
 * ufunc_ncfdtridfn_ptr[2*1+1] = <void*>(<char*>"ncfdtridfn")
 * ufunc_ncfdtridfn_data[0] = &ufunc_ncfdtridfn_ptr[2*0]
 * ufunc_ncfdtridfn_data[1] = &ufunc_ncfdtridfn_ptr[2*1]             # <<<<<<<<<<<<<<
 * ncfdtridfn = np.PyUFunc_FromFuncAndData(ufunc_ncfdtridfn_loops, ufunc_ncfdtridfn_data, ufunc_ncfdtridfn_types, 2, 4, 1, 0, "ncfdtridfn", ufunc_ncfdtridfn_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":11446
 * ufunc_ncfdtridfn_data[0] = &ufunc_ncfdtridfn_ptr[2*0]
 * ufunc_ncfdtridfn_data[1] = &ufunc_ncfdtridfn_ptr[2*1]
 * ncfdtridfn = np.PyUFunc_FromFuncAndData(ufunc_ncfdtridfn_loops, ufunc_ncfdtridfn_data, ufunc_ncfdtridfn_types, 2, 4, 1, 0, "ncfdtridfn", ufunc_ncfdtridfn_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ncfdtrinc_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_types, 2, 4, 1, 0, ((char *)"ncfdtridfn"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtridfn_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ncfdtridfn, __pyx_t_2) < 0) __PYX_ERR(1, 11446, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":11452
 * cdef void *ufunc_ncfdtrinc_data[2]
 * cdef char ufunc_ncfdtrinc_types[10]
 * cdef char *ufunc_ncfdtrinc_doc = (             # <<<<<<<<<<<<<<
 *     "ncfdtrinc(dfn, dfd, p, f)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_doc = ((char *)"ncfdtrinc(dfn, dfd, p, f)\n\nCalculate non-centrality parameter for non-central F distribution.\n\nThis is the inverse with respect to `nc` of `ncfdtr`.\nSee `ncfdtr` for more details.\n\nParameters\n----------\ndfn : array_like\n    Degrees of freedom of the numerator sum of squares.  Range (0, inf).\ndfd : array_like\n    Degrees of freedom of the denominator sum of squares.  Range (0, inf).\np : array_like\n    Value of the cumulative distribution function.  Must be in the\n    range [0, 1].\nf : array_like\n    Quantiles, i.e. the upper limit of integration.\n\nReturns\n-------\nnc : float\n    Noncentrality parameter.\n\nSee Also\n--------\nncfdtr : CDF of the non-central F distribution.\nncfdtri : Quantile function; inverse of `ncfdtr` with respect to `f`.\nncfdtridfd : Inverse of `ncfdtr` with respect to `dfd`.\nncfdtridfn : Inverse of `ncfdtr` with respect to `dfn`.\n\nExamples\n--------\n>>> from scipy.special import ncfdtr, ncfdtrinc\n\nCompute the CDF for several values of `nc`:\n\n>>> nc = [0.5, 1.5, 2.0]\n>>> p = ncfdtr(2, 3, nc, 15)\n>>> p\narray([ 0.96309246,  0.94327955,  0.93304098])\n\nCompute the inverse.  We recover the values of `nc`, as expected:\n\n>>> ncfdtrinc(2, 3, p, 15)\narray([ 0.5,  1.5,  2. ])");

  /* "scipy/special/_ufuncs.pyx":11499
 *     ">>> ncfdtrinc(2, 3, p, 15)\n"
 *     "array([ 0.5,  1.5,  2. ])")
 * ufunc_ncfdtrinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtrinc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_ffff_f);

  /* "scipy/special/_ufuncs.pyx":11500
 *     "array([ 0.5,  1.5,  2. ])")
 * ufunc_ncfdtrinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_ncfdtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd__As_dddd_d);

  /* "scipy/special/_ufuncs.pyx":11501
 * ufunc_ncfdtrinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd__As_ffff_f
 * ufunc_ncfdtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtrinc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11502
 * ufunc_ncfdtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd__As_dddd_d
 * ufunc_ncfdtrinc_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11503
 * ufunc_ncfdtrinc_types[0] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11504
 * ufunc_ncfdtrinc_types[1] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11505
 * ufunc_ncfdtrinc_types[2] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11506
 * ufunc_ncfdtrinc_types[3] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11507
 * ufunc_ncfdtrinc_types[4] = <char>NPY_FLOAT
 * ufunc_ncfdtrinc_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11508
 * ufunc_ncfdtrinc_types[5] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11509
 * ufunc_ncfdtrinc_types[6] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_ptr[2*0] = <void*>_func_cdffnc5_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11510
 * ufunc_ncfdtrinc_types[7] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_ptr[2*0] = <void*>_func_cdffnc5_wrap
 * ufunc_ncfdtrinc_ptr[2*0+1] = <void*>(<char*>"ncfdtrinc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11511
 * ufunc_ncfdtrinc_types[8] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_ptr[2*0] = <void*>_func_cdffnc5_wrap             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_ptr[2*0+1] = <void*>(<char*>"ncfdtrinc")
 * ufunc_ncfdtrinc_ptr[2*1] = <void*>_func_cdffnc5_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_ptr[0]) = ((void *)cdffnc5_wrap);

  /* "scipy/special/_ufuncs.pyx":11512
 * ufunc_ncfdtrinc_types[9] = <char>NPY_DOUBLE
 * ufunc_ncfdtrinc_ptr[2*0] = <void*>_func_cdffnc5_wrap
 * ufunc_ncfdtrinc_ptr[2*0+1] = <void*>(<char*>"ncfdtrinc")             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_ptr[2*1] = <void*>_func_cdffnc5_wrap
 * ufunc_ncfdtrinc_ptr[2*1+1] = <void*>(<char*>"ncfdtrinc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_ptr[1]) = ((void *)((char *)((char *)"ncfdtrinc")));

  /* "scipy/special/_ufuncs.pyx":11513
 * ufunc_ncfdtrinc_ptr[2*0] = <void*>_func_cdffnc5_wrap
 * ufunc_ncfdtrinc_ptr[2*0+1] = <void*>(<char*>"ncfdtrinc")
 * ufunc_ncfdtrinc_ptr[2*1] = <void*>_func_cdffnc5_wrap             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_ptr[2*1+1] = <void*>(<char*>"ncfdtrinc")
 * ufunc_ncfdtrinc_data[0] = &ufunc_ncfdtrinc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_ptr[2]) = ((void *)cdffnc5_wrap);

  /* "scipy/special/_ufuncs.pyx":11514
 * ufunc_ncfdtrinc_ptr[2*0+1] = <void*>(<char*>"ncfdtrinc")
 * ufunc_ncfdtrinc_ptr[2*1] = <void*>_func_cdffnc5_wrap
 * ufunc_ncfdtrinc_ptr[2*1+1] = <void*>(<char*>"ncfdtrinc")             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_data[0] = &ufunc_ncfdtrinc_ptr[2*0]
 * ufunc_ncfdtrinc_data[1] = &ufunc_ncfdtrinc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_ptr[3]) = ((void *)((char *)((char *)"ncfdtrinc")));

  /* "scipy/special/_ufuncs.pyx":11515
 * ufunc_ncfdtrinc_ptr[2*1] = <void*>_func_cdffnc5_wrap
 * ufunc_ncfdtrinc_ptr[2*1+1] = <void*>(<char*>"ncfdtrinc")
 * ufunc_ncfdtrinc_data[0] = &ufunc_ncfdtrinc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ncfdtrinc_data[1] = &ufunc_ncfdtrinc_ptr[2*1]
 * ncfdtrinc = np.PyUFunc_FromFuncAndData(ufunc_ncfdtrinc_loops, ufunc_ncfdtrinc_data, ufunc_ncfdtrinc_types, 2, 4, 1, 0, "ncfdtrinc", ufunc_ncfdtrinc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":11516
 * ufunc_ncfdtrinc_ptr[2*1+1] = <void*>(<char*>"ncfdtrinc")
 * ufunc_ncfdtrinc_data[0] = &ufunc_ncfdtrinc_ptr[2*0]
 * ufunc_ncfdtrinc_data[1] = &ufunc_ncfdtrinc_ptr[2*1]             # <<<<<<<<<<<<<<
 * ncfdtrinc = np.PyUFunc_FromFuncAndData(ufunc_ncfdtrinc_loops, ufunc_ncfdtrinc_data, ufunc_ncfdtrinc_types, 2, 4, 1, 0, "ncfdtrinc", ufunc_ncfdtrinc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":11517
 * ufunc_ncfdtrinc_data[0] = &ufunc_ncfdtrinc_ptr[2*0]
 * ufunc_ncfdtrinc_data[1] = &ufunc_ncfdtrinc_ptr[2*1]
 * ncfdtrinc = np.PyUFunc_FromFuncAndData(ufunc_ncfdtrinc_loops, ufunc_ncfdtrinc_data, ufunc_ncfdtrinc_types, 2, 4, 1, 0, "ncfdtrinc", ufunc_ncfdtrinc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nctdtr_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_types, 2, 4, 1, 0, ((char *)"ncfdtrinc"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_ncfdtrinc_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11517, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ncfdtrinc, __pyx_t_2) < 0) __PYX_ERR(1, 11517, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":11523
 * cdef void *ufunc_nctdtr_data[2]
 * cdef char ufunc_nctdtr_types[8]
 * cdef char *ufunc_nctdtr_doc = (             # <<<<<<<<<<<<<<
 *     "nctdtr(df, nc, t)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_doc = ((char *)"nctdtr(df, nc, t)\n\nCumulative distribution function of the non-central `t` distribution.\n\nParameters\n----------\ndf : array_like\n    Degrees of freedom of the distribution.  Should be in range (0, inf).\nnc : array_like\n    Noncentrality parameter.  Should be in range (-1e6, 1e6).\nt : array_like\n    Quantiles, i.e. the upper limit of integration.\n\nReturns\n-------\ncdf : float or ndarray\n    The calculated CDF.  If all inputs are scalar, the return will be a\n    float.  Otherwise it will be an array.\n\nSee Also\n--------\nnctdtrit : Inverse CDF (iCDF) of the non-central t distribution.\nnctdtridf : Calculate degrees of freedom, given CDF and iCDF values.\nnctdtrinc : Calculate non-centrality parameter, given CDF iCDF values.\n\nExamples\n--------\n>>> from scipy import special\n>>> from scipy import stats\n>>> import matplotlib.pyplot as plt\n\nPlot the CDF of the non-central t distribution, for nc=0.  Compare with the\nt-distribution from scipy.stats:\n\n>>> x = np.linspace(-5, 5, num=500)\n>>> df = 3\n>>> nct_stats = stats.t.cdf(x, df)\n>>> nct_special = special.nctdtr(df, 0, x)\n\n>>> fig = plt.figure()\n>>> ax = fig.add_subplot(111)\n>>> ax.plot(x, nct_stats, 'b-', lw=3)\n>>> ax.plot(x, nct_special, 'r-')\n>>> plt.show()");

  /* "scipy/special/_ufuncs.pyx":11568
 *     ">>> ax.plot(x, nct_special, 'r-')\n"
 *     ">>> plt.show()")
 * ufunc_nctdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":11569
 *     ">>> plt.show()")
 * ufunc_nctdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nctdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":11570
 * ufunc_nctdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nctdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11571
 * ufunc_nctdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtr_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11572
 * ufunc_nctdtr_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11573
 * ufunc_nctdtr_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtr_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11574
 * ufunc_nctdtr_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtr_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11575
 * ufunc_nctdtr_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtr_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtr_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11576
 * ufunc_nctdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtr_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtr_ptr[2*0] = <void*>_func_cdftnc1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11577
 * ufunc_nctdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtr_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_ptr[2*0] = <void*>_func_cdftnc1_wrap
 * ufunc_nctdtr_ptr[2*0+1] = <void*>(<char*>"nctdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11578
 * ufunc_nctdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtr_ptr[2*0] = <void*>_func_cdftnc1_wrap             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_ptr[2*0+1] = <void*>(<char*>"nctdtr")
 * ufunc_nctdtr_ptr[2*1] = <void*>_func_cdftnc1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_ptr[0]) = ((void *)cdftnc1_wrap);

  /* "scipy/special/_ufuncs.pyx":11579
 * ufunc_nctdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtr_ptr[2*0] = <void*>_func_cdftnc1_wrap
 * ufunc_nctdtr_ptr[2*0+1] = <void*>(<char*>"nctdtr")             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_ptr[2*1] = <void*>_func_cdftnc1_wrap
 * ufunc_nctdtr_ptr[2*1+1] = <void*>(<char*>"nctdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_ptr[1]) = ((void *)((char *)((char *)"nctdtr")));

  /* "scipy/special/_ufuncs.pyx":11580
 * ufunc_nctdtr_ptr[2*0] = <void*>_func_cdftnc1_wrap
 * ufunc_nctdtr_ptr[2*0+1] = <void*>(<char*>"nctdtr")
 * ufunc_nctdtr_ptr[2*1] = <void*>_func_cdftnc1_wrap             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_ptr[2*1+1] = <void*>(<char*>"nctdtr")
 * ufunc_nctdtr_data[0] = &ufunc_nctdtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_ptr[2]) = ((void *)cdftnc1_wrap);

  /* "scipy/special/_ufuncs.pyx":11581
 * ufunc_nctdtr_ptr[2*0+1] = <void*>(<char*>"nctdtr")
 * ufunc_nctdtr_ptr[2*1] = <void*>_func_cdftnc1_wrap
 * ufunc_nctdtr_ptr[2*1+1] = <void*>(<char*>"nctdtr")             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_data[0] = &ufunc_nctdtr_ptr[2*0]
 * ufunc_nctdtr_data[1] = &ufunc_nctdtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_ptr[3]) = ((void *)((char *)((char *)"nctdtr")));

  /* "scipy/special/_ufuncs.pyx":11582
 * ufunc_nctdtr_ptr[2*1] = <void*>_func_cdftnc1_wrap
 * ufunc_nctdtr_ptr[2*1+1] = <void*>(<char*>"nctdtr")
 * ufunc_nctdtr_data[0] = &ufunc_nctdtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nctdtr_data[1] = &ufunc_nctdtr_ptr[2*1]
 * nctdtr = np.PyUFunc_FromFuncAndData(ufunc_nctdtr_loops, ufunc_nctdtr_data, ufunc_nctdtr_types, 2, 3, 1, 0, "nctdtr", ufunc_nctdtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":11583
 * ufunc_nctdtr_ptr[2*1+1] = <void*>(<char*>"nctdtr")
 * ufunc_nctdtr_data[0] = &ufunc_nctdtr_ptr[2*0]
 * ufunc_nctdtr_data[1] = &ufunc_nctdtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * nctdtr = np.PyUFunc_FromFuncAndData(ufunc_nctdtr_loops, ufunc_nctdtr_data, ufunc_nctdtr_types, 2, 3, 1, 0, "nctdtr", ufunc_nctdtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":11584
 * ufunc_nctdtr_data[0] = &ufunc_nctdtr_ptr[2*0]
 * ufunc_nctdtr_data[1] = &ufunc_nctdtr_ptr[2*1]
 * nctdtr = np.PyUFunc_FromFuncAndData(ufunc_nctdtr_loops, ufunc_nctdtr_data, ufunc_nctdtr_types, 2, 3, 1, 0, "nctdtr", ufunc_nctdtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nctdtridf_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_types, 2, 3, 1, 0, ((char *)"nctdtr"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtr_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nctdtr, __pyx_t_2) < 0) __PYX_ERR(1, 11584, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":11590
 * cdef void *ufunc_nctdtridf_data[2]
 * cdef char ufunc_nctdtridf_types[8]
 * cdef char *ufunc_nctdtridf_doc = (             # <<<<<<<<<<<<<<
 *     "nctdtridf(p, nc, t)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_doc = ((char *)"nctdtridf(p, nc, t)\n\nCalculate degrees of freedom for non-central t distribution.\n\nSee `nctdtr` for more details.\n\nParameters\n----------\np : array_like\n    CDF values, in range (0, 1].\nnc : array_like\n    Noncentrality parameter.  Should be in range (-1e6, 1e6).\nt : array_like\n    Quantiles, i.e. the upper limit of integration.");

  /* "scipy/special/_ufuncs.pyx":11605
 *     "t : array_like\n"
 *     "    Quantiles, i.e. the upper limit of integration.")
 * ufunc_nctdtridf_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtridf_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":11606
 *     "    Quantiles, i.e. the upper limit of integration.")
 * ufunc_nctdtridf_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nctdtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":11607
 * ufunc_nctdtridf_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nctdtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtridf_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11608
 * ufunc_nctdtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtridf_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11609
 * ufunc_nctdtridf_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11610
 * ufunc_nctdtridf_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11611
 * ufunc_nctdtridf_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11612
 * ufunc_nctdtridf_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtridf_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11613
 * ufunc_nctdtridf_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_ptr[2*0] = <void*>_func_cdftnc3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11614
 * ufunc_nctdtridf_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_ptr[2*0] = <void*>_func_cdftnc3_wrap
 * ufunc_nctdtridf_ptr[2*0+1] = <void*>(<char*>"nctdtridf")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11615
 * ufunc_nctdtridf_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_ptr[2*0] = <void*>_func_cdftnc3_wrap             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_ptr[2*0+1] = <void*>(<char*>"nctdtridf")
 * ufunc_nctdtridf_ptr[2*1] = <void*>_func_cdftnc3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_ptr[0]) = ((void *)cdftnc3_wrap);

  /* "scipy/special/_ufuncs.pyx":11616
 * ufunc_nctdtridf_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtridf_ptr[2*0] = <void*>_func_cdftnc3_wrap
 * ufunc_nctdtridf_ptr[2*0+1] = <void*>(<char*>"nctdtridf")             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_ptr[2*1] = <void*>_func_cdftnc3_wrap
 * ufunc_nctdtridf_ptr[2*1+1] = <void*>(<char*>"nctdtridf")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_ptr[1]) = ((void *)((char *)((char *)"nctdtridf")));

  /* "scipy/special/_ufuncs.pyx":11617
 * ufunc_nctdtridf_ptr[2*0] = <void*>_func_cdftnc3_wrap
 * ufunc_nctdtridf_ptr[2*0+1] = <void*>(<char*>"nctdtridf")
 * ufunc_nctdtridf_ptr[2*1] = <void*>_func_cdftnc3_wrap             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_ptr[2*1+1] = <void*>(<char*>"nctdtridf")
 * ufunc_nctdtridf_data[0] = &ufunc_nctdtridf_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_ptr[2]) = ((void *)cdftnc3_wrap);

  /* "scipy/special/_ufuncs.pyx":11618
 * ufunc_nctdtridf_ptr[2*0+1] = <void*>(<char*>"nctdtridf")
 * ufunc_nctdtridf_ptr[2*1] = <void*>_func_cdftnc3_wrap
 * ufunc_nctdtridf_ptr[2*1+1] = <void*>(<char*>"nctdtridf")             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_data[0] = &ufunc_nctdtridf_ptr[2*0]
 * ufunc_nctdtridf_data[1] = &ufunc_nctdtridf_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_ptr[3]) = ((void *)((char *)((char *)"nctdtridf")));

  /* "scipy/special/_ufuncs.pyx":11619
 * ufunc_nctdtridf_ptr[2*1] = <void*>_func_cdftnc3_wrap
 * ufunc_nctdtridf_ptr[2*1+1] = <void*>(<char*>"nctdtridf")
 * ufunc_nctdtridf_data[0] = &ufunc_nctdtridf_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nctdtridf_data[1] = &ufunc_nctdtridf_ptr[2*1]
 * nctdtridf = np.PyUFunc_FromFuncAndData(ufunc_nctdtridf_loops, ufunc_nctdtridf_data, ufunc_nctdtridf_types, 2, 3, 1, 0, "nctdtridf", ufunc_nctdtridf_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":11620
 * ufunc_nctdtridf_ptr[2*1+1] = <void*>(<char*>"nctdtridf")
 * ufunc_nctdtridf_data[0] = &ufunc_nctdtridf_ptr[2*0]
 * ufunc_nctdtridf_data[1] = &ufunc_nctdtridf_ptr[2*1]             # <<<<<<<<<<<<<<
 * nctdtridf = np.PyUFunc_FromFuncAndData(ufunc_nctdtridf_loops, ufunc_nctdtridf_data, ufunc_nctdtridf_types, 2, 3, 1, 0, "nctdtridf", ufunc_nctdtridf_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":11621
 * ufunc_nctdtridf_data[0] = &ufunc_nctdtridf_ptr[2*0]
 * ufunc_nctdtridf_data[1] = &ufunc_nctdtridf_ptr[2*1]
 * nctdtridf = np.PyUFunc_FromFuncAndData(ufunc_nctdtridf_loops, ufunc_nctdtridf_data, ufunc_nctdtridf_types, 2, 3, 1, 0, "nctdtridf", ufunc_nctdtridf_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nctdtrinc_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_types, 2, 3, 1, 0, ((char *)"nctdtridf"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtridf_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11621, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nctdtridf, __pyx_t_2) < 0) __PYX_ERR(1, 11621, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":11627
 * cdef void *ufunc_nctdtrinc_data[2]
 * cdef char ufunc_nctdtrinc_types[8]
 * cdef char *ufunc_nctdtrinc_doc = (             # <<<<<<<<<<<<<<
 *     "nctdtrinc(df, p, t)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_doc = ((char *)"nctdtrinc(df, p, t)\n\nCalculate non-centrality parameter for non-central t distribution.\n\nSee `nctdtr` for more details.\n\nParameters\n----------\ndf : array_like\n    Degrees of freedom of the distribution.  Should be in range (0, inf).\np : array_like\n    CDF values, in range (0, 1].\nt : array_like\n    Quantiles, i.e. the upper limit of integration.");

  /* "scipy/special/_ufuncs.pyx":11642
 *     "t : array_like\n"
 *     "    Quantiles, i.e. the upper limit of integration.")
 * ufunc_nctdtrinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtrinc_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":11643
 *     "    Quantiles, i.e. the upper limit of integration.")
 * ufunc_nctdtrinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nctdtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":11644
 * ufunc_nctdtrinc_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nctdtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtrinc_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11645
 * ufunc_nctdtrinc_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtrinc_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11646
 * ufunc_nctdtrinc_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11647
 * ufunc_nctdtrinc_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11648
 * ufunc_nctdtrinc_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11649
 * ufunc_nctdtrinc_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtrinc_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11650
 * ufunc_nctdtrinc_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_ptr[2*0] = <void*>_func_cdftnc4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11651
 * ufunc_nctdtrinc_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_ptr[2*0] = <void*>_func_cdftnc4_wrap
 * ufunc_nctdtrinc_ptr[2*0+1] = <void*>(<char*>"nctdtrinc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11652
 * ufunc_nctdtrinc_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_ptr[2*0] = <void*>_func_cdftnc4_wrap             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_ptr[2*0+1] = <void*>(<char*>"nctdtrinc")
 * ufunc_nctdtrinc_ptr[2*1] = <void*>_func_cdftnc4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_ptr[0]) = ((void *)cdftnc4_wrap);

  /* "scipy/special/_ufuncs.pyx":11653
 * ufunc_nctdtrinc_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtrinc_ptr[2*0] = <void*>_func_cdftnc4_wrap
 * ufunc_nctdtrinc_ptr[2*0+1] = <void*>(<char*>"nctdtrinc")             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_ptr[2*1] = <void*>_func_cdftnc4_wrap
 * ufunc_nctdtrinc_ptr[2*1+1] = <void*>(<char*>"nctdtrinc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_ptr[1]) = ((void *)((char *)((char *)"nctdtrinc")));

  /* "scipy/special/_ufuncs.pyx":11654
 * ufunc_nctdtrinc_ptr[2*0] = <void*>_func_cdftnc4_wrap
 * ufunc_nctdtrinc_ptr[2*0+1] = <void*>(<char*>"nctdtrinc")
 * ufunc_nctdtrinc_ptr[2*1] = <void*>_func_cdftnc4_wrap             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_ptr[2*1+1] = <void*>(<char*>"nctdtrinc")
 * ufunc_nctdtrinc_data[0] = &ufunc_nctdtrinc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_ptr[2]) = ((void *)cdftnc4_wrap);

  /* "scipy/special/_ufuncs.pyx":11655
 * ufunc_nctdtrinc_ptr[2*0+1] = <void*>(<char*>"nctdtrinc")
 * ufunc_nctdtrinc_ptr[2*1] = <void*>_func_cdftnc4_wrap
 * ufunc_nctdtrinc_ptr[2*1+1] = <void*>(<char*>"nctdtrinc")             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_data[0] = &ufunc_nctdtrinc_ptr[2*0]
 * ufunc_nctdtrinc_data[1] = &ufunc_nctdtrinc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_ptr[3]) = ((void *)((char *)((char *)"nctdtrinc")));

  /* "scipy/special/_ufuncs.pyx":11656
 * ufunc_nctdtrinc_ptr[2*1] = <void*>_func_cdftnc4_wrap
 * ufunc_nctdtrinc_ptr[2*1+1] = <void*>(<char*>"nctdtrinc")
 * ufunc_nctdtrinc_data[0] = &ufunc_nctdtrinc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nctdtrinc_data[1] = &ufunc_nctdtrinc_ptr[2*1]
 * nctdtrinc = np.PyUFunc_FromFuncAndData(ufunc_nctdtrinc_loops, ufunc_nctdtrinc_data, ufunc_nctdtrinc_types, 2, 3, 1, 0, "nctdtrinc", ufunc_nctdtrinc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":11657
 * ufunc_nctdtrinc_ptr[2*1+1] = <void*>(<char*>"nctdtrinc")
 * ufunc_nctdtrinc_data[0] = &ufunc_nctdtrinc_ptr[2*0]
 * ufunc_nctdtrinc_data[1] = &ufunc_nctdtrinc_ptr[2*1]             # <<<<<<<<<<<<<<
 * nctdtrinc = np.PyUFunc_FromFuncAndData(ufunc_nctdtrinc_loops, ufunc_nctdtrinc_data, ufunc_nctdtrinc_types, 2, 3, 1, 0, "nctdtrinc", ufunc_nctdtrinc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":11658
 * ufunc_nctdtrinc_data[0] = &ufunc_nctdtrinc_ptr[2*0]
 * ufunc_nctdtrinc_data[1] = &ufunc_nctdtrinc_ptr[2*1]
 * nctdtrinc = np.PyUFunc_FromFuncAndData(ufunc_nctdtrinc_loops, ufunc_nctdtrinc_data, ufunc_nctdtrinc_types, 2, 3, 1, 0, "nctdtrinc", ufunc_nctdtrinc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nctdtrit_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_types, 2, 3, 1, 0, ((char *)"nctdtrinc"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrinc_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11658, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nctdtrinc, __pyx_t_2) < 0) __PYX_ERR(1, 11658, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":11664
 * cdef void *ufunc_nctdtrit_data[2]
 * cdef char ufunc_nctdtrit_types[8]
 * cdef char *ufunc_nctdtrit_doc = (             # <<<<<<<<<<<<<<
 *     "nctdtrit(df, nc, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_doc = ((char *)"nctdtrit(df, nc, p)\n\nInverse cumulative distribution function of the non-central t distribution.\n\nSee `nctdtr` for more details.\n\nParameters\n----------\ndf : array_like\n    Degrees of freedom of the distribution.  Should be in range (0, inf).\nnc : array_like\n    Noncentrality parameter.  Should be in range (-1e6, 1e6).\np : array_like\n    CDF values, in range (0, 1].");

  /* "scipy/special/_ufuncs.pyx":11679
 *     "p : array_like\n"
 *     "    CDF values, in range (0, 1].")
 * ufunc_nctdtrit_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtrit_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":11680
 *     "    CDF values, in range (0, 1].")
 * ufunc_nctdtrit_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nctdtrit_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":11681
 * ufunc_nctdtrit_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nctdtrit_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtrit_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11682
 * ufunc_nctdtrit_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nctdtrit_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11683
 * ufunc_nctdtrit_types[0] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11684
 * ufunc_nctdtrit_types[1] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11685
 * ufunc_nctdtrit_types[2] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11686
 * ufunc_nctdtrit_types[3] = <char>NPY_FLOAT
 * ufunc_nctdtrit_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11687
 * ufunc_nctdtrit_types[4] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_ptr[2*0] = <void*>_func_cdftnc2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11688
 * ufunc_nctdtrit_types[5] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_ptr[2*0] = <void*>_func_cdftnc2_wrap
 * ufunc_nctdtrit_ptr[2*0+1] = <void*>(<char*>"nctdtrit")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11689
 * ufunc_nctdtrit_types[6] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_ptr[2*0] = <void*>_func_cdftnc2_wrap             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_ptr[2*0+1] = <void*>(<char*>"nctdtrit")
 * ufunc_nctdtrit_ptr[2*1] = <void*>_func_cdftnc2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_ptr[0]) = ((void *)cdftnc2_wrap);

  /* "scipy/special/_ufuncs.pyx":11690
 * ufunc_nctdtrit_types[7] = <char>NPY_DOUBLE
 * ufunc_nctdtrit_ptr[2*0] = <void*>_func_cdftnc2_wrap
 * ufunc_nctdtrit_ptr[2*0+1] = <void*>(<char*>"nctdtrit")             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_ptr[2*1] = <void*>_func_cdftnc2_wrap
 * ufunc_nctdtrit_ptr[2*1+1] = <void*>(<char*>"nctdtrit")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_ptr[1]) = ((void *)((char *)((char *)"nctdtrit")));

  /* "scipy/special/_ufuncs.pyx":11691
 * ufunc_nctdtrit_ptr[2*0] = <void*>_func_cdftnc2_wrap
 * ufunc_nctdtrit_ptr[2*0+1] = <void*>(<char*>"nctdtrit")
 * ufunc_nctdtrit_ptr[2*1] = <void*>_func_cdftnc2_wrap             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_ptr[2*1+1] = <void*>(<char*>"nctdtrit")
 * ufunc_nctdtrit_data[0] = &ufunc_nctdtrit_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_ptr[2]) = ((void *)cdftnc2_wrap);

  /* "scipy/special/_ufuncs.pyx":11692
 * ufunc_nctdtrit_ptr[2*0+1] = <void*>(<char*>"nctdtrit")
 * ufunc_nctdtrit_ptr[2*1] = <void*>_func_cdftnc2_wrap
 * ufunc_nctdtrit_ptr[2*1+1] = <void*>(<char*>"nctdtrit")             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_data[0] = &ufunc_nctdtrit_ptr[2*0]
 * ufunc_nctdtrit_data[1] = &ufunc_nctdtrit_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_ptr[3]) = ((void *)((char *)((char *)"nctdtrit")));

  /* "scipy/special/_ufuncs.pyx":11693
 * ufunc_nctdtrit_ptr[2*1] = <void*>_func_cdftnc2_wrap
 * ufunc_nctdtrit_ptr[2*1+1] = <void*>(<char*>"nctdtrit")
 * ufunc_nctdtrit_data[0] = &ufunc_nctdtrit_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nctdtrit_data[1] = &ufunc_nctdtrit_ptr[2*1]
 * nctdtrit = np.PyUFunc_FromFuncAndData(ufunc_nctdtrit_loops, ufunc_nctdtrit_data, ufunc_nctdtrit_types, 2, 3, 1, 0, "nctdtrit", ufunc_nctdtrit_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":11694
 * ufunc_nctdtrit_ptr[2*1+1] = <void*>(<char*>"nctdtrit")
 * ufunc_nctdtrit_data[0] = &ufunc_nctdtrit_ptr[2*0]
 * ufunc_nctdtrit_data[1] = &ufunc_nctdtrit_ptr[2*1]             # <<<<<<<<<<<<<<
 * nctdtrit = np.PyUFunc_FromFuncAndData(ufunc_nctdtrit_loops, ufunc_nctdtrit_data, ufunc_nctdtrit_types, 2, 3, 1, 0, "nctdtrit", ufunc_nctdtrit_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":11695
 * ufunc_nctdtrit_data[0] = &ufunc_nctdtrit_ptr[2*0]
 * ufunc_nctdtrit_data[1] = &ufunc_nctdtrit_ptr[2*1]
 * nctdtrit = np.PyUFunc_FromFuncAndData(ufunc_nctdtrit_loops, ufunc_nctdtrit_data, ufunc_nctdtrit_types, 2, 3, 1, 0, "nctdtrit", ufunc_nctdtrit_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ndtr_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_types, 2, 3, 1, 0, ((char *)"nctdtrit"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_nctdtrit_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11695, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nctdtrit, __pyx_t_2) < 0) __PYX_ERR(1, 11695, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":11701
 * cdef void *ufunc_ndtr_data[4]
 * cdef char ufunc_ndtr_types[8]
 * cdef char *ufunc_ndtr_doc = (             # <<<<<<<<<<<<<<
 *     "ndtr(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_doc = ((char *)"ndtr(x)\n\nGaussian cumulative distribution function.\n\nReturns the area under the standard Gaussian probability\ndensity function, integrated from minus infinity to `x`\n\n.. math::\n\n   \\frac{1}{\\sqrt{2\\pi}} \\int_{-\\infty}^x \\exp(-t^2/2) dt\n\nParameters\n----------\nx : array_like, real or complex\n    Argument\n\nReturns\n-------\nndarray\n    The value of the normal CDF evaluated at `x`\n\nSee Also\n--------\nerf\nerfc\nscipy.stats.norm\nlog_ndtr");

  /* "scipy/special/_ufuncs.pyx":11729
 *     "scipy.stats.norm\n"
 *     "log_ndtr")
 * ufunc_ndtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_ndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ndtr_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":11730
 *     "log_ndtr")
 * ufunc_ndtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_ndtr_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_ndtr_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":11731
 * ufunc_ndtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ndtr_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_ndtr_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_ndtr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":11732
 * ufunc_ndtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ndtr_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_ndtr_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_ndtr_types[0] = <char>NPY_FLOAT
 * ufunc_ndtr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":11733
 * ufunc_ndtr_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_ndtr_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_ndtr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ndtr_types[1] = <char>NPY_FLOAT
 * ufunc_ndtr_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11734
 * ufunc_ndtr_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_ndtr_types[0] = <char>NPY_FLOAT
 * ufunc_ndtr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ndtr_types[2] = <char>NPY_DOUBLE
 * ufunc_ndtr_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11735
 * ufunc_ndtr_types[0] = <char>NPY_FLOAT
 * ufunc_ndtr_types[1] = <char>NPY_FLOAT
 * ufunc_ndtr_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ndtr_types[3] = <char>NPY_DOUBLE
 * ufunc_ndtr_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11736
 * ufunc_ndtr_types[1] = <char>NPY_FLOAT
 * ufunc_ndtr_types[2] = <char>NPY_DOUBLE
 * ufunc_ndtr_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ndtr_types[4] = <char>NPY_CFLOAT
 * ufunc_ndtr_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11737
 * ufunc_ndtr_types[2] = <char>NPY_DOUBLE
 * ufunc_ndtr_types[3] = <char>NPY_DOUBLE
 * ufunc_ndtr_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_ndtr_types[5] = <char>NPY_CFLOAT
 * ufunc_ndtr_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":11738
 * ufunc_ndtr_types[3] = <char>NPY_DOUBLE
 * ufunc_ndtr_types[4] = <char>NPY_CFLOAT
 * ufunc_ndtr_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_ndtr_types[6] = <char>NPY_CDOUBLE
 * ufunc_ndtr_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":11739
 * ufunc_ndtr_types[4] = <char>NPY_CFLOAT
 * ufunc_ndtr_types[5] = <char>NPY_CFLOAT
 * ufunc_ndtr_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ndtr_types[7] = <char>NPY_CDOUBLE
 * ufunc_ndtr_ptr[2*0] = <void*>_func_ndtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":11740
 * ufunc_ndtr_types[5] = <char>NPY_CFLOAT
 * ufunc_ndtr_types[6] = <char>NPY_CDOUBLE
 * ufunc_ndtr_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ndtr_ptr[2*0] = <void*>_func_ndtr
 * ufunc_ndtr_ptr[2*0+1] = <void*>(<char*>"ndtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":11741
 * ufunc_ndtr_types[6] = <char>NPY_CDOUBLE
 * ufunc_ndtr_types[7] = <char>NPY_CDOUBLE
 * ufunc_ndtr_ptr[2*0] = <void*>_func_ndtr             # <<<<<<<<<<<<<<
 * ufunc_ndtr_ptr[2*0+1] = <void*>(<char*>"ndtr")
 * ufunc_ndtr_ptr[2*1] = <void*>_func_ndtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_ptr[0]) = ((void *)ndtr);

  /* "scipy/special/_ufuncs.pyx":11742
 * ufunc_ndtr_types[7] = <char>NPY_CDOUBLE
 * ufunc_ndtr_ptr[2*0] = <void*>_func_ndtr
 * ufunc_ndtr_ptr[2*0+1] = <void*>(<char*>"ndtr")             # <<<<<<<<<<<<<<
 * ufunc_ndtr_ptr[2*1] = <void*>_func_ndtr
 * ufunc_ndtr_ptr[2*1+1] = <void*>(<char*>"ndtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_ptr[1]) = ((void *)((char *)((char *)"ndtr")));

  /* "scipy/special/_ufuncs.pyx":11743
 * ufunc_ndtr_ptr[2*0] = <void*>_func_ndtr
 * ufunc_ndtr_ptr[2*0+1] = <void*>(<char*>"ndtr")
 * ufunc_ndtr_ptr[2*1] = <void*>_func_ndtr             # <<<<<<<<<<<<<<
 * ufunc_ndtr_ptr[2*1+1] = <void*>(<char*>"ndtr")
 * ufunc_ndtr_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_ndtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_ptr[2]) = ((void *)ndtr);

  /* "scipy/special/_ufuncs.pyx":11744
 * ufunc_ndtr_ptr[2*0+1] = <void*>(<char*>"ndtr")
 * ufunc_ndtr_ptr[2*1] = <void*>_func_ndtr
 * ufunc_ndtr_ptr[2*1+1] = <void*>(<char*>"ndtr")             # <<<<<<<<<<<<<<
 * ufunc_ndtr_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_ndtr
 * ufunc_ndtr_ptr[2*2+1] = <void*>(<char*>"ndtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_ptr[3]) = ((void *)((char *)((char *)"ndtr")));

  /* "scipy/special/_ufuncs.pyx":11745
 * ufunc_ndtr_ptr[2*1] = <void*>_func_ndtr
 * ufunc_ndtr_ptr[2*1+1] = <void*>(<char*>"ndtr")
 * ufunc_ndtr_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_ndtr             # <<<<<<<<<<<<<<
 * ufunc_ndtr_ptr[2*2+1] = <void*>(<char*>"ndtr")
 * ufunc_ndtr_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_ndtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_ptr[4]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_ndtr);

  /* "scipy/special/_ufuncs.pyx":11746
 * ufunc_ndtr_ptr[2*1+1] = <void*>(<char*>"ndtr")
 * ufunc_ndtr_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_ndtr
 * ufunc_ndtr_ptr[2*2+1] = <void*>(<char*>"ndtr")             # <<<<<<<<<<<<<<
 * ufunc_ndtr_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_ndtr
 * ufunc_ndtr_ptr[2*3+1] = <void*>(<char*>"ndtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_ptr[5]) = ((void *)((char *)((char *)"ndtr")));

  /* "scipy/special/_ufuncs.pyx":11747
 * ufunc_ndtr_ptr[2*2] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_ndtr
 * ufunc_ndtr_ptr[2*2+1] = <void*>(<char*>"ndtr")
 * ufunc_ndtr_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_ndtr             # <<<<<<<<<<<<<<
 * ufunc_ndtr_ptr[2*3+1] = <void*>(<char*>"ndtr")
 * ufunc_ndtr_data[0] = &ufunc_ndtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_ptr[6]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_ndtr);

  /* "scipy/special/_ufuncs.pyx":11748
 * ufunc_ndtr_ptr[2*2+1] = <void*>(<char*>"ndtr")
 * ufunc_ndtr_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_ndtr
 * ufunc_ndtr_ptr[2*3+1] = <void*>(<char*>"ndtr")             # <<<<<<<<<<<<<<
 * ufunc_ndtr_data[0] = &ufunc_ndtr_ptr[2*0]
 * ufunc_ndtr_data[1] = &ufunc_ndtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_ptr[7]) = ((void *)((char *)((char *)"ndtr")));

  /* "scipy/special/_ufuncs.pyx":11749
 * ufunc_ndtr_ptr[2*3] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_ndtr
 * ufunc_ndtr_ptr[2*3+1] = <void*>(<char*>"ndtr")
 * ufunc_ndtr_data[0] = &ufunc_ndtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ndtr_data[1] = &ufunc_ndtr_ptr[2*1]
 * ufunc_ndtr_data[2] = &ufunc_ndtr_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":11750
 * ufunc_ndtr_ptr[2*3+1] = <void*>(<char*>"ndtr")
 * ufunc_ndtr_data[0] = &ufunc_ndtr_ptr[2*0]
 * ufunc_ndtr_data[1] = &ufunc_ndtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_ndtr_data[2] = &ufunc_ndtr_ptr[2*2]
 * ufunc_ndtr_data[3] = &ufunc_ndtr_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":11751
 * ufunc_ndtr_data[0] = &ufunc_ndtr_ptr[2*0]
 * ufunc_ndtr_data[1] = &ufunc_ndtr_ptr[2*1]
 * ufunc_ndtr_data[2] = &ufunc_ndtr_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_ndtr_data[3] = &ufunc_ndtr_ptr[2*3]
 * ndtr = np.PyUFunc_FromFuncAndData(ufunc_ndtr_loops, ufunc_ndtr_data, ufunc_ndtr_types, 4, 1, 1, 0, "ndtr", ufunc_ndtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":11752
 * ufunc_ndtr_data[1] = &ufunc_ndtr_ptr[2*1]
 * ufunc_ndtr_data[2] = &ufunc_ndtr_ptr[2*2]
 * ufunc_ndtr_data[3] = &ufunc_ndtr_ptr[2*3]             # <<<<<<<<<<<<<<
 * ndtr = np.PyUFunc_FromFuncAndData(ufunc_ndtr_loops, ufunc_ndtr_data, ufunc_ndtr_types, 4, 1, 1, 0, "ndtr", ufunc_ndtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":11753
 * ufunc_ndtr_data[2] = &ufunc_ndtr_ptr[2*2]
 * ufunc_ndtr_data[3] = &ufunc_ndtr_ptr[2*3]
 * ndtr = np.PyUFunc_FromFuncAndData(ufunc_ndtr_loops, ufunc_ndtr_data, ufunc_ndtr_types, 4, 1, 1, 0, "ndtr", ufunc_ndtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_ndtri_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_types, 4, 1, 1, 0, ((char *)"ndtr"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtr_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11753, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ndtr, __pyx_t_2) < 0) __PYX_ERR(1, 11753, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":11759
 * cdef void *ufunc_ndtri_data[2]
 * cdef char ufunc_ndtri_types[4]
 * cdef char *ufunc_ndtri_doc = (             # <<<<<<<<<<<<<<
 *     "ndtri(y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_doc = ((char *)"ndtri(y)\n\nInverse of `ndtr` vs x\n\nReturns the argument x for which the area under the Gaussian\nprobability density function (integrated from minus infinity to `x`)\nis equal to y.");

  /* "scipy/special/_ufuncs.pyx":11767
 *     "probability density function (integrated from minus infinity to `x`)\n"
 *     "is equal to y.")
 * ufunc_ndtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_ndtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ndtri_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":11768
 *     "is equal to y.")
 * ufunc_ndtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ndtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_ndtri_types[0] = <char>NPY_FLOAT
 * ufunc_ndtri_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":11769
 * ufunc_ndtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_ndtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ndtri_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ndtri_types[1] = <char>NPY_FLOAT
 * ufunc_ndtri_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11770
 * ufunc_ndtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_ndtri_types[0] = <char>NPY_FLOAT
 * ufunc_ndtri_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_ndtri_types[2] = <char>NPY_DOUBLE
 * ufunc_ndtri_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11771
 * ufunc_ndtri_types[0] = <char>NPY_FLOAT
 * ufunc_ndtri_types[1] = <char>NPY_FLOAT
 * ufunc_ndtri_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ndtri_types[3] = <char>NPY_DOUBLE
 * ufunc_ndtri_ptr[2*0] = <void*>_func_ndtri
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11772
 * ufunc_ndtri_types[1] = <char>NPY_FLOAT
 * ufunc_ndtri_types[2] = <char>NPY_DOUBLE
 * ufunc_ndtri_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_ndtri_ptr[2*0] = <void*>_func_ndtri
 * ufunc_ndtri_ptr[2*0+1] = <void*>(<char*>"ndtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11773
 * ufunc_ndtri_types[2] = <char>NPY_DOUBLE
 * ufunc_ndtri_types[3] = <char>NPY_DOUBLE
 * ufunc_ndtri_ptr[2*0] = <void*>_func_ndtri             # <<<<<<<<<<<<<<
 * ufunc_ndtri_ptr[2*0+1] = <void*>(<char*>"ndtri")
 * ufunc_ndtri_ptr[2*1] = <void*>_func_ndtri
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_ptr[0]) = ((void *)ndtri);

  /* "scipy/special/_ufuncs.pyx":11774
 * ufunc_ndtri_types[3] = <char>NPY_DOUBLE
 * ufunc_ndtri_ptr[2*0] = <void*>_func_ndtri
 * ufunc_ndtri_ptr[2*0+1] = <void*>(<char*>"ndtri")             # <<<<<<<<<<<<<<
 * ufunc_ndtri_ptr[2*1] = <void*>_func_ndtri
 * ufunc_ndtri_ptr[2*1+1] = <void*>(<char*>"ndtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_ptr[1]) = ((void *)((char *)((char *)"ndtri")));

  /* "scipy/special/_ufuncs.pyx":11775
 * ufunc_ndtri_ptr[2*0] = <void*>_func_ndtri
 * ufunc_ndtri_ptr[2*0+1] = <void*>(<char*>"ndtri")
 * ufunc_ndtri_ptr[2*1] = <void*>_func_ndtri             # <<<<<<<<<<<<<<
 * ufunc_ndtri_ptr[2*1+1] = <void*>(<char*>"ndtri")
 * ufunc_ndtri_data[0] = &ufunc_ndtri_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_ptr[2]) = ((void *)ndtri);

  /* "scipy/special/_ufuncs.pyx":11776
 * ufunc_ndtri_ptr[2*0+1] = <void*>(<char*>"ndtri")
 * ufunc_ndtri_ptr[2*1] = <void*>_func_ndtri
 * ufunc_ndtri_ptr[2*1+1] = <void*>(<char*>"ndtri")             # <<<<<<<<<<<<<<
 * ufunc_ndtri_data[0] = &ufunc_ndtri_ptr[2*0]
 * ufunc_ndtri_data[1] = &ufunc_ndtri_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_ptr[3]) = ((void *)((char *)((char *)"ndtri")));

  /* "scipy/special/_ufuncs.pyx":11777
 * ufunc_ndtri_ptr[2*1] = <void*>_func_ndtri
 * ufunc_ndtri_ptr[2*1+1] = <void*>(<char*>"ndtri")
 * ufunc_ndtri_data[0] = &ufunc_ndtri_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_ndtri_data[1] = &ufunc_ndtri_ptr[2*1]
 * ndtri = np.PyUFunc_FromFuncAndData(ufunc_ndtri_loops, ufunc_ndtri_data, ufunc_ndtri_types, 2, 1, 1, 0, "ndtri", ufunc_ndtri_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":11778
 * ufunc_ndtri_ptr[2*1+1] = <void*>(<char*>"ndtri")
 * ufunc_ndtri_data[0] = &ufunc_ndtri_ptr[2*0]
 * ufunc_ndtri_data[1] = &ufunc_ndtri_ptr[2*1]             # <<<<<<<<<<<<<<
 * ndtri = np.PyUFunc_FromFuncAndData(ufunc_ndtri_loops, ufunc_ndtri_data, ufunc_ndtri_types, 2, 1, 1, 0, "ndtri", ufunc_ndtri_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":11779
 * ufunc_ndtri_data[0] = &ufunc_ndtri_ptr[2*0]
 * ufunc_ndtri_data[1] = &ufunc_ndtri_ptr[2*1]
 * ndtri = np.PyUFunc_FromFuncAndData(ufunc_ndtri_loops, ufunc_ndtri_data, ufunc_ndtri_types, 2, 1, 1, 0, "ndtri", ufunc_ndtri_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nrdtrimn_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_types, 2, 1, 1, 0, ((char *)"ndtri"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_ndtri_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11779, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ndtri, __pyx_t_2) < 0) __PYX_ERR(1, 11779, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":11785
 * cdef void *ufunc_nrdtrimn_data[2]
 * cdef char ufunc_nrdtrimn_types[8]
 * cdef char *ufunc_nrdtrimn_doc = (             # <<<<<<<<<<<<<<
 *     "nrdtrimn(p, x, std)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_doc = ((char *)"nrdtrimn(p, x, std)\n\nCalculate mean of normal distribution given other params.\n\nParameters\n----------\np : array_like\n    CDF values, in range (0, 1].\nx : array_like\n    Quantiles, i.e. the upper limit of integration.\nstd : array_like\n    Standard deviation.\n\nReturns\n-------\nmn : float or ndarray\n    The mean of the normal distribution.\n\nSee Also\n--------\nnrdtrimn, ndtr");

  /* "scipy/special/_ufuncs.pyx":11807
 *     "--------\n"
 *     "nrdtrimn, ndtr")
 * ufunc_nrdtrimn_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nrdtrimn_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":11808
 *     "nrdtrimn, ndtr")
 * ufunc_nrdtrimn_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nrdtrimn_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_types[0] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":11809
 * ufunc_nrdtrimn_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nrdtrimn_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nrdtrimn_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_types[1] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11810
 * ufunc_nrdtrimn_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nrdtrimn_types[0] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_types[2] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11811
 * ufunc_nrdtrimn_types[0] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[1] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_types[3] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11812
 * ufunc_nrdtrimn_types[1] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[2] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_types[4] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11813
 * ufunc_nrdtrimn_types[2] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[3] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_types[5] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11814
 * ufunc_nrdtrimn_types[3] = <char>NPY_FLOAT
 * ufunc_nrdtrimn_types[4] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_types[6] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11815
 * ufunc_nrdtrimn_types[4] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_types[5] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_types[7] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_ptr[2*0] = <void*>_func_cdfnor3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11816
 * ufunc_nrdtrimn_types[5] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_types[6] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_ptr[2*0] = <void*>_func_cdfnor3_wrap
 * ufunc_nrdtrimn_ptr[2*0+1] = <void*>(<char*>"nrdtrimn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11817
 * ufunc_nrdtrimn_types[6] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_types[7] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_ptr[2*0] = <void*>_func_cdfnor3_wrap             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_ptr[2*0+1] = <void*>(<char*>"nrdtrimn")
 * ufunc_nrdtrimn_ptr[2*1] = <void*>_func_cdfnor3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_ptr[0]) = ((void *)cdfnor3_wrap);

  /* "scipy/special/_ufuncs.pyx":11818
 * ufunc_nrdtrimn_types[7] = <char>NPY_DOUBLE
 * ufunc_nrdtrimn_ptr[2*0] = <void*>_func_cdfnor3_wrap
 * ufunc_nrdtrimn_ptr[2*0+1] = <void*>(<char*>"nrdtrimn")             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_ptr[2*1] = <void*>_func_cdfnor3_wrap
 * ufunc_nrdtrimn_ptr[2*1+1] = <void*>(<char*>"nrdtrimn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_ptr[1]) = ((void *)((char *)((char *)"nrdtrimn")));

  /* "scipy/special/_ufuncs.pyx":11819
 * ufunc_nrdtrimn_ptr[2*0] = <void*>_func_cdfnor3_wrap
 * ufunc_nrdtrimn_ptr[2*0+1] = <void*>(<char*>"nrdtrimn")
 * ufunc_nrdtrimn_ptr[2*1] = <void*>_func_cdfnor3_wrap             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_ptr[2*1+1] = <void*>(<char*>"nrdtrimn")
 * ufunc_nrdtrimn_data[0] = &ufunc_nrdtrimn_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_ptr[2]) = ((void *)cdfnor3_wrap);

  /* "scipy/special/_ufuncs.pyx":11820
 * ufunc_nrdtrimn_ptr[2*0+1] = <void*>(<char*>"nrdtrimn")
 * ufunc_nrdtrimn_ptr[2*1] = <void*>_func_cdfnor3_wrap
 * ufunc_nrdtrimn_ptr[2*1+1] = <void*>(<char*>"nrdtrimn")             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_data[0] = &ufunc_nrdtrimn_ptr[2*0]
 * ufunc_nrdtrimn_data[1] = &ufunc_nrdtrimn_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_ptr[3]) = ((void *)((char *)((char *)"nrdtrimn")));

  /* "scipy/special/_ufuncs.pyx":11821
 * ufunc_nrdtrimn_ptr[2*1] = <void*>_func_cdfnor3_wrap
 * ufunc_nrdtrimn_ptr[2*1+1] = <void*>(<char*>"nrdtrimn")
 * ufunc_nrdtrimn_data[0] = &ufunc_nrdtrimn_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nrdtrimn_data[1] = &ufunc_nrdtrimn_ptr[2*1]
 * nrdtrimn = np.PyUFunc_FromFuncAndData(ufunc_nrdtrimn_loops, ufunc_nrdtrimn_data, ufunc_nrdtrimn_types, 2, 3, 1, 0, "nrdtrimn", ufunc_nrdtrimn_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":11822
 * ufunc_nrdtrimn_ptr[2*1+1] = <void*>(<char*>"nrdtrimn")
 * ufunc_nrdtrimn_data[0] = &ufunc_nrdtrimn_ptr[2*0]
 * ufunc_nrdtrimn_data[1] = &ufunc_nrdtrimn_ptr[2*1]             # <<<<<<<<<<<<<<
 * nrdtrimn = np.PyUFunc_FromFuncAndData(ufunc_nrdtrimn_loops, ufunc_nrdtrimn_data, ufunc_nrdtrimn_types, 2, 3, 1, 0, "nrdtrimn", ufunc_nrdtrimn_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":11823
 * ufunc_nrdtrimn_data[0] = &ufunc_nrdtrimn_ptr[2*0]
 * ufunc_nrdtrimn_data[1] = &ufunc_nrdtrimn_ptr[2*1]
 * nrdtrimn = np.PyUFunc_FromFuncAndData(ufunc_nrdtrimn_loops, ufunc_nrdtrimn_data, ufunc_nrdtrimn_types, 2, 3, 1, 0, "nrdtrimn", ufunc_nrdtrimn_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_nrdtrisd_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_types, 2, 3, 1, 0, ((char *)"nrdtrimn"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrimn_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11823, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nrdtrimn, __pyx_t_2) < 0) __PYX_ERR(1, 11823, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":11829
 * cdef void *ufunc_nrdtrisd_data[2]
 * cdef char ufunc_nrdtrisd_types[8]
 * cdef char *ufunc_nrdtrisd_doc = (             # <<<<<<<<<<<<<<
 *     "nrdtrisd(p, x, mn)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_doc = ((char *)"nrdtrisd(p, x, mn)\n\nCalculate standard deviation of normal distribution given other params.\n\nParameters\n----------\np : array_like\n    CDF values, in range (0, 1].\nx : array_like\n    Quantiles, i.e. the upper limit of integration.\nmn : float or ndarray\n    The mean of the normal distribution.\n\nReturns\n-------\nstd : array_like\n    Standard deviation.\n\nSee Also\n--------\nndtr");

  /* "scipy/special/_ufuncs.pyx":11851
 *     "--------\n"
 *     "ndtr")
 * ufunc_nrdtrisd_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nrdtrisd_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":11852
 *     "ndtr")
 * ufunc_nrdtrisd_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nrdtrisd_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_types[0] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":11853
 * ufunc_nrdtrisd_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_nrdtrisd_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nrdtrisd_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_types[1] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11854
 * ufunc_nrdtrisd_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_nrdtrisd_types[0] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_types[2] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11855
 * ufunc_nrdtrisd_types[0] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[1] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_types[3] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11856
 * ufunc_nrdtrisd_types[1] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[2] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_types[4] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11857
 * ufunc_nrdtrisd_types[2] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[3] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_types[5] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11858
 * ufunc_nrdtrisd_types[3] = <char>NPY_FLOAT
 * ufunc_nrdtrisd_types[4] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_types[6] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11859
 * ufunc_nrdtrisd_types[4] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_types[5] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_types[7] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_ptr[2*0] = <void*>_func_cdfnor4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11860
 * ufunc_nrdtrisd_types[5] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_types[6] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_ptr[2*0] = <void*>_func_cdfnor4_wrap
 * ufunc_nrdtrisd_ptr[2*0+1] = <void*>(<char*>"nrdtrisd")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11861
 * ufunc_nrdtrisd_types[6] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_types[7] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_ptr[2*0] = <void*>_func_cdfnor4_wrap             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_ptr[2*0+1] = <void*>(<char*>"nrdtrisd")
 * ufunc_nrdtrisd_ptr[2*1] = <void*>_func_cdfnor4_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_ptr[0]) = ((void *)cdfnor4_wrap);

  /* "scipy/special/_ufuncs.pyx":11862
 * ufunc_nrdtrisd_types[7] = <char>NPY_DOUBLE
 * ufunc_nrdtrisd_ptr[2*0] = <void*>_func_cdfnor4_wrap
 * ufunc_nrdtrisd_ptr[2*0+1] = <void*>(<char*>"nrdtrisd")             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_ptr[2*1] = <void*>_func_cdfnor4_wrap
 * ufunc_nrdtrisd_ptr[2*1+1] = <void*>(<char*>"nrdtrisd")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_ptr[1]) = ((void *)((char *)((char *)"nrdtrisd")));

  /* "scipy/special/_ufuncs.pyx":11863
 * ufunc_nrdtrisd_ptr[2*0] = <void*>_func_cdfnor4_wrap
 * ufunc_nrdtrisd_ptr[2*0+1] = <void*>(<char*>"nrdtrisd")
 * ufunc_nrdtrisd_ptr[2*1] = <void*>_func_cdfnor4_wrap             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_ptr[2*1+1] = <void*>(<char*>"nrdtrisd")
 * ufunc_nrdtrisd_data[0] = &ufunc_nrdtrisd_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_ptr[2]) = ((void *)cdfnor4_wrap);

  /* "scipy/special/_ufuncs.pyx":11864
 * ufunc_nrdtrisd_ptr[2*0+1] = <void*>(<char*>"nrdtrisd")
 * ufunc_nrdtrisd_ptr[2*1] = <void*>_func_cdfnor4_wrap
 * ufunc_nrdtrisd_ptr[2*1+1] = <void*>(<char*>"nrdtrisd")             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_data[0] = &ufunc_nrdtrisd_ptr[2*0]
 * ufunc_nrdtrisd_data[1] = &ufunc_nrdtrisd_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_ptr[3]) = ((void *)((char *)((char *)"nrdtrisd")));

  /* "scipy/special/_ufuncs.pyx":11865
 * ufunc_nrdtrisd_ptr[2*1] = <void*>_func_cdfnor4_wrap
 * ufunc_nrdtrisd_ptr[2*1+1] = <void*>(<char*>"nrdtrisd")
 * ufunc_nrdtrisd_data[0] = &ufunc_nrdtrisd_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_nrdtrisd_data[1] = &ufunc_nrdtrisd_ptr[2*1]
 * nrdtrisd = np.PyUFunc_FromFuncAndData(ufunc_nrdtrisd_loops, ufunc_nrdtrisd_data, ufunc_nrdtrisd_types, 2, 3, 1, 0, "nrdtrisd", ufunc_nrdtrisd_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":11866
 * ufunc_nrdtrisd_ptr[2*1+1] = <void*>(<char*>"nrdtrisd")
 * ufunc_nrdtrisd_data[0] = &ufunc_nrdtrisd_ptr[2*0]
 * ufunc_nrdtrisd_data[1] = &ufunc_nrdtrisd_ptr[2*1]             # <<<<<<<<<<<<<<
 * nrdtrisd = np.PyUFunc_FromFuncAndData(ufunc_nrdtrisd_loops, ufunc_nrdtrisd_data, ufunc_nrdtrisd_types, 2, 3, 1, 0, "nrdtrisd", ufunc_nrdtrisd_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":11867
 * ufunc_nrdtrisd_data[0] = &ufunc_nrdtrisd_ptr[2*0]
 * ufunc_nrdtrisd_data[1] = &ufunc_nrdtrisd_ptr[2*1]
 * nrdtrisd = np.PyUFunc_FromFuncAndData(ufunc_nrdtrisd_loops, ufunc_nrdtrisd_data, ufunc_nrdtrisd_types, 2, 3, 1, 0, "nrdtrisd", ufunc_nrdtrisd_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_obl_ang1_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_types, 2, 3, 1, 0, ((char *)"nrdtrisd"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_nrdtrisd_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11867, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_nrdtrisd, __pyx_t_2) < 0) __PYX_ERR(1, 11867, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":11873
 * cdef void *ufunc_obl_ang1_data[2]
 * cdef char ufunc_obl_ang1_types[12]
 * cdef char *ufunc_obl_ang1_doc = (             # <<<<<<<<<<<<<<
 *     "obl_ang1(m, n, c, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_doc = ((char *)"obl_ang1(m, n, c, x)\n\nOblate spheroidal angular function of the first kind and its derivative\n\nComputes the oblate spheroidal angular function of the first kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x");

  /* "scipy/special/_ufuncs.pyx":11888
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_obl_ang1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_obl_ang1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff);

  /* "scipy/special/_ufuncs.pyx":11889
 *     "    Value of the derivative vs x")
 * ufunc_obl_ang1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_obl_ang1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[0] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd);

  /* "scipy/special/_ufuncs.pyx":11890
 * ufunc_obl_ang1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_obl_ang1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_obl_ang1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[1] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11891
 * ufunc_obl_ang1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_obl_ang1_types[0] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[2] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11892
 * ufunc_obl_ang1_types[0] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[1] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[3] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11893
 * ufunc_obl_ang1_types[1] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[2] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[4] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11894
 * ufunc_obl_ang1_types[2] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[3] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[5] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11895
 * ufunc_obl_ang1_types[3] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[4] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11896
 * ufunc_obl_ang1_types[4] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[5] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11897
 * ufunc_obl_ang1_types[5] = <char>NPY_FLOAT
 * ufunc_obl_ang1_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11898
 * ufunc_obl_ang1_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11899
 * ufunc_obl_ang1_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11900
 * ufunc_obl_ang1_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_ptr[2*0] = <void*>_func_oblate_aswfa_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11901
 * ufunc_obl_ang1_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_ptr[2*0] = <void*>_func_oblate_aswfa_nocv_wrap
 * ufunc_obl_ang1_ptr[2*0+1] = <void*>(<char*>"obl_ang1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11902
 * ufunc_obl_ang1_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_ptr[2*0] = <void*>_func_oblate_aswfa_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_ptr[2*0+1] = <void*>(<char*>"obl_ang1")
 * ufunc_obl_ang1_ptr[2*1] = <void*>_func_oblate_aswfa_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_ptr[0]) = ((void *)oblate_aswfa_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":11903
 * ufunc_obl_ang1_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_ptr[2*0] = <void*>_func_oblate_aswfa_nocv_wrap
 * ufunc_obl_ang1_ptr[2*0+1] = <void*>(<char*>"obl_ang1")             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_ptr[2*1] = <void*>_func_oblate_aswfa_nocv_wrap
 * ufunc_obl_ang1_ptr[2*1+1] = <void*>(<char*>"obl_ang1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_ptr[1]) = ((void *)((char *)((char *)"obl_ang1")));

  /* "scipy/special/_ufuncs.pyx":11904
 * ufunc_obl_ang1_ptr[2*0] = <void*>_func_oblate_aswfa_nocv_wrap
 * ufunc_obl_ang1_ptr[2*0+1] = <void*>(<char*>"obl_ang1")
 * ufunc_obl_ang1_ptr[2*1] = <void*>_func_oblate_aswfa_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_ptr[2*1+1] = <void*>(<char*>"obl_ang1")
 * ufunc_obl_ang1_data[0] = &ufunc_obl_ang1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_ptr[2]) = ((void *)oblate_aswfa_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":11905
 * ufunc_obl_ang1_ptr[2*0+1] = <void*>(<char*>"obl_ang1")
 * ufunc_obl_ang1_ptr[2*1] = <void*>_func_oblate_aswfa_nocv_wrap
 * ufunc_obl_ang1_ptr[2*1+1] = <void*>(<char*>"obl_ang1")             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_data[0] = &ufunc_obl_ang1_ptr[2*0]
 * ufunc_obl_ang1_data[1] = &ufunc_obl_ang1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_ptr[3]) = ((void *)((char *)((char *)"obl_ang1")));

  /* "scipy/special/_ufuncs.pyx":11906
 * ufunc_obl_ang1_ptr[2*1] = <void*>_func_oblate_aswfa_nocv_wrap
 * ufunc_obl_ang1_ptr[2*1+1] = <void*>(<char*>"obl_ang1")
 * ufunc_obl_ang1_data[0] = &ufunc_obl_ang1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_data[1] = &ufunc_obl_ang1_ptr[2*1]
 * obl_ang1 = np.PyUFunc_FromFuncAndData(ufunc_obl_ang1_loops, ufunc_obl_ang1_data, ufunc_obl_ang1_types, 2, 4, 2, 0, "obl_ang1", ufunc_obl_ang1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":11907
 * ufunc_obl_ang1_ptr[2*1+1] = <void*>(<char*>"obl_ang1")
 * ufunc_obl_ang1_data[0] = &ufunc_obl_ang1_ptr[2*0]
 * ufunc_obl_ang1_data[1] = &ufunc_obl_ang1_ptr[2*1]             # <<<<<<<<<<<<<<
 * obl_ang1 = np.PyUFunc_FromFuncAndData(ufunc_obl_ang1_loops, ufunc_obl_ang1_data, ufunc_obl_ang1_types, 2, 4, 2, 0, "obl_ang1", ufunc_obl_ang1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":11908
 * ufunc_obl_ang1_data[0] = &ufunc_obl_ang1_ptr[2*0]
 * ufunc_obl_ang1_data[1] = &ufunc_obl_ang1_ptr[2*1]
 * obl_ang1 = np.PyUFunc_FromFuncAndData(ufunc_obl_ang1_loops, ufunc_obl_ang1_data, ufunc_obl_ang1_types, 2, 4, 2, 0, "obl_ang1", ufunc_obl_ang1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_obl_ang1_cv_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_types, 2, 4, 2, 0, ((char *)"obl_ang1"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11908, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_obl_ang1, __pyx_t_2) < 0) __PYX_ERR(1, 11908, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":11914
 * cdef void *ufunc_obl_ang1_cv_data[2]
 * cdef char ufunc_obl_ang1_cv_types[14]
 * cdef char *ufunc_obl_ang1_cv_doc = (             # <<<<<<<<<<<<<<
 *     "obl_ang1_cv(m, n, c, cv, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_doc = ((char *)"obl_ang1_cv(m, n, c, cv, x)\n\nOblate spheroidal angular function obl_ang1 for precomputed characteristic value\n\nComputes the oblate spheroidal angular function of the first kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``. Requires\npre-computed characteristic value.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x");

  /* "scipy/special/_ufuncs.pyx":11930
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_obl_ang1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_obl_ang1_cv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_fffff_ff);

  /* "scipy/special/_ufuncs.pyx":11931
 *     "    Value of the derivative vs x")
 * ufunc_obl_ang1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_obl_ang1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_ddddd_dd);

  /* "scipy/special/_ufuncs.pyx":11932
 * ufunc_obl_ang1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_obl_ang1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_obl_ang1_cv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11933
 * ufunc_obl_ang1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_obl_ang1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11934
 * ufunc_obl_ang1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11935
 * ufunc_obl_ang1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11936
 * ufunc_obl_ang1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11937
 * ufunc_obl_ang1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11938
 * ufunc_obl_ang1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11939
 * ufunc_obl_ang1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11940
 * ufunc_obl_ang1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_obl_ang1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11941
 * ufunc_obl_ang1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11942
 * ufunc_obl_ang1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11943
 * ufunc_obl_ang1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11944
 * ufunc_obl_ang1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_ptr[2*0] = <void*>_func_oblate_aswfa_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11945
 * ufunc_obl_ang1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_ptr[2*0] = <void*>_func_oblate_aswfa_wrap
 * ufunc_obl_ang1_cv_ptr[2*0+1] = <void*>(<char*>"obl_ang1_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11946
 * ufunc_obl_ang1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_ptr[2*0] = <void*>_func_oblate_aswfa_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_ptr[2*0+1] = <void*>(<char*>"obl_ang1_cv")
 * ufunc_obl_ang1_cv_ptr[2*1] = <void*>_func_oblate_aswfa_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_ptr[0]) = ((void *)oblate_aswfa_wrap);

  /* "scipy/special/_ufuncs.pyx":11947
 * ufunc_obl_ang1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_obl_ang1_cv_ptr[2*0] = <void*>_func_oblate_aswfa_wrap
 * ufunc_obl_ang1_cv_ptr[2*0+1] = <void*>(<char*>"obl_ang1_cv")             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_ptr[2*1] = <void*>_func_oblate_aswfa_wrap
 * ufunc_obl_ang1_cv_ptr[2*1+1] = <void*>(<char*>"obl_ang1_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_ptr[1]) = ((void *)((char *)((char *)"obl_ang1_cv")));

  /* "scipy/special/_ufuncs.pyx":11948
 * ufunc_obl_ang1_cv_ptr[2*0] = <void*>_func_oblate_aswfa_wrap
 * ufunc_obl_ang1_cv_ptr[2*0+1] = <void*>(<char*>"obl_ang1_cv")
 * ufunc_obl_ang1_cv_ptr[2*1] = <void*>_func_oblate_aswfa_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_ptr[2*1+1] = <void*>(<char*>"obl_ang1_cv")
 * ufunc_obl_ang1_cv_data[0] = &ufunc_obl_ang1_cv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_ptr[2]) = ((void *)oblate_aswfa_wrap);

  /* "scipy/special/_ufuncs.pyx":11949
 * ufunc_obl_ang1_cv_ptr[2*0+1] = <void*>(<char*>"obl_ang1_cv")
 * ufunc_obl_ang1_cv_ptr[2*1] = <void*>_func_oblate_aswfa_wrap
 * ufunc_obl_ang1_cv_ptr[2*1+1] = <void*>(<char*>"obl_ang1_cv")             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_data[0] = &ufunc_obl_ang1_cv_ptr[2*0]
 * ufunc_obl_ang1_cv_data[1] = &ufunc_obl_ang1_cv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_ptr[3]) = ((void *)((char *)((char *)"obl_ang1_cv")));

  /* "scipy/special/_ufuncs.pyx":11950
 * ufunc_obl_ang1_cv_ptr[2*1] = <void*>_func_oblate_aswfa_wrap
 * ufunc_obl_ang1_cv_ptr[2*1+1] = <void*>(<char*>"obl_ang1_cv")
 * ufunc_obl_ang1_cv_data[0] = &ufunc_obl_ang1_cv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_obl_ang1_cv_data[1] = &ufunc_obl_ang1_cv_ptr[2*1]
 * obl_ang1_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_ang1_cv_loops, ufunc_obl_ang1_cv_data, ufunc_obl_ang1_cv_types, 2, 5, 2, 0, "obl_ang1_cv", ufunc_obl_ang1_cv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":11951
 * ufunc_obl_ang1_cv_ptr[2*1+1] = <void*>(<char*>"obl_ang1_cv")
 * ufunc_obl_ang1_cv_data[0] = &ufunc_obl_ang1_cv_ptr[2*0]
 * ufunc_obl_ang1_cv_data[1] = &ufunc_obl_ang1_cv_ptr[2*1]             # <<<<<<<<<<<<<<
 * obl_ang1_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_ang1_cv_loops, ufunc_obl_ang1_cv_data, ufunc_obl_ang1_cv_types, 2, 5, 2, 0, "obl_ang1_cv", ufunc_obl_ang1_cv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":11952
 * ufunc_obl_ang1_cv_data[0] = &ufunc_obl_ang1_cv_ptr[2*0]
 * ufunc_obl_ang1_cv_data[1] = &ufunc_obl_ang1_cv_ptr[2*1]
 * obl_ang1_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_ang1_cv_loops, ufunc_obl_ang1_cv_data, ufunc_obl_ang1_cv_types, 2, 5, 2, 0, "obl_ang1_cv", ufunc_obl_ang1_cv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_obl_cv_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_types, 2, 5, 2, 0, ((char *)"obl_ang1_cv"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_ang1_cv_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11952, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_obl_ang1_cv, __pyx_t_2) < 0) __PYX_ERR(1, 11952, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":11958
 * cdef void *ufunc_obl_cv_data[2]
 * cdef char ufunc_obl_cv_types[8]
 * cdef char *ufunc_obl_cv_doc = (             # <<<<<<<<<<<<<<
 *     "obl_cv(m, n, c)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_doc = ((char *)"obl_cv(m, n, c)\n\nCharacteristic value of oblate spheroidal function\n\nComputes the characteristic value of oblate spheroidal wave\nfunctions of order `m`, `n` (n>=m) and spheroidal parameter `c`.");

  /* "scipy/special/_ufuncs.pyx":11965
 *     "Computes the characteristic value of oblate spheroidal wave\n"
 *     "functions of order `m`, `n` (n>=m) and spheroidal parameter `c`.")
 * ufunc_obl_cv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_obl_cv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":11966
 *     "functions of order `m`, `n` (n>=m) and spheroidal parameter `c`.")
 * ufunc_obl_cv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_obl_cv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":11967
 * ufunc_obl_cv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_obl_cv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_obl_cv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11968
 * ufunc_obl_cv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_obl_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11969
 * ufunc_obl_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11970
 * ufunc_obl_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_types[4] = <char>NPY_DOUBLE
 * ufunc_obl_cv_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":11971
 * ufunc_obl_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_types[5] = <char>NPY_DOUBLE
 * ufunc_obl_cv_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11972
 * ufunc_obl_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_cv_types[4] = <char>NPY_DOUBLE
 * ufunc_obl_cv_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_cv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11973
 * ufunc_obl_cv_types[4] = <char>NPY_DOUBLE
 * ufunc_obl_cv_types[5] = <char>NPY_DOUBLE
 * ufunc_obl_cv_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_cv_ptr[2*0] = <void*>_func_oblate_segv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11974
 * ufunc_obl_cv_types[5] = <char>NPY_DOUBLE
 * ufunc_obl_cv_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_cv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_ptr[2*0] = <void*>_func_oblate_segv_wrap
 * ufunc_obl_cv_ptr[2*0+1] = <void*>(<char*>"obl_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":11975
 * ufunc_obl_cv_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_cv_ptr[2*0] = <void*>_func_oblate_segv_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_ptr[2*0+1] = <void*>(<char*>"obl_cv")
 * ufunc_obl_cv_ptr[2*1] = <void*>_func_oblate_segv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_ptr[0]) = ((void *)oblate_segv_wrap);

  /* "scipy/special/_ufuncs.pyx":11976
 * ufunc_obl_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_cv_ptr[2*0] = <void*>_func_oblate_segv_wrap
 * ufunc_obl_cv_ptr[2*0+1] = <void*>(<char*>"obl_cv")             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_ptr[2*1] = <void*>_func_oblate_segv_wrap
 * ufunc_obl_cv_ptr[2*1+1] = <void*>(<char*>"obl_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_ptr[1]) = ((void *)((char *)((char *)"obl_cv")));

  /* "scipy/special/_ufuncs.pyx":11977
 * ufunc_obl_cv_ptr[2*0] = <void*>_func_oblate_segv_wrap
 * ufunc_obl_cv_ptr[2*0+1] = <void*>(<char*>"obl_cv")
 * ufunc_obl_cv_ptr[2*1] = <void*>_func_oblate_segv_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_ptr[2*1+1] = <void*>(<char*>"obl_cv")
 * ufunc_obl_cv_data[0] = &ufunc_obl_cv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_ptr[2]) = ((void *)oblate_segv_wrap);

  /* "scipy/special/_ufuncs.pyx":11978
 * ufunc_obl_cv_ptr[2*0+1] = <void*>(<char*>"obl_cv")
 * ufunc_obl_cv_ptr[2*1] = <void*>_func_oblate_segv_wrap
 * ufunc_obl_cv_ptr[2*1+1] = <void*>(<char*>"obl_cv")             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_data[0] = &ufunc_obl_cv_ptr[2*0]
 * ufunc_obl_cv_data[1] = &ufunc_obl_cv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_ptr[3]) = ((void *)((char *)((char *)"obl_cv")));

  /* "scipy/special/_ufuncs.pyx":11979
 * ufunc_obl_cv_ptr[2*1] = <void*>_func_oblate_segv_wrap
 * ufunc_obl_cv_ptr[2*1+1] = <void*>(<char*>"obl_cv")
 * ufunc_obl_cv_data[0] = &ufunc_obl_cv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_obl_cv_data[1] = &ufunc_obl_cv_ptr[2*1]
 * obl_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_cv_loops, ufunc_obl_cv_data, ufunc_obl_cv_types, 2, 3, 1, 0, "obl_cv", ufunc_obl_cv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":11980
 * ufunc_obl_cv_ptr[2*1+1] = <void*>(<char*>"obl_cv")
 * ufunc_obl_cv_data[0] = &ufunc_obl_cv_ptr[2*0]
 * ufunc_obl_cv_data[1] = &ufunc_obl_cv_ptr[2*1]             # <<<<<<<<<<<<<<
 * obl_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_cv_loops, ufunc_obl_cv_data, ufunc_obl_cv_types, 2, 3, 1, 0, "obl_cv", ufunc_obl_cv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":11981
 * ufunc_obl_cv_data[0] = &ufunc_obl_cv_ptr[2*0]
 * ufunc_obl_cv_data[1] = &ufunc_obl_cv_ptr[2*1]
 * obl_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_cv_loops, ufunc_obl_cv_data, ufunc_obl_cv_types, 2, 3, 1, 0, "obl_cv", ufunc_obl_cv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_obl_rad1_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_types, 2, 3, 1, 0, ((char *)"obl_cv"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_cv_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11981, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_obl_cv, __pyx_t_2) < 0) __PYX_ERR(1, 11981, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":11987
 * cdef void *ufunc_obl_rad1_data[2]
 * cdef char ufunc_obl_rad1_types[12]
 * cdef char *ufunc_obl_rad1_doc = (             # <<<<<<<<<<<<<<
 *     "obl_rad1(m, n, c, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_doc = ((char *)"obl_rad1(m, n, c, x)\n\nOblate spheroidal radial function of the first kind and its derivative\n\nComputes the oblate spheroidal radial function of the first kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x");

  /* "scipy/special/_ufuncs.pyx":12002
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_obl_rad1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_obl_rad1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff);

  /* "scipy/special/_ufuncs.pyx":12003
 *     "    Value of the derivative vs x")
 * ufunc_obl_rad1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_obl_rad1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd);

  /* "scipy/special/_ufuncs.pyx":12004
 * ufunc_obl_rad1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_obl_rad1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_obl_rad1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12005
 * ufunc_obl_rad1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_obl_rad1_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12006
 * ufunc_obl_rad1_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12007
 * ufunc_obl_rad1_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12008
 * ufunc_obl_rad1_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12009
 * ufunc_obl_rad1_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12010
 * ufunc_obl_rad1_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12011
 * ufunc_obl_rad1_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad1_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12012
 * ufunc_obl_rad1_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12013
 * ufunc_obl_rad1_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12014
 * ufunc_obl_rad1_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_ptr[2*0] = <void*>_func_oblate_radial1_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12015
 * ufunc_obl_rad1_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_ptr[2*0] = <void*>_func_oblate_radial1_nocv_wrap
 * ufunc_obl_rad1_ptr[2*0+1] = <void*>(<char*>"obl_rad1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12016
 * ufunc_obl_rad1_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_ptr[2*0] = <void*>_func_oblate_radial1_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_ptr[2*0+1] = <void*>(<char*>"obl_rad1")
 * ufunc_obl_rad1_ptr[2*1] = <void*>_func_oblate_radial1_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_ptr[0]) = ((void *)oblate_radial1_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":12017
 * ufunc_obl_rad1_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_ptr[2*0] = <void*>_func_oblate_radial1_nocv_wrap
 * ufunc_obl_rad1_ptr[2*0+1] = <void*>(<char*>"obl_rad1")             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_ptr[2*1] = <void*>_func_oblate_radial1_nocv_wrap
 * ufunc_obl_rad1_ptr[2*1+1] = <void*>(<char*>"obl_rad1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_ptr[1]) = ((void *)((char *)((char *)"obl_rad1")));

  /* "scipy/special/_ufuncs.pyx":12018
 * ufunc_obl_rad1_ptr[2*0] = <void*>_func_oblate_radial1_nocv_wrap
 * ufunc_obl_rad1_ptr[2*0+1] = <void*>(<char*>"obl_rad1")
 * ufunc_obl_rad1_ptr[2*1] = <void*>_func_oblate_radial1_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_ptr[2*1+1] = <void*>(<char*>"obl_rad1")
 * ufunc_obl_rad1_data[0] = &ufunc_obl_rad1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_ptr[2]) = ((void *)oblate_radial1_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":12019
 * ufunc_obl_rad1_ptr[2*0+1] = <void*>(<char*>"obl_rad1")
 * ufunc_obl_rad1_ptr[2*1] = <void*>_func_oblate_radial1_nocv_wrap
 * ufunc_obl_rad1_ptr[2*1+1] = <void*>(<char*>"obl_rad1")             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_data[0] = &ufunc_obl_rad1_ptr[2*0]
 * ufunc_obl_rad1_data[1] = &ufunc_obl_rad1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_ptr[3]) = ((void *)((char *)((char *)"obl_rad1")));

  /* "scipy/special/_ufuncs.pyx":12020
 * ufunc_obl_rad1_ptr[2*1] = <void*>_func_oblate_radial1_nocv_wrap
 * ufunc_obl_rad1_ptr[2*1+1] = <void*>(<char*>"obl_rad1")
 * ufunc_obl_rad1_data[0] = &ufunc_obl_rad1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_data[1] = &ufunc_obl_rad1_ptr[2*1]
 * obl_rad1 = np.PyUFunc_FromFuncAndData(ufunc_obl_rad1_loops, ufunc_obl_rad1_data, ufunc_obl_rad1_types, 2, 4, 2, 0, "obl_rad1", ufunc_obl_rad1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12021
 * ufunc_obl_rad1_ptr[2*1+1] = <void*>(<char*>"obl_rad1")
 * ufunc_obl_rad1_data[0] = &ufunc_obl_rad1_ptr[2*0]
 * ufunc_obl_rad1_data[1] = &ufunc_obl_rad1_ptr[2*1]             # <<<<<<<<<<<<<<
 * obl_rad1 = np.PyUFunc_FromFuncAndData(ufunc_obl_rad1_loops, ufunc_obl_rad1_data, ufunc_obl_rad1_types, 2, 4, 2, 0, "obl_rad1", ufunc_obl_rad1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12022
 * ufunc_obl_rad1_data[0] = &ufunc_obl_rad1_ptr[2*0]
 * ufunc_obl_rad1_data[1] = &ufunc_obl_rad1_ptr[2*1]
 * obl_rad1 = np.PyUFunc_FromFuncAndData(ufunc_obl_rad1_loops, ufunc_obl_rad1_data, ufunc_obl_rad1_types, 2, 4, 2, 0, "obl_rad1", ufunc_obl_rad1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_obl_rad1_cv_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_types, 2, 4, 2, 0, ((char *)"obl_rad1"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12022, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_obl_rad1, __pyx_t_2) < 0) __PYX_ERR(1, 12022, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12028
 * cdef void *ufunc_obl_rad1_cv_data[2]
 * cdef char ufunc_obl_rad1_cv_types[14]
 * cdef char *ufunc_obl_rad1_cv_doc = (             # <<<<<<<<<<<<<<
 *     "obl_rad1_cv(m, n, c, cv, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_doc = ((char *)"obl_rad1_cv(m, n, c, cv, x)\n\nOblate spheroidal radial function obl_rad1 for precomputed characteristic value\n\nComputes the oblate spheroidal radial function of the first kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``. Requires\npre-computed characteristic value.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x");

  /* "scipy/special/_ufuncs.pyx":12044
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_obl_rad1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_obl_rad1_cv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_fffff_ff);

  /* "scipy/special/_ufuncs.pyx":12045
 *     "    Value of the derivative vs x")
 * ufunc_obl_rad1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_obl_rad1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_ddddd_dd);

  /* "scipy/special/_ufuncs.pyx":12046
 * ufunc_obl_rad1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_obl_rad1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_obl_rad1_cv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12047
 * ufunc_obl_rad1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_obl_rad1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12048
 * ufunc_obl_rad1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12049
 * ufunc_obl_rad1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12050
 * ufunc_obl_rad1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12051
 * ufunc_obl_rad1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12052
 * ufunc_obl_rad1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12053
 * ufunc_obl_rad1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12054
 * ufunc_obl_rad1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_obl_rad1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12055
 * ufunc_obl_rad1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12056
 * ufunc_obl_rad1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12057
 * ufunc_obl_rad1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12058
 * ufunc_obl_rad1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_ptr[2*0] = <void*>_func_oblate_radial1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12059
 * ufunc_obl_rad1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_ptr[2*0] = <void*>_func_oblate_radial1_wrap
 * ufunc_obl_rad1_cv_ptr[2*0+1] = <void*>(<char*>"obl_rad1_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12060
 * ufunc_obl_rad1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_ptr[2*0] = <void*>_func_oblate_radial1_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_ptr[2*0+1] = <void*>(<char*>"obl_rad1_cv")
 * ufunc_obl_rad1_cv_ptr[2*1] = <void*>_func_oblate_radial1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_ptr[0]) = ((void *)oblate_radial1_wrap);

  /* "scipy/special/_ufuncs.pyx":12061
 * ufunc_obl_rad1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_obl_rad1_cv_ptr[2*0] = <void*>_func_oblate_radial1_wrap
 * ufunc_obl_rad1_cv_ptr[2*0+1] = <void*>(<char*>"obl_rad1_cv")             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_ptr[2*1] = <void*>_func_oblate_radial1_wrap
 * ufunc_obl_rad1_cv_ptr[2*1+1] = <void*>(<char*>"obl_rad1_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_ptr[1]) = ((void *)((char *)((char *)"obl_rad1_cv")));

  /* "scipy/special/_ufuncs.pyx":12062
 * ufunc_obl_rad1_cv_ptr[2*0] = <void*>_func_oblate_radial1_wrap
 * ufunc_obl_rad1_cv_ptr[2*0+1] = <void*>(<char*>"obl_rad1_cv")
 * ufunc_obl_rad1_cv_ptr[2*1] = <void*>_func_oblate_radial1_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_ptr[2*1+1] = <void*>(<char*>"obl_rad1_cv")
 * ufunc_obl_rad1_cv_data[0] = &ufunc_obl_rad1_cv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_ptr[2]) = ((void *)oblate_radial1_wrap);

  /* "scipy/special/_ufuncs.pyx":12063
 * ufunc_obl_rad1_cv_ptr[2*0+1] = <void*>(<char*>"obl_rad1_cv")
 * ufunc_obl_rad1_cv_ptr[2*1] = <void*>_func_oblate_radial1_wrap
 * ufunc_obl_rad1_cv_ptr[2*1+1] = <void*>(<char*>"obl_rad1_cv")             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_data[0] = &ufunc_obl_rad1_cv_ptr[2*0]
 * ufunc_obl_rad1_cv_data[1] = &ufunc_obl_rad1_cv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_ptr[3]) = ((void *)((char *)((char *)"obl_rad1_cv")));

  /* "scipy/special/_ufuncs.pyx":12064
 * ufunc_obl_rad1_cv_ptr[2*1] = <void*>_func_oblate_radial1_wrap
 * ufunc_obl_rad1_cv_ptr[2*1+1] = <void*>(<char*>"obl_rad1_cv")
 * ufunc_obl_rad1_cv_data[0] = &ufunc_obl_rad1_cv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_obl_rad1_cv_data[1] = &ufunc_obl_rad1_cv_ptr[2*1]
 * obl_rad1_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_rad1_cv_loops, ufunc_obl_rad1_cv_data, ufunc_obl_rad1_cv_types, 2, 5, 2, 0, "obl_rad1_cv", ufunc_obl_rad1_cv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12065
 * ufunc_obl_rad1_cv_ptr[2*1+1] = <void*>(<char*>"obl_rad1_cv")
 * ufunc_obl_rad1_cv_data[0] = &ufunc_obl_rad1_cv_ptr[2*0]
 * ufunc_obl_rad1_cv_data[1] = &ufunc_obl_rad1_cv_ptr[2*1]             # <<<<<<<<<<<<<<
 * obl_rad1_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_rad1_cv_loops, ufunc_obl_rad1_cv_data, ufunc_obl_rad1_cv_types, 2, 5, 2, 0, "obl_rad1_cv", ufunc_obl_rad1_cv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12066
 * ufunc_obl_rad1_cv_data[0] = &ufunc_obl_rad1_cv_ptr[2*0]
 * ufunc_obl_rad1_cv_data[1] = &ufunc_obl_rad1_cv_ptr[2*1]
 * obl_rad1_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_rad1_cv_loops, ufunc_obl_rad1_cv_data, ufunc_obl_rad1_cv_types, 2, 5, 2, 0, "obl_rad1_cv", ufunc_obl_rad1_cv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_obl_rad2_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_types, 2, 5, 2, 0, ((char *)"obl_rad1_cv"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad1_cv_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12066, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_obl_rad1_cv, __pyx_t_2) < 0) __PYX_ERR(1, 12066, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12072
 * cdef void *ufunc_obl_rad2_data[2]
 * cdef char ufunc_obl_rad2_types[12]
 * cdef char *ufunc_obl_rad2_doc = (             # <<<<<<<<<<<<<<
 *     "obl_rad2(m, n, c, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_doc = ((char *)"obl_rad2(m, n, c, x)\n\nOblate spheroidal radial function of the second kind and its derivative.\n\nComputes the oblate spheroidal radial function of the second kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x");

  /* "scipy/special/_ufuncs.pyx":12087
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_obl_rad2_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_obl_rad2_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff);

  /* "scipy/special/_ufuncs.pyx":12088
 *     "    Value of the derivative vs x")
 * ufunc_obl_rad2_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_obl_rad2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd);

  /* "scipy/special/_ufuncs.pyx":12089
 * ufunc_obl_rad2_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_obl_rad2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_obl_rad2_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12090
 * ufunc_obl_rad2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_obl_rad2_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12091
 * ufunc_obl_rad2_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12092
 * ufunc_obl_rad2_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12093
 * ufunc_obl_rad2_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12094
 * ufunc_obl_rad2_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12095
 * ufunc_obl_rad2_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12096
 * ufunc_obl_rad2_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad2_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12097
 * ufunc_obl_rad2_types[6] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12098
 * ufunc_obl_rad2_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12099
 * ufunc_obl_rad2_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_ptr[2*0] = <void*>_func_oblate_radial2_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12100
 * ufunc_obl_rad2_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_ptr[2*0] = <void*>_func_oblate_radial2_nocv_wrap
 * ufunc_obl_rad2_ptr[2*0+1] = <void*>(<char*>"obl_rad2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12101
 * ufunc_obl_rad2_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_ptr[2*0] = <void*>_func_oblate_radial2_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_ptr[2*0+1] = <void*>(<char*>"obl_rad2")
 * ufunc_obl_rad2_ptr[2*1] = <void*>_func_oblate_radial2_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_ptr[0]) = ((void *)oblate_radial2_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":12102
 * ufunc_obl_rad2_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_ptr[2*0] = <void*>_func_oblate_radial2_nocv_wrap
 * ufunc_obl_rad2_ptr[2*0+1] = <void*>(<char*>"obl_rad2")             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_ptr[2*1] = <void*>_func_oblate_radial2_nocv_wrap
 * ufunc_obl_rad2_ptr[2*1+1] = <void*>(<char*>"obl_rad2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_ptr[1]) = ((void *)((char *)((char *)"obl_rad2")));

  /* "scipy/special/_ufuncs.pyx":12103
 * ufunc_obl_rad2_ptr[2*0] = <void*>_func_oblate_radial2_nocv_wrap
 * ufunc_obl_rad2_ptr[2*0+1] = <void*>(<char*>"obl_rad2")
 * ufunc_obl_rad2_ptr[2*1] = <void*>_func_oblate_radial2_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_ptr[2*1+1] = <void*>(<char*>"obl_rad2")
 * ufunc_obl_rad2_data[0] = &ufunc_obl_rad2_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_ptr[2]) = ((void *)oblate_radial2_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":12104
 * ufunc_obl_rad2_ptr[2*0+1] = <void*>(<char*>"obl_rad2")
 * ufunc_obl_rad2_ptr[2*1] = <void*>_func_oblate_radial2_nocv_wrap
 * ufunc_obl_rad2_ptr[2*1+1] = <void*>(<char*>"obl_rad2")             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_data[0] = &ufunc_obl_rad2_ptr[2*0]
 * ufunc_obl_rad2_data[1] = &ufunc_obl_rad2_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_ptr[3]) = ((void *)((char *)((char *)"obl_rad2")));

  /* "scipy/special/_ufuncs.pyx":12105
 * ufunc_obl_rad2_ptr[2*1] = <void*>_func_oblate_radial2_nocv_wrap
 * ufunc_obl_rad2_ptr[2*1+1] = <void*>(<char*>"obl_rad2")
 * ufunc_obl_rad2_data[0] = &ufunc_obl_rad2_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_data[1] = &ufunc_obl_rad2_ptr[2*1]
 * obl_rad2 = np.PyUFunc_FromFuncAndData(ufunc_obl_rad2_loops, ufunc_obl_rad2_data, ufunc_obl_rad2_types, 2, 4, 2, 0, "obl_rad2", ufunc_obl_rad2_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12106
 * ufunc_obl_rad2_ptr[2*1+1] = <void*>(<char*>"obl_rad2")
 * ufunc_obl_rad2_data[0] = &ufunc_obl_rad2_ptr[2*0]
 * ufunc_obl_rad2_data[1] = &ufunc_obl_rad2_ptr[2*1]             # <<<<<<<<<<<<<<
 * obl_rad2 = np.PyUFunc_FromFuncAndData(ufunc_obl_rad2_loops, ufunc_obl_rad2_data, ufunc_obl_rad2_types, 2, 4, 2, 0, "obl_rad2", ufunc_obl_rad2_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12107
 * ufunc_obl_rad2_data[0] = &ufunc_obl_rad2_ptr[2*0]
 * ufunc_obl_rad2_data[1] = &ufunc_obl_rad2_ptr[2*1]
 * obl_rad2 = np.PyUFunc_FromFuncAndData(ufunc_obl_rad2_loops, ufunc_obl_rad2_data, ufunc_obl_rad2_types, 2, 4, 2, 0, "obl_rad2", ufunc_obl_rad2_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_obl_rad2_cv_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_types, 2, 4, 2, 0, ((char *)"obl_rad2"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_obl_rad2, __pyx_t_2) < 0) __PYX_ERR(1, 12107, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12113
 * cdef void *ufunc_obl_rad2_cv_data[2]
 * cdef char ufunc_obl_rad2_cv_types[14]
 * cdef char *ufunc_obl_rad2_cv_doc = (             # <<<<<<<<<<<<<<
 *     "obl_rad2_cv(m, n, c, cv, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_doc = ((char *)"obl_rad2_cv(m, n, c, cv, x)\n\nOblate spheroidal radial function obl_rad2 for precomputed characteristic value\n\nComputes the oblate spheroidal radial function of the second kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``. Requires\npre-computed characteristic value.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x");

  /* "scipy/special/_ufuncs.pyx":12129
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_obl_rad2_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_obl_rad2_cv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_fffff_ff);

  /* "scipy/special/_ufuncs.pyx":12130
 *     "    Value of the derivative vs x")
 * ufunc_obl_rad2_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_obl_rad2_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_ddddd_dd);

  /* "scipy/special/_ufuncs.pyx":12131
 * ufunc_obl_rad2_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_obl_rad2_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_obl_rad2_cv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12132
 * ufunc_obl_rad2_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_obl_rad2_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12133
 * ufunc_obl_rad2_cv_types[0] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12134
 * ufunc_obl_rad2_cv_types[1] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12135
 * ufunc_obl_rad2_cv_types[2] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12136
 * ufunc_obl_rad2_cv_types[3] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[6] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12137
 * ufunc_obl_rad2_cv_types[4] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12138
 * ufunc_obl_rad2_cv_types[5] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[6] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12139
 * ufunc_obl_rad2_cv_types[6] = <char>NPY_FLOAT
 * ufunc_obl_rad2_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12140
 * ufunc_obl_rad2_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12141
 * ufunc_obl_rad2_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12142
 * ufunc_obl_rad2_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12143
 * ufunc_obl_rad2_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_ptr[2*0] = <void*>_func_oblate_radial2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12144
 * ufunc_obl_rad2_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_ptr[2*0] = <void*>_func_oblate_radial2_wrap
 * ufunc_obl_rad2_cv_ptr[2*0+1] = <void*>(<char*>"obl_rad2_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12145
 * ufunc_obl_rad2_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_ptr[2*0] = <void*>_func_oblate_radial2_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_ptr[2*0+1] = <void*>(<char*>"obl_rad2_cv")
 * ufunc_obl_rad2_cv_ptr[2*1] = <void*>_func_oblate_radial2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_ptr[0]) = ((void *)oblate_radial2_wrap);

  /* "scipy/special/_ufuncs.pyx":12146
 * ufunc_obl_rad2_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_obl_rad2_cv_ptr[2*0] = <void*>_func_oblate_radial2_wrap
 * ufunc_obl_rad2_cv_ptr[2*0+1] = <void*>(<char*>"obl_rad2_cv")             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_ptr[2*1] = <void*>_func_oblate_radial2_wrap
 * ufunc_obl_rad2_cv_ptr[2*1+1] = <void*>(<char*>"obl_rad2_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_ptr[1]) = ((void *)((char *)((char *)"obl_rad2_cv")));

  /* "scipy/special/_ufuncs.pyx":12147
 * ufunc_obl_rad2_cv_ptr[2*0] = <void*>_func_oblate_radial2_wrap
 * ufunc_obl_rad2_cv_ptr[2*0+1] = <void*>(<char*>"obl_rad2_cv")
 * ufunc_obl_rad2_cv_ptr[2*1] = <void*>_func_oblate_radial2_wrap             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_ptr[2*1+1] = <void*>(<char*>"obl_rad2_cv")
 * ufunc_obl_rad2_cv_data[0] = &ufunc_obl_rad2_cv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_ptr[2]) = ((void *)oblate_radial2_wrap);

  /* "scipy/special/_ufuncs.pyx":12148
 * ufunc_obl_rad2_cv_ptr[2*0+1] = <void*>(<char*>"obl_rad2_cv")
 * ufunc_obl_rad2_cv_ptr[2*1] = <void*>_func_oblate_radial2_wrap
 * ufunc_obl_rad2_cv_ptr[2*1+1] = <void*>(<char*>"obl_rad2_cv")             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_data[0] = &ufunc_obl_rad2_cv_ptr[2*0]
 * ufunc_obl_rad2_cv_data[1] = &ufunc_obl_rad2_cv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_ptr[3]) = ((void *)((char *)((char *)"obl_rad2_cv")));

  /* "scipy/special/_ufuncs.pyx":12149
 * ufunc_obl_rad2_cv_ptr[2*1] = <void*>_func_oblate_radial2_wrap
 * ufunc_obl_rad2_cv_ptr[2*1+1] = <void*>(<char*>"obl_rad2_cv")
 * ufunc_obl_rad2_cv_data[0] = &ufunc_obl_rad2_cv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_obl_rad2_cv_data[1] = &ufunc_obl_rad2_cv_ptr[2*1]
 * obl_rad2_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_rad2_cv_loops, ufunc_obl_rad2_cv_data, ufunc_obl_rad2_cv_types, 2, 5, 2, 0, "obl_rad2_cv", ufunc_obl_rad2_cv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12150
 * ufunc_obl_rad2_cv_ptr[2*1+1] = <void*>(<char*>"obl_rad2_cv")
 * ufunc_obl_rad2_cv_data[0] = &ufunc_obl_rad2_cv_ptr[2*0]
 * ufunc_obl_rad2_cv_data[1] = &ufunc_obl_rad2_cv_ptr[2*1]             # <<<<<<<<<<<<<<
 * obl_rad2_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_rad2_cv_loops, ufunc_obl_rad2_cv_data, ufunc_obl_rad2_cv_types, 2, 5, 2, 0, "obl_rad2_cv", ufunc_obl_rad2_cv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12151
 * ufunc_obl_rad2_cv_data[0] = &ufunc_obl_rad2_cv_ptr[2*0]
 * ufunc_obl_rad2_cv_data[1] = &ufunc_obl_rad2_cv_ptr[2*1]
 * obl_rad2_cv = np.PyUFunc_FromFuncAndData(ufunc_obl_rad2_cv_loops, ufunc_obl_rad2_cv_data, ufunc_obl_rad2_cv_types, 2, 5, 2, 0, "obl_rad2_cv", ufunc_obl_rad2_cv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_owens_t_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_types, 2, 5, 2, 0, ((char *)"obl_rad2_cv"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_obl_rad2_cv_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_obl_rad2_cv, __pyx_t_2) < 0) __PYX_ERR(1, 12151, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12157
 * cdef void *ufunc_owens_t_data[2]
 * cdef char ufunc_owens_t_types[6]
 * cdef char *ufunc_owens_t_doc = (             # <<<<<<<<<<<<<<
 *     "owens_t(h, a)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_doc = ((char *)"owens_t(h, a)\n\nOwen's T Function.\n\nThe function T(h, a) gives the probability of the event\n(X > h and 0 < Y < a * X) where X and Y are independent\nstandard normal random variables.\n\nParameters\n----------\nh: array_like\n    Input value.\na: array_like\n    Input value.\n\nReturns\n-------\nt: scalar or ndarray\n    Probability of the event (X > h and 0 < Y < a * X),\n    where X and Y are independent standard normal random variables.\n\nExamples\n--------\n>>> from scipy import special\n>>> a = 3.5\n>>> h = 0.78\n>>> special.owens_t(h, a)\n0.10877216734852274\n\nReferences\n----------\n.. [1] M. Patefield and D. Tandy, \"Fast and accurate calculation of\n       Owen's T Function\", Statistical Software vol. 5, pp. 1-25, 2000.");

  /* "scipy/special/_ufuncs.pyx":12191
 *     ".. [1] M. Patefield and D. Tandy, \"Fast and accurate calculation of\n"
 *     "       Owen's T Function\", Statistical Software vol. 5, pp. 1-25, 2000.")
 * ufunc_owens_t_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_owens_t_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_owens_t_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":12192
 *     "       Owen's T Function\", Statistical Software vol. 5, pp. 1-25, 2000.")
 * ufunc_owens_t_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_owens_t_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_owens_t_types[0] = <char>NPY_FLOAT
 * ufunc_owens_t_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":12193
 * ufunc_owens_t_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_owens_t_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_owens_t_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_owens_t_types[1] = <char>NPY_FLOAT
 * ufunc_owens_t_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12194
 * ufunc_owens_t_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_owens_t_types[0] = <char>NPY_FLOAT
 * ufunc_owens_t_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_owens_t_types[2] = <char>NPY_FLOAT
 * ufunc_owens_t_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12195
 * ufunc_owens_t_types[0] = <char>NPY_FLOAT
 * ufunc_owens_t_types[1] = <char>NPY_FLOAT
 * ufunc_owens_t_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_owens_t_types[3] = <char>NPY_DOUBLE
 * ufunc_owens_t_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12196
 * ufunc_owens_t_types[1] = <char>NPY_FLOAT
 * ufunc_owens_t_types[2] = <char>NPY_FLOAT
 * ufunc_owens_t_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_owens_t_types[4] = <char>NPY_DOUBLE
 * ufunc_owens_t_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12197
 * ufunc_owens_t_types[2] = <char>NPY_FLOAT
 * ufunc_owens_t_types[3] = <char>NPY_DOUBLE
 * ufunc_owens_t_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_owens_t_types[5] = <char>NPY_DOUBLE
 * ufunc_owens_t_ptr[2*0] = <void*>_func_owens_t
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12198
 * ufunc_owens_t_types[3] = <char>NPY_DOUBLE
 * ufunc_owens_t_types[4] = <char>NPY_DOUBLE
 * ufunc_owens_t_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_owens_t_ptr[2*0] = <void*>_func_owens_t
 * ufunc_owens_t_ptr[2*0+1] = <void*>(<char*>"owens_t")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12199
 * ufunc_owens_t_types[4] = <char>NPY_DOUBLE
 * ufunc_owens_t_types[5] = <char>NPY_DOUBLE
 * ufunc_owens_t_ptr[2*0] = <void*>_func_owens_t             # <<<<<<<<<<<<<<
 * ufunc_owens_t_ptr[2*0+1] = <void*>(<char*>"owens_t")
 * ufunc_owens_t_ptr[2*1] = <void*>_func_owens_t
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_ptr[0]) = ((void *)owens_t);

  /* "scipy/special/_ufuncs.pyx":12200
 * ufunc_owens_t_types[5] = <char>NPY_DOUBLE
 * ufunc_owens_t_ptr[2*0] = <void*>_func_owens_t
 * ufunc_owens_t_ptr[2*0+1] = <void*>(<char*>"owens_t")             # <<<<<<<<<<<<<<
 * ufunc_owens_t_ptr[2*1] = <void*>_func_owens_t
 * ufunc_owens_t_ptr[2*1+1] = <void*>(<char*>"owens_t")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_ptr[1]) = ((void *)((char *)((char *)"owens_t")));

  /* "scipy/special/_ufuncs.pyx":12201
 * ufunc_owens_t_ptr[2*0] = <void*>_func_owens_t
 * ufunc_owens_t_ptr[2*0+1] = <void*>(<char*>"owens_t")
 * ufunc_owens_t_ptr[2*1] = <void*>_func_owens_t             # <<<<<<<<<<<<<<
 * ufunc_owens_t_ptr[2*1+1] = <void*>(<char*>"owens_t")
 * ufunc_owens_t_data[0] = &ufunc_owens_t_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_ptr[2]) = ((void *)owens_t);

  /* "scipy/special/_ufuncs.pyx":12202
 * ufunc_owens_t_ptr[2*0+1] = <void*>(<char*>"owens_t")
 * ufunc_owens_t_ptr[2*1] = <void*>_func_owens_t
 * ufunc_owens_t_ptr[2*1+1] = <void*>(<char*>"owens_t")             # <<<<<<<<<<<<<<
 * ufunc_owens_t_data[0] = &ufunc_owens_t_ptr[2*0]
 * ufunc_owens_t_data[1] = &ufunc_owens_t_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_ptr[3]) = ((void *)((char *)((char *)"owens_t")));

  /* "scipy/special/_ufuncs.pyx":12203
 * ufunc_owens_t_ptr[2*1] = <void*>_func_owens_t
 * ufunc_owens_t_ptr[2*1+1] = <void*>(<char*>"owens_t")
 * ufunc_owens_t_data[0] = &ufunc_owens_t_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_owens_t_data[1] = &ufunc_owens_t_ptr[2*1]
 * owens_t = np.PyUFunc_FromFuncAndData(ufunc_owens_t_loops, ufunc_owens_t_data, ufunc_owens_t_types, 2, 2, 1, 0, "owens_t", ufunc_owens_t_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12204
 * ufunc_owens_t_ptr[2*1+1] = <void*>(<char*>"owens_t")
 * ufunc_owens_t_data[0] = &ufunc_owens_t_ptr[2*0]
 * ufunc_owens_t_data[1] = &ufunc_owens_t_ptr[2*1]             # <<<<<<<<<<<<<<
 * owens_t = np.PyUFunc_FromFuncAndData(ufunc_owens_t_loops, ufunc_owens_t_data, ufunc_owens_t_types, 2, 2, 1, 0, "owens_t", ufunc_owens_t_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12205
 * ufunc_owens_t_data[0] = &ufunc_owens_t_ptr[2*0]
 * ufunc_owens_t_data[1] = &ufunc_owens_t_ptr[2*1]
 * owens_t = np.PyUFunc_FromFuncAndData(ufunc_owens_t_loops, ufunc_owens_t_data, ufunc_owens_t_types, 2, 2, 1, 0, "owens_t", ufunc_owens_t_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pbdv_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_types, 2, 2, 1, 0, ((char *)"owens_t"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_owens_t_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_owens_t, __pyx_t_2) < 0) __PYX_ERR(1, 12205, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12211
 * cdef void *ufunc_pbdv_data[2]
 * cdef char ufunc_pbdv_types[8]
 * cdef char *ufunc_pbdv_doc = (             # <<<<<<<<<<<<<<
 *     "pbdv(v, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_doc = ((char *)"pbdv(v, x)\n\nParabolic cylinder function D\n\nReturns (d, dp) the parabolic cylinder function Dv(x) in d and the\nderivative, Dv'(x) in dp.\n\nReturns\n-------\nd\n    Value of the function\ndp\n    Value of the derivative vs x");

  /* "scipy/special/_ufuncs.pyx":12225
 *     "dp\n"
 *     "    Value of the derivative vs x")
 * ufunc_pbdv_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_ff_ff             # <<<<<<<<<<<<<<
 * ufunc_pbdv_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd
 * ufunc_pbdv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dd_As_ff_ff);

  /* "scipy/special/_ufuncs.pyx":12226
 *     "    Value of the derivative vs x")
 * ufunc_pbdv_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_ff_ff
 * ufunc_pbdv_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd             # <<<<<<<<<<<<<<
 * ufunc_pbdv_types[0] = <char>NPY_FLOAT
 * ufunc_pbdv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dd_As_dd_dd);

  /* "scipy/special/_ufuncs.pyx":12227
 * ufunc_pbdv_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_ff_ff
 * ufunc_pbdv_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd
 * ufunc_pbdv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbdv_types[1] = <char>NPY_FLOAT
 * ufunc_pbdv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12228
 * ufunc_pbdv_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd
 * ufunc_pbdv_types[0] = <char>NPY_FLOAT
 * ufunc_pbdv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbdv_types[2] = <char>NPY_FLOAT
 * ufunc_pbdv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12229
 * ufunc_pbdv_types[0] = <char>NPY_FLOAT
 * ufunc_pbdv_types[1] = <char>NPY_FLOAT
 * ufunc_pbdv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbdv_types[3] = <char>NPY_FLOAT
 * ufunc_pbdv_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12230
 * ufunc_pbdv_types[1] = <char>NPY_FLOAT
 * ufunc_pbdv_types[2] = <char>NPY_FLOAT
 * ufunc_pbdv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbdv_types[4] = <char>NPY_DOUBLE
 * ufunc_pbdv_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12231
 * ufunc_pbdv_types[2] = <char>NPY_FLOAT
 * ufunc_pbdv_types[3] = <char>NPY_FLOAT
 * ufunc_pbdv_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbdv_types[5] = <char>NPY_DOUBLE
 * ufunc_pbdv_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12232
 * ufunc_pbdv_types[3] = <char>NPY_FLOAT
 * ufunc_pbdv_types[4] = <char>NPY_DOUBLE
 * ufunc_pbdv_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbdv_types[6] = <char>NPY_DOUBLE
 * ufunc_pbdv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12233
 * ufunc_pbdv_types[4] = <char>NPY_DOUBLE
 * ufunc_pbdv_types[5] = <char>NPY_DOUBLE
 * ufunc_pbdv_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbdv_types[7] = <char>NPY_DOUBLE
 * ufunc_pbdv_ptr[2*0] = <void*>_func_pbdv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12234
 * ufunc_pbdv_types[5] = <char>NPY_DOUBLE
 * ufunc_pbdv_types[6] = <char>NPY_DOUBLE
 * ufunc_pbdv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbdv_ptr[2*0] = <void*>_func_pbdv_wrap
 * ufunc_pbdv_ptr[2*0+1] = <void*>(<char*>"pbdv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12235
 * ufunc_pbdv_types[6] = <char>NPY_DOUBLE
 * ufunc_pbdv_types[7] = <char>NPY_DOUBLE
 * ufunc_pbdv_ptr[2*0] = <void*>_func_pbdv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pbdv_ptr[2*0+1] = <void*>(<char*>"pbdv")
 * ufunc_pbdv_ptr[2*1] = <void*>_func_pbdv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_ptr[0]) = ((void *)pbdv_wrap);

  /* "scipy/special/_ufuncs.pyx":12236
 * ufunc_pbdv_types[7] = <char>NPY_DOUBLE
 * ufunc_pbdv_ptr[2*0] = <void*>_func_pbdv_wrap
 * ufunc_pbdv_ptr[2*0+1] = <void*>(<char*>"pbdv")             # <<<<<<<<<<<<<<
 * ufunc_pbdv_ptr[2*1] = <void*>_func_pbdv_wrap
 * ufunc_pbdv_ptr[2*1+1] = <void*>(<char*>"pbdv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_ptr[1]) = ((void *)((char *)((char *)"pbdv")));

  /* "scipy/special/_ufuncs.pyx":12237
 * ufunc_pbdv_ptr[2*0] = <void*>_func_pbdv_wrap
 * ufunc_pbdv_ptr[2*0+1] = <void*>(<char*>"pbdv")
 * ufunc_pbdv_ptr[2*1] = <void*>_func_pbdv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pbdv_ptr[2*1+1] = <void*>(<char*>"pbdv")
 * ufunc_pbdv_data[0] = &ufunc_pbdv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_ptr[2]) = ((void *)pbdv_wrap);

  /* "scipy/special/_ufuncs.pyx":12238
 * ufunc_pbdv_ptr[2*0+1] = <void*>(<char*>"pbdv")
 * ufunc_pbdv_ptr[2*1] = <void*>_func_pbdv_wrap
 * ufunc_pbdv_ptr[2*1+1] = <void*>(<char*>"pbdv")             # <<<<<<<<<<<<<<
 * ufunc_pbdv_data[0] = &ufunc_pbdv_ptr[2*0]
 * ufunc_pbdv_data[1] = &ufunc_pbdv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_ptr[3]) = ((void *)((char *)((char *)"pbdv")));

  /* "scipy/special/_ufuncs.pyx":12239
 * ufunc_pbdv_ptr[2*1] = <void*>_func_pbdv_wrap
 * ufunc_pbdv_ptr[2*1+1] = <void*>(<char*>"pbdv")
 * ufunc_pbdv_data[0] = &ufunc_pbdv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pbdv_data[1] = &ufunc_pbdv_ptr[2*1]
 * pbdv = np.PyUFunc_FromFuncAndData(ufunc_pbdv_loops, ufunc_pbdv_data, ufunc_pbdv_types, 2, 2, 2, 0, "pbdv", ufunc_pbdv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12240
 * ufunc_pbdv_ptr[2*1+1] = <void*>(<char*>"pbdv")
 * ufunc_pbdv_data[0] = &ufunc_pbdv_ptr[2*0]
 * ufunc_pbdv_data[1] = &ufunc_pbdv_ptr[2*1]             # <<<<<<<<<<<<<<
 * pbdv = np.PyUFunc_FromFuncAndData(ufunc_pbdv_loops, ufunc_pbdv_data, ufunc_pbdv_types, 2, 2, 2, 0, "pbdv", ufunc_pbdv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12241
 * ufunc_pbdv_data[0] = &ufunc_pbdv_ptr[2*0]
 * ufunc_pbdv_data[1] = &ufunc_pbdv_ptr[2*1]
 * pbdv = np.PyUFunc_FromFuncAndData(ufunc_pbdv_loops, ufunc_pbdv_data, ufunc_pbdv_types, 2, 2, 2, 0, "pbdv", ufunc_pbdv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pbvv_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_types, 2, 2, 2, 0, ((char *)"pbdv"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbdv_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pbdv, __pyx_t_2) < 0) __PYX_ERR(1, 12241, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12247
 * cdef void *ufunc_pbvv_data[2]
 * cdef char ufunc_pbvv_types[8]
 * cdef char *ufunc_pbvv_doc = (             # <<<<<<<<<<<<<<
 *     "pbvv(v, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_doc = ((char *)"pbvv(v, x)\n\nParabolic cylinder function V\n\nReturns the parabolic cylinder function Vv(x) in v and the\nderivative, Vv'(x) in vp.\n\nReturns\n-------\nv\n    Value of the function\nvp\n    Value of the derivative vs x");

  /* "scipy/special/_ufuncs.pyx":12261
 *     "vp\n"
 *     "    Value of the derivative vs x")
 * ufunc_pbvv_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_ff_ff             # <<<<<<<<<<<<<<
 * ufunc_pbvv_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd
 * ufunc_pbvv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dd_As_ff_ff);

  /* "scipy/special/_ufuncs.pyx":12262
 *     "    Value of the derivative vs x")
 * ufunc_pbvv_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_ff_ff
 * ufunc_pbvv_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd             # <<<<<<<<<<<<<<
 * ufunc_pbvv_types[0] = <char>NPY_FLOAT
 * ufunc_pbvv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dd_As_dd_dd);

  /* "scipy/special/_ufuncs.pyx":12263
 * ufunc_pbvv_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_ff_ff
 * ufunc_pbvv_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd
 * ufunc_pbvv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbvv_types[1] = <char>NPY_FLOAT
 * ufunc_pbvv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12264
 * ufunc_pbvv_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd
 * ufunc_pbvv_types[0] = <char>NPY_FLOAT
 * ufunc_pbvv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbvv_types[2] = <char>NPY_FLOAT
 * ufunc_pbvv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12265
 * ufunc_pbvv_types[0] = <char>NPY_FLOAT
 * ufunc_pbvv_types[1] = <char>NPY_FLOAT
 * ufunc_pbvv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbvv_types[3] = <char>NPY_FLOAT
 * ufunc_pbvv_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12266
 * ufunc_pbvv_types[1] = <char>NPY_FLOAT
 * ufunc_pbvv_types[2] = <char>NPY_FLOAT
 * ufunc_pbvv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbvv_types[4] = <char>NPY_DOUBLE
 * ufunc_pbvv_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12267
 * ufunc_pbvv_types[2] = <char>NPY_FLOAT
 * ufunc_pbvv_types[3] = <char>NPY_FLOAT
 * ufunc_pbvv_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbvv_types[5] = <char>NPY_DOUBLE
 * ufunc_pbvv_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12268
 * ufunc_pbvv_types[3] = <char>NPY_FLOAT
 * ufunc_pbvv_types[4] = <char>NPY_DOUBLE
 * ufunc_pbvv_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbvv_types[6] = <char>NPY_DOUBLE
 * ufunc_pbvv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12269
 * ufunc_pbvv_types[4] = <char>NPY_DOUBLE
 * ufunc_pbvv_types[5] = <char>NPY_DOUBLE
 * ufunc_pbvv_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbvv_types[7] = <char>NPY_DOUBLE
 * ufunc_pbvv_ptr[2*0] = <void*>_func_pbvv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12270
 * ufunc_pbvv_types[5] = <char>NPY_DOUBLE
 * ufunc_pbvv_types[6] = <char>NPY_DOUBLE
 * ufunc_pbvv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbvv_ptr[2*0] = <void*>_func_pbvv_wrap
 * ufunc_pbvv_ptr[2*0+1] = <void*>(<char*>"pbvv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12271
 * ufunc_pbvv_types[6] = <char>NPY_DOUBLE
 * ufunc_pbvv_types[7] = <char>NPY_DOUBLE
 * ufunc_pbvv_ptr[2*0] = <void*>_func_pbvv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pbvv_ptr[2*0+1] = <void*>(<char*>"pbvv")
 * ufunc_pbvv_ptr[2*1] = <void*>_func_pbvv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_ptr[0]) = ((void *)pbvv_wrap);

  /* "scipy/special/_ufuncs.pyx":12272
 * ufunc_pbvv_types[7] = <char>NPY_DOUBLE
 * ufunc_pbvv_ptr[2*0] = <void*>_func_pbvv_wrap
 * ufunc_pbvv_ptr[2*0+1] = <void*>(<char*>"pbvv")             # <<<<<<<<<<<<<<
 * ufunc_pbvv_ptr[2*1] = <void*>_func_pbvv_wrap
 * ufunc_pbvv_ptr[2*1+1] = <void*>(<char*>"pbvv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_ptr[1]) = ((void *)((char *)((char *)"pbvv")));

  /* "scipy/special/_ufuncs.pyx":12273
 * ufunc_pbvv_ptr[2*0] = <void*>_func_pbvv_wrap
 * ufunc_pbvv_ptr[2*0+1] = <void*>(<char*>"pbvv")
 * ufunc_pbvv_ptr[2*1] = <void*>_func_pbvv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pbvv_ptr[2*1+1] = <void*>(<char*>"pbvv")
 * ufunc_pbvv_data[0] = &ufunc_pbvv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_ptr[2]) = ((void *)pbvv_wrap);

  /* "scipy/special/_ufuncs.pyx":12274
 * ufunc_pbvv_ptr[2*0+1] = <void*>(<char*>"pbvv")
 * ufunc_pbvv_ptr[2*1] = <void*>_func_pbvv_wrap
 * ufunc_pbvv_ptr[2*1+1] = <void*>(<char*>"pbvv")             # <<<<<<<<<<<<<<
 * ufunc_pbvv_data[0] = &ufunc_pbvv_ptr[2*0]
 * ufunc_pbvv_data[1] = &ufunc_pbvv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_ptr[3]) = ((void *)((char *)((char *)"pbvv")));

  /* "scipy/special/_ufuncs.pyx":12275
 * ufunc_pbvv_ptr[2*1] = <void*>_func_pbvv_wrap
 * ufunc_pbvv_ptr[2*1+1] = <void*>(<char*>"pbvv")
 * ufunc_pbvv_data[0] = &ufunc_pbvv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pbvv_data[1] = &ufunc_pbvv_ptr[2*1]
 * pbvv = np.PyUFunc_FromFuncAndData(ufunc_pbvv_loops, ufunc_pbvv_data, ufunc_pbvv_types, 2, 2, 2, 0, "pbvv", ufunc_pbvv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12276
 * ufunc_pbvv_ptr[2*1+1] = <void*>(<char*>"pbvv")
 * ufunc_pbvv_data[0] = &ufunc_pbvv_ptr[2*0]
 * ufunc_pbvv_data[1] = &ufunc_pbvv_ptr[2*1]             # <<<<<<<<<<<<<<
 * pbvv = np.PyUFunc_FromFuncAndData(ufunc_pbvv_loops, ufunc_pbvv_data, ufunc_pbvv_types, 2, 2, 2, 0, "pbvv", ufunc_pbvv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12277
 * ufunc_pbvv_data[0] = &ufunc_pbvv_ptr[2*0]
 * ufunc_pbvv_data[1] = &ufunc_pbvv_ptr[2*1]
 * pbvv = np.PyUFunc_FromFuncAndData(ufunc_pbvv_loops, ufunc_pbvv_data, ufunc_pbvv_types, 2, 2, 2, 0, "pbvv", ufunc_pbvv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pbwa_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_types, 2, 2, 2, 0, ((char *)"pbvv"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbvv_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12277, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pbvv, __pyx_t_2) < 0) __PYX_ERR(1, 12277, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12283
 * cdef void *ufunc_pbwa_data[2]
 * cdef char ufunc_pbwa_types[8]
 * cdef char *ufunc_pbwa_doc = (             # <<<<<<<<<<<<<<
 *     "pbwa(a, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_doc = ((char *)"pbwa(a, x)\n\nParabolic cylinder function W.\n\nThe function is a particular solution to the differential equation\n\n.. math::\n\n    y'' + \\left(\\frac{1}{4}x^2 - a\\right)y = 0,\n\nfor a full definition see section 12.14 in [1]_.\n\nParameters\n----------\na : array_like\n    Real parameter\nx : array_like\n    Real argument\n\nReturns\n-------\nw : scalar or ndarray\n    Value of the function\nwp : scalar or ndarray\n    Value of the derivative in x\n\nNotes\n-----\nThe function is a wrapper for a Fortran routine by Zhang and Jin\n[2]_. The implementation is accurate only for ``|a|, |x| < 5`` and\nreturns NaN outside that range.\n\nReferences\n----------\n.. [1] Digital Library of Mathematical Functions, 14.30.\n       https://dlmf.nist.gov/14.30\n.. [2] Zhang, Shanjie and Jin, Jianming. \"Computation of Special\n       Functions\", John Wiley and Sons, 1996.\n       https://people.sc.fsu.edu/~jburkardt/f_src/special_functions/special_functions.html");

  /* "scipy/special/_ufuncs.pyx":12323
 *     "       Functions\", John Wiley and Sons, 1996.\n"
 *     "       https://people.sc.fsu.edu/~jburkardt/f_src/special_functions/special_functions.html")
 * ufunc_pbwa_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_ff_ff             # <<<<<<<<<<<<<<
 * ufunc_pbwa_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd
 * ufunc_pbwa_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dd_As_ff_ff);

  /* "scipy/special/_ufuncs.pyx":12324
 *     "       https://people.sc.fsu.edu/~jburkardt/f_src/special_functions/special_functions.html")
 * ufunc_pbwa_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_ff_ff
 * ufunc_pbwa_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd             # <<<<<<<<<<<<<<
 * ufunc_pbwa_types[0] = <char>NPY_FLOAT
 * ufunc_pbwa_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_dd_dd_As_dd_dd);

  /* "scipy/special/_ufuncs.pyx":12325
 * ufunc_pbwa_loops[0] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_ff_ff
 * ufunc_pbwa_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd
 * ufunc_pbwa_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbwa_types[1] = <char>NPY_FLOAT
 * ufunc_pbwa_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12326
 * ufunc_pbwa_loops[1] = <np.PyUFuncGenericFunction>loop_i_dd_dd_As_dd_dd
 * ufunc_pbwa_types[0] = <char>NPY_FLOAT
 * ufunc_pbwa_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbwa_types[2] = <char>NPY_FLOAT
 * ufunc_pbwa_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12327
 * ufunc_pbwa_types[0] = <char>NPY_FLOAT
 * ufunc_pbwa_types[1] = <char>NPY_FLOAT
 * ufunc_pbwa_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbwa_types[3] = <char>NPY_FLOAT
 * ufunc_pbwa_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12328
 * ufunc_pbwa_types[1] = <char>NPY_FLOAT
 * ufunc_pbwa_types[2] = <char>NPY_FLOAT
 * ufunc_pbwa_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pbwa_types[4] = <char>NPY_DOUBLE
 * ufunc_pbwa_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12329
 * ufunc_pbwa_types[2] = <char>NPY_FLOAT
 * ufunc_pbwa_types[3] = <char>NPY_FLOAT
 * ufunc_pbwa_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbwa_types[5] = <char>NPY_DOUBLE
 * ufunc_pbwa_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12330
 * ufunc_pbwa_types[3] = <char>NPY_FLOAT
 * ufunc_pbwa_types[4] = <char>NPY_DOUBLE
 * ufunc_pbwa_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbwa_types[6] = <char>NPY_DOUBLE
 * ufunc_pbwa_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12331
 * ufunc_pbwa_types[4] = <char>NPY_DOUBLE
 * ufunc_pbwa_types[5] = <char>NPY_DOUBLE
 * ufunc_pbwa_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbwa_types[7] = <char>NPY_DOUBLE
 * ufunc_pbwa_ptr[2*0] = <void*>_func_pbwa_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12332
 * ufunc_pbwa_types[5] = <char>NPY_DOUBLE
 * ufunc_pbwa_types[6] = <char>NPY_DOUBLE
 * ufunc_pbwa_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pbwa_ptr[2*0] = <void*>_func_pbwa_wrap
 * ufunc_pbwa_ptr[2*0+1] = <void*>(<char*>"pbwa")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12333
 * ufunc_pbwa_types[6] = <char>NPY_DOUBLE
 * ufunc_pbwa_types[7] = <char>NPY_DOUBLE
 * ufunc_pbwa_ptr[2*0] = <void*>_func_pbwa_wrap             # <<<<<<<<<<<<<<
 * ufunc_pbwa_ptr[2*0+1] = <void*>(<char*>"pbwa")
 * ufunc_pbwa_ptr[2*1] = <void*>_func_pbwa_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_ptr[0]) = ((void *)pbwa_wrap);

  /* "scipy/special/_ufuncs.pyx":12334
 * ufunc_pbwa_types[7] = <char>NPY_DOUBLE
 * ufunc_pbwa_ptr[2*0] = <void*>_func_pbwa_wrap
 * ufunc_pbwa_ptr[2*0+1] = <void*>(<char*>"pbwa")             # <<<<<<<<<<<<<<
 * ufunc_pbwa_ptr[2*1] = <void*>_func_pbwa_wrap
 * ufunc_pbwa_ptr[2*1+1] = <void*>(<char*>"pbwa")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_ptr[1]) = ((void *)((char *)((char *)"pbwa")));

  /* "scipy/special/_ufuncs.pyx":12335
 * ufunc_pbwa_ptr[2*0] = <void*>_func_pbwa_wrap
 * ufunc_pbwa_ptr[2*0+1] = <void*>(<char*>"pbwa")
 * ufunc_pbwa_ptr[2*1] = <void*>_func_pbwa_wrap             # <<<<<<<<<<<<<<
 * ufunc_pbwa_ptr[2*1+1] = <void*>(<char*>"pbwa")
 * ufunc_pbwa_data[0] = &ufunc_pbwa_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_ptr[2]) = ((void *)pbwa_wrap);

  /* "scipy/special/_ufuncs.pyx":12336
 * ufunc_pbwa_ptr[2*0+1] = <void*>(<char*>"pbwa")
 * ufunc_pbwa_ptr[2*1] = <void*>_func_pbwa_wrap
 * ufunc_pbwa_ptr[2*1+1] = <void*>(<char*>"pbwa")             # <<<<<<<<<<<<<<
 * ufunc_pbwa_data[0] = &ufunc_pbwa_ptr[2*0]
 * ufunc_pbwa_data[1] = &ufunc_pbwa_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_ptr[3]) = ((void *)((char *)((char *)"pbwa")));

  /* "scipy/special/_ufuncs.pyx":12337
 * ufunc_pbwa_ptr[2*1] = <void*>_func_pbwa_wrap
 * ufunc_pbwa_ptr[2*1+1] = <void*>(<char*>"pbwa")
 * ufunc_pbwa_data[0] = &ufunc_pbwa_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pbwa_data[1] = &ufunc_pbwa_ptr[2*1]
 * pbwa = np.PyUFunc_FromFuncAndData(ufunc_pbwa_loops, ufunc_pbwa_data, ufunc_pbwa_types, 2, 2, 2, 0, "pbwa", ufunc_pbwa_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12338
 * ufunc_pbwa_ptr[2*1+1] = <void*>(<char*>"pbwa")
 * ufunc_pbwa_data[0] = &ufunc_pbwa_ptr[2*0]
 * ufunc_pbwa_data[1] = &ufunc_pbwa_ptr[2*1]             # <<<<<<<<<<<<<<
 * pbwa = np.PyUFunc_FromFuncAndData(ufunc_pbwa_loops, ufunc_pbwa_data, ufunc_pbwa_types, 2, 2, 2, 0, "pbwa", ufunc_pbwa_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12339
 * ufunc_pbwa_data[0] = &ufunc_pbwa_ptr[2*0]
 * ufunc_pbwa_data[1] = &ufunc_pbwa_ptr[2*1]
 * pbwa = np.PyUFunc_FromFuncAndData(ufunc_pbwa_loops, ufunc_pbwa_data, ufunc_pbwa_types, 2, 2, 2, 0, "pbwa", ufunc_pbwa_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pdtr_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_types, 2, 2, 2, 0, ((char *)"pbwa"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_pbwa_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12339, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pbwa, __pyx_t_2) < 0) __PYX_ERR(1, 12339, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12345
 * cdef void *ufunc_pdtr_data[3]
 * cdef char ufunc_pdtr_types[9]
 * cdef char *ufunc_pdtr_doc = (             # <<<<<<<<<<<<<<
 *     "pdtr(k, m)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_doc = ((char *)"pdtr(k, m)\n\nPoisson cumulative distribution function\n\nReturns the sum of the first `k` terms of the Poisson distribution:\nsum(exp(-m) * m**j / j!, j=0..k) = gammaincc( k+1, m).  Arguments\nmust both be positive and `k` an integer.");

  /* "scipy/special/_ufuncs.pyx":12353
 *     "sum(exp(-m) * m**j / j!, j=0..k) = gammaincc( k+1, m).  Arguments\n"
 *     "must both be positive and `k` an integer.")
 * ufunc_pdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_pdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":12354
 *     "must both be positive and `k` an integer.")
 * ufunc_pdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_pdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_pdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtr_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":12355
 * ufunc_pdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_pdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_pdtr_types[0] = <char>NPY_LONG
 * ufunc_pdtr_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":12356
 * ufunc_pdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtr_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_pdtr_types[1] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":12357
 * ufunc_pdtr_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtr_types[0] = <char>NPY_LONG
 * ufunc_pdtr_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtr_types[2] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12358
 * ufunc_pdtr_types[0] = <char>NPY_LONG
 * ufunc_pdtr_types[1] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtr_types[3] = <char>NPY_FLOAT
 * ufunc_pdtr_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12359
 * ufunc_pdtr_types[1] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[2] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtr_types[4] = <char>NPY_FLOAT
 * ufunc_pdtr_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12360
 * ufunc_pdtr_types[2] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[3] = <char>NPY_FLOAT
 * ufunc_pdtr_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtr_types[5] = <char>NPY_FLOAT
 * ufunc_pdtr_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12361
 * ufunc_pdtr_types[3] = <char>NPY_FLOAT
 * ufunc_pdtr_types[4] = <char>NPY_FLOAT
 * ufunc_pdtr_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12362
 * ufunc_pdtr_types[4] = <char>NPY_FLOAT
 * ufunc_pdtr_types[5] = <char>NPY_FLOAT
 * ufunc_pdtr_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12363
 * ufunc_pdtr_types[5] = <char>NPY_FLOAT
 * ufunc_pdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_pdtr_ptr[2*0] = <void*>_func_pdtr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12364
 * ufunc_pdtr_types[6] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtr_ptr[2*0] = <void*>_func_pdtr
 * ufunc_pdtr_ptr[2*0+1] = <void*>(<char*>"pdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12365
 * ufunc_pdtr_types[7] = <char>NPY_DOUBLE
 * ufunc_pdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_pdtr_ptr[2*0] = <void*>_func_pdtr             # <<<<<<<<<<<<<<
 * ufunc_pdtr_ptr[2*0+1] = <void*>(<char*>"pdtr")
 * ufunc_pdtr_ptr[2*1] = <void*>_func_pdtr_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_ptr[0]) = ((void *)pdtr);

  /* "scipy/special/_ufuncs.pyx":12366
 * ufunc_pdtr_types[8] = <char>NPY_DOUBLE
 * ufunc_pdtr_ptr[2*0] = <void*>_func_pdtr
 * ufunc_pdtr_ptr[2*0+1] = <void*>(<char*>"pdtr")             # <<<<<<<<<<<<<<
 * ufunc_pdtr_ptr[2*1] = <void*>_func_pdtr_unsafe
 * ufunc_pdtr_ptr[2*1+1] = <void*>(<char*>"pdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_ptr[1]) = ((void *)((char *)((char *)"pdtr")));

  /* "scipy/special/_ufuncs.pyx":12367
 * ufunc_pdtr_ptr[2*0] = <void*>_func_pdtr
 * ufunc_pdtr_ptr[2*0+1] = <void*>(<char*>"pdtr")
 * ufunc_pdtr_ptr[2*1] = <void*>_func_pdtr_unsafe             # <<<<<<<<<<<<<<
 * ufunc_pdtr_ptr[2*1+1] = <void*>(<char*>"pdtr")
 * ufunc_pdtr_ptr[2*2] = <void*>_func_pdtr_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_pdtr_unsafe);

  /* "scipy/special/_ufuncs.pyx":12368
 * ufunc_pdtr_ptr[2*0+1] = <void*>(<char*>"pdtr")
 * ufunc_pdtr_ptr[2*1] = <void*>_func_pdtr_unsafe
 * ufunc_pdtr_ptr[2*1+1] = <void*>(<char*>"pdtr")             # <<<<<<<<<<<<<<
 * ufunc_pdtr_ptr[2*2] = <void*>_func_pdtr_unsafe
 * ufunc_pdtr_ptr[2*2+1] = <void*>(<char*>"pdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_ptr[3]) = ((void *)((char *)((char *)"pdtr")));

  /* "scipy/special/_ufuncs.pyx":12369
 * ufunc_pdtr_ptr[2*1] = <void*>_func_pdtr_unsafe
 * ufunc_pdtr_ptr[2*1+1] = <void*>(<char*>"pdtr")
 * ufunc_pdtr_ptr[2*2] = <void*>_func_pdtr_unsafe             # <<<<<<<<<<<<<<
 * ufunc_pdtr_ptr[2*2+1] = <void*>(<char*>"pdtr")
 * ufunc_pdtr_data[0] = &ufunc_pdtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_pdtr_unsafe);

  /* "scipy/special/_ufuncs.pyx":12370
 * ufunc_pdtr_ptr[2*1+1] = <void*>(<char*>"pdtr")
 * ufunc_pdtr_ptr[2*2] = <void*>_func_pdtr_unsafe
 * ufunc_pdtr_ptr[2*2+1] = <void*>(<char*>"pdtr")             # <<<<<<<<<<<<<<
 * ufunc_pdtr_data[0] = &ufunc_pdtr_ptr[2*0]
 * ufunc_pdtr_data[1] = &ufunc_pdtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_ptr[5]) = ((void *)((char *)((char *)"pdtr")));

  /* "scipy/special/_ufuncs.pyx":12371
 * ufunc_pdtr_ptr[2*2] = <void*>_func_pdtr_unsafe
 * ufunc_pdtr_ptr[2*2+1] = <void*>(<char*>"pdtr")
 * ufunc_pdtr_data[0] = &ufunc_pdtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pdtr_data[1] = &ufunc_pdtr_ptr[2*1]
 * ufunc_pdtr_data[2] = &ufunc_pdtr_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12372
 * ufunc_pdtr_ptr[2*2+1] = <void*>(<char*>"pdtr")
 * ufunc_pdtr_data[0] = &ufunc_pdtr_ptr[2*0]
 * ufunc_pdtr_data[1] = &ufunc_pdtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_pdtr_data[2] = &ufunc_pdtr_ptr[2*2]
 * pdtr = np.PyUFunc_FromFuncAndData(ufunc_pdtr_loops, ufunc_pdtr_data, ufunc_pdtr_types, 3, 2, 1, 0, "pdtr", ufunc_pdtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12373
 * ufunc_pdtr_data[0] = &ufunc_pdtr_ptr[2*0]
 * ufunc_pdtr_data[1] = &ufunc_pdtr_ptr[2*1]
 * ufunc_pdtr_data[2] = &ufunc_pdtr_ptr[2*2]             # <<<<<<<<<<<<<<
 * pdtr = np.PyUFunc_FromFuncAndData(ufunc_pdtr_loops, ufunc_pdtr_data, ufunc_pdtr_types, 3, 2, 1, 0, "pdtr", ufunc_pdtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":12374
 * ufunc_pdtr_data[1] = &ufunc_pdtr_ptr[2*1]
 * ufunc_pdtr_data[2] = &ufunc_pdtr_ptr[2*2]
 * pdtr = np.PyUFunc_FromFuncAndData(ufunc_pdtr_loops, ufunc_pdtr_data, ufunc_pdtr_types, 3, 2, 1, 0, "pdtr", ufunc_pdtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pdtrc_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_types, 3, 2, 1, 0, ((char *)"pdtr"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtr_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12374, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pdtr, __pyx_t_2) < 0) __PYX_ERR(1, 12374, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12380
 * cdef void *ufunc_pdtrc_data[3]
 * cdef char ufunc_pdtrc_types[9]
 * cdef char *ufunc_pdtrc_doc = (             # <<<<<<<<<<<<<<
 *     "pdtrc(k, m)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_doc = ((char *)"pdtrc(k, m)\n\nPoisson survival function\n\nReturns the sum of the terms from k+1 to infinity of the Poisson\ndistribution: sum(exp(-m) * m**j / j!, j=k+1..inf) = gammainc(\nk+1, m).  Arguments must both be positive and `k` an integer.");

  /* "scipy/special/_ufuncs.pyx":12388
 *     "distribution: sum(exp(-m) * m**j / j!, j=k+1..inf) = gammainc(\n"
 *     "k+1, m).  Arguments must both be positive and `k` an integer.")
 * ufunc_pdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":12389
 *     "k+1, m).  Arguments must both be positive and `k` an integer.")
 * ufunc_pdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_pdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtrc_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":12390
 * ufunc_pdtrc_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_pdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_types[0] = <char>NPY_LONG
 * ufunc_pdtrc_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":12391
 * ufunc_pdtrc_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtrc_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_types[1] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":12392
 * ufunc_pdtrc_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtrc_types[0] = <char>NPY_LONG
 * ufunc_pdtrc_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_types[2] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12393
 * ufunc_pdtrc_types[0] = <char>NPY_LONG
 * ufunc_pdtrc_types[1] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_types[3] = <char>NPY_FLOAT
 * ufunc_pdtrc_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12394
 * ufunc_pdtrc_types[1] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[2] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_types[4] = <char>NPY_FLOAT
 * ufunc_pdtrc_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12395
 * ufunc_pdtrc_types[2] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[3] = <char>NPY_FLOAT
 * ufunc_pdtrc_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_types[5] = <char>NPY_FLOAT
 * ufunc_pdtrc_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12396
 * ufunc_pdtrc_types[3] = <char>NPY_FLOAT
 * ufunc_pdtrc_types[4] = <char>NPY_FLOAT
 * ufunc_pdtrc_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_types[6] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12397
 * ufunc_pdtrc_types[4] = <char>NPY_FLOAT
 * ufunc_pdtrc_types[5] = <char>NPY_FLOAT
 * ufunc_pdtrc_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_types[7] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12398
 * ufunc_pdtrc_types[5] = <char>NPY_FLOAT
 * ufunc_pdtrc_types[6] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_types[8] = <char>NPY_DOUBLE
 * ufunc_pdtrc_ptr[2*0] = <void*>_func_pdtrc
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12399
 * ufunc_pdtrc_types[6] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[7] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_ptr[2*0] = <void*>_func_pdtrc
 * ufunc_pdtrc_ptr[2*0+1] = <void*>(<char*>"pdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12400
 * ufunc_pdtrc_types[7] = <char>NPY_DOUBLE
 * ufunc_pdtrc_types[8] = <char>NPY_DOUBLE
 * ufunc_pdtrc_ptr[2*0] = <void*>_func_pdtrc             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_ptr[2*0+1] = <void*>(<char*>"pdtrc")
 * ufunc_pdtrc_ptr[2*1] = <void*>_func_pdtrc_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_ptr[0]) = ((void *)pdtrc);

  /* "scipy/special/_ufuncs.pyx":12401
 * ufunc_pdtrc_types[8] = <char>NPY_DOUBLE
 * ufunc_pdtrc_ptr[2*0] = <void*>_func_pdtrc
 * ufunc_pdtrc_ptr[2*0+1] = <void*>(<char*>"pdtrc")             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_ptr[2*1] = <void*>_func_pdtrc_unsafe
 * ufunc_pdtrc_ptr[2*1+1] = <void*>(<char*>"pdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_ptr[1]) = ((void *)((char *)((char *)"pdtrc")));

  /* "scipy/special/_ufuncs.pyx":12402
 * ufunc_pdtrc_ptr[2*0] = <void*>_func_pdtrc
 * ufunc_pdtrc_ptr[2*0+1] = <void*>(<char*>"pdtrc")
 * ufunc_pdtrc_ptr[2*1] = <void*>_func_pdtrc_unsafe             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_ptr[2*1+1] = <void*>(<char*>"pdtrc")
 * ufunc_pdtrc_ptr[2*2] = <void*>_func_pdtrc_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_pdtrc_unsafe);

  /* "scipy/special/_ufuncs.pyx":12403
 * ufunc_pdtrc_ptr[2*0+1] = <void*>(<char*>"pdtrc")
 * ufunc_pdtrc_ptr[2*1] = <void*>_func_pdtrc_unsafe
 * ufunc_pdtrc_ptr[2*1+1] = <void*>(<char*>"pdtrc")             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_ptr[2*2] = <void*>_func_pdtrc_unsafe
 * ufunc_pdtrc_ptr[2*2+1] = <void*>(<char*>"pdtrc")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_ptr[3]) = ((void *)((char *)((char *)"pdtrc")));

  /* "scipy/special/_ufuncs.pyx":12404
 * ufunc_pdtrc_ptr[2*1] = <void*>_func_pdtrc_unsafe
 * ufunc_pdtrc_ptr[2*1+1] = <void*>(<char*>"pdtrc")
 * ufunc_pdtrc_ptr[2*2] = <void*>_func_pdtrc_unsafe             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_ptr[2*2+1] = <void*>(<char*>"pdtrc")
 * ufunc_pdtrc_data[0] = &ufunc_pdtrc_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_pdtrc_unsafe);

  /* "scipy/special/_ufuncs.pyx":12405
 * ufunc_pdtrc_ptr[2*1+1] = <void*>(<char*>"pdtrc")
 * ufunc_pdtrc_ptr[2*2] = <void*>_func_pdtrc_unsafe
 * ufunc_pdtrc_ptr[2*2+1] = <void*>(<char*>"pdtrc")             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_data[0] = &ufunc_pdtrc_ptr[2*0]
 * ufunc_pdtrc_data[1] = &ufunc_pdtrc_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_ptr[5]) = ((void *)((char *)((char *)"pdtrc")));

  /* "scipy/special/_ufuncs.pyx":12406
 * ufunc_pdtrc_ptr[2*2] = <void*>_func_pdtrc_unsafe
 * ufunc_pdtrc_ptr[2*2+1] = <void*>(<char*>"pdtrc")
 * ufunc_pdtrc_data[0] = &ufunc_pdtrc_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_data[1] = &ufunc_pdtrc_ptr[2*1]
 * ufunc_pdtrc_data[2] = &ufunc_pdtrc_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12407
 * ufunc_pdtrc_ptr[2*2+1] = <void*>(<char*>"pdtrc")
 * ufunc_pdtrc_data[0] = &ufunc_pdtrc_ptr[2*0]
 * ufunc_pdtrc_data[1] = &ufunc_pdtrc_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_pdtrc_data[2] = &ufunc_pdtrc_ptr[2*2]
 * pdtrc = np.PyUFunc_FromFuncAndData(ufunc_pdtrc_loops, ufunc_pdtrc_data, ufunc_pdtrc_types, 3, 2, 1, 0, "pdtrc", ufunc_pdtrc_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12408
 * ufunc_pdtrc_data[0] = &ufunc_pdtrc_ptr[2*0]
 * ufunc_pdtrc_data[1] = &ufunc_pdtrc_ptr[2*1]
 * ufunc_pdtrc_data[2] = &ufunc_pdtrc_ptr[2*2]             # <<<<<<<<<<<<<<
 * pdtrc = np.PyUFunc_FromFuncAndData(ufunc_pdtrc_loops, ufunc_pdtrc_data, ufunc_pdtrc_types, 3, 2, 1, 0, "pdtrc", ufunc_pdtrc_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":12409
 * ufunc_pdtrc_data[1] = &ufunc_pdtrc_ptr[2*1]
 * ufunc_pdtrc_data[2] = &ufunc_pdtrc_ptr[2*2]
 * pdtrc = np.PyUFunc_FromFuncAndData(ufunc_pdtrc_loops, ufunc_pdtrc_data, ufunc_pdtrc_types, 3, 2, 1, 0, "pdtrc", ufunc_pdtrc_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pdtri_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_types, 3, 2, 1, 0, ((char *)"pdtrc"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrc_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pdtrc, __pyx_t_2) < 0) __PYX_ERR(1, 12409, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12415
 * cdef void *ufunc_pdtri_data[3]
 * cdef char ufunc_pdtri_types[9]
 * cdef char *ufunc_pdtri_doc = (             # <<<<<<<<<<<<<<
 *     "pdtri(k, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_doc = ((char *)"pdtri(k, y)\n\nInverse to `pdtr` vs m\n\nReturns the Poisson variable `m` such that the sum from 0 to `k` of\nthe Poisson density is equal to the given probability `y`:\ncalculated by gammaincinv(k+1, y). `k` must be a nonnegative\ninteger and `y` between 0 and 1.");

  /* "scipy/special/_ufuncs.pyx":12424
 *     "calculated by gammaincinv(k+1, y). `k` must be a nonnegative\n"
 *     "integer and `y` between 0 and 1.")
 * ufunc_pdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_pdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":12425
 *     "integer and `y` between 0 and 1.")
 * ufunc_pdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_pdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_pdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtri_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":12426
 * ufunc_pdtri_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_pdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_pdtri_types[0] = <char>NPY_LONG
 * ufunc_pdtri_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":12427
 * ufunc_pdtri_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtri_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_pdtri_types[1] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":12428
 * ufunc_pdtri_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtri_types[0] = <char>NPY_LONG
 * ufunc_pdtri_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtri_types[2] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12429
 * ufunc_pdtri_types[0] = <char>NPY_LONG
 * ufunc_pdtri_types[1] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtri_types[3] = <char>NPY_FLOAT
 * ufunc_pdtri_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12430
 * ufunc_pdtri_types[1] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[2] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtri_types[4] = <char>NPY_FLOAT
 * ufunc_pdtri_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12431
 * ufunc_pdtri_types[2] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[3] = <char>NPY_FLOAT
 * ufunc_pdtri_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtri_types[5] = <char>NPY_FLOAT
 * ufunc_pdtri_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12432
 * ufunc_pdtri_types[3] = <char>NPY_FLOAT
 * ufunc_pdtri_types[4] = <char>NPY_FLOAT
 * ufunc_pdtri_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12433
 * ufunc_pdtri_types[4] = <char>NPY_FLOAT
 * ufunc_pdtri_types[5] = <char>NPY_FLOAT
 * ufunc_pdtri_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12434
 * ufunc_pdtri_types[5] = <char>NPY_FLOAT
 * ufunc_pdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_pdtri_ptr[2*0] = <void*>_func_pdtri
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12435
 * ufunc_pdtri_types[6] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtri_ptr[2*0] = <void*>_func_pdtri
 * ufunc_pdtri_ptr[2*0+1] = <void*>(<char*>"pdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12436
 * ufunc_pdtri_types[7] = <char>NPY_DOUBLE
 * ufunc_pdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_pdtri_ptr[2*0] = <void*>_func_pdtri             # <<<<<<<<<<<<<<
 * ufunc_pdtri_ptr[2*0+1] = <void*>(<char*>"pdtri")
 * ufunc_pdtri_ptr[2*1] = <void*>_func_pdtri_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_ptr[0]) = ((void *)pdtri);

  /* "scipy/special/_ufuncs.pyx":12437
 * ufunc_pdtri_types[8] = <char>NPY_DOUBLE
 * ufunc_pdtri_ptr[2*0] = <void*>_func_pdtri
 * ufunc_pdtri_ptr[2*0+1] = <void*>(<char*>"pdtri")             # <<<<<<<<<<<<<<
 * ufunc_pdtri_ptr[2*1] = <void*>_func_pdtri_unsafe
 * ufunc_pdtri_ptr[2*1+1] = <void*>(<char*>"pdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_ptr[1]) = ((void *)((char *)((char *)"pdtri")));

  /* "scipy/special/_ufuncs.pyx":12438
 * ufunc_pdtri_ptr[2*0] = <void*>_func_pdtri
 * ufunc_pdtri_ptr[2*0+1] = <void*>(<char*>"pdtri")
 * ufunc_pdtri_ptr[2*1] = <void*>_func_pdtri_unsafe             # <<<<<<<<<<<<<<
 * ufunc_pdtri_ptr[2*1+1] = <void*>(<char*>"pdtri")
 * ufunc_pdtri_ptr[2*2] = <void*>_func_pdtri_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_pdtri_unsafe);

  /* "scipy/special/_ufuncs.pyx":12439
 * ufunc_pdtri_ptr[2*0+1] = <void*>(<char*>"pdtri")
 * ufunc_pdtri_ptr[2*1] = <void*>_func_pdtri_unsafe
 * ufunc_pdtri_ptr[2*1+1] = <void*>(<char*>"pdtri")             # <<<<<<<<<<<<<<
 * ufunc_pdtri_ptr[2*2] = <void*>_func_pdtri_unsafe
 * ufunc_pdtri_ptr[2*2+1] = <void*>(<char*>"pdtri")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_ptr[3]) = ((void *)((char *)((char *)"pdtri")));

  /* "scipy/special/_ufuncs.pyx":12440
 * ufunc_pdtri_ptr[2*1] = <void*>_func_pdtri_unsafe
 * ufunc_pdtri_ptr[2*1+1] = <void*>(<char*>"pdtri")
 * ufunc_pdtri_ptr[2*2] = <void*>_func_pdtri_unsafe             # <<<<<<<<<<<<<<
 * ufunc_pdtri_ptr[2*2+1] = <void*>(<char*>"pdtri")
 * ufunc_pdtri_data[0] = &ufunc_pdtri_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_pdtri_unsafe);

  /* "scipy/special/_ufuncs.pyx":12441
 * ufunc_pdtri_ptr[2*1+1] = <void*>(<char*>"pdtri")
 * ufunc_pdtri_ptr[2*2] = <void*>_func_pdtri_unsafe
 * ufunc_pdtri_ptr[2*2+1] = <void*>(<char*>"pdtri")             # <<<<<<<<<<<<<<
 * ufunc_pdtri_data[0] = &ufunc_pdtri_ptr[2*0]
 * ufunc_pdtri_data[1] = &ufunc_pdtri_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_ptr[5]) = ((void *)((char *)((char *)"pdtri")));

  /* "scipy/special/_ufuncs.pyx":12442
 * ufunc_pdtri_ptr[2*2] = <void*>_func_pdtri_unsafe
 * ufunc_pdtri_ptr[2*2+1] = <void*>(<char*>"pdtri")
 * ufunc_pdtri_data[0] = &ufunc_pdtri_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pdtri_data[1] = &ufunc_pdtri_ptr[2*1]
 * ufunc_pdtri_data[2] = &ufunc_pdtri_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12443
 * ufunc_pdtri_ptr[2*2+1] = <void*>(<char*>"pdtri")
 * ufunc_pdtri_data[0] = &ufunc_pdtri_ptr[2*0]
 * ufunc_pdtri_data[1] = &ufunc_pdtri_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_pdtri_data[2] = &ufunc_pdtri_ptr[2*2]
 * pdtri = np.PyUFunc_FromFuncAndData(ufunc_pdtri_loops, ufunc_pdtri_data, ufunc_pdtri_types, 3, 2, 1, 0, "pdtri", ufunc_pdtri_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12444
 * ufunc_pdtri_data[0] = &ufunc_pdtri_ptr[2*0]
 * ufunc_pdtri_data[1] = &ufunc_pdtri_ptr[2*1]
 * ufunc_pdtri_data[2] = &ufunc_pdtri_ptr[2*2]             # <<<<<<<<<<<<<<
 * pdtri = np.PyUFunc_FromFuncAndData(ufunc_pdtri_loops, ufunc_pdtri_data, ufunc_pdtri_types, 3, 2, 1, 0, "pdtri", ufunc_pdtri_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":12445
 * ufunc_pdtri_data[1] = &ufunc_pdtri_ptr[2*1]
 * ufunc_pdtri_data[2] = &ufunc_pdtri_ptr[2*2]
 * pdtri = np.PyUFunc_FromFuncAndData(ufunc_pdtri_loops, ufunc_pdtri_data, ufunc_pdtri_types, 3, 2, 1, 0, "pdtri", ufunc_pdtri_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pdtrik_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_types, 3, 2, 1, 0, ((char *)"pdtri"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtri_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pdtri, __pyx_t_2) < 0) __PYX_ERR(1, 12445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12451
 * cdef void *ufunc_pdtrik_data[2]
 * cdef char ufunc_pdtrik_types[6]
 * cdef char *ufunc_pdtrik_doc = (             # <<<<<<<<<<<<<<
 *     "pdtrik(p, m)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_doc = ((char *)"pdtrik(p, m)\n\nInverse to `pdtr` vs k\n\nReturns the quantile k such that ``pdtr(k, m) = p``");

  /* "scipy/special/_ufuncs.pyx":12457
 *     "\n"
 *     "Returns the quantile k such that ``pdtr(k, m) = p``")
 * ufunc_pdtrik_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtrik_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":12458
 *     "Returns the quantile k such that ``pdtr(k, m) = p``")
 * ufunc_pdtrik_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_types[0] = <char>NPY_FLOAT
 * ufunc_pdtrik_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":12459
 * ufunc_pdtrik_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtrik_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_types[1] = <char>NPY_FLOAT
 * ufunc_pdtrik_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12460
 * ufunc_pdtrik_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pdtrik_types[0] = <char>NPY_FLOAT
 * ufunc_pdtrik_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_types[2] = <char>NPY_FLOAT
 * ufunc_pdtrik_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12461
 * ufunc_pdtrik_types[0] = <char>NPY_FLOAT
 * ufunc_pdtrik_types[1] = <char>NPY_FLOAT
 * ufunc_pdtrik_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_types[3] = <char>NPY_DOUBLE
 * ufunc_pdtrik_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12462
 * ufunc_pdtrik_types[1] = <char>NPY_FLOAT
 * ufunc_pdtrik_types[2] = <char>NPY_FLOAT
 * ufunc_pdtrik_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_types[4] = <char>NPY_DOUBLE
 * ufunc_pdtrik_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12463
 * ufunc_pdtrik_types[2] = <char>NPY_FLOAT
 * ufunc_pdtrik_types[3] = <char>NPY_DOUBLE
 * ufunc_pdtrik_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_types[5] = <char>NPY_DOUBLE
 * ufunc_pdtrik_ptr[2*0] = <void*>_func_cdfpoi2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12464
 * ufunc_pdtrik_types[3] = <char>NPY_DOUBLE
 * ufunc_pdtrik_types[4] = <char>NPY_DOUBLE
 * ufunc_pdtrik_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_ptr[2*0] = <void*>_func_cdfpoi2_wrap
 * ufunc_pdtrik_ptr[2*0+1] = <void*>(<char*>"pdtrik")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12465
 * ufunc_pdtrik_types[4] = <char>NPY_DOUBLE
 * ufunc_pdtrik_types[5] = <char>NPY_DOUBLE
 * ufunc_pdtrik_ptr[2*0] = <void*>_func_cdfpoi2_wrap             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_ptr[2*0+1] = <void*>(<char*>"pdtrik")
 * ufunc_pdtrik_ptr[2*1] = <void*>_func_cdfpoi2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_ptr[0]) = ((void *)cdfpoi2_wrap);

  /* "scipy/special/_ufuncs.pyx":12466
 * ufunc_pdtrik_types[5] = <char>NPY_DOUBLE
 * ufunc_pdtrik_ptr[2*0] = <void*>_func_cdfpoi2_wrap
 * ufunc_pdtrik_ptr[2*0+1] = <void*>(<char*>"pdtrik")             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_ptr[2*1] = <void*>_func_cdfpoi2_wrap
 * ufunc_pdtrik_ptr[2*1+1] = <void*>(<char*>"pdtrik")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_ptr[1]) = ((void *)((char *)((char *)"pdtrik")));

  /* "scipy/special/_ufuncs.pyx":12467
 * ufunc_pdtrik_ptr[2*0] = <void*>_func_cdfpoi2_wrap
 * ufunc_pdtrik_ptr[2*0+1] = <void*>(<char*>"pdtrik")
 * ufunc_pdtrik_ptr[2*1] = <void*>_func_cdfpoi2_wrap             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_ptr[2*1+1] = <void*>(<char*>"pdtrik")
 * ufunc_pdtrik_data[0] = &ufunc_pdtrik_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_ptr[2]) = ((void *)cdfpoi2_wrap);

  /* "scipy/special/_ufuncs.pyx":12468
 * ufunc_pdtrik_ptr[2*0+1] = <void*>(<char*>"pdtrik")
 * ufunc_pdtrik_ptr[2*1] = <void*>_func_cdfpoi2_wrap
 * ufunc_pdtrik_ptr[2*1+1] = <void*>(<char*>"pdtrik")             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_data[0] = &ufunc_pdtrik_ptr[2*0]
 * ufunc_pdtrik_data[1] = &ufunc_pdtrik_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_ptr[3]) = ((void *)((char *)((char *)"pdtrik")));

  /* "scipy/special/_ufuncs.pyx":12469
 * ufunc_pdtrik_ptr[2*1] = <void*>_func_cdfpoi2_wrap
 * ufunc_pdtrik_ptr[2*1+1] = <void*>(<char*>"pdtrik")
 * ufunc_pdtrik_data[0] = &ufunc_pdtrik_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pdtrik_data[1] = &ufunc_pdtrik_ptr[2*1]
 * pdtrik = np.PyUFunc_FromFuncAndData(ufunc_pdtrik_loops, ufunc_pdtrik_data, ufunc_pdtrik_types, 2, 2, 1, 0, "pdtrik", ufunc_pdtrik_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12470
 * ufunc_pdtrik_ptr[2*1+1] = <void*>(<char*>"pdtrik")
 * ufunc_pdtrik_data[0] = &ufunc_pdtrik_ptr[2*0]
 * ufunc_pdtrik_data[1] = &ufunc_pdtrik_ptr[2*1]             # <<<<<<<<<<<<<<
 * pdtrik = np.PyUFunc_FromFuncAndData(ufunc_pdtrik_loops, ufunc_pdtrik_data, ufunc_pdtrik_types, 2, 2, 1, 0, "pdtrik", ufunc_pdtrik_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12471
 * ufunc_pdtrik_data[0] = &ufunc_pdtrik_ptr[2*0]
 * ufunc_pdtrik_data[1] = &ufunc_pdtrik_ptr[2*1]
 * pdtrik = np.PyUFunc_FromFuncAndData(ufunc_pdtrik_loops, ufunc_pdtrik_data, ufunc_pdtrik_types, 2, 2, 1, 0, "pdtrik", ufunc_pdtrik_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_poch_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_types, 2, 2, 1, 0, ((char *)"pdtrik"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_pdtrik_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12471, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pdtrik, __pyx_t_2) < 0) __PYX_ERR(1, 12471, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12477
 * cdef void *ufunc_poch_data[2]
 * cdef char ufunc_poch_types[6]
 * cdef char *ufunc_poch_doc = (             # <<<<<<<<<<<<<<
 *     "poch(z, m)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_doc = ((char *)"poch(z, m)\n\nRising factorial (z)_m\n\nThe Pochhammer symbol (rising factorial), is defined as\n\n.. math::\n\n    (z)_m = \\frac{\\Gamma(z + m)}{\\Gamma(z)}\n\nFor positive integer `m` it reads\n\n.. math::\n\n    (z)_m = z (z + 1) ... (z + m - 1)\n\nParameters\n----------\nz : array_like\n    (int or float)\nm : array_like\n    (int or float)\n\nReturns\n-------\npoch : ndarray\n    The value of the function.");

  /* "scipy/special/_ufuncs.pyx":12505
 *     "poch : ndarray\n"
 *     "    The value of the function.")
 * ufunc_poch_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_poch_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_poch_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":12506
 *     "    The value of the function.")
 * ufunc_poch_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_poch_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_poch_types[0] = <char>NPY_FLOAT
 * ufunc_poch_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":12507
 * ufunc_poch_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_poch_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_poch_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_poch_types[1] = <char>NPY_FLOAT
 * ufunc_poch_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12508
 * ufunc_poch_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_poch_types[0] = <char>NPY_FLOAT
 * ufunc_poch_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_poch_types[2] = <char>NPY_FLOAT
 * ufunc_poch_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12509
 * ufunc_poch_types[0] = <char>NPY_FLOAT
 * ufunc_poch_types[1] = <char>NPY_FLOAT
 * ufunc_poch_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_poch_types[3] = <char>NPY_DOUBLE
 * ufunc_poch_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12510
 * ufunc_poch_types[1] = <char>NPY_FLOAT
 * ufunc_poch_types[2] = <char>NPY_FLOAT
 * ufunc_poch_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_poch_types[4] = <char>NPY_DOUBLE
 * ufunc_poch_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12511
 * ufunc_poch_types[2] = <char>NPY_FLOAT
 * ufunc_poch_types[3] = <char>NPY_DOUBLE
 * ufunc_poch_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_poch_types[5] = <char>NPY_DOUBLE
 * ufunc_poch_ptr[2*0] = <void*>_func_poch
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12512
 * ufunc_poch_types[3] = <char>NPY_DOUBLE
 * ufunc_poch_types[4] = <char>NPY_DOUBLE
 * ufunc_poch_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_poch_ptr[2*0] = <void*>_func_poch
 * ufunc_poch_ptr[2*0+1] = <void*>(<char*>"poch")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12513
 * ufunc_poch_types[4] = <char>NPY_DOUBLE
 * ufunc_poch_types[5] = <char>NPY_DOUBLE
 * ufunc_poch_ptr[2*0] = <void*>_func_poch             # <<<<<<<<<<<<<<
 * ufunc_poch_ptr[2*0+1] = <void*>(<char*>"poch")
 * ufunc_poch_ptr[2*1] = <void*>_func_poch
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_ptr[0]) = ((void *)poch);

  /* "scipy/special/_ufuncs.pyx":12514
 * ufunc_poch_types[5] = <char>NPY_DOUBLE
 * ufunc_poch_ptr[2*0] = <void*>_func_poch
 * ufunc_poch_ptr[2*0+1] = <void*>(<char*>"poch")             # <<<<<<<<<<<<<<
 * ufunc_poch_ptr[2*1] = <void*>_func_poch
 * ufunc_poch_ptr[2*1+1] = <void*>(<char*>"poch")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_ptr[1]) = ((void *)((char *)((char *)"poch")));

  /* "scipy/special/_ufuncs.pyx":12515
 * ufunc_poch_ptr[2*0] = <void*>_func_poch
 * ufunc_poch_ptr[2*0+1] = <void*>(<char*>"poch")
 * ufunc_poch_ptr[2*1] = <void*>_func_poch             # <<<<<<<<<<<<<<
 * ufunc_poch_ptr[2*1+1] = <void*>(<char*>"poch")
 * ufunc_poch_data[0] = &ufunc_poch_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_ptr[2]) = ((void *)poch);

  /* "scipy/special/_ufuncs.pyx":12516
 * ufunc_poch_ptr[2*0+1] = <void*>(<char*>"poch")
 * ufunc_poch_ptr[2*1] = <void*>_func_poch
 * ufunc_poch_ptr[2*1+1] = <void*>(<char*>"poch")             # <<<<<<<<<<<<<<
 * ufunc_poch_data[0] = &ufunc_poch_ptr[2*0]
 * ufunc_poch_data[1] = &ufunc_poch_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_ptr[3]) = ((void *)((char *)((char *)"poch")));

  /* "scipy/special/_ufuncs.pyx":12517
 * ufunc_poch_ptr[2*1] = <void*>_func_poch
 * ufunc_poch_ptr[2*1+1] = <void*>(<char*>"poch")
 * ufunc_poch_data[0] = &ufunc_poch_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_poch_data[1] = &ufunc_poch_ptr[2*1]
 * poch = np.PyUFunc_FromFuncAndData(ufunc_poch_loops, ufunc_poch_data, ufunc_poch_types, 2, 2, 1, 0, "poch", ufunc_poch_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12518
 * ufunc_poch_ptr[2*1+1] = <void*>(<char*>"poch")
 * ufunc_poch_data[0] = &ufunc_poch_ptr[2*0]
 * ufunc_poch_data[1] = &ufunc_poch_ptr[2*1]             # <<<<<<<<<<<<<<
 * poch = np.PyUFunc_FromFuncAndData(ufunc_poch_loops, ufunc_poch_data, ufunc_poch_types, 2, 2, 1, 0, "poch", ufunc_poch_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12519
 * ufunc_poch_data[0] = &ufunc_poch_ptr[2*0]
 * ufunc_poch_data[1] = &ufunc_poch_ptr[2*1]
 * poch = np.PyUFunc_FromFuncAndData(ufunc_poch_loops, ufunc_poch_data, ufunc_poch_types, 2, 2, 1, 0, "poch", ufunc_poch_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pro_ang1_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_types, 2, 2, 1, 0, ((char *)"poch"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_poch_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_poch, __pyx_t_2) < 0) __PYX_ERR(1, 12519, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12525
 * cdef void *ufunc_pro_ang1_data[2]
 * cdef char ufunc_pro_ang1_types[12]
 * cdef char *ufunc_pro_ang1_doc = (             # <<<<<<<<<<<<<<
 *     "pro_ang1(m, n, c, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_doc = ((char *)"pro_ang1(m, n, c, x)\n\nProlate spheroidal angular function of the first kind and its derivative\n\nComputes the prolate spheroidal angular function of the first kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x");

  /* "scipy/special/_ufuncs.pyx":12540
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_pro_ang1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_pro_ang1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff);

  /* "scipy/special/_ufuncs.pyx":12541
 *     "    Value of the derivative vs x")
 * ufunc_pro_ang1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_pro_ang1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[0] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd);

  /* "scipy/special/_ufuncs.pyx":12542
 * ufunc_pro_ang1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_pro_ang1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_pro_ang1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[1] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12543
 * ufunc_pro_ang1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_pro_ang1_types[0] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[2] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12544
 * ufunc_pro_ang1_types[0] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[1] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[3] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12545
 * ufunc_pro_ang1_types[1] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[2] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[4] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12546
 * ufunc_pro_ang1_types[2] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[3] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[5] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12547
 * ufunc_pro_ang1_types[3] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[4] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12548
 * ufunc_pro_ang1_types[4] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[5] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12549
 * ufunc_pro_ang1_types[5] = <char>NPY_FLOAT
 * ufunc_pro_ang1_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12550
 * ufunc_pro_ang1_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12551
 * ufunc_pro_ang1_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12552
 * ufunc_pro_ang1_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_ptr[2*0] = <void*>_func_prolate_aswfa_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12553
 * ufunc_pro_ang1_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_ptr[2*0] = <void*>_func_prolate_aswfa_nocv_wrap
 * ufunc_pro_ang1_ptr[2*0+1] = <void*>(<char*>"pro_ang1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12554
 * ufunc_pro_ang1_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_ptr[2*0] = <void*>_func_prolate_aswfa_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_ptr[2*0+1] = <void*>(<char*>"pro_ang1")
 * ufunc_pro_ang1_ptr[2*1] = <void*>_func_prolate_aswfa_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_ptr[0]) = ((void *)prolate_aswfa_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":12555
 * ufunc_pro_ang1_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_ptr[2*0] = <void*>_func_prolate_aswfa_nocv_wrap
 * ufunc_pro_ang1_ptr[2*0+1] = <void*>(<char*>"pro_ang1")             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_ptr[2*1] = <void*>_func_prolate_aswfa_nocv_wrap
 * ufunc_pro_ang1_ptr[2*1+1] = <void*>(<char*>"pro_ang1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_ptr[1]) = ((void *)((char *)((char *)"pro_ang1")));

  /* "scipy/special/_ufuncs.pyx":12556
 * ufunc_pro_ang1_ptr[2*0] = <void*>_func_prolate_aswfa_nocv_wrap
 * ufunc_pro_ang1_ptr[2*0+1] = <void*>(<char*>"pro_ang1")
 * ufunc_pro_ang1_ptr[2*1] = <void*>_func_prolate_aswfa_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_ptr[2*1+1] = <void*>(<char*>"pro_ang1")
 * ufunc_pro_ang1_data[0] = &ufunc_pro_ang1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_ptr[2]) = ((void *)prolate_aswfa_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":12557
 * ufunc_pro_ang1_ptr[2*0+1] = <void*>(<char*>"pro_ang1")
 * ufunc_pro_ang1_ptr[2*1] = <void*>_func_prolate_aswfa_nocv_wrap
 * ufunc_pro_ang1_ptr[2*1+1] = <void*>(<char*>"pro_ang1")             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_data[0] = &ufunc_pro_ang1_ptr[2*0]
 * ufunc_pro_ang1_data[1] = &ufunc_pro_ang1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_ptr[3]) = ((void *)((char *)((char *)"pro_ang1")));

  /* "scipy/special/_ufuncs.pyx":12558
 * ufunc_pro_ang1_ptr[2*1] = <void*>_func_prolate_aswfa_nocv_wrap
 * ufunc_pro_ang1_ptr[2*1+1] = <void*>(<char*>"pro_ang1")
 * ufunc_pro_ang1_data[0] = &ufunc_pro_ang1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_data[1] = &ufunc_pro_ang1_ptr[2*1]
 * pro_ang1 = np.PyUFunc_FromFuncAndData(ufunc_pro_ang1_loops, ufunc_pro_ang1_data, ufunc_pro_ang1_types, 2, 4, 2, 0, "pro_ang1", ufunc_pro_ang1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12559
 * ufunc_pro_ang1_ptr[2*1+1] = <void*>(<char*>"pro_ang1")
 * ufunc_pro_ang1_data[0] = &ufunc_pro_ang1_ptr[2*0]
 * ufunc_pro_ang1_data[1] = &ufunc_pro_ang1_ptr[2*1]             # <<<<<<<<<<<<<<
 * pro_ang1 = np.PyUFunc_FromFuncAndData(ufunc_pro_ang1_loops, ufunc_pro_ang1_data, ufunc_pro_ang1_types, 2, 4, 2, 0, "pro_ang1", ufunc_pro_ang1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12560
 * ufunc_pro_ang1_data[0] = &ufunc_pro_ang1_ptr[2*0]
 * ufunc_pro_ang1_data[1] = &ufunc_pro_ang1_ptr[2*1]
 * pro_ang1 = np.PyUFunc_FromFuncAndData(ufunc_pro_ang1_loops, ufunc_pro_ang1_data, ufunc_pro_ang1_types, 2, 4, 2, 0, "pro_ang1", ufunc_pro_ang1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pro_ang1_cv_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_types, 2, 4, 2, 0, ((char *)"pro_ang1"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12560, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pro_ang1, __pyx_t_2) < 0) __PYX_ERR(1, 12560, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12566
 * cdef void *ufunc_pro_ang1_cv_data[2]
 * cdef char ufunc_pro_ang1_cv_types[14]
 * cdef char *ufunc_pro_ang1_cv_doc = (             # <<<<<<<<<<<<<<
 *     "pro_ang1_cv(m, n, c, cv, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_doc = ((char *)"pro_ang1_cv(m, n, c, cv, x)\n\nProlate spheroidal angular function pro_ang1 for precomputed characteristic value\n\nComputes the prolate spheroidal angular function of the first kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``. Requires\npre-computed characteristic value.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x");

  /* "scipy/special/_ufuncs.pyx":12582
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_pro_ang1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_pro_ang1_cv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_fffff_ff);

  /* "scipy/special/_ufuncs.pyx":12583
 *     "    Value of the derivative vs x")
 * ufunc_pro_ang1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_pro_ang1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_ddddd_dd);

  /* "scipy/special/_ufuncs.pyx":12584
 * ufunc_pro_ang1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_pro_ang1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_pro_ang1_cv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12585
 * ufunc_pro_ang1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_pro_ang1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12586
 * ufunc_pro_ang1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12587
 * ufunc_pro_ang1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12588
 * ufunc_pro_ang1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12589
 * ufunc_pro_ang1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12590
 * ufunc_pro_ang1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12591
 * ufunc_pro_ang1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12592
 * ufunc_pro_ang1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_pro_ang1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12593
 * ufunc_pro_ang1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12594
 * ufunc_pro_ang1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12595
 * ufunc_pro_ang1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12596
 * ufunc_pro_ang1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_ptr[2*0] = <void*>_func_prolate_aswfa_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12597
 * ufunc_pro_ang1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_ptr[2*0] = <void*>_func_prolate_aswfa_wrap
 * ufunc_pro_ang1_cv_ptr[2*0+1] = <void*>(<char*>"pro_ang1_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12598
 * ufunc_pro_ang1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_ptr[2*0] = <void*>_func_prolate_aswfa_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_ptr[2*0+1] = <void*>(<char*>"pro_ang1_cv")
 * ufunc_pro_ang1_cv_ptr[2*1] = <void*>_func_prolate_aswfa_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_ptr[0]) = ((void *)prolate_aswfa_wrap);

  /* "scipy/special/_ufuncs.pyx":12599
 * ufunc_pro_ang1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_pro_ang1_cv_ptr[2*0] = <void*>_func_prolate_aswfa_wrap
 * ufunc_pro_ang1_cv_ptr[2*0+1] = <void*>(<char*>"pro_ang1_cv")             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_ptr[2*1] = <void*>_func_prolate_aswfa_wrap
 * ufunc_pro_ang1_cv_ptr[2*1+1] = <void*>(<char*>"pro_ang1_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_ptr[1]) = ((void *)((char *)((char *)"pro_ang1_cv")));

  /* "scipy/special/_ufuncs.pyx":12600
 * ufunc_pro_ang1_cv_ptr[2*0] = <void*>_func_prolate_aswfa_wrap
 * ufunc_pro_ang1_cv_ptr[2*0+1] = <void*>(<char*>"pro_ang1_cv")
 * ufunc_pro_ang1_cv_ptr[2*1] = <void*>_func_prolate_aswfa_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_ptr[2*1+1] = <void*>(<char*>"pro_ang1_cv")
 * ufunc_pro_ang1_cv_data[0] = &ufunc_pro_ang1_cv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_ptr[2]) = ((void *)prolate_aswfa_wrap);

  /* "scipy/special/_ufuncs.pyx":12601
 * ufunc_pro_ang1_cv_ptr[2*0+1] = <void*>(<char*>"pro_ang1_cv")
 * ufunc_pro_ang1_cv_ptr[2*1] = <void*>_func_prolate_aswfa_wrap
 * ufunc_pro_ang1_cv_ptr[2*1+1] = <void*>(<char*>"pro_ang1_cv")             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_data[0] = &ufunc_pro_ang1_cv_ptr[2*0]
 * ufunc_pro_ang1_cv_data[1] = &ufunc_pro_ang1_cv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_ptr[3]) = ((void *)((char *)((char *)"pro_ang1_cv")));

  /* "scipy/special/_ufuncs.pyx":12602
 * ufunc_pro_ang1_cv_ptr[2*1] = <void*>_func_prolate_aswfa_wrap
 * ufunc_pro_ang1_cv_ptr[2*1+1] = <void*>(<char*>"pro_ang1_cv")
 * ufunc_pro_ang1_cv_data[0] = &ufunc_pro_ang1_cv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pro_ang1_cv_data[1] = &ufunc_pro_ang1_cv_ptr[2*1]
 * pro_ang1_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_ang1_cv_loops, ufunc_pro_ang1_cv_data, ufunc_pro_ang1_cv_types, 2, 5, 2, 0, "pro_ang1_cv", ufunc_pro_ang1_cv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12603
 * ufunc_pro_ang1_cv_ptr[2*1+1] = <void*>(<char*>"pro_ang1_cv")
 * ufunc_pro_ang1_cv_data[0] = &ufunc_pro_ang1_cv_ptr[2*0]
 * ufunc_pro_ang1_cv_data[1] = &ufunc_pro_ang1_cv_ptr[2*1]             # <<<<<<<<<<<<<<
 * pro_ang1_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_ang1_cv_loops, ufunc_pro_ang1_cv_data, ufunc_pro_ang1_cv_types, 2, 5, 2, 0, "pro_ang1_cv", ufunc_pro_ang1_cv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12604
 * ufunc_pro_ang1_cv_data[0] = &ufunc_pro_ang1_cv_ptr[2*0]
 * ufunc_pro_ang1_cv_data[1] = &ufunc_pro_ang1_cv_ptr[2*1]
 * pro_ang1_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_ang1_cv_loops, ufunc_pro_ang1_cv_data, ufunc_pro_ang1_cv_types, 2, 5, 2, 0, "pro_ang1_cv", ufunc_pro_ang1_cv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pro_cv_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_types, 2, 5, 2, 0, ((char *)"pro_ang1_cv"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_ang1_cv_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12604, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pro_ang1_cv, __pyx_t_2) < 0) __PYX_ERR(1, 12604, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12610
 * cdef void *ufunc_pro_cv_data[2]
 * cdef char ufunc_pro_cv_types[8]
 * cdef char *ufunc_pro_cv_doc = (             # <<<<<<<<<<<<<<
 *     "pro_cv(m, n, c)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_doc = ((char *)"pro_cv(m, n, c)\n\nCharacteristic value of prolate spheroidal function\n\nComputes the characteristic value of prolate spheroidal wave\nfunctions of order `m`, `n` (n>=m) and spheroidal parameter `c`.");

  /* "scipy/special/_ufuncs.pyx":12617
 *     "Computes the characteristic value of prolate spheroidal wave\n"
 *     "functions of order `m`, `n` (n>=m) and spheroidal parameter `c`.")
 * ufunc_pro_cv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_pro_cv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":12618
 *     "functions of order `m`, `n` (n>=m) and spheroidal parameter `c`.")
 * ufunc_pro_cv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_pro_cv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":12619
 * ufunc_pro_cv_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_pro_cv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_pro_cv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12620
 * ufunc_pro_cv_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_pro_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12621
 * ufunc_pro_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12622
 * ufunc_pro_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_types[4] = <char>NPY_DOUBLE
 * ufunc_pro_cv_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12623
 * ufunc_pro_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_types[5] = <char>NPY_DOUBLE
 * ufunc_pro_cv_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12624
 * ufunc_pro_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_cv_types[4] = <char>NPY_DOUBLE
 * ufunc_pro_cv_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_cv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12625
 * ufunc_pro_cv_types[4] = <char>NPY_DOUBLE
 * ufunc_pro_cv_types[5] = <char>NPY_DOUBLE
 * ufunc_pro_cv_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_cv_ptr[2*0] = <void*>_func_prolate_segv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12626
 * ufunc_pro_cv_types[5] = <char>NPY_DOUBLE
 * ufunc_pro_cv_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_cv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_ptr[2*0] = <void*>_func_prolate_segv_wrap
 * ufunc_pro_cv_ptr[2*0+1] = <void*>(<char*>"pro_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12627
 * ufunc_pro_cv_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_cv_ptr[2*0] = <void*>_func_prolate_segv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_ptr[2*0+1] = <void*>(<char*>"pro_cv")
 * ufunc_pro_cv_ptr[2*1] = <void*>_func_prolate_segv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_ptr[0]) = ((void *)prolate_segv_wrap);

  /* "scipy/special/_ufuncs.pyx":12628
 * ufunc_pro_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_cv_ptr[2*0] = <void*>_func_prolate_segv_wrap
 * ufunc_pro_cv_ptr[2*0+1] = <void*>(<char*>"pro_cv")             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_ptr[2*1] = <void*>_func_prolate_segv_wrap
 * ufunc_pro_cv_ptr[2*1+1] = <void*>(<char*>"pro_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_ptr[1]) = ((void *)((char *)((char *)"pro_cv")));

  /* "scipy/special/_ufuncs.pyx":12629
 * ufunc_pro_cv_ptr[2*0] = <void*>_func_prolate_segv_wrap
 * ufunc_pro_cv_ptr[2*0+1] = <void*>(<char*>"pro_cv")
 * ufunc_pro_cv_ptr[2*1] = <void*>_func_prolate_segv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_ptr[2*1+1] = <void*>(<char*>"pro_cv")
 * ufunc_pro_cv_data[0] = &ufunc_pro_cv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_ptr[2]) = ((void *)prolate_segv_wrap);

  /* "scipy/special/_ufuncs.pyx":12630
 * ufunc_pro_cv_ptr[2*0+1] = <void*>(<char*>"pro_cv")
 * ufunc_pro_cv_ptr[2*1] = <void*>_func_prolate_segv_wrap
 * ufunc_pro_cv_ptr[2*1+1] = <void*>(<char*>"pro_cv")             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_data[0] = &ufunc_pro_cv_ptr[2*0]
 * ufunc_pro_cv_data[1] = &ufunc_pro_cv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_ptr[3]) = ((void *)((char *)((char *)"pro_cv")));

  /* "scipy/special/_ufuncs.pyx":12631
 * ufunc_pro_cv_ptr[2*1] = <void*>_func_prolate_segv_wrap
 * ufunc_pro_cv_ptr[2*1+1] = <void*>(<char*>"pro_cv")
 * ufunc_pro_cv_data[0] = &ufunc_pro_cv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pro_cv_data[1] = &ufunc_pro_cv_ptr[2*1]
 * pro_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_cv_loops, ufunc_pro_cv_data, ufunc_pro_cv_types, 2, 3, 1, 0, "pro_cv", ufunc_pro_cv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12632
 * ufunc_pro_cv_ptr[2*1+1] = <void*>(<char*>"pro_cv")
 * ufunc_pro_cv_data[0] = &ufunc_pro_cv_ptr[2*0]
 * ufunc_pro_cv_data[1] = &ufunc_pro_cv_ptr[2*1]             # <<<<<<<<<<<<<<
 * pro_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_cv_loops, ufunc_pro_cv_data, ufunc_pro_cv_types, 2, 3, 1, 0, "pro_cv", ufunc_pro_cv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12633
 * ufunc_pro_cv_data[0] = &ufunc_pro_cv_ptr[2*0]
 * ufunc_pro_cv_data[1] = &ufunc_pro_cv_ptr[2*1]
 * pro_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_cv_loops, ufunc_pro_cv_data, ufunc_pro_cv_types, 2, 3, 1, 0, "pro_cv", ufunc_pro_cv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pro_rad1_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_types, 2, 3, 1, 0, ((char *)"pro_cv"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_cv_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12633, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pro_cv, __pyx_t_2) < 0) __PYX_ERR(1, 12633, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12639
 * cdef void *ufunc_pro_rad1_data[2]
 * cdef char ufunc_pro_rad1_types[12]
 * cdef char *ufunc_pro_rad1_doc = (             # <<<<<<<<<<<<<<
 *     "pro_rad1(m, n, c, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_doc = ((char *)"pro_rad1(m, n, c, x)\n\nProlate spheroidal radial function of the first kind and its derivative\n\nComputes the prolate spheroidal radial function of the first kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x");

  /* "scipy/special/_ufuncs.pyx":12654
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_pro_rad1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_pro_rad1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff);

  /* "scipy/special/_ufuncs.pyx":12655
 *     "    Value of the derivative vs x")
 * ufunc_pro_rad1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_pro_rad1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd);

  /* "scipy/special/_ufuncs.pyx":12656
 * ufunc_pro_rad1_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_pro_rad1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_pro_rad1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12657
 * ufunc_pro_rad1_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_pro_rad1_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12658
 * ufunc_pro_rad1_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12659
 * ufunc_pro_rad1_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12660
 * ufunc_pro_rad1_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12661
 * ufunc_pro_rad1_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12662
 * ufunc_pro_rad1_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12663
 * ufunc_pro_rad1_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad1_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12664
 * ufunc_pro_rad1_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12665
 * ufunc_pro_rad1_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12666
 * ufunc_pro_rad1_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_ptr[2*0] = <void*>_func_prolate_radial1_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12667
 * ufunc_pro_rad1_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_ptr[2*0] = <void*>_func_prolate_radial1_nocv_wrap
 * ufunc_pro_rad1_ptr[2*0+1] = <void*>(<char*>"pro_rad1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12668
 * ufunc_pro_rad1_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_ptr[2*0] = <void*>_func_prolate_radial1_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_ptr[2*0+1] = <void*>(<char*>"pro_rad1")
 * ufunc_pro_rad1_ptr[2*1] = <void*>_func_prolate_radial1_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_ptr[0]) = ((void *)prolate_radial1_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":12669
 * ufunc_pro_rad1_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_ptr[2*0] = <void*>_func_prolate_radial1_nocv_wrap
 * ufunc_pro_rad1_ptr[2*0+1] = <void*>(<char*>"pro_rad1")             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_ptr[2*1] = <void*>_func_prolate_radial1_nocv_wrap
 * ufunc_pro_rad1_ptr[2*1+1] = <void*>(<char*>"pro_rad1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_ptr[1]) = ((void *)((char *)((char *)"pro_rad1")));

  /* "scipy/special/_ufuncs.pyx":12670
 * ufunc_pro_rad1_ptr[2*0] = <void*>_func_prolate_radial1_nocv_wrap
 * ufunc_pro_rad1_ptr[2*0+1] = <void*>(<char*>"pro_rad1")
 * ufunc_pro_rad1_ptr[2*1] = <void*>_func_prolate_radial1_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_ptr[2*1+1] = <void*>(<char*>"pro_rad1")
 * ufunc_pro_rad1_data[0] = &ufunc_pro_rad1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_ptr[2]) = ((void *)prolate_radial1_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":12671
 * ufunc_pro_rad1_ptr[2*0+1] = <void*>(<char*>"pro_rad1")
 * ufunc_pro_rad1_ptr[2*1] = <void*>_func_prolate_radial1_nocv_wrap
 * ufunc_pro_rad1_ptr[2*1+1] = <void*>(<char*>"pro_rad1")             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_data[0] = &ufunc_pro_rad1_ptr[2*0]
 * ufunc_pro_rad1_data[1] = &ufunc_pro_rad1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_ptr[3]) = ((void *)((char *)((char *)"pro_rad1")));

  /* "scipy/special/_ufuncs.pyx":12672
 * ufunc_pro_rad1_ptr[2*1] = <void*>_func_prolate_radial1_nocv_wrap
 * ufunc_pro_rad1_ptr[2*1+1] = <void*>(<char*>"pro_rad1")
 * ufunc_pro_rad1_data[0] = &ufunc_pro_rad1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_data[1] = &ufunc_pro_rad1_ptr[2*1]
 * pro_rad1 = np.PyUFunc_FromFuncAndData(ufunc_pro_rad1_loops, ufunc_pro_rad1_data, ufunc_pro_rad1_types, 2, 4, 2, 0, "pro_rad1", ufunc_pro_rad1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12673
 * ufunc_pro_rad1_ptr[2*1+1] = <void*>(<char*>"pro_rad1")
 * ufunc_pro_rad1_data[0] = &ufunc_pro_rad1_ptr[2*0]
 * ufunc_pro_rad1_data[1] = &ufunc_pro_rad1_ptr[2*1]             # <<<<<<<<<<<<<<
 * pro_rad1 = np.PyUFunc_FromFuncAndData(ufunc_pro_rad1_loops, ufunc_pro_rad1_data, ufunc_pro_rad1_types, 2, 4, 2, 0, "pro_rad1", ufunc_pro_rad1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12674
 * ufunc_pro_rad1_data[0] = &ufunc_pro_rad1_ptr[2*0]
 * ufunc_pro_rad1_data[1] = &ufunc_pro_rad1_ptr[2*1]
 * pro_rad1 = np.PyUFunc_FromFuncAndData(ufunc_pro_rad1_loops, ufunc_pro_rad1_data, ufunc_pro_rad1_types, 2, 4, 2, 0, "pro_rad1", ufunc_pro_rad1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pro_rad1_cv_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_types, 2, 4, 2, 0, ((char *)"pro_rad1"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12674, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pro_rad1, __pyx_t_2) < 0) __PYX_ERR(1, 12674, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12680
 * cdef void *ufunc_pro_rad1_cv_data[2]
 * cdef char ufunc_pro_rad1_cv_types[14]
 * cdef char *ufunc_pro_rad1_cv_doc = (             # <<<<<<<<<<<<<<
 *     "pro_rad1_cv(m, n, c, cv, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_doc = ((char *)"pro_rad1_cv(m, n, c, cv, x)\n\nProlate spheroidal radial function pro_rad1 for precomputed characteristic value\n\nComputes the prolate spheroidal radial function of the first kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``. Requires\npre-computed characteristic value.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x");

  /* "scipy/special/_ufuncs.pyx":12696
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_pro_rad1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_pro_rad1_cv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_fffff_ff);

  /* "scipy/special/_ufuncs.pyx":12697
 *     "    Value of the derivative vs x")
 * ufunc_pro_rad1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_pro_rad1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_ddddd_dd);

  /* "scipy/special/_ufuncs.pyx":12698
 * ufunc_pro_rad1_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_pro_rad1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_pro_rad1_cv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12699
 * ufunc_pro_rad1_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_pro_rad1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12700
 * ufunc_pro_rad1_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12701
 * ufunc_pro_rad1_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12702
 * ufunc_pro_rad1_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12703
 * ufunc_pro_rad1_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12704
 * ufunc_pro_rad1_cv_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12705
 * ufunc_pro_rad1_cv_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12706
 * ufunc_pro_rad1_cv_types[6] = <char>NPY_FLOAT
 * ufunc_pro_rad1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12707
 * ufunc_pro_rad1_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12708
 * ufunc_pro_rad1_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12709
 * ufunc_pro_rad1_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12710
 * ufunc_pro_rad1_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_ptr[2*0] = <void*>_func_prolate_radial1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12711
 * ufunc_pro_rad1_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_ptr[2*0] = <void*>_func_prolate_radial1_wrap
 * ufunc_pro_rad1_cv_ptr[2*0+1] = <void*>(<char*>"pro_rad1_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12712
 * ufunc_pro_rad1_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_ptr[2*0] = <void*>_func_prolate_radial1_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_ptr[2*0+1] = <void*>(<char*>"pro_rad1_cv")
 * ufunc_pro_rad1_cv_ptr[2*1] = <void*>_func_prolate_radial1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_ptr[0]) = ((void *)prolate_radial1_wrap);

  /* "scipy/special/_ufuncs.pyx":12713
 * ufunc_pro_rad1_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_pro_rad1_cv_ptr[2*0] = <void*>_func_prolate_radial1_wrap
 * ufunc_pro_rad1_cv_ptr[2*0+1] = <void*>(<char*>"pro_rad1_cv")             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_ptr[2*1] = <void*>_func_prolate_radial1_wrap
 * ufunc_pro_rad1_cv_ptr[2*1+1] = <void*>(<char*>"pro_rad1_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_ptr[1]) = ((void *)((char *)((char *)"pro_rad1_cv")));

  /* "scipy/special/_ufuncs.pyx":12714
 * ufunc_pro_rad1_cv_ptr[2*0] = <void*>_func_prolate_radial1_wrap
 * ufunc_pro_rad1_cv_ptr[2*0+1] = <void*>(<char*>"pro_rad1_cv")
 * ufunc_pro_rad1_cv_ptr[2*1] = <void*>_func_prolate_radial1_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_ptr[2*1+1] = <void*>(<char*>"pro_rad1_cv")
 * ufunc_pro_rad1_cv_data[0] = &ufunc_pro_rad1_cv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_ptr[2]) = ((void *)prolate_radial1_wrap);

  /* "scipy/special/_ufuncs.pyx":12715
 * ufunc_pro_rad1_cv_ptr[2*0+1] = <void*>(<char*>"pro_rad1_cv")
 * ufunc_pro_rad1_cv_ptr[2*1] = <void*>_func_prolate_radial1_wrap
 * ufunc_pro_rad1_cv_ptr[2*1+1] = <void*>(<char*>"pro_rad1_cv")             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_data[0] = &ufunc_pro_rad1_cv_ptr[2*0]
 * ufunc_pro_rad1_cv_data[1] = &ufunc_pro_rad1_cv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_ptr[3]) = ((void *)((char *)((char *)"pro_rad1_cv")));

  /* "scipy/special/_ufuncs.pyx":12716
 * ufunc_pro_rad1_cv_ptr[2*1] = <void*>_func_prolate_radial1_wrap
 * ufunc_pro_rad1_cv_ptr[2*1+1] = <void*>(<char*>"pro_rad1_cv")
 * ufunc_pro_rad1_cv_data[0] = &ufunc_pro_rad1_cv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pro_rad1_cv_data[1] = &ufunc_pro_rad1_cv_ptr[2*1]
 * pro_rad1_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_rad1_cv_loops, ufunc_pro_rad1_cv_data, ufunc_pro_rad1_cv_types, 2, 5, 2, 0, "pro_rad1_cv", ufunc_pro_rad1_cv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12717
 * ufunc_pro_rad1_cv_ptr[2*1+1] = <void*>(<char*>"pro_rad1_cv")
 * ufunc_pro_rad1_cv_data[0] = &ufunc_pro_rad1_cv_ptr[2*0]
 * ufunc_pro_rad1_cv_data[1] = &ufunc_pro_rad1_cv_ptr[2*1]             # <<<<<<<<<<<<<<
 * pro_rad1_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_rad1_cv_loops, ufunc_pro_rad1_cv_data, ufunc_pro_rad1_cv_types, 2, 5, 2, 0, "pro_rad1_cv", ufunc_pro_rad1_cv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12718
 * ufunc_pro_rad1_cv_data[0] = &ufunc_pro_rad1_cv_ptr[2*0]
 * ufunc_pro_rad1_cv_data[1] = &ufunc_pro_rad1_cv_ptr[2*1]
 * pro_rad1_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_rad1_cv_loops, ufunc_pro_rad1_cv_data, ufunc_pro_rad1_cv_types, 2, 5, 2, 0, "pro_rad1_cv", ufunc_pro_rad1_cv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pro_rad2_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_types, 2, 5, 2, 0, ((char *)"pro_rad1_cv"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad1_cv_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12718, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pro_rad1_cv, __pyx_t_2) < 0) __PYX_ERR(1, 12718, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12724
 * cdef void *ufunc_pro_rad2_data[2]
 * cdef char ufunc_pro_rad2_types[12]
 * cdef char *ufunc_pro_rad2_doc = (             # <<<<<<<<<<<<<<
 *     "pro_rad2(m, n, c, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_doc = ((char *)"pro_rad2(m, n, c, x)\n\nProlate spheroidal radial function of the second kind and its derivative\n\nComputes the prolate spheroidal radial function of the second kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x");

  /* "scipy/special/_ufuncs.pyx":12739
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_pro_rad2_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_pro_rad2_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_ffff_ff);

  /* "scipy/special/_ufuncs.pyx":12740
 *     "    Value of the derivative vs x")
 * ufunc_pro_rad2_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_pro_rad2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dddd_d_As_dddd_dd);

  /* "scipy/special/_ufuncs.pyx":12741
 * ufunc_pro_rad2_loops[0] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_ffff_ff
 * ufunc_pro_rad2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_pro_rad2_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12742
 * ufunc_pro_rad2_loops[1] = <np.PyUFuncGenericFunction>loop_d_dddd_d_As_dddd_dd
 * ufunc_pro_rad2_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12743
 * ufunc_pro_rad2_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12744
 * ufunc_pro_rad2_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12745
 * ufunc_pro_rad2_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12746
 * ufunc_pro_rad2_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12747
 * ufunc_pro_rad2_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12748
 * ufunc_pro_rad2_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad2_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12749
 * ufunc_pro_rad2_types[6] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12750
 * ufunc_pro_rad2_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12751
 * ufunc_pro_rad2_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_ptr[2*0] = <void*>_func_prolate_radial2_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12752
 * ufunc_pro_rad2_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_ptr[2*0] = <void*>_func_prolate_radial2_nocv_wrap
 * ufunc_pro_rad2_ptr[2*0+1] = <void*>(<char*>"pro_rad2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12753
 * ufunc_pro_rad2_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_ptr[2*0] = <void*>_func_prolate_radial2_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_ptr[2*0+1] = <void*>(<char*>"pro_rad2")
 * ufunc_pro_rad2_ptr[2*1] = <void*>_func_prolate_radial2_nocv_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_ptr[0]) = ((void *)prolate_radial2_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":12754
 * ufunc_pro_rad2_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_ptr[2*0] = <void*>_func_prolate_radial2_nocv_wrap
 * ufunc_pro_rad2_ptr[2*0+1] = <void*>(<char*>"pro_rad2")             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_ptr[2*1] = <void*>_func_prolate_radial2_nocv_wrap
 * ufunc_pro_rad2_ptr[2*1+1] = <void*>(<char*>"pro_rad2")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_ptr[1]) = ((void *)((char *)((char *)"pro_rad2")));

  /* "scipy/special/_ufuncs.pyx":12755
 * ufunc_pro_rad2_ptr[2*0] = <void*>_func_prolate_radial2_nocv_wrap
 * ufunc_pro_rad2_ptr[2*0+1] = <void*>(<char*>"pro_rad2")
 * ufunc_pro_rad2_ptr[2*1] = <void*>_func_prolate_radial2_nocv_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_ptr[2*1+1] = <void*>(<char*>"pro_rad2")
 * ufunc_pro_rad2_data[0] = &ufunc_pro_rad2_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_ptr[2]) = ((void *)prolate_radial2_nocv_wrap);

  /* "scipy/special/_ufuncs.pyx":12756
 * ufunc_pro_rad2_ptr[2*0+1] = <void*>(<char*>"pro_rad2")
 * ufunc_pro_rad2_ptr[2*1] = <void*>_func_prolate_radial2_nocv_wrap
 * ufunc_pro_rad2_ptr[2*1+1] = <void*>(<char*>"pro_rad2")             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_data[0] = &ufunc_pro_rad2_ptr[2*0]
 * ufunc_pro_rad2_data[1] = &ufunc_pro_rad2_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_ptr[3]) = ((void *)((char *)((char *)"pro_rad2")));

  /* "scipy/special/_ufuncs.pyx":12757
 * ufunc_pro_rad2_ptr[2*1] = <void*>_func_prolate_radial2_nocv_wrap
 * ufunc_pro_rad2_ptr[2*1+1] = <void*>(<char*>"pro_rad2")
 * ufunc_pro_rad2_data[0] = &ufunc_pro_rad2_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_data[1] = &ufunc_pro_rad2_ptr[2*1]
 * pro_rad2 = np.PyUFunc_FromFuncAndData(ufunc_pro_rad2_loops, ufunc_pro_rad2_data, ufunc_pro_rad2_types, 2, 4, 2, 0, "pro_rad2", ufunc_pro_rad2_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12758
 * ufunc_pro_rad2_ptr[2*1+1] = <void*>(<char*>"pro_rad2")
 * ufunc_pro_rad2_data[0] = &ufunc_pro_rad2_ptr[2*0]
 * ufunc_pro_rad2_data[1] = &ufunc_pro_rad2_ptr[2*1]             # <<<<<<<<<<<<<<
 * pro_rad2 = np.PyUFunc_FromFuncAndData(ufunc_pro_rad2_loops, ufunc_pro_rad2_data, ufunc_pro_rad2_types, 2, 4, 2, 0, "pro_rad2", ufunc_pro_rad2_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12759
 * ufunc_pro_rad2_data[0] = &ufunc_pro_rad2_ptr[2*0]
 * ufunc_pro_rad2_data[1] = &ufunc_pro_rad2_ptr[2*1]
 * pro_rad2 = np.PyUFunc_FromFuncAndData(ufunc_pro_rad2_loops, ufunc_pro_rad2_data, ufunc_pro_rad2_types, 2, 4, 2, 0, "pro_rad2", ufunc_pro_rad2_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pro_rad2_cv_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_types, 2, 4, 2, 0, ((char *)"pro_rad2"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12759, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pro_rad2, __pyx_t_2) < 0) __PYX_ERR(1, 12759, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12765
 * cdef void *ufunc_pro_rad2_cv_data[2]
 * cdef char ufunc_pro_rad2_cv_types[14]
 * cdef char *ufunc_pro_rad2_cv_doc = (             # <<<<<<<<<<<<<<
 *     "pro_rad2_cv(m, n, c, cv, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_doc = ((char *)"pro_rad2_cv(m, n, c, cv, x)\n\nProlate spheroidal radial function pro_rad2 for precomputed characteristic value\n\nComputes the prolate spheroidal radial function of the second kind\nand its derivative (with respect to `x`) for mode parameters m>=0\nand n>=m, spheroidal parameter `c` and ``|x| < 1.0``. Requires\npre-computed characteristic value.\n\nReturns\n-------\ns\n    Value of the function\nsp\n    Value of the derivative vs x");

  /* "scipy/special/_ufuncs.pyx":12781
 *     "sp\n"
 *     "    Value of the derivative vs x")
 * ufunc_pro_rad2_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_pro_rad2_cv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_fffff_ff);

  /* "scipy/special/_ufuncs.pyx":12782
 *     "    Value of the derivative vs x")
 * ufunc_pro_rad2_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_pro_rad2_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_ddddd_dd_As_ddddd_dd);

  /* "scipy/special/_ufuncs.pyx":12783
 * ufunc_pro_rad2_cv_loops[0] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_fffff_ff
 * ufunc_pro_rad2_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_pro_rad2_cv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12784
 * ufunc_pro_rad2_cv_loops[1] = <np.PyUFuncGenericFunction>loop_i_ddddd_dd_As_ddddd_dd
 * ufunc_pro_rad2_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12785
 * ufunc_pro_rad2_cv_types[0] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12786
 * ufunc_pro_rad2_cv_types[1] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12787
 * ufunc_pro_rad2_cv_types[2] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12788
 * ufunc_pro_rad2_cv_types[3] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[6] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12789
 * ufunc_pro_rad2_cv_types[4] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12790
 * ufunc_pro_rad2_cv_types[5] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[6] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12791
 * ufunc_pro_rad2_cv_types[6] = <char>NPY_FLOAT
 * ufunc_pro_rad2_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12792
 * ufunc_pro_rad2_cv_types[7] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12793
 * ufunc_pro_rad2_cv_types[8] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12794
 * ufunc_pro_rad2_cv_types[9] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12795
 * ufunc_pro_rad2_cv_types[10] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_ptr[2*0] = <void*>_func_prolate_radial2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12796
 * ufunc_pro_rad2_cv_types[11] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_ptr[2*0] = <void*>_func_prolate_radial2_wrap
 * ufunc_pro_rad2_cv_ptr[2*0+1] = <void*>(<char*>"pro_rad2_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12797
 * ufunc_pro_rad2_cv_types[12] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_ptr[2*0] = <void*>_func_prolate_radial2_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_ptr[2*0+1] = <void*>(<char*>"pro_rad2_cv")
 * ufunc_pro_rad2_cv_ptr[2*1] = <void*>_func_prolate_radial2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_ptr[0]) = ((void *)prolate_radial2_wrap);

  /* "scipy/special/_ufuncs.pyx":12798
 * ufunc_pro_rad2_cv_types[13] = <char>NPY_DOUBLE
 * ufunc_pro_rad2_cv_ptr[2*0] = <void*>_func_prolate_radial2_wrap
 * ufunc_pro_rad2_cv_ptr[2*0+1] = <void*>(<char*>"pro_rad2_cv")             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_ptr[2*1] = <void*>_func_prolate_radial2_wrap
 * ufunc_pro_rad2_cv_ptr[2*1+1] = <void*>(<char*>"pro_rad2_cv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_ptr[1]) = ((void *)((char *)((char *)"pro_rad2_cv")));

  /* "scipy/special/_ufuncs.pyx":12799
 * ufunc_pro_rad2_cv_ptr[2*0] = <void*>_func_prolate_radial2_wrap
 * ufunc_pro_rad2_cv_ptr[2*0+1] = <void*>(<char*>"pro_rad2_cv")
 * ufunc_pro_rad2_cv_ptr[2*1] = <void*>_func_prolate_radial2_wrap             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_ptr[2*1+1] = <void*>(<char*>"pro_rad2_cv")
 * ufunc_pro_rad2_cv_data[0] = &ufunc_pro_rad2_cv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_ptr[2]) = ((void *)prolate_radial2_wrap);

  /* "scipy/special/_ufuncs.pyx":12800
 * ufunc_pro_rad2_cv_ptr[2*0+1] = <void*>(<char*>"pro_rad2_cv")
 * ufunc_pro_rad2_cv_ptr[2*1] = <void*>_func_prolate_radial2_wrap
 * ufunc_pro_rad2_cv_ptr[2*1+1] = <void*>(<char*>"pro_rad2_cv")             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_data[0] = &ufunc_pro_rad2_cv_ptr[2*0]
 * ufunc_pro_rad2_cv_data[1] = &ufunc_pro_rad2_cv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_ptr[3]) = ((void *)((char *)((char *)"pro_rad2_cv")));

  /* "scipy/special/_ufuncs.pyx":12801
 * ufunc_pro_rad2_cv_ptr[2*1] = <void*>_func_prolate_radial2_wrap
 * ufunc_pro_rad2_cv_ptr[2*1+1] = <void*>(<char*>"pro_rad2_cv")
 * ufunc_pro_rad2_cv_data[0] = &ufunc_pro_rad2_cv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pro_rad2_cv_data[1] = &ufunc_pro_rad2_cv_ptr[2*1]
 * pro_rad2_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_rad2_cv_loops, ufunc_pro_rad2_cv_data, ufunc_pro_rad2_cv_types, 2, 5, 2, 0, "pro_rad2_cv", ufunc_pro_rad2_cv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12802
 * ufunc_pro_rad2_cv_ptr[2*1+1] = <void*>(<char*>"pro_rad2_cv")
 * ufunc_pro_rad2_cv_data[0] = &ufunc_pro_rad2_cv_ptr[2*0]
 * ufunc_pro_rad2_cv_data[1] = &ufunc_pro_rad2_cv_ptr[2*1]             # <<<<<<<<<<<<<<
 * pro_rad2_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_rad2_cv_loops, ufunc_pro_rad2_cv_data, ufunc_pro_rad2_cv_types, 2, 5, 2, 0, "pro_rad2_cv", ufunc_pro_rad2_cv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12803
 * ufunc_pro_rad2_cv_data[0] = &ufunc_pro_rad2_cv_ptr[2*0]
 * ufunc_pro_rad2_cv_data[1] = &ufunc_pro_rad2_cv_ptr[2*1]
 * pro_rad2_cv = np.PyUFunc_FromFuncAndData(ufunc_pro_rad2_cv_loops, ufunc_pro_rad2_cv_data, ufunc_pro_rad2_cv_types, 2, 5, 2, 0, "pro_rad2_cv", ufunc_pro_rad2_cv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_pseudo_huber_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_types, 2, 5, 2, 0, ((char *)"pro_rad2_cv"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_pro_rad2_cv_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12803, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pro_rad2_cv, __pyx_t_2) < 0) __PYX_ERR(1, 12803, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12809
 * cdef void *ufunc_pseudo_huber_data[2]
 * cdef char ufunc_pseudo_huber_types[6]
 * cdef char *ufunc_pseudo_huber_doc = (             # <<<<<<<<<<<<<<
 *     "pseudo_huber(delta, r)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_doc = ((char *)"pseudo_huber(delta, r)\n\nPseudo-Huber loss function.\n\n.. math:: \\mathrm{pseudo\\_huber}(\\delta, r) = \\delta^2 \\left( \\sqrt{ 1 + \\left( \\frac{r}{\\delta} \\right)^2 } - 1 \\right)\n\nParameters\n----------\ndelta : ndarray\n    Input array, indicating the soft quadratic vs. linear loss changepoint.\nr : ndarray\n    Input array, possibly representing residuals.\n\nReturns\n-------\nres : ndarray\n    The computed Pseudo-Huber loss function values.\n\nNotes\n-----\nThis function is convex in :math:`r`.\n\n.. versionadded:: 0.15.0");

  /* "scipy/special/_ufuncs.pyx":12833
 *     "\n"
 *     ".. versionadded:: 0.15.0")
 * ufunc_pseudo_huber_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pseudo_huber_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":12834
 *     ".. versionadded:: 0.15.0")
 * ufunc_pseudo_huber_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pseudo_huber_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_types[0] = <char>NPY_FLOAT
 * ufunc_pseudo_huber_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":12835
 * ufunc_pseudo_huber_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_pseudo_huber_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pseudo_huber_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_types[1] = <char>NPY_FLOAT
 * ufunc_pseudo_huber_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12836
 * ufunc_pseudo_huber_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_pseudo_huber_types[0] = <char>NPY_FLOAT
 * ufunc_pseudo_huber_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_types[2] = <char>NPY_FLOAT
 * ufunc_pseudo_huber_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12837
 * ufunc_pseudo_huber_types[0] = <char>NPY_FLOAT
 * ufunc_pseudo_huber_types[1] = <char>NPY_FLOAT
 * ufunc_pseudo_huber_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_types[3] = <char>NPY_DOUBLE
 * ufunc_pseudo_huber_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12838
 * ufunc_pseudo_huber_types[1] = <char>NPY_FLOAT
 * ufunc_pseudo_huber_types[2] = <char>NPY_FLOAT
 * ufunc_pseudo_huber_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_types[4] = <char>NPY_DOUBLE
 * ufunc_pseudo_huber_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12839
 * ufunc_pseudo_huber_types[2] = <char>NPY_FLOAT
 * ufunc_pseudo_huber_types[3] = <char>NPY_DOUBLE
 * ufunc_pseudo_huber_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_types[5] = <char>NPY_DOUBLE
 * ufunc_pseudo_huber_ptr[2*0] = <void*>_func_pseudo_huber
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12840
 * ufunc_pseudo_huber_types[3] = <char>NPY_DOUBLE
 * ufunc_pseudo_huber_types[4] = <char>NPY_DOUBLE
 * ufunc_pseudo_huber_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_ptr[2*0] = <void*>_func_pseudo_huber
 * ufunc_pseudo_huber_ptr[2*0+1] = <void*>(<char*>"pseudo_huber")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12841
 * ufunc_pseudo_huber_types[4] = <char>NPY_DOUBLE
 * ufunc_pseudo_huber_types[5] = <char>NPY_DOUBLE
 * ufunc_pseudo_huber_ptr[2*0] = <void*>_func_pseudo_huber             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_ptr[2*0+1] = <void*>(<char*>"pseudo_huber")
 * ufunc_pseudo_huber_ptr[2*1] = <void*>_func_pseudo_huber
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_ptr[0]) = ((void *)__pyx_f_5scipy_7special_16_convex_analysis_pseudo_huber);

  /* "scipy/special/_ufuncs.pyx":12842
 * ufunc_pseudo_huber_types[5] = <char>NPY_DOUBLE
 * ufunc_pseudo_huber_ptr[2*0] = <void*>_func_pseudo_huber
 * ufunc_pseudo_huber_ptr[2*0+1] = <void*>(<char*>"pseudo_huber")             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_ptr[2*1] = <void*>_func_pseudo_huber
 * ufunc_pseudo_huber_ptr[2*1+1] = <void*>(<char*>"pseudo_huber")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_ptr[1]) = ((void *)((char *)((char *)"pseudo_huber")));

  /* "scipy/special/_ufuncs.pyx":12843
 * ufunc_pseudo_huber_ptr[2*0] = <void*>_func_pseudo_huber
 * ufunc_pseudo_huber_ptr[2*0+1] = <void*>(<char*>"pseudo_huber")
 * ufunc_pseudo_huber_ptr[2*1] = <void*>_func_pseudo_huber             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_ptr[2*1+1] = <void*>(<char*>"pseudo_huber")
 * ufunc_pseudo_huber_data[0] = &ufunc_pseudo_huber_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_ptr[2]) = ((void *)__pyx_f_5scipy_7special_16_convex_analysis_pseudo_huber);

  /* "scipy/special/_ufuncs.pyx":12844
 * ufunc_pseudo_huber_ptr[2*0+1] = <void*>(<char*>"pseudo_huber")
 * ufunc_pseudo_huber_ptr[2*1] = <void*>_func_pseudo_huber
 * ufunc_pseudo_huber_ptr[2*1+1] = <void*>(<char*>"pseudo_huber")             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_data[0] = &ufunc_pseudo_huber_ptr[2*0]
 * ufunc_pseudo_huber_data[1] = &ufunc_pseudo_huber_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_ptr[3]) = ((void *)((char *)((char *)"pseudo_huber")));

  /* "scipy/special/_ufuncs.pyx":12845
 * ufunc_pseudo_huber_ptr[2*1] = <void*>_func_pseudo_huber
 * ufunc_pseudo_huber_ptr[2*1+1] = <void*>(<char*>"pseudo_huber")
 * ufunc_pseudo_huber_data[0] = &ufunc_pseudo_huber_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_pseudo_huber_data[1] = &ufunc_pseudo_huber_ptr[2*1]
 * pseudo_huber = np.PyUFunc_FromFuncAndData(ufunc_pseudo_huber_loops, ufunc_pseudo_huber_data, ufunc_pseudo_huber_types, 2, 2, 1, 0, "pseudo_huber", ufunc_pseudo_huber_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12846
 * ufunc_pseudo_huber_ptr[2*1+1] = <void*>(<char*>"pseudo_huber")
 * ufunc_pseudo_huber_data[0] = &ufunc_pseudo_huber_ptr[2*0]
 * ufunc_pseudo_huber_data[1] = &ufunc_pseudo_huber_ptr[2*1]             # <<<<<<<<<<<<<<
 * pseudo_huber = np.PyUFunc_FromFuncAndData(ufunc_pseudo_huber_loops, ufunc_pseudo_huber_data, ufunc_pseudo_huber_types, 2, 2, 1, 0, "pseudo_huber", ufunc_pseudo_huber_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12847
 * ufunc_pseudo_huber_data[0] = &ufunc_pseudo_huber_ptr[2*0]
 * ufunc_pseudo_huber_data[1] = &ufunc_pseudo_huber_ptr[2*1]
 * pseudo_huber = np.PyUFunc_FromFuncAndData(ufunc_pseudo_huber_loops, ufunc_pseudo_huber_data, ufunc_pseudo_huber_types, 2, 2, 1, 0, "pseudo_huber", ufunc_pseudo_huber_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_psi_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_types, 2, 2, 1, 0, ((char *)"pseudo_huber"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_pseudo_huber_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12847, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pseudo_huber, __pyx_t_2) < 0) __PYX_ERR(1, 12847, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12853
 * cdef void *ufunc_psi_data[4]
 * cdef char ufunc_psi_types[8]
 * cdef char *ufunc_psi_doc = (             # <<<<<<<<<<<<<<
 *     "psi(z, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_doc = ((char *)"psi(z, out=None)\n\nThe digamma function.\n\nThe logarithmic derivative of the gamma function evaluated at ``z``.\n\nParameters\n----------\nz : array_like\n    Real or complex argument.\nout : ndarray, optional\n    Array for the computed values of ``psi``.\n\nReturns\n-------\ndigamma : ndarray\n    Computed values of ``psi``.\n\nNotes\n-----\nFor large values not close to the negative real axis ``psi`` is\ncomputed using the asymptotic series (5.11.2) from [1]_. For small\narguments not close to the negative real axis the recurrence\nrelation (5.5.2) from [1]_ is used until the argument is large\nenough to use the asymptotic series. For values close to the\nnegative real axis the reflection formula (5.5.4) from [1]_ is\nused first.  Note that ``psi`` has a family of zeros on the\nnegative real axis which occur between the poles at nonpositive\nintegers. Around the zeros the reflection formula suffers from\ncancellation and the implementation loses precision. The sole\npositive zero and the first negative zero, however, are handled\nseparately by precomputing series expansions using [2]_, so the\nfunction should maintain full accuracy around the origin.\n\nReferences\n----------\n.. [1] NIST Digital Library of Mathematical Functions\n       https://dlmf.nist.gov/5\n.. [2] Fredrik Johansson and others.\n       \"mpmath: a Python library for arbitrary-precision floating-point arithmetic\"\n       (Version 0.19) http://mpmath.org/");

  /* "scipy/special/_ufuncs.pyx":12895
 *     "       \"mpmath: a Python library for arbitrary-precision floating-point arithmetic\"\n"
 *     "       (Version 0.19) http://mpmath.org/")
 * ufunc_psi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_psi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_psi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":12896
 *     "       (Version 0.19) http://mpmath.org/")
 * ufunc_psi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_psi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_psi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_psi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":12897
 * ufunc_psi_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_psi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_psi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_psi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_psi_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":12898
 * ufunc_psi_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_psi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_psi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_psi_types[0] = <char>NPY_FLOAT
 * ufunc_psi_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":12899
 * ufunc_psi_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_psi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_psi_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_psi_types[1] = <char>NPY_FLOAT
 * ufunc_psi_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12900
 * ufunc_psi_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_psi_types[0] = <char>NPY_FLOAT
 * ufunc_psi_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_psi_types[2] = <char>NPY_DOUBLE
 * ufunc_psi_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12901
 * ufunc_psi_types[0] = <char>NPY_FLOAT
 * ufunc_psi_types[1] = <char>NPY_FLOAT
 * ufunc_psi_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_psi_types[3] = <char>NPY_DOUBLE
 * ufunc_psi_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12902
 * ufunc_psi_types[1] = <char>NPY_FLOAT
 * ufunc_psi_types[2] = <char>NPY_DOUBLE
 * ufunc_psi_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_psi_types[4] = <char>NPY_CFLOAT
 * ufunc_psi_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12903
 * ufunc_psi_types[2] = <char>NPY_DOUBLE
 * ufunc_psi_types[3] = <char>NPY_DOUBLE
 * ufunc_psi_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_psi_types[5] = <char>NPY_CFLOAT
 * ufunc_psi_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":12904
 * ufunc_psi_types[3] = <char>NPY_DOUBLE
 * ufunc_psi_types[4] = <char>NPY_CFLOAT
 * ufunc_psi_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_psi_types[6] = <char>NPY_CDOUBLE
 * ufunc_psi_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":12905
 * ufunc_psi_types[4] = <char>NPY_CFLOAT
 * ufunc_psi_types[5] = <char>NPY_CFLOAT
 * ufunc_psi_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_psi_types[7] = <char>NPY_CDOUBLE
 * ufunc_psi_ptr[2*0] = <void*>_func_digamma
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":12906
 * ufunc_psi_types[5] = <char>NPY_CFLOAT
 * ufunc_psi_types[6] = <char>NPY_CDOUBLE
 * ufunc_psi_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_psi_ptr[2*0] = <void*>_func_digamma
 * ufunc_psi_ptr[2*0+1] = <void*>(<char*>"psi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":12907
 * ufunc_psi_types[6] = <char>NPY_CDOUBLE
 * ufunc_psi_types[7] = <char>NPY_CDOUBLE
 * ufunc_psi_ptr[2*0] = <void*>_func_digamma             # <<<<<<<<<<<<<<
 * ufunc_psi_ptr[2*0+1] = <void*>(<char*>"psi")
 * ufunc_psi_ptr[2*1] = <void*>_func_digamma
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[0]) = ((void *)__pyx_f_5scipy_7special_8_digamma_digamma);

  /* "scipy/special/_ufuncs.pyx":12908
 * ufunc_psi_types[7] = <char>NPY_CDOUBLE
 * ufunc_psi_ptr[2*0] = <void*>_func_digamma
 * ufunc_psi_ptr[2*0+1] = <void*>(<char*>"psi")             # <<<<<<<<<<<<<<
 * ufunc_psi_ptr[2*1] = <void*>_func_digamma
 * ufunc_psi_ptr[2*1+1] = <void*>(<char*>"psi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[1]) = ((void *)((char *)((char *)"psi")));

  /* "scipy/special/_ufuncs.pyx":12909
 * ufunc_psi_ptr[2*0] = <void*>_func_digamma
 * ufunc_psi_ptr[2*0+1] = <void*>(<char*>"psi")
 * ufunc_psi_ptr[2*1] = <void*>_func_digamma             # <<<<<<<<<<<<<<
 * ufunc_psi_ptr[2*1+1] = <void*>(<char*>"psi")
 * ufunc_psi_ptr[2*2] = <void*>_func_cdigamma
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[2]) = ((void *)__pyx_f_5scipy_7special_8_digamma_digamma);

  /* "scipy/special/_ufuncs.pyx":12910
 * ufunc_psi_ptr[2*0+1] = <void*>(<char*>"psi")
 * ufunc_psi_ptr[2*1] = <void*>_func_digamma
 * ufunc_psi_ptr[2*1+1] = <void*>(<char*>"psi")             # <<<<<<<<<<<<<<
 * ufunc_psi_ptr[2*2] = <void*>_func_cdigamma
 * ufunc_psi_ptr[2*2+1] = <void*>(<char*>"psi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[3]) = ((void *)((char *)((char *)"psi")));

  /* "scipy/special/_ufuncs.pyx":12911
 * ufunc_psi_ptr[2*1] = <void*>_func_digamma
 * ufunc_psi_ptr[2*1+1] = <void*>(<char*>"psi")
 * ufunc_psi_ptr[2*2] = <void*>_func_cdigamma             # <<<<<<<<<<<<<<
 * ufunc_psi_ptr[2*2+1] = <void*>(<char*>"psi")
 * ufunc_psi_ptr[2*3] = <void*>_func_cdigamma
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[4]) = ((void *)__pyx_f_5scipy_7special_8_digamma_cdigamma);

  /* "scipy/special/_ufuncs.pyx":12912
 * ufunc_psi_ptr[2*1+1] = <void*>(<char*>"psi")
 * ufunc_psi_ptr[2*2] = <void*>_func_cdigamma
 * ufunc_psi_ptr[2*2+1] = <void*>(<char*>"psi")             # <<<<<<<<<<<<<<
 * ufunc_psi_ptr[2*3] = <void*>_func_cdigamma
 * ufunc_psi_ptr[2*3+1] = <void*>(<char*>"psi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[5]) = ((void *)((char *)((char *)"psi")));

  /* "scipy/special/_ufuncs.pyx":12913
 * ufunc_psi_ptr[2*2] = <void*>_func_cdigamma
 * ufunc_psi_ptr[2*2+1] = <void*>(<char*>"psi")
 * ufunc_psi_ptr[2*3] = <void*>_func_cdigamma             # <<<<<<<<<<<<<<
 * ufunc_psi_ptr[2*3+1] = <void*>(<char*>"psi")
 * ufunc_psi_data[0] = &ufunc_psi_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[6]) = ((void *)__pyx_f_5scipy_7special_8_digamma_cdigamma);

  /* "scipy/special/_ufuncs.pyx":12914
 * ufunc_psi_ptr[2*2+1] = <void*>(<char*>"psi")
 * ufunc_psi_ptr[2*3] = <void*>_func_cdigamma
 * ufunc_psi_ptr[2*3+1] = <void*>(<char*>"psi")             # <<<<<<<<<<<<<<
 * ufunc_psi_data[0] = &ufunc_psi_ptr[2*0]
 * ufunc_psi_data[1] = &ufunc_psi_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[7]) = ((void *)((char *)((char *)"psi")));

  /* "scipy/special/_ufuncs.pyx":12915
 * ufunc_psi_ptr[2*3] = <void*>_func_cdigamma
 * ufunc_psi_ptr[2*3+1] = <void*>(<char*>"psi")
 * ufunc_psi_data[0] = &ufunc_psi_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_psi_data[1] = &ufunc_psi_ptr[2*1]
 * ufunc_psi_data[2] = &ufunc_psi_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12916
 * ufunc_psi_ptr[2*3+1] = <void*>(<char*>"psi")
 * ufunc_psi_data[0] = &ufunc_psi_ptr[2*0]
 * ufunc_psi_data[1] = &ufunc_psi_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_psi_data[2] = &ufunc_psi_ptr[2*2]
 * ufunc_psi_data[3] = &ufunc_psi_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12917
 * ufunc_psi_data[0] = &ufunc_psi_ptr[2*0]
 * ufunc_psi_data[1] = &ufunc_psi_ptr[2*1]
 * ufunc_psi_data[2] = &ufunc_psi_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_psi_data[3] = &ufunc_psi_ptr[2*3]
 * psi = np.PyUFunc_FromFuncAndData(ufunc_psi_loops, ufunc_psi_data, ufunc_psi_types, 4, 1, 1, 0, "psi", ufunc_psi_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":12918
 * ufunc_psi_data[1] = &ufunc_psi_ptr[2*1]
 * ufunc_psi_data[2] = &ufunc_psi_ptr[2*2]
 * ufunc_psi_data[3] = &ufunc_psi_ptr[2*3]             # <<<<<<<<<<<<<<
 * psi = np.PyUFunc_FromFuncAndData(ufunc_psi_loops, ufunc_psi_data, ufunc_psi_types, 4, 1, 1, 0, "psi", ufunc_psi_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":12919
 * ufunc_psi_data[2] = &ufunc_psi_ptr[2*2]
 * ufunc_psi_data[3] = &ufunc_psi_ptr[2*3]
 * psi = np.PyUFunc_FromFuncAndData(ufunc_psi_loops, ufunc_psi_data, ufunc_psi_types, 4, 1, 1, 0, "psi", ufunc_psi_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_radian_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_types, 4, 1, 1, 0, ((char *)"psi"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_psi_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12919, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_psi, __pyx_t_2) < 0) __PYX_ERR(1, 12919, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12925
 * cdef void *ufunc_radian_data[2]
 * cdef char ufunc_radian_types[8]
 * cdef char *ufunc_radian_doc = (             # <<<<<<<<<<<<<<
 *     "radian(d, m, s)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_doc = ((char *)"radian(d, m, s)\n\nConvert from degrees to radians\n\nReturns the angle given in (d)egrees, (m)inutes, and (s)econds in\nradians.");

  /* "scipy/special/_ufuncs.pyx":12932
 *     "Returns the angle given in (d)egrees, (m)inutes, and (s)econds in\n"
 *     "radians.")
 * ufunc_radian_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f             # <<<<<<<<<<<<<<
 * ufunc_radian_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_radian_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_fff_f);

  /* "scipy/special/_ufuncs.pyx":12933
 *     "radians.")
 * ufunc_radian_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_radian_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d             # <<<<<<<<<<<<<<
 * ufunc_radian_types[0] = <char>NPY_FLOAT
 * ufunc_radian_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_ddd__As_ddd_d);

  /* "scipy/special/_ufuncs.pyx":12934
 * ufunc_radian_loops[0] = <np.PyUFuncGenericFunction>loop_d_ddd__As_fff_f
 * ufunc_radian_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_radian_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_radian_types[1] = <char>NPY_FLOAT
 * ufunc_radian_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12935
 * ufunc_radian_loops[1] = <np.PyUFuncGenericFunction>loop_d_ddd__As_ddd_d
 * ufunc_radian_types[0] = <char>NPY_FLOAT
 * ufunc_radian_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_radian_types[2] = <char>NPY_FLOAT
 * ufunc_radian_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12936
 * ufunc_radian_types[0] = <char>NPY_FLOAT
 * ufunc_radian_types[1] = <char>NPY_FLOAT
 * ufunc_radian_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_radian_types[3] = <char>NPY_FLOAT
 * ufunc_radian_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12937
 * ufunc_radian_types[1] = <char>NPY_FLOAT
 * ufunc_radian_types[2] = <char>NPY_FLOAT
 * ufunc_radian_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_radian_types[4] = <char>NPY_DOUBLE
 * ufunc_radian_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12938
 * ufunc_radian_types[2] = <char>NPY_FLOAT
 * ufunc_radian_types[3] = <char>NPY_FLOAT
 * ufunc_radian_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_radian_types[5] = <char>NPY_DOUBLE
 * ufunc_radian_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12939
 * ufunc_radian_types[3] = <char>NPY_FLOAT
 * ufunc_radian_types[4] = <char>NPY_DOUBLE
 * ufunc_radian_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_radian_types[6] = <char>NPY_DOUBLE
 * ufunc_radian_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12940
 * ufunc_radian_types[4] = <char>NPY_DOUBLE
 * ufunc_radian_types[5] = <char>NPY_DOUBLE
 * ufunc_radian_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_radian_types[7] = <char>NPY_DOUBLE
 * ufunc_radian_ptr[2*0] = <void*>_func_radian
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12941
 * ufunc_radian_types[5] = <char>NPY_DOUBLE
 * ufunc_radian_types[6] = <char>NPY_DOUBLE
 * ufunc_radian_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_radian_ptr[2*0] = <void*>_func_radian
 * ufunc_radian_ptr[2*0+1] = <void*>(<char*>"radian")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12942
 * ufunc_radian_types[6] = <char>NPY_DOUBLE
 * ufunc_radian_types[7] = <char>NPY_DOUBLE
 * ufunc_radian_ptr[2*0] = <void*>_func_radian             # <<<<<<<<<<<<<<
 * ufunc_radian_ptr[2*0+1] = <void*>(<char*>"radian")
 * ufunc_radian_ptr[2*1] = <void*>_func_radian
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_ptr[0]) = ((void *)radian);

  /* "scipy/special/_ufuncs.pyx":12943
 * ufunc_radian_types[7] = <char>NPY_DOUBLE
 * ufunc_radian_ptr[2*0] = <void*>_func_radian
 * ufunc_radian_ptr[2*0+1] = <void*>(<char*>"radian")             # <<<<<<<<<<<<<<
 * ufunc_radian_ptr[2*1] = <void*>_func_radian
 * ufunc_radian_ptr[2*1+1] = <void*>(<char*>"radian")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_ptr[1]) = ((void *)((char *)((char *)"radian")));

  /* "scipy/special/_ufuncs.pyx":12944
 * ufunc_radian_ptr[2*0] = <void*>_func_radian
 * ufunc_radian_ptr[2*0+1] = <void*>(<char*>"radian")
 * ufunc_radian_ptr[2*1] = <void*>_func_radian             # <<<<<<<<<<<<<<
 * ufunc_radian_ptr[2*1+1] = <void*>(<char*>"radian")
 * ufunc_radian_data[0] = &ufunc_radian_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_ptr[2]) = ((void *)radian);

  /* "scipy/special/_ufuncs.pyx":12945
 * ufunc_radian_ptr[2*0+1] = <void*>(<char*>"radian")
 * ufunc_radian_ptr[2*1] = <void*>_func_radian
 * ufunc_radian_ptr[2*1+1] = <void*>(<char*>"radian")             # <<<<<<<<<<<<<<
 * ufunc_radian_data[0] = &ufunc_radian_ptr[2*0]
 * ufunc_radian_data[1] = &ufunc_radian_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_ptr[3]) = ((void *)((char *)((char *)"radian")));

  /* "scipy/special/_ufuncs.pyx":12946
 * ufunc_radian_ptr[2*1] = <void*>_func_radian
 * ufunc_radian_ptr[2*1+1] = <void*>(<char*>"radian")
 * ufunc_radian_data[0] = &ufunc_radian_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_radian_data[1] = &ufunc_radian_ptr[2*1]
 * radian = np.PyUFunc_FromFuncAndData(ufunc_radian_loops, ufunc_radian_data, ufunc_radian_types, 2, 3, 1, 0, "radian", ufunc_radian_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12947
 * ufunc_radian_ptr[2*1+1] = <void*>(<char*>"radian")
 * ufunc_radian_data[0] = &ufunc_radian_ptr[2*0]
 * ufunc_radian_data[1] = &ufunc_radian_ptr[2*1]             # <<<<<<<<<<<<<<
 * radian = np.PyUFunc_FromFuncAndData(ufunc_radian_loops, ufunc_radian_data, ufunc_radian_types, 2, 3, 1, 0, "radian", ufunc_radian_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12948
 * ufunc_radian_data[0] = &ufunc_radian_ptr[2*0]
 * ufunc_radian_data[1] = &ufunc_radian_ptr[2*1]
 * radian = np.PyUFunc_FromFuncAndData(ufunc_radian_loops, ufunc_radian_data, ufunc_radian_types, 2, 3, 1, 0, "radian", ufunc_radian_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_rel_entr_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_types, 2, 3, 1, 0, ((char *)"radian"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_radian_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12948, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_radian, __pyx_t_2) < 0) __PYX_ERR(1, 12948, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":12954
 * cdef void *ufunc_rel_entr_data[2]
 * cdef char ufunc_rel_entr_types[6]
 * cdef char *ufunc_rel_entr_doc = (             # <<<<<<<<<<<<<<
 *     "rel_entr(x, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_doc = ((char *)"rel_entr(x, y)\n\nElementwise function for computing relative entropy.\n\n.. math:: \\mathrm{rel\\_entr}(x, y) = \\begin{cases} x \\log(x / y) & x > 0, y > 0 \\\\ 0 & x = 0, y \\ge 0 \\\\ \\infty & \\text{otherwise} \\end{cases}\n\nParameters\n----------\nx : ndarray\n    First input array.\ny : ndarray\n    Second input array.\n\nReturns\n-------\nres : ndarray\n    Output array.\n\nSee Also\n--------\nentr, kl_div\n\nNotes\n-----\nThis function is jointly convex in x and y.\n\n.. versionadded:: 0.15.0");

  /* "scipy/special/_ufuncs.pyx":12982
 *     "\n"
 *     ".. versionadded:: 0.15.0")
 * ufunc_rel_entr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_rel_entr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":12983
 *     ".. versionadded:: 0.15.0")
 * ufunc_rel_entr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_rel_entr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_types[0] = <char>NPY_FLOAT
 * ufunc_rel_entr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":12984
 * ufunc_rel_entr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_rel_entr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_rel_entr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_types[1] = <char>NPY_FLOAT
 * ufunc_rel_entr_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12985
 * ufunc_rel_entr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_rel_entr_types[0] = <char>NPY_FLOAT
 * ufunc_rel_entr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_types[2] = <char>NPY_FLOAT
 * ufunc_rel_entr_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12986
 * ufunc_rel_entr_types[0] = <char>NPY_FLOAT
 * ufunc_rel_entr_types[1] = <char>NPY_FLOAT
 * ufunc_rel_entr_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_types[3] = <char>NPY_DOUBLE
 * ufunc_rel_entr_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":12987
 * ufunc_rel_entr_types[1] = <char>NPY_FLOAT
 * ufunc_rel_entr_types[2] = <char>NPY_FLOAT
 * ufunc_rel_entr_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_types[4] = <char>NPY_DOUBLE
 * ufunc_rel_entr_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12988
 * ufunc_rel_entr_types[2] = <char>NPY_FLOAT
 * ufunc_rel_entr_types[3] = <char>NPY_DOUBLE
 * ufunc_rel_entr_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_types[5] = <char>NPY_DOUBLE
 * ufunc_rel_entr_ptr[2*0] = <void*>_func_rel_entr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12989
 * ufunc_rel_entr_types[3] = <char>NPY_DOUBLE
 * ufunc_rel_entr_types[4] = <char>NPY_DOUBLE
 * ufunc_rel_entr_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_ptr[2*0] = <void*>_func_rel_entr
 * ufunc_rel_entr_ptr[2*0+1] = <void*>(<char*>"rel_entr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":12990
 * ufunc_rel_entr_types[4] = <char>NPY_DOUBLE
 * ufunc_rel_entr_types[5] = <char>NPY_DOUBLE
 * ufunc_rel_entr_ptr[2*0] = <void*>_func_rel_entr             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_ptr[2*0+1] = <void*>(<char*>"rel_entr")
 * ufunc_rel_entr_ptr[2*1] = <void*>_func_rel_entr
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_ptr[0]) = ((void *)__pyx_f_5scipy_7special_16_convex_analysis_rel_entr);

  /* "scipy/special/_ufuncs.pyx":12991
 * ufunc_rel_entr_types[5] = <char>NPY_DOUBLE
 * ufunc_rel_entr_ptr[2*0] = <void*>_func_rel_entr
 * ufunc_rel_entr_ptr[2*0+1] = <void*>(<char*>"rel_entr")             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_ptr[2*1] = <void*>_func_rel_entr
 * ufunc_rel_entr_ptr[2*1+1] = <void*>(<char*>"rel_entr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_ptr[1]) = ((void *)((char *)((char *)"rel_entr")));

  /* "scipy/special/_ufuncs.pyx":12992
 * ufunc_rel_entr_ptr[2*0] = <void*>_func_rel_entr
 * ufunc_rel_entr_ptr[2*0+1] = <void*>(<char*>"rel_entr")
 * ufunc_rel_entr_ptr[2*1] = <void*>_func_rel_entr             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_ptr[2*1+1] = <void*>(<char*>"rel_entr")
 * ufunc_rel_entr_data[0] = &ufunc_rel_entr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_ptr[2]) = ((void *)__pyx_f_5scipy_7special_16_convex_analysis_rel_entr);

  /* "scipy/special/_ufuncs.pyx":12993
 * ufunc_rel_entr_ptr[2*0+1] = <void*>(<char*>"rel_entr")
 * ufunc_rel_entr_ptr[2*1] = <void*>_func_rel_entr
 * ufunc_rel_entr_ptr[2*1+1] = <void*>(<char*>"rel_entr")             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_data[0] = &ufunc_rel_entr_ptr[2*0]
 * ufunc_rel_entr_data[1] = &ufunc_rel_entr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_ptr[3]) = ((void *)((char *)((char *)"rel_entr")));

  /* "scipy/special/_ufuncs.pyx":12994
 * ufunc_rel_entr_ptr[2*1] = <void*>_func_rel_entr
 * ufunc_rel_entr_ptr[2*1+1] = <void*>(<char*>"rel_entr")
 * ufunc_rel_entr_data[0] = &ufunc_rel_entr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_rel_entr_data[1] = &ufunc_rel_entr_ptr[2*1]
 * rel_entr = np.PyUFunc_FromFuncAndData(ufunc_rel_entr_loops, ufunc_rel_entr_data, ufunc_rel_entr_types, 2, 2, 1, 0, "rel_entr", ufunc_rel_entr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":12995
 * ufunc_rel_entr_ptr[2*1+1] = <void*>(<char*>"rel_entr")
 * ufunc_rel_entr_data[0] = &ufunc_rel_entr_ptr[2*0]
 * ufunc_rel_entr_data[1] = &ufunc_rel_entr_ptr[2*1]             # <<<<<<<<<<<<<<
 * rel_entr = np.PyUFunc_FromFuncAndData(ufunc_rel_entr_loops, ufunc_rel_entr_data, ufunc_rel_entr_types, 2, 2, 1, 0, "rel_entr", ufunc_rel_entr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":12996
 * ufunc_rel_entr_data[0] = &ufunc_rel_entr_ptr[2*0]
 * ufunc_rel_entr_data[1] = &ufunc_rel_entr_ptr[2*1]
 * rel_entr = np.PyUFunc_FromFuncAndData(ufunc_rel_entr_loops, ufunc_rel_entr_data, ufunc_rel_entr_types, 2, 2, 1, 0, "rel_entr", ufunc_rel_entr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_rgamma_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_types, 2, 2, 1, 0, ((char *)"rel_entr"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_rel_entr_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12996, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_rel_entr, __pyx_t_2) < 0) __PYX_ERR(1, 12996, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":13002
 * cdef void *ufunc_rgamma_data[4]
 * cdef char ufunc_rgamma_types[8]
 * cdef char *ufunc_rgamma_doc = (             # <<<<<<<<<<<<<<
 *     "rgamma(z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_doc = ((char *)"rgamma(z)\n\nGamma function inverted\n\nReturns ``1/gamma(x)``");

  /* "scipy/special/_ufuncs.pyx":13008
 *     "\n"
 *     "Returns ``1/gamma(x)``")
 * ufunc_rgamma_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_rgamma_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_rgamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":13009
 *     "Returns ``1/gamma(x)``")
 * ufunc_rgamma_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_rgamma_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_rgamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_rgamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":13010
 * ufunc_rgamma_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_rgamma_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_rgamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_rgamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_rgamma_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":13011
 * ufunc_rgamma_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_rgamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_rgamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_rgamma_types[0] = <char>NPY_FLOAT
 * ufunc_rgamma_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":13012
 * ufunc_rgamma_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_rgamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_rgamma_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_rgamma_types[1] = <char>NPY_FLOAT
 * ufunc_rgamma_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13013
 * ufunc_rgamma_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_rgamma_types[0] = <char>NPY_FLOAT
 * ufunc_rgamma_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_rgamma_types[2] = <char>NPY_DOUBLE
 * ufunc_rgamma_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13014
 * ufunc_rgamma_types[0] = <char>NPY_FLOAT
 * ufunc_rgamma_types[1] = <char>NPY_FLOAT
 * ufunc_rgamma_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_rgamma_types[3] = <char>NPY_DOUBLE
 * ufunc_rgamma_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13015
 * ufunc_rgamma_types[1] = <char>NPY_FLOAT
 * ufunc_rgamma_types[2] = <char>NPY_DOUBLE
 * ufunc_rgamma_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_rgamma_types[4] = <char>NPY_CFLOAT
 * ufunc_rgamma_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13016
 * ufunc_rgamma_types[2] = <char>NPY_DOUBLE
 * ufunc_rgamma_types[3] = <char>NPY_DOUBLE
 * ufunc_rgamma_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_rgamma_types[5] = <char>NPY_CFLOAT
 * ufunc_rgamma_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13017
 * ufunc_rgamma_types[3] = <char>NPY_DOUBLE
 * ufunc_rgamma_types[4] = <char>NPY_CFLOAT
 * ufunc_rgamma_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_rgamma_types[6] = <char>NPY_CDOUBLE
 * ufunc_rgamma_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13018
 * ufunc_rgamma_types[4] = <char>NPY_CFLOAT
 * ufunc_rgamma_types[5] = <char>NPY_CFLOAT
 * ufunc_rgamma_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_rgamma_types[7] = <char>NPY_CDOUBLE
 * ufunc_rgamma_ptr[2*0] = <void*>_func_rgamma
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13019
 * ufunc_rgamma_types[5] = <char>NPY_CFLOAT
 * ufunc_rgamma_types[6] = <char>NPY_CDOUBLE
 * ufunc_rgamma_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_rgamma_ptr[2*0] = <void*>_func_rgamma
 * ufunc_rgamma_ptr[2*0+1] = <void*>(<char*>"rgamma")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13020
 * ufunc_rgamma_types[6] = <char>NPY_CDOUBLE
 * ufunc_rgamma_types[7] = <char>NPY_CDOUBLE
 * ufunc_rgamma_ptr[2*0] = <void*>_func_rgamma             # <<<<<<<<<<<<<<
 * ufunc_rgamma_ptr[2*0+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_ptr[2*1] = <void*>_func_rgamma
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[0]) = ((void *)rgamma);

  /* "scipy/special/_ufuncs.pyx":13021
 * ufunc_rgamma_types[7] = <char>NPY_CDOUBLE
 * ufunc_rgamma_ptr[2*0] = <void*>_func_rgamma
 * ufunc_rgamma_ptr[2*0+1] = <void*>(<char*>"rgamma")             # <<<<<<<<<<<<<<
 * ufunc_rgamma_ptr[2*1] = <void*>_func_rgamma
 * ufunc_rgamma_ptr[2*1+1] = <void*>(<char*>"rgamma")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[1]) = ((void *)((char *)((char *)"rgamma")));

  /* "scipy/special/_ufuncs.pyx":13022
 * ufunc_rgamma_ptr[2*0] = <void*>_func_rgamma
 * ufunc_rgamma_ptr[2*0+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_ptr[2*1] = <void*>_func_rgamma             # <<<<<<<<<<<<<<
 * ufunc_rgamma_ptr[2*1+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_ptr[2*2] = <void*>_func_crgamma
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[2]) = ((void *)rgamma);

  /* "scipy/special/_ufuncs.pyx":13023
 * ufunc_rgamma_ptr[2*0+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_ptr[2*1] = <void*>_func_rgamma
 * ufunc_rgamma_ptr[2*1+1] = <void*>(<char*>"rgamma")             # <<<<<<<<<<<<<<
 * ufunc_rgamma_ptr[2*2] = <void*>_func_crgamma
 * ufunc_rgamma_ptr[2*2+1] = <void*>(<char*>"rgamma")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[3]) = ((void *)((char *)((char *)"rgamma")));

  /* "scipy/special/_ufuncs.pyx":13024
 * ufunc_rgamma_ptr[2*1] = <void*>_func_rgamma
 * ufunc_rgamma_ptr[2*1+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_ptr[2*2] = <void*>_func_crgamma             # <<<<<<<<<<<<<<
 * ufunc_rgamma_ptr[2*2+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_ptr[2*3] = <void*>_func_crgamma
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[4]) = ((void *)__pyx_f_5scipy_7special_9_loggamma_crgamma);

  /* "scipy/special/_ufuncs.pyx":13025
 * ufunc_rgamma_ptr[2*1+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_ptr[2*2] = <void*>_func_crgamma
 * ufunc_rgamma_ptr[2*2+1] = <void*>(<char*>"rgamma")             # <<<<<<<<<<<<<<
 * ufunc_rgamma_ptr[2*3] = <void*>_func_crgamma
 * ufunc_rgamma_ptr[2*3+1] = <void*>(<char*>"rgamma")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[5]) = ((void *)((char *)((char *)"rgamma")));

  /* "scipy/special/_ufuncs.pyx":13026
 * ufunc_rgamma_ptr[2*2] = <void*>_func_crgamma
 * ufunc_rgamma_ptr[2*2+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_ptr[2*3] = <void*>_func_crgamma             # <<<<<<<<<<<<<<
 * ufunc_rgamma_ptr[2*3+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_data[0] = &ufunc_rgamma_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[6]) = ((void *)__pyx_f_5scipy_7special_9_loggamma_crgamma);

  /* "scipy/special/_ufuncs.pyx":13027
 * ufunc_rgamma_ptr[2*2+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_ptr[2*3] = <void*>_func_crgamma
 * ufunc_rgamma_ptr[2*3+1] = <void*>(<char*>"rgamma")             # <<<<<<<<<<<<<<
 * ufunc_rgamma_data[0] = &ufunc_rgamma_ptr[2*0]
 * ufunc_rgamma_data[1] = &ufunc_rgamma_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[7]) = ((void *)((char *)((char *)"rgamma")));

  /* "scipy/special/_ufuncs.pyx":13028
 * ufunc_rgamma_ptr[2*3] = <void*>_func_crgamma
 * ufunc_rgamma_ptr[2*3+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_data[0] = &ufunc_rgamma_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_rgamma_data[1] = &ufunc_rgamma_ptr[2*1]
 * ufunc_rgamma_data[2] = &ufunc_rgamma_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":13029
 * ufunc_rgamma_ptr[2*3+1] = <void*>(<char*>"rgamma")
 * ufunc_rgamma_data[0] = &ufunc_rgamma_ptr[2*0]
 * ufunc_rgamma_data[1] = &ufunc_rgamma_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_rgamma_data[2] = &ufunc_rgamma_ptr[2*2]
 * ufunc_rgamma_data[3] = &ufunc_rgamma_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":13030
 * ufunc_rgamma_data[0] = &ufunc_rgamma_ptr[2*0]
 * ufunc_rgamma_data[1] = &ufunc_rgamma_ptr[2*1]
 * ufunc_rgamma_data[2] = &ufunc_rgamma_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_rgamma_data[3] = &ufunc_rgamma_ptr[2*3]
 * rgamma = np.PyUFunc_FromFuncAndData(ufunc_rgamma_loops, ufunc_rgamma_data, ufunc_rgamma_types, 4, 1, 1, 0, "rgamma", ufunc_rgamma_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":13031
 * ufunc_rgamma_data[1] = &ufunc_rgamma_ptr[2*1]
 * ufunc_rgamma_data[2] = &ufunc_rgamma_ptr[2*2]
 * ufunc_rgamma_data[3] = &ufunc_rgamma_ptr[2*3]             # <<<<<<<<<<<<<<
 * rgamma = np.PyUFunc_FromFuncAndData(ufunc_rgamma_loops, ufunc_rgamma_data, ufunc_rgamma_types, 4, 1, 1, 0, "rgamma", ufunc_rgamma_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":13032
 * ufunc_rgamma_data[2] = &ufunc_rgamma_ptr[2*2]
 * ufunc_rgamma_data[3] = &ufunc_rgamma_ptr[2*3]
 * rgamma = np.PyUFunc_FromFuncAndData(ufunc_rgamma_loops, ufunc_rgamma_data, ufunc_rgamma_types, 4, 1, 1, 0, "rgamma", ufunc_rgamma_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_round_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_types, 4, 1, 1, 0, ((char *)"rgamma"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_rgamma_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13032, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_rgamma, __pyx_t_2) < 0) __PYX_ERR(1, 13032, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":13038
 * cdef void *ufunc_round_data[2]
 * cdef char ufunc_round_types[4]
 * cdef char *ufunc_round_doc = (             # <<<<<<<<<<<<<<
 *     "round(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_round_doc = ((char *)"round(x)\n\nRound to nearest integer\n\nReturns the nearest integer to `x` as a double precision floating\npoint result.  If `x` ends in 0.5 exactly, the nearest even integer\nis chosen.");

  /* "scipy/special/_ufuncs.pyx":13046
 *     "point result.  If `x` ends in 0.5 exactly, the nearest even integer\n"
 *     "is chosen.")
 * ufunc_round_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_round_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_round_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":13047
 *     "is chosen.")
 * ufunc_round_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_round_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_round_types[0] = <char>NPY_FLOAT
 * ufunc_round_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":13048
 * ufunc_round_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_round_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_round_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_round_types[1] = <char>NPY_FLOAT
 * ufunc_round_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13049
 * ufunc_round_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_round_types[0] = <char>NPY_FLOAT
 * ufunc_round_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_round_types[2] = <char>NPY_DOUBLE
 * ufunc_round_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13050
 * ufunc_round_types[0] = <char>NPY_FLOAT
 * ufunc_round_types[1] = <char>NPY_FLOAT
 * ufunc_round_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_round_types[3] = <char>NPY_DOUBLE
 * ufunc_round_ptr[2*0] = <void*>_func_round
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13051
 * ufunc_round_types[1] = <char>NPY_FLOAT
 * ufunc_round_types[2] = <char>NPY_DOUBLE
 * ufunc_round_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_round_ptr[2*0] = <void*>_func_round
 * ufunc_round_ptr[2*0+1] = <void*>(<char*>"round")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13052
 * ufunc_round_types[2] = <char>NPY_DOUBLE
 * ufunc_round_types[3] = <char>NPY_DOUBLE
 * ufunc_round_ptr[2*0] = <void*>_func_round             # <<<<<<<<<<<<<<
 * ufunc_round_ptr[2*0+1] = <void*>(<char*>"round")
 * ufunc_round_ptr[2*1] = <void*>_func_round
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_ptr[0]) = ((void *)round);

  /* "scipy/special/_ufuncs.pyx":13053
 * ufunc_round_types[3] = <char>NPY_DOUBLE
 * ufunc_round_ptr[2*0] = <void*>_func_round
 * ufunc_round_ptr[2*0+1] = <void*>(<char*>"round")             # <<<<<<<<<<<<<<
 * ufunc_round_ptr[2*1] = <void*>_func_round
 * ufunc_round_ptr[2*1+1] = <void*>(<char*>"round")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_ptr[1]) = ((void *)((char *)((char *)"round")));

  /* "scipy/special/_ufuncs.pyx":13054
 * ufunc_round_ptr[2*0] = <void*>_func_round
 * ufunc_round_ptr[2*0+1] = <void*>(<char*>"round")
 * ufunc_round_ptr[2*1] = <void*>_func_round             # <<<<<<<<<<<<<<
 * ufunc_round_ptr[2*1+1] = <void*>(<char*>"round")
 * ufunc_round_data[0] = &ufunc_round_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_ptr[2]) = ((void *)round);

  /* "scipy/special/_ufuncs.pyx":13055
 * ufunc_round_ptr[2*0+1] = <void*>(<char*>"round")
 * ufunc_round_ptr[2*1] = <void*>_func_round
 * ufunc_round_ptr[2*1+1] = <void*>(<char*>"round")             # <<<<<<<<<<<<<<
 * ufunc_round_data[0] = &ufunc_round_ptr[2*0]
 * ufunc_round_data[1] = &ufunc_round_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_ptr[3]) = ((void *)((char *)((char *)"round")));

  /* "scipy/special/_ufuncs.pyx":13056
 * ufunc_round_ptr[2*1] = <void*>_func_round
 * ufunc_round_ptr[2*1+1] = <void*>(<char*>"round")
 * ufunc_round_data[0] = &ufunc_round_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_round_data[1] = &ufunc_round_ptr[2*1]
 * round = np.PyUFunc_FromFuncAndData(ufunc_round_loops, ufunc_round_data, ufunc_round_types, 2, 1, 1, 0, "round", ufunc_round_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":13057
 * ufunc_round_ptr[2*1+1] = <void*>(<char*>"round")
 * ufunc_round_data[0] = &ufunc_round_ptr[2*0]
 * ufunc_round_data[1] = &ufunc_round_ptr[2*1]             # <<<<<<<<<<<<<<
 * round = np.PyUFunc_FromFuncAndData(ufunc_round_loops, ufunc_round_data, ufunc_round_types, 2, 1, 1, 0, "round", ufunc_round_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":13058
 * ufunc_round_data[0] = &ufunc_round_ptr[2*0]
 * ufunc_round_data[1] = &ufunc_round_ptr[2*1]
 * round = np.PyUFunc_FromFuncAndData(ufunc_round_loops, ufunc_round_data, ufunc_round_types, 2, 1, 1, 0, "round", ufunc_round_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_shichi_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_round_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_round_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_round_types, 2, 1, 1, 0, ((char *)"round"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_round_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13058, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_round, __pyx_t_2) < 0) __PYX_ERR(1, 13058, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":13064
 * cdef void *ufunc_shichi_data[4]
 * cdef char ufunc_shichi_types[12]
 * cdef char *ufunc_shichi_doc = (             # <<<<<<<<<<<<<<
 *     "shichi(x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_doc = ((char *)"shichi(x, out=None)\n\nHyperbolic sine and cosine integrals.\n\nThe hyperbolic sine integral is\n\n.. math::\n\n  \\int_0^x \\frac{\\sinh{t}}{t}dt\n\nand the hyperbolic cosine integral is\n\n.. math::\n\n  \\gamma + \\log(x) + \\int_0^x \\frac{\\cosh{t} - 1}{t} dt\n\nwhere :math:`\\gamma` is Euler's constant and :math:`\\log` is the\nprinciple branch of the logarithm.\n\nParameters\n----------\nx : array_like\n    Real or complex points at which to compute the hyperbolic sine\n    and cosine integrals.\n\nReturns\n-------\nsi : ndarray\n    Hyperbolic sine integral at ``x``\nci : ndarray\n    Hyperbolic cosine integral at ``x``\n\nNotes\n-----\nFor real arguments with ``x < 0``, ``chi`` is the real part of the\nhyperbolic cosine integral. For such points ``chi(x)`` and ``chi(x\n+ 0j)`` differ by a factor of ``1j*pi``.\n\nFor real arguments the function is computed by calling Cephes'\n[1]_ *shichi* routine. For complex arguments the algorithm is based\non Mpmath's [2]_ *shi* and *chi* routines.\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/\n.. [2] Fredrik Johansson and others.\n       \"mpmath: a Python library for arbitrary-precision floating-point arithmetic\"\n       (Version 0.19) http://mpmath.org/");

  /* "scipy/special/_ufuncs.pyx":13114
 *     "       \"mpmath: a Python library for arbitrary-precision floating-point arithmetic\"\n"
 *     "       (Version 0.19) http://mpmath.org/")
 * ufunc_shichi_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff             # <<<<<<<<<<<<<<
 * ufunc_shichi_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_shichi_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_F_FF
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_f_ff);

  /* "scipy/special/_ufuncs.pyx":13115
 *     "       (Version 0.19) http://mpmath.org/")
 * ufunc_shichi_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_shichi_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd             # <<<<<<<<<<<<<<
 * ufunc_shichi_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_F_FF
 * ufunc_shichi_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_D_DD
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_d_dd);

  /* "scipy/special/_ufuncs.pyx":13116
 * ufunc_shichi_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_shichi_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_shichi_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_F_FF             # <<<<<<<<<<<<<<
 * ufunc_shichi_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_D_DD
 * ufunc_shichi_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DD_As_F_FF);

  /* "scipy/special/_ufuncs.pyx":13117
 * ufunc_shichi_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_shichi_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_F_FF
 * ufunc_shichi_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_D_DD             # <<<<<<<<<<<<<<
 * ufunc_shichi_types[0] = <char>NPY_FLOAT
 * ufunc_shichi_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DD_As_D_DD);

  /* "scipy/special/_ufuncs.pyx":13118
 * ufunc_shichi_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_F_FF
 * ufunc_shichi_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_D_DD
 * ufunc_shichi_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_shichi_types[1] = <char>NPY_FLOAT
 * ufunc_shichi_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13119
 * ufunc_shichi_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_D_DD
 * ufunc_shichi_types[0] = <char>NPY_FLOAT
 * ufunc_shichi_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_shichi_types[2] = <char>NPY_FLOAT
 * ufunc_shichi_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13120
 * ufunc_shichi_types[0] = <char>NPY_FLOAT
 * ufunc_shichi_types[1] = <char>NPY_FLOAT
 * ufunc_shichi_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_shichi_types[3] = <char>NPY_DOUBLE
 * ufunc_shichi_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13121
 * ufunc_shichi_types[1] = <char>NPY_FLOAT
 * ufunc_shichi_types[2] = <char>NPY_FLOAT
 * ufunc_shichi_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_shichi_types[4] = <char>NPY_DOUBLE
 * ufunc_shichi_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13122
 * ufunc_shichi_types[2] = <char>NPY_FLOAT
 * ufunc_shichi_types[3] = <char>NPY_DOUBLE
 * ufunc_shichi_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_shichi_types[5] = <char>NPY_DOUBLE
 * ufunc_shichi_types[6] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13123
 * ufunc_shichi_types[3] = <char>NPY_DOUBLE
 * ufunc_shichi_types[4] = <char>NPY_DOUBLE
 * ufunc_shichi_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_shichi_types[6] = <char>NPY_CFLOAT
 * ufunc_shichi_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13124
 * ufunc_shichi_types[4] = <char>NPY_DOUBLE
 * ufunc_shichi_types[5] = <char>NPY_DOUBLE
 * ufunc_shichi_types[6] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_shichi_types[7] = <char>NPY_CFLOAT
 * ufunc_shichi_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types[6]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13125
 * ufunc_shichi_types[5] = <char>NPY_DOUBLE
 * ufunc_shichi_types[6] = <char>NPY_CFLOAT
 * ufunc_shichi_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_shichi_types[8] = <char>NPY_CFLOAT
 * ufunc_shichi_types[9] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13126
 * ufunc_shichi_types[6] = <char>NPY_CFLOAT
 * ufunc_shichi_types[7] = <char>NPY_CFLOAT
 * ufunc_shichi_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_shichi_types[9] = <char>NPY_CDOUBLE
 * ufunc_shichi_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13127
 * ufunc_shichi_types[7] = <char>NPY_CFLOAT
 * ufunc_shichi_types[8] = <char>NPY_CFLOAT
 * ufunc_shichi_types[9] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_shichi_types[10] = <char>NPY_CDOUBLE
 * ufunc_shichi_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types[9]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13128
 * ufunc_shichi_types[8] = <char>NPY_CFLOAT
 * ufunc_shichi_types[9] = <char>NPY_CDOUBLE
 * ufunc_shichi_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_shichi_types[11] = <char>NPY_CDOUBLE
 * ufunc_shichi_ptr[2*0] = <void*>_func_shichi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13129
 * ufunc_shichi_types[9] = <char>NPY_CDOUBLE
 * ufunc_shichi_types[10] = <char>NPY_CDOUBLE
 * ufunc_shichi_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_shichi_ptr[2*0] = <void*>_func_shichi
 * ufunc_shichi_ptr[2*0+1] = <void*>(<char*>"shichi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13130
 * ufunc_shichi_types[10] = <char>NPY_CDOUBLE
 * ufunc_shichi_types[11] = <char>NPY_CDOUBLE
 * ufunc_shichi_ptr[2*0] = <void*>_func_shichi             # <<<<<<<<<<<<<<
 * ufunc_shichi_ptr[2*0+1] = <void*>(<char*>"shichi")
 * ufunc_shichi_ptr[2*1] = <void*>_func_shichi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_ptr[0]) = ((void *)shichi);

  /* "scipy/special/_ufuncs.pyx":13131
 * ufunc_shichi_types[11] = <char>NPY_CDOUBLE
 * ufunc_shichi_ptr[2*0] = <void*>_func_shichi
 * ufunc_shichi_ptr[2*0+1] = <void*>(<char*>"shichi")             # <<<<<<<<<<<<<<
 * ufunc_shichi_ptr[2*1] = <void*>_func_shichi
 * ufunc_shichi_ptr[2*1+1] = <void*>(<char*>"shichi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_ptr[1]) = ((void *)((char *)((char *)"shichi")));

  /* "scipy/special/_ufuncs.pyx":13132
 * ufunc_shichi_ptr[2*0] = <void*>_func_shichi
 * ufunc_shichi_ptr[2*0+1] = <void*>(<char*>"shichi")
 * ufunc_shichi_ptr[2*1] = <void*>_func_shichi             # <<<<<<<<<<<<<<
 * ufunc_shichi_ptr[2*1+1] = <void*>(<char*>"shichi")
 * ufunc_shichi_ptr[2*2] = <void*>_func_cshichi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_ptr[2]) = ((void *)shichi);

  /* "scipy/special/_ufuncs.pyx":13133
 * ufunc_shichi_ptr[2*0+1] = <void*>(<char*>"shichi")
 * ufunc_shichi_ptr[2*1] = <void*>_func_shichi
 * ufunc_shichi_ptr[2*1+1] = <void*>(<char*>"shichi")             # <<<<<<<<<<<<<<
 * ufunc_shichi_ptr[2*2] = <void*>_func_cshichi
 * ufunc_shichi_ptr[2*2+1] = <void*>(<char*>"shichi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_ptr[3]) = ((void *)((char *)((char *)"shichi")));

  /* "scipy/special/_ufuncs.pyx":13134
 * ufunc_shichi_ptr[2*1] = <void*>_func_shichi
 * ufunc_shichi_ptr[2*1+1] = <void*>(<char*>"shichi")
 * ufunc_shichi_ptr[2*2] = <void*>_func_cshichi             # <<<<<<<<<<<<<<
 * ufunc_shichi_ptr[2*2+1] = <void*>(<char*>"shichi")
 * ufunc_shichi_ptr[2*3] = <void*>_func_cshichi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_ptr[4]) = ((void *)__pyx_f_5scipy_7special_5_sici_cshichi);

  /* "scipy/special/_ufuncs.pyx":13135
 * ufunc_shichi_ptr[2*1+1] = <void*>(<char*>"shichi")
 * ufunc_shichi_ptr[2*2] = <void*>_func_cshichi
 * ufunc_shichi_ptr[2*2+1] = <void*>(<char*>"shichi")             # <<<<<<<<<<<<<<
 * ufunc_shichi_ptr[2*3] = <void*>_func_cshichi
 * ufunc_shichi_ptr[2*3+1] = <void*>(<char*>"shichi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_ptr[5]) = ((void *)((char *)((char *)"shichi")));

  /* "scipy/special/_ufuncs.pyx":13136
 * ufunc_shichi_ptr[2*2] = <void*>_func_cshichi
 * ufunc_shichi_ptr[2*2+1] = <void*>(<char*>"shichi")
 * ufunc_shichi_ptr[2*3] = <void*>_func_cshichi             # <<<<<<<<<<<<<<
 * ufunc_shichi_ptr[2*3+1] = <void*>(<char*>"shichi")
 * ufunc_shichi_data[0] = &ufunc_shichi_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_ptr[6]) = ((void *)__pyx_f_5scipy_7special_5_sici_cshichi);

  /* "scipy/special/_ufuncs.pyx":13137
 * ufunc_shichi_ptr[2*2+1] = <void*>(<char*>"shichi")
 * ufunc_shichi_ptr[2*3] = <void*>_func_cshichi
 * ufunc_shichi_ptr[2*3+1] = <void*>(<char*>"shichi")             # <<<<<<<<<<<<<<
 * ufunc_shichi_data[0] = &ufunc_shichi_ptr[2*0]
 * ufunc_shichi_data[1] = &ufunc_shichi_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_ptr[7]) = ((void *)((char *)((char *)"shichi")));

  /* "scipy/special/_ufuncs.pyx":13138
 * ufunc_shichi_ptr[2*3] = <void*>_func_cshichi
 * ufunc_shichi_ptr[2*3+1] = <void*>(<char*>"shichi")
 * ufunc_shichi_data[0] = &ufunc_shichi_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_shichi_data[1] = &ufunc_shichi_ptr[2*1]
 * ufunc_shichi_data[2] = &ufunc_shichi_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":13139
 * ufunc_shichi_ptr[2*3+1] = <void*>(<char*>"shichi")
 * ufunc_shichi_data[0] = &ufunc_shichi_ptr[2*0]
 * ufunc_shichi_data[1] = &ufunc_shichi_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_shichi_data[2] = &ufunc_shichi_ptr[2*2]
 * ufunc_shichi_data[3] = &ufunc_shichi_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":13140
 * ufunc_shichi_data[0] = &ufunc_shichi_ptr[2*0]
 * ufunc_shichi_data[1] = &ufunc_shichi_ptr[2*1]
 * ufunc_shichi_data[2] = &ufunc_shichi_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_shichi_data[3] = &ufunc_shichi_ptr[2*3]
 * shichi = np.PyUFunc_FromFuncAndData(ufunc_shichi_loops, ufunc_shichi_data, ufunc_shichi_types, 4, 1, 2, 0, "shichi", ufunc_shichi_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":13141
 * ufunc_shichi_data[1] = &ufunc_shichi_ptr[2*1]
 * ufunc_shichi_data[2] = &ufunc_shichi_ptr[2*2]
 * ufunc_shichi_data[3] = &ufunc_shichi_ptr[2*3]             # <<<<<<<<<<<<<<
 * shichi = np.PyUFunc_FromFuncAndData(ufunc_shichi_loops, ufunc_shichi_data, ufunc_shichi_types, 4, 1, 2, 0, "shichi", ufunc_shichi_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":13142
 * ufunc_shichi_data[2] = &ufunc_shichi_ptr[2*2]
 * ufunc_shichi_data[3] = &ufunc_shichi_ptr[2*3]
 * shichi = np.PyUFunc_FromFuncAndData(ufunc_shichi_loops, ufunc_shichi_data, ufunc_shichi_types, 4, 1, 2, 0, "shichi", ufunc_shichi_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_sici_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_types, 4, 1, 2, 0, ((char *)"shichi"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_shichi_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_shichi, __pyx_t_2) < 0) __PYX_ERR(1, 13142, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":13148
 * cdef void *ufunc_sici_data[4]
 * cdef char ufunc_sici_types[12]
 * cdef char *ufunc_sici_doc = (             # <<<<<<<<<<<<<<
 *     "sici(x, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_doc = ((char *)"sici(x, out=None)\n\nSine and cosine integrals.\n\nThe sine integral is\n\n.. math::\n\n  \\int_0^x \\frac{\\sin{t}}{t}dt\n\nand the cosine integral is\n\n.. math::\n\n  \\gamma + \\log(x) + \\int_0^x \\frac{\\cos{t} - 1}{t}dt\n\nwhere :math:`\\gamma` is Euler's constant and :math:`\\log` is the\nprinciple branch of the logarithm.\n\nParameters\n----------\nx : array_like\n    Real or complex points at which to compute the sine and cosine\n    integrals.\n\nReturns\n-------\nsi : ndarray\n    Sine integral at ``x``\nci : ndarray\n    Cosine integral at ``x``\n\nNotes\n-----\nFor real arguments with ``x < 0``, ``ci`` is the real part of the\ncosine integral. For such points ``ci(x)`` and ``ci(x + 0j)``\ndiffer by a factor of ``1j*pi``.\n\nFor real arguments the function is computed by calling Cephes'\n[1]_ *sici* routine. For complex arguments the algorithm is based\non Mpmath's [2]_ *si* and *ci* routines.\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/\n.. [2] Fredrik Johansson and others.\n       \"mpmath: a Python library for arbitrary-precision floating-point arithmetic\"\n       (Version 0.19) http://mpmath.org/");

  /* "scipy/special/_ufuncs.pyx":13198
 *     "       \"mpmath: a Python library for arbitrary-precision floating-point arithmetic\"\n"
 *     "       (Version 0.19) http://mpmath.org/")
 * ufunc_sici_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff             # <<<<<<<<<<<<<<
 * ufunc_sici_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_sici_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_F_FF
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_f_ff);

  /* "scipy/special/_ufuncs.pyx":13199
 *     "       (Version 0.19) http://mpmath.org/")
 * ufunc_sici_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_sici_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd             # <<<<<<<<<<<<<<
 * ufunc_sici_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_F_FF
 * ufunc_sici_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_D_DD
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_d_dd_As_d_dd);

  /* "scipy/special/_ufuncs.pyx":13200
 * ufunc_sici_loops[0] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_f_ff
 * ufunc_sici_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_sici_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_F_FF             # <<<<<<<<<<<<<<
 * ufunc_sici_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_D_DD
 * ufunc_sici_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DD_As_F_FF);

  /* "scipy/special/_ufuncs.pyx":13201
 * ufunc_sici_loops[1] = <np.PyUFuncGenericFunction>loop_i_d_dd_As_d_dd
 * ufunc_sici_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_F_FF
 * ufunc_sici_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_D_DD             # <<<<<<<<<<<<<<
 * ufunc_sici_types[0] = <char>NPY_FLOAT
 * ufunc_sici_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_i_D_DD_As_D_DD);

  /* "scipy/special/_ufuncs.pyx":13202
 * ufunc_sici_loops[2] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_F_FF
 * ufunc_sici_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_D_DD
 * ufunc_sici_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_sici_types[1] = <char>NPY_FLOAT
 * ufunc_sici_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13203
 * ufunc_sici_loops[3] = <np.PyUFuncGenericFunction>loop_i_D_DD_As_D_DD
 * ufunc_sici_types[0] = <char>NPY_FLOAT
 * ufunc_sici_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_sici_types[2] = <char>NPY_FLOAT
 * ufunc_sici_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13204
 * ufunc_sici_types[0] = <char>NPY_FLOAT
 * ufunc_sici_types[1] = <char>NPY_FLOAT
 * ufunc_sici_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_sici_types[3] = <char>NPY_DOUBLE
 * ufunc_sici_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13205
 * ufunc_sici_types[1] = <char>NPY_FLOAT
 * ufunc_sici_types[2] = <char>NPY_FLOAT
 * ufunc_sici_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sici_types[4] = <char>NPY_DOUBLE
 * ufunc_sici_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13206
 * ufunc_sici_types[2] = <char>NPY_FLOAT
 * ufunc_sici_types[3] = <char>NPY_DOUBLE
 * ufunc_sici_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sici_types[5] = <char>NPY_DOUBLE
 * ufunc_sici_types[6] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13207
 * ufunc_sici_types[3] = <char>NPY_DOUBLE
 * ufunc_sici_types[4] = <char>NPY_DOUBLE
 * ufunc_sici_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sici_types[6] = <char>NPY_CFLOAT
 * ufunc_sici_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13208
 * ufunc_sici_types[4] = <char>NPY_DOUBLE
 * ufunc_sici_types[5] = <char>NPY_DOUBLE
 * ufunc_sici_types[6] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_sici_types[7] = <char>NPY_CFLOAT
 * ufunc_sici_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types[6]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13209
 * ufunc_sici_types[5] = <char>NPY_DOUBLE
 * ufunc_sici_types[6] = <char>NPY_CFLOAT
 * ufunc_sici_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_sici_types[8] = <char>NPY_CFLOAT
 * ufunc_sici_types[9] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13210
 * ufunc_sici_types[6] = <char>NPY_CFLOAT
 * ufunc_sici_types[7] = <char>NPY_CFLOAT
 * ufunc_sici_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_sici_types[9] = <char>NPY_CDOUBLE
 * ufunc_sici_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13211
 * ufunc_sici_types[7] = <char>NPY_CFLOAT
 * ufunc_sici_types[8] = <char>NPY_CFLOAT
 * ufunc_sici_types[9] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sici_types[10] = <char>NPY_CDOUBLE
 * ufunc_sici_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types[9]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13212
 * ufunc_sici_types[8] = <char>NPY_CFLOAT
 * ufunc_sici_types[9] = <char>NPY_CDOUBLE
 * ufunc_sici_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sici_types[11] = <char>NPY_CDOUBLE
 * ufunc_sici_ptr[2*0] = <void*>_func_sici
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13213
 * ufunc_sici_types[9] = <char>NPY_CDOUBLE
 * ufunc_sici_types[10] = <char>NPY_CDOUBLE
 * ufunc_sici_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sici_ptr[2*0] = <void*>_func_sici
 * ufunc_sici_ptr[2*0+1] = <void*>(<char*>"sici")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13214
 * ufunc_sici_types[10] = <char>NPY_CDOUBLE
 * ufunc_sici_types[11] = <char>NPY_CDOUBLE
 * ufunc_sici_ptr[2*0] = <void*>_func_sici             # <<<<<<<<<<<<<<
 * ufunc_sici_ptr[2*0+1] = <void*>(<char*>"sici")
 * ufunc_sici_ptr[2*1] = <void*>_func_sici
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_ptr[0]) = ((void *)sici);

  /* "scipy/special/_ufuncs.pyx":13215
 * ufunc_sici_types[11] = <char>NPY_CDOUBLE
 * ufunc_sici_ptr[2*0] = <void*>_func_sici
 * ufunc_sici_ptr[2*0+1] = <void*>(<char*>"sici")             # <<<<<<<<<<<<<<
 * ufunc_sici_ptr[2*1] = <void*>_func_sici
 * ufunc_sici_ptr[2*1+1] = <void*>(<char*>"sici")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_ptr[1]) = ((void *)((char *)((char *)"sici")));

  /* "scipy/special/_ufuncs.pyx":13216
 * ufunc_sici_ptr[2*0] = <void*>_func_sici
 * ufunc_sici_ptr[2*0+1] = <void*>(<char*>"sici")
 * ufunc_sici_ptr[2*1] = <void*>_func_sici             # <<<<<<<<<<<<<<
 * ufunc_sici_ptr[2*1+1] = <void*>(<char*>"sici")
 * ufunc_sici_ptr[2*2] = <void*>_func_csici
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_ptr[2]) = ((void *)sici);

  /* "scipy/special/_ufuncs.pyx":13217
 * ufunc_sici_ptr[2*0+1] = <void*>(<char*>"sici")
 * ufunc_sici_ptr[2*1] = <void*>_func_sici
 * ufunc_sici_ptr[2*1+1] = <void*>(<char*>"sici")             # <<<<<<<<<<<<<<
 * ufunc_sici_ptr[2*2] = <void*>_func_csici
 * ufunc_sici_ptr[2*2+1] = <void*>(<char*>"sici")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_ptr[3]) = ((void *)((char *)((char *)"sici")));

  /* "scipy/special/_ufuncs.pyx":13218
 * ufunc_sici_ptr[2*1] = <void*>_func_sici
 * ufunc_sici_ptr[2*1+1] = <void*>(<char*>"sici")
 * ufunc_sici_ptr[2*2] = <void*>_func_csici             # <<<<<<<<<<<<<<
 * ufunc_sici_ptr[2*2+1] = <void*>(<char*>"sici")
 * ufunc_sici_ptr[2*3] = <void*>_func_csici
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_ptr[4]) = ((void *)__pyx_f_5scipy_7special_5_sici_csici);

  /* "scipy/special/_ufuncs.pyx":13219
 * ufunc_sici_ptr[2*1+1] = <void*>(<char*>"sici")
 * ufunc_sici_ptr[2*2] = <void*>_func_csici
 * ufunc_sici_ptr[2*2+1] = <void*>(<char*>"sici")             # <<<<<<<<<<<<<<
 * ufunc_sici_ptr[2*3] = <void*>_func_csici
 * ufunc_sici_ptr[2*3+1] = <void*>(<char*>"sici")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_ptr[5]) = ((void *)((char *)((char *)"sici")));

  /* "scipy/special/_ufuncs.pyx":13220
 * ufunc_sici_ptr[2*2] = <void*>_func_csici
 * ufunc_sici_ptr[2*2+1] = <void*>(<char*>"sici")
 * ufunc_sici_ptr[2*3] = <void*>_func_csici             # <<<<<<<<<<<<<<
 * ufunc_sici_ptr[2*3+1] = <void*>(<char*>"sici")
 * ufunc_sici_data[0] = &ufunc_sici_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_ptr[6]) = ((void *)__pyx_f_5scipy_7special_5_sici_csici);

  /* "scipy/special/_ufuncs.pyx":13221
 * ufunc_sici_ptr[2*2+1] = <void*>(<char*>"sici")
 * ufunc_sici_ptr[2*3] = <void*>_func_csici
 * ufunc_sici_ptr[2*3+1] = <void*>(<char*>"sici")             # <<<<<<<<<<<<<<
 * ufunc_sici_data[0] = &ufunc_sici_ptr[2*0]
 * ufunc_sici_data[1] = &ufunc_sici_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_ptr[7]) = ((void *)((char *)((char *)"sici")));

  /* "scipy/special/_ufuncs.pyx":13222
 * ufunc_sici_ptr[2*3] = <void*>_func_csici
 * ufunc_sici_ptr[2*3+1] = <void*>(<char*>"sici")
 * ufunc_sici_data[0] = &ufunc_sici_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_sici_data[1] = &ufunc_sici_ptr[2*1]
 * ufunc_sici_data[2] = &ufunc_sici_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":13223
 * ufunc_sici_ptr[2*3+1] = <void*>(<char*>"sici")
 * ufunc_sici_data[0] = &ufunc_sici_ptr[2*0]
 * ufunc_sici_data[1] = &ufunc_sici_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_sici_data[2] = &ufunc_sici_ptr[2*2]
 * ufunc_sici_data[3] = &ufunc_sici_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":13224
 * ufunc_sici_data[0] = &ufunc_sici_ptr[2*0]
 * ufunc_sici_data[1] = &ufunc_sici_ptr[2*1]
 * ufunc_sici_data[2] = &ufunc_sici_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_sici_data[3] = &ufunc_sici_ptr[2*3]
 * sici = np.PyUFunc_FromFuncAndData(ufunc_sici_loops, ufunc_sici_data, ufunc_sici_types, 4, 1, 2, 0, "sici", ufunc_sici_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":13225
 * ufunc_sici_data[1] = &ufunc_sici_ptr[2*1]
 * ufunc_sici_data[2] = &ufunc_sici_ptr[2*2]
 * ufunc_sici_data[3] = &ufunc_sici_ptr[2*3]             # <<<<<<<<<<<<<<
 * sici = np.PyUFunc_FromFuncAndData(ufunc_sici_loops, ufunc_sici_data, ufunc_sici_types, 4, 1, 2, 0, "sici", ufunc_sici_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":13226
 * ufunc_sici_data[2] = &ufunc_sici_ptr[2*2]
 * ufunc_sici_data[3] = &ufunc_sici_ptr[2*3]
 * sici = np.PyUFunc_FromFuncAndData(ufunc_sici_loops, ufunc_sici_data, ufunc_sici_types, 4, 1, 2, 0, "sici", ufunc_sici_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_sindg_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_types, 4, 1, 2, 0, ((char *)"sici"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_sici_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13226, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sici, __pyx_t_2) < 0) __PYX_ERR(1, 13226, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":13232
 * cdef void *ufunc_sindg_data[2]
 * cdef char ufunc_sindg_types[4]
 * cdef char *ufunc_sindg_doc = (             # <<<<<<<<<<<<<<
 *     "sindg(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_doc = ((char *)"sindg(x)\n\nSine of angle given in degrees");

  /* "scipy/special/_ufuncs.pyx":13236
 *     "\n"
 *     "Sine of angle given in degrees")
 * ufunc_sindg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_sindg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_sindg_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":13237
 *     "Sine of angle given in degrees")
 * ufunc_sindg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_sindg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_sindg_types[0] = <char>NPY_FLOAT
 * ufunc_sindg_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":13238
 * ufunc_sindg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_sindg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_sindg_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_sindg_types[1] = <char>NPY_FLOAT
 * ufunc_sindg_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13239
 * ufunc_sindg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_sindg_types[0] = <char>NPY_FLOAT
 * ufunc_sindg_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_sindg_types[2] = <char>NPY_DOUBLE
 * ufunc_sindg_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13240
 * ufunc_sindg_types[0] = <char>NPY_FLOAT
 * ufunc_sindg_types[1] = <char>NPY_FLOAT
 * ufunc_sindg_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sindg_types[3] = <char>NPY_DOUBLE
 * ufunc_sindg_ptr[2*0] = <void*>_func_sindg
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13241
 * ufunc_sindg_types[1] = <char>NPY_FLOAT
 * ufunc_sindg_types[2] = <char>NPY_DOUBLE
 * ufunc_sindg_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sindg_ptr[2*0] = <void*>_func_sindg
 * ufunc_sindg_ptr[2*0+1] = <void*>(<char*>"sindg")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13242
 * ufunc_sindg_types[2] = <char>NPY_DOUBLE
 * ufunc_sindg_types[3] = <char>NPY_DOUBLE
 * ufunc_sindg_ptr[2*0] = <void*>_func_sindg             # <<<<<<<<<<<<<<
 * ufunc_sindg_ptr[2*0+1] = <void*>(<char*>"sindg")
 * ufunc_sindg_ptr[2*1] = <void*>_func_sindg
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_ptr[0]) = ((void *)sindg);

  /* "scipy/special/_ufuncs.pyx":13243
 * ufunc_sindg_types[3] = <char>NPY_DOUBLE
 * ufunc_sindg_ptr[2*0] = <void*>_func_sindg
 * ufunc_sindg_ptr[2*0+1] = <void*>(<char*>"sindg")             # <<<<<<<<<<<<<<
 * ufunc_sindg_ptr[2*1] = <void*>_func_sindg
 * ufunc_sindg_ptr[2*1+1] = <void*>(<char*>"sindg")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_ptr[1]) = ((void *)((char *)((char *)"sindg")));

  /* "scipy/special/_ufuncs.pyx":13244
 * ufunc_sindg_ptr[2*0] = <void*>_func_sindg
 * ufunc_sindg_ptr[2*0+1] = <void*>(<char*>"sindg")
 * ufunc_sindg_ptr[2*1] = <void*>_func_sindg             # <<<<<<<<<<<<<<
 * ufunc_sindg_ptr[2*1+1] = <void*>(<char*>"sindg")
 * ufunc_sindg_data[0] = &ufunc_sindg_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_ptr[2]) = ((void *)sindg);

  /* "scipy/special/_ufuncs.pyx":13245
 * ufunc_sindg_ptr[2*0+1] = <void*>(<char*>"sindg")
 * ufunc_sindg_ptr[2*1] = <void*>_func_sindg
 * ufunc_sindg_ptr[2*1+1] = <void*>(<char*>"sindg")             # <<<<<<<<<<<<<<
 * ufunc_sindg_data[0] = &ufunc_sindg_ptr[2*0]
 * ufunc_sindg_data[1] = &ufunc_sindg_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_ptr[3]) = ((void *)((char *)((char *)"sindg")));

  /* "scipy/special/_ufuncs.pyx":13246
 * ufunc_sindg_ptr[2*1] = <void*>_func_sindg
 * ufunc_sindg_ptr[2*1+1] = <void*>(<char*>"sindg")
 * ufunc_sindg_data[0] = &ufunc_sindg_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_sindg_data[1] = &ufunc_sindg_ptr[2*1]
 * sindg = np.PyUFunc_FromFuncAndData(ufunc_sindg_loops, ufunc_sindg_data, ufunc_sindg_types, 2, 1, 1, 0, "sindg", ufunc_sindg_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":13247
 * ufunc_sindg_ptr[2*1+1] = <void*>(<char*>"sindg")
 * ufunc_sindg_data[0] = &ufunc_sindg_ptr[2*0]
 * ufunc_sindg_data[1] = &ufunc_sindg_ptr[2*1]             # <<<<<<<<<<<<<<
 * sindg = np.PyUFunc_FromFuncAndData(ufunc_sindg_loops, ufunc_sindg_data, ufunc_sindg_types, 2, 1, 1, 0, "sindg", ufunc_sindg_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":13248
 * ufunc_sindg_data[0] = &ufunc_sindg_ptr[2*0]
 * ufunc_sindg_data[1] = &ufunc_sindg_ptr[2*1]
 * sindg = np.PyUFunc_FromFuncAndData(ufunc_sindg_loops, ufunc_sindg_data, ufunc_sindg_types, 2, 1, 1, 0, "sindg", ufunc_sindg_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_smirnov_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_types, 2, 1, 1, 0, ((char *)"sindg"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_sindg_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sindg, __pyx_t_2) < 0) __PYX_ERR(1, 13248, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":13254
 * cdef void *ufunc_smirnov_data[3]
 * cdef char ufunc_smirnov_types[9]
 * cdef char *ufunc_smirnov_doc = (             # <<<<<<<<<<<<<<
 *     "smirnov(n, d)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_doc = ((char *)"smirnov(n, d)\n\nKolmogorov-Smirnov complementary cumulative distribution function\n\nReturns the exact Kolmogorov-Smirnov complementary cumulative\ndistribution function,(aka the Survival Function) of Dn+ (or Dn-)\nfor a one-sided test of equality between an empirical and a\ntheoretical distribution. It is equal to the probability that the\nmaximum difference between a theoretical distribution and an empirical\none based on `n` samples is greater than d.\n\nParameters\n----------\nn : int\n  Number of samples\nd : float array_like\n  Deviation between the Empirical CDF (ECDF) and the target CDF.\n\nReturns\n-------\nfloat\n    The value(s) of smirnov(n, d), Prob(Dn+ >= d) (Also Prob(Dn- >= d))\n\nNotes\n-----\n`smirnov` is used by `stats.kstest` in the application of the\nKolmogorov-Smirnov Goodness of Fit test. For historial reasons this\nfunction is exposed in `scpy.special`, but the recommended way to achieve\nthe most accurate CDF/SF/PDF/PPF/ISF computations is to use the\n`stats.ksone` distrubution.\n\nSee Also\n--------\nsmirnovi : The Inverse Survival Function for the distribution\nscipy.stats.ksone : Provides the functionality as a continuous distribution\nkolmogorov, kolmogi : Functions for the two-sided distribution\n\nExamples\n--------\n>>> from scipy.special import smirnov\n\nShow the probability of a gap at least as big as 0, 0.5 and 1.0 for a sample of size 5\n\n>>> smirnov(5, [0, 0.5, 1.0])\narray([ 1.   ,  0.056,  0.   ])\n\nCompare a sample of size 5 drawn from a source N(0.5, 1) distribution against\na target N(0, 1) CDF.\n\n>>> from scipy.stats import norm\n>>> n = 5\n>>> gendist = norm(0.5, 1)       # Normal distribution, mean 0.5, stddev 1\n>>> np.random.seed(seed=233423)  # Set the seed for reproducibility\n>>> x = np.sort(gendist.rvs(size=n))\n>>> x\narray([-0.20946287,  0.71688765,  0.95164151,  1.44590852,  3.08880533])\n>>> target = norm(0, 1)\n>>> cdfs = target.cdf(x)\n>>> cdfs\narray([ 0.41704346,  0.76327829,  0.82936059,  0.92589857, "" 0.99899518])\n# Construct the Empirical CDF and the K-S statistics (Dn+, Dn-, Dn)\n>>> ecdfs = np.arange(n+1, dtype=float)/n\n>>> cols = np.column_stack([x, ecdfs[1:], cdfs, cdfs - ecdfs[:n], ecdfs[1:] - cdfs])\n>>> np.set_printoptions(precision=3)\n>>> cols\narray([[ -2.095e-01,   2.000e-01,   4.170e-01,   4.170e-01,  -2.170e-01],\n       [  7.169e-01,   4.000e-01,   7.633e-01,   5.633e-01,  -3.633e-01],\n       [  9.516e-01,   6.000e-01,   8.294e-01,   4.294e-01,  -2.294e-01],\n       [  1.446e+00,   8.000e-01,   9.259e-01,   3.259e-01,  -1.259e-01],\n       [  3.089e+00,   1.000e+00,   9.990e-01,   1.990e-01,   1.005e-03]])\n>>> gaps = cols[:, -2:]\n>>> Dnpm = np.max(gaps, axis=0)\n>>> print('Dn-=%f, Dn+=%f' % (Dnpm[0], Dnpm[1]))\nDn-=0.563278, Dn+=0.001005\n>>> probs = smirnov(n, Dnpm)\n>>> print(chr(10).join(['For a sample of size %d drawn from a N(0, 1) distribution:' % n,\n...      ' Smirnov n=%d: Prob(Dn- >= %f) = %.4f' % (n, Dnpm[0], probs[0]),\n...      ' Smirnov n=%d: Prob(Dn+ >= %f) = %.4f' % (n, Dnpm[1], probs[1])]))\nFor a sample of size 5 drawn from a N(0, 1) distribution:\n Smirnov n=5: Prob(Dn- >= 0.563278) = 0.0250\n Smirnov n=5: Prob(Dn+ >= 0.001005) = 0.9990\n\nPlot the Empirical CDF against the target N(0, 1) CDF\n\n>>> import matplotlib.pyplot as plt\n>>> plt.step(np.concatenate([[-3], x]), ecdfs, where='post', label='Empirical CDF')\n>>> x3 = np.linspace(-3, 3, 100)\n>>> plt.plot(x3, target.cdf(x3), label='CDF for N(0, 1)')\n>>> plt.ylim([0, 1]); plt.grid(True); plt.legend();\n# Add vertical lines marking Dn+ and Dn-\n>>> iminus, iplus = np.argmax(gaps, axis=0)\n>>> plt.vlines([x[iminus]], ecdfs[iminus], cdfs[iminus], color='r', linestyle='dashed', lw=4)\n>>> plt.vlines([x[iplus]], cdfs[iplus], ecdfs[iplus+1], color='m', linestyle='dashed', lw=4)\n>>> plt.show()");

  /* "scipy/special/_ufuncs.pyx":13349
 *     ">>> plt.vlines([x[iplus]], cdfs[iplus], ecdfs[iplus+1], color='m', linestyle='dashed', lw=4)\n"
 *     ">>> plt.show()")
 * ufunc_smirnov_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_smirnov_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_smirnov_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":13350
 *     ">>> plt.show()")
 * ufunc_smirnov_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_smirnov_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_smirnov_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_smirnov_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":13351
 * ufunc_smirnov_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_smirnov_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_smirnov_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_smirnov_types[0] = <char>NPY_LONG
 * ufunc_smirnov_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":13352
 * ufunc_smirnov_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_smirnov_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_smirnov_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_smirnov_types[1] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":13353
 * ufunc_smirnov_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_smirnov_types[0] = <char>NPY_LONG
 * ufunc_smirnov_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_smirnov_types[2] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13354
 * ufunc_smirnov_types[0] = <char>NPY_LONG
 * ufunc_smirnov_types[1] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_smirnov_types[3] = <char>NPY_FLOAT
 * ufunc_smirnov_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13355
 * ufunc_smirnov_types[1] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[2] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_smirnov_types[4] = <char>NPY_FLOAT
 * ufunc_smirnov_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13356
 * ufunc_smirnov_types[2] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[3] = <char>NPY_FLOAT
 * ufunc_smirnov_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_smirnov_types[5] = <char>NPY_FLOAT
 * ufunc_smirnov_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13357
 * ufunc_smirnov_types[3] = <char>NPY_FLOAT
 * ufunc_smirnov_types[4] = <char>NPY_FLOAT
 * ufunc_smirnov_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_smirnov_types[6] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13358
 * ufunc_smirnov_types[4] = <char>NPY_FLOAT
 * ufunc_smirnov_types[5] = <char>NPY_FLOAT
 * ufunc_smirnov_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_smirnov_types[7] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13359
 * ufunc_smirnov_types[5] = <char>NPY_FLOAT
 * ufunc_smirnov_types[6] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_smirnov_types[8] = <char>NPY_DOUBLE
 * ufunc_smirnov_ptr[2*0] = <void*>_func_smirnov
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13360
 * ufunc_smirnov_types[6] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[7] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_smirnov_ptr[2*0] = <void*>_func_smirnov
 * ufunc_smirnov_ptr[2*0+1] = <void*>(<char*>"smirnov")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13361
 * ufunc_smirnov_types[7] = <char>NPY_DOUBLE
 * ufunc_smirnov_types[8] = <char>NPY_DOUBLE
 * ufunc_smirnov_ptr[2*0] = <void*>_func_smirnov             # <<<<<<<<<<<<<<
 * ufunc_smirnov_ptr[2*0+1] = <void*>(<char*>"smirnov")
 * ufunc_smirnov_ptr[2*1] = <void*>_func_smirnov_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_ptr[0]) = ((void *)smirnov);

  /* "scipy/special/_ufuncs.pyx":13362
 * ufunc_smirnov_types[8] = <char>NPY_DOUBLE
 * ufunc_smirnov_ptr[2*0] = <void*>_func_smirnov
 * ufunc_smirnov_ptr[2*0+1] = <void*>(<char*>"smirnov")             # <<<<<<<<<<<<<<
 * ufunc_smirnov_ptr[2*1] = <void*>_func_smirnov_unsafe
 * ufunc_smirnov_ptr[2*1+1] = <void*>(<char*>"smirnov")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_ptr[1]) = ((void *)((char *)((char *)"smirnov")));

  /* "scipy/special/_ufuncs.pyx":13363
 * ufunc_smirnov_ptr[2*0] = <void*>_func_smirnov
 * ufunc_smirnov_ptr[2*0+1] = <void*>(<char*>"smirnov")
 * ufunc_smirnov_ptr[2*1] = <void*>_func_smirnov_unsafe             # <<<<<<<<<<<<<<
 * ufunc_smirnov_ptr[2*1+1] = <void*>(<char*>"smirnov")
 * ufunc_smirnov_ptr[2*2] = <void*>_func_smirnov_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_smirnov_unsafe);

  /* "scipy/special/_ufuncs.pyx":13364
 * ufunc_smirnov_ptr[2*0+1] = <void*>(<char*>"smirnov")
 * ufunc_smirnov_ptr[2*1] = <void*>_func_smirnov_unsafe
 * ufunc_smirnov_ptr[2*1+1] = <void*>(<char*>"smirnov")             # <<<<<<<<<<<<<<
 * ufunc_smirnov_ptr[2*2] = <void*>_func_smirnov_unsafe
 * ufunc_smirnov_ptr[2*2+1] = <void*>(<char*>"smirnov")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_ptr[3]) = ((void *)((char *)((char *)"smirnov")));

  /* "scipy/special/_ufuncs.pyx":13365
 * ufunc_smirnov_ptr[2*1] = <void*>_func_smirnov_unsafe
 * ufunc_smirnov_ptr[2*1+1] = <void*>(<char*>"smirnov")
 * ufunc_smirnov_ptr[2*2] = <void*>_func_smirnov_unsafe             # <<<<<<<<<<<<<<
 * ufunc_smirnov_ptr[2*2+1] = <void*>(<char*>"smirnov")
 * ufunc_smirnov_data[0] = &ufunc_smirnov_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_smirnov_unsafe);

  /* "scipy/special/_ufuncs.pyx":13366
 * ufunc_smirnov_ptr[2*1+1] = <void*>(<char*>"smirnov")
 * ufunc_smirnov_ptr[2*2] = <void*>_func_smirnov_unsafe
 * ufunc_smirnov_ptr[2*2+1] = <void*>(<char*>"smirnov")             # <<<<<<<<<<<<<<
 * ufunc_smirnov_data[0] = &ufunc_smirnov_ptr[2*0]
 * ufunc_smirnov_data[1] = &ufunc_smirnov_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_ptr[5]) = ((void *)((char *)((char *)"smirnov")));

  /* "scipy/special/_ufuncs.pyx":13367
 * ufunc_smirnov_ptr[2*2] = <void*>_func_smirnov_unsafe
 * ufunc_smirnov_ptr[2*2+1] = <void*>(<char*>"smirnov")
 * ufunc_smirnov_data[0] = &ufunc_smirnov_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_smirnov_data[1] = &ufunc_smirnov_ptr[2*1]
 * ufunc_smirnov_data[2] = &ufunc_smirnov_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":13368
 * ufunc_smirnov_ptr[2*2+1] = <void*>(<char*>"smirnov")
 * ufunc_smirnov_data[0] = &ufunc_smirnov_ptr[2*0]
 * ufunc_smirnov_data[1] = &ufunc_smirnov_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_smirnov_data[2] = &ufunc_smirnov_ptr[2*2]
 * smirnov = np.PyUFunc_FromFuncAndData(ufunc_smirnov_loops, ufunc_smirnov_data, ufunc_smirnov_types, 3, 2, 1, 0, "smirnov", ufunc_smirnov_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":13369
 * ufunc_smirnov_data[0] = &ufunc_smirnov_ptr[2*0]
 * ufunc_smirnov_data[1] = &ufunc_smirnov_ptr[2*1]
 * ufunc_smirnov_data[2] = &ufunc_smirnov_ptr[2*2]             # <<<<<<<<<<<<<<
 * smirnov = np.PyUFunc_FromFuncAndData(ufunc_smirnov_loops, ufunc_smirnov_data, ufunc_smirnov_types, 3, 2, 1, 0, "smirnov", ufunc_smirnov_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":13370
 * ufunc_smirnov_data[1] = &ufunc_smirnov_ptr[2*1]
 * ufunc_smirnov_data[2] = &ufunc_smirnov_ptr[2*2]
 * smirnov = np.PyUFunc_FromFuncAndData(ufunc_smirnov_loops, ufunc_smirnov_data, ufunc_smirnov_types, 3, 2, 1, 0, "smirnov", ufunc_smirnov_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_smirnovi_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_types, 3, 2, 1, 0, ((char *)"smirnov"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnov_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13370, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_smirnov, __pyx_t_2) < 0) __PYX_ERR(1, 13370, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":13376
 * cdef void *ufunc_smirnovi_data[3]
 * cdef char ufunc_smirnovi_types[9]
 * cdef char *ufunc_smirnovi_doc = (             # <<<<<<<<<<<<<<
 *     "smirnovi(n, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_doc = ((char *)"smirnovi(n, p)\n\nInverse to `smirnov`\n\nReturns `d` such that ``smirnov(n, d) == p``, the critical value\ncorresponding to `p`.\n\nParameters\n----------\nn : int\n  Number of samples\np : float array_like\n    Probability\n\nReturns\n-------\nfloat\n    The value(s) of smirnovi(n, p), the critical values.\n\nNotes\n-----\n`smirnov` is used by `stats.kstest` in the application of the\nKolmogorov-Smirnov Goodness of Fit test. For historial reasons this\nfunction is exposed in `scpy.special`, but the recommended way to achieve\nthe most accurate CDF/SF/PDF/PPF/ISF computations is to use the\n`stats.ksone` distrubution.\n\nSee Also\n--------\nsmirnov  : The Survival Function (SF) for the distribution\nscipy.stats.ksone : Provides the functionality as a continuous distribution\nkolmogorov, kolmogi, scipy.stats.kstwobign : Functions for the two-sided distribution");

  /* "scipy/special/_ufuncs.pyx":13409
 *     "scipy.stats.ksone : Provides the functionality as a continuous distribution\n"
 *     "kolmogorov, kolmogi, scipy.stats.kstwobign : Functions for the two-sided distribution")
 * ufunc_smirnovi_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_smirnovi_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":13410
 *     "kolmogorov, kolmogi, scipy.stats.kstwobign : Functions for the two-sided distribution")
 * ufunc_smirnovi_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_smirnovi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_smirnovi_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":13411
 * ufunc_smirnovi_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_smirnovi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_smirnovi_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_types[0] = <char>NPY_LONG
 * ufunc_smirnovi_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":13412
 * ufunc_smirnovi_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_smirnovi_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_smirnovi_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_types[1] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":13413
 * ufunc_smirnovi_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_smirnovi_types[0] = <char>NPY_LONG
 * ufunc_smirnovi_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_types[2] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13414
 * ufunc_smirnovi_types[0] = <char>NPY_LONG
 * ufunc_smirnovi_types[1] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_types[3] = <char>NPY_FLOAT
 * ufunc_smirnovi_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13415
 * ufunc_smirnovi_types[1] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[2] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_types[4] = <char>NPY_FLOAT
 * ufunc_smirnovi_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13416
 * ufunc_smirnovi_types[2] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[3] = <char>NPY_FLOAT
 * ufunc_smirnovi_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_types[5] = <char>NPY_FLOAT
 * ufunc_smirnovi_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13417
 * ufunc_smirnovi_types[3] = <char>NPY_FLOAT
 * ufunc_smirnovi_types[4] = <char>NPY_FLOAT
 * ufunc_smirnovi_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_types[6] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13418
 * ufunc_smirnovi_types[4] = <char>NPY_FLOAT
 * ufunc_smirnovi_types[5] = <char>NPY_FLOAT
 * ufunc_smirnovi_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_types[7] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13419
 * ufunc_smirnovi_types[5] = <char>NPY_FLOAT
 * ufunc_smirnovi_types[6] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_types[8] = <char>NPY_DOUBLE
 * ufunc_smirnovi_ptr[2*0] = <void*>_func_smirnovi
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13420
 * ufunc_smirnovi_types[6] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[7] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_ptr[2*0] = <void*>_func_smirnovi
 * ufunc_smirnovi_ptr[2*0+1] = <void*>(<char*>"smirnovi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13421
 * ufunc_smirnovi_types[7] = <char>NPY_DOUBLE
 * ufunc_smirnovi_types[8] = <char>NPY_DOUBLE
 * ufunc_smirnovi_ptr[2*0] = <void*>_func_smirnovi             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_ptr[2*0+1] = <void*>(<char*>"smirnovi")
 * ufunc_smirnovi_ptr[2*1] = <void*>_func_smirnovi_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_ptr[0]) = ((void *)smirnovi);

  /* "scipy/special/_ufuncs.pyx":13422
 * ufunc_smirnovi_types[8] = <char>NPY_DOUBLE
 * ufunc_smirnovi_ptr[2*0] = <void*>_func_smirnovi
 * ufunc_smirnovi_ptr[2*0+1] = <void*>(<char*>"smirnovi")             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_ptr[2*1] = <void*>_func_smirnovi_unsafe
 * ufunc_smirnovi_ptr[2*1+1] = <void*>(<char*>"smirnovi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_ptr[1]) = ((void *)((char *)((char *)"smirnovi")));

  /* "scipy/special/_ufuncs.pyx":13423
 * ufunc_smirnovi_ptr[2*0] = <void*>_func_smirnovi
 * ufunc_smirnovi_ptr[2*0+1] = <void*>(<char*>"smirnovi")
 * ufunc_smirnovi_ptr[2*1] = <void*>_func_smirnovi_unsafe             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_ptr[2*1+1] = <void*>(<char*>"smirnovi")
 * ufunc_smirnovi_ptr[2*2] = <void*>_func_smirnovi_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_smirnovi_unsafe);

  /* "scipy/special/_ufuncs.pyx":13424
 * ufunc_smirnovi_ptr[2*0+1] = <void*>(<char*>"smirnovi")
 * ufunc_smirnovi_ptr[2*1] = <void*>_func_smirnovi_unsafe
 * ufunc_smirnovi_ptr[2*1+1] = <void*>(<char*>"smirnovi")             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_ptr[2*2] = <void*>_func_smirnovi_unsafe
 * ufunc_smirnovi_ptr[2*2+1] = <void*>(<char*>"smirnovi")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_ptr[3]) = ((void *)((char *)((char *)"smirnovi")));

  /* "scipy/special/_ufuncs.pyx":13425
 * ufunc_smirnovi_ptr[2*1] = <void*>_func_smirnovi_unsafe
 * ufunc_smirnovi_ptr[2*1+1] = <void*>(<char*>"smirnovi")
 * ufunc_smirnovi_ptr[2*2] = <void*>_func_smirnovi_unsafe             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_ptr[2*2+1] = <void*>(<char*>"smirnovi")
 * ufunc_smirnovi_data[0] = &ufunc_smirnovi_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_smirnovi_unsafe);

  /* "scipy/special/_ufuncs.pyx":13426
 * ufunc_smirnovi_ptr[2*1+1] = <void*>(<char*>"smirnovi")
 * ufunc_smirnovi_ptr[2*2] = <void*>_func_smirnovi_unsafe
 * ufunc_smirnovi_ptr[2*2+1] = <void*>(<char*>"smirnovi")             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_data[0] = &ufunc_smirnovi_ptr[2*0]
 * ufunc_smirnovi_data[1] = &ufunc_smirnovi_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_ptr[5]) = ((void *)((char *)((char *)"smirnovi")));

  /* "scipy/special/_ufuncs.pyx":13427
 * ufunc_smirnovi_ptr[2*2] = <void*>_func_smirnovi_unsafe
 * ufunc_smirnovi_ptr[2*2+1] = <void*>(<char*>"smirnovi")
 * ufunc_smirnovi_data[0] = &ufunc_smirnovi_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_data[1] = &ufunc_smirnovi_ptr[2*1]
 * ufunc_smirnovi_data[2] = &ufunc_smirnovi_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":13428
 * ufunc_smirnovi_ptr[2*2+1] = <void*>(<char*>"smirnovi")
 * ufunc_smirnovi_data[0] = &ufunc_smirnovi_ptr[2*0]
 * ufunc_smirnovi_data[1] = &ufunc_smirnovi_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_smirnovi_data[2] = &ufunc_smirnovi_ptr[2*2]
 * smirnovi = np.PyUFunc_FromFuncAndData(ufunc_smirnovi_loops, ufunc_smirnovi_data, ufunc_smirnovi_types, 3, 2, 1, 0, "smirnovi", ufunc_smirnovi_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":13429
 * ufunc_smirnovi_data[0] = &ufunc_smirnovi_ptr[2*0]
 * ufunc_smirnovi_data[1] = &ufunc_smirnovi_ptr[2*1]
 * ufunc_smirnovi_data[2] = &ufunc_smirnovi_ptr[2*2]             # <<<<<<<<<<<<<<
 * smirnovi = np.PyUFunc_FromFuncAndData(ufunc_smirnovi_loops, ufunc_smirnovi_data, ufunc_smirnovi_types, 3, 2, 1, 0, "smirnovi", ufunc_smirnovi_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":13430
 * ufunc_smirnovi_data[1] = &ufunc_smirnovi_ptr[2*1]
 * ufunc_smirnovi_data[2] = &ufunc_smirnovi_ptr[2*2]
 * smirnovi = np.PyUFunc_FromFuncAndData(ufunc_smirnovi_loops, ufunc_smirnovi_data, ufunc_smirnovi_types, 3, 2, 1, 0, "smirnovi", ufunc_smirnovi_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_spence_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_types, 3, 2, 1, 0, ((char *)"smirnovi"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_smirnovi_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_smirnovi, __pyx_t_2) < 0) __PYX_ERR(1, 13430, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":13436
 * cdef void *ufunc_spence_data[4]
 * cdef char ufunc_spence_types[8]
 * cdef char *ufunc_spence_doc = (             # <<<<<<<<<<<<<<
 *     "spence(z, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_doc = ((char *)"spence(z, out=None)\n\nSpence's function, also known as the dilogarithm.\n\nIt is defined to be\n\n.. math::\n  \\int_0^z \\frac{\\log(t)}{1 - t}dt\n\nfor complex :math:`z`, where the contour of integration is taken\nto avoid the branch cut of the logarithm. Spence's function is\nanalytic everywhere except the negative real axis where it has a\nbranch cut.\n\nParameters\n----------\nz : array_like\n    Points at which to evaluate Spence's function\n\nReturns\n-------\ns : ndarray\n    Computed values of Spence's function\n\nNotes\n-----\nThere is a different convention which defines Spence's function by\nthe integral\n\n.. math::\n  -\\int_0^z \\frac{\\log(1 - t)}{t}dt;\n\nthis is our ``spence(1 - z)``.");

  /* "scipy/special/_ufuncs.pyx":13470
 *     "\n"
 *     "this is our ``spence(1 - z)``.")
 * ufunc_spence_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_spence_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_spence_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":13471
 *     "this is our ``spence(1 - z)``.")
 * ufunc_spence_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_spence_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_spence_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_spence_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":13472
 * ufunc_spence_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_spence_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_spence_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_spence_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_spence_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":13473
 * ufunc_spence_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_spence_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_spence_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_spence_types[0] = <char>NPY_FLOAT
 * ufunc_spence_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":13474
 * ufunc_spence_loops[2] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_spence_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_spence_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_spence_types[1] = <char>NPY_FLOAT
 * ufunc_spence_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13475
 * ufunc_spence_loops[3] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_spence_types[0] = <char>NPY_FLOAT
 * ufunc_spence_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_spence_types[2] = <char>NPY_DOUBLE
 * ufunc_spence_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13476
 * ufunc_spence_types[0] = <char>NPY_FLOAT
 * ufunc_spence_types[1] = <char>NPY_FLOAT
 * ufunc_spence_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_spence_types[3] = <char>NPY_DOUBLE
 * ufunc_spence_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13477
 * ufunc_spence_types[1] = <char>NPY_FLOAT
 * ufunc_spence_types[2] = <char>NPY_DOUBLE
 * ufunc_spence_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_spence_types[4] = <char>NPY_CFLOAT
 * ufunc_spence_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13478
 * ufunc_spence_types[2] = <char>NPY_DOUBLE
 * ufunc_spence_types[3] = <char>NPY_DOUBLE
 * ufunc_spence_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_spence_types[5] = <char>NPY_CFLOAT
 * ufunc_spence_types[6] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13479
 * ufunc_spence_types[3] = <char>NPY_DOUBLE
 * ufunc_spence_types[4] = <char>NPY_CFLOAT
 * ufunc_spence_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_spence_types[6] = <char>NPY_CDOUBLE
 * ufunc_spence_types[7] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13480
 * ufunc_spence_types[4] = <char>NPY_CFLOAT
 * ufunc_spence_types[5] = <char>NPY_CFLOAT
 * ufunc_spence_types[6] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_spence_types[7] = <char>NPY_CDOUBLE
 * ufunc_spence_ptr[2*0] = <void*>_func_spence
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_types[6]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13481
 * ufunc_spence_types[5] = <char>NPY_CFLOAT
 * ufunc_spence_types[6] = <char>NPY_CDOUBLE
 * ufunc_spence_types[7] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_spence_ptr[2*0] = <void*>_func_spence
 * ufunc_spence_ptr[2*0+1] = <void*>(<char*>"spence")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_types[7]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13482
 * ufunc_spence_types[6] = <char>NPY_CDOUBLE
 * ufunc_spence_types[7] = <char>NPY_CDOUBLE
 * ufunc_spence_ptr[2*0] = <void*>_func_spence             # <<<<<<<<<<<<<<
 * ufunc_spence_ptr[2*0+1] = <void*>(<char*>"spence")
 * ufunc_spence_ptr[2*1] = <void*>_func_spence
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_ptr[0]) = ((void *)spence);

  /* "scipy/special/_ufuncs.pyx":13483
 * ufunc_spence_types[7] = <char>NPY_CDOUBLE
 * ufunc_spence_ptr[2*0] = <void*>_func_spence
 * ufunc_spence_ptr[2*0+1] = <void*>(<char*>"spence")             # <<<<<<<<<<<<<<
 * ufunc_spence_ptr[2*1] = <void*>_func_spence
 * ufunc_spence_ptr[2*1+1] = <void*>(<char*>"spence")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_ptr[1]) = ((void *)((char *)((char *)"spence")));

  /* "scipy/special/_ufuncs.pyx":13484
 * ufunc_spence_ptr[2*0] = <void*>_func_spence
 * ufunc_spence_ptr[2*0+1] = <void*>(<char*>"spence")
 * ufunc_spence_ptr[2*1] = <void*>_func_spence             # <<<<<<<<<<<<<<
 * ufunc_spence_ptr[2*1+1] = <void*>(<char*>"spence")
 * ufunc_spence_ptr[2*2] = <void*>_func_cspence
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_ptr[2]) = ((void *)spence);

  /* "scipy/special/_ufuncs.pyx":13485
 * ufunc_spence_ptr[2*0+1] = <void*>(<char*>"spence")
 * ufunc_spence_ptr[2*1] = <void*>_func_spence
 * ufunc_spence_ptr[2*1+1] = <void*>(<char*>"spence")             # <<<<<<<<<<<<<<
 * ufunc_spence_ptr[2*2] = <void*>_func_cspence
 * ufunc_spence_ptr[2*2+1] = <void*>(<char*>"spence")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_ptr[3]) = ((void *)((char *)((char *)"spence")));

  /* "scipy/special/_ufuncs.pyx":13486
 * ufunc_spence_ptr[2*1] = <void*>_func_spence
 * ufunc_spence_ptr[2*1+1] = <void*>(<char*>"spence")
 * ufunc_spence_ptr[2*2] = <void*>_func_cspence             # <<<<<<<<<<<<<<
 * ufunc_spence_ptr[2*2+1] = <void*>(<char*>"spence")
 * ufunc_spence_ptr[2*3] = <void*>_func_cspence
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_spence_cspence);

  /* "scipy/special/_ufuncs.pyx":13487
 * ufunc_spence_ptr[2*1+1] = <void*>(<char*>"spence")
 * ufunc_spence_ptr[2*2] = <void*>_func_cspence
 * ufunc_spence_ptr[2*2+1] = <void*>(<char*>"spence")             # <<<<<<<<<<<<<<
 * ufunc_spence_ptr[2*3] = <void*>_func_cspence
 * ufunc_spence_ptr[2*3+1] = <void*>(<char*>"spence")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_ptr[5]) = ((void *)((char *)((char *)"spence")));

  /* "scipy/special/_ufuncs.pyx":13488
 * ufunc_spence_ptr[2*2] = <void*>_func_cspence
 * ufunc_spence_ptr[2*2+1] = <void*>(<char*>"spence")
 * ufunc_spence_ptr[2*3] = <void*>_func_cspence             # <<<<<<<<<<<<<<
 * ufunc_spence_ptr[2*3+1] = <void*>(<char*>"spence")
 * ufunc_spence_data[0] = &ufunc_spence_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_ptr[6]) = ((void *)__pyx_f_5scipy_7special_7_spence_cspence);

  /* "scipy/special/_ufuncs.pyx":13489
 * ufunc_spence_ptr[2*2+1] = <void*>(<char*>"spence")
 * ufunc_spence_ptr[2*3] = <void*>_func_cspence
 * ufunc_spence_ptr[2*3+1] = <void*>(<char*>"spence")             # <<<<<<<<<<<<<<
 * ufunc_spence_data[0] = &ufunc_spence_ptr[2*0]
 * ufunc_spence_data[1] = &ufunc_spence_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_ptr[7]) = ((void *)((char *)((char *)"spence")));

  /* "scipy/special/_ufuncs.pyx":13490
 * ufunc_spence_ptr[2*3] = <void*>_func_cspence
 * ufunc_spence_ptr[2*3+1] = <void*>(<char*>"spence")
 * ufunc_spence_data[0] = &ufunc_spence_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_spence_data[1] = &ufunc_spence_ptr[2*1]
 * ufunc_spence_data[2] = &ufunc_spence_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":13491
 * ufunc_spence_ptr[2*3+1] = <void*>(<char*>"spence")
 * ufunc_spence_data[0] = &ufunc_spence_ptr[2*0]
 * ufunc_spence_data[1] = &ufunc_spence_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_spence_data[2] = &ufunc_spence_ptr[2*2]
 * ufunc_spence_data[3] = &ufunc_spence_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":13492
 * ufunc_spence_data[0] = &ufunc_spence_ptr[2*0]
 * ufunc_spence_data[1] = &ufunc_spence_ptr[2*1]
 * ufunc_spence_data[2] = &ufunc_spence_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_spence_data[3] = &ufunc_spence_ptr[2*3]
 * spence = np.PyUFunc_FromFuncAndData(ufunc_spence_loops, ufunc_spence_data, ufunc_spence_types, 4, 1, 1, 0, "spence", ufunc_spence_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":13493
 * ufunc_spence_data[1] = &ufunc_spence_ptr[2*1]
 * ufunc_spence_data[2] = &ufunc_spence_ptr[2*2]
 * ufunc_spence_data[3] = &ufunc_spence_ptr[2*3]             # <<<<<<<<<<<<<<
 * spence = np.PyUFunc_FromFuncAndData(ufunc_spence_loops, ufunc_spence_data, ufunc_spence_types, 4, 1, 1, 0, "spence", ufunc_spence_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":13494
 * ufunc_spence_data[2] = &ufunc_spence_ptr[2*2]
 * ufunc_spence_data[3] = &ufunc_spence_ptr[2*3]
 * spence = np.PyUFunc_FromFuncAndData(ufunc_spence_loops, ufunc_spence_data, ufunc_spence_types, 4, 1, 1, 0, "spence", ufunc_spence_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_sph_harm_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_types, 4, 1, 1, 0, ((char *)"spence"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_spence_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13494, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_spence, __pyx_t_2) < 0) __PYX_ERR(1, 13494, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":13500
 * cdef void *ufunc_sph_harm_data[3]
 * cdef char ufunc_sph_harm_types[15]
 * cdef char *ufunc_sph_harm_doc = (             # <<<<<<<<<<<<<<
 *     "sph_harm(m, n, theta, phi)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_doc = ((char *)"sph_harm(m, n, theta, phi)\n\nCompute spherical harmonics.\n\nThe spherical harmonics are defined as\n\n.. math::\n\n    Y^m_n(\\theta,\\phi) = \\sqrt{\\frac{2n+1}{4\\pi} \\frac{(n-m)!}{(n+m)!}}\n      e^{i m \\theta} P^m_n(\\cos(\\phi))\n\nwhere :math:`P_n^m` are the associated Legendre functions; see `lpmv`.\n\nParameters\n----------\nm : array_like\n    Order of the harmonic (int); must have ``|m| <= n``.\nn : array_like\n   Degree of the harmonic (int); must have ``n >= 0``. This is\n   often denoted by ``l`` (lower case L) in descriptions of\n   spherical harmonics.\ntheta : array_like\n   Azimuthal (longitudinal) coordinate; must be in ``[0, 2*pi]``.\nphi : array_like\n   Polar (colatitudinal) coordinate; must be in ``[0, pi]``.\n\nReturns\n-------\ny_mn : complex float\n   The harmonic :math:`Y^m_n` sampled at ``theta`` and ``phi``.\n\nNotes\n-----\nThere are different conventions for the meanings of the input\narguments ``theta`` and ``phi``. In SciPy ``theta`` is the\nazimuthal angle and ``phi`` is the polar angle. It is common to\nsee the opposite convention, that is, ``theta`` as the polar angle\nand ``phi`` as the azimuthal angle.\n\nNote that SciPy's spherical harmonics include the Condon-Shortley\nphase [2]_ because it is part of `lpmv`.\n\nWith SciPy's conventions, the first several spherical harmonics\nare\n\n.. math::\n\n    Y_0^0(\\theta, \\phi) &= \\frac{1}{2} \\sqrt{\\frac{1}{\\pi}} \\\\\n    Y_1^{-1}(\\theta, \\phi) &= \\frac{1}{2} \\sqrt{\\frac{3}{2\\pi}}\n                                e^{-i\\theta} \\sin(\\phi) \\\\\n    Y_1^0(\\theta, \\phi) &= \\frac{1}{2} \\sqrt{\\frac{3}{\\pi}}\n                             \\cos(\\phi) \\\\\n    Y_1^1(\\theta, \\phi) &= -\\frac{1}{2} \\sqrt{\\frac{3}{2\\pi}}\n                             e^{i\\theta} \\sin(\\phi).\n\nReferences\n----------\n.. [1] Digital Library of Mathematical Functions, 14.30.\n       https://dlmf.nist.gov/14.30\n.. [2] https://en.wikipedia.org/wiki/Spherical_harmonics#Condon.E2.80.9""3Shortley_phase");

  /* "scipy/special/_ufuncs.pyx":13561
 *     "       https://dlmf.nist.gov/14.30\n"
 *     ".. [2] https://en.wikipedia.org/wiki/Spherical_harmonics#Condon.E2.80.93Shortley_phase")
 * ufunc_sph_harm_loops[0] = <np.PyUFuncGenericFunction>loop_D_iidd__As_lldd_D             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_loops[1] = <np.PyUFuncGenericFunction>loop_D_dddd__As_ffff_F
 * ufunc_sph_harm_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddd__As_dddd_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_iidd__As_lldd_D);

  /* "scipy/special/_ufuncs.pyx":13562
 *     ".. [2] https://en.wikipedia.org/wiki/Spherical_harmonics#Condon.E2.80.93Shortley_phase")
 * ufunc_sph_harm_loops[0] = <np.PyUFuncGenericFunction>loop_D_iidd__As_lldd_D
 * ufunc_sph_harm_loops[1] = <np.PyUFuncGenericFunction>loop_D_dddd__As_ffff_F             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddd__As_dddd_D
 * ufunc_sph_harm_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dddd__As_ffff_F);

  /* "scipy/special/_ufuncs.pyx":13563
 * ufunc_sph_harm_loops[0] = <np.PyUFuncGenericFunction>loop_D_iidd__As_lldd_D
 * ufunc_sph_harm_loops[1] = <np.PyUFuncGenericFunction>loop_D_dddd__As_ffff_F
 * ufunc_sph_harm_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddd__As_dddd_D             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[0] = <char>NPY_LONG
 * ufunc_sph_harm_types[1] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dddd__As_dddd_D);

  /* "scipy/special/_ufuncs.pyx":13564
 * ufunc_sph_harm_loops[1] = <np.PyUFuncGenericFunction>loop_D_dddd__As_ffff_F
 * ufunc_sph_harm_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddd__As_dddd_D
 * ufunc_sph_harm_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[1] = <char>NPY_LONG
 * ufunc_sph_harm_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":13565
 * ufunc_sph_harm_loops[2] = <np.PyUFuncGenericFunction>loop_D_dddd__As_dddd_D
 * ufunc_sph_harm_types[0] = <char>NPY_LONG
 * ufunc_sph_harm_types[1] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[2] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[1]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":13566
 * ufunc_sph_harm_types[0] = <char>NPY_LONG
 * ufunc_sph_harm_types[1] = <char>NPY_LONG
 * ufunc_sph_harm_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[3] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[4] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13567
 * ufunc_sph_harm_types[1] = <char>NPY_LONG
 * ufunc_sph_harm_types[2] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[4] = <char>NPY_CDOUBLE
 * ufunc_sph_harm_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13568
 * ufunc_sph_harm_types[2] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[3] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[4] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[5] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[6] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[4]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13569
 * ufunc_sph_harm_types[3] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[4] = <char>NPY_CDOUBLE
 * ufunc_sph_harm_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[6] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[7] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13570
 * ufunc_sph_harm_types[4] = <char>NPY_CDOUBLE
 * ufunc_sph_harm_types[5] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[6] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[7] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[8] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[6]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13571
 * ufunc_sph_harm_types[5] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[6] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[7] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[8] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[9] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[7]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13572
 * ufunc_sph_harm_types[6] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[7] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[8] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[9] = <char>NPY_CFLOAT
 * ufunc_sph_harm_types[10] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[8]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13573
 * ufunc_sph_harm_types[7] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[8] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[9] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[10] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[11] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[9]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13574
 * ufunc_sph_harm_types[8] = <char>NPY_FLOAT
 * ufunc_sph_harm_types[9] = <char>NPY_CFLOAT
 * ufunc_sph_harm_types[10] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[11] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[12] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[10]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13575
 * ufunc_sph_harm_types[9] = <char>NPY_CFLOAT
 * ufunc_sph_harm_types[10] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[11] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[12] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[13] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[11]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13576
 * ufunc_sph_harm_types[10] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[11] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[12] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[13] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[14] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[12]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13577
 * ufunc_sph_harm_types[11] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[12] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[13] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_types[14] = <char>NPY_CDOUBLE
 * ufunc_sph_harm_ptr[2*0] = <void*>_func_sph_harmonic
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[13]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13578
 * ufunc_sph_harm_types[12] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[13] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[14] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_ptr[2*0] = <void*>_func_sph_harmonic
 * ufunc_sph_harm_ptr[2*0+1] = <void*>(<char*>"sph_harm")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types[14]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13579
 * ufunc_sph_harm_types[13] = <char>NPY_DOUBLE
 * ufunc_sph_harm_types[14] = <char>NPY_CDOUBLE
 * ufunc_sph_harm_ptr[2*0] = <void*>_func_sph_harmonic             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_ptr[2*0+1] = <void*>(<char*>"sph_harm")
 * ufunc_sph_harm_ptr[2*1] = <void*>_func_sph_harmonic_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_ptr[0]) = ((void *)__pyx_f_5scipy_7special_8sph_harm_sph_harmonic);

  /* "scipy/special/_ufuncs.pyx":13580
 * ufunc_sph_harm_types[14] = <char>NPY_CDOUBLE
 * ufunc_sph_harm_ptr[2*0] = <void*>_func_sph_harmonic
 * ufunc_sph_harm_ptr[2*0+1] = <void*>(<char*>"sph_harm")             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_ptr[2*1] = <void*>_func_sph_harmonic_unsafe
 * ufunc_sph_harm_ptr[2*1+1] = <void*>(<char*>"sph_harm")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_ptr[1]) = ((void *)((char *)((char *)"sph_harm")));

  /* "scipy/special/_ufuncs.pyx":13581
 * ufunc_sph_harm_ptr[2*0] = <void*>_func_sph_harmonic
 * ufunc_sph_harm_ptr[2*0+1] = <void*>(<char*>"sph_harm")
 * ufunc_sph_harm_ptr[2*1] = <void*>_func_sph_harmonic_unsafe             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_ptr[2*1+1] = <void*>(<char*>"sph_harm")
 * ufunc_sph_harm_ptr[2*2] = <void*>_func_sph_harmonic_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_sph_harmonic_unsafe);

  /* "scipy/special/_ufuncs.pyx":13582
 * ufunc_sph_harm_ptr[2*0+1] = <void*>(<char*>"sph_harm")
 * ufunc_sph_harm_ptr[2*1] = <void*>_func_sph_harmonic_unsafe
 * ufunc_sph_harm_ptr[2*1+1] = <void*>(<char*>"sph_harm")             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_ptr[2*2] = <void*>_func_sph_harmonic_unsafe
 * ufunc_sph_harm_ptr[2*2+1] = <void*>(<char*>"sph_harm")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_ptr[3]) = ((void *)((char *)((char *)"sph_harm")));

  /* "scipy/special/_ufuncs.pyx":13583
 * ufunc_sph_harm_ptr[2*1] = <void*>_func_sph_harmonic_unsafe
 * ufunc_sph_harm_ptr[2*1+1] = <void*>(<char*>"sph_harm")
 * ufunc_sph_harm_ptr[2*2] = <void*>_func_sph_harmonic_unsafe             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_ptr[2*2+1] = <void*>(<char*>"sph_harm")
 * ufunc_sph_harm_data[0] = &ufunc_sph_harm_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_sph_harmonic_unsafe);

  /* "scipy/special/_ufuncs.pyx":13584
 * ufunc_sph_harm_ptr[2*1+1] = <void*>(<char*>"sph_harm")
 * ufunc_sph_harm_ptr[2*2] = <void*>_func_sph_harmonic_unsafe
 * ufunc_sph_harm_ptr[2*2+1] = <void*>(<char*>"sph_harm")             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_data[0] = &ufunc_sph_harm_ptr[2*0]
 * ufunc_sph_harm_data[1] = &ufunc_sph_harm_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_ptr[5]) = ((void *)((char *)((char *)"sph_harm")));

  /* "scipy/special/_ufuncs.pyx":13585
 * ufunc_sph_harm_ptr[2*2] = <void*>_func_sph_harmonic_unsafe
 * ufunc_sph_harm_ptr[2*2+1] = <void*>(<char*>"sph_harm")
 * ufunc_sph_harm_data[0] = &ufunc_sph_harm_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_data[1] = &ufunc_sph_harm_ptr[2*1]
 * ufunc_sph_harm_data[2] = &ufunc_sph_harm_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":13586
 * ufunc_sph_harm_ptr[2*2+1] = <void*>(<char*>"sph_harm")
 * ufunc_sph_harm_data[0] = &ufunc_sph_harm_ptr[2*0]
 * ufunc_sph_harm_data[1] = &ufunc_sph_harm_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_sph_harm_data[2] = &ufunc_sph_harm_ptr[2*2]
 * sph_harm = np.PyUFunc_FromFuncAndData(ufunc_sph_harm_loops, ufunc_sph_harm_data, ufunc_sph_harm_types, 3, 4, 1, 0, "sph_harm", ufunc_sph_harm_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":13587
 * ufunc_sph_harm_data[0] = &ufunc_sph_harm_ptr[2*0]
 * ufunc_sph_harm_data[1] = &ufunc_sph_harm_ptr[2*1]
 * ufunc_sph_harm_data[2] = &ufunc_sph_harm_ptr[2*2]             # <<<<<<<<<<<<<<
 * sph_harm = np.PyUFunc_FromFuncAndData(ufunc_sph_harm_loops, ufunc_sph_harm_data, ufunc_sph_harm_types, 3, 4, 1, 0, "sph_harm", ufunc_sph_harm_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":13588
 * ufunc_sph_harm_data[1] = &ufunc_sph_harm_ptr[2*1]
 * ufunc_sph_harm_data[2] = &ufunc_sph_harm_ptr[2*2]
 * sph_harm = np.PyUFunc_FromFuncAndData(ufunc_sph_harm_loops, ufunc_sph_harm_data, ufunc_sph_harm_types, 3, 4, 1, 0, "sph_harm", ufunc_sph_harm_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_stdtr_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_types, 3, 4, 1, 0, ((char *)"sph_harm"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_sph_harm_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13588, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sph_harm, __pyx_t_2) < 0) __PYX_ERR(1, 13588, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":13594
 * cdef void *ufunc_stdtr_data[2]
 * cdef char ufunc_stdtr_types[6]
 * cdef char *ufunc_stdtr_doc = (             # <<<<<<<<<<<<<<
 *     "stdtr(df, t)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_doc = ((char *)"stdtr(df, t)\n\nStudent t distribution cumulative distribution function\n\nReturns the integral from minus infinity to t of the Student t\ndistribution with df > 0 degrees of freedom::\n\n   gamma((df+1)/2)/(sqrt(df*pi)*gamma(df/2)) *\n   integral((1+x**2/df)**(-df/2-1/2), x=-inf..t)");

  /* "scipy/special/_ufuncs.pyx":13604
 *     "   gamma((df+1)/2)/(sqrt(df*pi)*gamma(df/2)) *\n"
 *     "   integral((1+x**2/df)**(-df/2-1/2), x=-inf..t)")
 * ufunc_stdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_stdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_stdtr_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":13605
 *     "   integral((1+x**2/df)**(-df/2-1/2), x=-inf..t)")
 * ufunc_stdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_stdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_stdtr_types[0] = <char>NPY_FLOAT
 * ufunc_stdtr_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":13606
 * ufunc_stdtr_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_stdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_stdtr_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_stdtr_types[1] = <char>NPY_FLOAT
 * ufunc_stdtr_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13607
 * ufunc_stdtr_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_stdtr_types[0] = <char>NPY_FLOAT
 * ufunc_stdtr_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_stdtr_types[2] = <char>NPY_FLOAT
 * ufunc_stdtr_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13608
 * ufunc_stdtr_types[0] = <char>NPY_FLOAT
 * ufunc_stdtr_types[1] = <char>NPY_FLOAT
 * ufunc_stdtr_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_stdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_stdtr_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13609
 * ufunc_stdtr_types[1] = <char>NPY_FLOAT
 * ufunc_stdtr_types[2] = <char>NPY_FLOAT
 * ufunc_stdtr_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_stdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_stdtr_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13610
 * ufunc_stdtr_types[2] = <char>NPY_FLOAT
 * ufunc_stdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_stdtr_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_stdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_stdtr_ptr[2*0] = <void*>_func_cdft1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13611
 * ufunc_stdtr_types[3] = <char>NPY_DOUBLE
 * ufunc_stdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_stdtr_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_stdtr_ptr[2*0] = <void*>_func_cdft1_wrap
 * ufunc_stdtr_ptr[2*0+1] = <void*>(<char*>"stdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13612
 * ufunc_stdtr_types[4] = <char>NPY_DOUBLE
 * ufunc_stdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_stdtr_ptr[2*0] = <void*>_func_cdft1_wrap             # <<<<<<<<<<<<<<
 * ufunc_stdtr_ptr[2*0+1] = <void*>(<char*>"stdtr")
 * ufunc_stdtr_ptr[2*1] = <void*>_func_cdft1_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_ptr[0]) = ((void *)cdft1_wrap);

  /* "scipy/special/_ufuncs.pyx":13613
 * ufunc_stdtr_types[5] = <char>NPY_DOUBLE
 * ufunc_stdtr_ptr[2*0] = <void*>_func_cdft1_wrap
 * ufunc_stdtr_ptr[2*0+1] = <void*>(<char*>"stdtr")             # <<<<<<<<<<<<<<
 * ufunc_stdtr_ptr[2*1] = <void*>_func_cdft1_wrap
 * ufunc_stdtr_ptr[2*1+1] = <void*>(<char*>"stdtr")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_ptr[1]) = ((void *)((char *)((char *)"stdtr")));

  /* "scipy/special/_ufuncs.pyx":13614
 * ufunc_stdtr_ptr[2*0] = <void*>_func_cdft1_wrap
 * ufunc_stdtr_ptr[2*0+1] = <void*>(<char*>"stdtr")
 * ufunc_stdtr_ptr[2*1] = <void*>_func_cdft1_wrap             # <<<<<<<<<<<<<<
 * ufunc_stdtr_ptr[2*1+1] = <void*>(<char*>"stdtr")
 * ufunc_stdtr_data[0] = &ufunc_stdtr_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_ptr[2]) = ((void *)cdft1_wrap);

  /* "scipy/special/_ufuncs.pyx":13615
 * ufunc_stdtr_ptr[2*0+1] = <void*>(<char*>"stdtr")
 * ufunc_stdtr_ptr[2*1] = <void*>_func_cdft1_wrap
 * ufunc_stdtr_ptr[2*1+1] = <void*>(<char*>"stdtr")             # <<<<<<<<<<<<<<
 * ufunc_stdtr_data[0] = &ufunc_stdtr_ptr[2*0]
 * ufunc_stdtr_data[1] = &ufunc_stdtr_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_ptr[3]) = ((void *)((char *)((char *)"stdtr")));

  /* "scipy/special/_ufuncs.pyx":13616
 * ufunc_stdtr_ptr[2*1] = <void*>_func_cdft1_wrap
 * ufunc_stdtr_ptr[2*1+1] = <void*>(<char*>"stdtr")
 * ufunc_stdtr_data[0] = &ufunc_stdtr_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_stdtr_data[1] = &ufunc_stdtr_ptr[2*1]
 * stdtr = np.PyUFunc_FromFuncAndData(ufunc_stdtr_loops, ufunc_stdtr_data, ufunc_stdtr_types, 2, 2, 1, 0, "stdtr", ufunc_stdtr_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":13617
 * ufunc_stdtr_ptr[2*1+1] = <void*>(<char*>"stdtr")
 * ufunc_stdtr_data[0] = &ufunc_stdtr_ptr[2*0]
 * ufunc_stdtr_data[1] = &ufunc_stdtr_ptr[2*1]             # <<<<<<<<<<<<<<
 * stdtr = np.PyUFunc_FromFuncAndData(ufunc_stdtr_loops, ufunc_stdtr_data, ufunc_stdtr_types, 2, 2, 1, 0, "stdtr", ufunc_stdtr_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":13618
 * ufunc_stdtr_data[0] = &ufunc_stdtr_ptr[2*0]
 * ufunc_stdtr_data[1] = &ufunc_stdtr_ptr[2*1]
 * stdtr = np.PyUFunc_FromFuncAndData(ufunc_stdtr_loops, ufunc_stdtr_data, ufunc_stdtr_types, 2, 2, 1, 0, "stdtr", ufunc_stdtr_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_stdtridf_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_types, 2, 2, 1, 0, ((char *)"stdtr"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtr_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_stdtr, __pyx_t_2) < 0) __PYX_ERR(1, 13618, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":13624
 * cdef void *ufunc_stdtridf_data[2]
 * cdef char ufunc_stdtridf_types[6]
 * cdef char *ufunc_stdtridf_doc = (             # <<<<<<<<<<<<<<
 *     "stdtridf(p, t)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_doc = ((char *)"stdtridf(p, t)\n\nInverse of `stdtr` vs df\n\nReturns the argument df such that stdtr(df, t) is equal to `p`.");

  /* "scipy/special/_ufuncs.pyx":13630
 *     "\n"
 *     "Returns the argument df such that stdtr(df, t) is equal to `p`.")
 * ufunc_stdtridf_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_stdtridf_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":13631
 *     "Returns the argument df such that stdtr(df, t) is equal to `p`.")
 * ufunc_stdtridf_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_stdtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_types[0] = <char>NPY_FLOAT
 * ufunc_stdtridf_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":13632
 * ufunc_stdtridf_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_stdtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_stdtridf_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_types[1] = <char>NPY_FLOAT
 * ufunc_stdtridf_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13633
 * ufunc_stdtridf_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_stdtridf_types[0] = <char>NPY_FLOAT
 * ufunc_stdtridf_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_types[2] = <char>NPY_FLOAT
 * ufunc_stdtridf_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13634
 * ufunc_stdtridf_types[0] = <char>NPY_FLOAT
 * ufunc_stdtridf_types[1] = <char>NPY_FLOAT
 * ufunc_stdtridf_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_types[3] = <char>NPY_DOUBLE
 * ufunc_stdtridf_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13635
 * ufunc_stdtridf_types[1] = <char>NPY_FLOAT
 * ufunc_stdtridf_types[2] = <char>NPY_FLOAT
 * ufunc_stdtridf_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_types[4] = <char>NPY_DOUBLE
 * ufunc_stdtridf_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13636
 * ufunc_stdtridf_types[2] = <char>NPY_FLOAT
 * ufunc_stdtridf_types[3] = <char>NPY_DOUBLE
 * ufunc_stdtridf_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_types[5] = <char>NPY_DOUBLE
 * ufunc_stdtridf_ptr[2*0] = <void*>_func_cdft3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13637
 * ufunc_stdtridf_types[3] = <char>NPY_DOUBLE
 * ufunc_stdtridf_types[4] = <char>NPY_DOUBLE
 * ufunc_stdtridf_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_ptr[2*0] = <void*>_func_cdft3_wrap
 * ufunc_stdtridf_ptr[2*0+1] = <void*>(<char*>"stdtridf")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13638
 * ufunc_stdtridf_types[4] = <char>NPY_DOUBLE
 * ufunc_stdtridf_types[5] = <char>NPY_DOUBLE
 * ufunc_stdtridf_ptr[2*0] = <void*>_func_cdft3_wrap             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_ptr[2*0+1] = <void*>(<char*>"stdtridf")
 * ufunc_stdtridf_ptr[2*1] = <void*>_func_cdft3_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_ptr[0]) = ((void *)cdft3_wrap);

  /* "scipy/special/_ufuncs.pyx":13639
 * ufunc_stdtridf_types[5] = <char>NPY_DOUBLE
 * ufunc_stdtridf_ptr[2*0] = <void*>_func_cdft3_wrap
 * ufunc_stdtridf_ptr[2*0+1] = <void*>(<char*>"stdtridf")             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_ptr[2*1] = <void*>_func_cdft3_wrap
 * ufunc_stdtridf_ptr[2*1+1] = <void*>(<char*>"stdtridf")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_ptr[1]) = ((void *)((char *)((char *)"stdtridf")));

  /* "scipy/special/_ufuncs.pyx":13640
 * ufunc_stdtridf_ptr[2*0] = <void*>_func_cdft3_wrap
 * ufunc_stdtridf_ptr[2*0+1] = <void*>(<char*>"stdtridf")
 * ufunc_stdtridf_ptr[2*1] = <void*>_func_cdft3_wrap             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_ptr[2*1+1] = <void*>(<char*>"stdtridf")
 * ufunc_stdtridf_data[0] = &ufunc_stdtridf_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_ptr[2]) = ((void *)cdft3_wrap);

  /* "scipy/special/_ufuncs.pyx":13641
 * ufunc_stdtridf_ptr[2*0+1] = <void*>(<char*>"stdtridf")
 * ufunc_stdtridf_ptr[2*1] = <void*>_func_cdft3_wrap
 * ufunc_stdtridf_ptr[2*1+1] = <void*>(<char*>"stdtridf")             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_data[0] = &ufunc_stdtridf_ptr[2*0]
 * ufunc_stdtridf_data[1] = &ufunc_stdtridf_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_ptr[3]) = ((void *)((char *)((char *)"stdtridf")));

  /* "scipy/special/_ufuncs.pyx":13642
 * ufunc_stdtridf_ptr[2*1] = <void*>_func_cdft3_wrap
 * ufunc_stdtridf_ptr[2*1+1] = <void*>(<char*>"stdtridf")
 * ufunc_stdtridf_data[0] = &ufunc_stdtridf_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_stdtridf_data[1] = &ufunc_stdtridf_ptr[2*1]
 * stdtridf = np.PyUFunc_FromFuncAndData(ufunc_stdtridf_loops, ufunc_stdtridf_data, ufunc_stdtridf_types, 2, 2, 1, 0, "stdtridf", ufunc_stdtridf_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":13643
 * ufunc_stdtridf_ptr[2*1+1] = <void*>(<char*>"stdtridf")
 * ufunc_stdtridf_data[0] = &ufunc_stdtridf_ptr[2*0]
 * ufunc_stdtridf_data[1] = &ufunc_stdtridf_ptr[2*1]             # <<<<<<<<<<<<<<
 * stdtridf = np.PyUFunc_FromFuncAndData(ufunc_stdtridf_loops, ufunc_stdtridf_data, ufunc_stdtridf_types, 2, 2, 1, 0, "stdtridf", ufunc_stdtridf_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":13644
 * ufunc_stdtridf_data[0] = &ufunc_stdtridf_ptr[2*0]
 * ufunc_stdtridf_data[1] = &ufunc_stdtridf_ptr[2*1]
 * stdtridf = np.PyUFunc_FromFuncAndData(ufunc_stdtridf_loops, ufunc_stdtridf_data, ufunc_stdtridf_types, 2, 2, 1, 0, "stdtridf", ufunc_stdtridf_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_stdtrit_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_types, 2, 2, 1, 0, ((char *)"stdtridf"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtridf_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13644, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_stdtridf, __pyx_t_2) < 0) __PYX_ERR(1, 13644, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":13650
 * cdef void *ufunc_stdtrit_data[2]
 * cdef char ufunc_stdtrit_types[6]
 * cdef char *ufunc_stdtrit_doc = (             # <<<<<<<<<<<<<<
 *     "stdtrit(df, p)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_doc = ((char *)"stdtrit(df, p)\n\nInverse of `stdtr` vs `t`\n\nReturns the argument `t` such that stdtr(df, t) is equal to `p`.");

  /* "scipy/special/_ufuncs.pyx":13656
 *     "\n"
 *     "Returns the argument `t` such that stdtr(df, t) is equal to `p`.")
 * ufunc_stdtrit_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_stdtrit_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":13657
 *     "Returns the argument `t` such that stdtr(df, t) is equal to `p`.")
 * ufunc_stdtrit_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_stdtrit_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_types[0] = <char>NPY_FLOAT
 * ufunc_stdtrit_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":13658
 * ufunc_stdtrit_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_stdtrit_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_stdtrit_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_types[1] = <char>NPY_FLOAT
 * ufunc_stdtrit_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13659
 * ufunc_stdtrit_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_stdtrit_types[0] = <char>NPY_FLOAT
 * ufunc_stdtrit_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_types[2] = <char>NPY_FLOAT
 * ufunc_stdtrit_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13660
 * ufunc_stdtrit_types[0] = <char>NPY_FLOAT
 * ufunc_stdtrit_types[1] = <char>NPY_FLOAT
 * ufunc_stdtrit_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_types[3] = <char>NPY_DOUBLE
 * ufunc_stdtrit_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13661
 * ufunc_stdtrit_types[1] = <char>NPY_FLOAT
 * ufunc_stdtrit_types[2] = <char>NPY_FLOAT
 * ufunc_stdtrit_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_types[4] = <char>NPY_DOUBLE
 * ufunc_stdtrit_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13662
 * ufunc_stdtrit_types[2] = <char>NPY_FLOAT
 * ufunc_stdtrit_types[3] = <char>NPY_DOUBLE
 * ufunc_stdtrit_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_types[5] = <char>NPY_DOUBLE
 * ufunc_stdtrit_ptr[2*0] = <void*>_func_cdft2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13663
 * ufunc_stdtrit_types[3] = <char>NPY_DOUBLE
 * ufunc_stdtrit_types[4] = <char>NPY_DOUBLE
 * ufunc_stdtrit_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_ptr[2*0] = <void*>_func_cdft2_wrap
 * ufunc_stdtrit_ptr[2*0+1] = <void*>(<char*>"stdtrit")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13664
 * ufunc_stdtrit_types[4] = <char>NPY_DOUBLE
 * ufunc_stdtrit_types[5] = <char>NPY_DOUBLE
 * ufunc_stdtrit_ptr[2*0] = <void*>_func_cdft2_wrap             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_ptr[2*0+1] = <void*>(<char*>"stdtrit")
 * ufunc_stdtrit_ptr[2*1] = <void*>_func_cdft2_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_ptr[0]) = ((void *)cdft2_wrap);

  /* "scipy/special/_ufuncs.pyx":13665
 * ufunc_stdtrit_types[5] = <char>NPY_DOUBLE
 * ufunc_stdtrit_ptr[2*0] = <void*>_func_cdft2_wrap
 * ufunc_stdtrit_ptr[2*0+1] = <void*>(<char*>"stdtrit")             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_ptr[2*1] = <void*>_func_cdft2_wrap
 * ufunc_stdtrit_ptr[2*1+1] = <void*>(<char*>"stdtrit")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_ptr[1]) = ((void *)((char *)((char *)"stdtrit")));

  /* "scipy/special/_ufuncs.pyx":13666
 * ufunc_stdtrit_ptr[2*0] = <void*>_func_cdft2_wrap
 * ufunc_stdtrit_ptr[2*0+1] = <void*>(<char*>"stdtrit")
 * ufunc_stdtrit_ptr[2*1] = <void*>_func_cdft2_wrap             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_ptr[2*1+1] = <void*>(<char*>"stdtrit")
 * ufunc_stdtrit_data[0] = &ufunc_stdtrit_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_ptr[2]) = ((void *)cdft2_wrap);

  /* "scipy/special/_ufuncs.pyx":13667
 * ufunc_stdtrit_ptr[2*0+1] = <void*>(<char*>"stdtrit")
 * ufunc_stdtrit_ptr[2*1] = <void*>_func_cdft2_wrap
 * ufunc_stdtrit_ptr[2*1+1] = <void*>(<char*>"stdtrit")             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_data[0] = &ufunc_stdtrit_ptr[2*0]
 * ufunc_stdtrit_data[1] = &ufunc_stdtrit_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_ptr[3]) = ((void *)((char *)((char *)"stdtrit")));

  /* "scipy/special/_ufuncs.pyx":13668
 * ufunc_stdtrit_ptr[2*1] = <void*>_func_cdft2_wrap
 * ufunc_stdtrit_ptr[2*1+1] = <void*>(<char*>"stdtrit")
 * ufunc_stdtrit_data[0] = &ufunc_stdtrit_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_stdtrit_data[1] = &ufunc_stdtrit_ptr[2*1]
 * stdtrit = np.PyUFunc_FromFuncAndData(ufunc_stdtrit_loops, ufunc_stdtrit_data, ufunc_stdtrit_types, 2, 2, 1, 0, "stdtrit", ufunc_stdtrit_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":13669
 * ufunc_stdtrit_ptr[2*1+1] = <void*>(<char*>"stdtrit")
 * ufunc_stdtrit_data[0] = &ufunc_stdtrit_ptr[2*0]
 * ufunc_stdtrit_data[1] = &ufunc_stdtrit_ptr[2*1]             # <<<<<<<<<<<<<<
 * stdtrit = np.PyUFunc_FromFuncAndData(ufunc_stdtrit_loops, ufunc_stdtrit_data, ufunc_stdtrit_types, 2, 2, 1, 0, "stdtrit", ufunc_stdtrit_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":13670
 * ufunc_stdtrit_data[0] = &ufunc_stdtrit_ptr[2*0]
 * ufunc_stdtrit_data[1] = &ufunc_stdtrit_ptr[2*1]
 * stdtrit = np.PyUFunc_FromFuncAndData(ufunc_stdtrit_loops, ufunc_stdtrit_data, ufunc_stdtrit_types, 2, 2, 1, 0, "stdtrit", ufunc_stdtrit_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_struve_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_types, 2, 2, 1, 0, ((char *)"stdtrit"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_stdtrit_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13670, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_stdtrit, __pyx_t_2) < 0) __PYX_ERR(1, 13670, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":13676
 * cdef void *ufunc_struve_data[2]
 * cdef char ufunc_struve_types[6]
 * cdef char *ufunc_struve_doc = (             # <<<<<<<<<<<<<<
 *     "struve(v, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_doc = ((char *)"struve(v, x)\n\nStruve function.\n\nReturn the value of the Struve function of order `v` at `x`.  The Struve\nfunction is defined as,\n\n.. math::\n    H_v(x) = (z/2)^{v + 1} \\sum_{n=0}^\\infty \\frac{(-1)^n (z/2)^{2n}}{\\Gamma(n + \\frac{3}{2}) \\Gamma(n + v + \\frac{3}{2})},\n\nwhere :math:`\\Gamma` is the gamma function.\n\nParameters\n----------\nv : array_like\n    Order of the Struve function (float).\nx : array_like\n    Argument of the Struve function (float; must be positive unless `v` is\n    an integer).\n\nReturns\n-------\nH : ndarray\n    Value of the Struve function of order `v` at `x`.\n\nNotes\n-----\nThree methods discussed in [1]_ are used to evaluate the Struve function:\n\n- power series\n- expansion in Bessel functions (if :math:`|z| < |v| + 20`)\n- asymptotic large-z expansion (if :math:`z \\geq 0.7v + 12`)\n\nRounding errors are estimated based on the largest terms in the sums, and\nthe result associated with the smallest error is returned.\n\nSee also\n--------\nmodstruve\n\nReferences\n----------\n.. [1] NIST Digital Library of Mathematical Functions\n       https://dlmf.nist.gov/11");

  /* "scipy/special/_ufuncs.pyx":13721
 *     ".. [1] NIST Digital Library of Mathematical Functions\n"
 *     "       https://dlmf.nist.gov/11")
 * ufunc_struve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_struve_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_struve_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":13722
 *     "       https://dlmf.nist.gov/11")
 * ufunc_struve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_struve_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_struve_types[0] = <char>NPY_FLOAT
 * ufunc_struve_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":13723
 * ufunc_struve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_struve_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_struve_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_struve_types[1] = <char>NPY_FLOAT
 * ufunc_struve_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13724
 * ufunc_struve_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_struve_types[0] = <char>NPY_FLOAT
 * ufunc_struve_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_struve_types[2] = <char>NPY_FLOAT
 * ufunc_struve_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13725
 * ufunc_struve_types[0] = <char>NPY_FLOAT
 * ufunc_struve_types[1] = <char>NPY_FLOAT
 * ufunc_struve_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_struve_types[3] = <char>NPY_DOUBLE
 * ufunc_struve_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13726
 * ufunc_struve_types[1] = <char>NPY_FLOAT
 * ufunc_struve_types[2] = <char>NPY_FLOAT
 * ufunc_struve_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_struve_types[4] = <char>NPY_DOUBLE
 * ufunc_struve_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13727
 * ufunc_struve_types[2] = <char>NPY_FLOAT
 * ufunc_struve_types[3] = <char>NPY_DOUBLE
 * ufunc_struve_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_struve_types[5] = <char>NPY_DOUBLE
 * ufunc_struve_ptr[2*0] = <void*>_func_struve_h
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13728
 * ufunc_struve_types[3] = <char>NPY_DOUBLE
 * ufunc_struve_types[4] = <char>NPY_DOUBLE
 * ufunc_struve_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_struve_ptr[2*0] = <void*>_func_struve_h
 * ufunc_struve_ptr[2*0+1] = <void*>(<char*>"struve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13729
 * ufunc_struve_types[4] = <char>NPY_DOUBLE
 * ufunc_struve_types[5] = <char>NPY_DOUBLE
 * ufunc_struve_ptr[2*0] = <void*>_func_struve_h             # <<<<<<<<<<<<<<
 * ufunc_struve_ptr[2*0+1] = <void*>(<char*>"struve")
 * ufunc_struve_ptr[2*1] = <void*>_func_struve_h
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_ptr[0]) = ((void *)struve_h);

  /* "scipy/special/_ufuncs.pyx":13730
 * ufunc_struve_types[5] = <char>NPY_DOUBLE
 * ufunc_struve_ptr[2*0] = <void*>_func_struve_h
 * ufunc_struve_ptr[2*0+1] = <void*>(<char*>"struve")             # <<<<<<<<<<<<<<
 * ufunc_struve_ptr[2*1] = <void*>_func_struve_h
 * ufunc_struve_ptr[2*1+1] = <void*>(<char*>"struve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_ptr[1]) = ((void *)((char *)((char *)"struve")));

  /* "scipy/special/_ufuncs.pyx":13731
 * ufunc_struve_ptr[2*0] = <void*>_func_struve_h
 * ufunc_struve_ptr[2*0+1] = <void*>(<char*>"struve")
 * ufunc_struve_ptr[2*1] = <void*>_func_struve_h             # <<<<<<<<<<<<<<
 * ufunc_struve_ptr[2*1+1] = <void*>(<char*>"struve")
 * ufunc_struve_data[0] = &ufunc_struve_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_ptr[2]) = ((void *)struve_h);

  /* "scipy/special/_ufuncs.pyx":13732
 * ufunc_struve_ptr[2*0+1] = <void*>(<char*>"struve")
 * ufunc_struve_ptr[2*1] = <void*>_func_struve_h
 * ufunc_struve_ptr[2*1+1] = <void*>(<char*>"struve")             # <<<<<<<<<<<<<<
 * ufunc_struve_data[0] = &ufunc_struve_ptr[2*0]
 * ufunc_struve_data[1] = &ufunc_struve_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_ptr[3]) = ((void *)((char *)((char *)"struve")));

  /* "scipy/special/_ufuncs.pyx":13733
 * ufunc_struve_ptr[2*1] = <void*>_func_struve_h
 * ufunc_struve_ptr[2*1+1] = <void*>(<char*>"struve")
 * ufunc_struve_data[0] = &ufunc_struve_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_struve_data[1] = &ufunc_struve_ptr[2*1]
 * struve = np.PyUFunc_FromFuncAndData(ufunc_struve_loops, ufunc_struve_data, ufunc_struve_types, 2, 2, 1, 0, "struve", ufunc_struve_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":13734
 * ufunc_struve_ptr[2*1+1] = <void*>(<char*>"struve")
 * ufunc_struve_data[0] = &ufunc_struve_ptr[2*0]
 * ufunc_struve_data[1] = &ufunc_struve_ptr[2*1]             # <<<<<<<<<<<<<<
 * struve = np.PyUFunc_FromFuncAndData(ufunc_struve_loops, ufunc_struve_data, ufunc_struve_types, 2, 2, 1, 0, "struve", ufunc_struve_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":13735
 * ufunc_struve_data[0] = &ufunc_struve_ptr[2*0]
 * ufunc_struve_data[1] = &ufunc_struve_ptr[2*1]
 * struve = np.PyUFunc_FromFuncAndData(ufunc_struve_loops, ufunc_struve_data, ufunc_struve_types, 2, 2, 1, 0, "struve", ufunc_struve_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_tandg_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_types, 2, 2, 1, 0, ((char *)"struve"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_struve_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13735, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_struve, __pyx_t_2) < 0) __PYX_ERR(1, 13735, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":13741
 * cdef void *ufunc_tandg_data[2]
 * cdef char ufunc_tandg_types[4]
 * cdef char *ufunc_tandg_doc = (             # <<<<<<<<<<<<<<
 *     "tandg(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_doc = ((char *)"tandg(x)\n\nTangent of angle x given in degrees.");

  /* "scipy/special/_ufuncs.pyx":13745
 *     "\n"
 *     "Tangent of angle x given in degrees.")
 * ufunc_tandg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_tandg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_tandg_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":13746
 *     "Tangent of angle x given in degrees.")
 * ufunc_tandg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_tandg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_tandg_types[0] = <char>NPY_FLOAT
 * ufunc_tandg_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":13747
 * ufunc_tandg_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_tandg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_tandg_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_tandg_types[1] = <char>NPY_FLOAT
 * ufunc_tandg_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13748
 * ufunc_tandg_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_tandg_types[0] = <char>NPY_FLOAT
 * ufunc_tandg_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_tandg_types[2] = <char>NPY_DOUBLE
 * ufunc_tandg_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13749
 * ufunc_tandg_types[0] = <char>NPY_FLOAT
 * ufunc_tandg_types[1] = <char>NPY_FLOAT
 * ufunc_tandg_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_tandg_types[3] = <char>NPY_DOUBLE
 * ufunc_tandg_ptr[2*0] = <void*>_func_tandg
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13750
 * ufunc_tandg_types[1] = <char>NPY_FLOAT
 * ufunc_tandg_types[2] = <char>NPY_DOUBLE
 * ufunc_tandg_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_tandg_ptr[2*0] = <void*>_func_tandg
 * ufunc_tandg_ptr[2*0+1] = <void*>(<char*>"tandg")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13751
 * ufunc_tandg_types[2] = <char>NPY_DOUBLE
 * ufunc_tandg_types[3] = <char>NPY_DOUBLE
 * ufunc_tandg_ptr[2*0] = <void*>_func_tandg             # <<<<<<<<<<<<<<
 * ufunc_tandg_ptr[2*0+1] = <void*>(<char*>"tandg")
 * ufunc_tandg_ptr[2*1] = <void*>_func_tandg
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_ptr[0]) = ((void *)tandg);

  /* "scipy/special/_ufuncs.pyx":13752
 * ufunc_tandg_types[3] = <char>NPY_DOUBLE
 * ufunc_tandg_ptr[2*0] = <void*>_func_tandg
 * ufunc_tandg_ptr[2*0+1] = <void*>(<char*>"tandg")             # <<<<<<<<<<<<<<
 * ufunc_tandg_ptr[2*1] = <void*>_func_tandg
 * ufunc_tandg_ptr[2*1+1] = <void*>(<char*>"tandg")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_ptr[1]) = ((void *)((char *)((char *)"tandg")));

  /* "scipy/special/_ufuncs.pyx":13753
 * ufunc_tandg_ptr[2*0] = <void*>_func_tandg
 * ufunc_tandg_ptr[2*0+1] = <void*>(<char*>"tandg")
 * ufunc_tandg_ptr[2*1] = <void*>_func_tandg             # <<<<<<<<<<<<<<
 * ufunc_tandg_ptr[2*1+1] = <void*>(<char*>"tandg")
 * ufunc_tandg_data[0] = &ufunc_tandg_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_ptr[2]) = ((void *)tandg);

  /* "scipy/special/_ufuncs.pyx":13754
 * ufunc_tandg_ptr[2*0+1] = <void*>(<char*>"tandg")
 * ufunc_tandg_ptr[2*1] = <void*>_func_tandg
 * ufunc_tandg_ptr[2*1+1] = <void*>(<char*>"tandg")             # <<<<<<<<<<<<<<
 * ufunc_tandg_data[0] = &ufunc_tandg_ptr[2*0]
 * ufunc_tandg_data[1] = &ufunc_tandg_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_ptr[3]) = ((void *)((char *)((char *)"tandg")));

  /* "scipy/special/_ufuncs.pyx":13755
 * ufunc_tandg_ptr[2*1] = <void*>_func_tandg
 * ufunc_tandg_ptr[2*1+1] = <void*>(<char*>"tandg")
 * ufunc_tandg_data[0] = &ufunc_tandg_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_tandg_data[1] = &ufunc_tandg_ptr[2*1]
 * tandg = np.PyUFunc_FromFuncAndData(ufunc_tandg_loops, ufunc_tandg_data, ufunc_tandg_types, 2, 1, 1, 0, "tandg", ufunc_tandg_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":13756
 * ufunc_tandg_ptr[2*1+1] = <void*>(<char*>"tandg")
 * ufunc_tandg_data[0] = &ufunc_tandg_ptr[2*0]
 * ufunc_tandg_data[1] = &ufunc_tandg_ptr[2*1]             # <<<<<<<<<<<<<<
 * tandg = np.PyUFunc_FromFuncAndData(ufunc_tandg_loops, ufunc_tandg_data, ufunc_tandg_types, 2, 1, 1, 0, "tandg", ufunc_tandg_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":13757
 * ufunc_tandg_data[0] = &ufunc_tandg_ptr[2*0]
 * ufunc_tandg_data[1] = &ufunc_tandg_ptr[2*1]
 * tandg = np.PyUFunc_FromFuncAndData(ufunc_tandg_loops, ufunc_tandg_data, ufunc_tandg_types, 2, 1, 1, 0, "tandg", ufunc_tandg_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_tklmbda_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_types, 2, 1, 1, 0, ((char *)"tandg"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_tandg_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13757, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_tandg, __pyx_t_2) < 0) __PYX_ERR(1, 13757, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":13763
 * cdef void *ufunc_tklmbda_data[2]
 * cdef char ufunc_tklmbda_types[6]
 * cdef char *ufunc_tklmbda_doc = (             # <<<<<<<<<<<<<<
 *     "tklmbda(x, lmbda)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_doc = ((char *)"tklmbda(x, lmbda)\n\nTukey-Lambda cumulative distribution function");

  /* "scipy/special/_ufuncs.pyx":13767
 *     "\n"
 *     "Tukey-Lambda cumulative distribution function")
 * ufunc_tklmbda_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_tklmbda_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":13768
 *     "Tukey-Lambda cumulative distribution function")
 * ufunc_tklmbda_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_tklmbda_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_types[0] = <char>NPY_FLOAT
 * ufunc_tklmbda_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":13769
 * ufunc_tklmbda_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_tklmbda_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_tklmbda_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_types[1] = <char>NPY_FLOAT
 * ufunc_tklmbda_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13770
 * ufunc_tklmbda_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_tklmbda_types[0] = <char>NPY_FLOAT
 * ufunc_tklmbda_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_types[2] = <char>NPY_FLOAT
 * ufunc_tklmbda_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13771
 * ufunc_tklmbda_types[0] = <char>NPY_FLOAT
 * ufunc_tklmbda_types[1] = <char>NPY_FLOAT
 * ufunc_tklmbda_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_types[3] = <char>NPY_DOUBLE
 * ufunc_tklmbda_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13772
 * ufunc_tklmbda_types[1] = <char>NPY_FLOAT
 * ufunc_tklmbda_types[2] = <char>NPY_FLOAT
 * ufunc_tklmbda_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_types[4] = <char>NPY_DOUBLE
 * ufunc_tklmbda_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13773
 * ufunc_tklmbda_types[2] = <char>NPY_FLOAT
 * ufunc_tklmbda_types[3] = <char>NPY_DOUBLE
 * ufunc_tklmbda_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_types[5] = <char>NPY_DOUBLE
 * ufunc_tklmbda_ptr[2*0] = <void*>_func_tukeylambdacdf
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13774
 * ufunc_tklmbda_types[3] = <char>NPY_DOUBLE
 * ufunc_tklmbda_types[4] = <char>NPY_DOUBLE
 * ufunc_tklmbda_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_ptr[2*0] = <void*>_func_tukeylambdacdf
 * ufunc_tklmbda_ptr[2*0+1] = <void*>(<char*>"tklmbda")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13775
 * ufunc_tklmbda_types[4] = <char>NPY_DOUBLE
 * ufunc_tklmbda_types[5] = <char>NPY_DOUBLE
 * ufunc_tklmbda_ptr[2*0] = <void*>_func_tukeylambdacdf             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_ptr[2*0+1] = <void*>(<char*>"tklmbda")
 * ufunc_tklmbda_ptr[2*1] = <void*>_func_tukeylambdacdf
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_ptr[0]) = ((void *)tukeylambdacdf);

  /* "scipy/special/_ufuncs.pyx":13776
 * ufunc_tklmbda_types[5] = <char>NPY_DOUBLE
 * ufunc_tklmbda_ptr[2*0] = <void*>_func_tukeylambdacdf
 * ufunc_tklmbda_ptr[2*0+1] = <void*>(<char*>"tklmbda")             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_ptr[2*1] = <void*>_func_tukeylambdacdf
 * ufunc_tklmbda_ptr[2*1+1] = <void*>(<char*>"tklmbda")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_ptr[1]) = ((void *)((char *)((char *)"tklmbda")));

  /* "scipy/special/_ufuncs.pyx":13777
 * ufunc_tklmbda_ptr[2*0] = <void*>_func_tukeylambdacdf
 * ufunc_tklmbda_ptr[2*0+1] = <void*>(<char*>"tklmbda")
 * ufunc_tklmbda_ptr[2*1] = <void*>_func_tukeylambdacdf             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_ptr[2*1+1] = <void*>(<char*>"tklmbda")
 * ufunc_tklmbda_data[0] = &ufunc_tklmbda_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_ptr[2]) = ((void *)tukeylambdacdf);

  /* "scipy/special/_ufuncs.pyx":13778
 * ufunc_tklmbda_ptr[2*0+1] = <void*>(<char*>"tklmbda")
 * ufunc_tklmbda_ptr[2*1] = <void*>_func_tukeylambdacdf
 * ufunc_tklmbda_ptr[2*1+1] = <void*>(<char*>"tklmbda")             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_data[0] = &ufunc_tklmbda_ptr[2*0]
 * ufunc_tklmbda_data[1] = &ufunc_tklmbda_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_ptr[3]) = ((void *)((char *)((char *)"tklmbda")));

  /* "scipy/special/_ufuncs.pyx":13779
 * ufunc_tklmbda_ptr[2*1] = <void*>_func_tukeylambdacdf
 * ufunc_tklmbda_ptr[2*1+1] = <void*>(<char*>"tklmbda")
 * ufunc_tklmbda_data[0] = &ufunc_tklmbda_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_tklmbda_data[1] = &ufunc_tklmbda_ptr[2*1]
 * tklmbda = np.PyUFunc_FromFuncAndData(ufunc_tklmbda_loops, ufunc_tklmbda_data, ufunc_tklmbda_types, 2, 2, 1, 0, "tklmbda", ufunc_tklmbda_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":13780
 * ufunc_tklmbda_ptr[2*1+1] = <void*>(<char*>"tklmbda")
 * ufunc_tklmbda_data[0] = &ufunc_tklmbda_ptr[2*0]
 * ufunc_tklmbda_data[1] = &ufunc_tklmbda_ptr[2*1]             # <<<<<<<<<<<<<<
 * tklmbda = np.PyUFunc_FromFuncAndData(ufunc_tklmbda_loops, ufunc_tklmbda_data, ufunc_tklmbda_types, 2, 2, 1, 0, "tklmbda", ufunc_tklmbda_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":13781
 * ufunc_tklmbda_data[0] = &ufunc_tklmbda_ptr[2*0]
 * ufunc_tklmbda_data[1] = &ufunc_tklmbda_ptr[2*1]
 * tklmbda = np.PyUFunc_FromFuncAndData(ufunc_tklmbda_loops, ufunc_tklmbda_data, ufunc_tklmbda_types, 2, 2, 1, 0, "tklmbda", ufunc_tklmbda_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_wofz_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_types, 2, 2, 1, 0, ((char *)"tklmbda"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_tklmbda_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13781, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_tklmbda, __pyx_t_2) < 0) __PYX_ERR(1, 13781, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":13787
 * cdef void *ufunc_wofz_data[2]
 * cdef char ufunc_wofz_types[4]
 * cdef char *ufunc_wofz_doc = (             # <<<<<<<<<<<<<<
 *     "wofz(z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_doc = ((char *)"wofz(z)\n\nFaddeeva function\n\nReturns the value of the Faddeeva function for complex argument::\n\n    exp(-z**2) * erfc(-i*z)\n\nSee Also\n--------\ndawsn, erf, erfc, erfcx, erfi\n\nReferences\n----------\n.. [1] Steven G. Johnson, Faddeeva W function implementation.\n   http://ab-initio.mit.edu/Faddeeva\n\nExamples\n--------\n>>> from scipy import special\n>>> import matplotlib.pyplot as plt\n\n>>> x = np.linspace(-3, 3)\n>>> z = special.wofz(x)\n\n>>> plt.plot(x, z.real, label='wofz(x).real')\n>>> plt.plot(x, z.imag, label='wofz(x).imag')\n>>> plt.xlabel('$x$')\n>>> plt.legend(framealpha=1, shadow=True)\n>>> plt.grid(alpha=0.25)\n>>> plt.show()");

  /* "scipy/special/_ufuncs.pyx":13819
 *     ">>> plt.grid(alpha=0.25)\n"
 *     ">>> plt.show()")
 * ufunc_wofz_loops[0] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_wofz_loops[1] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_wofz_types[0] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":13820
 *     ">>> plt.show()")
 * ufunc_wofz_loops[0] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_wofz_loops[1] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_wofz_types[0] = <char>NPY_CFLOAT
 * ufunc_wofz_types[1] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":13821
 * ufunc_wofz_loops[0] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_wofz_loops[1] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_wofz_types[0] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_wofz_types[1] = <char>NPY_CFLOAT
 * ufunc_wofz_types[2] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_types[0]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13822
 * ufunc_wofz_loops[1] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_wofz_types[0] = <char>NPY_CFLOAT
 * ufunc_wofz_types[1] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_wofz_types[2] = <char>NPY_CDOUBLE
 * ufunc_wofz_types[3] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_types[1]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13823
 * ufunc_wofz_types[0] = <char>NPY_CFLOAT
 * ufunc_wofz_types[1] = <char>NPY_CFLOAT
 * ufunc_wofz_types[2] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_wofz_types[3] = <char>NPY_CDOUBLE
 * ufunc_wofz_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_w
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_types[2]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13824
 * ufunc_wofz_types[1] = <char>NPY_CFLOAT
 * ufunc_wofz_types[2] = <char>NPY_CDOUBLE
 * ufunc_wofz_types[3] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_wofz_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_w
 * ufunc_wofz_ptr[2*0+1] = <void*>(<char*>"wofz")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_types[3]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13825
 * ufunc_wofz_types[2] = <char>NPY_CDOUBLE
 * ufunc_wofz_types[3] = <char>NPY_CDOUBLE
 * ufunc_wofz_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_w             # <<<<<<<<<<<<<<
 * ufunc_wofz_ptr[2*0+1] = <void*>(<char*>"wofz")
 * ufunc_wofz_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_w
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_ptr[0]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_w);

  /* "scipy/special/_ufuncs.pyx":13826
 * ufunc_wofz_types[3] = <char>NPY_CDOUBLE
 * ufunc_wofz_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_w
 * ufunc_wofz_ptr[2*0+1] = <void*>(<char*>"wofz")             # <<<<<<<<<<<<<<
 * ufunc_wofz_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_w
 * ufunc_wofz_ptr[2*1+1] = <void*>(<char*>"wofz")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_ptr[1]) = ((void *)((char *)((char *)"wofz")));

  /* "scipy/special/_ufuncs.pyx":13827
 * ufunc_wofz_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_w
 * ufunc_wofz_ptr[2*0+1] = <void*>(<char*>"wofz")
 * ufunc_wofz_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_w             # <<<<<<<<<<<<<<
 * ufunc_wofz_ptr[2*1+1] = <void*>(<char*>"wofz")
 * ufunc_wofz_data[0] = &ufunc_wofz_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_ptr[2]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_faddeeva_w);

  /* "scipy/special/_ufuncs.pyx":13828
 * ufunc_wofz_ptr[2*0+1] = <void*>(<char*>"wofz")
 * ufunc_wofz_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_w
 * ufunc_wofz_ptr[2*1+1] = <void*>(<char*>"wofz")             # <<<<<<<<<<<<<<
 * ufunc_wofz_data[0] = &ufunc_wofz_ptr[2*0]
 * ufunc_wofz_data[1] = &ufunc_wofz_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_ptr[3]) = ((void *)((char *)((char *)"wofz")));

  /* "scipy/special/_ufuncs.pyx":13829
 * ufunc_wofz_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_faddeeva_w
 * ufunc_wofz_ptr[2*1+1] = <void*>(<char*>"wofz")
 * ufunc_wofz_data[0] = &ufunc_wofz_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_wofz_data[1] = &ufunc_wofz_ptr[2*1]
 * wofz = np.PyUFunc_FromFuncAndData(ufunc_wofz_loops, ufunc_wofz_data, ufunc_wofz_types, 2, 1, 1, 0, "wofz", ufunc_wofz_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":13830
 * ufunc_wofz_ptr[2*1+1] = <void*>(<char*>"wofz")
 * ufunc_wofz_data[0] = &ufunc_wofz_ptr[2*0]
 * ufunc_wofz_data[1] = &ufunc_wofz_ptr[2*1]             # <<<<<<<<<<<<<<
 * wofz = np.PyUFunc_FromFuncAndData(ufunc_wofz_loops, ufunc_wofz_data, ufunc_wofz_types, 2, 1, 1, 0, "wofz", ufunc_wofz_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":13831
 * ufunc_wofz_data[0] = &ufunc_wofz_ptr[2*0]
 * ufunc_wofz_data[1] = &ufunc_wofz_ptr[2*1]
 * wofz = np.PyUFunc_FromFuncAndData(ufunc_wofz_loops, ufunc_wofz_data, ufunc_wofz_types, 2, 1, 1, 0, "wofz", ufunc_wofz_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_wrightomega_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_types, 2, 1, 1, 0, ((char *)"wofz"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_wofz_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_wofz, __pyx_t_2) < 0) __PYX_ERR(1, 13831, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":13837
 * cdef void *ufunc_wrightomega_data[2]
 * cdef char ufunc_wrightomega_types[4]
 * cdef char *ufunc_wrightomega_doc = (             # <<<<<<<<<<<<<<
 *     "wrightomega(z, out=None)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_doc = ((char *)"wrightomega(z, out=None)\n\nWright Omega function.\n\nDefined as the solution to\n\n.. math::\n\n    \\omega + \\log(\\omega) = z\n\nwhere :math:`\\log` is the principal branch of the complex logarithm.\n\nParameters\n----------\nz : array_like\n    Points at which to evaluate the Wright Omega function\n\nReturns\n-------\nomega : ndarray\n    Values of the Wright Omega function\n\nNotes\n-----\n.. versionadded:: 0.19.0\n\nThe function can also be defined as\n\n.. math::\n\n    \\omega(z) = W_{K(z)}(e^z)\n\nwhere :math:`K(z) = \\lceil (\\Im(z) - \\pi)/(2\\pi) \\rceil` is the\nunwinding number and :math:`W` is the Lambert W function.\n\nThe implementation here is taken from [1]_.\n\nSee Also\n--------\nlambertw : The Lambert W function\n\nReferences\n----------\n.. [1] Lawrence, Corless, and Jeffrey, \"Algorithm 917: Complex\n       Double-Precision Evaluation of the Wright :math:`\\omega`\n       Function.\" ACM Transactions on Mathematical Software,\n       2012. :doi:`10.1145/2168773.2168779`.");

  /* "scipy/special/_ufuncs.pyx":13885
 *     "       Function.\" ACM Transactions on Mathematical Software,\n"
 *     "       2012. :doi:`10.1145/2168773.2168779`.")
 * ufunc_wrightomega_loops[0] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F             # <<<<<<<<<<<<<<
 * ufunc_wrightomega_loops[1] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_wrightomega_types[0] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_F_F);

  /* "scipy/special/_ufuncs.pyx":13886
 *     "       2012. :doi:`10.1145/2168773.2168779`.")
 * ufunc_wrightomega_loops[0] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_wrightomega_loops[1] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D             # <<<<<<<<<<<<<<
 * ufunc_wrightomega_types[0] = <char>NPY_CFLOAT
 * ufunc_wrightomega_types[1] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_D__As_D_D);

  /* "scipy/special/_ufuncs.pyx":13887
 * ufunc_wrightomega_loops[0] = <np.PyUFuncGenericFunction>loop_D_D__As_F_F
 * ufunc_wrightomega_loops[1] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_wrightomega_types[0] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_wrightomega_types[1] = <char>NPY_CFLOAT
 * ufunc_wrightomega_types[2] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_types[0]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13888
 * ufunc_wrightomega_loops[1] = <np.PyUFuncGenericFunction>loop_D_D__As_D_D
 * ufunc_wrightomega_types[0] = <char>NPY_CFLOAT
 * ufunc_wrightomega_types[1] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_wrightomega_types[2] = <char>NPY_CDOUBLE
 * ufunc_wrightomega_types[3] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_types[1]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13889
 * ufunc_wrightomega_types[0] = <char>NPY_CFLOAT
 * ufunc_wrightomega_types[1] = <char>NPY_CFLOAT
 * ufunc_wrightomega_types[2] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_wrightomega_types[3] = <char>NPY_CDOUBLE
 * ufunc_wrightomega_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_wrightomega
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_types[2]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13890
 * ufunc_wrightomega_types[1] = <char>NPY_CFLOAT
 * ufunc_wrightomega_types[2] = <char>NPY_CDOUBLE
 * ufunc_wrightomega_types[3] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_wrightomega_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_wrightomega
 * ufunc_wrightomega_ptr[2*0+1] = <void*>(<char*>"wrightomega")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_types[3]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13891
 * ufunc_wrightomega_types[2] = <char>NPY_CDOUBLE
 * ufunc_wrightomega_types[3] = <char>NPY_CDOUBLE
 * ufunc_wrightomega_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_wrightomega             # <<<<<<<<<<<<<<
 * ufunc_wrightomega_ptr[2*0+1] = <void*>(<char*>"wrightomega")
 * ufunc_wrightomega_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_wrightomega
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_ptr[0]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_wrightomega);

  /* "scipy/special/_ufuncs.pyx":13892
 * ufunc_wrightomega_types[3] = <char>NPY_CDOUBLE
 * ufunc_wrightomega_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_wrightomega
 * ufunc_wrightomega_ptr[2*0+1] = <void*>(<char*>"wrightomega")             # <<<<<<<<<<<<<<
 * ufunc_wrightomega_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_wrightomega
 * ufunc_wrightomega_ptr[2*1+1] = <void*>(<char*>"wrightomega")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_ptr[1]) = ((void *)((char *)((char *)"wrightomega")));

  /* "scipy/special/_ufuncs.pyx":13893
 * ufunc_wrightomega_ptr[2*0] = <void*>scipy.special._ufuncs_cxx._export_wrightomega
 * ufunc_wrightomega_ptr[2*0+1] = <void*>(<char*>"wrightomega")
 * ufunc_wrightomega_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_wrightomega             # <<<<<<<<<<<<<<
 * ufunc_wrightomega_ptr[2*1+1] = <void*>(<char*>"wrightomega")
 * ufunc_wrightomega_data[0] = &ufunc_wrightomega_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_ptr[2]) = ((void *)__pyx_v_5scipy_7special_11_ufuncs_cxx__export_wrightomega);

  /* "scipy/special/_ufuncs.pyx":13894
 * ufunc_wrightomega_ptr[2*0+1] = <void*>(<char*>"wrightomega")
 * ufunc_wrightomega_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_wrightomega
 * ufunc_wrightomega_ptr[2*1+1] = <void*>(<char*>"wrightomega")             # <<<<<<<<<<<<<<
 * ufunc_wrightomega_data[0] = &ufunc_wrightomega_ptr[2*0]
 * ufunc_wrightomega_data[1] = &ufunc_wrightomega_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_ptr[3]) = ((void *)((char *)((char *)"wrightomega")));

  /* "scipy/special/_ufuncs.pyx":13895
 * ufunc_wrightomega_ptr[2*1] = <void*>scipy.special._ufuncs_cxx._export_wrightomega
 * ufunc_wrightomega_ptr[2*1+1] = <void*>(<char*>"wrightomega")
 * ufunc_wrightomega_data[0] = &ufunc_wrightomega_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_wrightomega_data[1] = &ufunc_wrightomega_ptr[2*1]
 * wrightomega = np.PyUFunc_FromFuncAndData(ufunc_wrightomega_loops, ufunc_wrightomega_data, ufunc_wrightomega_types, 2, 1, 1, 0, "wrightomega", ufunc_wrightomega_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":13896
 * ufunc_wrightomega_ptr[2*1+1] = <void*>(<char*>"wrightomega")
 * ufunc_wrightomega_data[0] = &ufunc_wrightomega_ptr[2*0]
 * ufunc_wrightomega_data[1] = &ufunc_wrightomega_ptr[2*1]             # <<<<<<<<<<<<<<
 * wrightomega = np.PyUFunc_FromFuncAndData(ufunc_wrightomega_loops, ufunc_wrightomega_data, ufunc_wrightomega_types, 2, 1, 1, 0, "wrightomega", ufunc_wrightomega_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":13897
 * ufunc_wrightomega_data[0] = &ufunc_wrightomega_ptr[2*0]
 * ufunc_wrightomega_data[1] = &ufunc_wrightomega_ptr[2*1]
 * wrightomega = np.PyUFunc_FromFuncAndData(ufunc_wrightomega_loops, ufunc_wrightomega_data, ufunc_wrightomega_types, 2, 1, 1, 0, "wrightomega", ufunc_wrightomega_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_xlog1py_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_types, 2, 1, 1, 0, ((char *)"wrightomega"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_wrightomega_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13897, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_wrightomega, __pyx_t_2) < 0) __PYX_ERR(1, 13897, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":13903
 * cdef void *ufunc_xlog1py_data[4]
 * cdef char ufunc_xlog1py_types[12]
 * cdef char *ufunc_xlog1py_doc = (             # <<<<<<<<<<<<<<
 *     "xlog1py(x, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_doc = ((char *)"xlog1py(x, y)\n\nCompute ``x*log1p(y)`` so that the result is 0 if ``x = 0``.\n\nParameters\n----------\nx : array_like\n    Multiplier\ny : array_like\n    Argument\n\nReturns\n-------\nz : array_like\n    Computed x*log1p(y)\n\nNotes\n-----\n\n.. versionadded:: 0.13.0");

  /* "scipy/special/_ufuncs.pyx":13924
 *     "\n"
 *     ".. versionadded:: 0.13.0")
 * ufunc_xlog1py_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_xlog1py_loops[2] = <np.PyUFuncGenericFunction>loop_D_DD__As_FF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":13925
 *     ".. versionadded:: 0.13.0")
 * ufunc_xlog1py_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_xlog1py_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_loops[2] = <np.PyUFuncGenericFunction>loop_D_DD__As_FF_F
 * ufunc_xlog1py_loops[3] = <np.PyUFuncGenericFunction>loop_D_DD__As_DD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":13926
 * ufunc_xlog1py_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_xlog1py_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_xlog1py_loops[2] = <np.PyUFuncGenericFunction>loop_D_DD__As_FF_F             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_loops[3] = <np.PyUFuncGenericFunction>loop_D_DD__As_DD_D
 * ufunc_xlog1py_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_DD__As_FF_F);

  /* "scipy/special/_ufuncs.pyx":13927
 * ufunc_xlog1py_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_xlog1py_loops[2] = <np.PyUFuncGenericFunction>loop_D_DD__As_FF_F
 * ufunc_xlog1py_loops[3] = <np.PyUFuncGenericFunction>loop_D_DD__As_DD_D             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_types[0] = <char>NPY_FLOAT
 * ufunc_xlog1py_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_DD__As_DD_D);

  /* "scipy/special/_ufuncs.pyx":13928
 * ufunc_xlog1py_loops[2] = <np.PyUFuncGenericFunction>loop_D_DD__As_FF_F
 * ufunc_xlog1py_loops[3] = <np.PyUFuncGenericFunction>loop_D_DD__As_DD_D
 * ufunc_xlog1py_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_types[1] = <char>NPY_FLOAT
 * ufunc_xlog1py_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13929
 * ufunc_xlog1py_loops[3] = <np.PyUFuncGenericFunction>loop_D_DD__As_DD_D
 * ufunc_xlog1py_types[0] = <char>NPY_FLOAT
 * ufunc_xlog1py_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_types[2] = <char>NPY_FLOAT
 * ufunc_xlog1py_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13930
 * ufunc_xlog1py_types[0] = <char>NPY_FLOAT
 * ufunc_xlog1py_types[1] = <char>NPY_FLOAT
 * ufunc_xlog1py_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_types[3] = <char>NPY_DOUBLE
 * ufunc_xlog1py_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13931
 * ufunc_xlog1py_types[1] = <char>NPY_FLOAT
 * ufunc_xlog1py_types[2] = <char>NPY_FLOAT
 * ufunc_xlog1py_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_types[4] = <char>NPY_DOUBLE
 * ufunc_xlog1py_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13932
 * ufunc_xlog1py_types[2] = <char>NPY_FLOAT
 * ufunc_xlog1py_types[3] = <char>NPY_DOUBLE
 * ufunc_xlog1py_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_types[5] = <char>NPY_DOUBLE
 * ufunc_xlog1py_types[6] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13933
 * ufunc_xlog1py_types[3] = <char>NPY_DOUBLE
 * ufunc_xlog1py_types[4] = <char>NPY_DOUBLE
 * ufunc_xlog1py_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_types[6] = <char>NPY_CFLOAT
 * ufunc_xlog1py_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13934
 * ufunc_xlog1py_types[4] = <char>NPY_DOUBLE
 * ufunc_xlog1py_types[5] = <char>NPY_DOUBLE
 * ufunc_xlog1py_types[6] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_types[7] = <char>NPY_CFLOAT
 * ufunc_xlog1py_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types[6]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13935
 * ufunc_xlog1py_types[5] = <char>NPY_DOUBLE
 * ufunc_xlog1py_types[6] = <char>NPY_CFLOAT
 * ufunc_xlog1py_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_types[8] = <char>NPY_CFLOAT
 * ufunc_xlog1py_types[9] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13936
 * ufunc_xlog1py_types[6] = <char>NPY_CFLOAT
 * ufunc_xlog1py_types[7] = <char>NPY_CFLOAT
 * ufunc_xlog1py_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_types[9] = <char>NPY_CDOUBLE
 * ufunc_xlog1py_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13937
 * ufunc_xlog1py_types[7] = <char>NPY_CFLOAT
 * ufunc_xlog1py_types[8] = <char>NPY_CFLOAT
 * ufunc_xlog1py_types[9] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_types[10] = <char>NPY_CDOUBLE
 * ufunc_xlog1py_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types[9]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13938
 * ufunc_xlog1py_types[8] = <char>NPY_CFLOAT
 * ufunc_xlog1py_types[9] = <char>NPY_CDOUBLE
 * ufunc_xlog1py_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_types[11] = <char>NPY_CDOUBLE
 * ufunc_xlog1py_ptr[2*0] = <void*>_func_xlog1py[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13939
 * ufunc_xlog1py_types[9] = <char>NPY_CDOUBLE
 * ufunc_xlog1py_types[10] = <char>NPY_CDOUBLE
 * ufunc_xlog1py_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_ptr[2*0] = <void*>_func_xlog1py[double]
 * ufunc_xlog1py_ptr[2*0+1] = <void*>(<char*>"xlog1py")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13940
 * ufunc_xlog1py_types[10] = <char>NPY_CDOUBLE
 * ufunc_xlog1py_types[11] = <char>NPY_CDOUBLE
 * ufunc_xlog1py_ptr[2*0] = <void*>_func_xlog1py[double]             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_ptr[2*0+1] = <void*>(<char*>"xlog1py")
 * ufunc_xlog1py_ptr[2*1] = <void*>_func_xlog1py[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_ptr[0]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_6_xlogy_xlog1py);

  /* "scipy/special/_ufuncs.pyx":13941
 * ufunc_xlog1py_types[11] = <char>NPY_CDOUBLE
 * ufunc_xlog1py_ptr[2*0] = <void*>_func_xlog1py[double]
 * ufunc_xlog1py_ptr[2*0+1] = <void*>(<char*>"xlog1py")             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_ptr[2*1] = <void*>_func_xlog1py[double]
 * ufunc_xlog1py_ptr[2*1+1] = <void*>(<char*>"xlog1py")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_ptr[1]) = ((void *)((char *)((char *)"xlog1py")));

  /* "scipy/special/_ufuncs.pyx":13942
 * ufunc_xlog1py_ptr[2*0] = <void*>_func_xlog1py[double]
 * ufunc_xlog1py_ptr[2*0+1] = <void*>(<char*>"xlog1py")
 * ufunc_xlog1py_ptr[2*1] = <void*>_func_xlog1py[double]             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_ptr[2*1+1] = <void*>(<char*>"xlog1py")
 * ufunc_xlog1py_ptr[2*2] = <void*>_func_xlog1py[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_6_xlogy_xlog1py);

  /* "scipy/special/_ufuncs.pyx":13943
 * ufunc_xlog1py_ptr[2*0+1] = <void*>(<char*>"xlog1py")
 * ufunc_xlog1py_ptr[2*1] = <void*>_func_xlog1py[double]
 * ufunc_xlog1py_ptr[2*1+1] = <void*>(<char*>"xlog1py")             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_ptr[2*2] = <void*>_func_xlog1py[double_complex]
 * ufunc_xlog1py_ptr[2*2+1] = <void*>(<char*>"xlog1py")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_ptr[3]) = ((void *)((char *)((char *)"xlog1py")));

  /* "scipy/special/_ufuncs.pyx":13944
 * ufunc_xlog1py_ptr[2*1] = <void*>_func_xlog1py[double]
 * ufunc_xlog1py_ptr[2*1+1] = <void*>(<char*>"xlog1py")
 * ufunc_xlog1py_ptr[2*2] = <void*>_func_xlog1py[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_ptr[2*2+1] = <void*>(<char*>"xlog1py")
 * ufunc_xlog1py_ptr[2*3] = <void*>_func_xlog1py[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_6_xlogy_xlog1py);

  /* "scipy/special/_ufuncs.pyx":13945
 * ufunc_xlog1py_ptr[2*1+1] = <void*>(<char*>"xlog1py")
 * ufunc_xlog1py_ptr[2*2] = <void*>_func_xlog1py[double_complex]
 * ufunc_xlog1py_ptr[2*2+1] = <void*>(<char*>"xlog1py")             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_ptr[2*3] = <void*>_func_xlog1py[double_complex]
 * ufunc_xlog1py_ptr[2*3+1] = <void*>(<char*>"xlog1py")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_ptr[5]) = ((void *)((char *)((char *)"xlog1py")));

  /* "scipy/special/_ufuncs.pyx":13946
 * ufunc_xlog1py_ptr[2*2] = <void*>_func_xlog1py[double_complex]
 * ufunc_xlog1py_ptr[2*2+1] = <void*>(<char*>"xlog1py")
 * ufunc_xlog1py_ptr[2*3] = <void*>_func_xlog1py[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_ptr[2*3+1] = <void*>(<char*>"xlog1py")
 * ufunc_xlog1py_data[0] = &ufunc_xlog1py_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_ptr[6]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_6_xlogy_xlog1py);

  /* "scipy/special/_ufuncs.pyx":13947
 * ufunc_xlog1py_ptr[2*2+1] = <void*>(<char*>"xlog1py")
 * ufunc_xlog1py_ptr[2*3] = <void*>_func_xlog1py[double_complex]
 * ufunc_xlog1py_ptr[2*3+1] = <void*>(<char*>"xlog1py")             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_data[0] = &ufunc_xlog1py_ptr[2*0]
 * ufunc_xlog1py_data[1] = &ufunc_xlog1py_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_ptr[7]) = ((void *)((char *)((char *)"xlog1py")));

  /* "scipy/special/_ufuncs.pyx":13948
 * ufunc_xlog1py_ptr[2*3] = <void*>_func_xlog1py[double_complex]
 * ufunc_xlog1py_ptr[2*3+1] = <void*>(<char*>"xlog1py")
 * ufunc_xlog1py_data[0] = &ufunc_xlog1py_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_data[1] = &ufunc_xlog1py_ptr[2*1]
 * ufunc_xlog1py_data[2] = &ufunc_xlog1py_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":13949
 * ufunc_xlog1py_ptr[2*3+1] = <void*>(<char*>"xlog1py")
 * ufunc_xlog1py_data[0] = &ufunc_xlog1py_ptr[2*0]
 * ufunc_xlog1py_data[1] = &ufunc_xlog1py_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_data[2] = &ufunc_xlog1py_ptr[2*2]
 * ufunc_xlog1py_data[3] = &ufunc_xlog1py_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":13950
 * ufunc_xlog1py_data[0] = &ufunc_xlog1py_ptr[2*0]
 * ufunc_xlog1py_data[1] = &ufunc_xlog1py_ptr[2*1]
 * ufunc_xlog1py_data[2] = &ufunc_xlog1py_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_xlog1py_data[3] = &ufunc_xlog1py_ptr[2*3]
 * xlog1py = np.PyUFunc_FromFuncAndData(ufunc_xlog1py_loops, ufunc_xlog1py_data, ufunc_xlog1py_types, 4, 2, 1, 0, "xlog1py", ufunc_xlog1py_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":13951
 * ufunc_xlog1py_data[1] = &ufunc_xlog1py_ptr[2*1]
 * ufunc_xlog1py_data[2] = &ufunc_xlog1py_ptr[2*2]
 * ufunc_xlog1py_data[3] = &ufunc_xlog1py_ptr[2*3]             # <<<<<<<<<<<<<<
 * xlog1py = np.PyUFunc_FromFuncAndData(ufunc_xlog1py_loops, ufunc_xlog1py_data, ufunc_xlog1py_types, 4, 2, 1, 0, "xlog1py", ufunc_xlog1py_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":13952
 * ufunc_xlog1py_data[2] = &ufunc_xlog1py_ptr[2*2]
 * ufunc_xlog1py_data[3] = &ufunc_xlog1py_ptr[2*3]
 * xlog1py = np.PyUFunc_FromFuncAndData(ufunc_xlog1py_loops, ufunc_xlog1py_data, ufunc_xlog1py_types, 4, 2, 1, 0, "xlog1py", ufunc_xlog1py_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_xlogy_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_types, 4, 2, 1, 0, ((char *)"xlog1py"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlog1py_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13952, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_xlog1py, __pyx_t_2) < 0) __PYX_ERR(1, 13952, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":13958
 * cdef void *ufunc_xlogy_data[4]
 * cdef char ufunc_xlogy_types[12]
 * cdef char *ufunc_xlogy_doc = (             # <<<<<<<<<<<<<<
 *     "xlogy(x, y)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_doc = ((char *)"xlogy(x, y)\n\nCompute ``x*log(y)`` so that the result is 0 if ``x = 0``.\n\nParameters\n----------\nx : array_like\n    Multiplier\ny : array_like\n    Argument\n\nReturns\n-------\nz : array_like\n    Computed x*log(y)\n\nNotes\n-----\n\n.. versionadded:: 0.13.0");

  /* "scipy/special/_ufuncs.pyx":13979
 *     "\n"
 *     ".. versionadded:: 0.13.0")
 * ufunc_xlogy_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_xlogy_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_xlogy_loops[2] = <np.PyUFuncGenericFunction>loop_D_DD__As_FF_F
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":13980
 *     ".. versionadded:: 0.13.0")
 * ufunc_xlogy_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_xlogy_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_xlogy_loops[2] = <np.PyUFuncGenericFunction>loop_D_DD__As_FF_F
 * ufunc_xlogy_loops[3] = <np.PyUFuncGenericFunction>loop_D_DD__As_DD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":13981
 * ufunc_xlogy_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_xlogy_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_xlogy_loops[2] = <np.PyUFuncGenericFunction>loop_D_DD__As_FF_F             # <<<<<<<<<<<<<<
 * ufunc_xlogy_loops[3] = <np.PyUFuncGenericFunction>loop_D_DD__As_DD_D
 * ufunc_xlogy_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_DD__As_FF_F);

  /* "scipy/special/_ufuncs.pyx":13982
 * ufunc_xlogy_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_xlogy_loops[2] = <np.PyUFuncGenericFunction>loop_D_DD__As_FF_F
 * ufunc_xlogy_loops[3] = <np.PyUFuncGenericFunction>loop_D_DD__As_DD_D             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[0] = <char>NPY_FLOAT
 * ufunc_xlogy_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_DD__As_DD_D);

  /* "scipy/special/_ufuncs.pyx":13983
 * ufunc_xlogy_loops[2] = <np.PyUFuncGenericFunction>loop_D_DD__As_FF_F
 * ufunc_xlogy_loops[3] = <np.PyUFuncGenericFunction>loop_D_DD__As_DD_D
 * ufunc_xlogy_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[1] = <char>NPY_FLOAT
 * ufunc_xlogy_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13984
 * ufunc_xlogy_loops[3] = <np.PyUFuncGenericFunction>loop_D_DD__As_DD_D
 * ufunc_xlogy_types[0] = <char>NPY_FLOAT
 * ufunc_xlogy_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[2] = <char>NPY_FLOAT
 * ufunc_xlogy_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13985
 * ufunc_xlogy_types[0] = <char>NPY_FLOAT
 * ufunc_xlogy_types[1] = <char>NPY_FLOAT
 * ufunc_xlogy_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[3] = <char>NPY_DOUBLE
 * ufunc_xlogy_types[4] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":13986
 * ufunc_xlogy_types[1] = <char>NPY_FLOAT
 * ufunc_xlogy_types[2] = <char>NPY_FLOAT
 * ufunc_xlogy_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[4] = <char>NPY_DOUBLE
 * ufunc_xlogy_types[5] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13987
 * ufunc_xlogy_types[2] = <char>NPY_FLOAT
 * ufunc_xlogy_types[3] = <char>NPY_DOUBLE
 * ufunc_xlogy_types[4] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[5] = <char>NPY_DOUBLE
 * ufunc_xlogy_types[6] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[4]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13988
 * ufunc_xlogy_types[3] = <char>NPY_DOUBLE
 * ufunc_xlogy_types[4] = <char>NPY_DOUBLE
 * ufunc_xlogy_types[5] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[6] = <char>NPY_CFLOAT
 * ufunc_xlogy_types[7] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[5]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":13989
 * ufunc_xlogy_types[4] = <char>NPY_DOUBLE
 * ufunc_xlogy_types[5] = <char>NPY_DOUBLE
 * ufunc_xlogy_types[6] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[7] = <char>NPY_CFLOAT
 * ufunc_xlogy_types[8] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[6]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13990
 * ufunc_xlogy_types[5] = <char>NPY_DOUBLE
 * ufunc_xlogy_types[6] = <char>NPY_CFLOAT
 * ufunc_xlogy_types[7] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[8] = <char>NPY_CFLOAT
 * ufunc_xlogy_types[9] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[7]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13991
 * ufunc_xlogy_types[6] = <char>NPY_CFLOAT
 * ufunc_xlogy_types[7] = <char>NPY_CFLOAT
 * ufunc_xlogy_types[8] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[9] = <char>NPY_CDOUBLE
 * ufunc_xlogy_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[8]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":13992
 * ufunc_xlogy_types[7] = <char>NPY_CFLOAT
 * ufunc_xlogy_types[8] = <char>NPY_CFLOAT
 * ufunc_xlogy_types[9] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[10] = <char>NPY_CDOUBLE
 * ufunc_xlogy_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[9]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13993
 * ufunc_xlogy_types[8] = <char>NPY_CFLOAT
 * ufunc_xlogy_types[9] = <char>NPY_CDOUBLE
 * ufunc_xlogy_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlogy_types[11] = <char>NPY_CDOUBLE
 * ufunc_xlogy_ptr[2*0] = <void*>_func_xlogy[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13994
 * ufunc_xlogy_types[9] = <char>NPY_CDOUBLE
 * ufunc_xlogy_types[10] = <char>NPY_CDOUBLE
 * ufunc_xlogy_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_xlogy_ptr[2*0] = <void*>_func_xlogy[double]
 * ufunc_xlogy_ptr[2*0+1] = <void*>(<char*>"xlogy")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":13995
 * ufunc_xlogy_types[10] = <char>NPY_CDOUBLE
 * ufunc_xlogy_types[11] = <char>NPY_CDOUBLE
 * ufunc_xlogy_ptr[2*0] = <void*>_func_xlogy[double]             # <<<<<<<<<<<<<<
 * ufunc_xlogy_ptr[2*0+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_ptr[2*1] = <void*>_func_xlogy[double]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[0]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_6_xlogy_xlogy);

  /* "scipy/special/_ufuncs.pyx":13996
 * ufunc_xlogy_types[11] = <char>NPY_CDOUBLE
 * ufunc_xlogy_ptr[2*0] = <void*>_func_xlogy[double]
 * ufunc_xlogy_ptr[2*0+1] = <void*>(<char*>"xlogy")             # <<<<<<<<<<<<<<
 * ufunc_xlogy_ptr[2*1] = <void*>_func_xlogy[double]
 * ufunc_xlogy_ptr[2*1+1] = <void*>(<char*>"xlogy")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[1]) = ((void *)((char *)((char *)"xlogy")));

  /* "scipy/special/_ufuncs.pyx":13997
 * ufunc_xlogy_ptr[2*0] = <void*>_func_xlogy[double]
 * ufunc_xlogy_ptr[2*0+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_ptr[2*1] = <void*>_func_xlogy[double]             # <<<<<<<<<<<<<<
 * ufunc_xlogy_ptr[2*1+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_ptr[2*2] = <void*>_func_xlogy[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[2]) = ((void *)__pyx_fuse_0__pyx_f_5scipy_7special_6_xlogy_xlogy);

  /* "scipy/special/_ufuncs.pyx":13998
 * ufunc_xlogy_ptr[2*0+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_ptr[2*1] = <void*>_func_xlogy[double]
 * ufunc_xlogy_ptr[2*1+1] = <void*>(<char*>"xlogy")             # <<<<<<<<<<<<<<
 * ufunc_xlogy_ptr[2*2] = <void*>_func_xlogy[double_complex]
 * ufunc_xlogy_ptr[2*2+1] = <void*>(<char*>"xlogy")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[3]) = ((void *)((char *)((char *)"xlogy")));

  /* "scipy/special/_ufuncs.pyx":13999
 * ufunc_xlogy_ptr[2*1] = <void*>_func_xlogy[double]
 * ufunc_xlogy_ptr[2*1+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_ptr[2*2] = <void*>_func_xlogy[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_xlogy_ptr[2*2+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_ptr[2*3] = <void*>_func_xlogy[double_complex]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[4]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_6_xlogy_xlogy);

  /* "scipy/special/_ufuncs.pyx":14000
 * ufunc_xlogy_ptr[2*1+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_ptr[2*2] = <void*>_func_xlogy[double_complex]
 * ufunc_xlogy_ptr[2*2+1] = <void*>(<char*>"xlogy")             # <<<<<<<<<<<<<<
 * ufunc_xlogy_ptr[2*3] = <void*>_func_xlogy[double_complex]
 * ufunc_xlogy_ptr[2*3+1] = <void*>(<char*>"xlogy")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[5]) = ((void *)((char *)((char *)"xlogy")));

  /* "scipy/special/_ufuncs.pyx":14001
 * ufunc_xlogy_ptr[2*2] = <void*>_func_xlogy[double_complex]
 * ufunc_xlogy_ptr[2*2+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_ptr[2*3] = <void*>_func_xlogy[double_complex]             # <<<<<<<<<<<<<<
 * ufunc_xlogy_ptr[2*3+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_data[0] = &ufunc_xlogy_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[6]) = ((void *)__pyx_fuse_1__pyx_f_5scipy_7special_6_xlogy_xlogy);

  /* "scipy/special/_ufuncs.pyx":14002
 * ufunc_xlogy_ptr[2*2+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_ptr[2*3] = <void*>_func_xlogy[double_complex]
 * ufunc_xlogy_ptr[2*3+1] = <void*>(<char*>"xlogy")             # <<<<<<<<<<<<<<
 * ufunc_xlogy_data[0] = &ufunc_xlogy_ptr[2*0]
 * ufunc_xlogy_data[1] = &ufunc_xlogy_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[7]) = ((void *)((char *)((char *)"xlogy")));

  /* "scipy/special/_ufuncs.pyx":14003
 * ufunc_xlogy_ptr[2*3] = <void*>_func_xlogy[double_complex]
 * ufunc_xlogy_ptr[2*3+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_data[0] = &ufunc_xlogy_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_xlogy_data[1] = &ufunc_xlogy_ptr[2*1]
 * ufunc_xlogy_data[2] = &ufunc_xlogy_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":14004
 * ufunc_xlogy_ptr[2*3+1] = <void*>(<char*>"xlogy")
 * ufunc_xlogy_data[0] = &ufunc_xlogy_ptr[2*0]
 * ufunc_xlogy_data[1] = &ufunc_xlogy_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_xlogy_data[2] = &ufunc_xlogy_ptr[2*2]
 * ufunc_xlogy_data[3] = &ufunc_xlogy_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":14005
 * ufunc_xlogy_data[0] = &ufunc_xlogy_ptr[2*0]
 * ufunc_xlogy_data[1] = &ufunc_xlogy_ptr[2*1]
 * ufunc_xlogy_data[2] = &ufunc_xlogy_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_xlogy_data[3] = &ufunc_xlogy_ptr[2*3]
 * xlogy = np.PyUFunc_FromFuncAndData(ufunc_xlogy_loops, ufunc_xlogy_data, ufunc_xlogy_types, 4, 2, 1, 0, "xlogy", ufunc_xlogy_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":14006
 * ufunc_xlogy_data[1] = &ufunc_xlogy_ptr[2*1]
 * ufunc_xlogy_data[2] = &ufunc_xlogy_ptr[2*2]
 * ufunc_xlogy_data[3] = &ufunc_xlogy_ptr[2*3]             # <<<<<<<<<<<<<<
 * xlogy = np.PyUFunc_FromFuncAndData(ufunc_xlogy_loops, ufunc_xlogy_data, ufunc_xlogy_types, 4, 2, 1, 0, "xlogy", ufunc_xlogy_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":14007
 * ufunc_xlogy_data[2] = &ufunc_xlogy_ptr[2*2]
 * ufunc_xlogy_data[3] = &ufunc_xlogy_ptr[2*3]
 * xlogy = np.PyUFunc_FromFuncAndData(ufunc_xlogy_loops, ufunc_xlogy_data, ufunc_xlogy_types, 4, 2, 1, 0, "xlogy", ufunc_xlogy_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_y0_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_types, 4, 2, 1, 0, ((char *)"xlogy"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_xlogy_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 14007, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_xlogy, __pyx_t_2) < 0) __PYX_ERR(1, 14007, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":14013
 * cdef void *ufunc_y0_data[2]
 * cdef char ufunc_y0_types[4]
 * cdef char *ufunc_y0_doc = (             # <<<<<<<<<<<<<<
 *     "y0(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_doc = ((char *)"y0(x)\n\nBessel function of the second kind of order 0.\n\nParameters\n----------\nx : array_like\n    Argument (float).\n\nReturns\n-------\nY : ndarray\n    Value of the Bessel function of the second kind of order 0 at `x`.\n\nNotes\n-----\n\nThe domain is divided into the intervals [0, 5] and (5, infinity). In the\nfirst interval a rational approximation :math:`R(x)` is employed to\ncompute,\n\n.. math::\n\n    Y_0(x) = R(x) + \\frac{2 \\log(x) J_0(x)}{\\pi},\n\nwhere :math:`J_0` is the Bessel function of the first kind of order 0.\n\nIn the second interval, the Hankel asymptotic expansion is employed with\ntwo rational functions of degree 6/6 and 7/7.\n\nThis function is a wrapper for the Cephes [1]_ routine `y0`.\n\nSee also\n--------\nj0\nyv\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":14055
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_y0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_y0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_y0_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":14056
 *     "       http://www.netlib.org/cephes/")
 * ufunc_y0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_y0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_y0_types[0] = <char>NPY_FLOAT
 * ufunc_y0_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":14057
 * ufunc_y0_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_y0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_y0_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_y0_types[1] = <char>NPY_FLOAT
 * ufunc_y0_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":14058
 * ufunc_y0_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_y0_types[0] = <char>NPY_FLOAT
 * ufunc_y0_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_y0_types[2] = <char>NPY_DOUBLE
 * ufunc_y0_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":14059
 * ufunc_y0_types[0] = <char>NPY_FLOAT
 * ufunc_y0_types[1] = <char>NPY_FLOAT
 * ufunc_y0_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_y0_types[3] = <char>NPY_DOUBLE
 * ufunc_y0_ptr[2*0] = <void*>_func_y0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":14060
 * ufunc_y0_types[1] = <char>NPY_FLOAT
 * ufunc_y0_types[2] = <char>NPY_DOUBLE
 * ufunc_y0_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_y0_ptr[2*0] = <void*>_func_y0
 * ufunc_y0_ptr[2*0+1] = <void*>(<char*>"y0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":14061
 * ufunc_y0_types[2] = <char>NPY_DOUBLE
 * ufunc_y0_types[3] = <char>NPY_DOUBLE
 * ufunc_y0_ptr[2*0] = <void*>_func_y0             # <<<<<<<<<<<<<<
 * ufunc_y0_ptr[2*0+1] = <void*>(<char*>"y0")
 * ufunc_y0_ptr[2*1] = <void*>_func_y0
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_ptr[0]) = ((void *)y0);

  /* "scipy/special/_ufuncs.pyx":14062
 * ufunc_y0_types[3] = <char>NPY_DOUBLE
 * ufunc_y0_ptr[2*0] = <void*>_func_y0
 * ufunc_y0_ptr[2*0+1] = <void*>(<char*>"y0")             # <<<<<<<<<<<<<<
 * ufunc_y0_ptr[2*1] = <void*>_func_y0
 * ufunc_y0_ptr[2*1+1] = <void*>(<char*>"y0")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_ptr[1]) = ((void *)((char *)((char *)"y0")));

  /* "scipy/special/_ufuncs.pyx":14063
 * ufunc_y0_ptr[2*0] = <void*>_func_y0
 * ufunc_y0_ptr[2*0+1] = <void*>(<char*>"y0")
 * ufunc_y0_ptr[2*1] = <void*>_func_y0             # <<<<<<<<<<<<<<
 * ufunc_y0_ptr[2*1+1] = <void*>(<char*>"y0")
 * ufunc_y0_data[0] = &ufunc_y0_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_ptr[2]) = ((void *)y0);

  /* "scipy/special/_ufuncs.pyx":14064
 * ufunc_y0_ptr[2*0+1] = <void*>(<char*>"y0")
 * ufunc_y0_ptr[2*1] = <void*>_func_y0
 * ufunc_y0_ptr[2*1+1] = <void*>(<char*>"y0")             # <<<<<<<<<<<<<<
 * ufunc_y0_data[0] = &ufunc_y0_ptr[2*0]
 * ufunc_y0_data[1] = &ufunc_y0_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_ptr[3]) = ((void *)((char *)((char *)"y0")));

  /* "scipy/special/_ufuncs.pyx":14065
 * ufunc_y0_ptr[2*1] = <void*>_func_y0
 * ufunc_y0_ptr[2*1+1] = <void*>(<char*>"y0")
 * ufunc_y0_data[0] = &ufunc_y0_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_y0_data[1] = &ufunc_y0_ptr[2*1]
 * y0 = np.PyUFunc_FromFuncAndData(ufunc_y0_loops, ufunc_y0_data, ufunc_y0_types, 2, 1, 1, 0, "y0", ufunc_y0_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":14066
 * ufunc_y0_ptr[2*1+1] = <void*>(<char*>"y0")
 * ufunc_y0_data[0] = &ufunc_y0_ptr[2*0]
 * ufunc_y0_data[1] = &ufunc_y0_ptr[2*1]             # <<<<<<<<<<<<<<
 * y0 = np.PyUFunc_FromFuncAndData(ufunc_y0_loops, ufunc_y0_data, ufunc_y0_types, 2, 1, 1, 0, "y0", ufunc_y0_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":14067
 * ufunc_y0_data[0] = &ufunc_y0_ptr[2*0]
 * ufunc_y0_data[1] = &ufunc_y0_ptr[2*1]
 * y0 = np.PyUFunc_FromFuncAndData(ufunc_y0_loops, ufunc_y0_data, ufunc_y0_types, 2, 1, 1, 0, "y0", ufunc_y0_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_y1_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_types, 2, 1, 1, 0, ((char *)"y0"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_y0_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 14067, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_y0, __pyx_t_2) < 0) __PYX_ERR(1, 14067, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":14073
 * cdef void *ufunc_y1_data[2]
 * cdef char ufunc_y1_types[4]
 * cdef char *ufunc_y1_doc = (             # <<<<<<<<<<<<<<
 *     "y1(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_doc = ((char *)"y1(x)\n\nBessel function of the second kind of order 1.\n\nParameters\n----------\nx : array_like\n    Argument (float).\n\nReturns\n-------\nY : ndarray\n    Value of the Bessel function of the second kind of order 1 at `x`.\n\nNotes\n-----\n\nThe domain is divided into the intervals [0, 8] and (8, infinity). In the\nfirst interval a 25 term Chebyshev expansion is used, and computing\n:math:`J_1` (the Bessel function of the first kind) is required. In the\nsecond, the asymptotic trigonometric representation is employed using two\nrational functions of degree 5/5.\n\nThis function is a wrapper for the Cephes [1]_ routine `y1`.\n\nSee also\n--------\nj1\nyn\nyv\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":14109
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_y1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_y1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_y1_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":14110
 *     "       http://www.netlib.org/cephes/")
 * ufunc_y1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_y1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_y1_types[0] = <char>NPY_FLOAT
 * ufunc_y1_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":14111
 * ufunc_y1_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_y1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_y1_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_y1_types[1] = <char>NPY_FLOAT
 * ufunc_y1_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":14112
 * ufunc_y1_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_y1_types[0] = <char>NPY_FLOAT
 * ufunc_y1_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_y1_types[2] = <char>NPY_DOUBLE
 * ufunc_y1_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":14113
 * ufunc_y1_types[0] = <char>NPY_FLOAT
 * ufunc_y1_types[1] = <char>NPY_FLOAT
 * ufunc_y1_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_y1_types[3] = <char>NPY_DOUBLE
 * ufunc_y1_ptr[2*0] = <void*>_func_y1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":14114
 * ufunc_y1_types[1] = <char>NPY_FLOAT
 * ufunc_y1_types[2] = <char>NPY_DOUBLE
 * ufunc_y1_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_y1_ptr[2*0] = <void*>_func_y1
 * ufunc_y1_ptr[2*0+1] = <void*>(<char*>"y1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":14115
 * ufunc_y1_types[2] = <char>NPY_DOUBLE
 * ufunc_y1_types[3] = <char>NPY_DOUBLE
 * ufunc_y1_ptr[2*0] = <void*>_func_y1             # <<<<<<<<<<<<<<
 * ufunc_y1_ptr[2*0+1] = <void*>(<char*>"y1")
 * ufunc_y1_ptr[2*1] = <void*>_func_y1
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_ptr[0]) = ((void *)y1);

  /* "scipy/special/_ufuncs.pyx":14116
 * ufunc_y1_types[3] = <char>NPY_DOUBLE
 * ufunc_y1_ptr[2*0] = <void*>_func_y1
 * ufunc_y1_ptr[2*0+1] = <void*>(<char*>"y1")             # <<<<<<<<<<<<<<
 * ufunc_y1_ptr[2*1] = <void*>_func_y1
 * ufunc_y1_ptr[2*1+1] = <void*>(<char*>"y1")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_ptr[1]) = ((void *)((char *)((char *)"y1")));

  /* "scipy/special/_ufuncs.pyx":14117
 * ufunc_y1_ptr[2*0] = <void*>_func_y1
 * ufunc_y1_ptr[2*0+1] = <void*>(<char*>"y1")
 * ufunc_y1_ptr[2*1] = <void*>_func_y1             # <<<<<<<<<<<<<<
 * ufunc_y1_ptr[2*1+1] = <void*>(<char*>"y1")
 * ufunc_y1_data[0] = &ufunc_y1_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_ptr[2]) = ((void *)y1);

  /* "scipy/special/_ufuncs.pyx":14118
 * ufunc_y1_ptr[2*0+1] = <void*>(<char*>"y1")
 * ufunc_y1_ptr[2*1] = <void*>_func_y1
 * ufunc_y1_ptr[2*1+1] = <void*>(<char*>"y1")             # <<<<<<<<<<<<<<
 * ufunc_y1_data[0] = &ufunc_y1_ptr[2*0]
 * ufunc_y1_data[1] = &ufunc_y1_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_ptr[3]) = ((void *)((char *)((char *)"y1")));

  /* "scipy/special/_ufuncs.pyx":14119
 * ufunc_y1_ptr[2*1] = <void*>_func_y1
 * ufunc_y1_ptr[2*1+1] = <void*>(<char*>"y1")
 * ufunc_y1_data[0] = &ufunc_y1_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_y1_data[1] = &ufunc_y1_ptr[2*1]
 * y1 = np.PyUFunc_FromFuncAndData(ufunc_y1_loops, ufunc_y1_data, ufunc_y1_types, 2, 1, 1, 0, "y1", ufunc_y1_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":14120
 * ufunc_y1_ptr[2*1+1] = <void*>(<char*>"y1")
 * ufunc_y1_data[0] = &ufunc_y1_ptr[2*0]
 * ufunc_y1_data[1] = &ufunc_y1_ptr[2*1]             # <<<<<<<<<<<<<<
 * y1 = np.PyUFunc_FromFuncAndData(ufunc_y1_loops, ufunc_y1_data, ufunc_y1_types, 2, 1, 1, 0, "y1", ufunc_y1_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":14121
 * ufunc_y1_data[0] = &ufunc_y1_ptr[2*0]
 * ufunc_y1_data[1] = &ufunc_y1_ptr[2*1]
 * y1 = np.PyUFunc_FromFuncAndData(ufunc_y1_loops, ufunc_y1_data, ufunc_y1_types, 2, 1, 1, 0, "y1", ufunc_y1_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_yn_loops[3]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_types, 2, 1, 1, 0, ((char *)"y1"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_y1_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 14121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_y1, __pyx_t_2) < 0) __PYX_ERR(1, 14121, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":14127
 * cdef void *ufunc_yn_data[3]
 * cdef char ufunc_yn_types[9]
 * cdef char *ufunc_yn_doc = (             # <<<<<<<<<<<<<<
 *     "yn(n, x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_doc = ((char *)"yn(n, x)\n\nBessel function of the second kind of integer order and real argument.\n\nParameters\n----------\nn : array_like\n    Order (integer).\nz : array_like\n    Argument (float).\n\nReturns\n-------\nY : ndarray\n    Value of the Bessel function, :math:`Y_n(x)`.\n\nNotes\n-----\nWrapper for the Cephes [1]_ routine `yn`.\n\nThe function is evaluated by forward recurrence on `n`, starting with\nvalues computed by the Cephes routines `y0` and `y1`. If `n = 0` or 1,\nthe routine for `y0` or `y1` is called directly.\n\nSee also\n--------\nyv : For real order and real or complex argument.\n\nReferences\n----------\n.. [1] Cephes Mathematical Functions Library,\n       http://www.netlib.org/cephes/");

  /* "scipy/special/_ufuncs.pyx":14160
 *     ".. [1] Cephes Mathematical Functions Library,\n"
 *     "       http://www.netlib.org/cephes/")
 * ufunc_yn_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d             # <<<<<<<<<<<<<<
 * ufunc_yn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_yn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_id__As_ld_d);

  /* "scipy/special/_ufuncs.pyx":14161
 *     "       http://www.netlib.org/cephes/")
 * ufunc_yn_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_yn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_yn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_yn_types[0] = <char>NPY_LONG
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":14162
 * ufunc_yn_loops[0] = <np.PyUFuncGenericFunction>loop_d_id__As_ld_d
 * ufunc_yn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_yn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_yn_types[0] = <char>NPY_LONG
 * ufunc_yn_types[1] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":14163
 * ufunc_yn_loops[1] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_yn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_yn_types[0] = <char>NPY_LONG             # <<<<<<<<<<<<<<
 * ufunc_yn_types[1] = <char>NPY_DOUBLE
 * ufunc_yn_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types[0]) = ((char)NPY_LONG);

  /* "scipy/special/_ufuncs.pyx":14164
 * ufunc_yn_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_yn_types[0] = <char>NPY_LONG
 * ufunc_yn_types[1] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yn_types[2] = <char>NPY_DOUBLE
 * ufunc_yn_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types[1]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":14165
 * ufunc_yn_types[0] = <char>NPY_LONG
 * ufunc_yn_types[1] = <char>NPY_DOUBLE
 * ufunc_yn_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yn_types[3] = <char>NPY_FLOAT
 * ufunc_yn_types[4] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":14166
 * ufunc_yn_types[1] = <char>NPY_DOUBLE
 * ufunc_yn_types[2] = <char>NPY_DOUBLE
 * ufunc_yn_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yn_types[4] = <char>NPY_FLOAT
 * ufunc_yn_types[5] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":14167
 * ufunc_yn_types[2] = <char>NPY_DOUBLE
 * ufunc_yn_types[3] = <char>NPY_FLOAT
 * ufunc_yn_types[4] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yn_types[5] = <char>NPY_FLOAT
 * ufunc_yn_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types[4]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":14168
 * ufunc_yn_types[3] = <char>NPY_FLOAT
 * ufunc_yn_types[4] = <char>NPY_FLOAT
 * ufunc_yn_types[5] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yn_types[6] = <char>NPY_DOUBLE
 * ufunc_yn_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types[5]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":14169
 * ufunc_yn_types[4] = <char>NPY_FLOAT
 * ufunc_yn_types[5] = <char>NPY_FLOAT
 * ufunc_yn_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yn_types[7] = <char>NPY_DOUBLE
 * ufunc_yn_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":14170
 * ufunc_yn_types[5] = <char>NPY_FLOAT
 * ufunc_yn_types[6] = <char>NPY_DOUBLE
 * ufunc_yn_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yn_types[8] = <char>NPY_DOUBLE
 * ufunc_yn_ptr[2*0] = <void*>_func_yn
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":14171
 * ufunc_yn_types[6] = <char>NPY_DOUBLE
 * ufunc_yn_types[7] = <char>NPY_DOUBLE
 * ufunc_yn_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yn_ptr[2*0] = <void*>_func_yn
 * ufunc_yn_ptr[2*0+1] = <void*>(<char*>"yn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":14172
 * ufunc_yn_types[7] = <char>NPY_DOUBLE
 * ufunc_yn_types[8] = <char>NPY_DOUBLE
 * ufunc_yn_ptr[2*0] = <void*>_func_yn             # <<<<<<<<<<<<<<
 * ufunc_yn_ptr[2*0+1] = <void*>(<char*>"yn")
 * ufunc_yn_ptr[2*1] = <void*>_func_yn_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_ptr[0]) = ((void *)yn);

  /* "scipy/special/_ufuncs.pyx":14173
 * ufunc_yn_types[8] = <char>NPY_DOUBLE
 * ufunc_yn_ptr[2*0] = <void*>_func_yn
 * ufunc_yn_ptr[2*0+1] = <void*>(<char*>"yn")             # <<<<<<<<<<<<<<
 * ufunc_yn_ptr[2*1] = <void*>_func_yn_unsafe
 * ufunc_yn_ptr[2*1+1] = <void*>(<char*>"yn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_ptr[1]) = ((void *)((char *)((char *)"yn")));

  /* "scipy/special/_ufuncs.pyx":14174
 * ufunc_yn_ptr[2*0] = <void*>_func_yn
 * ufunc_yn_ptr[2*0+1] = <void*>(<char*>"yn")
 * ufunc_yn_ptr[2*1] = <void*>_func_yn_unsafe             # <<<<<<<<<<<<<<
 * ufunc_yn_ptr[2*1+1] = <void*>(<char*>"yn")
 * ufunc_yn_ptr[2*2] = <void*>_func_yn_unsafe
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_ptr[2]) = ((void *)__pyx_f_5scipy_7special_7_legacy_yn_unsafe);

  /* "scipy/special/_ufuncs.pyx":14175
 * ufunc_yn_ptr[2*0+1] = <void*>(<char*>"yn")
 * ufunc_yn_ptr[2*1] = <void*>_func_yn_unsafe
 * ufunc_yn_ptr[2*1+1] = <void*>(<char*>"yn")             # <<<<<<<<<<<<<<
 * ufunc_yn_ptr[2*2] = <void*>_func_yn_unsafe
 * ufunc_yn_ptr[2*2+1] = <void*>(<char*>"yn")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_ptr[3]) = ((void *)((char *)((char *)"yn")));

  /* "scipy/special/_ufuncs.pyx":14176
 * ufunc_yn_ptr[2*1] = <void*>_func_yn_unsafe
 * ufunc_yn_ptr[2*1+1] = <void*>(<char*>"yn")
 * ufunc_yn_ptr[2*2] = <void*>_func_yn_unsafe             # <<<<<<<<<<<<<<
 * ufunc_yn_ptr[2*2+1] = <void*>(<char*>"yn")
 * ufunc_yn_data[0] = &ufunc_yn_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_ptr[4]) = ((void *)__pyx_f_5scipy_7special_7_legacy_yn_unsafe);

  /* "scipy/special/_ufuncs.pyx":14177
 * ufunc_yn_ptr[2*1+1] = <void*>(<char*>"yn")
 * ufunc_yn_ptr[2*2] = <void*>_func_yn_unsafe
 * ufunc_yn_ptr[2*2+1] = <void*>(<char*>"yn")             # <<<<<<<<<<<<<<
 * ufunc_yn_data[0] = &ufunc_yn_ptr[2*0]
 * ufunc_yn_data[1] = &ufunc_yn_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_ptr[5]) = ((void *)((char *)((char *)"yn")));

  /* "scipy/special/_ufuncs.pyx":14178
 * ufunc_yn_ptr[2*2] = <void*>_func_yn_unsafe
 * ufunc_yn_ptr[2*2+1] = <void*>(<char*>"yn")
 * ufunc_yn_data[0] = &ufunc_yn_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_yn_data[1] = &ufunc_yn_ptr[2*1]
 * ufunc_yn_data[2] = &ufunc_yn_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":14179
 * ufunc_yn_ptr[2*2+1] = <void*>(<char*>"yn")
 * ufunc_yn_data[0] = &ufunc_yn_ptr[2*0]
 * ufunc_yn_data[1] = &ufunc_yn_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_yn_data[2] = &ufunc_yn_ptr[2*2]
 * yn = np.PyUFunc_FromFuncAndData(ufunc_yn_loops, ufunc_yn_data, ufunc_yn_types, 3, 2, 1, 0, "yn", ufunc_yn_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":14180
 * ufunc_yn_data[0] = &ufunc_yn_ptr[2*0]
 * ufunc_yn_data[1] = &ufunc_yn_ptr[2*1]
 * ufunc_yn_data[2] = &ufunc_yn_ptr[2*2]             # <<<<<<<<<<<<<<
 * yn = np.PyUFunc_FromFuncAndData(ufunc_yn_loops, ufunc_yn_data, ufunc_yn_types, 3, 2, 1, 0, "yn", ufunc_yn_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":14181
 * ufunc_yn_data[1] = &ufunc_yn_ptr[2*1]
 * ufunc_yn_data[2] = &ufunc_yn_ptr[2*2]
 * yn = np.PyUFunc_FromFuncAndData(ufunc_yn_loops, ufunc_yn_data, ufunc_yn_types, 3, 2, 1, 0, "yn", ufunc_yn_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_yv_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_types, 3, 2, 1, 0, ((char *)"yn"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_yn_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 14181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_yn, __pyx_t_2) < 0) __PYX_ERR(1, 14181, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":14187
 * cdef void *ufunc_yv_data[4]
 * cdef char ufunc_yv_types[12]
 * cdef char *ufunc_yv_doc = (             # <<<<<<<<<<<<<<
 *     "yv(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_doc = ((char *)"yv(v, z)\n\nBessel function of the second kind of real order and complex argument.\n\nParameters\n----------\nv : array_like\n    Order (float).\nz : array_like\n    Argument (float or complex).\n\nReturns\n-------\nY : ndarray\n    Value of the Bessel function of the second kind, :math:`Y_v(x)`.\n\nNotes\n-----\nFor positive `v` values, the computation is carried out using the\nAMOS [1]_ `zbesy` routine, which exploits the connection to the Hankel\nBessel functions :math:`H_v^{(1)}` and :math:`H_v^{(2)}`,\n\n.. math:: Y_v(z) = \\frac{1}{2\\imath} (H_v^{(1)} - H_v^{(2)}).\n\nFor negative `v` values the formula,\n\n.. math:: Y_{-v}(z) = Y_v(z) \\cos(\\pi v) + J_v(z) \\sin(\\pi v)\n\nis used, where :math:`J_v(z)` is the Bessel function of the first kind,\ncomputed using the AMOS routine `zbesj`.  Note that the second term is\nexactly zero for integer `v`; to improve accuracy the second term is\nexplicitly omitted for `v` values such that `v = floor(v)`.\n\nSee also\n--------\nyve : :math:`Y_v` with leading exponential behavior stripped off.\n\nReferences\n----------\n.. [1] Donald E. Amos, \"AMOS, A Portable Package for Bessel Functions\n       of a Complex Argument and Nonnegative Order\",\n       http://netlib.org/amos/");

  /* "scipy/special/_ufuncs.pyx":14230
 *     "       of a Complex Argument and Nonnegative Order\",\n"
 *     "       http://netlib.org/amos/")
 * ufunc_yv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_yv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_yv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":14231
 *     "       http://netlib.org/amos/")
 * ufunc_yv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_yv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_yv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_yv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":14232
 * ufunc_yv_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_yv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_yv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_yv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_yv_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":14233
 * ufunc_yv_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_yv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_yv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_yv_types[0] = <char>NPY_FLOAT
 * ufunc_yv_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":14234
 * ufunc_yv_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_yv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_yv_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yv_types[1] = <char>NPY_FLOAT
 * ufunc_yv_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":14235
 * ufunc_yv_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_yv_types[0] = <char>NPY_FLOAT
 * ufunc_yv_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yv_types[2] = <char>NPY_FLOAT
 * ufunc_yv_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":14236
 * ufunc_yv_types[0] = <char>NPY_FLOAT
 * ufunc_yv_types[1] = <char>NPY_FLOAT
 * ufunc_yv_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yv_types[3] = <char>NPY_FLOAT
 * ufunc_yv_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":14237
 * ufunc_yv_types[1] = <char>NPY_FLOAT
 * ufunc_yv_types[2] = <char>NPY_FLOAT
 * ufunc_yv_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yv_types[4] = <char>NPY_CFLOAT
 * ufunc_yv_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":14238
 * ufunc_yv_types[2] = <char>NPY_FLOAT
 * ufunc_yv_types[3] = <char>NPY_FLOAT
 * ufunc_yv_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_yv_types[5] = <char>NPY_CFLOAT
 * ufunc_yv_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":14239
 * ufunc_yv_types[3] = <char>NPY_FLOAT
 * ufunc_yv_types[4] = <char>NPY_CFLOAT
 * ufunc_yv_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_yv_types[6] = <char>NPY_DOUBLE
 * ufunc_yv_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":14240
 * ufunc_yv_types[4] = <char>NPY_CFLOAT
 * ufunc_yv_types[5] = <char>NPY_CFLOAT
 * ufunc_yv_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yv_types[7] = <char>NPY_DOUBLE
 * ufunc_yv_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":14241
 * ufunc_yv_types[5] = <char>NPY_CFLOAT
 * ufunc_yv_types[6] = <char>NPY_DOUBLE
 * ufunc_yv_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yv_types[8] = <char>NPY_DOUBLE
 * ufunc_yv_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":14242
 * ufunc_yv_types[6] = <char>NPY_DOUBLE
 * ufunc_yv_types[7] = <char>NPY_DOUBLE
 * ufunc_yv_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yv_types[9] = <char>NPY_DOUBLE
 * ufunc_yv_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":14243
 * ufunc_yv_types[7] = <char>NPY_DOUBLE
 * ufunc_yv_types[8] = <char>NPY_DOUBLE
 * ufunc_yv_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yv_types[10] = <char>NPY_CDOUBLE
 * ufunc_yv_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":14244
 * ufunc_yv_types[8] = <char>NPY_DOUBLE
 * ufunc_yv_types[9] = <char>NPY_DOUBLE
 * ufunc_yv_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yv_types[11] = <char>NPY_CDOUBLE
 * ufunc_yv_ptr[2*0] = <void*>_func_cbesy_wrap_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":14245
 * ufunc_yv_types[9] = <char>NPY_DOUBLE
 * ufunc_yv_types[10] = <char>NPY_CDOUBLE
 * ufunc_yv_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yv_ptr[2*0] = <void*>_func_cbesy_wrap_real
 * ufunc_yv_ptr[2*0+1] = <void*>(<char*>"yv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":14246
 * ufunc_yv_types[10] = <char>NPY_CDOUBLE
 * ufunc_yv_types[11] = <char>NPY_CDOUBLE
 * ufunc_yv_ptr[2*0] = <void*>_func_cbesy_wrap_real             # <<<<<<<<<<<<<<
 * ufunc_yv_ptr[2*0+1] = <void*>(<char*>"yv")
 * ufunc_yv_ptr[2*1] = <void*>_func_cbesy_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[0]) = ((void *)cbesy_wrap_real);

  /* "scipy/special/_ufuncs.pyx":14247
 * ufunc_yv_types[11] = <char>NPY_CDOUBLE
 * ufunc_yv_ptr[2*0] = <void*>_func_cbesy_wrap_real
 * ufunc_yv_ptr[2*0+1] = <void*>(<char*>"yv")             # <<<<<<<<<<<<<<
 * ufunc_yv_ptr[2*1] = <void*>_func_cbesy_wrap
 * ufunc_yv_ptr[2*1+1] = <void*>(<char*>"yv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[1]) = ((void *)((char *)((char *)"yv")));

  /* "scipy/special/_ufuncs.pyx":14248
 * ufunc_yv_ptr[2*0] = <void*>_func_cbesy_wrap_real
 * ufunc_yv_ptr[2*0+1] = <void*>(<char*>"yv")
 * ufunc_yv_ptr[2*1] = <void*>_func_cbesy_wrap             # <<<<<<<<<<<<<<
 * ufunc_yv_ptr[2*1+1] = <void*>(<char*>"yv")
 * ufunc_yv_ptr[2*2] = <void*>_func_cbesy_wrap_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[2]) = ((void *)cbesy_wrap);

  /* "scipy/special/_ufuncs.pyx":14249
 * ufunc_yv_ptr[2*0+1] = <void*>(<char*>"yv")
 * ufunc_yv_ptr[2*1] = <void*>_func_cbesy_wrap
 * ufunc_yv_ptr[2*1+1] = <void*>(<char*>"yv")             # <<<<<<<<<<<<<<
 * ufunc_yv_ptr[2*2] = <void*>_func_cbesy_wrap_real
 * ufunc_yv_ptr[2*2+1] = <void*>(<char*>"yv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[3]) = ((void *)((char *)((char *)"yv")));

  /* "scipy/special/_ufuncs.pyx":14250
 * ufunc_yv_ptr[2*1] = <void*>_func_cbesy_wrap
 * ufunc_yv_ptr[2*1+1] = <void*>(<char*>"yv")
 * ufunc_yv_ptr[2*2] = <void*>_func_cbesy_wrap_real             # <<<<<<<<<<<<<<
 * ufunc_yv_ptr[2*2+1] = <void*>(<char*>"yv")
 * ufunc_yv_ptr[2*3] = <void*>_func_cbesy_wrap
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[4]) = ((void *)cbesy_wrap_real);

  /* "scipy/special/_ufuncs.pyx":14251
 * ufunc_yv_ptr[2*1+1] = <void*>(<char*>"yv")
 * ufunc_yv_ptr[2*2] = <void*>_func_cbesy_wrap_real
 * ufunc_yv_ptr[2*2+1] = <void*>(<char*>"yv")             # <<<<<<<<<<<<<<
 * ufunc_yv_ptr[2*3] = <void*>_func_cbesy_wrap
 * ufunc_yv_ptr[2*3+1] = <void*>(<char*>"yv")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[5]) = ((void *)((char *)((char *)"yv")));

  /* "scipy/special/_ufuncs.pyx":14252
 * ufunc_yv_ptr[2*2] = <void*>_func_cbesy_wrap_real
 * ufunc_yv_ptr[2*2+1] = <void*>(<char*>"yv")
 * ufunc_yv_ptr[2*3] = <void*>_func_cbesy_wrap             # <<<<<<<<<<<<<<
 * ufunc_yv_ptr[2*3+1] = <void*>(<char*>"yv")
 * ufunc_yv_data[0] = &ufunc_yv_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[6]) = ((void *)cbesy_wrap);

  /* "scipy/special/_ufuncs.pyx":14253
 * ufunc_yv_ptr[2*2+1] = <void*>(<char*>"yv")
 * ufunc_yv_ptr[2*3] = <void*>_func_cbesy_wrap
 * ufunc_yv_ptr[2*3+1] = <void*>(<char*>"yv")             # <<<<<<<<<<<<<<
 * ufunc_yv_data[0] = &ufunc_yv_ptr[2*0]
 * ufunc_yv_data[1] = &ufunc_yv_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[7]) = ((void *)((char *)((char *)"yv")));

  /* "scipy/special/_ufuncs.pyx":14254
 * ufunc_yv_ptr[2*3] = <void*>_func_cbesy_wrap
 * ufunc_yv_ptr[2*3+1] = <void*>(<char*>"yv")
 * ufunc_yv_data[0] = &ufunc_yv_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_yv_data[1] = &ufunc_yv_ptr[2*1]
 * ufunc_yv_data[2] = &ufunc_yv_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":14255
 * ufunc_yv_ptr[2*3+1] = <void*>(<char*>"yv")
 * ufunc_yv_data[0] = &ufunc_yv_ptr[2*0]
 * ufunc_yv_data[1] = &ufunc_yv_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_yv_data[2] = &ufunc_yv_ptr[2*2]
 * ufunc_yv_data[3] = &ufunc_yv_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":14256
 * ufunc_yv_data[0] = &ufunc_yv_ptr[2*0]
 * ufunc_yv_data[1] = &ufunc_yv_ptr[2*1]
 * ufunc_yv_data[2] = &ufunc_yv_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_yv_data[3] = &ufunc_yv_ptr[2*3]
 * yv = np.PyUFunc_FromFuncAndData(ufunc_yv_loops, ufunc_yv_data, ufunc_yv_types, 4, 2, 1, 0, "yv", ufunc_yv_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":14257
 * ufunc_yv_data[1] = &ufunc_yv_ptr[2*1]
 * ufunc_yv_data[2] = &ufunc_yv_ptr[2*2]
 * ufunc_yv_data[3] = &ufunc_yv_ptr[2*3]             # <<<<<<<<<<<<<<
 * yv = np.PyUFunc_FromFuncAndData(ufunc_yv_loops, ufunc_yv_data, ufunc_yv_types, 4, 2, 1, 0, "yv", ufunc_yv_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":14258
 * ufunc_yv_data[2] = &ufunc_yv_ptr[2*2]
 * ufunc_yv_data[3] = &ufunc_yv_ptr[2*3]
 * yv = np.PyUFunc_FromFuncAndData(ufunc_yv_loops, ufunc_yv_data, ufunc_yv_types, 4, 2, 1, 0, "yv", ufunc_yv_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_yve_loops[4]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_types, 4, 2, 1, 0, ((char *)"yv"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_yv_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 14258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_yv, __pyx_t_2) < 0) __PYX_ERR(1, 14258, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":14264
 * cdef void *ufunc_yve_data[4]
 * cdef char ufunc_yve_types[12]
 * cdef char *ufunc_yve_doc = (             # <<<<<<<<<<<<<<
 *     "yve(v, z)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_doc = ((char *)"yve(v, z)\n\nExponentially scaled Bessel function of the second kind of real order.\n\nReturns the exponentially scaled Bessel function of the second\nkind of real order `v` at complex `z`::\n\n    yve(v, z) = yv(v, z) * exp(-abs(z.imag))\n\nParameters\n----------\nv : array_like\n    Order (float).\nz : array_like\n    Argument (float or complex).\n\nReturns\n-------\nY : ndarray\n    Value of the exponentially scaled Bessel function.\n\nNotes\n-----\nFor positive `v` values, the computation is carried out using the\nAMOS [1]_ `zbesy` routine, which exploits the connection to the Hankel\nBessel functions :math:`H_v^{(1)}` and :math:`H_v^{(2)}`,\n\n.. math:: Y_v(z) = \\frac{1}{2\\imath} (H_v^{(1)} - H_v^{(2)}).\n\nFor negative `v` values the formula,\n\n.. math:: Y_{-v}(z) = Y_v(z) \\cos(\\pi v) + J_v(z) \\sin(\\pi v)\n\nis used, where :math:`J_v(z)` is the Bessel function of the first kind,\ncomputed using the AMOS routine `zbesj`.  Note that the second term is\nexactly zero for integer `v`; to improve accuracy the second term is\nexplicitly omitted for `v` values such that `v = floor(v)`.\n\nReferences\n----------\n.. [1] Donald E. Amos, \"AMOS, A Portable Package for Bessel Functions\n       of a Complex Argument and Nonnegative Order\",\n       http://netlib.org/amos/");

  /* "scipy/special/_ufuncs.pyx":14308
 *     "       of a Complex Argument and Nonnegative Order\",\n"
 *     "       http://netlib.org/amos/")
 * ufunc_yve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f             # <<<<<<<<<<<<<<
 * ufunc_yve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_yve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_ff_f);

  /* "scipy/special/_ufuncs.pyx":14309
 *     "       http://netlib.org/amos/")
 * ufunc_yve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_yve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F             # <<<<<<<<<<<<<<
 * ufunc_yve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_yve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_fF_F);

  /* "scipy/special/_ufuncs.pyx":14310
 * ufunc_yve_loops[0] = <np.PyUFuncGenericFunction>loop_d_dd__As_ff_f
 * ufunc_yve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_yve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d             # <<<<<<<<<<<<<<
 * ufunc_yve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_yve_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_loops[2]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_dd__As_dd_d);

  /* "scipy/special/_ufuncs.pyx":14311
 * ufunc_yve_loops[1] = <np.PyUFuncGenericFunction>loop_D_dD__As_fF_F
 * ufunc_yve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_yve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D             # <<<<<<<<<<<<<<
 * ufunc_yve_types[0] = <char>NPY_FLOAT
 * ufunc_yve_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_loops[3]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_D_dD__As_dD_D);

  /* "scipy/special/_ufuncs.pyx":14312
 * ufunc_yve_loops[2] = <np.PyUFuncGenericFunction>loop_d_dd__As_dd_d
 * ufunc_yve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_yve_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yve_types[1] = <char>NPY_FLOAT
 * ufunc_yve_types[2] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":14313
 * ufunc_yve_loops[3] = <np.PyUFuncGenericFunction>loop_D_dD__As_dD_D
 * ufunc_yve_types[0] = <char>NPY_FLOAT
 * ufunc_yve_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yve_types[2] = <char>NPY_FLOAT
 * ufunc_yve_types[3] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":14314
 * ufunc_yve_types[0] = <char>NPY_FLOAT
 * ufunc_yve_types[1] = <char>NPY_FLOAT
 * ufunc_yve_types[2] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yve_types[3] = <char>NPY_FLOAT
 * ufunc_yve_types[4] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[2]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":14315
 * ufunc_yve_types[1] = <char>NPY_FLOAT
 * ufunc_yve_types[2] = <char>NPY_FLOAT
 * ufunc_yve_types[3] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_yve_types[4] = <char>NPY_CFLOAT
 * ufunc_yve_types[5] = <char>NPY_CFLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[3]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":14316
 * ufunc_yve_types[2] = <char>NPY_FLOAT
 * ufunc_yve_types[3] = <char>NPY_FLOAT
 * ufunc_yve_types[4] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_yve_types[5] = <char>NPY_CFLOAT
 * ufunc_yve_types[6] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[4]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":14317
 * ufunc_yve_types[3] = <char>NPY_FLOAT
 * ufunc_yve_types[4] = <char>NPY_CFLOAT
 * ufunc_yve_types[5] = <char>NPY_CFLOAT             # <<<<<<<<<<<<<<
 * ufunc_yve_types[6] = <char>NPY_DOUBLE
 * ufunc_yve_types[7] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[5]) = ((char)NPY_CFLOAT);

  /* "scipy/special/_ufuncs.pyx":14318
 * ufunc_yve_types[4] = <char>NPY_CFLOAT
 * ufunc_yve_types[5] = <char>NPY_CFLOAT
 * ufunc_yve_types[6] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yve_types[7] = <char>NPY_DOUBLE
 * ufunc_yve_types[8] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[6]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":14319
 * ufunc_yve_types[5] = <char>NPY_CFLOAT
 * ufunc_yve_types[6] = <char>NPY_DOUBLE
 * ufunc_yve_types[7] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yve_types[8] = <char>NPY_DOUBLE
 * ufunc_yve_types[9] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[7]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":14320
 * ufunc_yve_types[6] = <char>NPY_DOUBLE
 * ufunc_yve_types[7] = <char>NPY_DOUBLE
 * ufunc_yve_types[8] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yve_types[9] = <char>NPY_DOUBLE
 * ufunc_yve_types[10] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[8]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":14321
 * ufunc_yve_types[7] = <char>NPY_DOUBLE
 * ufunc_yve_types[8] = <char>NPY_DOUBLE
 * ufunc_yve_types[9] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yve_types[10] = <char>NPY_CDOUBLE
 * ufunc_yve_types[11] = <char>NPY_CDOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[9]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":14322
 * ufunc_yve_types[8] = <char>NPY_DOUBLE
 * ufunc_yve_types[9] = <char>NPY_DOUBLE
 * ufunc_yve_types[10] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yve_types[11] = <char>NPY_CDOUBLE
 * ufunc_yve_ptr[2*0] = <void*>_func_cbesy_wrap_e_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[10]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":14323
 * ufunc_yve_types[9] = <char>NPY_DOUBLE
 * ufunc_yve_types[10] = <char>NPY_CDOUBLE
 * ufunc_yve_types[11] = <char>NPY_CDOUBLE             # <<<<<<<<<<<<<<
 * ufunc_yve_ptr[2*0] = <void*>_func_cbesy_wrap_e_real
 * ufunc_yve_ptr[2*0+1] = <void*>(<char*>"yve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types[11]) = ((char)NPY_CDOUBLE);

  /* "scipy/special/_ufuncs.pyx":14324
 * ufunc_yve_types[10] = <char>NPY_CDOUBLE
 * ufunc_yve_types[11] = <char>NPY_CDOUBLE
 * ufunc_yve_ptr[2*0] = <void*>_func_cbesy_wrap_e_real             # <<<<<<<<<<<<<<
 * ufunc_yve_ptr[2*0+1] = <void*>(<char*>"yve")
 * ufunc_yve_ptr[2*1] = <void*>_func_cbesy_wrap_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[0]) = ((void *)cbesy_wrap_e_real);

  /* "scipy/special/_ufuncs.pyx":14325
 * ufunc_yve_types[11] = <char>NPY_CDOUBLE
 * ufunc_yve_ptr[2*0] = <void*>_func_cbesy_wrap_e_real
 * ufunc_yve_ptr[2*0+1] = <void*>(<char*>"yve")             # <<<<<<<<<<<<<<
 * ufunc_yve_ptr[2*1] = <void*>_func_cbesy_wrap_e
 * ufunc_yve_ptr[2*1+1] = <void*>(<char*>"yve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[1]) = ((void *)((char *)((char *)"yve")));

  /* "scipy/special/_ufuncs.pyx":14326
 * ufunc_yve_ptr[2*0] = <void*>_func_cbesy_wrap_e_real
 * ufunc_yve_ptr[2*0+1] = <void*>(<char*>"yve")
 * ufunc_yve_ptr[2*1] = <void*>_func_cbesy_wrap_e             # <<<<<<<<<<<<<<
 * ufunc_yve_ptr[2*1+1] = <void*>(<char*>"yve")
 * ufunc_yve_ptr[2*2] = <void*>_func_cbesy_wrap_e_real
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[2]) = ((void *)cbesy_wrap_e);

  /* "scipy/special/_ufuncs.pyx":14327
 * ufunc_yve_ptr[2*0+1] = <void*>(<char*>"yve")
 * ufunc_yve_ptr[2*1] = <void*>_func_cbesy_wrap_e
 * ufunc_yve_ptr[2*1+1] = <void*>(<char*>"yve")             # <<<<<<<<<<<<<<
 * ufunc_yve_ptr[2*2] = <void*>_func_cbesy_wrap_e_real
 * ufunc_yve_ptr[2*2+1] = <void*>(<char*>"yve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[3]) = ((void *)((char *)((char *)"yve")));

  /* "scipy/special/_ufuncs.pyx":14328
 * ufunc_yve_ptr[2*1] = <void*>_func_cbesy_wrap_e
 * ufunc_yve_ptr[2*1+1] = <void*>(<char*>"yve")
 * ufunc_yve_ptr[2*2] = <void*>_func_cbesy_wrap_e_real             # <<<<<<<<<<<<<<
 * ufunc_yve_ptr[2*2+1] = <void*>(<char*>"yve")
 * ufunc_yve_ptr[2*3] = <void*>_func_cbesy_wrap_e
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[4]) = ((void *)cbesy_wrap_e_real);

  /* "scipy/special/_ufuncs.pyx":14329
 * ufunc_yve_ptr[2*1+1] = <void*>(<char*>"yve")
 * ufunc_yve_ptr[2*2] = <void*>_func_cbesy_wrap_e_real
 * ufunc_yve_ptr[2*2+1] = <void*>(<char*>"yve")             # <<<<<<<<<<<<<<
 * ufunc_yve_ptr[2*3] = <void*>_func_cbesy_wrap_e
 * ufunc_yve_ptr[2*3+1] = <void*>(<char*>"yve")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[5]) = ((void *)((char *)((char *)"yve")));

  /* "scipy/special/_ufuncs.pyx":14330
 * ufunc_yve_ptr[2*2] = <void*>_func_cbesy_wrap_e_real
 * ufunc_yve_ptr[2*2+1] = <void*>(<char*>"yve")
 * ufunc_yve_ptr[2*3] = <void*>_func_cbesy_wrap_e             # <<<<<<<<<<<<<<
 * ufunc_yve_ptr[2*3+1] = <void*>(<char*>"yve")
 * ufunc_yve_data[0] = &ufunc_yve_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[6]) = ((void *)cbesy_wrap_e);

  /* "scipy/special/_ufuncs.pyx":14331
 * ufunc_yve_ptr[2*2+1] = <void*>(<char*>"yve")
 * ufunc_yve_ptr[2*3] = <void*>_func_cbesy_wrap_e
 * ufunc_yve_ptr[2*3+1] = <void*>(<char*>"yve")             # <<<<<<<<<<<<<<
 * ufunc_yve_data[0] = &ufunc_yve_ptr[2*0]
 * ufunc_yve_data[1] = &ufunc_yve_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[7]) = ((void *)((char *)((char *)"yve")));

  /* "scipy/special/_ufuncs.pyx":14332
 * ufunc_yve_ptr[2*3] = <void*>_func_cbesy_wrap_e
 * ufunc_yve_ptr[2*3+1] = <void*>(<char*>"yve")
 * ufunc_yve_data[0] = &ufunc_yve_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_yve_data[1] = &ufunc_yve_ptr[2*1]
 * ufunc_yve_data[2] = &ufunc_yve_ptr[2*2]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":14333
 * ufunc_yve_ptr[2*3+1] = <void*>(<char*>"yve")
 * ufunc_yve_data[0] = &ufunc_yve_ptr[2*0]
 * ufunc_yve_data[1] = &ufunc_yve_ptr[2*1]             # <<<<<<<<<<<<<<
 * ufunc_yve_data[2] = &ufunc_yve_ptr[2*2]
 * ufunc_yve_data[3] = &ufunc_yve_ptr[2*3]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":14334
 * ufunc_yve_data[0] = &ufunc_yve_ptr[2*0]
 * ufunc_yve_data[1] = &ufunc_yve_ptr[2*1]
 * ufunc_yve_data[2] = &ufunc_yve_ptr[2*2]             # <<<<<<<<<<<<<<
 * ufunc_yve_data[3] = &ufunc_yve_ptr[2*3]
 * yve = np.PyUFunc_FromFuncAndData(ufunc_yve_loops, ufunc_yve_data, ufunc_yve_types, 4, 2, 1, 0, "yve", ufunc_yve_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_data[2]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[4]));

  /* "scipy/special/_ufuncs.pyx":14335
 * ufunc_yve_data[1] = &ufunc_yve_ptr[2*1]
 * ufunc_yve_data[2] = &ufunc_yve_ptr[2*2]
 * ufunc_yve_data[3] = &ufunc_yve_ptr[2*3]             # <<<<<<<<<<<<<<
 * yve = np.PyUFunc_FromFuncAndData(ufunc_yve_loops, ufunc_yve_data, ufunc_yve_types, 4, 2, 1, 0, "yve", ufunc_yve_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_data[3]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_ptr[6]));

  /* "scipy/special/_ufuncs.pyx":14336
 * ufunc_yve_data[2] = &ufunc_yve_ptr[2*2]
 * ufunc_yve_data[3] = &ufunc_yve_ptr[2*3]
 * yve = np.PyUFunc_FromFuncAndData(ufunc_yve_loops, ufunc_yve_data, ufunc_yve_types, 4, 2, 1, 0, "yve", ufunc_yve_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * cdef np.PyUFuncGenericFunction ufunc_zetac_loops[2]
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_types, 4, 2, 1, 0, ((char *)"yve"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_yve_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 14336, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_yve, __pyx_t_2) < 0) __PYX_ERR(1, 14336, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":14342
 * cdef void *ufunc_zetac_data[2]
 * cdef char ufunc_zetac_types[4]
 * cdef char *ufunc_zetac_doc = (             # <<<<<<<<<<<<<<
 *     "zetac(x)\n"
 *     "\n"
 */
  __pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_doc = ((char *)"zetac(x)\n\nRiemann zeta function minus 1.\n\nThis function is defined as\n\n.. math:: \\zeta(x) = \\sum_{k=2}^{\\infty} 1 / k^x,\n\nwhere ``x > 1``.  For ``x < 1``, the analytic continuation is computed.\n\nBecause of limitations of the numerical algorithm, ``zetac(x)`` returns\n`nan` for `x` less than -30.8148.\n\nParameters\n----------\nx : array_like of float\n    Values at which to compute zeta(x) - 1 (must be real).\n\nReturns\n-------\nout : array_like\n    Values of zeta(x) - 1.\n\nSee Also\n--------\nzeta\n\nExamples\n--------\n>>> from scipy.special import zetac, zeta\n\nSome special values:\n\n>>> zetac(2), np.pi**2/6 - 1\n(0.64493406684822641, 0.6449340668482264)\n\n>>> zetac(-1), -1.0/12 - 1\n(-1.0833333333333333, -1.0833333333333333)\n\nCompare ``zetac(x)`` to ``zeta(x) - 1`` for large `x`:\n\n>>> zetac(60), zeta(60) - 1\n(8.673617380119933e-19, 0.0)");

  /* "scipy/special/_ufuncs.pyx":14386
 *     ">>> zetac(60), zeta(60) - 1\n"
 *     "(8.673617380119933e-19, 0.0)")
 * ufunc_zetac_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f             # <<<<<<<<<<<<<<
 * ufunc_zetac_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_zetac_types[0] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_loops[0]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_f_f);

  /* "scipy/special/_ufuncs.pyx":14387
 *     "(8.673617380119933e-19, 0.0)")
 * ufunc_zetac_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_zetac_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d             # <<<<<<<<<<<<<<
 * ufunc_zetac_types[0] = <char>NPY_FLOAT
 * ufunc_zetac_types[1] = <char>NPY_FLOAT
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_loops[1]) = ((PyUFuncGenericFunction)__pyx_f_5scipy_7special_7_ufuncs_loop_d_d__As_d_d);

  /* "scipy/special/_ufuncs.pyx":14388
 * ufunc_zetac_loops[0] = <np.PyUFuncGenericFunction>loop_d_d__As_f_f
 * ufunc_zetac_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_zetac_types[0] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_zetac_types[1] = <char>NPY_FLOAT
 * ufunc_zetac_types[2] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_types[0]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":14389
 * ufunc_zetac_loops[1] = <np.PyUFuncGenericFunction>loop_d_d__As_d_d
 * ufunc_zetac_types[0] = <char>NPY_FLOAT
 * ufunc_zetac_types[1] = <char>NPY_FLOAT             # <<<<<<<<<<<<<<
 * ufunc_zetac_types[2] = <char>NPY_DOUBLE
 * ufunc_zetac_types[3] = <char>NPY_DOUBLE
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_types[1]) = ((char)NPY_FLOAT);

  /* "scipy/special/_ufuncs.pyx":14390
 * ufunc_zetac_types[0] = <char>NPY_FLOAT
 * ufunc_zetac_types[1] = <char>NPY_FLOAT
 * ufunc_zetac_types[2] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_zetac_types[3] = <char>NPY_DOUBLE
 * ufunc_zetac_ptr[2*0] = <void*>_func_zetac
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_types[2]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":14391
 * ufunc_zetac_types[1] = <char>NPY_FLOAT
 * ufunc_zetac_types[2] = <char>NPY_DOUBLE
 * ufunc_zetac_types[3] = <char>NPY_DOUBLE             # <<<<<<<<<<<<<<
 * ufunc_zetac_ptr[2*0] = <void*>_func_zetac
 * ufunc_zetac_ptr[2*0+1] = <void*>(<char*>"zetac")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_types[3]) = ((char)NPY_DOUBLE);

  /* "scipy/special/_ufuncs.pyx":14392
 * ufunc_zetac_types[2] = <char>NPY_DOUBLE
 * ufunc_zetac_types[3] = <char>NPY_DOUBLE
 * ufunc_zetac_ptr[2*0] = <void*>_func_zetac             # <<<<<<<<<<<<<<
 * ufunc_zetac_ptr[2*0+1] = <void*>(<char*>"zetac")
 * ufunc_zetac_ptr[2*1] = <void*>_func_zetac
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_ptr[0]) = ((void *)zetac);

  /* "scipy/special/_ufuncs.pyx":14393
 * ufunc_zetac_types[3] = <char>NPY_DOUBLE
 * ufunc_zetac_ptr[2*0] = <void*>_func_zetac
 * ufunc_zetac_ptr[2*0+1] = <void*>(<char*>"zetac")             # <<<<<<<<<<<<<<
 * ufunc_zetac_ptr[2*1] = <void*>_func_zetac
 * ufunc_zetac_ptr[2*1+1] = <void*>(<char*>"zetac")
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_ptr[1]) = ((void *)((char *)((char *)"zetac")));

  /* "scipy/special/_ufuncs.pyx":14394
 * ufunc_zetac_ptr[2*0] = <void*>_func_zetac
 * ufunc_zetac_ptr[2*0+1] = <void*>(<char*>"zetac")
 * ufunc_zetac_ptr[2*1] = <void*>_func_zetac             # <<<<<<<<<<<<<<
 * ufunc_zetac_ptr[2*1+1] = <void*>(<char*>"zetac")
 * ufunc_zetac_data[0] = &ufunc_zetac_ptr[2*0]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_ptr[2]) = ((void *)zetac);

  /* "scipy/special/_ufuncs.pyx":14395
 * ufunc_zetac_ptr[2*0+1] = <void*>(<char*>"zetac")
 * ufunc_zetac_ptr[2*1] = <void*>_func_zetac
 * ufunc_zetac_ptr[2*1+1] = <void*>(<char*>"zetac")             # <<<<<<<<<<<<<<
 * ufunc_zetac_data[0] = &ufunc_zetac_ptr[2*0]
 * ufunc_zetac_data[1] = &ufunc_zetac_ptr[2*1]
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_ptr[3]) = ((void *)((char *)((char *)"zetac")));

  /* "scipy/special/_ufuncs.pyx":14396
 * ufunc_zetac_ptr[2*1] = <void*>_func_zetac
 * ufunc_zetac_ptr[2*1+1] = <void*>(<char*>"zetac")
 * ufunc_zetac_data[0] = &ufunc_zetac_ptr[2*0]             # <<<<<<<<<<<<<<
 * ufunc_zetac_data[1] = &ufunc_zetac_ptr[2*1]
 * zetac = np.PyUFunc_FromFuncAndData(ufunc_zetac_loops, ufunc_zetac_data, ufunc_zetac_types, 2, 1, 1, 0, "zetac", ufunc_zetac_doc, 0)
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_data[0]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_ptr[0]));

  /* "scipy/special/_ufuncs.pyx":14397
 * ufunc_zetac_ptr[2*1+1] = <void*>(<char*>"zetac")
 * ufunc_zetac_data[0] = &ufunc_zetac_ptr[2*0]
 * ufunc_zetac_data[1] = &ufunc_zetac_ptr[2*1]             # <<<<<<<<<<<<<<
 * zetac = np.PyUFunc_FromFuncAndData(ufunc_zetac_loops, ufunc_zetac_data, ufunc_zetac_types, 2, 1, 1, 0, "zetac", ufunc_zetac_doc, 0)
 * 
 */
  (__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_data[1]) = (&(__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_ptr[2]));

  /* "scipy/special/_ufuncs.pyx":14398
 * ufunc_zetac_data[0] = &ufunc_zetac_ptr[2*0]
 * ufunc_zetac_data[1] = &ufunc_zetac_ptr[2*1]
 * zetac = np.PyUFunc_FromFuncAndData(ufunc_zetac_loops, ufunc_zetac_data, ufunc_zetac_types, 2, 1, 1, 0, "zetac", ufunc_zetac_doc, 0)             # <<<<<<<<<<<<<<
 * 
 * #
 */
  __pyx_t_2 = PyUFunc_FromFuncAndData(__pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_loops, __pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_data, __pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_types, 2, 1, 1, 0, ((char *)"zetac"), __pyx_v_5scipy_7special_7_ufuncs_ufunc_zetac_doc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 14398, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_zetac, __pyx_t_2) < 0) __PYX_ERR(1, 14398, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":14403
 * # Aliases
 * #
 * jn = jv             # <<<<<<<<<<<<<<
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_jv); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 14403, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_jn, __pyx_t_2) < 0) __PYX_ERR(1, 14403, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "scipy/special/_ufuncs.pyx":1
 * # This file is automatically generated by _generate_pyx.py.             # <<<<<<<<<<<<<<
 * # Do not edit manually!
 * from __future__ import absolute_import
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_kp_u_geterr_line_28, __pyx_kp_u_Get_the_current_way_of_handling) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_kp_u_seterr_line_77, __pyx_kp_u_Set_how_special_function_errors) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "_spence.pxd":74
 * 
 * @cython.cdivision(True)
 * cdef inline double complex cspence_series1(double complex z) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     A series centered at z = 1 which enjoys faster convergence than
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init scipy.special._ufuncs", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init scipy.special._ufuncs");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectGetMethod */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if PY_MAJOR_VERSION >= 3
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type)))
        #endif
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (descr != NULL) {
        *method = descr;
        return 0;
    }
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(name));
#endif
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod0 */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* RaiseNoneIterError */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* UnpackTupleError */
static void __Pyx_UnpackTupleError(PyObject *t, Py_ssize_t index) {
    if (t == Py_None) {
      __Pyx_RaiseNoneNotIterableError();
    } else if (PyTuple_GET_SIZE(t) < index) {
      __Pyx_RaiseNeedMoreValuesError(PyTuple_GET_SIZE(t));
    } else {
      __Pyx_RaiseTooManyValuesError(index);
    }
}

/* UnpackTuple2 */
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
        PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2, int decref_tuple) {
    PyObject *value1 = NULL, *value2 = NULL;
#if CYTHON_COMPILING_IN_PYPY
    value1 = PySequence_ITEM(tuple, 0);  if (unlikely(!value1)) goto bad;
    value2 = PySequence_ITEM(tuple, 1);  if (unlikely(!value2)) goto bad;
#else
    value1 = PyTuple_GET_ITEM(tuple, 0);  Py_INCREF(value1);
    value2 = PyTuple_GET_ITEM(tuple, 1);  Py_INCREF(value2);
#endif
    if (decref_tuple) {
        Py_DECREF(tuple);
    }
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
#if CYTHON_COMPILING_IN_PYPY
bad:
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
#endif
}
static int __Pyx_unpack_tuple2_generic(PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2,
                                       int has_known_size, int decref_tuple) {
    Py_ssize_t index;
    PyObject *value1 = NULL, *value2 = NULL, *iter = NULL;
    iternextfunc iternext;
    iter = PyObject_GetIter(tuple);
    if (unlikely(!iter)) goto bad;
    if (decref_tuple) { Py_DECREF(tuple); tuple = NULL; }
    iternext = Py_TYPE(iter)->tp_iternext;
    value1 = iternext(iter); if (unlikely(!value1)) { index = 0; goto unpacking_failed; }
    value2 = iternext(iter); if (unlikely(!value2)) { index = 1; goto unpacking_failed; }
    if (!has_known_size && unlikely(__Pyx_IternextUnpackEndCheck(iternext(iter), 2))) goto bad;
    Py_DECREF(iter);
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
unpacking_failed:
    if (!has_known_size && __Pyx_IterFinish() == 0)
        __Pyx_RaiseNeedMoreValuesError(index);
bad:
    Py_XDECREF(iter);
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
}

/* dict_iter */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* iterable, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_source_is_dict) {
    is_dict = is_dict || likely(PyDict_CheckExact(iterable));
    *p_source_is_dict = is_dict;
    if (is_dict) {
#if !CYTHON_COMPILING_IN_PYPY
        *p_orig_length = PyDict_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
#elif PY_MAJOR_VERSION >= 3
        static PyObject *py_items = NULL, *py_keys = NULL, *py_values = NULL;
        PyObject **pp = NULL;
        if (method_name) {
            const char *name = PyUnicode_AsUTF8(method_name);
            if (strcmp(name, "iteritems") == 0) pp = &py_items;
            else if (strcmp(name, "iterkeys") == 0) pp = &py_keys;
            else if (strcmp(name, "itervalues") == 0) pp = &py_values;
            if (pp) {
                if (!*pp) {
                    *pp = PyUnicode_FromString(name + 4);
                    if (!*pp)
                        return NULL;
                }
                method_name = *pp;
            }
        }
#endif
    }
    *p_orig_length = 0;
    if (method_name) {
        PyObject* iter;
        iterable = __Pyx_PyObject_CallMethod0(iterable, method_name);
        if (!iterable)
            return NULL;
#if !CYTHON_COMPILING_IN_PYPY
        if (PyTuple_CheckExact(iterable) || PyList_CheckExact(iterable))
            return iterable;
#endif
        iter = PyObject_GetIter(iterable);
        Py_DECREF(iterable);
        return iter;
    }
    return PyObject_GetIter(iterable);
}
static CYTHON_INLINE int __Pyx_dict_iter_next(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem, int source_is_dict) {
    PyObject* next_item;
#if !CYTHON_COMPILING_IN_PYPY
    if (source_is_dict) {
        PyObject *key, *value;
        if (unlikely(orig_length != PyDict_Size(iter_obj))) {
            PyErr_SetString(PyExc_RuntimeError, "dictionary changed size during iteration");
            return -1;
        }
        if (unlikely(!PyDict_Next(iter_obj, ppos, &key, &value))) {
            return 0;
        }
        if (pitem) {
            PyObject* tuple = PyTuple_New(2);
            if (unlikely(!tuple)) {
                return -1;
            }
            Py_INCREF(key);
            Py_INCREF(value);
            PyTuple_SET_ITEM(tuple, 0, key);
            PyTuple_SET_ITEM(tuple, 1, value);
            *pitem = tuple;
        } else {
            if (pkey) {
                Py_INCREF(key);
                *pkey = key;
            }
            if (pvalue) {
                Py_INCREF(value);
                *pvalue = value;
            }
        }
        return 1;
    } else if (PyTuple_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyTuple_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyTuple_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else if (PyList_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyList_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyList_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else
#endif
    {
        next_item = PyIter_Next(iter_obj);
        if (unlikely(!next_item)) {
            return __Pyx_IterFinish();
        }
    }
    if (pitem) {
        *pitem = next_item;
    } else if (pkey && pvalue) {
        if (__Pyx_unpack_tuple2(next_item, pkey, pvalue, source_is_dict, source_is_dict, 1))
            return -1;
    } else if (pkey) {
        *pkey = next_item;
    } else {
        *pvalue = next_item;
    }
    return 1;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* KeywordStringCheck */
static int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* UnpackUnboundCMethod */
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    target->method = method;
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION >= 3
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    #endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    }
#endif
    return 0;
}

/* CallUnboundCMethod0 */
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_ASSUME_SAFE_MACROS
    args = PyTuple_New(1);
    if (unlikely(!args)) goto bad;
    Py_INCREF(self);
    PyTuple_SET_ITEM(args, 0, self);
#else
    args = PyTuple_Pack(1, self);
    if (unlikely(!args)) goto bad;
#endif
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    Py_DECREF(args);
bad:
    return result;
}

/* py_dict_keys */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Keys(PyObject* d) {
    if (PY_MAJOR_VERSION >= 3)
        return __Pyx_CallUnboundCMethod0(&__pyx_umethod_PyDict_Type_keys, d);
    else
        return PyDict_Keys(d);
}

/* CallUnboundCMethod2 */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030600B1
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2) {
    if (likely(cfunc->func)) {
        PyObject *args[2] = {arg1, arg2};
        if (cfunc->flag == METH_FASTCALL) {
            #if PY_VERSION_HEX >= 0x030700A0
            return (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)cfunc->func)(self, args, 2);
            #else
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, 2, NULL);
            #endif
        }
        #if PY_VERSION_HEX >= 0x030700A0
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, 2, NULL);
        #endif
    }
    return __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(3);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 1, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 2, arg2);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(3, self, arg1, arg2);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* CallUnboundCMethod1 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg) {
    if (likely(cfunc->func)) {
        int flag = cfunc->flag;
        if (flag == METH_O) {
            return (*(cfunc->func))(self, arg);
        } else if (PY_VERSION_HEX >= 0x030600B1 && flag == METH_FASTCALL) {
            if (PY_VERSION_HEX >= 0x030700A0) {
                return (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)cfunc->func)(self, &arg, 1);
            } else {
                return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, &arg, 1, NULL);
            }
        } else if (PY_VERSION_HEX >= 0x030700A0 && flag == (METH_FASTCALL | METH_KEYWORDS)) {
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, &arg, 1, NULL);
        }
    }
    return __Pyx__CallUnboundCMethod1(cfunc, self, arg);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(1);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 0, arg);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 1, arg);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(2, self, arg);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* py_dict_pop */
static CYTHON_INLINE PyObject *__Pyx_PyDict_Pop(PyObject *d, PyObject *key, PyObject *default_value) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX > 0x030600B3
    if ((1)) {
        return _PyDict_Pop(d, key, default_value);
    } else
#endif
    if (default_value) {
        return __Pyx_CallUnboundCMethod2(&__pyx_umethod_PyDict_Type_pop, d, key, default_value);
    } else {
        return __Pyx_CallUnboundCMethod1(&__pyx_umethod_PyDict_Type_pop, d, key);
    }
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* DictGetItem */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            if (unlikely(PyTuple_Check(key))) {
                PyObject* args = PyTuple_Pack(1, key);
                if (likely(args)) {
                    PyErr_SetObject(PyExc_KeyError, args);
                    Py_DECREF(args);
                }
            } else {
                PyErr_SetObject(PyExc_KeyError, key);
            }
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* None */
static CYTHON_INLINE long __Pyx_div_long(long a, long b) {
    long q = a / b;
    long r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* CalculateMetaclass */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* FetchCommonType */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunction */
#include <structmember.h>
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromString(m->func.m_ml->ml_name);
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_New(PyTypeObject *type, PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    __pyx_CyFunctionObject *op = PyObject_GC_New(__pyx_CyFunctionObject, type);
    if (op == NULL)
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    PyObject_GC_Track(op);
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* Py3ClassCreate */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* Declarations */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabs(b.real) >= fabs(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                double r = b.imag / b.real;
                double s = (double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            double r = b.real / b.imag;
            double s = (double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(a, a);
                    case 3:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, a);
                    case 4:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = pow(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_double(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_sf_action_t(sf_action_t value) {
    const sf_action_t neg_one = (sf_action_t) ((sf_action_t) 0 - (sf_action_t) 1), const_zero = (sf_action_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(sf_action_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(sf_action_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(sf_action_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(sf_action_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(sf_action_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(sf_action_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Py_intptr_t(Py_intptr_t value) {
    const Py_intptr_t neg_one = (Py_intptr_t) ((Py_intptr_t) 0 - (Py_intptr_t) 1), const_zero = (Py_intptr_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(Py_intptr_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(Py_intptr_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(Py_intptr_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(Py_intptr_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(Py_intptr_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(Py_intptr_t),
                                     little, !is_unsigned);
    }
}

/* Declarations */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsf(b.real) >= fabsf(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                float r = b.imag / b.real;
                float s = (float)(1.0) / (b.real + b.imag * r);
                return __pyx_t_float_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            float r = b.real / b.imag;
            float s = (float)(1.0) / (b.imag + b.real * r);
            return __pyx_t_float_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            float denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_float_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(a, a);
                    case 3:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, a);
                    case 4:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if (b.imag == 0) {
                    z.real = powf(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2f(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_float(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__NPY_TYPES(enum NPY_TYPES value) {
    const enum NPY_TYPES neg_one = (enum NPY_TYPES) ((enum NPY_TYPES) 0 - (enum NPY_TYPES) 1), const_zero = (enum NPY_TYPES) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum NPY_TYPES) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum NPY_TYPES) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(enum NPY_TYPES) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(enum NPY_TYPES) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum NPY_TYPES),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* None */
static CYTHON_INLINE long __Pyx_pow_long(long b, long e) {
    long t = b;
    switch (e) {
        case 3:
            t *= b;
        CYTHON_FALLTHROUGH;
        case 2:
            t *= b;
        CYTHON_FALLTHROUGH;
        case 1:
            return t;
        case 0:
            return 1;
    }
    #if 1
    if (unlikely(e<0)) return 0;
    #endif
    t = 1;
    while (likely(e)) {
        t *= (b * (e&1)) | ((~e)&1);
        b *= b;
        e >>= 1;
    }
    return t;
}

/* CIntFromPy */
static CYTHON_INLINE sf_error_t __Pyx_PyInt_As_sf_error_t(PyObject *x) {
    const sf_error_t neg_one = (sf_error_t) ((sf_error_t) 0 - (sf_error_t) 1), const_zero = (sf_error_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(sf_error_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(sf_error_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (sf_error_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (sf_error_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(sf_error_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(sf_error_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sf_error_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sf_error_t) >= 2 * PyLong_SHIFT) {
                            return (sf_error_t) (((((sf_error_t)digits[1]) << PyLong_SHIFT) | (sf_error_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(sf_error_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sf_error_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sf_error_t) >= 3 * PyLong_SHIFT) {
                            return (sf_error_t) (((((((sf_error_t)digits[2]) << PyLong_SHIFT) | (sf_error_t)digits[1]) << PyLong_SHIFT) | (sf_error_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(sf_error_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sf_error_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sf_error_t) >= 4 * PyLong_SHIFT) {
                            return (sf_error_t) (((((((((sf_error_t)digits[3]) << PyLong_SHIFT) | (sf_error_t)digits[2]) << PyLong_SHIFT) | (sf_error_t)digits[1]) << PyLong_SHIFT) | (sf_error_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (sf_error_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(sf_error_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(sf_error_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(sf_error_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(sf_error_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (sf_error_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(sf_error_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(sf_error_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(sf_error_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sf_error_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sf_error_t) - 1 > 2 * PyLong_SHIFT) {
                            return (sf_error_t) (((sf_error_t)-1)*(((((sf_error_t)digits[1]) << PyLong_SHIFT) | (sf_error_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(sf_error_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sf_error_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sf_error_t) - 1 > 2 * PyLong_SHIFT) {
                            return (sf_error_t) ((((((sf_error_t)digits[1]) << PyLong_SHIFT) | (sf_error_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(sf_error_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sf_error_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sf_error_t) - 1 > 3 * PyLong_SHIFT) {
                            return (sf_error_t) (((sf_error_t)-1)*(((((((sf_error_t)digits[2]) << PyLong_SHIFT) | (sf_error_t)digits[1]) << PyLong_SHIFT) | (sf_error_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(sf_error_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sf_error_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sf_error_t) - 1 > 3 * PyLong_SHIFT) {
                            return (sf_error_t) ((((((((sf_error_t)digits[2]) << PyLong_SHIFT) | (sf_error_t)digits[1]) << PyLong_SHIFT) | (sf_error_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(sf_error_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sf_error_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sf_error_t) - 1 > 4 * PyLong_SHIFT) {
                            return (sf_error_t) (((sf_error_t)-1)*(((((((((sf_error_t)digits[3]) << PyLong_SHIFT) | (sf_error_t)digits[2]) << PyLong_SHIFT) | (sf_error_t)digits[1]) << PyLong_SHIFT) | (sf_error_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(sf_error_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sf_error_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sf_error_t) - 1 > 4 * PyLong_SHIFT) {
                            return (sf_error_t) ((((((((((sf_error_t)digits[3]) << PyLong_SHIFT) | (sf_error_t)digits[2]) << PyLong_SHIFT) | (sf_error_t)digits[1]) << PyLong_SHIFT) | (sf_error_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(sf_error_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(sf_error_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(sf_error_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(sf_error_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            sf_error_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (sf_error_t) -1;
        }
    } else {
        sf_error_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (sf_error_t) -1;
        val = __Pyx_PyInt_As_sf_error_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to sf_error_t");
    return (sf_error_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to sf_error_t");
    return (sf_error_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE sf_action_t __Pyx_PyInt_As_sf_action_t(PyObject *x) {
    const sf_action_t neg_one = (sf_action_t) ((sf_action_t) 0 - (sf_action_t) 1), const_zero = (sf_action_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(sf_action_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(sf_action_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (sf_action_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (sf_action_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(sf_action_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(sf_action_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sf_action_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sf_action_t) >= 2 * PyLong_SHIFT) {
                            return (sf_action_t) (((((sf_action_t)digits[1]) << PyLong_SHIFT) | (sf_action_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(sf_action_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sf_action_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sf_action_t) >= 3 * PyLong_SHIFT) {
                            return (sf_action_t) (((((((sf_action_t)digits[2]) << PyLong_SHIFT) | (sf_action_t)digits[1]) << PyLong_SHIFT) | (sf_action_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(sf_action_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sf_action_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sf_action_t) >= 4 * PyLong_SHIFT) {
                            return (sf_action_t) (((((((((sf_action_t)digits[3]) << PyLong_SHIFT) | (sf_action_t)digits[2]) << PyLong_SHIFT) | (sf_action_t)digits[1]) << PyLong_SHIFT) | (sf_action_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (sf_action_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(sf_action_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(sf_action_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(sf_action_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(sf_action_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (sf_action_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(sf_action_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(sf_action_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(sf_action_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sf_action_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sf_action_t) - 1 > 2 * PyLong_SHIFT) {
                            return (sf_action_t) (((sf_action_t)-1)*(((((sf_action_t)digits[1]) << PyLong_SHIFT) | (sf_action_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(sf_action_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sf_action_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sf_action_t) - 1 > 2 * PyLong_SHIFT) {
                            return (sf_action_t) ((((((sf_action_t)digits[1]) << PyLong_SHIFT) | (sf_action_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(sf_action_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sf_action_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sf_action_t) - 1 > 3 * PyLong_SHIFT) {
                            return (sf_action_t) (((sf_action_t)-1)*(((((((sf_action_t)digits[2]) << PyLong_SHIFT) | (sf_action_t)digits[1]) << PyLong_SHIFT) | (sf_action_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(sf_action_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sf_action_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sf_action_t) - 1 > 3 * PyLong_SHIFT) {
                            return (sf_action_t) ((((((((sf_action_t)digits[2]) << PyLong_SHIFT) | (sf_action_t)digits[1]) << PyLong_SHIFT) | (sf_action_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(sf_action_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sf_action_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sf_action_t) - 1 > 4 * PyLong_SHIFT) {
                            return (sf_action_t) (((sf_action_t)-1)*(((((((((sf_action_t)digits[3]) << PyLong_SHIFT) | (sf_action_t)digits[2]) << PyLong_SHIFT) | (sf_action_t)digits[1]) << PyLong_SHIFT) | (sf_action_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(sf_action_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(sf_action_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(sf_action_t) - 1 > 4 * PyLong_SHIFT) {
                            return (sf_action_t) ((((((((((sf_action_t)digits[3]) << PyLong_SHIFT) | (sf_action_t)digits[2]) << PyLong_SHIFT) | (sf_action_t)digits[1]) << PyLong_SHIFT) | (sf_action_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(sf_action_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(sf_action_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(sf_action_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(sf_action_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            sf_action_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (sf_action_t) -1;
        }
    } else {
        sf_action_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (sf_action_t) -1;
        val = __Pyx_PyInt_As_sf_action_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to sf_action_t");
    return (sf_action_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to sf_action_t");
    return (sf_action_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE Py_intptr_t __Pyx_PyInt_As_Py_intptr_t(PyObject *x) {
    const Py_intptr_t neg_one = (Py_intptr_t) ((Py_intptr_t) 0 - (Py_intptr_t) 1), const_zero = (Py_intptr_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(Py_intptr_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (Py_intptr_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (Py_intptr_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(Py_intptr_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(Py_intptr_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) >= 2 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(Py_intptr_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) >= 3 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((((Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(Py_intptr_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) >= 4 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((((((Py_intptr_t)digits[3]) << PyLong_SHIFT) | (Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (Py_intptr_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(Py_intptr_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(Py_intptr_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (Py_intptr_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(Py_intptr_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(Py_intptr_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(Py_intptr_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 2 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((Py_intptr_t)-1)*(((((Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(Py_intptr_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 2 * PyLong_SHIFT) {
                            return (Py_intptr_t) ((((((Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(Py_intptr_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 3 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((Py_intptr_t)-1)*(((((((Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(Py_intptr_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 3 * PyLong_SHIFT) {
                            return (Py_intptr_t) ((((((((Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(Py_intptr_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 4 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((Py_intptr_t)-1)*(((((((((Py_intptr_t)digits[3]) << PyLong_SHIFT) | (Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(Py_intptr_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 4 * PyLong_SHIFT) {
                            return (Py_intptr_t) ((((((((((Py_intptr_t)digits[3]) << PyLong_SHIFT) | (Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(Py_intptr_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(Py_intptr_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            Py_intptr_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (Py_intptr_t) -1;
        }
    } else {
        Py_intptr_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (Py_intptr_t) -1;
        val = __Pyx_PyInt_As_Py_intptr_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to Py_intptr_t");
    return (Py_intptr_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to Py_intptr_t");
    return (Py_intptr_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* VoidPtrImport */
#ifndef __PYX_HAVE_RT_ImportVoidPtr
#define __PYX_HAVE_RT_ImportVoidPtr
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, name);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C variable %.200s",
                PyModule_GetName(module), name);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, desc);
        goto bad;
    }
    *p = PyCObject_AsVoidPtr(cobj);}
#endif
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* FunctionImport */
#ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
